/* Do not modify this file.  */
/* It is created automatically by the Makefile.  */
#include "register.h"
void register_all_tap_listeners(void) {
  { extern void register_tap_listener_afpstat (void);
    register_tap_listener_afpstat ();}
  { extern void register_tap_listener_ansi_astat (void);
    register_tap_listener_ansi_astat ();}
  { extern void register_tap_listener_gtkdhcpstat (void);
    register_tap_listener_gtkdhcpstat ();}
  { extern void register_tap_listener_camelcounter (void);
    register_tap_listener_camelcounter ();}
  { extern void register_tap_listener_camelsrt (void);
    register_tap_listener_camelsrt ();}
  { extern void register_tap_listener_dcerpcstat (void);
    register_tap_listener_dcerpcstat ();}
  { extern void register_tap_listener_gtkfunnel (void);
    register_tap_listener_gtkfunnel ();}
  { extern void register_tap_listener_gsm_astat (void);
    register_tap_listener_gsm_astat ();}
  { extern void register_tap_listener_h225counter (void);
    register_tap_listener_h225counter ();}
  { extern void register_tap_listener_h225rassrt (void);
    register_tap_listener_h225rassrt ();}
  { extern void register_tap_listener_gtkhttpstat (void);
    register_tap_listener_gtkhttpstat ();}
  { extern void register_tap_listener_iostat (void);
    register_tap_listener_iostat ();}
  { extern void register_tap_listener_iousers (void);
    register_tap_listener_iousers ();}
  { extern void register_tap_listener_mgcpstat (void);
    register_tap_listener_mgcpstat ();}
  { extern void register_tap_listener_protocolinfo (void);
    register_tap_listener_protocolinfo ();}
  { extern void register_tap_listener_protohierstat (void);
    register_tap_listener_protohierstat ();}
  { extern void register_tap_listener_radiusstat (void);
    register_tap_listener_radiusstat ();}
  { extern void register_tap_listener_rpcstat (void);
    register_tap_listener_rpcstat ();}
  { extern void register_tap_listener_rpcprogs (void);
    register_tap_listener_rpcprogs ();}
  { extern void register_tap_listener_sctpstat (void);
    register_tap_listener_sctpstat ();}
  { extern void register_tap_listener_sipstat (void);
    register_tap_listener_sipstat ();}
  { extern void register_tap_listener_smbsids (void);
    register_tap_listener_smbsids ();}
  { extern void register_tap_listener_smbstat (void);
    register_tap_listener_smbstat ();}
  { extern void register_tap_listener_stats_tree_stat (void);
    register_tap_listener_stats_tree_stat ();}
  { extern void register_tap_listener_wspstat (void);
    register_tap_listener_wspstat ();}
}
