/* Do not modify this file.  */
/* It is created automatically by the Makefile.  */

#include "register.h"
void
register_all_protocols(void)
{
  {extern void proto_register_2dparityfec (void); proto_register_2dparityfec ();}
  {extern void proto_register_3com_xns (void); proto_register_3com_xns ();}
  {extern void proto_register_9P (void); proto_register_9P ();}
  {extern void proto_register_ISystemActivator (void); proto_register_ISystemActivator ();}
  {extern void proto_register_MAP_DialoguePDU (void); proto_register_MAP_DialoguePDU ();}
  {extern void proto_register_a11 (void); proto_register_a11 ();}
  {extern void proto_register_aarp (void); proto_register_aarp ();}
  {extern void proto_register_acap (void); proto_register_acap ();}
  {extern void proto_register_acn (void); proto_register_acn ();}
  {extern void proto_register_acp133 (void); proto_register_acp133 ();}
  {extern void proto_register_acse (void); proto_register_acse ();}
  {extern void proto_register_actrace (void); proto_register_actrace ();}
  {extern void proto_register_afp (void); proto_register_afp ();}
  {extern void proto_register_afs (void); proto_register_afs ();}
  {extern void proto_register_afs4int (void); proto_register_afs4int ();}
  {extern void proto_register_aim (void); proto_register_aim ();}
  {extern void proto_register_aim_admin (void); proto_register_aim_admin ();}
  {extern void proto_register_aim_adverts (void); proto_register_aim_adverts ();}
  {extern void proto_register_aim_bos (void); proto_register_aim_bos ();}
  {extern void proto_register_aim_buddylist (void); proto_register_aim_buddylist ();}
  {extern void proto_register_aim_chat (void); proto_register_aim_chat ();}
  {extern void proto_register_aim_chatnav (void); proto_register_aim_chatnav ();}
  {extern void proto_register_aim_directory (void); proto_register_aim_directory ();}
  {extern void proto_register_aim_email (void); proto_register_aim_email ();}
  {extern void proto_register_aim_generic (void); proto_register_aim_generic ();}
  {extern void proto_register_aim_icq (void); proto_register_aim_icq ();}
  {extern void proto_register_aim_invitation (void); proto_register_aim_invitation ();}
  {extern void proto_register_aim_location (void); proto_register_aim_location ();}
  {extern void proto_register_aim_messaging (void); proto_register_aim_messaging ();}
  {extern void proto_register_aim_oft (void); proto_register_aim_oft ();}
  {extern void proto_register_aim_popup (void); proto_register_aim_popup ();}
  {extern void proto_register_aim_signon (void); proto_register_aim_signon ();}
  {extern void proto_register_aim_ssi (void); proto_register_aim_ssi ();}
  {extern void proto_register_aim_sst (void); proto_register_aim_sst ();}
  {extern void proto_register_aim_stats (void); proto_register_aim_stats ();}
  {extern void proto_register_aim_translate (void); proto_register_aim_translate ();}
  {extern void proto_register_aim_userlookup (void); proto_register_aim_userlookup ();}
  {extern void proto_register_ajp13 (void); proto_register_ajp13 ();}
  {extern void proto_register_alc (void); proto_register_alc ();}
  {extern void proto_register_alcap (void); proto_register_alcap ();}
  {extern void proto_register_amr (void); proto_register_amr ();}
  {extern void proto_register_ans (void); proto_register_ans ();}
  {extern void proto_register_ansi_637 (void); proto_register_ansi_637 ();}
  {extern void proto_register_ansi_683 (void); proto_register_ansi_683 ();}
  {extern void proto_register_ansi_801 (void); proto_register_ansi_801 ();}
  {extern void proto_register_ansi_a (void); proto_register_ansi_a ();}
  {extern void proto_register_ansi_map (void); proto_register_ansi_map ();}
  {extern void proto_register_aodv (void); proto_register_aodv ();}
  {extern void proto_register_aoe (void); proto_register_aoe ();}
  {extern void proto_register_ap1394 (void); proto_register_ap1394 ();}
  {extern void proto_register_arcnet (void); proto_register_arcnet ();}
  {extern void proto_register_armagetronad (void); proto_register_armagetronad ();}
  {extern void proto_register_arp (void); proto_register_arp ();}
  {extern void proto_register_aruba_adp (void); proto_register_aruba_adp ();}
  {extern void proto_register_asap (void); proto_register_asap ();}
  {extern void proto_register_ascend (void); proto_register_ascend ();}
  {extern void proto_register_asf (void); proto_register_asf ();}
  {extern void proto_register_atalk (void); proto_register_atalk ();}
  {extern void proto_register_atm (void); proto_register_atm ();}
  {extern void proto_register_auto_rp (void); proto_register_auto_rp ();}
  {extern void proto_register_ax4000 (void); proto_register_ax4000 ();}
  {extern void proto_register_bacapp (void); proto_register_bacapp ();}
  {extern void proto_register_bacnet (void); proto_register_bacnet ();}
  {extern void proto_register_bacp (void); proto_register_bacp ();}
  {extern void proto_register_bap (void); proto_register_bap ();}
  {extern void proto_register_basicxid (void); proto_register_basicxid ();}
  {extern void proto_register_bcp (void); proto_register_bcp ();}
  {extern void proto_register_beep (void); proto_register_beep ();}
  {extern void proto_register_ber (void); proto_register_ber ();}
  {extern void proto_register_bfd (void); proto_register_bfd ();}
  {extern void proto_register_bgp (void); proto_register_bgp ();}
  {extern void proto_register_bicc (void); proto_register_bicc ();}
  {extern void proto_register_bittorrent (void); proto_register_bittorrent ();}
  {extern void proto_register_bofl (void); proto_register_bofl ();}
  {extern void proto_register_bootp (void); proto_register_bootp ();}
  {extern void proto_register_bootparams (void); proto_register_bootparams ();}
  {extern void proto_register_bossvr (void); proto_register_bossvr ();}
  {extern void proto_register_bpdu (void); proto_register_bpdu ();}
  {extern void proto_register_brdwlk (void); proto_register_brdwlk ();}
  {extern void proto_register_bssap (void); proto_register_bssap ();}
  {extern void proto_register_bssgp (void); proto_register_bssgp ();}
  {extern void proto_register_btacl (void); proto_register_btacl ();}
  {extern void proto_register_bthci_cmd (void); proto_register_bthci_cmd ();}
  {extern void proto_register_bthci_evt (void); proto_register_bthci_evt ();}
  {extern void proto_register_btl2cap (void); proto_register_btl2cap ();}
  {extern void proto_register_btrfcomm (void); proto_register_btrfcomm ();}
  {extern void proto_register_btsco (void); proto_register_btsco ();}
  {extern void proto_register_btsdp (void); proto_register_btsdp ();}
  {extern void proto_register_budb (void); proto_register_budb ();}
  {extern void proto_register_butc (void); proto_register_butc ();}
  {extern void proto_register_bvlc (void); proto_register_bvlc ();}
  {extern void proto_register_calcappprotocol (void); proto_register_calcappprotocol ();}
  {extern void proto_register_camel (void); proto_register_camel ();}
  {extern void proto_register_cast (void); proto_register_cast ();}
  {extern void proto_register_catapult_dct2000 (void); proto_register_catapult_dct2000 ();}
  {extern void proto_register_cbcp (void); proto_register_cbcp ();}
  {extern void proto_register_ccp (void); proto_register_ccp ();}
  {extern void proto_register_ccsds (void); proto_register_ccsds ();}
  {extern void proto_register_cdp (void); proto_register_cdp ();}
  {extern void proto_register_cdpcp (void); proto_register_cdpcp ();}
  {extern void proto_register_cds_clerkserver (void); proto_register_cds_clerkserver ();}
  {extern void proto_register_cds_solicit (void); proto_register_cds_solicit ();}
  {extern void proto_register_cdt (void); proto_register_cdt ();}
  {extern void proto_register_cert (void); proto_register_cert ();}
  {extern void proto_register_cgmp (void); proto_register_cgmp ();}
  {extern void proto_register_chap (void); proto_register_chap ();}
  {extern void proto_register_chdlc (void); proto_register_chdlc ();}
  {extern void proto_register_cigi (void); proto_register_cigi ();}
  {extern void proto_register_cimd (void); proto_register_cimd ();}
  {extern void proto_register_cip (void); proto_register_cip ();}
  {extern void proto_register_cisco_oui (void); proto_register_cisco_oui ();}
  {extern void proto_register_clearcase (void); proto_register_clearcase ();}
  {extern void proto_register_clip (void); proto_register_clip ();}
  {extern void proto_register_clnp (void); proto_register_clnp ();}
  {extern void proto_register_cltp (void); proto_register_cltp ();}
  {extern void proto_register_cmip (void); proto_register_cmip ();}
  {extern void proto_register_cmp (void); proto_register_cmp ();}
  {extern void proto_register_cms (void); proto_register_cms ();}
  {extern void proto_register_comp_data (void); proto_register_comp_data ();}
  {extern void proto_register_componentstatusprotocol (void); proto_register_componentstatusprotocol ();}
  {extern void proto_register_conv (void); proto_register_conv ();}
  {extern void proto_register_cops (void); proto_register_cops ();}
  {extern void proto_register_cosine (void); proto_register_cosine ();}
  {extern void proto_register_cotp (void); proto_register_cotp ();}
  {extern void proto_register_cpfi (void); proto_register_cpfi ();}
  {extern void proto_register_cpha (void); proto_register_cpha ();}
  {extern void proto_register_cprpc_server (void); proto_register_cprpc_server ();}
  {extern void proto_register_crmf (void); proto_register_crmf ();}
  {extern void proto_register_csm_encaps (void); proto_register_csm_encaps ();}
  {extern void proto_register_cups (void); proto_register_cups ();}
  {extern void proto_register_cwids (void); proto_register_cwids ();}
  {extern void proto_register_daap (void); proto_register_daap ();}
  {extern void proto_register_dap (void); proto_register_dap ();}
  {extern void proto_register_data (void); proto_register_data ();}
  {extern void proto_register_daytime (void); proto_register_daytime ();}
  {extern void proto_register_dccp (void); proto_register_dccp ();}
  {extern void proto_register_dce_update (void); proto_register_dce_update ();}
  {extern void proto_register_dcerpc (void); proto_register_dcerpc ();}
  {extern void proto_register_dcerpc_atsvc (void); proto_register_dcerpc_atsvc ();}
  {extern void proto_register_dcerpc_browser (void); proto_register_dcerpc_browser ();}
  {extern void proto_register_dcerpc_dnsserver (void); proto_register_dcerpc_dnsserver ();}
  {extern void proto_register_dcerpc_dssetup (void); proto_register_dcerpc_dssetup ();}
  {extern void proto_register_dcerpc_efs (void); proto_register_dcerpc_efs ();}
  {extern void proto_register_dcerpc_eventlog (void); proto_register_dcerpc_eventlog ();}
  {extern void proto_register_dcerpc_frsapi (void); proto_register_dcerpc_frsapi ();}
  {extern void proto_register_dcerpc_frsrpc (void); proto_register_dcerpc_frsrpc ();}
  {extern void proto_register_dcerpc_initshutdown (void); proto_register_dcerpc_initshutdown ();}
  {extern void proto_register_dcerpc_lsa (void); proto_register_dcerpc_lsa ();}
  {extern void proto_register_dcerpc_mapi (void); proto_register_dcerpc_mapi ();}
  {extern void proto_register_dcerpc_messenger (void); proto_register_dcerpc_messenger ();}
  {extern void proto_register_dcerpc_netdfs (void); proto_register_dcerpc_netdfs ();}
  {extern void proto_register_dcerpc_netlogon (void); proto_register_dcerpc_netlogon ();}
  {extern void proto_register_dcerpc_nspi (void); proto_register_dcerpc_nspi ();}
  {extern void proto_register_dcerpc_pnp (void); proto_register_dcerpc_pnp ();}
  {extern void proto_register_dcerpc_rras (void); proto_register_dcerpc_rras ();}
  {extern void proto_register_dcerpc_rs_plcy (void); proto_register_dcerpc_rs_plcy ();}
  {extern void proto_register_dcerpc_samr (void); proto_register_dcerpc_samr ();}
  {extern void proto_register_dcerpc_spoolss (void); proto_register_dcerpc_spoolss ();}
  {extern void proto_register_dcerpc_srvsvc (void); proto_register_dcerpc_srvsvc ();}
  {extern void proto_register_dcerpc_svcctl (void); proto_register_dcerpc_svcctl ();}
  {extern void proto_register_dcerpc_tapi (void); proto_register_dcerpc_tapi ();}
  {extern void proto_register_dcerpc_trksvr (void); proto_register_dcerpc_trksvr ();}
  {extern void proto_register_dcerpc_winreg (void); proto_register_dcerpc_winreg ();}
  {extern void proto_register_dcerpc_wkssvc (void); proto_register_dcerpc_wkssvc ();}
  {extern void proto_register_dcerpc_wzcsvc (void); proto_register_dcerpc_wzcsvc ();}
  {extern void proto_register_dcm (void); proto_register_dcm ();}
  {extern void proto_register_dcom (void); proto_register_dcom ();}
  {extern void proto_register_dcom_cba (void); proto_register_dcom_cba ();}
  {extern void proto_register_dcom_cba_acco (void); proto_register_dcom_cba_acco ();}
  {extern void proto_register_dcom_dispatch (void); proto_register_dcom_dispatch ();}
  {extern void proto_register_dcp (void); proto_register_dcp ();}
  {extern void proto_register_dcp_etsi (void); proto_register_dcp_etsi ();}
  {extern void proto_register_ddtp (void); proto_register_ddtp ();}
  {extern void proto_register_dec_bpdu (void); proto_register_dec_bpdu ();}
  {extern void proto_register_dec_rt (void); proto_register_dec_rt ();}
  {extern void proto_register_dhcpfo (void); proto_register_dhcpfo ();}
  {extern void proto_register_dhcpv6 (void); proto_register_dhcpv6 ();}
  {extern void proto_register_diameter (void); proto_register_diameter ();}
  {extern void proto_register_dis (void); proto_register_dis ();}
  {extern void proto_register_disp (void); proto_register_disp ();}
  {extern void proto_register_distcc (void); proto_register_distcc ();}
  {extern void proto_register_dlsw (void); proto_register_dlsw ();}
  {extern void proto_register_dmp (void); proto_register_dmp ();}
  {extern void proto_register_dnp3 (void); proto_register_dnp3 ();}
  {extern void proto_register_dns (void); proto_register_dns ();}
  {extern void proto_register_dop (void); proto_register_dop ();}
  {extern void proto_register_drsuapi (void); proto_register_drsuapi ();}
  {extern void proto_register_dsi (void); proto_register_dsi ();}
  {extern void proto_register_dsp (void); proto_register_dsp ();}
  {extern void proto_register_dtls (void); proto_register_dtls ();}
  {extern void proto_register_dtp (void); proto_register_dtp ();}
  {extern void proto_register_dtsprovider (void); proto_register_dtsprovider ();}
  {extern void proto_register_dtsstime_req (void); proto_register_dtsstime_req ();}
  {extern void proto_register_dua (void); proto_register_dua ();}
  {extern void proto_register_dvmrp (void); proto_register_dvmrp ();}
  {extern void proto_register_e164 (void); proto_register_e164 ();}
  {extern void proto_register_e212 (void); proto_register_e212 ();}
  {extern void proto_register_eap (void); proto_register_eap ();}
  {extern void proto_register_eapol (void); proto_register_eapol ();}
  {extern void proto_register_echo (void); proto_register_echo ();}
  {extern void proto_register_edonkey (void); proto_register_edonkey ();}
  {extern void proto_register_edp (void); proto_register_edp ();}
  {extern void proto_register_eigrp (void); proto_register_eigrp ();}
  {extern void proto_register_enc (void); proto_register_enc ();}
  {extern void proto_register_enip (void); proto_register_enip ();}
  {extern void proto_register_enrp (void); proto_register_enrp ();}
  {extern void proto_register_epl (void); proto_register_epl ();}
  {extern void proto_register_epl_v1 (void); proto_register_epl_v1 ();}
  {extern void proto_register_epm (void); proto_register_epm ();}
  {extern void proto_register_esis (void); proto_register_esis ();}
  {extern void proto_register_ess (void); proto_register_ess ();}
  {extern void proto_register_eth (void); proto_register_eth ();}
  {extern void proto_register_etheric (void); proto_register_etheric ();}
  {extern void proto_register_etherip (void); proto_register_etherip ();}
  {extern void proto_register_ethertype (void); proto_register_ethertype ();}
  {extern void proto_register_exec (void); proto_register_exec ();}
  {extern void proto_register_extreme_oui (void); proto_register_extreme_oui ();}
  {extern void proto_register_fc (void); proto_register_fc ();}
  {extern void proto_register_fcct (void); proto_register_fcct ();}
  {extern void proto_register_fcdns (void); proto_register_fcdns ();}
  {extern void proto_register_fcels (void); proto_register_fcels ();}
  {extern void proto_register_fcfcs (void); proto_register_fcfcs ();}
  {extern void proto_register_fcfzs (void); proto_register_fcfzs ();}
  {extern void proto_register_fcip (void); proto_register_fcip ();}
  {extern void proto_register_fcp (void); proto_register_fcp ();}
  {extern void proto_register_fcsbccs (void); proto_register_fcsbccs ();}
  {extern void proto_register_fcsp (void); proto_register_fcsp ();}
  {extern void proto_register_fcswils (void); proto_register_fcswils ();}
  {extern void proto_register_fddi (void); proto_register_fddi ();}
  {extern void proto_register_fix (void); proto_register_fix ();}
  {extern void proto_register_fldb (void); proto_register_fldb ();}
  {extern void proto_register_fp (void); proto_register_fp ();}
  {extern void proto_register_fr (void); proto_register_fr ();}
  {extern void proto_register_fractalgeneratorprotocol (void); proto_register_fractalgeneratorprotocol ();}
  {extern void proto_register_frame (void); proto_register_frame ();}
  {extern void proto_register_ftam (void); proto_register_ftam ();}
  {extern void proto_register_ftbp (void); proto_register_ftbp ();}
  {extern void proto_register_ftp (void); proto_register_ftp ();}
  {extern void proto_register_ftserver (void); proto_register_ftserver ();}
  {extern void proto_register_fw1 (void); proto_register_fw1 ();}
  {extern void proto_register_g723 (void); proto_register_g723 ();}
  {extern void proto_register_gif (void); proto_register_gif ();}
  {extern void proto_register_gift (void); proto_register_gift ();}
  {extern void proto_register_giop (void); proto_register_giop ();}
  {extern void proto_register_gmrp (void); proto_register_gmrp ();}
  {extern void proto_register_gnm (void); proto_register_gnm ();}
  {extern void proto_register_gnutella (void); proto_register_gnutella ();}
  {extern void proto_register_gprs_ns (void); proto_register_gprs_ns ();}
  {extern void proto_register_gre (void); proto_register_gre ();}
  {extern void proto_register_gsm_a (void); proto_register_gsm_a ();}
  {extern void proto_register_gsm_map (void); proto_register_gsm_map ();}
  {extern void proto_register_gsm_sms (void); proto_register_gsm_sms ();}
  {extern void proto_register_gsm_sms_ud (void); proto_register_gsm_sms_ud ();}
  {extern void proto_register_gsm_ss (void); proto_register_gsm_ss ();}
  {extern void proto_register_gssapi (void); proto_register_gssapi ();}
  {extern void proto_register_gtp (void); proto_register_gtp ();}
  {extern void proto_register_gvrp (void); proto_register_gvrp ();}
  {extern void proto_register_h1 (void); proto_register_h1 ();}
  {extern void proto_register_h225 (void); proto_register_h225 ();}
  {extern void proto_register_h235 (void); proto_register_h235 ();}
  {extern void proto_register_h245 (void); proto_register_h245 ();}
  {extern void proto_register_h248 (void); proto_register_h248 ();}
  {extern void proto_register_h248_3gpp (void); proto_register_h248_3gpp ();}
  {extern void proto_register_h248_annex_c (void); proto_register_h248_annex_c ();}
  {extern void proto_register_h248_annex_e (void); proto_register_h248_annex_e ();}
  {extern void proto_register_h261 (void); proto_register_h261 ();}
  {extern void proto_register_h263 (void); proto_register_h263 ();}
  {extern void proto_register_h450 (void); proto_register_h450 ();}
  {extern void proto_register_hci_h4 (void); proto_register_hci_h4 ();}
  {extern void proto_register_hclnfsd (void); proto_register_hclnfsd ();}
  {extern void proto_register_homeplug (void); proto_register_homeplug ();}
  {extern void proto_register_hpext (void); proto_register_hpext ();}
  {extern void proto_register_hpsw (void); proto_register_hpsw ();}
  {extern void proto_register_hsrp (void); proto_register_hsrp ();}
  {extern void proto_register_http (void); proto_register_http ();}
  {extern void proto_register_hyperscsi (void); proto_register_hyperscsi ();}
  {extern void proto_register_iapp (void); proto_register_iapp ();}
  {extern void proto_register_iax2 (void); proto_register_iax2 ();}
  {extern void proto_register_ib (void); proto_register_ib ();}
  {extern void proto_register_icap (void); proto_register_icap ();}
  {extern void proto_register_icep (void); proto_register_icep ();}
  {extern void proto_register_icl_rpc (void); proto_register_icl_rpc ();}
  {extern void proto_register_icmp (void); proto_register_icmp ();}
  {extern void proto_register_icmpv6 (void); proto_register_icmpv6 ();}
  {extern void proto_register_icp (void); proto_register_icp ();}
  {extern void proto_register_icq (void); proto_register_icq ();}
  {extern void proto_register_idp (void); proto_register_idp ();}
  {extern void proto_register_ieee80211 (void); proto_register_ieee80211 ();}
  {extern void proto_register_ieee802a (void); proto_register_ieee802a ();}
  {extern void proto_register_ifcp (void); proto_register_ifcp ();}
  {extern void proto_register_igap (void); proto_register_igap ();}
  {extern void proto_register_igmp (void); proto_register_igmp ();}
  {extern void proto_register_igrp (void); proto_register_igrp ();}
  {extern void proto_register_imap (void); proto_register_imap ();}
  {extern void proto_register_inap (void); proto_register_inap ();}
  {extern void proto_register_ip (void); proto_register_ip ();}
  {extern void proto_register_ipcp (void); proto_register_ipcp ();}
  {extern void proto_register_ipdc (void); proto_register_ipdc ();}
  {extern void proto_register_ipfc (void); proto_register_ipfc ();}
  {extern void proto_register_ipmi (void); proto_register_ipmi ();}
  {extern void proto_register_ipp (void); proto_register_ipp ();}
  {extern void proto_register_ipsec (void); proto_register_ipsec ();}
  {extern void proto_register_ipv6 (void); proto_register_ipv6 ();}
  {extern void proto_register_ipv6cp (void); proto_register_ipv6cp ();}
  {extern void proto_register_ipvs_syncd (void); proto_register_ipvs_syncd ();}
  {extern void proto_register_ipx (void); proto_register_ipx ();}
  {extern void proto_register_ipxwan (void); proto_register_ipxwan ();}
  {extern void proto_register_irc (void); proto_register_irc ();}
  {extern void proto_register_isakmp (void); proto_register_isakmp ();}
  {extern void proto_register_iscsi (void); proto_register_iscsi ();}
  {extern void proto_register_isdn (void); proto_register_isdn ();}
  {extern void proto_register_isis (void); proto_register_isis ();}
  {extern void proto_register_isl (void); proto_register_isl ();}
  {extern void proto_register_ismp (void); proto_register_ismp ();}
  {extern void proto_register_isns (void); proto_register_isns ();}
  {extern void proto_register_isup (void); proto_register_isup ();}
  {extern void proto_register_isup_thin (void); proto_register_isup_thin ();}
  {extern void proto_register_iua (void); proto_register_iua ();}
  {extern void proto_register_iuup (void); proto_register_iuup ();}
  {extern void proto_register_jabber (void); proto_register_jabber ();}
  {extern void proto_register_jfif (void); proto_register_jfif ();}
  {extern void proto_register_jpeg (void); proto_register_jpeg ();}
  {extern void proto_register_juniper (void); proto_register_juniper ();}
  {extern void proto_register_jxta (void); proto_register_jxta ();}
  {extern void proto_register_k12 (void); proto_register_k12 ();}
  {extern void proto_register_kadm5 (void); proto_register_kadm5 ();}
  {extern void proto_register_kerberos (void); proto_register_kerberos ();}
  {extern void proto_register_kink (void); proto_register_kink ();}
  {extern void proto_register_kismet (void); proto_register_kismet ();}
  {extern void proto_register_klm (void); proto_register_klm ();}
  {extern void proto_register_kpasswd (void); proto_register_kpasswd ();}
  {extern void proto_register_krb4 (void); proto_register_krb4 ();}
  {extern void proto_register_krb5rpc (void); proto_register_krb5rpc ();}
  {extern void proto_register_l2tp (void); proto_register_l2tp ();}
  {extern void proto_register_lapb (void); proto_register_lapb ();}
  {extern void proto_register_lapbether (void); proto_register_lapbether ();}
  {extern void proto_register_lapd (void); proto_register_lapd ();}
  {extern void proto_register_laplink (void); proto_register_laplink ();}
  {extern void proto_register_lcp (void); proto_register_lcp ();}
  {extern void proto_register_ldap (void); proto_register_ldap ();}
  {extern void proto_register_ldp (void); proto_register_ldp ();}
  {extern void proto_register_lge_monitor (void); proto_register_lge_monitor ();}
  {extern void proto_register_linx (void); proto_register_linx ();}
  {extern void proto_register_llb (void); proto_register_llb ();}
  {extern void proto_register_llc (void); proto_register_llc ();}
  {extern void proto_register_llcgprs (void); proto_register_llcgprs ();}
  {extern void proto_register_lldp (void); proto_register_lldp ();}
  {extern void proto_register_llt (void); proto_register_llt ();}
  {extern void proto_register_lmi (void); proto_register_lmi ();}
  {extern void proto_register_lmp (void); proto_register_lmp ();}
  {extern void proto_register_logotypecertextn (void); proto_register_logotypecertextn ();}
  {extern void proto_register_loop (void); proto_register_loop ();}
  {extern void proto_register_lpd (void); proto_register_lpd ();}
  {extern void proto_register_lsc (void); proto_register_lsc ();}
  {extern void proto_register_lwapp (void); proto_register_lwapp ();}
  {extern void proto_register_m2pa (void); proto_register_m2pa ();}
  {extern void proto_register_m2tp (void); proto_register_m2tp ();}
  {extern void proto_register_m2ua (void); proto_register_m2ua ();}
  {extern void proto_register_m3ua (void); proto_register_m3ua ();}
  {extern void proto_register_macctrl (void); proto_register_macctrl ();}
  {extern void proto_register_manolito (void); proto_register_manolito ();}
  {extern void proto_register_mdshdr (void); proto_register_mdshdr ();}
  {extern void proto_register_media (void); proto_register_media ();}
  {extern void proto_register_message_http (void); proto_register_message_http ();}
  {extern void proto_register_mgmt (void); proto_register_mgmt ();}
  {extern void proto_register_mip (void); proto_register_mip ();}
  {extern void proto_register_mip6 (void); proto_register_mip6 ();}
  {extern void proto_register_mms (void); proto_register_mms ();}
  {extern void proto_register_mmse (void); proto_register_mmse ();}
  {extern void proto_register_modbus (void); proto_register_modbus ();}
  {extern void proto_register_mount (void); proto_register_mount ();}
  {extern void proto_register_mp (void); proto_register_mp ();}
  {extern void proto_register_mp2t (void); proto_register_mp2t ();}
  {extern void proto_register_mpeg1 (void); proto_register_mpeg1 ();}
  {extern void proto_register_mpls (void); proto_register_mpls ();}
  {extern void proto_register_mpls_echo (void); proto_register_mpls_echo ();}
  {extern void proto_register_mplscp (void); proto_register_mplscp ();}
  {extern void proto_register_mq (void); proto_register_mq ();}
  {extern void proto_register_mqpcf (void); proto_register_mqpcf ();}
  {extern void proto_register_mrdisc (void); proto_register_mrdisc ();}
  {extern void proto_register_msdp (void); proto_register_msdp ();}
  {extern void proto_register_msmms (void); proto_register_msmms ();}
  {extern void proto_register_msnip (void); proto_register_msnip ();}
  {extern void proto_register_msnlb (void); proto_register_msnlb ();}
  {extern void proto_register_msnms (void); proto_register_msnms ();}
  {extern void proto_register_msproxy (void); proto_register_msproxy ();}
  {extern void proto_register_msrp (void); proto_register_msrp ();}
  {extern void proto_register_mtp2 (void); proto_register_mtp2 ();}
  {extern void proto_register_mtp3 (void); proto_register_mtp3 ();}
  {extern void proto_register_mtp3mg (void); proto_register_mtp3mg ();}
  {extern void proto_register_multipart (void); proto_register_multipart ();}
  {extern void proto_register_mysql (void); proto_register_mysql ();}
  {extern void proto_register_nbap (void); proto_register_nbap ();}
  {extern void proto_register_nbd (void); proto_register_nbd ();}
  {extern void proto_register_nbipx (void); proto_register_nbipx ();}
  {extern void proto_register_nbt (void); proto_register_nbt ();}
  {extern void proto_register_ncp (void); proto_register_ncp ();}
  {extern void proto_register_ncp2222 (void); proto_register_ncp2222 ();}
  {extern void proto_register_ncs (void); proto_register_ncs ();}
  {extern void proto_register_ndmp (void); proto_register_ndmp ();}
  {extern void proto_register_ndps (void); proto_register_ndps ();}
  {extern void proto_register_netbios (void); proto_register_netbios ();}
  {extern void proto_register_netflow (void); proto_register_netflow ();}
  {extern void proto_register_netlib (void); proto_register_netlib ();}
  {extern void proto_register_netsync (void); proto_register_netsync ();}
  {extern void proto_register_nettl (void); proto_register_nettl ();}
  {extern void proto_register_newmail (void); proto_register_newmail ();}
  {extern void proto_register_nfs (void); proto_register_nfs ();}
  {extern void proto_register_nfsacl (void); proto_register_nfsacl ();}
  {extern void proto_register_nfsauth (void); proto_register_nfsauth ();}
  {extern void proto_register_nhrp (void); proto_register_nhrp ();}
  {extern void proto_register_nis (void); proto_register_nis ();}
  {extern void proto_register_niscb (void); proto_register_niscb ();}
  {extern void proto_register_njack (void); proto_register_njack ();}
  {extern void proto_register_nlm (void); proto_register_nlm ();}
  {extern void proto_register_nlsp (void); proto_register_nlsp ();}
  {extern void proto_register_nmas (void); proto_register_nmas ();}
  {extern void proto_register_nmpi (void); proto_register_nmpi ();}
  {extern void proto_register_nntp (void); proto_register_nntp ();}
  {extern void proto_register_nonstd (void); proto_register_nonstd ();}
  {extern void proto_register_norm (void); proto_register_norm ();}
  {extern void proto_register_nortel_oui (void); proto_register_nortel_oui ();}
  {extern void proto_register_ns_cert_exts (void); proto_register_ns_cert_exts ();}
  {extern void proto_register_nsip (void); proto_register_nsip ();}
  {extern void proto_register_ntlmssp (void); proto_register_ntlmssp ();}
  {extern void proto_register_ntp (void); proto_register_ntp ();}
  {extern void proto_register_null (void); proto_register_null ();}
  {extern void proto_register_ocsp (void); proto_register_ocsp ();}
  {extern void proto_register_oicq (void); proto_register_oicq ();}
  {extern void proto_register_old_pflog (void); proto_register_old_pflog ();}
  {extern void proto_register_olsr (void); proto_register_olsr ();}
  {extern void proto_register_omapi (void); proto_register_omapi ();}
  {extern void proto_register_osi (void); proto_register_osi ();}
  {extern void proto_register_osi_options (void); proto_register_osi_options ();}
  {extern void proto_register_osicp (void); proto_register_osicp ();}
  {extern void proto_register_ospf (void); proto_register_ospf ();}
  {extern void proto_register_oxid (void); proto_register_oxid ();}
  {extern void proto_register_p_mul (void); proto_register_p_mul ();}
  {extern void proto_register_packetcable (void); proto_register_packetcable ();}
  {extern void proto_register_pagp (void); proto_register_pagp ();}
  {extern void proto_register_pana (void); proto_register_pana ();}
  {extern void proto_register_pap (void); proto_register_pap ();}
  {extern void proto_register_pcnfsd (void); proto_register_pcnfsd ();}
  {extern void proto_register_per (void); proto_register_per ();}
  {extern void proto_register_pflog (void); proto_register_pflog ();}
  {extern void proto_register_pgm (void); proto_register_pgm ();}
  {extern void proto_register_pgsql (void); proto_register_pgsql ();}
  {extern void proto_register_pim (void); proto_register_pim ();}
  {extern void proto_register_pingpongprotocol (void); proto_register_pingpongprotocol ();}
  {extern void proto_register_pipe_dcerpc (void); proto_register_pipe_dcerpc ();}
  {extern void proto_register_pipe_lanman (void); proto_register_pipe_lanman ();}
  {extern void proto_register_pkcs1 (void); proto_register_pkcs1 ();}
  {extern void proto_register_pkcs12 (void); proto_register_pkcs12 ();}
  {extern void proto_register_pkinit (void); proto_register_pkinit ();}
  {extern void proto_register_pkix1explicit (void); proto_register_pkix1explicit ();}
  {extern void proto_register_pkix1implicit (void); proto_register_pkix1implicit ();}
  {extern void proto_register_pkixproxy (void); proto_register_pkixproxy ();}
  {extern void proto_register_pkixqualified (void); proto_register_pkixqualified ();}
  {extern void proto_register_pkixtsp (void); proto_register_pkixtsp ();}
  {extern void proto_register_pkt_ccc (void); proto_register_pkt_ccc ();}
  {extern void proto_register_pktc (void); proto_register_pktc ();}
  {extern void proto_register_pktc_mtafqdn (void); proto_register_pktc_mtafqdn ();}
  {extern void proto_register_pktgen (void); proto_register_pktgen ();}
  {extern void proto_register_pn_rt (void); proto_register_pn_rt ();}
  {extern void proto_register_png (void); proto_register_png ();}
  {extern void proto_register_pop (void); proto_register_pop ();}
  {extern void proto_register_portmap (void); proto_register_portmap ();}
  {extern void proto_register_ppp (void); proto_register_ppp ();}
  {extern void proto_register_ppp_raw_hdlc (void); proto_register_ppp_raw_hdlc ();}
  {extern void proto_register_pppmux (void); proto_register_pppmux ();}
  {extern void proto_register_pppmuxcp (void); proto_register_pppmuxcp ();}
  {extern void proto_register_pppoed (void); proto_register_pppoed ();}
  {extern void proto_register_pppoes (void); proto_register_pppoes ();}
  {extern void proto_register_pptp (void); proto_register_pptp ();}
  {extern void proto_register_pres (void); proto_register_pres ();}
  {extern void proto_register_prism (void); proto_register_prism ();}
  {extern void proto_register_ptp (void); proto_register_ptp ();}
  {extern void proto_register_pvfs (void); proto_register_pvfs ();}
  {extern void proto_register_q1950 (void); proto_register_q1950 ();}
  {extern void proto_register_q2931 (void); proto_register_q2931 ();}
  {extern void proto_register_q931 (void); proto_register_q931 ();}
  {extern void proto_register_q933 (void); proto_register_q933 ();}
  {extern void proto_register_qllc (void); proto_register_qllc ();}
  {extern void proto_register_quake (void); proto_register_quake ();}
  {extern void proto_register_quake2 (void); proto_register_quake2 ();}
  {extern void proto_register_quake3 (void); proto_register_quake3 ();}
  {extern void proto_register_quakeworld (void); proto_register_quakeworld ();}
  {extern void proto_register_radiotap (void); proto_register_radiotap ();}
  {extern void proto_register_radius (void); proto_register_radius ();}
  {extern void proto_register_ranap (void); proto_register_ranap ();}
  {extern void proto_register_raw (void); proto_register_raw ();}
  {extern void proto_register_rdaclif (void); proto_register_rdaclif ();}
  {extern void proto_register_rdm (void); proto_register_rdm ();}
  {extern void proto_register_rdt (void); proto_register_rdt ();}
  {extern void proto_register_redback (void); proto_register_redback ();}
  {extern void proto_register_remact (void); proto_register_remact ();}
  {extern void proto_register_remunk (void); proto_register_remunk ();}
  {extern void proto_register_rep_proc (void); proto_register_rep_proc ();}
  {extern void proto_register_retix_bpdu (void); proto_register_retix_bpdu ();}
  {extern void proto_register_rgmp (void); proto_register_rgmp ();}
  {extern void proto_register_rip (void); proto_register_rip ();}
  {extern void proto_register_ripng (void); proto_register_ripng ();}
  {extern void proto_register_rlogin (void); proto_register_rlogin ();}
  {extern void proto_register_rmcp (void); proto_register_rmcp ();}
  {extern void proto_register_rmi (void); proto_register_rmi ();}
  {extern void proto_register_rmp (void); proto_register_rmp ();}
  {extern void proto_register_rnsap (void); proto_register_rnsap ();}
  {extern void proto_register_roofnet (void); proto_register_roofnet ();}
  {extern void proto_register_ros (void); proto_register_ros ();}
  {extern void proto_register_roverride (void); proto_register_roverride ();}
  {extern void proto_register_rpc (void); proto_register_rpc ();}
  {extern void proto_register_rpl (void); proto_register_rpl ();}
  {extern void proto_register_rpriv (void); proto_register_rpriv ();}
  {extern void proto_register_rquota (void); proto_register_rquota ();}
  {extern void proto_register_rrlp (void); proto_register_rrlp ();}
  {extern void proto_register_rs_acct (void); proto_register_rs_acct ();}
  {extern void proto_register_rs_attr (void); proto_register_rs_attr ();}
  {extern void proto_register_rs_attr_schema (void); proto_register_rs_attr_schema ();}
  {extern void proto_register_rs_bind (void); proto_register_rs_bind ();}
  {extern void proto_register_rs_misc (void); proto_register_rs_misc ();}
  {extern void proto_register_rs_pgo (void); proto_register_rs_pgo ();}
  {extern void proto_register_rs_prop_acct (void); proto_register_rs_prop_acct ();}
  {extern void proto_register_rs_prop_acl (void); proto_register_rs_prop_acl ();}
  {extern void proto_register_rs_prop_attr (void); proto_register_rs_prop_attr ();}
  {extern void proto_register_rs_prop_pgo (void); proto_register_rs_prop_pgo ();}
  {extern void proto_register_rs_prop_plcy (void); proto_register_rs_prop_plcy ();}
  {extern void proto_register_rs_pwd_mgmt (void); proto_register_rs_pwd_mgmt ();}
  {extern void proto_register_rs_repadm (void); proto_register_rs_repadm ();}
  {extern void proto_register_rs_replist (void); proto_register_rs_replist ();}
  {extern void proto_register_rs_repmgr (void); proto_register_rs_repmgr ();}
  {extern void proto_register_rs_unix (void); proto_register_rs_unix ();}
  {extern void proto_register_rsec_login (void); proto_register_rsec_login ();}
  {extern void proto_register_rsh (void); proto_register_rsh ();}
  {extern void proto_register_rsp (void); proto_register_rsp ();}
  {extern void proto_register_rstat (void); proto_register_rstat ();}
  {extern void proto_register_rsvp (void); proto_register_rsvp ();}
  {extern void proto_register_rsync (void); proto_register_rsync ();}
  {extern void proto_register_rtcp (void); proto_register_rtcp ();}
  {extern void proto_register_rtp (void); proto_register_rtp ();}
  {extern void proto_register_rtp_events (void); proto_register_rtp_events ();}
  {extern void proto_register_rtps (void); proto_register_rtps ();}
  {extern void proto_register_rtse (void); proto_register_rtse ();}
  {extern void proto_register_rtsp (void); proto_register_rtsp ();}
  {extern void proto_register_rwall (void); proto_register_rwall ();}
  {extern void proto_register_rx (void); proto_register_rx ();}
  {extern void proto_register_s4406 (void); proto_register_s4406 ();}
  {extern void proto_register_s5066 (void); proto_register_s5066 ();}
  {extern void proto_register_sadmind (void); proto_register_sadmind ();}
  {extern void proto_register_sap (void); proto_register_sap ();}
  {extern void proto_register_sccp (void); proto_register_sccp ();}
  {extern void proto_register_sccpmg (void); proto_register_sccpmg ();}
  {extern void proto_register_scsi (void); proto_register_scsi ();}
  {extern void proto_register_scsi_mmc (void); proto_register_scsi_mmc ();}
  {extern void proto_register_scsi_osd (void); proto_register_scsi_osd ();}
  {extern void proto_register_scsi_sbc (void); proto_register_scsi_sbc ();}
  {extern void proto_register_scsi_smc (void); proto_register_scsi_smc ();}
  {extern void proto_register_scsi_ssc (void); proto_register_scsi_ssc ();}
  {extern void proto_register_sctp (void); proto_register_sctp ();}
  {extern void proto_register_sdlc (void); proto_register_sdlc ();}
  {extern void proto_register_sdp (void); proto_register_sdp ();}
  {extern void proto_register_sebek (void); proto_register_sebek ();}
  {extern void proto_register_secidmap (void); proto_register_secidmap ();}
  {extern void proto_register_ses (void); proto_register_ses ();}
  {extern void proto_register_sflow (void); proto_register_sflow ();}
  {extern void proto_register_sigcomp (void); proto_register_sigcomp ();}
  {extern void proto_register_sip (void); proto_register_sip ();}
  {extern void proto_register_sipfrag (void); proto_register_sipfrag ();}
  {extern void proto_register_sir (void); proto_register_sir ();}
  {extern void proto_register_skinny (void); proto_register_skinny ();}
  {extern void proto_register_slarp (void); proto_register_slarp ();}
  {extern void proto_register_slimp3 (void); proto_register_slimp3 ();}
  {extern void proto_register_sll (void); proto_register_sll ();}
  {extern void proto_register_slow_protocols (void); proto_register_slow_protocols ();}
  {extern void proto_register_slsk (void); proto_register_slsk ();}
  {extern void proto_register_smb (void); proto_register_smb ();}
  {extern void proto_register_smb2 (void); proto_register_smb2 ();}
  {extern void proto_register_smb_browse (void); proto_register_smb_browse ();}
  {extern void proto_register_smb_logon (void); proto_register_smb_logon ();}
  {extern void proto_register_smb_mailslot (void); proto_register_smb_mailslot ();}
  {extern void proto_register_smb_pipe (void); proto_register_smb_pipe ();}
  {extern void proto_register_smb_sidsnooping (void); proto_register_smb_sidsnooping ();}
  {extern void proto_register_smpp (void); proto_register_smpp ();}
  {extern void proto_register_smrse (void); proto_register_smrse ();}
  {extern void proto_register_smtp (void); proto_register_smtp ();}
  {extern void proto_register_smux (void); proto_register_smux ();}
  {extern void proto_register_sna (void); proto_register_sna ();}
  {extern void proto_register_snaeth (void); proto_register_snaeth ();}
  {extern void proto_register_sndcp (void); proto_register_sndcp ();}
  {extern void proto_register_snmp (void); proto_register_snmp ();}
  {extern void proto_register_socks (void); proto_register_socks ();}
  {extern void proto_register_sonmp (void); proto_register_sonmp ();}
  {extern void proto_register_spnego (void); proto_register_spnego ();}
  {extern void proto_register_spp (void); proto_register_spp ();}
  {extern void proto_register_spray (void); proto_register_spray ();}
  {extern void proto_register_srvloc (void); proto_register_srvloc ();}
  {extern void proto_register_sscf (void); proto_register_sscf ();}
  {extern void proto_register_sscop (void); proto_register_sscop ();}
  {extern void proto_register_ssh (void); proto_register_ssh ();}
  {extern void proto_register_ssl (void); proto_register_ssl ();}
  {extern void proto_register_sss (void); proto_register_sss ();}
  {extern void proto_register_stat (void); proto_register_stat ();}
  {extern void proto_register_statnotify (void); proto_register_statnotify ();}
  {extern void proto_register_stun (void); proto_register_stun ();}
  {extern void proto_register_stun2 (void); proto_register_stun2 ();}
  {extern void proto_register_sua (void); proto_register_sua ();}
  {extern void proto_register_symantec (void); proto_register_symantec ();}
  {extern void proto_register_synergy (void); proto_register_synergy ();}
  {extern void proto_register_syslog (void); proto_register_syslog ();}
  {extern void proto_register_t38 (void); proto_register_t38 ();}
  {extern void proto_register_tacacs (void); proto_register_tacacs ();}
  {extern void proto_register_tacplus (void); proto_register_tacplus ();}
  {extern void proto_register_tali (void); proto_register_tali ();}
  {extern void proto_register_tcap (void); proto_register_tcap ();}
  {extern void proto_register_tcp (void); proto_register_tcp ();}
  {extern void proto_register_teimanagement (void); proto_register_teimanagement ();}
  {extern void proto_register_telkonet (void); proto_register_telkonet ();}
  {extern void proto_register_telnet (void); proto_register_telnet ();}
  {extern void proto_register_teredo (void); proto_register_teredo ();}
  {extern void proto_register_text_lines (void); proto_register_text_lines ();}
  {extern void proto_register_tftp (void); proto_register_tftp ();}
  {extern void proto_register_time (void); proto_register_time ();}
  {extern void proto_register_tipc (void); proto_register_tipc ();}
  {extern void proto_register_tivoconnect (void); proto_register_tivoconnect ();}
  {extern void proto_register_tkn4int (void); proto_register_tkn4int ();}
  {extern void proto_register_tns (void); proto_register_tns ();}
  {extern void proto_register_tpcp (void); proto_register_tpcp ();}
  {extern void proto_register_tpkt (void); proto_register_tpkt ();}
  {extern void proto_register_tr (void); proto_register_tr ();}
  {extern void proto_register_trmac (void); proto_register_trmac ();}
  {extern void proto_register_tsp (void); proto_register_tsp ();}
  {extern void proto_register_tuxedo (void); proto_register_tuxedo ();}
  {extern void proto_register_tzsp (void); proto_register_tzsp ();}
  {extern void proto_register_ubikdisk (void); proto_register_ubikdisk ();}
  {extern void proto_register_ubikvote (void); proto_register_ubikvote ();}
  {extern void proto_register_ucp (void); proto_register_ucp ();}
  {extern void proto_register_udld (void); proto_register_udld ();}
  {extern void proto_register_udp (void); proto_register_udp ();}
  {extern void proto_register_udpencap (void); proto_register_udpencap ();}
  {extern void proto_register_ulp (void); proto_register_ulp ();}
  {extern void proto_register_uma (void); proto_register_uma ();}
  {extern void proto_register_umts_rrc (void); proto_register_umts_rrc ();}
  {extern void proto_register_umts_rrc_ies (void); proto_register_umts_rrc_ies ();}
  {extern void proto_register_umts_rrc_pdu_def (void); proto_register_umts_rrc_pdu_def ();}
  {extern void proto_register_usb (void); proto_register_usb ();}
  {extern void proto_register_usb_ms (void); proto_register_usb_ms ();}
  {extern void proto_register_user_encap (void); proto_register_user_encap ();}
  {extern void proto_register_v120 (void); proto_register_v120 ();}
  {extern void proto_register_vines_arp (void); proto_register_vines_arp ();}
  {extern void proto_register_vines_echo (void); proto_register_vines_echo ();}
  {extern void proto_register_vines_frp (void); proto_register_vines_frp ();}
  {extern void proto_register_vines_icp (void); proto_register_vines_icp ();}
  {extern void proto_register_vines_ip (void); proto_register_vines_ip ();}
  {extern void proto_register_vines_ipc (void); proto_register_vines_ipc ();}
  {extern void proto_register_vines_llc (void); proto_register_vines_llc ();}
  {extern void proto_register_vines_rtp (void); proto_register_vines_rtp ();}
  {extern void proto_register_vines_spp (void); proto_register_vines_spp ();}
  {extern void proto_register_vj (void); proto_register_vj ();}
  {extern void proto_register_vlan (void); proto_register_vlan ();}
  {extern void proto_register_vnc (void); proto_register_vnc ();}
  {extern void proto_register_vrrp (void); proto_register_vrrp ();}
  {extern void proto_register_vtp (void); proto_register_vtp ();}
  {extern void proto_register_wbxml (void); proto_register_wbxml ();}
  {extern void proto_register_wccp (void); proto_register_wccp ();}
  {extern void proto_register_wcp (void); proto_register_wcp ();}
  {extern void proto_register_wfleet_hdlc (void); proto_register_wfleet_hdlc ();}
  {extern void proto_register_who (void); proto_register_who ();}
  {extern void proto_register_winsrepl (void); proto_register_winsrepl ();}
  {extern void proto_register_wlancap (void); proto_register_wlancap ();}
  {extern void proto_register_wlancertextn (void); proto_register_wlancertextn ();}
  {extern void proto_register_wlccp (void); proto_register_wlccp ();}
  {extern void proto_register_wlccp_oui (void); proto_register_wlccp_oui ();}
  {extern void proto_register_wsp (void); proto_register_wsp ();}
  {extern void proto_register_wtls (void); proto_register_wtls ();}
  {extern void proto_register_wtp (void); proto_register_wtp ();}
  {extern void proto_register_x11 (void); proto_register_x11 ();}
  {extern void proto_register_x25 (void); proto_register_x25 ();}
  {extern void proto_register_x29 (void); proto_register_x29 ();}
  {extern void proto_register_x411 (void); proto_register_x411 ();}
  {extern void proto_register_x420 (void); proto_register_x420 ();}
  {extern void proto_register_x509af (void); proto_register_x509af ();}
  {extern void proto_register_x509ce (void); proto_register_x509ce ();}
  {extern void proto_register_x509if (void); proto_register_x509if ();}
  {extern void proto_register_x509sat (void); proto_register_x509sat ();}
  {extern void proto_register_xdmcp (void); proto_register_xdmcp ();}
  {extern void proto_register_xml (void); proto_register_xml ();}
  {extern void proto_register_xot (void); proto_register_xot ();}
  {extern void proto_register_xyplex (void); proto_register_xyplex ();}
  {extern void proto_register_yhoo (void); proto_register_yhoo ();}
  {extern void proto_register_ymsg (void); proto_register_ymsg ();}
  {extern void proto_register_ypbind (void); proto_register_ypbind ();}
  {extern void proto_register_yppasswd (void); proto_register_yppasswd ();}
  {extern void proto_register_ypserv (void); proto_register_ypserv ();}
  {extern void proto_register_ypxfr (void); proto_register_ypxfr ();}
  {extern void proto_register_zebra (void); proto_register_zebra ();}
}

void
register_all_protocol_handoffs(void)
{
  {extern void proto_reg_handoff_2dparityfec (void); proto_reg_handoff_2dparityfec ();}
  {extern void proto_reg_handoff_3com_xns (void); proto_reg_handoff_3com_xns ();}
  {extern void proto_reg_handoff_9P (void); proto_reg_handoff_9P ();}
  {extern void proto_reg_handoff_ISystemActivator (void); proto_reg_handoff_ISystemActivator ();}
  {extern void proto_reg_handoff_MAP_DialoguePDU (void); proto_reg_handoff_MAP_DialoguePDU ();}
  {extern void proto_reg_handoff_a11 (void); proto_reg_handoff_a11 ();}
  {extern void proto_reg_handoff_aarp (void); proto_reg_handoff_aarp ();}
  {extern void proto_reg_handoff_acap (void); proto_reg_handoff_acap ();}
  {extern void proto_reg_handoff_acn (void); proto_reg_handoff_acn ();}
  {extern void proto_reg_handoff_acp133 (void); proto_reg_handoff_acp133 ();}
  {extern void proto_reg_handoff_acse (void); proto_reg_handoff_acse ();}
  {extern void proto_reg_handoff_actrace (void); proto_reg_handoff_actrace ();}
  {extern void proto_reg_handoff_afp (void); proto_reg_handoff_afp ();}
  {extern void proto_reg_handoff_afs4int (void); proto_reg_handoff_afs4int ();}
  {extern void proto_reg_handoff_aim (void); proto_reg_handoff_aim ();}
  {extern void proto_reg_handoff_aim_admin (void); proto_reg_handoff_aim_admin ();}
  {extern void proto_reg_handoff_aim_adverts (void); proto_reg_handoff_aim_adverts ();}
  {extern void proto_reg_handoff_aim_bos (void); proto_reg_handoff_aim_bos ();}
  {extern void proto_reg_handoff_aim_buddylist (void); proto_reg_handoff_aim_buddylist ();}
  {extern void proto_reg_handoff_aim_chat (void); proto_reg_handoff_aim_chat ();}
  {extern void proto_reg_handoff_aim_chatnav (void); proto_reg_handoff_aim_chatnav ();}
  {extern void proto_reg_handoff_aim_directory (void); proto_reg_handoff_aim_directory ();}
  {extern void proto_reg_handoff_aim_email (void); proto_reg_handoff_aim_email ();}
  {extern void proto_reg_handoff_aim_generic (void); proto_reg_handoff_aim_generic ();}
  {extern void proto_reg_handoff_aim_icq (void); proto_reg_handoff_aim_icq ();}
  {extern void proto_reg_handoff_aim_invitation (void); proto_reg_handoff_aim_invitation ();}
  {extern void proto_reg_handoff_aim_location (void); proto_reg_handoff_aim_location ();}
  {extern void proto_reg_handoff_aim_messaging (void); proto_reg_handoff_aim_messaging ();}
  {extern void proto_reg_handoff_aim_oft (void); proto_reg_handoff_aim_oft ();}
  {extern void proto_reg_handoff_aim_popup (void); proto_reg_handoff_aim_popup ();}
  {extern void proto_reg_handoff_aim_signon (void); proto_reg_handoff_aim_signon ();}
  {extern void proto_reg_handoff_aim_ssi (void); proto_reg_handoff_aim_ssi ();}
  {extern void proto_reg_handoff_aim_sst (void); proto_reg_handoff_aim_sst ();}
  {extern void proto_reg_handoff_aim_stats (void); proto_reg_handoff_aim_stats ();}
  {extern void proto_reg_handoff_aim_translate (void); proto_reg_handoff_aim_translate ();}
  {extern void proto_reg_handoff_aim_userlookup (void); proto_reg_handoff_aim_userlookup ();}
  {extern void proto_reg_handoff_ajp13 (void); proto_reg_handoff_ajp13 ();}
  {extern void proto_reg_handoff_alc (void); proto_reg_handoff_alc ();}
  {extern void proto_reg_handoff_alcap (void); proto_reg_handoff_alcap ();}
  {extern void proto_reg_handoff_amr (void); proto_reg_handoff_amr ();}
  {extern void proto_reg_handoff_ans (void); proto_reg_handoff_ans ();}
  {extern void proto_reg_handoff_ansi_637 (void); proto_reg_handoff_ansi_637 ();}
  {extern void proto_reg_handoff_ansi_683 (void); proto_reg_handoff_ansi_683 ();}
  {extern void proto_reg_handoff_ansi_801 (void); proto_reg_handoff_ansi_801 ();}
  {extern void proto_reg_handoff_ansi_a (void); proto_reg_handoff_ansi_a ();}
  {extern void proto_reg_handoff_aodv (void); proto_reg_handoff_aodv ();}
  {extern void proto_reg_handoff_aoe (void); proto_reg_handoff_aoe ();}
  {extern void proto_reg_handoff_ap1394 (void); proto_reg_handoff_ap1394 ();}
  {extern void proto_reg_handoff_arcnet (void); proto_reg_handoff_arcnet ();}
  {extern void proto_reg_handoff_armagetronad (void); proto_reg_handoff_armagetronad ();}
  {extern void proto_reg_handoff_arp (void); proto_reg_handoff_arp ();}
  {extern void proto_reg_handoff_aruba_adp (void); proto_reg_handoff_aruba_adp ();}
  {extern void proto_reg_handoff_asap (void); proto_reg_handoff_asap ();}
  {extern void proto_reg_handoff_ascend (void); proto_reg_handoff_ascend ();}
  {extern void proto_reg_handoff_asf (void); proto_reg_handoff_asf ();}
  {extern void proto_reg_handoff_atalk (void); proto_reg_handoff_atalk ();}
  {extern void proto_reg_handoff_atm (void); proto_reg_handoff_atm ();}
  {extern void proto_reg_handoff_auto_rp (void); proto_reg_handoff_auto_rp ();}
  {extern void proto_reg_handoff_ax4000 (void); proto_reg_handoff_ax4000 ();}
  {extern void proto_reg_handoff_bacapp (void); proto_reg_handoff_bacapp ();}
  {extern void proto_reg_handoff_bacnet (void); proto_reg_handoff_bacnet ();}
  {extern void proto_reg_handoff_bacp (void); proto_reg_handoff_bacp ();}
  {extern void proto_reg_handoff_bap (void); proto_reg_handoff_bap ();}
  {extern void proto_reg_handoff_bcp (void); proto_reg_handoff_bcp ();}
  {extern void proto_reg_handoff_beep (void); proto_reg_handoff_beep ();}
  {extern void proto_reg_handoff_ber (void); proto_reg_handoff_ber ();}
  {extern void proto_reg_handoff_bfd (void); proto_reg_handoff_bfd ();}
  {extern void proto_reg_handoff_bgp (void); proto_reg_handoff_bgp ();}
  {extern void proto_reg_handoff_bicc (void); proto_reg_handoff_bicc ();}
  {extern void proto_reg_handoff_bittorrent (void); proto_reg_handoff_bittorrent ();}
  {extern void proto_reg_handoff_bofl (void); proto_reg_handoff_bofl ();}
  {extern void proto_reg_handoff_bootp (void); proto_reg_handoff_bootp ();}
  {extern void proto_reg_handoff_bootparams (void); proto_reg_handoff_bootparams ();}
  {extern void proto_reg_handoff_bossvr (void); proto_reg_handoff_bossvr ();}
  {extern void proto_reg_handoff_bpdu (void); proto_reg_handoff_bpdu ();}
  {extern void proto_reg_handoff_brdwlk (void); proto_reg_handoff_brdwlk ();}
  {extern void proto_reg_handoff_bssap (void); proto_reg_handoff_bssap ();}
  {extern void proto_reg_handoff_bssgp (void); proto_reg_handoff_bssgp ();}
  {extern void proto_reg_handoff_btacl (void); proto_reg_handoff_btacl ();}
  {extern void proto_reg_handoff_bthci_cmd (void); proto_reg_handoff_bthci_cmd ();}
  {extern void proto_reg_handoff_bthci_evt (void); proto_reg_handoff_bthci_evt ();}
  {extern void proto_reg_handoff_btl2cap (void); proto_reg_handoff_btl2cap ();}
  {extern void proto_reg_handoff_btrfcomm (void); proto_reg_handoff_btrfcomm ();}
  {extern void proto_reg_handoff_btsco (void); proto_reg_handoff_btsco ();}
  {extern void proto_reg_handoff_btsdp (void); proto_reg_handoff_btsdp ();}
  {extern void proto_reg_handoff_budb (void); proto_reg_handoff_budb ();}
  {extern void proto_reg_handoff_butc (void); proto_reg_handoff_butc ();}
  {extern void proto_reg_handoff_bvlc (void); proto_reg_handoff_bvlc ();}
  {extern void proto_reg_handoff_calcappprotocol (void); proto_reg_handoff_calcappprotocol ();}
  {extern void proto_reg_handoff_camel (void); proto_reg_handoff_camel ();}
  {extern void proto_reg_handoff_cast (void); proto_reg_handoff_cast ();}
  {extern void proto_reg_handoff_catapult_dct2000 (void); proto_reg_handoff_catapult_dct2000 ();}
  {extern void proto_reg_handoff_cbcp (void); proto_reg_handoff_cbcp ();}
  {extern void proto_reg_handoff_ccp (void); proto_reg_handoff_ccp ();}
  {extern void proto_reg_handoff_ccsds (void); proto_reg_handoff_ccsds ();}
  {extern void proto_reg_handoff_cdp (void); proto_reg_handoff_cdp ();}
  {extern void proto_reg_handoff_cdpcp (void); proto_reg_handoff_cdpcp ();}
  {extern void proto_reg_handoff_cds_clerkserver (void); proto_reg_handoff_cds_clerkserver ();}
  {extern void proto_reg_handoff_cds_solicit (void); proto_reg_handoff_cds_solicit ();}
  {extern void proto_reg_handoff_cdt (void); proto_reg_handoff_cdt ();}
  {extern void proto_reg_handoff_cert (void); proto_reg_handoff_cert ();}
  {extern void proto_reg_handoff_cgmp (void); proto_reg_handoff_cgmp ();}
  {extern void proto_reg_handoff_chap (void); proto_reg_handoff_chap ();}
  {extern void proto_reg_handoff_chdlc (void); proto_reg_handoff_chdlc ();}
  {extern void proto_reg_handoff_cigi (void); proto_reg_handoff_cigi ();}
  {extern void proto_reg_handoff_cimd (void); proto_reg_handoff_cimd ();}
  {extern void proto_reg_handoff_cip (void); proto_reg_handoff_cip ();}
  {extern void proto_reg_handoff_clearcase (void); proto_reg_handoff_clearcase ();}
  {extern void proto_reg_handoff_clip (void); proto_reg_handoff_clip ();}
  {extern void proto_reg_handoff_clnp (void); proto_reg_handoff_clnp ();}
  {extern void proto_reg_handoff_cmip (void); proto_reg_handoff_cmip ();}
  {extern void proto_reg_handoff_cmp (void); proto_reg_handoff_cmp ();}
  {extern void proto_reg_handoff_cms (void); proto_reg_handoff_cms ();}
  {extern void proto_reg_handoff_comp_data (void); proto_reg_handoff_comp_data ();}
  {extern void proto_reg_handoff_componentstatusprotocol (void); proto_reg_handoff_componentstatusprotocol ();}
  {extern void proto_reg_handoff_conv (void); proto_reg_handoff_conv ();}
  {extern void proto_reg_handoff_cops (void); proto_reg_handoff_cops ();}
  {extern void proto_reg_handoff_cosine (void); proto_reg_handoff_cosine ();}
  {extern void proto_reg_handoff_cotp (void); proto_reg_handoff_cotp ();}
  {extern void proto_reg_handoff_cpfi (void); proto_reg_handoff_cpfi ();}
  {extern void proto_reg_handoff_cpha (void); proto_reg_handoff_cpha ();}
  {extern void proto_reg_handoff_cprpc_server (void); proto_reg_handoff_cprpc_server ();}
  {extern void proto_reg_handoff_crmf (void); proto_reg_handoff_crmf ();}
  {extern void proto_reg_handoff_csm_encaps (void); proto_reg_handoff_csm_encaps ();}
  {extern void proto_reg_handoff_cups (void); proto_reg_handoff_cups ();}
  {extern void proto_reg_handoff_cwids (void); proto_reg_handoff_cwids ();}
  {extern void proto_reg_handoff_daap (void); proto_reg_handoff_daap ();}
  {extern void proto_reg_handoff_dap (void); proto_reg_handoff_dap ();}
  {extern void proto_reg_handoff_daytime (void); proto_reg_handoff_daytime ();}
  {extern void proto_reg_handoff_dccp (void); proto_reg_handoff_dccp ();}
  {extern void proto_reg_handoff_dce_update (void); proto_reg_handoff_dce_update ();}
  {extern void proto_reg_handoff_dcerpc (void); proto_reg_handoff_dcerpc ();}
  {extern void proto_reg_handoff_dcerpc_atsvc (void); proto_reg_handoff_dcerpc_atsvc ();}
  {extern void proto_reg_handoff_dcerpc_browser (void); proto_reg_handoff_dcerpc_browser ();}
  {extern void proto_reg_handoff_dcerpc_dnsserver (void); proto_reg_handoff_dcerpc_dnsserver ();}
  {extern void proto_reg_handoff_dcerpc_dssetup (void); proto_reg_handoff_dcerpc_dssetup ();}
  {extern void proto_reg_handoff_dcerpc_efs (void); proto_reg_handoff_dcerpc_efs ();}
  {extern void proto_reg_handoff_dcerpc_eventlog (void); proto_reg_handoff_dcerpc_eventlog ();}
  {extern void proto_reg_handoff_dcerpc_frsapi (void); proto_reg_handoff_dcerpc_frsapi ();}
  {extern void proto_reg_handoff_dcerpc_frsrpc (void); proto_reg_handoff_dcerpc_frsrpc ();}
  {extern void proto_reg_handoff_dcerpc_initshutdown (void); proto_reg_handoff_dcerpc_initshutdown ();}
  {extern void proto_reg_handoff_dcerpc_lsa (void); proto_reg_handoff_dcerpc_lsa ();}
  {extern void proto_reg_handoff_dcerpc_mapi (void); proto_reg_handoff_dcerpc_mapi ();}
  {extern void proto_reg_handoff_dcerpc_messenger (void); proto_reg_handoff_dcerpc_messenger ();}
  {extern void proto_reg_handoff_dcerpc_netdfs (void); proto_reg_handoff_dcerpc_netdfs ();}
  {extern void proto_reg_handoff_dcerpc_netlogon (void); proto_reg_handoff_dcerpc_netlogon ();}
  {extern void proto_reg_handoff_dcerpc_nspi (void); proto_reg_handoff_dcerpc_nspi ();}
  {extern void proto_reg_handoff_dcerpc_pnp (void); proto_reg_handoff_dcerpc_pnp ();}
  {extern void proto_reg_handoff_dcerpc_rras (void); proto_reg_handoff_dcerpc_rras ();}
  {extern void proto_reg_handoff_dcerpc_rs_plcy (void); proto_reg_handoff_dcerpc_rs_plcy ();}
  {extern void proto_reg_handoff_dcerpc_samr (void); proto_reg_handoff_dcerpc_samr ();}
  {extern void proto_reg_handoff_dcerpc_spoolss (void); proto_reg_handoff_dcerpc_spoolss ();}
  {extern void proto_reg_handoff_dcerpc_srvsvc (void); proto_reg_handoff_dcerpc_srvsvc ();}
  {extern void proto_reg_handoff_dcerpc_svcctl (void); proto_reg_handoff_dcerpc_svcctl ();}
  {extern void proto_reg_handoff_dcerpc_tapi (void); proto_reg_handoff_dcerpc_tapi ();}
  {extern void proto_reg_handoff_dcerpc_trksvr (void); proto_reg_handoff_dcerpc_trksvr ();}
  {extern void proto_reg_handoff_dcerpc_winreg (void); proto_reg_handoff_dcerpc_winreg ();}
  {extern void proto_reg_handoff_dcerpc_wkssvc (void); proto_reg_handoff_dcerpc_wkssvc ();}
  {extern void proto_reg_handoff_dcerpc_wzcsvc (void); proto_reg_handoff_dcerpc_wzcsvc ();}
  {extern void proto_reg_handoff_dcm (void); proto_reg_handoff_dcm ();}
  {extern void proto_reg_handoff_dcom (void); proto_reg_handoff_dcom ();}
  {extern void proto_reg_handoff_dcom_cba (void); proto_reg_handoff_dcom_cba ();}
  {extern void proto_reg_handoff_dcom_cba_acco (void); proto_reg_handoff_dcom_cba_acco ();}
  {extern void proto_reg_handoff_dcom_dispatch (void); proto_reg_handoff_dcom_dispatch ();}
  {extern void proto_reg_handoff_dcp (void); proto_reg_handoff_dcp ();}
  {extern void proto_reg_handoff_dcp_etsi (void); proto_reg_handoff_dcp_etsi ();}
  {extern void proto_reg_handoff_ddtp (void); proto_reg_handoff_ddtp ();}
  {extern void proto_reg_handoff_dec_bpdu (void); proto_reg_handoff_dec_bpdu ();}
  {extern void proto_reg_handoff_dec_rt (void); proto_reg_handoff_dec_rt ();}
  {extern void proto_reg_handoff_dhcpfo (void); proto_reg_handoff_dhcpfo ();}
  {extern void proto_reg_handoff_dhcpv6 (void); proto_reg_handoff_dhcpv6 ();}
  {extern void proto_reg_handoff_diameter (void); proto_reg_handoff_diameter ();}
  {extern void proto_reg_handoff_dis (void); proto_reg_handoff_dis ();}
  {extern void proto_reg_handoff_disp (void); proto_reg_handoff_disp ();}
  {extern void proto_reg_handoff_distcc (void); proto_reg_handoff_distcc ();}
  {extern void proto_reg_handoff_dlsw (void); proto_reg_handoff_dlsw ();}
  {extern void proto_reg_handoff_dmp (void); proto_reg_handoff_dmp ();}
  {extern void proto_reg_handoff_dnp3 (void); proto_reg_handoff_dnp3 ();}
  {extern void proto_reg_handoff_dns (void); proto_reg_handoff_dns ();}
  {extern void proto_reg_handoff_dop (void); proto_reg_handoff_dop ();}
  {extern void proto_reg_handoff_drsuapi (void); proto_reg_handoff_drsuapi ();}
  {extern void proto_reg_handoff_dsi (void); proto_reg_handoff_dsi ();}
  {extern void proto_reg_handoff_dsp (void); proto_reg_handoff_dsp ();}
  {extern void proto_reg_handoff_dtls (void); proto_reg_handoff_dtls ();}
  {extern void proto_reg_handoff_dtp (void); proto_reg_handoff_dtp ();}
  {extern void proto_reg_handoff_dtsprovider (void); proto_reg_handoff_dtsprovider ();}
  {extern void proto_reg_handoff_dtsstime_req (void); proto_reg_handoff_dtsstime_req ();}
  {extern void proto_reg_handoff_dua (void); proto_reg_handoff_dua ();}
  {extern void proto_reg_handoff_eap (void); proto_reg_handoff_eap ();}
  {extern void proto_reg_handoff_eapol (void); proto_reg_handoff_eapol ();}
  {extern void proto_reg_handoff_echo (void); proto_reg_handoff_echo ();}
  {extern void proto_reg_handoff_edonkey (void); proto_reg_handoff_edonkey ();}
  {extern void proto_reg_handoff_edp (void); proto_reg_handoff_edp ();}
  {extern void proto_reg_handoff_eigrp (void); proto_reg_handoff_eigrp ();}
  {extern void proto_reg_handoff_enc (void); proto_reg_handoff_enc ();}
  {extern void proto_reg_handoff_enip (void); proto_reg_handoff_enip ();}
  {extern void proto_reg_handoff_enrp (void); proto_reg_handoff_enrp ();}
  {extern void proto_reg_handoff_epl (void); proto_reg_handoff_epl ();}
  {extern void proto_reg_handoff_epl_v1 (void); proto_reg_handoff_epl_v1 ();}
  {extern void proto_reg_handoff_epm (void); proto_reg_handoff_epm ();}
  {extern void proto_reg_handoff_esis (void); proto_reg_handoff_esis ();}
  {extern void proto_reg_handoff_ess (void); proto_reg_handoff_ess ();}
  {extern void proto_reg_handoff_eth (void); proto_reg_handoff_eth ();}
  {extern void proto_reg_handoff_etheric (void); proto_reg_handoff_etheric ();}
  {extern void proto_reg_handoff_etherip (void); proto_reg_handoff_etherip ();}
  {extern void proto_reg_handoff_ethertype (void); proto_reg_handoff_ethertype ();}
  {extern void proto_reg_handoff_exec (void); proto_reg_handoff_exec ();}
  {extern void proto_reg_handoff_fc (void); proto_reg_handoff_fc ();}
  {extern void proto_reg_handoff_fcct (void); proto_reg_handoff_fcct ();}
  {extern void proto_reg_handoff_fcdns (void); proto_reg_handoff_fcdns ();}
  {extern void proto_reg_handoff_fcels (void); proto_reg_handoff_fcels ();}
  {extern void proto_reg_handoff_fcfcs (void); proto_reg_handoff_fcfcs ();}
  {extern void proto_reg_handoff_fcfzs (void); proto_reg_handoff_fcfzs ();}
  {extern void proto_reg_handoff_fcip (void); proto_reg_handoff_fcip ();}
  {extern void proto_reg_handoff_fcp (void); proto_reg_handoff_fcp ();}
  {extern void proto_reg_handoff_fcsbccs (void); proto_reg_handoff_fcsbccs ();}
  {extern void proto_reg_handoff_fcsp (void); proto_reg_handoff_fcsp ();}
  {extern void proto_reg_handoff_fcswils (void); proto_reg_handoff_fcswils ();}
  {extern void proto_reg_handoff_fddi (void); proto_reg_handoff_fddi ();}
  {extern void proto_reg_handoff_fix (void); proto_reg_handoff_fix ();}
  {extern void proto_reg_handoff_fldb (void); proto_reg_handoff_fldb ();}
  {extern void proto_reg_handoff_fp (void); proto_reg_handoff_fp ();}
  {extern void proto_reg_handoff_fr (void); proto_reg_handoff_fr ();}
  {extern void proto_reg_handoff_fractalgeneratorprotocol (void); proto_reg_handoff_fractalgeneratorprotocol ();}
  {extern void proto_reg_handoff_frame (void); proto_reg_handoff_frame ();}
  {extern void proto_reg_handoff_ftam (void); proto_reg_handoff_ftam ();}
  {extern void proto_reg_handoff_ftbp (void); proto_reg_handoff_ftbp ();}
  {extern void proto_reg_handoff_ftp (void); proto_reg_handoff_ftp ();}
  {extern void proto_reg_handoff_ftserver (void); proto_reg_handoff_ftserver ();}
  {extern void proto_reg_handoff_g723 (void); proto_reg_handoff_g723 ();}
  {extern void proto_reg_handoff_gif (void); proto_reg_handoff_gif ();}
  {extern void proto_reg_handoff_gift (void); proto_reg_handoff_gift ();}
  {extern void proto_reg_handoff_giop (void); proto_reg_handoff_giop ();}
  {extern void proto_reg_handoff_gmrp (void); proto_reg_handoff_gmrp ();}
  {extern void proto_reg_handoff_gnm (void); proto_reg_handoff_gnm ();}
  {extern void proto_reg_handoff_gnutella (void); proto_reg_handoff_gnutella ();}
  {extern void proto_reg_handoff_gprs_ns (void); proto_reg_handoff_gprs_ns ();}
  {extern void proto_reg_handoff_gre (void); proto_reg_handoff_gre ();}
  {extern void proto_reg_handoff_gsm_a (void); proto_reg_handoff_gsm_a ();}
  {extern void proto_reg_handoff_gsm_map (void); proto_reg_handoff_gsm_map ();}
  {extern void proto_reg_handoff_gsm_sms (void); proto_reg_handoff_gsm_sms ();}
  {extern void proto_reg_handoff_gsm_sms_ud (void); proto_reg_handoff_gsm_sms_ud ();}
  {extern void proto_reg_handoff_gsm_ss (void); proto_reg_handoff_gsm_ss ();}
  {extern void proto_reg_handoff_gssapi (void); proto_reg_handoff_gssapi ();}
  {extern void proto_reg_handoff_gtp (void); proto_reg_handoff_gtp ();}
  {extern void proto_reg_handoff_gvrp (void); proto_reg_handoff_gvrp ();}
  {extern void proto_reg_handoff_h1 (void); proto_reg_handoff_h1 ();}
  {extern void proto_reg_handoff_h225 (void); proto_reg_handoff_h225 ();}
  {extern void proto_reg_handoff_h235 (void); proto_reg_handoff_h235 ();}
  {extern void proto_reg_handoff_h245 (void); proto_reg_handoff_h245 ();}
  {extern void proto_reg_handoff_h248 (void); proto_reg_handoff_h248 ();}
  {extern void proto_reg_handoff_h248_3gpp (void); proto_reg_handoff_h248_3gpp ();}
  {extern void proto_reg_handoff_h248_annex_e (void); proto_reg_handoff_h248_annex_e ();}
  {extern void proto_reg_handoff_h261 (void); proto_reg_handoff_h261 ();}
  {extern void proto_reg_handoff_h263 (void); proto_reg_handoff_h263 ();}
  {extern void proto_reg_handoff_h4501 (void); proto_reg_handoff_h4501 ();}
  {extern void proto_reg_handoff_hci_h4 (void); proto_reg_handoff_hci_h4 ();}
  {extern void proto_reg_handoff_hclnfsd (void); proto_reg_handoff_hclnfsd ();}
  {extern void proto_reg_handoff_homeplug (void); proto_reg_handoff_homeplug ();}
  {extern void proto_reg_handoff_hpext (void); proto_reg_handoff_hpext ();}
  {extern void proto_reg_handoff_hpsw (void); proto_reg_handoff_hpsw ();}
  {extern void proto_reg_handoff_hsrp (void); proto_reg_handoff_hsrp ();}
  {extern void proto_reg_handoff_http (void); proto_reg_handoff_http ();}
  {extern void proto_reg_handoff_hyperscsi (void); proto_reg_handoff_hyperscsi ();}
  {extern void proto_reg_handoff_iapp (void); proto_reg_handoff_iapp ();}
  {extern void proto_reg_handoff_iax2 (void); proto_reg_handoff_iax2 ();}
  {extern void proto_reg_handoff_ib (void); proto_reg_handoff_ib ();}
  {extern void proto_reg_handoff_icap (void); proto_reg_handoff_icap ();}
  {extern void proto_reg_handoff_icep (void); proto_reg_handoff_icep ();}
  {extern void proto_reg_handoff_icl_rpc (void); proto_reg_handoff_icl_rpc ();}
  {extern void proto_reg_handoff_icmp (void); proto_reg_handoff_icmp ();}
  {extern void proto_reg_handoff_icmpv6 (void); proto_reg_handoff_icmpv6 ();}
  {extern void proto_reg_handoff_icp (void); proto_reg_handoff_icp ();}
  {extern void proto_reg_handoff_icq (void); proto_reg_handoff_icq ();}
  {extern void proto_reg_handoff_idp (void); proto_reg_handoff_idp ();}
  {extern void proto_reg_handoff_ieee80211 (void); proto_reg_handoff_ieee80211 ();}
  {extern void proto_reg_handoff_ieee802_3 (void); proto_reg_handoff_ieee802_3 ();}
  {extern void proto_reg_handoff_ieee802a (void); proto_reg_handoff_ieee802a ();}
  {extern void proto_reg_handoff_ifcp (void); proto_reg_handoff_ifcp ();}
  {extern void proto_reg_handoff_igmp (void); proto_reg_handoff_igmp ();}
  {extern void proto_reg_handoff_igrp (void); proto_reg_handoff_igrp ();}
  {extern void proto_reg_handoff_imap (void); proto_reg_handoff_imap ();}
  {extern void proto_reg_handoff_inap (void); proto_reg_handoff_inap ();}
  {extern void proto_reg_handoff_ip (void); proto_reg_handoff_ip ();}
  {extern void proto_reg_handoff_ipcp (void); proto_reg_handoff_ipcp ();}
  {extern void proto_reg_handoff_ipdc (void); proto_reg_handoff_ipdc ();}
  {extern void proto_reg_handoff_ipfc (void); proto_reg_handoff_ipfc ();}
  {extern void proto_reg_handoff_ipmi (void); proto_reg_handoff_ipmi ();}
  {extern void proto_reg_handoff_ipp (void); proto_reg_handoff_ipp ();}
  {extern void proto_reg_handoff_ipsec (void); proto_reg_handoff_ipsec ();}
  {extern void proto_reg_handoff_ipv6 (void); proto_reg_handoff_ipv6 ();}
  {extern void proto_reg_handoff_ipv6cp (void); proto_reg_handoff_ipv6cp ();}
  {extern void proto_reg_handoff_ipvs_syncd (void); proto_reg_handoff_ipvs_syncd ();}
  {extern void proto_reg_handoff_ipx (void); proto_reg_handoff_ipx ();}
  {extern void proto_reg_handoff_ipxwan (void); proto_reg_handoff_ipxwan ();}
  {extern void proto_reg_handoff_irc (void); proto_reg_handoff_irc ();}
  {extern void proto_reg_handoff_isakmp (void); proto_reg_handoff_isakmp ();}
  {extern void proto_reg_handoff_iscsi (void); proto_reg_handoff_iscsi ();}
  {extern void proto_reg_handoff_isdn (void); proto_reg_handoff_isdn ();}
  {extern void proto_reg_handoff_isis (void); proto_reg_handoff_isis ();}
  {extern void proto_reg_handoff_isl (void); proto_reg_handoff_isl ();}
  {extern void proto_reg_handoff_ismp (void); proto_reg_handoff_ismp ();}
  {extern void proto_reg_handoff_isns (void); proto_reg_handoff_isns ();}
  {extern void proto_reg_handoff_isup (void); proto_reg_handoff_isup ();}
  {extern void proto_reg_handoff_isup_thin (void); proto_reg_handoff_isup_thin ();}
  {extern void proto_reg_handoff_iua (void); proto_reg_handoff_iua ();}
  {extern void proto_reg_handoff_iuup (void); proto_reg_handoff_iuup ();}
  {extern void proto_reg_handoff_jabber (void); proto_reg_handoff_jabber ();}
  {extern void proto_reg_handoff_jfif (void); proto_reg_handoff_jfif ();}
  {extern void proto_reg_handoff_jpeg (void); proto_reg_handoff_jpeg ();}
  {extern void proto_reg_handoff_juniper (void); proto_reg_handoff_juniper ();}
  {extern void proto_reg_handoff_jxta (void); proto_reg_handoff_jxta ();}
  {extern void proto_reg_handoff_k12 (void); proto_reg_handoff_k12 ();}
  {extern void proto_reg_handoff_kadm5 (void); proto_reg_handoff_kadm5 ();}
  {extern void proto_reg_handoff_kerberos (void); proto_reg_handoff_kerberos ();}
  {extern void proto_reg_handoff_kink (void); proto_reg_handoff_kink ();}
  {extern void proto_reg_handoff_kismet (void); proto_reg_handoff_kismet ();}
  {extern void proto_reg_handoff_klm (void); proto_reg_handoff_klm ();}
  {extern void proto_reg_handoff_kpasswd (void); proto_reg_handoff_kpasswd ();}
  {extern void proto_reg_handoff_krb4 (void); proto_reg_handoff_krb4 ();}
  {extern void proto_reg_handoff_krb5rpc (void); proto_reg_handoff_krb5rpc ();}
  {extern void proto_reg_handoff_l2tp (void); proto_reg_handoff_l2tp ();}
  {extern void proto_reg_handoff_lapb (void); proto_reg_handoff_lapb ();}
  {extern void proto_reg_handoff_lapbether (void); proto_reg_handoff_lapbether ();}
  {extern void proto_reg_handoff_lapd (void); proto_reg_handoff_lapd ();}
  {extern void proto_reg_handoff_laplink (void); proto_reg_handoff_laplink ();}
  {extern void proto_reg_handoff_lcp (void); proto_reg_handoff_lcp ();}
  {extern void proto_reg_handoff_ldap (void); proto_reg_handoff_ldap ();}
  {extern void proto_reg_handoff_ldp (void); proto_reg_handoff_ldp ();}
  {extern void proto_reg_handoff_lge_monitor (void); proto_reg_handoff_lge_monitor ();}
  {extern void proto_reg_handoff_linx (void); proto_reg_handoff_linx ();}
  {extern void proto_reg_handoff_llb (void); proto_reg_handoff_llb ();}
  {extern void proto_reg_handoff_llc (void); proto_reg_handoff_llc ();}
  {extern void proto_reg_handoff_llcgprs (void); proto_reg_handoff_llcgprs ();}
  {extern void proto_reg_handoff_lldp (void); proto_reg_handoff_lldp ();}
  {extern void proto_reg_handoff_llt (void); proto_reg_handoff_llt ();}
  {extern void proto_reg_handoff_lmi (void); proto_reg_handoff_lmi ();}
  {extern void proto_reg_handoff_lmp (void); proto_reg_handoff_lmp ();}
  {extern void proto_reg_handoff_logotypecertextn (void); proto_reg_handoff_logotypecertextn ();}
  {extern void proto_reg_handoff_loop (void); proto_reg_handoff_loop ();}
  {extern void proto_reg_handoff_lpd (void); proto_reg_handoff_lpd ();}
  {extern void proto_reg_handoff_lsc (void); proto_reg_handoff_lsc ();}
  {extern void proto_reg_handoff_lwapp (void); proto_reg_handoff_lwapp ();}
  {extern void proto_reg_handoff_m2pa (void); proto_reg_handoff_m2pa ();}
  {extern void proto_reg_handoff_m2tp (void); proto_reg_handoff_m2tp ();}
  {extern void proto_reg_handoff_m2ua (void); proto_reg_handoff_m2ua ();}
  {extern void proto_reg_handoff_m3ua (void); proto_reg_handoff_m3ua ();}
  {extern void proto_reg_handoff_macctrl (void); proto_reg_handoff_macctrl ();}
  {extern void proto_reg_handoff_manolito (void); proto_reg_handoff_manolito ();}
  {extern void proto_reg_handoff_mbtcp (void); proto_reg_handoff_mbtcp ();}
  {extern void proto_reg_handoff_mdshdr (void); proto_reg_handoff_mdshdr ();}
  {extern void proto_reg_handoff_message_http (void); proto_reg_handoff_message_http ();}
  {extern void proto_reg_handoff_mgmt (void); proto_reg_handoff_mgmt ();}
  {extern void proto_reg_handoff_mip (void); proto_reg_handoff_mip ();}
  {extern void proto_reg_handoff_mip6 (void); proto_reg_handoff_mip6 ();}
  {extern void proto_reg_handoff_mms (void); proto_reg_handoff_mms ();}
  {extern void proto_reg_handoff_mmse (void); proto_reg_handoff_mmse ();}
  {extern void proto_reg_handoff_mount (void); proto_reg_handoff_mount ();}
  {extern void proto_reg_handoff_mp (void); proto_reg_handoff_mp ();}
  {extern void proto_reg_handoff_mp2t (void); proto_reg_handoff_mp2t ();}
  {extern void proto_reg_handoff_mpeg1 (void); proto_reg_handoff_mpeg1 ();}
  {extern void proto_reg_handoff_mpls (void); proto_reg_handoff_mpls ();}
  {extern void proto_reg_handoff_mpls_echo (void); proto_reg_handoff_mpls_echo ();}
  {extern void proto_reg_handoff_mplscp (void); proto_reg_handoff_mplscp ();}
  {extern void proto_reg_handoff_mq (void); proto_reg_handoff_mq ();}
  {extern void proto_reg_handoff_mqpcf (void); proto_reg_handoff_mqpcf ();}
  {extern void proto_reg_handoff_msdp (void); proto_reg_handoff_msdp ();}
  {extern void proto_reg_handoff_msmms_command (void); proto_reg_handoff_msmms_command ();}
  {extern void proto_reg_handoff_msnlb (void); proto_reg_handoff_msnlb ();}
  {extern void proto_reg_handoff_msnms (void); proto_reg_handoff_msnms ();}
  {extern void proto_reg_handoff_msproxy (void); proto_reg_handoff_msproxy ();}
  {extern void proto_reg_handoff_msrp (void); proto_reg_handoff_msrp ();}
  {extern void proto_reg_handoff_mtp2 (void); proto_reg_handoff_mtp2 ();}
  {extern void proto_reg_handoff_mtp3 (void); proto_reg_handoff_mtp3 ();}
  {extern void proto_reg_handoff_mtp3mg (void); proto_reg_handoff_mtp3mg ();}
  {extern void proto_reg_handoff_multipart (void); proto_reg_handoff_multipart ();}
  {extern void proto_reg_handoff_mysql (void); proto_reg_handoff_mysql ();}
  {extern void proto_reg_handoff_nbap (void); proto_reg_handoff_nbap ();}
  {extern void proto_reg_handoff_nbd (void); proto_reg_handoff_nbd ();}
  {extern void proto_reg_handoff_nbipx (void); proto_reg_handoff_nbipx ();}
  {extern void proto_reg_handoff_nbt (void); proto_reg_handoff_nbt ();}
  {extern void proto_reg_handoff_ncp (void); proto_reg_handoff_ncp ();}
  {extern void proto_reg_handoff_ncs (void); proto_reg_handoff_ncs ();}
  {extern void proto_reg_handoff_ndmp (void); proto_reg_handoff_ndmp ();}
  {extern void proto_reg_handoff_ndps (void); proto_reg_handoff_ndps ();}
  {extern void proto_reg_handoff_netbios (void); proto_reg_handoff_netbios ();}
  {extern void proto_reg_handoff_netflow (void); proto_reg_handoff_netflow ();}
  {extern void proto_reg_handoff_netsync (void); proto_reg_handoff_netsync ();}
  {extern void proto_reg_handoff_nettl (void); proto_reg_handoff_nettl ();}
  {extern void proto_reg_handoff_newmail (void); proto_reg_handoff_newmail ();}
  {extern void proto_reg_handoff_nfs (void); proto_reg_handoff_nfs ();}
  {extern void proto_reg_handoff_nfsacl (void); proto_reg_handoff_nfsacl ();}
  {extern void proto_reg_handoff_nfsauth (void); proto_reg_handoff_nfsauth ();}
  {extern void proto_reg_handoff_nhrp (void); proto_reg_handoff_nhrp ();}
  {extern void proto_reg_handoff_nis (void); proto_reg_handoff_nis ();}
  {extern void proto_reg_handoff_niscb (void); proto_reg_handoff_niscb ();}
  {extern void proto_reg_handoff_njack (void); proto_reg_handoff_njack ();}
  {extern void proto_reg_handoff_nlm (void); proto_reg_handoff_nlm ();}
  {extern void proto_reg_handoff_nlsp (void); proto_reg_handoff_nlsp ();}
  {extern void proto_reg_handoff_nmpi (void); proto_reg_handoff_nmpi ();}
  {extern void proto_reg_handoff_nntp (void); proto_reg_handoff_nntp ();}
  {extern void proto_reg_handoff_nonstd (void); proto_reg_handoff_nonstd ();}
  {extern void proto_reg_handoff_norm (void); proto_reg_handoff_norm ();}
  {extern void proto_reg_handoff_ns_cert_exts (void); proto_reg_handoff_ns_cert_exts ();}
  {extern void proto_reg_handoff_nsip (void); proto_reg_handoff_nsip ();}
  {extern void proto_reg_handoff_ntlmssp (void); proto_reg_handoff_ntlmssp ();}
  {extern void proto_reg_handoff_ntp (void); proto_reg_handoff_ntp ();}
  {extern void proto_reg_handoff_null (void); proto_reg_handoff_null ();}
  {extern void proto_reg_handoff_ocsp (void); proto_reg_handoff_ocsp ();}
  {extern void proto_reg_handoff_oicq (void); proto_reg_handoff_oicq ();}
  {extern void proto_reg_handoff_old_pflog (void); proto_reg_handoff_old_pflog ();}
  {extern void proto_reg_handoff_olsr (void); proto_reg_handoff_olsr ();}
  {extern void proto_reg_handoff_omapi (void); proto_reg_handoff_omapi ();}
  {extern void proto_reg_handoff_osi (void); proto_reg_handoff_osi ();}
  {extern void proto_reg_handoff_osicp (void); proto_reg_handoff_osicp ();}
  {extern void proto_reg_handoff_ospf (void); proto_reg_handoff_ospf ();}
  {extern void proto_reg_handoff_oxid (void); proto_reg_handoff_oxid ();}
  {extern void proto_reg_handoff_p_mul (void); proto_reg_handoff_p_mul ();}
  {extern void proto_reg_handoff_packetcable (void); proto_reg_handoff_packetcable ();}
  {extern void proto_reg_handoff_pagp (void); proto_reg_handoff_pagp ();}
  {extern void proto_reg_handoff_pana (void); proto_reg_handoff_pana ();}
  {extern void proto_reg_handoff_pap (void); proto_reg_handoff_pap ();}
  {extern void proto_reg_handoff_pcnfsd (void); proto_reg_handoff_pcnfsd ();}
  {extern void proto_reg_handoff_per (void); proto_reg_handoff_per ();}
  {extern void proto_reg_handoff_pflog (void); proto_reg_handoff_pflog ();}
  {extern void proto_reg_handoff_pgm (void); proto_reg_handoff_pgm ();}
  {extern void proto_reg_handoff_pgsql (void); proto_reg_handoff_pgsql ();}
  {extern void proto_reg_handoff_pim (void); proto_reg_handoff_pim ();}
  {extern void proto_reg_handoff_pingpongprotocol (void); proto_reg_handoff_pingpongprotocol ();}
  {extern void proto_reg_handoff_pkcs1 (void); proto_reg_handoff_pkcs1 ();}
  {extern void proto_reg_handoff_pkcs12 (void); proto_reg_handoff_pkcs12 ();}
  {extern void proto_reg_handoff_pkinit (void); proto_reg_handoff_pkinit ();}
  {extern void proto_reg_handoff_pkix1explicit (void); proto_reg_handoff_pkix1explicit ();}
  {extern void proto_reg_handoff_pkix1implicit (void); proto_reg_handoff_pkix1implicit ();}
  {extern void proto_reg_handoff_pkixproxy (void); proto_reg_handoff_pkixproxy ();}
  {extern void proto_reg_handoff_pkixqualified (void); proto_reg_handoff_pkixqualified ();}
  {extern void proto_reg_handoff_pkixtsp (void); proto_reg_handoff_pkixtsp ();}
  {extern void proto_reg_handoff_pkt_ccc (void); proto_reg_handoff_pkt_ccc ();}
  {extern void proto_reg_handoff_pktc (void); proto_reg_handoff_pktc ();}
  {extern void proto_reg_handoff_pktc_mtafqdn (void); proto_reg_handoff_pktc_mtafqdn ();}
  {extern void proto_reg_handoff_pktgen (void); proto_reg_handoff_pktgen ();}
  {extern void proto_reg_handoff_pn_rt (void); proto_reg_handoff_pn_rt ();}
  {extern void proto_reg_handoff_png (void); proto_reg_handoff_png ();}
  {extern void proto_reg_handoff_pop (void); proto_reg_handoff_pop ();}
  {extern void proto_reg_handoff_portmap (void); proto_reg_handoff_portmap ();}
  {extern void proto_reg_handoff_ppp (void); proto_reg_handoff_ppp ();}
  {extern void proto_reg_handoff_ppp_raw_hdlc (void); proto_reg_handoff_ppp_raw_hdlc ();}
  {extern void proto_reg_handoff_pppmux (void); proto_reg_handoff_pppmux ();}
  {extern void proto_reg_handoff_pppmuxcp (void); proto_reg_handoff_pppmuxcp ();}
  {extern void proto_reg_handoff_pppoed (void); proto_reg_handoff_pppoed ();}
  {extern void proto_reg_handoff_pppoes (void); proto_reg_handoff_pppoes ();}
  {extern void proto_reg_handoff_pptp (void); proto_reg_handoff_pptp ();}
  {extern void proto_reg_handoff_pres (void); proto_reg_handoff_pres ();}
  {extern void proto_reg_handoff_prism (void); proto_reg_handoff_prism ();}
  {extern void proto_reg_handoff_ptp (void); proto_reg_handoff_ptp ();}
  {extern void proto_reg_handoff_pvfs (void); proto_reg_handoff_pvfs ();}
  {extern void proto_reg_handoff_q1950 (void); proto_reg_handoff_q1950 ();}
  {extern void proto_reg_handoff_q931 (void); proto_reg_handoff_q931 ();}
  {extern void proto_reg_handoff_q933 (void); proto_reg_handoff_q933 ();}
  {extern void proto_reg_handoff_qllc (void); proto_reg_handoff_qllc ();}
  {extern void proto_reg_handoff_quake (void); proto_reg_handoff_quake ();}
  {extern void proto_reg_handoff_quake2 (void); proto_reg_handoff_quake2 ();}
  {extern void proto_reg_handoff_quake3 (void); proto_reg_handoff_quake3 ();}
  {extern void proto_reg_handoff_quakeworld (void); proto_reg_handoff_quakeworld ();}
  {extern void proto_reg_handoff_radiotap (void); proto_reg_handoff_radiotap ();}
  {extern void proto_reg_handoff_radius (void); proto_reg_handoff_radius ();}
  {extern void proto_reg_handoff_ranap (void); proto_reg_handoff_ranap ();}
  {extern void proto_reg_handoff_raw (void); proto_reg_handoff_raw ();}
  {extern void proto_reg_handoff_rdaclif (void); proto_reg_handoff_rdaclif ();}
  {extern void proto_reg_handoff_rdm (void); proto_reg_handoff_rdm ();}
  {extern void proto_reg_handoff_rdt (void); proto_reg_handoff_rdt ();}
  {extern void proto_reg_handoff_redback (void); proto_reg_handoff_redback ();}
  {extern void proto_reg_handoff_remact (void); proto_reg_handoff_remact ();}
  {extern void proto_reg_handoff_remunk (void); proto_reg_handoff_remunk ();}
  {extern void proto_reg_handoff_rep_proc (void); proto_reg_handoff_rep_proc ();}
  {extern void proto_reg_handoff_retix_bpdu (void); proto_reg_handoff_retix_bpdu ();}
  {extern void proto_reg_handoff_rip (void); proto_reg_handoff_rip ();}
  {extern void proto_reg_handoff_ripng (void); proto_reg_handoff_ripng ();}
  {extern void proto_reg_handoff_rlogin (void); proto_reg_handoff_rlogin ();}
  {extern void proto_reg_handoff_rmcp (void); proto_reg_handoff_rmcp ();}
  {extern void proto_reg_handoff_rmi (void); proto_reg_handoff_rmi ();}
  {extern void proto_reg_handoff_rmp (void); proto_reg_handoff_rmp ();}
  {extern void proto_reg_handoff_rnsap (void); proto_reg_handoff_rnsap ();}
  {extern void proto_reg_handoff_roofnet (void); proto_reg_handoff_roofnet ();}
  {extern void proto_reg_handoff_ros (void); proto_reg_handoff_ros ();}
  {extern void proto_reg_handoff_roverride (void); proto_reg_handoff_roverride ();}
  {extern void proto_reg_handoff_rpc (void); proto_reg_handoff_rpc ();}
  {extern void proto_reg_handoff_rpl (void); proto_reg_handoff_rpl ();}
  {extern void proto_reg_handoff_rpriv (void); proto_reg_handoff_rpriv ();}
  {extern void proto_reg_handoff_rquota (void); proto_reg_handoff_rquota ();}
  {extern void proto_reg_handoff_rrlp (void); proto_reg_handoff_rrlp ();}
  {extern void proto_reg_handoff_rs_acct (void); proto_reg_handoff_rs_acct ();}
  {extern void proto_reg_handoff_rs_attr (void); proto_reg_handoff_rs_attr ();}
  {extern void proto_reg_handoff_rs_attr_schema (void); proto_reg_handoff_rs_attr_schema ();}
  {extern void proto_reg_handoff_rs_bind (void); proto_reg_handoff_rs_bind ();}
  {extern void proto_reg_handoff_rs_misc (void); proto_reg_handoff_rs_misc ();}
  {extern void proto_reg_handoff_rs_pgo (void); proto_reg_handoff_rs_pgo ();}
  {extern void proto_reg_handoff_rs_prop_acct (void); proto_reg_handoff_rs_prop_acct ();}
  {extern void proto_reg_handoff_rs_prop_acl (void); proto_reg_handoff_rs_prop_acl ();}
  {extern void proto_reg_handoff_rs_prop_attr (void); proto_reg_handoff_rs_prop_attr ();}
  {extern void proto_reg_handoff_rs_prop_pgo (void); proto_reg_handoff_rs_prop_pgo ();}
  {extern void proto_reg_handoff_rs_prop_plcy (void); proto_reg_handoff_rs_prop_plcy ();}
  {extern void proto_reg_handoff_rs_pwd_mgmt (void); proto_reg_handoff_rs_pwd_mgmt ();}
  {extern void proto_reg_handoff_rs_repadm (void); proto_reg_handoff_rs_repadm ();}
  {extern void proto_reg_handoff_rs_replist (void); proto_reg_handoff_rs_replist ();}
  {extern void proto_reg_handoff_rs_repmgr (void); proto_reg_handoff_rs_repmgr ();}
  {extern void proto_reg_handoff_rs_unix (void); proto_reg_handoff_rs_unix ();}
  {extern void proto_reg_handoff_rsec_login (void); proto_reg_handoff_rsec_login ();}
  {extern void proto_reg_handoff_rsh (void); proto_reg_handoff_rsh ();}
  {extern void proto_reg_handoff_rsp (void); proto_reg_handoff_rsp ();}
  {extern void proto_reg_handoff_rstat (void); proto_reg_handoff_rstat ();}
  {extern void proto_reg_handoff_rsvp (void); proto_reg_handoff_rsvp ();}
  {extern void proto_reg_handoff_rsync (void); proto_reg_handoff_rsync ();}
  {extern void proto_reg_handoff_rtcp (void); proto_reg_handoff_rtcp ();}
  {extern void proto_reg_handoff_rtp (void); proto_reg_handoff_rtp ();}
  {extern void proto_reg_handoff_rtp_events (void); proto_reg_handoff_rtp_events ();}
  {extern void proto_reg_handoff_rtps (void); proto_reg_handoff_rtps ();}
  {extern void proto_reg_handoff_rtse (void); proto_reg_handoff_rtse ();}
  {extern void proto_reg_handoff_rtsp (void); proto_reg_handoff_rtsp ();}
  {extern void proto_reg_handoff_rwall (void); proto_reg_handoff_rwall ();}
  {extern void proto_reg_handoff_rx (void); proto_reg_handoff_rx ();}
  {extern void proto_reg_handoff_s4406 (void); proto_reg_handoff_s4406 ();}
  {extern void proto_reg_handoff_s5066 (void); proto_reg_handoff_s5066 ();}
  {extern void proto_reg_handoff_sadmind (void); proto_reg_handoff_sadmind ();}
  {extern void proto_reg_handoff_sap (void); proto_reg_handoff_sap ();}
  {extern void proto_reg_handoff_sccp (void); proto_reg_handoff_sccp ();}
  {extern void proto_reg_handoff_sccpmg (void); proto_reg_handoff_sccpmg ();}
  {extern void proto_reg_handoff_scsi (void); proto_reg_handoff_scsi ();}
  {extern void proto_reg_handoff_scsi_mmc (void); proto_reg_handoff_scsi_mmc ();}
  {extern void proto_reg_handoff_scsi_osd (void); proto_reg_handoff_scsi_osd ();}
  {extern void proto_reg_handoff_scsi_sbc (void); proto_reg_handoff_scsi_sbc ();}
  {extern void proto_reg_handoff_scsi_smc (void); proto_reg_handoff_scsi_smc ();}
  {extern void proto_reg_handoff_scsi_ssc (void); proto_reg_handoff_scsi_ssc ();}
  {extern void proto_reg_handoff_sctp (void); proto_reg_handoff_sctp ();}
  {extern void proto_reg_handoff_sdlc (void); proto_reg_handoff_sdlc ();}
  {extern void proto_reg_handoff_sdp (void); proto_reg_handoff_sdp ();}
  {extern void proto_reg_handoff_sebek (void); proto_reg_handoff_sebek ();}
  {extern void proto_reg_handoff_secidmap (void); proto_reg_handoff_secidmap ();}
  {extern void proto_reg_handoff_ses (void); proto_reg_handoff_ses ();}
  {extern void proto_reg_handoff_sflow (void); proto_reg_handoff_sflow ();}
  {extern void proto_reg_handoff_sigcomp (void); proto_reg_handoff_sigcomp ();}
  {extern void proto_reg_handoff_sip (void); proto_reg_handoff_sip ();}
  {extern void proto_reg_handoff_sipfrag (void); proto_reg_handoff_sipfrag ();}
  {extern void proto_reg_handoff_sir (void); proto_reg_handoff_sir ();}
  {extern void proto_reg_handoff_skinny (void); proto_reg_handoff_skinny ();}
  {extern void proto_reg_handoff_slarp (void); proto_reg_handoff_slarp ();}
  {extern void proto_reg_handoff_slimp3 (void); proto_reg_handoff_slimp3 ();}
  {extern void proto_reg_handoff_sll (void); proto_reg_handoff_sll ();}
  {extern void proto_reg_handoff_slow_protocols (void); proto_reg_handoff_slow_protocols ();}
  {extern void proto_reg_handoff_slsk (void); proto_reg_handoff_slsk ();}
  {extern void proto_reg_handoff_smb (void); proto_reg_handoff_smb ();}
  {extern void proto_reg_handoff_smb2 (void); proto_reg_handoff_smb2 ();}
  {extern void proto_reg_handoff_smb_mailslot (void); proto_reg_handoff_smb_mailslot ();}
  {extern void proto_reg_handoff_smb_pipe (void); proto_reg_handoff_smb_pipe ();}
  {extern void proto_reg_handoff_smb_sidsnooping (void); proto_reg_handoff_smb_sidsnooping ();}
  {extern void proto_reg_handoff_smpp (void); proto_reg_handoff_smpp ();}
  {extern void proto_reg_handoff_smrse (void); proto_reg_handoff_smrse ();}
  {extern void proto_reg_handoff_smtp (void); proto_reg_handoff_smtp ();}
  {extern void proto_reg_handoff_smux (void); proto_reg_handoff_smux ();}
  {extern void proto_reg_handoff_sna (void); proto_reg_handoff_sna ();}
  {extern void proto_reg_handoff_snaeth (void); proto_reg_handoff_snaeth ();}
  {extern void proto_reg_handoff_sndcp (void); proto_reg_handoff_sndcp ();}
  {extern void proto_reg_handoff_snmp (void); proto_reg_handoff_snmp ();}
  {extern void proto_reg_handoff_socks (void); proto_reg_handoff_socks ();}
  {extern void proto_reg_handoff_sonmp (void); proto_reg_handoff_sonmp ();}
  {extern void proto_reg_handoff_spnego (void); proto_reg_handoff_spnego ();}
  {extern void proto_reg_handoff_spp (void); proto_reg_handoff_spp ();}
  {extern void proto_reg_handoff_spray (void); proto_reg_handoff_spray ();}
  {extern void proto_reg_handoff_srvloc (void); proto_reg_handoff_srvloc ();}
  {extern void proto_reg_handoff_sscf (void); proto_reg_handoff_sscf ();}
  {extern void proto_reg_handoff_sscop (void); proto_reg_handoff_sscop ();}
  {extern void proto_reg_handoff_ssh (void); proto_reg_handoff_ssh ();}
  {extern void proto_reg_handoff_ssl (void); proto_reg_handoff_ssl ();}
  {extern void proto_reg_handoff_stat (void); proto_reg_handoff_stat ();}
  {extern void proto_reg_handoff_statnotify (void); proto_reg_handoff_statnotify ();}
  {extern void proto_reg_handoff_stun (void); proto_reg_handoff_stun ();}
  {extern void proto_reg_handoff_stun2 (void); proto_reg_handoff_stun2 ();}
  {extern void proto_reg_handoff_sua (void); proto_reg_handoff_sua ();}
  {extern void proto_reg_handoff_symantec (void); proto_reg_handoff_symantec ();}
  {extern void proto_reg_handoff_synergy (void); proto_reg_handoff_synergy ();}
  {extern void proto_reg_handoff_syslog (void); proto_reg_handoff_syslog ();}
  {extern void proto_reg_handoff_t38 (void); proto_reg_handoff_t38 ();}
  {extern void proto_reg_handoff_tacacs (void); proto_reg_handoff_tacacs ();}
  {extern void proto_reg_handoff_tacplus (void); proto_reg_handoff_tacplus ();}
  {extern void proto_reg_handoff_tali (void); proto_reg_handoff_tali ();}
  {extern void proto_reg_handoff_tcap (void); proto_reg_handoff_tcap ();}
  {extern void proto_reg_handoff_tcp (void); proto_reg_handoff_tcp ();}
  {extern void proto_reg_handoff_tds (void); proto_reg_handoff_tds ();}
  {extern void proto_reg_handoff_teimanagement (void); proto_reg_handoff_teimanagement ();}
  {extern void proto_reg_handoff_telkonet (void); proto_reg_handoff_telkonet ();}
  {extern void proto_reg_handoff_telnet (void); proto_reg_handoff_telnet ();}
  {extern void proto_reg_handoff_teredo (void); proto_reg_handoff_teredo ();}
  {extern void proto_reg_handoff_text_lines (void); proto_reg_handoff_text_lines ();}
  {extern void proto_reg_handoff_tftp (void); proto_reg_handoff_tftp ();}
  {extern void proto_reg_handoff_time (void); proto_reg_handoff_time ();}
  {extern void proto_reg_handoff_tipc (void); proto_reg_handoff_tipc ();}
  {extern void proto_reg_handoff_tivoconnect (void); proto_reg_handoff_tivoconnect ();}
  {extern void proto_reg_handoff_tkn4int (void); proto_reg_handoff_tkn4int ();}
  {extern void proto_reg_handoff_tns (void); proto_reg_handoff_tns ();}
  {extern void proto_reg_handoff_tpcp (void); proto_reg_handoff_tpcp ();}
  {extern void proto_reg_handoff_tpkt (void); proto_reg_handoff_tpkt ();}
  {extern void proto_reg_handoff_tr (void); proto_reg_handoff_tr ();}
  {extern void proto_reg_handoff_tsp (void); proto_reg_handoff_tsp ();}
  {extern void proto_reg_handoff_tuxedo (void); proto_reg_handoff_tuxedo ();}
  {extern void proto_reg_handoff_tzsp (void); proto_reg_handoff_tzsp ();}
  {extern void proto_reg_handoff_ubikdisk (void); proto_reg_handoff_ubikdisk ();}
  {extern void proto_reg_handoff_ubikvote (void); proto_reg_handoff_ubikvote ();}
  {extern void proto_reg_handoff_ucp (void); proto_reg_handoff_ucp ();}
  {extern void proto_reg_handoff_udld (void); proto_reg_handoff_udld ();}
  {extern void proto_reg_handoff_udp (void); proto_reg_handoff_udp ();}
  {extern void proto_reg_handoff_udpencap (void); proto_reg_handoff_udpencap ();}
  {extern void proto_reg_handoff_ulp (void); proto_reg_handoff_ulp ();}
  {extern void proto_reg_handoff_uma (void); proto_reg_handoff_uma ();}
  {extern void proto_reg_handoff_umts_rrc (void); proto_reg_handoff_umts_rrc ();}
  {extern void proto_reg_handoff_umts_rrc_ies (void); proto_reg_handoff_umts_rrc_ies ();}
  {extern void proto_reg_handoff_umts_rrc_pdu_def (void); proto_reg_handoff_umts_rrc_pdu_def ();}
  {extern void proto_reg_handoff_usb (void); proto_reg_handoff_usb ();}
  {extern void proto_reg_handoff_usb_ms (void); proto_reg_handoff_usb_ms ();}
  {extern void proto_reg_handoff_user_encap (void); proto_reg_handoff_user_encap ();}
  {extern void proto_reg_handoff_v120 (void); proto_reg_handoff_v120 ();}
  {extern void proto_reg_handoff_vines_arp (void); proto_reg_handoff_vines_arp ();}
  {extern void proto_reg_handoff_vines_echo (void); proto_reg_handoff_vines_echo ();}
  {extern void proto_reg_handoff_vines_frp (void); proto_reg_handoff_vines_frp ();}
  {extern void proto_reg_handoff_vines_icp (void); proto_reg_handoff_vines_icp ();}
  {extern void proto_reg_handoff_vines_ip (void); proto_reg_handoff_vines_ip ();}
  {extern void proto_reg_handoff_vines_ipc (void); proto_reg_handoff_vines_ipc ();}
  {extern void proto_reg_handoff_vines_llc (void); proto_reg_handoff_vines_llc ();}
  {extern void proto_reg_handoff_vines_rtp (void); proto_reg_handoff_vines_rtp ();}
  {extern void proto_reg_handoff_vines_spp (void); proto_reg_handoff_vines_spp ();}
  {extern void proto_reg_handoff_vj (void); proto_reg_handoff_vj ();}
  {extern void proto_reg_handoff_vlan (void); proto_reg_handoff_vlan ();}
  {extern void proto_reg_handoff_vnc (void); proto_reg_handoff_vnc ();}
  {extern void proto_reg_handoff_vrrp (void); proto_reg_handoff_vrrp ();}
  {extern void proto_reg_handoff_vtp (void); proto_reg_handoff_vtp ();}
  {extern void proto_reg_handoff_wbxml (void); proto_reg_handoff_wbxml ();}
  {extern void proto_reg_handoff_wccp (void); proto_reg_handoff_wccp ();}
  {extern void proto_reg_handoff_wcp (void); proto_reg_handoff_wcp ();}
  {extern void proto_reg_handoff_wfleet_hdlc (void); proto_reg_handoff_wfleet_hdlc ();}
  {extern void proto_reg_handoff_who (void); proto_reg_handoff_who ();}
  {extern void proto_reg_handoff_winsrepl (void); proto_reg_handoff_winsrepl ();}
  {extern void proto_reg_handoff_wlancap (void); proto_reg_handoff_wlancap ();}
  {extern void proto_reg_handoff_wlancertextn (void); proto_reg_handoff_wlancertextn ();}
  {extern void proto_reg_handoff_wlccp (void); proto_reg_handoff_wlccp ();}
  {extern void proto_reg_handoff_wsp (void); proto_reg_handoff_wsp ();}
  {extern void proto_reg_handoff_wtls (void); proto_reg_handoff_wtls ();}
  {extern void proto_reg_handoff_wtp (void); proto_reg_handoff_wtp ();}
  {extern void proto_reg_handoff_x11 (void); proto_reg_handoff_x11 ();}
  {extern void proto_reg_handoff_x25 (void); proto_reg_handoff_x25 ();}
  {extern void proto_reg_handoff_x29 (void); proto_reg_handoff_x29 ();}
  {extern void proto_reg_handoff_x411 (void); proto_reg_handoff_x411 ();}
  {extern void proto_reg_handoff_x420 (void); proto_reg_handoff_x420 ();}
  {extern void proto_reg_handoff_x509af (void); proto_reg_handoff_x509af ();}
  {extern void proto_reg_handoff_x509ce (void); proto_reg_handoff_x509ce ();}
  {extern void proto_reg_handoff_x509if (void); proto_reg_handoff_x509if ();}
  {extern void proto_reg_handoff_x509sat (void); proto_reg_handoff_x509sat ();}
  {extern void proto_reg_handoff_xdmcp (void); proto_reg_handoff_xdmcp ();}
  {extern void proto_reg_handoff_xml (void); proto_reg_handoff_xml ();}
  {extern void proto_reg_handoff_xot (void); proto_reg_handoff_xot ();}
  {extern void proto_reg_handoff_xyplex (void); proto_reg_handoff_xyplex ();}
  {extern void proto_reg_handoff_yhoo (void); proto_reg_handoff_yhoo ();}
  {extern void proto_reg_handoff_ymsg (void); proto_reg_handoff_ymsg ();}
  {extern void proto_reg_handoff_ypbind (void); proto_reg_handoff_ypbind ();}
  {extern void proto_reg_handoff_yppasswd (void); proto_reg_handoff_yppasswd ();}
  {extern void proto_reg_handoff_ypserv (void); proto_reg_handoff_ypserv ();}
  {extern void proto_reg_handoff_ypxfr (void); proto_reg_handoff_ypxfr ();}
  {extern void proto_reg_handoff_zebra (void); proto_reg_handoff_zebra ();}
}
