/* This file is generated by ./process-x11-fields.pl, do not edit. */

{ &hf_x11_above_sibling, { "above-sibling", "x11.above-sibling", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_acceleration_denominator, { "acceleration-denominator", "x11.acceleration-denominator", FT_INT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_acceleration_numerator, { "acceleration-numerator", "x11.acceleration-numerator", FT_INT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_access_mode, { "access-mode", "x11.access-mode", FT_UINT8, BASE_DEC, VALS(access_mode_vals), 0, "", HFILL }},
{ &hf_x11_address, { "address", "x11.address", FT_BYTES, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_ip_address, { "ip-address", "x11.ip-address", FT_IPv4, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_address_length, { "address-length", "x11.address-length", FT_UINT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_alloc, { "alloc", "x11.alloc", FT_UINT8, BASE_DEC, VALS(alloc_vals), 0, "", HFILL }},
{ &hf_x11_allow_events_mode, { "allow-events-mode", "x11.allow-events-mode", FT_UINT8, BASE_DEC, VALS(allow_events_mode_vals), 0, "", HFILL }},
{ &hf_x11_allow_exposures, { "allow-exposures", "x11.allow-exposures", FT_UINT8, BASE_DEC, VALS(yes_no_default_vals), 0, "", HFILL }},
{ &hf_x11_arcs, { "arcs", "x11.arcs", FT_NONE, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_arc, { "arc", "x11.arc", FT_NONE, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_arc_x, { "x", "x11.arc.x", FT_INT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_arc_y, { "y", "x11.arc.y", FT_INT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_arc_width, { "width", "x11.arc.width", FT_UINT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_arc_height, { "height", "x11.arc.height", FT_UINT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_arc_angle1, { "angle1", "x11.arc.angle1", FT_INT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_arc_angle2, { "angle2", "x11.arc.angle2", FT_INT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_arc_mode, { "arc-mode", "x11.arc-mode", FT_UINT8, BASE_DEC, VALS(arc_mode_vals), 0, "Tell us if we're drawing an arc or a pie", HFILL }},
{ &hf_x11_atom, { "atom", "x11.atom", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_authorization_protocol_name_length, { "authorization-protocol-name-length", "x11.authorization-protocol-name-length", FT_UINT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_authorization_protocol_name, { "authorization-protocol-name", "x11.authorization-protocol-name", FT_STRING, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_authorization_protocol_data_length, { "authorization-protocol-data-length", "x11.authorization-protocol-data-length", FT_UINT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_authorization_protocol_data, { "authorization-protocol-data", "x11.authorization-protocol-data", FT_STRING, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_auto_repeat_mode, { "auto-repeat-mode", "x11.auto-repeat-mode", FT_UINT8, BASE_DEC, VALS(auto_repeat_mode_vals), 0, "", HFILL }},
{ &hf_x11_bitmap_format_bit_order, { "bitmap-format-bit-order", "x11.bitmap-format-bit-order", FT_UINT8, BASE_HEX, VALS(image_byte_order_vals), 0, "", HFILL }},
{ &hf_x11_bitmap_format_scanline_pad, { "bitmap-format-scanline-pad", "x11.bitmap-format-scanline-pad", FT_UINT8, BASE_DEC, NULL, 0, "bitmap format scanline-pad", HFILL }},
{ &hf_x11_bitmap_format_scanline_unit, { "bitmap-format-scanline-unit", "x11.bitmap-format-scanline-unit", FT_UINT8, BASE_DEC, NULL, 0, "bitmap format scanline unit", HFILL }},
{ &hf_x11_bytes_after, { "bytes-after", "x11.bytes-after", FT_UINT32, BASE_DEC, NULL, 0, "bytes after", HFILL }},
{ &hf_x11_back_blue, { "back-blue", "x11.back-blue", FT_UINT16, BASE_DEC, NULL, 0, "Background blue value for a cursor", HFILL }},
{ &hf_x11_back_green, { "back-green", "x11.back-green", FT_UINT16, BASE_DEC, NULL, 0, "Background green value for a cursor", HFILL }},
{ &hf_x11_back_red, { "back-red", "x11.back-red", FT_UINT16, BASE_DEC, NULL, 0, "Background red value for a cursor", HFILL }},
{ &hf_x11_background, { "background", "x11.background", FT_UINT32, BASE_HEX, NULL, 0, "Background color", HFILL }},
{ &hf_x11_background_pixel, { "background-pixel", "x11.background-pixel", FT_UINT32, BASE_HEX, NULL, 0, "Background color for a window", HFILL }},
{ &hf_x11_background_pixmap, { "background-pixmap", "x11.background-pixmap", FT_UINT32, BASE_HEX, VALS(background_pixmap_vals), 0, "Background pixmap for a window", HFILL }},
{ &hf_x11_backing_pixel, { "backing-pixel", "x11.backing-pixel", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_backing_planes, { "backing-planes", "x11.backing-planes", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_backing_store, { "backing-store", "x11.backing-store", FT_UINT8, BASE_DEC, VALS(backing_store_vals), 0, "", HFILL }},
{ &hf_x11_bell_duration, { "bell-duration", "x11.bell-duration", FT_INT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_bell_percent, { "bell-percent", "x11.bell-percent", FT_INT8, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_bell_pitch, { "bell-pitch", "x11.bell-pitch", FT_INT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_bit_gravity, { "bit-gravity", "x11.bit-gravity", FT_UINT8, BASE_DEC, VALS(gravity_vals), 0, "", HFILL }},
{ &hf_x11_bit_plane, { "bit-plane", "x11.bit-plane", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_blue, { "blue", "x11.blue", FT_UINT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_blues, { "blues", "x11.blues", FT_UINT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_border_pixel, { "border-pixel", "x11.border-pixel", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_border_pixmap, { "border-pixmap", "x11.border-pixmap", FT_UINT32, BASE_HEX, VALS(border_pixmap_vals), 0, "", HFILL }},
{ &hf_x11_border_width, { "border-width", "x11.border-width", FT_UINT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_button, { "button", "x11.button", FT_UINT8, BASE_DEC, VALS(button_vals), 0, "", HFILL }},
{ &hf_x11_byte_order, { "byte-order", "x11.byte-order", FT_UINT8, BASE_HEX, VALS(byte_order_vals), 0, "", HFILL }},
{ &hf_x11_childwindow, { "childwindow", "x11.childwindow", FT_UINT32, BASE_HEX, NULL, 0, "childwindow", HFILL }},
{ &hf_x11_cap_style, { "cap-style", "x11.cap-style", FT_UINT8, BASE_DEC, VALS(cap_style_vals), 0, "", HFILL }},
{ &hf_x11_change_host_mode, { "change-host-mode", "x11.change-host-mode", FT_UINT8, BASE_DEC, VALS(insert_delete_vals), 0, "", HFILL }},
{ &hf_x11_cid, { "cid", "x11.cid", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_class, { "class", "x11.class", FT_UINT8, BASE_DEC, VALS(class_vals), 0, "", HFILL }},
{ &hf_x11_clip_mask, { "clip-mask", "x11.clip-mask", FT_UINT32, BASE_HEX, VALS(zero_is_none_vals), 0, "", HFILL }},
{ &hf_x11_clip_x_origin, { "clip-x-origin", "x11.clip-x-origin", FT_INT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_clip_y_origin, { "clip-y-origin", "x11.clip-y-origin", FT_INT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_close_down_mode, { "close-down-mode", "x11.close-down-mode", FT_UINT8, BASE_DEC, VALS(close_down_mode_vals), 0, "", HFILL }},
{ &hf_x11_cmap, { "cmap", "x11.cmap", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_colormap, { "colormap", "x11.colormap", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_colormap_state, { "colormap-state", "x11.colormap-state", FT_UINT8, BASE_DEC, VALS(colormap_state_vals), 0, "", HFILL }},
{ &hf_x11_color_items, { "color-items", "x11.color-items", FT_NONE, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_coloritem, { "coloritem", "x11.coloritem", FT_NONE, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_coloritem_pixel, { "pixel", "x11.coloritem.pixel", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_coloritem_red, { "red", "x11.coloritem.red", FT_UINT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_coloritem_green, { "green", "x11.coloritem.green", FT_UINT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_coloritem_blue, { "blue", "x11.coloritem.blue", FT_UINT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_coloritem_flags, { "flags", "x11.coloritem.flags", FT_UINT8, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_coloritem_flags_do_red, { "do-red", "x11.coloritem.flags.do-red", FT_BOOLEAN, 8, NULL, 0x01, "", HFILL }},
{ &hf_x11_coloritem_flags_do_green, { "do-green", "x11.coloritem.flags.do-green", FT_BOOLEAN, 8, NULL, 0x02, "", HFILL }},
{ &hf_x11_coloritem_flags_do_blue, { "do-blue", "x11.coloritem.flags.do-blue", FT_BOOLEAN, 8, NULL, 0x04, "", HFILL }},
{ &hf_x11_coloritem_flags_unused, { "unused", "x11.coloritem.flags.unused", FT_BOOLEAN, 8, NULL, 0xf8, "", HFILL }},
{ &hf_x11_coloritem_unused, { "unused", "x11.coloritem.unused", FT_NONE, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_colors, { "colors", "x11.colors", FT_UINT16, BASE_DEC, NULL, 0, "The number of color cells to allocate", HFILL }},
{ &hf_x11_configure_window_mask, { "configure-window-mask", "x11.configure-window-mask", FT_UINT16, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_configure_window_mask_x, { "x", "x11.configure-window-mask.x", FT_BOOLEAN, 16, NULL, 0x0001, "", HFILL }},
{ &hf_x11_configure_window_mask_y, { "y", "x11.configure-window-mask.y", FT_BOOLEAN, 16, NULL, 0x0002, "", HFILL }},
{ &hf_x11_configure_window_mask_width, { "width", "x11.configure-window-mask.width", FT_BOOLEAN, 16, NULL, 0x0004, "", HFILL }},
{ &hf_x11_configure_window_mask_height, { "height", "x11.configure-window-mask.height", FT_BOOLEAN, 16, NULL, 0x0008, "", HFILL }},
{ &hf_x11_configure_window_mask_border_width, { "border-width", "x11.configure-window-mask.border-width", FT_BOOLEAN, 16, NULL, 0x0010, "", HFILL }},
{ &hf_x11_configure_window_mask_sibling, { "sibling", "x11.configure-window-mask.sibling", FT_BOOLEAN, 16, NULL, 0x0020, "", HFILL }},
{ &hf_x11_configure_window_mask_stack_mode, { "stack-mode", "x11.configure-window-mask.stack-mode", FT_BOOLEAN, 16, NULL, 0x0040, "", HFILL }},
{ &hf_x11_confine_to, { "confine-to", "x11.confine-to", FT_UINT32, BASE_HEX, VALS(zero_is_none_vals), 0, "", HFILL }},
{ &hf_x11_contiguous, { "contiguous", "x11.contiguous", FT_BOOLEAN, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_coordinate_mode, { "coordinate-mode", "x11.coordinate-mode", FT_UINT8, BASE_DEC, VALS(coordinate_mode_vals), 0, "", HFILL }},
{ &hf_x11_count, { "count", "x11.count", FT_UINT8, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_cursor, { "cursor", "x11.cursor", FT_UINT32, BASE_HEX, VALS(zero_is_none_vals), 0, "", HFILL }},
{ &hf_x11_dash_offset, { "dash-offset", "x11.dash-offset", FT_UINT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_dashes, { "dashes", "x11.dashes", FT_BYTES, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_dashes_length, { "dashes-length", "x11.dashes-length", FT_UINT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_do_acceleration, { "do-acceleration", "x11.do-acceleration", FT_BOOLEAN, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_do_threshold, { "do-threshold", "x11.do-threshold", FT_BOOLEAN, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_detail, { "detail", "x11.detail", FT_UINT8, BASE_DEC, NULL, 0, "detail", HFILL }},
{ &hf_x11_do_not_propagate_mask, { "do-not-propagate-mask", "x11.do-not-propagate-mask", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_do_not_propagate_mask_KeyPress, { "KeyPress", "x11.do-not-propagate-mask.KeyPress", FT_BOOLEAN, 32, NULL, 0x00000001, "", HFILL }},
{ &hf_x11_do_not_propagate_mask_KeyRelease, { "KeyRelease", "x11.do-not-propagate-mask.KeyRelease", FT_BOOLEAN, 32, NULL, 0x00000002, "", HFILL }},
{ &hf_x11_do_not_propagate_mask_ButtonPress, { "ButtonPress", "x11.do-not-propagate-mask.ButtonPress", FT_BOOLEAN, 32, NULL, 0x00000004, "", HFILL }},
{ &hf_x11_do_not_propagate_mask_ButtonRelease, { "ButtonRelease", "x11.do-not-propagate-mask.ButtonRelease", FT_BOOLEAN, 32, NULL, 0x00000008, "", HFILL }},
{ &hf_x11_do_not_propagate_mask_PointerMotion, { "PointerMotion", "x11.do-not-propagate-mask.PointerMotion", FT_BOOLEAN, 32, NULL, 0x00000040, "", HFILL }},
{ &hf_x11_do_not_propagate_mask_Button1Motion, { "Button1Motion", "x11.do-not-propagate-mask.Button1Motion", FT_BOOLEAN, 32, NULL, 0x00000100, "", HFILL }},
{ &hf_x11_do_not_propagate_mask_Button2Motion, { "Button2Motion", "x11.do-not-propagate-mask.Button2Motion", FT_BOOLEAN, 32, NULL, 0x00000200, "", HFILL }},
{ &hf_x11_do_not_propagate_mask_Button3Motion, { "Button3Motion", "x11.do-not-propagate-mask.Button3Motion", FT_BOOLEAN, 32, NULL, 0x00000400, "", HFILL }},
{ &hf_x11_do_not_propagate_mask_Button4Motion, { "Button4Motion", "x11.do-not-propagate-mask.Button4Motion", FT_BOOLEAN, 32, NULL, 0x00000800, "", HFILL }},
{ &hf_x11_do_not_propagate_mask_Button5Motion, { "Button5Motion", "x11.do-not-propagate-mask.Button5Motion", FT_BOOLEAN, 32, NULL, 0x00001000, "", HFILL }},
{ &hf_x11_do_not_propagate_mask_ButtonMotion, { "ButtonMotion", "x11.do-not-propagate-mask.ButtonMotion", FT_BOOLEAN, 32, NULL, 0x00002000, "", HFILL }},
{ &hf_x11_do_not_propagate_mask_erroneous_bits, { "erroneous-bits", "x11.do-not-propagate-mask.erroneous-bits", FT_BOOLEAN, 32, NULL, 0xffffc0b0, "", HFILL }},
{ &hf_x11_event_sequencenumber, { "event-sequencenumber", "x11.event-sequencenumber", FT_UINT16, BASE_DEC, NULL, 0, "event sequencenumber", HFILL }},
{ &hf_x11_error, { "error", "x11.error", FT_UINT8, BASE_DEC, NULL, 0, "error", HFILL }},
{ &hf_x11_error_badvalue, { "error-badvalue", "x11.error-badvalue", FT_UINT32, BASE_DEC, NULL, 0, "error badvalue", HFILL }},
{ &hf_x11_error_sequencenumber, { "error_sequencenumber", "x11.error_sequencenumber", FT_UINT16, BASE_DEC, NULL, 0, "error sequencenumber", HFILL }},
{ &hf_x11_errorcode, { "errorcode", "x11.errorcode", FT_UINT8, BASE_DEC, VALS(errorcode_vals), 0, "errrorcode", HFILL }},
{ &hf_x11_event_x, { "event-x", "x11.event-x", FT_UINT16, BASE_DEC, NULL, 0, "event x", HFILL }},
{ &hf_x11_event_y, { "event-y", "x11.event-y", FT_UINT16, BASE_DEC, NULL, 0, "event y", HFILL }},
{ &hf_x11_eventbutton, { "eventbutton", "x11.eventbutton", FT_UINT8, BASE_DEC, NULL, 0, "eventbutton", HFILL }},
{ &hf_x11_eventcode, { "eventcode", "x11.eventcode", FT_UINT8, BASE_DEC, VALS(eventcode_vals), 0, "eventcode", HFILL }},
{ &hf_x11_eventwindow, { "eventwindow", "x11.eventwindow", FT_UINT32, BASE_HEX, NULL, 0, "eventwindow", HFILL }},
{ &hf_x11_first_event, { "first-event", "x11.first-event", FT_UINT8, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_first_error, { "first-error", "x11.first-error", FT_UINT8, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_gc_dashes, { "gc-dashes", "x11.gc-dashes", FT_UINT8, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_gc_value_mask, { "gc-value-mask", "x11.gc-value-mask", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_gc_value_mask_function, { "function", "x11.gc-value-mask.function", FT_BOOLEAN, 32, NULL, 0x00000001, "", HFILL }},
{ &hf_x11_gc_value_mask_plane_mask, { "plane-mask", "x11.gc-value-mask.plane-mask", FT_BOOLEAN, 32, NULL, 0x00000002, "", HFILL }},
{ &hf_x11_gc_value_mask_foreground, { "foreground", "x11.gc-value-mask.foreground", FT_BOOLEAN, 32, NULL, 0x00000004, "", HFILL }},
{ &hf_x11_gc_value_mask_background, { "background", "x11.gc-value-mask.background", FT_BOOLEAN, 32, NULL, 0x00000008, "", HFILL }},
{ &hf_x11_gc_value_mask_line_width, { "line-width", "x11.gc-value-mask.line-width", FT_BOOLEAN, 32, NULL, 0x00000010, "", HFILL }},
{ &hf_x11_gc_value_mask_line_style, { "line-style", "x11.gc-value-mask.line-style", FT_BOOLEAN, 32, NULL, 0x00000020, "", HFILL }},
{ &hf_x11_gc_value_mask_cap_style, { "cap-style", "x11.gc-value-mask.cap-style", FT_BOOLEAN, 32, NULL, 0x00000040, "", HFILL }},
{ &hf_x11_gc_value_mask_join_style, { "join-style", "x11.gc-value-mask.join-style", FT_BOOLEAN, 32, NULL, 0x00000080, "", HFILL }},
{ &hf_x11_gc_value_mask_fill_style, { "fill-style", "x11.gc-value-mask.fill-style", FT_BOOLEAN, 32, NULL, 0x00000100, "", HFILL }},
{ &hf_x11_gc_value_mask_fill_rule, { "fill-rule", "x11.gc-value-mask.fill-rule", FT_BOOLEAN, 32, NULL, 0x00000200, "", HFILL }},
{ &hf_x11_gc_value_mask_tile, { "tile", "x11.gc-value-mask.tile", FT_BOOLEAN, 32, NULL, 0x00000400, "", HFILL }},
{ &hf_x11_gc_value_mask_stipple, { "stipple", "x11.gc-value-mask.stipple", FT_BOOLEAN, 32, NULL, 0x00000800, "", HFILL }},
{ &hf_x11_gc_value_mask_tile_stipple_x_origin, { "tile-stipple-x-origin", "x11.gc-value-mask.tile-stipple-x-origin", FT_BOOLEAN, 32, NULL, 0x00001000, "", HFILL }},
{ &hf_x11_gc_value_mask_tile_stipple_y_origin, { "tile-stipple-y-origin", "x11.gc-value-mask.tile-stipple-y-origin", FT_BOOLEAN, 32, NULL, 0x00002000, "", HFILL }},
{ &hf_x11_gc_value_mask_font, { "font", "x11.gc-value-mask.font", FT_BOOLEAN, 32, NULL, 0x00004000, "", HFILL }},
{ &hf_x11_gc_value_mask_subwindow_mode, { "subwindow-mode", "x11.gc-value-mask.subwindow-mode", FT_BOOLEAN, 32, NULL, 0x00008000, "", HFILL }},
{ &hf_x11_gc_value_mask_graphics_exposures, { "graphics-exposures", "x11.gc-value-mask.graphics-exposures", FT_BOOLEAN, 32, NULL, 0x00010000, "", HFILL }},
{ &hf_x11_gc_value_mask_clip_x_origin, { "clip-x-origin", "x11.gc-value-mask.clip-x-origin", FT_BOOLEAN, 32, NULL, 0x00020000, "", HFILL }},
{ &hf_x11_gc_value_mask_clip_y_origin, { "clip-y-origin", "x11.gc-value-mask.clip-y-origin", FT_BOOLEAN, 32, NULL, 0x00040000, "", HFILL }},
{ &hf_x11_gc_value_mask_clip_mask, { "clip-mask", "x11.gc-value-mask.clip-mask", FT_BOOLEAN, 32, NULL, 0x00080000, "", HFILL }},
{ &hf_x11_gc_value_mask_dash_offset, { "dash-offset", "x11.gc-value-mask.dash-offset", FT_BOOLEAN, 32, NULL, 0x00100000, "", HFILL }},
{ &hf_x11_gc_value_mask_gc_dashes, { "gc-dashes", "x11.gc-value-mask.gc-dashes", FT_BOOLEAN, 32, NULL, 0x00200000, "", HFILL }},
{ &hf_x11_gc_value_mask_arc_mode, { "arc-mode", "x11.gc-value-mask.arc-mode", FT_BOOLEAN, 32, NULL, 0x00400000, "", HFILL }},
{ &hf_x11_green, { "green", "x11.green", FT_UINT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_greens, { "greens", "x11.greens", FT_UINT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_data, { "data", "x11.data", FT_BYTES, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_data_length, { "data-length", "x11.data-length", FT_UINT32, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_delete, { "delete", "x11.delete", FT_BOOLEAN, BASE_NONE, NULL, 0, "Delete this property after reading", HFILL }},
{ &hf_x11_delta, { "delta", "x11.delta", FT_INT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_depth, { "depth", "x11.depth", FT_UINT8, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_destination, { "destination", "x11.destination", FT_UINT8, BASE_DEC, VALS(destination_vals), 0, "", HFILL }},
{ &hf_x11_direction, { "direction", "x11.direction", FT_UINT8, BASE_DEC, VALS(direction_vals), 0, "", HFILL }},
{ &hf_x11_drawable, { "drawable", "x11.drawable", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_dst_drawable, { "dst-drawable", "x11.dst-drawable", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_dst_gc, { "dst-gc", "x11.dst-gc", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_dst_window, { "dst-window", "x11.dst-window", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_dst_x, { "dst-x", "x11.dst-x", FT_INT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_dst_y, { "dst-y", "x11.dst-y", FT_INT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_event_detail, { "event-detail", "x11.event-detail", FT_UINT8, BASE_DEC, VALS(event_detail_vals), 0, "", HFILL }},
{ &hf_x11_event_mask, { "event-mask", "x11.event-mask", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_event_mask_KeyPress, { "KeyPress", "x11.event-mask.KeyPress", FT_BOOLEAN, 32, NULL, 0x00000001, "", HFILL }},
{ &hf_x11_event_mask_KeyRelease, { "KeyRelease", "x11.event-mask.KeyRelease", FT_BOOLEAN, 32, NULL, 0x00000002, "", HFILL }},
{ &hf_x11_event_mask_ButtonPress, { "ButtonPress", "x11.event-mask.ButtonPress", FT_BOOLEAN, 32, NULL, 0x00000004, "", HFILL }},
{ &hf_x11_event_mask_ButtonRelease, { "ButtonRelease", "x11.event-mask.ButtonRelease", FT_BOOLEAN, 32, NULL, 0x00000008, "", HFILL }},
{ &hf_x11_event_mask_EnterWindow, { "EnterWindow", "x11.event-mask.EnterWindow", FT_BOOLEAN, 32, NULL, 0x00000010, "", HFILL }},
{ &hf_x11_event_mask_LeaveWindow, { "LeaveWindow", "x11.event-mask.LeaveWindow", FT_BOOLEAN, 32, NULL, 0x00000020, "", HFILL }},
{ &hf_x11_event_mask_PointerMotion, { "PointerMotion", "x11.event-mask.PointerMotion", FT_BOOLEAN, 32, NULL, 0x00000040, "", HFILL }},
{ &hf_x11_event_mask_PointerMotionHint, { "PointerMotionHint", "x11.event-mask.PointerMotionHint", FT_BOOLEAN, 32, NULL, 0x00000080, "", HFILL }},
{ &hf_x11_event_mask_Button1Motion, { "Button1Motion", "x11.event-mask.Button1Motion", FT_BOOLEAN, 32, NULL, 0x00000100, "", HFILL }},
{ &hf_x11_event_mask_Button2Motion, { "Button2Motion", "x11.event-mask.Button2Motion", FT_BOOLEAN, 32, NULL, 0x00000200, "", HFILL }},
{ &hf_x11_event_mask_Button3Motion, { "Button3Motion", "x11.event-mask.Button3Motion", FT_BOOLEAN, 32, NULL, 0x00000400, "", HFILL }},
{ &hf_x11_event_mask_Button4Motion, { "Button4Motion", "x11.event-mask.Button4Motion", FT_BOOLEAN, 32, NULL, 0x00000800, "", HFILL }},
{ &hf_x11_event_mask_Button5Motion, { "Button5Motion", "x11.event-mask.Button5Motion", FT_BOOLEAN, 32, NULL, 0x00001000, "", HFILL }},
{ &hf_x11_event_mask_ButtonMotion, { "ButtonMotion", "x11.event-mask.ButtonMotion", FT_BOOLEAN, 32, NULL, 0x00002000, "", HFILL }},
{ &hf_x11_event_mask_KeymapState, { "KeymapState", "x11.event-mask.KeymapState", FT_BOOLEAN, 32, NULL, 0x00004000, "", HFILL }},
{ &hf_x11_event_mask_Exposure, { "Exposure", "x11.event-mask.Exposure", FT_BOOLEAN, 32, NULL, 0x00008000, "", HFILL }},
{ &hf_x11_event_mask_VisibilityChange, { "VisibilityChange", "x11.event-mask.VisibilityChange", FT_BOOLEAN, 32, NULL, 0x00010000, "", HFILL }},
{ &hf_x11_event_mask_StructureNotify, { "StructureNotify", "x11.event-mask.StructureNotify", FT_BOOLEAN, 32, NULL, 0x00020000, "", HFILL }},
{ &hf_x11_event_mask_ResizeRedirect, { "ResizeRedirect", "x11.event-mask.ResizeRedirect", FT_BOOLEAN, 32, NULL, 0x00040000, "", HFILL }},
{ &hf_x11_event_mask_SubstructureNotify, { "SubstructureNotify", "x11.event-mask.SubstructureNotify", FT_BOOLEAN, 32, NULL, 0x00080000, "", HFILL }},
{ &hf_x11_event_mask_SubstructureRedirect, { "SubstructureRedirect", "x11.event-mask.SubstructureRedirect", FT_BOOLEAN, 32, NULL, 0x00100000, "", HFILL }},
{ &hf_x11_event_mask_FocusChange, { "FocusChange", "x11.event-mask.FocusChange", FT_BOOLEAN, 32, NULL, 0x00200000, "", HFILL }},
{ &hf_x11_event_mask_PropertyChange, { "PropertyChange", "x11.event-mask.PropertyChange", FT_BOOLEAN, 32, NULL, 0x00400000, "", HFILL }},
{ &hf_x11_event_mask_ColormapChange, { "ColormapChange", "x11.event-mask.ColormapChange", FT_BOOLEAN, 32, NULL, 0x00800000, "", HFILL }},
{ &hf_x11_event_mask_OwnerGrabButton, { "OwnerGrabButton", "x11.event-mask.OwnerGrabButton", FT_BOOLEAN, 32, NULL, 0x01000000, "", HFILL }},
{ &hf_x11_event_mask_erroneous_bits, { "erroneous-bits", "x11.event-mask.erroneous-bits", FT_BOOLEAN, 32, NULL, 0xfe000000, "", HFILL }},
{ &hf_x11_exact_blue, { "exact-blue", "x11.exact-blue", FT_UINT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_exact_green, { "exact-green", "x11.exact-green", FT_UINT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_exact_red, { "exact-red", "x11.exact-red", FT_UINT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_exposures, { "exposures", "x11.exposures", FT_BOOLEAN, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_family, { "family", "x11.family", FT_UINT8, BASE_DEC, VALS(family_vals), 0, "", HFILL }},
{ &hf_x11_fid, { "fid", "x11.fid", FT_UINT32, BASE_HEX, NULL, 0, "Font id", HFILL }},
{ &hf_x11_fill_rule, { "fill-rule", "x11.fill-rule", FT_UINT8, BASE_DEC, VALS(fill_rule_vals), 0, "", HFILL }},
{ &hf_x11_fill_style, { "fill-style", "x11.fill-style", FT_UINT8, BASE_DEC, VALS(fill_style_vals), 0, "", HFILL }},
{ &hf_x11_first_keycode, { "first-keycode", "x11.first-keycode", FT_UINT8, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_focus, { "focus", "x11.focus", FT_UINT8, BASE_DEC, VALS(focus_vals), 0, "", HFILL }},
{ &hf_x11_focus_detail, { "focus-detail", "x11.focus-detail", FT_UINT8, BASE_DEC, VALS(focus_detail_vals), 0, "", HFILL }},
{ &hf_x11_focus_mode, { "focus-mode", "x11.focus-mode", FT_UINT8, BASE_DEC, VALS(focus_mode_vals), 0, "", HFILL }},
{ &hf_x11_font, { "font", "x11.font", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_fore_blue, { "fore-blue", "x11.fore-blue", FT_UINT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_fore_green, { "fore-green", "x11.fore-green", FT_UINT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_fore_red, { "fore-red", "x11.fore-red", FT_UINT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_foreground, { "foreground", "x11.foreground", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_format, { "format", "x11.format", FT_UINT8, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_from_configure, { "from-configure", "x11.from-configure", FT_BOOLEAN, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_function, { "function", "x11.function", FT_UINT8, BASE_DEC, VALS(function_vals), 0, "", HFILL }},
{ &hf_x11_gc, { "gc", "x11.gc", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_get_property_type, { "get-property-type", "x11.get-property-type", FT_UINT32, BASE_HEX, VALS(zero_is_any_property_type_vals), 0, "", HFILL }},
{ &hf_x11_grab_mode, { "grab-mode", "x11.grab-mode", FT_UINT8, BASE_DEC, VALS(grab_mode_vals), 0, "", HFILL }},
{ &hf_x11_grab_status, { "grab-status", "x11.grab-status", FT_UINT8, BASE_DEC, VALS(grab_status_vals), 0, "", HFILL }},
{ &hf_x11_grab_window, { "grab-window", "x11.grab-window", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_graphics_exposures, { "graphics-exposures", "x11.graphics-exposures", FT_BOOLEAN, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_height, { "height", "x11.height", FT_UINT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_image_byte_order, { "image-byte-order", "x11.image-byte-order", FT_UINT8, BASE_HEX, VALS(image_byte_order_vals), 0, "", HFILL }},
{ &hf_x11_initial_connection, { "initial-connection", "x11.initial-connection", FT_NONE, BASE_NONE, NULL, 0, "undecoded", HFILL }},
{ &hf_x11_image_format, { "image-format", "x11.image-format", FT_UINT8, BASE_DEC, VALS(image_format_vals), 0, "", HFILL }},
{ &hf_x11_image_pixmap_format, { "image-pixmap-format", "x11.image-pixmap-format", FT_UINT8, BASE_DEC, VALS(image_pixmap_format_vals), 0, "", HFILL }},
{ &hf_x11_interval, { "interval", "x11.interval", FT_INT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_items, { "items", "x11.items", FT_NONE, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_join_style, { "join-style", "x11.join-style", FT_UINT8, BASE_DEC, VALS(join_style_vals), 0, "", HFILL }},
{ &hf_x11_key, { "key", "x11.key", FT_UINT8, BASE_DEC, VALS(key_vals), 0, "", HFILL }},
{ &hf_x11_key_click_percent, { "key-click-percent", "x11.key-click-percent", FT_INT8, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_keyboard_key, { "keyboard-key", "x11.keyboard-key", FT_UINT8, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_keyboard_mode, { "keyboard-mode", "x11.keyboard-mode", FT_UINT8, BASE_DEC, VALS(pointer_keyboard_mode_vals), 0, "", HFILL }},
{ &hf_x11_keybut_mask_erroneous_bits, { "keybut-mask-erroneous-bits", "x11.keybut-mask-erroneous-bits", FT_BOOLEAN, 16, NULL, 0xe000, "keybut mask erroneous bits", HFILL }},
{ &hf_x11_keycode, { "keycode", "x11.keycode", FT_UINT8, BASE_HEX, NULL, 0, "keycode", HFILL }},
{ &hf_x11_keyboard_value_mask, { "keyboard-value-mask", "x11.keyboard-value-mask", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_keyboard_value_mask_key_click_percent, { "key-click-percent", "x11.keyboard-value-mask.key-click-percent", FT_BOOLEAN, 32, NULL, 0x0001, "", HFILL }},
{ &hf_x11_keyboard_value_mask_bell_percent, { "bell-percent", "x11.keyboard-value-mask.bell-percent", FT_BOOLEAN, 32, NULL, 0x0002, "", HFILL }},
{ &hf_x11_keyboard_value_mask_bell_pitch, { "bell-pitch", "x11.keyboard-value-mask.bell-pitch", FT_BOOLEAN, 32, NULL, 0x0004, "", HFILL }},
{ &hf_x11_keyboard_value_mask_bell_duration, { "bell-duration", "x11.keyboard-value-mask.bell-duration", FT_BOOLEAN, 32, NULL, 0x0008, "", HFILL }},
{ &hf_x11_keyboard_value_mask_led, { "led", "x11.keyboard-value-mask.led", FT_BOOLEAN, 32, NULL, 0x0010, "", HFILL }},
{ &hf_x11_keyboard_value_mask_led_mode, { "led-mode", "x11.keyboard-value-mask.led-mode", FT_BOOLEAN, 32, NULL, 0x0020, "", HFILL }},
{ &hf_x11_keyboard_value_mask_keyboard_key, { "keyboard-key", "x11.keyboard-value-mask.keyboard-key", FT_BOOLEAN, 32, NULL, 0x0040, "", HFILL }},
{ &hf_x11_keyboard_value_mask_auto_repeat_mode, { "auto-repeat-mode", "x11.keyboard-value-mask.auto-repeat-mode", FT_BOOLEAN, 32, NULL, 0x0080, "", HFILL }},
{ &hf_x11_keycode_count, { "keycode-count", "x11.keycode-count", FT_UINT8, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_keycodes, { "keycodes", "x11.keycodes", FT_NONE, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_keycodes_item, { "item", "x11.keycodes.item", FT_BYTES, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_keycodes_per_modifier, { "keycodes-per-modifier", "x11.keycodes-per-modifier", FT_UINT8, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_keys, { "keys", "x11.keys", FT_BYTES, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_keysyms, { "keysyms", "x11.keysyms", FT_NONE, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_keysyms_item, { "item", "x11.keysyms.item", FT_NONE, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_keysyms_item_keysym, { "keysym", "x11.keysyms.item.keysym", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_keysyms_per_keycode, { "keysyms-per-keycode", "x11.keysyms-per-keycode", FT_UINT8, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_length_of_reason, { "length-of-reason", "x11.length-of-reason", FT_UINT8, BASE_DEC, NULL, 0, "length of reason", HFILL }},
{ &hf_x11_length_of_vendor, { "length-of-vendor", "x11.length-of-vendor", FT_UINT16, BASE_DEC, NULL, 0, "length of vendor", HFILL }},
{ &hf_x11_led, { "led", "x11.led", FT_UINT8, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_led_mode, { "led-mode", "x11.led-mode", FT_UINT8, BASE_DEC, VALS(on_off_vals), 0, "", HFILL }},
{ &hf_x11_left_pad, { "left-pad", "x11.left-pad", FT_UINT8, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_line_style, { "line-style", "x11.line-style", FT_UINT8, BASE_DEC, VALS(line_style_vals), 0, "", HFILL }},
{ &hf_x11_line_width, { "line-width", "x11.line-width", FT_UINT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_long_length, { "long-length", "x11.long-length", FT_UINT32, BASE_DEC, NULL, 0, "The maximum length of the property in bytes", HFILL }},
{ &hf_x11_long_offset, { "long-offset", "x11.long-offset", FT_UINT32, BASE_DEC, NULL, 0, "The starting position in the property bytes array", HFILL }},
{ &hf_x11_map, { "map", "x11.map", FT_BYTES, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_map_length, { "map-length", "x11.map-length", FT_UINT8, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_mask, { "mask", "x11.mask", FT_UINT32, BASE_HEX, VALS(zero_is_none_vals), 0, "", HFILL }},
{ &hf_x11_mask_char, { "mask-char", "x11.mask-char", FT_UINT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_mask_font, { "mask-font", "x11.mask-font", FT_UINT32, BASE_HEX, VALS(zero_is_none_vals), 0, "", HFILL }},
{ &hf_x11_max_names, { "max-names", "x11.max-names", FT_UINT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_mid, { "mid", "x11.mid", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_mode, { "mode", "x11.mode", FT_UINT8, BASE_DEC, VALS(mode_vals), 0, "", HFILL }},
{ &hf_x11_major_opcode, { "major-opcode", "x11.major-opcode", FT_UINT16, BASE_DEC, NULL, 0, "major opcode", HFILL }},
{ &hf_x11_max_keycode, { "max-keycode", "x11.max-keycode", FT_UINT8, BASE_DEC, NULL, 0, "max keycode", HFILL }},
{ &hf_x11_maximum_request_length, { "maximum-request-length", "x11.maximum-request-length", FT_UINT16, BASE_DEC, NULL, 0, "maximum request length", HFILL }},
{ &hf_x11_min_keycode, { "min-keycode", "x11.min-keycode", FT_UINT8, BASE_DEC, NULL, 0, "min keycode", HFILL }},
{ &hf_x11_minor_opcode, { "minor-opcode", "x11.minor-opcode", FT_UINT16, BASE_DEC, NULL, 0, "minor opcode", HFILL }},
{ &hf_x11_modifiers_mask, { "modifiers-mask", "x11.modifiers-mask", FT_UINT16, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_modifiers_mask_Shift, { "Shift", "x11.modifiers-mask.Shift", FT_BOOLEAN, 16, NULL, 0x0001, "", HFILL }},
{ &hf_x11_modifiers_mask_Lock, { "Lock", "x11.modifiers-mask.Lock", FT_BOOLEAN, 16, NULL, 0x0002, "", HFILL }},
{ &hf_x11_modifiers_mask_Control, { "Control", "x11.modifiers-mask.Control", FT_BOOLEAN, 16, NULL, 0x0004, "", HFILL }},
{ &hf_x11_modifiers_mask_Mod1, { "Mod1", "x11.modifiers-mask.Mod1", FT_BOOLEAN, 16, NULL, 0x0008, "", HFILL }},
{ &hf_x11_modifiers_mask_Mod2, { "Mod2", "x11.modifiers-mask.Mod2", FT_BOOLEAN, 16, NULL, 0x0010, "", HFILL }},
{ &hf_x11_modifiers_mask_Mod3, { "Mod3", "x11.modifiers-mask.Mod3", FT_BOOLEAN, 16, NULL, 0x0020, "", HFILL }},
{ &hf_x11_modifiers_mask_Mod4, { "Mod4", "x11.modifiers-mask.Mod4", FT_BOOLEAN, 16, NULL, 0x0040, "", HFILL }},
{ &hf_x11_modifiers_mask_Mod5, { "Mod5", "x11.modifiers-mask.Mod5", FT_BOOLEAN, 16, NULL, 0x0080, "", HFILL }},
{ &hf_x11_modifiers_mask_Button1, { "Button1", "x11.modifiers-mask.Button1", FT_BOOLEAN, 16, NULL, 0x0100, "", HFILL }},
{ &hf_x11_modifiers_mask_Button2, { "Button2", "x11.modifiers-mask.Button2", FT_BOOLEAN, 16, NULL, 0x0200, "", HFILL }},
{ &hf_x11_modifiers_mask_Button3, { "Button3", "x11.modifiers-mask.Button3", FT_BOOLEAN, 16, NULL, 0x0400, "", HFILL }},
{ &hf_x11_modifiers_mask_Button4, { "Button4", "x11.modifiers-mask.Button4", FT_BOOLEAN, 16, NULL, 0x0800, "", HFILL }},
{ &hf_x11_modifiers_mask_Button5, { "Button5", "x11.modifiers-mask.Button5", FT_BOOLEAN, 16, NULL, 0x1000, "", HFILL }},
{ &hf_x11_modifiers_mask_AnyModifier, { "AnyModifier", "x11.modifiers-mask.AnyModifier", FT_UINT16, BASE_HEX, NULL, 0x8000, "", HFILL }},
{ &hf_x11_modifiers_mask_erroneous_bits, { "erroneous-bits", "x11.modifiers-mask.erroneous-bits", FT_BOOLEAN, 16, NULL, 0xff00, "", HFILL }},
{ &hf_x11_motion_buffer_size, { "motion-buffer-size", "x11.motion-buffer-size", FT_UINT16, BASE_DEC, NULL, 0, "motion buffer size", HFILL }},
{ &hf_x11_new, { "new", "x11.new", FT_BOOLEAN, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_number_of_formats_in_pixmap_formats, { "number-of-formats-in-pixmap-formats", "x11.number-of-formats-in-pixmap-formats", FT_UINT8, BASE_DEC, NULL, 0, "number of formats in pixmap formats", HFILL }},
{ &hf_x11_number_of_screens_in_roots, { "number-of-screens-in-roots", "x11.number-of-screens-in-roots", FT_UINT8, BASE_DEC, NULL, 0, "number of screens in roots", HFILL }},
{ &hf_x11_name, { "name", "x11.name", FT_STRING, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_name_length, { "name-length", "x11.name-length", FT_UINT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_odd_length, { "odd-length", "x11.odd-length", FT_BOOLEAN, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_only_if_exists, { "only-if-exists", "x11.only-if-exists", FT_BOOLEAN, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_opcode, { "opcode", "x11.opcode", FT_UINT8, BASE_DEC, VALS(opcode_vals), 0, "", HFILL }},
{ &hf_x11_ordering, { "ordering", "x11.ordering", FT_UINT8, BASE_DEC, VALS(ordering_vals), 0, "", HFILL }},
{ &hf_x11_override_redirect, { "override-redirect", "x11.override-redirect", FT_BOOLEAN, BASE_NONE, NULL, 0, "Window manager doesn't manage this window when true", HFILL }},
{ &hf_x11_owner, { "owner", "x11.owner", FT_UINT32, BASE_HEX, VALS(zero_is_none_vals), 0, "", HFILL }},
{ &hf_x11_owner_events, { "owner-events", "x11.owner-events", FT_BOOLEAN, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_parent, { "parent", "x11.parent", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_path, { "path", "x11.path", FT_NONE, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_path_string, { "string", "x11.path.string", FT_STRING, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_pattern, { "pattern", "x11.pattern", FT_STRING, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_pattern_length, { "pattern-length", "x11.pattern-length", FT_UINT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_percent, { "percent", "x11.percent", FT_UINT8, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_pid, { "pid", "x11.pid", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_pixel, { "pixel", "x11.pixel", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_pixels, { "pixels", "x11.pixels", FT_NONE, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_pixels_item, { "pixels_item", "x11.pixels_item", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_pixmap, { "pixmap", "x11.pixmap", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_place, { "place", "x11.place", FT_UINT8, BASE_DEC, VALS(place_vals), 0, "", HFILL }},
{ &hf_x11_plane_mask, { "plane-mask", "x11.plane-mask", FT_UINT32, BASE_HEX, VALS(plane_mask_vals), 0, "", HFILL }},
{ &hf_x11_planes, { "planes", "x11.planes", FT_UINT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_point, { "point", "x11.point", FT_NONE, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_points, { "points", "x11.points", FT_NONE, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_point_x, { "point-x", "x11.point-x", FT_INT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_point_y, { "point-y", "x11.point-y", FT_INT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_pointer_event_mask, { "pointer-event-mask", "x11.pointer-event-mask", FT_UINT16, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_pointer_event_mask_ButtonPress, { "ButtonPress", "x11.pointer-event-mask.ButtonPress", FT_BOOLEAN, 16, NULL, 0x0004, "", HFILL }},
{ &hf_x11_pointer_event_mask_ButtonRelease, { "ButtonRelease", "x11.pointer-event-mask.ButtonRelease", FT_BOOLEAN, 16, NULL, 0x0008, "", HFILL }},
{ &hf_x11_pointer_event_mask_EnterWindow, { "EnterWindow", "x11.pointer-event-mask.EnterWindow", FT_BOOLEAN, 16, NULL, 0x0010, "", HFILL }},
{ &hf_x11_pointer_event_mask_LeaveWindow, { "LeaveWindow", "x11.pointer-event-mask.LeaveWindow", FT_BOOLEAN, 16, NULL, 0x0020, "", HFILL }},
{ &hf_x11_pointer_event_mask_PointerMotion, { "PointerMotion", "x11.pointer-event-mask.PointerMotion", FT_BOOLEAN, 16, NULL, 0x0040, "", HFILL }},
{ &hf_x11_pointer_event_mask_PointerMotionHint, { "PointerMotionHint", "x11.pointer-event-mask.PointerMotionHint", FT_BOOLEAN, 16, NULL, 0x0080, "", HFILL }},
{ &hf_x11_pointer_event_mask_Button1Motion, { "Button1Motion", "x11.pointer-event-mask.Button1Motion", FT_BOOLEAN, 16, NULL, 0x0100, "", HFILL }},
{ &hf_x11_pointer_event_mask_Button2Motion, { "Button2Motion", "x11.pointer-event-mask.Button2Motion", FT_BOOLEAN, 16, NULL, 0x0200, "", HFILL }},
{ &hf_x11_pointer_event_mask_Button3Motion, { "Button3Motion", "x11.pointer-event-mask.Button3Motion", FT_BOOLEAN, 16, NULL, 0x0400, "", HFILL }},
{ &hf_x11_pointer_event_mask_Button4Motion, { "Button4Motion", "x11.pointer-event-mask.Button4Motion", FT_BOOLEAN, 16, NULL, 0x0800, "", HFILL }},
{ &hf_x11_pointer_event_mask_Button5Motion, { "Button5Motion", "x11.pointer-event-mask.Button5Motion", FT_BOOLEAN, 16, NULL, 0x1000, "", HFILL }},
{ &hf_x11_pointer_event_mask_ButtonMotion, { "ButtonMotion", "x11.pointer-event-mask.ButtonMotion", FT_BOOLEAN, 16, NULL, 0x2000, "", HFILL }},
{ &hf_x11_pointer_event_mask_KeymapState, { "KeymapState", "x11.pointer-event-mask.KeymapState", FT_BOOLEAN, 16, NULL, 0x4000, "", HFILL }},
{ &hf_x11_pointer_event_mask_erroneous_bits, { "erroneous-bits", "x11.pointer-event-mask.erroneous-bits", FT_BOOLEAN, 16, NULL, 0x8003, "", HFILL }},
{ &hf_x11_pointer_mode, { "pointer-mode", "x11.pointer-mode", FT_UINT8, BASE_DEC, VALS(pointer_keyboard_mode_vals), 0, "", HFILL }},
{ &hf_x11_prefer_blanking, { "prefer-blanking", "x11.prefer-blanking", FT_UINT8, BASE_DEC, VALS(yes_no_default_vals), 0, "", HFILL }},
{ &hf_x11_present, { "present", "x11.present", FT_BOOLEAN, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_propagate, { "propagate", "x11.propagate", FT_BOOLEAN, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_properties, { "properties", "x11.properties", FT_NONE, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_properties_item, { "item", "x11.properties.item", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_property, { "property", "x11.property", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_property_number, { "property-number", "x11.property-number", FT_UINT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_property_state, { "property-state", "x11.property-state", FT_UINT8, BASE_DEC, VALS(property_state_vals), 0, "", HFILL }},
{ &hf_x11_protocol_major_version, { "protocol-major-version", "x11.protocol-major-version", FT_UINT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_protocol_minor_version, { "protocol-minor-version", "x11.protocol-minor-version", FT_UINT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_reason, { "reason", "x11.reason", FT_STRING, BASE_HEX, NULL, 0, "reason", HFILL }},
{ &hf_x11_rectangle_height, { "rectangle-height", "x11.rectangle-height", FT_UINT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_rectangles, { "rectangles", "x11.rectangles", FT_NONE, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_rectangle, { "rectangle", "x11.rectangle", FT_NONE, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_rectangle_width, { "rectangle-width", "x11.rectangle-width", FT_UINT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_rectangle_x, { "rectangle-x", "x11.rectangle-x", FT_INT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_rectangle_y, { "rectangle-y", "x11.rectangle-y", FT_INT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_red, { "red", "x11.red", FT_UINT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_reds, { "reds", "x11.reds", FT_UINT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_request, { "request", "x11.request", FT_UINT8, BASE_DEC, VALS(opcode_vals), 0, "", HFILL }},
{ &hf_x11_requestor, { "requestor", "x11.requestor", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_request_length, { "request-length", "x11.request-length", FT_UINT16, BASE_DEC, NULL, 0, "Request length", HFILL }},
{ &hf_x11_resource, { "resource", "x11.resource", FT_UINT32, BASE_HEX, VALS(all_temporary_vals), 0, "", HFILL }},
{ &hf_x11_revert_to, { "revert-to", "x11.revert-to", FT_UINT8, BASE_DEC, VALS(revert_to_vals), 0, "", HFILL }},
{ &hf_x11_release_number, { "release-number", "x11.release-number", FT_UINT32, BASE_DEC, NULL, 0, "release number", HFILL }},
{ &hf_x11_reply, { "reply", "x11.reply", FT_UINT8, BASE_DEC, NULL, 0, "reply", HFILL }},
{ &hf_x11_reply_sequencenumber, { "reply-sequencenumber", "x11.reply-sequencenumber", FT_UINT16, BASE_DEC, VALS(opcode_vals), 0, "", HFILL }},
{ &hf_x11_replylength, { "replylength", "x11.replylength", FT_UINT32, BASE_DEC, NULL, 0, "replylength", HFILL }},
{ &hf_x11_replyopcode, { "replyopcode", "x11.replyopcode", FT_UINT8, BASE_DEC, VALS(opcode_vals), 0, "", HFILL }},
{ &hf_x11_resource_id_base, { "resource-id-base", "x11.resource-id-base", FT_UINT32, BASE_HEX, NULL, 0, "resource id base", HFILL }},
{ &hf_x11_resource_id_mask, { "resource-id-mask", "x11.resource-id-mask", FT_UINT32, BASE_HEX, NULL, 0, "resource id mask", HFILL }},
{ &hf_x11_root_x, { "root-x", "x11.root-x", FT_UINT16, BASE_DEC, NULL, 0, "root x", HFILL }},
{ &hf_x11_root_y, { "root-y", "x11.root-y", FT_UINT16, BASE_DEC, NULL, 0, "root y", HFILL }},
{ &hf_x11_rootwindow, { "rootwindow", "x11.rootwindow", FT_UINT32, BASE_HEX, NULL, 0, "rootwindow", HFILL }},
{ &hf_x11_same_screen, { "same-screen", "x11.same-screen", FT_BOOLEAN, BASE_NONE, NULL, 0, "same screen", HFILL }},
{ &hf_x11_same_screen_focus_mask, { "same-screen-focus-mask", "x11.same-screen-focus-mask", FT_UINT8, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_same_screen_focus_mask_focus, { "focus", "x11.same-screen-focus-mask.focus", FT_BOOLEAN, 8, NULL, 0x01, "", HFILL }},
{ &hf_x11_same_screen_focus_mask_same_screen, { "same-screen", "x11.same-screen-focus-mask.same-screen", FT_BOOLEAN, 8, NULL, 0x02, "", HFILL }},
{ &hf_x11_success, { "success", "x11.success", FT_UINT8, BASE_DEC, NULL, 0, "success", HFILL }},
{ &hf_x11_save_set_mode, { "save-set-mode", "x11.save-set-mode", FT_UINT8, BASE_DEC, VALS(insert_delete_vals), 0, "", HFILL }},
{ &hf_x11_save_under, { "save-under", "x11.save-under", FT_BOOLEAN, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_screen_saver_mode, { "screen-saver-mode", "x11.screen-saver-mode", FT_UINT8, BASE_DEC, VALS(screen_saver_mode_vals), 0, "", HFILL }},
{ &hf_x11_segment, { "segment", "x11.segment", FT_NONE, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_segments, { "segments", "x11.segments", FT_NONE, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_segment_x1, { "segment_x1", "x11.segment_x1", FT_INT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_segment_x2, { "segment_x2", "x11.segment_x2", FT_INT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_segment_y1, { "segment_y1", "x11.segment_y1", FT_INT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_segment_y2, { "segment_y2", "x11.segment_y2", FT_INT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_selection, { "selection", "x11.selection", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_shape, { "shape", "x11.shape", FT_UINT8, BASE_DEC, VALS(shape_vals), 0, "", HFILL }},
{ &hf_x11_sibling, { "sibling", "x11.sibling", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_source_pixmap, { "source-pixmap", "x11.source-pixmap", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_source_font, { "source-font", "x11.source-font", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_source_char, { "source-char", "x11.source-char", FT_UINT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_src_cmap, { "src-cmap", "x11.src-cmap", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_src_drawable, { "src-drawable", "x11.src-drawable", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_src_gc, { "src-gc", "x11.src-gc", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_src_height, { "src-height", "x11.src-height", FT_UINT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_src_width, { "src-width", "x11.src-width", FT_UINT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_src_window, { "src-window", "x11.src-window", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_src_x, { "src-x", "x11.src-x", FT_INT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_src_y, { "src-y", "x11.src-y", FT_INT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_start, { "start", "x11.start", FT_UINT32, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_stack_mode, { "stack-mode", "x11.stack-mode", FT_UINT8, BASE_DEC, VALS(stack_mode_vals), 0, "", HFILL }},
{ &hf_x11_stipple, { "stipple", "x11.stipple", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_stop, { "stop", "x11.stop", FT_UINT32, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_str_number_in_path, { "str-number-in-path", "x11.str-number-in-path", FT_UINT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_string, { "string", "x11.string", FT_STRING, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_string16, { "string16", "x11.string16", FT_STRING, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_string16_bytes, { "bytes", "x11.string16.bytes", FT_BYTES, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_string_length, { "string-length", "x11.string-length", FT_UINT32, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_subwindow_mode, { "subwindow-mode", "x11.subwindow-mode", FT_UINT8, BASE_DEC, VALS(subwindow_mode_vals), 0, "", HFILL }},
{ &hf_x11_target, { "target", "x11.target", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_textitem, { "textitem", "x11.textitem", FT_NONE, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_textitem_font, { "font", "x11.textitem.font", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_textitem_string, { "string", "x11.textitem.string", FT_NONE, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_textitem_string_delta, { "delta", "x11.textitem.string.delta", FT_INT8, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_textitem_string_string8, { "string8", "x11.textitem.string.string8", FT_STRING, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_textitem_string_string16, { "string16", "x11.textitem.string.string16", FT_STRING, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_textitem_string_string16_bytes, { "bytes", "x11.textitem.string.string16.bytes", FT_BYTES, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_threshold, { "threshold", "x11.threshold", FT_INT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_tile, { "tile", "x11.tile", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_tile_stipple_x_origin, { "tile-stipple-x-origin", "x11.tile-stipple-x-origin", FT_INT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_tile_stipple_y_origin, { "tile-stipple-y-origin", "x11.tile-stipple-y-origin", FT_INT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_time, { "time", "x11.time", FT_UINT32, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_timeout, { "timeout", "x11.timeout", FT_INT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_type, { "type", "x11.type", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_undecoded, { "undecoded", "x11.undecoded", FT_NONE, BASE_NONE, NULL, 0, "Yet undecoded by dissector", HFILL }},
{ &hf_x11_unused, { "unused", "x11.unused", FT_NONE, BASE_NONE, NULL, 0, "", HFILL }},
{ &hf_x11_valuelength, { "valuelength", "x11.valuelength", FT_UINT32, BASE_DEC, NULL, 0, "valuelength", HFILL }},
{ &hf_x11_vendor, { "vendor", "x11.vendor", FT_STRING, BASE_HEX, NULL, 0, "vendor", HFILL }},
{ &hf_x11_visibility_state, { "visibility-state", "x11.visibility-state", FT_UINT8, BASE_DEC, VALS(visibility_state_vals), 0, "", HFILL }},
{ &hf_x11_visual, { "visual", "x11.visual", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_visual_blue, { "visual-blue", "x11.visual-blue", FT_UINT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_visual_green, { "visual-green", "x11.visual-green", FT_UINT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_visual_red, { "visual-red", "x11.visual-red", FT_UINT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_visualid, { "visualid", "x11.visualid", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_warp_pointer_dst_window, { "warp-pointer-dst-window", "x11.warp-pointer-dst-window", FT_UINT32, BASE_HEX, VALS(zero_is_none_vals), 0, "", HFILL }},
{ &hf_x11_warp_pointer_src_window, { "warp-pointer-src-window", "x11.warp-pointer-src-window", FT_UINT32, BASE_HEX, VALS(zero_is_none_vals), 0, "", HFILL }},
{ &hf_x11_wid, { "wid", "x11.wid", FT_UINT32, BASE_HEX, NULL, 0, "Window id", HFILL }},
{ &hf_x11_width, { "width", "x11.width", FT_UINT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_win_gravity, { "win-gravity", "x11.win-gravity", FT_UINT8, BASE_DEC, VALS(gravity_vals), 0, "", HFILL }},
{ &hf_x11_win_x, { "win-x", "x11.win-x", FT_INT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_win_y, { "win-y", "x11.win-y", FT_INT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_window, { "window", "x11.window", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_window_class, { "window-class", "x11.window-class", FT_UINT16, BASE_DEC, VALS(window_class_vals), 0, "Window class", HFILL }},
{ &hf_x11_window_value_mask, { "window-value-mask", "x11.window-value-mask", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
{ &hf_x11_window_value_mask_background_pixmap, { "background-pixmap", "x11.window-value-mask.background-pixmap", FT_BOOLEAN, 32, NULL, 0x00000001, "", HFILL }},
{ &hf_x11_window_value_mask_background_pixel, { "background-pixel", "x11.window-value-mask.background-pixel", FT_BOOLEAN, 32, NULL, 0x00000002, "", HFILL }},
{ &hf_x11_window_value_mask_border_pixmap, { "border-pixmap", "x11.window-value-mask.border-pixmap", FT_BOOLEAN, 32, NULL, 0x00000004, "", HFILL }},
{ &hf_x11_window_value_mask_border_pixel, { "border-pixel", "x11.window-value-mask.border-pixel", FT_BOOLEAN, 32, NULL, 0x00000008, "", HFILL }},
{ &hf_x11_window_value_mask_bit_gravity, { "bit-gravity", "x11.window-value-mask.bit-gravity", FT_BOOLEAN, 32, NULL, 0x00000010, "", HFILL }},
{ &hf_x11_window_value_mask_win_gravity, { "win-gravity", "x11.window-value-mask.win-gravity", FT_BOOLEAN, 32, NULL, 0x00000020, "", HFILL }},
{ &hf_x11_window_value_mask_backing_store, { "backing-store", "x11.window-value-mask.backing-store", FT_BOOLEAN, 32, NULL, 0x00000040, "", HFILL }},
{ &hf_x11_window_value_mask_backing_planes, { "backing-planes", "x11.window-value-mask.backing-planes", FT_BOOLEAN, 32, NULL, 0x00000080, "", HFILL }},
{ &hf_x11_window_value_mask_backing_pixel, { "backing-pixel", "x11.window-value-mask.backing-pixel", FT_BOOLEAN, 32, NULL, 0x00000100, "", HFILL }},
{ &hf_x11_window_value_mask_override_redirect, { "override-redirect", "x11.window-value-mask.override-redirect", FT_BOOLEAN, 32, NULL, 0x00000200, "", HFILL }},
{ &hf_x11_window_value_mask_save_under, { "save-under", "x11.window-value-mask.save-under", FT_BOOLEAN, 32, NULL, 0x00000400, "", HFILL }},
{ &hf_x11_window_value_mask_event_mask, { "event-mask", "x11.window-value-mask.event-mask", FT_BOOLEAN, 32, NULL, 0x00000800, "", HFILL }},
{ &hf_x11_window_value_mask_do_not_propagate_mask, { "do-not-propagate-mask", "x11.window-value-mask.do-not-propagate-mask", FT_BOOLEAN, 32, NULL, 0x00001000, "", HFILL }},
{ &hf_x11_window_value_mask_colormap, { "colormap", "x11.window-value-mask.colormap", FT_BOOLEAN, 32, NULL, 0x00002000, "", HFILL }},
{ &hf_x11_window_value_mask_cursor, { "cursor", "x11.window-value-mask.cursor", FT_BOOLEAN, 32, NULL, 0x00004000, "", HFILL }},
{ &hf_x11_x, { "x", "x11.x", FT_INT16, BASE_DEC, NULL, 0, "", HFILL }},
{ &hf_x11_y, { "y", "x11.y", FT_INT16, BASE_DEC, NULL, 0, "", HFILL }},
