#define TOKEN_TAG_START                       1
#define TOKEN_DOCTYPE_KW                      2
#define TOKEN_NAME                            3
#define TOKEN_OPEN_BRACKET                    4
#define TOKEN_CLOSE_BRACKET                   5
#define TOKEN_TAG_STOP                        6
#define TOKEN_ATTLIST_KW                      7
#define TOKEN_ELEMENT_KW                      8
#define TOKEN_ATT_TYPE                        9
#define TOKEN_ATT_DEF                        10
#define TOKEN_ATT_DEF_WITH_VALUE             11
#define TOKEN_QUOTED                         12
#define TOKEN_IMPLIED_KW                     13
#define TOKEN_REQUIRED_KW                    14
#define TOKEN_OPEN_PARENS                    15
#define TOKEN_CLOSE_PARENS                   16
#define TOKEN_PIPE                           17
#define TOKEN_STAR                           18
#define TOKEN_PLUS                           19
#define TOKEN_QUESTION                       20
#define TOKEN_ELEM_DATA                      21
#define TOKEN_COMMA                          22
#define TOKEN_EMPTY_KW                       23
