#define TOKEN_TEST_AND                        1
#define TOKEN_TEST_OR                         2
#define TOKEN_TEST_EQ                         3
#define TOKEN_TEST_NE                         4
#define TOKEN_TEST_LT                         5
#define TOKEN_TEST_LE                         6
#define TOKEN_TEST_GT                         7
#define TOKEN_TEST_GE                         8
#define TOKEN_TEST_CONTAINS                   9
#define TOKEN_TEST_MATCHES                   10
#define TOKEN_TEST_BITWISE_AND               11
#define TOKEN_TEST_NOT                       12
#define TOKEN_FIELD                          13
#define TOKEN_STRING                         14
#define TOKEN_UNPARSED                       15
#define TOKEN_LBRACKET                       16
#define TOKEN_RBRACKET                       17
#define TOKEN_COMMA                          18
#define TOKEN_INTEGER                        19
#define TOKEN_COLON                          20
#define TOKEN_HYPHEN                         21
#define TOKEN_FUNCTION                       22
#define TOKEN_LPAREN                         23
#define TOKEN_RPAREN                         24
