/* Do not modify this file.                                                   */
/* It is created automatically by the ASN.1 to Wireshark dissector compiler   */
/* .\packet-umts_rrc_pdu_def.c                                                */
/* ../../tools/asn2wrs.py -u -e -p umts_rrc_pdu_def -c umts_rrc_pdu_def.cnf -s packet-umts_rrc_pdu_def-template umts_rrc_PDU-definitions.asn */

/* Input file: packet-umts_rrc_pdu_def-template.c */

#line 1 "packet-umts_rrc_pdu_def-template.c"
/* packet-umts_rrc_pdu_def.c
 * Routines for Universal Mobile Telecommunications System (UMTS);
 * Radio Resource Control (RRC) protocol specification 	
 * (3GPP TS 25.331 version 6.7.0 Release 6) chapter 11.2	PDU definitions
 * Copyright 2006, Anders Broman <anders.broman@ericsson.com>
 *
 * $Id: packet-umts_rrc_pdu_def.c 18921 2006-08-16 05:58:39Z kukosa $
 *
 * Ethereal - Network traffic analyzer
 * By Gerald Combs <gerald@ethereal.com>
 * Copyright 1998 Gerald Combs
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Ref: 3GPP TS 25.423 version 6.7.0 Release 6
 */

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <glib.h>
#include <epan/packet.h>
#include <epan/conversation.h>

#include <stdio.h>
#include <string.h>

#include "packet-ber.h"
#include "packet-per.h"
#include "packet-umts_rrc_ies.h"
#include "packet-umts_rrc_pdu_def.h"

#define PNAME  "Universal Mobile Telecommunications System (UMTS) Radio Resource Control (RRC) protocol PDU definitions"
#define PSNAME "UMTS_RRC_PDU_DEF"
#define PFNAME "umts_rrc_pdu_def"

static dissector_handle_t umts_rrc_pdu_def_handle=NULL;

/* Include constants */

/*--- Included file: packet-umts_rrc_pdu_def-val.h ---*/
#line 1 "packet-umts_rrc_pdu_def-val.h"
#define maxSIBperMsg                   16
#define maxURNTIGroup                  8

/*--- End of included file: packet-umts_rrc_pdu_def-val.h ---*/
#line 54 "packet-umts_rrc_pdu_def-template.c"

/* Initialize the protocol and registered fields */
static int proto_umts_rrc_pdu_def = -1;



/*--- Included file: packet-umts_rrc_pdu_def-hf.c ---*/
#line 1 "packet-umts_rrc_pdu_def-hf.c"
static int hf_umts_rrc_pdu_def_r3 = -1;           /* T_r3 */
static int hf_umts_rrc_pdu_def_activeSetUpdate_r3 = -1;  /* ActiveSetUpdate_r3_IEs */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions = -1;  /* T_laterNonCriticalExtensions */
static int hf_umts_rrc_pdu_def_activeSetUpdate_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions = -1;  /* T_v4b0NonCriticalExtensions */
static int hf_umts_rrc_pdu_def_activeSetUpdate_v4b0ext = -1;  /* ActiveSetUpdate_v4b0ext_IEs */
static int hf_umts_rrc_pdu_def_v590NonCriticalExtensions = -1;  /* T_v590NonCriticalExtensions */
static int hf_umts_rrc_pdu_def_activeSetUpdate_v590ext = -1;  /* ActiveSetUpdate_v590ext_IEs */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions = -1;  /* T_v6xyNonCriticalExtensions */
static int hf_umts_rrc_pdu_def_activeSetUpdate_v6xyext = -1;  /* ActiveSetUpdate_v6xyext_IEs */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions = -1;  /* T_nonCriticalExtensions */
static int hf_umts_rrc_pdu_def_later_than_r3 = -1;  /* T_later_than_r3 */
static int hf_umts_rrc_pdu_def_rrc_TransactionIdentifier = -1;  /* RRC_TransactionIdentifier */
static int hf_umts_rrc_pdu_def_criticalExtensions = -1;  /* T_criticalExtensions */
static int hf_umts_rrc_pdu_def_r6 = -1;           /* T_r6 */
static int hf_umts_rrc_pdu_def_activeSetUpdate_r6 = -1;  /* ActiveSetUpdate_r6_IEs */
static int hf_umts_rrc_pdu_def_activeSetUpdate_r6_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions1 = -1;  /* T_nonCriticalExtensions1 */
static int hf_umts_rrc_pdu_def_criticalExtensions1 = -1;  /* T_criticalExtensions1 */
static int hf_umts_rrc_pdu_def_dummy_IntegrityProtectionModeInfo = -1;  /* IntegrityProtectionModeInfo */
static int hf_umts_rrc_pdu_def_dummy2_CipheringModeInfo = -1;  /* CipheringModeInfo */
static int hf_umts_rrc_pdu_def_activationTime = -1;  /* ActivationTime */
static int hf_umts_rrc_pdu_def_newU_RNTI = -1;    /* U_RNTI */
static int hf_umts_rrc_pdu_def_cn_InformationInfo = -1;  /* CN_InformationInfo */
static int hf_umts_rrc_pdu_def_dummy3_DL_CounterSynchronisationInfo = -1;  /* DL_CounterSynchronisationInfo */
static int hf_umts_rrc_pdu_def_maxAllowedUL_TX_Power = -1;  /* MaxAllowedUL_TX_Power */
static int hf_umts_rrc_pdu_def_rl_AdditionInformationList = -1;  /* RL_AdditionInformationList */
static int hf_umts_rrc_pdu_def_rl_RemovalInformationList = -1;  /* RL_RemovalInformationList */
static int hf_umts_rrc_pdu_def_tx_DiversityMode = -1;  /* TX_DiversityMode */
static int hf_umts_rrc_pdu_def_dummy4_SSDT_Information = -1;  /* SSDT_Information */
static int hf_umts_rrc_pdu_def_dummy_SSDT_UL = -1;  /* SSDT_UL */
static int hf_umts_rrc_pdu_def_cell_id_PerRL_List = -1;  /* CellIdentity_PerRL_List */
static int hf_umts_rrc_pdu_def_dpc_Mode = -1;     /* DPC_Mode */
static int hf_umts_rrc_pdu_def_dl_TPC_PowerOffsetPerRL_List = -1;  /* DL_TPC_PowerOffsetPerRL_List */
static int hf_umts_rrc_pdu_def_primary_plmn_Identity = -1;  /* PLMN_Identity */
static int hf_umts_rrc_pdu_def_newH_RNTI = -1;    /* H_RNTI */
static int hf_umts_rrc_pdu_def_newPrimary_E_RNTI = -1;  /* E_RNTI */
static int hf_umts_rrc_pdu_def_newSecondary_E_RNTI = -1;  /* E_RNTI */
static int hf_umts_rrc_pdu_def_cn_InformationInfo1 = -1;  /* CN_InformationInfo_r6 */
static int hf_umts_rrc_pdu_def_rl_AdditionInformationList1 = -1;  /* RL_AdditionInformationList_r6 */
static int hf_umts_rrc_pdu_def_serving_HSDSCH_CellInformation = -1;  /* Serving_HSDSCH_CellInformation */
static int hf_umts_rrc_pdu_def_scheduling_E_DCH_CellInformation = -1;  /* Scheduling_E_DCH_CellInformation */
static int hf_umts_rrc_pdu_def_dummy_IntegrityProtActivationInfo = -1;  /* IntegrityProtActivationInfo */
static int hf_umts_rrc_pdu_def_dummy2_RB_ActivationTimeInfoList = -1;  /* RB_ActivationTimeInfoList */
static int hf_umts_rrc_pdu_def_dummy3_UL_CounterSynchronisationInfo = -1;  /* UL_CounterSynchronisationInfo */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions1 = -1;  /* T_laterNonCriticalExtensions1 */
static int hf_umts_rrc_pdu_def_activeSetUpdateComplete_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions2 = -1;  /* T_nonCriticalExtensions2 */
static int hf_umts_rrc_pdu_def_failureCause = -1;  /* FailureCauseWithProtErr */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions2 = -1;  /* T_laterNonCriticalExtensions2 */
static int hf_umts_rrc_pdu_def_activeSetUpdateFailure_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions3 = -1;  /* T_nonCriticalExtensions3 */
static int hf_umts_rrc_pdu_def_r31 = -1;          /* T_r31 */
static int hf_umts_rrc_pdu_def_assistanceDataDelivery_r3 = -1;  /* AssistanceDataDelivery_r3_IEs */
static int hf_umts_rrc_pdu_def_v3a0NonCriticalExtensions = -1;  /* T_v3a0NonCriticalExtensions */
static int hf_umts_rrc_pdu_def_assistanceDataDelivery_v3a0ext = -1;  /* AssistanceDataDelivery_v3a0ext */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions3 = -1;  /* T_laterNonCriticalExtensions3 */
static int hf_umts_rrc_pdu_def_assistanceDataDelivery_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions1 = -1;  /* T_v4b0NonCriticalExtensions1 */
static int hf_umts_rrc_pdu_def_assistanceDataDelivery_v4b0ext = -1;  /* AssistanceDataDelivery_v4b0ext_IEs */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions4 = -1;  /* T_nonCriticalExtensions4 */
static int hf_umts_rrc_pdu_def_later_than_r31 = -1;  /* T_later_than_r31 */
static int hf_umts_rrc_pdu_def_criticalExtensions2 = -1;  /* T_criticalExtensions2 */
static int hf_umts_rrc_pdu_def_ue_positioning_GPS_AssistanceData = -1;  /* UE_Positioning_GPS_AssistanceData */
static int hf_umts_rrc_pdu_def_ue_positioning_OTDOA_AssistanceData_UEB = -1;  /* UE_Positioning_OTDOA_AssistanceData_UEB */
static int hf_umts_rrc_pdu_def_sfn_Offset_Validity = -1;  /* SFN_Offset_Validity */
static int hf_umts_rrc_pdu_def_ue_Positioning_OTDOA_AssistanceData_r4ext = -1;  /* UE_Positioning_OTDOA_AssistanceData_r4ext */
static int hf_umts_rrc_pdu_def_r32 = -1;          /* T_r32 */
static int hf_umts_rrc_pdu_def_cellChangeOrderFromUTRAN_IEs = -1;  /* CellChangeOrderFromUTRAN_r3_IEs */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions4 = -1;  /* T_laterNonCriticalExtensions4 */
static int hf_umts_rrc_pdu_def_cellChangeOrderFromUTRAN_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v590NonCriticalExtensions1 = -1;  /* T_v590NonCriticalExtensions1 */
static int hf_umts_rrc_pdu_def_cellChangeOrderFromUTRAN_v590ext = -1;  /* CellChangeOrderFromUTRAN_v590ext_IEs */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions5 = -1;  /* T_nonCriticalExtensions5 */
static int hf_umts_rrc_pdu_def_later_than_r32 = -1;  /* T_later_than_r32 */
static int hf_umts_rrc_pdu_def_criticalExtensions3 = -1;  /* T_criticalExtensions3 */
static int hf_umts_rrc_pdu_def_rab_InformationList = -1;  /* RAB_InformationList */
static int hf_umts_rrc_pdu_def_interRAT_TargetCellDescription = -1;  /* InterRAT_TargetCellDescription */
static int hf_umts_rrc_pdu_def_geran_SystemInfoType = -1;  /* T_geran_SystemInfoType */
static int hf_umts_rrc_pdu_def_sI = -1;           /* GERAN_SystemInformation */
static int hf_umts_rrc_pdu_def_pSI = -1;          /* GERAN_SystemInformation */
static int hf_umts_rrc_pdu_def_r33 = -1;          /* T_r33 */
static int hf_umts_rrc_pdu_def_cellChangeOrderFromUTRANFailure_r3 = -1;  /* CellChangeOrderFromUTRANFailure_r3_IEs */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions5 = -1;  /* T_laterNonCriticalExtensions5 */
static int hf_umts_rrc_pdu_def_cellChangeOrderFromUTRANFailure_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions6 = -1;  /* T_nonCriticalExtensions6 */
static int hf_umts_rrc_pdu_def_dummy = -1;        /* T_dummy */
static int hf_umts_rrc_pdu_def_criticalExtensions4 = -1;  /* T_criticalExtensions4 */
static int hf_umts_rrc_pdu_def_interRAT_ChangeFailureCause = -1;  /* InterRAT_ChangeFailureCause */
static int hf_umts_rrc_pdu_def_u_RNTI = -1;       /* U_RNTI */
static int hf_umts_rrc_pdu_def_startList = -1;    /* STARTList */
static int hf_umts_rrc_pdu_def_am_RLC_ErrorIndicationRb2_3or4 = -1;  /* BOOLEAN */
static int hf_umts_rrc_pdu_def_am_RLC_ErrorIndicationRb5orAbove = -1;  /* BOOLEAN */
static int hf_umts_rrc_pdu_def_cellUpdateCause = -1;  /* CellUpdateCause */
static int hf_umts_rrc_pdu_def_failureCause1 = -1;  /* FailureCauseWithProtErrTrId */
static int hf_umts_rrc_pdu_def_rb_timer_indicator = -1;  /* Rb_timer_indicator */
static int hf_umts_rrc_pdu_def_measuredResultsOnRACH = -1;  /* MeasuredResultsOnRACH */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions6 = -1;  /* T_laterNonCriticalExtensions6 */
static int hf_umts_rrc_pdu_def_cellUpdate_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v590NonCriticalExtensions2 = -1;  /* T_v590NonCriticalExtensions2 */
static int hf_umts_rrc_pdu_def_cellUpdate_v590ext = -1;  /* CellUpdate_v590ext */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions1 = -1;  /* T_v6xyNonCriticalExtensions1 */
static int hf_umts_rrc_pdu_def_cellUpdate_v6xyext = -1;  /* CellUpdate_v6xyext_IEs */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions7 = -1;  /* T_nonCriticalExtensions7 */
static int hf_umts_rrc_pdu_def_establishmentCause = -1;  /* EstablishmentCause */
static int hf_umts_rrc_pdu_def_cellUpdateCause_ext = -1;  /* CellUpdateCause_ext */
static int hf_umts_rrc_pdu_def_trafficVolumeIndicator = -1;  /* T_trafficVolumeIndicator */
static int hf_umts_rrc_pdu_def_measuredResultsOnRACHinterFreq = -1;  /* MeasuredResultsOnRACHinterFreq */
static int hf_umts_rrc_pdu_def_reconfigurationStatusIndicator = -1;  /* T_reconfigurationStatusIndicator */
static int hf_umts_rrc_pdu_def_r34 = -1;          /* T_r34 */
static int hf_umts_rrc_pdu_def_cellUpdateConfirm_r3 = -1;  /* CellUpdateConfirm_r3_IEs */
static int hf_umts_rrc_pdu_def_v3a0NonCriticalExtensions1 = -1;  /* T_v3a0NonCriticalExtensions1 */
static int hf_umts_rrc_pdu_def_cellUpdateConfirm_v3a0ext = -1;  /* CellUpdateConfirm_v3a0ext */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions7 = -1;  /* T_laterNonCriticalExtensions7 */
static int hf_umts_rrc_pdu_def_cellUpdateConfirm_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions2 = -1;  /* T_v4b0NonCriticalExtensions2 */
static int hf_umts_rrc_pdu_def_cellUpdateConfirm_v4b0ext = -1;  /* CellUpdateConfirm_v4b0ext_IEs */
static int hf_umts_rrc_pdu_def_v590NonCriticalExtenstions = -1;  /* T_v590NonCriticalExtenstions */
static int hf_umts_rrc_pdu_def_cellUpdateConfirm_v590ext = -1;  /* CellUpdateConfirm_v590ext_IEs */
static int hf_umts_rrc_pdu_def_v5d0NonCriticalExtenstions = -1;  /* T_v5d0NonCriticalExtenstions */
static int hf_umts_rrc_pdu_def_cellUpdateConfirm_v5d0ext = -1;  /* CellUpdateConfirm_v5d0ext_IEs */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions2 = -1;  /* T_v6xyNonCriticalExtensions2 */
static int hf_umts_rrc_pdu_def_cellUpdateConfirm_v6xyext = -1;  /* CellUpdateConfirm_v6xyext_IEs */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions8 = -1;  /* T_nonCriticalExtensions8 */
static int hf_umts_rrc_pdu_def_later_than_r33 = -1;  /* T_later_than_r33 */
static int hf_umts_rrc_pdu_def_criticalExtensions5 = -1;  /* T_criticalExtensions5 */
static int hf_umts_rrc_pdu_def_r4 = -1;           /* T_r4 */
static int hf_umts_rrc_pdu_def_cellUpdateConfirm_r4 = -1;  /* CellUpdateConfirm_r4_IEs */
static int hf_umts_rrc_pdu_def_v4d0NonCriticalExtensions = -1;  /* T_v4d0NonCriticalExtensions */
static int hf_umts_rrc_pdu_def_cellUpdateConfirm_r4_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v590NonCriticalExtenstions1 = -1;  /* T_v590NonCriticalExtenstions1 */
static int hf_umts_rrc_pdu_def_v5d0NonCriticalExtenstions1 = -1;  /* T_v5d0NonCriticalExtenstions1 */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions3 = -1;  /* T_v6xyNonCriticalExtensions3 */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions9 = -1;  /* T_nonCriticalExtensions9 */
static int hf_umts_rrc_pdu_def_criticalExtensions6 = -1;  /* T_criticalExtensions6 */
static int hf_umts_rrc_pdu_def_r5 = -1;           /* T_r5 */
static int hf_umts_rrc_pdu_def_cellUpdateConfirm_r5 = -1;  /* CellUpdateConfirm_r5_IEs */
static int hf_umts_rrc_pdu_def_cellUpdateConfirm_r5_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v5d0NonCriticalExtenstions2 = -1;  /* T_v5d0NonCriticalExtenstions2 */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions4 = -1;  /* T_v6xyNonCriticalExtensions4 */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions10 = -1;  /* T_nonCriticalExtensions10 */
static int hf_umts_rrc_pdu_def_criticalExtensions7 = -1;  /* T_criticalExtensions7 */
static int hf_umts_rrc_pdu_def_r61 = -1;          /* T_r61 */
static int hf_umts_rrc_pdu_def_cellUpdateConfirm_r6 = -1;  /* CellUpdateConfirm_r6_IEs */
static int hf_umts_rrc_pdu_def_cellUpdateConfirm_r6_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions11 = -1;  /* T_nonCriticalExtensions11 */
static int hf_umts_rrc_pdu_def_criticalExtensions8 = -1;  /* T_criticalExtensions8 */
static int hf_umts_rrc_pdu_def_integrityProtectionModeInfo = -1;  /* IntegrityProtectionModeInfo */
static int hf_umts_rrc_pdu_def_cipheringModeInfo = -1;  /* CipheringModeInfo */
static int hf_umts_rrc_pdu_def_new_U_RNTI = -1;   /* U_RNTI */
static int hf_umts_rrc_pdu_def_new_C_RNTI = -1;   /* C_RNTI */
static int hf_umts_rrc_pdu_def_rrc_StateIndicator = -1;  /* RRC_StateIndicator */
static int hf_umts_rrc_pdu_def_utran_DRX_CycleLengthCoeff = -1;  /* UTRAN_DRX_CycleLengthCoefficient */
static int hf_umts_rrc_pdu_def_rlc_Re_establishIndicatorRb2_3or4 = -1;  /* BOOLEAN */
static int hf_umts_rrc_pdu_def_rlc_Re_establishIndicatorRb5orAbove = -1;  /* BOOLEAN */
static int hf_umts_rrc_pdu_def_ura_Identity = -1;  /* URA_Identity */
static int hf_umts_rrc_pdu_def_rb_InformationReleaseList = -1;  /* RB_InformationReleaseList */
static int hf_umts_rrc_pdu_def_rb_InformationReconfigList = -1;  /* RB_InformationReconfigList */
static int hf_umts_rrc_pdu_def_rb_InformationAffectedList = -1;  /* RB_InformationAffectedList */
static int hf_umts_rrc_pdu_def_dl_CounterSynchronisationInfo = -1;  /* DL_CounterSynchronisationInfo */
static int hf_umts_rrc_pdu_def_ul_CommonTransChInfo = -1;  /* UL_CommonTransChInfo */
static int hf_umts_rrc_pdu_def_ul_deletedTransChInfoList = -1;  /* UL_DeletedTransChInfoList */
static int hf_umts_rrc_pdu_def_ul_AddReconfTransChInfoList = -1;  /* UL_AddReconfTransChInfoList */
static int hf_umts_rrc_pdu_def_modeSpecificTransChInfo = -1;  /* T_modeSpecificTransChInfo */
static int hf_umts_rrc_pdu_def_fdd = -1;          /* T_fdd */
static int hf_umts_rrc_pdu_def_dummy_CPCH_SetID = -1;  /* CPCH_SetID */
static int hf_umts_rrc_pdu_def_dummy2_DRAC_StaticInformationList = -1;  /* DRAC_StaticInformationList */
static int hf_umts_rrc_pdu_def_tdd = -1;          /* NULL */
static int hf_umts_rrc_pdu_def_dl_CommonTransChInfo = -1;  /* DL_CommonTransChInfo */
static int hf_umts_rrc_pdu_def_dl_DeletedTransChInfoList = -1;  /* DL_DeletedTransChInfoList */
static int hf_umts_rrc_pdu_def_dl_AddReconfTransChInfoList = -1;  /* DL_AddReconfTransChInfoList */
static int hf_umts_rrc_pdu_def_frequencyInfo = -1;  /* FrequencyInfo */
static int hf_umts_rrc_pdu_def_ul_ChannelRequirement = -1;  /* UL_ChannelRequirement */
static int hf_umts_rrc_pdu_def_modeSpecificPhysChInfo = -1;  /* T_modeSpecificPhysChInfo */
static int hf_umts_rrc_pdu_def_fdd1 = -1;         /* T_fdd1 */
static int hf_umts_rrc_pdu_def_dummy_DL_PDSCH_Information = -1;  /* DL_PDSCH_Information */
static int hf_umts_rrc_pdu_def_dl_CommonInformation = -1;  /* DL_CommonInformation */
static int hf_umts_rrc_pdu_def_dl_InformationPerRL_List = -1;  /* DL_InformationPerRL_List */
static int hf_umts_rrc_pdu_def_new_DSCH_RNTI = -1;  /* DSCH_RNTI */
static int hf_umts_rrc_pdu_def_pdcp_ROHC_TargetMode = -1;  /* PDCP_ROHC_TargetMode */
static int hf_umts_rrc_pdu_def_rb_InformationReconfigList1 = -1;  /* RB_InformationReconfigList_r4 */
static int hf_umts_rrc_pdu_def_ul_CommonTransChInfo1 = -1;  /* UL_CommonTransChInfo_r4 */
static int hf_umts_rrc_pdu_def_modeSpecificTransChInfo1 = -1;  /* T_modeSpecificTransChInfo1 */
static int hf_umts_rrc_pdu_def_fdd2 = -1;         /* T_fdd2 */
static int hf_umts_rrc_pdu_def_dl_CommonTransChInfo1 = -1;  /* DL_CommonTransChInfo_r4 */
static int hf_umts_rrc_pdu_def_dl_AddReconfTransChInfoList1 = -1;  /* DL_AddReconfTransChInfoList_r4 */
static int hf_umts_rrc_pdu_def_ul_ChannelRequirement1 = -1;  /* UL_ChannelRequirement_r4 */
static int hf_umts_rrc_pdu_def_modeSpecificPhysChInfo1 = -1;  /* T_modeSpecificPhysChInfo1 */
static int hf_umts_rrc_pdu_def_fdd3 = -1;         /* T_fdd3 */
static int hf_umts_rrc_pdu_def_dl_CommonInformation1 = -1;  /* DL_CommonInformation_r4 */
static int hf_umts_rrc_pdu_def_dl_InformationPerRL_List1 = -1;  /* DL_InformationPerRL_List_r4 */
static int hf_umts_rrc_pdu_def_new_H_RNTI = -1;   /* H_RNTI */
static int hf_umts_rrc_pdu_def_rb_InformationReconfigList2 = -1;  /* RB_InformationReconfigList_r5 */
static int hf_umts_rrc_pdu_def_rb_InformationAffectedList1 = -1;  /* RB_InformationAffectedList_r5 */
static int hf_umts_rrc_pdu_def_dl_CounterSynchronisationInfo1 = -1;  /* DL_CounterSynchronisationInfo_r5 */
static int hf_umts_rrc_pdu_def_modeSpecificTransChInfo2 = -1;  /* T_modeSpecificTransChInfo2 */
static int hf_umts_rrc_pdu_def_fdd4 = -1;         /* T_fdd4 */
static int hf_umts_rrc_pdu_def_dl_DeletedTransChInfoList1 = -1;  /* DL_DeletedTransChInfoList_r5 */
static int hf_umts_rrc_pdu_def_dl_AddReconfTransChInfoList2 = -1;  /* DL_AddReconfTransChInfoList_r5 */
static int hf_umts_rrc_pdu_def_ul_ChannelRequirement2 = -1;  /* UL_ChannelRequirement_r5 */
static int hf_umts_rrc_pdu_def_modeSpecificPhysChInfo2 = -1;  /* T_modeSpecificPhysChInfo2 */
static int hf_umts_rrc_pdu_def_fdd5 = -1;         /* T_fdd5 */
static int hf_umts_rrc_pdu_def_dl_HSPDSCH_Information = -1;  /* DL_HSPDSCH_Information */
static int hf_umts_rrc_pdu_def_dl_CommonInformation2 = -1;  /* DL_CommonInformation_r5 */
static int hf_umts_rrc_pdu_def_dl_InformationPerRL_List2 = -1;  /* DL_InformationPerRL_List_r5 */
static int hf_umts_rrc_pdu_def_rb_InformationReconfigList3 = -1;  /* RB_InformationReconfigList_r6 */
static int hf_umts_rrc_pdu_def_rb_InformationAffectedList2 = -1;  /* RB_InformationAffectedList_r6 */
static int hf_umts_rrc_pdu_def_ul_deletedTransChInfoList1 = -1;  /* UL_DeletedTransChInfoList_r6 */
static int hf_umts_rrc_pdu_def_ul_AddReconfTransChInfoList1 = -1;  /* UL_AddReconfTransChInfoList_r6 */
static int hf_umts_rrc_pdu_def_ul_ChannelRequirement3 = -1;  /* UL_ChannelRequirement_r6 */
static int hf_umts_rrc_pdu_def_ul_EDCH_Information = -1;  /* UL_EDCH_Information_r6 */
static int hf_umts_rrc_pdu_def_dl_HSPDSCH_Information1 = -1;  /* DL_HSPDSCH_Information_r6 */
static int hf_umts_rrc_pdu_def_dl_CommonInformation3 = -1;  /* DL_CommonInformation_r6 */
static int hf_umts_rrc_pdu_def_dl_InformationPerRL_List3 = -1;  /* DL_InformationPerRL_List_r6 */
static int hf_umts_rrc_pdu_def_mbms_PL_ServiceRestrictInfo = -1;  /* MBMS_PL_ServiceRestrictInfo_r6 */
static int hf_umts_rrc_pdu_def_harq_Preamble_Mode = -1;  /* HARQ_Preamble_Mode */
static int hf_umts_rrc_pdu_def_beaconPLEst = -1;  /* BEACON_PL_Est */
static int hf_umts_rrc_pdu_def_postVerificationPeriod = -1;  /* T_postVerificationPeriod */
static int hf_umts_rrc_pdu_def_dhs_sync = -1;     /* DHS_Sync */
static int hf_umts_rrc_pdu_def_r35 = -1;          /* T_r35 */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions8 = -1;  /* T_laterNonCriticalExtensions8 */
static int hf_umts_rrc_pdu_def_cellUpdateConfirm_CCCH_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions3 = -1;  /* T_v4b0NonCriticalExtensions3 */
static int hf_umts_rrc_pdu_def_v590NonCriticalExtensions3 = -1;  /* T_v590NonCriticalExtensions3 */
static int hf_umts_rrc_pdu_def_v5d0NonCriticalExtenstions3 = -1;  /* T_v5d0NonCriticalExtenstions3 */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions5 = -1;  /* T_v6xyNonCriticalExtensions5 */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions12 = -1;  /* T_nonCriticalExtensions12 */
static int hf_umts_rrc_pdu_def_later_than_r34 = -1;  /* T_later_than_r34 */
static int hf_umts_rrc_pdu_def_criticalExtensions9 = -1;  /* T_criticalExtensions9 */
static int hf_umts_rrc_pdu_def_r41 = -1;          /* T_r41 */
static int hf_umts_rrc_pdu_def_v4d0NonCriticalExtensions1 = -1;  /* T_v4d0NonCriticalExtensions1 */
static int hf_umts_rrc_pdu_def_cellUpdateConfirm_CCCH_r4_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v590NonCriticalExtensions4 = -1;  /* T_v590NonCriticalExtensions4 */
static int hf_umts_rrc_pdu_def_v5d0NonCriticalExtenstions4 = -1;  /* T_v5d0NonCriticalExtenstions4 */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions6 = -1;  /* T_v6xyNonCriticalExtensions6 */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions13 = -1;  /* T_nonCriticalExtensions13 */
static int hf_umts_rrc_pdu_def_criticalExtensions10 = -1;  /* T_criticalExtensions10 */
static int hf_umts_rrc_pdu_def_r51 = -1;          /* T_r51 */
static int hf_umts_rrc_pdu_def_cellUpdateConfirm_CCCH_r5_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v5d0NonCriticalExtenstions5 = -1;  /* T_v5d0NonCriticalExtenstions5 */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions7 = -1;  /* T_v6xyNonCriticalExtensions7 */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions14 = -1;  /* T_nonCriticalExtensions14 */
static int hf_umts_rrc_pdu_def_criticalExtensions11 = -1;  /* T_criticalExtensions11 */
static int hf_umts_rrc_pdu_def_r62 = -1;          /* T_r62 */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions15 = -1;  /* T_nonCriticalExtensions15 */
static int hf_umts_rrc_pdu_def_criticalExtensions12 = -1;  /* T_criticalExtensions12 */
static int hf_umts_rrc_pdu_def_r36 = -1;          /* T_r36 */
static int hf_umts_rrc_pdu_def_counterCheck_r3 = -1;  /* CounterCheck_r3_IEs */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions9 = -1;  /* T_laterNonCriticalExtensions9 */
static int hf_umts_rrc_pdu_def_counterCheck_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions16 = -1;  /* T_nonCriticalExtensions16 */
static int hf_umts_rrc_pdu_def_later_than_r35 = -1;  /* T_later_than_r35 */
static int hf_umts_rrc_pdu_def_criticalExtensions13 = -1;  /* T_criticalExtensions13 */
static int hf_umts_rrc_pdu_def_rb_COUNT_C_MSB_InformationList = -1;  /* RB_COUNT_C_MSB_InformationList */
static int hf_umts_rrc_pdu_def_rb_COUNT_C_InformationList = -1;  /* RB_COUNT_C_InformationList */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions10 = -1;  /* T_laterNonCriticalExtensions10 */
static int hf_umts_rrc_pdu_def_counterCheckResponse_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions17 = -1;  /* T_nonCriticalExtensions17 */
static int hf_umts_rrc_pdu_def_r37 = -1;          /* T_r37 */
static int hf_umts_rrc_pdu_def_downlinkDirectTransfer_r3 = -1;  /* DownlinkDirectTransfer_r3_IEs */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions11 = -1;  /* T_laterNonCriticalExtensions11 */
static int hf_umts_rrc_pdu_def_downlinkDirectTransfer_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions18 = -1;  /* T_nonCriticalExtensions18 */
static int hf_umts_rrc_pdu_def_later_than_r36 = -1;  /* T_later_than_r36 */
static int hf_umts_rrc_pdu_def_criticalExtensions14 = -1;  /* T_criticalExtensions14 */
static int hf_umts_rrc_pdu_def_cn_DomainIdentity = -1;  /* CN_DomainIdentity */
static int hf_umts_rrc_pdu_def_nas_Message = -1;  /* NAS_Message */
static int hf_umts_rrc_pdu_def_r38 = -1;          /* T_r38 */
static int hf_umts_rrc_pdu_def_handoverToUTRANCommand_r3 = -1;  /* HandoverToUTRANCommand_r3_IEs */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions19 = -1;  /* T_nonCriticalExtensions19 */
static int hf_umts_rrc_pdu_def_criticalExtensions15 = -1;  /* T_criticalExtensions15 */
static int hf_umts_rrc_pdu_def_r42 = -1;          /* T_r42 */
static int hf_umts_rrc_pdu_def_handoverToUTRANCommand_r4 = -1;  /* HandoverToUTRANCommand_r4_IEs */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions20 = -1;  /* T_nonCriticalExtensions20 */
static int hf_umts_rrc_pdu_def_criticalExtensions16 = -1;  /* T_criticalExtensions16 */
static int hf_umts_rrc_pdu_def_r52 = -1;          /* T_r52 */
static int hf_umts_rrc_pdu_def_handoverToUTRANCommand_r5 = -1;  /* HandoverToUTRANCommand_r5_IEs */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions21 = -1;  /* T_nonCriticalExtensions21 */
static int hf_umts_rrc_pdu_def_criticalExtensions17 = -1;  /* T_criticalExtensions17 */
static int hf_umts_rrc_pdu_def_r63 = -1;          /* T_r63 */
static int hf_umts_rrc_pdu_def_handoverToUTRANCommand_r6 = -1;  /* HandoverToUTRANCommand_r6_IEs */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions22 = -1;  /* T_nonCriticalExtensions22 */
static int hf_umts_rrc_pdu_def_criticalExtensions18 = -1;  /* T_criticalExtensions18 */
static int hf_umts_rrc_pdu_def_new_U_RNTI1 = -1;  /* U_RNTI_Short */
static int hf_umts_rrc_pdu_def_dummy_ActivationTime = -1;  /* ActivationTime */
static int hf_umts_rrc_pdu_def_cipheringAlgorithm = -1;  /* CipheringAlgorithm */
static int hf_umts_rrc_pdu_def_specificationMode = -1;  /* T_specificationMode */
static int hf_umts_rrc_pdu_def_complete = -1;     /* T_complete */
static int hf_umts_rrc_pdu_def_srb_InformationSetupList = -1;  /* SRB_InformationSetupList */
static int hf_umts_rrc_pdu_def_rab_InformationSetupList = -1;  /* RAB_InformationSetupList */
static int hf_umts_rrc_pdu_def_ul_DPCH_Info = -1;  /* UL_DPCH_Info */
static int hf_umts_rrc_pdu_def_modeSpecificInfo = -1;  /* T_modeSpecificInfo */
static int hf_umts_rrc_pdu_def_fdd6 = -1;         /* T_fdd6 */
static int hf_umts_rrc_pdu_def_dummy2_CPCH_SetInfo = -1;  /* CPCH_SetInfo */
static int hf_umts_rrc_pdu_def_preconfiguration = -1;  /* T_preconfiguration */
static int hf_umts_rrc_pdu_def_preConfigMode = -1;  /* T_preConfigMode */
static int hf_umts_rrc_pdu_def_predefinedConfigIdentity = -1;  /* PredefinedConfigIdentity */
static int hf_umts_rrc_pdu_def_defaultConfig = -1;  /* T_defaultConfig */
static int hf_umts_rrc_pdu_def_defaultConfigMode = -1;  /* DefaultConfigMode */
static int hf_umts_rrc_pdu_def_defaultConfigIdentity = -1;  /* DefaultConfigIdentity */
static int hf_umts_rrc_pdu_def_rab_Info = -1;     /* RAB_Info_Post */
static int hf_umts_rrc_pdu_def_modeSpecificInfo1 = -1;  /* T_modeSpecificInfo1 */
static int hf_umts_rrc_pdu_def_fdd7 = -1;         /* T_fdd7 */
static int hf_umts_rrc_pdu_def_ul_DPCH_Info1 = -1;  /* UL_DPCH_InfoPostFDD */
static int hf_umts_rrc_pdu_def_dl_CommonInformationPost = -1;  /* DL_CommonInformationPost */
static int hf_umts_rrc_pdu_def_dl_InformationPerRL_List4 = -1;  /* DL_InformationPerRL_ListPostFDD */
static int hf_umts_rrc_pdu_def_frequencyInfo1 = -1;  /* FrequencyInfoFDD */
static int hf_umts_rrc_pdu_def_tdd1 = -1;         /* T_tdd */
static int hf_umts_rrc_pdu_def_ul_DPCH_Info2 = -1;  /* UL_DPCH_InfoPostTDD */
static int hf_umts_rrc_pdu_def_dl_InformationPerRL = -1;  /* DL_InformationPerRL_PostTDD */
static int hf_umts_rrc_pdu_def_frequencyInfo2 = -1;  /* FrequencyInfoTDD */
static int hf_umts_rrc_pdu_def_primaryCCPCH_TX_Power = -1;  /* PrimaryCCPCH_TX_Power */
static int hf_umts_rrc_pdu_def_specificationMode1 = -1;  /* T_specificationMode1 */
static int hf_umts_rrc_pdu_def_complete1 = -1;    /* T_complete1 */
static int hf_umts_rrc_pdu_def_rab_InformationSetupList1 = -1;  /* RAB_InformationSetupList_r4 */
static int hf_umts_rrc_pdu_def_ul_DPCH_Info3 = -1;  /* UL_DPCH_Info_r4 */
static int hf_umts_rrc_pdu_def_modeSpecificInfo2 = -1;  /* T_modeSpecificInfo2 */
static int hf_umts_rrc_pdu_def_fdd8 = -1;         /* T_fdd8 */
static int hf_umts_rrc_pdu_def_preconfiguration1 = -1;  /* T_preconfiguration1 */
static int hf_umts_rrc_pdu_def_preConfigMode1 = -1;  /* T_preConfigMode1 */
static int hf_umts_rrc_pdu_def_defaultConfig1 = -1;  /* T_defaultConfig1 */
static int hf_umts_rrc_pdu_def_defaultConfigIdentity1 = -1;  /* DefaultConfigIdentity_r4 */
static int hf_umts_rrc_pdu_def_modeSpecificInfo3 = -1;  /* T_modeSpecificInfo3 */
static int hf_umts_rrc_pdu_def_fdd9 = -1;         /* T_fdd9 */
static int hf_umts_rrc_pdu_def_tdd2 = -1;         /* T_tdd1 */
static int hf_umts_rrc_pdu_def_tdd384 = -1;       /* T_tdd384 */
static int hf_umts_rrc_pdu_def_tdd128 = -1;       /* T_tdd128 */
static int hf_umts_rrc_pdu_def_ul_DPCH_Info4 = -1;  /* UL_DPCH_InfoPostTDD_LCR_r4 */
static int hf_umts_rrc_pdu_def_dl_InformationPerRL1 = -1;  /* DL_InformationPerRL_PostTDD_LCR_r4 */
static int hf_umts_rrc_pdu_def_specificationMode2 = -1;  /* T_specificationMode2 */
static int hf_umts_rrc_pdu_def_complete2 = -1;    /* T_complete2 */
static int hf_umts_rrc_pdu_def_srb_InformationSetupList1 = -1;  /* SRB_InformationSetupList_r5 */
static int hf_umts_rrc_pdu_def_rab_InformationSetupList2 = -1;  /* RAB_InformationSetupList_r5 */
static int hf_umts_rrc_pdu_def_ul_DPCH_Info5 = -1;  /* UL_DPCH_Info_r5 */
static int hf_umts_rrc_pdu_def_modeSpecificInfo4 = -1;  /* T_modeSpecificInfo4 */
static int hf_umts_rrc_pdu_def_fdd10 = -1;        /* T_fdd10 */
static int hf_umts_rrc_pdu_def_preconfiguration2 = -1;  /* T_preconfiguration2 */
static int hf_umts_rrc_pdu_def_preConfigMode2 = -1;  /* T_preConfigMode2 */
static int hf_umts_rrc_pdu_def_defaultConfig2 = -1;  /* T_defaultConfig2 */
static int hf_umts_rrc_pdu_def_defaultConfigIdentity2 = -1;  /* DefaultConfigIdentity_r5 */
static int hf_umts_rrc_pdu_def_modeSpecificInfo5 = -1;  /* T_modeSpecificInfo5 */
static int hf_umts_rrc_pdu_def_fdd11 = -1;        /* T_fdd11 */
static int hf_umts_rrc_pdu_def_tdd3 = -1;         /* T_tdd2 */
static int hf_umts_rrc_pdu_def_tdd3841 = -1;      /* T_tdd3841 */
static int hf_umts_rrc_pdu_def_tdd1281 = -1;      /* T_tdd1281 */
static int hf_umts_rrc_pdu_def_specificationMode3 = -1;  /* T_specificationMode3 */
static int hf_umts_rrc_pdu_def_complete3 = -1;    /* T_complete3 */
static int hf_umts_rrc_pdu_def_srb_InformationSetupList2 = -1;  /* SRB_InformationSetupList_r6 */
static int hf_umts_rrc_pdu_def_rab_InformationSetupList3 = -1;  /* RAB_InformationSetupList_r6 */
static int hf_umts_rrc_pdu_def_ul_DPCH_Info6 = -1;  /* UL_DPCH_Info_r6 */
static int hf_umts_rrc_pdu_def_count_C_ActivationTime = -1;  /* ActivationTime */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions12 = -1;  /* T_laterNonCriticalExtensions12 */
static int hf_umts_rrc_pdu_def_handoverToUTRANComplete_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions23 = -1;  /* T_nonCriticalExtensions23 */
static int hf_umts_rrc_pdu_def_intraDomainNasNodeSelector = -1;  /* IntraDomainNasNodeSelector */
static int hf_umts_rrc_pdu_def_v3a0NonCriticalExtensions2 = -1;  /* T_v3a0NonCriticalExtensions2 */
static int hf_umts_rrc_pdu_def_initialDirectTransfer_v3a0ext = -1;  /* InitialDirectTransfer_v3a0ext */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions13 = -1;  /* T_laterNonCriticalExtensions13 */
static int hf_umts_rrc_pdu_def_initialDirectTransfer_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v590NonCriticalExtensions5 = -1;  /* T_v590NonCriticalExtensions5 */
static int hf_umts_rrc_pdu_def_initialDirectTransfer_v590ext = -1;  /* InitialDirectTransfer_v590ext */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions8 = -1;  /* T_v6xyNonCriticalExtensions8 */
static int hf_umts_rrc_pdu_def_initialDirectTransfer_v6xyext = -1;  /* InitialDirectTransfer_v6xyext_IEs */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions24 = -1;  /* T_nonCriticalExtensions24 */
static int hf_umts_rrc_pdu_def_start_Value = -1;  /* START_Value */
static int hf_umts_rrc_pdu_def_plmn_Identity = -1;  /* PLMN_Identity */
static int hf_umts_rrc_pdu_def_mbms_JoinedInformation = -1;  /* MBMS_JoinedInformation_r6 */
static int hf_umts_rrc_pdu_def_r39 = -1;          /* T_r39 */
static int hf_umts_rrc_pdu_def_handoverFromUTRANCommand_GSM_r3 = -1;  /* HandoverFromUTRANCommand_GSM_r3_IEs */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions14 = -1;  /* T_laterNonCriticalExtensions14 */
static int hf_umts_rrc_pdu_def_handoverFromUTRANCommand_GSM_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions9 = -1;  /* T_v6xyNonCriticalExtensions9 */
static int hf_umts_rrc_pdu_def_handoverFromUTRANCommand_GSM_v6xyext = -1;  /* HandoverFromUTRANCommand_GSM_v6xyext_IEs */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions25 = -1;  /* T_nonCriticalExtensions25 */
static int hf_umts_rrc_pdu_def_later_than_r37 = -1;  /* T_later_than_r37 */
static int hf_umts_rrc_pdu_def_criticalExtensions19 = -1;  /* T_criticalExtensions19 */
static int hf_umts_rrc_pdu_def_r64 = -1;          /* T_r64 */
static int hf_umts_rrc_pdu_def_handoverFromUTRANCommand_GSM_r6 = -1;  /* HandoverFromUTRANCommand_GSM_r6_IEs */
static int hf_umts_rrc_pdu_def_handoverFromUTRANCommand_GSM_r6_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions26 = -1;  /* T_nonCriticalExtensions26 */
static int hf_umts_rrc_pdu_def_criticalExtensions20 = -1;  /* T_criticalExtensions20 */
static int hf_umts_rrc_pdu_def_toHandoverRAB_Info = -1;  /* RAB_Info */
static int hf_umts_rrc_pdu_def_frequency_band = -1;  /* Frequency_Band */
static int hf_umts_rrc_pdu_def_gsm_message = -1;  /* T_gsm_message */
static int hf_umts_rrc_pdu_def_single_GSM_Message = -1;  /* T_single_GSM_Message */
static int hf_umts_rrc_pdu_def_gsm_MessageList = -1;  /* T_gsm_MessageList */
static int hf_umts_rrc_pdu_def_gsm_Messages = -1;  /* GSM_MessageList */
static int hf_umts_rrc_pdu_def_toHandoverRAB_Info1 = -1;  /* RAB_InformationList_r6 */
static int hf_umts_rrc_pdu_def_gsm_message1 = -1;  /* T_gsm_message1 */
static int hf_umts_rrc_pdu_def_single_GSM_Message1 = -1;  /* T_single_GSM_Message1 */
static int hf_umts_rrc_pdu_def_gsm_MessageList1 = -1;  /* T_gsm_MessageList1 */
static int hf_umts_rrc_pdu_def_geran_SystemInfoType1 = -1;  /* T_geran_SystemInfoType1 */
static int hf_umts_rrc_pdu_def_geran_SystemInfoType2 = -1;  /* T_geran_SystemInfoType2 */
static int hf_umts_rrc_pdu_def_handoverFromUTRANCommand_GERANIu = -1;  /* T_handoverFromUTRANCommand_GERANIu */
static int hf_umts_rrc_pdu_def_r53 = -1;          /* T_r53 */
static int hf_umts_rrc_pdu_def_handoverFromUTRANCommand_GERANIu_r5 = -1;  /* HandoverFromUTRANCommand_GERANIu_r5_IEs */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions27 = -1;  /* T_nonCriticalExtensions27 */
static int hf_umts_rrc_pdu_def_later_than_r5 = -1;  /* T_later_than_r5 */
static int hf_umts_rrc_pdu_def_criticalExtensions21 = -1;  /* T_criticalExtensions21 */
static int hf_umts_rrc_pdu_def_frequency_Band = -1;  /* Frequency_Band */
static int hf_umts_rrc_pdu_def_geranIu_Message = -1;  /* T_geranIu_Message */
static int hf_umts_rrc_pdu_def_single_GERANIu_Message = -1;  /* T_single_GERANIu_Message */
static int hf_umts_rrc_pdu_def_geranIu_MessageList = -1;  /* T_geranIu_MessageList */
static int hf_umts_rrc_pdu_def_geranIu_Messages = -1;  /* GERANIu_MessageList */
static int hf_umts_rrc_pdu_def_r310 = -1;         /* T_r310 */
static int hf_umts_rrc_pdu_def_handoverFromUTRANCommand_CDMA2000_r3 = -1;  /* HandoverFromUTRANCommand_CDMA2000_r3_IEs */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions15 = -1;  /* T_laterNonCriticalExtensions15 */
static int hf_umts_rrc_pdu_def_handoverFromUTRANCommand_CDMA2000_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions28 = -1;  /* T_nonCriticalExtensions28 */
static int hf_umts_rrc_pdu_def_later_than_r38 = -1;  /* T_later_than_r38 */
static int hf_umts_rrc_pdu_def_criticalExtensions22 = -1;  /* T_criticalExtensions22 */
static int hf_umts_rrc_pdu_def_cdma2000_MessageList = -1;  /* CDMA2000_MessageList */
static int hf_umts_rrc_pdu_def_interRAT_HO_FailureCause = -1;  /* InterRAT_HO_FailureCause */
static int hf_umts_rrc_pdu_def_interRATMessage = -1;  /* T_interRATMessage */
static int hf_umts_rrc_pdu_def_gsm = -1;          /* T_gsm */
static int hf_umts_rrc_pdu_def_gsm_MessageList2 = -1;  /* GSM_MessageList */
static int hf_umts_rrc_pdu_def_cdma2000 = -1;     /* T_cdma2000 */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions16 = -1;  /* T_laterNonCriticalExtensions16 */
static int hf_umts_rrc_pdu_def_handoverFromUTRANFailure_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v590NonCriticalExtensions6 = -1;  /* T_v590NonCriticalExtensions6 */
static int hf_umts_rrc_pdu_def_handoverFromUTRANFailure_v590ext = -1;  /* HandoverFromUtranFailure_v590ext_IEs */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions29 = -1;  /* T_nonCriticalExtensions29 */
static int hf_umts_rrc_pdu_def_geranIu_MessageList1 = -1;  /* GERANIu_MessageList */
static int hf_umts_rrc_pdu_def_predefinedConfigStatusList = -1;  /* T_predefinedConfigStatusList */
static int hf_umts_rrc_pdu_def_absent = -1;       /* NULL */
static int hf_umts_rrc_pdu_def_present = -1;      /* PredefinedConfigStatusList */
static int hf_umts_rrc_pdu_def_uE_SecurityInformation = -1;  /* T_uE_SecurityInformation */
static int hf_umts_rrc_pdu_def_present1 = -1;     /* UE_SecurityInformation */
static int hf_umts_rrc_pdu_def_ue_CapabilityContainer = -1;  /* T_ue_CapabilityContainer */
static int hf_umts_rrc_pdu_def_present2 = -1;     /* OCTET_STRING_SIZE_0_63 */
static int hf_umts_rrc_pdu_def_v390NonCriticalExtensions = -1;  /* T_v390NonCriticalExtensions */
static int hf_umts_rrc_pdu_def_present3 = -1;     /* T_present */
static int hf_umts_rrc_pdu_def_interRATHandoverInfo_v390ext = -1;  /* InterRATHandoverInfo_v390ext_IEs */
static int hf_umts_rrc_pdu_def_v3a0NonCriticalExtensions3 = -1;  /* T_v3a0NonCriticalExtensions3 */
static int hf_umts_rrc_pdu_def_interRATHandoverInfo_v3a0ext = -1;  /* InterRATHandoverInfo_v3a0ext_IEs */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions17 = -1;  /* T_laterNonCriticalExtensions17 */
static int hf_umts_rrc_pdu_def_interRATHandoverInfo_v3d0ext = -1;  /* InterRATHandoverInfo_v3d0ext_IEs */
static int hf_umts_rrc_pdu_def_interRATHandoverInfo_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v3g0NonCriticalExtensions = -1;  /* T_v3g0NonCriticalExtensions */
static int hf_umts_rrc_pdu_def_interRATHandoverInfo_v3g0ext = -1;  /* InterRATHandoverInfo_v3g0ext_IEs */
static int hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions4 = -1;  /* T_v4b0NonCriticalExtensions4 */
static int hf_umts_rrc_pdu_def_interRATHandoverInfo_v4b0ext = -1;  /* InterRATHandoverInfo_v4b0ext_IEs */
static int hf_umts_rrc_pdu_def_v4d0NonCriticalExtensions2 = -1;  /* T_v4d0NonCriticalExtensions2 */
static int hf_umts_rrc_pdu_def_interRATHandoverInfo_v4d0ext = -1;  /* InterRATHandoverInfo_v4d0ext_IEs */
static int hf_umts_rrc_pdu_def_v590NonCriticalExtensions7 = -1;  /* T_v590NonCriticalExtensions7 */
static int hf_umts_rrc_pdu_def_interRATHandoverInfo_v590ext = -1;  /* InterRATHandoverInfo_v590ext_IEs */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions10 = -1;  /* T_v6xyNonCriticalExtensions10 */
static int hf_umts_rrc_pdu_def_interRATHandoverInfo_v6xy0ext = -1;  /* InterRATHandoverInfo_v6xyext_IEs */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions30 = -1;  /* T_nonCriticalExtensions30 */
static int hf_umts_rrc_pdu_def_ue_RadioAccessCapability_v380ext = -1;  /* UE_RadioAccessCapability_v380ext */
static int hf_umts_rrc_pdu_def_dl_PhysChCapabilityFDD_v380ext = -1;  /* DL_PhysChCapabilityFDD_v380ext */
static int hf_umts_rrc_pdu_def_ue_RadioAccessCapability_v3a0ext = -1;  /* UE_RadioAccessCapability_v3a0ext */
static int hf_umts_rrc_pdu_def_uESpecificBehaviourInformation1interRAT = -1;  /* UESpecificBehaviourInformation1interRAT */
static int hf_umts_rrc_pdu_def_ue_RadioAccessCapability_v3g0ext = -1;  /* UE_RadioAccessCapability_v3g0ext */
static int hf_umts_rrc_pdu_def_accessStratumReleaseIndicator = -1;  /* AccessStratumReleaseIndicator */
static int hf_umts_rrc_pdu_def_tdd128_RF_Capability = -1;  /* RadioFrequencyBandTDDList */
static int hf_umts_rrc_pdu_def_predefinedConfigStatusListComp = -1;  /* PredefinedConfigStatusListComp */
static int hf_umts_rrc_pdu_def_ue_RadioAccessCapabilityComp = -1;  /* UE_RadioAccessCapabilityComp */
static int hf_umts_rrc_pdu_def_ue_SecurityInformation2 = -1;  /* UE_SecurityInformation2 */
static int hf_umts_rrc_pdu_def_r311 = -1;         /* T_r311 */
static int hf_umts_rrc_pdu_def_measurementControl_r3 = -1;  /* MeasurementControl_r3_IEs */
static int hf_umts_rrc_pdu_def_v390nonCriticalExtensions = -1;  /* T_v390nonCriticalExtensions */
static int hf_umts_rrc_pdu_def_measurementControl_v390ext = -1;  /* MeasurementControl_v390ext */
static int hf_umts_rrc_pdu_def_v3a0NonCriticalExtensions4 = -1;  /* T_v3a0NonCriticalExtensions4 */
static int hf_umts_rrc_pdu_def_measurementControl_v3a0ext = -1;  /* MeasurementControl_v3a0ext */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions18 = -1;  /* T_laterNonCriticalExtensions18 */
static int hf_umts_rrc_pdu_def_measurementControl_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions5 = -1;  /* T_v4b0NonCriticalExtensions5 */
static int hf_umts_rrc_pdu_def_v590NonCriticalExtensions8 = -1;  /* T_v590NonCriticalExtensions8 */
static int hf_umts_rrc_pdu_def_measurementControl_v590ext = -1;  /* MeasurementControl_v590ext_IEs */
static int hf_umts_rrc_pdu_def_v5b0NonCriticalExtensions = -1;  /* T_v5b0NonCriticalExtensions */
static int hf_umts_rrc_pdu_def_measurementControl_v5b0ext = -1;  /* MeasurementControl_v5b0ext_IEs */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions31 = -1;  /* T_nonCriticalExtensions31 */
static int hf_umts_rrc_pdu_def_later_than_r39 = -1;  /* T_later_than_r39 */
static int hf_umts_rrc_pdu_def_criticalExtensions23 = -1;  /* T_criticalExtensions23 */
static int hf_umts_rrc_pdu_def_r43 = -1;          /* T_r43 */
static int hf_umts_rrc_pdu_def_measurementControl_r4 = -1;  /* MeasurementControl_r4_IEs */
static int hf_umts_rrc_pdu_def_v4d0NonCriticalExtensions3 = -1;  /* T_v4d0NonCriticalExtensions3 */
static int hf_umts_rrc_pdu_def_measurementControl_r4_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v590NonCriticalExtensions9 = -1;  /* T_v590NonCriticalExtensions9 */
static int hf_umts_rrc_pdu_def_v5b0NonCriticalExtensions1 = -1;  /* T_v5b0NonCriticalExtensions1 */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions32 = -1;  /* T_nonCriticalExtensions32 */
static int hf_umts_rrc_pdu_def_criticalExtensions24 = -1;  /* T_criticalExtensions24 */
static int hf_umts_rrc_pdu_def_measurementIdentity = -1;  /* MeasurementIdentity */
static int hf_umts_rrc_pdu_def_measurementCommand = -1;  /* MeasurementCommand */
static int hf_umts_rrc_pdu_def_measurementReportingMode = -1;  /* MeasurementReportingMode */
static int hf_umts_rrc_pdu_def_additionalMeasurementList = -1;  /* AdditionalMeasurementID_List */
static int hf_umts_rrc_pdu_def_dpch_CompressedModeStatusInfo = -1;  /* DPCH_CompressedModeStatusInfo */
static int hf_umts_rrc_pdu_def_ue_Positioning_Measurement_v390ext = -1;  /* UE_Positioning_Measurement_v390ext */
static int hf_umts_rrc_pdu_def_measurementCommand1 = -1;  /* MeasurementCommand_r4 */
static int hf_umts_rrc_pdu_def_measurementCommand_v590ext = -1;  /* T_measurementCommand_v590ext */
static int hf_umts_rrc_pdu_def_intra_frequency = -1;  /* Intra_FreqEventCriteriaList_v590ext */
static int hf_umts_rrc_pdu_def_inter_frequency = -1;  /* Inter_FreqEventCriteriaList_v590ext */
static int hf_umts_rrc_pdu_def_intraFreqReportingCriteria_1b_r5 = -1;  /* IntraFreqReportingCriteria_1b_r5 */
static int hf_umts_rrc_pdu_def_intraFreqEvent_1d_r5 = -1;  /* IntraFreqEvent_1d_r5 */
static int hf_umts_rrc_pdu_def_rrc_TransactionIdentifier_MSP_v590ext = -1;  /* RRC_TransactionIdentifier */
static int hf_umts_rrc_pdu_def_interRATCellInfoIndication = -1;  /* InterRATCellInfoIndication */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions19 = -1;  /* T_laterNonCriticalExtensions19 */
static int hf_umts_rrc_pdu_def_measurementControlFailure_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v590NonCriticalExtensions10 = -1;  /* T_v590NonCriticalExtensions10 */
static int hf_umts_rrc_pdu_def_measurementControlFailure_v590ext = -1;  /* MeasurementControlFailure_v590ext_IEs */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions33 = -1;  /* T_nonCriticalExtensions33 */
static int hf_umts_rrc_pdu_def_measuredResults = -1;  /* MeasuredResults */
static int hf_umts_rrc_pdu_def_additionalMeasuredResults = -1;  /* MeasuredResultsList */
static int hf_umts_rrc_pdu_def_eventResults = -1;  /* EventResults */
static int hf_umts_rrc_pdu_def_v390nonCriticalExtensions1 = -1;  /* T_v390nonCriticalExtensions1 */
static int hf_umts_rrc_pdu_def_measurementReport_v390ext = -1;  /* MeasurementReport_v390ext */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions20 = -1;  /* T_laterNonCriticalExtensions20 */
static int hf_umts_rrc_pdu_def_measurementReport_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions6 = -1;  /* T_v4b0NonCriticalExtensions6 */
static int hf_umts_rrc_pdu_def_measurementReport_v4b0ext = -1;  /* MeasurementReport_v4b0ext_IEs */
static int hf_umts_rrc_pdu_def_v590NonCriticalExtensions11 = -1;  /* T_v590NonCriticalExtensions11 */
static int hf_umts_rrc_pdu_def_measurementReport_v590ext = -1;  /* MeasurementReport_v590ext_IEs */
static int hf_umts_rrc_pdu_def_v5b0NonCriticalExtensions2 = -1;  /* T_v5b0NonCriticalExtensions2 */
static int hf_umts_rrc_pdu_def_measurementReport_v5b0ext = -1;  /* MeasurementReport_v5b0ext_IEs */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions11 = -1;  /* T_v6xyNonCriticalExtensions11 */
static int hf_umts_rrc_pdu_def_measurementReport_v6xyext = -1;  /* MeasurementReport_v6xyext_IEs */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions34 = -1;  /* T_nonCriticalExtensions34 */
static int hf_umts_rrc_pdu_def_measuredResults_v390ext = -1;  /* MeasuredResults_v390ext */
static int hf_umts_rrc_pdu_def_interFreqEventResults_LCR = -1;  /* InterFreqEventResults_LCR_r4_ext */
static int hf_umts_rrc_pdu_def_additionalMeasuredResults_LCR = -1;  /* MeasuredResultsList_LCR_r4_ext */
static int hf_umts_rrc_pdu_def_gsmOTDreferenceCell = -1;  /* PrimaryCPICH_Info */
static int hf_umts_rrc_pdu_def_measuredResults_v590ext = -1;  /* MeasuredResults_v590ext */
static int hf_umts_rrc_pdu_def_pagingRecordList = -1;  /* PagingRecordList */
static int hf_umts_rrc_pdu_def_bcch_ModificationInfo = -1;  /* BCCH_ModificationInfo */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions21 = -1;  /* T_laterNonCriticalExtensions21 */
static int hf_umts_rrc_pdu_def_pagingType1_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v590NonCriticalExtensions12 = -1;  /* T_v590NonCriticalExtensions12 */
static int hf_umts_rrc_pdu_def_pagingType1_v590ext = -1;  /* PagingType1_v590ext_IEs */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions35 = -1;  /* T_nonCriticalExtensions35 */
static int hf_umts_rrc_pdu_def_pagingRecord2List = -1;  /* PagingRecord2List_r5 */
static int hf_umts_rrc_pdu_def_pagingCause = -1;  /* PagingCause */
static int hf_umts_rrc_pdu_def_pagingRecordTypeID = -1;  /* PagingRecordTypeID */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions22 = -1;  /* T_laterNonCriticalExtensions22 */
static int hf_umts_rrc_pdu_def_pagingType2_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions36 = -1;  /* T_nonCriticalExtensions36 */
static int hf_umts_rrc_pdu_def_r312 = -1;         /* T_r312 */
static int hf_umts_rrc_pdu_def_physicalChannelReconfiguration_r3 = -1;  /* PhysicalChannelReconfiguration_r3_IEs */
static int hf_umts_rrc_pdu_def_v3a0NonCriticalExtensions5 = -1;  /* T_v3a0NonCriticalExtensions5 */
static int hf_umts_rrc_pdu_def_physicalChannelReconfiguration_v3a0ext = -1;  /* PhysicalChannelReconfiguration_v3a0ext */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions23 = -1;  /* T_laterNonCriticalExtensions23 */
static int hf_umts_rrc_pdu_def_physicalChannelReconfiguration_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v4b0NonCriticalExtenstions = -1;  /* T_v4b0NonCriticalExtenstions */
static int hf_umts_rrc_pdu_def_physicalChannelReconfiguration_v4b0ext = -1;  /* PhysicalChannelReconfiguration_v4b0ext_IEs */
static int hf_umts_rrc_pdu_def_v590NonCriticalExtenstions2 = -1;  /* T_v590NonCriticalExtenstions2 */
static int hf_umts_rrc_pdu_def_physicalChannelReconfiguration_v590ext = -1;  /* PhysicalChannelReconfiguration_v590ext_IEs */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions12 = -1;  /* T_v6xyNonCriticalExtensions12 */
static int hf_umts_rrc_pdu_def_physicalChannelReconfiguration_v6xyext = -1;  /* PhysicalChannelReconfiguration_v6xyext_IEs */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions37 = -1;  /* T_nonCriticalExtensions37 */
static int hf_umts_rrc_pdu_def_later_than_r310 = -1;  /* T_later_than_r310 */
static int hf_umts_rrc_pdu_def_criticalExtensions25 = -1;  /* T_criticalExtensions25 */
static int hf_umts_rrc_pdu_def_r44 = -1;          /* T_r44 */
static int hf_umts_rrc_pdu_def_physicalChannelReconfiguration_r4 = -1;  /* PhysicalChannelReconfiguration_r4_IEs */
static int hf_umts_rrc_pdu_def_v4d0NonCriticalExtensions4 = -1;  /* T_v4d0NonCriticalExtensions4 */
static int hf_umts_rrc_pdu_def_physicalChannelReconfiguration_r4_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v590NonCriticalExtenstions3 = -1;  /* T_v590NonCriticalExtenstions3 */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions13 = -1;  /* T_v6xyNonCriticalExtensions13 */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions38 = -1;  /* T_nonCriticalExtensions38 */
static int hf_umts_rrc_pdu_def_criticalExtensions26 = -1;  /* T_criticalExtensions26 */
static int hf_umts_rrc_pdu_def_r54 = -1;          /* T_r54 */
static int hf_umts_rrc_pdu_def_physicalChannelReconfiguration_r5 = -1;  /* PhysicalChannelReconfiguration_r5_IEs */
static int hf_umts_rrc_pdu_def_physicalChannelReconfiguration_r5_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions14 = -1;  /* T_v6xyNonCriticalExtensions14 */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions39 = -1;  /* T_nonCriticalExtensions39 */
static int hf_umts_rrc_pdu_def_criticalExtensions27 = -1;  /* T_criticalExtensions27 */
static int hf_umts_rrc_pdu_def_r65 = -1;          /* T_r65 */
static int hf_umts_rrc_pdu_def_physicalChannelReconfiguration_r6 = -1;  /* PhysicalChannelReconfiguration_r6_IEs */
static int hf_umts_rrc_pdu_def_physicalChannelReconfiguration_r6_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions40 = -1;  /* T_nonCriticalExtensions40 */
static int hf_umts_rrc_pdu_def_criticalExtensions28 = -1;  /* T_criticalExtensions28 */
static int hf_umts_rrc_pdu_def_ul_ChannelRequirement4 = -1;  /* UL_ChannelRequirementWithCPCH_SetID */
static int hf_umts_rrc_pdu_def_modeSpecificInfo6 = -1;  /* T_modeSpecificInfo6 */
static int hf_umts_rrc_pdu_def_fdd12 = -1;        /* T_fdd12 */
static int hf_umts_rrc_pdu_def_ul_ChannelRequirement5 = -1;  /* UL_ChannelRequirementWithCPCH_SetID_r4 */
static int hf_umts_rrc_pdu_def_modeSpecificInfo7 = -1;  /* T_modeSpecificInfo7 */
static int hf_umts_rrc_pdu_def_fdd13 = -1;        /* T_fdd13 */
static int hf_umts_rrc_pdu_def_ul_ChannelRequirement6 = -1;  /* UL_ChannelRequirementWithCPCH_SetID_r5 */
static int hf_umts_rrc_pdu_def_modeSpecificInfo8 = -1;  /* T_modeSpecificInfo8 */
static int hf_umts_rrc_pdu_def_fdd14 = -1;        /* T_fdd14 */
static int hf_umts_rrc_pdu_def_delayRestrictionFlag = -1;  /* DelayRestrictionFlag */
static int hf_umts_rrc_pdu_def_postVerificationPeriod1 = -1;  /* T_postVerificationPeriod1 */
static int hf_umts_rrc_pdu_def_timingMaintainedSynchInd = -1;  /* T_timingMaintainedSynchInd */
static int hf_umts_rrc_pdu_def_ul_IntegProtActivationInfo = -1;  /* IntegrityProtActivationInfo */
static int hf_umts_rrc_pdu_def_ul_TimingAdvance = -1;  /* UL_TimingAdvance */
static int hf_umts_rrc_pdu_def_dummy_RB_ActivationTimeInfoList = -1;  /* RB_ActivationTimeInfoList */
static int hf_umts_rrc_pdu_def_ul_CounterSynchronisationInfo = -1;  /* UL_CounterSynchronisationInfo */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions24 = -1;  /* T_laterNonCriticalExtensions24 */
static int hf_umts_rrc_pdu_def_physicalChannelReconfigurationComplete_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions41 = -1;  /* T_nonCriticalExtensions41 */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions25 = -1;  /* T_laterNonCriticalExtensions25 */
static int hf_umts_rrc_pdu_def_physicalChannelReconfigurationFailure_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions42 = -1;  /* T_nonCriticalExtensions42 */
static int hf_umts_rrc_pdu_def_r313 = -1;         /* T_r313 */
static int hf_umts_rrc_pdu_def_physicalSharedChannelAllocation_r3 = -1;  /* PhysicalSharedChannelAllocation_r3_IEs */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions26 = -1;  /* T_laterNonCriticalExtensions26 */
static int hf_umts_rrc_pdu_def_physicalSharedChannelAllocation_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions43 = -1;  /* T_nonCriticalExtensions43 */
static int hf_umts_rrc_pdu_def_later_than_r311 = -1;  /* T_later_than_r311 */
static int hf_umts_rrc_pdu_def_dsch_RNTI = -1;    /* DSCH_RNTI */
static int hf_umts_rrc_pdu_def_criticalExtensions29 = -1;  /* T_criticalExtensions29 */
static int hf_umts_rrc_pdu_def_r45 = -1;          /* T_r45 */
static int hf_umts_rrc_pdu_def_physicalSharedChannelAllocation_r4 = -1;  /* PhysicalSharedChannelAllocation_r4_IEs */
static int hf_umts_rrc_pdu_def_v4d0NonCriticalExtensions5 = -1;  /* T_v4d0NonCriticalExtensions5 */
static int hf_umts_rrc_pdu_def_physicalSharedChannelAllocation_r4_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions15 = -1;  /* T_v6xyNonCriticalExtensions15 */
static int hf_umts_rrc_pdu_def_physicalSharedChannelAllocation_v6xyext = -1;  /* PhysicalSharedChannelAllocation_v6xyext_IEs */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions44 = -1;  /* T_nonCriticalExtensions44 */
static int hf_umts_rrc_pdu_def_criticalExtensions30 = -1;  /* T_criticalExtensions30 */
static int hf_umts_rrc_pdu_def_ul_TimingAdvance1 = -1;  /* UL_TimingAdvanceControl */
static int hf_umts_rrc_pdu_def_pusch_CapacityAllocationInfo = -1;  /* PUSCH_CapacityAllocationInfo */
static int hf_umts_rrc_pdu_def_pdsch_CapacityAllocationInfo = -1;  /* PDSCH_CapacityAllocationInfo */
static int hf_umts_rrc_pdu_def_confirmRequest = -1;  /* T_confirmRequest */
static int hf_umts_rrc_pdu_def_trafficVolumeReportRequest = -1;  /* INTEGER_0_255 */
static int hf_umts_rrc_pdu_def_iscpTimeslotList = -1;  /* TimeslotList */
static int hf_umts_rrc_pdu_def_requestPCCPCHRSCP = -1;  /* BOOLEAN */
static int hf_umts_rrc_pdu_def_ul_TimingAdvance2 = -1;  /* UL_TimingAdvanceControl_r4 */
static int hf_umts_rrc_pdu_def_pusch_CapacityAllocationInfo1 = -1;  /* PUSCH_CapacityAllocationInfo_r4 */
static int hf_umts_rrc_pdu_def_pdsch_CapacityAllocationInfo1 = -1;  /* PDSCH_CapacityAllocationInfo_r4 */
static int hf_umts_rrc_pdu_def_confirmRequest1 = -1;  /* T_confirmRequest1 */
static int hf_umts_rrc_pdu_def_iscpTimeslotList1 = -1;  /* TimeslotList_r4 */
static int hf_umts_rrc_pdu_def_trafficVolume = -1;  /* TrafficVolumeMeasuredResultsList */
static int hf_umts_rrc_pdu_def_timeslotListWithISCP = -1;  /* TimeslotListWithISCP */
static int hf_umts_rrc_pdu_def_primaryCCPCH_RSCP = -1;  /* PrimaryCCPCH_RSCP */
static int hf_umts_rrc_pdu_def_allocationConfirmation = -1;  /* T_allocationConfirmation */
static int hf_umts_rrc_pdu_def_pdschConfirmation = -1;  /* PDSCH_Identity */
static int hf_umts_rrc_pdu_def_puschConfirmation = -1;  /* PUSCH_Identity */
static int hf_umts_rrc_pdu_def_protocolErrorIndicator = -1;  /* ProtocolErrorIndicatorWithMoreInfo */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions27 = -1;  /* T_laterNonCriticalExtensions27 */
static int hf_umts_rrc_pdu_def_puschCapacityRequest_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v590NonCriticalExtensions13 = -1;  /* T_v590NonCriticalExtensions13 */
static int hf_umts_rrc_pdu_def_puschCapacityRequest_v590ext = -1;  /* PUSCHCapacityRequest_v590ext */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions45 = -1;  /* T_nonCriticalExtensions45 */
static int hf_umts_rrc_pdu_def_primaryCCPCH_RSCP_delta = -1;  /* DeltaRSCP */
static int hf_umts_rrc_pdu_def_r314 = -1;         /* T_r314 */
static int hf_umts_rrc_pdu_def_radioBearerReconfiguration_r3 = -1;  /* RadioBearerReconfiguration_r3_IEs */
static int hf_umts_rrc_pdu_def_v3aoNonCriticalExtensions = -1;  /* T_v3aoNonCriticalExtensions */
static int hf_umts_rrc_pdu_def_radioBearerReconfiguration_v3a0ext = -1;  /* RadioBearerReconfiguration_v3a0ext */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions28 = -1;  /* T_laterNonCriticalExtensions28 */
static int hf_umts_rrc_pdu_def_radioBearerReconfiguration_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions7 = -1;  /* T_v4b0NonCriticalExtensions7 */
static int hf_umts_rrc_pdu_def_radioBearerReconfiguration_v4b0ext = -1;  /* RadioBearerReconfiguration_v4b0ext_IEs */
static int hf_umts_rrc_pdu_def_v590NonCriticalExtensions14 = -1;  /* T_v590NonCriticalExtensions14 */
static int hf_umts_rrc_pdu_def_radioBearerReconfiguration_v590ext = -1;  /* RadioBearerReconfiguration_v590ext_IEs */
static int hf_umts_rrc_pdu_def_v5d0NonCriticalExtenstions6 = -1;  /* T_v5d0NonCriticalExtenstions6 */
static int hf_umts_rrc_pdu_def_radioBearerReconfiguration_v5d0ext = -1;  /* RadioBearerReconfiguration_v5d0ext_IEs */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions16 = -1;  /* T_v6xyNonCriticalExtensions16 */
static int hf_umts_rrc_pdu_def_radioBearerReconfiguration_v6xyext = -1;  /* RadioBearerReconfiguration_v6xyext_IEs */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions46 = -1;  /* T_nonCriticalExtensions46 */
static int hf_umts_rrc_pdu_def_later_than_r312 = -1;  /* T_later_than_r312 */
static int hf_umts_rrc_pdu_def_criticalExtensions31 = -1;  /* T_criticalExtensions31 */
static int hf_umts_rrc_pdu_def_r46 = -1;          /* T_r46 */
static int hf_umts_rrc_pdu_def_radioBearerReconfiguration_r4 = -1;  /* RadioBearerReconfiguration_r4_IEs */
static int hf_umts_rrc_pdu_def_v4d0NonCriticalExtensions6 = -1;  /* T_v4d0NonCriticalExtensions6 */
static int hf_umts_rrc_pdu_def_radioBearerReconfiguration_r4_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v590NonCriticalExtensions15 = -1;  /* T_v590NonCriticalExtensions15 */
static int hf_umts_rrc_pdu_def_v5d0NonCriticalExtenstions7 = -1;  /* T_v5d0NonCriticalExtenstions7 */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions17 = -1;  /* T_v6xyNonCriticalExtensions17 */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions47 = -1;  /* T_nonCriticalExtensions47 */
static int hf_umts_rrc_pdu_def_criticalExtensions32 = -1;  /* T_criticalExtensions32 */
static int hf_umts_rrc_pdu_def_r55 = -1;          /* T_r55 */
static int hf_umts_rrc_pdu_def_radioBearerReconfiguration_r5 = -1;  /* RadioBearerReconfiguration_r5_IEs */
static int hf_umts_rrc_pdu_def_radioBearerReconfiguration_r5_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v5d0NonCriticalExtenstions8 = -1;  /* T_v5d0NonCriticalExtenstions8 */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions18 = -1;  /* T_v6xyNonCriticalExtensions18 */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions48 = -1;  /* T_nonCriticalExtensions48 */
static int hf_umts_rrc_pdu_def_criticalExtensions33 = -1;  /* T_criticalExtensions33 */
static int hf_umts_rrc_pdu_def_r66 = -1;          /* T_r66 */
static int hf_umts_rrc_pdu_def_radioBearerReconfiguration_r6 = -1;  /* RadioBearerReconfiguration_r6_IEs */
static int hf_umts_rrc_pdu_def_radioBearerReconfiguration_r6_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions49 = -1;  /* T_nonCriticalExtensions49 */
static int hf_umts_rrc_pdu_def_criticalExtensions34 = -1;  /* T_criticalExtensions34 */
static int hf_umts_rrc_pdu_def_rab_InformationReconfigList = -1;  /* RAB_InformationReconfigList */
static int hf_umts_rrc_pdu_def_dummy1 = -1;       /* T_dummy1 */
static int hf_umts_rrc_pdu_def_fdd15 = -1;        /* T_fdd15 */
static int hf_umts_rrc_pdu_def_dummy11 = -1;      /* CPCH_SetID */
static int hf_umts_rrc_pdu_def_dummy2 = -1;       /* DRAC_StaticInformationList */
static int hf_umts_rrc_pdu_def_dl_AddReconfTransChInfoList3 = -1;  /* DL_AddReconfTransChInfo2List */
static int hf_umts_rrc_pdu_def_modeSpecificPhysChInfo3 = -1;  /* T_modeSpecificPhysChInfo3 */
static int hf_umts_rrc_pdu_def_fdd16 = -1;        /* T_fdd16 */
static int hf_umts_rrc_pdu_def_radioBearerReconfiguration_r4_IEs_dummy = -1;  /* T_radioBearerReconfiguration_r4_IEs_dummy */
static int hf_umts_rrc_pdu_def_fdd17 = -1;        /* T_fdd17 */
static int hf_umts_rrc_pdu_def_dummy1_CPCH_SetID = -1;  /* CPCH_SetID */
static int hf_umts_rrc_pdu_def_modeSpecificPhysChInfo4 = -1;  /* T_modeSpecificPhysChInfo4 */
static int hf_umts_rrc_pdu_def_fdd18 = -1;        /* T_fdd18 */
static int hf_umts_rrc_pdu_def_specificationMode4 = -1;  /* T_specificationMode4 */
static int hf_umts_rrc_pdu_def_complete4 = -1;    /* T_complete4 */
static int hf_umts_rrc_pdu_def_rb_PDCPContextRelocationList = -1;  /* RB_PDCPContextRelocationList */
static int hf_umts_rrc_pdu_def_dummy_1a = -1;     /* T_dummy_1a */
static int hf_umts_rrc_pdu_def_fdd19 = -1;        /* T_fdd19 */
static int hf_umts_rrc_pdu_def_preconfiguration3 = -1;  /* T_preconfiguration3 */
static int hf_umts_rrc_pdu_def_preConfigMode3 = -1;  /* T_preConfigMode3 */
static int hf_umts_rrc_pdu_def_defaultConfig3 = -1;  /* T_defaultConfig3 */
static int hf_umts_rrc_pdu_def_modeSpecificPhysChInfo5 = -1;  /* T_modeSpecificPhysChInfo5 */
static int hf_umts_rrc_pdu_def_fdd20 = -1;        /* T_fdd20 */
static int hf_umts_rrc_pdu_def_specificationMode5 = -1;  /* T_specificationMode5 */
static int hf_umts_rrc_pdu_def_complete5 = -1;    /* T_complete5 */
static int hf_umts_rrc_pdu_def_preconfiguration4 = -1;  /* T_preconfiguration4 */
static int hf_umts_rrc_pdu_def_preConfigMode4 = -1;  /* T_preConfigMode4 */
static int hf_umts_rrc_pdu_def_defaultConfig4 = -1;  /* T_defaultConfig4 */
static int hf_umts_rrc_pdu_def_postVerificationPeriod2 = -1;  /* T_postVerificationPeriod2 */
static int hf_umts_rrc_pdu_def_timingMaintainedSynchInd1 = -1;  /* T_timingMaintainedSynchInd1 */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions29 = -1;  /* T_laterNonCriticalExtensions29 */
static int hf_umts_rrc_pdu_def_radioBearerReconfigurationComplete_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions50 = -1;  /* T_nonCriticalExtensions50 */
static int hf_umts_rrc_pdu_def_potentiallySuccesfulBearerList = -1;  /* RB_IdentityList */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions30 = -1;  /* T_laterNonCriticalExtensions30 */
static int hf_umts_rrc_pdu_def_radioBearerReconfigurationFailure_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions51 = -1;  /* T_nonCriticalExtensions51 */
static int hf_umts_rrc_pdu_def_r315 = -1;         /* T_r315 */
static int hf_umts_rrc_pdu_def_radioBearerRelease_r3 = -1;  /* RadioBearerRelease_r3_IEs */
static int hf_umts_rrc_pdu_def_v3a0NonCriticalExtensions6 = -1;  /* T_v3a0NonCriticalExtensions6 */
static int hf_umts_rrc_pdu_def_radioBearerRelease_v3a0ext = -1;  /* RadioBearerRelease_v3a0ext */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions31 = -1;  /* T_laterNonCriticalExtensions31 */
static int hf_umts_rrc_pdu_def_radioBearerRelease_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions8 = -1;  /* T_v4b0NonCriticalExtensions8 */
static int hf_umts_rrc_pdu_def_radioBearerRelease_v4b0ext = -1;  /* RadioBearerRelease_v4b0ext_IEs */
static int hf_umts_rrc_pdu_def_v590NonCriticalExtensions16 = -1;  /* T_v590NonCriticalExtensions16 */
static int hf_umts_rrc_pdu_def_radioBearerRelease_v590ext = -1;  /* RadioBearerRelease_v590ext_IEs */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions19 = -1;  /* T_v6xyNonCriticalExtensions19 */
static int hf_umts_rrc_pdu_def_radioBearerRelease_v6xyext = -1;  /* RadioBearerRelease_v6xyext_IEs */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions52 = -1;  /* T_nonCriticalExtensions52 */
static int hf_umts_rrc_pdu_def_later_than_r313 = -1;  /* T_later_than_r313 */
static int hf_umts_rrc_pdu_def_criticalExtensions35 = -1;  /* T_criticalExtensions35 */
static int hf_umts_rrc_pdu_def_r47 = -1;          /* T_r47 */
static int hf_umts_rrc_pdu_def_radioBearerRelease_r4 = -1;  /* RadioBearerRelease_r4_IEs */
static int hf_umts_rrc_pdu_def_v4d0NonCriticalExtensions7 = -1;  /* T_v4d0NonCriticalExtensions7 */
static int hf_umts_rrc_pdu_def_radioBearerRelease_r4_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v590NonCriticalExtensions17 = -1;  /* T_v590NonCriticalExtensions17 */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions20 = -1;  /* T_v6xyNonCriticalExtensions20 */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions53 = -1;  /* T_nonCriticalExtensions53 */
static int hf_umts_rrc_pdu_def_criticalExtensions36 = -1;  /* T_criticalExtensions36 */
static int hf_umts_rrc_pdu_def_r56 = -1;          /* T_r56 */
static int hf_umts_rrc_pdu_def_radioBearerRelease_r5 = -1;  /* RadioBearerRelease_r5_IEs */
static int hf_umts_rrc_pdu_def_radioBearerRelease_r5_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions21 = -1;  /* T_v6xyNonCriticalExtensions21 */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions54 = -1;  /* T_nonCriticalExtensions54 */
static int hf_umts_rrc_pdu_def_criticalExtensions37 = -1;  /* T_criticalExtensions37 */
static int hf_umts_rrc_pdu_def_r67 = -1;          /* T_r67 */
static int hf_umts_rrc_pdu_def_radioBearerRelease_r6 = -1;  /* RadioBearerRelease_r6_IEs */
static int hf_umts_rrc_pdu_def_radioBearerRelease_r6_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions55 = -1;  /* T_nonCriticalExtensions55 */
static int hf_umts_rrc_pdu_def_criticalExtensions38 = -1;  /* T_criticalExtensions38 */
static int hf_umts_rrc_pdu_def_signallingConnectionRelIndication = -1;  /* CN_DomainIdentity */
static int hf_umts_rrc_pdu_def_radioBearerRelease_r3_IEs_dummy = -1;  /* T_radioBearerRelease_r3_IEs_dummy */
static int hf_umts_rrc_pdu_def_fdd21 = -1;        /* T_fdd21 */
static int hf_umts_rrc_pdu_def_modeSpecificPhysChInfo6 = -1;  /* T_modeSpecificPhysChInfo6 */
static int hf_umts_rrc_pdu_def_fdd22 = -1;        /* T_fdd22 */
static int hf_umts_rrc_pdu_def_radioBearerRelease_r4_IEs_dummy = -1;  /* T_radioBearerRelease_r4_IEs_dummy */
static int hf_umts_rrc_pdu_def_fdd23 = -1;        /* T_fdd23 */
static int hf_umts_rrc_pdu_def_modeSpecificPhysChInfo7 = -1;  /* T_modeSpecificPhysChInfo7 */
static int hf_umts_rrc_pdu_def_fdd24 = -1;        /* T_fdd24 */
static int hf_umts_rrc_pdu_def_radioBearerRelease_r5_IEs_dummy = -1;  /* T_radioBearerRelease_r5_IEs_dummy */
static int hf_umts_rrc_pdu_def_fdd25 = -1;        /* T_fdd25 */
static int hf_umts_rrc_pdu_def_modeSpecificPhysChInfo8 = -1;  /* T_modeSpecificPhysChInfo8 */
static int hf_umts_rrc_pdu_def_fdd26 = -1;        /* T_fdd26 */
static int hf_umts_rrc_pdu_def_postVerificationPeriod3 = -1;  /* T_postVerificationPeriod3 */
static int hf_umts_rrc_pdu_def_timingMaintainedSynchInd2 = -1;  /* T_timingMaintainedSynchInd2 */
static int hf_umts_rrc_pdu_def_mbms_RB_ListReleasedToChangeTransferMode = -1;  /* RB_InformationReleaseList */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions32 = -1;  /* T_laterNonCriticalExtensions32 */
static int hf_umts_rrc_pdu_def_radioBearerReleaseComplete_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions56 = -1;  /* T_nonCriticalExtensions56 */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions33 = -1;  /* T_laterNonCriticalExtensions33 */
static int hf_umts_rrc_pdu_def_radioBearerReleaseFailure_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions57 = -1;  /* T_nonCriticalExtensions57 */
static int hf_umts_rrc_pdu_def_r316 = -1;         /* T_r316 */
static int hf_umts_rrc_pdu_def_radioBearerSetup_r3 = -1;  /* RadioBearerSetup_r3_IEs */
static int hf_umts_rrc_pdu_def_v3a0NonCriticalExtensions7 = -1;  /* T_v3a0NonCriticalExtensions7 */
static int hf_umts_rrc_pdu_def_radioBearerSetup_v3a0ext = -1;  /* RadioBearerSetup_v3a0ext */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions34 = -1;  /* T_laterNonCriticalExtensions34 */
static int hf_umts_rrc_pdu_def_radioBearerSetup_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions9 = -1;  /* T_v4b0NonCriticalExtensions9 */
static int hf_umts_rrc_pdu_def_radioBearerSetup_v4b0ext = -1;  /* RadioBearerSetup_v4b0ext_IEs */
static int hf_umts_rrc_pdu_def_v590NonCriticalExtensions18 = -1;  /* T_v590NonCriticalExtensions18 */
static int hf_umts_rrc_pdu_def_radioBearerSetup_v590ext = -1;  /* RadioBearerSetup_v590ext_IEs */
static int hf_umts_rrc_pdu_def_v5d0NonCriticalExtenstions9 = -1;  /* T_v5d0NonCriticalExtenstions9 */
static int hf_umts_rrc_pdu_def_radioBearerSetup_v5d0ext = -1;  /* RadioBearerSetup_v5d0ext_IEs */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions22 = -1;  /* T_v6xyNonCriticalExtensions22 */
static int hf_umts_rrc_pdu_def_radioBearerSetup_v6xyext = -1;  /* RadioBearerSetup_v6xyext_IEs */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions58 = -1;  /* T_nonCriticalExtensions58 */
static int hf_umts_rrc_pdu_def_later_than_r314 = -1;  /* T_later_than_r314 */
static int hf_umts_rrc_pdu_def_criticalExtensions39 = -1;  /* T_criticalExtensions39 */
static int hf_umts_rrc_pdu_def_r48 = -1;          /* T_r48 */
static int hf_umts_rrc_pdu_def_radioBearerSetup_r4 = -1;  /* RadioBearerSetup_r4_IEs */
static int hf_umts_rrc_pdu_def_v4d0NonCriticalExtensions8 = -1;  /* T_v4d0NonCriticalExtensions8 */
static int hf_umts_rrc_pdu_def_radioBearerSetup_r4_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v590NonCriticalExtensions19 = -1;  /* T_v590NonCriticalExtensions19 */
static int hf_umts_rrc_pdu_def_v5d0NonCriticalExtenstions10 = -1;  /* T_v5d0NonCriticalExtenstions10 */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions23 = -1;  /* T_v6xyNonCriticalExtensions23 */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions59 = -1;  /* T_nonCriticalExtensions59 */
static int hf_umts_rrc_pdu_def_criticalExtensions40 = -1;  /* T_criticalExtensions40 */
static int hf_umts_rrc_pdu_def_r57 = -1;          /* T_r57 */
static int hf_umts_rrc_pdu_def_radioBearerSetup_r5 = -1;  /* RadioBearerSetup_r5_IEs */
static int hf_umts_rrc_pdu_def_radioBearerSetup_r5_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v5d0NonCriticalExtenstions11 = -1;  /* T_v5d0NonCriticalExtenstions11 */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions24 = -1;  /* T_v6xyNonCriticalExtensions24 */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions60 = -1;  /* T_nonCriticalExtensions60 */
static int hf_umts_rrc_pdu_def_criticalExtensions41 = -1;  /* T_criticalExtensions41 */
static int hf_umts_rrc_pdu_def_r68 = -1;          /* T_r68 */
static int hf_umts_rrc_pdu_def_radioBearerSetup_r6 = -1;  /* RadioBearerSetup_r6_IEs */
static int hf_umts_rrc_pdu_def_radioBearerSetup_r6_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions61 = -1;  /* T_nonCriticalExtensions61 */
static int hf_umts_rrc_pdu_def_criticalExtensions42 = -1;  /* T_criticalExtensions42 */
static int hf_umts_rrc_pdu_def_radioBearerSetup_r3_IEs_dummy = -1;  /* T_radioBearerSetup_r3_IEs_dummy */
static int hf_umts_rrc_pdu_def_fdd27 = -1;        /* T_fdd27 */
static int hf_umts_rrc_pdu_def_modeSpecificPhysChInfo9 = -1;  /* T_modeSpecificPhysChInfo9 */
static int hf_umts_rrc_pdu_def_fdd28 = -1;        /* T_fdd28 */
static int hf_umts_rrc_pdu_def_radioBearerSetup_r4_IEs_dummy = -1;  /* T_radioBearerSetup_r4_IEs_dummy */
static int hf_umts_rrc_pdu_def_fdd29 = -1;        /* T_fdd29 */
static int hf_umts_rrc_pdu_def_modeSpecificPhysChInfo10 = -1;  /* T_modeSpecificPhysChInfo10 */
static int hf_umts_rrc_pdu_def_fdd30 = -1;        /* T_fdd30 */
static int hf_umts_rrc_pdu_def_radioBearerSetup_r5_IEs_dummy = -1;  /* T_radioBearerSetup_r5_IEs_dummy */
static int hf_umts_rrc_pdu_def_fdd31 = -1;        /* T_fdd31 */
static int hf_umts_rrc_pdu_def_modeSpecificPhysChInfo11 = -1;  /* T_modeSpecificPhysChInfo11 */
static int hf_umts_rrc_pdu_def_fdd32 = -1;        /* T_fdd32 */
static int hf_umts_rrc_pdu_def_postVerificationPeriod4 = -1;  /* T_postVerificationPeriod4 */
static int hf_umts_rrc_pdu_def_timingMaintainedSynchInd3 = -1;  /* T_timingMaintainedSynchInd3 */
static int hf_umts_rrc_pdu_def_rab_InformationSetupList4 = -1;  /* RAB_InformationSetupList_r6_ext */
static int hf_umts_rrc_pdu_def_specificationMode6 = -1;  /* T_specificationMode6 */
static int hf_umts_rrc_pdu_def_complete6 = -1;    /* T_complete6 */
static int hf_umts_rrc_pdu_def_preconfiguration5 = -1;  /* T_preconfiguration5 */
static int hf_umts_rrc_pdu_def_rab_Info1 = -1;    /* RAB_Info */
static int hf_umts_rrc_pdu_def_rb_InformationChangedList = -1;  /* RB_InformationChangedList_r6 */
static int hf_umts_rrc_pdu_def_powerOffsetInfoShort = -1;  /* PowerOffsetInfoShort */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions35 = -1;  /* T_laterNonCriticalExtensions35 */
static int hf_umts_rrc_pdu_def_radioBearerSetupComplete_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions62 = -1;  /* T_nonCriticalExtensions62 */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions36 = -1;  /* T_laterNonCriticalExtensions36 */
static int hf_umts_rrc_pdu_def_radioBearerSetupFailure_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions63 = -1;  /* T_nonCriticalExtensions63 */
static int hf_umts_rrc_pdu_def_r317 = -1;         /* T_r317 */
static int hf_umts_rrc_pdu_def_rrcConnectionReject_r3 = -1;  /* RRCConnectionReject_r3_IEs */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions37 = -1;  /* T_laterNonCriticalExtensions37 */
static int hf_umts_rrc_pdu_def_rrcConnectionReject_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions25 = -1;  /* T_v6xyNonCriticalExtensions25 */
static int hf_umts_rrc_pdu_def_rrcConnectionReject_v6xyext = -1;  /* RRCConnectionReject_v6xyext_IEs */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions64 = -1;  /* T_nonCriticalExtensions64 */
static int hf_umts_rrc_pdu_def_later_than_r315 = -1;  /* T_later_than_r315 */
static int hf_umts_rrc_pdu_def_initialUE_Identity = -1;  /* InitialUE_Identity */
static int hf_umts_rrc_pdu_def_criticalExtensions43 = -1;  /* T_criticalExtensions43 */
static int hf_umts_rrc_pdu_def_rejectionCause = -1;  /* RejectionCause */
static int hf_umts_rrc_pdu_def_waitTime = -1;     /* WaitTime */
static int hf_umts_rrc_pdu_def_redirectionInfo = -1;  /* RedirectionInfo */
static int hf_umts_rrc_pdu_def_redirectionInfo_v6xyext = -1;  /* GSM_TargetCellInfoList */
static int hf_umts_rrc_pdu_def_r318 = -1;         /* T_r318 */
static int hf_umts_rrc_pdu_def_rrcConnectionRelease_r3 = -1;  /* RRCConnectionRelease_r3_IEs */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions38 = -1;  /* T_laterNonCriticalExtensions38 */
static int hf_umts_rrc_pdu_def_rrcConnectionRelease_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions26 = -1;  /* T_v6xyNonCriticalExtensions26 */
static int hf_umts_rrc_pdu_def_rrcConnectionRelease_v6xyext = -1;  /* RRCConnectionRelease_v6xyext_IEs */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions65 = -1;  /* T_nonCriticalExtensions65 */
static int hf_umts_rrc_pdu_def_later_than_r316 = -1;  /* T_later_than_r316 */
static int hf_umts_rrc_pdu_def_criticalExtensions44 = -1;  /* T_criticalExtensions44 */
static int hf_umts_rrc_pdu_def_r49 = -1;          /* T_r49 */
static int hf_umts_rrc_pdu_def_rrcConnectionRelease_r4 = -1;  /* RRCConnectionRelease_r4_IEs */
static int hf_umts_rrc_pdu_def_v4d0NonCriticalExtensions9 = -1;  /* T_v4d0NonCriticalExtensions9 */
static int hf_umts_rrc_pdu_def_rrcConnectionRelease_r4_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions27 = -1;  /* T_v6xyNonCriticalExtensions27 */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions66 = -1;  /* T_nonCriticalExtensions66 */
static int hf_umts_rrc_pdu_def_criticalExtensions45 = -1;  /* T_criticalExtensions45 */
static int hf_umts_rrc_pdu_def_n_308 = -1;        /* N_308 */
static int hf_umts_rrc_pdu_def_releaseCause = -1;  /* ReleaseCause */
static int hf_umts_rrc_pdu_def_rplmn_information = -1;  /* Rplmn_Information */
static int hf_umts_rrc_pdu_def_rplmn_information1 = -1;  /* Rplmn_Information_r4 */
static int hf_umts_rrc_pdu_def_redirectionInfo_v6xyext1 = -1;  /* RedirectionInfo_r6 */
static int hf_umts_rrc_pdu_def_r319 = -1;         /* T_r319 */
static int hf_umts_rrc_pdu_def_rrcConnectionRelease_CCCH_r3 = -1;  /* RRCConnectionRelease_CCCH_r3_IEs */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions39 = -1;  /* T_laterNonCriticalExtensions39 */
static int hf_umts_rrc_pdu_def_rrcConnectionRelease_CCCH_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions67 = -1;  /* T_nonCriticalExtensions67 */
static int hf_umts_rrc_pdu_def_later_than_r317 = -1;  /* T_later_than_r317 */
static int hf_umts_rrc_pdu_def_criticalExtensions46 = -1;  /* T_criticalExtensions46 */
static int hf_umts_rrc_pdu_def_r410 = -1;         /* T_r410 */
static int hf_umts_rrc_pdu_def_rrcConnectionRelease_CCCH_r4 = -1;  /* RRCConnectionRelease_CCCH_r4_IEs */
static int hf_umts_rrc_pdu_def_v4d0NonCriticalExtensions10 = -1;  /* T_v4d0NonCriticalExtensions10 */
static int hf_umts_rrc_pdu_def_rrcConnectionRelease_CCCH_r4_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions68 = -1;  /* T_nonCriticalExtensions68 */
static int hf_umts_rrc_pdu_def_criticalExtensions47 = -1;  /* T_criticalExtensions47 */
static int hf_umts_rrc_pdu_def_groupIdentity = -1;  /* SEQUENCE_SIZE_1_maxURNTIGroup_OF_GroupReleaseInformation */
static int hf_umts_rrc_pdu_def_groupIdentity_item = -1;  /* GroupReleaseInformation */
static int hf_umts_rrc_pdu_def_criticalExtensions48 = -1;  /* T_criticalExtensions48 */
static int hf_umts_rrc_pdu_def_r58 = -1;          /* T_r58 */
static int hf_umts_rrc_pdu_def_rrcConnectionRelease_CCCH_r5 = -1;  /* RRCConnectionRelease_CCCH_r5_IEs */
static int hf_umts_rrc_pdu_def_rrcConnectionRelease_CCCH_r5_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions69 = -1;  /* T_nonCriticalExtensions69 */
static int hf_umts_rrc_pdu_def_criticalExtensions49 = -1;  /* T_criticalExtensions49 */
static int hf_umts_rrc_pdu_def_rrcConnectionRelease = -1;  /* RRCConnectionRelease_r3_IEs */
static int hf_umts_rrc_pdu_def_rrcConnectionRelease1 = -1;  /* RRCConnectionRelease_r4_IEs */
static int hf_umts_rrc_pdu_def_errorIndication = -1;  /* FailureCauseWithProtErr */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions40 = -1;  /* T_laterNonCriticalExtensions40 */
static int hf_umts_rrc_pdu_def_rrcConnectionReleaseComplete_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions70 = -1;  /* T_nonCriticalExtensions70 */
static int hf_umts_rrc_pdu_def_protocolErrorIndicator1 = -1;  /* ProtocolErrorIndicator */
static int hf_umts_rrc_pdu_def_v3d0NonCriticalExtensions = -1;  /* T_v3d0NonCriticalExtensions */
static int hf_umts_rrc_pdu_def_rRCConnectionRequest_v3d0ext = -1;  /* RRCConnectionRequest_v3d0ext_IEs */
static int hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions10 = -1;  /* T_v4b0NonCriticalExtensions10 */
static int hf_umts_rrc_pdu_def_rrcConnectionRequest_v4b0ext = -1;  /* RRCConnectionRequest_v4b0ext_IEs */
static int hf_umts_rrc_pdu_def_v590NonCriticalExtensions20 = -1;  /* T_v590NonCriticalExtensions20 */
static int hf_umts_rrc_pdu_def_rrcConnectionRequest_v590ext = -1;  /* RRCConnectionRequest_v590ext_IEs */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions28 = -1;  /* T_v6xyNonCriticalExtensions28 */
static int hf_umts_rrc_pdu_def_rrcConnectionRequest_v6xyext = -1;  /* RRCConnectionRequest_v6xyext_IEs */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions71 = -1;  /* T_nonCriticalExtensions71 */
static int hf_umts_rrc_pdu_def_uESpecificBehaviourInformation1idle = -1;  /* UESpecificBehaviourInformation1idle */
static int hf_umts_rrc_pdu_def_predefinedConfigStatusInfo = -1;  /* BOOLEAN */
static int hf_umts_rrc_pdu_def_ueCapabilityIndication = -1;  /* T_ueCapabilityIndication */
static int hf_umts_rrc_pdu_def_r320 = -1;         /* T_r320 */
static int hf_umts_rrc_pdu_def_rrcConnectionSetup_r3 = -1;  /* RRCConnectionSetup_r3_IEs */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions41 = -1;  /* T_laterNonCriticalExtensions41 */
static int hf_umts_rrc_pdu_def_rrcConnectionSetup_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions11 = -1;  /* T_v4b0NonCriticalExtensions11 */
static int hf_umts_rrc_pdu_def_rrcConnectionSetup_v4b0ext = -1;  /* RRCConnectionSetup_v4b0ext_IEs */
static int hf_umts_rrc_pdu_def_v590NonCriticalExtensions21 = -1;  /* T_v590NonCriticalExtensions21 */
static int hf_umts_rrc_pdu_def_rrcConnectionSetup_v590ext = -1;  /* RRCConnectionSetup_v590ext_IEs */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions72 = -1;  /* T_nonCriticalExtensions72 */
static int hf_umts_rrc_pdu_def_later_than_r318 = -1;  /* T_later_than_r318 */
static int hf_umts_rrc_pdu_def_criticalExtensions50 = -1;  /* T_criticalExtensions50 */
static int hf_umts_rrc_pdu_def_r411 = -1;         /* T_r411 */
static int hf_umts_rrc_pdu_def_rrcConnectionSetup_r4 = -1;  /* RRCConnectionSetup_r4_IEs */
static int hf_umts_rrc_pdu_def_v4d0NonCriticalExtensions11 = -1;  /* T_v4d0NonCriticalExtensions11 */
static int hf_umts_rrc_pdu_def_rrcConnectionSetup_r4_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v590NonCriticalExtensions22 = -1;  /* T_v590NonCriticalExtensions22 */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions29 = -1;  /* T_v6xyNonCriticalExtensions29 */
static int hf_umts_rrc_pdu_def_rrcConnectionSetup_v6xyext = -1;  /* RRCConnectionSetup_v6xyext_IEs */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions73 = -1;  /* T_nonCriticalExtensions73 */
static int hf_umts_rrc_pdu_def_criticalExtensions51 = -1;  /* T_criticalExtensions51 */
static int hf_umts_rrc_pdu_def_r59 = -1;          /* T_r59 */
static int hf_umts_rrc_pdu_def_rrcConnectionSetup_r5 = -1;  /* RRCConnectionSetup_r5_IEs */
static int hf_umts_rrc_pdu_def_rrcConnectionSetup_r5_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions30 = -1;  /* T_v6xyNonCriticalExtensions30 */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions74 = -1;  /* T_nonCriticalExtensions74 */
static int hf_umts_rrc_pdu_def_criticalExtensions52 = -1;  /* T_criticalExtensions52 */
static int hf_umts_rrc_pdu_def_r69 = -1;          /* T_r69 */
static int hf_umts_rrc_pdu_def_rrcConnectionSetup_r6 = -1;  /* RRCConnectionSetup_r6_IEs */
static int hf_umts_rrc_pdu_def_rrcConnectionSetup_r6_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions75 = -1;  /* T_nonCriticalExtensions75 */
static int hf_umts_rrc_pdu_def_criticalExtensions53 = -1;  /* T_criticalExtensions53 */
static int hf_umts_rrc_pdu_def_new_c_RNTI = -1;   /* C_RNTI */
static int hf_umts_rrc_pdu_def_capabilityUpdateRequirement = -1;  /* CapabilityUpdateRequirement */
static int hf_umts_rrc_pdu_def_srb_InformationSetupList3 = -1;  /* SRB_InformationSetupList2 */
static int hf_umts_rrc_pdu_def_capabilityUpdateRequirement_r4_ext = -1;  /* CapabilityUpdateRequirement_r4_ext */
static int hf_umts_rrc_pdu_def_systemSpecificCapUpdateReq = -1;  /* SystemSpecificCapUpdateReq_v590ext */
static int hf_umts_rrc_pdu_def_capabilityUpdateRequirement1 = -1;  /* CapabilityUpdateRequirement_r4 */
static int hf_umts_rrc_pdu_def_capabilityUpdateRequirement2 = -1;  /* CapabilityUpdateRequirement_r5 */
static int hf_umts_rrc_pdu_def_specificationMode7 = -1;  /* T_specificationMode7 */
static int hf_umts_rrc_pdu_def_complete7 = -1;    /* T_complete7 */
static int hf_umts_rrc_pdu_def_preconfiguration6 = -1;  /* T_preconfiguration6 */
static int hf_umts_rrc_pdu_def_preConfigMode5 = -1;  /* T_preConfigMode5 */
static int hf_umts_rrc_pdu_def_defaultConfig5 = -1;  /* T_defaultConfig5 */
static int hf_umts_rrc_pdu_def_dl_InformationPerRL_List5 = -1;  /* DL_InformationPerRL_List_r5bis */
static int hf_umts_rrc_pdu_def_postVerificationPeriod5 = -1;  /* T_postVerificationPeriod5 */
static int hf_umts_rrc_pdu_def_specificationMode8 = -1;  /* T_specificationMode8 */
static int hf_umts_rrc_pdu_def_complete8 = -1;    /* T_complete8 */
static int hf_umts_rrc_pdu_def_preconfiguration7 = -1;  /* T_preconfiguration7 */
static int hf_umts_rrc_pdu_def_preConfigMode6 = -1;  /* T_preConfigMode6 */
static int hf_umts_rrc_pdu_def_defaultConfig6 = -1;  /* T_defaultConfig6 */
static int hf_umts_rrc_pdu_def_ue_RadioAccessCapability = -1;  /* UE_RadioAccessCapability */
static int hf_umts_rrc_pdu_def_ue_RATSpecificCapability = -1;  /* InterRAT_UE_RadioAccessCapabilityList */
static int hf_umts_rrc_pdu_def_v370NonCriticalExtensions = -1;  /* T_v370NonCriticalExtensions */
static int hf_umts_rrc_pdu_def_rrcConnectionSetupComplete_v370ext = -1;  /* RRCConnectionSetupComplete_v370ext */
static int hf_umts_rrc_pdu_def_v380NonCriticalExtensions = -1;  /* T_v380NonCriticalExtensions */
static int hf_umts_rrc_pdu_def_rrcConnectionSetupComplete_v380ext = -1;  /* RRCConnectionSetupComplete_v380ext_IEs */
static int hf_umts_rrc_pdu_def_v3a0NonCriticalExtensions8 = -1;  /* T_v3a0NonCriticalExtensions8 */
static int hf_umts_rrc_pdu_def_rrcConnectionSetupComplete_v3a0ext = -1;  /* RRCConnectionSetupComplete_v3a0ext_IEs */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions42 = -1;  /* T_laterNonCriticalExtensions42 */
static int hf_umts_rrc_pdu_def_rrcConnectionSetupComplete_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v3g0NonCriticalExtensions1 = -1;  /* T_v3g0NonCriticalExtensions1 */
static int hf_umts_rrc_pdu_def_rrcConnectionSetupComplete_v3g0ext = -1;  /* RRCConnectionSetupComplete_v3g0ext_IEs */
static int hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions12 = -1;  /* T_v4b0NonCriticalExtensions12 */
static int hf_umts_rrc_pdu_def_rrcConnectionSetupComplete_v4b0ext = -1;  /* RRCConnectionSetupComplete_v4b0ext_IEs */
static int hf_umts_rrc_pdu_def_v590NonCriticalExtensions23 = -1;  /* T_v590NonCriticalExtensions23 */
static int hf_umts_rrc_pdu_def_rrcConnectionSetupComplete_v590ext = -1;  /* RRCConnectionSetupComplete_v590ext_IEs */
static int hf_umts_rrc_pdu_def_v5c0NonCriticalExtensions = -1;  /* T_v5c0NonCriticalExtensions */
static int hf_umts_rrc_pdu_def_rrcConnectionSetupComplete_v5c0ext = -1;  /* RRCConnectionSetupComplete_v5c0ext_IEs */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions31 = -1;  /* T_v6xyNonCriticalExtensions31 */
static int hf_umts_rrc_pdu_def_rrcConnectionSetupComplete_v6xyext = -1;  /* RRCConnectionSetupComplete_v6xyext_IEs */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions76 = -1;  /* T_nonCriticalExtensions76 */
static int hf_umts_rrc_pdu_def_ue_RadioAccessCapability_v370ext = -1;  /* UE_RadioAccessCapability_v370ext */
static int hf_umts_rrc_pdu_def_rrcConnectionSetupComplete_v650ext = -1;  /* RRCConnectionSetupComplete_v650ext_IEs */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions77 = -1;  /* T_nonCriticalExtensions77 */
static int hf_umts_rrc_pdu_def_ue_RadioAccessCapability_v4b0ext = -1;  /* UE_RadioAccessCapability_v4b0ext */
static int hf_umts_rrc_pdu_def_ue_RadioAccessCapability_v590ext = -1;  /* UE_RadioAccessCapability_v590ext */
static int hf_umts_rrc_pdu_def_ue_RATSpecificCapability_v590ext = -1;  /* InterRAT_UE_RadioAccessCapability_v590ext */
static int hf_umts_rrc_pdu_def_ue_RadioAccessCapability_v5c0ext = -1;  /* UE_RadioAccessCapability_v5c0ext */
static int hf_umts_rrc_pdu_def_ue_RadioAccessCapability_v650ext = -1;  /* UE_RadioAccessCapability_v650ext */
static int hf_umts_rrc_pdu_def_ueCapabilityContainer = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_r321 = -1;         /* T_r321 */
static int hf_umts_rrc_pdu_def_rRC_FailureInfo_r3 = -1;  /* RRC_FailureInfo_r3_IEs */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions43 = -1;  /* T_laterNonCriticalExtensions43 */
static int hf_umts_rrc_pdu_def_rrc_FailureInfo_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions78 = -1;  /* T_nonCriticalExtensions78 */
static int hf_umts_rrc_pdu_def_criticalExtensions54 = -1;  /* T_criticalExtensions54 */
static int hf_umts_rrc_pdu_def_failureCauseWithProtErr = -1;  /* FailureCauseWithProtErr */
static int hf_umts_rrc_pdu_def_protocolErrorInformation = -1;  /* ProtocolErrorMoreInformation */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions44 = -1;  /* T_laterNonCriticalExtensions44 */
static int hf_umts_rrc_pdu_def_rrcStatus_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions79 = -1;  /* T_nonCriticalExtensions79 */
static int hf_umts_rrc_pdu_def_r322 = -1;         /* T_r322 */
static int hf_umts_rrc_pdu_def_securityModeCommand_r3 = -1;  /* SecurityModeCommand_r3_IEs */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions45 = -1;  /* T_laterNonCriticalExtensions45 */
static int hf_umts_rrc_pdu_def_securityModeCommand_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions32 = -1;  /* T_v6xyNonCriticalExtensions32 */
static int hf_umts_rrc_pdu_def_securityModeCommand_v6xyext = -1;  /* SecurityModeCommand_v6xyext_IEs */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions80 = -1;  /* T_nonCriticalExtensions80 */
static int hf_umts_rrc_pdu_def_later_than_r319 = -1;  /* T_later_than_r319 */
static int hf_umts_rrc_pdu_def_criticalExtensions55 = -1;  /* T_criticalExtensions55 */
static int hf_umts_rrc_pdu_def_securityCapability = -1;  /* SecurityCapability */
static int hf_umts_rrc_pdu_def_ue_SystemSpecificSecurityCap = -1;  /* InterRAT_UE_SecurityCapList */
static int hf_umts_rrc_pdu_def_gsmSecurityCapability_v6xyext = -1;  /* GSMSecurityCapability_v6xyext */
static int hf_umts_rrc_pdu_def_rb_UL_CiphActivationTimeInfo = -1;  /* RB_ActivationTimeInfoList */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions46 = -1;  /* T_laterNonCriticalExtensions46 */
static int hf_umts_rrc_pdu_def_securityModeComplete_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions81 = -1;  /* T_nonCriticalExtensions81 */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions47 = -1;  /* T_laterNonCriticalExtensions47 */
static int hf_umts_rrc_pdu_def_securityModeFailure_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions82 = -1;  /* T_nonCriticalExtensions82 */
static int hf_umts_rrc_pdu_def_r323 = -1;         /* T_r323 */
static int hf_umts_rrc_pdu_def_signallingConnectionRelease_r3 = -1;  /* SignallingConnectionRelease_r3_IEs */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions48 = -1;  /* T_laterNonCriticalExtensions48 */
static int hf_umts_rrc_pdu_def_signallingConnectionRelease_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions83 = -1;  /* T_nonCriticalExtensions83 */
static int hf_umts_rrc_pdu_def_later_than_r320 = -1;  /* T_later_than_r320 */
static int hf_umts_rrc_pdu_def_criticalExtensions56 = -1;  /* T_criticalExtensions56 */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions49 = -1;  /* T_laterNonCriticalExtensions49 */
static int hf_umts_rrc_pdu_def_signallingConnectionReleaseIndication_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions84 = -1;  /* T_nonCriticalExtensions84 */
static int hf_umts_rrc_pdu_def_sfn_Prime = -1;    /* SFN_Prime */
static int hf_umts_rrc_pdu_def_payload = -1;      /* T_payload */
static int hf_umts_rrc_pdu_def_noSegment = -1;    /* NULL */
static int hf_umts_rrc_pdu_def_firstSegment = -1;  /* FirstSegment */
static int hf_umts_rrc_pdu_def_subsequentSegment = -1;  /* SubsequentSegment */
static int hf_umts_rrc_pdu_def_lastSegmentShort = -1;  /* LastSegmentShort */
static int hf_umts_rrc_pdu_def_lastAndFirst = -1;  /* T_lastAndFirst */
static int hf_umts_rrc_pdu_def_firstSegment1 = -1;  /* FirstSegmentShort */
static int hf_umts_rrc_pdu_def_lastAndComplete = -1;  /* T_lastAndComplete */
static int hf_umts_rrc_pdu_def_completeSIB_List = -1;  /* CompleteSIB_List */
static int hf_umts_rrc_pdu_def_lastAndCompleteAndFirst = -1;  /* T_lastAndCompleteAndFirst */
static int hf_umts_rrc_pdu_def_completeAndFirst = -1;  /* T_completeAndFirst */
static int hf_umts_rrc_pdu_def_completeSIB = -1;  /* CompleteSIB */
static int hf_umts_rrc_pdu_def_lastSegment = -1;  /* LastSegment */
static int hf_umts_rrc_pdu_def_spare5 = -1;       /* NULL */
static int hf_umts_rrc_pdu_def_spare4 = -1;       /* NULL */
static int hf_umts_rrc_pdu_def_spare3 = -1;       /* NULL */
static int hf_umts_rrc_pdu_def_spare2 = -1;       /* NULL */
static int hf_umts_rrc_pdu_def_spare1 = -1;       /* NULL */
static int hf_umts_rrc_pdu_def_payload1 = -1;     /* T_payload1 */
static int hf_umts_rrc_pdu_def_lastAndFirst1 = -1;  /* T_lastAndFirst1 */
static int hf_umts_rrc_pdu_def_lastAndComplete1 = -1;  /* T_lastAndComplete1 */
static int hf_umts_rrc_pdu_def_lastAndCompleteAndFirst1 = -1;  /* T_lastAndCompleteAndFirst1 */
static int hf_umts_rrc_pdu_def_completeAndFirst1 = -1;  /* T_completeAndFirst1 */
static int hf_umts_rrc_pdu_def_sib_Type = -1;     /* SIB_Type */
static int hf_umts_rrc_pdu_def_seg_Count = -1;    /* SegCount */
static int hf_umts_rrc_pdu_def_sib_Data_fixed = -1;  /* SIB_Data_fixed */
static int hf_umts_rrc_pdu_def_sib_Data_variable = -1;  /* SIB_Data_variable */
static int hf_umts_rrc_pdu_def_segmentIndex = -1;  /* SegmentIndex */
static int hf_umts_rrc_pdu_def_CompleteSIB_List_item = -1;  /* CompleteSIBshort */
static int hf_umts_rrc_pdu_def_sib_Data_fixed1 = -1;  /* BIT_STRING_SIZE_226 */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions50 = -1;  /* T_laterNonCriticalExtensions50 */
static int hf_umts_rrc_pdu_def_systemInformationChangeIndication_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions85 = -1;  /* T_nonCriticalExtensions85 */
static int hf_umts_rrc_pdu_def_r324 = -1;         /* T_r324 */
static int hf_umts_rrc_pdu_def_transportChannelReconfiguration_r3 = -1;  /* TransportChannelReconfiguration_r3_IEs */
static int hf_umts_rrc_pdu_def_v3a0NonCriticalExtensions9 = -1;  /* T_v3a0NonCriticalExtensions9 */
static int hf_umts_rrc_pdu_def_transportChannelReconfiguration_v3a0ext = -1;  /* TransportChannelReconfiguration_v3a0ext */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions51 = -1;  /* T_laterNonCriticalExtensions51 */
static int hf_umts_rrc_pdu_def_transportChannelReconfiguration_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions13 = -1;  /* T_v4b0NonCriticalExtensions13 */
static int hf_umts_rrc_pdu_def_transportChannelReconfiguration_v4b0ext = -1;  /* TransportChannelReconfiguration_v4b0ext_IEs */
static int hf_umts_rrc_pdu_def_v590NonCriticalExtensions24 = -1;  /* T_v590NonCriticalExtensions24 */
static int hf_umts_rrc_pdu_def_transportChannelReconfiguration_v590ext = -1;  /* TransportChannelReconfiguration_v590ext_IEs */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions33 = -1;  /* T_v6xyNonCriticalExtensions33 */
static int hf_umts_rrc_pdu_def_transportChannelReconfiguration_v6xyext = -1;  /* TransportChannelReconfiguration_v6xyext_IEs */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions86 = -1;  /* T_nonCriticalExtensions86 */
static int hf_umts_rrc_pdu_def_later_than_r321 = -1;  /* T_later_than_r321 */
static int hf_umts_rrc_pdu_def_criticalExtensions57 = -1;  /* T_criticalExtensions57 */
static int hf_umts_rrc_pdu_def_r412 = -1;         /* T_r412 */
static int hf_umts_rrc_pdu_def_transportChannelReconfiguration_r4 = -1;  /* TransportChannelReconfiguration_r4_IEs */
static int hf_umts_rrc_pdu_def_v4d0NonCriticalExtensions12 = -1;  /* T_v4d0NonCriticalExtensions12 */
static int hf_umts_rrc_pdu_def_transportChannelReconfiguration_r4_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v590NonCriticalExtensions25 = -1;  /* T_v590NonCriticalExtensions25 */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions34 = -1;  /* T_v6xyNonCriticalExtensions34 */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions87 = -1;  /* T_nonCriticalExtensions87 */
static int hf_umts_rrc_pdu_def_criticalExtensions58 = -1;  /* T_criticalExtensions58 */
static int hf_umts_rrc_pdu_def_r510 = -1;         /* T_r510 */
static int hf_umts_rrc_pdu_def_transportChannelReconfiguration_r5 = -1;  /* TransportChannelReconfiguration_r5_IEs */
static int hf_umts_rrc_pdu_def_transportChannelReconfiguration_r5_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions35 = -1;  /* T_v6xyNonCriticalExtensions35 */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions88 = -1;  /* T_nonCriticalExtensions88 */
static int hf_umts_rrc_pdu_def_criticalExtensions59 = -1;  /* T_criticalExtensions59 */
static int hf_umts_rrc_pdu_def_r610 = -1;         /* T_r610 */
static int hf_umts_rrc_pdu_def_transportChannelReconfiguration_r6 = -1;  /* TransportChannelReconfiguration_r6_IEs */
static int hf_umts_rrc_pdu_def_transportChannelReconfiguration_r6_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions89 = -1;  /* T_nonCriticalExtensions89 */
static int hf_umts_rrc_pdu_def_criticalExtensions60 = -1;  /* T_criticalExtensions60 */
static int hf_umts_rrc_pdu_def_transportChannelReconfiguration_r3_IEs_dummy = -1;  /* T_transportChannelReconfiguration_r3_IEs_dummy */
static int hf_umts_rrc_pdu_def_fdd33 = -1;        /* T_fdd33 */
static int hf_umts_rrc_pdu_def_modeSpecificPhysChInfo12 = -1;  /* T_modeSpecificPhysChInfo12 */
static int hf_umts_rrc_pdu_def_fdd34 = -1;        /* T_fdd34 */
static int hf_umts_rrc_pdu_def_transportChannelReconfiguration_r4_IEs_dummy = -1;  /* T_transportChannelReconfiguration_r4_IEs_dummy */
static int hf_umts_rrc_pdu_def_fdd35 = -1;        /* T_fdd35 */
static int hf_umts_rrc_pdu_def_modeSpecificPhysChInfo13 = -1;  /* T_modeSpecificPhysChInfo13 */
static int hf_umts_rrc_pdu_def_fdd36 = -1;        /* T_fdd36 */
static int hf_umts_rrc_pdu_def_transportChannelReconfiguration_r5_IEs_dummy = -1;  /* T_transportChannelReconfiguration_r5_IEs_dummy */
static int hf_umts_rrc_pdu_def_fdd37 = -1;        /* T_fdd37 */
static int hf_umts_rrc_pdu_def_modeSpecificPhysChInfo14 = -1;  /* T_modeSpecificPhysChInfo14 */
static int hf_umts_rrc_pdu_def_fdd38 = -1;        /* T_fdd38 */
static int hf_umts_rrc_pdu_def_postVerificationPeriod6 = -1;  /* T_postVerificationPeriod6 */
static int hf_umts_rrc_pdu_def_timingMaintainedSynchInd4 = -1;  /* T_timingMaintainedSynchInd4 */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions52 = -1;  /* T_laterNonCriticalExtensions52 */
static int hf_umts_rrc_pdu_def_transportChannelReconfigurationComplete_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions90 = -1;  /* T_nonCriticalExtensions90 */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions53 = -1;  /* T_laterNonCriticalExtensions53 */
static int hf_umts_rrc_pdu_def_transportChannelReconfigurationFailure_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions91 = -1;  /* T_nonCriticalExtensions91 */
static int hf_umts_rrc_pdu_def_modeSpecificInfo9 = -1;  /* T_modeSpecificInfo9 */
static int hf_umts_rrc_pdu_def_fdd39 = -1;        /* NULL */
static int hf_umts_rrc_pdu_def_tdd4 = -1;         /* T_tdd3 */
static int hf_umts_rrc_pdu_def_tfcs_ID = -1;      /* TFCS_Identity */
static int hf_umts_rrc_pdu_def_dpch_TFCS_InUplink = -1;  /* TFC_Subset */
static int hf_umts_rrc_pdu_def_activationTimeForTFCSubset = -1;  /* ActivationTime */
static int hf_umts_rrc_pdu_def_tfc_ControlDuration = -1;  /* TFC_ControlDuration */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions54 = -1;  /* T_laterNonCriticalExtensions54 */
static int hf_umts_rrc_pdu_def_transportFormatCombinationControl_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions92 = -1;  /* T_nonCriticalExtensions92 */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions55 = -1;  /* T_laterNonCriticalExtensions55 */
static int hf_umts_rrc_pdu_def_transportFormatCombinationControlFailure_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions93 = -1;  /* T_nonCriticalExtensions93 */
static int hf_umts_rrc_pdu_def_r325 = -1;         /* T_r325 */
static int hf_umts_rrc_pdu_def_ueCapabilityEnquiry_r3 = -1;  /* UECapabilityEnquiry_r3_IEs */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions56 = -1;  /* T_laterNonCriticalExtensions56 */
static int hf_umts_rrc_pdu_def_ueCapabilityEnquiry_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions14 = -1;  /* T_v4b0NonCriticalExtensions14 */
static int hf_umts_rrc_pdu_def_ueCapabilityEnquiry_v4b0ext = -1;  /* UECapabilityEnquiry_v4b0ext_IEs */
static int hf_umts_rrc_pdu_def_v590NonCriticalExtensions26 = -1;  /* T_v590NonCriticalExtensions26 */
static int hf_umts_rrc_pdu_def_ueCapabilityEnquiry_v590ext = -1;  /* UECapabilityEnquiry_v590ext_IEs */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions94 = -1;  /* T_nonCriticalExtensions94 */
static int hf_umts_rrc_pdu_def_later_than_r322 = -1;  /* T_later_than_r322 */
static int hf_umts_rrc_pdu_def_criticalExtensions61 = -1;  /* T_criticalExtensions61 */
static int hf_umts_rrc_pdu_def_v370NonCriticalExtensions1 = -1;  /* T_v370NonCriticalExtensions1 */
static int hf_umts_rrc_pdu_def_ueCapabilityInformation_v370ext = -1;  /* UECapabilityInformation_v370ext */
static int hf_umts_rrc_pdu_def_v380NonCriticalExtensions1 = -1;  /* T_v380NonCriticalExtensions1 */
static int hf_umts_rrc_pdu_def_ueCapabilityInformation_v380ext = -1;  /* UECapabilityInformation_v380ext_IEs */
static int hf_umts_rrc_pdu_def_v3a0NonCriticalExtensions10 = -1;  /* T_v3a0NonCriticalExtensions10 */
static int hf_umts_rrc_pdu_def_ueCapabilityInformation_v3a0ext = -1;  /* UECapabilityInformation_v3a0ext_IEs */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions57 = -1;  /* T_laterNonCriticalExtensions57 */
static int hf_umts_rrc_pdu_def_ueCapabilityInformation_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions15 = -1;  /* T_v4b0NonCriticalExtensions15 */
static int hf_umts_rrc_pdu_def_ueCapabilityInformation_v4b0ext = -1;  /* UECapabilityInformation_v4b0ext */
static int hf_umts_rrc_pdu_def_v590NonCriticalExtensions27 = -1;  /* T_v590NonCriticalExtensions27 */
static int hf_umts_rrc_pdu_def_ueCapabilityInformation_v590ext = -1;  /* UECapabilityInformation_v590ext */
static int hf_umts_rrc_pdu_def_v5c0NonCriticalExtensions1 = -1;  /* T_v5c0NonCriticalExtensions1 */
static int hf_umts_rrc_pdu_def_ueCapabilityInformation_v5c0ext = -1;  /* UECapabilityInformation_v5c0ext */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions36 = -1;  /* T_v6xyNonCriticalExtensions36 */
static int hf_umts_rrc_pdu_def_ueCapabilityInformation_v6xyext = -1;  /* UECapabilityInformation_v6xyext_IEs */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions95 = -1;  /* T_nonCriticalExtensions95 */
static int hf_umts_rrc_pdu_def_ueCapabilityInformation_v650ext = -1;  /* UECapabilityInformation_v650ext_IEs */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions96 = -1;  /* T_nonCriticalExtensions96 */
static int hf_umts_rrc_pdu_def_r326 = -1;         /* T_r326 */
static int hf_umts_rrc_pdu_def_ueCapabilityInformationConfirm_r3 = -1;  /* UECapabilityInformationConfirm_r3_IEs */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions58 = -1;  /* T_laterNonCriticalExtensions58 */
static int hf_umts_rrc_pdu_def_ueCapabilityInformationConfirm_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions97 = -1;  /* T_nonCriticalExtensions97 */
static int hf_umts_rrc_pdu_def_later_than_r323 = -1;  /* T_later_than_r323 */
static int hf_umts_rrc_pdu_def_criticalExtensions62 = -1;  /* T_criticalExtensions62 */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions59 = -1;  /* T_laterNonCriticalExtensions59 */
static int hf_umts_rrc_pdu_def_uplinkDirectTransfer_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions37 = -1;  /* T_v6xyNonCriticalExtensions37 */
static int hf_umts_rrc_pdu_def_uplinkDirectTransfer_v6xyext = -1;  /* UplinkDirectTransfer_v6xyext_IEs */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions98 = -1;  /* T_nonCriticalExtensions98 */
static int hf_umts_rrc_pdu_def_r327 = -1;         /* T_r327 */
static int hf_umts_rrc_pdu_def_uplinkPhysicalChannelControl_r3 = -1;  /* UplinkPhysicalChannelControl_r3_IEs */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions60 = -1;  /* T_laterNonCriticalExtensions60 */
static int hf_umts_rrc_pdu_def_uplinkPhysicalChannelControl_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions16 = -1;  /* T_v4b0NonCriticalExtensions16 */
static int hf_umts_rrc_pdu_def_uplinkPysicalChannelControl_v4b0ext = -1;  /* UplinkPhysicalChannelControl_v4b0ext_IEs */
static int hf_umts_rrc_pdu_def_noncriticalExtensions = -1;  /* T_noncriticalExtensions */
static int hf_umts_rrc_pdu_def_later_than_r324 = -1;  /* T_later_than_r324 */
static int hf_umts_rrc_pdu_def_criticalExtensions63 = -1;  /* T_criticalExtensions63 */
static int hf_umts_rrc_pdu_def_r413 = -1;         /* T_r413 */
static int hf_umts_rrc_pdu_def_uplinkPhysicalChannelControl_r4 = -1;  /* UplinkPhysicalChannelControl_r4_IEs */
static int hf_umts_rrc_pdu_def_v4d0NonCriticalExtensions13 = -1;  /* T_v4d0NonCriticalExtensions13 */
static int hf_umts_rrc_pdu_def_uplinkPhysicalChannelControl_r4_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions38 = -1;  /* T_v6xyNonCriticalExtensions38 */
static int hf_umts_rrc_pdu_def_uplinkPhysicalChannelControl_v6xyext = -1;  /* UplinkPhysicalChannelControl_v6xyext_IEs */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions99 = -1;  /* T_nonCriticalExtensions99 */
static int hf_umts_rrc_pdu_def_criticalExtensions64 = -1;  /* T_criticalExtensions64 */
static int hf_umts_rrc_pdu_def_r511 = -1;         /* T_r511 */
static int hf_umts_rrc_pdu_def_uplinkPhysicalChannelControl_r5 = -1;  /* UplinkPhysicalChannelControl_r5_IEs */
static int hf_umts_rrc_pdu_def_uplinkPhysicalChannelControl_r5_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions39 = -1;  /* T_v6xyNonCriticalExtensions39 */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions100 = -1;  /* T_nonCriticalExtensions100 */
static int hf_umts_rrc_pdu_def_criticalExtensions65 = -1;  /* T_criticalExtensions65 */
static int hf_umts_rrc_pdu_def_ccTrCH_PowerControlInfo = -1;  /* CCTrCH_PowerControlInfo */
static int hf_umts_rrc_pdu_def_timingAdvance = -1;  /* UL_TimingAdvanceControl */
static int hf_umts_rrc_pdu_def_alpha = -1;        /* Alpha */
static int hf_umts_rrc_pdu_def_specialBurstScheduling = -1;  /* SpecialBurstScheduling */
static int hf_umts_rrc_pdu_def_prach_ConstantValue = -1;  /* ConstantValueTdd */
static int hf_umts_rrc_pdu_def_pusch_ConstantValue = -1;  /* ConstantValueTdd */
static int hf_umts_rrc_pdu_def_openLoopPowerControl_IPDL_TDD = -1;  /* OpenLoopPowerControl_IPDL_TDD_r4 */
static int hf_umts_rrc_pdu_def_ccTrCH_PowerControlInfo1 = -1;  /* CCTrCH_PowerControlInfo_r4 */
static int hf_umts_rrc_pdu_def_tddOption = -1;    /* T_tddOption */
static int hf_umts_rrc_pdu_def_tdd3842 = -1;      /* T_tdd3842 */
static int hf_umts_rrc_pdu_def_timingAdvance1 = -1;  /* UL_TimingAdvanceControl_r4 */
static int hf_umts_rrc_pdu_def_tdd1282 = -1;      /* T_tdd1282 */
static int hf_umts_rrc_pdu_def_ul_SynchronisationParameters = -1;  /* UL_SynchronisationParameters_r4 */
static int hf_umts_rrc_pdu_def_ccTrCH_PowerControlInfo2 = -1;  /* CCTrCH_PowerControlInfo_r5 */
static int hf_umts_rrc_pdu_def_tddOption1 = -1;   /* T_tddOption1 */
static int hf_umts_rrc_pdu_def_tdd3843 = -1;      /* T_tdd3843 */
static int hf_umts_rrc_pdu_def_hs_SICH_PowerControl = -1;  /* HS_SICH_Power_Control_Info_TDD384 */
static int hf_umts_rrc_pdu_def_tdd1283 = -1;      /* T_tdd1283 */
static int hf_umts_rrc_pdu_def_ura_UpdateCause = -1;  /* URA_UpdateCause */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions61 = -1;  /* T_laterNonCriticalExtensions61 */
static int hf_umts_rrc_pdu_def_uraUpdate_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions101 = -1;  /* T_nonCriticalExtensions101 */
static int hf_umts_rrc_pdu_def_r328 = -1;         /* T_r328 */
static int hf_umts_rrc_pdu_def_uraUpdateConfirm_r3 = -1;  /* URAUpdateConfirm_r3_IEs */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions62 = -1;  /* T_laterNonCriticalExtensions62 */
static int hf_umts_rrc_pdu_def_uraUpdateConfirm_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions40 = -1;  /* T_v6xyNonCriticalExtensions40 */
static int hf_umts_rrc_pdu_def_uraUpdateConfirm_v6xyext = -1;  /* URAUpdateConfirm_v6xyext_IEs */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions102 = -1;  /* T_nonCriticalExtensions102 */
static int hf_umts_rrc_pdu_def_later_than_r325 = -1;  /* T_later_than_r325 */
static int hf_umts_rrc_pdu_def_criticalExtensions66 = -1;  /* T_criticalExtensions66 */
static int hf_umts_rrc_pdu_def_r512 = -1;         /* T_r512 */
static int hf_umts_rrc_pdu_def_uraUpdateConfirm_r5 = -1;  /* URAUpdateConfirm_r5_IEs */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions41 = -1;  /* T_v6xyNonCriticalExtensions41 */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions103 = -1;  /* T_nonCriticalExtensions103 */
static int hf_umts_rrc_pdu_def_criticalExtensions67 = -1;  /* T_criticalExtensions67 */
static int hf_umts_rrc_pdu_def_r329 = -1;         /* T_r329 */
static int hf_umts_rrc_pdu_def_uraUpdateConfirm_CCCH_r3 = -1;  /* URAUpdateConfirm_CCCH_r3_IEs */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions63 = -1;  /* T_laterNonCriticalExtensions63 */
static int hf_umts_rrc_pdu_def_uraUpdateConfirm_CCCH_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions42 = -1;  /* T_v6xyNonCriticalExtensions42 */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions104 = -1;  /* T_nonCriticalExtensions104 */
static int hf_umts_rrc_pdu_def_later_than_r326 = -1;  /* T_later_than_r326 */
static int hf_umts_rrc_pdu_def_criticalExtensions68 = -1;  /* T_criticalExtensions68 */
static int hf_umts_rrc_pdu_def_uraUpdateConfirm = -1;  /* URAUpdateConfirm_r3_IEs */
static int hf_umts_rrc_pdu_def_r330 = -1;         /* T_r330 */
static int hf_umts_rrc_pdu_def_utranMobilityInformation_r3 = -1;  /* UTRANMobilityInformation_r3_IEs */
static int hf_umts_rrc_pdu_def_v3a0NonCriticalExtensions11 = -1;  /* T_v3a0NonCriticalExtensions11 */
static int hf_umts_rrc_pdu_def_utranMobilityInformation_v3a0ext = -1;  /* UTRANMobilityInformation_v3a0ext_IEs */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions64 = -1;  /* T_laterNonCriticalExtensions64 */
static int hf_umts_rrc_pdu_def_utranMobilityInformation_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions43 = -1;  /* T_v6xyNonCriticalExtensions43 */
static int hf_umts_rrc_pdu_def_utranMobilityInformation_v6xyext = -1;  /* UtranMobilityInformation_v6xyext_IEs */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions105 = -1;  /* T_nonCriticalExtensions105 */
static int hf_umts_rrc_pdu_def_later_than_r327 = -1;  /* T_later_than_r327 */
static int hf_umts_rrc_pdu_def_criticalExtensions69 = -1;  /* T_criticalExtensions69 */
static int hf_umts_rrc_pdu_def_r513 = -1;         /* T_r513 */
static int hf_umts_rrc_pdu_def_utranMobilityInformation_r5 = -1;  /* UTRANMobilityInformation_r5_IEs */
static int hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions44 = -1;  /* T_v6xyNonCriticalExtensions44 */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions106 = -1;  /* T_nonCriticalExtensions106 */
static int hf_umts_rrc_pdu_def_criticalExtensions70 = -1;  /* T_criticalExtensions70 */
static int hf_umts_rrc_pdu_def_ue_ConnTimersAndConstants = -1;  /* UE_ConnTimersAndConstants */
static int hf_umts_rrc_pdu_def_cn_InformationInfo2 = -1;  /* CN_InformationInfoFull */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions107 = -1;  /* T_nonCriticalExtensions107 */
static int hf_umts_rrc_pdu_def_ue_ConnTimersAndConstants_v3a0ext = -1;  /* UE_ConnTimersAndConstants_v3a0ext */
static int hf_umts_rrc_pdu_def_ue_ConnTimersAndConstants1 = -1;  /* UE_ConnTimersAndConstants_r5 */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions65 = -1;  /* T_laterNonCriticalExtensions65 */
static int hf_umts_rrc_pdu_def_utranMobilityInformationConfirm_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions108 = -1;  /* T_nonCriticalExtensions108 */
static int hf_umts_rrc_pdu_def_laterNonCriticalExtensions66 = -1;  /* T_laterNonCriticalExtensions66 */
static int hf_umts_rrc_pdu_def_utranMobilityInformationFailure_r3_add_ext = -1;  /* BIT_STRING */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions109 = -1;  /* T_nonCriticalExtensions109 */
static int hf_umts_rrc_pdu_def_mbms_ServiceAccessInfoList = -1;  /* MBMS_ServiceAccessInfoList_r6 */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions110 = -1;  /* T_nonCriticalExtensions110 */
static int hf_umts_rrc_pdu_def_mbms_CommonRBInformationList = -1;  /* MBMS_CommonRBInformationList_r6 */
static int hf_umts_rrc_pdu_def_mbms_TranspChInfoForEachTrCh = -1;  /* MBMS_TranspChInfoForEachTrCh_r6 */
static int hf_umts_rrc_pdu_def_mbms_TranspChInfoForEachCCTrCh = -1;  /* MBMS_TranspChInfoForEachCCTrCh_r6 */
static int hf_umts_rrc_pdu_def_mbms_PhyChInformationList = -1;  /* MBMS_PhyChInformationList_r6 */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions111 = -1;  /* T_nonCriticalExtensions111 */
static int hf_umts_rrc_pdu_def_mbms_CurrentCell_SCCPCHList = -1;  /* MBMS_CurrentCell_SCCPCHList_r6 */
static int hf_umts_rrc_pdu_def_mbms_SIBType5_SCCPCHList = -1;  /* MBMS_SIBType5_SCCPCHList_r6 */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions112 = -1;  /* T_nonCriticalExtensions112 */
static int hf_umts_rrc_pdu_def_mbms_PreferredFrequencyInfo = -1;  /* MBMS_PreferredFrequencyList_r6 */
static int hf_umts_rrc_pdu_def_mbms_TimersAndCouneters = -1;  /* MBMS_TimersAndCouneters_r6 */
static int hf_umts_rrc_pdu_def_michConfigurationInfo = -1;  /* MBMS_MICHConfigurationInfo_r6 */
static int hf_umts_rrc_pdu_def_cellGroupIdentity = -1;  /* MBMS_CellGroupIdentity_r6 */
static int hf_umts_rrc_pdu_def_mschDefaultConfigurationInfo = -1;  /* MBMS_MSCHConfigurationInfo_r6 */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions113 = -1;  /* T_nonCriticalExtensions113 */
static int hf_umts_rrc_pdu_def_mbms_PreferredFreqRequest = -1;  /* MBMS_ServiceIdentity */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions114 = -1;  /* T_nonCriticalExtensions114 */
static int hf_umts_rrc_pdu_def_modifedServiceList = -1;  /* MBMS_ModifedServiceList_r6 */
static int hf_umts_rrc_pdu_def_mbms_ReacquireMCCH = -1;  /* T_mbms_ReacquireMCCH */
static int hf_umts_rrc_pdu_def_mbms_DynamicPersistenceLevel = -1;  /* DynamicPersistenceLevel */
static int hf_umts_rrc_pdu_def_endOfModifiedMCCHInformation = -1;  /* INTEGER_1_16 */
static int hf_umts_rrc_pdu_def_mbms_AllUnmodifiedPTMServices = -1;  /* T_mbms_AllUnmodifiedPTMServices */
static int hf_umts_rrc_pdu_def_mbms_PtMActivationTime = -1;  /* MBMS_PtMActivationTime */
static int hf_umts_rrc_pdu_def_mbmsNumberOfNeighbourCells = -1;  /* MBMS_NumberOfNeighbourCells_r6 */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions115 = -1;  /* T_nonCriticalExtensions115 */
static int hf_umts_rrc_pdu_def_neighbouringCellIdentity = -1;  /* IntraFreqCellID */
static int hf_umts_rrc_pdu_def_neighbouringCellSCCPCHList = -1;  /* MBMS_NeighbouringCellSCCPCHList_r6 */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions116 = -1;  /* T_nonCriticalExtensions116 */
static int hf_umts_rrc_pdu_def_serviceSchedulingInfoList = -1;  /* MBMS_ServiceSchedulingInfoList_r6 */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions117 = -1;  /* T_nonCriticalExtensions117 */
static int hf_umts_rrc_pdu_def_unmodifiedServiceList = -1;  /* MBMS_UnmodifiedServiceList_r6 */
static int hf_umts_rrc_pdu_def_nonCriticalExtensions118 = -1;  /* T_nonCriticalExtensions118 */

/*--- End of included file: packet-umts_rrc_pdu_def-hf.c ---*/
#line 60 "packet-umts_rrc_pdu_def-template.c"

/* Initialize the subtree pointers */
static int ett_umts_rrc_pdu_def = -1;


/*--- Included file: packet-umts_rrc_pdu_def-ett.c ---*/
#line 1 "packet-umts_rrc_pdu_def-ett.c"
static gint ett_umts_rrc_pdu_def_ActiveSetUpdate = -1;
static gint ett_umts_rrc_pdu_def_T_r3 = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions = -1;
static gint ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions = -1;
static gint ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions = -1;
static gint ett_umts_rrc_pdu_def_T_later_than_r3 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions = -1;
static gint ett_umts_rrc_pdu_def_T_r6 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions1 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions1 = -1;
static gint ett_umts_rrc_pdu_def_ActiveSetUpdate_r3_IEs = -1;
static gint ett_umts_rrc_pdu_def_ActiveSetUpdate_v4b0ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_ActiveSetUpdate_v590ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_ActiveSetUpdate_v6xyext_IEs = -1;
static gint ett_umts_rrc_pdu_def_ActiveSetUpdate_r6_IEs = -1;
static gint ett_umts_rrc_pdu_def_ActiveSetUpdateComplete = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions1 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions2 = -1;
static gint ett_umts_rrc_pdu_def_ActiveSetUpdateFailure = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions2 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions3 = -1;
static gint ett_umts_rrc_pdu_def_AssistanceDataDelivery = -1;
static gint ett_umts_rrc_pdu_def_T_r31 = -1;
static gint ett_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions3 = -1;
static gint ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions1 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions4 = -1;
static gint ett_umts_rrc_pdu_def_T_later_than_r31 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions2 = -1;
static gint ett_umts_rrc_pdu_def_AssistanceDataDelivery_r3_IEs = -1;
static gint ett_umts_rrc_pdu_def_AssistanceDataDelivery_v3a0ext = -1;
static gint ett_umts_rrc_pdu_def_AssistanceDataDelivery_v4b0ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_CellChangeOrderFromUTRAN = -1;
static gint ett_umts_rrc_pdu_def_T_r32 = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions4 = -1;
static gint ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions1 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions5 = -1;
static gint ett_umts_rrc_pdu_def_T_later_than_r32 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions3 = -1;
static gint ett_umts_rrc_pdu_def_CellChangeOrderFromUTRAN_r3_IEs = -1;
static gint ett_umts_rrc_pdu_def_CellChangeOrderFromUTRAN_v590ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_T_geran_SystemInfoType = -1;
static gint ett_umts_rrc_pdu_def_CellChangeOrderFromUTRANFailure = -1;
static gint ett_umts_rrc_pdu_def_T_r33 = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions5 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions6 = -1;
static gint ett_umts_rrc_pdu_def_T_dummy = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions4 = -1;
static gint ett_umts_rrc_pdu_def_CellChangeOrderFromUTRANFailure_r3_IEs = -1;
static gint ett_umts_rrc_pdu_def_CellUpdate = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions6 = -1;
static gint ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions2 = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions1 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions7 = -1;
static gint ett_umts_rrc_pdu_def_CellUpdate_v590ext = -1;
static gint ett_umts_rrc_pdu_def_CellUpdate_v6xyext_IEs = -1;
static gint ett_umts_rrc_pdu_def_CellUpdateConfirm = -1;
static gint ett_umts_rrc_pdu_def_T_r34 = -1;
static gint ett_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions1 = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions7 = -1;
static gint ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions2 = -1;
static gint ett_umts_rrc_pdu_def_T_v590NonCriticalExtenstions = -1;
static gint ett_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions2 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions8 = -1;
static gint ett_umts_rrc_pdu_def_T_later_than_r33 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions5 = -1;
static gint ett_umts_rrc_pdu_def_T_r4 = -1;
static gint ett_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions = -1;
static gint ett_umts_rrc_pdu_def_T_v590NonCriticalExtenstions1 = -1;
static gint ett_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions1 = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions3 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions9 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions6 = -1;
static gint ett_umts_rrc_pdu_def_T_r5 = -1;
static gint ett_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions2 = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions4 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions10 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions7 = -1;
static gint ett_umts_rrc_pdu_def_T_r61 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions11 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions8 = -1;
static gint ett_umts_rrc_pdu_def_CellUpdateConfirm_r3_IEs = -1;
static gint ett_umts_rrc_pdu_def_T_modeSpecificTransChInfo = -1;
static gint ett_umts_rrc_pdu_def_T_fdd = -1;
static gint ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo = -1;
static gint ett_umts_rrc_pdu_def_T_fdd1 = -1;
static gint ett_umts_rrc_pdu_def_CellUpdateConfirm_v3a0ext = -1;
static gint ett_umts_rrc_pdu_def_CellUpdateConfirm_v4b0ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_CellUpdateConfirm_v590ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_CellUpdateConfirm_v5d0ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_CellUpdateConfirm_r4_IEs = -1;
static gint ett_umts_rrc_pdu_def_T_modeSpecificTransChInfo1 = -1;
static gint ett_umts_rrc_pdu_def_T_fdd2 = -1;
static gint ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo1 = -1;
static gint ett_umts_rrc_pdu_def_T_fdd3 = -1;
static gint ett_umts_rrc_pdu_def_CellUpdateConfirm_r5_IEs = -1;
static gint ett_umts_rrc_pdu_def_T_modeSpecificTransChInfo2 = -1;
static gint ett_umts_rrc_pdu_def_T_fdd4 = -1;
static gint ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo2 = -1;
static gint ett_umts_rrc_pdu_def_T_fdd5 = -1;
static gint ett_umts_rrc_pdu_def_CellUpdateConfirm_r6_IEs = -1;
static gint ett_umts_rrc_pdu_def_CellUpdateConfirm_v6xyext_IEs = -1;
static gint ett_umts_rrc_pdu_def_CellUpdateConfirm_CCCH = -1;
static gint ett_umts_rrc_pdu_def_T_r35 = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions8 = -1;
static gint ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions3 = -1;
static gint ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions3 = -1;
static gint ett_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions3 = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions5 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions12 = -1;
static gint ett_umts_rrc_pdu_def_T_later_than_r34 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions9 = -1;
static gint ett_umts_rrc_pdu_def_T_r41 = -1;
static gint ett_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions1 = -1;
static gint ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions4 = -1;
static gint ett_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions4 = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions6 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions13 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions10 = -1;
static gint ett_umts_rrc_pdu_def_T_r51 = -1;
static gint ett_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions5 = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions7 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions14 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions11 = -1;
static gint ett_umts_rrc_pdu_def_T_r62 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions15 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions12 = -1;
static gint ett_umts_rrc_pdu_def_CounterCheck = -1;
static gint ett_umts_rrc_pdu_def_T_r36 = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions9 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions16 = -1;
static gint ett_umts_rrc_pdu_def_T_later_than_r35 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions13 = -1;
static gint ett_umts_rrc_pdu_def_CounterCheck_r3_IEs = -1;
static gint ett_umts_rrc_pdu_def_CounterCheckResponse = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions10 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions17 = -1;
static gint ett_umts_rrc_pdu_def_DownlinkDirectTransfer = -1;
static gint ett_umts_rrc_pdu_def_T_r37 = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions11 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions18 = -1;
static gint ett_umts_rrc_pdu_def_T_later_than_r36 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions14 = -1;
static gint ett_umts_rrc_pdu_def_DownlinkDirectTransfer_r3_IEs = -1;
static gint ett_umts_rrc_pdu_def_HandoverToUTRANCommand = -1;
static gint ett_umts_rrc_pdu_def_T_r38 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions19 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions15 = -1;
static gint ett_umts_rrc_pdu_def_T_r42 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions20 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions16 = -1;
static gint ett_umts_rrc_pdu_def_T_r52 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions21 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions17 = -1;
static gint ett_umts_rrc_pdu_def_T_r63 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions22 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions18 = -1;
static gint ett_umts_rrc_pdu_def_HandoverToUTRANCommand_r3_IEs = -1;
static gint ett_umts_rrc_pdu_def_T_specificationMode = -1;
static gint ett_umts_rrc_pdu_def_T_complete = -1;
static gint ett_umts_rrc_pdu_def_T_modeSpecificInfo = -1;
static gint ett_umts_rrc_pdu_def_T_fdd6 = -1;
static gint ett_umts_rrc_pdu_def_T_preconfiguration = -1;
static gint ett_umts_rrc_pdu_def_T_preConfigMode = -1;
static gint ett_umts_rrc_pdu_def_T_defaultConfig = -1;
static gint ett_umts_rrc_pdu_def_T_modeSpecificInfo1 = -1;
static gint ett_umts_rrc_pdu_def_T_fdd7 = -1;
static gint ett_umts_rrc_pdu_def_T_tdd = -1;
static gint ett_umts_rrc_pdu_def_HandoverToUTRANCommand_r4_IEs = -1;
static gint ett_umts_rrc_pdu_def_T_specificationMode1 = -1;
static gint ett_umts_rrc_pdu_def_T_complete1 = -1;
static gint ett_umts_rrc_pdu_def_T_modeSpecificInfo2 = -1;
static gint ett_umts_rrc_pdu_def_T_fdd8 = -1;
static gint ett_umts_rrc_pdu_def_T_preconfiguration1 = -1;
static gint ett_umts_rrc_pdu_def_T_preConfigMode1 = -1;
static gint ett_umts_rrc_pdu_def_T_defaultConfig1 = -1;
static gint ett_umts_rrc_pdu_def_T_modeSpecificInfo3 = -1;
static gint ett_umts_rrc_pdu_def_T_fdd9 = -1;
static gint ett_umts_rrc_pdu_def_T_tdd1 = -1;
static gint ett_umts_rrc_pdu_def_T_tdd384 = -1;
static gint ett_umts_rrc_pdu_def_T_tdd128 = -1;
static gint ett_umts_rrc_pdu_def_HandoverToUTRANCommand_r5_IEs = -1;
static gint ett_umts_rrc_pdu_def_T_specificationMode2 = -1;
static gint ett_umts_rrc_pdu_def_T_complete2 = -1;
static gint ett_umts_rrc_pdu_def_T_modeSpecificInfo4 = -1;
static gint ett_umts_rrc_pdu_def_T_fdd10 = -1;
static gint ett_umts_rrc_pdu_def_T_preconfiguration2 = -1;
static gint ett_umts_rrc_pdu_def_T_preConfigMode2 = -1;
static gint ett_umts_rrc_pdu_def_T_defaultConfig2 = -1;
static gint ett_umts_rrc_pdu_def_T_modeSpecificInfo5 = -1;
static gint ett_umts_rrc_pdu_def_T_fdd11 = -1;
static gint ett_umts_rrc_pdu_def_T_tdd2 = -1;
static gint ett_umts_rrc_pdu_def_T_tdd3841 = -1;
static gint ett_umts_rrc_pdu_def_T_tdd1281 = -1;
static gint ett_umts_rrc_pdu_def_HandoverToUTRANCommand_r6_IEs = -1;
static gint ett_umts_rrc_pdu_def_T_specificationMode3 = -1;
static gint ett_umts_rrc_pdu_def_T_complete3 = -1;
static gint ett_umts_rrc_pdu_def_HandoverToUTRANComplete = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions12 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions23 = -1;
static gint ett_umts_rrc_pdu_def_InitialDirectTransfer = -1;
static gint ett_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions2 = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions13 = -1;
static gint ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions5 = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions8 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions24 = -1;
static gint ett_umts_rrc_pdu_def_InitialDirectTransfer_v3a0ext = -1;
static gint ett_umts_rrc_pdu_def_InitialDirectTransfer_v590ext = -1;
static gint ett_umts_rrc_pdu_def_InitialDirectTransfer_v6xyext_IEs = -1;
static gint ett_umts_rrc_pdu_def_HandoverFromUTRANCommand_GSM = -1;
static gint ett_umts_rrc_pdu_def_T_r39 = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions14 = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions9 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions25 = -1;
static gint ett_umts_rrc_pdu_def_T_later_than_r37 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions19 = -1;
static gint ett_umts_rrc_pdu_def_T_r64 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions26 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions20 = -1;
static gint ett_umts_rrc_pdu_def_HandoverFromUTRANCommand_GSM_r3_IEs = -1;
static gint ett_umts_rrc_pdu_def_T_gsm_message = -1;
static gint ett_umts_rrc_pdu_def_T_single_GSM_Message = -1;
static gint ett_umts_rrc_pdu_def_T_gsm_MessageList = -1;
static gint ett_umts_rrc_pdu_def_HandoverFromUTRANCommand_GSM_r6_IEs = -1;
static gint ett_umts_rrc_pdu_def_T_gsm_message1 = -1;
static gint ett_umts_rrc_pdu_def_T_single_GSM_Message1 = -1;
static gint ett_umts_rrc_pdu_def_T_gsm_MessageList1 = -1;
static gint ett_umts_rrc_pdu_def_T_geran_SystemInfoType1 = -1;
static gint ett_umts_rrc_pdu_def_HandoverFromUTRANCommand_GSM_v6xyext_IEs = -1;
static gint ett_umts_rrc_pdu_def_T_geran_SystemInfoType2 = -1;
static gint ett_umts_rrc_pdu_def_HandoverFromUTRANCommand_GERANIu = -1;
static gint ett_umts_rrc_pdu_def_T_handoverFromUTRANCommand_GERANIu = -1;
static gint ett_umts_rrc_pdu_def_T_r53 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions27 = -1;
static gint ett_umts_rrc_pdu_def_T_later_than_r5 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions21 = -1;
static gint ett_umts_rrc_pdu_def_HandoverFromUTRANCommand_GERANIu_r5_IEs = -1;
static gint ett_umts_rrc_pdu_def_T_geranIu_Message = -1;
static gint ett_umts_rrc_pdu_def_T_single_GERANIu_Message = -1;
static gint ett_umts_rrc_pdu_def_T_geranIu_MessageList = -1;
static gint ett_umts_rrc_pdu_def_HandoverFromUTRANCommand_CDMA2000 = -1;
static gint ett_umts_rrc_pdu_def_T_r310 = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions15 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions28 = -1;
static gint ett_umts_rrc_pdu_def_T_later_than_r38 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions22 = -1;
static gint ett_umts_rrc_pdu_def_HandoverFromUTRANCommand_CDMA2000_r3_IEs = -1;
static gint ett_umts_rrc_pdu_def_HandoverFromUTRANFailure = -1;
static gint ett_umts_rrc_pdu_def_T_interRATMessage = -1;
static gint ett_umts_rrc_pdu_def_T_gsm = -1;
static gint ett_umts_rrc_pdu_def_T_cdma2000 = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions16 = -1;
static gint ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions6 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions29 = -1;
static gint ett_umts_rrc_pdu_def_HandoverFromUtranFailure_v590ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_InterRATHandoverInfo = -1;
static gint ett_umts_rrc_pdu_def_T_predefinedConfigStatusList = -1;
static gint ett_umts_rrc_pdu_def_T_uE_SecurityInformation = -1;
static gint ett_umts_rrc_pdu_def_T_ue_CapabilityContainer = -1;
static gint ett_umts_rrc_pdu_def_T_v390NonCriticalExtensions = -1;
static gint ett_umts_rrc_pdu_def_T_present = -1;
static gint ett_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions3 = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions17 = -1;
static gint ett_umts_rrc_pdu_def_T_v3g0NonCriticalExtensions = -1;
static gint ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions4 = -1;
static gint ett_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions2 = -1;
static gint ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions7 = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions10 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions30 = -1;
static gint ett_umts_rrc_pdu_def_InterRATHandoverInfo_v390ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_InterRATHandoverInfo_v3a0ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_InterRATHandoverInfo_v3d0ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_InterRATHandoverInfo_v3g0ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_InterRATHandoverInfo_v4b0ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_InterRATHandoverInfo_v4d0ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_InterRATHandoverInfo_v590ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_InterRATHandoverInfo_v6xyext_IEs = -1;
static gint ett_umts_rrc_pdu_def_MeasurementControl = -1;
static gint ett_umts_rrc_pdu_def_T_r311 = -1;
static gint ett_umts_rrc_pdu_def_T_v390nonCriticalExtensions = -1;
static gint ett_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions4 = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions18 = -1;
static gint ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions5 = -1;
static gint ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions8 = -1;
static gint ett_umts_rrc_pdu_def_T_v5b0NonCriticalExtensions = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions31 = -1;
static gint ett_umts_rrc_pdu_def_T_later_than_r39 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions23 = -1;
static gint ett_umts_rrc_pdu_def_T_r43 = -1;
static gint ett_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions3 = -1;
static gint ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions9 = -1;
static gint ett_umts_rrc_pdu_def_T_v5b0NonCriticalExtensions1 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions32 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions24 = -1;
static gint ett_umts_rrc_pdu_def_MeasurementControl_r3_IEs = -1;
static gint ett_umts_rrc_pdu_def_MeasurementControl_v390ext = -1;
static gint ett_umts_rrc_pdu_def_MeasurementControl_v3a0ext = -1;
static gint ett_umts_rrc_pdu_def_MeasurementControl_r4_IEs = -1;
static gint ett_umts_rrc_pdu_def_MeasurementControl_v590ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_T_measurementCommand_v590ext = -1;
static gint ett_umts_rrc_pdu_def_MeasurementControl_v5b0ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_MeasurementControlFailure = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions19 = -1;
static gint ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions10 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions33 = -1;
static gint ett_umts_rrc_pdu_def_MeasurementControlFailure_v590ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_MeasurementReport = -1;
static gint ett_umts_rrc_pdu_def_T_v390nonCriticalExtensions1 = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions20 = -1;
static gint ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions6 = -1;
static gint ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions11 = -1;
static gint ett_umts_rrc_pdu_def_T_v5b0NonCriticalExtensions2 = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions11 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions34 = -1;
static gint ett_umts_rrc_pdu_def_MeasurementReport_v390ext = -1;
static gint ett_umts_rrc_pdu_def_MeasurementReport_v4b0ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_MeasurementReport_v590ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_MeasurementReport_v5b0ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_MeasurementReport_v6xyext_IEs = -1;
static gint ett_umts_rrc_pdu_def_PagingType1 = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions21 = -1;
static gint ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions12 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions35 = -1;
static gint ett_umts_rrc_pdu_def_PagingType1_v590ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_PagingType2 = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions22 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions36 = -1;
static gint ett_umts_rrc_pdu_def_PhysicalChannelReconfiguration = -1;
static gint ett_umts_rrc_pdu_def_T_r312 = -1;
static gint ett_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions5 = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions23 = -1;
static gint ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtenstions = -1;
static gint ett_umts_rrc_pdu_def_T_v590NonCriticalExtenstions2 = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions12 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions37 = -1;
static gint ett_umts_rrc_pdu_def_T_later_than_r310 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions25 = -1;
static gint ett_umts_rrc_pdu_def_T_r44 = -1;
static gint ett_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions4 = -1;
static gint ett_umts_rrc_pdu_def_T_v590NonCriticalExtenstions3 = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions13 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions38 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions26 = -1;
static gint ett_umts_rrc_pdu_def_T_r54 = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions14 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions39 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions27 = -1;
static gint ett_umts_rrc_pdu_def_T_r65 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions40 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions28 = -1;
static gint ett_umts_rrc_pdu_def_PhysicalChannelReconfiguration_r3_IEs = -1;
static gint ett_umts_rrc_pdu_def_T_modeSpecificInfo6 = -1;
static gint ett_umts_rrc_pdu_def_T_fdd12 = -1;
static gint ett_umts_rrc_pdu_def_PhysicalChannelReconfiguration_v3a0ext = -1;
static gint ett_umts_rrc_pdu_def_PhysicalChannelReconfiguration_v4b0ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_PhysicalChannelReconfiguration_v590ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_PhysicalChannelReconfiguration_r4_IEs = -1;
static gint ett_umts_rrc_pdu_def_T_modeSpecificInfo7 = -1;
static gint ett_umts_rrc_pdu_def_T_fdd13 = -1;
static gint ett_umts_rrc_pdu_def_PhysicalChannelReconfiguration_r5_IEs = -1;
static gint ett_umts_rrc_pdu_def_T_modeSpecificInfo8 = -1;
static gint ett_umts_rrc_pdu_def_T_fdd14 = -1;
static gint ett_umts_rrc_pdu_def_PhysicalChannelReconfiguration_r6_IEs = -1;
static gint ett_umts_rrc_pdu_def_PhysicalChannelReconfiguration_v6xyext_IEs = -1;
static gint ett_umts_rrc_pdu_def_PhysicalChannelReconfigurationComplete = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions24 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions41 = -1;
static gint ett_umts_rrc_pdu_def_PhysicalChannelReconfigurationFailure = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions25 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions42 = -1;
static gint ett_umts_rrc_pdu_def_PhysicalSharedChannelAllocation = -1;
static gint ett_umts_rrc_pdu_def_T_r313 = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions26 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions43 = -1;
static gint ett_umts_rrc_pdu_def_T_later_than_r311 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions29 = -1;
static gint ett_umts_rrc_pdu_def_T_r45 = -1;
static gint ett_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions5 = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions15 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions44 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions30 = -1;
static gint ett_umts_rrc_pdu_def_PhysicalSharedChannelAllocation_r3_IEs = -1;
static gint ett_umts_rrc_pdu_def_PhysicalSharedChannelAllocation_r4_IEs = -1;
static gint ett_umts_rrc_pdu_def_PhysicalSharedChannelAllocation_v6xyext_IEs = -1;
static gint ett_umts_rrc_pdu_def_PUSCHCapacityRequest = -1;
static gint ett_umts_rrc_pdu_def_T_allocationConfirmation = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions27 = -1;
static gint ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions13 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions45 = -1;
static gint ett_umts_rrc_pdu_def_PUSCHCapacityRequest_v590ext = -1;
static gint ett_umts_rrc_pdu_def_RadioBearerReconfiguration = -1;
static gint ett_umts_rrc_pdu_def_T_r314 = -1;
static gint ett_umts_rrc_pdu_def_T_v3aoNonCriticalExtensions = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions28 = -1;
static gint ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions7 = -1;
static gint ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions14 = -1;
static gint ett_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions6 = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions16 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions46 = -1;
static gint ett_umts_rrc_pdu_def_T_later_than_r312 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions31 = -1;
static gint ett_umts_rrc_pdu_def_T_r46 = -1;
static gint ett_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions6 = -1;
static gint ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions15 = -1;
static gint ett_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions7 = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions17 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions47 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions32 = -1;
static gint ett_umts_rrc_pdu_def_T_r55 = -1;
static gint ett_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions8 = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions18 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions48 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions33 = -1;
static gint ett_umts_rrc_pdu_def_T_r66 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions49 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions34 = -1;
static gint ett_umts_rrc_pdu_def_RadioBearerReconfiguration_r3_IEs = -1;
static gint ett_umts_rrc_pdu_def_T_dummy1 = -1;
static gint ett_umts_rrc_pdu_def_T_fdd15 = -1;
static gint ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo3 = -1;
static gint ett_umts_rrc_pdu_def_T_fdd16 = -1;
static gint ett_umts_rrc_pdu_def_RadioBearerReconfiguration_v3a0ext = -1;
static gint ett_umts_rrc_pdu_def_RadioBearerReconfiguration_v4b0ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_RadioBearerReconfiguration_v590ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_RadioBearerReconfiguration_v5d0ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_RadioBearerReconfiguration_r4_IEs = -1;
static gint ett_umts_rrc_pdu_def_T_radioBearerReconfiguration_r4_IEs_dummy = -1;
static gint ett_umts_rrc_pdu_def_T_fdd17 = -1;
static gint ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo4 = -1;
static gint ett_umts_rrc_pdu_def_T_fdd18 = -1;
static gint ett_umts_rrc_pdu_def_RadioBearerReconfiguration_r5_IEs = -1;
static gint ett_umts_rrc_pdu_def_T_specificationMode4 = -1;
static gint ett_umts_rrc_pdu_def_T_complete4 = -1;
static gint ett_umts_rrc_pdu_def_T_dummy_1a = -1;
static gint ett_umts_rrc_pdu_def_T_fdd19 = -1;
static gint ett_umts_rrc_pdu_def_T_preconfiguration3 = -1;
static gint ett_umts_rrc_pdu_def_T_preConfigMode3 = -1;
static gint ett_umts_rrc_pdu_def_T_defaultConfig3 = -1;
static gint ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo5 = -1;
static gint ett_umts_rrc_pdu_def_T_fdd20 = -1;
static gint ett_umts_rrc_pdu_def_RadioBearerReconfiguration_r6_IEs = -1;
static gint ett_umts_rrc_pdu_def_T_specificationMode5 = -1;
static gint ett_umts_rrc_pdu_def_T_complete5 = -1;
static gint ett_umts_rrc_pdu_def_T_preconfiguration4 = -1;
static gint ett_umts_rrc_pdu_def_T_preConfigMode4 = -1;
static gint ett_umts_rrc_pdu_def_T_defaultConfig4 = -1;
static gint ett_umts_rrc_pdu_def_RadioBearerReconfiguration_v6xyext_IEs = -1;
static gint ett_umts_rrc_pdu_def_RadioBearerReconfigurationComplete = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions29 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions50 = -1;
static gint ett_umts_rrc_pdu_def_RadioBearerReconfigurationFailure = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions30 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions51 = -1;
static gint ett_umts_rrc_pdu_def_RadioBearerRelease = -1;
static gint ett_umts_rrc_pdu_def_T_r315 = -1;
static gint ett_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions6 = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions31 = -1;
static gint ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions8 = -1;
static gint ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions16 = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions19 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions52 = -1;
static gint ett_umts_rrc_pdu_def_T_later_than_r313 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions35 = -1;
static gint ett_umts_rrc_pdu_def_T_r47 = -1;
static gint ett_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions7 = -1;
static gint ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions17 = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions20 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions53 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions36 = -1;
static gint ett_umts_rrc_pdu_def_T_r56 = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions21 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions54 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions37 = -1;
static gint ett_umts_rrc_pdu_def_T_r67 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions55 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions38 = -1;
static gint ett_umts_rrc_pdu_def_RadioBearerRelease_r3_IEs = -1;
static gint ett_umts_rrc_pdu_def_T_radioBearerRelease_r3_IEs_dummy = -1;
static gint ett_umts_rrc_pdu_def_T_fdd21 = -1;
static gint ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo6 = -1;
static gint ett_umts_rrc_pdu_def_T_fdd22 = -1;
static gint ett_umts_rrc_pdu_def_RadioBearerRelease_v3a0ext = -1;
static gint ett_umts_rrc_pdu_def_RadioBearerRelease_v4b0ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_RadioBearerRelease_v590ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_RadioBearerRelease_r4_IEs = -1;
static gint ett_umts_rrc_pdu_def_T_radioBearerRelease_r4_IEs_dummy = -1;
static gint ett_umts_rrc_pdu_def_T_fdd23 = -1;
static gint ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo7 = -1;
static gint ett_umts_rrc_pdu_def_T_fdd24 = -1;
static gint ett_umts_rrc_pdu_def_RadioBearerRelease_r5_IEs = -1;
static gint ett_umts_rrc_pdu_def_T_radioBearerRelease_r5_IEs_dummy = -1;
static gint ett_umts_rrc_pdu_def_T_fdd25 = -1;
static gint ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo8 = -1;
static gint ett_umts_rrc_pdu_def_T_fdd26 = -1;
static gint ett_umts_rrc_pdu_def_RadioBearerRelease_v6xyext_IEs = -1;
static gint ett_umts_rrc_pdu_def_RadioBearerRelease_r6_IEs = -1;
static gint ett_umts_rrc_pdu_def_RadioBearerReleaseComplete = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions32 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions56 = -1;
static gint ett_umts_rrc_pdu_def_RadioBearerReleaseFailure = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions33 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions57 = -1;
static gint ett_umts_rrc_pdu_def_RadioBearerSetup = -1;
static gint ett_umts_rrc_pdu_def_T_r316 = -1;
static gint ett_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions7 = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions34 = -1;
static gint ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions9 = -1;
static gint ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions18 = -1;
static gint ett_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions9 = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions22 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions58 = -1;
static gint ett_umts_rrc_pdu_def_T_later_than_r314 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions39 = -1;
static gint ett_umts_rrc_pdu_def_T_r48 = -1;
static gint ett_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions8 = -1;
static gint ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions19 = -1;
static gint ett_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions10 = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions23 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions59 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions40 = -1;
static gint ett_umts_rrc_pdu_def_T_r57 = -1;
static gint ett_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions11 = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions24 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions60 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions41 = -1;
static gint ett_umts_rrc_pdu_def_T_r68 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions61 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions42 = -1;
static gint ett_umts_rrc_pdu_def_RadioBearerSetup_r3_IEs = -1;
static gint ett_umts_rrc_pdu_def_T_radioBearerSetup_r3_IEs_dummy = -1;
static gint ett_umts_rrc_pdu_def_T_fdd27 = -1;
static gint ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo9 = -1;
static gint ett_umts_rrc_pdu_def_T_fdd28 = -1;
static gint ett_umts_rrc_pdu_def_RadioBearerSetup_v3a0ext = -1;
static gint ett_umts_rrc_pdu_def_RadioBearerSetup_v4b0ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_RadioBearerSetup_v590ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_RadioBearerSetup_v5d0ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_RadioBearerSetup_r4_IEs = -1;
static gint ett_umts_rrc_pdu_def_T_radioBearerSetup_r4_IEs_dummy = -1;
static gint ett_umts_rrc_pdu_def_T_fdd29 = -1;
static gint ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo10 = -1;
static gint ett_umts_rrc_pdu_def_T_fdd30 = -1;
static gint ett_umts_rrc_pdu_def_RadioBearerSetup_r5_IEs = -1;
static gint ett_umts_rrc_pdu_def_T_radioBearerSetup_r5_IEs_dummy = -1;
static gint ett_umts_rrc_pdu_def_T_fdd31 = -1;
static gint ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo11 = -1;
static gint ett_umts_rrc_pdu_def_T_fdd32 = -1;
static gint ett_umts_rrc_pdu_def_RadioBearerSetup_v6xyext_IEs = -1;
static gint ett_umts_rrc_pdu_def_RadioBearerSetup_r6_IEs = -1;
static gint ett_umts_rrc_pdu_def_T_specificationMode6 = -1;
static gint ett_umts_rrc_pdu_def_T_complete6 = -1;
static gint ett_umts_rrc_pdu_def_T_preconfiguration5 = -1;
static gint ett_umts_rrc_pdu_def_RadioBearerSetupComplete = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions35 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions62 = -1;
static gint ett_umts_rrc_pdu_def_RadioBearerSetupFailure = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions36 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions63 = -1;
static gint ett_umts_rrc_pdu_def_RRCConnectionReject = -1;
static gint ett_umts_rrc_pdu_def_T_r317 = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions37 = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions25 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions64 = -1;
static gint ett_umts_rrc_pdu_def_T_later_than_r315 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions43 = -1;
static gint ett_umts_rrc_pdu_def_RRCConnectionReject_r3_IEs = -1;
static gint ett_umts_rrc_pdu_def_RRCConnectionReject_v6xyext_IEs = -1;
static gint ett_umts_rrc_pdu_def_RRCConnectionRelease = -1;
static gint ett_umts_rrc_pdu_def_T_r318 = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions38 = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions26 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions65 = -1;
static gint ett_umts_rrc_pdu_def_T_later_than_r316 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions44 = -1;
static gint ett_umts_rrc_pdu_def_T_r49 = -1;
static gint ett_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions9 = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions27 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions66 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions45 = -1;
static gint ett_umts_rrc_pdu_def_RRCConnectionRelease_r3_IEs = -1;
static gint ett_umts_rrc_pdu_def_RRCConnectionRelease_r4_IEs = -1;
static gint ett_umts_rrc_pdu_def_RRCConnectionRelease_v6xyext_IEs = -1;
static gint ett_umts_rrc_pdu_def_RRCConnectionRelease_CCCH = -1;
static gint ett_umts_rrc_pdu_def_T_r319 = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions39 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions67 = -1;
static gint ett_umts_rrc_pdu_def_T_later_than_r317 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions46 = -1;
static gint ett_umts_rrc_pdu_def_T_r410 = -1;
static gint ett_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions10 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions68 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions47 = -1;
static gint ett_umts_rrc_pdu_def_SEQUENCE_SIZE_1_maxURNTIGroup_OF_GroupReleaseInformation = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions48 = -1;
static gint ett_umts_rrc_pdu_def_T_r58 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions69 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions49 = -1;
static gint ett_umts_rrc_pdu_def_RRCConnectionRelease_CCCH_r3_IEs = -1;
static gint ett_umts_rrc_pdu_def_RRCConnectionRelease_CCCH_r4_IEs = -1;
static gint ett_umts_rrc_pdu_def_RRCConnectionReleaseComplete = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions40 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions70 = -1;
static gint ett_umts_rrc_pdu_def_RRCConnectionRequest = -1;
static gint ett_umts_rrc_pdu_def_T_v3d0NonCriticalExtensions = -1;
static gint ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions10 = -1;
static gint ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions20 = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions28 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions71 = -1;
static gint ett_umts_rrc_pdu_def_RRCConnectionRequest_v3d0ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_RRCConnectionRequest_v4b0ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_RRCConnectionRequest_v590ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_RRCConnectionRequest_v6xyext_IEs = -1;
static gint ett_umts_rrc_pdu_def_RRCConnectionSetup = -1;
static gint ett_umts_rrc_pdu_def_T_r320 = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions41 = -1;
static gint ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions11 = -1;
static gint ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions21 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions72 = -1;
static gint ett_umts_rrc_pdu_def_T_later_than_r318 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions50 = -1;
static gint ett_umts_rrc_pdu_def_T_r411 = -1;
static gint ett_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions11 = -1;
static gint ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions22 = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions29 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions73 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions51 = -1;
static gint ett_umts_rrc_pdu_def_T_r59 = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions30 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions74 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions52 = -1;
static gint ett_umts_rrc_pdu_def_T_r69 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions75 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions53 = -1;
static gint ett_umts_rrc_pdu_def_RRCConnectionSetup_r3_IEs = -1;
static gint ett_umts_rrc_pdu_def_RRCConnectionSetup_v4b0ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_RRCConnectionSetup_v590ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_RRCConnectionSetup_r4_IEs = -1;
static gint ett_umts_rrc_pdu_def_RRCConnectionSetup_r5_IEs = -1;
static gint ett_umts_rrc_pdu_def_T_specificationMode7 = -1;
static gint ett_umts_rrc_pdu_def_T_complete7 = -1;
static gint ett_umts_rrc_pdu_def_T_preconfiguration6 = -1;
static gint ett_umts_rrc_pdu_def_T_preConfigMode5 = -1;
static gint ett_umts_rrc_pdu_def_T_defaultConfig5 = -1;
static gint ett_umts_rrc_pdu_def_RRCConnectionSetup_v6xyext_IEs = -1;
static gint ett_umts_rrc_pdu_def_RRCConnectionSetup_r6_IEs = -1;
static gint ett_umts_rrc_pdu_def_T_specificationMode8 = -1;
static gint ett_umts_rrc_pdu_def_T_complete8 = -1;
static gint ett_umts_rrc_pdu_def_T_preconfiguration7 = -1;
static gint ett_umts_rrc_pdu_def_T_preConfigMode6 = -1;
static gint ett_umts_rrc_pdu_def_T_defaultConfig6 = -1;
static gint ett_umts_rrc_pdu_def_RRCConnectionSetupComplete = -1;
static gint ett_umts_rrc_pdu_def_T_v370NonCriticalExtensions = -1;
static gint ett_umts_rrc_pdu_def_T_v380NonCriticalExtensions = -1;
static gint ett_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions8 = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions42 = -1;
static gint ett_umts_rrc_pdu_def_T_v3g0NonCriticalExtensions1 = -1;
static gint ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions12 = -1;
static gint ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions23 = -1;
static gint ett_umts_rrc_pdu_def_T_v5c0NonCriticalExtensions = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions31 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions76 = -1;
static gint ett_umts_rrc_pdu_def_RRCConnectionSetupComplete_v370ext = -1;
static gint ett_umts_rrc_pdu_def_RRCConnectionSetupComplete_v380ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_RRCConnectionSetupComplete_v3a0ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_RRCConnectionSetupComplete_v3g0ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_RRCConnectionSetupComplete_r3_add_ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions77 = -1;
static gint ett_umts_rrc_pdu_def_RRCConnectionSetupComplete_v4b0ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_RRCConnectionSetupComplete_v590ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_RRCConnectionSetupComplete_v5c0ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_RRCConnectionSetupComplete_v650ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_RRCConnectionSetupComplete_v6xyext_IEs = -1;
static gint ett_umts_rrc_pdu_def_RRC_FailureInfo = -1;
static gint ett_umts_rrc_pdu_def_T_r321 = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions43 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions78 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions54 = -1;
static gint ett_umts_rrc_pdu_def_RRC_FailureInfo_r3_IEs = -1;
static gint ett_umts_rrc_pdu_def_RRCStatus = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions44 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions79 = -1;
static gint ett_umts_rrc_pdu_def_SecurityModeCommand = -1;
static gint ett_umts_rrc_pdu_def_T_r322 = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions45 = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions32 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions80 = -1;
static gint ett_umts_rrc_pdu_def_T_later_than_r319 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions55 = -1;
static gint ett_umts_rrc_pdu_def_SecurityModeCommand_r3_IEs = -1;
static gint ett_umts_rrc_pdu_def_SecurityModeCommand_v6xyext_IEs = -1;
static gint ett_umts_rrc_pdu_def_SecurityModeComplete = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions46 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions81 = -1;
static gint ett_umts_rrc_pdu_def_SecurityModeFailure = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions47 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions82 = -1;
static gint ett_umts_rrc_pdu_def_SignallingConnectionRelease = -1;
static gint ett_umts_rrc_pdu_def_T_r323 = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions48 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions83 = -1;
static gint ett_umts_rrc_pdu_def_T_later_than_r320 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions56 = -1;
static gint ett_umts_rrc_pdu_def_SignallingConnectionRelease_r3_IEs = -1;
static gint ett_umts_rrc_pdu_def_SignallingConnectionReleaseIndication = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions49 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions84 = -1;
static gint ett_umts_rrc_pdu_def_SystemInformation_BCH = -1;
static gint ett_umts_rrc_pdu_def_T_payload = -1;
static gint ett_umts_rrc_pdu_def_T_lastAndFirst = -1;
static gint ett_umts_rrc_pdu_def_T_lastAndComplete = -1;
static gint ett_umts_rrc_pdu_def_T_lastAndCompleteAndFirst = -1;
static gint ett_umts_rrc_pdu_def_T_completeAndFirst = -1;
static gint ett_umts_rrc_pdu_def_SystemInformation_FACH = -1;
static gint ett_umts_rrc_pdu_def_T_payload1 = -1;
static gint ett_umts_rrc_pdu_def_T_lastAndFirst1 = -1;
static gint ett_umts_rrc_pdu_def_T_lastAndComplete1 = -1;
static gint ett_umts_rrc_pdu_def_T_lastAndCompleteAndFirst1 = -1;
static gint ett_umts_rrc_pdu_def_T_completeAndFirst1 = -1;
static gint ett_umts_rrc_pdu_def_FirstSegment = -1;
static gint ett_umts_rrc_pdu_def_FirstSegmentShort = -1;
static gint ett_umts_rrc_pdu_def_SubsequentSegment = -1;
static gint ett_umts_rrc_pdu_def_LastSegment = -1;
static gint ett_umts_rrc_pdu_def_LastSegmentShort = -1;
static gint ett_umts_rrc_pdu_def_CompleteSIB_List = -1;
static gint ett_umts_rrc_pdu_def_CompleteSIB = -1;
static gint ett_umts_rrc_pdu_def_CompleteSIBshort = -1;
static gint ett_umts_rrc_pdu_def_SystemInformationChangeIndication = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions50 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions85 = -1;
static gint ett_umts_rrc_pdu_def_TransportChannelReconfiguration = -1;
static gint ett_umts_rrc_pdu_def_T_r324 = -1;
static gint ett_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions9 = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions51 = -1;
static gint ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions13 = -1;
static gint ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions24 = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions33 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions86 = -1;
static gint ett_umts_rrc_pdu_def_T_later_than_r321 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions57 = -1;
static gint ett_umts_rrc_pdu_def_T_r412 = -1;
static gint ett_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions12 = -1;
static gint ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions25 = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions34 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions87 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions58 = -1;
static gint ett_umts_rrc_pdu_def_T_r510 = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions35 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions88 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions59 = -1;
static gint ett_umts_rrc_pdu_def_T_r610 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions89 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions60 = -1;
static gint ett_umts_rrc_pdu_def_TransportChannelReconfiguration_r3_IEs = -1;
static gint ett_umts_rrc_pdu_def_T_transportChannelReconfiguration_r3_IEs_dummy = -1;
static gint ett_umts_rrc_pdu_def_T_fdd33 = -1;
static gint ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo12 = -1;
static gint ett_umts_rrc_pdu_def_T_fdd34 = -1;
static gint ett_umts_rrc_pdu_def_TransportChannelReconfiguration_v3a0ext = -1;
static gint ett_umts_rrc_pdu_def_TransportChannelReconfiguration_v4b0ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_TransportChannelReconfiguration_v590ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_TransportChannelReconfiguration_r4_IEs = -1;
static gint ett_umts_rrc_pdu_def_T_transportChannelReconfiguration_r4_IEs_dummy = -1;
static gint ett_umts_rrc_pdu_def_T_fdd35 = -1;
static gint ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo13 = -1;
static gint ett_umts_rrc_pdu_def_T_fdd36 = -1;
static gint ett_umts_rrc_pdu_def_TransportChannelReconfiguration_r5_IEs = -1;
static gint ett_umts_rrc_pdu_def_T_transportChannelReconfiguration_r5_IEs_dummy = -1;
static gint ett_umts_rrc_pdu_def_T_fdd37 = -1;
static gint ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo14 = -1;
static gint ett_umts_rrc_pdu_def_T_fdd38 = -1;
static gint ett_umts_rrc_pdu_def_TransportChannelReconfiguration_v6xyext_IEs = -1;
static gint ett_umts_rrc_pdu_def_TransportChannelReconfiguration_r6_IEs = -1;
static gint ett_umts_rrc_pdu_def_TransportChannelReconfigurationComplete = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions52 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions90 = -1;
static gint ett_umts_rrc_pdu_def_TransportChannelReconfigurationFailure = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions53 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions91 = -1;
static gint ett_umts_rrc_pdu_def_TransportFormatCombinationControl = -1;
static gint ett_umts_rrc_pdu_def_T_modeSpecificInfo9 = -1;
static gint ett_umts_rrc_pdu_def_T_tdd3 = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions54 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions92 = -1;
static gint ett_umts_rrc_pdu_def_TransportFormatCombinationControlFailure = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions55 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions93 = -1;
static gint ett_umts_rrc_pdu_def_UECapabilityEnquiry = -1;
static gint ett_umts_rrc_pdu_def_T_r325 = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions56 = -1;
static gint ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions14 = -1;
static gint ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions26 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions94 = -1;
static gint ett_umts_rrc_pdu_def_T_later_than_r322 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions61 = -1;
static gint ett_umts_rrc_pdu_def_UECapabilityEnquiry_r3_IEs = -1;
static gint ett_umts_rrc_pdu_def_UECapabilityEnquiry_v4b0ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_UECapabilityEnquiry_v590ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_UECapabilityInformation = -1;
static gint ett_umts_rrc_pdu_def_T_v370NonCriticalExtensions1 = -1;
static gint ett_umts_rrc_pdu_def_T_v380NonCriticalExtensions1 = -1;
static gint ett_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions10 = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions57 = -1;
static gint ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions15 = -1;
static gint ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions27 = -1;
static gint ett_umts_rrc_pdu_def_T_v5c0NonCriticalExtensions1 = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions36 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions95 = -1;
static gint ett_umts_rrc_pdu_def_UECapabilityInformation_v370ext = -1;
static gint ett_umts_rrc_pdu_def_UECapabilityInformation_v380ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_UECapabilityInformation_v3a0ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_UECapabilityInformation_r3_add_ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions96 = -1;
static gint ett_umts_rrc_pdu_def_UECapabilityInformation_v4b0ext = -1;
static gint ett_umts_rrc_pdu_def_UECapabilityInformation_v590ext = -1;
static gint ett_umts_rrc_pdu_def_UECapabilityInformation_v5c0ext = -1;
static gint ett_umts_rrc_pdu_def_UECapabilityInformation_v650ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_UECapabilityInformation_v6xyext_IEs = -1;
static gint ett_umts_rrc_pdu_def_UECapabilityInformationConfirm = -1;
static gint ett_umts_rrc_pdu_def_T_r326 = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions58 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions97 = -1;
static gint ett_umts_rrc_pdu_def_T_later_than_r323 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions62 = -1;
static gint ett_umts_rrc_pdu_def_UECapabilityInformationConfirm_r3_IEs = -1;
static gint ett_umts_rrc_pdu_def_UplinkDirectTransfer = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions59 = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions37 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions98 = -1;
static gint ett_umts_rrc_pdu_def_UplinkDirectTransfer_v6xyext_IEs = -1;
static gint ett_umts_rrc_pdu_def_UplinkPhysicalChannelControl = -1;
static gint ett_umts_rrc_pdu_def_T_r327 = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions60 = -1;
static gint ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions16 = -1;
static gint ett_umts_rrc_pdu_def_T_noncriticalExtensions = -1;
static gint ett_umts_rrc_pdu_def_T_later_than_r324 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions63 = -1;
static gint ett_umts_rrc_pdu_def_T_r413 = -1;
static gint ett_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions13 = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions38 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions99 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions64 = -1;
static gint ett_umts_rrc_pdu_def_T_r511 = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions39 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions100 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions65 = -1;
static gint ett_umts_rrc_pdu_def_UplinkPhysicalChannelControl_r3_IEs = -1;
static gint ett_umts_rrc_pdu_def_UplinkPhysicalChannelControl_v4b0ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_UplinkPhysicalChannelControl_r4_IEs = -1;
static gint ett_umts_rrc_pdu_def_T_tddOption = -1;
static gint ett_umts_rrc_pdu_def_T_tdd3842 = -1;
static gint ett_umts_rrc_pdu_def_T_tdd1282 = -1;
static gint ett_umts_rrc_pdu_def_UplinkPhysicalChannelControl_r5_IEs = -1;
static gint ett_umts_rrc_pdu_def_T_tddOption1 = -1;
static gint ett_umts_rrc_pdu_def_T_tdd3843 = -1;
static gint ett_umts_rrc_pdu_def_T_tdd1283 = -1;
static gint ett_umts_rrc_pdu_def_UplinkPhysicalChannelControl_v6xyext_IEs = -1;
static gint ett_umts_rrc_pdu_def_URAUpdate = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions61 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions101 = -1;
static gint ett_umts_rrc_pdu_def_URAUpdateConfirm = -1;
static gint ett_umts_rrc_pdu_def_T_r328 = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions62 = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions40 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions102 = -1;
static gint ett_umts_rrc_pdu_def_T_later_than_r325 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions66 = -1;
static gint ett_umts_rrc_pdu_def_T_r512 = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions41 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions103 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions67 = -1;
static gint ett_umts_rrc_pdu_def_URAUpdateConfirm_r3_IEs = -1;
static gint ett_umts_rrc_pdu_def_URAUpdateConfirm_r5_IEs = -1;
static gint ett_umts_rrc_pdu_def_URAUpdateConfirm_v6xyext_IEs = -1;
static gint ett_umts_rrc_pdu_def_URAUpdateConfirm_CCCH = -1;
static gint ett_umts_rrc_pdu_def_T_r329 = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions63 = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions42 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions104 = -1;
static gint ett_umts_rrc_pdu_def_T_later_than_r326 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions68 = -1;
static gint ett_umts_rrc_pdu_def_URAUpdateConfirm_CCCH_r3_IEs = -1;
static gint ett_umts_rrc_pdu_def_UTRANMobilityInformation = -1;
static gint ett_umts_rrc_pdu_def_T_r330 = -1;
static gint ett_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions11 = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions64 = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions43 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions105 = -1;
static gint ett_umts_rrc_pdu_def_T_later_than_r327 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions69 = -1;
static gint ett_umts_rrc_pdu_def_T_r513 = -1;
static gint ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions44 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions106 = -1;
static gint ett_umts_rrc_pdu_def_T_criticalExtensions70 = -1;
static gint ett_umts_rrc_pdu_def_UTRANMobilityInformation_r3_IEs = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions107 = -1;
static gint ett_umts_rrc_pdu_def_UTRANMobilityInformation_v3a0ext_IEs = -1;
static gint ett_umts_rrc_pdu_def_UTRANMobilityInformation_r5_IEs = -1;
static gint ett_umts_rrc_pdu_def_UtranMobilityInformation_v6xyext_IEs = -1;
static gint ett_umts_rrc_pdu_def_UTRANMobilityInformationConfirm = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions65 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions108 = -1;
static gint ett_umts_rrc_pdu_def_UTRANMobilityInformationFailure = -1;
static gint ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions66 = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions109 = -1;
static gint ett_umts_rrc_pdu_def_MBMSAccessInformation = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions110 = -1;
static gint ett_umts_rrc_pdu_def_MBMSCommonPTMRBInformation = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions111 = -1;
static gint ett_umts_rrc_pdu_def_MBMSCurrentCellPTMRBInformation = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions112 = -1;
static gint ett_umts_rrc_pdu_def_MBMSGeneralInformation = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions113 = -1;
static gint ett_umts_rrc_pdu_def_MBMSModificationRequest = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions114 = -1;
static gint ett_umts_rrc_pdu_def_MBMSModifiedServicesInformation = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions115 = -1;
static gint ett_umts_rrc_pdu_def_MBMSNeighbouringCellPTMRBInformation = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions116 = -1;
static gint ett_umts_rrc_pdu_def_MBMSSchedulingInformation = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions117 = -1;
static gint ett_umts_rrc_pdu_def_MBMSUnmodifiedServicesInformation = -1;
static gint ett_umts_rrc_pdu_def_T_nonCriticalExtensions118 = -1;

/*--- End of included file: packet-umts_rrc_pdu_def-ett.c ---*/
#line 65 "packet-umts_rrc_pdu_def-template.c"


/*--- Included file: packet-umts_rrc_pdu_def-fn.c ---*/
#line 1 "packet-umts_rrc_pdu_def-fn.c"

static const per_sequence_t ActiveSetUpdate_r3_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_dummy_IntegrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_IntegrityProtectionModeInfo },
  { &hf_umts_rrc_pdu_def_dummy2_CipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CipheringModeInfo },
  { &hf_umts_rrc_pdu_def_activationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_newU_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_U_RNTI },
  { &hf_umts_rrc_pdu_def_cn_InformationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CN_InformationInfo },
  { &hf_umts_rrc_pdu_def_dummy3_DL_CounterSynchronisationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CounterSynchronisationInfo },
  { &hf_umts_rrc_pdu_def_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MaxAllowedUL_TX_Power },
  { &hf_umts_rrc_pdu_def_rl_AdditionInformationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RL_AdditionInformationList },
  { &hf_umts_rrc_pdu_def_rl_RemovalInformationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RL_RemovalInformationList },
  { &hf_umts_rrc_pdu_def_tx_DiversityMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_TX_DiversityMode },
  { &hf_umts_rrc_pdu_def_dummy4_SSDT_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_SSDT_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_ActiveSetUpdate_r3_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_ActiveSetUpdate_r3_IEs, ActiveSetUpdate_r3_IEs_sequence);

  return offset;
}



static int
dissect_umts_rrc_pdu_def_BIT_STRING(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     NO_BOUND, NO_BOUND, FALSE, NULL);

  return offset;
}


static const per_sequence_t ActiveSetUpdate_v4b0ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_dummy_SSDT_UL, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_SSDT_UL },
  { &hf_umts_rrc_pdu_def_cell_id_PerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CellIdentity_PerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_ActiveSetUpdate_v4b0ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_ActiveSetUpdate_v4b0ext_IEs, ActiveSetUpdate_v4b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t ActiveSetUpdate_v590ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_dpc_Mode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DPC_Mode },
  { &hf_umts_rrc_pdu_def_dl_TPC_PowerOffsetPerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_TPC_PowerOffsetPerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_ActiveSetUpdate_v590ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_ActiveSetUpdate_v590ext_IEs, ActiveSetUpdate_v590ext_IEs_sequence);

  return offset;
}


static const per_sequence_t ActiveSetUpdate_v6xyext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_primary_plmn_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_PLMN_Identity },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_ActiveSetUpdate_v6xyext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_ActiveSetUpdate_v6xyext_IEs, ActiveSetUpdate_v6xyext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions, T_nonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions_sequence[] = {
  { &hf_umts_rrc_pdu_def_activeSetUpdate_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_ActiveSetUpdate_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions, T_v6xyNonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions_sequence[] = {
  { &hf_umts_rrc_pdu_def_activeSetUpdate_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_ActiveSetUpdate_v590ext_IEs },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions, T_v590NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions_sequence[] = {
  { &hf_umts_rrc_pdu_def_activeSetUpdate_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_ActiveSetUpdate_v4b0ext_IEs },
  { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions, T_v4b0NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions_sequence[] = {
  { &hf_umts_rrc_pdu_def_activeSetUpdate_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions, T_laterNonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_r3_sequence[] = {
  { &hf_umts_rrc_pdu_def_activeSetUpdate_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_ActiveSetUpdate_r3_IEs },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r3, T_r3_sequence);

  return offset;
}


static const per_sequence_t ActiveSetUpdate_r6_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_activationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_newU_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_U_RNTI },
  { &hf_umts_rrc_pdu_def_newH_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_H_RNTI },
  { &hf_umts_rrc_pdu_def_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_E_RNTI },
  { &hf_umts_rrc_pdu_def_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_E_RNTI },
  { &hf_umts_rrc_pdu_def_cn_InformationInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CN_InformationInfo_r6 },
  { &hf_umts_rrc_pdu_def_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MaxAllowedUL_TX_Power },
  { &hf_umts_rrc_pdu_def_rl_AdditionInformationList1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RL_AdditionInformationList_r6 },
  { &hf_umts_rrc_pdu_def_cell_id_PerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CellIdentity_PerRL_List },
  { &hf_umts_rrc_pdu_def_rl_RemovalInformationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RL_RemovalInformationList },
  { &hf_umts_rrc_pdu_def_tx_DiversityMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_TX_DiversityMode },
  { &hf_umts_rrc_pdu_def_dpc_Mode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DPC_Mode },
  { &hf_umts_rrc_pdu_def_serving_HSDSCH_CellInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_Serving_HSDSCH_CellInformation },
  { &hf_umts_rrc_pdu_def_scheduling_E_DCH_CellInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_Scheduling_E_DCH_CellInformation },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_ActiveSetUpdate_r6_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_ActiveSetUpdate_r6_IEs, ActiveSetUpdate_r6_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions1_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions1, T_nonCriticalExtensions1_sequence);

  return offset;
}


static const per_sequence_t T_r6_sequence[] = {
  { &hf_umts_rrc_pdu_def_activeSetUpdate_r6, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_ActiveSetUpdate_r6_IEs },
  { &hf_umts_rrc_pdu_def_activeSetUpdate_r6_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions1 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r6(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r6, T_r6_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions1_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_criticalExtensions1, T_criticalExtensions1_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_criticalExtensions_vals[] = {
  {   0, "r6" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r6 , ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r6 },
  {   1, &hf_umts_rrc_pdu_def_criticalExtensions1, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_criticalExtensions1 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_criticalExtensions, T_criticalExtensions_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_later_than_r3_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_criticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_criticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_later_than_r3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_later_than_r3, T_later_than_r3_sequence);

  return offset;
}


const value_string umts_rrc_pdu_def_ActiveSetUpdate_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t ActiveSetUpdate_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r3 , ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r3 },
  {   1, &hf_umts_rrc_pdu_def_later_than_r3, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_later_than_r3 },
  { 0, NULL, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_ActiveSetUpdate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_ActiveSetUpdate, ActiveSetUpdate_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions2_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions2, T_nonCriticalExtensions2_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions1_sequence[] = {
  { &hf_umts_rrc_pdu_def_activeSetUpdateComplete_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions1, T_laterNonCriticalExtensions1_sequence);

  return offset;
}


static const per_sequence_t ActiveSetUpdateComplete_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_dummy_IntegrityProtActivationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_IntegrityProtActivationInfo },
  { &hf_umts_rrc_pdu_def_dummy2_RB_ActivationTimeInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_ActivationTimeInfoList },
  { &hf_umts_rrc_pdu_def_dummy3_UL_CounterSynchronisationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_CounterSynchronisationInfo },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions1 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_ActiveSetUpdateComplete(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_ActiveSetUpdateComplete, ActiveSetUpdateComplete_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions3_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions3, T_nonCriticalExtensions3_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions2_sequence[] = {
  { &hf_umts_rrc_pdu_def_activeSetUpdateFailure_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions3 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions2, T_laterNonCriticalExtensions2_sequence);

  return offset;
}


static const per_sequence_t ActiveSetUpdateFailure_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_failureCause, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_FailureCauseWithProtErr },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions2 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_ActiveSetUpdateFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_ActiveSetUpdateFailure, ActiveSetUpdateFailure_sequence);

  return offset;
}


static const per_sequence_t AssistanceDataDelivery_r3_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_ue_positioning_GPS_AssistanceData, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UE_Positioning_GPS_AssistanceData },
  { &hf_umts_rrc_pdu_def_ue_positioning_OTDOA_AssistanceData_UEB, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UE_Positioning_OTDOA_AssistanceData_UEB },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_AssistanceDataDelivery_r3_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_AssistanceDataDelivery_r3_IEs, AssistanceDataDelivery_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t AssistanceDataDelivery_v3a0ext_sequence[] = {
  { &hf_umts_rrc_pdu_def_sfn_Offset_Validity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_SFN_Offset_Validity },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_AssistanceDataDelivery_v3a0ext(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_AssistanceDataDelivery_v3a0ext, AssistanceDataDelivery_v3a0ext_sequence);

  return offset;
}


static const per_sequence_t AssistanceDataDelivery_v4b0ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_ue_Positioning_OTDOA_AssistanceData_r4ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UE_Positioning_OTDOA_AssistanceData_r4ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_AssistanceDataDelivery_v4b0ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_AssistanceDataDelivery_v4b0ext_IEs, AssistanceDataDelivery_v4b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions4_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions4(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions4, T_nonCriticalExtensions4_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions1_sequence[] = {
  { &hf_umts_rrc_pdu_def_assistanceDataDelivery_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_AssistanceDataDelivery_v4b0ext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions4, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions1, T_v4b0NonCriticalExtensions1_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions3_sequence[] = {
  { &hf_umts_rrc_pdu_def_assistanceDataDelivery_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions1 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions3, T_laterNonCriticalExtensions3_sequence);

  return offset;
}


static const per_sequence_t T_v3a0NonCriticalExtensions_sequence[] = {
  { &hf_umts_rrc_pdu_def_assistanceDataDelivery_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_AssistanceDataDelivery_v3a0ext },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions3 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions, T_v3a0NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_r31_sequence[] = {
  { &hf_umts_rrc_pdu_def_assistanceDataDelivery_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_AssistanceDataDelivery_r3_IEs },
  { &hf_umts_rrc_pdu_def_v3a0NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r31(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r31, T_r31_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions2_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_criticalExtensions2, T_criticalExtensions2_sequence);

  return offset;
}


static const per_sequence_t T_later_than_r31_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_criticalExtensions2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_criticalExtensions2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_later_than_r31(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_later_than_r31, T_later_than_r31_sequence);

  return offset;
}


const value_string umts_rrc_pdu_def_AssistanceDataDelivery_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t AssistanceDataDelivery_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r31, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r31 },
  {   1, &hf_umts_rrc_pdu_def_later_than_r31, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_later_than_r31 },
  { 0, NULL, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_AssistanceDataDelivery(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_AssistanceDataDelivery, AssistanceDataDelivery_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t CellChangeOrderFromUTRAN_r3_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_dummy_IntegrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_IntegrityProtectionModeInfo },
  { &hf_umts_rrc_pdu_def_activationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_rab_InformationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RAB_InformationList },
  { &hf_umts_rrc_pdu_def_interRAT_TargetCellDescription, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_InterRAT_TargetCellDescription },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_CellChangeOrderFromUTRAN_r3_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_CellChangeOrderFromUTRAN_r3_IEs, CellChangeOrderFromUTRAN_r3_IEs_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_geran_SystemInfoType_vals[] = {
  {   0, "sI" },
  {   1, "pSI" },
  { 0, NULL }
};

static const per_choice_t T_geran_SystemInfoType_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_sI , ASN1_NO_EXTENSIONS     , dissect_umts_rrc_ies_GERAN_SystemInformation },
  {   1, &hf_umts_rrc_pdu_def_pSI, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_ies_GERAN_SystemInformation },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_geran_SystemInfoType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_geran_SystemInfoType, T_geran_SystemInfoType_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t CellChangeOrderFromUTRAN_v590ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_geran_SystemInfoType, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_geran_SystemInfoType },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_CellChangeOrderFromUTRAN_v590ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_CellChangeOrderFromUTRAN_v590ext_IEs, CellChangeOrderFromUTRAN_v590ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions5_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions5(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions5, T_nonCriticalExtensions5_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions1_sequence[] = {
  { &hf_umts_rrc_pdu_def_cellChangeOrderFromUTRAN_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_CellChangeOrderFromUTRAN_v590ext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions5, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions1, T_v590NonCriticalExtensions1_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions4_sequence[] = {
  { &hf_umts_rrc_pdu_def_cellChangeOrderFromUTRAN_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions1 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions4(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions4, T_laterNonCriticalExtensions4_sequence);

  return offset;
}


static const per_sequence_t T_r32_sequence[] = {
  { &hf_umts_rrc_pdu_def_cellChangeOrderFromUTRAN_IEs, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_CellChangeOrderFromUTRAN_r3_IEs },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions4, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r32(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r32, T_r32_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions3_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_criticalExtensions3, T_criticalExtensions3_sequence);

  return offset;
}


static const per_sequence_t T_later_than_r32_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_criticalExtensions3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_criticalExtensions3 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_later_than_r32(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_later_than_r32, T_later_than_r32_sequence);

  return offset;
}


const value_string umts_rrc_pdu_def_CellChangeOrderFromUTRAN_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t CellChangeOrderFromUTRAN_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r32, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r32 },
  {   1, &hf_umts_rrc_pdu_def_later_than_r32, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_later_than_r32 },
  { 0, NULL, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_CellChangeOrderFromUTRAN(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_CellChangeOrderFromUTRAN, CellChangeOrderFromUTRAN_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t CellChangeOrderFromUTRANFailure_r3_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_dummy_IntegrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_IntegrityProtectionModeInfo },
  { &hf_umts_rrc_pdu_def_interRAT_ChangeFailureCause, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_InterRAT_ChangeFailureCause },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_CellChangeOrderFromUTRANFailure_r3_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_CellChangeOrderFromUTRANFailure_r3_IEs, CellChangeOrderFromUTRANFailure_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions6_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions6(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions6, T_nonCriticalExtensions6_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions5_sequence[] = {
  { &hf_umts_rrc_pdu_def_cellChangeOrderFromUTRANFailure_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions6, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions5(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions5, T_laterNonCriticalExtensions5_sequence);

  return offset;
}


static const per_sequence_t T_r33_sequence[] = {
  { &hf_umts_rrc_pdu_def_cellChangeOrderFromUTRANFailure_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_CellChangeOrderFromUTRANFailure_r3_IEs },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions5, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r33(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r33, T_r33_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions4_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions4(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_criticalExtensions4, T_criticalExtensions4_sequence);

  return offset;
}


static const per_sequence_t T_dummy_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_criticalExtensions4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_criticalExtensions4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_dummy(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_dummy, T_dummy_sequence);

  return offset;
}


const value_string umts_rrc_pdu_def_CellChangeOrderFromUTRANFailure_vals[] = {
  {   0, "r3" },
  {   1, "dummy" },
  { 0, NULL }
};

static const per_choice_t CellChangeOrderFromUTRANFailure_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r33, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r33 },
  {   1, &hf_umts_rrc_pdu_def_dummy, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_dummy },
  { 0, NULL, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_CellChangeOrderFromUTRANFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_CellChangeOrderFromUTRANFailure, CellChangeOrderFromUTRANFailure_choice,
                                 NULL);

  return offset;
}



static int
dissect_umts_rrc_pdu_def_BOOLEAN(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_boolean(tvb, offset, actx, tree, hf_index, NULL);

  return offset;
}


static const per_sequence_t CellUpdate_v590ext_sequence[] = {
  { &hf_umts_rrc_pdu_def_establishmentCause, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_EstablishmentCause },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_CellUpdate_v590ext(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_CellUpdate_v590ext, CellUpdate_v590ext_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_trafficVolumeIndicator_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_umts_rrc_pdu_def_T_trafficVolumeIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_reconfigurationStatusIndicator_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_umts_rrc_pdu_def_T_reconfigurationStatusIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t CellUpdate_v6xyext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_cellUpdateCause_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CellUpdateCause_ext },
  { &hf_umts_rrc_pdu_def_trafficVolumeIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_trafficVolumeIndicator },
  { &hf_umts_rrc_pdu_def_measuredResultsOnRACHinterFreq, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MeasuredResultsOnRACHinterFreq },
  { &hf_umts_rrc_pdu_def_reconfigurationStatusIndicator, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_reconfigurationStatusIndicator },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_CellUpdate_v6xyext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_CellUpdate_v6xyext_IEs, CellUpdate_v6xyext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions7_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions7(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions7, T_nonCriticalExtensions7_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions1_sequence[] = {
  { &hf_umts_rrc_pdu_def_cellUpdate_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_CellUpdate_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions7, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions1, T_v6xyNonCriticalExtensions1_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions2_sequence[] = {
  { &hf_umts_rrc_pdu_def_cellUpdate_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_CellUpdate_v590ext },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions1 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions2, T_v590NonCriticalExtensions2_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions6_sequence[] = {
  { &hf_umts_rrc_pdu_def_cellUpdate_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions6(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions6, T_laterNonCriticalExtensions6_sequence);

  return offset;
}


static const per_sequence_t CellUpdate_sequence[] = {
  { &hf_umts_rrc_pdu_def_u_RNTI, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_U_RNTI },
  { &hf_umts_rrc_pdu_def_startList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_STARTList },
  { &hf_umts_rrc_pdu_def_am_RLC_ErrorIndicationRb2_3or4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_BOOLEAN },
  { &hf_umts_rrc_pdu_def_am_RLC_ErrorIndicationRb5orAbove, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_BOOLEAN },
  { &hf_umts_rrc_pdu_def_cellUpdateCause, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_CellUpdateCause },
  { &hf_umts_rrc_pdu_def_failureCause1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_FailureCauseWithProtErrTrId },
  { &hf_umts_rrc_pdu_def_rb_timer_indicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_Rb_timer_indicator },
  { &hf_umts_rrc_pdu_def_measuredResultsOnRACH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MeasuredResultsOnRACH },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions6, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions6 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_CellUpdate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_CellUpdate, CellUpdate_sequence);

  return offset;
}


static const per_sequence_t T_fdd_sequence[] = {
  { &hf_umts_rrc_pdu_def_dummy_CPCH_SetID, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CPCH_SetID },
  { &hf_umts_rrc_pdu_def_dummy2_DRAC_StaticInformationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DRAC_StaticInformationList },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_fdd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_fdd, T_fdd_sequence);

  return offset;
}



static int
dissect_umts_rrc_pdu_def_NULL(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_null(tvb, offset, actx, tree, hf_index);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_modeSpecificTransChInfo_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificTransChInfo_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_fdd, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_fdd },
  {   1, &hf_umts_rrc_pdu_def_tdd, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_modeSpecificTransChInfo(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_modeSpecificTransChInfo, T_modeSpecificTransChInfo_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd1_sequence[] = {
  { &hf_umts_rrc_pdu_def_dummy_DL_PDSCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_PDSCH_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_fdd1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_fdd1, T_fdd1_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_modeSpecificPhysChInfo_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificPhysChInfo_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_fdd1, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_fdd1 },
  {   1, &hf_umts_rrc_pdu_def_tdd, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_modeSpecificPhysChInfo(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo, T_modeSpecificPhysChInfo_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t CellUpdateConfirm_r3_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_IntegrityProtectionModeInfo },
  { &hf_umts_rrc_pdu_def_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CipheringModeInfo },
  { &hf_umts_rrc_pdu_def_activationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_new_U_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_U_RNTI },
  { &hf_umts_rrc_pdu_def_new_C_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_C_RNTI },
  { &hf_umts_rrc_pdu_def_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_StateIndicator },
  { &hf_umts_rrc_pdu_def_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_umts_rrc_pdu_def_rlc_Re_establishIndicatorRb2_3or4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_BOOLEAN },
  { &hf_umts_rrc_pdu_def_rlc_Re_establishIndicatorRb5orAbove, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_BOOLEAN },
  { &hf_umts_rrc_pdu_def_cn_InformationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CN_InformationInfo },
  { &hf_umts_rrc_pdu_def_ura_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_URA_Identity },
  { &hf_umts_rrc_pdu_def_rb_InformationReleaseList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_InformationReleaseList },
  { &hf_umts_rrc_pdu_def_rb_InformationReconfigList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_InformationReconfigList },
  { &hf_umts_rrc_pdu_def_rb_InformationAffectedList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_InformationAffectedList },
  { &hf_umts_rrc_pdu_def_dl_CounterSynchronisationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CounterSynchronisationInfo },
  { &hf_umts_rrc_pdu_def_ul_CommonTransChInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_CommonTransChInfo },
  { &hf_umts_rrc_pdu_def_ul_deletedTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_DeletedTransChInfoList },
  { &hf_umts_rrc_pdu_def_ul_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_AddReconfTransChInfoList },
  { &hf_umts_rrc_pdu_def_modeSpecificTransChInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_modeSpecificTransChInfo },
  { &hf_umts_rrc_pdu_def_dl_CommonTransChInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonTransChInfo },
  { &hf_umts_rrc_pdu_def_dl_DeletedTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_DeletedTransChInfoList },
  { &hf_umts_rrc_pdu_def_dl_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_AddReconfTransChInfoList },
  { &hf_umts_rrc_pdu_def_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_FrequencyInfo },
  { &hf_umts_rrc_pdu_def_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MaxAllowedUL_TX_Power },
  { &hf_umts_rrc_pdu_def_ul_ChannelRequirement, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_ChannelRequirement },
  { &hf_umts_rrc_pdu_def_modeSpecificPhysChInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_modeSpecificPhysChInfo },
  { &hf_umts_rrc_pdu_def_dl_CommonInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonInformation },
  { &hf_umts_rrc_pdu_def_dl_InformationPerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_InformationPerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_CellUpdateConfirm_r3_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_CellUpdateConfirm_r3_IEs, CellUpdateConfirm_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t CellUpdateConfirm_v3a0ext_sequence[] = {
  { &hf_umts_rrc_pdu_def_new_DSCH_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DSCH_RNTI },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_CellUpdateConfirm_v3a0ext(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_CellUpdateConfirm_v3a0ext, CellUpdateConfirm_v3a0ext_sequence);

  return offset;
}


static const per_sequence_t CellUpdateConfirm_v4b0ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_dummy_SSDT_UL, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_SSDT_UL },
  { &hf_umts_rrc_pdu_def_cell_id_PerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CellIdentity_PerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_CellUpdateConfirm_v4b0ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_CellUpdateConfirm_v4b0ext_IEs, CellUpdateConfirm_v4b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t CellUpdateConfirm_v590ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_dl_TPC_PowerOffsetPerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_TPC_PowerOffsetPerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_CellUpdateConfirm_v590ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_CellUpdateConfirm_v590ext_IEs, CellUpdateConfirm_v590ext_IEs_sequence);

  return offset;
}


static const per_sequence_t CellUpdateConfirm_v5d0ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_pdcp_ROHC_TargetMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_PDCP_ROHC_TargetMode },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_CellUpdateConfirm_v5d0ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_CellUpdateConfirm_v5d0ext_IEs, CellUpdateConfirm_v5d0ext_IEs_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_postVerificationPeriod_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_umts_rrc_pdu_def_T_postVerificationPeriod(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t CellUpdateConfirm_v6xyext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_primary_plmn_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_PLMN_Identity },
  { &hf_umts_rrc_pdu_def_harq_Preamble_Mode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_HARQ_Preamble_Mode },
  { &hf_umts_rrc_pdu_def_beaconPLEst, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_BEACON_PL_Est },
  { &hf_umts_rrc_pdu_def_postVerificationPeriod, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_postVerificationPeriod },
  { &hf_umts_rrc_pdu_def_dhs_sync, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DHS_Sync },
  { &hf_umts_rrc_pdu_def_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MBMS_PL_ServiceRestrictInfo_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_CellUpdateConfirm_v6xyext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_CellUpdateConfirm_v6xyext_IEs, CellUpdateConfirm_v6xyext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions8_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions8(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions8, T_nonCriticalExtensions8_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions2_sequence[] = {
  { &hf_umts_rrc_pdu_def_cellUpdateConfirm_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_CellUpdateConfirm_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions8, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions2, T_v6xyNonCriticalExtensions2_sequence);

  return offset;
}


static const per_sequence_t T_v5d0NonCriticalExtenstions_sequence[] = {
  { &hf_umts_rrc_pdu_def_cellUpdateConfirm_v5d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_CellUpdateConfirm_v5d0ext_IEs },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions, T_v5d0NonCriticalExtenstions_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtenstions_sequence[] = {
  { &hf_umts_rrc_pdu_def_cellUpdateConfirm_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_CellUpdateConfirm_v590ext_IEs },
  { &hf_umts_rrc_pdu_def_v5d0NonCriticalExtenstions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v590NonCriticalExtenstions(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v590NonCriticalExtenstions, T_v590NonCriticalExtenstions_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions2_sequence[] = {
  { &hf_umts_rrc_pdu_def_cellUpdateConfirm_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_CellUpdateConfirm_v4b0ext_IEs },
  { &hf_umts_rrc_pdu_def_v590NonCriticalExtenstions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v590NonCriticalExtenstions },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions2, T_v4b0NonCriticalExtensions2_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions7_sequence[] = {
  { &hf_umts_rrc_pdu_def_cellUpdateConfirm_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions7(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions7, T_laterNonCriticalExtensions7_sequence);

  return offset;
}


static const per_sequence_t T_v3a0NonCriticalExtensions1_sequence[] = {
  { &hf_umts_rrc_pdu_def_cellUpdateConfirm_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_CellUpdateConfirm_v3a0ext },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions7, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions1, T_v3a0NonCriticalExtensions1_sequence);

  return offset;
}


static const per_sequence_t T_r34_sequence[] = {
  { &hf_umts_rrc_pdu_def_cellUpdateConfirm_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_CellUpdateConfirm_r3_IEs },
  { &hf_umts_rrc_pdu_def_v3a0NonCriticalExtensions1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions1 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r34(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r34, T_r34_sequence);

  return offset;
}


static const per_sequence_t T_fdd2_sequence[] = {
  { &hf_umts_rrc_pdu_def_dummy_CPCH_SetID, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CPCH_SetID },
  { &hf_umts_rrc_pdu_def_dummy2_DRAC_StaticInformationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DRAC_StaticInformationList },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_fdd2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_fdd2, T_fdd2_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_modeSpecificTransChInfo1_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificTransChInfo1_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_fdd2, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_fdd2 },
  {   1, &hf_umts_rrc_pdu_def_tdd, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_modeSpecificTransChInfo1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_modeSpecificTransChInfo1, T_modeSpecificTransChInfo1_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd3_sequence[] = {
  { &hf_umts_rrc_pdu_def_dummy_DL_PDSCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_PDSCH_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_fdd3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_fdd3, T_fdd3_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_modeSpecificPhysChInfo1_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificPhysChInfo1_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_fdd3, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_fdd3 },
  {   1, &hf_umts_rrc_pdu_def_tdd, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_modeSpecificPhysChInfo1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo1, T_modeSpecificPhysChInfo1_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t CellUpdateConfirm_r4_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_IntegrityProtectionModeInfo },
  { &hf_umts_rrc_pdu_def_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CipheringModeInfo },
  { &hf_umts_rrc_pdu_def_activationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_new_U_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_U_RNTI },
  { &hf_umts_rrc_pdu_def_new_C_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_C_RNTI },
  { &hf_umts_rrc_pdu_def_new_DSCH_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DSCH_RNTI },
  { &hf_umts_rrc_pdu_def_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_StateIndicator },
  { &hf_umts_rrc_pdu_def_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_umts_rrc_pdu_def_rlc_Re_establishIndicatorRb2_3or4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_BOOLEAN },
  { &hf_umts_rrc_pdu_def_rlc_Re_establishIndicatorRb5orAbove, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_BOOLEAN },
  { &hf_umts_rrc_pdu_def_cn_InformationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CN_InformationInfo },
  { &hf_umts_rrc_pdu_def_ura_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_URA_Identity },
  { &hf_umts_rrc_pdu_def_rb_InformationReleaseList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_InformationReleaseList },
  { &hf_umts_rrc_pdu_def_rb_InformationReconfigList1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_InformationReconfigList_r4 },
  { &hf_umts_rrc_pdu_def_rb_InformationAffectedList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_InformationAffectedList },
  { &hf_umts_rrc_pdu_def_dl_CounterSynchronisationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CounterSynchronisationInfo },
  { &hf_umts_rrc_pdu_def_ul_CommonTransChInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_CommonTransChInfo_r4 },
  { &hf_umts_rrc_pdu_def_ul_deletedTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_DeletedTransChInfoList },
  { &hf_umts_rrc_pdu_def_ul_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_AddReconfTransChInfoList },
  { &hf_umts_rrc_pdu_def_modeSpecificTransChInfo1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_modeSpecificTransChInfo1 },
  { &hf_umts_rrc_pdu_def_dl_CommonTransChInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonTransChInfo_r4 },
  { &hf_umts_rrc_pdu_def_dl_DeletedTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_DeletedTransChInfoList },
  { &hf_umts_rrc_pdu_def_dl_AddReconfTransChInfoList1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_AddReconfTransChInfoList_r4 },
  { &hf_umts_rrc_pdu_def_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_FrequencyInfo },
  { &hf_umts_rrc_pdu_def_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MaxAllowedUL_TX_Power },
  { &hf_umts_rrc_pdu_def_ul_ChannelRequirement1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_ChannelRequirement_r4 },
  { &hf_umts_rrc_pdu_def_modeSpecificPhysChInfo1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_modeSpecificPhysChInfo1 },
  { &hf_umts_rrc_pdu_def_dl_CommonInformation1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonInformation_r4 },
  { &hf_umts_rrc_pdu_def_dl_InformationPerRL_List1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_InformationPerRL_List_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_CellUpdateConfirm_r4_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_CellUpdateConfirm_r4_IEs, CellUpdateConfirm_r4_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions9_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions9(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions9, T_nonCriticalExtensions9_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions3_sequence[] = {
  { &hf_umts_rrc_pdu_def_cellUpdateConfirm_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_CellUpdateConfirm_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions9, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions3, T_v6xyNonCriticalExtensions3_sequence);

  return offset;
}


static const per_sequence_t T_v5d0NonCriticalExtenstions1_sequence[] = {
  { &hf_umts_rrc_pdu_def_cellUpdateConfirm_v5d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_CellUpdateConfirm_v5d0ext_IEs },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions3 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions1, T_v5d0NonCriticalExtenstions1_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtenstions1_sequence[] = {
  { &hf_umts_rrc_pdu_def_cellUpdateConfirm_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_CellUpdateConfirm_v590ext_IEs },
  { &hf_umts_rrc_pdu_def_v5d0NonCriticalExtenstions1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions1 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v590NonCriticalExtenstions1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v590NonCriticalExtenstions1, T_v590NonCriticalExtenstions1_sequence);

  return offset;
}


static const per_sequence_t T_v4d0NonCriticalExtensions_sequence[] = {
  { &hf_umts_rrc_pdu_def_cellUpdateConfirm_r4_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v590NonCriticalExtenstions1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v590NonCriticalExtenstions1 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions, T_v4d0NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_r4_sequence[] = {
  { &hf_umts_rrc_pdu_def_cellUpdateConfirm_r4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_CellUpdateConfirm_r4_IEs },
  { &hf_umts_rrc_pdu_def_v4d0NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r4(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r4, T_r4_sequence);

  return offset;
}


static const per_sequence_t T_fdd4_sequence[] = {
  { &hf_umts_rrc_pdu_def_dummy_CPCH_SetID, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CPCH_SetID },
  { &hf_umts_rrc_pdu_def_dummy2_DRAC_StaticInformationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DRAC_StaticInformationList },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_fdd4(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_fdd4, T_fdd4_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_modeSpecificTransChInfo2_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificTransChInfo2_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_fdd4, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_fdd4 },
  {   1, &hf_umts_rrc_pdu_def_tdd, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_modeSpecificTransChInfo2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_modeSpecificTransChInfo2, T_modeSpecificTransChInfo2_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd5_sequence[] = {
  { &hf_umts_rrc_pdu_def_dummy_DL_PDSCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_PDSCH_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_fdd5(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_fdd5, T_fdd5_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_modeSpecificPhysChInfo2_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificPhysChInfo2_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_fdd5, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_fdd5 },
  {   1, &hf_umts_rrc_pdu_def_tdd, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_modeSpecificPhysChInfo2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo2, T_modeSpecificPhysChInfo2_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t CellUpdateConfirm_r5_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_IntegrityProtectionModeInfo },
  { &hf_umts_rrc_pdu_def_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CipheringModeInfo },
  { &hf_umts_rrc_pdu_def_activationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_new_U_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_U_RNTI },
  { &hf_umts_rrc_pdu_def_new_C_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_C_RNTI },
  { &hf_umts_rrc_pdu_def_new_DSCH_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DSCH_RNTI },
  { &hf_umts_rrc_pdu_def_new_H_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_H_RNTI },
  { &hf_umts_rrc_pdu_def_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_StateIndicator },
  { &hf_umts_rrc_pdu_def_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_umts_rrc_pdu_def_rlc_Re_establishIndicatorRb2_3or4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_BOOLEAN },
  { &hf_umts_rrc_pdu_def_rlc_Re_establishIndicatorRb5orAbove, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_BOOLEAN },
  { &hf_umts_rrc_pdu_def_cn_InformationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CN_InformationInfo },
  { &hf_umts_rrc_pdu_def_ura_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_URA_Identity },
  { &hf_umts_rrc_pdu_def_rb_InformationReleaseList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_InformationReleaseList },
  { &hf_umts_rrc_pdu_def_rb_InformationReconfigList2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_InformationReconfigList_r5 },
  { &hf_umts_rrc_pdu_def_rb_InformationAffectedList1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_InformationAffectedList_r5 },
  { &hf_umts_rrc_pdu_def_dl_CounterSynchronisationInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CounterSynchronisationInfo_r5 },
  { &hf_umts_rrc_pdu_def_ul_CommonTransChInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_CommonTransChInfo_r4 },
  { &hf_umts_rrc_pdu_def_ul_deletedTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_DeletedTransChInfoList },
  { &hf_umts_rrc_pdu_def_ul_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_AddReconfTransChInfoList },
  { &hf_umts_rrc_pdu_def_modeSpecificTransChInfo2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_modeSpecificTransChInfo2 },
  { &hf_umts_rrc_pdu_def_dl_CommonTransChInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonTransChInfo_r4 },
  { &hf_umts_rrc_pdu_def_dl_DeletedTransChInfoList1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_DeletedTransChInfoList_r5 },
  { &hf_umts_rrc_pdu_def_dl_AddReconfTransChInfoList2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_AddReconfTransChInfoList_r5 },
  { &hf_umts_rrc_pdu_def_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_FrequencyInfo },
  { &hf_umts_rrc_pdu_def_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MaxAllowedUL_TX_Power },
  { &hf_umts_rrc_pdu_def_ul_ChannelRequirement2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_ChannelRequirement_r5 },
  { &hf_umts_rrc_pdu_def_modeSpecificPhysChInfo2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_modeSpecificPhysChInfo2 },
  { &hf_umts_rrc_pdu_def_dl_HSPDSCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_HSPDSCH_Information },
  { &hf_umts_rrc_pdu_def_dl_CommonInformation2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonInformation_r5 },
  { &hf_umts_rrc_pdu_def_dl_InformationPerRL_List2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_InformationPerRL_List_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_CellUpdateConfirm_r5_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_CellUpdateConfirm_r5_IEs, CellUpdateConfirm_r5_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions10_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions10(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions10, T_nonCriticalExtensions10_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions4_sequence[] = {
  { &hf_umts_rrc_pdu_def_cellUpdateConfirm_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_CellUpdateConfirm_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions10, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions4(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions4, T_v6xyNonCriticalExtensions4_sequence);

  return offset;
}


static const per_sequence_t T_v5d0NonCriticalExtenstions2_sequence[] = {
  { &hf_umts_rrc_pdu_def_cellUpdateConfirm_v5d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_CellUpdateConfirm_v5d0ext_IEs },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions4, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions2, T_v5d0NonCriticalExtenstions2_sequence);

  return offset;
}


static const per_sequence_t T_r5_sequence[] = {
  { &hf_umts_rrc_pdu_def_cellUpdateConfirm_r5, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_CellUpdateConfirm_r5_IEs },
  { &hf_umts_rrc_pdu_def_cellUpdateConfirm_r5_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v5d0NonCriticalExtenstions2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r5(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r5, T_r5_sequence);

  return offset;
}


static const per_sequence_t CellUpdateConfirm_r6_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_IntegrityProtectionModeInfo },
  { &hf_umts_rrc_pdu_def_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CipheringModeInfo },
  { &hf_umts_rrc_pdu_def_activationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_new_U_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_U_RNTI },
  { &hf_umts_rrc_pdu_def_new_C_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_C_RNTI },
  { &hf_umts_rrc_pdu_def_new_DSCH_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DSCH_RNTI },
  { &hf_umts_rrc_pdu_def_new_H_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_H_RNTI },
  { &hf_umts_rrc_pdu_def_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_E_RNTI },
  { &hf_umts_rrc_pdu_def_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_E_RNTI },
  { &hf_umts_rrc_pdu_def_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_StateIndicator },
  { &hf_umts_rrc_pdu_def_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_umts_rrc_pdu_def_rlc_Re_establishIndicatorRb2_3or4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_BOOLEAN },
  { &hf_umts_rrc_pdu_def_rlc_Re_establishIndicatorRb5orAbove, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_BOOLEAN },
  { &hf_umts_rrc_pdu_def_cn_InformationInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CN_InformationInfo_r6 },
  { &hf_umts_rrc_pdu_def_ura_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_URA_Identity },
  { &hf_umts_rrc_pdu_def_rb_InformationReleaseList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_InformationReleaseList },
  { &hf_umts_rrc_pdu_def_rb_InformationReconfigList3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_InformationReconfigList_r6 },
  { &hf_umts_rrc_pdu_def_rb_InformationAffectedList2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_InformationAffectedList_r6 },
  { &hf_umts_rrc_pdu_def_dl_CounterSynchronisationInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CounterSynchronisationInfo_r5 },
  { &hf_umts_rrc_pdu_def_pdcp_ROHC_TargetMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_PDCP_ROHC_TargetMode },
  { &hf_umts_rrc_pdu_def_ul_CommonTransChInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_CommonTransChInfo_r4 },
  { &hf_umts_rrc_pdu_def_ul_deletedTransChInfoList1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_DeletedTransChInfoList_r6 },
  { &hf_umts_rrc_pdu_def_ul_AddReconfTransChInfoList1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_AddReconfTransChInfoList_r6 },
  { &hf_umts_rrc_pdu_def_dl_CommonTransChInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonTransChInfo_r4 },
  { &hf_umts_rrc_pdu_def_dl_DeletedTransChInfoList1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_DeletedTransChInfoList_r5 },
  { &hf_umts_rrc_pdu_def_dl_AddReconfTransChInfoList2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_AddReconfTransChInfoList_r5 },
  { &hf_umts_rrc_pdu_def_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_FrequencyInfo },
  { &hf_umts_rrc_pdu_def_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MaxAllowedUL_TX_Power },
  { &hf_umts_rrc_pdu_def_ul_ChannelRequirement3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_ChannelRequirement_r6 },
  { &hf_umts_rrc_pdu_def_ul_EDCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_EDCH_Information_r6 },
  { &hf_umts_rrc_pdu_def_dl_HSPDSCH_Information1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_HSPDSCH_Information_r6 },
  { &hf_umts_rrc_pdu_def_dl_CommonInformation3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonInformation_r6 },
  { &hf_umts_rrc_pdu_def_dl_InformationPerRL_List3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_InformationPerRL_List_r6 },
  { &hf_umts_rrc_pdu_def_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MBMS_PL_ServiceRestrictInfo_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_CellUpdateConfirm_r6_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_CellUpdateConfirm_r6_IEs, CellUpdateConfirm_r6_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions11_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions11(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions11, T_nonCriticalExtensions11_sequence);

  return offset;
}


static const per_sequence_t T_r61_sequence[] = {
  { &hf_umts_rrc_pdu_def_cellUpdateConfirm_r6, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_CellUpdateConfirm_r6_IEs },
  { &hf_umts_rrc_pdu_def_cellUpdateConfirm_r6_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions11, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r61(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r61, T_r61_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions8_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions8(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_criticalExtensions8, T_criticalExtensions8_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_criticalExtensions7_vals[] = {
  {   0, "r6" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions7_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r61, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r61 },
  {   1, &hf_umts_rrc_pdu_def_criticalExtensions8, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_criticalExtensions8 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions7(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_criticalExtensions7, T_criticalExtensions7_choice,
                                 NULL);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_criticalExtensions6_vals[] = {
  {   0, "r5" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions6_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r5 , ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r5 },
  {   1, &hf_umts_rrc_pdu_def_criticalExtensions7, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_criticalExtensions7 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions6(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_criticalExtensions6, T_criticalExtensions6_choice,
                                 NULL);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_criticalExtensions5_vals[] = {
  {   0, "r4" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions5_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r4 , ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r4 },
  {   1, &hf_umts_rrc_pdu_def_criticalExtensions6, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_criticalExtensions6 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions5(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_criticalExtensions5, T_criticalExtensions5_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_later_than_r33_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_criticalExtensions5, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_criticalExtensions5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_later_than_r33(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_later_than_r33, T_later_than_r33_sequence);

  return offset;
}


const value_string umts_rrc_pdu_def_CellUpdateConfirm_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t CellUpdateConfirm_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r34, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r34 },
  {   1, &hf_umts_rrc_pdu_def_later_than_r33, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_later_than_r33 },
  { 0, NULL, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_CellUpdateConfirm(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_CellUpdateConfirm, CellUpdateConfirm_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions12_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions12(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions12, T_nonCriticalExtensions12_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions5_sequence[] = {
  { &hf_umts_rrc_pdu_def_cellUpdateConfirm_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_CellUpdateConfirm_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions12, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions12 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions5(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions5, T_v6xyNonCriticalExtensions5_sequence);

  return offset;
}


static const per_sequence_t T_v5d0NonCriticalExtenstions3_sequence[] = {
  { &hf_umts_rrc_pdu_def_cellUpdateConfirm_v5d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_CellUpdateConfirm_v5d0ext_IEs },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions5, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions3, T_v5d0NonCriticalExtenstions3_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions3_sequence[] = {
  { &hf_umts_rrc_pdu_def_cellUpdateConfirm_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_CellUpdateConfirm_v590ext_IEs },
  { &hf_umts_rrc_pdu_def_v5d0NonCriticalExtenstions3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions3 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions3, T_v590NonCriticalExtensions3_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions3_sequence[] = {
  { &hf_umts_rrc_pdu_def_cellUpdateConfirm_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_CellUpdateConfirm_v4b0ext_IEs },
  { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions3 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions3, T_v4b0NonCriticalExtensions3_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions8_sequence[] = {
  { &hf_umts_rrc_pdu_def_cellUpdateConfirm_CCCH_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions3 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions8(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions8, T_laterNonCriticalExtensions8_sequence);

  return offset;
}


static const per_sequence_t T_r35_sequence[] = {
  { &hf_umts_rrc_pdu_def_u_RNTI, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_U_RNTI },
  { &hf_umts_rrc_pdu_def_cellUpdateConfirm_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_CellUpdateConfirm_r3_IEs },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions8, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r35(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r35, T_r35_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions13_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions13(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions13, T_nonCriticalExtensions13_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions6_sequence[] = {
  { &hf_umts_rrc_pdu_def_cellUpdateConfirm_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_CellUpdateConfirm_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions13, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions13 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions6(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions6, T_v6xyNonCriticalExtensions6_sequence);

  return offset;
}


static const per_sequence_t T_v5d0NonCriticalExtenstions4_sequence[] = {
  { &hf_umts_rrc_pdu_def_cellUpdateConfirm_v5d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_CellUpdateConfirm_v5d0ext_IEs },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions6, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions4(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions4, T_v5d0NonCriticalExtenstions4_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions4_sequence[] = {
  { &hf_umts_rrc_pdu_def_cellUpdateConfirm_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_CellUpdateConfirm_v590ext_IEs },
  { &hf_umts_rrc_pdu_def_v5d0NonCriticalExtenstions4, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions4(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions4, T_v590NonCriticalExtensions4_sequence);

  return offset;
}


static const per_sequence_t T_v4d0NonCriticalExtensions1_sequence[] = {
  { &hf_umts_rrc_pdu_def_cellUpdateConfirm_CCCH_r4_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions4, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions1, T_v4d0NonCriticalExtensions1_sequence);

  return offset;
}


static const per_sequence_t T_r41_sequence[] = {
  { &hf_umts_rrc_pdu_def_cellUpdateConfirm_r4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_CellUpdateConfirm_r4_IEs },
  { &hf_umts_rrc_pdu_def_v4d0NonCriticalExtensions1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions1 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r41(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r41, T_r41_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions14_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions14(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions14, T_nonCriticalExtensions14_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions7_sequence[] = {
  { &hf_umts_rrc_pdu_def_cellUpdateConfirm_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_CellUpdateConfirm_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions14, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions14 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions7(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions7, T_v6xyNonCriticalExtensions7_sequence);

  return offset;
}


static const per_sequence_t T_v5d0NonCriticalExtenstions5_sequence[] = {
  { &hf_umts_rrc_pdu_def_cellUpdateConfirm_v5d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_CellUpdateConfirm_v5d0ext_IEs },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions7, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions5(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions5, T_v5d0NonCriticalExtenstions5_sequence);

  return offset;
}


static const per_sequence_t T_r51_sequence[] = {
  { &hf_umts_rrc_pdu_def_cellUpdateConfirm_r5, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_CellUpdateConfirm_r5_IEs },
  { &hf_umts_rrc_pdu_def_cellUpdateConfirm_CCCH_r5_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v5d0NonCriticalExtenstions5, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r51(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r51, T_r51_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions15_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions15(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions15, T_nonCriticalExtensions15_sequence);

  return offset;
}


static const per_sequence_t T_r62_sequence[] = {
  { &hf_umts_rrc_pdu_def_cellUpdateConfirm_r6, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_CellUpdateConfirm_r6_IEs },
  { &hf_umts_rrc_pdu_def_cellUpdateConfirm_r6_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions15, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions15 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r62(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r62, T_r62_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions12_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions12(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_criticalExtensions12, T_criticalExtensions12_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_criticalExtensions11_vals[] = {
  {   0, "r6" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions11_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r62, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r62 },
  {   1, &hf_umts_rrc_pdu_def_criticalExtensions12, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_criticalExtensions12 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions11(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_criticalExtensions11, T_criticalExtensions11_choice,
                                 NULL);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_criticalExtensions10_vals[] = {
  {   0, "r5" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions10_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r51, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r51 },
  {   1, &hf_umts_rrc_pdu_def_criticalExtensions11, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_criticalExtensions11 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions10(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_criticalExtensions10, T_criticalExtensions10_choice,
                                 NULL);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_criticalExtensions9_vals[] = {
  {   0, "r4" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions9_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r41, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r41 },
  {   1, &hf_umts_rrc_pdu_def_criticalExtensions10, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_criticalExtensions10 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions9(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_criticalExtensions9, T_criticalExtensions9_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_later_than_r34_sequence[] = {
  { &hf_umts_rrc_pdu_def_u_RNTI, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_U_RNTI },
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_criticalExtensions9, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_criticalExtensions9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_later_than_r34(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_later_than_r34, T_later_than_r34_sequence);

  return offset;
}


const value_string umts_rrc_pdu_def_CellUpdateConfirm_CCCH_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t CellUpdateConfirm_CCCH_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r35, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r35 },
  {   1, &hf_umts_rrc_pdu_def_later_than_r34, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_later_than_r34 },
  { 0, NULL, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_CellUpdateConfirm_CCCH(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_CellUpdateConfirm_CCCH, CellUpdateConfirm_CCCH_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t CounterCheck_r3_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_rb_COUNT_C_MSB_InformationList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RB_COUNT_C_MSB_InformationList },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_CounterCheck_r3_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_CounterCheck_r3_IEs, CounterCheck_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions16_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions16(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions16, T_nonCriticalExtensions16_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions9_sequence[] = {
  { &hf_umts_rrc_pdu_def_counterCheck_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions16, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions16 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions9(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions9, T_laterNonCriticalExtensions9_sequence);

  return offset;
}


static const per_sequence_t T_r36_sequence[] = {
  { &hf_umts_rrc_pdu_def_counterCheck_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_CounterCheck_r3_IEs },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions9, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r36(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r36, T_r36_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions13_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions13(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_criticalExtensions13, T_criticalExtensions13_sequence);

  return offset;
}


static const per_sequence_t T_later_than_r35_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_criticalExtensions13, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_criticalExtensions13 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_later_than_r35(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_later_than_r35, T_later_than_r35_sequence);

  return offset;
}


const value_string umts_rrc_pdu_def_CounterCheck_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t CounterCheck_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r36, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r36 },
  {   1, &hf_umts_rrc_pdu_def_later_than_r35, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_later_than_r35 },
  { 0, NULL, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_CounterCheck(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_CounterCheck, CounterCheck_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions17_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions17(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions17, T_nonCriticalExtensions17_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions10_sequence[] = {
  { &hf_umts_rrc_pdu_def_counterCheckResponse_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions17, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions17 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions10(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions10, T_laterNonCriticalExtensions10_sequence);

  return offset;
}


static const per_sequence_t CounterCheckResponse_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_rb_COUNT_C_InformationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_COUNT_C_InformationList },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions10, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions10 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_CounterCheckResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_CounterCheckResponse, CounterCheckResponse_sequence);

  return offset;
}


static const per_sequence_t DownlinkDirectTransfer_r3_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_cn_DomainIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_CN_DomainIdentity },
  { &hf_umts_rrc_pdu_def_nas_Message, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_NAS_Message },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_DownlinkDirectTransfer_r3_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_DownlinkDirectTransfer_r3_IEs, DownlinkDirectTransfer_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions18_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions18(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions18, T_nonCriticalExtensions18_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions11_sequence[] = {
  { &hf_umts_rrc_pdu_def_downlinkDirectTransfer_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions18, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions18 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions11(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions11, T_laterNonCriticalExtensions11_sequence);

  return offset;
}


static const per_sequence_t T_r37_sequence[] = {
  { &hf_umts_rrc_pdu_def_downlinkDirectTransfer_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_DownlinkDirectTransfer_r3_IEs },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions11, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r37(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r37, T_r37_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions14_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions14(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_criticalExtensions14, T_criticalExtensions14_sequence);

  return offset;
}


static const per_sequence_t T_later_than_r36_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_criticalExtensions14, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_criticalExtensions14 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_later_than_r36(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_later_than_r36, T_later_than_r36_sequence);

  return offset;
}


const value_string umts_rrc_pdu_def_DownlinkDirectTransfer_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t DownlinkDirectTransfer_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r37, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r37 },
  {   1, &hf_umts_rrc_pdu_def_later_than_r36, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_later_than_r36 },
  { 0, NULL, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_DownlinkDirectTransfer(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_DownlinkDirectTransfer, DownlinkDirectTransfer_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd6_sequence[] = {
  { &hf_umts_rrc_pdu_def_dummy_DL_PDSCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_PDSCH_Information },
  { &hf_umts_rrc_pdu_def_dummy2_CPCH_SetInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CPCH_SetInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_fdd6(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_fdd6, T_fdd6_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_modeSpecificInfo_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_fdd6, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_fdd6 },
  {   1, &hf_umts_rrc_pdu_def_tdd, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_modeSpecificInfo(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_modeSpecificInfo, T_modeSpecificInfo_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_complete_sequence[] = {
  { &hf_umts_rrc_pdu_def_srb_InformationSetupList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_SRB_InformationSetupList },
  { &hf_umts_rrc_pdu_def_rab_InformationSetupList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RAB_InformationSetupList },
  { &hf_umts_rrc_pdu_def_ul_CommonTransChInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_UL_CommonTransChInfo },
  { &hf_umts_rrc_pdu_def_ul_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_UL_AddReconfTransChInfoList },
  { &hf_umts_rrc_pdu_def_dl_CommonTransChInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DL_CommonTransChInfo },
  { &hf_umts_rrc_pdu_def_dl_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DL_AddReconfTransChInfoList },
  { &hf_umts_rrc_pdu_def_ul_DPCH_Info, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_UL_DPCH_Info },
  { &hf_umts_rrc_pdu_def_modeSpecificInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_modeSpecificInfo },
  { &hf_umts_rrc_pdu_def_dl_CommonInformation, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DL_CommonInformation },
  { &hf_umts_rrc_pdu_def_dl_InformationPerRL_List, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DL_InformationPerRL_List },
  { &hf_umts_rrc_pdu_def_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_FrequencyInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_complete(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_complete, T_complete_sequence);

  return offset;
}


static const per_sequence_t T_defaultConfig_sequence[] = {
  { &hf_umts_rrc_pdu_def_defaultConfigMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DefaultConfigMode },
  { &hf_umts_rrc_pdu_def_defaultConfigIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DefaultConfigIdentity },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_defaultConfig(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_defaultConfig, T_defaultConfig_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_preConfigMode_vals[] = {
  {   0, "predefinedConfigIdentity" },
  {   1, "defaultConfig" },
  { 0, NULL }
};

static const per_choice_t T_preConfigMode_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_predefinedConfigIdentity, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_ies_PredefinedConfigIdentity },
  {   1, &hf_umts_rrc_pdu_def_defaultConfig, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_defaultConfig },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_preConfigMode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_preConfigMode, T_preConfigMode_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd7_sequence[] = {
  { &hf_umts_rrc_pdu_def_ul_DPCH_Info1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_UL_DPCH_InfoPostFDD },
  { &hf_umts_rrc_pdu_def_dl_CommonInformationPost, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DL_CommonInformationPost },
  { &hf_umts_rrc_pdu_def_dl_InformationPerRL_List4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DL_InformationPerRL_ListPostFDD },
  { &hf_umts_rrc_pdu_def_frequencyInfo1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_FrequencyInfoFDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_fdd7(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_fdd7, T_fdd7_sequence);

  return offset;
}


static const per_sequence_t T_tdd_sequence[] = {
  { &hf_umts_rrc_pdu_def_ul_DPCH_Info2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_UL_DPCH_InfoPostTDD },
  { &hf_umts_rrc_pdu_def_dl_CommonInformationPost, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DL_CommonInformationPost },
  { &hf_umts_rrc_pdu_def_dl_InformationPerRL, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DL_InformationPerRL_PostTDD },
  { &hf_umts_rrc_pdu_def_frequencyInfo2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_FrequencyInfoTDD },
  { &hf_umts_rrc_pdu_def_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_PrimaryCCPCH_TX_Power },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_tdd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_tdd, T_tdd_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_modeSpecificInfo1_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo1_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_fdd7, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_fdd7 },
  {   1, &hf_umts_rrc_pdu_def_tdd1, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_tdd },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_modeSpecificInfo1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_modeSpecificInfo1, T_modeSpecificInfo1_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_preconfiguration_sequence[] = {
  { &hf_umts_rrc_pdu_def_preConfigMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_preConfigMode },
  { &hf_umts_rrc_pdu_def_rab_Info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RAB_Info_Post },
  { &hf_umts_rrc_pdu_def_modeSpecificInfo1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_modeSpecificInfo1 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_preconfiguration(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_preconfiguration, T_preconfiguration_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_specificationMode_vals[] = {
  {   0, "complete" },
  {   1, "preconfiguration" },
  { 0, NULL }
};

static const per_choice_t T_specificationMode_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_complete, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_complete },
  {   1, &hf_umts_rrc_pdu_def_preconfiguration, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_preconfiguration },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_specificationMode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_specificationMode, T_specificationMode_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HandoverToUTRANCommand_r3_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_new_U_RNTI1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_U_RNTI_Short },
  { &hf_umts_rrc_pdu_def_dummy_ActivationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_cipheringAlgorithm, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CipheringAlgorithm },
  { &hf_umts_rrc_pdu_def_specificationMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_specificationMode },
  { &hf_umts_rrc_pdu_def_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_MaxAllowedUL_TX_Power },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_HandoverToUTRANCommand_r3_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_HandoverToUTRANCommand_r3_IEs, HandoverToUTRANCommand_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions19_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions19(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions19, T_nonCriticalExtensions19_sequence);

  return offset;
}


static const per_sequence_t T_r38_sequence[] = {
  { &hf_umts_rrc_pdu_def_handoverToUTRANCommand_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_HandoverToUTRANCommand_r3_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions19, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions19 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r38(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r38, T_r38_sequence);

  return offset;
}


static const per_sequence_t T_fdd8_sequence[] = {
  { &hf_umts_rrc_pdu_def_dummy_DL_PDSCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_PDSCH_Information },
  { &hf_umts_rrc_pdu_def_dummy2_CPCH_SetInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CPCH_SetInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_fdd8(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_fdd8, T_fdd8_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_modeSpecificInfo2_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo2_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_fdd8, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_fdd8 },
  {   1, &hf_umts_rrc_pdu_def_tdd, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_modeSpecificInfo2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_modeSpecificInfo2, T_modeSpecificInfo2_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_complete1_sequence[] = {
  { &hf_umts_rrc_pdu_def_srb_InformationSetupList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_SRB_InformationSetupList },
  { &hf_umts_rrc_pdu_def_rab_InformationSetupList1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RAB_InformationSetupList_r4 },
  { &hf_umts_rrc_pdu_def_ul_CommonTransChInfo1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_UL_CommonTransChInfo_r4 },
  { &hf_umts_rrc_pdu_def_ul_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_UL_AddReconfTransChInfoList },
  { &hf_umts_rrc_pdu_def_dl_CommonTransChInfo1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DL_CommonTransChInfo_r4 },
  { &hf_umts_rrc_pdu_def_dl_AddReconfTransChInfoList1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DL_AddReconfTransChInfoList_r4 },
  { &hf_umts_rrc_pdu_def_ul_DPCH_Info3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_UL_DPCH_Info_r4 },
  { &hf_umts_rrc_pdu_def_modeSpecificInfo2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_modeSpecificInfo2 },
  { &hf_umts_rrc_pdu_def_dl_CommonInformation1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DL_CommonInformation_r4 },
  { &hf_umts_rrc_pdu_def_dl_InformationPerRL_List1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DL_InformationPerRL_List_r4 },
  { &hf_umts_rrc_pdu_def_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_FrequencyInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_complete1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_complete1, T_complete1_sequence);

  return offset;
}


static const per_sequence_t T_defaultConfig1_sequence[] = {
  { &hf_umts_rrc_pdu_def_defaultConfigMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DefaultConfigMode },
  { &hf_umts_rrc_pdu_def_defaultConfigIdentity1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DefaultConfigIdentity_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_defaultConfig1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_defaultConfig1, T_defaultConfig1_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_preConfigMode1_vals[] = {
  {   0, "predefinedConfigIdentity" },
  {   1, "defaultConfig" },
  { 0, NULL }
};

static const per_choice_t T_preConfigMode1_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_predefinedConfigIdentity, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_ies_PredefinedConfigIdentity },
  {   1, &hf_umts_rrc_pdu_def_defaultConfig1, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_defaultConfig1 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_preConfigMode1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_preConfigMode1, T_preConfigMode1_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd9_sequence[] = {
  { &hf_umts_rrc_pdu_def_ul_DPCH_Info1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_UL_DPCH_InfoPostFDD },
  { &hf_umts_rrc_pdu_def_dl_CommonInformationPost, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DL_CommonInformationPost },
  { &hf_umts_rrc_pdu_def_dl_InformationPerRL_List4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DL_InformationPerRL_ListPostFDD },
  { &hf_umts_rrc_pdu_def_frequencyInfo1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_FrequencyInfoFDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_fdd9(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_fdd9, T_fdd9_sequence);

  return offset;
}


static const per_sequence_t T_tdd384_sequence[] = {
  { &hf_umts_rrc_pdu_def_ul_DPCH_Info2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_UL_DPCH_InfoPostTDD },
  { &hf_umts_rrc_pdu_def_dl_InformationPerRL, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DL_InformationPerRL_PostTDD },
  { &hf_umts_rrc_pdu_def_frequencyInfo2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_FrequencyInfoTDD },
  { &hf_umts_rrc_pdu_def_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_PrimaryCCPCH_TX_Power },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_tdd384(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_tdd384, T_tdd384_sequence);

  return offset;
}


static const per_sequence_t T_tdd128_sequence[] = {
  { &hf_umts_rrc_pdu_def_ul_DPCH_Info4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_UL_DPCH_InfoPostTDD_LCR_r4 },
  { &hf_umts_rrc_pdu_def_dl_InformationPerRL1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DL_InformationPerRL_PostTDD_LCR_r4 },
  { &hf_umts_rrc_pdu_def_frequencyInfo2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_FrequencyInfoTDD },
  { &hf_umts_rrc_pdu_def_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_PrimaryCCPCH_TX_Power },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_tdd128(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_tdd128, T_tdd128_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_tdd1_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tdd1_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_tdd384, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_tdd384 },
  {   1, &hf_umts_rrc_pdu_def_tdd128, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_tdd128 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_tdd1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_tdd1, T_tdd1_choice,
                                 NULL);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_modeSpecificInfo3_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo3_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_fdd9, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_fdd9 },
  {   1, &hf_umts_rrc_pdu_def_tdd2, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_tdd1 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_modeSpecificInfo3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_modeSpecificInfo3, T_modeSpecificInfo3_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_preconfiguration1_sequence[] = {
  { &hf_umts_rrc_pdu_def_preConfigMode1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_preConfigMode1 },
  { &hf_umts_rrc_pdu_def_rab_Info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RAB_Info_Post },
  { &hf_umts_rrc_pdu_def_modeSpecificInfo3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_modeSpecificInfo3 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_preconfiguration1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_preconfiguration1, T_preconfiguration1_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_specificationMode1_vals[] = {
  {   0, "complete" },
  {   1, "preconfiguration" },
  { 0, NULL }
};

static const per_choice_t T_specificationMode1_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_complete1, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_complete1 },
  {   1, &hf_umts_rrc_pdu_def_preconfiguration1, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_preconfiguration1 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_specificationMode1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_specificationMode1, T_specificationMode1_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HandoverToUTRANCommand_r4_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_new_U_RNTI1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_U_RNTI_Short },
  { &hf_umts_rrc_pdu_def_cipheringAlgorithm, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CipheringAlgorithm },
  { &hf_umts_rrc_pdu_def_specificationMode1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_specificationMode1 },
  { &hf_umts_rrc_pdu_def_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_MaxAllowedUL_TX_Power },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_HandoverToUTRANCommand_r4_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_HandoverToUTRANCommand_r4_IEs, HandoverToUTRANCommand_r4_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions20_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions20(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions20, T_nonCriticalExtensions20_sequence);

  return offset;
}


static const per_sequence_t T_r42_sequence[] = {
  { &hf_umts_rrc_pdu_def_handoverToUTRANCommand_r4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_HandoverToUTRANCommand_r4_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions20, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions20 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r42(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r42, T_r42_sequence);

  return offset;
}


static const per_sequence_t T_fdd10_sequence[] = {
  { &hf_umts_rrc_pdu_def_dummy_DL_PDSCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_PDSCH_Information },
  { &hf_umts_rrc_pdu_def_dummy2_CPCH_SetInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CPCH_SetInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_fdd10(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_fdd10, T_fdd10_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_modeSpecificInfo4_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo4_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_fdd10, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_fdd10 },
  {   1, &hf_umts_rrc_pdu_def_tdd, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_modeSpecificInfo4(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_modeSpecificInfo4, T_modeSpecificInfo4_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_complete2_sequence[] = {
  { &hf_umts_rrc_pdu_def_srb_InformationSetupList1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_SRB_InformationSetupList_r5 },
  { &hf_umts_rrc_pdu_def_rab_InformationSetupList2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RAB_InformationSetupList_r5 },
  { &hf_umts_rrc_pdu_def_ul_CommonTransChInfo1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_UL_CommonTransChInfo_r4 },
  { &hf_umts_rrc_pdu_def_ul_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_UL_AddReconfTransChInfoList },
  { &hf_umts_rrc_pdu_def_dl_CommonTransChInfo1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DL_CommonTransChInfo_r4 },
  { &hf_umts_rrc_pdu_def_dl_AddReconfTransChInfoList2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DL_AddReconfTransChInfoList_r5 },
  { &hf_umts_rrc_pdu_def_ul_DPCH_Info5, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_UL_DPCH_Info_r5 },
  { &hf_umts_rrc_pdu_def_modeSpecificInfo4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_modeSpecificInfo4 },
  { &hf_umts_rrc_pdu_def_dl_CommonInformation1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DL_CommonInformation_r4 },
  { &hf_umts_rrc_pdu_def_dl_InformationPerRL_List2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DL_InformationPerRL_List_r5 },
  { &hf_umts_rrc_pdu_def_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_FrequencyInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_complete2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_complete2, T_complete2_sequence);

  return offset;
}


static const per_sequence_t T_defaultConfig2_sequence[] = {
  { &hf_umts_rrc_pdu_def_defaultConfigMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DefaultConfigMode },
  { &hf_umts_rrc_pdu_def_defaultConfigIdentity2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DefaultConfigIdentity_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_defaultConfig2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_defaultConfig2, T_defaultConfig2_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_preConfigMode2_vals[] = {
  {   0, "predefinedConfigIdentity" },
  {   1, "defaultConfig" },
  { 0, NULL }
};

static const per_choice_t T_preConfigMode2_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_predefinedConfigIdentity, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_ies_PredefinedConfigIdentity },
  {   1, &hf_umts_rrc_pdu_def_defaultConfig2, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_defaultConfig2 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_preConfigMode2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_preConfigMode2, T_preConfigMode2_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd11_sequence[] = {
  { &hf_umts_rrc_pdu_def_ul_DPCH_Info1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_UL_DPCH_InfoPostFDD },
  { &hf_umts_rrc_pdu_def_dl_CommonInformationPost, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DL_CommonInformationPost },
  { &hf_umts_rrc_pdu_def_dl_InformationPerRL_List4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DL_InformationPerRL_ListPostFDD },
  { &hf_umts_rrc_pdu_def_frequencyInfo1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_FrequencyInfoFDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_fdd11(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_fdd11, T_fdd11_sequence);

  return offset;
}


static const per_sequence_t T_tdd3841_sequence[] = {
  { &hf_umts_rrc_pdu_def_ul_DPCH_Info2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_UL_DPCH_InfoPostTDD },
  { &hf_umts_rrc_pdu_def_dl_InformationPerRL, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DL_InformationPerRL_PostTDD },
  { &hf_umts_rrc_pdu_def_frequencyInfo2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_FrequencyInfoTDD },
  { &hf_umts_rrc_pdu_def_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_PrimaryCCPCH_TX_Power },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_tdd3841(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_tdd3841, T_tdd3841_sequence);

  return offset;
}


static const per_sequence_t T_tdd1281_sequence[] = {
  { &hf_umts_rrc_pdu_def_ul_DPCH_Info4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_UL_DPCH_InfoPostTDD_LCR_r4 },
  { &hf_umts_rrc_pdu_def_dl_InformationPerRL1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DL_InformationPerRL_PostTDD_LCR_r4 },
  { &hf_umts_rrc_pdu_def_frequencyInfo2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_FrequencyInfoTDD },
  { &hf_umts_rrc_pdu_def_primaryCCPCH_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_PrimaryCCPCH_TX_Power },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_tdd1281(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_tdd1281, T_tdd1281_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_tdd2_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tdd2_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_tdd3841, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_tdd3841 },
  {   1, &hf_umts_rrc_pdu_def_tdd1281, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_tdd1281 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_tdd2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_tdd2, T_tdd2_choice,
                                 NULL);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_modeSpecificInfo5_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo5_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_fdd11, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_fdd11 },
  {   1, &hf_umts_rrc_pdu_def_tdd3, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_tdd2 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_modeSpecificInfo5(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_modeSpecificInfo5, T_modeSpecificInfo5_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_preconfiguration2_sequence[] = {
  { &hf_umts_rrc_pdu_def_preConfigMode2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_preConfigMode2 },
  { &hf_umts_rrc_pdu_def_rab_Info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RAB_Info_Post },
  { &hf_umts_rrc_pdu_def_modeSpecificInfo5, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_modeSpecificInfo5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_preconfiguration2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_preconfiguration2, T_preconfiguration2_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_specificationMode2_vals[] = {
  {   0, "complete" },
  {   1, "preconfiguration" },
  { 0, NULL }
};

static const per_choice_t T_specificationMode2_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_complete2, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_complete2 },
  {   1, &hf_umts_rrc_pdu_def_preconfiguration2, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_preconfiguration2 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_specificationMode2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_specificationMode2, T_specificationMode2_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HandoverToUTRANCommand_r5_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_new_U_RNTI1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_U_RNTI_Short },
  { &hf_umts_rrc_pdu_def_cipheringAlgorithm, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CipheringAlgorithm },
  { &hf_umts_rrc_pdu_def_specificationMode2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_specificationMode2 },
  { &hf_umts_rrc_pdu_def_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_MaxAllowedUL_TX_Power },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_HandoverToUTRANCommand_r5_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_HandoverToUTRANCommand_r5_IEs, HandoverToUTRANCommand_r5_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions21_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions21(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions21, T_nonCriticalExtensions21_sequence);

  return offset;
}


static const per_sequence_t T_r52_sequence[] = {
  { &hf_umts_rrc_pdu_def_handoverToUTRANCommand_r5, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_HandoverToUTRANCommand_r5_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions21, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions21 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r52(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r52, T_r52_sequence);

  return offset;
}


static const per_sequence_t T_complete3_sequence[] = {
  { &hf_umts_rrc_pdu_def_srb_InformationSetupList2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_SRB_InformationSetupList_r6 },
  { &hf_umts_rrc_pdu_def_rab_InformationSetupList3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RAB_InformationSetupList_r6 },
  { &hf_umts_rrc_pdu_def_ul_CommonTransChInfo1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_UL_CommonTransChInfo_r4 },
  { &hf_umts_rrc_pdu_def_ul_AddReconfTransChInfoList1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_UL_AddReconfTransChInfoList_r6 },
  { &hf_umts_rrc_pdu_def_dl_CommonTransChInfo1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DL_CommonTransChInfo_r4 },
  { &hf_umts_rrc_pdu_def_dl_AddReconfTransChInfoList2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DL_AddReconfTransChInfoList_r5 },
  { &hf_umts_rrc_pdu_def_ul_DPCH_Info6, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_UL_DPCH_Info_r6 },
  { &hf_umts_rrc_pdu_def_dl_CommonInformation1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DL_CommonInformation_r4 },
  { &hf_umts_rrc_pdu_def_dl_InformationPerRL_List3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DL_InformationPerRL_List_r6 },
  { &hf_umts_rrc_pdu_def_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_FrequencyInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_complete3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_complete3, T_complete3_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_specificationMode3_vals[] = {
  {   0, "complete" },
  { 0, NULL }
};

static const per_choice_t T_specificationMode3_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_complete3, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_complete3 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_specificationMode3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_specificationMode3, T_specificationMode3_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HandoverToUTRANCommand_r6_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_new_U_RNTI1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_U_RNTI_Short },
  { &hf_umts_rrc_pdu_def_cipheringAlgorithm, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CipheringAlgorithm },
  { &hf_umts_rrc_pdu_def_specificationMode3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_specificationMode3 },
  { &hf_umts_rrc_pdu_def_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_MaxAllowedUL_TX_Power },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_HandoverToUTRANCommand_r6_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_HandoverToUTRANCommand_r6_IEs, HandoverToUTRANCommand_r6_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions22_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions22(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions22, T_nonCriticalExtensions22_sequence);

  return offset;
}


static const per_sequence_t T_r63_sequence[] = {
  { &hf_umts_rrc_pdu_def_handoverToUTRANCommand_r6, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_HandoverToUTRANCommand_r6_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions22, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions22 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r63(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r63, T_r63_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions18_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions18(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_criticalExtensions18, T_criticalExtensions18_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_criticalExtensions17_vals[] = {
  {   0, "r6" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions17_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r63, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r63 },
  {   1, &hf_umts_rrc_pdu_def_criticalExtensions18, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_criticalExtensions18 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions17(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_criticalExtensions17, T_criticalExtensions17_choice,
                                 NULL);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_criticalExtensions16_vals[] = {
  {   0, "r5" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions16_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r52, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r52 },
  {   1, &hf_umts_rrc_pdu_def_criticalExtensions17, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_criticalExtensions17 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions16(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_criticalExtensions16, T_criticalExtensions16_choice,
                                 NULL);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_criticalExtensions15_vals[] = {
  {   0, "r4" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions15_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r42, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r42 },
  {   1, &hf_umts_rrc_pdu_def_criticalExtensions16, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_criticalExtensions16 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions15(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_criticalExtensions15, T_criticalExtensions15_choice,
                                 NULL);

  return offset;
}


static const value_string umts_rrc_pdu_def_HandoverToUTRANCommand_vals[] = {
  {   0, "r3" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t HandoverToUTRANCommand_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r38, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r38 },
  {   1, &hf_umts_rrc_pdu_def_criticalExtensions15, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_criticalExtensions15 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_HandoverToUTRANCommand(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_HandoverToUTRANCommand, HandoverToUTRANCommand_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions23_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions23(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions23, T_nonCriticalExtensions23_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions12_sequence[] = {
  { &hf_umts_rrc_pdu_def_handoverToUTRANComplete_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions23, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions23 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions12(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions12, T_laterNonCriticalExtensions12_sequence);

  return offset;
}


static const per_sequence_t HandoverToUTRANComplete_sequence[] = {
  { &hf_umts_rrc_pdu_def_startList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_STARTList },
  { &hf_umts_rrc_pdu_def_count_C_ActivationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions12, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions12 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_HandoverToUTRANComplete(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_HandoverToUTRANComplete, HandoverToUTRANComplete_sequence);

  return offset;
}


static const per_sequence_t InitialDirectTransfer_v3a0ext_sequence[] = {
  { &hf_umts_rrc_pdu_def_start_Value, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_START_Value },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_InitialDirectTransfer_v3a0ext(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_InitialDirectTransfer_v3a0ext, InitialDirectTransfer_v3a0ext_sequence);

  return offset;
}


static const per_sequence_t InitialDirectTransfer_v590ext_sequence[] = {
  { &hf_umts_rrc_pdu_def_establishmentCause, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_EstablishmentCause },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_InitialDirectTransfer_v590ext(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_InitialDirectTransfer_v590ext, InitialDirectTransfer_v590ext_sequence);

  return offset;
}


static const per_sequence_t InitialDirectTransfer_v6xyext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_plmn_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_PLMN_Identity },
  { &hf_umts_rrc_pdu_def_measuredResultsOnRACHinterFreq, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MeasuredResultsOnRACHinterFreq },
  { &hf_umts_rrc_pdu_def_mbms_JoinedInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MBMS_JoinedInformation_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_InitialDirectTransfer_v6xyext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_InitialDirectTransfer_v6xyext_IEs, InitialDirectTransfer_v6xyext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions24_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions24(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions24, T_nonCriticalExtensions24_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions8_sequence[] = {
  { &hf_umts_rrc_pdu_def_initialDirectTransfer_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_InitialDirectTransfer_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions24, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions24 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions8(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions8, T_v6xyNonCriticalExtensions8_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions5_sequence[] = {
  { &hf_umts_rrc_pdu_def_initialDirectTransfer_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_InitialDirectTransfer_v590ext },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions8, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions5(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions5, T_v590NonCriticalExtensions5_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions13_sequence[] = {
  { &hf_umts_rrc_pdu_def_initialDirectTransfer_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions5, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions13(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions13, T_laterNonCriticalExtensions13_sequence);

  return offset;
}


static const per_sequence_t T_v3a0NonCriticalExtensions2_sequence[] = {
  { &hf_umts_rrc_pdu_def_initialDirectTransfer_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_InitialDirectTransfer_v3a0ext },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions13, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions13 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions2, T_v3a0NonCriticalExtensions2_sequence);

  return offset;
}


static const per_sequence_t InitialDirectTransfer_sequence[] = {
  { &hf_umts_rrc_pdu_def_cn_DomainIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_CN_DomainIdentity },
  { &hf_umts_rrc_pdu_def_intraDomainNasNodeSelector, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_IntraDomainNasNodeSelector },
  { &hf_umts_rrc_pdu_def_nas_Message, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_NAS_Message },
  { &hf_umts_rrc_pdu_def_measuredResultsOnRACH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MeasuredResultsOnRACH },
  { &hf_umts_rrc_pdu_def_v3a0NonCriticalExtensions2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions2 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_InitialDirectTransfer(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_InitialDirectTransfer, InitialDirectTransfer_sequence);

  return offset;
}


static const per_sequence_t T_single_GSM_Message_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_single_GSM_Message(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_single_GSM_Message, T_single_GSM_Message_sequence);

  return offset;
}


static const per_sequence_t T_gsm_MessageList_sequence[] = {
  { &hf_umts_rrc_pdu_def_gsm_Messages, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_GSM_MessageList },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_gsm_MessageList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_gsm_MessageList, T_gsm_MessageList_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_gsm_message_vals[] = {
  {   0, "single-GSM-Message" },
  {   1, "gsm-MessageList" },
  { 0, NULL }
};

static const per_choice_t T_gsm_message_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_single_GSM_Message, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_single_GSM_Message },
  {   1, &hf_umts_rrc_pdu_def_gsm_MessageList, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_gsm_MessageList },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_gsm_message(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_gsm_message, T_gsm_message_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HandoverFromUTRANCommand_GSM_r3_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_activationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_toHandoverRAB_Info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RAB_Info },
  { &hf_umts_rrc_pdu_def_frequency_band, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_Frequency_Band },
  { &hf_umts_rrc_pdu_def_gsm_message, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_gsm_message },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_HandoverFromUTRANCommand_GSM_r3_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_HandoverFromUTRANCommand_GSM_r3_IEs, HandoverFromUTRANCommand_GSM_r3_IEs_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_geran_SystemInfoType2_vals[] = {
  {   0, "sI" },
  {   1, "pSI" },
  { 0, NULL }
};

static const per_choice_t T_geran_SystemInfoType2_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_sI , ASN1_NO_EXTENSIONS     , dissect_umts_rrc_ies_GERAN_SystemInformation },
  {   1, &hf_umts_rrc_pdu_def_pSI, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_ies_GERAN_SystemInformation },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_geran_SystemInfoType2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_geran_SystemInfoType2, T_geran_SystemInfoType2_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HandoverFromUTRANCommand_GSM_v6xyext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_geran_SystemInfoType2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_geran_SystemInfoType2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_HandoverFromUTRANCommand_GSM_v6xyext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_HandoverFromUTRANCommand_GSM_v6xyext_IEs, HandoverFromUTRANCommand_GSM_v6xyext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions25_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions25(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions25, T_nonCriticalExtensions25_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions9_sequence[] = {
  { &hf_umts_rrc_pdu_def_handoverFromUTRANCommand_GSM_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_HandoverFromUTRANCommand_GSM_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions25, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions25 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions9(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions9, T_v6xyNonCriticalExtensions9_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions14_sequence[] = {
  { &hf_umts_rrc_pdu_def_handoverFromUTRANCommand_GSM_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions9, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions14(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions14, T_laterNonCriticalExtensions14_sequence);

  return offset;
}


static const per_sequence_t T_r39_sequence[] = {
  { &hf_umts_rrc_pdu_def_handoverFromUTRANCommand_GSM_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_HandoverFromUTRANCommand_GSM_r3_IEs },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions14, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions14 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r39(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r39, T_r39_sequence);

  return offset;
}


static const per_sequence_t T_single_GSM_Message1_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_single_GSM_Message1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_single_GSM_Message1, T_single_GSM_Message1_sequence);

  return offset;
}


static const per_sequence_t T_gsm_MessageList1_sequence[] = {
  { &hf_umts_rrc_pdu_def_gsm_Messages, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_GSM_MessageList },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_gsm_MessageList1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_gsm_MessageList1, T_gsm_MessageList1_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_gsm_message1_vals[] = {
  {   0, "single-GSM-Message" },
  {   1, "gsm-MessageList" },
  { 0, NULL }
};

static const per_choice_t T_gsm_message1_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_single_GSM_Message1, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_single_GSM_Message1 },
  {   1, &hf_umts_rrc_pdu_def_gsm_MessageList1, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_gsm_MessageList1 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_gsm_message1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_gsm_message1, T_gsm_message1_choice,
                                 NULL);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_geran_SystemInfoType1_vals[] = {
  {   0, "sI" },
  {   1, "pSI" },
  { 0, NULL }
};

static const per_choice_t T_geran_SystemInfoType1_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_sI , ASN1_NO_EXTENSIONS     , dissect_umts_rrc_ies_GERAN_SystemInformation },
  {   1, &hf_umts_rrc_pdu_def_pSI, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_ies_GERAN_SystemInformation },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_geran_SystemInfoType1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_geran_SystemInfoType1, T_geran_SystemInfoType1_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HandoverFromUTRANCommand_GSM_r6_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_activationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_toHandoverRAB_Info1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RAB_InformationList_r6 },
  { &hf_umts_rrc_pdu_def_frequency_band, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_Frequency_Band },
  { &hf_umts_rrc_pdu_def_gsm_message1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_gsm_message1 },
  { &hf_umts_rrc_pdu_def_geran_SystemInfoType1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_geran_SystemInfoType1 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_HandoverFromUTRANCommand_GSM_r6_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_HandoverFromUTRANCommand_GSM_r6_IEs, HandoverFromUTRANCommand_GSM_r6_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions26_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions26(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions26, T_nonCriticalExtensions26_sequence);

  return offset;
}


static const per_sequence_t T_r64_sequence[] = {
  { &hf_umts_rrc_pdu_def_handoverFromUTRANCommand_GSM_r6, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_HandoverFromUTRANCommand_GSM_r6_IEs },
  { &hf_umts_rrc_pdu_def_handoverFromUTRANCommand_GSM_r6_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions26, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions26 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r64(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r64, T_r64_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions20_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions20(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_criticalExtensions20, T_criticalExtensions20_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_criticalExtensions19_vals[] = {
  {   0, "r6" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions19_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r64, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r64 },
  {   1, &hf_umts_rrc_pdu_def_criticalExtensions20, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_criticalExtensions20 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions19(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_criticalExtensions19, T_criticalExtensions19_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_later_than_r37_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_criticalExtensions19, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_criticalExtensions19 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_later_than_r37(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_later_than_r37, T_later_than_r37_sequence);

  return offset;
}


const value_string umts_rrc_pdu_def_HandoverFromUTRANCommand_GSM_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t HandoverFromUTRANCommand_GSM_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r39, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r39 },
  {   1, &hf_umts_rrc_pdu_def_later_than_r37, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_later_than_r37 },
  { 0, NULL, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_HandoverFromUTRANCommand_GSM(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_HandoverFromUTRANCommand_GSM, HandoverFromUTRANCommand_GSM_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_single_GERANIu_Message_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_single_GERANIu_Message(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_single_GERANIu_Message, T_single_GERANIu_Message_sequence);

  return offset;
}


static const per_sequence_t T_geranIu_MessageList_sequence[] = {
  { &hf_umts_rrc_pdu_def_geranIu_Messages, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_GERANIu_MessageList },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_geranIu_MessageList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_geranIu_MessageList, T_geranIu_MessageList_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_geranIu_Message_vals[] = {
  {   0, "single-GERANIu-Message" },
  {   1, "geranIu-MessageList" },
  { 0, NULL }
};

static const per_choice_t T_geranIu_Message_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_single_GERANIu_Message, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_single_GERANIu_Message },
  {   1, &hf_umts_rrc_pdu_def_geranIu_MessageList, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_geranIu_MessageList },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_geranIu_Message(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_geranIu_Message, T_geranIu_Message_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HandoverFromUTRANCommand_GERANIu_r5_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_activationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_frequency_Band, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_Frequency_Band },
  { &hf_umts_rrc_pdu_def_geranIu_Message, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_geranIu_Message },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_HandoverFromUTRANCommand_GERANIu_r5_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_HandoverFromUTRANCommand_GERANIu_r5_IEs, HandoverFromUTRANCommand_GERANIu_r5_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions27_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions27(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions27, T_nonCriticalExtensions27_sequence);

  return offset;
}


static const per_sequence_t T_r53_sequence[] = {
  { &hf_umts_rrc_pdu_def_handoverFromUTRANCommand_GERANIu_r5, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_HandoverFromUTRANCommand_GERANIu_r5_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions27, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions27 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r53(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r53, T_r53_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions21_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions21(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_criticalExtensions21, T_criticalExtensions21_sequence);

  return offset;
}


static const per_sequence_t T_later_than_r5_sequence[] = {
  { &hf_umts_rrc_pdu_def_criticalExtensions21, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_criticalExtensions21 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_later_than_r5(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_later_than_r5, T_later_than_r5_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_handoverFromUTRANCommand_GERANIu_vals[] = {
  {   0, "r5" },
  {   1, "later-than-r5" },
  { 0, NULL }
};

static const per_choice_t T_handoverFromUTRANCommand_GERANIu_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r53, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r53 },
  {   1, &hf_umts_rrc_pdu_def_later_than_r5, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_later_than_r5 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_handoverFromUTRANCommand_GERANIu(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_handoverFromUTRANCommand_GERANIu, T_handoverFromUTRANCommand_GERANIu_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HandoverFromUTRANCommand_GERANIu_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_handoverFromUTRANCommand_GERANIu, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_handoverFromUTRANCommand_GERANIu },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_HandoverFromUTRANCommand_GERANIu(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_HandoverFromUTRANCommand_GERANIu, HandoverFromUTRANCommand_GERANIu_sequence);

  return offset;
}


static const per_sequence_t HandoverFromUTRANCommand_CDMA2000_r3_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_activationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_toHandoverRAB_Info, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RAB_Info },
  { &hf_umts_rrc_pdu_def_cdma2000_MessageList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_CDMA2000_MessageList },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_HandoverFromUTRANCommand_CDMA2000_r3_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_HandoverFromUTRANCommand_CDMA2000_r3_IEs, HandoverFromUTRANCommand_CDMA2000_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions28_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions28(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions28, T_nonCriticalExtensions28_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions15_sequence[] = {
  { &hf_umts_rrc_pdu_def_handoverFromUTRANCommand_CDMA2000_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions28, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions28 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions15(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions15, T_laterNonCriticalExtensions15_sequence);

  return offset;
}


static const per_sequence_t T_r310_sequence[] = {
  { &hf_umts_rrc_pdu_def_handoverFromUTRANCommand_CDMA2000_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_HandoverFromUTRANCommand_CDMA2000_r3_IEs },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions15, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions15 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r310(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r310, T_r310_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions22_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions22(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_criticalExtensions22, T_criticalExtensions22_sequence);

  return offset;
}


static const per_sequence_t T_later_than_r38_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_criticalExtensions22, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_criticalExtensions22 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_later_than_r38(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_later_than_r38, T_later_than_r38_sequence);

  return offset;
}


const value_string umts_rrc_pdu_def_HandoverFromUTRANCommand_CDMA2000_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t HandoverFromUTRANCommand_CDMA2000_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r310, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r310 },
  {   1, &hf_umts_rrc_pdu_def_later_than_r38, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_later_than_r38 },
  { 0, NULL, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_HandoverFromUTRANCommand_CDMA2000(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_HandoverFromUTRANCommand_CDMA2000, HandoverFromUTRANCommand_CDMA2000_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_gsm_sequence[] = {
  { &hf_umts_rrc_pdu_def_gsm_MessageList2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_GSM_MessageList },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_gsm(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_gsm, T_gsm_sequence);

  return offset;
}


static const per_sequence_t T_cdma2000_sequence[] = {
  { &hf_umts_rrc_pdu_def_cdma2000_MessageList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_CDMA2000_MessageList },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_cdma2000(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_cdma2000, T_cdma2000_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_interRATMessage_vals[] = {
  {   0, "gsm" },
  {   1, "cdma2000" },
  { 0, NULL }
};

static const per_choice_t T_interRATMessage_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_gsm, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_gsm },
  {   1, &hf_umts_rrc_pdu_def_cdma2000, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_cdma2000 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_interRATMessage(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_interRATMessage, T_interRATMessage_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t HandoverFromUtranFailure_v590ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_geranIu_MessageList1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_GERANIu_MessageList },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_HandoverFromUtranFailure_v590ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_HandoverFromUtranFailure_v590ext_IEs, HandoverFromUtranFailure_v590ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions29_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions29(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions29, T_nonCriticalExtensions29_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions6_sequence[] = {
  { &hf_umts_rrc_pdu_def_handoverFromUTRANFailure_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_HandoverFromUtranFailure_v590ext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions29, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions29 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions6(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions6, T_v590NonCriticalExtensions6_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions16_sequence[] = {
  { &hf_umts_rrc_pdu_def_handoverFromUTRANFailure_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions6, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions16(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions16, T_laterNonCriticalExtensions16_sequence);

  return offset;
}


static const per_sequence_t HandoverFromUTRANFailure_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_interRAT_HO_FailureCause, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_InterRAT_HO_FailureCause },
  { &hf_umts_rrc_pdu_def_interRATMessage, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_interRATMessage },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions16, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions16 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_HandoverFromUTRANFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_HandoverFromUTRANFailure, HandoverFromUTRANFailure_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_predefinedConfigStatusList_vals[] = {
  {   0, "absent" },
  {   1, "present" },
  { 0, NULL }
};

static const per_choice_t T_predefinedConfigStatusList_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_absent, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  {   1, &hf_umts_rrc_pdu_def_present, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_ies_PredefinedConfigStatusList },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_predefinedConfigStatusList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_predefinedConfigStatusList, T_predefinedConfigStatusList_choice,
                                 NULL);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_uE_SecurityInformation_vals[] = {
  {   0, "absent" },
  {   1, "present" },
  { 0, NULL }
};

static const per_choice_t T_uE_SecurityInformation_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_absent, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  {   1, &hf_umts_rrc_pdu_def_present1, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_ies_UE_SecurityInformation },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_uE_SecurityInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_uE_SecurityInformation, T_uE_SecurityInformation_choice,
                                 NULL);

  return offset;
}



static int
dissect_umts_rrc_pdu_def_OCTET_STRING_SIZE_0_63(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_octet_string(tvb, offset, actx, tree, hf_index,
                                       0, 63, NULL);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_ue_CapabilityContainer_vals[] = {
  {   0, "absent" },
  {   1, "present" },
  { 0, NULL }
};

static const per_choice_t T_ue_CapabilityContainer_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_absent, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  {   1, &hf_umts_rrc_pdu_def_present2, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_OCTET_STRING_SIZE_0_63 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_ue_CapabilityContainer(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_ue_CapabilityContainer, T_ue_CapabilityContainer_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t InterRATHandoverInfo_v390ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_ue_RadioAccessCapability_v380ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UE_RadioAccessCapability_v380ext },
  { &hf_umts_rrc_pdu_def_dl_PhysChCapabilityFDD_v380ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DL_PhysChCapabilityFDD_v380ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_InterRATHandoverInfo_v390ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_InterRATHandoverInfo_v390ext_IEs, InterRATHandoverInfo_v390ext_IEs_sequence);

  return offset;
}


static const per_sequence_t InterRATHandoverInfo_v3a0ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_ue_RadioAccessCapability_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UE_RadioAccessCapability_v3a0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_InterRATHandoverInfo_v3a0ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_InterRATHandoverInfo_v3a0ext_IEs, InterRATHandoverInfo_v3a0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t InterRATHandoverInfo_v3d0ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_uESpecificBehaviourInformation1interRAT, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UESpecificBehaviourInformation1interRAT },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_InterRATHandoverInfo_v3d0ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_InterRATHandoverInfo_v3d0ext_IEs, InterRATHandoverInfo_v3d0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t InterRATHandoverInfo_v3g0ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_ue_RadioAccessCapability_v3g0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UE_RadioAccessCapability_v3g0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_InterRATHandoverInfo_v3g0ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_InterRATHandoverInfo_v3g0ext_IEs, InterRATHandoverInfo_v3g0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t InterRATHandoverInfo_v4b0ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_accessStratumReleaseIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_AccessStratumReleaseIndicator },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_InterRATHandoverInfo_v4b0ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_InterRATHandoverInfo_v4b0ext_IEs, InterRATHandoverInfo_v4b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t InterRATHandoverInfo_v4d0ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_tdd128_RF_Capability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RadioFrequencyBandTDDList },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_InterRATHandoverInfo_v4d0ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_InterRATHandoverInfo_v4d0ext_IEs, InterRATHandoverInfo_v4d0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t InterRATHandoverInfo_v590ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_predefinedConfigStatusListComp, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_PredefinedConfigStatusListComp },
  { &hf_umts_rrc_pdu_def_ue_RadioAccessCapabilityComp, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UE_RadioAccessCapabilityComp },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_InterRATHandoverInfo_v590ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_InterRATHandoverInfo_v590ext_IEs, InterRATHandoverInfo_v590ext_IEs_sequence);

  return offset;
}


static const per_sequence_t InterRATHandoverInfo_v6xyext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_ue_SecurityInformation2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UE_SecurityInformation2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_InterRATHandoverInfo_v6xyext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_InterRATHandoverInfo_v6xyext_IEs, InterRATHandoverInfo_v6xyext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions30_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions30(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions30, T_nonCriticalExtensions30_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions10_sequence[] = {
  { &hf_umts_rrc_pdu_def_interRATHandoverInfo_v6xy0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_InterRATHandoverInfo_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions30, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions30 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions10(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions10, T_v6xyNonCriticalExtensions10_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions7_sequence[] = {
  { &hf_umts_rrc_pdu_def_interRATHandoverInfo_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_InterRATHandoverInfo_v590ext_IEs },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions10, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions7(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions7, T_v590NonCriticalExtensions7_sequence);

  return offset;
}


static const per_sequence_t T_v4d0NonCriticalExtensions2_sequence[] = {
  { &hf_umts_rrc_pdu_def_interRATHandoverInfo_v4d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_InterRATHandoverInfo_v4d0ext_IEs },
  { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions7, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions2, T_v4d0NonCriticalExtensions2_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions4_sequence[] = {
  { &hf_umts_rrc_pdu_def_interRATHandoverInfo_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_InterRATHandoverInfo_v4b0ext_IEs },
  { &hf_umts_rrc_pdu_def_v4d0NonCriticalExtensions2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions4(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions4, T_v4b0NonCriticalExtensions4_sequence);

  return offset;
}


static const per_sequence_t T_v3g0NonCriticalExtensions_sequence[] = {
  { &hf_umts_rrc_pdu_def_interRATHandoverInfo_v3g0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_InterRATHandoverInfo_v3g0ext_IEs },
  { &hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions4, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v3g0NonCriticalExtensions(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v3g0NonCriticalExtensions, T_v3g0NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions17_sequence[] = {
  { &hf_umts_rrc_pdu_def_interRATHandoverInfo_v3d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_InterRATHandoverInfo_v3d0ext_IEs },
  { &hf_umts_rrc_pdu_def_interRATHandoverInfo_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v3g0NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v3g0NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions17(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions17, T_laterNonCriticalExtensions17_sequence);

  return offset;
}


static const per_sequence_t T_v3a0NonCriticalExtensions3_sequence[] = {
  { &hf_umts_rrc_pdu_def_interRATHandoverInfo_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_InterRATHandoverInfo_v3a0ext_IEs },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions17, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions17 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions3, T_v3a0NonCriticalExtensions3_sequence);

  return offset;
}


static const per_sequence_t T_present_sequence[] = {
  { &hf_umts_rrc_pdu_def_interRATHandoverInfo_v390ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_InterRATHandoverInfo_v390ext_IEs },
  { &hf_umts_rrc_pdu_def_v3a0NonCriticalExtensions3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions3 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_present(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_present, T_present_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_v390NonCriticalExtensions_vals[] = {
  {   0, "absent" },
  {   1, "present" },
  { 0, NULL }
};

static const per_choice_t T_v390NonCriticalExtensions_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_absent, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  {   1, &hf_umts_rrc_pdu_def_present3, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_present },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v390NonCriticalExtensions(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_v390NonCriticalExtensions, T_v390NonCriticalExtensions_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t InterRATHandoverInfo_sequence[] = {
  { &hf_umts_rrc_pdu_def_predefinedConfigStatusList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_predefinedConfigStatusList },
  { &hf_umts_rrc_pdu_def_uE_SecurityInformation, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_uE_SecurityInformation },
  { &hf_umts_rrc_pdu_def_ue_CapabilityContainer, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_ue_CapabilityContainer },
  { &hf_umts_rrc_pdu_def_v390NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_v390NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_InterRATHandoverInfo(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_InterRATHandoverInfo, InterRATHandoverInfo_sequence);

  return offset;
}


static const per_sequence_t MeasurementControl_r3_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_measurementIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_MeasurementIdentity },
  { &hf_umts_rrc_pdu_def_measurementCommand, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_MeasurementCommand },
  { &hf_umts_rrc_pdu_def_measurementReportingMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MeasurementReportingMode },
  { &hf_umts_rrc_pdu_def_additionalMeasurementList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_AdditionalMeasurementID_List },
  { &hf_umts_rrc_pdu_def_dpch_CompressedModeStatusInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DPCH_CompressedModeStatusInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_MeasurementControl_r3_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_MeasurementControl_r3_IEs, MeasurementControl_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t MeasurementControl_v390ext_sequence[] = {
  { &hf_umts_rrc_pdu_def_ue_Positioning_Measurement_v390ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UE_Positioning_Measurement_v390ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_MeasurementControl_v390ext(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_MeasurementControl_v390ext, MeasurementControl_v390ext_sequence);

  return offset;
}


static const per_sequence_t MeasurementControl_v3a0ext_sequence[] = {
  { &hf_umts_rrc_pdu_def_sfn_Offset_Validity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_SFN_Offset_Validity },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_MeasurementControl_v3a0ext(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_MeasurementControl_v3a0ext, MeasurementControl_v3a0ext_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_measurementCommand_v590ext_vals[] = {
  {   0, "intra-frequency" },
  {   1, "inter-frequency" },
  { 0, NULL }
};

static const per_choice_t T_measurementCommand_v590ext_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_intra_frequency, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_ies_Intra_FreqEventCriteriaList_v590ext },
  {   1, &hf_umts_rrc_pdu_def_inter_frequency, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_ies_Inter_FreqEventCriteriaList_v590ext },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_measurementCommand_v590ext(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_measurementCommand_v590ext, T_measurementCommand_v590ext_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t MeasurementControl_v590ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_measurementCommand_v590ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_measurementCommand_v590ext },
  { &hf_umts_rrc_pdu_def_intraFreqReportingCriteria_1b_r5, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_IntraFreqReportingCriteria_1b_r5 },
  { &hf_umts_rrc_pdu_def_intraFreqEvent_1d_r5, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_IntraFreqEvent_1d_r5 },
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier_MSP_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_MeasurementControl_v590ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_MeasurementControl_v590ext_IEs, MeasurementControl_v590ext_IEs_sequence);

  return offset;
}


static const per_sequence_t MeasurementControl_v5b0ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_interRATCellInfoIndication, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_InterRATCellInfoIndication },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_MeasurementControl_v5b0ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_MeasurementControl_v5b0ext_IEs, MeasurementControl_v5b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions31_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions31(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions31, T_nonCriticalExtensions31_sequence);

  return offset;
}


static const per_sequence_t T_v5b0NonCriticalExtensions_sequence[] = {
  { &hf_umts_rrc_pdu_def_measurementControl_v5b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_MeasurementControl_v5b0ext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions31, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions31 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v5b0NonCriticalExtensions(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v5b0NonCriticalExtensions, T_v5b0NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions8_sequence[] = {
  { &hf_umts_rrc_pdu_def_measurementControl_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_MeasurementControl_v590ext_IEs },
  { &hf_umts_rrc_pdu_def_v5b0NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v5b0NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions8(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions8, T_v590NonCriticalExtensions8_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions5_sequence[] = {
  { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions8, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions5(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions5, T_v4b0NonCriticalExtensions5_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions18_sequence[] = {
  { &hf_umts_rrc_pdu_def_measurementControl_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions5, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions18(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions18, T_laterNonCriticalExtensions18_sequence);

  return offset;
}


static const per_sequence_t T_v3a0NonCriticalExtensions4_sequence[] = {
  { &hf_umts_rrc_pdu_def_measurementControl_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_MeasurementControl_v3a0ext },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions18, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions18 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions4(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions4, T_v3a0NonCriticalExtensions4_sequence);

  return offset;
}


static const per_sequence_t T_v390nonCriticalExtensions_sequence[] = {
  { &hf_umts_rrc_pdu_def_measurementControl_v390ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_MeasurementControl_v390ext },
  { &hf_umts_rrc_pdu_def_v3a0NonCriticalExtensions4, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v390nonCriticalExtensions(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v390nonCriticalExtensions, T_v390nonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_r311_sequence[] = {
  { &hf_umts_rrc_pdu_def_measurementControl_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_MeasurementControl_r3_IEs },
  { &hf_umts_rrc_pdu_def_v390nonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v390nonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r311(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r311, T_r311_sequence);

  return offset;
}


static const per_sequence_t MeasurementControl_r4_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_measurementIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_MeasurementIdentity },
  { &hf_umts_rrc_pdu_def_measurementCommand1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_MeasurementCommand_r4 },
  { &hf_umts_rrc_pdu_def_measurementReportingMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MeasurementReportingMode },
  { &hf_umts_rrc_pdu_def_additionalMeasurementList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_AdditionalMeasurementID_List },
  { &hf_umts_rrc_pdu_def_dpch_CompressedModeStatusInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DPCH_CompressedModeStatusInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_MeasurementControl_r4_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_MeasurementControl_r4_IEs, MeasurementControl_r4_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions32_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions32(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions32, T_nonCriticalExtensions32_sequence);

  return offset;
}


static const per_sequence_t T_v5b0NonCriticalExtensions1_sequence[] = {
  { &hf_umts_rrc_pdu_def_measurementControl_v5b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_MeasurementControl_v5b0ext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions32, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions32 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v5b0NonCriticalExtensions1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v5b0NonCriticalExtensions1, T_v5b0NonCriticalExtensions1_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions9_sequence[] = {
  { &hf_umts_rrc_pdu_def_measurementControl_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_MeasurementControl_v590ext_IEs },
  { &hf_umts_rrc_pdu_def_v5b0NonCriticalExtensions1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v5b0NonCriticalExtensions1 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions9(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions9, T_v590NonCriticalExtensions9_sequence);

  return offset;
}


static const per_sequence_t T_v4d0NonCriticalExtensions3_sequence[] = {
  { &hf_umts_rrc_pdu_def_measurementControl_r4_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions9, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions3, T_v4d0NonCriticalExtensions3_sequence);

  return offset;
}


static const per_sequence_t T_r43_sequence[] = {
  { &hf_umts_rrc_pdu_def_measurementControl_r4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_MeasurementControl_r4_IEs },
  { &hf_umts_rrc_pdu_def_v4d0NonCriticalExtensions3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions3 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r43(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r43, T_r43_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions24_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions24(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_criticalExtensions24, T_criticalExtensions24_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_criticalExtensions23_vals[] = {
  {   0, "r4" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions23_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r43, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r43 },
  {   1, &hf_umts_rrc_pdu_def_criticalExtensions24, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_criticalExtensions24 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions23(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_criticalExtensions23, T_criticalExtensions23_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_later_than_r39_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_criticalExtensions23, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_criticalExtensions23 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_later_than_r39(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_later_than_r39, T_later_than_r39_sequence);

  return offset;
}


const value_string umts_rrc_pdu_def_MeasurementControl_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t MeasurementControl_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r311, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r311 },
  {   1, &hf_umts_rrc_pdu_def_later_than_r39, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_later_than_r39 },
  { 0, NULL, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_MeasurementControl(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_MeasurementControl, MeasurementControl_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t MeasurementControlFailure_v590ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier_MSP_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_MeasurementControlFailure_v590ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_MeasurementControlFailure_v590ext_IEs, MeasurementControlFailure_v590ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions33_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions33(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions33, T_nonCriticalExtensions33_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions10_sequence[] = {
  { &hf_umts_rrc_pdu_def_measurementControlFailure_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_MeasurementControlFailure_v590ext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions33, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions33 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions10(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions10, T_v590NonCriticalExtensions10_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions19_sequence[] = {
  { &hf_umts_rrc_pdu_def_measurementControlFailure_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions10, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions19(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions19, T_laterNonCriticalExtensions19_sequence);

  return offset;
}


static const per_sequence_t MeasurementControlFailure_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_failureCause, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_FailureCauseWithProtErr },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions19, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions19 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_MeasurementControlFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_MeasurementControlFailure, MeasurementControlFailure_sequence);

  return offset;
}


static const per_sequence_t MeasurementReport_v390ext_sequence[] = {
  { &hf_umts_rrc_pdu_def_measuredResults_v390ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MeasuredResults_v390ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_MeasurementReport_v390ext(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_MeasurementReport_v390ext, MeasurementReport_v390ext_sequence);

  return offset;
}


static const per_sequence_t MeasurementReport_v4b0ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_interFreqEventResults_LCR, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_InterFreqEventResults_LCR_r4_ext },
  { &hf_umts_rrc_pdu_def_additionalMeasuredResults_LCR, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MeasuredResultsList_LCR_r4_ext },
  { &hf_umts_rrc_pdu_def_gsmOTDreferenceCell, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_PrimaryCPICH_Info },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_MeasurementReport_v4b0ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_MeasurementReport_v4b0ext_IEs, MeasurementReport_v4b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t MeasurementReport_v590ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_measuredResults_v590ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MeasuredResults_v590ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_MeasurementReport_v590ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_MeasurementReport_v590ext_IEs, MeasurementReport_v590ext_IEs_sequence);

  return offset;
}


static const per_sequence_t MeasurementReport_v5b0ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_interRATCellInfoIndication, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_InterRATCellInfoIndication },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_MeasurementReport_v5b0ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_MeasurementReport_v5b0ext_IEs, MeasurementReport_v5b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t MeasurementReport_v6xyext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_measuredResultsOnRACHinterFreq, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MeasuredResultsOnRACHinterFreq },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_MeasurementReport_v6xyext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_MeasurementReport_v6xyext_IEs, MeasurementReport_v6xyext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions34_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions34(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions34, T_nonCriticalExtensions34_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions11_sequence[] = {
  { &hf_umts_rrc_pdu_def_measurementReport_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_MeasurementReport_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions34, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions34 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions11(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions11, T_v6xyNonCriticalExtensions11_sequence);

  return offset;
}


static const per_sequence_t T_v5b0NonCriticalExtensions2_sequence[] = {
  { &hf_umts_rrc_pdu_def_measurementReport_v5b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_MeasurementReport_v5b0ext_IEs },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions11, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v5b0NonCriticalExtensions2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v5b0NonCriticalExtensions2, T_v5b0NonCriticalExtensions2_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions11_sequence[] = {
  { &hf_umts_rrc_pdu_def_measurementReport_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_MeasurementReport_v590ext_IEs },
  { &hf_umts_rrc_pdu_def_v5b0NonCriticalExtensions2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v5b0NonCriticalExtensions2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions11(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions11, T_v590NonCriticalExtensions11_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions6_sequence[] = {
  { &hf_umts_rrc_pdu_def_measurementReport_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_MeasurementReport_v4b0ext_IEs },
  { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions11, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions6(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions6, T_v4b0NonCriticalExtensions6_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions20_sequence[] = {
  { &hf_umts_rrc_pdu_def_measurementReport_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions6, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions20(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions20, T_laterNonCriticalExtensions20_sequence);

  return offset;
}


static const per_sequence_t T_v390nonCriticalExtensions1_sequence[] = {
  { &hf_umts_rrc_pdu_def_measurementReport_v390ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_MeasurementReport_v390ext },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions20, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions20 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v390nonCriticalExtensions1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v390nonCriticalExtensions1, T_v390nonCriticalExtensions1_sequence);

  return offset;
}


static const per_sequence_t MeasurementReport_sequence[] = {
  { &hf_umts_rrc_pdu_def_measurementIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_MeasurementIdentity },
  { &hf_umts_rrc_pdu_def_measuredResults, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MeasuredResults },
  { &hf_umts_rrc_pdu_def_measuredResultsOnRACH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MeasuredResultsOnRACH },
  { &hf_umts_rrc_pdu_def_additionalMeasuredResults, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MeasuredResultsList },
  { &hf_umts_rrc_pdu_def_eventResults, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_EventResults },
  { &hf_umts_rrc_pdu_def_v390nonCriticalExtensions1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v390nonCriticalExtensions1 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_MeasurementReport(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_MeasurementReport, MeasurementReport_sequence);

  return offset;
}


static const per_sequence_t PagingType1_v590ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_pagingRecord2List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_PagingRecord2List_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_PagingType1_v590ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_PagingType1_v590ext_IEs, PagingType1_v590ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions35_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions35(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions35, T_nonCriticalExtensions35_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions12_sequence[] = {
  { &hf_umts_rrc_pdu_def_pagingType1_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_PagingType1_v590ext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions35, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions35 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions12(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions12, T_v590NonCriticalExtensions12_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions21_sequence[] = {
  { &hf_umts_rrc_pdu_def_pagingType1_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions12, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions12 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions21(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions21, T_laterNonCriticalExtensions21_sequence);

  return offset;
}


static const per_sequence_t PagingType1_sequence[] = {
  { &hf_umts_rrc_pdu_def_pagingRecordList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_PagingRecordList },
  { &hf_umts_rrc_pdu_def_bcch_ModificationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_BCCH_ModificationInfo },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions21, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions21 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_PagingType1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_PagingType1, PagingType1_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions36_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions36(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions36, T_nonCriticalExtensions36_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions22_sequence[] = {
  { &hf_umts_rrc_pdu_def_pagingType2_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions36, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions36 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions22(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions22, T_laterNonCriticalExtensions22_sequence);

  return offset;
}


static const per_sequence_t PagingType2_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_pagingCause, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_PagingCause },
  { &hf_umts_rrc_pdu_def_cn_DomainIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_CN_DomainIdentity },
  { &hf_umts_rrc_pdu_def_pagingRecordTypeID, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_PagingRecordTypeID },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions22, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions22 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_PagingType2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_PagingType2, PagingType2_sequence);

  return offset;
}


static const per_sequence_t T_fdd12_sequence[] = {
  { &hf_umts_rrc_pdu_def_dummy_DL_PDSCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_PDSCH_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_fdd12(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_fdd12, T_fdd12_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_modeSpecificInfo6_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo6_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_fdd12, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_fdd12 },
  {   1, &hf_umts_rrc_pdu_def_tdd, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_modeSpecificInfo6(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_modeSpecificInfo6, T_modeSpecificInfo6_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t PhysicalChannelReconfiguration_r3_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_IntegrityProtectionModeInfo },
  { &hf_umts_rrc_pdu_def_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CipheringModeInfo },
  { &hf_umts_rrc_pdu_def_activationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_new_U_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_U_RNTI },
  { &hf_umts_rrc_pdu_def_new_C_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_C_RNTI },
  { &hf_umts_rrc_pdu_def_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_StateIndicator },
  { &hf_umts_rrc_pdu_def_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_umts_rrc_pdu_def_cn_InformationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CN_InformationInfo },
  { &hf_umts_rrc_pdu_def_ura_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_URA_Identity },
  { &hf_umts_rrc_pdu_def_dl_CounterSynchronisationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CounterSynchronisationInfo },
  { &hf_umts_rrc_pdu_def_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_FrequencyInfo },
  { &hf_umts_rrc_pdu_def_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MaxAllowedUL_TX_Power },
  { &hf_umts_rrc_pdu_def_ul_ChannelRequirement4, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_ChannelRequirementWithCPCH_SetID },
  { &hf_umts_rrc_pdu_def_modeSpecificInfo6, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_modeSpecificInfo6 },
  { &hf_umts_rrc_pdu_def_dl_CommonInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonInformation },
  { &hf_umts_rrc_pdu_def_dl_InformationPerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_InformationPerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_PhysicalChannelReconfiguration_r3_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_PhysicalChannelReconfiguration_r3_IEs, PhysicalChannelReconfiguration_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t PhysicalChannelReconfiguration_v3a0ext_sequence[] = {
  { &hf_umts_rrc_pdu_def_new_DSCH_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DSCH_RNTI },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_PhysicalChannelReconfiguration_v3a0ext(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_PhysicalChannelReconfiguration_v3a0ext, PhysicalChannelReconfiguration_v3a0ext_sequence);

  return offset;
}


static const per_sequence_t PhysicalChannelReconfiguration_v4b0ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_dummy_SSDT_UL, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_SSDT_UL },
  { &hf_umts_rrc_pdu_def_cell_id_PerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CellIdentity_PerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_PhysicalChannelReconfiguration_v4b0ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_PhysicalChannelReconfiguration_v4b0ext_IEs, PhysicalChannelReconfiguration_v4b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t PhysicalChannelReconfiguration_v590ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_dl_TPC_PowerOffsetPerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_TPC_PowerOffsetPerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_PhysicalChannelReconfiguration_v590ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_PhysicalChannelReconfiguration_v590ext_IEs, PhysicalChannelReconfiguration_v590ext_IEs_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_postVerificationPeriod1_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_umts_rrc_pdu_def_T_postVerificationPeriod1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_timingMaintainedSynchInd_vals[] = {
  {   0, "false" },
  { 0, NULL }
};


static int
dissect_umts_rrc_pdu_def_T_timingMaintainedSynchInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t PhysicalChannelReconfiguration_v6xyext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_delayRestrictionFlag, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DelayRestrictionFlag },
  { &hf_umts_rrc_pdu_def_primary_plmn_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_PLMN_Identity },
  { &hf_umts_rrc_pdu_def_harq_Preamble_Mode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_HARQ_Preamble_Mode },
  { &hf_umts_rrc_pdu_def_beaconPLEst, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_BEACON_PL_Est },
  { &hf_umts_rrc_pdu_def_postVerificationPeriod1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_postVerificationPeriod1 },
  { &hf_umts_rrc_pdu_def_dhs_sync, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DHS_Sync },
  { &hf_umts_rrc_pdu_def_timingMaintainedSynchInd, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_timingMaintainedSynchInd },
  { &hf_umts_rrc_pdu_def_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MBMS_PL_ServiceRestrictInfo_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_PhysicalChannelReconfiguration_v6xyext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_PhysicalChannelReconfiguration_v6xyext_IEs, PhysicalChannelReconfiguration_v6xyext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions37_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions37(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions37, T_nonCriticalExtensions37_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions12_sequence[] = {
  { &hf_umts_rrc_pdu_def_physicalChannelReconfiguration_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_PhysicalChannelReconfiguration_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions37, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions37 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions12(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions12, T_v6xyNonCriticalExtensions12_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtenstions2_sequence[] = {
  { &hf_umts_rrc_pdu_def_physicalChannelReconfiguration_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_PhysicalChannelReconfiguration_v590ext_IEs },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions12, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions12 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v590NonCriticalExtenstions2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v590NonCriticalExtenstions2, T_v590NonCriticalExtenstions2_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtenstions_sequence[] = {
  { &hf_umts_rrc_pdu_def_physicalChannelReconfiguration_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_PhysicalChannelReconfiguration_v4b0ext_IEs },
  { &hf_umts_rrc_pdu_def_v590NonCriticalExtenstions2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v590NonCriticalExtenstions2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v4b0NonCriticalExtenstions(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtenstions, T_v4b0NonCriticalExtenstions_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions23_sequence[] = {
  { &hf_umts_rrc_pdu_def_physicalChannelReconfiguration_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v4b0NonCriticalExtenstions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v4b0NonCriticalExtenstions },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions23(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions23, T_laterNonCriticalExtensions23_sequence);

  return offset;
}


static const per_sequence_t T_v3a0NonCriticalExtensions5_sequence[] = {
  { &hf_umts_rrc_pdu_def_physicalChannelReconfiguration_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_PhysicalChannelReconfiguration_v3a0ext },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions23, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions23 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions5(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions5, T_v3a0NonCriticalExtensions5_sequence);

  return offset;
}


static const per_sequence_t T_r312_sequence[] = {
  { &hf_umts_rrc_pdu_def_physicalChannelReconfiguration_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_PhysicalChannelReconfiguration_r3_IEs },
  { &hf_umts_rrc_pdu_def_v3a0NonCriticalExtensions5, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r312(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r312, T_r312_sequence);

  return offset;
}


static const per_sequence_t T_fdd13_sequence[] = {
  { &hf_umts_rrc_pdu_def_dummy_DL_PDSCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_PDSCH_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_fdd13(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_fdd13, T_fdd13_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_modeSpecificInfo7_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo7_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_fdd13, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_fdd13 },
  {   1, &hf_umts_rrc_pdu_def_tdd, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_modeSpecificInfo7(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_modeSpecificInfo7, T_modeSpecificInfo7_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t PhysicalChannelReconfiguration_r4_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_IntegrityProtectionModeInfo },
  { &hf_umts_rrc_pdu_def_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CipheringModeInfo },
  { &hf_umts_rrc_pdu_def_activationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_new_U_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_U_RNTI },
  { &hf_umts_rrc_pdu_def_new_C_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_C_RNTI },
  { &hf_umts_rrc_pdu_def_new_DSCH_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DSCH_RNTI },
  { &hf_umts_rrc_pdu_def_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_StateIndicator },
  { &hf_umts_rrc_pdu_def_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_umts_rrc_pdu_def_cn_InformationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CN_InformationInfo },
  { &hf_umts_rrc_pdu_def_ura_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_URA_Identity },
  { &hf_umts_rrc_pdu_def_dl_CounterSynchronisationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CounterSynchronisationInfo },
  { &hf_umts_rrc_pdu_def_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_FrequencyInfo },
  { &hf_umts_rrc_pdu_def_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MaxAllowedUL_TX_Power },
  { &hf_umts_rrc_pdu_def_ul_ChannelRequirement5, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_ChannelRequirementWithCPCH_SetID_r4 },
  { &hf_umts_rrc_pdu_def_modeSpecificInfo7, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_modeSpecificInfo7 },
  { &hf_umts_rrc_pdu_def_dl_CommonInformation1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonInformation_r4 },
  { &hf_umts_rrc_pdu_def_dl_InformationPerRL_List1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_InformationPerRL_List_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_PhysicalChannelReconfiguration_r4_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_PhysicalChannelReconfiguration_r4_IEs, PhysicalChannelReconfiguration_r4_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions38_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions38(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions38, T_nonCriticalExtensions38_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions13_sequence[] = {
  { &hf_umts_rrc_pdu_def_physicalChannelReconfiguration_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_PhysicalChannelReconfiguration_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions38, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions38 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions13(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions13, T_v6xyNonCriticalExtensions13_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtenstions3_sequence[] = {
  { &hf_umts_rrc_pdu_def_physicalChannelReconfiguration_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_PhysicalChannelReconfiguration_v590ext_IEs },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions13, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions13 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v590NonCriticalExtenstions3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v590NonCriticalExtenstions3, T_v590NonCriticalExtenstions3_sequence);

  return offset;
}


static const per_sequence_t T_v4d0NonCriticalExtensions4_sequence[] = {
  { &hf_umts_rrc_pdu_def_physicalChannelReconfiguration_r4_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v590NonCriticalExtenstions3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v590NonCriticalExtenstions3 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions4(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions4, T_v4d0NonCriticalExtensions4_sequence);

  return offset;
}


static const per_sequence_t T_r44_sequence[] = {
  { &hf_umts_rrc_pdu_def_physicalChannelReconfiguration_r4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_PhysicalChannelReconfiguration_r4_IEs },
  { &hf_umts_rrc_pdu_def_v4d0NonCriticalExtensions4, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r44(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r44, T_r44_sequence);

  return offset;
}


static const per_sequence_t T_fdd14_sequence[] = {
  { &hf_umts_rrc_pdu_def_dummy_DL_PDSCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_PDSCH_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_fdd14(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_fdd14, T_fdd14_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_modeSpecificInfo8_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo8_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_fdd14, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_fdd14 },
  {   1, &hf_umts_rrc_pdu_def_tdd, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_modeSpecificInfo8(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_modeSpecificInfo8, T_modeSpecificInfo8_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t PhysicalChannelReconfiguration_r5_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_IntegrityProtectionModeInfo },
  { &hf_umts_rrc_pdu_def_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CipheringModeInfo },
  { &hf_umts_rrc_pdu_def_activationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_new_U_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_U_RNTI },
  { &hf_umts_rrc_pdu_def_new_C_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_C_RNTI },
  { &hf_umts_rrc_pdu_def_new_DSCH_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DSCH_RNTI },
  { &hf_umts_rrc_pdu_def_new_H_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_H_RNTI },
  { &hf_umts_rrc_pdu_def_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_StateIndicator },
  { &hf_umts_rrc_pdu_def_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_umts_rrc_pdu_def_cn_InformationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CN_InformationInfo },
  { &hf_umts_rrc_pdu_def_ura_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_URA_Identity },
  { &hf_umts_rrc_pdu_def_dl_CounterSynchronisationInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CounterSynchronisationInfo_r5 },
  { &hf_umts_rrc_pdu_def_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_FrequencyInfo },
  { &hf_umts_rrc_pdu_def_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MaxAllowedUL_TX_Power },
  { &hf_umts_rrc_pdu_def_ul_ChannelRequirement6, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_ChannelRequirementWithCPCH_SetID_r5 },
  { &hf_umts_rrc_pdu_def_modeSpecificInfo8, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_modeSpecificInfo8 },
  { &hf_umts_rrc_pdu_def_dl_HSPDSCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_HSPDSCH_Information },
  { &hf_umts_rrc_pdu_def_dl_CommonInformation2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonInformation_r5 },
  { &hf_umts_rrc_pdu_def_dl_InformationPerRL_List2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_InformationPerRL_List_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_PhysicalChannelReconfiguration_r5_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_PhysicalChannelReconfiguration_r5_IEs, PhysicalChannelReconfiguration_r5_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions39_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions39(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions39, T_nonCriticalExtensions39_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions14_sequence[] = {
  { &hf_umts_rrc_pdu_def_physicalChannelReconfiguration_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_PhysicalChannelReconfiguration_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions39, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions39 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions14(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions14, T_v6xyNonCriticalExtensions14_sequence);

  return offset;
}


static const per_sequence_t T_r54_sequence[] = {
  { &hf_umts_rrc_pdu_def_physicalChannelReconfiguration_r5, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_PhysicalChannelReconfiguration_r5_IEs },
  { &hf_umts_rrc_pdu_def_physicalChannelReconfiguration_r5_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions14, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions14 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r54(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r54, T_r54_sequence);

  return offset;
}


static const per_sequence_t PhysicalChannelReconfiguration_r6_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_IntegrityProtectionModeInfo },
  { &hf_umts_rrc_pdu_def_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CipheringModeInfo },
  { &hf_umts_rrc_pdu_def_activationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_delayRestrictionFlag, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DelayRestrictionFlag },
  { &hf_umts_rrc_pdu_def_new_U_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_U_RNTI },
  { &hf_umts_rrc_pdu_def_new_C_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_C_RNTI },
  { &hf_umts_rrc_pdu_def_new_DSCH_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DSCH_RNTI },
  { &hf_umts_rrc_pdu_def_new_H_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_H_RNTI },
  { &hf_umts_rrc_pdu_def_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_E_RNTI },
  { &hf_umts_rrc_pdu_def_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_E_RNTI },
  { &hf_umts_rrc_pdu_def_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_StateIndicator },
  { &hf_umts_rrc_pdu_def_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_umts_rrc_pdu_def_cn_InformationInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CN_InformationInfo_r6 },
  { &hf_umts_rrc_pdu_def_ura_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_URA_Identity },
  { &hf_umts_rrc_pdu_def_dl_CounterSynchronisationInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CounterSynchronisationInfo_r5 },
  { &hf_umts_rrc_pdu_def_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_FrequencyInfo },
  { &hf_umts_rrc_pdu_def_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MaxAllowedUL_TX_Power },
  { &hf_umts_rrc_pdu_def_ul_ChannelRequirement3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_ChannelRequirement_r6 },
  { &hf_umts_rrc_pdu_def_ul_EDCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_EDCH_Information_r6 },
  { &hf_umts_rrc_pdu_def_dl_HSPDSCH_Information1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_HSPDSCH_Information_r6 },
  { &hf_umts_rrc_pdu_def_dl_CommonInformation3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonInformation_r6 },
  { &hf_umts_rrc_pdu_def_dl_InformationPerRL_List3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_InformationPerRL_List_r6 },
  { &hf_umts_rrc_pdu_def_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MBMS_PL_ServiceRestrictInfo_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_PhysicalChannelReconfiguration_r6_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_PhysicalChannelReconfiguration_r6_IEs, PhysicalChannelReconfiguration_r6_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions40_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions40(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions40, T_nonCriticalExtensions40_sequence);

  return offset;
}


static const per_sequence_t T_r65_sequence[] = {
  { &hf_umts_rrc_pdu_def_physicalChannelReconfiguration_r6, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_PhysicalChannelReconfiguration_r6_IEs },
  { &hf_umts_rrc_pdu_def_physicalChannelReconfiguration_r6_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions40, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions40 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r65(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r65, T_r65_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions28_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions28(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_criticalExtensions28, T_criticalExtensions28_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_criticalExtensions27_vals[] = {
  {   0, "r6" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions27_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r65, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r65 },
  {   1, &hf_umts_rrc_pdu_def_criticalExtensions28, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_criticalExtensions28 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions27(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_criticalExtensions27, T_criticalExtensions27_choice,
                                 NULL);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_criticalExtensions26_vals[] = {
  {   0, "r5" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions26_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r54, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r54 },
  {   1, &hf_umts_rrc_pdu_def_criticalExtensions27, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_criticalExtensions27 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions26(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_criticalExtensions26, T_criticalExtensions26_choice,
                                 NULL);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_criticalExtensions25_vals[] = {
  {   0, "r4" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions25_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r44, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r44 },
  {   1, &hf_umts_rrc_pdu_def_criticalExtensions26, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_criticalExtensions26 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions25(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_criticalExtensions25, T_criticalExtensions25_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_later_than_r310_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_criticalExtensions25, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_criticalExtensions25 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_later_than_r310(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_later_than_r310, T_later_than_r310_sequence);

  return offset;
}


const value_string umts_rrc_pdu_def_PhysicalChannelReconfiguration_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t PhysicalChannelReconfiguration_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r312, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r312 },
  {   1, &hf_umts_rrc_pdu_def_later_than_r310, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_later_than_r310 },
  { 0, NULL, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_PhysicalChannelReconfiguration(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_PhysicalChannelReconfiguration, PhysicalChannelReconfiguration_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions41_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions41(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions41, T_nonCriticalExtensions41_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions24_sequence[] = {
  { &hf_umts_rrc_pdu_def_physicalChannelReconfigurationComplete_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions41, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions41 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions24(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions24, T_laterNonCriticalExtensions24_sequence);

  return offset;
}


static const per_sequence_t PhysicalChannelReconfigurationComplete_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_ul_IntegProtActivationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_IntegrityProtActivationInfo },
  { &hf_umts_rrc_pdu_def_ul_TimingAdvance, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_TimingAdvance },
  { &hf_umts_rrc_pdu_def_count_C_ActivationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_dummy_RB_ActivationTimeInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_ActivationTimeInfoList },
  { &hf_umts_rrc_pdu_def_ul_CounterSynchronisationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_CounterSynchronisationInfo },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions24, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions24 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_PhysicalChannelReconfigurationComplete(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_PhysicalChannelReconfigurationComplete, PhysicalChannelReconfigurationComplete_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions42_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions42(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions42, T_nonCriticalExtensions42_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions25_sequence[] = {
  { &hf_umts_rrc_pdu_def_physicalChannelReconfigurationFailure_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions42, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions42 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions25(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions25, T_laterNonCriticalExtensions25_sequence);

  return offset;
}


static const per_sequence_t PhysicalChannelReconfigurationFailure_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_failureCause, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_FailureCauseWithProtErr },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions25, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions25 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_PhysicalChannelReconfigurationFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_PhysicalChannelReconfigurationFailure, PhysicalChannelReconfigurationFailure_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_confirmRequest_vals[] = {
  {   0, "confirmPDSCH" },
  {   1, "confirmPUSCH" },
  { 0, NULL }
};


static int
dissect_umts_rrc_pdu_def_T_confirmRequest(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_umts_rrc_pdu_def_INTEGER_0_255(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 255U, NULL, FALSE);

  return offset;
}


static const per_sequence_t PhysicalSharedChannelAllocation_r3_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_dsch_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DSCH_RNTI },
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_ul_TimingAdvance1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_TimingAdvanceControl },
  { &hf_umts_rrc_pdu_def_pusch_CapacityAllocationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_PUSCH_CapacityAllocationInfo },
  { &hf_umts_rrc_pdu_def_pdsch_CapacityAllocationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_PDSCH_CapacityAllocationInfo },
  { &hf_umts_rrc_pdu_def_confirmRequest, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_confirmRequest },
  { &hf_umts_rrc_pdu_def_trafficVolumeReportRequest, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_INTEGER_0_255 },
  { &hf_umts_rrc_pdu_def_iscpTimeslotList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_TimeslotList },
  { &hf_umts_rrc_pdu_def_requestPCCPCHRSCP, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_PhysicalSharedChannelAllocation_r3_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_PhysicalSharedChannelAllocation_r3_IEs, PhysicalSharedChannelAllocation_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions43_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions43(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions43, T_nonCriticalExtensions43_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions26_sequence[] = {
  { &hf_umts_rrc_pdu_def_physicalSharedChannelAllocation_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions43, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions43 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions26(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions26, T_laterNonCriticalExtensions26_sequence);

  return offset;
}


static const per_sequence_t T_r313_sequence[] = {
  { &hf_umts_rrc_pdu_def_physicalSharedChannelAllocation_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_PhysicalSharedChannelAllocation_r3_IEs },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions26, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions26 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r313(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r313, T_r313_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_confirmRequest1_vals[] = {
  {   0, "confirmPDSCH" },
  {   1, "confirmPUSCH" },
  { 0, NULL }
};


static int
dissect_umts_rrc_pdu_def_T_confirmRequest1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t PhysicalSharedChannelAllocation_r4_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_ul_TimingAdvance2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_TimingAdvanceControl_r4 },
  { &hf_umts_rrc_pdu_def_pusch_CapacityAllocationInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_PUSCH_CapacityAllocationInfo_r4 },
  { &hf_umts_rrc_pdu_def_pdsch_CapacityAllocationInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_PDSCH_CapacityAllocationInfo_r4 },
  { &hf_umts_rrc_pdu_def_confirmRequest1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_confirmRequest1 },
  { &hf_umts_rrc_pdu_def_trafficVolumeReportRequest, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_INTEGER_0_255 },
  { &hf_umts_rrc_pdu_def_iscpTimeslotList1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_TimeslotList_r4 },
  { &hf_umts_rrc_pdu_def_requestPCCPCHRSCP, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_PhysicalSharedChannelAllocation_r4_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_PhysicalSharedChannelAllocation_r4_IEs, PhysicalSharedChannelAllocation_r4_IEs_sequence);

  return offset;
}


static const per_sequence_t PhysicalSharedChannelAllocation_v6xyext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_beaconPLEst, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_BEACON_PL_Est },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_PhysicalSharedChannelAllocation_v6xyext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_PhysicalSharedChannelAllocation_v6xyext_IEs, PhysicalSharedChannelAllocation_v6xyext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions44_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions44(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions44, T_nonCriticalExtensions44_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions15_sequence[] = {
  { &hf_umts_rrc_pdu_def_physicalSharedChannelAllocation_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_PhysicalSharedChannelAllocation_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions44, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions44 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions15(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions15, T_v6xyNonCriticalExtensions15_sequence);

  return offset;
}


static const per_sequence_t T_v4d0NonCriticalExtensions5_sequence[] = {
  { &hf_umts_rrc_pdu_def_physicalSharedChannelAllocation_r4_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions15, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions15 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions5(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions5, T_v4d0NonCriticalExtensions5_sequence);

  return offset;
}


static const per_sequence_t T_r45_sequence[] = {
  { &hf_umts_rrc_pdu_def_physicalSharedChannelAllocation_r4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_PhysicalSharedChannelAllocation_r4_IEs },
  { &hf_umts_rrc_pdu_def_v4d0NonCriticalExtensions5, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r45(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r45, T_r45_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions30_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions30(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_criticalExtensions30, T_criticalExtensions30_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_criticalExtensions29_vals[] = {
  {   0, "r4" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions29_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r45, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r45 },
  {   1, &hf_umts_rrc_pdu_def_criticalExtensions30, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_criticalExtensions30 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions29(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_criticalExtensions29, T_criticalExtensions29_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_later_than_r311_sequence[] = {
  { &hf_umts_rrc_pdu_def_dsch_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DSCH_RNTI },
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_criticalExtensions29, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_criticalExtensions29 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_later_than_r311(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_later_than_r311, T_later_than_r311_sequence);

  return offset;
}


const value_string umts_rrc_pdu_def_PhysicalSharedChannelAllocation_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t PhysicalSharedChannelAllocation_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r313, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r313 },
  {   1, &hf_umts_rrc_pdu_def_later_than_r311, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_later_than_r311 },
  { 0, NULL, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_PhysicalSharedChannelAllocation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_PhysicalSharedChannelAllocation, PhysicalSharedChannelAllocation_choice,
                                 NULL);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_allocationConfirmation_vals[] = {
  {   0, "pdschConfirmation" },
  {   1, "puschConfirmation" },
  { 0, NULL }
};

static const per_choice_t T_allocationConfirmation_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_pdschConfirmation, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_ies_PDSCH_Identity },
  {   1, &hf_umts_rrc_pdu_def_puschConfirmation, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_ies_PUSCH_Identity },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_allocationConfirmation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_allocationConfirmation, T_allocationConfirmation_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t PUSCHCapacityRequest_v590ext_sequence[] = {
  { &hf_umts_rrc_pdu_def_primaryCCPCH_RSCP_delta, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DeltaRSCP },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_PUSCHCapacityRequest_v590ext(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_PUSCHCapacityRequest_v590ext, PUSCHCapacityRequest_v590ext_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions45_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions45(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions45, T_nonCriticalExtensions45_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions13_sequence[] = {
  { &hf_umts_rrc_pdu_def_puschCapacityRequest_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_PUSCHCapacityRequest_v590ext },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions45, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions45 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions13(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions13, T_v590NonCriticalExtensions13_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions27_sequence[] = {
  { &hf_umts_rrc_pdu_def_puschCapacityRequest_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions13, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions13 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions27(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions27, T_laterNonCriticalExtensions27_sequence);

  return offset;
}


static const per_sequence_t PUSCHCapacityRequest_sequence[] = {
  { &hf_umts_rrc_pdu_def_dsch_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DSCH_RNTI },
  { &hf_umts_rrc_pdu_def_trafficVolume, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_TrafficVolumeMeasuredResultsList },
  { &hf_umts_rrc_pdu_def_timeslotListWithISCP, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_TimeslotListWithISCP },
  { &hf_umts_rrc_pdu_def_primaryCCPCH_RSCP, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_PrimaryCCPCH_RSCP },
  { &hf_umts_rrc_pdu_def_allocationConfirmation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_allocationConfirmation },
  { &hf_umts_rrc_pdu_def_protocolErrorIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_ProtocolErrorIndicatorWithMoreInfo },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions27, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions27 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_PUSCHCapacityRequest(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_PUSCHCapacityRequest, PUSCHCapacityRequest_sequence);

  return offset;
}


static const per_sequence_t T_fdd15_sequence[] = {
  { &hf_umts_rrc_pdu_def_dummy11, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CPCH_SetID },
  { &hf_umts_rrc_pdu_def_dummy2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DRAC_StaticInformationList },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_fdd15(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_fdd15, T_fdd15_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_dummy1_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_dummy1_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_fdd15, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_fdd15 },
  {   1, &hf_umts_rrc_pdu_def_tdd, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_dummy1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_dummy1, T_dummy1_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd16_sequence[] = {
  { &hf_umts_rrc_pdu_def_dummy_DL_PDSCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_PDSCH_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_fdd16(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_fdd16, T_fdd16_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_modeSpecificPhysChInfo3_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificPhysChInfo3_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_fdd16, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_fdd16 },
  {   1, &hf_umts_rrc_pdu_def_tdd, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_modeSpecificPhysChInfo3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo3, T_modeSpecificPhysChInfo3_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RadioBearerReconfiguration_r3_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_IntegrityProtectionModeInfo },
  { &hf_umts_rrc_pdu_def_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CipheringModeInfo },
  { &hf_umts_rrc_pdu_def_activationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_new_U_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_U_RNTI },
  { &hf_umts_rrc_pdu_def_new_C_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_C_RNTI },
  { &hf_umts_rrc_pdu_def_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_StateIndicator },
  { &hf_umts_rrc_pdu_def_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_umts_rrc_pdu_def_cn_InformationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CN_InformationInfo },
  { &hf_umts_rrc_pdu_def_ura_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_URA_Identity },
  { &hf_umts_rrc_pdu_def_rab_InformationReconfigList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RAB_InformationReconfigList },
  { &hf_umts_rrc_pdu_def_rb_InformationReconfigList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RB_InformationReconfigList },
  { &hf_umts_rrc_pdu_def_rb_InformationAffectedList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_InformationAffectedList },
  { &hf_umts_rrc_pdu_def_ul_CommonTransChInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_CommonTransChInfo },
  { &hf_umts_rrc_pdu_def_ul_deletedTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_DeletedTransChInfoList },
  { &hf_umts_rrc_pdu_def_ul_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_AddReconfTransChInfoList },
  { &hf_umts_rrc_pdu_def_dummy1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_dummy1 },
  { &hf_umts_rrc_pdu_def_dl_CommonTransChInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonTransChInfo },
  { &hf_umts_rrc_pdu_def_dl_DeletedTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_DeletedTransChInfoList },
  { &hf_umts_rrc_pdu_def_dl_AddReconfTransChInfoList3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_AddReconfTransChInfo2List },
  { &hf_umts_rrc_pdu_def_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_FrequencyInfo },
  { &hf_umts_rrc_pdu_def_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MaxAllowedUL_TX_Power },
  { &hf_umts_rrc_pdu_def_ul_ChannelRequirement, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_ChannelRequirement },
  { &hf_umts_rrc_pdu_def_modeSpecificPhysChInfo3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_modeSpecificPhysChInfo3 },
  { &hf_umts_rrc_pdu_def_dl_CommonInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonInformation },
  { &hf_umts_rrc_pdu_def_dl_InformationPerRL_List, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DL_InformationPerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RadioBearerReconfiguration_r3_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RadioBearerReconfiguration_r3_IEs, RadioBearerReconfiguration_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerReconfiguration_v3a0ext_sequence[] = {
  { &hf_umts_rrc_pdu_def_new_DSCH_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DSCH_RNTI },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RadioBearerReconfiguration_v3a0ext(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RadioBearerReconfiguration_v3a0ext, RadioBearerReconfiguration_v3a0ext_sequence);

  return offset;
}


static const per_sequence_t RadioBearerReconfiguration_v4b0ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_dummy_SSDT_UL, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_SSDT_UL },
  { &hf_umts_rrc_pdu_def_cell_id_PerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CellIdentity_PerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RadioBearerReconfiguration_v4b0ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RadioBearerReconfiguration_v4b0ext_IEs, RadioBearerReconfiguration_v4b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerReconfiguration_v590ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_dl_TPC_PowerOffsetPerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_TPC_PowerOffsetPerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RadioBearerReconfiguration_v590ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RadioBearerReconfiguration_v590ext_IEs, RadioBearerReconfiguration_v590ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerReconfiguration_v5d0ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_pdcp_ROHC_TargetMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_PDCP_ROHC_TargetMode },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RadioBearerReconfiguration_v5d0ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RadioBearerReconfiguration_v5d0ext_IEs, RadioBearerReconfiguration_v5d0ext_IEs_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_postVerificationPeriod2_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_umts_rrc_pdu_def_T_postVerificationPeriod2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_timingMaintainedSynchInd1_vals[] = {
  {   0, "false" },
  { 0, NULL }
};


static int
dissect_umts_rrc_pdu_def_T_timingMaintainedSynchInd1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t RadioBearerReconfiguration_v6xyext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_delayRestrictionFlag, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DelayRestrictionFlag },
  { &hf_umts_rrc_pdu_def_primary_plmn_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_PLMN_Identity },
  { &hf_umts_rrc_pdu_def_harq_Preamble_Mode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_HARQ_Preamble_Mode },
  { &hf_umts_rrc_pdu_def_beaconPLEst, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_BEACON_PL_Est },
  { &hf_umts_rrc_pdu_def_postVerificationPeriod2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_postVerificationPeriod2 },
  { &hf_umts_rrc_pdu_def_dhs_sync, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DHS_Sync },
  { &hf_umts_rrc_pdu_def_timingMaintainedSynchInd1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_timingMaintainedSynchInd1 },
  { &hf_umts_rrc_pdu_def_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MBMS_PL_ServiceRestrictInfo_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RadioBearerReconfiguration_v6xyext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RadioBearerReconfiguration_v6xyext_IEs, RadioBearerReconfiguration_v6xyext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions46_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions46(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions46, T_nonCriticalExtensions46_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions16_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerReconfiguration_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RadioBearerReconfiguration_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions46, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions46 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions16(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions16, T_v6xyNonCriticalExtensions16_sequence);

  return offset;
}


static const per_sequence_t T_v5d0NonCriticalExtenstions6_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerReconfiguration_v5d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RadioBearerReconfiguration_v5d0ext_IEs },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions16, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions16 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions6(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions6, T_v5d0NonCriticalExtenstions6_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions14_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerReconfiguration_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RadioBearerReconfiguration_v590ext_IEs },
  { &hf_umts_rrc_pdu_def_v5d0NonCriticalExtenstions6, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions14(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions14, T_v590NonCriticalExtensions14_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions7_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerReconfiguration_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RadioBearerReconfiguration_v4b0ext_IEs },
  { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions14, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions14 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions7(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions7, T_v4b0NonCriticalExtensions7_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions28_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerReconfiguration_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions7, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions28(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions28, T_laterNonCriticalExtensions28_sequence);

  return offset;
}


static const per_sequence_t T_v3aoNonCriticalExtensions_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerReconfiguration_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RadioBearerReconfiguration_v3a0ext },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions28, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions28 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v3aoNonCriticalExtensions(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v3aoNonCriticalExtensions, T_v3aoNonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_r314_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerReconfiguration_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RadioBearerReconfiguration_r3_IEs },
  { &hf_umts_rrc_pdu_def_v3aoNonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v3aoNonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r314(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r314, T_r314_sequence);

  return offset;
}


static const per_sequence_t T_fdd17_sequence[] = {
  { &hf_umts_rrc_pdu_def_dummy1_CPCH_SetID, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CPCH_SetID },
  { &hf_umts_rrc_pdu_def_dummy2_DRAC_StaticInformationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DRAC_StaticInformationList },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_fdd17(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_fdd17, T_fdd17_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_radioBearerReconfiguration_r4_IEs_dummy_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_radioBearerReconfiguration_r4_IEs_dummy_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_fdd17, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_fdd17 },
  {   1, &hf_umts_rrc_pdu_def_tdd, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_radioBearerReconfiguration_r4_IEs_dummy(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_radioBearerReconfiguration_r4_IEs_dummy, T_radioBearerReconfiguration_r4_IEs_dummy_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd18_sequence[] = {
  { &hf_umts_rrc_pdu_def_dummy_DL_PDSCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_PDSCH_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_fdd18(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_fdd18, T_fdd18_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_modeSpecificPhysChInfo4_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificPhysChInfo4_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_fdd18, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_fdd18 },
  {   1, &hf_umts_rrc_pdu_def_tdd, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_modeSpecificPhysChInfo4(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo4, T_modeSpecificPhysChInfo4_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RadioBearerReconfiguration_r4_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_IntegrityProtectionModeInfo },
  { &hf_umts_rrc_pdu_def_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CipheringModeInfo },
  { &hf_umts_rrc_pdu_def_activationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_new_U_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_U_RNTI },
  { &hf_umts_rrc_pdu_def_new_C_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_C_RNTI },
  { &hf_umts_rrc_pdu_def_new_DSCH_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DSCH_RNTI },
  { &hf_umts_rrc_pdu_def_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_StateIndicator },
  { &hf_umts_rrc_pdu_def_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_umts_rrc_pdu_def_cn_InformationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CN_InformationInfo },
  { &hf_umts_rrc_pdu_def_ura_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_URA_Identity },
  { &hf_umts_rrc_pdu_def_rab_InformationReconfigList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RAB_InformationReconfigList },
  { &hf_umts_rrc_pdu_def_rb_InformationReconfigList1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_InformationReconfigList_r4 },
  { &hf_umts_rrc_pdu_def_rb_InformationAffectedList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_InformationAffectedList },
  { &hf_umts_rrc_pdu_def_ul_CommonTransChInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_CommonTransChInfo_r4 },
  { &hf_umts_rrc_pdu_def_ul_deletedTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_DeletedTransChInfoList },
  { &hf_umts_rrc_pdu_def_ul_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_AddReconfTransChInfoList },
  { &hf_umts_rrc_pdu_def_radioBearerReconfiguration_r4_IEs_dummy, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_radioBearerReconfiguration_r4_IEs_dummy },
  { &hf_umts_rrc_pdu_def_dl_CommonTransChInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonTransChInfo_r4 },
  { &hf_umts_rrc_pdu_def_dl_DeletedTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_DeletedTransChInfoList },
  { &hf_umts_rrc_pdu_def_dl_AddReconfTransChInfoList1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_AddReconfTransChInfoList_r4 },
  { &hf_umts_rrc_pdu_def_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_FrequencyInfo },
  { &hf_umts_rrc_pdu_def_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MaxAllowedUL_TX_Power },
  { &hf_umts_rrc_pdu_def_ul_ChannelRequirement1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_ChannelRequirement_r4 },
  { &hf_umts_rrc_pdu_def_modeSpecificPhysChInfo4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_modeSpecificPhysChInfo4 },
  { &hf_umts_rrc_pdu_def_dl_CommonInformation1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonInformation_r4 },
  { &hf_umts_rrc_pdu_def_dl_InformationPerRL_List1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_InformationPerRL_List_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RadioBearerReconfiguration_r4_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RadioBearerReconfiguration_r4_IEs, RadioBearerReconfiguration_r4_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions47_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions47(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions47, T_nonCriticalExtensions47_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions17_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerReconfiguration_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RadioBearerReconfiguration_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions47, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions47 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions17(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions17, T_v6xyNonCriticalExtensions17_sequence);

  return offset;
}


static const per_sequence_t T_v5d0NonCriticalExtenstions7_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerReconfiguration_v5d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RadioBearerReconfiguration_v5d0ext_IEs },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions17, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions17 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions7(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions7, T_v5d0NonCriticalExtenstions7_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions15_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerReconfiguration_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RadioBearerReconfiguration_v590ext_IEs },
  { &hf_umts_rrc_pdu_def_v5d0NonCriticalExtenstions7, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions15(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions15, T_v590NonCriticalExtensions15_sequence);

  return offset;
}


static const per_sequence_t T_v4d0NonCriticalExtensions6_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerReconfiguration_r4_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions15, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions15 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions6(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions6, T_v4d0NonCriticalExtensions6_sequence);

  return offset;
}


static const per_sequence_t T_r46_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerReconfiguration_r4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RadioBearerReconfiguration_r4_IEs },
  { &hf_umts_rrc_pdu_def_v4d0NonCriticalExtensions6, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r46(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r46, T_r46_sequence);

  return offset;
}


static const per_sequence_t T_fdd19_sequence[] = {
  { &hf_umts_rrc_pdu_def_dummy1_CPCH_SetID, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CPCH_SetID },
  { &hf_umts_rrc_pdu_def_dummy2_DRAC_StaticInformationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DRAC_StaticInformationList },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_fdd19(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_fdd19, T_fdd19_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_dummy_1a_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_dummy_1a_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_fdd19, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_fdd19 },
  {   1, &hf_umts_rrc_pdu_def_tdd, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_dummy_1a(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_dummy_1a, T_dummy_1a_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_complete4_sequence[] = {
  { &hf_umts_rrc_pdu_def_rab_InformationReconfigList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RAB_InformationReconfigList },
  { &hf_umts_rrc_pdu_def_rb_InformationReconfigList2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_InformationReconfigList_r5 },
  { &hf_umts_rrc_pdu_def_rb_InformationAffectedList1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_InformationAffectedList_r5 },
  { &hf_umts_rrc_pdu_def_rb_PDCPContextRelocationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_PDCPContextRelocationList },
  { &hf_umts_rrc_pdu_def_ul_CommonTransChInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_CommonTransChInfo_r4 },
  { &hf_umts_rrc_pdu_def_ul_deletedTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_DeletedTransChInfoList },
  { &hf_umts_rrc_pdu_def_ul_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_AddReconfTransChInfoList },
  { &hf_umts_rrc_pdu_def_dummy_1a, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_dummy_1a },
  { &hf_umts_rrc_pdu_def_dl_CommonTransChInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonTransChInfo_r4 },
  { &hf_umts_rrc_pdu_def_dl_DeletedTransChInfoList1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_DeletedTransChInfoList_r5 },
  { &hf_umts_rrc_pdu_def_dl_AddReconfTransChInfoList2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_AddReconfTransChInfoList_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_complete4(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_complete4, T_complete4_sequence);

  return offset;
}


static const per_sequence_t T_defaultConfig3_sequence[] = {
  { &hf_umts_rrc_pdu_def_defaultConfigMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DefaultConfigMode },
  { &hf_umts_rrc_pdu_def_defaultConfigIdentity2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DefaultConfigIdentity_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_defaultConfig3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_defaultConfig3, T_defaultConfig3_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_preConfigMode3_vals[] = {
  {   0, "predefinedConfigIdentity" },
  {   1, "defaultConfig" },
  { 0, NULL }
};

static const per_choice_t T_preConfigMode3_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_predefinedConfigIdentity, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_ies_PredefinedConfigIdentity },
  {   1, &hf_umts_rrc_pdu_def_defaultConfig3, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_defaultConfig3 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_preConfigMode3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_preConfigMode3, T_preConfigMode3_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_preconfiguration3_sequence[] = {
  { &hf_umts_rrc_pdu_def_preConfigMode3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_preConfigMode3 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_preconfiguration3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_preconfiguration3, T_preconfiguration3_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_specificationMode4_vals[] = {
  {   0, "complete" },
  {   1, "preconfiguration" },
  { 0, NULL }
};

static const per_choice_t T_specificationMode4_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_complete4, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_complete4 },
  {   1, &hf_umts_rrc_pdu_def_preconfiguration3, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_preconfiguration3 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_specificationMode4(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_specificationMode4, T_specificationMode4_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd20_sequence[] = {
  { &hf_umts_rrc_pdu_def_dummy_DL_PDSCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_PDSCH_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_fdd20(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_fdd20, T_fdd20_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_modeSpecificPhysChInfo5_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificPhysChInfo5_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_fdd20, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_fdd20 },
  {   1, &hf_umts_rrc_pdu_def_tdd, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_modeSpecificPhysChInfo5(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo5, T_modeSpecificPhysChInfo5_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RadioBearerReconfiguration_r5_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_IntegrityProtectionModeInfo },
  { &hf_umts_rrc_pdu_def_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CipheringModeInfo },
  { &hf_umts_rrc_pdu_def_activationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_new_U_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_U_RNTI },
  { &hf_umts_rrc_pdu_def_new_C_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_C_RNTI },
  { &hf_umts_rrc_pdu_def_new_DSCH_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DSCH_RNTI },
  { &hf_umts_rrc_pdu_def_new_H_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_H_RNTI },
  { &hf_umts_rrc_pdu_def_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_StateIndicator },
  { &hf_umts_rrc_pdu_def_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_umts_rrc_pdu_def_cn_InformationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CN_InformationInfo },
  { &hf_umts_rrc_pdu_def_ura_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_URA_Identity },
  { &hf_umts_rrc_pdu_def_specificationMode4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_specificationMode4 },
  { &hf_umts_rrc_pdu_def_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_FrequencyInfo },
  { &hf_umts_rrc_pdu_def_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MaxAllowedUL_TX_Power },
  { &hf_umts_rrc_pdu_def_ul_ChannelRequirement2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_ChannelRequirement_r5 },
  { &hf_umts_rrc_pdu_def_modeSpecificPhysChInfo5, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_modeSpecificPhysChInfo5 },
  { &hf_umts_rrc_pdu_def_dl_HSPDSCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_HSPDSCH_Information },
  { &hf_umts_rrc_pdu_def_dl_CommonInformation2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonInformation_r5 },
  { &hf_umts_rrc_pdu_def_dl_InformationPerRL_List2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_InformationPerRL_List_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RadioBearerReconfiguration_r5_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RadioBearerReconfiguration_r5_IEs, RadioBearerReconfiguration_r5_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions48_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions48(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions48, T_nonCriticalExtensions48_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions18_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerReconfiguration_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RadioBearerReconfiguration_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions48, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions48 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions18(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions18, T_v6xyNonCriticalExtensions18_sequence);

  return offset;
}


static const per_sequence_t T_v5d0NonCriticalExtenstions8_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerReconfiguration_v5d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RadioBearerReconfiguration_v5d0ext_IEs },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions18, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions18 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions8(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions8, T_v5d0NonCriticalExtenstions8_sequence);

  return offset;
}


static const per_sequence_t T_r55_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerReconfiguration_r5, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RadioBearerReconfiguration_r5_IEs },
  { &hf_umts_rrc_pdu_def_radioBearerReconfiguration_r5_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v5d0NonCriticalExtenstions8, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r55(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r55, T_r55_sequence);

  return offset;
}


static const per_sequence_t T_complete5_sequence[] = {
  { &hf_umts_rrc_pdu_def_rab_InformationReconfigList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RAB_InformationReconfigList },
  { &hf_umts_rrc_pdu_def_rb_InformationReconfigList3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_InformationReconfigList_r6 },
  { &hf_umts_rrc_pdu_def_rb_InformationAffectedList2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_InformationAffectedList_r6 },
  { &hf_umts_rrc_pdu_def_rb_PDCPContextRelocationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_PDCPContextRelocationList },
  { &hf_umts_rrc_pdu_def_pdcp_ROHC_TargetMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_PDCP_ROHC_TargetMode },
  { &hf_umts_rrc_pdu_def_ul_CommonTransChInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_CommonTransChInfo_r4 },
  { &hf_umts_rrc_pdu_def_ul_deletedTransChInfoList1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_DeletedTransChInfoList_r6 },
  { &hf_umts_rrc_pdu_def_ul_AddReconfTransChInfoList1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_AddReconfTransChInfoList_r6 },
  { &hf_umts_rrc_pdu_def_dl_CommonTransChInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonTransChInfo_r4 },
  { &hf_umts_rrc_pdu_def_dl_DeletedTransChInfoList1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_DeletedTransChInfoList_r5 },
  { &hf_umts_rrc_pdu_def_dl_AddReconfTransChInfoList2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_AddReconfTransChInfoList_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_complete5(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_complete5, T_complete5_sequence);

  return offset;
}


static const per_sequence_t T_defaultConfig4_sequence[] = {
  { &hf_umts_rrc_pdu_def_defaultConfigMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DefaultConfigMode },
  { &hf_umts_rrc_pdu_def_defaultConfigIdentity2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DefaultConfigIdentity_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_defaultConfig4(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_defaultConfig4, T_defaultConfig4_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_preConfigMode4_vals[] = {
  {   0, "predefinedConfigIdentity" },
  {   1, "defaultConfig" },
  { 0, NULL }
};

static const per_choice_t T_preConfigMode4_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_predefinedConfigIdentity, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_ies_PredefinedConfigIdentity },
  {   1, &hf_umts_rrc_pdu_def_defaultConfig4, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_defaultConfig4 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_preConfigMode4(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_preConfigMode4, T_preConfigMode4_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_preconfiguration4_sequence[] = {
  { &hf_umts_rrc_pdu_def_preConfigMode4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_preConfigMode4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_preconfiguration4(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_preconfiguration4, T_preconfiguration4_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_specificationMode5_vals[] = {
  {   0, "complete" },
  {   1, "preconfiguration" },
  { 0, NULL }
};

static const per_choice_t T_specificationMode5_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_complete5, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_complete5 },
  {   1, &hf_umts_rrc_pdu_def_preconfiguration4, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_preconfiguration4 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_specificationMode5(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_specificationMode5, T_specificationMode5_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RadioBearerReconfiguration_r6_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_IntegrityProtectionModeInfo },
  { &hf_umts_rrc_pdu_def_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CipheringModeInfo },
  { &hf_umts_rrc_pdu_def_activationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_delayRestrictionFlag, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DelayRestrictionFlag },
  { &hf_umts_rrc_pdu_def_new_U_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_U_RNTI },
  { &hf_umts_rrc_pdu_def_new_C_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_C_RNTI },
  { &hf_umts_rrc_pdu_def_new_DSCH_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DSCH_RNTI },
  { &hf_umts_rrc_pdu_def_new_H_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_H_RNTI },
  { &hf_umts_rrc_pdu_def_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_E_RNTI },
  { &hf_umts_rrc_pdu_def_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_E_RNTI },
  { &hf_umts_rrc_pdu_def_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_StateIndicator },
  { &hf_umts_rrc_pdu_def_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_umts_rrc_pdu_def_cn_InformationInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CN_InformationInfo_r6 },
  { &hf_umts_rrc_pdu_def_ura_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_URA_Identity },
  { &hf_umts_rrc_pdu_def_specificationMode5, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_specificationMode5 },
  { &hf_umts_rrc_pdu_def_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_FrequencyInfo },
  { &hf_umts_rrc_pdu_def_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MaxAllowedUL_TX_Power },
  { &hf_umts_rrc_pdu_def_ul_ChannelRequirement3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_ChannelRequirement_r6 },
  { &hf_umts_rrc_pdu_def_ul_EDCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_EDCH_Information_r6 },
  { &hf_umts_rrc_pdu_def_dl_HSPDSCH_Information1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_HSPDSCH_Information_r6 },
  { &hf_umts_rrc_pdu_def_dl_CommonInformation3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonInformation_r6 },
  { &hf_umts_rrc_pdu_def_dl_InformationPerRL_List3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_InformationPerRL_List_r6 },
  { &hf_umts_rrc_pdu_def_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MBMS_PL_ServiceRestrictInfo_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RadioBearerReconfiguration_r6_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RadioBearerReconfiguration_r6_IEs, RadioBearerReconfiguration_r6_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions49_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions49(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions49, T_nonCriticalExtensions49_sequence);

  return offset;
}


static const per_sequence_t T_r66_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerReconfiguration_r6, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RadioBearerReconfiguration_r6_IEs },
  { &hf_umts_rrc_pdu_def_radioBearerReconfiguration_r6_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions49, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions49 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r66(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r66, T_r66_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions34_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions34(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_criticalExtensions34, T_criticalExtensions34_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_criticalExtensions33_vals[] = {
  {   0, "r6" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions33_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r66, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r66 },
  {   1, &hf_umts_rrc_pdu_def_criticalExtensions34, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_criticalExtensions34 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions33(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_criticalExtensions33, T_criticalExtensions33_choice,
                                 NULL);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_criticalExtensions32_vals[] = {
  {   0, "r5" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions32_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r55, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r55 },
  {   1, &hf_umts_rrc_pdu_def_criticalExtensions33, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_criticalExtensions33 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions32(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_criticalExtensions32, T_criticalExtensions32_choice,
                                 NULL);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_criticalExtensions31_vals[] = {
  {   0, "r4" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions31_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r46, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r46 },
  {   1, &hf_umts_rrc_pdu_def_criticalExtensions32, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_criticalExtensions32 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions31(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_criticalExtensions31, T_criticalExtensions31_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_later_than_r312_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_criticalExtensions31, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_criticalExtensions31 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_later_than_r312(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_later_than_r312, T_later_than_r312_sequence);

  return offset;
}


const value_string umts_rrc_pdu_def_RadioBearerReconfiguration_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t RadioBearerReconfiguration_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r314, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r314 },
  {   1, &hf_umts_rrc_pdu_def_later_than_r312, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_later_than_r312 },
  { 0, NULL, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_RadioBearerReconfiguration(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_RadioBearerReconfiguration, RadioBearerReconfiguration_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions50_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions50(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions50, T_nonCriticalExtensions50_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions29_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerReconfigurationComplete_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions50, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions50 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions29(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions29, T_laterNonCriticalExtensions29_sequence);

  return offset;
}


static const per_sequence_t RadioBearerReconfigurationComplete_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_ul_IntegProtActivationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_IntegrityProtActivationInfo },
  { &hf_umts_rrc_pdu_def_ul_TimingAdvance, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_TimingAdvance },
  { &hf_umts_rrc_pdu_def_count_C_ActivationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_dummy_RB_ActivationTimeInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_ActivationTimeInfoList },
  { &hf_umts_rrc_pdu_def_ul_CounterSynchronisationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_CounterSynchronisationInfo },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions29, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions29 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_RadioBearerReconfigurationComplete(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RadioBearerReconfigurationComplete, RadioBearerReconfigurationComplete_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions51_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions51(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions51, T_nonCriticalExtensions51_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions30_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerReconfigurationFailure_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions51, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions51 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions30(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions30, T_laterNonCriticalExtensions30_sequence);

  return offset;
}


static const per_sequence_t RadioBearerReconfigurationFailure_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_failureCause, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_FailureCauseWithProtErr },
  { &hf_umts_rrc_pdu_def_potentiallySuccesfulBearerList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_IdentityList },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions30, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions30 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_RadioBearerReconfigurationFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RadioBearerReconfigurationFailure, RadioBearerReconfigurationFailure_sequence);

  return offset;
}


static const per_sequence_t T_fdd21_sequence[] = {
  { &hf_umts_rrc_pdu_def_dummy1_CPCH_SetID, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CPCH_SetID },
  { &hf_umts_rrc_pdu_def_dummy2_DRAC_StaticInformationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DRAC_StaticInformationList },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_fdd21(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_fdd21, T_fdd21_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_radioBearerRelease_r3_IEs_dummy_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_radioBearerRelease_r3_IEs_dummy_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_fdd21, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_fdd21 },
  {   1, &hf_umts_rrc_pdu_def_tdd, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_radioBearerRelease_r3_IEs_dummy(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_radioBearerRelease_r3_IEs_dummy, T_radioBearerRelease_r3_IEs_dummy_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd22_sequence[] = {
  { &hf_umts_rrc_pdu_def_dummy_DL_PDSCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_PDSCH_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_fdd22(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_fdd22, T_fdd22_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_modeSpecificPhysChInfo6_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificPhysChInfo6_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_fdd22, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_fdd22 },
  {   1, &hf_umts_rrc_pdu_def_tdd, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_modeSpecificPhysChInfo6(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo6, T_modeSpecificPhysChInfo6_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RadioBearerRelease_r3_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_IntegrityProtectionModeInfo },
  { &hf_umts_rrc_pdu_def_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CipheringModeInfo },
  { &hf_umts_rrc_pdu_def_activationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_new_U_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_U_RNTI },
  { &hf_umts_rrc_pdu_def_new_C_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_C_RNTI },
  { &hf_umts_rrc_pdu_def_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_StateIndicator },
  { &hf_umts_rrc_pdu_def_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_umts_rrc_pdu_def_cn_InformationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CN_InformationInfo },
  { &hf_umts_rrc_pdu_def_signallingConnectionRelIndication, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CN_DomainIdentity },
  { &hf_umts_rrc_pdu_def_ura_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_URA_Identity },
  { &hf_umts_rrc_pdu_def_rab_InformationReconfigList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RAB_InformationReconfigList },
  { &hf_umts_rrc_pdu_def_rb_InformationReleaseList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RB_InformationReleaseList },
  { &hf_umts_rrc_pdu_def_rb_InformationAffectedList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_InformationAffectedList },
  { &hf_umts_rrc_pdu_def_dl_CounterSynchronisationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CounterSynchronisationInfo },
  { &hf_umts_rrc_pdu_def_ul_CommonTransChInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_CommonTransChInfo },
  { &hf_umts_rrc_pdu_def_ul_deletedTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_DeletedTransChInfoList },
  { &hf_umts_rrc_pdu_def_ul_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_AddReconfTransChInfoList },
  { &hf_umts_rrc_pdu_def_radioBearerRelease_r3_IEs_dummy, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_radioBearerRelease_r3_IEs_dummy },
  { &hf_umts_rrc_pdu_def_dl_CommonTransChInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonTransChInfo },
  { &hf_umts_rrc_pdu_def_dl_DeletedTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_DeletedTransChInfoList },
  { &hf_umts_rrc_pdu_def_dl_AddReconfTransChInfoList3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_AddReconfTransChInfo2List },
  { &hf_umts_rrc_pdu_def_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_FrequencyInfo },
  { &hf_umts_rrc_pdu_def_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MaxAllowedUL_TX_Power },
  { &hf_umts_rrc_pdu_def_ul_ChannelRequirement, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_ChannelRequirement },
  { &hf_umts_rrc_pdu_def_modeSpecificPhysChInfo6, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_modeSpecificPhysChInfo6 },
  { &hf_umts_rrc_pdu_def_dl_CommonInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonInformation },
  { &hf_umts_rrc_pdu_def_dl_InformationPerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_InformationPerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RadioBearerRelease_r3_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RadioBearerRelease_r3_IEs, RadioBearerRelease_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerRelease_v3a0ext_sequence[] = {
  { &hf_umts_rrc_pdu_def_new_DSCH_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DSCH_RNTI },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RadioBearerRelease_v3a0ext(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RadioBearerRelease_v3a0ext, RadioBearerRelease_v3a0ext_sequence);

  return offset;
}


static const per_sequence_t RadioBearerRelease_v4b0ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_dummy_SSDT_UL, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_SSDT_UL },
  { &hf_umts_rrc_pdu_def_cell_id_PerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CellIdentity_PerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RadioBearerRelease_v4b0ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RadioBearerRelease_v4b0ext_IEs, RadioBearerRelease_v4b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerRelease_v590ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_dl_TPC_PowerOffsetPerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_TPC_PowerOffsetPerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RadioBearerRelease_v590ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RadioBearerRelease_v590ext_IEs, RadioBearerRelease_v590ext_IEs_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_postVerificationPeriod3_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_umts_rrc_pdu_def_T_postVerificationPeriod3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_timingMaintainedSynchInd2_vals[] = {
  {   0, "false" },
  { 0, NULL }
};


static int
dissect_umts_rrc_pdu_def_T_timingMaintainedSynchInd2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t RadioBearerRelease_v6xyext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_primary_plmn_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_PLMN_Identity },
  { &hf_umts_rrc_pdu_def_harq_Preamble_Mode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_HARQ_Preamble_Mode },
  { &hf_umts_rrc_pdu_def_beaconPLEst, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_BEACON_PL_Est },
  { &hf_umts_rrc_pdu_def_postVerificationPeriod3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_postVerificationPeriod3 },
  { &hf_umts_rrc_pdu_def_dhs_sync, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DHS_Sync },
  { &hf_umts_rrc_pdu_def_timingMaintainedSynchInd2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_timingMaintainedSynchInd2 },
  { &hf_umts_rrc_pdu_def_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MBMS_PL_ServiceRestrictInfo_r6 },
  { &hf_umts_rrc_pdu_def_mbms_RB_ListReleasedToChangeTransferMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_InformationReleaseList },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RadioBearerRelease_v6xyext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RadioBearerRelease_v6xyext_IEs, RadioBearerRelease_v6xyext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions52_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions52(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions52, T_nonCriticalExtensions52_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions19_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerRelease_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RadioBearerRelease_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions52, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions52 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions19(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions19, T_v6xyNonCriticalExtensions19_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions16_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerRelease_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RadioBearerRelease_v590ext_IEs },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions19, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions19 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions16(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions16, T_v590NonCriticalExtensions16_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions8_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerRelease_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RadioBearerRelease_v4b0ext_IEs },
  { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions16, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions16 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions8(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions8, T_v4b0NonCriticalExtensions8_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions31_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerRelease_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions8, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions31(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions31, T_laterNonCriticalExtensions31_sequence);

  return offset;
}


static const per_sequence_t T_v3a0NonCriticalExtensions6_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerRelease_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RadioBearerRelease_v3a0ext },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions31, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions31 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions6(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions6, T_v3a0NonCriticalExtensions6_sequence);

  return offset;
}


static const per_sequence_t T_r315_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerRelease_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RadioBearerRelease_r3_IEs },
  { &hf_umts_rrc_pdu_def_v3a0NonCriticalExtensions6, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r315(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r315, T_r315_sequence);

  return offset;
}


static const per_sequence_t T_fdd23_sequence[] = {
  { &hf_umts_rrc_pdu_def_dummy1_CPCH_SetID, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CPCH_SetID },
  { &hf_umts_rrc_pdu_def_dummy2_DRAC_StaticInformationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DRAC_StaticInformationList },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_fdd23(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_fdd23, T_fdd23_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_radioBearerRelease_r4_IEs_dummy_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_radioBearerRelease_r4_IEs_dummy_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_fdd23, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_fdd23 },
  {   1, &hf_umts_rrc_pdu_def_tdd, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_radioBearerRelease_r4_IEs_dummy(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_radioBearerRelease_r4_IEs_dummy, T_radioBearerRelease_r4_IEs_dummy_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd24_sequence[] = {
  { &hf_umts_rrc_pdu_def_dummy_DL_PDSCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_PDSCH_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_fdd24(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_fdd24, T_fdd24_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_modeSpecificPhysChInfo7_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificPhysChInfo7_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_fdd24, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_fdd24 },
  {   1, &hf_umts_rrc_pdu_def_tdd, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_modeSpecificPhysChInfo7(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo7, T_modeSpecificPhysChInfo7_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RadioBearerRelease_r4_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_IntegrityProtectionModeInfo },
  { &hf_umts_rrc_pdu_def_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CipheringModeInfo },
  { &hf_umts_rrc_pdu_def_activationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_new_U_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_U_RNTI },
  { &hf_umts_rrc_pdu_def_new_C_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_C_RNTI },
  { &hf_umts_rrc_pdu_def_new_DSCH_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DSCH_RNTI },
  { &hf_umts_rrc_pdu_def_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_StateIndicator },
  { &hf_umts_rrc_pdu_def_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_umts_rrc_pdu_def_cn_InformationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CN_InformationInfo },
  { &hf_umts_rrc_pdu_def_signallingConnectionRelIndication, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CN_DomainIdentity },
  { &hf_umts_rrc_pdu_def_ura_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_URA_Identity },
  { &hf_umts_rrc_pdu_def_rab_InformationReconfigList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RAB_InformationReconfigList },
  { &hf_umts_rrc_pdu_def_rb_InformationReleaseList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RB_InformationReleaseList },
  { &hf_umts_rrc_pdu_def_rb_InformationAffectedList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_InformationAffectedList },
  { &hf_umts_rrc_pdu_def_dl_CounterSynchronisationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CounterSynchronisationInfo },
  { &hf_umts_rrc_pdu_def_ul_CommonTransChInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_CommonTransChInfo_r4 },
  { &hf_umts_rrc_pdu_def_ul_deletedTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_DeletedTransChInfoList },
  { &hf_umts_rrc_pdu_def_ul_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_AddReconfTransChInfoList },
  { &hf_umts_rrc_pdu_def_radioBearerRelease_r4_IEs_dummy, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_radioBearerRelease_r4_IEs_dummy },
  { &hf_umts_rrc_pdu_def_dl_CommonTransChInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonTransChInfo_r4 },
  { &hf_umts_rrc_pdu_def_dl_DeletedTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_DeletedTransChInfoList },
  { &hf_umts_rrc_pdu_def_dl_AddReconfTransChInfoList1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_AddReconfTransChInfoList_r4 },
  { &hf_umts_rrc_pdu_def_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_FrequencyInfo },
  { &hf_umts_rrc_pdu_def_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MaxAllowedUL_TX_Power },
  { &hf_umts_rrc_pdu_def_ul_ChannelRequirement1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_ChannelRequirement_r4 },
  { &hf_umts_rrc_pdu_def_modeSpecificPhysChInfo7, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_modeSpecificPhysChInfo7 },
  { &hf_umts_rrc_pdu_def_dl_CommonInformation1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonInformation_r4 },
  { &hf_umts_rrc_pdu_def_dl_InformationPerRL_List1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_InformationPerRL_List_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RadioBearerRelease_r4_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RadioBearerRelease_r4_IEs, RadioBearerRelease_r4_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions53_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions53(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions53, T_nonCriticalExtensions53_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions20_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerRelease_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RadioBearerRelease_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions53, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions53 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions20(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions20, T_v6xyNonCriticalExtensions20_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions17_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerRelease_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RadioBearerRelease_v590ext_IEs },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions20, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions20 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions17(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions17, T_v590NonCriticalExtensions17_sequence);

  return offset;
}


static const per_sequence_t T_v4d0NonCriticalExtensions7_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerRelease_r4_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions17, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions17 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions7(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions7, T_v4d0NonCriticalExtensions7_sequence);

  return offset;
}


static const per_sequence_t T_r47_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerRelease_r4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RadioBearerRelease_r4_IEs },
  { &hf_umts_rrc_pdu_def_v4d0NonCriticalExtensions7, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r47(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r47, T_r47_sequence);

  return offset;
}


static const per_sequence_t T_fdd25_sequence[] = {
  { &hf_umts_rrc_pdu_def_dummy1_CPCH_SetID, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CPCH_SetID },
  { &hf_umts_rrc_pdu_def_dummy2_DRAC_StaticInformationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DRAC_StaticInformationList },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_fdd25(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_fdd25, T_fdd25_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_radioBearerRelease_r5_IEs_dummy_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_radioBearerRelease_r5_IEs_dummy_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_fdd25, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_fdd25 },
  {   1, &hf_umts_rrc_pdu_def_tdd, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_radioBearerRelease_r5_IEs_dummy(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_radioBearerRelease_r5_IEs_dummy, T_radioBearerRelease_r5_IEs_dummy_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd26_sequence[] = {
  { &hf_umts_rrc_pdu_def_dummy_DL_PDSCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_PDSCH_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_fdd26(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_fdd26, T_fdd26_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_modeSpecificPhysChInfo8_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificPhysChInfo8_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_fdd26, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_fdd26 },
  {   1, &hf_umts_rrc_pdu_def_tdd, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_modeSpecificPhysChInfo8(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo8, T_modeSpecificPhysChInfo8_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RadioBearerRelease_r5_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_IntegrityProtectionModeInfo },
  { &hf_umts_rrc_pdu_def_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CipheringModeInfo },
  { &hf_umts_rrc_pdu_def_activationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_new_U_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_U_RNTI },
  { &hf_umts_rrc_pdu_def_new_C_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_C_RNTI },
  { &hf_umts_rrc_pdu_def_new_DSCH_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DSCH_RNTI },
  { &hf_umts_rrc_pdu_def_new_H_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_H_RNTI },
  { &hf_umts_rrc_pdu_def_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_StateIndicator },
  { &hf_umts_rrc_pdu_def_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_umts_rrc_pdu_def_cn_InformationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CN_InformationInfo },
  { &hf_umts_rrc_pdu_def_signallingConnectionRelIndication, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CN_DomainIdentity },
  { &hf_umts_rrc_pdu_def_ura_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_URA_Identity },
  { &hf_umts_rrc_pdu_def_rab_InformationReconfigList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RAB_InformationReconfigList },
  { &hf_umts_rrc_pdu_def_rb_InformationReleaseList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RB_InformationReleaseList },
  { &hf_umts_rrc_pdu_def_rb_InformationAffectedList1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_InformationAffectedList_r5 },
  { &hf_umts_rrc_pdu_def_dl_CounterSynchronisationInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CounterSynchronisationInfo_r5 },
  { &hf_umts_rrc_pdu_def_ul_CommonTransChInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_CommonTransChInfo_r4 },
  { &hf_umts_rrc_pdu_def_ul_deletedTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_DeletedTransChInfoList },
  { &hf_umts_rrc_pdu_def_ul_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_AddReconfTransChInfoList },
  { &hf_umts_rrc_pdu_def_radioBearerRelease_r5_IEs_dummy, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_radioBearerRelease_r5_IEs_dummy },
  { &hf_umts_rrc_pdu_def_dl_CommonTransChInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonTransChInfo_r4 },
  { &hf_umts_rrc_pdu_def_dl_DeletedTransChInfoList1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_DeletedTransChInfoList_r5 },
  { &hf_umts_rrc_pdu_def_dl_AddReconfTransChInfoList2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_AddReconfTransChInfoList_r5 },
  { &hf_umts_rrc_pdu_def_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_FrequencyInfo },
  { &hf_umts_rrc_pdu_def_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MaxAllowedUL_TX_Power },
  { &hf_umts_rrc_pdu_def_ul_ChannelRequirement2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_ChannelRequirement_r5 },
  { &hf_umts_rrc_pdu_def_modeSpecificPhysChInfo8, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_modeSpecificPhysChInfo8 },
  { &hf_umts_rrc_pdu_def_dl_HSPDSCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_HSPDSCH_Information },
  { &hf_umts_rrc_pdu_def_dl_CommonInformation2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonInformation_r5 },
  { &hf_umts_rrc_pdu_def_dl_InformationPerRL_List2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_InformationPerRL_List_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RadioBearerRelease_r5_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RadioBearerRelease_r5_IEs, RadioBearerRelease_r5_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions54_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions54(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions54, T_nonCriticalExtensions54_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions21_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerRelease_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RadioBearerRelease_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions54, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions54 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions21(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions21, T_v6xyNonCriticalExtensions21_sequence);

  return offset;
}


static const per_sequence_t T_r56_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerRelease_r5, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RadioBearerRelease_r5_IEs },
  { &hf_umts_rrc_pdu_def_radioBearerRelease_r5_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions21, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions21 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r56(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r56, T_r56_sequence);

  return offset;
}


static const per_sequence_t RadioBearerRelease_r6_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_IntegrityProtectionModeInfo },
  { &hf_umts_rrc_pdu_def_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CipheringModeInfo },
  { &hf_umts_rrc_pdu_def_activationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_new_U_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_U_RNTI },
  { &hf_umts_rrc_pdu_def_new_C_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_C_RNTI },
  { &hf_umts_rrc_pdu_def_new_DSCH_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DSCH_RNTI },
  { &hf_umts_rrc_pdu_def_new_H_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_H_RNTI },
  { &hf_umts_rrc_pdu_def_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_E_RNTI },
  { &hf_umts_rrc_pdu_def_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_E_RNTI },
  { &hf_umts_rrc_pdu_def_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_StateIndicator },
  { &hf_umts_rrc_pdu_def_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_umts_rrc_pdu_def_cn_InformationInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CN_InformationInfo_r6 },
  { &hf_umts_rrc_pdu_def_signallingConnectionRelIndication, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CN_DomainIdentity },
  { &hf_umts_rrc_pdu_def_ura_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_URA_Identity },
  { &hf_umts_rrc_pdu_def_rab_InformationReconfigList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RAB_InformationReconfigList },
  { &hf_umts_rrc_pdu_def_rb_InformationReleaseList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RB_InformationReleaseList },
  { &hf_umts_rrc_pdu_def_rb_InformationReconfigList3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_InformationReconfigList_r6 },
  { &hf_umts_rrc_pdu_def_rb_InformationAffectedList2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_InformationAffectedList_r6 },
  { &hf_umts_rrc_pdu_def_dl_CounterSynchronisationInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CounterSynchronisationInfo_r5 },
  { &hf_umts_rrc_pdu_def_ul_CommonTransChInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_CommonTransChInfo_r4 },
  { &hf_umts_rrc_pdu_def_ul_deletedTransChInfoList1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_DeletedTransChInfoList_r6 },
  { &hf_umts_rrc_pdu_def_ul_AddReconfTransChInfoList1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_AddReconfTransChInfoList_r6 },
  { &hf_umts_rrc_pdu_def_dl_CommonTransChInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonTransChInfo_r4 },
  { &hf_umts_rrc_pdu_def_dl_DeletedTransChInfoList1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_DeletedTransChInfoList_r5 },
  { &hf_umts_rrc_pdu_def_dl_AddReconfTransChInfoList2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_AddReconfTransChInfoList_r5 },
  { &hf_umts_rrc_pdu_def_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_FrequencyInfo },
  { &hf_umts_rrc_pdu_def_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MaxAllowedUL_TX_Power },
  { &hf_umts_rrc_pdu_def_ul_ChannelRequirement3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_ChannelRequirement_r6 },
  { &hf_umts_rrc_pdu_def_ul_EDCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_EDCH_Information_r6 },
  { &hf_umts_rrc_pdu_def_dl_HSPDSCH_Information1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_HSPDSCH_Information_r6 },
  { &hf_umts_rrc_pdu_def_dl_CommonInformation3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonInformation_r6 },
  { &hf_umts_rrc_pdu_def_dl_InformationPerRL_List3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_InformationPerRL_List_r6 },
  { &hf_umts_rrc_pdu_def_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MBMS_PL_ServiceRestrictInfo_r6 },
  { &hf_umts_rrc_pdu_def_mbms_RB_ListReleasedToChangeTransferMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_InformationReleaseList },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RadioBearerRelease_r6_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RadioBearerRelease_r6_IEs, RadioBearerRelease_r6_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions55_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions55(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions55, T_nonCriticalExtensions55_sequence);

  return offset;
}


static const per_sequence_t T_r67_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerRelease_r6, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RadioBearerRelease_r6_IEs },
  { &hf_umts_rrc_pdu_def_radioBearerRelease_r6_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions55, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions55 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r67(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r67, T_r67_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions38_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions38(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_criticalExtensions38, T_criticalExtensions38_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_criticalExtensions37_vals[] = {
  {   0, "r6" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions37_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r67, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r67 },
  {   1, &hf_umts_rrc_pdu_def_criticalExtensions38, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_criticalExtensions38 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions37(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_criticalExtensions37, T_criticalExtensions37_choice,
                                 NULL);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_criticalExtensions36_vals[] = {
  {   0, "r5" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions36_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r56, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r56 },
  {   1, &hf_umts_rrc_pdu_def_criticalExtensions37, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_criticalExtensions37 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions36(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_criticalExtensions36, T_criticalExtensions36_choice,
                                 NULL);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_criticalExtensions35_vals[] = {
  {   0, "r4" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions35_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r47, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r47 },
  {   1, &hf_umts_rrc_pdu_def_criticalExtensions36, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_criticalExtensions36 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions35(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_criticalExtensions35, T_criticalExtensions35_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_later_than_r313_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_criticalExtensions35, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_criticalExtensions35 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_later_than_r313(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_later_than_r313, T_later_than_r313_sequence);

  return offset;
}


const value_string umts_rrc_pdu_def_RadioBearerRelease_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t RadioBearerRelease_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r315, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r315 },
  {   1, &hf_umts_rrc_pdu_def_later_than_r313, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_later_than_r313 },
  { 0, NULL, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_RadioBearerRelease(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_RadioBearerRelease, RadioBearerRelease_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions56_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions56(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions56, T_nonCriticalExtensions56_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions32_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerReleaseComplete_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions56, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions56 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions32(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions32, T_laterNonCriticalExtensions32_sequence);

  return offset;
}


static const per_sequence_t RadioBearerReleaseComplete_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_ul_IntegProtActivationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_IntegrityProtActivationInfo },
  { &hf_umts_rrc_pdu_def_ul_TimingAdvance, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_TimingAdvance },
  { &hf_umts_rrc_pdu_def_count_C_ActivationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_dummy_RB_ActivationTimeInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_ActivationTimeInfoList },
  { &hf_umts_rrc_pdu_def_ul_CounterSynchronisationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_CounterSynchronisationInfo },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions32, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions32 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_RadioBearerReleaseComplete(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RadioBearerReleaseComplete, RadioBearerReleaseComplete_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions57_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions57(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions57, T_nonCriticalExtensions57_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions33_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerReleaseFailure_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions57, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions57 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions33(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions33, T_laterNonCriticalExtensions33_sequence);

  return offset;
}


static const per_sequence_t RadioBearerReleaseFailure_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_failureCause, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_FailureCauseWithProtErr },
  { &hf_umts_rrc_pdu_def_potentiallySuccesfulBearerList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_IdentityList },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions33, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions33 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_RadioBearerReleaseFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RadioBearerReleaseFailure, RadioBearerReleaseFailure_sequence);

  return offset;
}


static const per_sequence_t T_fdd27_sequence[] = {
  { &hf_umts_rrc_pdu_def_dummy1_CPCH_SetID, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CPCH_SetID },
  { &hf_umts_rrc_pdu_def_dummy2_DRAC_StaticInformationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DRAC_StaticInformationList },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_fdd27(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_fdd27, T_fdd27_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_radioBearerSetup_r3_IEs_dummy_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_radioBearerSetup_r3_IEs_dummy_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_fdd27, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_fdd27 },
  {   1, &hf_umts_rrc_pdu_def_tdd, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_radioBearerSetup_r3_IEs_dummy(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_radioBearerSetup_r3_IEs_dummy, T_radioBearerSetup_r3_IEs_dummy_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd28_sequence[] = {
  { &hf_umts_rrc_pdu_def_dummy_DL_PDSCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_PDSCH_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_fdd28(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_fdd28, T_fdd28_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_modeSpecificPhysChInfo9_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificPhysChInfo9_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_fdd28, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_fdd28 },
  {   1, &hf_umts_rrc_pdu_def_tdd, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_modeSpecificPhysChInfo9(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo9, T_modeSpecificPhysChInfo9_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RadioBearerSetup_r3_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_IntegrityProtectionModeInfo },
  { &hf_umts_rrc_pdu_def_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CipheringModeInfo },
  { &hf_umts_rrc_pdu_def_activationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_new_U_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_U_RNTI },
  { &hf_umts_rrc_pdu_def_new_C_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_C_RNTI },
  { &hf_umts_rrc_pdu_def_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_StateIndicator },
  { &hf_umts_rrc_pdu_def_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_umts_rrc_pdu_def_ura_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_URA_Identity },
  { &hf_umts_rrc_pdu_def_cn_InformationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CN_InformationInfo },
  { &hf_umts_rrc_pdu_def_srb_InformationSetupList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_SRB_InformationSetupList },
  { &hf_umts_rrc_pdu_def_rab_InformationSetupList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RAB_InformationSetupList },
  { &hf_umts_rrc_pdu_def_rb_InformationAffectedList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_InformationAffectedList },
  { &hf_umts_rrc_pdu_def_dl_CounterSynchronisationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CounterSynchronisationInfo },
  { &hf_umts_rrc_pdu_def_ul_CommonTransChInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_CommonTransChInfo },
  { &hf_umts_rrc_pdu_def_ul_deletedTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_DeletedTransChInfoList },
  { &hf_umts_rrc_pdu_def_ul_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_AddReconfTransChInfoList },
  { &hf_umts_rrc_pdu_def_radioBearerSetup_r3_IEs_dummy, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_radioBearerSetup_r3_IEs_dummy },
  { &hf_umts_rrc_pdu_def_dl_CommonTransChInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonTransChInfo },
  { &hf_umts_rrc_pdu_def_dl_DeletedTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_DeletedTransChInfoList },
  { &hf_umts_rrc_pdu_def_dl_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_AddReconfTransChInfoList },
  { &hf_umts_rrc_pdu_def_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_FrequencyInfo },
  { &hf_umts_rrc_pdu_def_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MaxAllowedUL_TX_Power },
  { &hf_umts_rrc_pdu_def_ul_ChannelRequirement, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_ChannelRequirement },
  { &hf_umts_rrc_pdu_def_modeSpecificPhysChInfo9, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_modeSpecificPhysChInfo9 },
  { &hf_umts_rrc_pdu_def_dl_CommonInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonInformation },
  { &hf_umts_rrc_pdu_def_dl_InformationPerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_InformationPerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RadioBearerSetup_r3_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RadioBearerSetup_r3_IEs, RadioBearerSetup_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerSetup_v3a0ext_sequence[] = {
  { &hf_umts_rrc_pdu_def_new_DSCH_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DSCH_RNTI },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RadioBearerSetup_v3a0ext(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RadioBearerSetup_v3a0ext, RadioBearerSetup_v3a0ext_sequence);

  return offset;
}


static const per_sequence_t RadioBearerSetup_v4b0ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_dummy_SSDT_UL, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_SSDT_UL },
  { &hf_umts_rrc_pdu_def_cell_id_PerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CellIdentity_PerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RadioBearerSetup_v4b0ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RadioBearerSetup_v4b0ext_IEs, RadioBearerSetup_v4b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerSetup_v590ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_dl_TPC_PowerOffsetPerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_TPC_PowerOffsetPerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RadioBearerSetup_v590ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RadioBearerSetup_v590ext_IEs, RadioBearerSetup_v590ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RadioBearerSetup_v5d0ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_pdcp_ROHC_TargetMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_PDCP_ROHC_TargetMode },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RadioBearerSetup_v5d0ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RadioBearerSetup_v5d0ext_IEs, RadioBearerSetup_v5d0ext_IEs_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_postVerificationPeriod4_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_umts_rrc_pdu_def_T_postVerificationPeriod4(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_timingMaintainedSynchInd3_vals[] = {
  {   0, "false" },
  { 0, NULL }
};


static int
dissect_umts_rrc_pdu_def_T_timingMaintainedSynchInd3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t RadioBearerSetup_v6xyext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_primary_plmn_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_PLMN_Identity },
  { &hf_umts_rrc_pdu_def_harq_Preamble_Mode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_HARQ_Preamble_Mode },
  { &hf_umts_rrc_pdu_def_beaconPLEst, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_BEACON_PL_Est },
  { &hf_umts_rrc_pdu_def_postVerificationPeriod4, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_postVerificationPeriod4 },
  { &hf_umts_rrc_pdu_def_dhs_sync, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DHS_Sync },
  { &hf_umts_rrc_pdu_def_timingMaintainedSynchInd3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_timingMaintainedSynchInd3 },
  { &hf_umts_rrc_pdu_def_rab_InformationSetupList4, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RAB_InformationSetupList_r6_ext },
  { &hf_umts_rrc_pdu_def_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MBMS_PL_ServiceRestrictInfo_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RadioBearerSetup_v6xyext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RadioBearerSetup_v6xyext_IEs, RadioBearerSetup_v6xyext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions58_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions58(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions58, T_nonCriticalExtensions58_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions22_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerSetup_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RadioBearerSetup_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions58, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions58 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions22(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions22, T_v6xyNonCriticalExtensions22_sequence);

  return offset;
}


static const per_sequence_t T_v5d0NonCriticalExtenstions9_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerSetup_v5d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RadioBearerSetup_v5d0ext_IEs },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions22, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions22 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions9(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions9, T_v5d0NonCriticalExtenstions9_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions18_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerSetup_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RadioBearerSetup_v590ext_IEs },
  { &hf_umts_rrc_pdu_def_v5d0NonCriticalExtenstions9, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions18(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions18, T_v590NonCriticalExtensions18_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions9_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerSetup_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RadioBearerSetup_v4b0ext_IEs },
  { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions18, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions18 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions9(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions9, T_v4b0NonCriticalExtensions9_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions34_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerSetup_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions9, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions34(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions34, T_laterNonCriticalExtensions34_sequence);

  return offset;
}


static const per_sequence_t T_v3a0NonCriticalExtensions7_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerSetup_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RadioBearerSetup_v3a0ext },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions34, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions34 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions7(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions7, T_v3a0NonCriticalExtensions7_sequence);

  return offset;
}


static const per_sequence_t T_r316_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerSetup_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RadioBearerSetup_r3_IEs },
  { &hf_umts_rrc_pdu_def_v3a0NonCriticalExtensions7, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions7 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r316(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r316, T_r316_sequence);

  return offset;
}


static const per_sequence_t T_fdd29_sequence[] = {
  { &hf_umts_rrc_pdu_def_dummy1_CPCH_SetID, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CPCH_SetID },
  { &hf_umts_rrc_pdu_def_dummy2_DRAC_StaticInformationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DRAC_StaticInformationList },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_fdd29(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_fdd29, T_fdd29_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_radioBearerSetup_r4_IEs_dummy_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_radioBearerSetup_r4_IEs_dummy_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_fdd29, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_fdd29 },
  {   1, &hf_umts_rrc_pdu_def_tdd, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_radioBearerSetup_r4_IEs_dummy(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_radioBearerSetup_r4_IEs_dummy, T_radioBearerSetup_r4_IEs_dummy_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd30_sequence[] = {
  { &hf_umts_rrc_pdu_def_dummy_DL_PDSCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_PDSCH_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_fdd30(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_fdd30, T_fdd30_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_modeSpecificPhysChInfo10_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificPhysChInfo10_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_fdd30, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_fdd30 },
  {   1, &hf_umts_rrc_pdu_def_tdd, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_modeSpecificPhysChInfo10(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo10, T_modeSpecificPhysChInfo10_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RadioBearerSetup_r4_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_IntegrityProtectionModeInfo },
  { &hf_umts_rrc_pdu_def_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CipheringModeInfo },
  { &hf_umts_rrc_pdu_def_activationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_new_U_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_U_RNTI },
  { &hf_umts_rrc_pdu_def_new_C_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_C_RNTI },
  { &hf_umts_rrc_pdu_def_new_DSCH_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DSCH_RNTI },
  { &hf_umts_rrc_pdu_def_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_StateIndicator },
  { &hf_umts_rrc_pdu_def_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_umts_rrc_pdu_def_ura_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_URA_Identity },
  { &hf_umts_rrc_pdu_def_cn_InformationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CN_InformationInfo },
  { &hf_umts_rrc_pdu_def_srb_InformationSetupList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_SRB_InformationSetupList },
  { &hf_umts_rrc_pdu_def_rab_InformationSetupList1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RAB_InformationSetupList_r4 },
  { &hf_umts_rrc_pdu_def_rb_InformationAffectedList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_InformationAffectedList },
  { &hf_umts_rrc_pdu_def_dl_CounterSynchronisationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CounterSynchronisationInfo },
  { &hf_umts_rrc_pdu_def_ul_CommonTransChInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_CommonTransChInfo_r4 },
  { &hf_umts_rrc_pdu_def_ul_deletedTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_DeletedTransChInfoList },
  { &hf_umts_rrc_pdu_def_ul_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_AddReconfTransChInfoList },
  { &hf_umts_rrc_pdu_def_radioBearerSetup_r4_IEs_dummy, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_radioBearerSetup_r4_IEs_dummy },
  { &hf_umts_rrc_pdu_def_dl_CommonTransChInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonTransChInfo_r4 },
  { &hf_umts_rrc_pdu_def_dl_DeletedTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_DeletedTransChInfoList },
  { &hf_umts_rrc_pdu_def_dl_AddReconfTransChInfoList1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_AddReconfTransChInfoList_r4 },
  { &hf_umts_rrc_pdu_def_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_FrequencyInfo },
  { &hf_umts_rrc_pdu_def_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MaxAllowedUL_TX_Power },
  { &hf_umts_rrc_pdu_def_ul_ChannelRequirement1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_ChannelRequirement_r4 },
  { &hf_umts_rrc_pdu_def_modeSpecificPhysChInfo10, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_modeSpecificPhysChInfo10 },
  { &hf_umts_rrc_pdu_def_dl_CommonInformation1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonInformation_r4 },
  { &hf_umts_rrc_pdu_def_dl_InformationPerRL_List1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_InformationPerRL_List_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RadioBearerSetup_r4_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RadioBearerSetup_r4_IEs, RadioBearerSetup_r4_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions59_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions59(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions59, T_nonCriticalExtensions59_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions23_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerSetup_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RadioBearerSetup_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions59, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions59 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions23(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions23, T_v6xyNonCriticalExtensions23_sequence);

  return offset;
}


static const per_sequence_t T_v5d0NonCriticalExtenstions10_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerSetup_v5d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RadioBearerSetup_v5d0ext_IEs },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions23, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions23 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions10(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions10, T_v5d0NonCriticalExtenstions10_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions19_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerSetup_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RadioBearerSetup_v590ext_IEs },
  { &hf_umts_rrc_pdu_def_v5d0NonCriticalExtenstions10, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions19(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions19, T_v590NonCriticalExtensions19_sequence);

  return offset;
}


static const per_sequence_t T_v4d0NonCriticalExtensions8_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerSetup_r4_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions19, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions19 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions8(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions8, T_v4d0NonCriticalExtensions8_sequence);

  return offset;
}


static const per_sequence_t T_r48_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerSetup_r4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RadioBearerSetup_r4_IEs },
  { &hf_umts_rrc_pdu_def_v4d0NonCriticalExtensions8, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r48(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r48, T_r48_sequence);

  return offset;
}


static const per_sequence_t T_fdd31_sequence[] = {
  { &hf_umts_rrc_pdu_def_dummy1_CPCH_SetID, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CPCH_SetID },
  { &hf_umts_rrc_pdu_def_dummy2_DRAC_StaticInformationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DRAC_StaticInformationList },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_fdd31(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_fdd31, T_fdd31_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_radioBearerSetup_r5_IEs_dummy_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_radioBearerSetup_r5_IEs_dummy_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_fdd31, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_fdd31 },
  {   1, &hf_umts_rrc_pdu_def_tdd, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_radioBearerSetup_r5_IEs_dummy(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_radioBearerSetup_r5_IEs_dummy, T_radioBearerSetup_r5_IEs_dummy_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd32_sequence[] = {
  { &hf_umts_rrc_pdu_def_dummy_DL_PDSCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_PDSCH_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_fdd32(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_fdd32, T_fdd32_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_modeSpecificPhysChInfo11_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificPhysChInfo11_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_fdd32, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_fdd32 },
  {   1, &hf_umts_rrc_pdu_def_tdd, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_modeSpecificPhysChInfo11(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo11, T_modeSpecificPhysChInfo11_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RadioBearerSetup_r5_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_IntegrityProtectionModeInfo },
  { &hf_umts_rrc_pdu_def_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CipheringModeInfo },
  { &hf_umts_rrc_pdu_def_activationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_new_U_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_U_RNTI },
  { &hf_umts_rrc_pdu_def_new_C_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_C_RNTI },
  { &hf_umts_rrc_pdu_def_new_DSCH_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DSCH_RNTI },
  { &hf_umts_rrc_pdu_def_new_H_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_H_RNTI },
  { &hf_umts_rrc_pdu_def_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_StateIndicator },
  { &hf_umts_rrc_pdu_def_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_umts_rrc_pdu_def_ura_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_URA_Identity },
  { &hf_umts_rrc_pdu_def_cn_InformationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CN_InformationInfo },
  { &hf_umts_rrc_pdu_def_srb_InformationSetupList1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_SRB_InformationSetupList_r5 },
  { &hf_umts_rrc_pdu_def_rab_InformationSetupList2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RAB_InformationSetupList_r5 },
  { &hf_umts_rrc_pdu_def_rb_InformationAffectedList1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_InformationAffectedList_r5 },
  { &hf_umts_rrc_pdu_def_dl_CounterSynchronisationInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CounterSynchronisationInfo_r5 },
  { &hf_umts_rrc_pdu_def_ul_CommonTransChInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_CommonTransChInfo_r4 },
  { &hf_umts_rrc_pdu_def_ul_deletedTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_DeletedTransChInfoList },
  { &hf_umts_rrc_pdu_def_ul_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_AddReconfTransChInfoList },
  { &hf_umts_rrc_pdu_def_radioBearerSetup_r5_IEs_dummy, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_radioBearerSetup_r5_IEs_dummy },
  { &hf_umts_rrc_pdu_def_dl_CommonTransChInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonTransChInfo_r4 },
  { &hf_umts_rrc_pdu_def_dl_DeletedTransChInfoList1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_DeletedTransChInfoList_r5 },
  { &hf_umts_rrc_pdu_def_dl_AddReconfTransChInfoList2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_AddReconfTransChInfoList_r5 },
  { &hf_umts_rrc_pdu_def_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_FrequencyInfo },
  { &hf_umts_rrc_pdu_def_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MaxAllowedUL_TX_Power },
  { &hf_umts_rrc_pdu_def_ul_ChannelRequirement2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_ChannelRequirement_r5 },
  { &hf_umts_rrc_pdu_def_modeSpecificPhysChInfo11, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_modeSpecificPhysChInfo11 },
  { &hf_umts_rrc_pdu_def_dl_HSPDSCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_HSPDSCH_Information },
  { &hf_umts_rrc_pdu_def_dl_CommonInformation2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonInformation_r5 },
  { &hf_umts_rrc_pdu_def_dl_InformationPerRL_List2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_InformationPerRL_List_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RadioBearerSetup_r5_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RadioBearerSetup_r5_IEs, RadioBearerSetup_r5_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions60_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions60(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions60, T_nonCriticalExtensions60_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions24_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerSetup_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RadioBearerSetup_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions60, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions60 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions24(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions24, T_v6xyNonCriticalExtensions24_sequence);

  return offset;
}


static const per_sequence_t T_v5d0NonCriticalExtenstions11_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerSetup_v5d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RadioBearerSetup_v5d0ext_IEs },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions24, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions24 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions11(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions11, T_v5d0NonCriticalExtenstions11_sequence);

  return offset;
}


static const per_sequence_t T_r57_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerSetup_r5, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RadioBearerSetup_r5_IEs },
  { &hf_umts_rrc_pdu_def_radioBearerSetup_r5_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v5d0NonCriticalExtenstions11, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r57(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r57, T_r57_sequence);

  return offset;
}


static const per_sequence_t T_complete6_sequence[] = {
  { &hf_umts_rrc_pdu_def_srb_InformationSetupList2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_SRB_InformationSetupList_r6 },
  { &hf_umts_rrc_pdu_def_rab_InformationSetupList3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RAB_InformationSetupList_r6 },
  { &hf_umts_rrc_pdu_def_rb_InformationReconfigList3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_InformationReconfigList_r6 },
  { &hf_umts_rrc_pdu_def_rb_InformationAffectedList2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_InformationAffectedList_r6 },
  { &hf_umts_rrc_pdu_def_dl_CounterSynchronisationInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CounterSynchronisationInfo_r5 },
  { &hf_umts_rrc_pdu_def_pdcp_ROHC_TargetMode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_PDCP_ROHC_TargetMode },
  { &hf_umts_rrc_pdu_def_ul_CommonTransChInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_CommonTransChInfo_r4 },
  { &hf_umts_rrc_pdu_def_ul_deletedTransChInfoList1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_DeletedTransChInfoList_r6 },
  { &hf_umts_rrc_pdu_def_ul_AddReconfTransChInfoList1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_AddReconfTransChInfoList_r6 },
  { &hf_umts_rrc_pdu_def_dl_CommonTransChInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonTransChInfo_r4 },
  { &hf_umts_rrc_pdu_def_dl_DeletedTransChInfoList1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_DeletedTransChInfoList_r5 },
  { &hf_umts_rrc_pdu_def_dl_AddReconfTransChInfoList2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_AddReconfTransChInfoList_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_complete6(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_complete6, T_complete6_sequence);

  return offset;
}


static const per_sequence_t T_preconfiguration5_sequence[] = {
  { &hf_umts_rrc_pdu_def_rab_Info1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RAB_Info },
  { &hf_umts_rrc_pdu_def_defaultConfigMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DefaultConfigMode },
  { &hf_umts_rrc_pdu_def_defaultConfigIdentity2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DefaultConfigIdentity_r5 },
  { &hf_umts_rrc_pdu_def_rb_InformationChangedList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_InformationChangedList_r6 },
  { &hf_umts_rrc_pdu_def_powerOffsetInfoShort, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_PowerOffsetInfoShort },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_preconfiguration5(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_preconfiguration5, T_preconfiguration5_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_specificationMode6_vals[] = {
  {   0, "complete" },
  {   1, "preconfiguration" },
  { 0, NULL }
};

static const per_choice_t T_specificationMode6_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_complete6, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_complete6 },
  {   1, &hf_umts_rrc_pdu_def_preconfiguration5, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_preconfiguration5 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_specificationMode6(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_specificationMode6, T_specificationMode6_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RadioBearerSetup_r6_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_IntegrityProtectionModeInfo },
  { &hf_umts_rrc_pdu_def_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CipheringModeInfo },
  { &hf_umts_rrc_pdu_def_activationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_new_U_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_U_RNTI },
  { &hf_umts_rrc_pdu_def_new_C_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_C_RNTI },
  { &hf_umts_rrc_pdu_def_new_DSCH_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DSCH_RNTI },
  { &hf_umts_rrc_pdu_def_new_H_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_H_RNTI },
  { &hf_umts_rrc_pdu_def_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_E_RNTI },
  { &hf_umts_rrc_pdu_def_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_E_RNTI },
  { &hf_umts_rrc_pdu_def_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_StateIndicator },
  { &hf_umts_rrc_pdu_def_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_umts_rrc_pdu_def_ura_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_URA_Identity },
  { &hf_umts_rrc_pdu_def_cn_InformationInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CN_InformationInfo_r6 },
  { &hf_umts_rrc_pdu_def_specificationMode6, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_specificationMode6 },
  { &hf_umts_rrc_pdu_def_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_FrequencyInfo },
  { &hf_umts_rrc_pdu_def_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MaxAllowedUL_TX_Power },
  { &hf_umts_rrc_pdu_def_ul_ChannelRequirement3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_ChannelRequirement_r6 },
  { &hf_umts_rrc_pdu_def_ul_EDCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_EDCH_Information_r6 },
  { &hf_umts_rrc_pdu_def_dl_HSPDSCH_Information1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_HSPDSCH_Information_r6 },
  { &hf_umts_rrc_pdu_def_dl_CommonInformation3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonInformation_r6 },
  { &hf_umts_rrc_pdu_def_dl_InformationPerRL_List3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_InformationPerRL_List_r6 },
  { &hf_umts_rrc_pdu_def_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MBMS_PL_ServiceRestrictInfo_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RadioBearerSetup_r6_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RadioBearerSetup_r6_IEs, RadioBearerSetup_r6_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions61_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions61(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions61, T_nonCriticalExtensions61_sequence);

  return offset;
}


static const per_sequence_t T_r68_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerSetup_r6, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RadioBearerSetup_r6_IEs },
  { &hf_umts_rrc_pdu_def_radioBearerSetup_r6_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions61, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions61 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r68(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r68, T_r68_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions42_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions42(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_criticalExtensions42, T_criticalExtensions42_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_criticalExtensions41_vals[] = {
  {   0, "r6" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions41_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r68, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r68 },
  {   1, &hf_umts_rrc_pdu_def_criticalExtensions42, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_criticalExtensions42 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions41(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_criticalExtensions41, T_criticalExtensions41_choice,
                                 NULL);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_criticalExtensions40_vals[] = {
  {   0, "r5" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions40_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r57, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r57 },
  {   1, &hf_umts_rrc_pdu_def_criticalExtensions41, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_criticalExtensions41 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions40(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_criticalExtensions40, T_criticalExtensions40_choice,
                                 NULL);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_criticalExtensions39_vals[] = {
  {   0, "r4" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions39_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r48, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r48 },
  {   1, &hf_umts_rrc_pdu_def_criticalExtensions40, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_criticalExtensions40 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions39(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_criticalExtensions39, T_criticalExtensions39_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_later_than_r314_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_criticalExtensions39, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_criticalExtensions39 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_later_than_r314(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_later_than_r314, T_later_than_r314_sequence);

  return offset;
}


const value_string umts_rrc_pdu_def_RadioBearerSetup_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t RadioBearerSetup_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r316, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r316 },
  {   1, &hf_umts_rrc_pdu_def_later_than_r314, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_later_than_r314 },
  { 0, NULL, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_RadioBearerSetup(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_RadioBearerSetup, RadioBearerSetup_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions62_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions62(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions62, T_nonCriticalExtensions62_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions35_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerSetupComplete_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions62, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions62 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions35(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions35, T_laterNonCriticalExtensions35_sequence);

  return offset;
}


static const per_sequence_t RadioBearerSetupComplete_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_ul_IntegProtActivationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_IntegrityProtActivationInfo },
  { &hf_umts_rrc_pdu_def_ul_TimingAdvance, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_TimingAdvance },
  { &hf_umts_rrc_pdu_def_start_Value, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_START_Value },
  { &hf_umts_rrc_pdu_def_count_C_ActivationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_dummy_RB_ActivationTimeInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_ActivationTimeInfoList },
  { &hf_umts_rrc_pdu_def_ul_CounterSynchronisationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_CounterSynchronisationInfo },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions35, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions35 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_RadioBearerSetupComplete(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RadioBearerSetupComplete, RadioBearerSetupComplete_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions63_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions63(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions63, T_nonCriticalExtensions63_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions36_sequence[] = {
  { &hf_umts_rrc_pdu_def_radioBearerSetupFailure_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions63, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions63 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions36(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions36, T_laterNonCriticalExtensions36_sequence);

  return offset;
}


static const per_sequence_t RadioBearerSetupFailure_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_failureCause, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_FailureCauseWithProtErr },
  { &hf_umts_rrc_pdu_def_potentiallySuccesfulBearerList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_IdentityList },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions36, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions36 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_RadioBearerSetupFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RadioBearerSetupFailure, RadioBearerSetupFailure_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionReject_r3_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_initialUE_Identity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_InitialUE_Identity },
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_rejectionCause, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RejectionCause },
  { &hf_umts_rrc_pdu_def_waitTime, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_WaitTime },
  { &hf_umts_rrc_pdu_def_redirectionInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RedirectionInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RRCConnectionReject_r3_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RRCConnectionReject_r3_IEs, RRCConnectionReject_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionReject_v6xyext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_redirectionInfo_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_GSM_TargetCellInfoList },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RRCConnectionReject_v6xyext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RRCConnectionReject_v6xyext_IEs, RRCConnectionReject_v6xyext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions64_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions64(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions64, T_nonCriticalExtensions64_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions25_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrcConnectionReject_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RRCConnectionReject_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions64, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions64 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions25(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions25, T_v6xyNonCriticalExtensions25_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions37_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrcConnectionReject_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions25, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions25 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions37(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions37, T_laterNonCriticalExtensions37_sequence);

  return offset;
}


static const per_sequence_t T_r317_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrcConnectionReject_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RRCConnectionReject_r3_IEs },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions37, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions37 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r317(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r317, T_r317_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions43_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions43(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_criticalExtensions43, T_criticalExtensions43_sequence);

  return offset;
}


static const per_sequence_t T_later_than_r315_sequence[] = {
  { &hf_umts_rrc_pdu_def_initialUE_Identity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_InitialUE_Identity },
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_criticalExtensions43, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_criticalExtensions43 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_later_than_r315(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_later_than_r315, T_later_than_r315_sequence);

  return offset;
}


const value_string umts_rrc_pdu_def_RRCConnectionReject_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t RRCConnectionReject_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r317, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r317 },
  {   1, &hf_umts_rrc_pdu_def_later_than_r315, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_later_than_r315 },
  { 0, NULL, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_RRCConnectionReject(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_RRCConnectionReject, RRCConnectionReject_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RRCConnectionRelease_r3_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_n_308, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_N_308 },
  { &hf_umts_rrc_pdu_def_releaseCause, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_ReleaseCause },
  { &hf_umts_rrc_pdu_def_rplmn_information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_Rplmn_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RRCConnectionRelease_r3_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RRCConnectionRelease_r3_IEs, RRCConnectionRelease_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionRelease_v6xyext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_redirectionInfo_v6xyext1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RedirectionInfo_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RRCConnectionRelease_v6xyext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RRCConnectionRelease_v6xyext_IEs, RRCConnectionRelease_v6xyext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions65_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions65(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions65, T_nonCriticalExtensions65_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions26_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrcConnectionRelease_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RRCConnectionRelease_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions65, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions65 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions26(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions26, T_v6xyNonCriticalExtensions26_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions38_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrcConnectionRelease_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions26, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions26 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions38(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions38, T_laterNonCriticalExtensions38_sequence);

  return offset;
}


static const per_sequence_t T_r318_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrcConnectionRelease_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RRCConnectionRelease_r3_IEs },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions38, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions38 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r318(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r318, T_r318_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionRelease_r4_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_n_308, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_N_308 },
  { &hf_umts_rrc_pdu_def_releaseCause, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_ReleaseCause },
  { &hf_umts_rrc_pdu_def_rplmn_information1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_Rplmn_Information_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RRCConnectionRelease_r4_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RRCConnectionRelease_r4_IEs, RRCConnectionRelease_r4_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions66_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions66(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions66, T_nonCriticalExtensions66_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions27_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrcConnectionRelease_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RRCConnectionRelease_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions66, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions66 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions27(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions27, T_v6xyNonCriticalExtensions27_sequence);

  return offset;
}


static const per_sequence_t T_v4d0NonCriticalExtensions9_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrcConnectionRelease_r4_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions27, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions27 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions9(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions9, T_v4d0NonCriticalExtensions9_sequence);

  return offset;
}


static const per_sequence_t T_r49_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrcConnectionRelease_r4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RRCConnectionRelease_r4_IEs },
  { &hf_umts_rrc_pdu_def_v4d0NonCriticalExtensions9, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r49(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r49, T_r49_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions45_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions45(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_criticalExtensions45, T_criticalExtensions45_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_criticalExtensions44_vals[] = {
  {   0, "r4" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions44_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r49, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r49 },
  {   1, &hf_umts_rrc_pdu_def_criticalExtensions45, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_criticalExtensions45 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions44(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_criticalExtensions44, T_criticalExtensions44_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_later_than_r316_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_criticalExtensions44, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_criticalExtensions44 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_later_than_r316(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_later_than_r316, T_later_than_r316_sequence);

  return offset;
}


const value_string umts_rrc_pdu_def_RRCConnectionRelease_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t RRCConnectionRelease_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r318, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r318 },
  {   1, &hf_umts_rrc_pdu_def_later_than_r316, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_later_than_r316 },
  { 0, NULL, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_RRCConnectionRelease(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_RRCConnectionRelease, RRCConnectionRelease_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RRCConnectionRelease_CCCH_r3_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_u_RNTI, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_U_RNTI },
  { &hf_umts_rrc_pdu_def_rrcConnectionRelease, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RRCConnectionRelease_r3_IEs },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RRCConnectionRelease_CCCH_r3_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RRCConnectionRelease_CCCH_r3_IEs, RRCConnectionRelease_CCCH_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions67_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions67(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions67, T_nonCriticalExtensions67_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions39_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrcConnectionRelease_CCCH_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions67, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions67 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions39(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions39, T_laterNonCriticalExtensions39_sequence);

  return offset;
}


static const per_sequence_t T_r319_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrcConnectionRelease_CCCH_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RRCConnectionRelease_CCCH_r3_IEs },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions39, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions39 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r319(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r319, T_r319_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionRelease_CCCH_r4_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrcConnectionRelease1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RRCConnectionRelease_r4_IEs },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RRCConnectionRelease_CCCH_r4_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RRCConnectionRelease_CCCH_r4_IEs, RRCConnectionRelease_CCCH_r4_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions68_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions68(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions68, T_nonCriticalExtensions68_sequence);

  return offset;
}


static const per_sequence_t T_v4d0NonCriticalExtensions10_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrcConnectionRelease_CCCH_r4_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions68, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions68 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions10(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions10, T_v4d0NonCriticalExtensions10_sequence);

  return offset;
}


static const per_sequence_t T_r410_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrcConnectionRelease_CCCH_r4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RRCConnectionRelease_CCCH_r4_IEs },
  { &hf_umts_rrc_pdu_def_v4d0NonCriticalExtensions10, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r410(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r410, T_r410_sequence);

  return offset;
}


static const per_sequence_t SEQUENCE_SIZE_1_maxURNTIGroup_OF_GroupReleaseInformation_sequence_of[1] = {
  { &hf_umts_rrc_pdu_def_groupIdentity_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_GroupReleaseInformation },
};

static int
dissect_umts_rrc_pdu_def_SEQUENCE_SIZE_1_maxURNTIGroup_OF_GroupReleaseInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_umts_rrc_pdu_def_SEQUENCE_SIZE_1_maxURNTIGroup_OF_GroupReleaseInformation, SEQUENCE_SIZE_1_maxURNTIGroup_OF_GroupReleaseInformation_sequence_of,
                                                  1, maxURNTIGroup);

  return offset;
}



static int
dissect_umts_rrc_pdu_def_RRCConnectionRelease_CCCH_r5_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_umts_rrc_pdu_def_RRCConnectionRelease_CCCH_r4_IEs(tvb, offset, actx, tree, hf_index);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions69_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions69(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions69, T_nonCriticalExtensions69_sequence);

  return offset;
}


static const per_sequence_t T_r58_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrcConnectionRelease_CCCH_r5, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RRCConnectionRelease_CCCH_r5_IEs },
  { &hf_umts_rrc_pdu_def_rrcConnectionRelease_CCCH_r5_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions69, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions69 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r58(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r58, T_r58_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions49_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions49(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_criticalExtensions49, T_criticalExtensions49_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_criticalExtensions48_vals[] = {
  {   0, "r5" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions48_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r58, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r58 },
  {   1, &hf_umts_rrc_pdu_def_criticalExtensions49, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_criticalExtensions49 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions48(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_criticalExtensions48, T_criticalExtensions48_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_criticalExtensions47_sequence[] = {
  { &hf_umts_rrc_pdu_def_groupIdentity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_SEQUENCE_SIZE_1_maxURNTIGroup_OF_GroupReleaseInformation },
  { &hf_umts_rrc_pdu_def_criticalExtensions48, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_criticalExtensions48 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions47(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_criticalExtensions47, T_criticalExtensions47_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_criticalExtensions46_vals[] = {
  {   0, "r4" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions46_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r410, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r410 },
  {   1, &hf_umts_rrc_pdu_def_criticalExtensions47, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_criticalExtensions47 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions46(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_criticalExtensions46, T_criticalExtensions46_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_later_than_r317_sequence[] = {
  { &hf_umts_rrc_pdu_def_u_RNTI, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_U_RNTI },
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_criticalExtensions46, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_criticalExtensions46 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_later_than_r317(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_later_than_r317, T_later_than_r317_sequence);

  return offset;
}


const value_string umts_rrc_pdu_def_RRCConnectionRelease_CCCH_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t RRCConnectionRelease_CCCH_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r319, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r319 },
  {   1, &hf_umts_rrc_pdu_def_later_than_r317, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_later_than_r317 },
  { 0, NULL, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_RRCConnectionRelease_CCCH(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_RRCConnectionRelease_CCCH, RRCConnectionRelease_CCCH_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions70_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions70(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions70, T_nonCriticalExtensions70_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions40_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrcConnectionReleaseComplete_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions70, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions70 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions40(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions40, T_laterNonCriticalExtensions40_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionReleaseComplete_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_errorIndication, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_FailureCauseWithProtErr },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions40, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions40 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_RRCConnectionReleaseComplete(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RRCConnectionReleaseComplete, RRCConnectionReleaseComplete_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionRequest_v3d0ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_uESpecificBehaviourInformation1idle, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UESpecificBehaviourInformation1idle },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RRCConnectionRequest_v3d0ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RRCConnectionRequest_v3d0ext_IEs, RRCConnectionRequest_v3d0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionRequest_v4b0ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_accessStratumReleaseIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_AccessStratumReleaseIndicator },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RRCConnectionRequest_v4b0ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RRCConnectionRequest_v4b0ext_IEs, RRCConnectionRequest_v4b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionRequest_v590ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_predefinedConfigStatusInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_BOOLEAN },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RRCConnectionRequest_v590ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RRCConnectionRequest_v590ext_IEs, RRCConnectionRequest_v590ext_IEs_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_ueCapabilityIndication_vals[] = {
  {   0, "hsdch" },
  {   1, "hsdch-edch" },
  { 0, NULL }
};


static int
dissect_umts_rrc_pdu_def_T_ueCapabilityIndication(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t RRCConnectionRequest_v6xyext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_ueCapabilityIndication, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_ueCapabilityIndication },
  { &hf_umts_rrc_pdu_def_measuredResultsOnRACHinterFreq, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MeasuredResultsOnRACHinterFreq },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RRCConnectionRequest_v6xyext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RRCConnectionRequest_v6xyext_IEs, RRCConnectionRequest_v6xyext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions71_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions71(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions71, T_nonCriticalExtensions71_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions28_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrcConnectionRequest_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RRCConnectionRequest_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions71, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions71 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions28(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions28, T_v6xyNonCriticalExtensions28_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions20_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrcConnectionRequest_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RRCConnectionRequest_v590ext_IEs },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions28, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions28 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions20(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions20, T_v590NonCriticalExtensions20_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions10_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrcConnectionRequest_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RRCConnectionRequest_v4b0ext_IEs },
  { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions20, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions20 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions10(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions10, T_v4b0NonCriticalExtensions10_sequence);

  return offset;
}


static const per_sequence_t T_v3d0NonCriticalExtensions_sequence[] = {
  { &hf_umts_rrc_pdu_def_rRCConnectionRequest_v3d0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RRCConnectionRequest_v3d0ext_IEs },
  { &hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions10, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v3d0NonCriticalExtensions(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v3d0NonCriticalExtensions, T_v3d0NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionRequest_sequence[] = {
  { &hf_umts_rrc_pdu_def_initialUE_Identity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_InitialUE_Identity },
  { &hf_umts_rrc_pdu_def_establishmentCause, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_EstablishmentCause },
  { &hf_umts_rrc_pdu_def_protocolErrorIndicator1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_ProtocolErrorIndicator },
  { &hf_umts_rrc_pdu_def_measuredResultsOnRACH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MeasuredResultsOnRACH },
  { &hf_umts_rrc_pdu_def_v3d0NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v3d0NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_RRCConnectionRequest(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RRCConnectionRequest, RRCConnectionRequest_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionSetup_r3_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_initialUE_Identity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_InitialUE_Identity },
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_activationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_new_U_RNTI, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_U_RNTI },
  { &hf_umts_rrc_pdu_def_new_c_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_C_RNTI },
  { &hf_umts_rrc_pdu_def_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_StateIndicator },
  { &hf_umts_rrc_pdu_def_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_umts_rrc_pdu_def_capabilityUpdateRequirement, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CapabilityUpdateRequirement },
  { &hf_umts_rrc_pdu_def_srb_InformationSetupList3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_SRB_InformationSetupList2 },
  { &hf_umts_rrc_pdu_def_ul_CommonTransChInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_CommonTransChInfo },
  { &hf_umts_rrc_pdu_def_ul_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_UL_AddReconfTransChInfoList },
  { &hf_umts_rrc_pdu_def_dl_CommonTransChInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonTransChInfo },
  { &hf_umts_rrc_pdu_def_dl_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DL_AddReconfTransChInfoList },
  { &hf_umts_rrc_pdu_def_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_FrequencyInfo },
  { &hf_umts_rrc_pdu_def_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MaxAllowedUL_TX_Power },
  { &hf_umts_rrc_pdu_def_ul_ChannelRequirement, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_ChannelRequirement },
  { &hf_umts_rrc_pdu_def_dl_CommonInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonInformation },
  { &hf_umts_rrc_pdu_def_dl_InformationPerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_InformationPerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RRCConnectionSetup_r3_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RRCConnectionSetup_r3_IEs, RRCConnectionSetup_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionSetup_v4b0ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_capabilityUpdateRequirement_r4_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CapabilityUpdateRequirement_r4_ext },
  { &hf_umts_rrc_pdu_def_dummy_SSDT_UL, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_SSDT_UL },
  { &hf_umts_rrc_pdu_def_cell_id_PerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CellIdentity_PerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RRCConnectionSetup_v4b0ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RRCConnectionSetup_v4b0ext_IEs, RRCConnectionSetup_v4b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionSetup_v590ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_systemSpecificCapUpdateReq, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_SystemSpecificCapUpdateReq_v590ext },
  { &hf_umts_rrc_pdu_def_dl_TPC_PowerOffsetPerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_TPC_PowerOffsetPerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RRCConnectionSetup_v590ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RRCConnectionSetup_v590ext_IEs, RRCConnectionSetup_v590ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions72_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions72(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions72, T_nonCriticalExtensions72_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions21_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrcConnectionSetup_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RRCConnectionSetup_v590ext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions72, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions72 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions21(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions21, T_v590NonCriticalExtensions21_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions11_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrcConnectionSetup_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RRCConnectionSetup_v4b0ext_IEs },
  { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions21, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions21 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions11(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions11, T_v4b0NonCriticalExtensions11_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions41_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrcConnectionSetup_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions11, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions41(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions41, T_laterNonCriticalExtensions41_sequence);

  return offset;
}


static const per_sequence_t T_r320_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrcConnectionSetup_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RRCConnectionSetup_r3_IEs },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions41, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions41 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r320(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r320, T_r320_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionSetup_r4_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_activationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_new_U_RNTI, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_U_RNTI },
  { &hf_umts_rrc_pdu_def_new_c_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_C_RNTI },
  { &hf_umts_rrc_pdu_def_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_StateIndicator },
  { &hf_umts_rrc_pdu_def_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_umts_rrc_pdu_def_capabilityUpdateRequirement1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CapabilityUpdateRequirement_r4 },
  { &hf_umts_rrc_pdu_def_srb_InformationSetupList3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_SRB_InformationSetupList2 },
  { &hf_umts_rrc_pdu_def_ul_CommonTransChInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_CommonTransChInfo_r4 },
  { &hf_umts_rrc_pdu_def_ul_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_AddReconfTransChInfoList },
  { &hf_umts_rrc_pdu_def_dl_CommonTransChInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonTransChInfo_r4 },
  { &hf_umts_rrc_pdu_def_dl_AddReconfTransChInfoList1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_AddReconfTransChInfoList_r4 },
  { &hf_umts_rrc_pdu_def_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_FrequencyInfo },
  { &hf_umts_rrc_pdu_def_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MaxAllowedUL_TX_Power },
  { &hf_umts_rrc_pdu_def_ul_ChannelRequirement1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_ChannelRequirement_r4 },
  { &hf_umts_rrc_pdu_def_dl_CommonInformation1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonInformation_r4 },
  { &hf_umts_rrc_pdu_def_dl_InformationPerRL_List1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_InformationPerRL_List_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RRCConnectionSetup_r4_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RRCConnectionSetup_r4_IEs, RRCConnectionSetup_r4_IEs_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_postVerificationPeriod5_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_umts_rrc_pdu_def_T_postVerificationPeriod5(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t RRCConnectionSetup_v6xyext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_beaconPLEst, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_BEACON_PL_Est },
  { &hf_umts_rrc_pdu_def_postVerificationPeriod5, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_postVerificationPeriod5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RRCConnectionSetup_v6xyext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RRCConnectionSetup_v6xyext_IEs, RRCConnectionSetup_v6xyext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions73_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions73(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions73, T_nonCriticalExtensions73_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions29_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrcConnectionSetup_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RRCConnectionSetup_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions73, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions73 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions29(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions29, T_v6xyNonCriticalExtensions29_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions22_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrcConnectionSetup_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RRCConnectionSetup_v590ext_IEs },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions29, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions29 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions22(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions22, T_v590NonCriticalExtensions22_sequence);

  return offset;
}


static const per_sequence_t T_v4d0NonCriticalExtensions11_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrcConnectionSetup_r4_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions22, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions22 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions11(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions11, T_v4d0NonCriticalExtensions11_sequence);

  return offset;
}


static const per_sequence_t T_r411_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrcConnectionSetup_r4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RRCConnectionSetup_r4_IEs },
  { &hf_umts_rrc_pdu_def_v4d0NonCriticalExtensions11, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r411(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r411, T_r411_sequence);

  return offset;
}


static const per_sequence_t T_complete7_sequence[] = {
  { &hf_umts_rrc_pdu_def_srb_InformationSetupList3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_SRB_InformationSetupList2 },
  { &hf_umts_rrc_pdu_def_ul_CommonTransChInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_CommonTransChInfo_r4 },
  { &hf_umts_rrc_pdu_def_ul_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_AddReconfTransChInfoList },
  { &hf_umts_rrc_pdu_def_dl_CommonTransChInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonTransChInfo_r4 },
  { &hf_umts_rrc_pdu_def_dl_AddReconfTransChInfoList1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_AddReconfTransChInfoList_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_complete7(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_complete7, T_complete7_sequence);

  return offset;
}


static const per_sequence_t T_defaultConfig5_sequence[] = {
  { &hf_umts_rrc_pdu_def_defaultConfigMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DefaultConfigMode },
  { &hf_umts_rrc_pdu_def_defaultConfigIdentity2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DefaultConfigIdentity_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_defaultConfig5(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_defaultConfig5, T_defaultConfig5_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_preConfigMode5_vals[] = {
  {   0, "predefinedConfigIdentity" },
  {   1, "defaultConfig" },
  { 0, NULL }
};

static const per_choice_t T_preConfigMode5_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_predefinedConfigIdentity, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_ies_PredefinedConfigIdentity },
  {   1, &hf_umts_rrc_pdu_def_defaultConfig5, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_defaultConfig5 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_preConfigMode5(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_preConfigMode5, T_preConfigMode5_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_preconfiguration6_sequence[] = {
  { &hf_umts_rrc_pdu_def_preConfigMode5, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_preConfigMode5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_preconfiguration6(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_preconfiguration6, T_preconfiguration6_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_specificationMode7_vals[] = {
  {   0, "complete" },
  {   1, "preconfiguration" },
  { 0, NULL }
};

static const per_choice_t T_specificationMode7_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_complete7, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_complete7 },
  {   1, &hf_umts_rrc_pdu_def_preconfiguration6, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_preconfiguration6 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_specificationMode7(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_specificationMode7, T_specificationMode7_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RRCConnectionSetup_r5_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_activationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_new_U_RNTI, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_U_RNTI },
  { &hf_umts_rrc_pdu_def_new_c_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_C_RNTI },
  { &hf_umts_rrc_pdu_def_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_StateIndicator },
  { &hf_umts_rrc_pdu_def_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_umts_rrc_pdu_def_capabilityUpdateRequirement2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CapabilityUpdateRequirement_r5 },
  { &hf_umts_rrc_pdu_def_specificationMode7, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_specificationMode7 },
  { &hf_umts_rrc_pdu_def_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_FrequencyInfo },
  { &hf_umts_rrc_pdu_def_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MaxAllowedUL_TX_Power },
  { &hf_umts_rrc_pdu_def_ul_ChannelRequirement1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_ChannelRequirement_r4 },
  { &hf_umts_rrc_pdu_def_dl_CommonInformation1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonInformation_r4 },
  { &hf_umts_rrc_pdu_def_dl_InformationPerRL_List5, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_InformationPerRL_List_r5bis },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RRCConnectionSetup_r5_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RRCConnectionSetup_r5_IEs, RRCConnectionSetup_r5_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions74_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions74(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions74, T_nonCriticalExtensions74_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions30_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrcConnectionSetup_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RRCConnectionSetup_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions74, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions74 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions30(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions30, T_v6xyNonCriticalExtensions30_sequence);

  return offset;
}


static const per_sequence_t T_r59_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrcConnectionSetup_r5, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RRCConnectionSetup_r5_IEs },
  { &hf_umts_rrc_pdu_def_rrcConnectionSetup_r5_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions30, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions30 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r59(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r59, T_r59_sequence);

  return offset;
}


static const per_sequence_t T_complete8_sequence[] = {
  { &hf_umts_rrc_pdu_def_srb_InformationSetupList2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_SRB_InformationSetupList_r6 },
  { &hf_umts_rrc_pdu_def_ul_CommonTransChInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_CommonTransChInfo_r4 },
  { &hf_umts_rrc_pdu_def_ul_AddReconfTransChInfoList1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_AddReconfTransChInfoList_r6 },
  { &hf_umts_rrc_pdu_def_dl_CommonTransChInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonTransChInfo_r4 },
  { &hf_umts_rrc_pdu_def_dl_AddReconfTransChInfoList2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_AddReconfTransChInfoList_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_complete8(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_complete8, T_complete8_sequence);

  return offset;
}


static const per_sequence_t T_defaultConfig6_sequence[] = {
  { &hf_umts_rrc_pdu_def_defaultConfigMode, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DefaultConfigMode },
  { &hf_umts_rrc_pdu_def_defaultConfigIdentity2, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DefaultConfigIdentity_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_defaultConfig6(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_defaultConfig6, T_defaultConfig6_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_preConfigMode6_vals[] = {
  {   0, "predefinedConfigIdentity" },
  {   1, "defaultConfig" },
  { 0, NULL }
};

static const per_choice_t T_preConfigMode6_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_predefinedConfigIdentity, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_ies_PredefinedConfigIdentity },
  {   1, &hf_umts_rrc_pdu_def_defaultConfig6, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_defaultConfig6 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_preConfigMode6(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_preConfigMode6, T_preConfigMode6_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_preconfiguration7_sequence[] = {
  { &hf_umts_rrc_pdu_def_preConfigMode6, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_preConfigMode6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_preconfiguration7(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_preconfiguration7, T_preconfiguration7_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_specificationMode8_vals[] = {
  {   0, "complete" },
  {   1, "preconfiguration" },
  { 0, NULL }
};

static const per_choice_t T_specificationMode8_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_complete8, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_complete8 },
  {   1, &hf_umts_rrc_pdu_def_preconfiguration7, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_preconfiguration7 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_specificationMode8(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_specificationMode8, T_specificationMode8_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RRCConnectionSetup_r6_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_activationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_new_U_RNTI, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_U_RNTI },
  { &hf_umts_rrc_pdu_def_new_c_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_C_RNTI },
  { &hf_umts_rrc_pdu_def_new_H_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_H_RNTI },
  { &hf_umts_rrc_pdu_def_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_E_RNTI },
  { &hf_umts_rrc_pdu_def_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_E_RNTI },
  { &hf_umts_rrc_pdu_def_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_StateIndicator },
  { &hf_umts_rrc_pdu_def_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_umts_rrc_pdu_def_capabilityUpdateRequirement2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CapabilityUpdateRequirement_r5 },
  { &hf_umts_rrc_pdu_def_specificationMode8, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_specificationMode8 },
  { &hf_umts_rrc_pdu_def_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_FrequencyInfo },
  { &hf_umts_rrc_pdu_def_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MaxAllowedUL_TX_Power },
  { &hf_umts_rrc_pdu_def_ul_ChannelRequirement3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_ChannelRequirement_r6 },
  { &hf_umts_rrc_pdu_def_ul_EDCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_EDCH_Information_r6 },
  { &hf_umts_rrc_pdu_def_dl_HSPDSCH_Information1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_HSPDSCH_Information_r6 },
  { &hf_umts_rrc_pdu_def_dl_CommonInformation3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonInformation_r6 },
  { &hf_umts_rrc_pdu_def_dl_InformationPerRL_List3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_InformationPerRL_List_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RRCConnectionSetup_r6_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RRCConnectionSetup_r6_IEs, RRCConnectionSetup_r6_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions75_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions75(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions75, T_nonCriticalExtensions75_sequence);

  return offset;
}


static const per_sequence_t T_r69_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrcConnectionSetup_r6, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RRCConnectionSetup_r6_IEs },
  { &hf_umts_rrc_pdu_def_rrcConnectionSetup_r6_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions75, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions75 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r69(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r69, T_r69_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions53_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions53(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_criticalExtensions53, T_criticalExtensions53_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_criticalExtensions52_vals[] = {
  {   0, "r6" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions52_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r69, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r69 },
  {   1, &hf_umts_rrc_pdu_def_criticalExtensions53, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_criticalExtensions53 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions52(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_criticalExtensions52, T_criticalExtensions52_choice,
                                 NULL);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_criticalExtensions51_vals[] = {
  {   0, "r5" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions51_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r59, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r59 },
  {   1, &hf_umts_rrc_pdu_def_criticalExtensions52, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_criticalExtensions52 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions51(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_criticalExtensions51, T_criticalExtensions51_choice,
                                 NULL);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_criticalExtensions50_vals[] = {
  {   0, "r4" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions50_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r411, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r411 },
  {   1, &hf_umts_rrc_pdu_def_criticalExtensions51, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_criticalExtensions51 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions50(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_criticalExtensions50, T_criticalExtensions50_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_later_than_r318_sequence[] = {
  { &hf_umts_rrc_pdu_def_initialUE_Identity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_InitialUE_Identity },
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_criticalExtensions50, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_criticalExtensions50 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_later_than_r318(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_later_than_r318, T_later_than_r318_sequence);

  return offset;
}


const value_string umts_rrc_pdu_def_RRCConnectionSetup_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t RRCConnectionSetup_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r320, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r320 },
  {   1, &hf_umts_rrc_pdu_def_later_than_r318, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_later_than_r318 },
  { 0, NULL, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_RRCConnectionSetup(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_RRCConnectionSetup, RRCConnectionSetup_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t RRCConnectionSetupComplete_v370ext_sequence[] = {
  { &hf_umts_rrc_pdu_def_ue_RadioAccessCapability_v370ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UE_RadioAccessCapability_v370ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RRCConnectionSetupComplete_v370ext(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RRCConnectionSetupComplete_v370ext, RRCConnectionSetupComplete_v370ext_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionSetupComplete_v380ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_ue_RadioAccessCapability_v380ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UE_RadioAccessCapability_v380ext },
  { &hf_umts_rrc_pdu_def_dl_PhysChCapabilityFDD_v380ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DL_PhysChCapabilityFDD_v380ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RRCConnectionSetupComplete_v380ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RRCConnectionSetupComplete_v380ext_IEs, RRCConnectionSetupComplete_v380ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionSetupComplete_v3a0ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_ue_RadioAccessCapability_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UE_RadioAccessCapability_v3a0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RRCConnectionSetupComplete_v3a0ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RRCConnectionSetupComplete_v3a0ext_IEs, RRCConnectionSetupComplete_v3a0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionSetupComplete_v3g0ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_ue_RadioAccessCapability_v3g0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UE_RadioAccessCapability_v3g0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RRCConnectionSetupComplete_v3g0ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RRCConnectionSetupComplete_v3g0ext_IEs, RRCConnectionSetupComplete_v3g0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionSetupComplete_v4b0ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_ue_RadioAccessCapability_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UE_RadioAccessCapability_v4b0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RRCConnectionSetupComplete_v4b0ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RRCConnectionSetupComplete_v4b0ext_IEs, RRCConnectionSetupComplete_v4b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionSetupComplete_v590ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_ue_RadioAccessCapability_v590ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UE_RadioAccessCapability_v590ext },
  { &hf_umts_rrc_pdu_def_ue_RATSpecificCapability_v590ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_InterRAT_UE_RadioAccessCapability_v590ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RRCConnectionSetupComplete_v590ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RRCConnectionSetupComplete_v590ext_IEs, RRCConnectionSetupComplete_v590ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionSetupComplete_v5c0ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_ue_RadioAccessCapability_v5c0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UE_RadioAccessCapability_v5c0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RRCConnectionSetupComplete_v5c0ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RRCConnectionSetupComplete_v5c0ext_IEs, RRCConnectionSetupComplete_v5c0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionSetupComplete_v6xyext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_ueCapabilityContainer, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RRCConnectionSetupComplete_v6xyext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RRCConnectionSetupComplete_v6xyext_IEs, RRCConnectionSetupComplete_v6xyext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions76_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions76(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions76, T_nonCriticalExtensions76_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions31_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrcConnectionSetupComplete_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RRCConnectionSetupComplete_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions76, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions76 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions31(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions31, T_v6xyNonCriticalExtensions31_sequence);

  return offset;
}


static const per_sequence_t T_v5c0NonCriticalExtensions_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrcConnectionSetupComplete_v5c0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RRCConnectionSetupComplete_v5c0ext_IEs },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions31, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions31 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v5c0NonCriticalExtensions(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v5c0NonCriticalExtensions, T_v5c0NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions23_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrcConnectionSetupComplete_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RRCConnectionSetupComplete_v590ext_IEs },
  { &hf_umts_rrc_pdu_def_v5c0NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v5c0NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions23(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions23, T_v590NonCriticalExtensions23_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions12_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrcConnectionSetupComplete_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RRCConnectionSetupComplete_v4b0ext_IEs },
  { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions23, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions23 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions12(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions12, T_v4b0NonCriticalExtensions12_sequence);

  return offset;
}


static const per_sequence_t T_v3g0NonCriticalExtensions1_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrcConnectionSetupComplete_v3g0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RRCConnectionSetupComplete_v3g0ext_IEs },
  { &hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions12, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions12 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v3g0NonCriticalExtensions1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v3g0NonCriticalExtensions1, T_v3g0NonCriticalExtensions1_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions42_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrcConnectionSetupComplete_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v3g0NonCriticalExtensions1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v3g0NonCriticalExtensions1 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions42(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions42, T_laterNonCriticalExtensions42_sequence);

  return offset;
}


static const per_sequence_t T_v3a0NonCriticalExtensions8_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrcConnectionSetupComplete_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RRCConnectionSetupComplete_v3a0ext_IEs },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions42, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions42 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions8(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions8, T_v3a0NonCriticalExtensions8_sequence);

  return offset;
}


static const per_sequence_t T_v380NonCriticalExtensions_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrcConnectionSetupComplete_v380ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RRCConnectionSetupComplete_v380ext_IEs },
  { &hf_umts_rrc_pdu_def_v3a0NonCriticalExtensions8, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions8 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v380NonCriticalExtensions(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v380NonCriticalExtensions, T_v380NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_v370NonCriticalExtensions_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrcConnectionSetupComplete_v370ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RRCConnectionSetupComplete_v370ext },
  { &hf_umts_rrc_pdu_def_v380NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v380NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v370NonCriticalExtensions(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v370NonCriticalExtensions, T_v370NonCriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionSetupComplete_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_startList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_STARTList },
  { &hf_umts_rrc_pdu_def_ue_RadioAccessCapability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UE_RadioAccessCapability },
  { &hf_umts_rrc_pdu_def_ue_RATSpecificCapability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_InterRAT_UE_RadioAccessCapabilityList },
  { &hf_umts_rrc_pdu_def_v370NonCriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v370NonCriticalExtensions },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_RRCConnectionSetupComplete(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RRCConnectionSetupComplete, RRCConnectionSetupComplete_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionSetupComplete_v650ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_ue_RadioAccessCapability_v650ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_UE_RadioAccessCapability_v650ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RRCConnectionSetupComplete_v650ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RRCConnectionSetupComplete_v650ext_IEs, RRCConnectionSetupComplete_v650ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions77_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions77(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions77, T_nonCriticalExtensions77_sequence);

  return offset;
}


static const per_sequence_t RRCConnectionSetupComplete_r3_add_ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrcConnectionSetupComplete_v650ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_RRCConnectionSetupComplete_v650ext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions77, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions77 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RRCConnectionSetupComplete_r3_add_ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RRCConnectionSetupComplete_r3_add_ext_IEs, RRCConnectionSetupComplete_r3_add_ext_IEs_sequence);

  return offset;
}


static const per_sequence_t RRC_FailureInfo_r3_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_failureCauseWithProtErr, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_FailureCauseWithProtErr },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RRC_FailureInfo_r3_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RRC_FailureInfo_r3_IEs, RRC_FailureInfo_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions78_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions78(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions78, T_nonCriticalExtensions78_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions43_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_FailureInfo_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions78, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions78 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions43(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions43, T_laterNonCriticalExtensions43_sequence);

  return offset;
}


static const per_sequence_t T_r321_sequence[] = {
  { &hf_umts_rrc_pdu_def_rRC_FailureInfo_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_RRC_FailureInfo_r3_IEs },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions43, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions43 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r321(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r321, T_r321_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions54_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions54(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_criticalExtensions54, T_criticalExtensions54_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_RRC_FailureInfo_vals[] = {
  {   0, "r3" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t RRC_FailureInfo_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r321, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r321 },
  {   1, &hf_umts_rrc_pdu_def_criticalExtensions54, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_criticalExtensions54 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_RRC_FailureInfo(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_RRC_FailureInfo, RRC_FailureInfo_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions79_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions79(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions79, T_nonCriticalExtensions79_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions44_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrcStatus_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions79, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions79 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions44(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions44, T_laterNonCriticalExtensions44_sequence);

  return offset;
}


static const per_sequence_t RRCStatus_sequence[] = {
  { &hf_umts_rrc_pdu_def_protocolErrorInformation, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_ProtocolErrorMoreInformation },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions44, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions44 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_RRCStatus(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_RRCStatus, RRCStatus_sequence);

  return offset;
}


static const per_sequence_t SecurityModeCommand_r3_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_securityCapability, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_SecurityCapability },
  { &hf_umts_rrc_pdu_def_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CipheringModeInfo },
  { &hf_umts_rrc_pdu_def_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_IntegrityProtectionModeInfo },
  { &hf_umts_rrc_pdu_def_cn_DomainIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_CN_DomainIdentity },
  { &hf_umts_rrc_pdu_def_ue_SystemSpecificSecurityCap, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_InterRAT_UE_SecurityCapList },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_SecurityModeCommand_r3_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_SecurityModeCommand_r3_IEs, SecurityModeCommand_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t SecurityModeCommand_v6xyext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_gsmSecurityCapability_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_GSMSecurityCapability_v6xyext },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_SecurityModeCommand_v6xyext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_SecurityModeCommand_v6xyext_IEs, SecurityModeCommand_v6xyext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions80_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions80(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions80, T_nonCriticalExtensions80_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions32_sequence[] = {
  { &hf_umts_rrc_pdu_def_securityModeCommand_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_SecurityModeCommand_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions80, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions80 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions32(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions32, T_v6xyNonCriticalExtensions32_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions45_sequence[] = {
  { &hf_umts_rrc_pdu_def_securityModeCommand_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions32, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions32 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions45(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions45, T_laterNonCriticalExtensions45_sequence);

  return offset;
}


static const per_sequence_t T_r322_sequence[] = {
  { &hf_umts_rrc_pdu_def_securityModeCommand_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_SecurityModeCommand_r3_IEs },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions45, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions45 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r322(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r322, T_r322_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions55_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions55(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_criticalExtensions55, T_criticalExtensions55_sequence);

  return offset;
}


static const per_sequence_t T_later_than_r319_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_criticalExtensions55, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_criticalExtensions55 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_later_than_r319(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_later_than_r319, T_later_than_r319_sequence);

  return offset;
}


const value_string umts_rrc_pdu_def_SecurityModeCommand_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t SecurityModeCommand_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r322, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r322 },
  {   1, &hf_umts_rrc_pdu_def_later_than_r319, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_later_than_r319 },
  { 0, NULL, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_SecurityModeCommand(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_SecurityModeCommand, SecurityModeCommand_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions81_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions81(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions81, T_nonCriticalExtensions81_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions46_sequence[] = {
  { &hf_umts_rrc_pdu_def_securityModeComplete_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions81, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions81 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions46(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions46, T_laterNonCriticalExtensions46_sequence);

  return offset;
}


static const per_sequence_t SecurityModeComplete_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_ul_IntegProtActivationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_IntegrityProtActivationInfo },
  { &hf_umts_rrc_pdu_def_rb_UL_CiphActivationTimeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_ActivationTimeInfoList },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions46, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions46 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_SecurityModeComplete(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_SecurityModeComplete, SecurityModeComplete_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions82_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions82(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions82, T_nonCriticalExtensions82_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions47_sequence[] = {
  { &hf_umts_rrc_pdu_def_securityModeFailure_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions82, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions82 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions47(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions47, T_laterNonCriticalExtensions47_sequence);

  return offset;
}


static const per_sequence_t SecurityModeFailure_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_failureCause, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_FailureCauseWithProtErr },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions47, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions47 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_SecurityModeFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_SecurityModeFailure, SecurityModeFailure_sequence);

  return offset;
}


static const per_sequence_t SignallingConnectionRelease_r3_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_cn_DomainIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_CN_DomainIdentity },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_SignallingConnectionRelease_r3_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_SignallingConnectionRelease_r3_IEs, SignallingConnectionRelease_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions83_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions83(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions83, T_nonCriticalExtensions83_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions48_sequence[] = {
  { &hf_umts_rrc_pdu_def_signallingConnectionRelease_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions83, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions83 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions48(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions48, T_laterNonCriticalExtensions48_sequence);

  return offset;
}


static const per_sequence_t T_r323_sequence[] = {
  { &hf_umts_rrc_pdu_def_signallingConnectionRelease_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_SignallingConnectionRelease_r3_IEs },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions48, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions48 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r323(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r323, T_r323_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions56_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions56(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_criticalExtensions56, T_criticalExtensions56_sequence);

  return offset;
}


static const per_sequence_t T_later_than_r320_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_criticalExtensions56, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_criticalExtensions56 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_later_than_r320(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_later_than_r320, T_later_than_r320_sequence);

  return offset;
}


const value_string umts_rrc_pdu_def_SignallingConnectionRelease_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t SignallingConnectionRelease_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r323, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r323 },
  {   1, &hf_umts_rrc_pdu_def_later_than_r320, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_later_than_r320 },
  { 0, NULL, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_SignallingConnectionRelease(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_SignallingConnectionRelease, SignallingConnectionRelease_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions84_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions84(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions84, T_nonCriticalExtensions84_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions49_sequence[] = {
  { &hf_umts_rrc_pdu_def_signallingConnectionReleaseIndication_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions84, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions84 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions49(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions49, T_laterNonCriticalExtensions49_sequence);

  return offset;
}


static const per_sequence_t SignallingConnectionReleaseIndication_sequence[] = {
  { &hf_umts_rrc_pdu_def_cn_DomainIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_CN_DomainIdentity },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions49, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions49 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_SignallingConnectionReleaseIndication(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_SignallingConnectionReleaseIndication, SignallingConnectionReleaseIndication_sequence);

  return offset;
}


static const per_sequence_t FirstSegment_sequence[] = {
  { &hf_umts_rrc_pdu_def_sib_Type, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_SIB_Type },
  { &hf_umts_rrc_pdu_def_seg_Count, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_SegCount },
  { &hf_umts_rrc_pdu_def_sib_Data_fixed, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_SIB_Data_fixed },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_FirstSegment(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_FirstSegment, FirstSegment_sequence);

  return offset;
}


static const per_sequence_t SubsequentSegment_sequence[] = {
  { &hf_umts_rrc_pdu_def_sib_Type, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_SIB_Type },
  { &hf_umts_rrc_pdu_def_segmentIndex, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_SegmentIndex },
  { &hf_umts_rrc_pdu_def_sib_Data_fixed, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_SIB_Data_fixed },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_SubsequentSegment(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_SubsequentSegment, SubsequentSegment_sequence);

  return offset;
}


static const per_sequence_t LastSegmentShort_sequence[] = {
  { &hf_umts_rrc_pdu_def_sib_Type, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_SIB_Type },
  { &hf_umts_rrc_pdu_def_segmentIndex, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_SegmentIndex },
  { &hf_umts_rrc_pdu_def_sib_Data_variable, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_SIB_Data_variable },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_LastSegmentShort(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_LastSegmentShort, LastSegmentShort_sequence);

  return offset;
}


static const per_sequence_t FirstSegmentShort_sequence[] = {
  { &hf_umts_rrc_pdu_def_sib_Type, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_SIB_Type },
  { &hf_umts_rrc_pdu_def_seg_Count, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_SegCount },
  { &hf_umts_rrc_pdu_def_sib_Data_variable, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_SIB_Data_variable },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_FirstSegmentShort(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_FirstSegmentShort, FirstSegmentShort_sequence);

  return offset;
}


static const per_sequence_t T_lastAndFirst_sequence[] = {
  { &hf_umts_rrc_pdu_def_lastSegmentShort, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_LastSegmentShort },
  { &hf_umts_rrc_pdu_def_firstSegment1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_FirstSegmentShort },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_lastAndFirst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_lastAndFirst, T_lastAndFirst_sequence);

  return offset;
}


static const per_sequence_t CompleteSIBshort_sequence[] = {
  { &hf_umts_rrc_pdu_def_sib_Type, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_SIB_Type },
  { &hf_umts_rrc_pdu_def_sib_Data_variable, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_SIB_Data_variable },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_CompleteSIBshort(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_CompleteSIBshort, CompleteSIBshort_sequence);

  return offset;
}


static const per_sequence_t CompleteSIB_List_sequence_of[1] = {
  { &hf_umts_rrc_pdu_def_CompleteSIB_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_CompleteSIBshort },
};

static int
dissect_umts_rrc_pdu_def_CompleteSIB_List(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_umts_rrc_pdu_def_CompleteSIB_List, CompleteSIB_List_sequence_of,
                                                  1, maxSIBperMsg);

  return offset;
}


static const per_sequence_t T_lastAndComplete_sequence[] = {
  { &hf_umts_rrc_pdu_def_lastSegmentShort, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_LastSegmentShort },
  { &hf_umts_rrc_pdu_def_completeSIB_List, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_CompleteSIB_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_lastAndComplete(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_lastAndComplete, T_lastAndComplete_sequence);

  return offset;
}


static const per_sequence_t T_lastAndCompleteAndFirst_sequence[] = {
  { &hf_umts_rrc_pdu_def_lastSegmentShort, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_LastSegmentShort },
  { &hf_umts_rrc_pdu_def_completeSIB_List, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_CompleteSIB_List },
  { &hf_umts_rrc_pdu_def_firstSegment1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_FirstSegmentShort },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_lastAndCompleteAndFirst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_lastAndCompleteAndFirst, T_lastAndCompleteAndFirst_sequence);

  return offset;
}


static const per_sequence_t T_completeAndFirst_sequence[] = {
  { &hf_umts_rrc_pdu_def_completeSIB_List, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_CompleteSIB_List },
  { &hf_umts_rrc_pdu_def_firstSegment1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_FirstSegmentShort },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_completeAndFirst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_completeAndFirst, T_completeAndFirst_sequence);

  return offset;
}



static int
dissect_umts_rrc_pdu_def_BIT_STRING_SIZE_226(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     226, 226, FALSE, NULL);

  return offset;
}


static const per_sequence_t CompleteSIB_sequence[] = {
  { &hf_umts_rrc_pdu_def_sib_Type, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_SIB_Type },
  { &hf_umts_rrc_pdu_def_sib_Data_fixed1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_BIT_STRING_SIZE_226 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_CompleteSIB(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_CompleteSIB, CompleteSIB_sequence);

  return offset;
}


static const per_sequence_t LastSegment_sequence[] = {
  { &hf_umts_rrc_pdu_def_sib_Type, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_SIB_Type },
  { &hf_umts_rrc_pdu_def_segmentIndex, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_SegmentIndex },
  { &hf_umts_rrc_pdu_def_sib_Data_fixed, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_SIB_Data_fixed },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_LastSegment(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_LastSegment, LastSegment_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_payload_vals[] = {
  {   0, "noSegment" },
  {   1, "firstSegment" },
  {   2, "subsequentSegment" },
  {   3, "lastSegmentShort" },
  {   4, "lastAndFirst" },
  {   5, "lastAndComplete" },
  {   6, "lastAndCompleteAndFirst" },
  {   7, "completeSIB-List" },
  {   8, "completeAndFirst" },
  {   9, "completeSIB" },
  {  10, "lastSegment" },
  {  11, "spare5" },
  {  12, "spare4" },
  {  13, "spare3" },
  {  14, "spare2" },
  {  15, "spare1" },
  { 0, NULL }
};

static const per_choice_t T_payload_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_noSegment, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  {   1, &hf_umts_rrc_pdu_def_firstSegment, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_FirstSegment },
  {   2, &hf_umts_rrc_pdu_def_subsequentSegment, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_SubsequentSegment },
  {   3, &hf_umts_rrc_pdu_def_lastSegmentShort, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_LastSegmentShort },
  {   4, &hf_umts_rrc_pdu_def_lastAndFirst, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_lastAndFirst },
  {   5, &hf_umts_rrc_pdu_def_lastAndComplete, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_lastAndComplete },
  {   6, &hf_umts_rrc_pdu_def_lastAndCompleteAndFirst, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_lastAndCompleteAndFirst },
  {   7, &hf_umts_rrc_pdu_def_completeSIB_List, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_CompleteSIB_List },
  {   8, &hf_umts_rrc_pdu_def_completeAndFirst, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_completeAndFirst },
  {   9, &hf_umts_rrc_pdu_def_completeSIB, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_CompleteSIB },
  {  10, &hf_umts_rrc_pdu_def_lastSegment, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_LastSegment },
  {  11, &hf_umts_rrc_pdu_def_spare5, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  {  12, &hf_umts_rrc_pdu_def_spare4, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  {  13, &hf_umts_rrc_pdu_def_spare3, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  {  14, &hf_umts_rrc_pdu_def_spare2, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  {  15, &hf_umts_rrc_pdu_def_spare1, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_payload(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_payload, T_payload_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t SystemInformation_BCH_sequence[] = {
  { &hf_umts_rrc_pdu_def_sfn_Prime, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_SFN_Prime },
  { &hf_umts_rrc_pdu_def_payload, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_payload },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_SystemInformation_BCH(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_SystemInformation_BCH, SystemInformation_BCH_sequence);

  return offset;
}


static const per_sequence_t T_lastAndFirst1_sequence[] = {
  { &hf_umts_rrc_pdu_def_lastSegmentShort, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_LastSegmentShort },
  { &hf_umts_rrc_pdu_def_firstSegment1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_FirstSegmentShort },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_lastAndFirst1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_lastAndFirst1, T_lastAndFirst1_sequence);

  return offset;
}


static const per_sequence_t T_lastAndComplete1_sequence[] = {
  { &hf_umts_rrc_pdu_def_lastSegmentShort, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_LastSegmentShort },
  { &hf_umts_rrc_pdu_def_completeSIB_List, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_CompleteSIB_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_lastAndComplete1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_lastAndComplete1, T_lastAndComplete1_sequence);

  return offset;
}


static const per_sequence_t T_lastAndCompleteAndFirst1_sequence[] = {
  { &hf_umts_rrc_pdu_def_lastSegmentShort, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_LastSegmentShort },
  { &hf_umts_rrc_pdu_def_completeSIB_List, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_CompleteSIB_List },
  { &hf_umts_rrc_pdu_def_firstSegment1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_FirstSegmentShort },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_lastAndCompleteAndFirst1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_lastAndCompleteAndFirst1, T_lastAndCompleteAndFirst1_sequence);

  return offset;
}


static const per_sequence_t T_completeAndFirst1_sequence[] = {
  { &hf_umts_rrc_pdu_def_completeSIB_List, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_CompleteSIB_List },
  { &hf_umts_rrc_pdu_def_firstSegment1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_FirstSegmentShort },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_completeAndFirst1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_completeAndFirst1, T_completeAndFirst1_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_payload1_vals[] = {
  {   0, "noSegment" },
  {   1, "firstSegment" },
  {   2, "subsequentSegment" },
  {   3, "lastSegmentShort" },
  {   4, "lastAndFirst" },
  {   5, "lastAndComplete" },
  {   6, "lastAndCompleteAndFirst" },
  {   7, "completeSIB-List" },
  {   8, "completeAndFirst" },
  {   9, "completeSIB" },
  {  10, "lastSegment" },
  {  11, "spare5" },
  {  12, "spare4" },
  {  13, "spare3" },
  {  14, "spare2" },
  {  15, "spare1" },
  { 0, NULL }
};

static const per_choice_t T_payload1_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_noSegment, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  {   1, &hf_umts_rrc_pdu_def_firstSegment, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_FirstSegment },
  {   2, &hf_umts_rrc_pdu_def_subsequentSegment, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_SubsequentSegment },
  {   3, &hf_umts_rrc_pdu_def_lastSegmentShort, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_LastSegmentShort },
  {   4, &hf_umts_rrc_pdu_def_lastAndFirst1, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_lastAndFirst1 },
  {   5, &hf_umts_rrc_pdu_def_lastAndComplete1, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_lastAndComplete1 },
  {   6, &hf_umts_rrc_pdu_def_lastAndCompleteAndFirst1, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_lastAndCompleteAndFirst1 },
  {   7, &hf_umts_rrc_pdu_def_completeSIB_List, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_CompleteSIB_List },
  {   8, &hf_umts_rrc_pdu_def_completeAndFirst1, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_completeAndFirst1 },
  {   9, &hf_umts_rrc_pdu_def_completeSIB, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_CompleteSIB },
  {  10, &hf_umts_rrc_pdu_def_lastSegment, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_LastSegment },
  {  11, &hf_umts_rrc_pdu_def_spare5, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  {  12, &hf_umts_rrc_pdu_def_spare4, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  {  13, &hf_umts_rrc_pdu_def_spare3, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  {  14, &hf_umts_rrc_pdu_def_spare2, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  {  15, &hf_umts_rrc_pdu_def_spare1, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_payload1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_payload1, T_payload1_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t SystemInformation_FACH_sequence[] = {
  { &hf_umts_rrc_pdu_def_payload1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_payload1 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_SystemInformation_FACH(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_SystemInformation_FACH, SystemInformation_FACH_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions85_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions85(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions85, T_nonCriticalExtensions85_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions50_sequence[] = {
  { &hf_umts_rrc_pdu_def_systemInformationChangeIndication_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions85, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions85 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions50(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions50, T_laterNonCriticalExtensions50_sequence);

  return offset;
}


static const per_sequence_t SystemInformationChangeIndication_sequence[] = {
  { &hf_umts_rrc_pdu_def_bcch_ModificationInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_BCCH_ModificationInfo },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions50, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions50 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_SystemInformationChangeIndication(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_SystemInformationChangeIndication, SystemInformationChangeIndication_sequence);

  return offset;
}


static const per_sequence_t T_fdd33_sequence[] = {
  { &hf_umts_rrc_pdu_def_dummy1_CPCH_SetID, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CPCH_SetID },
  { &hf_umts_rrc_pdu_def_dummy2_DRAC_StaticInformationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DRAC_StaticInformationList },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_fdd33(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_fdd33, T_fdd33_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_transportChannelReconfiguration_r3_IEs_dummy_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_transportChannelReconfiguration_r3_IEs_dummy_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_fdd33, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_fdd33 },
  {   1, &hf_umts_rrc_pdu_def_tdd, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_transportChannelReconfiguration_r3_IEs_dummy(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_transportChannelReconfiguration_r3_IEs_dummy, T_transportChannelReconfiguration_r3_IEs_dummy_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd34_sequence[] = {
  { &hf_umts_rrc_pdu_def_dummy_DL_PDSCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_PDSCH_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_fdd34(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_fdd34, T_fdd34_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_modeSpecificPhysChInfo12_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificPhysChInfo12_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_fdd34, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_fdd34 },
  {   1, &hf_umts_rrc_pdu_def_tdd, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_modeSpecificPhysChInfo12(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo12, T_modeSpecificPhysChInfo12_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t TransportChannelReconfiguration_r3_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_IntegrityProtectionModeInfo },
  { &hf_umts_rrc_pdu_def_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CipheringModeInfo },
  { &hf_umts_rrc_pdu_def_activationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_new_U_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_U_RNTI },
  { &hf_umts_rrc_pdu_def_new_C_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_C_RNTI },
  { &hf_umts_rrc_pdu_def_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_StateIndicator },
  { &hf_umts_rrc_pdu_def_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_umts_rrc_pdu_def_cn_InformationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CN_InformationInfo },
  { &hf_umts_rrc_pdu_def_ura_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_URA_Identity },
  { &hf_umts_rrc_pdu_def_dl_CounterSynchronisationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CounterSynchronisationInfo },
  { &hf_umts_rrc_pdu_def_ul_CommonTransChInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_CommonTransChInfo },
  { &hf_umts_rrc_pdu_def_ul_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_AddReconfTransChInfoList },
  { &hf_umts_rrc_pdu_def_transportChannelReconfiguration_r3_IEs_dummy, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_transportChannelReconfiguration_r3_IEs_dummy },
  { &hf_umts_rrc_pdu_def_dl_CommonTransChInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonTransChInfo },
  { &hf_umts_rrc_pdu_def_dl_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_AddReconfTransChInfoList },
  { &hf_umts_rrc_pdu_def_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_FrequencyInfo },
  { &hf_umts_rrc_pdu_def_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MaxAllowedUL_TX_Power },
  { &hf_umts_rrc_pdu_def_ul_ChannelRequirement, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_ChannelRequirement },
  { &hf_umts_rrc_pdu_def_modeSpecificPhysChInfo12, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_modeSpecificPhysChInfo12 },
  { &hf_umts_rrc_pdu_def_dl_CommonInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonInformation },
  { &hf_umts_rrc_pdu_def_dl_InformationPerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_InformationPerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_TransportChannelReconfiguration_r3_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_TransportChannelReconfiguration_r3_IEs, TransportChannelReconfiguration_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t TransportChannelReconfiguration_v3a0ext_sequence[] = {
  { &hf_umts_rrc_pdu_def_new_DSCH_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DSCH_RNTI },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_TransportChannelReconfiguration_v3a0ext(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_TransportChannelReconfiguration_v3a0ext, TransportChannelReconfiguration_v3a0ext_sequence);

  return offset;
}


static const per_sequence_t TransportChannelReconfiguration_v4b0ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_dummy_SSDT_UL, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_SSDT_UL },
  { &hf_umts_rrc_pdu_def_cell_id_PerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CellIdentity_PerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_TransportChannelReconfiguration_v4b0ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_TransportChannelReconfiguration_v4b0ext_IEs, TransportChannelReconfiguration_v4b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t TransportChannelReconfiguration_v590ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_dl_TPC_PowerOffsetPerRL_List, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_TPC_PowerOffsetPerRL_List },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_TransportChannelReconfiguration_v590ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_TransportChannelReconfiguration_v590ext_IEs, TransportChannelReconfiguration_v590ext_IEs_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_postVerificationPeriod6_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_umts_rrc_pdu_def_T_postVerificationPeriod6(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_timingMaintainedSynchInd4_vals[] = {
  {   0, "false" },
  { 0, NULL }
};


static int
dissect_umts_rrc_pdu_def_T_timingMaintainedSynchInd4(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t TransportChannelReconfiguration_v6xyext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_delayRestrictionFlag, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DelayRestrictionFlag },
  { &hf_umts_rrc_pdu_def_primary_plmn_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_PLMN_Identity },
  { &hf_umts_rrc_pdu_def_harq_Preamble_Mode, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_HARQ_Preamble_Mode },
  { &hf_umts_rrc_pdu_def_beaconPLEst, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_BEACON_PL_Est },
  { &hf_umts_rrc_pdu_def_postVerificationPeriod6, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_postVerificationPeriod6 },
  { &hf_umts_rrc_pdu_def_dhs_sync, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DHS_Sync },
  { &hf_umts_rrc_pdu_def_timingMaintainedSynchInd4, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_timingMaintainedSynchInd4 },
  { &hf_umts_rrc_pdu_def_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MBMS_PL_ServiceRestrictInfo_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_TransportChannelReconfiguration_v6xyext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_TransportChannelReconfiguration_v6xyext_IEs, TransportChannelReconfiguration_v6xyext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions86_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions86(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions86, T_nonCriticalExtensions86_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions33_sequence[] = {
  { &hf_umts_rrc_pdu_def_transportChannelReconfiguration_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_TransportChannelReconfiguration_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions86, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions86 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions33(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions33, T_v6xyNonCriticalExtensions33_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions24_sequence[] = {
  { &hf_umts_rrc_pdu_def_transportChannelReconfiguration_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_TransportChannelReconfiguration_v590ext_IEs },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions33, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions33 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions24(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions24, T_v590NonCriticalExtensions24_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions13_sequence[] = {
  { &hf_umts_rrc_pdu_def_transportChannelReconfiguration_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_TransportChannelReconfiguration_v4b0ext_IEs },
  { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions24, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions24 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions13(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions13, T_v4b0NonCriticalExtensions13_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions51_sequence[] = {
  { &hf_umts_rrc_pdu_def_transportChannelReconfiguration_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions13, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions13 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions51(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions51, T_laterNonCriticalExtensions51_sequence);

  return offset;
}


static const per_sequence_t T_v3a0NonCriticalExtensions9_sequence[] = {
  { &hf_umts_rrc_pdu_def_transportChannelReconfiguration_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_TransportChannelReconfiguration_v3a0ext },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions51, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions51 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions9(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions9, T_v3a0NonCriticalExtensions9_sequence);

  return offset;
}


static const per_sequence_t T_r324_sequence[] = {
  { &hf_umts_rrc_pdu_def_transportChannelReconfiguration_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_TransportChannelReconfiguration_r3_IEs },
  { &hf_umts_rrc_pdu_def_v3a0NonCriticalExtensions9, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions9 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r324(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r324, T_r324_sequence);

  return offset;
}


static const per_sequence_t T_fdd35_sequence[] = {
  { &hf_umts_rrc_pdu_def_dummy1_CPCH_SetID, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CPCH_SetID },
  { &hf_umts_rrc_pdu_def_dummy2_DRAC_StaticInformationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DRAC_StaticInformationList },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_fdd35(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_fdd35, T_fdd35_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_transportChannelReconfiguration_r4_IEs_dummy_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_transportChannelReconfiguration_r4_IEs_dummy_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_fdd35, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_fdd35 },
  {   1, &hf_umts_rrc_pdu_def_tdd, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_transportChannelReconfiguration_r4_IEs_dummy(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_transportChannelReconfiguration_r4_IEs_dummy, T_transportChannelReconfiguration_r4_IEs_dummy_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd36_sequence[] = {
  { &hf_umts_rrc_pdu_def_dummy_DL_PDSCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_PDSCH_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_fdd36(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_fdd36, T_fdd36_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_modeSpecificPhysChInfo13_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificPhysChInfo13_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_fdd36, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_fdd36 },
  {   1, &hf_umts_rrc_pdu_def_tdd, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_modeSpecificPhysChInfo13(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo13, T_modeSpecificPhysChInfo13_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t TransportChannelReconfiguration_r4_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_IntegrityProtectionModeInfo },
  { &hf_umts_rrc_pdu_def_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CipheringModeInfo },
  { &hf_umts_rrc_pdu_def_activationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_new_U_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_U_RNTI },
  { &hf_umts_rrc_pdu_def_new_C_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_C_RNTI },
  { &hf_umts_rrc_pdu_def_new_DSCH_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DSCH_RNTI },
  { &hf_umts_rrc_pdu_def_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_StateIndicator },
  { &hf_umts_rrc_pdu_def_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_umts_rrc_pdu_def_cn_InformationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CN_InformationInfo },
  { &hf_umts_rrc_pdu_def_ura_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_URA_Identity },
  { &hf_umts_rrc_pdu_def_dl_CounterSynchronisationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CounterSynchronisationInfo },
  { &hf_umts_rrc_pdu_def_ul_CommonTransChInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_CommonTransChInfo_r4 },
  { &hf_umts_rrc_pdu_def_ul_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_AddReconfTransChInfoList },
  { &hf_umts_rrc_pdu_def_transportChannelReconfiguration_r4_IEs_dummy, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_transportChannelReconfiguration_r4_IEs_dummy },
  { &hf_umts_rrc_pdu_def_dl_CommonTransChInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonTransChInfo_r4 },
  { &hf_umts_rrc_pdu_def_dl_AddReconfTransChInfoList1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_AddReconfTransChInfoList_r4 },
  { &hf_umts_rrc_pdu_def_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_FrequencyInfo },
  { &hf_umts_rrc_pdu_def_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MaxAllowedUL_TX_Power },
  { &hf_umts_rrc_pdu_def_ul_ChannelRequirement1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_ChannelRequirement_r4 },
  { &hf_umts_rrc_pdu_def_modeSpecificPhysChInfo13, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_modeSpecificPhysChInfo13 },
  { &hf_umts_rrc_pdu_def_dl_CommonInformation1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonInformation_r4 },
  { &hf_umts_rrc_pdu_def_dl_InformationPerRL_List1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_InformationPerRL_List_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_TransportChannelReconfiguration_r4_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_TransportChannelReconfiguration_r4_IEs, TransportChannelReconfiguration_r4_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions87_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions87(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions87, T_nonCriticalExtensions87_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions34_sequence[] = {
  { &hf_umts_rrc_pdu_def_transportChannelReconfiguration_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_TransportChannelReconfiguration_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions87, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions87 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions34(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions34, T_v6xyNonCriticalExtensions34_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions25_sequence[] = {
  { &hf_umts_rrc_pdu_def_transportChannelReconfiguration_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_TransportChannelReconfiguration_v590ext_IEs },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions34, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions34 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions25(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions25, T_v590NonCriticalExtensions25_sequence);

  return offset;
}


static const per_sequence_t T_v4d0NonCriticalExtensions12_sequence[] = {
  { &hf_umts_rrc_pdu_def_transportChannelReconfiguration_r4_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions25, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions25 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions12(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions12, T_v4d0NonCriticalExtensions12_sequence);

  return offset;
}


static const per_sequence_t T_r412_sequence[] = {
  { &hf_umts_rrc_pdu_def_transportChannelReconfiguration_r4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_TransportChannelReconfiguration_r4_IEs },
  { &hf_umts_rrc_pdu_def_v4d0NonCriticalExtensions12, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions12 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r412(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r412, T_r412_sequence);

  return offset;
}


static const per_sequence_t T_fdd37_sequence[] = {
  { &hf_umts_rrc_pdu_def_dummy1_CPCH_SetID, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CPCH_SetID },
  { &hf_umts_rrc_pdu_def_dummy2_DRAC_StaticInformationList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DRAC_StaticInformationList },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_fdd37(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_fdd37, T_fdd37_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_transportChannelReconfiguration_r5_IEs_dummy_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_transportChannelReconfiguration_r5_IEs_dummy_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_fdd37, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_fdd37 },
  {   1, &hf_umts_rrc_pdu_def_tdd, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_transportChannelReconfiguration_r5_IEs_dummy(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_transportChannelReconfiguration_r5_IEs_dummy, T_transportChannelReconfiguration_r5_IEs_dummy_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_fdd38_sequence[] = {
  { &hf_umts_rrc_pdu_def_dummy_DL_PDSCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_PDSCH_Information },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_fdd38(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_fdd38, T_fdd38_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_modeSpecificPhysChInfo14_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificPhysChInfo14_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_fdd38, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_fdd38 },
  {   1, &hf_umts_rrc_pdu_def_tdd, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_modeSpecificPhysChInfo14(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo14, T_modeSpecificPhysChInfo14_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t TransportChannelReconfiguration_r5_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_IntegrityProtectionModeInfo },
  { &hf_umts_rrc_pdu_def_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CipheringModeInfo },
  { &hf_umts_rrc_pdu_def_activationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_new_U_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_U_RNTI },
  { &hf_umts_rrc_pdu_def_new_C_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_C_RNTI },
  { &hf_umts_rrc_pdu_def_new_DSCH_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DSCH_RNTI },
  { &hf_umts_rrc_pdu_def_new_H_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_H_RNTI },
  { &hf_umts_rrc_pdu_def_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_StateIndicator },
  { &hf_umts_rrc_pdu_def_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_umts_rrc_pdu_def_cn_InformationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CN_InformationInfo },
  { &hf_umts_rrc_pdu_def_ura_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_URA_Identity },
  { &hf_umts_rrc_pdu_def_dl_CounterSynchronisationInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CounterSynchronisationInfo_r5 },
  { &hf_umts_rrc_pdu_def_ul_CommonTransChInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_CommonTransChInfo_r4 },
  { &hf_umts_rrc_pdu_def_ul_AddReconfTransChInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_AddReconfTransChInfoList },
  { &hf_umts_rrc_pdu_def_transportChannelReconfiguration_r5_IEs_dummy, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_transportChannelReconfiguration_r5_IEs_dummy },
  { &hf_umts_rrc_pdu_def_dl_CommonTransChInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonTransChInfo_r4 },
  { &hf_umts_rrc_pdu_def_dl_AddReconfTransChInfoList2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_AddReconfTransChInfoList_r5 },
  { &hf_umts_rrc_pdu_def_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_FrequencyInfo },
  { &hf_umts_rrc_pdu_def_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MaxAllowedUL_TX_Power },
  { &hf_umts_rrc_pdu_def_ul_ChannelRequirement2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_ChannelRequirement_r5 },
  { &hf_umts_rrc_pdu_def_modeSpecificPhysChInfo14, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_modeSpecificPhysChInfo14 },
  { &hf_umts_rrc_pdu_def_dl_HSPDSCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_HSPDSCH_Information },
  { &hf_umts_rrc_pdu_def_dl_CommonInformation2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonInformation_r5 },
  { &hf_umts_rrc_pdu_def_dl_InformationPerRL_List2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_InformationPerRL_List_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_TransportChannelReconfiguration_r5_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_TransportChannelReconfiguration_r5_IEs, TransportChannelReconfiguration_r5_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions88_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions88(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions88, T_nonCriticalExtensions88_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions35_sequence[] = {
  { &hf_umts_rrc_pdu_def_transportChannelReconfiguration_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_TransportChannelReconfiguration_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions88, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions88 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions35(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions35, T_v6xyNonCriticalExtensions35_sequence);

  return offset;
}


static const per_sequence_t T_r510_sequence[] = {
  { &hf_umts_rrc_pdu_def_transportChannelReconfiguration_r5, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_TransportChannelReconfiguration_r5_IEs },
  { &hf_umts_rrc_pdu_def_transportChannelReconfiguration_r5_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions35, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions35 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r510(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r510, T_r510_sequence);

  return offset;
}


static const per_sequence_t TransportChannelReconfiguration_r6_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_IntegrityProtectionModeInfo },
  { &hf_umts_rrc_pdu_def_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CipheringModeInfo },
  { &hf_umts_rrc_pdu_def_activationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_delayRestrictionFlag, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DelayRestrictionFlag },
  { &hf_umts_rrc_pdu_def_new_U_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_U_RNTI },
  { &hf_umts_rrc_pdu_def_new_C_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_C_RNTI },
  { &hf_umts_rrc_pdu_def_new_DSCH_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DSCH_RNTI },
  { &hf_umts_rrc_pdu_def_new_H_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_H_RNTI },
  { &hf_umts_rrc_pdu_def_newPrimary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_E_RNTI },
  { &hf_umts_rrc_pdu_def_newSecondary_E_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_E_RNTI },
  { &hf_umts_rrc_pdu_def_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_StateIndicator },
  { &hf_umts_rrc_pdu_def_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_umts_rrc_pdu_def_cn_InformationInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CN_InformationInfo_r6 },
  { &hf_umts_rrc_pdu_def_ura_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_URA_Identity },
  { &hf_umts_rrc_pdu_def_dl_CounterSynchronisationInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CounterSynchronisationInfo_r5 },
  { &hf_umts_rrc_pdu_def_ul_CommonTransChInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_CommonTransChInfo_r4 },
  { &hf_umts_rrc_pdu_def_ul_AddReconfTransChInfoList1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_AddReconfTransChInfoList_r6 },
  { &hf_umts_rrc_pdu_def_dl_CommonTransChInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonTransChInfo_r4 },
  { &hf_umts_rrc_pdu_def_dl_AddReconfTransChInfoList2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_AddReconfTransChInfoList_r5 },
  { &hf_umts_rrc_pdu_def_frequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_FrequencyInfo },
  { &hf_umts_rrc_pdu_def_maxAllowedUL_TX_Power, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MaxAllowedUL_TX_Power },
  { &hf_umts_rrc_pdu_def_ul_ChannelRequirement3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_ChannelRequirement_r6 },
  { &hf_umts_rrc_pdu_def_ul_EDCH_Information, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_EDCH_Information_r6 },
  { &hf_umts_rrc_pdu_def_dl_HSPDSCH_Information1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_HSPDSCH_Information_r6 },
  { &hf_umts_rrc_pdu_def_dl_CommonInformation3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CommonInformation_r6 },
  { &hf_umts_rrc_pdu_def_dl_InformationPerRL_List3, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_InformationPerRL_List_r6 },
  { &hf_umts_rrc_pdu_def_mbms_PL_ServiceRestrictInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MBMS_PL_ServiceRestrictInfo_r6 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_TransportChannelReconfiguration_r6_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_TransportChannelReconfiguration_r6_IEs, TransportChannelReconfiguration_r6_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions89_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions89(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions89, T_nonCriticalExtensions89_sequence);

  return offset;
}


static const per_sequence_t T_r610_sequence[] = {
  { &hf_umts_rrc_pdu_def_transportChannelReconfiguration_r6, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_TransportChannelReconfiguration_r6_IEs },
  { &hf_umts_rrc_pdu_def_transportChannelReconfiguration_r6_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions89, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions89 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r610(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r610, T_r610_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions60_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions60(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_criticalExtensions60, T_criticalExtensions60_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_criticalExtensions59_vals[] = {
  {   0, "r6" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions59_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r610, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r610 },
  {   1, &hf_umts_rrc_pdu_def_criticalExtensions60, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_criticalExtensions60 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions59(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_criticalExtensions59, T_criticalExtensions59_choice,
                                 NULL);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_criticalExtensions58_vals[] = {
  {   0, "r5" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions58_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r510, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r510 },
  {   1, &hf_umts_rrc_pdu_def_criticalExtensions59, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_criticalExtensions59 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions58(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_criticalExtensions58, T_criticalExtensions58_choice,
                                 NULL);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_criticalExtensions57_vals[] = {
  {   0, "r4" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions57_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r412, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r412 },
  {   1, &hf_umts_rrc_pdu_def_criticalExtensions58, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_criticalExtensions58 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions57(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_criticalExtensions57, T_criticalExtensions57_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_later_than_r321_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_criticalExtensions57, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_criticalExtensions57 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_later_than_r321(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_later_than_r321, T_later_than_r321_sequence);

  return offset;
}


const value_string umts_rrc_pdu_def_TransportChannelReconfiguration_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t TransportChannelReconfiguration_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r324, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r324 },
  {   1, &hf_umts_rrc_pdu_def_later_than_r321, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_later_than_r321 },
  { 0, NULL, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_TransportChannelReconfiguration(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_TransportChannelReconfiguration, TransportChannelReconfiguration_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions90_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions90(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions90, T_nonCriticalExtensions90_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions52_sequence[] = {
  { &hf_umts_rrc_pdu_def_transportChannelReconfigurationComplete_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions90, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions90 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions52(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions52, T_laterNonCriticalExtensions52_sequence);

  return offset;
}


static const per_sequence_t TransportChannelReconfigurationComplete_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_ul_IntegProtActivationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_IntegrityProtActivationInfo },
  { &hf_umts_rrc_pdu_def_ul_TimingAdvance, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_TimingAdvance },
  { &hf_umts_rrc_pdu_def_count_C_ActivationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_dummy_RB_ActivationTimeInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_ActivationTimeInfoList },
  { &hf_umts_rrc_pdu_def_ul_CounterSynchronisationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_CounterSynchronisationInfo },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions52, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions52 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_TransportChannelReconfigurationComplete(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_TransportChannelReconfigurationComplete, TransportChannelReconfigurationComplete_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions91_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions91(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions91, T_nonCriticalExtensions91_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions53_sequence[] = {
  { &hf_umts_rrc_pdu_def_transportChannelReconfigurationFailure_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions91, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions91 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions53(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions53, T_laterNonCriticalExtensions53_sequence);

  return offset;
}


static const per_sequence_t TransportChannelReconfigurationFailure_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_failureCause, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_FailureCauseWithProtErr },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions53, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions53 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_TransportChannelReconfigurationFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_TransportChannelReconfigurationFailure, TransportChannelReconfigurationFailure_sequence);

  return offset;
}


static const per_sequence_t T_tdd3_sequence[] = {
  { &hf_umts_rrc_pdu_def_tfcs_ID, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_TFCS_Identity },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_tdd3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_tdd3, T_tdd3_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_modeSpecificInfo9_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_modeSpecificInfo9_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_fdd39, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_NULL },
  {   1, &hf_umts_rrc_pdu_def_tdd4, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_tdd3 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_modeSpecificInfo9(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_modeSpecificInfo9, T_modeSpecificInfo9_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions92_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions92(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions92, T_nonCriticalExtensions92_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions54_sequence[] = {
  { &hf_umts_rrc_pdu_def_transportFormatCombinationControl_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions92, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions92 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions54(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions54, T_laterNonCriticalExtensions54_sequence);

  return offset;
}


static const per_sequence_t TransportFormatCombinationControl_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_modeSpecificInfo9, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_modeSpecificInfo9 },
  { &hf_umts_rrc_pdu_def_dpch_TFCS_InUplink, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_TFC_Subset },
  { &hf_umts_rrc_pdu_def_activationTimeForTFCSubset, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_tfc_ControlDuration, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_TFC_ControlDuration },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions54, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions54 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_TransportFormatCombinationControl(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_TransportFormatCombinationControl, TransportFormatCombinationControl_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions93_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions93(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions93, T_nonCriticalExtensions93_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions55_sequence[] = {
  { &hf_umts_rrc_pdu_def_transportFormatCombinationControlFailure_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions93, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions93 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions55(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions55, T_laterNonCriticalExtensions55_sequence);

  return offset;
}


static const per_sequence_t TransportFormatCombinationControlFailure_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_failureCause, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_FailureCauseWithProtErr },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions55, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions55 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_TransportFormatCombinationControlFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_TransportFormatCombinationControlFailure, TransportFormatCombinationControlFailure_sequence);

  return offset;
}


static const per_sequence_t UECapabilityEnquiry_r3_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_capabilityUpdateRequirement, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_CapabilityUpdateRequirement },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_UECapabilityEnquiry_r3_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_UECapabilityEnquiry_r3_IEs, UECapabilityEnquiry_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t UECapabilityEnquiry_v4b0ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_capabilityUpdateRequirement_r4_ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_CapabilityUpdateRequirement_r4_ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_UECapabilityEnquiry_v4b0ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_UECapabilityEnquiry_v4b0ext_IEs, UECapabilityEnquiry_v4b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t UECapabilityEnquiry_v590ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_systemSpecificCapUpdateReq, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_SystemSpecificCapUpdateReq_v590ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_UECapabilityEnquiry_v590ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_UECapabilityEnquiry_v590ext_IEs, UECapabilityEnquiry_v590ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions94_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions94(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions94, T_nonCriticalExtensions94_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions26_sequence[] = {
  { &hf_umts_rrc_pdu_def_ueCapabilityEnquiry_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_UECapabilityEnquiry_v590ext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions94, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions94 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions26(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions26, T_v590NonCriticalExtensions26_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions14_sequence[] = {
  { &hf_umts_rrc_pdu_def_ueCapabilityEnquiry_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_UECapabilityEnquiry_v4b0ext_IEs },
  { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions26, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions26 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions14(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions14, T_v4b0NonCriticalExtensions14_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions56_sequence[] = {
  { &hf_umts_rrc_pdu_def_ueCapabilityEnquiry_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions14, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions14 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions56(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions56, T_laterNonCriticalExtensions56_sequence);

  return offset;
}


static const per_sequence_t T_r325_sequence[] = {
  { &hf_umts_rrc_pdu_def_ueCapabilityEnquiry_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_UECapabilityEnquiry_r3_IEs },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions56, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions56 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r325(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r325, T_r325_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions61_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions61(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_criticalExtensions61, T_criticalExtensions61_sequence);

  return offset;
}


static const per_sequence_t T_later_than_r322_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_criticalExtensions61, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_criticalExtensions61 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_later_than_r322(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_later_than_r322, T_later_than_r322_sequence);

  return offset;
}


const value_string umts_rrc_pdu_def_UECapabilityEnquiry_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t UECapabilityEnquiry_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r325, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r325 },
  {   1, &hf_umts_rrc_pdu_def_later_than_r322, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_later_than_r322 },
  { 0, NULL, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_UECapabilityEnquiry(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_UECapabilityEnquiry, UECapabilityEnquiry_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UECapabilityInformation_v370ext_sequence[] = {
  { &hf_umts_rrc_pdu_def_ue_RadioAccessCapability_v370ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UE_RadioAccessCapability_v370ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_UECapabilityInformation_v370ext(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_UECapabilityInformation_v370ext, UECapabilityInformation_v370ext_sequence);

  return offset;
}


static const per_sequence_t UECapabilityInformation_v380ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_ue_RadioAccessCapability_v380ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UE_RadioAccessCapability_v380ext },
  { &hf_umts_rrc_pdu_def_dl_PhysChCapabilityFDD_v380ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_DL_PhysChCapabilityFDD_v380ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_UECapabilityInformation_v380ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_UECapabilityInformation_v380ext_IEs, UECapabilityInformation_v380ext_IEs_sequence);

  return offset;
}


static const per_sequence_t UECapabilityInformation_v3a0ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_ue_RadioAccessCapability_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UE_RadioAccessCapability_v3a0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_UECapabilityInformation_v3a0ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_UECapabilityInformation_v3a0ext_IEs, UECapabilityInformation_v3a0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t UECapabilityInformation_v4b0ext_sequence[] = {
  { &hf_umts_rrc_pdu_def_ue_RadioAccessCapability_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UE_RadioAccessCapability_v4b0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_UECapabilityInformation_v4b0ext(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_UECapabilityInformation_v4b0ext, UECapabilityInformation_v4b0ext_sequence);

  return offset;
}


static const per_sequence_t UECapabilityInformation_v590ext_sequence[] = {
  { &hf_umts_rrc_pdu_def_ue_RadioAccessCapability_v3g0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UE_RadioAccessCapability_v3g0ext },
  { &hf_umts_rrc_pdu_def_ue_RadioAccessCapability_v590ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UE_RadioAccessCapability_v590ext },
  { &hf_umts_rrc_pdu_def_ue_RATSpecificCapability_v590ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_InterRAT_UE_RadioAccessCapability_v590ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_UECapabilityInformation_v590ext(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_UECapabilityInformation_v590ext, UECapabilityInformation_v590ext_sequence);

  return offset;
}


static const per_sequence_t UECapabilityInformation_v5c0ext_sequence[] = {
  { &hf_umts_rrc_pdu_def_ue_RadioAccessCapability_v5c0ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UE_RadioAccessCapability_v5c0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_UECapabilityInformation_v5c0ext(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_UECapabilityInformation_v5c0ext, UECapabilityInformation_v5c0ext_sequence);

  return offset;
}


static const per_sequence_t UECapabilityInformation_v6xyext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_ueCapabilityContainer, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_UECapabilityInformation_v6xyext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_UECapabilityInformation_v6xyext_IEs, UECapabilityInformation_v6xyext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions95_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions95(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions95, T_nonCriticalExtensions95_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions36_sequence[] = {
  { &hf_umts_rrc_pdu_def_ueCapabilityInformation_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_UECapabilityInformation_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions95, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions95 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions36(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions36, T_v6xyNonCriticalExtensions36_sequence);

  return offset;
}


static const per_sequence_t T_v5c0NonCriticalExtensions1_sequence[] = {
  { &hf_umts_rrc_pdu_def_ueCapabilityInformation_v5c0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_UECapabilityInformation_v5c0ext },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions36, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions36 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v5c0NonCriticalExtensions1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v5c0NonCriticalExtensions1, T_v5c0NonCriticalExtensions1_sequence);

  return offset;
}


static const per_sequence_t T_v590NonCriticalExtensions27_sequence[] = {
  { &hf_umts_rrc_pdu_def_ueCapabilityInformation_v590ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_UECapabilityInformation_v590ext },
  { &hf_umts_rrc_pdu_def_v5c0NonCriticalExtensions1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v5c0NonCriticalExtensions1 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions27(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions27, T_v590NonCriticalExtensions27_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions15_sequence[] = {
  { &hf_umts_rrc_pdu_def_ueCapabilityInformation_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_UECapabilityInformation_v4b0ext },
  { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions27, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v590NonCriticalExtensions27 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions15(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions15, T_v4b0NonCriticalExtensions15_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions57_sequence[] = {
  { &hf_umts_rrc_pdu_def_ueCapabilityInformation_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions15, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions15 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions57(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions57, T_laterNonCriticalExtensions57_sequence);

  return offset;
}


static const per_sequence_t T_v3a0NonCriticalExtensions10_sequence[] = {
  { &hf_umts_rrc_pdu_def_ueCapabilityInformation_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_UECapabilityInformation_v3a0ext_IEs },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions57, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions57 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions10(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions10, T_v3a0NonCriticalExtensions10_sequence);

  return offset;
}


static const per_sequence_t T_v380NonCriticalExtensions1_sequence[] = {
  { &hf_umts_rrc_pdu_def_ueCapabilityInformation_v380ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_UECapabilityInformation_v380ext_IEs },
  { &hf_umts_rrc_pdu_def_v3a0NonCriticalExtensions10, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions10 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v380NonCriticalExtensions1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v380NonCriticalExtensions1, T_v380NonCriticalExtensions1_sequence);

  return offset;
}


static const per_sequence_t T_v370NonCriticalExtensions1_sequence[] = {
  { &hf_umts_rrc_pdu_def_ueCapabilityInformation_v370ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_UECapabilityInformation_v370ext },
  { &hf_umts_rrc_pdu_def_v380NonCriticalExtensions1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v380NonCriticalExtensions1 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v370NonCriticalExtensions1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v370NonCriticalExtensions1, T_v370NonCriticalExtensions1_sequence);

  return offset;
}


static const per_sequence_t UECapabilityInformation_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_ue_RadioAccessCapability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UE_RadioAccessCapability },
  { &hf_umts_rrc_pdu_def_ue_RATSpecificCapability, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_InterRAT_UE_RadioAccessCapabilityList },
  { &hf_umts_rrc_pdu_def_v370NonCriticalExtensions1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v370NonCriticalExtensions1 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_UECapabilityInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_UECapabilityInformation, UECapabilityInformation_sequence);

  return offset;
}


static const per_sequence_t UECapabilityInformation_v650ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_ue_RadioAccessCapability_v650ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_UE_RadioAccessCapability_v650ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_UECapabilityInformation_v650ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_UECapabilityInformation_v650ext_IEs, UECapabilityInformation_v650ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions96_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions96(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions96, T_nonCriticalExtensions96_sequence);

  return offset;
}


static const per_sequence_t UECapabilityInformation_r3_add_ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_ueCapabilityInformation_v650ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_UECapabilityInformation_v650ext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions96, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions96 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_UECapabilityInformation_r3_add_ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_UECapabilityInformation_r3_add_ext_IEs, UECapabilityInformation_r3_add_ext_IEs_sequence);

  return offset;
}


static const per_sequence_t UECapabilityInformationConfirm_r3_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_UECapabilityInformationConfirm_r3_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_UECapabilityInformationConfirm_r3_IEs, UECapabilityInformationConfirm_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions97_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions97(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions97, T_nonCriticalExtensions97_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions58_sequence[] = {
  { &hf_umts_rrc_pdu_def_ueCapabilityInformationConfirm_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions97, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions97 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions58(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions58, T_laterNonCriticalExtensions58_sequence);

  return offset;
}


static const per_sequence_t T_r326_sequence[] = {
  { &hf_umts_rrc_pdu_def_ueCapabilityInformationConfirm_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_UECapabilityInformationConfirm_r3_IEs },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions58, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions58 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r326(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r326, T_r326_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions62_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions62(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_criticalExtensions62, T_criticalExtensions62_sequence);

  return offset;
}


static const per_sequence_t T_later_than_r323_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_criticalExtensions62, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_criticalExtensions62 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_later_than_r323(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_later_than_r323, T_later_than_r323_sequence);

  return offset;
}


const value_string umts_rrc_pdu_def_UECapabilityInformationConfirm_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t UECapabilityInformationConfirm_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r326, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r326 },
  {   1, &hf_umts_rrc_pdu_def_later_than_r323, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_later_than_r323 },
  { 0, NULL, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_UECapabilityInformationConfirm(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_UECapabilityInformationConfirm, UECapabilityInformationConfirm_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UplinkDirectTransfer_v6xyext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_measuredResultsOnRACHinterFreq, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MeasuredResultsOnRACHinterFreq },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_UplinkDirectTransfer_v6xyext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_UplinkDirectTransfer_v6xyext_IEs, UplinkDirectTransfer_v6xyext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions98_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions98(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions98, T_nonCriticalExtensions98_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions37_sequence[] = {
  { &hf_umts_rrc_pdu_def_uplinkDirectTransfer_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_UplinkDirectTransfer_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions98, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions98 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions37(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions37, T_v6xyNonCriticalExtensions37_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions59_sequence[] = {
  { &hf_umts_rrc_pdu_def_uplinkDirectTransfer_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions37, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions37 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions59(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions59, T_laterNonCriticalExtensions59_sequence);

  return offset;
}


static const per_sequence_t UplinkDirectTransfer_sequence[] = {
  { &hf_umts_rrc_pdu_def_cn_DomainIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_CN_DomainIdentity },
  { &hf_umts_rrc_pdu_def_nas_Message, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_NAS_Message },
  { &hf_umts_rrc_pdu_def_measuredResultsOnRACH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MeasuredResultsOnRACH },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions59, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions59 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_UplinkDirectTransfer(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_UplinkDirectTransfer, UplinkDirectTransfer_sequence);

  return offset;
}


static const per_sequence_t UplinkPhysicalChannelControl_r3_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_ccTrCH_PowerControlInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CCTrCH_PowerControlInfo },
  { &hf_umts_rrc_pdu_def_timingAdvance, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_TimingAdvanceControl },
  { &hf_umts_rrc_pdu_def_alpha, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_Alpha },
  { &hf_umts_rrc_pdu_def_specialBurstScheduling, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_SpecialBurstScheduling },
  { &hf_umts_rrc_pdu_def_prach_ConstantValue, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ConstantValueTdd },
  { &hf_umts_rrc_pdu_def_pusch_ConstantValue, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ConstantValueTdd },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_UplinkPhysicalChannelControl_r3_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_UplinkPhysicalChannelControl_r3_IEs, UplinkPhysicalChannelControl_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t UplinkPhysicalChannelControl_v4b0ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_openLoopPowerControl_IPDL_TDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_OpenLoopPowerControl_IPDL_TDD_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_UplinkPhysicalChannelControl_v4b0ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_UplinkPhysicalChannelControl_v4b0ext_IEs, UplinkPhysicalChannelControl_v4b0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_noncriticalExtensions_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_noncriticalExtensions(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_noncriticalExtensions, T_noncriticalExtensions_sequence);

  return offset;
}


static const per_sequence_t T_v4b0NonCriticalExtensions16_sequence[] = {
  { &hf_umts_rrc_pdu_def_uplinkPysicalChannelControl_v4b0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_UplinkPhysicalChannelControl_v4b0ext_IEs },
  { &hf_umts_rrc_pdu_def_noncriticalExtensions, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_noncriticalExtensions },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions16(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions16, T_v4b0NonCriticalExtensions16_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions60_sequence[] = {
  { &hf_umts_rrc_pdu_def_uplinkPhysicalChannelControl_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions16, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions16 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions60(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions60, T_laterNonCriticalExtensions60_sequence);

  return offset;
}


static const per_sequence_t T_r327_sequence[] = {
  { &hf_umts_rrc_pdu_def_uplinkPhysicalChannelControl_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_UplinkPhysicalChannelControl_r3_IEs },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions60, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions60 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r327(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r327, T_r327_sequence);

  return offset;
}


static const per_sequence_t T_tdd3842_sequence[] = {
  { &hf_umts_rrc_pdu_def_timingAdvance1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_TimingAdvanceControl_r4 },
  { &hf_umts_rrc_pdu_def_alpha, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_Alpha },
  { &hf_umts_rrc_pdu_def_prach_ConstantValue, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ConstantValueTdd },
  { &hf_umts_rrc_pdu_def_pusch_ConstantValue, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ConstantValueTdd },
  { &hf_umts_rrc_pdu_def_openLoopPowerControl_IPDL_TDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_OpenLoopPowerControl_IPDL_TDD_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_tdd3842(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_tdd3842, T_tdd3842_sequence);

  return offset;
}


static const per_sequence_t T_tdd1282_sequence[] = {
  { &hf_umts_rrc_pdu_def_ul_SynchronisationParameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_SynchronisationParameters_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_tdd1282(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_tdd1282, T_tdd1282_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_tddOption_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tddOption_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_tdd3842, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_tdd3842 },
  {   1, &hf_umts_rrc_pdu_def_tdd1282, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_tdd1282 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_tddOption(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_tddOption, T_tddOption_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UplinkPhysicalChannelControl_r4_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_ccTrCH_PowerControlInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CCTrCH_PowerControlInfo_r4 },
  { &hf_umts_rrc_pdu_def_specialBurstScheduling, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_SpecialBurstScheduling },
  { &hf_umts_rrc_pdu_def_tddOption, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_tddOption },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_UplinkPhysicalChannelControl_r4_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_UplinkPhysicalChannelControl_r4_IEs, UplinkPhysicalChannelControl_r4_IEs_sequence);

  return offset;
}


static const per_sequence_t UplinkPhysicalChannelControl_v6xyext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_beaconPLEst, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_BEACON_PL_Est },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_UplinkPhysicalChannelControl_v6xyext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_UplinkPhysicalChannelControl_v6xyext_IEs, UplinkPhysicalChannelControl_v6xyext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions99_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions99(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions99, T_nonCriticalExtensions99_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions38_sequence[] = {
  { &hf_umts_rrc_pdu_def_uplinkPhysicalChannelControl_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_UplinkPhysicalChannelControl_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions99, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions99 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions38(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions38, T_v6xyNonCriticalExtensions38_sequence);

  return offset;
}


static const per_sequence_t T_v4d0NonCriticalExtensions13_sequence[] = {
  { &hf_umts_rrc_pdu_def_uplinkPhysicalChannelControl_r4_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions38, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions38 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions13(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions13, T_v4d0NonCriticalExtensions13_sequence);

  return offset;
}


static const per_sequence_t T_r413_sequence[] = {
  { &hf_umts_rrc_pdu_def_uplinkPhysicalChannelControl_r4, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_UplinkPhysicalChannelControl_r4_IEs },
  { &hf_umts_rrc_pdu_def_v4d0NonCriticalExtensions13, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions13 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r413(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r413, T_r413_sequence);

  return offset;
}


static const per_sequence_t T_tdd3843_sequence[] = {
  { &hf_umts_rrc_pdu_def_timingAdvance1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_TimingAdvanceControl_r4 },
  { &hf_umts_rrc_pdu_def_alpha, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_Alpha },
  { &hf_umts_rrc_pdu_def_prach_ConstantValue, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ConstantValueTdd },
  { &hf_umts_rrc_pdu_def_pusch_ConstantValue, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ConstantValueTdd },
  { &hf_umts_rrc_pdu_def_openLoopPowerControl_IPDL_TDD, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_OpenLoopPowerControl_IPDL_TDD_r4 },
  { &hf_umts_rrc_pdu_def_hs_SICH_PowerControl, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_HS_SICH_Power_Control_Info_TDD384 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_tdd3843(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_tdd3843, T_tdd3843_sequence);

  return offset;
}


static const per_sequence_t T_tdd1283_sequence[] = {
  { &hf_umts_rrc_pdu_def_ul_SynchronisationParameters, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_SynchronisationParameters_r4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_tdd1283(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_tdd1283, T_tdd1283_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_tddOption1_vals[] = {
  {   0, "tdd384" },
  {   1, "tdd128" },
  { 0, NULL }
};

static const per_choice_t T_tddOption1_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_tdd3843, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_tdd3843 },
  {   1, &hf_umts_rrc_pdu_def_tdd1283, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_tdd1283 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_tddOption1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_tddOption1, T_tddOption1_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t UplinkPhysicalChannelControl_r5_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_ccTrCH_PowerControlInfo2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CCTrCH_PowerControlInfo_r5 },
  { &hf_umts_rrc_pdu_def_specialBurstScheduling, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_SpecialBurstScheduling },
  { &hf_umts_rrc_pdu_def_tddOption1, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_tddOption1 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_UplinkPhysicalChannelControl_r5_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_UplinkPhysicalChannelControl_r5_IEs, UplinkPhysicalChannelControl_r5_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions100_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions100(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions100, T_nonCriticalExtensions100_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions39_sequence[] = {
  { &hf_umts_rrc_pdu_def_uplinkPhysicalChannelControl_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_UplinkPhysicalChannelControl_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions100, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions100 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions39(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions39, T_v6xyNonCriticalExtensions39_sequence);

  return offset;
}


static const per_sequence_t T_r511_sequence[] = {
  { &hf_umts_rrc_pdu_def_uplinkPhysicalChannelControl_r5, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_UplinkPhysicalChannelControl_r5_IEs },
  { &hf_umts_rrc_pdu_def_uplinkPhysicalChannelControl_r5_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions39, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions39 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r511(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r511, T_r511_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions65_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions65(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_criticalExtensions65, T_criticalExtensions65_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_criticalExtensions64_vals[] = {
  {   0, "r5" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions64_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r511, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r511 },
  {   1, &hf_umts_rrc_pdu_def_criticalExtensions65, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_criticalExtensions65 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions64(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_criticalExtensions64, T_criticalExtensions64_choice,
                                 NULL);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_criticalExtensions63_vals[] = {
  {   0, "r4" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions63_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r413, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r413 },
  {   1, &hf_umts_rrc_pdu_def_criticalExtensions64, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_criticalExtensions64 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions63(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_criticalExtensions63, T_criticalExtensions63_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_later_than_r324_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_criticalExtensions63, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_criticalExtensions63 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_later_than_r324(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_later_than_r324, T_later_than_r324_sequence);

  return offset;
}


const value_string umts_rrc_pdu_def_UplinkPhysicalChannelControl_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t UplinkPhysicalChannelControl_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r327, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r327 },
  {   1, &hf_umts_rrc_pdu_def_later_than_r324, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_later_than_r324 },
  { 0, NULL, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_UplinkPhysicalChannelControl(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_UplinkPhysicalChannelControl, UplinkPhysicalChannelControl_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions101_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions101(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions101, T_nonCriticalExtensions101_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions61_sequence[] = {
  { &hf_umts_rrc_pdu_def_uraUpdate_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions101, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions101 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions61(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions61, T_laterNonCriticalExtensions61_sequence);

  return offset;
}


static const per_sequence_t URAUpdate_sequence[] = {
  { &hf_umts_rrc_pdu_def_u_RNTI, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_U_RNTI },
  { &hf_umts_rrc_pdu_def_ura_UpdateCause, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_URA_UpdateCause },
  { &hf_umts_rrc_pdu_def_protocolErrorIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_ProtocolErrorIndicatorWithMoreInfo },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions61, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions61 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_URAUpdate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_URAUpdate, URAUpdate_sequence);

  return offset;
}


static const per_sequence_t URAUpdateConfirm_r3_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_IntegrityProtectionModeInfo },
  { &hf_umts_rrc_pdu_def_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CipheringModeInfo },
  { &hf_umts_rrc_pdu_def_new_U_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_U_RNTI },
  { &hf_umts_rrc_pdu_def_new_C_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_C_RNTI },
  { &hf_umts_rrc_pdu_def_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_StateIndicator },
  { &hf_umts_rrc_pdu_def_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_umts_rrc_pdu_def_cn_InformationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CN_InformationInfo },
  { &hf_umts_rrc_pdu_def_ura_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_URA_Identity },
  { &hf_umts_rrc_pdu_def_dl_CounterSynchronisationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CounterSynchronisationInfo },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_URAUpdateConfirm_r3_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_URAUpdateConfirm_r3_IEs, URAUpdateConfirm_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t URAUpdateConfirm_v6xyext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_primary_plmn_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_PLMN_Identity },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_URAUpdateConfirm_v6xyext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_URAUpdateConfirm_v6xyext_IEs, URAUpdateConfirm_v6xyext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions102_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions102(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions102, T_nonCriticalExtensions102_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions40_sequence[] = {
  { &hf_umts_rrc_pdu_def_uraUpdateConfirm_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_URAUpdateConfirm_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions102, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions102 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions40(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions40, T_v6xyNonCriticalExtensions40_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions62_sequence[] = {
  { &hf_umts_rrc_pdu_def_uraUpdateConfirm_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions40, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions40 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions62(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions62, T_laterNonCriticalExtensions62_sequence);

  return offset;
}


static const per_sequence_t T_r328_sequence[] = {
  { &hf_umts_rrc_pdu_def_uraUpdateConfirm_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_URAUpdateConfirm_r3_IEs },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions62, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions62 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r328(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r328, T_r328_sequence);

  return offset;
}


static const per_sequence_t URAUpdateConfirm_r5_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_IntegrityProtectionModeInfo },
  { &hf_umts_rrc_pdu_def_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CipheringModeInfo },
  { &hf_umts_rrc_pdu_def_new_U_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_U_RNTI },
  { &hf_umts_rrc_pdu_def_new_C_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_C_RNTI },
  { &hf_umts_rrc_pdu_def_rrc_StateIndicator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_StateIndicator },
  { &hf_umts_rrc_pdu_def_utran_DRX_CycleLengthCoeff, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UTRAN_DRX_CycleLengthCoefficient },
  { &hf_umts_rrc_pdu_def_cn_InformationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CN_InformationInfo },
  { &hf_umts_rrc_pdu_def_ura_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_URA_Identity },
  { &hf_umts_rrc_pdu_def_dl_CounterSynchronisationInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CounterSynchronisationInfo_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_URAUpdateConfirm_r5_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_URAUpdateConfirm_r5_IEs, URAUpdateConfirm_r5_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions103_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions103(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions103, T_nonCriticalExtensions103_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions41_sequence[] = {
  { &hf_umts_rrc_pdu_def_uraUpdateConfirm_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_URAUpdateConfirm_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions103, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions103 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions41(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions41, T_v6xyNonCriticalExtensions41_sequence);

  return offset;
}


static const per_sequence_t T_r512_sequence[] = {
  { &hf_umts_rrc_pdu_def_uraUpdateConfirm_r5, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_URAUpdateConfirm_r5_IEs },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions41, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions41 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r512(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r512, T_r512_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions67_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions67(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_criticalExtensions67, T_criticalExtensions67_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_criticalExtensions66_vals[] = {
  {   0, "r5" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions66_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r512, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r512 },
  {   1, &hf_umts_rrc_pdu_def_criticalExtensions67, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_criticalExtensions67 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions66(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_criticalExtensions66, T_criticalExtensions66_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_later_than_r325_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_criticalExtensions66, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_criticalExtensions66 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_later_than_r325(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_later_than_r325, T_later_than_r325_sequence);

  return offset;
}


const value_string umts_rrc_pdu_def_URAUpdateConfirm_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t URAUpdateConfirm_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r328, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r328 },
  {   1, &hf_umts_rrc_pdu_def_later_than_r325, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_later_than_r325 },
  { 0, NULL, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_URAUpdateConfirm(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_URAUpdateConfirm, URAUpdateConfirm_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t URAUpdateConfirm_CCCH_r3_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_u_RNTI, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_U_RNTI },
  { &hf_umts_rrc_pdu_def_uraUpdateConfirm, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_URAUpdateConfirm_r3_IEs },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_URAUpdateConfirm_CCCH_r3_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_URAUpdateConfirm_CCCH_r3_IEs, URAUpdateConfirm_CCCH_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions104_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions104(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions104, T_nonCriticalExtensions104_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions42_sequence[] = {
  { &hf_umts_rrc_pdu_def_uraUpdateConfirm_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_URAUpdateConfirm_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions104, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions104 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions42(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions42, T_v6xyNonCriticalExtensions42_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions63_sequence[] = {
  { &hf_umts_rrc_pdu_def_uraUpdateConfirm_CCCH_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions42, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions42 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions63(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions63, T_laterNonCriticalExtensions63_sequence);

  return offset;
}


static const per_sequence_t T_r329_sequence[] = {
  { &hf_umts_rrc_pdu_def_uraUpdateConfirm_CCCH_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_URAUpdateConfirm_CCCH_r3_IEs },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions63, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions63 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r329(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r329, T_r329_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions68_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions68(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_criticalExtensions68, T_criticalExtensions68_sequence);

  return offset;
}


static const per_sequence_t T_later_than_r326_sequence[] = {
  { &hf_umts_rrc_pdu_def_u_RNTI, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_U_RNTI },
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_criticalExtensions68, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_criticalExtensions68 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_later_than_r326(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_later_than_r326, T_later_than_r326_sequence);

  return offset;
}


const value_string umts_rrc_pdu_def_URAUpdateConfirm_CCCH_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t URAUpdateConfirm_CCCH_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r329, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r329 },
  {   1, &hf_umts_rrc_pdu_def_later_than_r326, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_later_than_r326 },
  { 0, NULL, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_URAUpdateConfirm_CCCH(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_URAUpdateConfirm_CCCH, URAUpdateConfirm_CCCH_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions107_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions107(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions107, T_nonCriticalExtensions107_sequence);

  return offset;
}


static const per_sequence_t UTRANMobilityInformation_r3_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_IntegrityProtectionModeInfo },
  { &hf_umts_rrc_pdu_def_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CipheringModeInfo },
  { &hf_umts_rrc_pdu_def_new_U_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_U_RNTI },
  { &hf_umts_rrc_pdu_def_new_C_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_C_RNTI },
  { &hf_umts_rrc_pdu_def_ue_ConnTimersAndConstants, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UE_ConnTimersAndConstants },
  { &hf_umts_rrc_pdu_def_cn_InformationInfo2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CN_InformationInfoFull },
  { &hf_umts_rrc_pdu_def_ura_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_URA_Identity },
  { &hf_umts_rrc_pdu_def_dl_CounterSynchronisationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CounterSynchronisationInfo },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions107, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions107 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_UTRANMobilityInformation_r3_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_UTRANMobilityInformation_r3_IEs, UTRANMobilityInformation_r3_IEs_sequence);

  return offset;
}


static const per_sequence_t UTRANMobilityInformation_v3a0ext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_ue_ConnTimersAndConstants_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_UE_ConnTimersAndConstants_v3a0ext },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_UTRANMobilityInformation_v3a0ext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_UTRANMobilityInformation_v3a0ext_IEs, UTRANMobilityInformation_v3a0ext_IEs_sequence);

  return offset;
}


static const per_sequence_t UtranMobilityInformation_v6xyext_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_primary_plmn_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_PLMN_Identity },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_UtranMobilityInformation_v6xyext_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_UtranMobilityInformation_v6xyext_IEs, UtranMobilityInformation_v6xyext_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions105_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions105(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions105, T_nonCriticalExtensions105_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions43_sequence[] = {
  { &hf_umts_rrc_pdu_def_utranMobilityInformation_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_UtranMobilityInformation_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions105, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions105 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions43(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions43, T_v6xyNonCriticalExtensions43_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions64_sequence[] = {
  { &hf_umts_rrc_pdu_def_utranMobilityInformation_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions43, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions43 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions64(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions64, T_laterNonCriticalExtensions64_sequence);

  return offset;
}


static const per_sequence_t T_v3a0NonCriticalExtensions11_sequence[] = {
  { &hf_umts_rrc_pdu_def_utranMobilityInformation_v3a0ext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_UTRANMobilityInformation_v3a0ext_IEs },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions64, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions64 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions11(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions11, T_v3a0NonCriticalExtensions11_sequence);

  return offset;
}


static const per_sequence_t T_r330_sequence[] = {
  { &hf_umts_rrc_pdu_def_utranMobilityInformation_r3, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_UTRANMobilityInformation_r3_IEs },
  { &hf_umts_rrc_pdu_def_v3a0NonCriticalExtensions11, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions11 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r330(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r330, T_r330_sequence);

  return offset;
}


static const per_sequence_t UTRANMobilityInformation_r5_IEs_sequence[] = {
  { &hf_umts_rrc_pdu_def_integrityProtectionModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_IntegrityProtectionModeInfo },
  { &hf_umts_rrc_pdu_def_cipheringModeInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CipheringModeInfo },
  { &hf_umts_rrc_pdu_def_new_U_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_U_RNTI },
  { &hf_umts_rrc_pdu_def_new_C_RNTI, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_C_RNTI },
  { &hf_umts_rrc_pdu_def_ue_ConnTimersAndConstants1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UE_ConnTimersAndConstants_r5 },
  { &hf_umts_rrc_pdu_def_cn_InformationInfo2, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_CN_InformationInfoFull },
  { &hf_umts_rrc_pdu_def_ura_Identity, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_URA_Identity },
  { &hf_umts_rrc_pdu_def_dl_CounterSynchronisationInfo1, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DL_CounterSynchronisationInfo_r5 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_UTRANMobilityInformation_r5_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_UTRANMobilityInformation_r5_IEs, UTRANMobilityInformation_r5_IEs_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions106_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions106(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions106, T_nonCriticalExtensions106_sequence);

  return offset;
}


static const per_sequence_t T_v6xyNonCriticalExtensions44_sequence[] = {
  { &hf_umts_rrc_pdu_def_utranMobilityInformation_v6xyext, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_UtranMobilityInformation_v6xyext_IEs },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions106, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions106 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions44(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions44, T_v6xyNonCriticalExtensions44_sequence);

  return offset;
}


static const per_sequence_t T_r513_sequence[] = {
  { &hf_umts_rrc_pdu_def_utranMobilityInformation_r5, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_UTRANMobilityInformation_r5_IEs },
  { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions44, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions44 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_r513(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_r513, T_r513_sequence);

  return offset;
}


static const per_sequence_t T_criticalExtensions70_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions70(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_criticalExtensions70, T_criticalExtensions70_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_criticalExtensions69_vals[] = {
  {   0, "r5" },
  {   1, "criticalExtensions" },
  { 0, NULL }
};

static const per_choice_t T_criticalExtensions69_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r513, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r513 },
  {   1, &hf_umts_rrc_pdu_def_criticalExtensions70, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_criticalExtensions70 },
  { 0, NULL, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_criticalExtensions69(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_T_criticalExtensions69, T_criticalExtensions69_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_later_than_r327_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_criticalExtensions69, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_pdu_def_T_criticalExtensions69 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_later_than_r327(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_later_than_r327, T_later_than_r327_sequence);

  return offset;
}


const value_string umts_rrc_pdu_def_UTRANMobilityInformation_vals[] = {
  {   0, "r3" },
  {   1, "later-than-r3" },
  { 0, NULL }
};

static const per_choice_t UTRANMobilityInformation_choice[] = {
  {   0, &hf_umts_rrc_pdu_def_r330, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_r330 },
  {   1, &hf_umts_rrc_pdu_def_later_than_r327, ASN1_NO_EXTENSIONS     , dissect_umts_rrc_pdu_def_T_later_than_r327 },
  { 0, NULL, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_UTRANMobilityInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_umts_rrc_pdu_def_UTRANMobilityInformation, UTRANMobilityInformation_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions108_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions108(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions108, T_nonCriticalExtensions108_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions65_sequence[] = {
  { &hf_umts_rrc_pdu_def_utranMobilityInformationConfirm_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions108, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions108 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions65(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions65, T_laterNonCriticalExtensions65_sequence);

  return offset;
}


static const per_sequence_t UTRANMobilityInformationConfirm_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_ul_IntegProtActivationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_IntegrityProtActivationInfo },
  { &hf_umts_rrc_pdu_def_count_C_ActivationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_ActivationTime },
  { &hf_umts_rrc_pdu_def_dummy_RB_ActivationTimeInfoList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_ActivationTimeInfoList },
  { &hf_umts_rrc_pdu_def_ul_CounterSynchronisationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_UL_CounterSynchronisationInfo },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions65, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions65 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_UTRANMobilityInformationConfirm(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_UTRANMobilityInformationConfirm, UTRANMobilityInformationConfirm_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions109_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions109(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions109, T_nonCriticalExtensions109_sequence);

  return offset;
}


static const per_sequence_t T_laterNonCriticalExtensions66_sequence[] = {
  { &hf_umts_rrc_pdu_def_utranMobilityInformationFailure_r3_add_ext, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_BIT_STRING },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions109, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions109 },
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions66(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions66, T_laterNonCriticalExtensions66_sequence);

  return offset;
}


static const per_sequence_t UTRANMobilityInformationFailure_sequence[] = {
  { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_RRC_TransactionIdentifier },
  { &hf_umts_rrc_pdu_def_failureCause, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_FailureCauseWithProtErr },
  { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions66, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_laterNonCriticalExtensions66 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_UTRANMobilityInformationFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_UTRANMobilityInformationFailure, UTRANMobilityInformationFailure_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions110_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions110(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions110, T_nonCriticalExtensions110_sequence);

  return offset;
}


static const per_sequence_t MBMSAccessInformation_sequence[] = {
  { &hf_umts_rrc_pdu_def_mbms_ServiceAccessInfoList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_MBMS_ServiceAccessInfoList_r6 },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions110, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions110 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_MBMSAccessInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_MBMSAccessInformation, MBMSAccessInformation_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions111_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions111(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions111, T_nonCriticalExtensions111_sequence);

  return offset;
}


static const per_sequence_t MBMSCommonPTMRBInformation_sequence[] = {
  { &hf_umts_rrc_pdu_def_mbms_CommonRBInformationList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_MBMS_CommonRBInformationList_r6 },
  { &hf_umts_rrc_pdu_def_mbms_TranspChInfoForEachTrCh, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_MBMS_TranspChInfoForEachTrCh_r6 },
  { &hf_umts_rrc_pdu_def_mbms_TranspChInfoForEachCCTrCh, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_MBMS_TranspChInfoForEachCCTrCh_r6 },
  { &hf_umts_rrc_pdu_def_mbms_PhyChInformationList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_MBMS_PhyChInformationList_r6 },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions111, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions111 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_MBMSCommonPTMRBInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_MBMSCommonPTMRBInformation, MBMSCommonPTMRBInformation_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions112_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions112(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions112, T_nonCriticalExtensions112_sequence);

  return offset;
}


static const per_sequence_t MBMSCurrentCellPTMRBInformation_sequence[] = {
  { &hf_umts_rrc_pdu_def_mbms_CurrentCell_SCCPCHList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MBMS_CurrentCell_SCCPCHList_r6 },
  { &hf_umts_rrc_pdu_def_mbms_SIBType5_SCCPCHList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MBMS_SIBType5_SCCPCHList_r6 },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions112, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions112 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_MBMSCurrentCellPTMRBInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_MBMSCurrentCellPTMRBInformation, MBMSCurrentCellPTMRBInformation_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions113_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions113(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions113, T_nonCriticalExtensions113_sequence);

  return offset;
}


static const per_sequence_t MBMSGeneralInformation_sequence[] = {
  { &hf_umts_rrc_pdu_def_mbms_PreferredFrequencyInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MBMS_PreferredFrequencyList_r6 },
  { &hf_umts_rrc_pdu_def_mbms_TimersAndCouneters, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_MBMS_TimersAndCouneters_r6 },
  { &hf_umts_rrc_pdu_def_michConfigurationInfo, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_MBMS_MICHConfigurationInfo_r6 },
  { &hf_umts_rrc_pdu_def_cellGroupIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_MBMS_CellGroupIdentity_r6 },
  { &hf_umts_rrc_pdu_def_mschDefaultConfigurationInfo, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MBMS_MSCHConfigurationInfo_r6 },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions113, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions113 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_MBMSGeneralInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_MBMSGeneralInformation, MBMSGeneralInformation_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions114_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions114(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions114, T_nonCriticalExtensions114_sequence);

  return offset;
}


static const per_sequence_t MBMSModificationRequest_sequence[] = {
  { &hf_umts_rrc_pdu_def_mbms_PreferredFreqRequest, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MBMS_ServiceIdentity },
  { &hf_umts_rrc_pdu_def_rb_InformationReleaseList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_RB_InformationReleaseList },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions114, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions114 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_MBMSModificationRequest(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_MBMSModificationRequest, MBMSModificationRequest_sequence);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_mbms_ReacquireMCCH_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_umts_rrc_pdu_def_T_mbms_ReacquireMCCH(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_umts_rrc_pdu_def_INTEGER_1_16(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 16U, NULL, FALSE);

  return offset;
}


static const value_string umts_rrc_pdu_def_T_mbms_AllUnmodifiedPTMServices_vals[] = {
  {   0, "true" },
  { 0, NULL }
};


static int
dissect_umts_rrc_pdu_def_T_mbms_AllUnmodifiedPTMServices(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions115_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions115(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions115, T_nonCriticalExtensions115_sequence);

  return offset;
}


static const per_sequence_t MBMSModifiedServicesInformation_sequence[] = {
  { &hf_umts_rrc_pdu_def_modifedServiceList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MBMS_ModifedServiceList_r6 },
  { &hf_umts_rrc_pdu_def_mbms_ReacquireMCCH, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_mbms_ReacquireMCCH },
  { &hf_umts_rrc_pdu_def_mbms_DynamicPersistenceLevel, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_DynamicPersistenceLevel },
  { &hf_umts_rrc_pdu_def_endOfModifiedMCCHInformation, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_INTEGER_1_16 },
  { &hf_umts_rrc_pdu_def_mbms_AllUnmodifiedPTMServices, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_mbms_AllUnmodifiedPTMServices },
  { &hf_umts_rrc_pdu_def_mbms_PtMActivationTime, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MBMS_PtMActivationTime },
  { &hf_umts_rrc_pdu_def_mbmsNumberOfNeighbourCells, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_MBMS_NumberOfNeighbourCells_r6 },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions115, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions115 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_MBMSModifiedServicesInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_MBMSModifiedServicesInformation, MBMSModifiedServicesInformation_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions116_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions116(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions116, T_nonCriticalExtensions116_sequence);

  return offset;
}


static const per_sequence_t MBMSNeighbouringCellPTMRBInformation_sequence[] = {
  { &hf_umts_rrc_pdu_def_neighbouringCellIdentity, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_IntraFreqCellID },
  { &hf_umts_rrc_pdu_def_neighbouringCellSCCPCHList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_MBMS_NeighbouringCellSCCPCHList_r6 },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions116, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions116 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_MBMSNeighbouringCellPTMRBInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_MBMSNeighbouringCellPTMRBInformation, MBMSNeighbouringCellPTMRBInformation_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions117_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions117(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions117, T_nonCriticalExtensions117_sequence);

  return offset;
}


static const per_sequence_t MBMSSchedulingInformation_sequence[] = {
  { &hf_umts_rrc_pdu_def_serviceSchedulingInfoList, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_umts_rrc_ies_MBMS_ServiceSchedulingInfoList_r6 },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions117, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions117 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_MBMSSchedulingInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_MBMSSchedulingInformation, MBMSSchedulingInformation_sequence);

  return offset;
}


static const per_sequence_t T_nonCriticalExtensions118_sequence[] = {
  { NULL, 0, 0, NULL }
};

static int
dissect_umts_rrc_pdu_def_T_nonCriticalExtensions118(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_T_nonCriticalExtensions118, T_nonCriticalExtensions118_sequence);

  return offset;
}


static const per_sequence_t MBMSUnmodifiedServicesInformation_sequence[] = {
  { &hf_umts_rrc_pdu_def_unmodifiedServiceList, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_ies_MBMS_UnmodifiedServiceList_r6 },
  { &hf_umts_rrc_pdu_def_nonCriticalExtensions118, ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_umts_rrc_pdu_def_T_nonCriticalExtensions118 },
  { NULL, 0, 0, NULL }
};

int
dissect_umts_rrc_pdu_def_MBMSUnmodifiedServicesInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_umts_rrc_pdu_def_MBMSUnmodifiedServicesInformation, MBMSUnmodifiedServicesInformation_sequence);

  return offset;
}


/*--- End of included file: packet-umts_rrc_pdu_def-fn.c ---*/
#line 67 "packet-umts_rrc_pdu_def-template.c"


static void
dissect_umts_rrc_pdu_def(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
{
	/* 
	 * Dummy function, currently not used
	 */

}
/*--- proto_register_umts_rrc_pdu_def -------------------------------------------*/
void proto_register_umts_rrc_pdu_def(void) {

  /* List of fields */
  static hf_register_info hf[] = {


/*--- Included file: packet-umts_rrc_pdu_def-hfarr.c ---*/
#line 1 "packet-umts_rrc_pdu_def-hfarr.c"
    { &hf_umts_rrc_pdu_def_r3,
      { "r3", "umts_rrc_pdu_def.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r3", HFILL }},
    { &hf_umts_rrc_pdu_def_activeSetUpdate_r3,
      { "activeSetUpdate-r3", "umts_rrc_pdu_def.activeSetUpdate_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.ActiveSetUpdate_r3_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions", HFILL }},
    { &hf_umts_rrc_pdu_def_activeSetUpdate_r3_add_ext,
      { "activeSetUpdate-r3-add-ext", "umts_rrc_pdu_def.activeSetUpdate_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions,
      { "v4b0NonCriticalExtensions", "umts_rrc_pdu_def.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v4b0NonCriticalExtensions", HFILL }},
    { &hf_umts_rrc_pdu_def_activeSetUpdate_v4b0ext,
      { "activeSetUpdate-v4b0ext", "umts_rrc_pdu_def.activeSetUpdate_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.ActiveSetUpdate_v4b0ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions,
      { "v590NonCriticalExtensions", "umts_rrc_pdu_def.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v590NonCriticalExtensions", HFILL }},
    { &hf_umts_rrc_pdu_def_activeSetUpdate_v590ext,
      { "activeSetUpdate-v590ext", "umts_rrc_pdu_def.activeSetUpdate_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.ActiveSetUpdate_v590ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions", HFILL }},
    { &hf_umts_rrc_pdu_def_activeSetUpdate_v6xyext,
      { "activeSetUpdate-v6xyext", "umts_rrc_pdu_def.activeSetUpdate_v6xyext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.ActiveSetUpdate_v6xyext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions", HFILL }},
    { &hf_umts_rrc_pdu_def_later_than_r3,
      { "later-than-r3", "umts_rrc_pdu_def.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_later_than_r3", HFILL }},
    { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier,
      { "rrc-TransactionIdentifier", "umts_rrc_pdu_def.rrc_TransactionIdentifier",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.RRC_TransactionIdentifier", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_criticalExtensions_vals), 0,
        "umts_rrc_pdu_def.T_criticalExtensions", HFILL }},
    { &hf_umts_rrc_pdu_def_r6,
      { "r6", "umts_rrc_pdu_def.r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r6", HFILL }},
    { &hf_umts_rrc_pdu_def_activeSetUpdate_r6,
      { "activeSetUpdate-r6", "umts_rrc_pdu_def.activeSetUpdate_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.ActiveSetUpdate_r6_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_activeSetUpdate_r6_add_ext,
      { "activeSetUpdate-r6-add-ext", "umts_rrc_pdu_def.activeSetUpdate_r6_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions1,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions1", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions1,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_criticalExtensions1", HFILL }},
    { &hf_umts_rrc_pdu_def_dummy_IntegrityProtectionModeInfo,
      { "dummy-IntegrityProtectionModeInfo", "umts_rrc_pdu_def.dummy_IntegrityProtectionModeInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.IntegrityProtectionModeInfo", HFILL }},
    { &hf_umts_rrc_pdu_def_dummy2_CipheringModeInfo,
      { "dummy2-CipheringModeInfo", "umts_rrc_pdu_def.dummy2_CipheringModeInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.CipheringModeInfo", HFILL }},
    { &hf_umts_rrc_pdu_def_activationTime,
      { "activationTime", "umts_rrc_pdu_def.activationTime",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.ActivationTime", HFILL }},
    { &hf_umts_rrc_pdu_def_newU_RNTI,
      { "newU-RNTI", "umts_rrc_pdu_def.newU_RNTI",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.U_RNTI", HFILL }},
    { &hf_umts_rrc_pdu_def_cn_InformationInfo,
      { "cn-InformationInfo", "umts_rrc_pdu_def.cn_InformationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.CN_InformationInfo", HFILL }},
    { &hf_umts_rrc_pdu_def_dummy3_DL_CounterSynchronisationInfo,
      { "dummy3-DL-CounterSynchronisationInfo", "umts_rrc_pdu_def.dummy3_DL_CounterSynchronisationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.DL_CounterSynchronisationInfo", HFILL }},
    { &hf_umts_rrc_pdu_def_maxAllowedUL_TX_Power,
      { "maxAllowedUL-TX-Power", "umts_rrc_pdu_def.maxAllowedUL_TX_Power",
        FT_INT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.MaxAllowedUL_TX_Power", HFILL }},
    { &hf_umts_rrc_pdu_def_rl_AdditionInformationList,
      { "rl-AdditionInformationList", "umts_rrc_pdu_def.rl_AdditionInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.RL_AdditionInformationList", HFILL }},
    { &hf_umts_rrc_pdu_def_rl_RemovalInformationList,
      { "rl-RemovalInformationList", "umts_rrc_pdu_def.rl_RemovalInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.RL_RemovalInformationList", HFILL }},
    { &hf_umts_rrc_pdu_def_tx_DiversityMode,
      { "tx-DiversityMode", "umts_rrc_pdu_def.tx_DiversityMode",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_TX_DiversityMode_vals), 0,
        "umts_rrc_ies.TX_DiversityMode", HFILL }},
    { &hf_umts_rrc_pdu_def_dummy4_SSDT_Information,
      { "dummy4-SSDT-Information", "umts_rrc_pdu_def.dummy4_SSDT_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.SSDT_Information", HFILL }},
    { &hf_umts_rrc_pdu_def_dummy_SSDT_UL,
      { "dummy-SSDT-UL", "umts_rrc_pdu_def.dummy_SSDT_UL",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_SSDT_UL_vals), 0,
        "umts_rrc_ies.SSDT_UL", HFILL }},
    { &hf_umts_rrc_pdu_def_cell_id_PerRL_List,
      { "cell-id-PerRL-List", "umts_rrc_pdu_def.cell_id_PerRL_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.CellIdentity_PerRL_List", HFILL }},
    { &hf_umts_rrc_pdu_def_dpc_Mode,
      { "dpc-Mode", "umts_rrc_pdu_def.dpc_Mode",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_DPC_Mode_vals), 0,
        "umts_rrc_ies.DPC_Mode", HFILL }},
    { &hf_umts_rrc_pdu_def_dl_TPC_PowerOffsetPerRL_List,
      { "dl-TPC-PowerOffsetPerRL-List", "umts_rrc_pdu_def.dl_TPC_PowerOffsetPerRL_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.DL_TPC_PowerOffsetPerRL_List", HFILL }},
    { &hf_umts_rrc_pdu_def_primary_plmn_Identity,
      { "primary-plmn-Identity", "umts_rrc_pdu_def.primary_plmn_Identity",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.PLMN_Identity", HFILL }},
    { &hf_umts_rrc_pdu_def_newH_RNTI,
      { "newH-RNTI", "umts_rrc_pdu_def.newH_RNTI",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_ies.H_RNTI", HFILL }},
    { &hf_umts_rrc_pdu_def_newPrimary_E_RNTI,
      { "newPrimary-E-RNTI", "umts_rrc_pdu_def.newPrimary_E_RNTI",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_ies.E_RNTI", HFILL }},
    { &hf_umts_rrc_pdu_def_newSecondary_E_RNTI,
      { "newSecondary-E-RNTI", "umts_rrc_pdu_def.newSecondary_E_RNTI",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_ies.E_RNTI", HFILL }},
    { &hf_umts_rrc_pdu_def_cn_InformationInfo1,
      { "cn-InformationInfo", "umts_rrc_pdu_def.cn_InformationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.CN_InformationInfo_r6", HFILL }},
    { &hf_umts_rrc_pdu_def_rl_AdditionInformationList1,
      { "rl-AdditionInformationList", "umts_rrc_pdu_def.rl_AdditionInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.RL_AdditionInformationList_r6", HFILL }},
    { &hf_umts_rrc_pdu_def_serving_HSDSCH_CellInformation,
      { "serving-HSDSCH-CellInformation", "umts_rrc_pdu_def.serving_HSDSCH_CellInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.Serving_HSDSCH_CellInformation", HFILL }},
    { &hf_umts_rrc_pdu_def_scheduling_E_DCH_CellInformation,
      { "scheduling-E-DCH-CellInformation", "umts_rrc_pdu_def.scheduling_E_DCH_CellInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.Scheduling_E_DCH_CellInformation", HFILL }},
    { &hf_umts_rrc_pdu_def_dummy_IntegrityProtActivationInfo,
      { "dummy-IntegrityProtActivationInfo", "umts_rrc_pdu_def.dummy_IntegrityProtActivationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.IntegrityProtActivationInfo", HFILL }},
    { &hf_umts_rrc_pdu_def_dummy2_RB_ActivationTimeInfoList,
      { "dummy2-RB-ActivationTimeInfoList", "umts_rrc_pdu_def.dummy2_RB_ActivationTimeInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.RB_ActivationTimeInfoList", HFILL }},
    { &hf_umts_rrc_pdu_def_dummy3_UL_CounterSynchronisationInfo,
      { "dummy3-UL-CounterSynchronisationInfo", "umts_rrc_pdu_def.dummy3_UL_CounterSynchronisationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.UL_CounterSynchronisationInfo", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions1,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions1", HFILL }},
    { &hf_umts_rrc_pdu_def_activeSetUpdateComplete_r3_add_ext,
      { "activeSetUpdateComplete-r3-add-ext", "umts_rrc_pdu_def.activeSetUpdateComplete_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions2,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions2", HFILL }},
    { &hf_umts_rrc_pdu_def_failureCause,
      { "failureCause", "umts_rrc_pdu_def.failureCause",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_FailureCauseWithProtErr_vals), 0,
        "umts_rrc_ies.FailureCauseWithProtErr", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions2,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions2", HFILL }},
    { &hf_umts_rrc_pdu_def_activeSetUpdateFailure_r3_add_ext,
      { "activeSetUpdateFailure-r3-add-ext", "umts_rrc_pdu_def.activeSetUpdateFailure_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions3,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions3", HFILL }},
    { &hf_umts_rrc_pdu_def_r31,
      { "r3", "umts_rrc_pdu_def.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r31", HFILL }},
    { &hf_umts_rrc_pdu_def_assistanceDataDelivery_r3,
      { "assistanceDataDelivery-r3", "umts_rrc_pdu_def.assistanceDataDelivery_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.AssistanceDataDelivery_r3_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v3a0NonCriticalExtensions,
      { "v3a0NonCriticalExtensions", "umts_rrc_pdu_def.v3a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v3a0NonCriticalExtensions", HFILL }},
    { &hf_umts_rrc_pdu_def_assistanceDataDelivery_v3a0ext,
      { "assistanceDataDelivery-v3a0ext", "umts_rrc_pdu_def.assistanceDataDelivery_v3a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.AssistanceDataDelivery_v3a0ext", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions3,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions3", HFILL }},
    { &hf_umts_rrc_pdu_def_assistanceDataDelivery_r3_add_ext,
      { "assistanceDataDelivery-r3-add-ext", "umts_rrc_pdu_def.assistanceDataDelivery_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions1,
      { "v4b0NonCriticalExtensions", "umts_rrc_pdu_def.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v4b0NonCriticalExtensions1", HFILL }},
    { &hf_umts_rrc_pdu_def_assistanceDataDelivery_v4b0ext,
      { "assistanceDataDelivery-v4b0ext", "umts_rrc_pdu_def.assistanceDataDelivery_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.AssistanceDataDelivery_v4b0ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions4,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions4", HFILL }},
    { &hf_umts_rrc_pdu_def_later_than_r31,
      { "later-than-r3", "umts_rrc_pdu_def.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_later_than_r31", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions2,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_criticalExtensions2", HFILL }},
    { &hf_umts_rrc_pdu_def_ue_positioning_GPS_AssistanceData,
      { "ue-positioning-GPS-AssistanceData", "umts_rrc_pdu_def.ue_positioning_GPS_AssistanceData",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.UE_Positioning_GPS_AssistanceData", HFILL }},
    { &hf_umts_rrc_pdu_def_ue_positioning_OTDOA_AssistanceData_UEB,
      { "ue-positioning-OTDOA-AssistanceData-UEB", "umts_rrc_pdu_def.ue_positioning_OTDOA_AssistanceData_UEB",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.UE_Positioning_OTDOA_AssistanceData_UEB", HFILL }},
    { &hf_umts_rrc_pdu_def_sfn_Offset_Validity,
      { "sfn-Offset-Validity", "umts_rrc_pdu_def.sfn_Offset_Validity",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_SFN_Offset_Validity_vals), 0,
        "umts_rrc_ies.SFN_Offset_Validity", HFILL }},
    { &hf_umts_rrc_pdu_def_ue_Positioning_OTDOA_AssistanceData_r4ext,
      { "ue-Positioning-OTDOA-AssistanceData-r4ext", "umts_rrc_pdu_def.ue_Positioning_OTDOA_AssistanceData_r4ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.UE_Positioning_OTDOA_AssistanceData_r4ext", HFILL }},
    { &hf_umts_rrc_pdu_def_r32,
      { "r3", "umts_rrc_pdu_def.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r32", HFILL }},
    { &hf_umts_rrc_pdu_def_cellChangeOrderFromUTRAN_IEs,
      { "cellChangeOrderFromUTRAN-IEs", "umts_rrc_pdu_def.cellChangeOrderFromUTRAN_IEs",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.CellChangeOrderFromUTRAN_r3_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions4,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions4", HFILL }},
    { &hf_umts_rrc_pdu_def_cellChangeOrderFromUTRAN_r3_add_ext,
      { "cellChangeOrderFromUTRAN-r3-add-ext", "umts_rrc_pdu_def.cellChangeOrderFromUTRAN_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions1,
      { "v590NonCriticalExtensions", "umts_rrc_pdu_def.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v590NonCriticalExtensions1", HFILL }},
    { &hf_umts_rrc_pdu_def_cellChangeOrderFromUTRAN_v590ext,
      { "cellChangeOrderFromUTRAN-v590ext", "umts_rrc_pdu_def.cellChangeOrderFromUTRAN_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.CellChangeOrderFromUTRAN_v590ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions5,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions5", HFILL }},
    { &hf_umts_rrc_pdu_def_later_than_r32,
      { "later-than-r3", "umts_rrc_pdu_def.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_later_than_r32", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions3,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_criticalExtensions3", HFILL }},
    { &hf_umts_rrc_pdu_def_rab_InformationList,
      { "rab-InformationList", "umts_rrc_pdu_def.rab_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.RAB_InformationList", HFILL }},
    { &hf_umts_rrc_pdu_def_interRAT_TargetCellDescription,
      { "interRAT-TargetCellDescription", "umts_rrc_pdu_def.interRAT_TargetCellDescription",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.InterRAT_TargetCellDescription", HFILL }},
    { &hf_umts_rrc_pdu_def_geran_SystemInfoType,
      { "geran-SystemInfoType", "umts_rrc_pdu_def.geran_SystemInfoType",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_geran_SystemInfoType_vals), 0,
        "umts_rrc_pdu_def.T_geran_SystemInfoType", HFILL }},
    { &hf_umts_rrc_pdu_def_sI,
      { "sI", "umts_rrc_pdu_def.sI",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.GERAN_SystemInformation", HFILL }},
    { &hf_umts_rrc_pdu_def_pSI,
      { "pSI", "umts_rrc_pdu_def.pSI",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.GERAN_SystemInformation", HFILL }},
    { &hf_umts_rrc_pdu_def_r33,
      { "r3", "umts_rrc_pdu_def.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r33", HFILL }},
    { &hf_umts_rrc_pdu_def_cellChangeOrderFromUTRANFailure_r3,
      { "cellChangeOrderFromUTRANFailure-r3", "umts_rrc_pdu_def.cellChangeOrderFromUTRANFailure_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.CellChangeOrderFromUTRANFailure_r3_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions5,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions5", HFILL }},
    { &hf_umts_rrc_pdu_def_cellChangeOrderFromUTRANFailure_r3_add_ext,
      { "cellChangeOrderFromUTRANFailure-r3-add-ext", "umts_rrc_pdu_def.cellChangeOrderFromUTRANFailure_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions6,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions6", HFILL }},
    { &hf_umts_rrc_pdu_def_dummy,
      { "dummy", "umts_rrc_pdu_def.dummy",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_dummy", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions4,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_criticalExtensions4", HFILL }},
    { &hf_umts_rrc_pdu_def_interRAT_ChangeFailureCause,
      { "interRAT-ChangeFailureCause", "umts_rrc_pdu_def.interRAT_ChangeFailureCause",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_InterRAT_ChangeFailureCause_vals), 0,
        "umts_rrc_ies.InterRAT_ChangeFailureCause", HFILL }},
    { &hf_umts_rrc_pdu_def_u_RNTI,
      { "u-RNTI", "umts_rrc_pdu_def.u_RNTI",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.U_RNTI", HFILL }},
    { &hf_umts_rrc_pdu_def_startList,
      { "startList", "umts_rrc_pdu_def.startList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.STARTList", HFILL }},
    { &hf_umts_rrc_pdu_def_am_RLC_ErrorIndicationRb2_3or4,
      { "am-RLC-ErrorIndicationRb2-3or4", "umts_rrc_pdu_def.am_RLC_ErrorIndicationRb2_3or4",
        FT_BOOLEAN, 8, NULL, 0,
        "umts_rrc_pdu_def.BOOLEAN", HFILL }},
    { &hf_umts_rrc_pdu_def_am_RLC_ErrorIndicationRb5orAbove,
      { "am-RLC-ErrorIndicationRb5orAbove", "umts_rrc_pdu_def.am_RLC_ErrorIndicationRb5orAbove",
        FT_BOOLEAN, 8, NULL, 0,
        "umts_rrc_pdu_def.BOOLEAN", HFILL }},
    { &hf_umts_rrc_pdu_def_cellUpdateCause,
      { "cellUpdateCause", "umts_rrc_pdu_def.cellUpdateCause",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_CellUpdateCause_vals), 0,
        "umts_rrc_ies.CellUpdateCause", HFILL }},
    { &hf_umts_rrc_pdu_def_failureCause1,
      { "failureCause", "umts_rrc_pdu_def.failureCause",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.FailureCauseWithProtErrTrId", HFILL }},
    { &hf_umts_rrc_pdu_def_rb_timer_indicator,
      { "rb-timer-indicator", "umts_rrc_pdu_def.rb_timer_indicator",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.Rb_timer_indicator", HFILL }},
    { &hf_umts_rrc_pdu_def_measuredResultsOnRACH,
      { "measuredResultsOnRACH", "umts_rrc_pdu_def.measuredResultsOnRACH",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.MeasuredResultsOnRACH", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions6,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions6", HFILL }},
    { &hf_umts_rrc_pdu_def_cellUpdate_r3_add_ext,
      { "cellUpdate-r3-add-ext", "umts_rrc_pdu_def.cellUpdate_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions2,
      { "v590NonCriticalExtensions", "umts_rrc_pdu_def.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v590NonCriticalExtensions2", HFILL }},
    { &hf_umts_rrc_pdu_def_cellUpdate_v590ext,
      { "cellUpdate-v590ext", "umts_rrc_pdu_def.cellUpdate_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.CellUpdate_v590ext", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions1,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions1", HFILL }},
    { &hf_umts_rrc_pdu_def_cellUpdate_v6xyext,
      { "cellUpdate-v6xyext", "umts_rrc_pdu_def.cellUpdate_v6xyext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.CellUpdate_v6xyext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions7,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions7", HFILL }},
    { &hf_umts_rrc_pdu_def_establishmentCause,
      { "establishmentCause", "umts_rrc_pdu_def.establishmentCause",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_EstablishmentCause_vals), 0,
        "umts_rrc_ies.EstablishmentCause", HFILL }},
    { &hf_umts_rrc_pdu_def_cellUpdateCause_ext,
      { "cellUpdateCause-ext", "umts_rrc_pdu_def.cellUpdateCause_ext",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_CellUpdateCause_ext_vals), 0,
        "umts_rrc_ies.CellUpdateCause_ext", HFILL }},
    { &hf_umts_rrc_pdu_def_trafficVolumeIndicator,
      { "trafficVolumeIndicator", "umts_rrc_pdu_def.trafficVolumeIndicator",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_trafficVolumeIndicator_vals), 0,
        "umts_rrc_pdu_def.T_trafficVolumeIndicator", HFILL }},
    { &hf_umts_rrc_pdu_def_measuredResultsOnRACHinterFreq,
      { "measuredResultsOnRACHinterFreq", "umts_rrc_pdu_def.measuredResultsOnRACHinterFreq",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.MeasuredResultsOnRACHinterFreq", HFILL }},
    { &hf_umts_rrc_pdu_def_reconfigurationStatusIndicator,
      { "reconfigurationStatusIndicator", "umts_rrc_pdu_def.reconfigurationStatusIndicator",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_reconfigurationStatusIndicator_vals), 0,
        "umts_rrc_pdu_def.T_reconfigurationStatusIndicator", HFILL }},
    { &hf_umts_rrc_pdu_def_r34,
      { "r3", "umts_rrc_pdu_def.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r34", HFILL }},
    { &hf_umts_rrc_pdu_def_cellUpdateConfirm_r3,
      { "cellUpdateConfirm-r3", "umts_rrc_pdu_def.cellUpdateConfirm_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.CellUpdateConfirm_r3_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v3a0NonCriticalExtensions1,
      { "v3a0NonCriticalExtensions", "umts_rrc_pdu_def.v3a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v3a0NonCriticalExtensions1", HFILL }},
    { &hf_umts_rrc_pdu_def_cellUpdateConfirm_v3a0ext,
      { "cellUpdateConfirm-v3a0ext", "umts_rrc_pdu_def.cellUpdateConfirm_v3a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.CellUpdateConfirm_v3a0ext", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions7,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions7", HFILL }},
    { &hf_umts_rrc_pdu_def_cellUpdateConfirm_r3_add_ext,
      { "cellUpdateConfirm-r3-add-ext", "umts_rrc_pdu_def.cellUpdateConfirm_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions2,
      { "v4b0NonCriticalExtensions", "umts_rrc_pdu_def.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v4b0NonCriticalExtensions2", HFILL }},
    { &hf_umts_rrc_pdu_def_cellUpdateConfirm_v4b0ext,
      { "cellUpdateConfirm-v4b0ext", "umts_rrc_pdu_def.cellUpdateConfirm_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.CellUpdateConfirm_v4b0ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v590NonCriticalExtenstions,
      { "v590NonCriticalExtenstions", "umts_rrc_pdu_def.v590NonCriticalExtenstions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v590NonCriticalExtenstions", HFILL }},
    { &hf_umts_rrc_pdu_def_cellUpdateConfirm_v590ext,
      { "cellUpdateConfirm-v590ext", "umts_rrc_pdu_def.cellUpdateConfirm_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.CellUpdateConfirm_v590ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v5d0NonCriticalExtenstions,
      { "v5d0NonCriticalExtenstions", "umts_rrc_pdu_def.v5d0NonCriticalExtenstions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v5d0NonCriticalExtenstions", HFILL }},
    { &hf_umts_rrc_pdu_def_cellUpdateConfirm_v5d0ext,
      { "cellUpdateConfirm-v5d0ext", "umts_rrc_pdu_def.cellUpdateConfirm_v5d0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.CellUpdateConfirm_v5d0ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions2,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions2", HFILL }},
    { &hf_umts_rrc_pdu_def_cellUpdateConfirm_v6xyext,
      { "cellUpdateConfirm-v6xyext", "umts_rrc_pdu_def.cellUpdateConfirm_v6xyext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.CellUpdateConfirm_v6xyext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions8,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions8", HFILL }},
    { &hf_umts_rrc_pdu_def_later_than_r33,
      { "later-than-r3", "umts_rrc_pdu_def.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_later_than_r33", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions5,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_criticalExtensions5_vals), 0,
        "umts_rrc_pdu_def.T_criticalExtensions5", HFILL }},
    { &hf_umts_rrc_pdu_def_r4,
      { "r4", "umts_rrc_pdu_def.r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r4", HFILL }},
    { &hf_umts_rrc_pdu_def_cellUpdateConfirm_r4,
      { "cellUpdateConfirm-r4", "umts_rrc_pdu_def.cellUpdateConfirm_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.CellUpdateConfirm_r4_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v4d0NonCriticalExtensions,
      { "v4d0NonCriticalExtensions", "umts_rrc_pdu_def.v4d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v4d0NonCriticalExtensions", HFILL }},
    { &hf_umts_rrc_pdu_def_cellUpdateConfirm_r4_add_ext,
      { "cellUpdateConfirm-r4-add-ext", "umts_rrc_pdu_def.cellUpdateConfirm_r4_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v590NonCriticalExtenstions1,
      { "v590NonCriticalExtenstions", "umts_rrc_pdu_def.v590NonCriticalExtenstions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v590NonCriticalExtenstions1", HFILL }},
    { &hf_umts_rrc_pdu_def_v5d0NonCriticalExtenstions1,
      { "v5d0NonCriticalExtenstions", "umts_rrc_pdu_def.v5d0NonCriticalExtenstions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v5d0NonCriticalExtenstions1", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions3,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions3", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions9,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions9", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions6,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_criticalExtensions6_vals), 0,
        "umts_rrc_pdu_def.T_criticalExtensions6", HFILL }},
    { &hf_umts_rrc_pdu_def_r5,
      { "r5", "umts_rrc_pdu_def.r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r5", HFILL }},
    { &hf_umts_rrc_pdu_def_cellUpdateConfirm_r5,
      { "cellUpdateConfirm-r5", "umts_rrc_pdu_def.cellUpdateConfirm_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.CellUpdateConfirm_r5_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_cellUpdateConfirm_r5_add_ext,
      { "cellUpdateConfirm-r5-add-ext", "umts_rrc_pdu_def.cellUpdateConfirm_r5_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v5d0NonCriticalExtenstions2,
      { "v5d0NonCriticalExtenstions", "umts_rrc_pdu_def.v5d0NonCriticalExtenstions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v5d0NonCriticalExtenstions2", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions4,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions4", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions10,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions10", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions7,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_criticalExtensions7_vals), 0,
        "umts_rrc_pdu_def.T_criticalExtensions7", HFILL }},
    { &hf_umts_rrc_pdu_def_r61,
      { "r6", "umts_rrc_pdu_def.r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r61", HFILL }},
    { &hf_umts_rrc_pdu_def_cellUpdateConfirm_r6,
      { "cellUpdateConfirm-r6", "umts_rrc_pdu_def.cellUpdateConfirm_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.CellUpdateConfirm_r6_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_cellUpdateConfirm_r6_add_ext,
      { "cellUpdateConfirm-r6-add-ext", "umts_rrc_pdu_def.cellUpdateConfirm_r6_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions11,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions11", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions8,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_criticalExtensions8", HFILL }},
    { &hf_umts_rrc_pdu_def_integrityProtectionModeInfo,
      { "integrityProtectionModeInfo", "umts_rrc_pdu_def.integrityProtectionModeInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.IntegrityProtectionModeInfo", HFILL }},
    { &hf_umts_rrc_pdu_def_cipheringModeInfo,
      { "cipheringModeInfo", "umts_rrc_pdu_def.cipheringModeInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.CipheringModeInfo", HFILL }},
    { &hf_umts_rrc_pdu_def_new_U_RNTI,
      { "new-U-RNTI", "umts_rrc_pdu_def.new_U_RNTI",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.U_RNTI", HFILL }},
    { &hf_umts_rrc_pdu_def_new_C_RNTI,
      { "new-C-RNTI", "umts_rrc_pdu_def.new_C_RNTI",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_ies.C_RNTI", HFILL }},
    { &hf_umts_rrc_pdu_def_rrc_StateIndicator,
      { "rrc-StateIndicator", "umts_rrc_pdu_def.rrc_StateIndicator",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_RRC_StateIndicator_vals), 0,
        "umts_rrc_ies.RRC_StateIndicator", HFILL }},
    { &hf_umts_rrc_pdu_def_utran_DRX_CycleLengthCoeff,
      { "utran-DRX-CycleLengthCoeff", "umts_rrc_pdu_def.utran_DRX_CycleLengthCoeff",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.UTRAN_DRX_CycleLengthCoefficient", HFILL }},
    { &hf_umts_rrc_pdu_def_rlc_Re_establishIndicatorRb2_3or4,
      { "rlc-Re-establishIndicatorRb2-3or4", "umts_rrc_pdu_def.rlc_Re_establishIndicatorRb2_3or4",
        FT_BOOLEAN, 8, NULL, 0,
        "umts_rrc_pdu_def.BOOLEAN", HFILL }},
    { &hf_umts_rrc_pdu_def_rlc_Re_establishIndicatorRb5orAbove,
      { "rlc-Re-establishIndicatorRb5orAbove", "umts_rrc_pdu_def.rlc_Re_establishIndicatorRb5orAbove",
        FT_BOOLEAN, 8, NULL, 0,
        "umts_rrc_pdu_def.BOOLEAN", HFILL }},
    { &hf_umts_rrc_pdu_def_ura_Identity,
      { "ura-Identity", "umts_rrc_pdu_def.ura_Identity",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_ies.URA_Identity", HFILL }},
    { &hf_umts_rrc_pdu_def_rb_InformationReleaseList,
      { "rb-InformationReleaseList", "umts_rrc_pdu_def.rb_InformationReleaseList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.RB_InformationReleaseList", HFILL }},
    { &hf_umts_rrc_pdu_def_rb_InformationReconfigList,
      { "rb-InformationReconfigList", "umts_rrc_pdu_def.rb_InformationReconfigList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.RB_InformationReconfigList", HFILL }},
    { &hf_umts_rrc_pdu_def_rb_InformationAffectedList,
      { "rb-InformationAffectedList", "umts_rrc_pdu_def.rb_InformationAffectedList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.RB_InformationAffectedList", HFILL }},
    { &hf_umts_rrc_pdu_def_dl_CounterSynchronisationInfo,
      { "dl-CounterSynchronisationInfo", "umts_rrc_pdu_def.dl_CounterSynchronisationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.DL_CounterSynchronisationInfo", HFILL }},
    { &hf_umts_rrc_pdu_def_ul_CommonTransChInfo,
      { "ul-CommonTransChInfo", "umts_rrc_pdu_def.ul_CommonTransChInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.UL_CommonTransChInfo", HFILL }},
    { &hf_umts_rrc_pdu_def_ul_deletedTransChInfoList,
      { "ul-deletedTransChInfoList", "umts_rrc_pdu_def.ul_deletedTransChInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.UL_DeletedTransChInfoList", HFILL }},
    { &hf_umts_rrc_pdu_def_ul_AddReconfTransChInfoList,
      { "ul-AddReconfTransChInfoList", "umts_rrc_pdu_def.ul_AddReconfTransChInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.UL_AddReconfTransChInfoList", HFILL }},
    { &hf_umts_rrc_pdu_def_modeSpecificTransChInfo,
      { "modeSpecificTransChInfo", "umts_rrc_pdu_def.modeSpecificTransChInfo",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_modeSpecificTransChInfo_vals), 0,
        "umts_rrc_pdu_def.T_modeSpecificTransChInfo", HFILL }},
    { &hf_umts_rrc_pdu_def_fdd,
      { "fdd", "umts_rrc_pdu_def.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_fdd", HFILL }},
    { &hf_umts_rrc_pdu_def_dummy_CPCH_SetID,
      { "dummy-CPCH-SetID", "umts_rrc_pdu_def.dummy_CPCH_SetID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.CPCH_SetID", HFILL }},
    { &hf_umts_rrc_pdu_def_dummy2_DRAC_StaticInformationList,
      { "dummy2-DRAC-StaticInformationList", "umts_rrc_pdu_def.dummy2_DRAC_StaticInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.DRAC_StaticInformationList", HFILL }},
    { &hf_umts_rrc_pdu_def_tdd,
      { "tdd", "umts_rrc_pdu_def.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.NULL", HFILL }},
    { &hf_umts_rrc_pdu_def_dl_CommonTransChInfo,
      { "dl-CommonTransChInfo", "umts_rrc_pdu_def.dl_CommonTransChInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.DL_CommonTransChInfo", HFILL }},
    { &hf_umts_rrc_pdu_def_dl_DeletedTransChInfoList,
      { "dl-DeletedTransChInfoList", "umts_rrc_pdu_def.dl_DeletedTransChInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.DL_DeletedTransChInfoList", HFILL }},
    { &hf_umts_rrc_pdu_def_dl_AddReconfTransChInfoList,
      { "dl-AddReconfTransChInfoList", "umts_rrc_pdu_def.dl_AddReconfTransChInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.DL_AddReconfTransChInfoList", HFILL }},
    { &hf_umts_rrc_pdu_def_frequencyInfo,
      { "frequencyInfo", "umts_rrc_pdu_def.frequencyInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.FrequencyInfo", HFILL }},
    { &hf_umts_rrc_pdu_def_ul_ChannelRequirement,
      { "ul-ChannelRequirement", "umts_rrc_pdu_def.ul_ChannelRequirement",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_UL_ChannelRequirement_vals), 0,
        "umts_rrc_ies.UL_ChannelRequirement", HFILL }},
    { &hf_umts_rrc_pdu_def_modeSpecificPhysChInfo,
      { "modeSpecificPhysChInfo", "umts_rrc_pdu_def.modeSpecificPhysChInfo",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_modeSpecificPhysChInfo_vals), 0,
        "umts_rrc_pdu_def.T_modeSpecificPhysChInfo", HFILL }},
    { &hf_umts_rrc_pdu_def_fdd1,
      { "fdd", "umts_rrc_pdu_def.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_fdd1", HFILL }},
    { &hf_umts_rrc_pdu_def_dummy_DL_PDSCH_Information,
      { "dummy-DL-PDSCH-Information", "umts_rrc_pdu_def.dummy_DL_PDSCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.DL_PDSCH_Information", HFILL }},
    { &hf_umts_rrc_pdu_def_dl_CommonInformation,
      { "dl-CommonInformation", "umts_rrc_pdu_def.dl_CommonInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.DL_CommonInformation", HFILL }},
    { &hf_umts_rrc_pdu_def_dl_InformationPerRL_List,
      { "dl-InformationPerRL-List", "umts_rrc_pdu_def.dl_InformationPerRL_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.DL_InformationPerRL_List", HFILL }},
    { &hf_umts_rrc_pdu_def_new_DSCH_RNTI,
      { "new-DSCH-RNTI", "umts_rrc_pdu_def.new_DSCH_RNTI",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_ies.DSCH_RNTI", HFILL }},
    { &hf_umts_rrc_pdu_def_pdcp_ROHC_TargetMode,
      { "pdcp-ROHC-TargetMode", "umts_rrc_pdu_def.pdcp_ROHC_TargetMode",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_PDCP_ROHC_TargetMode_vals), 0,
        "umts_rrc_ies.PDCP_ROHC_TargetMode", HFILL }},
    { &hf_umts_rrc_pdu_def_rb_InformationReconfigList1,
      { "rb-InformationReconfigList", "umts_rrc_pdu_def.rb_InformationReconfigList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.RB_InformationReconfigList_r4", HFILL }},
    { &hf_umts_rrc_pdu_def_ul_CommonTransChInfo1,
      { "ul-CommonTransChInfo", "umts_rrc_pdu_def.ul_CommonTransChInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.UL_CommonTransChInfo_r4", HFILL }},
    { &hf_umts_rrc_pdu_def_modeSpecificTransChInfo1,
      { "modeSpecificTransChInfo", "umts_rrc_pdu_def.modeSpecificTransChInfo",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_modeSpecificTransChInfo1_vals), 0,
        "umts_rrc_pdu_def.T_modeSpecificTransChInfo1", HFILL }},
    { &hf_umts_rrc_pdu_def_fdd2,
      { "fdd", "umts_rrc_pdu_def.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_fdd2", HFILL }},
    { &hf_umts_rrc_pdu_def_dl_CommonTransChInfo1,
      { "dl-CommonTransChInfo", "umts_rrc_pdu_def.dl_CommonTransChInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.DL_CommonTransChInfo_r4", HFILL }},
    { &hf_umts_rrc_pdu_def_dl_AddReconfTransChInfoList1,
      { "dl-AddReconfTransChInfoList", "umts_rrc_pdu_def.dl_AddReconfTransChInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.DL_AddReconfTransChInfoList_r4", HFILL }},
    { &hf_umts_rrc_pdu_def_ul_ChannelRequirement1,
      { "ul-ChannelRequirement", "umts_rrc_pdu_def.ul_ChannelRequirement",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_UL_ChannelRequirement_r4_vals), 0,
        "umts_rrc_ies.UL_ChannelRequirement_r4", HFILL }},
    { &hf_umts_rrc_pdu_def_modeSpecificPhysChInfo1,
      { "modeSpecificPhysChInfo", "umts_rrc_pdu_def.modeSpecificPhysChInfo",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_modeSpecificPhysChInfo1_vals), 0,
        "umts_rrc_pdu_def.T_modeSpecificPhysChInfo1", HFILL }},
    { &hf_umts_rrc_pdu_def_fdd3,
      { "fdd", "umts_rrc_pdu_def.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_fdd3", HFILL }},
    { &hf_umts_rrc_pdu_def_dl_CommonInformation1,
      { "dl-CommonInformation", "umts_rrc_pdu_def.dl_CommonInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.DL_CommonInformation_r4", HFILL }},
    { &hf_umts_rrc_pdu_def_dl_InformationPerRL_List1,
      { "dl-InformationPerRL-List", "umts_rrc_pdu_def.dl_InformationPerRL_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.DL_InformationPerRL_List_r4", HFILL }},
    { &hf_umts_rrc_pdu_def_new_H_RNTI,
      { "new-H-RNTI", "umts_rrc_pdu_def.new_H_RNTI",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_ies.H_RNTI", HFILL }},
    { &hf_umts_rrc_pdu_def_rb_InformationReconfigList2,
      { "rb-InformationReconfigList", "umts_rrc_pdu_def.rb_InformationReconfigList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.RB_InformationReconfigList_r5", HFILL }},
    { &hf_umts_rrc_pdu_def_rb_InformationAffectedList1,
      { "rb-InformationAffectedList", "umts_rrc_pdu_def.rb_InformationAffectedList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.RB_InformationAffectedList_r5", HFILL }},
    { &hf_umts_rrc_pdu_def_dl_CounterSynchronisationInfo1,
      { "dl-CounterSynchronisationInfo", "umts_rrc_pdu_def.dl_CounterSynchronisationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.DL_CounterSynchronisationInfo_r5", HFILL }},
    { &hf_umts_rrc_pdu_def_modeSpecificTransChInfo2,
      { "modeSpecificTransChInfo", "umts_rrc_pdu_def.modeSpecificTransChInfo",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_modeSpecificTransChInfo2_vals), 0,
        "umts_rrc_pdu_def.T_modeSpecificTransChInfo2", HFILL }},
    { &hf_umts_rrc_pdu_def_fdd4,
      { "fdd", "umts_rrc_pdu_def.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_fdd4", HFILL }},
    { &hf_umts_rrc_pdu_def_dl_DeletedTransChInfoList1,
      { "dl-DeletedTransChInfoList", "umts_rrc_pdu_def.dl_DeletedTransChInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.DL_DeletedTransChInfoList_r5", HFILL }},
    { &hf_umts_rrc_pdu_def_dl_AddReconfTransChInfoList2,
      { "dl-AddReconfTransChInfoList", "umts_rrc_pdu_def.dl_AddReconfTransChInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.DL_AddReconfTransChInfoList_r5", HFILL }},
    { &hf_umts_rrc_pdu_def_ul_ChannelRequirement2,
      { "ul-ChannelRequirement", "umts_rrc_pdu_def.ul_ChannelRequirement",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_UL_ChannelRequirement_r5_vals), 0,
        "umts_rrc_ies.UL_ChannelRequirement_r5", HFILL }},
    { &hf_umts_rrc_pdu_def_modeSpecificPhysChInfo2,
      { "modeSpecificPhysChInfo", "umts_rrc_pdu_def.modeSpecificPhysChInfo",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_modeSpecificPhysChInfo2_vals), 0,
        "umts_rrc_pdu_def.T_modeSpecificPhysChInfo2", HFILL }},
    { &hf_umts_rrc_pdu_def_fdd5,
      { "fdd", "umts_rrc_pdu_def.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_fdd5", HFILL }},
    { &hf_umts_rrc_pdu_def_dl_HSPDSCH_Information,
      { "dl-HSPDSCH-Information", "umts_rrc_pdu_def.dl_HSPDSCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.DL_HSPDSCH_Information", HFILL }},
    { &hf_umts_rrc_pdu_def_dl_CommonInformation2,
      { "dl-CommonInformation", "umts_rrc_pdu_def.dl_CommonInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.DL_CommonInformation_r5", HFILL }},
    { &hf_umts_rrc_pdu_def_dl_InformationPerRL_List2,
      { "dl-InformationPerRL-List", "umts_rrc_pdu_def.dl_InformationPerRL_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.DL_InformationPerRL_List_r5", HFILL }},
    { &hf_umts_rrc_pdu_def_rb_InformationReconfigList3,
      { "rb-InformationReconfigList", "umts_rrc_pdu_def.rb_InformationReconfigList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.RB_InformationReconfigList_r6", HFILL }},
    { &hf_umts_rrc_pdu_def_rb_InformationAffectedList2,
      { "rb-InformationAffectedList", "umts_rrc_pdu_def.rb_InformationAffectedList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.RB_InformationAffectedList_r6", HFILL }},
    { &hf_umts_rrc_pdu_def_ul_deletedTransChInfoList1,
      { "ul-deletedTransChInfoList", "umts_rrc_pdu_def.ul_deletedTransChInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.UL_DeletedTransChInfoList_r6", HFILL }},
    { &hf_umts_rrc_pdu_def_ul_AddReconfTransChInfoList1,
      { "ul-AddReconfTransChInfoList", "umts_rrc_pdu_def.ul_AddReconfTransChInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.UL_AddReconfTransChInfoList_r6", HFILL }},
    { &hf_umts_rrc_pdu_def_ul_ChannelRequirement3,
      { "ul-ChannelRequirement", "umts_rrc_pdu_def.ul_ChannelRequirement",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.UL_ChannelRequirement_r6", HFILL }},
    { &hf_umts_rrc_pdu_def_ul_EDCH_Information,
      { "ul-EDCH-Information", "umts_rrc_pdu_def.ul_EDCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.UL_EDCH_Information_r6", HFILL }},
    { &hf_umts_rrc_pdu_def_dl_HSPDSCH_Information1,
      { "dl-HSPDSCH-Information", "umts_rrc_pdu_def.dl_HSPDSCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.DL_HSPDSCH_Information_r6", HFILL }},
    { &hf_umts_rrc_pdu_def_dl_CommonInformation3,
      { "dl-CommonInformation", "umts_rrc_pdu_def.dl_CommonInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.DL_CommonInformation_r6", HFILL }},
    { &hf_umts_rrc_pdu_def_dl_InformationPerRL_List3,
      { "dl-InformationPerRL-List", "umts_rrc_pdu_def.dl_InformationPerRL_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.DL_InformationPerRL_List_r6", HFILL }},
    { &hf_umts_rrc_pdu_def_mbms_PL_ServiceRestrictInfo,
      { "mbms-PL-ServiceRestrictInfo", "umts_rrc_pdu_def.mbms_PL_ServiceRestrictInfo",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_MBMS_PL_ServiceRestrictInfo_r6_vals), 0,
        "umts_rrc_ies.MBMS_PL_ServiceRestrictInfo_r6", HFILL }},
    { &hf_umts_rrc_pdu_def_harq_Preamble_Mode,
      { "harq-Preamble-Mode", "umts_rrc_pdu_def.harq_Preamble_Mode",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.HARQ_Preamble_Mode", HFILL }},
    { &hf_umts_rrc_pdu_def_beaconPLEst,
      { "beaconPLEst", "umts_rrc_pdu_def.beaconPLEst",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_BEACON_PL_Est_vals), 0,
        "umts_rrc_ies.BEACON_PL_Est", HFILL }},
    { &hf_umts_rrc_pdu_def_postVerificationPeriod,
      { "postVerificationPeriod", "umts_rrc_pdu_def.postVerificationPeriod",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_postVerificationPeriod_vals), 0,
        "umts_rrc_pdu_def.T_postVerificationPeriod", HFILL }},
    { &hf_umts_rrc_pdu_def_dhs_sync,
      { "dhs-sync", "umts_rrc_pdu_def.dhs_sync",
        FT_INT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.DHS_Sync", HFILL }},
    { &hf_umts_rrc_pdu_def_r35,
      { "r3", "umts_rrc_pdu_def.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r35", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions8,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions8", HFILL }},
    { &hf_umts_rrc_pdu_def_cellUpdateConfirm_CCCH_r3_add_ext,
      { "cellUpdateConfirm-CCCH-r3-add-ext", "umts_rrc_pdu_def.cellUpdateConfirm_CCCH_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions3,
      { "v4b0NonCriticalExtensions", "umts_rrc_pdu_def.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v4b0NonCriticalExtensions3", HFILL }},
    { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions3,
      { "v590NonCriticalExtensions", "umts_rrc_pdu_def.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v590NonCriticalExtensions3", HFILL }},
    { &hf_umts_rrc_pdu_def_v5d0NonCriticalExtenstions3,
      { "v5d0NonCriticalExtenstions", "umts_rrc_pdu_def.v5d0NonCriticalExtenstions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v5d0NonCriticalExtenstions3", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions5,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions5", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions12,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions12", HFILL }},
    { &hf_umts_rrc_pdu_def_later_than_r34,
      { "later-than-r3", "umts_rrc_pdu_def.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_later_than_r34", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions9,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_criticalExtensions9_vals), 0,
        "umts_rrc_pdu_def.T_criticalExtensions9", HFILL }},
    { &hf_umts_rrc_pdu_def_r41,
      { "r4", "umts_rrc_pdu_def.r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r41", HFILL }},
    { &hf_umts_rrc_pdu_def_v4d0NonCriticalExtensions1,
      { "v4d0NonCriticalExtensions", "umts_rrc_pdu_def.v4d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v4d0NonCriticalExtensions1", HFILL }},
    { &hf_umts_rrc_pdu_def_cellUpdateConfirm_CCCH_r4_add_ext,
      { "cellUpdateConfirm-CCCH-r4-add-ext", "umts_rrc_pdu_def.cellUpdateConfirm_CCCH_r4_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions4,
      { "v590NonCriticalExtensions", "umts_rrc_pdu_def.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v590NonCriticalExtensions4", HFILL }},
    { &hf_umts_rrc_pdu_def_v5d0NonCriticalExtenstions4,
      { "v5d0NonCriticalExtenstions", "umts_rrc_pdu_def.v5d0NonCriticalExtenstions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v5d0NonCriticalExtenstions4", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions6,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions6", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions13,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions13", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions10,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_criticalExtensions10_vals), 0,
        "umts_rrc_pdu_def.T_criticalExtensions10", HFILL }},
    { &hf_umts_rrc_pdu_def_r51,
      { "r5", "umts_rrc_pdu_def.r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r51", HFILL }},
    { &hf_umts_rrc_pdu_def_cellUpdateConfirm_CCCH_r5_add_ext,
      { "cellUpdateConfirm-CCCH-r5-add-ext", "umts_rrc_pdu_def.cellUpdateConfirm_CCCH_r5_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v5d0NonCriticalExtenstions5,
      { "v5d0NonCriticalExtenstions", "umts_rrc_pdu_def.v5d0NonCriticalExtenstions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v5d0NonCriticalExtenstions5", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions7,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions7", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions14,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions14", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions11,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_criticalExtensions11_vals), 0,
        "umts_rrc_pdu_def.T_criticalExtensions11", HFILL }},
    { &hf_umts_rrc_pdu_def_r62,
      { "r6", "umts_rrc_pdu_def.r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r62", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions15,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions15", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions12,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_criticalExtensions12", HFILL }},
    { &hf_umts_rrc_pdu_def_r36,
      { "r3", "umts_rrc_pdu_def.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r36", HFILL }},
    { &hf_umts_rrc_pdu_def_counterCheck_r3,
      { "counterCheck-r3", "umts_rrc_pdu_def.counterCheck_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.CounterCheck_r3_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions9,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions9", HFILL }},
    { &hf_umts_rrc_pdu_def_counterCheck_r3_add_ext,
      { "counterCheck-r3-add-ext", "umts_rrc_pdu_def.counterCheck_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions16,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions16", HFILL }},
    { &hf_umts_rrc_pdu_def_later_than_r35,
      { "later-than-r3", "umts_rrc_pdu_def.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_later_than_r35", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions13,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_criticalExtensions13", HFILL }},
    { &hf_umts_rrc_pdu_def_rb_COUNT_C_MSB_InformationList,
      { "rb-COUNT-C-MSB-InformationList", "umts_rrc_pdu_def.rb_COUNT_C_MSB_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.RB_COUNT_C_MSB_InformationList", HFILL }},
    { &hf_umts_rrc_pdu_def_rb_COUNT_C_InformationList,
      { "rb-COUNT-C-InformationList", "umts_rrc_pdu_def.rb_COUNT_C_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.RB_COUNT_C_InformationList", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions10,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions10", HFILL }},
    { &hf_umts_rrc_pdu_def_counterCheckResponse_r3_add_ext,
      { "counterCheckResponse-r3-add-ext", "umts_rrc_pdu_def.counterCheckResponse_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions17,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions17", HFILL }},
    { &hf_umts_rrc_pdu_def_r37,
      { "r3", "umts_rrc_pdu_def.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r37", HFILL }},
    { &hf_umts_rrc_pdu_def_downlinkDirectTransfer_r3,
      { "downlinkDirectTransfer-r3", "umts_rrc_pdu_def.downlinkDirectTransfer_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.DownlinkDirectTransfer_r3_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions11,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions11", HFILL }},
    { &hf_umts_rrc_pdu_def_downlinkDirectTransfer_r3_add_ext,
      { "downlinkDirectTransfer-r3-add-ext", "umts_rrc_pdu_def.downlinkDirectTransfer_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions18,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions18", HFILL }},
    { &hf_umts_rrc_pdu_def_later_than_r36,
      { "later-than-r3", "umts_rrc_pdu_def.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_later_than_r36", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions14,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_criticalExtensions14", HFILL }},
    { &hf_umts_rrc_pdu_def_cn_DomainIdentity,
      { "cn-DomainIdentity", "umts_rrc_pdu_def.cn_DomainIdentity",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_CN_DomainIdentity_vals), 0,
        "umts_rrc_ies.CN_DomainIdentity", HFILL }},
    { &hf_umts_rrc_pdu_def_nas_Message,
      { "nas-Message", "umts_rrc_pdu_def.nas_Message",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_ies.NAS_Message", HFILL }},
    { &hf_umts_rrc_pdu_def_r38,
      { "r3", "umts_rrc_pdu_def.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r38", HFILL }},
    { &hf_umts_rrc_pdu_def_handoverToUTRANCommand_r3,
      { "handoverToUTRANCommand-r3", "umts_rrc_pdu_def.handoverToUTRANCommand_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.HandoverToUTRANCommand_r3_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions19,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions19", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions15,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_criticalExtensions15_vals), 0,
        "umts_rrc_pdu_def.T_criticalExtensions15", HFILL }},
    { &hf_umts_rrc_pdu_def_r42,
      { "r4", "umts_rrc_pdu_def.r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r42", HFILL }},
    { &hf_umts_rrc_pdu_def_handoverToUTRANCommand_r4,
      { "handoverToUTRANCommand-r4", "umts_rrc_pdu_def.handoverToUTRANCommand_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.HandoverToUTRANCommand_r4_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions20,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions20", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions16,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_criticalExtensions16_vals), 0,
        "umts_rrc_pdu_def.T_criticalExtensions16", HFILL }},
    { &hf_umts_rrc_pdu_def_r52,
      { "r5", "umts_rrc_pdu_def.r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r52", HFILL }},
    { &hf_umts_rrc_pdu_def_handoverToUTRANCommand_r5,
      { "handoverToUTRANCommand-r5", "umts_rrc_pdu_def.handoverToUTRANCommand_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.HandoverToUTRANCommand_r5_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions21,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions21", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions17,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_criticalExtensions17_vals), 0,
        "umts_rrc_pdu_def.T_criticalExtensions17", HFILL }},
    { &hf_umts_rrc_pdu_def_r63,
      { "r6", "umts_rrc_pdu_def.r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r63", HFILL }},
    { &hf_umts_rrc_pdu_def_handoverToUTRANCommand_r6,
      { "handoverToUTRANCommand-r6", "umts_rrc_pdu_def.handoverToUTRANCommand_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.HandoverToUTRANCommand_r6_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions22,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions22", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions18,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_criticalExtensions18", HFILL }},
    { &hf_umts_rrc_pdu_def_new_U_RNTI1,
      { "new-U-RNTI", "umts_rrc_pdu_def.new_U_RNTI",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.U_RNTI_Short", HFILL }},
    { &hf_umts_rrc_pdu_def_dummy_ActivationTime,
      { "dummy-ActivationTime", "umts_rrc_pdu_def.dummy_ActivationTime",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.ActivationTime", HFILL }},
    { &hf_umts_rrc_pdu_def_cipheringAlgorithm,
      { "cipheringAlgorithm", "umts_rrc_pdu_def.cipheringAlgorithm",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_CipheringAlgorithm_vals), 0,
        "umts_rrc_ies.CipheringAlgorithm", HFILL }},
    { &hf_umts_rrc_pdu_def_specificationMode,
      { "specificationMode", "umts_rrc_pdu_def.specificationMode",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_specificationMode_vals), 0,
        "umts_rrc_pdu_def.T_specificationMode", HFILL }},
    { &hf_umts_rrc_pdu_def_complete,
      { "complete", "umts_rrc_pdu_def.complete",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_complete", HFILL }},
    { &hf_umts_rrc_pdu_def_srb_InformationSetupList,
      { "srb-InformationSetupList", "umts_rrc_pdu_def.srb_InformationSetupList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.SRB_InformationSetupList", HFILL }},
    { &hf_umts_rrc_pdu_def_rab_InformationSetupList,
      { "rab-InformationSetupList", "umts_rrc_pdu_def.rab_InformationSetupList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.RAB_InformationSetupList", HFILL }},
    { &hf_umts_rrc_pdu_def_ul_DPCH_Info,
      { "ul-DPCH-Info", "umts_rrc_pdu_def.ul_DPCH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.UL_DPCH_Info", HFILL }},
    { &hf_umts_rrc_pdu_def_modeSpecificInfo,
      { "modeSpecificInfo", "umts_rrc_pdu_def.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_modeSpecificInfo_vals), 0,
        "umts_rrc_pdu_def.T_modeSpecificInfo", HFILL }},
    { &hf_umts_rrc_pdu_def_fdd6,
      { "fdd", "umts_rrc_pdu_def.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_fdd6", HFILL }},
    { &hf_umts_rrc_pdu_def_dummy2_CPCH_SetInfo,
      { "dummy2-CPCH-SetInfo", "umts_rrc_pdu_def.dummy2_CPCH_SetInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.CPCH_SetInfo", HFILL }},
    { &hf_umts_rrc_pdu_def_preconfiguration,
      { "preconfiguration", "umts_rrc_pdu_def.preconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_preconfiguration", HFILL }},
    { &hf_umts_rrc_pdu_def_preConfigMode,
      { "preConfigMode", "umts_rrc_pdu_def.preConfigMode",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_preConfigMode_vals), 0,
        "umts_rrc_pdu_def.T_preConfigMode", HFILL }},
    { &hf_umts_rrc_pdu_def_predefinedConfigIdentity,
      { "predefinedConfigIdentity", "umts_rrc_pdu_def.predefinedConfigIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.PredefinedConfigIdentity", HFILL }},
    { &hf_umts_rrc_pdu_def_defaultConfig,
      { "defaultConfig", "umts_rrc_pdu_def.defaultConfig",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_defaultConfig", HFILL }},
    { &hf_umts_rrc_pdu_def_defaultConfigMode,
      { "defaultConfigMode", "umts_rrc_pdu_def.defaultConfigMode",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_DefaultConfigMode_vals), 0,
        "umts_rrc_ies.DefaultConfigMode", HFILL }},
    { &hf_umts_rrc_pdu_def_defaultConfigIdentity,
      { "defaultConfigIdentity", "umts_rrc_pdu_def.defaultConfigIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.DefaultConfigIdentity", HFILL }},
    { &hf_umts_rrc_pdu_def_rab_Info,
      { "rab-Info", "umts_rrc_pdu_def.rab_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.RAB_Info_Post", HFILL }},
    { &hf_umts_rrc_pdu_def_modeSpecificInfo1,
      { "modeSpecificInfo", "umts_rrc_pdu_def.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_modeSpecificInfo1_vals), 0,
        "umts_rrc_pdu_def.T_modeSpecificInfo1", HFILL }},
    { &hf_umts_rrc_pdu_def_fdd7,
      { "fdd", "umts_rrc_pdu_def.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_fdd7", HFILL }},
    { &hf_umts_rrc_pdu_def_ul_DPCH_Info1,
      { "ul-DPCH-Info", "umts_rrc_pdu_def.ul_DPCH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.UL_DPCH_InfoPostFDD", HFILL }},
    { &hf_umts_rrc_pdu_def_dl_CommonInformationPost,
      { "dl-CommonInformationPost", "umts_rrc_pdu_def.dl_CommonInformationPost",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.DL_CommonInformationPost", HFILL }},
    { &hf_umts_rrc_pdu_def_dl_InformationPerRL_List4,
      { "dl-InformationPerRL-List", "umts_rrc_pdu_def.dl_InformationPerRL_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.DL_InformationPerRL_ListPostFDD", HFILL }},
    { &hf_umts_rrc_pdu_def_frequencyInfo1,
      { "frequencyInfo", "umts_rrc_pdu_def.frequencyInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.FrequencyInfoFDD", HFILL }},
    { &hf_umts_rrc_pdu_def_tdd1,
      { "tdd", "umts_rrc_pdu_def.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_tdd", HFILL }},
    { &hf_umts_rrc_pdu_def_ul_DPCH_Info2,
      { "ul-DPCH-Info", "umts_rrc_pdu_def.ul_DPCH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.UL_DPCH_InfoPostTDD", HFILL }},
    { &hf_umts_rrc_pdu_def_dl_InformationPerRL,
      { "dl-InformationPerRL", "umts_rrc_pdu_def.dl_InformationPerRL",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.DL_InformationPerRL_PostTDD", HFILL }},
    { &hf_umts_rrc_pdu_def_frequencyInfo2,
      { "frequencyInfo", "umts_rrc_pdu_def.frequencyInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.FrequencyInfoTDD", HFILL }},
    { &hf_umts_rrc_pdu_def_primaryCCPCH_TX_Power,
      { "primaryCCPCH-TX-Power", "umts_rrc_pdu_def.primaryCCPCH_TX_Power",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.PrimaryCCPCH_TX_Power", HFILL }},
    { &hf_umts_rrc_pdu_def_specificationMode1,
      { "specificationMode", "umts_rrc_pdu_def.specificationMode",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_specificationMode1_vals), 0,
        "umts_rrc_pdu_def.T_specificationMode1", HFILL }},
    { &hf_umts_rrc_pdu_def_complete1,
      { "complete", "umts_rrc_pdu_def.complete",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_complete1", HFILL }},
    { &hf_umts_rrc_pdu_def_rab_InformationSetupList1,
      { "rab-InformationSetupList", "umts_rrc_pdu_def.rab_InformationSetupList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.RAB_InformationSetupList_r4", HFILL }},
    { &hf_umts_rrc_pdu_def_ul_DPCH_Info3,
      { "ul-DPCH-Info", "umts_rrc_pdu_def.ul_DPCH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.UL_DPCH_Info_r4", HFILL }},
    { &hf_umts_rrc_pdu_def_modeSpecificInfo2,
      { "modeSpecificInfo", "umts_rrc_pdu_def.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_modeSpecificInfo2_vals), 0,
        "umts_rrc_pdu_def.T_modeSpecificInfo2", HFILL }},
    { &hf_umts_rrc_pdu_def_fdd8,
      { "fdd", "umts_rrc_pdu_def.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_fdd8", HFILL }},
    { &hf_umts_rrc_pdu_def_preconfiguration1,
      { "preconfiguration", "umts_rrc_pdu_def.preconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_preconfiguration1", HFILL }},
    { &hf_umts_rrc_pdu_def_preConfigMode1,
      { "preConfigMode", "umts_rrc_pdu_def.preConfigMode",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_preConfigMode1_vals), 0,
        "umts_rrc_pdu_def.T_preConfigMode1", HFILL }},
    { &hf_umts_rrc_pdu_def_defaultConfig1,
      { "defaultConfig", "umts_rrc_pdu_def.defaultConfig",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_defaultConfig1", HFILL }},
    { &hf_umts_rrc_pdu_def_defaultConfigIdentity1,
      { "defaultConfigIdentity", "umts_rrc_pdu_def.defaultConfigIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.DefaultConfigIdentity_r4", HFILL }},
    { &hf_umts_rrc_pdu_def_modeSpecificInfo3,
      { "modeSpecificInfo", "umts_rrc_pdu_def.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_modeSpecificInfo3_vals), 0,
        "umts_rrc_pdu_def.T_modeSpecificInfo3", HFILL }},
    { &hf_umts_rrc_pdu_def_fdd9,
      { "fdd", "umts_rrc_pdu_def.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_fdd9", HFILL }},
    { &hf_umts_rrc_pdu_def_tdd2,
      { "tdd", "umts_rrc_pdu_def.tdd",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_tdd1_vals), 0,
        "umts_rrc_pdu_def.T_tdd1", HFILL }},
    { &hf_umts_rrc_pdu_def_tdd384,
      { "tdd384", "umts_rrc_pdu_def.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_tdd384", HFILL }},
    { &hf_umts_rrc_pdu_def_tdd128,
      { "tdd128", "umts_rrc_pdu_def.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_tdd128", HFILL }},
    { &hf_umts_rrc_pdu_def_ul_DPCH_Info4,
      { "ul-DPCH-Info", "umts_rrc_pdu_def.ul_DPCH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.UL_DPCH_InfoPostTDD_LCR_r4", HFILL }},
    { &hf_umts_rrc_pdu_def_dl_InformationPerRL1,
      { "dl-InformationPerRL", "umts_rrc_pdu_def.dl_InformationPerRL",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.DL_InformationPerRL_PostTDD_LCR_r4", HFILL }},
    { &hf_umts_rrc_pdu_def_specificationMode2,
      { "specificationMode", "umts_rrc_pdu_def.specificationMode",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_specificationMode2_vals), 0,
        "umts_rrc_pdu_def.T_specificationMode2", HFILL }},
    { &hf_umts_rrc_pdu_def_complete2,
      { "complete", "umts_rrc_pdu_def.complete",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_complete2", HFILL }},
    { &hf_umts_rrc_pdu_def_srb_InformationSetupList1,
      { "srb-InformationSetupList", "umts_rrc_pdu_def.srb_InformationSetupList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.SRB_InformationSetupList_r5", HFILL }},
    { &hf_umts_rrc_pdu_def_rab_InformationSetupList2,
      { "rab-InformationSetupList", "umts_rrc_pdu_def.rab_InformationSetupList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.RAB_InformationSetupList_r5", HFILL }},
    { &hf_umts_rrc_pdu_def_ul_DPCH_Info5,
      { "ul-DPCH-Info", "umts_rrc_pdu_def.ul_DPCH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.UL_DPCH_Info_r5", HFILL }},
    { &hf_umts_rrc_pdu_def_modeSpecificInfo4,
      { "modeSpecificInfo", "umts_rrc_pdu_def.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_modeSpecificInfo4_vals), 0,
        "umts_rrc_pdu_def.T_modeSpecificInfo4", HFILL }},
    { &hf_umts_rrc_pdu_def_fdd10,
      { "fdd", "umts_rrc_pdu_def.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_fdd10", HFILL }},
    { &hf_umts_rrc_pdu_def_preconfiguration2,
      { "preconfiguration", "umts_rrc_pdu_def.preconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_preconfiguration2", HFILL }},
    { &hf_umts_rrc_pdu_def_preConfigMode2,
      { "preConfigMode", "umts_rrc_pdu_def.preConfigMode",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_preConfigMode2_vals), 0,
        "umts_rrc_pdu_def.T_preConfigMode2", HFILL }},
    { &hf_umts_rrc_pdu_def_defaultConfig2,
      { "defaultConfig", "umts_rrc_pdu_def.defaultConfig",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_defaultConfig2", HFILL }},
    { &hf_umts_rrc_pdu_def_defaultConfigIdentity2,
      { "defaultConfigIdentity", "umts_rrc_pdu_def.defaultConfigIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.DefaultConfigIdentity_r5", HFILL }},
    { &hf_umts_rrc_pdu_def_modeSpecificInfo5,
      { "modeSpecificInfo", "umts_rrc_pdu_def.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_modeSpecificInfo5_vals), 0,
        "umts_rrc_pdu_def.T_modeSpecificInfo5", HFILL }},
    { &hf_umts_rrc_pdu_def_fdd11,
      { "fdd", "umts_rrc_pdu_def.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_fdd11", HFILL }},
    { &hf_umts_rrc_pdu_def_tdd3,
      { "tdd", "umts_rrc_pdu_def.tdd",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_tdd2_vals), 0,
        "umts_rrc_pdu_def.T_tdd2", HFILL }},
    { &hf_umts_rrc_pdu_def_tdd3841,
      { "tdd384", "umts_rrc_pdu_def.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_tdd3841", HFILL }},
    { &hf_umts_rrc_pdu_def_tdd1281,
      { "tdd128", "umts_rrc_pdu_def.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_tdd1281", HFILL }},
    { &hf_umts_rrc_pdu_def_specificationMode3,
      { "specificationMode", "umts_rrc_pdu_def.specificationMode",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_specificationMode3_vals), 0,
        "umts_rrc_pdu_def.T_specificationMode3", HFILL }},
    { &hf_umts_rrc_pdu_def_complete3,
      { "complete", "umts_rrc_pdu_def.complete",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_complete3", HFILL }},
    { &hf_umts_rrc_pdu_def_srb_InformationSetupList2,
      { "srb-InformationSetupList", "umts_rrc_pdu_def.srb_InformationSetupList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.SRB_InformationSetupList_r6", HFILL }},
    { &hf_umts_rrc_pdu_def_rab_InformationSetupList3,
      { "rab-InformationSetupList", "umts_rrc_pdu_def.rab_InformationSetupList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.RAB_InformationSetupList_r6", HFILL }},
    { &hf_umts_rrc_pdu_def_ul_DPCH_Info6,
      { "ul-DPCH-Info", "umts_rrc_pdu_def.ul_DPCH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.UL_DPCH_Info_r6", HFILL }},
    { &hf_umts_rrc_pdu_def_count_C_ActivationTime,
      { "count-C-ActivationTime", "umts_rrc_pdu_def.count_C_ActivationTime",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.ActivationTime", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions12,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions12", HFILL }},
    { &hf_umts_rrc_pdu_def_handoverToUTRANComplete_r3_add_ext,
      { "handoverToUTRANComplete-r3-add-ext", "umts_rrc_pdu_def.handoverToUTRANComplete_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions23,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions23", HFILL }},
    { &hf_umts_rrc_pdu_def_intraDomainNasNodeSelector,
      { "intraDomainNasNodeSelector", "umts_rrc_pdu_def.intraDomainNasNodeSelector",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.IntraDomainNasNodeSelector", HFILL }},
    { &hf_umts_rrc_pdu_def_v3a0NonCriticalExtensions2,
      { "v3a0NonCriticalExtensions", "umts_rrc_pdu_def.v3a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v3a0NonCriticalExtensions2", HFILL }},
    { &hf_umts_rrc_pdu_def_initialDirectTransfer_v3a0ext,
      { "initialDirectTransfer-v3a0ext", "umts_rrc_pdu_def.initialDirectTransfer_v3a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.InitialDirectTransfer_v3a0ext", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions13,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions13", HFILL }},
    { &hf_umts_rrc_pdu_def_initialDirectTransfer_r3_add_ext,
      { "initialDirectTransfer-r3-add-ext", "umts_rrc_pdu_def.initialDirectTransfer_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions5,
      { "v590NonCriticalExtensions", "umts_rrc_pdu_def.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v590NonCriticalExtensions5", HFILL }},
    { &hf_umts_rrc_pdu_def_initialDirectTransfer_v590ext,
      { "initialDirectTransfer-v590ext", "umts_rrc_pdu_def.initialDirectTransfer_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.InitialDirectTransfer_v590ext", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions8,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions8", HFILL }},
    { &hf_umts_rrc_pdu_def_initialDirectTransfer_v6xyext,
      { "initialDirectTransfer-v6xyext", "umts_rrc_pdu_def.initialDirectTransfer_v6xyext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.InitialDirectTransfer_v6xyext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions24,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions24", HFILL }},
    { &hf_umts_rrc_pdu_def_start_Value,
      { "start-Value", "umts_rrc_pdu_def.start_Value",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_ies.START_Value", HFILL }},
    { &hf_umts_rrc_pdu_def_plmn_Identity,
      { "plmn-Identity", "umts_rrc_pdu_def.plmn_Identity",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.PLMN_Identity", HFILL }},
    { &hf_umts_rrc_pdu_def_mbms_JoinedInformation,
      { "mbms-JoinedInformation", "umts_rrc_pdu_def.mbms_JoinedInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.MBMS_JoinedInformation_r6", HFILL }},
    { &hf_umts_rrc_pdu_def_r39,
      { "r3", "umts_rrc_pdu_def.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r39", HFILL }},
    { &hf_umts_rrc_pdu_def_handoverFromUTRANCommand_GSM_r3,
      { "handoverFromUTRANCommand-GSM-r3", "umts_rrc_pdu_def.handoverFromUTRANCommand_GSM_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.HandoverFromUTRANCommand_GSM_r3_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions14,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions14", HFILL }},
    { &hf_umts_rrc_pdu_def_handoverFromUTRANCommand_GSM_r3_add_ext,
      { "handoverFromUTRANCommand-GSM-r3-add-ext", "umts_rrc_pdu_def.handoverFromUTRANCommand_GSM_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions9,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions9", HFILL }},
    { &hf_umts_rrc_pdu_def_handoverFromUTRANCommand_GSM_v6xyext,
      { "handoverFromUTRANCommand-GSM-v6xyext", "umts_rrc_pdu_def.handoverFromUTRANCommand_GSM_v6xyext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.HandoverFromUTRANCommand_GSM_v6xyext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions25,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions25", HFILL }},
    { &hf_umts_rrc_pdu_def_later_than_r37,
      { "later-than-r3", "umts_rrc_pdu_def.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_later_than_r37", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions19,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_criticalExtensions19_vals), 0,
        "umts_rrc_pdu_def.T_criticalExtensions19", HFILL }},
    { &hf_umts_rrc_pdu_def_r64,
      { "r6", "umts_rrc_pdu_def.r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r64", HFILL }},
    { &hf_umts_rrc_pdu_def_handoverFromUTRANCommand_GSM_r6,
      { "handoverFromUTRANCommand-GSM-r6", "umts_rrc_pdu_def.handoverFromUTRANCommand_GSM_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.HandoverFromUTRANCommand_GSM_r6_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_handoverFromUTRANCommand_GSM_r6_add_ext,
      { "handoverFromUTRANCommand-GSM-r6-add-ext", "umts_rrc_pdu_def.handoverFromUTRANCommand_GSM_r6_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions26,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions26", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions20,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_criticalExtensions20", HFILL }},
    { &hf_umts_rrc_pdu_def_toHandoverRAB_Info,
      { "toHandoverRAB-Info", "umts_rrc_pdu_def.toHandoverRAB_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.RAB_Info", HFILL }},
    { &hf_umts_rrc_pdu_def_frequency_band,
      { "frequency-band", "umts_rrc_pdu_def.frequency_band",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_Frequency_Band_vals), 0,
        "umts_rrc_ies.Frequency_Band", HFILL }},
    { &hf_umts_rrc_pdu_def_gsm_message,
      { "gsm-message", "umts_rrc_pdu_def.gsm_message",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_gsm_message_vals), 0,
        "umts_rrc_pdu_def.T_gsm_message", HFILL }},
    { &hf_umts_rrc_pdu_def_single_GSM_Message,
      { "single-GSM-Message", "umts_rrc_pdu_def.single_GSM_Message",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_single_GSM_Message", HFILL }},
    { &hf_umts_rrc_pdu_def_gsm_MessageList,
      { "gsm-MessageList", "umts_rrc_pdu_def.gsm_MessageList",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_gsm_MessageList", HFILL }},
    { &hf_umts_rrc_pdu_def_gsm_Messages,
      { "gsm-Messages", "umts_rrc_pdu_def.gsm_Messages",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.GSM_MessageList", HFILL }},
    { &hf_umts_rrc_pdu_def_toHandoverRAB_Info1,
      { "toHandoverRAB-Info", "umts_rrc_pdu_def.toHandoverRAB_Info",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.RAB_InformationList_r6", HFILL }},
    { &hf_umts_rrc_pdu_def_gsm_message1,
      { "gsm-message", "umts_rrc_pdu_def.gsm_message",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_gsm_message1_vals), 0,
        "umts_rrc_pdu_def.T_gsm_message1", HFILL }},
    { &hf_umts_rrc_pdu_def_single_GSM_Message1,
      { "single-GSM-Message", "umts_rrc_pdu_def.single_GSM_Message",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_single_GSM_Message1", HFILL }},
    { &hf_umts_rrc_pdu_def_gsm_MessageList1,
      { "gsm-MessageList", "umts_rrc_pdu_def.gsm_MessageList",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_gsm_MessageList1", HFILL }},
    { &hf_umts_rrc_pdu_def_geran_SystemInfoType1,
      { "geran-SystemInfoType", "umts_rrc_pdu_def.geran_SystemInfoType",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_geran_SystemInfoType1_vals), 0,
        "umts_rrc_pdu_def.T_geran_SystemInfoType1", HFILL }},
    { &hf_umts_rrc_pdu_def_geran_SystemInfoType2,
      { "geran-SystemInfoType", "umts_rrc_pdu_def.geran_SystemInfoType",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_geran_SystemInfoType2_vals), 0,
        "umts_rrc_pdu_def.T_geran_SystemInfoType2", HFILL }},
    { &hf_umts_rrc_pdu_def_handoverFromUTRANCommand_GERANIu,
      { "handoverFromUTRANCommand-GERANIu", "umts_rrc_pdu_def.handoverFromUTRANCommand_GERANIu",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_handoverFromUTRANCommand_GERANIu_vals), 0,
        "umts_rrc_pdu_def.T_handoverFromUTRANCommand_GERANIu", HFILL }},
    { &hf_umts_rrc_pdu_def_r53,
      { "r5", "umts_rrc_pdu_def.r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r53", HFILL }},
    { &hf_umts_rrc_pdu_def_handoverFromUTRANCommand_GERANIu_r5,
      { "handoverFromUTRANCommand-GERANIu-r5", "umts_rrc_pdu_def.handoverFromUTRANCommand_GERANIu_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.HandoverFromUTRANCommand_GERANIu_r5_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions27,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions27", HFILL }},
    { &hf_umts_rrc_pdu_def_later_than_r5,
      { "later-than-r5", "umts_rrc_pdu_def.later_than_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_later_than_r5", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions21,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_criticalExtensions21", HFILL }},
    { &hf_umts_rrc_pdu_def_frequency_Band,
      { "frequency-Band", "umts_rrc_pdu_def.frequency_Band",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_Frequency_Band_vals), 0,
        "umts_rrc_ies.Frequency_Band", HFILL }},
    { &hf_umts_rrc_pdu_def_geranIu_Message,
      { "geranIu-Message", "umts_rrc_pdu_def.geranIu_Message",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_geranIu_Message_vals), 0,
        "umts_rrc_pdu_def.T_geranIu_Message", HFILL }},
    { &hf_umts_rrc_pdu_def_single_GERANIu_Message,
      { "single-GERANIu-Message", "umts_rrc_pdu_def.single_GERANIu_Message",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_single_GERANIu_Message", HFILL }},
    { &hf_umts_rrc_pdu_def_geranIu_MessageList,
      { "geranIu-MessageList", "umts_rrc_pdu_def.geranIu_MessageList",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_geranIu_MessageList", HFILL }},
    { &hf_umts_rrc_pdu_def_geranIu_Messages,
      { "geranIu-Messages", "umts_rrc_pdu_def.geranIu_Messages",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.GERANIu_MessageList", HFILL }},
    { &hf_umts_rrc_pdu_def_r310,
      { "r3", "umts_rrc_pdu_def.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r310", HFILL }},
    { &hf_umts_rrc_pdu_def_handoverFromUTRANCommand_CDMA2000_r3,
      { "handoverFromUTRANCommand-CDMA2000-r3", "umts_rrc_pdu_def.handoverFromUTRANCommand_CDMA2000_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.HandoverFromUTRANCommand_CDMA2000_r3_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions15,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions15", HFILL }},
    { &hf_umts_rrc_pdu_def_handoverFromUTRANCommand_CDMA2000_r3_add_ext,
      { "handoverFromUTRANCommand-CDMA2000-r3-add-ext", "umts_rrc_pdu_def.handoverFromUTRANCommand_CDMA2000_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions28,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions28", HFILL }},
    { &hf_umts_rrc_pdu_def_later_than_r38,
      { "later-than-r3", "umts_rrc_pdu_def.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_later_than_r38", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions22,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_criticalExtensions22", HFILL }},
    { &hf_umts_rrc_pdu_def_cdma2000_MessageList,
      { "cdma2000-MessageList", "umts_rrc_pdu_def.cdma2000_MessageList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.CDMA2000_MessageList", HFILL }},
    { &hf_umts_rrc_pdu_def_interRAT_HO_FailureCause,
      { "interRAT-HO-FailureCause", "umts_rrc_pdu_def.interRAT_HO_FailureCause",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_InterRAT_HO_FailureCause_vals), 0,
        "umts_rrc_ies.InterRAT_HO_FailureCause", HFILL }},
    { &hf_umts_rrc_pdu_def_interRATMessage,
      { "interRATMessage", "umts_rrc_pdu_def.interRATMessage",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_interRATMessage_vals), 0,
        "umts_rrc_pdu_def.T_interRATMessage", HFILL }},
    { &hf_umts_rrc_pdu_def_gsm,
      { "gsm", "umts_rrc_pdu_def.gsm",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_gsm", HFILL }},
    { &hf_umts_rrc_pdu_def_gsm_MessageList2,
      { "gsm-MessageList", "umts_rrc_pdu_def.gsm_MessageList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.GSM_MessageList", HFILL }},
    { &hf_umts_rrc_pdu_def_cdma2000,
      { "cdma2000", "umts_rrc_pdu_def.cdma2000",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_cdma2000", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions16,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions16", HFILL }},
    { &hf_umts_rrc_pdu_def_handoverFromUTRANFailure_r3_add_ext,
      { "handoverFromUTRANFailure-r3-add-ext", "umts_rrc_pdu_def.handoverFromUTRANFailure_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions6,
      { "v590NonCriticalExtensions", "umts_rrc_pdu_def.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v590NonCriticalExtensions6", HFILL }},
    { &hf_umts_rrc_pdu_def_handoverFromUTRANFailure_v590ext,
      { "handoverFromUTRANFailure-v590ext", "umts_rrc_pdu_def.handoverFromUTRANFailure_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.HandoverFromUtranFailure_v590ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions29,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions29", HFILL }},
    { &hf_umts_rrc_pdu_def_geranIu_MessageList1,
      { "geranIu-MessageList", "umts_rrc_pdu_def.geranIu_MessageList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.GERANIu_MessageList", HFILL }},
    { &hf_umts_rrc_pdu_def_predefinedConfigStatusList,
      { "predefinedConfigStatusList", "umts_rrc_pdu_def.predefinedConfigStatusList",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_predefinedConfigStatusList_vals), 0,
        "umts_rrc_pdu_def.T_predefinedConfigStatusList", HFILL }},
    { &hf_umts_rrc_pdu_def_absent,
      { "absent", "umts_rrc_pdu_def.absent",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.NULL", HFILL }},
    { &hf_umts_rrc_pdu_def_present,
      { "present", "umts_rrc_pdu_def.present",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.PredefinedConfigStatusList", HFILL }},
    { &hf_umts_rrc_pdu_def_uE_SecurityInformation,
      { "uE-SecurityInformation", "umts_rrc_pdu_def.uE_SecurityInformation",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_uE_SecurityInformation_vals), 0,
        "umts_rrc_pdu_def.T_uE_SecurityInformation", HFILL }},
    { &hf_umts_rrc_pdu_def_present1,
      { "present", "umts_rrc_pdu_def.present",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.UE_SecurityInformation", HFILL }},
    { &hf_umts_rrc_pdu_def_ue_CapabilityContainer,
      { "ue-CapabilityContainer", "umts_rrc_pdu_def.ue_CapabilityContainer",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_ue_CapabilityContainer_vals), 0,
        "umts_rrc_pdu_def.T_ue_CapabilityContainer", HFILL }},
    { &hf_umts_rrc_pdu_def_present2,
      { "present", "umts_rrc_pdu_def.present",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.OCTET_STRING_SIZE_0_63", HFILL }},
    { &hf_umts_rrc_pdu_def_v390NonCriticalExtensions,
      { "v390NonCriticalExtensions", "umts_rrc_pdu_def.v390NonCriticalExtensions",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_v390NonCriticalExtensions_vals), 0,
        "umts_rrc_pdu_def.T_v390NonCriticalExtensions", HFILL }},
    { &hf_umts_rrc_pdu_def_present3,
      { "present", "umts_rrc_pdu_def.present",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_present", HFILL }},
    { &hf_umts_rrc_pdu_def_interRATHandoverInfo_v390ext,
      { "interRATHandoverInfo-v390ext", "umts_rrc_pdu_def.interRATHandoverInfo_v390ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.InterRATHandoverInfo_v390ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v3a0NonCriticalExtensions3,
      { "v3a0NonCriticalExtensions", "umts_rrc_pdu_def.v3a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v3a0NonCriticalExtensions3", HFILL }},
    { &hf_umts_rrc_pdu_def_interRATHandoverInfo_v3a0ext,
      { "interRATHandoverInfo-v3a0ext", "umts_rrc_pdu_def.interRATHandoverInfo_v3a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.InterRATHandoverInfo_v3a0ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions17,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions17", HFILL }},
    { &hf_umts_rrc_pdu_def_interRATHandoverInfo_v3d0ext,
      { "interRATHandoverInfo-v3d0ext", "umts_rrc_pdu_def.interRATHandoverInfo_v3d0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.InterRATHandoverInfo_v3d0ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_interRATHandoverInfo_r3_add_ext,
      { "interRATHandoverInfo-r3-add-ext", "umts_rrc_pdu_def.interRATHandoverInfo_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v3g0NonCriticalExtensions,
      { "v3g0NonCriticalExtensions", "umts_rrc_pdu_def.v3g0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v3g0NonCriticalExtensions", HFILL }},
    { &hf_umts_rrc_pdu_def_interRATHandoverInfo_v3g0ext,
      { "interRATHandoverInfo-v3g0ext", "umts_rrc_pdu_def.interRATHandoverInfo_v3g0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.InterRATHandoverInfo_v3g0ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions4,
      { "v4b0NonCriticalExtensions", "umts_rrc_pdu_def.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v4b0NonCriticalExtensions4", HFILL }},
    { &hf_umts_rrc_pdu_def_interRATHandoverInfo_v4b0ext,
      { "interRATHandoverInfo-v4b0ext", "umts_rrc_pdu_def.interRATHandoverInfo_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.InterRATHandoverInfo_v4b0ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v4d0NonCriticalExtensions2,
      { "v4d0NonCriticalExtensions", "umts_rrc_pdu_def.v4d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v4d0NonCriticalExtensions2", HFILL }},
    { &hf_umts_rrc_pdu_def_interRATHandoverInfo_v4d0ext,
      { "interRATHandoverInfo-v4d0ext", "umts_rrc_pdu_def.interRATHandoverInfo_v4d0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.InterRATHandoverInfo_v4d0ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions7,
      { "v590NonCriticalExtensions", "umts_rrc_pdu_def.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v590NonCriticalExtensions7", HFILL }},
    { &hf_umts_rrc_pdu_def_interRATHandoverInfo_v590ext,
      { "interRATHandoverInfo-v590ext", "umts_rrc_pdu_def.interRATHandoverInfo_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.InterRATHandoverInfo_v590ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions10,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions10", HFILL }},
    { &hf_umts_rrc_pdu_def_interRATHandoverInfo_v6xy0ext,
      { "interRATHandoverInfo-v6xy0ext", "umts_rrc_pdu_def.interRATHandoverInfo_v6xy0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.InterRATHandoverInfo_v6xyext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions30,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions30", HFILL }},
    { &hf_umts_rrc_pdu_def_ue_RadioAccessCapability_v380ext,
      { "ue-RadioAccessCapability-v380ext", "umts_rrc_pdu_def.ue_RadioAccessCapability_v380ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.UE_RadioAccessCapability_v380ext", HFILL }},
    { &hf_umts_rrc_pdu_def_dl_PhysChCapabilityFDD_v380ext,
      { "dl-PhysChCapabilityFDD-v380ext", "umts_rrc_pdu_def.dl_PhysChCapabilityFDD_v380ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.DL_PhysChCapabilityFDD_v380ext", HFILL }},
    { &hf_umts_rrc_pdu_def_ue_RadioAccessCapability_v3a0ext,
      { "ue-RadioAccessCapability-v3a0ext", "umts_rrc_pdu_def.ue_RadioAccessCapability_v3a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.UE_RadioAccessCapability_v3a0ext", HFILL }},
    { &hf_umts_rrc_pdu_def_uESpecificBehaviourInformation1interRAT,
      { "uESpecificBehaviourInformation1interRAT", "umts_rrc_pdu_def.uESpecificBehaviourInformation1interRAT",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_ies.UESpecificBehaviourInformation1interRAT", HFILL }},
    { &hf_umts_rrc_pdu_def_ue_RadioAccessCapability_v3g0ext,
      { "ue-RadioAccessCapability-v3g0ext", "umts_rrc_pdu_def.ue_RadioAccessCapability_v3g0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.UE_RadioAccessCapability_v3g0ext", HFILL }},
    { &hf_umts_rrc_pdu_def_accessStratumReleaseIndicator,
      { "accessStratumReleaseIndicator", "umts_rrc_pdu_def.accessStratumReleaseIndicator",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_AccessStratumReleaseIndicator_vals), 0,
        "umts_rrc_ies.AccessStratumReleaseIndicator", HFILL }},
    { &hf_umts_rrc_pdu_def_tdd128_RF_Capability,
      { "tdd128-RF-Capability", "umts_rrc_pdu_def.tdd128_RF_Capability",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_RadioFrequencyBandTDDList_vals), 0,
        "umts_rrc_ies.RadioFrequencyBandTDDList", HFILL }},
    { &hf_umts_rrc_pdu_def_predefinedConfigStatusListComp,
      { "predefinedConfigStatusListComp", "umts_rrc_pdu_def.predefinedConfigStatusListComp",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.PredefinedConfigStatusListComp", HFILL }},
    { &hf_umts_rrc_pdu_def_ue_RadioAccessCapabilityComp,
      { "ue-RadioAccessCapabilityComp", "umts_rrc_pdu_def.ue_RadioAccessCapabilityComp",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.UE_RadioAccessCapabilityComp", HFILL }},
    { &hf_umts_rrc_pdu_def_ue_SecurityInformation2,
      { "ue-SecurityInformation2", "umts_rrc_pdu_def.ue_SecurityInformation2",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.UE_SecurityInformation2", HFILL }},
    { &hf_umts_rrc_pdu_def_r311,
      { "r3", "umts_rrc_pdu_def.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r311", HFILL }},
    { &hf_umts_rrc_pdu_def_measurementControl_r3,
      { "measurementControl-r3", "umts_rrc_pdu_def.measurementControl_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.MeasurementControl_r3_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v390nonCriticalExtensions,
      { "v390nonCriticalExtensions", "umts_rrc_pdu_def.v390nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v390nonCriticalExtensions", HFILL }},
    { &hf_umts_rrc_pdu_def_measurementControl_v390ext,
      { "measurementControl-v390ext", "umts_rrc_pdu_def.measurementControl_v390ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.MeasurementControl_v390ext", HFILL }},
    { &hf_umts_rrc_pdu_def_v3a0NonCriticalExtensions4,
      { "v3a0NonCriticalExtensions", "umts_rrc_pdu_def.v3a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v3a0NonCriticalExtensions4", HFILL }},
    { &hf_umts_rrc_pdu_def_measurementControl_v3a0ext,
      { "measurementControl-v3a0ext", "umts_rrc_pdu_def.measurementControl_v3a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.MeasurementControl_v3a0ext", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions18,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions18", HFILL }},
    { &hf_umts_rrc_pdu_def_measurementControl_r3_add_ext,
      { "measurementControl-r3-add-ext", "umts_rrc_pdu_def.measurementControl_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions5,
      { "v4b0NonCriticalExtensions", "umts_rrc_pdu_def.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v4b0NonCriticalExtensions5", HFILL }},
    { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions8,
      { "v590NonCriticalExtensions", "umts_rrc_pdu_def.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v590NonCriticalExtensions8", HFILL }},
    { &hf_umts_rrc_pdu_def_measurementControl_v590ext,
      { "measurementControl-v590ext", "umts_rrc_pdu_def.measurementControl_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.MeasurementControl_v590ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v5b0NonCriticalExtensions,
      { "v5b0NonCriticalExtensions", "umts_rrc_pdu_def.v5b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v5b0NonCriticalExtensions", HFILL }},
    { &hf_umts_rrc_pdu_def_measurementControl_v5b0ext,
      { "measurementControl-v5b0ext", "umts_rrc_pdu_def.measurementControl_v5b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.MeasurementControl_v5b0ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions31,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions31", HFILL }},
    { &hf_umts_rrc_pdu_def_later_than_r39,
      { "later-than-r3", "umts_rrc_pdu_def.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_later_than_r39", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions23,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_criticalExtensions23_vals), 0,
        "umts_rrc_pdu_def.T_criticalExtensions23", HFILL }},
    { &hf_umts_rrc_pdu_def_r43,
      { "r4", "umts_rrc_pdu_def.r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r43", HFILL }},
    { &hf_umts_rrc_pdu_def_measurementControl_r4,
      { "measurementControl-r4", "umts_rrc_pdu_def.measurementControl_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.MeasurementControl_r4_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v4d0NonCriticalExtensions3,
      { "v4d0NonCriticalExtensions", "umts_rrc_pdu_def.v4d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v4d0NonCriticalExtensions3", HFILL }},
    { &hf_umts_rrc_pdu_def_measurementControl_r4_add_ext,
      { "measurementControl-r4-add-ext", "umts_rrc_pdu_def.measurementControl_r4_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions9,
      { "v590NonCriticalExtensions", "umts_rrc_pdu_def.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v590NonCriticalExtensions9", HFILL }},
    { &hf_umts_rrc_pdu_def_v5b0NonCriticalExtensions1,
      { "v5b0NonCriticalExtensions", "umts_rrc_pdu_def.v5b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v5b0NonCriticalExtensions1", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions32,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions32", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions24,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_criticalExtensions24", HFILL }},
    { &hf_umts_rrc_pdu_def_measurementIdentity,
      { "measurementIdentity", "umts_rrc_pdu_def.measurementIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.MeasurementIdentity", HFILL }},
    { &hf_umts_rrc_pdu_def_measurementCommand,
      { "measurementCommand", "umts_rrc_pdu_def.measurementCommand",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_MeasurementCommand_vals), 0,
        "umts_rrc_ies.MeasurementCommand", HFILL }},
    { &hf_umts_rrc_pdu_def_measurementReportingMode,
      { "measurementReportingMode", "umts_rrc_pdu_def.measurementReportingMode",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.MeasurementReportingMode", HFILL }},
    { &hf_umts_rrc_pdu_def_additionalMeasurementList,
      { "additionalMeasurementList", "umts_rrc_pdu_def.additionalMeasurementList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.AdditionalMeasurementID_List", HFILL }},
    { &hf_umts_rrc_pdu_def_dpch_CompressedModeStatusInfo,
      { "dpch-CompressedModeStatusInfo", "umts_rrc_pdu_def.dpch_CompressedModeStatusInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.DPCH_CompressedModeStatusInfo", HFILL }},
    { &hf_umts_rrc_pdu_def_ue_Positioning_Measurement_v390ext,
      { "ue-Positioning-Measurement-v390ext", "umts_rrc_pdu_def.ue_Positioning_Measurement_v390ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.UE_Positioning_Measurement_v390ext", HFILL }},
    { &hf_umts_rrc_pdu_def_measurementCommand1,
      { "measurementCommand", "umts_rrc_pdu_def.measurementCommand",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_MeasurementCommand_r4_vals), 0,
        "umts_rrc_ies.MeasurementCommand_r4", HFILL }},
    { &hf_umts_rrc_pdu_def_measurementCommand_v590ext,
      { "measurementCommand-v590ext", "umts_rrc_pdu_def.measurementCommand_v590ext",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_measurementCommand_v590ext_vals), 0,
        "umts_rrc_pdu_def.T_measurementCommand_v590ext", HFILL }},
    { &hf_umts_rrc_pdu_def_intra_frequency,
      { "intra-frequency", "umts_rrc_pdu_def.intra_frequency",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.Intra_FreqEventCriteriaList_v590ext", HFILL }},
    { &hf_umts_rrc_pdu_def_inter_frequency,
      { "inter-frequency", "umts_rrc_pdu_def.inter_frequency",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.Inter_FreqEventCriteriaList_v590ext", HFILL }},
    { &hf_umts_rrc_pdu_def_intraFreqReportingCriteria_1b_r5,
      { "intraFreqReportingCriteria-1b-r5", "umts_rrc_pdu_def.intraFreqReportingCriteria_1b_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.IntraFreqReportingCriteria_1b_r5", HFILL }},
    { &hf_umts_rrc_pdu_def_intraFreqEvent_1d_r5,
      { "intraFreqEvent-1d-r5", "umts_rrc_pdu_def.intraFreqEvent_1d_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.IntraFreqEvent_1d_r5", HFILL }},
    { &hf_umts_rrc_pdu_def_rrc_TransactionIdentifier_MSP_v590ext,
      { "rrc-TransactionIdentifier-MSP-v590ext", "umts_rrc_pdu_def.rrc_TransactionIdentifier_MSP_v590ext",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.RRC_TransactionIdentifier", HFILL }},
    { &hf_umts_rrc_pdu_def_interRATCellInfoIndication,
      { "interRATCellInfoIndication", "umts_rrc_pdu_def.interRATCellInfoIndication",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.InterRATCellInfoIndication", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions19,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions19", HFILL }},
    { &hf_umts_rrc_pdu_def_measurementControlFailure_r3_add_ext,
      { "measurementControlFailure-r3-add-ext", "umts_rrc_pdu_def.measurementControlFailure_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions10,
      { "v590NonCriticalExtensions", "umts_rrc_pdu_def.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v590NonCriticalExtensions10", HFILL }},
    { &hf_umts_rrc_pdu_def_measurementControlFailure_v590ext,
      { "measurementControlFailure-v590ext", "umts_rrc_pdu_def.measurementControlFailure_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.MeasurementControlFailure_v590ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions33,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions33", HFILL }},
    { &hf_umts_rrc_pdu_def_measuredResults,
      { "measuredResults", "umts_rrc_pdu_def.measuredResults",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_MeasuredResults_vals), 0,
        "umts_rrc_ies.MeasuredResults", HFILL }},
    { &hf_umts_rrc_pdu_def_additionalMeasuredResults,
      { "additionalMeasuredResults", "umts_rrc_pdu_def.additionalMeasuredResults",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.MeasuredResultsList", HFILL }},
    { &hf_umts_rrc_pdu_def_eventResults,
      { "eventResults", "umts_rrc_pdu_def.eventResults",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_EventResults_vals), 0,
        "umts_rrc_ies.EventResults", HFILL }},
    { &hf_umts_rrc_pdu_def_v390nonCriticalExtensions1,
      { "v390nonCriticalExtensions", "umts_rrc_pdu_def.v390nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v390nonCriticalExtensions1", HFILL }},
    { &hf_umts_rrc_pdu_def_measurementReport_v390ext,
      { "measurementReport-v390ext", "umts_rrc_pdu_def.measurementReport_v390ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.MeasurementReport_v390ext", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions20,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions20", HFILL }},
    { &hf_umts_rrc_pdu_def_measurementReport_r3_add_ext,
      { "measurementReport-r3-add-ext", "umts_rrc_pdu_def.measurementReport_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions6,
      { "v4b0NonCriticalExtensions", "umts_rrc_pdu_def.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v4b0NonCriticalExtensions6", HFILL }},
    { &hf_umts_rrc_pdu_def_measurementReport_v4b0ext,
      { "measurementReport-v4b0ext", "umts_rrc_pdu_def.measurementReport_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.MeasurementReport_v4b0ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions11,
      { "v590NonCriticalExtensions", "umts_rrc_pdu_def.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v590NonCriticalExtensions11", HFILL }},
    { &hf_umts_rrc_pdu_def_measurementReport_v590ext,
      { "measurementReport-v590ext", "umts_rrc_pdu_def.measurementReport_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.MeasurementReport_v590ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v5b0NonCriticalExtensions2,
      { "v5b0NonCriticalExtensions", "umts_rrc_pdu_def.v5b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v5b0NonCriticalExtensions2", HFILL }},
    { &hf_umts_rrc_pdu_def_measurementReport_v5b0ext,
      { "measurementReport-v5b0ext", "umts_rrc_pdu_def.measurementReport_v5b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.MeasurementReport_v5b0ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions11,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions11", HFILL }},
    { &hf_umts_rrc_pdu_def_measurementReport_v6xyext,
      { "measurementReport-v6xyext", "umts_rrc_pdu_def.measurementReport_v6xyext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.MeasurementReport_v6xyext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions34,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions34", HFILL }},
    { &hf_umts_rrc_pdu_def_measuredResults_v390ext,
      { "measuredResults-v390ext", "umts_rrc_pdu_def.measuredResults_v390ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.MeasuredResults_v390ext", HFILL }},
    { &hf_umts_rrc_pdu_def_interFreqEventResults_LCR,
      { "interFreqEventResults-LCR", "umts_rrc_pdu_def.interFreqEventResults_LCR",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.InterFreqEventResults_LCR_r4_ext", HFILL }},
    { &hf_umts_rrc_pdu_def_additionalMeasuredResults_LCR,
      { "additionalMeasuredResults-LCR", "umts_rrc_pdu_def.additionalMeasuredResults_LCR",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.MeasuredResultsList_LCR_r4_ext", HFILL }},
    { &hf_umts_rrc_pdu_def_gsmOTDreferenceCell,
      { "gsmOTDreferenceCell", "umts_rrc_pdu_def.gsmOTDreferenceCell",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.PrimaryCPICH_Info", HFILL }},
    { &hf_umts_rrc_pdu_def_measuredResults_v590ext,
      { "measuredResults-v590ext", "umts_rrc_pdu_def.measuredResults_v590ext",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_MeasuredResults_v590ext_vals), 0,
        "umts_rrc_ies.MeasuredResults_v590ext", HFILL }},
    { &hf_umts_rrc_pdu_def_pagingRecordList,
      { "pagingRecordList", "umts_rrc_pdu_def.pagingRecordList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.PagingRecordList", HFILL }},
    { &hf_umts_rrc_pdu_def_bcch_ModificationInfo,
      { "bcch-ModificationInfo", "umts_rrc_pdu_def.bcch_ModificationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.BCCH_ModificationInfo", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions21,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions21", HFILL }},
    { &hf_umts_rrc_pdu_def_pagingType1_r3_add_ext,
      { "pagingType1-r3-add-ext", "umts_rrc_pdu_def.pagingType1_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions12,
      { "v590NonCriticalExtensions", "umts_rrc_pdu_def.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v590NonCriticalExtensions12", HFILL }},
    { &hf_umts_rrc_pdu_def_pagingType1_v590ext,
      { "pagingType1-v590ext", "umts_rrc_pdu_def.pagingType1_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.PagingType1_v590ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions35,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions35", HFILL }},
    { &hf_umts_rrc_pdu_def_pagingRecord2List,
      { "pagingRecord2List", "umts_rrc_pdu_def.pagingRecord2List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.PagingRecord2List_r5", HFILL }},
    { &hf_umts_rrc_pdu_def_pagingCause,
      { "pagingCause", "umts_rrc_pdu_def.pagingCause",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_PagingCause_vals), 0,
        "umts_rrc_ies.PagingCause", HFILL }},
    { &hf_umts_rrc_pdu_def_pagingRecordTypeID,
      { "pagingRecordTypeID", "umts_rrc_pdu_def.pagingRecordTypeID",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_PagingRecordTypeID_vals), 0,
        "umts_rrc_ies.PagingRecordTypeID", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions22,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions22", HFILL }},
    { &hf_umts_rrc_pdu_def_pagingType2_r3_add_ext,
      { "pagingType2-r3-add-ext", "umts_rrc_pdu_def.pagingType2_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions36,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions36", HFILL }},
    { &hf_umts_rrc_pdu_def_r312,
      { "r3", "umts_rrc_pdu_def.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r312", HFILL }},
    { &hf_umts_rrc_pdu_def_physicalChannelReconfiguration_r3,
      { "physicalChannelReconfiguration-r3", "umts_rrc_pdu_def.physicalChannelReconfiguration_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.PhysicalChannelReconfiguration_r3_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v3a0NonCriticalExtensions5,
      { "v3a0NonCriticalExtensions", "umts_rrc_pdu_def.v3a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v3a0NonCriticalExtensions5", HFILL }},
    { &hf_umts_rrc_pdu_def_physicalChannelReconfiguration_v3a0ext,
      { "physicalChannelReconfiguration-v3a0ext", "umts_rrc_pdu_def.physicalChannelReconfiguration_v3a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.PhysicalChannelReconfiguration_v3a0ext", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions23,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions23", HFILL }},
    { &hf_umts_rrc_pdu_def_physicalChannelReconfiguration_r3_add_ext,
      { "physicalChannelReconfiguration-r3-add-ext", "umts_rrc_pdu_def.physicalChannelReconfiguration_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v4b0NonCriticalExtenstions,
      { "v4b0NonCriticalExtenstions", "umts_rrc_pdu_def.v4b0NonCriticalExtenstions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v4b0NonCriticalExtenstions", HFILL }},
    { &hf_umts_rrc_pdu_def_physicalChannelReconfiguration_v4b0ext,
      { "physicalChannelReconfiguration-v4b0ext", "umts_rrc_pdu_def.physicalChannelReconfiguration_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.PhysicalChannelReconfiguration_v4b0ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v590NonCriticalExtenstions2,
      { "v590NonCriticalExtenstions", "umts_rrc_pdu_def.v590NonCriticalExtenstions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v590NonCriticalExtenstions2", HFILL }},
    { &hf_umts_rrc_pdu_def_physicalChannelReconfiguration_v590ext,
      { "physicalChannelReconfiguration-v590ext", "umts_rrc_pdu_def.physicalChannelReconfiguration_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.PhysicalChannelReconfiguration_v590ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions12,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions12", HFILL }},
    { &hf_umts_rrc_pdu_def_physicalChannelReconfiguration_v6xyext,
      { "physicalChannelReconfiguration-v6xyext", "umts_rrc_pdu_def.physicalChannelReconfiguration_v6xyext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.PhysicalChannelReconfiguration_v6xyext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions37,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions37", HFILL }},
    { &hf_umts_rrc_pdu_def_later_than_r310,
      { "later-than-r3", "umts_rrc_pdu_def.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_later_than_r310", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions25,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_criticalExtensions25_vals), 0,
        "umts_rrc_pdu_def.T_criticalExtensions25", HFILL }},
    { &hf_umts_rrc_pdu_def_r44,
      { "r4", "umts_rrc_pdu_def.r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r44", HFILL }},
    { &hf_umts_rrc_pdu_def_physicalChannelReconfiguration_r4,
      { "physicalChannelReconfiguration-r4", "umts_rrc_pdu_def.physicalChannelReconfiguration_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.PhysicalChannelReconfiguration_r4_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v4d0NonCriticalExtensions4,
      { "v4d0NonCriticalExtensions", "umts_rrc_pdu_def.v4d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v4d0NonCriticalExtensions4", HFILL }},
    { &hf_umts_rrc_pdu_def_physicalChannelReconfiguration_r4_add_ext,
      { "physicalChannelReconfiguration-r4-add-ext", "umts_rrc_pdu_def.physicalChannelReconfiguration_r4_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v590NonCriticalExtenstions3,
      { "v590NonCriticalExtenstions", "umts_rrc_pdu_def.v590NonCriticalExtenstions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v590NonCriticalExtenstions3", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions13,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions13", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions38,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions38", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions26,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_criticalExtensions26_vals), 0,
        "umts_rrc_pdu_def.T_criticalExtensions26", HFILL }},
    { &hf_umts_rrc_pdu_def_r54,
      { "r5", "umts_rrc_pdu_def.r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r54", HFILL }},
    { &hf_umts_rrc_pdu_def_physicalChannelReconfiguration_r5,
      { "physicalChannelReconfiguration-r5", "umts_rrc_pdu_def.physicalChannelReconfiguration_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.PhysicalChannelReconfiguration_r5_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_physicalChannelReconfiguration_r5_add_ext,
      { "physicalChannelReconfiguration-r5-add-ext", "umts_rrc_pdu_def.physicalChannelReconfiguration_r5_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions14,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions14", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions39,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions39", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions27,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_criticalExtensions27_vals), 0,
        "umts_rrc_pdu_def.T_criticalExtensions27", HFILL }},
    { &hf_umts_rrc_pdu_def_r65,
      { "r6", "umts_rrc_pdu_def.r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r65", HFILL }},
    { &hf_umts_rrc_pdu_def_physicalChannelReconfiguration_r6,
      { "physicalChannelReconfiguration-r6", "umts_rrc_pdu_def.physicalChannelReconfiguration_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.PhysicalChannelReconfiguration_r6_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_physicalChannelReconfiguration_r6_add_ext,
      { "physicalChannelReconfiguration-r6-add-ext", "umts_rrc_pdu_def.physicalChannelReconfiguration_r6_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions40,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions40", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions28,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_criticalExtensions28", HFILL }},
    { &hf_umts_rrc_pdu_def_ul_ChannelRequirement4,
      { "ul-ChannelRequirement", "umts_rrc_pdu_def.ul_ChannelRequirement",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_UL_ChannelRequirementWithCPCH_SetID_vals), 0,
        "umts_rrc_ies.UL_ChannelRequirementWithCPCH_SetID", HFILL }},
    { &hf_umts_rrc_pdu_def_modeSpecificInfo6,
      { "modeSpecificInfo", "umts_rrc_pdu_def.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_modeSpecificInfo6_vals), 0,
        "umts_rrc_pdu_def.T_modeSpecificInfo6", HFILL }},
    { &hf_umts_rrc_pdu_def_fdd12,
      { "fdd", "umts_rrc_pdu_def.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_fdd12", HFILL }},
    { &hf_umts_rrc_pdu_def_ul_ChannelRequirement5,
      { "ul-ChannelRequirement", "umts_rrc_pdu_def.ul_ChannelRequirement",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_UL_ChannelRequirementWithCPCH_SetID_r4_vals), 0,
        "umts_rrc_ies.UL_ChannelRequirementWithCPCH_SetID_r4", HFILL }},
    { &hf_umts_rrc_pdu_def_modeSpecificInfo7,
      { "modeSpecificInfo", "umts_rrc_pdu_def.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_modeSpecificInfo7_vals), 0,
        "umts_rrc_pdu_def.T_modeSpecificInfo7", HFILL }},
    { &hf_umts_rrc_pdu_def_fdd13,
      { "fdd", "umts_rrc_pdu_def.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_fdd13", HFILL }},
    { &hf_umts_rrc_pdu_def_ul_ChannelRequirement6,
      { "ul-ChannelRequirement", "umts_rrc_pdu_def.ul_ChannelRequirement",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_UL_ChannelRequirementWithCPCH_SetID_r5_vals), 0,
        "umts_rrc_ies.UL_ChannelRequirementWithCPCH_SetID_r5", HFILL }},
    { &hf_umts_rrc_pdu_def_modeSpecificInfo8,
      { "modeSpecificInfo", "umts_rrc_pdu_def.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_modeSpecificInfo8_vals), 0,
        "umts_rrc_pdu_def.T_modeSpecificInfo8", HFILL }},
    { &hf_umts_rrc_pdu_def_fdd14,
      { "fdd", "umts_rrc_pdu_def.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_fdd14", HFILL }},
    { &hf_umts_rrc_pdu_def_delayRestrictionFlag,
      { "delayRestrictionFlag", "umts_rrc_pdu_def.delayRestrictionFlag",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_DelayRestrictionFlag_vals), 0,
        "umts_rrc_ies.DelayRestrictionFlag", HFILL }},
    { &hf_umts_rrc_pdu_def_postVerificationPeriod1,
      { "postVerificationPeriod", "umts_rrc_pdu_def.postVerificationPeriod",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_postVerificationPeriod1_vals), 0,
        "umts_rrc_pdu_def.T_postVerificationPeriod1", HFILL }},
    { &hf_umts_rrc_pdu_def_timingMaintainedSynchInd,
      { "timingMaintainedSynchInd", "umts_rrc_pdu_def.timingMaintainedSynchInd",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_timingMaintainedSynchInd_vals), 0,
        "umts_rrc_pdu_def.T_timingMaintainedSynchInd", HFILL }},
    { &hf_umts_rrc_pdu_def_ul_IntegProtActivationInfo,
      { "ul-IntegProtActivationInfo", "umts_rrc_pdu_def.ul_IntegProtActivationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.IntegrityProtActivationInfo", HFILL }},
    { &hf_umts_rrc_pdu_def_ul_TimingAdvance,
      { "ul-TimingAdvance", "umts_rrc_pdu_def.ul_TimingAdvance",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.UL_TimingAdvance", HFILL }},
    { &hf_umts_rrc_pdu_def_dummy_RB_ActivationTimeInfoList,
      { "dummy-RB-ActivationTimeInfoList", "umts_rrc_pdu_def.dummy_RB_ActivationTimeInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.RB_ActivationTimeInfoList", HFILL }},
    { &hf_umts_rrc_pdu_def_ul_CounterSynchronisationInfo,
      { "ul-CounterSynchronisationInfo", "umts_rrc_pdu_def.ul_CounterSynchronisationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.UL_CounterSynchronisationInfo", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions24,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions24", HFILL }},
    { &hf_umts_rrc_pdu_def_physicalChannelReconfigurationComplete_r3_add_ext,
      { "physicalChannelReconfigurationComplete-r3-add-ext", "umts_rrc_pdu_def.physicalChannelReconfigurationComplete_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions41,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions41", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions25,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions25", HFILL }},
    { &hf_umts_rrc_pdu_def_physicalChannelReconfigurationFailure_r3_add_ext,
      { "physicalChannelReconfigurationFailure-r3-add-ext", "umts_rrc_pdu_def.physicalChannelReconfigurationFailure_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions42,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions42", HFILL }},
    { &hf_umts_rrc_pdu_def_r313,
      { "r3", "umts_rrc_pdu_def.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r313", HFILL }},
    { &hf_umts_rrc_pdu_def_physicalSharedChannelAllocation_r3,
      { "physicalSharedChannelAllocation-r3", "umts_rrc_pdu_def.physicalSharedChannelAllocation_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.PhysicalSharedChannelAllocation_r3_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions26,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions26", HFILL }},
    { &hf_umts_rrc_pdu_def_physicalSharedChannelAllocation_r3_add_ext,
      { "physicalSharedChannelAllocation-r3-add-ext", "umts_rrc_pdu_def.physicalSharedChannelAllocation_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions43,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions43", HFILL }},
    { &hf_umts_rrc_pdu_def_later_than_r311,
      { "later-than-r3", "umts_rrc_pdu_def.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_later_than_r311", HFILL }},
    { &hf_umts_rrc_pdu_def_dsch_RNTI,
      { "dsch-RNTI", "umts_rrc_pdu_def.dsch_RNTI",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_ies.DSCH_RNTI", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions29,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_criticalExtensions29_vals), 0,
        "umts_rrc_pdu_def.T_criticalExtensions29", HFILL }},
    { &hf_umts_rrc_pdu_def_r45,
      { "r4", "umts_rrc_pdu_def.r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r45", HFILL }},
    { &hf_umts_rrc_pdu_def_physicalSharedChannelAllocation_r4,
      { "physicalSharedChannelAllocation-r4", "umts_rrc_pdu_def.physicalSharedChannelAllocation_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.PhysicalSharedChannelAllocation_r4_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v4d0NonCriticalExtensions5,
      { "v4d0NonCriticalExtensions", "umts_rrc_pdu_def.v4d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v4d0NonCriticalExtensions5", HFILL }},
    { &hf_umts_rrc_pdu_def_physicalSharedChannelAllocation_r4_add_ext,
      { "physicalSharedChannelAllocation-r4-add-ext", "umts_rrc_pdu_def.physicalSharedChannelAllocation_r4_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions15,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions15", HFILL }},
    { &hf_umts_rrc_pdu_def_physicalSharedChannelAllocation_v6xyext,
      { "physicalSharedChannelAllocation-v6xyext", "umts_rrc_pdu_def.physicalSharedChannelAllocation_v6xyext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.PhysicalSharedChannelAllocation_v6xyext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions44,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions44", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions30,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_criticalExtensions30", HFILL }},
    { &hf_umts_rrc_pdu_def_ul_TimingAdvance1,
      { "ul-TimingAdvance", "umts_rrc_pdu_def.ul_TimingAdvance",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_UL_TimingAdvanceControl_vals), 0,
        "umts_rrc_ies.UL_TimingAdvanceControl", HFILL }},
    { &hf_umts_rrc_pdu_def_pusch_CapacityAllocationInfo,
      { "pusch-CapacityAllocationInfo", "umts_rrc_pdu_def.pusch_CapacityAllocationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.PUSCH_CapacityAllocationInfo", HFILL }},
    { &hf_umts_rrc_pdu_def_pdsch_CapacityAllocationInfo,
      { "pdsch-CapacityAllocationInfo", "umts_rrc_pdu_def.pdsch_CapacityAllocationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.PDSCH_CapacityAllocationInfo", HFILL }},
    { &hf_umts_rrc_pdu_def_confirmRequest,
      { "confirmRequest", "umts_rrc_pdu_def.confirmRequest",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_confirmRequest_vals), 0,
        "umts_rrc_pdu_def.T_confirmRequest", HFILL }},
    { &hf_umts_rrc_pdu_def_trafficVolumeReportRequest,
      { "trafficVolumeReportRequest", "umts_rrc_pdu_def.trafficVolumeReportRequest",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_pdu_def.INTEGER_0_255", HFILL }},
    { &hf_umts_rrc_pdu_def_iscpTimeslotList,
      { "iscpTimeslotList", "umts_rrc_pdu_def.iscpTimeslotList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.TimeslotList", HFILL }},
    { &hf_umts_rrc_pdu_def_requestPCCPCHRSCP,
      { "requestPCCPCHRSCP", "umts_rrc_pdu_def.requestPCCPCHRSCP",
        FT_BOOLEAN, 8, NULL, 0,
        "umts_rrc_pdu_def.BOOLEAN", HFILL }},
    { &hf_umts_rrc_pdu_def_ul_TimingAdvance2,
      { "ul-TimingAdvance", "umts_rrc_pdu_def.ul_TimingAdvance",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_UL_TimingAdvanceControl_r4_vals), 0,
        "umts_rrc_ies.UL_TimingAdvanceControl_r4", HFILL }},
    { &hf_umts_rrc_pdu_def_pusch_CapacityAllocationInfo1,
      { "pusch-CapacityAllocationInfo", "umts_rrc_pdu_def.pusch_CapacityAllocationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.PUSCH_CapacityAllocationInfo_r4", HFILL }},
    { &hf_umts_rrc_pdu_def_pdsch_CapacityAllocationInfo1,
      { "pdsch-CapacityAllocationInfo", "umts_rrc_pdu_def.pdsch_CapacityAllocationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.PDSCH_CapacityAllocationInfo_r4", HFILL }},
    { &hf_umts_rrc_pdu_def_confirmRequest1,
      { "confirmRequest", "umts_rrc_pdu_def.confirmRequest",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_confirmRequest1_vals), 0,
        "umts_rrc_pdu_def.T_confirmRequest1", HFILL }},
    { &hf_umts_rrc_pdu_def_iscpTimeslotList1,
      { "iscpTimeslotList", "umts_rrc_pdu_def.iscpTimeslotList",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_TimeslotList_r4_vals), 0,
        "umts_rrc_ies.TimeslotList_r4", HFILL }},
    { &hf_umts_rrc_pdu_def_trafficVolume,
      { "trafficVolume", "umts_rrc_pdu_def.trafficVolume",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.TrafficVolumeMeasuredResultsList", HFILL }},
    { &hf_umts_rrc_pdu_def_timeslotListWithISCP,
      { "timeslotListWithISCP", "umts_rrc_pdu_def.timeslotListWithISCP",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.TimeslotListWithISCP", HFILL }},
    { &hf_umts_rrc_pdu_def_primaryCCPCH_RSCP,
      { "primaryCCPCH-RSCP", "umts_rrc_pdu_def.primaryCCPCH_RSCP",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.PrimaryCCPCH_RSCP", HFILL }},
    { &hf_umts_rrc_pdu_def_allocationConfirmation,
      { "allocationConfirmation", "umts_rrc_pdu_def.allocationConfirmation",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_allocationConfirmation_vals), 0,
        "umts_rrc_pdu_def.T_allocationConfirmation", HFILL }},
    { &hf_umts_rrc_pdu_def_pdschConfirmation,
      { "pdschConfirmation", "umts_rrc_pdu_def.pdschConfirmation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.PDSCH_Identity", HFILL }},
    { &hf_umts_rrc_pdu_def_puschConfirmation,
      { "puschConfirmation", "umts_rrc_pdu_def.puschConfirmation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.PUSCH_Identity", HFILL }},
    { &hf_umts_rrc_pdu_def_protocolErrorIndicator,
      { "protocolErrorIndicator", "umts_rrc_pdu_def.protocolErrorIndicator",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_ProtocolErrorIndicatorWithMoreInfo_vals), 0,
        "umts_rrc_ies.ProtocolErrorIndicatorWithMoreInfo", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions27,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions27", HFILL }},
    { &hf_umts_rrc_pdu_def_puschCapacityRequest_r3_add_ext,
      { "puschCapacityRequest-r3-add-ext", "umts_rrc_pdu_def.puschCapacityRequest_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions13,
      { "v590NonCriticalExtensions", "umts_rrc_pdu_def.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v590NonCriticalExtensions13", HFILL }},
    { &hf_umts_rrc_pdu_def_puschCapacityRequest_v590ext,
      { "puschCapacityRequest-v590ext", "umts_rrc_pdu_def.puschCapacityRequest_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.PUSCHCapacityRequest_v590ext", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions45,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions45", HFILL }},
    { &hf_umts_rrc_pdu_def_primaryCCPCH_RSCP_delta,
      { "primaryCCPCH-RSCP-delta", "umts_rrc_pdu_def.primaryCCPCH_RSCP_delta",
        FT_INT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.DeltaRSCP", HFILL }},
    { &hf_umts_rrc_pdu_def_r314,
      { "r3", "umts_rrc_pdu_def.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r314", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerReconfiguration_r3,
      { "radioBearerReconfiguration-r3", "umts_rrc_pdu_def.radioBearerReconfiguration_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RadioBearerReconfiguration_r3_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v3aoNonCriticalExtensions,
      { "v3aoNonCriticalExtensions", "umts_rrc_pdu_def.v3aoNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v3aoNonCriticalExtensions", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerReconfiguration_v3a0ext,
      { "radioBearerReconfiguration-v3a0ext", "umts_rrc_pdu_def.radioBearerReconfiguration_v3a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RadioBearerReconfiguration_v3a0ext", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions28,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions28", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerReconfiguration_r3_add_ext,
      { "radioBearerReconfiguration-r3-add-ext", "umts_rrc_pdu_def.radioBearerReconfiguration_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions7,
      { "v4b0NonCriticalExtensions", "umts_rrc_pdu_def.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v4b0NonCriticalExtensions7", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerReconfiguration_v4b0ext,
      { "radioBearerReconfiguration-v4b0ext", "umts_rrc_pdu_def.radioBearerReconfiguration_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RadioBearerReconfiguration_v4b0ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions14,
      { "v590NonCriticalExtensions", "umts_rrc_pdu_def.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v590NonCriticalExtensions14", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerReconfiguration_v590ext,
      { "radioBearerReconfiguration-v590ext", "umts_rrc_pdu_def.radioBearerReconfiguration_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RadioBearerReconfiguration_v590ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v5d0NonCriticalExtenstions6,
      { "v5d0NonCriticalExtenstions", "umts_rrc_pdu_def.v5d0NonCriticalExtenstions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v5d0NonCriticalExtenstions6", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerReconfiguration_v5d0ext,
      { "radioBearerReconfiguration-v5d0ext", "umts_rrc_pdu_def.radioBearerReconfiguration_v5d0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RadioBearerReconfiguration_v5d0ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions16,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions16", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerReconfiguration_v6xyext,
      { "radioBearerReconfiguration-v6xyext", "umts_rrc_pdu_def.radioBearerReconfiguration_v6xyext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RadioBearerReconfiguration_v6xyext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions46,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions46", HFILL }},
    { &hf_umts_rrc_pdu_def_later_than_r312,
      { "later-than-r3", "umts_rrc_pdu_def.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_later_than_r312", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions31,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_criticalExtensions31_vals), 0,
        "umts_rrc_pdu_def.T_criticalExtensions31", HFILL }},
    { &hf_umts_rrc_pdu_def_r46,
      { "r4", "umts_rrc_pdu_def.r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r46", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerReconfiguration_r4,
      { "radioBearerReconfiguration-r4", "umts_rrc_pdu_def.radioBearerReconfiguration_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RadioBearerReconfiguration_r4_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v4d0NonCriticalExtensions6,
      { "v4d0NonCriticalExtensions", "umts_rrc_pdu_def.v4d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v4d0NonCriticalExtensions6", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerReconfiguration_r4_add_ext,
      { "radioBearerReconfiguration-r4-add-ext", "umts_rrc_pdu_def.radioBearerReconfiguration_r4_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions15,
      { "v590NonCriticalExtensions", "umts_rrc_pdu_def.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v590NonCriticalExtensions15", HFILL }},
    { &hf_umts_rrc_pdu_def_v5d0NonCriticalExtenstions7,
      { "v5d0NonCriticalExtenstions", "umts_rrc_pdu_def.v5d0NonCriticalExtenstions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v5d0NonCriticalExtenstions7", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions17,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions17", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions47,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions47", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions32,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_criticalExtensions32_vals), 0,
        "umts_rrc_pdu_def.T_criticalExtensions32", HFILL }},
    { &hf_umts_rrc_pdu_def_r55,
      { "r5", "umts_rrc_pdu_def.r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r55", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerReconfiguration_r5,
      { "radioBearerReconfiguration-r5", "umts_rrc_pdu_def.radioBearerReconfiguration_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RadioBearerReconfiguration_r5_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerReconfiguration_r5_add_ext,
      { "radioBearerReconfiguration-r5-add-ext", "umts_rrc_pdu_def.radioBearerReconfiguration_r5_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v5d0NonCriticalExtenstions8,
      { "v5d0NonCriticalExtenstions", "umts_rrc_pdu_def.v5d0NonCriticalExtenstions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v5d0NonCriticalExtenstions8", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions18,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions18", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions48,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions48", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions33,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_criticalExtensions33_vals), 0,
        "umts_rrc_pdu_def.T_criticalExtensions33", HFILL }},
    { &hf_umts_rrc_pdu_def_r66,
      { "r6", "umts_rrc_pdu_def.r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r66", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerReconfiguration_r6,
      { "radioBearerReconfiguration-r6", "umts_rrc_pdu_def.radioBearerReconfiguration_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RadioBearerReconfiguration_r6_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerReconfiguration_r6_add_ext,
      { "radioBearerReconfiguration-r6-add-ext", "umts_rrc_pdu_def.radioBearerReconfiguration_r6_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions49,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions49", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions34,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_criticalExtensions34", HFILL }},
    { &hf_umts_rrc_pdu_def_rab_InformationReconfigList,
      { "rab-InformationReconfigList", "umts_rrc_pdu_def.rab_InformationReconfigList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.RAB_InformationReconfigList", HFILL }},
    { &hf_umts_rrc_pdu_def_dummy1,
      { "dummy", "umts_rrc_pdu_def.dummy",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_dummy1_vals), 0,
        "umts_rrc_pdu_def.T_dummy1", HFILL }},
    { &hf_umts_rrc_pdu_def_fdd15,
      { "fdd", "umts_rrc_pdu_def.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_fdd15", HFILL }},
    { &hf_umts_rrc_pdu_def_dummy11,
      { "dummy1", "umts_rrc_pdu_def.dummy1",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.CPCH_SetID", HFILL }},
    { &hf_umts_rrc_pdu_def_dummy2,
      { "dummy2", "umts_rrc_pdu_def.dummy2",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.DRAC_StaticInformationList", HFILL }},
    { &hf_umts_rrc_pdu_def_dl_AddReconfTransChInfoList3,
      { "dl-AddReconfTransChInfoList", "umts_rrc_pdu_def.dl_AddReconfTransChInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.DL_AddReconfTransChInfo2List", HFILL }},
    { &hf_umts_rrc_pdu_def_modeSpecificPhysChInfo3,
      { "modeSpecificPhysChInfo", "umts_rrc_pdu_def.modeSpecificPhysChInfo",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_modeSpecificPhysChInfo3_vals), 0,
        "umts_rrc_pdu_def.T_modeSpecificPhysChInfo3", HFILL }},
    { &hf_umts_rrc_pdu_def_fdd16,
      { "fdd", "umts_rrc_pdu_def.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_fdd16", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerReconfiguration_r4_IEs_dummy,
      { "radioBearerReconfiguration-r4-IEs-dummy", "umts_rrc_pdu_def.radioBearerReconfiguration_r4_IEs_dummy",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_radioBearerReconfiguration_r4_IEs_dummy_vals), 0,
        "umts_rrc_pdu_def.T_radioBearerReconfiguration_r4_IEs_dummy", HFILL }},
    { &hf_umts_rrc_pdu_def_fdd17,
      { "fdd", "umts_rrc_pdu_def.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_fdd17", HFILL }},
    { &hf_umts_rrc_pdu_def_dummy1_CPCH_SetID,
      { "dummy1-CPCH-SetID", "umts_rrc_pdu_def.dummy1_CPCH_SetID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.CPCH_SetID", HFILL }},
    { &hf_umts_rrc_pdu_def_modeSpecificPhysChInfo4,
      { "modeSpecificPhysChInfo", "umts_rrc_pdu_def.modeSpecificPhysChInfo",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_modeSpecificPhysChInfo4_vals), 0,
        "umts_rrc_pdu_def.T_modeSpecificPhysChInfo4", HFILL }},
    { &hf_umts_rrc_pdu_def_fdd18,
      { "fdd", "umts_rrc_pdu_def.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_fdd18", HFILL }},
    { &hf_umts_rrc_pdu_def_specificationMode4,
      { "specificationMode", "umts_rrc_pdu_def.specificationMode",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_specificationMode4_vals), 0,
        "umts_rrc_pdu_def.T_specificationMode4", HFILL }},
    { &hf_umts_rrc_pdu_def_complete4,
      { "complete", "umts_rrc_pdu_def.complete",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_complete4", HFILL }},
    { &hf_umts_rrc_pdu_def_rb_PDCPContextRelocationList,
      { "rb-PDCPContextRelocationList", "umts_rrc_pdu_def.rb_PDCPContextRelocationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.RB_PDCPContextRelocationList", HFILL }},
    { &hf_umts_rrc_pdu_def_dummy_1a,
      { "dummy-1a", "umts_rrc_pdu_def.dummy_1a",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_dummy_1a_vals), 0,
        "umts_rrc_pdu_def.T_dummy_1a", HFILL }},
    { &hf_umts_rrc_pdu_def_fdd19,
      { "fdd", "umts_rrc_pdu_def.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_fdd19", HFILL }},
    { &hf_umts_rrc_pdu_def_preconfiguration3,
      { "preconfiguration", "umts_rrc_pdu_def.preconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_preconfiguration3", HFILL }},
    { &hf_umts_rrc_pdu_def_preConfigMode3,
      { "preConfigMode", "umts_rrc_pdu_def.preConfigMode",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_preConfigMode3_vals), 0,
        "umts_rrc_pdu_def.T_preConfigMode3", HFILL }},
    { &hf_umts_rrc_pdu_def_defaultConfig3,
      { "defaultConfig", "umts_rrc_pdu_def.defaultConfig",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_defaultConfig3", HFILL }},
    { &hf_umts_rrc_pdu_def_modeSpecificPhysChInfo5,
      { "modeSpecificPhysChInfo", "umts_rrc_pdu_def.modeSpecificPhysChInfo",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_modeSpecificPhysChInfo5_vals), 0,
        "umts_rrc_pdu_def.T_modeSpecificPhysChInfo5", HFILL }},
    { &hf_umts_rrc_pdu_def_fdd20,
      { "fdd", "umts_rrc_pdu_def.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_fdd20", HFILL }},
    { &hf_umts_rrc_pdu_def_specificationMode5,
      { "specificationMode", "umts_rrc_pdu_def.specificationMode",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_specificationMode5_vals), 0,
        "umts_rrc_pdu_def.T_specificationMode5", HFILL }},
    { &hf_umts_rrc_pdu_def_complete5,
      { "complete", "umts_rrc_pdu_def.complete",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_complete5", HFILL }},
    { &hf_umts_rrc_pdu_def_preconfiguration4,
      { "preconfiguration", "umts_rrc_pdu_def.preconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_preconfiguration4", HFILL }},
    { &hf_umts_rrc_pdu_def_preConfigMode4,
      { "preConfigMode", "umts_rrc_pdu_def.preConfigMode",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_preConfigMode4_vals), 0,
        "umts_rrc_pdu_def.T_preConfigMode4", HFILL }},
    { &hf_umts_rrc_pdu_def_defaultConfig4,
      { "defaultConfig", "umts_rrc_pdu_def.defaultConfig",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_defaultConfig4", HFILL }},
    { &hf_umts_rrc_pdu_def_postVerificationPeriod2,
      { "postVerificationPeriod", "umts_rrc_pdu_def.postVerificationPeriod",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_postVerificationPeriod2_vals), 0,
        "umts_rrc_pdu_def.T_postVerificationPeriod2", HFILL }},
    { &hf_umts_rrc_pdu_def_timingMaintainedSynchInd1,
      { "timingMaintainedSynchInd", "umts_rrc_pdu_def.timingMaintainedSynchInd",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_timingMaintainedSynchInd1_vals), 0,
        "umts_rrc_pdu_def.T_timingMaintainedSynchInd1", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions29,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions29", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerReconfigurationComplete_r3_add_ext,
      { "radioBearerReconfigurationComplete-r3-add-ext", "umts_rrc_pdu_def.radioBearerReconfigurationComplete_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions50,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions50", HFILL }},
    { &hf_umts_rrc_pdu_def_potentiallySuccesfulBearerList,
      { "potentiallySuccesfulBearerList", "umts_rrc_pdu_def.potentiallySuccesfulBearerList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.RB_IdentityList", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions30,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions30", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerReconfigurationFailure_r3_add_ext,
      { "radioBearerReconfigurationFailure-r3-add-ext", "umts_rrc_pdu_def.radioBearerReconfigurationFailure_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions51,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions51", HFILL }},
    { &hf_umts_rrc_pdu_def_r315,
      { "r3", "umts_rrc_pdu_def.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r315", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerRelease_r3,
      { "radioBearerRelease-r3", "umts_rrc_pdu_def.radioBearerRelease_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RadioBearerRelease_r3_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v3a0NonCriticalExtensions6,
      { "v3a0NonCriticalExtensions", "umts_rrc_pdu_def.v3a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v3a0NonCriticalExtensions6", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerRelease_v3a0ext,
      { "radioBearerRelease-v3a0ext", "umts_rrc_pdu_def.radioBearerRelease_v3a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RadioBearerRelease_v3a0ext", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions31,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions31", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerRelease_r3_add_ext,
      { "radioBearerRelease-r3-add-ext", "umts_rrc_pdu_def.radioBearerRelease_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions8,
      { "v4b0NonCriticalExtensions", "umts_rrc_pdu_def.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v4b0NonCriticalExtensions8", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerRelease_v4b0ext,
      { "radioBearerRelease-v4b0ext", "umts_rrc_pdu_def.radioBearerRelease_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RadioBearerRelease_v4b0ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions16,
      { "v590NonCriticalExtensions", "umts_rrc_pdu_def.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v590NonCriticalExtensions16", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerRelease_v590ext,
      { "radioBearerRelease-v590ext", "umts_rrc_pdu_def.radioBearerRelease_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RadioBearerRelease_v590ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions19,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions19", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerRelease_v6xyext,
      { "radioBearerRelease-v6xyext", "umts_rrc_pdu_def.radioBearerRelease_v6xyext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RadioBearerRelease_v6xyext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions52,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions52", HFILL }},
    { &hf_umts_rrc_pdu_def_later_than_r313,
      { "later-than-r3", "umts_rrc_pdu_def.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_later_than_r313", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions35,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_criticalExtensions35_vals), 0,
        "umts_rrc_pdu_def.T_criticalExtensions35", HFILL }},
    { &hf_umts_rrc_pdu_def_r47,
      { "r4", "umts_rrc_pdu_def.r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r47", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerRelease_r4,
      { "radioBearerRelease-r4", "umts_rrc_pdu_def.radioBearerRelease_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RadioBearerRelease_r4_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v4d0NonCriticalExtensions7,
      { "v4d0NonCriticalExtensions", "umts_rrc_pdu_def.v4d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v4d0NonCriticalExtensions7", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerRelease_r4_add_ext,
      { "radioBearerRelease-r4-add-ext", "umts_rrc_pdu_def.radioBearerRelease_r4_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions17,
      { "v590NonCriticalExtensions", "umts_rrc_pdu_def.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v590NonCriticalExtensions17", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions20,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions20", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions53,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions53", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions36,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_criticalExtensions36_vals), 0,
        "umts_rrc_pdu_def.T_criticalExtensions36", HFILL }},
    { &hf_umts_rrc_pdu_def_r56,
      { "r5", "umts_rrc_pdu_def.r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r56", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerRelease_r5,
      { "radioBearerRelease-r5", "umts_rrc_pdu_def.radioBearerRelease_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RadioBearerRelease_r5_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerRelease_r5_add_ext,
      { "radioBearerRelease-r5-add-ext", "umts_rrc_pdu_def.radioBearerRelease_r5_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions21,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions21", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions54,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions54", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions37,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_criticalExtensions37_vals), 0,
        "umts_rrc_pdu_def.T_criticalExtensions37", HFILL }},
    { &hf_umts_rrc_pdu_def_r67,
      { "r6", "umts_rrc_pdu_def.r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r67", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerRelease_r6,
      { "radioBearerRelease-r6", "umts_rrc_pdu_def.radioBearerRelease_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RadioBearerRelease_r6_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerRelease_r6_add_ext,
      { "radioBearerRelease-r6-add-ext", "umts_rrc_pdu_def.radioBearerRelease_r6_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions55,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions55", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions38,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_criticalExtensions38", HFILL }},
    { &hf_umts_rrc_pdu_def_signallingConnectionRelIndication,
      { "signallingConnectionRelIndication", "umts_rrc_pdu_def.signallingConnectionRelIndication",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_CN_DomainIdentity_vals), 0,
        "umts_rrc_ies.CN_DomainIdentity", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerRelease_r3_IEs_dummy,
      { "radioBearerRelease-r3-IEs-dummy", "umts_rrc_pdu_def.radioBearerRelease_r3_IEs_dummy",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_radioBearerRelease_r3_IEs_dummy_vals), 0,
        "umts_rrc_pdu_def.T_radioBearerRelease_r3_IEs_dummy", HFILL }},
    { &hf_umts_rrc_pdu_def_fdd21,
      { "fdd", "umts_rrc_pdu_def.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_fdd21", HFILL }},
    { &hf_umts_rrc_pdu_def_modeSpecificPhysChInfo6,
      { "modeSpecificPhysChInfo", "umts_rrc_pdu_def.modeSpecificPhysChInfo",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_modeSpecificPhysChInfo6_vals), 0,
        "umts_rrc_pdu_def.T_modeSpecificPhysChInfo6", HFILL }},
    { &hf_umts_rrc_pdu_def_fdd22,
      { "fdd", "umts_rrc_pdu_def.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_fdd22", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerRelease_r4_IEs_dummy,
      { "radioBearerRelease-r4-IEs-dummy", "umts_rrc_pdu_def.radioBearerRelease_r4_IEs_dummy",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_radioBearerRelease_r4_IEs_dummy_vals), 0,
        "umts_rrc_pdu_def.T_radioBearerRelease_r4_IEs_dummy", HFILL }},
    { &hf_umts_rrc_pdu_def_fdd23,
      { "fdd", "umts_rrc_pdu_def.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_fdd23", HFILL }},
    { &hf_umts_rrc_pdu_def_modeSpecificPhysChInfo7,
      { "modeSpecificPhysChInfo", "umts_rrc_pdu_def.modeSpecificPhysChInfo",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_modeSpecificPhysChInfo7_vals), 0,
        "umts_rrc_pdu_def.T_modeSpecificPhysChInfo7", HFILL }},
    { &hf_umts_rrc_pdu_def_fdd24,
      { "fdd", "umts_rrc_pdu_def.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_fdd24", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerRelease_r5_IEs_dummy,
      { "radioBearerRelease-r5-IEs-dummy", "umts_rrc_pdu_def.radioBearerRelease_r5_IEs_dummy",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_radioBearerRelease_r5_IEs_dummy_vals), 0,
        "umts_rrc_pdu_def.T_radioBearerRelease_r5_IEs_dummy", HFILL }},
    { &hf_umts_rrc_pdu_def_fdd25,
      { "fdd", "umts_rrc_pdu_def.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_fdd25", HFILL }},
    { &hf_umts_rrc_pdu_def_modeSpecificPhysChInfo8,
      { "modeSpecificPhysChInfo", "umts_rrc_pdu_def.modeSpecificPhysChInfo",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_modeSpecificPhysChInfo8_vals), 0,
        "umts_rrc_pdu_def.T_modeSpecificPhysChInfo8", HFILL }},
    { &hf_umts_rrc_pdu_def_fdd26,
      { "fdd", "umts_rrc_pdu_def.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_fdd26", HFILL }},
    { &hf_umts_rrc_pdu_def_postVerificationPeriod3,
      { "postVerificationPeriod", "umts_rrc_pdu_def.postVerificationPeriod",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_postVerificationPeriod3_vals), 0,
        "umts_rrc_pdu_def.T_postVerificationPeriod3", HFILL }},
    { &hf_umts_rrc_pdu_def_timingMaintainedSynchInd2,
      { "timingMaintainedSynchInd", "umts_rrc_pdu_def.timingMaintainedSynchInd",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_timingMaintainedSynchInd2_vals), 0,
        "umts_rrc_pdu_def.T_timingMaintainedSynchInd2", HFILL }},
    { &hf_umts_rrc_pdu_def_mbms_RB_ListReleasedToChangeTransferMode,
      { "mbms-RB-ListReleasedToChangeTransferMode", "umts_rrc_pdu_def.mbms_RB_ListReleasedToChangeTransferMode",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.RB_InformationReleaseList", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions32,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions32", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerReleaseComplete_r3_add_ext,
      { "radioBearerReleaseComplete-r3-add-ext", "umts_rrc_pdu_def.radioBearerReleaseComplete_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions56,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions56", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions33,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions33", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerReleaseFailure_r3_add_ext,
      { "radioBearerReleaseFailure-r3-add-ext", "umts_rrc_pdu_def.radioBearerReleaseFailure_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions57,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions57", HFILL }},
    { &hf_umts_rrc_pdu_def_r316,
      { "r3", "umts_rrc_pdu_def.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r316", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerSetup_r3,
      { "radioBearerSetup-r3", "umts_rrc_pdu_def.radioBearerSetup_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RadioBearerSetup_r3_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v3a0NonCriticalExtensions7,
      { "v3a0NonCriticalExtensions", "umts_rrc_pdu_def.v3a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v3a0NonCriticalExtensions7", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerSetup_v3a0ext,
      { "radioBearerSetup-v3a0ext", "umts_rrc_pdu_def.radioBearerSetup_v3a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RadioBearerSetup_v3a0ext", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions34,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions34", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerSetup_r3_add_ext,
      { "radioBearerSetup-r3-add-ext", "umts_rrc_pdu_def.radioBearerSetup_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions9,
      { "v4b0NonCriticalExtensions", "umts_rrc_pdu_def.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v4b0NonCriticalExtensions9", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerSetup_v4b0ext,
      { "radioBearerSetup-v4b0ext", "umts_rrc_pdu_def.radioBearerSetup_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RadioBearerSetup_v4b0ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions18,
      { "v590NonCriticalExtensions", "umts_rrc_pdu_def.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v590NonCriticalExtensions18", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerSetup_v590ext,
      { "radioBearerSetup-v590ext", "umts_rrc_pdu_def.radioBearerSetup_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RadioBearerSetup_v590ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v5d0NonCriticalExtenstions9,
      { "v5d0NonCriticalExtenstions", "umts_rrc_pdu_def.v5d0NonCriticalExtenstions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v5d0NonCriticalExtenstions9", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerSetup_v5d0ext,
      { "radioBearerSetup-v5d0ext", "umts_rrc_pdu_def.radioBearerSetup_v5d0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RadioBearerSetup_v5d0ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions22,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions22", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerSetup_v6xyext,
      { "radioBearerSetup-v6xyext", "umts_rrc_pdu_def.radioBearerSetup_v6xyext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RadioBearerSetup_v6xyext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions58,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions58", HFILL }},
    { &hf_umts_rrc_pdu_def_later_than_r314,
      { "later-than-r3", "umts_rrc_pdu_def.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_later_than_r314", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions39,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_criticalExtensions39_vals), 0,
        "umts_rrc_pdu_def.T_criticalExtensions39", HFILL }},
    { &hf_umts_rrc_pdu_def_r48,
      { "r4", "umts_rrc_pdu_def.r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r48", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerSetup_r4,
      { "radioBearerSetup-r4", "umts_rrc_pdu_def.radioBearerSetup_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RadioBearerSetup_r4_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v4d0NonCriticalExtensions8,
      { "v4d0NonCriticalExtensions", "umts_rrc_pdu_def.v4d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v4d0NonCriticalExtensions8", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerSetup_r4_add_ext,
      { "radioBearerSetup-r4-add-ext", "umts_rrc_pdu_def.radioBearerSetup_r4_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions19,
      { "v590NonCriticalExtensions", "umts_rrc_pdu_def.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v590NonCriticalExtensions19", HFILL }},
    { &hf_umts_rrc_pdu_def_v5d0NonCriticalExtenstions10,
      { "v5d0NonCriticalExtenstions", "umts_rrc_pdu_def.v5d0NonCriticalExtenstions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v5d0NonCriticalExtenstions10", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions23,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions23", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions59,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions59", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions40,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_criticalExtensions40_vals), 0,
        "umts_rrc_pdu_def.T_criticalExtensions40", HFILL }},
    { &hf_umts_rrc_pdu_def_r57,
      { "r5", "umts_rrc_pdu_def.r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r57", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerSetup_r5,
      { "radioBearerSetup-r5", "umts_rrc_pdu_def.radioBearerSetup_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RadioBearerSetup_r5_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerSetup_r5_add_ext,
      { "radioBearerSetup-r5-add-ext", "umts_rrc_pdu_def.radioBearerSetup_r5_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v5d0NonCriticalExtenstions11,
      { "v5d0NonCriticalExtenstions", "umts_rrc_pdu_def.v5d0NonCriticalExtenstions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v5d0NonCriticalExtenstions11", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions24,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions24", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions60,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions60", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions41,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_criticalExtensions41_vals), 0,
        "umts_rrc_pdu_def.T_criticalExtensions41", HFILL }},
    { &hf_umts_rrc_pdu_def_r68,
      { "r6", "umts_rrc_pdu_def.r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r68", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerSetup_r6,
      { "radioBearerSetup-r6", "umts_rrc_pdu_def.radioBearerSetup_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RadioBearerSetup_r6_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerSetup_r6_add_ext,
      { "radioBearerSetup-r6-add-ext", "umts_rrc_pdu_def.radioBearerSetup_r6_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions61,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions61", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions42,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_criticalExtensions42", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerSetup_r3_IEs_dummy,
      { "radioBearerSetup-r3-IEs-dummy", "umts_rrc_pdu_def.radioBearerSetup_r3_IEs_dummy",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_radioBearerSetup_r3_IEs_dummy_vals), 0,
        "umts_rrc_pdu_def.T_radioBearerSetup_r3_IEs_dummy", HFILL }},
    { &hf_umts_rrc_pdu_def_fdd27,
      { "fdd", "umts_rrc_pdu_def.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_fdd27", HFILL }},
    { &hf_umts_rrc_pdu_def_modeSpecificPhysChInfo9,
      { "modeSpecificPhysChInfo", "umts_rrc_pdu_def.modeSpecificPhysChInfo",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_modeSpecificPhysChInfo9_vals), 0,
        "umts_rrc_pdu_def.T_modeSpecificPhysChInfo9", HFILL }},
    { &hf_umts_rrc_pdu_def_fdd28,
      { "fdd", "umts_rrc_pdu_def.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_fdd28", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerSetup_r4_IEs_dummy,
      { "radioBearerSetup-r4-IEs-dummy", "umts_rrc_pdu_def.radioBearerSetup_r4_IEs_dummy",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_radioBearerSetup_r4_IEs_dummy_vals), 0,
        "umts_rrc_pdu_def.T_radioBearerSetup_r4_IEs_dummy", HFILL }},
    { &hf_umts_rrc_pdu_def_fdd29,
      { "fdd", "umts_rrc_pdu_def.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_fdd29", HFILL }},
    { &hf_umts_rrc_pdu_def_modeSpecificPhysChInfo10,
      { "modeSpecificPhysChInfo", "umts_rrc_pdu_def.modeSpecificPhysChInfo",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_modeSpecificPhysChInfo10_vals), 0,
        "umts_rrc_pdu_def.T_modeSpecificPhysChInfo10", HFILL }},
    { &hf_umts_rrc_pdu_def_fdd30,
      { "fdd", "umts_rrc_pdu_def.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_fdd30", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerSetup_r5_IEs_dummy,
      { "radioBearerSetup-r5-IEs-dummy", "umts_rrc_pdu_def.radioBearerSetup_r5_IEs_dummy",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_radioBearerSetup_r5_IEs_dummy_vals), 0,
        "umts_rrc_pdu_def.T_radioBearerSetup_r5_IEs_dummy", HFILL }},
    { &hf_umts_rrc_pdu_def_fdd31,
      { "fdd", "umts_rrc_pdu_def.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_fdd31", HFILL }},
    { &hf_umts_rrc_pdu_def_modeSpecificPhysChInfo11,
      { "modeSpecificPhysChInfo", "umts_rrc_pdu_def.modeSpecificPhysChInfo",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_modeSpecificPhysChInfo11_vals), 0,
        "umts_rrc_pdu_def.T_modeSpecificPhysChInfo11", HFILL }},
    { &hf_umts_rrc_pdu_def_fdd32,
      { "fdd", "umts_rrc_pdu_def.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_fdd32", HFILL }},
    { &hf_umts_rrc_pdu_def_postVerificationPeriod4,
      { "postVerificationPeriod", "umts_rrc_pdu_def.postVerificationPeriod",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_postVerificationPeriod4_vals), 0,
        "umts_rrc_pdu_def.T_postVerificationPeriod4", HFILL }},
    { &hf_umts_rrc_pdu_def_timingMaintainedSynchInd3,
      { "timingMaintainedSynchInd", "umts_rrc_pdu_def.timingMaintainedSynchInd",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_timingMaintainedSynchInd3_vals), 0,
        "umts_rrc_pdu_def.T_timingMaintainedSynchInd3", HFILL }},
    { &hf_umts_rrc_pdu_def_rab_InformationSetupList4,
      { "rab-InformationSetupList", "umts_rrc_pdu_def.rab_InformationSetupList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.RAB_InformationSetupList_r6_ext", HFILL }},
    { &hf_umts_rrc_pdu_def_specificationMode6,
      { "specificationMode", "umts_rrc_pdu_def.specificationMode",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_specificationMode6_vals), 0,
        "umts_rrc_pdu_def.T_specificationMode6", HFILL }},
    { &hf_umts_rrc_pdu_def_complete6,
      { "complete", "umts_rrc_pdu_def.complete",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_complete6", HFILL }},
    { &hf_umts_rrc_pdu_def_preconfiguration5,
      { "preconfiguration", "umts_rrc_pdu_def.preconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_preconfiguration5", HFILL }},
    { &hf_umts_rrc_pdu_def_rab_Info1,
      { "rab-Info", "umts_rrc_pdu_def.rab_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.RAB_Info", HFILL }},
    { &hf_umts_rrc_pdu_def_rb_InformationChangedList,
      { "rb-InformationChangedList", "umts_rrc_pdu_def.rb_InformationChangedList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.RB_InformationChangedList_r6", HFILL }},
    { &hf_umts_rrc_pdu_def_powerOffsetInfoShort,
      { "powerOffsetInfoShort", "umts_rrc_pdu_def.powerOffsetInfoShort",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.PowerOffsetInfoShort", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions35,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions35", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerSetupComplete_r3_add_ext,
      { "radioBearerSetupComplete-r3-add-ext", "umts_rrc_pdu_def.radioBearerSetupComplete_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions62,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions62", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions36,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions36", HFILL }},
    { &hf_umts_rrc_pdu_def_radioBearerSetupFailure_r3_add_ext,
      { "radioBearerSetupFailure-r3-add-ext", "umts_rrc_pdu_def.radioBearerSetupFailure_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions63,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions63", HFILL }},
    { &hf_umts_rrc_pdu_def_r317,
      { "r3", "umts_rrc_pdu_def.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r317", HFILL }},
    { &hf_umts_rrc_pdu_def_rrcConnectionReject_r3,
      { "rrcConnectionReject-r3", "umts_rrc_pdu_def.rrcConnectionReject_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RRCConnectionReject_r3_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions37,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions37", HFILL }},
    { &hf_umts_rrc_pdu_def_rrcConnectionReject_r3_add_ext,
      { "rrcConnectionReject-r3-add-ext", "umts_rrc_pdu_def.rrcConnectionReject_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions25,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions25", HFILL }},
    { &hf_umts_rrc_pdu_def_rrcConnectionReject_v6xyext,
      { "rrcConnectionReject-v6xyext", "umts_rrc_pdu_def.rrcConnectionReject_v6xyext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RRCConnectionReject_v6xyext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions64,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions64", HFILL }},
    { &hf_umts_rrc_pdu_def_later_than_r315,
      { "later-than-r3", "umts_rrc_pdu_def.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_later_than_r315", HFILL }},
    { &hf_umts_rrc_pdu_def_initialUE_Identity,
      { "initialUE-Identity", "umts_rrc_pdu_def.initialUE_Identity",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_InitialUE_Identity_vals), 0,
        "umts_rrc_ies.InitialUE_Identity", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions43,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_criticalExtensions43", HFILL }},
    { &hf_umts_rrc_pdu_def_rejectionCause,
      { "rejectionCause", "umts_rrc_pdu_def.rejectionCause",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_RejectionCause_vals), 0,
        "umts_rrc_ies.RejectionCause", HFILL }},
    { &hf_umts_rrc_pdu_def_waitTime,
      { "waitTime", "umts_rrc_pdu_def.waitTime",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.WaitTime", HFILL }},
    { &hf_umts_rrc_pdu_def_redirectionInfo,
      { "redirectionInfo", "umts_rrc_pdu_def.redirectionInfo",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_RedirectionInfo_vals), 0,
        "umts_rrc_ies.RedirectionInfo", HFILL }},
    { &hf_umts_rrc_pdu_def_redirectionInfo_v6xyext,
      { "redirectionInfo-v6xyext", "umts_rrc_pdu_def.redirectionInfo_v6xyext",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.GSM_TargetCellInfoList", HFILL }},
    { &hf_umts_rrc_pdu_def_r318,
      { "r3", "umts_rrc_pdu_def.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r318", HFILL }},
    { &hf_umts_rrc_pdu_def_rrcConnectionRelease_r3,
      { "rrcConnectionRelease-r3", "umts_rrc_pdu_def.rrcConnectionRelease_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RRCConnectionRelease_r3_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions38,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions38", HFILL }},
    { &hf_umts_rrc_pdu_def_rrcConnectionRelease_r3_add_ext,
      { "rrcConnectionRelease-r3-add-ext", "umts_rrc_pdu_def.rrcConnectionRelease_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions26,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions26", HFILL }},
    { &hf_umts_rrc_pdu_def_rrcConnectionRelease_v6xyext,
      { "rrcConnectionRelease-v6xyext", "umts_rrc_pdu_def.rrcConnectionRelease_v6xyext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RRCConnectionRelease_v6xyext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions65,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions65", HFILL }},
    { &hf_umts_rrc_pdu_def_later_than_r316,
      { "later-than-r3", "umts_rrc_pdu_def.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_later_than_r316", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions44,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_criticalExtensions44_vals), 0,
        "umts_rrc_pdu_def.T_criticalExtensions44", HFILL }},
    { &hf_umts_rrc_pdu_def_r49,
      { "r4", "umts_rrc_pdu_def.r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r49", HFILL }},
    { &hf_umts_rrc_pdu_def_rrcConnectionRelease_r4,
      { "rrcConnectionRelease-r4", "umts_rrc_pdu_def.rrcConnectionRelease_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RRCConnectionRelease_r4_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v4d0NonCriticalExtensions9,
      { "v4d0NonCriticalExtensions", "umts_rrc_pdu_def.v4d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v4d0NonCriticalExtensions9", HFILL }},
    { &hf_umts_rrc_pdu_def_rrcConnectionRelease_r4_add_ext,
      { "rrcConnectionRelease-r4-add-ext", "umts_rrc_pdu_def.rrcConnectionRelease_r4_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions27,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions27", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions66,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions66", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions45,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_criticalExtensions45", HFILL }},
    { &hf_umts_rrc_pdu_def_n_308,
      { "n-308", "umts_rrc_pdu_def.n_308",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.N_308", HFILL }},
    { &hf_umts_rrc_pdu_def_releaseCause,
      { "releaseCause", "umts_rrc_pdu_def.releaseCause",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_ReleaseCause_vals), 0,
        "umts_rrc_ies.ReleaseCause", HFILL }},
    { &hf_umts_rrc_pdu_def_rplmn_information,
      { "rplmn-information", "umts_rrc_pdu_def.rplmn_information",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.Rplmn_Information", HFILL }},
    { &hf_umts_rrc_pdu_def_rplmn_information1,
      { "rplmn-information", "umts_rrc_pdu_def.rplmn_information",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.Rplmn_Information_r4", HFILL }},
    { &hf_umts_rrc_pdu_def_redirectionInfo_v6xyext1,
      { "redirectionInfo-v6xyext", "umts_rrc_pdu_def.redirectionInfo_v6xyext",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_RedirectionInfo_r6_vals), 0,
        "umts_rrc_ies.RedirectionInfo_r6", HFILL }},
    { &hf_umts_rrc_pdu_def_r319,
      { "r3", "umts_rrc_pdu_def.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r319", HFILL }},
    { &hf_umts_rrc_pdu_def_rrcConnectionRelease_CCCH_r3,
      { "rrcConnectionRelease-CCCH-r3", "umts_rrc_pdu_def.rrcConnectionRelease_CCCH_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RRCConnectionRelease_CCCH_r3_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions39,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions39", HFILL }},
    { &hf_umts_rrc_pdu_def_rrcConnectionRelease_CCCH_r3_add_ext,
      { "rrcConnectionRelease-CCCH-r3-add-ext", "umts_rrc_pdu_def.rrcConnectionRelease_CCCH_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions67,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions67", HFILL }},
    { &hf_umts_rrc_pdu_def_later_than_r317,
      { "later-than-r3", "umts_rrc_pdu_def.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_later_than_r317", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions46,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_criticalExtensions46_vals), 0,
        "umts_rrc_pdu_def.T_criticalExtensions46", HFILL }},
    { &hf_umts_rrc_pdu_def_r410,
      { "r4", "umts_rrc_pdu_def.r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r410", HFILL }},
    { &hf_umts_rrc_pdu_def_rrcConnectionRelease_CCCH_r4,
      { "rrcConnectionRelease-CCCH-r4", "umts_rrc_pdu_def.rrcConnectionRelease_CCCH_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RRCConnectionRelease_CCCH_r4_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v4d0NonCriticalExtensions10,
      { "v4d0NonCriticalExtensions", "umts_rrc_pdu_def.v4d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v4d0NonCriticalExtensions10", HFILL }},
    { &hf_umts_rrc_pdu_def_rrcConnectionRelease_CCCH_r4_add_ext,
      { "rrcConnectionRelease-CCCH-r4-add-ext", "umts_rrc_pdu_def.rrcConnectionRelease_CCCH_r4_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions68,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions68", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions47,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_criticalExtensions47", HFILL }},
    { &hf_umts_rrc_pdu_def_groupIdentity,
      { "groupIdentity", "umts_rrc_pdu_def.groupIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_pdu_def.SEQUENCE_SIZE_1_maxURNTIGroup_OF_GroupReleaseInformation", HFILL }},
    { &hf_umts_rrc_pdu_def_groupIdentity_item,
      { "Item", "umts_rrc_pdu_def.groupIdentity_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.GroupReleaseInformation", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions48,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_criticalExtensions48_vals), 0,
        "umts_rrc_pdu_def.T_criticalExtensions48", HFILL }},
    { &hf_umts_rrc_pdu_def_r58,
      { "r5", "umts_rrc_pdu_def.r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r58", HFILL }},
    { &hf_umts_rrc_pdu_def_rrcConnectionRelease_CCCH_r5,
      { "rrcConnectionRelease-CCCH-r5", "umts_rrc_pdu_def.rrcConnectionRelease_CCCH_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RRCConnectionRelease_CCCH_r5_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_rrcConnectionRelease_CCCH_r5_add_ext,
      { "rrcConnectionRelease-CCCH-r5-add-ext", "umts_rrc_pdu_def.rrcConnectionRelease_CCCH_r5_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions69,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions69", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions49,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_criticalExtensions49", HFILL }},
    { &hf_umts_rrc_pdu_def_rrcConnectionRelease,
      { "rrcConnectionRelease", "umts_rrc_pdu_def.rrcConnectionRelease",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RRCConnectionRelease_r3_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_rrcConnectionRelease1,
      { "rrcConnectionRelease", "umts_rrc_pdu_def.rrcConnectionRelease",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RRCConnectionRelease_r4_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_errorIndication,
      { "errorIndication", "umts_rrc_pdu_def.errorIndication",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_FailureCauseWithProtErr_vals), 0,
        "umts_rrc_ies.FailureCauseWithProtErr", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions40,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions40", HFILL }},
    { &hf_umts_rrc_pdu_def_rrcConnectionReleaseComplete_r3_add_ext,
      { "rrcConnectionReleaseComplete-r3-add-ext", "umts_rrc_pdu_def.rrcConnectionReleaseComplete_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions70,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions70", HFILL }},
    { &hf_umts_rrc_pdu_def_protocolErrorIndicator1,
      { "protocolErrorIndicator", "umts_rrc_pdu_def.protocolErrorIndicator",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_ProtocolErrorIndicator_vals), 0,
        "umts_rrc_ies.ProtocolErrorIndicator", HFILL }},
    { &hf_umts_rrc_pdu_def_v3d0NonCriticalExtensions,
      { "v3d0NonCriticalExtensions", "umts_rrc_pdu_def.v3d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v3d0NonCriticalExtensions", HFILL }},
    { &hf_umts_rrc_pdu_def_rRCConnectionRequest_v3d0ext,
      { "rRCConnectionRequest-v3d0ext", "umts_rrc_pdu_def.rRCConnectionRequest_v3d0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RRCConnectionRequest_v3d0ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions10,
      { "v4b0NonCriticalExtensions", "umts_rrc_pdu_def.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v4b0NonCriticalExtensions10", HFILL }},
    { &hf_umts_rrc_pdu_def_rrcConnectionRequest_v4b0ext,
      { "rrcConnectionRequest-v4b0ext", "umts_rrc_pdu_def.rrcConnectionRequest_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RRCConnectionRequest_v4b0ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions20,
      { "v590NonCriticalExtensions", "umts_rrc_pdu_def.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v590NonCriticalExtensions20", HFILL }},
    { &hf_umts_rrc_pdu_def_rrcConnectionRequest_v590ext,
      { "rrcConnectionRequest-v590ext", "umts_rrc_pdu_def.rrcConnectionRequest_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RRCConnectionRequest_v590ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions28,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions28", HFILL }},
    { &hf_umts_rrc_pdu_def_rrcConnectionRequest_v6xyext,
      { "rrcConnectionRequest-v6xyext", "umts_rrc_pdu_def.rrcConnectionRequest_v6xyext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RRCConnectionRequest_v6xyext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions71,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions71", HFILL }},
    { &hf_umts_rrc_pdu_def_uESpecificBehaviourInformation1idle,
      { "uESpecificBehaviourInformation1idle", "umts_rrc_pdu_def.uESpecificBehaviourInformation1idle",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_ies.UESpecificBehaviourInformation1idle", HFILL }},
    { &hf_umts_rrc_pdu_def_predefinedConfigStatusInfo,
      { "predefinedConfigStatusInfo", "umts_rrc_pdu_def.predefinedConfigStatusInfo",
        FT_BOOLEAN, 8, NULL, 0,
        "umts_rrc_pdu_def.BOOLEAN", HFILL }},
    { &hf_umts_rrc_pdu_def_ueCapabilityIndication,
      { "ueCapabilityIndication", "umts_rrc_pdu_def.ueCapabilityIndication",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_ueCapabilityIndication_vals), 0,
        "umts_rrc_pdu_def.T_ueCapabilityIndication", HFILL }},
    { &hf_umts_rrc_pdu_def_r320,
      { "r3", "umts_rrc_pdu_def.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r320", HFILL }},
    { &hf_umts_rrc_pdu_def_rrcConnectionSetup_r3,
      { "rrcConnectionSetup-r3", "umts_rrc_pdu_def.rrcConnectionSetup_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RRCConnectionSetup_r3_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions41,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions41", HFILL }},
    { &hf_umts_rrc_pdu_def_rrcConnectionSetup_r3_add_ext,
      { "rrcConnectionSetup-r3-add-ext", "umts_rrc_pdu_def.rrcConnectionSetup_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions11,
      { "v4b0NonCriticalExtensions", "umts_rrc_pdu_def.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v4b0NonCriticalExtensions11", HFILL }},
    { &hf_umts_rrc_pdu_def_rrcConnectionSetup_v4b0ext,
      { "rrcConnectionSetup-v4b0ext", "umts_rrc_pdu_def.rrcConnectionSetup_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RRCConnectionSetup_v4b0ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions21,
      { "v590NonCriticalExtensions", "umts_rrc_pdu_def.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v590NonCriticalExtensions21", HFILL }},
    { &hf_umts_rrc_pdu_def_rrcConnectionSetup_v590ext,
      { "rrcConnectionSetup-v590ext", "umts_rrc_pdu_def.rrcConnectionSetup_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RRCConnectionSetup_v590ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions72,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions72", HFILL }},
    { &hf_umts_rrc_pdu_def_later_than_r318,
      { "later-than-r3", "umts_rrc_pdu_def.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_later_than_r318", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions50,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_criticalExtensions50_vals), 0,
        "umts_rrc_pdu_def.T_criticalExtensions50", HFILL }},
    { &hf_umts_rrc_pdu_def_r411,
      { "r4", "umts_rrc_pdu_def.r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r411", HFILL }},
    { &hf_umts_rrc_pdu_def_rrcConnectionSetup_r4,
      { "rrcConnectionSetup-r4", "umts_rrc_pdu_def.rrcConnectionSetup_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RRCConnectionSetup_r4_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v4d0NonCriticalExtensions11,
      { "v4d0NonCriticalExtensions", "umts_rrc_pdu_def.v4d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v4d0NonCriticalExtensions11", HFILL }},
    { &hf_umts_rrc_pdu_def_rrcConnectionSetup_r4_add_ext,
      { "rrcConnectionSetup-r4-add-ext", "umts_rrc_pdu_def.rrcConnectionSetup_r4_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions22,
      { "v590NonCriticalExtensions", "umts_rrc_pdu_def.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v590NonCriticalExtensions22", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions29,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions29", HFILL }},
    { &hf_umts_rrc_pdu_def_rrcConnectionSetup_v6xyext,
      { "rrcConnectionSetup-v6xyext", "umts_rrc_pdu_def.rrcConnectionSetup_v6xyext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RRCConnectionSetup_v6xyext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions73,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions73", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions51,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_criticalExtensions51_vals), 0,
        "umts_rrc_pdu_def.T_criticalExtensions51", HFILL }},
    { &hf_umts_rrc_pdu_def_r59,
      { "r5", "umts_rrc_pdu_def.r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r59", HFILL }},
    { &hf_umts_rrc_pdu_def_rrcConnectionSetup_r5,
      { "rrcConnectionSetup-r5", "umts_rrc_pdu_def.rrcConnectionSetup_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RRCConnectionSetup_r5_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_rrcConnectionSetup_r5_add_ext,
      { "rrcConnectionSetup-r5-add-ext", "umts_rrc_pdu_def.rrcConnectionSetup_r5_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions30,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions30", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions74,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions74", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions52,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_criticalExtensions52_vals), 0,
        "umts_rrc_pdu_def.T_criticalExtensions52", HFILL }},
    { &hf_umts_rrc_pdu_def_r69,
      { "r6", "umts_rrc_pdu_def.r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r69", HFILL }},
    { &hf_umts_rrc_pdu_def_rrcConnectionSetup_r6,
      { "rrcConnectionSetup-r6", "umts_rrc_pdu_def.rrcConnectionSetup_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RRCConnectionSetup_r6_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_rrcConnectionSetup_r6_add_ext,
      { "rrcConnectionSetup-r6-add-ext", "umts_rrc_pdu_def.rrcConnectionSetup_r6_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions75,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions75", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions53,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_criticalExtensions53", HFILL }},
    { &hf_umts_rrc_pdu_def_new_c_RNTI,
      { "new-c-RNTI", "umts_rrc_pdu_def.new_c_RNTI",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_ies.C_RNTI", HFILL }},
    { &hf_umts_rrc_pdu_def_capabilityUpdateRequirement,
      { "capabilityUpdateRequirement", "umts_rrc_pdu_def.capabilityUpdateRequirement",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.CapabilityUpdateRequirement", HFILL }},
    { &hf_umts_rrc_pdu_def_srb_InformationSetupList3,
      { "srb-InformationSetupList", "umts_rrc_pdu_def.srb_InformationSetupList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.SRB_InformationSetupList2", HFILL }},
    { &hf_umts_rrc_pdu_def_capabilityUpdateRequirement_r4_ext,
      { "capabilityUpdateRequirement-r4-ext", "umts_rrc_pdu_def.capabilityUpdateRequirement_r4_ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.CapabilityUpdateRequirement_r4_ext", HFILL }},
    { &hf_umts_rrc_pdu_def_systemSpecificCapUpdateReq,
      { "systemSpecificCapUpdateReq", "umts_rrc_pdu_def.systemSpecificCapUpdateReq",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_SystemSpecificCapUpdateReq_v590ext_vals), 0,
        "umts_rrc_ies.SystemSpecificCapUpdateReq_v590ext", HFILL }},
    { &hf_umts_rrc_pdu_def_capabilityUpdateRequirement1,
      { "capabilityUpdateRequirement", "umts_rrc_pdu_def.capabilityUpdateRequirement",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.CapabilityUpdateRequirement_r4", HFILL }},
    { &hf_umts_rrc_pdu_def_capabilityUpdateRequirement2,
      { "capabilityUpdateRequirement", "umts_rrc_pdu_def.capabilityUpdateRequirement",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.CapabilityUpdateRequirement_r5", HFILL }},
    { &hf_umts_rrc_pdu_def_specificationMode7,
      { "specificationMode", "umts_rrc_pdu_def.specificationMode",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_specificationMode7_vals), 0,
        "umts_rrc_pdu_def.T_specificationMode7", HFILL }},
    { &hf_umts_rrc_pdu_def_complete7,
      { "complete", "umts_rrc_pdu_def.complete",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_complete7", HFILL }},
    { &hf_umts_rrc_pdu_def_preconfiguration6,
      { "preconfiguration", "umts_rrc_pdu_def.preconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_preconfiguration6", HFILL }},
    { &hf_umts_rrc_pdu_def_preConfigMode5,
      { "preConfigMode", "umts_rrc_pdu_def.preConfigMode",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_preConfigMode5_vals), 0,
        "umts_rrc_pdu_def.T_preConfigMode5", HFILL }},
    { &hf_umts_rrc_pdu_def_defaultConfig5,
      { "defaultConfig", "umts_rrc_pdu_def.defaultConfig",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_defaultConfig5", HFILL }},
    { &hf_umts_rrc_pdu_def_dl_InformationPerRL_List5,
      { "dl-InformationPerRL-List", "umts_rrc_pdu_def.dl_InformationPerRL_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.DL_InformationPerRL_List_r5bis", HFILL }},
    { &hf_umts_rrc_pdu_def_postVerificationPeriod5,
      { "postVerificationPeriod", "umts_rrc_pdu_def.postVerificationPeriod",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_postVerificationPeriod5_vals), 0,
        "umts_rrc_pdu_def.T_postVerificationPeriod5", HFILL }},
    { &hf_umts_rrc_pdu_def_specificationMode8,
      { "specificationMode", "umts_rrc_pdu_def.specificationMode",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_specificationMode8_vals), 0,
        "umts_rrc_pdu_def.T_specificationMode8", HFILL }},
    { &hf_umts_rrc_pdu_def_complete8,
      { "complete", "umts_rrc_pdu_def.complete",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_complete8", HFILL }},
    { &hf_umts_rrc_pdu_def_preconfiguration7,
      { "preconfiguration", "umts_rrc_pdu_def.preconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_preconfiguration7", HFILL }},
    { &hf_umts_rrc_pdu_def_preConfigMode6,
      { "preConfigMode", "umts_rrc_pdu_def.preConfigMode",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_preConfigMode6_vals), 0,
        "umts_rrc_pdu_def.T_preConfigMode6", HFILL }},
    { &hf_umts_rrc_pdu_def_defaultConfig6,
      { "defaultConfig", "umts_rrc_pdu_def.defaultConfig",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_defaultConfig6", HFILL }},
    { &hf_umts_rrc_pdu_def_ue_RadioAccessCapability,
      { "ue-RadioAccessCapability", "umts_rrc_pdu_def.ue_RadioAccessCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.UE_RadioAccessCapability", HFILL }},
    { &hf_umts_rrc_pdu_def_ue_RATSpecificCapability,
      { "ue-RATSpecificCapability", "umts_rrc_pdu_def.ue_RATSpecificCapability",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.InterRAT_UE_RadioAccessCapabilityList", HFILL }},
    { &hf_umts_rrc_pdu_def_v370NonCriticalExtensions,
      { "v370NonCriticalExtensions", "umts_rrc_pdu_def.v370NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v370NonCriticalExtensions", HFILL }},
    { &hf_umts_rrc_pdu_def_rrcConnectionSetupComplete_v370ext,
      { "rrcConnectionSetupComplete-v370ext", "umts_rrc_pdu_def.rrcConnectionSetupComplete_v370ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RRCConnectionSetupComplete_v370ext", HFILL }},
    { &hf_umts_rrc_pdu_def_v380NonCriticalExtensions,
      { "v380NonCriticalExtensions", "umts_rrc_pdu_def.v380NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v380NonCriticalExtensions", HFILL }},
    { &hf_umts_rrc_pdu_def_rrcConnectionSetupComplete_v380ext,
      { "rrcConnectionSetupComplete-v380ext", "umts_rrc_pdu_def.rrcConnectionSetupComplete_v380ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RRCConnectionSetupComplete_v380ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v3a0NonCriticalExtensions8,
      { "v3a0NonCriticalExtensions", "umts_rrc_pdu_def.v3a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v3a0NonCriticalExtensions8", HFILL }},
    { &hf_umts_rrc_pdu_def_rrcConnectionSetupComplete_v3a0ext,
      { "rrcConnectionSetupComplete-v3a0ext", "umts_rrc_pdu_def.rrcConnectionSetupComplete_v3a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RRCConnectionSetupComplete_v3a0ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions42,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions42", HFILL }},
    { &hf_umts_rrc_pdu_def_rrcConnectionSetupComplete_r3_add_ext,
      { "rrcConnectionSetupComplete-r3-add-ext", "umts_rrc_pdu_def.rrcConnectionSetupComplete_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v3g0NonCriticalExtensions1,
      { "v3g0NonCriticalExtensions", "umts_rrc_pdu_def.v3g0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v3g0NonCriticalExtensions1", HFILL }},
    { &hf_umts_rrc_pdu_def_rrcConnectionSetupComplete_v3g0ext,
      { "rrcConnectionSetupComplete-v3g0ext", "umts_rrc_pdu_def.rrcConnectionSetupComplete_v3g0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RRCConnectionSetupComplete_v3g0ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions12,
      { "v4b0NonCriticalExtensions", "umts_rrc_pdu_def.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v4b0NonCriticalExtensions12", HFILL }},
    { &hf_umts_rrc_pdu_def_rrcConnectionSetupComplete_v4b0ext,
      { "rrcConnectionSetupComplete-v4b0ext", "umts_rrc_pdu_def.rrcConnectionSetupComplete_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RRCConnectionSetupComplete_v4b0ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions23,
      { "v590NonCriticalExtensions", "umts_rrc_pdu_def.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v590NonCriticalExtensions23", HFILL }},
    { &hf_umts_rrc_pdu_def_rrcConnectionSetupComplete_v590ext,
      { "rrcConnectionSetupComplete-v590ext", "umts_rrc_pdu_def.rrcConnectionSetupComplete_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RRCConnectionSetupComplete_v590ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v5c0NonCriticalExtensions,
      { "v5c0NonCriticalExtensions", "umts_rrc_pdu_def.v5c0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v5c0NonCriticalExtensions", HFILL }},
    { &hf_umts_rrc_pdu_def_rrcConnectionSetupComplete_v5c0ext,
      { "rrcConnectionSetupComplete-v5c0ext", "umts_rrc_pdu_def.rrcConnectionSetupComplete_v5c0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RRCConnectionSetupComplete_v5c0ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions31,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions31", HFILL }},
    { &hf_umts_rrc_pdu_def_rrcConnectionSetupComplete_v6xyext,
      { "rrcConnectionSetupComplete-v6xyext", "umts_rrc_pdu_def.rrcConnectionSetupComplete_v6xyext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RRCConnectionSetupComplete_v6xyext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions76,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions76", HFILL }},
    { &hf_umts_rrc_pdu_def_ue_RadioAccessCapability_v370ext,
      { "ue-RadioAccessCapability-v370ext", "umts_rrc_pdu_def.ue_RadioAccessCapability_v370ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.UE_RadioAccessCapability_v370ext", HFILL }},
    { &hf_umts_rrc_pdu_def_rrcConnectionSetupComplete_v650ext,
      { "rrcConnectionSetupComplete-v650ext", "umts_rrc_pdu_def.rrcConnectionSetupComplete_v650ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RRCConnectionSetupComplete_v650ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions77,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions77", HFILL }},
    { &hf_umts_rrc_pdu_def_ue_RadioAccessCapability_v4b0ext,
      { "ue-RadioAccessCapability-v4b0ext", "umts_rrc_pdu_def.ue_RadioAccessCapability_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.UE_RadioAccessCapability_v4b0ext", HFILL }},
    { &hf_umts_rrc_pdu_def_ue_RadioAccessCapability_v590ext,
      { "ue-RadioAccessCapability-v590ext", "umts_rrc_pdu_def.ue_RadioAccessCapability_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.UE_RadioAccessCapability_v590ext", HFILL }},
    { &hf_umts_rrc_pdu_def_ue_RATSpecificCapability_v590ext,
      { "ue-RATSpecificCapability-v590ext", "umts_rrc_pdu_def.ue_RATSpecificCapability_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.InterRAT_UE_RadioAccessCapability_v590ext", HFILL }},
    { &hf_umts_rrc_pdu_def_ue_RadioAccessCapability_v5c0ext,
      { "ue-RadioAccessCapability-v5c0ext", "umts_rrc_pdu_def.ue_RadioAccessCapability_v5c0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.UE_RadioAccessCapability_v5c0ext", HFILL }},
    { &hf_umts_rrc_pdu_def_ue_RadioAccessCapability_v650ext,
      { "ue-RadioAccessCapability-v650ext", "umts_rrc_pdu_def.ue_RadioAccessCapability_v650ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.UE_RadioAccessCapability_v650ext", HFILL }},
    { &hf_umts_rrc_pdu_def_ueCapabilityContainer,
      { "ueCapabilityContainer", "umts_rrc_pdu_def.ueCapabilityContainer",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_r321,
      { "r3", "umts_rrc_pdu_def.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r321", HFILL }},
    { &hf_umts_rrc_pdu_def_rRC_FailureInfo_r3,
      { "rRC-FailureInfo-r3", "umts_rrc_pdu_def.rRC_FailureInfo_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.RRC_FailureInfo_r3_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions43,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions43", HFILL }},
    { &hf_umts_rrc_pdu_def_rrc_FailureInfo_r3_add_ext,
      { "rrc-FailureInfo-r3-add-ext", "umts_rrc_pdu_def.rrc_FailureInfo_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions78,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions78", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions54,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_criticalExtensions54", HFILL }},
    { &hf_umts_rrc_pdu_def_failureCauseWithProtErr,
      { "failureCauseWithProtErr", "umts_rrc_pdu_def.failureCauseWithProtErr",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_FailureCauseWithProtErr_vals), 0,
        "umts_rrc_ies.FailureCauseWithProtErr", HFILL }},
    { &hf_umts_rrc_pdu_def_protocolErrorInformation,
      { "protocolErrorInformation", "umts_rrc_pdu_def.protocolErrorInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.ProtocolErrorMoreInformation", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions44,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions44", HFILL }},
    { &hf_umts_rrc_pdu_def_rrcStatus_r3_add_ext,
      { "rrcStatus-r3-add-ext", "umts_rrc_pdu_def.rrcStatus_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions79,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions79", HFILL }},
    { &hf_umts_rrc_pdu_def_r322,
      { "r3", "umts_rrc_pdu_def.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r322", HFILL }},
    { &hf_umts_rrc_pdu_def_securityModeCommand_r3,
      { "securityModeCommand-r3", "umts_rrc_pdu_def.securityModeCommand_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.SecurityModeCommand_r3_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions45,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions45", HFILL }},
    { &hf_umts_rrc_pdu_def_securityModeCommand_r3_add_ext,
      { "securityModeCommand-r3-add-ext", "umts_rrc_pdu_def.securityModeCommand_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions32,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions32", HFILL }},
    { &hf_umts_rrc_pdu_def_securityModeCommand_v6xyext,
      { "securityModeCommand-v6xyext", "umts_rrc_pdu_def.securityModeCommand_v6xyext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.SecurityModeCommand_v6xyext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions80,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions80", HFILL }},
    { &hf_umts_rrc_pdu_def_later_than_r319,
      { "later-than-r3", "umts_rrc_pdu_def.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_later_than_r319", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions55,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_criticalExtensions55", HFILL }},
    { &hf_umts_rrc_pdu_def_securityCapability,
      { "securityCapability", "umts_rrc_pdu_def.securityCapability",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.SecurityCapability", HFILL }},
    { &hf_umts_rrc_pdu_def_ue_SystemSpecificSecurityCap,
      { "ue-SystemSpecificSecurityCap", "umts_rrc_pdu_def.ue_SystemSpecificSecurityCap",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.InterRAT_UE_SecurityCapList", HFILL }},
    { &hf_umts_rrc_pdu_def_gsmSecurityCapability_v6xyext,
      { "gsmSecurityCapability-v6xyext", "umts_rrc_pdu_def.gsmSecurityCapability_v6xyext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_ies.GSMSecurityCapability_v6xyext", HFILL }},
    { &hf_umts_rrc_pdu_def_rb_UL_CiphActivationTimeInfo,
      { "rb-UL-CiphActivationTimeInfo", "umts_rrc_pdu_def.rb_UL_CiphActivationTimeInfo",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.RB_ActivationTimeInfoList", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions46,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions46", HFILL }},
    { &hf_umts_rrc_pdu_def_securityModeComplete_r3_add_ext,
      { "securityModeComplete-r3-add-ext", "umts_rrc_pdu_def.securityModeComplete_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions81,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions81", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions47,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions47", HFILL }},
    { &hf_umts_rrc_pdu_def_securityModeFailure_r3_add_ext,
      { "securityModeFailure-r3-add-ext", "umts_rrc_pdu_def.securityModeFailure_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions82,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions82", HFILL }},
    { &hf_umts_rrc_pdu_def_r323,
      { "r3", "umts_rrc_pdu_def.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r323", HFILL }},
    { &hf_umts_rrc_pdu_def_signallingConnectionRelease_r3,
      { "signallingConnectionRelease-r3", "umts_rrc_pdu_def.signallingConnectionRelease_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.SignallingConnectionRelease_r3_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions48,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions48", HFILL }},
    { &hf_umts_rrc_pdu_def_signallingConnectionRelease_r3_add_ext,
      { "signallingConnectionRelease-r3-add-ext", "umts_rrc_pdu_def.signallingConnectionRelease_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions83,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions83", HFILL }},
    { &hf_umts_rrc_pdu_def_later_than_r320,
      { "later-than-r3", "umts_rrc_pdu_def.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_later_than_r320", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions56,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_criticalExtensions56", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions49,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions49", HFILL }},
    { &hf_umts_rrc_pdu_def_signallingConnectionReleaseIndication_r3_add_ext,
      { "signallingConnectionReleaseIndication-r3-add-ext", "umts_rrc_pdu_def.signallingConnectionReleaseIndication_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions84,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions84", HFILL }},
    { &hf_umts_rrc_pdu_def_sfn_Prime,
      { "sfn-Prime", "umts_rrc_pdu_def.sfn_Prime",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.SFN_Prime", HFILL }},
    { &hf_umts_rrc_pdu_def_payload,
      { "payload", "umts_rrc_pdu_def.payload",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_payload_vals), 0,
        "umts_rrc_pdu_def.T_payload", HFILL }},
    { &hf_umts_rrc_pdu_def_noSegment,
      { "noSegment", "umts_rrc_pdu_def.noSegment",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.NULL", HFILL }},
    { &hf_umts_rrc_pdu_def_firstSegment,
      { "firstSegment", "umts_rrc_pdu_def.firstSegment",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.FirstSegment", HFILL }},
    { &hf_umts_rrc_pdu_def_subsequentSegment,
      { "subsequentSegment", "umts_rrc_pdu_def.subsequentSegment",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.SubsequentSegment", HFILL }},
    { &hf_umts_rrc_pdu_def_lastSegmentShort,
      { "lastSegmentShort", "umts_rrc_pdu_def.lastSegmentShort",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.LastSegmentShort", HFILL }},
    { &hf_umts_rrc_pdu_def_lastAndFirst,
      { "lastAndFirst", "umts_rrc_pdu_def.lastAndFirst",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_lastAndFirst", HFILL }},
    { &hf_umts_rrc_pdu_def_firstSegment1,
      { "firstSegment", "umts_rrc_pdu_def.firstSegment",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.FirstSegmentShort", HFILL }},
    { &hf_umts_rrc_pdu_def_lastAndComplete,
      { "lastAndComplete", "umts_rrc_pdu_def.lastAndComplete",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_lastAndComplete", HFILL }},
    { &hf_umts_rrc_pdu_def_completeSIB_List,
      { "completeSIB-List", "umts_rrc_pdu_def.completeSIB_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_pdu_def.CompleteSIB_List", HFILL }},
    { &hf_umts_rrc_pdu_def_lastAndCompleteAndFirst,
      { "lastAndCompleteAndFirst", "umts_rrc_pdu_def.lastAndCompleteAndFirst",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_lastAndCompleteAndFirst", HFILL }},
    { &hf_umts_rrc_pdu_def_completeAndFirst,
      { "completeAndFirst", "umts_rrc_pdu_def.completeAndFirst",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_completeAndFirst", HFILL }},
    { &hf_umts_rrc_pdu_def_completeSIB,
      { "completeSIB", "umts_rrc_pdu_def.completeSIB",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.CompleteSIB", HFILL }},
    { &hf_umts_rrc_pdu_def_lastSegment,
      { "lastSegment", "umts_rrc_pdu_def.lastSegment",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.LastSegment", HFILL }},
    { &hf_umts_rrc_pdu_def_spare5,
      { "spare5", "umts_rrc_pdu_def.spare5",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.NULL", HFILL }},
    { &hf_umts_rrc_pdu_def_spare4,
      { "spare4", "umts_rrc_pdu_def.spare4",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.NULL", HFILL }},
    { &hf_umts_rrc_pdu_def_spare3,
      { "spare3", "umts_rrc_pdu_def.spare3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.NULL", HFILL }},
    { &hf_umts_rrc_pdu_def_spare2,
      { "spare2", "umts_rrc_pdu_def.spare2",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.NULL", HFILL }},
    { &hf_umts_rrc_pdu_def_spare1,
      { "spare1", "umts_rrc_pdu_def.spare1",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.NULL", HFILL }},
    { &hf_umts_rrc_pdu_def_payload1,
      { "payload", "umts_rrc_pdu_def.payload",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_payload1_vals), 0,
        "umts_rrc_pdu_def.T_payload1", HFILL }},
    { &hf_umts_rrc_pdu_def_lastAndFirst1,
      { "lastAndFirst", "umts_rrc_pdu_def.lastAndFirst",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_lastAndFirst1", HFILL }},
    { &hf_umts_rrc_pdu_def_lastAndComplete1,
      { "lastAndComplete", "umts_rrc_pdu_def.lastAndComplete",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_lastAndComplete1", HFILL }},
    { &hf_umts_rrc_pdu_def_lastAndCompleteAndFirst1,
      { "lastAndCompleteAndFirst", "umts_rrc_pdu_def.lastAndCompleteAndFirst",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_lastAndCompleteAndFirst1", HFILL }},
    { &hf_umts_rrc_pdu_def_completeAndFirst1,
      { "completeAndFirst", "umts_rrc_pdu_def.completeAndFirst",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_completeAndFirst1", HFILL }},
    { &hf_umts_rrc_pdu_def_sib_Type,
      { "sib-Type", "umts_rrc_pdu_def.sib_Type",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_SIB_Type_vals), 0,
        "umts_rrc_ies.SIB_Type", HFILL }},
    { &hf_umts_rrc_pdu_def_seg_Count,
      { "seg-Count", "umts_rrc_pdu_def.seg_Count",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.SegCount", HFILL }},
    { &hf_umts_rrc_pdu_def_sib_Data_fixed,
      { "sib-Data-fixed", "umts_rrc_pdu_def.sib_Data_fixed",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_ies.SIB_Data_fixed", HFILL }},
    { &hf_umts_rrc_pdu_def_sib_Data_variable,
      { "sib-Data-variable", "umts_rrc_pdu_def.sib_Data_variable",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_ies.SIB_Data_variable", HFILL }},
    { &hf_umts_rrc_pdu_def_segmentIndex,
      { "segmentIndex", "umts_rrc_pdu_def.segmentIndex",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.SegmentIndex", HFILL }},
    { &hf_umts_rrc_pdu_def_CompleteSIB_List_item,
      { "Item", "umts_rrc_pdu_def.CompleteSIB_List_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.CompleteSIBshort", HFILL }},
    { &hf_umts_rrc_pdu_def_sib_Data_fixed1,
      { "sib-Data-fixed", "umts_rrc_pdu_def.sib_Data_fixed",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING_SIZE_226", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions50,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions50", HFILL }},
    { &hf_umts_rrc_pdu_def_systemInformationChangeIndication_r3_add_ext,
      { "systemInformationChangeIndication-r3-add-ext", "umts_rrc_pdu_def.systemInformationChangeIndication_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions85,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions85", HFILL }},
    { &hf_umts_rrc_pdu_def_r324,
      { "r3", "umts_rrc_pdu_def.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r324", HFILL }},
    { &hf_umts_rrc_pdu_def_transportChannelReconfiguration_r3,
      { "transportChannelReconfiguration-r3", "umts_rrc_pdu_def.transportChannelReconfiguration_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.TransportChannelReconfiguration_r3_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v3a0NonCriticalExtensions9,
      { "v3a0NonCriticalExtensions", "umts_rrc_pdu_def.v3a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v3a0NonCriticalExtensions9", HFILL }},
    { &hf_umts_rrc_pdu_def_transportChannelReconfiguration_v3a0ext,
      { "transportChannelReconfiguration-v3a0ext", "umts_rrc_pdu_def.transportChannelReconfiguration_v3a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.TransportChannelReconfiguration_v3a0ext", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions51,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions51", HFILL }},
    { &hf_umts_rrc_pdu_def_transportChannelReconfiguration_r3_add_ext,
      { "transportChannelReconfiguration-r3-add-ext", "umts_rrc_pdu_def.transportChannelReconfiguration_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions13,
      { "v4b0NonCriticalExtensions", "umts_rrc_pdu_def.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v4b0NonCriticalExtensions13", HFILL }},
    { &hf_umts_rrc_pdu_def_transportChannelReconfiguration_v4b0ext,
      { "transportChannelReconfiguration-v4b0ext", "umts_rrc_pdu_def.transportChannelReconfiguration_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.TransportChannelReconfiguration_v4b0ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions24,
      { "v590NonCriticalExtensions", "umts_rrc_pdu_def.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v590NonCriticalExtensions24", HFILL }},
    { &hf_umts_rrc_pdu_def_transportChannelReconfiguration_v590ext,
      { "transportChannelReconfiguration-v590ext", "umts_rrc_pdu_def.transportChannelReconfiguration_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.TransportChannelReconfiguration_v590ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions33,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions33", HFILL }},
    { &hf_umts_rrc_pdu_def_transportChannelReconfiguration_v6xyext,
      { "transportChannelReconfiguration-v6xyext", "umts_rrc_pdu_def.transportChannelReconfiguration_v6xyext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.TransportChannelReconfiguration_v6xyext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions86,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions86", HFILL }},
    { &hf_umts_rrc_pdu_def_later_than_r321,
      { "later-than-r3", "umts_rrc_pdu_def.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_later_than_r321", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions57,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_criticalExtensions57_vals), 0,
        "umts_rrc_pdu_def.T_criticalExtensions57", HFILL }},
    { &hf_umts_rrc_pdu_def_r412,
      { "r4", "umts_rrc_pdu_def.r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r412", HFILL }},
    { &hf_umts_rrc_pdu_def_transportChannelReconfiguration_r4,
      { "transportChannelReconfiguration-r4", "umts_rrc_pdu_def.transportChannelReconfiguration_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.TransportChannelReconfiguration_r4_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v4d0NonCriticalExtensions12,
      { "v4d0NonCriticalExtensions", "umts_rrc_pdu_def.v4d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v4d0NonCriticalExtensions12", HFILL }},
    { &hf_umts_rrc_pdu_def_transportChannelReconfiguration_r4_add_ext,
      { "transportChannelReconfiguration-r4-add-ext", "umts_rrc_pdu_def.transportChannelReconfiguration_r4_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions25,
      { "v590NonCriticalExtensions", "umts_rrc_pdu_def.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v590NonCriticalExtensions25", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions34,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions34", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions87,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions87", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions58,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_criticalExtensions58_vals), 0,
        "umts_rrc_pdu_def.T_criticalExtensions58", HFILL }},
    { &hf_umts_rrc_pdu_def_r510,
      { "r5", "umts_rrc_pdu_def.r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r510", HFILL }},
    { &hf_umts_rrc_pdu_def_transportChannelReconfiguration_r5,
      { "transportChannelReconfiguration-r5", "umts_rrc_pdu_def.transportChannelReconfiguration_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.TransportChannelReconfiguration_r5_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_transportChannelReconfiguration_r5_add_ext,
      { "transportChannelReconfiguration-r5-add-ext", "umts_rrc_pdu_def.transportChannelReconfiguration_r5_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions35,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions35", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions88,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions88", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions59,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_criticalExtensions59_vals), 0,
        "umts_rrc_pdu_def.T_criticalExtensions59", HFILL }},
    { &hf_umts_rrc_pdu_def_r610,
      { "r6", "umts_rrc_pdu_def.r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r610", HFILL }},
    { &hf_umts_rrc_pdu_def_transportChannelReconfiguration_r6,
      { "transportChannelReconfiguration-r6", "umts_rrc_pdu_def.transportChannelReconfiguration_r6",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.TransportChannelReconfiguration_r6_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_transportChannelReconfiguration_r6_add_ext,
      { "transportChannelReconfiguration-r6-add-ext", "umts_rrc_pdu_def.transportChannelReconfiguration_r6_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions89,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions89", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions60,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_criticalExtensions60", HFILL }},
    { &hf_umts_rrc_pdu_def_transportChannelReconfiguration_r3_IEs_dummy,
      { "transportChannelReconfiguration-r3-IEs-dummy", "umts_rrc_pdu_def.transportChannelReconfiguration_r3_IEs_dummy",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_transportChannelReconfiguration_r3_IEs_dummy_vals), 0,
        "umts_rrc_pdu_def.T_transportChannelReconfiguration_r3_IEs_dummy", HFILL }},
    { &hf_umts_rrc_pdu_def_fdd33,
      { "fdd", "umts_rrc_pdu_def.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_fdd33", HFILL }},
    { &hf_umts_rrc_pdu_def_modeSpecificPhysChInfo12,
      { "modeSpecificPhysChInfo", "umts_rrc_pdu_def.modeSpecificPhysChInfo",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_modeSpecificPhysChInfo12_vals), 0,
        "umts_rrc_pdu_def.T_modeSpecificPhysChInfo12", HFILL }},
    { &hf_umts_rrc_pdu_def_fdd34,
      { "fdd", "umts_rrc_pdu_def.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_fdd34", HFILL }},
    { &hf_umts_rrc_pdu_def_transportChannelReconfiguration_r4_IEs_dummy,
      { "transportChannelReconfiguration-r4-IEs-dummy", "umts_rrc_pdu_def.transportChannelReconfiguration_r4_IEs_dummy",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_transportChannelReconfiguration_r4_IEs_dummy_vals), 0,
        "umts_rrc_pdu_def.T_transportChannelReconfiguration_r4_IEs_dummy", HFILL }},
    { &hf_umts_rrc_pdu_def_fdd35,
      { "fdd", "umts_rrc_pdu_def.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_fdd35", HFILL }},
    { &hf_umts_rrc_pdu_def_modeSpecificPhysChInfo13,
      { "modeSpecificPhysChInfo", "umts_rrc_pdu_def.modeSpecificPhysChInfo",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_modeSpecificPhysChInfo13_vals), 0,
        "umts_rrc_pdu_def.T_modeSpecificPhysChInfo13", HFILL }},
    { &hf_umts_rrc_pdu_def_fdd36,
      { "fdd", "umts_rrc_pdu_def.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_fdd36", HFILL }},
    { &hf_umts_rrc_pdu_def_transportChannelReconfiguration_r5_IEs_dummy,
      { "transportChannelReconfiguration-r5-IEs-dummy", "umts_rrc_pdu_def.transportChannelReconfiguration_r5_IEs_dummy",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_transportChannelReconfiguration_r5_IEs_dummy_vals), 0,
        "umts_rrc_pdu_def.T_transportChannelReconfiguration_r5_IEs_dummy", HFILL }},
    { &hf_umts_rrc_pdu_def_fdd37,
      { "fdd", "umts_rrc_pdu_def.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_fdd37", HFILL }},
    { &hf_umts_rrc_pdu_def_modeSpecificPhysChInfo14,
      { "modeSpecificPhysChInfo", "umts_rrc_pdu_def.modeSpecificPhysChInfo",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_modeSpecificPhysChInfo14_vals), 0,
        "umts_rrc_pdu_def.T_modeSpecificPhysChInfo14", HFILL }},
    { &hf_umts_rrc_pdu_def_fdd38,
      { "fdd", "umts_rrc_pdu_def.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_fdd38", HFILL }},
    { &hf_umts_rrc_pdu_def_postVerificationPeriod6,
      { "postVerificationPeriod", "umts_rrc_pdu_def.postVerificationPeriod",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_postVerificationPeriod6_vals), 0,
        "umts_rrc_pdu_def.T_postVerificationPeriod6", HFILL }},
    { &hf_umts_rrc_pdu_def_timingMaintainedSynchInd4,
      { "timingMaintainedSynchInd", "umts_rrc_pdu_def.timingMaintainedSynchInd",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_timingMaintainedSynchInd4_vals), 0,
        "umts_rrc_pdu_def.T_timingMaintainedSynchInd4", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions52,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions52", HFILL }},
    { &hf_umts_rrc_pdu_def_transportChannelReconfigurationComplete_r3_add_ext,
      { "transportChannelReconfigurationComplete-r3-add-ext", "umts_rrc_pdu_def.transportChannelReconfigurationComplete_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions90,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions90", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions53,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions53", HFILL }},
    { &hf_umts_rrc_pdu_def_transportChannelReconfigurationFailure_r3_add_ext,
      { "transportChannelReconfigurationFailure-r3-add-ext", "umts_rrc_pdu_def.transportChannelReconfigurationFailure_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions91,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions91", HFILL }},
    { &hf_umts_rrc_pdu_def_modeSpecificInfo9,
      { "modeSpecificInfo", "umts_rrc_pdu_def.modeSpecificInfo",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_modeSpecificInfo9_vals), 0,
        "umts_rrc_pdu_def.T_modeSpecificInfo9", HFILL }},
    { &hf_umts_rrc_pdu_def_fdd39,
      { "fdd", "umts_rrc_pdu_def.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.NULL", HFILL }},
    { &hf_umts_rrc_pdu_def_tdd4,
      { "tdd", "umts_rrc_pdu_def.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_tdd3", HFILL }},
    { &hf_umts_rrc_pdu_def_tfcs_ID,
      { "tfcs-ID", "umts_rrc_pdu_def.tfcs_ID",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.TFCS_Identity", HFILL }},
    { &hf_umts_rrc_pdu_def_dpch_TFCS_InUplink,
      { "dpch-TFCS-InUplink", "umts_rrc_pdu_def.dpch_TFCS_InUplink",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_TFC_Subset_vals), 0,
        "umts_rrc_ies.TFC_Subset", HFILL }},
    { &hf_umts_rrc_pdu_def_activationTimeForTFCSubset,
      { "activationTimeForTFCSubset", "umts_rrc_pdu_def.activationTimeForTFCSubset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.ActivationTime", HFILL }},
    { &hf_umts_rrc_pdu_def_tfc_ControlDuration,
      { "tfc-ControlDuration", "umts_rrc_pdu_def.tfc_ControlDuration",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_TFC_ControlDuration_vals), 0,
        "umts_rrc_ies.TFC_ControlDuration", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions54,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions54", HFILL }},
    { &hf_umts_rrc_pdu_def_transportFormatCombinationControl_r3_add_ext,
      { "transportFormatCombinationControl-r3-add-ext", "umts_rrc_pdu_def.transportFormatCombinationControl_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions92,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions92", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions55,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions55", HFILL }},
    { &hf_umts_rrc_pdu_def_transportFormatCombinationControlFailure_r3_add_ext,
      { "transportFormatCombinationControlFailure-r3-add-ext", "umts_rrc_pdu_def.transportFormatCombinationControlFailure_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions93,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions93", HFILL }},
    { &hf_umts_rrc_pdu_def_r325,
      { "r3", "umts_rrc_pdu_def.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r325", HFILL }},
    { &hf_umts_rrc_pdu_def_ueCapabilityEnquiry_r3,
      { "ueCapabilityEnquiry-r3", "umts_rrc_pdu_def.ueCapabilityEnquiry_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.UECapabilityEnquiry_r3_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions56,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions56", HFILL }},
    { &hf_umts_rrc_pdu_def_ueCapabilityEnquiry_r3_add_ext,
      { "ueCapabilityEnquiry-r3-add-ext", "umts_rrc_pdu_def.ueCapabilityEnquiry_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions14,
      { "v4b0NonCriticalExtensions", "umts_rrc_pdu_def.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v4b0NonCriticalExtensions14", HFILL }},
    { &hf_umts_rrc_pdu_def_ueCapabilityEnquiry_v4b0ext,
      { "ueCapabilityEnquiry-v4b0ext", "umts_rrc_pdu_def.ueCapabilityEnquiry_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.UECapabilityEnquiry_v4b0ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions26,
      { "v590NonCriticalExtensions", "umts_rrc_pdu_def.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v590NonCriticalExtensions26", HFILL }},
    { &hf_umts_rrc_pdu_def_ueCapabilityEnquiry_v590ext,
      { "ueCapabilityEnquiry-v590ext", "umts_rrc_pdu_def.ueCapabilityEnquiry_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.UECapabilityEnquiry_v590ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions94,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions94", HFILL }},
    { &hf_umts_rrc_pdu_def_later_than_r322,
      { "later-than-r3", "umts_rrc_pdu_def.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_later_than_r322", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions61,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_criticalExtensions61", HFILL }},
    { &hf_umts_rrc_pdu_def_v370NonCriticalExtensions1,
      { "v370NonCriticalExtensions", "umts_rrc_pdu_def.v370NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v370NonCriticalExtensions1", HFILL }},
    { &hf_umts_rrc_pdu_def_ueCapabilityInformation_v370ext,
      { "ueCapabilityInformation-v370ext", "umts_rrc_pdu_def.ueCapabilityInformation_v370ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.UECapabilityInformation_v370ext", HFILL }},
    { &hf_umts_rrc_pdu_def_v380NonCriticalExtensions1,
      { "v380NonCriticalExtensions", "umts_rrc_pdu_def.v380NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v380NonCriticalExtensions1", HFILL }},
    { &hf_umts_rrc_pdu_def_ueCapabilityInformation_v380ext,
      { "ueCapabilityInformation-v380ext", "umts_rrc_pdu_def.ueCapabilityInformation_v380ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.UECapabilityInformation_v380ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v3a0NonCriticalExtensions10,
      { "v3a0NonCriticalExtensions", "umts_rrc_pdu_def.v3a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v3a0NonCriticalExtensions10", HFILL }},
    { &hf_umts_rrc_pdu_def_ueCapabilityInformation_v3a0ext,
      { "ueCapabilityInformation-v3a0ext", "umts_rrc_pdu_def.ueCapabilityInformation_v3a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.UECapabilityInformation_v3a0ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions57,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions57", HFILL }},
    { &hf_umts_rrc_pdu_def_ueCapabilityInformation_r3_add_ext,
      { "ueCapabilityInformation-r3-add-ext", "umts_rrc_pdu_def.ueCapabilityInformation_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions15,
      { "v4b0NonCriticalExtensions", "umts_rrc_pdu_def.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v4b0NonCriticalExtensions15", HFILL }},
    { &hf_umts_rrc_pdu_def_ueCapabilityInformation_v4b0ext,
      { "ueCapabilityInformation-v4b0ext", "umts_rrc_pdu_def.ueCapabilityInformation_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.UECapabilityInformation_v4b0ext", HFILL }},
    { &hf_umts_rrc_pdu_def_v590NonCriticalExtensions27,
      { "v590NonCriticalExtensions", "umts_rrc_pdu_def.v590NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v590NonCriticalExtensions27", HFILL }},
    { &hf_umts_rrc_pdu_def_ueCapabilityInformation_v590ext,
      { "ueCapabilityInformation-v590ext", "umts_rrc_pdu_def.ueCapabilityInformation_v590ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.UECapabilityInformation_v590ext", HFILL }},
    { &hf_umts_rrc_pdu_def_v5c0NonCriticalExtensions1,
      { "v5c0NonCriticalExtensions", "umts_rrc_pdu_def.v5c0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v5c0NonCriticalExtensions1", HFILL }},
    { &hf_umts_rrc_pdu_def_ueCapabilityInformation_v5c0ext,
      { "ueCapabilityInformation-v5c0ext", "umts_rrc_pdu_def.ueCapabilityInformation_v5c0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.UECapabilityInformation_v5c0ext", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions36,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions36", HFILL }},
    { &hf_umts_rrc_pdu_def_ueCapabilityInformation_v6xyext,
      { "ueCapabilityInformation-v6xyext", "umts_rrc_pdu_def.ueCapabilityInformation_v6xyext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.UECapabilityInformation_v6xyext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions95,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions95", HFILL }},
    { &hf_umts_rrc_pdu_def_ueCapabilityInformation_v650ext,
      { "ueCapabilityInformation-v650ext", "umts_rrc_pdu_def.ueCapabilityInformation_v650ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.UECapabilityInformation_v650ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions96,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions96", HFILL }},
    { &hf_umts_rrc_pdu_def_r326,
      { "r3", "umts_rrc_pdu_def.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r326", HFILL }},
    { &hf_umts_rrc_pdu_def_ueCapabilityInformationConfirm_r3,
      { "ueCapabilityInformationConfirm-r3", "umts_rrc_pdu_def.ueCapabilityInformationConfirm_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.UECapabilityInformationConfirm_r3_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions58,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions58", HFILL }},
    { &hf_umts_rrc_pdu_def_ueCapabilityInformationConfirm_r3_add_ext,
      { "ueCapabilityInformationConfirm-r3-add-ext", "umts_rrc_pdu_def.ueCapabilityInformationConfirm_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions97,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions97", HFILL }},
    { &hf_umts_rrc_pdu_def_later_than_r323,
      { "later-than-r3", "umts_rrc_pdu_def.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_later_than_r323", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions62,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_criticalExtensions62", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions59,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions59", HFILL }},
    { &hf_umts_rrc_pdu_def_uplinkDirectTransfer_r3_add_ext,
      { "uplinkDirectTransfer-r3-add-ext", "umts_rrc_pdu_def.uplinkDirectTransfer_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions37,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions37", HFILL }},
    { &hf_umts_rrc_pdu_def_uplinkDirectTransfer_v6xyext,
      { "uplinkDirectTransfer-v6xyext", "umts_rrc_pdu_def.uplinkDirectTransfer_v6xyext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.UplinkDirectTransfer_v6xyext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions98,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions98", HFILL }},
    { &hf_umts_rrc_pdu_def_r327,
      { "r3", "umts_rrc_pdu_def.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r327", HFILL }},
    { &hf_umts_rrc_pdu_def_uplinkPhysicalChannelControl_r3,
      { "uplinkPhysicalChannelControl-r3", "umts_rrc_pdu_def.uplinkPhysicalChannelControl_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.UplinkPhysicalChannelControl_r3_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions60,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions60", HFILL }},
    { &hf_umts_rrc_pdu_def_uplinkPhysicalChannelControl_r3_add_ext,
      { "uplinkPhysicalChannelControl-r3-add-ext", "umts_rrc_pdu_def.uplinkPhysicalChannelControl_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v4b0NonCriticalExtensions16,
      { "v4b0NonCriticalExtensions", "umts_rrc_pdu_def.v4b0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v4b0NonCriticalExtensions16", HFILL }},
    { &hf_umts_rrc_pdu_def_uplinkPysicalChannelControl_v4b0ext,
      { "uplinkPysicalChannelControl-v4b0ext", "umts_rrc_pdu_def.uplinkPysicalChannelControl_v4b0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.UplinkPhysicalChannelControl_v4b0ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_noncriticalExtensions,
      { "noncriticalExtensions", "umts_rrc_pdu_def.noncriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_noncriticalExtensions", HFILL }},
    { &hf_umts_rrc_pdu_def_later_than_r324,
      { "later-than-r3", "umts_rrc_pdu_def.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_later_than_r324", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions63,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_criticalExtensions63_vals), 0,
        "umts_rrc_pdu_def.T_criticalExtensions63", HFILL }},
    { &hf_umts_rrc_pdu_def_r413,
      { "r4", "umts_rrc_pdu_def.r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r413", HFILL }},
    { &hf_umts_rrc_pdu_def_uplinkPhysicalChannelControl_r4,
      { "uplinkPhysicalChannelControl-r4", "umts_rrc_pdu_def.uplinkPhysicalChannelControl_r4",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.UplinkPhysicalChannelControl_r4_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v4d0NonCriticalExtensions13,
      { "v4d0NonCriticalExtensions", "umts_rrc_pdu_def.v4d0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v4d0NonCriticalExtensions13", HFILL }},
    { &hf_umts_rrc_pdu_def_uplinkPhysicalChannelControl_r4_add_ext,
      { "uplinkPhysicalChannelControl-r4-add-ext", "umts_rrc_pdu_def.uplinkPhysicalChannelControl_r4_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions38,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions38", HFILL }},
    { &hf_umts_rrc_pdu_def_uplinkPhysicalChannelControl_v6xyext,
      { "uplinkPhysicalChannelControl-v6xyext", "umts_rrc_pdu_def.uplinkPhysicalChannelControl_v6xyext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.UplinkPhysicalChannelControl_v6xyext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions99,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions99", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions64,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_criticalExtensions64_vals), 0,
        "umts_rrc_pdu_def.T_criticalExtensions64", HFILL }},
    { &hf_umts_rrc_pdu_def_r511,
      { "r5", "umts_rrc_pdu_def.r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r511", HFILL }},
    { &hf_umts_rrc_pdu_def_uplinkPhysicalChannelControl_r5,
      { "uplinkPhysicalChannelControl-r5", "umts_rrc_pdu_def.uplinkPhysicalChannelControl_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.UplinkPhysicalChannelControl_r5_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_uplinkPhysicalChannelControl_r5_add_ext,
      { "uplinkPhysicalChannelControl-r5-add-ext", "umts_rrc_pdu_def.uplinkPhysicalChannelControl_r5_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions39,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions39", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions100,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions100", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions65,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_criticalExtensions65", HFILL }},
    { &hf_umts_rrc_pdu_def_ccTrCH_PowerControlInfo,
      { "ccTrCH-PowerControlInfo", "umts_rrc_pdu_def.ccTrCH_PowerControlInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.CCTrCH_PowerControlInfo", HFILL }},
    { &hf_umts_rrc_pdu_def_timingAdvance,
      { "timingAdvance", "umts_rrc_pdu_def.timingAdvance",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_UL_TimingAdvanceControl_vals), 0,
        "umts_rrc_ies.UL_TimingAdvanceControl", HFILL }},
    { &hf_umts_rrc_pdu_def_alpha,
      { "alpha", "umts_rrc_pdu_def.alpha",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.Alpha", HFILL }},
    { &hf_umts_rrc_pdu_def_specialBurstScheduling,
      { "specialBurstScheduling", "umts_rrc_pdu_def.specialBurstScheduling",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.SpecialBurstScheduling", HFILL }},
    { &hf_umts_rrc_pdu_def_prach_ConstantValue,
      { "prach-ConstantValue", "umts_rrc_pdu_def.prach_ConstantValue",
        FT_INT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.ConstantValueTdd", HFILL }},
    { &hf_umts_rrc_pdu_def_pusch_ConstantValue,
      { "pusch-ConstantValue", "umts_rrc_pdu_def.pusch_ConstantValue",
        FT_INT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.ConstantValueTdd", HFILL }},
    { &hf_umts_rrc_pdu_def_openLoopPowerControl_IPDL_TDD,
      { "openLoopPowerControl-IPDL-TDD", "umts_rrc_pdu_def.openLoopPowerControl_IPDL_TDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.OpenLoopPowerControl_IPDL_TDD_r4", HFILL }},
    { &hf_umts_rrc_pdu_def_ccTrCH_PowerControlInfo1,
      { "ccTrCH-PowerControlInfo", "umts_rrc_pdu_def.ccTrCH_PowerControlInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.CCTrCH_PowerControlInfo_r4", HFILL }},
    { &hf_umts_rrc_pdu_def_tddOption,
      { "tddOption", "umts_rrc_pdu_def.tddOption",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_tddOption_vals), 0,
        "umts_rrc_pdu_def.T_tddOption", HFILL }},
    { &hf_umts_rrc_pdu_def_tdd3842,
      { "tdd384", "umts_rrc_pdu_def.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_tdd3842", HFILL }},
    { &hf_umts_rrc_pdu_def_timingAdvance1,
      { "timingAdvance", "umts_rrc_pdu_def.timingAdvance",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_UL_TimingAdvanceControl_r4_vals), 0,
        "umts_rrc_ies.UL_TimingAdvanceControl_r4", HFILL }},
    { &hf_umts_rrc_pdu_def_tdd1282,
      { "tdd128", "umts_rrc_pdu_def.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_tdd1282", HFILL }},
    { &hf_umts_rrc_pdu_def_ul_SynchronisationParameters,
      { "ul-SynchronisationParameters", "umts_rrc_pdu_def.ul_SynchronisationParameters",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.UL_SynchronisationParameters_r4", HFILL }},
    { &hf_umts_rrc_pdu_def_ccTrCH_PowerControlInfo2,
      { "ccTrCH-PowerControlInfo", "umts_rrc_pdu_def.ccTrCH_PowerControlInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.CCTrCH_PowerControlInfo_r5", HFILL }},
    { &hf_umts_rrc_pdu_def_tddOption1,
      { "tddOption", "umts_rrc_pdu_def.tddOption",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_tddOption1_vals), 0,
        "umts_rrc_pdu_def.T_tddOption1", HFILL }},
    { &hf_umts_rrc_pdu_def_tdd3843,
      { "tdd384", "umts_rrc_pdu_def.tdd384",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_tdd3843", HFILL }},
    { &hf_umts_rrc_pdu_def_hs_SICH_PowerControl,
      { "hs-SICH-PowerControl", "umts_rrc_pdu_def.hs_SICH_PowerControl",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.HS_SICH_Power_Control_Info_TDD384", HFILL }},
    { &hf_umts_rrc_pdu_def_tdd1283,
      { "tdd128", "umts_rrc_pdu_def.tdd128",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_tdd1283", HFILL }},
    { &hf_umts_rrc_pdu_def_ura_UpdateCause,
      { "ura-UpdateCause", "umts_rrc_pdu_def.ura_UpdateCause",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_ies_URA_UpdateCause_vals), 0,
        "umts_rrc_ies.URA_UpdateCause", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions61,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions61", HFILL }},
    { &hf_umts_rrc_pdu_def_uraUpdate_r3_add_ext,
      { "uraUpdate-r3-add-ext", "umts_rrc_pdu_def.uraUpdate_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions101,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions101", HFILL }},
    { &hf_umts_rrc_pdu_def_r328,
      { "r3", "umts_rrc_pdu_def.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r328", HFILL }},
    { &hf_umts_rrc_pdu_def_uraUpdateConfirm_r3,
      { "uraUpdateConfirm-r3", "umts_rrc_pdu_def.uraUpdateConfirm_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.URAUpdateConfirm_r3_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions62,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions62", HFILL }},
    { &hf_umts_rrc_pdu_def_uraUpdateConfirm_r3_add_ext,
      { "uraUpdateConfirm-r3-add-ext", "umts_rrc_pdu_def.uraUpdateConfirm_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions40,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions40", HFILL }},
    { &hf_umts_rrc_pdu_def_uraUpdateConfirm_v6xyext,
      { "uraUpdateConfirm-v6xyext", "umts_rrc_pdu_def.uraUpdateConfirm_v6xyext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.URAUpdateConfirm_v6xyext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions102,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions102", HFILL }},
    { &hf_umts_rrc_pdu_def_later_than_r325,
      { "later-than-r3", "umts_rrc_pdu_def.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_later_than_r325", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions66,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_criticalExtensions66_vals), 0,
        "umts_rrc_pdu_def.T_criticalExtensions66", HFILL }},
    { &hf_umts_rrc_pdu_def_r512,
      { "r5", "umts_rrc_pdu_def.r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r512", HFILL }},
    { &hf_umts_rrc_pdu_def_uraUpdateConfirm_r5,
      { "uraUpdateConfirm-r5", "umts_rrc_pdu_def.uraUpdateConfirm_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.URAUpdateConfirm_r5_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions41,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions41", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions103,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions103", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions67,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_criticalExtensions67", HFILL }},
    { &hf_umts_rrc_pdu_def_r329,
      { "r3", "umts_rrc_pdu_def.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r329", HFILL }},
    { &hf_umts_rrc_pdu_def_uraUpdateConfirm_CCCH_r3,
      { "uraUpdateConfirm-CCCH-r3", "umts_rrc_pdu_def.uraUpdateConfirm_CCCH_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.URAUpdateConfirm_CCCH_r3_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions63,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions63", HFILL }},
    { &hf_umts_rrc_pdu_def_uraUpdateConfirm_CCCH_r3_add_ext,
      { "uraUpdateConfirm-CCCH-r3-add-ext", "umts_rrc_pdu_def.uraUpdateConfirm_CCCH_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions42,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions42", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions104,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions104", HFILL }},
    { &hf_umts_rrc_pdu_def_later_than_r326,
      { "later-than-r3", "umts_rrc_pdu_def.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_later_than_r326", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions68,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_criticalExtensions68", HFILL }},
    { &hf_umts_rrc_pdu_def_uraUpdateConfirm,
      { "uraUpdateConfirm", "umts_rrc_pdu_def.uraUpdateConfirm",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.URAUpdateConfirm_r3_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_r330,
      { "r3", "umts_rrc_pdu_def.r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r330", HFILL }},
    { &hf_umts_rrc_pdu_def_utranMobilityInformation_r3,
      { "utranMobilityInformation-r3", "umts_rrc_pdu_def.utranMobilityInformation_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.UTRANMobilityInformation_r3_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v3a0NonCriticalExtensions11,
      { "v3a0NonCriticalExtensions", "umts_rrc_pdu_def.v3a0NonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v3a0NonCriticalExtensions11", HFILL }},
    { &hf_umts_rrc_pdu_def_utranMobilityInformation_v3a0ext,
      { "utranMobilityInformation-v3a0ext", "umts_rrc_pdu_def.utranMobilityInformation_v3a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.UTRANMobilityInformation_v3a0ext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions64,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions64", HFILL }},
    { &hf_umts_rrc_pdu_def_utranMobilityInformation_r3_add_ext,
      { "utranMobilityInformation-r3-add-ext", "umts_rrc_pdu_def.utranMobilityInformation_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions43,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions43", HFILL }},
    { &hf_umts_rrc_pdu_def_utranMobilityInformation_v6xyext,
      { "utranMobilityInformation-v6xyext", "umts_rrc_pdu_def.utranMobilityInformation_v6xyext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.UtranMobilityInformation_v6xyext_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions105,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions105", HFILL }},
    { &hf_umts_rrc_pdu_def_later_than_r327,
      { "later-than-r3", "umts_rrc_pdu_def.later_than_r3",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_later_than_r327", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions69,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_criticalExtensions69_vals), 0,
        "umts_rrc_pdu_def.T_criticalExtensions69", HFILL }},
    { &hf_umts_rrc_pdu_def_r513,
      { "r5", "umts_rrc_pdu_def.r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_r513", HFILL }},
    { &hf_umts_rrc_pdu_def_utranMobilityInformation_r5,
      { "utranMobilityInformation-r5", "umts_rrc_pdu_def.utranMobilityInformation_r5",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.UTRANMobilityInformation_r5_IEs", HFILL }},
    { &hf_umts_rrc_pdu_def_v6xyNonCriticalExtensions44,
      { "v6xyNonCriticalExtensions", "umts_rrc_pdu_def.v6xyNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_v6xyNonCriticalExtensions44", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions106,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions106", HFILL }},
    { &hf_umts_rrc_pdu_def_criticalExtensions70,
      { "criticalExtensions", "umts_rrc_pdu_def.criticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_criticalExtensions70", HFILL }},
    { &hf_umts_rrc_pdu_def_ue_ConnTimersAndConstants,
      { "ue-ConnTimersAndConstants", "umts_rrc_pdu_def.ue_ConnTimersAndConstants",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.UE_ConnTimersAndConstants", HFILL }},
    { &hf_umts_rrc_pdu_def_cn_InformationInfo2,
      { "cn-InformationInfo", "umts_rrc_pdu_def.cn_InformationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.CN_InformationInfoFull", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions107,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions107", HFILL }},
    { &hf_umts_rrc_pdu_def_ue_ConnTimersAndConstants_v3a0ext,
      { "ue-ConnTimersAndConstants-v3a0ext", "umts_rrc_pdu_def.ue_ConnTimersAndConstants_v3a0ext",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.UE_ConnTimersAndConstants_v3a0ext", HFILL }},
    { &hf_umts_rrc_pdu_def_ue_ConnTimersAndConstants1,
      { "ue-ConnTimersAndConstants", "umts_rrc_pdu_def.ue_ConnTimersAndConstants",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.UE_ConnTimersAndConstants_r5", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions65,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions65", HFILL }},
    { &hf_umts_rrc_pdu_def_utranMobilityInformationConfirm_r3_add_ext,
      { "utranMobilityInformationConfirm-r3-add-ext", "umts_rrc_pdu_def.utranMobilityInformationConfirm_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions108,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions108", HFILL }},
    { &hf_umts_rrc_pdu_def_laterNonCriticalExtensions66,
      { "laterNonCriticalExtensions", "umts_rrc_pdu_def.laterNonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_laterNonCriticalExtensions66", HFILL }},
    { &hf_umts_rrc_pdu_def_utranMobilityInformationFailure_r3_add_ext,
      { "utranMobilityInformationFailure-r3-add-ext", "umts_rrc_pdu_def.utranMobilityInformationFailure_r3_add_ext",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_pdu_def.BIT_STRING", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions109,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions109", HFILL }},
    { &hf_umts_rrc_pdu_def_mbms_ServiceAccessInfoList,
      { "mbms-ServiceAccessInfoList", "umts_rrc_pdu_def.mbms_ServiceAccessInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.MBMS_ServiceAccessInfoList_r6", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions110,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions110", HFILL }},
    { &hf_umts_rrc_pdu_def_mbms_CommonRBInformationList,
      { "mbms-CommonRBInformationList", "umts_rrc_pdu_def.mbms_CommonRBInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.MBMS_CommonRBInformationList_r6", HFILL }},
    { &hf_umts_rrc_pdu_def_mbms_TranspChInfoForEachTrCh,
      { "mbms-TranspChInfoForEachTrCh", "umts_rrc_pdu_def.mbms_TranspChInfoForEachTrCh",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.MBMS_TranspChInfoForEachTrCh_r6", HFILL }},
    { &hf_umts_rrc_pdu_def_mbms_TranspChInfoForEachCCTrCh,
      { "mbms-TranspChInfoForEachCCTrCh", "umts_rrc_pdu_def.mbms_TranspChInfoForEachCCTrCh",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.MBMS_TranspChInfoForEachCCTrCh_r6", HFILL }},
    { &hf_umts_rrc_pdu_def_mbms_PhyChInformationList,
      { "mbms-PhyChInformationList", "umts_rrc_pdu_def.mbms_PhyChInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.MBMS_PhyChInformationList_r6", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions111,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions111", HFILL }},
    { &hf_umts_rrc_pdu_def_mbms_CurrentCell_SCCPCHList,
      { "mbms-CurrentCell-SCCPCHList", "umts_rrc_pdu_def.mbms_CurrentCell_SCCPCHList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.MBMS_CurrentCell_SCCPCHList_r6", HFILL }},
    { &hf_umts_rrc_pdu_def_mbms_SIBType5_SCCPCHList,
      { "mbms-SIBType5-SCCPCHList", "umts_rrc_pdu_def.mbms_SIBType5_SCCPCHList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.MBMS_SIBType5_SCCPCHList_r6", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions112,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions112", HFILL }},
    { &hf_umts_rrc_pdu_def_mbms_PreferredFrequencyInfo,
      { "mbms-PreferredFrequencyInfo", "umts_rrc_pdu_def.mbms_PreferredFrequencyInfo",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.MBMS_PreferredFrequencyList_r6", HFILL }},
    { &hf_umts_rrc_pdu_def_mbms_TimersAndCouneters,
      { "mbms-TimersAndCouneters", "umts_rrc_pdu_def.mbms_TimersAndCouneters",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.MBMS_TimersAndCouneters_r6", HFILL }},
    { &hf_umts_rrc_pdu_def_michConfigurationInfo,
      { "michConfigurationInfo", "umts_rrc_pdu_def.michConfigurationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.MBMS_MICHConfigurationInfo_r6", HFILL }},
    { &hf_umts_rrc_pdu_def_cellGroupIdentity,
      { "cellGroupIdentity", "umts_rrc_pdu_def.cellGroupIdentity",
        FT_BYTES, BASE_HEX, NULL, 0,
        "umts_rrc_ies.MBMS_CellGroupIdentity_r6", HFILL }},
    { &hf_umts_rrc_pdu_def_mschDefaultConfigurationInfo,
      { "mschDefaultConfigurationInfo", "umts_rrc_pdu_def.mschDefaultConfigurationInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.MBMS_MSCHConfigurationInfo_r6", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions113,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions113", HFILL }},
    { &hf_umts_rrc_pdu_def_mbms_PreferredFreqRequest,
      { "mbms-PreferredFreqRequest", "umts_rrc_pdu_def.mbms_PreferredFreqRequest",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_ies.MBMS_ServiceIdentity", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions114,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions114", HFILL }},
    { &hf_umts_rrc_pdu_def_modifedServiceList,
      { "modifedServiceList", "umts_rrc_pdu_def.modifedServiceList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.MBMS_ModifedServiceList_r6", HFILL }},
    { &hf_umts_rrc_pdu_def_mbms_ReacquireMCCH,
      { "mbms-ReacquireMCCH", "umts_rrc_pdu_def.mbms_ReacquireMCCH",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_mbms_ReacquireMCCH_vals), 0,
        "umts_rrc_pdu_def.T_mbms_ReacquireMCCH", HFILL }},
    { &hf_umts_rrc_pdu_def_mbms_DynamicPersistenceLevel,
      { "mbms-DynamicPersistenceLevel", "umts_rrc_pdu_def.mbms_DynamicPersistenceLevel",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.DynamicPersistenceLevel", HFILL }},
    { &hf_umts_rrc_pdu_def_endOfModifiedMCCHInformation,
      { "endOfModifiedMCCHInformation", "umts_rrc_pdu_def.endOfModifiedMCCHInformation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_pdu_def.INTEGER_1_16", HFILL }},
    { &hf_umts_rrc_pdu_def_mbms_AllUnmodifiedPTMServices,
      { "mbms-AllUnmodifiedPTMServices", "umts_rrc_pdu_def.mbms_AllUnmodifiedPTMServices",
        FT_UINT32, BASE_DEC, VALS(umts_rrc_pdu_def_T_mbms_AllUnmodifiedPTMServices_vals), 0,
        "umts_rrc_pdu_def.T_mbms_AllUnmodifiedPTMServices", HFILL }},
    { &hf_umts_rrc_pdu_def_mbms_PtMActivationTime,
      { "mbms-PtMActivationTime", "umts_rrc_pdu_def.mbms_PtMActivationTime",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.MBMS_PtMActivationTime", HFILL }},
    { &hf_umts_rrc_pdu_def_mbmsNumberOfNeighbourCells,
      { "mbmsNumberOfNeighbourCells", "umts_rrc_pdu_def.mbmsNumberOfNeighbourCells",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.MBMS_NumberOfNeighbourCells_r6", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions115,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions115", HFILL }},
    { &hf_umts_rrc_pdu_def_neighbouringCellIdentity,
      { "neighbouringCellIdentity", "umts_rrc_pdu_def.neighbouringCellIdentity",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.IntraFreqCellID", HFILL }},
    { &hf_umts_rrc_pdu_def_neighbouringCellSCCPCHList,
      { "neighbouringCellSCCPCHList", "umts_rrc_pdu_def.neighbouringCellSCCPCHList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.MBMS_NeighbouringCellSCCPCHList_r6", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions116,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions116", HFILL }},
    { &hf_umts_rrc_pdu_def_serviceSchedulingInfoList,
      { "serviceSchedulingInfoList", "umts_rrc_pdu_def.serviceSchedulingInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.MBMS_ServiceSchedulingInfoList_r6", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions117,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions117", HFILL }},
    { &hf_umts_rrc_pdu_def_unmodifiedServiceList,
      { "unmodifiedServiceList", "umts_rrc_pdu_def.unmodifiedServiceList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "umts_rrc_ies.MBMS_UnmodifiedServiceList_r6", HFILL }},
    { &hf_umts_rrc_pdu_def_nonCriticalExtensions118,
      { "nonCriticalExtensions", "umts_rrc_pdu_def.nonCriticalExtensions",
        FT_NONE, BASE_NONE, NULL, 0,
        "umts_rrc_pdu_def.T_nonCriticalExtensions118", HFILL }},

/*--- End of included file: packet-umts_rrc_pdu_def-hfarr.c ---*/
#line 84 "packet-umts_rrc_pdu_def-template.c"
  };

  /* List of subtrees */
  static gint *ett[] = {
		  &ett_umts_rrc_pdu_def,

/*--- Included file: packet-umts_rrc_pdu_def-ettarr.c ---*/
#line 1 "packet-umts_rrc_pdu_def-ettarr.c"
    &ett_umts_rrc_pdu_def_ActiveSetUpdate,
    &ett_umts_rrc_pdu_def_T_r3,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions,
    &ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions,
    &ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions,
    &ett_umts_rrc_pdu_def_T_later_than_r3,
    &ett_umts_rrc_pdu_def_T_criticalExtensions,
    &ett_umts_rrc_pdu_def_T_r6,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions1,
    &ett_umts_rrc_pdu_def_T_criticalExtensions1,
    &ett_umts_rrc_pdu_def_ActiveSetUpdate_r3_IEs,
    &ett_umts_rrc_pdu_def_ActiveSetUpdate_v4b0ext_IEs,
    &ett_umts_rrc_pdu_def_ActiveSetUpdate_v590ext_IEs,
    &ett_umts_rrc_pdu_def_ActiveSetUpdate_v6xyext_IEs,
    &ett_umts_rrc_pdu_def_ActiveSetUpdate_r6_IEs,
    &ett_umts_rrc_pdu_def_ActiveSetUpdateComplete,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions1,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions2,
    &ett_umts_rrc_pdu_def_ActiveSetUpdateFailure,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions2,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions3,
    &ett_umts_rrc_pdu_def_AssistanceDataDelivery,
    &ett_umts_rrc_pdu_def_T_r31,
    &ett_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions3,
    &ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions1,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions4,
    &ett_umts_rrc_pdu_def_T_later_than_r31,
    &ett_umts_rrc_pdu_def_T_criticalExtensions2,
    &ett_umts_rrc_pdu_def_AssistanceDataDelivery_r3_IEs,
    &ett_umts_rrc_pdu_def_AssistanceDataDelivery_v3a0ext,
    &ett_umts_rrc_pdu_def_AssistanceDataDelivery_v4b0ext_IEs,
    &ett_umts_rrc_pdu_def_CellChangeOrderFromUTRAN,
    &ett_umts_rrc_pdu_def_T_r32,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions4,
    &ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions1,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions5,
    &ett_umts_rrc_pdu_def_T_later_than_r32,
    &ett_umts_rrc_pdu_def_T_criticalExtensions3,
    &ett_umts_rrc_pdu_def_CellChangeOrderFromUTRAN_r3_IEs,
    &ett_umts_rrc_pdu_def_CellChangeOrderFromUTRAN_v590ext_IEs,
    &ett_umts_rrc_pdu_def_T_geran_SystemInfoType,
    &ett_umts_rrc_pdu_def_CellChangeOrderFromUTRANFailure,
    &ett_umts_rrc_pdu_def_T_r33,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions5,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions6,
    &ett_umts_rrc_pdu_def_T_dummy,
    &ett_umts_rrc_pdu_def_T_criticalExtensions4,
    &ett_umts_rrc_pdu_def_CellChangeOrderFromUTRANFailure_r3_IEs,
    &ett_umts_rrc_pdu_def_CellUpdate,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions6,
    &ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions2,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions1,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions7,
    &ett_umts_rrc_pdu_def_CellUpdate_v590ext,
    &ett_umts_rrc_pdu_def_CellUpdate_v6xyext_IEs,
    &ett_umts_rrc_pdu_def_CellUpdateConfirm,
    &ett_umts_rrc_pdu_def_T_r34,
    &ett_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions1,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions7,
    &ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions2,
    &ett_umts_rrc_pdu_def_T_v590NonCriticalExtenstions,
    &ett_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions2,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions8,
    &ett_umts_rrc_pdu_def_T_later_than_r33,
    &ett_umts_rrc_pdu_def_T_criticalExtensions5,
    &ett_umts_rrc_pdu_def_T_r4,
    &ett_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions,
    &ett_umts_rrc_pdu_def_T_v590NonCriticalExtenstions1,
    &ett_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions1,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions3,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions9,
    &ett_umts_rrc_pdu_def_T_criticalExtensions6,
    &ett_umts_rrc_pdu_def_T_r5,
    &ett_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions2,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions4,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions10,
    &ett_umts_rrc_pdu_def_T_criticalExtensions7,
    &ett_umts_rrc_pdu_def_T_r61,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions11,
    &ett_umts_rrc_pdu_def_T_criticalExtensions8,
    &ett_umts_rrc_pdu_def_CellUpdateConfirm_r3_IEs,
    &ett_umts_rrc_pdu_def_T_modeSpecificTransChInfo,
    &ett_umts_rrc_pdu_def_T_fdd,
    &ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo,
    &ett_umts_rrc_pdu_def_T_fdd1,
    &ett_umts_rrc_pdu_def_CellUpdateConfirm_v3a0ext,
    &ett_umts_rrc_pdu_def_CellUpdateConfirm_v4b0ext_IEs,
    &ett_umts_rrc_pdu_def_CellUpdateConfirm_v590ext_IEs,
    &ett_umts_rrc_pdu_def_CellUpdateConfirm_v5d0ext_IEs,
    &ett_umts_rrc_pdu_def_CellUpdateConfirm_r4_IEs,
    &ett_umts_rrc_pdu_def_T_modeSpecificTransChInfo1,
    &ett_umts_rrc_pdu_def_T_fdd2,
    &ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo1,
    &ett_umts_rrc_pdu_def_T_fdd3,
    &ett_umts_rrc_pdu_def_CellUpdateConfirm_r5_IEs,
    &ett_umts_rrc_pdu_def_T_modeSpecificTransChInfo2,
    &ett_umts_rrc_pdu_def_T_fdd4,
    &ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo2,
    &ett_umts_rrc_pdu_def_T_fdd5,
    &ett_umts_rrc_pdu_def_CellUpdateConfirm_r6_IEs,
    &ett_umts_rrc_pdu_def_CellUpdateConfirm_v6xyext_IEs,
    &ett_umts_rrc_pdu_def_CellUpdateConfirm_CCCH,
    &ett_umts_rrc_pdu_def_T_r35,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions8,
    &ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions3,
    &ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions3,
    &ett_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions3,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions5,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions12,
    &ett_umts_rrc_pdu_def_T_later_than_r34,
    &ett_umts_rrc_pdu_def_T_criticalExtensions9,
    &ett_umts_rrc_pdu_def_T_r41,
    &ett_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions1,
    &ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions4,
    &ett_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions4,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions6,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions13,
    &ett_umts_rrc_pdu_def_T_criticalExtensions10,
    &ett_umts_rrc_pdu_def_T_r51,
    &ett_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions5,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions7,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions14,
    &ett_umts_rrc_pdu_def_T_criticalExtensions11,
    &ett_umts_rrc_pdu_def_T_r62,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions15,
    &ett_umts_rrc_pdu_def_T_criticalExtensions12,
    &ett_umts_rrc_pdu_def_CounterCheck,
    &ett_umts_rrc_pdu_def_T_r36,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions9,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions16,
    &ett_umts_rrc_pdu_def_T_later_than_r35,
    &ett_umts_rrc_pdu_def_T_criticalExtensions13,
    &ett_umts_rrc_pdu_def_CounterCheck_r3_IEs,
    &ett_umts_rrc_pdu_def_CounterCheckResponse,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions10,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions17,
    &ett_umts_rrc_pdu_def_DownlinkDirectTransfer,
    &ett_umts_rrc_pdu_def_T_r37,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions11,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions18,
    &ett_umts_rrc_pdu_def_T_later_than_r36,
    &ett_umts_rrc_pdu_def_T_criticalExtensions14,
    &ett_umts_rrc_pdu_def_DownlinkDirectTransfer_r3_IEs,
    &ett_umts_rrc_pdu_def_HandoverToUTRANCommand,
    &ett_umts_rrc_pdu_def_T_r38,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions19,
    &ett_umts_rrc_pdu_def_T_criticalExtensions15,
    &ett_umts_rrc_pdu_def_T_r42,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions20,
    &ett_umts_rrc_pdu_def_T_criticalExtensions16,
    &ett_umts_rrc_pdu_def_T_r52,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions21,
    &ett_umts_rrc_pdu_def_T_criticalExtensions17,
    &ett_umts_rrc_pdu_def_T_r63,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions22,
    &ett_umts_rrc_pdu_def_T_criticalExtensions18,
    &ett_umts_rrc_pdu_def_HandoverToUTRANCommand_r3_IEs,
    &ett_umts_rrc_pdu_def_T_specificationMode,
    &ett_umts_rrc_pdu_def_T_complete,
    &ett_umts_rrc_pdu_def_T_modeSpecificInfo,
    &ett_umts_rrc_pdu_def_T_fdd6,
    &ett_umts_rrc_pdu_def_T_preconfiguration,
    &ett_umts_rrc_pdu_def_T_preConfigMode,
    &ett_umts_rrc_pdu_def_T_defaultConfig,
    &ett_umts_rrc_pdu_def_T_modeSpecificInfo1,
    &ett_umts_rrc_pdu_def_T_fdd7,
    &ett_umts_rrc_pdu_def_T_tdd,
    &ett_umts_rrc_pdu_def_HandoverToUTRANCommand_r4_IEs,
    &ett_umts_rrc_pdu_def_T_specificationMode1,
    &ett_umts_rrc_pdu_def_T_complete1,
    &ett_umts_rrc_pdu_def_T_modeSpecificInfo2,
    &ett_umts_rrc_pdu_def_T_fdd8,
    &ett_umts_rrc_pdu_def_T_preconfiguration1,
    &ett_umts_rrc_pdu_def_T_preConfigMode1,
    &ett_umts_rrc_pdu_def_T_defaultConfig1,
    &ett_umts_rrc_pdu_def_T_modeSpecificInfo3,
    &ett_umts_rrc_pdu_def_T_fdd9,
    &ett_umts_rrc_pdu_def_T_tdd1,
    &ett_umts_rrc_pdu_def_T_tdd384,
    &ett_umts_rrc_pdu_def_T_tdd128,
    &ett_umts_rrc_pdu_def_HandoverToUTRANCommand_r5_IEs,
    &ett_umts_rrc_pdu_def_T_specificationMode2,
    &ett_umts_rrc_pdu_def_T_complete2,
    &ett_umts_rrc_pdu_def_T_modeSpecificInfo4,
    &ett_umts_rrc_pdu_def_T_fdd10,
    &ett_umts_rrc_pdu_def_T_preconfiguration2,
    &ett_umts_rrc_pdu_def_T_preConfigMode2,
    &ett_umts_rrc_pdu_def_T_defaultConfig2,
    &ett_umts_rrc_pdu_def_T_modeSpecificInfo5,
    &ett_umts_rrc_pdu_def_T_fdd11,
    &ett_umts_rrc_pdu_def_T_tdd2,
    &ett_umts_rrc_pdu_def_T_tdd3841,
    &ett_umts_rrc_pdu_def_T_tdd1281,
    &ett_umts_rrc_pdu_def_HandoverToUTRANCommand_r6_IEs,
    &ett_umts_rrc_pdu_def_T_specificationMode3,
    &ett_umts_rrc_pdu_def_T_complete3,
    &ett_umts_rrc_pdu_def_HandoverToUTRANComplete,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions12,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions23,
    &ett_umts_rrc_pdu_def_InitialDirectTransfer,
    &ett_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions2,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions13,
    &ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions5,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions8,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions24,
    &ett_umts_rrc_pdu_def_InitialDirectTransfer_v3a0ext,
    &ett_umts_rrc_pdu_def_InitialDirectTransfer_v590ext,
    &ett_umts_rrc_pdu_def_InitialDirectTransfer_v6xyext_IEs,
    &ett_umts_rrc_pdu_def_HandoverFromUTRANCommand_GSM,
    &ett_umts_rrc_pdu_def_T_r39,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions14,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions9,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions25,
    &ett_umts_rrc_pdu_def_T_later_than_r37,
    &ett_umts_rrc_pdu_def_T_criticalExtensions19,
    &ett_umts_rrc_pdu_def_T_r64,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions26,
    &ett_umts_rrc_pdu_def_T_criticalExtensions20,
    &ett_umts_rrc_pdu_def_HandoverFromUTRANCommand_GSM_r3_IEs,
    &ett_umts_rrc_pdu_def_T_gsm_message,
    &ett_umts_rrc_pdu_def_T_single_GSM_Message,
    &ett_umts_rrc_pdu_def_T_gsm_MessageList,
    &ett_umts_rrc_pdu_def_HandoverFromUTRANCommand_GSM_r6_IEs,
    &ett_umts_rrc_pdu_def_T_gsm_message1,
    &ett_umts_rrc_pdu_def_T_single_GSM_Message1,
    &ett_umts_rrc_pdu_def_T_gsm_MessageList1,
    &ett_umts_rrc_pdu_def_T_geran_SystemInfoType1,
    &ett_umts_rrc_pdu_def_HandoverFromUTRANCommand_GSM_v6xyext_IEs,
    &ett_umts_rrc_pdu_def_T_geran_SystemInfoType2,
    &ett_umts_rrc_pdu_def_HandoverFromUTRANCommand_GERANIu,
    &ett_umts_rrc_pdu_def_T_handoverFromUTRANCommand_GERANIu,
    &ett_umts_rrc_pdu_def_T_r53,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions27,
    &ett_umts_rrc_pdu_def_T_later_than_r5,
    &ett_umts_rrc_pdu_def_T_criticalExtensions21,
    &ett_umts_rrc_pdu_def_HandoverFromUTRANCommand_GERANIu_r5_IEs,
    &ett_umts_rrc_pdu_def_T_geranIu_Message,
    &ett_umts_rrc_pdu_def_T_single_GERANIu_Message,
    &ett_umts_rrc_pdu_def_T_geranIu_MessageList,
    &ett_umts_rrc_pdu_def_HandoverFromUTRANCommand_CDMA2000,
    &ett_umts_rrc_pdu_def_T_r310,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions15,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions28,
    &ett_umts_rrc_pdu_def_T_later_than_r38,
    &ett_umts_rrc_pdu_def_T_criticalExtensions22,
    &ett_umts_rrc_pdu_def_HandoverFromUTRANCommand_CDMA2000_r3_IEs,
    &ett_umts_rrc_pdu_def_HandoverFromUTRANFailure,
    &ett_umts_rrc_pdu_def_T_interRATMessage,
    &ett_umts_rrc_pdu_def_T_gsm,
    &ett_umts_rrc_pdu_def_T_cdma2000,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions16,
    &ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions6,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions29,
    &ett_umts_rrc_pdu_def_HandoverFromUtranFailure_v590ext_IEs,
    &ett_umts_rrc_pdu_def_InterRATHandoverInfo,
    &ett_umts_rrc_pdu_def_T_predefinedConfigStatusList,
    &ett_umts_rrc_pdu_def_T_uE_SecurityInformation,
    &ett_umts_rrc_pdu_def_T_ue_CapabilityContainer,
    &ett_umts_rrc_pdu_def_T_v390NonCriticalExtensions,
    &ett_umts_rrc_pdu_def_T_present,
    &ett_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions3,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions17,
    &ett_umts_rrc_pdu_def_T_v3g0NonCriticalExtensions,
    &ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions4,
    &ett_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions2,
    &ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions7,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions10,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions30,
    &ett_umts_rrc_pdu_def_InterRATHandoverInfo_v390ext_IEs,
    &ett_umts_rrc_pdu_def_InterRATHandoverInfo_v3a0ext_IEs,
    &ett_umts_rrc_pdu_def_InterRATHandoverInfo_v3d0ext_IEs,
    &ett_umts_rrc_pdu_def_InterRATHandoverInfo_v3g0ext_IEs,
    &ett_umts_rrc_pdu_def_InterRATHandoverInfo_v4b0ext_IEs,
    &ett_umts_rrc_pdu_def_InterRATHandoverInfo_v4d0ext_IEs,
    &ett_umts_rrc_pdu_def_InterRATHandoverInfo_v590ext_IEs,
    &ett_umts_rrc_pdu_def_InterRATHandoverInfo_v6xyext_IEs,
    &ett_umts_rrc_pdu_def_MeasurementControl,
    &ett_umts_rrc_pdu_def_T_r311,
    &ett_umts_rrc_pdu_def_T_v390nonCriticalExtensions,
    &ett_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions4,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions18,
    &ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions5,
    &ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions8,
    &ett_umts_rrc_pdu_def_T_v5b0NonCriticalExtensions,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions31,
    &ett_umts_rrc_pdu_def_T_later_than_r39,
    &ett_umts_rrc_pdu_def_T_criticalExtensions23,
    &ett_umts_rrc_pdu_def_T_r43,
    &ett_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions3,
    &ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions9,
    &ett_umts_rrc_pdu_def_T_v5b0NonCriticalExtensions1,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions32,
    &ett_umts_rrc_pdu_def_T_criticalExtensions24,
    &ett_umts_rrc_pdu_def_MeasurementControl_r3_IEs,
    &ett_umts_rrc_pdu_def_MeasurementControl_v390ext,
    &ett_umts_rrc_pdu_def_MeasurementControl_v3a0ext,
    &ett_umts_rrc_pdu_def_MeasurementControl_r4_IEs,
    &ett_umts_rrc_pdu_def_MeasurementControl_v590ext_IEs,
    &ett_umts_rrc_pdu_def_T_measurementCommand_v590ext,
    &ett_umts_rrc_pdu_def_MeasurementControl_v5b0ext_IEs,
    &ett_umts_rrc_pdu_def_MeasurementControlFailure,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions19,
    &ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions10,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions33,
    &ett_umts_rrc_pdu_def_MeasurementControlFailure_v590ext_IEs,
    &ett_umts_rrc_pdu_def_MeasurementReport,
    &ett_umts_rrc_pdu_def_T_v390nonCriticalExtensions1,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions20,
    &ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions6,
    &ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions11,
    &ett_umts_rrc_pdu_def_T_v5b0NonCriticalExtensions2,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions11,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions34,
    &ett_umts_rrc_pdu_def_MeasurementReport_v390ext,
    &ett_umts_rrc_pdu_def_MeasurementReport_v4b0ext_IEs,
    &ett_umts_rrc_pdu_def_MeasurementReport_v590ext_IEs,
    &ett_umts_rrc_pdu_def_MeasurementReport_v5b0ext_IEs,
    &ett_umts_rrc_pdu_def_MeasurementReport_v6xyext_IEs,
    &ett_umts_rrc_pdu_def_PagingType1,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions21,
    &ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions12,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions35,
    &ett_umts_rrc_pdu_def_PagingType1_v590ext_IEs,
    &ett_umts_rrc_pdu_def_PagingType2,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions22,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions36,
    &ett_umts_rrc_pdu_def_PhysicalChannelReconfiguration,
    &ett_umts_rrc_pdu_def_T_r312,
    &ett_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions5,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions23,
    &ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtenstions,
    &ett_umts_rrc_pdu_def_T_v590NonCriticalExtenstions2,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions12,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions37,
    &ett_umts_rrc_pdu_def_T_later_than_r310,
    &ett_umts_rrc_pdu_def_T_criticalExtensions25,
    &ett_umts_rrc_pdu_def_T_r44,
    &ett_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions4,
    &ett_umts_rrc_pdu_def_T_v590NonCriticalExtenstions3,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions13,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions38,
    &ett_umts_rrc_pdu_def_T_criticalExtensions26,
    &ett_umts_rrc_pdu_def_T_r54,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions14,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions39,
    &ett_umts_rrc_pdu_def_T_criticalExtensions27,
    &ett_umts_rrc_pdu_def_T_r65,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions40,
    &ett_umts_rrc_pdu_def_T_criticalExtensions28,
    &ett_umts_rrc_pdu_def_PhysicalChannelReconfiguration_r3_IEs,
    &ett_umts_rrc_pdu_def_T_modeSpecificInfo6,
    &ett_umts_rrc_pdu_def_T_fdd12,
    &ett_umts_rrc_pdu_def_PhysicalChannelReconfiguration_v3a0ext,
    &ett_umts_rrc_pdu_def_PhysicalChannelReconfiguration_v4b0ext_IEs,
    &ett_umts_rrc_pdu_def_PhysicalChannelReconfiguration_v590ext_IEs,
    &ett_umts_rrc_pdu_def_PhysicalChannelReconfiguration_r4_IEs,
    &ett_umts_rrc_pdu_def_T_modeSpecificInfo7,
    &ett_umts_rrc_pdu_def_T_fdd13,
    &ett_umts_rrc_pdu_def_PhysicalChannelReconfiguration_r5_IEs,
    &ett_umts_rrc_pdu_def_T_modeSpecificInfo8,
    &ett_umts_rrc_pdu_def_T_fdd14,
    &ett_umts_rrc_pdu_def_PhysicalChannelReconfiguration_r6_IEs,
    &ett_umts_rrc_pdu_def_PhysicalChannelReconfiguration_v6xyext_IEs,
    &ett_umts_rrc_pdu_def_PhysicalChannelReconfigurationComplete,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions24,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions41,
    &ett_umts_rrc_pdu_def_PhysicalChannelReconfigurationFailure,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions25,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions42,
    &ett_umts_rrc_pdu_def_PhysicalSharedChannelAllocation,
    &ett_umts_rrc_pdu_def_T_r313,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions26,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions43,
    &ett_umts_rrc_pdu_def_T_later_than_r311,
    &ett_umts_rrc_pdu_def_T_criticalExtensions29,
    &ett_umts_rrc_pdu_def_T_r45,
    &ett_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions5,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions15,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions44,
    &ett_umts_rrc_pdu_def_T_criticalExtensions30,
    &ett_umts_rrc_pdu_def_PhysicalSharedChannelAllocation_r3_IEs,
    &ett_umts_rrc_pdu_def_PhysicalSharedChannelAllocation_r4_IEs,
    &ett_umts_rrc_pdu_def_PhysicalSharedChannelAllocation_v6xyext_IEs,
    &ett_umts_rrc_pdu_def_PUSCHCapacityRequest,
    &ett_umts_rrc_pdu_def_T_allocationConfirmation,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions27,
    &ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions13,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions45,
    &ett_umts_rrc_pdu_def_PUSCHCapacityRequest_v590ext,
    &ett_umts_rrc_pdu_def_RadioBearerReconfiguration,
    &ett_umts_rrc_pdu_def_T_r314,
    &ett_umts_rrc_pdu_def_T_v3aoNonCriticalExtensions,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions28,
    &ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions7,
    &ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions14,
    &ett_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions6,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions16,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions46,
    &ett_umts_rrc_pdu_def_T_later_than_r312,
    &ett_umts_rrc_pdu_def_T_criticalExtensions31,
    &ett_umts_rrc_pdu_def_T_r46,
    &ett_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions6,
    &ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions15,
    &ett_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions7,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions17,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions47,
    &ett_umts_rrc_pdu_def_T_criticalExtensions32,
    &ett_umts_rrc_pdu_def_T_r55,
    &ett_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions8,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions18,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions48,
    &ett_umts_rrc_pdu_def_T_criticalExtensions33,
    &ett_umts_rrc_pdu_def_T_r66,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions49,
    &ett_umts_rrc_pdu_def_T_criticalExtensions34,
    &ett_umts_rrc_pdu_def_RadioBearerReconfiguration_r3_IEs,
    &ett_umts_rrc_pdu_def_T_dummy1,
    &ett_umts_rrc_pdu_def_T_fdd15,
    &ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo3,
    &ett_umts_rrc_pdu_def_T_fdd16,
    &ett_umts_rrc_pdu_def_RadioBearerReconfiguration_v3a0ext,
    &ett_umts_rrc_pdu_def_RadioBearerReconfiguration_v4b0ext_IEs,
    &ett_umts_rrc_pdu_def_RadioBearerReconfiguration_v590ext_IEs,
    &ett_umts_rrc_pdu_def_RadioBearerReconfiguration_v5d0ext_IEs,
    &ett_umts_rrc_pdu_def_RadioBearerReconfiguration_r4_IEs,
    &ett_umts_rrc_pdu_def_T_radioBearerReconfiguration_r4_IEs_dummy,
    &ett_umts_rrc_pdu_def_T_fdd17,
    &ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo4,
    &ett_umts_rrc_pdu_def_T_fdd18,
    &ett_umts_rrc_pdu_def_RadioBearerReconfiguration_r5_IEs,
    &ett_umts_rrc_pdu_def_T_specificationMode4,
    &ett_umts_rrc_pdu_def_T_complete4,
    &ett_umts_rrc_pdu_def_T_dummy_1a,
    &ett_umts_rrc_pdu_def_T_fdd19,
    &ett_umts_rrc_pdu_def_T_preconfiguration3,
    &ett_umts_rrc_pdu_def_T_preConfigMode3,
    &ett_umts_rrc_pdu_def_T_defaultConfig3,
    &ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo5,
    &ett_umts_rrc_pdu_def_T_fdd20,
    &ett_umts_rrc_pdu_def_RadioBearerReconfiguration_r6_IEs,
    &ett_umts_rrc_pdu_def_T_specificationMode5,
    &ett_umts_rrc_pdu_def_T_complete5,
    &ett_umts_rrc_pdu_def_T_preconfiguration4,
    &ett_umts_rrc_pdu_def_T_preConfigMode4,
    &ett_umts_rrc_pdu_def_T_defaultConfig4,
    &ett_umts_rrc_pdu_def_RadioBearerReconfiguration_v6xyext_IEs,
    &ett_umts_rrc_pdu_def_RadioBearerReconfigurationComplete,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions29,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions50,
    &ett_umts_rrc_pdu_def_RadioBearerReconfigurationFailure,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions30,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions51,
    &ett_umts_rrc_pdu_def_RadioBearerRelease,
    &ett_umts_rrc_pdu_def_T_r315,
    &ett_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions6,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions31,
    &ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions8,
    &ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions16,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions19,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions52,
    &ett_umts_rrc_pdu_def_T_later_than_r313,
    &ett_umts_rrc_pdu_def_T_criticalExtensions35,
    &ett_umts_rrc_pdu_def_T_r47,
    &ett_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions7,
    &ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions17,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions20,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions53,
    &ett_umts_rrc_pdu_def_T_criticalExtensions36,
    &ett_umts_rrc_pdu_def_T_r56,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions21,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions54,
    &ett_umts_rrc_pdu_def_T_criticalExtensions37,
    &ett_umts_rrc_pdu_def_T_r67,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions55,
    &ett_umts_rrc_pdu_def_T_criticalExtensions38,
    &ett_umts_rrc_pdu_def_RadioBearerRelease_r3_IEs,
    &ett_umts_rrc_pdu_def_T_radioBearerRelease_r3_IEs_dummy,
    &ett_umts_rrc_pdu_def_T_fdd21,
    &ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo6,
    &ett_umts_rrc_pdu_def_T_fdd22,
    &ett_umts_rrc_pdu_def_RadioBearerRelease_v3a0ext,
    &ett_umts_rrc_pdu_def_RadioBearerRelease_v4b0ext_IEs,
    &ett_umts_rrc_pdu_def_RadioBearerRelease_v590ext_IEs,
    &ett_umts_rrc_pdu_def_RadioBearerRelease_r4_IEs,
    &ett_umts_rrc_pdu_def_T_radioBearerRelease_r4_IEs_dummy,
    &ett_umts_rrc_pdu_def_T_fdd23,
    &ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo7,
    &ett_umts_rrc_pdu_def_T_fdd24,
    &ett_umts_rrc_pdu_def_RadioBearerRelease_r5_IEs,
    &ett_umts_rrc_pdu_def_T_radioBearerRelease_r5_IEs_dummy,
    &ett_umts_rrc_pdu_def_T_fdd25,
    &ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo8,
    &ett_umts_rrc_pdu_def_T_fdd26,
    &ett_umts_rrc_pdu_def_RadioBearerRelease_v6xyext_IEs,
    &ett_umts_rrc_pdu_def_RadioBearerRelease_r6_IEs,
    &ett_umts_rrc_pdu_def_RadioBearerReleaseComplete,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions32,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions56,
    &ett_umts_rrc_pdu_def_RadioBearerReleaseFailure,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions33,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions57,
    &ett_umts_rrc_pdu_def_RadioBearerSetup,
    &ett_umts_rrc_pdu_def_T_r316,
    &ett_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions7,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions34,
    &ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions9,
    &ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions18,
    &ett_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions9,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions22,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions58,
    &ett_umts_rrc_pdu_def_T_later_than_r314,
    &ett_umts_rrc_pdu_def_T_criticalExtensions39,
    &ett_umts_rrc_pdu_def_T_r48,
    &ett_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions8,
    &ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions19,
    &ett_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions10,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions23,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions59,
    &ett_umts_rrc_pdu_def_T_criticalExtensions40,
    &ett_umts_rrc_pdu_def_T_r57,
    &ett_umts_rrc_pdu_def_T_v5d0NonCriticalExtenstions11,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions24,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions60,
    &ett_umts_rrc_pdu_def_T_criticalExtensions41,
    &ett_umts_rrc_pdu_def_T_r68,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions61,
    &ett_umts_rrc_pdu_def_T_criticalExtensions42,
    &ett_umts_rrc_pdu_def_RadioBearerSetup_r3_IEs,
    &ett_umts_rrc_pdu_def_T_radioBearerSetup_r3_IEs_dummy,
    &ett_umts_rrc_pdu_def_T_fdd27,
    &ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo9,
    &ett_umts_rrc_pdu_def_T_fdd28,
    &ett_umts_rrc_pdu_def_RadioBearerSetup_v3a0ext,
    &ett_umts_rrc_pdu_def_RadioBearerSetup_v4b0ext_IEs,
    &ett_umts_rrc_pdu_def_RadioBearerSetup_v590ext_IEs,
    &ett_umts_rrc_pdu_def_RadioBearerSetup_v5d0ext_IEs,
    &ett_umts_rrc_pdu_def_RadioBearerSetup_r4_IEs,
    &ett_umts_rrc_pdu_def_T_radioBearerSetup_r4_IEs_dummy,
    &ett_umts_rrc_pdu_def_T_fdd29,
    &ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo10,
    &ett_umts_rrc_pdu_def_T_fdd30,
    &ett_umts_rrc_pdu_def_RadioBearerSetup_r5_IEs,
    &ett_umts_rrc_pdu_def_T_radioBearerSetup_r5_IEs_dummy,
    &ett_umts_rrc_pdu_def_T_fdd31,
    &ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo11,
    &ett_umts_rrc_pdu_def_T_fdd32,
    &ett_umts_rrc_pdu_def_RadioBearerSetup_v6xyext_IEs,
    &ett_umts_rrc_pdu_def_RadioBearerSetup_r6_IEs,
    &ett_umts_rrc_pdu_def_T_specificationMode6,
    &ett_umts_rrc_pdu_def_T_complete6,
    &ett_umts_rrc_pdu_def_T_preconfiguration5,
    &ett_umts_rrc_pdu_def_RadioBearerSetupComplete,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions35,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions62,
    &ett_umts_rrc_pdu_def_RadioBearerSetupFailure,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions36,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions63,
    &ett_umts_rrc_pdu_def_RRCConnectionReject,
    &ett_umts_rrc_pdu_def_T_r317,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions37,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions25,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions64,
    &ett_umts_rrc_pdu_def_T_later_than_r315,
    &ett_umts_rrc_pdu_def_T_criticalExtensions43,
    &ett_umts_rrc_pdu_def_RRCConnectionReject_r3_IEs,
    &ett_umts_rrc_pdu_def_RRCConnectionReject_v6xyext_IEs,
    &ett_umts_rrc_pdu_def_RRCConnectionRelease,
    &ett_umts_rrc_pdu_def_T_r318,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions38,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions26,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions65,
    &ett_umts_rrc_pdu_def_T_later_than_r316,
    &ett_umts_rrc_pdu_def_T_criticalExtensions44,
    &ett_umts_rrc_pdu_def_T_r49,
    &ett_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions9,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions27,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions66,
    &ett_umts_rrc_pdu_def_T_criticalExtensions45,
    &ett_umts_rrc_pdu_def_RRCConnectionRelease_r3_IEs,
    &ett_umts_rrc_pdu_def_RRCConnectionRelease_r4_IEs,
    &ett_umts_rrc_pdu_def_RRCConnectionRelease_v6xyext_IEs,
    &ett_umts_rrc_pdu_def_RRCConnectionRelease_CCCH,
    &ett_umts_rrc_pdu_def_T_r319,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions39,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions67,
    &ett_umts_rrc_pdu_def_T_later_than_r317,
    &ett_umts_rrc_pdu_def_T_criticalExtensions46,
    &ett_umts_rrc_pdu_def_T_r410,
    &ett_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions10,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions68,
    &ett_umts_rrc_pdu_def_T_criticalExtensions47,
    &ett_umts_rrc_pdu_def_SEQUENCE_SIZE_1_maxURNTIGroup_OF_GroupReleaseInformation,
    &ett_umts_rrc_pdu_def_T_criticalExtensions48,
    &ett_umts_rrc_pdu_def_T_r58,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions69,
    &ett_umts_rrc_pdu_def_T_criticalExtensions49,
    &ett_umts_rrc_pdu_def_RRCConnectionRelease_CCCH_r3_IEs,
    &ett_umts_rrc_pdu_def_RRCConnectionRelease_CCCH_r4_IEs,
    &ett_umts_rrc_pdu_def_RRCConnectionReleaseComplete,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions40,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions70,
    &ett_umts_rrc_pdu_def_RRCConnectionRequest,
    &ett_umts_rrc_pdu_def_T_v3d0NonCriticalExtensions,
    &ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions10,
    &ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions20,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions28,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions71,
    &ett_umts_rrc_pdu_def_RRCConnectionRequest_v3d0ext_IEs,
    &ett_umts_rrc_pdu_def_RRCConnectionRequest_v4b0ext_IEs,
    &ett_umts_rrc_pdu_def_RRCConnectionRequest_v590ext_IEs,
    &ett_umts_rrc_pdu_def_RRCConnectionRequest_v6xyext_IEs,
    &ett_umts_rrc_pdu_def_RRCConnectionSetup,
    &ett_umts_rrc_pdu_def_T_r320,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions41,
    &ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions11,
    &ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions21,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions72,
    &ett_umts_rrc_pdu_def_T_later_than_r318,
    &ett_umts_rrc_pdu_def_T_criticalExtensions50,
    &ett_umts_rrc_pdu_def_T_r411,
    &ett_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions11,
    &ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions22,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions29,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions73,
    &ett_umts_rrc_pdu_def_T_criticalExtensions51,
    &ett_umts_rrc_pdu_def_T_r59,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions30,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions74,
    &ett_umts_rrc_pdu_def_T_criticalExtensions52,
    &ett_umts_rrc_pdu_def_T_r69,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions75,
    &ett_umts_rrc_pdu_def_T_criticalExtensions53,
    &ett_umts_rrc_pdu_def_RRCConnectionSetup_r3_IEs,
    &ett_umts_rrc_pdu_def_RRCConnectionSetup_v4b0ext_IEs,
    &ett_umts_rrc_pdu_def_RRCConnectionSetup_v590ext_IEs,
    &ett_umts_rrc_pdu_def_RRCConnectionSetup_r4_IEs,
    &ett_umts_rrc_pdu_def_RRCConnectionSetup_r5_IEs,
    &ett_umts_rrc_pdu_def_T_specificationMode7,
    &ett_umts_rrc_pdu_def_T_complete7,
    &ett_umts_rrc_pdu_def_T_preconfiguration6,
    &ett_umts_rrc_pdu_def_T_preConfigMode5,
    &ett_umts_rrc_pdu_def_T_defaultConfig5,
    &ett_umts_rrc_pdu_def_RRCConnectionSetup_v6xyext_IEs,
    &ett_umts_rrc_pdu_def_RRCConnectionSetup_r6_IEs,
    &ett_umts_rrc_pdu_def_T_specificationMode8,
    &ett_umts_rrc_pdu_def_T_complete8,
    &ett_umts_rrc_pdu_def_T_preconfiguration7,
    &ett_umts_rrc_pdu_def_T_preConfigMode6,
    &ett_umts_rrc_pdu_def_T_defaultConfig6,
    &ett_umts_rrc_pdu_def_RRCConnectionSetupComplete,
    &ett_umts_rrc_pdu_def_T_v370NonCriticalExtensions,
    &ett_umts_rrc_pdu_def_T_v380NonCriticalExtensions,
    &ett_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions8,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions42,
    &ett_umts_rrc_pdu_def_T_v3g0NonCriticalExtensions1,
    &ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions12,
    &ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions23,
    &ett_umts_rrc_pdu_def_T_v5c0NonCriticalExtensions,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions31,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions76,
    &ett_umts_rrc_pdu_def_RRCConnectionSetupComplete_v370ext,
    &ett_umts_rrc_pdu_def_RRCConnectionSetupComplete_v380ext_IEs,
    &ett_umts_rrc_pdu_def_RRCConnectionSetupComplete_v3a0ext_IEs,
    &ett_umts_rrc_pdu_def_RRCConnectionSetupComplete_v3g0ext_IEs,
    &ett_umts_rrc_pdu_def_RRCConnectionSetupComplete_r3_add_ext_IEs,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions77,
    &ett_umts_rrc_pdu_def_RRCConnectionSetupComplete_v4b0ext_IEs,
    &ett_umts_rrc_pdu_def_RRCConnectionSetupComplete_v590ext_IEs,
    &ett_umts_rrc_pdu_def_RRCConnectionSetupComplete_v5c0ext_IEs,
    &ett_umts_rrc_pdu_def_RRCConnectionSetupComplete_v650ext_IEs,
    &ett_umts_rrc_pdu_def_RRCConnectionSetupComplete_v6xyext_IEs,
    &ett_umts_rrc_pdu_def_RRC_FailureInfo,
    &ett_umts_rrc_pdu_def_T_r321,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions43,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions78,
    &ett_umts_rrc_pdu_def_T_criticalExtensions54,
    &ett_umts_rrc_pdu_def_RRC_FailureInfo_r3_IEs,
    &ett_umts_rrc_pdu_def_RRCStatus,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions44,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions79,
    &ett_umts_rrc_pdu_def_SecurityModeCommand,
    &ett_umts_rrc_pdu_def_T_r322,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions45,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions32,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions80,
    &ett_umts_rrc_pdu_def_T_later_than_r319,
    &ett_umts_rrc_pdu_def_T_criticalExtensions55,
    &ett_umts_rrc_pdu_def_SecurityModeCommand_r3_IEs,
    &ett_umts_rrc_pdu_def_SecurityModeCommand_v6xyext_IEs,
    &ett_umts_rrc_pdu_def_SecurityModeComplete,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions46,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions81,
    &ett_umts_rrc_pdu_def_SecurityModeFailure,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions47,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions82,
    &ett_umts_rrc_pdu_def_SignallingConnectionRelease,
    &ett_umts_rrc_pdu_def_T_r323,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions48,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions83,
    &ett_umts_rrc_pdu_def_T_later_than_r320,
    &ett_umts_rrc_pdu_def_T_criticalExtensions56,
    &ett_umts_rrc_pdu_def_SignallingConnectionRelease_r3_IEs,
    &ett_umts_rrc_pdu_def_SignallingConnectionReleaseIndication,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions49,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions84,
    &ett_umts_rrc_pdu_def_SystemInformation_BCH,
    &ett_umts_rrc_pdu_def_T_payload,
    &ett_umts_rrc_pdu_def_T_lastAndFirst,
    &ett_umts_rrc_pdu_def_T_lastAndComplete,
    &ett_umts_rrc_pdu_def_T_lastAndCompleteAndFirst,
    &ett_umts_rrc_pdu_def_T_completeAndFirst,
    &ett_umts_rrc_pdu_def_SystemInformation_FACH,
    &ett_umts_rrc_pdu_def_T_payload1,
    &ett_umts_rrc_pdu_def_T_lastAndFirst1,
    &ett_umts_rrc_pdu_def_T_lastAndComplete1,
    &ett_umts_rrc_pdu_def_T_lastAndCompleteAndFirst1,
    &ett_umts_rrc_pdu_def_T_completeAndFirst1,
    &ett_umts_rrc_pdu_def_FirstSegment,
    &ett_umts_rrc_pdu_def_FirstSegmentShort,
    &ett_umts_rrc_pdu_def_SubsequentSegment,
    &ett_umts_rrc_pdu_def_LastSegment,
    &ett_umts_rrc_pdu_def_LastSegmentShort,
    &ett_umts_rrc_pdu_def_CompleteSIB_List,
    &ett_umts_rrc_pdu_def_CompleteSIB,
    &ett_umts_rrc_pdu_def_CompleteSIBshort,
    &ett_umts_rrc_pdu_def_SystemInformationChangeIndication,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions50,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions85,
    &ett_umts_rrc_pdu_def_TransportChannelReconfiguration,
    &ett_umts_rrc_pdu_def_T_r324,
    &ett_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions9,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions51,
    &ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions13,
    &ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions24,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions33,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions86,
    &ett_umts_rrc_pdu_def_T_later_than_r321,
    &ett_umts_rrc_pdu_def_T_criticalExtensions57,
    &ett_umts_rrc_pdu_def_T_r412,
    &ett_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions12,
    &ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions25,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions34,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions87,
    &ett_umts_rrc_pdu_def_T_criticalExtensions58,
    &ett_umts_rrc_pdu_def_T_r510,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions35,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions88,
    &ett_umts_rrc_pdu_def_T_criticalExtensions59,
    &ett_umts_rrc_pdu_def_T_r610,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions89,
    &ett_umts_rrc_pdu_def_T_criticalExtensions60,
    &ett_umts_rrc_pdu_def_TransportChannelReconfiguration_r3_IEs,
    &ett_umts_rrc_pdu_def_T_transportChannelReconfiguration_r3_IEs_dummy,
    &ett_umts_rrc_pdu_def_T_fdd33,
    &ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo12,
    &ett_umts_rrc_pdu_def_T_fdd34,
    &ett_umts_rrc_pdu_def_TransportChannelReconfiguration_v3a0ext,
    &ett_umts_rrc_pdu_def_TransportChannelReconfiguration_v4b0ext_IEs,
    &ett_umts_rrc_pdu_def_TransportChannelReconfiguration_v590ext_IEs,
    &ett_umts_rrc_pdu_def_TransportChannelReconfiguration_r4_IEs,
    &ett_umts_rrc_pdu_def_T_transportChannelReconfiguration_r4_IEs_dummy,
    &ett_umts_rrc_pdu_def_T_fdd35,
    &ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo13,
    &ett_umts_rrc_pdu_def_T_fdd36,
    &ett_umts_rrc_pdu_def_TransportChannelReconfiguration_r5_IEs,
    &ett_umts_rrc_pdu_def_T_transportChannelReconfiguration_r5_IEs_dummy,
    &ett_umts_rrc_pdu_def_T_fdd37,
    &ett_umts_rrc_pdu_def_T_modeSpecificPhysChInfo14,
    &ett_umts_rrc_pdu_def_T_fdd38,
    &ett_umts_rrc_pdu_def_TransportChannelReconfiguration_v6xyext_IEs,
    &ett_umts_rrc_pdu_def_TransportChannelReconfiguration_r6_IEs,
    &ett_umts_rrc_pdu_def_TransportChannelReconfigurationComplete,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions52,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions90,
    &ett_umts_rrc_pdu_def_TransportChannelReconfigurationFailure,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions53,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions91,
    &ett_umts_rrc_pdu_def_TransportFormatCombinationControl,
    &ett_umts_rrc_pdu_def_T_modeSpecificInfo9,
    &ett_umts_rrc_pdu_def_T_tdd3,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions54,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions92,
    &ett_umts_rrc_pdu_def_TransportFormatCombinationControlFailure,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions55,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions93,
    &ett_umts_rrc_pdu_def_UECapabilityEnquiry,
    &ett_umts_rrc_pdu_def_T_r325,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions56,
    &ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions14,
    &ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions26,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions94,
    &ett_umts_rrc_pdu_def_T_later_than_r322,
    &ett_umts_rrc_pdu_def_T_criticalExtensions61,
    &ett_umts_rrc_pdu_def_UECapabilityEnquiry_r3_IEs,
    &ett_umts_rrc_pdu_def_UECapabilityEnquiry_v4b0ext_IEs,
    &ett_umts_rrc_pdu_def_UECapabilityEnquiry_v590ext_IEs,
    &ett_umts_rrc_pdu_def_UECapabilityInformation,
    &ett_umts_rrc_pdu_def_T_v370NonCriticalExtensions1,
    &ett_umts_rrc_pdu_def_T_v380NonCriticalExtensions1,
    &ett_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions10,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions57,
    &ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions15,
    &ett_umts_rrc_pdu_def_T_v590NonCriticalExtensions27,
    &ett_umts_rrc_pdu_def_T_v5c0NonCriticalExtensions1,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions36,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions95,
    &ett_umts_rrc_pdu_def_UECapabilityInformation_v370ext,
    &ett_umts_rrc_pdu_def_UECapabilityInformation_v380ext_IEs,
    &ett_umts_rrc_pdu_def_UECapabilityInformation_v3a0ext_IEs,
    &ett_umts_rrc_pdu_def_UECapabilityInformation_r3_add_ext_IEs,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions96,
    &ett_umts_rrc_pdu_def_UECapabilityInformation_v4b0ext,
    &ett_umts_rrc_pdu_def_UECapabilityInformation_v590ext,
    &ett_umts_rrc_pdu_def_UECapabilityInformation_v5c0ext,
    &ett_umts_rrc_pdu_def_UECapabilityInformation_v650ext_IEs,
    &ett_umts_rrc_pdu_def_UECapabilityInformation_v6xyext_IEs,
    &ett_umts_rrc_pdu_def_UECapabilityInformationConfirm,
    &ett_umts_rrc_pdu_def_T_r326,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions58,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions97,
    &ett_umts_rrc_pdu_def_T_later_than_r323,
    &ett_umts_rrc_pdu_def_T_criticalExtensions62,
    &ett_umts_rrc_pdu_def_UECapabilityInformationConfirm_r3_IEs,
    &ett_umts_rrc_pdu_def_UplinkDirectTransfer,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions59,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions37,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions98,
    &ett_umts_rrc_pdu_def_UplinkDirectTransfer_v6xyext_IEs,
    &ett_umts_rrc_pdu_def_UplinkPhysicalChannelControl,
    &ett_umts_rrc_pdu_def_T_r327,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions60,
    &ett_umts_rrc_pdu_def_T_v4b0NonCriticalExtensions16,
    &ett_umts_rrc_pdu_def_T_noncriticalExtensions,
    &ett_umts_rrc_pdu_def_T_later_than_r324,
    &ett_umts_rrc_pdu_def_T_criticalExtensions63,
    &ett_umts_rrc_pdu_def_T_r413,
    &ett_umts_rrc_pdu_def_T_v4d0NonCriticalExtensions13,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions38,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions99,
    &ett_umts_rrc_pdu_def_T_criticalExtensions64,
    &ett_umts_rrc_pdu_def_T_r511,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions39,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions100,
    &ett_umts_rrc_pdu_def_T_criticalExtensions65,
    &ett_umts_rrc_pdu_def_UplinkPhysicalChannelControl_r3_IEs,
    &ett_umts_rrc_pdu_def_UplinkPhysicalChannelControl_v4b0ext_IEs,
    &ett_umts_rrc_pdu_def_UplinkPhysicalChannelControl_r4_IEs,
    &ett_umts_rrc_pdu_def_T_tddOption,
    &ett_umts_rrc_pdu_def_T_tdd3842,
    &ett_umts_rrc_pdu_def_T_tdd1282,
    &ett_umts_rrc_pdu_def_UplinkPhysicalChannelControl_r5_IEs,
    &ett_umts_rrc_pdu_def_T_tddOption1,
    &ett_umts_rrc_pdu_def_T_tdd3843,
    &ett_umts_rrc_pdu_def_T_tdd1283,
    &ett_umts_rrc_pdu_def_UplinkPhysicalChannelControl_v6xyext_IEs,
    &ett_umts_rrc_pdu_def_URAUpdate,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions61,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions101,
    &ett_umts_rrc_pdu_def_URAUpdateConfirm,
    &ett_umts_rrc_pdu_def_T_r328,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions62,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions40,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions102,
    &ett_umts_rrc_pdu_def_T_later_than_r325,
    &ett_umts_rrc_pdu_def_T_criticalExtensions66,
    &ett_umts_rrc_pdu_def_T_r512,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions41,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions103,
    &ett_umts_rrc_pdu_def_T_criticalExtensions67,
    &ett_umts_rrc_pdu_def_URAUpdateConfirm_r3_IEs,
    &ett_umts_rrc_pdu_def_URAUpdateConfirm_r5_IEs,
    &ett_umts_rrc_pdu_def_URAUpdateConfirm_v6xyext_IEs,
    &ett_umts_rrc_pdu_def_URAUpdateConfirm_CCCH,
    &ett_umts_rrc_pdu_def_T_r329,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions63,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions42,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions104,
    &ett_umts_rrc_pdu_def_T_later_than_r326,
    &ett_umts_rrc_pdu_def_T_criticalExtensions68,
    &ett_umts_rrc_pdu_def_URAUpdateConfirm_CCCH_r3_IEs,
    &ett_umts_rrc_pdu_def_UTRANMobilityInformation,
    &ett_umts_rrc_pdu_def_T_r330,
    &ett_umts_rrc_pdu_def_T_v3a0NonCriticalExtensions11,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions64,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions43,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions105,
    &ett_umts_rrc_pdu_def_T_later_than_r327,
    &ett_umts_rrc_pdu_def_T_criticalExtensions69,
    &ett_umts_rrc_pdu_def_T_r513,
    &ett_umts_rrc_pdu_def_T_v6xyNonCriticalExtensions44,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions106,
    &ett_umts_rrc_pdu_def_T_criticalExtensions70,
    &ett_umts_rrc_pdu_def_UTRANMobilityInformation_r3_IEs,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions107,
    &ett_umts_rrc_pdu_def_UTRANMobilityInformation_v3a0ext_IEs,
    &ett_umts_rrc_pdu_def_UTRANMobilityInformation_r5_IEs,
    &ett_umts_rrc_pdu_def_UtranMobilityInformation_v6xyext_IEs,
    &ett_umts_rrc_pdu_def_UTRANMobilityInformationConfirm,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions65,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions108,
    &ett_umts_rrc_pdu_def_UTRANMobilityInformationFailure,
    &ett_umts_rrc_pdu_def_T_laterNonCriticalExtensions66,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions109,
    &ett_umts_rrc_pdu_def_MBMSAccessInformation,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions110,
    &ett_umts_rrc_pdu_def_MBMSCommonPTMRBInformation,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions111,
    &ett_umts_rrc_pdu_def_MBMSCurrentCellPTMRBInformation,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions112,
    &ett_umts_rrc_pdu_def_MBMSGeneralInformation,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions113,
    &ett_umts_rrc_pdu_def_MBMSModificationRequest,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions114,
    &ett_umts_rrc_pdu_def_MBMSModifiedServicesInformation,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions115,
    &ett_umts_rrc_pdu_def_MBMSNeighbouringCellPTMRBInformation,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions116,
    &ett_umts_rrc_pdu_def_MBMSSchedulingInformation,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions117,
    &ett_umts_rrc_pdu_def_MBMSUnmodifiedServicesInformation,
    &ett_umts_rrc_pdu_def_T_nonCriticalExtensions118,

/*--- End of included file: packet-umts_rrc_pdu_def-ettarr.c ---*/
#line 90 "packet-umts_rrc_pdu_def-template.c"
  };


  /* Register protocol */
  proto_umts_rrc_pdu_def = proto_register_protocol(PNAME, PSNAME, PFNAME);
  /* Register fields and subtrees */
  proto_register_field_array(proto_umts_rrc_pdu_def, hf, array_length(hf));
  proto_register_subtree_array(ett, array_length(ett));

 
  register_dissector("umts_rrc_pdu_def", dissect_umts_rrc_pdu_def, proto_umts_rrc_pdu_def);


}


/*--- proto_reg_handoff_umts_rrc_pdu_def ---------------------------------------*/
void
proto_reg_handoff_umts_rrc_pdu_def(void)
{

	umts_rrc_pdu_def_handle = find_dissector("umts_rrc_pdu_def");

}


