/* Do not modify this file.                                                   */
/* It is created automatically by the ASN.1 to Wireshark dissector compiler   */
/* .\packet-nbap.c                                                            */
/* ../../tools/asn2wrs.py -e -F -p nbap -c nbap.cnf -s packet-nbap-template nbap.asn */

/* Input file: packet-nbap-template.c */

#line 1 "packet-nbap-template.c"
/* packet-nbap.c
 * Routines for UMTS Node B Application Part(NBAP) packet dissection
 * Copyright 2005, Anders Broman <anders.broman@ericsson.com>
 *
 * $Id: packet-nbap.c 18921 2006-08-16 05:58:39Z kukosa $
 *
 * Wireshark - Network traffic analyzer
 * By Gerald Combs <gerald@wireshark.org>
 * Copyright 1998 Gerald Combs
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Ref: 3GPP TS 25.433 version 6.6.0 Release 6
 */

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <glib.h>
#include <epan/packet.h>
#include <epan/conversation.h>

#include <stdio.h>
#include <string.h>

#include "packet-nbap.h"
#include "packet-ber.h"
#include "packet-per.h"

#define PNAME  "UTRAN Iub interface NBAP signalling"
#define PSNAME "NBAP"
#define PFNAME "nbap"



static dissector_handle_t nbap_handle=NULL;

/* Initialize the protocol and registered fields */
static int proto_nbap = -1;

static int hf_nbap_pdu_length = -1;
static int hf_nbap_IE_length = -1;


/*--- Included file: packet-nbap-hf.c ---*/
#line 1 "packet-nbap-hf.c"
static int hf_nbap_NBAP_PDU_PDU = -1;             /* NBAP_PDU */
static int hf_nbap_initiatingMessage = -1;        /* InitiatingMessage */
static int hf_nbap_succesfulOutcome = -1;         /* SuccessfulOutcome */
static int hf_nbap_unsuccesfulOutcome = -1;       /* UnsuccessfulOutcome */
static int hf_nbap_outcome = -1;                  /* Outcome */
static int hf_nbap_procedureID = -1;              /* ProcedureID */
static int hf_nbap_criticality = -1;              /* Criticality */
static int hf_nbap_messageDiscriminator = -1;     /* MessageDiscriminator */
static int hf_nbap_transactionID = -1;            /* TransactionID */
static int hf_nbap_initiatingMessageValue = -1;   /* InitiatingMessageValue */
static int hf_nbap_successfulOutcomeValue = -1;   /* SuccessfulOutcomeValue */
static int hf_nbap_unsuccessfulOutcomeValue = -1;  /* UnsuccessfulOutcomeValue */
static int hf_nbap_outcomeValue = -1;             /* OutcomeValue */
static int hf_nbap_id_audit = -1;                 /* AuditRequest */
static int hf_nbap_id_auditRequired = -1;         /* AuditRequiredIndication */
static int hf_nbap_id_blockResource = -1;         /* BlockResourceRequest */
static int hf_nbap_id_cellDeletion = -1;          /* CellDeletionRequest */
static int hf_nbap_id_cellReconfiguration = -1;   /* CellReconfigurationRequestFDD */
static int hf_nbap_id_cellReconfiguration_tdd = -1;  /* CellReconfigurationRequestTDD */
static int hf_nbap_id_cellSetup = -1;             /* CellSetupRequestFDD */
static int hf_nbap_id_cellSetup_tdd = -1;         /* CellSetupRequestTDD */
static int hf_nbap_id_commonMeasurementFailure = -1;  /* CommonMeasurementFailureIndication */
static int hf_nbap_id_commonMeasurementInitiation = -1;  /* CommonMeasurementInitiationRequest */
static int hf_nbap_id_commonMeasurementReport = -1;  /* CommonMeasurementReport */
static int hf_nbap_id_commonMeasurementTermination = -1;  /* CommonMeasurementTerminationRequest */
static int hf_nbap_id_commonTransportChannelDelete = -1;  /* CommonTransportChannelDeletionRequest */
static int hf_nbap_id_commonTransportChannelReconfigure = -1;  /* CommonTransportChannelReconfigurationRequestFDD */
static int hf_nbap_id_commonTransportChannelReconfigure_tdd = -1;  /* CommonTransportChannelReconfigurationRequestTDD */
static int hf_nbap_id_commonTransportChannelSetup = -1;  /* CommonTransportChannelSetupRequestFDD */
static int hf_nbap_id_commonTransportChannelSetup_tdd = -1;  /* CommonTransportChannelSetupRequestTDD */
static int hf_nbap_id_reset = -1;                 /* ResetRequest */
static int hf_nbap_id_compressedModeCommand = -1;  /* CompressedModeCommand */
static int hf_nbap_id_dedicatedMeasurementFailure = -1;  /* DedicatedMeasurementFailureIndication */
static int hf_nbap_id_dedicatedMeasurementInitiation = -1;  /* DedicatedMeasurementInitiationRequest */
static int hf_nbap_id_dedicatedMeasurementReport = -1;  /* DedicatedMeasurementReport */
static int hf_nbap_id_dedicatedMeasurementTermination = -1;  /* DedicatedMeasurementTerminationRequest */
static int hf_nbap_id_downlinkPowerControl = -1;  /* DL_PowerControlRequest */
static int hf_nbap_id_errorIndicationForDedicated = -1;  /* ErrorIndication */
static int hf_nbap_id_privateMessageForDedicated = -1;  /* PrivateMessage */
static int hf_nbap_id_radioLinkAddition = -1;     /* RadioLinkAdditionRequestFDD */
static int hf_nbap_id_radioLinkDeletion = -1;     /* RadioLinkDeletionRequest */
static int hf_nbap_id_radioLinkFailure = -1;      /* RadioLinkFailureIndication */
static int hf_nbap_id_radioLinkRestoration = -1;  /* RadioLinkRestoreIndication */
static int hf_nbap_id_radioLinkSetup = -1;        /* RadioLinkSetupRequestFDD */
static int hf_nbap_id_radioLinkSetup_tdd = -1;    /* RadioLinkSetupRequestTDD */
static int hf_nbap_id_resourceStatusIndication = -1;  /* ResourceStatusIndication */
static int hf_nbap_id_synchronisedRadioLinkReconfigurationCancellation = -1;  /* RadioLinkReconfigurationCancel */
static int hf_nbap_id_synchronisedRadioLinkReconfigurationCommit = -1;  /* RadioLinkReconfigurationCommit */
static int hf_nbap_id_synchronisedRadioLinkReconfigurationPreparation = -1;  /* RadioLinkReconfigurationPrepareFDD */
static int hf_nbap_id_synchronisedRadioLinkReconfigurationPreparation_tdd = -1;  /* RadioLinkReconfigurationPrepareTDD */
static int hf_nbap_id_systemInformationUpdate = -1;  /* SystemInformationUpdateRequest */
static int hf_nbap_id_unblockResource = -1;       /* UnblockResourceIndication */
static int hf_nbap_id_unSynchronisedRadioLinkReconfiguration = -1;  /* RadioLinkReconfigurationRequestFDD */
static int hf_nbap_id_unSynchronisedRadioLinkReconfiguration_tdd = -1;  /* RadioLinkReconfigurationRequestTDD */
static int hf_nbap_id_downlinkPowerTimeslotControl = -1;  /* DL_PowerTimeslotControlRequest */
static int hf_nbap_id_errorIndicationForCommon = -1;  /* ErrorIndication */
static int hf_nbap_id_privateMessageForCommon = -1;  /* PrivateMessage */
static int hf_nbap_id_physicalSharedChannelReconfiguration = -1;  /* PhysicalSharedChannelReconfigurationRequestFDD */
static int hf_nbap_id_physicalSharedChannelReconfiguration_tdd = -1;  /* PhysicalSharedChannelReconfigurationRequestTDD */
static int hf_nbap_id_radioLinkPreemption = -1;   /* RadioLinkPreemptionRequiredIndication */
static int hf_nbap_id_informationExchangeFailure = -1;  /* InformationExchangeFailureIndication */
static int hf_nbap_id_informationExchangeInitiation = -1;  /* InformationExchangeInitiationRequest */
static int hf_nbap_id_informationExchangeTermination = -1;  /* InformationExchangeTerminationRequest */
static int hf_nbap_id_informationReporting = -1;  /* InformationReport */
static int hf_nbap_id_cellSynchronisationAdjustment = -1;  /* CellSynchronisationAdjustmentRequestTDD */
static int hf_nbap_id_cellSynchronisationInitiation = -1;  /* CellSynchronisationInitiationRequestTDD */
static int hf_nbap_id_cellSynchronisationReconfiguration = -1;  /* CellSynchronisationReconfigurationRequestTDD */
static int hf_nbap_id_cellSynchronisationReporting = -1;  /* CellSynchronisationReportTDD */
static int hf_nbap_id_cellSynchronisationTermination = -1;  /* CellSynchronisationTerminationRequestTDD */
static int hf_nbap_id_cellSynchronisationFailure = -1;  /* CellSynchronisationFailureIndicationTDD */
static int hf_nbap_id_BearerRearrangement = -1;   /* BearerRearrangementIndication */
static int hf_nbap_id_radioLinkActivation = -1;   /* RadioLinkActivationCommandFDD */
static int hf_nbap_id_radioLinkActivation_tdd = -1;  /* RadioLinkActivationCommandTDD */
static int hf_nbap_id_radioLinkParameterUpdate = -1;  /* RadioLinkParameterUpdateIndicationFDD */
static int hf_nbap_id_radioLinkParameterUpdate_tdd = -1;  /* RadioLinkParameterUpdateIndicationTDD */
static int hf_nbap_id_mBMSNotificationUpdate = -1;  /* MBMSNotificationUpdateCommand */
static int hf_nbap_id_audit1 = -1;                /* AuditResponse */
static int hf_nbap_id_blockResource1 = -1;        /* BlockResourceResponse */
static int hf_nbap_id_cellDeletion1 = -1;         /* CellDeletionResponse */
static int hf_nbap_id_cellReconfiguration1 = -1;  /* CellReconfigurationResponse */
static int hf_nbap_id_cellSetup1 = -1;            /* CellSetupResponse */
static int hf_nbap_id_commonMeasurementInitiation1 = -1;  /* CommonMeasurementInitiationFailure */
static int hf_nbap_id_commonTransportChannelDelete1 = -1;  /* CommonTransportChannelDeletionResponse */
static int hf_nbap_id_commonTransportChannelReconfigure1 = -1;  /* CommonTransportChannelReconfigurationResponse */
static int hf_nbap_id_commonTransportChannelSetup1 = -1;  /* CommonTransportChannelSetupResponse */
static int hf_nbap_id_reset1 = -1;                /* ResetResponse */
static int hf_nbap_id_dedicatedMeasurementInitiation1 = -1;  /* DedicatedMeasurementInitiationResponse */
static int hf_nbap_id_radioLinkAddition1 = -1;    /* RadioLinkAdditionResponseFDD */
static int hf_nbap_id_radioLinkAddition_tdd = -1;  /* RadioLinkAdditionResponseTDD */
static int hf_nbap_id_radioLinkDeletion1 = -1;    /* RadioLinkDeletionResponse */
static int hf_nbap_id_radioLinkSetup1 = -1;       /* RadioLinkSetupResponseFDD */
static int hf_nbap_id_radioLinkSetup_tdd1 = -1;   /* RadioLinkSetupResponseTDD */
static int hf_nbap_id_synchronisedRadioLinkReconfigurationPreparation1 = -1;  /* RadioLinkReconfigurationReady */
static int hf_nbap_id_systemInformationUpdate1 = -1;  /* SystemInformationUpdateResponse */
static int hf_nbap_id_unSynchronisedRadioLinkReconfiguration1 = -1;  /* RadioLinkReconfigurationResponse */
static int hf_nbap_id_physicalSharedChannelReconfiguration1 = -1;  /* PhysicalSharedChannelReconfigurationResponse */
static int hf_nbap_id_informationExchangeInitiation1 = -1;  /* InformationExchangeInitiationResponse */
static int hf_nbap_id_cellSynchronisationAdjustment1 = -1;  /* CellSynchronisationAdjustmentResponseTDD */
static int hf_nbap_id_cellSynchronisationInitiation1 = -1;  /* CellSynchronisationInitiationResponseTDD */
static int hf_nbap_id_cellSynchronisationReconfiguration1 = -1;  /* CellSynchronisationReconfigurationResponseTDD */
static int hf_nbap_id_audit2 = -1;                /* AuditFailure */
static int hf_nbap_id_blockResource2 = -1;        /* BlockResourceFailure */
static int hf_nbap_id_cellReconfiguration2 = -1;  /* CellReconfigurationFailure */
static int hf_nbap_id_cellSetup2 = -1;            /* CellSetupFailure */
static int hf_nbap_id_commonMeasurementInitiation2 = -1;  /* CommonMeasurementInitiationResponse */
static int hf_nbap_id_commonTransportChannelReconfigure2 = -1;  /* CommonTransportChannelReconfigurationFailure */
static int hf_nbap_id_commonTransportChannelSetup2 = -1;  /* CommonTransportChannelSetupFailure */
static int hf_nbap_id_dedicatedMeasurementInitiation2 = -1;  /* DedicatedMeasurementInitiationFailure */
static int hf_nbap_id_radioLinkAddition2 = -1;    /* RadioLinkAdditionFailureFDD */
static int hf_nbap_id_radioLinkAddition_tdd1 = -1;  /* RadioLinkAdditionFailureTDD */
static int hf_nbap_id_radioLinkSetup2 = -1;       /* RadioLinkSetupFailureFDD */
static int hf_nbap_id_radioLinkSetup_tdd2 = -1;   /* RadioLinkSetupFailureTDD */
static int hf_nbap_id_synchronisedRadioLinkReconfigurationPreparation2 = -1;  /* RadioLinkReconfigurationFailure */
static int hf_nbap_id_systemInformationUpdate2 = -1;  /* SystemInformationUpdateFailure */
static int hf_nbap_id_unSynchronisedRadioLinkReconfiguration2 = -1;  /* RadioLinkReconfigurationFailure */
static int hf_nbap_id_physicalSharedChannelReconfiguration2 = -1;  /* PhysicalSharedChannelReconfigurationFailure */
static int hf_nbap_id_informationExchangeInitiation2 = -1;  /* InformationExchangeInitiationFailure */
static int hf_nbap_id_cellSynchronisationAdjustment2 = -1;  /* CellSynchronisationAdjustmentFailureTDD */
static int hf_nbap_id_cellSynchronisationInitiation2 = -1;  /* CellSynchronisationInitiationFailureTDD */
static int hf_nbap_id_cellSynchronisationReconfiguration2 = -1;  /* CellSynchronisationReconfigurationFailureTDD */
static int hf_nbap_id_AICH_Information = -1;      /* Common_PhysicalChannel_Status_Information */
static int hf_nbap_id_BCH_Information = -1;       /* Common_TransportChannel_Status_Information */
static int hf_nbap_id_BCCH_ModificationTime = -1;  /* BCCH_ModificationTime */
static int hf_nbap_id_BlockingPriorityIndicator = -1;  /* BlockingPriorityIndicator */
static int hf_nbap_id_Cause = -1;                 /* Cause */
static int hf_nbap_id_CCP_InformationItem_AuditRsp = -1;  /* CCP_InformationItem_AuditRsp */
static int hf_nbap_id_CCP_InformationList_AuditRsp = -1;  /* CCP_InformationList_AuditRsp */
static int hf_nbap_id_CCP_InformationItem_ResourceStatusInd = -1;  /* CCP_InformationItem_ResourceStatusInd */
static int hf_nbap_id_Cell_InformationItem_AuditRsp = -1;  /* Cell_InformationItem_AuditRsp */
static int hf_nbap_id_Cell_InformationItem_ResourceStatusInd = -1;  /* Cell_InformationItem_ResourceStatusInd */
static int hf_nbap_id_Cell_InformationList_AuditRsp = -1;  /* Cell_InformationList_AuditRsp */
static int hf_nbap_id_CellParameterID = -1;       /* CellParameterID */
static int hf_nbap_id_CFN = -1;                   /* CFN */
static int hf_nbap_id_C_ID = -1;                  /* C_ID */
static int hf_nbap_id_CommonMeasurementAccuracy = -1;  /* CommonMeasurementAccuracy */
static int hf_nbap_id_CommonMeasurementObjectType_CM_Rprt = -1;  /* CommonMeasurementObjectType_CM_Rprt */
static int hf_nbap_id_CommonMeasurementObjectType_CM_Rqst = -1;  /* CommonMeasurementObjectType_CM_Rqst */
static int hf_nbap_id_CommonMeasurementObjectType_CM_Rsp = -1;  /* CommonMeasurementObjectType_CM_Rsp */
static int hf_nbap_id_CommonMeasurementType = -1;  /* CommonMeasurementType */
static int hf_nbap_id_CommonPhysicalChannelID = -1;  /* CommonPhysicalChannelID */
static int hf_nbap_id_CommonPhysicalChannelType_CTCH_SetupRqstFDD = -1;  /* CommonPhysicalChannelType_CTCH_SetupRqstFDD */
static int hf_nbap_id_CommonPhysicalChannelType_CTCH_SetupRqstTDD = -1;  /* CommonPhysicalChannelType_CTCH_SetupRqstTDD */
static int hf_nbap_id_CommunicationControlPortID = -1;  /* CommunicationControlPortID */
static int hf_nbap_id_ConfigurationGenerationID = -1;  /* ConfigurationGenerationID */
static int hf_nbap_id_CRNC_CommunicationContextID = -1;  /* CRNC_CommunicationContextID */
static int hf_nbap_id_CriticalityDiagnostics = -1;  /* CriticalityDiagnostics */
static int hf_nbap_id_DCHs_to_Add_FDD = -1;       /* DCH_FDD_Information */
static int hf_nbap_id_DCHs_to_Add_TDD = -1;       /* DCH_TDD_Information */
static int hf_nbap_id_DCH_DeleteList_RL_ReconfPrepFDD = -1;  /* DCH_DeleteList_RL_ReconfPrepFDD */
static int hf_nbap_id_DCH_DeleteList_RL_ReconfPrepTDD = -1;  /* DCH_DeleteList_RL_ReconfPrepTDD */
static int hf_nbap_id_DCH_DeleteList_RL_ReconfRqstFDD = -1;  /* DCH_DeleteList_RL_ReconfRqstFDD */
static int hf_nbap_id_DCH_DeleteList_RL_ReconfRqstTDD = -1;  /* DCH_DeleteList_RL_ReconfRqstTDD */
static int hf_nbap_id_DCH_FDD_Information = -1;   /* DCH_FDD_Information */
static int hf_nbap_id_DCH_TDD_Information = -1;   /* DCH_TDD_Information */
static int hf_nbap_id_DCH_InformationResponse = -1;  /* DCH_InformationResponse */
static int hf_nbap_id_FDD_DCHs_to_Modify = -1;    /* FDD_DCHs_to_Modify */
static int hf_nbap_id_TDD_DCHs_to_Modify = -1;    /* TDD_DCHs_to_Modify */
static int hf_nbap_id_DCH_RearrangeList_Bearer_RearrangeInd = -1;  /* DCH_RearrangeList_Bearer_RearrangeInd */
static int hf_nbap_id_DedicatedMeasurementObjectType_DM_Rprt = -1;  /* DedicatedMeasurementObjectType_DM_Rprt */
static int hf_nbap_id_DedicatedMeasurementObjectType_DM_Rqst = -1;  /* DedicatedMeasurementObjectType_DM_Rqst */
static int hf_nbap_id_DedicatedMeasurementObjectType_DM_Rsp = -1;  /* DedicatedMeasurementObjectType_DM_Rsp */
static int hf_nbap_id_DedicatedMeasurementType = -1;  /* DedicatedMeasurementType */
static int hf_nbap_id_DL_CCTrCH_InformationItem_RL_SetupRqstTDD = -1;  /* DL_CCTrCH_InformationItem_RL_SetupRqstTDD */
static int hf_nbap_id_DL_CCTrCH_InformationList_RL_AdditionRqstTDD = -1;  /* DL_CCTrCH_InformationList_RL_AdditionRqstTDD */
static int hf_nbap_id_DL_CCTrCH_InformationList_RL_SetupRqstTDD = -1;  /* DL_CCTrCH_InformationList_RL_SetupRqstTDD */
static int hf_nbap_id_DL_DPCH_InformationItem_RL_AdditionRqstTDD = -1;  /* DL_DPCH_InformationItem_RL_AdditionRqstTDD */
static int hf_nbap_id_DL_DPCH_InformationList_RL_SetupRqstTDD = -1;  /* DL_DPCH_InformationItem_RL_SetupRqstTDD */
static int hf_nbap_id_DL_DPCH_Information_RL_ReconfPrepFDD = -1;  /* DL_DPCH_Information_RL_ReconfPrepFDD */
static int hf_nbap_id_DL_DPCH_Information_RL_ReconfRqstFDD = -1;  /* DL_DPCH_Information_RL_ReconfRqstFDD */
static int hf_nbap_id_DL_DPCH_Information_RL_SetupRqstFDD = -1;  /* DL_DPCH_Information_RL_SetupRqstFDD */
static int hf_nbap_id_DL_DPCH_TimingAdjustment = -1;  /* DL_DPCH_TimingAdjustment */
static int hf_nbap_id_DL_ReferencePowerInformationItem_DL_PC_Rqst = -1;  /* DL_ReferencePowerInformationItem_DL_PC_Rqst */
static int hf_nbap_id_DLReferencePower = -1;      /* DL_Power */
static int hf_nbap_id_DLReferencePowerList_DL_PC_Rqst = -1;  /* DL_ReferencePowerInformationList_DL_PC_Rqst */
static int hf_nbap_id_DSCHs_to_Add_TDD = -1;      /* DSCH_TDD_Information */
static int hf_nbap_id_DSCH_Information_DeleteList_RL_ReconfPrepTDD = -1;  /* DSCH_Information_DeleteList_RL_ReconfPrepTDD */
static int hf_nbap_id_DSCH_Information_ModifyList_RL_ReconfPrepTDD = -1;  /* DSCH_Information_ModifyList_RL_ReconfPrepTDD */
static int hf_nbap_id_DSCH_InformationResponse = -1;  /* DSCH_InformationResponse */
static int hf_nbap_id_DSCH_TDD_Information = -1;  /* DSCH_TDD_Information */
static int hf_nbap_id_DSCH_RearrangeList_Bearer_RearrangeInd = -1;  /* DSCH_RearrangeList_Bearer_RearrangeInd */
static int hf_nbap_id_End_Of_Audit_Sequence_Indicator = -1;  /* End_Of_Audit_Sequence_Indicator */
static int hf_nbap_id_FACH_Information = -1;      /* Common_TransportChannel_Status_Information */
static int hf_nbap_id_FACH_ParametersList_CTCH_ReconfRqstTDD = -1;  /* FACH_ParametersList_CTCH_ReconfRqstTDD */
static int hf_nbap_id_FACH_ParametersListIE_CTCH_SetupRqstFDD = -1;  /* FACH_ParametersListIE_CTCH_SetupRqstFDD */
static int hf_nbap_id_FACH_ParametersListIE_CTCH_SetupRqstTDD = -1;  /* FACH_ParametersListIE_CTCH_SetupRqstTDD */
static int hf_nbap_id_IndicationType_ResourceStatusInd = -1;  /* IndicationType_ResourceStatusInd */
static int hf_nbap_id_Local_Cell_ID = -1;         /* Local_Cell_ID */
static int hf_nbap_id_Local_Cell_Group_InformationItem_AuditRsp = -1;  /* Local_Cell_Group_InformationItem_AuditRsp */
static int hf_nbap_id_Local_Cell_Group_InformationItem_ResourceStatusInd = -1;  /* Local_Cell_Group_InformationItem_ResourceStatusInd */
static int hf_nbap_id_Local_Cell_Group_InformationItem2_ResourceStatusInd = -1;  /* Local_Cell_Group_InformationItem2_ResourceStatusInd */
static int hf_nbap_id_Local_Cell_Group_InformationList_AuditRsp = -1;  /* Local_Cell_Group_InformationList_AuditRsp */
static int hf_nbap_id_Local_Cell_InformationItem_AuditRsp = -1;  /* Local_Cell_InformationItem_AuditRsp */
static int hf_nbap_id_Local_Cell_InformationItem_ResourceStatusInd = -1;  /* Local_Cell_InformationItem_ResourceStatusInd */
static int hf_nbap_id_Local_Cell_InformationItem2_ResourceStatusInd = -1;  /* Local_Cell_InformationItem2_ResourceStatusInd */
static int hf_nbap_id_Local_Cell_InformationList_AuditRsp = -1;  /* Local_Cell_InformationList_AuditRsp */
static int hf_nbap_id_AdjustmentPeriod = -1;      /* AdjustmentPeriod */
static int hf_nbap_id_MaxAdjustmentStep = -1;     /* MaxAdjustmentStep */
static int hf_nbap_id_MaximumTransmissionPower = -1;  /* MaximumTransmissionPower */
static int hf_nbap_id_MeasurementFilterCoefficient = -1;  /* MeasurementFilterCoefficient */
static int hf_nbap_id_MeasurementID = -1;         /* MeasurementID */
static int hf_nbap_id_MessageStructure = -1;      /* MessageStructure */
static int hf_nbap_id_MIB_SB_SIB_InformationList_SystemInfoUpdateRqst = -1;  /* MIB_SB_SIB_InformationList_SystemInfoUpdateRqst */
static int hf_nbap_id_NodeB_CommunicationContextID = -1;  /* NodeB_CommunicationContextID */
static int hf_nbap_id_NeighbouringCellMeasurementInformation = -1;  /* NeighbouringCellMeasurementInformation */
static int hf_nbap_id_P_CCPCH_Information = -1;   /* Common_PhysicalChannel_Status_Information */
static int hf_nbap_id_P_CPICH_Information = -1;   /* Common_PhysicalChannel_Status_Information */
static int hf_nbap_id_P_SCH_Information = -1;     /* Common_PhysicalChannel_Status_Information */
static int hf_nbap_id_PCCPCH_Information_Cell_ReconfRqstTDD = -1;  /* PCCPCH_Information_Cell_ReconfRqstTDD */
static int hf_nbap_id_PCCPCH_Information_Cell_SetupRqstTDD = -1;  /* PCCPCH_Information_Cell_SetupRqstTDD */
static int hf_nbap_id_PCH_Parameters_CTCH_ReconfRqstTDD = -1;  /* PCH_Parameters_CTCH_ReconfRqstTDD */
static int hf_nbap_id_PCH_ParametersItem_CTCH_SetupRqstFDD = -1;  /* PCH_ParametersItem_CTCH_SetupRqstFDD */
static int hf_nbap_id_PCH_ParametersItem_CTCH_SetupRqstTDD = -1;  /* PCH_ParametersItem_CTCH_SetupRqstTDD */
static int hf_nbap_id_PCH_Information = -1;       /* Common_TransportChannel_Status_Information */
static int hf_nbap_id_PDSCH_Information_AddListIE_PSCH_ReconfRqst = -1;  /* PDSCH_Information_AddItem_PSCH_ReconfRqst */
static int hf_nbap_id_PDSCH_Information_ModifyListIE_PSCH_ReconfRqst = -1;  /* PDSCH_Information_ModifyItem_PSCH_ReconfRqst */
static int hf_nbap_id_PDSCHSets_AddList_PSCH_ReconfRqst = -1;  /* PDSCHSets_AddList_PSCH_ReconfRqst */
static int hf_nbap_id_PDSCHSets_DeleteList_PSCH_ReconfRqst = -1;  /* PDSCHSets_DeleteList_PSCH_ReconfRqst */
static int hf_nbap_id_PDSCHSets_ModifyList_PSCH_ReconfRqst = -1;  /* PDSCHSets_ModifyList_PSCH_ReconfRqst */
static int hf_nbap_id_PICH_Information = -1;      /* Common_PhysicalChannel_Status_Information */
static int hf_nbap_id_PICH_Parameters_CTCH_ReconfRqstTDD = -1;  /* PICH_Parameters_CTCH_ReconfRqstTDD */
static int hf_nbap_id_PowerAdjustmentType = -1;   /* PowerAdjustmentType */
static int hf_nbap_id_PRACH_Information = -1;     /* Common_PhysicalChannel_Status_Information */
static int hf_nbap_id_PrimaryCCPCH_Information_Cell_ReconfRqstFDD = -1;  /* PrimaryCCPCH_Information_Cell_ReconfRqstFDD */
static int hf_nbap_id_PrimaryCCPCH_Information_Cell_SetupRqstFDD = -1;  /* PrimaryCCPCH_Information_Cell_SetupRqstFDD */
static int hf_nbap_id_PrimaryCPICH_Information_Cell_ReconfRqstFDD = -1;  /* PrimaryCPICH_Information_Cell_ReconfRqstFDD */
static int hf_nbap_id_PrimaryCPICH_Information_Cell_SetupRqstFDD = -1;  /* PrimaryCPICH_Information_Cell_SetupRqstFDD */
static int hf_nbap_id_PrimarySCH_Information_Cell_ReconfRqstFDD = -1;  /* PrimarySCH_Information_Cell_ReconfRqstFDD */
static int hf_nbap_id_PrimarySCH_Information_Cell_SetupRqstFDD = -1;  /* PrimarySCH_Information_Cell_SetupRqstFDD */
static int hf_nbap_id_PrimaryScramblingCode = -1;  /* PrimaryScramblingCode */
static int hf_nbap_id_SCH_Information_Cell_ReconfRqstTDD = -1;  /* SCH_Information_Cell_ReconfRqstTDD */
static int hf_nbap_id_SCH_Information_Cell_SetupRqstTDD = -1;  /* SCH_Information_Cell_SetupRqstTDD */
static int hf_nbap_id_PUSCH_Information_AddListIE_PSCH_ReconfRqst = -1;  /* PUSCH_Information_AddItem_PSCH_ReconfRqst */
static int hf_nbap_id_PUSCH_Information_ModifyListIE_PSCH_ReconfRqst = -1;  /* PUSCH_Information_ModifyItem_PSCH_ReconfRqst */
static int hf_nbap_id_PUSCHSets_AddList_PSCH_ReconfRqst = -1;  /* PUSCHSets_AddList_PSCH_ReconfRqst */
static int hf_nbap_id_PUSCHSets_DeleteList_PSCH_ReconfRqst = -1;  /* PUSCHSets_DeleteList_PSCH_ReconfRqst */
static int hf_nbap_id_PUSCHSets_ModifyList_PSCH_ReconfRqst = -1;  /* PUSCHSets_ModifyList_PSCH_ReconfRqst */
static int hf_nbap_id_RACH_Information = -1;      /* Common_TransportChannel_Status_Information */
static int hf_nbap_id_RACH_ParametersItem_CTCH_SetupRqstFDD = -1;  /* RACH_ParametersItem_CTCH_SetupRqstFDD */
static int hf_nbap_id_RACH_ParameterItem_CTCH_SetupRqstTDD = -1;  /* RACH_ParameterItem_CTCH_SetupRqstTDD */
static int hf_nbap_id_ReportCharacteristics = -1;  /* ReportCharacteristics */
static int hf_nbap_id_Reporting_Object_RL_FailureInd = -1;  /* Reporting_Object_RL_FailureInd */
static int hf_nbap_id_Reporting_Object_RL_RestoreInd = -1;  /* Reporting_Object_RL_RestoreInd */
static int hf_nbap_id_RL_InformationItem_DM_Rprt = -1;  /* RL_InformationItem_DM_Rprt */
static int hf_nbap_id_RL_InformationItem_DM_Rqst = -1;  /* RL_InformationItem_DM_Rqst */
static int hf_nbap_id_RL_InformationItem_DM_Rsp = -1;  /* RL_InformationItem_DM_Rsp */
static int hf_nbap_id_RL_InformationItem_RL_AdditionRqstFDD = -1;  /* RL_InformationItem_RL_AdditionRqstFDD */
static int hf_nbap_id_RL_informationItem_RL_DeletionRqst = -1;  /* RL_informationItem_RL_DeletionRqst */
static int hf_nbap_id_RL_InformationItem_RL_FailureInd = -1;  /* RL_InformationItem_RL_FailureInd */
static int hf_nbap_id_RL_InformationItem_RL_PreemptRequiredInd = -1;  /* RL_InformationItem_RL_PreemptRequiredInd */
static int hf_nbap_id_RL_InformationItem_RL_ReconfPrepFDD = -1;  /* RL_InformationItem_RL_ReconfPrepFDD */
static int hf_nbap_id_RL_InformationItem_RL_ReconfRqstFDD = -1;  /* RL_InformationItem_RL_ReconfPrepFDD */
static int hf_nbap_id_RL_InformationItem_RL_RestoreInd = -1;  /* RL_InformationItem_RL_RestoreInd */
static int hf_nbap_id_RL_InformationItem_RL_SetupRqstFDD = -1;  /* RL_InformationItem_RL_SetupRqstFDD */
static int hf_nbap_id_RL_InformationList_RL_AdditionRqstFDD = -1;  /* RL_InformationList_RL_AdditionRqstFDD */
static int hf_nbap_id_RL_informationList_RL_DeletionRqst = -1;  /* RL_informationList_RL_DeletionRqst */
static int hf_nbap_id_RL_InformationList_RL_PreemptRequiredInd = -1;  /* RL_InformationList_RL_PreemptRequiredInd */
static int hf_nbap_id_RL_InformationList_RL_ReconfPrepFDD = -1;  /* RL_InformationList_RL_ReconfPrepFDD */
static int hf_nbap_id_RL_InformationList_RL_ReconfRqstFDD = -1;  /* RL_InformationList_RL_ReconfRqstFDD */
static int hf_nbap_id_RL_InformationList_RL_SetupRqstFDD = -1;  /* RL_InformationList_RL_SetupRqstFDD */
static int hf_nbap_id_RL_InformationResponseItem_RL_AdditionRspFDD = -1;  /* RL_InformationResponseItem_RL_AdditionRspFDD */
static int hf_nbap_id_RL_InformationResponseItem_RL_ReconfReady = -1;  /* RL_InformationResponseItem_RL_ReconfReady */
static int hf_nbap_id_RL_InformationResponseItem_RL_ReconfRsp = -1;  /* RL_InformationResponseItem_RL_ReconfRsp */
static int hf_nbap_id_RL_InformationResponseItem_RL_SetupRspFDD = -1;  /* RL_InformationResponseItem_RL_SetupRspFDD */
static int hf_nbap_id_RL_InformationResponseList_RL_AdditionRspFDD = -1;  /* RL_InformationResponseList_RL_AdditionRspFDD */
static int hf_nbap_id_RL_InformationResponseList_RL_ReconfReady = -1;  /* RL_InformationResponseList_RL_ReconfReady */
static int hf_nbap_id_RL_InformationResponseList_RL_ReconfRsp = -1;  /* RL_InformationResponseList_RL_ReconfRsp */
static int hf_nbap_id_RL_InformationResponseList_RL_SetupRspFDD = -1;  /* RL_InformationResponseList_RL_SetupRspFDD */
static int hf_nbap_id_RL_InformationResponse_RL_AdditionRspTDD = -1;  /* RL_InformationResponse_RL_AdditionRspTDD */
static int hf_nbap_id_RL_InformationResponse_RL_SetupRspTDD = -1;  /* RL_InformationResponse_RL_SetupRspTDD */
static int hf_nbap_id_RL_Information_RL_AdditionRqstTDD = -1;  /* RL_Information_RL_AdditionRqstTDD */
static int hf_nbap_id_RL_Information_RL_ReconfRqstTDD = -1;  /* RL_Information_RL_ReconfRqstTDD */
static int hf_nbap_id_RL_Information_RL_ReconfPrepTDD = -1;  /* RL_Information_RL_ReconfPrepTDD */
static int hf_nbap_id_RL_Information_RL_SetupRqstTDD = -1;  /* RL_Information_RL_SetupRqstTDD */
static int hf_nbap_id_RL_ReconfigurationFailureItem_RL_ReconfFailure = -1;  /* RL_ReconfigurationFailureItem_RL_ReconfFailure */
static int hf_nbap_id_RL_Set_InformationItem_DM_Rprt = -1;  /* RL_Set_InformationItem_DM_Rprt */
static int hf_nbap_id_RL_Set_InformationItem_DM_Rsp = -1;  /* RL_Set_InformationItem_DM_Rsp */
static int hf_nbap_id_RL_Set_InformationItem_RL_FailureInd = -1;  /* RL_Set_InformationItem_RL_FailureInd */
static int hf_nbap_id_RL_Set_InformationItem_RL_RestoreInd = -1;  /* RL_Set_InformationItem_RL_RestoreInd */
static int hf_nbap_id_S_CCPCH_Information = -1;   /* Common_PhysicalChannel_Status_Information */
static int hf_nbap_id_S_CPICH_Information = -1;   /* Common_PhysicalChannel_Status_Information */
static int hf_nbap_id_SCH_Information = -1;       /* Common_PhysicalChannel_Status_Information */
static int hf_nbap_id_S_SCH_Information = -1;     /* Common_PhysicalChannel_Status_Information */
static int hf_nbap_id_Secondary_CCPCHListIE_CTCH_ReconfRqstTDD = -1;  /* Secondary_CCPCHListIE_CTCH_ReconfRqstTDD */
static int hf_nbap_id_Secondary_CCPCH_parameterListIE_CTCH_SetupRqstTDD = -1;  /* Secondary_CCPCH_parameterListIE_CTCH_SetupRqstTDD */
static int hf_nbap_id_Secondary_CCPCH_Parameters_CTCH_ReconfRqstTDD = -1;  /* Secondary_CCPCH_Parameters_CTCH_ReconfRqstTDD */
static int hf_nbap_id_SecondaryCPICH_InformationItem_Cell_ReconfRqstFDD = -1;  /* SecondaryCPICH_InformationItem_Cell_ReconfRqstFDD */
static int hf_nbap_id_SecondaryCPICH_InformationItem_Cell_SetupRqstFDD = -1;  /* SecondaryCPICH_InformationItem_Cell_SetupRqstFDD */
static int hf_nbap_id_SecondaryCPICH_InformationList_Cell_ReconfRqstFDD = -1;  /* SecondaryCPICH_InformationList_Cell_ReconfRqstFDD */
static int hf_nbap_id_SecondaryCPICH_InformationList_Cell_SetupRqstFDD = -1;  /* SecondaryCPICH_InformationList_Cell_SetupRqstFDD */
static int hf_nbap_id_SecondarySCH_Information_Cell_ReconfRqstFDD = -1;  /* SecondarySCH_Information_Cell_ReconfRqstFDD */
static int hf_nbap_id_SecondarySCH_Information_Cell_SetupRqstFDD = -1;  /* SecondarySCH_Information_Cell_SetupRqstFDD */
static int hf_nbap_id_SegmentInformationListIE_SystemInfoUpdate = -1;  /* SegmentInformationListIE_SystemInfoUpdate */
static int hf_nbap_id_SFN = -1;                   /* SFN */
static int hf_nbap_id_SignallingBearerRequestIndicator = -1;  /* SignallingBearerRequestIndicator */
static int hf_nbap_id_ShutdownTimer = -1;         /* ShutdownTimer */
static int hf_nbap_id_Start_Of_Audit_Sequence_Indicator = -1;  /* Start_Of_Audit_Sequence_Indicator */
static int hf_nbap_id_Successful_RL_InformationRespItem_RL_AdditionFailureFDD = -1;  /* Successful_RL_InformationRespItem_RL_AdditionFailureFDD */
static int hf_nbap_id_Successful_RL_InformationRespItem_RL_SetupFailureFDD = -1;  /* Successful_RL_InformationRespItem_RL_SetupFailureFDD */
static int hf_nbap_id_SyncCase = -1;              /* SyncCase */
static int hf_nbap_id_SyncCaseIndicatorItem_Cell_SetupRqstTDD_PSCH = -1;  /* SyncCaseIndicatorItem_Cell_SetupRqstTDD_PSCH */
static int hf_nbap_id_T_Cell = -1;                /* T_Cell */
static int hf_nbap_id_TargetCommunicationControlPortID = -1;  /* CommunicationControlPortID */
static int hf_nbap_id_TimeSlotConfigurationList_Cell_ReconfRqstTDD = -1;  /* TimeSlotConfigurationList_Cell_ReconfRqstTDD */
static int hf_nbap_id_TimeSlotConfigurationList_Cell_SetupRqstTDD = -1;  /* TimeSlotConfigurationList_Cell_SetupRqstTDD */
static int hf_nbap_id_TransmissionDiversityApplied = -1;  /* TransmissionDiversityApplied */
static int hf_nbap_id_TypeOfError = -1;           /* TypeOfError */
static int hf_nbap_id_UARFCNforNt = -1;           /* UARFCN */
static int hf_nbap_id_UARFCNforNd = -1;           /* UARFCN */
static int hf_nbap_id_UARFCNforNu = -1;           /* UARFCN */
static int hf_nbap_id_UL_CCTrCH_InformationItem_RL_SetupRqstTDD = -1;  /* UL_CCTrCH_InformationItem_RL_SetupRqstTDD */
static int hf_nbap_id_UL_CCTrCH_InformationList_RL_AdditionRqstTDD = -1;  /* UL_CCTrCH_InformationList_RL_SetupRqstTDD */
static int hf_nbap_id_UL_CCTrCH_InformationList_RL_SetupRqstTDD = -1;  /* UL_CCTrCH_InformationList_RL_SetupRqstTDD */
static int hf_nbap_id_UL_DPCH_InformationItem_RL_AdditionRqstTDD = -1;  /* UL_DPCH_InformationItem_RL_AdditionRqstTDD */
static int hf_nbap_id_UL_DPCH_InformationList_RL_SetupRqstTDD = -1;  /* UL_DPCH_InformationItem_RL_SetupRqstTDD */
static int hf_nbap_id_UL_DPCH_Information_RL_ReconfPrepFDD = -1;  /* UL_DPCH_Information_RL_ReconfPrepFDD */
static int hf_nbap_id_UL_DPCH_Information_RL_ReconfRqstFDD = -1;  /* UL_DPCH_Information_RL_ReconfRqstFDD */
static int hf_nbap_id_UL_DPCH_Information_RL_SetupRqstFDD = -1;  /* UL_DPCH_Information_RL_SetupRqstFDD */
static int hf_nbap_id_Unsuccessful_RL_InformationRespItem_RL_AdditionFailureFDD = -1;  /* Unsuccessful_RL_InformationRespItem_RL_AdditionFailureFDD */
static int hf_nbap_id_Unsuccessful_RL_InformationRespItem_RL_SetupFailureFDD = -1;  /* Unsuccessful_RL_InformationRespItem_RL_SetupFailureFDD */
static int hf_nbap_id_Unsuccessful_RL_InformationResp_RL_AdditionFailureTDD = -1;  /* Unsuccessful_RL_InformationResp_RL_AdditionFailureTDD */
static int hf_nbap_id_Unsuccessful_RL_InformationResp_RL_SetupFailureTDD = -1;  /* Unsuccessful_RL_InformationResp_RL_SetupFailureTDD */
static int hf_nbap_id_USCH_Information_Add = -1;  /* USCH_Information */
static int hf_nbap_id_USCH_Information_DeleteList_RL_ReconfPrepTDD = -1;  /* USCH_Information_DeleteList_RL_ReconfPrepTDD */
static int hf_nbap_id_USCH_Information_ModifyList_RL_ReconfPrepTDD = -1;  /* USCH_Information_ModifyList_RL_ReconfPrepTDD */
static int hf_nbap_id_USCH_InformationResponse = -1;  /* USCH_InformationResponse */
static int hf_nbap_id_USCH_Information = -1;      /* USCH_Information */
static int hf_nbap_id_USCH_RearrangeList_Bearer_RearrangeInd = -1;  /* USCH_RearrangeList_Bearer_RearrangeInd */
static int hf_nbap_id_Active_Pattern_Sequence_Information = -1;  /* Active_Pattern_Sequence_Information */
static int hf_nbap_id_AICH_ParametersListIE_CTCH_ReconfRqstFDD = -1;  /* AICH_ParametersListIE_CTCH_ReconfRqstFDD */
static int hf_nbap_id_AdjustmentRatio = -1;       /* ScaledAdjustmentRatio */
static int hf_nbap_id_FACH_ParametersListIE_CTCH_ReconfRqstFDD = -1;  /* FACH_ParametersListIE_CTCH_ReconfRqstFDD */
static int hf_nbap_id_CauseLevel_PSCH_ReconfFailure = -1;  /* CauseLevel_PSCH_ReconfFailure */
static int hf_nbap_id_CauseLevel_RL_AdditionFailureFDD = -1;  /* CauseLevel_RL_AdditionFailureFDD */
static int hf_nbap_id_CauseLevel_RL_AdditionFailureTDD = -1;  /* CauseLevel_RL_AdditionFailureTDD */
static int hf_nbap_id_CauseLevel_RL_ReconfFailure = -1;  /* CauseLevel_RL_ReconfFailure */
static int hf_nbap_id_CauseLevel_RL_SetupFailureFDD = -1;  /* CauseLevel_RL_SetupFailureFDD */
static int hf_nbap_id_CauseLevel_RL_SetupFailureTDD = -1;  /* CauseLevel_RL_SetupFailureTDD */
static int hf_nbap_id_Closed_Loop_Timing_Adjustment_Mode = -1;  /* Closedlooptimingadjustmentmode */
static int hf_nbap_id_CommonPhysicalChannelType_CTCH_ReconfRqstFDD = -1;  /* CommonPhysicalChannelType_CTCH_ReconfRqstFDD */
static int hf_nbap_id_Compressed_Mode_Deactivation_Flag = -1;  /* Compressed_Mode_Deactivation_Flag */
static int hf_nbap_id_DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD = -1;  /* DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD */
static int hf_nbap_id_DL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD = -1;  /* DL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD */
static int hf_nbap_id_DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD = -1;  /* DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD */
static int hf_nbap_id_DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD = -1;  /* DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD */
static int hf_nbap_id_DL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD = -1;  /* DL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD */
static int hf_nbap_id_DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD = -1;  /* DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD */
static int hf_nbap_id_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD = -1;  /* DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD */
static int hf_nbap_id_DL_DPCH_InformationAddListIE_RL_ReconfPrepTDD = -1;  /* DL_DPCH_InformationAddItem_RL_ReconfPrepTDD */
static int hf_nbap_id_DL_DPCH_InformationModify_AddListIE_RL_ReconfPrepTDD = -1;  /* DL_DPCH_InformationModify_AddItem_RL_ReconfPrepTDD */
static int hf_nbap_id_DL_DPCH_InformationModify_DeleteListIE_RL_ReconfPrepTDD = -1;  /* DL_DPCH_InformationModify_DeleteItem_RL_ReconfPrepTDD */
static int hf_nbap_id_DL_DPCH_InformationModify_ModifyListIE_RL_ReconfPrepTDD = -1;  /* DL_DPCH_InformationModify_ModifyItem_RL_ReconfPrepTDD */
static int hf_nbap_id_DL_TPC_Pattern01Count = -1;  /* DL_TPC_Pattern01Count */
static int hf_nbap_id_DPC_Mode = -1;              /* DPC_Mode */
static int hf_nbap_id_DPCHConstant = -1;          /* ConstantValue */
static int hf_nbap_id_FACH_ParametersList_CTCH_SetupRsp = -1;  /* FACH_CommonTransportChannel_InformationResponse */
static int hf_nbap_id_Limited_power_increase_information_Cell_SetupRqstFDD = -1;  /* Limited_power_increase_information_Cell_SetupRqstFDD */
static int hf_nbap_id_PCH_Parameters_CTCH_SetupRsp = -1;  /* CommonTransportChannel_InformationResponse */
static int hf_nbap_id_PCH_ParametersItem_CTCH_ReconfRqstFDD = -1;  /* PCH_ParametersItem_CTCH_ReconfRqstFDD */
static int hf_nbap_id_PICH_ParametersItem_CTCH_ReconfRqstFDD = -1;  /* PICH_ParametersItem_CTCH_ReconfRqstFDD */
static int hf_nbap_id_PRACHConstant = -1;         /* ConstantValue */
static int hf_nbap_id_PRACH_ParametersListIE_CTCH_ReconfRqstFDD = -1;  /* PRACH_ParametersListIE_CTCH_ReconfRqstFDD */
static int hf_nbap_id_PUSCHConstant = -1;         /* ConstantValue */
static int hf_nbap_id_RACH_Parameters_CTCH_SetupRsp = -1;  /* CommonTransportChannel_InformationResponse */
static int hf_nbap_id_Synchronisation_Configuration_Cell_ReconfRqst = -1;  /* Synchronisation_Configuration_Cell_ReconfRqst */
static int hf_nbap_id_Synchronisation_Configuration_Cell_SetupRqst = -1;  /* Synchronisation_Configuration_Cell_SetupRqst */
static int hf_nbap_id_Transmission_Gap_Pattern_Sequence_Information = -1;  /* Transmission_Gap_Pattern_Sequence_Information */
static int hf_nbap_id_UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD = -1;  /* UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD */
static int hf_nbap_id_UL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD = -1;  /* UL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD */
static int hf_nbap_id_UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD = -1;  /* UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD */
static int hf_nbap_id_UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD = -1;  /* UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD */
static int hf_nbap_id_UL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD = -1;  /* UL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD */
static int hf_nbap_id_UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD = -1;  /* UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD */
static int hf_nbap_id_UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD = -1;  /* UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD */
static int hf_nbap_id_UL_DPCH_InformationAddListIE_RL_ReconfPrepTDD = -1;  /* UL_DPCH_InformationAddItem_RL_ReconfPrepTDD */
static int hf_nbap_id_UL_DPCH_InformationModify_AddListIE_RL_ReconfPrepTDD = -1;  /* UL_DPCH_InformationModify_AddItem_RL_ReconfPrepTDD */
static int hf_nbap_id_UL_DPCH_InformationModify_DeleteListIE_RL_ReconfPrepTDD = -1;  /* UL_DPCH_InformationModify_DeleteListIE_RL_ReconfPrepTDD */
static int hf_nbap_id_UL_DPCH_InformationModify_ModifyListIE_RL_ReconfPrepTDD = -1;  /* UL_DPCH_InformationModify_ModifyItem_RL_ReconfPrepTDD */
static int hf_nbap_id_Unsuccessful_PDSCHSetItem_PSCH_ReconfFailureTDD = -1;  /* Unsuccessful_PDSCHSetItem_PSCH_ReconfFailureTDD */
static int hf_nbap_id_Unsuccessful_PUSCHSetItem_PSCH_ReconfFailureTDD = -1;  /* Unsuccessful_PUSCHSetItem_PSCH_ReconfFailureTDD */
static int hf_nbap_id_CommunicationContextInfoItem_Reset = -1;  /* CommunicationContextInfoItem_Reset */
static int hf_nbap_id_CommunicationControlPortInfoItem_Reset = -1;  /* CommunicationControlPortInfoItem_Reset */
static int hf_nbap_id_ResetIndicator = -1;        /* ResetIndicator */
static int hf_nbap_id_TimingAdvanceApplied = -1;  /* TimingAdvanceApplied */
static int hf_nbap_id_CFNReportingIndicator = -1;  /* FNReportingIndicator */
static int hf_nbap_id_SFNReportingIndicator = -1;  /* FNReportingIndicator */
static int hf_nbap_id_InnerLoopDLPCStatus = -1;   /* InnerLoopDLPCStatus */
static int hf_nbap_id_TimeslotISCPInfo = -1;      /* DL_TimeslotISCPInfo */
static int hf_nbap_id_PICH_ParametersItem_CTCH_SetupRqstTDD = -1;  /* PICH_ParametersItem_CTCH_SetupRqstTDD */
static int hf_nbap_id_PRACH_ParametersItem_CTCH_SetupRqstTDD = -1;  /* PRACH_ParametersItem_CTCH_SetupRqstTDD */
static int hf_nbap_id_CCTrCH_InformationItem_RL_FailureInd = -1;  /* CCTrCH_InformationItem_RL_FailureInd */
static int hf_nbap_id_CCTrCH_InformationItem_RL_RestoreInd = -1;  /* CCTrCH_InformationItem_RL_RestoreInd */
static int hf_nbap_id_CauseLevel_SyncAdjustmntFailureTDD = -1;  /* CauseLevel_SyncAdjustmntFailureTDD */
static int hf_nbap_id_CellAdjustmentInfo_SyncAdjustmntRqstTDD = -1;  /* CellAdjustmentInfo_SyncAdjustmentRqstTDD */
static int hf_nbap_id_CellAdjustmentInfoItem_SyncAdjustmentRqstTDD = -1;  /* CellAdjustmentInfoItem_SyncAdjustmentRqstTDD */
static int hf_nbap_id_CellSyncBurstInfoList_CellSyncReconfRqstTDD = -1;  /* CellSyncBurstInfoList_CellSyncReconfRqstTDD */
static int hf_nbap_id_CellSyncBurstTransInit_CellSyncInitiationRqstTDD = -1;  /* CellSyncBurstTransInit_CellSyncInitiationRqstTDD */
static int hf_nbap_id_CellSyncBurstMeasureInit_CellSyncInitiationRqstTDD = -1;  /* CellSyncBurstMeasureInit_CellSyncInitiationRqstTDD */
static int hf_nbap_id_CellSyncBurstMeasReconfiguration_CellSyncReconfRqstTDD = -1;  /* CellSyncBurstMeasInfo_CellSyncReconfRqstTDD */
static int hf_nbap_id_CellSyncBurstMeasInfoList_CellSyncReconfRqstTDD = -1;  /* CellSyncBurstMeasInfoList_CellSyncReconfRqstTDD */
static int hf_nbap_id_CellSyncBurstTransReconfInfo_CellSyncReconfRqstTDD = -1;  /* CellSyncBurstTransReconfInfo_CellSyncReconfRqstTDD */
static int hf_nbap_id_CellSyncInfo_CellSyncReprtTDD = -1;  /* CellSyncInfo_CellSyncReprtTDD */
static int hf_nbap_id_CSBTransmissionID = -1;     /* CSBTransmissionID */
static int hf_nbap_id_CSBMeasurementID = -1;      /* CSBMeasurementID */
static int hf_nbap_id_NCyclesPerSFNperiod = -1;   /* NCyclesPerSFNperiod */
static int hf_nbap_id_NRepetitionsPerCyclePeriod = -1;  /* NRepetitionsPerCyclePeriod */
static int hf_nbap_id_SyncFrameNumber = -1;       /* SyncFrameNumber */
static int hf_nbap_id_SynchronisationReportType = -1;  /* SynchronisationReportType */
static int hf_nbap_id_SynchronisationReportCharacteristics = -1;  /* SynchronisationReportCharacteristics */
static int hf_nbap_id_Unsuccessful_cell_InformationRespItem_SyncAdjustmntFailureTDD = -1;  /* Unsuccessful_cell_InformationRespItem_SyncAdjustmntFailureTDD */
static int hf_nbap_id_ReferenceClockAvailability = -1;  /* ReferenceClockAvailability */
static int hf_nbap_id_ReferenceSFNoffset = -1;    /* ReferenceSFNoffset */
static int hf_nbap_id_InformationExchangeID = -1;  /* InformationExchangeID */
static int hf_nbap_id_InformationExchangeObjectType_InfEx_Rqst = -1;  /* InformationExchangeObjectType_InfEx_Rqst */
static int hf_nbap_id_InformationType = -1;       /* InformationReportCharacteristics */
static int hf_nbap_id_InformationReportCharacteristics = -1;  /* InformationReportCharacteristics */
static int hf_nbap_id_InformationExchangeObjectType_InfEx_Rsp = -1;  /* InformationExchangeObjectType_InfEx_Rsp */
static int hf_nbap_id_InformationExchangeObjectType_InfEx_Rprt = -1;  /* InformationExchangeObjectType_InfEx_Rprt */
static int hf_nbap_id_IPDLParameter_Information_Cell_ReconfRqstFDD = -1;  /* IPDLParameter_Information_Cell_ReconfRqstFDD */
static int hf_nbap_id_IPDLParameter_Information_Cell_SetupRqstFDD = -1;  /* IPDLParameter_Information_Cell_SetupRqstFDD */
static int hf_nbap_id_IPDLParameter_Information_Cell_ReconfRqstTDD = -1;  /* IPDLParameter_Information_Cell_SetupRqstTDD */
static int hf_nbap_id_IPDLParameter_Information_Cell_SetupRqstTDD = -1;  /* IPDLParameter_Information_Cell_SetupRqstTDD */
static int hf_nbap_id_DL_DPCH_LCR_Information_RL_SetupRqstTDD = -1;  /* DL_DPCH_LCR_Information_RL_SetupRqstTDD */
static int hf_nbap_id_DwPCH_LCR_Information = -1;  /* Common_PhysicalChannel_Status_Information */
static int hf_nbap_id_DwPCH_LCR_InformationList_AuditRsp = -1;  /* Common_PhysicalChannel_Status_Information */
static int hf_nbap_id_DwPCH_LCR_Information_Cell_SetupRqstTDD = -1;  /* DwPCH_LCR_Information_Cell_SetupRqstTDD */
static int hf_nbap_id_DwPCH_LCR_Information_Cell_ReconfRqstTDD = -1;  /* DwPCH_LCR_Information_Cell_ReconfRqstTDD */
static int hf_nbap_id_DwPCH_LCR_Information_ResourceStatusInd = -1;  /* DwPCH_LCR_Information_ResourceStatusInd */
static int hf_nbap_id_maxFACH_Power_LCR_CTCH_SetupRqstTDD = -1;  /* DL_Power */
static int hf_nbap_id_maxFACH_Power_LCR_CTCH_ReconfRqstTDD = -1;  /* DL_Power */
static int hf_nbap_id_FPACH_LCR_Information = -1;  /* Common_PhysicalChannel_Status_Information */
static int hf_nbap_id_FPACH_LCR_Information_AuditRsp = -1;  /* Common_PhysicalChannel_Status_Information */
static int hf_nbap_id_FPACH_LCR_InformationList_AuditRsp = -1;  /* FPACH_LCR_InformationList_AuditRsp */
static int hf_nbap_id_FPACH_LCR_InformationList_ResourceStatusInd = -1;  /* FPACH_LCR_InformationList_ResourceStatusInd */
static int hf_nbap_id_FPACH_LCR_Parameters_CTCH_SetupRqstTDD = -1;  /* FPACH_LCR_Parameters_CTCH_SetupRqstTDD */
static int hf_nbap_id_FPACH_LCR_Parameters_CTCH_ReconfRqstTDD = -1;  /* FPACH_LCR_Parameters_CTCH_ReconfRqstTDD */
static int hf_nbap_id_PCCPCH_LCR_Information_Cell_SetupRqstTDD = -1;  /* PCCPCH_LCR_Information_Cell_SetupRqstTDD */
static int hf_nbap_id_PCH_Power_LCR_CTCH_SetupRqstTDD = -1;  /* DL_Power */
static int hf_nbap_id_PCH_Power_LCR_CTCH_ReconfRqstTDD = -1;  /* DL_Power */
static int hf_nbap_id_PICH_LCR_Parameters_CTCH_SetupRqstTDD = -1;  /* PICH_LCR_Parameters_CTCH_SetupRqstTDD */
static int hf_nbap_id_PRACH_LCR_ParametersList_CTCH_SetupRqstTDD = -1;  /* PRACH_LCR_ParametersList_CTCH_SetupRqstTDD */
static int hf_nbap_id_RL_InformationResponse_LCR_RL_SetupRspTDD = -1;  /* RL_InformationResponse_LCR_RL_SetupRspTDD */
static int hf_nbap_id_Secondary_CCPCH_LCR_parameterList_CTCH_SetupRqstTDD = -1;  /* Secondary_CCPCH_LCR_parameterList_CTCH_SetupRqstTDD */
static int hf_nbap_id_TimeSlot = -1;              /* TimeSlot */
static int hf_nbap_id_TimeSlotConfigurationList_LCR_Cell_ReconfRqstTDD = -1;  /* TimeSlotConfigurationList_LCR_Cell_ReconfRqstTDD */
static int hf_nbap_id_TimeSlotConfigurationList_LCR_Cell_SetupRqstTDD = -1;  /* TimeSlotConfigurationList_LCR_Cell_SetupRqstTDD */
static int hf_nbap_id_TimeslotISCP_LCR_InfoList_RL_SetupRqstTDD = -1;  /* DL_TimeslotISCPInfoLCR */
static int hf_nbap_id_TimeSlotLCR_CM_Rqst = -1;   /* TimeSlotLCR */
static int hf_nbap_id_UL_DPCH_LCR_Information_RL_SetupRqstTDD = -1;  /* UL_DPCH_LCR_Information_RL_SetupRqstTDD */
static int hf_nbap_id_DL_DPCH_InformationItem_LCR_RL_AdditionRqstTDD = -1;  /* DL_DPCH_InformationItem_LCR_RL_AdditionRqstTDD */
static int hf_nbap_id_UL_DPCH_InformationItem_LCR_RL_AdditionRqstTDD = -1;  /* UL_DPCH_InformationItem_LCR_RL_AdditionRqstTDD */
static int hf_nbap_id_TimeslotISCP_InformationList_LCR_RL_AdditionRqstTDD = -1;  /* DL_TimeslotISCPInfoLCR */
static int hf_nbap_id_DL_DPCH_LCR_InformationAddList_RL_ReconfPrepTDD = -1;  /* DL_DPCH_LCR_InformationAddList_RL_ReconfPrepTDD */
static int hf_nbap_id_DL_DPCH_LCR_InformationModify_AddList_RL_ReconfPrepTDD = -1;  /* DL_DPCH_LCR_InformationModify_AddList_RL_ReconfPrepTDD */
static int hf_nbap_id_DL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD = -1;  /* DL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD */
static int hf_nbap_id_TimeslotISCPInfoList_LCR_DL_PC_RqstTDD = -1;  /* DL_TimeslotISCPInfoLCR */
static int hf_nbap_id_UL_DPCH_LCR_InformationAddListIE_RL_ReconfPrepTDD = -1;  /* UL_DPCH_LCR_InformationAddList_RL_ReconfPrepTDD */
static int hf_nbap_id_UL_DPCH_LCR_InformationModify_AddList = -1;  /* UL_DPCH_LCR_InformationModify_AddList_RL_ReconfPrepTDD */
static int hf_nbap_id_UL_TimeslotLCR_Information_RL_ReconfPrepTDD = -1;  /* UL_TimeslotLCR_InformationModify_ModifyList_RL_ReconfPrepTDD */
static int hf_nbap_id_UL_SIRTarget = -1;          /* UL_SIR */
static int hf_nbap_id_PDSCH_AddInformation_LCR_PSCH_ReconfRqst = -1;  /* PDSCH_AddInformation_LCR_AddItem_PSCH_ReconfRqst */
static int hf_nbap_id_PDSCH_ModifyInformation_LCR_PSCH_ReconfRqst = -1;  /* PDSCH_ModifyInformation_LCR_ModifyItem_PSCH_ReconfRqst */
static int hf_nbap_id_PUSCH_AddInformation_LCR_PSCH_ReconfRqst = -1;  /* PUSCH_AddInformation_LCR_AddItem_PSCH_ReconfRqst */
static int hf_nbap_id_PUSCH_ModifyInformation_LCR_PSCH_ReconfRqst = -1;  /* PUSCH_ModifyInformation_LCR_ModifyItem_PSCH_ReconfRqst */
static int hf_nbap_id_timeslotInfo_CellSyncInitiationRqstTDD = -1;  /* TimeslotInfo_CellSyncInitiationRqstTDD */
static int hf_nbap_id_SyncReportType_CellSyncReprtTDD = -1;  /* SyncReportType_CellSyncReprtTDD */
static int hf_nbap_id_Power_Local_Cell_Group_InformationItem_AuditRsp = -1;  /* Power_Local_Cell_Group_InformationItem_AuditRsp */
static int hf_nbap_id_Power_Local_Cell_Group_InformationItem_ResourceStatusInd = -1;  /* Power_Local_Cell_Group_InformationItem_ResourceStatusInd */
static int hf_nbap_id_Power_Local_Cell_Group_InformationItem2_ResourceStatusInd = -1;  /* Power_Local_Cell_Group_InformationItem2_ResourceStatusInd */
static int hf_nbap_id_Power_Local_Cell_Group_InformationList_AuditRsp = -1;  /* Power_Local_Cell_Group_InformationList_AuditRsp */
static int hf_nbap_id_Power_Local_Cell_Group_InformationList_ResourceStatusInd = -1;  /* Power_Local_Cell_Group_InformationList_ResourceStatusInd */
static int hf_nbap_id_Power_Local_Cell_Group_InformationList2_ResourceStatusInd = -1;  /* Power_Local_Cell_Group_InformationList2_ResourceStatusInd */
static int hf_nbap_id_Power_Local_Cell_Group_ID = -1;  /* Local_Cell_ID */
static int hf_nbap_id_PUSCH_Info_DM_Rqst = -1;    /* PUSCH_Info_DM_Rqst */
static int hf_nbap_id_PUSCH_Info_DM_Rsp = -1;     /* PUSCH_Info_DM_Rsp */
static int hf_nbap_id_PUSCH_Info_DM_Rprt = -1;    /* PUSCH_Info_DM_Rprt */
static int hf_nbap_id_InitDL_Power = -1;          /* DL_Power */
static int hf_nbap_id_cellSyncBurstRepetitionPeriod = -1;  /* CellSyncBurstRepetitionPeriod */
static int hf_nbap_id_ReportCharacteristicsType_OnModification = -1;  /* ReportCharacteristicsType_OnModification */
static int hf_nbap_id_SFNSFNMeasurementValueInformation = -1;  /* SFNSFNMeasurementValueInformation */
static int hf_nbap_id_SFNSFNMeasurementThresholdInformation = -1;  /* SFNSFNMeasurementThresholdInformation */
static int hf_nbap_id_TUTRANGPSMeasurementValueInformation = -1;  /* TUTRANGPSMeasurementValueInformation */
static int hf_nbap_id_TUTRANGPSMeasurementThresholdInformation = -1;  /* TUTRANGPSMeasurementThresholdInformation */
static int hf_nbap_id_Rx_Timing_Deviation_Value_LCR = -1;  /* Rx_Timing_Deviation_Value_LCR */
static int hf_nbap_id_RL_InformationResponse_LCR_RL_AdditionRspTDD = -1;  /* RL_InformationResponse_LCR_RL_AdditionRspTDD */
static int hf_nbap_id_DL_PowerBalancing_Information = -1;  /* DL_PowerBalancing_Information */
static int hf_nbap_id_DL_PowerBalancing_ActivationIndicator = -1;  /* DL_PowerBalancing_ActivationIndicator */
static int hf_nbap_id_DL_PowerBalancing_UpdatedIndicator = -1;  /* DL_PowerBalancing_UpdatedIndicator */
static int hf_nbap_id_CCTrCH_Initial_DL_Power_RL_SetupRqstTDD = -1;  /* DL_Power */
static int hf_nbap_id_CCTrCH_Initial_DL_Power_RL_AdditionRqstTDD = -1;  /* DL_Power */
static int hf_nbap_id_CCTrCH_Initial_DL_Power_RL_ReconfPrepTDD = -1;  /* DL_Power */
static int hf_nbap_id_IPDLParameter_Information_LCR_Cell_SetupRqstTDD = -1;  /* IPDLParameter_Information_LCR_Cell_SetupRqstTDD */
static int hf_nbap_id_IPDLParameter_Information_LCR_Cell_ReconfRqstTDD = -1;  /* IPDLParameter_Information_LCR_Cell_ReconfRqstTDD */
static int hf_nbap_id_HS_PDSCH_HS_SCCH_E_AGCH_E_RGCH_E_HICH_MaxPower_PSCH_ReconfRqst = -1;  /* MaximumTransmissionPower */
static int hf_nbap_id_HS_PDSCH_HS_SCCH_ScramblingCode_PSCH_ReconfRqst = -1;  /* DL_ScramblingCode */
static int hf_nbap_id_HS_PDSCH_FDD_Code_Information_PSCH_ReconfRqst = -1;  /* HS_PDSCH_FDD_Code_Information */
static int hf_nbap_id_HS_SCCH_FDD_Code_Information_PSCH_ReconfRqst = -1;  /* HS_SCCH_FDD_Code_Information */
static int hf_nbap_id_HS_PDSCH_TDD_Information_PSCH_ReconfRqst = -1;  /* HS_PDSCH_TDD_Information_PSCH_ReconfRqst */
static int hf_nbap_id_Add_To_HS_SCCH_Resource_Pool_PSCH_ReconfRqst = -1;  /* Add_To_HS_SCCH_Resource_Pool_PSCH_ReconfRqst */
static int hf_nbap_id_Modify_HS_SCCH_Resource_Pool_PSCH_ReconfRqst = -1;  /* Modify_HS_SCCH_Resource_Pool_PSCH_ReconfRqst */
static int hf_nbap_id_Delete_From_HS_SCCH_Resource_Pool_PSCH_ReconfRqst = -1;  /* Delete_From_HS_SCCH_Resource_Pool_PSCH_ReconfRqst */
static int hf_nbap_id_bindingID = -1;             /* BindingID */
static int hf_nbap_id_RL_Specific_DCH_Info = -1;  /* RL_Specific_DCH_Info */
static int hf_nbap_id_transportlayeraddress = -1;  /* TransportLayerAddress */
static int hf_nbap_id_DelayedActivation = -1;     /* DelayedActivation */
static int hf_nbap_id_DelayedActivationList_RL_ActivationCmdFDD = -1;  /* DelayedActivationInformationList_RL_ActivationCmdFDD */
static int hf_nbap_id_DelayedActivationInformation_RL_ActivationCmdFDD = -1;  /* DelayedActivationInformation_RL_ActivationCmdFDD */
static int hf_nbap_id_DelayedActivationList_RL_ActivationCmdTDD = -1;  /* DelayedActivationInformationList_RL_ActivationCmdTDD */
static int hf_nbap_id_DelayedActivationInformation_RL_ActivationCmdTDD = -1;  /* DelayedActivationInformation_RL_ActivationCmdTDD */
static int hf_nbap_id_neighbouringTDDCellMeasurementInformationLCR = -1;  /* NeighbouringTDDCellMeasurementInformationLCR */
static int hf_nbap_id_SYNCDlCodeId_TransInitLCR_CellSyncInitiationRqstTDD = -1;  /* SYNCDlCodeId_TransInitLCR_CellSyncInitiationRqstTDD */
static int hf_nbap_id_SYNCDlCodeId_MeasureInitLCR_CellSyncInitiationRqstTDD = -1;  /* SYNCDlCodeId_MeasureInitLCR_CellSyncInitiationRqstTDD */
static int hf_nbap_id_SYNCDlCodeIdTransReconfInfoLCR_CellSyncReconfRqstTDD = -1;  /* SYNCDlCodeIdTransReconfInfoLCR_CellSyncReconfRqstTDD */
static int hf_nbap_id_SYNCDlCodeIdMeasReconfigurationLCR_CellSyncReconfRqstTDD = -1;  /* SYNCDlCodeIdMeasInfoLCR_CellSyncReconfRqstTDD */
static int hf_nbap_id_SYNCDlCodeIdMeasInfoList_CellSyncReconfRqstTDD = -1;  /* SYNCDlCodeIdMeasInfoList_CellSyncReconfRqstTDD */
static int hf_nbap_id_SyncDLCodeIdsMeasInfoList_CellSyncReprtTDD = -1;  /* SyncDLCodeIdsMeasInfoList_CellSyncReprtTDD */
static int hf_nbap_id_SyncDLCodeIdThreInfoLCR = -1;  /* SyncDLCodeIdThreInfoLCR */
static int hf_nbap_id_NSubCyclesPerCyclePeriod_CellSyncReconfRqstTDD = -1;  /* NSubCyclesPerCyclePeriod */
static int hf_nbap_id_DwPCH_Power = -1;           /* DwPCH_Power */
static int hf_nbap_id_AccumulatedClockupdate_CellSyncReprtTDD = -1;  /* TimingAdjustmentValue */
static int hf_nbap_id_Angle_Of_Arrival_Value_LCR = -1;  /* Angle_Of_Arrival_Value_LCR */
static int hf_nbap_id_HSDSCH_FDD_Information = -1;  /* HSDSCH_FDD_Information */
static int hf_nbap_id_HSDSCH_FDD_Information_Response = -1;  /* HSDSCH_FDD_Information_Response */
static int hf_nbap_id_HSDSCH_Information_to_Modify = -1;  /* HSDSCH_Information_to_Modify */
static int hf_nbap_id_HSDSCH_RNTI = -1;           /* HSDSCH_RNTI */
static int hf_nbap_id_HSDSCH_TDD_Information = -1;  /* HSDSCH_TDD_Information */
static int hf_nbap_id_HSDSCH_TDD_Information_Response = -1;  /* HSDSCH_TDD_Information_Response */
static int hf_nbap_id_HSPDSCH_RL_ID = -1;         /* RL_ID */
static int hf_nbap_id_PrimCCPCH_RSCP_DL_PC_RqstTDD = -1;  /* PrimaryCCPCH_RSCP */
static int hf_nbap_id_PDSCH_RL_ID = -1;           /* RL_ID */
static int hf_nbap_id_HSDSCH_RearrangeList_Bearer_RearrangeInd = -1;  /* HSDSCH_RearrangeList_Bearer_RearrangeInd */
static int hf_nbap_id_UL_Synchronisation_Parameters_LCR = -1;  /* UL_Synchronisation_Parameters_LCR */
static int hf_nbap_id_HSDSCH_FDD_Update_Information = -1;  /* HSDSCH_FDD_Update_Information */
static int hf_nbap_id_HSDSCH_TDD_Update_Information = -1;  /* HSDSCH_TDD_Update_Information */
static int hf_nbap_id_DL_DPCH_TimeSlotFormat_LCR_ModifyItem_RL_ReconfPrepTDD = -1;  /* TDD_DL_DPCH_TimeSlotFormat_LCR */
static int hf_nbap_id_UL_DPCH_TimeSlotFormat_LCR_ModifyItem_RL_ReconfPrepTDD = -1;  /* TDD_UL_DPCH_TimeSlotFormat_LCR */
static int hf_nbap_id_TDD_TPC_UplinkStepSize_LCR_RL_SetupRqstTDD = -1;  /* TDD_TPC_UplinkStepSize_LCR */
static int hf_nbap_id_TDD_TPC_UplinkStepSize_LCR_RL_AdditionRqstTDD = -1;  /* TDD_TPC_UplinkStepSize_LCR */
static int hf_nbap_id_TDD_TPC_DownlinkStepSize_RL_AdditionRqstTDD = -1;  /* TDD_TPC_DownlinkStepSize */
static int hf_nbap_id_TDD_TPC_UplinkStepSize_InformationAdd_LCR_RL_ReconfPrepTDD = -1;  /* TDD_TPC_UplinkStepSize_LCR */
static int hf_nbap_id_TDD_TPC_UplinkStepSize_InformationModify_LCR_RL_ReconfPrepTDD = -1;  /* TDD_TPC_UplinkStepSize_LCR */
static int hf_nbap_id_TDD_TPC_DownlinkStepSize_InformationModify_RL_ReconfPrepTDD = -1;  /* TDD_TPC_DownlinkStepSize */
static int hf_nbap_id_TDD_TPC_DownlinkStepSize_InformationAdd_RL_ReconfPrepTDD = -1;  /* TDD_TPC_DownlinkStepSize */
static int hf_nbap_id_CCTrCH_Maximum_DL_Power_RL_SetupRqstTDD = -1;  /* DL_Power */
static int hf_nbap_id_CCTrCH_Minimum_DL_Power_RL_SetupRqstTDD = -1;  /* DL_Power */
static int hf_nbap_id_CCTrCH_Maximum_DL_Power_RL_AdditionRqstTDD = -1;  /* DL_Power */
static int hf_nbap_id_CCTrCH_Minimum_DL_Power_RL_AdditionRqstTDD = -1;  /* DL_Power */
static int hf_nbap_id_CCTrCH_Maximum_DL_Power_InformationAdd_RL_ReconfPrepTDD = -1;  /* DL_Power */
static int hf_nbap_id_CCTrCH_Minimum_DL_Power_InformationAdd_RL_ReconfPrepTDD = -1;  /* DL_Power */
static int hf_nbap_id_CCTrCH_Maximum_DL_Power_InformationModify_RL_ReconfPrepTDD = -1;  /* DL_Power */
static int hf_nbap_id_CCTrCH_Minimum_DL_Power_InformationModify_RL_ReconfPrepTDD = -1;  /* DL_Power */
static int hf_nbap_id_Maximum_DL_Power_Modify_LCR_InformationModify_RL_ReconfPrepTDD = -1;  /* DL_Power */
static int hf_nbap_id_Minimum_DL_Power_Modify_LCR_InformationModify_RL_ReconfPrepTDD = -1;  /* DL_Power */
static int hf_nbap_id_DL_DPCH_LCR_InformationModify_ModifyList_RL_ReconfRqstTDD = -1;  /* DL_DPCH_LCR_InformationModify_ModifyList_RL_ReconfRqstTDD */
static int hf_nbap_id_CCTrCH_Maximum_DL_Power_InformationModify_RL_ReconfRqstTDD = -1;  /* DL_Power */
static int hf_nbap_id_CCTrCH_Minimum_DL_Power_InformationModify_RL_ReconfRqstTDD = -1;  /* DL_Power */
static int hf_nbap_id_Initial_DL_Power_TimeslotLCR_InformationItem = -1;  /* DL_Power */
static int hf_nbap_id_Maximum_DL_Power_TimeslotLCR_InformationItem = -1;  /* DL_Power */
static int hf_nbap_id_Minimum_DL_Power_TimeslotLCR_InformationItem = -1;  /* DL_Power */
static int hf_nbap_id_HS_DSCHProvidedBitRateValueInformation = -1;  /* HS_DSCHProvidedBitRate */
static int hf_nbap_id_HS_DSCHRequiredPowerValueInformation = -1;  /* HS_DSCHRequiredPower */
static int hf_nbap_id_HS_DSCHRequiredPowerValue = -1;  /* HS_DSCHRequiredPowerValue */
static int hf_nbap_id_TransmittedCarrierPowerOfAllCodesNotUsedForHSTransmission = -1;  /* TransmittedCarrierPowerOfAllCodesNotUsedForHSTransmissionValue */
static int hf_nbap_id_HS_SICH_Reception_Quality = -1;  /* HS_SICH_Reception_Quality_Value */
static int hf_nbap_id_HS_SICH_Reception_Quality_Measurement_Value = -1;  /* HS_SICH_Reception_Quality_Measurement_Value */
static int hf_nbap_id_HSSICH_Info_DM_Rprt = -1;   /* HS_SICH_ID */
static int hf_nbap_id_HSSICH_Info_DM_Rqst = -1;   /* HSSICH_Info_DM_Rqst */
static int hf_nbap_id_HSSICH_Info_DM_Rsp = -1;    /* HS_SICH_ID */
static int hf_nbap_id_Best_Cell_Portions_Value = -1;  /* Best_Cell_Portions_Value */
static int hf_nbap_id_Primary_CPICH_Usage_for_Channel_Estimation = -1;  /* Primary_CPICH_Usage_for_Channel_Estimation */
static int hf_nbap_id_Secondary_CPICH_Information_Change = -1;  /* Secondary_CPICH_Information_Change */
static int hf_nbap_id_NumberOfReportedCellPortions = -1;  /* NumberOfReportedCellPortions */
static int hf_nbap_id_CellPortion_InformationItem_Cell_SetupRqstFDD = -1;  /* CellPortion_InformationItem_Cell_SetupRqstFDD */
static int hf_nbap_id_CellPortion_InformationList_Cell_SetupRqstFDD = -1;  /* CellPortion_InformationList_Cell_SetupRqstFDD */
static int hf_nbap_id_TimeslotISCP_LCR_InfoList_RL_ReconfPrepTDD = -1;  /* DL_TimeslotISCPInfoLCR */
static int hf_nbap_id_Secondary_CPICH_Information = -1;  /* CommonPhysicalChannelID */
static int hf_nbap_id_Received_total_wide_band_power_For_CellPortion = -1;  /* Received_total_wide_band_power_Value_IncrDecrThres */
static int hf_nbap_id_Unidirectional_DCH_Indicator = -1;  /* Unidirectional_DCH_Indicator */
static int hf_nbap_id_TimingAdjustmentValueLCR = -1;  /* TimingAdjustmentValueLCR */
static int hf_nbap_id_multipleRL_ul_DPCH_InformationList = -1;  /* MultipleRL_UL_DPCH_InformationAddList_RL_ReconfPrepTDD */
static int hf_nbap_id_multipleRL_ul_DPCH_InformationModifyList = -1;  /* MultipleRL_UL_DPCH_InformationModifyList_RL_ReconfPrepTDD */
static int hf_nbap_id_RL_ID = -1;                 /* RL_ID */
static int hf_nbap_id_SAT_Info_Almanac_ExtItem = -1;  /* SAT_Info_Almanac_ExtItem */
static int hf_nbap_id_HSDPA_Capability = -1;      /* HSDPA_Capability */
static int hf_nbap_id_HSDSCH_Resources_Information_AuditRsp = -1;  /* HS_DSCH_Resources_Information_AuditRsp */
static int hf_nbap_id_HSDSCH_Resources_Information_ResourceStatusInd = -1;  /* HS_DSCH_Resources_Information_ResourceStatusInd */
static int hf_nbap_id_HSDSCH_MACdFlows_to_Add = -1;  /* HSDSCH_MACdFlows_Information */
static int hf_nbap_id_HSDSCH_MACdFlows_to_Delete = -1;  /* HSDSCH_MACdFlows_to_Delete */
static int hf_nbap_id_HSDSCH_Information_to_Modify_Unsynchronised = -1;  /* HSDSCH_Information_to_Modify_Unsynchronised */
static int hf_nbap_id_TnlQos = -1;                /* TnlQos */
static int hf_nbap_id_Received_total_wide_band_power_For_CellPortion_Value = -1;  /* Received_total_wide_band_power_For_CellPortion_Value */
static int hf_nbap_id_Transmitted_Carrier_Power_For_CellPortion = -1;  /* Transmitted_Carrier_Power_Value */
static int hf_nbap_id_Transmitted_Carrier_Power_For_CellPortion_Value = -1;  /* Transmitted_Carrier_Power_For_CellPortion_Value */
static int hf_nbap_id_TransmittedCarrierPowerOfAllCodesNotUsedForHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHOrE_HICHTransmissionCellPortion = -1;  /* TransmittedCarrierPowerOfAllCodesNotUsedForHSTransmissionValue */
static int hf_nbap_id_TransmittedCarrierPowerOfAllCodesNotUsedForHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHOrE_HICHTransmissionCellPortionValue = -1;  /* TransmittedCarrierPowerOfAllCodesNotUsedForHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHOrE_HICHTransmissionCellPortionValue */
static int hf_nbap_id_UpPTSInterferenceValue = -1;  /* UpPTSInterferenceValue */
static int hf_nbap_id_PrimaryCCPCH_RSCP_Delta = -1;  /* PrimaryCCPCH_RSCP_Delta */
static int hf_nbap_id_MeasurementRecoveryBehavior = -1;  /* MeasurementRecoveryBehavior */
static int hf_nbap_id_MeasurementRecoveryReportingIndicator = -1;  /* MeasurementRecoveryReportingIndicator */
static int hf_nbap_id_MeasurementRecoverySupportIndicator = -1;  /* MeasurementRecoverySupportIndicator */
static int hf_nbap_id_Tstd_indicator = -1;        /* TSTD_Indicator */
static int hf_nbap_id_multiple_RL_Information_RL_ReconfPrepTDD = -1;  /* MultipleRL_Information_RL_ReconfPrepTDD */
static int hf_nbap_id_multiple_RL_Information_RL_ReconfRqstTDD = -1;  /* Multiple_RL_Information_RL_ReconfRqstTDD */
static int hf_nbap_id_DL_DPCH_Power_Information_RL_ReconfPrepFDD = -1;  /* DL_DPCH_Power_Information_RL_ReconfPrepFDD */
static int hf_nbap_id_F_DPCH_Information_RL_ReconfPrepFDD = -1;  /* F_DPCH_Information_RL_ReconfPrepFDD */
static int hf_nbap_id_F_DPCH_Information_RL_SetupRqstFDD = -1;  /* F_DPCH_Information_RL_SetupRqstFDD */
static int hf_nbap_id_Additional_S_CCPCH_Parameters_CTCH_ReconfRqstTDD = -1;  /* Secondary_CCPCH_parameterExtendedList_CTCH_ReconfRqstTDD */
static int hf_nbap_id_Additional_S_CCPCH_Parameters_CTCH_SetupRqstTDD = -1;  /* Secondary_CCPCH_parameterExtendedList_CTCH_SetupRqstTDD */
static int hf_nbap_id_Additional_S_CCPCH_LCR_Parameters_CTCH_ReconfRqstTDD = -1;  /* Secondary_CCPCH_LCR_parameterExtendedList_CTCH_ReconfRqstTDD */
static int hf_nbap_id_Additional_S_CCPCH_LCR_Parameters_CTCH_SetupRqstTDD = -1;  /* Secondary_CCPCH_LCR_parameterExtendedList_CTCH_SetupRqstTDD */
static int hf_nbap_id_MICH_CFN = -1;              /* MICH_CFN */
static int hf_nbap_id_MICH_Information_AuditRsp = -1;  /* Common_PhysicalChannel_Status_Information */
static int hf_nbap_id_MICH_Information_ResourceStatusInd = -1;  /* Common_PhysicalChannel_Status_Information */
static int hf_nbap_id_MICH_Parameters_CTCH_ReconfRqstFDD = -1;  /* MICH_Parameters_CTCH_ReconfRqstFDD */
static int hf_nbap_id_MICH_Parameters_CTCH_ReconfRqstTDD = -1;  /* MICH_Parameters_CTCH_ReconfRqstTDD */
static int hf_nbap_id_MICH_Parameters_CTCH_SetupRqstFDD = -1;  /* MICH_Parameters_CTCH_SetupRqstFDD */
static int hf_nbap_id_MICH_Parameters_CTCH_SetupRqstTDD = -1;  /* MICH_Parameters_CTCH_SetupRqstTDD */
static int hf_nbap_id_Modification_Period = -1;   /* Modification_Period */
static int hf_nbap_id_NI_Information_NotifUpdateCmd = -1;  /* NI_Information */
static int hf_nbap_id_S_CCPCH_InformationListExt_AuditRsp = -1;  /* S_CCPCH_InformationListExt_AuditRsp */
static int hf_nbap_id_S_CCPCH_InformationListExt_ResourceStatusInd = -1;  /* S_CCPCH_InformationListExt_ResourceStatusInd */
static int hf_nbap_id_S_CCPCH_LCR_InformationListExt_AuditRsp = -1;  /* S_CCPCH_LCR_InformationListExt_AuditRsp */
static int hf_nbap_id_S_CCPCH_LCR_InformationListExt_ResourceStatusInd = -1;  /* S_CCPCH_LCR_InformationListExt_ResourceStatusInd */
static int hf_nbap_id_HARQ_Preamble_Mode = -1;    /* HARQ_Preamble_Mode */
static int hf_nbap_id_Initial_DL_DPCH_TimingAdjustment = -1;  /* DL_DPCH_TimingAdjustment */
static int hf_nbap_id_Initial_DL_DPCH_TimingAdjustment_Allowed = -1;  /* Initial_DL_DPCH_TimingAdjustment_Allowed */
static int hf_nbap_id_DLTransmissionBranchLoadValue = -1;  /* DLTransmissionBranchLoadValue */
static int hf_nbap_id_Power_Local_Cell_Group_choice_CM_Rqst = -1;  /* PowerLocalCellGroup_CM_Rqst */
static int hf_nbap_id_Power_Local_Cell_Group_choice_CM_Rsp = -1;  /* PowerLocalCellGroup_CM_Rsp */
static int hf_nbap_id_Power_Local_Cell_Group_choice_CM_Rprt = -1;  /* PowerLocalCellGroup_CM_Rprt */
static int hf_nbap_id_SynchronisationIndicator = -1;  /* SynchronisationIndicator */
static int hf_nbap_id_HSDPA_And_EDCH_CellPortion_Information_PSCH_ReconfRqst = -1;  /* HSDPA_And_EDCH_CellPortion_Information_PSCH_ReconfRqst */
static int hf_nbap_id_HSDPA_And_EDCH_CellPortion_InformationList_PSCH_ReconfRqst = -1;  /* HSDPA_And_EDCH_CellPortion_InformationList_PSCH_ReconfRqst */
static int hf_nbap_id_HS_DSCHRequiredPowerValue_For_Cell_Portion = -1;  /* HS_DSCHRequiredPowerValue */
static int hf_nbap_id_HS_DSCHRequiredPowerValueInformation_For_CellPortion = -1;  /* HS_DSCHRequiredPowerValueInformation_For_CellPortion */
static int hf_nbap_id_HS_DSCHProvidedBitRateValueInformation_For_CellPortion = -1;  /* HS_DSCHProvidedBitRateValueInformation_For_CellPortion */
static int hf_nbap_id_E_AGCH_And_E_RGCH_E_HICH_FDD_Scrambling_Code = -1;  /* DL_ScramblingCode */
static int hf_nbap_id_E_AGCH_FDD_Code_Information = -1;  /* E_AGCH_FDD_Code_Information */
static int hf_nbap_id_E_DCH_Capability = -1;      /* E_DCH_Capability */
static int hf_nbap_id_E_DCH_FDD_DL_Control_Channel_Information = -1;  /* E_DCH_FDD_DL_Control_Channel_Information */
static int hf_nbap_id_E_DCH_FDD_Information = -1;  /* E_DCH_FDD_Information */
static int hf_nbap_id_E_DCH_FDD_Information_Response = -1;  /* E_DCH_FDD_Information_Response */
static int hf_nbap_id_E_DCH_FDD_Information_to_Modify = -1;  /* E_DCH_FDD_Information_to_Modify */
static int hf_nbap_id_E_DCH_MACdFlows_to_Add = -1;  /* E_DCH_MACdFlows_Information */
static int hf_nbap_id_E_DCH_MACdFlows_to_Delete = -1;  /* E_DCH_MACdFlows_to_Delete */
static int hf_nbap_id_E_DCH_Resources_Information_AuditRsp = -1;  /* E_DCH_Resources_Information_AuditRsp */
static int hf_nbap_id_E_DCH_Resources_Information_ResourceStatusInd = -1;  /* E_DCH_Resources_Information_ResourceStatusInd */
static int hf_nbap_id_E_DCH_RL_Indication = -1;   /* E_DCH_RL_Indication */
static int hf_nbap_id_E_DCH_RL_Set_ID = -1;       /* RL_Set_ID */
static int hf_nbap_id_E_DPCH_Information_RL_ReconfPrepFDD = -1;  /* E_DPCH_Information_RL_ReconfPrepFDD */
static int hf_nbap_id_E_DPCH_Information_RL_SetupRqstFDD = -1;  /* E_DPCH_Information_RL_SetupRqstFDD */
static int hf_nbap_id_E_RGCH_E_HICH_FDD_Code_Information = -1;  /* E_RGCH_E_HICH_FDD_Code_Information */
static int hf_nbap_id_Serving_E_DCH_RL_ID = -1;   /* Serving_E_DCH_RL_ID */
static int hf_nbap_id_UL_DPDCH_Indicator_For_E_DCH_Operation = -1;  /* UL_DPDCH_Indicator_For_E_DCH_Operation */
static int hf_nbap_id_FDD_S_CCPCH_FrameOffset_CTCH_SetupRqstFDD = -1;  /* FDD_S_CCPCH_FrameOffset */
static int hf_nbap_id_E_DPCH_Information_RL_ReconfRqstFDD = -1;  /* E_DPCH_Information_RL_ReconfRqstFDD */
static int hf_nbap_id_Maximum_Target_ReceivedTotalWideBandPower = -1;  /* Maximum_Target_ReceivedTotalWideBandPower */
static int hf_nbap_id_E_DCHProvidedBitRateValueInformation = -1;  /* E_DCHProvidedBitRate */
static int hf_nbap_id_HARQ_Preamble_Mode_Activation_Indicator = -1;  /* HARQ_Preamble_Mode_Activation_Indicator */
static int hf_nbap_id_RL_Specific_E_DCH_Info = -1;  /* RL_Specific_E_DCH_Info */
static int hf_nbap_id_E_DCH_CapacityConsumptionLaw = -1;  /* DedicatedChannelsCapacityConsumptionLaw */
static int hf_nbap_id_multiple_DedicatedMeasurementValueList_TDD_DM_Rsp = -1;  /* Multiple_DedicatedMeasurementValueList_TDD_DM_Rsp */
static int hf_nbap_id_multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp = -1;  /* Multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp */
static int hf_nbap_id_E_DCH_RearrangeList_Bearer_RearrangeInd = -1;  /* E_DCH_RearrangeList_Bearer_RearrangeInd */
static int hf_nbap_id_multipleRL_dl_CCTrCH_InformationModifyList_RL_ReconfRqstTDD = -1;  /* MultipleRL_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD */
static int hf_nbap_id_Target_NonServing_EDCH_To_Total_EDCH_Power_Ratio = -1;  /* Target_NonServing_EDCH_To_Total_EDCH_Power_Ratio */
static int hf_nbap_id_CellPortion_InformationItem_Cell_ReconfRqstFDD = -1;  /* CellPortion_InformationItem_Cell_ReconfRqstFDD */
static int hf_nbap_id_CellPortion_InformationList_Cell_ReconfRqstFDD = -1;  /* CellPortion_InformationList_Cell_ReconfRqstFDD */
static int hf_nbap_id_multiple_PUSCH_InfoList_DM_Rsp = -1;  /* Multiple_PUSCH_InfoList_DM_Rsp */
static int hf_nbap_id_multiple_PUSCH_InfoList_DM_Rprt = -1;  /* Multiple_PUSCH_InfoList_DM_Rprt */
static int hf_nbap_id_Reference_ReceivedTotalWideBandPower = -1;  /* Reference_ReceivedTotalWideBandPower */
static int hf_nbap_protocolIEs = -1;              /* ProtocolIE_Container */
static int hf_nbap_protocolExtensions = -1;       /* ProtocolExtensionContainer */
static int hf_nbap_secondary_CCPCH_parameters = -1;  /* Secondary_CCPCH_CTCH_SetupRqstFDD */
static int hf_nbap_pRACH_parameters = -1;         /* PRACH_CTCH_SetupRqstFDD */
static int hf_nbap_notUsed_pCPCHes_parameters = -1;  /* NULL */
static int hf_nbap_commonPhysicalChannelID = -1;  /* CommonPhysicalChannelID */
static int hf_nbap_fdd_S_CCPCH_Offset = -1;       /* FDD_S_CCPCH_Offset */
static int hf_nbap_dl_ScramblingCode = -1;        /* DL_ScramblingCode */
static int hf_nbap_fdd_DL_ChannelisationCodeNumber = -1;  /* FDD_DL_ChannelisationCodeNumber */
static int hf_nbap_tFCS = -1;                     /* TFCS */
static int hf_nbap_secondary_CCPCH_SlotFormat = -1;  /* SecondaryCCPCH_SlotFormat */
static int hf_nbap_tFCI_Presence = -1;            /* TFCI_Presence */
static int hf_nbap_multiplexingPosition = -1;     /* MultiplexingPosition */
static int hf_nbap_powerOffsetInformation = -1;   /* PowerOffsetInformation_CTCH_SetupRqstFDD */
static int hf_nbap_sTTD_Indicator = -1;           /* STTD_Indicator */
static int hf_nbap_fACH_Parameters = -1;          /* FACH_ParametersList_CTCH_SetupRqstFDD */
static int hf_nbap_pCH_Parameters = -1;           /* PCH_Parameters_CTCH_SetupRqstFDD */
static int hf_nbap_iE_Extensions = -1;            /* ProtocolExtensionContainer */
static int hf_nbap_pO1_ForTFCI_Bits = -1;         /* PowerOffset */
static int hf_nbap_pO3_ForPilotBits = -1;         /* PowerOffset */
static int hf_nbap_FACH_ParametersListIE_CTCH_SetupRqstFDD_item = -1;  /* FACH_ParametersItem_CTCH_SetupRqstFDD */
static int hf_nbap_commonTransportChannelID = -1;  /* CommonTransportChannelID */
static int hf_nbap_transportFormatSet = -1;       /* TransportFormatSet */
static int hf_nbap_toAWS = -1;                    /* ToAWS */
static int hf_nbap_toAWE = -1;                    /* ToAWE */
static int hf_nbap_maxFACH_Power = -1;            /* DL_Power */
static int hf_nbap_pCH_Power = -1;                /* DL_Power */
static int hf_nbap_pICH_Parameters = -1;          /* PICH_Parameters_CTCH_SetupRqstFDD */
static int hf_nbap_fdd_dl_ChannelisationCodeNumber = -1;  /* FDD_DL_ChannelisationCodeNumber */
static int hf_nbap_pICH_Power = -1;               /* PICH_Power */
static int hf_nbap_pICH_Mode = -1;                /* PICH_Mode */
static int hf_nbap_mICH_Power = -1;               /* PICH_Power */
static int hf_nbap_mICH_Mode = -1;                /* MICH_Mode */
static int hf_nbap_scramblingCodeNumber = -1;     /* ScramblingCodeNumber */
static int hf_nbap_preambleSignatures = -1;       /* PreambleSignatures */
static int hf_nbap_allowedSlotFormatInformation = -1;  /* AllowedSlotFormatInformationList_CTCH_SetupRqstFDD */
static int hf_nbap_rACH_SubChannelNumbers = -1;   /* RACH_SubChannelNumbers */
static int hf_nbap_ul_punctureLimit = -1;         /* PunctureLimit */
static int hf_nbap_preambleThreshold = -1;        /* PreambleThreshold */
static int hf_nbap_rACH_Parameters = -1;          /* RACH_Parameters_CTCH_SetupRqstFDD */
static int hf_nbap_aICH_Parameters = -1;          /* AICH_Parameters_CTCH_SetupRqstFDD */
static int hf_nbap_AllowedSlotFormatInformationList_CTCH_SetupRqstFDD_item = -1;  /* AllowedSlotFormatInformationItem_CTCH_SetupRqstFDD */
static int hf_nbap_rACHSlotFormat = -1;           /* RACH_SlotFormat */
static int hf_nbap_aICH_TransmissionTiming = -1;  /* AICH_TransmissionTiming */
static int hf_nbap_aICH_Power = -1;               /* AICH_Power */
static int hf_nbap_secondary_CCPCH_parameters1 = -1;  /* Secondary_CCPCH_CTCH_SetupRqstTDD */
static int hf_nbap_pRACH_parameters1 = -1;        /* PRACH_CTCH_SetupRqstTDD */
static int hf_nbap_sCCPCH_CCTrCH_ID = -1;         /* CCTrCH_ID */
static int hf_nbap_tFCI_Coding = -1;              /* TFCI_Coding */
static int hf_nbap_punctureLimit = -1;            /* PunctureLimit */
static int hf_nbap_secondaryCCPCH_parameterList = -1;  /* Secondary_CCPCH_parameterList_CTCH_SetupRqstTDD */
static int hf_nbap_fACH_ParametersList = -1;      /* FACH_ParametersList_CTCH_SetupRqstTDD */
static int hf_nbap_pCH_Parameters1 = -1;          /* PCH_Parameters_CTCH_SetupRqstTDD */
static int hf_nbap_Secondary_CCPCH_parameterListIE_CTCH_SetupRqstTDD_item = -1;  /* Secondary_CCPCH_parameterItem_CTCH_SetupRqstTDD */
static int hf_nbap_tdd_ChannelisationCode = -1;   /* TDD_ChannelisationCode */
static int hf_nbap_timeslot = -1;                 /* TimeSlot */
static int hf_nbap_midambleShiftandBurstType = -1;  /* MidambleShiftAndBurstType */
static int hf_nbap_tdd_PhysicalChannelOffset = -1;  /* TDD_PhysicalChannelOffset */
static int hf_nbap_repetitionPeriod = -1;         /* RepetitionPeriod */
static int hf_nbap_repetitionLength = -1;         /* RepetitionLength */
static int hf_nbap_s_CCPCH_Power = -1;            /* DL_Power */
static int hf_nbap_FACH_ParametersListIE_CTCH_SetupRqstTDD_item = -1;  /* FACH_ParametersItem_CTCH_SetupRqstTDD */
static int hf_nbap_fACH_CCTrCH_ID = -1;           /* CCTrCH_ID */
static int hf_nbap_dl_TransportFormatSet = -1;    /* TransportFormatSet */
static int hf_nbap_pCH_CCTrCH_ID = -1;            /* CCTrCH_ID */
static int hf_nbap_pICH_Parameters1 = -1;         /* PICH_Parameters_CTCH_SetupRqstTDD */
static int hf_nbap_timeSlot = -1;                 /* TimeSlot */
static int hf_nbap_midambleshiftAndBurstType = -1;  /* MidambleShiftAndBurstType */
static int hf_nbap_pagingIndicatorLength = -1;    /* PagingIndicatorLength */
static int hf_nbap_tdd_ChannelisationCodeLCR = -1;  /* TDD_ChannelisationCodeLCR */
static int hf_nbap_timeSlotLCR = -1;              /* TimeSlotLCR */
static int hf_nbap_midambleShiftLCR = -1;         /* MidambleShiftLCR */
static int hf_nbap_second_TDD_ChannelisationCodeLCR = -1;  /* TDD_ChannelisationCodeLCR */
static int hf_nbap_Secondary_CCPCH_LCR_parameterList_CTCH_SetupRqstTDD_item = -1;  /* Secondary_CCPCH_LCR_parameterItem_CTCH_SetupRqstTDD */
static int hf_nbap_timeslotLCR = -1;              /* TimeSlotLCR */
static int hf_nbap_s_CCPCH_TimeSlotFormat_LCR = -1;  /* TDD_DL_DPCH_TimeSlotFormat_LCR */
static int hf_nbap_notificationIndicatorLength = -1;  /* NotificationIndicatorLength */
static int hf_nbap_mICH_TDDOption_Specific_Parameters = -1;  /* MICH_TDDOption_Specific_Parameters_CTCH_SetupRqstTDD */
static int hf_nbap_hCR_TDD = -1;                  /* MICH_HCR_Parameters_CTCH_SetupRqstTDD */
static int hf_nbap_lCR_TDD = -1;                  /* MICH_LCR_Parameters_CTCH_SetupRqstTDD */
static int hf_nbap_tSTD_Indicator = -1;           /* TSTD_Indicator */
static int hf_nbap_Secondary_CCPCH_parameterExtendedList_CTCH_SetupRqstTDD_item = -1;  /* Secondary_CCPCH_parameterItem_CTCH_SetupRqstTDD */
static int hf_nbap_Secondary_CCPCH_LCR_parameterExtendedList_CTCH_SetupRqstTDD_item = -1;  /* Secondary_CCPCH_LCR_parameterItem_CTCH_SetupRqstTDD */
static int hf_nbap_pRACH_Parameters_CTCH_SetupRqstTDD = -1;  /* PRACH_Parameters_CTCH_SetupRqstTDD */
static int hf_nbap_maxPRACH_MidambleShifts = -1;  /* MaxPRACH_MidambleShifts */
static int hf_nbap_pRACH_Midamble = -1;           /* PRACH_Midamble */
static int hf_nbap_rACH = -1;                     /* RACH_Parameter_CTCH_SetupRqstTDD */
static int hf_nbap_uL_TransportFormatSet = -1;    /* TransportFormatSet */
static int hf_nbap_PRACH_LCR_ParametersList_CTCH_SetupRqstTDD_item = -1;  /* PRACH_LCR_ParametersItem_CTCH_SetupRqstTDD */
static int hf_nbap_fPACH_Power = -1;              /* FPACH_Power */
static int hf_nbap_FACH_CommonTransportChannel_InformationResponse_item = -1;  /* CommonTransportChannel_InformationResponse */
static int hf_nbap_secondary_CCPCH_parameters2 = -1;  /* Secondary_CCPCHList_CTCH_ReconfRqstFDD */
static int hf_nbap_pRACH_parameters2 = -1;        /* PRACHList_CTCH_ReconfRqstFDD */
static int hf_nbap_notUsed_cPCH_parameters = -1;  /* NULL */
static int hf_nbap_fACH_ParametersList_CTCH_ReconfRqstFDD = -1;  /* FACH_ParametersList_CTCH_ReconfRqstFDD */
static int hf_nbap_pCH_Parameters_CTCH_ReconfRqstFDD = -1;  /* PCH_Parameters_CTCH_ReconfRqstFDD */
static int hf_nbap_pICH_Parameters_CTCH_ReconfRqstFDD = -1;  /* PICH_Parameters_CTCH_ReconfRqstFDD */
static int hf_nbap_FACH_ParametersListIE_CTCH_ReconfRqstFDD_item = -1;  /* FACH_ParametersItem_CTCH_ReconfRqstFDD */
static int hf_nbap_pRACH_ParametersList_CTCH_ReconfRqstFDD = -1;  /* PRACH_ParametersList_CTCH_ReconfRqstFDD */
static int hf_nbap_aICH_ParametersList_CTCH_ReconfRqstFDD = -1;  /* AICH_ParametersList_CTCH_ReconfRqstFDD */
static int hf_nbap_PRACH_ParametersListIE_CTCH_ReconfRqstFDD_item = -1;  /* PRACH_ParametersItem_CTCH_ReconfRqstFDD */
static int hf_nbap_allowedSlotFormatInformation1 = -1;  /* AllowedSlotFormatInformationList_CTCH_ReconfRqstFDD */
static int hf_nbap_AllowedSlotFormatInformationList_CTCH_ReconfRqstFDD_item = -1;  /* AllowedSlotFormatInformationItem_CTCH_ReconfRqstFDD */
static int hf_nbap_rACH_SlotFormat = -1;          /* RACH_SlotFormat */
static int hf_nbap_AICH_ParametersListIE_CTCH_ReconfRqstFDD_item = -1;  /* AICH_ParametersItem_CTCH_ReconfRqstFDD */
static int hf_nbap_cCTrCH_ID = -1;                /* CCTrCH_ID */
static int hf_nbap_secondaryCCPCHList = -1;       /* Secondary_CCPCHList_CTCH_ReconfRqstTDD */
static int hf_nbap_Secondary_CCPCHListIE_CTCH_ReconfRqstTDD_item = -1;  /* Secondary_CCPCHItem_CTCH_ReconfRqstTDD */
static int hf_nbap_sCCPCH_Power = -1;             /* DL_Power */
static int hf_nbap_FACH_ParametersList_CTCH_ReconfRqstTDD_item = -1;  /* FACH_ParametersItem_CTCH_ReconfRqstTDD */
static int hf_nbap_commonPhysicalChannelId = -1;  /* CommonPhysicalChannelID */
static int hf_nbap_fPACHPower = -1;               /* FPACH_Power */
static int hf_nbap_Secondary_CCPCH_parameterExtendedList_CTCH_ReconfRqstTDD_item = -1;  /* Secondary_CCPCHItem_CTCH_ReconfRqstTDD */
static int hf_nbap_Secondary_CCPCH_LCR_parameterExtendedList_CTCH_ReconfRqstTDD_item = -1;  /* Secondary_CCPCHItem_CTCH_ReconfRqstTDD */
static int hf_nbap_Cell_InformationList_AuditRsp_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_c_ID = -1;                     /* C_ID */
static int hf_nbap_configurationGenerationID = -1;  /* ConfigurationGenerationID */
static int hf_nbap_resourceOperationalState = -1;  /* ResourceOperationalState */
static int hf_nbap_availabilityStatus = -1;       /* AvailabilityStatus */
static int hf_nbap_local_Cell_ID = -1;            /* Local_Cell_ID */
static int hf_nbap_primary_SCH_Information = -1;  /* P_SCH_Information_AuditRsp */
static int hf_nbap_secondary_SCH_Information = -1;  /* S_SCH_Information_AuditRsp */
static int hf_nbap_primary_CPICH_Information = -1;  /* P_CPICH_Information_AuditRsp */
static int hf_nbap_secondary_CPICH_InformationList = -1;  /* S_CPICH_InformationList_AuditRsp */
static int hf_nbap_primary_CCPCH_Information = -1;  /* P_CCPCH_Information_AuditRsp */
static int hf_nbap_bCH_Information = -1;          /* BCH_Information_AuditRsp */
static int hf_nbap_secondary_CCPCH_InformationList = -1;  /* S_CCPCH_InformationList_AuditRsp */
static int hf_nbap_pCH_Information = -1;          /* PCH_Information_AuditRsp */
static int hf_nbap_pICH_Information = -1;         /* PICH_Information_AuditRsp */
static int hf_nbap_fACH_InformationList = -1;     /* FACH_InformationList_AuditRsp */
static int hf_nbap_pRACH_InformationList = -1;    /* PRACH_InformationList_AuditRsp */
static int hf_nbap_rACH_InformationList = -1;     /* RACH_InformationList_AuditRsp */
static int hf_nbap_aICH_InformationList = -1;     /* AICH_InformationList_AuditRsp */
static int hf_nbap_notUsed_1_pCPCH_InformationList = -1;  /* NULL */
static int hf_nbap_notUsed_2_cPCH_InformationList = -1;  /* NULL */
static int hf_nbap_notUsed_3_aP_AICH_InformationList = -1;  /* NULL */
static int hf_nbap_notUsed_4_cDCA_ICH_InformationList = -1;  /* NULL */
static int hf_nbap_sCH_Information = -1;          /* SCH_Information_AuditRsp */
static int hf_nbap_S_CPICH_InformationList_AuditRsp_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_S_CCPCH_InformationList_AuditRsp_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_FACH_InformationList_AuditRsp_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_PRACH_InformationList_AuditRsp_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_RACH_InformationList_AuditRsp_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_AICH_InformationList_AuditRsp_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_CCP_InformationList_AuditRsp_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_communicationControlPortID = -1;  /* CommunicationControlPortID */
static int hf_nbap_FPACH_LCR_InformationList_AuditRsp_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_S_CCPCH_InformationListExt_AuditRsp_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_S_CCPCH_LCR_InformationListExt_AuditRsp_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_Local_Cell_InformationList_AuditRsp_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_dl_or_global_capacityCredit = -1;  /* DL_or_Global_CapacityCredit */
static int hf_nbap_ul_capacityCredit = -1;        /* UL_CapacityCredit */
static int hf_nbap_commonChannelsCapacityConsumptionLaw = -1;  /* CommonChannelsCapacityConsumptionLaw */
static int hf_nbap_dedicatedChannelsCapacityConsumptionLaw = -1;  /* DedicatedChannelsCapacityConsumptionLaw */
static int hf_nbap_maximumDL_PowerCapability = -1;  /* MaximumDL_PowerCapability */
static int hf_nbap_minSpreadingFactor = -1;       /* MinSpreadingFactor */
static int hf_nbap_minimumDL_PowerCapability = -1;  /* MinimumDL_PowerCapability */
static int hf_nbap_local_Cell_Group_ID = -1;      /* Local_Cell_ID */
static int hf_nbap_Local_Cell_Group_InformationList_AuditRsp_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_Power_Local_Cell_Group_InformationList_AuditRsp_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_power_Local_Cell_Group_ID = -1;  /* Local_Cell_ID */
static int hf_nbap_cell = -1;                     /* Cell_CM_Rqst */
static int hf_nbap_rACH1 = -1;                    /* RACH_CM_Rqst */
static int hf_nbap_notUsed_cPCH = -1;             /* NULL */
static int hf_nbap_extension_CommonMeasurementObjectType_CM_Rqst = -1;  /* Extension_CommonMeasurementObjectType_CM_Rqst */
static int hf_nbap_powerLocalCellGroupID = -1;    /* Local_Cell_ID */
static int hf_nbap_cell1 = -1;                    /* Cell_CM_Rsp */
static int hf_nbap_rACH2 = -1;                    /* RACH_CM_Rsp */
static int hf_nbap_extension_CommonMeasurementObjectType_CM_Rsp = -1;  /* Extension_CommonMeasurementObjectType_CM_Rsp */
static int hf_nbap_commonMeasurementValue = -1;   /* CommonMeasurementValue */
static int hf_nbap_n_INSYNC_IND = -1;             /* N_INSYNC_IND */
static int hf_nbap_n_OUTSYNC_IND = -1;            /* N_OUTSYNC_IND */
static int hf_nbap_t_RLFAILURE = -1;              /* T_RLFAILURE */
static int hf_nbap_primarySCH_Power = -1;         /* DL_Power */
static int hf_nbap_secondarySCH_Power = -1;       /* DL_Power */
static int hf_nbap_primaryCPICH_Power = -1;       /* PrimaryCPICH_Power */
static int hf_nbap_transmitDiversityIndicator = -1;  /* TransmitDiversityIndicator */
static int hf_nbap_SecondaryCPICH_InformationList_Cell_SetupRqstFDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_fDD_DL_ChannelisationCodeNumber = -1;  /* FDD_DL_ChannelisationCodeNumber */
static int hf_nbap_secondaryCPICH_Power = -1;     /* DL_Power */
static int hf_nbap_bCH_information = -1;          /* BCH_Information_Cell_SetupRqstFDD */
static int hf_nbap_bCH_Power = -1;                /* DL_Power */
static int hf_nbap_powerRaiseLimit = -1;          /* PowerRaiseLimit */
static int hf_nbap_dLPowerAveragingWindowSize = -1;  /* DLPowerAveragingWindowSize */
static int hf_nbap_iPDL_FDD_Parameters = -1;      /* IPDL_FDD_Parameters */
static int hf_nbap_iPDL_Indicator = -1;           /* IPDL_Indicator */
static int hf_nbap_CellPortion_InformationList_Cell_SetupRqstFDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_cellPortionID = -1;            /* CellPortionID */
static int hf_nbap_associatedSecondaryCPICH = -1;  /* CommonPhysicalChannelID */
static int hf_nbap_maximumTransmissionPowerforCellPortion = -1;  /* MaximumTransmissionPower */
static int hf_nbap_syncCaseIndicator = -1;        /* SyncCaseIndicator_Cell_SetupRqstTDD_PSCH */
static int hf_nbap_sCH_Power = -1;                /* DL_Power */
static int hf_nbap_case1 = -1;                    /* Case1_Cell_SetupRqstTDD */
static int hf_nbap_case2 = -1;                    /* Case2_Cell_SetupRqstTDD */
static int hf_nbap_sCH_TimeSlot = -1;             /* SCH_TimeSlot */
static int hf_nbap_pCCPCH_Power = -1;             /* PCCPCH_Power */
static int hf_nbap_sCTD_Indicator = -1;           /* SCTD_Indicator */
static int hf_nbap_TimeSlotConfigurationList_Cell_SetupRqstTDD_item = -1;  /* TimeSlotConfigurationItem_Cell_SetupRqstTDD */
static int hf_nbap_timeSlotStatus = -1;           /* TimeSlotStatus */
static int hf_nbap_timeSlotDirection = -1;        /* TimeSlotDirection */
static int hf_nbap_TimeSlotConfigurationList_LCR_Cell_SetupRqstTDD_item = -1;  /* TimeSlotConfigurationItem_LCR_Cell_SetupRqstTDD */
static int hf_nbap_dwPCH_Power = -1;              /* DwPCH_Power */
static int hf_nbap_iPDL_TDD_Parameters = -1;      /* IPDL_TDD_Parameters */
static int hf_nbap_iPDL_TDD_Parameters_LCR = -1;  /* IPDL_TDD_Parameters_LCR */
static int hf_nbap_SecondaryCPICH_InformationList_Cell_ReconfRqstFDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_bCH_information1 = -1;         /* BCH_information_Cell_ReconfRqstFDD */
static int hf_nbap_CellPortion_InformationList_Cell_ReconfRqstFDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_TimeSlotConfigurationList_Cell_ReconfRqstTDD_item = -1;  /* TimeSlotConfigurationItem_Cell_ReconfRqstTDD */
static int hf_nbap_TimeSlotConfigurationList_LCR_Cell_ReconfRqstTDD_item = -1;  /* TimeSlotConfigurationItem_LCR_Cell_ReconfRqstTDD */
static int hf_nbap_no_Failure = -1;               /* No_Failure_ResourceStatusInd */
static int hf_nbap_serviceImpacting = -1;         /* ServiceImpacting_ResourceStatusInd */
static int hf_nbap_local_Cell_InformationList = -1;  /* Local_Cell_InformationList_ResourceStatusInd */
static int hf_nbap_local_Cell_Group_InformationList = -1;  /* Local_Cell_Group_InformationList_ResourceStatusInd */
static int hf_nbap_Local_Cell_InformationList_ResourceStatusInd_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_local_CellID = -1;             /* Local_Cell_ID */
static int hf_nbap_addorDeleteIndicator = -1;     /* AddorDeleteIndicator */
static int hf_nbap_Local_Cell_Group_InformationList_ResourceStatusInd_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_Power_Local_Cell_Group_InformationList_ResourceStatusInd_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_local_Cell_InformationList1 = -1;  /* Local_Cell_InformationList2_ResourceStatusInd */
static int hf_nbap_local_Cell_Group_InformationList1 = -1;  /* Local_Cell_Group_InformationList2_ResourceStatusInd */
static int hf_nbap_cCP_InformationList = -1;      /* CCP_InformationList_ResourceStatusInd */
static int hf_nbap_cell_InformationList = -1;     /* Cell_InformationList_ResourceStatusInd */
static int hf_nbap_Local_Cell_InformationList2_ResourceStatusInd_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_maximum_DL_PowerCapability = -1;  /* MaximumDL_PowerCapability */
static int hf_nbap_Local_Cell_Group_InformationList2_ResourceStatusInd_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_Power_Local_Cell_Group_InformationList2_ResourceStatusInd_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_CCP_InformationList_ResourceStatusInd_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_Cell_InformationList_ResourceStatusInd_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_primary_SCH_Information1 = -1;  /* P_SCH_Information_ResourceStatusInd */
static int hf_nbap_secondary_SCH_Information1 = -1;  /* S_SCH_Information_ResourceStatusInd */
static int hf_nbap_primary_CPICH_Information1 = -1;  /* P_CPICH_Information_ResourceStatusInd */
static int hf_nbap_secondary_CPICH_Information = -1;  /* S_CPICH_InformationList_ResourceStatusInd */
static int hf_nbap_primary_CCPCH_Information1 = -1;  /* P_CCPCH_Information_ResourceStatusInd */
static int hf_nbap_bCH_Information1 = -1;         /* BCH_Information_ResourceStatusInd */
static int hf_nbap_secondary_CCPCH_InformationList1 = -1;  /* S_CCPCH_InformationList_ResourceStatusInd */
static int hf_nbap_pCH_Information1 = -1;         /* PCH_Information_ResourceStatusInd */
static int hf_nbap_pICH_Information1 = -1;        /* PICH_Information_ResourceStatusInd */
static int hf_nbap_fACH_InformationList1 = -1;    /* FACH_InformationList_ResourceStatusInd */
static int hf_nbap_pRACH_InformationList1 = -1;   /* PRACH_InformationList_ResourceStatusInd */
static int hf_nbap_rACH_InformationList1 = -1;    /* RACH_InformationList_ResourceStatusInd */
static int hf_nbap_aICH_InformationList1 = -1;    /* AICH_InformationList_ResourceStatusInd */
static int hf_nbap_sCH_Information1 = -1;         /* SCH_Information_ResourceStatusInd */
static int hf_nbap_S_CPICH_InformationList_ResourceStatusInd_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_S_CCPCH_InformationList_ResourceStatusInd_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_FACH_InformationList_ResourceStatusInd_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_PRACH_InformationList_ResourceStatusInd_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_RACH_InformationList_ResourceStatusInd_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_AICH_InformationList_ResourceStatusInd_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_FPACH_LCR_InformationList_ResourceStatusInd_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_S_CCPCH_InformationListExt_ResourceStatusInd_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_S_CCPCH_LCR_InformationListExt_ResourceStatusInd_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_MIB_SB_SIB_InformationList_SystemInfoUpdateRqst_item = -1;  /* MIB_SB_SIB_InformationItem_SystemInfoUpdateRqst */
static int hf_nbap_iB_Type = -1;                  /* IB_Type */
static int hf_nbap_iB_OC_ID = -1;                 /* IB_OC_ID */
static int hf_nbap_deletionIndicator = -1;        /* DeletionIndicator_SystemInfoUpdate */
static int hf_nbap_no_Deletion = -1;              /* No_Deletion_SystemInfoUpdate */
static int hf_nbap_yes_Deletion = -1;             /* NULL */
static int hf_nbap_sIB_Originator = -1;           /* SIB_Originator */
static int hf_nbap_iB_SG_REP = -1;                /* IB_SG_REP */
static int hf_nbap_segmentInformationList = -1;   /* SegmentInformationList_SystemInfoUpdate */
static int hf_nbap_SegmentInformationListIE_SystemInfoUpdate_item = -1;  /* SegmentInformationItem_SystemInfoUpdate */
static int hf_nbap_iB_SG_POS = -1;                /* IB_SG_POS */
static int hf_nbap_segment_Type = -1;             /* Segment_Type */
static int hf_nbap_iB_SG_DATA = -1;               /* IB_SG_DATA */
static int hf_nbap_ul_ScramblingCode = -1;        /* UL_ScramblingCode */
static int hf_nbap_minUL_ChannelisationCodeLength = -1;  /* MinUL_ChannelisationCodeLength */
static int hf_nbap_maxNrOfUL_DPDCHs = -1;         /* MaxNrOfUL_DPDCHs */
static int hf_nbap_ul_PunctureLimit = -1;         /* PunctureLimit */
static int hf_nbap_ul_DPCCH_SlotFormat = -1;      /* UL_DPCCH_SlotFormat */
static int hf_nbap_ul_SIR_Target = -1;            /* UL_SIR */
static int hf_nbap_diversityMode = -1;            /* DiversityMode */
static int hf_nbap_not_Used_sSDT_CellID_Length = -1;  /* NULL */
static int hf_nbap_not_Used_s_FieldLength = -1;   /* NULL */
static int hf_nbap_dl_DPCH_SlotFormat = -1;       /* DL_DPCH_SlotFormat */
static int hf_nbap_tFCI_SignallingMode = -1;      /* TFCI_SignallingMode */
static int hf_nbap_not_Used_pDSCH_RL_ID = -1;     /* NULL */
static int hf_nbap_not_Used_pDSCH_CodeMapping = -1;  /* NULL */
static int hf_nbap_powerOffsetInformation1 = -1;  /* PowerOffsetInformation_RL_SetupRqstFDD */
static int hf_nbap_fdd_TPC_DownlinkStepSize = -1;  /* FDD_TPC_DownlinkStepSize */
static int hf_nbap_limitedPowerIncrease = -1;     /* LimitedPowerIncrease */
static int hf_nbap_innerLoopDLPCStatus = -1;      /* InnerLoopDLPCStatus */
static int hf_nbap_pO2_ForTPC_Bits = -1;          /* PowerOffset */
static int hf_nbap_RL_InformationList_RL_SetupRqstFDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_rL_ID = -1;                    /* RL_ID */
static int hf_nbap_firstRLS_indicator = -1;       /* FirstRLS_Indicator */
static int hf_nbap_frameOffset = -1;              /* FrameOffset */
static int hf_nbap_chipOffset = -1;               /* ChipOffset */
static int hf_nbap_propagationDelay = -1;         /* PropagationDelay */
static int hf_nbap_diversityControlField = -1;    /* DiversityControlField */
static int hf_nbap_dl_CodeInformation = -1;       /* FDD_DL_CodeInformation */
static int hf_nbap_initialDL_transmissionPower = -1;  /* DL_Power */
static int hf_nbap_maximumDL_power = -1;          /* DL_Power */
static int hf_nbap_minimumDL_power = -1;          /* DL_Power */
static int hf_nbap_not_Used_sSDT_Cell_Identity = -1;  /* NULL */
static int hf_nbap_maxSet_E_DPDCHs = -1;          /* Max_Set_E_DPDCHs */
static int hf_nbap_e_TFCS_Information = -1;       /* E_TFCS_Information */
static int hf_nbap_e_TTI = -1;                    /* E_TTI */
static int hf_nbap_e_DPCCH_PO = -1;               /* E_DPCCH_PO */
static int hf_nbap_powerOffsetInformation2 = -1;  /* PowerOffsetInformation_F_DPCH_RL_SetupRqstFDD */
static int hf_nbap_UL_CCTrCH_InformationList_RL_SetupRqstTDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_uL_DPCH_Information = -1;      /* UL_DPCH_Information_RL_SetupRqstTDD */
static int hf_nbap_tdd_DPCHOffset = -1;           /* TDD_DPCHOffset */
static int hf_nbap_uL_Timeslot_Information = -1;  /* UL_Timeslot_Information */
static int hf_nbap_uL_TimeslotLCR_Information = -1;  /* UL_TimeslotLCR_Information */
static int hf_nbap_DL_CCTrCH_InformationList_RL_SetupRqstTDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_tdd_TPC_DownlinkStepSize = -1;  /* TDD_TPC_DownlinkStepSize */
static int hf_nbap_cCTrCH_TPCList = -1;           /* CCTrCH_TPCList_RL_SetupRqstTDD */
static int hf_nbap_dL_DPCH_Information = -1;      /* DL_DPCH_Information_RL_SetupRqstTDD */
static int hf_nbap_CCTrCH_TPCList_RL_SetupRqstTDD_item = -1;  /* CCTrCH_TPCItem_RL_SetupRqstTDD */
static int hf_nbap_dL_Timeslot_Information = -1;  /* DL_Timeslot_Information */
static int hf_nbap_dL_TimeslotLCR_Information = -1;  /* DL_TimeslotLCR_Information */
static int hf_nbap_tstdIndicator = -1;            /* TSTD_Indicator */
static int hf_nbap_specialBurstScheduling = -1;   /* SpecialBurstScheduling */
static int hf_nbap_dL_TimeSlotISCPInfo = -1;      /* DL_TimeslotISCPInfo */
static int hf_nbap_RL_InformationResponseList_RL_SetupRspFDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_rL_Set_ID = -1;                /* RL_Set_ID */
static int hf_nbap_received_total_wide_band_power = -1;  /* Received_total_wide_band_power_Value */
static int hf_nbap_diversityIndication = -1;      /* DiversityIndication_RL_SetupRspFDD */
static int hf_nbap_not_Used_dSCH_InformationResponseList = -1;  /* NULL */
static int hf_nbap_sSDT_SupportIndicator = -1;    /* SSDT_SupportIndicator */
static int hf_nbap_combining = -1;                /* Combining_RL_SetupRspFDD */
static int hf_nbap_nonCombiningOrFirstRL = -1;    /* NonCombiningOrFirstRL_RL_SetupRspFDD */
static int hf_nbap_dCH_InformationResponse = -1;  /* DCH_InformationResponse */
static int hf_nbap_uL_TimeSlot_ISCP_Info = -1;    /* UL_TimeSlot_ISCP_Info */
static int hf_nbap_ul_PhysCH_SF_Variation = -1;   /* UL_PhysCH_SF_Variation */
static int hf_nbap_dCH_InformationResponseList = -1;  /* DCH_InformationResponseList_RL_SetupRspTDD */
static int hf_nbap_dSCH_InformationResponseList = -1;  /* DSCH_InformationResponseList_RL_SetupRspTDD */
static int hf_nbap_uSCH_InformationResponseList = -1;  /* USCH_InformationResponseList_RL_SetupRspTDD */
static int hf_nbap_uL_TimeSlot_ISCP_LCR_Info = -1;  /* UL_TimeSlot_ISCP_LCR_Info */
static int hf_nbap_generalCause = -1;             /* GeneralCauseList_RL_SetupFailureFDD */
static int hf_nbap_rLSpecificCause = -1;          /* RLSpecificCauseList_RL_SetupFailureFDD */
static int hf_nbap_cause = -1;                    /* Cause */
static int hf_nbap_unsuccessful_RL_InformationRespList_RL_SetupFailureFDD = -1;  /* Unsuccessful_RL_InformationRespList_RL_SetupFailureFDD */
static int hf_nbap_successful_RL_InformationRespList_RL_SetupFailureFDD = -1;  /* Successful_RL_InformationRespList_RL_SetupFailureFDD */
static int hf_nbap_Unsuccessful_RL_InformationRespList_RL_SetupFailureFDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_Successful_RL_InformationRespList_RL_SetupFailureFDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_diversityIndication1 = -1;     /* DiversityIndication_RL_SetupFailureFDD */
static int hf_nbap_not_Used_tFCI2_BearerInformationResponse = -1;  /* NULL */
static int hf_nbap_combining1 = -1;               /* Combining_RL_SetupFailureFDD */
static int hf_nbap_nonCombiningOrFirstRL1 = -1;   /* NonCombiningOrFirstRL_RL_SetupFailureFDD */
static int hf_nbap_generalCause1 = -1;            /* GeneralCauseList_RL_SetupFailureTDD */
static int hf_nbap_rLSpecificCause1 = -1;         /* RLSpecificCauseList_RL_SetupFailureTDD */
static int hf_nbap_unsuccessful_RL_InformationRespItem_RL_SetupFailureTDD = -1;  /* Unsuccessful_RL_InformationRespItem_RL_SetupFailureTDD */
static int hf_nbap_RL_InformationList_RL_AdditionRqstFDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_initialDL_TransmissionPower = -1;  /* DL_Power */
static int hf_nbap_maximumDL_Power = -1;          /* DL_Power */
static int hf_nbap_minimumDL_Power = -1;          /* DL_Power */
static int hf_nbap_not_Used_sSDT_CellIdentity = -1;  /* NULL */
static int hf_nbap_UL_CCTrCH_InformationList_RL_AdditionRqstTDD_item = -1;  /* UL_CCTrCH_InformationItem_RL_AdditionRqstTDD */
static int hf_nbap_uL_DPCH_Information1 = -1;     /* UL_DPCH_InformationList_RL_AdditionRqstTDD */
static int hf_nbap_DL_CCTrCH_InformationList_RL_AdditionRqstTDD_item = -1;  /* DL_CCTrCH_InformationItem_RL_AdditionRqstTDD */
static int hf_nbap_dL_DPCH_Information1 = -1;     /* DL_DPCH_InformationList_RL_AdditionRqstTDD */
static int hf_nbap_initial_DL_Transmission_Power = -1;  /* DL_Power */
static int hf_nbap_RL_InformationResponseList_RL_AdditionRspFDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_diversityIndication2 = -1;     /* DiversityIndication_RL_AdditionRspFDD */
static int hf_nbap_combining2 = -1;               /* Combining_RL_AdditionRspFDD */
static int hf_nbap_non_combining = -1;            /* Non_Combining_RL_AdditionRspFDD */
static int hf_nbap_dCH_Information = -1;          /* DCH_Information_RL_AdditionRspTDD */
static int hf_nbap_dSCH_InformationResponseList1 = -1;  /* DSCH_InformationResponseList_RL_AdditionRspTDD */
static int hf_nbap_uSCH_InformationResponseList1 = -1;  /* USCH_InformationResponseList_RL_AdditionRspTDD */
static int hf_nbap_diversityIndication3 = -1;     /* DiversityIndication_RL_AdditionRspTDD */
static int hf_nbap_combining3 = -1;               /* Combining_RL_AdditionRspTDD */
static int hf_nbap_non_Combining = -1;            /* Non_Combining_RL_AdditionRspTDD */
static int hf_nbap_uL_TimeSlot_ISCP_InfoLCR = -1;  /* UL_TimeSlot_ISCP_LCR_Info */
static int hf_nbap_generalCause2 = -1;            /* GeneralCauseList_RL_AdditionFailureFDD */
static int hf_nbap_rLSpecificCause2 = -1;         /* RLSpecificCauseList_RL_AdditionFailureFDD */
static int hf_nbap_unsuccessful_RL_InformationRespList_RL_AdditionFailureFDD = -1;  /* Unsuccessful_RL_InformationRespList_RL_AdditionFailureFDD */
static int hf_nbap_successful_RL_InformationRespList_RL_AdditionFailureFDD = -1;  /* Successful_RL_InformationRespList_RL_AdditionFailureFDD */
static int hf_nbap_Unsuccessful_RL_InformationRespList_RL_AdditionFailureFDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_Successful_RL_InformationRespList_RL_AdditionFailureFDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_diversityIndication4 = -1;     /* DiversityIndication_RL_AdditionFailureFDD */
static int hf_nbap_combining4 = -1;               /* Combining_RL_AdditionFailureFDD */
static int hf_nbap_non_Combining1 = -1;           /* Non_Combining_RL_AdditionFailureFDD */
static int hf_nbap_generalCause3 = -1;            /* GeneralCauseList_RL_AdditionFailureTDD */
static int hf_nbap_rLSpecificCause3 = -1;         /* RLSpecificCauseList_RL_AdditionFailureTDD */
static int hf_nbap_unsuccessful_RL_InformationRespItem_RL_AdditionFailureTDD = -1;  /* Unsuccessful_RL_InformationRespItem_RL_AdditionFailureTDD */
static int hf_nbap_not_Used_sSDT_CellIDLength = -1;  /* NULL */
static int hf_nbap_powerOffsetInformation3 = -1;  /* PowerOffsetInformation_RL_ReconfPrepFDD */
static int hf_nbap_DCH_DeleteList_RL_ReconfPrepFDD_item = -1;  /* DCH_DeleteItem_RL_ReconfPrepFDD */
static int hf_nbap_dCH_ID = -1;                   /* DCH_ID */
static int hf_nbap_RL_InformationList_RL_ReconfPrepFDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_maxDL_Power = -1;              /* DL_Power */
static int hf_nbap_minDL_Power = -1;              /* DL_Power */
static int hf_nbap_not_Used_sSDT_Indication = -1;  /* NULL */
static int hf_nbap_powerOffsetInformation4 = -1;  /* PowerOffsetInformation_F_DPCH_RL_ReconfPrepFDD */
static int hf_nbap_UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD_item = -1;  /* UL_CCTrCH_InformationAddItem_RL_ReconfPrepTDD */
static int hf_nbap_ul_DPCH_InformationList = -1;  /* UL_DPCH_InformationAddList_RL_ReconfPrepTDD */
static int hf_nbap_MultipleRL_UL_DPCH_InformationAddList_RL_ReconfPrepTDD_item = -1;  /* MultipleRL_UL_DPCH_InformationAddListIE_RL_ReconfPrepTDD */
static int hf_nbap_ul_DPCH_InformationListLCR = -1;  /* UL_DPCH_LCR_InformationAddList_RL_ReconfPrepTDD */
static int hf_nbap_ul_sir_target = -1;            /* UL_SIR */
static int hf_nbap_tDD_TPC_UplinkStepSize_LCR = -1;  /* TDD_TPC_UplinkStepSize_LCR */
static int hf_nbap_uL_Timeslot_InformationLCR = -1;  /* UL_TimeslotLCR_Information */
static int hf_nbap_UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD_item = -1;  /* UL_CCTrCH_InformationModifyItem_RL_ReconfPrepTDD */
static int hf_nbap_ul_DPCH_InformationAddList = -1;  /* UL_DPCH_InformationModify_AddList_RL_ReconfPrepTDD */
static int hf_nbap_ul_DPCH_InformationModifyList = -1;  /* UL_DPCH_InformationModify_ModifyList_RL_ReconfPrepTDD */
static int hf_nbap_ul_DPCH_InformationDeleteList = -1;  /* UL_DPCH_InformationModify_DeleteList_RL_ReconfPrepTDD */
static int hf_nbap_MultipleRL_UL_DPCH_InformationModifyList_RL_ReconfPrepTDD_item = -1;  /* MultipleRL_UL_DPCH_InformationModifyListIE_RL_ReconfPrepTDD */
static int hf_nbap_ul_DPCH_InformationAddListLCR = -1;  /* UL_DPCH_LCR_InformationModify_AddList_RL_ReconfPrepTDD */
static int hf_nbap_uL_Timeslot_InformationModify_ModifyList_RL_ReconfPrepTDD = -1;  /* UL_Timeslot_InformationModify_ModifyList_RL_ReconfPrepTDD */
static int hf_nbap_UL_Timeslot_InformationModify_ModifyList_RL_ReconfPrepTDD_item = -1;  /* UL_Timeslot_InformationModify_ModifyItem_RL_ReconfPrepTDD */
static int hf_nbap_midambleShiftAndBurstType = -1;  /* MidambleShiftAndBurstType */
static int hf_nbap_uL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD = -1;  /* UL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD */
static int hf_nbap_UL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD_item = -1;  /* UL_Code_InformationModify_ModifyItem_RL_ReconfPrepTDD */
static int hf_nbap_dPCH_ID = -1;                  /* DPCH_ID */
static int hf_nbap_UL_TimeslotLCR_InformationModify_ModifyList_RL_ReconfPrepTDD_item = -1;  /* UL_Timeslot_LCR_InformationModify_ModifyItem_RL_ReconfPrepTDD */
static int hf_nbap_uL_Code_InformationModify_ModifyList_RL_ReconfPrepTDDLCR = -1;  /* UL_Code_InformationModify_ModifyList_RL_ReconfPrepTDDLCR */
static int hf_nbap_UL_Code_InformationModify_ModifyList_RL_ReconfPrepTDDLCR_item = -1;  /* UL_Code_InformationModify_ModifyItem_RL_ReconfPrepTDDLCR */
static int hf_nbap_UL_DPCH_InformationModify_DeleteListIE_RL_ReconfPrepTDD_item = -1;  /* UL_DPCH_InformationModify_DeleteItem_RL_ReconfPrepTDD */
static int hf_nbap_UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD_item = -1;  /* UL_CCTrCH_InformationDeleteItem_RL_ReconfPrepTDD */
static int hf_nbap_DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD_item = -1;  /* DL_CCTrCH_InformationAddItem_RL_ReconfPrepTDD */
static int hf_nbap_cCTrCH_TPCList1 = -1;          /* CCTrCH_TPCAddList_RL_ReconfPrepTDD */
static int hf_nbap_dl_DPCH_InformationList = -1;  /* DL_DPCH_InformationAddList_RL_ReconfPrepTDD */
static int hf_nbap_MultipleRL_DL_DPCH_InformationAddList_RL_ReconfPrepTDD_item = -1;  /* MultipleRL_DL_DPCH_InformationAddListIE_RL_ReconfPrepTDD */
static int hf_nbap_dl_DPCH_InformationListLCR = -1;  /* DL_DPCH_LCR_InformationAddList_RL_ReconfPrepTDD */
static int hf_nbap_cCTrCH_Initial_DL_Power = -1;  /* DL_Power */
static int hf_nbap_tDD_TPC_DownlinkStepSize = -1;  /* TDD_TPC_DownlinkStepSize */
static int hf_nbap_cCTrCH_Maximum_DL_Power_InformationAdd_RL_ReconfPrepTDD = -1;  /* DL_Power */
static int hf_nbap_cCTrCH_Minimum_DL_Power_InformationAdd_RL_ReconfPrepTDD = -1;  /* DL_Power */
static int hf_nbap_CCTrCH_TPCAddList_RL_ReconfPrepTDD_item = -1;  /* CCTrCH_TPCAddItem_RL_ReconfPrepTDD */
static int hf_nbap_dL_Timeslot_InformationLCR = -1;  /* DL_TimeslotLCR_Information */
static int hf_nbap_DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD_item = -1;  /* DL_CCTrCH_InformationModifyItem_RL_ReconfPrepTDD */
static int hf_nbap_cCTrCH_TPCList2 = -1;          /* CCTrCH_TPCModifyList_RL_ReconfPrepTDD */
static int hf_nbap_dl_DPCH_InformationAddList = -1;  /* DL_DPCH_InformationModify_AddList_RL_ReconfPrepTDD */
static int hf_nbap_dl_DPCH_InformationModifyList = -1;  /* DL_DPCH_InformationModify_ModifyList_RL_ReconfPrepTDD */
static int hf_nbap_dl_DPCH_InformationDeleteList = -1;  /* DL_DPCH_InformationModify_DeleteList_RL_ReconfPrepTDD */
static int hf_nbap_MultipleRL_DL_DPCH_InformationModifyList_RL_ReconfPrepTDD_item = -1;  /* MultipleRL_DL_DPCH_InformationModifyListIE_RL_ReconfPrepTDD */
static int hf_nbap_dl_DPCH_InformationAddListLCR = -1;  /* DL_DPCH_LCR_InformationModify_AddList_RL_ReconfPrepTDD */
static int hf_nbap_tDD_TPC_DownlinkStepSize_InformationModify_RL_ReconfPrepTDD = -1;  /* TDD_TPC_DownlinkStepSize */
static int hf_nbap_cCTrCH_Maximum_DL_Power_InformationModify_RL_ReconfPrepTDD = -1;  /* DL_Power */
static int hf_nbap_cCTrCH_Minimum_DL_Power_InformationModify_RL_ReconfPrepTDD = -1;  /* DL_Power */
static int hf_nbap_CCTrCH_TPCModifyList_RL_ReconfPrepTDD_item = -1;  /* CCTrCH_TPCModifyItem_RL_ReconfPrepTDD */
static int hf_nbap_dL_Timeslot_InformationAddModify_ModifyList_RL_ReconfPrepTDD = -1;  /* DL_Timeslot_InformationModify_ModifyList_RL_ReconfPrepTDD */
static int hf_nbap_DL_Timeslot_InformationModify_ModifyList_RL_ReconfPrepTDD_item = -1;  /* DL_Timeslot_InformationModify_ModifyItem_RL_ReconfPrepTDD */
static int hf_nbap_dL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD = -1;  /* DL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD */
static int hf_nbap_DL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD_item = -1;  /* DL_Code_InformationModify_ModifyItem_RL_ReconfPrepTDD */
static int hf_nbap_DL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD_item = -1;  /* DL_Timeslot_LCR_InformationModify_ModifyItem_RL_ReconfPrepTDD */
static int hf_nbap_dL_Code_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD = -1;  /* DL_Code_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD */
static int hf_nbap_DL_Code_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD_item = -1;  /* DL_Code_LCR_InformationModify_ModifyItem_RL_ReconfPrepTDD */
static int hf_nbap_DL_DPCH_InformationModify_DeleteListIE_RL_ReconfPrepTDD_item = -1;  /* DL_DPCH_InformationModify_DeleteItem_RL_ReconfPrepTDD */
static int hf_nbap_DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD_item = -1;  /* DL_CCTrCH_InformationDeleteItem_RL_ReconfPrepTDD */
static int hf_nbap_DCH_DeleteList_RL_ReconfPrepTDD_item = -1;  /* DCH_DeleteItem_RL_ReconfPrepTDD */
static int hf_nbap_DSCH_Information_ModifyList_RL_ReconfPrepTDD_item = -1;  /* DSCH_Information_ModifyItem_RL_ReconfPrepTDD */
static int hf_nbap_dSCH_ID = -1;                  /* DSCH_ID */
static int hf_nbap_allocationRetentionPriority = -1;  /* AllocationRetentionPriority */
static int hf_nbap_frameHandlingPriority = -1;    /* FrameHandlingPriority */
static int hf_nbap_transportBearerRequestIndicator = -1;  /* TransportBearerRequestIndicator */
static int hf_nbap_DSCH_Information_DeleteList_RL_ReconfPrepTDD_item = -1;  /* DSCH_Information_DeleteItem_RL_ReconfPrepTDD */
static int hf_nbap_USCH_Information_ModifyList_RL_ReconfPrepTDD_item = -1;  /* USCH_Information_ModifyItem_RL_ReconfPrepTDD */
static int hf_nbap_uSCH_ID = -1;                  /* USCH_ID */
static int hf_nbap_USCH_Information_DeleteList_RL_ReconfPrepTDD_item = -1;  /* USCH_Information_DeleteItem_RL_ReconfPrepTDD */
static int hf_nbap_MultipleRL_Information_RL_ReconfPrepTDD_item = -1;  /* RL_Information_RL_ReconfPrepTDD */
static int hf_nbap_RL_InformationResponseList_RL_ReconfReady_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_dCH_InformationResponseList_RL_ReconfReady = -1;  /* DCH_InformationResponseList_RL_ReconfReady */
static int hf_nbap_dSCH_InformationResponseList_RL_ReconfReady = -1;  /* DSCH_InformationResponseList_RL_ReconfReady */
static int hf_nbap_uSCH_InformationResponseList_RL_ReconfReady = -1;  /* USCH_InformationResponseList_RL_ReconfReady */
static int hf_nbap_generalCause4 = -1;            /* GeneralCauseList_RL_ReconfFailure */
static int hf_nbap_rLSpecificCause4 = -1;         /* RLSpecificCauseList_RL_ReconfFailure */
static int hf_nbap_rL_ReconfigurationFailureList_RL_ReconfFailure = -1;  /* RL_ReconfigurationFailureList_RL_ReconfFailure */
static int hf_nbap_RL_ReconfigurationFailureList_RL_ReconfFailure_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_ul_TFCS = -1;                  /* TFCS */
static int hf_nbap_dl_TFCS = -1;                  /* TFCS */
static int hf_nbap_DCH_DeleteList_RL_ReconfRqstFDD_item = -1;  /* DCH_DeleteItem_RL_ReconfRqstFDD */
static int hf_nbap_RL_InformationList_RL_ReconfRqstFDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_MultipleRL_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD_item = -1;  /* MultipleRL_DL_CCTrCH_InformationModifyListIE_RL_ReconfRqstTDD */
static int hf_nbap_dl_DPCH_LCR_InformationModifyList = -1;  /* DL_DPCH_LCR_InformationModify_ModifyList_RL_ReconfRqstTDD */
static int hf_nbap_cCTrCH_Maximum_DL_Power_InformationModify_RL_ReconfRqstTDD = -1;  /* DL_Power */
static int hf_nbap_cCTrCH_Minimum_DL_Power_InformationModify_RL_ReconfRqstTDD = -1;  /* DL_Power */
static int hf_nbap_dL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfRqstTDD = -1;  /* DL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfRqstTDD */
static int hf_nbap_DL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfRqstTDD_item = -1;  /* DL_Timeslot_LCR_InformationModify_ModifyItem_RL_ReconfRqstTDD */
static int hf_nbap_maxPowerLCR = -1;              /* DL_Power */
static int hf_nbap_minPowerLCR = -1;              /* DL_Power */
static int hf_nbap_DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_DCH_DeleteList_RL_ReconfRqstTDD_item = -1;  /* DCH_DeleteItem_RL_ReconfRqstTDD */
static int hf_nbap_Multiple_RL_Information_RL_ReconfRqstTDD_item = -1;  /* RL_Information_RL_ReconfRqstTDD */
static int hf_nbap_RL_InformationResponseList_RL_ReconfRsp_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_dCH_InformationResponseList_RL_ReconfRsp = -1;  /* DCH_InformationResponseList_RL_ReconfRsp */
static int hf_nbap_RL_informationList_RL_DeletionRqst_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_DL_ReferencePowerInformationList_DL_PC_Rqst_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_dl_ReferencePower = -1;        /* DL_Power */
static int hf_nbap_rL = -1;                       /* RL_DM_Rqst */
static int hf_nbap_rLS = -1;                      /* RL_Set_DM_Rqst */
static int hf_nbap_all_RL = -1;                   /* AllRL_DM_Rqst */
static int hf_nbap_all_RLS = -1;                  /* AllRL_Set_DM_Rqst */
static int hf_nbap_rL_InformationList = -1;       /* RL_InformationList_DM_Rqst */
static int hf_nbap_RL_InformationList_DM_Rqst_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_PUSCH_Info_DM_Rqst_item = -1;  /* PUSCH_ID */
static int hf_nbap_HSSICH_Info_DM_Rqst_item = -1;  /* HS_SICH_ID */
static int hf_nbap_rL_Set_InformationList_DM_Rqst = -1;  /* RL_Set_InformationList_DM_Rqst */
static int hf_nbap_RL_Set_InformationList_DM_Rqst_item = -1;  /* RL_Set_InformationItem_DM_Rqst */
static int hf_nbap_rL1 = -1;                      /* RL_DM_Rsp */
static int hf_nbap_rLS1 = -1;                     /* RL_Set_DM_Rsp */
static int hf_nbap_all_RL1 = -1;                  /* RL_DM_Rsp */
static int hf_nbap_all_RLS1 = -1;                 /* RL_Set_DM_Rsp */
static int hf_nbap_rL_InformationList_DM_Rsp = -1;  /* RL_InformationList_DM_Rsp */
static int hf_nbap_RL_InformationList_DM_Rsp_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_dedicatedMeasurementValue = -1;  /* DedicatedMeasurementValue */
static int hf_nbap_cFN = -1;                      /* CFN */
static int hf_nbap_PUSCH_Info_DM_Rsp_item = -1;   /* PUSCH_ID */
static int hf_nbap_Multiple_PUSCH_InfoList_DM_Rsp_item = -1;  /* Multiple_PUSCH_InfoListIE_DM_Rsp */
static int hf_nbap_pUSCH_ID = -1;                 /* PUSCH_ID */
static int hf_nbap_rL_Set_InformationList_DM_Rsp = -1;  /* RL_Set_InformationList_DM_Rsp */
static int hf_nbap_RL_Set_InformationList_DM_Rsp_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_Multiple_DedicatedMeasurementValueList_TDD_DM_Rsp_item = -1;  /* Multiple_DedicatedMeasurementValueItem_TDD_DM_Rsp */
static int hf_nbap_Multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp_item = -1;  /* Multiple_DedicatedMeasurementValueItem_LCR_TDD_DM_Rsp */
static int hf_nbap_rL2 = -1;                      /* RL_DM_Rprt */
static int hf_nbap_rLS2 = -1;                     /* RL_Set_DM_Rprt */
static int hf_nbap_all_RL2 = -1;                  /* RL_DM_Rprt */
static int hf_nbap_all_RLS2 = -1;                 /* RL_Set_DM_Rprt */
static int hf_nbap_rL_InformationList_DM_Rprt = -1;  /* RL_InformationList_DM_Rprt */
static int hf_nbap_RL_InformationList_DM_Rprt_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_dedicatedMeasurementValueInformation = -1;  /* DedicatedMeasurementValueInformation */
static int hf_nbap_PUSCH_Info_DM_Rprt_item = -1;  /* PUSCH_ID */
static int hf_nbap_Multiple_PUSCH_InfoList_DM_Rprt_item = -1;  /* Multiple_PUSCH_InfoListIE_DM_Rprt */
static int hf_nbap_rL_Set_InformationList_DM_Rprt = -1;  /* RL_Set_InformationList_DM_Rprt */
static int hf_nbap_RL_Set_InformationList_DM_Rprt_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_rL3 = -1;                      /* RL_RL_FailureInd */
static int hf_nbap_rL_Set = -1;                   /* RL_Set_RL_FailureInd */
static int hf_nbap_cCTrCH = -1;                   /* CCTrCH_RL_FailureInd */
static int hf_nbap_rL_InformationList_RL_FailureInd = -1;  /* RL_InformationList_RL_FailureInd */
static int hf_nbap_RL_InformationList_RL_FailureInd_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_rL_Set_InformationList_RL_FailureInd = -1;  /* RL_Set_InformationList_RL_FailureInd */
static int hf_nbap_RL_Set_InformationList_RL_FailureInd_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_cCTrCH_InformationList_RL_FailureInd = -1;  /* CCTrCH_InformationList_RL_FailureInd */
static int hf_nbap_CCTrCH_InformationList_RL_FailureInd_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_RL_InformationList_RL_PreemptRequiredInd_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_rL4 = -1;                      /* RL_RL_RestoreInd */
static int hf_nbap_rL_Set1 = -1;                  /* RL_Set_RL_RestoreInd */
static int hf_nbap_cCTrCH1 = -1;                  /* CCTrCH_RL_RestoreInd */
static int hf_nbap_rL_InformationList_RL_RestoreInd = -1;  /* RL_InformationList_RL_RestoreInd */
static int hf_nbap_RL_InformationList_RL_RestoreInd_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_rL_Set_InformationList_RL_RestoreInd = -1;  /* RL_Set_InformationList_RL_RestoreInd */
static int hf_nbap_RL_Set_InformationList_RL_RestoreInd_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_cCTrCH_InformationList_RL_RestoreInd = -1;  /* CCTrCH_InformationList_RL_RestoreInd */
static int hf_nbap_CCTrCH_InformationList_RL_RestoreInd_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_privateIEs = -1;               /* PrivateIE_Container */
static int hf_nbap_HSDPA_And_EDCH_CellPortion_InformationList_PSCH_ReconfRqst_item = -1;  /* HSDPA_And_EDCH_CellPortion_InformationItem_PSCH_ReconfRqst */
static int hf_nbap_hS_PDSCH_HS_SCCH_ScramblingCode_PSCH_ReconfRqst = -1;  /* DL_ScramblingCode */
static int hf_nbap_hS_PDSCH_FDD_Code_Information_PSCH_ReconfRqst = -1;  /* HS_PDSCH_FDD_Code_Information */
static int hf_nbap_hS_SCCH_FDD_Code_Information_PSCH_ReconfRqst = -1;  /* HS_SCCH_FDD_Code_Information */
static int hf_nbap_hS_PDSCH_HS_SCCH_E_AGCH_E_RGCH_E_HICH_MaxPower_PSCH_ReconfRqst = -1;  /* MaximumTransmissionPower */
static int hf_nbap_PDSCHSets_AddList_PSCH_ReconfRqst_item = -1;  /* PDSCHSets_AddItem_PSCH_ReconfRqst */
static int hf_nbap_pDSCHSet_ID = -1;              /* PDSCHSet_ID */
static int hf_nbap_pDSCH_InformationList = -1;    /* PDSCH_Information_AddList_PSCH_ReconfRqst */
static int hf_nbap_dL_Timeslot_InformationAddList_PSCH_ReconfRqst = -1;  /* DL_Timeslot_InformationAddList_PSCH_ReconfRqst */
static int hf_nbap_DL_Timeslot_InformationAddList_PSCH_ReconfRqst_item = -1;  /* DL_Timeslot_InformationAddItem_PSCH_ReconfRqst */
static int hf_nbap_dL_Code_InformationAddList_PSCH_ReconfRqst = -1;  /* DL_Code_InformationAddList_PSCH_ReconfRqst */
static int hf_nbap_DL_Code_InformationAddList_PSCH_ReconfRqst_item = -1;  /* DL_Code_InformationAddItem_PSCH_ReconfRqst */
static int hf_nbap_pDSCH_ID = -1;                 /* PDSCH_ID */
static int hf_nbap_dL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst = -1;  /* DL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst */
static int hf_nbap_DL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst_item = -1;  /* DL_Timeslot_InformationAddItem_LCR_PSCH_ReconfRqst */
static int hf_nbap_dL_Code_InformationAddList_LCR_PSCH_ReconfRqst = -1;  /* DL_Code_InformationAddList_LCR_PSCH_ReconfRqst */
static int hf_nbap_DL_Code_InformationAddList_LCR_PSCH_ReconfRqst_item = -1;  /* DL_Code_InformationAddItem_LCR_PSCH_ReconfRqst */
static int hf_nbap_PDSCHSets_ModifyList_PSCH_ReconfRqst_item = -1;  /* PDSCHSets_ModifyItem_PSCH_ReconfRqst */
static int hf_nbap_pDSCH_InformationList1 = -1;   /* PDSCH_Information_ModifyList_PSCH_ReconfRqst */
static int hf_nbap_dL_Timeslot_InformationModifyList_PSCH_ReconfRqst = -1;  /* DL_Timeslot_InformationModifyList_PSCH_ReconfRqst */
static int hf_nbap_DL_Timeslot_InformationModifyList_PSCH_ReconfRqst_item = -1;  /* DL_Timeslot_InformationModifyItem_PSCH_ReconfRqst */
static int hf_nbap_dL_Code_InformationModifyList_PSCH_ReconfRqst = -1;  /* DL_Code_InformationModifyList_PSCH_ReconfRqst */
static int hf_nbap_DL_Code_InformationModifyList_PSCH_ReconfRqst_item = -1;  /* DL_Code_InformationModifyItem_PSCH_ReconfRqst */
static int hf_nbap_dL_Timeslot_LCR_InformationModifyList_PSCH_ReconfRqst = -1;  /* DL_Timeslot_LCR_InformationModifyList_PSCH_ReconfRqst */
static int hf_nbap_DL_Timeslot_LCR_InformationModifyList_PSCH_ReconfRqst_item = -1;  /* DL_Timeslot_LCR_InformationModifyItem_PSCH_ReconfRqst */
static int hf_nbap_dL_Code_LCR_InformationModifyList_PSCH_ReconfRqst = -1;  /* DL_Code_LCR_InformationModifyList_PSCH_ReconfRqst */
static int hf_nbap_DL_Code_LCR_InformationModifyList_PSCH_ReconfRqst_item = -1;  /* DL_Code_LCR_InformationModifyItem_PSCH_ReconfRqst */
static int hf_nbap_PDSCHSets_DeleteList_PSCH_ReconfRqst_item = -1;  /* PDSCHSets_DeleteItem_PSCH_ReconfRqst */
static int hf_nbap_PUSCHSets_AddList_PSCH_ReconfRqst_item = -1;  /* PUSCHSets_AddItem_PSCH_ReconfRqst */
static int hf_nbap_pUSCHSet_ID = -1;              /* PUSCHSet_ID */
static int hf_nbap_pUSCH_InformationList = -1;    /* PUSCH_Information_AddList_PSCH_ReconfRqst */
static int hf_nbap_uL_Timeslot_InformationAddList_PSCH_ReconfRqst = -1;  /* UL_Timeslot_InformationAddList_PSCH_ReconfRqst */
static int hf_nbap_UL_Timeslot_InformationAddList_PSCH_ReconfRqst_item = -1;  /* UL_Timeslot_InformationAddItem_PSCH_ReconfRqst */
static int hf_nbap_uL_Code_InformationAddList_PSCH_ReconfRqst = -1;  /* UL_Code_InformationAddList_PSCH_ReconfRqst */
static int hf_nbap_UL_Code_InformationAddList_PSCH_ReconfRqst_item = -1;  /* UL_Code_InformationAddItem_PSCH_ReconfRqst */
static int hf_nbap_uL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst = -1;  /* UL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst */
static int hf_nbap_UL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst_item = -1;  /* UL_Timeslot_InformationAddItem_LCR_PSCH_ReconfRqst */
static int hf_nbap_uL_Code_InformationAddList_LCR_PSCH_ReconfRqst = -1;  /* UL_Code_InformationAddList_LCR_PSCH_ReconfRqst */
static int hf_nbap_UL_Code_InformationAddList_LCR_PSCH_ReconfRqst_item = -1;  /* UL_Code_InformationAddItem_LCR_PSCH_ReconfRqst */
static int hf_nbap_PUSCHSets_ModifyList_PSCH_ReconfRqst_item = -1;  /* PUSCHSets_ModifyItem_PSCH_ReconfRqst */
static int hf_nbap_pUSCH_InformationList1 = -1;   /* PUSCH_Information_ModifyList_PSCH_ReconfRqst */
static int hf_nbap_uL_Timeslot_InformationModifyList_PSCH_ReconfRqst = -1;  /* UL_Timeslot_InformationModifyList_PSCH_ReconfRqst */
static int hf_nbap_UL_Timeslot_InformationModifyList_PSCH_ReconfRqst_item = -1;  /* UL_Timeslot_InformationModifyItem_PSCH_ReconfRqst */
static int hf_nbap_uL_Code_InformationModifyList_PSCH_ReconfRqst = -1;  /* UL_Code_InformationModifyList_PSCH_ReconfRqst */
static int hf_nbap_UL_Code_InformationModifyList_PSCH_ReconfRqst_item = -1;  /* UL_Code_InformationModifyItem_PSCH_ReconfRqst */
static int hf_nbap_uL_Timeslot_InformationModifyList_LCR_PSCH_ReconfRqst = -1;  /* UL_Timeslot_LCR_InformationModifyList_PSCH_ReconfRqst */
static int hf_nbap_UL_Timeslot_LCR_InformationModifyList_PSCH_ReconfRqst_item = -1;  /* UL_Timeslot_LCR_InformationModifyItem_PSCH_ReconfRqst */
static int hf_nbap_uL_Code_LCR_InformationModifyList_PSCH_ReconfRqst = -1;  /* UL_Code_LCR_InformationModifyList_PSCH_ReconfRqst */
static int hf_nbap_UL_Code_LCR_InformationModifyList_PSCH_ReconfRqst_item = -1;  /* UL_Code_LCR_InformationModifyItem_PSCH_ReconfRqst */
static int hf_nbap_PUSCHSets_DeleteList_PSCH_ReconfRqst_item = -1;  /* PUSCHSets_DeleteItem_PSCH_ReconfRqst */
static int hf_nbap_dL_HS_PDSCH_Timeslot_Information_PSCH_ReconfRqst = -1;  /* DL_HS_PDSCH_Timeslot_Information_PSCH_ReconfRqst */
static int hf_nbap_dL_HS_PDSCH_Timeslot_Information_LCR_PSCH_ReconfRqst = -1;  /* DL_HS_PDSCH_Timeslot_Information_LCR_PSCH_ReconfRqst */
static int hf_nbap_DL_HS_PDSCH_Timeslot_Information_PSCH_ReconfRqst_item = -1;  /* DL_HS_PDSCH_Timeslot_InformationItem_PSCH_ReconfRqst */
static int hf_nbap_dl_HS_PDSCH_Codelist_PSCH_ReconfRqst = -1;  /* DL_HS_PDSCH_Codelist_PSCH_ReconfRqst */
static int hf_nbap_maxHSDSCH_HSSCCH_Power = -1;   /* MaximumTransmissionPower */
static int hf_nbap_DL_HS_PDSCH_Codelist_PSCH_ReconfRqst_item = -1;  /* TDD_ChannelisationCode */
static int hf_nbap_DL_HS_PDSCH_Timeslot_Information_LCR_PSCH_ReconfRqst_item = -1;  /* DL_HS_PDSCH_Timeslot_InformationItem_LCR_PSCH_ReconfRqst */
static int hf_nbap_timeSlot1 = -1;                /* TimeSlotLCR */
static int hf_nbap_midambleShiftAndBurstType1 = -1;  /* MidambleShiftLCR */
static int hf_nbap_dl_HS_PDSCH_Codelist_LCR_PSCH_ReconfRqst = -1;  /* DL_HS_PDSCH_Codelist_LCR_PSCH_ReconfRqst */
static int hf_nbap_DL_HS_PDSCH_Codelist_LCR_PSCH_ReconfRqst_item = -1;  /* TDD_ChannelisationCode */
static int hf_nbap_hS_SCCH_Information_PSCH_ReconfRqst = -1;  /* HS_SCCH_Information_PSCH_ReconfRqst */
static int hf_nbap_hS_SCCH_Information_LCR_PSCH_ReconfRqst = -1;  /* HS_SCCH_Information_LCR_PSCH_ReconfRqst */
static int hf_nbap_HS_SCCH_Information_PSCH_ReconfRqst_item = -1;  /* HS_SCCH_InformationItem_PSCH_ReconfRqst */
static int hf_nbap_hS_SCCH_ID = -1;               /* HS_SCCH_ID */
static int hf_nbap_hS_SCCH_MaxPower = -1;         /* DL_Power */
static int hf_nbap_hS_SICH_Information = -1;      /* HS_SICH_Information_PSCH_ReconfRqst */
static int hf_nbap_hsSICH_ID = -1;                /* HS_SICH_ID */
static int hf_nbap_HS_SCCH_Information_LCR_PSCH_ReconfRqst_item = -1;  /* HS_SCCH_InformationItem_LCR_PSCH_ReconfRqst */
static int hf_nbap_first_TDD_ChannelisationCode = -1;  /* TDD_ChannelisationCode */
static int hf_nbap_second_TDD_ChannelisationCode = -1;  /* TDD_ChannelisationCode */
static int hf_nbap_hS_SICH_Information_LCR = -1;  /* HS_SICH_Information_LCR_PSCH_ReconfRqst */
static int hf_nbap_hS_SCCH_InformationModify_PSCH_ReconfRqst = -1;  /* HS_SCCH_InformationModify_PSCH_ReconfRqst */
static int hf_nbap_hS_SCCH_InformationModify_LCR_PSCH_ReconfRqst = -1;  /* HS_SCCH_InformationModify_LCR_PSCH_ReconfRqst */
static int hf_nbap_HS_SCCH_InformationModify_PSCH_ReconfRqst_item = -1;  /* HS_SCCH_InformationModifyItem_PSCH_ReconfRqst */
static int hf_nbap_hS_SICH_Information1 = -1;     /* HS_SICH_InformationModify_PSCH_ReconfRqst */
static int hf_nbap_HS_SCCH_InformationModify_LCR_PSCH_ReconfRqst_item = -1;  /* HS_SCCH_InformationModifyItem_LCR_PSCH_ReconfRqst */
static int hf_nbap_hS_SICH_Information_LCR1 = -1;  /* HS_SICH_InformationModify_LCR_PSCH_ReconfRqst */
static int hf_nbap_Delete_From_HS_SCCH_Resource_Pool_PSCH_ReconfRqst_item = -1;  /* Delete_From_HS_SCCH_Resource_PoolItem_PSCH_ReconfRqst */
static int hf_nbap_generalCause5 = -1;            /* GeneralCauseList_PSCH_ReconfFailure */
static int hf_nbap_setSpecificCause = -1;         /* SetSpecificCauseList_PSCH_ReconfFailureTDD */
static int hf_nbap_unsuccessful_PDSCHSetList_PSCH_ReconfFailureTDD = -1;  /* Unsuccessful_PDSCHSetList_PSCH_ReconfFailureTDD */
static int hf_nbap_unsuccessful_PUSCHSetList_PSCH_ReconfFailureTDD = -1;  /* Unsuccessful_PUSCHSetList_PSCH_ReconfFailureTDD */
static int hf_nbap_Unsuccessful_PDSCHSetList_PSCH_ReconfFailureTDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_Unsuccessful_PUSCHSetList_PSCH_ReconfFailureTDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_communicationContext = -1;     /* CommunicationContextList_Reset */
static int hf_nbap_communicationControlPort = -1;  /* CommunicationControlPortList_Reset */
static int hf_nbap_nodeB = -1;                    /* NULL */
static int hf_nbap_communicationContextInfoList_Reset = -1;  /* CommunicationContextInfoList_Reset */
static int hf_nbap_CommunicationContextInfoList_Reset_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_communicationContextType_Reset = -1;  /* CommunicationContextType_Reset */
static int hf_nbap_cRNC_CommunicationContextID = -1;  /* CRNC_CommunicationContextID */
static int hf_nbap_nodeB_CommunicationContextID = -1;  /* NodeB_CommunicationContextID */
static int hf_nbap_communicationControlPortInfoList_Reset = -1;  /* CommunicationControlPortInfoList_Reset */
static int hf_nbap_CommunicationControlPortInfoList_Reset_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_cell2 = -1;                    /* Cell_InfEx_Rqst */
static int hf_nbap_cell3 = -1;                    /* Cell_InfEx_Rsp */
static int hf_nbap_requestedDataValue = -1;       /* RequestedDataValue */
static int hf_nbap_cell4 = -1;                    /* Cell_Inf_Rprt */
static int hf_nbap_requestedDataValueInformation = -1;  /* RequestedDataValueInformation */
static int hf_nbap_cSBTransmissionID = -1;        /* CSBTransmissionID */
static int hf_nbap_sfn = -1;                      /* SFN */
static int hf_nbap_cellSyncBurstCode = -1;        /* CellSyncBurstCode */
static int hf_nbap_cellSyncBurstCodeShift = -1;   /* CellSyncBurstCodeShift */
static int hf_nbap_initialDLTransPower = -1;      /* DL_Power */
static int hf_nbap_TimeslotInfo_CellSyncInitiationRqstTDD_item = -1;  /* TimeSlot */
static int hf_nbap_cSBMeasurementID = -1;         /* CSBMeasurementID */
static int hf_nbap_synchronisationReportType = -1;  /* SynchronisationReportType */
static int hf_nbap_synchronisationReportCharacteristics = -1;  /* SynchronisationReportCharacteristics */
static int hf_nbap_uARFCN = -1;                   /* UARFCN */
static int hf_nbap_sYNCDlCodeId = -1;             /* SYNCDlCodeId */
static int hf_nbap_CellSyncBurstTransReconfInfo_CellSyncReconfRqstTDD_item = -1;  /* CellSyncBurstTransInfoItem_CellSyncReconfRqstTDD */
static int hf_nbap_syncFrameNumberToTransmit = -1;  /* SyncFrameNumber */
static int hf_nbap_dlTransPower = -1;             /* DL_Power */
static int hf_nbap_cellSyncBurstMeasInfoList_CellSyncReconfRqstTDD = -1;  /* CellSyncBurstMeasInfoList_CellSyncReconfRqstTDD */
static int hf_nbap_synchronisationReportType1 = -1;  /* SynchronisationReportTypeIE */
static int hf_nbap_synchronisationReportCharacteristics1 = -1;  /* SynchronisationReportCharacteristicsIE */
static int hf_nbap_CellSyncBurstMeasInfoListIE_CellSyncReconfRqstTDD_item = -1;  /* CellSyncBurstMeasInfoItem_CellSyncReconfRqstTDD */
static int hf_nbap_syncFrameNrToReceive = -1;     /* SyncFrameNumber */
static int hf_nbap_syncBurstInfo = -1;            /* CellSyncBurstInfoList_CellSyncReconfRqstTDD */
static int hf_nbap_CellSyncBurstInfoList_CellSyncReconfRqstTDD_item = -1;  /* CellSyncBurstInfoItem_CellSyncReconfRqstTDD */
static int hf_nbap_SYNCDlCodeIdTransReconfInfoLCR_CellSyncReconfRqstTDD_item = -1;  /* SYNCDlCodeIdTransReconfItemLCR_CellSyncReconfRqstTDD */
static int hf_nbap_syncFrameNumberforTransmit = -1;  /* SyncFrameNumber */
static int hf_nbap_sYNCDlCodeIdMeasInfoList = -1;  /* SYNCDlCodeIdMeasInfoList_CellSyncReconfRqstTDD */
static int hf_nbap_SYNCDlCodeIdMeasInfoList_CellSyncReconfRqstTDD_item = -1;  /* SYNCDlCodeIdMeasInfoItem_CellSyncReconfRqstTDD */
static int hf_nbap_sYNCDlCodeIdInfoLCR = -1;      /* SYNCDlCodeIdInfoListLCR_CellSyncReconfRqstTDD */
static int hf_nbap_SYNCDlCodeIdInfoListLCR_CellSyncReconfRqstTDD_item = -1;  /* SYNCDlCodeIdInfoItemLCR_CellSyncReconfRqstTDD */
static int hf_nbap_propagationDelayCompensation = -1;  /* TimingAdjustmentValueLCR */
static int hf_nbap_CellAdjustmentInfo_SyncAdjustmentRqstTDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_frameAdjustmentValue = -1;     /* FrameAdjustmentValue */
static int hf_nbap_timingAdjustmentValue = -1;    /* TimingAdjustmentValue */
static int hf_nbap_dLTransPower = -1;             /* DL_Power */
static int hf_nbap_generalCause6 = -1;            /* GeneralCauseList_SyncAdjustmntFailureTDD */
static int hf_nbap_cellSpecificCause = -1;        /* CellSpecificCauseList_SyncAdjustmntFailureTDD */
static int hf_nbap_unsuccessful_cell_InformationRespList_SyncAdjustmntFailureTDD = -1;  /* Unsuccessful_cell_InformationRespList_SyncAdjustmntFailureTDD */
static int hf_nbap_Unsuccessful_cell_InformationRespList_SyncAdjustmntFailureTDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_CellSyncInfo_CellSyncReprtTDD_item = -1;  /* CellSyncInfoItemIE_CellSyncReprtTDD */
static int hf_nbap_c_ID_CellSyncReprtTDD = -1;    /* C_ID_IE_CellSyncReprtTDD */
static int hf_nbap_syncReportType_CellSyncReprtTDD = -1;  /* SyncReportTypeIE_CellSyncReprtTDD */
static int hf_nbap_intStdPhSyncInfo_CellSyncReprtTDD = -1;  /* IntStdPhCellSyncInfo_CellSyncReprtTDD */
static int hf_nbap_lateEntrantCell = -1;          /* NULL */
static int hf_nbap_frequencyAcquisition = -1;     /* NULL */
static int hf_nbap_cellSyncBurstMeasuredInfo = -1;  /* CellSyncBurstMeasInfoList_CellSyncReprtTDD */
static int hf_nbap_CellSyncBurstMeasInfoList_CellSyncReprtTDD_item = -1;  /* CellSyncBurstMeasInfoItem_CellSyncReprtTDD */
static int hf_nbap_sFN = -1;                      /* SFN */
static int hf_nbap_cellSyncBurstInfo_CellSyncReprtTDD = -1;  /* SEQUENCE_SIZE_1_16_OF_CellSyncBurstInfo_CellSyncReprtTDD */
static int hf_nbap_cellSyncBurstInfo_CellSyncReprtTDD_item = -1;  /* CellSyncBurstInfo_CellSyncReprtTDD */
static int hf_nbap_cellSyncBurstAvailable = -1;   /* CellSyncBurstAvailable_CellSyncReprtTDD */
static int hf_nbap_cellSyncBurstNotAvailable = -1;  /* NULL */
static int hf_nbap_cellSyncBurstTiming = -1;      /* CellSyncBurstTiming */
static int hf_nbap_cellSyncBurstSIR = -1;         /* CellSyncBurstSIR */
static int hf_nbap_SyncDLCodeIdsMeasInfoList_CellSyncReprtTDD_item = -1;  /* SyncDLCodeIdsMeasInfoItem_CellSyncReprtTDD */
static int hf_nbap_syncDLCodeIdInfo_CellSyncReprtTDD = -1;  /* SyncDLCodeIdInfo_CellSyncReprtTDD */
static int hf_nbap_SyncDLCodeIdInfo_CellSyncReprtTDD_item = -1;  /* SyncDLCodeIdItem_CellSyncReprtTDD */
static int hf_nbap_syncDLCodeIdAvailable = -1;    /* SyncDLCodeIdAvailable_CellSyncReprtTDD */
static int hf_nbap_syncDLCodeIDNotAvailable = -1;  /* NULL */
static int hf_nbap_syncDLCodeIdTiming = -1;       /* CellSyncBurstTimingLCR */
static int hf_nbap_syncDLCodeIdSIR = -1;          /* CellSyncBurstSIR */
static int hf_nbap_DCH_RearrangeList_Bearer_RearrangeInd_item = -1;  /* DCH_RearrangeItem_Bearer_RearrangeInd */
static int hf_nbap_DSCH_RearrangeList_Bearer_RearrangeInd_item = -1;  /* DSCH_RearrangeItem_Bearer_RearrangeInd */
static int hf_nbap_USCH_RearrangeList_Bearer_RearrangeInd_item = -1;  /* USCH_RearrangeItem_Bearer_RearrangeInd */
static int hf_nbap_HSDSCH_RearrangeList_Bearer_RearrangeInd_item = -1;  /* HSDSCH_RearrangeItem_Bearer_RearrangeInd */
static int hf_nbap_hsDSCH_MACdFlow_ID = -1;       /* HSDSCH_MACdFlow_ID */
static int hf_nbap_E_DCH_RearrangeList_Bearer_RearrangeInd_item = -1;  /* E_DCH_RearrangeItem_Bearer_RearrangeInd */
static int hf_nbap_e_DCH_MACdFlow_ID = -1;        /* E_DCH_MACdFlow_ID */
static int hf_nbap_DelayedActivationInformationList_RL_ActivationCmdFDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_delayed_activation_update = -1;  /* DelayedActivationUpdate */
static int hf_nbap_DelayedActivationInformationList_RL_ActivationCmdTDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_nbap_cell5 = -1;                    /* Cell_CM_Rprt */
static int hf_nbap_rACH3 = -1;                    /* RACH_CM_Rprt */
static int hf_nbap_extension_CommonMeasurementObjectType_CM_Rprt = -1;  /* Extension_CommonMeasurementObjectType_CM_Rprt */
static int hf_nbap_commonMeasurementValueInformation = -1;  /* CommonMeasurementValueInformation */
static int hf_nbap_cMConfigurationChangeCFN = -1;  /* CFN */
static int hf_nbap_transmission_Gap_Pattern_Sequence_Status = -1;  /* Transmission_Gap_Pattern_Sequence_Status_List */
static int hf_nbap_Transmission_Gap_Pattern_Sequence_Status_List_item = -1;  /* Transmission_Gap_Pattern_Sequence_Status_List_item */
static int hf_nbap_tGPSID = -1;                   /* TGPSID */
static int hf_nbap_tGPRC = -1;                    /* TGPRC */
static int hf_nbap_tGCFN = -1;                    /* CFN */
static int hf_nbap_priorityLevel = -1;            /* PriorityLevel */
static int hf_nbap_pre_emptionCapability = -1;    /* Pre_emptionCapability */
static int hf_nbap_pre_emptionVulnerability = -1;  /* Pre_emptionVulnerability */
static int hf_nbap_aOA_LCR = -1;                  /* AOA_LCR */
static int hf_nbap_aOA_LCR_Accuracy_Class = -1;   /* AOA_LCR_Accuracy_Class */
static int hf_nbap_Best_Cell_Portions_Value_item = -1;  /* Best_Cell_Portions_Item */
static int hf_nbap_sIRValue = -1;                 /* SIR_Value */
static int hf_nbap_radioNetwork = -1;             /* CauseRadioNetwork */
static int hf_nbap_transport = -1;                /* CauseTransport */
static int hf_nbap_protocol = -1;                 /* CauseProtocol */
static int hf_nbap_misc = -1;                     /* CauseMisc */
static int hf_nbap_initialPhase_0_1048575 = -1;   /* INTEGER_0_1048575_ */
static int hf_nbap_steadyStatePhase = -1;         /* INTEGER_0_255_ */
static int hf_nbap_initialPhase = -1;             /* INTEGER_0_524287_ */
static int hf_nbap_steadyStatePhase1 = -1;        /* INTEGER_0_127_ */
static int hf_nbap_CommonChannelsCapacityConsumptionLaw_item = -1;  /* CommonChannelsCapacityConsumptionLaw_item */
static int hf_nbap_dl_Cost = -1;                  /* INTEGER_0_65535 */
static int hf_nbap_ul_Cost = -1;                  /* INTEGER_0_65535 */
static int hf_nbap_tUTRANGPSMeasurementAccuracyClass = -1;  /* TUTRANGPSAccuracyClass */
static int hf_nbap_transmitted_carrier_power = -1;  /* Transmitted_Carrier_Power_Value */
static int hf_nbap_acknowledged_prach_preambles = -1;  /* Acknowledged_PRACH_preambles_Value */
static int hf_nbap_uL_TimeslotISCP = -1;          /* UL_TimeslotISCP_Value */
static int hf_nbap_notUsed_1_acknowledged_PCPCH_access_preambles = -1;  /* NULL */
static int hf_nbap_notUsed_2_detected_PCPCH_access_preambles = -1;  /* NULL */
static int hf_nbap_extension_CommonMeasurementValue = -1;  /* Extension_CommonMeasurementValue */
static int hf_nbap_measurementAvailable = -1;     /* CommonMeasurementAvailable */
static int hf_nbap_measurementnotAvailable = -1;  /* CommonMeasurementnotAvailable */
static int hf_nbap_commonmeasurementValue = -1;   /* CommonMeasurementValue */
static int hf_nbap_ie_Extensions = -1;            /* ProtocolExtensionContainer */
static int hf_nbap_bindingID = -1;                /* BindingID */
static int hf_nbap_transportLayerAddress = -1;    /* TransportLayerAddress */
static int hf_nbap_triggeringMessage = -1;        /* TriggeringMessage */
static int hf_nbap_procedureCriticality = -1;     /* Criticality */
static int hf_nbap_iEsCriticalityDiagnostics = -1;  /* CriticalityDiagnostics_IE_List */
static int hf_nbap_CriticalityDiagnostics_IE_List_item = -1;  /* CriticalityDiagnostics_IE_List_item */
static int hf_nbap_iECriticality = -1;            /* Criticality */
static int hf_nbap_iE_ID = -1;                    /* ProtocolIE_ID */
static int hf_nbap_repetitionNumber = -1;         /* RepetitionNumber0 */
static int hf_nbap_DCH_FDD_Information_item = -1;  /* DCH_FDD_InformationItem */
static int hf_nbap_payloadCRC_PresenceIndicator = -1;  /* PayloadCRC_PresenceIndicator */
static int hf_nbap_ul_FP_Mode = -1;               /* UL_FP_Mode */
static int hf_nbap_dCH_SpecificInformationList = -1;  /* DCH_Specific_FDD_InformationList */
static int hf_nbap_DCH_Specific_FDD_InformationList_item = -1;  /* DCH_Specific_FDD_Item */
static int hf_nbap_ul_TransportFormatSet = -1;    /* TransportFormatSet */
static int hf_nbap_qE_Selector = -1;              /* QE_Selector */
static int hf_nbap_DCH_InformationResponse_item = -1;  /* DCH_InformationResponseItem */
static int hf_nbap_DCH_TDD_Information_item = -1;  /* DCH_TDD_InformationItem */
static int hf_nbap_dCH_SpecificInformationList1 = -1;  /* DCH_Specific_TDD_InformationList */
static int hf_nbap_DCH_Specific_TDD_InformationList_item = -1;  /* DCH_Specific_TDD_Item */
static int hf_nbap_ul_CCTrCH_ID = -1;             /* CCTrCH_ID */
static int hf_nbap_dl_CCTrCH_ID = -1;             /* CCTrCH_ID */
static int hf_nbap_FDD_DCHs_to_Modify_item = -1;  /* FDD_DCHs_to_ModifyItem */
static int hf_nbap_dCH_SpecificInformationList2 = -1;  /* DCH_ModifySpecificInformation_FDD */
static int hf_nbap_DCH_ModifySpecificInformation_FDD_item = -1;  /* DCH_ModifySpecificItem_FDD */
static int hf_nbap_TDD_DCHs_to_Modify_item = -1;  /* DCH_ModifyItem_TDD */
static int hf_nbap_dCH_SpecificInformationList3 = -1;  /* DCH_ModifySpecificInformation_TDD */
static int hf_nbap_DCH_ModifySpecificInformation_TDD_item = -1;  /* DCH_ModifySpecificItem_TDD */
static int hf_nbap_DedicatedChannelsCapacityConsumptionLaw_item = -1;  /* DedicatedChannelsCapacityConsumptionLaw_item */
static int hf_nbap_dl_Cost_1 = -1;                /* INTEGER_0_65535 */
static int hf_nbap_dl_Cost_2 = -1;                /* INTEGER_0_65535 */
static int hf_nbap_ul_Cost_1 = -1;                /* INTEGER_0_65535 */
static int hf_nbap_ul_Cost_2 = -1;                /* INTEGER_0_65535 */
static int hf_nbap_sIR_Value = -1;                /* SIR_Value */
static int hf_nbap_sIR_ErrorValue = -1;           /* SIR_Error_Value */
static int hf_nbap_transmittedCodePowerValue = -1;  /* Transmitted_Code_Power_Value */
static int hf_nbap_rSCP = -1;                     /* RSCP_Value */
static int hf_nbap_rxTimingDeviationValue = -1;   /* Rx_Timing_Deviation_Value */
static int hf_nbap_roundTripTime = -1;            /* Round_Trip_Time_Value */
static int hf_nbap_extension_DedicatedMeasurementValue = -1;  /* Extension_DedicatedMeasurementValue */
static int hf_nbap_measurementAvailable1 = -1;    /* DedicatedMeasurementAvailable */
static int hf_nbap_measurementnotAvailable1 = -1;  /* DedicatedMeasurementnotAvailable */
static int hf_nbap_dedicatedmeasurementValue = -1;  /* DedicatedMeasurementValue */
static int hf_nbap_cfn = -1;                      /* CFN */
static int hf_nbap_separate_indication = -1;      /* NULL */
static int hf_nbap_activate = -1;                 /* Activate_Info */
static int hf_nbap_deactivate = -1;               /* Deactivate_Info */
static int hf_nbap_activation_type = -1;          /* Execution_Type */
static int hf_nbap_initial_dl_tx_power = -1;      /* DL_Power */
static int hf_nbap_firstRLS_Indicator = -1;       /* FirstRLS_Indicator */
static int hf_nbap_propagation_delay = -1;        /* PropagationDelay */
static int hf_nbap_deactivation_type = -1;        /* Execution_Type */
static int hf_nbap_synchronised = -1;             /* CFN */
static int hf_nbap_unsynchronised = -1;           /* NULL */
static int hf_nbap_gpstow = -1;                   /* GPSTOW */
static int hf_nbap_status_health = -1;            /* GPS_Status_Health */
static int hf_nbap_satelliteinfo = -1;            /* SAT_Info_DGPSCorrections */
static int hf_nbap_prcdeviation = -1;             /* PRCDeviation */
static int hf_nbap_DL_Timeslot_Information_item = -1;  /* DL_Timeslot_InformationItem */
static int hf_nbap_dL_Code_Information = -1;      /* TDD_DL_Code_Information */
static int hf_nbap_DL_TimeslotLCR_Information_item = -1;  /* DL_TimeslotLCR_InformationItem */
static int hf_nbap_dL_Code_LCR_Information = -1;  /* TDD_DL_Code_LCR_Information */
static int hf_nbap_powerAdjustmentType = -1;      /* PowerAdjustmentType */
static int hf_nbap_dLReferencePower = -1;         /* DL_Power */
static int hf_nbap_dLReferencePowerList_DL_PC_Rqst = -1;  /* DL_ReferencePowerInformationList */
static int hf_nbap_maxAdjustmentStep = -1;        /* MaxAdjustmentStep */
static int hf_nbap_adjustmentPeriod = -1;         /* AdjustmentPeriod */
static int hf_nbap_adjustmentRatio = -1;          /* ScaledAdjustmentRatio */
static int hf_nbap_DL_ReferencePowerInformationList_item = -1;  /* DL_ReferencePowerInformationItem */
static int hf_nbap_dl_Reference_Power = -1;       /* DL_Power */
static int hf_nbap_DL_TimeslotISCPInfo_item = -1;  /* DL_TimeslotISCPInfoItem */
static int hf_nbap_dL_TimeslotISCP = -1;          /* DL_TimeslotISCP */
static int hf_nbap_DL_TimeslotISCPInfoLCR_item = -1;  /* DL_TimeslotISCPInfoItemLCR */
static int hf_nbap_DSCH_InformationResponse_item = -1;  /* DSCH_InformationResponseItem */
static int hf_nbap_DSCH_TDD_Information_item = -1;  /* DSCH_TDD_InformationItem */
static int hf_nbap_replace = -1;                  /* E_AGCH_FDD_Code_List */
static int hf_nbap_remove = -1;                   /* NULL */
static int hf_nbap_E_AGCH_FDD_Code_List_item = -1;  /* FDD_DL_ChannelisationCodeNumber */
static int hf_nbap_e_AGCH_And_E_RGCH_E_HICH_FDD_Scrambling_Code = -1;  /* DL_ScramblingCode */
static int hf_nbap_e_AGCH_Channelisation_Code = -1;  /* FDD_DL_ChannelisationCodeNumber */
static int hf_nbap_primary_e_RNTI = -1;           /* E_RNTI */
static int hf_nbap_secondary_e_RNTI = -1;         /* E_RNTI */
static int hf_nbap_e_RGCH_E_HICH_Channelisation_Code = -1;  /* FDD_DL_ChannelisationCodeNumber */
static int hf_nbap_e_RGCH_Signature_Sequence = -1;  /* E_RGCH_Signature_Sequence */
static int hf_nbap_e_HICH_Signature_Sequence = -1;  /* E_HICH_Signature_Sequence */
static int hf_nbap_serving_Grant_Value = -1;      /* E_Serving_Grant_Value */
static int hf_nbap_primary_Secondary_Grant_Selector = -1;  /* E_Primary_Secondary_Grant_Selector */
static int hf_nbap_e_RGCH_Release_Indicator = -1;  /* E_RGCH_Release_Indicator */
static int hf_nbap_e_DCH_MACdFlows_Information = -1;  /* E_DCH_MACdFlows_Information */
static int hf_nbap_e_DCH_MACdFlow_Specific_InformationResp = -1;  /* E_DCH_MACdFlow_Specific_InformationResp */
static int hf_nbap_e_DCH_MACdFlow_Specific_Info_to_Modify = -1;  /* E_DCH_MACdFlow_Specific_InfoList_to_Modify */
static int hf_nbap_e_DCH_Non_Scheduled_Transmission_Grant = -1;  /* E_DCH_Non_Scheduled_Transmission_Grant_Items */
static int hf_nbap_e_DCH_Scheduled_Transmission_Grant = -1;  /* NULL */
static int hf_nbap_E_DCH_LogicalChannelInformation_item = -1;  /* E_DCH_LogicalChannelInformationItem */
static int hf_nbap_logicalChannelId = -1;         /* LogicalChannelID */
static int hf_nbap_schedulingPriorityIndicator = -1;  /* SchedulingPriorityIndicator */
static int hf_nbap_mACesGuaranteedBitRate = -1;   /* MACesGuaranteedBitRate */
static int hf_nbap_e_DCH_DDI_Value = -1;          /* E_DCH_DDI_Value */
static int hf_nbap_mACd_PDU_Size_List = -1;       /* E_DCH_MACdPDU_SizeList */
static int hf_nbap_E_DCH_MACdPDU_SizeList_item = -1;  /* E_DCH_MACdPDU_SizeListItem */
static int hf_nbap_mACdPDU_Size = -1;             /* MACdPDU_Size */
static int hf_nbap_E_DCH_LogicalChannelToModify_item = -1;  /* E_DCH_LogicalChannelToModifyItem */
static int hf_nbap_mACd_PDU_Size_List1 = -1;      /* E_DCH_MACdPDU_SizeToModifyList */
static int hf_nbap_E_DCH_MACdPDU_SizeToModifyList_item = -1;  /* E_DCH_MACdPDU_SizeListItem */
static int hf_nbap_E_DCH_LogicalChannelToDelete_item = -1;  /* E_DCH_LogicalChannelToDeleteItem */
static int hf_nbap_e_DCH_MACdFlow_Specific_Info = -1;  /* E_DCH_MACdFlow_Specific_InfoList */
static int hf_nbap_E_DCH_MACdFlow_Specific_InfoList_item = -1;  /* E_DCH_MACdFlow_Specific_InfoItem */
static int hf_nbap_tnlQos = -1;                   /* TnlQos */
static int hf_nbap_maximum_Number_of_Retransmissions_For_E_DCH = -1;  /* Maximum_Number_of_Retransmissions_For_E_DCH */
static int hf_nbap_eDCH_HARQ_PO_FDD = -1;         /* E_DCH_HARQ_PO_FDD */
static int hf_nbap_eDCH_MACdFlow_Multiplexing_List = -1;  /* E_DCH_MACdFlow_Multiplexing_List */
static int hf_nbap_eDCH_Grant_Type_Information = -1;  /* E_DCH_Grant_Type_Information */
static int hf_nbap_bundlingModeIndicator = -1;    /* BundlingModeIndicator */
static int hf_nbap_eDCHLogicalChannelInformation = -1;  /* E_DCH_LogicalChannelInformation */
static int hf_nbap_E_DCH_MACdFlow_Specific_InformationResp_item = -1;  /* E_DCH_MACdFlow_Specific_InformationResp_Item */
static int hf_nbap_E_DCH_MACdFlow_Specific_InfoList_to_Modify_item = -1;  /* E_DCH_MACdFlow_Specific_InfoItem_to_Modify */
static int hf_nbap_eDCH_LogicalChannelToAdd = -1;  /* E_DCH_LogicalChannelInformation */
static int hf_nbap_eDCH_LogicalChannelToModify = -1;  /* E_DCH_LogicalChannelToModify */
static int hf_nbap_eDCH_LogicalChannelToDelete = -1;  /* E_DCH_LogicalChannelToDelete */
static int hf_nbap_E_DCH_MACdFlows_to_Delete_item = -1;  /* E_DCH_MACdFlow_to_Delete_Item */
static int hf_nbap_maxBits_MACe_PDU_non_scheduled = -1;  /* Max_Bits_MACe_PDU_non_scheduled */
static int hf_nbap_hARQ_Process_Allocation_2ms = -1;  /* HARQ_Process_Allocation_2ms_EDCH */
static int hf_nbap_replace1 = -1;                 /* E_RGCH_E_HICH_FDD_Code_List */
static int hf_nbap_E_RGCH_E_HICH_FDD_Code_List_item = -1;  /* FDD_DL_ChannelisationCodeNumber */
static int hf_nbap_e_DCH_TFCI_Table_Index = -1;   /* E_DCH_TFCI_Table_Index */
static int hf_nbap_e_DCH_Min_Set_E_TFCI = -1;     /* E_TFCI */
static int hf_nbap_reference_E_TFCI_Information = -1;  /* Reference_E_TFCI_Information */
static int hf_nbap_E_DCHProvidedBitRate_item = -1;  /* E_DCHProvidedBitRate_Item */
static int hf_nbap_e_DCHProvidedBitRateValue = -1;  /* E_DCHProvidedBitRateValue */
static int hf_nbap_FDD_DL_CodeInformation_item = -1;  /* FDD_DL_CodeInformationItem */
static int hf_nbap_transmissionGapPatternSequenceCodeInformation = -1;  /* TransmissionGapPatternSequenceCodeInformation */
static int hf_nbap_wna_alm = -1;                  /* BIT_STRING_SIZE_8 */
static int hf_nbap_sat_info_almanac = -1;         /* SAT_Info_Almanac */
static int hf_nbap_sVGlobalHealth_alm = -1;       /* BIT_STRING_SIZE_364 */
static int hf_nbap_alpha_zero_ionos = -1;         /* BIT_STRING_SIZE_8 */
static int hf_nbap_alpha_one_ionos = -1;          /* BIT_STRING_SIZE_8 */
static int hf_nbap_alpha_two_ionos = -1;          /* BIT_STRING_SIZE_8 */
static int hf_nbap_alpha_three_ionos = -1;        /* BIT_STRING_SIZE_8 */
static int hf_nbap_beta_zero_ionos = -1;          /* BIT_STRING_SIZE_8 */
static int hf_nbap_beta_one_ionos = -1;           /* BIT_STRING_SIZE_8 */
static int hf_nbap_beta_two_ionos = -1;           /* BIT_STRING_SIZE_8 */
static int hf_nbap_beta_three_ionos = -1;         /* BIT_STRING_SIZE_8 */
static int hf_nbap_GPS_Information_item = -1;     /* GPS_Information_Item */
static int hf_nbap_bad_satellites = -1;           /* GPSBadSat_Info_RealTime_Integrity */
static int hf_nbap_no_bad_satellites = -1;        /* NULL */
static int hf_nbap_sat_info = -1;                 /* SATInfo_RealTime_Integrity */
static int hf_nbap_GPS_NavigationModel_and_TimeRecovery_item = -1;  /* GPS_NavandRecovery_Item */
static int hf_nbap_tx_tow_nav = -1;               /* INTEGER_0_1048575 */
static int hf_nbap_sat_id_nav = -1;               /* SAT_ID */
static int hf_nbap_tlm_message_nav = -1;          /* BIT_STRING_SIZE_14 */
static int hf_nbap_tlm_revd_c_nav = -1;           /* BIT_STRING_SIZE_2 */
static int hf_nbap_ho_word_nav = -1;              /* BIT_STRING_SIZE_22 */
static int hf_nbap_w_n_nav = -1;                  /* BIT_STRING_SIZE_10 */
static int hf_nbap_ca_or_p_on_l2_nav = -1;        /* BIT_STRING_SIZE_2 */
static int hf_nbap_user_range_accuracy_index_nav = -1;  /* BIT_STRING_SIZE_4 */
static int hf_nbap_sv_health_nav = -1;            /* BIT_STRING_SIZE_6 */
static int hf_nbap_iodc_nav = -1;                 /* BIT_STRING_SIZE_10 */
static int hf_nbap_l2_p_dataflag_nav = -1;        /* BIT_STRING_SIZE_1 */
static int hf_nbap_sf1_reserved_nav = -1;         /* BIT_STRING_SIZE_87 */
static int hf_nbap_t_gd_nav = -1;                 /* BIT_STRING_SIZE_8 */
static int hf_nbap_t_oc_nav = -1;                 /* BIT_STRING_SIZE_16 */
static int hf_nbap_a_f_2_nav = -1;                /* BIT_STRING_SIZE_8 */
static int hf_nbap_a_f_1_nav = -1;                /* BIT_STRING_SIZE_16 */
static int hf_nbap_a_f_zero_nav = -1;             /* BIT_STRING_SIZE_22 */
static int hf_nbap_c_rs_nav = -1;                 /* BIT_STRING_SIZE_16 */
static int hf_nbap_delta_n_nav = -1;              /* BIT_STRING_SIZE_16 */
static int hf_nbap_m_zero_nav = -1;               /* BIT_STRING_SIZE_32 */
static int hf_nbap_c_uc_nav = -1;                 /* BIT_STRING_SIZE_16 */
static int hf_nbap_gps_e_nav = -1;                /* BIT_STRING_SIZE_32 */
static int hf_nbap_c_us_nav = -1;                 /* BIT_STRING_SIZE_16 */
static int hf_nbap_a_sqrt_nav = -1;               /* BIT_STRING_SIZE_32 */
static int hf_nbap_t_oe_nav = -1;                 /* BIT_STRING_SIZE_16 */
static int hf_nbap_fit_interval_flag_nav = -1;    /* BIT_STRING_SIZE_1 */
static int hf_nbap_aodo_nav = -1;                 /* BIT_STRING_SIZE_5 */
static int hf_nbap_c_ic_nav = -1;                 /* BIT_STRING_SIZE_16 */
static int hf_nbap_omega_zero_nav = -1;           /* BIT_STRING_SIZE_32 */
static int hf_nbap_c_is_nav = -1;                 /* BIT_STRING_SIZE_16 */
static int hf_nbap_i_zero_nav = -1;               /* BIT_STRING_SIZE_32 */
static int hf_nbap_c_rc_nav = -1;                 /* BIT_STRING_SIZE_16 */
static int hf_nbap_gps_omega_nav = -1;            /* BIT_STRING_SIZE_32 */
static int hf_nbap_omegadot_nav = -1;             /* BIT_STRING_SIZE_24 */
static int hf_nbap_idot_nav = -1;                 /* BIT_STRING_SIZE_14 */
static int hf_nbap_spare_zero_fill = -1;          /* BIT_STRING_SIZE_20 */
static int hf_nbap_latitudeSign = -1;             /* T_latitudeSign */
static int hf_nbap_latitude = -1;                 /* INTEGER_0_8388607 */
static int hf_nbap_longitude = -1;                /* INTEGER_M8388608_8388607 */
static int hf_nbap_directionOfAltitude = -1;      /* T_directionOfAltitude */
static int hf_nbap_altitude = -1;                 /* INTEGER_0_32767 */
static int hf_nbap_a_one_utc = -1;                /* BIT_STRING_SIZE_24 */
static int hf_nbap_a_zero_utc = -1;               /* BIT_STRING_SIZE_32 */
static int hf_nbap_t_ot_utc = -1;                 /* BIT_STRING_SIZE_8 */
static int hf_nbap_delta_t_ls_utc = -1;           /* BIT_STRING_SIZE_8 */
static int hf_nbap_w_n_t_utc = -1;                /* BIT_STRING_SIZE_8 */
static int hf_nbap_w_n_lsf_utc = -1;              /* BIT_STRING_SIZE_8 */
static int hf_nbap_dn_utc = -1;                   /* BIT_STRING_SIZE_8 */
static int hf_nbap_delta_t_lsf_utc = -1;          /* BIT_STRING_SIZE_8 */
static int hf_nbap_implicit = -1;                 /* HARQ_MemoryPartitioning_Implicit */
static int hf_nbap_explicit = -1;                 /* HARQ_MemoryPartitioning_Explicit */
static int hf_nbap_number_of_Processes = -1;      /* INTEGER_1_8_ */
static int hf_nbap_hARQ_MemoryPartitioningList = -1;  /* HARQ_MemoryPartitioningList */
static int hf_nbap_HARQ_MemoryPartitioningList_item = -1;  /* HARQ_MemoryPartitioningItem */
static int hf_nbap_process_Memory_Size = -1;      /* T_process_Memory_Size */
static int hf_nbap_HS_DSCHProvidedBitRate_item = -1;  /* HS_DSCHProvidedBitRate_Item */
static int hf_nbap_hS_DSCHProvidedBitRateValue = -1;  /* HS_DSCHProvidedBitRateValue */
static int hf_nbap_HS_DSCHProvidedBitRateValueInformation_For_CellPortion_item = -1;  /* HS_DSCHProvidedBitRateValueInformation_For_CellPortion_Item */
static int hf_nbap_hS_DSCHProvidedBitRateValue1 = -1;  /* HS_DSCHProvidedBitRate */
static int hf_nbap_HS_DSCHRequiredPower_item = -1;  /* HS_DSCHRequiredPower_Item */
static int hf_nbap_hS_DSCHRequiredPowerValue = -1;  /* HS_DSCHRequiredPowerValue */
static int hf_nbap_hS_DSCHRequiredPowerPerUEInformation = -1;  /* HS_DSCHRequiredPowerPerUEInformation */
static int hf_nbap_HS_DSCHRequiredPowerPerUEInformation_item = -1;  /* HS_DSCHRequiredPowerPerUEInformation_Item */
static int hf_nbap_hS_DSCHRequiredPowerPerUEWeight = -1;  /* HS_DSCHRequiredPowerPerUEWeight */
static int hf_nbap_HS_DSCHRequiredPowerValueInformation_For_CellPortion_item = -1;  /* HS_DSCHRequiredPowerValueInformation_For_CellPortion_Item */
static int hf_nbap_hS_DSCHRequiredPowerValue1 = -1;  /* HS_DSCHRequiredPower */
static int hf_nbap_hSDSCH_MACdFlows_Information = -1;  /* HSDSCH_MACdFlows_Information */
static int hf_nbap_ueCapability_Info = -1;        /* UE_Capability_Information */
static int hf_nbap_mAChs_Reordering_Buffer_Size_for_RLC_UM = -1;  /* MAChsReorderingBufferSize_for_RLC_UM */
static int hf_nbap_cqiFeedback_CycleK = -1;       /* CQI_Feedback_Cycle */
static int hf_nbap_cqiRepetitionFactor = -1;      /* CQI_RepetitionFactor */
static int hf_nbap_ackNackRepetitionFactor = -1;  /* AckNack_RepetitionFactor */
static int hf_nbap_cqiPowerOffset = -1;           /* CQI_Power_Offset */
static int hf_nbap_ackPowerOffset = -1;           /* Ack_Power_Offset */
static int hf_nbap_nackPowerOffset = -1;          /* Nack_Power_Offset */
static int hf_nbap_hsscch_PowerOffset = -1;       /* HSSCCH_PowerOffset */
static int hf_nbap_measurement_Power_Offset = -1;  /* Measurement_Power_Offset */
static int hf_nbap_tDD_AckNack_Power_Offset = -1;  /* TDD_AckNack_Power_Offset */
static int hf_nbap_hsDSCH_MACdFlow_Specific_Info_to_Modify = -1;  /* HSDSCH_MACdFlow_Specific_InfoList_to_Modify */
static int hf_nbap_priorityQueueInfotoModify = -1;  /* PriorityQueue_InfoList_to_Modify */
static int hf_nbap_hSSCCHCodeChangeGrant = -1;    /* HSSCCH_Code_Change_Grant */
static int hf_nbap_tDDAckNackPowerOffset = -1;    /* TDD_AckNack_Power_Offset */
static int hf_nbap_HSDSCH_MACdFlow_Specific_InfoList_to_Modify_item = -1;  /* HSDSCH_MACdFlow_Specific_InfoItem_to_Modify */
static int hf_nbap_priorityQueueInfotoModifyUnsynchronised = -1;  /* PriorityQueue_InfoList_to_Modify_Unsynchronised */
static int hf_nbap_hsDSCH_MACdFlow_Specific_InformationResp = -1;  /* HSDSCH_MACdFlow_Specific_InformationResp */
static int hf_nbap_hsSCCH_Specific_Information_ResponseFDD = -1;  /* HSSCCH_Specific_InformationRespListFDD */
static int hf_nbap_hARQ_MemoryPartitioning = -1;  /* HARQ_MemoryPartitioning */
static int hf_nbap_hsSCCH_Specific_Information_ResponseTDD = -1;  /* HSSCCH_Specific_InformationRespListTDD */
static int hf_nbap_hsSCCH_Specific_Information_ResponseTDDLCR = -1;  /* HSSCCH_Specific_InformationRespListTDDLCR */
static int hf_nbap_HSDSCH_MACdFlow_Specific_InformationResp_item = -1;  /* HSDSCH_MACdFlow_Specific_InformationResp_Item */
static int hf_nbap_hsDSCHMacdFlow_Id = -1;        /* HSDSCH_MACdFlow_ID */
static int hf_nbap_hSDSCH_Initial_Capacity_Allocation = -1;  /* HSDSCH_Initial_Capacity_Allocation */
static int hf_nbap_hSDSCH_MACdFlow_Specific_Info = -1;  /* HSDSCH_MACdFlow_Specific_InfoList */
static int hf_nbap_priorityQueue_Info = -1;       /* PriorityQueue_InfoList */
static int hf_nbap_HSDSCH_MACdFlow_Specific_InfoList_item = -1;  /* HSDSCH_MACdFlow_Specific_InfoItem */
static int hf_nbap_HSDSCH_MACdFlows_to_Delete_item = -1;  /* HSDSCH_MACdFlows_to_Delete_Item */
static int hf_nbap_HSDSCH_Initial_Capacity_Allocation_item = -1;  /* HSDSCH_Initial_Capacity_AllocationItem */
static int hf_nbap_maximum_MACdPDU_Size = -1;     /* MACdPDU_Size */
static int hf_nbap_hSDSCH_InitialWindowSize = -1;  /* HSDSCH_InitialWindowSize */
static int hf_nbap_HSSCCH_Specific_InformationRespListFDD_item = -1;  /* HSSCCH_Codes */
static int hf_nbap_codeNumber = -1;               /* INTEGER_0_127 */
static int hf_nbap_HSSCCH_Specific_InformationRespListTDD_item = -1;  /* HSSCCH_Specific_InformationRespItemTDD */
static int hf_nbap_tDD_ChannelisationCode = -1;   /* TDD_ChannelisationCode */
static int hf_nbap_hSSICH_Info = -1;              /* HSSICH_Info */
static int hf_nbap_HSSCCH_Specific_InformationRespListTDDLCR_item = -1;  /* HSSCCH_Specific_InformationRespItemTDDLCR */
static int hf_nbap_hSSICH_InfoLCR = -1;           /* HSSICH_InfoLCR */
static int hf_nbap_failed_HS_SICH = -1;           /* HS_SICH_failed */
static int hf_nbap_missed_HS_SICH = -1;           /* HS_SICH_missed */
static int hf_nbap_total_HS_SICH = -1;            /* HS_SICH_total */
static int hf_nbap_number_of_HS_PDSCH_codes = -1;  /* INTEGER_0_15 */
static int hf_nbap_hS_PDSCH_Start_code_number = -1;  /* HS_PDSCH_Start_code_number */
static int hf_nbap_replace2 = -1;                 /* HS_SCCH_FDD_Code_List */
static int hf_nbap_HS_SCCH_FDD_Code_List_item = -1;  /* HS_SCCH_FDD_Code_Information_Item */
static int hf_nbap_hsSCCHCodeChangeIndicator = -1;  /* HSSCCH_CodeChangeIndicator */
static int hf_nbap_onDemand = -1;                 /* NULL */
static int hf_nbap_periodic = -1;                 /* InformationReportCharacteristicsType_ReportPeriodicity */
static int hf_nbap_onModification = -1;           /* InformationReportCharacteristicsType_OnModification */
static int hf_nbap_min = -1;                      /* ReportPeriodicity_Scaledmin */
static int hf_nbap_hours = -1;                    /* ReportPeriodicity_Scaledhour */
static int hf_nbap_information_thresholds = -1;   /* InformationThresholds */
static int hf_nbap_dgps = -1;                     /* DGPSThresholds */
static int hf_nbap_information_Type_Item = -1;    /* Information_Type_Item */
static int hf_nbap_gPSInformation = -1;           /* GPS_Information */
static int hf_nbap_iP_SpacingFDD = -1;            /* T_iP_SpacingFDD */
static int hf_nbap_iP_Length = -1;                /* T_iP_Length */
static int hf_nbap_seed = -1;                     /* INTEGER_0_63 */
static int hf_nbap_burstModeParams = -1;          /* BurstModeParams */
static int hf_nbap_iP_Offset = -1;                /* INTEGER_0_9 */
static int hf_nbap_iP_SpacingTDD = -1;            /* T_iP_SpacingTDD */
static int hf_nbap_iP_Start = -1;                 /* INTEGER_0_4095 */
static int hf_nbap_iP_Slot = -1;                  /* INTEGER_0_14 */
static int hf_nbap_iP_PCCPCH = -1;                /* T_iP_PCCPCH */
static int hf_nbap_iP_SpacingTDD1 = -1;           /* T_iP_SpacingTDD1 */
static int hf_nbap_iP_Sub = -1;                   /* T_iP_Sub */
static int hf_nbap_burstStart = -1;               /* INTEGER_0_15 */
static int hf_nbap_burstLength = -1;              /* INTEGER_10_25 */
static int hf_nbap_burstFreq = -1;                /* INTEGER_1_16 */
static int hf_nbap_MACdPDU_Size_Indexlist_item = -1;  /* MACdPDU_Size_IndexItem */
static int hf_nbap_sID = -1;                      /* SID */
static int hf_nbap_macdPDU_Size = -1;             /* MACdPDU_Size */
static int hf_nbap_MACdPDU_Size_Indexlist_to_Modify_item = -1;  /* MACdPDU_Size_IndexItem_to_Modify */
static int hf_nbap_MessageStructure_item = -1;    /* MessageStructure_item */
static int hf_nbap_repetitionNumber1 = -1;        /* RepetitionNumber1 */
static int hf_nbap_type1 = -1;                    /* T_type1 */
static int hf_nbap_midambleConfigurationBurstType1And3 = -1;  /* MidambleConfigurationBurstType1And3 */
static int hf_nbap_midambleAllocationMode = -1;   /* T_midambleAllocationMode */
static int hf_nbap_defaultMidamble = -1;          /* NULL */
static int hf_nbap_commonMidamble = -1;           /* NULL */
static int hf_nbap_ueSpecificMidamble = -1;       /* MidambleShiftLong */
static int hf_nbap_type2 = -1;                    /* T_type2 */
static int hf_nbap_midambleConfigurationBurstType2 = -1;  /* MidambleConfigurationBurstType2 */
static int hf_nbap_midambleAllocationMode1 = -1;  /* T_midambleAllocationMode1 */
static int hf_nbap_ueSpecificMidamble1 = -1;      /* MidambleShiftShort */
static int hf_nbap_type3 = -1;                    /* T_type3 */
static int hf_nbap_midambleAllocationMode2 = -1;  /* T_midambleAllocationMode2 */
static int hf_nbap_midambleAllocationMode3 = -1;  /* MidambleAllocationMode */
static int hf_nbap_midambleShift = -1;            /* MidambleShiftLong */
static int hf_nbap_midambleConfigurationLCR = -1;  /* MidambleConfigurationLCR */
static int hf_nbap_addPriorityQueue = -1;         /* PriorityQueue_InfoItem_to_Add */
static int hf_nbap_modifyPriorityQueue = -1;      /* PriorityQueue_InfoItem_to_Modify */
static int hf_nbap_deletePriorityQueue = -1;      /* PriorityQueue_Id */
static int hf_nbap_NeighbouringCellMeasurementInformation_item = -1;  /* NeighbouringCellMeasurementInformation_item */
static int hf_nbap_neighbouringFDDCellMeasurementInformation = -1;  /* NeighbouringFDDCellMeasurementInformation */
static int hf_nbap_neighbouringTDDCellMeasurementInformation = -1;  /* NeighbouringTDDCellMeasurementInformation */
static int hf_nbap_extension_neighbouringCellMeasurementInformation = -1;  /* Extension_neighbouringCellMeasurementInformation */
static int hf_nbap_uC_Id = -1;                    /* UC_Id */
static int hf_nbap_primaryScramblingCode = -1;    /* PrimaryScramblingCode */
static int hf_nbap_cellParameterID = -1;          /* CellParameterID */
static int hf_nbap_NI_Information_item = -1;      /* Notification_Indicator */
static int hf_nbap_PriorityQueue_InfoList_item = -1;  /* PriorityQueue_InfoItem */
static int hf_nbap_priorityQueueId = -1;          /* PriorityQueue_Id */
static int hf_nbap_associatedHSDSCH_MACdFlow = -1;  /* HSDSCH_MACdFlow_ID */
static int hf_nbap_t1 = -1;                       /* T1 */
static int hf_nbap_discardTimer = -1;             /* DiscardTimer */
static int hf_nbap_mAC_hsWindowSize = -1;         /* MAC_hsWindowSize */
static int hf_nbap_mAChsGuaranteedBitRate = -1;   /* MAChsGuaranteedBitRate */
static int hf_nbap_macdPDU_Size_Index = -1;       /* MACdPDU_Size_Indexlist */
static int hf_nbap_rLC_Mode = -1;                 /* RLC_Mode */
static int hf_nbap_PriorityQueue_InfoList_to_Modify_item = -1;  /* ModifyPriorityQueue */
static int hf_nbap_macdPDU_Size_Index_to_Modify = -1;  /* MACdPDU_Size_Indexlist_to_Modify */
static int hf_nbap_PriorityQueue_InfoList_to_Modify_Unsynchronised_item = -1;  /* PriorityQueue_InfoItem_to_Modify_Unsynchronised */
static int hf_nbap_RL_Specific_DCH_Info_item = -1;  /* RL_Specific_DCH_Info_Item */
static int hf_nbap_dCH_id = -1;                   /* DCH_ID */
static int hf_nbap_transportlayeraddress = -1;    /* TransportLayerAddress */
static int hf_nbap_RL_Specific_E_DCH_Info_item = -1;  /* RL_Specific_E_DCH_Info_Item */
static int hf_nbap_Reference_E_TFCI_Information_item = -1;  /* Reference_E_TFCI_Information_Item */
static int hf_nbap_reference_E_TFCI = -1;         /* E_TFCI */
static int hf_nbap_reference_E_TFCI_PO = -1;      /* Reference_E_TFCI_PO */
static int hf_nbap_periodic1 = -1;                /* ReportCharacteristicsType_ReportPeriodicity */
static int hf_nbap_event_a = -1;                  /* ReportCharacteristicsType_EventA */
static int hf_nbap_event_b = -1;                  /* ReportCharacteristicsType_EventB */
static int hf_nbap_event_c = -1;                  /* ReportCharacteristicsType_EventC */
static int hf_nbap_event_d = -1;                  /* ReportCharacteristicsType_EventD */
static int hf_nbap_event_e = -1;                  /* ReportCharacteristicsType_EventE */
static int hf_nbap_event_f = -1;                  /* ReportCharacteristicsType_EventF */
static int hf_nbap_extension_ReportCharacteristics = -1;  /* Extension_ReportCharacteristics */
static int hf_nbap_measurementThreshold = -1;     /* ReportCharacteristicsType_MeasurementThreshold */
static int hf_nbap_measurementHysteresisTime = -1;  /* ReportCharacteristicsType_ScaledMeasurementHysteresisTime */
static int hf_nbap_measurementIncreaseThreshold = -1;  /* ReportCharacteristicsType_MeasurementIncreaseDecreaseThreshold */
static int hf_nbap_measurementChangeTime = -1;    /* ReportCharacteristicsType_ScaledMeasurementChangeTime */
static int hf_nbap_measurementDecreaseThreshold = -1;  /* ReportCharacteristicsType_MeasurementIncreaseDecreaseThreshold */
static int hf_nbap_measurementThreshold1 = -1;    /* ReportCharacteristicsType_MeasurementThreshold */
static int hf_nbap_measurementThreshold2 = -1;    /* ReportCharacteristicsType_MeasurementThreshold */
static int hf_nbap_reportPeriodicity = -1;        /* ReportCharacteristicsType_ReportPeriodicity */
static int hf_nbap_received_total_wide_band_power1 = -1;  /* Received_total_wide_band_power_Value_IncrDecrThres */
static int hf_nbap_uL_TimeslotISCP1 = -1;         /* UL_TimeslotISCP_Value_IncrDecrThres */
static int hf_nbap_sir = -1;                      /* SIR_Value_IncrDecrThres */
static int hf_nbap_sir_error = -1;                /* SIR_Error_Value_IncrDecrThres */
static int hf_nbap_transmitted_code_power = -1;   /* Transmitted_Code_Power_Value_IncrDecrThres */
static int hf_nbap_rscp = -1;                     /* RSCP_Value_IncrDecrThres */
static int hf_nbap_round_trip_time = -1;          /* Round_Trip_Time_IncrDecrThres */
static int hf_nbap_extension_ReportCharacteristicsType_MeasurementIncreaseDecreaseThreshold = -1;  /* Extension_ReportCharacteristicsType_MeasurementIncreaseDecreaseThreshold */
static int hf_nbap_sir1 = -1;                     /* SIR_Value */
static int hf_nbap_sir_error1 = -1;               /* SIR_Error_Value */
static int hf_nbap_transmitted_code_power1 = -1;  /* Transmitted_Code_Power_Value */
static int hf_nbap_rscp1 = -1;                    /* RSCP_Value */
static int hf_nbap_rx_timing_deviation = -1;      /* Rx_Timing_Deviation_Value */
static int hf_nbap_round_trip_time1 = -1;         /* Round_Trip_Time_Value */
static int hf_nbap_extension_ReportCharacteristicsType_MeasurementThreshold = -1;  /* Extension_ReportCharacteristicsType_MeasurementThreshold */
static int hf_nbap_msec = -1;                     /* MeasurementChangeTime_Scaledmsec */
static int hf_nbap_msec1 = -1;                    /* MeasurementHysteresisTime_Scaledmsec */
static int hf_nbap_msec2 = -1;                    /* ReportPeriodicity_Scaledmsec */
static int hf_nbap_Received_total_wide_band_power_For_CellPortion_Value_item = -1;  /* Received_total_wide_band_power_For_CellPortion_Value_Item */
static int hf_nbap_received_total_wide_band_power_value = -1;  /* Received_total_wide_band_power_Value */
static int hf_nbap_informationAvailable = -1;     /* InformationAvailable */
static int hf_nbap_informationnotAvailable = -1;  /* InformationnotAvailable */
static int hf_nbap_requesteddataValue = -1;       /* RequestedDataValue */
static int hf_nbap_dgps_corrections = -1;         /* DGPSCorrections */
static int hf_nbap_gps_navandrecovery = -1;       /* GPS_NavigationModel_and_TimeRecovery */
static int hf_nbap_gps_ionos_model = -1;          /* GPS_Ionospheric_Model */
static int hf_nbap_gps_utc_model = -1;            /* GPS_UTC_Model */
static int hf_nbap_gps_almanac = -1;              /* GPS_Almanac */
static int hf_nbap_gps_rt_integrity = -1;         /* GPS_RealTime_Integrity */
static int hf_nbap_gpsrxpos = -1;                 /* GPS_RX_POS */
static int hf_nbap_SAT_Info_Almanac_item = -1;    /* SAT_Info_Almanac_Item */
static int hf_nbap_data_id = -1;                  /* DATA_ID */
static int hf_nbap_sat_id = -1;                   /* SAT_ID */
static int hf_nbap_gps_e_alm = -1;                /* BIT_STRING_SIZE_16 */
static int hf_nbap_gps_toa_alm = -1;              /* BIT_STRING_SIZE_8 */
static int hf_nbap_gps_delta_I_alm = -1;          /* BIT_STRING_SIZE_16 */
static int hf_nbap_omegadot_alm = -1;             /* BIT_STRING_SIZE_16 */
static int hf_nbap_svhealth_alm = -1;             /* BIT_STRING_SIZE_8 */
static int hf_nbap_gps_a_sqrt_alm = -1;           /* BIT_STRING_SIZE_24 */
static int hf_nbap_omegazero_alm = -1;            /* BIT_STRING_SIZE_24 */
static int hf_nbap_m_zero_alm = -1;               /* BIT_STRING_SIZE_24 */
static int hf_nbap_gps_omega_alm = -1;            /* BIT_STRING_SIZE_24 */
static int hf_nbap_gps_af_zero_alm = -1;          /* BIT_STRING_SIZE_11 */
static int hf_nbap_gps_af_one_alm = -1;           /* BIT_STRING_SIZE_11 */
static int hf_nbap_SAT_Info_Almanac_ExtList_item = -1;  /* SAT_Info_Almanac_ExtItem */
static int hf_nbap_SAT_Info_DGPSCorrections_item = -1;  /* SAT_Info_DGPSCorrections_Item */
static int hf_nbap_iode_dgps = -1;                /* BIT_STRING_SIZE_8 */
static int hf_nbap_udre = -1;                     /* UDRE */
static int hf_nbap_prc = -1;                      /* PRC */
static int hf_nbap_range_correction_rate = -1;    /* Range_Correction_Rate */
static int hf_nbap_SATInfo_RealTime_Integrity_item = -1;  /* SAT_Info_RealTime_Integrity_Item */
static int hf_nbap_bad_sat_id = -1;               /* SAT_ID */
static int hf_nbap_new_secondary_CPICH = -1;      /* CommonPhysicalChannelID */
static int hf_nbap_secondary_CPICH_shall_not_be_used = -1;  /* NULL */
static int hf_nbap_serving_E_DCH_RL_in_this_NodeB = -1;  /* Serving_E_DCH_RL_in_this_NodeB */
static int hf_nbap_serving_E_DCH_RL_not_in_this_NodeB = -1;  /* NULL */
static int hf_nbap_sFNSFNChangeLimit = -1;        /* SFNSFNChangeLimit */
static int hf_nbap_predictedSFNSFNDeviationLimit = -1;  /* PredictedSFNSFNDeviationLimit */
static int hf_nbap_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation = -1;  /* T_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation */
static int hf_nbap_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item = -1;  /* T_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item */
static int hf_nbap_sFNSFNValue = -1;              /* SFNSFNValue */
static int hf_nbap_sFNSFNQuality = -1;            /* SFNSFNQuality */
static int hf_nbap_sFNSFNDriftRate = -1;          /* SFNSFNDriftRate */
static int hf_nbap_sFNSFNDriftRateQuality = -1;   /* SFNSFNDriftRateQuality */
static int hf_nbap_sFNSFNTimeStampInformation = -1;  /* SFNSFNTimeStampInformation */
static int hf_nbap_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation = -1;  /* T_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation */
static int hf_nbap_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item = -1;  /* T_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item */
static int hf_nbap_sFNSFNTimeStamp_FDD = -1;      /* SFN */
static int hf_nbap_sFNSFNTimeStamp_TDD = -1;      /* SFNSFNTimeStamp_TDD */
static int hf_nbap_sFNSFN_FDD = -1;               /* SFNSFN_FDD */
static int hf_nbap_sFNSFN_TDD = -1;               /* SFNSFN_TDD */
static int hf_nbap_synchronisationReportCharacteristicsType = -1;  /* SynchronisationReportCharacteristicsType */
static int hf_nbap_synchronisationReportCharactThreExc = -1;  /* SynchronisationReportCharactThreExc */
static int hf_nbap_SynchronisationReportCharactThreExc_item = -1;  /* SynchronisationReportCharactThreInfoItem */
static int hf_nbap_syncFrameNumber = -1;          /* SyncFrameNumber */
static int hf_nbap_cellSyncBurstInformation = -1;  /* SEQUENCE_SIZE_1_16_OF_SynchronisationReportCharactCellSyncBurstInfoItem */
static int hf_nbap_cellSyncBurstInformation_item = -1;  /* SynchronisationReportCharactCellSyncBurstInfoItem */
static int hf_nbap_cellSyncBurstTimingThreshold = -1;  /* CellSyncBurstTimingThreshold */
static int hf_nbap_SyncDLCodeIdThreInfoLCR_item = -1;  /* SyncDLCodeIdThreInfoList */
static int hf_nbap_syncFrameNoToReceive = -1;     /* SyncFrameNumber */
static int hf_nbap_syncDLCodeIdInfoLCR = -1;      /* SyncDLCodeInfoListLCR */
static int hf_nbap_SyncDLCodeInfoListLCR_item = -1;  /* SyncDLCodeInfoItemLCR */
static int hf_nbap_syncDLCodeId = -1;             /* SYNCDlCodeId */
static int hf_nbap_syncDLCodeIdArrivTime = -1;    /* CellSyncBurstTimingLCR */
static int hf_nbap_syncDLCodeIdTimingThre = -1;   /* CellSyncBurstTimingThreshold */
static int hf_nbap_modulation = -1;               /* Modulation */
static int hf_nbap_TDD_DL_Code_Information_item = -1;  /* TDD_DL_Code_InformationItem */
static int hf_nbap_TDD_DL_Code_LCR_Information_item = -1;  /* TDD_DL_Code_LCR_InformationItem */
static int hf_nbap_tdd_DL_DPCH_TimeSlotFormat_LCR = -1;  /* TDD_DL_DPCH_TimeSlotFormat_LCR */
static int hf_nbap_qPSK = -1;                     /* QPSK_DL_DPCH_TimeSlotFormatTDD_LCR */
static int hf_nbap_eightPSK = -1;                 /* EightPSK_DL_DPCH_TimeSlotFormatTDD_LCR */
static int hf_nbap_initialOffset = -1;            /* INTEGER_0_255 */
static int hf_nbap_noinitialOffset = -1;          /* INTEGER_0_63 */
static int hf_nbap_signalledGainFactors = -1;     /* T_signalledGainFactors */
static int hf_nbap_gainFactor = -1;               /* T_gainFactor */
static int hf_nbap_fdd = -1;                      /* T_fdd */
static int hf_nbap_betaC = -1;                    /* BetaCD */
static int hf_nbap_betaD = -1;                    /* BetaCD */
static int hf_nbap_tdd = -1;                      /* BetaCD */
static int hf_nbap_refTFCNumber = -1;             /* RefTFCNumber */
static int hf_nbap_computedGainFactors = -1;      /* RefTFCNumber */
static int hf_nbap_TDD_UL_Code_Information_item = -1;  /* TDD_UL_Code_InformationItem */
static int hf_nbap_TDD_UL_Code_LCR_Information_item = -1;  /* TDD_UL_Code_LCR_InformationItem */
static int hf_nbap_tdd_UL_DPCH_TimeSlotFormat_LCR = -1;  /* TDD_UL_DPCH_TimeSlotFormat_LCR */
static int hf_nbap_qPSK1 = -1;                    /* QPSK_UL_DPCH_TimeSlotFormatTDD_LCR */
static int hf_nbap_eightPSK1 = -1;                /* EightPSK_UL_DPCH_TimeSlotFormatTDD_LCR */
static int hf_nbap_tFCI_SignallingOption = -1;    /* TFCI_SignallingMode_TFCI_SignallingOption */
static int hf_nbap_not_Used_splitType = -1;       /* NULL */
static int hf_nbap_not_Used_lengthOfTFCI2 = -1;   /* NULL */
static int hf_nbap_initialPhase1 = -1;            /* INTEGER_0_1048575_ */
static int hf_nbap_dsField = -1;                  /* DsField */
static int hf_nbap_genericTrafficCategory = -1;   /* GenericTrafficCategory */
static int hf_nbap_Transmission_Gap_Pattern_Sequence_Information_item = -1;  /* Transmission_Gap_Pattern_Sequence_Information_item */
static int hf_nbap_tGSN = -1;                     /* TGSN */
static int hf_nbap_tGL1 = -1;                     /* GapLength */
static int hf_nbap_tGL2 = -1;                     /* GapLength */
static int hf_nbap_tGD = -1;                      /* TGD */
static int hf_nbap_tGPL1 = -1;                    /* GapDuration */
static int hf_nbap_not_to_be_used_1 = -1;         /* GapDuration */
static int hf_nbap_uL_DL_mode = -1;               /* UL_DL_mode */
static int hf_nbap_downlink_Compressed_Mode_Method = -1;  /* Downlink_Compressed_Mode_Method */
static int hf_nbap_uplink_Compressed_Mode_Method = -1;  /* Uplink_Compressed_Mode_Method */
static int hf_nbap_dL_FrameType = -1;             /* DL_FrameType */
static int hf_nbap_delta_SIR1 = -1;               /* DeltaSIR */
static int hf_nbap_delta_SIR_after1 = -1;         /* DeltaSIR */
static int hf_nbap_delta_SIR2 = -1;               /* DeltaSIR */
static int hf_nbap_delta_SIR_after2 = -1;         /* DeltaSIR */
static int hf_nbap_TransmittedCarrierPowerOfAllCodesNotUsedForHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHOrE_HICHTransmissionCellPortionValue_item = -1;  /* TransmittedCarrierPowerOfAllCodesNotUsedForHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHOrE_HICHTransmissionCellPortionValue_Item */
static int hf_nbap_transmittedCarrierPowerOfAllCodesNotUsedForHSTransmissionValue = -1;  /* TransmittedCarrierPowerOfAllCodesNotUsedForHSTransmissionValue */
static int hf_nbap_Transmitted_Carrier_Power_For_CellPortion_Value_item = -1;  /* Transmitted_Carrier_Power_For_CellPortion_Value_Item */
static int hf_nbap_transmitted_Carrier_Power_Value = -1;  /* Transmitted_Carrier_Power_Value */
static int hf_nbap_tFCSvalues = -1;               /* T_tFCSvalues */
static int hf_nbap_no_Split_in_TFCI = -1;         /* TFCS_TFCSList */
static int hf_nbap_not_Used_split_in_TFCI = -1;   /* NULL */
static int hf_nbap_TFCS_TFCSList_item = -1;       /* TFCS_TFCSList_item */
static int hf_nbap_cTFC = -1;                     /* TFCS_CTFC */
static int hf_nbap_tFC_Beta = -1;                 /* TransportFormatCombination_Beta */
static int hf_nbap_ctfc2bit = -1;                 /* INTEGER_0_3 */
static int hf_nbap_ctfc4bit = -1;                 /* INTEGER_0_15 */
static int hf_nbap_ctfc6bit = -1;                 /* INTEGER_0_63 */
static int hf_nbap_ctfc8bit = -1;                 /* INTEGER_0_255 */
static int hf_nbap_ctfc12bit = -1;                /* INTEGER_0_4095 */
static int hf_nbap_ctfc16bit = -1;                /* INTEGER_0_65535 */
static int hf_nbap_ctfcmaxbit = -1;               /* INTEGER_0_16777215 */
static int hf_nbap_dynamicParts = -1;             /* TransportFormatSet_DynamicPartList */
static int hf_nbap_semi_staticPart = -1;          /* TransportFormatSet_Semi_staticPart */
static int hf_nbap_TransportFormatSet_DynamicPartList_item = -1;  /* TransportFormatSet_DynamicPartList_item */
static int hf_nbap_nrOfTransportBlocks = -1;      /* TransportFormatSet_NrOfTransportBlocks */
static int hf_nbap_transportBlockSize = -1;       /* TransportFormatSet_TransportBlockSize */
static int hf_nbap_mode = -1;                     /* TransportFormatSet_ModeDP */
static int hf_nbap_transmissionTimeIntervalInformation = -1;  /* TransmissionTimeIntervalInformation */
static int hf_nbap_TransmissionTimeIntervalInformation_item = -1;  /* TransmissionTimeIntervalInformation_item */
static int hf_nbap_transmissionTimeInterval = -1;  /* TransportFormatSet_TransmissionTimeIntervalDynamic */
static int hf_nbap_transmissionTimeInterval1 = -1;  /* TransportFormatSet_TransmissionTimeIntervalSemiStatic */
static int hf_nbap_channelCoding = -1;            /* TransportFormatSet_ChannelCodingType */
static int hf_nbap_codingRate = -1;               /* TransportFormatSet_CodingRate */
static int hf_nbap_rateMatchingAttribute = -1;    /* TransportFormatSet_RateMatchingAttribute */
static int hf_nbap_cRC_Size = -1;                 /* TransportFormatSet_CRC_Size */
static int hf_nbap_mode1 = -1;                    /* TransportFormatSet_ModeSSP */
static int hf_nbap_tdd1 = -1;                     /* TDD_TransportFormatSet_ModeDP */
static int hf_nbap_notApplicable = -1;            /* NULL */
static int hf_nbap_tdd2 = -1;                     /* TransportFormatSet_SecondInterleavingMode */
static int hf_nbap_ms_part = -1;                  /* INTEGER_0_16383 */
static int hf_nbap_ls_part = -1;                  /* INTEGER_0_4294967295 */
static int hf_nbap_tUTRANGPSChangeLimit = -1;     /* TUTRANGPSChangeLimit */
static int hf_nbap_predictedTUTRANGPSDeviationLimit = -1;  /* PredictedTUTRANGPSDeviationLimit */
static int hf_nbap_tUTRANGPS = -1;                /* TUTRANGPS */
static int hf_nbap_tUTRANGPSQuality = -1;         /* TUTRANGPSQuality */
static int hf_nbap_tUTRANGPSDriftRate = -1;       /* TUTRANGPSDriftRate */
static int hf_nbap_tUTRANGPSDriftRateQuality = -1;  /* TUTRANGPSDriftRateQuality */
static int hf_nbap_rNC_ID = -1;                   /* RNC_ID */
static int hf_nbap_hSDSCH_Physical_Layer_Category = -1;  /* INTEGER_1_64_ */
static int hf_nbap_UL_Timeslot_Information_item = -1;  /* UL_Timeslot_InformationItem */
static int hf_nbap_uL_Code_InformationList = -1;  /* TDD_UL_Code_Information */
static int hf_nbap_UL_TimeslotLCR_Information_item = -1;  /* UL_TimeslotLCR_InformationItem */
static int hf_nbap_uL_Code_InformationList1 = -1;  /* TDD_UL_Code_LCR_Information */
static int hf_nbap_uL_ScramblingCodeNumber = -1;  /* UL_ScramblingCodeNumber */
static int hf_nbap_uL_ScramblingCodeLength = -1;  /* UL_ScramblingCodeLength */
static int hf_nbap_uL_Synchronisation_StepSize = -1;  /* UL_Synchronisation_StepSize */
static int hf_nbap_uL_Synchronisation_Frequency = -1;  /* UL_Synchronisation_Frequency */
static int hf_nbap_UL_TimeSlot_ISCP_Info_item = -1;  /* UL_TimeSlot_ISCP_InfoItem */
static int hf_nbap_iSCP = -1;                     /* UL_TimeslotISCP_Value */
static int hf_nbap_UL_TimeSlot_ISCP_LCR_Info_item = -1;  /* UL_TimeSlot_ISCP_LCR_InfoItem */
static int hf_nbap_USCH_Information_item = -1;    /* USCH_InformationItem */
static int hf_nbap_USCH_InformationResponse_item = -1;  /* USCH_InformationResponseItem */
static int hf_nbap_local = -1;                    /* INTEGER_0_65535 */
static int hf_nbap_global = -1;                   /* OBJECT_IDENTIFIER */
static int hf_nbap_procedureCode = -1;            /* ProcedureCode */
static int hf_nbap_ddMode = -1;                   /* T_ddMode */
static int hf_nbap_shortTransActionId = -1;       /* INTEGER_0_127 */
static int hf_nbap_longTransActionId = -1;        /* INTEGER_0_32767 */
static int hf_nbap_ProtocolIE_Container_item = -1;  /* ProtocolIE_Field */
static int hf_nbap_id = -1;                       /* ProtocolIE_ID */
static int hf_nbap_value = -1;                    /* ProtocolIEValue */
static int hf_nbap_ProtocolIE_ContainerPair_item = -1;  /* ProtocolIE_FieldPair */
static int hf_nbap_firstCriticality = -1;         /* Criticality */
static int hf_nbap_firstValue = -1;               /* FirstValue */
static int hf_nbap_secondCriticality = -1;        /* Criticality */
static int hf_nbap_secondValue = -1;              /* SecondValue */
static int hf_nbap_ProtocolIE_ContainerList_item = -1;  /* ProtocolIE_Container */
static int hf_nbap_ProtocolIE_ContainerPairList_item = -1;  /* ProtocolIE_ContainerPair */
static int hf_nbap_ProtocolExtensionContainer_item = -1;  /* ProtocolExtensionField */
static int hf_nbap_extensionValue = -1;           /* Extension */
static int hf_nbap_PrivateIE_Container_item = -1;  /* PrivateIE_Field */
static int hf_nbap_privateIEid = -1;              /* PrivateIE_ID */
static int hf_nbap_privateIEvalue = -1;           /* PrivateIEvalue */
/* named bits */
static int hf_nbap_PreambleSignatures_signature15 = -1;
static int hf_nbap_PreambleSignatures_signature14 = -1;
static int hf_nbap_PreambleSignatures_signature13 = -1;
static int hf_nbap_PreambleSignatures_signature12 = -1;
static int hf_nbap_PreambleSignatures_signature11 = -1;
static int hf_nbap_PreambleSignatures_signature10 = -1;
static int hf_nbap_PreambleSignatures_signature9 = -1;
static int hf_nbap_PreambleSignatures_signature8 = -1;
static int hf_nbap_PreambleSignatures_signature7 = -1;
static int hf_nbap_PreambleSignatures_signature6 = -1;
static int hf_nbap_PreambleSignatures_signature5 = -1;
static int hf_nbap_PreambleSignatures_signature4 = -1;
static int hf_nbap_PreambleSignatures_signature3 = -1;
static int hf_nbap_PreambleSignatures_signature2 = -1;
static int hf_nbap_PreambleSignatures_signature1 = -1;
static int hf_nbap_PreambleSignatures_signature0 = -1;
static int hf_nbap_RACH_SubChannelNumbers_subCh11 = -1;
static int hf_nbap_RACH_SubChannelNumbers_subCh10 = -1;
static int hf_nbap_RACH_SubChannelNumbers_subCh9 = -1;
static int hf_nbap_RACH_SubChannelNumbers_subCh8 = -1;
static int hf_nbap_RACH_SubChannelNumbers_subCh7 = -1;
static int hf_nbap_RACH_SubChannelNumbers_subCh6 = -1;
static int hf_nbap_RACH_SubChannelNumbers_subCh5 = -1;
static int hf_nbap_RACH_SubChannelNumbers_subCh4 = -1;
static int hf_nbap_RACH_SubChannelNumbers_subCh3 = -1;
static int hf_nbap_RACH_SubChannelNumbers_subCh2 = -1;
static int hf_nbap_RACH_SubChannelNumbers_subCh1 = -1;
static int hf_nbap_RACH_SubChannelNumbers_subCh0 = -1;

/*--- End of included file: packet-nbap-hf.c ---*/
#line 58 "packet-nbap-template.c"

/* Initialize the subtree pointers */
static int ett_nbap = -1;
static int ett_nbap_initiatingMessageValue = -1;
static int ett_nbap_ProtocolIEValueValue = -1;
static int ett_nbap_SuccessfulOutcomeValue = -1;
static int ett_nbap_UnsuccessfulOutcomeValue = -1;


/*--- Included file: packet-nbap-ett.c ---*/
#line 1 "packet-nbap-ett.c"
static gint ett_nbap_NBAP_PDU = -1;
static gint ett_nbap_InitiatingMessage = -1;
static gint ett_nbap_SuccessfulOutcome = -1;
static gint ett_nbap_UnsuccessfulOutcome = -1;
static gint ett_nbap_Outcome = -1;
static gint ett_nbap_DummyInitiatingmessages = -1;
static gint ett_nbap_DummySuccessfullOutcomemessages = -1;
static gint ett_nbap_DummyUnsuccessfullOutcomemessages = -1;
static gint ett_nbap_DymmyProtocolIE_ID = -1;
static gint ett_nbap_CommonTransportChannelSetupRequestFDD = -1;
static gint ett_nbap_CommonPhysicalChannelType_CTCH_SetupRqstFDD = -1;
static gint ett_nbap_Secondary_CCPCH_CTCH_SetupRqstFDD = -1;
static gint ett_nbap_PowerOffsetInformation_CTCH_SetupRqstFDD = -1;
static gint ett_nbap_FACH_ParametersListIE_CTCH_SetupRqstFDD = -1;
static gint ett_nbap_FACH_ParametersItem_CTCH_SetupRqstFDD = -1;
static gint ett_nbap_PCH_ParametersItem_CTCH_SetupRqstFDD = -1;
static gint ett_nbap_PICH_Parameters_CTCH_SetupRqstFDD = -1;
static gint ett_nbap_MICH_Parameters_CTCH_SetupRqstFDD = -1;
static gint ett_nbap_PRACH_CTCH_SetupRqstFDD = -1;
static gint ett_nbap_AllowedSlotFormatInformationList_CTCH_SetupRqstFDD = -1;
static gint ett_nbap_AllowedSlotFormatInformationItem_CTCH_SetupRqstFDD = -1;
static gint ett_nbap_RACH_ParametersItem_CTCH_SetupRqstFDD = -1;
static gint ett_nbap_AICH_Parameters_CTCH_SetupRqstFDD = -1;
static gint ett_nbap_CommonTransportChannelSetupRequestTDD = -1;
static gint ett_nbap_CommonPhysicalChannelType_CTCH_SetupRqstTDD = -1;
static gint ett_nbap_Secondary_CCPCH_CTCH_SetupRqstTDD = -1;
static gint ett_nbap_Secondary_CCPCH_parameterListIE_CTCH_SetupRqstTDD = -1;
static gint ett_nbap_Secondary_CCPCH_parameterItem_CTCH_SetupRqstTDD = -1;
static gint ett_nbap_FACH_ParametersListIE_CTCH_SetupRqstTDD = -1;
static gint ett_nbap_FACH_ParametersItem_CTCH_SetupRqstTDD = -1;
static gint ett_nbap_PCH_ParametersItem_CTCH_SetupRqstTDD = -1;
static gint ett_nbap_PICH_ParametersItem_CTCH_SetupRqstTDD = -1;
static gint ett_nbap_PICH_LCR_Parameters_CTCH_SetupRqstTDD = -1;
static gint ett_nbap_Secondary_CCPCH_LCR_parameterList_CTCH_SetupRqstTDD = -1;
static gint ett_nbap_Secondary_CCPCH_LCR_parameterItem_CTCH_SetupRqstTDD = -1;
static gint ett_nbap_MICH_Parameters_CTCH_SetupRqstTDD = -1;
static gint ett_nbap_MICH_TDDOption_Specific_Parameters_CTCH_SetupRqstTDD = -1;
static gint ett_nbap_MICH_HCR_Parameters_CTCH_SetupRqstTDD = -1;
static gint ett_nbap_MICH_LCR_Parameters_CTCH_SetupRqstTDD = -1;
static gint ett_nbap_Secondary_CCPCH_parameterExtendedList_CTCH_SetupRqstTDD = -1;
static gint ett_nbap_Secondary_CCPCH_LCR_parameterExtendedList_CTCH_SetupRqstTDD = -1;
static gint ett_nbap_PRACH_CTCH_SetupRqstTDD = -1;
static gint ett_nbap_PRACH_ParametersItem_CTCH_SetupRqstTDD = -1;
static gint ett_nbap_RACH_ParameterItem_CTCH_SetupRqstTDD = -1;
static gint ett_nbap_PRACH_LCR_ParametersList_CTCH_SetupRqstTDD = -1;
static gint ett_nbap_PRACH_LCR_ParametersItem_CTCH_SetupRqstTDD = -1;
static gint ett_nbap_FPACH_LCR_Parameters_CTCH_SetupRqstTDD = -1;
static gint ett_nbap_CommonTransportChannelSetupResponse = -1;
static gint ett_nbap_FACH_CommonTransportChannel_InformationResponse = -1;
static gint ett_nbap_CommonTransportChannelSetupFailure = -1;
static gint ett_nbap_CommonTransportChannelReconfigurationRequestFDD = -1;
static gint ett_nbap_CommonPhysicalChannelType_CTCH_ReconfRqstFDD = -1;
static gint ett_nbap_Secondary_CCPCHList_CTCH_ReconfRqstFDD = -1;
static gint ett_nbap_FACH_ParametersListIE_CTCH_ReconfRqstFDD = -1;
static gint ett_nbap_FACH_ParametersItem_CTCH_ReconfRqstFDD = -1;
static gint ett_nbap_PCH_ParametersItem_CTCH_ReconfRqstFDD = -1;
static gint ett_nbap_PICH_ParametersItem_CTCH_ReconfRqstFDD = -1;
static gint ett_nbap_MICH_Parameters_CTCH_ReconfRqstFDD = -1;
static gint ett_nbap_PRACHList_CTCH_ReconfRqstFDD = -1;
static gint ett_nbap_PRACH_ParametersListIE_CTCH_ReconfRqstFDD = -1;
static gint ett_nbap_PRACH_ParametersItem_CTCH_ReconfRqstFDD = -1;
static gint ett_nbap_AllowedSlotFormatInformationList_CTCH_ReconfRqstFDD = -1;
static gint ett_nbap_AllowedSlotFormatInformationItem_CTCH_ReconfRqstFDD = -1;
static gint ett_nbap_AICH_ParametersListIE_CTCH_ReconfRqstFDD = -1;
static gint ett_nbap_AICH_ParametersItem_CTCH_ReconfRqstFDD = -1;
static gint ett_nbap_CommonTransportChannelReconfigurationRequestTDD = -1;
static gint ett_nbap_Secondary_CCPCH_Parameters_CTCH_ReconfRqstTDD = -1;
static gint ett_nbap_Secondary_CCPCHListIE_CTCH_ReconfRqstTDD = -1;
static gint ett_nbap_Secondary_CCPCHItem_CTCH_ReconfRqstTDD = -1;
static gint ett_nbap_PICH_Parameters_CTCH_ReconfRqstTDD = -1;
static gint ett_nbap_FACH_ParametersList_CTCH_ReconfRqstTDD = -1;
static gint ett_nbap_FACH_ParametersItem_CTCH_ReconfRqstTDD = -1;
static gint ett_nbap_PCH_Parameters_CTCH_ReconfRqstTDD = -1;
static gint ett_nbap_FPACH_LCR_Parameters_CTCH_ReconfRqstTDD = -1;
static gint ett_nbap_MICH_Parameters_CTCH_ReconfRqstTDD = -1;
static gint ett_nbap_Secondary_CCPCH_parameterExtendedList_CTCH_ReconfRqstTDD = -1;
static gint ett_nbap_Secondary_CCPCH_LCR_parameterExtendedList_CTCH_ReconfRqstTDD = -1;
static gint ett_nbap_CommonTransportChannelReconfigurationResponse = -1;
static gint ett_nbap_CommonTransportChannelReconfigurationFailure = -1;
static gint ett_nbap_CommonTransportChannelDeletionRequest = -1;
static gint ett_nbap_CommonTransportChannelDeletionResponse = -1;
static gint ett_nbap_BlockResourceRequest = -1;
static gint ett_nbap_BlockResourceResponse = -1;
static gint ett_nbap_BlockResourceFailure = -1;
static gint ett_nbap_UnblockResourceIndication = -1;
static gint ett_nbap_AuditRequiredIndication = -1;
static gint ett_nbap_AuditRequest = -1;
static gint ett_nbap_AuditResponse = -1;
static gint ett_nbap_Cell_InformationList_AuditRsp = -1;
static gint ett_nbap_Cell_InformationItem_AuditRsp = -1;
static gint ett_nbap_S_CPICH_InformationList_AuditRsp = -1;
static gint ett_nbap_S_CCPCH_InformationList_AuditRsp = -1;
static gint ett_nbap_FACH_InformationList_AuditRsp = -1;
static gint ett_nbap_PRACH_InformationList_AuditRsp = -1;
static gint ett_nbap_RACH_InformationList_AuditRsp = -1;
static gint ett_nbap_AICH_InformationList_AuditRsp = -1;
static gint ett_nbap_CCP_InformationList_AuditRsp = -1;
static gint ett_nbap_CCP_InformationItem_AuditRsp = -1;
static gint ett_nbap_FPACH_LCR_InformationList_AuditRsp = -1;
static gint ett_nbap_HS_DSCH_Resources_Information_AuditRsp = -1;
static gint ett_nbap_S_CCPCH_InformationListExt_AuditRsp = -1;
static gint ett_nbap_S_CCPCH_LCR_InformationListExt_AuditRsp = -1;
static gint ett_nbap_E_DCH_Resources_Information_AuditRsp = -1;
static gint ett_nbap_Local_Cell_InformationList_AuditRsp = -1;
static gint ett_nbap_Local_Cell_InformationItem_AuditRsp = -1;
static gint ett_nbap_Local_Cell_Group_InformationList_AuditRsp = -1;
static gint ett_nbap_Local_Cell_Group_InformationItem_AuditRsp = -1;
static gint ett_nbap_Power_Local_Cell_Group_InformationList_AuditRsp = -1;
static gint ett_nbap_Power_Local_Cell_Group_InformationItem_AuditRsp = -1;
static gint ett_nbap_AuditFailure = -1;
static gint ett_nbap_CommonMeasurementInitiationRequest = -1;
static gint ett_nbap_CommonMeasurementObjectType_CM_Rqst = -1;
static gint ett_nbap_Cell_CM_Rqst = -1;
static gint ett_nbap_RACH_CM_Rqst = -1;
static gint ett_nbap_PowerLocalCellGroup_CM_Rqst = -1;
static gint ett_nbap_CommonMeasurementInitiationResponse = -1;
static gint ett_nbap_CommonMeasurementObjectType_CM_Rsp = -1;
static gint ett_nbap_Cell_CM_Rsp = -1;
static gint ett_nbap_RACH_CM_Rsp = -1;
static gint ett_nbap_PowerLocalCellGroup_CM_Rsp = -1;
static gint ett_nbap_CommonMeasurementInitiationFailure = -1;
static gint ett_nbap_CellSetupRequestFDD = -1;
static gint ett_nbap_Synchronisation_Configuration_Cell_SetupRqst = -1;
static gint ett_nbap_PrimarySCH_Information_Cell_SetupRqstFDD = -1;
static gint ett_nbap_SecondarySCH_Information_Cell_SetupRqstFDD = -1;
static gint ett_nbap_PrimaryCPICH_Information_Cell_SetupRqstFDD = -1;
static gint ett_nbap_SecondaryCPICH_InformationList_Cell_SetupRqstFDD = -1;
static gint ett_nbap_SecondaryCPICH_InformationItem_Cell_SetupRqstFDD = -1;
static gint ett_nbap_PrimaryCCPCH_Information_Cell_SetupRqstFDD = -1;
static gint ett_nbap_BCH_Information_Cell_SetupRqstFDD = -1;
static gint ett_nbap_Limited_power_increase_information_Cell_SetupRqstFDD = -1;
static gint ett_nbap_IPDLParameter_Information_Cell_SetupRqstFDD = -1;
static gint ett_nbap_CellPortion_InformationList_Cell_SetupRqstFDD = -1;
static gint ett_nbap_CellPortion_InformationItem_Cell_SetupRqstFDD = -1;
static gint ett_nbap_CellSetupRequestTDD = -1;
static gint ett_nbap_SCH_Information_Cell_SetupRqstTDD = -1;
static gint ett_nbap_SyncCaseIndicatorItem_Cell_SetupRqstTDD_PSCH = -1;
static gint ett_nbap_Case1_Cell_SetupRqstTDD = -1;
static gint ett_nbap_Case2_Cell_SetupRqstTDD = -1;
static gint ett_nbap_PCCPCH_Information_Cell_SetupRqstTDD = -1;
static gint ett_nbap_TimeSlotConfigurationList_Cell_SetupRqstTDD = -1;
static gint ett_nbap_TimeSlotConfigurationItem_Cell_SetupRqstTDD = -1;
static gint ett_nbap_TimeSlotConfigurationList_LCR_Cell_SetupRqstTDD = -1;
static gint ett_nbap_TimeSlotConfigurationItem_LCR_Cell_SetupRqstTDD = -1;
static gint ett_nbap_PCCPCH_LCR_Information_Cell_SetupRqstTDD = -1;
static gint ett_nbap_DwPCH_LCR_Information_Cell_SetupRqstTDD = -1;
static gint ett_nbap_IPDLParameter_Information_Cell_SetupRqstTDD = -1;
static gint ett_nbap_IPDLParameter_Information_LCR_Cell_SetupRqstTDD = -1;
static gint ett_nbap_CellSetupResponse = -1;
static gint ett_nbap_CellSetupFailure = -1;
static gint ett_nbap_CellReconfigurationRequestFDD = -1;
static gint ett_nbap_Synchronisation_Configuration_Cell_ReconfRqst = -1;
static gint ett_nbap_PrimarySCH_Information_Cell_ReconfRqstFDD = -1;
static gint ett_nbap_SecondarySCH_Information_Cell_ReconfRqstFDD = -1;
static gint ett_nbap_PrimaryCPICH_Information_Cell_ReconfRqstFDD = -1;
static gint ett_nbap_SecondaryCPICH_InformationList_Cell_ReconfRqstFDD = -1;
static gint ett_nbap_SecondaryCPICH_InformationItem_Cell_ReconfRqstFDD = -1;
static gint ett_nbap_PrimaryCCPCH_Information_Cell_ReconfRqstFDD = -1;
static gint ett_nbap_BCH_information_Cell_ReconfRqstFDD = -1;
static gint ett_nbap_IPDLParameter_Information_Cell_ReconfRqstFDD = -1;
static gint ett_nbap_CellPortion_InformationList_Cell_ReconfRqstFDD = -1;
static gint ett_nbap_CellPortion_InformationItem_Cell_ReconfRqstFDD = -1;
static gint ett_nbap_CellReconfigurationRequestTDD = -1;
static gint ett_nbap_SCH_Information_Cell_ReconfRqstTDD = -1;
static gint ett_nbap_PCCPCH_Information_Cell_ReconfRqstTDD = -1;
static gint ett_nbap_TimeSlotConfigurationList_Cell_ReconfRqstTDD = -1;
static gint ett_nbap_TimeSlotConfigurationItem_Cell_ReconfRqstTDD = -1;
static gint ett_nbap_TimeSlotConfigurationList_LCR_Cell_ReconfRqstTDD = -1;
static gint ett_nbap_TimeSlotConfigurationItem_LCR_Cell_ReconfRqstTDD = -1;
static gint ett_nbap_DwPCH_LCR_Information_Cell_ReconfRqstTDD = -1;
static gint ett_nbap_IPDLParameter_Information_Cell_ReconfRqstTDD = -1;
static gint ett_nbap_IPDLParameter_Information_LCR_Cell_ReconfRqstTDD = -1;
static gint ett_nbap_CellReconfigurationResponse = -1;
static gint ett_nbap_CellReconfigurationFailure = -1;
static gint ett_nbap_CellDeletionRequest = -1;
static gint ett_nbap_CellDeletionResponse = -1;
static gint ett_nbap_ResourceStatusIndication = -1;
static gint ett_nbap_IndicationType_ResourceStatusInd = -1;
static gint ett_nbap_No_Failure_ResourceStatusInd = -1;
static gint ett_nbap_Local_Cell_InformationList_ResourceStatusInd = -1;
static gint ett_nbap_Local_Cell_InformationItem_ResourceStatusInd = -1;
static gint ett_nbap_Local_Cell_Group_InformationList_ResourceStatusInd = -1;
static gint ett_nbap_Local_Cell_Group_InformationItem_ResourceStatusInd = -1;
static gint ett_nbap_Power_Local_Cell_Group_InformationList_ResourceStatusInd = -1;
static gint ett_nbap_Power_Local_Cell_Group_InformationItem_ResourceStatusInd = -1;
static gint ett_nbap_ServiceImpacting_ResourceStatusInd = -1;
static gint ett_nbap_Local_Cell_InformationList2_ResourceStatusInd = -1;
static gint ett_nbap_Local_Cell_InformationItem2_ResourceStatusInd = -1;
static gint ett_nbap_Local_Cell_Group_InformationList2_ResourceStatusInd = -1;
static gint ett_nbap_Local_Cell_Group_InformationItem2_ResourceStatusInd = -1;
static gint ett_nbap_Power_Local_Cell_Group_InformationList2_ResourceStatusInd = -1;
static gint ett_nbap_Power_Local_Cell_Group_InformationItem2_ResourceStatusInd = -1;
static gint ett_nbap_CCP_InformationList_ResourceStatusInd = -1;
static gint ett_nbap_CCP_InformationItem_ResourceStatusInd = -1;
static gint ett_nbap_Cell_InformationList_ResourceStatusInd = -1;
static gint ett_nbap_Cell_InformationItem_ResourceStatusInd = -1;
static gint ett_nbap_S_CPICH_InformationList_ResourceStatusInd = -1;
static gint ett_nbap_S_CCPCH_InformationList_ResourceStatusInd = -1;
static gint ett_nbap_FACH_InformationList_ResourceStatusInd = -1;
static gint ett_nbap_PRACH_InformationList_ResourceStatusInd = -1;
static gint ett_nbap_RACH_InformationList_ResourceStatusInd = -1;
static gint ett_nbap_AICH_InformationList_ResourceStatusInd = -1;
static gint ett_nbap_FPACH_LCR_InformationList_ResourceStatusInd = -1;
static gint ett_nbap_HS_DSCH_Resources_Information_ResourceStatusInd = -1;
static gint ett_nbap_S_CCPCH_InformationListExt_ResourceStatusInd = -1;
static gint ett_nbap_S_CCPCH_LCR_InformationListExt_ResourceStatusInd = -1;
static gint ett_nbap_E_DCH_Resources_Information_ResourceStatusInd = -1;
static gint ett_nbap_SystemInformationUpdateRequest = -1;
static gint ett_nbap_MIB_SB_SIB_InformationList_SystemInfoUpdateRqst = -1;
static gint ett_nbap_MIB_SB_SIB_InformationItem_SystemInfoUpdateRqst = -1;
static gint ett_nbap_DeletionIndicator_SystemInfoUpdate = -1;
static gint ett_nbap_No_Deletion_SystemInfoUpdate = -1;
static gint ett_nbap_SegmentInformationListIE_SystemInfoUpdate = -1;
static gint ett_nbap_SegmentInformationItem_SystemInfoUpdate = -1;
static gint ett_nbap_SystemInformationUpdateResponse = -1;
static gint ett_nbap_SystemInformationUpdateFailure = -1;
static gint ett_nbap_RadioLinkSetupRequestFDD = -1;
static gint ett_nbap_UL_DPCH_Information_RL_SetupRqstFDD = -1;
static gint ett_nbap_DL_DPCH_Information_RL_SetupRqstFDD = -1;
static gint ett_nbap_PowerOffsetInformation_RL_SetupRqstFDD = -1;
static gint ett_nbap_RL_InformationList_RL_SetupRqstFDD = -1;
static gint ett_nbap_RL_InformationItem_RL_SetupRqstFDD = -1;
static gint ett_nbap_E_DPCH_Information_RL_SetupRqstFDD = -1;
static gint ett_nbap_F_DPCH_Information_RL_SetupRqstFDD = -1;
static gint ett_nbap_PowerOffsetInformation_F_DPCH_RL_SetupRqstFDD = -1;
static gint ett_nbap_RadioLinkSetupRequestTDD = -1;
static gint ett_nbap_UL_CCTrCH_InformationList_RL_SetupRqstTDD = -1;
static gint ett_nbap_UL_CCTrCH_InformationItem_RL_SetupRqstTDD = -1;
static gint ett_nbap_UL_DPCH_InformationItem_RL_SetupRqstTDD = -1;
static gint ett_nbap_UL_DPCH_LCR_Information_RL_SetupRqstTDD = -1;
static gint ett_nbap_DL_CCTrCH_InformationList_RL_SetupRqstTDD = -1;
static gint ett_nbap_DL_CCTrCH_InformationItem_RL_SetupRqstTDD = -1;
static gint ett_nbap_CCTrCH_TPCList_RL_SetupRqstTDD = -1;
static gint ett_nbap_CCTrCH_TPCItem_RL_SetupRqstTDD = -1;
static gint ett_nbap_DL_DPCH_InformationItem_RL_SetupRqstTDD = -1;
static gint ett_nbap_DL_DPCH_LCR_Information_RL_SetupRqstTDD = -1;
static gint ett_nbap_RL_Information_RL_SetupRqstTDD = -1;
static gint ett_nbap_RadioLinkSetupResponseFDD = -1;
static gint ett_nbap_RL_InformationResponseList_RL_SetupRspFDD = -1;
static gint ett_nbap_RL_InformationResponseItem_RL_SetupRspFDD = -1;
static gint ett_nbap_DiversityIndication_RL_SetupRspFDD = -1;
static gint ett_nbap_Combining_RL_SetupRspFDD = -1;
static gint ett_nbap_NonCombiningOrFirstRL_RL_SetupRspFDD = -1;
static gint ett_nbap_RadioLinkSetupResponseTDD = -1;
static gint ett_nbap_RL_InformationResponse_RL_SetupRspTDD = -1;
static gint ett_nbap_RL_InformationResponse_LCR_RL_SetupRspTDD = -1;
static gint ett_nbap_RadioLinkSetupFailureFDD = -1;
static gint ett_nbap_CauseLevel_RL_SetupFailureFDD = -1;
static gint ett_nbap_GeneralCauseList_RL_SetupFailureFDD = -1;
static gint ett_nbap_RLSpecificCauseList_RL_SetupFailureFDD = -1;
static gint ett_nbap_Unsuccessful_RL_InformationRespList_RL_SetupFailureFDD = -1;
static gint ett_nbap_Unsuccessful_RL_InformationRespItem_RL_SetupFailureFDD = -1;
static gint ett_nbap_Successful_RL_InformationRespList_RL_SetupFailureFDD = -1;
static gint ett_nbap_Successful_RL_InformationRespItem_RL_SetupFailureFDD = -1;
static gint ett_nbap_DiversityIndication_RL_SetupFailureFDD = -1;
static gint ett_nbap_Combining_RL_SetupFailureFDD = -1;
static gint ett_nbap_NonCombiningOrFirstRL_RL_SetupFailureFDD = -1;
static gint ett_nbap_RadioLinkSetupFailureTDD = -1;
static gint ett_nbap_CauseLevel_RL_SetupFailureTDD = -1;
static gint ett_nbap_GeneralCauseList_RL_SetupFailureTDD = -1;
static gint ett_nbap_RLSpecificCauseList_RL_SetupFailureTDD = -1;
static gint ett_nbap_Unsuccessful_RL_InformationResp_RL_SetupFailureTDD = -1;
static gint ett_nbap_RadioLinkAdditionRequestFDD = -1;
static gint ett_nbap_RL_InformationList_RL_AdditionRqstFDD = -1;
static gint ett_nbap_RL_InformationItem_RL_AdditionRqstFDD = -1;
static gint ett_nbap_RadioLinkAdditionRequestTDD = -1;
static gint ett_nbap_UL_CCTrCH_InformationList_RL_AdditionRqstTDD = -1;
static gint ett_nbap_UL_CCTrCH_InformationItem_RL_AdditionRqstTDD = -1;
static gint ett_nbap_UL_DPCH_InformationItem_RL_AdditionRqstTDD = -1;
static gint ett_nbap_DL_CCTrCH_InformationList_RL_AdditionRqstTDD = -1;
static gint ett_nbap_DL_CCTrCH_InformationItem_RL_AdditionRqstTDD = -1;
static gint ett_nbap_DL_DPCH_InformationItem_RL_AdditionRqstTDD = -1;
static gint ett_nbap_RL_Information_RL_AdditionRqstTDD = -1;
static gint ett_nbap_UL_DPCH_InformationItem_LCR_RL_AdditionRqstTDD = -1;
static gint ett_nbap_DL_DPCH_InformationItem_LCR_RL_AdditionRqstTDD = -1;
static gint ett_nbap_RadioLinkAdditionResponseFDD = -1;
static gint ett_nbap_RL_InformationResponseList_RL_AdditionRspFDD = -1;
static gint ett_nbap_RL_InformationResponseItem_RL_AdditionRspFDD = -1;
static gint ett_nbap_DiversityIndication_RL_AdditionRspFDD = -1;
static gint ett_nbap_Combining_RL_AdditionRspFDD = -1;
static gint ett_nbap_Non_Combining_RL_AdditionRspFDD = -1;
static gint ett_nbap_RadioLinkAdditionResponseTDD = -1;
static gint ett_nbap_RL_InformationResponse_RL_AdditionRspTDD = -1;
static gint ett_nbap_DCH_Information_RL_AdditionRspTDD = -1;
static gint ett_nbap_DiversityIndication_RL_AdditionRspTDD = -1;
static gint ett_nbap_Combining_RL_AdditionRspTDD = -1;
static gint ett_nbap_Non_Combining_RL_AdditionRspTDD = -1;
static gint ett_nbap_RL_InformationResponse_LCR_RL_AdditionRspTDD = -1;
static gint ett_nbap_RadioLinkAdditionFailureFDD = -1;
static gint ett_nbap_CauseLevel_RL_AdditionFailureFDD = -1;
static gint ett_nbap_GeneralCauseList_RL_AdditionFailureFDD = -1;
static gint ett_nbap_RLSpecificCauseList_RL_AdditionFailureFDD = -1;
static gint ett_nbap_Unsuccessful_RL_InformationRespList_RL_AdditionFailureFDD = -1;
static gint ett_nbap_Unsuccessful_RL_InformationRespItem_RL_AdditionFailureFDD = -1;
static gint ett_nbap_Successful_RL_InformationRespList_RL_AdditionFailureFDD = -1;
static gint ett_nbap_Successful_RL_InformationRespItem_RL_AdditionFailureFDD = -1;
static gint ett_nbap_DiversityIndication_RL_AdditionFailureFDD = -1;
static gint ett_nbap_Combining_RL_AdditionFailureFDD = -1;
static gint ett_nbap_Non_Combining_RL_AdditionFailureFDD = -1;
static gint ett_nbap_RadioLinkAdditionFailureTDD = -1;
static gint ett_nbap_CauseLevel_RL_AdditionFailureTDD = -1;
static gint ett_nbap_GeneralCauseList_RL_AdditionFailureTDD = -1;
static gint ett_nbap_RLSpecificCauseList_RL_AdditionFailureTDD = -1;
static gint ett_nbap_Unsuccessful_RL_InformationResp_RL_AdditionFailureTDD = -1;
static gint ett_nbap_RadioLinkReconfigurationPrepareFDD = -1;
static gint ett_nbap_UL_DPCH_Information_RL_ReconfPrepFDD = -1;
static gint ett_nbap_DL_DPCH_Information_RL_ReconfPrepFDD = -1;
static gint ett_nbap_DL_DPCH_Power_Information_RL_ReconfPrepFDD = -1;
static gint ett_nbap_PowerOffsetInformation_RL_ReconfPrepFDD = -1;
static gint ett_nbap_DCH_DeleteList_RL_ReconfPrepFDD = -1;
static gint ett_nbap_DCH_DeleteItem_RL_ReconfPrepFDD = -1;
static gint ett_nbap_RL_InformationList_RL_ReconfPrepFDD = -1;
static gint ett_nbap_RL_InformationItem_RL_ReconfPrepFDD = -1;
static gint ett_nbap_E_DPCH_Information_RL_ReconfPrepFDD = -1;
static gint ett_nbap_F_DPCH_Information_RL_ReconfPrepFDD = -1;
static gint ett_nbap_PowerOffsetInformation_F_DPCH_RL_ReconfPrepFDD = -1;
static gint ett_nbap_RadioLinkReconfigurationPrepareTDD = -1;
static gint ett_nbap_UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD = -1;
static gint ett_nbap_UL_CCTrCH_InformationAddItem_RL_ReconfPrepTDD = -1;
static gint ett_nbap_MultipleRL_UL_DPCH_InformationAddList_RL_ReconfPrepTDD = -1;
static gint ett_nbap_MultipleRL_UL_DPCH_InformationAddListIE_RL_ReconfPrepTDD = -1;
static gint ett_nbap_UL_DPCH_InformationAddItem_RL_ReconfPrepTDD = -1;
static gint ett_nbap_UL_DPCH_LCR_InformationAddList_RL_ReconfPrepTDD = -1;
static gint ett_nbap_UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD = -1;
static gint ett_nbap_UL_CCTrCH_InformationModifyItem_RL_ReconfPrepTDD = -1;
static gint ett_nbap_MultipleRL_UL_DPCH_InformationModifyList_RL_ReconfPrepTDD = -1;
static gint ett_nbap_MultipleRL_UL_DPCH_InformationModifyListIE_RL_ReconfPrepTDD = -1;
static gint ett_nbap_UL_DPCH_InformationModify_AddItem_RL_ReconfPrepTDD = -1;
static gint ett_nbap_UL_DPCH_LCR_InformationModify_AddList_RL_ReconfPrepTDD = -1;
static gint ett_nbap_UL_DPCH_InformationModify_ModifyItem_RL_ReconfPrepTDD = -1;
static gint ett_nbap_UL_Timeslot_InformationModify_ModifyList_RL_ReconfPrepTDD = -1;
static gint ett_nbap_UL_Timeslot_InformationModify_ModifyItem_RL_ReconfPrepTDD = -1;
static gint ett_nbap_UL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD = -1;
static gint ett_nbap_UL_Code_InformationModify_ModifyItem_RL_ReconfPrepTDD = -1;
static gint ett_nbap_UL_TimeslotLCR_InformationModify_ModifyList_RL_ReconfPrepTDD = -1;
static gint ett_nbap_UL_Timeslot_LCR_InformationModify_ModifyItem_RL_ReconfPrepTDD = -1;
static gint ett_nbap_UL_Code_InformationModify_ModifyList_RL_ReconfPrepTDDLCR = -1;
static gint ett_nbap_UL_Code_InformationModify_ModifyItem_RL_ReconfPrepTDDLCR = -1;
static gint ett_nbap_UL_DPCH_InformationModify_DeleteListIE_RL_ReconfPrepTDD = -1;
static gint ett_nbap_UL_DPCH_InformationModify_DeleteItem_RL_ReconfPrepTDD = -1;
static gint ett_nbap_UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD = -1;
static gint ett_nbap_UL_CCTrCH_InformationDeleteItem_RL_ReconfPrepTDD = -1;
static gint ett_nbap_DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD = -1;
static gint ett_nbap_DL_CCTrCH_InformationAddItem_RL_ReconfPrepTDD = -1;
static gint ett_nbap_MultipleRL_DL_DPCH_InformationAddList_RL_ReconfPrepTDD = -1;
static gint ett_nbap_MultipleRL_DL_DPCH_InformationAddListIE_RL_ReconfPrepTDD = -1;
static gint ett_nbap_CCTrCH_TPCAddList_RL_ReconfPrepTDD = -1;
static gint ett_nbap_CCTrCH_TPCAddItem_RL_ReconfPrepTDD = -1;
static gint ett_nbap_DL_DPCH_InformationAddItem_RL_ReconfPrepTDD = -1;
static gint ett_nbap_DL_DPCH_LCR_InformationAddList_RL_ReconfPrepTDD = -1;
static gint ett_nbap_DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD = -1;
static gint ett_nbap_DL_CCTrCH_InformationModifyItem_RL_ReconfPrepTDD = -1;
static gint ett_nbap_MultipleRL_DL_DPCH_InformationModifyList_RL_ReconfPrepTDD = -1;
static gint ett_nbap_MultipleRL_DL_DPCH_InformationModifyListIE_RL_ReconfPrepTDD = -1;
static gint ett_nbap_CCTrCH_TPCModifyList_RL_ReconfPrepTDD = -1;
static gint ett_nbap_CCTrCH_TPCModifyItem_RL_ReconfPrepTDD = -1;
static gint ett_nbap_DL_DPCH_InformationModify_AddItem_RL_ReconfPrepTDD = -1;
static gint ett_nbap_DL_DPCH_LCR_InformationModify_AddList_RL_ReconfPrepTDD = -1;
static gint ett_nbap_DL_DPCH_InformationModify_ModifyItem_RL_ReconfPrepTDD = -1;
static gint ett_nbap_DL_Timeslot_InformationModify_ModifyList_RL_ReconfPrepTDD = -1;
static gint ett_nbap_DL_Timeslot_InformationModify_ModifyItem_RL_ReconfPrepTDD = -1;
static gint ett_nbap_DL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD = -1;
static gint ett_nbap_DL_Code_InformationModify_ModifyItem_RL_ReconfPrepTDD = -1;
static gint ett_nbap_DL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD = -1;
static gint ett_nbap_DL_Timeslot_LCR_InformationModify_ModifyItem_RL_ReconfPrepTDD = -1;
static gint ett_nbap_DL_Code_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD = -1;
static gint ett_nbap_DL_Code_LCR_InformationModify_ModifyItem_RL_ReconfPrepTDD = -1;
static gint ett_nbap_DL_DPCH_InformationModify_DeleteListIE_RL_ReconfPrepTDD = -1;
static gint ett_nbap_DL_DPCH_InformationModify_DeleteItem_RL_ReconfPrepTDD = -1;
static gint ett_nbap_DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD = -1;
static gint ett_nbap_DL_CCTrCH_InformationDeleteItem_RL_ReconfPrepTDD = -1;
static gint ett_nbap_DCH_DeleteList_RL_ReconfPrepTDD = -1;
static gint ett_nbap_DCH_DeleteItem_RL_ReconfPrepTDD = -1;
static gint ett_nbap_DSCH_Information_ModifyList_RL_ReconfPrepTDD = -1;
static gint ett_nbap_DSCH_Information_ModifyItem_RL_ReconfPrepTDD = -1;
static gint ett_nbap_DSCH_Information_DeleteList_RL_ReconfPrepTDD = -1;
static gint ett_nbap_DSCH_Information_DeleteItem_RL_ReconfPrepTDD = -1;
static gint ett_nbap_USCH_Information_ModifyList_RL_ReconfPrepTDD = -1;
static gint ett_nbap_USCH_Information_ModifyItem_RL_ReconfPrepTDD = -1;
static gint ett_nbap_USCH_Information_DeleteList_RL_ReconfPrepTDD = -1;
static gint ett_nbap_USCH_Information_DeleteItem_RL_ReconfPrepTDD = -1;
static gint ett_nbap_MultipleRL_Information_RL_ReconfPrepTDD = -1;
static gint ett_nbap_RL_Information_RL_ReconfPrepTDD = -1;
static gint ett_nbap_RadioLinkReconfigurationReady = -1;
static gint ett_nbap_RL_InformationResponseList_RL_ReconfReady = -1;
static gint ett_nbap_RL_InformationResponseItem_RL_ReconfReady = -1;
static gint ett_nbap_RadioLinkReconfigurationFailure = -1;
static gint ett_nbap_CauseLevel_RL_ReconfFailure = -1;
static gint ett_nbap_GeneralCauseList_RL_ReconfFailure = -1;
static gint ett_nbap_RLSpecificCauseList_RL_ReconfFailure = -1;
static gint ett_nbap_RL_ReconfigurationFailureList_RL_ReconfFailure = -1;
static gint ett_nbap_RL_ReconfigurationFailureItem_RL_ReconfFailure = -1;
static gint ett_nbap_RadioLinkReconfigurationCommit = -1;
static gint ett_nbap_RadioLinkReconfigurationCancel = -1;
static gint ett_nbap_RadioLinkReconfigurationRequestFDD = -1;
static gint ett_nbap_UL_DPCH_Information_RL_ReconfRqstFDD = -1;
static gint ett_nbap_DL_DPCH_Information_RL_ReconfRqstFDD = -1;
static gint ett_nbap_DCH_DeleteList_RL_ReconfRqstFDD = -1;
static gint ett_nbap_DCH_DeleteItem_RL_ReconfRqstFDD = -1;
static gint ett_nbap_RL_InformationList_RL_ReconfRqstFDD = -1;
static gint ett_nbap_RL_InformationItem_RL_ReconfRqstFDD = -1;
static gint ett_nbap_E_DPCH_Information_RL_ReconfRqstFDD = -1;
static gint ett_nbap_RadioLinkReconfigurationRequestTDD = -1;
static gint ett_nbap_UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD = -1;
static gint ett_nbap_UL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD = -1;
static gint ett_nbap_UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD = -1;
static gint ett_nbap_UL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD = -1;
static gint ett_nbap_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD = -1;
static gint ett_nbap_DL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD = -1;
static gint ett_nbap_MultipleRL_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD = -1;
static gint ett_nbap_MultipleRL_DL_CCTrCH_InformationModifyListIE_RL_ReconfRqstTDD = -1;
static gint ett_nbap_DL_DPCH_LCR_InformationModify_ModifyList_RL_ReconfRqstTDD = -1;
static gint ett_nbap_DL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfRqstTDD = -1;
static gint ett_nbap_DL_Timeslot_LCR_InformationModify_ModifyItem_RL_ReconfRqstTDD = -1;
static gint ett_nbap_DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD = -1;
static gint ett_nbap_DL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD = -1;
static gint ett_nbap_DCH_DeleteList_RL_ReconfRqstTDD = -1;
static gint ett_nbap_DCH_DeleteItem_RL_ReconfRqstTDD = -1;
static gint ett_nbap_Multiple_RL_Information_RL_ReconfRqstTDD = -1;
static gint ett_nbap_RL_Information_RL_ReconfRqstTDD = -1;
static gint ett_nbap_RadioLinkReconfigurationResponse = -1;
static gint ett_nbap_RL_InformationResponseList_RL_ReconfRsp = -1;
static gint ett_nbap_RL_InformationResponseItem_RL_ReconfRsp = -1;
static gint ett_nbap_RadioLinkDeletionRequest = -1;
static gint ett_nbap_RL_informationList_RL_DeletionRqst = -1;
static gint ett_nbap_RL_informationItem_RL_DeletionRqst = -1;
static gint ett_nbap_RadioLinkDeletionResponse = -1;
static gint ett_nbap_DL_PowerControlRequest = -1;
static gint ett_nbap_DL_ReferencePowerInformationList_DL_PC_Rqst = -1;
static gint ett_nbap_DL_ReferencePowerInformationItem_DL_PC_Rqst = -1;
static gint ett_nbap_DL_PowerTimeslotControlRequest = -1;
static gint ett_nbap_DedicatedMeasurementInitiationRequest = -1;
static gint ett_nbap_DedicatedMeasurementObjectType_DM_Rqst = -1;
static gint ett_nbap_RL_DM_Rqst = -1;
static gint ett_nbap_RL_InformationList_DM_Rqst = -1;
static gint ett_nbap_RL_InformationItem_DM_Rqst = -1;
static gint ett_nbap_PUSCH_Info_DM_Rqst = -1;
static gint ett_nbap_HSSICH_Info_DM_Rqst = -1;
static gint ett_nbap_RL_Set_DM_Rqst = -1;
static gint ett_nbap_RL_Set_InformationList_DM_Rqst = -1;
static gint ett_nbap_RL_Set_InformationItem_DM_Rqst = -1;
static gint ett_nbap_DedicatedMeasurementInitiationResponse = -1;
static gint ett_nbap_DedicatedMeasurementObjectType_DM_Rsp = -1;
static gint ett_nbap_RL_DM_Rsp = -1;
static gint ett_nbap_RL_InformationList_DM_Rsp = -1;
static gint ett_nbap_RL_InformationItem_DM_Rsp = -1;
static gint ett_nbap_PUSCH_Info_DM_Rsp = -1;
static gint ett_nbap_Multiple_PUSCH_InfoList_DM_Rsp = -1;
static gint ett_nbap_Multiple_PUSCH_InfoListIE_DM_Rsp = -1;
static gint ett_nbap_RL_Set_DM_Rsp = -1;
static gint ett_nbap_RL_Set_InformationList_DM_Rsp = -1;
static gint ett_nbap_RL_Set_InformationItem_DM_Rsp = -1;
static gint ett_nbap_Multiple_DedicatedMeasurementValueList_TDD_DM_Rsp = -1;
static gint ett_nbap_Multiple_DedicatedMeasurementValueItem_TDD_DM_Rsp = -1;
static gint ett_nbap_Multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp = -1;
static gint ett_nbap_Multiple_DedicatedMeasurementValueItem_LCR_TDD_DM_Rsp = -1;
static gint ett_nbap_DedicatedMeasurementInitiationFailure = -1;
static gint ett_nbap_DedicatedMeasurementReport = -1;
static gint ett_nbap_DedicatedMeasurementObjectType_DM_Rprt = -1;
static gint ett_nbap_RL_DM_Rprt = -1;
static gint ett_nbap_RL_InformationList_DM_Rprt = -1;
static gint ett_nbap_RL_InformationItem_DM_Rprt = -1;
static gint ett_nbap_PUSCH_Info_DM_Rprt = -1;
static gint ett_nbap_Multiple_PUSCH_InfoList_DM_Rprt = -1;
static gint ett_nbap_Multiple_PUSCH_InfoListIE_DM_Rprt = -1;
static gint ett_nbap_RL_Set_DM_Rprt = -1;
static gint ett_nbap_RL_Set_InformationList_DM_Rprt = -1;
static gint ett_nbap_RL_Set_InformationItem_DM_Rprt = -1;
static gint ett_nbap_DedicatedMeasurementTerminationRequest = -1;
static gint ett_nbap_DedicatedMeasurementFailureIndication = -1;
static gint ett_nbap_RadioLinkFailureIndication = -1;
static gint ett_nbap_Reporting_Object_RL_FailureInd = -1;
static gint ett_nbap_RL_RL_FailureInd = -1;
static gint ett_nbap_RL_InformationList_RL_FailureInd = -1;
static gint ett_nbap_RL_InformationItem_RL_FailureInd = -1;
static gint ett_nbap_RL_Set_RL_FailureInd = -1;
static gint ett_nbap_RL_Set_InformationList_RL_FailureInd = -1;
static gint ett_nbap_RL_Set_InformationItem_RL_FailureInd = -1;
static gint ett_nbap_CCTrCH_RL_FailureInd = -1;
static gint ett_nbap_CCTrCH_InformationList_RL_FailureInd = -1;
static gint ett_nbap_CCTrCH_InformationItem_RL_FailureInd = -1;
static gint ett_nbap_RadioLinkPreemptionRequiredIndication = -1;
static gint ett_nbap_RL_InformationList_RL_PreemptRequiredInd = -1;
static gint ett_nbap_RL_InformationItem_RL_PreemptRequiredInd = -1;
static gint ett_nbap_RadioLinkRestoreIndication = -1;
static gint ett_nbap_Reporting_Object_RL_RestoreInd = -1;
static gint ett_nbap_RL_RL_RestoreInd = -1;
static gint ett_nbap_RL_InformationList_RL_RestoreInd = -1;
static gint ett_nbap_RL_InformationItem_RL_RestoreInd = -1;
static gint ett_nbap_RL_Set_RL_RestoreInd = -1;
static gint ett_nbap_RL_Set_InformationList_RL_RestoreInd = -1;
static gint ett_nbap_RL_Set_InformationItem_RL_RestoreInd = -1;
static gint ett_nbap_CCTrCH_RL_RestoreInd = -1;
static gint ett_nbap_CCTrCH_InformationList_RL_RestoreInd = -1;
static gint ett_nbap_CCTrCH_InformationItem_RL_RestoreInd = -1;
static gint ett_nbap_CompressedModeCommand = -1;
static gint ett_nbap_ErrorIndication = -1;
static gint ett_nbap_PrivateMessage = -1;
static gint ett_nbap_PhysicalSharedChannelReconfigurationRequestFDD = -1;
static gint ett_nbap_HSDPA_And_EDCH_CellPortion_InformationList_PSCH_ReconfRqst = -1;
static gint ett_nbap_HSDPA_And_EDCH_CellPortion_InformationItem_PSCH_ReconfRqst = -1;
static gint ett_nbap_PhysicalSharedChannelReconfigurationRequestTDD = -1;
static gint ett_nbap_PDSCHSets_AddList_PSCH_ReconfRqst = -1;
static gint ett_nbap_PDSCHSets_AddItem_PSCH_ReconfRqst = -1;
static gint ett_nbap_PDSCH_Information_AddItem_PSCH_ReconfRqst = -1;
static gint ett_nbap_DL_Timeslot_InformationAddList_PSCH_ReconfRqst = -1;
static gint ett_nbap_DL_Timeslot_InformationAddItem_PSCH_ReconfRqst = -1;
static gint ett_nbap_DL_Code_InformationAddList_PSCH_ReconfRqst = -1;
static gint ett_nbap_DL_Code_InformationAddItem_PSCH_ReconfRqst = -1;
static gint ett_nbap_PDSCH_AddInformation_LCR_AddItem_PSCH_ReconfRqst = -1;
static gint ett_nbap_DL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst = -1;
static gint ett_nbap_DL_Timeslot_InformationAddItem_LCR_PSCH_ReconfRqst = -1;
static gint ett_nbap_DL_Code_InformationAddList_LCR_PSCH_ReconfRqst = -1;
static gint ett_nbap_DL_Code_InformationAddItem_LCR_PSCH_ReconfRqst = -1;
static gint ett_nbap_PDSCHSets_ModifyList_PSCH_ReconfRqst = -1;
static gint ett_nbap_PDSCHSets_ModifyItem_PSCH_ReconfRqst = -1;
static gint ett_nbap_PDSCH_Information_ModifyItem_PSCH_ReconfRqst = -1;
static gint ett_nbap_DL_Timeslot_InformationModifyList_PSCH_ReconfRqst = -1;
static gint ett_nbap_DL_Timeslot_InformationModifyItem_PSCH_ReconfRqst = -1;
static gint ett_nbap_DL_Code_InformationModifyList_PSCH_ReconfRqst = -1;
static gint ett_nbap_DL_Code_InformationModifyItem_PSCH_ReconfRqst = -1;
static gint ett_nbap_PDSCH_ModifyInformation_LCR_ModifyItem_PSCH_ReconfRqst = -1;
static gint ett_nbap_DL_Timeslot_LCR_InformationModifyList_PSCH_ReconfRqst = -1;
static gint ett_nbap_DL_Timeslot_LCR_InformationModifyItem_PSCH_ReconfRqst = -1;
static gint ett_nbap_DL_Code_LCR_InformationModifyList_PSCH_ReconfRqst = -1;
static gint ett_nbap_DL_Code_LCR_InformationModifyItem_PSCH_ReconfRqst = -1;
static gint ett_nbap_PDSCHSets_DeleteList_PSCH_ReconfRqst = -1;
static gint ett_nbap_PDSCHSets_DeleteItem_PSCH_ReconfRqst = -1;
static gint ett_nbap_PUSCHSets_AddList_PSCH_ReconfRqst = -1;
static gint ett_nbap_PUSCHSets_AddItem_PSCH_ReconfRqst = -1;
static gint ett_nbap_PUSCH_Information_AddItem_PSCH_ReconfRqst = -1;
static gint ett_nbap_UL_Timeslot_InformationAddList_PSCH_ReconfRqst = -1;
static gint ett_nbap_UL_Timeslot_InformationAddItem_PSCH_ReconfRqst = -1;
static gint ett_nbap_UL_Code_InformationAddList_PSCH_ReconfRqst = -1;
static gint ett_nbap_UL_Code_InformationAddItem_PSCH_ReconfRqst = -1;
static gint ett_nbap_PUSCH_AddInformation_LCR_AddItem_PSCH_ReconfRqst = -1;
static gint ett_nbap_UL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst = -1;
static gint ett_nbap_UL_Timeslot_InformationAddItem_LCR_PSCH_ReconfRqst = -1;
static gint ett_nbap_UL_Code_InformationAddList_LCR_PSCH_ReconfRqst = -1;
static gint ett_nbap_UL_Code_InformationAddItem_LCR_PSCH_ReconfRqst = -1;
static gint ett_nbap_PUSCHSets_ModifyList_PSCH_ReconfRqst = -1;
static gint ett_nbap_PUSCHSets_ModifyItem_PSCH_ReconfRqst = -1;
static gint ett_nbap_PUSCH_Information_ModifyItem_PSCH_ReconfRqst = -1;
static gint ett_nbap_UL_Timeslot_InformationModifyList_PSCH_ReconfRqst = -1;
static gint ett_nbap_UL_Timeslot_InformationModifyItem_PSCH_ReconfRqst = -1;
static gint ett_nbap_UL_Code_InformationModifyList_PSCH_ReconfRqst = -1;
static gint ett_nbap_UL_Code_InformationModifyItem_PSCH_ReconfRqst = -1;
static gint ett_nbap_PUSCH_ModifyInformation_LCR_ModifyItem_PSCH_ReconfRqst = -1;
static gint ett_nbap_UL_Timeslot_LCR_InformationModifyList_PSCH_ReconfRqst = -1;
static gint ett_nbap_UL_Timeslot_LCR_InformationModifyItem_PSCH_ReconfRqst = -1;
static gint ett_nbap_UL_Code_LCR_InformationModifyList_PSCH_ReconfRqst = -1;
static gint ett_nbap_UL_Code_LCR_InformationModifyItem_PSCH_ReconfRqst = -1;
static gint ett_nbap_PUSCHSets_DeleteList_PSCH_ReconfRqst = -1;
static gint ett_nbap_PUSCHSets_DeleteItem_PSCH_ReconfRqst = -1;
static gint ett_nbap_HS_PDSCH_TDD_Information_PSCH_ReconfRqst = -1;
static gint ett_nbap_DL_HS_PDSCH_Timeslot_Information_PSCH_ReconfRqst = -1;
static gint ett_nbap_DL_HS_PDSCH_Timeslot_InformationItem_PSCH_ReconfRqst = -1;
static gint ett_nbap_DL_HS_PDSCH_Codelist_PSCH_ReconfRqst = -1;
static gint ett_nbap_DL_HS_PDSCH_Timeslot_Information_LCR_PSCH_ReconfRqst = -1;
static gint ett_nbap_DL_HS_PDSCH_Timeslot_InformationItem_LCR_PSCH_ReconfRqst = -1;
static gint ett_nbap_DL_HS_PDSCH_Codelist_LCR_PSCH_ReconfRqst = -1;
static gint ett_nbap_Add_To_HS_SCCH_Resource_Pool_PSCH_ReconfRqst = -1;
static gint ett_nbap_HS_SCCH_Information_PSCH_ReconfRqst = -1;
static gint ett_nbap_HS_SCCH_InformationItem_PSCH_ReconfRqst = -1;
static gint ett_nbap_HS_SICH_Information_PSCH_ReconfRqst = -1;
static gint ett_nbap_HS_SCCH_Information_LCR_PSCH_ReconfRqst = -1;
static gint ett_nbap_HS_SCCH_InformationItem_LCR_PSCH_ReconfRqst = -1;
static gint ett_nbap_HS_SICH_Information_LCR_PSCH_ReconfRqst = -1;
static gint ett_nbap_Modify_HS_SCCH_Resource_Pool_PSCH_ReconfRqst = -1;
static gint ett_nbap_HS_SCCH_InformationModify_PSCH_ReconfRqst = -1;
static gint ett_nbap_HS_SCCH_InformationModifyItem_PSCH_ReconfRqst = -1;
static gint ett_nbap_HS_SICH_InformationModify_PSCH_ReconfRqst = -1;
static gint ett_nbap_HS_SCCH_InformationModify_LCR_PSCH_ReconfRqst = -1;
static gint ett_nbap_HS_SCCH_InformationModifyItem_LCR_PSCH_ReconfRqst = -1;
static gint ett_nbap_HS_SICH_InformationModify_LCR_PSCH_ReconfRqst = -1;
static gint ett_nbap_Delete_From_HS_SCCH_Resource_Pool_PSCH_ReconfRqst = -1;
static gint ett_nbap_Delete_From_HS_SCCH_Resource_PoolItem_PSCH_ReconfRqst = -1;
static gint ett_nbap_PhysicalSharedChannelReconfigurationResponse = -1;
static gint ett_nbap_PhysicalSharedChannelReconfigurationFailure = -1;
static gint ett_nbap_CauseLevel_PSCH_ReconfFailure = -1;
static gint ett_nbap_GeneralCauseList_PSCH_ReconfFailure = -1;
static gint ett_nbap_SetSpecificCauseList_PSCH_ReconfFailureTDD = -1;
static gint ett_nbap_Unsuccessful_PDSCHSetList_PSCH_ReconfFailureTDD = -1;
static gint ett_nbap_Unsuccessful_PDSCHSetItem_PSCH_ReconfFailureTDD = -1;
static gint ett_nbap_Unsuccessful_PUSCHSetList_PSCH_ReconfFailureTDD = -1;
static gint ett_nbap_Unsuccessful_PUSCHSetItem_PSCH_ReconfFailureTDD = -1;
static gint ett_nbap_ResetRequest = -1;
static gint ett_nbap_ResetIndicator = -1;
static gint ett_nbap_CommunicationContextList_Reset = -1;
static gint ett_nbap_CommunicationContextInfoList_Reset = -1;
static gint ett_nbap_CommunicationContextInfoItem_Reset = -1;
static gint ett_nbap_CommunicationContextType_Reset = -1;
static gint ett_nbap_CommunicationControlPortList_Reset = -1;
static gint ett_nbap_CommunicationControlPortInfoList_Reset = -1;
static gint ett_nbap_CommunicationControlPortInfoItem_Reset = -1;
static gint ett_nbap_ResetResponse = -1;
static gint ett_nbap_InformationExchangeInitiationRequest = -1;
static gint ett_nbap_InformationExchangeObjectType_InfEx_Rqst = -1;
static gint ett_nbap_Cell_InfEx_Rqst = -1;
static gint ett_nbap_InformationExchangeInitiationResponse = -1;
static gint ett_nbap_InformationExchangeObjectType_InfEx_Rsp = -1;
static gint ett_nbap_Cell_InfEx_Rsp = -1;
static gint ett_nbap_InformationExchangeInitiationFailure = -1;
static gint ett_nbap_InformationReport = -1;
static gint ett_nbap_InformationExchangeObjectType_InfEx_Rprt = -1;
static gint ett_nbap_Cell_Inf_Rprt = -1;
static gint ett_nbap_InformationExchangeTerminationRequest = -1;
static gint ett_nbap_InformationExchangeFailureIndication = -1;
static gint ett_nbap_CellSynchronisationInitiationRequestTDD = -1;
static gint ett_nbap_CellSyncBurstTransInit_CellSyncInitiationRqstTDD = -1;
static gint ett_nbap_TimeslotInfo_CellSyncInitiationRqstTDD = -1;
static gint ett_nbap_CellSyncBurstMeasureInit_CellSyncInitiationRqstTDD = -1;
static gint ett_nbap_SYNCDlCodeId_TransInitLCR_CellSyncInitiationRqstTDD = -1;
static gint ett_nbap_SYNCDlCodeId_MeasureInitLCR_CellSyncInitiationRqstTDD = -1;
static gint ett_nbap_CellSynchronisationInitiationResponseTDD = -1;
static gint ett_nbap_CellSynchronisationInitiationFailureTDD = -1;
static gint ett_nbap_CellSynchronisationReconfigurationRequestTDD = -1;
static gint ett_nbap_CellSyncBurstTransReconfInfo_CellSyncReconfRqstTDD = -1;
static gint ett_nbap_CellSyncBurstTransInfoItem_CellSyncReconfRqstTDD = -1;
static gint ett_nbap_CellSyncBurstMeasInfo_CellSyncReconfRqstTDD = -1;
static gint ett_nbap_CellSyncBurstMeasInfoListIE_CellSyncReconfRqstTDD = -1;
static gint ett_nbap_CellSyncBurstMeasInfoItem_CellSyncReconfRqstTDD = -1;
static gint ett_nbap_CellSyncBurstInfoList_CellSyncReconfRqstTDD = -1;
static gint ett_nbap_CellSyncBurstInfoItem_CellSyncReconfRqstTDD = -1;
static gint ett_nbap_SYNCDlCodeIdTransReconfInfoLCR_CellSyncReconfRqstTDD = -1;
static gint ett_nbap_SYNCDlCodeIdTransReconfItemLCR_CellSyncReconfRqstTDD = -1;
static gint ett_nbap_SYNCDlCodeIdMeasInfoLCR_CellSyncReconfRqstTDD = -1;
static gint ett_nbap_SYNCDlCodeIdMeasInfoList_CellSyncReconfRqstTDD = -1;
static gint ett_nbap_SYNCDlCodeIdMeasInfoItem_CellSyncReconfRqstTDD = -1;
static gint ett_nbap_SYNCDlCodeIdInfoListLCR_CellSyncReconfRqstTDD = -1;
static gint ett_nbap_SYNCDlCodeIdInfoItemLCR_CellSyncReconfRqstTDD = -1;
static gint ett_nbap_CellSynchronisationReconfigurationResponseTDD = -1;
static gint ett_nbap_CellSynchronisationReconfigurationFailureTDD = -1;
static gint ett_nbap_CellSynchronisationAdjustmentRequestTDD = -1;
static gint ett_nbap_CellAdjustmentInfo_SyncAdjustmentRqstTDD = -1;
static gint ett_nbap_CellAdjustmentInfoItem_SyncAdjustmentRqstTDD = -1;
static gint ett_nbap_CellSynchronisationAdjustmentResponseTDD = -1;
static gint ett_nbap_CellSynchronisationAdjustmentFailureTDD = -1;
static gint ett_nbap_CauseLevel_SyncAdjustmntFailureTDD = -1;
static gint ett_nbap_GeneralCauseList_SyncAdjustmntFailureTDD = -1;
static gint ett_nbap_CellSpecificCauseList_SyncAdjustmntFailureTDD = -1;
static gint ett_nbap_Unsuccessful_cell_InformationRespList_SyncAdjustmntFailureTDD = -1;
static gint ett_nbap_Unsuccessful_cell_InformationRespItem_SyncAdjustmntFailureTDD = -1;
static gint ett_nbap_CellSynchronisationTerminationRequestTDD = -1;
static gint ett_nbap_CellSynchronisationFailureIndicationTDD = -1;
static gint ett_nbap_CellSynchronisationReportTDD = -1;
static gint ett_nbap_CellSyncInfo_CellSyncReprtTDD = -1;
static gint ett_nbap_CellSyncInfoItemIE_CellSyncReprtTDD = -1;
static gint ett_nbap_SyncReportType_CellSyncReprtTDD = -1;
static gint ett_nbap_IntStdPhCellSyncInfo_CellSyncReprtTDD = -1;
static gint ett_nbap_CellSyncBurstMeasInfoList_CellSyncReprtTDD = -1;
static gint ett_nbap_CellSyncBurstMeasInfoItem_CellSyncReprtTDD = -1;
static gint ett_nbap_SEQUENCE_SIZE_1_16_OF_CellSyncBurstInfo_CellSyncReprtTDD = -1;
static gint ett_nbap_CellSyncBurstInfo_CellSyncReprtTDD = -1;
static gint ett_nbap_CellSyncBurstAvailable_CellSyncReprtTDD = -1;
static gint ett_nbap_SyncDLCodeIdsMeasInfoList_CellSyncReprtTDD = -1;
static gint ett_nbap_SyncDLCodeIdsMeasInfoItem_CellSyncReprtTDD = -1;
static gint ett_nbap_SyncDLCodeIdInfo_CellSyncReprtTDD = -1;
static gint ett_nbap_SyncDLCodeIdItem_CellSyncReprtTDD = -1;
static gint ett_nbap_SyncDLCodeIdAvailable_CellSyncReprtTDD = -1;
static gint ett_nbap_BearerRearrangementIndication = -1;
static gint ett_nbap_DCH_RearrangeList_Bearer_RearrangeInd = -1;
static gint ett_nbap_DCH_RearrangeItem_Bearer_RearrangeInd = -1;
static gint ett_nbap_DSCH_RearrangeList_Bearer_RearrangeInd = -1;
static gint ett_nbap_DSCH_RearrangeItem_Bearer_RearrangeInd = -1;
static gint ett_nbap_USCH_RearrangeList_Bearer_RearrangeInd = -1;
static gint ett_nbap_USCH_RearrangeItem_Bearer_RearrangeInd = -1;
static gint ett_nbap_HSDSCH_RearrangeList_Bearer_RearrangeInd = -1;
static gint ett_nbap_HSDSCH_RearrangeItem_Bearer_RearrangeInd = -1;
static gint ett_nbap_E_DCH_RearrangeList_Bearer_RearrangeInd = -1;
static gint ett_nbap_E_DCH_RearrangeItem_Bearer_RearrangeInd = -1;
static gint ett_nbap_RadioLinkActivationCommandFDD = -1;
static gint ett_nbap_DelayedActivationInformationList_RL_ActivationCmdFDD = -1;
static gint ett_nbap_DelayedActivationInformation_RL_ActivationCmdFDD = -1;
static gint ett_nbap_RadioLinkActivationCommandTDD = -1;
static gint ett_nbap_DelayedActivationInformationList_RL_ActivationCmdTDD = -1;
static gint ett_nbap_DelayedActivationInformation_RL_ActivationCmdTDD = -1;
static gint ett_nbap_RadioLinkParameterUpdateIndicationFDD = -1;
static gint ett_nbap_RadioLinkParameterUpdateIndicationTDD = -1;
static gint ett_nbap_MBMSNotificationUpdateCommand = -1;
static gint ett_nbap_CommonMeasurementReport = -1;
static gint ett_nbap_CommonMeasurementObjectType_CM_Rprt = -1;
static gint ett_nbap_Cell_CM_Rprt = -1;
static gint ett_nbap_RACH_CM_Rprt = -1;
static gint ett_nbap_PowerLocalCellGroup_CM_Rprt = -1;
static gint ett_nbap_CommonMeasurementTerminationRequest = -1;
static gint ett_nbap_CommonMeasurementFailureIndication = -1;
static gint ett_nbap_Active_Pattern_Sequence_Information = -1;
static gint ett_nbap_Transmission_Gap_Pattern_Sequence_Status_List = -1;
static gint ett_nbap_Transmission_Gap_Pattern_Sequence_Status_List_item = -1;
static gint ett_nbap_AllocationRetentionPriority = -1;
static gint ett_nbap_Angle_Of_Arrival_Value_LCR = -1;
static gint ett_nbap_Best_Cell_Portions_Value = -1;
static gint ett_nbap_Best_Cell_Portions_Item = -1;
static gint ett_nbap_Cause = -1;
static gint ett_nbap_CellSyncBurstTiming = -1;
static gint ett_nbap_CellSyncBurstTimingLCR = -1;
static gint ett_nbap_CommonChannelsCapacityConsumptionLaw = -1;
static gint ett_nbap_CommonChannelsCapacityConsumptionLaw_item = -1;
static gint ett_nbap_CommonMeasurementAccuracy = -1;
static gint ett_nbap_CommonMeasurementValue = -1;
static gint ett_nbap_CommonMeasurementValueInformation = -1;
static gint ett_nbap_CommonMeasurementAvailable = -1;
static gint ett_nbap_Common_PhysicalChannel_Status_Information = -1;
static gint ett_nbap_CommonTransportChannel_InformationResponse = -1;
static gint ett_nbap_Common_TransportChannel_Status_Information = -1;
static gint ett_nbap_CriticalityDiagnostics = -1;
static gint ett_nbap_CriticalityDiagnostics_IE_List = -1;
static gint ett_nbap_CriticalityDiagnostics_IE_List_item = -1;
static gint ett_nbap_DCH_FDD_Information = -1;
static gint ett_nbap_DCH_FDD_InformationItem = -1;
static gint ett_nbap_DCH_Specific_FDD_InformationList = -1;
static gint ett_nbap_DCH_Specific_FDD_Item = -1;
static gint ett_nbap_DCH_InformationResponse = -1;
static gint ett_nbap_DCH_InformationResponseItem = -1;
static gint ett_nbap_DCH_TDD_Information = -1;
static gint ett_nbap_DCH_TDD_InformationItem = -1;
static gint ett_nbap_DCH_Specific_TDD_InformationList = -1;
static gint ett_nbap_DCH_Specific_TDD_Item = -1;
static gint ett_nbap_FDD_DCHs_to_Modify = -1;
static gint ett_nbap_FDD_DCHs_to_ModifyItem = -1;
static gint ett_nbap_DCH_ModifySpecificInformation_FDD = -1;
static gint ett_nbap_DCH_ModifySpecificItem_FDD = -1;
static gint ett_nbap_TDD_DCHs_to_Modify = -1;
static gint ett_nbap_DCH_ModifyItem_TDD = -1;
static gint ett_nbap_DCH_ModifySpecificInformation_TDD = -1;
static gint ett_nbap_DCH_ModifySpecificItem_TDD = -1;
static gint ett_nbap_DedicatedChannelsCapacityConsumptionLaw = -1;
static gint ett_nbap_DedicatedChannelsCapacityConsumptionLaw_item = -1;
static gint ett_nbap_DedicatedMeasurementValue = -1;
static gint ett_nbap_DedicatedMeasurementValueInformation = -1;
static gint ett_nbap_DedicatedMeasurementAvailable = -1;
static gint ett_nbap_DelayedActivation = -1;
static gint ett_nbap_DelayedActivationUpdate = -1;
static gint ett_nbap_Activate_Info = -1;
static gint ett_nbap_Deactivate_Info = -1;
static gint ett_nbap_Execution_Type = -1;
static gint ett_nbap_DGPSCorrections = -1;
static gint ett_nbap_DGPSThresholds = -1;
static gint ett_nbap_DL_Timeslot_Information = -1;
static gint ett_nbap_DL_Timeslot_InformationItem = -1;
static gint ett_nbap_DL_TimeslotLCR_Information = -1;
static gint ett_nbap_DL_TimeslotLCR_InformationItem = -1;
static gint ett_nbap_DL_PowerBalancing_Information = -1;
static gint ett_nbap_DL_ReferencePowerInformationList = -1;
static gint ett_nbap_DL_ReferencePowerInformationItem = -1;
static gint ett_nbap_DL_TimeslotISCPInfo = -1;
static gint ett_nbap_DL_TimeslotISCPInfoItem = -1;
static gint ett_nbap_DL_TimeslotISCPInfoLCR = -1;
static gint ett_nbap_DL_TimeslotISCPInfoItemLCR = -1;
static gint ett_nbap_DSCH_InformationResponse = -1;
static gint ett_nbap_DSCH_InformationResponseItem = -1;
static gint ett_nbap_DSCH_TDD_Information = -1;
static gint ett_nbap_DSCH_TDD_InformationItem = -1;
static gint ett_nbap_E_AGCH_FDD_Code_Information = -1;
static gint ett_nbap_E_AGCH_FDD_Code_List = -1;
static gint ett_nbap_E_DCH_FDD_DL_Control_Channel_Information = -1;
static gint ett_nbap_E_DCH_FDD_Information = -1;
static gint ett_nbap_E_DCH_FDD_Information_Response = -1;
static gint ett_nbap_E_DCH_FDD_Information_to_Modify = -1;
static gint ett_nbap_E_DCH_Grant_Type_Information = -1;
static gint ett_nbap_E_DCH_LogicalChannelInformation = -1;
static gint ett_nbap_E_DCH_LogicalChannelInformationItem = -1;
static gint ett_nbap_E_DCH_MACdPDU_SizeList = -1;
static gint ett_nbap_E_DCH_MACdPDU_SizeListItem = -1;
static gint ett_nbap_E_DCH_LogicalChannelToModify = -1;
static gint ett_nbap_E_DCH_LogicalChannelToModifyItem = -1;
static gint ett_nbap_E_DCH_MACdPDU_SizeToModifyList = -1;
static gint ett_nbap_E_DCH_LogicalChannelToDelete = -1;
static gint ett_nbap_E_DCH_LogicalChannelToDeleteItem = -1;
static gint ett_nbap_E_DCH_MACdFlows_Information = -1;
static gint ett_nbap_E_DCH_MACdFlow_Specific_InfoList = -1;
static gint ett_nbap_E_DCH_MACdFlow_Specific_InfoItem = -1;
static gint ett_nbap_E_DCH_MACdFlow_Specific_InformationResp = -1;
static gint ett_nbap_E_DCH_MACdFlow_Specific_InformationResp_Item = -1;
static gint ett_nbap_E_DCH_MACdFlow_Specific_InfoList_to_Modify = -1;
static gint ett_nbap_E_DCH_MACdFlow_Specific_InfoItem_to_Modify = -1;
static gint ett_nbap_E_DCH_MACdFlows_to_Delete = -1;
static gint ett_nbap_E_DCH_MACdFlow_to_Delete_Item = -1;
static gint ett_nbap_E_DCH_Non_Scheduled_Transmission_Grant_Items = -1;
static gint ett_nbap_E_RGCH_E_HICH_FDD_Code_Information = -1;
static gint ett_nbap_E_RGCH_E_HICH_FDD_Code_List = -1;
static gint ett_nbap_E_TFCS_Information = -1;
static gint ett_nbap_E_DCHProvidedBitRate = -1;
static gint ett_nbap_E_DCHProvidedBitRate_Item = -1;
static gint ett_nbap_FDD_DL_CodeInformation = -1;
static gint ett_nbap_FDD_DL_CodeInformationItem = -1;
static gint ett_nbap_GPS_Almanac = -1;
static gint ett_nbap_GPS_Ionospheric_Model = -1;
static gint ett_nbap_GPS_Information = -1;
static gint ett_nbap_GPS_RealTime_Integrity = -1;
static gint ett_nbap_GPSBadSat_Info_RealTime_Integrity = -1;
static gint ett_nbap_GPS_NavigationModel_and_TimeRecovery = -1;
static gint ett_nbap_GPS_NavandRecovery_Item = -1;
static gint ett_nbap_GPS_RX_POS = -1;
static gint ett_nbap_GPS_UTC_Model = -1;
static gint ett_nbap_HARQ_MemoryPartitioning = -1;
static gint ett_nbap_HARQ_MemoryPartitioning_Implicit = -1;
static gint ett_nbap_HARQ_MemoryPartitioning_Explicit = -1;
static gint ett_nbap_HARQ_MemoryPartitioningList = -1;
static gint ett_nbap_HARQ_MemoryPartitioningItem = -1;
static gint ett_nbap_HS_DSCHProvidedBitRate = -1;
static gint ett_nbap_HS_DSCHProvidedBitRate_Item = -1;
static gint ett_nbap_HS_DSCHProvidedBitRateValueInformation_For_CellPortion = -1;
static gint ett_nbap_HS_DSCHProvidedBitRateValueInformation_For_CellPortion_Item = -1;
static gint ett_nbap_HS_DSCHRequiredPower = -1;
static gint ett_nbap_HS_DSCHRequiredPower_Item = -1;
static gint ett_nbap_HS_DSCHRequiredPowerPerUEInformation = -1;
static gint ett_nbap_HS_DSCHRequiredPowerPerUEInformation_Item = -1;
static gint ett_nbap_HS_DSCHRequiredPowerValueInformation_For_CellPortion = -1;
static gint ett_nbap_HS_DSCHRequiredPowerValueInformation_For_CellPortion_Item = -1;
static gint ett_nbap_HSDSCH_FDD_Information = -1;
static gint ett_nbap_HSDSCH_TDD_Information = -1;
static gint ett_nbap_HSDSCH_Information_to_Modify = -1;
static gint ett_nbap_HSDSCH_MACdFlow_Specific_InfoList_to_Modify = -1;
static gint ett_nbap_HSDSCH_MACdFlow_Specific_InfoItem_to_Modify = -1;
static gint ett_nbap_HSDSCH_Information_to_Modify_Unsynchronised = -1;
static gint ett_nbap_HSDSCH_FDD_Information_Response = -1;
static gint ett_nbap_HSDSCH_TDD_Information_Response = -1;
static gint ett_nbap_HSDSCH_MACdFlow_Specific_InformationResp = -1;
static gint ett_nbap_HSDSCH_MACdFlow_Specific_InformationResp_Item = -1;
static gint ett_nbap_HSDSCH_MACdFlows_Information = -1;
static gint ett_nbap_HSDSCH_MACdFlow_Specific_InfoList = -1;
static gint ett_nbap_HSDSCH_MACdFlow_Specific_InfoItem = -1;
static gint ett_nbap_HSDSCH_MACdFlows_to_Delete = -1;
static gint ett_nbap_HSDSCH_MACdFlows_to_Delete_Item = -1;
static gint ett_nbap_HSDSCH_Initial_Capacity_Allocation = -1;
static gint ett_nbap_HSDSCH_Initial_Capacity_AllocationItem = -1;
static gint ett_nbap_HSSCCH_Specific_InformationRespListFDD = -1;
static gint ett_nbap_HSSCCH_Codes = -1;
static gint ett_nbap_HSSCCH_Specific_InformationRespListTDD = -1;
static gint ett_nbap_HSSCCH_Specific_InformationRespItemTDD = -1;
static gint ett_nbap_HSSCCH_Specific_InformationRespListTDDLCR = -1;
static gint ett_nbap_HSSCCH_Specific_InformationRespItemTDDLCR = -1;
static gint ett_nbap_HSSICH_Info = -1;
static gint ett_nbap_HSSICH_InfoLCR = -1;
static gint ett_nbap_HS_SICH_Reception_Quality_Value = -1;
static gint ett_nbap_HS_PDSCH_FDD_Code_Information = -1;
static gint ett_nbap_HS_SCCH_FDD_Code_Information = -1;
static gint ett_nbap_HS_SCCH_FDD_Code_List = -1;
static gint ett_nbap_HSDSCH_FDD_Update_Information = -1;
static gint ett_nbap_HSDSCH_TDD_Update_Information = -1;
static gint ett_nbap_InformationReportCharacteristics = -1;
static gint ett_nbap_InformationReportCharacteristicsType_ReportPeriodicity = -1;
static gint ett_nbap_InformationReportCharacteristicsType_OnModification = -1;
static gint ett_nbap_InformationThresholds = -1;
static gint ett_nbap_InformationType = -1;
static gint ett_nbap_IPDL_FDD_Parameters = -1;
static gint ett_nbap_IPDL_TDD_Parameters = -1;
static gint ett_nbap_IPDL_TDD_Parameters_LCR = -1;
static gint ett_nbap_BurstModeParams = -1;
static gint ett_nbap_MACdPDU_Size_Indexlist = -1;
static gint ett_nbap_MACdPDU_Size_IndexItem = -1;
static gint ett_nbap_MACdPDU_Size_Indexlist_to_Modify = -1;
static gint ett_nbap_MACdPDU_Size_IndexItem_to_Modify = -1;
static gint ett_nbap_MessageStructure = -1;
static gint ett_nbap_MessageStructure_item = -1;
static gint ett_nbap_MidambleShiftAndBurstType = -1;
static gint ett_nbap_T_type1 = -1;
static gint ett_nbap_T_midambleAllocationMode = -1;
static gint ett_nbap_T_type2 = -1;
static gint ett_nbap_T_midambleAllocationMode1 = -1;
static gint ett_nbap_T_type3 = -1;
static gint ett_nbap_T_midambleAllocationMode2 = -1;
static gint ett_nbap_MidambleShiftLCR = -1;
static gint ett_nbap_ModifyPriorityQueue = -1;
static gint ett_nbap_NeighbouringCellMeasurementInformation = -1;
static gint ett_nbap_NeighbouringCellMeasurementInformation_item = -1;
static gint ett_nbap_NeighbouringFDDCellMeasurementInformation = -1;
static gint ett_nbap_NeighbouringTDDCellMeasurementInformation = -1;
static gint ett_nbap_NeighbouringTDDCellMeasurementInformationLCR = -1;
static gint ett_nbap_NI_Information = -1;
static gint ett_nbap_PreambleSignatures = -1;
static gint ett_nbap_PriorityQueue_InfoList = -1;
static gint ett_nbap_PriorityQueue_InfoItem = -1;
static gint ett_nbap_PriorityQueue_InfoList_to_Modify = -1;
static gint ett_nbap_PriorityQueue_InfoItem_to_Add = -1;
static gint ett_nbap_PriorityQueue_InfoItem_to_Modify = -1;
static gint ett_nbap_PriorityQueue_InfoList_to_Modify_Unsynchronised = -1;
static gint ett_nbap_PriorityQueue_InfoItem_to_Modify_Unsynchronised = -1;
static gint ett_nbap_RACH_SubChannelNumbers = -1;
static gint ett_nbap_RL_Specific_DCH_Info = -1;
static gint ett_nbap_RL_Specific_DCH_Info_Item = -1;
static gint ett_nbap_RL_Specific_E_DCH_Info = -1;
static gint ett_nbap_RL_Specific_E_DCH_Info_Item = -1;
static gint ett_nbap_Reference_E_TFCI_Information = -1;
static gint ett_nbap_Reference_E_TFCI_Information_Item = -1;
static gint ett_nbap_ReportCharacteristics = -1;
static gint ett_nbap_ReportCharacteristicsType_EventA = -1;
static gint ett_nbap_ReportCharacteristicsType_EventB = -1;
static gint ett_nbap_ReportCharacteristicsType_EventC = -1;
static gint ett_nbap_ReportCharacteristicsType_EventD = -1;
static gint ett_nbap_ReportCharacteristicsType_EventE = -1;
static gint ett_nbap_ReportCharacteristicsType_EventF = -1;
static gint ett_nbap_ReportCharacteristicsType_OnModification = -1;
static gint ett_nbap_ReportCharacteristicsType_MeasurementIncreaseDecreaseThreshold = -1;
static gint ett_nbap_ReportCharacteristicsType_MeasurementThreshold = -1;
static gint ett_nbap_ReportCharacteristicsType_ScaledMeasurementChangeTime = -1;
static gint ett_nbap_ReportCharacteristicsType_ScaledMeasurementHysteresisTime = -1;
static gint ett_nbap_ReportCharacteristicsType_ReportPeriodicity = -1;
static gint ett_nbap_Received_total_wide_band_power_For_CellPortion_Value = -1;
static gint ett_nbap_Received_total_wide_band_power_For_CellPortion_Value_Item = -1;
static gint ett_nbap_RequestedDataValueInformation = -1;
static gint ett_nbap_InformationAvailable = -1;
static gint ett_nbap_RequestedDataValue = -1;
static gint ett_nbap_SAT_Info_Almanac = -1;
static gint ett_nbap_SAT_Info_Almanac_Item = -1;
static gint ett_nbap_SAT_Info_Almanac_ExtList = -1;
static gint ett_nbap_SAT_Info_Almanac_ExtItem = -1;
static gint ett_nbap_SAT_Info_DGPSCorrections = -1;
static gint ett_nbap_SAT_Info_DGPSCorrections_Item = -1;
static gint ett_nbap_SATInfo_RealTime_Integrity = -1;
static gint ett_nbap_SAT_Info_RealTime_Integrity_Item = -1;
static gint ett_nbap_Secondary_CPICH_Information_Change = -1;
static gint ett_nbap_Serving_E_DCH_RL_ID = -1;
static gint ett_nbap_Serving_E_DCH_RL_in_this_NodeB = -1;
static gint ett_nbap_SFNSFNMeasurementThresholdInformation = -1;
static gint ett_nbap_SFNSFNMeasurementValueInformation = -1;
static gint ett_nbap_T_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation = -1;
static gint ett_nbap_T_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item = -1;
static gint ett_nbap_T_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation = -1;
static gint ett_nbap_T_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item = -1;
static gint ett_nbap_SFNSFNTimeStampInformation = -1;
static gint ett_nbap_SFNSFNTimeStamp_TDD = -1;
static gint ett_nbap_SFNSFNValue = -1;
static gint ett_nbap_SynchronisationReportCharacteristics = -1;
static gint ett_nbap_SynchronisationReportCharactThreExc = -1;
static gint ett_nbap_SynchronisationReportCharactThreInfoItem = -1;
static gint ett_nbap_SEQUENCE_SIZE_1_16_OF_SynchronisationReportCharactCellSyncBurstInfoItem = -1;
static gint ett_nbap_SynchronisationReportCharactCellSyncBurstInfoItem = -1;
static gint ett_nbap_SyncDLCodeIdThreInfoLCR = -1;
static gint ett_nbap_SyncDLCodeIdThreInfoList = -1;
static gint ett_nbap_SyncDLCodeInfoListLCR = -1;
static gint ett_nbap_SyncDLCodeInfoItemLCR = -1;
static gint ett_nbap_TDD_ChannelisationCodeLCR = -1;
static gint ett_nbap_TDD_DL_Code_Information = -1;
static gint ett_nbap_TDD_DL_Code_InformationItem = -1;
static gint ett_nbap_TDD_DL_Code_LCR_Information = -1;
static gint ett_nbap_TDD_DL_Code_LCR_InformationItem = -1;
static gint ett_nbap_TDD_DL_DPCH_TimeSlotFormat_LCR = -1;
static gint ett_nbap_TDD_DPCHOffset = -1;
static gint ett_nbap_TransportFormatCombination_Beta = -1;
static gint ett_nbap_T_signalledGainFactors = -1;
static gint ett_nbap_T_gainFactor = -1;
static gint ett_nbap_T_fdd = -1;
static gint ett_nbap_TDD_UL_Code_Information = -1;
static gint ett_nbap_TDD_UL_Code_InformationItem = -1;
static gint ett_nbap_TDD_UL_Code_LCR_Information = -1;
static gint ett_nbap_TDD_UL_Code_LCR_InformationItem = -1;
static gint ett_nbap_TDD_UL_DPCH_TimeSlotFormat_LCR = -1;
static gint ett_nbap_TFCI_SignallingMode = -1;
static gint ett_nbap_TimingAdjustmentValue = -1;
static gint ett_nbap_TimingAdjustmentValueLCR = -1;
static gint ett_nbap_TnlQos = -1;
static gint ett_nbap_Transmission_Gap_Pattern_Sequence_Information = -1;
static gint ett_nbap_Transmission_Gap_Pattern_Sequence_Information_item = -1;
static gint ett_nbap_TransmittedCarrierPowerOfAllCodesNotUsedForHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHOrE_HICHTransmissionCellPortionValue = -1;
static gint ett_nbap_TransmittedCarrierPowerOfAllCodesNotUsedForHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHOrE_HICHTransmissionCellPortionValue_Item = -1;
static gint ett_nbap_Transmitted_Carrier_Power_For_CellPortion_Value = -1;
static gint ett_nbap_Transmitted_Carrier_Power_For_CellPortion_Value_Item = -1;
static gint ett_nbap_TFCS = -1;
static gint ett_nbap_T_tFCSvalues = -1;
static gint ett_nbap_TFCS_TFCSList = -1;
static gint ett_nbap_TFCS_TFCSList_item = -1;
static gint ett_nbap_TFCS_CTFC = -1;
static gint ett_nbap_TransportFormatSet = -1;
static gint ett_nbap_TransportFormatSet_DynamicPartList = -1;
static gint ett_nbap_TransportFormatSet_DynamicPartList_item = -1;
static gint ett_nbap_TDD_TransportFormatSet_ModeDP = -1;
static gint ett_nbap_TransmissionTimeIntervalInformation = -1;
static gint ett_nbap_TransmissionTimeIntervalInformation_item = -1;
static gint ett_nbap_TransportFormatSet_Semi_staticPart = -1;
static gint ett_nbap_TransportFormatSet_ModeDP = -1;
static gint ett_nbap_TransportFormatSet_ModeSSP = -1;
static gint ett_nbap_TUTRANGPS = -1;
static gint ett_nbap_TUTRANGPSMeasurementThresholdInformation = -1;
static gint ett_nbap_TUTRANGPSMeasurementValueInformation = -1;
static gint ett_nbap_UC_Id = -1;
static gint ett_nbap_UE_Capability_Information = -1;
static gint ett_nbap_UL_Timeslot_Information = -1;
static gint ett_nbap_UL_Timeslot_InformationItem = -1;
static gint ett_nbap_UL_TimeslotLCR_Information = -1;
static gint ett_nbap_UL_TimeslotLCR_InformationItem = -1;
static gint ett_nbap_UL_ScramblingCode = -1;
static gint ett_nbap_UL_Synchronisation_Parameters_LCR = -1;
static gint ett_nbap_UL_TimeSlot_ISCP_Info = -1;
static gint ett_nbap_UL_TimeSlot_ISCP_InfoItem = -1;
static gint ett_nbap_UL_TimeSlot_ISCP_LCR_Info = -1;
static gint ett_nbap_UL_TimeSlot_ISCP_LCR_InfoItem = -1;
static gint ett_nbap_USCH_Information = -1;
static gint ett_nbap_USCH_InformationItem = -1;
static gint ett_nbap_USCH_InformationResponse = -1;
static gint ett_nbap_USCH_InformationResponseItem = -1;
static gint ett_nbap_PrivateIE_ID = -1;
static gint ett_nbap_ProcedureID = -1;
static gint ett_nbap_TransactionID = -1;
static gint ett_nbap_ProtocolIE_Container = -1;
static gint ett_nbap_ProtocolIE_Field = -1;
static gint ett_nbap_ProtocolIE_ContainerPair = -1;
static gint ett_nbap_ProtocolIE_FieldPair = -1;
static gint ett_nbap_ProtocolIE_ContainerList = -1;
static gint ett_nbap_ProtocolIE_ContainerPairList = -1;
static gint ett_nbap_ProtocolExtensionContainer = -1;
static gint ett_nbap_ProtocolExtensionField = -1;
static gint ett_nbap_PrivateIE_Container = -1;
static gint ett_nbap_PrivateIE_Field = -1;

/*--- End of included file: packet-nbap-ett.c ---*/
#line 67 "packet-nbap-template.c"

/* Global variables */
static proto_tree *top_tree;
static guint32 ProcedureCode;
static guint32 ProtocolIE_ID;
static guint32 ddMode;

#define BYTE_ALIGN_OFFSET(offset)		\
	if(offset&0x07){			\
		offset=(offset&0xfffffff8)+8;	\
	}
#define NBAP_FDD 1
/* Prodedure ID:s */
#define NBAP_ID_AUDIT 												0
#define NBAP_ID_AUDITREQUIRED 										1
#define NBAP_ID_BLOCKRESOURCE 										2
#define NBAP_ID_CELLDELETION 										3
#define NBAP_ID_CELLRECONFIGURATION 								4
#define NBAP_ID_CELLSETUP 											5
#define NBAP_ID_COMMONMEASUREMENTFAILURE 							6
#define NBAP_ID_COMMONMEASUREMENTINITIATION 						7
#define NBAP_ID_COMMONMEASUREMENTREPORT 							8
#define NBAP_ID_COMMONMEASUREMENTTERMINATION 						9
#define NBAP_ID_COMMONTRANSPORTCHANNELDELETE 						10
#define NBAP_ID_COMMONTRANSPORTCHANNELRECONFIGURE 					11
#define NBAP_ID_COMMONTRANSPORTCHANNELSETUP 						12
#define NBAP_ID_RESET												13
#define NBAP_ID_COMPRESSEDMODECOMMAND 								14
#define NBAP_ID_DEDICATEDMEASUREMENTFAILURE 						16
#define NBAP_ID_DEDICATEDMEASUREMENTINITIATION 						17
#define NBAP_ID_DEDICATEDMEASUREMENTREPORT 							18
#define NBAP_ID_DEDICATEDMEASUREMENTTERMINATION 					19
#define NBAP_ID_DOWNLINKPOWERCONTROL 								20
#define NBAP_ID_ERRORINDICATIONFORDEDICATED							21
#define NBAP_ID_PRIVATEMESSAGEFORDEDICATED							22
#define NBAP_ID_RADIOLINKADDITION 									23
#define NBAP_ID_RADIOLINKDELETION 									24
#define NBAP_ID_RADIOLINKFAILURE 									25
#define NBAP_ID_RADIOLINKRESTORATION 								26
#define NBAP_ID_RADIOLINKSETUP 										27
#define NBAP_ID_RESOURCESTATUSINDICATION 							28
#define NBAP_ID_SYNCHRONISEDRADIOLINKRECONFIGURATIONCANCELLATION	29
#define NBAP_ID_SYNCHRONISEDRADIOLINKRECONFIGURATIONCOMMIT			30
#define NBAP_ID_SYNCHRONISEDRADIOLINKRECONFIGURATIONPREPARATION 	31
#define NBAP_ID_SYSTEMINFORMATIONUPDATE 							32
#define NBAP_ID_UNBLOCKRESOURCE 									33
#define NBAP_ID_UNSYNCHRONISEDRADIOLINKRECONFIGURATION 				34
#define NBAP_ID_ERRORINDICATIONFORCOMMON							35
#define NBAP_ID_PRIVATEMESSAGEFORCOMMON								36
#define NBAP_ID_PHYSICALSHAREDCHANNELRECONFIGURATION 				37
#define NBAP_ID_DOWNLINKPOWERTIMESLOTCONTROL 						38
#define NBAP_ID_RADIOLINKPREEMPTION									39
#define NBAP_ID_INFORMATIONEXCHANGEFAILURE							40
#define NBAP_ID_INFORMATIONEXCHANGEINITIATION						41
#define NBAP_ID_INFORMATIONEXCHANGETERMINATION						42
#define NBAP_ID_INFORMATIONREPORTING								43
#define NBAP_ID_CELLSYNCHRONISATIONADJUSTMENT						44
#define NBAP_ID_CELLSYNCHRONISATIONINITIATION						45
#define NBAP_ID_CELLSYNCHRONISATIONRECONFIGURATION					46
#define NBAP_ID_CELLSYNCHRONISATIONREPORTING						47
#define NBAP_ID_CELLSYNCHRONISATIONTERMINATION						48
#define NBAP_ID_CELLSYNCHRONISATIONFAILURE							49
#define NBAP_ID_BEARERREARRANGEMENT									50
#define NBAP_ID_RADIOLINKACTIVATION									51
#define NBAP_ID_RADIOLINKPARAMETERUPDATE							52
#define NBAP_ID_MBMSNOTIFICATIONUPDATE								53

/* Protocol IE:s */
#define NBAP_IE_ID_AICH_INFORMATION													0
#define NBAP_IE_ID_AICH_INFORMATIONITEM_RESOURCESTATUSIND							1
#define NBAP_IE_ID_BCH_INFORMATION													7
#define NBAP_IE_ID_BCH_INFORMATIONITEM_RESOURCESTATUSIND							8
#define NBAP_IE_ID_BCCH_MODIFICATIONTIME											9
#define NBAP_IE_ID_BLOCKINGPRIORITYINDICATOR										10
#define NBAP_IE_ID_CAUSE															13
#define NBAP_IE_ID_CCP_INFORMATIONITEM_AUDITRSP										14
#define NBAP_IE_ID_CCP_INFORMATIONLIST_AUDITRSP										15
#define NBAP_IE_ID_CCP_INFORMATIONITEM_RESOURCESTATUSIND							16
#define NBAP_IE_ID_CELL_INFORMATIONITEM_AUDITRSP									17
#define NBAP_IE_ID_CELL_INFORMATIONITEM_RESOURCESTATUSIND							18
#define NBAP_IE_ID_CELL_INFORMATIONLIST_AUDITRSP									19
#define NBAP_IE_ID_CELLPARAMETERID													23
#define NBAP_IE_ID_CFN																24
#define NBAP_IE_ID_C_ID																25

#define NBAP_IE_ID_COMMONMEASUREMENTACCURACY										39
#define NBAP_IE_ID_COMMONMEASUREMENTOBJECTTYPE_CM_RPRT								31
#define NBAP_IE_ID_COMMONMEASUREMENTOBJECTTYPE_CM_RQST								32
#define NBAP_IE_ID_COMMONMEASUREMENTOBJECTTYPE_CM_RSP								33
#define NBAP_IE_ID_COMMONMEASUREMENTTYPE											34
#define NBAP_IE_ID_COMMONPHYSICALCHANNELID											35
#define NBAP_IE_ID_COMMONPHYSICALCHANNELTYPE_CTCH_SETUPRQSTFDD						36
#define NBAP_IE_ID_COMMONPHYSICALCHANNELTYPE_CTCH_SETUPRQSTTDD						37
#define NBAP_IE_ID_COMMUNICATIONCONTROLPORTID										40
#define NBAP_IE_ID_CONFIGURATIONGENERATIONID										43
#define NBAP_IE_ID_CRNC_COMMUNICATIONCONTEXTID										44
#define NBAP_IE_ID_CRITICALITYDIAGNOSTICS											45
#define NBAP_IE_ID_DCHS_TO_ADD_FDD													48
#define NBAP_IE_ID_DCH_ADDLIST_RL_RECONFPREPTDD										49
#define NBAP_IE_ID_DCHS_TO_ADD_TDD													50
#define NBAP_IE_ID_DCH_DELETELIST_RL_RECONFPREPFDD									52
#define NBAP_IE_ID_DCH_DELETELIST_RL_RECONFPREPTDD									53
#define NBAP_IE_ID_DCH_DELETELIST_RL_RECONFRQSTFDD									54
#define NBAP_IE_ID_DCH_DELETELIST_RL_RECONFRQSTTDD									55
#define NBAP_IE_ID_DCH_FDD_INFORMATION												56

#define NBAP_IE_ID_DCH_TDD_INFORMATION												57
#define NBAP_IE_ID_DCH_INFORMATIONRESPONSE											59
#define NBAP_IE_ID_FDD_DCHS_TO_MODIFY												62
#define NBAP_IE_ID_TDD_DCHS_TO_MODIFY												63
#define NBAP_IE_ID_DCH_MODIFYLIST_RL_RECONFRQSTTDD									65
#define NBAP_IE_ID_DCH_REARRANGELIST_BEARER_REARRANGEIND							135
#define NBAP_IE_ID_DEDICATEDMEASUREMENTOBJECTTYPE_DM_RPRT							67
#define NBAP_IE_ID_DEDICATEDMEASUREMENTOBJECTTYPE_DM_RQST							68
#define NBAP_IE_ID_DEDICATEDMEASUREMENTOBJECTTYPE_DM_RSP							69
#define NBAP_IE_ID_DEDICATEDMEASUREMENTTYPE											70
#define NBAP_IE_ID_DL_CCTRCH_INFORMATIONITEM_RL_SETUPRQSTTDD						72
#define NBAP_IE_ID_DL_CCTRCH_INFORMATIONLIST_RL_ADDITIONRQSTTDD						73
#define NBAP_IE_ID_DL_CCTRCH_INFORMATIONLIST_RL_SETUPRQSTTDD						76
#define NBAP_IE_ID_DL_DPCH_INFORMATIONITEM_RL_ADDITIONRQSTTDD						77
#define NBAP_IE_ID_DL_DPCH_INFORMATIONLIST_RL_SETUPRQSTTDD							79
#define NBAP_IE_ID_DL_DPCH_INFORMATION_RL_RECONFPREPFDD								81
#define NBAP_IE_ID_DL_DPCH_INFORMATION_RL_RECONFRQSTFDD								82
#define NBAP_IE_ID_DL_DPCH_INFORMATION_RL_SETUPRQSTFDD								83
#define NBAP_IE_ID_DL_DPCH_TIMINGADJUSTMENT											21
#define NBAP_IE_ID_DL_REFERENCEPOWERINFORMATIONITEM_DL_PC_RQST						84
#define NBAP_IE_ID_DLREFERENCEPOWER													85
#define NBAP_IE_ID_DLREFERENCEPOWERLIST_DL_PC_RQST									86
#define NBAP_IE_ID_UNUSED_PROTOCOLIE_ID_87											87
#define NBAP_IE_ID_UNUSED_PROTOCOLIE_ID_89											89
#define NBAP_IE_ID_UNUSED_PROTOCOLIE_ID_91											91
#define NBAP_IE_ID_UNUSED_PROTOCOLIE_ID_93											93
#define NBAP_IE_ID_DSCHS_TO_ADD_TDD													96
#define NBAP_IE_ID_DSCH_INFORMATION_DELETELIST_RL_RECONFPREPTDD						98
#define NBAP_IE_ID_DSCH_INFORMATION_MODIFYLIST_RL_RECONFPREPTDD						100
#define NBAP_IE_ID_DSCH_INFORMATIONRESPONSE											105
#define NBAP_IE_ID_UNUSED_PROTOCOLIE_ID_106											106
#define NBAP_IE_ID_DSCH_TDD_INFORMATION												107
#define NBAP_IE_ID_UNUSED_PROTOCOLIE_ID_108											108
#define NBAP_IE_ID_UNUSED_PROTOCOLIE_ID_112											112
#define NBAP_IE_ID_DSCH_REARRANGELIST_BEARER_REARRANGEIND							136
#define NBAP_IE_ID_END_OF_AUDIT_SEQUENCE_INDICATOR 									113
#define NBAP_IE_ID_FACH_INFORMATION													116
#define NBAP_IE_ID_FACH_INFORMATIONITEM_RESOURCESTATUSIND							117
#define NBAP_IE_ID_FACH_PARAMETERSLIST_CTCH_RECONFRQSTTDD							120
#define NBAP_IE_ID_FACH_PARAMETERSLISTIE_CTCH_SETUPRQSTFDD							121
#define NBAP_IE_ID_FACH_PARAMETERSLISTIE_CTCH_SETUPRQSTTDD							122
#define NBAP_IE_ID_INDICATIONTYPE_RESOURCESTATUSIND									123
#define NBAP_IE_ID_LOCAL_CELL_ID													124
#define NBAP_IE_ID_LOCAL_CELL_GROUP_INFORMATIONITEM_AUDITRSP						2
#define NBAP_IE_ID_LOCAL_CELL_GROUP_INFORMATIONITEM_RESOURCESTATUSIND				3
#define NBAP_IE_ID_LOCAL_CELL_GROUP_INFORMATIONITEM2_RESOURCESTATUSIND				4
#define NBAP_IE_ID_LOCAL_CELL_GROUP_INFORMATIONLIST_AUDITRSP						5
#define NBAP_IE_ID_LOCAL_CELL_INFORMATIONITEM_AUDITRSP								125
#define NBAP_IE_ID_LOCAL_CELL_INFORMATIONITEM_RESOURCESTATUSIND						126
#define NBAP_IE_ID_LOCAL_CELL_INFORMATIONITEM2_RESOURCESTATUSIND					127
#define NBAP_IE_ID_LOCAL_CELL_INFORMATIONLIST_AUDITRSP								128
#define NBAP_IE_ID_ADJUSTMENTPERIOD													129
#define NBAP_IE_ID_MAXADJUSTMENTSTEP												130
#define NBAP_IE_ID_MAXIMUMTRANSMISSIONPOWER											131
#define NBAP_IE_ID_MEASUREMENTFILTERCOEFFICIENT										132
#define NBAP_IE_ID_MEASUREMENTID													133
#define NBAP_IE_ID_MESSAGESTRUCTURE													115
#define NBAP_IE_ID_MIB_SB_SIB_INFORMATIONLIST_SYSTEMINFOUPDATERQST					134
#define NBAP_IE_ID_NODEB_COMMUNICATIONCONTEXTID										143
#define NBAP_IE_ID_NEIGHBOURINGCELLMEASUREMENTINFORMATION							455
#define NBAP_IE_ID_P_CCPCH_INFORMATION												144
#define NBAP_IE_ID_P_CCPCH_INFORMATIONITEM_RESOURCESTATUSIND						145
#define NBAP_IE_ID_P_CPICH_INFORMATION												146
#define NBAP_IE_ID_P_CPICH_INFORMATIONITEM_RESOURCESTATUSIND						147
#define NBAP_IE_ID_P_SCH_INFORMATION												148
#define NBAP_IE_ID_PCCPCH_INFORMATION_CELL_RECONFRQSTTDD							150
#define NBAP_IE_ID_PCCPCH_INFORMATION_CELL_SETUPRQSTTDD								151
#define NBAP_IE_ID_PCH_PARAMETERS_CTCH_RECONFRQSTTDD								155
#define NBAP_IE_ID_PCH_PARAMETERSITEM_CTCH_SETUPRQSTFDD								156
#define NBAP_IE_ID_PCH_PARAMETERSITEM_CTCH_SETUPRQSTTDD								157
#define NBAP_IE_ID_PCH_INFORMATION													158
#define NBAP_IE_ID_PDSCH_INFORMATION_ADDLISTIE_PSCH_RECONFRQST						161
#define NBAP_IE_ID_PDSCH_INFORMATION_MODIFYLISTIE_PSCH_RECONFRQST					162
#define NBAP_IE_ID_PDSCHSETS_ADDLIST_PSCH_RECONFRQST								163
#define NBAP_IE_ID_PDSCHSETS_DELETELIST_PSCH_RECONFRQST								164
#define NBAP_IE_ID_PDSCHSETS_MODIFYLIST_PSCH_RECONFRQST								165
#define NBAP_IE_ID_PICH_INFORMATION													166
#define NBAP_IE_ID_PICH_PARAMETERS_CTCH_RECONFRQSTTDD								168
#define NBAP_IE_ID_POWERADJUSTMENTTYPE												169
#define NBAP_IE_ID_PRACH_INFORMATION												170
#define NBAP_IE_ID_PRIMARYCCPCH_INFORMATION_CELL_RECONFRQSTFDD						175
#define NBAP_IE_ID_PRIMARYCCPCH_INFORMATION_CELL_SETUPRQSTFDD						176
#define NBAP_IE_ID_PRIMARYCPICH_INFORMATION_CELL_RECONFRQSTFDD						177
#define NBAP_IE_ID_PRIMARYCPICH_INFORMATION_CELL_SETUPRQSTFDD						178
#define NBAP_IE_ID_PRIMARYSCH_INFORMATION_CELL_RECONFRQSTFDD						179
#define NBAP_IE_ID_PRIMARYSCH_INFORMATION_CELL_SETUPRQSTFDD							180
#define NBAP_IE_ID_PRIMARYSCRAMBLINGCODE											181
#define NBAP_IE_ID_SCH_INFORMATION_CELL_RECONFRQSTTDD								183
#define NBAP_IE_ID_SCH_INFORMATION_CELL_SETUPRQSTTDD								184
#define NBAP_IE_ID_PUSCH_INFORMATION_ADDLISTIE_PSCH_RECONFRQST						185
#define NBAP_IE_ID_PUSCH_INFORMATION_MODIFYLISTIE_PSCH_RECONFRQST					186
#define NBAP_IE_ID_PUSCHSETS_ADDLIST_PSCH_RECONFRQST								187
#define NBAP_IE_ID_PUSCHSETS_DELETELIST_PSCH_RECONFRQST								188
#define NBAP_IE_ID_PUSCHSETS_MODIFYLIST_PSCH_RECONFRQST								189
#define NBAP_IE_ID_RACH_INFORMATION													190
#define NBAP_IE_ID_RACH_PARAMETERSITEM_CTCH_SETUPRQSTFDD							196
#define NBAP_IE_ID_RACH_PARAMETERITEM_CTCH_SETUPRQSTTDD								197
#define NBAP_IE_ID_REPORTCHARACTERISTICS											198
#define NBAP_IE_ID_REPORTING_OBJECT_RL_FAILUREIND									199
#define NBAP_IE_ID_REPORTING_OBJECT_RL_RESTOREIND									200
#define NBAP_IE_ID_RL_INFORMATIONITEM_DM_RPRT										202
#define NBAP_IE_ID_RL_INFORMATIONITEM_DM_RQST										203
#define NBAP_IE_ID_RL_INFORMATIONITEM_DM_RSP										204
#define NBAP_IE_ID_RL_INFORMATIONITEM_RL_ADDITIONRQSTFDD							205
#define NBAP_IE_ID_RL_INFORMATIONITEM_RL_DELETIONRQST								206
#define NBAP_IE_ID_RL_INFORMATIONITEM_RL_FAILUREIND									207
#define NBAP_IE_ID_RL_INFORMATIONITEM_RL_PREEMPTREQUIREDIND							286
#define NBAP_IE_ID_RL_INFORMATIONITEM_RL_RECONFPREPFDD								208
#define NBAP_IE_ID_RL_INFORMATIONITEM_RL_RECONFRQSTFDD								209
#define NBAP_IE_ID_RL_INFORMATIONITEM_RL_RESTOREIND									210
#define NBAP_IE_ID_RL_INFORMATIONITEM_RL_SETUPRQSTFDD								211
#define NBAP_IE_ID_RL_INFORMATIONLIST_RL_ADDITIONRQSTFDD							212
#define NBAP_IE_ID_RL_INFORMATIONLIST_RL_DELETIONRQST								213
#define NBAP_IE_ID_RL_INFORMATIONLIST_RL_PREEMPTREQUIREDIND							237
#define NBAP_IE_ID_RL_INFORMATIONLIST_RL_RECONFPREPFDD								214
#define NBAP_IE_ID_RL_INFORMATIONLIST_RL_RECONFRQSTFDD								215
#define NBAP_IE_ID_RL_INFORMATIONLIST_RL_SETUPRQSTFDD								216
#define NBAP_IE_ID_RL_INFORMATIONRESPONSEITEM_RL_ADDITIONRSPFDD						217
#define NBAP_IE_ID_RL_INFORMATIONRESPONSEITEM_RL_RECONFREADY						218
#define NBAP_IE_ID_RL_INFORMATIONRESPONSEITEM_RL_RECONFRSP							219
#define NBAP_IE_ID_RL_INFORMATIONRESPONSEITEM_RL_SETUPRSPFDD						220
#define NBAP_IE_ID_RL_INFORMATIONRESPONSELIST_RL_ADDITIONRSPFDD						221
#define NBAP_IE_ID_RL_INFORMATIONRESPONSELIST_RL_RECONFREADY						222
#define NBAP_IE_ID_RL_INFORMATIONRESPONSELIST_RL_RECONFRSP							223
#define NBAP_IE_ID_RL_INFORMATIONRESPONSELIST_RL_SETUPRSPFDD						224
#define NBAP_IE_ID_RL_INFORMATIONRESPONSE_RL_ADDITIONRSPTDD							225
#define NBAP_IE_ID_RL_INFORMATIONRESPONSE_RL_SETUPRSPTDD							226
#define NBAP_IE_ID_RL_INFORMATION_RL_ADDITIONRQSTTDD								227
#define NBAP_IE_ID_RL_INFORMATION_RL_RECONFRQSTTDD									228
#define NBAP_IE_ID_RL_INFORMATION_RL_RECONFPREPTDD									229
#define NBAP_IE_ID_RL_INFORMATION_RL_SETUPRQSTTDD									230
#define NBAP_IE_ID_RL_RECONFIGURATIONFAILUREITEM_RL_RECONFFAILURE					236
#define NBAP_IE_ID_RL_SET_INFORMATIONITEM_DM_RPRT									238
#define NBAP_IE_ID_RL_SET_INFORMATIONITEM_DM_RSP									240
#define NBAP_IE_ID_RL_SET_INFORMATIONITEM_RL_FAILUREIND								241
#define NBAP_IE_ID_RL_SET_INFORMATIONITEM_RL_RESTOREIND								242
#define NBAP_IE_ID_S_CCPCH_INFORMATION												247
#define NBAP_IE_ID_S_CPICH_INFORMATION												249
#define NBAP_IE_ID_SCH_INFORMATION													251
#define NBAP_IE_ID_S_SCH_INFORMATION												253
#define NBAP_IE_ID_SECONDARY_CCPCHLISTIE_CTCH_RECONFRQSTTDD							257
#define NBAP_IE_ID_SECONDARY_CCPCH_PARAMETERLISTIE_CTCH_SETUPRQSTTDD				258
#define NBAP_IE_ID_SECONDARY_CCPCH_PARAMETERS_CTCH_RECONFRQSTTDD					259
#define NBAP_IE_ID_SECONDARYCPICH_INFORMATIONITEM_CELL_RECONFRQSTFDD				260
#define NBAP_IE_ID_SECONDARYCPICH_INFORMATIONITEM_CELL_SETUPRQSTFDD					261
#define NBAP_IE_ID_SECONDARYCPICH_INFORMATIONLIST_CELL_RECONFRQSTFDD				262
#define NBAP_IE_ID_SECONDARYCPICH_INFORMATIONLIST_CELL_SETUPRQSTFDD					263
#define NBAP_IE_ID_SECONDARYSCH_INFORMATION_CELL_RECONFRQSTFDD						264
#define NBAP_IE_ID_SECONDARYSCH_INFORMATION_CELL_SETUPRQSTFDD						265
#define NBAP_IE_ID_SEGMENTINFORMATIONLISTIE_SYSTEMINFOUPDATE						266
#define NBAP_IE_ID_SFN																268
#define NBAP_IE_ID_SIGNALLINGBEARERREQUESTINDICATOR									138
#define NBAP_IE_ID_SHUTDOWNTIMER													269
#define NBAP_IE_ID_START_OF_AUDIT_SEQUENCE_INDICATOR								114
#define NBAP_IE_ID_SUCCESSFUL_RL_INFORMATIONRESPITEM_RL_ADDITIONFAILUREFDD			270
#define NBAP_IE_ID_SUCCESSFUL_RL_INFORMATIONRESPITEM_RL_SETUPFAILUREFDD				271
#define NBAP_IE_ID_SYNCCASE															274
#define NBAP_IE_ID_SYNCCASEINDICATORITEM_CELL_SETUPRQSTTDD_PSCH						275
#define NBAP_IE_ID_T_CELL															276
#define NBAP_IE_ID_TARGETCOMMUNICATIONCONTROLPORTID									139
#define NBAP_IE_ID_TIMESLOTCONFIGURATIONLIST_CELL_RECONFRQSTTDD						277
#define NBAP_IE_ID_TIMESLOTCONFIGURATIONLIST_CELL_SETUPRQSTTDD						278
#define NBAP_IE_ID_TRANSMISSIONDIVERSITYAPPLIED										279
#define NBAP_IE_ID_TYPEOFERROR														508
#define NBAP_IE_ID_UARFCNFORNT														280
#define NBAP_IE_ID_UARFCNFORND														281
#define NBAP_IE_ID_UARFCNFORNU														282
#define NBAP_IE_ID_UL_CCTRCH_INFORMATIONITEM_RL_SETUPRQSTTDD						284
#define NBAP_IE_ID_UL_CCTRCH_INFORMATIONLIST_RL_ADDITIONRQSTTDD						285
#define NBAP_IE_ID_UL_CCTRCH_INFORMATIONLIST_RL_SETUPRQSTTDD						288
#define NBAP_IE_ID_UL_DPCH_INFORMATIONITEM_RL_ADDITIONRQSTTDD						289
#define NBAP_IE_ID_UL_DPCH_INFORMATIONLIST_RL_SETUPRQSTTDD							291
#define NBAP_IE_ID_UL_DPCH_INFORMATION_RL_RECONFPREPFDD								293
#define NBAP_IE_ID_UL_DPCH_INFORMATION_RL_RECONFRQSTFDD								294
#define NBAP_IE_ID_UL_DPCH_INFORMATION_RL_SETUPRQSTFDD								295
#define NBAP_IE_ID_UNSUCCESSFUL_RL_INFORMATIONRESPITEM_RL_ADDITIONFAILUREFDD		296
#define NBAP_IE_ID_UNSUCCESSFUL_RL_INFORMATIONRESPITEM_RL_SETUPFAILUREFDD			297
#define NBAP_IE_ID_UNSUCCESSFUL_RL_INFORMATIONRESP_RL_ADDITIONFAILURETDD			300
#define NBAP_IE_ID_UNSUCCESSFUL_RL_INFORMATIONRESP_RL_SETUPFAILURETDD				301
#define NBAP_IE_ID_USCH_INFORMATION_ADD												302
#define NBAP_IE_ID_USCH_INFORMATION_DELETELIST_RL_RECONFPREPTDD						304
#define NBAP_IE_ID_USCH_INFORMATION_MODIFYLIST_RL_RECONFPREPTDD						306
#define NBAP_IE_ID_USCH_INFORMATIONRESPONSE											309
#define NBAP_IE_ID_USCH_INFORMATION													310
#define NBAP_IE_ID_USCH_REARRANGELIST_BEARER_REARRANGEIND							141
#define NBAP_IE_ID_ACTIVE_PATTERN_SEQUENCE_INFORMATION 								315
#define NBAP_IE_ID_AICH_PARAMETERSLISTIE_CTCH_RECONFRQSTFDD							316
#define NBAP_IE_ID_ADJUSTMENTRATIO													317
#define NBAP_IE_ID_NOT_USED_320														320
#define NBAP_IE_ID_NOT_USED_322														322
#define NBAP_IE_ID_FACH_PARAMETERSLISTIE_CTCH_RECONFRQSTFDD							323
#define NBAP_IE_ID_CAUSELEVEL_PSCH_RECONFFAILURE									324
#define NBAP_IE_ID_CAUSELEVEL_RL_ADDITIONFAILUREFDD									325
#define NBAP_IE_ID_CAUSELEVEL_RL_ADDITIONFAILURETDD									326
#define NBAP_IE_ID_CAUSELEVEL_RL_RECONFFAILURE										327
#define NBAP_IE_ID_CAUSELEVEL_RL_SETUPFAILUREFDD									328
#define NBAP_IE_ID_CAUSELEVEL_RL_SETUPFAILURETDD									329
#define NBAP_IE_ID_NOT_USED_330														330
#define NBAP_IE_ID_NOT_USED_332														332
#define NBAP_IE_ID_CLOSED_LOOP_TIMING_ADJUSTMENT_MODE								333
#define NBAP_IE_ID_COMMONPHYSICALCHANNELTYPE_CTCH_RECONFRQSTFDD						334
#define NBAP_IE_ID_COMPRESSED_MODE_DEACTIVATION_FLAG								335
#define NBAP_IE_ID_NOT_USED_336														336
#define NBAP_IE_ID_NOT_USED_342														342
#define NBAP_IE_ID_NOT_USED_343														343
#define NBAP_IE_ID_DL_CCTRCH_INFORMATIONADDLIST_RL_RECONFPREPTDD					346
#define NBAP_IE_ID_DL_CCTRCH_INFORMATIONDELETEITEM_RL_RECONFRQSTTDD					347
#define NBAP_IE_ID_DL_CCTRCH_INFORMATIONDELETELIST_RL_RECONFPREPTDD					348
#define NBAP_IE_ID_DL_CCTRCH_INFORMATIONDELETELIST_RL_RECONFRQSTTDD					349
#define NBAP_IE_ID_DL_CCTRCH_INFORMATIONMODIFYITEM_RL_RECONFRQSTTDD					350
#define NBAP_IE_ID_DL_CCTRCH_INFORMATIONMODIFYLIST_RL_RECONFPREPTDD					351
#define NBAP_IE_ID_DL_CCTRCH_INFORMATIONMODIFYLIST_RL_RECONFRQSTTDD					352
#define NBAP_IE_ID_DL_DPCH_INFORMATIONADDLISTIE_RL_RECONFPREPTDD					353
#define NBAP_IE_ID_DL_DPCH_INFORMATIONMODIFY_ADDLISTIE_RL_RECONFPREPTDD				355
#define NBAP_IE_ID_DL_DPCH_INFORMATIONMODIFY_DELETELISTIE_RL_RECONFPREPTDD			356
#define NBAP_IE_ID_DL_DPCH_INFORMATIONMODIFY_MODIFYLISTIE_RL_RECONFPREPTDD			357
#define NBAP_IE_ID_DL_TPC_PATTERN01COUNT 											358
#define NBAP_IE_ID_DPC_MODE															450
#define NBAP_IE_ID_DPCHCONSTANT														359
#define NBAP_IE_ID_UNUSED_PROTOCOLIE_ID_94											94
#define NBAP_IE_ID_UNUSED_PROTOCOLIE_ID_110											110
#define NBAP_IE_ID_UNUSED_PROTOCOLIE_ID_111											111
#define NBAP_IE_ID_FACH_PARAMETERSLIST_CTCH_SETUPRSP								362
#define NBAP_IE_ID_LIMITED_POWER_INCREASE_INFORMATION_CELL_SETUPRQSTFDD				369
#define NBAP_IE_ID_PCH_PARAMETERS_CTCH_SETUPRSP										374
#define NBAP_IE_ID_PCH_PARAMETERSITEM_CTCH_RECONFRQSTFDD							375
#define NBAP_IE_ID_NOT_USED_376														376
#define NBAP_IE_ID_PICH_PARAMETERSITEM_CTCH_RECONFRQSTFDD							380
#define NBAP_IE_ID_PRACHCONSTANT													381
#define NBAP_IE_ID_PRACH_PARAMETERSLISTIE_CTCH_RECONFRQSTFDD						383
#define NBAP_IE_ID_PUSCHCONSTANT													384
#define NBAP_IE_ID_RACH_PARAMETERS_CTCH_SETUPRSP									385
#define NBAP_IE_ID_UNUSED_PROTOCOLIE_ID_443											443
#define NBAP_IE_ID_SYNCHRONISATION_CONFIGURATION_CELL_RECONFRQST					393
#define NBAP_IE_ID_SYNCHRONISATION_CONFIGURATION_CELL_SETUPRQST						394
#define NBAP_IE_ID_TRANSMISSION_GAP_PATTERN_SEQUENCE_INFORMATION					395
#define NBAP_IE_ID_UL_CCTRCH_INFORMATIONADDLIST_RL_RECONFPREPTDD					396
#define NBAP_IE_ID_UL_CCTRCH_INFORMATIONDELETEITEM_RL_RECONFRQSTTDD					397
#define NBAP_IE_ID_UL_CCTRCH_INFORMATIONDELETELIST_RL_RECONFPREPTDD					398
#define NBAP_IE_ID_UL_CCTRCH_INFORMATIONDELETELIST_RL_RECONFRQSTTDD					399
#define NBAP_IE_ID_UL_CCTRCH_INFORMATIONMODIFYITEM_RL_RECONFRQSTTDD					400
#define NBAP_IE_ID_UL_CCTRCH_INFORMATIONMODIFYLIST_RL_RECONFPREPTDD					401
#define NBAP_IE_ID_UL_CCTRCH_INFORMATIONMODIFYLIST_RL_RECONFRQSTTDD					402
#define NBAP_IE_ID_UL_DPCH_INFORMATIONADDLISTIE_RL_RECONFPREPTDD					403
#define NBAP_IE_ID_UL_DPCH_INFORMATIONMODIFY_ADDLISTIE_RL_RECONFPREPTDD				405
#define NBAP_IE_ID_UL_DPCH_INFORMATIONMODIFY_DELETELISTIE_RL_RECONFPREPTDD			406
#define NBAP_IE_ID_UL_DPCH_INFORMATIONMODIFY_MODIFYLISTIE_RL_RECONFPREPTDD			407
#define NBAP_IE_ID_UNSUCCESSFUL_PDSCHSETITEM_PSCH_RECONFFAILURETDD					408
#define NBAP_IE_ID_UNSUCCESSFUL_PUSCHSETITEM_PSCH_RECONFFAILURETDD					409
#define NBAP_IE_ID_COMMUNICATIONCONTEXTINFOITEM_RESET								412
#define NBAP_IE_ID_COMMUNICATIONCONTROLPORTINFOITEM_RESET							414
#define NBAP_IE_ID_RESETINDICATOR													416
#define NBAP_IE_ID_UNUSED_PROTOCOLIE_ID_417											417
#define NBAP_IE_ID_UNUSED_PROTOCOLIE_ID_418											418
#define NBAP_IE_ID_UNUSED_PROTOCOLIE_ID_419											419
#define NBAP_IE_ID_UNUSED_PROTOCOLIE_ID_142											142
#define NBAP_IE_ID_TIMINGADVANCEAPPLIED												287
#define NBAP_IE_ID_CFNREPORTINGINDICATOR											6
#define NBAP_IE_ID_SFNREPORTINGINDICATOR											11
#define NBAP_IE_ID_INNERLOOPDLPCSTATUS												12
#define NBAP_IE_ID_TIMESLOTISCPINFO													283
#define NBAP_IE_ID_PICH_PARAMETERSITEM_CTCH_SETUPRQSTTDD							167
#define NBAP_IE_ID_PRACH_PARAMETERSITEM_CTCH_SETUPRQSTTDD							20
#define NBAP_IE_ID_CCTRCH_INFORMATIONITEM_RL_FAILUREIND								46
#define NBAP_IE_ID_CCTRCH_INFORMATIONITEM_RL_RESTOREIND								47
#define NBAP_IE_ID_CAUSELEVEL_SYNCADJUSTMNTFAILURETDD								420
#define NBAP_IE_ID_CELLADJUSTMENTINFO_SYNCADJUSTMNTRQSTTDD							421
#define NBAP_IE_ID_CELLADJUSTMENTINFOITEM_SYNCADJUSTMENTRQSTTDD						494
#define NBAP_IE_ID_CELLSYNCBURSTINFOLIST_CELLSYNCRECONFRQSTTDD						482
#define NBAP_IE_ID_CELLSYNCBURSTTRANSINIT_CELLSYNCINITIATIONRQSTTDD					422
#define NBAP_IE_ID_CELLSYNCBURSTMEASUREINIT_CELLSYNCINITIATIONRQSTTDD				423
#define NBAP_IE_ID_CELLSYNCBURSTTRANSRECONFIGURATION_CELLSYNCRECONFRQSTTDD			424
#define NBAP_IE_ID_CELLSYNCBURSTMEASRECONFIGURATION_CELLSYNCRECONFRQSTTDD			425
#define NBAP_IE_ID_CELLSYNCBURSTTRANSINFOLIST_CELLSYNCRECONFRQSTTDD					426
#define NBAP_IE_ID_CELLSYNCBURSTMEASINFOLIST_CELLSYNCRECONFRQSTTDD					427
#define NBAP_IE_ID_CELLSYNCBURSTTRANSRECONFINFO_CELLSYNCRECONFRQSTTDD				428
#define NBAP_IE_ID_CELLSYNCINFO_CELLSYNCREPRTTDD									429
#define NBAP_IE_ID_CSBTRANSMISSIONID												430
#define NBAP_IE_ID_CSBMEASUREMENTID													431
#define NBAP_IE_ID_INTSTDPHCELLSYNCINFOITEM_CELLSYNCREPRTTDD						432
#define NBAP_IE_ID_NCYCLESPERSFNPERIOD 												433
#define NBAP_IE_ID_NREPETITIONSPERCYCLEPERIOD										434
#define NBAP_IE_ID_SYNCFRAMENUMBER													437
#define NBAP_IE_ID_SYNCHRONISATIONREPORTTYPE										438
#define NBAP_IE_ID_SYNCHRONISATIONREPORTCHARACTERISTICS								439
#define NBAP_IE_ID_UNSUCCESSFUL_CELL_INFORMATIONRESPITEM_SYNCADJUSTMNTFAILURETDD	440
#define NBAP_IE_ID_LATEENTRANCECELLSYNCINFOITEM_CELLSYNCREPRTTDD					119
#define NBAP_IE_ID_REFERENCECLOCKAVAILABILITY										435
#define NBAP_IE_ID_REFERENCESFNOFFSET												436
#define NBAP_IE_ID_INFORMATIONEXCHANGEID											444
#define NBAP_IE_ID_INFORMATIONEXCHANGEOBJECTTYPE_INFEX_RQST							445
#define NBAP_IE_ID_INFORMATIONTYPE													446
#define NBAP_IE_ID_INFORMATIONREPORTCHARACTERISTICS									447
#define NBAP_IE_ID_INFORMATIONEXCHANGEOBJECTTYPE_INFEX_RSP							448
#define NBAP_IE_ID_INFORMATIONEXCHANGEOBJECTTYPE_INFEX_RPRT							449
#define NBAP_IE_ID_IPDLPARAMETER_INFORMATION_CELL_RECONFRQSTFDD						451
#define NBAP_IE_ID_IPDLPARAMETER_INFORMATION_CELL_SETUPRQSTFDD						452
#define NBAP_IE_ID_IPDLPARAMETER_INFORMATION_CELL_RECONFRQSTTDD						453
#define NBAP_IE_ID_IPDLPARAMETER_INFORMATION_CELL_SETUPRQSTTDD						454
#define NBAP_IE_ID_DL_DPCH_LCR_INFORMATION_RL_SETUPRQSTTDD							74
#define NBAP_IE_ID_DWPCH_LCR_INFORMATION											78
#define NBAP_IE_ID_DWPCH_LCR_INFORMATIONLIST_AUDITRSP                   			90
#define NBAP_IE_ID_DWPCH_LCR_INFORMATION_CELL_SETUPRQSTTDD                   		97
#define NBAP_IE_ID_DWPCH_LCR_INFORMATION_CELL_RECONFRQSTTDD							99
#define NBAP_IE_ID_DWPCH_LCR_INFORMATION_RESOURCESTATUSIND							101
#define NBAP_IE_ID_MAXFACH_POWER_LCR_CTCH_SETUPRQSTTDD                   			154
#define NBAP_IE_ID_MAXFACH_POWER_LCR_CTCH_RECONFRQSTTDD                   			174
#define NBAP_IE_ID_FPACH_LCR_INFORMATION											290
#define NBAP_IE_ID_FPACH_LCR_INFORMATION_AUDITRSP                   				292
#define NBAP_IE_ID_FPACH_LCR_INFORMATIONLIST_AUDITRSP                   			22
#define NBAP_IE_ID_FPACH_LCR_INFORMATIONLIST_RESOURCESTATUSIND                   	311
#define NBAP_IE_ID_FPACH_LCR_PARAMETERS_CTCH_SETUPRQSTTDD                   		312
#define NBAP_IE_ID_FPACH_LCR_PARAMETERS_CTCH_RECONFRQSTTDD                   		314
#define NBAP_IE_ID_PCCPCH_LCR_INFORMATION_CELL_SETUPRQSTTDD                   		456
#define NBAP_IE_ID_PCH_POWER_LCR_CTCH_SETUPRQSTTDD									457
#define NBAP_IE_ID_PCH_POWER_LCR_CTCH_RECONFRQSTTDD									458
#define NBAP_IE_ID_PICH_LCR_PARAMETERS_CTCH_SETUPRQSTTDD                   			459
#define NBAP_IE_ID_PRACH_LCR_PARAMETERSLIST_CTCH_SETUPRQSTTDD                  		461
#define NBAP_IE_ID_RL_INFORMATIONRESPONSE_LCR_RL_SETUPRSPTDD	                   	463
#define NBAP_IE_ID_SECONDARY_CCPCH_LCR_PARAMETERLIST_CTCH_SETUPRQSTTDD              465
#define NBAP_IE_ID_TIMESLOT															495
#define NBAP_IE_ID_TIMESLOTCONFIGURATIONLIST_LCR_CELL_RECONFRQSTTDD                 466
#define NBAP_IE_ID_TIMESLOTCONFIGURATIONLIST_LCR_CELL_SETUPRQSTTDD                  467
#define NBAP_IE_ID_TIMESLOTISCP_LCR_INFOLIST_RL_SETUPRQSTTDD                   		468
#define NBAP_IE_ID_TIMESLOTLCR_CM_RQST                   							469
#define NBAP_IE_ID_UL_DPCH_LCR_INFORMATION_RL_SETUPRQSTTDD                   		470
#define NBAP_IE_ID_DL_DPCH_INFORMATIONITEM_LCR_RL_ADDITIONRQSTTDD					472
#define NBAP_IE_ID_UL_DPCH_INFORMATIONITEM_LCR_RL_ADDITIONRQSTTDD					473
#define NBAP_IE_ID_TIMESLOTISCP_INFORMATIONLIST_LCR_RL_ADDITIONRQSTTDD				474
#define NBAP_IE_ID_DL_DPCH_LCR_INFORMATIONADDLIST_RL_RECONFPREPTDD					475
#define NBAP_IE_ID_DL_DPCH_LCR_INFORMATIONMODIFY_ADDLIST_RL_RECONFPREPTDD			477
#define NBAP_IE_ID_DL_TIMESLOT_LCR_INFORMATIONMODIFY_MODIFYLIST_RL_RECONFPREPTDD	479
#define NBAP_IE_ID_TIMESLOTISCPINFOLIST_LCR_DL_PC_RQSTTDD							480
#define NBAP_IE_ID_UL_DPCH_LCR_INFORMATIONADDLISTIE_RL_RECONFPREPTDD				481
#define NBAP_IE_ID_UL_DPCH_LCR_INFORMATIONMODIFY_ADDLIST							483
#define NBAP_IE_ID_UL_TIMESLOTLCR_INFORMATION_RL_RECONFPREPTDD						485
#define NBAP_IE_ID_UL_SIRTARGET														510
#define NBAP_IE_ID_PDSCH_ADDINFORMATION_LCR_PSCH_RECONFRQST							486
#define NBAP_IE_ID_PDSCH_ADDINFORMATION_LCR_ADDLISTIE_PSCH_RECONFRQST				487
#define NBAP_IE_ID_UNUSED_PROTOCOLIE_ID_26											 26
#define NBAP_IE_ID_UNUSED_PROTOCOLIE_ID_27											 27
#define NBAP_IE_ID_PDSCH_MODIFYINFORMATION_LCR_PSCH_RECONFRQST						488
#define NBAP_IE_ID_PDSCH_MODIFYINFORMATION_LCR_MODIFYLISTIE_PSCH_RECONFRQST			489
#define NBAP_IE_ID_PUSCH_ADDINFORMATION_LCR_PSCH_RECONFRQST							490
#define NBAP_IE_ID_PUSCH_ADDINFORMATION_LCR_ADDLISTIE_PSCH_RECONFRQST				491
#define NBAP_IE_ID_PUSCH_MODIFYINFORMATION_LCR_PSCH_RECONFRQST						492
#define NBAP_IE_ID_PUSCH_MODIFYINFORMATION_LCR_MODIFYLISTIE_PSCH_RECONFRQST			493
#define NBAP_IE_ID_TIMESLOTINFO_CELLSYNCINITIATIONRQSTTDD							496
#define NBAP_IE_ID_SYNCREPORTTYPE_CELLSYNCREPRTTDD									497
#define NBAP_IE_ID_POWER_LOCAL_CELL_GROUP_INFORMATIONITEM_AUDITRSP					498
#define NBAP_IE_ID_POWER_LOCAL_CELL_GROUP_INFORMATIONITEM_RESOURCESTATUSIND			499
#define NBAP_IE_ID_POWER_LOCAL_CELL_GROUP_INFORMATIONITEM2_RESOURCESTATUSIND		500
#define NBAP_IE_ID_POWER_LOCAL_CELL_GROUP_INFORMATIONLIST_AUDITRSP					501
#define NBAP_IE_ID_POWER_LOCAL_CELL_GROUP_INFORMATIONLIST_RESOURCESTATUSIND			502
#define NBAP_IE_ID_POWER_LOCAL_CELL_GROUP_INFORMATIONLIST2_RESOURCESTATUSIND		503
#define NBAP_IE_ID_POWER_LOCAL_CELL_GROUP_ID										504
#define NBAP_IE_ID_PUSCH_INFO_DM_RQST												505
#define NBAP_IE_ID_PUSCH_INFO_DM_RSP												506
#define NBAP_IE_ID_PUSCH_INFO_DM_RPRT												507
#define NBAP_IE_ID_INITDL_POWER														509
#define NBAP_IE_ID_CELLSYNCBURSTREPETITIONPERIOD									511
#define NBAP_IE_ID_REPORTCHARACTERISTICSTYPE_ONMODIFICATION							512
#define NBAP_IE_ID_SFNSFNMEASUREMENTVALUEINFORMATION								513
#define NBAP_IE_ID_SFNSFNMEASUREMENTTHRESHOLDINFORMATION							514
#define NBAP_IE_ID_TUTRANGPSMEASUREMENTVALUEINFORMATION								515
#define NBAP_IE_ID_TUTRANGPSMEASUREMENTTHRESHOLDINFORMATION							516
#define NBAP_IE_ID_RX_TIMING_DEVIATION_VALUE_LCR									520
#define NBAP_IE_ID_RL_INFORMATIONRESPONSE_LCR_RL_ADDITIONRSPTDD						51
#define NBAP_IE_ID_DL_POWERBALANCING_INFORMATION									28
#define NBAP_IE_ID_DL_POWERBALANCING_ACTIVATIONINDICATOR							29
#define NBAP_IE_ID_DL_POWERBALANCING_UPDATEDINDICATOR								30
#define NBAP_IE_ID_CCTRCH_INITIAL_DL_POWER_RL_SETUPRQSTTDD							517
#define NBAP_IE_ID_CCTRCH_INITIAL_DL_POWER_RL_ADDITIONRQSTTDD						518
#define NBAP_IE_ID_CCTRCH_INITIAL_DL_POWER_RL_RECONFPREPTDD							519
#define NBAP_IE_ID_IPDLPARAMETER_INFORMATION_LCR_CELL_SETUPRQSTTDD					41
#define NBAP_IE_ID_IPDLPARAMETER_INFORMATION_LCR_CELL_RECONFRQSTTDD 				42
#define NBAP_IE_ID_HS_PDSCH_HS_SCCH_E_AGCH_E_RGCH_E_HICH_MAXPOWER_PSCH_RECONFRQST	522
#define NBAP_IE_ID_HS_PDSCH_HS_SCCH_SCRAMBLINGCODE_PSCH_RECONFRQST					523
#define NBAP_IE_ID_HS_PDSCH_FDD_CODE_INFORMATION_PSCH_RECONFRQST					524
#define NBAP_IE_ID_HS_SCCH_FDD_CODE_INFORMATION_PSCH_RECONFRQST						525
#define NBAP_IE_ID_HS_PDSCH_TDD_INFORMATION_PSCH_RECONFRQST							526
#define NBAP_IE_ID_ADD_TO_HS_SCCH_RESOURCE_POOL_PSCH_RECONFRQST						527
#define NBAP_IE_ID_MODIFY_HS_SCCH_RESOURCE_POOL_PSCH_RECONFRQST						528
#define NBAP_IE_ID_DELETE_FROM_HS_SCCH_RESOURCE_POOL_PSCH_RECONFRQST				529
#define NBAP_IE_ID_BINDINGID														102
#define NBAP_IE_ID_RL_SPECIFIC_DCH_INFO												103
#define NBAP_IE_ID_TRANSPORTLAYERADDRESS 											104
#define NBAP_IE_ID_DELAYEDACTIVATION 												231
#define NBAP_IE_ID_DELAYEDACTIVATIONLIST_RL_ACTIVATIONCMDFDD						232
#define NBAP_IE_ID_DELAYEDACTIVATIONINFORMATION_RL_ACTIVATIONCMDFDD					233
#define NBAP_IE_ID_DELAYEDACTIVATIONLIST_RL_ACTIVATIONCMDTDD						234
#define NBAP_IE_ID_DELAYEDACTIVATIONINFORMATION_RL_ACTIVATIONCMDTDD					235
#define NBAP_IE_ID_NEIGHBOURINGTDDCELLMEASUREMENTINFORMATIONLCR						58
#define NBAP_IE_ID_SYNCDLCODEID_TRANSINITLCR_CELLSYNCINITIATIONRQSTTDD				543
#define NBAP_IE_ID_SYNCDLCODEID_MEASUREINITLCR_CELLSYNCINITIATIONRQSTTDD			544
#define NBAP_IE_ID_SYNCDLCODEIDTRANSRECONFINFOLCR_CELLSYNCRECONFRQSTTDD				545
#define NBAP_IE_ID_SYNCDLCODEIDMEASRECONFIGURATIONLCR_CELLSYNCRECONFRQSTTDD			546
#define NBAP_IE_ID_SYNCDLCODEIDMEASINFOLIST_CELLSYNCRECONFRQSTTDD					547
#define NBAP_IE_ID_SYNCDLCODEIDSMEASINFOLIST_CELLSYNCREPRTTDD						548
#define NBAP_IE_ID_SYNCDLCODEIDTHREINFOLCR											549
#define NBAP_IE_ID_NSUBCYCLESPERCYCLEPERIOD_CELLSYNCRECONFRQSTTDD					550
#define NBAP_IE_ID_DWPCH_POWER														551
#define NBAP_IE_ID_ACCUMULATEDCLOCKUPDATE_CELLSYNCREPRTTDD							552
#define NBAP_IE_ID_ANGLE_OF_ARRIVAL_VALUE_LCR										521
#define NBAP_IE_ID_HSDSCH_FDD_INFORMATION											530
#define NBAP_IE_ID_HSDSCH_FDD_INFORMATION_RESPONSE									531
#define NBAP_IE_ID_HSDSCH_INFORMATION_TO_MODIFY										534
#define NBAP_IE_ID_HSDSCH_RNTI														535
#define NBAP_IE_ID_HSDSCH_TDD_INFORMATION											536
#define NBAP_IE_ID_HSDSCH_TDD_INFORMATION_RESPONSE									537
#define NBAP_IE_ID_HSPDSCH_RL_ID													541
#define NBAP_IE_ID_PRIMCCPCH_RSCP_DL_PC_RQSTTDD										542
#define NBAP_IE_ID_UNUSED_PROTOCOLIE_ID_64											64
#define NBAP_IE_ID_PDSCH_RL_ID														66
#define NBAP_IE_ID_HSDSCH_REARRANGELIST_BEARER_REARRANGEIND							553
#define NBAP_IE_ID_UL_SYNCHRONISATION_PARAMETERS_LCR								554
#define NBAP_IE_ID_HSDSCH_FDD_UPDATE_INFORMATION									555
#define NBAP_IE_ID_HSDSCH_TDD_UPDATE_INFORMATION									556
#define NBAP_IE_ID_DL_DPCH_TIMESLOTFORMAT_LCR_MODIFYITEM_RL_RECONFPREPTDD			558
#define NBAP_IE_ID_UL_DPCH_TIMESLOTFORMAT_LCR_MODIFYITEM_RL_RECONFPREPTDD			559
#define NBAP_IE_ID_TDD_TPC_UPLINKSTEPSIZE_LCR_RL_SETUPRQSTTDD						560
#define NBAP_IE_ID_TDD_TPC_UPLINKSTEPSIZE_LCR_RL_ADDITIONRQSTTDD					561
#define NBAP_IE_ID_TDD_TPC_DOWNLINKSTEPSIZE_RL_ADDITIONRQSTTDD						562
#define NBAP_IE_ID_TDD_TPC_UPLINKSTEPSIZE_INFORMATIONADD_LCR_RL_RECONFPREPTDD		563
#define NBAP_IE_ID_TDD_TPC_UPLINKSTEPSIZE_INFORMATIONMODIFY_LCR_RL_RECONFPREPTDD	564
#define NBAP_IE_ID_TDD_TPC_DOWNLINKSTEPSIZE_INFORMATIONMODIFY_RL_RECONFPREPTDD		565
#define NBAP_IE_ID_TDD_TPC_DOWNLINKSTEPSIZE_INFORMATIONADD_RL_RECONFPREPTDD			566
#define NBAP_IE_ID_CCTRCH_MAXIMUM_DL_POWER_RL_SETUPRQSTTDD							567
#define NBAP_IE_ID_CCTRCH_MINIMUM_DL_POWER_RL_SETUPRQSTTDD							568
#define NBAP_IE_ID_CCTRCH_MAXIMUM_DL_POWER_RL_ADDITIONRQSTTDD						569
#define NBAP_IE_ID_CCTRCH_MINIMUM_DL_POWER_RL_ADDITIONRQSTTDD						570
#define NBAP_IE_ID_CCTRCH_MAXIMUM_DL_POWER_INFORMATIONADD_RL_RECONFPREPTDD			571
#define NBAP_IE_ID_CCTRCH_MINIMUM_DL_POWER_INFORMATIONADD_RL_RECONFPREPTDD			572
#define NBAP_IE_ID_CCTRCH_MAXIMUM_DL_POWER_INFORMATIONMODIFY_RL_RECONFPREPTDD		573
#define NBAP_IE_ID_CCTRCH_MINIMUM_DL_POWER_INFORMATIONMODIFY_RL_RECONFPREPTDD		574
#define NBAP_IE_ID_MAXIMUM_DL_POWER_MODIFY_LCR_INFORMATIONMODIFY_RL_RECONFPREPTDD	575
#define NBAP_IE_ID_MINIMUM_DL_POWER_MODIFY_LCR_INFORMATIONMODIFY_RL_RECONFPREPTDD	576
#define NBAP_IE_ID_DL_DPCH_LCR_INFORMATIONMODIFY_MODIFYLIST_RL_RECONFRQSTTDD		577
#define NBAP_IE_ID_CCTRCH_MAXIMUM_DL_POWER_INFORMATIONMODIFY_RL_RECONFRQSTTDD		578
#define NBAP_IE_ID_CCTRCH_MINIMUM_DL_POWER_INFORMATIONMODIFY_RL_RECONFRQSTTDD		579
#define NBAP_IE_ID_INITIAL_DL_POWER_TIMESLOTLCR_INFORMATIONITEM						580
#define NBAP_IE_ID_MAXIMUM_DL_POWER_TIMESLOTLCR_INFORMATIONITEM						581
#define NBAP_IE_ID_MINIMUM_DL_POWER_TIMESLOTLCR_INFORMATIONITEM						582
#define NBAP_IE_ID_HS_DSCHPROVIDEDBITRATEVALUEINFORMATION							583
#define NBAP_IE_ID_HS_DSCHREQUIREDPOWERVALUEINFORMATION								585
#define NBAP_IE_ID_HS_DSCHREQUIREDPOWERVALUE										586
#define NBAP_IE_ID_TRANSMITTEDCARRIERPOWEROFALLCODESNOTUSEDFORHSTRANSMISSION		587
#define NBAP_IE_ID_HS_SICH_RECEPTION_QUALITY 										588
#define NBAP_IE_ID_HS_SICH_RECEPTION_QUALITY_MEASUREMENT_VALUE 						589
#define NBAP_IE_ID_HSSICH_INFO_DM_RPRT 												590
#define NBAP_IE_ID_HSSICH_INFO_DM_RQST												591
#define NBAP_IE_ID_HSSICH_INFO_DM_RSP												592
#define NBAP_IE_ID_BEST_CELL_PORTIONS_VALUE											593
#define NBAP_IE_ID_PRIMARY_CPICH_USAGE_FOR_CHANNEL_ESTIMATION						594
#define NBAP_IE_ID_SECONDARY_CPICH_INFORMATION_CHANGE								595
#define NBAP_IE_ID_NUMBEROFREPORTEDCELLPORTIONS										596
#define NBAP_IE_ID_CELLPORTION_INFORMATIONITEM_CELL_SETUPRQSTFDD					597
#define NBAP_IE_ID_CELLPORTION_INFORMATIONLIST_CELL_SETUPRQSTFDD					598
#define NBAP_IE_ID_TIMESLOTISCP_LCR_INFOLIST_RL_RECONFPREPTDD						599
#define NBAP_IE_ID_SECONDARY_CPICH_INFORMATION										600
#define NBAP_IE_ID_RECEIVED_TOTAL_WIDE_BAND_POWER_FOR_CELLPORTION					601
#define NBAP_IE_ID_UNIDIRECTIONAL_DCH_INDICATOR										602
#define NBAP_IE_ID_TIMINGADJUSTMENTVALUELCR											603
#define NBAP_IE_ID_MULTIPLERL_DL_DPCH_INFORMATIONLIST								604
#define NBAP_IE_ID_MULTIPLERL_DL_DPCH_INFORMATIONMODIFYLIST							605
#define NBAP_IE_ID_MULTIPLERL_UL_DPCH_INFORMATIONLIST								606
#define NBAP_IE_ID_MULTIPLERL_UL_DPCH_INFORMATIONMODIFYLIST							607
#define NBAP_IE_ID_RL_ID															608
#define NBAP_IE_ID_SAT_INFO_ALMANAC_EXTITEM											609
#define NBAP_IE_ID_HSDPA_CAPABILITY													610
#define NBAP_IE_ID_HSDSCH_RESOURCES_INFORMATION_AUDITRSP							611
#define NBAP_IE_ID_HSDSCH_RESOURCES_INFORMATION_RESOURCESTATUSIND					612
#define NBAP_IE_ID_HSDSCH_MACDFLOWS_TO_ADD											613
#define NBAP_IE_ID_HSDSCH_MACDFLOWS_TO_DELETE										614
#define NBAP_IE_ID_HSDSCH_INFORMATION_TO_MODIFY_UNSYNCHRONISED						615
#define NBAP_IE_ID_TNLQOS															616
#define NBAP_IE_ID_RECEIVED_TOTAL_WIDE_BAND_POWER_FOR_CELLPORTION_VALUE				617
#define NBAP_IE_ID_TRANSMITTED_CARRIER_POWER_FOR_CELLPORTION						618
#define NBAP_IE_ID_TRANSMITTED_CARRIER_POWER_FOR_CELLPORTION_VALUE 					619
#define NBAP_IE_ID_TRANSMITTEDCARRIERPOWEROFALLCODESNOTUSEDFORHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHORE_HICHTRANSMISSIONCELLPORTION		620
#define NBAP_IE_ID_TRANSMITTEDCARRIERPOWEROFALLCODESNOTUSEDFORHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHORE_HICHTRANSMISSIONCELLPORTIONVALUE	621
#define NBAP_IE_ID_UPPTSINTERFERENCEVALUE											622
#define NBAP_IE_ID_PRIMARYCCPCH_RSCP_DELTA											623
#define NBAP_IE_ID_MEASUREMENTRECOVERYBEHAVIOR										624
#define NBAP_IE_ID_MEASUREMENTRECOVERYREPORTINGINDICATOR							625
#define NBAP_IE_ID_MEASUREMENTRECOVERYSUPPORTINDICATOR								626
#define NBAP_IE_ID_TSTD_INDICATOR													627
#define NBAP_IE_ID_MULTIPLE_RL_INFORMATION_RL_RECONFPREPTDD							628
#define NBAP_IE_ID_MULTIPLE_RL_INFORMATION_RL_RECONFRQSTTDD							629
#define NBAP_IE_ID_DL_DPCH_POWER_INFORMATION_RL_RECONFPREPFDD						630
#define NBAP_IE_ID_F_DPCH_INFORMATION_RL_RECONFPREPFDD								631
#define NBAP_IE_ID_F_DPCH_INFORMATION_RL_SETUPRQSTFDD								632
#define NBAP_IE_ID_ADDITIONAL_S_CCPCH_PARAMETERS_CTCH_RECONFRQSTTDD					633
#define NBAP_IE_ID_ADDITIONAL_S_CCPCH_PARAMETERS_CTCH_SETUPRQSTTDD					634
#define NBAP_IE_ID_ADDITIONAL_S_CCPCH_LCR_PARAMETERS_CTCH_RECONFRQSTTDD				635
#define NBAP_IE_ID_ADDITIONAL_S_CCPCH_LCR_PARAMETERS_CTCH_SETUPRQSTTDD				636
#define NBAP_IE_ID_MICH_CFN															637
#define NBAP_IE_ID_MICH_INFORMATION_AUDITRSP										638
#define NBAP_IE_ID_MICH_INFORMATION_RESOURCESTATUSIND								639
#define NBAP_IE_ID_MICH_PARAMETERS_CTCH_RECONFRQSTFDD								640
#define NBAP_IE_ID_MICH_PARAMETERS_CTCH_RECONFRQSTTDD								641
#define NBAP_IE_ID_MICH_PARAMETERS_CTCH_SETUPRQSTFDD								642
#define NBAP_IE_ID_MICH_PARAMETERS_CTCH_SETUPRQSTTDD								643
#define NBAP_IE_ID_MODIFICATION_PERIOD												644
#define NBAP_IE_ID_NI_INFORMATION_NOTIFUPDATECMD									645
#define NBAP_IE_ID_S_CCPCH_INFORMATIONLISTEXT_AUDITRSP								646
#define NBAP_IE_ID_S_CCPCH_INFORMATIONLISTEXT_RESOURCESTATUSIND						647
#define NBAP_IE_ID_S_CCPCH_LCR_INFORMATIONLISTEXT_AUDITRSP							648
#define NBAP_IE_ID_S_CCPCH_LCR_INFORMATIONLISTEXT_RESOURCESTATUSIND					649
#define NBAP_IE_ID_HARQ_PREAMBLE_MODE												650
#define NBAP_IE_ID_INITIAL_DL_DPCH_TIMINGADJUSTMENT									651
#define NBAP_IE_ID_INITIAL_DL_DPCH_TIMINGADJUSTMENT_ALLOWED							652
#define NBAP_IE_ID_DLTRANSMISSIONBRANCHLOADVALUE									653
#define NBAP_IE_ID_POWER_LOCAL_CELL_GROUP_CHOICE_CM_RQST							654
#define NBAP_IE_ID_POWER_LOCAL_CELL_GROUP_CHOICE_CM_RSP								655
#define NBAP_IE_ID_POWER_LOCAL_CELL_GROUP_CHOICE_CM_RPRT							656
#define NBAP_IE_ID_SYNCHRONISATIONINDICATOR											657
#define NBAP_IE_ID_HSDPA_AND_EDCH_CELLPORTION_INFORMATION_PSCH_RECONFRQST			658
#define NBAP_IE_ID_HSDPA_AND_EDCH_CELLPORTION_INFORMATIONLIST_PSCH_RECONFRQST		659
#define NBAP_IE_ID_HS_DSCHREQUIREDPOWERVALUE_FOR_CELL_PORTION						660
#define NBAP_IE_ID_HS_DSCHREQUIREDPOWERVALUEINFORMATION_FOR_CELLPORTION				661
#define NBAP_IE_ID_HS_DSCHPROVIDEDBITRATEVALUEINFORMATION_FOR_CELLPORTION			662
#define NBAP_IE_ID_E_AGCH_AND_E_RGCH_E_HICH_FDD_SCRAMBLING_CODE						663
#define NBAP_IE_ID_E_AGCH_FDD_CODE_INFORMATION										664
#define NBAP_IE_ID_E_DCH_CAPABILITY													665
#define NBAP_IE_ID_E_DCH_FDD_DL_CONTROL_CHANNEL_INFORMATION							666
#define NBAP_IE_ID_E_DCH_FDD_INFORMATION											667
#define NBAP_IE_ID_E_DCH_FDD_INFORMATION_RESPONSE									668
#define NBAP_IE_ID_E_DCH_FDD_INFORMATION_TO_MODIFY									669
#define NBAP_IE_ID_E_DCH_MACDFLOWS_TO_ADD											670
#define NBAP_IE_ID_E_DCH_MACDFLOWS_TO_DELETE										671
#define NBAP_IE_ID_E_DCH_RESOURCES_INFORMATION_AUDITRSP								672
#define NBAP_IE_ID_E_DCH_RESOURCES_INFORMATION_RESOURCESTATUSIND					673
#define NBAP_IE_ID_E_DCH_RL_INDICATION												674
#define NBAP_IE_ID_E_DCH_RL_SET_ID													675
#define NBAP_IE_ID_E_DPCH_INFORMATION_RL_RECONFPREPFDD								676
#define NBAP_IE_ID_E_DPCH_INFORMATION_RL_SETUPRQSTFDD								677
#define NBAP_IE_ID_E_RGCH_E_HICH_FDD_CODE_INFORMATION								678
#define NBAP_IE_ID_SERVING_E_DCH_RL_ID												679
#define NBAP_IE_ID_UL_DPDCH_INDICATOR_FOR_E_DCH_OPERATION							680
#define NBAP_IE_ID_FDD_S_CCPCH_FRAMEOFFSET_CTCH_SETUPRQSTFDD						681
#define NBAP_IE_ID_E_DPCH_INFORMATION_RL_RECONFRQSTFDD								682
#define NBAP_IE_ID_MAXIMUM_TARGET_RECEIVEDTOTALWIDEBANDPOWER						683
#define NBAP_IE_ID_E_DCHPROVIDEDBITRATEVALUEINFORMATION								684
#define NBAP_IE_ID_HARQ_PREAMBLE_MODE_ACTIVATION_INDICATOR							685
#define NBAP_IE_ID_RL_SPECIFIC_E_DCH_INFO											686
#define NBAP_IE_ID_E_DCH_CAPACITYCONSUMPTIONLAW										687
#define NBAP_IE_ID_MULTIPLE_DEDICATEDMEASUREMENTVALUELIST_TDD_DM_RSP				688
#define NBAP_IE_ID_MULTIPLE_DEDICATEDMEASUREMENTVALUELIST_LCR_TDD_DM_RSP			689
#define NBAP_IE_ID_E_DCH_REARRANGELIST_BEARER_REARRANGEIND							690
#define NBAP_IE_ID_HSDPA_AND_EDCH_CELLPORTION_INFORMATIONLISTIE_PSCH_RECONFRQST		691
#define NBAP_IE_ID_MULTIPLERL_DL_CCTRCH_INFORMATIONMODIFYLIST_RL_RECONFRQSTTDD		692
#define NBAP_IE_ID_TARGET_NONSERVING_EDCH_TO_TOTAL_EDCH_POWER_RATIO					693
#define NBAP_IE_ID_CELLPORTION_INFORMATIONITEM_CELL_RECONFRQSTFDD					694
#define NBAP_IE_ID_CELLPORTION_INFORMATIONLIST_CELL_RECONFRQSTFDD					695
#define NBAP_IE_ID_MULTIPLE_PUSCH_INFOLIST_DM_RSP									696
#define NBAP_IE_ID_MULTIPLE_PUSCH_INFOLIST_DM_RPRT									697
#define NBAP_IE_ID_REFERENCE_RECEIVEDTOTALWIDEBANDPOWER								698


static int dissect_nbap_InitiatingMessageValueValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree);
static int dissect_nbap_SuccessfulOutcomeValueValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree);
static int dissect_nbap_UnsuccessfulOutcomeValueValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree);
static int dissect_nbap_ProtocolIEValueValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree);

/*--- Included file: packet-nbap-fn.c ---*/
#line 1 "packet-nbap-fn.c"
/*--- Fields for imported types ---*/



static const value_string nbap_ProcedureCode_vals[] = {
  {   0, "id-audit" },
  {   1, "id-auditRequired" },
  {   2, "id-blockResource" },
  {   3, "id-cellDeletion" },
  {   4, "id-cellReconfiguration" },
  {   5, "id-cellSetup" },
  {  45, "id-cellSynchronisationInitiation" },
  {  46, "id-cellSynchronisationReconfiguration" },
  {  47, "id-cellSynchronisationReporting" },
  {  48, "id-cellSynchronisationTermination" },
  {  49, "id-cellSynchronisationFailure" },
  {   6, "id-commonMeasurementFailure" },
  {   7, "id-commonMeasurementInitiation" },
  {   8, "id-commonMeasurementReport" },
  {   9, "id-commonMeasurementTermination" },
  {  10, "id-commonTransportChannelDelete" },
  {  11, "id-commonTransportChannelReconfigure" },
  {  12, "id-commonTransportChannelSetup" },
  {  14, "id-compressedModeCommand" },
  {  16, "id-dedicatedMeasurementFailure" },
  {  17, "id-dedicatedMeasurementInitiation" },
  {  18, "id-dedicatedMeasurementReport" },
  {  19, "id-dedicatedMeasurementTermination" },
  {  20, "id-downlinkPowerControl" },
  {  38, "id-downlinkPowerTimeslotControl" },
  {  35, "id-errorIndicationForCommon" },
  {  21, "id-errorIndicationForDedicated" },
  {  40, "id-informationExchangeFailure" },
  {  41, "id-informationExchangeInitiation" },
  {  42, "id-informationExchangeTermination" },
  {  43, "id-informationReporting" },
  {  50, "id-BearerRearrangement" },
  {  53, "id-mBMSNotificationUpdate" },
  {  37, "id-physicalSharedChannelReconfiguration" },
  {  36, "id-privateMessageForCommon" },
  {  22, "id-privateMessageForDedicated" },
  {  23, "id-radioLinkAddition" },
  {  24, "id-radioLinkDeletion" },
  {  25, "id-radioLinkFailure" },
  {  39, "id-radioLinkPreemption" },
  {  26, "id-radioLinkRestoration" },
  {  27, "id-radioLinkSetup" },
  {  13, "id-reset" },
  {  28, "id-resourceStatusIndication" },
  {  44, "id-cellSynchronisationAdjustment" },
  {  29, "id-synchronisedRadioLinkReconfigurationCancellation" },
  {  30, "id-synchronisedRadioLinkReconfigurationCommit" },
  {  31, "id-synchronisedRadioLinkReconfigurationPreparation" },
  {  32, "id-systemInformationUpdate" },
  {  33, "id-unblockResource" },
  {  34, "id-unSynchronisedRadioLinkReconfiguration" },
  {  51, "id-radioLinkActivation" },
  {  52, "id-radioLinkParameterUpdate" },
  { 0, NULL }
};


static int
dissect_nbap_ProcedureCode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 255U, &ProcedureCode, FALSE);

#line 23 "nbap.cnf"
	if (check_col(actx->pinfo->cinfo, COL_INFO))
       col_add_fstr(actx->pinfo->cinfo, COL_INFO, "%s ",
                   val_to_str(ProcedureCode, nbap_ProcedureCode_vals,
                              "unknown message"));

  return offset;
}
static int dissect_procedureCode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProcedureCode(tvb, offset, actx, tree, hf_nbap_procedureCode);
}


static const value_string nbap_T_ddMode_vals[] = {
  {   0, "tdd" },
  {   1, "fdd" },
  {   2, "common" },
  { 0, NULL }
};


static int
dissect_nbap_T_ddMode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, &ddMode, TRUE, 0, NULL);

  return offset;
}
static int dissect_ddMode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_T_ddMode(tvb, offset, actx, tree, hf_nbap_ddMode);
}


static const per_sequence_t ProcedureID_sequence[] = {
  { &hf_nbap_procedureCode  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProcedureCode },
  { &hf_nbap_ddMode         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_T_ddMode },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_ProcedureID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_ProcedureID, ProcedureID_sequence);

  return offset;
}
static int dissect_procedureID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProcedureID(tvb, offset, actx, tree, hf_nbap_procedureID);
}


static const value_string nbap_Criticality_vals[] = {
  {   0, "reject" },
  {   1, "ignore" },
  {   2, "notify" },
  { 0, NULL }
};


static int
dissect_nbap_Criticality(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_criticality(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Criticality(tvb, offset, actx, tree, hf_nbap_criticality);
}
static int dissect_procedureCriticality(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Criticality(tvb, offset, actx, tree, hf_nbap_procedureCriticality);
}
static int dissect_iECriticality(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Criticality(tvb, offset, actx, tree, hf_nbap_iECriticality);
}
static int dissect_firstCriticality(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Criticality(tvb, offset, actx, tree, hf_nbap_firstCriticality);
}
static int dissect_secondCriticality(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Criticality(tvb, offset, actx, tree, hf_nbap_secondCriticality);
}


static const value_string nbap_MessageDiscriminator_vals[] = {
  {   0, "common" },
  {   1, "dedicated" },
  { 0, NULL }
};


static int
dissect_nbap_MessageDiscriminator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_messageDiscriminator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MessageDiscriminator(tvb, offset, actx, tree, hf_nbap_messageDiscriminator);
}



static int
dissect_nbap_INTEGER_0_127(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 127U, NULL, FALSE);

  return offset;
}
static int dissect_codeNumber(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_INTEGER_0_127(tvb, offset, actx, tree, hf_nbap_codeNumber);
}
static int dissect_shortTransActionId(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_INTEGER_0_127(tvb, offset, actx, tree, hf_nbap_shortTransActionId);
}



static int
dissect_nbap_INTEGER_0_32767(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 32767U, NULL, FALSE);

  return offset;
}
static int dissect_altitude(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_INTEGER_0_32767(tvb, offset, actx, tree, hf_nbap_altitude);
}
static int dissect_longTransActionId(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_INTEGER_0_32767(tvb, offset, actx, tree, hf_nbap_longTransActionId);
}


static const value_string nbap_TransactionID_vals[] = {
  {   0, "shortTransActionId" },
  {   1, "longTransActionId" },
  { 0, NULL }
};

static const per_choice_t TransactionID_choice[] = {
  {   0, &hf_nbap_shortTransActionId, ASN1_NO_EXTENSIONS     , dissect_nbap_INTEGER_0_127 },
  {   1, &hf_nbap_longTransActionId, ASN1_NO_EXTENSIONS     , dissect_nbap_INTEGER_0_32767 },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_TransactionID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_TransactionID, TransactionID_choice,
                                 NULL);

  return offset;
}
static int dissect_transactionID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TransactionID(tvb, offset, actx, tree, hf_nbap_transactionID);
}



static int
dissect_nbap_InitiatingMessageValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
#line 44 "nbap.cnf"

	offset = dissect_nbap_InitiatingMessageValueValue(tvb, offset, actx, tree);



  return offset;
}
static int dissect_initiatingMessageValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_InitiatingMessageValue(tvb, offset, actx, tree, hf_nbap_initiatingMessageValue);
}


static const per_sequence_t InitiatingMessage_sequence[] = {
  { &hf_nbap_procedureID    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProcedureID },
  { &hf_nbap_criticality    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_Criticality },
  { &hf_nbap_messageDiscriminator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_MessageDiscriminator },
  { &hf_nbap_transactionID  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_TransactionID },
  { &hf_nbap_initiatingMessageValue, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_InitiatingMessageValue },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_InitiatingMessage(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_InitiatingMessage, InitiatingMessage_sequence);

  return offset;
}
static int dissect_initiatingMessage(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_InitiatingMessage(tvb, offset, actx, tree, hf_nbap_initiatingMessage);
}



static int
dissect_nbap_SuccessfulOutcomeValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
#line 48 "nbap.cnf"

	offset = dissect_nbap_SuccessfulOutcomeValueValue(tvb, offset, actx, tree);



  return offset;
}
static int dissect_successfulOutcomeValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SuccessfulOutcomeValue(tvb, offset, actx, tree, hf_nbap_successfulOutcomeValue);
}


static const per_sequence_t SuccessfulOutcome_sequence[] = {
  { &hf_nbap_procedureID    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProcedureID },
  { &hf_nbap_criticality    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_Criticality },
  { &hf_nbap_messageDiscriminator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_MessageDiscriminator },
  { &hf_nbap_transactionID  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_TransactionID },
  { &hf_nbap_successfulOutcomeValue, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_SuccessfulOutcomeValue },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_SuccessfulOutcome(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_SuccessfulOutcome, SuccessfulOutcome_sequence);

  return offset;
}
static int dissect_succesfulOutcome(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SuccessfulOutcome(tvb, offset, actx, tree, hf_nbap_succesfulOutcome);
}



static int
dissect_nbap_UnsuccessfulOutcomeValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
#line 60 "nbap.cnf"

	offset = dissect_nbap_UnsuccessfulOutcomeValueValue(tvb, offset, actx, tree);



  return offset;
}
static int dissect_unsuccessfulOutcomeValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UnsuccessfulOutcomeValue(tvb, offset, actx, tree, hf_nbap_unsuccessfulOutcomeValue);
}


static const per_sequence_t UnsuccessfulOutcome_sequence[] = {
  { &hf_nbap_procedureID    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProcedureID },
  { &hf_nbap_criticality    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_Criticality },
  { &hf_nbap_messageDiscriminator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_MessageDiscriminator },
  { &hf_nbap_transactionID  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_TransactionID },
  { &hf_nbap_unsuccessfulOutcomeValue, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_UnsuccessfulOutcomeValue },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_UnsuccessfulOutcome(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_UnsuccessfulOutcome, UnsuccessfulOutcome_sequence);

  return offset;
}
static int dissect_unsuccesfulOutcome(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UnsuccessfulOutcome(tvb, offset, actx, tree, hf_nbap_unsuccesfulOutcome);
}



static int
dissect_nbap_OutcomeValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
#line 52 "nbap.cnf"
	/* FIX ME */
	guint length;

	offset = dissect_per_length_determinant(tvb, offset, actx, tree, hf_nbap_pdu_length, &length);
	offset = offset + length;
	BYTE_ALIGN_OFFSET(offset)



  return offset;
}
static int dissect_outcomeValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_OutcomeValue(tvb, offset, actx, tree, hf_nbap_outcomeValue);
}


static const per_sequence_t Outcome_sequence[] = {
  { &hf_nbap_procedureID    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProcedureID },
  { &hf_nbap_criticality    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_Criticality },
  { &hf_nbap_messageDiscriminator, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_MessageDiscriminator },
  { &hf_nbap_transactionID  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_TransactionID },
  { &hf_nbap_outcomeValue   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_OutcomeValue },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Outcome(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Outcome, Outcome_sequence);

  return offset;
}
static int dissect_outcome(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Outcome(tvb, offset, actx, tree, hf_nbap_outcome);
}


static const value_string nbap_NBAP_PDU_vals[] = {
  {   0, "initiatingMessage" },
  {   1, "succesfulOutcome" },
  {   2, "unsuccesfulOutcome" },
  {   3, "outcome" },
  { 0, NULL }
};

static const per_choice_t NBAP_PDU_choice[] = {
  {   0, &hf_nbap_initiatingMessage, ASN1_EXTENSION_ROOT    , dissect_nbap_InitiatingMessage },
  {   1, &hf_nbap_succesfulOutcome, ASN1_EXTENSION_ROOT    , dissect_nbap_SuccessfulOutcome },
  {   2, &hf_nbap_unsuccesfulOutcome, ASN1_EXTENSION_ROOT    , dissect_nbap_UnsuccessfulOutcome },
  {   3, &hf_nbap_outcome        , ASN1_EXTENSION_ROOT    , dissect_nbap_Outcome },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_NBAP_PDU(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_NBAP_PDU, NBAP_PDU_choice,
                                 NULL);

  return offset;
}


static const value_string nbap_ProtocolIE_ID_vals[] = {
  {   0, "id-AICH-Information" },
  {   1, "id-AICH-InformationItem-ResourceStatusInd" },
  {   7, "id-BCH-Information" },
  {   8, "id-BCH-InformationItem-ResourceStatusInd" },
  {   9, "id-BCCH-ModificationTime" },
  {  10, "id-BlockingPriorityIndicator" },
  {  13, "id-Cause" },
  {  14, "id-CCP-InformationItem-AuditRsp" },
  {  15, "id-CCP-InformationList-AuditRsp" },
  {  16, "id-CCP-InformationItem-ResourceStatusInd" },
  {  17, "id-Cell-InformationItem-AuditRsp" },
  {  18, "id-Cell-InformationItem-ResourceStatusInd" },
  {  19, "id-Cell-InformationList-AuditRsp" },
  {  23, "id-CellParameterID" },
  {  24, "id-CFN" },
  {  25, "id-C-ID" },
  {  39, "id-CommonMeasurementAccuracy" },
  {  31, "id-CommonMeasurementObjectType-CM-Rprt" },
  {  32, "id-CommonMeasurementObjectType-CM-Rqst" },
  {  33, "id-CommonMeasurementObjectType-CM-Rsp" },
  {  34, "id-CommonMeasurementType" },
  {  35, "id-CommonPhysicalChannelID" },
  {  36, "id-CommonPhysicalChannelType-CTCH-SetupRqstFDD" },
  {  37, "id-CommonPhysicalChannelType-CTCH-SetupRqstTDD" },
  {  40, "id-CommunicationControlPortID" },
  {  43, "id-ConfigurationGenerationID" },
  {  44, "id-CRNC-CommunicationContextID" },
  {  45, "id-CriticalityDiagnostics" },
  {  48, "id-DCHs-to-Add-FDD" },
  {  49, "id-DCH-AddList-RL-ReconfPrepTDD" },
  {  50, "id-DCHs-to-Add-TDD" },
  {  52, "id-DCH-DeleteList-RL-ReconfPrepFDD" },
  {  53, "id-DCH-DeleteList-RL-ReconfPrepTDD" },
  {  54, "id-DCH-DeleteList-RL-ReconfRqstFDD" },
  {  55, "id-DCH-DeleteList-RL-ReconfRqstTDD" },
  {  56, "id-DCH-FDD-Information" },
  {  57, "id-DCH-TDD-Information" },
  {  59, "id-DCH-InformationResponse" },
  {  62, "id-FDD-DCHs-to-Modify" },
  {  63, "id-TDD-DCHs-to-Modify" },
  {  65, "id-DCH-ModifyList-RL-ReconfRqstTDD" },
  { 135, "id-DCH-RearrangeList-Bearer-RearrangeInd" },
  {  67, "id-DedicatedMeasurementObjectType-DM-Rprt" },
  {  68, "id-DedicatedMeasurementObjectType-DM-Rqst" },
  {  69, "id-DedicatedMeasurementObjectType-DM-Rsp" },
  {  70, "id-DedicatedMeasurementType" },
  {  72, "id-DL-CCTrCH-InformationItem-RL-SetupRqstTDD" },
  {  73, "id-DL-CCTrCH-InformationList-RL-AdditionRqstTDD" },
  {  76, "id-DL-CCTrCH-InformationList-RL-SetupRqstTDD" },
  {  77, "id-DL-DPCH-InformationItem-RL-AdditionRqstTDD" },
  {  79, "id-DL-DPCH-InformationList-RL-SetupRqstTDD" },
  {  81, "id-DL-DPCH-Information-RL-ReconfPrepFDD" },
  {  82, "id-DL-DPCH-Information-RL-ReconfRqstFDD" },
  {  83, "id-DL-DPCH-Information-RL-SetupRqstFDD" },
  {  21, "id-DL-DPCH-TimingAdjustment" },
  {  84, "id-DL-ReferencePowerInformationItem-DL-PC-Rqst" },
  {  85, "id-DLReferencePower" },
  {  86, "id-DLReferencePowerList-DL-PC-Rqst" },
  {  87, "id-Unused-ProtocolIE-ID-87" },
  {  89, "id-Unused-ProtocolIE-ID-89" },
  {  91, "id-Unused-ProtocolIE-ID-91" },
  {  93, "id-Unused-ProtocolIE-ID-93" },
  {  96, "id-DSCHs-to-Add-TDD" },
  {  98, "id-DSCH-Information-DeleteList-RL-ReconfPrepTDD" },
  { 100, "id-DSCH-Information-ModifyList-RL-ReconfPrepTDD" },
  { 105, "id-DSCH-InformationResponse" },
  { 106, "id-Unused-ProtocolIE-ID-106" },
  { 107, "id-DSCH-TDD-Information" },
  { 108, "id-Unused-ProtocolIE-ID-108" },
  { 112, "id-Unused-ProtocolIE-ID-112" },
  { 136, "id-DSCH-RearrangeList-Bearer-RearrangeInd" },
  { 113, "id-End-Of-Audit-Sequence-Indicator" },
  { 116, "id-FACH-Information" },
  { 117, "id-FACH-InformationItem-ResourceStatusInd" },
  { 120, "id-FACH-ParametersList-CTCH-ReconfRqstTDD" },
  { 121, "id-FACH-ParametersListIE-CTCH-SetupRqstFDD" },
  { 122, "id-FACH-ParametersListIE-CTCH-SetupRqstTDD" },
  { 123, "id-IndicationType-ResourceStatusInd" },
  { 124, "id-Local-Cell-ID" },
  {   2, "id-Local-Cell-Group-InformationItem-AuditRsp" },
  {   3, "id-Local-Cell-Group-InformationItem-ResourceStatusInd" },
  {   4, "id-Local-Cell-Group-InformationItem2-ResourceStatusInd" },
  {   5, "id-Local-Cell-Group-InformationList-AuditRsp" },
  { 125, "id-Local-Cell-InformationItem-AuditRsp" },
  { 126, "id-Local-Cell-InformationItem-ResourceStatusInd" },
  { 127, "id-Local-Cell-InformationItem2-ResourceStatusInd" },
  { 128, "id-Local-Cell-InformationList-AuditRsp" },
  { 129, "id-AdjustmentPeriod" },
  { 130, "id-MaxAdjustmentStep" },
  { 131, "id-MaximumTransmissionPower" },
  { 132, "id-MeasurementFilterCoefficient" },
  { 133, "id-MeasurementID" },
  { 115, "id-MessageStructure" },
  { 134, "id-MIB-SB-SIB-InformationList-SystemInfoUpdateRqst" },
  { 143, "id-NodeB-CommunicationContextID" },
  { 455, "id-NeighbouringCellMeasurementInformation" },
  { 144, "id-P-CCPCH-Information" },
  { 145, "id-P-CCPCH-InformationItem-ResourceStatusInd" },
  { 146, "id-P-CPICH-Information" },
  { 147, "id-P-CPICH-InformationItem-ResourceStatusInd" },
  { 148, "id-P-SCH-Information" },
  { 150, "id-PCCPCH-Information-Cell-ReconfRqstTDD" },
  { 151, "id-PCCPCH-Information-Cell-SetupRqstTDD" },
  { 155, "id-PCH-Parameters-CTCH-ReconfRqstTDD" },
  { 156, "id-PCH-ParametersItem-CTCH-SetupRqstFDD" },
  { 157, "id-PCH-ParametersItem-CTCH-SetupRqstTDD" },
  { 158, "id-PCH-Information" },
  { 161, "id-PDSCH-Information-AddListIE-PSCH-ReconfRqst" },
  { 162, "id-PDSCH-Information-ModifyListIE-PSCH-ReconfRqst" },
  { 163, "id-PDSCHSets-AddList-PSCH-ReconfRqst" },
  { 164, "id-PDSCHSets-DeleteList-PSCH-ReconfRqst" },
  { 165, "id-PDSCHSets-ModifyList-PSCH-ReconfRqst" },
  { 166, "id-PICH-Information" },
  { 168, "id-PICH-Parameters-CTCH-ReconfRqstTDD" },
  { 169, "id-PowerAdjustmentType" },
  { 170, "id-PRACH-Information" },
  { 175, "id-PrimaryCCPCH-Information-Cell-ReconfRqstFDD" },
  { 176, "id-PrimaryCCPCH-Information-Cell-SetupRqstFDD" },
  { 177, "id-PrimaryCPICH-Information-Cell-ReconfRqstFDD" },
  { 178, "id-PrimaryCPICH-Information-Cell-SetupRqstFDD" },
  { 179, "id-PrimarySCH-Information-Cell-ReconfRqstFDD" },
  { 180, "id-PrimarySCH-Information-Cell-SetupRqstFDD" },
  { 181, "id-PrimaryScramblingCode" },
  { 183, "id-SCH-Information-Cell-ReconfRqstTDD" },
  { 184, "id-SCH-Information-Cell-SetupRqstTDD" },
  { 185, "id-PUSCH-Information-AddListIE-PSCH-ReconfRqst" },
  { 186, "id-PUSCH-Information-ModifyListIE-PSCH-ReconfRqst" },
  { 187, "id-PUSCHSets-AddList-PSCH-ReconfRqst" },
  { 188, "id-PUSCHSets-DeleteList-PSCH-ReconfRqst" },
  { 189, "id-PUSCHSets-ModifyList-PSCH-ReconfRqst" },
  { 190, "id-RACH-Information" },
  { 196, "id-RACH-ParametersItem-CTCH-SetupRqstFDD" },
  { 197, "id-RACH-ParameterItem-CTCH-SetupRqstTDD" },
  { 198, "id-ReportCharacteristics" },
  { 199, "id-Reporting-Object-RL-FailureInd" },
  { 200, "id-Reporting-Object-RL-RestoreInd" },
  { 202, "id-RL-InformationItem-DM-Rprt" },
  { 203, "id-RL-InformationItem-DM-Rqst" },
  { 204, "id-RL-InformationItem-DM-Rsp" },
  { 205, "id-RL-InformationItem-RL-AdditionRqstFDD" },
  { 206, "id-RL-informationItem-RL-DeletionRqst" },
  { 207, "id-RL-InformationItem-RL-FailureInd" },
  { 286, "id-RL-InformationItem-RL-PreemptRequiredInd" },
  { 208, "id-RL-InformationItem-RL-ReconfPrepFDD" },
  { 209, "id-RL-InformationItem-RL-ReconfRqstFDD" },
  { 210, "id-RL-InformationItem-RL-RestoreInd" },
  { 211, "id-RL-InformationItem-RL-SetupRqstFDD" },
  { 212, "id-RL-InformationList-RL-AdditionRqstFDD" },
  { 213, "id-RL-informationList-RL-DeletionRqst" },
  { 237, "id-RL-InformationList-RL-PreemptRequiredInd" },
  { 214, "id-RL-InformationList-RL-ReconfPrepFDD" },
  { 215, "id-RL-InformationList-RL-ReconfRqstFDD" },
  { 216, "id-RL-InformationList-RL-SetupRqstFDD" },
  { 217, "id-RL-InformationResponseItem-RL-AdditionRspFDD" },
  { 218, "id-RL-InformationResponseItem-RL-ReconfReady" },
  { 219, "id-RL-InformationResponseItem-RL-ReconfRsp" },
  { 220, "id-RL-InformationResponseItem-RL-SetupRspFDD" },
  { 221, "id-RL-InformationResponseList-RL-AdditionRspFDD" },
  { 222, "id-RL-InformationResponseList-RL-ReconfReady" },
  { 223, "id-RL-InformationResponseList-RL-ReconfRsp" },
  { 224, "id-RL-InformationResponseList-RL-SetupRspFDD" },
  { 225, "id-RL-InformationResponse-RL-AdditionRspTDD" },
  { 226, "id-RL-InformationResponse-RL-SetupRspTDD" },
  { 227, "id-RL-Information-RL-AdditionRqstTDD" },
  { 228, "id-RL-Information-RL-ReconfRqstTDD" },
  { 229, "id-RL-Information-RL-ReconfPrepTDD" },
  { 230, "id-RL-Information-RL-SetupRqstTDD" },
  { 236, "id-RL-ReconfigurationFailureItem-RL-ReconfFailure" },
  { 238, "id-RL-Set-InformationItem-DM-Rprt" },
  { 240, "id-RL-Set-InformationItem-DM-Rsp" },
  { 241, "id-RL-Set-InformationItem-RL-FailureInd" },
  { 242, "id-RL-Set-InformationItem-RL-RestoreInd" },
  { 247, "id-S-CCPCH-Information" },
  { 249, "id-S-CPICH-Information" },
  { 251, "id-SCH-Information" },
  { 253, "id-S-SCH-Information" },
  { 257, "id-Secondary-CCPCHListIE-CTCH-ReconfRqstTDD" },
  { 258, "id-Secondary-CCPCH-parameterListIE-CTCH-SetupRqstTDD" },
  { 259, "id-Secondary-CCPCH-Parameters-CTCH-ReconfRqstTDD" },
  { 260, "id-SecondaryCPICH-InformationItem-Cell-ReconfRqstFDD" },
  { 261, "id-SecondaryCPICH-InformationItem-Cell-SetupRqstFDD" },
  { 262, "id-SecondaryCPICH-InformationList-Cell-ReconfRqstFDD" },
  { 263, "id-SecondaryCPICH-InformationList-Cell-SetupRqstFDD" },
  { 264, "id-SecondarySCH-Information-Cell-ReconfRqstFDD" },
  { 265, "id-SecondarySCH-Information-Cell-SetupRqstFDD" },
  { 266, "id-SegmentInformationListIE-SystemInfoUpdate" },
  { 268, "id-SFN" },
  { 138, "id-SignallingBearerRequestIndicator" },
  { 269, "id-ShutdownTimer" },
  { 114, "id-Start-Of-Audit-Sequence-Indicator" },
  { 270, "id-Successful-RL-InformationRespItem-RL-AdditionFailureFDD" },
  { 271, "id-Successful-RL-InformationRespItem-RL-SetupFailureFDD" },
  { 274, "id-SyncCase" },
  { 275, "id-SyncCaseIndicatorItem-Cell-SetupRqstTDD-PSCH" },
  { 276, "id-T-Cell" },
  { 139, "id-TargetCommunicationControlPortID" },
  { 277, "id-TimeSlotConfigurationList-Cell-ReconfRqstTDD" },
  { 278, "id-TimeSlotConfigurationList-Cell-SetupRqstTDD" },
  { 279, "id-TransmissionDiversityApplied" },
  { 508, "id-TypeOfError" },
  { 280, "id-UARFCNforNt" },
  { 281, "id-UARFCNforNd" },
  { 282, "id-UARFCNforNu" },
  { 284, "id-UL-CCTrCH-InformationItem-RL-SetupRqstTDD" },
  { 285, "id-UL-CCTrCH-InformationList-RL-AdditionRqstTDD" },
  { 288, "id-UL-CCTrCH-InformationList-RL-SetupRqstTDD" },
  { 289, "id-UL-DPCH-InformationItem-RL-AdditionRqstTDD" },
  { 291, "id-UL-DPCH-InformationList-RL-SetupRqstTDD" },
  { 293, "id-UL-DPCH-Information-RL-ReconfPrepFDD" },
  { 294, "id-UL-DPCH-Information-RL-ReconfRqstFDD" },
  { 295, "id-UL-DPCH-Information-RL-SetupRqstFDD" },
  { 296, "id-Unsuccessful-RL-InformationRespItem-RL-AdditionFailureFDD" },
  { 297, "id-Unsuccessful-RL-InformationRespItem-RL-SetupFailureFDD" },
  { 300, "id-Unsuccessful-RL-InformationResp-RL-AdditionFailureTDD" },
  { 301, "id-Unsuccessful-RL-InformationResp-RL-SetupFailureTDD" },
  { 302, "id-USCH-Information-Add" },
  { 304, "id-USCH-Information-DeleteList-RL-ReconfPrepTDD" },
  { 306, "id-USCH-Information-ModifyList-RL-ReconfPrepTDD" },
  { 309, "id-USCH-InformationResponse" },
  { 310, "id-USCH-Information" },
  { 141, "id-USCH-RearrangeList-Bearer-RearrangeInd" },
  { 315, "id-Active-Pattern-Sequence-Information" },
  { 316, "id-AICH-ParametersListIE-CTCH-ReconfRqstFDD" },
  { 317, "id-AdjustmentRatio" },
  { 320, "id-Not-Used-320" },
  { 322, "id-Not-Used-322" },
  { 323, "id-FACH-ParametersListIE-CTCH-ReconfRqstFDD" },
  { 324, "id-CauseLevel-PSCH-ReconfFailure" },
  { 325, "id-CauseLevel-RL-AdditionFailureFDD" },
  { 326, "id-CauseLevel-RL-AdditionFailureTDD" },
  { 327, "id-CauseLevel-RL-ReconfFailure" },
  { 328, "id-CauseLevel-RL-SetupFailureFDD" },
  { 329, "id-CauseLevel-RL-SetupFailureTDD" },
  { 330, "id-Not-Used-330" },
  { 332, "id-Not-Used-332" },
  { 333, "id-Closed-Loop-Timing-Adjustment-Mode" },
  { 334, "id-CommonPhysicalChannelType-CTCH-ReconfRqstFDD" },
  { 335, "id-Compressed-Mode-Deactivation-Flag" },
  { 336, "id-Not-Used-336" },
  { 342, "id-Not-Used-342" },
  { 343, "id-Not-Used-343" },
  { 346, "id-DL-CCTrCH-InformationAddList-RL-ReconfPrepTDD" },
  { 347, "id-DL-CCTrCH-InformationDeleteItem-RL-ReconfRqstTDD" },
  { 348, "id-DL-CCTrCH-InformationDeleteList-RL-ReconfPrepTDD" },
  { 349, "id-DL-CCTrCH-InformationDeleteList-RL-ReconfRqstTDD" },
  { 350, "id-DL-CCTrCH-InformationModifyItem-RL-ReconfRqstTDD" },
  { 351, "id-DL-CCTrCH-InformationModifyList-RL-ReconfPrepTDD" },
  { 352, "id-DL-CCTrCH-InformationModifyList-RL-ReconfRqstTDD" },
  { 353, "id-DL-DPCH-InformationAddListIE-RL-ReconfPrepTDD" },
  { 355, "id-DL-DPCH-InformationModify-AddListIE-RL-ReconfPrepTDD" },
  { 356, "id-DL-DPCH-InformationModify-DeleteListIE-RL-ReconfPrepTDD" },
  { 357, "id-DL-DPCH-InformationModify-ModifyListIE-RL-ReconfPrepTDD" },
  { 358, "id-DL-TPC-Pattern01Count" },
  { 450, "id-DPC-Mode" },
  { 359, "id-DPCHConstant" },
  {  94, "id-Unused-ProtocolIE-ID-94" },
  { 110, "id-Unused-ProtocolIE-ID-110" },
  { 111, "id-Unused-ProtocolIE-ID-111" },
  { 362, "id-FACH-ParametersList-CTCH-SetupRsp" },
  { 369, "id-Limited-power-increase-information-Cell-SetupRqstFDD" },
  { 374, "id-PCH-Parameters-CTCH-SetupRsp" },
  { 375, "id-PCH-ParametersItem-CTCH-ReconfRqstFDD" },
  { 376, "id-Not-Used-376" },
  { 380, "id-PICH-ParametersItem-CTCH-ReconfRqstFDD" },
  { 381, "id-PRACHConstant" },
  { 383, "id-PRACH-ParametersListIE-CTCH-ReconfRqstFDD" },
  { 384, "id-PUSCHConstant" },
  { 385, "id-RACH-Parameters-CTCH-SetupRsp" },
  { 443, "id-Unused-ProtocolIE-ID-443" },
  { 393, "id-Synchronisation-Configuration-Cell-ReconfRqst" },
  { 394, "id-Synchronisation-Configuration-Cell-SetupRqst" },
  { 395, "id-Transmission-Gap-Pattern-Sequence-Information" },
  { 396, "id-UL-CCTrCH-InformationAddList-RL-ReconfPrepTDD" },
  { 397, "id-UL-CCTrCH-InformationDeleteItem-RL-ReconfRqstTDD" },
  { 398, "id-UL-CCTrCH-InformationDeleteList-RL-ReconfPrepTDD" },
  { 399, "id-UL-CCTrCH-InformationDeleteList-RL-ReconfRqstTDD" },
  { 400, "id-UL-CCTrCH-InformationModifyItem-RL-ReconfRqstTDD" },
  { 401, "id-UL-CCTrCH-InformationModifyList-RL-ReconfPrepTDD" },
  { 402, "id-UL-CCTrCH-InformationModifyList-RL-ReconfRqstTDD" },
  { 403, "id-UL-DPCH-InformationAddListIE-RL-ReconfPrepTDD" },
  { 405, "id-UL-DPCH-InformationModify-AddListIE-RL-ReconfPrepTDD" },
  { 406, "id-UL-DPCH-InformationModify-DeleteListIE-RL-ReconfPrepTDD" },
  { 407, "id-UL-DPCH-InformationModify-ModifyListIE-RL-ReconfPrepTDD" },
  { 408, "id-Unsuccessful-PDSCHSetItem-PSCH-ReconfFailureTDD" },
  { 409, "id-Unsuccessful-PUSCHSetItem-PSCH-ReconfFailureTDD" },
  { 412, "id-CommunicationContextInfoItem-Reset" },
  { 414, "id-CommunicationControlPortInfoItem-Reset" },
  { 416, "id-ResetIndicator" },
  { 417, "id-Unused-ProtocolIE-ID-417" },
  { 418, "id-Unused-ProtocolIE-ID-418" },
  { 419, "id-Unused-ProtocolIE-ID-419" },
  { 142, "id-Unused-ProtocolIE-ID-142" },
  { 287, "id-TimingAdvanceApplied" },
  {   6, "id-CFNReportingIndicator" },
  {  11, "id-SFNReportingIndicator" },
  {  12, "id-InnerLoopDLPCStatus" },
  { 283, "id-TimeslotISCPInfo" },
  { 167, "id-PICH-ParametersItem-CTCH-SetupRqstTDD" },
  {  20, "id-PRACH-ParametersItem-CTCH-SetupRqstTDD" },
  {  46, "id-CCTrCH-InformationItem-RL-FailureInd" },
  {  47, "id-CCTrCH-InformationItem-RL-RestoreInd" },
  { 420, "id-CauseLevel-SyncAdjustmntFailureTDD" },
  { 421, "id-CellAdjustmentInfo-SyncAdjustmntRqstTDD" },
  { 494, "id-CellAdjustmentInfoItem-SyncAdjustmentRqstTDD" },
  { 482, "id-CellSyncBurstInfoList-CellSyncReconfRqstTDD" },
  { 422, "id-CellSyncBurstTransInit-CellSyncInitiationRqstTDD" },
  { 423, "id-CellSyncBurstMeasureInit-CellSyncInitiationRqstTDD" },
  { 424, "id-CellSyncBurstTransReconfiguration-CellSyncReconfRqstTDD" },
  { 425, "id-CellSyncBurstMeasReconfiguration-CellSyncReconfRqstTDD" },
  { 426, "id-CellSyncBurstTransInfoList-CellSyncReconfRqstTDD" },
  { 427, "id-CellSyncBurstMeasInfoList-CellSyncReconfRqstTDD" },
  { 428, "id-CellSyncBurstTransReconfInfo-CellSyncReconfRqstTDD" },
  { 429, "id-CellSyncInfo-CellSyncReprtTDD" },
  { 430, "id-CSBTransmissionID" },
  { 431, "id-CSBMeasurementID" },
  { 432, "id-IntStdPhCellSyncInfoItem-CellSyncReprtTDD" },
  { 433, "id-NCyclesPerSFNperiod" },
  { 434, "id-NRepetitionsPerCyclePeriod" },
  { 437, "id-SyncFrameNumber" },
  { 438, "id-SynchronisationReportType" },
  { 439, "id-SynchronisationReportCharacteristics" },
  { 440, "id-Unsuccessful-cell-InformationRespItem-SyncAdjustmntFailureTDD" },
  { 119, "id-LateEntranceCellSyncInfoItem-CellSyncReprtTDD" },
  { 435, "id-ReferenceClockAvailability" },
  { 436, "id-ReferenceSFNoffset" },
  { 444, "id-InformationExchangeID" },
  { 445, "id-InformationExchangeObjectType-InfEx-Rqst" },
  { 446, "id-InformationType" },
  { 447, "id-InformationReportCharacteristics" },
  { 448, "id-InformationExchangeObjectType-InfEx-Rsp" },
  { 449, "id-InformationExchangeObjectType-InfEx-Rprt" },
  { 451, "id-IPDLParameter-Information-Cell-ReconfRqstFDD" },
  { 452, "id-IPDLParameter-Information-Cell-SetupRqstFDD" },
  { 453, "id-IPDLParameter-Information-Cell-ReconfRqstTDD" },
  { 454, "id-IPDLParameter-Information-Cell-SetupRqstTDD" },
  {  74, "id-DL-DPCH-LCR-Information-RL-SetupRqstTDD" },
  {  78, "id-DwPCH-LCR-Information" },
  {  90, "id-DwPCH-LCR-InformationList-AuditRsp" },
  {  97, "id-DwPCH-LCR-Information-Cell-SetupRqstTDD" },
  {  99, "id-DwPCH-LCR-Information-Cell-ReconfRqstTDD" },
  { 101, "id-DwPCH-LCR-Information-ResourceStatusInd" },
  { 154, "id-maxFACH-Power-LCR-CTCH-SetupRqstTDD" },
  { 174, "id-maxFACH-Power-LCR-CTCH-ReconfRqstTDD" },
  { 290, "id-FPACH-LCR-Information" },
  { 292, "id-FPACH-LCR-Information-AuditRsp" },
  {  22, "id-FPACH-LCR-InformationList-AuditRsp" },
  { 311, "id-FPACH-LCR-InformationList-ResourceStatusInd" },
  { 312, "id-FPACH-LCR-Parameters-CTCH-SetupRqstTDD" },
  { 314, "id-FPACH-LCR-Parameters-CTCH-ReconfRqstTDD" },
  { 456, "id-PCCPCH-LCR-Information-Cell-SetupRqstTDD" },
  { 457, "id-PCH-Power-LCR-CTCH-SetupRqstTDD" },
  { 458, "id-PCH-Power-LCR-CTCH-ReconfRqstTDD" },
  { 459, "id-PICH-LCR-Parameters-CTCH-SetupRqstTDD" },
  { 461, "id-PRACH-LCR-ParametersList-CTCH-SetupRqstTDD" },
  { 463, "id-RL-InformationResponse-LCR-RL-SetupRspTDD" },
  { 465, "id-Secondary-CCPCH-LCR-parameterList-CTCH-SetupRqstTDD" },
  { 495, "id-TimeSlot" },
  { 466, "id-TimeSlotConfigurationList-LCR-Cell-ReconfRqstTDD" },
  { 467, "id-TimeSlotConfigurationList-LCR-Cell-SetupRqstTDD" },
  { 468, "id-TimeslotISCP-LCR-InfoList-RL-SetupRqstTDD" },
  { 469, "id-TimeSlotLCR-CM-Rqst" },
  { 470, "id-UL-DPCH-LCR-Information-RL-SetupRqstTDD" },
  { 472, "id-DL-DPCH-InformationItem-LCR-RL-AdditionRqstTDD" },
  { 473, "id-UL-DPCH-InformationItem-LCR-RL-AdditionRqstTDD" },
  { 474, "id-TimeslotISCP-InformationList-LCR-RL-AdditionRqstTDD" },
  { 475, "id-DL-DPCH-LCR-InformationAddList-RL-ReconfPrepTDD" },
  { 477, "id-DL-DPCH-LCR-InformationModify-AddList-RL-ReconfPrepTDD" },
  { 479, "id-DL-Timeslot-LCR-InformationModify-ModifyList-RL-ReconfPrepTDD" },
  { 480, "id-TimeslotISCPInfoList-LCR-DL-PC-RqstTDD" },
  { 481, "id-UL-DPCH-LCR-InformationAddListIE-RL-ReconfPrepTDD" },
  { 483, "id-UL-DPCH-LCR-InformationModify-AddList" },
  { 485, "id-UL-TimeslotLCR-Information-RL-ReconfPrepTDD" },
  { 510, "id-UL-SIRTarget" },
  { 486, "id-PDSCH-AddInformation-LCR-PSCH-ReconfRqst" },
  { 487, "id-PDSCH-AddInformation-LCR-AddListIE-PSCH-ReconfRqst" },
  {  26, "id-Unused-ProtocolIE-ID-26" },
  {  27, "id-Unused-ProtocolIE-ID-27" },
  { 488, "id-PDSCH-ModifyInformation-LCR-PSCH-ReconfRqst" },
  { 489, "id-PDSCH-ModifyInformation-LCR-ModifyListIE-PSCH-ReconfRqst" },
  { 490, "id-PUSCH-AddInformation-LCR-PSCH-ReconfRqst" },
  { 491, "id-PUSCH-AddInformation-LCR-AddListIE-PSCH-ReconfRqst" },
  { 492, "id-PUSCH-ModifyInformation-LCR-PSCH-ReconfRqst" },
  { 493, "id-PUSCH-ModifyInformation-LCR-ModifyListIE-PSCH-ReconfRqst" },
  { 496, "id-timeslotInfo-CellSyncInitiationRqstTDD" },
  { 497, "id-SyncReportType-CellSyncReprtTDD" },
  { 498, "id-Power-Local-Cell-Group-InformationItem-AuditRsp" },
  { 499, "id-Power-Local-Cell-Group-InformationItem-ResourceStatusInd" },
  { 500, "id-Power-Local-Cell-Group-InformationItem2-ResourceStatusInd" },
  { 501, "id-Power-Local-Cell-Group-InformationList-AuditRsp" },
  { 502, "id-Power-Local-Cell-Group-InformationList-ResourceStatusInd" },
  { 503, "id-Power-Local-Cell-Group-InformationList2-ResourceStatusInd" },
  { 504, "id-Power-Local-Cell-Group-ID" },
  { 505, "id-PUSCH-Info-DM-Rqst" },
  { 506, "id-PUSCH-Info-DM-Rsp" },
  { 507, "id-PUSCH-Info-DM-Rprt" },
  { 509, "id-InitDL-Power" },
  { 511, "id-cellSyncBurstRepetitionPeriod" },
  { 512, "id-ReportCharacteristicsType-OnModification" },
  { 513, "id-SFNSFNMeasurementValueInformation" },
  { 514, "id-SFNSFNMeasurementThresholdInformation" },
  { 515, "id-TUTRANGPSMeasurementValueInformation" },
  { 516, "id-TUTRANGPSMeasurementThresholdInformation" },
  { 520, "id-Rx-Timing-Deviation-Value-LCR" },
  {  51, "id-RL-InformationResponse-LCR-RL-AdditionRspTDD" },
  {  28, "id-DL-PowerBalancing-Information" },
  {  29, "id-DL-PowerBalancing-ActivationIndicator" },
  {  30, "id-DL-PowerBalancing-UpdatedIndicator" },
  { 517, "id-CCTrCH-Initial-DL-Power-RL-SetupRqstTDD" },
  { 518, "id-CCTrCH-Initial-DL-Power-RL-AdditionRqstTDD" },
  { 519, "id-CCTrCH-Initial-DL-Power-RL-ReconfPrepTDD" },
  {  41, "id-IPDLParameter-Information-LCR-Cell-SetupRqstTDD" },
  {  42, "id-IPDLParameter-Information-LCR-Cell-ReconfRqstTDD" },
  { 522, "id-HS-PDSCH-HS-SCCH-E-AGCH-E-RGCH-E-HICH-MaxPower-PSCH-ReconfRqst" },
  { 523, "id-HS-PDSCH-HS-SCCH-ScramblingCode-PSCH-ReconfRqst" },
  { 524, "id-HS-PDSCH-FDD-Code-Information-PSCH-ReconfRqst" },
  { 525, "id-HS-SCCH-FDD-Code-Information-PSCH-ReconfRqst" },
  { 526, "id-HS-PDSCH-TDD-Information-PSCH-ReconfRqst" },
  { 527, "id-Add-To-HS-SCCH-Resource-Pool-PSCH-ReconfRqst" },
  { 528, "id-Modify-HS-SCCH-Resource-Pool-PSCH-ReconfRqst" },
  { 529, "id-Delete-From-HS-SCCH-Resource-Pool-PSCH-ReconfRqst" },
  { 102, "id-bindingID" },
  { 103, "id-RL-Specific-DCH-Info" },
  { 104, "id-transportlayeraddress" },
  { 231, "id-DelayedActivation" },
  { 232, "id-DelayedActivationList-RL-ActivationCmdFDD" },
  { 233, "id-DelayedActivationInformation-RL-ActivationCmdFDD" },
  { 234, "id-DelayedActivationList-RL-ActivationCmdTDD" },
  { 235, "id-DelayedActivationInformation-RL-ActivationCmdTDD" },
  {  58, "id-neighbouringTDDCellMeasurementInformationLCR" },
  { 543, "id-SYNCDlCodeId-TransInitLCR-CellSyncInitiationRqstTDD" },
  { 544, "id-SYNCDlCodeId-MeasureInitLCR-CellSyncInitiationRqstTDD" },
  { 545, "id-SYNCDlCodeIdTransReconfInfoLCR-CellSyncReconfRqstTDD" },
  { 546, "id-SYNCDlCodeIdMeasReconfigurationLCR-CellSyncReconfRqstTDD" },
  { 547, "id-SYNCDlCodeIdMeasInfoList-CellSyncReconfRqstTDD" },
  { 548, "id-SyncDLCodeIdsMeasInfoList-CellSyncReprtTDD" },
  { 549, "id-SyncDLCodeIdThreInfoLCR" },
  { 550, "id-NSubCyclesPerCyclePeriod-CellSyncReconfRqstTDD" },
  { 551, "id-DwPCH-Power" },
  { 552, "id-AccumulatedClockupdate-CellSyncReprtTDD" },
  { 521, "id-Angle-Of-Arrival-Value-LCR" },
  { 530, "id-HSDSCH-FDD-Information" },
  { 531, "id-HSDSCH-FDD-Information-Response" },
  { 534, "id-HSDSCH-Information-to-Modify" },
  { 535, "id-HSDSCH-RNTI" },
  { 536, "id-HSDSCH-TDD-Information" },
  { 537, "id-HSDSCH-TDD-Information-Response" },
  { 541, "id-HSPDSCH-RL-ID" },
  { 542, "id-PrimCCPCH-RSCP-DL-PC-RqstTDD" },
  {  64, "id-Unused-ProtocolIE-ID-64" },
  {  66, "id-PDSCH-RL-ID" },
  { 553, "id-HSDSCH-RearrangeList-Bearer-RearrangeInd" },
  { 554, "id-UL-Synchronisation-Parameters-LCR" },
  { 555, "id-HSDSCH-FDD-Update-Information" },
  { 556, "id-HSDSCH-TDD-Update-Information" },
  { 558, "id-DL-DPCH-TimeSlotFormat-LCR-ModifyItem-RL-ReconfPrepTDD" },
  { 559, "id-UL-DPCH-TimeSlotFormat-LCR-ModifyItem-RL-ReconfPrepTDD" },
  { 560, "id-TDD-TPC-UplinkStepSize-LCR-RL-SetupRqstTDD" },
  { 561, "id-TDD-TPC-UplinkStepSize-LCR-RL-AdditionRqstTDD" },
  { 562, "id-TDD-TPC-DownlinkStepSize-RL-AdditionRqstTDD" },
  { 563, "id-TDD-TPC-UplinkStepSize-InformationAdd-LCR-RL-ReconfPrepTDD" },
  { 564, "id-TDD-TPC-UplinkStepSize-InformationModify-LCR-RL-ReconfPrepTDD" },
  { 565, "id-TDD-TPC-DownlinkStepSize-InformationModify-RL-ReconfPrepTDD" },
  { 566, "id-TDD-TPC-DownlinkStepSize-InformationAdd-RL-ReconfPrepTDD" },
  { 567, "id-CCTrCH-Maximum-DL-Power-RL-SetupRqstTDD" },
  { 568, "id-CCTrCH-Minimum-DL-Power-RL-SetupRqstTDD" },
  { 569, "id-CCTrCH-Maximum-DL-Power-RL-AdditionRqstTDD" },
  { 570, "id-CCTrCH-Minimum-DL-Power-RL-AdditionRqstTDD" },
  { 571, "id-CCTrCH-Maximum-DL-Power-InformationAdd-RL-ReconfPrepTDD" },
  { 572, "id-CCTrCH-Minimum-DL-Power-InformationAdd-RL-ReconfPrepTDD" },
  { 573, "id-CCTrCH-Maximum-DL-Power-InformationModify-RL-ReconfPrepTDD" },
  { 574, "id-CCTrCH-Minimum-DL-Power-InformationModify-RL-ReconfPrepTDD" },
  { 575, "id-Maximum-DL-Power-Modify-LCR-InformationModify-RL-ReconfPrepTDD" },
  { 576, "id-Minimum-DL-Power-Modify-LCR-InformationModify-RL-ReconfPrepTDD" },
  { 577, "id-DL-DPCH-LCR-InformationModify-ModifyList-RL-ReconfRqstTDD" },
  { 578, "id-CCTrCH-Maximum-DL-Power-InformationModify-RL-ReconfRqstTDD" },
  { 579, "id-CCTrCH-Minimum-DL-Power-InformationModify-RL-ReconfRqstTDD" },
  { 580, "id-Initial-DL-Power-TimeslotLCR-InformationItem" },
  { 581, "id-Maximum-DL-Power-TimeslotLCR-InformationItem" },
  { 582, "id-Minimum-DL-Power-TimeslotLCR-InformationItem" },
  { 583, "id-HS-DSCHProvidedBitRateValueInformation" },
  { 585, "id-HS-DSCHRequiredPowerValueInformation" },
  { 586, "id-HS-DSCHRequiredPowerValue" },
  { 587, "id-TransmittedCarrierPowerOfAllCodesNotUsedForHSTransmission" },
  { 588, "id-HS-SICH-Reception-Quality" },
  { 589, "id-HS-SICH-Reception-Quality-Measurement-Value" },
  { 590, "id-HSSICH-Info-DM-Rprt" },
  { 591, "id-HSSICH-Info-DM-Rqst" },
  { 592, "id-HSSICH-Info-DM-Rsp" },
  { 593, "id-Best-Cell-Portions-Value" },
  { 594, "id-Primary-CPICH-Usage-for-Channel-Estimation" },
  { 595, "id-Secondary-CPICH-Information-Change" },
  { 596, "id-NumberOfReportedCellPortions" },
  { 597, "id-CellPortion-InformationItem-Cell-SetupRqstFDD" },
  { 598, "id-CellPortion-InformationList-Cell-SetupRqstFDD" },
  { 599, "id-TimeslotISCP-LCR-InfoList-RL-ReconfPrepTDD" },
  { 600, "id-Secondary-CPICH-Information" },
  { 601, "id-Received-total-wide-band-power-For-CellPortion" },
  { 602, "id-Unidirectional-DCH-Indicator" },
  { 603, "id-TimingAdjustmentValueLCR" },
  { 604, "id-multipleRL-dl-DPCH-InformationList" },
  { 605, "id-multipleRL-dl-DPCH-InformationModifyList" },
  { 606, "id-multipleRL-ul-DPCH-InformationList" },
  { 607, "id-multipleRL-ul-DPCH-InformationModifyList" },
  { 608, "id-RL-ID" },
  { 609, "id-SAT-Info-Almanac-ExtItem" },
  { 610, "id-HSDPA-Capability" },
  { 611, "id-HSDSCH-Resources-Information-AuditRsp" },
  { 612, "id-HSDSCH-Resources-Information-ResourceStatusInd" },
  { 613, "id-HSDSCH-MACdFlows-to-Add" },
  { 614, "id-HSDSCH-MACdFlows-to-Delete" },
  { 615, "id-HSDSCH-Information-to-Modify-Unsynchronised" },
  { 616, "id-TnlQos" },
  { 617, "id-Received-total-wide-band-power-For-CellPortion-Value" },
  { 618, "id-Transmitted-Carrier-Power-For-CellPortion" },
  { 619, "id-Transmitted-Carrier-Power-For-CellPortion-Value" },
  { 620, "id-TransmittedCarrierPowerOfAllCodesNotUsedForHS-PDSCH-HS-SCCH-E-AGCH-E-RGCHOrE-HICHTransmissionCellPortion" },
  { 621, "id-TransmittedCarrierPowerOfAllCodesNotUsedForHS-PDSCH-HS-SCCH-E-AGCH-E-RGCHOrE-HICHTransmissionCellPortionValue" },
  { 622, "id-UpPTSInterferenceValue" },
  { 623, "id-PrimaryCCPCH-RSCP-Delta" },
  { 624, "id-MeasurementRecoveryBehavior" },
  { 625, "id-MeasurementRecoveryReportingIndicator" },
  { 626, "id-MeasurementRecoverySupportIndicator" },
  { 627, "id-Tstd-indicator" },
  { 628, "id-multiple-RL-Information-RL-ReconfPrepTDD" },
  { 629, "id-multiple-RL-Information-RL-ReconfRqstTDD" },
  { 630, "id-DL-DPCH-Power-Information-RL-ReconfPrepFDD" },
  { 631, "id-F-DPCH-Information-RL-ReconfPrepFDD" },
  { 632, "id-F-DPCH-Information-RL-SetupRqstFDD" },
  { 633, "id-Additional-S-CCPCH-Parameters-CTCH-ReconfRqstTDD" },
  { 634, "id-Additional-S-CCPCH-Parameters-CTCH-SetupRqstTDD" },
  { 635, "id-Additional-S-CCPCH-LCR-Parameters-CTCH-ReconfRqstTDD" },
  { 636, "id-Additional-S-CCPCH-LCR-Parameters-CTCH-SetupRqstTDD" },
  { 637, "id-MICH-CFN" },
  { 638, "id-MICH-Information-AuditRsp" },
  { 639, "id-MICH-Information-ResourceStatusInd" },
  { 640, "id-MICH-Parameters-CTCH-ReconfRqstFDD" },
  { 641, "id-MICH-Parameters-CTCH-ReconfRqstTDD" },
  { 642, "id-MICH-Parameters-CTCH-SetupRqstFDD" },
  { 643, "id-MICH-Parameters-CTCH-SetupRqstTDD" },
  { 644, "id-Modification-Period" },
  { 645, "id-NI-Information-NotifUpdateCmd" },
  { 646, "id-S-CCPCH-InformationListExt-AuditRsp" },
  { 647, "id-S-CCPCH-InformationListExt-ResourceStatusInd" },
  { 648, "id-S-CCPCH-LCR-InformationListExt-AuditRsp" },
  { 649, "id-S-CCPCH-LCR-InformationListExt-ResourceStatusInd" },
  { 650, "id-HARQ-Preamble-Mode" },
  { 651, "id-Initial-DL-DPCH-TimingAdjustment" },
  { 652, "id-Initial-DL-DPCH-TimingAdjustment-Allowed" },
  { 653, "id-DLTransmissionBranchLoadValue" },
  { 654, "id-Power-Local-Cell-Group-choice-CM-Rqst" },
  { 655, "id-Power-Local-Cell-Group-choice-CM-Rsp" },
  { 656, "id-Power-Local-Cell-Group-choice-CM-Rprt" },
  { 657, "id-SynchronisationIndicator" },
  { 658, "id-HSDPA-And-EDCH-CellPortion-Information-PSCH-ReconfRqst" },
  { 659, "id-HSDPA-And-EDCH-CellPortion-InformationList-PSCH-ReconfRqst" },
  { 660, "id-HS-DSCHRequiredPowerValue-For-Cell-Portion" },
  { 661, "id-HS-DSCHRequiredPowerValueInformation-For-CellPortion" },
  { 662, "id-HS-DSCHProvidedBitRateValueInformation-For-CellPortion" },
  { 663, "id-E-AGCH-And-E-RGCH-E-HICH-FDD-Scrambling-Code" },
  { 664, "id-E-AGCH-FDD-Code-Information" },
  { 665, "id-E-DCH-Capability" },
  { 666, "id-E-DCH-FDD-DL-Control-Channel-Information" },
  { 667, "id-E-DCH-FDD-Information" },
  { 668, "id-E-DCH-FDD-Information-Response" },
  { 669, "id-E-DCH-FDD-Information-to-Modify" },
  { 670, "id-E-DCH-MACdFlows-to-Add" },
  { 671, "id-E-DCH-MACdFlows-to-Delete" },
  { 672, "id-E-DCH-Resources-Information-AuditRsp" },
  { 673, "id-E-DCH-Resources-Information-ResourceStatusInd" },
  { 674, "id-E-DCH-RL-Indication" },
  { 675, "id-E-DCH-RL-Set-ID" },
  { 676, "id-E-DPCH-Information-RL-ReconfPrepFDD" },
  { 677, "id-E-DPCH-Information-RL-SetupRqstFDD" },
  { 678, "id-E-RGCH-E-HICH-FDD-Code-Information" },
  { 679, "id-Serving-E-DCH-RL-ID" },
  { 680, "id-UL-DPDCH-Indicator-For-E-DCH-Operation" },
  { 681, "id-FDD-S-CCPCH-FrameOffset-CTCH-SetupRqstFDD" },
  { 682, "id-E-DPCH-Information-RL-ReconfRqstFDD" },
  { 683, "id-Maximum-Target-ReceivedTotalWideBandPower" },
  { 684, "id-E-DCHProvidedBitRateValueInformation" },
  { 685, "id-HARQ-Preamble-Mode-Activation-Indicator" },
  { 686, "id-RL-Specific-E-DCH-Info" },
  { 687, "id-E-DCH-CapacityConsumptionLaw" },
  { 688, "id-multiple-DedicatedMeasurementValueList-TDD-DM-Rsp" },
  { 689, "id-multiple-DedicatedMeasurementValueList-LCR-TDD-DM-Rsp" },
  { 690, "id-E-DCH-RearrangeList-Bearer-RearrangeInd" },
  { 691, "id-HSDPA-And-EDCH-CellPortion-InformationListIE-PSCH-ReconfRqst" },
  { 692, "id-multipleRL-dl-CCTrCH-InformationModifyList-RL-ReconfRqstTDD" },
  { 693, "id-Target-NonServing-EDCH-To-Total-EDCH-Power-Ratio" },
  { 694, "id-CellPortion-InformationItem-Cell-ReconfRqstFDD" },
  { 695, "id-CellPortion-InformationList-Cell-ReconfRqstFDD" },
  { 696, "id-multiple-PUSCH-InfoList-DM-Rsp" },
  { 697, "id-multiple-PUSCH-InfoList-DM-Rprt" },
  { 698, "id-Reference-ReceivedTotalWideBandPower" },
  { 0, NULL }
};


static int
dissect_nbap_ProtocolIE_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 65535U, &ProtocolIE_ID, FALSE);

  return offset;
}
static int dissect_iE_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_ID(tvb, offset, actx, tree, hf_nbap_iE_ID);
}
static int dissect_id(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_ID(tvb, offset, actx, tree, hf_nbap_id);
}



static int
dissect_nbap_ProtocolIEValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
#line 64 "nbap.cnf"

	offset = dissect_nbap_ProtocolIEValueValue(tvb, offset, actx, tree);



  return offset;
}
static int dissect_value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIEValue(tvb, offset, actx, tree, hf_nbap_value);
}


static const per_sequence_t ProtocolIE_Field_sequence[] = {
  { &hf_nbap_id             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_ID },
  { &hf_nbap_criticality    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_Criticality },
  { &hf_nbap_value          , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIEValue },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_ProtocolIE_Field(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_ProtocolIE_Field, ProtocolIE_Field_sequence);

  return offset;
}
static int dissect_ProtocolIE_Container_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Field(tvb, offset, actx, tree, hf_nbap_ProtocolIE_Container_item);
}


static const per_sequence_t ProtocolIE_Container_sequence_of[1] = {
  { &hf_nbap_ProtocolIE_Container_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Field },
};

static int
dissect_nbap_ProtocolIE_Container(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_ProtocolIE_Container, ProtocolIE_Container_sequence_of,
                                                  0, 65535);

  return offset;
}
static int dissect_protocolIEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Container(tvb, offset, actx, tree, hf_nbap_protocolIEs);
}
static int dissect_ProtocolIE_ContainerList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Container(tvb, offset, actx, tree, hf_nbap_ProtocolIE_ContainerList_item);
}



static int
dissect_nbap_Extension(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
#line 36 "nbap.cnf"
	/* FIX ME */
	guint length;

	offset = dissect_per_length_determinant(tvb, offset, actx, tree, hf_nbap_IE_length, &length);
	offset = offset + length;
	BYTE_ALIGN_OFFSET(offset)



  return offset;
}
static int dissect_extensionValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Extension(tvb, offset, actx, tree, hf_nbap_extensionValue);
}


static const per_sequence_t ProtocolExtensionField_sequence[] = {
  { &hf_nbap_id             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_ID },
  { &hf_nbap_criticality    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_Criticality },
  { &hf_nbap_extensionValue , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_Extension },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_ProtocolExtensionField(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_ProtocolExtensionField, ProtocolExtensionField_sequence);

  return offset;
}
static int dissect_ProtocolExtensionContainer_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolExtensionField(tvb, offset, actx, tree, hf_nbap_ProtocolExtensionContainer_item);
}


static const per_sequence_t ProtocolExtensionContainer_sequence_of[1] = {
  { &hf_nbap_ProtocolExtensionContainer_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolExtensionField },
};

static int
dissect_nbap_ProtocolExtensionContainer(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_ProtocolExtensionContainer, ProtocolExtensionContainer_sequence_of,
                                                  1, 65535);

  return offset;
}
static int dissect_protocolExtensions(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolExtensionContainer(tvb, offset, actx, tree, hf_nbap_protocolExtensions);
}
static int dissect_iE_Extensions(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolExtensionContainer(tvb, offset, actx, tree, hf_nbap_iE_Extensions);
}
static int dissect_ie_Extensions(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolExtensionContainer(tvb, offset, actx, tree, hf_nbap_ie_Extensions);
}


static const per_sequence_t AuditRequest_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_AuditRequest(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_AuditRequest, AuditRequest_sequence);

  return offset;
}
static int dissect_id_audit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_AuditRequest(tvb, offset, actx, tree, hf_nbap_id_audit);
}


static const per_sequence_t AuditRequiredIndication_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_AuditRequiredIndication(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_AuditRequiredIndication, AuditRequiredIndication_sequence);

  return offset;
}
static int dissect_id_auditRequired(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_AuditRequiredIndication(tvb, offset, actx, tree, hf_nbap_id_auditRequired);
}


static const per_sequence_t BlockResourceRequest_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_BlockResourceRequest(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_BlockResourceRequest, BlockResourceRequest_sequence);

  return offset;
}
static int dissect_id_blockResource(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BlockResourceRequest(tvb, offset, actx, tree, hf_nbap_id_blockResource);
}


static const per_sequence_t CellDeletionRequest_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CellDeletionRequest(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CellDeletionRequest, CellDeletionRequest_sequence);

  return offset;
}
static int dissect_id_cellDeletion(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellDeletionRequest(tvb, offset, actx, tree, hf_nbap_id_cellDeletion);
}


static const per_sequence_t CellReconfigurationRequestFDD_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CellReconfigurationRequestFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CellReconfigurationRequestFDD, CellReconfigurationRequestFDD_sequence);

  return offset;
}
static int dissect_id_cellReconfiguration(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellReconfigurationRequestFDD(tvb, offset, actx, tree, hf_nbap_id_cellReconfiguration);
}


static const per_sequence_t CellReconfigurationRequestTDD_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CellReconfigurationRequestTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CellReconfigurationRequestTDD, CellReconfigurationRequestTDD_sequence);

  return offset;
}
static int dissect_id_cellReconfiguration_tdd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellReconfigurationRequestTDD(tvb, offset, actx, tree, hf_nbap_id_cellReconfiguration_tdd);
}


static const per_sequence_t CellSetupRequestFDD_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CellSetupRequestFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CellSetupRequestFDD, CellSetupRequestFDD_sequence);

  return offset;
}
static int dissect_id_cellSetup(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSetupRequestFDD(tvb, offset, actx, tree, hf_nbap_id_cellSetup);
}


static const per_sequence_t CellSetupRequestTDD_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CellSetupRequestTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CellSetupRequestTDD, CellSetupRequestTDD_sequence);

  return offset;
}
static int dissect_id_cellSetup_tdd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSetupRequestTDD(tvb, offset, actx, tree, hf_nbap_id_cellSetup_tdd);
}


static const per_sequence_t CommonMeasurementFailureIndication_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CommonMeasurementFailureIndication(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CommonMeasurementFailureIndication, CommonMeasurementFailureIndication_sequence);

  return offset;
}
static int dissect_id_commonMeasurementFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommonMeasurementFailureIndication(tvb, offset, actx, tree, hf_nbap_id_commonMeasurementFailure);
}


static const per_sequence_t CommonMeasurementInitiationRequest_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CommonMeasurementInitiationRequest(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CommonMeasurementInitiationRequest, CommonMeasurementInitiationRequest_sequence);

  return offset;
}
static int dissect_id_commonMeasurementInitiation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommonMeasurementInitiationRequest(tvb, offset, actx, tree, hf_nbap_id_commonMeasurementInitiation);
}


static const per_sequence_t CommonMeasurementReport_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CommonMeasurementReport(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CommonMeasurementReport, CommonMeasurementReport_sequence);

  return offset;
}
static int dissect_id_commonMeasurementReport(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommonMeasurementReport(tvb, offset, actx, tree, hf_nbap_id_commonMeasurementReport);
}


static const per_sequence_t CommonMeasurementTerminationRequest_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CommonMeasurementTerminationRequest(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CommonMeasurementTerminationRequest, CommonMeasurementTerminationRequest_sequence);

  return offset;
}
static int dissect_id_commonMeasurementTermination(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommonMeasurementTerminationRequest(tvb, offset, actx, tree, hf_nbap_id_commonMeasurementTermination);
}


static const per_sequence_t CommonTransportChannelDeletionRequest_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CommonTransportChannelDeletionRequest(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CommonTransportChannelDeletionRequest, CommonTransportChannelDeletionRequest_sequence);

  return offset;
}
static int dissect_id_commonTransportChannelDelete(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommonTransportChannelDeletionRequest(tvb, offset, actx, tree, hf_nbap_id_commonTransportChannelDelete);
}


static const per_sequence_t CommonTransportChannelReconfigurationRequestFDD_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CommonTransportChannelReconfigurationRequestFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CommonTransportChannelReconfigurationRequestFDD, CommonTransportChannelReconfigurationRequestFDD_sequence);

  return offset;
}
static int dissect_id_commonTransportChannelReconfigure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommonTransportChannelReconfigurationRequestFDD(tvb, offset, actx, tree, hf_nbap_id_commonTransportChannelReconfigure);
}


static const per_sequence_t CommonTransportChannelReconfigurationRequestTDD_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CommonTransportChannelReconfigurationRequestTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CommonTransportChannelReconfigurationRequestTDD, CommonTransportChannelReconfigurationRequestTDD_sequence);

  return offset;
}
static int dissect_id_commonTransportChannelReconfigure_tdd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommonTransportChannelReconfigurationRequestTDD(tvb, offset, actx, tree, hf_nbap_id_commonTransportChannelReconfigure_tdd);
}


static const per_sequence_t CommonTransportChannelSetupRequestFDD_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CommonTransportChannelSetupRequestFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CommonTransportChannelSetupRequestFDD, CommonTransportChannelSetupRequestFDD_sequence);

  return offset;
}
static int dissect_id_commonTransportChannelSetup(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommonTransportChannelSetupRequestFDD(tvb, offset, actx, tree, hf_nbap_id_commonTransportChannelSetup);
}


static const per_sequence_t CommonTransportChannelSetupRequestTDD_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CommonTransportChannelSetupRequestTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CommonTransportChannelSetupRequestTDD, CommonTransportChannelSetupRequestTDD_sequence);

  return offset;
}
static int dissect_id_commonTransportChannelSetup_tdd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommonTransportChannelSetupRequestTDD(tvb, offset, actx, tree, hf_nbap_id_commonTransportChannelSetup_tdd);
}


static const per_sequence_t ResetRequest_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_ResetRequest(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_ResetRequest, ResetRequest_sequence);

  return offset;
}
static int dissect_id_reset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ResetRequest(tvb, offset, actx, tree, hf_nbap_id_reset);
}


static const per_sequence_t CompressedModeCommand_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CompressedModeCommand(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CompressedModeCommand, CompressedModeCommand_sequence);

  return offset;
}
static int dissect_id_compressedModeCommand(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CompressedModeCommand(tvb, offset, actx, tree, hf_nbap_id_compressedModeCommand);
}


static const per_sequence_t DedicatedMeasurementFailureIndication_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DedicatedMeasurementFailureIndication(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DedicatedMeasurementFailureIndication, DedicatedMeasurementFailureIndication_sequence);

  return offset;
}
static int dissect_id_dedicatedMeasurementFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DedicatedMeasurementFailureIndication(tvb, offset, actx, tree, hf_nbap_id_dedicatedMeasurementFailure);
}


static const per_sequence_t DedicatedMeasurementInitiationRequest_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DedicatedMeasurementInitiationRequest(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DedicatedMeasurementInitiationRequest, DedicatedMeasurementInitiationRequest_sequence);

  return offset;
}
static int dissect_id_dedicatedMeasurementInitiation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DedicatedMeasurementInitiationRequest(tvb, offset, actx, tree, hf_nbap_id_dedicatedMeasurementInitiation);
}


static const per_sequence_t DedicatedMeasurementReport_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DedicatedMeasurementReport(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DedicatedMeasurementReport, DedicatedMeasurementReport_sequence);

  return offset;
}
static int dissect_id_dedicatedMeasurementReport(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DedicatedMeasurementReport(tvb, offset, actx, tree, hf_nbap_id_dedicatedMeasurementReport);
}


static const per_sequence_t DedicatedMeasurementTerminationRequest_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DedicatedMeasurementTerminationRequest(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DedicatedMeasurementTerminationRequest, DedicatedMeasurementTerminationRequest_sequence);

  return offset;
}
static int dissect_id_dedicatedMeasurementTermination(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DedicatedMeasurementTerminationRequest(tvb, offset, actx, tree, hf_nbap_id_dedicatedMeasurementTermination);
}


static const per_sequence_t DL_PowerControlRequest_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_PowerControlRequest(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_PowerControlRequest, DL_PowerControlRequest_sequence);

  return offset;
}
static int dissect_id_downlinkPowerControl(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_PowerControlRequest(tvb, offset, actx, tree, hf_nbap_id_downlinkPowerControl);
}


static const per_sequence_t ErrorIndication_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_ErrorIndication(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_ErrorIndication, ErrorIndication_sequence);

  return offset;
}
static int dissect_id_errorIndicationForDedicated(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ErrorIndication(tvb, offset, actx, tree, hf_nbap_id_errorIndicationForDedicated);
}
static int dissect_id_errorIndicationForCommon(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ErrorIndication(tvb, offset, actx, tree, hf_nbap_id_errorIndicationForCommon);
}



static int
dissect_nbap_INTEGER_0_65535(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 65535U, NULL, FALSE);

  return offset;
}
static int dissect_dl_Cost(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_INTEGER_0_65535(tvb, offset, actx, tree, hf_nbap_dl_Cost);
}
static int dissect_ul_Cost(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_INTEGER_0_65535(tvb, offset, actx, tree, hf_nbap_ul_Cost);
}
static int dissect_dl_Cost_1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_INTEGER_0_65535(tvb, offset, actx, tree, hf_nbap_dl_Cost_1);
}
static int dissect_dl_Cost_2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_INTEGER_0_65535(tvb, offset, actx, tree, hf_nbap_dl_Cost_2);
}
static int dissect_ul_Cost_1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_INTEGER_0_65535(tvb, offset, actx, tree, hf_nbap_ul_Cost_1);
}
static int dissect_ul_Cost_2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_INTEGER_0_65535(tvb, offset, actx, tree, hf_nbap_ul_Cost_2);
}
static int dissect_ctfc16bit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_INTEGER_0_65535(tvb, offset, actx, tree, hf_nbap_ctfc16bit);
}
static int dissect_local(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_INTEGER_0_65535(tvb, offset, actx, tree, hf_nbap_local);
}



static int
dissect_nbap_OBJECT_IDENTIFIER(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_object_identifier(tvb, offset, actx, tree, hf_index, NULL);

  return offset;
}
static int dissect_global(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_OBJECT_IDENTIFIER(tvb, offset, actx, tree, hf_nbap_global);
}


static const value_string nbap_PrivateIE_ID_vals[] = {
  {   0, "local" },
  {   1, "global" },
  { 0, NULL }
};

static const per_choice_t PrivateIE_ID_choice[] = {
  {   0, &hf_nbap_local          , ASN1_NO_EXTENSIONS     , dissect_nbap_INTEGER_0_65535 },
  {   1, &hf_nbap_global         , ASN1_NO_EXTENSIONS     , dissect_nbap_OBJECT_IDENTIFIER },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_PrivateIE_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_PrivateIE_ID, PrivateIE_ID_choice,
                                 NULL);

  return offset;
}
static int dissect_privateIEid(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PrivateIE_ID(tvb, offset, actx, tree, hf_nbap_privateIEid);
}



static int
dissect_nbap_PrivateIEvalue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
#line 85 "nbap.cnf"
	/* FIX ME */
	guint length;

	offset = dissect_per_length_determinant(tvb, offset, actx, tree, hf_nbap_IE_length, &length);
	offset = offset + length;
	BYTE_ALIGN_OFFSET(offset)


  return offset;
}
static int dissect_privateIEvalue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PrivateIEvalue(tvb, offset, actx, tree, hf_nbap_privateIEvalue);
}


static const per_sequence_t PrivateIE_Field_sequence[] = {
  { &hf_nbap_privateIEid    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_PrivateIE_ID },
  { &hf_nbap_criticality    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_Criticality },
  { &hf_nbap_privateIEvalue , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_PrivateIEvalue },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PrivateIE_Field(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PrivateIE_Field, PrivateIE_Field_sequence);

  return offset;
}
static int dissect_PrivateIE_Container_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PrivateIE_Field(tvb, offset, actx, tree, hf_nbap_PrivateIE_Container_item);
}


static const per_sequence_t PrivateIE_Container_sequence_of[1] = {
  { &hf_nbap_PrivateIE_Container_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_PrivateIE_Field },
};

static int
dissect_nbap_PrivateIE_Container(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_PrivateIE_Container, PrivateIE_Container_sequence_of,
                                                  1, 65535);

  return offset;
}
static int dissect_privateIEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PrivateIE_Container(tvb, offset, actx, tree, hf_nbap_privateIEs);
}


static const per_sequence_t PrivateMessage_sequence[] = {
  { &hf_nbap_privateIEs     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PrivateIE_Container },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PrivateMessage(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PrivateMessage, PrivateMessage_sequence);

  return offset;
}
static int dissect_id_privateMessageForDedicated(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PrivateMessage(tvb, offset, actx, tree, hf_nbap_id_privateMessageForDedicated);
}
static int dissect_id_privateMessageForCommon(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PrivateMessage(tvb, offset, actx, tree, hf_nbap_id_privateMessageForCommon);
}


static const per_sequence_t RadioLinkAdditionRequestFDD_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RadioLinkAdditionRequestFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RadioLinkAdditionRequestFDD, RadioLinkAdditionRequestFDD_sequence);

  return offset;
}
static int dissect_id_radioLinkAddition(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RadioLinkAdditionRequestFDD(tvb, offset, actx, tree, hf_nbap_id_radioLinkAddition);
}


static const per_sequence_t RadioLinkDeletionRequest_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RadioLinkDeletionRequest(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RadioLinkDeletionRequest, RadioLinkDeletionRequest_sequence);

  return offset;
}
static int dissect_id_radioLinkDeletion(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RadioLinkDeletionRequest(tvb, offset, actx, tree, hf_nbap_id_radioLinkDeletion);
}


static const per_sequence_t RadioLinkFailureIndication_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RadioLinkFailureIndication(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RadioLinkFailureIndication, RadioLinkFailureIndication_sequence);

  return offset;
}
static int dissect_id_radioLinkFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RadioLinkFailureIndication(tvb, offset, actx, tree, hf_nbap_id_radioLinkFailure);
}


static const per_sequence_t RadioLinkRestoreIndication_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RadioLinkRestoreIndication(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RadioLinkRestoreIndication, RadioLinkRestoreIndication_sequence);

  return offset;
}
static int dissect_id_radioLinkRestoration(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RadioLinkRestoreIndication(tvb, offset, actx, tree, hf_nbap_id_radioLinkRestoration);
}


static const per_sequence_t RadioLinkSetupRequestFDD_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RadioLinkSetupRequestFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RadioLinkSetupRequestFDD, RadioLinkSetupRequestFDD_sequence);

  return offset;
}
static int dissect_id_radioLinkSetup(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RadioLinkSetupRequestFDD(tvb, offset, actx, tree, hf_nbap_id_radioLinkSetup);
}


static const per_sequence_t RadioLinkSetupRequestTDD_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RadioLinkSetupRequestTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RadioLinkSetupRequestTDD, RadioLinkSetupRequestTDD_sequence);

  return offset;
}
static int dissect_id_radioLinkSetup_tdd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RadioLinkSetupRequestTDD(tvb, offset, actx, tree, hf_nbap_id_radioLinkSetup_tdd);
}


static const per_sequence_t ResourceStatusIndication_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_ResourceStatusIndication(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_ResourceStatusIndication, ResourceStatusIndication_sequence);

  return offset;
}
static int dissect_id_resourceStatusIndication(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ResourceStatusIndication(tvb, offset, actx, tree, hf_nbap_id_resourceStatusIndication);
}


static const per_sequence_t RadioLinkReconfigurationCancel_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RadioLinkReconfigurationCancel(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RadioLinkReconfigurationCancel, RadioLinkReconfigurationCancel_sequence);

  return offset;
}
static int dissect_id_synchronisedRadioLinkReconfigurationCancellation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RadioLinkReconfigurationCancel(tvb, offset, actx, tree, hf_nbap_id_synchronisedRadioLinkReconfigurationCancellation);
}


static const per_sequence_t RadioLinkReconfigurationCommit_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RadioLinkReconfigurationCommit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RadioLinkReconfigurationCommit, RadioLinkReconfigurationCommit_sequence);

  return offset;
}
static int dissect_id_synchronisedRadioLinkReconfigurationCommit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RadioLinkReconfigurationCommit(tvb, offset, actx, tree, hf_nbap_id_synchronisedRadioLinkReconfigurationCommit);
}


static const per_sequence_t RadioLinkReconfigurationPrepareFDD_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RadioLinkReconfigurationPrepareFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RadioLinkReconfigurationPrepareFDD, RadioLinkReconfigurationPrepareFDD_sequence);

  return offset;
}
static int dissect_id_synchronisedRadioLinkReconfigurationPreparation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RadioLinkReconfigurationPrepareFDD(tvb, offset, actx, tree, hf_nbap_id_synchronisedRadioLinkReconfigurationPreparation);
}


static const per_sequence_t RadioLinkReconfigurationPrepareTDD_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RadioLinkReconfigurationPrepareTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RadioLinkReconfigurationPrepareTDD, RadioLinkReconfigurationPrepareTDD_sequence);

  return offset;
}
static int dissect_id_synchronisedRadioLinkReconfigurationPreparation_tdd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RadioLinkReconfigurationPrepareTDD(tvb, offset, actx, tree, hf_nbap_id_synchronisedRadioLinkReconfigurationPreparation_tdd);
}


static const per_sequence_t SystemInformationUpdateRequest_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_SystemInformationUpdateRequest(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_SystemInformationUpdateRequest, SystemInformationUpdateRequest_sequence);

  return offset;
}
static int dissect_id_systemInformationUpdate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SystemInformationUpdateRequest(tvb, offset, actx, tree, hf_nbap_id_systemInformationUpdate);
}


static const per_sequence_t UnblockResourceIndication_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_UnblockResourceIndication(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_UnblockResourceIndication, UnblockResourceIndication_sequence);

  return offset;
}
static int dissect_id_unblockResource(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UnblockResourceIndication(tvb, offset, actx, tree, hf_nbap_id_unblockResource);
}


static const per_sequence_t RadioLinkReconfigurationRequestFDD_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RadioLinkReconfigurationRequestFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RadioLinkReconfigurationRequestFDD, RadioLinkReconfigurationRequestFDD_sequence);

  return offset;
}
static int dissect_id_unSynchronisedRadioLinkReconfiguration(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RadioLinkReconfigurationRequestFDD(tvb, offset, actx, tree, hf_nbap_id_unSynchronisedRadioLinkReconfiguration);
}


static const per_sequence_t RadioLinkReconfigurationRequestTDD_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RadioLinkReconfigurationRequestTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RadioLinkReconfigurationRequestTDD, RadioLinkReconfigurationRequestTDD_sequence);

  return offset;
}
static int dissect_id_unSynchronisedRadioLinkReconfiguration_tdd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RadioLinkReconfigurationRequestTDD(tvb, offset, actx, tree, hf_nbap_id_unSynchronisedRadioLinkReconfiguration_tdd);
}


static const per_sequence_t DL_PowerTimeslotControlRequest_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_PowerTimeslotControlRequest(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_PowerTimeslotControlRequest, DL_PowerTimeslotControlRequest_sequence);

  return offset;
}
static int dissect_id_downlinkPowerTimeslotControl(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_PowerTimeslotControlRequest(tvb, offset, actx, tree, hf_nbap_id_downlinkPowerTimeslotControl);
}


static const per_sequence_t PhysicalSharedChannelReconfigurationRequestFDD_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PhysicalSharedChannelReconfigurationRequestFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PhysicalSharedChannelReconfigurationRequestFDD, PhysicalSharedChannelReconfigurationRequestFDD_sequence);

  return offset;
}
static int dissect_id_physicalSharedChannelReconfiguration(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PhysicalSharedChannelReconfigurationRequestFDD(tvb, offset, actx, tree, hf_nbap_id_physicalSharedChannelReconfiguration);
}


static const per_sequence_t PhysicalSharedChannelReconfigurationRequestTDD_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PhysicalSharedChannelReconfigurationRequestTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PhysicalSharedChannelReconfigurationRequestTDD, PhysicalSharedChannelReconfigurationRequestTDD_sequence);

  return offset;
}
static int dissect_id_physicalSharedChannelReconfiguration_tdd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PhysicalSharedChannelReconfigurationRequestTDD(tvb, offset, actx, tree, hf_nbap_id_physicalSharedChannelReconfiguration_tdd);
}


static const per_sequence_t RadioLinkPreemptionRequiredIndication_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RadioLinkPreemptionRequiredIndication(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RadioLinkPreemptionRequiredIndication, RadioLinkPreemptionRequiredIndication_sequence);

  return offset;
}
static int dissect_id_radioLinkPreemption(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RadioLinkPreemptionRequiredIndication(tvb, offset, actx, tree, hf_nbap_id_radioLinkPreemption);
}


static const per_sequence_t InformationExchangeFailureIndication_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_InformationExchangeFailureIndication(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_InformationExchangeFailureIndication, InformationExchangeFailureIndication_sequence);

  return offset;
}
static int dissect_id_informationExchangeFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_InformationExchangeFailureIndication(tvb, offset, actx, tree, hf_nbap_id_informationExchangeFailure);
}


static const per_sequence_t InformationExchangeInitiationRequest_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_InformationExchangeInitiationRequest(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_InformationExchangeInitiationRequest, InformationExchangeInitiationRequest_sequence);

  return offset;
}
static int dissect_id_informationExchangeInitiation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_InformationExchangeInitiationRequest(tvb, offset, actx, tree, hf_nbap_id_informationExchangeInitiation);
}


static const per_sequence_t InformationExchangeTerminationRequest_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_InformationExchangeTerminationRequest(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_InformationExchangeTerminationRequest, InformationExchangeTerminationRequest_sequence);

  return offset;
}
static int dissect_id_informationExchangeTermination(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_InformationExchangeTerminationRequest(tvb, offset, actx, tree, hf_nbap_id_informationExchangeTermination);
}


static const per_sequence_t InformationReport_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_InformationReport(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_InformationReport, InformationReport_sequence);

  return offset;
}
static int dissect_id_informationReporting(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_InformationReport(tvb, offset, actx, tree, hf_nbap_id_informationReporting);
}


static const per_sequence_t CellSynchronisationAdjustmentRequestTDD_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CellSynchronisationAdjustmentRequestTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CellSynchronisationAdjustmentRequestTDD, CellSynchronisationAdjustmentRequestTDD_sequence);

  return offset;
}
static int dissect_id_cellSynchronisationAdjustment(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSynchronisationAdjustmentRequestTDD(tvb, offset, actx, tree, hf_nbap_id_cellSynchronisationAdjustment);
}


static const per_sequence_t CellSynchronisationInitiationRequestTDD_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CellSynchronisationInitiationRequestTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CellSynchronisationInitiationRequestTDD, CellSynchronisationInitiationRequestTDD_sequence);

  return offset;
}
static int dissect_id_cellSynchronisationInitiation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSynchronisationInitiationRequestTDD(tvb, offset, actx, tree, hf_nbap_id_cellSynchronisationInitiation);
}


static const per_sequence_t CellSynchronisationReconfigurationRequestTDD_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CellSynchronisationReconfigurationRequestTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CellSynchronisationReconfigurationRequestTDD, CellSynchronisationReconfigurationRequestTDD_sequence);

  return offset;
}
static int dissect_id_cellSynchronisationReconfiguration(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSynchronisationReconfigurationRequestTDD(tvb, offset, actx, tree, hf_nbap_id_cellSynchronisationReconfiguration);
}


static const per_sequence_t CellSynchronisationReportTDD_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CellSynchronisationReportTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CellSynchronisationReportTDD, CellSynchronisationReportTDD_sequence);

  return offset;
}
static int dissect_id_cellSynchronisationReporting(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSynchronisationReportTDD(tvb, offset, actx, tree, hf_nbap_id_cellSynchronisationReporting);
}


static const per_sequence_t CellSynchronisationTerminationRequestTDD_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CellSynchronisationTerminationRequestTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CellSynchronisationTerminationRequestTDD, CellSynchronisationTerminationRequestTDD_sequence);

  return offset;
}
static int dissect_id_cellSynchronisationTermination(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSynchronisationTerminationRequestTDD(tvb, offset, actx, tree, hf_nbap_id_cellSynchronisationTermination);
}


static const per_sequence_t CellSynchronisationFailureIndicationTDD_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CellSynchronisationFailureIndicationTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CellSynchronisationFailureIndicationTDD, CellSynchronisationFailureIndicationTDD_sequence);

  return offset;
}
static int dissect_id_cellSynchronisationFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSynchronisationFailureIndicationTDD(tvb, offset, actx, tree, hf_nbap_id_cellSynchronisationFailure);
}


static const per_sequence_t BearerRearrangementIndication_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_BearerRearrangementIndication(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_BearerRearrangementIndication, BearerRearrangementIndication_sequence);

  return offset;
}
static int dissect_id_BearerRearrangement(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BearerRearrangementIndication(tvb, offset, actx, tree, hf_nbap_id_BearerRearrangement);
}


static const per_sequence_t RadioLinkActivationCommandFDD_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RadioLinkActivationCommandFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RadioLinkActivationCommandFDD, RadioLinkActivationCommandFDD_sequence);

  return offset;
}
static int dissect_id_radioLinkActivation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RadioLinkActivationCommandFDD(tvb, offset, actx, tree, hf_nbap_id_radioLinkActivation);
}


static const per_sequence_t RadioLinkActivationCommandTDD_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RadioLinkActivationCommandTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RadioLinkActivationCommandTDD, RadioLinkActivationCommandTDD_sequence);

  return offset;
}
static int dissect_id_radioLinkActivation_tdd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RadioLinkActivationCommandTDD(tvb, offset, actx, tree, hf_nbap_id_radioLinkActivation_tdd);
}


static const per_sequence_t RadioLinkParameterUpdateIndicationFDD_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RadioLinkParameterUpdateIndicationFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RadioLinkParameterUpdateIndicationFDD, RadioLinkParameterUpdateIndicationFDD_sequence);

  return offset;
}
static int dissect_id_radioLinkParameterUpdate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RadioLinkParameterUpdateIndicationFDD(tvb, offset, actx, tree, hf_nbap_id_radioLinkParameterUpdate);
}


static const per_sequence_t RadioLinkParameterUpdateIndicationTDD_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RadioLinkParameterUpdateIndicationTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RadioLinkParameterUpdateIndicationTDD, RadioLinkParameterUpdateIndicationTDD_sequence);

  return offset;
}
static int dissect_id_radioLinkParameterUpdate_tdd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RadioLinkParameterUpdateIndicationTDD(tvb, offset, actx, tree, hf_nbap_id_radioLinkParameterUpdate_tdd);
}


static const per_sequence_t MBMSNotificationUpdateCommand_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_MBMSNotificationUpdateCommand(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_MBMSNotificationUpdateCommand, MBMSNotificationUpdateCommand_sequence);

  return offset;
}
static int dissect_id_mBMSNotificationUpdate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MBMSNotificationUpdateCommand(tvb, offset, actx, tree, hf_nbap_id_mBMSNotificationUpdate);
}


static const value_string nbap_DummyInitiatingmessages_vals[] = {
  {   0, "id-audit" },
  {   1, "id-auditRequired" },
  {   2, "id-blockResource" },
  {   3, "id-cellDeletion" },
  {   4, "id-cellReconfiguration" },
  {   5, "id-cellReconfiguration-tdd" },
  {   6, "id-cellSetup" },
  {   7, "id-cellSetup-tdd" },
  {   8, "id-commonMeasurementFailure" },
  {   9, "id-commonMeasurementInitiation" },
  {  10, "id-commonMeasurementReport" },
  {  11, "id-commonMeasurementTermination" },
  {  12, "id-commonTransportChannelDelete" },
  {  13, "id-commonTransportChannelReconfigure" },
  {  14, "id-commonTransportChannelReconfigure-tdd" },
  {  15, "id-commonTransportChannelSetup" },
  {  16, "id-commonTransportChannelSetup-tdd" },
  {  17, "id-reset" },
  {  18, "id-compressedModeCommand" },
  {  19, "id-dedicatedMeasurementFailure" },
  {  20, "id-dedicatedMeasurementInitiation" },
  {  21, "id-dedicatedMeasurementReport" },
  {  22, "id-dedicatedMeasurementTermination" },
  {  23, "id-downlinkPowerControl" },
  {  24, "id-errorIndicationForDedicated" },
  {  25, "id-privateMessageForDedicated" },
  {  26, "id-radioLinkAddition" },
  {  27, "id-radioLinkDeletion" },
  {  28, "id-radioLinkFailure" },
  {  29, "id-radioLinkRestoration" },
  {  30, "id-radioLinkSetup" },
  {  31, "id-radioLinkSetup-tdd" },
  {  32, "id-resourceStatusIndication" },
  {  33, "id-synchronisedRadioLinkReconfigurationCancellation" },
  {  34, "id-synchronisedRadioLinkReconfigurationCommit" },
  {  35, "id-synchronisedRadioLinkReconfigurationPreparation" },
  {  36, "id-synchronisedRadioLinkReconfigurationPreparation-tdd" },
  {  37, "id-systemInformationUpdate" },
  {  38, "id-unblockResource" },
  {  39, "id-unSynchronisedRadioLinkReconfiguration" },
  {  40, "id-unSynchronisedRadioLinkReconfiguration-tdd" },
  {  41, "id-downlinkPowerTimeslotControl" },
  {  42, "id-errorIndicationForCommon" },
  {  43, "id-privateMessageForCommon" },
  {  44, "id-physicalSharedChannelReconfiguration" },
  {  45, "id-physicalSharedChannelReconfiguration-tdd" },
  {  46, "id-radioLinkPreemption" },
  {  47, "id-informationExchangeFailure" },
  {  48, "id-informationExchangeInitiation" },
  {  49, "id-informationExchangeTermination" },
  {  50, "id-informationReporting" },
  {  51, "id-cellSynchronisationAdjustment" },
  {  52, "id-cellSynchronisationInitiation" },
  {  53, "id-cellSynchronisationReconfiguration" },
  {  54, "id-cellSynchronisationReporting" },
  {  55, "id-cellSynchronisationTermination" },
  {  56, "id-cellSynchronisationFailure" },
  {  57, "id-BearerRearrangement" },
  {  58, "id-radioLinkActivation" },
  {  59, "id-radioLinkActivation-tdd" },
  {  60, "id-radioLinkParameterUpdate" },
  {  61, "id-radioLinkParameterUpdate-tdd" },
  {  62, "id-mBMSNotificationUpdate" },
  { 0, NULL }
};

static const per_choice_t DummyInitiatingmessages_choice[] = {
  {   0, &hf_nbap_id_audit       , ASN1_EXTENSION_ROOT    , dissect_nbap_AuditRequest },
  {   1, &hf_nbap_id_auditRequired, ASN1_EXTENSION_ROOT    , dissect_nbap_AuditRequiredIndication },
  {   2, &hf_nbap_id_blockResource, ASN1_EXTENSION_ROOT    , dissect_nbap_BlockResourceRequest },
  {   3, &hf_nbap_id_cellDeletion, ASN1_EXTENSION_ROOT    , dissect_nbap_CellDeletionRequest },
  {   4, &hf_nbap_id_cellReconfiguration, ASN1_EXTENSION_ROOT    , dissect_nbap_CellReconfigurationRequestFDD },
  {   5, &hf_nbap_id_cellReconfiguration_tdd, ASN1_EXTENSION_ROOT    , dissect_nbap_CellReconfigurationRequestTDD },
  {   6, &hf_nbap_id_cellSetup   , ASN1_EXTENSION_ROOT    , dissect_nbap_CellSetupRequestFDD },
  {   7, &hf_nbap_id_cellSetup_tdd, ASN1_EXTENSION_ROOT    , dissect_nbap_CellSetupRequestTDD },
  {   8, &hf_nbap_id_commonMeasurementFailure, ASN1_EXTENSION_ROOT    , dissect_nbap_CommonMeasurementFailureIndication },
  {   9, &hf_nbap_id_commonMeasurementInitiation, ASN1_EXTENSION_ROOT    , dissect_nbap_CommonMeasurementInitiationRequest },
  {  10, &hf_nbap_id_commonMeasurementReport, ASN1_EXTENSION_ROOT    , dissect_nbap_CommonMeasurementReport },
  {  11, &hf_nbap_id_commonMeasurementTermination, ASN1_EXTENSION_ROOT    , dissect_nbap_CommonMeasurementTerminationRequest },
  {  12, &hf_nbap_id_commonTransportChannelDelete, ASN1_EXTENSION_ROOT    , dissect_nbap_CommonTransportChannelDeletionRequest },
  {  13, &hf_nbap_id_commonTransportChannelReconfigure, ASN1_EXTENSION_ROOT    , dissect_nbap_CommonTransportChannelReconfigurationRequestFDD },
  {  14, &hf_nbap_id_commonTransportChannelReconfigure_tdd, ASN1_EXTENSION_ROOT    , dissect_nbap_CommonTransportChannelReconfigurationRequestTDD },
  {  15, &hf_nbap_id_commonTransportChannelSetup, ASN1_EXTENSION_ROOT    , dissect_nbap_CommonTransportChannelSetupRequestFDD },
  {  16, &hf_nbap_id_commonTransportChannelSetup_tdd, ASN1_EXTENSION_ROOT    , dissect_nbap_CommonTransportChannelSetupRequestTDD },
  {  17, &hf_nbap_id_reset       , ASN1_EXTENSION_ROOT    , dissect_nbap_ResetRequest },
  {  18, &hf_nbap_id_compressedModeCommand, ASN1_EXTENSION_ROOT    , dissect_nbap_CompressedModeCommand },
  {  19, &hf_nbap_id_dedicatedMeasurementFailure, ASN1_EXTENSION_ROOT    , dissect_nbap_DedicatedMeasurementFailureIndication },
  {  20, &hf_nbap_id_dedicatedMeasurementInitiation, ASN1_EXTENSION_ROOT    , dissect_nbap_DedicatedMeasurementInitiationRequest },
  {  21, &hf_nbap_id_dedicatedMeasurementReport, ASN1_EXTENSION_ROOT    , dissect_nbap_DedicatedMeasurementReport },
  {  22, &hf_nbap_id_dedicatedMeasurementTermination, ASN1_EXTENSION_ROOT    , dissect_nbap_DedicatedMeasurementTerminationRequest },
  {  23, &hf_nbap_id_downlinkPowerControl, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_PowerControlRequest },
  {  24, &hf_nbap_id_errorIndicationForDedicated, ASN1_EXTENSION_ROOT    , dissect_nbap_ErrorIndication },
  {  25, &hf_nbap_id_privateMessageForDedicated, ASN1_EXTENSION_ROOT    , dissect_nbap_PrivateMessage },
  {  26, &hf_nbap_id_radioLinkAddition, ASN1_EXTENSION_ROOT    , dissect_nbap_RadioLinkAdditionRequestFDD },
  {  27, &hf_nbap_id_radioLinkDeletion, ASN1_EXTENSION_ROOT    , dissect_nbap_RadioLinkDeletionRequest },
  {  28, &hf_nbap_id_radioLinkFailure, ASN1_EXTENSION_ROOT    , dissect_nbap_RadioLinkFailureIndication },
  {  29, &hf_nbap_id_radioLinkRestoration, ASN1_EXTENSION_ROOT    , dissect_nbap_RadioLinkRestoreIndication },
  {  30, &hf_nbap_id_radioLinkSetup, ASN1_EXTENSION_ROOT    , dissect_nbap_RadioLinkSetupRequestFDD },
  {  31, &hf_nbap_id_radioLinkSetup_tdd, ASN1_EXTENSION_ROOT    , dissect_nbap_RadioLinkSetupRequestTDD },
  {  32, &hf_nbap_id_resourceStatusIndication, ASN1_EXTENSION_ROOT    , dissect_nbap_ResourceStatusIndication },
  {  33, &hf_nbap_id_synchronisedRadioLinkReconfigurationCancellation, ASN1_EXTENSION_ROOT    , dissect_nbap_RadioLinkReconfigurationCancel },
  {  34, &hf_nbap_id_synchronisedRadioLinkReconfigurationCommit, ASN1_EXTENSION_ROOT    , dissect_nbap_RadioLinkReconfigurationCommit },
  {  35, &hf_nbap_id_synchronisedRadioLinkReconfigurationPreparation, ASN1_EXTENSION_ROOT    , dissect_nbap_RadioLinkReconfigurationPrepareFDD },
  {  36, &hf_nbap_id_synchronisedRadioLinkReconfigurationPreparation_tdd, ASN1_EXTENSION_ROOT    , dissect_nbap_RadioLinkReconfigurationPrepareTDD },
  {  37, &hf_nbap_id_systemInformationUpdate, ASN1_EXTENSION_ROOT    , dissect_nbap_SystemInformationUpdateRequest },
  {  38, &hf_nbap_id_unblockResource, ASN1_EXTENSION_ROOT    , dissect_nbap_UnblockResourceIndication },
  {  39, &hf_nbap_id_unSynchronisedRadioLinkReconfiguration, ASN1_EXTENSION_ROOT    , dissect_nbap_RadioLinkReconfigurationRequestFDD },
  {  40, &hf_nbap_id_unSynchronisedRadioLinkReconfiguration_tdd, ASN1_EXTENSION_ROOT    , dissect_nbap_RadioLinkReconfigurationRequestTDD },
  {  41, &hf_nbap_id_downlinkPowerTimeslotControl, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_PowerTimeslotControlRequest },
  {  42, &hf_nbap_id_errorIndicationForCommon, ASN1_EXTENSION_ROOT    , dissect_nbap_ErrorIndication },
  {  43, &hf_nbap_id_privateMessageForCommon, ASN1_EXTENSION_ROOT    , dissect_nbap_PrivateMessage },
  {  44, &hf_nbap_id_physicalSharedChannelReconfiguration, ASN1_EXTENSION_ROOT    , dissect_nbap_PhysicalSharedChannelReconfigurationRequestFDD },
  {  45, &hf_nbap_id_physicalSharedChannelReconfiguration_tdd, ASN1_EXTENSION_ROOT    , dissect_nbap_PhysicalSharedChannelReconfigurationRequestTDD },
  {  46, &hf_nbap_id_radioLinkPreemption, ASN1_EXTENSION_ROOT    , dissect_nbap_RadioLinkPreemptionRequiredIndication },
  {  47, &hf_nbap_id_informationExchangeFailure, ASN1_EXTENSION_ROOT    , dissect_nbap_InformationExchangeFailureIndication },
  {  48, &hf_nbap_id_informationExchangeInitiation, ASN1_EXTENSION_ROOT    , dissect_nbap_InformationExchangeInitiationRequest },
  {  49, &hf_nbap_id_informationExchangeTermination, ASN1_EXTENSION_ROOT    , dissect_nbap_InformationExchangeTerminationRequest },
  {  50, &hf_nbap_id_informationReporting, ASN1_EXTENSION_ROOT    , dissect_nbap_InformationReport },
  {  51, &hf_nbap_id_cellSynchronisationAdjustment, ASN1_EXTENSION_ROOT    , dissect_nbap_CellSynchronisationAdjustmentRequestTDD },
  {  52, &hf_nbap_id_cellSynchronisationInitiation, ASN1_EXTENSION_ROOT    , dissect_nbap_CellSynchronisationInitiationRequestTDD },
  {  53, &hf_nbap_id_cellSynchronisationReconfiguration, ASN1_EXTENSION_ROOT    , dissect_nbap_CellSynchronisationReconfigurationRequestTDD },
  {  54, &hf_nbap_id_cellSynchronisationReporting, ASN1_EXTENSION_ROOT    , dissect_nbap_CellSynchronisationReportTDD },
  {  55, &hf_nbap_id_cellSynchronisationTermination, ASN1_EXTENSION_ROOT    , dissect_nbap_CellSynchronisationTerminationRequestTDD },
  {  56, &hf_nbap_id_cellSynchronisationFailure, ASN1_EXTENSION_ROOT    , dissect_nbap_CellSynchronisationFailureIndicationTDD },
  {  57, &hf_nbap_id_BearerRearrangement, ASN1_EXTENSION_ROOT    , dissect_nbap_BearerRearrangementIndication },
  {  58, &hf_nbap_id_radioLinkActivation, ASN1_EXTENSION_ROOT    , dissect_nbap_RadioLinkActivationCommandFDD },
  {  59, &hf_nbap_id_radioLinkActivation_tdd, ASN1_EXTENSION_ROOT    , dissect_nbap_RadioLinkActivationCommandTDD },
  {  60, &hf_nbap_id_radioLinkParameterUpdate, ASN1_EXTENSION_ROOT    , dissect_nbap_RadioLinkParameterUpdateIndicationFDD },
  {  61, &hf_nbap_id_radioLinkParameterUpdate_tdd, ASN1_EXTENSION_ROOT    , dissect_nbap_RadioLinkParameterUpdateIndicationTDD },
  {  62, &hf_nbap_id_mBMSNotificationUpdate, ASN1_EXTENSION_ROOT    , dissect_nbap_MBMSNotificationUpdateCommand },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_DummyInitiatingmessages(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_DummyInitiatingmessages, DummyInitiatingmessages_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t AuditResponse_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_AuditResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_AuditResponse, AuditResponse_sequence);

  return offset;
}
static int dissect_id_audit1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_AuditResponse(tvb, offset, actx, tree, hf_nbap_id_audit1);
}


static const per_sequence_t BlockResourceResponse_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_BlockResourceResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_BlockResourceResponse, BlockResourceResponse_sequence);

  return offset;
}
static int dissect_id_blockResource1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BlockResourceResponse(tvb, offset, actx, tree, hf_nbap_id_blockResource1);
}


static const per_sequence_t CellDeletionResponse_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CellDeletionResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CellDeletionResponse, CellDeletionResponse_sequence);

  return offset;
}
static int dissect_id_cellDeletion1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellDeletionResponse(tvb, offset, actx, tree, hf_nbap_id_cellDeletion1);
}


static const per_sequence_t CellReconfigurationResponse_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CellReconfigurationResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CellReconfigurationResponse, CellReconfigurationResponse_sequence);

  return offset;
}
static int dissect_id_cellReconfiguration1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellReconfigurationResponse(tvb, offset, actx, tree, hf_nbap_id_cellReconfiguration1);
}


static const per_sequence_t CellSetupResponse_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CellSetupResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CellSetupResponse, CellSetupResponse_sequence);

  return offset;
}
static int dissect_id_cellSetup1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSetupResponse(tvb, offset, actx, tree, hf_nbap_id_cellSetup1);
}


static const per_sequence_t CommonMeasurementInitiationFailure_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CommonMeasurementInitiationFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CommonMeasurementInitiationFailure, CommonMeasurementInitiationFailure_sequence);

  return offset;
}
static int dissect_id_commonMeasurementInitiation1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommonMeasurementInitiationFailure(tvb, offset, actx, tree, hf_nbap_id_commonMeasurementInitiation1);
}


static const per_sequence_t CommonTransportChannelDeletionResponse_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CommonTransportChannelDeletionResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CommonTransportChannelDeletionResponse, CommonTransportChannelDeletionResponse_sequence);

  return offset;
}
static int dissect_id_commonTransportChannelDelete1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommonTransportChannelDeletionResponse(tvb, offset, actx, tree, hf_nbap_id_commonTransportChannelDelete1);
}


static const per_sequence_t CommonTransportChannelReconfigurationResponse_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CommonTransportChannelReconfigurationResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CommonTransportChannelReconfigurationResponse, CommonTransportChannelReconfigurationResponse_sequence);

  return offset;
}
static int dissect_id_commonTransportChannelReconfigure1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommonTransportChannelReconfigurationResponse(tvb, offset, actx, tree, hf_nbap_id_commonTransportChannelReconfigure1);
}


static const per_sequence_t CommonTransportChannelSetupResponse_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CommonTransportChannelSetupResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CommonTransportChannelSetupResponse, CommonTransportChannelSetupResponse_sequence);

  return offset;
}
static int dissect_id_commonTransportChannelSetup1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommonTransportChannelSetupResponse(tvb, offset, actx, tree, hf_nbap_id_commonTransportChannelSetup1);
}


static const per_sequence_t ResetResponse_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_ResetResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_ResetResponse, ResetResponse_sequence);

  return offset;
}
static int dissect_id_reset1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ResetResponse(tvb, offset, actx, tree, hf_nbap_id_reset1);
}


static const per_sequence_t DedicatedMeasurementInitiationResponse_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DedicatedMeasurementInitiationResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DedicatedMeasurementInitiationResponse, DedicatedMeasurementInitiationResponse_sequence);

  return offset;
}
static int dissect_id_dedicatedMeasurementInitiation1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DedicatedMeasurementInitiationResponse(tvb, offset, actx, tree, hf_nbap_id_dedicatedMeasurementInitiation1);
}


static const per_sequence_t RadioLinkAdditionResponseFDD_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RadioLinkAdditionResponseFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RadioLinkAdditionResponseFDD, RadioLinkAdditionResponseFDD_sequence);

  return offset;
}
static int dissect_id_radioLinkAddition1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RadioLinkAdditionResponseFDD(tvb, offset, actx, tree, hf_nbap_id_radioLinkAddition1);
}


static const per_sequence_t RadioLinkAdditionResponseTDD_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RadioLinkAdditionResponseTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RadioLinkAdditionResponseTDD, RadioLinkAdditionResponseTDD_sequence);

  return offset;
}
static int dissect_id_radioLinkAddition_tdd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RadioLinkAdditionResponseTDD(tvb, offset, actx, tree, hf_nbap_id_radioLinkAddition_tdd);
}


static const per_sequence_t RadioLinkDeletionResponse_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RadioLinkDeletionResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RadioLinkDeletionResponse, RadioLinkDeletionResponse_sequence);

  return offset;
}
static int dissect_id_radioLinkDeletion1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RadioLinkDeletionResponse(tvb, offset, actx, tree, hf_nbap_id_radioLinkDeletion1);
}


static const per_sequence_t RadioLinkSetupResponseFDD_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RadioLinkSetupResponseFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RadioLinkSetupResponseFDD, RadioLinkSetupResponseFDD_sequence);

  return offset;
}
static int dissect_id_radioLinkSetup1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RadioLinkSetupResponseFDD(tvb, offset, actx, tree, hf_nbap_id_radioLinkSetup1);
}


static const per_sequence_t RadioLinkSetupResponseTDD_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RadioLinkSetupResponseTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RadioLinkSetupResponseTDD, RadioLinkSetupResponseTDD_sequence);

  return offset;
}
static int dissect_id_radioLinkSetup_tdd1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RadioLinkSetupResponseTDD(tvb, offset, actx, tree, hf_nbap_id_radioLinkSetup_tdd1);
}


static const per_sequence_t RadioLinkReconfigurationReady_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RadioLinkReconfigurationReady(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RadioLinkReconfigurationReady, RadioLinkReconfigurationReady_sequence);

  return offset;
}
static int dissect_id_synchronisedRadioLinkReconfigurationPreparation1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RadioLinkReconfigurationReady(tvb, offset, actx, tree, hf_nbap_id_synchronisedRadioLinkReconfigurationPreparation1);
}


static const per_sequence_t SystemInformationUpdateResponse_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_SystemInformationUpdateResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_SystemInformationUpdateResponse, SystemInformationUpdateResponse_sequence);

  return offset;
}
static int dissect_id_systemInformationUpdate1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SystemInformationUpdateResponse(tvb, offset, actx, tree, hf_nbap_id_systemInformationUpdate1);
}


static const per_sequence_t RadioLinkReconfigurationResponse_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RadioLinkReconfigurationResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RadioLinkReconfigurationResponse, RadioLinkReconfigurationResponse_sequence);

  return offset;
}
static int dissect_id_unSynchronisedRadioLinkReconfiguration1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RadioLinkReconfigurationResponse(tvb, offset, actx, tree, hf_nbap_id_unSynchronisedRadioLinkReconfiguration1);
}


static const per_sequence_t PhysicalSharedChannelReconfigurationResponse_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PhysicalSharedChannelReconfigurationResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PhysicalSharedChannelReconfigurationResponse, PhysicalSharedChannelReconfigurationResponse_sequence);

  return offset;
}
static int dissect_id_physicalSharedChannelReconfiguration1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PhysicalSharedChannelReconfigurationResponse(tvb, offset, actx, tree, hf_nbap_id_physicalSharedChannelReconfiguration1);
}


static const per_sequence_t InformationExchangeInitiationResponse_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_InformationExchangeInitiationResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_InformationExchangeInitiationResponse, InformationExchangeInitiationResponse_sequence);

  return offset;
}
static int dissect_id_informationExchangeInitiation1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_InformationExchangeInitiationResponse(tvb, offset, actx, tree, hf_nbap_id_informationExchangeInitiation1);
}


static const per_sequence_t CellSynchronisationAdjustmentResponseTDD_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CellSynchronisationAdjustmentResponseTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CellSynchronisationAdjustmentResponseTDD, CellSynchronisationAdjustmentResponseTDD_sequence);

  return offset;
}
static int dissect_id_cellSynchronisationAdjustment1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSynchronisationAdjustmentResponseTDD(tvb, offset, actx, tree, hf_nbap_id_cellSynchronisationAdjustment1);
}


static const per_sequence_t CellSynchronisationInitiationResponseTDD_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CellSynchronisationInitiationResponseTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CellSynchronisationInitiationResponseTDD, CellSynchronisationInitiationResponseTDD_sequence);

  return offset;
}
static int dissect_id_cellSynchronisationInitiation1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSynchronisationInitiationResponseTDD(tvb, offset, actx, tree, hf_nbap_id_cellSynchronisationInitiation1);
}


static const per_sequence_t CellSynchronisationReconfigurationResponseTDD_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CellSynchronisationReconfigurationResponseTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CellSynchronisationReconfigurationResponseTDD, CellSynchronisationReconfigurationResponseTDD_sequence);

  return offset;
}
static int dissect_id_cellSynchronisationReconfiguration1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSynchronisationReconfigurationResponseTDD(tvb, offset, actx, tree, hf_nbap_id_cellSynchronisationReconfiguration1);
}


static const value_string nbap_DummySuccessfullOutcomemessages_vals[] = {
  {   0, "id-audit" },
  {   1, "id-blockResource" },
  {   2, "id-cellDeletion" },
  {   3, "id-cellReconfiguration" },
  {   4, "id-cellSetup" },
  {   5, "id-commonMeasurementInitiation" },
  {   6, "id-commonTransportChannelDelete" },
  {   7, "id-commonTransportChannelReconfigure" },
  {   8, "id-commonTransportChannelSetup" },
  {   9, "id-reset" },
  {  10, "id-dedicatedMeasurementInitiation" },
  {  11, "id-radioLinkAddition" },
  {  12, "id-radioLinkAddition-tdd" },
  {  13, "id-radioLinkDeletion" },
  {  14, "id-radioLinkSetup" },
  {  15, "id-radioLinkSetup-tdd" },
  {  16, "id-synchronisedRadioLinkReconfigurationPreparation" },
  {  17, "id-systemInformationUpdate" },
  {  18, "id-unSynchronisedRadioLinkReconfiguration" },
  {  19, "id-physicalSharedChannelReconfiguration" },
  {  20, "id-informationExchangeInitiation" },
  {  21, "id-cellSynchronisationAdjustment" },
  {  22, "id-cellSynchronisationInitiation" },
  {  23, "id-cellSynchronisationReconfiguration" },
  { 0, NULL }
};

static const per_choice_t DummySuccessfullOutcomemessages_choice[] = {
  {   0, &hf_nbap_id_audit1      , ASN1_EXTENSION_ROOT    , dissect_nbap_AuditResponse },
  {   1, &hf_nbap_id_blockResource1, ASN1_EXTENSION_ROOT    , dissect_nbap_BlockResourceResponse },
  {   2, &hf_nbap_id_cellDeletion1, ASN1_EXTENSION_ROOT    , dissect_nbap_CellDeletionResponse },
  {   3, &hf_nbap_id_cellReconfiguration1, ASN1_EXTENSION_ROOT    , dissect_nbap_CellReconfigurationResponse },
  {   4, &hf_nbap_id_cellSetup1  , ASN1_EXTENSION_ROOT    , dissect_nbap_CellSetupResponse },
  {   5, &hf_nbap_id_commonMeasurementInitiation1, ASN1_EXTENSION_ROOT    , dissect_nbap_CommonMeasurementInitiationFailure },
  {   6, &hf_nbap_id_commonTransportChannelDelete1, ASN1_EXTENSION_ROOT    , dissect_nbap_CommonTransportChannelDeletionResponse },
  {   7, &hf_nbap_id_commonTransportChannelReconfigure1, ASN1_EXTENSION_ROOT    , dissect_nbap_CommonTransportChannelReconfigurationResponse },
  {   8, &hf_nbap_id_commonTransportChannelSetup1, ASN1_EXTENSION_ROOT    , dissect_nbap_CommonTransportChannelSetupResponse },
  {   9, &hf_nbap_id_reset1      , ASN1_EXTENSION_ROOT    , dissect_nbap_ResetResponse },
  {  10, &hf_nbap_id_dedicatedMeasurementInitiation1, ASN1_EXTENSION_ROOT    , dissect_nbap_DedicatedMeasurementInitiationResponse },
  {  11, &hf_nbap_id_radioLinkAddition1, ASN1_EXTENSION_ROOT    , dissect_nbap_RadioLinkAdditionResponseFDD },
  {  12, &hf_nbap_id_radioLinkAddition_tdd, ASN1_EXTENSION_ROOT    , dissect_nbap_RadioLinkAdditionResponseTDD },
  {  13, &hf_nbap_id_radioLinkDeletion1, ASN1_EXTENSION_ROOT    , dissect_nbap_RadioLinkDeletionResponse },
  {  14, &hf_nbap_id_radioLinkSetup1, ASN1_EXTENSION_ROOT    , dissect_nbap_RadioLinkSetupResponseFDD },
  {  15, &hf_nbap_id_radioLinkSetup_tdd1, ASN1_EXTENSION_ROOT    , dissect_nbap_RadioLinkSetupResponseTDD },
  {  16, &hf_nbap_id_synchronisedRadioLinkReconfigurationPreparation1, ASN1_EXTENSION_ROOT    , dissect_nbap_RadioLinkReconfigurationReady },
  {  17, &hf_nbap_id_systemInformationUpdate1, ASN1_EXTENSION_ROOT    , dissect_nbap_SystemInformationUpdateResponse },
  {  18, &hf_nbap_id_unSynchronisedRadioLinkReconfiguration1, ASN1_EXTENSION_ROOT    , dissect_nbap_RadioLinkReconfigurationResponse },
  {  19, &hf_nbap_id_physicalSharedChannelReconfiguration1, ASN1_EXTENSION_ROOT    , dissect_nbap_PhysicalSharedChannelReconfigurationResponse },
  {  20, &hf_nbap_id_informationExchangeInitiation1, ASN1_EXTENSION_ROOT    , dissect_nbap_InformationExchangeInitiationResponse },
  {  21, &hf_nbap_id_cellSynchronisationAdjustment1, ASN1_EXTENSION_ROOT    , dissect_nbap_CellSynchronisationAdjustmentResponseTDD },
  {  22, &hf_nbap_id_cellSynchronisationInitiation1, ASN1_EXTENSION_ROOT    , dissect_nbap_CellSynchronisationInitiationResponseTDD },
  {  23, &hf_nbap_id_cellSynchronisationReconfiguration1, ASN1_EXTENSION_ROOT    , dissect_nbap_CellSynchronisationReconfigurationResponseTDD },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_DummySuccessfullOutcomemessages(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_DummySuccessfullOutcomemessages, DummySuccessfullOutcomemessages_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t AuditFailure_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_AuditFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_AuditFailure, AuditFailure_sequence);

  return offset;
}
static int dissect_id_audit2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_AuditFailure(tvb, offset, actx, tree, hf_nbap_id_audit2);
}


static const per_sequence_t BlockResourceFailure_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_BlockResourceFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_BlockResourceFailure, BlockResourceFailure_sequence);

  return offset;
}
static int dissect_id_blockResource2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BlockResourceFailure(tvb, offset, actx, tree, hf_nbap_id_blockResource2);
}


static const per_sequence_t CellReconfigurationFailure_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CellReconfigurationFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CellReconfigurationFailure, CellReconfigurationFailure_sequence);

  return offset;
}
static int dissect_id_cellReconfiguration2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellReconfigurationFailure(tvb, offset, actx, tree, hf_nbap_id_cellReconfiguration2);
}


static const per_sequence_t CellSetupFailure_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CellSetupFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CellSetupFailure, CellSetupFailure_sequence);

  return offset;
}
static int dissect_id_cellSetup2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSetupFailure(tvb, offset, actx, tree, hf_nbap_id_cellSetup2);
}


static const per_sequence_t CommonMeasurementInitiationResponse_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CommonMeasurementInitiationResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CommonMeasurementInitiationResponse, CommonMeasurementInitiationResponse_sequence);

  return offset;
}
static int dissect_id_commonMeasurementInitiation2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommonMeasurementInitiationResponse(tvb, offset, actx, tree, hf_nbap_id_commonMeasurementInitiation2);
}


static const per_sequence_t CommonTransportChannelReconfigurationFailure_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CommonTransportChannelReconfigurationFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CommonTransportChannelReconfigurationFailure, CommonTransportChannelReconfigurationFailure_sequence);

  return offset;
}
static int dissect_id_commonTransportChannelReconfigure2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommonTransportChannelReconfigurationFailure(tvb, offset, actx, tree, hf_nbap_id_commonTransportChannelReconfigure2);
}


static const per_sequence_t CommonTransportChannelSetupFailure_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CommonTransportChannelSetupFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CommonTransportChannelSetupFailure, CommonTransportChannelSetupFailure_sequence);

  return offset;
}
static int dissect_id_commonTransportChannelSetup2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommonTransportChannelSetupFailure(tvb, offset, actx, tree, hf_nbap_id_commonTransportChannelSetup2);
}


static const per_sequence_t DedicatedMeasurementInitiationFailure_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DedicatedMeasurementInitiationFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DedicatedMeasurementInitiationFailure, DedicatedMeasurementInitiationFailure_sequence);

  return offset;
}
static int dissect_id_dedicatedMeasurementInitiation2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DedicatedMeasurementInitiationFailure(tvb, offset, actx, tree, hf_nbap_id_dedicatedMeasurementInitiation2);
}


static const per_sequence_t RadioLinkAdditionFailureFDD_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RadioLinkAdditionFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RadioLinkAdditionFailureFDD, RadioLinkAdditionFailureFDD_sequence);

  return offset;
}
static int dissect_id_radioLinkAddition2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RadioLinkAdditionFailureFDD(tvb, offset, actx, tree, hf_nbap_id_radioLinkAddition2);
}


static const per_sequence_t RadioLinkAdditionFailureTDD_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RadioLinkAdditionFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RadioLinkAdditionFailureTDD, RadioLinkAdditionFailureTDD_sequence);

  return offset;
}
static int dissect_id_radioLinkAddition_tdd1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RadioLinkAdditionFailureTDD(tvb, offset, actx, tree, hf_nbap_id_radioLinkAddition_tdd1);
}


static const per_sequence_t RadioLinkSetupFailureFDD_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RadioLinkSetupFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RadioLinkSetupFailureFDD, RadioLinkSetupFailureFDD_sequence);

  return offset;
}
static int dissect_id_radioLinkSetup2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RadioLinkSetupFailureFDD(tvb, offset, actx, tree, hf_nbap_id_radioLinkSetup2);
}


static const per_sequence_t RadioLinkSetupFailureTDD_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RadioLinkSetupFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RadioLinkSetupFailureTDD, RadioLinkSetupFailureTDD_sequence);

  return offset;
}
static int dissect_id_radioLinkSetup_tdd2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RadioLinkSetupFailureTDD(tvb, offset, actx, tree, hf_nbap_id_radioLinkSetup_tdd2);
}


static const per_sequence_t RadioLinkReconfigurationFailure_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RadioLinkReconfigurationFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RadioLinkReconfigurationFailure, RadioLinkReconfigurationFailure_sequence);

  return offset;
}
static int dissect_id_synchronisedRadioLinkReconfigurationPreparation2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RadioLinkReconfigurationFailure(tvb, offset, actx, tree, hf_nbap_id_synchronisedRadioLinkReconfigurationPreparation2);
}
static int dissect_id_unSynchronisedRadioLinkReconfiguration2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RadioLinkReconfigurationFailure(tvb, offset, actx, tree, hf_nbap_id_unSynchronisedRadioLinkReconfiguration2);
}


static const per_sequence_t SystemInformationUpdateFailure_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_SystemInformationUpdateFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_SystemInformationUpdateFailure, SystemInformationUpdateFailure_sequence);

  return offset;
}
static int dissect_id_systemInformationUpdate2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SystemInformationUpdateFailure(tvb, offset, actx, tree, hf_nbap_id_systemInformationUpdate2);
}


static const per_sequence_t PhysicalSharedChannelReconfigurationFailure_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PhysicalSharedChannelReconfigurationFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PhysicalSharedChannelReconfigurationFailure, PhysicalSharedChannelReconfigurationFailure_sequence);

  return offset;
}
static int dissect_id_physicalSharedChannelReconfiguration2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PhysicalSharedChannelReconfigurationFailure(tvb, offset, actx, tree, hf_nbap_id_physicalSharedChannelReconfiguration2);
}


static const per_sequence_t InformationExchangeInitiationFailure_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_InformationExchangeInitiationFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_InformationExchangeInitiationFailure, InformationExchangeInitiationFailure_sequence);

  return offset;
}
static int dissect_id_informationExchangeInitiation2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_InformationExchangeInitiationFailure(tvb, offset, actx, tree, hf_nbap_id_informationExchangeInitiation2);
}


static const per_sequence_t CellSynchronisationAdjustmentFailureTDD_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CellSynchronisationAdjustmentFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CellSynchronisationAdjustmentFailureTDD, CellSynchronisationAdjustmentFailureTDD_sequence);

  return offset;
}
static int dissect_id_cellSynchronisationAdjustment2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSynchronisationAdjustmentFailureTDD(tvb, offset, actx, tree, hf_nbap_id_cellSynchronisationAdjustment2);
}


static const per_sequence_t CellSynchronisationInitiationFailureTDD_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CellSynchronisationInitiationFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CellSynchronisationInitiationFailureTDD, CellSynchronisationInitiationFailureTDD_sequence);

  return offset;
}
static int dissect_id_cellSynchronisationInitiation2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSynchronisationInitiationFailureTDD(tvb, offset, actx, tree, hf_nbap_id_cellSynchronisationInitiation2);
}


static const per_sequence_t CellSynchronisationReconfigurationFailureTDD_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CellSynchronisationReconfigurationFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CellSynchronisationReconfigurationFailureTDD, CellSynchronisationReconfigurationFailureTDD_sequence);

  return offset;
}
static int dissect_id_cellSynchronisationReconfiguration2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSynchronisationReconfigurationFailureTDD(tvb, offset, actx, tree, hf_nbap_id_cellSynchronisationReconfiguration2);
}


static const value_string nbap_DummyUnsuccessfullOutcomemessages_vals[] = {
  {   0, "id-audit" },
  {   1, "id-blockResource" },
  {   2, "id-cellReconfiguration" },
  {   3, "id-cellSetup" },
  {   4, "id-commonMeasurementInitiation" },
  {   5, "id-commonTransportChannelReconfigure" },
  {   6, "id-commonTransportChannelSetup" },
  {   7, "id-dedicatedMeasurementInitiation" },
  {   8, "id-radioLinkAddition" },
  {   9, "id-radioLinkAddition-tdd" },
  {  10, "id-radioLinkSetup" },
  {  11, "id-radioLinkSetup-tdd" },
  {  12, "id-synchronisedRadioLinkReconfigurationPreparation" },
  {  13, "id-systemInformationUpdate" },
  {  14, "id-unSynchronisedRadioLinkReconfiguration" },
  {  15, "id-physicalSharedChannelReconfiguration" },
  {  16, "id-informationExchangeInitiation" },
  {  17, "id-cellSynchronisationAdjustment" },
  {  18, "id-cellSynchronisationInitiation" },
  {  19, "id-cellSynchronisationReconfiguration" },
  { 0, NULL }
};

static const per_choice_t DummyUnsuccessfullOutcomemessages_choice[] = {
  {   0, &hf_nbap_id_audit2      , ASN1_EXTENSION_ROOT    , dissect_nbap_AuditFailure },
  {   1, &hf_nbap_id_blockResource2, ASN1_EXTENSION_ROOT    , dissect_nbap_BlockResourceFailure },
  {   2, &hf_nbap_id_cellReconfiguration2, ASN1_EXTENSION_ROOT    , dissect_nbap_CellReconfigurationFailure },
  {   3, &hf_nbap_id_cellSetup2  , ASN1_EXTENSION_ROOT    , dissect_nbap_CellSetupFailure },
  {   4, &hf_nbap_id_commonMeasurementInitiation2, ASN1_EXTENSION_ROOT    , dissect_nbap_CommonMeasurementInitiationResponse },
  {   5, &hf_nbap_id_commonTransportChannelReconfigure2, ASN1_EXTENSION_ROOT    , dissect_nbap_CommonTransportChannelReconfigurationFailure },
  {   6, &hf_nbap_id_commonTransportChannelSetup2, ASN1_EXTENSION_ROOT    , dissect_nbap_CommonTransportChannelSetupFailure },
  {   7, &hf_nbap_id_dedicatedMeasurementInitiation2, ASN1_EXTENSION_ROOT    , dissect_nbap_DedicatedMeasurementInitiationFailure },
  {   8, &hf_nbap_id_radioLinkAddition2, ASN1_EXTENSION_ROOT    , dissect_nbap_RadioLinkAdditionFailureFDD },
  {   9, &hf_nbap_id_radioLinkAddition_tdd1, ASN1_EXTENSION_ROOT    , dissect_nbap_RadioLinkAdditionFailureTDD },
  {  10, &hf_nbap_id_radioLinkSetup2, ASN1_EXTENSION_ROOT    , dissect_nbap_RadioLinkSetupFailureFDD },
  {  11, &hf_nbap_id_radioLinkSetup_tdd2, ASN1_EXTENSION_ROOT    , dissect_nbap_RadioLinkSetupFailureTDD },
  {  12, &hf_nbap_id_synchronisedRadioLinkReconfigurationPreparation2, ASN1_EXTENSION_ROOT    , dissect_nbap_RadioLinkReconfigurationFailure },
  {  13, &hf_nbap_id_systemInformationUpdate2, ASN1_EXTENSION_ROOT    , dissect_nbap_SystemInformationUpdateFailure },
  {  14, &hf_nbap_id_unSynchronisedRadioLinkReconfiguration2, ASN1_EXTENSION_ROOT    , dissect_nbap_RadioLinkReconfigurationFailure },
  {  15, &hf_nbap_id_physicalSharedChannelReconfiguration2, ASN1_EXTENSION_ROOT    , dissect_nbap_PhysicalSharedChannelReconfigurationFailure },
  {  16, &hf_nbap_id_informationExchangeInitiation2, ASN1_EXTENSION_ROOT    , dissect_nbap_InformationExchangeInitiationFailure },
  {  17, &hf_nbap_id_cellSynchronisationAdjustment2, ASN1_EXTENSION_ROOT    , dissect_nbap_CellSynchronisationAdjustmentFailureTDD },
  {  18, &hf_nbap_id_cellSynchronisationInitiation2, ASN1_EXTENSION_ROOT    , dissect_nbap_CellSynchronisationInitiationFailureTDD },
  {  19, &hf_nbap_id_cellSynchronisationReconfiguration2, ASN1_EXTENSION_ROOT    , dissect_nbap_CellSynchronisationReconfigurationFailureTDD },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_DummyUnsuccessfullOutcomemessages(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_DummyUnsuccessfullOutcomemessages, DummyUnsuccessfullOutcomemessages_choice,
                                 NULL);

  return offset;
}



static int
dissect_nbap_CommonPhysicalChannelID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 255U, NULL, FALSE);

  return offset;
}
static int dissect_id_CommonPhysicalChannelID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommonPhysicalChannelID(tvb, offset, actx, tree, hf_nbap_id_CommonPhysicalChannelID);
}
static int dissect_id_Secondary_CPICH_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommonPhysicalChannelID(tvb, offset, actx, tree, hf_nbap_id_Secondary_CPICH_Information);
}
static int dissect_commonPhysicalChannelID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommonPhysicalChannelID(tvb, offset, actx, tree, hf_nbap_commonPhysicalChannelID);
}
static int dissect_commonPhysicalChannelId(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommonPhysicalChannelID(tvb, offset, actx, tree, hf_nbap_commonPhysicalChannelId);
}
static int dissect_associatedSecondaryCPICH(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommonPhysicalChannelID(tvb, offset, actx, tree, hf_nbap_associatedSecondaryCPICH);
}
static int dissect_new_secondary_CPICH(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommonPhysicalChannelID(tvb, offset, actx, tree, hf_nbap_new_secondary_CPICH);
}


static const value_string nbap_ResourceOperationalState_vals[] = {
  {   0, "enabled" },
  {   1, "disabled" },
  { 0, NULL }
};


static int
dissect_nbap_ResourceOperationalState(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_resourceOperationalState(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ResourceOperationalState(tvb, offset, actx, tree, hf_nbap_resourceOperationalState);
}


static const value_string nbap_AvailabilityStatus_vals[] = {
  {   0, "empty" },
  {   1, "in-test" },
  {   2, "failed" },
  {   3, "power-off" },
  {   4, "off-line" },
  {   5, "off-duty" },
  {   6, "dependency" },
  {   7, "degraded" },
  {   8, "not-installed" },
  {   9, "log-full" },
  { 0, NULL }
};


static int
dissect_nbap_AvailabilityStatus(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     10, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_availabilityStatus(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_AvailabilityStatus(tvb, offset, actx, tree, hf_nbap_availabilityStatus);
}


static const per_sequence_t Common_PhysicalChannel_Status_Information_sequence[] = {
  { &hf_nbap_commonPhysicalChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonPhysicalChannelID },
  { &hf_nbap_resourceOperationalState, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ResourceOperationalState },
  { &hf_nbap_availabilityStatus, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_AvailabilityStatus },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Common_PhysicalChannel_Status_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Common_PhysicalChannel_Status_Information, Common_PhysicalChannel_Status_Information_sequence);

  return offset;
}
static int dissect_id_AICH_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Common_PhysicalChannel_Status_Information(tvb, offset, actx, tree, hf_nbap_id_AICH_Information);
}
static int dissect_id_P_CCPCH_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Common_PhysicalChannel_Status_Information(tvb, offset, actx, tree, hf_nbap_id_P_CCPCH_Information);
}
static int dissect_id_P_CPICH_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Common_PhysicalChannel_Status_Information(tvb, offset, actx, tree, hf_nbap_id_P_CPICH_Information);
}
static int dissect_id_P_SCH_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Common_PhysicalChannel_Status_Information(tvb, offset, actx, tree, hf_nbap_id_P_SCH_Information);
}
static int dissect_id_PICH_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Common_PhysicalChannel_Status_Information(tvb, offset, actx, tree, hf_nbap_id_PICH_Information);
}
static int dissect_id_PRACH_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Common_PhysicalChannel_Status_Information(tvb, offset, actx, tree, hf_nbap_id_PRACH_Information);
}
static int dissect_id_S_CCPCH_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Common_PhysicalChannel_Status_Information(tvb, offset, actx, tree, hf_nbap_id_S_CCPCH_Information);
}
static int dissect_id_S_CPICH_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Common_PhysicalChannel_Status_Information(tvb, offset, actx, tree, hf_nbap_id_S_CPICH_Information);
}
static int dissect_id_SCH_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Common_PhysicalChannel_Status_Information(tvb, offset, actx, tree, hf_nbap_id_SCH_Information);
}
static int dissect_id_S_SCH_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Common_PhysicalChannel_Status_Information(tvb, offset, actx, tree, hf_nbap_id_S_SCH_Information);
}
static int dissect_id_DwPCH_LCR_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Common_PhysicalChannel_Status_Information(tvb, offset, actx, tree, hf_nbap_id_DwPCH_LCR_Information);
}
static int dissect_id_DwPCH_LCR_InformationList_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Common_PhysicalChannel_Status_Information(tvb, offset, actx, tree, hf_nbap_id_DwPCH_LCR_InformationList_AuditRsp);
}
static int dissect_id_FPACH_LCR_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Common_PhysicalChannel_Status_Information(tvb, offset, actx, tree, hf_nbap_id_FPACH_LCR_Information);
}
static int dissect_id_FPACH_LCR_Information_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Common_PhysicalChannel_Status_Information(tvb, offset, actx, tree, hf_nbap_id_FPACH_LCR_Information_AuditRsp);
}
static int dissect_id_MICH_Information_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Common_PhysicalChannel_Status_Information(tvb, offset, actx, tree, hf_nbap_id_MICH_Information_AuditRsp);
}
static int dissect_id_MICH_Information_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Common_PhysicalChannel_Status_Information(tvb, offset, actx, tree, hf_nbap_id_MICH_Information_ResourceStatusInd);
}



static int
dissect_nbap_CommonTransportChannelID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 255U, NULL, FALSE);

  return offset;
}
static int dissect_commonTransportChannelID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommonTransportChannelID(tvb, offset, actx, tree, hf_nbap_commonTransportChannelID);
}


static const per_sequence_t Common_TransportChannel_Status_Information_sequence[] = {
  { &hf_nbap_commonTransportChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonTransportChannelID },
  { &hf_nbap_resourceOperationalState, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ResourceOperationalState },
  { &hf_nbap_availabilityStatus, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_AvailabilityStatus },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Common_TransportChannel_Status_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Common_TransportChannel_Status_Information, Common_TransportChannel_Status_Information_sequence);

  return offset;
}
static int dissect_id_BCH_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Common_TransportChannel_Status_Information(tvb, offset, actx, tree, hf_nbap_id_BCH_Information);
}
static int dissect_id_FACH_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Common_TransportChannel_Status_Information(tvb, offset, actx, tree, hf_nbap_id_FACH_Information);
}
static int dissect_id_PCH_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Common_TransportChannel_Status_Information(tvb, offset, actx, tree, hf_nbap_id_PCH_Information);
}
static int dissect_id_RACH_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Common_TransportChannel_Status_Information(tvb, offset, actx, tree, hf_nbap_id_RACH_Information);
}



static int
dissect_nbap_BCCH_ModificationTime(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 511U, NULL, FALSE);

  return offset;
}
static int dissect_id_BCCH_ModificationTime(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BCCH_ModificationTime(tvb, offset, actx, tree, hf_nbap_id_BCCH_ModificationTime);
}


static const value_string nbap_BlockingPriorityIndicator_vals[] = {
  {   0, "high" },
  {   1, "normal" },
  {   2, "low" },
  { 0, NULL }
};


static int
dissect_nbap_BlockingPriorityIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_id_BlockingPriorityIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BlockingPriorityIndicator(tvb, offset, actx, tree, hf_nbap_id_BlockingPriorityIndicator);
}


static const value_string nbap_CauseRadioNetwork_vals[] = {
  {   0, "unknown-C-ID" },
  {   1, "cell-not-available" },
  {   2, "power-level-not-supported" },
  {   3, "dl-radio-resources-not-available" },
  {   4, "ul-radio-resources-not-available" },
  {   5, "rl-already-ActivatedOrAllocated" },
  {   6, "nodeB-Resources-unavailable" },
  {   7, "measurement-not-supported-for-the-object" },
  {   8, "combining-resources-not-available" },
  {   9, "requested-configuration-not-supported" },
  {  10, "synchronisation-failure" },
  {  11, "priority-transport-channel-established" },
  {  12, "sIB-Origination-in-Node-B-not-Supported" },
  {  13, "requested-tx-diversity-mode-not-supported" },
  {  14, "unspecified" },
  {  15, "bCCH-scheduling-error" },
  {  16, "measurement-temporarily-not-available" },
  {  17, "invalid-CM-settings" },
  {  18, "reconfiguration-CFN-not-elapsed" },
  {  19, "number-of-DL-codes-not-supported" },
  {  20, "s-cipch-not-supported" },
  {  21, "combining-not-supported" },
  {  22, "ul-sf-not-supported" },
  {  23, "dl-SF-not-supported" },
  {  24, "common-transport-channel-type-not-supported" },
  {  25, "dedicated-transport-channel-type-not-supported" },
  {  26, "downlink-shared-channel-type-not-supported" },
  {  27, "uplink-shared-channel-type-not-supported" },
  {  28, "cm-not-supported" },
  {  29, "tx-diversity-no-longer-supported" },
  {  30, "unknown-Local-Cell-ID" },
  {  31, "number-of-UL-codes-not-supported" },
  {  32, "information-temporarily-not-available" },
  {  33, "information-provision-not-supported-for-the-object" },
  {  34, "cell-synchronisation-not-supported" },
  {  35, "cell-synchronisation-adjustment-not-supported" },
  {  36, "dpc-mode-change-not-supported" },
  {  37, "iPDL-already-activated" },
  {  38, "iPDL-not-supported" },
  {  39, "iPDL-parameters-not-available" },
  {  40, "frequency-acquisition-not-supported" },
  {  41, "power-balancing-status-not-compatible" },
  {  42, "requested-typeofbearer-re-arrangement-not-supported" },
  {  43, "signalling-Bearer-Re-arrangement-not-supported" },
  {  44, "bearer-Re-arrangement-needed" },
  {  45, "delayed-activation-not-supported" },
  {  46, "rl-timing-adjustment-not-supported" },
  {  47, "mich-not-supported" },
  {  48, "f-DPCH-not-supported" },
  { 0, NULL }
};


static int
dissect_nbap_CauseRadioNetwork(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     31, NULL, TRUE, 18, NULL);

  return offset;
}
static int dissect_radioNetwork(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CauseRadioNetwork(tvb, offset, actx, tree, hf_nbap_radioNetwork);
}


static const value_string nbap_CauseTransport_vals[] = {
  {   0, "transport-resource-unavailable" },
  {   1, "unspecified" },
  { 0, NULL }
};


static int
dissect_nbap_CauseTransport(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_transport(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CauseTransport(tvb, offset, actx, tree, hf_nbap_transport);
}


static const value_string nbap_CauseProtocol_vals[] = {
  {   0, "transfer-syntax-error" },
  {   1, "abstract-syntax-error-reject" },
  {   2, "abstract-syntax-error-ignore-and-notify" },
  {   3, "message-not-compatible-with-receiver-state" },
  {   4, "semantic-error" },
  {   5, "unspecified" },
  {   6, "abstract-syntax-error-falsely-constructed-message" },
  { 0, NULL }
};


static int
dissect_nbap_CauseProtocol(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     7, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_protocol(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CauseProtocol(tvb, offset, actx, tree, hf_nbap_protocol);
}


static const value_string nbap_CauseMisc_vals[] = {
  {   0, "control-processing-overload" },
  {   1, "hardware-failure" },
  {   2, "oam-intervention" },
  {   3, "not-enough-user-plane-processing-resources" },
  {   4, "unspecified" },
  { 0, NULL }
};


static int
dissect_nbap_CauseMisc(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     5, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_misc(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CauseMisc(tvb, offset, actx, tree, hf_nbap_misc);
}


static const value_string nbap_Cause_vals[] = {
  {   0, "radioNetwork" },
  {   1, "transport" },
  {   2, "protocol" },
  {   3, "misc" },
  { 0, NULL }
};

static const per_choice_t Cause_choice[] = {
  {   0, &hf_nbap_radioNetwork   , ASN1_EXTENSION_ROOT    , dissect_nbap_CauseRadioNetwork },
  {   1, &hf_nbap_transport      , ASN1_EXTENSION_ROOT    , dissect_nbap_CauseTransport },
  {   2, &hf_nbap_protocol       , ASN1_EXTENSION_ROOT    , dissect_nbap_CauseProtocol },
  {   3, &hf_nbap_misc           , ASN1_EXTENSION_ROOT    , dissect_nbap_CauseMisc },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_Cause(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_Cause, Cause_choice,
                                 NULL);

  return offset;
}
static int dissect_id_Cause(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Cause(tvb, offset, actx, tree, hf_nbap_id_Cause);
}
static int dissect_cause(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Cause(tvb, offset, actx, tree, hf_nbap_cause);
}



static int
dissect_nbap_CommunicationControlPortID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 65535U, NULL, FALSE);

  return offset;
}
static int dissect_id_CommunicationControlPortID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommunicationControlPortID(tvb, offset, actx, tree, hf_nbap_id_CommunicationControlPortID);
}
static int dissect_id_TargetCommunicationControlPortID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommunicationControlPortID(tvb, offset, actx, tree, hf_nbap_id_TargetCommunicationControlPortID);
}
static int dissect_communicationControlPortID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommunicationControlPortID(tvb, offset, actx, tree, hf_nbap_communicationControlPortID);
}


static const per_sequence_t CCP_InformationItem_AuditRsp_sequence[] = {
  { &hf_nbap_communicationControlPortID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommunicationControlPortID },
  { &hf_nbap_resourceOperationalState, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ResourceOperationalState },
  { &hf_nbap_availabilityStatus, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_AvailabilityStatus },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CCP_InformationItem_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CCP_InformationItem_AuditRsp, CCP_InformationItem_AuditRsp_sequence);

  return offset;
}
static int dissect_id_CCP_InformationItem_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CCP_InformationItem_AuditRsp(tvb, offset, actx, tree, hf_nbap_id_CCP_InformationItem_AuditRsp);
}



static int
dissect_nbap_ProtocolIE_Single_Container(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Field(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_Cell_InformationList_AuditRsp_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_Cell_InformationList_AuditRsp_item);
}
static int dissect_S_CPICH_InformationList_AuditRsp_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_S_CPICH_InformationList_AuditRsp_item);
}
static int dissect_S_CCPCH_InformationList_AuditRsp_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_S_CCPCH_InformationList_AuditRsp_item);
}
static int dissect_FACH_InformationList_AuditRsp_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_FACH_InformationList_AuditRsp_item);
}
static int dissect_PRACH_InformationList_AuditRsp_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_PRACH_InformationList_AuditRsp_item);
}
static int dissect_RACH_InformationList_AuditRsp_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_RACH_InformationList_AuditRsp_item);
}
static int dissect_AICH_InformationList_AuditRsp_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_AICH_InformationList_AuditRsp_item);
}
static int dissect_CCP_InformationList_AuditRsp_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_CCP_InformationList_AuditRsp_item);
}
static int dissect_FPACH_LCR_InformationList_AuditRsp_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_FPACH_LCR_InformationList_AuditRsp_item);
}
static int dissect_S_CCPCH_InformationListExt_AuditRsp_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_S_CCPCH_InformationListExt_AuditRsp_item);
}
static int dissect_S_CCPCH_LCR_InformationListExt_AuditRsp_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_S_CCPCH_LCR_InformationListExt_AuditRsp_item);
}
static int dissect_Local_Cell_InformationList_AuditRsp_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_Local_Cell_InformationList_AuditRsp_item);
}
static int dissect_Local_Cell_Group_InformationList_AuditRsp_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_Local_Cell_Group_InformationList_AuditRsp_item);
}
static int dissect_Power_Local_Cell_Group_InformationList_AuditRsp_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_Power_Local_Cell_Group_InformationList_AuditRsp_item);
}
static int dissect_SecondaryCPICH_InformationList_Cell_SetupRqstFDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_SecondaryCPICH_InformationList_Cell_SetupRqstFDD_item);
}
static int dissect_CellPortion_InformationList_Cell_SetupRqstFDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_CellPortion_InformationList_Cell_SetupRqstFDD_item);
}
static int dissect_SecondaryCPICH_InformationList_Cell_ReconfRqstFDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_SecondaryCPICH_InformationList_Cell_ReconfRqstFDD_item);
}
static int dissect_CellPortion_InformationList_Cell_ReconfRqstFDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_CellPortion_InformationList_Cell_ReconfRqstFDD_item);
}
static int dissect_Local_Cell_InformationList_ResourceStatusInd_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_Local_Cell_InformationList_ResourceStatusInd_item);
}
static int dissect_Local_Cell_Group_InformationList_ResourceStatusInd_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_Local_Cell_Group_InformationList_ResourceStatusInd_item);
}
static int dissect_Power_Local_Cell_Group_InformationList_ResourceStatusInd_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_Power_Local_Cell_Group_InformationList_ResourceStatusInd_item);
}
static int dissect_Local_Cell_InformationList2_ResourceStatusInd_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_Local_Cell_InformationList2_ResourceStatusInd_item);
}
static int dissect_Local_Cell_Group_InformationList2_ResourceStatusInd_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_Local_Cell_Group_InformationList2_ResourceStatusInd_item);
}
static int dissect_Power_Local_Cell_Group_InformationList2_ResourceStatusInd_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_Power_Local_Cell_Group_InformationList2_ResourceStatusInd_item);
}
static int dissect_CCP_InformationList_ResourceStatusInd_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_CCP_InformationList_ResourceStatusInd_item);
}
static int dissect_Cell_InformationList_ResourceStatusInd_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_Cell_InformationList_ResourceStatusInd_item);
}
static int dissect_S_CPICH_InformationList_ResourceStatusInd_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_S_CPICH_InformationList_ResourceStatusInd_item);
}
static int dissect_S_CCPCH_InformationList_ResourceStatusInd_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_S_CCPCH_InformationList_ResourceStatusInd_item);
}
static int dissect_FACH_InformationList_ResourceStatusInd_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_FACH_InformationList_ResourceStatusInd_item);
}
static int dissect_PRACH_InformationList_ResourceStatusInd_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_PRACH_InformationList_ResourceStatusInd_item);
}
static int dissect_RACH_InformationList_ResourceStatusInd_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_RACH_InformationList_ResourceStatusInd_item);
}
static int dissect_AICH_InformationList_ResourceStatusInd_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_AICH_InformationList_ResourceStatusInd_item);
}
static int dissect_FPACH_LCR_InformationList_ResourceStatusInd_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_FPACH_LCR_InformationList_ResourceStatusInd_item);
}
static int dissect_S_CCPCH_InformationListExt_ResourceStatusInd_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_S_CCPCH_InformationListExt_ResourceStatusInd_item);
}
static int dissect_S_CCPCH_LCR_InformationListExt_ResourceStatusInd_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_S_CCPCH_LCR_InformationListExt_ResourceStatusInd_item);
}
static int dissect_RL_InformationList_RL_SetupRqstFDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_RL_InformationList_RL_SetupRqstFDD_item);
}
static int dissect_UL_CCTrCH_InformationList_RL_SetupRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_UL_CCTrCH_InformationList_RL_SetupRqstTDD_item);
}
static int dissect_DL_CCTrCH_InformationList_RL_SetupRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_DL_CCTrCH_InformationList_RL_SetupRqstTDD_item);
}
static int dissect_RL_InformationResponseList_RL_SetupRspFDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_RL_InformationResponseList_RL_SetupRspFDD_item);
}
static int dissect_Unsuccessful_RL_InformationRespList_RL_SetupFailureFDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_Unsuccessful_RL_InformationRespList_RL_SetupFailureFDD_item);
}
static int dissect_Successful_RL_InformationRespList_RL_SetupFailureFDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_Successful_RL_InformationRespList_RL_SetupFailureFDD_item);
}
static int dissect_RL_InformationList_RL_AdditionRqstFDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_RL_InformationList_RL_AdditionRqstFDD_item);
}
static int dissect_RL_InformationResponseList_RL_AdditionRspFDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_RL_InformationResponseList_RL_AdditionRspFDD_item);
}
static int dissect_Unsuccessful_RL_InformationRespList_RL_AdditionFailureFDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_Unsuccessful_RL_InformationRespList_RL_AdditionFailureFDD_item);
}
static int dissect_Successful_RL_InformationRespList_RL_AdditionFailureFDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_Successful_RL_InformationRespList_RL_AdditionFailureFDD_item);
}
static int dissect_RL_InformationList_RL_ReconfPrepFDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_RL_InformationList_RL_ReconfPrepFDD_item);
}
static int dissect_RL_InformationResponseList_RL_ReconfReady_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_RL_InformationResponseList_RL_ReconfReady_item);
}
static int dissect_RL_ReconfigurationFailureList_RL_ReconfFailure_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_RL_ReconfigurationFailureList_RL_ReconfFailure_item);
}
static int dissect_RL_InformationList_RL_ReconfRqstFDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_RL_InformationList_RL_ReconfRqstFDD_item);
}
static int dissect_UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD_item);
}
static int dissect_UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD_item);
}
static int dissect_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD_item);
}
static int dissect_DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD_item);
}
static int dissect_RL_InformationResponseList_RL_ReconfRsp_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_RL_InformationResponseList_RL_ReconfRsp_item);
}
static int dissect_RL_informationList_RL_DeletionRqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_RL_informationList_RL_DeletionRqst_item);
}
static int dissect_DL_ReferencePowerInformationList_DL_PC_Rqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_DL_ReferencePowerInformationList_DL_PC_Rqst_item);
}
static int dissect_RL_InformationList_DM_Rqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_RL_InformationList_DM_Rqst_item);
}
static int dissect_RL_InformationList_DM_Rsp_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_RL_InformationList_DM_Rsp_item);
}
static int dissect_RL_Set_InformationList_DM_Rsp_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_RL_Set_InformationList_DM_Rsp_item);
}
static int dissect_RL_InformationList_DM_Rprt_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_RL_InformationList_DM_Rprt_item);
}
static int dissect_RL_Set_InformationList_DM_Rprt_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_RL_Set_InformationList_DM_Rprt_item);
}
static int dissect_RL_InformationList_RL_FailureInd_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_RL_InformationList_RL_FailureInd_item);
}
static int dissect_RL_Set_InformationList_RL_FailureInd_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_RL_Set_InformationList_RL_FailureInd_item);
}
static int dissect_CCTrCH_InformationList_RL_FailureInd_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_CCTrCH_InformationList_RL_FailureInd_item);
}
static int dissect_RL_InformationList_RL_PreemptRequiredInd_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_RL_InformationList_RL_PreemptRequiredInd_item);
}
static int dissect_RL_InformationList_RL_RestoreInd_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_RL_InformationList_RL_RestoreInd_item);
}
static int dissect_RL_Set_InformationList_RL_RestoreInd_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_RL_Set_InformationList_RL_RestoreInd_item);
}
static int dissect_CCTrCH_InformationList_RL_RestoreInd_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_CCTrCH_InformationList_RL_RestoreInd_item);
}
static int dissect_Unsuccessful_PDSCHSetList_PSCH_ReconfFailureTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_Unsuccessful_PDSCHSetList_PSCH_ReconfFailureTDD_item);
}
static int dissect_Unsuccessful_PUSCHSetList_PSCH_ReconfFailureTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_Unsuccessful_PUSCHSetList_PSCH_ReconfFailureTDD_item);
}
static int dissect_CommunicationContextInfoList_Reset_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_CommunicationContextInfoList_Reset_item);
}
static int dissect_CommunicationControlPortInfoList_Reset_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_CommunicationControlPortInfoList_Reset_item);
}
static int dissect_CellAdjustmentInfo_SyncAdjustmentRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_CellAdjustmentInfo_SyncAdjustmentRqstTDD_item);
}
static int dissect_Unsuccessful_cell_InformationRespList_SyncAdjustmntFailureTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_Unsuccessful_cell_InformationRespList_SyncAdjustmntFailureTDD_item);
}
static int dissect_DelayedActivationInformationList_RL_ActivationCmdFDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_DelayedActivationInformationList_RL_ActivationCmdFDD_item);
}
static int dissect_DelayedActivationInformationList_RL_ActivationCmdTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_nbap_DelayedActivationInformationList_RL_ActivationCmdTDD_item);
}


static const per_sequence_t CCP_InformationList_AuditRsp_sequence_of[1] = {
  { &hf_nbap_CCP_InformationList_AuditRsp_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_CCP_InformationList_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_CCP_InformationList_AuditRsp, CCP_InformationList_AuditRsp_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_id_CCP_InformationList_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CCP_InformationList_AuditRsp(tvb, offset, actx, tree, hf_nbap_id_CCP_InformationList_AuditRsp);
}


static const per_sequence_t CCP_InformationItem_ResourceStatusInd_sequence[] = {
  { &hf_nbap_communicationControlPortID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommunicationControlPortID },
  { &hf_nbap_resourceOperationalState, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ResourceOperationalState },
  { &hf_nbap_availabilityStatus, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_AvailabilityStatus },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CCP_InformationItem_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CCP_InformationItem_ResourceStatusInd, CCP_InformationItem_ResourceStatusInd_sequence);

  return offset;
}
static int dissect_id_CCP_InformationItem_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CCP_InformationItem_ResourceStatusInd(tvb, offset, actx, tree, hf_nbap_id_CCP_InformationItem_ResourceStatusInd);
}



static int
dissect_nbap_C_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 65535U, NULL, FALSE);

  return offset;
}
static int dissect_id_C_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_C_ID(tvb, offset, actx, tree, hf_nbap_id_C_ID);
}
static int dissect_c_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_C_ID(tvb, offset, actx, tree, hf_nbap_c_ID);
}



static int
dissect_nbap_ConfigurationGenerationID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 255U, NULL, FALSE);

  return offset;
}
static int dissect_id_ConfigurationGenerationID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ConfigurationGenerationID(tvb, offset, actx, tree, hf_nbap_id_ConfigurationGenerationID);
}
static int dissect_configurationGenerationID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ConfigurationGenerationID(tvb, offset, actx, tree, hf_nbap_configurationGenerationID);
}



static int
dissect_nbap_Local_Cell_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 268435455U, NULL, FALSE);

  return offset;
}
static int dissect_id_Local_Cell_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Local_Cell_ID(tvb, offset, actx, tree, hf_nbap_id_Local_Cell_ID);
}
static int dissect_id_Power_Local_Cell_Group_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Local_Cell_ID(tvb, offset, actx, tree, hf_nbap_id_Power_Local_Cell_Group_ID);
}
static int dissect_local_Cell_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Local_Cell_ID(tvb, offset, actx, tree, hf_nbap_local_Cell_ID);
}
static int dissect_local_Cell_Group_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Local_Cell_ID(tvb, offset, actx, tree, hf_nbap_local_Cell_Group_ID);
}
static int dissect_power_Local_Cell_Group_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Local_Cell_ID(tvb, offset, actx, tree, hf_nbap_power_Local_Cell_Group_ID);
}
static int dissect_powerLocalCellGroupID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Local_Cell_ID(tvb, offset, actx, tree, hf_nbap_powerLocalCellGroupID);
}
static int dissect_local_CellID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Local_Cell_ID(tvb, offset, actx, tree, hf_nbap_local_CellID);
}



static int
dissect_nbap_P_SCH_Information_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_primary_SCH_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_P_SCH_Information_AuditRsp(tvb, offset, actx, tree, hf_nbap_primary_SCH_Information);
}



static int
dissect_nbap_S_SCH_Information_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_secondary_SCH_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_S_SCH_Information_AuditRsp(tvb, offset, actx, tree, hf_nbap_secondary_SCH_Information);
}



static int
dissect_nbap_P_CPICH_Information_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_primary_CPICH_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_P_CPICH_Information_AuditRsp(tvb, offset, actx, tree, hf_nbap_primary_CPICH_Information);
}


static const per_sequence_t S_CPICH_InformationList_AuditRsp_sequence_of[1] = {
  { &hf_nbap_S_CPICH_InformationList_AuditRsp_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_S_CPICH_InformationList_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_S_CPICH_InformationList_AuditRsp, S_CPICH_InformationList_AuditRsp_sequence_of,
                                                  1, 32);

  return offset;
}
static int dissect_secondary_CPICH_InformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_S_CPICH_InformationList_AuditRsp(tvb, offset, actx, tree, hf_nbap_secondary_CPICH_InformationList);
}



static int
dissect_nbap_P_CCPCH_Information_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_primary_CCPCH_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_P_CCPCH_Information_AuditRsp(tvb, offset, actx, tree, hf_nbap_primary_CCPCH_Information);
}



static int
dissect_nbap_BCH_Information_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_bCH_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BCH_Information_AuditRsp(tvb, offset, actx, tree, hf_nbap_bCH_Information);
}


static const per_sequence_t S_CCPCH_InformationList_AuditRsp_sequence_of[1] = {
  { &hf_nbap_S_CCPCH_InformationList_AuditRsp_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_S_CCPCH_InformationList_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_S_CCPCH_InformationList_AuditRsp, S_CCPCH_InformationList_AuditRsp_sequence_of,
                                                  1, 32);

  return offset;
}
static int dissect_secondary_CCPCH_InformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_S_CCPCH_InformationList_AuditRsp(tvb, offset, actx, tree, hf_nbap_secondary_CCPCH_InformationList);
}



static int
dissect_nbap_PCH_Information_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_pCH_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PCH_Information_AuditRsp(tvb, offset, actx, tree, hf_nbap_pCH_Information);
}



static int
dissect_nbap_PICH_Information_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_pICH_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PICH_Information_AuditRsp(tvb, offset, actx, tree, hf_nbap_pICH_Information);
}


static const per_sequence_t FACH_InformationList_AuditRsp_sequence_of[1] = {
  { &hf_nbap_FACH_InformationList_AuditRsp_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_FACH_InformationList_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_FACH_InformationList_AuditRsp, FACH_InformationList_AuditRsp_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_fACH_InformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_FACH_InformationList_AuditRsp(tvb, offset, actx, tree, hf_nbap_fACH_InformationList);
}


static const per_sequence_t PRACH_InformationList_AuditRsp_sequence_of[1] = {
  { &hf_nbap_PRACH_InformationList_AuditRsp_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_PRACH_InformationList_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_PRACH_InformationList_AuditRsp, PRACH_InformationList_AuditRsp_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_pRACH_InformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PRACH_InformationList_AuditRsp(tvb, offset, actx, tree, hf_nbap_pRACH_InformationList);
}


static const per_sequence_t RACH_InformationList_AuditRsp_sequence_of[1] = {
  { &hf_nbap_RACH_InformationList_AuditRsp_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_RACH_InformationList_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_RACH_InformationList_AuditRsp, RACH_InformationList_AuditRsp_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_rACH_InformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RACH_InformationList_AuditRsp(tvb, offset, actx, tree, hf_nbap_rACH_InformationList);
}


static const per_sequence_t AICH_InformationList_AuditRsp_sequence_of[1] = {
  { &hf_nbap_AICH_InformationList_AuditRsp_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_AICH_InformationList_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_AICH_InformationList_AuditRsp, AICH_InformationList_AuditRsp_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_aICH_InformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_AICH_InformationList_AuditRsp(tvb, offset, actx, tree, hf_nbap_aICH_InformationList);
}



static int
dissect_nbap_NULL(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_null(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_notUsed_pCPCHes_parameters(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NULL(tvb, offset, actx, tree, hf_nbap_notUsed_pCPCHes_parameters);
}
static int dissect_notUsed_cPCH_parameters(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NULL(tvb, offset, actx, tree, hf_nbap_notUsed_cPCH_parameters);
}
static int dissect_notUsed_1_pCPCH_InformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NULL(tvb, offset, actx, tree, hf_nbap_notUsed_1_pCPCH_InformationList);
}
static int dissect_notUsed_2_cPCH_InformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NULL(tvb, offset, actx, tree, hf_nbap_notUsed_2_cPCH_InformationList);
}
static int dissect_notUsed_3_aP_AICH_InformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NULL(tvb, offset, actx, tree, hf_nbap_notUsed_3_aP_AICH_InformationList);
}
static int dissect_notUsed_4_cDCA_ICH_InformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NULL(tvb, offset, actx, tree, hf_nbap_notUsed_4_cDCA_ICH_InformationList);
}
static int dissect_notUsed_cPCH(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NULL(tvb, offset, actx, tree, hf_nbap_notUsed_cPCH);
}
static int dissect_yes_Deletion(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NULL(tvb, offset, actx, tree, hf_nbap_yes_Deletion);
}
static int dissect_not_Used_sSDT_CellID_Length(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NULL(tvb, offset, actx, tree, hf_nbap_not_Used_sSDT_CellID_Length);
}
static int dissect_not_Used_s_FieldLength(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NULL(tvb, offset, actx, tree, hf_nbap_not_Used_s_FieldLength);
}
static int dissect_not_Used_pDSCH_RL_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NULL(tvb, offset, actx, tree, hf_nbap_not_Used_pDSCH_RL_ID);
}
static int dissect_not_Used_pDSCH_CodeMapping(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NULL(tvb, offset, actx, tree, hf_nbap_not_Used_pDSCH_CodeMapping);
}
static int dissect_not_Used_sSDT_Cell_Identity(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NULL(tvb, offset, actx, tree, hf_nbap_not_Used_sSDT_Cell_Identity);
}
static int dissect_not_Used_dSCH_InformationResponseList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NULL(tvb, offset, actx, tree, hf_nbap_not_Used_dSCH_InformationResponseList);
}
static int dissect_not_Used_tFCI2_BearerInformationResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NULL(tvb, offset, actx, tree, hf_nbap_not_Used_tFCI2_BearerInformationResponse);
}
static int dissect_not_Used_sSDT_CellIdentity(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NULL(tvb, offset, actx, tree, hf_nbap_not_Used_sSDT_CellIdentity);
}
static int dissect_not_Used_sSDT_CellIDLength(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NULL(tvb, offset, actx, tree, hf_nbap_not_Used_sSDT_CellIDLength);
}
static int dissect_not_Used_sSDT_Indication(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NULL(tvb, offset, actx, tree, hf_nbap_not_Used_sSDT_Indication);
}
static int dissect_nodeB(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NULL(tvb, offset, actx, tree, hf_nbap_nodeB);
}
static int dissect_lateEntrantCell(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NULL(tvb, offset, actx, tree, hf_nbap_lateEntrantCell);
}
static int dissect_frequencyAcquisition(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NULL(tvb, offset, actx, tree, hf_nbap_frequencyAcquisition);
}
static int dissect_cellSyncBurstNotAvailable(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NULL(tvb, offset, actx, tree, hf_nbap_cellSyncBurstNotAvailable);
}
static int dissect_syncDLCodeIDNotAvailable(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NULL(tvb, offset, actx, tree, hf_nbap_syncDLCodeIDNotAvailable);
}
static int dissect_notUsed_1_acknowledged_PCPCH_access_preambles(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NULL(tvb, offset, actx, tree, hf_nbap_notUsed_1_acknowledged_PCPCH_access_preambles);
}
static int dissect_notUsed_2_detected_PCPCH_access_preambles(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NULL(tvb, offset, actx, tree, hf_nbap_notUsed_2_detected_PCPCH_access_preambles);
}
static int dissect_separate_indication(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NULL(tvb, offset, actx, tree, hf_nbap_separate_indication);
}
static int dissect_unsynchronised(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NULL(tvb, offset, actx, tree, hf_nbap_unsynchronised);
}
static int dissect_remove(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NULL(tvb, offset, actx, tree, hf_nbap_remove);
}
static int dissect_e_DCH_Scheduled_Transmission_Grant(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NULL(tvb, offset, actx, tree, hf_nbap_e_DCH_Scheduled_Transmission_Grant);
}
static int dissect_no_bad_satellites(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NULL(tvb, offset, actx, tree, hf_nbap_no_bad_satellites);
}
static int dissect_onDemand(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NULL(tvb, offset, actx, tree, hf_nbap_onDemand);
}
static int dissect_defaultMidamble(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NULL(tvb, offset, actx, tree, hf_nbap_defaultMidamble);
}
static int dissect_commonMidamble(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NULL(tvb, offset, actx, tree, hf_nbap_commonMidamble);
}
static int dissect_secondary_CPICH_shall_not_be_used(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NULL(tvb, offset, actx, tree, hf_nbap_secondary_CPICH_shall_not_be_used);
}
static int dissect_serving_E_DCH_RL_not_in_this_NodeB(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NULL(tvb, offset, actx, tree, hf_nbap_serving_E_DCH_RL_not_in_this_NodeB);
}
static int dissect_not_Used_splitType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NULL(tvb, offset, actx, tree, hf_nbap_not_Used_splitType);
}
static int dissect_not_Used_lengthOfTFCI2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NULL(tvb, offset, actx, tree, hf_nbap_not_Used_lengthOfTFCI2);
}
static int dissect_not_Used_split_in_TFCI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NULL(tvb, offset, actx, tree, hf_nbap_not_Used_split_in_TFCI);
}
static int dissect_notApplicable(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NULL(tvb, offset, actx, tree, hf_nbap_notApplicable);
}



static int
dissect_nbap_SCH_Information_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_sCH_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SCH_Information_AuditRsp(tvb, offset, actx, tree, hf_nbap_sCH_Information);
}


static const per_sequence_t Cell_InformationItem_AuditRsp_sequence[] = {
  { &hf_nbap_c_ID           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_C_ID },
  { &hf_nbap_configurationGenerationID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ConfigurationGenerationID },
  { &hf_nbap_resourceOperationalState, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ResourceOperationalState },
  { &hf_nbap_availabilityStatus, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_AvailabilityStatus },
  { &hf_nbap_local_Cell_ID  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Local_Cell_ID },
  { &hf_nbap_primary_SCH_Information, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_P_SCH_Information_AuditRsp },
  { &hf_nbap_secondary_SCH_Information, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_S_SCH_Information_AuditRsp },
  { &hf_nbap_primary_CPICH_Information, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_P_CPICH_Information_AuditRsp },
  { &hf_nbap_secondary_CPICH_InformationList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_S_CPICH_InformationList_AuditRsp },
  { &hf_nbap_primary_CCPCH_Information, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_P_CCPCH_Information_AuditRsp },
  { &hf_nbap_bCH_Information, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_BCH_Information_AuditRsp },
  { &hf_nbap_secondary_CCPCH_InformationList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_S_CCPCH_InformationList_AuditRsp },
  { &hf_nbap_pCH_Information, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_PCH_Information_AuditRsp },
  { &hf_nbap_pICH_Information, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_PICH_Information_AuditRsp },
  { &hf_nbap_fACH_InformationList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_FACH_InformationList_AuditRsp },
  { &hf_nbap_pRACH_InformationList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_PRACH_InformationList_AuditRsp },
  { &hf_nbap_rACH_InformationList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_RACH_InformationList_AuditRsp },
  { &hf_nbap_aICH_InformationList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_AICH_InformationList_AuditRsp },
  { &hf_nbap_notUsed_1_pCPCH_InformationList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_NULL },
  { &hf_nbap_notUsed_2_cPCH_InformationList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_NULL },
  { &hf_nbap_notUsed_3_aP_AICH_InformationList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_NULL },
  { &hf_nbap_notUsed_4_cDCA_ICH_InformationList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_NULL },
  { &hf_nbap_sCH_Information, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_SCH_Information_AuditRsp },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Cell_InformationItem_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Cell_InformationItem_AuditRsp, Cell_InformationItem_AuditRsp_sequence);

  return offset;
}
static int dissect_id_Cell_InformationItem_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Cell_InformationItem_AuditRsp(tvb, offset, actx, tree, hf_nbap_id_Cell_InformationItem_AuditRsp);
}



static int
dissect_nbap_P_SCH_Information_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_primary_SCH_Information1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_P_SCH_Information_ResourceStatusInd(tvb, offset, actx, tree, hf_nbap_primary_SCH_Information1);
}



static int
dissect_nbap_S_SCH_Information_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_secondary_SCH_Information1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_S_SCH_Information_ResourceStatusInd(tvb, offset, actx, tree, hf_nbap_secondary_SCH_Information1);
}



static int
dissect_nbap_P_CPICH_Information_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_primary_CPICH_Information1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_P_CPICH_Information_ResourceStatusInd(tvb, offset, actx, tree, hf_nbap_primary_CPICH_Information1);
}


static const per_sequence_t S_CPICH_InformationList_ResourceStatusInd_sequence_of[1] = {
  { &hf_nbap_S_CPICH_InformationList_ResourceStatusInd_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_S_CPICH_InformationList_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_S_CPICH_InformationList_ResourceStatusInd, S_CPICH_InformationList_ResourceStatusInd_sequence_of,
                                                  1, 32);

  return offset;
}
static int dissect_secondary_CPICH_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_S_CPICH_InformationList_ResourceStatusInd(tvb, offset, actx, tree, hf_nbap_secondary_CPICH_Information);
}



static int
dissect_nbap_P_CCPCH_Information_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_primary_CCPCH_Information1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_P_CCPCH_Information_ResourceStatusInd(tvb, offset, actx, tree, hf_nbap_primary_CCPCH_Information1);
}



static int
dissect_nbap_BCH_Information_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_bCH_Information1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BCH_Information_ResourceStatusInd(tvb, offset, actx, tree, hf_nbap_bCH_Information1);
}


static const per_sequence_t S_CCPCH_InformationList_ResourceStatusInd_sequence_of[1] = {
  { &hf_nbap_S_CCPCH_InformationList_ResourceStatusInd_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_S_CCPCH_InformationList_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_S_CCPCH_InformationList_ResourceStatusInd, S_CCPCH_InformationList_ResourceStatusInd_sequence_of,
                                                  1, 32);

  return offset;
}
static int dissect_secondary_CCPCH_InformationList1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_S_CCPCH_InformationList_ResourceStatusInd(tvb, offset, actx, tree, hf_nbap_secondary_CCPCH_InformationList1);
}



static int
dissect_nbap_PCH_Information_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_pCH_Information1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PCH_Information_ResourceStatusInd(tvb, offset, actx, tree, hf_nbap_pCH_Information1);
}



static int
dissect_nbap_PICH_Information_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_pICH_Information1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PICH_Information_ResourceStatusInd(tvb, offset, actx, tree, hf_nbap_pICH_Information1);
}


static const per_sequence_t FACH_InformationList_ResourceStatusInd_sequence_of[1] = {
  { &hf_nbap_FACH_InformationList_ResourceStatusInd_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_FACH_InformationList_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_FACH_InformationList_ResourceStatusInd, FACH_InformationList_ResourceStatusInd_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_fACH_InformationList1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_FACH_InformationList_ResourceStatusInd(tvb, offset, actx, tree, hf_nbap_fACH_InformationList1);
}


static const per_sequence_t PRACH_InformationList_ResourceStatusInd_sequence_of[1] = {
  { &hf_nbap_PRACH_InformationList_ResourceStatusInd_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_PRACH_InformationList_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_PRACH_InformationList_ResourceStatusInd, PRACH_InformationList_ResourceStatusInd_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_pRACH_InformationList1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PRACH_InformationList_ResourceStatusInd(tvb, offset, actx, tree, hf_nbap_pRACH_InformationList1);
}


static const per_sequence_t RACH_InformationList_ResourceStatusInd_sequence_of[1] = {
  { &hf_nbap_RACH_InformationList_ResourceStatusInd_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_RACH_InformationList_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_RACH_InformationList_ResourceStatusInd, RACH_InformationList_ResourceStatusInd_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_rACH_InformationList1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RACH_InformationList_ResourceStatusInd(tvb, offset, actx, tree, hf_nbap_rACH_InformationList1);
}


static const per_sequence_t AICH_InformationList_ResourceStatusInd_sequence_of[1] = {
  { &hf_nbap_AICH_InformationList_ResourceStatusInd_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_AICH_InformationList_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_AICH_InformationList_ResourceStatusInd, AICH_InformationList_ResourceStatusInd_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_aICH_InformationList1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_AICH_InformationList_ResourceStatusInd(tvb, offset, actx, tree, hf_nbap_aICH_InformationList1);
}



static int
dissect_nbap_SCH_Information_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_sCH_Information1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SCH_Information_ResourceStatusInd(tvb, offset, actx, tree, hf_nbap_sCH_Information1);
}


static const per_sequence_t Cell_InformationItem_ResourceStatusInd_sequence[] = {
  { &hf_nbap_c_ID           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_C_ID },
  { &hf_nbap_resourceOperationalState, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ResourceOperationalState },
  { &hf_nbap_availabilityStatus, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_AvailabilityStatus },
  { &hf_nbap_primary_SCH_Information1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_P_SCH_Information_ResourceStatusInd },
  { &hf_nbap_secondary_SCH_Information1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_S_SCH_Information_ResourceStatusInd },
  { &hf_nbap_primary_CPICH_Information1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_P_CPICH_Information_ResourceStatusInd },
  { &hf_nbap_secondary_CPICH_Information, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_S_CPICH_InformationList_ResourceStatusInd },
  { &hf_nbap_primary_CCPCH_Information1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_P_CCPCH_Information_ResourceStatusInd },
  { &hf_nbap_bCH_Information1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_BCH_Information_ResourceStatusInd },
  { &hf_nbap_secondary_CCPCH_InformationList1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_S_CCPCH_InformationList_ResourceStatusInd },
  { &hf_nbap_pCH_Information1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_PCH_Information_ResourceStatusInd },
  { &hf_nbap_pICH_Information1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_PICH_Information_ResourceStatusInd },
  { &hf_nbap_fACH_InformationList1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_FACH_InformationList_ResourceStatusInd },
  { &hf_nbap_pRACH_InformationList1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_PRACH_InformationList_ResourceStatusInd },
  { &hf_nbap_rACH_InformationList1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_RACH_InformationList_ResourceStatusInd },
  { &hf_nbap_aICH_InformationList1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_AICH_InformationList_ResourceStatusInd },
  { &hf_nbap_notUsed_1_pCPCH_InformationList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_NULL },
  { &hf_nbap_notUsed_2_cPCH_InformationList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_NULL },
  { &hf_nbap_notUsed_3_aP_AICH_InformationList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_NULL },
  { &hf_nbap_notUsed_4_cDCA_ICH_InformationList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_NULL },
  { &hf_nbap_sCH_Information1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_SCH_Information_ResourceStatusInd },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Cell_InformationItem_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Cell_InformationItem_ResourceStatusInd, Cell_InformationItem_ResourceStatusInd_sequence);

  return offset;
}
static int dissect_id_Cell_InformationItem_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Cell_InformationItem_ResourceStatusInd(tvb, offset, actx, tree, hf_nbap_id_Cell_InformationItem_ResourceStatusInd);
}


static const per_sequence_t Cell_InformationList_AuditRsp_sequence_of[1] = {
  { &hf_nbap_Cell_InformationList_AuditRsp_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_Cell_InformationList_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_Cell_InformationList_AuditRsp, Cell_InformationList_AuditRsp_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_id_Cell_InformationList_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Cell_InformationList_AuditRsp(tvb, offset, actx, tree, hf_nbap_id_Cell_InformationList_AuditRsp);
}



static int
dissect_nbap_CellParameterID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 127U, NULL, TRUE);

  return offset;
}
static int dissect_id_CellParameterID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellParameterID(tvb, offset, actx, tree, hf_nbap_id_CellParameterID);
}
static int dissect_cellParameterID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellParameterID(tvb, offset, actx, tree, hf_nbap_cellParameterID);
}



static int
dissect_nbap_CFN(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 255U, NULL, FALSE);

  return offset;
}
static int dissect_id_CFN(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CFN(tvb, offset, actx, tree, hf_nbap_id_CFN);
}
static int dissect_cFN(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CFN(tvb, offset, actx, tree, hf_nbap_cFN);
}
static int dissect_cMConfigurationChangeCFN(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CFN(tvb, offset, actx, tree, hf_nbap_cMConfigurationChangeCFN);
}
static int dissect_tGCFN(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CFN(tvb, offset, actx, tree, hf_nbap_tGCFN);
}
static int dissect_cfn(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CFN(tvb, offset, actx, tree, hf_nbap_cfn);
}
static int dissect_synchronised(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CFN(tvb, offset, actx, tree, hf_nbap_synchronised);
}


static const value_string nbap_TUTRANGPSAccuracyClass_vals[] = {
  {   0, "accuracy-class-A" },
  {   1, "accuracy-class-B" },
  {   2, "accuracy-class-C" },
  { 0, NULL }
};


static int
dissect_nbap_TUTRANGPSAccuracyClass(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_tUTRANGPSMeasurementAccuracyClass(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TUTRANGPSAccuracyClass(tvb, offset, actx, tree, hf_nbap_tUTRANGPSMeasurementAccuracyClass);
}


static const value_string nbap_CommonMeasurementAccuracy_vals[] = {
  {   0, "tUTRANGPSMeasurementAccuracyClass" },
  { 0, NULL }
};

static const per_choice_t CommonMeasurementAccuracy_choice[] = {
  {   0, &hf_nbap_tUTRANGPSMeasurementAccuracyClass, ASN1_EXTENSION_ROOT    , dissect_nbap_TUTRANGPSAccuracyClass },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_CommonMeasurementAccuracy(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_CommonMeasurementAccuracy, CommonMeasurementAccuracy_choice,
                                 NULL);

  return offset;
}
static int dissect_id_CommonMeasurementAccuracy(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommonMeasurementAccuracy(tvb, offset, actx, tree, hf_nbap_id_CommonMeasurementAccuracy);
}



static int
dissect_nbap_Transmitted_Carrier_Power_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 100U, NULL, FALSE);

  return offset;
}
static int dissect_id_Transmitted_Carrier_Power_For_CellPortion(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Transmitted_Carrier_Power_Value(tvb, offset, actx, tree, hf_nbap_id_Transmitted_Carrier_Power_For_CellPortion);
}
static int dissect_transmitted_carrier_power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Transmitted_Carrier_Power_Value(tvb, offset, actx, tree, hf_nbap_transmitted_carrier_power);
}
static int dissect_transmitted_Carrier_Power_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Transmitted_Carrier_Power_Value(tvb, offset, actx, tree, hf_nbap_transmitted_Carrier_Power_Value);
}



static int
dissect_nbap_Received_total_wide_band_power_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 621U, NULL, FALSE);

  return offset;
}
static int dissect_received_total_wide_band_power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Received_total_wide_band_power_Value(tvb, offset, actx, tree, hf_nbap_received_total_wide_band_power);
}
static int dissect_received_total_wide_band_power_value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Received_total_wide_band_power_Value(tvb, offset, actx, tree, hf_nbap_received_total_wide_band_power_value);
}



static int
dissect_nbap_Acknowledged_PRACH_preambles_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 240U, NULL, TRUE);

  return offset;
}
static int dissect_acknowledged_prach_preambles(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Acknowledged_PRACH_preambles_Value(tvb, offset, actx, tree, hf_nbap_acknowledged_prach_preambles);
}



static int
dissect_nbap_UL_TimeslotISCP_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 127U, NULL, FALSE);

  return offset;
}
static int dissect_uL_TimeslotISCP(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_TimeslotISCP_Value(tvb, offset, actx, tree, hf_nbap_uL_TimeslotISCP);
}
static int dissect_iSCP(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_TimeslotISCP_Value(tvb, offset, actx, tree, hf_nbap_iSCP);
}



static int
dissect_nbap_Extension_CommonMeasurementValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_extension_CommonMeasurementValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Extension_CommonMeasurementValue(tvb, offset, actx, tree, hf_nbap_extension_CommonMeasurementValue);
}


static const value_string nbap_CommonMeasurementValue_vals[] = {
  {   0, "transmitted-carrier-power" },
  {   1, "received-total-wide-band-power" },
  {   2, "acknowledged-prach-preambles" },
  {   3, "uL-TimeslotISCP" },
  {   4, "notUsed-1-acknowledged-PCPCH-access-preambles" },
  {   5, "notUsed-2-detected-PCPCH-access-preambles" },
  {   6, "extension-CommonMeasurementValue" },
  { 0, NULL }
};

static const per_choice_t CommonMeasurementValue_choice[] = {
  {   0, &hf_nbap_transmitted_carrier_power, ASN1_EXTENSION_ROOT    , dissect_nbap_Transmitted_Carrier_Power_Value },
  {   1, &hf_nbap_received_total_wide_band_power, ASN1_EXTENSION_ROOT    , dissect_nbap_Received_total_wide_band_power_Value },
  {   2, &hf_nbap_acknowledged_prach_preambles, ASN1_EXTENSION_ROOT    , dissect_nbap_Acknowledged_PRACH_preambles_Value },
  {   3, &hf_nbap_uL_TimeslotISCP, ASN1_EXTENSION_ROOT    , dissect_nbap_UL_TimeslotISCP_Value },
  {   4, &hf_nbap_notUsed_1_acknowledged_PCPCH_access_preambles, ASN1_EXTENSION_ROOT    , dissect_nbap_NULL },
  {   5, &hf_nbap_notUsed_2_detected_PCPCH_access_preambles, ASN1_EXTENSION_ROOT    , dissect_nbap_NULL },
  {   6, &hf_nbap_extension_CommonMeasurementValue, ASN1_NOT_EXTENSION_ROOT, dissect_nbap_Extension_CommonMeasurementValue },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_CommonMeasurementValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_CommonMeasurementValue, CommonMeasurementValue_choice,
                                 NULL);

  return offset;
}
static int dissect_commonMeasurementValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommonMeasurementValue(tvb, offset, actx, tree, hf_nbap_commonMeasurementValue);
}
static int dissect_commonmeasurementValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommonMeasurementValue(tvb, offset, actx, tree, hf_nbap_commonmeasurementValue);
}


static const per_sequence_t CommonMeasurementAvailable_sequence[] = {
  { &hf_nbap_commonmeasurementValue, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonMeasurementValue },
  { &hf_nbap_ie_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CommonMeasurementAvailable(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CommonMeasurementAvailable, CommonMeasurementAvailable_sequence);

  return offset;
}
static int dissect_measurementAvailable(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommonMeasurementAvailable(tvb, offset, actx, tree, hf_nbap_measurementAvailable);
}



static int
dissect_nbap_CommonMeasurementnotAvailable(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_null(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_measurementnotAvailable(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommonMeasurementnotAvailable(tvb, offset, actx, tree, hf_nbap_measurementnotAvailable);
}


static const value_string nbap_CommonMeasurementValueInformation_vals[] = {
  {   0, "measurementAvailable" },
  {   1, "measurementnotAvailable" },
  { 0, NULL }
};

static const per_choice_t CommonMeasurementValueInformation_choice[] = {
  {   0, &hf_nbap_measurementAvailable, ASN1_NO_EXTENSIONS     , dissect_nbap_CommonMeasurementAvailable },
  {   1, &hf_nbap_measurementnotAvailable, ASN1_NO_EXTENSIONS     , dissect_nbap_CommonMeasurementnotAvailable },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_CommonMeasurementValueInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_CommonMeasurementValueInformation, CommonMeasurementValueInformation_choice,
                                 NULL);

  return offset;
}
static int dissect_commonMeasurementValueInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommonMeasurementValueInformation(tvb, offset, actx, tree, hf_nbap_commonMeasurementValueInformation);
}


static const per_sequence_t Cell_CM_Rprt_sequence[] = {
  { &hf_nbap_commonMeasurementValueInformation, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonMeasurementValueInformation },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Cell_CM_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Cell_CM_Rprt, Cell_CM_Rprt_sequence);

  return offset;
}
static int dissect_cell5(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Cell_CM_Rprt(tvb, offset, actx, tree, hf_nbap_cell5);
}


static const per_sequence_t RACH_CM_Rprt_sequence[] = {
  { &hf_nbap_commonMeasurementValueInformation, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonMeasurementValueInformation },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RACH_CM_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RACH_CM_Rprt, RACH_CM_Rprt_sequence);

  return offset;
}
static int dissect_rACH3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RACH_CM_Rprt(tvb, offset, actx, tree, hf_nbap_rACH3);
}



static int
dissect_nbap_Extension_CommonMeasurementObjectType_CM_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_extension_CommonMeasurementObjectType_CM_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Extension_CommonMeasurementObjectType_CM_Rprt(tvb, offset, actx, tree, hf_nbap_extension_CommonMeasurementObjectType_CM_Rprt);
}


static const value_string nbap_CommonMeasurementObjectType_CM_Rprt_vals[] = {
  {   0, "cell" },
  {   1, "rACH" },
  {   2, "notUsed-cPCH" },
  {   3, "extension-CommonMeasurementObjectType-CM-Rprt" },
  { 0, NULL }
};

static const per_choice_t CommonMeasurementObjectType_CM_Rprt_choice[] = {
  {   0, &hf_nbap_cell5          , ASN1_EXTENSION_ROOT    , dissect_nbap_Cell_CM_Rprt },
  {   1, &hf_nbap_rACH3          , ASN1_EXTENSION_ROOT    , dissect_nbap_RACH_CM_Rprt },
  {   2, &hf_nbap_notUsed_cPCH   , ASN1_EXTENSION_ROOT    , dissect_nbap_NULL },
  {   3, &hf_nbap_extension_CommonMeasurementObjectType_CM_Rprt, ASN1_NOT_EXTENSION_ROOT, dissect_nbap_Extension_CommonMeasurementObjectType_CM_Rprt },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_CommonMeasurementObjectType_CM_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_CommonMeasurementObjectType_CM_Rprt, CommonMeasurementObjectType_CM_Rprt_choice,
                                 NULL);

  return offset;
}
static int dissect_id_CommonMeasurementObjectType_CM_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommonMeasurementObjectType_CM_Rprt(tvb, offset, actx, tree, hf_nbap_id_CommonMeasurementObjectType_CM_Rprt);
}



static int
dissect_nbap_TimeSlot(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 14U, NULL, FALSE);

  return offset;
}
static int dissect_id_TimeSlot(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TimeSlot(tvb, offset, actx, tree, hf_nbap_id_TimeSlot);
}
static int dissect_timeslot(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TimeSlot(tvb, offset, actx, tree, hf_nbap_timeslot);
}
static int dissect_timeSlot(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TimeSlot(tvb, offset, actx, tree, hf_nbap_timeSlot);
}
static int dissect_TimeslotInfo_CellSyncInitiationRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TimeSlot(tvb, offset, actx, tree, hf_nbap_TimeslotInfo_CellSyncInitiationRqstTDD_item);
}


static const per_sequence_t Cell_CM_Rqst_sequence[] = {
  { &hf_nbap_c_ID           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_C_ID },
  { &hf_nbap_timeSlot       , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TimeSlot },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Cell_CM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Cell_CM_Rqst, Cell_CM_Rqst_sequence);

  return offset;
}
static int dissect_cell(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Cell_CM_Rqst(tvb, offset, actx, tree, hf_nbap_cell);
}


static const per_sequence_t RACH_CM_Rqst_sequence[] = {
  { &hf_nbap_c_ID           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_C_ID },
  { &hf_nbap_commonTransportChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonTransportChannelID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RACH_CM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RACH_CM_Rqst, RACH_CM_Rqst_sequence);

  return offset;
}
static int dissect_rACH1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RACH_CM_Rqst(tvb, offset, actx, tree, hf_nbap_rACH1);
}



static int
dissect_nbap_Extension_CommonMeasurementObjectType_CM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_extension_CommonMeasurementObjectType_CM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Extension_CommonMeasurementObjectType_CM_Rqst(tvb, offset, actx, tree, hf_nbap_extension_CommonMeasurementObjectType_CM_Rqst);
}


static const value_string nbap_CommonMeasurementObjectType_CM_Rqst_vals[] = {
  {   0, "cell" },
  {   1, "rACH" },
  {   2, "notUsed-cPCH" },
  {   3, "extension-CommonMeasurementObjectType-CM-Rqst" },
  { 0, NULL }
};

static const per_choice_t CommonMeasurementObjectType_CM_Rqst_choice[] = {
  {   0, &hf_nbap_cell           , ASN1_EXTENSION_ROOT    , dissect_nbap_Cell_CM_Rqst },
  {   1, &hf_nbap_rACH1          , ASN1_EXTENSION_ROOT    , dissect_nbap_RACH_CM_Rqst },
  {   2, &hf_nbap_notUsed_cPCH   , ASN1_EXTENSION_ROOT    , dissect_nbap_NULL },
  {   3, &hf_nbap_extension_CommonMeasurementObjectType_CM_Rqst, ASN1_NOT_EXTENSION_ROOT, dissect_nbap_Extension_CommonMeasurementObjectType_CM_Rqst },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_CommonMeasurementObjectType_CM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_CommonMeasurementObjectType_CM_Rqst, CommonMeasurementObjectType_CM_Rqst_choice,
                                 NULL);

  return offset;
}
static int dissect_id_CommonMeasurementObjectType_CM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommonMeasurementObjectType_CM_Rqst(tvb, offset, actx, tree, hf_nbap_id_CommonMeasurementObjectType_CM_Rqst);
}


static const per_sequence_t Cell_CM_Rsp_sequence[] = {
  { &hf_nbap_commonMeasurementValue, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonMeasurementValue },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Cell_CM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Cell_CM_Rsp, Cell_CM_Rsp_sequence);

  return offset;
}
static int dissect_cell1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Cell_CM_Rsp(tvb, offset, actx, tree, hf_nbap_cell1);
}


static const per_sequence_t RACH_CM_Rsp_sequence[] = {
  { &hf_nbap_commonMeasurementValue, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonMeasurementValue },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RACH_CM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RACH_CM_Rsp, RACH_CM_Rsp_sequence);

  return offset;
}
static int dissect_rACH2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RACH_CM_Rsp(tvb, offset, actx, tree, hf_nbap_rACH2);
}



static int
dissect_nbap_Extension_CommonMeasurementObjectType_CM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_extension_CommonMeasurementObjectType_CM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Extension_CommonMeasurementObjectType_CM_Rsp(tvb, offset, actx, tree, hf_nbap_extension_CommonMeasurementObjectType_CM_Rsp);
}


static const value_string nbap_CommonMeasurementObjectType_CM_Rsp_vals[] = {
  {   0, "cell" },
  {   1, "rACH" },
  {   2, "notUsed-cPCH" },
  {   3, "extension-CommonMeasurementObjectType-CM-Rsp" },
  { 0, NULL }
};

static const per_choice_t CommonMeasurementObjectType_CM_Rsp_choice[] = {
  {   0, &hf_nbap_cell1          , ASN1_EXTENSION_ROOT    , dissect_nbap_Cell_CM_Rsp },
  {   1, &hf_nbap_rACH2          , ASN1_EXTENSION_ROOT    , dissect_nbap_RACH_CM_Rsp },
  {   2, &hf_nbap_notUsed_cPCH   , ASN1_EXTENSION_ROOT    , dissect_nbap_NULL },
  {   3, &hf_nbap_extension_CommonMeasurementObjectType_CM_Rsp, ASN1_NOT_EXTENSION_ROOT, dissect_nbap_Extension_CommonMeasurementObjectType_CM_Rsp },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_CommonMeasurementObjectType_CM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_CommonMeasurementObjectType_CM_Rsp, CommonMeasurementObjectType_CM_Rsp_choice,
                                 NULL);

  return offset;
}
static int dissect_id_CommonMeasurementObjectType_CM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommonMeasurementObjectType_CM_Rsp(tvb, offset, actx, tree, hf_nbap_id_CommonMeasurementObjectType_CM_Rsp);
}


static const value_string nbap_CommonMeasurementType_vals[] = {
  {   0, "received-total-wide-band-power" },
  {   1, "transmitted-carrier-power" },
  {   2, "acknowledged-prach-preambles" },
  {   3, "ul-timeslot-iscp" },
  {   4, "notUsed-1-acknowledged-PCPCH-access-preambles" },
  {   5, "notUsed-2-detected-PCPCH-access-preambles" },
  {   6, "uTRAN-GPS-Timing-of-Cell-Frames-for-UE-Positioning" },
  {   7, "sFN-SFN-Observed-Time-Difference" },
  {   8, "transmittedCarrierPowerOfAllCodesNotUsedForHSTransmission" },
  {   9, "hS-DSCH-Required-Power" },
  {  10, "hS-DSCH-Provided-Bit-Rate" },
  {  11, "received-total-wide-band-power-for-cellPortion" },
  {  12, "transmitted-carrier-power-for-cellPortion" },
  {  13, "transmittedCarrierPowerOfAllCodesNotUsedForHS-PDSCH-HS-SCCH-E-AGCH-E-RGCHOrE-HICHTransmission-for-cellPortion" },
  {  14, "upPTS-Interference" },
  {  15, "dLTransmissionBranchLoad" },
  {  16, "hS-DSCH-Required-Power-for-cell-portion" },
  {  17, "hS-DSCH-Provided-Bit-Rate-for-cell-portion" },
  {  18, "e-DCH-Provided-Bit-Rate" },
  { 0, NULL }
};


static int
dissect_nbap_CommonMeasurementType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     6, NULL, TRUE, 13, NULL);

  return offset;
}
static int dissect_id_CommonMeasurementType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommonMeasurementType(tvb, offset, actx, tree, hf_nbap_id_CommonMeasurementType);
}



static int
dissect_nbap_FDD_S_CCPCH_Offset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 149U, NULL, FALSE);

  return offset;
}
static int dissect_fdd_S_CCPCH_Offset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_FDD_S_CCPCH_Offset(tvb, offset, actx, tree, hf_nbap_fdd_S_CCPCH_Offset);
}



static int
dissect_nbap_DL_ScramblingCode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 15U, NULL, FALSE);

  return offset;
}
static int dissect_id_HS_PDSCH_HS_SCCH_ScramblingCode_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_ScramblingCode(tvb, offset, actx, tree, hf_nbap_id_HS_PDSCH_HS_SCCH_ScramblingCode_PSCH_ReconfRqst);
}
static int dissect_id_E_AGCH_And_E_RGCH_E_HICH_FDD_Scrambling_Code(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_ScramblingCode(tvb, offset, actx, tree, hf_nbap_id_E_AGCH_And_E_RGCH_E_HICH_FDD_Scrambling_Code);
}
static int dissect_dl_ScramblingCode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_ScramblingCode(tvb, offset, actx, tree, hf_nbap_dl_ScramblingCode);
}
static int dissect_hS_PDSCH_HS_SCCH_ScramblingCode_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_ScramblingCode(tvb, offset, actx, tree, hf_nbap_hS_PDSCH_HS_SCCH_ScramblingCode_PSCH_ReconfRqst);
}
static int dissect_e_AGCH_And_E_RGCH_E_HICH_FDD_Scrambling_Code(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_ScramblingCode(tvb, offset, actx, tree, hf_nbap_e_AGCH_And_E_RGCH_E_HICH_FDD_Scrambling_Code);
}



static int
dissect_nbap_FDD_DL_ChannelisationCodeNumber(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 511U, NULL, FALSE);

  return offset;
}
static int dissect_fdd_DL_ChannelisationCodeNumber(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_FDD_DL_ChannelisationCodeNumber(tvb, offset, actx, tree, hf_nbap_fdd_DL_ChannelisationCodeNumber);
}
static int dissect_fdd_dl_ChannelisationCodeNumber(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_FDD_DL_ChannelisationCodeNumber(tvb, offset, actx, tree, hf_nbap_fdd_dl_ChannelisationCodeNumber);
}
static int dissect_fDD_DL_ChannelisationCodeNumber(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_FDD_DL_ChannelisationCodeNumber(tvb, offset, actx, tree, hf_nbap_fDD_DL_ChannelisationCodeNumber);
}
static int dissect_E_AGCH_FDD_Code_List_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_FDD_DL_ChannelisationCodeNumber(tvb, offset, actx, tree, hf_nbap_E_AGCH_FDD_Code_List_item);
}
static int dissect_e_AGCH_Channelisation_Code(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_FDD_DL_ChannelisationCodeNumber(tvb, offset, actx, tree, hf_nbap_e_AGCH_Channelisation_Code);
}
static int dissect_e_RGCH_E_HICH_Channelisation_Code(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_FDD_DL_ChannelisationCodeNumber(tvb, offset, actx, tree, hf_nbap_e_RGCH_E_HICH_Channelisation_Code);
}
static int dissect_E_RGCH_E_HICH_FDD_Code_List_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_FDD_DL_ChannelisationCodeNumber(tvb, offset, actx, tree, hf_nbap_E_RGCH_E_HICH_FDD_Code_List_item);
}



static int
dissect_nbap_INTEGER_0_3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 3U, NULL, FALSE);

  return offset;
}
static int dissect_ctfc2bit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_INTEGER_0_3(tvb, offset, actx, tree, hf_nbap_ctfc2bit);
}



static int
dissect_nbap_INTEGER_0_15(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 15U, NULL, FALSE);

  return offset;
}
static int dissect_number_of_HS_PDSCH_codes(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_INTEGER_0_15(tvb, offset, actx, tree, hf_nbap_number_of_HS_PDSCH_codes);
}
static int dissect_burstStart(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_INTEGER_0_15(tvb, offset, actx, tree, hf_nbap_burstStart);
}
static int dissect_ctfc4bit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_INTEGER_0_15(tvb, offset, actx, tree, hf_nbap_ctfc4bit);
}



static int
dissect_nbap_INTEGER_0_63(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 63U, NULL, FALSE);

  return offset;
}
static int dissect_seed(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_INTEGER_0_63(tvb, offset, actx, tree, hf_nbap_seed);
}
static int dissect_noinitialOffset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_INTEGER_0_63(tvb, offset, actx, tree, hf_nbap_noinitialOffset);
}
static int dissect_ctfc6bit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_INTEGER_0_63(tvb, offset, actx, tree, hf_nbap_ctfc6bit);
}



static int
dissect_nbap_INTEGER_0_255(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 255U, NULL, FALSE);

  return offset;
}
static int dissect_initialOffset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_INTEGER_0_255(tvb, offset, actx, tree, hf_nbap_initialOffset);
}
static int dissect_ctfc8bit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_INTEGER_0_255(tvb, offset, actx, tree, hf_nbap_ctfc8bit);
}



static int
dissect_nbap_INTEGER_0_4095(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 4095U, NULL, FALSE);

  return offset;
}
static int dissect_iP_Start(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_INTEGER_0_4095(tvb, offset, actx, tree, hf_nbap_iP_Start);
}
static int dissect_ctfc12bit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_INTEGER_0_4095(tvb, offset, actx, tree, hf_nbap_ctfc12bit);
}



static int
dissect_nbap_INTEGER_0_16777215(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 16777215U, NULL, FALSE);

  return offset;
}
static int dissect_ctfcmaxbit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_INTEGER_0_16777215(tvb, offset, actx, tree, hf_nbap_ctfcmaxbit);
}


static const value_string nbap_TFCS_CTFC_vals[] = {
  {   0, "ctfc2bit" },
  {   1, "ctfc4bit" },
  {   2, "ctfc6bit" },
  {   3, "ctfc8bit" },
  {   4, "ctfc12bit" },
  {   5, "ctfc16bit" },
  {   6, "ctfcmaxbit" },
  { 0, NULL }
};

static const per_choice_t TFCS_CTFC_choice[] = {
  {   0, &hf_nbap_ctfc2bit       , ASN1_NO_EXTENSIONS     , dissect_nbap_INTEGER_0_3 },
  {   1, &hf_nbap_ctfc4bit       , ASN1_NO_EXTENSIONS     , dissect_nbap_INTEGER_0_15 },
  {   2, &hf_nbap_ctfc6bit       , ASN1_NO_EXTENSIONS     , dissect_nbap_INTEGER_0_63 },
  {   3, &hf_nbap_ctfc8bit       , ASN1_NO_EXTENSIONS     , dissect_nbap_INTEGER_0_255 },
  {   4, &hf_nbap_ctfc12bit      , ASN1_NO_EXTENSIONS     , dissect_nbap_INTEGER_0_4095 },
  {   5, &hf_nbap_ctfc16bit      , ASN1_NO_EXTENSIONS     , dissect_nbap_INTEGER_0_65535 },
  {   6, &hf_nbap_ctfcmaxbit     , ASN1_NO_EXTENSIONS     , dissect_nbap_INTEGER_0_16777215 },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_TFCS_CTFC(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_TFCS_CTFC, TFCS_CTFC_choice,
                                 NULL);

  return offset;
}
static int dissect_cTFC(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TFCS_CTFC(tvb, offset, actx, tree, hf_nbap_cTFC);
}



static int
dissect_nbap_BetaCD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 15U, NULL, FALSE);

  return offset;
}
static int dissect_betaC(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BetaCD(tvb, offset, actx, tree, hf_nbap_betaC);
}
static int dissect_betaD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BetaCD(tvb, offset, actx, tree, hf_nbap_betaD);
}
static int dissect_tdd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BetaCD(tvb, offset, actx, tree, hf_nbap_tdd);
}


static const per_sequence_t T_fdd_sequence[] = {
  { &hf_nbap_betaC          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BetaCD },
  { &hf_nbap_betaD          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BetaCD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_T_fdd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_T_fdd, T_fdd_sequence);

  return offset;
}
static int dissect_fdd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_T_fdd(tvb, offset, actx, tree, hf_nbap_fdd);
}


static const value_string nbap_T_gainFactor_vals[] = {
  {   0, "fdd" },
  {   1, "tdd" },
  { 0, NULL }
};

static const per_choice_t T_gainFactor_choice[] = {
  {   0, &hf_nbap_fdd            , ASN1_EXTENSION_ROOT    , dissect_nbap_T_fdd },
  {   1, &hf_nbap_tdd            , ASN1_EXTENSION_ROOT    , dissect_nbap_BetaCD },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_T_gainFactor(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_T_gainFactor, T_gainFactor_choice,
                                 NULL);

  return offset;
}
static int dissect_gainFactor(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_T_gainFactor(tvb, offset, actx, tree, hf_nbap_gainFactor);
}



static int
dissect_nbap_RefTFCNumber(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 3U, NULL, FALSE);

  return offset;
}
static int dissect_refTFCNumber(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RefTFCNumber(tvb, offset, actx, tree, hf_nbap_refTFCNumber);
}
static int dissect_computedGainFactors(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RefTFCNumber(tvb, offset, actx, tree, hf_nbap_computedGainFactors);
}


static const per_sequence_t T_signalledGainFactors_sequence[] = {
  { &hf_nbap_gainFactor     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_T_gainFactor },
  { &hf_nbap_refTFCNumber   , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_RefTFCNumber },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_T_signalledGainFactors(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_T_signalledGainFactors, T_signalledGainFactors_sequence);

  return offset;
}
static int dissect_signalledGainFactors(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_T_signalledGainFactors(tvb, offset, actx, tree, hf_nbap_signalledGainFactors);
}


static const value_string nbap_TransportFormatCombination_Beta_vals[] = {
  {   0, "signalledGainFactors" },
  {   1, "computedGainFactors" },
  { 0, NULL }
};

static const per_choice_t TransportFormatCombination_Beta_choice[] = {
  {   0, &hf_nbap_signalledGainFactors, ASN1_EXTENSION_ROOT    , dissect_nbap_T_signalledGainFactors },
  {   1, &hf_nbap_computedGainFactors, ASN1_EXTENSION_ROOT    , dissect_nbap_RefTFCNumber },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_TransportFormatCombination_Beta(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_TransportFormatCombination_Beta, TransportFormatCombination_Beta_choice,
                                 NULL);

  return offset;
}
static int dissect_tFC_Beta(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TransportFormatCombination_Beta(tvb, offset, actx, tree, hf_nbap_tFC_Beta);
}


static const per_sequence_t TFCS_TFCSList_item_sequence[] = {
  { &hf_nbap_cTFC           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TFCS_CTFC },
  { &hf_nbap_tFC_Beta       , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TransportFormatCombination_Beta },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_TFCS_TFCSList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_TFCS_TFCSList_item, TFCS_TFCSList_item_sequence);

  return offset;
}
static int dissect_TFCS_TFCSList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TFCS_TFCSList_item(tvb, offset, actx, tree, hf_nbap_TFCS_TFCSList_item);
}


static const per_sequence_t TFCS_TFCSList_sequence_of[1] = {
  { &hf_nbap_TFCS_TFCSList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_TFCS_TFCSList_item },
};

static int
dissect_nbap_TFCS_TFCSList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_TFCS_TFCSList, TFCS_TFCSList_sequence_of,
                                                  1, 1024);

  return offset;
}
static int dissect_no_Split_in_TFCI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TFCS_TFCSList(tvb, offset, actx, tree, hf_nbap_no_Split_in_TFCI);
}


static const value_string nbap_T_tFCSvalues_vals[] = {
  {   0, "no-Split-in-TFCI" },
  {   1, "not-Used-split-in-TFCI" },
  { 0, NULL }
};

static const per_choice_t T_tFCSvalues_choice[] = {
  {   0, &hf_nbap_no_Split_in_TFCI, ASN1_EXTENSION_ROOT    , dissect_nbap_TFCS_TFCSList },
  {   1, &hf_nbap_not_Used_split_in_TFCI, ASN1_EXTENSION_ROOT    , dissect_nbap_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_T_tFCSvalues(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_T_tFCSvalues, T_tFCSvalues_choice,
                                 NULL);

  return offset;
}
static int dissect_tFCSvalues(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_T_tFCSvalues(tvb, offset, actx, tree, hf_nbap_tFCSvalues);
}


static const per_sequence_t TFCS_sequence[] = {
  { &hf_nbap_tFCSvalues     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_T_tFCSvalues },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_TFCS(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_TFCS, TFCS_sequence);

  return offset;
}
static int dissect_tFCS(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TFCS(tvb, offset, actx, tree, hf_nbap_tFCS);
}
static int dissect_ul_TFCS(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TFCS(tvb, offset, actx, tree, hf_nbap_ul_TFCS);
}
static int dissect_dl_TFCS(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TFCS(tvb, offset, actx, tree, hf_nbap_dl_TFCS);
}



static int
dissect_nbap_SecondaryCCPCH_SlotFormat(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 17U, NULL, TRUE);

  return offset;
}
static int dissect_secondary_CCPCH_SlotFormat(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SecondaryCCPCH_SlotFormat(tvb, offset, actx, tree, hf_nbap_secondary_CCPCH_SlotFormat);
}


static const value_string nbap_TFCI_Presence_vals[] = {
  {   0, "present" },
  {   1, "not-present" },
  { 0, NULL }
};


static int
dissect_nbap_TFCI_Presence(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_tFCI_Presence(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TFCI_Presence(tvb, offset, actx, tree, hf_nbap_tFCI_Presence);
}


static const value_string nbap_MultiplexingPosition_vals[] = {
  {   0, "fixed" },
  {   1, "flexible" },
  { 0, NULL }
};


static int
dissect_nbap_MultiplexingPosition(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_multiplexingPosition(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MultiplexingPosition(tvb, offset, actx, tree, hf_nbap_multiplexingPosition);
}



static int
dissect_nbap_PowerOffset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 24U, NULL, FALSE);

  return offset;
}
static int dissect_pO1_ForTFCI_Bits(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PowerOffset(tvb, offset, actx, tree, hf_nbap_pO1_ForTFCI_Bits);
}
static int dissect_pO3_ForPilotBits(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PowerOffset(tvb, offset, actx, tree, hf_nbap_pO3_ForPilotBits);
}
static int dissect_pO2_ForTPC_Bits(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PowerOffset(tvb, offset, actx, tree, hf_nbap_pO2_ForTPC_Bits);
}


static const per_sequence_t PowerOffsetInformation_CTCH_SetupRqstFDD_sequence[] = {
  { &hf_nbap_pO1_ForTFCI_Bits, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PowerOffset },
  { &hf_nbap_pO3_ForPilotBits, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PowerOffset },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PowerOffsetInformation_CTCH_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PowerOffsetInformation_CTCH_SetupRqstFDD, PowerOffsetInformation_CTCH_SetupRqstFDD_sequence);

  return offset;
}
static int dissect_powerOffsetInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PowerOffsetInformation_CTCH_SetupRqstFDD(tvb, offset, actx, tree, hf_nbap_powerOffsetInformation);
}


static const value_string nbap_STTD_Indicator_vals[] = {
  {   0, "active" },
  {   1, "inactive" },
  { 0, NULL }
};


static int
dissect_nbap_STTD_Indicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_sTTD_Indicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_STTD_Indicator(tvb, offset, actx, tree, hf_nbap_sTTD_Indicator);
}



static int
dissect_nbap_FACH_ParametersList_CTCH_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_fACH_Parameters(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_FACH_ParametersList_CTCH_SetupRqstFDD(tvb, offset, actx, tree, hf_nbap_fACH_Parameters);
}



static int
dissect_nbap_PCH_Parameters_CTCH_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_pCH_Parameters(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PCH_Parameters_CTCH_SetupRqstFDD(tvb, offset, actx, tree, hf_nbap_pCH_Parameters);
}


static const per_sequence_t Secondary_CCPCH_CTCH_SetupRqstFDD_sequence[] = {
  { &hf_nbap_commonPhysicalChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonPhysicalChannelID },
  { &hf_nbap_fdd_S_CCPCH_Offset, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_FDD_S_CCPCH_Offset },
  { &hf_nbap_dl_ScramblingCode, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_ScramblingCode },
  { &hf_nbap_fdd_DL_ChannelisationCodeNumber, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_FDD_DL_ChannelisationCodeNumber },
  { &hf_nbap_tFCS           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TFCS },
  { &hf_nbap_secondary_CCPCH_SlotFormat, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SecondaryCCPCH_SlotFormat },
  { &hf_nbap_tFCI_Presence  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TFCI_Presence },
  { &hf_nbap_multiplexingPosition, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MultiplexingPosition },
  { &hf_nbap_powerOffsetInformation, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PowerOffsetInformation_CTCH_SetupRqstFDD },
  { &hf_nbap_sTTD_Indicator , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_STTD_Indicator },
  { &hf_nbap_fACH_Parameters, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_FACH_ParametersList_CTCH_SetupRqstFDD },
  { &hf_nbap_pCH_Parameters , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_PCH_Parameters_CTCH_SetupRqstFDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Secondary_CCPCH_CTCH_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Secondary_CCPCH_CTCH_SetupRqstFDD, Secondary_CCPCH_CTCH_SetupRqstFDD_sequence);

  return offset;
}
static int dissect_secondary_CCPCH_parameters(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Secondary_CCPCH_CTCH_SetupRqstFDD(tvb, offset, actx, tree, hf_nbap_secondary_CCPCH_parameters);
}



static int
dissect_nbap_ScramblingCodeNumber(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 15U, NULL, FALSE);

  return offset;
}
static int dissect_scramblingCodeNumber(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ScramblingCodeNumber(tvb, offset, actx, tree, hf_nbap_scramblingCodeNumber);
}


static const asn_namedbit PreambleSignatures_bits[] = {
  {  0, &hf_nbap_PreambleSignatures_signature15, -1, -1, "signature15", NULL },
  {  1, &hf_nbap_PreambleSignatures_signature14, -1, -1, "signature14", NULL },
  {  2, &hf_nbap_PreambleSignatures_signature13, -1, -1, "signature13", NULL },
  {  3, &hf_nbap_PreambleSignatures_signature12, -1, -1, "signature12", NULL },
  {  4, &hf_nbap_PreambleSignatures_signature11, -1, -1, "signature11", NULL },
  {  5, &hf_nbap_PreambleSignatures_signature10, -1, -1, "signature10", NULL },
  {  6, &hf_nbap_PreambleSignatures_signature9, -1, -1, "signature9", NULL },
  {  7, &hf_nbap_PreambleSignatures_signature8, -1, -1, "signature8", NULL },
  {  8, &hf_nbap_PreambleSignatures_signature7, -1, -1, "signature7", NULL },
  {  9, &hf_nbap_PreambleSignatures_signature6, -1, -1, "signature6", NULL },
  { 10, &hf_nbap_PreambleSignatures_signature5, -1, -1, "signature5", NULL },
  { 11, &hf_nbap_PreambleSignatures_signature4, -1, -1, "signature4", NULL },
  { 12, &hf_nbap_PreambleSignatures_signature3, -1, -1, "signature3", NULL },
  { 13, &hf_nbap_PreambleSignatures_signature2, -1, -1, "signature2", NULL },
  { 14, &hf_nbap_PreambleSignatures_signature1, -1, -1, "signature1", NULL },
  { 15, &hf_nbap_PreambleSignatures_signature0, -1, -1, "signature0", NULL },
  { 0, NULL, 0, 0, NULL, NULL }
};

static int
dissect_nbap_PreambleSignatures(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     16, 16, FALSE, NULL);

  return offset;
}
static int dissect_preambleSignatures(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PreambleSignatures(tvb, offset, actx, tree, hf_nbap_preambleSignatures);
}


static const value_string nbap_RACH_SlotFormat_vals[] = {
  {   0, "v0" },
  {   1, "v1" },
  {   2, "v2" },
  {   3, "v3" },
  { 0, NULL }
};


static int
dissect_nbap_RACH_SlotFormat(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_rACHSlotFormat(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RACH_SlotFormat(tvb, offset, actx, tree, hf_nbap_rACHSlotFormat);
}
static int dissect_rACH_SlotFormat(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RACH_SlotFormat(tvb, offset, actx, tree, hf_nbap_rACH_SlotFormat);
}


static const per_sequence_t AllowedSlotFormatInformationItem_CTCH_SetupRqstFDD_sequence[] = {
  { &hf_nbap_rACHSlotFormat , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RACH_SlotFormat },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_AllowedSlotFormatInformationItem_CTCH_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_AllowedSlotFormatInformationItem_CTCH_SetupRqstFDD, AllowedSlotFormatInformationItem_CTCH_SetupRqstFDD_sequence);

  return offset;
}
static int dissect_AllowedSlotFormatInformationList_CTCH_SetupRqstFDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_AllowedSlotFormatInformationItem_CTCH_SetupRqstFDD(tvb, offset, actx, tree, hf_nbap_AllowedSlotFormatInformationList_CTCH_SetupRqstFDD_item);
}


static const per_sequence_t AllowedSlotFormatInformationList_CTCH_SetupRqstFDD_sequence_of[1] = {
  { &hf_nbap_AllowedSlotFormatInformationList_CTCH_SetupRqstFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_AllowedSlotFormatInformationItem_CTCH_SetupRqstFDD },
};

static int
dissect_nbap_AllowedSlotFormatInformationList_CTCH_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_AllowedSlotFormatInformationList_CTCH_SetupRqstFDD, AllowedSlotFormatInformationList_CTCH_SetupRqstFDD_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_allowedSlotFormatInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_AllowedSlotFormatInformationList_CTCH_SetupRqstFDD(tvb, offset, actx, tree, hf_nbap_allowedSlotFormatInformation);
}


static const asn_namedbit RACH_SubChannelNumbers_bits[] = {
  {  0, &hf_nbap_RACH_SubChannelNumbers_subCh11, -1, -1, "subCh11", NULL },
  {  1, &hf_nbap_RACH_SubChannelNumbers_subCh10, -1, -1, "subCh10", NULL },
  {  2, &hf_nbap_RACH_SubChannelNumbers_subCh9, -1, -1, "subCh9", NULL },
  {  3, &hf_nbap_RACH_SubChannelNumbers_subCh8, -1, -1, "subCh8", NULL },
  {  4, &hf_nbap_RACH_SubChannelNumbers_subCh7, -1, -1, "subCh7", NULL },
  {  5, &hf_nbap_RACH_SubChannelNumbers_subCh6, -1, -1, "subCh6", NULL },
  {  6, &hf_nbap_RACH_SubChannelNumbers_subCh5, -1, -1, "subCh5", NULL },
  {  7, &hf_nbap_RACH_SubChannelNumbers_subCh4, -1, -1, "subCh4", NULL },
  {  8, &hf_nbap_RACH_SubChannelNumbers_subCh3, -1, -1, "subCh3", NULL },
  {  9, &hf_nbap_RACH_SubChannelNumbers_subCh2, -1, -1, "subCh2", NULL },
  { 10, &hf_nbap_RACH_SubChannelNumbers_subCh1, -1, -1, "subCh1", NULL },
  { 11, &hf_nbap_RACH_SubChannelNumbers_subCh0, -1, -1, "subCh0", NULL },
  { 0, NULL, 0, 0, NULL, NULL }
};

static int
dissect_nbap_RACH_SubChannelNumbers(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     12, 12, FALSE, NULL);

  return offset;
}
static int dissect_rACH_SubChannelNumbers(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RACH_SubChannelNumbers(tvb, offset, actx, tree, hf_nbap_rACH_SubChannelNumbers);
}



static int
dissect_nbap_PunctureLimit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 15U, NULL, FALSE);

  return offset;
}
static int dissect_ul_punctureLimit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PunctureLimit(tvb, offset, actx, tree, hf_nbap_ul_punctureLimit);
}
static int dissect_punctureLimit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PunctureLimit(tvb, offset, actx, tree, hf_nbap_punctureLimit);
}
static int dissect_ul_PunctureLimit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PunctureLimit(tvb, offset, actx, tree, hf_nbap_ul_PunctureLimit);
}



static int
dissect_nbap_PreambleThreshold(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 72U, NULL, FALSE);

  return offset;
}
static int dissect_preambleThreshold(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PreambleThreshold(tvb, offset, actx, tree, hf_nbap_preambleThreshold);
}



static int
dissect_nbap_RACH_Parameters_CTCH_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_rACH_Parameters(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RACH_Parameters_CTCH_SetupRqstFDD(tvb, offset, actx, tree, hf_nbap_rACH_Parameters);
}


static const value_string nbap_AICH_TransmissionTiming_vals[] = {
  {   0, "v0" },
  {   1, "v1" },
  { 0, NULL }
};


static int
dissect_nbap_AICH_TransmissionTiming(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_aICH_TransmissionTiming(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_AICH_TransmissionTiming(tvb, offset, actx, tree, hf_nbap_aICH_TransmissionTiming);
}



static int
dissect_nbap_AICH_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              -22, 5U, NULL, FALSE);

  return offset;
}
static int dissect_aICH_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_AICH_Power(tvb, offset, actx, tree, hf_nbap_aICH_Power);
}


static const per_sequence_t AICH_Parameters_CTCH_SetupRqstFDD_sequence[] = {
  { &hf_nbap_commonPhysicalChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonPhysicalChannelID },
  { &hf_nbap_aICH_TransmissionTiming, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_AICH_TransmissionTiming },
  { &hf_nbap_fdd_dl_ChannelisationCodeNumber, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_FDD_DL_ChannelisationCodeNumber },
  { &hf_nbap_aICH_Power     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_AICH_Power },
  { &hf_nbap_sTTD_Indicator , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_STTD_Indicator },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_AICH_Parameters_CTCH_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_AICH_Parameters_CTCH_SetupRqstFDD, AICH_Parameters_CTCH_SetupRqstFDD_sequence);

  return offset;
}
static int dissect_aICH_Parameters(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_AICH_Parameters_CTCH_SetupRqstFDD(tvb, offset, actx, tree, hf_nbap_aICH_Parameters);
}


static const per_sequence_t PRACH_CTCH_SetupRqstFDD_sequence[] = {
  { &hf_nbap_commonPhysicalChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonPhysicalChannelID },
  { &hf_nbap_scramblingCodeNumber, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ScramblingCodeNumber },
  { &hf_nbap_tFCS           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TFCS },
  { &hf_nbap_preambleSignatures, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PreambleSignatures },
  { &hf_nbap_allowedSlotFormatInformation, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_AllowedSlotFormatInformationList_CTCH_SetupRqstFDD },
  { &hf_nbap_rACH_SubChannelNumbers, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RACH_SubChannelNumbers },
  { &hf_nbap_ul_punctureLimit, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PunctureLimit },
  { &hf_nbap_preambleThreshold, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PreambleThreshold },
  { &hf_nbap_rACH_Parameters, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RACH_Parameters_CTCH_SetupRqstFDD },
  { &hf_nbap_aICH_Parameters, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_AICH_Parameters_CTCH_SetupRqstFDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PRACH_CTCH_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PRACH_CTCH_SetupRqstFDD, PRACH_CTCH_SetupRqstFDD_sequence);

  return offset;
}
static int dissect_pRACH_parameters(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PRACH_CTCH_SetupRqstFDD(tvb, offset, actx, tree, hf_nbap_pRACH_parameters);
}


static const value_string nbap_CommonPhysicalChannelType_CTCH_SetupRqstFDD_vals[] = {
  {   0, "secondary-CCPCH-parameters" },
  {   1, "pRACH-parameters" },
  {   2, "notUsed-pCPCHes-parameters" },
  { 0, NULL }
};

static const per_choice_t CommonPhysicalChannelType_CTCH_SetupRqstFDD_choice[] = {
  {   0, &hf_nbap_secondary_CCPCH_parameters, ASN1_EXTENSION_ROOT    , dissect_nbap_Secondary_CCPCH_CTCH_SetupRqstFDD },
  {   1, &hf_nbap_pRACH_parameters, ASN1_EXTENSION_ROOT    , dissect_nbap_PRACH_CTCH_SetupRqstFDD },
  {   2, &hf_nbap_notUsed_pCPCHes_parameters, ASN1_EXTENSION_ROOT    , dissect_nbap_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_CommonPhysicalChannelType_CTCH_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_CommonPhysicalChannelType_CTCH_SetupRqstFDD, CommonPhysicalChannelType_CTCH_SetupRqstFDD_choice,
                                 NULL);

  return offset;
}
static int dissect_id_CommonPhysicalChannelType_CTCH_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommonPhysicalChannelType_CTCH_SetupRqstFDD(tvb, offset, actx, tree, hf_nbap_id_CommonPhysicalChannelType_CTCH_SetupRqstFDD);
}



static int
dissect_nbap_CCTrCH_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 15U, NULL, FALSE);

  return offset;
}
static int dissect_sCCPCH_CCTrCH_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CCTrCH_ID(tvb, offset, actx, tree, hf_nbap_sCCPCH_CCTrCH_ID);
}
static int dissect_fACH_CCTrCH_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CCTrCH_ID(tvb, offset, actx, tree, hf_nbap_fACH_CCTrCH_ID);
}
static int dissect_pCH_CCTrCH_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CCTrCH_ID(tvb, offset, actx, tree, hf_nbap_pCH_CCTrCH_ID);
}
static int dissect_cCTrCH_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CCTrCH_ID(tvb, offset, actx, tree, hf_nbap_cCTrCH_ID);
}
static int dissect_ul_CCTrCH_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CCTrCH_ID(tvb, offset, actx, tree, hf_nbap_ul_CCTrCH_ID);
}
static int dissect_dl_CCTrCH_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CCTrCH_ID(tvb, offset, actx, tree, hf_nbap_dl_CCTrCH_ID);
}


static const value_string nbap_TFCI_Coding_vals[] = {
  {   0, "v4" },
  {   1, "v8" },
  {   2, "v16" },
  {   3, "v32" },
  { 0, NULL }
};


static int
dissect_nbap_TFCI_Coding(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_tFCI_Coding(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TFCI_Coding(tvb, offset, actx, tree, hf_nbap_tFCI_Coding);
}



static int
dissect_nbap_Secondary_CCPCH_parameterList_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_secondaryCCPCH_parameterList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Secondary_CCPCH_parameterList_CTCH_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_secondaryCCPCH_parameterList);
}



static int
dissect_nbap_FACH_ParametersList_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_fACH_ParametersList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_FACH_ParametersList_CTCH_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_fACH_ParametersList);
}



static int
dissect_nbap_PCH_Parameters_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_pCH_Parameters1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PCH_Parameters_CTCH_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_pCH_Parameters1);
}


static const per_sequence_t Secondary_CCPCH_CTCH_SetupRqstTDD_sequence[] = {
  { &hf_nbap_sCCPCH_CCTrCH_ID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CCTrCH_ID },
  { &hf_nbap_tFCS           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TFCS },
  { &hf_nbap_tFCI_Coding    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TFCI_Coding },
  { &hf_nbap_punctureLimit  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PunctureLimit },
  { &hf_nbap_secondaryCCPCH_parameterList, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Secondary_CCPCH_parameterList_CTCH_SetupRqstTDD },
  { &hf_nbap_fACH_ParametersList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_FACH_ParametersList_CTCH_SetupRqstTDD },
  { &hf_nbap_pCH_Parameters1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_PCH_Parameters_CTCH_SetupRqstTDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Secondary_CCPCH_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Secondary_CCPCH_CTCH_SetupRqstTDD, Secondary_CCPCH_CTCH_SetupRqstTDD_sequence);

  return offset;
}
static int dissect_secondary_CCPCH_parameters1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Secondary_CCPCH_CTCH_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_secondary_CCPCH_parameters1);
}



static int
dissect_nbap_PRACH_Parameters_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_pRACH_Parameters_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PRACH_Parameters_CTCH_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_pRACH_Parameters_CTCH_SetupRqstTDD);
}


static const per_sequence_t PRACH_CTCH_SetupRqstTDD_sequence[] = {
  { &hf_nbap_pRACH_Parameters_CTCH_SetupRqstTDD, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PRACH_Parameters_CTCH_SetupRqstTDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PRACH_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PRACH_CTCH_SetupRqstTDD, PRACH_CTCH_SetupRqstTDD_sequence);

  return offset;
}
static int dissect_pRACH_parameters1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PRACH_CTCH_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_pRACH_parameters1);
}


static const value_string nbap_CommonPhysicalChannelType_CTCH_SetupRqstTDD_vals[] = {
  {   0, "secondary-CCPCH-parameters" },
  {   1, "pRACH-parameters" },
  { 0, NULL }
};

static const per_choice_t CommonPhysicalChannelType_CTCH_SetupRqstTDD_choice[] = {
  {   0, &hf_nbap_secondary_CCPCH_parameters1, ASN1_EXTENSION_ROOT    , dissect_nbap_Secondary_CCPCH_CTCH_SetupRqstTDD },
  {   1, &hf_nbap_pRACH_parameters1, ASN1_EXTENSION_ROOT    , dissect_nbap_PRACH_CTCH_SetupRqstTDD },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_CommonPhysicalChannelType_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_CommonPhysicalChannelType_CTCH_SetupRqstTDD, CommonPhysicalChannelType_CTCH_SetupRqstTDD_choice,
                                 NULL);

  return offset;
}
static int dissect_id_CommonPhysicalChannelType_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommonPhysicalChannelType_CTCH_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_id_CommonPhysicalChannelType_CTCH_SetupRqstTDD);
}



static int
dissect_nbap_CRNC_CommunicationContextID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 1048575U, NULL, FALSE);

  return offset;
}
static int dissect_id_CRNC_CommunicationContextID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CRNC_CommunicationContextID(tvb, offset, actx, tree, hf_nbap_id_CRNC_CommunicationContextID);
}
static int dissect_cRNC_CommunicationContextID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CRNC_CommunicationContextID(tvb, offset, actx, tree, hf_nbap_cRNC_CommunicationContextID);
}


static const value_string nbap_TriggeringMessage_vals[] = {
  {   0, "initiating-message" },
  {   1, "successful-outcome" },
  {   2, "unsuccessfull-outcome" },
  {   3, "outcome" },
  { 0, NULL }
};


static int
dissect_nbap_TriggeringMessage(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_triggeringMessage(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TriggeringMessage(tvb, offset, actx, tree, hf_nbap_triggeringMessage);
}



static int
dissect_nbap_RepetitionNumber0(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 255U, NULL, FALSE);

  return offset;
}
static int dissect_repetitionNumber(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RepetitionNumber0(tvb, offset, actx, tree, hf_nbap_repetitionNumber);
}


static const per_sequence_t CriticalityDiagnostics_IE_List_item_sequence[] = {
  { &hf_nbap_iECriticality  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Criticality },
  { &hf_nbap_iE_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_ID },
  { &hf_nbap_repetitionNumber, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_RepetitionNumber0 },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CriticalityDiagnostics_IE_List_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CriticalityDiagnostics_IE_List_item, CriticalityDiagnostics_IE_List_item_sequence);

  return offset;
}
static int dissect_CriticalityDiagnostics_IE_List_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CriticalityDiagnostics_IE_List_item(tvb, offset, actx, tree, hf_nbap_CriticalityDiagnostics_IE_List_item);
}


static const per_sequence_t CriticalityDiagnostics_IE_List_sequence_of[1] = {
  { &hf_nbap_CriticalityDiagnostics_IE_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_CriticalityDiagnostics_IE_List_item },
};

static int
dissect_nbap_CriticalityDiagnostics_IE_List(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_CriticalityDiagnostics_IE_List, CriticalityDiagnostics_IE_List_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_iEsCriticalityDiagnostics(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CriticalityDiagnostics_IE_List(tvb, offset, actx, tree, hf_nbap_iEsCriticalityDiagnostics);
}


static const per_sequence_t CriticalityDiagnostics_sequence[] = {
  { &hf_nbap_procedureID    , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProcedureID },
  { &hf_nbap_triggeringMessage, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TriggeringMessage },
  { &hf_nbap_procedureCriticality, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_Criticality },
  { &hf_nbap_transactionID  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TransactionID },
  { &hf_nbap_iEsCriticalityDiagnostics, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_CriticalityDiagnostics_IE_List },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CriticalityDiagnostics(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CriticalityDiagnostics, CriticalityDiagnostics_sequence);

  return offset;
}
static int dissect_id_CriticalityDiagnostics(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CriticalityDiagnostics(tvb, offset, actx, tree, hf_nbap_id_CriticalityDiagnostics);
}


static const value_string nbap_PayloadCRC_PresenceIndicator_vals[] = {
  {   0, "cRC-Included" },
  {   1, "cRC-NotIncluded" },
  { 0, NULL }
};


static int
dissect_nbap_PayloadCRC_PresenceIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_payloadCRC_PresenceIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PayloadCRC_PresenceIndicator(tvb, offset, actx, tree, hf_nbap_payloadCRC_PresenceIndicator);
}


static const value_string nbap_UL_FP_Mode_vals[] = {
  {   0, "normal" },
  {   1, "silent" },
  { 0, NULL }
};


static int
dissect_nbap_UL_FP_Mode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_ul_FP_Mode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_FP_Mode(tvb, offset, actx, tree, hf_nbap_ul_FP_Mode);
}



static int
dissect_nbap_ToAWS(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 1279U, NULL, FALSE);

  return offset;
}
static int dissect_toAWS(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ToAWS(tvb, offset, actx, tree, hf_nbap_toAWS);
}



static int
dissect_nbap_ToAWE(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 2559U, NULL, FALSE);

  return offset;
}
static int dissect_toAWE(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ToAWE(tvb, offset, actx, tree, hf_nbap_toAWE);
}



static int
dissect_nbap_DCH_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 255U, NULL, FALSE);

  return offset;
}
static int dissect_dCH_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DCH_ID(tvb, offset, actx, tree, hf_nbap_dCH_ID);
}
static int dissect_dCH_id(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DCH_ID(tvb, offset, actx, tree, hf_nbap_dCH_id);
}



static int
dissect_nbap_TransportFormatSet_NrOfTransportBlocks(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 512U, NULL, FALSE);

  return offset;
}
static int dissect_nrOfTransportBlocks(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TransportFormatSet_NrOfTransportBlocks(tvb, offset, actx, tree, hf_nbap_nrOfTransportBlocks);
}



static int
dissect_nbap_TransportFormatSet_TransportBlockSize(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 5000U, NULL, FALSE);

  return offset;
}
static int dissect_transportBlockSize(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TransportFormatSet_TransportBlockSize(tvb, offset, actx, tree, hf_nbap_transportBlockSize);
}


static const value_string nbap_TransportFormatSet_TransmissionTimeIntervalDynamic_vals[] = {
  {   0, "msec-10" },
  {   1, "msec-20" },
  {   2, "msec-40" },
  {   3, "msec-80" },
  { 0, NULL }
};


static int
dissect_nbap_TransportFormatSet_TransmissionTimeIntervalDynamic(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_transmissionTimeInterval(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TransportFormatSet_TransmissionTimeIntervalDynamic(tvb, offset, actx, tree, hf_nbap_transmissionTimeInterval);
}


static const per_sequence_t TransmissionTimeIntervalInformation_item_sequence[] = {
  { &hf_nbap_transmissionTimeInterval, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TransportFormatSet_TransmissionTimeIntervalDynamic },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_TransmissionTimeIntervalInformation_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_TransmissionTimeIntervalInformation_item, TransmissionTimeIntervalInformation_item_sequence);

  return offset;
}
static int dissect_TransmissionTimeIntervalInformation_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TransmissionTimeIntervalInformation_item(tvb, offset, actx, tree, hf_nbap_TransmissionTimeIntervalInformation_item);
}


static const per_sequence_t TransmissionTimeIntervalInformation_sequence_of[1] = {
  { &hf_nbap_TransmissionTimeIntervalInformation_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_TransmissionTimeIntervalInformation_item },
};

static int
dissect_nbap_TransmissionTimeIntervalInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_TransmissionTimeIntervalInformation, TransmissionTimeIntervalInformation_sequence_of,
                                                  1, 4);

  return offset;
}
static int dissect_transmissionTimeIntervalInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TransmissionTimeIntervalInformation(tvb, offset, actx, tree, hf_nbap_transmissionTimeIntervalInformation);
}


static const per_sequence_t TDD_TransportFormatSet_ModeDP_sequence[] = {
  { &hf_nbap_transmissionTimeIntervalInformation, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TransmissionTimeIntervalInformation },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_TDD_TransportFormatSet_ModeDP(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_TDD_TransportFormatSet_ModeDP, TDD_TransportFormatSet_ModeDP_sequence);

  return offset;
}
static int dissect_tdd1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TDD_TransportFormatSet_ModeDP(tvb, offset, actx, tree, hf_nbap_tdd1);
}


static const value_string nbap_TransportFormatSet_ModeDP_vals[] = {
  {   0, "tdd" },
  {   1, "notApplicable" },
  { 0, NULL }
};

static const per_choice_t TransportFormatSet_ModeDP_choice[] = {
  {   0, &hf_nbap_tdd1           , ASN1_EXTENSION_ROOT    , dissect_nbap_TDD_TransportFormatSet_ModeDP },
  {   1, &hf_nbap_notApplicable  , ASN1_EXTENSION_ROOT    , dissect_nbap_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_TransportFormatSet_ModeDP(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_TransportFormatSet_ModeDP, TransportFormatSet_ModeDP_choice,
                                 NULL);

  return offset;
}
static int dissect_mode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TransportFormatSet_ModeDP(tvb, offset, actx, tree, hf_nbap_mode);
}


static const per_sequence_t TransportFormatSet_DynamicPartList_item_sequence[] = {
  { &hf_nbap_nrOfTransportBlocks, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TransportFormatSet_NrOfTransportBlocks },
  { &hf_nbap_transportBlockSize, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TransportFormatSet_TransportBlockSize },
  { &hf_nbap_mode           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TransportFormatSet_ModeDP },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_TransportFormatSet_DynamicPartList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_TransportFormatSet_DynamicPartList_item, TransportFormatSet_DynamicPartList_item_sequence);

  return offset;
}
static int dissect_TransportFormatSet_DynamicPartList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TransportFormatSet_DynamicPartList_item(tvb, offset, actx, tree, hf_nbap_TransportFormatSet_DynamicPartList_item);
}


static const per_sequence_t TransportFormatSet_DynamicPartList_sequence_of[1] = {
  { &hf_nbap_TransportFormatSet_DynamicPartList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_TransportFormatSet_DynamicPartList_item },
};

static int
dissect_nbap_TransportFormatSet_DynamicPartList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_TransportFormatSet_DynamicPartList, TransportFormatSet_DynamicPartList_sequence_of,
                                                  1, 32);

  return offset;
}
static int dissect_dynamicParts(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TransportFormatSet_DynamicPartList(tvb, offset, actx, tree, hf_nbap_dynamicParts);
}


static const value_string nbap_TransportFormatSet_TransmissionTimeIntervalSemiStatic_vals[] = {
  {   0, "msec-10" },
  {   1, "msec-20" },
  {   2, "msec-40" },
  {   3, "msec-80" },
  {   4, "dynamic" },
  {   5, "msec-5" },
  { 0, NULL }
};


static int
dissect_nbap_TransportFormatSet_TransmissionTimeIntervalSemiStatic(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     5, NULL, TRUE, 1, NULL);

  return offset;
}
static int dissect_transmissionTimeInterval1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TransportFormatSet_TransmissionTimeIntervalSemiStatic(tvb, offset, actx, tree, hf_nbap_transmissionTimeInterval1);
}


static const value_string nbap_TransportFormatSet_ChannelCodingType_vals[] = {
  {   0, "no-codingTDD" },
  {   1, "convolutional-coding" },
  {   2, "turbo-coding" },
  { 0, NULL }
};


static int
dissect_nbap_TransportFormatSet_ChannelCodingType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_channelCoding(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TransportFormatSet_ChannelCodingType(tvb, offset, actx, tree, hf_nbap_channelCoding);
}


static const value_string nbap_TransportFormatSet_CodingRate_vals[] = {
  {   0, "half" },
  {   1, "third" },
  { 0, NULL }
};


static int
dissect_nbap_TransportFormatSet_CodingRate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_codingRate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TransportFormatSet_CodingRate(tvb, offset, actx, tree, hf_nbap_codingRate);
}



static int
dissect_nbap_TransportFormatSet_RateMatchingAttribute(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 256U, NULL, FALSE);

  return offset;
}
static int dissect_rateMatchingAttribute(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TransportFormatSet_RateMatchingAttribute(tvb, offset, actx, tree, hf_nbap_rateMatchingAttribute);
}


static const value_string nbap_TransportFormatSet_CRC_Size_vals[] = {
  {   0, "v0" },
  {   1, "v8" },
  {   2, "v12" },
  {   3, "v16" },
  {   4, "v24" },
  { 0, NULL }
};


static int
dissect_nbap_TransportFormatSet_CRC_Size(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     5, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_cRC_Size(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TransportFormatSet_CRC_Size(tvb, offset, actx, tree, hf_nbap_cRC_Size);
}


static const value_string nbap_TransportFormatSet_SecondInterleavingMode_vals[] = {
  {   0, "frame-related" },
  {   1, "timeSlot-related" },
  { 0, NULL }
};


static int
dissect_nbap_TransportFormatSet_SecondInterleavingMode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_tdd2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TransportFormatSet_SecondInterleavingMode(tvb, offset, actx, tree, hf_nbap_tdd2);
}


static const value_string nbap_TransportFormatSet_ModeSSP_vals[] = {
  {   0, "tdd" },
  {   1, "notApplicable" },
  { 0, NULL }
};

static const per_choice_t TransportFormatSet_ModeSSP_choice[] = {
  {   0, &hf_nbap_tdd2           , ASN1_EXTENSION_ROOT    , dissect_nbap_TransportFormatSet_SecondInterleavingMode },
  {   1, &hf_nbap_notApplicable  , ASN1_EXTENSION_ROOT    , dissect_nbap_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_TransportFormatSet_ModeSSP(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_TransportFormatSet_ModeSSP, TransportFormatSet_ModeSSP_choice,
                                 NULL);

  return offset;
}
static int dissect_mode1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TransportFormatSet_ModeSSP(tvb, offset, actx, tree, hf_nbap_mode1);
}


static const per_sequence_t TransportFormatSet_Semi_staticPart_sequence[] = {
  { &hf_nbap_transmissionTimeInterval1, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TransportFormatSet_TransmissionTimeIntervalSemiStatic },
  { &hf_nbap_channelCoding  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TransportFormatSet_ChannelCodingType },
  { &hf_nbap_codingRate     , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TransportFormatSet_CodingRate },
  { &hf_nbap_rateMatchingAttribute, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TransportFormatSet_RateMatchingAttribute },
  { &hf_nbap_cRC_Size       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TransportFormatSet_CRC_Size },
  { &hf_nbap_mode1          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TransportFormatSet_ModeSSP },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_TransportFormatSet_Semi_staticPart(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_TransportFormatSet_Semi_staticPart, TransportFormatSet_Semi_staticPart_sequence);

  return offset;
}
static int dissect_semi_staticPart(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TransportFormatSet_Semi_staticPart(tvb, offset, actx, tree, hf_nbap_semi_staticPart);
}


static const per_sequence_t TransportFormatSet_sequence[] = {
  { &hf_nbap_dynamicParts   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TransportFormatSet_DynamicPartList },
  { &hf_nbap_semi_staticPart, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TransportFormatSet_Semi_staticPart },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_TransportFormatSet(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_TransportFormatSet, TransportFormatSet_sequence);

  return offset;
}
static int dissect_transportFormatSet(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TransportFormatSet(tvb, offset, actx, tree, hf_nbap_transportFormatSet);
}
static int dissect_dl_TransportFormatSet(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TransportFormatSet(tvb, offset, actx, tree, hf_nbap_dl_TransportFormatSet);
}
static int dissect_uL_TransportFormatSet(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TransportFormatSet(tvb, offset, actx, tree, hf_nbap_uL_TransportFormatSet);
}
static int dissect_ul_TransportFormatSet(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TransportFormatSet(tvb, offset, actx, tree, hf_nbap_ul_TransportFormatSet);
}



static int
dissect_nbap_PriorityLevel(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 15U, NULL, FALSE);

  return offset;
}
static int dissect_priorityLevel(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PriorityLevel(tvb, offset, actx, tree, hf_nbap_priorityLevel);
}


static const value_string nbap_Pre_emptionCapability_vals[] = {
  {   0, "shall-not-trigger-pre-emption" },
  {   1, "may-trigger-pre-emption" },
  { 0, NULL }
};


static int
dissect_nbap_Pre_emptionCapability(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_pre_emptionCapability(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Pre_emptionCapability(tvb, offset, actx, tree, hf_nbap_pre_emptionCapability);
}


static const value_string nbap_Pre_emptionVulnerability_vals[] = {
  {   0, "not-pre-emptable" },
  {   1, "pre-emptable" },
  { 0, NULL }
};


static int
dissect_nbap_Pre_emptionVulnerability(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_pre_emptionVulnerability(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Pre_emptionVulnerability(tvb, offset, actx, tree, hf_nbap_pre_emptionVulnerability);
}


static const per_sequence_t AllocationRetentionPriority_sequence[] = {
  { &hf_nbap_priorityLevel  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PriorityLevel },
  { &hf_nbap_pre_emptionCapability, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Pre_emptionCapability },
  { &hf_nbap_pre_emptionVulnerability, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Pre_emptionVulnerability },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_AllocationRetentionPriority(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_AllocationRetentionPriority, AllocationRetentionPriority_sequence);

  return offset;
}
static int dissect_allocationRetentionPriority(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_AllocationRetentionPriority(tvb, offset, actx, tree, hf_nbap_allocationRetentionPriority);
}



static int
dissect_nbap_FrameHandlingPriority(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 15U, NULL, FALSE);

  return offset;
}
static int dissect_frameHandlingPriority(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_FrameHandlingPriority(tvb, offset, actx, tree, hf_nbap_frameHandlingPriority);
}


static const value_string nbap_QE_Selector_vals[] = {
  {   0, "selected" },
  {   1, "non-selected" },
  { 0, NULL }
};


static int
dissect_nbap_QE_Selector(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_qE_Selector(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_QE_Selector(tvb, offset, actx, tree, hf_nbap_qE_Selector);
}


static const per_sequence_t DCH_Specific_FDD_Item_sequence[] = {
  { &hf_nbap_dCH_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DCH_ID },
  { &hf_nbap_ul_TransportFormatSet, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TransportFormatSet },
  { &hf_nbap_dl_TransportFormatSet, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TransportFormatSet },
  { &hf_nbap_allocationRetentionPriority, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_AllocationRetentionPriority },
  { &hf_nbap_frameHandlingPriority, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_FrameHandlingPriority },
  { &hf_nbap_qE_Selector    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_QE_Selector },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DCH_Specific_FDD_Item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DCH_Specific_FDD_Item, DCH_Specific_FDD_Item_sequence);

  return offset;
}
static int dissect_DCH_Specific_FDD_InformationList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DCH_Specific_FDD_Item(tvb, offset, actx, tree, hf_nbap_DCH_Specific_FDD_InformationList_item);
}


static const per_sequence_t DCH_Specific_FDD_InformationList_sequence_of[1] = {
  { &hf_nbap_DCH_Specific_FDD_InformationList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DCH_Specific_FDD_Item },
};

static int
dissect_nbap_DCH_Specific_FDD_InformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DCH_Specific_FDD_InformationList, DCH_Specific_FDD_InformationList_sequence_of,
                                                  1, 128);

  return offset;
}
static int dissect_dCH_SpecificInformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DCH_Specific_FDD_InformationList(tvb, offset, actx, tree, hf_nbap_dCH_SpecificInformationList);
}


static const per_sequence_t DCH_FDD_InformationItem_sequence[] = {
  { &hf_nbap_payloadCRC_PresenceIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PayloadCRC_PresenceIndicator },
  { &hf_nbap_ul_FP_Mode     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UL_FP_Mode },
  { &hf_nbap_toAWS          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ToAWS },
  { &hf_nbap_toAWE          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ToAWE },
  { &hf_nbap_dCH_SpecificInformationList, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DCH_Specific_FDD_InformationList },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DCH_FDD_InformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DCH_FDD_InformationItem, DCH_FDD_InformationItem_sequence);

  return offset;
}
static int dissect_DCH_FDD_Information_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DCH_FDD_InformationItem(tvb, offset, actx, tree, hf_nbap_DCH_FDD_Information_item);
}


static const per_sequence_t DCH_FDD_Information_sequence_of[1] = {
  { &hf_nbap_DCH_FDD_Information_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DCH_FDD_InformationItem },
};

static int
dissect_nbap_DCH_FDD_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DCH_FDD_Information, DCH_FDD_Information_sequence_of,
                                                  1, 128);

  return offset;
}
static int dissect_id_DCHs_to_Add_FDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DCH_FDD_Information(tvb, offset, actx, tree, hf_nbap_id_DCHs_to_Add_FDD);
}
static int dissect_id_DCH_FDD_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DCH_FDD_Information(tvb, offset, actx, tree, hf_nbap_id_DCH_FDD_Information);
}


static const per_sequence_t DCH_Specific_TDD_Item_sequence[] = {
  { &hf_nbap_dCH_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DCH_ID },
  { &hf_nbap_ul_CCTrCH_ID   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CCTrCH_ID },
  { &hf_nbap_dl_CCTrCH_ID   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CCTrCH_ID },
  { &hf_nbap_ul_TransportFormatSet, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TransportFormatSet },
  { &hf_nbap_dl_TransportFormatSet, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TransportFormatSet },
  { &hf_nbap_allocationRetentionPriority, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_AllocationRetentionPriority },
  { &hf_nbap_frameHandlingPriority, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_FrameHandlingPriority },
  { &hf_nbap_qE_Selector    , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_QE_Selector },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DCH_Specific_TDD_Item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DCH_Specific_TDD_Item, DCH_Specific_TDD_Item_sequence);

  return offset;
}
static int dissect_DCH_Specific_TDD_InformationList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DCH_Specific_TDD_Item(tvb, offset, actx, tree, hf_nbap_DCH_Specific_TDD_InformationList_item);
}


static const per_sequence_t DCH_Specific_TDD_InformationList_sequence_of[1] = {
  { &hf_nbap_DCH_Specific_TDD_InformationList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DCH_Specific_TDD_Item },
};

static int
dissect_nbap_DCH_Specific_TDD_InformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DCH_Specific_TDD_InformationList, DCH_Specific_TDD_InformationList_sequence_of,
                                                  1, 128);

  return offset;
}
static int dissect_dCH_SpecificInformationList1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DCH_Specific_TDD_InformationList(tvb, offset, actx, tree, hf_nbap_dCH_SpecificInformationList1);
}


static const per_sequence_t DCH_TDD_InformationItem_sequence[] = {
  { &hf_nbap_payloadCRC_PresenceIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PayloadCRC_PresenceIndicator },
  { &hf_nbap_ul_FP_Mode     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UL_FP_Mode },
  { &hf_nbap_toAWS          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ToAWS },
  { &hf_nbap_toAWE          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ToAWE },
  { &hf_nbap_dCH_SpecificInformationList1, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DCH_Specific_TDD_InformationList },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DCH_TDD_InformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DCH_TDD_InformationItem, DCH_TDD_InformationItem_sequence);

  return offset;
}
static int dissect_DCH_TDD_Information_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DCH_TDD_InformationItem(tvb, offset, actx, tree, hf_nbap_DCH_TDD_Information_item);
}


static const per_sequence_t DCH_TDD_Information_sequence_of[1] = {
  { &hf_nbap_DCH_TDD_Information_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DCH_TDD_InformationItem },
};

static int
dissect_nbap_DCH_TDD_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DCH_TDD_Information, DCH_TDD_Information_sequence_of,
                                                  1, 128);

  return offset;
}
static int dissect_id_DCHs_to_Add_TDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DCH_TDD_Information(tvb, offset, actx, tree, hf_nbap_id_DCHs_to_Add_TDD);
}
static int dissect_id_DCH_TDD_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DCH_TDD_Information(tvb, offset, actx, tree, hf_nbap_id_DCH_TDD_Information);
}


static const per_sequence_t DCH_DeleteItem_RL_ReconfPrepFDD_sequence[] = {
  { &hf_nbap_dCH_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DCH_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DCH_DeleteItem_RL_ReconfPrepFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DCH_DeleteItem_RL_ReconfPrepFDD, DCH_DeleteItem_RL_ReconfPrepFDD_sequence);

  return offset;
}
static int dissect_DCH_DeleteList_RL_ReconfPrepFDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DCH_DeleteItem_RL_ReconfPrepFDD(tvb, offset, actx, tree, hf_nbap_DCH_DeleteList_RL_ReconfPrepFDD_item);
}


static const per_sequence_t DCH_DeleteList_RL_ReconfPrepFDD_sequence_of[1] = {
  { &hf_nbap_DCH_DeleteList_RL_ReconfPrepFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DCH_DeleteItem_RL_ReconfPrepFDD },
};

static int
dissect_nbap_DCH_DeleteList_RL_ReconfPrepFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DCH_DeleteList_RL_ReconfPrepFDD, DCH_DeleteList_RL_ReconfPrepFDD_sequence_of,
                                                  1, 128);

  return offset;
}
static int dissect_id_DCH_DeleteList_RL_ReconfPrepFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DCH_DeleteList_RL_ReconfPrepFDD(tvb, offset, actx, tree, hf_nbap_id_DCH_DeleteList_RL_ReconfPrepFDD);
}


static const per_sequence_t DCH_DeleteItem_RL_ReconfPrepTDD_sequence[] = {
  { &hf_nbap_dCH_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DCH_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DCH_DeleteItem_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DCH_DeleteItem_RL_ReconfPrepTDD, DCH_DeleteItem_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_DCH_DeleteList_RL_ReconfPrepTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DCH_DeleteItem_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_DCH_DeleteList_RL_ReconfPrepTDD_item);
}


static const per_sequence_t DCH_DeleteList_RL_ReconfPrepTDD_sequence_of[1] = {
  { &hf_nbap_DCH_DeleteList_RL_ReconfPrepTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DCH_DeleteItem_RL_ReconfPrepTDD },
};

static int
dissect_nbap_DCH_DeleteList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DCH_DeleteList_RL_ReconfPrepTDD, DCH_DeleteList_RL_ReconfPrepTDD_sequence_of,
                                                  1, 128);

  return offset;
}
static int dissect_id_DCH_DeleteList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DCH_DeleteList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_id_DCH_DeleteList_RL_ReconfPrepTDD);
}


static const per_sequence_t DCH_DeleteItem_RL_ReconfRqstFDD_sequence[] = {
  { &hf_nbap_dCH_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DCH_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DCH_DeleteItem_RL_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DCH_DeleteItem_RL_ReconfRqstFDD, DCH_DeleteItem_RL_ReconfRqstFDD_sequence);

  return offset;
}
static int dissect_DCH_DeleteList_RL_ReconfRqstFDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DCH_DeleteItem_RL_ReconfRqstFDD(tvb, offset, actx, tree, hf_nbap_DCH_DeleteList_RL_ReconfRqstFDD_item);
}


static const per_sequence_t DCH_DeleteList_RL_ReconfRqstFDD_sequence_of[1] = {
  { &hf_nbap_DCH_DeleteList_RL_ReconfRqstFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DCH_DeleteItem_RL_ReconfRqstFDD },
};

static int
dissect_nbap_DCH_DeleteList_RL_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DCH_DeleteList_RL_ReconfRqstFDD, DCH_DeleteList_RL_ReconfRqstFDD_sequence_of,
                                                  1, 128);

  return offset;
}
static int dissect_id_DCH_DeleteList_RL_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DCH_DeleteList_RL_ReconfRqstFDD(tvb, offset, actx, tree, hf_nbap_id_DCH_DeleteList_RL_ReconfRqstFDD);
}


static const per_sequence_t DCH_DeleteItem_RL_ReconfRqstTDD_sequence[] = {
  { &hf_nbap_dCH_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DCH_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DCH_DeleteItem_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DCH_DeleteItem_RL_ReconfRqstTDD, DCH_DeleteItem_RL_ReconfRqstTDD_sequence);

  return offset;
}
static int dissect_DCH_DeleteList_RL_ReconfRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DCH_DeleteItem_RL_ReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_DCH_DeleteList_RL_ReconfRqstTDD_item);
}


static const per_sequence_t DCH_DeleteList_RL_ReconfRqstTDD_sequence_of[1] = {
  { &hf_nbap_DCH_DeleteList_RL_ReconfRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DCH_DeleteItem_RL_ReconfRqstTDD },
};

static int
dissect_nbap_DCH_DeleteList_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DCH_DeleteList_RL_ReconfRqstTDD, DCH_DeleteList_RL_ReconfRqstTDD_sequence_of,
                                                  1, 128);

  return offset;
}
static int dissect_id_DCH_DeleteList_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DCH_DeleteList_RL_ReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_id_DCH_DeleteList_RL_ReconfRqstTDD);
}



static int
dissect_nbap_BindingID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_octet_string(tvb, offset, actx, tree, hf_index,
                                       1, 4, NULL);

  return offset;
}
static int dissect_id_bindingID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BindingID(tvb, offset, actx, tree, hf_nbap_id_bindingID);
}
static int dissect_bindingID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BindingID(tvb, offset, actx, tree, hf_nbap_bindingID);
}



static int
dissect_nbap_TransportLayerAddress(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     1, 160, TRUE, NULL);

  return offset;
}
static int dissect_id_transportlayeraddress(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TransportLayerAddress(tvb, offset, actx, tree, hf_nbap_id_transportlayeraddress);
}
static int dissect_transportLayerAddress(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TransportLayerAddress(tvb, offset, actx, tree, hf_nbap_transportLayerAddress);
}
static int dissect_transportlayeraddress(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TransportLayerAddress(tvb, offset, actx, tree, hf_nbap_transportlayeraddress);
}


static const per_sequence_t DCH_InformationResponseItem_sequence[] = {
  { &hf_nbap_dCH_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DCH_ID },
  { &hf_nbap_bindingID      , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_BindingID },
  { &hf_nbap_transportLayerAddress, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TransportLayerAddress },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DCH_InformationResponseItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DCH_InformationResponseItem, DCH_InformationResponseItem_sequence);

  return offset;
}
static int dissect_DCH_InformationResponse_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DCH_InformationResponseItem(tvb, offset, actx, tree, hf_nbap_DCH_InformationResponse_item);
}


static const per_sequence_t DCH_InformationResponse_sequence_of[1] = {
  { &hf_nbap_DCH_InformationResponse_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DCH_InformationResponseItem },
};

static int
dissect_nbap_DCH_InformationResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DCH_InformationResponse, DCH_InformationResponse_sequence_of,
                                                  1, 128);

  return offset;
}
static int dissect_id_DCH_InformationResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DCH_InformationResponse(tvb, offset, actx, tree, hf_nbap_id_DCH_InformationResponse);
}
static int dissect_dCH_InformationResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DCH_InformationResponse(tvb, offset, actx, tree, hf_nbap_dCH_InformationResponse);
}


static const value_string nbap_TransportBearerRequestIndicator_vals[] = {
  {   0, "bearerRequested" },
  {   1, "bearerNotRequested" },
  { 0, NULL }
};


static int
dissect_nbap_TransportBearerRequestIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_transportBearerRequestIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TransportBearerRequestIndicator(tvb, offset, actx, tree, hf_nbap_transportBearerRequestIndicator);
}


static const per_sequence_t DCH_ModifySpecificItem_FDD_sequence[] = {
  { &hf_nbap_dCH_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DCH_ID },
  { &hf_nbap_ul_TransportFormatSet, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TransportFormatSet },
  { &hf_nbap_dl_TransportFormatSet, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TransportFormatSet },
  { &hf_nbap_allocationRetentionPriority, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_AllocationRetentionPriority },
  { &hf_nbap_frameHandlingPriority, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_FrameHandlingPriority },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DCH_ModifySpecificItem_FDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DCH_ModifySpecificItem_FDD, DCH_ModifySpecificItem_FDD_sequence);

  return offset;
}
static int dissect_DCH_ModifySpecificInformation_FDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DCH_ModifySpecificItem_FDD(tvb, offset, actx, tree, hf_nbap_DCH_ModifySpecificInformation_FDD_item);
}


static const per_sequence_t DCH_ModifySpecificInformation_FDD_sequence_of[1] = {
  { &hf_nbap_DCH_ModifySpecificInformation_FDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DCH_ModifySpecificItem_FDD },
};

static int
dissect_nbap_DCH_ModifySpecificInformation_FDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DCH_ModifySpecificInformation_FDD, DCH_ModifySpecificInformation_FDD_sequence_of,
                                                  1, 128);

  return offset;
}
static int dissect_dCH_SpecificInformationList2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DCH_ModifySpecificInformation_FDD(tvb, offset, actx, tree, hf_nbap_dCH_SpecificInformationList2);
}


static const per_sequence_t FDD_DCHs_to_ModifyItem_sequence[] = {
  { &hf_nbap_ul_FP_Mode     , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_UL_FP_Mode },
  { &hf_nbap_toAWS          , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ToAWS },
  { &hf_nbap_toAWE          , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ToAWE },
  { &hf_nbap_transportBearerRequestIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TransportBearerRequestIndicator },
  { &hf_nbap_dCH_SpecificInformationList2, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DCH_ModifySpecificInformation_FDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_FDD_DCHs_to_ModifyItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_FDD_DCHs_to_ModifyItem, FDD_DCHs_to_ModifyItem_sequence);

  return offset;
}
static int dissect_FDD_DCHs_to_Modify_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_FDD_DCHs_to_ModifyItem(tvb, offset, actx, tree, hf_nbap_FDD_DCHs_to_Modify_item);
}


static const per_sequence_t FDD_DCHs_to_Modify_sequence_of[1] = {
  { &hf_nbap_FDD_DCHs_to_Modify_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_FDD_DCHs_to_ModifyItem },
};

static int
dissect_nbap_FDD_DCHs_to_Modify(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_FDD_DCHs_to_Modify, FDD_DCHs_to_Modify_sequence_of,
                                                  1, 128);

  return offset;
}
static int dissect_id_FDD_DCHs_to_Modify(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_FDD_DCHs_to_Modify(tvb, offset, actx, tree, hf_nbap_id_FDD_DCHs_to_Modify);
}


static const per_sequence_t DCH_ModifySpecificItem_TDD_sequence[] = {
  { &hf_nbap_dCH_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DCH_ID },
  { &hf_nbap_ul_CCTrCH_ID   , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_CCTrCH_ID },
  { &hf_nbap_dl_CCTrCH_ID   , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_CCTrCH_ID },
  { &hf_nbap_ul_TransportFormatSet, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TransportFormatSet },
  { &hf_nbap_dl_TransportFormatSet, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TransportFormatSet },
  { &hf_nbap_allocationRetentionPriority, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_AllocationRetentionPriority },
  { &hf_nbap_frameHandlingPriority, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_FrameHandlingPriority },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DCH_ModifySpecificItem_TDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DCH_ModifySpecificItem_TDD, DCH_ModifySpecificItem_TDD_sequence);

  return offset;
}
static int dissect_DCH_ModifySpecificInformation_TDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DCH_ModifySpecificItem_TDD(tvb, offset, actx, tree, hf_nbap_DCH_ModifySpecificInformation_TDD_item);
}


static const per_sequence_t DCH_ModifySpecificInformation_TDD_sequence_of[1] = {
  { &hf_nbap_DCH_ModifySpecificInformation_TDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DCH_ModifySpecificItem_TDD },
};

static int
dissect_nbap_DCH_ModifySpecificInformation_TDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DCH_ModifySpecificInformation_TDD, DCH_ModifySpecificInformation_TDD_sequence_of,
                                                  1, 128);

  return offset;
}
static int dissect_dCH_SpecificInformationList3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DCH_ModifySpecificInformation_TDD(tvb, offset, actx, tree, hf_nbap_dCH_SpecificInformationList3);
}


static const per_sequence_t DCH_ModifyItem_TDD_sequence[] = {
  { &hf_nbap_ul_FP_Mode     , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_UL_FP_Mode },
  { &hf_nbap_toAWS          , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ToAWS },
  { &hf_nbap_toAWE          , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ToAWE },
  { &hf_nbap_transportBearerRequestIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TransportBearerRequestIndicator },
  { &hf_nbap_dCH_SpecificInformationList3, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DCH_ModifySpecificInformation_TDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DCH_ModifyItem_TDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DCH_ModifyItem_TDD, DCH_ModifyItem_TDD_sequence);

  return offset;
}
static int dissect_TDD_DCHs_to_Modify_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DCH_ModifyItem_TDD(tvb, offset, actx, tree, hf_nbap_TDD_DCHs_to_Modify_item);
}


static const per_sequence_t TDD_DCHs_to_Modify_sequence_of[1] = {
  { &hf_nbap_TDD_DCHs_to_Modify_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DCH_ModifyItem_TDD },
};

static int
dissect_nbap_TDD_DCHs_to_Modify(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_TDD_DCHs_to_Modify, TDD_DCHs_to_Modify_sequence_of,
                                                  1, 128);

  return offset;
}
static int dissect_id_TDD_DCHs_to_Modify(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TDD_DCHs_to_Modify(tvb, offset, actx, tree, hf_nbap_id_TDD_DCHs_to_Modify);
}


static const per_sequence_t DCH_RearrangeItem_Bearer_RearrangeInd_sequence[] = {
  { &hf_nbap_dCH_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DCH_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DCH_RearrangeItem_Bearer_RearrangeInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DCH_RearrangeItem_Bearer_RearrangeInd, DCH_RearrangeItem_Bearer_RearrangeInd_sequence);

  return offset;
}
static int dissect_DCH_RearrangeList_Bearer_RearrangeInd_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DCH_RearrangeItem_Bearer_RearrangeInd(tvb, offset, actx, tree, hf_nbap_DCH_RearrangeList_Bearer_RearrangeInd_item);
}


static const per_sequence_t DCH_RearrangeList_Bearer_RearrangeInd_sequence_of[1] = {
  { &hf_nbap_DCH_RearrangeList_Bearer_RearrangeInd_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DCH_RearrangeItem_Bearer_RearrangeInd },
};

static int
dissect_nbap_DCH_RearrangeList_Bearer_RearrangeInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DCH_RearrangeList_Bearer_RearrangeInd, DCH_RearrangeList_Bearer_RearrangeInd_sequence_of,
                                                  1, 128);

  return offset;
}
static int dissect_id_DCH_RearrangeList_Bearer_RearrangeInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DCH_RearrangeList_Bearer_RearrangeInd(tvb, offset, actx, tree, hf_nbap_id_DCH_RearrangeList_Bearer_RearrangeInd);
}


static const per_sequence_t RL_InformationList_DM_Rprt_sequence_of[1] = {
  { &hf_nbap_RL_InformationList_DM_Rprt_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_RL_InformationList_DM_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_RL_InformationList_DM_Rprt, RL_InformationList_DM_Rprt_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_rL_InformationList_DM_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_InformationList_DM_Rprt(tvb, offset, actx, tree, hf_nbap_rL_InformationList_DM_Rprt);
}


static const per_sequence_t RL_DM_Rprt_sequence[] = {
  { &hf_nbap_rL_InformationList_DM_Rprt, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_InformationList_DM_Rprt },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RL_DM_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RL_DM_Rprt, RL_DM_Rprt_sequence);

  return offset;
}
static int dissect_rL2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_DM_Rprt(tvb, offset, actx, tree, hf_nbap_rL2);
}
static int dissect_all_RL2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_DM_Rprt(tvb, offset, actx, tree, hf_nbap_all_RL2);
}


static const per_sequence_t RL_Set_InformationList_DM_Rprt_sequence_of[1] = {
  { &hf_nbap_RL_Set_InformationList_DM_Rprt_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_RL_Set_InformationList_DM_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_RL_Set_InformationList_DM_Rprt, RL_Set_InformationList_DM_Rprt_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_rL_Set_InformationList_DM_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_Set_InformationList_DM_Rprt(tvb, offset, actx, tree, hf_nbap_rL_Set_InformationList_DM_Rprt);
}


static const per_sequence_t RL_Set_DM_Rprt_sequence[] = {
  { &hf_nbap_rL_Set_InformationList_DM_Rprt, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_Set_InformationList_DM_Rprt },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RL_Set_DM_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RL_Set_DM_Rprt, RL_Set_DM_Rprt_sequence);

  return offset;
}
static int dissect_rLS2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_Set_DM_Rprt(tvb, offset, actx, tree, hf_nbap_rLS2);
}
static int dissect_all_RLS2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_Set_DM_Rprt(tvb, offset, actx, tree, hf_nbap_all_RLS2);
}


static const value_string nbap_DedicatedMeasurementObjectType_DM_Rprt_vals[] = {
  {   0, "rL" },
  {   1, "rLS" },
  {   2, "all-RL" },
  {   3, "all-RLS" },
  { 0, NULL }
};

static const per_choice_t DedicatedMeasurementObjectType_DM_Rprt_choice[] = {
  {   0, &hf_nbap_rL2            , ASN1_EXTENSION_ROOT    , dissect_nbap_RL_DM_Rprt },
  {   1, &hf_nbap_rLS2           , ASN1_EXTENSION_ROOT    , dissect_nbap_RL_Set_DM_Rprt },
  {   2, &hf_nbap_all_RL2        , ASN1_EXTENSION_ROOT    , dissect_nbap_RL_DM_Rprt },
  {   3, &hf_nbap_all_RLS2       , ASN1_EXTENSION_ROOT    , dissect_nbap_RL_Set_DM_Rprt },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_DedicatedMeasurementObjectType_DM_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_DedicatedMeasurementObjectType_DM_Rprt, DedicatedMeasurementObjectType_DM_Rprt_choice,
                                 NULL);

  return offset;
}
static int dissect_id_DedicatedMeasurementObjectType_DM_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DedicatedMeasurementObjectType_DM_Rprt(tvb, offset, actx, tree, hf_nbap_id_DedicatedMeasurementObjectType_DM_Rprt);
}


static const per_sequence_t RL_InformationList_DM_Rqst_sequence_of[1] = {
  { &hf_nbap_RL_InformationList_DM_Rqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_RL_InformationList_DM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_RL_InformationList_DM_Rqst, RL_InformationList_DM_Rqst_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_rL_InformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_InformationList_DM_Rqst(tvb, offset, actx, tree, hf_nbap_rL_InformationList);
}


static const per_sequence_t RL_DM_Rqst_sequence[] = {
  { &hf_nbap_rL_InformationList, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_InformationList_DM_Rqst },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RL_DM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RL_DM_Rqst, RL_DM_Rqst_sequence);

  return offset;
}
static int dissect_rL(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_DM_Rqst(tvb, offset, actx, tree, hf_nbap_rL);
}



static int
dissect_nbap_RL_Set_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 31U, NULL, FALSE);

  return offset;
}
static int dissect_id_E_DCH_RL_Set_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_Set_ID(tvb, offset, actx, tree, hf_nbap_id_E_DCH_RL_Set_ID);
}
static int dissect_rL_Set_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_Set_ID(tvb, offset, actx, tree, hf_nbap_rL_Set_ID);
}


static const per_sequence_t RL_Set_InformationItem_DM_Rqst_sequence[] = {
  { &hf_nbap_rL_Set_ID      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_Set_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RL_Set_InformationItem_DM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RL_Set_InformationItem_DM_Rqst, RL_Set_InformationItem_DM_Rqst_sequence);

  return offset;
}
static int dissect_RL_Set_InformationList_DM_Rqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_Set_InformationItem_DM_Rqst(tvb, offset, actx, tree, hf_nbap_RL_Set_InformationList_DM_Rqst_item);
}


static const per_sequence_t RL_Set_InformationList_DM_Rqst_sequence_of[1] = {
  { &hf_nbap_RL_Set_InformationList_DM_Rqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_RL_Set_InformationItem_DM_Rqst },
};

static int
dissect_nbap_RL_Set_InformationList_DM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_RL_Set_InformationList_DM_Rqst, RL_Set_InformationList_DM_Rqst_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_rL_Set_InformationList_DM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_Set_InformationList_DM_Rqst(tvb, offset, actx, tree, hf_nbap_rL_Set_InformationList_DM_Rqst);
}


static const per_sequence_t RL_Set_DM_Rqst_sequence[] = {
  { &hf_nbap_rL_Set_InformationList_DM_Rqst, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_Set_InformationList_DM_Rqst },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RL_Set_DM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RL_Set_DM_Rqst, RL_Set_DM_Rqst_sequence);

  return offset;
}
static int dissect_rLS(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_Set_DM_Rqst(tvb, offset, actx, tree, hf_nbap_rLS);
}



static int
dissect_nbap_AllRL_DM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_null(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_all_RL(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_AllRL_DM_Rqst(tvb, offset, actx, tree, hf_nbap_all_RL);
}



static int
dissect_nbap_AllRL_Set_DM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_null(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_all_RLS(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_AllRL_Set_DM_Rqst(tvb, offset, actx, tree, hf_nbap_all_RLS);
}


static const value_string nbap_DedicatedMeasurementObjectType_DM_Rqst_vals[] = {
  {   0, "rL" },
  {   1, "rLS" },
  {   2, "all-RL" },
  {   3, "all-RLS" },
  { 0, NULL }
};

static const per_choice_t DedicatedMeasurementObjectType_DM_Rqst_choice[] = {
  {   0, &hf_nbap_rL             , ASN1_EXTENSION_ROOT    , dissect_nbap_RL_DM_Rqst },
  {   1, &hf_nbap_rLS            , ASN1_EXTENSION_ROOT    , dissect_nbap_RL_Set_DM_Rqst },
  {   2, &hf_nbap_all_RL         , ASN1_EXTENSION_ROOT    , dissect_nbap_AllRL_DM_Rqst },
  {   3, &hf_nbap_all_RLS        , ASN1_EXTENSION_ROOT    , dissect_nbap_AllRL_Set_DM_Rqst },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_DedicatedMeasurementObjectType_DM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_DedicatedMeasurementObjectType_DM_Rqst, DedicatedMeasurementObjectType_DM_Rqst_choice,
                                 NULL);

  return offset;
}
static int dissect_id_DedicatedMeasurementObjectType_DM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DedicatedMeasurementObjectType_DM_Rqst(tvb, offset, actx, tree, hf_nbap_id_DedicatedMeasurementObjectType_DM_Rqst);
}


static const per_sequence_t RL_InformationList_DM_Rsp_sequence_of[1] = {
  { &hf_nbap_RL_InformationList_DM_Rsp_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_RL_InformationList_DM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_RL_InformationList_DM_Rsp, RL_InformationList_DM_Rsp_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_rL_InformationList_DM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_InformationList_DM_Rsp(tvb, offset, actx, tree, hf_nbap_rL_InformationList_DM_Rsp);
}


static const per_sequence_t RL_DM_Rsp_sequence[] = {
  { &hf_nbap_rL_InformationList_DM_Rsp, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_InformationList_DM_Rsp },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RL_DM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RL_DM_Rsp, RL_DM_Rsp_sequence);

  return offset;
}
static int dissect_rL1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_DM_Rsp(tvb, offset, actx, tree, hf_nbap_rL1);
}
static int dissect_all_RL1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_DM_Rsp(tvb, offset, actx, tree, hf_nbap_all_RL1);
}


static const per_sequence_t RL_Set_InformationList_DM_Rsp_sequence_of[1] = {
  { &hf_nbap_RL_Set_InformationList_DM_Rsp_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_RL_Set_InformationList_DM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_RL_Set_InformationList_DM_Rsp, RL_Set_InformationList_DM_Rsp_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_rL_Set_InformationList_DM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_Set_InformationList_DM_Rsp(tvb, offset, actx, tree, hf_nbap_rL_Set_InformationList_DM_Rsp);
}


static const per_sequence_t RL_Set_DM_Rsp_sequence[] = {
  { &hf_nbap_rL_Set_InformationList_DM_Rsp, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_Set_InformationList_DM_Rsp },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RL_Set_DM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RL_Set_DM_Rsp, RL_Set_DM_Rsp_sequence);

  return offset;
}
static int dissect_rLS1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_Set_DM_Rsp(tvb, offset, actx, tree, hf_nbap_rLS1);
}
static int dissect_all_RLS1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_Set_DM_Rsp(tvb, offset, actx, tree, hf_nbap_all_RLS1);
}


static const value_string nbap_DedicatedMeasurementObjectType_DM_Rsp_vals[] = {
  {   0, "rL" },
  {   1, "rLS" },
  {   2, "all-RL" },
  {   3, "all-RLS" },
  { 0, NULL }
};

static const per_choice_t DedicatedMeasurementObjectType_DM_Rsp_choice[] = {
  {   0, &hf_nbap_rL1            , ASN1_EXTENSION_ROOT    , dissect_nbap_RL_DM_Rsp },
  {   1, &hf_nbap_rLS1           , ASN1_EXTENSION_ROOT    , dissect_nbap_RL_Set_DM_Rsp },
  {   2, &hf_nbap_all_RL1        , ASN1_EXTENSION_ROOT    , dissect_nbap_RL_DM_Rsp },
  {   3, &hf_nbap_all_RLS1       , ASN1_EXTENSION_ROOT    , dissect_nbap_RL_Set_DM_Rsp },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_DedicatedMeasurementObjectType_DM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_DedicatedMeasurementObjectType_DM_Rsp, DedicatedMeasurementObjectType_DM_Rsp_choice,
                                 NULL);

  return offset;
}
static int dissect_id_DedicatedMeasurementObjectType_DM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DedicatedMeasurementObjectType_DM_Rsp(tvb, offset, actx, tree, hf_nbap_id_DedicatedMeasurementObjectType_DM_Rsp);
}


static const value_string nbap_DedicatedMeasurementType_vals[] = {
  {   0, "sir" },
  {   1, "sir-error" },
  {   2, "transmitted-code-power" },
  {   3, "rscp" },
  {   4, "rx-timing-deviation" },
  {   5, "round-trip-time" },
  {   6, "rx-timing-deviation-LCR" },
  {   7, "angle-Of-Arrival-LCR" },
  {   8, "hs-sich-quality" },
  {   9, "best-Cell-Portions" },
  { 0, NULL }
};


static int
dissect_nbap_DedicatedMeasurementType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     6, NULL, TRUE, 4, NULL);

  return offset;
}
static int dissect_id_DedicatedMeasurementType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DedicatedMeasurementType(tvb, offset, actx, tree, hf_nbap_id_DedicatedMeasurementType);
}


static const value_string nbap_TDD_TPC_DownlinkStepSize_vals[] = {
  {   0, "step-size1" },
  {   1, "step-size2" },
  {   2, "step-size3" },
  { 0, NULL }
};


static int
dissect_nbap_TDD_TPC_DownlinkStepSize(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_id_TDD_TPC_DownlinkStepSize_RL_AdditionRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TDD_TPC_DownlinkStepSize(tvb, offset, actx, tree, hf_nbap_id_TDD_TPC_DownlinkStepSize_RL_AdditionRqstTDD);
}
static int dissect_id_TDD_TPC_DownlinkStepSize_InformationModify_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TDD_TPC_DownlinkStepSize(tvb, offset, actx, tree, hf_nbap_id_TDD_TPC_DownlinkStepSize_InformationModify_RL_ReconfPrepTDD);
}
static int dissect_id_TDD_TPC_DownlinkStepSize_InformationAdd_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TDD_TPC_DownlinkStepSize(tvb, offset, actx, tree, hf_nbap_id_TDD_TPC_DownlinkStepSize_InformationAdd_RL_ReconfPrepTDD);
}
static int dissect_tdd_TPC_DownlinkStepSize(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TDD_TPC_DownlinkStepSize(tvb, offset, actx, tree, hf_nbap_tdd_TPC_DownlinkStepSize);
}
static int dissect_tDD_TPC_DownlinkStepSize(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TDD_TPC_DownlinkStepSize(tvb, offset, actx, tree, hf_nbap_tDD_TPC_DownlinkStepSize);
}
static int dissect_tDD_TPC_DownlinkStepSize_InformationModify_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TDD_TPC_DownlinkStepSize(tvb, offset, actx, tree, hf_nbap_tDD_TPC_DownlinkStepSize_InformationModify_RL_ReconfPrepTDD);
}


static const per_sequence_t CCTrCH_TPCItem_RL_SetupRqstTDD_sequence[] = {
  { &hf_nbap_cCTrCH_ID      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CCTrCH_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CCTrCH_TPCItem_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CCTrCH_TPCItem_RL_SetupRqstTDD, CCTrCH_TPCItem_RL_SetupRqstTDD_sequence);

  return offset;
}
static int dissect_CCTrCH_TPCList_RL_SetupRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CCTrCH_TPCItem_RL_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_CCTrCH_TPCList_RL_SetupRqstTDD_item);
}


static const per_sequence_t CCTrCH_TPCList_RL_SetupRqstTDD_sequence_of[1] = {
  { &hf_nbap_CCTrCH_TPCList_RL_SetupRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_CCTrCH_TPCItem_RL_SetupRqstTDD },
};

static int
dissect_nbap_CCTrCH_TPCList_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_CCTrCH_TPCList_RL_SetupRqstTDD, CCTrCH_TPCList_RL_SetupRqstTDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_cCTrCH_TPCList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CCTrCH_TPCList_RL_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_cCTrCH_TPCList);
}



static int
dissect_nbap_DL_DPCH_Information_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_dL_DPCH_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_DPCH_Information_RL_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_dL_DPCH_Information);
}


static const per_sequence_t DL_CCTrCH_InformationItem_RL_SetupRqstTDD_sequence[] = {
  { &hf_nbap_cCTrCH_ID      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CCTrCH_ID },
  { &hf_nbap_tFCS           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TFCS },
  { &hf_nbap_tFCI_Coding    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TFCI_Coding },
  { &hf_nbap_punctureLimit  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PunctureLimit },
  { &hf_nbap_tdd_TPC_DownlinkStepSize, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_TPC_DownlinkStepSize },
  { &hf_nbap_cCTrCH_TPCList , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_CCTrCH_TPCList_RL_SetupRqstTDD },
  { &hf_nbap_dL_DPCH_Information, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_DPCH_Information_RL_SetupRqstTDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_CCTrCH_InformationItem_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_CCTrCH_InformationItem_RL_SetupRqstTDD, DL_CCTrCH_InformationItem_RL_SetupRqstTDD_sequence);

  return offset;
}
static int dissect_id_DL_CCTrCH_InformationItem_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_CCTrCH_InformationItem_RL_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_id_DL_CCTrCH_InformationItem_RL_SetupRqstTDD);
}



static int
dissect_nbap_DL_DPCH_InformationList_RL_AdditionRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_dL_DPCH_Information1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_DPCH_InformationList_RL_AdditionRqstTDD(tvb, offset, actx, tree, hf_nbap_dL_DPCH_Information1);
}


static const per_sequence_t DL_CCTrCH_InformationItem_RL_AdditionRqstTDD_sequence[] = {
  { &hf_nbap_cCTrCH_ID      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CCTrCH_ID },
  { &hf_nbap_dL_DPCH_Information1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_DPCH_InformationList_RL_AdditionRqstTDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_CCTrCH_InformationItem_RL_AdditionRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_CCTrCH_InformationItem_RL_AdditionRqstTDD, DL_CCTrCH_InformationItem_RL_AdditionRqstTDD_sequence);

  return offset;
}
static int dissect_DL_CCTrCH_InformationList_RL_AdditionRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_CCTrCH_InformationItem_RL_AdditionRqstTDD(tvb, offset, actx, tree, hf_nbap_DL_CCTrCH_InformationList_RL_AdditionRqstTDD_item);
}


static const per_sequence_t DL_CCTrCH_InformationList_RL_AdditionRqstTDD_sequence_of[1] = {
  { &hf_nbap_DL_CCTrCH_InformationList_RL_AdditionRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DL_CCTrCH_InformationItem_RL_AdditionRqstTDD },
};

static int
dissect_nbap_DL_CCTrCH_InformationList_RL_AdditionRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DL_CCTrCH_InformationList_RL_AdditionRqstTDD, DL_CCTrCH_InformationList_RL_AdditionRqstTDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_DL_CCTrCH_InformationList_RL_AdditionRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_CCTrCH_InformationList_RL_AdditionRqstTDD(tvb, offset, actx, tree, hf_nbap_id_DL_CCTrCH_InformationList_RL_AdditionRqstTDD);
}


static const per_sequence_t DL_CCTrCH_InformationList_RL_SetupRqstTDD_sequence_of[1] = {
  { &hf_nbap_DL_CCTrCH_InformationList_RL_SetupRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_DL_CCTrCH_InformationList_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DL_CCTrCH_InformationList_RL_SetupRqstTDD, DL_CCTrCH_InformationList_RL_SetupRqstTDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_DL_CCTrCH_InformationList_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_CCTrCH_InformationList_RL_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_id_DL_CCTrCH_InformationList_RL_SetupRqstTDD);
}


static const value_string nbap_RepetitionPeriod_vals[] = {
  {   0, "v1" },
  {   1, "v2" },
  {   2, "v4" },
  {   3, "v8" },
  {   4, "v16" },
  {   5, "v32" },
  {   6, "v64" },
  { 0, NULL }
};


static int
dissect_nbap_RepetitionPeriod(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     7, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_repetitionPeriod(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RepetitionPeriod(tvb, offset, actx, tree, hf_nbap_repetitionPeriod);
}



static int
dissect_nbap_RepetitionLength(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 63U, NULL, FALSE);

  return offset;
}
static int dissect_repetitionLength(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RepetitionLength(tvb, offset, actx, tree, hf_nbap_repetitionLength);
}


static const value_string nbap_TDD_DPCHOffset_vals[] = {
  {   0, "initialOffset" },
  {   1, "noinitialOffset" },
  { 0, NULL }
};

static const per_choice_t TDD_DPCHOffset_choice[] = {
  {   0, &hf_nbap_initialOffset  , ASN1_NO_EXTENSIONS     , dissect_nbap_INTEGER_0_255 },
  {   1, &hf_nbap_noinitialOffset, ASN1_NO_EXTENSIONS     , dissect_nbap_INTEGER_0_63 },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_TDD_DPCHOffset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_TDD_DPCHOffset, TDD_DPCHOffset_choice,
                                 NULL);

  return offset;
}
static int dissect_tdd_DPCHOffset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TDD_DPCHOffset(tvb, offset, actx, tree, hf_nbap_tdd_DPCHOffset);
}


static const value_string nbap_MidambleConfigurationBurstType1And3_vals[] = {
  {   0, "v4" },
  {   1, "v8" },
  {   2, "v16" },
  { 0, NULL }
};


static int
dissect_nbap_MidambleConfigurationBurstType1And3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_midambleConfigurationBurstType1And3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MidambleConfigurationBurstType1And3(tvb, offset, actx, tree, hf_nbap_midambleConfigurationBurstType1And3);
}



static int
dissect_nbap_MidambleShiftLong(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 15U, NULL, FALSE);

  return offset;
}
static int dissect_ueSpecificMidamble(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MidambleShiftLong(tvb, offset, actx, tree, hf_nbap_ueSpecificMidamble);
}
static int dissect_midambleShift(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MidambleShiftLong(tvb, offset, actx, tree, hf_nbap_midambleShift);
}


static const value_string nbap_T_midambleAllocationMode_vals[] = {
  {   0, "defaultMidamble" },
  {   1, "commonMidamble" },
  {   2, "ueSpecificMidamble" },
  { 0, NULL }
};

static const per_choice_t T_midambleAllocationMode_choice[] = {
  {   0, &hf_nbap_defaultMidamble, ASN1_EXTENSION_ROOT    , dissect_nbap_NULL },
  {   1, &hf_nbap_commonMidamble , ASN1_EXTENSION_ROOT    , dissect_nbap_NULL },
  {   2, &hf_nbap_ueSpecificMidamble, ASN1_EXTENSION_ROOT    , dissect_nbap_MidambleShiftLong },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_T_midambleAllocationMode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_T_midambleAllocationMode, T_midambleAllocationMode_choice,
                                 NULL);

  return offset;
}
static int dissect_midambleAllocationMode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_T_midambleAllocationMode(tvb, offset, actx, tree, hf_nbap_midambleAllocationMode);
}


static const per_sequence_t T_type1_sequence[] = {
  { &hf_nbap_midambleConfigurationBurstType1And3, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MidambleConfigurationBurstType1And3 },
  { &hf_nbap_midambleAllocationMode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_T_midambleAllocationMode },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_T_type1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_T_type1, T_type1_sequence);

  return offset;
}
static int dissect_type1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_T_type1(tvb, offset, actx, tree, hf_nbap_type1);
}


static const value_string nbap_MidambleConfigurationBurstType2_vals[] = {
  {   0, "v3" },
  {   1, "v6" },
  { 0, NULL }
};


static int
dissect_nbap_MidambleConfigurationBurstType2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_midambleConfigurationBurstType2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MidambleConfigurationBurstType2(tvb, offset, actx, tree, hf_nbap_midambleConfigurationBurstType2);
}



static int
dissect_nbap_MidambleShiftShort(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 5U, NULL, FALSE);

  return offset;
}
static int dissect_ueSpecificMidamble1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MidambleShiftShort(tvb, offset, actx, tree, hf_nbap_ueSpecificMidamble1);
}


static const value_string nbap_T_midambleAllocationMode1_vals[] = {
  {   0, "defaultMidamble" },
  {   1, "commonMidamble" },
  {   2, "ueSpecificMidamble" },
  { 0, NULL }
};

static const per_choice_t T_midambleAllocationMode1_choice[] = {
  {   0, &hf_nbap_defaultMidamble, ASN1_EXTENSION_ROOT    , dissect_nbap_NULL },
  {   1, &hf_nbap_commonMidamble , ASN1_EXTENSION_ROOT    , dissect_nbap_NULL },
  {   2, &hf_nbap_ueSpecificMidamble1, ASN1_EXTENSION_ROOT    , dissect_nbap_MidambleShiftShort },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_T_midambleAllocationMode1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_T_midambleAllocationMode1, T_midambleAllocationMode1_choice,
                                 NULL);

  return offset;
}
static int dissect_midambleAllocationMode1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_T_midambleAllocationMode1(tvb, offset, actx, tree, hf_nbap_midambleAllocationMode1);
}


static const per_sequence_t T_type2_sequence[] = {
  { &hf_nbap_midambleConfigurationBurstType2, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MidambleConfigurationBurstType2 },
  { &hf_nbap_midambleAllocationMode1, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_T_midambleAllocationMode1 },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_T_type2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_T_type2, T_type2_sequence);

  return offset;
}
static int dissect_type2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_T_type2(tvb, offset, actx, tree, hf_nbap_type2);
}


static const value_string nbap_T_midambleAllocationMode2_vals[] = {
  {   0, "defaultMidamble" },
  {   1, "ueSpecificMidamble" },
  { 0, NULL }
};

static const per_choice_t T_midambleAllocationMode2_choice[] = {
  {   0, &hf_nbap_defaultMidamble, ASN1_EXTENSION_ROOT    , dissect_nbap_NULL },
  {   1, &hf_nbap_ueSpecificMidamble, ASN1_EXTENSION_ROOT    , dissect_nbap_MidambleShiftLong },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_T_midambleAllocationMode2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_T_midambleAllocationMode2, T_midambleAllocationMode2_choice,
                                 NULL);

  return offset;
}
static int dissect_midambleAllocationMode2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_T_midambleAllocationMode2(tvb, offset, actx, tree, hf_nbap_midambleAllocationMode2);
}


static const per_sequence_t T_type3_sequence[] = {
  { &hf_nbap_midambleConfigurationBurstType1And3, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MidambleConfigurationBurstType1And3 },
  { &hf_nbap_midambleAllocationMode2, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_T_midambleAllocationMode2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_T_type3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_T_type3, T_type3_sequence);

  return offset;
}
static int dissect_type3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_T_type3(tvb, offset, actx, tree, hf_nbap_type3);
}


static const value_string nbap_MidambleShiftAndBurstType_vals[] = {
  {   0, "type1" },
  {   1, "type2" },
  {   2, "type3" },
  { 0, NULL }
};

static const per_choice_t MidambleShiftAndBurstType_choice[] = {
  {   0, &hf_nbap_type1          , ASN1_EXTENSION_ROOT    , dissect_nbap_T_type1 },
  {   1, &hf_nbap_type2          , ASN1_EXTENSION_ROOT    , dissect_nbap_T_type2 },
  {   2, &hf_nbap_type3          , ASN1_EXTENSION_ROOT    , dissect_nbap_T_type3 },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_MidambleShiftAndBurstType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_MidambleShiftAndBurstType, MidambleShiftAndBurstType_choice,
                                 NULL);

  return offset;
}
static int dissect_midambleShiftandBurstType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MidambleShiftAndBurstType(tvb, offset, actx, tree, hf_nbap_midambleShiftandBurstType);
}
static int dissect_midambleshiftAndBurstType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MidambleShiftAndBurstType(tvb, offset, actx, tree, hf_nbap_midambleshiftAndBurstType);
}
static int dissect_midambleShiftAndBurstType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MidambleShiftAndBurstType(tvb, offset, actx, tree, hf_nbap_midambleShiftAndBurstType);
}



static int
dissect_nbap_DPCH_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 239U, NULL, FALSE);

  return offset;
}
static int dissect_dPCH_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DPCH_ID(tvb, offset, actx, tree, hf_nbap_dPCH_ID);
}


static const value_string nbap_TDD_ChannelisationCode_vals[] = {
  {   0, "chCode1div1" },
  {   1, "chCode2div1" },
  {   2, "chCode2div2" },
  {   3, "chCode4div1" },
  {   4, "chCode4div2" },
  {   5, "chCode4div3" },
  {   6, "chCode4div4" },
  {   7, "chCode8div1" },
  {   8, "chCode8div2" },
  {   9, "chCode8div3" },
  {  10, "chCode8div4" },
  {  11, "chCode8div5" },
  {  12, "chCode8div6" },
  {  13, "chCode8div7" },
  {  14, "chCode8div8" },
  {  15, "chCode16div1" },
  {  16, "chCode16div2" },
  {  17, "chCode16div3" },
  {  18, "chCode16div4" },
  {  19, "chCode16div5" },
  {  20, "chCode16div6" },
  {  21, "chCode16div7" },
  {  22, "chCode16div8" },
  {  23, "chCode16div9" },
  {  24, "chCode16div10" },
  {  25, "chCode16div11" },
  {  26, "chCode16div12" },
  {  27, "chCode16div13" },
  {  28, "chCode16div14" },
  {  29, "chCode16div15" },
  {  30, "chCode16div16" },
  { 0, NULL }
};


static int
dissect_nbap_TDD_ChannelisationCode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     31, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_tdd_ChannelisationCode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TDD_ChannelisationCode(tvb, offset, actx, tree, hf_nbap_tdd_ChannelisationCode);
}
static int dissect_DL_HS_PDSCH_Codelist_PSCH_ReconfRqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TDD_ChannelisationCode(tvb, offset, actx, tree, hf_nbap_DL_HS_PDSCH_Codelist_PSCH_ReconfRqst_item);
}
static int dissect_DL_HS_PDSCH_Codelist_LCR_PSCH_ReconfRqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TDD_ChannelisationCode(tvb, offset, actx, tree, hf_nbap_DL_HS_PDSCH_Codelist_LCR_PSCH_ReconfRqst_item);
}
static int dissect_first_TDD_ChannelisationCode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TDD_ChannelisationCode(tvb, offset, actx, tree, hf_nbap_first_TDD_ChannelisationCode);
}
static int dissect_second_TDD_ChannelisationCode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TDD_ChannelisationCode(tvb, offset, actx, tree, hf_nbap_second_TDD_ChannelisationCode);
}
static int dissect_tDD_ChannelisationCode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TDD_ChannelisationCode(tvb, offset, actx, tree, hf_nbap_tDD_ChannelisationCode);
}


static const per_sequence_t TDD_DL_Code_InformationItem_sequence[] = {
  { &hf_nbap_dPCH_ID        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DPCH_ID },
  { &hf_nbap_tdd_ChannelisationCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_ChannelisationCode },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_TDD_DL_Code_InformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_TDD_DL_Code_InformationItem, TDD_DL_Code_InformationItem_sequence);

  return offset;
}
static int dissect_TDD_DL_Code_Information_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TDD_DL_Code_InformationItem(tvb, offset, actx, tree, hf_nbap_TDD_DL_Code_Information_item);
}


static const per_sequence_t TDD_DL_Code_Information_sequence_of[1] = {
  { &hf_nbap_TDD_DL_Code_Information_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_DL_Code_InformationItem },
};

static int
dissect_nbap_TDD_DL_Code_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_TDD_DL_Code_Information, TDD_DL_Code_Information_sequence_of,
                                                  1, 240);

  return offset;
}
static int dissect_dL_Code_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TDD_DL_Code_Information(tvb, offset, actx, tree, hf_nbap_dL_Code_Information);
}


static const per_sequence_t DL_Timeslot_InformationItem_sequence[] = {
  { &hf_nbap_timeSlot       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlot },
  { &hf_nbap_midambleShiftAndBurstType, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MidambleShiftAndBurstType },
  { &hf_nbap_tFCI_Presence  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TFCI_Presence },
  { &hf_nbap_dL_Code_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_DL_Code_Information },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_Timeslot_InformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_Timeslot_InformationItem, DL_Timeslot_InformationItem_sequence);

  return offset;
}
static int dissect_DL_Timeslot_Information_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Timeslot_InformationItem(tvb, offset, actx, tree, hf_nbap_DL_Timeslot_Information_item);
}


static const per_sequence_t DL_Timeslot_Information_sequence_of[1] = {
  { &hf_nbap_DL_Timeslot_Information_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Timeslot_InformationItem },
};

static int
dissect_nbap_DL_Timeslot_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DL_Timeslot_Information, DL_Timeslot_Information_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_dL_Timeslot_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Timeslot_Information(tvb, offset, actx, tree, hf_nbap_dL_Timeslot_Information);
}


static const per_sequence_t DL_DPCH_InformationItem_RL_AdditionRqstTDD_sequence[] = {
  { &hf_nbap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionPeriod },
  { &hf_nbap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionLength },
  { &hf_nbap_tdd_DPCHOffset , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_DPCHOffset },
  { &hf_nbap_dL_Timeslot_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Timeslot_Information },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_DPCH_InformationItem_RL_AdditionRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_DPCH_InformationItem_RL_AdditionRqstTDD, DL_DPCH_InformationItem_RL_AdditionRqstTDD_sequence);

  return offset;
}
static int dissect_id_DL_DPCH_InformationItem_RL_AdditionRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_DPCH_InformationItem_RL_AdditionRqstTDD(tvb, offset, actx, tree, hf_nbap_id_DL_DPCH_InformationItem_RL_AdditionRqstTDD);
}


static const per_sequence_t DL_DPCH_InformationItem_RL_SetupRqstTDD_sequence[] = {
  { &hf_nbap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionPeriod },
  { &hf_nbap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionLength },
  { &hf_nbap_tdd_DPCHOffset , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_DPCHOffset },
  { &hf_nbap_dL_Timeslot_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Timeslot_Information },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_DPCH_InformationItem_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_DPCH_InformationItem_RL_SetupRqstTDD, DL_DPCH_InformationItem_RL_SetupRqstTDD_sequence);

  return offset;
}
static int dissect_id_DL_DPCH_InformationList_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_DPCH_InformationItem_RL_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_id_DL_DPCH_InformationList_RL_SetupRqstTDD);
}



static int
dissect_nbap_DL_DPCH_SlotFormat(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 16U, NULL, TRUE);

  return offset;
}
static int dissect_dl_DPCH_SlotFormat(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_DPCH_SlotFormat(tvb, offset, actx, tree, hf_nbap_dl_DPCH_SlotFormat);
}


static const value_string nbap_TFCI_SignallingMode_TFCI_SignallingOption_vals[] = {
  {   0, "normal" },
  {   1, "not-Used-split" },
  { 0, NULL }
};


static int
dissect_nbap_TFCI_SignallingMode_TFCI_SignallingOption(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_tFCI_SignallingOption(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TFCI_SignallingMode_TFCI_SignallingOption(tvb, offset, actx, tree, hf_nbap_tFCI_SignallingOption);
}


static const per_sequence_t TFCI_SignallingMode_sequence[] = {
  { &hf_nbap_tFCI_SignallingOption, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TFCI_SignallingMode_TFCI_SignallingOption },
  { &hf_nbap_not_Used_splitType, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_NULL },
  { &hf_nbap_not_Used_lengthOfTFCI2, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_NULL },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_TFCI_SignallingMode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_TFCI_SignallingMode, TFCI_SignallingMode_sequence);

  return offset;
}
static int dissect_tFCI_SignallingMode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TFCI_SignallingMode(tvb, offset, actx, tree, hf_nbap_tFCI_SignallingMode);
}


static const value_string nbap_LimitedPowerIncrease_vals[] = {
  {   0, "used" },
  {   1, "not-used" },
  { 0, NULL }
};


static int
dissect_nbap_LimitedPowerIncrease(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_limitedPowerIncrease(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_LimitedPowerIncrease(tvb, offset, actx, tree, hf_nbap_limitedPowerIncrease);
}


static const per_sequence_t DL_DPCH_Information_RL_ReconfPrepFDD_sequence[] = {
  { &hf_nbap_tFCS           , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TFCS },
  { &hf_nbap_dl_DPCH_SlotFormat, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_DPCH_SlotFormat },
  { &hf_nbap_tFCI_SignallingMode, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TFCI_SignallingMode },
  { &hf_nbap_tFCI_Presence  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TFCI_Presence },
  { &hf_nbap_multiplexingPosition, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_MultiplexingPosition },
  { &hf_nbap_not_Used_pDSCH_CodeMapping, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_NULL },
  { &hf_nbap_not_Used_pDSCH_RL_ID, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_NULL },
  { &hf_nbap_limitedPowerIncrease, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_LimitedPowerIncrease },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_DPCH_Information_RL_ReconfPrepFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_DPCH_Information_RL_ReconfPrepFDD, DL_DPCH_Information_RL_ReconfPrepFDD_sequence);

  return offset;
}
static int dissect_id_DL_DPCH_Information_RL_ReconfPrepFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_DPCH_Information_RL_ReconfPrepFDD(tvb, offset, actx, tree, hf_nbap_id_DL_DPCH_Information_RL_ReconfPrepFDD);
}


static const per_sequence_t DL_DPCH_Information_RL_ReconfRqstFDD_sequence[] = {
  { &hf_nbap_dl_TFCS        , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TFCS },
  { &hf_nbap_tFCI_SignallingMode, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TFCI_SignallingMode },
  { &hf_nbap_limitedPowerIncrease, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_LimitedPowerIncrease },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_DPCH_Information_RL_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_DPCH_Information_RL_ReconfRqstFDD, DL_DPCH_Information_RL_ReconfRqstFDD_sequence);

  return offset;
}
static int dissect_id_DL_DPCH_Information_RL_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_DPCH_Information_RL_ReconfRqstFDD(tvb, offset, actx, tree, hf_nbap_id_DL_DPCH_Information_RL_ReconfRqstFDD);
}


static const per_sequence_t PowerOffsetInformation_RL_SetupRqstFDD_sequence[] = {
  { &hf_nbap_pO1_ForTFCI_Bits, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PowerOffset },
  { &hf_nbap_pO2_ForTPC_Bits, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PowerOffset },
  { &hf_nbap_pO3_ForPilotBits, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PowerOffset },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PowerOffsetInformation_RL_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PowerOffsetInformation_RL_SetupRqstFDD, PowerOffsetInformation_RL_SetupRqstFDD_sequence);

  return offset;
}
static int dissect_powerOffsetInformation1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PowerOffsetInformation_RL_SetupRqstFDD(tvb, offset, actx, tree, hf_nbap_powerOffsetInformation1);
}


static const value_string nbap_FDD_TPC_DownlinkStepSize_vals[] = {
  {   0, "step-size0-5" },
  {   1, "step-size1" },
  {   2, "step-size1-5" },
  {   3, "step-size2" },
  { 0, NULL }
};


static int
dissect_nbap_FDD_TPC_DownlinkStepSize(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_fdd_TPC_DownlinkStepSize(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_FDD_TPC_DownlinkStepSize(tvb, offset, actx, tree, hf_nbap_fdd_TPC_DownlinkStepSize);
}


static const value_string nbap_InnerLoopDLPCStatus_vals[] = {
  {   0, "active" },
  {   1, "inactive" },
  { 0, NULL }
};


static int
dissect_nbap_InnerLoopDLPCStatus(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_InnerLoopDLPCStatus(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_InnerLoopDLPCStatus(tvb, offset, actx, tree, hf_nbap_id_InnerLoopDLPCStatus);
}
static int dissect_innerLoopDLPCStatus(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_InnerLoopDLPCStatus(tvb, offset, actx, tree, hf_nbap_innerLoopDLPCStatus);
}


static const per_sequence_t DL_DPCH_Information_RL_SetupRqstFDD_sequence[] = {
  { &hf_nbap_tFCS           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TFCS },
  { &hf_nbap_dl_DPCH_SlotFormat, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_DPCH_SlotFormat },
  { &hf_nbap_tFCI_SignallingMode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TFCI_SignallingMode },
  { &hf_nbap_tFCI_Presence  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TFCI_Presence },
  { &hf_nbap_multiplexingPosition, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MultiplexingPosition },
  { &hf_nbap_not_Used_pDSCH_RL_ID, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_NULL },
  { &hf_nbap_not_Used_pDSCH_CodeMapping, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_NULL },
  { &hf_nbap_powerOffsetInformation1, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PowerOffsetInformation_RL_SetupRqstFDD },
  { &hf_nbap_fdd_TPC_DownlinkStepSize, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_FDD_TPC_DownlinkStepSize },
  { &hf_nbap_limitedPowerIncrease, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_LimitedPowerIncrease },
  { &hf_nbap_innerLoopDLPCStatus, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_InnerLoopDLPCStatus },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_DPCH_Information_RL_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_DPCH_Information_RL_SetupRqstFDD, DL_DPCH_Information_RL_SetupRqstFDD_sequence);

  return offset;
}
static int dissect_id_DL_DPCH_Information_RL_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_DPCH_Information_RL_SetupRqstFDD(tvb, offset, actx, tree, hf_nbap_id_DL_DPCH_Information_RL_SetupRqstFDD);
}


static const value_string nbap_DL_DPCH_TimingAdjustment_vals[] = {
  {   0, "timing-advance" },
  {   1, "timing-delay" },
  { 0, NULL }
};


static int
dissect_nbap_DL_DPCH_TimingAdjustment(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_DL_DPCH_TimingAdjustment(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_DPCH_TimingAdjustment(tvb, offset, actx, tree, hf_nbap_id_DL_DPCH_TimingAdjustment);
}
static int dissect_id_Initial_DL_DPCH_TimingAdjustment(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_DPCH_TimingAdjustment(tvb, offset, actx, tree, hf_nbap_id_Initial_DL_DPCH_TimingAdjustment);
}



static int
dissect_nbap_RL_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 31U, NULL, FALSE);

  return offset;
}
static int dissect_id_HSPDSCH_RL_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_ID(tvb, offset, actx, tree, hf_nbap_id_HSPDSCH_RL_ID);
}
static int dissect_id_PDSCH_RL_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_ID(tvb, offset, actx, tree, hf_nbap_id_PDSCH_RL_ID);
}
static int dissect_id_RL_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_ID(tvb, offset, actx, tree, hf_nbap_id_RL_ID);
}
static int dissect_rL_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_ID(tvb, offset, actx, tree, hf_nbap_rL_ID);
}



static int
dissect_nbap_DL_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              -350, 150U, NULL, FALSE);

  return offset;
}
static int dissect_id_DLReferencePower(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_id_DLReferencePower);
}
static int dissect_id_maxFACH_Power_LCR_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_id_maxFACH_Power_LCR_CTCH_SetupRqstTDD);
}
static int dissect_id_maxFACH_Power_LCR_CTCH_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_id_maxFACH_Power_LCR_CTCH_ReconfRqstTDD);
}
static int dissect_id_PCH_Power_LCR_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_id_PCH_Power_LCR_CTCH_SetupRqstTDD);
}
static int dissect_id_PCH_Power_LCR_CTCH_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_id_PCH_Power_LCR_CTCH_ReconfRqstTDD);
}
static int dissect_id_InitDL_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_id_InitDL_Power);
}
static int dissect_id_CCTrCH_Initial_DL_Power_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_id_CCTrCH_Initial_DL_Power_RL_SetupRqstTDD);
}
static int dissect_id_CCTrCH_Initial_DL_Power_RL_AdditionRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_id_CCTrCH_Initial_DL_Power_RL_AdditionRqstTDD);
}
static int dissect_id_CCTrCH_Initial_DL_Power_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_id_CCTrCH_Initial_DL_Power_RL_ReconfPrepTDD);
}
static int dissect_id_CCTrCH_Maximum_DL_Power_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_id_CCTrCH_Maximum_DL_Power_RL_SetupRqstTDD);
}
static int dissect_id_CCTrCH_Minimum_DL_Power_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_id_CCTrCH_Minimum_DL_Power_RL_SetupRqstTDD);
}
static int dissect_id_CCTrCH_Maximum_DL_Power_RL_AdditionRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_id_CCTrCH_Maximum_DL_Power_RL_AdditionRqstTDD);
}
static int dissect_id_CCTrCH_Minimum_DL_Power_RL_AdditionRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_id_CCTrCH_Minimum_DL_Power_RL_AdditionRqstTDD);
}
static int dissect_id_CCTrCH_Maximum_DL_Power_InformationAdd_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_id_CCTrCH_Maximum_DL_Power_InformationAdd_RL_ReconfPrepTDD);
}
static int dissect_id_CCTrCH_Minimum_DL_Power_InformationAdd_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_id_CCTrCH_Minimum_DL_Power_InformationAdd_RL_ReconfPrepTDD);
}
static int dissect_id_CCTrCH_Maximum_DL_Power_InformationModify_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_id_CCTrCH_Maximum_DL_Power_InformationModify_RL_ReconfPrepTDD);
}
static int dissect_id_CCTrCH_Minimum_DL_Power_InformationModify_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_id_CCTrCH_Minimum_DL_Power_InformationModify_RL_ReconfPrepTDD);
}
static int dissect_id_Maximum_DL_Power_Modify_LCR_InformationModify_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_id_Maximum_DL_Power_Modify_LCR_InformationModify_RL_ReconfPrepTDD);
}
static int dissect_id_Minimum_DL_Power_Modify_LCR_InformationModify_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_id_Minimum_DL_Power_Modify_LCR_InformationModify_RL_ReconfPrepTDD);
}
static int dissect_id_CCTrCH_Maximum_DL_Power_InformationModify_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_id_CCTrCH_Maximum_DL_Power_InformationModify_RL_ReconfRqstTDD);
}
static int dissect_id_CCTrCH_Minimum_DL_Power_InformationModify_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_id_CCTrCH_Minimum_DL_Power_InformationModify_RL_ReconfRqstTDD);
}
static int dissect_id_Initial_DL_Power_TimeslotLCR_InformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_id_Initial_DL_Power_TimeslotLCR_InformationItem);
}
static int dissect_id_Maximum_DL_Power_TimeslotLCR_InformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_id_Maximum_DL_Power_TimeslotLCR_InformationItem);
}
static int dissect_id_Minimum_DL_Power_TimeslotLCR_InformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_id_Minimum_DL_Power_TimeslotLCR_InformationItem);
}
static int dissect_maxFACH_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_maxFACH_Power);
}
static int dissect_pCH_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_pCH_Power);
}
static int dissect_s_CCPCH_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_s_CCPCH_Power);
}
static int dissect_sCCPCH_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_sCCPCH_Power);
}
static int dissect_primarySCH_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_primarySCH_Power);
}
static int dissect_secondarySCH_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_secondarySCH_Power);
}
static int dissect_secondaryCPICH_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_secondaryCPICH_Power);
}
static int dissect_bCH_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_bCH_Power);
}
static int dissect_sCH_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_sCH_Power);
}
static int dissect_initialDL_transmissionPower(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_initialDL_transmissionPower);
}
static int dissect_maximumDL_power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_maximumDL_power);
}
static int dissect_minimumDL_power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_minimumDL_power);
}
static int dissect_initialDL_TransmissionPower(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_initialDL_TransmissionPower);
}
static int dissect_maximumDL_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_maximumDL_Power);
}
static int dissect_minimumDL_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_minimumDL_Power);
}
static int dissect_initial_DL_Transmission_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_initial_DL_Transmission_Power);
}
static int dissect_maxDL_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_maxDL_Power);
}
static int dissect_minDL_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_minDL_Power);
}
static int dissect_cCTrCH_Initial_DL_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_cCTrCH_Initial_DL_Power);
}
static int dissect_cCTrCH_Maximum_DL_Power_InformationAdd_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_cCTrCH_Maximum_DL_Power_InformationAdd_RL_ReconfPrepTDD);
}
static int dissect_cCTrCH_Minimum_DL_Power_InformationAdd_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_cCTrCH_Minimum_DL_Power_InformationAdd_RL_ReconfPrepTDD);
}
static int dissect_cCTrCH_Maximum_DL_Power_InformationModify_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_cCTrCH_Maximum_DL_Power_InformationModify_RL_ReconfPrepTDD);
}
static int dissect_cCTrCH_Minimum_DL_Power_InformationModify_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_cCTrCH_Minimum_DL_Power_InformationModify_RL_ReconfPrepTDD);
}
static int dissect_cCTrCH_Maximum_DL_Power_InformationModify_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_cCTrCH_Maximum_DL_Power_InformationModify_RL_ReconfRqstTDD);
}
static int dissect_cCTrCH_Minimum_DL_Power_InformationModify_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_cCTrCH_Minimum_DL_Power_InformationModify_RL_ReconfRqstTDD);
}
static int dissect_maxPowerLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_maxPowerLCR);
}
static int dissect_minPowerLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_minPowerLCR);
}
static int dissect_dl_ReferencePower(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_dl_ReferencePower);
}
static int dissect_hS_SCCH_MaxPower(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_hS_SCCH_MaxPower);
}
static int dissect_initialDLTransPower(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_initialDLTransPower);
}
static int dissect_dlTransPower(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_dlTransPower);
}
static int dissect_dLTransPower(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_dLTransPower);
}
static int dissect_initial_dl_tx_power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_initial_dl_tx_power);
}
static int dissect_dLReferencePower(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_dLReferencePower);
}
static int dissect_dl_Reference_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Power(tvb, offset, actx, tree, hf_nbap_dl_Reference_Power);
}


static const per_sequence_t DL_ReferencePowerInformationItem_DL_PC_Rqst_sequence[] = {
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_ID },
  { &hf_nbap_dl_ReferencePower, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Power },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_ReferencePowerInformationItem_DL_PC_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_ReferencePowerInformationItem_DL_PC_Rqst, DL_ReferencePowerInformationItem_DL_PC_Rqst_sequence);

  return offset;
}
static int dissect_id_DL_ReferencePowerInformationItem_DL_PC_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_ReferencePowerInformationItem_DL_PC_Rqst(tvb, offset, actx, tree, hf_nbap_id_DL_ReferencePowerInformationItem_DL_PC_Rqst);
}


static const per_sequence_t DL_ReferencePowerInformationList_DL_PC_Rqst_sequence_of[1] = {
  { &hf_nbap_DL_ReferencePowerInformationList_DL_PC_Rqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_DL_ReferencePowerInformationList_DL_PC_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DL_ReferencePowerInformationList_DL_PC_Rqst, DL_ReferencePowerInformationList_DL_PC_Rqst_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_DLReferencePowerList_DL_PC_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_ReferencePowerInformationList_DL_PC_Rqst(tvb, offset, actx, tree, hf_nbap_id_DLReferencePowerList_DL_PC_Rqst);
}



static int
dissect_nbap_DSCH_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 255U, NULL, FALSE);

  return offset;
}
static int dissect_dSCH_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DSCH_ID(tvb, offset, actx, tree, hf_nbap_dSCH_ID);
}


static const per_sequence_t DSCH_TDD_InformationItem_sequence[] = {
  { &hf_nbap_dSCH_ID        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DSCH_ID },
  { &hf_nbap_cCTrCH_ID      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CCTrCH_ID },
  { &hf_nbap_transportFormatSet, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TransportFormatSet },
  { &hf_nbap_allocationRetentionPriority, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_AllocationRetentionPriority },
  { &hf_nbap_frameHandlingPriority, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_FrameHandlingPriority },
  { &hf_nbap_toAWS          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ToAWS },
  { &hf_nbap_toAWE          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ToAWE },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DSCH_TDD_InformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DSCH_TDD_InformationItem, DSCH_TDD_InformationItem_sequence);

  return offset;
}
static int dissect_DSCH_TDD_Information_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DSCH_TDD_InformationItem(tvb, offset, actx, tree, hf_nbap_DSCH_TDD_Information_item);
}


static const per_sequence_t DSCH_TDD_Information_sequence_of[1] = {
  { &hf_nbap_DSCH_TDD_Information_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DSCH_TDD_InformationItem },
};

static int
dissect_nbap_DSCH_TDD_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DSCH_TDD_Information, DSCH_TDD_Information_sequence_of,
                                                  1, 32);

  return offset;
}
static int dissect_id_DSCHs_to_Add_TDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DSCH_TDD_Information(tvb, offset, actx, tree, hf_nbap_id_DSCHs_to_Add_TDD);
}
static int dissect_id_DSCH_TDD_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DSCH_TDD_Information(tvb, offset, actx, tree, hf_nbap_id_DSCH_TDD_Information);
}


static const per_sequence_t DSCH_Information_DeleteItem_RL_ReconfPrepTDD_sequence[] = {
  { &hf_nbap_dSCH_ID        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DSCH_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DSCH_Information_DeleteItem_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DSCH_Information_DeleteItem_RL_ReconfPrepTDD, DSCH_Information_DeleteItem_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_DSCH_Information_DeleteList_RL_ReconfPrepTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DSCH_Information_DeleteItem_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_DSCH_Information_DeleteList_RL_ReconfPrepTDD_item);
}


static const per_sequence_t DSCH_Information_DeleteList_RL_ReconfPrepTDD_sequence_of[1] = {
  { &hf_nbap_DSCH_Information_DeleteList_RL_ReconfPrepTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DSCH_Information_DeleteItem_RL_ReconfPrepTDD },
};

static int
dissect_nbap_DSCH_Information_DeleteList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DSCH_Information_DeleteList_RL_ReconfPrepTDD, DSCH_Information_DeleteList_RL_ReconfPrepTDD_sequence_of,
                                                  1, 32);

  return offset;
}
static int dissect_id_DSCH_Information_DeleteList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DSCH_Information_DeleteList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_id_DSCH_Information_DeleteList_RL_ReconfPrepTDD);
}


static const per_sequence_t DSCH_Information_ModifyItem_RL_ReconfPrepTDD_sequence[] = {
  { &hf_nbap_dSCH_ID        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DSCH_ID },
  { &hf_nbap_cCTrCH_ID      , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_CCTrCH_ID },
  { &hf_nbap_transportFormatSet, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TransportFormatSet },
  { &hf_nbap_allocationRetentionPriority, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_AllocationRetentionPriority },
  { &hf_nbap_frameHandlingPriority, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_FrameHandlingPriority },
  { &hf_nbap_toAWS          , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ToAWS },
  { &hf_nbap_toAWE          , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ToAWE },
  { &hf_nbap_transportBearerRequestIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TransportBearerRequestIndicator },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DSCH_Information_ModifyItem_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DSCH_Information_ModifyItem_RL_ReconfPrepTDD, DSCH_Information_ModifyItem_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_DSCH_Information_ModifyList_RL_ReconfPrepTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DSCH_Information_ModifyItem_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_DSCH_Information_ModifyList_RL_ReconfPrepTDD_item);
}


static const per_sequence_t DSCH_Information_ModifyList_RL_ReconfPrepTDD_sequence_of[1] = {
  { &hf_nbap_DSCH_Information_ModifyList_RL_ReconfPrepTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DSCH_Information_ModifyItem_RL_ReconfPrepTDD },
};

static int
dissect_nbap_DSCH_Information_ModifyList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DSCH_Information_ModifyList_RL_ReconfPrepTDD, DSCH_Information_ModifyList_RL_ReconfPrepTDD_sequence_of,
                                                  1, 32);

  return offset;
}
static int dissect_id_DSCH_Information_ModifyList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DSCH_Information_ModifyList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_id_DSCH_Information_ModifyList_RL_ReconfPrepTDD);
}


static const per_sequence_t DSCH_InformationResponseItem_sequence[] = {
  { &hf_nbap_dSCH_ID        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DSCH_ID },
  { &hf_nbap_bindingID      , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_BindingID },
  { &hf_nbap_transportLayerAddress, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TransportLayerAddress },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DSCH_InformationResponseItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DSCH_InformationResponseItem, DSCH_InformationResponseItem_sequence);

  return offset;
}
static int dissect_DSCH_InformationResponse_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DSCH_InformationResponseItem(tvb, offset, actx, tree, hf_nbap_DSCH_InformationResponse_item);
}


static const per_sequence_t DSCH_InformationResponse_sequence_of[1] = {
  { &hf_nbap_DSCH_InformationResponse_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DSCH_InformationResponseItem },
};

static int
dissect_nbap_DSCH_InformationResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DSCH_InformationResponse, DSCH_InformationResponse_sequence_of,
                                                  1, 32);

  return offset;
}
static int dissect_id_DSCH_InformationResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DSCH_InformationResponse(tvb, offset, actx, tree, hf_nbap_id_DSCH_InformationResponse);
}


static const per_sequence_t DSCH_RearrangeItem_Bearer_RearrangeInd_sequence[] = {
  { &hf_nbap_dSCH_ID        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DSCH_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DSCH_RearrangeItem_Bearer_RearrangeInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DSCH_RearrangeItem_Bearer_RearrangeInd, DSCH_RearrangeItem_Bearer_RearrangeInd_sequence);

  return offset;
}
static int dissect_DSCH_RearrangeList_Bearer_RearrangeInd_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DSCH_RearrangeItem_Bearer_RearrangeInd(tvb, offset, actx, tree, hf_nbap_DSCH_RearrangeList_Bearer_RearrangeInd_item);
}


static const per_sequence_t DSCH_RearrangeList_Bearer_RearrangeInd_sequence_of[1] = {
  { &hf_nbap_DSCH_RearrangeList_Bearer_RearrangeInd_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DSCH_RearrangeItem_Bearer_RearrangeInd },
};

static int
dissect_nbap_DSCH_RearrangeList_Bearer_RearrangeInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DSCH_RearrangeList_Bearer_RearrangeInd, DSCH_RearrangeList_Bearer_RearrangeInd_sequence_of,
                                                  1, 128);

  return offset;
}
static int dissect_id_DSCH_RearrangeList_Bearer_RearrangeInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DSCH_RearrangeList_Bearer_RearrangeInd(tvb, offset, actx, tree, hf_nbap_id_DSCH_RearrangeList_Bearer_RearrangeInd);
}


static const value_string nbap_End_Of_Audit_Sequence_Indicator_vals[] = {
  {   0, "end-of-audit-sequence" },
  {   1, "not-end-of-audit-sequence" },
  { 0, NULL }
};


static int
dissect_nbap_End_Of_Audit_Sequence_Indicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_End_Of_Audit_Sequence_Indicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_End_Of_Audit_Sequence_Indicator(tvb, offset, actx, tree, hf_nbap_id_End_Of_Audit_Sequence_Indicator);
}


static const per_sequence_t FACH_ParametersItem_CTCH_ReconfRqstTDD_sequence[] = {
  { &hf_nbap_commonTransportChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonTransportChannelID },
  { &hf_nbap_toAWS          , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ToAWS },
  { &hf_nbap_toAWE          , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ToAWE },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_FACH_ParametersItem_CTCH_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_FACH_ParametersItem_CTCH_ReconfRqstTDD, FACH_ParametersItem_CTCH_ReconfRqstTDD_sequence);

  return offset;
}
static int dissect_FACH_ParametersList_CTCH_ReconfRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_FACH_ParametersItem_CTCH_ReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_FACH_ParametersList_CTCH_ReconfRqstTDD_item);
}


static const per_sequence_t FACH_ParametersList_CTCH_ReconfRqstTDD_sequence_of[1] = {
  { &hf_nbap_FACH_ParametersList_CTCH_ReconfRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_FACH_ParametersItem_CTCH_ReconfRqstTDD },
};

static int
dissect_nbap_FACH_ParametersList_CTCH_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_FACH_ParametersList_CTCH_ReconfRqstTDD, FACH_ParametersList_CTCH_ReconfRqstTDD_sequence_of,
                                                  0, 8);

  return offset;
}
static int dissect_id_FACH_ParametersList_CTCH_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_FACH_ParametersList_CTCH_ReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_id_FACH_ParametersList_CTCH_ReconfRqstTDD);
}


static const per_sequence_t FACH_ParametersItem_CTCH_SetupRqstFDD_sequence[] = {
  { &hf_nbap_commonTransportChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonTransportChannelID },
  { &hf_nbap_transportFormatSet, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TransportFormatSet },
  { &hf_nbap_toAWS          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ToAWS },
  { &hf_nbap_toAWE          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ToAWE },
  { &hf_nbap_maxFACH_Power  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Power },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_FACH_ParametersItem_CTCH_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_FACH_ParametersItem_CTCH_SetupRqstFDD, FACH_ParametersItem_CTCH_SetupRqstFDD_sequence);

  return offset;
}
static int dissect_FACH_ParametersListIE_CTCH_SetupRqstFDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_FACH_ParametersItem_CTCH_SetupRqstFDD(tvb, offset, actx, tree, hf_nbap_FACH_ParametersListIE_CTCH_SetupRqstFDD_item);
}


static const per_sequence_t FACH_ParametersListIE_CTCH_SetupRqstFDD_sequence_of[1] = {
  { &hf_nbap_FACH_ParametersListIE_CTCH_SetupRqstFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_FACH_ParametersItem_CTCH_SetupRqstFDD },
};

static int
dissect_nbap_FACH_ParametersListIE_CTCH_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_FACH_ParametersListIE_CTCH_SetupRqstFDD, FACH_ParametersListIE_CTCH_SetupRqstFDD_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_id_FACH_ParametersListIE_CTCH_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_FACH_ParametersListIE_CTCH_SetupRqstFDD(tvb, offset, actx, tree, hf_nbap_id_FACH_ParametersListIE_CTCH_SetupRqstFDD);
}


static const per_sequence_t FACH_ParametersItem_CTCH_SetupRqstTDD_sequence[] = {
  { &hf_nbap_commonTransportChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonTransportChannelID },
  { &hf_nbap_fACH_CCTrCH_ID , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CCTrCH_ID },
  { &hf_nbap_dl_TransportFormatSet, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TransportFormatSet },
  { &hf_nbap_toAWS          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ToAWS },
  { &hf_nbap_toAWE          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ToAWE },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_FACH_ParametersItem_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_FACH_ParametersItem_CTCH_SetupRqstTDD, FACH_ParametersItem_CTCH_SetupRqstTDD_sequence);

  return offset;
}
static int dissect_FACH_ParametersListIE_CTCH_SetupRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_FACH_ParametersItem_CTCH_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_FACH_ParametersListIE_CTCH_SetupRqstTDD_item);
}


static const per_sequence_t FACH_ParametersListIE_CTCH_SetupRqstTDD_sequence_of[1] = {
  { &hf_nbap_FACH_ParametersListIE_CTCH_SetupRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_FACH_ParametersItem_CTCH_SetupRqstTDD },
};

static int
dissect_nbap_FACH_ParametersListIE_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_FACH_ParametersListIE_CTCH_SetupRqstTDD, FACH_ParametersListIE_CTCH_SetupRqstTDD_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_id_FACH_ParametersListIE_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_FACH_ParametersListIE_CTCH_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_id_FACH_ParametersListIE_CTCH_SetupRqstTDD);
}


static const per_sequence_t Local_Cell_InformationList_ResourceStatusInd_sequence_of[1] = {
  { &hf_nbap_Local_Cell_InformationList_ResourceStatusInd_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_Local_Cell_InformationList_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_Local_Cell_InformationList_ResourceStatusInd, Local_Cell_InformationList_ResourceStatusInd_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_local_Cell_InformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Local_Cell_InformationList_ResourceStatusInd(tvb, offset, actx, tree, hf_nbap_local_Cell_InformationList);
}


static const per_sequence_t Local_Cell_Group_InformationList_ResourceStatusInd_sequence_of[1] = {
  { &hf_nbap_Local_Cell_Group_InformationList_ResourceStatusInd_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_Local_Cell_Group_InformationList_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_Local_Cell_Group_InformationList_ResourceStatusInd, Local_Cell_Group_InformationList_ResourceStatusInd_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_local_Cell_Group_InformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Local_Cell_Group_InformationList_ResourceStatusInd(tvb, offset, actx, tree, hf_nbap_local_Cell_Group_InformationList);
}


static const per_sequence_t No_Failure_ResourceStatusInd_sequence[] = {
  { &hf_nbap_local_Cell_InformationList, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Local_Cell_InformationList_ResourceStatusInd },
  { &hf_nbap_local_Cell_Group_InformationList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_Local_Cell_Group_InformationList_ResourceStatusInd },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_No_Failure_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_No_Failure_ResourceStatusInd, No_Failure_ResourceStatusInd_sequence);

  return offset;
}
static int dissect_no_Failure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_No_Failure_ResourceStatusInd(tvb, offset, actx, tree, hf_nbap_no_Failure);
}


static const per_sequence_t Local_Cell_InformationList2_ResourceStatusInd_sequence_of[1] = {
  { &hf_nbap_Local_Cell_InformationList2_ResourceStatusInd_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_Local_Cell_InformationList2_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_Local_Cell_InformationList2_ResourceStatusInd, Local_Cell_InformationList2_ResourceStatusInd_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_local_Cell_InformationList1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Local_Cell_InformationList2_ResourceStatusInd(tvb, offset, actx, tree, hf_nbap_local_Cell_InformationList1);
}


static const per_sequence_t Local_Cell_Group_InformationList2_ResourceStatusInd_sequence_of[1] = {
  { &hf_nbap_Local_Cell_Group_InformationList2_ResourceStatusInd_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_Local_Cell_Group_InformationList2_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_Local_Cell_Group_InformationList2_ResourceStatusInd, Local_Cell_Group_InformationList2_ResourceStatusInd_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_local_Cell_Group_InformationList1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Local_Cell_Group_InformationList2_ResourceStatusInd(tvb, offset, actx, tree, hf_nbap_local_Cell_Group_InformationList1);
}


static const per_sequence_t CCP_InformationList_ResourceStatusInd_sequence_of[1] = {
  { &hf_nbap_CCP_InformationList_ResourceStatusInd_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_CCP_InformationList_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_CCP_InformationList_ResourceStatusInd, CCP_InformationList_ResourceStatusInd_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_cCP_InformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CCP_InformationList_ResourceStatusInd(tvb, offset, actx, tree, hf_nbap_cCP_InformationList);
}


static const per_sequence_t Cell_InformationList_ResourceStatusInd_sequence_of[1] = {
  { &hf_nbap_Cell_InformationList_ResourceStatusInd_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_Cell_InformationList_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_Cell_InformationList_ResourceStatusInd, Cell_InformationList_ResourceStatusInd_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_cell_InformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Cell_InformationList_ResourceStatusInd(tvb, offset, actx, tree, hf_nbap_cell_InformationList);
}


static const per_sequence_t ServiceImpacting_ResourceStatusInd_sequence[] = {
  { &hf_nbap_local_Cell_InformationList1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_Local_Cell_InformationList2_ResourceStatusInd },
  { &hf_nbap_local_Cell_Group_InformationList1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_Local_Cell_Group_InformationList2_ResourceStatusInd },
  { &hf_nbap_cCP_InformationList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_CCP_InformationList_ResourceStatusInd },
  { &hf_nbap_cell_InformationList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_Cell_InformationList_ResourceStatusInd },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_ServiceImpacting_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_ServiceImpacting_ResourceStatusInd, ServiceImpacting_ResourceStatusInd_sequence);

  return offset;
}
static int dissect_serviceImpacting(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ServiceImpacting_ResourceStatusInd(tvb, offset, actx, tree, hf_nbap_serviceImpacting);
}


static const value_string nbap_IndicationType_ResourceStatusInd_vals[] = {
  {   0, "no-Failure" },
  {   1, "serviceImpacting" },
  { 0, NULL }
};

static const per_choice_t IndicationType_ResourceStatusInd_choice[] = {
  {   0, &hf_nbap_no_Failure     , ASN1_EXTENSION_ROOT    , dissect_nbap_No_Failure_ResourceStatusInd },
  {   1, &hf_nbap_serviceImpacting, ASN1_EXTENSION_ROOT    , dissect_nbap_ServiceImpacting_ResourceStatusInd },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_IndicationType_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_IndicationType_ResourceStatusInd, IndicationType_ResourceStatusInd_choice,
                                 NULL);

  return offset;
}
static int dissect_id_IndicationType_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_IndicationType_ResourceStatusInd(tvb, offset, actx, tree, hf_nbap_id_IndicationType_ResourceStatusInd);
}



static int
dissect_nbap_DL_or_Global_CapacityCredit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 65535U, NULL, FALSE);

  return offset;
}
static int dissect_dl_or_global_capacityCredit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_or_Global_CapacityCredit(tvb, offset, actx, tree, hf_nbap_dl_or_global_capacityCredit);
}



static int
dissect_nbap_UL_CapacityCredit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 65535U, NULL, FALSE);

  return offset;
}
static int dissect_ul_capacityCredit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_CapacityCredit(tvb, offset, actx, tree, hf_nbap_ul_capacityCredit);
}


static const per_sequence_t CommonChannelsCapacityConsumptionLaw_item_sequence[] = {
  { &hf_nbap_dl_Cost        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_INTEGER_0_65535 },
  { &hf_nbap_ul_Cost        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_INTEGER_0_65535 },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CommonChannelsCapacityConsumptionLaw_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CommonChannelsCapacityConsumptionLaw_item, CommonChannelsCapacityConsumptionLaw_item_sequence);

  return offset;
}
static int dissect_CommonChannelsCapacityConsumptionLaw_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommonChannelsCapacityConsumptionLaw_item(tvb, offset, actx, tree, hf_nbap_CommonChannelsCapacityConsumptionLaw_item);
}


static const per_sequence_t CommonChannelsCapacityConsumptionLaw_sequence_of[1] = {
  { &hf_nbap_CommonChannelsCapacityConsumptionLaw_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_CommonChannelsCapacityConsumptionLaw_item },
};

static int
dissect_nbap_CommonChannelsCapacityConsumptionLaw(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_CommonChannelsCapacityConsumptionLaw, CommonChannelsCapacityConsumptionLaw_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_commonChannelsCapacityConsumptionLaw(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommonChannelsCapacityConsumptionLaw(tvb, offset, actx, tree, hf_nbap_commonChannelsCapacityConsumptionLaw);
}


static const per_sequence_t DedicatedChannelsCapacityConsumptionLaw_item_sequence[] = {
  { &hf_nbap_dl_Cost_1      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_INTEGER_0_65535 },
  { &hf_nbap_dl_Cost_2      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_INTEGER_0_65535 },
  { &hf_nbap_ul_Cost_1      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_INTEGER_0_65535 },
  { &hf_nbap_ul_Cost_2      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_INTEGER_0_65535 },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DedicatedChannelsCapacityConsumptionLaw_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DedicatedChannelsCapacityConsumptionLaw_item, DedicatedChannelsCapacityConsumptionLaw_item_sequence);

  return offset;
}
static int dissect_DedicatedChannelsCapacityConsumptionLaw_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DedicatedChannelsCapacityConsumptionLaw_item(tvb, offset, actx, tree, hf_nbap_DedicatedChannelsCapacityConsumptionLaw_item);
}


static const per_sequence_t DedicatedChannelsCapacityConsumptionLaw_sequence_of[1] = {
  { &hf_nbap_DedicatedChannelsCapacityConsumptionLaw_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DedicatedChannelsCapacityConsumptionLaw_item },
};

static int
dissect_nbap_DedicatedChannelsCapacityConsumptionLaw(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DedicatedChannelsCapacityConsumptionLaw, DedicatedChannelsCapacityConsumptionLaw_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_id_E_DCH_CapacityConsumptionLaw(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DedicatedChannelsCapacityConsumptionLaw(tvb, offset, actx, tree, hf_nbap_id_E_DCH_CapacityConsumptionLaw);
}
static int dissect_dedicatedChannelsCapacityConsumptionLaw(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DedicatedChannelsCapacityConsumptionLaw(tvb, offset, actx, tree, hf_nbap_dedicatedChannelsCapacityConsumptionLaw);
}


static const per_sequence_t Local_Cell_Group_InformationItem_AuditRsp_sequence[] = {
  { &hf_nbap_local_Cell_Group_ID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Local_Cell_ID },
  { &hf_nbap_dl_or_global_capacityCredit, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_or_Global_CapacityCredit },
  { &hf_nbap_ul_capacityCredit, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_UL_CapacityCredit },
  { &hf_nbap_commonChannelsCapacityConsumptionLaw, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonChannelsCapacityConsumptionLaw },
  { &hf_nbap_dedicatedChannelsCapacityConsumptionLaw, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DedicatedChannelsCapacityConsumptionLaw },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Local_Cell_Group_InformationItem_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Local_Cell_Group_InformationItem_AuditRsp, Local_Cell_Group_InformationItem_AuditRsp_sequence);

  return offset;
}
static int dissect_id_Local_Cell_Group_InformationItem_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Local_Cell_Group_InformationItem_AuditRsp(tvb, offset, actx, tree, hf_nbap_id_Local_Cell_Group_InformationItem_AuditRsp);
}


static const per_sequence_t Local_Cell_Group_InformationItem_ResourceStatusInd_sequence[] = {
  { &hf_nbap_local_Cell_Group_ID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Local_Cell_ID },
  { &hf_nbap_dl_or_global_capacityCredit, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_or_Global_CapacityCredit },
  { &hf_nbap_ul_capacityCredit, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_UL_CapacityCredit },
  { &hf_nbap_commonChannelsCapacityConsumptionLaw, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonChannelsCapacityConsumptionLaw },
  { &hf_nbap_dedicatedChannelsCapacityConsumptionLaw, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DedicatedChannelsCapacityConsumptionLaw },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Local_Cell_Group_InformationItem_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Local_Cell_Group_InformationItem_ResourceStatusInd, Local_Cell_Group_InformationItem_ResourceStatusInd_sequence);

  return offset;
}
static int dissect_id_Local_Cell_Group_InformationItem_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Local_Cell_Group_InformationItem_ResourceStatusInd(tvb, offset, actx, tree, hf_nbap_id_Local_Cell_Group_InformationItem_ResourceStatusInd);
}


static const per_sequence_t Local_Cell_Group_InformationItem2_ResourceStatusInd_sequence[] = {
  { &hf_nbap_local_Cell_Group_ID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Local_Cell_ID },
  { &hf_nbap_dl_or_global_capacityCredit, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_or_Global_CapacityCredit },
  { &hf_nbap_ul_capacityCredit, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_UL_CapacityCredit },
  { &hf_nbap_commonChannelsCapacityConsumptionLaw, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_CommonChannelsCapacityConsumptionLaw },
  { &hf_nbap_dedicatedChannelsCapacityConsumptionLaw, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DedicatedChannelsCapacityConsumptionLaw },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Local_Cell_Group_InformationItem2_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Local_Cell_Group_InformationItem2_ResourceStatusInd, Local_Cell_Group_InformationItem2_ResourceStatusInd_sequence);

  return offset;
}
static int dissect_id_Local_Cell_Group_InformationItem2_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Local_Cell_Group_InformationItem2_ResourceStatusInd(tvb, offset, actx, tree, hf_nbap_id_Local_Cell_Group_InformationItem2_ResourceStatusInd);
}


static const per_sequence_t Local_Cell_Group_InformationList_AuditRsp_sequence_of[1] = {
  { &hf_nbap_Local_Cell_Group_InformationList_AuditRsp_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_Local_Cell_Group_InformationList_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_Local_Cell_Group_InformationList_AuditRsp, Local_Cell_Group_InformationList_AuditRsp_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_id_Local_Cell_Group_InformationList_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Local_Cell_Group_InformationList_AuditRsp(tvb, offset, actx, tree, hf_nbap_id_Local_Cell_Group_InformationList_AuditRsp);
}



static int
dissect_nbap_MaximumDL_PowerCapability(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 500U, NULL, FALSE);

  return offset;
}
static int dissect_maximumDL_PowerCapability(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MaximumDL_PowerCapability(tvb, offset, actx, tree, hf_nbap_maximumDL_PowerCapability);
}
static int dissect_maximum_DL_PowerCapability(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MaximumDL_PowerCapability(tvb, offset, actx, tree, hf_nbap_maximum_DL_PowerCapability);
}


static const value_string nbap_MinSpreadingFactor_vals[] = {
  {   0, "v4" },
  {   1, "v8" },
  {   2, "v16" },
  {   3, "v32" },
  {   4, "v64" },
  {   5, "v128" },
  {   6, "v256" },
  {   7, "v512" },
  { 0, NULL }
};


static int
dissect_nbap_MinSpreadingFactor(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_minSpreadingFactor(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MinSpreadingFactor(tvb, offset, actx, tree, hf_nbap_minSpreadingFactor);
}



static int
dissect_nbap_MinimumDL_PowerCapability(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 800U, NULL, FALSE);

  return offset;
}
static int dissect_minimumDL_PowerCapability(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MinimumDL_PowerCapability(tvb, offset, actx, tree, hf_nbap_minimumDL_PowerCapability);
}


static const per_sequence_t Local_Cell_InformationItem_AuditRsp_sequence[] = {
  { &hf_nbap_local_Cell_ID  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Local_Cell_ID },
  { &hf_nbap_dl_or_global_capacityCredit, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_or_Global_CapacityCredit },
  { &hf_nbap_ul_capacityCredit, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_UL_CapacityCredit },
  { &hf_nbap_commonChannelsCapacityConsumptionLaw, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonChannelsCapacityConsumptionLaw },
  { &hf_nbap_dedicatedChannelsCapacityConsumptionLaw, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DedicatedChannelsCapacityConsumptionLaw },
  { &hf_nbap_maximumDL_PowerCapability, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_MaximumDL_PowerCapability },
  { &hf_nbap_minSpreadingFactor, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_MinSpreadingFactor },
  { &hf_nbap_minimumDL_PowerCapability, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_MinimumDL_PowerCapability },
  { &hf_nbap_local_Cell_Group_ID, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_Local_Cell_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Local_Cell_InformationItem_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Local_Cell_InformationItem_AuditRsp, Local_Cell_InformationItem_AuditRsp_sequence);

  return offset;
}
static int dissect_id_Local_Cell_InformationItem_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Local_Cell_InformationItem_AuditRsp(tvb, offset, actx, tree, hf_nbap_id_Local_Cell_InformationItem_AuditRsp);
}


static const value_string nbap_AddorDeleteIndicator_vals[] = {
  {   0, "add" },
  {   1, "delete" },
  { 0, NULL }
};


static int
dissect_nbap_AddorDeleteIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_addorDeleteIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_AddorDeleteIndicator(tvb, offset, actx, tree, hf_nbap_addorDeleteIndicator);
}


static const per_sequence_t Local_Cell_InformationItem_ResourceStatusInd_sequence[] = {
  { &hf_nbap_local_CellID   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Local_Cell_ID },
  { &hf_nbap_addorDeleteIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_AddorDeleteIndicator },
  { &hf_nbap_dl_or_global_capacityCredit, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_or_Global_CapacityCredit },
  { &hf_nbap_ul_capacityCredit, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_UL_CapacityCredit },
  { &hf_nbap_commonChannelsCapacityConsumptionLaw, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_CommonChannelsCapacityConsumptionLaw },
  { &hf_nbap_dedicatedChannelsCapacityConsumptionLaw, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DedicatedChannelsCapacityConsumptionLaw },
  { &hf_nbap_maximumDL_PowerCapability, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_MaximumDL_PowerCapability },
  { &hf_nbap_minSpreadingFactor, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_MinSpreadingFactor },
  { &hf_nbap_minimumDL_PowerCapability, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_MinimumDL_PowerCapability },
  { &hf_nbap_local_Cell_Group_ID, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_Local_Cell_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Local_Cell_InformationItem_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Local_Cell_InformationItem_ResourceStatusInd, Local_Cell_InformationItem_ResourceStatusInd_sequence);

  return offset;
}
static int dissect_id_Local_Cell_InformationItem_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Local_Cell_InformationItem_ResourceStatusInd(tvb, offset, actx, tree, hf_nbap_id_Local_Cell_InformationItem_ResourceStatusInd);
}


static const per_sequence_t Local_Cell_InformationItem2_ResourceStatusInd_sequence[] = {
  { &hf_nbap_local_Cell_ID  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Local_Cell_ID },
  { &hf_nbap_dl_or_global_capacityCredit, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_or_Global_CapacityCredit },
  { &hf_nbap_ul_capacityCredit, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_UL_CapacityCredit },
  { &hf_nbap_commonChannelsCapacityConsumptionLaw, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_CommonChannelsCapacityConsumptionLaw },
  { &hf_nbap_dedicatedChannelsCapacityConsumptionLaw, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DedicatedChannelsCapacityConsumptionLaw },
  { &hf_nbap_maximum_DL_PowerCapability, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_MaximumDL_PowerCapability },
  { &hf_nbap_minSpreadingFactor, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_MinSpreadingFactor },
  { &hf_nbap_minimumDL_PowerCapability, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_MinimumDL_PowerCapability },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Local_Cell_InformationItem2_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Local_Cell_InformationItem2_ResourceStatusInd, Local_Cell_InformationItem2_ResourceStatusInd_sequence);

  return offset;
}
static int dissect_id_Local_Cell_InformationItem2_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Local_Cell_InformationItem2_ResourceStatusInd(tvb, offset, actx, tree, hf_nbap_id_Local_Cell_InformationItem2_ResourceStatusInd);
}


static const per_sequence_t Local_Cell_InformationList_AuditRsp_sequence_of[1] = {
  { &hf_nbap_Local_Cell_InformationList_AuditRsp_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_Local_Cell_InformationList_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_Local_Cell_InformationList_AuditRsp, Local_Cell_InformationList_AuditRsp_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_id_Local_Cell_InformationList_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Local_Cell_InformationList_AuditRsp(tvb, offset, actx, tree, hf_nbap_id_Local_Cell_InformationList_AuditRsp);
}



static int
dissect_nbap_AdjustmentPeriod(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 256U, NULL, FALSE);

  return offset;
}
static int dissect_id_AdjustmentPeriod(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_AdjustmentPeriod(tvb, offset, actx, tree, hf_nbap_id_AdjustmentPeriod);
}
static int dissect_adjustmentPeriod(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_AdjustmentPeriod(tvb, offset, actx, tree, hf_nbap_adjustmentPeriod);
}



static int
dissect_nbap_MaxAdjustmentStep(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 10U, NULL, FALSE);

  return offset;
}
static int dissect_id_MaxAdjustmentStep(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MaxAdjustmentStep(tvb, offset, actx, tree, hf_nbap_id_MaxAdjustmentStep);
}
static int dissect_maxAdjustmentStep(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MaxAdjustmentStep(tvb, offset, actx, tree, hf_nbap_maxAdjustmentStep);
}



static int
dissect_nbap_MaximumTransmissionPower(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 500U, NULL, FALSE);

  return offset;
}
static int dissect_id_MaximumTransmissionPower(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MaximumTransmissionPower(tvb, offset, actx, tree, hf_nbap_id_MaximumTransmissionPower);
}
static int dissect_id_HS_PDSCH_HS_SCCH_E_AGCH_E_RGCH_E_HICH_MaxPower_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MaximumTransmissionPower(tvb, offset, actx, tree, hf_nbap_id_HS_PDSCH_HS_SCCH_E_AGCH_E_RGCH_E_HICH_MaxPower_PSCH_ReconfRqst);
}
static int dissect_maximumTransmissionPowerforCellPortion(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MaximumTransmissionPower(tvb, offset, actx, tree, hf_nbap_maximumTransmissionPowerforCellPortion);
}
static int dissect_hS_PDSCH_HS_SCCH_E_AGCH_E_RGCH_E_HICH_MaxPower_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MaximumTransmissionPower(tvb, offset, actx, tree, hf_nbap_hS_PDSCH_HS_SCCH_E_AGCH_E_RGCH_E_HICH_MaxPower_PSCH_ReconfRqst);
}
static int dissect_maxHSDSCH_HSSCCH_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MaximumTransmissionPower(tvb, offset, actx, tree, hf_nbap_maxHSDSCH_HSSCCH_Power);
}


static const value_string nbap_MeasurementFilterCoefficient_vals[] = {
  {   0, "k0" },
  {   1, "k1" },
  {   2, "k2" },
  {   3, "k3" },
  {   4, "k4" },
  {   5, "k5" },
  {   6, "k6" },
  {   7, "k7" },
  {   8, "k8" },
  {   9, "k9" },
  {  10, "k11" },
  {  11, "k13" },
  {  12, "k15" },
  {  13, "k17" },
  {  14, "k19" },
  { 0, NULL }
};


static int
dissect_nbap_MeasurementFilterCoefficient(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     15, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_id_MeasurementFilterCoefficient(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MeasurementFilterCoefficient(tvb, offset, actx, tree, hf_nbap_id_MeasurementFilterCoefficient);
}



static int
dissect_nbap_MeasurementID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 1048575U, NULL, FALSE);

  return offset;
}
static int dissect_id_MeasurementID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MeasurementID(tvb, offset, actx, tree, hf_nbap_id_MeasurementID);
}



static int
dissect_nbap_RepetitionNumber1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 256U, NULL, FALSE);

  return offset;
}
static int dissect_repetitionNumber1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RepetitionNumber1(tvb, offset, actx, tree, hf_nbap_repetitionNumber1);
}


static const per_sequence_t MessageStructure_item_sequence[] = {
  { &hf_nbap_iE_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_ID },
  { &hf_nbap_repetitionNumber1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_RepetitionNumber1 },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_MessageStructure_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_MessageStructure_item, MessageStructure_item_sequence);

  return offset;
}
static int dissect_MessageStructure_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MessageStructure_item(tvb, offset, actx, tree, hf_nbap_MessageStructure_item);
}


static const per_sequence_t MessageStructure_sequence_of[1] = {
  { &hf_nbap_MessageStructure_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_MessageStructure_item },
};

static int
dissect_nbap_MessageStructure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_MessageStructure, MessageStructure_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_id_MessageStructure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MessageStructure(tvb, offset, actx, tree, hf_nbap_id_MessageStructure);
}


static const value_string nbap_IB_Type_vals[] = {
  {   0, "mIB" },
  {   1, "sB1" },
  {   2, "sB2" },
  {   3, "sIB1" },
  {   4, "sIB2" },
  {   5, "sIB3" },
  {   6, "sIB4" },
  {   7, "sIB5" },
  {   8, "sIB6" },
  {   9, "sIB7" },
  {  10, "sIB8" },
  {  11, "sIB9" },
  {  12, "sIB10" },
  {  13, "sIB11" },
  {  14, "sIB12" },
  {  15, "sIB13" },
  {  16, "sIB13dot1" },
  {  17, "sIB13dot2" },
  {  18, "sIB13dot3" },
  {  19, "sIB13dot4" },
  {  20, "sIB14" },
  {  21, "sIB15" },
  {  22, "sIB15dot1" },
  {  23, "sIB15dot2" },
  {  24, "sIB15dot3" },
  {  25, "sIB16" },
  {  26, "sIB17" },
  {  27, "sIB15dot4" },
  {  28, "sIB18" },
  {  29, "sIB15dot5" },
  {  30, "sIB5bis" },
  { 0, NULL }
};


static int
dissect_nbap_IB_Type(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     26, NULL, TRUE, 5, NULL);

  return offset;
}
static int dissect_iB_Type(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_IB_Type(tvb, offset, actx, tree, hf_nbap_iB_Type);
}



static int
dissect_nbap_IB_OC_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 16U, NULL, FALSE);

  return offset;
}
static int dissect_iB_OC_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_IB_OC_ID(tvb, offset, actx, tree, hf_nbap_iB_OC_ID);
}


static const value_string nbap_SIB_Originator_vals[] = {
  {   0, "nodeB" },
  {   1, "cRNC" },
  { 0, NULL }
};


static int
dissect_nbap_SIB_Originator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_sIB_Originator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SIB_Originator(tvb, offset, actx, tree, hf_nbap_sIB_Originator);
}


static const value_string nbap_IB_SG_REP_vals[] = {
  {   0, "rep4" },
  {   1, "rep8" },
  {   2, "rep16" },
  {   3, "rep32" },
  {   4, "rep64" },
  {   5, "rep128" },
  {   6, "rep256" },
  {   7, "rep512" },
  {   8, "rep1024" },
  {   9, "rep2048" },
  {  10, "rep4096" },
  { 0, NULL }
};


static int
dissect_nbap_IB_SG_REP(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     11, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_iB_SG_REP(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_IB_SG_REP(tvb, offset, actx, tree, hf_nbap_iB_SG_REP);
}



static int
dissect_nbap_SegmentInformationList_SystemInfoUpdate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_segmentInformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SegmentInformationList_SystemInfoUpdate(tvb, offset, actx, tree, hf_nbap_segmentInformationList);
}


static const per_sequence_t No_Deletion_SystemInfoUpdate_sequence[] = {
  { &hf_nbap_sIB_Originator , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_SIB_Originator },
  { &hf_nbap_iB_SG_REP      , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_IB_SG_REP },
  { &hf_nbap_segmentInformationList, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SegmentInformationList_SystemInfoUpdate },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_No_Deletion_SystemInfoUpdate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_No_Deletion_SystemInfoUpdate, No_Deletion_SystemInfoUpdate_sequence);

  return offset;
}
static int dissect_no_Deletion(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_No_Deletion_SystemInfoUpdate(tvb, offset, actx, tree, hf_nbap_no_Deletion);
}


static const value_string nbap_DeletionIndicator_SystemInfoUpdate_vals[] = {
  {   0, "no-Deletion" },
  {   1, "yes-Deletion" },
  { 0, NULL }
};

static const per_choice_t DeletionIndicator_SystemInfoUpdate_choice[] = {
  {   0, &hf_nbap_no_Deletion    , ASN1_NO_EXTENSIONS     , dissect_nbap_No_Deletion_SystemInfoUpdate },
  {   1, &hf_nbap_yes_Deletion   , ASN1_NO_EXTENSIONS     , dissect_nbap_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_DeletionIndicator_SystemInfoUpdate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_DeletionIndicator_SystemInfoUpdate, DeletionIndicator_SystemInfoUpdate_choice,
                                 NULL);

  return offset;
}
static int dissect_deletionIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DeletionIndicator_SystemInfoUpdate(tvb, offset, actx, tree, hf_nbap_deletionIndicator);
}


static const per_sequence_t MIB_SB_SIB_InformationItem_SystemInfoUpdateRqst_sequence[] = {
  { &hf_nbap_iB_Type        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_IB_Type },
  { &hf_nbap_iB_OC_ID       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_IB_OC_ID },
  { &hf_nbap_deletionIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DeletionIndicator_SystemInfoUpdate },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_MIB_SB_SIB_InformationItem_SystemInfoUpdateRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_MIB_SB_SIB_InformationItem_SystemInfoUpdateRqst, MIB_SB_SIB_InformationItem_SystemInfoUpdateRqst_sequence);

  return offset;
}
static int dissect_MIB_SB_SIB_InformationList_SystemInfoUpdateRqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MIB_SB_SIB_InformationItem_SystemInfoUpdateRqst(tvb, offset, actx, tree, hf_nbap_MIB_SB_SIB_InformationList_SystemInfoUpdateRqst_item);
}


static const per_sequence_t MIB_SB_SIB_InformationList_SystemInfoUpdateRqst_sequence_of[1] = {
  { &hf_nbap_MIB_SB_SIB_InformationList_SystemInfoUpdateRqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_MIB_SB_SIB_InformationItem_SystemInfoUpdateRqst },
};

static int
dissect_nbap_MIB_SB_SIB_InformationList_SystemInfoUpdateRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_MIB_SB_SIB_InformationList_SystemInfoUpdateRqst, MIB_SB_SIB_InformationList_SystemInfoUpdateRqst_sequence_of,
                                                  1, 64);

  return offset;
}
static int dissect_id_MIB_SB_SIB_InformationList_SystemInfoUpdateRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MIB_SB_SIB_InformationList_SystemInfoUpdateRqst(tvb, offset, actx, tree, hf_nbap_id_MIB_SB_SIB_InformationList_SystemInfoUpdateRqst);
}



static int
dissect_nbap_NodeB_CommunicationContextID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 1048575U, NULL, FALSE);

  return offset;
}
static int dissect_id_NodeB_CommunicationContextID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NodeB_CommunicationContextID(tvb, offset, actx, tree, hf_nbap_id_NodeB_CommunicationContextID);
}
static int dissect_nodeB_CommunicationContextID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NodeB_CommunicationContextID(tvb, offset, actx, tree, hf_nbap_nodeB_CommunicationContextID);
}



static int
dissect_nbap_RNC_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 4095U, NULL, FALSE);

  return offset;
}
static int dissect_rNC_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RNC_ID(tvb, offset, actx, tree, hf_nbap_rNC_ID);
}


static const per_sequence_t UC_Id_sequence[] = {
  { &hf_nbap_rNC_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RNC_ID },
  { &hf_nbap_c_ID           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_C_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_UC_Id(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_UC_Id, UC_Id_sequence);

  return offset;
}
static int dissect_uC_Id(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UC_Id(tvb, offset, actx, tree, hf_nbap_uC_Id);
}



static int
dissect_nbap_UARFCN(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 16383U, NULL, TRUE);

  return offset;
}
static int dissect_id_UARFCNforNt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UARFCN(tvb, offset, actx, tree, hf_nbap_id_UARFCNforNt);
}
static int dissect_id_UARFCNforNd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UARFCN(tvb, offset, actx, tree, hf_nbap_id_UARFCNforNd);
}
static int dissect_id_UARFCNforNu(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UARFCN(tvb, offset, actx, tree, hf_nbap_id_UARFCNforNu);
}
static int dissect_uARFCN(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UARFCN(tvb, offset, actx, tree, hf_nbap_uARFCN);
}



static int
dissect_nbap_PrimaryScramblingCode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 511U, NULL, FALSE);

  return offset;
}
static int dissect_id_PrimaryScramblingCode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PrimaryScramblingCode(tvb, offset, actx, tree, hf_nbap_id_PrimaryScramblingCode);
}
static int dissect_primaryScramblingCode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PrimaryScramblingCode(tvb, offset, actx, tree, hf_nbap_primaryScramblingCode);
}


static const per_sequence_t NeighbouringFDDCellMeasurementInformation_sequence[] = {
  { &hf_nbap_uC_Id          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UC_Id },
  { &hf_nbap_uARFCN         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UARFCN },
  { &hf_nbap_primaryScramblingCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PrimaryScramblingCode },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_NeighbouringFDDCellMeasurementInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_NeighbouringFDDCellMeasurementInformation, NeighbouringFDDCellMeasurementInformation_sequence);

  return offset;
}
static int dissect_neighbouringFDDCellMeasurementInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NeighbouringFDDCellMeasurementInformation(tvb, offset, actx, tree, hf_nbap_neighbouringFDDCellMeasurementInformation);
}


static const per_sequence_t NeighbouringTDDCellMeasurementInformation_sequence[] = {
  { &hf_nbap_uC_Id          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UC_Id },
  { &hf_nbap_uARFCN         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UARFCN },
  { &hf_nbap_cellParameterID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CellParameterID },
  { &hf_nbap_timeSlot       , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TimeSlot },
  { &hf_nbap_midambleShiftAndBurstType, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_MidambleShiftAndBurstType },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_NeighbouringTDDCellMeasurementInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_NeighbouringTDDCellMeasurementInformation, NeighbouringTDDCellMeasurementInformation_sequence);

  return offset;
}
static int dissect_neighbouringTDDCellMeasurementInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NeighbouringTDDCellMeasurementInformation(tvb, offset, actx, tree, hf_nbap_neighbouringTDDCellMeasurementInformation);
}



static int
dissect_nbap_Extension_neighbouringCellMeasurementInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_extension_neighbouringCellMeasurementInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Extension_neighbouringCellMeasurementInformation(tvb, offset, actx, tree, hf_nbap_extension_neighbouringCellMeasurementInformation);
}


static const value_string nbap_NeighbouringCellMeasurementInformation_item_vals[] = {
  {   0, "neighbouringFDDCellMeasurementInformation" },
  {   1, "neighbouringTDDCellMeasurementInformation" },
  {   2, "extension-neighbouringCellMeasurementInformation" },
  { 0, NULL }
};

static const per_choice_t NeighbouringCellMeasurementInformation_item_choice[] = {
  {   0, &hf_nbap_neighbouringFDDCellMeasurementInformation, ASN1_EXTENSION_ROOT    , dissect_nbap_NeighbouringFDDCellMeasurementInformation },
  {   1, &hf_nbap_neighbouringTDDCellMeasurementInformation, ASN1_EXTENSION_ROOT    , dissect_nbap_NeighbouringTDDCellMeasurementInformation },
  {   2, &hf_nbap_extension_neighbouringCellMeasurementInformation, ASN1_NOT_EXTENSION_ROOT, dissect_nbap_Extension_neighbouringCellMeasurementInformation },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_NeighbouringCellMeasurementInformation_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_NeighbouringCellMeasurementInformation_item, NeighbouringCellMeasurementInformation_item_choice,
                                 NULL);

  return offset;
}
static int dissect_NeighbouringCellMeasurementInformation_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NeighbouringCellMeasurementInformation_item(tvb, offset, actx, tree, hf_nbap_NeighbouringCellMeasurementInformation_item);
}


static const per_sequence_t NeighbouringCellMeasurementInformation_sequence_of[1] = {
  { &hf_nbap_NeighbouringCellMeasurementInformation_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_NeighbouringCellMeasurementInformation_item },
};

static int
dissect_nbap_NeighbouringCellMeasurementInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_NeighbouringCellMeasurementInformation, NeighbouringCellMeasurementInformation_sequence_of,
                                                  1, 96);

  return offset;
}
static int dissect_id_NeighbouringCellMeasurementInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NeighbouringCellMeasurementInformation(tvb, offset, actx, tree, hf_nbap_id_NeighbouringCellMeasurementInformation);
}



static int
dissect_nbap_PCCPCH_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              -150, 400U, NULL, TRUE);

  return offset;
}
static int dissect_pCCPCH_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PCCPCH_Power(tvb, offset, actx, tree, hf_nbap_pCCPCH_Power);
}


static const per_sequence_t PCCPCH_Information_Cell_ReconfRqstTDD_sequence[] = {
  { &hf_nbap_commonPhysicalChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonPhysicalChannelID },
  { &hf_nbap_pCCPCH_Power   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PCCPCH_Power },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PCCPCH_Information_Cell_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PCCPCH_Information_Cell_ReconfRqstTDD, PCCPCH_Information_Cell_ReconfRqstTDD_sequence);

  return offset;
}
static int dissect_id_PCCPCH_Information_Cell_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PCCPCH_Information_Cell_ReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_id_PCCPCH_Information_Cell_ReconfRqstTDD);
}



static int
dissect_nbap_TDD_PhysicalChannelOffset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 63U, NULL, FALSE);

  return offset;
}
static int dissect_tdd_PhysicalChannelOffset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TDD_PhysicalChannelOffset(tvb, offset, actx, tree, hf_nbap_tdd_PhysicalChannelOffset);
}


static const value_string nbap_SCTD_Indicator_vals[] = {
  {   0, "active" },
  {   1, "inactive" },
  { 0, NULL }
};


static int
dissect_nbap_SCTD_Indicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_sCTD_Indicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SCTD_Indicator(tvb, offset, actx, tree, hf_nbap_sCTD_Indicator);
}


static const per_sequence_t PCCPCH_Information_Cell_SetupRqstTDD_sequence[] = {
  { &hf_nbap_commonPhysicalChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonPhysicalChannelID },
  { &hf_nbap_tdd_PhysicalChannelOffset, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_PhysicalChannelOffset },
  { &hf_nbap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionPeriod },
  { &hf_nbap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionLength },
  { &hf_nbap_pCCPCH_Power   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PCCPCH_Power },
  { &hf_nbap_sCTD_Indicator , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SCTD_Indicator },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PCCPCH_Information_Cell_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PCCPCH_Information_Cell_SetupRqstTDD, PCCPCH_Information_Cell_SetupRqstTDD_sequence);

  return offset;
}
static int dissect_id_PCCPCH_Information_Cell_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PCCPCH_Information_Cell_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_id_PCCPCH_Information_Cell_SetupRqstTDD);
}


static const per_sequence_t PCH_Parameters_CTCH_ReconfRqstTDD_sequence[] = {
  { &hf_nbap_commonTransportChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonTransportChannelID },
  { &hf_nbap_toAWS          , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ToAWS },
  { &hf_nbap_toAWE          , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ToAWE },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PCH_Parameters_CTCH_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PCH_Parameters_CTCH_ReconfRqstTDD, PCH_Parameters_CTCH_ReconfRqstTDD_sequence);

  return offset;
}
static int dissect_id_PCH_Parameters_CTCH_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PCH_Parameters_CTCH_ReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_id_PCH_Parameters_CTCH_ReconfRqstTDD);
}



static int
dissect_nbap_PICH_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              -10, 5U, NULL, FALSE);

  return offset;
}
static int dissect_pICH_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PICH_Power(tvb, offset, actx, tree, hf_nbap_pICH_Power);
}
static int dissect_mICH_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PICH_Power(tvb, offset, actx, tree, hf_nbap_mICH_Power);
}


static const value_string nbap_PICH_Mode_vals[] = {
  {   0, "v18" },
  {   1, "v36" },
  {   2, "v72" },
  {   3, "v144" },
  { 0, NULL }
};


static int
dissect_nbap_PICH_Mode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_pICH_Mode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PICH_Mode(tvb, offset, actx, tree, hf_nbap_pICH_Mode);
}


static const per_sequence_t PICH_Parameters_CTCH_SetupRqstFDD_sequence[] = {
  { &hf_nbap_commonPhysicalChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonPhysicalChannelID },
  { &hf_nbap_fdd_dl_ChannelisationCodeNumber, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_FDD_DL_ChannelisationCodeNumber },
  { &hf_nbap_pICH_Power     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PICH_Power },
  { &hf_nbap_pICH_Mode      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PICH_Mode },
  { &hf_nbap_sTTD_Indicator , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_STTD_Indicator },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PICH_Parameters_CTCH_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PICH_Parameters_CTCH_SetupRqstFDD, PICH_Parameters_CTCH_SetupRqstFDD_sequence);

  return offset;
}
static int dissect_pICH_Parameters(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PICH_Parameters_CTCH_SetupRqstFDD(tvb, offset, actx, tree, hf_nbap_pICH_Parameters);
}


static const per_sequence_t PCH_ParametersItem_CTCH_SetupRqstFDD_sequence[] = {
  { &hf_nbap_commonTransportChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonTransportChannelID },
  { &hf_nbap_transportFormatSet, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TransportFormatSet },
  { &hf_nbap_toAWS          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ToAWS },
  { &hf_nbap_toAWE          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ToAWE },
  { &hf_nbap_pCH_Power      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Power },
  { &hf_nbap_pICH_Parameters, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PICH_Parameters_CTCH_SetupRqstFDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PCH_ParametersItem_CTCH_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PCH_ParametersItem_CTCH_SetupRqstFDD, PCH_ParametersItem_CTCH_SetupRqstFDD_sequence);

  return offset;
}
static int dissect_id_PCH_ParametersItem_CTCH_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PCH_ParametersItem_CTCH_SetupRqstFDD(tvb, offset, actx, tree, hf_nbap_id_PCH_ParametersItem_CTCH_SetupRqstFDD);
}



static int
dissect_nbap_PICH_Parameters_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_pICH_Parameters1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PICH_Parameters_CTCH_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_pICH_Parameters1);
}


static const per_sequence_t PCH_ParametersItem_CTCH_SetupRqstTDD_sequence[] = {
  { &hf_nbap_commonTransportChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonTransportChannelID },
  { &hf_nbap_pCH_CCTrCH_ID  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CCTrCH_ID },
  { &hf_nbap_dl_TransportFormatSet, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TransportFormatSet },
  { &hf_nbap_toAWS          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ToAWS },
  { &hf_nbap_toAWE          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ToAWE },
  { &hf_nbap_pICH_Parameters1, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PICH_Parameters_CTCH_SetupRqstTDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PCH_ParametersItem_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PCH_ParametersItem_CTCH_SetupRqstTDD, PCH_ParametersItem_CTCH_SetupRqstTDD_sequence);

  return offset;
}
static int dissect_id_PCH_ParametersItem_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PCH_ParametersItem_CTCH_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_id_PCH_ParametersItem_CTCH_SetupRqstTDD);
}



static int
dissect_nbap_PDSCH_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 255U, NULL, FALSE);

  return offset;
}
static int dissect_pDSCH_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PDSCH_ID(tvb, offset, actx, tree, hf_nbap_pDSCH_ID);
}


static const per_sequence_t DL_Code_InformationAddItem_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_pDSCH_ID       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PDSCH_ID },
  { &hf_nbap_tdd_ChannelisationCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_ChannelisationCode },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_Code_InformationAddItem_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_Code_InformationAddItem_PSCH_ReconfRqst, DL_Code_InformationAddItem_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_DL_Code_InformationAddList_PSCH_ReconfRqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Code_InformationAddItem_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_DL_Code_InformationAddList_PSCH_ReconfRqst_item);
}


static const per_sequence_t DL_Code_InformationAddList_PSCH_ReconfRqst_sequence_of[1] = {
  { &hf_nbap_DL_Code_InformationAddList_PSCH_ReconfRqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Code_InformationAddItem_PSCH_ReconfRqst },
};

static int
dissect_nbap_DL_Code_InformationAddList_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DL_Code_InformationAddList_PSCH_ReconfRqst, DL_Code_InformationAddList_PSCH_ReconfRqst_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_dL_Code_InformationAddList_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Code_InformationAddList_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_dL_Code_InformationAddList_PSCH_ReconfRqst);
}


static const per_sequence_t DL_Timeslot_InformationAddItem_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_timeSlot       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlot },
  { &hf_nbap_midambleShiftAndBurstType, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MidambleShiftAndBurstType },
  { &hf_nbap_tFCI_Presence  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TFCI_Presence },
  { &hf_nbap_dL_Code_InformationAddList_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Code_InformationAddList_PSCH_ReconfRqst },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_Timeslot_InformationAddItem_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_Timeslot_InformationAddItem_PSCH_ReconfRqst, DL_Timeslot_InformationAddItem_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_DL_Timeslot_InformationAddList_PSCH_ReconfRqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Timeslot_InformationAddItem_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_DL_Timeslot_InformationAddList_PSCH_ReconfRqst_item);
}


static const per_sequence_t DL_Timeslot_InformationAddList_PSCH_ReconfRqst_sequence_of[1] = {
  { &hf_nbap_DL_Timeslot_InformationAddList_PSCH_ReconfRqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Timeslot_InformationAddItem_PSCH_ReconfRqst },
};

static int
dissect_nbap_DL_Timeslot_InformationAddList_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DL_Timeslot_InformationAddList_PSCH_ReconfRqst, DL_Timeslot_InformationAddList_PSCH_ReconfRqst_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_dL_Timeslot_InformationAddList_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Timeslot_InformationAddList_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_dL_Timeslot_InformationAddList_PSCH_ReconfRqst);
}


static const per_sequence_t PDSCH_Information_AddItem_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionPeriod },
  { &hf_nbap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionLength },
  { &hf_nbap_tdd_PhysicalChannelOffset, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_PhysicalChannelOffset },
  { &hf_nbap_dL_Timeslot_InformationAddList_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Timeslot_InformationAddList_PSCH_ReconfRqst },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PDSCH_Information_AddItem_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PDSCH_Information_AddItem_PSCH_ReconfRqst, PDSCH_Information_AddItem_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_id_PDSCH_Information_AddListIE_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PDSCH_Information_AddItem_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_id_PDSCH_Information_AddListIE_PSCH_ReconfRqst);
}


static const per_sequence_t DL_Code_InformationModifyItem_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_pDSCH_ID       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PDSCH_ID },
  { &hf_nbap_tdd_ChannelisationCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_ChannelisationCode },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_Code_InformationModifyItem_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_Code_InformationModifyItem_PSCH_ReconfRqst, DL_Code_InformationModifyItem_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_DL_Code_InformationModifyList_PSCH_ReconfRqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Code_InformationModifyItem_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_DL_Code_InformationModifyList_PSCH_ReconfRqst_item);
}


static const per_sequence_t DL_Code_InformationModifyList_PSCH_ReconfRqst_sequence_of[1] = {
  { &hf_nbap_DL_Code_InformationModifyList_PSCH_ReconfRqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Code_InformationModifyItem_PSCH_ReconfRqst },
};

static int
dissect_nbap_DL_Code_InformationModifyList_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DL_Code_InformationModifyList_PSCH_ReconfRqst, DL_Code_InformationModifyList_PSCH_ReconfRqst_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_dL_Code_InformationModifyList_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Code_InformationModifyList_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_dL_Code_InformationModifyList_PSCH_ReconfRqst);
}


static const per_sequence_t DL_Timeslot_InformationModifyItem_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_timeSlot       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlot },
  { &hf_nbap_midambleShiftAndBurstType, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_MidambleShiftAndBurstType },
  { &hf_nbap_tFCI_Presence  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TFCI_Presence },
  { &hf_nbap_dL_Code_InformationModifyList_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_Code_InformationModifyList_PSCH_ReconfRqst },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_Timeslot_InformationModifyItem_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_Timeslot_InformationModifyItem_PSCH_ReconfRqst, DL_Timeslot_InformationModifyItem_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_DL_Timeslot_InformationModifyList_PSCH_ReconfRqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Timeslot_InformationModifyItem_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_DL_Timeslot_InformationModifyList_PSCH_ReconfRqst_item);
}


static const per_sequence_t DL_Timeslot_InformationModifyList_PSCH_ReconfRqst_sequence_of[1] = {
  { &hf_nbap_DL_Timeslot_InformationModifyList_PSCH_ReconfRqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Timeslot_InformationModifyItem_PSCH_ReconfRqst },
};

static int
dissect_nbap_DL_Timeslot_InformationModifyList_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DL_Timeslot_InformationModifyList_PSCH_ReconfRqst, DL_Timeslot_InformationModifyList_PSCH_ReconfRqst_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_dL_Timeslot_InformationModifyList_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Timeslot_InformationModifyList_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_dL_Timeslot_InformationModifyList_PSCH_ReconfRqst);
}


static const per_sequence_t PDSCH_Information_ModifyItem_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_RepetitionPeriod },
  { &hf_nbap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_RepetitionLength },
  { &hf_nbap_tdd_PhysicalChannelOffset, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TDD_PhysicalChannelOffset },
  { &hf_nbap_dL_Timeslot_InformationModifyList_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_Timeslot_InformationModifyList_PSCH_ReconfRqst },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PDSCH_Information_ModifyItem_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PDSCH_Information_ModifyItem_PSCH_ReconfRqst, PDSCH_Information_ModifyItem_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_id_PDSCH_Information_ModifyListIE_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PDSCH_Information_ModifyItem_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_id_PDSCH_Information_ModifyListIE_PSCH_ReconfRqst);
}



static int
dissect_nbap_PDSCHSet_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 255U, NULL, FALSE);

  return offset;
}
static int dissect_pDSCHSet_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PDSCHSet_ID(tvb, offset, actx, tree, hf_nbap_pDSCHSet_ID);
}



static int
dissect_nbap_PDSCH_Information_AddList_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_pDSCH_InformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PDSCH_Information_AddList_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_pDSCH_InformationList);
}


static const per_sequence_t PDSCHSets_AddItem_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_pDSCHSet_ID    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PDSCHSet_ID },
  { &hf_nbap_pDSCH_InformationList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_PDSCH_Information_AddList_PSCH_ReconfRqst },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PDSCHSets_AddItem_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PDSCHSets_AddItem_PSCH_ReconfRqst, PDSCHSets_AddItem_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_PDSCHSets_AddList_PSCH_ReconfRqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PDSCHSets_AddItem_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_PDSCHSets_AddList_PSCH_ReconfRqst_item);
}


static const per_sequence_t PDSCHSets_AddList_PSCH_ReconfRqst_sequence_of[1] = {
  { &hf_nbap_PDSCHSets_AddList_PSCH_ReconfRqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_PDSCHSets_AddItem_PSCH_ReconfRqst },
};

static int
dissect_nbap_PDSCHSets_AddList_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_PDSCHSets_AddList_PSCH_ReconfRqst, PDSCHSets_AddList_PSCH_ReconfRqst_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_id_PDSCHSets_AddList_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PDSCHSets_AddList_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_id_PDSCHSets_AddList_PSCH_ReconfRqst);
}


static const per_sequence_t PDSCHSets_DeleteItem_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_pDSCHSet_ID    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PDSCHSet_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PDSCHSets_DeleteItem_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PDSCHSets_DeleteItem_PSCH_ReconfRqst, PDSCHSets_DeleteItem_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_PDSCHSets_DeleteList_PSCH_ReconfRqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PDSCHSets_DeleteItem_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_PDSCHSets_DeleteList_PSCH_ReconfRqst_item);
}


static const per_sequence_t PDSCHSets_DeleteList_PSCH_ReconfRqst_sequence_of[1] = {
  { &hf_nbap_PDSCHSets_DeleteList_PSCH_ReconfRqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_PDSCHSets_DeleteItem_PSCH_ReconfRqst },
};

static int
dissect_nbap_PDSCHSets_DeleteList_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_PDSCHSets_DeleteList_PSCH_ReconfRqst, PDSCHSets_DeleteList_PSCH_ReconfRqst_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_id_PDSCHSets_DeleteList_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PDSCHSets_DeleteList_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_id_PDSCHSets_DeleteList_PSCH_ReconfRqst);
}



static int
dissect_nbap_PDSCH_Information_ModifyList_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_pDSCH_InformationList1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PDSCH_Information_ModifyList_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_pDSCH_InformationList1);
}


static const per_sequence_t PDSCHSets_ModifyItem_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_pDSCHSet_ID    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PDSCHSet_ID },
  { &hf_nbap_pDSCH_InformationList1, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PDSCH_Information_ModifyList_PSCH_ReconfRqst },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PDSCHSets_ModifyItem_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PDSCHSets_ModifyItem_PSCH_ReconfRqst, PDSCHSets_ModifyItem_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_PDSCHSets_ModifyList_PSCH_ReconfRqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PDSCHSets_ModifyItem_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_PDSCHSets_ModifyList_PSCH_ReconfRqst_item);
}


static const per_sequence_t PDSCHSets_ModifyList_PSCH_ReconfRqst_sequence_of[1] = {
  { &hf_nbap_PDSCHSets_ModifyList_PSCH_ReconfRqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_PDSCHSets_ModifyItem_PSCH_ReconfRqst },
};

static int
dissect_nbap_PDSCHSets_ModifyList_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_PDSCHSets_ModifyList_PSCH_ReconfRqst, PDSCHSets_ModifyList_PSCH_ReconfRqst_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_id_PDSCHSets_ModifyList_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PDSCHSets_ModifyList_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_id_PDSCHSets_ModifyList_PSCH_ReconfRqst);
}


static const per_sequence_t PICH_Parameters_CTCH_ReconfRqstTDD_sequence[] = {
  { &hf_nbap_commonPhysicalChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonPhysicalChannelID },
  { &hf_nbap_pICH_Power     , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_PICH_Power },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PICH_Parameters_CTCH_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PICH_Parameters_CTCH_ReconfRqstTDD, PICH_Parameters_CTCH_ReconfRqstTDD_sequence);

  return offset;
}
static int dissect_id_PICH_Parameters_CTCH_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PICH_Parameters_CTCH_ReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_id_PICH_Parameters_CTCH_ReconfRqstTDD);
}


static const value_string nbap_PowerAdjustmentType_vals[] = {
  {   0, "none" },
  {   1, "common" },
  {   2, "individual" },
  { 0, NULL }
};


static int
dissect_nbap_PowerAdjustmentType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_PowerAdjustmentType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PowerAdjustmentType(tvb, offset, actx, tree, hf_nbap_id_PowerAdjustmentType);
}
static int dissect_powerAdjustmentType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PowerAdjustmentType(tvb, offset, actx, tree, hf_nbap_powerAdjustmentType);
}


static const per_sequence_t BCH_information_Cell_ReconfRqstFDD_sequence[] = {
  { &hf_nbap_commonTransportChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonTransportChannelID },
  { &hf_nbap_bCH_Power      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Power },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_BCH_information_Cell_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_BCH_information_Cell_ReconfRqstFDD, BCH_information_Cell_ReconfRqstFDD_sequence);

  return offset;
}
static int dissect_bCH_information1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BCH_information_Cell_ReconfRqstFDD(tvb, offset, actx, tree, hf_nbap_bCH_information1);
}


static const per_sequence_t PrimaryCCPCH_Information_Cell_ReconfRqstFDD_sequence[] = {
  { &hf_nbap_bCH_information1, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BCH_information_Cell_ReconfRqstFDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PrimaryCCPCH_Information_Cell_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PrimaryCCPCH_Information_Cell_ReconfRqstFDD, PrimaryCCPCH_Information_Cell_ReconfRqstFDD_sequence);

  return offset;
}
static int dissect_id_PrimaryCCPCH_Information_Cell_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PrimaryCCPCH_Information_Cell_ReconfRqstFDD(tvb, offset, actx, tree, hf_nbap_id_PrimaryCCPCH_Information_Cell_ReconfRqstFDD);
}


static const per_sequence_t BCH_Information_Cell_SetupRqstFDD_sequence[] = {
  { &hf_nbap_commonTransportChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonTransportChannelID },
  { &hf_nbap_bCH_Power      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Power },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_BCH_Information_Cell_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_BCH_Information_Cell_SetupRqstFDD, BCH_Information_Cell_SetupRqstFDD_sequence);

  return offset;
}
static int dissect_bCH_information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BCH_Information_Cell_SetupRqstFDD(tvb, offset, actx, tree, hf_nbap_bCH_information);
}


static const per_sequence_t PrimaryCCPCH_Information_Cell_SetupRqstFDD_sequence[] = {
  { &hf_nbap_commonPhysicalChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonPhysicalChannelID },
  { &hf_nbap_bCH_information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BCH_Information_Cell_SetupRqstFDD },
  { &hf_nbap_sTTD_Indicator , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_STTD_Indicator },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PrimaryCCPCH_Information_Cell_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PrimaryCCPCH_Information_Cell_SetupRqstFDD, PrimaryCCPCH_Information_Cell_SetupRqstFDD_sequence);

  return offset;
}
static int dissect_id_PrimaryCCPCH_Information_Cell_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PrimaryCCPCH_Information_Cell_SetupRqstFDD(tvb, offset, actx, tree, hf_nbap_id_PrimaryCCPCH_Information_Cell_SetupRqstFDD);
}



static int
dissect_nbap_PrimaryCPICH_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              -100, 500U, NULL, FALSE);

  return offset;
}
static int dissect_primaryCPICH_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PrimaryCPICH_Power(tvb, offset, actx, tree, hf_nbap_primaryCPICH_Power);
}


static const per_sequence_t PrimaryCPICH_Information_Cell_ReconfRqstFDD_sequence[] = {
  { &hf_nbap_commonPhysicalChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonPhysicalChannelID },
  { &hf_nbap_primaryCPICH_Power, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PrimaryCPICH_Power },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PrimaryCPICH_Information_Cell_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PrimaryCPICH_Information_Cell_ReconfRqstFDD, PrimaryCPICH_Information_Cell_ReconfRqstFDD_sequence);

  return offset;
}
static int dissect_id_PrimaryCPICH_Information_Cell_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PrimaryCPICH_Information_Cell_ReconfRqstFDD(tvb, offset, actx, tree, hf_nbap_id_PrimaryCPICH_Information_Cell_ReconfRqstFDD);
}


static const value_string nbap_TransmitDiversityIndicator_vals[] = {
  {   0, "active" },
  {   1, "inactive" },
  { 0, NULL }
};


static int
dissect_nbap_TransmitDiversityIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_transmitDiversityIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TransmitDiversityIndicator(tvb, offset, actx, tree, hf_nbap_transmitDiversityIndicator);
}


static const per_sequence_t PrimaryCPICH_Information_Cell_SetupRqstFDD_sequence[] = {
  { &hf_nbap_commonPhysicalChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonPhysicalChannelID },
  { &hf_nbap_primaryCPICH_Power, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PrimaryCPICH_Power },
  { &hf_nbap_transmitDiversityIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TransmitDiversityIndicator },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PrimaryCPICH_Information_Cell_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PrimaryCPICH_Information_Cell_SetupRqstFDD, PrimaryCPICH_Information_Cell_SetupRqstFDD_sequence);

  return offset;
}
static int dissect_id_PrimaryCPICH_Information_Cell_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PrimaryCPICH_Information_Cell_SetupRqstFDD(tvb, offset, actx, tree, hf_nbap_id_PrimaryCPICH_Information_Cell_SetupRqstFDD);
}


static const per_sequence_t PrimarySCH_Information_Cell_ReconfRqstFDD_sequence[] = {
  { &hf_nbap_commonPhysicalChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonPhysicalChannelID },
  { &hf_nbap_primarySCH_Power, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Power },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PrimarySCH_Information_Cell_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PrimarySCH_Information_Cell_ReconfRqstFDD, PrimarySCH_Information_Cell_ReconfRqstFDD_sequence);

  return offset;
}
static int dissect_id_PrimarySCH_Information_Cell_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PrimarySCH_Information_Cell_ReconfRqstFDD(tvb, offset, actx, tree, hf_nbap_id_PrimarySCH_Information_Cell_ReconfRqstFDD);
}


static const value_string nbap_TSTD_Indicator_vals[] = {
  {   0, "active" },
  {   1, "inactive" },
  { 0, NULL }
};


static int
dissect_nbap_TSTD_Indicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_Tstd_indicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TSTD_Indicator(tvb, offset, actx, tree, hf_nbap_id_Tstd_indicator);
}
static int dissect_tSTD_Indicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TSTD_Indicator(tvb, offset, actx, tree, hf_nbap_tSTD_Indicator);
}
static int dissect_tstdIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TSTD_Indicator(tvb, offset, actx, tree, hf_nbap_tstdIndicator);
}


static const per_sequence_t PrimarySCH_Information_Cell_SetupRqstFDD_sequence[] = {
  { &hf_nbap_commonPhysicalChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonPhysicalChannelID },
  { &hf_nbap_primarySCH_Power, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Power },
  { &hf_nbap_tSTD_Indicator , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TSTD_Indicator },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PrimarySCH_Information_Cell_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PrimarySCH_Information_Cell_SetupRqstFDD, PrimarySCH_Information_Cell_SetupRqstFDD_sequence);

  return offset;
}
static int dissect_id_PrimarySCH_Information_Cell_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PrimarySCH_Information_Cell_SetupRqstFDD(tvb, offset, actx, tree, hf_nbap_id_PrimarySCH_Information_Cell_SetupRqstFDD);
}


static const per_sequence_t SCH_Information_Cell_ReconfRqstTDD_sequence[] = {
  { &hf_nbap_commonPhysicalChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonPhysicalChannelID },
  { &hf_nbap_sCH_Power      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Power },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_SCH_Information_Cell_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_SCH_Information_Cell_ReconfRqstTDD, SCH_Information_Cell_ReconfRqstTDD_sequence);

  return offset;
}
static int dissect_id_SCH_Information_Cell_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SCH_Information_Cell_ReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_id_SCH_Information_Cell_ReconfRqstTDD);
}



static int
dissect_nbap_SyncCaseIndicator_Cell_SetupRqstTDD_PSCH(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_syncCaseIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SyncCaseIndicator_Cell_SetupRqstTDD_PSCH(tvb, offset, actx, tree, hf_nbap_syncCaseIndicator);
}


static const per_sequence_t SCH_Information_Cell_SetupRqstTDD_sequence[] = {
  { &hf_nbap_commonPhysicalChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonPhysicalChannelID },
  { &hf_nbap_syncCaseIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SyncCaseIndicator_Cell_SetupRqstTDD_PSCH },
  { &hf_nbap_sCH_Power      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Power },
  { &hf_nbap_tSTD_Indicator , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TSTD_Indicator },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_SCH_Information_Cell_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_SCH_Information_Cell_SetupRqstTDD, SCH_Information_Cell_SetupRqstTDD_sequence);

  return offset;
}
static int dissect_id_SCH_Information_Cell_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SCH_Information_Cell_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_id_SCH_Information_Cell_SetupRqstTDD);
}



static int
dissect_nbap_PUSCH_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 255U, NULL, FALSE);

  return offset;
}
static int dissect_PUSCH_Info_DM_Rqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PUSCH_ID(tvb, offset, actx, tree, hf_nbap_PUSCH_Info_DM_Rqst_item);
}
static int dissect_PUSCH_Info_DM_Rsp_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PUSCH_ID(tvb, offset, actx, tree, hf_nbap_PUSCH_Info_DM_Rsp_item);
}
static int dissect_pUSCH_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PUSCH_ID(tvb, offset, actx, tree, hf_nbap_pUSCH_ID);
}
static int dissect_PUSCH_Info_DM_Rprt_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PUSCH_ID(tvb, offset, actx, tree, hf_nbap_PUSCH_Info_DM_Rprt_item);
}


static const per_sequence_t UL_Code_InformationAddItem_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_pUSCH_ID       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PUSCH_ID },
  { &hf_nbap_tdd_ChannelisationCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_ChannelisationCode },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_UL_Code_InformationAddItem_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_UL_Code_InformationAddItem_PSCH_ReconfRqst, UL_Code_InformationAddItem_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_UL_Code_InformationAddList_PSCH_ReconfRqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_Code_InformationAddItem_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_UL_Code_InformationAddList_PSCH_ReconfRqst_item);
}


static const per_sequence_t UL_Code_InformationAddList_PSCH_ReconfRqst_sequence_of[1] = {
  { &hf_nbap_UL_Code_InformationAddList_PSCH_ReconfRqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_UL_Code_InformationAddItem_PSCH_ReconfRqst },
};

static int
dissect_nbap_UL_Code_InformationAddList_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_UL_Code_InformationAddList_PSCH_ReconfRqst, UL_Code_InformationAddList_PSCH_ReconfRqst_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_uL_Code_InformationAddList_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_Code_InformationAddList_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_uL_Code_InformationAddList_PSCH_ReconfRqst);
}


static const per_sequence_t UL_Timeslot_InformationAddItem_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_timeSlot       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlot },
  { &hf_nbap_midambleShiftAndBurstType, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MidambleShiftAndBurstType },
  { &hf_nbap_tFCI_Presence  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TFCI_Presence },
  { &hf_nbap_uL_Code_InformationAddList_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UL_Code_InformationAddList_PSCH_ReconfRqst },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_UL_Timeslot_InformationAddItem_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_UL_Timeslot_InformationAddItem_PSCH_ReconfRqst, UL_Timeslot_InformationAddItem_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_UL_Timeslot_InformationAddList_PSCH_ReconfRqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_Timeslot_InformationAddItem_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_UL_Timeslot_InformationAddList_PSCH_ReconfRqst_item);
}


static const per_sequence_t UL_Timeslot_InformationAddList_PSCH_ReconfRqst_sequence_of[1] = {
  { &hf_nbap_UL_Timeslot_InformationAddList_PSCH_ReconfRqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_UL_Timeslot_InformationAddItem_PSCH_ReconfRqst },
};

static int
dissect_nbap_UL_Timeslot_InformationAddList_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_UL_Timeslot_InformationAddList_PSCH_ReconfRqst, UL_Timeslot_InformationAddList_PSCH_ReconfRqst_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_uL_Timeslot_InformationAddList_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_Timeslot_InformationAddList_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_uL_Timeslot_InformationAddList_PSCH_ReconfRqst);
}


static const per_sequence_t PUSCH_Information_AddItem_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionPeriod },
  { &hf_nbap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionLength },
  { &hf_nbap_tdd_PhysicalChannelOffset, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_PhysicalChannelOffset },
  { &hf_nbap_uL_Timeslot_InformationAddList_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UL_Timeslot_InformationAddList_PSCH_ReconfRqst },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PUSCH_Information_AddItem_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PUSCH_Information_AddItem_PSCH_ReconfRqst, PUSCH_Information_AddItem_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_id_PUSCH_Information_AddListIE_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PUSCH_Information_AddItem_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_id_PUSCH_Information_AddListIE_PSCH_ReconfRqst);
}


static const per_sequence_t UL_Code_InformationModifyItem_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_pUSCH_ID       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PUSCH_ID },
  { &hf_nbap_tdd_ChannelisationCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_ChannelisationCode },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_UL_Code_InformationModifyItem_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_UL_Code_InformationModifyItem_PSCH_ReconfRqst, UL_Code_InformationModifyItem_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_UL_Code_InformationModifyList_PSCH_ReconfRqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_Code_InformationModifyItem_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_UL_Code_InformationModifyList_PSCH_ReconfRqst_item);
}


static const per_sequence_t UL_Code_InformationModifyList_PSCH_ReconfRqst_sequence_of[1] = {
  { &hf_nbap_UL_Code_InformationModifyList_PSCH_ReconfRqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_UL_Code_InformationModifyItem_PSCH_ReconfRqst },
};

static int
dissect_nbap_UL_Code_InformationModifyList_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_UL_Code_InformationModifyList_PSCH_ReconfRqst, UL_Code_InformationModifyList_PSCH_ReconfRqst_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_uL_Code_InformationModifyList_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_Code_InformationModifyList_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_uL_Code_InformationModifyList_PSCH_ReconfRqst);
}


static const per_sequence_t UL_Timeslot_InformationModifyItem_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_timeSlot       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlot },
  { &hf_nbap_midambleShiftAndBurstType, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_MidambleShiftAndBurstType },
  { &hf_nbap_tFCI_Presence  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TFCI_Presence },
  { &hf_nbap_uL_Code_InformationModifyList_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_UL_Code_InformationModifyList_PSCH_ReconfRqst },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_UL_Timeslot_InformationModifyItem_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_UL_Timeslot_InformationModifyItem_PSCH_ReconfRqst, UL_Timeslot_InformationModifyItem_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_UL_Timeslot_InformationModifyList_PSCH_ReconfRqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_Timeslot_InformationModifyItem_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_UL_Timeslot_InformationModifyList_PSCH_ReconfRqst_item);
}


static const per_sequence_t UL_Timeslot_InformationModifyList_PSCH_ReconfRqst_sequence_of[1] = {
  { &hf_nbap_UL_Timeslot_InformationModifyList_PSCH_ReconfRqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_UL_Timeslot_InformationModifyItem_PSCH_ReconfRqst },
};

static int
dissect_nbap_UL_Timeslot_InformationModifyList_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_UL_Timeslot_InformationModifyList_PSCH_ReconfRqst, UL_Timeslot_InformationModifyList_PSCH_ReconfRqst_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_uL_Timeslot_InformationModifyList_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_Timeslot_InformationModifyList_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_uL_Timeslot_InformationModifyList_PSCH_ReconfRqst);
}


static const per_sequence_t PUSCH_Information_ModifyItem_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_RepetitionPeriod },
  { &hf_nbap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_RepetitionLength },
  { &hf_nbap_tdd_PhysicalChannelOffset, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TDD_PhysicalChannelOffset },
  { &hf_nbap_uL_Timeslot_InformationModifyList_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_UL_Timeslot_InformationModifyList_PSCH_ReconfRqst },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PUSCH_Information_ModifyItem_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PUSCH_Information_ModifyItem_PSCH_ReconfRqst, PUSCH_Information_ModifyItem_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_id_PUSCH_Information_ModifyListIE_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PUSCH_Information_ModifyItem_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_id_PUSCH_Information_ModifyListIE_PSCH_ReconfRqst);
}



static int
dissect_nbap_PUSCHSet_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 255U, NULL, FALSE);

  return offset;
}
static int dissect_pUSCHSet_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PUSCHSet_ID(tvb, offset, actx, tree, hf_nbap_pUSCHSet_ID);
}



static int
dissect_nbap_PUSCH_Information_AddList_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_pUSCH_InformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PUSCH_Information_AddList_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_pUSCH_InformationList);
}


static const per_sequence_t PUSCHSets_AddItem_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_pUSCHSet_ID    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PUSCHSet_ID },
  { &hf_nbap_pUSCH_InformationList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_PUSCH_Information_AddList_PSCH_ReconfRqst },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PUSCHSets_AddItem_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PUSCHSets_AddItem_PSCH_ReconfRqst, PUSCHSets_AddItem_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_PUSCHSets_AddList_PSCH_ReconfRqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PUSCHSets_AddItem_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_PUSCHSets_AddList_PSCH_ReconfRqst_item);
}


static const per_sequence_t PUSCHSets_AddList_PSCH_ReconfRqst_sequence_of[1] = {
  { &hf_nbap_PUSCHSets_AddList_PSCH_ReconfRqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_PUSCHSets_AddItem_PSCH_ReconfRqst },
};

static int
dissect_nbap_PUSCHSets_AddList_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_PUSCHSets_AddList_PSCH_ReconfRqst, PUSCHSets_AddList_PSCH_ReconfRqst_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_id_PUSCHSets_AddList_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PUSCHSets_AddList_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_id_PUSCHSets_AddList_PSCH_ReconfRqst);
}


static const per_sequence_t PUSCHSets_DeleteItem_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_pUSCHSet_ID    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PUSCHSet_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PUSCHSets_DeleteItem_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PUSCHSets_DeleteItem_PSCH_ReconfRqst, PUSCHSets_DeleteItem_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_PUSCHSets_DeleteList_PSCH_ReconfRqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PUSCHSets_DeleteItem_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_PUSCHSets_DeleteList_PSCH_ReconfRqst_item);
}


static const per_sequence_t PUSCHSets_DeleteList_PSCH_ReconfRqst_sequence_of[1] = {
  { &hf_nbap_PUSCHSets_DeleteList_PSCH_ReconfRqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_PUSCHSets_DeleteItem_PSCH_ReconfRqst },
};

static int
dissect_nbap_PUSCHSets_DeleteList_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_PUSCHSets_DeleteList_PSCH_ReconfRqst, PUSCHSets_DeleteList_PSCH_ReconfRqst_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_id_PUSCHSets_DeleteList_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PUSCHSets_DeleteList_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_id_PUSCHSets_DeleteList_PSCH_ReconfRqst);
}



static int
dissect_nbap_PUSCH_Information_ModifyList_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_pUSCH_InformationList1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PUSCH_Information_ModifyList_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_pUSCH_InformationList1);
}


static const per_sequence_t PUSCHSets_ModifyItem_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_pUSCHSet_ID    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PUSCHSet_ID },
  { &hf_nbap_pUSCH_InformationList1, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PUSCH_Information_ModifyList_PSCH_ReconfRqst },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PUSCHSets_ModifyItem_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PUSCHSets_ModifyItem_PSCH_ReconfRqst, PUSCHSets_ModifyItem_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_PUSCHSets_ModifyList_PSCH_ReconfRqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PUSCHSets_ModifyItem_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_PUSCHSets_ModifyList_PSCH_ReconfRqst_item);
}


static const per_sequence_t PUSCHSets_ModifyList_PSCH_ReconfRqst_sequence_of[1] = {
  { &hf_nbap_PUSCHSets_ModifyList_PSCH_ReconfRqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_PUSCHSets_ModifyItem_PSCH_ReconfRqst },
};

static int
dissect_nbap_PUSCHSets_ModifyList_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_PUSCHSets_ModifyList_PSCH_ReconfRqst, PUSCHSets_ModifyList_PSCH_ReconfRqst_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_id_PUSCHSets_ModifyList_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PUSCHSets_ModifyList_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_id_PUSCHSets_ModifyList_PSCH_ReconfRqst);
}


static const per_sequence_t RACH_ParametersItem_CTCH_SetupRqstFDD_sequence[] = {
  { &hf_nbap_commonTransportChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonTransportChannelID },
  { &hf_nbap_transportFormatSet, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TransportFormatSet },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RACH_ParametersItem_CTCH_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RACH_ParametersItem_CTCH_SetupRqstFDD, RACH_ParametersItem_CTCH_SetupRqstFDD_sequence);

  return offset;
}
static int dissect_id_RACH_ParametersItem_CTCH_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RACH_ParametersItem_CTCH_SetupRqstFDD(tvb, offset, actx, tree, hf_nbap_id_RACH_ParametersItem_CTCH_SetupRqstFDD);
}


static const per_sequence_t RACH_ParameterItem_CTCH_SetupRqstTDD_sequence[] = {
  { &hf_nbap_commonTransportChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonTransportChannelID },
  { &hf_nbap_uL_TransportFormatSet, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TransportFormatSet },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RACH_ParameterItem_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RACH_ParameterItem_CTCH_SetupRqstTDD, RACH_ParameterItem_CTCH_SetupRqstTDD_sequence);

  return offset;
}
static int dissect_id_RACH_ParameterItem_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RACH_ParameterItem_CTCH_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_id_RACH_ParameterItem_CTCH_SetupRqstTDD);
}



static int
dissect_nbap_ReportPeriodicity_Scaledmsec(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 6000U, NULL, TRUE);

  return offset;
}
static int dissect_msec2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ReportPeriodicity_Scaledmsec(tvb, offset, actx, tree, hf_nbap_msec2);
}



static int
dissect_nbap_ReportPeriodicity_Scaledmin(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 60U, NULL, TRUE);

  return offset;
}
static int dissect_min(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ReportPeriodicity_Scaledmin(tvb, offset, actx, tree, hf_nbap_min);
}


static const value_string nbap_ReportCharacteristicsType_ReportPeriodicity_vals[] = {
  {   0, "msec" },
  {   1, "min" },
  { 0, NULL }
};

static const per_choice_t ReportCharacteristicsType_ReportPeriodicity_choice[] = {
  {   0, &hf_nbap_msec2          , ASN1_EXTENSION_ROOT    , dissect_nbap_ReportPeriodicity_Scaledmsec },
  {   1, &hf_nbap_min            , ASN1_EXTENSION_ROOT    , dissect_nbap_ReportPeriodicity_Scaledmin },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_ReportCharacteristicsType_ReportPeriodicity(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_ReportCharacteristicsType_ReportPeriodicity, ReportCharacteristicsType_ReportPeriodicity_choice,
                                 NULL);

  return offset;
}
static int dissect_periodic1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ReportCharacteristicsType_ReportPeriodicity(tvb, offset, actx, tree, hf_nbap_periodic1);
}
static int dissect_reportPeriodicity(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ReportCharacteristicsType_ReportPeriodicity(tvb, offset, actx, tree, hf_nbap_reportPeriodicity);
}



static int
dissect_nbap_SIR_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 63U, NULL, FALSE);

  return offset;
}
static int dissect_sIRValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SIR_Value(tvb, offset, actx, tree, hf_nbap_sIRValue);
}
static int dissect_sIR_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SIR_Value(tvb, offset, actx, tree, hf_nbap_sIR_Value);
}
static int dissect_sir1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SIR_Value(tvb, offset, actx, tree, hf_nbap_sir1);
}



static int
dissect_nbap_SIR_Error_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 125U, NULL, FALSE);

  return offset;
}
static int dissect_sIR_ErrorValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SIR_Error_Value(tvb, offset, actx, tree, hf_nbap_sIR_ErrorValue);
}
static int dissect_sir_error1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SIR_Error_Value(tvb, offset, actx, tree, hf_nbap_sir_error1);
}



static int
dissect_nbap_Transmitted_Code_Power_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 127U, NULL, FALSE);

  return offset;
}
static int dissect_transmittedCodePowerValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Transmitted_Code_Power_Value(tvb, offset, actx, tree, hf_nbap_transmittedCodePowerValue);
}
static int dissect_transmitted_code_power1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Transmitted_Code_Power_Value(tvb, offset, actx, tree, hf_nbap_transmitted_code_power1);
}



static int
dissect_nbap_RSCP_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 127U, NULL, FALSE);

  return offset;
}
static int dissect_rSCP(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RSCP_Value(tvb, offset, actx, tree, hf_nbap_rSCP);
}
static int dissect_rscp1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RSCP_Value(tvb, offset, actx, tree, hf_nbap_rscp1);
}



static int
dissect_nbap_Rx_Timing_Deviation_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 8191U, NULL, FALSE);

  return offset;
}
static int dissect_rxTimingDeviationValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Rx_Timing_Deviation_Value(tvb, offset, actx, tree, hf_nbap_rxTimingDeviationValue);
}
static int dissect_rx_timing_deviation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Rx_Timing_Deviation_Value(tvb, offset, actx, tree, hf_nbap_rx_timing_deviation);
}



static int
dissect_nbap_Round_Trip_Time_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 32767U, NULL, FALSE);

  return offset;
}
static int dissect_roundTripTime(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Round_Trip_Time_Value(tvb, offset, actx, tree, hf_nbap_roundTripTime);
}
static int dissect_round_trip_time1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Round_Trip_Time_Value(tvb, offset, actx, tree, hf_nbap_round_trip_time1);
}



static int
dissect_nbap_Extension_ReportCharacteristicsType_MeasurementThreshold(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_extension_ReportCharacteristicsType_MeasurementThreshold(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Extension_ReportCharacteristicsType_MeasurementThreshold(tvb, offset, actx, tree, hf_nbap_extension_ReportCharacteristicsType_MeasurementThreshold);
}


static const value_string nbap_ReportCharacteristicsType_MeasurementThreshold_vals[] = {
  {   0, "received-total-wide-band-power" },
  {   1, "transmitted-carrier-power" },
  {   2, "acknowledged-prach-preambles" },
  {   3, "uL-TimeslotISCP" },
  {   4, "sir" },
  {   5, "sir-error" },
  {   6, "transmitted-code-power" },
  {   7, "rscp" },
  {   8, "rx-timing-deviation" },
  {   9, "round-trip-time" },
  {  10, "notUsed-1-acknowledged-PCPCH-access-preambles" },
  {  11, "notUsed-2-detected-PCPCH-access-preambles" },
  {  12, "extension-ReportCharacteristicsType-MeasurementThreshold" },
  { 0, NULL }
};

static const per_choice_t ReportCharacteristicsType_MeasurementThreshold_choice[] = {
  {   0, &hf_nbap_received_total_wide_band_power, ASN1_EXTENSION_ROOT    , dissect_nbap_Received_total_wide_band_power_Value },
  {   1, &hf_nbap_transmitted_carrier_power, ASN1_EXTENSION_ROOT    , dissect_nbap_Transmitted_Carrier_Power_Value },
  {   2, &hf_nbap_acknowledged_prach_preambles, ASN1_EXTENSION_ROOT    , dissect_nbap_Acknowledged_PRACH_preambles_Value },
  {   3, &hf_nbap_uL_TimeslotISCP, ASN1_EXTENSION_ROOT    , dissect_nbap_UL_TimeslotISCP_Value },
  {   4, &hf_nbap_sir1           , ASN1_EXTENSION_ROOT    , dissect_nbap_SIR_Value },
  {   5, &hf_nbap_sir_error1     , ASN1_EXTENSION_ROOT    , dissect_nbap_SIR_Error_Value },
  {   6, &hf_nbap_transmitted_code_power1, ASN1_EXTENSION_ROOT    , dissect_nbap_Transmitted_Code_Power_Value },
  {   7, &hf_nbap_rscp1          , ASN1_EXTENSION_ROOT    , dissect_nbap_RSCP_Value },
  {   8, &hf_nbap_rx_timing_deviation, ASN1_EXTENSION_ROOT    , dissect_nbap_Rx_Timing_Deviation_Value },
  {   9, &hf_nbap_round_trip_time1, ASN1_EXTENSION_ROOT    , dissect_nbap_Round_Trip_Time_Value },
  {  10, &hf_nbap_notUsed_1_acknowledged_PCPCH_access_preambles, ASN1_EXTENSION_ROOT    , dissect_nbap_NULL },
  {  11, &hf_nbap_notUsed_2_detected_PCPCH_access_preambles, ASN1_EXTENSION_ROOT    , dissect_nbap_NULL },
  {  12, &hf_nbap_extension_ReportCharacteristicsType_MeasurementThreshold, ASN1_NOT_EXTENSION_ROOT, dissect_nbap_Extension_ReportCharacteristicsType_MeasurementThreshold },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_ReportCharacteristicsType_MeasurementThreshold(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_ReportCharacteristicsType_MeasurementThreshold, ReportCharacteristicsType_MeasurementThreshold_choice,
                                 NULL);

  return offset;
}
static int dissect_measurementThreshold(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ReportCharacteristicsType_MeasurementThreshold(tvb, offset, actx, tree, hf_nbap_measurementThreshold);
}
static int dissect_measurementThreshold1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ReportCharacteristicsType_MeasurementThreshold(tvb, offset, actx, tree, hf_nbap_measurementThreshold1);
}
static int dissect_measurementThreshold2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ReportCharacteristicsType_MeasurementThreshold(tvb, offset, actx, tree, hf_nbap_measurementThreshold2);
}



static int
dissect_nbap_MeasurementHysteresisTime_Scaledmsec(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 6000U, NULL, TRUE);

  return offset;
}
static int dissect_msec1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MeasurementHysteresisTime_Scaledmsec(tvb, offset, actx, tree, hf_nbap_msec1);
}


static const value_string nbap_ReportCharacteristicsType_ScaledMeasurementHysteresisTime_vals[] = {
  {   0, "msec" },
  { 0, NULL }
};

static const per_choice_t ReportCharacteristicsType_ScaledMeasurementHysteresisTime_choice[] = {
  {   0, &hf_nbap_msec1          , ASN1_EXTENSION_ROOT    , dissect_nbap_MeasurementHysteresisTime_Scaledmsec },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_ReportCharacteristicsType_ScaledMeasurementHysteresisTime(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_ReportCharacteristicsType_ScaledMeasurementHysteresisTime, ReportCharacteristicsType_ScaledMeasurementHysteresisTime_choice,
                                 NULL);

  return offset;
}
static int dissect_measurementHysteresisTime(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ReportCharacteristicsType_ScaledMeasurementHysteresisTime(tvb, offset, actx, tree, hf_nbap_measurementHysteresisTime);
}


static const per_sequence_t ReportCharacteristicsType_EventA_sequence[] = {
  { &hf_nbap_measurementThreshold, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ReportCharacteristicsType_MeasurementThreshold },
  { &hf_nbap_measurementHysteresisTime, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ReportCharacteristicsType_ScaledMeasurementHysteresisTime },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_ReportCharacteristicsType_EventA(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_ReportCharacteristicsType_EventA, ReportCharacteristicsType_EventA_sequence);

  return offset;
}
static int dissect_event_a(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ReportCharacteristicsType_EventA(tvb, offset, actx, tree, hf_nbap_event_a);
}


static const per_sequence_t ReportCharacteristicsType_EventB_sequence[] = {
  { &hf_nbap_measurementThreshold, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ReportCharacteristicsType_MeasurementThreshold },
  { &hf_nbap_measurementHysteresisTime, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ReportCharacteristicsType_ScaledMeasurementHysteresisTime },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_ReportCharacteristicsType_EventB(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_ReportCharacteristicsType_EventB, ReportCharacteristicsType_EventB_sequence);

  return offset;
}
static int dissect_event_b(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ReportCharacteristicsType_EventB(tvb, offset, actx, tree, hf_nbap_event_b);
}



static int
dissect_nbap_Received_total_wide_band_power_Value_IncrDecrThres(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 620U, NULL, FALSE);

  return offset;
}
static int dissect_id_Received_total_wide_band_power_For_CellPortion(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Received_total_wide_band_power_Value_IncrDecrThres(tvb, offset, actx, tree, hf_nbap_id_Received_total_wide_band_power_For_CellPortion);
}
static int dissect_received_total_wide_band_power1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Received_total_wide_band_power_Value_IncrDecrThres(tvb, offset, actx, tree, hf_nbap_received_total_wide_band_power1);
}



static int
dissect_nbap_UL_TimeslotISCP_Value_IncrDecrThres(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 126U, NULL, FALSE);

  return offset;
}
static int dissect_uL_TimeslotISCP1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_TimeslotISCP_Value_IncrDecrThres(tvb, offset, actx, tree, hf_nbap_uL_TimeslotISCP1);
}



static int
dissect_nbap_SIR_Value_IncrDecrThres(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 62U, NULL, FALSE);

  return offset;
}
static int dissect_sir(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SIR_Value_IncrDecrThres(tvb, offset, actx, tree, hf_nbap_sir);
}



static int
dissect_nbap_SIR_Error_Value_IncrDecrThres(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 124U, NULL, FALSE);

  return offset;
}
static int dissect_sir_error(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SIR_Error_Value_IncrDecrThres(tvb, offset, actx, tree, hf_nbap_sir_error);
}



static int
dissect_nbap_Transmitted_Code_Power_Value_IncrDecrThres(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 112U, NULL, TRUE);

  return offset;
}
static int dissect_transmitted_code_power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Transmitted_Code_Power_Value_IncrDecrThres(tvb, offset, actx, tree, hf_nbap_transmitted_code_power);
}



static int
dissect_nbap_RSCP_Value_IncrDecrThres(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 126U, NULL, FALSE);

  return offset;
}
static int dissect_rscp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RSCP_Value_IncrDecrThres(tvb, offset, actx, tree, hf_nbap_rscp);
}



static int
dissect_nbap_Round_Trip_Time_IncrDecrThres(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 32766U, NULL, FALSE);

  return offset;
}
static int dissect_round_trip_time(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Round_Trip_Time_IncrDecrThres(tvb, offset, actx, tree, hf_nbap_round_trip_time);
}



static int
dissect_nbap_Extension_ReportCharacteristicsType_MeasurementIncreaseDecreaseThreshold(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_extension_ReportCharacteristicsType_MeasurementIncreaseDecreaseThreshold(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Extension_ReportCharacteristicsType_MeasurementIncreaseDecreaseThreshold(tvb, offset, actx, tree, hf_nbap_extension_ReportCharacteristicsType_MeasurementIncreaseDecreaseThreshold);
}


static const value_string nbap_ReportCharacteristicsType_MeasurementIncreaseDecreaseThreshold_vals[] = {
  {   0, "received-total-wide-band-power" },
  {   1, "transmitted-carrier-power" },
  {   2, "acknowledged-prach-preambles" },
  {   3, "uL-TimeslotISCP" },
  {   4, "sir" },
  {   5, "sir-error" },
  {   6, "transmitted-code-power" },
  {   7, "rscp" },
  {   8, "round-trip-time" },
  {   9, "notUsed-1-acknowledged-PCPCH-access-preambles" },
  {  10, "notUsed-2-detected-PCPCH-access-preambles" },
  {  11, "extension-ReportCharacteristicsType-MeasurementIncreaseDecreaseThreshold" },
  { 0, NULL }
};

static const per_choice_t ReportCharacteristicsType_MeasurementIncreaseDecreaseThreshold_choice[] = {
  {   0, &hf_nbap_received_total_wide_band_power1, ASN1_EXTENSION_ROOT    , dissect_nbap_Received_total_wide_band_power_Value_IncrDecrThres },
  {   1, &hf_nbap_transmitted_carrier_power, ASN1_EXTENSION_ROOT    , dissect_nbap_Transmitted_Carrier_Power_Value },
  {   2, &hf_nbap_acknowledged_prach_preambles, ASN1_EXTENSION_ROOT    , dissect_nbap_Acknowledged_PRACH_preambles_Value },
  {   3, &hf_nbap_uL_TimeslotISCP1, ASN1_EXTENSION_ROOT    , dissect_nbap_UL_TimeslotISCP_Value_IncrDecrThres },
  {   4, &hf_nbap_sir            , ASN1_EXTENSION_ROOT    , dissect_nbap_SIR_Value_IncrDecrThres },
  {   5, &hf_nbap_sir_error      , ASN1_EXTENSION_ROOT    , dissect_nbap_SIR_Error_Value_IncrDecrThres },
  {   6, &hf_nbap_transmitted_code_power, ASN1_EXTENSION_ROOT    , dissect_nbap_Transmitted_Code_Power_Value_IncrDecrThres },
  {   7, &hf_nbap_rscp           , ASN1_EXTENSION_ROOT    , dissect_nbap_RSCP_Value_IncrDecrThres },
  {   8, &hf_nbap_round_trip_time, ASN1_EXTENSION_ROOT    , dissect_nbap_Round_Trip_Time_IncrDecrThres },
  {   9, &hf_nbap_notUsed_1_acknowledged_PCPCH_access_preambles, ASN1_EXTENSION_ROOT    , dissect_nbap_NULL },
  {  10, &hf_nbap_notUsed_2_detected_PCPCH_access_preambles, ASN1_EXTENSION_ROOT    , dissect_nbap_NULL },
  {  11, &hf_nbap_extension_ReportCharacteristicsType_MeasurementIncreaseDecreaseThreshold, ASN1_NOT_EXTENSION_ROOT, dissect_nbap_Extension_ReportCharacteristicsType_MeasurementIncreaseDecreaseThreshold },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_ReportCharacteristicsType_MeasurementIncreaseDecreaseThreshold(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_ReportCharacteristicsType_MeasurementIncreaseDecreaseThreshold, ReportCharacteristicsType_MeasurementIncreaseDecreaseThreshold_choice,
                                 NULL);

  return offset;
}
static int dissect_measurementIncreaseThreshold(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ReportCharacteristicsType_MeasurementIncreaseDecreaseThreshold(tvb, offset, actx, tree, hf_nbap_measurementIncreaseThreshold);
}
static int dissect_measurementDecreaseThreshold(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ReportCharacteristicsType_MeasurementIncreaseDecreaseThreshold(tvb, offset, actx, tree, hf_nbap_measurementDecreaseThreshold);
}



static int
dissect_nbap_MeasurementChangeTime_Scaledmsec(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 6000U, NULL, TRUE);

  return offset;
}
static int dissect_msec(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MeasurementChangeTime_Scaledmsec(tvb, offset, actx, tree, hf_nbap_msec);
}


static const value_string nbap_ReportCharacteristicsType_ScaledMeasurementChangeTime_vals[] = {
  {   0, "msec" },
  { 0, NULL }
};

static const per_choice_t ReportCharacteristicsType_ScaledMeasurementChangeTime_choice[] = {
  {   0, &hf_nbap_msec           , ASN1_EXTENSION_ROOT    , dissect_nbap_MeasurementChangeTime_Scaledmsec },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_ReportCharacteristicsType_ScaledMeasurementChangeTime(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_ReportCharacteristicsType_ScaledMeasurementChangeTime, ReportCharacteristicsType_ScaledMeasurementChangeTime_choice,
                                 NULL);

  return offset;
}
static int dissect_measurementChangeTime(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ReportCharacteristicsType_ScaledMeasurementChangeTime(tvb, offset, actx, tree, hf_nbap_measurementChangeTime);
}


static const per_sequence_t ReportCharacteristicsType_EventC_sequence[] = {
  { &hf_nbap_measurementIncreaseThreshold, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ReportCharacteristicsType_MeasurementIncreaseDecreaseThreshold },
  { &hf_nbap_measurementChangeTime, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ReportCharacteristicsType_ScaledMeasurementChangeTime },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_ReportCharacteristicsType_EventC(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_ReportCharacteristicsType_EventC, ReportCharacteristicsType_EventC_sequence);

  return offset;
}
static int dissect_event_c(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ReportCharacteristicsType_EventC(tvb, offset, actx, tree, hf_nbap_event_c);
}


static const per_sequence_t ReportCharacteristicsType_EventD_sequence[] = {
  { &hf_nbap_measurementDecreaseThreshold, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ReportCharacteristicsType_MeasurementIncreaseDecreaseThreshold },
  { &hf_nbap_measurementChangeTime, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ReportCharacteristicsType_ScaledMeasurementChangeTime },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_ReportCharacteristicsType_EventD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_ReportCharacteristicsType_EventD, ReportCharacteristicsType_EventD_sequence);

  return offset;
}
static int dissect_event_d(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ReportCharacteristicsType_EventD(tvb, offset, actx, tree, hf_nbap_event_d);
}


static const per_sequence_t ReportCharacteristicsType_EventE_sequence[] = {
  { &hf_nbap_measurementThreshold1, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ReportCharacteristicsType_MeasurementThreshold },
  { &hf_nbap_measurementThreshold2, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ReportCharacteristicsType_MeasurementThreshold },
  { &hf_nbap_measurementHysteresisTime, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ReportCharacteristicsType_ScaledMeasurementHysteresisTime },
  { &hf_nbap_reportPeriodicity, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ReportCharacteristicsType_ReportPeriodicity },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_ReportCharacteristicsType_EventE(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_ReportCharacteristicsType_EventE, ReportCharacteristicsType_EventE_sequence);

  return offset;
}
static int dissect_event_e(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ReportCharacteristicsType_EventE(tvb, offset, actx, tree, hf_nbap_event_e);
}


static const per_sequence_t ReportCharacteristicsType_EventF_sequence[] = {
  { &hf_nbap_measurementThreshold1, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ReportCharacteristicsType_MeasurementThreshold },
  { &hf_nbap_measurementThreshold2, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ReportCharacteristicsType_MeasurementThreshold },
  { &hf_nbap_measurementHysteresisTime, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ReportCharacteristicsType_ScaledMeasurementHysteresisTime },
  { &hf_nbap_reportPeriodicity, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ReportCharacteristicsType_ReportPeriodicity },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_ReportCharacteristicsType_EventF(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_ReportCharacteristicsType_EventF, ReportCharacteristicsType_EventF_sequence);

  return offset;
}
static int dissect_event_f(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ReportCharacteristicsType_EventF(tvb, offset, actx, tree, hf_nbap_event_f);
}



static int
dissect_nbap_Extension_ReportCharacteristics(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_extension_ReportCharacteristics(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Extension_ReportCharacteristics(tvb, offset, actx, tree, hf_nbap_extension_ReportCharacteristics);
}


static const value_string nbap_ReportCharacteristics_vals[] = {
  {   0, "onDemand" },
  {   1, "periodic" },
  {   2, "event-a" },
  {   3, "event-b" },
  {   4, "event-c" },
  {   5, "event-d" },
  {   6, "event-e" },
  {   7, "event-f" },
  {   8, "extension-ReportCharacteristics" },
  { 0, NULL }
};

static const per_choice_t ReportCharacteristics_choice[] = {
  {   0, &hf_nbap_onDemand       , ASN1_EXTENSION_ROOT    , dissect_nbap_NULL },
  {   1, &hf_nbap_periodic1      , ASN1_EXTENSION_ROOT    , dissect_nbap_ReportCharacteristicsType_ReportPeriodicity },
  {   2, &hf_nbap_event_a        , ASN1_EXTENSION_ROOT    , dissect_nbap_ReportCharacteristicsType_EventA },
  {   3, &hf_nbap_event_b        , ASN1_EXTENSION_ROOT    , dissect_nbap_ReportCharacteristicsType_EventB },
  {   4, &hf_nbap_event_c        , ASN1_EXTENSION_ROOT    , dissect_nbap_ReportCharacteristicsType_EventC },
  {   5, &hf_nbap_event_d        , ASN1_EXTENSION_ROOT    , dissect_nbap_ReportCharacteristicsType_EventD },
  {   6, &hf_nbap_event_e        , ASN1_EXTENSION_ROOT    , dissect_nbap_ReportCharacteristicsType_EventE },
  {   7, &hf_nbap_event_f        , ASN1_EXTENSION_ROOT    , dissect_nbap_ReportCharacteristicsType_EventF },
  {   8, &hf_nbap_extension_ReportCharacteristics, ASN1_NOT_EXTENSION_ROOT, dissect_nbap_Extension_ReportCharacteristics },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_ReportCharacteristics(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_ReportCharacteristics, ReportCharacteristics_choice,
                                 NULL);

  return offset;
}
static int dissect_id_ReportCharacteristics(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ReportCharacteristics(tvb, offset, actx, tree, hf_nbap_id_ReportCharacteristics);
}


static const per_sequence_t RL_InformationList_RL_FailureInd_sequence_of[1] = {
  { &hf_nbap_RL_InformationList_RL_FailureInd_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_RL_InformationList_RL_FailureInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_RL_InformationList_RL_FailureInd, RL_InformationList_RL_FailureInd_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_rL_InformationList_RL_FailureInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_InformationList_RL_FailureInd(tvb, offset, actx, tree, hf_nbap_rL_InformationList_RL_FailureInd);
}


static const per_sequence_t RL_RL_FailureInd_sequence[] = {
  { &hf_nbap_rL_InformationList_RL_FailureInd, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_InformationList_RL_FailureInd },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RL_RL_FailureInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RL_RL_FailureInd, RL_RL_FailureInd_sequence);

  return offset;
}
static int dissect_rL3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_RL_FailureInd(tvb, offset, actx, tree, hf_nbap_rL3);
}


static const per_sequence_t RL_Set_InformationList_RL_FailureInd_sequence_of[1] = {
  { &hf_nbap_RL_Set_InformationList_RL_FailureInd_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_RL_Set_InformationList_RL_FailureInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_RL_Set_InformationList_RL_FailureInd, RL_Set_InformationList_RL_FailureInd_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_rL_Set_InformationList_RL_FailureInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_Set_InformationList_RL_FailureInd(tvb, offset, actx, tree, hf_nbap_rL_Set_InformationList_RL_FailureInd);
}


static const per_sequence_t RL_Set_RL_FailureInd_sequence[] = {
  { &hf_nbap_rL_Set_InformationList_RL_FailureInd, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_Set_InformationList_RL_FailureInd },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RL_Set_RL_FailureInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RL_Set_RL_FailureInd, RL_Set_RL_FailureInd_sequence);

  return offset;
}
static int dissect_rL_Set(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_Set_RL_FailureInd(tvb, offset, actx, tree, hf_nbap_rL_Set);
}


static const per_sequence_t CCTrCH_InformationList_RL_FailureInd_sequence_of[1] = {
  { &hf_nbap_CCTrCH_InformationList_RL_FailureInd_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_CCTrCH_InformationList_RL_FailureInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_CCTrCH_InformationList_RL_FailureInd, CCTrCH_InformationList_RL_FailureInd_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_cCTrCH_InformationList_RL_FailureInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CCTrCH_InformationList_RL_FailureInd(tvb, offset, actx, tree, hf_nbap_cCTrCH_InformationList_RL_FailureInd);
}


static const per_sequence_t CCTrCH_RL_FailureInd_sequence[] = {
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_ID },
  { &hf_nbap_cCTrCH_InformationList_RL_FailureInd, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CCTrCH_InformationList_RL_FailureInd },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CCTrCH_RL_FailureInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CCTrCH_RL_FailureInd, CCTrCH_RL_FailureInd_sequence);

  return offset;
}
static int dissect_cCTrCH(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CCTrCH_RL_FailureInd(tvb, offset, actx, tree, hf_nbap_cCTrCH);
}


static const value_string nbap_Reporting_Object_RL_FailureInd_vals[] = {
  {   0, "rL" },
  {   1, "rL-Set" },
  {   2, "cCTrCH" },
  { 0, NULL }
};

static const per_choice_t Reporting_Object_RL_FailureInd_choice[] = {
  {   0, &hf_nbap_rL3            , ASN1_EXTENSION_ROOT    , dissect_nbap_RL_RL_FailureInd },
  {   1, &hf_nbap_rL_Set         , ASN1_EXTENSION_ROOT    , dissect_nbap_RL_Set_RL_FailureInd },
  {   2, &hf_nbap_cCTrCH         , ASN1_NOT_EXTENSION_ROOT, dissect_nbap_CCTrCH_RL_FailureInd },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_Reporting_Object_RL_FailureInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_Reporting_Object_RL_FailureInd, Reporting_Object_RL_FailureInd_choice,
                                 NULL);

  return offset;
}
static int dissect_id_Reporting_Object_RL_FailureInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Reporting_Object_RL_FailureInd(tvb, offset, actx, tree, hf_nbap_id_Reporting_Object_RL_FailureInd);
}


static const per_sequence_t RL_InformationList_RL_RestoreInd_sequence_of[1] = {
  { &hf_nbap_RL_InformationList_RL_RestoreInd_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_RL_InformationList_RL_RestoreInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_RL_InformationList_RL_RestoreInd, RL_InformationList_RL_RestoreInd_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_rL_InformationList_RL_RestoreInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_InformationList_RL_RestoreInd(tvb, offset, actx, tree, hf_nbap_rL_InformationList_RL_RestoreInd);
}


static const per_sequence_t RL_RL_RestoreInd_sequence[] = {
  { &hf_nbap_rL_InformationList_RL_RestoreInd, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_InformationList_RL_RestoreInd },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RL_RL_RestoreInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RL_RL_RestoreInd, RL_RL_RestoreInd_sequence);

  return offset;
}
static int dissect_rL4(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_RL_RestoreInd(tvb, offset, actx, tree, hf_nbap_rL4);
}


static const per_sequence_t RL_Set_InformationList_RL_RestoreInd_sequence_of[1] = {
  { &hf_nbap_RL_Set_InformationList_RL_RestoreInd_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_RL_Set_InformationList_RL_RestoreInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_RL_Set_InformationList_RL_RestoreInd, RL_Set_InformationList_RL_RestoreInd_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_rL_Set_InformationList_RL_RestoreInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_Set_InformationList_RL_RestoreInd(tvb, offset, actx, tree, hf_nbap_rL_Set_InformationList_RL_RestoreInd);
}


static const per_sequence_t RL_Set_RL_RestoreInd_sequence[] = {
  { &hf_nbap_rL_Set_InformationList_RL_RestoreInd, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_Set_InformationList_RL_RestoreInd },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RL_Set_RL_RestoreInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RL_Set_RL_RestoreInd, RL_Set_RL_RestoreInd_sequence);

  return offset;
}
static int dissect_rL_Set1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_Set_RL_RestoreInd(tvb, offset, actx, tree, hf_nbap_rL_Set1);
}


static const per_sequence_t CCTrCH_InformationList_RL_RestoreInd_sequence_of[1] = {
  { &hf_nbap_CCTrCH_InformationList_RL_RestoreInd_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_CCTrCH_InformationList_RL_RestoreInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_CCTrCH_InformationList_RL_RestoreInd, CCTrCH_InformationList_RL_RestoreInd_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_cCTrCH_InformationList_RL_RestoreInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CCTrCH_InformationList_RL_RestoreInd(tvb, offset, actx, tree, hf_nbap_cCTrCH_InformationList_RL_RestoreInd);
}


static const per_sequence_t CCTrCH_RL_RestoreInd_sequence[] = {
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_ID },
  { &hf_nbap_cCTrCH_InformationList_RL_RestoreInd, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CCTrCH_InformationList_RL_RestoreInd },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CCTrCH_RL_RestoreInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CCTrCH_RL_RestoreInd, CCTrCH_RL_RestoreInd_sequence);

  return offset;
}
static int dissect_cCTrCH1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CCTrCH_RL_RestoreInd(tvb, offset, actx, tree, hf_nbap_cCTrCH1);
}


static const value_string nbap_Reporting_Object_RL_RestoreInd_vals[] = {
  {   0, "rL" },
  {   1, "rL-Set" },
  {   2, "cCTrCH" },
  { 0, NULL }
};

static const per_choice_t Reporting_Object_RL_RestoreInd_choice[] = {
  {   0, &hf_nbap_rL4            , ASN1_EXTENSION_ROOT    , dissect_nbap_RL_RL_RestoreInd },
  {   1, &hf_nbap_rL_Set1        , ASN1_EXTENSION_ROOT    , dissect_nbap_RL_Set_RL_RestoreInd },
  {   2, &hf_nbap_cCTrCH1        , ASN1_NOT_EXTENSION_ROOT, dissect_nbap_CCTrCH_RL_RestoreInd },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_Reporting_Object_RL_RestoreInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_Reporting_Object_RL_RestoreInd, Reporting_Object_RL_RestoreInd_choice,
                                 NULL);

  return offset;
}
static int dissect_id_Reporting_Object_RL_RestoreInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Reporting_Object_RL_RestoreInd(tvb, offset, actx, tree, hf_nbap_id_Reporting_Object_RL_RestoreInd);
}



static int
dissect_nbap_Extension_DedicatedMeasurementValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_extension_DedicatedMeasurementValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Extension_DedicatedMeasurementValue(tvb, offset, actx, tree, hf_nbap_extension_DedicatedMeasurementValue);
}


static const value_string nbap_DedicatedMeasurementValue_vals[] = {
  {   0, "sIR-Value" },
  {   1, "sIR-ErrorValue" },
  {   2, "transmittedCodePowerValue" },
  {   3, "rSCP" },
  {   4, "rxTimingDeviationValue" },
  {   5, "roundTripTime" },
  {   6, "extension-DedicatedMeasurementValue" },
  { 0, NULL }
};

static const per_choice_t DedicatedMeasurementValue_choice[] = {
  {   0, &hf_nbap_sIR_Value      , ASN1_EXTENSION_ROOT    , dissect_nbap_SIR_Value },
  {   1, &hf_nbap_sIR_ErrorValue , ASN1_EXTENSION_ROOT    , dissect_nbap_SIR_Error_Value },
  {   2, &hf_nbap_transmittedCodePowerValue, ASN1_EXTENSION_ROOT    , dissect_nbap_Transmitted_Code_Power_Value },
  {   3, &hf_nbap_rSCP           , ASN1_EXTENSION_ROOT    , dissect_nbap_RSCP_Value },
  {   4, &hf_nbap_rxTimingDeviationValue, ASN1_EXTENSION_ROOT    , dissect_nbap_Rx_Timing_Deviation_Value },
  {   5, &hf_nbap_roundTripTime  , ASN1_EXTENSION_ROOT    , dissect_nbap_Round_Trip_Time_Value },
  {   6, &hf_nbap_extension_DedicatedMeasurementValue, ASN1_NOT_EXTENSION_ROOT, dissect_nbap_Extension_DedicatedMeasurementValue },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_DedicatedMeasurementValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_DedicatedMeasurementValue, DedicatedMeasurementValue_choice,
                                 NULL);

  return offset;
}
static int dissect_dedicatedMeasurementValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DedicatedMeasurementValue(tvb, offset, actx, tree, hf_nbap_dedicatedMeasurementValue);
}
static int dissect_dedicatedmeasurementValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DedicatedMeasurementValue(tvb, offset, actx, tree, hf_nbap_dedicatedmeasurementValue);
}


static const per_sequence_t DedicatedMeasurementAvailable_sequence[] = {
  { &hf_nbap_dedicatedmeasurementValue, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DedicatedMeasurementValue },
  { &hf_nbap_cFN            , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_CFN },
  { &hf_nbap_ie_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DedicatedMeasurementAvailable(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DedicatedMeasurementAvailable, DedicatedMeasurementAvailable_sequence);

  return offset;
}
static int dissect_measurementAvailable1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DedicatedMeasurementAvailable(tvb, offset, actx, tree, hf_nbap_measurementAvailable1);
}



static int
dissect_nbap_DedicatedMeasurementnotAvailable(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_null(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_measurementnotAvailable1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DedicatedMeasurementnotAvailable(tvb, offset, actx, tree, hf_nbap_measurementnotAvailable1);
}


static const value_string nbap_DedicatedMeasurementValueInformation_vals[] = {
  {   0, "measurementAvailable" },
  {   1, "measurementnotAvailable" },
  { 0, NULL }
};

static const per_choice_t DedicatedMeasurementValueInformation_choice[] = {
  {   0, &hf_nbap_measurementAvailable1, ASN1_NO_EXTENSIONS     , dissect_nbap_DedicatedMeasurementAvailable },
  {   1, &hf_nbap_measurementnotAvailable1, ASN1_NO_EXTENSIONS     , dissect_nbap_DedicatedMeasurementnotAvailable },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_DedicatedMeasurementValueInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_DedicatedMeasurementValueInformation, DedicatedMeasurementValueInformation_choice,
                                 NULL);

  return offset;
}
static int dissect_dedicatedMeasurementValueInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DedicatedMeasurementValueInformation(tvb, offset, actx, tree, hf_nbap_dedicatedMeasurementValueInformation);
}


static const per_sequence_t RL_InformationItem_DM_Rprt_sequence[] = {
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_ID },
  { &hf_nbap_dPCH_ID        , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DPCH_ID },
  { &hf_nbap_dedicatedMeasurementValueInformation, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DedicatedMeasurementValueInformation },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RL_InformationItem_DM_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RL_InformationItem_DM_Rprt, RL_InformationItem_DM_Rprt_sequence);

  return offset;
}
static int dissect_id_RL_InformationItem_DM_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_InformationItem_DM_Rprt(tvb, offset, actx, tree, hf_nbap_id_RL_InformationItem_DM_Rprt);
}


static const per_sequence_t RL_InformationItem_DM_Rqst_sequence[] = {
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_ID },
  { &hf_nbap_dPCH_ID        , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DPCH_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RL_InformationItem_DM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RL_InformationItem_DM_Rqst, RL_InformationItem_DM_Rqst_sequence);

  return offset;
}
static int dissect_id_RL_InformationItem_DM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_InformationItem_DM_Rqst(tvb, offset, actx, tree, hf_nbap_id_RL_InformationItem_DM_Rqst);
}


static const per_sequence_t RL_InformationItem_DM_Rsp_sequence[] = {
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_ID },
  { &hf_nbap_dPCH_ID        , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DPCH_ID },
  { &hf_nbap_dedicatedMeasurementValue, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DedicatedMeasurementValue },
  { &hf_nbap_cFN            , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_CFN },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RL_InformationItem_DM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RL_InformationItem_DM_Rsp, RL_InformationItem_DM_Rsp_sequence);

  return offset;
}
static int dissect_id_RL_InformationItem_DM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_InformationItem_DM_Rsp(tvb, offset, actx, tree, hf_nbap_id_RL_InformationItem_DM_Rsp);
}



static int
dissect_nbap_FrameOffset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 255U, NULL, FALSE);

  return offset;
}
static int dissect_frameOffset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_FrameOffset(tvb, offset, actx, tree, hf_nbap_frameOffset);
}



static int
dissect_nbap_ChipOffset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 38399U, NULL, FALSE);

  return offset;
}
static int dissect_chipOffset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ChipOffset(tvb, offset, actx, tree, hf_nbap_chipOffset);
}


static const value_string nbap_DiversityControlField_vals[] = {
  {   0, "may" },
  {   1, "must" },
  {   2, "must-not" },
  { 0, NULL }
};


static int
dissect_nbap_DiversityControlField(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_diversityControlField(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DiversityControlField(tvb, offset, actx, tree, hf_nbap_diversityControlField);
}


static const value_string nbap_TransmissionGapPatternSequenceCodeInformation_vals[] = {
  {   0, "code-change" },
  {   1, "nocode-change" },
  { 0, NULL }
};


static int
dissect_nbap_TransmissionGapPatternSequenceCodeInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_transmissionGapPatternSequenceCodeInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TransmissionGapPatternSequenceCodeInformation(tvb, offset, actx, tree, hf_nbap_transmissionGapPatternSequenceCodeInformation);
}


static const per_sequence_t FDD_DL_CodeInformationItem_sequence[] = {
  { &hf_nbap_dl_ScramblingCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_ScramblingCode },
  { &hf_nbap_fdd_DL_ChannelisationCodeNumber, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_FDD_DL_ChannelisationCodeNumber },
  { &hf_nbap_transmissionGapPatternSequenceCodeInformation, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TransmissionGapPatternSequenceCodeInformation },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_FDD_DL_CodeInformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_FDD_DL_CodeInformationItem, FDD_DL_CodeInformationItem_sequence);

  return offset;
}
static int dissect_FDD_DL_CodeInformation_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_FDD_DL_CodeInformationItem(tvb, offset, actx, tree, hf_nbap_FDD_DL_CodeInformation_item);
}


static const per_sequence_t FDD_DL_CodeInformation_sequence_of[1] = {
  { &hf_nbap_FDD_DL_CodeInformation_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_FDD_DL_CodeInformationItem },
};

static int
dissect_nbap_FDD_DL_CodeInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_FDD_DL_CodeInformation, FDD_DL_CodeInformation_sequence_of,
                                                  1, 10);

  return offset;
}
static int dissect_dl_CodeInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_FDD_DL_CodeInformation(tvb, offset, actx, tree, hf_nbap_dl_CodeInformation);
}


static const per_sequence_t RL_InformationItem_RL_AdditionRqstFDD_sequence[] = {
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_ID },
  { &hf_nbap_c_ID           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_C_ID },
  { &hf_nbap_frameOffset    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_FrameOffset },
  { &hf_nbap_chipOffset     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ChipOffset },
  { &hf_nbap_diversityControlField, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DiversityControlField },
  { &hf_nbap_dl_CodeInformation, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_FDD_DL_CodeInformation },
  { &hf_nbap_initialDL_TransmissionPower, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_Power },
  { &hf_nbap_maximumDL_Power, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_Power },
  { &hf_nbap_minimumDL_Power, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_Power },
  { &hf_nbap_not_Used_sSDT_CellIdentity, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_NULL },
  { &hf_nbap_transmitDiversityIndicator, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TransmitDiversityIndicator },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RL_InformationItem_RL_AdditionRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RL_InformationItem_RL_AdditionRqstFDD, RL_InformationItem_RL_AdditionRqstFDD_sequence);

  return offset;
}
static int dissect_id_RL_InformationItem_RL_AdditionRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_InformationItem_RL_AdditionRqstFDD(tvb, offset, actx, tree, hf_nbap_id_RL_InformationItem_RL_AdditionRqstFDD);
}


static const per_sequence_t RL_informationItem_RL_DeletionRqst_sequence[] = {
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RL_informationItem_RL_DeletionRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RL_informationItem_RL_DeletionRqst, RL_informationItem_RL_DeletionRqst_sequence);

  return offset;
}
static int dissect_id_RL_informationItem_RL_DeletionRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_informationItem_RL_DeletionRqst(tvb, offset, actx, tree, hf_nbap_id_RL_informationItem_RL_DeletionRqst);
}


static const per_sequence_t RL_InformationItem_RL_FailureInd_sequence[] = {
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_ID },
  { &hf_nbap_cause          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Cause },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RL_InformationItem_RL_FailureInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RL_InformationItem_RL_FailureInd, RL_InformationItem_RL_FailureInd_sequence);

  return offset;
}
static int dissect_id_RL_InformationItem_RL_FailureInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_InformationItem_RL_FailureInd(tvb, offset, actx, tree, hf_nbap_id_RL_InformationItem_RL_FailureInd);
}


static const per_sequence_t RL_InformationItem_RL_PreemptRequiredInd_sequence[] = {
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RL_InformationItem_RL_PreemptRequiredInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RL_InformationItem_RL_PreemptRequiredInd, RL_InformationItem_RL_PreemptRequiredInd_sequence);

  return offset;
}
static int dissect_id_RL_InformationItem_RL_PreemptRequiredInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_InformationItem_RL_PreemptRequiredInd(tvb, offset, actx, tree, hf_nbap_id_RL_InformationItem_RL_PreemptRequiredInd);
}


static const per_sequence_t RL_InformationItem_RL_ReconfPrepFDD_sequence[] = {
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_ID },
  { &hf_nbap_dl_CodeInformation, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_FDD_DL_CodeInformation },
  { &hf_nbap_maxDL_Power    , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_Power },
  { &hf_nbap_minDL_Power    , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_Power },
  { &hf_nbap_not_Used_sSDT_Indication, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_NULL },
  { &hf_nbap_not_Used_sSDT_Cell_Identity, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_NULL },
  { &hf_nbap_transmitDiversityIndicator, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TransmitDiversityIndicator },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RL_InformationItem_RL_ReconfPrepFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RL_InformationItem_RL_ReconfPrepFDD, RL_InformationItem_RL_ReconfPrepFDD_sequence);

  return offset;
}
static int dissect_id_RL_InformationItem_RL_ReconfPrepFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_InformationItem_RL_ReconfPrepFDD(tvb, offset, actx, tree, hf_nbap_id_RL_InformationItem_RL_ReconfPrepFDD);
}
static int dissect_id_RL_InformationItem_RL_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_InformationItem_RL_ReconfPrepFDD(tvb, offset, actx, tree, hf_nbap_id_RL_InformationItem_RL_ReconfRqstFDD);
}


static const per_sequence_t RL_InformationItem_RL_RestoreInd_sequence[] = {
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RL_InformationItem_RL_RestoreInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RL_InformationItem_RL_RestoreInd, RL_InformationItem_RL_RestoreInd_sequence);

  return offset;
}
static int dissect_id_RL_InformationItem_RL_RestoreInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_InformationItem_RL_RestoreInd(tvb, offset, actx, tree, hf_nbap_id_RL_InformationItem_RL_RestoreInd);
}


static const value_string nbap_FirstRLS_Indicator_vals[] = {
  {   0, "first-RLS" },
  {   1, "not-first-RLS" },
  { 0, NULL }
};


static int
dissect_nbap_FirstRLS_Indicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_firstRLS_indicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_FirstRLS_Indicator(tvb, offset, actx, tree, hf_nbap_firstRLS_indicator);
}
static int dissect_firstRLS_Indicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_FirstRLS_Indicator(tvb, offset, actx, tree, hf_nbap_firstRLS_Indicator);
}



static int
dissect_nbap_PropagationDelay(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 255U, NULL, FALSE);

  return offset;
}
static int dissect_propagationDelay(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PropagationDelay(tvb, offset, actx, tree, hf_nbap_propagationDelay);
}
static int dissect_propagation_delay(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PropagationDelay(tvb, offset, actx, tree, hf_nbap_propagation_delay);
}


static const per_sequence_t RL_InformationItem_RL_SetupRqstFDD_sequence[] = {
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_ID },
  { &hf_nbap_c_ID           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_C_ID },
  { &hf_nbap_firstRLS_indicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_FirstRLS_Indicator },
  { &hf_nbap_frameOffset    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_FrameOffset },
  { &hf_nbap_chipOffset     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ChipOffset },
  { &hf_nbap_propagationDelay, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_PropagationDelay },
  { &hf_nbap_diversityControlField, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DiversityControlField },
  { &hf_nbap_dl_CodeInformation, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_FDD_DL_CodeInformation },
  { &hf_nbap_initialDL_transmissionPower, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Power },
  { &hf_nbap_maximumDL_power, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Power },
  { &hf_nbap_minimumDL_power, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Power },
  { &hf_nbap_not_Used_sSDT_Cell_Identity, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_NULL },
  { &hf_nbap_transmitDiversityIndicator, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TransmitDiversityIndicator },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RL_InformationItem_RL_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RL_InformationItem_RL_SetupRqstFDD, RL_InformationItem_RL_SetupRqstFDD_sequence);

  return offset;
}
static int dissect_id_RL_InformationItem_RL_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_InformationItem_RL_SetupRqstFDD(tvb, offset, actx, tree, hf_nbap_id_RL_InformationItem_RL_SetupRqstFDD);
}


static const per_sequence_t RL_InformationList_RL_AdditionRqstFDD_sequence_of[1] = {
  { &hf_nbap_RL_InformationList_RL_AdditionRqstFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_RL_InformationList_RL_AdditionRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_RL_InformationList_RL_AdditionRqstFDD, RL_InformationList_RL_AdditionRqstFDD_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_id_RL_InformationList_RL_AdditionRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_InformationList_RL_AdditionRqstFDD(tvb, offset, actx, tree, hf_nbap_id_RL_InformationList_RL_AdditionRqstFDD);
}


static const per_sequence_t RL_informationList_RL_DeletionRqst_sequence_of[1] = {
  { &hf_nbap_RL_informationList_RL_DeletionRqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_RL_informationList_RL_DeletionRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_RL_informationList_RL_DeletionRqst, RL_informationList_RL_DeletionRqst_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_RL_informationList_RL_DeletionRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_informationList_RL_DeletionRqst(tvb, offset, actx, tree, hf_nbap_id_RL_informationList_RL_DeletionRqst);
}


static const per_sequence_t RL_InformationList_RL_PreemptRequiredInd_sequence_of[1] = {
  { &hf_nbap_RL_InformationList_RL_PreemptRequiredInd_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_RL_InformationList_RL_PreemptRequiredInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_RL_InformationList_RL_PreemptRequiredInd, RL_InformationList_RL_PreemptRequiredInd_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_RL_InformationList_RL_PreemptRequiredInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_InformationList_RL_PreemptRequiredInd(tvb, offset, actx, tree, hf_nbap_id_RL_InformationList_RL_PreemptRequiredInd);
}


static const per_sequence_t RL_InformationList_RL_ReconfPrepFDD_sequence_of[1] = {
  { &hf_nbap_RL_InformationList_RL_ReconfPrepFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_RL_InformationList_RL_ReconfPrepFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_RL_InformationList_RL_ReconfPrepFDD, RL_InformationList_RL_ReconfPrepFDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_RL_InformationList_RL_ReconfPrepFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_InformationList_RL_ReconfPrepFDD(tvb, offset, actx, tree, hf_nbap_id_RL_InformationList_RL_ReconfPrepFDD);
}


static const per_sequence_t RL_InformationList_RL_ReconfRqstFDD_sequence_of[1] = {
  { &hf_nbap_RL_InformationList_RL_ReconfRqstFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_RL_InformationList_RL_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_RL_InformationList_RL_ReconfRqstFDD, RL_InformationList_RL_ReconfRqstFDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_RL_InformationList_RL_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_InformationList_RL_ReconfRqstFDD(tvb, offset, actx, tree, hf_nbap_id_RL_InformationList_RL_ReconfRqstFDD);
}


static const per_sequence_t RL_InformationList_RL_SetupRqstFDD_sequence_of[1] = {
  { &hf_nbap_RL_InformationList_RL_SetupRqstFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_RL_InformationList_RL_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_RL_InformationList_RL_SetupRqstFDD, RL_InformationList_RL_SetupRqstFDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_RL_InformationList_RL_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_InformationList_RL_SetupRqstFDD(tvb, offset, actx, tree, hf_nbap_id_RL_InformationList_RL_SetupRqstFDD);
}


static const per_sequence_t Combining_RL_AdditionRspFDD_sequence[] = {
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Combining_RL_AdditionRspFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Combining_RL_AdditionRspFDD, Combining_RL_AdditionRspFDD_sequence);

  return offset;
}
static int dissect_combining2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Combining_RL_AdditionRspFDD(tvb, offset, actx, tree, hf_nbap_combining2);
}


static const per_sequence_t Non_Combining_RL_AdditionRspFDD_sequence[] = {
  { &hf_nbap_dCH_InformationResponse, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DCH_InformationResponse },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Non_Combining_RL_AdditionRspFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Non_Combining_RL_AdditionRspFDD, Non_Combining_RL_AdditionRspFDD_sequence);

  return offset;
}
static int dissect_non_combining(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Non_Combining_RL_AdditionRspFDD(tvb, offset, actx, tree, hf_nbap_non_combining);
}


static const value_string nbap_DiversityIndication_RL_AdditionRspFDD_vals[] = {
  {   0, "combining" },
  {   1, "non-combining" },
  { 0, NULL }
};

static const per_choice_t DiversityIndication_RL_AdditionRspFDD_choice[] = {
  {   0, &hf_nbap_combining2     , ASN1_NO_EXTENSIONS     , dissect_nbap_Combining_RL_AdditionRspFDD },
  {   1, &hf_nbap_non_combining  , ASN1_NO_EXTENSIONS     , dissect_nbap_Non_Combining_RL_AdditionRspFDD },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_DiversityIndication_RL_AdditionRspFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_DiversityIndication_RL_AdditionRspFDD, DiversityIndication_RL_AdditionRspFDD_choice,
                                 NULL);

  return offset;
}
static int dissect_diversityIndication2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DiversityIndication_RL_AdditionRspFDD(tvb, offset, actx, tree, hf_nbap_diversityIndication2);
}


static const value_string nbap_SSDT_SupportIndicator_vals[] = {
  {   0, "not-Used-sSDT-Supported" },
  {   1, "sSDT-not-supported" },
  { 0, NULL }
};


static int
dissect_nbap_SSDT_SupportIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_sSDT_SupportIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SSDT_SupportIndicator(tvb, offset, actx, tree, hf_nbap_sSDT_SupportIndicator);
}


static const per_sequence_t RL_InformationResponseItem_RL_AdditionRspFDD_sequence[] = {
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_ID },
  { &hf_nbap_rL_Set_ID      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_Set_ID },
  { &hf_nbap_received_total_wide_band_power, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Received_total_wide_band_power_Value },
  { &hf_nbap_diversityIndication2, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DiversityIndication_RL_AdditionRspFDD },
  { &hf_nbap_sSDT_SupportIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SSDT_SupportIndicator },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RL_InformationResponseItem_RL_AdditionRspFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RL_InformationResponseItem_RL_AdditionRspFDD, RL_InformationResponseItem_RL_AdditionRspFDD_sequence);

  return offset;
}
static int dissect_id_RL_InformationResponseItem_RL_AdditionRspFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_InformationResponseItem_RL_AdditionRspFDD(tvb, offset, actx, tree, hf_nbap_id_RL_InformationResponseItem_RL_AdditionRspFDD);
}



static int
dissect_nbap_DCH_InformationResponseList_RL_ReconfReady(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_dCH_InformationResponseList_RL_ReconfReady(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DCH_InformationResponseList_RL_ReconfReady(tvb, offset, actx, tree, hf_nbap_dCH_InformationResponseList_RL_ReconfReady);
}



static int
dissect_nbap_DSCH_InformationResponseList_RL_ReconfReady(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_dSCH_InformationResponseList_RL_ReconfReady(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DSCH_InformationResponseList_RL_ReconfReady(tvb, offset, actx, tree, hf_nbap_dSCH_InformationResponseList_RL_ReconfReady);
}



static int
dissect_nbap_USCH_InformationResponseList_RL_ReconfReady(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_uSCH_InformationResponseList_RL_ReconfReady(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_USCH_InformationResponseList_RL_ReconfReady(tvb, offset, actx, tree, hf_nbap_uSCH_InformationResponseList_RL_ReconfReady);
}


static const per_sequence_t RL_InformationResponseItem_RL_ReconfReady_sequence[] = {
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_ID },
  { &hf_nbap_dCH_InformationResponseList_RL_ReconfReady, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DCH_InformationResponseList_RL_ReconfReady },
  { &hf_nbap_dSCH_InformationResponseList_RL_ReconfReady, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DSCH_InformationResponseList_RL_ReconfReady },
  { &hf_nbap_uSCH_InformationResponseList_RL_ReconfReady, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_USCH_InformationResponseList_RL_ReconfReady },
  { &hf_nbap_not_Used_tFCI2_BearerInformationResponse, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_NULL },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RL_InformationResponseItem_RL_ReconfReady(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RL_InformationResponseItem_RL_ReconfReady, RL_InformationResponseItem_RL_ReconfReady_sequence);

  return offset;
}
static int dissect_id_RL_InformationResponseItem_RL_ReconfReady(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_InformationResponseItem_RL_ReconfReady(tvb, offset, actx, tree, hf_nbap_id_RL_InformationResponseItem_RL_ReconfReady);
}



static int
dissect_nbap_DCH_InformationResponseList_RL_ReconfRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_dCH_InformationResponseList_RL_ReconfRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DCH_InformationResponseList_RL_ReconfRsp(tvb, offset, actx, tree, hf_nbap_dCH_InformationResponseList_RL_ReconfRsp);
}


static const per_sequence_t RL_InformationResponseItem_RL_ReconfRsp_sequence[] = {
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_ID },
  { &hf_nbap_dCH_InformationResponseList_RL_ReconfRsp, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DCH_InformationResponseList_RL_ReconfRsp },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RL_InformationResponseItem_RL_ReconfRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RL_InformationResponseItem_RL_ReconfRsp, RL_InformationResponseItem_RL_ReconfRsp_sequence);

  return offset;
}
static int dissect_id_RL_InformationResponseItem_RL_ReconfRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_InformationResponseItem_RL_ReconfRsp(tvb, offset, actx, tree, hf_nbap_id_RL_InformationResponseItem_RL_ReconfRsp);
}


static const per_sequence_t Combining_RL_SetupRspFDD_sequence[] = {
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Combining_RL_SetupRspFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Combining_RL_SetupRspFDD, Combining_RL_SetupRspFDD_sequence);

  return offset;
}
static int dissect_combining(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Combining_RL_SetupRspFDD(tvb, offset, actx, tree, hf_nbap_combining);
}


static const per_sequence_t NonCombiningOrFirstRL_RL_SetupRspFDD_sequence[] = {
  { &hf_nbap_dCH_InformationResponse, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DCH_InformationResponse },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_NonCombiningOrFirstRL_RL_SetupRspFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_NonCombiningOrFirstRL_RL_SetupRspFDD, NonCombiningOrFirstRL_RL_SetupRspFDD_sequence);

  return offset;
}
static int dissect_nonCombiningOrFirstRL(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NonCombiningOrFirstRL_RL_SetupRspFDD(tvb, offset, actx, tree, hf_nbap_nonCombiningOrFirstRL);
}


static const value_string nbap_DiversityIndication_RL_SetupRspFDD_vals[] = {
  {   0, "combining" },
  {   1, "nonCombiningOrFirstRL" },
  { 0, NULL }
};

static const per_choice_t DiversityIndication_RL_SetupRspFDD_choice[] = {
  {   0, &hf_nbap_combining      , ASN1_NO_EXTENSIONS     , dissect_nbap_Combining_RL_SetupRspFDD },
  {   1, &hf_nbap_nonCombiningOrFirstRL, ASN1_NO_EXTENSIONS     , dissect_nbap_NonCombiningOrFirstRL_RL_SetupRspFDD },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_DiversityIndication_RL_SetupRspFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_DiversityIndication_RL_SetupRspFDD, DiversityIndication_RL_SetupRspFDD_choice,
                                 NULL);

  return offset;
}
static int dissect_diversityIndication(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DiversityIndication_RL_SetupRspFDD(tvb, offset, actx, tree, hf_nbap_diversityIndication);
}


static const per_sequence_t RL_InformationResponseItem_RL_SetupRspFDD_sequence[] = {
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_ID },
  { &hf_nbap_rL_Set_ID      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_Set_ID },
  { &hf_nbap_received_total_wide_band_power, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Received_total_wide_band_power_Value },
  { &hf_nbap_diversityIndication, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DiversityIndication_RL_SetupRspFDD },
  { &hf_nbap_not_Used_dSCH_InformationResponseList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_NULL },
  { &hf_nbap_sSDT_SupportIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SSDT_SupportIndicator },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RL_InformationResponseItem_RL_SetupRspFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RL_InformationResponseItem_RL_SetupRspFDD, RL_InformationResponseItem_RL_SetupRspFDD_sequence);

  return offset;
}
static int dissect_id_RL_InformationResponseItem_RL_SetupRspFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_InformationResponseItem_RL_SetupRspFDD(tvb, offset, actx, tree, hf_nbap_id_RL_InformationResponseItem_RL_SetupRspFDD);
}


static const per_sequence_t RL_InformationResponseList_RL_AdditionRspFDD_sequence_of[1] = {
  { &hf_nbap_RL_InformationResponseList_RL_AdditionRspFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_RL_InformationResponseList_RL_AdditionRspFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_RL_InformationResponseList_RL_AdditionRspFDD, RL_InformationResponseList_RL_AdditionRspFDD_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_id_RL_InformationResponseList_RL_AdditionRspFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_InformationResponseList_RL_AdditionRspFDD(tvb, offset, actx, tree, hf_nbap_id_RL_InformationResponseList_RL_AdditionRspFDD);
}


static const per_sequence_t RL_InformationResponseList_RL_ReconfReady_sequence_of[1] = {
  { &hf_nbap_RL_InformationResponseList_RL_ReconfReady_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_RL_InformationResponseList_RL_ReconfReady(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_RL_InformationResponseList_RL_ReconfReady, RL_InformationResponseList_RL_ReconfReady_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_id_RL_InformationResponseList_RL_ReconfReady(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_InformationResponseList_RL_ReconfReady(tvb, offset, actx, tree, hf_nbap_id_RL_InformationResponseList_RL_ReconfReady);
}


static const per_sequence_t RL_InformationResponseList_RL_ReconfRsp_sequence_of[1] = {
  { &hf_nbap_RL_InformationResponseList_RL_ReconfRsp_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_RL_InformationResponseList_RL_ReconfRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_RL_InformationResponseList_RL_ReconfRsp, RL_InformationResponseList_RL_ReconfRsp_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_RL_InformationResponseList_RL_ReconfRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_InformationResponseList_RL_ReconfRsp(tvb, offset, actx, tree, hf_nbap_id_RL_InformationResponseList_RL_ReconfRsp);
}


static const per_sequence_t RL_InformationResponseList_RL_SetupRspFDD_sequence_of[1] = {
  { &hf_nbap_RL_InformationResponseList_RL_SetupRspFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_RL_InformationResponseList_RL_SetupRspFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_RL_InformationResponseList_RL_SetupRspFDD, RL_InformationResponseList_RL_SetupRspFDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_RL_InformationResponseList_RL_SetupRspFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_InformationResponseList_RL_SetupRspFDD(tvb, offset, actx, tree, hf_nbap_id_RL_InformationResponseList_RL_SetupRspFDD);
}


static const per_sequence_t UL_TimeSlot_ISCP_InfoItem_sequence[] = {
  { &hf_nbap_timeSlot       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlot },
  { &hf_nbap_iSCP           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UL_TimeslotISCP_Value },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_UL_TimeSlot_ISCP_InfoItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_UL_TimeSlot_ISCP_InfoItem, UL_TimeSlot_ISCP_InfoItem_sequence);

  return offset;
}
static int dissect_UL_TimeSlot_ISCP_Info_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_TimeSlot_ISCP_InfoItem(tvb, offset, actx, tree, hf_nbap_UL_TimeSlot_ISCP_Info_item);
}


static const per_sequence_t UL_TimeSlot_ISCP_Info_sequence_of[1] = {
  { &hf_nbap_UL_TimeSlot_ISCP_Info_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_UL_TimeSlot_ISCP_InfoItem },
};

static int
dissect_nbap_UL_TimeSlot_ISCP_Info(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_UL_TimeSlot_ISCP_Info, UL_TimeSlot_ISCP_Info_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_uL_TimeSlot_ISCP_Info(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_TimeSlot_ISCP_Info(tvb, offset, actx, tree, hf_nbap_uL_TimeSlot_ISCP_Info);
}


static const value_string nbap_UL_PhysCH_SF_Variation_vals[] = {
  {   0, "sf-variation-supported" },
  {   1, "sf-variation-not-supported" },
  { 0, NULL }
};


static int
dissect_nbap_UL_PhysCH_SF_Variation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_ul_PhysCH_SF_Variation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_PhysCH_SF_Variation(tvb, offset, actx, tree, hf_nbap_ul_PhysCH_SF_Variation);
}


static const per_sequence_t Combining_RL_AdditionRspTDD_sequence[] = {
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Combining_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Combining_RL_AdditionRspTDD, Combining_RL_AdditionRspTDD_sequence);

  return offset;
}
static int dissect_combining3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Combining_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_nbap_combining3);
}


static const per_sequence_t Non_Combining_RL_AdditionRspTDD_sequence[] = {
  { &hf_nbap_dCH_InformationResponse, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DCH_InformationResponse },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Non_Combining_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Non_Combining_RL_AdditionRspTDD, Non_Combining_RL_AdditionRspTDD_sequence);

  return offset;
}
static int dissect_non_Combining(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Non_Combining_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_nbap_non_Combining);
}


static const value_string nbap_DiversityIndication_RL_AdditionRspTDD_vals[] = {
  {   0, "combining" },
  {   1, "non-Combining" },
  { 0, NULL }
};

static const per_choice_t DiversityIndication_RL_AdditionRspTDD_choice[] = {
  {   0, &hf_nbap_combining3     , ASN1_NO_EXTENSIONS     , dissect_nbap_Combining_RL_AdditionRspTDD },
  {   1, &hf_nbap_non_Combining  , ASN1_NO_EXTENSIONS     , dissect_nbap_Non_Combining_RL_AdditionRspTDD },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_DiversityIndication_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_DiversityIndication_RL_AdditionRspTDD, DiversityIndication_RL_AdditionRspTDD_choice,
                                 NULL);

  return offset;
}
static int dissect_diversityIndication3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DiversityIndication_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_nbap_diversityIndication3);
}


static const per_sequence_t DCH_Information_RL_AdditionRspTDD_sequence[] = {
  { &hf_nbap_diversityIndication3, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DiversityIndication_RL_AdditionRspTDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DCH_Information_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DCH_Information_RL_AdditionRspTDD, DCH_Information_RL_AdditionRspTDD_sequence);

  return offset;
}
static int dissect_dCH_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DCH_Information_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_nbap_dCH_Information);
}



static int
dissect_nbap_DSCH_InformationResponseList_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_dSCH_InformationResponseList1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DSCH_InformationResponseList_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_nbap_dSCH_InformationResponseList1);
}



static int
dissect_nbap_USCH_InformationResponseList_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_uSCH_InformationResponseList1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_USCH_InformationResponseList_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_nbap_uSCH_InformationResponseList1);
}


static const per_sequence_t RL_InformationResponse_RL_AdditionRspTDD_sequence[] = {
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_ID },
  { &hf_nbap_uL_TimeSlot_ISCP_Info, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UL_TimeSlot_ISCP_Info },
  { &hf_nbap_ul_PhysCH_SF_Variation, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UL_PhysCH_SF_Variation },
  { &hf_nbap_dCH_Information, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DCH_Information_RL_AdditionRspTDD },
  { &hf_nbap_dSCH_InformationResponseList1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DSCH_InformationResponseList_RL_AdditionRspTDD },
  { &hf_nbap_uSCH_InformationResponseList1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_USCH_InformationResponseList_RL_AdditionRspTDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RL_InformationResponse_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RL_InformationResponse_RL_AdditionRspTDD, RL_InformationResponse_RL_AdditionRspTDD_sequence);

  return offset;
}
static int dissect_id_RL_InformationResponse_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_InformationResponse_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_nbap_id_RL_InformationResponse_RL_AdditionRspTDD);
}



static int
dissect_nbap_DCH_InformationResponseList_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_dCH_InformationResponseList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DCH_InformationResponseList_RL_SetupRspTDD(tvb, offset, actx, tree, hf_nbap_dCH_InformationResponseList);
}



static int
dissect_nbap_DSCH_InformationResponseList_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_dSCH_InformationResponseList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DSCH_InformationResponseList_RL_SetupRspTDD(tvb, offset, actx, tree, hf_nbap_dSCH_InformationResponseList);
}



static int
dissect_nbap_USCH_InformationResponseList_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_uSCH_InformationResponseList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_USCH_InformationResponseList_RL_SetupRspTDD(tvb, offset, actx, tree, hf_nbap_uSCH_InformationResponseList);
}


static const per_sequence_t RL_InformationResponse_RL_SetupRspTDD_sequence[] = {
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_ID },
  { &hf_nbap_uL_TimeSlot_ISCP_Info, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UL_TimeSlot_ISCP_Info },
  { &hf_nbap_ul_PhysCH_SF_Variation, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UL_PhysCH_SF_Variation },
  { &hf_nbap_dCH_InformationResponseList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DCH_InformationResponseList_RL_SetupRspTDD },
  { &hf_nbap_dSCH_InformationResponseList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DSCH_InformationResponseList_RL_SetupRspTDD },
  { &hf_nbap_uSCH_InformationResponseList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_USCH_InformationResponseList_RL_SetupRspTDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RL_InformationResponse_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RL_InformationResponse_RL_SetupRspTDD, RL_InformationResponse_RL_SetupRspTDD_sequence);

  return offset;
}
static int dissect_id_RL_InformationResponse_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_InformationResponse_RL_SetupRspTDD(tvb, offset, actx, tree, hf_nbap_id_RL_InformationResponse_RL_SetupRspTDD);
}



static int
dissect_nbap_DL_TimeslotISCP(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 91U, NULL, FALSE);

  return offset;
}
static int dissect_dL_TimeslotISCP(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_TimeslotISCP(tvb, offset, actx, tree, hf_nbap_dL_TimeslotISCP);
}


static const per_sequence_t DL_TimeslotISCPInfoItem_sequence[] = {
  { &hf_nbap_timeSlot       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlot },
  { &hf_nbap_dL_TimeslotISCP, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_TimeslotISCP },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_TimeslotISCPInfoItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_TimeslotISCPInfoItem, DL_TimeslotISCPInfoItem_sequence);

  return offset;
}
static int dissect_DL_TimeslotISCPInfo_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_TimeslotISCPInfoItem(tvb, offset, actx, tree, hf_nbap_DL_TimeslotISCPInfo_item);
}


static const per_sequence_t DL_TimeslotISCPInfo_sequence_of[1] = {
  { &hf_nbap_DL_TimeslotISCPInfo_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DL_TimeslotISCPInfoItem },
};

static int
dissect_nbap_DL_TimeslotISCPInfo(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DL_TimeslotISCPInfo, DL_TimeslotISCPInfo_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_id_TimeslotISCPInfo(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_TimeslotISCPInfo(tvb, offset, actx, tree, hf_nbap_id_TimeslotISCPInfo);
}
static int dissect_dL_TimeSlotISCPInfo(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_TimeslotISCPInfo(tvb, offset, actx, tree, hf_nbap_dL_TimeSlotISCPInfo);
}


static const per_sequence_t RL_Information_RL_AdditionRqstTDD_sequence[] = {
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_ID },
  { &hf_nbap_c_ID           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_C_ID },
  { &hf_nbap_frameOffset    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_FrameOffset },
  { &hf_nbap_diversityControlField, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DiversityControlField },
  { &hf_nbap_initial_DL_Transmission_Power, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_Power },
  { &hf_nbap_maximumDL_Power, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_Power },
  { &hf_nbap_minimumDL_Power, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_Power },
  { &hf_nbap_dL_TimeSlotISCPInfo, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_TimeslotISCPInfo },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RL_Information_RL_AdditionRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RL_Information_RL_AdditionRqstTDD, RL_Information_RL_AdditionRqstTDD_sequence);

  return offset;
}
static int dissect_id_RL_Information_RL_AdditionRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_Information_RL_AdditionRqstTDD(tvb, offset, actx, tree, hf_nbap_id_RL_Information_RL_AdditionRqstTDD);
}


static const per_sequence_t RL_Information_RL_ReconfRqstTDD_sequence[] = {
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_ID },
  { &hf_nbap_maxDL_Power    , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_Power },
  { &hf_nbap_minDL_Power    , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_Power },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RL_Information_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RL_Information_RL_ReconfRqstTDD, RL_Information_RL_ReconfRqstTDD_sequence);

  return offset;
}
static int dissect_id_RL_Information_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_Information_RL_ReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_id_RL_Information_RL_ReconfRqstTDD);
}
static int dissect_Multiple_RL_Information_RL_ReconfRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_Information_RL_ReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_Multiple_RL_Information_RL_ReconfRqstTDD_item);
}


static const per_sequence_t RL_Information_RL_ReconfPrepTDD_sequence[] = {
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_ID },
  { &hf_nbap_maxDL_Power    , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_Power },
  { &hf_nbap_minDL_Power    , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_Power },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RL_Information_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RL_Information_RL_ReconfPrepTDD, RL_Information_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_id_RL_Information_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_Information_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_id_RL_Information_RL_ReconfPrepTDD);
}
static int dissect_MultipleRL_Information_RL_ReconfPrepTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_Information_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_MultipleRL_Information_RL_ReconfPrepTDD_item);
}



static int
dissect_nbap_SpecialBurstScheduling(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 256U, NULL, FALSE);

  return offset;
}
static int dissect_specialBurstScheduling(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SpecialBurstScheduling(tvb, offset, actx, tree, hf_nbap_specialBurstScheduling);
}


static const per_sequence_t RL_Information_RL_SetupRqstTDD_sequence[] = {
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_ID },
  { &hf_nbap_c_ID           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_C_ID },
  { &hf_nbap_frameOffset    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_FrameOffset },
  { &hf_nbap_specialBurstScheduling, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SpecialBurstScheduling },
  { &hf_nbap_initialDL_transmissionPower, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Power },
  { &hf_nbap_maximumDL_power, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Power },
  { &hf_nbap_minimumDL_power, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Power },
  { &hf_nbap_dL_TimeSlotISCPInfo, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_TimeslotISCPInfo },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RL_Information_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RL_Information_RL_SetupRqstTDD, RL_Information_RL_SetupRqstTDD_sequence);

  return offset;
}
static int dissect_id_RL_Information_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_Information_RL_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_id_RL_Information_RL_SetupRqstTDD);
}


static const per_sequence_t RL_ReconfigurationFailureItem_RL_ReconfFailure_sequence[] = {
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_ID },
  { &hf_nbap_cause          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Cause },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RL_ReconfigurationFailureItem_RL_ReconfFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RL_ReconfigurationFailureItem_RL_ReconfFailure, RL_ReconfigurationFailureItem_RL_ReconfFailure_sequence);

  return offset;
}
static int dissect_id_RL_ReconfigurationFailureItem_RL_ReconfFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_ReconfigurationFailureItem_RL_ReconfFailure(tvb, offset, actx, tree, hf_nbap_id_RL_ReconfigurationFailureItem_RL_ReconfFailure);
}


static const per_sequence_t RL_Set_InformationItem_DM_Rprt_sequence[] = {
  { &hf_nbap_rL_Set_ID      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_Set_ID },
  { &hf_nbap_dedicatedMeasurementValueInformation, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DedicatedMeasurementValueInformation },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RL_Set_InformationItem_DM_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RL_Set_InformationItem_DM_Rprt, RL_Set_InformationItem_DM_Rprt_sequence);

  return offset;
}
static int dissect_id_RL_Set_InformationItem_DM_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_Set_InformationItem_DM_Rprt(tvb, offset, actx, tree, hf_nbap_id_RL_Set_InformationItem_DM_Rprt);
}


static const per_sequence_t RL_Set_InformationItem_DM_Rsp_sequence[] = {
  { &hf_nbap_rL_Set_ID      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_Set_ID },
  { &hf_nbap_dedicatedMeasurementValue, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DedicatedMeasurementValue },
  { &hf_nbap_cFN            , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_CFN },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RL_Set_InformationItem_DM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RL_Set_InformationItem_DM_Rsp, RL_Set_InformationItem_DM_Rsp_sequence);

  return offset;
}
static int dissect_id_RL_Set_InformationItem_DM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_Set_InformationItem_DM_Rsp(tvb, offset, actx, tree, hf_nbap_id_RL_Set_InformationItem_DM_Rsp);
}


static const per_sequence_t RL_Set_InformationItem_RL_FailureInd_sequence[] = {
  { &hf_nbap_rL_Set_ID      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_Set_ID },
  { &hf_nbap_cause          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Cause },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RL_Set_InformationItem_RL_FailureInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RL_Set_InformationItem_RL_FailureInd, RL_Set_InformationItem_RL_FailureInd_sequence);

  return offset;
}
static int dissect_id_RL_Set_InformationItem_RL_FailureInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_Set_InformationItem_RL_FailureInd(tvb, offset, actx, tree, hf_nbap_id_RL_Set_InformationItem_RL_FailureInd);
}


static const per_sequence_t RL_Set_InformationItem_RL_RestoreInd_sequence[] = {
  { &hf_nbap_rL_Set_ID      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_Set_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RL_Set_InformationItem_RL_RestoreInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RL_Set_InformationItem_RL_RestoreInd, RL_Set_InformationItem_RL_RestoreInd_sequence);

  return offset;
}
static int dissect_id_RL_Set_InformationItem_RL_RestoreInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_Set_InformationItem_RL_RestoreInd(tvb, offset, actx, tree, hf_nbap_id_RL_Set_InformationItem_RL_RestoreInd);
}


static const per_sequence_t Secondary_CCPCHItem_CTCH_ReconfRqstTDD_sequence[] = {
  { &hf_nbap_commonPhysicalChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonPhysicalChannelID },
  { &hf_nbap_sCCPCH_Power   , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_Power },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Secondary_CCPCHItem_CTCH_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Secondary_CCPCHItem_CTCH_ReconfRqstTDD, Secondary_CCPCHItem_CTCH_ReconfRqstTDD_sequence);

  return offset;
}
static int dissect_Secondary_CCPCHListIE_CTCH_ReconfRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Secondary_CCPCHItem_CTCH_ReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_Secondary_CCPCHListIE_CTCH_ReconfRqstTDD_item);
}
static int dissect_Secondary_CCPCH_parameterExtendedList_CTCH_ReconfRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Secondary_CCPCHItem_CTCH_ReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_Secondary_CCPCH_parameterExtendedList_CTCH_ReconfRqstTDD_item);
}
static int dissect_Secondary_CCPCH_LCR_parameterExtendedList_CTCH_ReconfRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Secondary_CCPCHItem_CTCH_ReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_Secondary_CCPCH_LCR_parameterExtendedList_CTCH_ReconfRqstTDD_item);
}


static const per_sequence_t Secondary_CCPCHListIE_CTCH_ReconfRqstTDD_sequence_of[1] = {
  { &hf_nbap_Secondary_CCPCHListIE_CTCH_ReconfRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_Secondary_CCPCHItem_CTCH_ReconfRqstTDD },
};

static int
dissect_nbap_Secondary_CCPCHListIE_CTCH_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_Secondary_CCPCHListIE_CTCH_ReconfRqstTDD, Secondary_CCPCHListIE_CTCH_ReconfRqstTDD_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_id_Secondary_CCPCHListIE_CTCH_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Secondary_CCPCHListIE_CTCH_ReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_id_Secondary_CCPCHListIE_CTCH_ReconfRqstTDD);
}


static const per_sequence_t Secondary_CCPCH_parameterItem_CTCH_SetupRqstTDD_sequence[] = {
  { &hf_nbap_commonPhysicalChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonPhysicalChannelID },
  { &hf_nbap_tdd_ChannelisationCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_ChannelisationCode },
  { &hf_nbap_timeslot       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlot },
  { &hf_nbap_midambleShiftandBurstType, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MidambleShiftAndBurstType },
  { &hf_nbap_tdd_PhysicalChannelOffset, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_PhysicalChannelOffset },
  { &hf_nbap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionPeriod },
  { &hf_nbap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionLength },
  { &hf_nbap_s_CCPCH_Power  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Power },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Secondary_CCPCH_parameterItem_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Secondary_CCPCH_parameterItem_CTCH_SetupRqstTDD, Secondary_CCPCH_parameterItem_CTCH_SetupRqstTDD_sequence);

  return offset;
}
static int dissect_Secondary_CCPCH_parameterListIE_CTCH_SetupRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Secondary_CCPCH_parameterItem_CTCH_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_Secondary_CCPCH_parameterListIE_CTCH_SetupRqstTDD_item);
}
static int dissect_Secondary_CCPCH_parameterExtendedList_CTCH_SetupRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Secondary_CCPCH_parameterItem_CTCH_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_Secondary_CCPCH_parameterExtendedList_CTCH_SetupRqstTDD_item);
}


static const per_sequence_t Secondary_CCPCH_parameterListIE_CTCH_SetupRqstTDD_sequence_of[1] = {
  { &hf_nbap_Secondary_CCPCH_parameterListIE_CTCH_SetupRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_Secondary_CCPCH_parameterItem_CTCH_SetupRqstTDD },
};

static int
dissect_nbap_Secondary_CCPCH_parameterListIE_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_Secondary_CCPCH_parameterListIE_CTCH_SetupRqstTDD, Secondary_CCPCH_parameterListIE_CTCH_SetupRqstTDD_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_id_Secondary_CCPCH_parameterListIE_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Secondary_CCPCH_parameterListIE_CTCH_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_id_Secondary_CCPCH_parameterListIE_CTCH_SetupRqstTDD);
}



static int
dissect_nbap_Secondary_CCPCHList_CTCH_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_secondaryCCPCHList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Secondary_CCPCHList_CTCH_ReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_secondaryCCPCHList);
}


static const per_sequence_t Secondary_CCPCH_Parameters_CTCH_ReconfRqstTDD_sequence[] = {
  { &hf_nbap_cCTrCH_ID      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CCTrCH_ID },
  { &hf_nbap_secondaryCCPCHList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_Secondary_CCPCHList_CTCH_ReconfRqstTDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Secondary_CCPCH_Parameters_CTCH_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Secondary_CCPCH_Parameters_CTCH_ReconfRqstTDD, Secondary_CCPCH_Parameters_CTCH_ReconfRqstTDD_sequence);

  return offset;
}
static int dissect_id_Secondary_CCPCH_Parameters_CTCH_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Secondary_CCPCH_Parameters_CTCH_ReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_id_Secondary_CCPCH_Parameters_CTCH_ReconfRqstTDD);
}


static const per_sequence_t SecondaryCPICH_InformationItem_Cell_ReconfRqstFDD_sequence[] = {
  { &hf_nbap_commonPhysicalChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonPhysicalChannelID },
  { &hf_nbap_secondaryCPICH_Power, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Power },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_SecondaryCPICH_InformationItem_Cell_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_SecondaryCPICH_InformationItem_Cell_ReconfRqstFDD, SecondaryCPICH_InformationItem_Cell_ReconfRqstFDD_sequence);

  return offset;
}
static int dissect_id_SecondaryCPICH_InformationItem_Cell_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SecondaryCPICH_InformationItem_Cell_ReconfRqstFDD(tvb, offset, actx, tree, hf_nbap_id_SecondaryCPICH_InformationItem_Cell_ReconfRqstFDD);
}


static const per_sequence_t SecondaryCPICH_InformationItem_Cell_SetupRqstFDD_sequence[] = {
  { &hf_nbap_commonPhysicalChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonPhysicalChannelID },
  { &hf_nbap_dl_ScramblingCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_ScramblingCode },
  { &hf_nbap_fDD_DL_ChannelisationCodeNumber, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_FDD_DL_ChannelisationCodeNumber },
  { &hf_nbap_secondaryCPICH_Power, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Power },
  { &hf_nbap_transmitDiversityIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TransmitDiversityIndicator },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_SecondaryCPICH_InformationItem_Cell_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_SecondaryCPICH_InformationItem_Cell_SetupRqstFDD, SecondaryCPICH_InformationItem_Cell_SetupRqstFDD_sequence);

  return offset;
}
static int dissect_id_SecondaryCPICH_InformationItem_Cell_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SecondaryCPICH_InformationItem_Cell_SetupRqstFDD(tvb, offset, actx, tree, hf_nbap_id_SecondaryCPICH_InformationItem_Cell_SetupRqstFDD);
}


static const per_sequence_t SecondaryCPICH_InformationList_Cell_ReconfRqstFDD_sequence_of[1] = {
  { &hf_nbap_SecondaryCPICH_InformationList_Cell_ReconfRqstFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_SecondaryCPICH_InformationList_Cell_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_SecondaryCPICH_InformationList_Cell_ReconfRqstFDD, SecondaryCPICH_InformationList_Cell_ReconfRqstFDD_sequence_of,
                                                  1, 32);

  return offset;
}
static int dissect_id_SecondaryCPICH_InformationList_Cell_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SecondaryCPICH_InformationList_Cell_ReconfRqstFDD(tvb, offset, actx, tree, hf_nbap_id_SecondaryCPICH_InformationList_Cell_ReconfRqstFDD);
}


static const per_sequence_t SecondaryCPICH_InformationList_Cell_SetupRqstFDD_sequence_of[1] = {
  { &hf_nbap_SecondaryCPICH_InformationList_Cell_SetupRqstFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_SecondaryCPICH_InformationList_Cell_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_SecondaryCPICH_InformationList_Cell_SetupRqstFDD, SecondaryCPICH_InformationList_Cell_SetupRqstFDD_sequence_of,
                                                  1, 32);

  return offset;
}
static int dissect_id_SecondaryCPICH_InformationList_Cell_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SecondaryCPICH_InformationList_Cell_SetupRqstFDD(tvb, offset, actx, tree, hf_nbap_id_SecondaryCPICH_InformationList_Cell_SetupRqstFDD);
}


static const per_sequence_t SecondarySCH_Information_Cell_ReconfRqstFDD_sequence[] = {
  { &hf_nbap_commonPhysicalChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonPhysicalChannelID },
  { &hf_nbap_secondarySCH_Power, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Power },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_SecondarySCH_Information_Cell_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_SecondarySCH_Information_Cell_ReconfRqstFDD, SecondarySCH_Information_Cell_ReconfRqstFDD_sequence);

  return offset;
}
static int dissect_id_SecondarySCH_Information_Cell_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SecondarySCH_Information_Cell_ReconfRqstFDD(tvb, offset, actx, tree, hf_nbap_id_SecondarySCH_Information_Cell_ReconfRqstFDD);
}


static const per_sequence_t SecondarySCH_Information_Cell_SetupRqstFDD_sequence[] = {
  { &hf_nbap_commonPhysicalChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonPhysicalChannelID },
  { &hf_nbap_secondarySCH_Power, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Power },
  { &hf_nbap_tSTD_Indicator , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TSTD_Indicator },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_SecondarySCH_Information_Cell_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_SecondarySCH_Information_Cell_SetupRqstFDD, SecondarySCH_Information_Cell_SetupRqstFDD_sequence);

  return offset;
}
static int dissect_id_SecondarySCH_Information_Cell_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SecondarySCH_Information_Cell_SetupRqstFDD(tvb, offset, actx, tree, hf_nbap_id_SecondarySCH_Information_Cell_SetupRqstFDD);
}



static int
dissect_nbap_IB_SG_POS(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 4094U, NULL, FALSE);

  return offset;
}
static int dissect_iB_SG_POS(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_IB_SG_POS(tvb, offset, actx, tree, hf_nbap_iB_SG_POS);
}


static const value_string nbap_Segment_Type_vals[] = {
  {   0, "first-segment" },
  {   1, "first-segment-short" },
  {   2, "subsequent-segment" },
  {   3, "last-segment" },
  {   4, "last-segment-short" },
  {   5, "complete-SIB" },
  {   6, "complete-SIB-short" },
  { 0, NULL }
};


static int
dissect_nbap_Segment_Type(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     7, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_segment_Type(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Segment_Type(tvb, offset, actx, tree, hf_nbap_segment_Type);
}



static int
dissect_nbap_IB_SG_DATA(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     NO_BOUND, NO_BOUND, FALSE, NULL);

  return offset;
}
static int dissect_iB_SG_DATA(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_IB_SG_DATA(tvb, offset, actx, tree, hf_nbap_iB_SG_DATA);
}


static const per_sequence_t SegmentInformationItem_SystemInfoUpdate_sequence[] = {
  { &hf_nbap_iB_SG_POS      , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_IB_SG_POS },
  { &hf_nbap_segment_Type   , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_Segment_Type },
  { &hf_nbap_iB_SG_DATA     , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_IB_SG_DATA },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_SegmentInformationItem_SystemInfoUpdate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_SegmentInformationItem_SystemInfoUpdate, SegmentInformationItem_SystemInfoUpdate_sequence);

  return offset;
}
static int dissect_SegmentInformationListIE_SystemInfoUpdate_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SegmentInformationItem_SystemInfoUpdate(tvb, offset, actx, tree, hf_nbap_SegmentInformationListIE_SystemInfoUpdate_item);
}


static const per_sequence_t SegmentInformationListIE_SystemInfoUpdate_sequence_of[1] = {
  { &hf_nbap_SegmentInformationListIE_SystemInfoUpdate_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_SegmentInformationItem_SystemInfoUpdate },
};

static int
dissect_nbap_SegmentInformationListIE_SystemInfoUpdate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_SegmentInformationListIE_SystemInfoUpdate, SegmentInformationListIE_SystemInfoUpdate_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_SegmentInformationListIE_SystemInfoUpdate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SegmentInformationListIE_SystemInfoUpdate(tvb, offset, actx, tree, hf_nbap_id_SegmentInformationListIE_SystemInfoUpdate);
}



static int
dissect_nbap_SFN(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 4095U, NULL, FALSE);

  return offset;
}
static int dissect_id_SFN(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SFN(tvb, offset, actx, tree, hf_nbap_id_SFN);
}
static int dissect_sfn(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SFN(tvb, offset, actx, tree, hf_nbap_sfn);
}
static int dissect_sFN(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SFN(tvb, offset, actx, tree, hf_nbap_sFN);
}
static int dissect_sFNSFNTimeStamp_FDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SFN(tvb, offset, actx, tree, hf_nbap_sFNSFNTimeStamp_FDD);
}


static const value_string nbap_SignallingBearerRequestIndicator_vals[] = {
  {   0, "bearerRequested" },
  { 0, NULL }
};


static int
dissect_nbap_SignallingBearerRequestIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_SignallingBearerRequestIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SignallingBearerRequestIndicator(tvb, offset, actx, tree, hf_nbap_id_SignallingBearerRequestIndicator);
}



static int
dissect_nbap_ShutdownTimer(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 3600U, NULL, FALSE);

  return offset;
}
static int dissect_id_ShutdownTimer(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ShutdownTimer(tvb, offset, actx, tree, hf_nbap_id_ShutdownTimer);
}


static const value_string nbap_Start_Of_Audit_Sequence_Indicator_vals[] = {
  {   0, "start-of-audit-sequence" },
  {   1, "not-start-of-audit-sequence" },
  { 0, NULL }
};


static int
dissect_nbap_Start_Of_Audit_Sequence_Indicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_Start_Of_Audit_Sequence_Indicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Start_Of_Audit_Sequence_Indicator(tvb, offset, actx, tree, hf_nbap_id_Start_Of_Audit_Sequence_Indicator);
}


static const per_sequence_t Combining_RL_AdditionFailureFDD_sequence[] = {
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Combining_RL_AdditionFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Combining_RL_AdditionFailureFDD, Combining_RL_AdditionFailureFDD_sequence);

  return offset;
}
static int dissect_combining4(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Combining_RL_AdditionFailureFDD(tvb, offset, actx, tree, hf_nbap_combining4);
}


static const per_sequence_t Non_Combining_RL_AdditionFailureFDD_sequence[] = {
  { &hf_nbap_dCH_InformationResponse, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DCH_InformationResponse },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Non_Combining_RL_AdditionFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Non_Combining_RL_AdditionFailureFDD, Non_Combining_RL_AdditionFailureFDD_sequence);

  return offset;
}
static int dissect_non_Combining1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Non_Combining_RL_AdditionFailureFDD(tvb, offset, actx, tree, hf_nbap_non_Combining1);
}


static const value_string nbap_DiversityIndication_RL_AdditionFailureFDD_vals[] = {
  {   0, "combining" },
  {   1, "non-Combining" },
  { 0, NULL }
};

static const per_choice_t DiversityIndication_RL_AdditionFailureFDD_choice[] = {
  {   0, &hf_nbap_combining4     , ASN1_NO_EXTENSIONS     , dissect_nbap_Combining_RL_AdditionFailureFDD },
  {   1, &hf_nbap_non_Combining1 , ASN1_NO_EXTENSIONS     , dissect_nbap_Non_Combining_RL_AdditionFailureFDD },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_DiversityIndication_RL_AdditionFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_DiversityIndication_RL_AdditionFailureFDD, DiversityIndication_RL_AdditionFailureFDD_choice,
                                 NULL);

  return offset;
}
static int dissect_diversityIndication4(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DiversityIndication_RL_AdditionFailureFDD(tvb, offset, actx, tree, hf_nbap_diversityIndication4);
}


static const per_sequence_t Successful_RL_InformationRespItem_RL_AdditionFailureFDD_sequence[] = {
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_ID },
  { &hf_nbap_rL_Set_ID      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_Set_ID },
  { &hf_nbap_received_total_wide_band_power, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Received_total_wide_band_power_Value },
  { &hf_nbap_diversityIndication4, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DiversityIndication_RL_AdditionFailureFDD },
  { &hf_nbap_sSDT_SupportIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SSDT_SupportIndicator },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Successful_RL_InformationRespItem_RL_AdditionFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Successful_RL_InformationRespItem_RL_AdditionFailureFDD, Successful_RL_InformationRespItem_RL_AdditionFailureFDD_sequence);

  return offset;
}
static int dissect_id_Successful_RL_InformationRespItem_RL_AdditionFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Successful_RL_InformationRespItem_RL_AdditionFailureFDD(tvb, offset, actx, tree, hf_nbap_id_Successful_RL_InformationRespItem_RL_AdditionFailureFDD);
}


static const per_sequence_t Combining_RL_SetupFailureFDD_sequence[] = {
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Combining_RL_SetupFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Combining_RL_SetupFailureFDD, Combining_RL_SetupFailureFDD_sequence);

  return offset;
}
static int dissect_combining1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Combining_RL_SetupFailureFDD(tvb, offset, actx, tree, hf_nbap_combining1);
}


static const per_sequence_t NonCombiningOrFirstRL_RL_SetupFailureFDD_sequence[] = {
  { &hf_nbap_dCH_InformationResponse, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DCH_InformationResponse },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_NonCombiningOrFirstRL_RL_SetupFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_NonCombiningOrFirstRL_RL_SetupFailureFDD, NonCombiningOrFirstRL_RL_SetupFailureFDD_sequence);

  return offset;
}
static int dissect_nonCombiningOrFirstRL1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NonCombiningOrFirstRL_RL_SetupFailureFDD(tvb, offset, actx, tree, hf_nbap_nonCombiningOrFirstRL1);
}


static const value_string nbap_DiversityIndication_RL_SetupFailureFDD_vals[] = {
  {   0, "combining" },
  {   1, "nonCombiningOrFirstRL" },
  { 0, NULL }
};

static const per_choice_t DiversityIndication_RL_SetupFailureFDD_choice[] = {
  {   0, &hf_nbap_combining1     , ASN1_NO_EXTENSIONS     , dissect_nbap_Combining_RL_SetupFailureFDD },
  {   1, &hf_nbap_nonCombiningOrFirstRL1, ASN1_NO_EXTENSIONS     , dissect_nbap_NonCombiningOrFirstRL_RL_SetupFailureFDD },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_DiversityIndication_RL_SetupFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_DiversityIndication_RL_SetupFailureFDD, DiversityIndication_RL_SetupFailureFDD_choice,
                                 NULL);

  return offset;
}
static int dissect_diversityIndication1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DiversityIndication_RL_SetupFailureFDD(tvb, offset, actx, tree, hf_nbap_diversityIndication1);
}


static const per_sequence_t Successful_RL_InformationRespItem_RL_SetupFailureFDD_sequence[] = {
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_ID },
  { &hf_nbap_rL_Set_ID      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_Set_ID },
  { &hf_nbap_received_total_wide_band_power, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Received_total_wide_band_power_Value },
  { &hf_nbap_diversityIndication1, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DiversityIndication_RL_SetupFailureFDD },
  { &hf_nbap_not_Used_dSCH_InformationResponseList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_NULL },
  { &hf_nbap_not_Used_tFCI2_BearerInformationResponse, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_NULL },
  { &hf_nbap_sSDT_SupportIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SSDT_SupportIndicator },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Successful_RL_InformationRespItem_RL_SetupFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Successful_RL_InformationRespItem_RL_SetupFailureFDD, Successful_RL_InformationRespItem_RL_SetupFailureFDD_sequence);

  return offset;
}
static int dissect_id_Successful_RL_InformationRespItem_RL_SetupFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Successful_RL_InformationRespItem_RL_SetupFailureFDD(tvb, offset, actx, tree, hf_nbap_id_Successful_RL_InformationRespItem_RL_SetupFailureFDD);
}



static int
dissect_nbap_SyncCase(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 2U, NULL, TRUE);

  return offset;
}
static int dissect_id_SyncCase(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SyncCase(tvb, offset, actx, tree, hf_nbap_id_SyncCase);
}


static const per_sequence_t Case1_Cell_SetupRqstTDD_sequence[] = {
  { &hf_nbap_timeSlot       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlot },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Case1_Cell_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Case1_Cell_SetupRqstTDD, Case1_Cell_SetupRqstTDD_sequence);

  return offset;
}
static int dissect_case1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Case1_Cell_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_case1);
}



static int
dissect_nbap_SCH_TimeSlot(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 6U, NULL, FALSE);

  return offset;
}
static int dissect_sCH_TimeSlot(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SCH_TimeSlot(tvb, offset, actx, tree, hf_nbap_sCH_TimeSlot);
}


static const per_sequence_t Case2_Cell_SetupRqstTDD_sequence[] = {
  { &hf_nbap_sCH_TimeSlot   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SCH_TimeSlot },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Case2_Cell_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Case2_Cell_SetupRqstTDD, Case2_Cell_SetupRqstTDD_sequence);

  return offset;
}
static int dissect_case2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Case2_Cell_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_case2);
}


static const value_string nbap_SyncCaseIndicatorItem_Cell_SetupRqstTDD_PSCH_vals[] = {
  {   0, "case1" },
  {   1, "case2" },
  { 0, NULL }
};

static const per_choice_t SyncCaseIndicatorItem_Cell_SetupRqstTDD_PSCH_choice[] = {
  {   0, &hf_nbap_case1          , ASN1_EXTENSION_ROOT    , dissect_nbap_Case1_Cell_SetupRqstTDD },
  {   1, &hf_nbap_case2          , ASN1_EXTENSION_ROOT    , dissect_nbap_Case2_Cell_SetupRqstTDD },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_SyncCaseIndicatorItem_Cell_SetupRqstTDD_PSCH(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_SyncCaseIndicatorItem_Cell_SetupRqstTDD_PSCH, SyncCaseIndicatorItem_Cell_SetupRqstTDD_PSCH_choice,
                                 NULL);

  return offset;
}
static int dissect_id_SyncCaseIndicatorItem_Cell_SetupRqstTDD_PSCH(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SyncCaseIndicatorItem_Cell_SetupRqstTDD_PSCH(tvb, offset, actx, tree, hf_nbap_id_SyncCaseIndicatorItem_Cell_SetupRqstTDD_PSCH);
}


static const value_string nbap_T_Cell_vals[] = {
  {   0, "v0" },
  {   1, "v1" },
  {   2, "v2" },
  {   3, "v3" },
  {   4, "v4" },
  {   5, "v5" },
  {   6, "v6" },
  {   7, "v7" },
  {   8, "v8" },
  {   9, "v9" },
  { 0, NULL }
};


static int
dissect_nbap_T_Cell(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     10, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_T_Cell(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_T_Cell(tvb, offset, actx, tree, hf_nbap_id_T_Cell);
}


static const value_string nbap_TimeSlotStatus_vals[] = {
  {   0, "active" },
  {   1, "not-active" },
  { 0, NULL }
};


static int
dissect_nbap_TimeSlotStatus(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_timeSlotStatus(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TimeSlotStatus(tvb, offset, actx, tree, hf_nbap_timeSlotStatus);
}


static const value_string nbap_TimeSlotDirection_vals[] = {
  {   0, "ul" },
  {   1, "dl" },
  { 0, NULL }
};


static int
dissect_nbap_TimeSlotDirection(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_timeSlotDirection(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TimeSlotDirection(tvb, offset, actx, tree, hf_nbap_timeSlotDirection);
}


static const per_sequence_t TimeSlotConfigurationItem_Cell_ReconfRqstTDD_sequence[] = {
  { &hf_nbap_timeSlot       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlot },
  { &hf_nbap_timeSlotStatus , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlotStatus },
  { &hf_nbap_timeSlotDirection, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlotDirection },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_TimeSlotConfigurationItem_Cell_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_TimeSlotConfigurationItem_Cell_ReconfRqstTDD, TimeSlotConfigurationItem_Cell_ReconfRqstTDD_sequence);

  return offset;
}
static int dissect_TimeSlotConfigurationList_Cell_ReconfRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TimeSlotConfigurationItem_Cell_ReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_TimeSlotConfigurationList_Cell_ReconfRqstTDD_item);
}


static const per_sequence_t TimeSlotConfigurationList_Cell_ReconfRqstTDD_sequence_of[1] = {
  { &hf_nbap_TimeSlotConfigurationList_Cell_ReconfRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlotConfigurationItem_Cell_ReconfRqstTDD },
};

static int
dissect_nbap_TimeSlotConfigurationList_Cell_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_TimeSlotConfigurationList_Cell_ReconfRqstTDD, TimeSlotConfigurationList_Cell_ReconfRqstTDD_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_id_TimeSlotConfigurationList_Cell_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TimeSlotConfigurationList_Cell_ReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_id_TimeSlotConfigurationList_Cell_ReconfRqstTDD);
}


static const per_sequence_t TimeSlotConfigurationItem_Cell_SetupRqstTDD_sequence[] = {
  { &hf_nbap_timeSlot       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlot },
  { &hf_nbap_timeSlotStatus , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlotStatus },
  { &hf_nbap_timeSlotDirection, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlotDirection },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_TimeSlotConfigurationItem_Cell_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_TimeSlotConfigurationItem_Cell_SetupRqstTDD, TimeSlotConfigurationItem_Cell_SetupRqstTDD_sequence);

  return offset;
}
static int dissect_TimeSlotConfigurationList_Cell_SetupRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TimeSlotConfigurationItem_Cell_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_TimeSlotConfigurationList_Cell_SetupRqstTDD_item);
}


static const per_sequence_t TimeSlotConfigurationList_Cell_SetupRqstTDD_sequence_of[1] = {
  { &hf_nbap_TimeSlotConfigurationList_Cell_SetupRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlotConfigurationItem_Cell_SetupRqstTDD },
};

static int
dissect_nbap_TimeSlotConfigurationList_Cell_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_TimeSlotConfigurationList_Cell_SetupRqstTDD, TimeSlotConfigurationList_Cell_SetupRqstTDD_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_id_TimeSlotConfigurationList_Cell_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TimeSlotConfigurationList_Cell_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_id_TimeSlotConfigurationList_Cell_SetupRqstTDD);
}



static int
dissect_nbap_TransmissionDiversityApplied(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_boolean(tvb, offset, actx, tree, hf_index, NULL);

  return offset;
}
static int dissect_id_TransmissionDiversityApplied(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TransmissionDiversityApplied(tvb, offset, actx, tree, hf_nbap_id_TransmissionDiversityApplied);
}


static const value_string nbap_TypeOfError_vals[] = {
  {   0, "not-understood" },
  {   1, "missing" },
  { 0, NULL }
};


static int
dissect_nbap_TypeOfError(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_id_TypeOfError(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TypeOfError(tvb, offset, actx, tree, hf_nbap_id_TypeOfError);
}



static int
dissect_nbap_UL_DPCH_Information_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_uL_DPCH_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_DPCH_Information_RL_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_uL_DPCH_Information);
}


static const per_sequence_t UL_CCTrCH_InformationItem_RL_SetupRqstTDD_sequence[] = {
  { &hf_nbap_cCTrCH_ID      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CCTrCH_ID },
  { &hf_nbap_tFCS           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TFCS },
  { &hf_nbap_tFCI_Coding    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TFCI_Coding },
  { &hf_nbap_punctureLimit  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PunctureLimit },
  { &hf_nbap_uL_DPCH_Information, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_UL_DPCH_Information_RL_SetupRqstTDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_UL_CCTrCH_InformationItem_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_UL_CCTrCH_InformationItem_RL_SetupRqstTDD, UL_CCTrCH_InformationItem_RL_SetupRqstTDD_sequence);

  return offset;
}
static int dissect_id_UL_CCTrCH_InformationItem_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_CCTrCH_InformationItem_RL_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_id_UL_CCTrCH_InformationItem_RL_SetupRqstTDD);
}


static const per_sequence_t UL_CCTrCH_InformationList_RL_SetupRqstTDD_sequence_of[1] = {
  { &hf_nbap_UL_CCTrCH_InformationList_RL_SetupRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_UL_CCTrCH_InformationList_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_UL_CCTrCH_InformationList_RL_SetupRqstTDD, UL_CCTrCH_InformationList_RL_SetupRqstTDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_UL_CCTrCH_InformationList_RL_AdditionRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_CCTrCH_InformationList_RL_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_id_UL_CCTrCH_InformationList_RL_AdditionRqstTDD);
}
static int dissect_id_UL_CCTrCH_InformationList_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_CCTrCH_InformationList_RL_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_id_UL_CCTrCH_InformationList_RL_SetupRqstTDD);
}


static const per_sequence_t TDD_UL_Code_InformationItem_sequence[] = {
  { &hf_nbap_dPCH_ID        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DPCH_ID },
  { &hf_nbap_tdd_ChannelisationCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_ChannelisationCode },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_TDD_UL_Code_InformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_TDD_UL_Code_InformationItem, TDD_UL_Code_InformationItem_sequence);

  return offset;
}
static int dissect_TDD_UL_Code_Information_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TDD_UL_Code_InformationItem(tvb, offset, actx, tree, hf_nbap_TDD_UL_Code_Information_item);
}


static const per_sequence_t TDD_UL_Code_Information_sequence_of[1] = {
  { &hf_nbap_TDD_UL_Code_Information_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_UL_Code_InformationItem },
};

static int
dissect_nbap_TDD_UL_Code_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_TDD_UL_Code_Information, TDD_UL_Code_Information_sequence_of,
                                                  1, 240);

  return offset;
}
static int dissect_uL_Code_InformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TDD_UL_Code_Information(tvb, offset, actx, tree, hf_nbap_uL_Code_InformationList);
}


static const per_sequence_t UL_Timeslot_InformationItem_sequence[] = {
  { &hf_nbap_timeSlot       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlot },
  { &hf_nbap_midambleShiftAndBurstType, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MidambleShiftAndBurstType },
  { &hf_nbap_tFCI_Presence  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TFCI_Presence },
  { &hf_nbap_uL_Code_InformationList, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_UL_Code_Information },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_UL_Timeslot_InformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_UL_Timeslot_InformationItem, UL_Timeslot_InformationItem_sequence);

  return offset;
}
static int dissect_UL_Timeslot_Information_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_Timeslot_InformationItem(tvb, offset, actx, tree, hf_nbap_UL_Timeslot_Information_item);
}


static const per_sequence_t UL_Timeslot_Information_sequence_of[1] = {
  { &hf_nbap_UL_Timeslot_Information_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_UL_Timeslot_InformationItem },
};

static int
dissect_nbap_UL_Timeslot_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_UL_Timeslot_Information, UL_Timeslot_Information_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_uL_Timeslot_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_Timeslot_Information(tvb, offset, actx, tree, hf_nbap_uL_Timeslot_Information);
}


static const per_sequence_t UL_DPCH_InformationItem_RL_AdditionRqstTDD_sequence[] = {
  { &hf_nbap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionPeriod },
  { &hf_nbap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionLength },
  { &hf_nbap_tdd_DPCHOffset , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_DPCHOffset },
  { &hf_nbap_uL_Timeslot_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UL_Timeslot_Information },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_UL_DPCH_InformationItem_RL_AdditionRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_UL_DPCH_InformationItem_RL_AdditionRqstTDD, UL_DPCH_InformationItem_RL_AdditionRqstTDD_sequence);

  return offset;
}
static int dissect_id_UL_DPCH_InformationItem_RL_AdditionRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_DPCH_InformationItem_RL_AdditionRqstTDD(tvb, offset, actx, tree, hf_nbap_id_UL_DPCH_InformationItem_RL_AdditionRqstTDD);
}


static const per_sequence_t UL_DPCH_InformationItem_RL_SetupRqstTDD_sequence[] = {
  { &hf_nbap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionPeriod },
  { &hf_nbap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionLength },
  { &hf_nbap_tdd_DPCHOffset , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_DPCHOffset },
  { &hf_nbap_uL_Timeslot_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UL_Timeslot_Information },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_UL_DPCH_InformationItem_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_UL_DPCH_InformationItem_RL_SetupRqstTDD, UL_DPCH_InformationItem_RL_SetupRqstTDD_sequence);

  return offset;
}
static int dissect_id_UL_DPCH_InformationList_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_DPCH_InformationItem_RL_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_id_UL_DPCH_InformationList_RL_SetupRqstTDD);
}



static int
dissect_nbap_UL_ScramblingCodeNumber(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 16777215U, NULL, FALSE);

  return offset;
}
static int dissect_uL_ScramblingCodeNumber(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_ScramblingCodeNumber(tvb, offset, actx, tree, hf_nbap_uL_ScramblingCodeNumber);
}


static const value_string nbap_UL_ScramblingCodeLength_vals[] = {
  {   0, "short" },
  {   1, "long" },
  { 0, NULL }
};


static int
dissect_nbap_UL_ScramblingCodeLength(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_uL_ScramblingCodeLength(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_ScramblingCodeLength(tvb, offset, actx, tree, hf_nbap_uL_ScramblingCodeLength);
}


static const per_sequence_t UL_ScramblingCode_sequence[] = {
  { &hf_nbap_uL_ScramblingCodeNumber, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UL_ScramblingCodeNumber },
  { &hf_nbap_uL_ScramblingCodeLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UL_ScramblingCodeLength },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_UL_ScramblingCode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_UL_ScramblingCode, UL_ScramblingCode_sequence);

  return offset;
}
static int dissect_ul_ScramblingCode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_ScramblingCode(tvb, offset, actx, tree, hf_nbap_ul_ScramblingCode);
}



static int
dissect_nbap_UL_SIR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              -82, 173U, NULL, FALSE);

  return offset;
}
static int dissect_id_UL_SIRTarget(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_SIR(tvb, offset, actx, tree, hf_nbap_id_UL_SIRTarget);
}
static int dissect_ul_SIR_Target(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_SIR(tvb, offset, actx, tree, hf_nbap_ul_SIR_Target);
}
static int dissect_ul_sir_target(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_SIR(tvb, offset, actx, tree, hf_nbap_ul_sir_target);
}


static const value_string nbap_MinUL_ChannelisationCodeLength_vals[] = {
  {   0, "v4" },
  {   1, "v8" },
  {   2, "v16" },
  {   3, "v32" },
  {   4, "v64" },
  {   5, "v128" },
  {   6, "v256" },
  { 0, NULL }
};


static int
dissect_nbap_MinUL_ChannelisationCodeLength(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     7, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_minUL_ChannelisationCodeLength(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MinUL_ChannelisationCodeLength(tvb, offset, actx, tree, hf_nbap_minUL_ChannelisationCodeLength);
}



static int
dissect_nbap_MaxNrOfUL_DPDCHs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 6U, NULL, FALSE);

  return offset;
}
static int dissect_maxNrOfUL_DPDCHs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MaxNrOfUL_DPDCHs(tvb, offset, actx, tree, hf_nbap_maxNrOfUL_DPDCHs);
}



static int
dissect_nbap_UL_DPCCH_SlotFormat(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 5U, NULL, TRUE);

  return offset;
}
static int dissect_ul_DPCCH_SlotFormat(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_DPCCH_SlotFormat(tvb, offset, actx, tree, hf_nbap_ul_DPCCH_SlotFormat);
}


static const value_string nbap_DiversityMode_vals[] = {
  {   0, "none" },
  {   1, "sTTD" },
  {   2, "closed-loop-mode1" },
  {   3, "not-used-closed-loop-mode2" },
  { 0, NULL }
};


static int
dissect_nbap_DiversityMode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_diversityMode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DiversityMode(tvb, offset, actx, tree, hf_nbap_diversityMode);
}


static const per_sequence_t UL_DPCH_Information_RL_ReconfPrepFDD_sequence[] = {
  { &hf_nbap_ul_ScramblingCode, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_UL_ScramblingCode },
  { &hf_nbap_ul_SIR_Target  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_UL_SIR },
  { &hf_nbap_minUL_ChannelisationCodeLength, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_MinUL_ChannelisationCodeLength },
  { &hf_nbap_maxNrOfUL_DPDCHs, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_MaxNrOfUL_DPDCHs },
  { &hf_nbap_ul_PunctureLimit, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_PunctureLimit },
  { &hf_nbap_tFCS           , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TFCS },
  { &hf_nbap_ul_DPCCH_SlotFormat, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_UL_DPCCH_SlotFormat },
  { &hf_nbap_diversityMode  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DiversityMode },
  { &hf_nbap_not_Used_sSDT_CellIDLength, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_NULL },
  { &hf_nbap_not_Used_s_FieldLength, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_NULL },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_UL_DPCH_Information_RL_ReconfPrepFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_UL_DPCH_Information_RL_ReconfPrepFDD, UL_DPCH_Information_RL_ReconfPrepFDD_sequence);

  return offset;
}
static int dissect_id_UL_DPCH_Information_RL_ReconfPrepFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_DPCH_Information_RL_ReconfPrepFDD(tvb, offset, actx, tree, hf_nbap_id_UL_DPCH_Information_RL_ReconfPrepFDD);
}


static const per_sequence_t UL_DPCH_Information_RL_ReconfRqstFDD_sequence[] = {
  { &hf_nbap_ul_TFCS        , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TFCS },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_UL_DPCH_Information_RL_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_UL_DPCH_Information_RL_ReconfRqstFDD, UL_DPCH_Information_RL_ReconfRqstFDD_sequence);

  return offset;
}
static int dissect_id_UL_DPCH_Information_RL_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_DPCH_Information_RL_ReconfRqstFDD(tvb, offset, actx, tree, hf_nbap_id_UL_DPCH_Information_RL_ReconfRqstFDD);
}


static const per_sequence_t UL_DPCH_Information_RL_SetupRqstFDD_sequence[] = {
  { &hf_nbap_ul_ScramblingCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UL_ScramblingCode },
  { &hf_nbap_minUL_ChannelisationCodeLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MinUL_ChannelisationCodeLength },
  { &hf_nbap_maxNrOfUL_DPDCHs, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_MaxNrOfUL_DPDCHs },
  { &hf_nbap_ul_PunctureLimit, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PunctureLimit },
  { &hf_nbap_tFCS           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TFCS },
  { &hf_nbap_ul_DPCCH_SlotFormat, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UL_DPCCH_SlotFormat },
  { &hf_nbap_ul_SIR_Target  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UL_SIR },
  { &hf_nbap_diversityMode  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DiversityMode },
  { &hf_nbap_not_Used_sSDT_CellID_Length, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_NULL },
  { &hf_nbap_not_Used_s_FieldLength, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_NULL },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_UL_DPCH_Information_RL_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_UL_DPCH_Information_RL_SetupRqstFDD, UL_DPCH_Information_RL_SetupRqstFDD_sequence);

  return offset;
}
static int dissect_id_UL_DPCH_Information_RL_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_DPCH_Information_RL_SetupRqstFDD(tvb, offset, actx, tree, hf_nbap_id_UL_DPCH_Information_RL_SetupRqstFDD);
}


static const per_sequence_t Unsuccessful_RL_InformationRespItem_RL_AdditionFailureFDD_sequence[] = {
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_ID },
  { &hf_nbap_cause          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Cause },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Unsuccessful_RL_InformationRespItem_RL_AdditionFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Unsuccessful_RL_InformationRespItem_RL_AdditionFailureFDD, Unsuccessful_RL_InformationRespItem_RL_AdditionFailureFDD_sequence);

  return offset;
}
static int dissect_id_Unsuccessful_RL_InformationRespItem_RL_AdditionFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Unsuccessful_RL_InformationRespItem_RL_AdditionFailureFDD(tvb, offset, actx, tree, hf_nbap_id_Unsuccessful_RL_InformationRespItem_RL_AdditionFailureFDD);
}


static const per_sequence_t Unsuccessful_RL_InformationRespItem_RL_SetupFailureFDD_sequence[] = {
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_ID },
  { &hf_nbap_cause          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Cause },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Unsuccessful_RL_InformationRespItem_RL_SetupFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Unsuccessful_RL_InformationRespItem_RL_SetupFailureFDD, Unsuccessful_RL_InformationRespItem_RL_SetupFailureFDD_sequence);

  return offset;
}
static int dissect_id_Unsuccessful_RL_InformationRespItem_RL_SetupFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Unsuccessful_RL_InformationRespItem_RL_SetupFailureFDD(tvb, offset, actx, tree, hf_nbap_id_Unsuccessful_RL_InformationRespItem_RL_SetupFailureFDD);
}


static const per_sequence_t Unsuccessful_RL_InformationResp_RL_AdditionFailureTDD_sequence[] = {
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_ID },
  { &hf_nbap_cause          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Cause },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Unsuccessful_RL_InformationResp_RL_AdditionFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Unsuccessful_RL_InformationResp_RL_AdditionFailureTDD, Unsuccessful_RL_InformationResp_RL_AdditionFailureTDD_sequence);

  return offset;
}
static int dissect_id_Unsuccessful_RL_InformationResp_RL_AdditionFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Unsuccessful_RL_InformationResp_RL_AdditionFailureTDD(tvb, offset, actx, tree, hf_nbap_id_Unsuccessful_RL_InformationResp_RL_AdditionFailureTDD);
}


static const per_sequence_t Unsuccessful_RL_InformationResp_RL_SetupFailureTDD_sequence[] = {
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_ID },
  { &hf_nbap_cause          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Cause },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Unsuccessful_RL_InformationResp_RL_SetupFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Unsuccessful_RL_InformationResp_RL_SetupFailureTDD, Unsuccessful_RL_InformationResp_RL_SetupFailureTDD_sequence);

  return offset;
}
static int dissect_id_Unsuccessful_RL_InformationResp_RL_SetupFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Unsuccessful_RL_InformationResp_RL_SetupFailureTDD(tvb, offset, actx, tree, hf_nbap_id_Unsuccessful_RL_InformationResp_RL_SetupFailureTDD);
}



static int
dissect_nbap_USCH_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 255U, NULL, FALSE);

  return offset;
}
static int dissect_uSCH_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_USCH_ID(tvb, offset, actx, tree, hf_nbap_uSCH_ID);
}


static const per_sequence_t USCH_InformationItem_sequence[] = {
  { &hf_nbap_uSCH_ID        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_USCH_ID },
  { &hf_nbap_cCTrCH_ID      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CCTrCH_ID },
  { &hf_nbap_transportFormatSet, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TransportFormatSet },
  { &hf_nbap_allocationRetentionPriority, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_AllocationRetentionPriority },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_USCH_InformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_USCH_InformationItem, USCH_InformationItem_sequence);

  return offset;
}
static int dissect_USCH_Information_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_USCH_InformationItem(tvb, offset, actx, tree, hf_nbap_USCH_Information_item);
}


static const per_sequence_t USCH_Information_sequence_of[1] = {
  { &hf_nbap_USCH_Information_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_USCH_InformationItem },
};

static int
dissect_nbap_USCH_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_USCH_Information, USCH_Information_sequence_of,
                                                  1, 32);

  return offset;
}
static int dissect_id_USCH_Information_Add(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_USCH_Information(tvb, offset, actx, tree, hf_nbap_id_USCH_Information_Add);
}
static int dissect_id_USCH_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_USCH_Information(tvb, offset, actx, tree, hf_nbap_id_USCH_Information);
}


static const per_sequence_t USCH_Information_DeleteItem_RL_ReconfPrepTDD_sequence[] = {
  { &hf_nbap_uSCH_ID        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_USCH_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_USCH_Information_DeleteItem_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_USCH_Information_DeleteItem_RL_ReconfPrepTDD, USCH_Information_DeleteItem_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_USCH_Information_DeleteList_RL_ReconfPrepTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_USCH_Information_DeleteItem_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_USCH_Information_DeleteList_RL_ReconfPrepTDD_item);
}


static const per_sequence_t USCH_Information_DeleteList_RL_ReconfPrepTDD_sequence_of[1] = {
  { &hf_nbap_USCH_Information_DeleteList_RL_ReconfPrepTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_USCH_Information_DeleteItem_RL_ReconfPrepTDD },
};

static int
dissect_nbap_USCH_Information_DeleteList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_USCH_Information_DeleteList_RL_ReconfPrepTDD, USCH_Information_DeleteList_RL_ReconfPrepTDD_sequence_of,
                                                  1, 32);

  return offset;
}
static int dissect_id_USCH_Information_DeleteList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_USCH_Information_DeleteList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_id_USCH_Information_DeleteList_RL_ReconfPrepTDD);
}


static const per_sequence_t USCH_Information_ModifyItem_RL_ReconfPrepTDD_sequence[] = {
  { &hf_nbap_uSCH_ID        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_USCH_ID },
  { &hf_nbap_transportFormatSet, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TransportFormatSet },
  { &hf_nbap_allocationRetentionPriority, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_AllocationRetentionPriority },
  { &hf_nbap_cCTrCH_ID      , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_CCTrCH_ID },
  { &hf_nbap_transportBearerRequestIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TransportBearerRequestIndicator },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_USCH_Information_ModifyItem_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_USCH_Information_ModifyItem_RL_ReconfPrepTDD, USCH_Information_ModifyItem_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_USCH_Information_ModifyList_RL_ReconfPrepTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_USCH_Information_ModifyItem_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_USCH_Information_ModifyList_RL_ReconfPrepTDD_item);
}


static const per_sequence_t USCH_Information_ModifyList_RL_ReconfPrepTDD_sequence_of[1] = {
  { &hf_nbap_USCH_Information_ModifyList_RL_ReconfPrepTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_USCH_Information_ModifyItem_RL_ReconfPrepTDD },
};

static int
dissect_nbap_USCH_Information_ModifyList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_USCH_Information_ModifyList_RL_ReconfPrepTDD, USCH_Information_ModifyList_RL_ReconfPrepTDD_sequence_of,
                                                  1, 32);

  return offset;
}
static int dissect_id_USCH_Information_ModifyList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_USCH_Information_ModifyList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_id_USCH_Information_ModifyList_RL_ReconfPrepTDD);
}


static const per_sequence_t USCH_InformationResponseItem_sequence[] = {
  { &hf_nbap_uSCH_ID        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_USCH_ID },
  { &hf_nbap_bindingID      , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_BindingID },
  { &hf_nbap_transportLayerAddress, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TransportLayerAddress },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_USCH_InformationResponseItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_USCH_InformationResponseItem, USCH_InformationResponseItem_sequence);

  return offset;
}
static int dissect_USCH_InformationResponse_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_USCH_InformationResponseItem(tvb, offset, actx, tree, hf_nbap_USCH_InformationResponse_item);
}


static const per_sequence_t USCH_InformationResponse_sequence_of[1] = {
  { &hf_nbap_USCH_InformationResponse_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_USCH_InformationResponseItem },
};

static int
dissect_nbap_USCH_InformationResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_USCH_InformationResponse, USCH_InformationResponse_sequence_of,
                                                  1, 32);

  return offset;
}
static int dissect_id_USCH_InformationResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_USCH_InformationResponse(tvb, offset, actx, tree, hf_nbap_id_USCH_InformationResponse);
}


static const per_sequence_t USCH_RearrangeItem_Bearer_RearrangeInd_sequence[] = {
  { &hf_nbap_uSCH_ID        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_USCH_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_USCH_RearrangeItem_Bearer_RearrangeInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_USCH_RearrangeItem_Bearer_RearrangeInd, USCH_RearrangeItem_Bearer_RearrangeInd_sequence);

  return offset;
}
static int dissect_USCH_RearrangeList_Bearer_RearrangeInd_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_USCH_RearrangeItem_Bearer_RearrangeInd(tvb, offset, actx, tree, hf_nbap_USCH_RearrangeList_Bearer_RearrangeInd_item);
}


static const per_sequence_t USCH_RearrangeList_Bearer_RearrangeInd_sequence_of[1] = {
  { &hf_nbap_USCH_RearrangeList_Bearer_RearrangeInd_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_USCH_RearrangeItem_Bearer_RearrangeInd },
};

static int
dissect_nbap_USCH_RearrangeList_Bearer_RearrangeInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_USCH_RearrangeList_Bearer_RearrangeInd, USCH_RearrangeList_Bearer_RearrangeInd_sequence_of,
                                                  1, 32);

  return offset;
}
static int dissect_id_USCH_RearrangeList_Bearer_RearrangeInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_USCH_RearrangeList_Bearer_RearrangeInd(tvb, offset, actx, tree, hf_nbap_id_USCH_RearrangeList_Bearer_RearrangeInd);
}



static int
dissect_nbap_TGPSID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 6U, NULL, FALSE);

  return offset;
}
static int dissect_tGPSID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TGPSID(tvb, offset, actx, tree, hf_nbap_tGPSID);
}



static int
dissect_nbap_TGPRC(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 511U, NULL, FALSE);

  return offset;
}
static int dissect_tGPRC(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TGPRC(tvb, offset, actx, tree, hf_nbap_tGPRC);
}


static const per_sequence_t Transmission_Gap_Pattern_Sequence_Status_List_item_sequence[] = {
  { &hf_nbap_tGPSID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TGPSID },
  { &hf_nbap_tGPRC          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TGPRC },
  { &hf_nbap_tGCFN          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CFN },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Transmission_Gap_Pattern_Sequence_Status_List_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Transmission_Gap_Pattern_Sequence_Status_List_item, Transmission_Gap_Pattern_Sequence_Status_List_item_sequence);

  return offset;
}
static int dissect_Transmission_Gap_Pattern_Sequence_Status_List_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Transmission_Gap_Pattern_Sequence_Status_List_item(tvb, offset, actx, tree, hf_nbap_Transmission_Gap_Pattern_Sequence_Status_List_item);
}


static const per_sequence_t Transmission_Gap_Pattern_Sequence_Status_List_sequence_of[1] = {
  { &hf_nbap_Transmission_Gap_Pattern_Sequence_Status_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_Transmission_Gap_Pattern_Sequence_Status_List_item },
};

static int
dissect_nbap_Transmission_Gap_Pattern_Sequence_Status_List(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_Transmission_Gap_Pattern_Sequence_Status_List, Transmission_Gap_Pattern_Sequence_Status_List_sequence_of,
                                                  0, 6);

  return offset;
}
static int dissect_transmission_Gap_Pattern_Sequence_Status(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Transmission_Gap_Pattern_Sequence_Status_List(tvb, offset, actx, tree, hf_nbap_transmission_Gap_Pattern_Sequence_Status);
}


static const per_sequence_t Active_Pattern_Sequence_Information_sequence[] = {
  { &hf_nbap_cMConfigurationChangeCFN, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CFN },
  { &hf_nbap_transmission_Gap_Pattern_Sequence_Status, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_Transmission_Gap_Pattern_Sequence_Status_List },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Active_Pattern_Sequence_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Active_Pattern_Sequence_Information, Active_Pattern_Sequence_Information_sequence);

  return offset;
}
static int dissect_id_Active_Pattern_Sequence_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Active_Pattern_Sequence_Information(tvb, offset, actx, tree, hf_nbap_id_Active_Pattern_Sequence_Information);
}


static const per_sequence_t AICH_ParametersItem_CTCH_ReconfRqstFDD_sequence[] = {
  { &hf_nbap_commonPhysicalChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonPhysicalChannelID },
  { &hf_nbap_aICH_Power     , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_AICH_Power },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_AICH_ParametersItem_CTCH_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_AICH_ParametersItem_CTCH_ReconfRqstFDD, AICH_ParametersItem_CTCH_ReconfRqstFDD_sequence);

  return offset;
}
static int dissect_AICH_ParametersListIE_CTCH_ReconfRqstFDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_AICH_ParametersItem_CTCH_ReconfRqstFDD(tvb, offset, actx, tree, hf_nbap_AICH_ParametersListIE_CTCH_ReconfRqstFDD_item);
}


static const per_sequence_t AICH_ParametersListIE_CTCH_ReconfRqstFDD_sequence_of[1] = {
  { &hf_nbap_AICH_ParametersListIE_CTCH_ReconfRqstFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_AICH_ParametersItem_CTCH_ReconfRqstFDD },
};

static int
dissect_nbap_AICH_ParametersListIE_CTCH_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_AICH_ParametersListIE_CTCH_ReconfRqstFDD, AICH_ParametersListIE_CTCH_ReconfRqstFDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_AICH_ParametersListIE_CTCH_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_AICH_ParametersListIE_CTCH_ReconfRqstFDD(tvb, offset, actx, tree, hf_nbap_id_AICH_ParametersListIE_CTCH_ReconfRqstFDD);
}



static int
dissect_nbap_ScaledAdjustmentRatio(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 100U, NULL, FALSE);

  return offset;
}
static int dissect_id_AdjustmentRatio(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ScaledAdjustmentRatio(tvb, offset, actx, tree, hf_nbap_id_AdjustmentRatio);
}
static int dissect_adjustmentRatio(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ScaledAdjustmentRatio(tvb, offset, actx, tree, hf_nbap_adjustmentRatio);
}


static const per_sequence_t FACH_ParametersItem_CTCH_ReconfRqstFDD_sequence[] = {
  { &hf_nbap_commonTransportChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonTransportChannelID },
  { &hf_nbap_maxFACH_Power  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_Power },
  { &hf_nbap_toAWS          , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ToAWS },
  { &hf_nbap_toAWE          , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ToAWE },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_FACH_ParametersItem_CTCH_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_FACH_ParametersItem_CTCH_ReconfRqstFDD, FACH_ParametersItem_CTCH_ReconfRqstFDD_sequence);

  return offset;
}
static int dissect_FACH_ParametersListIE_CTCH_ReconfRqstFDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_FACH_ParametersItem_CTCH_ReconfRqstFDD(tvb, offset, actx, tree, hf_nbap_FACH_ParametersListIE_CTCH_ReconfRqstFDD_item);
}


static const per_sequence_t FACH_ParametersListIE_CTCH_ReconfRqstFDD_sequence_of[1] = {
  { &hf_nbap_FACH_ParametersListIE_CTCH_ReconfRqstFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_FACH_ParametersItem_CTCH_ReconfRqstFDD },
};

static int
dissect_nbap_FACH_ParametersListIE_CTCH_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_FACH_ParametersListIE_CTCH_ReconfRqstFDD, FACH_ParametersListIE_CTCH_ReconfRqstFDD_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_id_FACH_ParametersListIE_CTCH_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_FACH_ParametersListIE_CTCH_ReconfRqstFDD(tvb, offset, actx, tree, hf_nbap_id_FACH_ParametersListIE_CTCH_ReconfRqstFDD);
}


static const per_sequence_t GeneralCauseList_PSCH_ReconfFailure_sequence[] = {
  { &hf_nbap_cause          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Cause },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_GeneralCauseList_PSCH_ReconfFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_GeneralCauseList_PSCH_ReconfFailure, GeneralCauseList_PSCH_ReconfFailure_sequence);

  return offset;
}
static int dissect_generalCause5(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_GeneralCauseList_PSCH_ReconfFailure(tvb, offset, actx, tree, hf_nbap_generalCause5);
}


static const per_sequence_t Unsuccessful_PDSCHSetList_PSCH_ReconfFailureTDD_sequence_of[1] = {
  { &hf_nbap_Unsuccessful_PDSCHSetList_PSCH_ReconfFailureTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_Unsuccessful_PDSCHSetList_PSCH_ReconfFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_Unsuccessful_PDSCHSetList_PSCH_ReconfFailureTDD, Unsuccessful_PDSCHSetList_PSCH_ReconfFailureTDD_sequence_of,
                                                  0, 256);

  return offset;
}
static int dissect_unsuccessful_PDSCHSetList_PSCH_ReconfFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Unsuccessful_PDSCHSetList_PSCH_ReconfFailureTDD(tvb, offset, actx, tree, hf_nbap_unsuccessful_PDSCHSetList_PSCH_ReconfFailureTDD);
}


static const per_sequence_t Unsuccessful_PUSCHSetList_PSCH_ReconfFailureTDD_sequence_of[1] = {
  { &hf_nbap_Unsuccessful_PUSCHSetList_PSCH_ReconfFailureTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_Unsuccessful_PUSCHSetList_PSCH_ReconfFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_Unsuccessful_PUSCHSetList_PSCH_ReconfFailureTDD, Unsuccessful_PUSCHSetList_PSCH_ReconfFailureTDD_sequence_of,
                                                  0, 256);

  return offset;
}
static int dissect_unsuccessful_PUSCHSetList_PSCH_ReconfFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Unsuccessful_PUSCHSetList_PSCH_ReconfFailureTDD(tvb, offset, actx, tree, hf_nbap_unsuccessful_PUSCHSetList_PSCH_ReconfFailureTDD);
}


static const per_sequence_t SetSpecificCauseList_PSCH_ReconfFailureTDD_sequence[] = {
  { &hf_nbap_unsuccessful_PDSCHSetList_PSCH_ReconfFailureTDD, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_Unsuccessful_PDSCHSetList_PSCH_ReconfFailureTDD },
  { &hf_nbap_unsuccessful_PUSCHSetList_PSCH_ReconfFailureTDD, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_Unsuccessful_PUSCHSetList_PSCH_ReconfFailureTDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_SetSpecificCauseList_PSCH_ReconfFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_SetSpecificCauseList_PSCH_ReconfFailureTDD, SetSpecificCauseList_PSCH_ReconfFailureTDD_sequence);

  return offset;
}
static int dissect_setSpecificCause(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SetSpecificCauseList_PSCH_ReconfFailureTDD(tvb, offset, actx, tree, hf_nbap_setSpecificCause);
}


static const value_string nbap_CauseLevel_PSCH_ReconfFailure_vals[] = {
  {   0, "generalCause" },
  {   1, "setSpecificCause" },
  { 0, NULL }
};

static const per_choice_t CauseLevel_PSCH_ReconfFailure_choice[] = {
  {   0, &hf_nbap_generalCause5  , ASN1_EXTENSION_ROOT    , dissect_nbap_GeneralCauseList_PSCH_ReconfFailure },
  {   1, &hf_nbap_setSpecificCause, ASN1_EXTENSION_ROOT    , dissect_nbap_SetSpecificCauseList_PSCH_ReconfFailureTDD },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_CauseLevel_PSCH_ReconfFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_CauseLevel_PSCH_ReconfFailure, CauseLevel_PSCH_ReconfFailure_choice,
                                 NULL);

  return offset;
}
static int dissect_id_CauseLevel_PSCH_ReconfFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CauseLevel_PSCH_ReconfFailure(tvb, offset, actx, tree, hf_nbap_id_CauseLevel_PSCH_ReconfFailure);
}


static const per_sequence_t GeneralCauseList_RL_AdditionFailureFDD_sequence[] = {
  { &hf_nbap_cause          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Cause },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_GeneralCauseList_RL_AdditionFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_GeneralCauseList_RL_AdditionFailureFDD, GeneralCauseList_RL_AdditionFailureFDD_sequence);

  return offset;
}
static int dissect_generalCause2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_GeneralCauseList_RL_AdditionFailureFDD(tvb, offset, actx, tree, hf_nbap_generalCause2);
}


static const per_sequence_t Unsuccessful_RL_InformationRespList_RL_AdditionFailureFDD_sequence_of[1] = {
  { &hf_nbap_Unsuccessful_RL_InformationRespList_RL_AdditionFailureFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_Unsuccessful_RL_InformationRespList_RL_AdditionFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_Unsuccessful_RL_InformationRespList_RL_AdditionFailureFDD, Unsuccessful_RL_InformationRespList_RL_AdditionFailureFDD_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_unsuccessful_RL_InformationRespList_RL_AdditionFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Unsuccessful_RL_InformationRespList_RL_AdditionFailureFDD(tvb, offset, actx, tree, hf_nbap_unsuccessful_RL_InformationRespList_RL_AdditionFailureFDD);
}


static const per_sequence_t Successful_RL_InformationRespList_RL_AdditionFailureFDD_sequence_of[1] = {
  { &hf_nbap_Successful_RL_InformationRespList_RL_AdditionFailureFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_Successful_RL_InformationRespList_RL_AdditionFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_Successful_RL_InformationRespList_RL_AdditionFailureFDD, Successful_RL_InformationRespList_RL_AdditionFailureFDD_sequence_of,
                                                  1, 14);

  return offset;
}
static int dissect_successful_RL_InformationRespList_RL_AdditionFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Successful_RL_InformationRespList_RL_AdditionFailureFDD(tvb, offset, actx, tree, hf_nbap_successful_RL_InformationRespList_RL_AdditionFailureFDD);
}


static const per_sequence_t RLSpecificCauseList_RL_AdditionFailureFDD_sequence[] = {
  { &hf_nbap_unsuccessful_RL_InformationRespList_RL_AdditionFailureFDD, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Unsuccessful_RL_InformationRespList_RL_AdditionFailureFDD },
  { &hf_nbap_successful_RL_InformationRespList_RL_AdditionFailureFDD, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_Successful_RL_InformationRespList_RL_AdditionFailureFDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RLSpecificCauseList_RL_AdditionFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RLSpecificCauseList_RL_AdditionFailureFDD, RLSpecificCauseList_RL_AdditionFailureFDD_sequence);

  return offset;
}
static int dissect_rLSpecificCause2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RLSpecificCauseList_RL_AdditionFailureFDD(tvb, offset, actx, tree, hf_nbap_rLSpecificCause2);
}


static const value_string nbap_CauseLevel_RL_AdditionFailureFDD_vals[] = {
  {   0, "generalCause" },
  {   1, "rLSpecificCause" },
  { 0, NULL }
};

static const per_choice_t CauseLevel_RL_AdditionFailureFDD_choice[] = {
  {   0, &hf_nbap_generalCause2  , ASN1_EXTENSION_ROOT    , dissect_nbap_GeneralCauseList_RL_AdditionFailureFDD },
  {   1, &hf_nbap_rLSpecificCause2, ASN1_EXTENSION_ROOT    , dissect_nbap_RLSpecificCauseList_RL_AdditionFailureFDD },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_CauseLevel_RL_AdditionFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_CauseLevel_RL_AdditionFailureFDD, CauseLevel_RL_AdditionFailureFDD_choice,
                                 NULL);

  return offset;
}
static int dissect_id_CauseLevel_RL_AdditionFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CauseLevel_RL_AdditionFailureFDD(tvb, offset, actx, tree, hf_nbap_id_CauseLevel_RL_AdditionFailureFDD);
}


static const per_sequence_t GeneralCauseList_RL_AdditionFailureTDD_sequence[] = {
  { &hf_nbap_cause          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Cause },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_GeneralCauseList_RL_AdditionFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_GeneralCauseList_RL_AdditionFailureTDD, GeneralCauseList_RL_AdditionFailureTDD_sequence);

  return offset;
}
static int dissect_generalCause3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_GeneralCauseList_RL_AdditionFailureTDD(tvb, offset, actx, tree, hf_nbap_generalCause3);
}



static int
dissect_nbap_Unsuccessful_RL_InformationRespItem_RL_AdditionFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_unsuccessful_RL_InformationRespItem_RL_AdditionFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Unsuccessful_RL_InformationRespItem_RL_AdditionFailureTDD(tvb, offset, actx, tree, hf_nbap_unsuccessful_RL_InformationRespItem_RL_AdditionFailureTDD);
}


static const per_sequence_t RLSpecificCauseList_RL_AdditionFailureTDD_sequence[] = {
  { &hf_nbap_unsuccessful_RL_InformationRespItem_RL_AdditionFailureTDD, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Unsuccessful_RL_InformationRespItem_RL_AdditionFailureTDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RLSpecificCauseList_RL_AdditionFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RLSpecificCauseList_RL_AdditionFailureTDD, RLSpecificCauseList_RL_AdditionFailureTDD_sequence);

  return offset;
}
static int dissect_rLSpecificCause3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RLSpecificCauseList_RL_AdditionFailureTDD(tvb, offset, actx, tree, hf_nbap_rLSpecificCause3);
}


static const value_string nbap_CauseLevel_RL_AdditionFailureTDD_vals[] = {
  {   0, "generalCause" },
  {   1, "rLSpecificCause" },
  { 0, NULL }
};

static const per_choice_t CauseLevel_RL_AdditionFailureTDD_choice[] = {
  {   0, &hf_nbap_generalCause3  , ASN1_EXTENSION_ROOT    , dissect_nbap_GeneralCauseList_RL_AdditionFailureTDD },
  {   1, &hf_nbap_rLSpecificCause3, ASN1_EXTENSION_ROOT    , dissect_nbap_RLSpecificCauseList_RL_AdditionFailureTDD },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_CauseLevel_RL_AdditionFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_CauseLevel_RL_AdditionFailureTDD, CauseLevel_RL_AdditionFailureTDD_choice,
                                 NULL);

  return offset;
}
static int dissect_id_CauseLevel_RL_AdditionFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CauseLevel_RL_AdditionFailureTDD(tvb, offset, actx, tree, hf_nbap_id_CauseLevel_RL_AdditionFailureTDD);
}


static const per_sequence_t GeneralCauseList_RL_ReconfFailure_sequence[] = {
  { &hf_nbap_cause          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Cause },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_GeneralCauseList_RL_ReconfFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_GeneralCauseList_RL_ReconfFailure, GeneralCauseList_RL_ReconfFailure_sequence);

  return offset;
}
static int dissect_generalCause4(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_GeneralCauseList_RL_ReconfFailure(tvb, offset, actx, tree, hf_nbap_generalCause4);
}


static const per_sequence_t RL_ReconfigurationFailureList_RL_ReconfFailure_sequence_of[1] = {
  { &hf_nbap_RL_ReconfigurationFailureList_RL_ReconfFailure_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_RL_ReconfigurationFailureList_RL_ReconfFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_RL_ReconfigurationFailureList_RL_ReconfFailure, RL_ReconfigurationFailureList_RL_ReconfFailure_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_rL_ReconfigurationFailureList_RL_ReconfFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_ReconfigurationFailureList_RL_ReconfFailure(tvb, offset, actx, tree, hf_nbap_rL_ReconfigurationFailureList_RL_ReconfFailure);
}


static const per_sequence_t RLSpecificCauseList_RL_ReconfFailure_sequence[] = {
  { &hf_nbap_rL_ReconfigurationFailureList_RL_ReconfFailure, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_RL_ReconfigurationFailureList_RL_ReconfFailure },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RLSpecificCauseList_RL_ReconfFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RLSpecificCauseList_RL_ReconfFailure, RLSpecificCauseList_RL_ReconfFailure_sequence);

  return offset;
}
static int dissect_rLSpecificCause4(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RLSpecificCauseList_RL_ReconfFailure(tvb, offset, actx, tree, hf_nbap_rLSpecificCause4);
}


static const value_string nbap_CauseLevel_RL_ReconfFailure_vals[] = {
  {   0, "generalCause" },
  {   1, "rLSpecificCause" },
  { 0, NULL }
};

static const per_choice_t CauseLevel_RL_ReconfFailure_choice[] = {
  {   0, &hf_nbap_generalCause4  , ASN1_EXTENSION_ROOT    , dissect_nbap_GeneralCauseList_RL_ReconfFailure },
  {   1, &hf_nbap_rLSpecificCause4, ASN1_EXTENSION_ROOT    , dissect_nbap_RLSpecificCauseList_RL_ReconfFailure },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_CauseLevel_RL_ReconfFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_CauseLevel_RL_ReconfFailure, CauseLevel_RL_ReconfFailure_choice,
                                 NULL);

  return offset;
}
static int dissect_id_CauseLevel_RL_ReconfFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CauseLevel_RL_ReconfFailure(tvb, offset, actx, tree, hf_nbap_id_CauseLevel_RL_ReconfFailure);
}


static const per_sequence_t GeneralCauseList_RL_SetupFailureFDD_sequence[] = {
  { &hf_nbap_cause          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Cause },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_GeneralCauseList_RL_SetupFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_GeneralCauseList_RL_SetupFailureFDD, GeneralCauseList_RL_SetupFailureFDD_sequence);

  return offset;
}
static int dissect_generalCause(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_GeneralCauseList_RL_SetupFailureFDD(tvb, offset, actx, tree, hf_nbap_generalCause);
}


static const per_sequence_t Unsuccessful_RL_InformationRespList_RL_SetupFailureFDD_sequence_of[1] = {
  { &hf_nbap_Unsuccessful_RL_InformationRespList_RL_SetupFailureFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_Unsuccessful_RL_InformationRespList_RL_SetupFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_Unsuccessful_RL_InformationRespList_RL_SetupFailureFDD, Unsuccessful_RL_InformationRespList_RL_SetupFailureFDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_unsuccessful_RL_InformationRespList_RL_SetupFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Unsuccessful_RL_InformationRespList_RL_SetupFailureFDD(tvb, offset, actx, tree, hf_nbap_unsuccessful_RL_InformationRespList_RL_SetupFailureFDD);
}


static const per_sequence_t Successful_RL_InformationRespList_RL_SetupFailureFDD_sequence_of[1] = {
  { &hf_nbap_Successful_RL_InformationRespList_RL_SetupFailureFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_Successful_RL_InformationRespList_RL_SetupFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_Successful_RL_InformationRespList_RL_SetupFailureFDD, Successful_RL_InformationRespList_RL_SetupFailureFDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_successful_RL_InformationRespList_RL_SetupFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Successful_RL_InformationRespList_RL_SetupFailureFDD(tvb, offset, actx, tree, hf_nbap_successful_RL_InformationRespList_RL_SetupFailureFDD);
}


static const per_sequence_t RLSpecificCauseList_RL_SetupFailureFDD_sequence[] = {
  { &hf_nbap_unsuccessful_RL_InformationRespList_RL_SetupFailureFDD, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Unsuccessful_RL_InformationRespList_RL_SetupFailureFDD },
  { &hf_nbap_successful_RL_InformationRespList_RL_SetupFailureFDD, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_Successful_RL_InformationRespList_RL_SetupFailureFDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RLSpecificCauseList_RL_SetupFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RLSpecificCauseList_RL_SetupFailureFDD, RLSpecificCauseList_RL_SetupFailureFDD_sequence);

  return offset;
}
static int dissect_rLSpecificCause(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RLSpecificCauseList_RL_SetupFailureFDD(tvb, offset, actx, tree, hf_nbap_rLSpecificCause);
}


static const value_string nbap_CauseLevel_RL_SetupFailureFDD_vals[] = {
  {   0, "generalCause" },
  {   1, "rLSpecificCause" },
  { 0, NULL }
};

static const per_choice_t CauseLevel_RL_SetupFailureFDD_choice[] = {
  {   0, &hf_nbap_generalCause   , ASN1_EXTENSION_ROOT    , dissect_nbap_GeneralCauseList_RL_SetupFailureFDD },
  {   1, &hf_nbap_rLSpecificCause, ASN1_EXTENSION_ROOT    , dissect_nbap_RLSpecificCauseList_RL_SetupFailureFDD },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_CauseLevel_RL_SetupFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_CauseLevel_RL_SetupFailureFDD, CauseLevel_RL_SetupFailureFDD_choice,
                                 NULL);

  return offset;
}
static int dissect_id_CauseLevel_RL_SetupFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CauseLevel_RL_SetupFailureFDD(tvb, offset, actx, tree, hf_nbap_id_CauseLevel_RL_SetupFailureFDD);
}


static const per_sequence_t GeneralCauseList_RL_SetupFailureTDD_sequence[] = {
  { &hf_nbap_cause          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Cause },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_GeneralCauseList_RL_SetupFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_GeneralCauseList_RL_SetupFailureTDD, GeneralCauseList_RL_SetupFailureTDD_sequence);

  return offset;
}
static int dissect_generalCause1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_GeneralCauseList_RL_SetupFailureTDD(tvb, offset, actx, tree, hf_nbap_generalCause1);
}



static int
dissect_nbap_Unsuccessful_RL_InformationRespItem_RL_SetupFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_unsuccessful_RL_InformationRespItem_RL_SetupFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Unsuccessful_RL_InformationRespItem_RL_SetupFailureTDD(tvb, offset, actx, tree, hf_nbap_unsuccessful_RL_InformationRespItem_RL_SetupFailureTDD);
}


static const per_sequence_t RLSpecificCauseList_RL_SetupFailureTDD_sequence[] = {
  { &hf_nbap_unsuccessful_RL_InformationRespItem_RL_SetupFailureTDD, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Unsuccessful_RL_InformationRespItem_RL_SetupFailureTDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RLSpecificCauseList_RL_SetupFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RLSpecificCauseList_RL_SetupFailureTDD, RLSpecificCauseList_RL_SetupFailureTDD_sequence);

  return offset;
}
static int dissect_rLSpecificCause1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RLSpecificCauseList_RL_SetupFailureTDD(tvb, offset, actx, tree, hf_nbap_rLSpecificCause1);
}


static const value_string nbap_CauseLevel_RL_SetupFailureTDD_vals[] = {
  {   0, "generalCause" },
  {   1, "rLSpecificCause" },
  { 0, NULL }
};

static const per_choice_t CauseLevel_RL_SetupFailureTDD_choice[] = {
  {   0, &hf_nbap_generalCause1  , ASN1_EXTENSION_ROOT    , dissect_nbap_GeneralCauseList_RL_SetupFailureTDD },
  {   1, &hf_nbap_rLSpecificCause1, ASN1_EXTENSION_ROOT    , dissect_nbap_RLSpecificCauseList_RL_SetupFailureTDD },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_CauseLevel_RL_SetupFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_CauseLevel_RL_SetupFailureTDD, CauseLevel_RL_SetupFailureTDD_choice,
                                 NULL);

  return offset;
}
static int dissect_id_CauseLevel_RL_SetupFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CauseLevel_RL_SetupFailureTDD(tvb, offset, actx, tree, hf_nbap_id_CauseLevel_RL_SetupFailureTDD);
}


static const value_string nbap_Closedlooptimingadjustmentmode_vals[] = {
  {   0, "adj-1-slot" },
  {   1, "adj-2-slot" },
  { 0, NULL }
};


static int
dissect_nbap_Closedlooptimingadjustmentmode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_id_Closed_Loop_Timing_Adjustment_Mode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Closedlooptimingadjustmentmode(tvb, offset, actx, tree, hf_nbap_id_Closed_Loop_Timing_Adjustment_Mode);
}



static int
dissect_nbap_FACH_ParametersList_CTCH_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_fACH_ParametersList_CTCH_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_FACH_ParametersList_CTCH_ReconfRqstFDD(tvb, offset, actx, tree, hf_nbap_fACH_ParametersList_CTCH_ReconfRqstFDD);
}



static int
dissect_nbap_PCH_Parameters_CTCH_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_pCH_Parameters_CTCH_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PCH_Parameters_CTCH_ReconfRqstFDD(tvb, offset, actx, tree, hf_nbap_pCH_Parameters_CTCH_ReconfRqstFDD);
}



static int
dissect_nbap_PICH_Parameters_CTCH_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_pICH_Parameters_CTCH_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PICH_Parameters_CTCH_ReconfRqstFDD(tvb, offset, actx, tree, hf_nbap_pICH_Parameters_CTCH_ReconfRqstFDD);
}


static const per_sequence_t Secondary_CCPCHList_CTCH_ReconfRqstFDD_sequence[] = {
  { &hf_nbap_fACH_ParametersList_CTCH_ReconfRqstFDD, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_FACH_ParametersList_CTCH_ReconfRqstFDD },
  { &hf_nbap_pCH_Parameters_CTCH_ReconfRqstFDD, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_PCH_Parameters_CTCH_ReconfRqstFDD },
  { &hf_nbap_pICH_Parameters_CTCH_ReconfRqstFDD, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_PICH_Parameters_CTCH_ReconfRqstFDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Secondary_CCPCHList_CTCH_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Secondary_CCPCHList_CTCH_ReconfRqstFDD, Secondary_CCPCHList_CTCH_ReconfRqstFDD_sequence);

  return offset;
}
static int dissect_secondary_CCPCH_parameters2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Secondary_CCPCHList_CTCH_ReconfRqstFDD(tvb, offset, actx, tree, hf_nbap_secondary_CCPCH_parameters2);
}



static int
dissect_nbap_PRACH_ParametersList_CTCH_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_pRACH_ParametersList_CTCH_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PRACH_ParametersList_CTCH_ReconfRqstFDD(tvb, offset, actx, tree, hf_nbap_pRACH_ParametersList_CTCH_ReconfRqstFDD);
}



static int
dissect_nbap_AICH_ParametersList_CTCH_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_aICH_ParametersList_CTCH_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_AICH_ParametersList_CTCH_ReconfRqstFDD(tvb, offset, actx, tree, hf_nbap_aICH_ParametersList_CTCH_ReconfRqstFDD);
}


static const per_sequence_t PRACHList_CTCH_ReconfRqstFDD_sequence[] = {
  { &hf_nbap_pRACH_ParametersList_CTCH_ReconfRqstFDD, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_PRACH_ParametersList_CTCH_ReconfRqstFDD },
  { &hf_nbap_aICH_ParametersList_CTCH_ReconfRqstFDD, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_AICH_ParametersList_CTCH_ReconfRqstFDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PRACHList_CTCH_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PRACHList_CTCH_ReconfRqstFDD, PRACHList_CTCH_ReconfRqstFDD_sequence);

  return offset;
}
static int dissect_pRACH_parameters2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PRACHList_CTCH_ReconfRqstFDD(tvb, offset, actx, tree, hf_nbap_pRACH_parameters2);
}


static const value_string nbap_CommonPhysicalChannelType_CTCH_ReconfRqstFDD_vals[] = {
  {   0, "secondary-CCPCH-parameters" },
  {   1, "pRACH-parameters" },
  {   2, "notUsed-cPCH-parameters" },
  { 0, NULL }
};

static const per_choice_t CommonPhysicalChannelType_CTCH_ReconfRqstFDD_choice[] = {
  {   0, &hf_nbap_secondary_CCPCH_parameters2, ASN1_EXTENSION_ROOT    , dissect_nbap_Secondary_CCPCHList_CTCH_ReconfRqstFDD },
  {   1, &hf_nbap_pRACH_parameters2, ASN1_EXTENSION_ROOT    , dissect_nbap_PRACHList_CTCH_ReconfRqstFDD },
  {   2, &hf_nbap_notUsed_cPCH_parameters, ASN1_EXTENSION_ROOT    , dissect_nbap_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_CommonPhysicalChannelType_CTCH_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_CommonPhysicalChannelType_CTCH_ReconfRqstFDD, CommonPhysicalChannelType_CTCH_ReconfRqstFDD_choice,
                                 NULL);

  return offset;
}
static int dissect_id_CommonPhysicalChannelType_CTCH_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommonPhysicalChannelType_CTCH_ReconfRqstFDD(tvb, offset, actx, tree, hf_nbap_id_CommonPhysicalChannelType_CTCH_ReconfRqstFDD);
}


static const value_string nbap_Compressed_Mode_Deactivation_Flag_vals[] = {
  {   0, "deactivate" },
  {   1, "maintain-Active" },
  { 0, NULL }
};


static int
dissect_nbap_Compressed_Mode_Deactivation_Flag(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_Compressed_Mode_Deactivation_Flag(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Compressed_Mode_Deactivation_Flag(tvb, offset, actx, tree, hf_nbap_id_Compressed_Mode_Deactivation_Flag);
}


static const per_sequence_t CCTrCH_TPCAddItem_RL_ReconfPrepTDD_sequence[] = {
  { &hf_nbap_cCTrCH_ID      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CCTrCH_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CCTrCH_TPCAddItem_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CCTrCH_TPCAddItem_RL_ReconfPrepTDD, CCTrCH_TPCAddItem_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_CCTrCH_TPCAddList_RL_ReconfPrepTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CCTrCH_TPCAddItem_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_CCTrCH_TPCAddList_RL_ReconfPrepTDD_item);
}


static const per_sequence_t CCTrCH_TPCAddList_RL_ReconfPrepTDD_sequence_of[1] = {
  { &hf_nbap_CCTrCH_TPCAddList_RL_ReconfPrepTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_CCTrCH_TPCAddItem_RL_ReconfPrepTDD },
};

static int
dissect_nbap_CCTrCH_TPCAddList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_CCTrCH_TPCAddList_RL_ReconfPrepTDD, CCTrCH_TPCAddList_RL_ReconfPrepTDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_cCTrCH_TPCList1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CCTrCH_TPCAddList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_cCTrCH_TPCList1);
}



static int
dissect_nbap_DL_DPCH_InformationAddList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_dl_DPCH_InformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_DPCH_InformationAddList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_dl_DPCH_InformationList);
}


static const per_sequence_t DL_CCTrCH_InformationAddItem_RL_ReconfPrepTDD_sequence[] = {
  { &hf_nbap_cCTrCH_ID      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CCTrCH_ID },
  { &hf_nbap_tFCS           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TFCS },
  { &hf_nbap_tFCI_Coding    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TFCI_Coding },
  { &hf_nbap_punctureLimit  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PunctureLimit },
  { &hf_nbap_cCTrCH_TPCList1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_CCTrCH_TPCAddList_RL_ReconfPrepTDD },
  { &hf_nbap_dl_DPCH_InformationList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_DPCH_InformationAddList_RL_ReconfPrepTDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_CCTrCH_InformationAddItem_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_CCTrCH_InformationAddItem_RL_ReconfPrepTDD, DL_CCTrCH_InformationAddItem_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_CCTrCH_InformationAddItem_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD_item);
}


static const per_sequence_t DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD_sequence_of[1] = {
  { &hf_nbap_DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DL_CCTrCH_InformationAddItem_RL_ReconfPrepTDD },
};

static int
dissect_nbap_DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD, DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_id_DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD);
}


static const per_sequence_t DL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD_sequence[] = {
  { &hf_nbap_cCTrCH_ID      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CCTrCH_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD, DL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD_sequence);

  return offset;
}
static int dissect_id_DL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_id_DL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD);
}


static const per_sequence_t DL_CCTrCH_InformationDeleteItem_RL_ReconfPrepTDD_sequence[] = {
  { &hf_nbap_cCTrCH_ID      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CCTrCH_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_CCTrCH_InformationDeleteItem_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_CCTrCH_InformationDeleteItem_RL_ReconfPrepTDD, DL_CCTrCH_InformationDeleteItem_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_CCTrCH_InformationDeleteItem_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD_item);
}


static const per_sequence_t DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD_sequence_of[1] = {
  { &hf_nbap_DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DL_CCTrCH_InformationDeleteItem_RL_ReconfPrepTDD },
};

static int
dissect_nbap_DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD, DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_id_DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD);
}


static const per_sequence_t DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD_sequence_of[1] = {
  { &hf_nbap_DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD, DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_id_DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD);
}


static const per_sequence_t DL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD_sequence[] = {
  { &hf_nbap_cCTrCH_ID      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CCTrCH_ID },
  { &hf_nbap_tFCS           , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TFCS },
  { &hf_nbap_punctureLimit  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_PunctureLimit },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD, DL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD_sequence);

  return offset;
}
static int dissect_id_DL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_id_DL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD);
}


static const per_sequence_t CCTrCH_TPCModifyItem_RL_ReconfPrepTDD_sequence[] = {
  { &hf_nbap_cCTrCH_ID      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CCTrCH_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CCTrCH_TPCModifyItem_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CCTrCH_TPCModifyItem_RL_ReconfPrepTDD, CCTrCH_TPCModifyItem_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_CCTrCH_TPCModifyList_RL_ReconfPrepTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CCTrCH_TPCModifyItem_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_CCTrCH_TPCModifyList_RL_ReconfPrepTDD_item);
}


static const per_sequence_t CCTrCH_TPCModifyList_RL_ReconfPrepTDD_sequence_of[1] = {
  { &hf_nbap_CCTrCH_TPCModifyList_RL_ReconfPrepTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_CCTrCH_TPCModifyItem_RL_ReconfPrepTDD },
};

static int
dissect_nbap_CCTrCH_TPCModifyList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_CCTrCH_TPCModifyList_RL_ReconfPrepTDD, CCTrCH_TPCModifyList_RL_ReconfPrepTDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_cCTrCH_TPCList2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CCTrCH_TPCModifyList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_cCTrCH_TPCList2);
}



static int
dissect_nbap_DL_DPCH_InformationModify_AddList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_dl_DPCH_InformationAddList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_DPCH_InformationModify_AddList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_dl_DPCH_InformationAddList);
}



static int
dissect_nbap_DL_DPCH_InformationModify_ModifyList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_dl_DPCH_InformationModifyList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_DPCH_InformationModify_ModifyList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_dl_DPCH_InformationModifyList);
}



static int
dissect_nbap_DL_DPCH_InformationModify_DeleteList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_dl_DPCH_InformationDeleteList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_DPCH_InformationModify_DeleteList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_dl_DPCH_InformationDeleteList);
}


static const per_sequence_t DL_CCTrCH_InformationModifyItem_RL_ReconfPrepTDD_sequence[] = {
  { &hf_nbap_cCTrCH_ID      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CCTrCH_ID },
  { &hf_nbap_tFCS           , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TFCS },
  { &hf_nbap_tFCI_Coding    , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TFCI_Coding },
  { &hf_nbap_punctureLimit  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_PunctureLimit },
  { &hf_nbap_cCTrCH_TPCList2, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_CCTrCH_TPCModifyList_RL_ReconfPrepTDD },
  { &hf_nbap_dl_DPCH_InformationAddList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_DPCH_InformationModify_AddList_RL_ReconfPrepTDD },
  { &hf_nbap_dl_DPCH_InformationModifyList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_DPCH_InformationModify_ModifyList_RL_ReconfPrepTDD },
  { &hf_nbap_dl_DPCH_InformationDeleteList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_DPCH_InformationModify_DeleteList_RL_ReconfPrepTDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_CCTrCH_InformationModifyItem_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_CCTrCH_InformationModifyItem_RL_ReconfPrepTDD, DL_CCTrCH_InformationModifyItem_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_CCTrCH_InformationModifyItem_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD_item);
}


static const per_sequence_t DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD_sequence_of[1] = {
  { &hf_nbap_DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DL_CCTrCH_InformationModifyItem_RL_ReconfPrepTDD },
};

static int
dissect_nbap_DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD, DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_id_DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD);
}


static const per_sequence_t DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD_sequence_of[1] = {
  { &hf_nbap_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD, DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_id_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD);
}


static const per_sequence_t DL_DPCH_InformationAddItem_RL_ReconfPrepTDD_sequence[] = {
  { &hf_nbap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionPeriod },
  { &hf_nbap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionLength },
  { &hf_nbap_tdd_DPCHOffset , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_DPCHOffset },
  { &hf_nbap_dL_Timeslot_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Timeslot_Information },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_DPCH_InformationAddItem_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_DPCH_InformationAddItem_RL_ReconfPrepTDD, DL_DPCH_InformationAddItem_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_id_DL_DPCH_InformationAddListIE_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_DPCH_InformationAddItem_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_id_DL_DPCH_InformationAddListIE_RL_ReconfPrepTDD);
}


static const per_sequence_t DL_DPCH_InformationModify_AddItem_RL_ReconfPrepTDD_sequence[] = {
  { &hf_nbap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionPeriod },
  { &hf_nbap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionLength },
  { &hf_nbap_tdd_DPCHOffset , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_DPCHOffset },
  { &hf_nbap_dL_Timeslot_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Timeslot_Information },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_DPCH_InformationModify_AddItem_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_DPCH_InformationModify_AddItem_RL_ReconfPrepTDD, DL_DPCH_InformationModify_AddItem_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_id_DL_DPCH_InformationModify_AddListIE_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_DPCH_InformationModify_AddItem_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_id_DL_DPCH_InformationModify_AddListIE_RL_ReconfPrepTDD);
}


static const per_sequence_t DL_DPCH_InformationModify_DeleteItem_RL_ReconfPrepTDD_sequence[] = {
  { &hf_nbap_dPCH_ID        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DPCH_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_DPCH_InformationModify_DeleteItem_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_DPCH_InformationModify_DeleteItem_RL_ReconfPrepTDD, DL_DPCH_InformationModify_DeleteItem_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_id_DL_DPCH_InformationModify_DeleteListIE_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_DPCH_InformationModify_DeleteItem_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_id_DL_DPCH_InformationModify_DeleteListIE_RL_ReconfPrepTDD);
}
static int dissect_DL_DPCH_InformationModify_DeleteListIE_RL_ReconfPrepTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_DPCH_InformationModify_DeleteItem_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_DL_DPCH_InformationModify_DeleteListIE_RL_ReconfPrepTDD_item);
}


static const per_sequence_t DL_Code_InformationModify_ModifyItem_RL_ReconfPrepTDD_sequence[] = {
  { &hf_nbap_dPCH_ID        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DPCH_ID },
  { &hf_nbap_tdd_ChannelisationCode, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TDD_ChannelisationCode },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_Code_InformationModify_ModifyItem_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_Code_InformationModify_ModifyItem_RL_ReconfPrepTDD, DL_Code_InformationModify_ModifyItem_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_DL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Code_InformationModify_ModifyItem_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_DL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD_item);
}


static const per_sequence_t DL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD_sequence_of[1] = {
  { &hf_nbap_DL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Code_InformationModify_ModifyItem_RL_ReconfPrepTDD },
};

static int
dissect_nbap_DL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD, DL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD_sequence_of,
                                                  0, 240);

  return offset;
}
static int dissect_dL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_dL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD);
}


static const per_sequence_t DL_Timeslot_InformationModify_ModifyItem_RL_ReconfPrepTDD_sequence[] = {
  { &hf_nbap_timeSlot       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlot },
  { &hf_nbap_midambleShiftAndBurstType, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_MidambleShiftAndBurstType },
  { &hf_nbap_tFCI_Presence  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TFCI_Presence },
  { &hf_nbap_dL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_Timeslot_InformationModify_ModifyItem_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_Timeslot_InformationModify_ModifyItem_RL_ReconfPrepTDD, DL_Timeslot_InformationModify_ModifyItem_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_DL_Timeslot_InformationModify_ModifyList_RL_ReconfPrepTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Timeslot_InformationModify_ModifyItem_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_DL_Timeslot_InformationModify_ModifyList_RL_ReconfPrepTDD_item);
}


static const per_sequence_t DL_Timeslot_InformationModify_ModifyList_RL_ReconfPrepTDD_sequence_of[1] = {
  { &hf_nbap_DL_Timeslot_InformationModify_ModifyList_RL_ReconfPrepTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Timeslot_InformationModify_ModifyItem_RL_ReconfPrepTDD },
};

static int
dissect_nbap_DL_Timeslot_InformationModify_ModifyList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DL_Timeslot_InformationModify_ModifyList_RL_ReconfPrepTDD, DL_Timeslot_InformationModify_ModifyList_RL_ReconfPrepTDD_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_dL_Timeslot_InformationAddModify_ModifyList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Timeslot_InformationModify_ModifyList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_dL_Timeslot_InformationAddModify_ModifyList_RL_ReconfPrepTDD);
}


static const per_sequence_t DL_DPCH_InformationModify_ModifyItem_RL_ReconfPrepTDD_sequence[] = {
  { &hf_nbap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_RepetitionPeriod },
  { &hf_nbap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_RepetitionLength },
  { &hf_nbap_tdd_DPCHOffset , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TDD_DPCHOffset },
  { &hf_nbap_dL_Timeslot_InformationAddModify_ModifyList_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_Timeslot_InformationModify_ModifyList_RL_ReconfPrepTDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_DPCH_InformationModify_ModifyItem_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_DPCH_InformationModify_ModifyItem_RL_ReconfPrepTDD, DL_DPCH_InformationModify_ModifyItem_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_id_DL_DPCH_InformationModify_ModifyListIE_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_DPCH_InformationModify_ModifyItem_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_id_DL_DPCH_InformationModify_ModifyListIE_RL_ReconfPrepTDD);
}



static int
dissect_nbap_DL_TPC_Pattern01Count(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 30U, NULL, TRUE);

  return offset;
}
static int dissect_id_DL_TPC_Pattern01Count(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_TPC_Pattern01Count(tvb, offset, actx, tree, hf_nbap_id_DL_TPC_Pattern01Count);
}


static const value_string nbap_DPC_Mode_vals[] = {
  {   0, "mode0" },
  {   1, "mode1" },
  { 0, NULL }
};


static int
dissect_nbap_DPC_Mode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_id_DPC_Mode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DPC_Mode(tvb, offset, actx, tree, hf_nbap_id_DPC_Mode);
}



static int
dissect_nbap_ConstantValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              -10, 10U, NULL, TRUE);

  return offset;
}
static int dissect_id_DPCHConstant(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ConstantValue(tvb, offset, actx, tree, hf_nbap_id_DPCHConstant);
}
static int dissect_id_PRACHConstant(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ConstantValue(tvb, offset, actx, tree, hf_nbap_id_PRACHConstant);
}
static int dissect_id_PUSCHConstant(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ConstantValue(tvb, offset, actx, tree, hf_nbap_id_PUSCHConstant);
}


static const per_sequence_t CommonTransportChannel_InformationResponse_sequence[] = {
  { &hf_nbap_commonTransportChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonTransportChannelID },
  { &hf_nbap_bindingID      , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_BindingID },
  { &hf_nbap_transportLayerAddress, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TransportLayerAddress },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CommonTransportChannel_InformationResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CommonTransportChannel_InformationResponse, CommonTransportChannel_InformationResponse_sequence);

  return offset;
}
static int dissect_id_PCH_Parameters_CTCH_SetupRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommonTransportChannel_InformationResponse(tvb, offset, actx, tree, hf_nbap_id_PCH_Parameters_CTCH_SetupRsp);
}
static int dissect_id_RACH_Parameters_CTCH_SetupRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommonTransportChannel_InformationResponse(tvb, offset, actx, tree, hf_nbap_id_RACH_Parameters_CTCH_SetupRsp);
}
static int dissect_FACH_CommonTransportChannel_InformationResponse_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommonTransportChannel_InformationResponse(tvb, offset, actx, tree, hf_nbap_FACH_CommonTransportChannel_InformationResponse_item);
}


static const per_sequence_t FACH_CommonTransportChannel_InformationResponse_sequence_of[1] = {
  { &hf_nbap_FACH_CommonTransportChannel_InformationResponse_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_CommonTransportChannel_InformationResponse },
};

static int
dissect_nbap_FACH_CommonTransportChannel_InformationResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_FACH_CommonTransportChannel_InformationResponse, FACH_CommonTransportChannel_InformationResponse_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_id_FACH_ParametersList_CTCH_SetupRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_FACH_CommonTransportChannel_InformationResponse(tvb, offset, actx, tree, hf_nbap_id_FACH_ParametersList_CTCH_SetupRsp);
}



static int
dissect_nbap_PowerRaiseLimit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 10U, NULL, FALSE);

  return offset;
}
static int dissect_powerRaiseLimit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PowerRaiseLimit(tvb, offset, actx, tree, hf_nbap_powerRaiseLimit);
}



static int
dissect_nbap_DLPowerAveragingWindowSize(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 60U, NULL, FALSE);

  return offset;
}
static int dissect_dLPowerAveragingWindowSize(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DLPowerAveragingWindowSize(tvb, offset, actx, tree, hf_nbap_dLPowerAveragingWindowSize);
}


static const per_sequence_t Limited_power_increase_information_Cell_SetupRqstFDD_sequence[] = {
  { &hf_nbap_powerRaiseLimit, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PowerRaiseLimit },
  { &hf_nbap_dLPowerAveragingWindowSize, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DLPowerAveragingWindowSize },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Limited_power_increase_information_Cell_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Limited_power_increase_information_Cell_SetupRqstFDD, Limited_power_increase_information_Cell_SetupRqstFDD_sequence);

  return offset;
}
static int dissect_id_Limited_power_increase_information_Cell_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Limited_power_increase_information_Cell_SetupRqstFDD(tvb, offset, actx, tree, hf_nbap_id_Limited_power_increase_information_Cell_SetupRqstFDD);
}


static const per_sequence_t PCH_ParametersItem_CTCH_ReconfRqstFDD_sequence[] = {
  { &hf_nbap_commonTransportChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonTransportChannelID },
  { &hf_nbap_pCH_Power      , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_Power },
  { &hf_nbap_toAWS          , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ToAWS },
  { &hf_nbap_toAWE          , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ToAWE },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PCH_ParametersItem_CTCH_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PCH_ParametersItem_CTCH_ReconfRqstFDD, PCH_ParametersItem_CTCH_ReconfRqstFDD_sequence);

  return offset;
}
static int dissect_id_PCH_ParametersItem_CTCH_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PCH_ParametersItem_CTCH_ReconfRqstFDD(tvb, offset, actx, tree, hf_nbap_id_PCH_ParametersItem_CTCH_ReconfRqstFDD);
}


static const per_sequence_t PICH_ParametersItem_CTCH_ReconfRqstFDD_sequence[] = {
  { &hf_nbap_commonPhysicalChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonPhysicalChannelID },
  { &hf_nbap_pICH_Power     , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_PICH_Power },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PICH_ParametersItem_CTCH_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PICH_ParametersItem_CTCH_ReconfRqstFDD, PICH_ParametersItem_CTCH_ReconfRqstFDD_sequence);

  return offset;
}
static int dissect_id_PICH_ParametersItem_CTCH_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PICH_ParametersItem_CTCH_ReconfRqstFDD(tvb, offset, actx, tree, hf_nbap_id_PICH_ParametersItem_CTCH_ReconfRqstFDD);
}


static const per_sequence_t AllowedSlotFormatInformationItem_CTCH_ReconfRqstFDD_sequence[] = {
  { &hf_nbap_rACH_SlotFormat, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RACH_SlotFormat },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_AllowedSlotFormatInformationItem_CTCH_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_AllowedSlotFormatInformationItem_CTCH_ReconfRqstFDD, AllowedSlotFormatInformationItem_CTCH_ReconfRqstFDD_sequence);

  return offset;
}
static int dissect_AllowedSlotFormatInformationList_CTCH_ReconfRqstFDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_AllowedSlotFormatInformationItem_CTCH_ReconfRqstFDD(tvb, offset, actx, tree, hf_nbap_AllowedSlotFormatInformationList_CTCH_ReconfRqstFDD_item);
}


static const per_sequence_t AllowedSlotFormatInformationList_CTCH_ReconfRqstFDD_sequence_of[1] = {
  { &hf_nbap_AllowedSlotFormatInformationList_CTCH_ReconfRqstFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_AllowedSlotFormatInformationItem_CTCH_ReconfRqstFDD },
};

static int
dissect_nbap_AllowedSlotFormatInformationList_CTCH_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_AllowedSlotFormatInformationList_CTCH_ReconfRqstFDD, AllowedSlotFormatInformationList_CTCH_ReconfRqstFDD_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_allowedSlotFormatInformation1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_AllowedSlotFormatInformationList_CTCH_ReconfRqstFDD(tvb, offset, actx, tree, hf_nbap_allowedSlotFormatInformation1);
}


static const per_sequence_t PRACH_ParametersItem_CTCH_ReconfRqstFDD_sequence[] = {
  { &hf_nbap_commonPhysicalChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonPhysicalChannelID },
  { &hf_nbap_preambleSignatures, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_PreambleSignatures },
  { &hf_nbap_allowedSlotFormatInformation1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_AllowedSlotFormatInformationList_CTCH_ReconfRqstFDD },
  { &hf_nbap_rACH_SubChannelNumbers, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_RACH_SubChannelNumbers },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PRACH_ParametersItem_CTCH_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PRACH_ParametersItem_CTCH_ReconfRqstFDD, PRACH_ParametersItem_CTCH_ReconfRqstFDD_sequence);

  return offset;
}
static int dissect_PRACH_ParametersListIE_CTCH_ReconfRqstFDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PRACH_ParametersItem_CTCH_ReconfRqstFDD(tvb, offset, actx, tree, hf_nbap_PRACH_ParametersListIE_CTCH_ReconfRqstFDD_item);
}


static const per_sequence_t PRACH_ParametersListIE_CTCH_ReconfRqstFDD_sequence_of[1] = {
  { &hf_nbap_PRACH_ParametersListIE_CTCH_ReconfRqstFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_PRACH_ParametersItem_CTCH_ReconfRqstFDD },
};

static int
dissect_nbap_PRACH_ParametersListIE_CTCH_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_PRACH_ParametersListIE_CTCH_ReconfRqstFDD, PRACH_ParametersListIE_CTCH_ReconfRqstFDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_PRACH_ParametersListIE_CTCH_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PRACH_ParametersListIE_CTCH_ReconfRqstFDD(tvb, offset, actx, tree, hf_nbap_id_PRACH_ParametersListIE_CTCH_ReconfRqstFDD);
}



static int
dissect_nbap_N_INSYNC_IND(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 256U, NULL, FALSE);

  return offset;
}
static int dissect_n_INSYNC_IND(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_N_INSYNC_IND(tvb, offset, actx, tree, hf_nbap_n_INSYNC_IND);
}



static int
dissect_nbap_N_OUTSYNC_IND(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 256U, NULL, FALSE);

  return offset;
}
static int dissect_n_OUTSYNC_IND(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_N_OUTSYNC_IND(tvb, offset, actx, tree, hf_nbap_n_OUTSYNC_IND);
}



static int
dissect_nbap_T_RLFAILURE(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 255U, NULL, FALSE);

  return offset;
}
static int dissect_t_RLFAILURE(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_T_RLFAILURE(tvb, offset, actx, tree, hf_nbap_t_RLFAILURE);
}


static const per_sequence_t Synchronisation_Configuration_Cell_ReconfRqst_sequence[] = {
  { &hf_nbap_n_INSYNC_IND   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_N_INSYNC_IND },
  { &hf_nbap_n_OUTSYNC_IND  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_N_OUTSYNC_IND },
  { &hf_nbap_t_RLFAILURE    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_T_RLFAILURE },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Synchronisation_Configuration_Cell_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Synchronisation_Configuration_Cell_ReconfRqst, Synchronisation_Configuration_Cell_ReconfRqst_sequence);

  return offset;
}
static int dissect_id_Synchronisation_Configuration_Cell_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Synchronisation_Configuration_Cell_ReconfRqst(tvb, offset, actx, tree, hf_nbap_id_Synchronisation_Configuration_Cell_ReconfRqst);
}


static const per_sequence_t Synchronisation_Configuration_Cell_SetupRqst_sequence[] = {
  { &hf_nbap_n_INSYNC_IND   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_N_INSYNC_IND },
  { &hf_nbap_n_OUTSYNC_IND  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_N_OUTSYNC_IND },
  { &hf_nbap_t_RLFAILURE    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_T_RLFAILURE },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Synchronisation_Configuration_Cell_SetupRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Synchronisation_Configuration_Cell_SetupRqst, Synchronisation_Configuration_Cell_SetupRqst_sequence);

  return offset;
}
static int dissect_id_Synchronisation_Configuration_Cell_SetupRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Synchronisation_Configuration_Cell_SetupRqst(tvb, offset, actx, tree, hf_nbap_id_Synchronisation_Configuration_Cell_SetupRqst);
}



static int
dissect_nbap_TGSN(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 14U, NULL, FALSE);

  return offset;
}
static int dissect_tGSN(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TGSN(tvb, offset, actx, tree, hf_nbap_tGSN);
}



static int
dissect_nbap_GapLength(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 14U, NULL, FALSE);

  return offset;
}
static int dissect_tGL1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_GapLength(tvb, offset, actx, tree, hf_nbap_tGL1);
}
static int dissect_tGL2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_GapLength(tvb, offset, actx, tree, hf_nbap_tGL2);
}



static int
dissect_nbap_TGD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              15U, 269U, NULL, FALSE);

  return offset;
}
static int dissect_tGD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TGD(tvb, offset, actx, tree, hf_nbap_tGD);
}



static int
dissect_nbap_GapDuration(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 144U, NULL, TRUE);

  return offset;
}
static int dissect_tGPL1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_GapDuration(tvb, offset, actx, tree, hf_nbap_tGPL1);
}
static int dissect_not_to_be_used_1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_GapDuration(tvb, offset, actx, tree, hf_nbap_not_to_be_used_1);
}


static const value_string nbap_UL_DL_mode_vals[] = {
  {   0, "ul-only" },
  {   1, "dl-only" },
  {   2, "both-ul-and-dl" },
  { 0, NULL }
};


static int
dissect_nbap_UL_DL_mode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_uL_DL_mode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_DL_mode(tvb, offset, actx, tree, hf_nbap_uL_DL_mode);
}


static const value_string nbap_Downlink_Compressed_Mode_Method_vals[] = {
  {   0, "not-Used-puncturing" },
  {   1, "sFdiv2" },
  {   2, "higher-layer-scheduling" },
  { 0, NULL }
};


static int
dissect_nbap_Downlink_Compressed_Mode_Method(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_downlink_Compressed_Mode_Method(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Downlink_Compressed_Mode_Method(tvb, offset, actx, tree, hf_nbap_downlink_Compressed_Mode_Method);
}


static const value_string nbap_Uplink_Compressed_Mode_Method_vals[] = {
  {   0, "sFdiv2" },
  {   1, "higher-layer-scheduling" },
  { 0, NULL }
};


static int
dissect_nbap_Uplink_Compressed_Mode_Method(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_uplink_Compressed_Mode_Method(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Uplink_Compressed_Mode_Method(tvb, offset, actx, tree, hf_nbap_uplink_Compressed_Mode_Method);
}


static const value_string nbap_DL_FrameType_vals[] = {
  {   0, "typeA" },
  {   1, "typeB" },
  { 0, NULL }
};


static int
dissect_nbap_DL_FrameType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_dL_FrameType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_FrameType(tvb, offset, actx, tree, hf_nbap_dL_FrameType);
}



static int
dissect_nbap_DeltaSIR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 30U, NULL, FALSE);

  return offset;
}
static int dissect_delta_SIR1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DeltaSIR(tvb, offset, actx, tree, hf_nbap_delta_SIR1);
}
static int dissect_delta_SIR_after1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DeltaSIR(tvb, offset, actx, tree, hf_nbap_delta_SIR_after1);
}
static int dissect_delta_SIR2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DeltaSIR(tvb, offset, actx, tree, hf_nbap_delta_SIR2);
}
static int dissect_delta_SIR_after2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DeltaSIR(tvb, offset, actx, tree, hf_nbap_delta_SIR_after2);
}


static const per_sequence_t Transmission_Gap_Pattern_Sequence_Information_item_sequence[] = {
  { &hf_nbap_tGPSID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TGPSID },
  { &hf_nbap_tGSN           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TGSN },
  { &hf_nbap_tGL1           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_GapLength },
  { &hf_nbap_tGL2           , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_GapLength },
  { &hf_nbap_tGD            , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TGD },
  { &hf_nbap_tGPL1          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_GapDuration },
  { &hf_nbap_not_to_be_used_1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_GapDuration },
  { &hf_nbap_uL_DL_mode     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UL_DL_mode },
  { &hf_nbap_downlink_Compressed_Mode_Method, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_Downlink_Compressed_Mode_Method },
  { &hf_nbap_uplink_Compressed_Mode_Method, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_Uplink_Compressed_Mode_Method },
  { &hf_nbap_dL_FrameType   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_FrameType },
  { &hf_nbap_delta_SIR1     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DeltaSIR },
  { &hf_nbap_delta_SIR_after1, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DeltaSIR },
  { &hf_nbap_delta_SIR2     , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DeltaSIR },
  { &hf_nbap_delta_SIR_after2, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DeltaSIR },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Transmission_Gap_Pattern_Sequence_Information_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Transmission_Gap_Pattern_Sequence_Information_item, Transmission_Gap_Pattern_Sequence_Information_item_sequence);

  return offset;
}
static int dissect_Transmission_Gap_Pattern_Sequence_Information_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Transmission_Gap_Pattern_Sequence_Information_item(tvb, offset, actx, tree, hf_nbap_Transmission_Gap_Pattern_Sequence_Information_item);
}


static const per_sequence_t Transmission_Gap_Pattern_Sequence_Information_sequence_of[1] = {
  { &hf_nbap_Transmission_Gap_Pattern_Sequence_Information_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_Transmission_Gap_Pattern_Sequence_Information_item },
};

static int
dissect_nbap_Transmission_Gap_Pattern_Sequence_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_Transmission_Gap_Pattern_Sequence_Information, Transmission_Gap_Pattern_Sequence_Information_sequence_of,
                                                  1, 6);

  return offset;
}
static int dissect_id_Transmission_Gap_Pattern_Sequence_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Transmission_Gap_Pattern_Sequence_Information(tvb, offset, actx, tree, hf_nbap_id_Transmission_Gap_Pattern_Sequence_Information);
}



static int
dissect_nbap_UL_DPCH_InformationAddList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_ul_DPCH_InformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_DPCH_InformationAddList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_ul_DPCH_InformationList);
}


static const per_sequence_t UL_CCTrCH_InformationAddItem_RL_ReconfPrepTDD_sequence[] = {
  { &hf_nbap_cCTrCH_ID      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CCTrCH_ID },
  { &hf_nbap_tFCS           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TFCS },
  { &hf_nbap_tFCI_Coding    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TFCI_Coding },
  { &hf_nbap_punctureLimit  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PunctureLimit },
  { &hf_nbap_ul_DPCH_InformationList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_UL_DPCH_InformationAddList_RL_ReconfPrepTDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_UL_CCTrCH_InformationAddItem_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_UL_CCTrCH_InformationAddItem_RL_ReconfPrepTDD, UL_CCTrCH_InformationAddItem_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_CCTrCH_InformationAddItem_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD_item);
}


static const per_sequence_t UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD_sequence_of[1] = {
  { &hf_nbap_UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_UL_CCTrCH_InformationAddItem_RL_ReconfPrepTDD },
};

static int
dissect_nbap_UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD, UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_id_UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD);
}


static const per_sequence_t UL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD_sequence[] = {
  { &hf_nbap_cCTrCH_ID      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CCTrCH_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_UL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_UL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD, UL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD_sequence);

  return offset;
}
static int dissect_id_UL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_id_UL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD);
}


static const per_sequence_t UL_CCTrCH_InformationDeleteItem_RL_ReconfPrepTDD_sequence[] = {
  { &hf_nbap_cCTrCH_ID      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CCTrCH_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_UL_CCTrCH_InformationDeleteItem_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_UL_CCTrCH_InformationDeleteItem_RL_ReconfPrepTDD, UL_CCTrCH_InformationDeleteItem_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_CCTrCH_InformationDeleteItem_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD_item);
}


static const per_sequence_t UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD_sequence_of[1] = {
  { &hf_nbap_UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_UL_CCTrCH_InformationDeleteItem_RL_ReconfPrepTDD },
};

static int
dissect_nbap_UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD, UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_id_UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD);
}


static const per_sequence_t UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD_sequence_of[1] = {
  { &hf_nbap_UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD, UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_id_UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD);
}


static const per_sequence_t UL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD_sequence[] = {
  { &hf_nbap_cCTrCH_ID      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CCTrCH_ID },
  { &hf_nbap_tFCS           , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TFCS },
  { &hf_nbap_punctureLimit  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_PunctureLimit },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_UL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_UL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD, UL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD_sequence);

  return offset;
}
static int dissect_id_UL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_id_UL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD);
}



static int
dissect_nbap_UL_DPCH_InformationModify_AddList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_ul_DPCH_InformationAddList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_DPCH_InformationModify_AddList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_ul_DPCH_InformationAddList);
}



static int
dissect_nbap_UL_DPCH_InformationModify_ModifyList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_ul_DPCH_InformationModifyList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_DPCH_InformationModify_ModifyList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_ul_DPCH_InformationModifyList);
}



static int
dissect_nbap_UL_DPCH_InformationModify_DeleteList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_ul_DPCH_InformationDeleteList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_DPCH_InformationModify_DeleteList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_ul_DPCH_InformationDeleteList);
}


static const per_sequence_t UL_CCTrCH_InformationModifyItem_RL_ReconfPrepTDD_sequence[] = {
  { &hf_nbap_cCTrCH_ID      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CCTrCH_ID },
  { &hf_nbap_tFCS           , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TFCS },
  { &hf_nbap_tFCI_Coding    , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TFCI_Coding },
  { &hf_nbap_punctureLimit  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_PunctureLimit },
  { &hf_nbap_ul_DPCH_InformationAddList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_UL_DPCH_InformationModify_AddList_RL_ReconfPrepTDD },
  { &hf_nbap_ul_DPCH_InformationModifyList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_UL_DPCH_InformationModify_ModifyList_RL_ReconfPrepTDD },
  { &hf_nbap_ul_DPCH_InformationDeleteList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_UL_DPCH_InformationModify_DeleteList_RL_ReconfPrepTDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_UL_CCTrCH_InformationModifyItem_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_UL_CCTrCH_InformationModifyItem_RL_ReconfPrepTDD, UL_CCTrCH_InformationModifyItem_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_CCTrCH_InformationModifyItem_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD_item);
}


static const per_sequence_t UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD_sequence_of[1] = {
  { &hf_nbap_UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_UL_CCTrCH_InformationModifyItem_RL_ReconfPrepTDD },
};

static int
dissect_nbap_UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD, UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_id_UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD);
}


static const per_sequence_t UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD_sequence_of[1] = {
  { &hf_nbap_UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD, UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_id_UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD);
}


static const per_sequence_t UL_DPCH_InformationAddItem_RL_ReconfPrepTDD_sequence[] = {
  { &hf_nbap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionPeriod },
  { &hf_nbap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionLength },
  { &hf_nbap_tdd_DPCHOffset , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_DPCHOffset },
  { &hf_nbap_uL_Timeslot_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UL_Timeslot_Information },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_UL_DPCH_InformationAddItem_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_UL_DPCH_InformationAddItem_RL_ReconfPrepTDD, UL_DPCH_InformationAddItem_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_id_UL_DPCH_InformationAddListIE_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_DPCH_InformationAddItem_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_id_UL_DPCH_InformationAddListIE_RL_ReconfPrepTDD);
}


static const per_sequence_t UL_DPCH_InformationModify_AddItem_RL_ReconfPrepTDD_sequence[] = {
  { &hf_nbap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionPeriod },
  { &hf_nbap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionLength },
  { &hf_nbap_tdd_DPCHOffset , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_DPCHOffset },
  { &hf_nbap_uL_Timeslot_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UL_Timeslot_Information },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_UL_DPCH_InformationModify_AddItem_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_UL_DPCH_InformationModify_AddItem_RL_ReconfPrepTDD, UL_DPCH_InformationModify_AddItem_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_id_UL_DPCH_InformationModify_AddListIE_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_DPCH_InformationModify_AddItem_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_id_UL_DPCH_InformationModify_AddListIE_RL_ReconfPrepTDD);
}


static const per_sequence_t UL_DPCH_InformationModify_DeleteItem_RL_ReconfPrepTDD_sequence[] = {
  { &hf_nbap_dPCH_ID        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DPCH_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_UL_DPCH_InformationModify_DeleteItem_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_UL_DPCH_InformationModify_DeleteItem_RL_ReconfPrepTDD, UL_DPCH_InformationModify_DeleteItem_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_UL_DPCH_InformationModify_DeleteListIE_RL_ReconfPrepTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_DPCH_InformationModify_DeleteItem_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_UL_DPCH_InformationModify_DeleteListIE_RL_ReconfPrepTDD_item);
}


static const per_sequence_t UL_DPCH_InformationModify_DeleteListIE_RL_ReconfPrepTDD_sequence_of[1] = {
  { &hf_nbap_UL_DPCH_InformationModify_DeleteListIE_RL_ReconfPrepTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_UL_DPCH_InformationModify_DeleteItem_RL_ReconfPrepTDD },
};

static int
dissect_nbap_UL_DPCH_InformationModify_DeleteListIE_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_UL_DPCH_InformationModify_DeleteListIE_RL_ReconfPrepTDD, UL_DPCH_InformationModify_DeleteListIE_RL_ReconfPrepTDD_sequence_of,
                                                  1, 240);

  return offset;
}
static int dissect_id_UL_DPCH_InformationModify_DeleteListIE_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_DPCH_InformationModify_DeleteListIE_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_id_UL_DPCH_InformationModify_DeleteListIE_RL_ReconfPrepTDD);
}


static const per_sequence_t UL_Code_InformationModify_ModifyItem_RL_ReconfPrepTDD_sequence[] = {
  { &hf_nbap_dPCH_ID        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DPCH_ID },
  { &hf_nbap_tdd_ChannelisationCode, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TDD_ChannelisationCode },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_UL_Code_InformationModify_ModifyItem_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_UL_Code_InformationModify_ModifyItem_RL_ReconfPrepTDD, UL_Code_InformationModify_ModifyItem_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_UL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_Code_InformationModify_ModifyItem_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_UL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD_item);
}


static const per_sequence_t UL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD_sequence_of[1] = {
  { &hf_nbap_UL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_UL_Code_InformationModify_ModifyItem_RL_ReconfPrepTDD },
};

static int
dissect_nbap_UL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_UL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD, UL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD_sequence_of,
                                                  1, 240);

  return offset;
}
static int dissect_uL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_uL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD);
}


static const per_sequence_t UL_Timeslot_InformationModify_ModifyItem_RL_ReconfPrepTDD_sequence[] = {
  { &hf_nbap_timeSlot       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlot },
  { &hf_nbap_midambleShiftAndBurstType, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_MidambleShiftAndBurstType },
  { &hf_nbap_tFCI_Presence  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TFCI_Presence },
  { &hf_nbap_uL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_UL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_UL_Timeslot_InformationModify_ModifyItem_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_UL_Timeslot_InformationModify_ModifyItem_RL_ReconfPrepTDD, UL_Timeslot_InformationModify_ModifyItem_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_UL_Timeslot_InformationModify_ModifyList_RL_ReconfPrepTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_Timeslot_InformationModify_ModifyItem_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_UL_Timeslot_InformationModify_ModifyList_RL_ReconfPrepTDD_item);
}


static const per_sequence_t UL_Timeslot_InformationModify_ModifyList_RL_ReconfPrepTDD_sequence_of[1] = {
  { &hf_nbap_UL_Timeslot_InformationModify_ModifyList_RL_ReconfPrepTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_UL_Timeslot_InformationModify_ModifyItem_RL_ReconfPrepTDD },
};

static int
dissect_nbap_UL_Timeslot_InformationModify_ModifyList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_UL_Timeslot_InformationModify_ModifyList_RL_ReconfPrepTDD, UL_Timeslot_InformationModify_ModifyList_RL_ReconfPrepTDD_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_uL_Timeslot_InformationModify_ModifyList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_Timeslot_InformationModify_ModifyList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_uL_Timeslot_InformationModify_ModifyList_RL_ReconfPrepTDD);
}


static const per_sequence_t UL_DPCH_InformationModify_ModifyItem_RL_ReconfPrepTDD_sequence[] = {
  { &hf_nbap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_RepetitionPeriod },
  { &hf_nbap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_RepetitionLength },
  { &hf_nbap_tdd_DPCHOffset , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TDD_DPCHOffset },
  { &hf_nbap_uL_Timeslot_InformationModify_ModifyList_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_UL_Timeslot_InformationModify_ModifyList_RL_ReconfPrepTDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_UL_DPCH_InformationModify_ModifyItem_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_UL_DPCH_InformationModify_ModifyItem_RL_ReconfPrepTDD, UL_DPCH_InformationModify_ModifyItem_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_id_UL_DPCH_InformationModify_ModifyListIE_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_DPCH_InformationModify_ModifyItem_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_id_UL_DPCH_InformationModify_ModifyListIE_RL_ReconfPrepTDD);
}


static const per_sequence_t Unsuccessful_PDSCHSetItem_PSCH_ReconfFailureTDD_sequence[] = {
  { &hf_nbap_pDSCHSet_ID    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PDSCHSet_ID },
  { &hf_nbap_cause          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Cause },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Unsuccessful_PDSCHSetItem_PSCH_ReconfFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Unsuccessful_PDSCHSetItem_PSCH_ReconfFailureTDD, Unsuccessful_PDSCHSetItem_PSCH_ReconfFailureTDD_sequence);

  return offset;
}
static int dissect_id_Unsuccessful_PDSCHSetItem_PSCH_ReconfFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Unsuccessful_PDSCHSetItem_PSCH_ReconfFailureTDD(tvb, offset, actx, tree, hf_nbap_id_Unsuccessful_PDSCHSetItem_PSCH_ReconfFailureTDD);
}


static const per_sequence_t Unsuccessful_PUSCHSetItem_PSCH_ReconfFailureTDD_sequence[] = {
  { &hf_nbap_pUSCHSet_ID    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PUSCHSet_ID },
  { &hf_nbap_cause          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Cause },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Unsuccessful_PUSCHSetItem_PSCH_ReconfFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Unsuccessful_PUSCHSetItem_PSCH_ReconfFailureTDD, Unsuccessful_PUSCHSetItem_PSCH_ReconfFailureTDD_sequence);

  return offset;
}
static int dissect_id_Unsuccessful_PUSCHSetItem_PSCH_ReconfFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Unsuccessful_PUSCHSetItem_PSCH_ReconfFailureTDD(tvb, offset, actx, tree, hf_nbap_id_Unsuccessful_PUSCHSetItem_PSCH_ReconfFailureTDD);
}


static const value_string nbap_CommunicationContextType_Reset_vals[] = {
  {   0, "cRNC-CommunicationContextID" },
  {   1, "nodeB-CommunicationContextID" },
  { 0, NULL }
};

static const per_choice_t CommunicationContextType_Reset_choice[] = {
  {   0, &hf_nbap_cRNC_CommunicationContextID, ASN1_EXTENSION_ROOT    , dissect_nbap_CRNC_CommunicationContextID },
  {   1, &hf_nbap_nodeB_CommunicationContextID, ASN1_EXTENSION_ROOT    , dissect_nbap_NodeB_CommunicationContextID },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_CommunicationContextType_Reset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_CommunicationContextType_Reset, CommunicationContextType_Reset_choice,
                                 NULL);

  return offset;
}
static int dissect_communicationContextType_Reset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommunicationContextType_Reset(tvb, offset, actx, tree, hf_nbap_communicationContextType_Reset);
}


static const per_sequence_t CommunicationContextInfoItem_Reset_sequence[] = {
  { &hf_nbap_communicationContextType_Reset, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommunicationContextType_Reset },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CommunicationContextInfoItem_Reset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CommunicationContextInfoItem_Reset, CommunicationContextInfoItem_Reset_sequence);

  return offset;
}
static int dissect_id_CommunicationContextInfoItem_Reset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommunicationContextInfoItem_Reset(tvb, offset, actx, tree, hf_nbap_id_CommunicationContextInfoItem_Reset);
}


static const per_sequence_t CommunicationControlPortInfoItem_Reset_sequence[] = {
  { &hf_nbap_communicationControlPortID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommunicationControlPortID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CommunicationControlPortInfoItem_Reset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CommunicationControlPortInfoItem_Reset, CommunicationControlPortInfoItem_Reset_sequence);

  return offset;
}
static int dissect_id_CommunicationControlPortInfoItem_Reset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommunicationControlPortInfoItem_Reset(tvb, offset, actx, tree, hf_nbap_id_CommunicationControlPortInfoItem_Reset);
}


static const per_sequence_t CommunicationContextInfoList_Reset_sequence_of[1] = {
  { &hf_nbap_CommunicationContextInfoList_Reset_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_CommunicationContextInfoList_Reset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_CommunicationContextInfoList_Reset, CommunicationContextInfoList_Reset_sequence_of,
                                                  1, 1048575);

  return offset;
}
static int dissect_communicationContextInfoList_Reset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommunicationContextInfoList_Reset(tvb, offset, actx, tree, hf_nbap_communicationContextInfoList_Reset);
}


static const per_sequence_t CommunicationContextList_Reset_sequence[] = {
  { &hf_nbap_communicationContextInfoList_Reset, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommunicationContextInfoList_Reset },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CommunicationContextList_Reset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CommunicationContextList_Reset, CommunicationContextList_Reset_sequence);

  return offset;
}
static int dissect_communicationContext(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommunicationContextList_Reset(tvb, offset, actx, tree, hf_nbap_communicationContext);
}


static const per_sequence_t CommunicationControlPortInfoList_Reset_sequence_of[1] = {
  { &hf_nbap_CommunicationControlPortInfoList_Reset_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_CommunicationControlPortInfoList_Reset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_CommunicationControlPortInfoList_Reset, CommunicationControlPortInfoList_Reset_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_communicationControlPortInfoList_Reset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommunicationControlPortInfoList_Reset(tvb, offset, actx, tree, hf_nbap_communicationControlPortInfoList_Reset);
}


static const per_sequence_t CommunicationControlPortList_Reset_sequence[] = {
  { &hf_nbap_communicationControlPortInfoList_Reset, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommunicationControlPortInfoList_Reset },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CommunicationControlPortList_Reset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CommunicationControlPortList_Reset, CommunicationControlPortList_Reset_sequence);

  return offset;
}
static int dissect_communicationControlPort(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CommunicationControlPortList_Reset(tvb, offset, actx, tree, hf_nbap_communicationControlPort);
}


static const value_string nbap_ResetIndicator_vals[] = {
  {   0, "communicationContext" },
  {   1, "communicationControlPort" },
  {   2, "nodeB" },
  { 0, NULL }
};

static const per_choice_t ResetIndicator_choice[] = {
  {   0, &hf_nbap_communicationContext, ASN1_EXTENSION_ROOT    , dissect_nbap_CommunicationContextList_Reset },
  {   1, &hf_nbap_communicationControlPort, ASN1_EXTENSION_ROOT    , dissect_nbap_CommunicationControlPortList_Reset },
  {   2, &hf_nbap_nodeB          , ASN1_EXTENSION_ROOT    , dissect_nbap_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_ResetIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_ResetIndicator, ResetIndicator_choice,
                                 NULL);

  return offset;
}
static int dissect_id_ResetIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ResetIndicator(tvb, offset, actx, tree, hf_nbap_id_ResetIndicator);
}


static const value_string nbap_TimingAdvanceApplied_vals[] = {
  {   0, "yes" },
  {   1, "no" },
  { 0, NULL }
};


static int
dissect_nbap_TimingAdvanceApplied(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_TimingAdvanceApplied(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TimingAdvanceApplied(tvb, offset, actx, tree, hf_nbap_id_TimingAdvanceApplied);
}


static const value_string nbap_FNReportingIndicator_vals[] = {
  {   0, "fN-reporting-required" },
  {   1, "fN-reporting-not-required" },
  { 0, NULL }
};


static int
dissect_nbap_FNReportingIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_CFNReportingIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_FNReportingIndicator(tvb, offset, actx, tree, hf_nbap_id_CFNReportingIndicator);
}
static int dissect_id_SFNReportingIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_FNReportingIndicator(tvb, offset, actx, tree, hf_nbap_id_SFNReportingIndicator);
}


static const value_string nbap_PagingIndicatorLength_vals[] = {
  {   0, "v2" },
  {   1, "v4" },
  {   2, "v8" },
  { 0, NULL }
};


static int
dissect_nbap_PagingIndicatorLength(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_pagingIndicatorLength(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PagingIndicatorLength(tvb, offset, actx, tree, hf_nbap_pagingIndicatorLength);
}


static const per_sequence_t PICH_ParametersItem_CTCH_SetupRqstTDD_sequence[] = {
  { &hf_nbap_commonPhysicalChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonPhysicalChannelID },
  { &hf_nbap_tdd_ChannelisationCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_ChannelisationCode },
  { &hf_nbap_timeSlot       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlot },
  { &hf_nbap_midambleshiftAndBurstType, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MidambleShiftAndBurstType },
  { &hf_nbap_tdd_PhysicalChannelOffset, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_PhysicalChannelOffset },
  { &hf_nbap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionPeriod },
  { &hf_nbap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionLength },
  { &hf_nbap_pagingIndicatorLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PagingIndicatorLength },
  { &hf_nbap_pICH_Power     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PICH_Power },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PICH_ParametersItem_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PICH_ParametersItem_CTCH_SetupRqstTDD, PICH_ParametersItem_CTCH_SetupRqstTDD_sequence);

  return offset;
}
static int dissect_id_PICH_ParametersItem_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PICH_ParametersItem_CTCH_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_id_PICH_ParametersItem_CTCH_SetupRqstTDD);
}


static const value_string nbap_MaxPRACH_MidambleShifts_vals[] = {
  {   0, "shift4" },
  {   1, "shift8" },
  { 0, NULL }
};


static int
dissect_nbap_MaxPRACH_MidambleShifts(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_maxPRACH_MidambleShifts(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MaxPRACH_MidambleShifts(tvb, offset, actx, tree, hf_nbap_maxPRACH_MidambleShifts);
}


static const value_string nbap_PRACH_Midamble_vals[] = {
  {   0, "inverted" },
  {   1, "direct" },
  { 0, NULL }
};


static int
dissect_nbap_PRACH_Midamble(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_pRACH_Midamble(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PRACH_Midamble(tvb, offset, actx, tree, hf_nbap_pRACH_Midamble);
}



static int
dissect_nbap_RACH_Parameter_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_rACH(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RACH_Parameter_CTCH_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_rACH);
}


static const per_sequence_t PRACH_ParametersItem_CTCH_SetupRqstTDD_sequence[] = {
  { &hf_nbap_commonPhysicalChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonPhysicalChannelID },
  { &hf_nbap_tFCS           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TFCS },
  { &hf_nbap_timeslot       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlot },
  { &hf_nbap_tdd_ChannelisationCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_ChannelisationCode },
  { &hf_nbap_maxPRACH_MidambleShifts, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MaxPRACH_MidambleShifts },
  { &hf_nbap_pRACH_Midamble , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PRACH_Midamble },
  { &hf_nbap_rACH           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RACH_Parameter_CTCH_SetupRqstTDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PRACH_ParametersItem_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PRACH_ParametersItem_CTCH_SetupRqstTDD, PRACH_ParametersItem_CTCH_SetupRqstTDD_sequence);

  return offset;
}
static int dissect_id_PRACH_ParametersItem_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PRACH_ParametersItem_CTCH_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_id_PRACH_ParametersItem_CTCH_SetupRqstTDD);
}


static const per_sequence_t CCTrCH_InformationItem_RL_FailureInd_sequence[] = {
  { &hf_nbap_cCTrCH_ID      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CCTrCH_ID },
  { &hf_nbap_cause          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Cause },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CCTrCH_InformationItem_RL_FailureInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CCTrCH_InformationItem_RL_FailureInd, CCTrCH_InformationItem_RL_FailureInd_sequence);

  return offset;
}
static int dissect_id_CCTrCH_InformationItem_RL_FailureInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CCTrCH_InformationItem_RL_FailureInd(tvb, offset, actx, tree, hf_nbap_id_CCTrCH_InformationItem_RL_FailureInd);
}


static const per_sequence_t CCTrCH_InformationItem_RL_RestoreInd_sequence[] = {
  { &hf_nbap_cCTrCH_ID      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CCTrCH_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CCTrCH_InformationItem_RL_RestoreInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CCTrCH_InformationItem_RL_RestoreInd, CCTrCH_InformationItem_RL_RestoreInd_sequence);

  return offset;
}
static int dissect_id_CCTrCH_InformationItem_RL_RestoreInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CCTrCH_InformationItem_RL_RestoreInd(tvb, offset, actx, tree, hf_nbap_id_CCTrCH_InformationItem_RL_RestoreInd);
}


static const per_sequence_t GeneralCauseList_SyncAdjustmntFailureTDD_sequence[] = {
  { &hf_nbap_cause          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Cause },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_GeneralCauseList_SyncAdjustmntFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_GeneralCauseList_SyncAdjustmntFailureTDD, GeneralCauseList_SyncAdjustmntFailureTDD_sequence);

  return offset;
}
static int dissect_generalCause6(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_GeneralCauseList_SyncAdjustmntFailureTDD(tvb, offset, actx, tree, hf_nbap_generalCause6);
}


static const per_sequence_t Unsuccessful_cell_InformationRespList_SyncAdjustmntFailureTDD_sequence_of[1] = {
  { &hf_nbap_Unsuccessful_cell_InformationRespList_SyncAdjustmntFailureTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_Unsuccessful_cell_InformationRespList_SyncAdjustmntFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_Unsuccessful_cell_InformationRespList_SyncAdjustmntFailureTDD, Unsuccessful_cell_InformationRespList_SyncAdjustmntFailureTDD_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_unsuccessful_cell_InformationRespList_SyncAdjustmntFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Unsuccessful_cell_InformationRespList_SyncAdjustmntFailureTDD(tvb, offset, actx, tree, hf_nbap_unsuccessful_cell_InformationRespList_SyncAdjustmntFailureTDD);
}


static const per_sequence_t CellSpecificCauseList_SyncAdjustmntFailureTDD_sequence[] = {
  { &hf_nbap_unsuccessful_cell_InformationRespList_SyncAdjustmntFailureTDD, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Unsuccessful_cell_InformationRespList_SyncAdjustmntFailureTDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CellSpecificCauseList_SyncAdjustmntFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CellSpecificCauseList_SyncAdjustmntFailureTDD, CellSpecificCauseList_SyncAdjustmntFailureTDD_sequence);

  return offset;
}
static int dissect_cellSpecificCause(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSpecificCauseList_SyncAdjustmntFailureTDD(tvb, offset, actx, tree, hf_nbap_cellSpecificCause);
}


static const value_string nbap_CauseLevel_SyncAdjustmntFailureTDD_vals[] = {
  {   0, "generalCause" },
  {   1, "cellSpecificCause" },
  { 0, NULL }
};

static const per_choice_t CauseLevel_SyncAdjustmntFailureTDD_choice[] = {
  {   0, &hf_nbap_generalCause6  , ASN1_EXTENSION_ROOT    , dissect_nbap_GeneralCauseList_SyncAdjustmntFailureTDD },
  {   1, &hf_nbap_cellSpecificCause, ASN1_EXTENSION_ROOT    , dissect_nbap_CellSpecificCauseList_SyncAdjustmntFailureTDD },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_CauseLevel_SyncAdjustmntFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_CauseLevel_SyncAdjustmntFailureTDD, CauseLevel_SyncAdjustmntFailureTDD_choice,
                                 NULL);

  return offset;
}
static int dissect_id_CauseLevel_SyncAdjustmntFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CauseLevel_SyncAdjustmntFailureTDD(tvb, offset, actx, tree, hf_nbap_id_CauseLevel_SyncAdjustmntFailureTDD);
}


static const per_sequence_t CellAdjustmentInfo_SyncAdjustmentRqstTDD_sequence_of[1] = {
  { &hf_nbap_CellAdjustmentInfo_SyncAdjustmentRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_CellAdjustmentInfo_SyncAdjustmentRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_CellAdjustmentInfo_SyncAdjustmentRqstTDD, CellAdjustmentInfo_SyncAdjustmentRqstTDD_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_id_CellAdjustmentInfo_SyncAdjustmntRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellAdjustmentInfo_SyncAdjustmentRqstTDD(tvb, offset, actx, tree, hf_nbap_id_CellAdjustmentInfo_SyncAdjustmntRqstTDD);
}



static int
dissect_nbap_FrameAdjustmentValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 4095U, NULL, FALSE);

  return offset;
}
static int dissect_frameAdjustmentValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_FrameAdjustmentValue(tvb, offset, actx, tree, hf_nbap_frameAdjustmentValue);
}



static int
dissect_nbap_INTEGER_0_1048575_(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 1048575U, NULL, TRUE);

  return offset;
}
static int dissect_initialPhase_0_1048575(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_INTEGER_0_1048575_(tvb, offset, actx, tree, hf_nbap_initialPhase_0_1048575);
}
static int dissect_initialPhase1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_INTEGER_0_1048575_(tvb, offset, actx, tree, hf_nbap_initialPhase1);
}



static int
dissect_nbap_INTEGER_0_255_(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 255U, NULL, TRUE);

  return offset;
}
static int dissect_steadyStatePhase(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_INTEGER_0_255_(tvb, offset, actx, tree, hf_nbap_steadyStatePhase);
}


static const value_string nbap_TimingAdjustmentValue_vals[] = {
  {   0, "initialPhase" },
  {   1, "steadyStatePhase" },
  { 0, NULL }
};

static const per_choice_t TimingAdjustmentValue_choice[] = {
  {   0, &hf_nbap_initialPhase1  , ASN1_NO_EXTENSIONS     , dissect_nbap_INTEGER_0_1048575_ },
  {   1, &hf_nbap_steadyStatePhase, ASN1_NO_EXTENSIONS     , dissect_nbap_INTEGER_0_255_ },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_TimingAdjustmentValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_TimingAdjustmentValue, TimingAdjustmentValue_choice,
                                 NULL);

  return offset;
}
static int dissect_id_AccumulatedClockupdate_CellSyncReprtTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TimingAdjustmentValue(tvb, offset, actx, tree, hf_nbap_id_AccumulatedClockupdate_CellSyncReprtTDD);
}
static int dissect_timingAdjustmentValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TimingAdjustmentValue(tvb, offset, actx, tree, hf_nbap_timingAdjustmentValue);
}


static const per_sequence_t CellAdjustmentInfoItem_SyncAdjustmentRqstTDD_sequence[] = {
  { &hf_nbap_c_ID           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_C_ID },
  { &hf_nbap_frameAdjustmentValue, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_FrameAdjustmentValue },
  { &hf_nbap_timingAdjustmentValue, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TimingAdjustmentValue },
  { &hf_nbap_dLTransPower   , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_Power },
  { &hf_nbap_sfn            , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_SFN },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CellAdjustmentInfoItem_SyncAdjustmentRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CellAdjustmentInfoItem_SyncAdjustmentRqstTDD, CellAdjustmentInfoItem_SyncAdjustmentRqstTDD_sequence);

  return offset;
}
static int dissect_id_CellAdjustmentInfoItem_SyncAdjustmentRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellAdjustmentInfoItem_SyncAdjustmentRqstTDD(tvb, offset, actx, tree, hf_nbap_id_CellAdjustmentInfoItem_SyncAdjustmentRqstTDD);
}



static int
dissect_nbap_CSBMeasurementID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 65535U, NULL, FALSE);

  return offset;
}
static int dissect_id_CSBMeasurementID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CSBMeasurementID(tvb, offset, actx, tree, hf_nbap_id_CSBMeasurementID);
}
static int dissect_cSBMeasurementID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CSBMeasurementID(tvb, offset, actx, tree, hf_nbap_cSBMeasurementID);
}



static int
dissect_nbap_CellSyncBurstCode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 7U, NULL, TRUE);

  return offset;
}
static int dissect_cellSyncBurstCode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSyncBurstCode(tvb, offset, actx, tree, hf_nbap_cellSyncBurstCode);
}



static int
dissect_nbap_CellSyncBurstCodeShift(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 7U, NULL, FALSE);

  return offset;
}
static int dissect_cellSyncBurstCodeShift(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSyncBurstCodeShift(tvb, offset, actx, tree, hf_nbap_cellSyncBurstCodeShift);
}


static const per_sequence_t CellSyncBurstInfoItem_CellSyncReconfRqstTDD_sequence[] = {
  { &hf_nbap_cSBMeasurementID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CSBMeasurementID },
  { &hf_nbap_cellSyncBurstCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CellSyncBurstCode },
  { &hf_nbap_cellSyncBurstCodeShift, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CellSyncBurstCodeShift },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CellSyncBurstInfoItem_CellSyncReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CellSyncBurstInfoItem_CellSyncReconfRqstTDD, CellSyncBurstInfoItem_CellSyncReconfRqstTDD_sequence);

  return offset;
}
static int dissect_CellSyncBurstInfoList_CellSyncReconfRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSyncBurstInfoItem_CellSyncReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_CellSyncBurstInfoList_CellSyncReconfRqstTDD_item);
}


static const per_sequence_t CellSyncBurstInfoList_CellSyncReconfRqstTDD_sequence_of[1] = {
  { &hf_nbap_CellSyncBurstInfoList_CellSyncReconfRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_CellSyncBurstInfoItem_CellSyncReconfRqstTDD },
};

static int
dissect_nbap_CellSyncBurstInfoList_CellSyncReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_CellSyncBurstInfoList_CellSyncReconfRqstTDD, CellSyncBurstInfoList_CellSyncReconfRqstTDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_CellSyncBurstInfoList_CellSyncReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSyncBurstInfoList_CellSyncReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_id_CellSyncBurstInfoList_CellSyncReconfRqstTDD);
}
static int dissect_syncBurstInfo(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSyncBurstInfoList_CellSyncReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_syncBurstInfo);
}



static int
dissect_nbap_CSBTransmissionID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 65535U, NULL, FALSE);

  return offset;
}
static int dissect_id_CSBTransmissionID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CSBTransmissionID(tvb, offset, actx, tree, hf_nbap_id_CSBTransmissionID);
}
static int dissect_cSBTransmissionID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CSBTransmissionID(tvb, offset, actx, tree, hf_nbap_cSBTransmissionID);
}


static const per_sequence_t CellSyncBurstTransInit_CellSyncInitiationRqstTDD_sequence[] = {
  { &hf_nbap_cSBTransmissionID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CSBTransmissionID },
  { &hf_nbap_sfn            , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SFN },
  { &hf_nbap_cellSyncBurstCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CellSyncBurstCode },
  { &hf_nbap_cellSyncBurstCodeShift, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CellSyncBurstCodeShift },
  { &hf_nbap_initialDLTransPower, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Power },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CellSyncBurstTransInit_CellSyncInitiationRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CellSyncBurstTransInit_CellSyncInitiationRqstTDD, CellSyncBurstTransInit_CellSyncInitiationRqstTDD_sequence);

  return offset;
}
static int dissect_id_CellSyncBurstTransInit_CellSyncInitiationRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSyncBurstTransInit_CellSyncInitiationRqstTDD(tvb, offset, actx, tree, hf_nbap_id_CellSyncBurstTransInit_CellSyncInitiationRqstTDD);
}


static const value_string nbap_SynchronisationReportType_vals[] = {
  {   0, "initialPhase" },
  {   1, "steadyStatePhase" },
  {   2, "lateEntrantCell" },
  {   3, "frequencyAcquisition" },
  { 0, NULL }
};


static int
dissect_nbap_SynchronisationReportType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_id_SynchronisationReportType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SynchronisationReportType(tvb, offset, actx, tree, hf_nbap_id_SynchronisationReportType);
}
static int dissect_synchronisationReportType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SynchronisationReportType(tvb, offset, actx, tree, hf_nbap_synchronisationReportType);
}


static const value_string nbap_SynchronisationReportCharacteristicsType_vals[] = {
  {   0, "frameRelated" },
  {   1, "sFNperiodRelated" },
  {   2, "cycleLengthRelated" },
  {   3, "thresholdExceeding" },
  {   4, "frequencyAcquisitionCompleted" },
  { 0, NULL }
};


static int
dissect_nbap_SynchronisationReportCharacteristicsType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     5, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_synchronisationReportCharacteristicsType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SynchronisationReportCharacteristicsType(tvb, offset, actx, tree, hf_nbap_synchronisationReportCharacteristicsType);
}



static int
dissect_nbap_SyncFrameNumber(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 10U, NULL, FALSE);

  return offset;
}
static int dissect_id_SyncFrameNumber(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SyncFrameNumber(tvb, offset, actx, tree, hf_nbap_id_SyncFrameNumber);
}
static int dissect_syncFrameNumberToTransmit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SyncFrameNumber(tvb, offset, actx, tree, hf_nbap_syncFrameNumberToTransmit);
}
static int dissect_syncFrameNrToReceive(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SyncFrameNumber(tvb, offset, actx, tree, hf_nbap_syncFrameNrToReceive);
}
static int dissect_syncFrameNumberforTransmit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SyncFrameNumber(tvb, offset, actx, tree, hf_nbap_syncFrameNumberforTransmit);
}
static int dissect_syncFrameNumber(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SyncFrameNumber(tvb, offset, actx, tree, hf_nbap_syncFrameNumber);
}
static int dissect_syncFrameNoToReceive(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SyncFrameNumber(tvb, offset, actx, tree, hf_nbap_syncFrameNoToReceive);
}


static const value_string nbap_CellSyncBurstTiming_vals[] = {
  {   0, "initialPhase" },
  {   1, "steadyStatePhase" },
  { 0, NULL }
};

static const per_choice_t CellSyncBurstTiming_choice[] = {
  {   0, &hf_nbap_initialPhase_0_1048575, ASN1_NO_EXTENSIONS     , dissect_nbap_INTEGER_0_1048575_ },
  {   1, &hf_nbap_steadyStatePhase, ASN1_NO_EXTENSIONS     , dissect_nbap_INTEGER_0_255_ },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_CellSyncBurstTiming(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_CellSyncBurstTiming, CellSyncBurstTiming_choice,
                                 NULL);

  return offset;
}
static int dissect_cellSyncBurstTiming(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSyncBurstTiming(tvb, offset, actx, tree, hf_nbap_cellSyncBurstTiming);
}



static int
dissect_nbap_CellSyncBurstTimingThreshold(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 254U, NULL, FALSE);

  return offset;
}
static int dissect_cellSyncBurstTimingThreshold(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSyncBurstTimingThreshold(tvb, offset, actx, tree, hf_nbap_cellSyncBurstTimingThreshold);
}
static int dissect_syncDLCodeIdTimingThre(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSyncBurstTimingThreshold(tvb, offset, actx, tree, hf_nbap_syncDLCodeIdTimingThre);
}


static const per_sequence_t SynchronisationReportCharactCellSyncBurstInfoItem_sequence[] = {
  { &hf_nbap_cellSyncBurstCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CellSyncBurstCode },
  { &hf_nbap_cellSyncBurstCodeShift, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CellSyncBurstCodeShift },
  { &hf_nbap_cellSyncBurstTiming, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_CellSyncBurstTiming },
  { &hf_nbap_cellSyncBurstTimingThreshold, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_CellSyncBurstTimingThreshold },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_SynchronisationReportCharactCellSyncBurstInfoItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_SynchronisationReportCharactCellSyncBurstInfoItem, SynchronisationReportCharactCellSyncBurstInfoItem_sequence);

  return offset;
}
static int dissect_cellSyncBurstInformation_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SynchronisationReportCharactCellSyncBurstInfoItem(tvb, offset, actx, tree, hf_nbap_cellSyncBurstInformation_item);
}


static const per_sequence_t SEQUENCE_SIZE_1_16_OF_SynchronisationReportCharactCellSyncBurstInfoItem_sequence_of[1] = {
  { &hf_nbap_cellSyncBurstInformation_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_SynchronisationReportCharactCellSyncBurstInfoItem },
};

static int
dissect_nbap_SEQUENCE_SIZE_1_16_OF_SynchronisationReportCharactCellSyncBurstInfoItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_SEQUENCE_SIZE_1_16_OF_SynchronisationReportCharactCellSyncBurstInfoItem, SEQUENCE_SIZE_1_16_OF_SynchronisationReportCharactCellSyncBurstInfoItem_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_cellSyncBurstInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SEQUENCE_SIZE_1_16_OF_SynchronisationReportCharactCellSyncBurstInfoItem(tvb, offset, actx, tree, hf_nbap_cellSyncBurstInformation);
}


static const per_sequence_t SynchronisationReportCharactThreInfoItem_sequence[] = {
  { &hf_nbap_syncFrameNumber, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SyncFrameNumber },
  { &hf_nbap_cellSyncBurstInformation, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SEQUENCE_SIZE_1_16_OF_SynchronisationReportCharactCellSyncBurstInfoItem },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_SynchronisationReportCharactThreInfoItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_SynchronisationReportCharactThreInfoItem, SynchronisationReportCharactThreInfoItem_sequence);

  return offset;
}
static int dissect_SynchronisationReportCharactThreExc_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SynchronisationReportCharactThreInfoItem(tvb, offset, actx, tree, hf_nbap_SynchronisationReportCharactThreExc_item);
}


static const per_sequence_t SynchronisationReportCharactThreExc_sequence_of[1] = {
  { &hf_nbap_SynchronisationReportCharactThreExc_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_SynchronisationReportCharactThreInfoItem },
};

static int
dissect_nbap_SynchronisationReportCharactThreExc(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_SynchronisationReportCharactThreExc, SynchronisationReportCharactThreExc_sequence_of,
                                                  1, 10);

  return offset;
}
static int dissect_synchronisationReportCharactThreExc(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SynchronisationReportCharactThreExc(tvb, offset, actx, tree, hf_nbap_synchronisationReportCharactThreExc);
}


static const per_sequence_t SynchronisationReportCharacteristics_sequence[] = {
  { &hf_nbap_synchronisationReportCharacteristicsType, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SynchronisationReportCharacteristicsType },
  { &hf_nbap_synchronisationReportCharactThreExc, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_SynchronisationReportCharactThreExc },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_SynchronisationReportCharacteristics(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_SynchronisationReportCharacteristics, SynchronisationReportCharacteristics_sequence);

  return offset;
}
static int dissect_id_SynchronisationReportCharacteristics(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SynchronisationReportCharacteristics(tvb, offset, actx, tree, hf_nbap_id_SynchronisationReportCharacteristics);
}
static int dissect_synchronisationReportCharacteristics(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SynchronisationReportCharacteristics(tvb, offset, actx, tree, hf_nbap_synchronisationReportCharacteristics);
}


static const per_sequence_t CellSyncBurstMeasureInit_CellSyncInitiationRqstTDD_sequence[] = {
  { &hf_nbap_cSBMeasurementID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CSBMeasurementID },
  { &hf_nbap_cellSyncBurstCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CellSyncBurstCode },
  { &hf_nbap_cellSyncBurstCodeShift, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CellSyncBurstCodeShift },
  { &hf_nbap_synchronisationReportType, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SynchronisationReportType },
  { &hf_nbap_sfn            , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_SFN },
  { &hf_nbap_synchronisationReportCharacteristics, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SynchronisationReportCharacteristics },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CellSyncBurstMeasureInit_CellSyncInitiationRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CellSyncBurstMeasureInit_CellSyncInitiationRqstTDD, CellSyncBurstMeasureInit_CellSyncInitiationRqstTDD_sequence);

  return offset;
}
static int dissect_id_CellSyncBurstMeasureInit_CellSyncInitiationRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSyncBurstMeasureInit_CellSyncInitiationRqstTDD(tvb, offset, actx, tree, hf_nbap_id_CellSyncBurstMeasureInit_CellSyncInitiationRqstTDD);
}



static int
dissect_nbap_CellSyncBurstMeasInfoList_CellSyncReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_id_CellSyncBurstMeasInfoList_CellSyncReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSyncBurstMeasInfoList_CellSyncReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_id_CellSyncBurstMeasInfoList_CellSyncReconfRqstTDD);
}
static int dissect_cellSyncBurstMeasInfoList_CellSyncReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSyncBurstMeasInfoList_CellSyncReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_cellSyncBurstMeasInfoList_CellSyncReconfRqstTDD);
}



static int
dissect_nbap_SynchronisationReportTypeIE(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_synchronisationReportType1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SynchronisationReportTypeIE(tvb, offset, actx, tree, hf_nbap_synchronisationReportType1);
}



static int
dissect_nbap_SynchronisationReportCharacteristicsIE(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_synchronisationReportCharacteristics1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SynchronisationReportCharacteristicsIE(tvb, offset, actx, tree, hf_nbap_synchronisationReportCharacteristics1);
}


static const per_sequence_t CellSyncBurstMeasInfo_CellSyncReconfRqstTDD_sequence[] = {
  { &hf_nbap_cellSyncBurstMeasInfoList_CellSyncReconfRqstTDD, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CellSyncBurstMeasInfoList_CellSyncReconfRqstTDD },
  { &hf_nbap_synchronisationReportType1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_SynchronisationReportTypeIE },
  { &hf_nbap_synchronisationReportCharacteristics1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_SynchronisationReportCharacteristicsIE },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CellSyncBurstMeasInfo_CellSyncReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CellSyncBurstMeasInfo_CellSyncReconfRqstTDD, CellSyncBurstMeasInfo_CellSyncReconfRqstTDD_sequence);

  return offset;
}
static int dissect_id_CellSyncBurstMeasReconfiguration_CellSyncReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSyncBurstMeasInfo_CellSyncReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_id_CellSyncBurstMeasReconfiguration_CellSyncReconfRqstTDD);
}


static const per_sequence_t CellSyncBurstTransInfoItem_CellSyncReconfRqstTDD_sequence[] = {
  { &hf_nbap_cSBTransmissionID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CSBTransmissionID },
  { &hf_nbap_syncFrameNumberToTransmit, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SyncFrameNumber },
  { &hf_nbap_cellSyncBurstCode, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_CellSyncBurstCode },
  { &hf_nbap_cellSyncBurstCodeShift, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_CellSyncBurstCodeShift },
  { &hf_nbap_dlTransPower   , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_Power },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CellSyncBurstTransInfoItem_CellSyncReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CellSyncBurstTransInfoItem_CellSyncReconfRqstTDD, CellSyncBurstTransInfoItem_CellSyncReconfRqstTDD_sequence);

  return offset;
}
static int dissect_CellSyncBurstTransReconfInfo_CellSyncReconfRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSyncBurstTransInfoItem_CellSyncReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_CellSyncBurstTransReconfInfo_CellSyncReconfRqstTDD_item);
}


static const per_sequence_t CellSyncBurstTransReconfInfo_CellSyncReconfRqstTDD_sequence_of[1] = {
  { &hf_nbap_CellSyncBurstTransReconfInfo_CellSyncReconfRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_CellSyncBurstTransInfoItem_CellSyncReconfRqstTDD },
};

static int
dissect_nbap_CellSyncBurstTransReconfInfo_CellSyncReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_CellSyncBurstTransReconfInfo_CellSyncReconfRqstTDD, CellSyncBurstTransReconfInfo_CellSyncReconfRqstTDD_sequence_of,
                                                  1, 10);

  return offset;
}
static int dissect_id_CellSyncBurstTransReconfInfo_CellSyncReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSyncBurstTransReconfInfo_CellSyncReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_id_CellSyncBurstTransReconfInfo_CellSyncReconfRqstTDD);
}



static int
dissect_nbap_C_ID_IE_CellSyncReprtTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_c_ID_CellSyncReprtTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_C_ID_IE_CellSyncReprtTDD(tvb, offset, actx, tree, hf_nbap_c_ID_CellSyncReprtTDD);
}



static int
dissect_nbap_SyncReportTypeIE_CellSyncReprtTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_syncReportType_CellSyncReprtTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SyncReportTypeIE_CellSyncReprtTDD(tvb, offset, actx, tree, hf_nbap_syncReportType_CellSyncReprtTDD);
}


static const per_sequence_t CellSyncInfoItemIE_CellSyncReprtTDD_sequence[] = {
  { &hf_nbap_c_ID_CellSyncReprtTDD, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_C_ID_IE_CellSyncReprtTDD },
  { &hf_nbap_syncReportType_CellSyncReprtTDD, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_SyncReportTypeIE_CellSyncReprtTDD },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CellSyncInfoItemIE_CellSyncReprtTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CellSyncInfoItemIE_CellSyncReprtTDD, CellSyncInfoItemIE_CellSyncReprtTDD_sequence);

  return offset;
}
static int dissect_CellSyncInfo_CellSyncReprtTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSyncInfoItemIE_CellSyncReprtTDD(tvb, offset, actx, tree, hf_nbap_CellSyncInfo_CellSyncReprtTDD_item);
}


static const per_sequence_t CellSyncInfo_CellSyncReprtTDD_sequence_of[1] = {
  { &hf_nbap_CellSyncInfo_CellSyncReprtTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_CellSyncInfoItemIE_CellSyncReprtTDD },
};

static int
dissect_nbap_CellSyncInfo_CellSyncReprtTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_CellSyncInfo_CellSyncReprtTDD, CellSyncInfo_CellSyncReprtTDD_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_id_CellSyncInfo_CellSyncReprtTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSyncInfo_CellSyncReprtTDD(tvb, offset, actx, tree, hf_nbap_id_CellSyncInfo_CellSyncReprtTDD);
}


static const value_string nbap_NCyclesPerSFNperiod_vals[] = {
  {   0, "v1" },
  {   1, "v2" },
  {   2, "v4" },
  {   3, "v8" },
  {   4, "v16" },
  {   5, "v32" },
  {   6, "v64" },
  { 0, NULL }
};


static int
dissect_nbap_NCyclesPerSFNperiod(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, TRUE, 3, NULL);

  return offset;
}
static int dissect_id_NCyclesPerSFNperiod(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NCyclesPerSFNperiod(tvb, offset, actx, tree, hf_nbap_id_NCyclesPerSFNperiod);
}



static int
dissect_nbap_NRepetitionsPerCyclePeriod(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              2U, 10U, NULL, FALSE);

  return offset;
}
static int dissect_id_NRepetitionsPerCyclePeriod(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NRepetitionsPerCyclePeriod(tvb, offset, actx, tree, hf_nbap_id_NRepetitionsPerCyclePeriod);
}


static const per_sequence_t Unsuccessful_cell_InformationRespItem_SyncAdjustmntFailureTDD_sequence[] = {
  { &hf_nbap_c_ID           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_C_ID },
  { &hf_nbap_cause          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Cause },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Unsuccessful_cell_InformationRespItem_SyncAdjustmntFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Unsuccessful_cell_InformationRespItem_SyncAdjustmntFailureTDD, Unsuccessful_cell_InformationRespItem_SyncAdjustmntFailureTDD_sequence);

  return offset;
}
static int dissect_id_Unsuccessful_cell_InformationRespItem_SyncAdjustmntFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Unsuccessful_cell_InformationRespItem_SyncAdjustmntFailureTDD(tvb, offset, actx, tree, hf_nbap_id_Unsuccessful_cell_InformationRespItem_SyncAdjustmntFailureTDD);
}


static const value_string nbap_ReferenceClockAvailability_vals[] = {
  {   0, "available" },
  {   1, "notAvailable" },
  { 0, NULL }
};


static int
dissect_nbap_ReferenceClockAvailability(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_ReferenceClockAvailability(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ReferenceClockAvailability(tvb, offset, actx, tree, hf_nbap_id_ReferenceClockAvailability);
}



static int
dissect_nbap_ReferenceSFNoffset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 255U, NULL, FALSE);

  return offset;
}
static int dissect_id_ReferenceSFNoffset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ReferenceSFNoffset(tvb, offset, actx, tree, hf_nbap_id_ReferenceSFNoffset);
}



static int
dissect_nbap_InformationExchangeID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 1048575U, NULL, FALSE);

  return offset;
}
static int dissect_id_InformationExchangeID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_InformationExchangeID(tvb, offset, actx, tree, hf_nbap_id_InformationExchangeID);
}


static const per_sequence_t Cell_InfEx_Rqst_sequence[] = {
  { &hf_nbap_c_ID           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_C_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Cell_InfEx_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Cell_InfEx_Rqst, Cell_InfEx_Rqst_sequence);

  return offset;
}
static int dissect_cell2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Cell_InfEx_Rqst(tvb, offset, actx, tree, hf_nbap_cell2);
}


static const value_string nbap_InformationExchangeObjectType_InfEx_Rqst_vals[] = {
  {   0, "cell" },
  { 0, NULL }
};

static const per_choice_t InformationExchangeObjectType_InfEx_Rqst_choice[] = {
  {   0, &hf_nbap_cell2          , ASN1_EXTENSION_ROOT    , dissect_nbap_Cell_InfEx_Rqst },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_InformationExchangeObjectType_InfEx_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_InformationExchangeObjectType_InfEx_Rqst, InformationExchangeObjectType_InfEx_Rqst_choice,
                                 NULL);

  return offset;
}
static int dissect_id_InformationExchangeObjectType_InfEx_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_InformationExchangeObjectType_InfEx_Rqst(tvb, offset, actx, tree, hf_nbap_id_InformationExchangeObjectType_InfEx_Rqst);
}



static int
dissect_nbap_ReportPeriodicity_Scaledhour(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 24U, NULL, TRUE);

  return offset;
}
static int dissect_hours(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ReportPeriodicity_Scaledhour(tvb, offset, actx, tree, hf_nbap_hours);
}


static const value_string nbap_InformationReportCharacteristicsType_ReportPeriodicity_vals[] = {
  {   0, "min" },
  {   1, "hours" },
  { 0, NULL }
};

static const per_choice_t InformationReportCharacteristicsType_ReportPeriodicity_choice[] = {
  {   0, &hf_nbap_min            , ASN1_EXTENSION_ROOT    , dissect_nbap_ReportPeriodicity_Scaledmin },
  {   1, &hf_nbap_hours          , ASN1_EXTENSION_ROOT    , dissect_nbap_ReportPeriodicity_Scaledhour },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_InformationReportCharacteristicsType_ReportPeriodicity(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_InformationReportCharacteristicsType_ReportPeriodicity, InformationReportCharacteristicsType_ReportPeriodicity_choice,
                                 NULL);

  return offset;
}
static int dissect_periodic(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_InformationReportCharacteristicsType_ReportPeriodicity(tvb, offset, actx, tree, hf_nbap_periodic);
}


static const value_string nbap_PRCDeviation_vals[] = {
  {   0, "one" },
  {   1, "two" },
  {   2, "five" },
  {   3, "ten" },
  { 0, NULL }
};


static int
dissect_nbap_PRCDeviation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_prcdeviation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PRCDeviation(tvb, offset, actx, tree, hf_nbap_prcdeviation);
}


static const per_sequence_t DGPSThresholds_sequence[] = {
  { &hf_nbap_prcdeviation   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PRCDeviation },
  { &hf_nbap_ie_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DGPSThresholds(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DGPSThresholds, DGPSThresholds_sequence);

  return offset;
}
static int dissect_dgps(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DGPSThresholds(tvb, offset, actx, tree, hf_nbap_dgps);
}


static const value_string nbap_InformationThresholds_vals[] = {
  {   0, "dgps" },
  { 0, NULL }
};

static const per_choice_t InformationThresholds_choice[] = {
  {   0, &hf_nbap_dgps           , ASN1_EXTENSION_ROOT    , dissect_nbap_DGPSThresholds },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_InformationThresholds(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_InformationThresholds, InformationThresholds_choice,
                                 NULL);

  return offset;
}
static int dissect_information_thresholds(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_InformationThresholds(tvb, offset, actx, tree, hf_nbap_information_thresholds);
}


static const per_sequence_t InformationReportCharacteristicsType_OnModification_sequence[] = {
  { &hf_nbap_information_thresholds, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_InformationThresholds },
  { &hf_nbap_ie_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_InformationReportCharacteristicsType_OnModification(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_InformationReportCharacteristicsType_OnModification, InformationReportCharacteristicsType_OnModification_sequence);

  return offset;
}
static int dissect_onModification(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_InformationReportCharacteristicsType_OnModification(tvb, offset, actx, tree, hf_nbap_onModification);
}


static const value_string nbap_InformationReportCharacteristics_vals[] = {
  {   0, "onDemand" },
  {   1, "periodic" },
  {   2, "onModification" },
  { 0, NULL }
};

static const per_choice_t InformationReportCharacteristics_choice[] = {
  {   0, &hf_nbap_onDemand       , ASN1_EXTENSION_ROOT    , dissect_nbap_NULL },
  {   1, &hf_nbap_periodic       , ASN1_EXTENSION_ROOT    , dissect_nbap_InformationReportCharacteristicsType_ReportPeriodicity },
  {   2, &hf_nbap_onModification , ASN1_EXTENSION_ROOT    , dissect_nbap_InformationReportCharacteristicsType_OnModification },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_InformationReportCharacteristics(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_InformationReportCharacteristics, InformationReportCharacteristics_choice,
                                 NULL);

  return offset;
}
static int dissect_id_InformationType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_InformationReportCharacteristics(tvb, offset, actx, tree, hf_nbap_id_InformationType);
}
static int dissect_id_InformationReportCharacteristics(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_InformationReportCharacteristics(tvb, offset, actx, tree, hf_nbap_id_InformationReportCharacteristics);
}



static int
dissect_nbap_GPSTOW(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 604799U, NULL, FALSE);

  return offset;
}
static int dissect_gpstow(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_GPSTOW(tvb, offset, actx, tree, hf_nbap_gpstow);
}


static const value_string nbap_GPS_Status_Health_vals[] = {
  {   0, "udre-scale-1dot0" },
  {   1, "udre-scale-0dot75" },
  {   2, "udre-scale-0dot5" },
  {   3, "udre-scale-0dot3" },
  {   4, "udre-scale-0dot1" },
  {   5, "no-data" },
  {   6, "invalid-data" },
  { 0, NULL }
};


static int
dissect_nbap_GPS_Status_Health(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     7, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_status_health(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_GPS_Status_Health(tvb, offset, actx, tree, hf_nbap_status_health);
}



static int
dissect_nbap_SAT_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 63U, NULL, FALSE);

  return offset;
}
static int dissect_sat_id_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SAT_ID(tvb, offset, actx, tree, hf_nbap_sat_id_nav);
}
static int dissect_sat_id(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SAT_ID(tvb, offset, actx, tree, hf_nbap_sat_id);
}
static int dissect_bad_sat_id(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SAT_ID(tvb, offset, actx, tree, hf_nbap_bad_sat_id);
}



static int
dissect_nbap_BIT_STRING_SIZE_8(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     8, 8, FALSE, NULL);

  return offset;
}
static int dissect_wna_alm(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_8(tvb, offset, actx, tree, hf_nbap_wna_alm);
}
static int dissect_alpha_zero_ionos(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_8(tvb, offset, actx, tree, hf_nbap_alpha_zero_ionos);
}
static int dissect_alpha_one_ionos(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_8(tvb, offset, actx, tree, hf_nbap_alpha_one_ionos);
}
static int dissect_alpha_two_ionos(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_8(tvb, offset, actx, tree, hf_nbap_alpha_two_ionos);
}
static int dissect_alpha_three_ionos(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_8(tvb, offset, actx, tree, hf_nbap_alpha_three_ionos);
}
static int dissect_beta_zero_ionos(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_8(tvb, offset, actx, tree, hf_nbap_beta_zero_ionos);
}
static int dissect_beta_one_ionos(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_8(tvb, offset, actx, tree, hf_nbap_beta_one_ionos);
}
static int dissect_beta_two_ionos(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_8(tvb, offset, actx, tree, hf_nbap_beta_two_ionos);
}
static int dissect_beta_three_ionos(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_8(tvb, offset, actx, tree, hf_nbap_beta_three_ionos);
}
static int dissect_t_gd_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_8(tvb, offset, actx, tree, hf_nbap_t_gd_nav);
}
static int dissect_a_f_2_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_8(tvb, offset, actx, tree, hf_nbap_a_f_2_nav);
}
static int dissect_t_ot_utc(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_8(tvb, offset, actx, tree, hf_nbap_t_ot_utc);
}
static int dissect_delta_t_ls_utc(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_8(tvb, offset, actx, tree, hf_nbap_delta_t_ls_utc);
}
static int dissect_w_n_t_utc(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_8(tvb, offset, actx, tree, hf_nbap_w_n_t_utc);
}
static int dissect_w_n_lsf_utc(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_8(tvb, offset, actx, tree, hf_nbap_w_n_lsf_utc);
}
static int dissect_dn_utc(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_8(tvb, offset, actx, tree, hf_nbap_dn_utc);
}
static int dissect_delta_t_lsf_utc(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_8(tvb, offset, actx, tree, hf_nbap_delta_t_lsf_utc);
}
static int dissect_gps_toa_alm(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_8(tvb, offset, actx, tree, hf_nbap_gps_toa_alm);
}
static int dissect_svhealth_alm(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_8(tvb, offset, actx, tree, hf_nbap_svhealth_alm);
}
static int dissect_iode_dgps(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_8(tvb, offset, actx, tree, hf_nbap_iode_dgps);
}


static const value_string nbap_UDRE_vals[] = {
  {   0, "udre-minusequal-one-m" },
  {   1, "udre-betweenoneandfour-m" },
  {   2, "udre-betweenfourandeight-m" },
  {   3, "udre-greaterequaleight-m" },
  { 0, NULL }
};


static int
dissect_nbap_UDRE(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_udre(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UDRE(tvb, offset, actx, tree, hf_nbap_udre);
}



static int
dissect_nbap_PRC(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              -2047, 2047U, NULL, FALSE);

  return offset;
}
static int dissect_prc(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PRC(tvb, offset, actx, tree, hf_nbap_prc);
}



static int
dissect_nbap_Range_Correction_Rate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              -127, 127U, NULL, FALSE);

  return offset;
}
static int dissect_range_correction_rate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Range_Correction_Rate(tvb, offset, actx, tree, hf_nbap_range_correction_rate);
}


static const per_sequence_t SAT_Info_DGPSCorrections_Item_sequence[] = {
  { &hf_nbap_sat_id         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SAT_ID },
  { &hf_nbap_iode_dgps      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_8 },
  { &hf_nbap_udre           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UDRE },
  { &hf_nbap_prc            , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PRC },
  { &hf_nbap_range_correction_rate, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Range_Correction_Rate },
  { &hf_nbap_ie_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_SAT_Info_DGPSCorrections_Item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_SAT_Info_DGPSCorrections_Item, SAT_Info_DGPSCorrections_Item_sequence);

  return offset;
}
static int dissect_SAT_Info_DGPSCorrections_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SAT_Info_DGPSCorrections_Item(tvb, offset, actx, tree, hf_nbap_SAT_Info_DGPSCorrections_item);
}


static const per_sequence_t SAT_Info_DGPSCorrections_sequence_of[1] = {
  { &hf_nbap_SAT_Info_DGPSCorrections_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_SAT_Info_DGPSCorrections_Item },
};

static int
dissect_nbap_SAT_Info_DGPSCorrections(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_SAT_Info_DGPSCorrections, SAT_Info_DGPSCorrections_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_satelliteinfo(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SAT_Info_DGPSCorrections(tvb, offset, actx, tree, hf_nbap_satelliteinfo);
}


static const per_sequence_t DGPSCorrections_sequence[] = {
  { &hf_nbap_gpstow         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_GPSTOW },
  { &hf_nbap_status_health  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_GPS_Status_Health },
  { &hf_nbap_satelliteinfo  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SAT_Info_DGPSCorrections },
  { &hf_nbap_ie_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DGPSCorrections(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DGPSCorrections, DGPSCorrections_sequence);

  return offset;
}
static int dissect_dgps_corrections(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DGPSCorrections(tvb, offset, actx, tree, hf_nbap_dgps_corrections);
}



static int
dissect_nbap_INTEGER_0_1048575(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 1048575U, NULL, FALSE);

  return offset;
}
static int dissect_tx_tow_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_INTEGER_0_1048575(tvb, offset, actx, tree, hf_nbap_tx_tow_nav);
}



static int
dissect_nbap_BIT_STRING_SIZE_14(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     14, 14, FALSE, NULL);

  return offset;
}
static int dissect_tlm_message_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_14(tvb, offset, actx, tree, hf_nbap_tlm_message_nav);
}
static int dissect_idot_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_14(tvb, offset, actx, tree, hf_nbap_idot_nav);
}



static int
dissect_nbap_BIT_STRING_SIZE_2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     2, 2, FALSE, NULL);

  return offset;
}
static int dissect_tlm_revd_c_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_2(tvb, offset, actx, tree, hf_nbap_tlm_revd_c_nav);
}
static int dissect_ca_or_p_on_l2_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_2(tvb, offset, actx, tree, hf_nbap_ca_or_p_on_l2_nav);
}



static int
dissect_nbap_BIT_STRING_SIZE_22(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     22, 22, FALSE, NULL);

  return offset;
}
static int dissect_ho_word_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_22(tvb, offset, actx, tree, hf_nbap_ho_word_nav);
}
static int dissect_a_f_zero_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_22(tvb, offset, actx, tree, hf_nbap_a_f_zero_nav);
}



static int
dissect_nbap_BIT_STRING_SIZE_10(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     10, 10, FALSE, NULL);

  return offset;
}
static int dissect_w_n_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_10(tvb, offset, actx, tree, hf_nbap_w_n_nav);
}
static int dissect_iodc_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_10(tvb, offset, actx, tree, hf_nbap_iodc_nav);
}



static int
dissect_nbap_BIT_STRING_SIZE_4(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     4, 4, FALSE, NULL);

  return offset;
}
static int dissect_user_range_accuracy_index_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_4(tvb, offset, actx, tree, hf_nbap_user_range_accuracy_index_nav);
}



static int
dissect_nbap_BIT_STRING_SIZE_6(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     6, 6, FALSE, NULL);

  return offset;
}
static int dissect_sv_health_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_6(tvb, offset, actx, tree, hf_nbap_sv_health_nav);
}



static int
dissect_nbap_BIT_STRING_SIZE_1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     1, 1, FALSE, NULL);

  return offset;
}
static int dissect_l2_p_dataflag_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_1(tvb, offset, actx, tree, hf_nbap_l2_p_dataflag_nav);
}
static int dissect_fit_interval_flag_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_1(tvb, offset, actx, tree, hf_nbap_fit_interval_flag_nav);
}



static int
dissect_nbap_BIT_STRING_SIZE_87(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     87, 87, FALSE, NULL);

  return offset;
}
static int dissect_sf1_reserved_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_87(tvb, offset, actx, tree, hf_nbap_sf1_reserved_nav);
}



static int
dissect_nbap_BIT_STRING_SIZE_16(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     16, 16, FALSE, NULL);

  return offset;
}
static int dissect_t_oc_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_16(tvb, offset, actx, tree, hf_nbap_t_oc_nav);
}
static int dissect_a_f_1_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_16(tvb, offset, actx, tree, hf_nbap_a_f_1_nav);
}
static int dissect_c_rs_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_16(tvb, offset, actx, tree, hf_nbap_c_rs_nav);
}
static int dissect_delta_n_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_16(tvb, offset, actx, tree, hf_nbap_delta_n_nav);
}
static int dissect_c_uc_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_16(tvb, offset, actx, tree, hf_nbap_c_uc_nav);
}
static int dissect_c_us_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_16(tvb, offset, actx, tree, hf_nbap_c_us_nav);
}
static int dissect_t_oe_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_16(tvb, offset, actx, tree, hf_nbap_t_oe_nav);
}
static int dissect_c_ic_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_16(tvb, offset, actx, tree, hf_nbap_c_ic_nav);
}
static int dissect_c_is_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_16(tvb, offset, actx, tree, hf_nbap_c_is_nav);
}
static int dissect_c_rc_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_16(tvb, offset, actx, tree, hf_nbap_c_rc_nav);
}
static int dissect_gps_e_alm(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_16(tvb, offset, actx, tree, hf_nbap_gps_e_alm);
}
static int dissect_gps_delta_I_alm(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_16(tvb, offset, actx, tree, hf_nbap_gps_delta_I_alm);
}
static int dissect_omegadot_alm(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_16(tvb, offset, actx, tree, hf_nbap_omegadot_alm);
}



static int
dissect_nbap_BIT_STRING_SIZE_32(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     32, 32, FALSE, NULL);

  return offset;
}
static int dissect_m_zero_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_32(tvb, offset, actx, tree, hf_nbap_m_zero_nav);
}
static int dissect_gps_e_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_32(tvb, offset, actx, tree, hf_nbap_gps_e_nav);
}
static int dissect_a_sqrt_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_32(tvb, offset, actx, tree, hf_nbap_a_sqrt_nav);
}
static int dissect_omega_zero_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_32(tvb, offset, actx, tree, hf_nbap_omega_zero_nav);
}
static int dissect_i_zero_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_32(tvb, offset, actx, tree, hf_nbap_i_zero_nav);
}
static int dissect_gps_omega_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_32(tvb, offset, actx, tree, hf_nbap_gps_omega_nav);
}
static int dissect_a_zero_utc(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_32(tvb, offset, actx, tree, hf_nbap_a_zero_utc);
}



static int
dissect_nbap_BIT_STRING_SIZE_5(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     5, 5, FALSE, NULL);

  return offset;
}
static int dissect_aodo_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_5(tvb, offset, actx, tree, hf_nbap_aodo_nav);
}



static int
dissect_nbap_BIT_STRING_SIZE_24(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     24, 24, FALSE, NULL);

  return offset;
}
static int dissect_omegadot_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_24(tvb, offset, actx, tree, hf_nbap_omegadot_nav);
}
static int dissect_a_one_utc(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_24(tvb, offset, actx, tree, hf_nbap_a_one_utc);
}
static int dissect_gps_a_sqrt_alm(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_24(tvb, offset, actx, tree, hf_nbap_gps_a_sqrt_alm);
}
static int dissect_omegazero_alm(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_24(tvb, offset, actx, tree, hf_nbap_omegazero_alm);
}
static int dissect_m_zero_alm(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_24(tvb, offset, actx, tree, hf_nbap_m_zero_alm);
}
static int dissect_gps_omega_alm(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_24(tvb, offset, actx, tree, hf_nbap_gps_omega_alm);
}



static int
dissect_nbap_BIT_STRING_SIZE_20(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     20, 20, FALSE, NULL);

  return offset;
}
static int dissect_spare_zero_fill(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_20(tvb, offset, actx, tree, hf_nbap_spare_zero_fill);
}


static const per_sequence_t GPS_NavandRecovery_Item_sequence[] = {
  { &hf_nbap_tx_tow_nav     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_INTEGER_0_1048575 },
  { &hf_nbap_sat_id_nav     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SAT_ID },
  { &hf_nbap_tlm_message_nav, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_14 },
  { &hf_nbap_tlm_revd_c_nav , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_2 },
  { &hf_nbap_ho_word_nav    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_22 },
  { &hf_nbap_w_n_nav        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_10 },
  { &hf_nbap_ca_or_p_on_l2_nav, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_2 },
  { &hf_nbap_user_range_accuracy_index_nav, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_4 },
  { &hf_nbap_sv_health_nav  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_6 },
  { &hf_nbap_iodc_nav       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_10 },
  { &hf_nbap_l2_p_dataflag_nav, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_1 },
  { &hf_nbap_sf1_reserved_nav, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_87 },
  { &hf_nbap_t_gd_nav       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_8 },
  { &hf_nbap_t_oc_nav       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_16 },
  { &hf_nbap_a_f_2_nav      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_8 },
  { &hf_nbap_a_f_1_nav      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_16 },
  { &hf_nbap_a_f_zero_nav   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_22 },
  { &hf_nbap_c_rs_nav       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_16 },
  { &hf_nbap_delta_n_nav    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_16 },
  { &hf_nbap_m_zero_nav     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_32 },
  { &hf_nbap_c_uc_nav       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_16 },
  { &hf_nbap_gps_e_nav      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_32 },
  { &hf_nbap_c_us_nav       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_16 },
  { &hf_nbap_a_sqrt_nav     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_32 },
  { &hf_nbap_t_oe_nav       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_16 },
  { &hf_nbap_fit_interval_flag_nav, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_1 },
  { &hf_nbap_aodo_nav       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_5 },
  { &hf_nbap_c_ic_nav       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_16 },
  { &hf_nbap_omega_zero_nav , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_32 },
  { &hf_nbap_c_is_nav       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_16 },
  { &hf_nbap_i_zero_nav     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_32 },
  { &hf_nbap_c_rc_nav       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_16 },
  { &hf_nbap_gps_omega_nav  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_32 },
  { &hf_nbap_omegadot_nav   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_24 },
  { &hf_nbap_idot_nav       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_14 },
  { &hf_nbap_spare_zero_fill, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_20 },
  { &hf_nbap_ie_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_GPS_NavandRecovery_Item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_GPS_NavandRecovery_Item, GPS_NavandRecovery_Item_sequence);

  return offset;
}
static int dissect_GPS_NavigationModel_and_TimeRecovery_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_GPS_NavandRecovery_Item(tvb, offset, actx, tree, hf_nbap_GPS_NavigationModel_and_TimeRecovery_item);
}


static const per_sequence_t GPS_NavigationModel_and_TimeRecovery_sequence_of[1] = {
  { &hf_nbap_GPS_NavigationModel_and_TimeRecovery_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_GPS_NavandRecovery_Item },
};

static int
dissect_nbap_GPS_NavigationModel_and_TimeRecovery(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_GPS_NavigationModel_and_TimeRecovery, GPS_NavigationModel_and_TimeRecovery_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_gps_navandrecovery(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_GPS_NavigationModel_and_TimeRecovery(tvb, offset, actx, tree, hf_nbap_gps_navandrecovery);
}


static const per_sequence_t GPS_Ionospheric_Model_sequence[] = {
  { &hf_nbap_alpha_zero_ionos, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_8 },
  { &hf_nbap_alpha_one_ionos, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_8 },
  { &hf_nbap_alpha_two_ionos, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_8 },
  { &hf_nbap_alpha_three_ionos, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_8 },
  { &hf_nbap_beta_zero_ionos, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_8 },
  { &hf_nbap_beta_one_ionos , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_8 },
  { &hf_nbap_beta_two_ionos , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_8 },
  { &hf_nbap_beta_three_ionos, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_8 },
  { &hf_nbap_ie_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_GPS_Ionospheric_Model(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_GPS_Ionospheric_Model, GPS_Ionospheric_Model_sequence);

  return offset;
}
static int dissect_gps_ionos_model(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_GPS_Ionospheric_Model(tvb, offset, actx, tree, hf_nbap_gps_ionos_model);
}


static const per_sequence_t GPS_UTC_Model_sequence[] = {
  { &hf_nbap_a_one_utc      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_24 },
  { &hf_nbap_a_zero_utc     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_32 },
  { &hf_nbap_t_ot_utc       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_8 },
  { &hf_nbap_delta_t_ls_utc , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_8 },
  { &hf_nbap_w_n_t_utc      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_8 },
  { &hf_nbap_w_n_lsf_utc    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_8 },
  { &hf_nbap_dn_utc         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_8 },
  { &hf_nbap_delta_t_lsf_utc, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_8 },
  { &hf_nbap_ie_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_GPS_UTC_Model(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_GPS_UTC_Model, GPS_UTC_Model_sequence);

  return offset;
}
static int dissect_gps_utc_model(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_GPS_UTC_Model(tvb, offset, actx, tree, hf_nbap_gps_utc_model);
}



static int
dissect_nbap_DATA_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 3U, NULL, FALSE);

  return offset;
}
static int dissect_data_id(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DATA_ID(tvb, offset, actx, tree, hf_nbap_data_id);
}



static int
dissect_nbap_BIT_STRING_SIZE_11(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     11, 11, FALSE, NULL);

  return offset;
}
static int dissect_gps_af_zero_alm(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_11(tvb, offset, actx, tree, hf_nbap_gps_af_zero_alm);
}
static int dissect_gps_af_one_alm(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_11(tvb, offset, actx, tree, hf_nbap_gps_af_one_alm);
}


static const per_sequence_t SAT_Info_Almanac_Item_sequence[] = {
  { &hf_nbap_data_id        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DATA_ID },
  { &hf_nbap_sat_id         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SAT_ID },
  { &hf_nbap_gps_e_alm      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_16 },
  { &hf_nbap_gps_toa_alm    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_8 },
  { &hf_nbap_gps_delta_I_alm, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_16 },
  { &hf_nbap_omegadot_alm   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_16 },
  { &hf_nbap_svhealth_alm   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_8 },
  { &hf_nbap_gps_a_sqrt_alm , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_24 },
  { &hf_nbap_omegazero_alm  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_24 },
  { &hf_nbap_m_zero_alm     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_24 },
  { &hf_nbap_gps_omega_alm  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_24 },
  { &hf_nbap_gps_af_zero_alm, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_11 },
  { &hf_nbap_gps_af_one_alm , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_11 },
  { &hf_nbap_ie_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_SAT_Info_Almanac_Item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_SAT_Info_Almanac_Item, SAT_Info_Almanac_Item_sequence);

  return offset;
}
static int dissect_SAT_Info_Almanac_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SAT_Info_Almanac_Item(tvb, offset, actx, tree, hf_nbap_SAT_Info_Almanac_item);
}


static const per_sequence_t SAT_Info_Almanac_sequence_of[1] = {
  { &hf_nbap_SAT_Info_Almanac_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_SAT_Info_Almanac_Item },
};

static int
dissect_nbap_SAT_Info_Almanac(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_SAT_Info_Almanac, SAT_Info_Almanac_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_sat_info_almanac(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SAT_Info_Almanac(tvb, offset, actx, tree, hf_nbap_sat_info_almanac);
}



static int
dissect_nbap_BIT_STRING_SIZE_364(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     364, 364, FALSE, NULL);

  return offset;
}
static int dissect_sVGlobalHealth_alm(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BIT_STRING_SIZE_364(tvb, offset, actx, tree, hf_nbap_sVGlobalHealth_alm);
}


static const per_sequence_t GPS_Almanac_sequence[] = {
  { &hf_nbap_wna_alm        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_8 },
  { &hf_nbap_sat_info_almanac, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SAT_Info_Almanac },
  { &hf_nbap_sVGlobalHealth_alm, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_BIT_STRING_SIZE_364 },
  { &hf_nbap_ie_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_GPS_Almanac(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_GPS_Almanac, GPS_Almanac_sequence);

  return offset;
}
static int dissect_gps_almanac(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_GPS_Almanac(tvb, offset, actx, tree, hf_nbap_gps_almanac);
}


static const per_sequence_t SAT_Info_RealTime_Integrity_Item_sequence[] = {
  { &hf_nbap_bad_sat_id     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SAT_ID },
  { &hf_nbap_ie_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_SAT_Info_RealTime_Integrity_Item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_SAT_Info_RealTime_Integrity_Item, SAT_Info_RealTime_Integrity_Item_sequence);

  return offset;
}
static int dissect_SATInfo_RealTime_Integrity_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SAT_Info_RealTime_Integrity_Item(tvb, offset, actx, tree, hf_nbap_SATInfo_RealTime_Integrity_item);
}


static const per_sequence_t SATInfo_RealTime_Integrity_sequence_of[1] = {
  { &hf_nbap_SATInfo_RealTime_Integrity_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_SAT_Info_RealTime_Integrity_Item },
};

static int
dissect_nbap_SATInfo_RealTime_Integrity(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_SATInfo_RealTime_Integrity, SATInfo_RealTime_Integrity_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_sat_info(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SATInfo_RealTime_Integrity(tvb, offset, actx, tree, hf_nbap_sat_info);
}


static const per_sequence_t GPSBadSat_Info_RealTime_Integrity_sequence[] = {
  { &hf_nbap_sat_info       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SATInfo_RealTime_Integrity },
  { &hf_nbap_ie_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_GPSBadSat_Info_RealTime_Integrity(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_GPSBadSat_Info_RealTime_Integrity, GPSBadSat_Info_RealTime_Integrity_sequence);

  return offset;
}
static int dissect_bad_satellites(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_GPSBadSat_Info_RealTime_Integrity(tvb, offset, actx, tree, hf_nbap_bad_satellites);
}


static const value_string nbap_GPS_RealTime_Integrity_vals[] = {
  {   0, "bad-satellites" },
  {   1, "no-bad-satellites" },
  { 0, NULL }
};

static const per_choice_t GPS_RealTime_Integrity_choice[] = {
  {   0, &hf_nbap_bad_satellites , ASN1_NO_EXTENSIONS     , dissect_nbap_GPSBadSat_Info_RealTime_Integrity },
  {   1, &hf_nbap_no_bad_satellites, ASN1_NO_EXTENSIONS     , dissect_nbap_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_GPS_RealTime_Integrity(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_GPS_RealTime_Integrity, GPS_RealTime_Integrity_choice,
                                 NULL);

  return offset;
}
static int dissect_gps_rt_integrity(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_GPS_RealTime_Integrity(tvb, offset, actx, tree, hf_nbap_gps_rt_integrity);
}


static const value_string nbap_T_latitudeSign_vals[] = {
  {   0, "north" },
  {   1, "south" },
  { 0, NULL }
};


static int
dissect_nbap_T_latitudeSign(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_latitudeSign(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_T_latitudeSign(tvb, offset, actx, tree, hf_nbap_latitudeSign);
}



static int
dissect_nbap_INTEGER_0_8388607(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 8388607U, NULL, FALSE);

  return offset;
}
static int dissect_latitude(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_INTEGER_0_8388607(tvb, offset, actx, tree, hf_nbap_latitude);
}



static int
dissect_nbap_INTEGER_M8388608_8388607(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              -8388608, 8388607U, NULL, FALSE);

  return offset;
}
static int dissect_longitude(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_INTEGER_M8388608_8388607(tvb, offset, actx, tree, hf_nbap_longitude);
}


static const value_string nbap_T_directionOfAltitude_vals[] = {
  {   0, "height" },
  {   1, "depth" },
  { 0, NULL }
};


static int
dissect_nbap_T_directionOfAltitude(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_directionOfAltitude(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_T_directionOfAltitude(tvb, offset, actx, tree, hf_nbap_directionOfAltitude);
}


static const per_sequence_t GPS_RX_POS_sequence[] = {
  { &hf_nbap_latitudeSign   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_T_latitudeSign },
  { &hf_nbap_latitude       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_INTEGER_0_8388607 },
  { &hf_nbap_longitude      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_INTEGER_M8388608_8388607 },
  { &hf_nbap_directionOfAltitude, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_T_directionOfAltitude },
  { &hf_nbap_altitude       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_INTEGER_0_32767 },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_GPS_RX_POS(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_GPS_RX_POS, GPS_RX_POS_sequence);

  return offset;
}
static int dissect_gpsrxpos(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_GPS_RX_POS(tvb, offset, actx, tree, hf_nbap_gpsrxpos);
}


static const per_sequence_t RequestedDataValue_sequence[] = {
  { &hf_nbap_dgps_corrections, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DGPSCorrections },
  { &hf_nbap_gps_navandrecovery, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_GPS_NavigationModel_and_TimeRecovery },
  { &hf_nbap_gps_ionos_model, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_GPS_Ionospheric_Model },
  { &hf_nbap_gps_utc_model  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_GPS_UTC_Model },
  { &hf_nbap_gps_almanac    , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_GPS_Almanac },
  { &hf_nbap_gps_rt_integrity, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_GPS_RealTime_Integrity },
  { &hf_nbap_gpsrxpos       , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_GPS_RX_POS },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RequestedDataValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RequestedDataValue, RequestedDataValue_sequence);

  return offset;
}
static int dissect_requestedDataValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RequestedDataValue(tvb, offset, actx, tree, hf_nbap_requestedDataValue);
}
static int dissect_requesteddataValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RequestedDataValue(tvb, offset, actx, tree, hf_nbap_requesteddataValue);
}


static const per_sequence_t Cell_InfEx_Rsp_sequence[] = {
  { &hf_nbap_requestedDataValue, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RequestedDataValue },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Cell_InfEx_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Cell_InfEx_Rsp, Cell_InfEx_Rsp_sequence);

  return offset;
}
static int dissect_cell3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Cell_InfEx_Rsp(tvb, offset, actx, tree, hf_nbap_cell3);
}


static const value_string nbap_InformationExchangeObjectType_InfEx_Rsp_vals[] = {
  {   0, "cell" },
  { 0, NULL }
};

static const per_choice_t InformationExchangeObjectType_InfEx_Rsp_choice[] = {
  {   0, &hf_nbap_cell3          , ASN1_EXTENSION_ROOT    , dissect_nbap_Cell_InfEx_Rsp },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_InformationExchangeObjectType_InfEx_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_InformationExchangeObjectType_InfEx_Rsp, InformationExchangeObjectType_InfEx_Rsp_choice,
                                 NULL);

  return offset;
}
static int dissect_id_InformationExchangeObjectType_InfEx_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_InformationExchangeObjectType_InfEx_Rsp(tvb, offset, actx, tree, hf_nbap_id_InformationExchangeObjectType_InfEx_Rsp);
}


static const per_sequence_t InformationAvailable_sequence[] = {
  { &hf_nbap_requesteddataValue, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RequestedDataValue },
  { &hf_nbap_ie_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_InformationAvailable(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_InformationAvailable, InformationAvailable_sequence);

  return offset;
}
static int dissect_informationAvailable(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_InformationAvailable(tvb, offset, actx, tree, hf_nbap_informationAvailable);
}



static int
dissect_nbap_InformationnotAvailable(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_null(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_informationnotAvailable(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_InformationnotAvailable(tvb, offset, actx, tree, hf_nbap_informationnotAvailable);
}


static const value_string nbap_RequestedDataValueInformation_vals[] = {
  {   0, "informationAvailable" },
  {   1, "informationnotAvailable" },
  { 0, NULL }
};

static const per_choice_t RequestedDataValueInformation_choice[] = {
  {   0, &hf_nbap_informationAvailable, ASN1_NO_EXTENSIONS     , dissect_nbap_InformationAvailable },
  {   1, &hf_nbap_informationnotAvailable, ASN1_NO_EXTENSIONS     , dissect_nbap_InformationnotAvailable },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_RequestedDataValueInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_RequestedDataValueInformation, RequestedDataValueInformation_choice,
                                 NULL);

  return offset;
}
static int dissect_requestedDataValueInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RequestedDataValueInformation(tvb, offset, actx, tree, hf_nbap_requestedDataValueInformation);
}


static const per_sequence_t Cell_Inf_Rprt_sequence[] = {
  { &hf_nbap_requestedDataValueInformation, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RequestedDataValueInformation },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Cell_Inf_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Cell_Inf_Rprt, Cell_Inf_Rprt_sequence);

  return offset;
}
static int dissect_cell4(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Cell_Inf_Rprt(tvb, offset, actx, tree, hf_nbap_cell4);
}


static const value_string nbap_InformationExchangeObjectType_InfEx_Rprt_vals[] = {
  {   0, "cell" },
  { 0, NULL }
};

static const per_choice_t InformationExchangeObjectType_InfEx_Rprt_choice[] = {
  {   0, &hf_nbap_cell4          , ASN1_EXTENSION_ROOT    , dissect_nbap_Cell_Inf_Rprt },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_InformationExchangeObjectType_InfEx_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_InformationExchangeObjectType_InfEx_Rprt, InformationExchangeObjectType_InfEx_Rprt_choice,
                                 NULL);

  return offset;
}
static int dissect_id_InformationExchangeObjectType_InfEx_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_InformationExchangeObjectType_InfEx_Rprt(tvb, offset, actx, tree, hf_nbap_id_InformationExchangeObjectType_InfEx_Rprt);
}


static const value_string nbap_T_iP_SpacingFDD_vals[] = {
  {   0, "sp5" },
  {   1, "sp7" },
  {   2, "sp10" },
  {   3, "sp15" },
  {   4, "sp20" },
  {   5, "sp30" },
  {   6, "sp40" },
  {   7, "sp50" },
  { 0, NULL }
};


static int
dissect_nbap_T_iP_SpacingFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_iP_SpacingFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_T_iP_SpacingFDD(tvb, offset, actx, tree, hf_nbap_iP_SpacingFDD);
}


static const value_string nbap_T_iP_Length_vals[] = {
  {   0, "len5" },
  {   1, "len10" },
  { 0, NULL }
};


static int
dissect_nbap_T_iP_Length(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_iP_Length(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_T_iP_Length(tvb, offset, actx, tree, hf_nbap_iP_Length);
}



static int
dissect_nbap_INTEGER_10_25(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              10U, 25U, NULL, FALSE);

  return offset;
}
static int dissect_burstLength(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_INTEGER_10_25(tvb, offset, actx, tree, hf_nbap_burstLength);
}



static int
dissect_nbap_INTEGER_1_16(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 16U, NULL, FALSE);

  return offset;
}
static int dissect_burstFreq(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_INTEGER_1_16(tvb, offset, actx, tree, hf_nbap_burstFreq);
}


static const per_sequence_t BurstModeParams_sequence[] = {
  { &hf_nbap_burstStart     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_INTEGER_0_15 },
  { &hf_nbap_burstLength    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_INTEGER_10_25 },
  { &hf_nbap_burstFreq      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_INTEGER_1_16 },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_BurstModeParams(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_BurstModeParams, BurstModeParams_sequence);

  return offset;
}
static int dissect_burstModeParams(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BurstModeParams(tvb, offset, actx, tree, hf_nbap_burstModeParams);
}



static int
dissect_nbap_INTEGER_0_9(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 9U, NULL, FALSE);

  return offset;
}
static int dissect_iP_Offset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_INTEGER_0_9(tvb, offset, actx, tree, hf_nbap_iP_Offset);
}


static const per_sequence_t IPDL_FDD_Parameters_sequence[] = {
  { &hf_nbap_iP_SpacingFDD  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_T_iP_SpacingFDD },
  { &hf_nbap_iP_Length      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_T_iP_Length },
  { &hf_nbap_seed           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_INTEGER_0_63 },
  { &hf_nbap_burstModeParams, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_BurstModeParams },
  { &hf_nbap_iP_Offset      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_INTEGER_0_9 },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_IPDL_FDD_Parameters(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_IPDL_FDD_Parameters, IPDL_FDD_Parameters_sequence);

  return offset;
}
static int dissect_iPDL_FDD_Parameters(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_IPDL_FDD_Parameters(tvb, offset, actx, tree, hf_nbap_iPDL_FDD_Parameters);
}


static const value_string nbap_IPDL_Indicator_vals[] = {
  {   0, "active" },
  {   1, "inactive" },
  { 0, NULL }
};


static int
dissect_nbap_IPDL_Indicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_iPDL_Indicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_IPDL_Indicator(tvb, offset, actx, tree, hf_nbap_iPDL_Indicator);
}


static const per_sequence_t IPDLParameter_Information_Cell_ReconfRqstFDD_sequence[] = {
  { &hf_nbap_iPDL_FDD_Parameters, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_IPDL_FDD_Parameters },
  { &hf_nbap_iPDL_Indicator , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_IPDL_Indicator },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_IPDLParameter_Information_Cell_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_IPDLParameter_Information_Cell_ReconfRqstFDD, IPDLParameter_Information_Cell_ReconfRqstFDD_sequence);

  return offset;
}
static int dissect_id_IPDLParameter_Information_Cell_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_IPDLParameter_Information_Cell_ReconfRqstFDD(tvb, offset, actx, tree, hf_nbap_id_IPDLParameter_Information_Cell_ReconfRqstFDD);
}


static const per_sequence_t IPDLParameter_Information_Cell_SetupRqstFDD_sequence[] = {
  { &hf_nbap_iPDL_FDD_Parameters, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_IPDL_FDD_Parameters },
  { &hf_nbap_iPDL_Indicator , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_IPDL_Indicator },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_IPDLParameter_Information_Cell_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_IPDLParameter_Information_Cell_SetupRqstFDD, IPDLParameter_Information_Cell_SetupRqstFDD_sequence);

  return offset;
}
static int dissect_id_IPDLParameter_Information_Cell_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_IPDLParameter_Information_Cell_SetupRqstFDD(tvb, offset, actx, tree, hf_nbap_id_IPDLParameter_Information_Cell_SetupRqstFDD);
}


static const value_string nbap_T_iP_SpacingTDD_vals[] = {
  {   0, "sp30" },
  {   1, "sp40" },
  {   2, "sp50" },
  {   3, "sp70" },
  {   4, "sp100" },
  { 0, NULL }
};


static int
dissect_nbap_T_iP_SpacingTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     5, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_iP_SpacingTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_T_iP_SpacingTDD(tvb, offset, actx, tree, hf_nbap_iP_SpacingTDD);
}



static int
dissect_nbap_INTEGER_0_14(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 14U, NULL, FALSE);

  return offset;
}
static int dissect_iP_Slot(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_INTEGER_0_14(tvb, offset, actx, tree, hf_nbap_iP_Slot);
}


static const value_string nbap_T_iP_PCCPCH_vals[] = {
  {   0, "switchOff-1-Frame" },
  {   1, "switchOff-2-Frames" },
  { 0, NULL }
};


static int
dissect_nbap_T_iP_PCCPCH(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_iP_PCCPCH(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_T_iP_PCCPCH(tvb, offset, actx, tree, hf_nbap_iP_PCCPCH);
}


static const per_sequence_t IPDL_TDD_Parameters_sequence[] = {
  { &hf_nbap_iP_SpacingTDD  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_T_iP_SpacingTDD },
  { &hf_nbap_iP_Start       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_INTEGER_0_4095 },
  { &hf_nbap_iP_Slot        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_INTEGER_0_14 },
  { &hf_nbap_iP_PCCPCH      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_T_iP_PCCPCH },
  { &hf_nbap_burstModeParams, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_BurstModeParams },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_IPDL_TDD_Parameters(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_IPDL_TDD_Parameters, IPDL_TDD_Parameters_sequence);

  return offset;
}
static int dissect_iPDL_TDD_Parameters(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_IPDL_TDD_Parameters(tvb, offset, actx, tree, hf_nbap_iPDL_TDD_Parameters);
}


static const per_sequence_t IPDLParameter_Information_Cell_SetupRqstTDD_sequence[] = {
  { &hf_nbap_iPDL_TDD_Parameters, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_IPDL_TDD_Parameters },
  { &hf_nbap_iPDL_Indicator , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_IPDL_Indicator },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_IPDLParameter_Information_Cell_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_IPDLParameter_Information_Cell_SetupRqstTDD, IPDLParameter_Information_Cell_SetupRqstTDD_sequence);

  return offset;
}
static int dissect_id_IPDLParameter_Information_Cell_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_IPDLParameter_Information_Cell_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_id_IPDLParameter_Information_Cell_ReconfRqstTDD);
}
static int dissect_id_IPDLParameter_Information_Cell_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_IPDLParameter_Information_Cell_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_id_IPDLParameter_Information_Cell_SetupRqstTDD);
}



static int
dissect_nbap_TimeSlotLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 6U, NULL, FALSE);

  return offset;
}
static int dissect_id_TimeSlotLCR_CM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TimeSlotLCR(tvb, offset, actx, tree, hf_nbap_id_TimeSlotLCR_CM_Rqst);
}
static int dissect_timeSlotLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TimeSlotLCR(tvb, offset, actx, tree, hf_nbap_timeSlotLCR);
}
static int dissect_timeslotLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TimeSlotLCR(tvb, offset, actx, tree, hf_nbap_timeslotLCR);
}
static int dissect_timeSlot1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TimeSlotLCR(tvb, offset, actx, tree, hf_nbap_timeSlot1);
}


static const value_string nbap_MidambleAllocationMode_vals[] = {
  {   0, "defaultMidamble" },
  {   1, "commonMidamble" },
  {   2, "uESpecificMidamble" },
  { 0, NULL }
};


static int
dissect_nbap_MidambleAllocationMode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_midambleAllocationMode3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MidambleAllocationMode(tvb, offset, actx, tree, hf_nbap_midambleAllocationMode3);
}


static const value_string nbap_MidambleConfigurationLCR_vals[] = {
  {   0, "v2" },
  {   1, "v4" },
  {   2, "v6" },
  {   3, "v8" },
  {   4, "v10" },
  {   5, "v12" },
  {   6, "v14" },
  {   7, "v16" },
  { 0, NULL }
};


static int
dissect_nbap_MidambleConfigurationLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_midambleConfigurationLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MidambleConfigurationLCR(tvb, offset, actx, tree, hf_nbap_midambleConfigurationLCR);
}


static const per_sequence_t MidambleShiftLCR_sequence[] = {
  { &hf_nbap_midambleAllocationMode3, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MidambleAllocationMode },
  { &hf_nbap_midambleShift  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_MidambleShiftLong },
  { &hf_nbap_midambleConfigurationLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MidambleConfigurationLCR },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_MidambleShiftLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_MidambleShiftLCR, MidambleShiftLCR_sequence);

  return offset;
}
static int dissect_midambleShiftLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MidambleShiftLCR(tvb, offset, actx, tree, hf_nbap_midambleShiftLCR);
}
static int dissect_midambleShiftAndBurstType1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MidambleShiftLCR(tvb, offset, actx, tree, hf_nbap_midambleShiftAndBurstType1);
}


static const value_string nbap_Modulation_vals[] = {
  {   0, "qPSK" },
  {   1, "eightPSK" },
  { 0, NULL }
};


static int
dissect_nbap_Modulation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_modulation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Modulation(tvb, offset, actx, tree, hf_nbap_modulation);
}


static const per_sequence_t TDD_ChannelisationCodeLCR_sequence[] = {
  { &hf_nbap_tDD_ChannelisationCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_ChannelisationCode },
  { &hf_nbap_modulation     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Modulation },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_TDD_ChannelisationCodeLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_TDD_ChannelisationCodeLCR, TDD_ChannelisationCodeLCR_sequence);

  return offset;
}
static int dissect_tdd_ChannelisationCodeLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TDD_ChannelisationCodeLCR(tvb, offset, actx, tree, hf_nbap_tdd_ChannelisationCodeLCR);
}
static int dissect_second_TDD_ChannelisationCodeLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TDD_ChannelisationCodeLCR(tvb, offset, actx, tree, hf_nbap_second_TDD_ChannelisationCodeLCR);
}



static int
dissect_nbap_QPSK_DL_DPCH_TimeSlotFormatTDD_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 24U, NULL, TRUE);

  return offset;
}
static int dissect_qPSK(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_QPSK_DL_DPCH_TimeSlotFormatTDD_LCR(tvb, offset, actx, tree, hf_nbap_qPSK);
}



static int
dissect_nbap_EightPSK_DL_DPCH_TimeSlotFormatTDD_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 24U, NULL, TRUE);

  return offset;
}
static int dissect_eightPSK(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_EightPSK_DL_DPCH_TimeSlotFormatTDD_LCR(tvb, offset, actx, tree, hf_nbap_eightPSK);
}


static const value_string nbap_TDD_DL_DPCH_TimeSlotFormat_LCR_vals[] = {
  {   0, "qPSK" },
  {   1, "eightPSK" },
  { 0, NULL }
};

static const per_choice_t TDD_DL_DPCH_TimeSlotFormat_LCR_choice[] = {
  {   0, &hf_nbap_qPSK           , ASN1_EXTENSION_ROOT    , dissect_nbap_QPSK_DL_DPCH_TimeSlotFormatTDD_LCR },
  {   1, &hf_nbap_eightPSK       , ASN1_EXTENSION_ROOT    , dissect_nbap_EightPSK_DL_DPCH_TimeSlotFormatTDD_LCR },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_TDD_DL_DPCH_TimeSlotFormat_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_TDD_DL_DPCH_TimeSlotFormat_LCR, TDD_DL_DPCH_TimeSlotFormat_LCR_choice,
                                 NULL);

  return offset;
}
static int dissect_id_DL_DPCH_TimeSlotFormat_LCR_ModifyItem_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TDD_DL_DPCH_TimeSlotFormat_LCR(tvb, offset, actx, tree, hf_nbap_id_DL_DPCH_TimeSlotFormat_LCR_ModifyItem_RL_ReconfPrepTDD);
}
static int dissect_s_CCPCH_TimeSlotFormat_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TDD_DL_DPCH_TimeSlotFormat_LCR(tvb, offset, actx, tree, hf_nbap_s_CCPCH_TimeSlotFormat_LCR);
}
static int dissect_tdd_DL_DPCH_TimeSlotFormat_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TDD_DL_DPCH_TimeSlotFormat_LCR(tvb, offset, actx, tree, hf_nbap_tdd_DL_DPCH_TimeSlotFormat_LCR);
}


static const per_sequence_t TDD_DL_Code_LCR_InformationItem_sequence[] = {
  { &hf_nbap_dPCH_ID        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DPCH_ID },
  { &hf_nbap_tdd_ChannelisationCodeLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_ChannelisationCodeLCR },
  { &hf_nbap_tdd_DL_DPCH_TimeSlotFormat_LCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_DL_DPCH_TimeSlotFormat_LCR },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_TDD_DL_Code_LCR_InformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_TDD_DL_Code_LCR_InformationItem, TDD_DL_Code_LCR_InformationItem_sequence);

  return offset;
}
static int dissect_TDD_DL_Code_LCR_Information_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TDD_DL_Code_LCR_InformationItem(tvb, offset, actx, tree, hf_nbap_TDD_DL_Code_LCR_Information_item);
}


static const per_sequence_t TDD_DL_Code_LCR_Information_sequence_of[1] = {
  { &hf_nbap_TDD_DL_Code_LCR_Information_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_DL_Code_LCR_InformationItem },
};

static int
dissect_nbap_TDD_DL_Code_LCR_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_TDD_DL_Code_LCR_Information, TDD_DL_Code_LCR_Information_sequence_of,
                                                  1, 240);

  return offset;
}
static int dissect_dL_Code_LCR_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TDD_DL_Code_LCR_Information(tvb, offset, actx, tree, hf_nbap_dL_Code_LCR_Information);
}


static const per_sequence_t DL_TimeslotLCR_InformationItem_sequence[] = {
  { &hf_nbap_timeSlotLCR    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlotLCR },
  { &hf_nbap_midambleShiftLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MidambleShiftLCR },
  { &hf_nbap_tFCI_Presence  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TFCI_Presence },
  { &hf_nbap_dL_Code_LCR_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_DL_Code_LCR_Information },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_TimeslotLCR_InformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_TimeslotLCR_InformationItem, DL_TimeslotLCR_InformationItem_sequence);

  return offset;
}
static int dissect_DL_TimeslotLCR_Information_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_TimeslotLCR_InformationItem(tvb, offset, actx, tree, hf_nbap_DL_TimeslotLCR_Information_item);
}


static const per_sequence_t DL_TimeslotLCR_Information_sequence_of[1] = {
  { &hf_nbap_DL_TimeslotLCR_Information_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DL_TimeslotLCR_InformationItem },
};

static int
dissect_nbap_DL_TimeslotLCR_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DL_TimeslotLCR_Information, DL_TimeslotLCR_Information_sequence_of,
                                                  1, 6);

  return offset;
}
static int dissect_dL_TimeslotLCR_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_TimeslotLCR_Information(tvb, offset, actx, tree, hf_nbap_dL_TimeslotLCR_Information);
}
static int dissect_dL_Timeslot_InformationLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_TimeslotLCR_Information(tvb, offset, actx, tree, hf_nbap_dL_Timeslot_InformationLCR);
}


static const per_sequence_t DL_DPCH_LCR_Information_RL_SetupRqstTDD_sequence[] = {
  { &hf_nbap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionPeriod },
  { &hf_nbap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionLength },
  { &hf_nbap_tdd_DPCHOffset , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_DPCHOffset },
  { &hf_nbap_dL_TimeslotLCR_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_TimeslotLCR_Information },
  { &hf_nbap_tstdIndicator  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TSTD_Indicator },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_DPCH_LCR_Information_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_DPCH_LCR_Information_RL_SetupRqstTDD, DL_DPCH_LCR_Information_RL_SetupRqstTDD_sequence);

  return offset;
}
static int dissect_id_DL_DPCH_LCR_Information_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_DPCH_LCR_Information_RL_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_id_DL_DPCH_LCR_Information_RL_SetupRqstTDD);
}



static int
dissect_nbap_DwPCH_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              -150, 400U, NULL, TRUE);

  return offset;
}
static int dissect_id_DwPCH_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DwPCH_Power(tvb, offset, actx, tree, hf_nbap_id_DwPCH_Power);
}
static int dissect_dwPCH_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DwPCH_Power(tvb, offset, actx, tree, hf_nbap_dwPCH_Power);
}


static const per_sequence_t DwPCH_LCR_Information_Cell_SetupRqstTDD_sequence[] = {
  { &hf_nbap_commonPhysicalChannelId, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonPhysicalChannelID },
  { &hf_nbap_tSTD_Indicator , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TSTD_Indicator },
  { &hf_nbap_dwPCH_Power    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DwPCH_Power },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DwPCH_LCR_Information_Cell_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DwPCH_LCR_Information_Cell_SetupRqstTDD, DwPCH_LCR_Information_Cell_SetupRqstTDD_sequence);

  return offset;
}
static int dissect_id_DwPCH_LCR_Information_Cell_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DwPCH_LCR_Information_Cell_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_id_DwPCH_LCR_Information_Cell_SetupRqstTDD);
}


static const per_sequence_t DwPCH_LCR_Information_Cell_ReconfRqstTDD_sequence[] = {
  { &hf_nbap_commonPhysicalChannelId, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonPhysicalChannelID },
  { &hf_nbap_dwPCH_Power    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DwPCH_Power },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DwPCH_LCR_Information_Cell_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DwPCH_LCR_Information_Cell_ReconfRqstTDD, DwPCH_LCR_Information_Cell_ReconfRqstTDD_sequence);

  return offset;
}
static int dissect_id_DwPCH_LCR_Information_Cell_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DwPCH_LCR_Information_Cell_ReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_id_DwPCH_LCR_Information_Cell_ReconfRqstTDD);
}



static int
dissect_nbap_DwPCH_LCR_Information_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_id_DwPCH_LCR_Information_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DwPCH_LCR_Information_ResourceStatusInd(tvb, offset, actx, tree, hf_nbap_id_DwPCH_LCR_Information_ResourceStatusInd);
}


static const per_sequence_t FPACH_LCR_InformationList_AuditRsp_sequence_of[1] = {
  { &hf_nbap_FPACH_LCR_InformationList_AuditRsp_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_FPACH_LCR_InformationList_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_FPACH_LCR_InformationList_AuditRsp, FPACH_LCR_InformationList_AuditRsp_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_id_FPACH_LCR_InformationList_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_FPACH_LCR_InformationList_AuditRsp(tvb, offset, actx, tree, hf_nbap_id_FPACH_LCR_InformationList_AuditRsp);
}


static const per_sequence_t FPACH_LCR_InformationList_ResourceStatusInd_sequence_of[1] = {
  { &hf_nbap_FPACH_LCR_InformationList_ResourceStatusInd_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_FPACH_LCR_InformationList_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_FPACH_LCR_InformationList_ResourceStatusInd, FPACH_LCR_InformationList_ResourceStatusInd_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_id_FPACH_LCR_InformationList_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_FPACH_LCR_InformationList_ResourceStatusInd(tvb, offset, actx, tree, hf_nbap_id_FPACH_LCR_InformationList_ResourceStatusInd);
}



static int
dissect_nbap_FPACH_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              -150, 400U, NULL, TRUE);

  return offset;
}
static int dissect_fPACH_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_FPACH_Power(tvb, offset, actx, tree, hf_nbap_fPACH_Power);
}
static int dissect_fPACHPower(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_FPACH_Power(tvb, offset, actx, tree, hf_nbap_fPACHPower);
}


static const per_sequence_t FPACH_LCR_Parameters_CTCH_SetupRqstTDD_sequence[] = {
  { &hf_nbap_commonPhysicalChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonPhysicalChannelID },
  { &hf_nbap_tdd_ChannelisationCodeLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_ChannelisationCodeLCR },
  { &hf_nbap_timeslotLCR    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlotLCR },
  { &hf_nbap_midambleShiftLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MidambleShiftLCR },
  { &hf_nbap_fPACH_Power    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_FPACH_Power },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_FPACH_LCR_Parameters_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_FPACH_LCR_Parameters_CTCH_SetupRqstTDD, FPACH_LCR_Parameters_CTCH_SetupRqstTDD_sequence);

  return offset;
}
static int dissect_id_FPACH_LCR_Parameters_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_FPACH_LCR_Parameters_CTCH_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_id_FPACH_LCR_Parameters_CTCH_SetupRqstTDD);
}


static const per_sequence_t FPACH_LCR_Parameters_CTCH_ReconfRqstTDD_sequence[] = {
  { &hf_nbap_commonPhysicalChannelId, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonPhysicalChannelID },
  { &hf_nbap_fPACHPower     , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_FPACH_Power },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_FPACH_LCR_Parameters_CTCH_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_FPACH_LCR_Parameters_CTCH_ReconfRqstTDD, FPACH_LCR_Parameters_CTCH_ReconfRqstTDD_sequence);

  return offset;
}
static int dissect_id_FPACH_LCR_Parameters_CTCH_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_FPACH_LCR_Parameters_CTCH_ReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_id_FPACH_LCR_Parameters_CTCH_ReconfRqstTDD);
}


static const per_sequence_t PCCPCH_LCR_Information_Cell_SetupRqstTDD_sequence[] = {
  { &hf_nbap_commonPhysicalChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonPhysicalChannelID },
  { &hf_nbap_tdd_PhysicalChannelOffset, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_PhysicalChannelOffset },
  { &hf_nbap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionPeriod },
  { &hf_nbap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionLength },
  { &hf_nbap_pCCPCH_Power   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PCCPCH_Power },
  { &hf_nbap_sCTD_Indicator , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SCTD_Indicator },
  { &hf_nbap_tSTD_Indicator , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TSTD_Indicator },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PCCPCH_LCR_Information_Cell_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PCCPCH_LCR_Information_Cell_SetupRqstTDD, PCCPCH_LCR_Information_Cell_SetupRqstTDD_sequence);

  return offset;
}
static int dissect_id_PCCPCH_LCR_Information_Cell_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PCCPCH_LCR_Information_Cell_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_id_PCCPCH_LCR_Information_Cell_SetupRqstTDD);
}


static const per_sequence_t PICH_LCR_Parameters_CTCH_SetupRqstTDD_sequence[] = {
  { &hf_nbap_commonPhysicalChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonPhysicalChannelID },
  { &hf_nbap_tdd_ChannelisationCodeLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_ChannelisationCodeLCR },
  { &hf_nbap_timeSlotLCR    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlotLCR },
  { &hf_nbap_midambleShiftLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MidambleShiftLCR },
  { &hf_nbap_tdd_PhysicalChannelOffset, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_PhysicalChannelOffset },
  { &hf_nbap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionPeriod },
  { &hf_nbap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionLength },
  { &hf_nbap_pagingIndicatorLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PagingIndicatorLength },
  { &hf_nbap_pICH_Power     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PICH_Power },
  { &hf_nbap_second_TDD_ChannelisationCodeLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_ChannelisationCodeLCR },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PICH_LCR_Parameters_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PICH_LCR_Parameters_CTCH_SetupRqstTDD, PICH_LCR_Parameters_CTCH_SetupRqstTDD_sequence);

  return offset;
}
static int dissect_id_PICH_LCR_Parameters_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PICH_LCR_Parameters_CTCH_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_id_PICH_LCR_Parameters_CTCH_SetupRqstTDD);
}


static const per_sequence_t PRACH_LCR_ParametersItem_CTCH_SetupRqstTDD_sequence[] = {
  { &hf_nbap_commonPhysicalChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonPhysicalChannelID },
  { &hf_nbap_tFCS           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TFCS },
  { &hf_nbap_timeslotLCR    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlotLCR },
  { &hf_nbap_tdd_ChannelisationCodeLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_ChannelisationCodeLCR },
  { &hf_nbap_midambleShiftLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MidambleShiftLCR },
  { &hf_nbap_rACH           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RACH_Parameter_CTCH_SetupRqstTDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PRACH_LCR_ParametersItem_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PRACH_LCR_ParametersItem_CTCH_SetupRqstTDD, PRACH_LCR_ParametersItem_CTCH_SetupRqstTDD_sequence);

  return offset;
}
static int dissect_PRACH_LCR_ParametersList_CTCH_SetupRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PRACH_LCR_ParametersItem_CTCH_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_PRACH_LCR_ParametersList_CTCH_SetupRqstTDD_item);
}


static const per_sequence_t PRACH_LCR_ParametersList_CTCH_SetupRqstTDD_sequence_of[1] = {
  { &hf_nbap_PRACH_LCR_ParametersList_CTCH_SetupRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_PRACH_LCR_ParametersItem_CTCH_SetupRqstTDD },
};

static int
dissect_nbap_PRACH_LCR_ParametersList_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_PRACH_LCR_ParametersList_CTCH_SetupRqstTDD, PRACH_LCR_ParametersList_CTCH_SetupRqstTDD_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_id_PRACH_LCR_ParametersList_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PRACH_LCR_ParametersList_CTCH_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_id_PRACH_LCR_ParametersList_CTCH_SetupRqstTDD);
}


static const per_sequence_t UL_TimeSlot_ISCP_LCR_InfoItem_sequence[] = {
  { &hf_nbap_timeSlotLCR    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlotLCR },
  { &hf_nbap_iSCP           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UL_TimeslotISCP_Value },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_UL_TimeSlot_ISCP_LCR_InfoItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_UL_TimeSlot_ISCP_LCR_InfoItem, UL_TimeSlot_ISCP_LCR_InfoItem_sequence);

  return offset;
}
static int dissect_UL_TimeSlot_ISCP_LCR_Info_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_TimeSlot_ISCP_LCR_InfoItem(tvb, offset, actx, tree, hf_nbap_UL_TimeSlot_ISCP_LCR_Info_item);
}


static const per_sequence_t UL_TimeSlot_ISCP_LCR_Info_sequence_of[1] = {
  { &hf_nbap_UL_TimeSlot_ISCP_LCR_Info_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_UL_TimeSlot_ISCP_LCR_InfoItem },
};

static int
dissect_nbap_UL_TimeSlot_ISCP_LCR_Info(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_UL_TimeSlot_ISCP_LCR_Info, UL_TimeSlot_ISCP_LCR_Info_sequence_of,
                                                  1, 6);

  return offset;
}
static int dissect_uL_TimeSlot_ISCP_LCR_Info(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_TimeSlot_ISCP_LCR_Info(tvb, offset, actx, tree, hf_nbap_uL_TimeSlot_ISCP_LCR_Info);
}
static int dissect_uL_TimeSlot_ISCP_InfoLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_TimeSlot_ISCP_LCR_Info(tvb, offset, actx, tree, hf_nbap_uL_TimeSlot_ISCP_InfoLCR);
}


static const per_sequence_t RL_InformationResponse_LCR_RL_SetupRspTDD_sequence[] = {
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_ID },
  { &hf_nbap_uL_TimeSlot_ISCP_LCR_Info, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UL_TimeSlot_ISCP_LCR_Info },
  { &hf_nbap_ul_PhysCH_SF_Variation, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UL_PhysCH_SF_Variation },
  { &hf_nbap_dCH_InformationResponseList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DCH_InformationResponseList_RL_SetupRspTDD },
  { &hf_nbap_dSCH_InformationResponseList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DSCH_InformationResponseList_RL_SetupRspTDD },
  { &hf_nbap_uSCH_InformationResponseList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_USCH_InformationResponseList_RL_SetupRspTDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RL_InformationResponse_LCR_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RL_InformationResponse_LCR_RL_SetupRspTDD, RL_InformationResponse_LCR_RL_SetupRspTDD_sequence);

  return offset;
}
static int dissect_id_RL_InformationResponse_LCR_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_InformationResponse_LCR_RL_SetupRspTDD(tvb, offset, actx, tree, hf_nbap_id_RL_InformationResponse_LCR_RL_SetupRspTDD);
}


static const per_sequence_t Secondary_CCPCH_LCR_parameterItem_CTCH_SetupRqstTDD_sequence[] = {
  { &hf_nbap_commonPhysicalChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonPhysicalChannelID },
  { &hf_nbap_tdd_ChannelisationCodeLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_ChannelisationCodeLCR },
  { &hf_nbap_timeslotLCR    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlotLCR },
  { &hf_nbap_midambleShiftLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MidambleShiftLCR },
  { &hf_nbap_tdd_PhysicalChannelOffset, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_PhysicalChannelOffset },
  { &hf_nbap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionPeriod },
  { &hf_nbap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionLength },
  { &hf_nbap_s_CCPCH_Power  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Power },
  { &hf_nbap_s_CCPCH_TimeSlotFormat_LCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_DL_DPCH_TimeSlotFormat_LCR },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Secondary_CCPCH_LCR_parameterItem_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Secondary_CCPCH_LCR_parameterItem_CTCH_SetupRqstTDD, Secondary_CCPCH_LCR_parameterItem_CTCH_SetupRqstTDD_sequence);

  return offset;
}
static int dissect_Secondary_CCPCH_LCR_parameterList_CTCH_SetupRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Secondary_CCPCH_LCR_parameterItem_CTCH_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_Secondary_CCPCH_LCR_parameterList_CTCH_SetupRqstTDD_item);
}
static int dissect_Secondary_CCPCH_LCR_parameterExtendedList_CTCH_SetupRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Secondary_CCPCH_LCR_parameterItem_CTCH_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_Secondary_CCPCH_LCR_parameterExtendedList_CTCH_SetupRqstTDD_item);
}


static const per_sequence_t Secondary_CCPCH_LCR_parameterList_CTCH_SetupRqstTDD_sequence_of[1] = {
  { &hf_nbap_Secondary_CCPCH_LCR_parameterList_CTCH_SetupRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_Secondary_CCPCH_LCR_parameterItem_CTCH_SetupRqstTDD },
};

static int
dissect_nbap_Secondary_CCPCH_LCR_parameterList_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_Secondary_CCPCH_LCR_parameterList_CTCH_SetupRqstTDD, Secondary_CCPCH_LCR_parameterList_CTCH_SetupRqstTDD_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_id_Secondary_CCPCH_LCR_parameterList_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Secondary_CCPCH_LCR_parameterList_CTCH_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_id_Secondary_CCPCH_LCR_parameterList_CTCH_SetupRqstTDD);
}


static const per_sequence_t TimeSlotConfigurationItem_LCR_Cell_ReconfRqstTDD_sequence[] = {
  { &hf_nbap_timeSlotLCR    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlotLCR },
  { &hf_nbap_timeSlotStatus , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlotStatus },
  { &hf_nbap_timeSlotDirection, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlotDirection },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_TimeSlotConfigurationItem_LCR_Cell_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_TimeSlotConfigurationItem_LCR_Cell_ReconfRqstTDD, TimeSlotConfigurationItem_LCR_Cell_ReconfRqstTDD_sequence);

  return offset;
}
static int dissect_TimeSlotConfigurationList_LCR_Cell_ReconfRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TimeSlotConfigurationItem_LCR_Cell_ReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_TimeSlotConfigurationList_LCR_Cell_ReconfRqstTDD_item);
}


static const per_sequence_t TimeSlotConfigurationList_LCR_Cell_ReconfRqstTDD_sequence_of[1] = {
  { &hf_nbap_TimeSlotConfigurationList_LCR_Cell_ReconfRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlotConfigurationItem_LCR_Cell_ReconfRqstTDD },
};

static int
dissect_nbap_TimeSlotConfigurationList_LCR_Cell_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_TimeSlotConfigurationList_LCR_Cell_ReconfRqstTDD, TimeSlotConfigurationList_LCR_Cell_ReconfRqstTDD_sequence_of,
                                                  1, 7);

  return offset;
}
static int dissect_id_TimeSlotConfigurationList_LCR_Cell_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TimeSlotConfigurationList_LCR_Cell_ReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_id_TimeSlotConfigurationList_LCR_Cell_ReconfRqstTDD);
}


static const per_sequence_t TimeSlotConfigurationItem_LCR_Cell_SetupRqstTDD_sequence[] = {
  { &hf_nbap_timeSlotLCR    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlotLCR },
  { &hf_nbap_timeSlotStatus , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlotStatus },
  { &hf_nbap_timeSlotDirection, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlotDirection },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_TimeSlotConfigurationItem_LCR_Cell_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_TimeSlotConfigurationItem_LCR_Cell_SetupRqstTDD, TimeSlotConfigurationItem_LCR_Cell_SetupRqstTDD_sequence);

  return offset;
}
static int dissect_TimeSlotConfigurationList_LCR_Cell_SetupRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TimeSlotConfigurationItem_LCR_Cell_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_TimeSlotConfigurationList_LCR_Cell_SetupRqstTDD_item);
}


static const per_sequence_t TimeSlotConfigurationList_LCR_Cell_SetupRqstTDD_sequence_of[1] = {
  { &hf_nbap_TimeSlotConfigurationList_LCR_Cell_SetupRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlotConfigurationItem_LCR_Cell_SetupRqstTDD },
};

static int
dissect_nbap_TimeSlotConfigurationList_LCR_Cell_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_TimeSlotConfigurationList_LCR_Cell_SetupRqstTDD, TimeSlotConfigurationList_LCR_Cell_SetupRqstTDD_sequence_of,
                                                  1, 7);

  return offset;
}
static int dissect_id_TimeSlotConfigurationList_LCR_Cell_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TimeSlotConfigurationList_LCR_Cell_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_id_TimeSlotConfigurationList_LCR_Cell_SetupRqstTDD);
}


static const per_sequence_t DL_TimeslotISCPInfoItemLCR_sequence[] = {
  { &hf_nbap_timeSlotLCR    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlotLCR },
  { &hf_nbap_dL_TimeslotISCP, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_TimeslotISCP },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_TimeslotISCPInfoItemLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_TimeslotISCPInfoItemLCR, DL_TimeslotISCPInfoItemLCR_sequence);

  return offset;
}
static int dissect_DL_TimeslotISCPInfoLCR_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_TimeslotISCPInfoItemLCR(tvb, offset, actx, tree, hf_nbap_DL_TimeslotISCPInfoLCR_item);
}


static const per_sequence_t DL_TimeslotISCPInfoLCR_sequence_of[1] = {
  { &hf_nbap_DL_TimeslotISCPInfoLCR_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DL_TimeslotISCPInfoItemLCR },
};

static int
dissect_nbap_DL_TimeslotISCPInfoLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DL_TimeslotISCPInfoLCR, DL_TimeslotISCPInfoLCR_sequence_of,
                                                  1, 6);

  return offset;
}
static int dissect_id_TimeslotISCP_LCR_InfoList_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_TimeslotISCPInfoLCR(tvb, offset, actx, tree, hf_nbap_id_TimeslotISCP_LCR_InfoList_RL_SetupRqstTDD);
}
static int dissect_id_TimeslotISCP_InformationList_LCR_RL_AdditionRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_TimeslotISCPInfoLCR(tvb, offset, actx, tree, hf_nbap_id_TimeslotISCP_InformationList_LCR_RL_AdditionRqstTDD);
}
static int dissect_id_TimeslotISCPInfoList_LCR_DL_PC_RqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_TimeslotISCPInfoLCR(tvb, offset, actx, tree, hf_nbap_id_TimeslotISCPInfoList_LCR_DL_PC_RqstTDD);
}
static int dissect_id_TimeslotISCP_LCR_InfoList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_TimeslotISCPInfoLCR(tvb, offset, actx, tree, hf_nbap_id_TimeslotISCP_LCR_InfoList_RL_ReconfPrepTDD);
}



static int
dissect_nbap_QPSK_UL_DPCH_TimeSlotFormatTDD_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 69U, NULL, TRUE);

  return offset;
}
static int dissect_qPSK1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_QPSK_UL_DPCH_TimeSlotFormatTDD_LCR(tvb, offset, actx, tree, hf_nbap_qPSK1);
}



static int
dissect_nbap_EightPSK_UL_DPCH_TimeSlotFormatTDD_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 24U, NULL, TRUE);

  return offset;
}
static int dissect_eightPSK1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_EightPSK_UL_DPCH_TimeSlotFormatTDD_LCR(tvb, offset, actx, tree, hf_nbap_eightPSK1);
}


static const value_string nbap_TDD_UL_DPCH_TimeSlotFormat_LCR_vals[] = {
  {   0, "qPSK" },
  {   1, "eightPSK" },
  { 0, NULL }
};

static const per_choice_t TDD_UL_DPCH_TimeSlotFormat_LCR_choice[] = {
  {   0, &hf_nbap_qPSK1          , ASN1_EXTENSION_ROOT    , dissect_nbap_QPSK_UL_DPCH_TimeSlotFormatTDD_LCR },
  {   1, &hf_nbap_eightPSK1      , ASN1_EXTENSION_ROOT    , dissect_nbap_EightPSK_UL_DPCH_TimeSlotFormatTDD_LCR },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_TDD_UL_DPCH_TimeSlotFormat_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_TDD_UL_DPCH_TimeSlotFormat_LCR, TDD_UL_DPCH_TimeSlotFormat_LCR_choice,
                                 NULL);

  return offset;
}
static int dissect_id_UL_DPCH_TimeSlotFormat_LCR_ModifyItem_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TDD_UL_DPCH_TimeSlotFormat_LCR(tvb, offset, actx, tree, hf_nbap_id_UL_DPCH_TimeSlotFormat_LCR_ModifyItem_RL_ReconfPrepTDD);
}
static int dissect_tdd_UL_DPCH_TimeSlotFormat_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TDD_UL_DPCH_TimeSlotFormat_LCR(tvb, offset, actx, tree, hf_nbap_tdd_UL_DPCH_TimeSlotFormat_LCR);
}


static const per_sequence_t TDD_UL_Code_LCR_InformationItem_sequence[] = {
  { &hf_nbap_dPCH_ID        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DPCH_ID },
  { &hf_nbap_tdd_ChannelisationCodeLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_ChannelisationCodeLCR },
  { &hf_nbap_tdd_UL_DPCH_TimeSlotFormat_LCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_UL_DPCH_TimeSlotFormat_LCR },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_TDD_UL_Code_LCR_InformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_TDD_UL_Code_LCR_InformationItem, TDD_UL_Code_LCR_InformationItem_sequence);

  return offset;
}
static int dissect_TDD_UL_Code_LCR_Information_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TDD_UL_Code_LCR_InformationItem(tvb, offset, actx, tree, hf_nbap_TDD_UL_Code_LCR_Information_item);
}


static const per_sequence_t TDD_UL_Code_LCR_Information_sequence_of[1] = {
  { &hf_nbap_TDD_UL_Code_LCR_Information_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_UL_Code_LCR_InformationItem },
};

static int
dissect_nbap_TDD_UL_Code_LCR_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_TDD_UL_Code_LCR_Information, TDD_UL_Code_LCR_Information_sequence_of,
                                                  1, 240);

  return offset;
}
static int dissect_uL_Code_InformationList1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TDD_UL_Code_LCR_Information(tvb, offset, actx, tree, hf_nbap_uL_Code_InformationList1);
}


static const per_sequence_t UL_TimeslotLCR_InformationItem_sequence[] = {
  { &hf_nbap_timeSlotLCR    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlotLCR },
  { &hf_nbap_midambleShiftLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MidambleShiftLCR },
  { &hf_nbap_tFCI_Presence  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TFCI_Presence },
  { &hf_nbap_uL_Code_InformationList1, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_UL_Code_LCR_Information },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_UL_TimeslotLCR_InformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_UL_TimeslotLCR_InformationItem, UL_TimeslotLCR_InformationItem_sequence);

  return offset;
}
static int dissect_UL_TimeslotLCR_Information_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_TimeslotLCR_InformationItem(tvb, offset, actx, tree, hf_nbap_UL_TimeslotLCR_Information_item);
}


static const per_sequence_t UL_TimeslotLCR_Information_sequence_of[1] = {
  { &hf_nbap_UL_TimeslotLCR_Information_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_UL_TimeslotLCR_InformationItem },
};

static int
dissect_nbap_UL_TimeslotLCR_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_UL_TimeslotLCR_Information, UL_TimeslotLCR_Information_sequence_of,
                                                  1, 6);

  return offset;
}
static int dissect_uL_TimeslotLCR_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_TimeslotLCR_Information(tvb, offset, actx, tree, hf_nbap_uL_TimeslotLCR_Information);
}
static int dissect_uL_Timeslot_InformationLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_TimeslotLCR_Information(tvb, offset, actx, tree, hf_nbap_uL_Timeslot_InformationLCR);
}


static const per_sequence_t UL_DPCH_LCR_Information_RL_SetupRqstTDD_sequence[] = {
  { &hf_nbap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionPeriod },
  { &hf_nbap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionLength },
  { &hf_nbap_tdd_DPCHOffset , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_DPCHOffset },
  { &hf_nbap_uL_TimeslotLCR_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UL_TimeslotLCR_Information },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_UL_DPCH_LCR_Information_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_UL_DPCH_LCR_Information_RL_SetupRqstTDD, UL_DPCH_LCR_Information_RL_SetupRqstTDD_sequence);

  return offset;
}
static int dissect_id_UL_DPCH_LCR_Information_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_DPCH_LCR_Information_RL_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_id_UL_DPCH_LCR_Information_RL_SetupRqstTDD);
}


static const per_sequence_t DL_DPCH_InformationItem_LCR_RL_AdditionRqstTDD_sequence[] = {
  { &hf_nbap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionPeriod },
  { &hf_nbap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionLength },
  { &hf_nbap_tdd_DPCHOffset , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_DPCHOffset },
  { &hf_nbap_dL_TimeslotLCR_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_TimeslotLCR_Information },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_DPCH_InformationItem_LCR_RL_AdditionRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_DPCH_InformationItem_LCR_RL_AdditionRqstTDD, DL_DPCH_InformationItem_LCR_RL_AdditionRqstTDD_sequence);

  return offset;
}
static int dissect_id_DL_DPCH_InformationItem_LCR_RL_AdditionRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_DPCH_InformationItem_LCR_RL_AdditionRqstTDD(tvb, offset, actx, tree, hf_nbap_id_DL_DPCH_InformationItem_LCR_RL_AdditionRqstTDD);
}


static const per_sequence_t UL_DPCH_InformationItem_LCR_RL_AdditionRqstTDD_sequence[] = {
  { &hf_nbap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionPeriod },
  { &hf_nbap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionLength },
  { &hf_nbap_tdd_DPCHOffset , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_DPCHOffset },
  { &hf_nbap_uL_TimeslotLCR_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UL_TimeslotLCR_Information },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_UL_DPCH_InformationItem_LCR_RL_AdditionRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_UL_DPCH_InformationItem_LCR_RL_AdditionRqstTDD, UL_DPCH_InformationItem_LCR_RL_AdditionRqstTDD_sequence);

  return offset;
}
static int dissect_id_UL_DPCH_InformationItem_LCR_RL_AdditionRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_DPCH_InformationItem_LCR_RL_AdditionRqstTDD(tvb, offset, actx, tree, hf_nbap_id_UL_DPCH_InformationItem_LCR_RL_AdditionRqstTDD);
}


static const per_sequence_t DL_DPCH_LCR_InformationAddList_RL_ReconfPrepTDD_sequence[] = {
  { &hf_nbap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionPeriod },
  { &hf_nbap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionLength },
  { &hf_nbap_tdd_DPCHOffset , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_DPCHOffset },
  { &hf_nbap_dL_Timeslot_InformationLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_TimeslotLCR_Information },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_DPCH_LCR_InformationAddList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_DPCH_LCR_InformationAddList_RL_ReconfPrepTDD, DL_DPCH_LCR_InformationAddList_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_id_DL_DPCH_LCR_InformationAddList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_DPCH_LCR_InformationAddList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_id_DL_DPCH_LCR_InformationAddList_RL_ReconfPrepTDD);
}
static int dissect_dl_DPCH_InformationListLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_DPCH_LCR_InformationAddList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_dl_DPCH_InformationListLCR);
}


static const per_sequence_t DL_DPCH_LCR_InformationModify_AddList_RL_ReconfPrepTDD_sequence[] = {
  { &hf_nbap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionPeriod },
  { &hf_nbap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionLength },
  { &hf_nbap_tdd_DPCHOffset , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_DPCHOffset },
  { &hf_nbap_dL_Timeslot_InformationLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_TimeslotLCR_Information },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_DPCH_LCR_InformationModify_AddList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_DPCH_LCR_InformationModify_AddList_RL_ReconfPrepTDD, DL_DPCH_LCR_InformationModify_AddList_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_id_DL_DPCH_LCR_InformationModify_AddList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_DPCH_LCR_InformationModify_AddList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_id_DL_DPCH_LCR_InformationModify_AddList_RL_ReconfPrepTDD);
}
static int dissect_dl_DPCH_InformationAddListLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_DPCH_LCR_InformationModify_AddList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_dl_DPCH_InformationAddListLCR);
}


static const per_sequence_t DL_Code_LCR_InformationModify_ModifyItem_RL_ReconfPrepTDD_sequence[] = {
  { &hf_nbap_dPCH_ID        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DPCH_ID },
  { &hf_nbap_tdd_ChannelisationCodeLCR, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TDD_ChannelisationCodeLCR },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_Code_LCR_InformationModify_ModifyItem_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_Code_LCR_InformationModify_ModifyItem_RL_ReconfPrepTDD, DL_Code_LCR_InformationModify_ModifyItem_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_DL_Code_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Code_LCR_InformationModify_ModifyItem_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_DL_Code_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD_item);
}


static const per_sequence_t DL_Code_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD_sequence_of[1] = {
  { &hf_nbap_DL_Code_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Code_LCR_InformationModify_ModifyItem_RL_ReconfPrepTDD },
};

static int
dissect_nbap_DL_Code_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DL_Code_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD, DL_Code_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD_sequence_of,
                                                  1, 240);

  return offset;
}
static int dissect_dL_Code_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Code_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_dL_Code_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD);
}


static const per_sequence_t DL_Timeslot_LCR_InformationModify_ModifyItem_RL_ReconfPrepTDD_sequence[] = {
  { &hf_nbap_timeSlotLCR    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlotLCR },
  { &hf_nbap_midambleShiftLCR, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_MidambleShiftLCR },
  { &hf_nbap_tFCI_Presence  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TFCI_Presence },
  { &hf_nbap_dL_Code_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_Code_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_Timeslot_LCR_InformationModify_ModifyItem_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_Timeslot_LCR_InformationModify_ModifyItem_RL_ReconfPrepTDD, DL_Timeslot_LCR_InformationModify_ModifyItem_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_DL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Timeslot_LCR_InformationModify_ModifyItem_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_DL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD_item);
}


static const per_sequence_t DL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD_sequence_of[1] = {
  { &hf_nbap_DL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Timeslot_LCR_InformationModify_ModifyItem_RL_ReconfPrepTDD },
};

static int
dissect_nbap_DL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD, DL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD_sequence_of,
                                                  1, 240);

  return offset;
}
static int dissect_id_DL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_id_DL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD);
}


static const per_sequence_t UL_DPCH_LCR_InformationAddList_RL_ReconfPrepTDD_sequence[] = {
  { &hf_nbap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionPeriod },
  { &hf_nbap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionLength },
  { &hf_nbap_tdd_DPCHOffset , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_DPCHOffset },
  { &hf_nbap_uL_Timeslot_InformationLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UL_TimeslotLCR_Information },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_UL_DPCH_LCR_InformationAddList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_UL_DPCH_LCR_InformationAddList_RL_ReconfPrepTDD, UL_DPCH_LCR_InformationAddList_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_id_UL_DPCH_LCR_InformationAddListIE_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_DPCH_LCR_InformationAddList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_id_UL_DPCH_LCR_InformationAddListIE_RL_ReconfPrepTDD);
}
static int dissect_ul_DPCH_InformationListLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_DPCH_LCR_InformationAddList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_ul_DPCH_InformationListLCR);
}


static const per_sequence_t UL_DPCH_LCR_InformationModify_AddList_RL_ReconfPrepTDD_sequence[] = {
  { &hf_nbap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionPeriod },
  { &hf_nbap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionLength },
  { &hf_nbap_tdd_DPCHOffset , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_DPCHOffset },
  { &hf_nbap_uL_Timeslot_InformationLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UL_TimeslotLCR_Information },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_UL_DPCH_LCR_InformationModify_AddList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_UL_DPCH_LCR_InformationModify_AddList_RL_ReconfPrepTDD, UL_DPCH_LCR_InformationModify_AddList_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_id_UL_DPCH_LCR_InformationModify_AddList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_DPCH_LCR_InformationModify_AddList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_id_UL_DPCH_LCR_InformationModify_AddList);
}
static int dissect_ul_DPCH_InformationAddListLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_DPCH_LCR_InformationModify_AddList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_ul_DPCH_InformationAddListLCR);
}


static const per_sequence_t UL_Code_InformationModify_ModifyItem_RL_ReconfPrepTDDLCR_sequence[] = {
  { &hf_nbap_dPCH_ID        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DPCH_ID },
  { &hf_nbap_tdd_ChannelisationCodeLCR, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TDD_ChannelisationCodeLCR },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_UL_Code_InformationModify_ModifyItem_RL_ReconfPrepTDDLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_UL_Code_InformationModify_ModifyItem_RL_ReconfPrepTDDLCR, UL_Code_InformationModify_ModifyItem_RL_ReconfPrepTDDLCR_sequence);

  return offset;
}
static int dissect_UL_Code_InformationModify_ModifyList_RL_ReconfPrepTDDLCR_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_Code_InformationModify_ModifyItem_RL_ReconfPrepTDDLCR(tvb, offset, actx, tree, hf_nbap_UL_Code_InformationModify_ModifyList_RL_ReconfPrepTDDLCR_item);
}


static const per_sequence_t UL_Code_InformationModify_ModifyList_RL_ReconfPrepTDDLCR_sequence_of[1] = {
  { &hf_nbap_UL_Code_InformationModify_ModifyList_RL_ReconfPrepTDDLCR_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_UL_Code_InformationModify_ModifyItem_RL_ReconfPrepTDDLCR },
};

static int
dissect_nbap_UL_Code_InformationModify_ModifyList_RL_ReconfPrepTDDLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_UL_Code_InformationModify_ModifyList_RL_ReconfPrepTDDLCR, UL_Code_InformationModify_ModifyList_RL_ReconfPrepTDDLCR_sequence_of,
                                                  1, 240);

  return offset;
}
static int dissect_uL_Code_InformationModify_ModifyList_RL_ReconfPrepTDDLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_Code_InformationModify_ModifyList_RL_ReconfPrepTDDLCR(tvb, offset, actx, tree, hf_nbap_uL_Code_InformationModify_ModifyList_RL_ReconfPrepTDDLCR);
}


static const per_sequence_t UL_Timeslot_LCR_InformationModify_ModifyItem_RL_ReconfPrepTDD_sequence[] = {
  { &hf_nbap_timeSlotLCR    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlotLCR },
  { &hf_nbap_midambleShiftLCR, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_MidambleShiftLCR },
  { &hf_nbap_tFCI_Presence  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TFCI_Presence },
  { &hf_nbap_uL_Code_InformationModify_ModifyList_RL_ReconfPrepTDDLCR, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_UL_Code_InformationModify_ModifyList_RL_ReconfPrepTDDLCR },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_UL_Timeslot_LCR_InformationModify_ModifyItem_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_UL_Timeslot_LCR_InformationModify_ModifyItem_RL_ReconfPrepTDD, UL_Timeslot_LCR_InformationModify_ModifyItem_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_UL_TimeslotLCR_InformationModify_ModifyList_RL_ReconfPrepTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_Timeslot_LCR_InformationModify_ModifyItem_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_UL_TimeslotLCR_InformationModify_ModifyList_RL_ReconfPrepTDD_item);
}


static const per_sequence_t UL_TimeslotLCR_InformationModify_ModifyList_RL_ReconfPrepTDD_sequence_of[1] = {
  { &hf_nbap_UL_TimeslotLCR_InformationModify_ModifyList_RL_ReconfPrepTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_UL_Timeslot_LCR_InformationModify_ModifyItem_RL_ReconfPrepTDD },
};

static int
dissect_nbap_UL_TimeslotLCR_InformationModify_ModifyList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_UL_TimeslotLCR_InformationModify_ModifyList_RL_ReconfPrepTDD, UL_TimeslotLCR_InformationModify_ModifyList_RL_ReconfPrepTDD_sequence_of,
                                                  1, 6);

  return offset;
}
static int dissect_id_UL_TimeslotLCR_Information_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_TimeslotLCR_InformationModify_ModifyList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_id_UL_TimeslotLCR_Information_RL_ReconfPrepTDD);
}


static const per_sequence_t DL_Code_InformationAddItem_LCR_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_pDSCH_ID       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PDSCH_ID },
  { &hf_nbap_tdd_ChannelisationCodeLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_ChannelisationCodeLCR },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_Code_InformationAddItem_LCR_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_Code_InformationAddItem_LCR_PSCH_ReconfRqst, DL_Code_InformationAddItem_LCR_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_DL_Code_InformationAddList_LCR_PSCH_ReconfRqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Code_InformationAddItem_LCR_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_DL_Code_InformationAddList_LCR_PSCH_ReconfRqst_item);
}


static const per_sequence_t DL_Code_InformationAddList_LCR_PSCH_ReconfRqst_sequence_of[1] = {
  { &hf_nbap_DL_Code_InformationAddList_LCR_PSCH_ReconfRqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Code_InformationAddItem_LCR_PSCH_ReconfRqst },
};

static int
dissect_nbap_DL_Code_InformationAddList_LCR_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DL_Code_InformationAddList_LCR_PSCH_ReconfRqst, DL_Code_InformationAddList_LCR_PSCH_ReconfRqst_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_dL_Code_InformationAddList_LCR_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Code_InformationAddList_LCR_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_dL_Code_InformationAddList_LCR_PSCH_ReconfRqst);
}


static const per_sequence_t DL_Timeslot_InformationAddItem_LCR_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_timeSlotLCR    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlotLCR },
  { &hf_nbap_midambleShiftLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MidambleShiftLCR },
  { &hf_nbap_tFCI_Presence  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TFCI_Presence },
  { &hf_nbap_dL_Code_InformationAddList_LCR_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Code_InformationAddList_LCR_PSCH_ReconfRqst },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_Timeslot_InformationAddItem_LCR_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_Timeslot_InformationAddItem_LCR_PSCH_ReconfRqst, DL_Timeslot_InformationAddItem_LCR_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_DL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Timeslot_InformationAddItem_LCR_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_DL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst_item);
}


static const per_sequence_t DL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst_sequence_of[1] = {
  { &hf_nbap_DL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Timeslot_InformationAddItem_LCR_PSCH_ReconfRqst },
};

static int
dissect_nbap_DL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst, DL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst_sequence_of,
                                                  1, 6);

  return offset;
}
static int dissect_dL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_dL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst);
}


static const per_sequence_t PDSCH_AddInformation_LCR_AddItem_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionPeriod },
  { &hf_nbap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionLength },
  { &hf_nbap_tdd_PhysicalChannelOffset, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_PhysicalChannelOffset },
  { &hf_nbap_dL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PDSCH_AddInformation_LCR_AddItem_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PDSCH_AddInformation_LCR_AddItem_PSCH_ReconfRqst, PDSCH_AddInformation_LCR_AddItem_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_id_PDSCH_AddInformation_LCR_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PDSCH_AddInformation_LCR_AddItem_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_id_PDSCH_AddInformation_LCR_PSCH_ReconfRqst);
}


static const per_sequence_t DL_Code_LCR_InformationModifyItem_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_pDSCH_ID       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PDSCH_ID },
  { &hf_nbap_tdd_ChannelisationCodeLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_ChannelisationCodeLCR },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_Code_LCR_InformationModifyItem_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_Code_LCR_InformationModifyItem_PSCH_ReconfRqst, DL_Code_LCR_InformationModifyItem_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_DL_Code_LCR_InformationModifyList_PSCH_ReconfRqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Code_LCR_InformationModifyItem_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_DL_Code_LCR_InformationModifyList_PSCH_ReconfRqst_item);
}


static const per_sequence_t DL_Code_LCR_InformationModifyList_PSCH_ReconfRqst_sequence_of[1] = {
  { &hf_nbap_DL_Code_LCR_InformationModifyList_PSCH_ReconfRqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Code_LCR_InformationModifyItem_PSCH_ReconfRqst },
};

static int
dissect_nbap_DL_Code_LCR_InformationModifyList_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DL_Code_LCR_InformationModifyList_PSCH_ReconfRqst, DL_Code_LCR_InformationModifyList_PSCH_ReconfRqst_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_dL_Code_LCR_InformationModifyList_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Code_LCR_InformationModifyList_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_dL_Code_LCR_InformationModifyList_PSCH_ReconfRqst);
}


static const per_sequence_t DL_Timeslot_LCR_InformationModifyItem_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_timeSlotLCR    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlotLCR },
  { &hf_nbap_midambleShiftLCR, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_MidambleShiftLCR },
  { &hf_nbap_tFCI_Presence  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TFCI_Presence },
  { &hf_nbap_dL_Code_LCR_InformationModifyList_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_Code_LCR_InformationModifyList_PSCH_ReconfRqst },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_Timeslot_LCR_InformationModifyItem_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_Timeslot_LCR_InformationModifyItem_PSCH_ReconfRqst, DL_Timeslot_LCR_InformationModifyItem_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_DL_Timeslot_LCR_InformationModifyList_PSCH_ReconfRqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Timeslot_LCR_InformationModifyItem_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_DL_Timeslot_LCR_InformationModifyList_PSCH_ReconfRqst_item);
}


static const per_sequence_t DL_Timeslot_LCR_InformationModifyList_PSCH_ReconfRqst_sequence_of[1] = {
  { &hf_nbap_DL_Timeslot_LCR_InformationModifyList_PSCH_ReconfRqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Timeslot_LCR_InformationModifyItem_PSCH_ReconfRqst },
};

static int
dissect_nbap_DL_Timeslot_LCR_InformationModifyList_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DL_Timeslot_LCR_InformationModifyList_PSCH_ReconfRqst, DL_Timeslot_LCR_InformationModifyList_PSCH_ReconfRqst_sequence_of,
                                                  1, 6);

  return offset;
}
static int dissect_dL_Timeslot_LCR_InformationModifyList_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Timeslot_LCR_InformationModifyList_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_dL_Timeslot_LCR_InformationModifyList_PSCH_ReconfRqst);
}


static const per_sequence_t PDSCH_ModifyInformation_LCR_ModifyItem_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_RepetitionPeriod },
  { &hf_nbap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_RepetitionLength },
  { &hf_nbap_tdd_PhysicalChannelOffset, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TDD_PhysicalChannelOffset },
  { &hf_nbap_dL_Timeslot_LCR_InformationModifyList_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_Timeslot_LCR_InformationModifyList_PSCH_ReconfRqst },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PDSCH_ModifyInformation_LCR_ModifyItem_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PDSCH_ModifyInformation_LCR_ModifyItem_PSCH_ReconfRqst, PDSCH_ModifyInformation_LCR_ModifyItem_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_id_PDSCH_ModifyInformation_LCR_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PDSCH_ModifyInformation_LCR_ModifyItem_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_id_PDSCH_ModifyInformation_LCR_PSCH_ReconfRqst);
}


static const per_sequence_t UL_Code_InformationAddItem_LCR_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_pUSCH_ID       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PUSCH_ID },
  { &hf_nbap_tdd_ChannelisationCodeLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_ChannelisationCodeLCR },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_UL_Code_InformationAddItem_LCR_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_UL_Code_InformationAddItem_LCR_PSCH_ReconfRqst, UL_Code_InformationAddItem_LCR_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_UL_Code_InformationAddList_LCR_PSCH_ReconfRqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_Code_InformationAddItem_LCR_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_UL_Code_InformationAddList_LCR_PSCH_ReconfRqst_item);
}


static const per_sequence_t UL_Code_InformationAddList_LCR_PSCH_ReconfRqst_sequence_of[1] = {
  { &hf_nbap_UL_Code_InformationAddList_LCR_PSCH_ReconfRqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_UL_Code_InformationAddItem_LCR_PSCH_ReconfRqst },
};

static int
dissect_nbap_UL_Code_InformationAddList_LCR_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_UL_Code_InformationAddList_LCR_PSCH_ReconfRqst, UL_Code_InformationAddList_LCR_PSCH_ReconfRqst_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_uL_Code_InformationAddList_LCR_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_Code_InformationAddList_LCR_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_uL_Code_InformationAddList_LCR_PSCH_ReconfRqst);
}


static const per_sequence_t UL_Timeslot_InformationAddItem_LCR_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_timeSlotLCR    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlotLCR },
  { &hf_nbap_midambleShiftLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MidambleShiftLCR },
  { &hf_nbap_tFCI_Presence  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TFCI_Presence },
  { &hf_nbap_uL_Code_InformationAddList_LCR_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UL_Code_InformationAddList_LCR_PSCH_ReconfRqst },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_UL_Timeslot_InformationAddItem_LCR_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_UL_Timeslot_InformationAddItem_LCR_PSCH_ReconfRqst, UL_Timeslot_InformationAddItem_LCR_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_UL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_Timeslot_InformationAddItem_LCR_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_UL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst_item);
}


static const per_sequence_t UL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst_sequence_of[1] = {
  { &hf_nbap_UL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_UL_Timeslot_InformationAddItem_LCR_PSCH_ReconfRqst },
};

static int
dissect_nbap_UL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_UL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst, UL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst_sequence_of,
                                                  1, 6);

  return offset;
}
static int dissect_uL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_uL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst);
}


static const per_sequence_t PUSCH_AddInformation_LCR_AddItem_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionPeriod },
  { &hf_nbap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionLength },
  { &hf_nbap_tdd_PhysicalChannelOffset, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_PhysicalChannelOffset },
  { &hf_nbap_uL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PUSCH_AddInformation_LCR_AddItem_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PUSCH_AddInformation_LCR_AddItem_PSCH_ReconfRqst, PUSCH_AddInformation_LCR_AddItem_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_id_PUSCH_AddInformation_LCR_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PUSCH_AddInformation_LCR_AddItem_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_id_PUSCH_AddInformation_LCR_PSCH_ReconfRqst);
}


static const per_sequence_t UL_Code_LCR_InformationModifyItem_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_pUSCH_ID       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PUSCH_ID },
  { &hf_nbap_tdd_ChannelisationCodeLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_ChannelisationCodeLCR },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_UL_Code_LCR_InformationModifyItem_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_UL_Code_LCR_InformationModifyItem_PSCH_ReconfRqst, UL_Code_LCR_InformationModifyItem_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_UL_Code_LCR_InformationModifyList_PSCH_ReconfRqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_Code_LCR_InformationModifyItem_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_UL_Code_LCR_InformationModifyList_PSCH_ReconfRqst_item);
}


static const per_sequence_t UL_Code_LCR_InformationModifyList_PSCH_ReconfRqst_sequence_of[1] = {
  { &hf_nbap_UL_Code_LCR_InformationModifyList_PSCH_ReconfRqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_UL_Code_LCR_InformationModifyItem_PSCH_ReconfRqst },
};

static int
dissect_nbap_UL_Code_LCR_InformationModifyList_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_UL_Code_LCR_InformationModifyList_PSCH_ReconfRqst, UL_Code_LCR_InformationModifyList_PSCH_ReconfRqst_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_uL_Code_LCR_InformationModifyList_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_Code_LCR_InformationModifyList_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_uL_Code_LCR_InformationModifyList_PSCH_ReconfRqst);
}


static const per_sequence_t UL_Timeslot_LCR_InformationModifyItem_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_timeSlotLCR    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlotLCR },
  { &hf_nbap_midambleShiftLCR, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_MidambleShiftLCR },
  { &hf_nbap_tFCI_Presence  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TFCI_Presence },
  { &hf_nbap_uL_Code_LCR_InformationModifyList_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_UL_Code_LCR_InformationModifyList_PSCH_ReconfRqst },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_UL_Timeslot_LCR_InformationModifyItem_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_UL_Timeslot_LCR_InformationModifyItem_PSCH_ReconfRqst, UL_Timeslot_LCR_InformationModifyItem_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_UL_Timeslot_LCR_InformationModifyList_PSCH_ReconfRqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_Timeslot_LCR_InformationModifyItem_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_UL_Timeslot_LCR_InformationModifyList_PSCH_ReconfRqst_item);
}


static const per_sequence_t UL_Timeslot_LCR_InformationModifyList_PSCH_ReconfRqst_sequence_of[1] = {
  { &hf_nbap_UL_Timeslot_LCR_InformationModifyList_PSCH_ReconfRqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_UL_Timeslot_LCR_InformationModifyItem_PSCH_ReconfRqst },
};

static int
dissect_nbap_UL_Timeslot_LCR_InformationModifyList_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_UL_Timeslot_LCR_InformationModifyList_PSCH_ReconfRqst, UL_Timeslot_LCR_InformationModifyList_PSCH_ReconfRqst_sequence_of,
                                                  1, 6);

  return offset;
}
static int dissect_uL_Timeslot_InformationModifyList_LCR_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_Timeslot_LCR_InformationModifyList_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_uL_Timeslot_InformationModifyList_LCR_PSCH_ReconfRqst);
}


static const per_sequence_t PUSCH_ModifyInformation_LCR_ModifyItem_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_RepetitionPeriod },
  { &hf_nbap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_RepetitionLength },
  { &hf_nbap_tdd_PhysicalChannelOffset, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TDD_PhysicalChannelOffset },
  { &hf_nbap_uL_Timeslot_InformationModifyList_LCR_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_UL_Timeslot_LCR_InformationModifyList_PSCH_ReconfRqst },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PUSCH_ModifyInformation_LCR_ModifyItem_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PUSCH_ModifyInformation_LCR_ModifyItem_PSCH_ReconfRqst, PUSCH_ModifyInformation_LCR_ModifyItem_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_id_PUSCH_ModifyInformation_LCR_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PUSCH_ModifyInformation_LCR_ModifyItem_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_id_PUSCH_ModifyInformation_LCR_PSCH_ReconfRqst);
}


static const per_sequence_t TimeslotInfo_CellSyncInitiationRqstTDD_sequence_of[1] = {
  { &hf_nbap_TimeslotInfo_CellSyncInitiationRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlot },
};

static int
dissect_nbap_TimeslotInfo_CellSyncInitiationRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_TimeslotInfo_CellSyncInitiationRqstTDD, TimeslotInfo_CellSyncInitiationRqstTDD_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_id_timeslotInfo_CellSyncInitiationRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TimeslotInfo_CellSyncInitiationRqstTDD(tvb, offset, actx, tree, hf_nbap_id_timeslotInfo_CellSyncInitiationRqstTDD);
}



static int
dissect_nbap_CellSyncBurstSIR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 31U, NULL, FALSE);

  return offset;
}
static int dissect_cellSyncBurstSIR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSyncBurstSIR(tvb, offset, actx, tree, hf_nbap_cellSyncBurstSIR);
}
static int dissect_syncDLCodeIdSIR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSyncBurstSIR(tvb, offset, actx, tree, hf_nbap_syncDLCodeIdSIR);
}


static const per_sequence_t CellSyncBurstAvailable_CellSyncReprtTDD_sequence[] = {
  { &hf_nbap_cellSyncBurstTiming, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CellSyncBurstTiming },
  { &hf_nbap_cellSyncBurstSIR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CellSyncBurstSIR },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CellSyncBurstAvailable_CellSyncReprtTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CellSyncBurstAvailable_CellSyncReprtTDD, CellSyncBurstAvailable_CellSyncReprtTDD_sequence);

  return offset;
}
static int dissect_cellSyncBurstAvailable(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSyncBurstAvailable_CellSyncReprtTDD(tvb, offset, actx, tree, hf_nbap_cellSyncBurstAvailable);
}


static const value_string nbap_CellSyncBurstInfo_CellSyncReprtTDD_vals[] = {
  {   0, "cellSyncBurstAvailable" },
  {   1, "cellSyncBurstNotAvailable" },
  { 0, NULL }
};

static const per_choice_t CellSyncBurstInfo_CellSyncReprtTDD_choice[] = {
  {   0, &hf_nbap_cellSyncBurstAvailable, ASN1_EXTENSION_ROOT    , dissect_nbap_CellSyncBurstAvailable_CellSyncReprtTDD },
  {   1, &hf_nbap_cellSyncBurstNotAvailable, ASN1_EXTENSION_ROOT    , dissect_nbap_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_CellSyncBurstInfo_CellSyncReprtTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_CellSyncBurstInfo_CellSyncReprtTDD, CellSyncBurstInfo_CellSyncReprtTDD_choice,
                                 NULL);

  return offset;
}
static int dissect_cellSyncBurstInfo_CellSyncReprtTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSyncBurstInfo_CellSyncReprtTDD(tvb, offset, actx, tree, hf_nbap_cellSyncBurstInfo_CellSyncReprtTDD_item);
}


static const per_sequence_t SEQUENCE_SIZE_1_16_OF_CellSyncBurstInfo_CellSyncReprtTDD_sequence_of[1] = {
  { &hf_nbap_cellSyncBurstInfo_CellSyncReprtTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_CellSyncBurstInfo_CellSyncReprtTDD },
};

static int
dissect_nbap_SEQUENCE_SIZE_1_16_OF_CellSyncBurstInfo_CellSyncReprtTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_SEQUENCE_SIZE_1_16_OF_CellSyncBurstInfo_CellSyncReprtTDD, SEQUENCE_SIZE_1_16_OF_CellSyncBurstInfo_CellSyncReprtTDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_cellSyncBurstInfo_CellSyncReprtTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SEQUENCE_SIZE_1_16_OF_CellSyncBurstInfo_CellSyncReprtTDD(tvb, offset, actx, tree, hf_nbap_cellSyncBurstInfo_CellSyncReprtTDD);
}


static const per_sequence_t CellSyncBurstMeasInfoItem_CellSyncReprtTDD_sequence[] = {
  { &hf_nbap_sFN            , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SFN },
  { &hf_nbap_cellSyncBurstInfo_CellSyncReprtTDD, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SEQUENCE_SIZE_1_16_OF_CellSyncBurstInfo_CellSyncReprtTDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CellSyncBurstMeasInfoItem_CellSyncReprtTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CellSyncBurstMeasInfoItem_CellSyncReprtTDD, CellSyncBurstMeasInfoItem_CellSyncReprtTDD_sequence);

  return offset;
}
static int dissect_CellSyncBurstMeasInfoList_CellSyncReprtTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSyncBurstMeasInfoItem_CellSyncReprtTDD(tvb, offset, actx, tree, hf_nbap_CellSyncBurstMeasInfoList_CellSyncReprtTDD_item);
}


static const per_sequence_t CellSyncBurstMeasInfoList_CellSyncReprtTDD_sequence_of[1] = {
  { &hf_nbap_CellSyncBurstMeasInfoList_CellSyncReprtTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_CellSyncBurstMeasInfoItem_CellSyncReprtTDD },
};

static int
dissect_nbap_CellSyncBurstMeasInfoList_CellSyncReprtTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_CellSyncBurstMeasInfoList_CellSyncReprtTDD, CellSyncBurstMeasInfoList_CellSyncReprtTDD_sequence_of,
                                                  0, 10);

  return offset;
}
static int dissect_cellSyncBurstMeasuredInfo(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSyncBurstMeasInfoList_CellSyncReprtTDD(tvb, offset, actx, tree, hf_nbap_cellSyncBurstMeasuredInfo);
}


static const per_sequence_t IntStdPhCellSyncInfo_CellSyncReprtTDD_sequence[] = {
  { &hf_nbap_cellSyncBurstMeasuredInfo, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CellSyncBurstMeasInfoList_CellSyncReprtTDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_IntStdPhCellSyncInfo_CellSyncReprtTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_IntStdPhCellSyncInfo_CellSyncReprtTDD, IntStdPhCellSyncInfo_CellSyncReprtTDD_sequence);

  return offset;
}
static int dissect_intStdPhSyncInfo_CellSyncReprtTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_IntStdPhCellSyncInfo_CellSyncReprtTDD(tvb, offset, actx, tree, hf_nbap_intStdPhSyncInfo_CellSyncReprtTDD);
}


static const value_string nbap_SyncReportType_CellSyncReprtTDD_vals[] = {
  {   0, "intStdPhSyncInfo-CellSyncReprtTDD" },
  {   1, "lateEntrantCell" },
  {   2, "frequencyAcquisition" },
  { 0, NULL }
};

static const per_choice_t SyncReportType_CellSyncReprtTDD_choice[] = {
  {   0, &hf_nbap_intStdPhSyncInfo_CellSyncReprtTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_IntStdPhCellSyncInfo_CellSyncReprtTDD },
  {   1, &hf_nbap_lateEntrantCell, ASN1_EXTENSION_ROOT    , dissect_nbap_NULL },
  {   2, &hf_nbap_frequencyAcquisition, ASN1_EXTENSION_ROOT    , dissect_nbap_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_SyncReportType_CellSyncReprtTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_SyncReportType_CellSyncReprtTDD, SyncReportType_CellSyncReprtTDD_choice,
                                 NULL);

  return offset;
}
static int dissect_id_SyncReportType_CellSyncReprtTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SyncReportType_CellSyncReprtTDD(tvb, offset, actx, tree, hf_nbap_id_SyncReportType_CellSyncReprtTDD);
}


static const per_sequence_t Power_Local_Cell_Group_InformationItem_AuditRsp_sequence[] = {
  { &hf_nbap_power_Local_Cell_Group_ID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Local_Cell_ID },
  { &hf_nbap_maximumDL_PowerCapability, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MaximumDL_PowerCapability },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Power_Local_Cell_Group_InformationItem_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Power_Local_Cell_Group_InformationItem_AuditRsp, Power_Local_Cell_Group_InformationItem_AuditRsp_sequence);

  return offset;
}
static int dissect_id_Power_Local_Cell_Group_InformationItem_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Power_Local_Cell_Group_InformationItem_AuditRsp(tvb, offset, actx, tree, hf_nbap_id_Power_Local_Cell_Group_InformationItem_AuditRsp);
}


static const per_sequence_t Power_Local_Cell_Group_InformationItem_ResourceStatusInd_sequence[] = {
  { &hf_nbap_power_Local_Cell_Group_ID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Local_Cell_ID },
  { &hf_nbap_maximumDL_PowerCapability, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MaximumDL_PowerCapability },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Power_Local_Cell_Group_InformationItem_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Power_Local_Cell_Group_InformationItem_ResourceStatusInd, Power_Local_Cell_Group_InformationItem_ResourceStatusInd_sequence);

  return offset;
}
static int dissect_id_Power_Local_Cell_Group_InformationItem_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Power_Local_Cell_Group_InformationItem_ResourceStatusInd(tvb, offset, actx, tree, hf_nbap_id_Power_Local_Cell_Group_InformationItem_ResourceStatusInd);
}


static const per_sequence_t Power_Local_Cell_Group_InformationItem2_ResourceStatusInd_sequence[] = {
  { &hf_nbap_power_Local_Cell_Group_ID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Local_Cell_ID },
  { &hf_nbap_maximumDL_PowerCapability, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MaximumDL_PowerCapability },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Power_Local_Cell_Group_InformationItem2_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Power_Local_Cell_Group_InformationItem2_ResourceStatusInd, Power_Local_Cell_Group_InformationItem2_ResourceStatusInd_sequence);

  return offset;
}
static int dissect_id_Power_Local_Cell_Group_InformationItem2_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Power_Local_Cell_Group_InformationItem2_ResourceStatusInd(tvb, offset, actx, tree, hf_nbap_id_Power_Local_Cell_Group_InformationItem2_ResourceStatusInd);
}


static const per_sequence_t Power_Local_Cell_Group_InformationList_AuditRsp_sequence_of[1] = {
  { &hf_nbap_Power_Local_Cell_Group_InformationList_AuditRsp_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_Power_Local_Cell_Group_InformationList_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_Power_Local_Cell_Group_InformationList_AuditRsp, Power_Local_Cell_Group_InformationList_AuditRsp_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_id_Power_Local_Cell_Group_InformationList_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Power_Local_Cell_Group_InformationList_AuditRsp(tvb, offset, actx, tree, hf_nbap_id_Power_Local_Cell_Group_InformationList_AuditRsp);
}


static const per_sequence_t Power_Local_Cell_Group_InformationList_ResourceStatusInd_sequence_of[1] = {
  { &hf_nbap_Power_Local_Cell_Group_InformationList_ResourceStatusInd_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_Power_Local_Cell_Group_InformationList_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_Power_Local_Cell_Group_InformationList_ResourceStatusInd, Power_Local_Cell_Group_InformationList_ResourceStatusInd_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_id_Power_Local_Cell_Group_InformationList_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Power_Local_Cell_Group_InformationList_ResourceStatusInd(tvb, offset, actx, tree, hf_nbap_id_Power_Local_Cell_Group_InformationList_ResourceStatusInd);
}


static const per_sequence_t Power_Local_Cell_Group_InformationList2_ResourceStatusInd_sequence_of[1] = {
  { &hf_nbap_Power_Local_Cell_Group_InformationList2_ResourceStatusInd_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_Power_Local_Cell_Group_InformationList2_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_Power_Local_Cell_Group_InformationList2_ResourceStatusInd, Power_Local_Cell_Group_InformationList2_ResourceStatusInd_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_id_Power_Local_Cell_Group_InformationList2_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Power_Local_Cell_Group_InformationList2_ResourceStatusInd(tvb, offset, actx, tree, hf_nbap_id_Power_Local_Cell_Group_InformationList2_ResourceStatusInd);
}


static const per_sequence_t PUSCH_Info_DM_Rqst_sequence_of[1] = {
  { &hf_nbap_PUSCH_Info_DM_Rqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_PUSCH_ID },
};

static int
dissect_nbap_PUSCH_Info_DM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_PUSCH_Info_DM_Rqst, PUSCH_Info_DM_Rqst_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_id_PUSCH_Info_DM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PUSCH_Info_DM_Rqst(tvb, offset, actx, tree, hf_nbap_id_PUSCH_Info_DM_Rqst);
}


static const per_sequence_t PUSCH_Info_DM_Rsp_sequence_of[1] = {
  { &hf_nbap_PUSCH_Info_DM_Rsp_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_PUSCH_ID },
};

static int
dissect_nbap_PUSCH_Info_DM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_PUSCH_Info_DM_Rsp, PUSCH_Info_DM_Rsp_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_id_PUSCH_Info_DM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PUSCH_Info_DM_Rsp(tvb, offset, actx, tree, hf_nbap_id_PUSCH_Info_DM_Rsp);
}


static const per_sequence_t PUSCH_Info_DM_Rprt_sequence_of[1] = {
  { &hf_nbap_PUSCH_Info_DM_Rprt_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_PUSCH_ID },
};

static int
dissect_nbap_PUSCH_Info_DM_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_PUSCH_Info_DM_Rprt, PUSCH_Info_DM_Rprt_sequence_of,
                                                  0, 256);

  return offset;
}
static int dissect_id_PUSCH_Info_DM_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PUSCH_Info_DM_Rprt(tvb, offset, actx, tree, hf_nbap_id_PUSCH_Info_DM_Rprt);
}



static int
dissect_nbap_CellSyncBurstRepetitionPeriod(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 4095U, NULL, FALSE);

  return offset;
}
static int dissect_id_cellSyncBurstRepetitionPeriod(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSyncBurstRepetitionPeriod(tvb, offset, actx, tree, hf_nbap_id_cellSyncBurstRepetitionPeriod);
}


static const per_sequence_t ReportCharacteristicsType_OnModification_sequence[] = {
  { &hf_nbap_measurementThreshold, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ReportCharacteristicsType_MeasurementThreshold },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_ReportCharacteristicsType_OnModification(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_ReportCharacteristicsType_OnModification, ReportCharacteristicsType_OnModification_sequence);

  return offset;
}
static int dissect_id_ReportCharacteristicsType_OnModification(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ReportCharacteristicsType_OnModification(tvb, offset, actx, tree, hf_nbap_id_ReportCharacteristicsType_OnModification);
}



static int
dissect_nbap_SFNSFN_FDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 614399U, NULL, FALSE);

  return offset;
}
static int dissect_sFNSFN_FDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SFNSFN_FDD(tvb, offset, actx, tree, hf_nbap_sFNSFN_FDD);
}



static int
dissect_nbap_SFNSFN_TDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 40961U, NULL, FALSE);

  return offset;
}
static int dissect_sFNSFN_TDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SFNSFN_TDD(tvb, offset, actx, tree, hf_nbap_sFNSFN_TDD);
}


static const value_string nbap_SFNSFNValue_vals[] = {
  {   0, "sFNSFN-FDD" },
  {   1, "sFNSFN-TDD" },
  { 0, NULL }
};

static const per_choice_t SFNSFNValue_choice[] = {
  {   0, &hf_nbap_sFNSFN_FDD     , ASN1_EXTENSION_ROOT    , dissect_nbap_SFNSFN_FDD },
  {   1, &hf_nbap_sFNSFN_TDD     , ASN1_EXTENSION_ROOT    , dissect_nbap_SFNSFN_TDD },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_SFNSFNValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_SFNSFNValue, SFNSFNValue_choice,
                                 NULL);

  return offset;
}
static int dissect_sFNSFNValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SFNSFNValue(tvb, offset, actx, tree, hf_nbap_sFNSFNValue);
}



static int
dissect_nbap_SFNSFNQuality(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 255U, NULL, FALSE);

  return offset;
}
static int dissect_sFNSFNQuality(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SFNSFNQuality(tvb, offset, actx, tree, hf_nbap_sFNSFNQuality);
}



static int
dissect_nbap_SFNSFNDriftRate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              -100, 100U, NULL, FALSE);

  return offset;
}
static int dissect_sFNSFNDriftRate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SFNSFNDriftRate(tvb, offset, actx, tree, hf_nbap_sFNSFNDriftRate);
}



static int
dissect_nbap_SFNSFNDriftRateQuality(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 100U, NULL, FALSE);

  return offset;
}
static int dissect_sFNSFNDriftRateQuality(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SFNSFNDriftRateQuality(tvb, offset, actx, tree, hf_nbap_sFNSFNDriftRateQuality);
}


static const per_sequence_t SFNSFNTimeStamp_TDD_sequence[] = {
  { &hf_nbap_sFN            , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SFN },
  { &hf_nbap_timeSlot       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlot },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_SFNSFNTimeStamp_TDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_SFNSFNTimeStamp_TDD, SFNSFNTimeStamp_TDD_sequence);

  return offset;
}
static int dissect_sFNSFNTimeStamp_TDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SFNSFNTimeStamp_TDD(tvb, offset, actx, tree, hf_nbap_sFNSFNTimeStamp_TDD);
}


static const value_string nbap_SFNSFNTimeStampInformation_vals[] = {
  {   0, "sFNSFNTimeStamp-FDD" },
  {   1, "sFNSFNTimeStamp-TDD" },
  { 0, NULL }
};

static const per_choice_t SFNSFNTimeStampInformation_choice[] = {
  {   0, &hf_nbap_sFNSFNTimeStamp_FDD, ASN1_EXTENSION_ROOT    , dissect_nbap_SFN },
  {   1, &hf_nbap_sFNSFNTimeStamp_TDD, ASN1_EXTENSION_ROOT    , dissect_nbap_SFNSFNTimeStamp_TDD },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_SFNSFNTimeStampInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_SFNSFNTimeStampInformation, SFNSFNTimeStampInformation_choice,
                                 NULL);

  return offset;
}
static int dissect_sFNSFNTimeStampInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SFNSFNTimeStampInformation(tvb, offset, actx, tree, hf_nbap_sFNSFNTimeStampInformation);
}


static const per_sequence_t T_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item_sequence[] = {
  { &hf_nbap_uC_Id          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UC_Id },
  { &hf_nbap_sFNSFNValue    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SFNSFNValue },
  { &hf_nbap_sFNSFNQuality  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_SFNSFNQuality },
  { &hf_nbap_sFNSFNDriftRate, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SFNSFNDriftRate },
  { &hf_nbap_sFNSFNDriftRateQuality, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_SFNSFNDriftRateQuality },
  { &hf_nbap_sFNSFNTimeStampInformation, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SFNSFNTimeStampInformation },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_T_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_T_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item, T_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item_sequence);

  return offset;
}
static int dissect_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_T_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item(tvb, offset, actx, tree, hf_nbap_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item);
}


static const per_sequence_t T_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_sequence_of[1] = {
  { &hf_nbap_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_T_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item },
};

static int
dissect_nbap_T_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_T_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation, T_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_sequence_of,
                                                  1, 96);

  return offset;
}
static int dissect_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_T_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation(tvb, offset, actx, tree, hf_nbap_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation);
}


static const per_sequence_t T_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item_sequence[] = {
  { &hf_nbap_uC_Id          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UC_Id },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_T_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_T_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item, T_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item_sequence);

  return offset;
}
static int dissect_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_T_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item(tvb, offset, actx, tree, hf_nbap_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item);
}


static const per_sequence_t T_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_sequence_of[1] = {
  { &hf_nbap_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_T_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item },
};

static int
dissect_nbap_T_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_T_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation, T_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_sequence_of,
                                                  0, 95);

  return offset;
}
static int dissect_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_T_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation(tvb, offset, actx, tree, hf_nbap_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation);
}


static const per_sequence_t SFNSFNMeasurementValueInformation_sequence[] = {
  { &hf_nbap_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_T_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation },
  { &hf_nbap_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_T_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_SFNSFNMeasurementValueInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_SFNSFNMeasurementValueInformation, SFNSFNMeasurementValueInformation_sequence);

  return offset;
}
static int dissect_id_SFNSFNMeasurementValueInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SFNSFNMeasurementValueInformation(tvb, offset, actx, tree, hf_nbap_id_SFNSFNMeasurementValueInformation);
}



static int
dissect_nbap_SFNSFNChangeLimit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 256U, NULL, FALSE);

  return offset;
}
static int dissect_sFNSFNChangeLimit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SFNSFNChangeLimit(tvb, offset, actx, tree, hf_nbap_sFNSFNChangeLimit);
}



static int
dissect_nbap_PredictedSFNSFNDeviationLimit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 256U, NULL, FALSE);

  return offset;
}
static int dissect_predictedSFNSFNDeviationLimit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PredictedSFNSFNDeviationLimit(tvb, offset, actx, tree, hf_nbap_predictedSFNSFNDeviationLimit);
}


static const per_sequence_t SFNSFNMeasurementThresholdInformation_sequence[] = {
  { &hf_nbap_sFNSFNChangeLimit, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_SFNSFNChangeLimit },
  { &hf_nbap_predictedSFNSFNDeviationLimit, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_PredictedSFNSFNDeviationLimit },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_SFNSFNMeasurementThresholdInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_SFNSFNMeasurementThresholdInformation, SFNSFNMeasurementThresholdInformation_sequence);

  return offset;
}
static int dissect_id_SFNSFNMeasurementThresholdInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SFNSFNMeasurementThresholdInformation(tvb, offset, actx, tree, hf_nbap_id_SFNSFNMeasurementThresholdInformation);
}



static int
dissect_nbap_INTEGER_0_16383(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 16383U, NULL, FALSE);

  return offset;
}
static int dissect_ms_part(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_INTEGER_0_16383(tvb, offset, actx, tree, hf_nbap_ms_part);
}



static int
dissect_nbap_INTEGER_0_4294967295(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 4294967295U, NULL, FALSE);

  return offset;
}
static int dissect_ls_part(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_INTEGER_0_4294967295(tvb, offset, actx, tree, hf_nbap_ls_part);
}


static const per_sequence_t TUTRANGPS_sequence[] = {
  { &hf_nbap_ms_part        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_INTEGER_0_16383 },
  { &hf_nbap_ls_part        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_INTEGER_0_4294967295 },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_TUTRANGPS(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_TUTRANGPS, TUTRANGPS_sequence);

  return offset;
}
static int dissect_tUTRANGPS(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TUTRANGPS(tvb, offset, actx, tree, hf_nbap_tUTRANGPS);
}



static int
dissect_nbap_TUTRANGPSQuality(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 255U, NULL, FALSE);

  return offset;
}
static int dissect_tUTRANGPSQuality(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TUTRANGPSQuality(tvb, offset, actx, tree, hf_nbap_tUTRANGPSQuality);
}



static int
dissect_nbap_TUTRANGPSDriftRate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              -50, 50U, NULL, FALSE);

  return offset;
}
static int dissect_tUTRANGPSDriftRate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TUTRANGPSDriftRate(tvb, offset, actx, tree, hf_nbap_tUTRANGPSDriftRate);
}



static int
dissect_nbap_TUTRANGPSDriftRateQuality(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 50U, NULL, FALSE);

  return offset;
}
static int dissect_tUTRANGPSDriftRateQuality(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TUTRANGPSDriftRateQuality(tvb, offset, actx, tree, hf_nbap_tUTRANGPSDriftRateQuality);
}


static const per_sequence_t TUTRANGPSMeasurementValueInformation_sequence[] = {
  { &hf_nbap_tUTRANGPS      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TUTRANGPS },
  { &hf_nbap_tUTRANGPSQuality, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TUTRANGPSQuality },
  { &hf_nbap_tUTRANGPSDriftRate, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TUTRANGPSDriftRate },
  { &hf_nbap_tUTRANGPSDriftRateQuality, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TUTRANGPSDriftRateQuality },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_TUTRANGPSMeasurementValueInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_TUTRANGPSMeasurementValueInformation, TUTRANGPSMeasurementValueInformation_sequence);

  return offset;
}
static int dissect_id_TUTRANGPSMeasurementValueInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TUTRANGPSMeasurementValueInformation(tvb, offset, actx, tree, hf_nbap_id_TUTRANGPSMeasurementValueInformation);
}



static int
dissect_nbap_TUTRANGPSChangeLimit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 256U, NULL, FALSE);

  return offset;
}
static int dissect_tUTRANGPSChangeLimit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TUTRANGPSChangeLimit(tvb, offset, actx, tree, hf_nbap_tUTRANGPSChangeLimit);
}



static int
dissect_nbap_PredictedTUTRANGPSDeviationLimit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 256U, NULL, FALSE);

  return offset;
}
static int dissect_predictedTUTRANGPSDeviationLimit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PredictedTUTRANGPSDeviationLimit(tvb, offset, actx, tree, hf_nbap_predictedTUTRANGPSDeviationLimit);
}


static const per_sequence_t TUTRANGPSMeasurementThresholdInformation_sequence[] = {
  { &hf_nbap_tUTRANGPSChangeLimit, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TUTRANGPSChangeLimit },
  { &hf_nbap_predictedTUTRANGPSDeviationLimit, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_PredictedTUTRANGPSDeviationLimit },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_TUTRANGPSMeasurementThresholdInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_TUTRANGPSMeasurementThresholdInformation, TUTRANGPSMeasurementThresholdInformation_sequence);

  return offset;
}
static int dissect_id_TUTRANGPSMeasurementThresholdInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TUTRANGPSMeasurementThresholdInformation(tvb, offset, actx, tree, hf_nbap_id_TUTRANGPSMeasurementThresholdInformation);
}



static int
dissect_nbap_Rx_Timing_Deviation_Value_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 511U, NULL, FALSE);

  return offset;
}
static int dissect_id_Rx_Timing_Deviation_Value_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Rx_Timing_Deviation_Value_LCR(tvb, offset, actx, tree, hf_nbap_id_Rx_Timing_Deviation_Value_LCR);
}


static const per_sequence_t RL_InformationResponse_LCR_RL_AdditionRspTDD_sequence[] = {
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_ID },
  { &hf_nbap_uL_TimeSlot_ISCP_InfoLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UL_TimeSlot_ISCP_LCR_Info },
  { &hf_nbap_ul_PhysCH_SF_Variation, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UL_PhysCH_SF_Variation },
  { &hf_nbap_dCH_Information, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DCH_Information_RL_AdditionRspTDD },
  { &hf_nbap_dSCH_InformationResponseList1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DSCH_InformationResponseList_RL_AdditionRspTDD },
  { &hf_nbap_uSCH_InformationResponseList1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_USCH_InformationResponseList_RL_AdditionRspTDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RL_InformationResponse_LCR_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RL_InformationResponse_LCR_RL_AdditionRspTDD, RL_InformationResponse_LCR_RL_AdditionRspTDD_sequence);

  return offset;
}
static int dissect_id_RL_InformationResponse_LCR_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_InformationResponse_LCR_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_nbap_id_RL_InformationResponse_LCR_RL_AdditionRspTDD);
}


static const per_sequence_t DL_ReferencePowerInformationItem_sequence[] = {
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_ID },
  { &hf_nbap_dl_Reference_Power, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Power },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_ReferencePowerInformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_ReferencePowerInformationItem, DL_ReferencePowerInformationItem_sequence);

  return offset;
}
static int dissect_DL_ReferencePowerInformationList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_ReferencePowerInformationItem(tvb, offset, actx, tree, hf_nbap_DL_ReferencePowerInformationList_item);
}


static const per_sequence_t DL_ReferencePowerInformationList_sequence_of[1] = {
  { &hf_nbap_DL_ReferencePowerInformationList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DL_ReferencePowerInformationItem },
};

static int
dissect_nbap_DL_ReferencePowerInformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DL_ReferencePowerInformationList, DL_ReferencePowerInformationList_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_dLReferencePowerList_DL_PC_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_ReferencePowerInformationList(tvb, offset, actx, tree, hf_nbap_dLReferencePowerList_DL_PC_Rqst);
}


static const per_sequence_t DL_PowerBalancing_Information_sequence[] = {
  { &hf_nbap_powerAdjustmentType, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PowerAdjustmentType },
  { &hf_nbap_dLReferencePower, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_Power },
  { &hf_nbap_dLReferencePowerList_DL_PC_Rqst, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_ReferencePowerInformationList },
  { &hf_nbap_maxAdjustmentStep, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_MaxAdjustmentStep },
  { &hf_nbap_adjustmentPeriod, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_AdjustmentPeriod },
  { &hf_nbap_adjustmentRatio, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ScaledAdjustmentRatio },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_PowerBalancing_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_PowerBalancing_Information, DL_PowerBalancing_Information_sequence);

  return offset;
}
static int dissect_id_DL_PowerBalancing_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_PowerBalancing_Information(tvb, offset, actx, tree, hf_nbap_id_DL_PowerBalancing_Information);
}


static const value_string nbap_DL_PowerBalancing_ActivationIndicator_vals[] = {
  {   0, "dL-PowerBalancing-Activated" },
  { 0, NULL }
};


static int
dissect_nbap_DL_PowerBalancing_ActivationIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_DL_PowerBalancing_ActivationIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_PowerBalancing_ActivationIndicator(tvb, offset, actx, tree, hf_nbap_id_DL_PowerBalancing_ActivationIndicator);
}


static const value_string nbap_DL_PowerBalancing_UpdatedIndicator_vals[] = {
  {   0, "dL-PowerBalancing-Updated" },
  { 0, NULL }
};


static int
dissect_nbap_DL_PowerBalancing_UpdatedIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_DL_PowerBalancing_UpdatedIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_PowerBalancing_UpdatedIndicator(tvb, offset, actx, tree, hf_nbap_id_DL_PowerBalancing_UpdatedIndicator);
}


static const value_string nbap_T_iP_SpacingTDD1_vals[] = {
  {   0, "sp30" },
  {   1, "sp40" },
  {   2, "sp50" },
  {   3, "sp70" },
  {   4, "sp100" },
  { 0, NULL }
};


static int
dissect_nbap_T_iP_SpacingTDD1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     5, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_iP_SpacingTDD1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_T_iP_SpacingTDD1(tvb, offset, actx, tree, hf_nbap_iP_SpacingTDD1);
}


static const value_string nbap_T_iP_Sub_vals[] = {
  {   0, "first" },
  {   1, "second" },
  {   2, "both" },
  { 0, NULL }
};


static int
dissect_nbap_T_iP_Sub(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_iP_Sub(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_T_iP_Sub(tvb, offset, actx, tree, hf_nbap_iP_Sub);
}


static const per_sequence_t IPDL_TDD_Parameters_LCR_sequence[] = {
  { &hf_nbap_iP_SpacingTDD1 , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_T_iP_SpacingTDD1 },
  { &hf_nbap_iP_Start       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_INTEGER_0_4095 },
  { &hf_nbap_iP_Sub         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_T_iP_Sub },
  { &hf_nbap_burstModeParams, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_BurstModeParams },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_IPDL_TDD_Parameters_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_IPDL_TDD_Parameters_LCR, IPDL_TDD_Parameters_LCR_sequence);

  return offset;
}
static int dissect_iPDL_TDD_Parameters_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_IPDL_TDD_Parameters_LCR(tvb, offset, actx, tree, hf_nbap_iPDL_TDD_Parameters_LCR);
}


static const per_sequence_t IPDLParameter_Information_LCR_Cell_SetupRqstTDD_sequence[] = {
  { &hf_nbap_iPDL_TDD_Parameters_LCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_IPDL_TDD_Parameters_LCR },
  { &hf_nbap_iPDL_Indicator , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_IPDL_Indicator },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_IPDLParameter_Information_LCR_Cell_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_IPDLParameter_Information_LCR_Cell_SetupRqstTDD, IPDLParameter_Information_LCR_Cell_SetupRqstTDD_sequence);

  return offset;
}
static int dissect_id_IPDLParameter_Information_LCR_Cell_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_IPDLParameter_Information_LCR_Cell_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_id_IPDLParameter_Information_LCR_Cell_SetupRqstTDD);
}


static const per_sequence_t IPDLParameter_Information_LCR_Cell_ReconfRqstTDD_sequence[] = {
  { &hf_nbap_iPDL_TDD_Parameters_LCR, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_IPDL_TDD_Parameters_LCR },
  { &hf_nbap_iPDL_Indicator , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_IPDL_Indicator },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_IPDLParameter_Information_LCR_Cell_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_IPDLParameter_Information_LCR_Cell_ReconfRqstTDD, IPDLParameter_Information_LCR_Cell_ReconfRqstTDD_sequence);

  return offset;
}
static int dissect_id_IPDLParameter_Information_LCR_Cell_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_IPDLParameter_Information_LCR_Cell_ReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_id_IPDLParameter_Information_LCR_Cell_ReconfRqstTDD);
}



static int
dissect_nbap_HS_PDSCH_Start_code_number(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 15U, NULL, FALSE);

  return offset;
}
static int dissect_hS_PDSCH_Start_code_number(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_PDSCH_Start_code_number(tvb, offset, actx, tree, hf_nbap_hS_PDSCH_Start_code_number);
}


static const per_sequence_t HS_PDSCH_FDD_Code_Information_sequence[] = {
  { &hf_nbap_number_of_HS_PDSCH_codes, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_INTEGER_0_15 },
  { &hf_nbap_hS_PDSCH_Start_code_number, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_HS_PDSCH_Start_code_number },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_HS_PDSCH_FDD_Code_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_HS_PDSCH_FDD_Code_Information, HS_PDSCH_FDD_Code_Information_sequence);

  return offset;
}
static int dissect_id_HS_PDSCH_FDD_Code_Information_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_PDSCH_FDD_Code_Information(tvb, offset, actx, tree, hf_nbap_id_HS_PDSCH_FDD_Code_Information_PSCH_ReconfRqst);
}
static int dissect_hS_PDSCH_FDD_Code_Information_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_PDSCH_FDD_Code_Information(tvb, offset, actx, tree, hf_nbap_hS_PDSCH_FDD_Code_Information_PSCH_ReconfRqst);
}



static int
dissect_nbap_HS_SCCH_FDD_Code_Information_Item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 127U, NULL, FALSE);

  return offset;
}
static int dissect_HS_SCCH_FDD_Code_List_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_SCCH_FDD_Code_Information_Item(tvb, offset, actx, tree, hf_nbap_HS_SCCH_FDD_Code_List_item);
}


static const per_sequence_t HS_SCCH_FDD_Code_List_sequence_of[1] = {
  { &hf_nbap_HS_SCCH_FDD_Code_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_HS_SCCH_FDD_Code_Information_Item },
};

static int
dissect_nbap_HS_SCCH_FDD_Code_List(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_HS_SCCH_FDD_Code_List, HS_SCCH_FDD_Code_List_sequence_of,
                                                  1, 32);

  return offset;
}
static int dissect_replace2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_SCCH_FDD_Code_List(tvb, offset, actx, tree, hf_nbap_replace2);
}


static const value_string nbap_HS_SCCH_FDD_Code_Information_vals[] = {
  {   0, "replace" },
  {   1, "remove" },
  { 0, NULL }
};

static const per_choice_t HS_SCCH_FDD_Code_Information_choice[] = {
  {   0, &hf_nbap_replace2       , ASN1_EXTENSION_ROOT    , dissect_nbap_HS_SCCH_FDD_Code_List },
  {   1, &hf_nbap_remove         , ASN1_EXTENSION_ROOT    , dissect_nbap_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_HS_SCCH_FDD_Code_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_HS_SCCH_FDD_Code_Information, HS_SCCH_FDD_Code_Information_choice,
                                 NULL);

  return offset;
}
static int dissect_id_HS_SCCH_FDD_Code_Information_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_SCCH_FDD_Code_Information(tvb, offset, actx, tree, hf_nbap_id_HS_SCCH_FDD_Code_Information_PSCH_ReconfRqst);
}
static int dissect_hS_SCCH_FDD_Code_Information_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_SCCH_FDD_Code_Information(tvb, offset, actx, tree, hf_nbap_hS_SCCH_FDD_Code_Information_PSCH_ReconfRqst);
}


static const per_sequence_t DL_HS_PDSCH_Codelist_PSCH_ReconfRqst_sequence_of[1] = {
  { &hf_nbap_DL_HS_PDSCH_Codelist_PSCH_ReconfRqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_ChannelisationCode },
};

static int
dissect_nbap_DL_HS_PDSCH_Codelist_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DL_HS_PDSCH_Codelist_PSCH_ReconfRqst, DL_HS_PDSCH_Codelist_PSCH_ReconfRqst_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_dl_HS_PDSCH_Codelist_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_HS_PDSCH_Codelist_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_dl_HS_PDSCH_Codelist_PSCH_ReconfRqst);
}


static const per_sequence_t DL_HS_PDSCH_Timeslot_InformationItem_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_timeSlot       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlot },
  { &hf_nbap_midambleShiftAndBurstType, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MidambleShiftAndBurstType },
  { &hf_nbap_dl_HS_PDSCH_Codelist_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_HS_PDSCH_Codelist_PSCH_ReconfRqst },
  { &hf_nbap_maxHSDSCH_HSSCCH_Power, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_MaximumTransmissionPower },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_HS_PDSCH_Timeslot_InformationItem_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_HS_PDSCH_Timeslot_InformationItem_PSCH_ReconfRqst, DL_HS_PDSCH_Timeslot_InformationItem_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_DL_HS_PDSCH_Timeslot_Information_PSCH_ReconfRqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_HS_PDSCH_Timeslot_InformationItem_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_DL_HS_PDSCH_Timeslot_Information_PSCH_ReconfRqst_item);
}


static const per_sequence_t DL_HS_PDSCH_Timeslot_Information_PSCH_ReconfRqst_sequence_of[1] = {
  { &hf_nbap_DL_HS_PDSCH_Timeslot_Information_PSCH_ReconfRqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DL_HS_PDSCH_Timeslot_InformationItem_PSCH_ReconfRqst },
};

static int
dissect_nbap_DL_HS_PDSCH_Timeslot_Information_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DL_HS_PDSCH_Timeslot_Information_PSCH_ReconfRqst, DL_HS_PDSCH_Timeslot_Information_PSCH_ReconfRqst_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_dL_HS_PDSCH_Timeslot_Information_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_HS_PDSCH_Timeslot_Information_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_dL_HS_PDSCH_Timeslot_Information_PSCH_ReconfRqst);
}


static const per_sequence_t DL_HS_PDSCH_Codelist_LCR_PSCH_ReconfRqst_sequence_of[1] = {
  { &hf_nbap_DL_HS_PDSCH_Codelist_LCR_PSCH_ReconfRqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_ChannelisationCode },
};

static int
dissect_nbap_DL_HS_PDSCH_Codelist_LCR_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DL_HS_PDSCH_Codelist_LCR_PSCH_ReconfRqst, DL_HS_PDSCH_Codelist_LCR_PSCH_ReconfRqst_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_dl_HS_PDSCH_Codelist_LCR_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_HS_PDSCH_Codelist_LCR_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_dl_HS_PDSCH_Codelist_LCR_PSCH_ReconfRqst);
}


static const per_sequence_t DL_HS_PDSCH_Timeslot_InformationItem_LCR_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_timeSlot1      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlotLCR },
  { &hf_nbap_midambleShiftAndBurstType1, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MidambleShiftLCR },
  { &hf_nbap_dl_HS_PDSCH_Codelist_LCR_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_HS_PDSCH_Codelist_LCR_PSCH_ReconfRqst },
  { &hf_nbap_maxHSDSCH_HSSCCH_Power, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_MaximumTransmissionPower },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_HS_PDSCH_Timeslot_InformationItem_LCR_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_HS_PDSCH_Timeslot_InformationItem_LCR_PSCH_ReconfRqst, DL_HS_PDSCH_Timeslot_InformationItem_LCR_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_DL_HS_PDSCH_Timeslot_Information_LCR_PSCH_ReconfRqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_HS_PDSCH_Timeslot_InformationItem_LCR_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_DL_HS_PDSCH_Timeslot_Information_LCR_PSCH_ReconfRqst_item);
}


static const per_sequence_t DL_HS_PDSCH_Timeslot_Information_LCR_PSCH_ReconfRqst_sequence_of[1] = {
  { &hf_nbap_DL_HS_PDSCH_Timeslot_Information_LCR_PSCH_ReconfRqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DL_HS_PDSCH_Timeslot_InformationItem_LCR_PSCH_ReconfRqst },
};

static int
dissect_nbap_DL_HS_PDSCH_Timeslot_Information_LCR_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DL_HS_PDSCH_Timeslot_Information_LCR_PSCH_ReconfRqst, DL_HS_PDSCH_Timeslot_Information_LCR_PSCH_ReconfRqst_sequence_of,
                                                  1, 6);

  return offset;
}
static int dissect_dL_HS_PDSCH_Timeslot_Information_LCR_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_HS_PDSCH_Timeslot_Information_LCR_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_dL_HS_PDSCH_Timeslot_Information_LCR_PSCH_ReconfRqst);
}


static const per_sequence_t HS_PDSCH_TDD_Information_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_dL_HS_PDSCH_Timeslot_Information_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_HS_PDSCH_Timeslot_Information_PSCH_ReconfRqst },
  { &hf_nbap_dL_HS_PDSCH_Timeslot_Information_LCR_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_HS_PDSCH_Timeslot_Information_LCR_PSCH_ReconfRqst },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_HS_PDSCH_TDD_Information_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_HS_PDSCH_TDD_Information_PSCH_ReconfRqst, HS_PDSCH_TDD_Information_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_id_HS_PDSCH_TDD_Information_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_PDSCH_TDD_Information_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_id_HS_PDSCH_TDD_Information_PSCH_ReconfRqst);
}



static int
dissect_nbap_HS_SCCH_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 31U, NULL, FALSE);

  return offset;
}
static int dissect_hS_SCCH_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_SCCH_ID(tvb, offset, actx, tree, hf_nbap_hS_SCCH_ID);
}



static int
dissect_nbap_HS_SICH_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 31U, NULL, FALSE);

  return offset;
}
static int dissect_id_HSSICH_Info_DM_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_SICH_ID(tvb, offset, actx, tree, hf_nbap_id_HSSICH_Info_DM_Rprt);
}
static int dissect_id_HSSICH_Info_DM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_SICH_ID(tvb, offset, actx, tree, hf_nbap_id_HSSICH_Info_DM_Rsp);
}
static int dissect_HSSICH_Info_DM_Rqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_SICH_ID(tvb, offset, actx, tree, hf_nbap_HSSICH_Info_DM_Rqst_item);
}
static int dissect_hsSICH_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_SICH_ID(tvb, offset, actx, tree, hf_nbap_hsSICH_ID);
}


static const per_sequence_t HS_SICH_Information_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_hsSICH_ID      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_HS_SICH_ID },
  { &hf_nbap_timeSlot       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlot },
  { &hf_nbap_midambleShiftAndBurstType, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MidambleShiftAndBurstType },
  { &hf_nbap_tdd_ChannelisationCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_ChannelisationCode },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_HS_SICH_Information_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_HS_SICH_Information_PSCH_ReconfRqst, HS_SICH_Information_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_hS_SICH_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_SICH_Information_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_hS_SICH_Information);
}


static const per_sequence_t HS_SCCH_InformationItem_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_hS_SCCH_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_HS_SCCH_ID },
  { &hf_nbap_timeSlot       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlot },
  { &hf_nbap_midambleShiftAndBurstType, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MidambleShiftAndBurstType },
  { &hf_nbap_tdd_ChannelisationCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_ChannelisationCode },
  { &hf_nbap_hS_SCCH_MaxPower, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Power },
  { &hf_nbap_hS_SICH_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_HS_SICH_Information_PSCH_ReconfRqst },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_HS_SCCH_InformationItem_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_HS_SCCH_InformationItem_PSCH_ReconfRqst, HS_SCCH_InformationItem_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_HS_SCCH_Information_PSCH_ReconfRqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_SCCH_InformationItem_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_HS_SCCH_Information_PSCH_ReconfRqst_item);
}


static const per_sequence_t HS_SCCH_Information_PSCH_ReconfRqst_sequence_of[1] = {
  { &hf_nbap_HS_SCCH_Information_PSCH_ReconfRqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_HS_SCCH_InformationItem_PSCH_ReconfRqst },
};

static int
dissect_nbap_HS_SCCH_Information_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_HS_SCCH_Information_PSCH_ReconfRqst, HS_SCCH_Information_PSCH_ReconfRqst_sequence_of,
                                                  1, 32);

  return offset;
}
static int dissect_hS_SCCH_Information_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_SCCH_Information_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_hS_SCCH_Information_PSCH_ReconfRqst);
}


static const per_sequence_t HS_SICH_Information_LCR_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_hsSICH_ID      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_HS_SICH_ID },
  { &hf_nbap_timeSlotLCR    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlotLCR },
  { &hf_nbap_midambleShiftLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MidambleShiftLCR },
  { &hf_nbap_tdd_ChannelisationCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_ChannelisationCode },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_HS_SICH_Information_LCR_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_HS_SICH_Information_LCR_PSCH_ReconfRqst, HS_SICH_Information_LCR_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_hS_SICH_Information_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_SICH_Information_LCR_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_hS_SICH_Information_LCR);
}


static const per_sequence_t HS_SCCH_InformationItem_LCR_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_hS_SCCH_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_HS_SCCH_ID },
  { &hf_nbap_timeSlotLCR    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlotLCR },
  { &hf_nbap_midambleShiftLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MidambleShiftLCR },
  { &hf_nbap_first_TDD_ChannelisationCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_ChannelisationCode },
  { &hf_nbap_second_TDD_ChannelisationCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_ChannelisationCode },
  { &hf_nbap_hS_SCCH_MaxPower, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Power },
  { &hf_nbap_hS_SICH_Information_LCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_HS_SICH_Information_LCR_PSCH_ReconfRqst },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_HS_SCCH_InformationItem_LCR_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_HS_SCCH_InformationItem_LCR_PSCH_ReconfRqst, HS_SCCH_InformationItem_LCR_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_HS_SCCH_Information_LCR_PSCH_ReconfRqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_SCCH_InformationItem_LCR_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_HS_SCCH_Information_LCR_PSCH_ReconfRqst_item);
}


static const per_sequence_t HS_SCCH_Information_LCR_PSCH_ReconfRqst_sequence_of[1] = {
  { &hf_nbap_HS_SCCH_Information_LCR_PSCH_ReconfRqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_HS_SCCH_InformationItem_LCR_PSCH_ReconfRqst },
};

static int
dissect_nbap_HS_SCCH_Information_LCR_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_HS_SCCH_Information_LCR_PSCH_ReconfRqst, HS_SCCH_Information_LCR_PSCH_ReconfRqst_sequence_of,
                                                  1, 32);

  return offset;
}
static int dissect_hS_SCCH_Information_LCR_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_SCCH_Information_LCR_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_hS_SCCH_Information_LCR_PSCH_ReconfRqst);
}


static const per_sequence_t Add_To_HS_SCCH_Resource_Pool_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_hS_SCCH_Information_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_HS_SCCH_Information_PSCH_ReconfRqst },
  { &hf_nbap_hS_SCCH_Information_LCR_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_HS_SCCH_Information_LCR_PSCH_ReconfRqst },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Add_To_HS_SCCH_Resource_Pool_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Add_To_HS_SCCH_Resource_Pool_PSCH_ReconfRqst, Add_To_HS_SCCH_Resource_Pool_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_id_Add_To_HS_SCCH_Resource_Pool_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Add_To_HS_SCCH_Resource_Pool_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_id_Add_To_HS_SCCH_Resource_Pool_PSCH_ReconfRqst);
}


static const per_sequence_t HS_SICH_InformationModify_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_hsSICH_ID      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_HS_SICH_ID },
  { &hf_nbap_timeSlot       , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TimeSlot },
  { &hf_nbap_midambleShiftAndBurstType, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_MidambleShiftAndBurstType },
  { &hf_nbap_tdd_ChannelisationCode, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TDD_ChannelisationCode },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_HS_SICH_InformationModify_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_HS_SICH_InformationModify_PSCH_ReconfRqst, HS_SICH_InformationModify_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_hS_SICH_Information1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_SICH_InformationModify_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_hS_SICH_Information1);
}


static const per_sequence_t HS_SCCH_InformationModifyItem_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_hS_SCCH_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_HS_SCCH_ID },
  { &hf_nbap_timeSlot       , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TimeSlot },
  { &hf_nbap_midambleShiftAndBurstType, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_MidambleShiftAndBurstType },
  { &hf_nbap_tdd_ChannelisationCode, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TDD_ChannelisationCode },
  { &hf_nbap_hS_SCCH_MaxPower, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_Power },
  { &hf_nbap_hS_SICH_Information1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_HS_SICH_InformationModify_PSCH_ReconfRqst },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_HS_SCCH_InformationModifyItem_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_HS_SCCH_InformationModifyItem_PSCH_ReconfRqst, HS_SCCH_InformationModifyItem_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_HS_SCCH_InformationModify_PSCH_ReconfRqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_SCCH_InformationModifyItem_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_HS_SCCH_InformationModify_PSCH_ReconfRqst_item);
}


static const per_sequence_t HS_SCCH_InformationModify_PSCH_ReconfRqst_sequence_of[1] = {
  { &hf_nbap_HS_SCCH_InformationModify_PSCH_ReconfRqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_HS_SCCH_InformationModifyItem_PSCH_ReconfRqst },
};

static int
dissect_nbap_HS_SCCH_InformationModify_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_HS_SCCH_InformationModify_PSCH_ReconfRqst, HS_SCCH_InformationModify_PSCH_ReconfRqst_sequence_of,
                                                  1, 32);

  return offset;
}
static int dissect_hS_SCCH_InformationModify_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_SCCH_InformationModify_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_hS_SCCH_InformationModify_PSCH_ReconfRqst);
}


static const per_sequence_t HS_SICH_InformationModify_LCR_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_hsSICH_ID      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_HS_SICH_ID },
  { &hf_nbap_timeSlotLCR    , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TimeSlotLCR },
  { &hf_nbap_midambleShiftLCR, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_MidambleShiftLCR },
  { &hf_nbap_tdd_ChannelisationCode, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TDD_ChannelisationCode },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_HS_SICH_InformationModify_LCR_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_HS_SICH_InformationModify_LCR_PSCH_ReconfRqst, HS_SICH_InformationModify_LCR_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_hS_SICH_Information_LCR1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_SICH_InformationModify_LCR_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_hS_SICH_Information_LCR1);
}


static const per_sequence_t HS_SCCH_InformationModifyItem_LCR_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_hS_SCCH_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_HS_SCCH_ID },
  { &hf_nbap_timeSlotLCR    , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TimeSlotLCR },
  { &hf_nbap_midambleShiftLCR, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_MidambleShiftLCR },
  { &hf_nbap_first_TDD_ChannelisationCode, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TDD_ChannelisationCode },
  { &hf_nbap_second_TDD_ChannelisationCode, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TDD_ChannelisationCode },
  { &hf_nbap_hS_SCCH_MaxPower, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_Power },
  { &hf_nbap_hS_SICH_Information_LCR1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_HS_SICH_InformationModify_LCR_PSCH_ReconfRqst },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_HS_SCCH_InformationModifyItem_LCR_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_HS_SCCH_InformationModifyItem_LCR_PSCH_ReconfRqst, HS_SCCH_InformationModifyItem_LCR_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_HS_SCCH_InformationModify_LCR_PSCH_ReconfRqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_SCCH_InformationModifyItem_LCR_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_HS_SCCH_InformationModify_LCR_PSCH_ReconfRqst_item);
}


static const per_sequence_t HS_SCCH_InformationModify_LCR_PSCH_ReconfRqst_sequence_of[1] = {
  { &hf_nbap_HS_SCCH_InformationModify_LCR_PSCH_ReconfRqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_HS_SCCH_InformationModifyItem_LCR_PSCH_ReconfRqst },
};

static int
dissect_nbap_HS_SCCH_InformationModify_LCR_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_HS_SCCH_InformationModify_LCR_PSCH_ReconfRqst, HS_SCCH_InformationModify_LCR_PSCH_ReconfRqst_sequence_of,
                                                  1, 32);

  return offset;
}
static int dissect_hS_SCCH_InformationModify_LCR_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_SCCH_InformationModify_LCR_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_hS_SCCH_InformationModify_LCR_PSCH_ReconfRqst);
}


static const per_sequence_t Modify_HS_SCCH_Resource_Pool_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_hS_SCCH_InformationModify_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_HS_SCCH_InformationModify_PSCH_ReconfRqst },
  { &hf_nbap_hS_SCCH_InformationModify_LCR_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_HS_SCCH_InformationModify_LCR_PSCH_ReconfRqst },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Modify_HS_SCCH_Resource_Pool_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Modify_HS_SCCH_Resource_Pool_PSCH_ReconfRqst, Modify_HS_SCCH_Resource_Pool_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_id_Modify_HS_SCCH_Resource_Pool_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Modify_HS_SCCH_Resource_Pool_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_id_Modify_HS_SCCH_Resource_Pool_PSCH_ReconfRqst);
}


static const per_sequence_t Delete_From_HS_SCCH_Resource_PoolItem_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_hS_SCCH_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_HS_SCCH_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Delete_From_HS_SCCH_Resource_PoolItem_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Delete_From_HS_SCCH_Resource_PoolItem_PSCH_ReconfRqst, Delete_From_HS_SCCH_Resource_PoolItem_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_Delete_From_HS_SCCH_Resource_Pool_PSCH_ReconfRqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Delete_From_HS_SCCH_Resource_PoolItem_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_Delete_From_HS_SCCH_Resource_Pool_PSCH_ReconfRqst_item);
}


static const per_sequence_t Delete_From_HS_SCCH_Resource_Pool_PSCH_ReconfRqst_sequence_of[1] = {
  { &hf_nbap_Delete_From_HS_SCCH_Resource_Pool_PSCH_ReconfRqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_Delete_From_HS_SCCH_Resource_PoolItem_PSCH_ReconfRqst },
};

static int
dissect_nbap_Delete_From_HS_SCCH_Resource_Pool_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_Delete_From_HS_SCCH_Resource_Pool_PSCH_ReconfRqst, Delete_From_HS_SCCH_Resource_Pool_PSCH_ReconfRqst_sequence_of,
                                                  1, 32);

  return offset;
}
static int dissect_id_Delete_From_HS_SCCH_Resource_Pool_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Delete_From_HS_SCCH_Resource_Pool_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_id_Delete_From_HS_SCCH_Resource_Pool_PSCH_ReconfRqst);
}


static const per_sequence_t RL_Specific_DCH_Info_Item_sequence[] = {
  { &hf_nbap_dCH_id         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DCH_ID },
  { &hf_nbap_bindingID      , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_BindingID },
  { &hf_nbap_transportlayeraddress, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TransportLayerAddress },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RL_Specific_DCH_Info_Item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RL_Specific_DCH_Info_Item, RL_Specific_DCH_Info_Item_sequence);

  return offset;
}
static int dissect_RL_Specific_DCH_Info_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_Specific_DCH_Info_Item(tvb, offset, actx, tree, hf_nbap_RL_Specific_DCH_Info_item);
}


static const per_sequence_t RL_Specific_DCH_Info_sequence_of[1] = {
  { &hf_nbap_RL_Specific_DCH_Info_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_RL_Specific_DCH_Info_Item },
};

static int
dissect_nbap_RL_Specific_DCH_Info(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_RL_Specific_DCH_Info, RL_Specific_DCH_Info_sequence_of,
                                                  1, 128);

  return offset;
}
static int dissect_id_RL_Specific_DCH_Info(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_Specific_DCH_Info(tvb, offset, actx, tree, hf_nbap_id_RL_Specific_DCH_Info);
}


static const value_string nbap_DelayedActivation_vals[] = {
  {   0, "cfn" },
  {   1, "separate-indication" },
  { 0, NULL }
};

static const per_choice_t DelayedActivation_choice[] = {
  {   0, &hf_nbap_cfn            , ASN1_NO_EXTENSIONS     , dissect_nbap_CFN },
  {   1, &hf_nbap_separate_indication, ASN1_NO_EXTENSIONS     , dissect_nbap_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_DelayedActivation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_DelayedActivation, DelayedActivation_choice,
                                 NULL);

  return offset;
}
static int dissect_id_DelayedActivation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DelayedActivation(tvb, offset, actx, tree, hf_nbap_id_DelayedActivation);
}


static const per_sequence_t DelayedActivationInformationList_RL_ActivationCmdFDD_sequence_of[1] = {
  { &hf_nbap_DelayedActivationInformationList_RL_ActivationCmdFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_DelayedActivationInformationList_RL_ActivationCmdFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DelayedActivationInformationList_RL_ActivationCmdFDD, DelayedActivationInformationList_RL_ActivationCmdFDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_DelayedActivationList_RL_ActivationCmdFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DelayedActivationInformationList_RL_ActivationCmdFDD(tvb, offset, actx, tree, hf_nbap_id_DelayedActivationList_RL_ActivationCmdFDD);
}


static const value_string nbap_Execution_Type_vals[] = {
  {   0, "synchronised" },
  {   1, "unsynchronised" },
  { 0, NULL }
};

static const per_choice_t Execution_Type_choice[] = {
  {   0, &hf_nbap_synchronised   , ASN1_NO_EXTENSIONS     , dissect_nbap_CFN },
  {   1, &hf_nbap_unsynchronised , ASN1_NO_EXTENSIONS     , dissect_nbap_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_Execution_Type(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_Execution_Type, Execution_Type_choice,
                                 NULL);

  return offset;
}
static int dissect_activation_type(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Execution_Type(tvb, offset, actx, tree, hf_nbap_activation_type);
}
static int dissect_deactivation_type(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Execution_Type(tvb, offset, actx, tree, hf_nbap_deactivation_type);
}


static const per_sequence_t Activate_Info_sequence[] = {
  { &hf_nbap_activation_type, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Execution_Type },
  { &hf_nbap_initial_dl_tx_power, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Power },
  { &hf_nbap_firstRLS_Indicator, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_FirstRLS_Indicator },
  { &hf_nbap_propagation_delay, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_PropagationDelay },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Activate_Info(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Activate_Info, Activate_Info_sequence);

  return offset;
}
static int dissect_activate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Activate_Info(tvb, offset, actx, tree, hf_nbap_activate);
}


static const per_sequence_t Deactivate_Info_sequence[] = {
  { &hf_nbap_deactivation_type, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Execution_Type },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Deactivate_Info(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Deactivate_Info, Deactivate_Info_sequence);

  return offset;
}
static int dissect_deactivate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Deactivate_Info(tvb, offset, actx, tree, hf_nbap_deactivate);
}


static const value_string nbap_DelayedActivationUpdate_vals[] = {
  {   0, "activate" },
  {   1, "deactivate" },
  { 0, NULL }
};

static const per_choice_t DelayedActivationUpdate_choice[] = {
  {   0, &hf_nbap_activate       , ASN1_NO_EXTENSIONS     , dissect_nbap_Activate_Info },
  {   1, &hf_nbap_deactivate     , ASN1_NO_EXTENSIONS     , dissect_nbap_Deactivate_Info },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_DelayedActivationUpdate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_DelayedActivationUpdate, DelayedActivationUpdate_choice,
                                 NULL);

  return offset;
}
static int dissect_delayed_activation_update(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DelayedActivationUpdate(tvb, offset, actx, tree, hf_nbap_delayed_activation_update);
}


static const per_sequence_t DelayedActivationInformation_RL_ActivationCmdFDD_sequence[] = {
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_ID },
  { &hf_nbap_delayed_activation_update, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DelayedActivationUpdate },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DelayedActivationInformation_RL_ActivationCmdFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DelayedActivationInformation_RL_ActivationCmdFDD, DelayedActivationInformation_RL_ActivationCmdFDD_sequence);

  return offset;
}
static int dissect_id_DelayedActivationInformation_RL_ActivationCmdFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DelayedActivationInformation_RL_ActivationCmdFDD(tvb, offset, actx, tree, hf_nbap_id_DelayedActivationInformation_RL_ActivationCmdFDD);
}


static const per_sequence_t DelayedActivationInformationList_RL_ActivationCmdTDD_sequence_of[1] = {
  { &hf_nbap_DelayedActivationInformationList_RL_ActivationCmdTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_DelayedActivationInformationList_RL_ActivationCmdTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DelayedActivationInformationList_RL_ActivationCmdTDD, DelayedActivationInformationList_RL_ActivationCmdTDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_DelayedActivationList_RL_ActivationCmdTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DelayedActivationInformationList_RL_ActivationCmdTDD(tvb, offset, actx, tree, hf_nbap_id_DelayedActivationList_RL_ActivationCmdTDD);
}


static const per_sequence_t DelayedActivationInformation_RL_ActivationCmdTDD_sequence[] = {
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_ID },
  { &hf_nbap_delayed_activation_update, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DelayedActivationUpdate },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DelayedActivationInformation_RL_ActivationCmdTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DelayedActivationInformation_RL_ActivationCmdTDD, DelayedActivationInformation_RL_ActivationCmdTDD_sequence);

  return offset;
}
static int dissect_id_DelayedActivationInformation_RL_ActivationCmdTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DelayedActivationInformation_RL_ActivationCmdTDD(tvb, offset, actx, tree, hf_nbap_id_DelayedActivationInformation_RL_ActivationCmdTDD);
}


static const per_sequence_t NeighbouringTDDCellMeasurementInformationLCR_sequence[] = {
  { &hf_nbap_uC_Id          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UC_Id },
  { &hf_nbap_uARFCN         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UARFCN },
  { &hf_nbap_cellParameterID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CellParameterID },
  { &hf_nbap_timeSlotLCR    , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TimeSlotLCR },
  { &hf_nbap_midambleShiftLCR, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_MidambleShiftLCR },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_NeighbouringTDDCellMeasurementInformationLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_NeighbouringTDDCellMeasurementInformationLCR, NeighbouringTDDCellMeasurementInformationLCR_sequence);

  return offset;
}
static int dissect_id_neighbouringTDDCellMeasurementInformationLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NeighbouringTDDCellMeasurementInformationLCR(tvb, offset, actx, tree, hf_nbap_id_neighbouringTDDCellMeasurementInformationLCR);
}



static int
dissect_nbap_SYNCDlCodeId(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 32U, NULL, TRUE);

  return offset;
}
static int dissect_sYNCDlCodeId(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SYNCDlCodeId(tvb, offset, actx, tree, hf_nbap_sYNCDlCodeId);
}
static int dissect_syncDLCodeId(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SYNCDlCodeId(tvb, offset, actx, tree, hf_nbap_syncDLCodeId);
}


static const per_sequence_t SYNCDlCodeId_TransInitLCR_CellSyncInitiationRqstTDD_sequence[] = {
  { &hf_nbap_cSBTransmissionID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CSBTransmissionID },
  { &hf_nbap_sfn            , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SFN },
  { &hf_nbap_uARFCN         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UARFCN },
  { &hf_nbap_sYNCDlCodeId   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SYNCDlCodeId },
  { &hf_nbap_dwPCH_Power    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DwPCH_Power },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_SYNCDlCodeId_TransInitLCR_CellSyncInitiationRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_SYNCDlCodeId_TransInitLCR_CellSyncInitiationRqstTDD, SYNCDlCodeId_TransInitLCR_CellSyncInitiationRqstTDD_sequence);

  return offset;
}
static int dissect_id_SYNCDlCodeId_TransInitLCR_CellSyncInitiationRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SYNCDlCodeId_TransInitLCR_CellSyncInitiationRqstTDD(tvb, offset, actx, tree, hf_nbap_id_SYNCDlCodeId_TransInitLCR_CellSyncInitiationRqstTDD);
}


static const per_sequence_t SYNCDlCodeId_MeasureInitLCR_CellSyncInitiationRqstTDD_sequence[] = {
  { &hf_nbap_cSBMeasurementID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CSBMeasurementID },
  { &hf_nbap_sfn            , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_SFN },
  { &hf_nbap_uARFCN         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UARFCN },
  { &hf_nbap_sYNCDlCodeId   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SYNCDlCodeId },
  { &hf_nbap_synchronisationReportType, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SynchronisationReportType },
  { &hf_nbap_synchronisationReportCharacteristics, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SynchronisationReportCharacteristics },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_SYNCDlCodeId_MeasureInitLCR_CellSyncInitiationRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_SYNCDlCodeId_MeasureInitLCR_CellSyncInitiationRqstTDD, SYNCDlCodeId_MeasureInitLCR_CellSyncInitiationRqstTDD_sequence);

  return offset;
}
static int dissect_id_SYNCDlCodeId_MeasureInitLCR_CellSyncInitiationRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SYNCDlCodeId_MeasureInitLCR_CellSyncInitiationRqstTDD(tvb, offset, actx, tree, hf_nbap_id_SYNCDlCodeId_MeasureInitLCR_CellSyncInitiationRqstTDD);
}


static const per_sequence_t SYNCDlCodeIdTransReconfItemLCR_CellSyncReconfRqstTDD_sequence[] = {
  { &hf_nbap_cSBTransmissionID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CSBTransmissionID },
  { &hf_nbap_syncFrameNumberforTransmit, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SyncFrameNumber },
  { &hf_nbap_uARFCN         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UARFCN },
  { &hf_nbap_sYNCDlCodeId   , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_SYNCDlCodeId },
  { &hf_nbap_dwPCH_Power    , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DwPCH_Power },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_SYNCDlCodeIdTransReconfItemLCR_CellSyncReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_SYNCDlCodeIdTransReconfItemLCR_CellSyncReconfRqstTDD, SYNCDlCodeIdTransReconfItemLCR_CellSyncReconfRqstTDD_sequence);

  return offset;
}
static int dissect_SYNCDlCodeIdTransReconfInfoLCR_CellSyncReconfRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SYNCDlCodeIdTransReconfItemLCR_CellSyncReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_SYNCDlCodeIdTransReconfInfoLCR_CellSyncReconfRqstTDD_item);
}


static const per_sequence_t SYNCDlCodeIdTransReconfInfoLCR_CellSyncReconfRqstTDD_sequence_of[1] = {
  { &hf_nbap_SYNCDlCodeIdTransReconfInfoLCR_CellSyncReconfRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_SYNCDlCodeIdTransReconfItemLCR_CellSyncReconfRqstTDD },
};

static int
dissect_nbap_SYNCDlCodeIdTransReconfInfoLCR_CellSyncReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_SYNCDlCodeIdTransReconfInfoLCR_CellSyncReconfRqstTDD, SYNCDlCodeIdTransReconfInfoLCR_CellSyncReconfRqstTDD_sequence_of,
                                                  1, 512);

  return offset;
}
static int dissect_id_SYNCDlCodeIdTransReconfInfoLCR_CellSyncReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SYNCDlCodeIdTransReconfInfoLCR_CellSyncReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_id_SYNCDlCodeIdTransReconfInfoLCR_CellSyncReconfRqstTDD);
}



static int
dissect_nbap_INTEGER_0_524287_(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 524287U, NULL, TRUE);

  return offset;
}
static int dissect_initialPhase(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_INTEGER_0_524287_(tvb, offset, actx, tree, hf_nbap_initialPhase);
}



static int
dissect_nbap_INTEGER_0_127_(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 127U, NULL, TRUE);

  return offset;
}
static int dissect_steadyStatePhase1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_INTEGER_0_127_(tvb, offset, actx, tree, hf_nbap_steadyStatePhase1);
}


static const value_string nbap_TimingAdjustmentValueLCR_vals[] = {
  {   0, "initialPhase" },
  {   1, "steadyStatePhase" },
  { 0, NULL }
};

static const per_choice_t TimingAdjustmentValueLCR_choice[] = {
  {   0, &hf_nbap_initialPhase   , ASN1_NO_EXTENSIONS     , dissect_nbap_INTEGER_0_524287_ },
  {   1, &hf_nbap_steadyStatePhase1, ASN1_NO_EXTENSIONS     , dissect_nbap_INTEGER_0_127_ },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_TimingAdjustmentValueLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_TimingAdjustmentValueLCR, TimingAdjustmentValueLCR_choice,
                                 NULL);

  return offset;
}
static int dissect_id_TimingAdjustmentValueLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TimingAdjustmentValueLCR(tvb, offset, actx, tree, hf_nbap_id_TimingAdjustmentValueLCR);
}
static int dissect_propagationDelayCompensation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TimingAdjustmentValueLCR(tvb, offset, actx, tree, hf_nbap_propagationDelayCompensation);
}


static const per_sequence_t SYNCDlCodeIdInfoItemLCR_CellSyncReconfRqstTDD_sequence[] = {
  { &hf_nbap_cSBMeasurementID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CSBMeasurementID },
  { &hf_nbap_sYNCDlCodeId   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SYNCDlCodeId },
  { &hf_nbap_uARFCN         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UARFCN },
  { &hf_nbap_propagationDelayCompensation, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TimingAdjustmentValueLCR },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_SYNCDlCodeIdInfoItemLCR_CellSyncReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_SYNCDlCodeIdInfoItemLCR_CellSyncReconfRqstTDD, SYNCDlCodeIdInfoItemLCR_CellSyncReconfRqstTDD_sequence);

  return offset;
}
static int dissect_SYNCDlCodeIdInfoListLCR_CellSyncReconfRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SYNCDlCodeIdInfoItemLCR_CellSyncReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_SYNCDlCodeIdInfoListLCR_CellSyncReconfRqstTDD_item);
}


static const per_sequence_t SYNCDlCodeIdInfoListLCR_CellSyncReconfRqstTDD_sequence_of[1] = {
  { &hf_nbap_SYNCDlCodeIdInfoListLCR_CellSyncReconfRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_SYNCDlCodeIdInfoItemLCR_CellSyncReconfRqstTDD },
};

static int
dissect_nbap_SYNCDlCodeIdInfoListLCR_CellSyncReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_SYNCDlCodeIdInfoListLCR_CellSyncReconfRqstTDD, SYNCDlCodeIdInfoListLCR_CellSyncReconfRqstTDD_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_sYNCDlCodeIdInfoLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SYNCDlCodeIdInfoListLCR_CellSyncReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_sYNCDlCodeIdInfoLCR);
}


static const per_sequence_t SYNCDlCodeIdMeasInfoItem_CellSyncReconfRqstTDD_sequence[] = {
  { &hf_nbap_syncFrameNrToReceive, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SyncFrameNumber },
  { &hf_nbap_sYNCDlCodeIdInfoLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SYNCDlCodeIdInfoListLCR_CellSyncReconfRqstTDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_SYNCDlCodeIdMeasInfoItem_CellSyncReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_SYNCDlCodeIdMeasInfoItem_CellSyncReconfRqstTDD, SYNCDlCodeIdMeasInfoItem_CellSyncReconfRqstTDD_sequence);

  return offset;
}
static int dissect_SYNCDlCodeIdMeasInfoList_CellSyncReconfRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SYNCDlCodeIdMeasInfoItem_CellSyncReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_SYNCDlCodeIdMeasInfoList_CellSyncReconfRqstTDD_item);
}


static const per_sequence_t SYNCDlCodeIdMeasInfoList_CellSyncReconfRqstTDD_sequence_of[1] = {
  { &hf_nbap_SYNCDlCodeIdMeasInfoList_CellSyncReconfRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_SYNCDlCodeIdMeasInfoItem_CellSyncReconfRqstTDD },
};

static int
dissect_nbap_SYNCDlCodeIdMeasInfoList_CellSyncReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_SYNCDlCodeIdMeasInfoList_CellSyncReconfRqstTDD, SYNCDlCodeIdMeasInfoList_CellSyncReconfRqstTDD_sequence_of,
                                                  1, 32);

  return offset;
}
static int dissect_id_SYNCDlCodeIdMeasInfoList_CellSyncReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SYNCDlCodeIdMeasInfoList_CellSyncReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_id_SYNCDlCodeIdMeasInfoList_CellSyncReconfRqstTDD);
}
static int dissect_sYNCDlCodeIdMeasInfoList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SYNCDlCodeIdMeasInfoList_CellSyncReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_sYNCDlCodeIdMeasInfoList);
}


static const per_sequence_t SYNCDlCodeIdMeasInfoLCR_CellSyncReconfRqstTDD_sequence[] = {
  { &hf_nbap_sYNCDlCodeIdMeasInfoList, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SYNCDlCodeIdMeasInfoList_CellSyncReconfRqstTDD },
  { &hf_nbap_synchronisationReportType, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_SynchronisationReportType },
  { &hf_nbap_synchronisationReportCharacteristics, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_SynchronisationReportCharacteristics },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_SYNCDlCodeIdMeasInfoLCR_CellSyncReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_SYNCDlCodeIdMeasInfoLCR_CellSyncReconfRqstTDD, SYNCDlCodeIdMeasInfoLCR_CellSyncReconfRqstTDD_sequence);

  return offset;
}
static int dissect_id_SYNCDlCodeIdMeasReconfigurationLCR_CellSyncReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SYNCDlCodeIdMeasInfoLCR_CellSyncReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_id_SYNCDlCodeIdMeasReconfigurationLCR_CellSyncReconfRqstTDD);
}


static const value_string nbap_CellSyncBurstTimingLCR_vals[] = {
  {   0, "initialPhase" },
  {   1, "steadyStatePhase" },
  { 0, NULL }
};

static const per_choice_t CellSyncBurstTimingLCR_choice[] = {
  {   0, &hf_nbap_initialPhase   , ASN1_NO_EXTENSIONS     , dissect_nbap_INTEGER_0_524287_ },
  {   1, &hf_nbap_steadyStatePhase1, ASN1_NO_EXTENSIONS     , dissect_nbap_INTEGER_0_127_ },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_CellSyncBurstTimingLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_CellSyncBurstTimingLCR, CellSyncBurstTimingLCR_choice,
                                 NULL);

  return offset;
}
static int dissect_syncDLCodeIdTiming(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSyncBurstTimingLCR(tvb, offset, actx, tree, hf_nbap_syncDLCodeIdTiming);
}
static int dissect_syncDLCodeIdArrivTime(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSyncBurstTimingLCR(tvb, offset, actx, tree, hf_nbap_syncDLCodeIdArrivTime);
}


static const per_sequence_t SyncDLCodeIdAvailable_CellSyncReprtTDD_sequence[] = {
  { &hf_nbap_syncDLCodeIdTiming, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CellSyncBurstTimingLCR },
  { &hf_nbap_syncDLCodeIdSIR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CellSyncBurstSIR },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_SyncDLCodeIdAvailable_CellSyncReprtTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_SyncDLCodeIdAvailable_CellSyncReprtTDD, SyncDLCodeIdAvailable_CellSyncReprtTDD_sequence);

  return offset;
}
static int dissect_syncDLCodeIdAvailable(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SyncDLCodeIdAvailable_CellSyncReprtTDD(tvb, offset, actx, tree, hf_nbap_syncDLCodeIdAvailable);
}


static const value_string nbap_SyncDLCodeIdItem_CellSyncReprtTDD_vals[] = {
  {   0, "syncDLCodeIdAvailable" },
  {   1, "syncDLCodeIDNotAvailable" },
  { 0, NULL }
};

static const per_choice_t SyncDLCodeIdItem_CellSyncReprtTDD_choice[] = {
  {   0, &hf_nbap_syncDLCodeIdAvailable, ASN1_EXTENSION_ROOT    , dissect_nbap_SyncDLCodeIdAvailable_CellSyncReprtTDD },
  {   1, &hf_nbap_syncDLCodeIDNotAvailable, ASN1_EXTENSION_ROOT    , dissect_nbap_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_SyncDLCodeIdItem_CellSyncReprtTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_SyncDLCodeIdItem_CellSyncReprtTDD, SyncDLCodeIdItem_CellSyncReprtTDD_choice,
                                 NULL);

  return offset;
}
static int dissect_SyncDLCodeIdInfo_CellSyncReprtTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SyncDLCodeIdItem_CellSyncReprtTDD(tvb, offset, actx, tree, hf_nbap_SyncDLCodeIdInfo_CellSyncReprtTDD_item);
}


static const per_sequence_t SyncDLCodeIdInfo_CellSyncReprtTDD_sequence_of[1] = {
  { &hf_nbap_SyncDLCodeIdInfo_CellSyncReprtTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_SyncDLCodeIdItem_CellSyncReprtTDD },
};

static int
dissect_nbap_SyncDLCodeIdInfo_CellSyncReprtTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_SyncDLCodeIdInfo_CellSyncReprtTDD, SyncDLCodeIdInfo_CellSyncReprtTDD_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_syncDLCodeIdInfo_CellSyncReprtTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SyncDLCodeIdInfo_CellSyncReprtTDD(tvb, offset, actx, tree, hf_nbap_syncDLCodeIdInfo_CellSyncReprtTDD);
}


static const per_sequence_t SyncDLCodeIdsMeasInfoItem_CellSyncReprtTDD_sequence[] = {
  { &hf_nbap_sFN            , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SFN },
  { &hf_nbap_syncDLCodeIdInfo_CellSyncReprtTDD, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SyncDLCodeIdInfo_CellSyncReprtTDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_SyncDLCodeIdsMeasInfoItem_CellSyncReprtTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_SyncDLCodeIdsMeasInfoItem_CellSyncReprtTDD, SyncDLCodeIdsMeasInfoItem_CellSyncReprtTDD_sequence);

  return offset;
}
static int dissect_SyncDLCodeIdsMeasInfoList_CellSyncReprtTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SyncDLCodeIdsMeasInfoItem_CellSyncReprtTDD(tvb, offset, actx, tree, hf_nbap_SyncDLCodeIdsMeasInfoList_CellSyncReprtTDD_item);
}


static const per_sequence_t SyncDLCodeIdsMeasInfoList_CellSyncReprtTDD_sequence_of[1] = {
  { &hf_nbap_SyncDLCodeIdsMeasInfoList_CellSyncReprtTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_SyncDLCodeIdsMeasInfoItem_CellSyncReprtTDD },
};

static int
dissect_nbap_SyncDLCodeIdsMeasInfoList_CellSyncReprtTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_SyncDLCodeIdsMeasInfoList_CellSyncReprtTDD, SyncDLCodeIdsMeasInfoList_CellSyncReprtTDD_sequence_of,
                                                  0, 512);

  return offset;
}
static int dissect_id_SyncDLCodeIdsMeasInfoList_CellSyncReprtTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SyncDLCodeIdsMeasInfoList_CellSyncReprtTDD(tvb, offset, actx, tree, hf_nbap_id_SyncDLCodeIdsMeasInfoList_CellSyncReprtTDD);
}


static const per_sequence_t SyncDLCodeInfoItemLCR_sequence[] = {
  { &hf_nbap_syncDLCodeId   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SYNCDlCodeId },
  { &hf_nbap_syncDLCodeIdArrivTime, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_CellSyncBurstTimingLCR },
  { &hf_nbap_syncDLCodeIdTimingThre, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_CellSyncBurstTimingThreshold },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_SyncDLCodeInfoItemLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_SyncDLCodeInfoItemLCR, SyncDLCodeInfoItemLCR_sequence);

  return offset;
}
static int dissect_SyncDLCodeInfoListLCR_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SyncDLCodeInfoItemLCR(tvb, offset, actx, tree, hf_nbap_SyncDLCodeInfoListLCR_item);
}


static const per_sequence_t SyncDLCodeInfoListLCR_sequence_of[1] = {
  { &hf_nbap_SyncDLCodeInfoListLCR_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_SyncDLCodeInfoItemLCR },
};

static int
dissect_nbap_SyncDLCodeInfoListLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_SyncDLCodeInfoListLCR, SyncDLCodeInfoListLCR_sequence_of,
                                                  1, 32);

  return offset;
}
static int dissect_syncDLCodeIdInfoLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SyncDLCodeInfoListLCR(tvb, offset, actx, tree, hf_nbap_syncDLCodeIdInfoLCR);
}


static const per_sequence_t SyncDLCodeIdThreInfoList_sequence[] = {
  { &hf_nbap_syncFrameNoToReceive, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SyncFrameNumber },
  { &hf_nbap_syncDLCodeIdInfoLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SyncDLCodeInfoListLCR },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_SyncDLCodeIdThreInfoList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_SyncDLCodeIdThreInfoList, SyncDLCodeIdThreInfoList_sequence);

  return offset;
}
static int dissect_SyncDLCodeIdThreInfoLCR_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SyncDLCodeIdThreInfoList(tvb, offset, actx, tree, hf_nbap_SyncDLCodeIdThreInfoLCR_item);
}


static const per_sequence_t SyncDLCodeIdThreInfoLCR_sequence_of[1] = {
  { &hf_nbap_SyncDLCodeIdThreInfoLCR_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_SyncDLCodeIdThreInfoList },
};

static int
dissect_nbap_SyncDLCodeIdThreInfoLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_SyncDLCodeIdThreInfoLCR, SyncDLCodeIdThreInfoLCR_sequence_of,
                                                  0, 512);

  return offset;
}
static int dissect_id_SyncDLCodeIdThreInfoLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SyncDLCodeIdThreInfoLCR(tvb, offset, actx, tree, hf_nbap_id_SyncDLCodeIdThreInfoLCR);
}



static int
dissect_nbap_NSubCyclesPerCyclePeriod(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 16U, NULL, TRUE);

  return offset;
}
static int dissect_id_NSubCyclesPerCyclePeriod_CellSyncReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NSubCyclesPerCyclePeriod(tvb, offset, actx, tree, hf_nbap_id_NSubCyclesPerCyclePeriod_CellSyncReconfRqstTDD);
}



static int
dissect_nbap_AOA_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 719U, NULL, FALSE);

  return offset;
}
static int dissect_aOA_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_AOA_LCR(tvb, offset, actx, tree, hf_nbap_aOA_LCR);
}


static const value_string nbap_AOA_LCR_Accuracy_Class_vals[] = {
  {   0, "a" },
  {   1, "b" },
  {   2, "c" },
  {   3, "d" },
  {   4, "e" },
  {   5, "f" },
  {   6, "g" },
  {   7, "h" },
  { 0, NULL }
};


static int
dissect_nbap_AOA_LCR_Accuracy_Class(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_aOA_LCR_Accuracy_Class(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_AOA_LCR_Accuracy_Class(tvb, offset, actx, tree, hf_nbap_aOA_LCR_Accuracy_Class);
}


static const per_sequence_t Angle_Of_Arrival_Value_LCR_sequence[] = {
  { &hf_nbap_aOA_LCR        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_AOA_LCR },
  { &hf_nbap_aOA_LCR_Accuracy_Class, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_AOA_LCR_Accuracy_Class },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Angle_Of_Arrival_Value_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Angle_Of_Arrival_Value_LCR, Angle_Of_Arrival_Value_LCR_sequence);

  return offset;
}
static int dissect_id_Angle_Of_Arrival_Value_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Angle_Of_Arrival_Value_LCR(tvb, offset, actx, tree, hf_nbap_id_Angle_Of_Arrival_Value_LCR);
}



static int
dissect_nbap_HSDSCH_MACdFlow_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 7U, NULL, FALSE);

  return offset;
}
static int dissect_hsDSCH_MACdFlow_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HSDSCH_MACdFlow_ID(tvb, offset, actx, tree, hf_nbap_hsDSCH_MACdFlow_ID);
}
static int dissect_hsDSCHMacdFlow_Id(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HSDSCH_MACdFlow_ID(tvb, offset, actx, tree, hf_nbap_hsDSCHMacdFlow_Id);
}
static int dissect_associatedHSDSCH_MACdFlow(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HSDSCH_MACdFlow_ID(tvb, offset, actx, tree, hf_nbap_associatedHSDSCH_MACdFlow);
}


static const per_sequence_t HSDSCH_MACdFlow_Specific_InfoItem_sequence[] = {
  { &hf_nbap_hsDSCH_MACdFlow_ID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_HSDSCH_MACdFlow_ID },
  { &hf_nbap_allocationRetentionPriority, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_AllocationRetentionPriority },
  { &hf_nbap_bindingID      , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_BindingID },
  { &hf_nbap_transportLayerAddress, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TransportLayerAddress },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_HSDSCH_MACdFlow_Specific_InfoItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_HSDSCH_MACdFlow_Specific_InfoItem, HSDSCH_MACdFlow_Specific_InfoItem_sequence);

  return offset;
}
static int dissect_HSDSCH_MACdFlow_Specific_InfoList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HSDSCH_MACdFlow_Specific_InfoItem(tvb, offset, actx, tree, hf_nbap_HSDSCH_MACdFlow_Specific_InfoList_item);
}


static const per_sequence_t HSDSCH_MACdFlow_Specific_InfoList_sequence_of[1] = {
  { &hf_nbap_HSDSCH_MACdFlow_Specific_InfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_HSDSCH_MACdFlow_Specific_InfoItem },
};

static int
dissect_nbap_HSDSCH_MACdFlow_Specific_InfoList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_HSDSCH_MACdFlow_Specific_InfoList, HSDSCH_MACdFlow_Specific_InfoList_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_hSDSCH_MACdFlow_Specific_Info(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HSDSCH_MACdFlow_Specific_InfoList(tvb, offset, actx, tree, hf_nbap_hSDSCH_MACdFlow_Specific_Info);
}



static int
dissect_nbap_PriorityQueue_Id(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 7U, NULL, FALSE);

  return offset;
}
static int dissect_deletePriorityQueue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PriorityQueue_Id(tvb, offset, actx, tree, hf_nbap_deletePriorityQueue);
}
static int dissect_priorityQueueId(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PriorityQueue_Id(tvb, offset, actx, tree, hf_nbap_priorityQueueId);
}



static int
dissect_nbap_SchedulingPriorityIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 15U, NULL, FALSE);

  return offset;
}
static int dissect_schedulingPriorityIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SchedulingPriorityIndicator(tvb, offset, actx, tree, hf_nbap_schedulingPriorityIndicator);
}


static const value_string nbap_T1_vals[] = {
  {   0, "v10" },
  {   1, "v20" },
  {   2, "v30" },
  {   3, "v40" },
  {   4, "v50" },
  {   5, "v60" },
  {   6, "v70" },
  {   7, "v80" },
  {   8, "v90" },
  {   9, "v100" },
  {  10, "v120" },
  {  11, "v140" },
  {  12, "v160" },
  {  13, "v200" },
  {  14, "v300" },
  {  15, "v400" },
  { 0, NULL }
};


static int
dissect_nbap_T1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_t1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_T1(tvb, offset, actx, tree, hf_nbap_t1);
}


static const value_string nbap_DiscardTimer_vals[] = {
  {   0, "v20" },
  {   1, "v40" },
  {   2, "v60" },
  {   3, "v80" },
  {   4, "v100" },
  {   5, "v120" },
  {   6, "v140" },
  {   7, "v160" },
  {   8, "v180" },
  {   9, "v200" },
  {  10, "v250" },
  {  11, "v300" },
  {  12, "v400" },
  {  13, "v500" },
  {  14, "v750" },
  {  15, "v1000" },
  {  16, "v1250" },
  {  17, "v1500" },
  {  18, "v1750" },
  {  19, "v2000" },
  {  20, "v2500" },
  {  21, "v3000" },
  {  22, "v3500" },
  {  23, "v4000" },
  {  24, "v4500" },
  {  25, "v5000" },
  {  26, "v7500" },
  { 0, NULL }
};


static int
dissect_nbap_DiscardTimer(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     27, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_discardTimer(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DiscardTimer(tvb, offset, actx, tree, hf_nbap_discardTimer);
}


static const value_string nbap_MAC_hsWindowSize_vals[] = {
  {   0, "v4" },
  {   1, "v6" },
  {   2, "v8" },
  {   3, "v12" },
  {   4, "v16" },
  {   5, "v24" },
  {   6, "v32" },
  { 0, NULL }
};


static int
dissect_nbap_MAC_hsWindowSize(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     7, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_mAC_hsWindowSize(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MAC_hsWindowSize(tvb, offset, actx, tree, hf_nbap_mAC_hsWindowSize);
}



static int
dissect_nbap_MAChsGuaranteedBitRate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 16777215U, NULL, TRUE);

  return offset;
}
static int dissect_mAChsGuaranteedBitRate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MAChsGuaranteedBitRate(tvb, offset, actx, tree, hf_nbap_mAChsGuaranteedBitRate);
}



static int
dissect_nbap_SID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 7U, NULL, FALSE);

  return offset;
}
static int dissect_sID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SID(tvb, offset, actx, tree, hf_nbap_sID);
}



static int
dissect_nbap_MACdPDU_Size(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 5000U, NULL, TRUE);

  return offset;
}
static int dissect_mACdPDU_Size(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MACdPDU_Size(tvb, offset, actx, tree, hf_nbap_mACdPDU_Size);
}
static int dissect_maximum_MACdPDU_Size(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MACdPDU_Size(tvb, offset, actx, tree, hf_nbap_maximum_MACdPDU_Size);
}
static int dissect_macdPDU_Size(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MACdPDU_Size(tvb, offset, actx, tree, hf_nbap_macdPDU_Size);
}


static const per_sequence_t MACdPDU_Size_IndexItem_sequence[] = {
  { &hf_nbap_sID            , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SID },
  { &hf_nbap_macdPDU_Size   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MACdPDU_Size },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_MACdPDU_Size_IndexItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_MACdPDU_Size_IndexItem, MACdPDU_Size_IndexItem_sequence);

  return offset;
}
static int dissect_MACdPDU_Size_Indexlist_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MACdPDU_Size_IndexItem(tvb, offset, actx, tree, hf_nbap_MACdPDU_Size_Indexlist_item);
}


static const per_sequence_t MACdPDU_Size_Indexlist_sequence_of[1] = {
  { &hf_nbap_MACdPDU_Size_Indexlist_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_MACdPDU_Size_IndexItem },
};

static int
dissect_nbap_MACdPDU_Size_Indexlist(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_MACdPDU_Size_Indexlist, MACdPDU_Size_Indexlist_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_macdPDU_Size_Index(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MACdPDU_Size_Indexlist(tvb, offset, actx, tree, hf_nbap_macdPDU_Size_Index);
}


static const value_string nbap_RLC_Mode_vals[] = {
  {   0, "rLC-AM" },
  {   1, "rLC-UM" },
  { 0, NULL }
};


static int
dissect_nbap_RLC_Mode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_rLC_Mode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RLC_Mode(tvb, offset, actx, tree, hf_nbap_rLC_Mode);
}


static const per_sequence_t PriorityQueue_InfoItem_sequence[] = {
  { &hf_nbap_priorityQueueId, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PriorityQueue_Id },
  { &hf_nbap_associatedHSDSCH_MACdFlow, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_HSDSCH_MACdFlow_ID },
  { &hf_nbap_schedulingPriorityIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SchedulingPriorityIndicator },
  { &hf_nbap_t1             , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_T1 },
  { &hf_nbap_discardTimer   , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DiscardTimer },
  { &hf_nbap_mAC_hsWindowSize, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MAC_hsWindowSize },
  { &hf_nbap_mAChsGuaranteedBitRate, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_MAChsGuaranteedBitRate },
  { &hf_nbap_macdPDU_Size_Index, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MACdPDU_Size_Indexlist },
  { &hf_nbap_rLC_Mode       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RLC_Mode },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PriorityQueue_InfoItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PriorityQueue_InfoItem, PriorityQueue_InfoItem_sequence);

  return offset;
}
static int dissect_PriorityQueue_InfoList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PriorityQueue_InfoItem(tvb, offset, actx, tree, hf_nbap_PriorityQueue_InfoList_item);
}


static const per_sequence_t PriorityQueue_InfoList_sequence_of[1] = {
  { &hf_nbap_PriorityQueue_InfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_PriorityQueue_InfoItem },
};

static int
dissect_nbap_PriorityQueue_InfoList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_PriorityQueue_InfoList, PriorityQueue_InfoList_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_priorityQueue_Info(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PriorityQueue_InfoList(tvb, offset, actx, tree, hf_nbap_priorityQueue_Info);
}


static const per_sequence_t HSDSCH_MACdFlows_Information_sequence[] = {
  { &hf_nbap_hSDSCH_MACdFlow_Specific_Info, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_HSDSCH_MACdFlow_Specific_InfoList },
  { &hf_nbap_priorityQueue_Info, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PriorityQueue_InfoList },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_HSDSCH_MACdFlows_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_HSDSCH_MACdFlows_Information, HSDSCH_MACdFlows_Information_sequence);

  return offset;
}
static int dissect_id_HSDSCH_MACdFlows_to_Add(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HSDSCH_MACdFlows_Information(tvb, offset, actx, tree, hf_nbap_id_HSDSCH_MACdFlows_to_Add);
}
static int dissect_hSDSCH_MACdFlows_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HSDSCH_MACdFlows_Information(tvb, offset, actx, tree, hf_nbap_hSDSCH_MACdFlows_Information);
}



static int
dissect_nbap_INTEGER_1_64_(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 64U, NULL, TRUE);

  return offset;
}
static int dissect_hSDSCH_Physical_Layer_Category(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_INTEGER_1_64_(tvb, offset, actx, tree, hf_nbap_hSDSCH_Physical_Layer_Category);
}


static const per_sequence_t UE_Capability_Information_sequence[] = {
  { &hf_nbap_hSDSCH_Physical_Layer_Category, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_INTEGER_1_64_ },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_UE_Capability_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_UE_Capability_Information, UE_Capability_Information_sequence);

  return offset;
}
static int dissect_ueCapability_Info(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UE_Capability_Information(tvb, offset, actx, tree, hf_nbap_ueCapability_Info);
}



static int
dissect_nbap_MAChsReorderingBufferSize_for_RLC_UM(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 300U, NULL, TRUE);

  return offset;
}
static int dissect_mAChs_Reordering_Buffer_Size_for_RLC_UM(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MAChsReorderingBufferSize_for_RLC_UM(tvb, offset, actx, tree, hf_nbap_mAChs_Reordering_Buffer_Size_for_RLC_UM);
}


static const value_string nbap_CQI_Feedback_Cycle_vals[] = {
  {   0, "v0" },
  {   1, "v2" },
  {   2, "v4" },
  {   3, "v8" },
  {   4, "v10" },
  {   5, "v20" },
  {   6, "v40" },
  {   7, "v80" },
  {   8, "v160" },
  { 0, NULL }
};


static int
dissect_nbap_CQI_Feedback_Cycle(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     9, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_cqiFeedback_CycleK(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CQI_Feedback_Cycle(tvb, offset, actx, tree, hf_nbap_cqiFeedback_CycleK);
}



static int
dissect_nbap_CQI_RepetitionFactor(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 4U, NULL, TRUE);

  return offset;
}
static int dissect_cqiRepetitionFactor(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CQI_RepetitionFactor(tvb, offset, actx, tree, hf_nbap_cqiRepetitionFactor);
}



static int
dissect_nbap_AckNack_RepetitionFactor(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 4U, NULL, TRUE);

  return offset;
}
static int dissect_ackNackRepetitionFactor(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_AckNack_RepetitionFactor(tvb, offset, actx, tree, hf_nbap_ackNackRepetitionFactor);
}



static int
dissect_nbap_CQI_Power_Offset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 8U, NULL, TRUE);

  return offset;
}
static int dissect_cqiPowerOffset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CQI_Power_Offset(tvb, offset, actx, tree, hf_nbap_cqiPowerOffset);
}



static int
dissect_nbap_Ack_Power_Offset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 8U, NULL, TRUE);

  return offset;
}
static int dissect_ackPowerOffset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Ack_Power_Offset(tvb, offset, actx, tree, hf_nbap_ackPowerOffset);
}



static int
dissect_nbap_Nack_Power_Offset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 8U, NULL, TRUE);

  return offset;
}
static int dissect_nackPowerOffset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Nack_Power_Offset(tvb, offset, actx, tree, hf_nbap_nackPowerOffset);
}



static int
dissect_nbap_HSSCCH_PowerOffset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 255U, NULL, FALSE);

  return offset;
}
static int dissect_hsscch_PowerOffset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HSSCCH_PowerOffset(tvb, offset, actx, tree, hf_nbap_hsscch_PowerOffset);
}



static int
dissect_nbap_Measurement_Power_Offset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              -12, 26U, NULL, FALSE);

  return offset;
}
static int dissect_measurement_Power_Offset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Measurement_Power_Offset(tvb, offset, actx, tree, hf_nbap_measurement_Power_Offset);
}


static const per_sequence_t HSDSCH_FDD_Information_sequence[] = {
  { &hf_nbap_hSDSCH_MACdFlows_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_HSDSCH_MACdFlows_Information },
  { &hf_nbap_ueCapability_Info, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UE_Capability_Information },
  { &hf_nbap_mAChs_Reordering_Buffer_Size_for_RLC_UM, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MAChsReorderingBufferSize_for_RLC_UM },
  { &hf_nbap_cqiFeedback_CycleK, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CQI_Feedback_Cycle },
  { &hf_nbap_cqiRepetitionFactor, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_CQI_RepetitionFactor },
  { &hf_nbap_ackNackRepetitionFactor, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_AckNack_RepetitionFactor },
  { &hf_nbap_cqiPowerOffset , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CQI_Power_Offset },
  { &hf_nbap_ackPowerOffset , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Ack_Power_Offset },
  { &hf_nbap_nackPowerOffset, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Nack_Power_Offset },
  { &hf_nbap_hsscch_PowerOffset, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_HSSCCH_PowerOffset },
  { &hf_nbap_measurement_Power_Offset, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_Measurement_Power_Offset },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_HSDSCH_FDD_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_HSDSCH_FDD_Information, HSDSCH_FDD_Information_sequence);

  return offset;
}
static int dissect_id_HSDSCH_FDD_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HSDSCH_FDD_Information(tvb, offset, actx, tree, hf_nbap_id_HSDSCH_FDD_Information);
}



static int
dissect_nbap_HSDSCH_InitialWindowSize(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 255U, NULL, FALSE);

  return offset;
}
static int dissect_hSDSCH_InitialWindowSize(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HSDSCH_InitialWindowSize(tvb, offset, actx, tree, hf_nbap_hSDSCH_InitialWindowSize);
}


static const per_sequence_t HSDSCH_Initial_Capacity_AllocationItem_sequence[] = {
  { &hf_nbap_schedulingPriorityIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SchedulingPriorityIndicator },
  { &hf_nbap_maximum_MACdPDU_Size, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MACdPDU_Size },
  { &hf_nbap_hSDSCH_InitialWindowSize, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_HSDSCH_InitialWindowSize },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_HSDSCH_Initial_Capacity_AllocationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_HSDSCH_Initial_Capacity_AllocationItem, HSDSCH_Initial_Capacity_AllocationItem_sequence);

  return offset;
}
static int dissect_HSDSCH_Initial_Capacity_Allocation_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HSDSCH_Initial_Capacity_AllocationItem(tvb, offset, actx, tree, hf_nbap_HSDSCH_Initial_Capacity_Allocation_item);
}


static const per_sequence_t HSDSCH_Initial_Capacity_Allocation_sequence_of[1] = {
  { &hf_nbap_HSDSCH_Initial_Capacity_Allocation_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_HSDSCH_Initial_Capacity_AllocationItem },
};

static int
dissect_nbap_HSDSCH_Initial_Capacity_Allocation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_HSDSCH_Initial_Capacity_Allocation, HSDSCH_Initial_Capacity_Allocation_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_hSDSCH_Initial_Capacity_Allocation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HSDSCH_Initial_Capacity_Allocation(tvb, offset, actx, tree, hf_nbap_hSDSCH_Initial_Capacity_Allocation);
}


static const per_sequence_t HSDSCH_MACdFlow_Specific_InformationResp_Item_sequence[] = {
  { &hf_nbap_hsDSCHMacdFlow_Id, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_HSDSCH_MACdFlow_ID },
  { &hf_nbap_bindingID      , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_BindingID },
  { &hf_nbap_transportLayerAddress, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TransportLayerAddress },
  { &hf_nbap_hSDSCH_Initial_Capacity_Allocation, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_HSDSCH_Initial_Capacity_Allocation },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_HSDSCH_MACdFlow_Specific_InformationResp_Item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_HSDSCH_MACdFlow_Specific_InformationResp_Item, HSDSCH_MACdFlow_Specific_InformationResp_Item_sequence);

  return offset;
}
static int dissect_HSDSCH_MACdFlow_Specific_InformationResp_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HSDSCH_MACdFlow_Specific_InformationResp_Item(tvb, offset, actx, tree, hf_nbap_HSDSCH_MACdFlow_Specific_InformationResp_item);
}


static const per_sequence_t HSDSCH_MACdFlow_Specific_InformationResp_sequence_of[1] = {
  { &hf_nbap_HSDSCH_MACdFlow_Specific_InformationResp_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_HSDSCH_MACdFlow_Specific_InformationResp_Item },
};

static int
dissect_nbap_HSDSCH_MACdFlow_Specific_InformationResp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_HSDSCH_MACdFlow_Specific_InformationResp, HSDSCH_MACdFlow_Specific_InformationResp_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_hsDSCH_MACdFlow_Specific_InformationResp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HSDSCH_MACdFlow_Specific_InformationResp(tvb, offset, actx, tree, hf_nbap_hsDSCH_MACdFlow_Specific_InformationResp);
}


static const per_sequence_t HSSCCH_Codes_sequence[] = {
  { &hf_nbap_codeNumber     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_INTEGER_0_127 },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_HSSCCH_Codes(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_HSSCCH_Codes, HSSCCH_Codes_sequence);

  return offset;
}
static int dissect_HSSCCH_Specific_InformationRespListFDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HSSCCH_Codes(tvb, offset, actx, tree, hf_nbap_HSSCCH_Specific_InformationRespListFDD_item);
}


static const per_sequence_t HSSCCH_Specific_InformationRespListFDD_sequence_of[1] = {
  { &hf_nbap_HSSCCH_Specific_InformationRespListFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_HSSCCH_Codes },
};

static int
dissect_nbap_HSSCCH_Specific_InformationRespListFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_HSSCCH_Specific_InformationRespListFDD, HSSCCH_Specific_InformationRespListFDD_sequence_of,
                                                  1, 4);

  return offset;
}
static int dissect_hsSCCH_Specific_Information_ResponseFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HSSCCH_Specific_InformationRespListFDD(tvb, offset, actx, tree, hf_nbap_hsSCCH_Specific_Information_ResponseFDD);
}



static int
dissect_nbap_INTEGER_1_8_(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 8U, NULL, TRUE);

  return offset;
}
static int dissect_number_of_Processes(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_INTEGER_1_8_(tvb, offset, actx, tree, hf_nbap_number_of_Processes);
}


static const per_sequence_t HARQ_MemoryPartitioning_Implicit_sequence[] = {
  { &hf_nbap_number_of_Processes, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_INTEGER_1_8_ },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_HARQ_MemoryPartitioning_Implicit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_HARQ_MemoryPartitioning_Implicit, HARQ_MemoryPartitioning_Implicit_sequence);

  return offset;
}
static int dissect_implicit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HARQ_MemoryPartitioning_Implicit(tvb, offset, actx, tree, hf_nbap_implicit);
}


static const value_string nbap_T_process_Memory_Size_vals[] = {
  {   0, "hms800" },
  {   1, "hms1600" },
  {   2, "hms2400" },
  {   3, "hms3200" },
  {   4, "hms4000" },
  {   5, "hms4800" },
  {   6, "hms5600" },
  {   7, "hms6400" },
  {   8, "hms7200" },
  {   9, "hms8000" },
  {  10, "hms8800" },
  {  11, "hms9600" },
  {  12, "hms10400" },
  {  13, "hms11200" },
  {  14, "hms12000" },
  {  15, "hms12800" },
  {  16, "hms13600" },
  {  17, "hms14400" },
  {  18, "hms15200" },
  {  19, "hms16000" },
  {  20, "hms17600" },
  {  21, "hms19200" },
  {  22, "hms20800" },
  {  23, "hms22400" },
  {  24, "hms24000" },
  {  25, "hms25600" },
  {  26, "hms27200" },
  {  27, "hms28800" },
  {  28, "hms30400" },
  {  29, "hms32000" },
  {  30, "hms36000" },
  {  31, "hms40000" },
  {  32, "hms44000" },
  {  33, "hms48000" },
  {  34, "hms52000" },
  {  35, "hms56000" },
  {  36, "hms60000" },
  {  37, "hms64000" },
  {  38, "hms68000" },
  {  39, "hms72000" },
  {  40, "hms76000" },
  {  41, "hms80000" },
  {  42, "hms88000" },
  {  43, "hms96000" },
  {  44, "hms104000" },
  {  45, "hms112000" },
  {  46, "hms120000" },
  {  47, "hms128000" },
  {  48, "hms136000" },
  {  49, "hms144000" },
  {  50, "hms152000" },
  {  51, "hms160000" },
  {  52, "hms176000" },
  {  53, "hms192000" },
  {  54, "hms208000" },
  {  55, "hms224000" },
  {  56, "hms240000" },
  {  57, "hms256000" },
  {  58, "hms272000" },
  {  59, "hms288000" },
  {  60, "hms304000" },
  { 0, NULL }
};


static int
dissect_nbap_T_process_Memory_Size(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     61, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_process_Memory_Size(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_T_process_Memory_Size(tvb, offset, actx, tree, hf_nbap_process_Memory_Size);
}


static const per_sequence_t HARQ_MemoryPartitioningItem_sequence[] = {
  { &hf_nbap_process_Memory_Size, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_T_process_Memory_Size },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_HARQ_MemoryPartitioningItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_HARQ_MemoryPartitioningItem, HARQ_MemoryPartitioningItem_sequence);

  return offset;
}
static int dissect_HARQ_MemoryPartitioningList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HARQ_MemoryPartitioningItem(tvb, offset, actx, tree, hf_nbap_HARQ_MemoryPartitioningList_item);
}


static const per_sequence_t HARQ_MemoryPartitioningList_sequence_of[1] = {
  { &hf_nbap_HARQ_MemoryPartitioningList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_HARQ_MemoryPartitioningItem },
};

static int
dissect_nbap_HARQ_MemoryPartitioningList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_HARQ_MemoryPartitioningList, HARQ_MemoryPartitioningList_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_hARQ_MemoryPartitioningList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HARQ_MemoryPartitioningList(tvb, offset, actx, tree, hf_nbap_hARQ_MemoryPartitioningList);
}


static const per_sequence_t HARQ_MemoryPartitioning_Explicit_sequence[] = {
  { &hf_nbap_hARQ_MemoryPartitioningList, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_HARQ_MemoryPartitioningList },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_HARQ_MemoryPartitioning_Explicit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_HARQ_MemoryPartitioning_Explicit, HARQ_MemoryPartitioning_Explicit_sequence);

  return offset;
}
static int dissect_explicit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HARQ_MemoryPartitioning_Explicit(tvb, offset, actx, tree, hf_nbap_explicit);
}


static const value_string nbap_HARQ_MemoryPartitioning_vals[] = {
  {   0, "implicit" },
  {   1, "explicit" },
  { 0, NULL }
};

static const per_choice_t HARQ_MemoryPartitioning_choice[] = {
  {   0, &hf_nbap_implicit       , ASN1_EXTENSION_ROOT    , dissect_nbap_HARQ_MemoryPartitioning_Implicit },
  {   1, &hf_nbap_explicit       , ASN1_EXTENSION_ROOT    , dissect_nbap_HARQ_MemoryPartitioning_Explicit },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_HARQ_MemoryPartitioning(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_HARQ_MemoryPartitioning, HARQ_MemoryPartitioning_choice,
                                 NULL);

  return offset;
}
static int dissect_hARQ_MemoryPartitioning(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HARQ_MemoryPartitioning(tvb, offset, actx, tree, hf_nbap_hARQ_MemoryPartitioning);
}


static const per_sequence_t HSDSCH_FDD_Information_Response_sequence[] = {
  { &hf_nbap_hsDSCH_MACdFlow_Specific_InformationResp, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_HSDSCH_MACdFlow_Specific_InformationResp },
  { &hf_nbap_hsSCCH_Specific_Information_ResponseFDD, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_HSSCCH_Specific_InformationRespListFDD },
  { &hf_nbap_hARQ_MemoryPartitioning, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_HARQ_MemoryPartitioning },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_HSDSCH_FDD_Information_Response(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_HSDSCH_FDD_Information_Response, HSDSCH_FDD_Information_Response_sequence);

  return offset;
}
static int dissect_id_HSDSCH_FDD_Information_Response(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HSDSCH_FDD_Information_Response(tvb, offset, actx, tree, hf_nbap_id_HSDSCH_FDD_Information_Response);
}


static const per_sequence_t HSDSCH_MACdFlow_Specific_InfoItem_to_Modify_sequence[] = {
  { &hf_nbap_hsDSCH_MACdFlow_ID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_HSDSCH_MACdFlow_ID },
  { &hf_nbap_allocationRetentionPriority, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_AllocationRetentionPriority },
  { &hf_nbap_transportBearerRequestIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TransportBearerRequestIndicator },
  { &hf_nbap_bindingID      , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_BindingID },
  { &hf_nbap_transportLayerAddress, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TransportLayerAddress },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_HSDSCH_MACdFlow_Specific_InfoItem_to_Modify(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_HSDSCH_MACdFlow_Specific_InfoItem_to_Modify, HSDSCH_MACdFlow_Specific_InfoItem_to_Modify_sequence);

  return offset;
}
static int dissect_HSDSCH_MACdFlow_Specific_InfoList_to_Modify_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HSDSCH_MACdFlow_Specific_InfoItem_to_Modify(tvb, offset, actx, tree, hf_nbap_HSDSCH_MACdFlow_Specific_InfoList_to_Modify_item);
}


static const per_sequence_t HSDSCH_MACdFlow_Specific_InfoList_to_Modify_sequence_of[1] = {
  { &hf_nbap_HSDSCH_MACdFlow_Specific_InfoList_to_Modify_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_HSDSCH_MACdFlow_Specific_InfoItem_to_Modify },
};

static int
dissect_nbap_HSDSCH_MACdFlow_Specific_InfoList_to_Modify(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_HSDSCH_MACdFlow_Specific_InfoList_to_Modify, HSDSCH_MACdFlow_Specific_InfoList_to_Modify_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_hsDSCH_MACdFlow_Specific_Info_to_Modify(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HSDSCH_MACdFlow_Specific_InfoList_to_Modify(tvb, offset, actx, tree, hf_nbap_hsDSCH_MACdFlow_Specific_Info_to_Modify);
}


static const per_sequence_t PriorityQueue_InfoItem_to_Add_sequence[] = {
  { &hf_nbap_priorityQueueId, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PriorityQueue_Id },
  { &hf_nbap_associatedHSDSCH_MACdFlow, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_HSDSCH_MACdFlow_ID },
  { &hf_nbap_schedulingPriorityIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SchedulingPriorityIndicator },
  { &hf_nbap_t1             , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_T1 },
  { &hf_nbap_discardTimer   , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DiscardTimer },
  { &hf_nbap_mAC_hsWindowSize, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MAC_hsWindowSize },
  { &hf_nbap_mAChsGuaranteedBitRate, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_MAChsGuaranteedBitRate },
  { &hf_nbap_macdPDU_Size_Index, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MACdPDU_Size_Indexlist },
  { &hf_nbap_rLC_Mode       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RLC_Mode },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PriorityQueue_InfoItem_to_Add(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PriorityQueue_InfoItem_to_Add, PriorityQueue_InfoItem_to_Add_sequence);

  return offset;
}
static int dissect_addPriorityQueue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PriorityQueue_InfoItem_to_Add(tvb, offset, actx, tree, hf_nbap_addPriorityQueue);
}


static const per_sequence_t MACdPDU_Size_IndexItem_to_Modify_sequence[] = {
  { &hf_nbap_sID            , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SID },
  { &hf_nbap_macdPDU_Size   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MACdPDU_Size },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_MACdPDU_Size_IndexItem_to_Modify(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_MACdPDU_Size_IndexItem_to_Modify, MACdPDU_Size_IndexItem_to_Modify_sequence);

  return offset;
}
static int dissect_MACdPDU_Size_Indexlist_to_Modify_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MACdPDU_Size_IndexItem_to_Modify(tvb, offset, actx, tree, hf_nbap_MACdPDU_Size_Indexlist_to_Modify_item);
}


static const per_sequence_t MACdPDU_Size_Indexlist_to_Modify_sequence_of[1] = {
  { &hf_nbap_MACdPDU_Size_Indexlist_to_Modify_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_MACdPDU_Size_IndexItem_to_Modify },
};

static int
dissect_nbap_MACdPDU_Size_Indexlist_to_Modify(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_MACdPDU_Size_Indexlist_to_Modify, MACdPDU_Size_Indexlist_to_Modify_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_macdPDU_Size_Index_to_Modify(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MACdPDU_Size_Indexlist_to_Modify(tvb, offset, actx, tree, hf_nbap_macdPDU_Size_Index_to_Modify);
}


static const per_sequence_t PriorityQueue_InfoItem_to_Modify_sequence[] = {
  { &hf_nbap_priorityQueueId, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PriorityQueue_Id },
  { &hf_nbap_schedulingPriorityIndicator, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_SchedulingPriorityIndicator },
  { &hf_nbap_t1             , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_T1 },
  { &hf_nbap_discardTimer   , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DiscardTimer },
  { &hf_nbap_mAC_hsWindowSize, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_MAC_hsWindowSize },
  { &hf_nbap_mAChsGuaranteedBitRate, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_MAChsGuaranteedBitRate },
  { &hf_nbap_macdPDU_Size_Index_to_Modify, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_MACdPDU_Size_Indexlist_to_Modify },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PriorityQueue_InfoItem_to_Modify(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PriorityQueue_InfoItem_to_Modify, PriorityQueue_InfoItem_to_Modify_sequence);

  return offset;
}
static int dissect_modifyPriorityQueue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PriorityQueue_InfoItem_to_Modify(tvb, offset, actx, tree, hf_nbap_modifyPriorityQueue);
}


static const value_string nbap_ModifyPriorityQueue_vals[] = {
  {   0, "addPriorityQueue" },
  {   1, "modifyPriorityQueue" },
  {   2, "deletePriorityQueue" },
  { 0, NULL }
};

static const per_choice_t ModifyPriorityQueue_choice[] = {
  {   0, &hf_nbap_addPriorityQueue, ASN1_EXTENSION_ROOT    , dissect_nbap_PriorityQueue_InfoItem_to_Add },
  {   1, &hf_nbap_modifyPriorityQueue, ASN1_EXTENSION_ROOT    , dissect_nbap_PriorityQueue_InfoItem_to_Modify },
  {   2, &hf_nbap_deletePriorityQueue, ASN1_EXTENSION_ROOT    , dissect_nbap_PriorityQueue_Id },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_ModifyPriorityQueue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_ModifyPriorityQueue, ModifyPriorityQueue_choice,
                                 NULL);

  return offset;
}
static int dissect_PriorityQueue_InfoList_to_Modify_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ModifyPriorityQueue(tvb, offset, actx, tree, hf_nbap_PriorityQueue_InfoList_to_Modify_item);
}


static const per_sequence_t PriorityQueue_InfoList_to_Modify_sequence_of[1] = {
  { &hf_nbap_PriorityQueue_InfoList_to_Modify_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ModifyPriorityQueue },
};

static int
dissect_nbap_PriorityQueue_InfoList_to_Modify(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_PriorityQueue_InfoList_to_Modify, PriorityQueue_InfoList_to_Modify_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_priorityQueueInfotoModify(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PriorityQueue_InfoList_to_Modify(tvb, offset, actx, tree, hf_nbap_priorityQueueInfotoModify);
}


static const value_string nbap_HSSCCH_Code_Change_Grant_vals[] = {
  {   0, "changeGranted" },
  { 0, NULL }
};


static int
dissect_nbap_HSSCCH_Code_Change_Grant(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_hSSCCHCodeChangeGrant(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HSSCCH_Code_Change_Grant(tvb, offset, actx, tree, hf_nbap_hSSCCHCodeChangeGrant);
}



static int
dissect_nbap_TDD_AckNack_Power_Offset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              -7, 8U, NULL, TRUE);

  return offset;
}
static int dissect_tDD_AckNack_Power_Offset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TDD_AckNack_Power_Offset(tvb, offset, actx, tree, hf_nbap_tDD_AckNack_Power_Offset);
}
static int dissect_tDDAckNackPowerOffset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TDD_AckNack_Power_Offset(tvb, offset, actx, tree, hf_nbap_tDDAckNackPowerOffset);
}


static const per_sequence_t HSDSCH_Information_to_Modify_sequence[] = {
  { &hf_nbap_hsDSCH_MACdFlow_Specific_Info_to_Modify, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_HSDSCH_MACdFlow_Specific_InfoList_to_Modify },
  { &hf_nbap_priorityQueueInfotoModify, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_PriorityQueue_InfoList_to_Modify },
  { &hf_nbap_mAChs_Reordering_Buffer_Size_for_RLC_UM, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_MAChsReorderingBufferSize_for_RLC_UM },
  { &hf_nbap_cqiFeedback_CycleK, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_CQI_Feedback_Cycle },
  { &hf_nbap_cqiRepetitionFactor, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_CQI_RepetitionFactor },
  { &hf_nbap_ackNackRepetitionFactor, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_AckNack_RepetitionFactor },
  { &hf_nbap_cqiPowerOffset , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_CQI_Power_Offset },
  { &hf_nbap_ackPowerOffset , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_Ack_Power_Offset },
  { &hf_nbap_nackPowerOffset, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_Nack_Power_Offset },
  { &hf_nbap_hsscch_PowerOffset, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_HSSCCH_PowerOffset },
  { &hf_nbap_measurement_Power_Offset, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_Measurement_Power_Offset },
  { &hf_nbap_hSSCCHCodeChangeGrant, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_HSSCCH_Code_Change_Grant },
  { &hf_nbap_tDDAckNackPowerOffset, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TDD_AckNack_Power_Offset },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_HSDSCH_Information_to_Modify(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_HSDSCH_Information_to_Modify, HSDSCH_Information_to_Modify_sequence);

  return offset;
}
static int dissect_id_HSDSCH_Information_to_Modify(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HSDSCH_Information_to_Modify(tvb, offset, actx, tree, hf_nbap_id_HSDSCH_Information_to_Modify);
}



static int
dissect_nbap_HSDSCH_RNTI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 65535U, NULL, FALSE);

  return offset;
}
static int dissect_id_HSDSCH_RNTI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HSDSCH_RNTI(tvb, offset, actx, tree, hf_nbap_id_HSDSCH_RNTI);
}


static const per_sequence_t HSDSCH_TDD_Information_sequence[] = {
  { &hf_nbap_hSDSCH_MACdFlows_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_HSDSCH_MACdFlows_Information },
  { &hf_nbap_ueCapability_Info, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UE_Capability_Information },
  { &hf_nbap_mAChs_Reordering_Buffer_Size_for_RLC_UM, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MAChsReorderingBufferSize_for_RLC_UM },
  { &hf_nbap_tDD_AckNack_Power_Offset, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_AckNack_Power_Offset },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_HSDSCH_TDD_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_HSDSCH_TDD_Information, HSDSCH_TDD_Information_sequence);

  return offset;
}
static int dissect_id_HSDSCH_TDD_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HSDSCH_TDD_Information(tvb, offset, actx, tree, hf_nbap_id_HSDSCH_TDD_Information);
}


static const per_sequence_t HSSICH_Info_sequence[] = {
  { &hf_nbap_hsSICH_ID      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_HS_SICH_ID },
  { &hf_nbap_timeslot       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlot },
  { &hf_nbap_midambleShiftAndBurstType, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MidambleShiftAndBurstType },
  { &hf_nbap_tDD_ChannelisationCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_ChannelisationCode },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_HSSICH_Info(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_HSSICH_Info, HSSICH_Info_sequence);

  return offset;
}
static int dissect_hSSICH_Info(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HSSICH_Info(tvb, offset, actx, tree, hf_nbap_hSSICH_Info);
}


static const per_sequence_t HSSCCH_Specific_InformationRespItemTDD_sequence[] = {
  { &hf_nbap_timeslot       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlot },
  { &hf_nbap_midambleShiftAndBurstType, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MidambleShiftAndBurstType },
  { &hf_nbap_tDD_ChannelisationCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_ChannelisationCode },
  { &hf_nbap_hSSICH_Info    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_HSSICH_Info },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_HSSCCH_Specific_InformationRespItemTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_HSSCCH_Specific_InformationRespItemTDD, HSSCCH_Specific_InformationRespItemTDD_sequence);

  return offset;
}
static int dissect_HSSCCH_Specific_InformationRespListTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HSSCCH_Specific_InformationRespItemTDD(tvb, offset, actx, tree, hf_nbap_HSSCCH_Specific_InformationRespListTDD_item);
}


static const per_sequence_t HSSCCH_Specific_InformationRespListTDD_sequence_of[1] = {
  { &hf_nbap_HSSCCH_Specific_InformationRespListTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_HSSCCH_Specific_InformationRespItemTDD },
};

static int
dissect_nbap_HSSCCH_Specific_InformationRespListTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_HSSCCH_Specific_InformationRespListTDD, HSSCCH_Specific_InformationRespListTDD_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_hsSCCH_Specific_Information_ResponseTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HSSCCH_Specific_InformationRespListTDD(tvb, offset, actx, tree, hf_nbap_hsSCCH_Specific_Information_ResponseTDD);
}


static const per_sequence_t HSSICH_InfoLCR_sequence[] = {
  { &hf_nbap_hsSICH_ID      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_HS_SICH_ID },
  { &hf_nbap_timeslotLCR    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlotLCR },
  { &hf_nbap_midambleShiftLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MidambleShiftLCR },
  { &hf_nbap_tDD_ChannelisationCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_ChannelisationCode },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_HSSICH_InfoLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_HSSICH_InfoLCR, HSSICH_InfoLCR_sequence);

  return offset;
}
static int dissect_hSSICH_InfoLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HSSICH_InfoLCR(tvb, offset, actx, tree, hf_nbap_hSSICH_InfoLCR);
}


static const per_sequence_t HSSCCH_Specific_InformationRespItemTDDLCR_sequence[] = {
  { &hf_nbap_timeslotLCR    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlotLCR },
  { &hf_nbap_midambleShiftLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MidambleShiftLCR },
  { &hf_nbap_first_TDD_ChannelisationCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_ChannelisationCode },
  { &hf_nbap_second_TDD_ChannelisationCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_ChannelisationCode },
  { &hf_nbap_hSSICH_InfoLCR , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_HSSICH_InfoLCR },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_HSSCCH_Specific_InformationRespItemTDDLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_HSSCCH_Specific_InformationRespItemTDDLCR, HSSCCH_Specific_InformationRespItemTDDLCR_sequence);

  return offset;
}
static int dissect_HSSCCH_Specific_InformationRespListTDDLCR_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HSSCCH_Specific_InformationRespItemTDDLCR(tvb, offset, actx, tree, hf_nbap_HSSCCH_Specific_InformationRespListTDDLCR_item);
}


static const per_sequence_t HSSCCH_Specific_InformationRespListTDDLCR_sequence_of[1] = {
  { &hf_nbap_HSSCCH_Specific_InformationRespListTDDLCR_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_HSSCCH_Specific_InformationRespItemTDDLCR },
};

static int
dissect_nbap_HSSCCH_Specific_InformationRespListTDDLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_HSSCCH_Specific_InformationRespListTDDLCR, HSSCCH_Specific_InformationRespListTDDLCR_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_hsSCCH_Specific_Information_ResponseTDDLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HSSCCH_Specific_InformationRespListTDDLCR(tvb, offset, actx, tree, hf_nbap_hsSCCH_Specific_Information_ResponseTDDLCR);
}


static const per_sequence_t HSDSCH_TDD_Information_Response_sequence[] = {
  { &hf_nbap_hsDSCH_MACdFlow_Specific_InformationResp, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_HSDSCH_MACdFlow_Specific_InformationResp },
  { &hf_nbap_hsSCCH_Specific_Information_ResponseTDD, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_HSSCCH_Specific_InformationRespListTDD },
  { &hf_nbap_hsSCCH_Specific_Information_ResponseTDDLCR, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_HSSCCH_Specific_InformationRespListTDDLCR },
  { &hf_nbap_hARQ_MemoryPartitioning, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_HARQ_MemoryPartitioning },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_HSDSCH_TDD_Information_Response(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_HSDSCH_TDD_Information_Response, HSDSCH_TDD_Information_Response_sequence);

  return offset;
}
static int dissect_id_HSDSCH_TDD_Information_Response(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HSDSCH_TDD_Information_Response(tvb, offset, actx, tree, hf_nbap_id_HSDSCH_TDD_Information_Response);
}



static int
dissect_nbap_PrimaryCCPCH_RSCP(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 91U, NULL, FALSE);

  return offset;
}
static int dissect_id_PrimCCPCH_RSCP_DL_PC_RqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PrimaryCCPCH_RSCP(tvb, offset, actx, tree, hf_nbap_id_PrimCCPCH_RSCP_DL_PC_RqstTDD);
}


static const per_sequence_t HSDSCH_RearrangeItem_Bearer_RearrangeInd_sequence[] = {
  { &hf_nbap_hsDSCH_MACdFlow_ID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_HSDSCH_MACdFlow_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_HSDSCH_RearrangeItem_Bearer_RearrangeInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_HSDSCH_RearrangeItem_Bearer_RearrangeInd, HSDSCH_RearrangeItem_Bearer_RearrangeInd_sequence);

  return offset;
}
static int dissect_HSDSCH_RearrangeList_Bearer_RearrangeInd_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HSDSCH_RearrangeItem_Bearer_RearrangeInd(tvb, offset, actx, tree, hf_nbap_HSDSCH_RearrangeList_Bearer_RearrangeInd_item);
}


static const per_sequence_t HSDSCH_RearrangeList_Bearer_RearrangeInd_sequence_of[1] = {
  { &hf_nbap_HSDSCH_RearrangeList_Bearer_RearrangeInd_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_HSDSCH_RearrangeItem_Bearer_RearrangeInd },
};

static int
dissect_nbap_HSDSCH_RearrangeList_Bearer_RearrangeInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_HSDSCH_RearrangeList_Bearer_RearrangeInd, HSDSCH_RearrangeList_Bearer_RearrangeInd_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_id_HSDSCH_RearrangeList_Bearer_RearrangeInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HSDSCH_RearrangeList_Bearer_RearrangeInd(tvb, offset, actx, tree, hf_nbap_id_HSDSCH_RearrangeList_Bearer_RearrangeInd);
}



static int
dissect_nbap_UL_Synchronisation_StepSize(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 8U, NULL, FALSE);

  return offset;
}
static int dissect_uL_Synchronisation_StepSize(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_Synchronisation_StepSize(tvb, offset, actx, tree, hf_nbap_uL_Synchronisation_StepSize);
}



static int
dissect_nbap_UL_Synchronisation_Frequency(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 8U, NULL, FALSE);

  return offset;
}
static int dissect_uL_Synchronisation_Frequency(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_Synchronisation_Frequency(tvb, offset, actx, tree, hf_nbap_uL_Synchronisation_Frequency);
}


static const per_sequence_t UL_Synchronisation_Parameters_LCR_sequence[] = {
  { &hf_nbap_uL_Synchronisation_StepSize, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UL_Synchronisation_StepSize },
  { &hf_nbap_uL_Synchronisation_Frequency, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_UL_Synchronisation_Frequency },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_UL_Synchronisation_Parameters_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_UL_Synchronisation_Parameters_LCR, UL_Synchronisation_Parameters_LCR_sequence);

  return offset;
}
static int dissect_id_UL_Synchronisation_Parameters_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_Synchronisation_Parameters_LCR(tvb, offset, actx, tree, hf_nbap_id_UL_Synchronisation_Parameters_LCR);
}


static const value_string nbap_HSSCCH_CodeChangeIndicator_vals[] = {
  {   0, "hsSCCHCodeChangeNeeded" },
  { 0, NULL }
};


static int
dissect_nbap_HSSCCH_CodeChangeIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_hsSCCHCodeChangeIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HSSCCH_CodeChangeIndicator(tvb, offset, actx, tree, hf_nbap_hsSCCHCodeChangeIndicator);
}


static const per_sequence_t HSDSCH_FDD_Update_Information_sequence[] = {
  { &hf_nbap_hsSCCHCodeChangeIndicator, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_HSSCCH_CodeChangeIndicator },
  { &hf_nbap_cqiFeedback_CycleK, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_CQI_Feedback_Cycle },
  { &hf_nbap_cqiRepetitionFactor, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_CQI_RepetitionFactor },
  { &hf_nbap_ackNackRepetitionFactor, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_AckNack_RepetitionFactor },
  { &hf_nbap_cqiPowerOffset , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_CQI_Power_Offset },
  { &hf_nbap_ackPowerOffset , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_Ack_Power_Offset },
  { &hf_nbap_nackPowerOffset, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_Nack_Power_Offset },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_HSDSCH_FDD_Update_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_HSDSCH_FDD_Update_Information, HSDSCH_FDD_Update_Information_sequence);

  return offset;
}
static int dissect_id_HSDSCH_FDD_Update_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HSDSCH_FDD_Update_Information(tvb, offset, actx, tree, hf_nbap_id_HSDSCH_FDD_Update_Information);
}


static const per_sequence_t HSDSCH_TDD_Update_Information_sequence[] = {
  { &hf_nbap_hsSCCHCodeChangeIndicator, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_HSSCCH_CodeChangeIndicator },
  { &hf_nbap_tDDAckNackPowerOffset, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TDD_AckNack_Power_Offset },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_HSDSCH_TDD_Update_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_HSDSCH_TDD_Update_Information, HSDSCH_TDD_Update_Information_sequence);

  return offset;
}
static int dissect_id_HSDSCH_TDD_Update_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HSDSCH_TDD_Update_Information(tvb, offset, actx, tree, hf_nbap_id_HSDSCH_TDD_Update_Information);
}


static const value_string nbap_TDD_TPC_UplinkStepSize_LCR_vals[] = {
  {   0, "step-size1" },
  {   1, "step-size2" },
  {   2, "step-size3" },
  { 0, NULL }
};


static int
dissect_nbap_TDD_TPC_UplinkStepSize_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_id_TDD_TPC_UplinkStepSize_LCR_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TDD_TPC_UplinkStepSize_LCR(tvb, offset, actx, tree, hf_nbap_id_TDD_TPC_UplinkStepSize_LCR_RL_SetupRqstTDD);
}
static int dissect_id_TDD_TPC_UplinkStepSize_LCR_RL_AdditionRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TDD_TPC_UplinkStepSize_LCR(tvb, offset, actx, tree, hf_nbap_id_TDD_TPC_UplinkStepSize_LCR_RL_AdditionRqstTDD);
}
static int dissect_id_TDD_TPC_UplinkStepSize_InformationAdd_LCR_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TDD_TPC_UplinkStepSize_LCR(tvb, offset, actx, tree, hf_nbap_id_TDD_TPC_UplinkStepSize_InformationAdd_LCR_RL_ReconfPrepTDD);
}
static int dissect_id_TDD_TPC_UplinkStepSize_InformationModify_LCR_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TDD_TPC_UplinkStepSize_LCR(tvb, offset, actx, tree, hf_nbap_id_TDD_TPC_UplinkStepSize_InformationModify_LCR_RL_ReconfPrepTDD);
}
static int dissect_tDD_TPC_UplinkStepSize_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TDD_TPC_UplinkStepSize_LCR(tvb, offset, actx, tree, hf_nbap_tDD_TPC_UplinkStepSize_LCR);
}


static const per_sequence_t DL_Timeslot_LCR_InformationModify_ModifyItem_RL_ReconfRqstTDD_sequence[] = {
  { &hf_nbap_timeSlotLCR    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlotLCR },
  { &hf_nbap_maxPowerLCR    , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_Power },
  { &hf_nbap_minPowerLCR    , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_Power },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_Timeslot_LCR_InformationModify_ModifyItem_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_Timeslot_LCR_InformationModify_ModifyItem_RL_ReconfRqstTDD, DL_Timeslot_LCR_InformationModify_ModifyItem_RL_ReconfRqstTDD_sequence);

  return offset;
}
static int dissect_DL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Timeslot_LCR_InformationModify_ModifyItem_RL_ReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_DL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfRqstTDD_item);
}


static const per_sequence_t DL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfRqstTDD_sequence_of[1] = {
  { &hf_nbap_DL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DL_Timeslot_LCR_InformationModify_ModifyItem_RL_ReconfRqstTDD },
};

static int
dissect_nbap_DL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfRqstTDD, DL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfRqstTDD_sequence_of,
                                                  1, 6);

  return offset;
}
static int dissect_dL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_dL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfRqstTDD);
}


static const per_sequence_t DL_DPCH_LCR_InformationModify_ModifyList_RL_ReconfRqstTDD_sequence[] = {
  { &hf_nbap_dL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfRqstTDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_DPCH_LCR_InformationModify_ModifyList_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_DPCH_LCR_InformationModify_ModifyList_RL_ReconfRqstTDD, DL_DPCH_LCR_InformationModify_ModifyList_RL_ReconfRqstTDD_sequence);

  return offset;
}
static int dissect_id_DL_DPCH_LCR_InformationModify_ModifyList_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_DPCH_LCR_InformationModify_ModifyList_RL_ReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_id_DL_DPCH_LCR_InformationModify_ModifyList_RL_ReconfRqstTDD);
}
static int dissect_dl_DPCH_LCR_InformationModifyList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_DPCH_LCR_InformationModify_ModifyList_RL_ReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_dl_DPCH_LCR_InformationModifyList);
}



static int
dissect_nbap_HS_DSCHProvidedBitRateValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 16777215U, NULL, TRUE);

  return offset;
}
static int dissect_hS_DSCHProvidedBitRateValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_DSCHProvidedBitRateValue(tvb, offset, actx, tree, hf_nbap_hS_DSCHProvidedBitRateValue);
}


static const per_sequence_t HS_DSCHProvidedBitRate_Item_sequence[] = {
  { &hf_nbap_schedulingPriorityIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SchedulingPriorityIndicator },
  { &hf_nbap_hS_DSCHProvidedBitRateValue, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_HS_DSCHProvidedBitRateValue },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_HS_DSCHProvidedBitRate_Item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_HS_DSCHProvidedBitRate_Item, HS_DSCHProvidedBitRate_Item_sequence);

  return offset;
}
static int dissect_HS_DSCHProvidedBitRate_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_DSCHProvidedBitRate_Item(tvb, offset, actx, tree, hf_nbap_HS_DSCHProvidedBitRate_item);
}


static const per_sequence_t HS_DSCHProvidedBitRate_sequence_of[1] = {
  { &hf_nbap_HS_DSCHProvidedBitRate_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_HS_DSCHProvidedBitRate_Item },
};

static int
dissect_nbap_HS_DSCHProvidedBitRate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_HS_DSCHProvidedBitRate, HS_DSCHProvidedBitRate_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_HS_DSCHProvidedBitRateValueInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_DSCHProvidedBitRate(tvb, offset, actx, tree, hf_nbap_id_HS_DSCHProvidedBitRateValueInformation);
}
static int dissect_hS_DSCHProvidedBitRateValue1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_DSCHProvidedBitRate(tvb, offset, actx, tree, hf_nbap_hS_DSCHProvidedBitRateValue1);
}



static int
dissect_nbap_HS_DSCHRequiredPowerValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 1000U, NULL, FALSE);

  return offset;
}
static int dissect_id_HS_DSCHRequiredPowerValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_DSCHRequiredPowerValue(tvb, offset, actx, tree, hf_nbap_id_HS_DSCHRequiredPowerValue);
}
static int dissect_id_HS_DSCHRequiredPowerValue_For_Cell_Portion(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_DSCHRequiredPowerValue(tvb, offset, actx, tree, hf_nbap_id_HS_DSCHRequiredPowerValue_For_Cell_Portion);
}
static int dissect_hS_DSCHRequiredPowerValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_DSCHRequiredPowerValue(tvb, offset, actx, tree, hf_nbap_hS_DSCHRequiredPowerValue);
}



static int
dissect_nbap_HS_DSCHRequiredPowerPerUEWeight(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 100U, NULL, FALSE);

  return offset;
}
static int dissect_hS_DSCHRequiredPowerPerUEWeight(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_DSCHRequiredPowerPerUEWeight(tvb, offset, actx, tree, hf_nbap_hS_DSCHRequiredPowerPerUEWeight);
}


static const per_sequence_t HS_DSCHRequiredPowerPerUEInformation_Item_sequence[] = {
  { &hf_nbap_cRNC_CommunicationContextID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CRNC_CommunicationContextID },
  { &hf_nbap_hS_DSCHRequiredPowerPerUEWeight, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_HS_DSCHRequiredPowerPerUEWeight },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_HS_DSCHRequiredPowerPerUEInformation_Item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_HS_DSCHRequiredPowerPerUEInformation_Item, HS_DSCHRequiredPowerPerUEInformation_Item_sequence);

  return offset;
}
static int dissect_HS_DSCHRequiredPowerPerUEInformation_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_DSCHRequiredPowerPerUEInformation_Item(tvb, offset, actx, tree, hf_nbap_HS_DSCHRequiredPowerPerUEInformation_item);
}


static const per_sequence_t HS_DSCHRequiredPowerPerUEInformation_sequence_of[1] = {
  { &hf_nbap_HS_DSCHRequiredPowerPerUEInformation_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_HS_DSCHRequiredPowerPerUEInformation_Item },
};

static int
dissect_nbap_HS_DSCHRequiredPowerPerUEInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_HS_DSCHRequiredPowerPerUEInformation, HS_DSCHRequiredPowerPerUEInformation_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_hS_DSCHRequiredPowerPerUEInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_DSCHRequiredPowerPerUEInformation(tvb, offset, actx, tree, hf_nbap_hS_DSCHRequiredPowerPerUEInformation);
}


static const per_sequence_t HS_DSCHRequiredPower_Item_sequence[] = {
  { &hf_nbap_schedulingPriorityIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SchedulingPriorityIndicator },
  { &hf_nbap_hS_DSCHRequiredPowerValue, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_HS_DSCHRequiredPowerValue },
  { &hf_nbap_hS_DSCHRequiredPowerPerUEInformation, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_HS_DSCHRequiredPowerPerUEInformation },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_HS_DSCHRequiredPower_Item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_HS_DSCHRequiredPower_Item, HS_DSCHRequiredPower_Item_sequence);

  return offset;
}
static int dissect_HS_DSCHRequiredPower_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_DSCHRequiredPower_Item(tvb, offset, actx, tree, hf_nbap_HS_DSCHRequiredPower_item);
}


static const per_sequence_t HS_DSCHRequiredPower_sequence_of[1] = {
  { &hf_nbap_HS_DSCHRequiredPower_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_HS_DSCHRequiredPower_Item },
};

static int
dissect_nbap_HS_DSCHRequiredPower(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_HS_DSCHRequiredPower, HS_DSCHRequiredPower_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_HS_DSCHRequiredPowerValueInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_DSCHRequiredPower(tvb, offset, actx, tree, hf_nbap_id_HS_DSCHRequiredPowerValueInformation);
}
static int dissect_hS_DSCHRequiredPowerValue1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_DSCHRequiredPower(tvb, offset, actx, tree, hf_nbap_hS_DSCHRequiredPowerValue1);
}



static int
dissect_nbap_TransmittedCarrierPowerOfAllCodesNotUsedForHSTransmissionValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 100U, NULL, FALSE);

  return offset;
}
static int dissect_id_TransmittedCarrierPowerOfAllCodesNotUsedForHSTransmission(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TransmittedCarrierPowerOfAllCodesNotUsedForHSTransmissionValue(tvb, offset, actx, tree, hf_nbap_id_TransmittedCarrierPowerOfAllCodesNotUsedForHSTransmission);
}
static int dissect_id_TransmittedCarrierPowerOfAllCodesNotUsedForHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHOrE_HICHTransmissionCellPortion(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TransmittedCarrierPowerOfAllCodesNotUsedForHSTransmissionValue(tvb, offset, actx, tree, hf_nbap_id_TransmittedCarrierPowerOfAllCodesNotUsedForHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHOrE_HICHTransmissionCellPortion);
}
static int dissect_transmittedCarrierPowerOfAllCodesNotUsedForHSTransmissionValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TransmittedCarrierPowerOfAllCodesNotUsedForHSTransmissionValue(tvb, offset, actx, tree, hf_nbap_transmittedCarrierPowerOfAllCodesNotUsedForHSTransmissionValue);
}



static int
dissect_nbap_HS_SICH_failed(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 20U, NULL, FALSE);

  return offset;
}
static int dissect_failed_HS_SICH(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_SICH_failed(tvb, offset, actx, tree, hf_nbap_failed_HS_SICH);
}



static int
dissect_nbap_HS_SICH_missed(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 20U, NULL, FALSE);

  return offset;
}
static int dissect_missed_HS_SICH(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_SICH_missed(tvb, offset, actx, tree, hf_nbap_missed_HS_SICH);
}



static int
dissect_nbap_HS_SICH_total(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 20U, NULL, FALSE);

  return offset;
}
static int dissect_total_HS_SICH(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_SICH_total(tvb, offset, actx, tree, hf_nbap_total_HS_SICH);
}


static const per_sequence_t HS_SICH_Reception_Quality_Value_sequence[] = {
  { &hf_nbap_failed_HS_SICH , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_HS_SICH_failed },
  { &hf_nbap_missed_HS_SICH , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_HS_SICH_missed },
  { &hf_nbap_total_HS_SICH  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_HS_SICH_total },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_HS_SICH_Reception_Quality_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_HS_SICH_Reception_Quality_Value, HS_SICH_Reception_Quality_Value_sequence);

  return offset;
}
static int dissect_id_HS_SICH_Reception_Quality(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_SICH_Reception_Quality_Value(tvb, offset, actx, tree, hf_nbap_id_HS_SICH_Reception_Quality);
}



static int
dissect_nbap_HS_SICH_Reception_Quality_Measurement_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 20U, NULL, FALSE);

  return offset;
}
static int dissect_id_HS_SICH_Reception_Quality_Measurement_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_SICH_Reception_Quality_Measurement_Value(tvb, offset, actx, tree, hf_nbap_id_HS_SICH_Reception_Quality_Measurement_Value);
}


static const per_sequence_t HSSICH_Info_DM_Rqst_sequence_of[1] = {
  { &hf_nbap_HSSICH_Info_DM_Rqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_HS_SICH_ID },
};

static int
dissect_nbap_HSSICH_Info_DM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_HSSICH_Info_DM_Rqst, HSSICH_Info_DM_Rqst_sequence_of,
                                                  1, 4);

  return offset;
}
static int dissect_id_HSSICH_Info_DM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HSSICH_Info_DM_Rqst(tvb, offset, actx, tree, hf_nbap_id_HSSICH_Info_DM_Rqst);
}



static int
dissect_nbap_CellPortionID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 63U, NULL, TRUE);

  return offset;
}
static int dissect_cellPortionID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellPortionID(tvb, offset, actx, tree, hf_nbap_cellPortionID);
}


static const per_sequence_t Best_Cell_Portions_Item_sequence[] = {
  { &hf_nbap_cellPortionID  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CellPortionID },
  { &hf_nbap_sIRValue       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SIR_Value },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Best_Cell_Portions_Item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Best_Cell_Portions_Item, Best_Cell_Portions_Item_sequence);

  return offset;
}
static int dissect_Best_Cell_Portions_Value_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Best_Cell_Portions_Item(tvb, offset, actx, tree, hf_nbap_Best_Cell_Portions_Value_item);
}


static const per_sequence_t Best_Cell_Portions_Value_sequence_of[1] = {
  { &hf_nbap_Best_Cell_Portions_Value_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_Best_Cell_Portions_Item },
};

static int
dissect_nbap_Best_Cell_Portions_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_Best_Cell_Portions_Value, Best_Cell_Portions_Value_sequence_of,
                                                  1, 64);

  return offset;
}
static int dissect_id_Best_Cell_Portions_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Best_Cell_Portions_Value(tvb, offset, actx, tree, hf_nbap_id_Best_Cell_Portions_Value);
}


static const value_string nbap_Primary_CPICH_Usage_for_Channel_Estimation_vals[] = {
  {   0, "primary-CPICH-may-be-used" },
  {   1, "primary-CPICH-shall-not-be-used" },
  { 0, NULL }
};


static int
dissect_nbap_Primary_CPICH_Usage_for_Channel_Estimation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_Primary_CPICH_Usage_for_Channel_Estimation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Primary_CPICH_Usage_for_Channel_Estimation(tvb, offset, actx, tree, hf_nbap_id_Primary_CPICH_Usage_for_Channel_Estimation);
}


static const value_string nbap_Secondary_CPICH_Information_Change_vals[] = {
  {   0, "new-secondary-CPICH" },
  {   1, "secondary-CPICH-shall-not-be-used" },
  { 0, NULL }
};

static const per_choice_t Secondary_CPICH_Information_Change_choice[] = {
  {   0, &hf_nbap_new_secondary_CPICH, ASN1_EXTENSION_ROOT    , dissect_nbap_CommonPhysicalChannelID },
  {   1, &hf_nbap_secondary_CPICH_shall_not_be_used, ASN1_EXTENSION_ROOT    , dissect_nbap_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_Secondary_CPICH_Information_Change(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_Secondary_CPICH_Information_Change, Secondary_CPICH_Information_Change_choice,
                                 NULL);

  return offset;
}
static int dissect_id_Secondary_CPICH_Information_Change(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Secondary_CPICH_Information_Change(tvb, offset, actx, tree, hf_nbap_id_Secondary_CPICH_Information_Change);
}



static int
dissect_nbap_NumberOfReportedCellPortions(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 64U, NULL, TRUE);

  return offset;
}
static int dissect_id_NumberOfReportedCellPortions(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NumberOfReportedCellPortions(tvb, offset, actx, tree, hf_nbap_id_NumberOfReportedCellPortions);
}


static const per_sequence_t CellPortion_InformationItem_Cell_SetupRqstFDD_sequence[] = {
  { &hf_nbap_cellPortionID  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CellPortionID },
  { &hf_nbap_associatedSecondaryCPICH, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonPhysicalChannelID },
  { &hf_nbap_maximumTransmissionPowerforCellPortion, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MaximumTransmissionPower },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CellPortion_InformationItem_Cell_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CellPortion_InformationItem_Cell_SetupRqstFDD, CellPortion_InformationItem_Cell_SetupRqstFDD_sequence);

  return offset;
}
static int dissect_id_CellPortion_InformationItem_Cell_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellPortion_InformationItem_Cell_SetupRqstFDD(tvb, offset, actx, tree, hf_nbap_id_CellPortion_InformationItem_Cell_SetupRqstFDD);
}


static const per_sequence_t CellPortion_InformationList_Cell_SetupRqstFDD_sequence_of[1] = {
  { &hf_nbap_CellPortion_InformationList_Cell_SetupRqstFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_CellPortion_InformationList_Cell_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_CellPortion_InformationList_Cell_SetupRqstFDD, CellPortion_InformationList_Cell_SetupRqstFDD_sequence_of,
                                                  1, 64);

  return offset;
}
static int dissect_id_CellPortion_InformationList_Cell_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellPortion_InformationList_Cell_SetupRqstFDD(tvb, offset, actx, tree, hf_nbap_id_CellPortion_InformationList_Cell_SetupRqstFDD);
}


static const value_string nbap_Unidirectional_DCH_Indicator_vals[] = {
  {   0, "downlink-DCH-only" },
  {   1, "uplink-DCH-only" },
  { 0, NULL }
};


static int
dissect_nbap_Unidirectional_DCH_Indicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_Unidirectional_DCH_Indicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Unidirectional_DCH_Indicator(tvb, offset, actx, tree, hf_nbap_id_Unidirectional_DCH_Indicator);
}


static const per_sequence_t MultipleRL_UL_DPCH_InformationAddListIE_RL_ReconfPrepTDD_sequence[] = {
  { &hf_nbap_ul_DPCH_InformationList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_UL_DPCH_InformationAddList_RL_ReconfPrepTDD },
  { &hf_nbap_ul_DPCH_InformationListLCR, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_UL_DPCH_LCR_InformationAddList_RL_ReconfPrepTDD },
  { &hf_nbap_ul_sir_target  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_UL_SIR },
  { &hf_nbap_tDD_TPC_UplinkStepSize_LCR, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TDD_TPC_UplinkStepSize_LCR },
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_RL_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_MultipleRL_UL_DPCH_InformationAddListIE_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_MultipleRL_UL_DPCH_InformationAddListIE_RL_ReconfPrepTDD, MultipleRL_UL_DPCH_InformationAddListIE_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_MultipleRL_UL_DPCH_InformationAddList_RL_ReconfPrepTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MultipleRL_UL_DPCH_InformationAddListIE_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_MultipleRL_UL_DPCH_InformationAddList_RL_ReconfPrepTDD_item);
}


static const per_sequence_t MultipleRL_UL_DPCH_InformationAddList_RL_ReconfPrepTDD_sequence_of[1] = {
  { &hf_nbap_MultipleRL_UL_DPCH_InformationAddList_RL_ReconfPrepTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_MultipleRL_UL_DPCH_InformationAddListIE_RL_ReconfPrepTDD },
};

static int
dissect_nbap_MultipleRL_UL_DPCH_InformationAddList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_MultipleRL_UL_DPCH_InformationAddList_RL_ReconfPrepTDD, MultipleRL_UL_DPCH_InformationAddList_RL_ReconfPrepTDD_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_id_multipleRL_ul_DPCH_InformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MultipleRL_UL_DPCH_InformationAddList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_id_multipleRL_ul_DPCH_InformationList);
}


static const per_sequence_t MultipleRL_UL_DPCH_InformationModifyListIE_RL_ReconfPrepTDD_sequence[] = {
  { &hf_nbap_ul_DPCH_InformationAddList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_UL_DPCH_InformationModify_AddList_RL_ReconfPrepTDD },
  { &hf_nbap_ul_DPCH_InformationModifyList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_UL_DPCH_InformationModify_ModifyList_RL_ReconfPrepTDD },
  { &hf_nbap_ul_DPCH_InformationDeleteList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_UL_DPCH_InformationModify_DeleteList_RL_ReconfPrepTDD },
  { &hf_nbap_ul_DPCH_InformationAddListLCR, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_UL_DPCH_LCR_InformationModify_AddList_RL_ReconfPrepTDD },
  { &hf_nbap_ul_sir_target  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_UL_SIR },
  { &hf_nbap_tDD_TPC_UplinkStepSize_LCR, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TDD_TPC_UplinkStepSize_LCR },
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_RL_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_MultipleRL_UL_DPCH_InformationModifyListIE_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_MultipleRL_UL_DPCH_InformationModifyListIE_RL_ReconfPrepTDD, MultipleRL_UL_DPCH_InformationModifyListIE_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_MultipleRL_UL_DPCH_InformationModifyList_RL_ReconfPrepTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MultipleRL_UL_DPCH_InformationModifyListIE_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_MultipleRL_UL_DPCH_InformationModifyList_RL_ReconfPrepTDD_item);
}


static const per_sequence_t MultipleRL_UL_DPCH_InformationModifyList_RL_ReconfPrepTDD_sequence_of[1] = {
  { &hf_nbap_MultipleRL_UL_DPCH_InformationModifyList_RL_ReconfPrepTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_MultipleRL_UL_DPCH_InformationModifyListIE_RL_ReconfPrepTDD },
};

static int
dissect_nbap_MultipleRL_UL_DPCH_InformationModifyList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_MultipleRL_UL_DPCH_InformationModifyList_RL_ReconfPrepTDD, MultipleRL_UL_DPCH_InformationModifyList_RL_ReconfPrepTDD_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_id_multipleRL_ul_DPCH_InformationModifyList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MultipleRL_UL_DPCH_InformationModifyList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_id_multipleRL_ul_DPCH_InformationModifyList);
}


static const per_sequence_t SAT_Info_Almanac_ExtItem_sequence[] = {
  { &hf_nbap_data_id        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DATA_ID },
  { &hf_nbap_sat_id         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SAT_ID },
  { &hf_nbap_gps_e_alm      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_16 },
  { &hf_nbap_gps_toa_alm    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_8 },
  { &hf_nbap_gps_delta_I_alm, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_16 },
  { &hf_nbap_omegadot_alm   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_16 },
  { &hf_nbap_svhealth_alm   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_8 },
  { &hf_nbap_gps_a_sqrt_alm , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_24 },
  { &hf_nbap_omegazero_alm  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_24 },
  { &hf_nbap_m_zero_alm     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_24 },
  { &hf_nbap_gps_omega_alm  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_24 },
  { &hf_nbap_gps_af_zero_alm, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_11 },
  { &hf_nbap_gps_af_one_alm , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_BIT_STRING_SIZE_11 },
  { &hf_nbap_ie_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_SAT_Info_Almanac_ExtItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_SAT_Info_Almanac_ExtItem, SAT_Info_Almanac_ExtItem_sequence);

  return offset;
}
static int dissect_id_SAT_Info_Almanac_ExtItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SAT_Info_Almanac_ExtItem(tvb, offset, actx, tree, hf_nbap_id_SAT_Info_Almanac_ExtItem);
}
static int dissect_SAT_Info_Almanac_ExtList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SAT_Info_Almanac_ExtItem(tvb, offset, actx, tree, hf_nbap_SAT_Info_Almanac_ExtList_item);
}


static const value_string nbap_HSDPA_Capability_vals[] = {
  {   0, "hsdpa-capable" },
  {   1, "hsdpa-non-capable" },
  { 0, NULL }
};


static int
dissect_nbap_HSDPA_Capability(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_HSDPA_Capability(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HSDPA_Capability(tvb, offset, actx, tree, hf_nbap_id_HSDPA_Capability);
}


static const per_sequence_t HS_DSCH_Resources_Information_AuditRsp_sequence[] = {
  { &hf_nbap_resourceOperationalState, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ResourceOperationalState },
  { &hf_nbap_availabilityStatus, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_AvailabilityStatus },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_HS_DSCH_Resources_Information_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_HS_DSCH_Resources_Information_AuditRsp, HS_DSCH_Resources_Information_AuditRsp_sequence);

  return offset;
}
static int dissect_id_HSDSCH_Resources_Information_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_DSCH_Resources_Information_AuditRsp(tvb, offset, actx, tree, hf_nbap_id_HSDSCH_Resources_Information_AuditRsp);
}


static const per_sequence_t HS_DSCH_Resources_Information_ResourceStatusInd_sequence[] = {
  { &hf_nbap_resourceOperationalState, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ResourceOperationalState },
  { &hf_nbap_availabilityStatus, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_AvailabilityStatus },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_HS_DSCH_Resources_Information_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_HS_DSCH_Resources_Information_ResourceStatusInd, HS_DSCH_Resources_Information_ResourceStatusInd_sequence);

  return offset;
}
static int dissect_id_HSDSCH_Resources_Information_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_DSCH_Resources_Information_ResourceStatusInd(tvb, offset, actx, tree, hf_nbap_id_HSDSCH_Resources_Information_ResourceStatusInd);
}


static const per_sequence_t HSDSCH_MACdFlows_to_Delete_Item_sequence[] = {
  { &hf_nbap_hsDSCH_MACdFlow_ID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_HSDSCH_MACdFlow_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_HSDSCH_MACdFlows_to_Delete_Item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_HSDSCH_MACdFlows_to_Delete_Item, HSDSCH_MACdFlows_to_Delete_Item_sequence);

  return offset;
}
static int dissect_HSDSCH_MACdFlows_to_Delete_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HSDSCH_MACdFlows_to_Delete_Item(tvb, offset, actx, tree, hf_nbap_HSDSCH_MACdFlows_to_Delete_item);
}


static const per_sequence_t HSDSCH_MACdFlows_to_Delete_sequence_of[1] = {
  { &hf_nbap_HSDSCH_MACdFlows_to_Delete_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_HSDSCH_MACdFlows_to_Delete_Item },
};

static int
dissect_nbap_HSDSCH_MACdFlows_to_Delete(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_HSDSCH_MACdFlows_to_Delete, HSDSCH_MACdFlows_to_Delete_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_id_HSDSCH_MACdFlows_to_Delete(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HSDSCH_MACdFlows_to_Delete(tvb, offset, actx, tree, hf_nbap_id_HSDSCH_MACdFlows_to_Delete);
}


static const per_sequence_t PriorityQueue_InfoItem_to_Modify_Unsynchronised_sequence[] = {
  { &hf_nbap_priorityQueueId, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PriorityQueue_Id },
  { &hf_nbap_schedulingPriorityIndicator, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_SchedulingPriorityIndicator },
  { &hf_nbap_discardTimer   , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DiscardTimer },
  { &hf_nbap_mAChsGuaranteedBitRate, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_MAChsGuaranteedBitRate },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PriorityQueue_InfoItem_to_Modify_Unsynchronised(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PriorityQueue_InfoItem_to_Modify_Unsynchronised, PriorityQueue_InfoItem_to_Modify_Unsynchronised_sequence);

  return offset;
}
static int dissect_PriorityQueue_InfoList_to_Modify_Unsynchronised_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PriorityQueue_InfoItem_to_Modify_Unsynchronised(tvb, offset, actx, tree, hf_nbap_PriorityQueue_InfoList_to_Modify_Unsynchronised_item);
}


static const per_sequence_t PriorityQueue_InfoList_to_Modify_Unsynchronised_sequence_of[1] = {
  { &hf_nbap_PriorityQueue_InfoList_to_Modify_Unsynchronised_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_PriorityQueue_InfoItem_to_Modify_Unsynchronised },
};

static int
dissect_nbap_PriorityQueue_InfoList_to_Modify_Unsynchronised(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_PriorityQueue_InfoList_to_Modify_Unsynchronised, PriorityQueue_InfoList_to_Modify_Unsynchronised_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_priorityQueueInfotoModifyUnsynchronised(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PriorityQueue_InfoList_to_Modify_Unsynchronised(tvb, offset, actx, tree, hf_nbap_priorityQueueInfotoModifyUnsynchronised);
}


static const per_sequence_t HSDSCH_Information_to_Modify_Unsynchronised_sequence[] = {
  { &hf_nbap_hsDSCH_MACdFlow_Specific_Info_to_Modify, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_HSDSCH_MACdFlow_Specific_InfoList_to_Modify },
  { &hf_nbap_priorityQueueInfotoModifyUnsynchronised, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_PriorityQueue_InfoList_to_Modify_Unsynchronised },
  { &hf_nbap_cqiPowerOffset , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_CQI_Power_Offset },
  { &hf_nbap_ackPowerOffset , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_Ack_Power_Offset },
  { &hf_nbap_nackPowerOffset, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_Nack_Power_Offset },
  { &hf_nbap_hsscch_PowerOffset, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_HSSCCH_PowerOffset },
  { &hf_nbap_tDDAckNackPowerOffset, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TDD_AckNack_Power_Offset },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_HSDSCH_Information_to_Modify_Unsynchronised(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_HSDSCH_Information_to_Modify_Unsynchronised, HSDSCH_Information_to_Modify_Unsynchronised_sequence);

  return offset;
}
static int dissect_id_HSDSCH_Information_to_Modify_Unsynchronised(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HSDSCH_Information_to_Modify_Unsynchronised(tvb, offset, actx, tree, hf_nbap_id_HSDSCH_Information_to_Modify_Unsynchronised);
}



static int
dissect_nbap_DsField(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     8, 8, FALSE, NULL);

  return offset;
}
static int dissect_dsField(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DsField(tvb, offset, actx, tree, hf_nbap_dsField);
}



static int
dissect_nbap_GenericTrafficCategory(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     8, 8, FALSE, NULL);

  return offset;
}
static int dissect_genericTrafficCategory(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_GenericTrafficCategory(tvb, offset, actx, tree, hf_nbap_genericTrafficCategory);
}


static const value_string nbap_TnlQos_vals[] = {
  {   0, "dsField" },
  {   1, "genericTrafficCategory" },
  { 0, NULL }
};

static const per_choice_t TnlQos_choice[] = {
  {   0, &hf_nbap_dsField        , ASN1_EXTENSION_ROOT    , dissect_nbap_DsField },
  {   1, &hf_nbap_genericTrafficCategory, ASN1_EXTENSION_ROOT    , dissect_nbap_GenericTrafficCategory },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_TnlQos(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_TnlQos, TnlQos_choice,
                                 NULL);

  return offset;
}
static int dissect_id_TnlQos(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TnlQos(tvb, offset, actx, tree, hf_nbap_id_TnlQos);
}
static int dissect_tnlQos(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TnlQos(tvb, offset, actx, tree, hf_nbap_tnlQos);
}


static const per_sequence_t Received_total_wide_band_power_For_CellPortion_Value_Item_sequence[] = {
  { &hf_nbap_cellPortionID  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CellPortionID },
  { &hf_nbap_received_total_wide_band_power_value, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Received_total_wide_band_power_Value },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Received_total_wide_band_power_For_CellPortion_Value_Item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Received_total_wide_band_power_For_CellPortion_Value_Item, Received_total_wide_band_power_For_CellPortion_Value_Item_sequence);

  return offset;
}
static int dissect_Received_total_wide_band_power_For_CellPortion_Value_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Received_total_wide_band_power_For_CellPortion_Value_Item(tvb, offset, actx, tree, hf_nbap_Received_total_wide_band_power_For_CellPortion_Value_item);
}


static const per_sequence_t Received_total_wide_band_power_For_CellPortion_Value_sequence_of[1] = {
  { &hf_nbap_Received_total_wide_band_power_For_CellPortion_Value_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_Received_total_wide_band_power_For_CellPortion_Value_Item },
};

static int
dissect_nbap_Received_total_wide_band_power_For_CellPortion_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_Received_total_wide_band_power_For_CellPortion_Value, Received_total_wide_band_power_For_CellPortion_Value_sequence_of,
                                                  1, 64);

  return offset;
}
static int dissect_id_Received_total_wide_band_power_For_CellPortion_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Received_total_wide_band_power_For_CellPortion_Value(tvb, offset, actx, tree, hf_nbap_id_Received_total_wide_band_power_For_CellPortion_Value);
}


static const per_sequence_t Transmitted_Carrier_Power_For_CellPortion_Value_Item_sequence[] = {
  { &hf_nbap_cellPortionID  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CellPortionID },
  { &hf_nbap_transmitted_Carrier_Power_Value, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Transmitted_Carrier_Power_Value },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Transmitted_Carrier_Power_For_CellPortion_Value_Item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Transmitted_Carrier_Power_For_CellPortion_Value_Item, Transmitted_Carrier_Power_For_CellPortion_Value_Item_sequence);

  return offset;
}
static int dissect_Transmitted_Carrier_Power_For_CellPortion_Value_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Transmitted_Carrier_Power_For_CellPortion_Value_Item(tvb, offset, actx, tree, hf_nbap_Transmitted_Carrier_Power_For_CellPortion_Value_item);
}


static const per_sequence_t Transmitted_Carrier_Power_For_CellPortion_Value_sequence_of[1] = {
  { &hf_nbap_Transmitted_Carrier_Power_For_CellPortion_Value_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_Transmitted_Carrier_Power_For_CellPortion_Value_Item },
};

static int
dissect_nbap_Transmitted_Carrier_Power_For_CellPortion_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_Transmitted_Carrier_Power_For_CellPortion_Value, Transmitted_Carrier_Power_For_CellPortion_Value_sequence_of,
                                                  1, 64);

  return offset;
}
static int dissect_id_Transmitted_Carrier_Power_For_CellPortion_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Transmitted_Carrier_Power_For_CellPortion_Value(tvb, offset, actx, tree, hf_nbap_id_Transmitted_Carrier_Power_For_CellPortion_Value);
}


static const per_sequence_t TransmittedCarrierPowerOfAllCodesNotUsedForHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHOrE_HICHTransmissionCellPortionValue_Item_sequence[] = {
  { &hf_nbap_cellPortionID  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CellPortionID },
  { &hf_nbap_transmittedCarrierPowerOfAllCodesNotUsedForHSTransmissionValue, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TransmittedCarrierPowerOfAllCodesNotUsedForHSTransmissionValue },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_TransmittedCarrierPowerOfAllCodesNotUsedForHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHOrE_HICHTransmissionCellPortionValue_Item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_TransmittedCarrierPowerOfAllCodesNotUsedForHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHOrE_HICHTransmissionCellPortionValue_Item, TransmittedCarrierPowerOfAllCodesNotUsedForHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHOrE_HICHTransmissionCellPortionValue_Item_sequence);

  return offset;
}
static int dissect_TransmittedCarrierPowerOfAllCodesNotUsedForHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHOrE_HICHTransmissionCellPortionValue_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TransmittedCarrierPowerOfAllCodesNotUsedForHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHOrE_HICHTransmissionCellPortionValue_Item(tvb, offset, actx, tree, hf_nbap_TransmittedCarrierPowerOfAllCodesNotUsedForHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHOrE_HICHTransmissionCellPortionValue_item);
}


static const per_sequence_t TransmittedCarrierPowerOfAllCodesNotUsedForHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHOrE_HICHTransmissionCellPortionValue_sequence_of[1] = {
  { &hf_nbap_TransmittedCarrierPowerOfAllCodesNotUsedForHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHOrE_HICHTransmissionCellPortionValue_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_TransmittedCarrierPowerOfAllCodesNotUsedForHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHOrE_HICHTransmissionCellPortionValue_Item },
};

static int
dissect_nbap_TransmittedCarrierPowerOfAllCodesNotUsedForHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHOrE_HICHTransmissionCellPortionValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_TransmittedCarrierPowerOfAllCodesNotUsedForHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHOrE_HICHTransmissionCellPortionValue, TransmittedCarrierPowerOfAllCodesNotUsedForHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHOrE_HICHTransmissionCellPortionValue_sequence_of,
                                                  1, 64);

  return offset;
}
static int dissect_id_TransmittedCarrierPowerOfAllCodesNotUsedForHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHOrE_HICHTransmissionCellPortionValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_TransmittedCarrierPowerOfAllCodesNotUsedForHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHOrE_HICHTransmissionCellPortionValue(tvb, offset, actx, tree, hf_nbap_id_TransmittedCarrierPowerOfAllCodesNotUsedForHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHOrE_HICHTransmissionCellPortionValue);
}



static int
dissect_nbap_UpPTSInterferenceValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 127U, NULL, TRUE);

  return offset;
}
static int dissect_id_UpPTSInterferenceValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UpPTSInterferenceValue(tvb, offset, actx, tree, hf_nbap_id_UpPTSInterferenceValue);
}



static int
dissect_nbap_PrimaryCCPCH_RSCP_Delta(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              -5, -1, NULL, TRUE);

  return offset;
}
static int dissect_id_PrimaryCCPCH_RSCP_Delta(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PrimaryCCPCH_RSCP_Delta(tvb, offset, actx, tree, hf_nbap_id_PrimaryCCPCH_RSCP_Delta);
}



static int
dissect_nbap_MeasurementRecoveryBehavior(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_null(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_id_MeasurementRecoveryBehavior(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MeasurementRecoveryBehavior(tvb, offset, actx, tree, hf_nbap_id_MeasurementRecoveryBehavior);
}



static int
dissect_nbap_MeasurementRecoveryReportingIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_null(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_id_MeasurementRecoveryReportingIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MeasurementRecoveryReportingIndicator(tvb, offset, actx, tree, hf_nbap_id_MeasurementRecoveryReportingIndicator);
}



static int
dissect_nbap_MeasurementRecoverySupportIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_null(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_id_MeasurementRecoverySupportIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MeasurementRecoverySupportIndicator(tvb, offset, actx, tree, hf_nbap_id_MeasurementRecoverySupportIndicator);
}


static const per_sequence_t MultipleRL_Information_RL_ReconfPrepTDD_sequence_of[1] = {
  { &hf_nbap_MultipleRL_Information_RL_ReconfPrepTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_RL_Information_RL_ReconfPrepTDD },
};

static int
dissect_nbap_MultipleRL_Information_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_MultipleRL_Information_RL_ReconfPrepTDD, MultipleRL_Information_RL_ReconfPrepTDD_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_id_multiple_RL_Information_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MultipleRL_Information_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_id_multiple_RL_Information_RL_ReconfPrepTDD);
}


static const per_sequence_t Multiple_RL_Information_RL_ReconfRqstTDD_sequence_of[1] = {
  { &hf_nbap_Multiple_RL_Information_RL_ReconfRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_RL_Information_RL_ReconfRqstTDD },
};

static int
dissect_nbap_Multiple_RL_Information_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_Multiple_RL_Information_RL_ReconfRqstTDD, Multiple_RL_Information_RL_ReconfRqstTDD_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_id_multiple_RL_Information_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Multiple_RL_Information_RL_ReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_id_multiple_RL_Information_RL_ReconfRqstTDD);
}


static const per_sequence_t PowerOffsetInformation_RL_ReconfPrepFDD_sequence[] = {
  { &hf_nbap_pO1_ForTFCI_Bits, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PowerOffset },
  { &hf_nbap_pO2_ForTPC_Bits, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PowerOffset },
  { &hf_nbap_pO3_ForPilotBits, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PowerOffset },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PowerOffsetInformation_RL_ReconfPrepFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PowerOffsetInformation_RL_ReconfPrepFDD, PowerOffsetInformation_RL_ReconfPrepFDD_sequence);

  return offset;
}
static int dissect_powerOffsetInformation3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PowerOffsetInformation_RL_ReconfPrepFDD(tvb, offset, actx, tree, hf_nbap_powerOffsetInformation3);
}


static const per_sequence_t DL_DPCH_Power_Information_RL_ReconfPrepFDD_sequence[] = {
  { &hf_nbap_powerOffsetInformation3, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PowerOffsetInformation_RL_ReconfPrepFDD },
  { &hf_nbap_fdd_TPC_DownlinkStepSize, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_FDD_TPC_DownlinkStepSize },
  { &hf_nbap_innerLoopDLPCStatus, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_InnerLoopDLPCStatus },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_DL_DPCH_Power_Information_RL_ReconfPrepFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_DL_DPCH_Power_Information_RL_ReconfPrepFDD, DL_DPCH_Power_Information_RL_ReconfPrepFDD_sequence);

  return offset;
}
static int dissect_id_DL_DPCH_Power_Information_RL_ReconfPrepFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DL_DPCH_Power_Information_RL_ReconfPrepFDD(tvb, offset, actx, tree, hf_nbap_id_DL_DPCH_Power_Information_RL_ReconfPrepFDD);
}


static const per_sequence_t PowerOffsetInformation_F_DPCH_RL_ReconfPrepFDD_sequence[] = {
  { &hf_nbap_pO2_ForTPC_Bits, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PowerOffset },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PowerOffsetInformation_F_DPCH_RL_ReconfPrepFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PowerOffsetInformation_F_DPCH_RL_ReconfPrepFDD, PowerOffsetInformation_F_DPCH_RL_ReconfPrepFDD_sequence);

  return offset;
}
static int dissect_powerOffsetInformation4(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PowerOffsetInformation_F_DPCH_RL_ReconfPrepFDD(tvb, offset, actx, tree, hf_nbap_powerOffsetInformation4);
}


static const per_sequence_t F_DPCH_Information_RL_ReconfPrepFDD_sequence[] = {
  { &hf_nbap_powerOffsetInformation4, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PowerOffsetInformation_F_DPCH_RL_ReconfPrepFDD },
  { &hf_nbap_fdd_TPC_DownlinkStepSize, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_FDD_TPC_DownlinkStepSize },
  { &hf_nbap_limitedPowerIncrease, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_LimitedPowerIncrease },
  { &hf_nbap_innerLoopDLPCStatus, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_InnerLoopDLPCStatus },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_F_DPCH_Information_RL_ReconfPrepFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_F_DPCH_Information_RL_ReconfPrepFDD, F_DPCH_Information_RL_ReconfPrepFDD_sequence);

  return offset;
}
static int dissect_id_F_DPCH_Information_RL_ReconfPrepFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_F_DPCH_Information_RL_ReconfPrepFDD(tvb, offset, actx, tree, hf_nbap_id_F_DPCH_Information_RL_ReconfPrepFDD);
}


static const per_sequence_t PowerOffsetInformation_F_DPCH_RL_SetupRqstFDD_sequence[] = {
  { &hf_nbap_pO2_ForTPC_Bits, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PowerOffset },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PowerOffsetInformation_F_DPCH_RL_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PowerOffsetInformation_F_DPCH_RL_SetupRqstFDD, PowerOffsetInformation_F_DPCH_RL_SetupRqstFDD_sequence);

  return offset;
}
static int dissect_powerOffsetInformation2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PowerOffsetInformation_F_DPCH_RL_SetupRqstFDD(tvb, offset, actx, tree, hf_nbap_powerOffsetInformation2);
}


static const per_sequence_t F_DPCH_Information_RL_SetupRqstFDD_sequence[] = {
  { &hf_nbap_powerOffsetInformation2, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PowerOffsetInformation_F_DPCH_RL_SetupRqstFDD },
  { &hf_nbap_fdd_TPC_DownlinkStepSize, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_FDD_TPC_DownlinkStepSize },
  { &hf_nbap_limitedPowerIncrease, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_LimitedPowerIncrease },
  { &hf_nbap_innerLoopDLPCStatus, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_InnerLoopDLPCStatus },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_F_DPCH_Information_RL_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_F_DPCH_Information_RL_SetupRqstFDD, F_DPCH_Information_RL_SetupRqstFDD_sequence);

  return offset;
}
static int dissect_id_F_DPCH_Information_RL_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_F_DPCH_Information_RL_SetupRqstFDD(tvb, offset, actx, tree, hf_nbap_id_F_DPCH_Information_RL_SetupRqstFDD);
}


static const per_sequence_t Secondary_CCPCH_parameterExtendedList_CTCH_ReconfRqstTDD_sequence_of[1] = {
  { &hf_nbap_Secondary_CCPCH_parameterExtendedList_CTCH_ReconfRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_Secondary_CCPCHItem_CTCH_ReconfRqstTDD },
};

static int
dissect_nbap_Secondary_CCPCH_parameterExtendedList_CTCH_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_Secondary_CCPCH_parameterExtendedList_CTCH_ReconfRqstTDD, Secondary_CCPCH_parameterExtendedList_CTCH_ReconfRqstTDD_sequence_of,
                                                  1, 232);

  return offset;
}
static int dissect_id_Additional_S_CCPCH_Parameters_CTCH_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Secondary_CCPCH_parameterExtendedList_CTCH_ReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_id_Additional_S_CCPCH_Parameters_CTCH_ReconfRqstTDD);
}


static const per_sequence_t Secondary_CCPCH_parameterExtendedList_CTCH_SetupRqstTDD_sequence_of[1] = {
  { &hf_nbap_Secondary_CCPCH_parameterExtendedList_CTCH_SetupRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_Secondary_CCPCH_parameterItem_CTCH_SetupRqstTDD },
};

static int
dissect_nbap_Secondary_CCPCH_parameterExtendedList_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_Secondary_CCPCH_parameterExtendedList_CTCH_SetupRqstTDD, Secondary_CCPCH_parameterExtendedList_CTCH_SetupRqstTDD_sequence_of,
                                                  1, 232);

  return offset;
}
static int dissect_id_Additional_S_CCPCH_Parameters_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Secondary_CCPCH_parameterExtendedList_CTCH_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_id_Additional_S_CCPCH_Parameters_CTCH_SetupRqstTDD);
}


static const per_sequence_t Secondary_CCPCH_LCR_parameterExtendedList_CTCH_ReconfRqstTDD_sequence_of[1] = {
  { &hf_nbap_Secondary_CCPCH_LCR_parameterExtendedList_CTCH_ReconfRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_Secondary_CCPCHItem_CTCH_ReconfRqstTDD },
};

static int
dissect_nbap_Secondary_CCPCH_LCR_parameterExtendedList_CTCH_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_Secondary_CCPCH_LCR_parameterExtendedList_CTCH_ReconfRqstTDD, Secondary_CCPCH_LCR_parameterExtendedList_CTCH_ReconfRqstTDD_sequence_of,
                                                  1, 88);

  return offset;
}
static int dissect_id_Additional_S_CCPCH_LCR_Parameters_CTCH_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Secondary_CCPCH_LCR_parameterExtendedList_CTCH_ReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_id_Additional_S_CCPCH_LCR_Parameters_CTCH_ReconfRqstTDD);
}


static const per_sequence_t Secondary_CCPCH_LCR_parameterExtendedList_CTCH_SetupRqstTDD_sequence_of[1] = {
  { &hf_nbap_Secondary_CCPCH_LCR_parameterExtendedList_CTCH_SetupRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_Secondary_CCPCH_LCR_parameterItem_CTCH_SetupRqstTDD },
};

static int
dissect_nbap_Secondary_CCPCH_LCR_parameterExtendedList_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_Secondary_CCPCH_LCR_parameterExtendedList_CTCH_SetupRqstTDD, Secondary_CCPCH_LCR_parameterExtendedList_CTCH_SetupRqstTDD_sequence_of,
                                                  1, 88);

  return offset;
}
static int dissect_id_Additional_S_CCPCH_LCR_Parameters_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Secondary_CCPCH_LCR_parameterExtendedList_CTCH_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_id_Additional_S_CCPCH_LCR_Parameters_CTCH_SetupRqstTDD);
}



static int
dissect_nbap_MICH_CFN(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 4095U, NULL, FALSE);

  return offset;
}
static int dissect_id_MICH_CFN(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MICH_CFN(tvb, offset, actx, tree, hf_nbap_id_MICH_CFN);
}


static const per_sequence_t MICH_Parameters_CTCH_ReconfRqstFDD_sequence[] = {
  { &hf_nbap_commonPhysicalChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonPhysicalChannelID },
  { &hf_nbap_mICH_Power     , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_PICH_Power },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_MICH_Parameters_CTCH_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_MICH_Parameters_CTCH_ReconfRqstFDD, MICH_Parameters_CTCH_ReconfRqstFDD_sequence);

  return offset;
}
static int dissect_id_MICH_Parameters_CTCH_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MICH_Parameters_CTCH_ReconfRqstFDD(tvb, offset, actx, tree, hf_nbap_id_MICH_Parameters_CTCH_ReconfRqstFDD);
}


static const per_sequence_t MICH_Parameters_CTCH_ReconfRqstTDD_sequence[] = {
  { &hf_nbap_commonPhysicalChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonPhysicalChannelID },
  { &hf_nbap_mICH_Power     , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_PICH_Power },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_MICH_Parameters_CTCH_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_MICH_Parameters_CTCH_ReconfRqstTDD, MICH_Parameters_CTCH_ReconfRqstTDD_sequence);

  return offset;
}
static int dissect_id_MICH_Parameters_CTCH_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MICH_Parameters_CTCH_ReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_id_MICH_Parameters_CTCH_ReconfRqstTDD);
}


static const value_string nbap_MICH_Mode_vals[] = {
  {   0, "v18" },
  {   1, "v36" },
  {   2, "v72" },
  {   3, "v144" },
  { 0, NULL }
};


static int
dissect_nbap_MICH_Mode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_mICH_Mode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MICH_Mode(tvb, offset, actx, tree, hf_nbap_mICH_Mode);
}


static const per_sequence_t MICH_Parameters_CTCH_SetupRqstFDD_sequence[] = {
  { &hf_nbap_commonPhysicalChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonPhysicalChannelID },
  { &hf_nbap_fdd_dl_ChannelisationCodeNumber, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_FDD_DL_ChannelisationCodeNumber },
  { &hf_nbap_mICH_Power     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PICH_Power },
  { &hf_nbap_mICH_Mode      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MICH_Mode },
  { &hf_nbap_sTTD_Indicator , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_STTD_Indicator },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_MICH_Parameters_CTCH_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_MICH_Parameters_CTCH_SetupRqstFDD, MICH_Parameters_CTCH_SetupRqstFDD_sequence);

  return offset;
}
static int dissect_id_MICH_Parameters_CTCH_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MICH_Parameters_CTCH_SetupRqstFDD(tvb, offset, actx, tree, hf_nbap_id_MICH_Parameters_CTCH_SetupRqstFDD);
}


static const value_string nbap_NotificationIndicatorLength_vals[] = {
  {   0, "v2" },
  {   1, "v4" },
  {   2, "v8" },
  { 0, NULL }
};


static int
dissect_nbap_NotificationIndicatorLength(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_notificationIndicatorLength(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NotificationIndicatorLength(tvb, offset, actx, tree, hf_nbap_notificationIndicatorLength);
}


static const per_sequence_t MICH_HCR_Parameters_CTCH_SetupRqstTDD_sequence[] = {
  { &hf_nbap_tdd_ChannelisationCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_ChannelisationCode },
  { &hf_nbap_timeSlot       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlot },
  { &hf_nbap_midambleshiftAndBurstType, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MidambleShiftAndBurstType },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_MICH_HCR_Parameters_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_MICH_HCR_Parameters_CTCH_SetupRqstTDD, MICH_HCR_Parameters_CTCH_SetupRqstTDD_sequence);

  return offset;
}
static int dissect_hCR_TDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MICH_HCR_Parameters_CTCH_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_hCR_TDD);
}


static const per_sequence_t MICH_LCR_Parameters_CTCH_SetupRqstTDD_sequence[] = {
  { &hf_nbap_tdd_ChannelisationCodeLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_ChannelisationCodeLCR },
  { &hf_nbap_timeSlotLCR    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TimeSlotLCR },
  { &hf_nbap_midambleShiftLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MidambleShiftLCR },
  { &hf_nbap_second_TDD_ChannelisationCodeLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_ChannelisationCodeLCR },
  { &hf_nbap_tSTD_Indicator , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TSTD_Indicator },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_MICH_LCR_Parameters_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_MICH_LCR_Parameters_CTCH_SetupRqstTDD, MICH_LCR_Parameters_CTCH_SetupRqstTDD_sequence);

  return offset;
}
static int dissect_lCR_TDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MICH_LCR_Parameters_CTCH_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_lCR_TDD);
}


static const value_string nbap_MICH_TDDOption_Specific_Parameters_CTCH_SetupRqstTDD_vals[] = {
  {   0, "hCR-TDD" },
  {   1, "lCR-TDD" },
  { 0, NULL }
};

static const per_choice_t MICH_TDDOption_Specific_Parameters_CTCH_SetupRqstTDD_choice[] = {
  {   0, &hf_nbap_hCR_TDD        , ASN1_EXTENSION_ROOT    , dissect_nbap_MICH_HCR_Parameters_CTCH_SetupRqstTDD },
  {   1, &hf_nbap_lCR_TDD        , ASN1_EXTENSION_ROOT    , dissect_nbap_MICH_LCR_Parameters_CTCH_SetupRqstTDD },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_MICH_TDDOption_Specific_Parameters_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_MICH_TDDOption_Specific_Parameters_CTCH_SetupRqstTDD, MICH_TDDOption_Specific_Parameters_CTCH_SetupRqstTDD_choice,
                                 NULL);

  return offset;
}
static int dissect_mICH_TDDOption_Specific_Parameters(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MICH_TDDOption_Specific_Parameters_CTCH_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_mICH_TDDOption_Specific_Parameters);
}


static const per_sequence_t MICH_Parameters_CTCH_SetupRqstTDD_sequence[] = {
  { &hf_nbap_commonPhysicalChannelID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonPhysicalChannelID },
  { &hf_nbap_tdd_PhysicalChannelOffset, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TDD_PhysicalChannelOffset },
  { &hf_nbap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionPeriod },
  { &hf_nbap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RepetitionLength },
  { &hf_nbap_notificationIndicatorLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_NotificationIndicatorLength },
  { &hf_nbap_mICH_Power     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PICH_Power },
  { &hf_nbap_mICH_TDDOption_Specific_Parameters, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MICH_TDDOption_Specific_Parameters_CTCH_SetupRqstTDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_MICH_Parameters_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_MICH_Parameters_CTCH_SetupRqstTDD, MICH_Parameters_CTCH_SetupRqstTDD_sequence);

  return offset;
}
static int dissect_id_MICH_Parameters_CTCH_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MICH_Parameters_CTCH_SetupRqstTDD(tvb, offset, actx, tree, hf_nbap_id_MICH_Parameters_CTCH_SetupRqstTDD);
}


static const value_string nbap_Modification_Period_vals[] = {
  {   0, "v5120" },
  {   1, "v10240" },
  { 0, NULL }
};


static int
dissect_nbap_Modification_Period(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_id_Modification_Period(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Modification_Period(tvb, offset, actx, tree, hf_nbap_id_Modification_Period);
}



static int
dissect_nbap_Notification_Indicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 65535U, NULL, FALSE);

  return offset;
}
static int dissect_NI_Information_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Notification_Indicator(tvb, offset, actx, tree, hf_nbap_NI_Information_item);
}


static const per_sequence_t NI_Information_sequence_of[1] = {
  { &hf_nbap_NI_Information_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_Notification_Indicator },
};

static int
dissect_nbap_NI_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_NI_Information, NI_Information_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_id_NI_Information_NotifUpdateCmd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_NI_Information(tvb, offset, actx, tree, hf_nbap_id_NI_Information_NotifUpdateCmd);
}


static const per_sequence_t S_CCPCH_InformationListExt_AuditRsp_sequence_of[1] = {
  { &hf_nbap_S_CCPCH_InformationListExt_AuditRsp_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_S_CCPCH_InformationListExt_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_S_CCPCH_InformationListExt_AuditRsp, S_CCPCH_InformationListExt_AuditRsp_sequence_of,
                                                  1, 208);

  return offset;
}
static int dissect_id_S_CCPCH_InformationListExt_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_S_CCPCH_InformationListExt_AuditRsp(tvb, offset, actx, tree, hf_nbap_id_S_CCPCH_InformationListExt_AuditRsp);
}


static const per_sequence_t S_CCPCH_InformationListExt_ResourceStatusInd_sequence_of[1] = {
  { &hf_nbap_S_CCPCH_InformationListExt_ResourceStatusInd_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_S_CCPCH_InformationListExt_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_S_CCPCH_InformationListExt_ResourceStatusInd, S_CCPCH_InformationListExt_ResourceStatusInd_sequence_of,
                                                  1, 208);

  return offset;
}
static int dissect_id_S_CCPCH_InformationListExt_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_S_CCPCH_InformationListExt_ResourceStatusInd(tvb, offset, actx, tree, hf_nbap_id_S_CCPCH_InformationListExt_ResourceStatusInd);
}


static const per_sequence_t S_CCPCH_LCR_InformationListExt_AuditRsp_sequence_of[1] = {
  { &hf_nbap_S_CCPCH_LCR_InformationListExt_AuditRsp_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_S_CCPCH_LCR_InformationListExt_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_S_CCPCH_LCR_InformationListExt_AuditRsp, S_CCPCH_LCR_InformationListExt_AuditRsp_sequence_of,
                                                  1, 64);

  return offset;
}
static int dissect_id_S_CCPCH_LCR_InformationListExt_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_S_CCPCH_LCR_InformationListExt_AuditRsp(tvb, offset, actx, tree, hf_nbap_id_S_CCPCH_LCR_InformationListExt_AuditRsp);
}


static const per_sequence_t S_CCPCH_LCR_InformationListExt_ResourceStatusInd_sequence_of[1] = {
  { &hf_nbap_S_CCPCH_LCR_InformationListExt_ResourceStatusInd_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_S_CCPCH_LCR_InformationListExt_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_S_CCPCH_LCR_InformationListExt_ResourceStatusInd, S_CCPCH_LCR_InformationListExt_ResourceStatusInd_sequence_of,
                                                  1, 208);

  return offset;
}
static int dissect_id_S_CCPCH_LCR_InformationListExt_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_S_CCPCH_LCR_InformationListExt_ResourceStatusInd(tvb, offset, actx, tree, hf_nbap_id_S_CCPCH_LCR_InformationListExt_ResourceStatusInd);
}


static const value_string nbap_HARQ_Preamble_Mode_vals[] = {
  {   0, "mode0" },
  {   1, "mode1" },
  { 0, NULL }
};


static int
dissect_nbap_HARQ_Preamble_Mode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_HARQ_Preamble_Mode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HARQ_Preamble_Mode(tvb, offset, actx, tree, hf_nbap_id_HARQ_Preamble_Mode);
}


static const value_string nbap_Initial_DL_DPCH_TimingAdjustment_Allowed_vals[] = {
  {   0, "initial-DL-DPCH-TimingAdjustment-Allowed" },
  { 0, NULL }
};


static int
dissect_nbap_Initial_DL_DPCH_TimingAdjustment_Allowed(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_Initial_DL_DPCH_TimingAdjustment_Allowed(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Initial_DL_DPCH_TimingAdjustment_Allowed(tvb, offset, actx, tree, hf_nbap_id_Initial_DL_DPCH_TimingAdjustment_Allowed);
}



static int
dissect_nbap_DLTransmissionBranchLoadValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 101U, NULL, TRUE);

  return offset;
}
static int dissect_id_DLTransmissionBranchLoadValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_DLTransmissionBranchLoadValue(tvb, offset, actx, tree, hf_nbap_id_DLTransmissionBranchLoadValue);
}


static const per_sequence_t PowerLocalCellGroup_CM_Rqst_sequence[] = {
  { &hf_nbap_powerLocalCellGroupID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Local_Cell_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PowerLocalCellGroup_CM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PowerLocalCellGroup_CM_Rqst, PowerLocalCellGroup_CM_Rqst_sequence);

  return offset;
}
static int dissect_id_Power_Local_Cell_Group_choice_CM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PowerLocalCellGroup_CM_Rqst(tvb, offset, actx, tree, hf_nbap_id_Power_Local_Cell_Group_choice_CM_Rqst);
}


static const per_sequence_t PowerLocalCellGroup_CM_Rsp_sequence[] = {
  { &hf_nbap_commonMeasurementValue, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonMeasurementValue },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PowerLocalCellGroup_CM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PowerLocalCellGroup_CM_Rsp, PowerLocalCellGroup_CM_Rsp_sequence);

  return offset;
}
static int dissect_id_Power_Local_Cell_Group_choice_CM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PowerLocalCellGroup_CM_Rsp(tvb, offset, actx, tree, hf_nbap_id_Power_Local_Cell_Group_choice_CM_Rsp);
}


static const per_sequence_t PowerLocalCellGroup_CM_Rprt_sequence[] = {
  { &hf_nbap_commonMeasurementValueInformation, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CommonMeasurementValueInformation },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_PowerLocalCellGroup_CM_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_PowerLocalCellGroup_CM_Rprt, PowerLocalCellGroup_CM_Rprt_sequence);

  return offset;
}
static int dissect_id_Power_Local_Cell_Group_choice_CM_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_PowerLocalCellGroup_CM_Rprt(tvb, offset, actx, tree, hf_nbap_id_Power_Local_Cell_Group_choice_CM_Rprt);
}


static const value_string nbap_SynchronisationIndicator_vals[] = {
  {   0, "timingMaintainedSynchronisation" },
  { 0, NULL }
};


static int
dissect_nbap_SynchronisationIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_id_SynchronisationIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SynchronisationIndicator(tvb, offset, actx, tree, hf_nbap_id_SynchronisationIndicator);
}



static int
dissect_nbap_HSDPA_And_EDCH_CellPortion_Information_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_id_HSDPA_And_EDCH_CellPortion_Information_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HSDPA_And_EDCH_CellPortion_Information_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_id_HSDPA_And_EDCH_CellPortion_Information_PSCH_ReconfRqst);
}


static const per_sequence_t HSDPA_And_EDCH_CellPortion_InformationItem_PSCH_ReconfRqst_sequence[] = {
  { &hf_nbap_cellPortionID  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CellPortionID },
  { &hf_nbap_hS_PDSCH_HS_SCCH_ScramblingCode_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_ScramblingCode },
  { &hf_nbap_hS_PDSCH_FDD_Code_Information_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_HS_PDSCH_FDD_Code_Information },
  { &hf_nbap_hS_SCCH_FDD_Code_Information_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_HS_SCCH_FDD_Code_Information },
  { &hf_nbap_hS_PDSCH_HS_SCCH_E_AGCH_E_RGCH_E_HICH_MaxPower_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_MaximumTransmissionPower },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_HSDPA_And_EDCH_CellPortion_InformationItem_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_HSDPA_And_EDCH_CellPortion_InformationItem_PSCH_ReconfRqst, HSDPA_And_EDCH_CellPortion_InformationItem_PSCH_ReconfRqst_sequence);

  return offset;
}
static int dissect_HSDPA_And_EDCH_CellPortion_InformationList_PSCH_ReconfRqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HSDPA_And_EDCH_CellPortion_InformationItem_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_HSDPA_And_EDCH_CellPortion_InformationList_PSCH_ReconfRqst_item);
}


static const per_sequence_t HSDPA_And_EDCH_CellPortion_InformationList_PSCH_ReconfRqst_sequence_of[1] = {
  { &hf_nbap_HSDPA_And_EDCH_CellPortion_InformationList_PSCH_ReconfRqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_HSDPA_And_EDCH_CellPortion_InformationItem_PSCH_ReconfRqst },
};

static int
dissect_nbap_HSDPA_And_EDCH_CellPortion_InformationList_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_HSDPA_And_EDCH_CellPortion_InformationList_PSCH_ReconfRqst, HSDPA_And_EDCH_CellPortion_InformationList_PSCH_ReconfRqst_sequence_of,
                                                  1, 64);

  return offset;
}
static int dissect_id_HSDPA_And_EDCH_CellPortion_InformationList_PSCH_ReconfRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HSDPA_And_EDCH_CellPortion_InformationList_PSCH_ReconfRqst(tvb, offset, actx, tree, hf_nbap_id_HSDPA_And_EDCH_CellPortion_InformationList_PSCH_ReconfRqst);
}


static const per_sequence_t HS_DSCHRequiredPowerValueInformation_For_CellPortion_Item_sequence[] = {
  { &hf_nbap_cellPortionID  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CellPortionID },
  { &hf_nbap_hS_DSCHRequiredPowerValue1, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_HS_DSCHRequiredPower },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_HS_DSCHRequiredPowerValueInformation_For_CellPortion_Item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_HS_DSCHRequiredPowerValueInformation_For_CellPortion_Item, HS_DSCHRequiredPowerValueInformation_For_CellPortion_Item_sequence);

  return offset;
}
static int dissect_HS_DSCHRequiredPowerValueInformation_For_CellPortion_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_DSCHRequiredPowerValueInformation_For_CellPortion_Item(tvb, offset, actx, tree, hf_nbap_HS_DSCHRequiredPowerValueInformation_For_CellPortion_item);
}


static const per_sequence_t HS_DSCHRequiredPowerValueInformation_For_CellPortion_sequence_of[1] = {
  { &hf_nbap_HS_DSCHRequiredPowerValueInformation_For_CellPortion_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_HS_DSCHRequiredPowerValueInformation_For_CellPortion_Item },
};

static int
dissect_nbap_HS_DSCHRequiredPowerValueInformation_For_CellPortion(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_HS_DSCHRequiredPowerValueInformation_For_CellPortion, HS_DSCHRequiredPowerValueInformation_For_CellPortion_sequence_of,
                                                  1, 64);

  return offset;
}
static int dissect_id_HS_DSCHRequiredPowerValueInformation_For_CellPortion(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_DSCHRequiredPowerValueInformation_For_CellPortion(tvb, offset, actx, tree, hf_nbap_id_HS_DSCHRequiredPowerValueInformation_For_CellPortion);
}


static const per_sequence_t HS_DSCHProvidedBitRateValueInformation_For_CellPortion_Item_sequence[] = {
  { &hf_nbap_cellPortionID  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CellPortionID },
  { &hf_nbap_hS_DSCHProvidedBitRateValue1, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_HS_DSCHProvidedBitRate },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_HS_DSCHProvidedBitRateValueInformation_For_CellPortion_Item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_HS_DSCHProvidedBitRateValueInformation_For_CellPortion_Item, HS_DSCHProvidedBitRateValueInformation_For_CellPortion_Item_sequence);

  return offset;
}
static int dissect_HS_DSCHProvidedBitRateValueInformation_For_CellPortion_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_DSCHProvidedBitRateValueInformation_For_CellPortion_Item(tvb, offset, actx, tree, hf_nbap_HS_DSCHProvidedBitRateValueInformation_For_CellPortion_item);
}


static const per_sequence_t HS_DSCHProvidedBitRateValueInformation_For_CellPortion_sequence_of[1] = {
  { &hf_nbap_HS_DSCHProvidedBitRateValueInformation_For_CellPortion_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_HS_DSCHProvidedBitRateValueInformation_For_CellPortion_Item },
};

static int
dissect_nbap_HS_DSCHProvidedBitRateValueInformation_For_CellPortion(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_HS_DSCHProvidedBitRateValueInformation_For_CellPortion, HS_DSCHProvidedBitRateValueInformation_For_CellPortion_sequence_of,
                                                  1, 64);

  return offset;
}
static int dissect_id_HS_DSCHProvidedBitRateValueInformation_For_CellPortion(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HS_DSCHProvidedBitRateValueInformation_For_CellPortion(tvb, offset, actx, tree, hf_nbap_id_HS_DSCHProvidedBitRateValueInformation_For_CellPortion);
}


static const per_sequence_t E_AGCH_FDD_Code_List_sequence_of[1] = {
  { &hf_nbap_E_AGCH_FDD_Code_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_FDD_DL_ChannelisationCodeNumber },
};

static int
dissect_nbap_E_AGCH_FDD_Code_List(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_E_AGCH_FDD_Code_List, E_AGCH_FDD_Code_List_sequence_of,
                                                  1, 32);

  return offset;
}
static int dissect_replace(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_AGCH_FDD_Code_List(tvb, offset, actx, tree, hf_nbap_replace);
}


static const value_string nbap_E_AGCH_FDD_Code_Information_vals[] = {
  {   0, "replace" },
  {   1, "remove" },
  { 0, NULL }
};

static const per_choice_t E_AGCH_FDD_Code_Information_choice[] = {
  {   0, &hf_nbap_replace        , ASN1_EXTENSION_ROOT    , dissect_nbap_E_AGCH_FDD_Code_List },
  {   1, &hf_nbap_remove         , ASN1_EXTENSION_ROOT    , dissect_nbap_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_E_AGCH_FDD_Code_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_E_AGCH_FDD_Code_Information, E_AGCH_FDD_Code_Information_choice,
                                 NULL);

  return offset;
}
static int dissect_id_E_AGCH_FDD_Code_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_AGCH_FDD_Code_Information(tvb, offset, actx, tree, hf_nbap_id_E_AGCH_FDD_Code_Information);
}


static const value_string nbap_E_DCH_Capability_vals[] = {
  {   0, "e-DCH-capable" },
  {   1, "e-DCH-non-capable" },
  { 0, NULL }
};


static int
dissect_nbap_E_DCH_Capability(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_E_DCH_Capability(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DCH_Capability(tvb, offset, actx, tree, hf_nbap_id_E_DCH_Capability);
}



static int
dissect_nbap_E_RNTI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 65535U, NULL, FALSE);

  return offset;
}
static int dissect_primary_e_RNTI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_RNTI(tvb, offset, actx, tree, hf_nbap_primary_e_RNTI);
}
static int dissect_secondary_e_RNTI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_RNTI(tvb, offset, actx, tree, hf_nbap_secondary_e_RNTI);
}



static int
dissect_nbap_E_RGCH_Signature_Sequence(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 39U, NULL, FALSE);

  return offset;
}
static int dissect_e_RGCH_Signature_Sequence(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_RGCH_Signature_Sequence(tvb, offset, actx, tree, hf_nbap_e_RGCH_Signature_Sequence);
}



static int
dissect_nbap_E_HICH_Signature_Sequence(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 39U, NULL, FALSE);

  return offset;
}
static int dissect_e_HICH_Signature_Sequence(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_HICH_Signature_Sequence(tvb, offset, actx, tree, hf_nbap_e_HICH_Signature_Sequence);
}



static int
dissect_nbap_E_Serving_Grant_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 31U, NULL, TRUE);

  return offset;
}
static int dissect_serving_Grant_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_Serving_Grant_Value(tvb, offset, actx, tree, hf_nbap_serving_Grant_Value);
}


static const value_string nbap_E_Primary_Secondary_Grant_Selector_vals[] = {
  {   0, "primary" },
  {   1, "secondary" },
  { 0, NULL }
};


static int
dissect_nbap_E_Primary_Secondary_Grant_Selector(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_primary_Secondary_Grant_Selector(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_Primary_Secondary_Grant_Selector(tvb, offset, actx, tree, hf_nbap_primary_Secondary_Grant_Selector);
}


static const value_string nbap_E_RGCH_Release_Indicator_vals[] = {
  {   0, "e-RGCHreleased" },
  { 0, NULL }
};


static int
dissect_nbap_E_RGCH_Release_Indicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_e_RGCH_Release_Indicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_RGCH_Release_Indicator(tvb, offset, actx, tree, hf_nbap_e_RGCH_Release_Indicator);
}


static const per_sequence_t E_DCH_FDD_DL_Control_Channel_Information_sequence[] = {
  { &hf_nbap_e_AGCH_And_E_RGCH_E_HICH_FDD_Scrambling_Code, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_ScramblingCode },
  { &hf_nbap_e_AGCH_Channelisation_Code, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_FDD_DL_ChannelisationCodeNumber },
  { &hf_nbap_primary_e_RNTI , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_E_RNTI },
  { &hf_nbap_secondary_e_RNTI, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_E_RNTI },
  { &hf_nbap_e_RGCH_E_HICH_Channelisation_Code, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_FDD_DL_ChannelisationCodeNumber },
  { &hf_nbap_e_RGCH_Signature_Sequence, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_E_RGCH_Signature_Sequence },
  { &hf_nbap_e_HICH_Signature_Sequence, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_E_HICH_Signature_Sequence },
  { &hf_nbap_serving_Grant_Value, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_E_Serving_Grant_Value },
  { &hf_nbap_primary_Secondary_Grant_Selector, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_E_Primary_Secondary_Grant_Selector },
  { &hf_nbap_e_RGCH_Release_Indicator, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_E_RGCH_Release_Indicator },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_E_DCH_FDD_DL_Control_Channel_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_E_DCH_FDD_DL_Control_Channel_Information, E_DCH_FDD_DL_Control_Channel_Information_sequence);

  return offset;
}
static int dissect_id_E_DCH_FDD_DL_Control_Channel_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DCH_FDD_DL_Control_Channel_Information(tvb, offset, actx, tree, hf_nbap_id_E_DCH_FDD_DL_Control_Channel_Information);
}



static int
dissect_nbap_E_DCH_MACdFlow_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 7U, NULL, FALSE);

  return offset;
}
static int dissect_e_DCH_MACdFlow_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DCH_MACdFlow_ID(tvb, offset, actx, tree, hf_nbap_e_DCH_MACdFlow_ID);
}



static int
dissect_nbap_Maximum_Number_of_Retransmissions_For_E_DCH(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 15U, NULL, FALSE);

  return offset;
}
static int dissect_maximum_Number_of_Retransmissions_For_E_DCH(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Maximum_Number_of_Retransmissions_For_E_DCH(tvb, offset, actx, tree, hf_nbap_maximum_Number_of_Retransmissions_For_E_DCH);
}



static int
dissect_nbap_E_DCH_HARQ_PO_FDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 6U, NULL, FALSE);

  return offset;
}
static int dissect_eDCH_HARQ_PO_FDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DCH_HARQ_PO_FDD(tvb, offset, actx, tree, hf_nbap_eDCH_HARQ_PO_FDD);
}



static int
dissect_nbap_E_DCH_MACdFlow_Multiplexing_List(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     8, 8, FALSE, NULL);

  return offset;
}
static int dissect_eDCH_MACdFlow_Multiplexing_List(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DCH_MACdFlow_Multiplexing_List(tvb, offset, actx, tree, hf_nbap_eDCH_MACdFlow_Multiplexing_List);
}



static int
dissect_nbap_Max_Bits_MACe_PDU_non_scheduled(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 19982U, NULL, FALSE);

  return offset;
}
static int dissect_maxBits_MACe_PDU_non_scheduled(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Max_Bits_MACe_PDU_non_scheduled(tvb, offset, actx, tree, hf_nbap_maxBits_MACe_PDU_non_scheduled);
}



static int
dissect_nbap_HARQ_Process_Allocation_2ms_EDCH(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     8, 8, FALSE, NULL);

  return offset;
}
static int dissect_hARQ_Process_Allocation_2ms(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HARQ_Process_Allocation_2ms_EDCH(tvb, offset, actx, tree, hf_nbap_hARQ_Process_Allocation_2ms);
}


static const per_sequence_t E_DCH_Non_Scheduled_Transmission_Grant_Items_sequence[] = {
  { &hf_nbap_maxBits_MACe_PDU_non_scheduled, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Max_Bits_MACe_PDU_non_scheduled },
  { &hf_nbap_hARQ_Process_Allocation_2ms, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_HARQ_Process_Allocation_2ms_EDCH },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_E_DCH_Non_Scheduled_Transmission_Grant_Items(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_E_DCH_Non_Scheduled_Transmission_Grant_Items, E_DCH_Non_Scheduled_Transmission_Grant_Items_sequence);

  return offset;
}
static int dissect_e_DCH_Non_Scheduled_Transmission_Grant(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DCH_Non_Scheduled_Transmission_Grant_Items(tvb, offset, actx, tree, hf_nbap_e_DCH_Non_Scheduled_Transmission_Grant);
}


static const value_string nbap_E_DCH_Grant_Type_Information_vals[] = {
  {   0, "e-DCH-Non-Scheduled-Transmission-Grant" },
  {   1, "e-DCH-Scheduled-Transmission-Grant" },
  { 0, NULL }
};

static const per_choice_t E_DCH_Grant_Type_Information_choice[] = {
  {   0, &hf_nbap_e_DCH_Non_Scheduled_Transmission_Grant, ASN1_EXTENSION_ROOT    , dissect_nbap_E_DCH_Non_Scheduled_Transmission_Grant_Items },
  {   1, &hf_nbap_e_DCH_Scheduled_Transmission_Grant, ASN1_EXTENSION_ROOT    , dissect_nbap_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_E_DCH_Grant_Type_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_E_DCH_Grant_Type_Information, E_DCH_Grant_Type_Information_choice,
                                 NULL);

  return offset;
}
static int dissect_eDCH_Grant_Type_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DCH_Grant_Type_Information(tvb, offset, actx, tree, hf_nbap_eDCH_Grant_Type_Information);
}


static const value_string nbap_BundlingModeIndicator_vals[] = {
  {   0, "bundling" },
  {   1, "no-bundling" },
  { 0, NULL }
};


static int
dissect_nbap_BundlingModeIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_bundlingModeIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_BundlingModeIndicator(tvb, offset, actx, tree, hf_nbap_bundlingModeIndicator);
}



static int
dissect_nbap_LogicalChannelID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 15U, NULL, FALSE);

  return offset;
}
static int dissect_logicalChannelId(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_LogicalChannelID(tvb, offset, actx, tree, hf_nbap_logicalChannelId);
}



static int
dissect_nbap_MACesGuaranteedBitRate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 16777215U, NULL, TRUE);

  return offset;
}
static int dissect_mACesGuaranteedBitRate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MACesGuaranteedBitRate(tvb, offset, actx, tree, hf_nbap_mACesGuaranteedBitRate);
}



static int
dissect_nbap_E_DCH_DDI_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 63U, NULL, FALSE);

  return offset;
}
static int dissect_e_DCH_DDI_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DCH_DDI_Value(tvb, offset, actx, tree, hf_nbap_e_DCH_DDI_Value);
}


static const per_sequence_t E_DCH_MACdPDU_SizeListItem_sequence[] = {
  { &hf_nbap_mACdPDU_Size   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MACdPDU_Size },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_E_DCH_MACdPDU_SizeListItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_E_DCH_MACdPDU_SizeListItem, E_DCH_MACdPDU_SizeListItem_sequence);

  return offset;
}
static int dissect_E_DCH_MACdPDU_SizeList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DCH_MACdPDU_SizeListItem(tvb, offset, actx, tree, hf_nbap_E_DCH_MACdPDU_SizeList_item);
}
static int dissect_E_DCH_MACdPDU_SizeToModifyList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DCH_MACdPDU_SizeListItem(tvb, offset, actx, tree, hf_nbap_E_DCH_MACdPDU_SizeToModifyList_item);
}


static const per_sequence_t E_DCH_MACdPDU_SizeList_sequence_of[1] = {
  { &hf_nbap_E_DCH_MACdPDU_SizeList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_E_DCH_MACdPDU_SizeListItem },
};

static int
dissect_nbap_E_DCH_MACdPDU_SizeList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_E_DCH_MACdPDU_SizeList, E_DCH_MACdPDU_SizeList_sequence_of,
                                                  1, 64);

  return offset;
}
static int dissect_mACd_PDU_Size_List(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DCH_MACdPDU_SizeList(tvb, offset, actx, tree, hf_nbap_mACd_PDU_Size_List);
}


static const per_sequence_t E_DCH_LogicalChannelInformationItem_sequence[] = {
  { &hf_nbap_logicalChannelId, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_LogicalChannelID },
  { &hf_nbap_schedulingPriorityIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SchedulingPriorityIndicator },
  { &hf_nbap_mACesGuaranteedBitRate, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_MACesGuaranteedBitRate },
  { &hf_nbap_e_DCH_DDI_Value, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_E_DCH_DDI_Value },
  { &hf_nbap_mACd_PDU_Size_List, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_E_DCH_MACdPDU_SizeList },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_E_DCH_LogicalChannelInformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_E_DCH_LogicalChannelInformationItem, E_DCH_LogicalChannelInformationItem_sequence);

  return offset;
}
static int dissect_E_DCH_LogicalChannelInformation_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DCH_LogicalChannelInformationItem(tvb, offset, actx, tree, hf_nbap_E_DCH_LogicalChannelInformation_item);
}


static const per_sequence_t E_DCH_LogicalChannelInformation_sequence_of[1] = {
  { &hf_nbap_E_DCH_LogicalChannelInformation_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_E_DCH_LogicalChannelInformationItem },
};

static int
dissect_nbap_E_DCH_LogicalChannelInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_E_DCH_LogicalChannelInformation, E_DCH_LogicalChannelInformation_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_eDCHLogicalChannelInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DCH_LogicalChannelInformation(tvb, offset, actx, tree, hf_nbap_eDCHLogicalChannelInformation);
}
static int dissect_eDCH_LogicalChannelToAdd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DCH_LogicalChannelInformation(tvb, offset, actx, tree, hf_nbap_eDCH_LogicalChannelToAdd);
}


static const per_sequence_t E_DCH_MACdFlow_Specific_InfoItem_sequence[] = {
  { &hf_nbap_e_DCH_MACdFlow_ID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_E_DCH_MACdFlow_ID },
  { &hf_nbap_bindingID      , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_BindingID },
  { &hf_nbap_transportLayerAddress, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TransportLayerAddress },
  { &hf_nbap_allocationRetentionPriority, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_AllocationRetentionPriority },
  { &hf_nbap_tnlQos         , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TnlQos },
  { &hf_nbap_payloadCRC_PresenceIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PayloadCRC_PresenceIndicator },
  { &hf_nbap_maximum_Number_of_Retransmissions_For_E_DCH, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Maximum_Number_of_Retransmissions_For_E_DCH },
  { &hf_nbap_eDCH_HARQ_PO_FDD, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_E_DCH_HARQ_PO_FDD },
  { &hf_nbap_eDCH_MACdFlow_Multiplexing_List, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_E_DCH_MACdFlow_Multiplexing_List },
  { &hf_nbap_eDCH_Grant_Type_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_E_DCH_Grant_Type_Information },
  { &hf_nbap_bundlingModeIndicator, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_BundlingModeIndicator },
  { &hf_nbap_eDCHLogicalChannelInformation, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_E_DCH_LogicalChannelInformation },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_E_DCH_MACdFlow_Specific_InfoItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_E_DCH_MACdFlow_Specific_InfoItem, E_DCH_MACdFlow_Specific_InfoItem_sequence);

  return offset;
}
static int dissect_E_DCH_MACdFlow_Specific_InfoList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DCH_MACdFlow_Specific_InfoItem(tvb, offset, actx, tree, hf_nbap_E_DCH_MACdFlow_Specific_InfoList_item);
}


static const per_sequence_t E_DCH_MACdFlow_Specific_InfoList_sequence_of[1] = {
  { &hf_nbap_E_DCH_MACdFlow_Specific_InfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_E_DCH_MACdFlow_Specific_InfoItem },
};

static int
dissect_nbap_E_DCH_MACdFlow_Specific_InfoList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_E_DCH_MACdFlow_Specific_InfoList, E_DCH_MACdFlow_Specific_InfoList_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_e_DCH_MACdFlow_Specific_Info(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DCH_MACdFlow_Specific_InfoList(tvb, offset, actx, tree, hf_nbap_e_DCH_MACdFlow_Specific_Info);
}


static const per_sequence_t E_DCH_MACdFlows_Information_sequence[] = {
  { &hf_nbap_e_DCH_MACdFlow_Specific_Info, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_E_DCH_MACdFlow_Specific_InfoList },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_E_DCH_MACdFlows_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_E_DCH_MACdFlows_Information, E_DCH_MACdFlows_Information_sequence);

  return offset;
}
static int dissect_id_E_DCH_MACdFlows_to_Add(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DCH_MACdFlows_Information(tvb, offset, actx, tree, hf_nbap_id_E_DCH_MACdFlows_to_Add);
}
static int dissect_e_DCH_MACdFlows_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DCH_MACdFlows_Information(tvb, offset, actx, tree, hf_nbap_e_DCH_MACdFlows_Information);
}


static const per_sequence_t E_DCH_FDD_Information_sequence[] = {
  { &hf_nbap_e_DCH_MACdFlows_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_E_DCH_MACdFlows_Information },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_E_DCH_FDD_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_E_DCH_FDD_Information, E_DCH_FDD_Information_sequence);

  return offset;
}
static int dissect_id_E_DCH_FDD_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DCH_FDD_Information(tvb, offset, actx, tree, hf_nbap_id_E_DCH_FDD_Information);
}


static const per_sequence_t E_DCH_MACdFlow_Specific_InformationResp_Item_sequence[] = {
  { &hf_nbap_e_DCH_MACdFlow_ID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_E_DCH_MACdFlow_ID },
  { &hf_nbap_bindingID      , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_BindingID },
  { &hf_nbap_transportLayerAddress, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TransportLayerAddress },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_E_DCH_MACdFlow_Specific_InformationResp_Item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_E_DCH_MACdFlow_Specific_InformationResp_Item, E_DCH_MACdFlow_Specific_InformationResp_Item_sequence);

  return offset;
}
static int dissect_E_DCH_MACdFlow_Specific_InformationResp_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DCH_MACdFlow_Specific_InformationResp_Item(tvb, offset, actx, tree, hf_nbap_E_DCH_MACdFlow_Specific_InformationResp_item);
}


static const per_sequence_t E_DCH_MACdFlow_Specific_InformationResp_sequence_of[1] = {
  { &hf_nbap_E_DCH_MACdFlow_Specific_InformationResp_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_E_DCH_MACdFlow_Specific_InformationResp_Item },
};

static int
dissect_nbap_E_DCH_MACdFlow_Specific_InformationResp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_E_DCH_MACdFlow_Specific_InformationResp, E_DCH_MACdFlow_Specific_InformationResp_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_e_DCH_MACdFlow_Specific_InformationResp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DCH_MACdFlow_Specific_InformationResp(tvb, offset, actx, tree, hf_nbap_e_DCH_MACdFlow_Specific_InformationResp);
}


static const per_sequence_t E_DCH_FDD_Information_Response_sequence[] = {
  { &hf_nbap_e_DCH_MACdFlow_Specific_InformationResp, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_E_DCH_MACdFlow_Specific_InformationResp },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_E_DCH_FDD_Information_Response(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_E_DCH_FDD_Information_Response, E_DCH_FDD_Information_Response_sequence);

  return offset;
}
static int dissect_id_E_DCH_FDD_Information_Response(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DCH_FDD_Information_Response(tvb, offset, actx, tree, hf_nbap_id_E_DCH_FDD_Information_Response);
}


static const per_sequence_t E_DCH_MACdPDU_SizeToModifyList_sequence_of[1] = {
  { &hf_nbap_E_DCH_MACdPDU_SizeToModifyList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_E_DCH_MACdPDU_SizeListItem },
};

static int
dissect_nbap_E_DCH_MACdPDU_SizeToModifyList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_E_DCH_MACdPDU_SizeToModifyList, E_DCH_MACdPDU_SizeToModifyList_sequence_of,
                                                  0, 64);

  return offset;
}
static int dissect_mACd_PDU_Size_List1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DCH_MACdPDU_SizeToModifyList(tvb, offset, actx, tree, hf_nbap_mACd_PDU_Size_List1);
}


static const per_sequence_t E_DCH_LogicalChannelToModifyItem_sequence[] = {
  { &hf_nbap_logicalChannelId, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_LogicalChannelID },
  { &hf_nbap_schedulingPriorityIndicator, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_SchedulingPriorityIndicator },
  { &hf_nbap_mACesGuaranteedBitRate, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_MACesGuaranteedBitRate },
  { &hf_nbap_e_DCH_DDI_Value, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_E_DCH_DDI_Value },
  { &hf_nbap_mACd_PDU_Size_List1, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_E_DCH_MACdPDU_SizeToModifyList },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_E_DCH_LogicalChannelToModifyItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_E_DCH_LogicalChannelToModifyItem, E_DCH_LogicalChannelToModifyItem_sequence);

  return offset;
}
static int dissect_E_DCH_LogicalChannelToModify_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DCH_LogicalChannelToModifyItem(tvb, offset, actx, tree, hf_nbap_E_DCH_LogicalChannelToModify_item);
}


static const per_sequence_t E_DCH_LogicalChannelToModify_sequence_of[1] = {
  { &hf_nbap_E_DCH_LogicalChannelToModify_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_E_DCH_LogicalChannelToModifyItem },
};

static int
dissect_nbap_E_DCH_LogicalChannelToModify(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_E_DCH_LogicalChannelToModify, E_DCH_LogicalChannelToModify_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_eDCH_LogicalChannelToModify(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DCH_LogicalChannelToModify(tvb, offset, actx, tree, hf_nbap_eDCH_LogicalChannelToModify);
}


static const per_sequence_t E_DCH_LogicalChannelToDeleteItem_sequence[] = {
  { &hf_nbap_logicalChannelId, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_LogicalChannelID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_E_DCH_LogicalChannelToDeleteItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_E_DCH_LogicalChannelToDeleteItem, E_DCH_LogicalChannelToDeleteItem_sequence);

  return offset;
}
static int dissect_E_DCH_LogicalChannelToDelete_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DCH_LogicalChannelToDeleteItem(tvb, offset, actx, tree, hf_nbap_E_DCH_LogicalChannelToDelete_item);
}


static const per_sequence_t E_DCH_LogicalChannelToDelete_sequence_of[1] = {
  { &hf_nbap_E_DCH_LogicalChannelToDelete_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_E_DCH_LogicalChannelToDeleteItem },
};

static int
dissect_nbap_E_DCH_LogicalChannelToDelete(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_E_DCH_LogicalChannelToDelete, E_DCH_LogicalChannelToDelete_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_eDCH_LogicalChannelToDelete(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DCH_LogicalChannelToDelete(tvb, offset, actx, tree, hf_nbap_eDCH_LogicalChannelToDelete);
}


static const per_sequence_t E_DCH_MACdFlow_Specific_InfoItem_to_Modify_sequence[] = {
  { &hf_nbap_e_DCH_MACdFlow_ID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_E_DCH_MACdFlow_ID },
  { &hf_nbap_allocationRetentionPriority, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_AllocationRetentionPriority },
  { &hf_nbap_transportBearerRequestIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_TransportBearerRequestIndicator },
  { &hf_nbap_bindingID      , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_BindingID },
  { &hf_nbap_transportLayerAddress, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TransportLayerAddress },
  { &hf_nbap_tnlQos         , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TnlQos },
  { &hf_nbap_maximum_Number_of_Retransmissions_For_E_DCH, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_Maximum_Number_of_Retransmissions_For_E_DCH },
  { &hf_nbap_eDCH_HARQ_PO_FDD, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_E_DCH_HARQ_PO_FDD },
  { &hf_nbap_eDCH_MACdFlow_Multiplexing_List, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_E_DCH_MACdFlow_Multiplexing_List },
  { &hf_nbap_eDCH_Grant_Type_Information, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_E_DCH_Grant_Type_Information },
  { &hf_nbap_eDCH_LogicalChannelToAdd, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_E_DCH_LogicalChannelInformation },
  { &hf_nbap_eDCH_LogicalChannelToModify, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_E_DCH_LogicalChannelToModify },
  { &hf_nbap_eDCH_LogicalChannelToDelete, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_E_DCH_LogicalChannelToDelete },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_E_DCH_MACdFlow_Specific_InfoItem_to_Modify(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_E_DCH_MACdFlow_Specific_InfoItem_to_Modify, E_DCH_MACdFlow_Specific_InfoItem_to_Modify_sequence);

  return offset;
}
static int dissect_E_DCH_MACdFlow_Specific_InfoList_to_Modify_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DCH_MACdFlow_Specific_InfoItem_to_Modify(tvb, offset, actx, tree, hf_nbap_E_DCH_MACdFlow_Specific_InfoList_to_Modify_item);
}


static const per_sequence_t E_DCH_MACdFlow_Specific_InfoList_to_Modify_sequence_of[1] = {
  { &hf_nbap_E_DCH_MACdFlow_Specific_InfoList_to_Modify_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_E_DCH_MACdFlow_Specific_InfoItem_to_Modify },
};

static int
dissect_nbap_E_DCH_MACdFlow_Specific_InfoList_to_Modify(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_E_DCH_MACdFlow_Specific_InfoList_to_Modify, E_DCH_MACdFlow_Specific_InfoList_to_Modify_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_e_DCH_MACdFlow_Specific_Info_to_Modify(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DCH_MACdFlow_Specific_InfoList_to_Modify(tvb, offset, actx, tree, hf_nbap_e_DCH_MACdFlow_Specific_Info_to_Modify);
}


static const per_sequence_t E_DCH_FDD_Information_to_Modify_sequence[] = {
  { &hf_nbap_e_DCH_MACdFlow_Specific_Info_to_Modify, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_E_DCH_MACdFlow_Specific_InfoList_to_Modify },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_E_DCH_FDD_Information_to_Modify(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_E_DCH_FDD_Information_to_Modify, E_DCH_FDD_Information_to_Modify_sequence);

  return offset;
}
static int dissect_id_E_DCH_FDD_Information_to_Modify(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DCH_FDD_Information_to_Modify(tvb, offset, actx, tree, hf_nbap_id_E_DCH_FDD_Information_to_Modify);
}


static const per_sequence_t E_DCH_MACdFlow_to_Delete_Item_sequence[] = {
  { &hf_nbap_e_DCH_MACdFlow_ID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_E_DCH_MACdFlow_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_E_DCH_MACdFlow_to_Delete_Item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_E_DCH_MACdFlow_to_Delete_Item, E_DCH_MACdFlow_to_Delete_Item_sequence);

  return offset;
}
static int dissect_E_DCH_MACdFlows_to_Delete_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DCH_MACdFlow_to_Delete_Item(tvb, offset, actx, tree, hf_nbap_E_DCH_MACdFlows_to_Delete_item);
}


static const per_sequence_t E_DCH_MACdFlows_to_Delete_sequence_of[1] = {
  { &hf_nbap_E_DCH_MACdFlows_to_Delete_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_E_DCH_MACdFlow_to_Delete_Item },
};

static int
dissect_nbap_E_DCH_MACdFlows_to_Delete(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_E_DCH_MACdFlows_to_Delete, E_DCH_MACdFlows_to_Delete_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_id_E_DCH_MACdFlows_to_Delete(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DCH_MACdFlows_to_Delete(tvb, offset, actx, tree, hf_nbap_id_E_DCH_MACdFlows_to_Delete);
}


static const per_sequence_t E_DCH_Resources_Information_AuditRsp_sequence[] = {
  { &hf_nbap_resourceOperationalState, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ResourceOperationalState },
  { &hf_nbap_availabilityStatus, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_AvailabilityStatus },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_E_DCH_Resources_Information_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_E_DCH_Resources_Information_AuditRsp, E_DCH_Resources_Information_AuditRsp_sequence);

  return offset;
}
static int dissect_id_E_DCH_Resources_Information_AuditRsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DCH_Resources_Information_AuditRsp(tvb, offset, actx, tree, hf_nbap_id_E_DCH_Resources_Information_AuditRsp);
}


static const per_sequence_t E_DCH_Resources_Information_ResourceStatusInd_sequence[] = {
  { &hf_nbap_resourceOperationalState, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ResourceOperationalState },
  { &hf_nbap_availabilityStatus, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_AvailabilityStatus },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_E_DCH_Resources_Information_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_E_DCH_Resources_Information_ResourceStatusInd, E_DCH_Resources_Information_ResourceStatusInd_sequence);

  return offset;
}
static int dissect_id_E_DCH_Resources_Information_ResourceStatusInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DCH_Resources_Information_ResourceStatusInd(tvb, offset, actx, tree, hf_nbap_id_E_DCH_Resources_Information_ResourceStatusInd);
}


static const value_string nbap_E_DCH_RL_Indication_vals[] = {
  {   0, "e-DCH" },
  {   1, "non-e-DCH" },
  { 0, NULL }
};


static int
dissect_nbap_E_DCH_RL_Indication(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_E_DCH_RL_Indication(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DCH_RL_Indication(tvb, offset, actx, tree, hf_nbap_id_E_DCH_RL_Indication);
}


static const value_string nbap_Max_Set_E_DPDCHs_vals[] = {
  {   0, "vN64" },
  {   1, "vN32" },
  {   2, "vN16" },
  {   3, "vN8" },
  {   4, "vN4" },
  {   5, "v2xN4" },
  {   6, "v2xN2" },
  {   7, "v2xN2plus2xN4" },
  { 0, NULL }
};


static int
dissect_nbap_Max_Set_E_DPDCHs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_maxSet_E_DPDCHs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Max_Set_E_DPDCHs(tvb, offset, actx, tree, hf_nbap_maxSet_E_DPDCHs);
}



static int
dissect_nbap_E_DCH_TFCI_Table_Index(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 1U, NULL, TRUE);

  return offset;
}
static int dissect_e_DCH_TFCI_Table_Index(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DCH_TFCI_Table_Index(tvb, offset, actx, tree, hf_nbap_e_DCH_TFCI_Table_Index);
}



static int
dissect_nbap_E_TFCI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 127U, NULL, FALSE);

  return offset;
}
static int dissect_e_DCH_Min_Set_E_TFCI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_TFCI(tvb, offset, actx, tree, hf_nbap_e_DCH_Min_Set_E_TFCI);
}
static int dissect_reference_E_TFCI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_TFCI(tvb, offset, actx, tree, hf_nbap_reference_E_TFCI);
}



static int
dissect_nbap_Reference_E_TFCI_PO(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 29U, NULL, FALSE);

  return offset;
}
static int dissect_reference_E_TFCI_PO(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Reference_E_TFCI_PO(tvb, offset, actx, tree, hf_nbap_reference_E_TFCI_PO);
}


static const per_sequence_t Reference_E_TFCI_Information_Item_sequence[] = {
  { &hf_nbap_reference_E_TFCI, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_E_TFCI },
  { &hf_nbap_reference_E_TFCI_PO, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Reference_E_TFCI_PO },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Reference_E_TFCI_Information_Item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Reference_E_TFCI_Information_Item, Reference_E_TFCI_Information_Item_sequence);

  return offset;
}
static int dissect_Reference_E_TFCI_Information_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Reference_E_TFCI_Information_Item(tvb, offset, actx, tree, hf_nbap_Reference_E_TFCI_Information_item);
}


static const per_sequence_t Reference_E_TFCI_Information_sequence_of[1] = {
  { &hf_nbap_Reference_E_TFCI_Information_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_Reference_E_TFCI_Information_Item },
};

static int
dissect_nbap_Reference_E_TFCI_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_Reference_E_TFCI_Information, Reference_E_TFCI_Information_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_reference_E_TFCI_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Reference_E_TFCI_Information(tvb, offset, actx, tree, hf_nbap_reference_E_TFCI_Information);
}


static const per_sequence_t E_TFCS_Information_sequence[] = {
  { &hf_nbap_e_DCH_TFCI_Table_Index, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_E_DCH_TFCI_Table_Index },
  { &hf_nbap_e_DCH_Min_Set_E_TFCI, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_E_TFCI },
  { &hf_nbap_reference_E_TFCI_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Reference_E_TFCI_Information },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_E_TFCS_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_E_TFCS_Information, E_TFCS_Information_sequence);

  return offset;
}
static int dissect_e_TFCS_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_TFCS_Information(tvb, offset, actx, tree, hf_nbap_e_TFCS_Information);
}


static const value_string nbap_E_TTI_vals[] = {
  {   0, "e-TTI-2ms" },
  {   1, "e-TTI-10ms" },
  { 0, NULL }
};


static int
dissect_nbap_E_TTI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_e_TTI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_TTI(tvb, offset, actx, tree, hf_nbap_e_TTI);
}



static int
dissect_nbap_E_DPCCH_PO(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 8U, NULL, FALSE);

  return offset;
}
static int dissect_e_DPCCH_PO(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DPCCH_PO(tvb, offset, actx, tree, hf_nbap_e_DPCCH_PO);
}


static const per_sequence_t E_DPCH_Information_RL_ReconfPrepFDD_sequence[] = {
  { &hf_nbap_maxSet_E_DPDCHs, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_Max_Set_E_DPDCHs },
  { &hf_nbap_ul_PunctureLimit, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_PunctureLimit },
  { &hf_nbap_e_TFCS_Information, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_E_TFCS_Information },
  { &hf_nbap_e_TTI          , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_E_TTI },
  { &hf_nbap_e_DPCCH_PO     , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_E_DPCCH_PO },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_E_DPCH_Information_RL_ReconfPrepFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_E_DPCH_Information_RL_ReconfPrepFDD, E_DPCH_Information_RL_ReconfPrepFDD_sequence);

  return offset;
}
static int dissect_id_E_DPCH_Information_RL_ReconfPrepFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DPCH_Information_RL_ReconfPrepFDD(tvb, offset, actx, tree, hf_nbap_id_E_DPCH_Information_RL_ReconfPrepFDD);
}


static const per_sequence_t E_DPCH_Information_RL_SetupRqstFDD_sequence[] = {
  { &hf_nbap_maxSet_E_DPDCHs, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Max_Set_E_DPDCHs },
  { &hf_nbap_ul_PunctureLimit, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_PunctureLimit },
  { &hf_nbap_e_TFCS_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_E_TFCS_Information },
  { &hf_nbap_e_TTI          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_E_TTI },
  { &hf_nbap_e_DPCCH_PO     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_E_DPCCH_PO },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_E_DPCH_Information_RL_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_E_DPCH_Information_RL_SetupRqstFDD, E_DPCH_Information_RL_SetupRqstFDD_sequence);

  return offset;
}
static int dissect_id_E_DPCH_Information_RL_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DPCH_Information_RL_SetupRqstFDD(tvb, offset, actx, tree, hf_nbap_id_E_DPCH_Information_RL_SetupRqstFDD);
}


static const per_sequence_t E_RGCH_E_HICH_FDD_Code_List_sequence_of[1] = {
  { &hf_nbap_E_RGCH_E_HICH_FDD_Code_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_FDD_DL_ChannelisationCodeNumber },
};

static int
dissect_nbap_E_RGCH_E_HICH_FDD_Code_List(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_E_RGCH_E_HICH_FDD_Code_List, E_RGCH_E_HICH_FDD_Code_List_sequence_of,
                                                  1, 32);

  return offset;
}
static int dissect_replace1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_RGCH_E_HICH_FDD_Code_List(tvb, offset, actx, tree, hf_nbap_replace1);
}


static const value_string nbap_E_RGCH_E_HICH_FDD_Code_Information_vals[] = {
  {   0, "replace" },
  {   1, "remove" },
  { 0, NULL }
};

static const per_choice_t E_RGCH_E_HICH_FDD_Code_Information_choice[] = {
  {   0, &hf_nbap_replace1       , ASN1_EXTENSION_ROOT    , dissect_nbap_E_RGCH_E_HICH_FDD_Code_List },
  {   1, &hf_nbap_remove         , ASN1_EXTENSION_ROOT    , dissect_nbap_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_E_RGCH_E_HICH_FDD_Code_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_E_RGCH_E_HICH_FDD_Code_Information, E_RGCH_E_HICH_FDD_Code_Information_choice,
                                 NULL);

  return offset;
}
static int dissect_id_E_RGCH_E_HICH_FDD_Code_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_RGCH_E_HICH_FDD_Code_Information(tvb, offset, actx, tree, hf_nbap_id_E_RGCH_E_HICH_FDD_Code_Information);
}


static const per_sequence_t Serving_E_DCH_RL_in_this_NodeB_sequence[] = {
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Serving_E_DCH_RL_in_this_NodeB(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Serving_E_DCH_RL_in_this_NodeB, Serving_E_DCH_RL_in_this_NodeB_sequence);

  return offset;
}
static int dissect_serving_E_DCH_RL_in_this_NodeB(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Serving_E_DCH_RL_in_this_NodeB(tvb, offset, actx, tree, hf_nbap_serving_E_DCH_RL_in_this_NodeB);
}


static const value_string nbap_Serving_E_DCH_RL_ID_vals[] = {
  {   0, "serving-E-DCH-RL-in-this-NodeB" },
  {   1, "serving-E-DCH-RL-not-in-this-NodeB" },
  { 0, NULL }
};

static const per_choice_t Serving_E_DCH_RL_ID_choice[] = {
  {   0, &hf_nbap_serving_E_DCH_RL_in_this_NodeB, ASN1_EXTENSION_ROOT    , dissect_nbap_Serving_E_DCH_RL_in_this_NodeB },
  {   1, &hf_nbap_serving_E_DCH_RL_not_in_this_NodeB, ASN1_EXTENSION_ROOT    , dissect_nbap_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_Serving_E_DCH_RL_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_Serving_E_DCH_RL_ID, Serving_E_DCH_RL_ID_choice,
                                 NULL);

  return offset;
}
static int dissect_id_Serving_E_DCH_RL_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Serving_E_DCH_RL_ID(tvb, offset, actx, tree, hf_nbap_id_Serving_E_DCH_RL_ID);
}


static const value_string nbap_UL_DPDCH_Indicator_For_E_DCH_Operation_vals[] = {
  {   0, "ul-DPDCH-present" },
  {   1, "ul-DPDCH-not-present" },
  { 0, NULL }
};


static int
dissect_nbap_UL_DPDCH_Indicator_For_E_DCH_Operation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_UL_DPDCH_Indicator_For_E_DCH_Operation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_DPDCH_Indicator_For_E_DCH_Operation(tvb, offset, actx, tree, hf_nbap_id_UL_DPDCH_Indicator_For_E_DCH_Operation);
}


static const value_string nbap_FDD_S_CCPCH_FrameOffset_vals[] = {
  {   0, "v1" },
  {   1, "v2" },
  {   2, "v4" },
  { 0, NULL }
};


static int
dissect_nbap_FDD_S_CCPCH_FrameOffset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_id_FDD_S_CCPCH_FrameOffset_CTCH_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_FDD_S_CCPCH_FrameOffset(tvb, offset, actx, tree, hf_nbap_id_FDD_S_CCPCH_FrameOffset_CTCH_SetupRqstFDD);
}


static const per_sequence_t E_DPCH_Information_RL_ReconfRqstFDD_sequence[] = {
  { &hf_nbap_e_TFCS_Information, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_E_TFCS_Information },
  { &hf_nbap_e_DPCCH_PO     , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_E_DPCCH_PO },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_E_DPCH_Information_RL_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_E_DPCH_Information_RL_ReconfRqstFDD, E_DPCH_Information_RL_ReconfRqstFDD_sequence);

  return offset;
}
static int dissect_id_E_DPCH_Information_RL_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DPCH_Information_RL_ReconfRqstFDD(tvb, offset, actx, tree, hf_nbap_id_E_DPCH_Information_RL_ReconfRqstFDD);
}



static int
dissect_nbap_Maximum_Target_ReceivedTotalWideBandPower(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 621U, NULL, FALSE);

  return offset;
}
static int dissect_id_Maximum_Target_ReceivedTotalWideBandPower(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Maximum_Target_ReceivedTotalWideBandPower(tvb, offset, actx, tree, hf_nbap_id_Maximum_Target_ReceivedTotalWideBandPower);
}



static int
dissect_nbap_E_DCHProvidedBitRateValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 16777215U, NULL, TRUE);

  return offset;
}
static int dissect_e_DCHProvidedBitRateValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DCHProvidedBitRateValue(tvb, offset, actx, tree, hf_nbap_e_DCHProvidedBitRateValue);
}


static const per_sequence_t E_DCHProvidedBitRate_Item_sequence[] = {
  { &hf_nbap_schedulingPriorityIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SchedulingPriorityIndicator },
  { &hf_nbap_e_DCHProvidedBitRateValue, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_E_DCHProvidedBitRateValue },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_E_DCHProvidedBitRate_Item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_E_DCHProvidedBitRate_Item, E_DCHProvidedBitRate_Item_sequence);

  return offset;
}
static int dissect_E_DCHProvidedBitRate_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DCHProvidedBitRate_Item(tvb, offset, actx, tree, hf_nbap_E_DCHProvidedBitRate_item);
}


static const per_sequence_t E_DCHProvidedBitRate_sequence_of[1] = {
  { &hf_nbap_E_DCHProvidedBitRate_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_E_DCHProvidedBitRate_Item },
};

static int
dissect_nbap_E_DCHProvidedBitRate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_E_DCHProvidedBitRate, E_DCHProvidedBitRate_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_E_DCHProvidedBitRateValueInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DCHProvidedBitRate(tvb, offset, actx, tree, hf_nbap_id_E_DCHProvidedBitRateValueInformation);
}


static const value_string nbap_HARQ_Preamble_Mode_Activation_Indicator_vals[] = {
  {   0, "harqPreambleModeActivated" },
  { 0, NULL }
};


static int
dissect_nbap_HARQ_Preamble_Mode_Activation_Indicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_HARQ_Preamble_Mode_Activation_Indicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_HARQ_Preamble_Mode_Activation_Indicator(tvb, offset, actx, tree, hf_nbap_id_HARQ_Preamble_Mode_Activation_Indicator);
}


static const per_sequence_t RL_Specific_E_DCH_Info_Item_sequence[] = {
  { &hf_nbap_e_DCH_MACdFlow_ID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_E_DCH_MACdFlow_ID },
  { &hf_nbap_bindingID      , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_BindingID },
  { &hf_nbap_transportlayeraddress, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TransportLayerAddress },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RL_Specific_E_DCH_Info_Item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RL_Specific_E_DCH_Info_Item, RL_Specific_E_DCH_Info_Item_sequence);

  return offset;
}
static int dissect_RL_Specific_E_DCH_Info_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_Specific_E_DCH_Info_Item(tvb, offset, actx, tree, hf_nbap_RL_Specific_E_DCH_Info_item);
}


static const per_sequence_t RL_Specific_E_DCH_Info_sequence_of[1] = {
  { &hf_nbap_RL_Specific_E_DCH_Info_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_RL_Specific_E_DCH_Info_Item },
};

static int
dissect_nbap_RL_Specific_E_DCH_Info(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_RL_Specific_E_DCH_Info, RL_Specific_E_DCH_Info_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_id_RL_Specific_E_DCH_Info(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_RL_Specific_E_DCH_Info(tvb, offset, actx, tree, hf_nbap_id_RL_Specific_E_DCH_Info);
}


static const per_sequence_t Multiple_DedicatedMeasurementValueItem_TDD_DM_Rsp_sequence[] = {
  { &hf_nbap_dPCH_ID        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DPCH_ID },
  { &hf_nbap_dedicatedMeasurementValue, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DedicatedMeasurementValue },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Multiple_DedicatedMeasurementValueItem_TDD_DM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Multiple_DedicatedMeasurementValueItem_TDD_DM_Rsp, Multiple_DedicatedMeasurementValueItem_TDD_DM_Rsp_sequence);

  return offset;
}
static int dissect_Multiple_DedicatedMeasurementValueList_TDD_DM_Rsp_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Multiple_DedicatedMeasurementValueItem_TDD_DM_Rsp(tvb, offset, actx, tree, hf_nbap_Multiple_DedicatedMeasurementValueList_TDD_DM_Rsp_item);
}


static const per_sequence_t Multiple_DedicatedMeasurementValueList_TDD_DM_Rsp_sequence_of[1] = {
  { &hf_nbap_Multiple_DedicatedMeasurementValueList_TDD_DM_Rsp_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_Multiple_DedicatedMeasurementValueItem_TDD_DM_Rsp },
};

static int
dissect_nbap_Multiple_DedicatedMeasurementValueList_TDD_DM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_Multiple_DedicatedMeasurementValueList_TDD_DM_Rsp, Multiple_DedicatedMeasurementValueList_TDD_DM_Rsp_sequence_of,
                                                  1, 239);

  return offset;
}
static int dissect_id_multiple_DedicatedMeasurementValueList_TDD_DM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Multiple_DedicatedMeasurementValueList_TDD_DM_Rsp(tvb, offset, actx, tree, hf_nbap_id_multiple_DedicatedMeasurementValueList_TDD_DM_Rsp);
}


static const per_sequence_t Multiple_DedicatedMeasurementValueItem_LCR_TDD_DM_Rsp_sequence[] = {
  { &hf_nbap_dPCH_ID        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DPCH_ID },
  { &hf_nbap_dedicatedMeasurementValue, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_DedicatedMeasurementValue },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Multiple_DedicatedMeasurementValueItem_LCR_TDD_DM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Multiple_DedicatedMeasurementValueItem_LCR_TDD_DM_Rsp, Multiple_DedicatedMeasurementValueItem_LCR_TDD_DM_Rsp_sequence);

  return offset;
}
static int dissect_Multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Multiple_DedicatedMeasurementValueItem_LCR_TDD_DM_Rsp(tvb, offset, actx, tree, hf_nbap_Multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp_item);
}


static const per_sequence_t Multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp_sequence_of[1] = {
  { &hf_nbap_Multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_Multiple_DedicatedMeasurementValueItem_LCR_TDD_DM_Rsp },
};

static int
dissect_nbap_Multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_Multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp, Multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp_sequence_of,
                                                  1, 95);

  return offset;
}
static int dissect_id_multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp(tvb, offset, actx, tree, hf_nbap_id_multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp);
}


static const per_sequence_t E_DCH_RearrangeItem_Bearer_RearrangeInd_sequence[] = {
  { &hf_nbap_e_DCH_MACdFlow_ID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_E_DCH_MACdFlow_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_E_DCH_RearrangeItem_Bearer_RearrangeInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_E_DCH_RearrangeItem_Bearer_RearrangeInd, E_DCH_RearrangeItem_Bearer_RearrangeInd_sequence);

  return offset;
}
static int dissect_E_DCH_RearrangeList_Bearer_RearrangeInd_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DCH_RearrangeItem_Bearer_RearrangeInd(tvb, offset, actx, tree, hf_nbap_E_DCH_RearrangeList_Bearer_RearrangeInd_item);
}


static const per_sequence_t E_DCH_RearrangeList_Bearer_RearrangeInd_sequence_of[1] = {
  { &hf_nbap_E_DCH_RearrangeList_Bearer_RearrangeInd_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_E_DCH_RearrangeItem_Bearer_RearrangeInd },
};

static int
dissect_nbap_E_DCH_RearrangeList_Bearer_RearrangeInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_E_DCH_RearrangeList_Bearer_RearrangeInd, E_DCH_RearrangeList_Bearer_RearrangeInd_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_id_E_DCH_RearrangeList_Bearer_RearrangeInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_E_DCH_RearrangeList_Bearer_RearrangeInd(tvb, offset, actx, tree, hf_nbap_id_E_DCH_RearrangeList_Bearer_RearrangeInd);
}


static const per_sequence_t MultipleRL_DL_CCTrCH_InformationModifyListIE_RL_ReconfRqstTDD_sequence[] = {
  { &hf_nbap_dl_DPCH_LCR_InformationModifyList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_DPCH_LCR_InformationModify_ModifyList_RL_ReconfRqstTDD },
  { &hf_nbap_cCTrCH_Maximum_DL_Power_InformationModify_RL_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_Power },
  { &hf_nbap_cCTrCH_Minimum_DL_Power_InformationModify_RL_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_Power },
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_RL_ID },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_MultipleRL_DL_CCTrCH_InformationModifyListIE_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_MultipleRL_DL_CCTrCH_InformationModifyListIE_RL_ReconfRqstTDD, MultipleRL_DL_CCTrCH_InformationModifyListIE_RL_ReconfRqstTDD_sequence);

  return offset;
}
static int dissect_MultipleRL_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MultipleRL_DL_CCTrCH_InformationModifyListIE_RL_ReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_MultipleRL_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD_item);
}


static const per_sequence_t MultipleRL_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD_sequence_of[1] = {
  { &hf_nbap_MultipleRL_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_MultipleRL_DL_CCTrCH_InformationModifyListIE_RL_ReconfRqstTDD },
};

static int
dissect_nbap_MultipleRL_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_MultipleRL_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD, MultipleRL_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_id_multipleRL_dl_CCTrCH_InformationModifyList_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MultipleRL_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_id_multipleRL_dl_CCTrCH_InformationModifyList_RL_ReconfRqstTDD);
}



static int
dissect_nbap_Target_NonServing_EDCH_To_Total_EDCH_Power_Ratio(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 100U, NULL, FALSE);

  return offset;
}
static int dissect_id_Target_NonServing_EDCH_To_Total_EDCH_Power_Ratio(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Target_NonServing_EDCH_To_Total_EDCH_Power_Ratio(tvb, offset, actx, tree, hf_nbap_id_Target_NonServing_EDCH_To_Total_EDCH_Power_Ratio);
}


static const per_sequence_t CellPortion_InformationItem_Cell_ReconfRqstFDD_sequence[] = {
  { &hf_nbap_cellPortionID  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CellPortionID },
  { &hf_nbap_maximumTransmissionPowerforCellPortion, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_MaximumTransmissionPower },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CellPortion_InformationItem_Cell_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CellPortion_InformationItem_Cell_ReconfRqstFDD, CellPortion_InformationItem_Cell_ReconfRqstFDD_sequence);

  return offset;
}
static int dissect_id_CellPortion_InformationItem_Cell_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellPortion_InformationItem_Cell_ReconfRqstFDD(tvb, offset, actx, tree, hf_nbap_id_CellPortion_InformationItem_Cell_ReconfRqstFDD);
}


static const per_sequence_t CellPortion_InformationList_Cell_ReconfRqstFDD_sequence_of[1] = {
  { &hf_nbap_CellPortion_InformationList_Cell_ReconfRqstFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Single_Container },
};

static int
dissect_nbap_CellPortion_InformationList_Cell_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_CellPortion_InformationList_Cell_ReconfRqstFDD, CellPortion_InformationList_Cell_ReconfRqstFDD_sequence_of,
                                                  1, 64);

  return offset;
}
static int dissect_id_CellPortion_InformationList_Cell_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellPortion_InformationList_Cell_ReconfRqstFDD(tvb, offset, actx, tree, hf_nbap_id_CellPortion_InformationList_Cell_ReconfRqstFDD);
}


static const per_sequence_t Multiple_PUSCH_InfoListIE_DM_Rsp_sequence[] = {
  { &hf_nbap_pUSCH_ID       , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_PUSCH_ID },
  { &hf_nbap_dedicatedMeasurementValue, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DedicatedMeasurementValue },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Multiple_PUSCH_InfoListIE_DM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Multiple_PUSCH_InfoListIE_DM_Rsp, Multiple_PUSCH_InfoListIE_DM_Rsp_sequence);

  return offset;
}
static int dissect_Multiple_PUSCH_InfoList_DM_Rsp_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Multiple_PUSCH_InfoListIE_DM_Rsp(tvb, offset, actx, tree, hf_nbap_Multiple_PUSCH_InfoList_DM_Rsp_item);
}


static const per_sequence_t Multiple_PUSCH_InfoList_DM_Rsp_sequence_of[1] = {
  { &hf_nbap_Multiple_PUSCH_InfoList_DM_Rsp_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_Multiple_PUSCH_InfoListIE_DM_Rsp },
};

static int
dissect_nbap_Multiple_PUSCH_InfoList_DM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_Multiple_PUSCH_InfoList_DM_Rsp, Multiple_PUSCH_InfoList_DM_Rsp_sequence_of,
                                                  1, 255);

  return offset;
}
static int dissect_id_multiple_PUSCH_InfoList_DM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Multiple_PUSCH_InfoList_DM_Rsp(tvb, offset, actx, tree, hf_nbap_id_multiple_PUSCH_InfoList_DM_Rsp);
}


static const per_sequence_t Multiple_PUSCH_InfoListIE_DM_Rprt_sequence[] = {
  { &hf_nbap_pUSCH_ID       , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_PUSCH_ID },
  { &hf_nbap_dedicatedMeasurementValue, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DedicatedMeasurementValue },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_Multiple_PUSCH_InfoListIE_DM_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_Multiple_PUSCH_InfoListIE_DM_Rprt, Multiple_PUSCH_InfoListIE_DM_Rprt_sequence);

  return offset;
}
static int dissect_Multiple_PUSCH_InfoList_DM_Rprt_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Multiple_PUSCH_InfoListIE_DM_Rprt(tvb, offset, actx, tree, hf_nbap_Multiple_PUSCH_InfoList_DM_Rprt_item);
}


static const per_sequence_t Multiple_PUSCH_InfoList_DM_Rprt_sequence_of[1] = {
  { &hf_nbap_Multiple_PUSCH_InfoList_DM_Rprt_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_Multiple_PUSCH_InfoListIE_DM_Rprt },
};

static int
dissect_nbap_Multiple_PUSCH_InfoList_DM_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_Multiple_PUSCH_InfoList_DM_Rprt, Multiple_PUSCH_InfoList_DM_Rprt_sequence_of,
                                                  1, 255);

  return offset;
}
static int dissect_id_multiple_PUSCH_InfoList_DM_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Multiple_PUSCH_InfoList_DM_Rprt(tvb, offset, actx, tree, hf_nbap_id_multiple_PUSCH_InfoList_DM_Rprt);
}



static int
dissect_nbap_Reference_ReceivedTotalWideBandPower(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 621U, NULL, FALSE);

  return offset;
}
static int dissect_id_Reference_ReceivedTotalWideBandPower(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Reference_ReceivedTotalWideBandPower(tvb, offset, actx, tree, hf_nbap_id_Reference_ReceivedTotalWideBandPower);
}


static const value_string nbap_DymmyProtocolIE_ID_vals[] = {
  {   0, "id-AICH-Information" },
  {   1, "id-BCH-Information" },
  {   2, "id-BCCH-ModificationTime" },
  {   3, "id-BlockingPriorityIndicator" },
  {   4, "id-Cause" },
  {   5, "id-CCP-InformationItem-AuditRsp" },
  {   6, "id-CCP-InformationList-AuditRsp" },
  {   7, "id-CCP-InformationItem-ResourceStatusInd" },
  {   8, "id-Cell-InformationItem-AuditRsp" },
  {   9, "id-Cell-InformationItem-ResourceStatusInd" },
  {  10, "id-Cell-InformationList-AuditRsp" },
  {  11, "id-CellParameterID" },
  {  12, "id-CFN" },
  {  13, "id-C-ID" },
  {  14, "id-CommonMeasurementAccuracy" },
  {  15, "id-CommonMeasurementObjectType-CM-Rprt" },
  {  16, "id-CommonMeasurementObjectType-CM-Rqst" },
  {  17, "id-CommonMeasurementObjectType-CM-Rsp" },
  {  18, "id-CommonMeasurementType" },
  {  19, "id-CommonPhysicalChannelID" },
  {  20, "id-CommonPhysicalChannelType-CTCH-SetupRqstFDD" },
  {  21, "id-CommonPhysicalChannelType-CTCH-SetupRqstTDD" },
  {  22, "id-CommunicationControlPortID" },
  {  23, "id-ConfigurationGenerationID" },
  {  24, "id-CRNC-CommunicationContextID" },
  {  25, "id-CriticalityDiagnostics" },
  {  26, "id-DCHs-to-Add-FDD" },
  {  27, "id-DCHs-to-Add-TDD" },
  {  28, "id-DCH-DeleteList-RL-ReconfPrepFDD" },
  {  29, "id-DCH-DeleteList-RL-ReconfPrepTDD" },
  {  30, "id-DCH-DeleteList-RL-ReconfRqstFDD" },
  {  31, "id-DCH-DeleteList-RL-ReconfRqstTDD" },
  {  32, "id-DCH-FDD-Information" },
  {  33, "id-DCH-TDD-Information" },
  {  34, "id-DCH-InformationResponse" },
  {  35, "id-FDD-DCHs-to-Modify" },
  {  36, "id-TDD-DCHs-to-Modify" },
  {  37, "id-DCH-RearrangeList-Bearer-RearrangeInd" },
  {  38, "id-DedicatedMeasurementObjectType-DM-Rprt" },
  {  39, "id-DedicatedMeasurementObjectType-DM-Rqst" },
  {  40, "id-DedicatedMeasurementObjectType-DM-Rsp" },
  {  41, "id-DedicatedMeasurementType" },
  {  42, "id-DL-CCTrCH-InformationItem-RL-SetupRqstTDD" },
  {  43, "id-DL-CCTrCH-InformationList-RL-AdditionRqstTDD" },
  {  44, "id-DL-CCTrCH-InformationList-RL-SetupRqstTDD" },
  {  45, "id-DL-DPCH-InformationItem-RL-AdditionRqstTDD" },
  {  46, "id-DL-DPCH-InformationList-RL-SetupRqstTDD" },
  {  47, "id-DL-DPCH-Information-RL-ReconfPrepFDD" },
  {  48, "id-DL-DPCH-Information-RL-ReconfRqstFDD" },
  {  49, "id-DL-DPCH-Information-RL-SetupRqstFDD" },
  {  50, "id-DL-DPCH-TimingAdjustment" },
  {  51, "id-DL-ReferencePowerInformationItem-DL-PC-Rqst" },
  {  52, "id-DLReferencePower" },
  {  53, "id-DLReferencePowerList-DL-PC-Rqst" },
  {  54, "id-DSCHs-to-Add-TDD" },
  {  55, "id-DSCH-Information-DeleteList-RL-ReconfPrepTDD" },
  {  56, "id-DSCH-Information-ModifyList-RL-ReconfPrepTDD" },
  {  57, "id-DSCH-InformationResponse" },
  {  58, "id-DSCH-TDD-Information" },
  {  59, "id-DSCH-RearrangeList-Bearer-RearrangeInd" },
  {  60, "id-End-Of-Audit-Sequence-Indicator" },
  {  61, "id-FACH-Information" },
  {  62, "id-FACH-ParametersList-CTCH-ReconfRqstTDD" },
  {  63, "id-FACH-ParametersListIE-CTCH-SetupRqstFDD" },
  {  64, "id-FACH-ParametersListIE-CTCH-SetupRqstTDD" },
  {  65, "id-IndicationType-ResourceStatusInd" },
  {  66, "id-Local-Cell-ID" },
  {  67, "id-Local-Cell-Group-InformationItem-AuditRsp" },
  {  68, "id-Local-Cell-Group-InformationItem-ResourceStatusInd" },
  {  69, "id-Local-Cell-Group-InformationItem2-ResourceStatusInd" },
  {  70, "id-Local-Cell-Group-InformationList-AuditRsp" },
  {  71, "id-Local-Cell-InformationItem-AuditRsp" },
  {  72, "id-Local-Cell-InformationItem-ResourceStatusInd" },
  {  73, "id-Local-Cell-InformationItem2-ResourceStatusInd" },
  {  74, "id-Local-Cell-InformationList-AuditRsp" },
  {  75, "id-AdjustmentPeriod" },
  {  76, "id-MaxAdjustmentStep" },
  {  77, "id-MaximumTransmissionPower" },
  {  78, "id-MeasurementFilterCoefficient" },
  {  79, "id-MeasurementID" },
  {  80, "id-MessageStructure" },
  {  81, "id-MIB-SB-SIB-InformationList-SystemInfoUpdateRqst" },
  {  82, "id-NodeB-CommunicationContextID" },
  {  83, "id-NeighbouringCellMeasurementInformation" },
  {  84, "id-P-CCPCH-Information" },
  {  85, "id-P-CPICH-Information" },
  {  86, "id-P-SCH-Information" },
  {  87, "id-PCCPCH-Information-Cell-ReconfRqstTDD" },
  {  88, "id-PCCPCH-Information-Cell-SetupRqstTDD" },
  {  89, "id-PCH-Parameters-CTCH-ReconfRqstTDD" },
  {  90, "id-PCH-ParametersItem-CTCH-SetupRqstFDD" },
  {  91, "id-PCH-ParametersItem-CTCH-SetupRqstTDD" },
  {  92, "id-PCH-Information" },
  {  93, "id-PDSCH-Information-AddListIE-PSCH-ReconfRqst" },
  {  94, "id-PDSCH-Information-ModifyListIE-PSCH-ReconfRqst" },
  {  95, "id-PDSCHSets-AddList-PSCH-ReconfRqst" },
  {  96, "id-PDSCHSets-DeleteList-PSCH-ReconfRqst" },
  {  97, "id-PDSCHSets-ModifyList-PSCH-ReconfRqst" },
  {  98, "id-PICH-Information" },
  {  99, "id-PICH-Parameters-CTCH-ReconfRqstTDD" },
  { 100, "id-PowerAdjustmentType" },
  { 101, "id-PRACH-Information" },
  { 102, "id-PrimaryCCPCH-Information-Cell-ReconfRqstFDD" },
  { 103, "id-PrimaryCCPCH-Information-Cell-SetupRqstFDD" },
  { 104, "id-PrimaryCPICH-Information-Cell-ReconfRqstFDD" },
  { 105, "id-PrimaryCPICH-Information-Cell-SetupRqstFDD" },
  { 106, "id-PrimarySCH-Information-Cell-ReconfRqstFDD" },
  { 107, "id-PrimarySCH-Information-Cell-SetupRqstFDD" },
  { 108, "id-PrimaryScramblingCode" },
  { 109, "id-SCH-Information-Cell-ReconfRqstTDD" },
  { 110, "id-SCH-Information-Cell-SetupRqstTDD" },
  { 111, "id-PUSCH-Information-AddListIE-PSCH-ReconfRqst" },
  { 112, "id-PUSCH-Information-ModifyListIE-PSCH-ReconfRqst" },
  { 113, "id-PUSCHSets-AddList-PSCH-ReconfRqst" },
  { 114, "id-PUSCHSets-DeleteList-PSCH-ReconfRqst" },
  { 115, "id-PUSCHSets-ModifyList-PSCH-ReconfRqst" },
  { 116, "id-RACH-Information" },
  { 117, "id-RACH-ParametersItem-CTCH-SetupRqstFDD" },
  { 118, "id-RACH-ParameterItem-CTCH-SetupRqstTDD" },
  { 119, "id-ReportCharacteristics" },
  { 120, "id-Reporting-Object-RL-FailureInd" },
  { 121, "id-Reporting-Object-RL-RestoreInd" },
  { 122, "id-RL-InformationItem-DM-Rprt" },
  { 123, "id-RL-InformationItem-DM-Rqst" },
  { 124, "id-RL-InformationItem-DM-Rsp" },
  { 125, "id-RL-InformationItem-RL-AdditionRqstFDD" },
  { 126, "id-RL-informationItem-RL-DeletionRqst" },
  { 127, "id-RL-InformationItem-RL-FailureInd" },
  { 128, "id-RL-InformationItem-RL-PreemptRequiredInd" },
  { 129, "id-RL-InformationItem-RL-ReconfPrepFDD" },
  { 130, "id-RL-InformationItem-RL-ReconfRqstFDD" },
  { 131, "id-RL-InformationItem-RL-RestoreInd" },
  { 132, "id-RL-InformationItem-RL-SetupRqstFDD" },
  { 133, "id-RL-InformationList-RL-AdditionRqstFDD" },
  { 134, "id-RL-informationList-RL-DeletionRqst" },
  { 135, "id-RL-InformationList-RL-PreemptRequiredInd" },
  { 136, "id-RL-InformationList-RL-ReconfPrepFDD" },
  { 137, "id-RL-InformationList-RL-ReconfRqstFDD" },
  { 138, "id-RL-InformationList-RL-SetupRqstFDD" },
  { 139, "id-RL-InformationResponseItem-RL-AdditionRspFDD" },
  { 140, "id-RL-InformationResponseItem-RL-ReconfReady" },
  { 141, "id-RL-InformationResponseItem-RL-ReconfRsp" },
  { 142, "id-RL-InformationResponseItem-RL-SetupRspFDD" },
  { 143, "id-RL-InformationResponseList-RL-AdditionRspFDD" },
  { 144, "id-RL-InformationResponseList-RL-ReconfReady" },
  { 145, "id-RL-InformationResponseList-RL-ReconfRsp" },
  { 146, "id-RL-InformationResponseList-RL-SetupRspFDD" },
  { 147, "id-RL-InformationResponse-RL-AdditionRspTDD" },
  { 148, "id-RL-InformationResponse-RL-SetupRspTDD" },
  { 149, "id-RL-Information-RL-AdditionRqstTDD" },
  { 150, "id-RL-Information-RL-ReconfRqstTDD" },
  { 151, "id-RL-Information-RL-ReconfPrepTDD" },
  { 152, "id-RL-Information-RL-SetupRqstTDD" },
  { 153, "id-RL-ReconfigurationFailureItem-RL-ReconfFailure" },
  { 154, "id-RL-Set-InformationItem-DM-Rprt" },
  { 155, "id-RL-Set-InformationItem-DM-Rsp" },
  { 156, "id-RL-Set-InformationItem-RL-FailureInd" },
  { 157, "id-RL-Set-InformationItem-RL-RestoreInd" },
  { 158, "id-S-CCPCH-Information" },
  { 159, "id-S-CPICH-Information" },
  { 160, "id-SCH-Information" },
  { 161, "id-S-SCH-Information" },
  { 162, "id-Secondary-CCPCHListIE-CTCH-ReconfRqstTDD" },
  { 163, "id-Secondary-CCPCH-parameterListIE-CTCH-SetupRqstTDD" },
  { 164, "id-Secondary-CCPCH-Parameters-CTCH-ReconfRqstTDD" },
  { 165, "id-SecondaryCPICH-InformationItem-Cell-ReconfRqstFDD" },
  { 166, "id-SecondaryCPICH-InformationItem-Cell-SetupRqstFDD" },
  { 167, "id-SecondaryCPICH-InformationList-Cell-ReconfRqstFDD" },
  { 168, "id-SecondaryCPICH-InformationList-Cell-SetupRqstFDD" },
  { 169, "id-SecondarySCH-Information-Cell-ReconfRqstFDD" },
  { 170, "id-SecondarySCH-Information-Cell-SetupRqstFDD" },
  { 171, "id-SegmentInformationListIE-SystemInfoUpdate" },
  { 172, "id-SFN" },
  { 173, "id-SignallingBearerRequestIndicator" },
  { 174, "id-ShutdownTimer" },
  { 175, "id-Start-Of-Audit-Sequence-Indicator" },
  { 176, "id-Successful-RL-InformationRespItem-RL-AdditionFailureFDD" },
  { 177, "id-Successful-RL-InformationRespItem-RL-SetupFailureFDD" },
  { 178, "id-SyncCase" },
  { 179, "id-SyncCaseIndicatorItem-Cell-SetupRqstTDD-PSCH" },
  { 180, "id-T-Cell" },
  { 181, "id-TargetCommunicationControlPortID" },
  { 182, "id-TimeSlotConfigurationList-Cell-ReconfRqstTDD" },
  { 183, "id-TimeSlotConfigurationList-Cell-SetupRqstTDD" },
  { 184, "id-TransmissionDiversityApplied" },
  { 185, "id-TypeOfError" },
  { 186, "id-UARFCNforNt" },
  { 187, "id-UARFCNforNd" },
  { 188, "id-UARFCNforNu" },
  { 189, "id-UL-CCTrCH-InformationItem-RL-SetupRqstTDD" },
  { 190, "id-UL-CCTrCH-InformationList-RL-AdditionRqstTDD" },
  { 191, "id-UL-CCTrCH-InformationList-RL-SetupRqstTDD" },
  { 192, "id-UL-DPCH-InformationItem-RL-AdditionRqstTDD" },
  { 193, "id-UL-DPCH-InformationList-RL-SetupRqstTDD" },
  { 194, "id-UL-DPCH-Information-RL-ReconfPrepFDD" },
  { 195, "id-UL-DPCH-Information-RL-ReconfRqstFDD" },
  { 196, "id-UL-DPCH-Information-RL-SetupRqstFDD" },
  { 197, "id-Unsuccessful-RL-InformationRespItem-RL-AdditionFailureFDD" },
  { 198, "id-Unsuccessful-RL-InformationRespItem-RL-SetupFailureFDD" },
  { 199, "id-Unsuccessful-RL-InformationResp-RL-AdditionFailureTDD" },
  { 200, "id-Unsuccessful-RL-InformationResp-RL-SetupFailureTDD" },
  { 201, "id-USCH-Information-Add" },
  { 202, "id-USCH-Information-DeleteList-RL-ReconfPrepTDD" },
  { 203, "id-USCH-Information-ModifyList-RL-ReconfPrepTDD" },
  { 204, "id-USCH-InformationResponse" },
  { 205, "id-USCH-Information" },
  { 206, "id-USCH-RearrangeList-Bearer-RearrangeInd" },
  { 207, "id-Active-Pattern-Sequence-Information" },
  { 208, "id-AICH-ParametersListIE-CTCH-ReconfRqstFDD" },
  { 209, "id-AdjustmentRatio" },
  { 210, "id-FACH-ParametersListIE-CTCH-ReconfRqstFDD" },
  { 211, "id-CauseLevel-PSCH-ReconfFailure" },
  { 212, "id-CauseLevel-RL-AdditionFailureFDD" },
  { 213, "id-CauseLevel-RL-AdditionFailureTDD" },
  { 214, "id-CauseLevel-RL-ReconfFailure" },
  { 215, "id-CauseLevel-RL-SetupFailureFDD" },
  { 216, "id-CauseLevel-RL-SetupFailureTDD" },
  { 217, "id-Closed-Loop-Timing-Adjustment-Mode" },
  { 218, "id-CommonPhysicalChannelType-CTCH-ReconfRqstFDD" },
  { 219, "id-Compressed-Mode-Deactivation-Flag" },
  { 220, "id-DL-CCTrCH-InformationAddList-RL-ReconfPrepTDD" },
  { 221, "id-DL-CCTrCH-InformationDeleteItem-RL-ReconfRqstTDD" },
  { 222, "id-DL-CCTrCH-InformationDeleteList-RL-ReconfPrepTDD" },
  { 223, "id-DL-CCTrCH-InformationDeleteList-RL-ReconfRqstTDD" },
  { 224, "id-DL-CCTrCH-InformationModifyItem-RL-ReconfRqstTDD" },
  { 225, "id-DL-CCTrCH-InformationModifyList-RL-ReconfPrepTDD" },
  { 226, "id-DL-CCTrCH-InformationModifyList-RL-ReconfRqstTDD" },
  { 227, "id-DL-DPCH-InformationAddListIE-RL-ReconfPrepTDD" },
  { 228, "id-DL-DPCH-InformationModify-AddListIE-RL-ReconfPrepTDD" },
  { 229, "id-DL-DPCH-InformationModify-DeleteListIE-RL-ReconfPrepTDD" },
  { 230, "id-DL-DPCH-InformationModify-ModifyListIE-RL-ReconfPrepTDD" },
  { 231, "id-DL-TPC-Pattern01Count" },
  { 232, "id-DPC-Mode" },
  { 233, "id-DPCHConstant" },
  { 234, "id-FACH-ParametersList-CTCH-SetupRsp" },
  { 235, "id-Limited-power-increase-information-Cell-SetupRqstFDD" },
  { 236, "id-PCH-Parameters-CTCH-SetupRsp" },
  { 237, "id-PCH-ParametersItem-CTCH-ReconfRqstFDD" },
  { 238, "id-PICH-ParametersItem-CTCH-ReconfRqstFDD" },
  { 239, "id-PRACHConstant" },
  { 240, "id-PRACH-ParametersListIE-CTCH-ReconfRqstFDD" },
  { 241, "id-PUSCHConstant" },
  { 242, "id-RACH-Parameters-CTCH-SetupRsp" },
  { 243, "id-Synchronisation-Configuration-Cell-ReconfRqst" },
  { 244, "id-Synchronisation-Configuration-Cell-SetupRqst" },
  { 245, "id-Transmission-Gap-Pattern-Sequence-Information" },
  { 246, "id-UL-CCTrCH-InformationAddList-RL-ReconfPrepTDD" },
  { 247, "id-UL-CCTrCH-InformationDeleteItem-RL-ReconfRqstTDD" },
  { 248, "id-UL-CCTrCH-InformationDeleteList-RL-ReconfPrepTDD" },
  { 249, "id-UL-CCTrCH-InformationDeleteList-RL-ReconfRqstTDD" },
  { 250, "id-UL-CCTrCH-InformationModifyItem-RL-ReconfRqstTDD" },
  { 251, "id-UL-CCTrCH-InformationModifyList-RL-ReconfPrepTDD" },
  { 252, "id-UL-CCTrCH-InformationModifyList-RL-ReconfRqstTDD" },
  { 253, "id-UL-DPCH-InformationAddListIE-RL-ReconfPrepTDD" },
  { 254, "id-UL-DPCH-InformationModify-AddListIE-RL-ReconfPrepTDD" },
  { 255, "id-UL-DPCH-InformationModify-DeleteListIE-RL-ReconfPrepTDD" },
  { 256, "id-UL-DPCH-InformationModify-ModifyListIE-RL-ReconfPrepTDD" },
  { 257, "id-Unsuccessful-PDSCHSetItem-PSCH-ReconfFailureTDD" },
  { 258, "id-Unsuccessful-PUSCHSetItem-PSCH-ReconfFailureTDD" },
  { 259, "id-CommunicationContextInfoItem-Reset" },
  { 260, "id-CommunicationControlPortInfoItem-Reset" },
  { 261, "id-ResetIndicator" },
  { 262, "id-TimingAdvanceApplied" },
  { 263, "id-CFNReportingIndicator" },
  { 264, "id-SFNReportingIndicator" },
  { 265, "id-InnerLoopDLPCStatus" },
  { 266, "id-TimeslotISCPInfo" },
  { 267, "id-PICH-ParametersItem-CTCH-SetupRqstTDD" },
  { 268, "id-PRACH-ParametersItem-CTCH-SetupRqstTDD" },
  { 269, "id-CCTrCH-InformationItem-RL-FailureInd" },
  { 270, "id-CCTrCH-InformationItem-RL-RestoreInd" },
  { 271, "id-CauseLevel-SyncAdjustmntFailureTDD" },
  { 272, "id-CellAdjustmentInfo-SyncAdjustmntRqstTDD" },
  { 273, "id-CellAdjustmentInfoItem-SyncAdjustmentRqstTDD" },
  { 274, "id-CellSyncBurstInfoList-CellSyncReconfRqstTDD" },
  { 275, "id-CellSyncBurstTransInit-CellSyncInitiationRqstTDD" },
  { 276, "id-CellSyncBurstMeasureInit-CellSyncInitiationRqstTDD" },
  { 277, "id-CellSyncBurstMeasReconfiguration-CellSyncReconfRqstTDD" },
  { 278, "id-CellSyncBurstMeasInfoList-CellSyncReconfRqstTDD" },
  { 279, "id-CellSyncBurstTransReconfInfo-CellSyncReconfRqstTDD" },
  { 280, "id-CellSyncInfo-CellSyncReprtTDD" },
  { 281, "id-CSBTransmissionID" },
  { 282, "id-CSBMeasurementID" },
  { 283, "id-NCyclesPerSFNperiod" },
  { 284, "id-NRepetitionsPerCyclePeriod" },
  { 285, "id-SyncFrameNumber" },
  { 286, "id-SynchronisationReportType" },
  { 287, "id-SynchronisationReportCharacteristics" },
  { 288, "id-Unsuccessful-cell-InformationRespItem-SyncAdjustmntFailureTDD" },
  { 289, "id-ReferenceClockAvailability" },
  { 290, "id-ReferenceSFNoffset" },
  { 291, "id-InformationExchangeID" },
  { 292, "id-InformationExchangeObjectType-InfEx-Rqst" },
  { 293, "id-InformationType" },
  { 294, "id-InformationReportCharacteristics" },
  { 295, "id-InformationExchangeObjectType-InfEx-Rsp" },
  { 296, "id-InformationExchangeObjectType-InfEx-Rprt" },
  { 297, "id-IPDLParameter-Information-Cell-ReconfRqstFDD" },
  { 298, "id-IPDLParameter-Information-Cell-SetupRqstFDD" },
  { 299, "id-IPDLParameter-Information-Cell-ReconfRqstTDD" },
  { 300, "id-IPDLParameter-Information-Cell-SetupRqstTDD" },
  { 301, "id-DL-DPCH-LCR-Information-RL-SetupRqstTDD" },
  { 302, "id-DwPCH-LCR-Information" },
  { 303, "id-DwPCH-LCR-InformationList-AuditRsp" },
  { 304, "id-DwPCH-LCR-Information-Cell-SetupRqstTDD" },
  { 305, "id-DwPCH-LCR-Information-Cell-ReconfRqstTDD" },
  { 306, "id-DwPCH-LCR-Information-ResourceStatusInd" },
  { 307, "id-maxFACH-Power-LCR-CTCH-SetupRqstTDD" },
  { 308, "id-maxFACH-Power-LCR-CTCH-ReconfRqstTDD" },
  { 309, "id-FPACH-LCR-Information" },
  { 310, "id-FPACH-LCR-Information-AuditRsp" },
  { 311, "id-FPACH-LCR-InformationList-AuditRsp" },
  { 312, "id-FPACH-LCR-InformationList-ResourceStatusInd" },
  { 313, "id-FPACH-LCR-Parameters-CTCH-SetupRqstTDD" },
  { 314, "id-FPACH-LCR-Parameters-CTCH-ReconfRqstTDD" },
  { 315, "id-PCCPCH-LCR-Information-Cell-SetupRqstTDD" },
  { 316, "id-PCH-Power-LCR-CTCH-SetupRqstTDD" },
  { 317, "id-PCH-Power-LCR-CTCH-ReconfRqstTDD" },
  { 318, "id-PICH-LCR-Parameters-CTCH-SetupRqstTDD" },
  { 319, "id-PRACH-LCR-ParametersList-CTCH-SetupRqstTDD" },
  { 320, "id-RL-InformationResponse-LCR-RL-SetupRspTDD" },
  { 321, "id-Secondary-CCPCH-LCR-parameterList-CTCH-SetupRqstTDD" },
  { 322, "id-TimeSlot" },
  { 323, "id-TimeSlotConfigurationList-LCR-Cell-ReconfRqstTDD" },
  { 324, "id-TimeSlotConfigurationList-LCR-Cell-SetupRqstTDD" },
  { 325, "id-TimeslotISCP-LCR-InfoList-RL-SetupRqstTDD" },
  { 326, "id-TimeSlotLCR-CM-Rqst" },
  { 327, "id-UL-DPCH-LCR-Information-RL-SetupRqstTDD" },
  { 328, "id-DL-DPCH-InformationItem-LCR-RL-AdditionRqstTDD" },
  { 329, "id-UL-DPCH-InformationItem-LCR-RL-AdditionRqstTDD" },
  { 330, "id-TimeslotISCP-InformationList-LCR-RL-AdditionRqstTDD" },
  { 331, "id-DL-DPCH-LCR-InformationAddList-RL-ReconfPrepTDD" },
  { 332, "id-DL-DPCH-LCR-InformationModify-AddList-RL-ReconfPrepTDD" },
  { 333, "id-DL-Timeslot-LCR-InformationModify-ModifyList-RL-ReconfPrepTDD" },
  { 334, "id-TimeslotISCPInfoList-LCR-DL-PC-RqstTDD" },
  { 335, "id-UL-DPCH-LCR-InformationAddListIE-RL-ReconfPrepTDD" },
  { 336, "id-UL-DPCH-LCR-InformationModify-AddList" },
  { 337, "id-UL-TimeslotLCR-Information-RL-ReconfPrepTDD" },
  { 338, "id-UL-SIRTarget" },
  { 339, "id-PDSCH-AddInformation-LCR-PSCH-ReconfRqst" },
  { 340, "id-PDSCH-ModifyInformation-LCR-PSCH-ReconfRqst" },
  { 341, "id-PUSCH-AddInformation-LCR-PSCH-ReconfRqst" },
  { 342, "id-PUSCH-ModifyInformation-LCR-PSCH-ReconfRqst" },
  { 343, "id-timeslotInfo-CellSyncInitiationRqstTDD" },
  { 344, "id-SyncReportType-CellSyncReprtTDD" },
  { 345, "id-Power-Local-Cell-Group-InformationItem-AuditRsp" },
  { 346, "id-Power-Local-Cell-Group-InformationItem-ResourceStatusInd" },
  { 347, "id-Power-Local-Cell-Group-InformationItem2-ResourceStatusInd" },
  { 348, "id-Power-Local-Cell-Group-InformationList-AuditRsp" },
  { 349, "id-Power-Local-Cell-Group-InformationList-ResourceStatusInd" },
  { 350, "id-Power-Local-Cell-Group-InformationList2-ResourceStatusInd" },
  { 351, "id-Power-Local-Cell-Group-ID" },
  { 352, "id-PUSCH-Info-DM-Rqst" },
  { 353, "id-PUSCH-Info-DM-Rsp" },
  { 354, "id-PUSCH-Info-DM-Rprt" },
  { 355, "id-InitDL-Power" },
  { 356, "id-cellSyncBurstRepetitionPeriod" },
  { 357, "id-ReportCharacteristicsType-OnModification" },
  { 358, "id-SFNSFNMeasurementValueInformation" },
  { 359, "id-SFNSFNMeasurementThresholdInformation" },
  { 360, "id-TUTRANGPSMeasurementValueInformation" },
  { 361, "id-TUTRANGPSMeasurementThresholdInformation" },
  { 362, "id-Rx-Timing-Deviation-Value-LCR" },
  { 363, "id-RL-InformationResponse-LCR-RL-AdditionRspTDD" },
  { 364, "id-DL-PowerBalancing-Information" },
  { 365, "id-DL-PowerBalancing-ActivationIndicator" },
  { 366, "id-DL-PowerBalancing-UpdatedIndicator" },
  { 367, "id-CCTrCH-Initial-DL-Power-RL-SetupRqstTDD" },
  { 368, "id-CCTrCH-Initial-DL-Power-RL-AdditionRqstTDD" },
  { 369, "id-CCTrCH-Initial-DL-Power-RL-ReconfPrepTDD" },
  { 370, "id-IPDLParameter-Information-LCR-Cell-SetupRqstTDD" },
  { 371, "id-IPDLParameter-Information-LCR-Cell-ReconfRqstTDD" },
  { 372, "id-HS-PDSCH-HS-SCCH-E-AGCH-E-RGCH-E-HICH-MaxPower-PSCH-ReconfRqst" },
  { 373, "id-HS-PDSCH-HS-SCCH-ScramblingCode-PSCH-ReconfRqst" },
  { 374, "id-HS-PDSCH-FDD-Code-Information-PSCH-ReconfRqst" },
  { 375, "id-HS-SCCH-FDD-Code-Information-PSCH-ReconfRqst" },
  { 376, "id-HS-PDSCH-TDD-Information-PSCH-ReconfRqst" },
  { 377, "id-Add-To-HS-SCCH-Resource-Pool-PSCH-ReconfRqst" },
  { 378, "id-Modify-HS-SCCH-Resource-Pool-PSCH-ReconfRqst" },
  { 379, "id-Delete-From-HS-SCCH-Resource-Pool-PSCH-ReconfRqst" },
  { 380, "id-bindingID" },
  { 381, "id-RL-Specific-DCH-Info" },
  { 382, "id-transportlayeraddress" },
  { 383, "id-DelayedActivation" },
  { 384, "id-DelayedActivationList-RL-ActivationCmdFDD" },
  { 385, "id-DelayedActivationInformation-RL-ActivationCmdFDD" },
  { 386, "id-DelayedActivationList-RL-ActivationCmdTDD" },
  { 387, "id-DelayedActivationInformation-RL-ActivationCmdTDD" },
  { 388, "id-neighbouringTDDCellMeasurementInformationLCR" },
  { 389, "id-SYNCDlCodeId-TransInitLCR-CellSyncInitiationRqstTDD" },
  { 390, "id-SYNCDlCodeId-MeasureInitLCR-CellSyncInitiationRqstTDD" },
  { 391, "id-SYNCDlCodeIdTransReconfInfoLCR-CellSyncReconfRqstTDD" },
  { 392, "id-SYNCDlCodeIdMeasReconfigurationLCR-CellSyncReconfRqstTDD" },
  { 393, "id-SYNCDlCodeIdMeasInfoList-CellSyncReconfRqstTDD" },
  { 394, "id-SyncDLCodeIdsMeasInfoList-CellSyncReprtTDD" },
  { 395, "id-SyncDLCodeIdThreInfoLCR" },
  { 396, "id-NSubCyclesPerCyclePeriod-CellSyncReconfRqstTDD" },
  { 397, "id-DwPCH-Power" },
  { 398, "id-AccumulatedClockupdate-CellSyncReprtTDD" },
  { 399, "id-Angle-Of-Arrival-Value-LCR" },
  { 400, "id-HSDSCH-FDD-Information" },
  { 401, "id-HSDSCH-FDD-Information-Response" },
  { 402, "id-HSDSCH-Information-to-Modify" },
  { 403, "id-HSDSCH-RNTI" },
  { 404, "id-HSDSCH-TDD-Information" },
  { 405, "id-HSDSCH-TDD-Information-Response" },
  { 406, "id-HSPDSCH-RL-ID" },
  { 407, "id-PrimCCPCH-RSCP-DL-PC-RqstTDD" },
  { 408, "id-PDSCH-RL-ID" },
  { 409, "id-HSDSCH-RearrangeList-Bearer-RearrangeInd" },
  { 410, "id-UL-Synchronisation-Parameters-LCR" },
  { 411, "id-HSDSCH-FDD-Update-Information" },
  { 412, "id-HSDSCH-TDD-Update-Information" },
  { 413, "id-DL-DPCH-TimeSlotFormat-LCR-ModifyItem-RL-ReconfPrepTDD" },
  { 414, "id-UL-DPCH-TimeSlotFormat-LCR-ModifyItem-RL-ReconfPrepTDD" },
  { 415, "id-TDD-TPC-UplinkStepSize-LCR-RL-SetupRqstTDD" },
  { 416, "id-TDD-TPC-UplinkStepSize-LCR-RL-AdditionRqstTDD" },
  { 417, "id-TDD-TPC-DownlinkStepSize-RL-AdditionRqstTDD" },
  { 418, "id-TDD-TPC-UplinkStepSize-InformationAdd-LCR-RL-ReconfPrepTDD" },
  { 419, "id-TDD-TPC-UplinkStepSize-InformationModify-LCR-RL-ReconfPrepTDD" },
  { 420, "id-TDD-TPC-DownlinkStepSize-InformationModify-RL-ReconfPrepTDD" },
  { 421, "id-TDD-TPC-DownlinkStepSize-InformationAdd-RL-ReconfPrepTDD" },
  { 422, "id-CCTrCH-Maximum-DL-Power-RL-SetupRqstTDD" },
  { 423, "id-CCTrCH-Minimum-DL-Power-RL-SetupRqstTDD" },
  { 424, "id-CCTrCH-Maximum-DL-Power-RL-AdditionRqstTDD" },
  { 425, "id-CCTrCH-Minimum-DL-Power-RL-AdditionRqstTDD" },
  { 426, "id-CCTrCH-Maximum-DL-Power-InformationAdd-RL-ReconfPrepTDD" },
  { 427, "id-CCTrCH-Minimum-DL-Power-InformationAdd-RL-ReconfPrepTDD" },
  { 428, "id-CCTrCH-Maximum-DL-Power-InformationModify-RL-ReconfPrepTDD" },
  { 429, "id-CCTrCH-Minimum-DL-Power-InformationModify-RL-ReconfPrepTDD" },
  { 430, "id-Maximum-DL-Power-Modify-LCR-InformationModify-RL-ReconfPrepTDD" },
  { 431, "id-Minimum-DL-Power-Modify-LCR-InformationModify-RL-ReconfPrepTDD" },
  { 432, "id-DL-DPCH-LCR-InformationModify-ModifyList-RL-ReconfRqstTDD" },
  { 433, "id-CCTrCH-Maximum-DL-Power-InformationModify-RL-ReconfRqstTDD" },
  { 434, "id-CCTrCH-Minimum-DL-Power-InformationModify-RL-ReconfRqstTDD" },
  { 435, "id-Initial-DL-Power-TimeslotLCR-InformationItem" },
  { 436, "id-Maximum-DL-Power-TimeslotLCR-InformationItem" },
  { 437, "id-Minimum-DL-Power-TimeslotLCR-InformationItem" },
  { 438, "id-HS-DSCHProvidedBitRateValueInformation" },
  { 439, "id-HS-DSCHRequiredPowerValueInformation" },
  { 440, "id-HS-DSCHRequiredPowerValue" },
  { 441, "id-TransmittedCarrierPowerOfAllCodesNotUsedForHSTransmission" },
  { 442, "id-HS-SICH-Reception-Quality" },
  { 443, "id-HS-SICH-Reception-Quality-Measurement-Value" },
  { 444, "id-HSSICH-Info-DM-Rprt" },
  { 445, "id-HSSICH-Info-DM-Rqst" },
  { 446, "id-HSSICH-Info-DM-Rsp" },
  { 447, "id-Best-Cell-Portions-Value" },
  { 448, "id-Primary-CPICH-Usage-for-Channel-Estimation" },
  { 449, "id-Secondary-CPICH-Information-Change" },
  { 450, "id-NumberOfReportedCellPortions" },
  { 451, "id-CellPortion-InformationItem-Cell-SetupRqstFDD" },
  { 452, "id-CellPortion-InformationList-Cell-SetupRqstFDD" },
  { 453, "id-TimeslotISCP-LCR-InfoList-RL-ReconfPrepTDD" },
  { 454, "id-Secondary-CPICH-Information" },
  { 455, "id-Received-total-wide-band-power-For-CellPortion" },
  { 456, "id-Unidirectional-DCH-Indicator" },
  { 457, "id-TimingAdjustmentValueLCR" },
  { 458, "id-multipleRL-ul-DPCH-InformationList" },
  { 459, "id-multipleRL-ul-DPCH-InformationModifyList" },
  { 460, "id-RL-ID" },
  { 461, "id-SAT-Info-Almanac-ExtItem" },
  { 462, "id-HSDPA-Capability" },
  { 463, "id-HSDSCH-Resources-Information-AuditRsp" },
  { 464, "id-HSDSCH-Resources-Information-ResourceStatusInd" },
  { 465, "id-HSDSCH-MACdFlows-to-Add" },
  { 466, "id-HSDSCH-MACdFlows-to-Delete" },
  { 467, "id-HSDSCH-Information-to-Modify-Unsynchronised" },
  { 468, "id-TnlQos" },
  { 469, "id-Received-total-wide-band-power-For-CellPortion-Value" },
  { 470, "id-Transmitted-Carrier-Power-For-CellPortion" },
  { 471, "id-Transmitted-Carrier-Power-For-CellPortion-Value" },
  { 472, "id-TransmittedCarrierPowerOfAllCodesNotUsedForHS-PDSCH-HS-SCCH-E-AGCH-E-RGCHOrE-HICHTransmissionCellPortion" },
  { 473, "id-TransmittedCarrierPowerOfAllCodesNotUsedForHS-PDSCH-HS-SCCH-E-AGCH-E-RGCHOrE-HICHTransmissionCellPortionValue" },
  { 474, "id-UpPTSInterferenceValue" },
  { 475, "id-PrimaryCCPCH-RSCP-Delta" },
  { 476, "id-MeasurementRecoveryBehavior" },
  { 477, "id-MeasurementRecoveryReportingIndicator" },
  { 478, "id-MeasurementRecoverySupportIndicator" },
  { 479, "id-Tstd-indicator" },
  { 480, "id-multiple-RL-Information-RL-ReconfPrepTDD" },
  { 481, "id-multiple-RL-Information-RL-ReconfRqstTDD" },
  { 482, "id-DL-DPCH-Power-Information-RL-ReconfPrepFDD" },
  { 483, "id-F-DPCH-Information-RL-ReconfPrepFDD" },
  { 484, "id-F-DPCH-Information-RL-SetupRqstFDD" },
  { 485, "id-Additional-S-CCPCH-Parameters-CTCH-ReconfRqstTDD" },
  { 486, "id-Additional-S-CCPCH-Parameters-CTCH-SetupRqstTDD" },
  { 487, "id-Additional-S-CCPCH-LCR-Parameters-CTCH-ReconfRqstTDD" },
  { 488, "id-Additional-S-CCPCH-LCR-Parameters-CTCH-SetupRqstTDD" },
  { 489, "id-MICH-CFN" },
  { 490, "id-MICH-Information-AuditRsp" },
  { 491, "id-MICH-Information-ResourceStatusInd" },
  { 492, "id-MICH-Parameters-CTCH-ReconfRqstFDD" },
  { 493, "id-MICH-Parameters-CTCH-ReconfRqstTDD" },
  { 494, "id-MICH-Parameters-CTCH-SetupRqstFDD" },
  { 495, "id-MICH-Parameters-CTCH-SetupRqstTDD" },
  { 496, "id-Modification-Period" },
  { 497, "id-NI-Information-NotifUpdateCmd" },
  { 498, "id-S-CCPCH-InformationListExt-AuditRsp" },
  { 499, "id-S-CCPCH-InformationListExt-ResourceStatusInd" },
  { 500, "id-S-CCPCH-LCR-InformationListExt-AuditRsp" },
  { 501, "id-S-CCPCH-LCR-InformationListExt-ResourceStatusInd" },
  { 502, "id-HARQ-Preamble-Mode" },
  { 503, "id-Initial-DL-DPCH-TimingAdjustment" },
  { 504, "id-Initial-DL-DPCH-TimingAdjustment-Allowed" },
  { 505, "id-DLTransmissionBranchLoadValue" },
  { 506, "id-Power-Local-Cell-Group-choice-CM-Rqst" },
  { 507, "id-Power-Local-Cell-Group-choice-CM-Rsp" },
  { 508, "id-Power-Local-Cell-Group-choice-CM-Rprt" },
  { 509, "id-SynchronisationIndicator" },
  { 510, "id-HSDPA-And-EDCH-CellPortion-Information-PSCH-ReconfRqst" },
  { 511, "id-HSDPA-And-EDCH-CellPortion-InformationList-PSCH-ReconfRqst" },
  { 512, "id-HS-DSCHRequiredPowerValue-For-Cell-Portion" },
  { 513, "id-HS-DSCHRequiredPowerValueInformation-For-CellPortion" },
  { 514, "id-HS-DSCHProvidedBitRateValueInformation-For-CellPortion" },
  { 515, "id-E-AGCH-And-E-RGCH-E-HICH-FDD-Scrambling-Code" },
  { 516, "id-E-AGCH-FDD-Code-Information" },
  { 517, "id-E-DCH-Capability" },
  { 518, "id-E-DCH-FDD-DL-Control-Channel-Information" },
  { 519, "id-E-DCH-FDD-Information" },
  { 520, "id-E-DCH-FDD-Information-Response" },
  { 521, "id-E-DCH-FDD-Information-to-Modify" },
  { 522, "id-E-DCH-MACdFlows-to-Add" },
  { 523, "id-E-DCH-MACdFlows-to-Delete" },
  { 524, "id-E-DCH-Resources-Information-AuditRsp" },
  { 525, "id-E-DCH-Resources-Information-ResourceStatusInd" },
  { 526, "id-E-DCH-RL-Indication" },
  { 527, "id-E-DCH-RL-Set-ID" },
  { 528, "id-E-DPCH-Information-RL-ReconfPrepFDD" },
  { 529, "id-E-DPCH-Information-RL-SetupRqstFDD" },
  { 530, "id-E-RGCH-E-HICH-FDD-Code-Information" },
  { 531, "id-Serving-E-DCH-RL-ID" },
  { 532, "id-UL-DPDCH-Indicator-For-E-DCH-Operation" },
  { 533, "id-FDD-S-CCPCH-FrameOffset-CTCH-SetupRqstFDD" },
  { 534, "id-E-DPCH-Information-RL-ReconfRqstFDD" },
  { 535, "id-Maximum-Target-ReceivedTotalWideBandPower" },
  { 536, "id-E-DCHProvidedBitRateValueInformation" },
  { 537, "id-HARQ-Preamble-Mode-Activation-Indicator" },
  { 538, "id-RL-Specific-E-DCH-Info" },
  { 539, "id-E-DCH-CapacityConsumptionLaw" },
  { 540, "id-multiple-DedicatedMeasurementValueList-TDD-DM-Rsp" },
  { 541, "id-multiple-DedicatedMeasurementValueList-LCR-TDD-DM-Rsp" },
  { 542, "id-E-DCH-RearrangeList-Bearer-RearrangeInd" },
  { 543, "id-multipleRL-dl-CCTrCH-InformationModifyList-RL-ReconfRqstTDD" },
  { 544, "id-Target-NonServing-EDCH-To-Total-EDCH-Power-Ratio" },
  { 545, "id-CellPortion-InformationItem-Cell-ReconfRqstFDD" },
  { 546, "id-CellPortion-InformationList-Cell-ReconfRqstFDD" },
  { 547, "id-multiple-PUSCH-InfoList-DM-Rsp" },
  { 548, "id-multiple-PUSCH-InfoList-DM-Rprt" },
  { 549, "id-Reference-ReceivedTotalWideBandPower" },
  { 0, NULL }
};

static const per_choice_t DymmyProtocolIE_ID_choice[] = {
  {   0, &hf_nbap_id_AICH_Information, ASN1_EXTENSION_ROOT    , dissect_nbap_Common_PhysicalChannel_Status_Information },
  {   1, &hf_nbap_id_BCH_Information, ASN1_EXTENSION_ROOT    , dissect_nbap_Common_TransportChannel_Status_Information },
  {   2, &hf_nbap_id_BCCH_ModificationTime, ASN1_EXTENSION_ROOT    , dissect_nbap_BCCH_ModificationTime },
  {   3, &hf_nbap_id_BlockingPriorityIndicator, ASN1_EXTENSION_ROOT    , dissect_nbap_BlockingPriorityIndicator },
  {   4, &hf_nbap_id_Cause       , ASN1_EXTENSION_ROOT    , dissect_nbap_Cause },
  {   5, &hf_nbap_id_CCP_InformationItem_AuditRsp, ASN1_EXTENSION_ROOT    , dissect_nbap_CCP_InformationItem_AuditRsp },
  {   6, &hf_nbap_id_CCP_InformationList_AuditRsp, ASN1_EXTENSION_ROOT    , dissect_nbap_CCP_InformationList_AuditRsp },
  {   7, &hf_nbap_id_CCP_InformationItem_ResourceStatusInd, ASN1_EXTENSION_ROOT    , dissect_nbap_CCP_InformationItem_ResourceStatusInd },
  {   8, &hf_nbap_id_Cell_InformationItem_AuditRsp, ASN1_EXTENSION_ROOT    , dissect_nbap_Cell_InformationItem_AuditRsp },
  {   9, &hf_nbap_id_Cell_InformationItem_ResourceStatusInd, ASN1_EXTENSION_ROOT    , dissect_nbap_Cell_InformationItem_ResourceStatusInd },
  {  10, &hf_nbap_id_Cell_InformationList_AuditRsp, ASN1_EXTENSION_ROOT    , dissect_nbap_Cell_InformationList_AuditRsp },
  {  11, &hf_nbap_id_CellParameterID, ASN1_EXTENSION_ROOT    , dissect_nbap_CellParameterID },
  {  12, &hf_nbap_id_CFN         , ASN1_EXTENSION_ROOT    , dissect_nbap_CFN },
  {  13, &hf_nbap_id_C_ID        , ASN1_EXTENSION_ROOT    , dissect_nbap_C_ID },
  {  14, &hf_nbap_id_CommonMeasurementAccuracy, ASN1_EXTENSION_ROOT    , dissect_nbap_CommonMeasurementAccuracy },
  {  15, &hf_nbap_id_CommonMeasurementObjectType_CM_Rprt, ASN1_EXTENSION_ROOT    , dissect_nbap_CommonMeasurementObjectType_CM_Rprt },
  {  16, &hf_nbap_id_CommonMeasurementObjectType_CM_Rqst, ASN1_EXTENSION_ROOT    , dissect_nbap_CommonMeasurementObjectType_CM_Rqst },
  {  17, &hf_nbap_id_CommonMeasurementObjectType_CM_Rsp, ASN1_EXTENSION_ROOT    , dissect_nbap_CommonMeasurementObjectType_CM_Rsp },
  {  18, &hf_nbap_id_CommonMeasurementType, ASN1_EXTENSION_ROOT    , dissect_nbap_CommonMeasurementType },
  {  19, &hf_nbap_id_CommonPhysicalChannelID, ASN1_EXTENSION_ROOT    , dissect_nbap_CommonPhysicalChannelID },
  {  20, &hf_nbap_id_CommonPhysicalChannelType_CTCH_SetupRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_CommonPhysicalChannelType_CTCH_SetupRqstFDD },
  {  21, &hf_nbap_id_CommonPhysicalChannelType_CTCH_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_CommonPhysicalChannelType_CTCH_SetupRqstTDD },
  {  22, &hf_nbap_id_CommunicationControlPortID, ASN1_EXTENSION_ROOT    , dissect_nbap_CommunicationControlPortID },
  {  23, &hf_nbap_id_ConfigurationGenerationID, ASN1_EXTENSION_ROOT    , dissect_nbap_ConfigurationGenerationID },
  {  24, &hf_nbap_id_CRNC_CommunicationContextID, ASN1_EXTENSION_ROOT    , dissect_nbap_CRNC_CommunicationContextID },
  {  25, &hf_nbap_id_CriticalityDiagnostics, ASN1_EXTENSION_ROOT    , dissect_nbap_CriticalityDiagnostics },
  {  26, &hf_nbap_id_DCHs_to_Add_FDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DCH_FDD_Information },
  {  27, &hf_nbap_id_DCHs_to_Add_TDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DCH_TDD_Information },
  {  28, &hf_nbap_id_DCH_DeleteList_RL_ReconfPrepFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DCH_DeleteList_RL_ReconfPrepFDD },
  {  29, &hf_nbap_id_DCH_DeleteList_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DCH_DeleteList_RL_ReconfPrepTDD },
  {  30, &hf_nbap_id_DCH_DeleteList_RL_ReconfRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DCH_DeleteList_RL_ReconfRqstFDD },
  {  31, &hf_nbap_id_DCH_DeleteList_RL_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DCH_DeleteList_RL_ReconfRqstTDD },
  {  32, &hf_nbap_id_DCH_FDD_Information, ASN1_EXTENSION_ROOT    , dissect_nbap_DCH_FDD_Information },
  {  33, &hf_nbap_id_DCH_TDD_Information, ASN1_EXTENSION_ROOT    , dissect_nbap_DCH_TDD_Information },
  {  34, &hf_nbap_id_DCH_InformationResponse, ASN1_EXTENSION_ROOT    , dissect_nbap_DCH_InformationResponse },
  {  35, &hf_nbap_id_FDD_DCHs_to_Modify, ASN1_EXTENSION_ROOT    , dissect_nbap_FDD_DCHs_to_Modify },
  {  36, &hf_nbap_id_TDD_DCHs_to_Modify, ASN1_EXTENSION_ROOT    , dissect_nbap_TDD_DCHs_to_Modify },
  {  37, &hf_nbap_id_DCH_RearrangeList_Bearer_RearrangeInd, ASN1_EXTENSION_ROOT    , dissect_nbap_DCH_RearrangeList_Bearer_RearrangeInd },
  {  38, &hf_nbap_id_DedicatedMeasurementObjectType_DM_Rprt, ASN1_EXTENSION_ROOT    , dissect_nbap_DedicatedMeasurementObjectType_DM_Rprt },
  {  39, &hf_nbap_id_DedicatedMeasurementObjectType_DM_Rqst, ASN1_EXTENSION_ROOT    , dissect_nbap_DedicatedMeasurementObjectType_DM_Rqst },
  {  40, &hf_nbap_id_DedicatedMeasurementObjectType_DM_Rsp, ASN1_EXTENSION_ROOT    , dissect_nbap_DedicatedMeasurementObjectType_DM_Rsp },
  {  41, &hf_nbap_id_DedicatedMeasurementType, ASN1_EXTENSION_ROOT    , dissect_nbap_DedicatedMeasurementType },
  {  42, &hf_nbap_id_DL_CCTrCH_InformationItem_RL_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_CCTrCH_InformationItem_RL_SetupRqstTDD },
  {  43, &hf_nbap_id_DL_CCTrCH_InformationList_RL_AdditionRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_CCTrCH_InformationList_RL_AdditionRqstTDD },
  {  44, &hf_nbap_id_DL_CCTrCH_InformationList_RL_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_CCTrCH_InformationList_RL_SetupRqstTDD },
  {  45, &hf_nbap_id_DL_DPCH_InformationItem_RL_AdditionRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_DPCH_InformationItem_RL_AdditionRqstTDD },
  {  46, &hf_nbap_id_DL_DPCH_InformationList_RL_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_DPCH_InformationItem_RL_SetupRqstTDD },
  {  47, &hf_nbap_id_DL_DPCH_Information_RL_ReconfPrepFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_DPCH_Information_RL_ReconfPrepFDD },
  {  48, &hf_nbap_id_DL_DPCH_Information_RL_ReconfRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_DPCH_Information_RL_ReconfRqstFDD },
  {  49, &hf_nbap_id_DL_DPCH_Information_RL_SetupRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_DPCH_Information_RL_SetupRqstFDD },
  {  50, &hf_nbap_id_DL_DPCH_TimingAdjustment, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_DPCH_TimingAdjustment },
  {  51, &hf_nbap_id_DL_ReferencePowerInformationItem_DL_PC_Rqst, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_ReferencePowerInformationItem_DL_PC_Rqst },
  {  52, &hf_nbap_id_DLReferencePower, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_Power },
  {  53, &hf_nbap_id_DLReferencePowerList_DL_PC_Rqst, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_ReferencePowerInformationList_DL_PC_Rqst },
  {  54, &hf_nbap_id_DSCHs_to_Add_TDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DSCH_TDD_Information },
  {  55, &hf_nbap_id_DSCH_Information_DeleteList_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DSCH_Information_DeleteList_RL_ReconfPrepTDD },
  {  56, &hf_nbap_id_DSCH_Information_ModifyList_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DSCH_Information_ModifyList_RL_ReconfPrepTDD },
  {  57, &hf_nbap_id_DSCH_InformationResponse, ASN1_EXTENSION_ROOT    , dissect_nbap_DSCH_InformationResponse },
  {  58, &hf_nbap_id_DSCH_TDD_Information, ASN1_EXTENSION_ROOT    , dissect_nbap_DSCH_TDD_Information },
  {  59, &hf_nbap_id_DSCH_RearrangeList_Bearer_RearrangeInd, ASN1_EXTENSION_ROOT    , dissect_nbap_DSCH_RearrangeList_Bearer_RearrangeInd },
  {  60, &hf_nbap_id_End_Of_Audit_Sequence_Indicator, ASN1_EXTENSION_ROOT    , dissect_nbap_End_Of_Audit_Sequence_Indicator },
  {  61, &hf_nbap_id_FACH_Information, ASN1_EXTENSION_ROOT    , dissect_nbap_Common_TransportChannel_Status_Information },
  {  62, &hf_nbap_id_FACH_ParametersList_CTCH_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_FACH_ParametersList_CTCH_ReconfRqstTDD },
  {  63, &hf_nbap_id_FACH_ParametersListIE_CTCH_SetupRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_FACH_ParametersListIE_CTCH_SetupRqstFDD },
  {  64, &hf_nbap_id_FACH_ParametersListIE_CTCH_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_FACH_ParametersListIE_CTCH_SetupRqstTDD },
  {  65, &hf_nbap_id_IndicationType_ResourceStatusInd, ASN1_EXTENSION_ROOT    , dissect_nbap_IndicationType_ResourceStatusInd },
  {  66, &hf_nbap_id_Local_Cell_ID, ASN1_EXTENSION_ROOT    , dissect_nbap_Local_Cell_ID },
  {  67, &hf_nbap_id_Local_Cell_Group_InformationItem_AuditRsp, ASN1_EXTENSION_ROOT    , dissect_nbap_Local_Cell_Group_InformationItem_AuditRsp },
  {  68, &hf_nbap_id_Local_Cell_Group_InformationItem_ResourceStatusInd, ASN1_EXTENSION_ROOT    , dissect_nbap_Local_Cell_Group_InformationItem_ResourceStatusInd },
  {  69, &hf_nbap_id_Local_Cell_Group_InformationItem2_ResourceStatusInd, ASN1_EXTENSION_ROOT    , dissect_nbap_Local_Cell_Group_InformationItem2_ResourceStatusInd },
  {  70, &hf_nbap_id_Local_Cell_Group_InformationList_AuditRsp, ASN1_EXTENSION_ROOT    , dissect_nbap_Local_Cell_Group_InformationList_AuditRsp },
  {  71, &hf_nbap_id_Local_Cell_InformationItem_AuditRsp, ASN1_EXTENSION_ROOT    , dissect_nbap_Local_Cell_InformationItem_AuditRsp },
  {  72, &hf_nbap_id_Local_Cell_InformationItem_ResourceStatusInd, ASN1_EXTENSION_ROOT    , dissect_nbap_Local_Cell_InformationItem_ResourceStatusInd },
  {  73, &hf_nbap_id_Local_Cell_InformationItem2_ResourceStatusInd, ASN1_EXTENSION_ROOT    , dissect_nbap_Local_Cell_InformationItem2_ResourceStatusInd },
  {  74, &hf_nbap_id_Local_Cell_InformationList_AuditRsp, ASN1_EXTENSION_ROOT    , dissect_nbap_Local_Cell_InformationList_AuditRsp },
  {  75, &hf_nbap_id_AdjustmentPeriod, ASN1_EXTENSION_ROOT    , dissect_nbap_AdjustmentPeriod },
  {  76, &hf_nbap_id_MaxAdjustmentStep, ASN1_EXTENSION_ROOT    , dissect_nbap_MaxAdjustmentStep },
  {  77, &hf_nbap_id_MaximumTransmissionPower, ASN1_EXTENSION_ROOT    , dissect_nbap_MaximumTransmissionPower },
  {  78, &hf_nbap_id_MeasurementFilterCoefficient, ASN1_EXTENSION_ROOT    , dissect_nbap_MeasurementFilterCoefficient },
  {  79, &hf_nbap_id_MeasurementID, ASN1_EXTENSION_ROOT    , dissect_nbap_MeasurementID },
  {  80, &hf_nbap_id_MessageStructure, ASN1_EXTENSION_ROOT    , dissect_nbap_MessageStructure },
  {  81, &hf_nbap_id_MIB_SB_SIB_InformationList_SystemInfoUpdateRqst, ASN1_EXTENSION_ROOT    , dissect_nbap_MIB_SB_SIB_InformationList_SystemInfoUpdateRqst },
  {  82, &hf_nbap_id_NodeB_CommunicationContextID, ASN1_EXTENSION_ROOT    , dissect_nbap_NodeB_CommunicationContextID },
  {  83, &hf_nbap_id_NeighbouringCellMeasurementInformation, ASN1_EXTENSION_ROOT    , dissect_nbap_NeighbouringCellMeasurementInformation },
  {  84, &hf_nbap_id_P_CCPCH_Information, ASN1_EXTENSION_ROOT    , dissect_nbap_Common_PhysicalChannel_Status_Information },
  {  85, &hf_nbap_id_P_CPICH_Information, ASN1_EXTENSION_ROOT    , dissect_nbap_Common_PhysicalChannel_Status_Information },
  {  86, &hf_nbap_id_P_SCH_Information, ASN1_EXTENSION_ROOT    , dissect_nbap_Common_PhysicalChannel_Status_Information },
  {  87, &hf_nbap_id_PCCPCH_Information_Cell_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_PCCPCH_Information_Cell_ReconfRqstTDD },
  {  88, &hf_nbap_id_PCCPCH_Information_Cell_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_PCCPCH_Information_Cell_SetupRqstTDD },
  {  89, &hf_nbap_id_PCH_Parameters_CTCH_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_PCH_Parameters_CTCH_ReconfRqstTDD },
  {  90, &hf_nbap_id_PCH_ParametersItem_CTCH_SetupRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_PCH_ParametersItem_CTCH_SetupRqstFDD },
  {  91, &hf_nbap_id_PCH_ParametersItem_CTCH_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_PCH_ParametersItem_CTCH_SetupRqstTDD },
  {  92, &hf_nbap_id_PCH_Information, ASN1_EXTENSION_ROOT    , dissect_nbap_Common_TransportChannel_Status_Information },
  {  93, &hf_nbap_id_PDSCH_Information_AddListIE_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , dissect_nbap_PDSCH_Information_AddItem_PSCH_ReconfRqst },
  {  94, &hf_nbap_id_PDSCH_Information_ModifyListIE_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , dissect_nbap_PDSCH_Information_ModifyItem_PSCH_ReconfRqst },
  {  95, &hf_nbap_id_PDSCHSets_AddList_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , dissect_nbap_PDSCHSets_AddList_PSCH_ReconfRqst },
  {  96, &hf_nbap_id_PDSCHSets_DeleteList_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , dissect_nbap_PDSCHSets_DeleteList_PSCH_ReconfRqst },
  {  97, &hf_nbap_id_PDSCHSets_ModifyList_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , dissect_nbap_PDSCHSets_ModifyList_PSCH_ReconfRqst },
  {  98, &hf_nbap_id_PICH_Information, ASN1_EXTENSION_ROOT    , dissect_nbap_Common_PhysicalChannel_Status_Information },
  {  99, &hf_nbap_id_PICH_Parameters_CTCH_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_PICH_Parameters_CTCH_ReconfRqstTDD },
  { 100, &hf_nbap_id_PowerAdjustmentType, ASN1_EXTENSION_ROOT    , dissect_nbap_PowerAdjustmentType },
  { 101, &hf_nbap_id_PRACH_Information, ASN1_EXTENSION_ROOT    , dissect_nbap_Common_PhysicalChannel_Status_Information },
  { 102, &hf_nbap_id_PrimaryCCPCH_Information_Cell_ReconfRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_PrimaryCCPCH_Information_Cell_ReconfRqstFDD },
  { 103, &hf_nbap_id_PrimaryCCPCH_Information_Cell_SetupRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_PrimaryCCPCH_Information_Cell_SetupRqstFDD },
  { 104, &hf_nbap_id_PrimaryCPICH_Information_Cell_ReconfRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_PrimaryCPICH_Information_Cell_ReconfRqstFDD },
  { 105, &hf_nbap_id_PrimaryCPICH_Information_Cell_SetupRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_PrimaryCPICH_Information_Cell_SetupRqstFDD },
  { 106, &hf_nbap_id_PrimarySCH_Information_Cell_ReconfRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_PrimarySCH_Information_Cell_ReconfRqstFDD },
  { 107, &hf_nbap_id_PrimarySCH_Information_Cell_SetupRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_PrimarySCH_Information_Cell_SetupRqstFDD },
  { 108, &hf_nbap_id_PrimaryScramblingCode, ASN1_EXTENSION_ROOT    , dissect_nbap_PrimaryScramblingCode },
  { 109, &hf_nbap_id_SCH_Information_Cell_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_SCH_Information_Cell_ReconfRqstTDD },
  { 110, &hf_nbap_id_SCH_Information_Cell_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_SCH_Information_Cell_SetupRqstTDD },
  { 111, &hf_nbap_id_PUSCH_Information_AddListIE_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , dissect_nbap_PUSCH_Information_AddItem_PSCH_ReconfRqst },
  { 112, &hf_nbap_id_PUSCH_Information_ModifyListIE_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , dissect_nbap_PUSCH_Information_ModifyItem_PSCH_ReconfRqst },
  { 113, &hf_nbap_id_PUSCHSets_AddList_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , dissect_nbap_PUSCHSets_AddList_PSCH_ReconfRqst },
  { 114, &hf_nbap_id_PUSCHSets_DeleteList_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , dissect_nbap_PUSCHSets_DeleteList_PSCH_ReconfRqst },
  { 115, &hf_nbap_id_PUSCHSets_ModifyList_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , dissect_nbap_PUSCHSets_ModifyList_PSCH_ReconfRqst },
  { 116, &hf_nbap_id_RACH_Information, ASN1_EXTENSION_ROOT    , dissect_nbap_Common_TransportChannel_Status_Information },
  { 117, &hf_nbap_id_RACH_ParametersItem_CTCH_SetupRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_RACH_ParametersItem_CTCH_SetupRqstFDD },
  { 118, &hf_nbap_id_RACH_ParameterItem_CTCH_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_RACH_ParameterItem_CTCH_SetupRqstTDD },
  { 119, &hf_nbap_id_ReportCharacteristics, ASN1_EXTENSION_ROOT    , dissect_nbap_ReportCharacteristics },
  { 120, &hf_nbap_id_Reporting_Object_RL_FailureInd, ASN1_EXTENSION_ROOT    , dissect_nbap_Reporting_Object_RL_FailureInd },
  { 121, &hf_nbap_id_Reporting_Object_RL_RestoreInd, ASN1_EXTENSION_ROOT    , dissect_nbap_Reporting_Object_RL_RestoreInd },
  { 122, &hf_nbap_id_RL_InformationItem_DM_Rprt, ASN1_EXTENSION_ROOT    , dissect_nbap_RL_InformationItem_DM_Rprt },
  { 123, &hf_nbap_id_RL_InformationItem_DM_Rqst, ASN1_EXTENSION_ROOT    , dissect_nbap_RL_InformationItem_DM_Rqst },
  { 124, &hf_nbap_id_RL_InformationItem_DM_Rsp, ASN1_EXTENSION_ROOT    , dissect_nbap_RL_InformationItem_DM_Rsp },
  { 125, &hf_nbap_id_RL_InformationItem_RL_AdditionRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_RL_InformationItem_RL_AdditionRqstFDD },
  { 126, &hf_nbap_id_RL_informationItem_RL_DeletionRqst, ASN1_EXTENSION_ROOT    , dissect_nbap_RL_informationItem_RL_DeletionRqst },
  { 127, &hf_nbap_id_RL_InformationItem_RL_FailureInd, ASN1_EXTENSION_ROOT    , dissect_nbap_RL_InformationItem_RL_FailureInd },
  { 128, &hf_nbap_id_RL_InformationItem_RL_PreemptRequiredInd, ASN1_EXTENSION_ROOT    , dissect_nbap_RL_InformationItem_RL_PreemptRequiredInd },
  { 129, &hf_nbap_id_RL_InformationItem_RL_ReconfPrepFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_RL_InformationItem_RL_ReconfPrepFDD },
  { 130, &hf_nbap_id_RL_InformationItem_RL_ReconfRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_RL_InformationItem_RL_ReconfPrepFDD },
  { 131, &hf_nbap_id_RL_InformationItem_RL_RestoreInd, ASN1_EXTENSION_ROOT    , dissect_nbap_RL_InformationItem_RL_RestoreInd },
  { 132, &hf_nbap_id_RL_InformationItem_RL_SetupRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_RL_InformationItem_RL_SetupRqstFDD },
  { 133, &hf_nbap_id_RL_InformationList_RL_AdditionRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_RL_InformationList_RL_AdditionRqstFDD },
  { 134, &hf_nbap_id_RL_informationList_RL_DeletionRqst, ASN1_EXTENSION_ROOT    , dissect_nbap_RL_informationList_RL_DeletionRqst },
  { 135, &hf_nbap_id_RL_InformationList_RL_PreemptRequiredInd, ASN1_EXTENSION_ROOT    , dissect_nbap_RL_InformationList_RL_PreemptRequiredInd },
  { 136, &hf_nbap_id_RL_InformationList_RL_ReconfPrepFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_RL_InformationList_RL_ReconfPrepFDD },
  { 137, &hf_nbap_id_RL_InformationList_RL_ReconfRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_RL_InformationList_RL_ReconfRqstFDD },
  { 138, &hf_nbap_id_RL_InformationList_RL_SetupRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_RL_InformationList_RL_SetupRqstFDD },
  { 139, &hf_nbap_id_RL_InformationResponseItem_RL_AdditionRspFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_RL_InformationResponseItem_RL_AdditionRspFDD },
  { 140, &hf_nbap_id_RL_InformationResponseItem_RL_ReconfReady, ASN1_EXTENSION_ROOT    , dissect_nbap_RL_InformationResponseItem_RL_ReconfReady },
  { 141, &hf_nbap_id_RL_InformationResponseItem_RL_ReconfRsp, ASN1_EXTENSION_ROOT    , dissect_nbap_RL_InformationResponseItem_RL_ReconfRsp },
  { 142, &hf_nbap_id_RL_InformationResponseItem_RL_SetupRspFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_RL_InformationResponseItem_RL_SetupRspFDD },
  { 143, &hf_nbap_id_RL_InformationResponseList_RL_AdditionRspFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_RL_InformationResponseList_RL_AdditionRspFDD },
  { 144, &hf_nbap_id_RL_InformationResponseList_RL_ReconfReady, ASN1_EXTENSION_ROOT    , dissect_nbap_RL_InformationResponseList_RL_ReconfReady },
  { 145, &hf_nbap_id_RL_InformationResponseList_RL_ReconfRsp, ASN1_EXTENSION_ROOT    , dissect_nbap_RL_InformationResponseList_RL_ReconfRsp },
  { 146, &hf_nbap_id_RL_InformationResponseList_RL_SetupRspFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_RL_InformationResponseList_RL_SetupRspFDD },
  { 147, &hf_nbap_id_RL_InformationResponse_RL_AdditionRspTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_RL_InformationResponse_RL_AdditionRspTDD },
  { 148, &hf_nbap_id_RL_InformationResponse_RL_SetupRspTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_RL_InformationResponse_RL_SetupRspTDD },
  { 149, &hf_nbap_id_RL_Information_RL_AdditionRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_RL_Information_RL_AdditionRqstTDD },
  { 150, &hf_nbap_id_RL_Information_RL_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_RL_Information_RL_ReconfRqstTDD },
  { 151, &hf_nbap_id_RL_Information_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_RL_Information_RL_ReconfPrepTDD },
  { 152, &hf_nbap_id_RL_Information_RL_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_RL_Information_RL_SetupRqstTDD },
  { 153, &hf_nbap_id_RL_ReconfigurationFailureItem_RL_ReconfFailure, ASN1_EXTENSION_ROOT    , dissect_nbap_RL_ReconfigurationFailureItem_RL_ReconfFailure },
  { 154, &hf_nbap_id_RL_Set_InformationItem_DM_Rprt, ASN1_EXTENSION_ROOT    , dissect_nbap_RL_Set_InformationItem_DM_Rprt },
  { 155, &hf_nbap_id_RL_Set_InformationItem_DM_Rsp, ASN1_EXTENSION_ROOT    , dissect_nbap_RL_Set_InformationItem_DM_Rsp },
  { 156, &hf_nbap_id_RL_Set_InformationItem_RL_FailureInd, ASN1_EXTENSION_ROOT    , dissect_nbap_RL_Set_InformationItem_RL_FailureInd },
  { 157, &hf_nbap_id_RL_Set_InformationItem_RL_RestoreInd, ASN1_EXTENSION_ROOT    , dissect_nbap_RL_Set_InformationItem_RL_RestoreInd },
  { 158, &hf_nbap_id_S_CCPCH_Information, ASN1_EXTENSION_ROOT    , dissect_nbap_Common_PhysicalChannel_Status_Information },
  { 159, &hf_nbap_id_S_CPICH_Information, ASN1_EXTENSION_ROOT    , dissect_nbap_Common_PhysicalChannel_Status_Information },
  { 160, &hf_nbap_id_SCH_Information, ASN1_EXTENSION_ROOT    , dissect_nbap_Common_PhysicalChannel_Status_Information },
  { 161, &hf_nbap_id_S_SCH_Information, ASN1_EXTENSION_ROOT    , dissect_nbap_Common_PhysicalChannel_Status_Information },
  { 162, &hf_nbap_id_Secondary_CCPCHListIE_CTCH_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_Secondary_CCPCHListIE_CTCH_ReconfRqstTDD },
  { 163, &hf_nbap_id_Secondary_CCPCH_parameterListIE_CTCH_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_Secondary_CCPCH_parameterListIE_CTCH_SetupRqstTDD },
  { 164, &hf_nbap_id_Secondary_CCPCH_Parameters_CTCH_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_Secondary_CCPCH_Parameters_CTCH_ReconfRqstTDD },
  { 165, &hf_nbap_id_SecondaryCPICH_InformationItem_Cell_ReconfRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_SecondaryCPICH_InformationItem_Cell_ReconfRqstFDD },
  { 166, &hf_nbap_id_SecondaryCPICH_InformationItem_Cell_SetupRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_SecondaryCPICH_InformationItem_Cell_SetupRqstFDD },
  { 167, &hf_nbap_id_SecondaryCPICH_InformationList_Cell_ReconfRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_SecondaryCPICH_InformationList_Cell_ReconfRqstFDD },
  { 168, &hf_nbap_id_SecondaryCPICH_InformationList_Cell_SetupRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_SecondaryCPICH_InformationList_Cell_SetupRqstFDD },
  { 169, &hf_nbap_id_SecondarySCH_Information_Cell_ReconfRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_SecondarySCH_Information_Cell_ReconfRqstFDD },
  { 170, &hf_nbap_id_SecondarySCH_Information_Cell_SetupRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_SecondarySCH_Information_Cell_SetupRqstFDD },
  { 171, &hf_nbap_id_SegmentInformationListIE_SystemInfoUpdate, ASN1_EXTENSION_ROOT    , dissect_nbap_SegmentInformationListIE_SystemInfoUpdate },
  { 172, &hf_nbap_id_SFN         , ASN1_EXTENSION_ROOT    , dissect_nbap_SFN },
  { 173, &hf_nbap_id_SignallingBearerRequestIndicator, ASN1_EXTENSION_ROOT    , dissect_nbap_SignallingBearerRequestIndicator },
  { 174, &hf_nbap_id_ShutdownTimer, ASN1_EXTENSION_ROOT    , dissect_nbap_ShutdownTimer },
  { 175, &hf_nbap_id_Start_Of_Audit_Sequence_Indicator, ASN1_EXTENSION_ROOT    , dissect_nbap_Start_Of_Audit_Sequence_Indicator },
  { 176, &hf_nbap_id_Successful_RL_InformationRespItem_RL_AdditionFailureFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_Successful_RL_InformationRespItem_RL_AdditionFailureFDD },
  { 177, &hf_nbap_id_Successful_RL_InformationRespItem_RL_SetupFailureFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_Successful_RL_InformationRespItem_RL_SetupFailureFDD },
  { 178, &hf_nbap_id_SyncCase    , ASN1_EXTENSION_ROOT    , dissect_nbap_SyncCase },
  { 179, &hf_nbap_id_SyncCaseIndicatorItem_Cell_SetupRqstTDD_PSCH, ASN1_EXTENSION_ROOT    , dissect_nbap_SyncCaseIndicatorItem_Cell_SetupRqstTDD_PSCH },
  { 180, &hf_nbap_id_T_Cell      , ASN1_EXTENSION_ROOT    , dissect_nbap_T_Cell },
  { 181, &hf_nbap_id_TargetCommunicationControlPortID, ASN1_EXTENSION_ROOT    , dissect_nbap_CommunicationControlPortID },
  { 182, &hf_nbap_id_TimeSlotConfigurationList_Cell_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_TimeSlotConfigurationList_Cell_ReconfRqstTDD },
  { 183, &hf_nbap_id_TimeSlotConfigurationList_Cell_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_TimeSlotConfigurationList_Cell_SetupRqstTDD },
  { 184, &hf_nbap_id_TransmissionDiversityApplied, ASN1_EXTENSION_ROOT    , dissect_nbap_TransmissionDiversityApplied },
  { 185, &hf_nbap_id_TypeOfError , ASN1_EXTENSION_ROOT    , dissect_nbap_TypeOfError },
  { 186, &hf_nbap_id_UARFCNforNt , ASN1_EXTENSION_ROOT    , dissect_nbap_UARFCN },
  { 187, &hf_nbap_id_UARFCNforNd , ASN1_EXTENSION_ROOT    , dissect_nbap_UARFCN },
  { 188, &hf_nbap_id_UARFCNforNu , ASN1_EXTENSION_ROOT    , dissect_nbap_UARFCN },
  { 189, &hf_nbap_id_UL_CCTrCH_InformationItem_RL_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_UL_CCTrCH_InformationItem_RL_SetupRqstTDD },
  { 190, &hf_nbap_id_UL_CCTrCH_InformationList_RL_AdditionRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_UL_CCTrCH_InformationList_RL_SetupRqstTDD },
  { 191, &hf_nbap_id_UL_CCTrCH_InformationList_RL_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_UL_CCTrCH_InformationList_RL_SetupRqstTDD },
  { 192, &hf_nbap_id_UL_DPCH_InformationItem_RL_AdditionRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_UL_DPCH_InformationItem_RL_AdditionRqstTDD },
  { 193, &hf_nbap_id_UL_DPCH_InformationList_RL_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_UL_DPCH_InformationItem_RL_SetupRqstTDD },
  { 194, &hf_nbap_id_UL_DPCH_Information_RL_ReconfPrepFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_UL_DPCH_Information_RL_ReconfPrepFDD },
  { 195, &hf_nbap_id_UL_DPCH_Information_RL_ReconfRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_UL_DPCH_Information_RL_ReconfRqstFDD },
  { 196, &hf_nbap_id_UL_DPCH_Information_RL_SetupRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_UL_DPCH_Information_RL_SetupRqstFDD },
  { 197, &hf_nbap_id_Unsuccessful_RL_InformationRespItem_RL_AdditionFailureFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_Unsuccessful_RL_InformationRespItem_RL_AdditionFailureFDD },
  { 198, &hf_nbap_id_Unsuccessful_RL_InformationRespItem_RL_SetupFailureFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_Unsuccessful_RL_InformationRespItem_RL_SetupFailureFDD },
  { 199, &hf_nbap_id_Unsuccessful_RL_InformationResp_RL_AdditionFailureTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_Unsuccessful_RL_InformationResp_RL_AdditionFailureTDD },
  { 200, &hf_nbap_id_Unsuccessful_RL_InformationResp_RL_SetupFailureTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_Unsuccessful_RL_InformationResp_RL_SetupFailureTDD },
  { 201, &hf_nbap_id_USCH_Information_Add, ASN1_EXTENSION_ROOT    , dissect_nbap_USCH_Information },
  { 202, &hf_nbap_id_USCH_Information_DeleteList_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_USCH_Information_DeleteList_RL_ReconfPrepTDD },
  { 203, &hf_nbap_id_USCH_Information_ModifyList_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_USCH_Information_ModifyList_RL_ReconfPrepTDD },
  { 204, &hf_nbap_id_USCH_InformationResponse, ASN1_EXTENSION_ROOT    , dissect_nbap_USCH_InformationResponse },
  { 205, &hf_nbap_id_USCH_Information, ASN1_EXTENSION_ROOT    , dissect_nbap_USCH_Information },
  { 206, &hf_nbap_id_USCH_RearrangeList_Bearer_RearrangeInd, ASN1_EXTENSION_ROOT    , dissect_nbap_USCH_RearrangeList_Bearer_RearrangeInd },
  { 207, &hf_nbap_id_Active_Pattern_Sequence_Information, ASN1_EXTENSION_ROOT    , dissect_nbap_Active_Pattern_Sequence_Information },
  { 208, &hf_nbap_id_AICH_ParametersListIE_CTCH_ReconfRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_AICH_ParametersListIE_CTCH_ReconfRqstFDD },
  { 209, &hf_nbap_id_AdjustmentRatio, ASN1_EXTENSION_ROOT    , dissect_nbap_ScaledAdjustmentRatio },
  { 210, &hf_nbap_id_FACH_ParametersListIE_CTCH_ReconfRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_FACH_ParametersListIE_CTCH_ReconfRqstFDD },
  { 211, &hf_nbap_id_CauseLevel_PSCH_ReconfFailure, ASN1_EXTENSION_ROOT    , dissect_nbap_CauseLevel_PSCH_ReconfFailure },
  { 212, &hf_nbap_id_CauseLevel_RL_AdditionFailureFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_CauseLevel_RL_AdditionFailureFDD },
  { 213, &hf_nbap_id_CauseLevel_RL_AdditionFailureTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_CauseLevel_RL_AdditionFailureTDD },
  { 214, &hf_nbap_id_CauseLevel_RL_ReconfFailure, ASN1_EXTENSION_ROOT    , dissect_nbap_CauseLevel_RL_ReconfFailure },
  { 215, &hf_nbap_id_CauseLevel_RL_SetupFailureFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_CauseLevel_RL_SetupFailureFDD },
  { 216, &hf_nbap_id_CauseLevel_RL_SetupFailureTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_CauseLevel_RL_SetupFailureTDD },
  { 217, &hf_nbap_id_Closed_Loop_Timing_Adjustment_Mode, ASN1_EXTENSION_ROOT    , dissect_nbap_Closedlooptimingadjustmentmode },
  { 218, &hf_nbap_id_CommonPhysicalChannelType_CTCH_ReconfRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_CommonPhysicalChannelType_CTCH_ReconfRqstFDD },
  { 219, &hf_nbap_id_Compressed_Mode_Deactivation_Flag, ASN1_EXTENSION_ROOT    , dissect_nbap_Compressed_Mode_Deactivation_Flag },
  { 220, &hf_nbap_id_DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD },
  { 221, &hf_nbap_id_DL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD },
  { 222, &hf_nbap_id_DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD },
  { 223, &hf_nbap_id_DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD },
  { 224, &hf_nbap_id_DL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD },
  { 225, &hf_nbap_id_DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD },
  { 226, &hf_nbap_id_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD },
  { 227, &hf_nbap_id_DL_DPCH_InformationAddListIE_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_DPCH_InformationAddItem_RL_ReconfPrepTDD },
  { 228, &hf_nbap_id_DL_DPCH_InformationModify_AddListIE_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_DPCH_InformationModify_AddItem_RL_ReconfPrepTDD },
  { 229, &hf_nbap_id_DL_DPCH_InformationModify_DeleteListIE_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_DPCH_InformationModify_DeleteItem_RL_ReconfPrepTDD },
  { 230, &hf_nbap_id_DL_DPCH_InformationModify_ModifyListIE_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_DPCH_InformationModify_ModifyItem_RL_ReconfPrepTDD },
  { 231, &hf_nbap_id_DL_TPC_Pattern01Count, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_TPC_Pattern01Count },
  { 232, &hf_nbap_id_DPC_Mode    , ASN1_EXTENSION_ROOT    , dissect_nbap_DPC_Mode },
  { 233, &hf_nbap_id_DPCHConstant, ASN1_EXTENSION_ROOT    , dissect_nbap_ConstantValue },
  { 234, &hf_nbap_id_FACH_ParametersList_CTCH_SetupRsp, ASN1_EXTENSION_ROOT    , dissect_nbap_FACH_CommonTransportChannel_InformationResponse },
  { 235, &hf_nbap_id_Limited_power_increase_information_Cell_SetupRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_Limited_power_increase_information_Cell_SetupRqstFDD },
  { 236, &hf_nbap_id_PCH_Parameters_CTCH_SetupRsp, ASN1_EXTENSION_ROOT    , dissect_nbap_CommonTransportChannel_InformationResponse },
  { 237, &hf_nbap_id_PCH_ParametersItem_CTCH_ReconfRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_PCH_ParametersItem_CTCH_ReconfRqstFDD },
  { 238, &hf_nbap_id_PICH_ParametersItem_CTCH_ReconfRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_PICH_ParametersItem_CTCH_ReconfRqstFDD },
  { 239, &hf_nbap_id_PRACHConstant, ASN1_EXTENSION_ROOT    , dissect_nbap_ConstantValue },
  { 240, &hf_nbap_id_PRACH_ParametersListIE_CTCH_ReconfRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_PRACH_ParametersListIE_CTCH_ReconfRqstFDD },
  { 241, &hf_nbap_id_PUSCHConstant, ASN1_EXTENSION_ROOT    , dissect_nbap_ConstantValue },
  { 242, &hf_nbap_id_RACH_Parameters_CTCH_SetupRsp, ASN1_EXTENSION_ROOT    , dissect_nbap_CommonTransportChannel_InformationResponse },
  { 243, &hf_nbap_id_Synchronisation_Configuration_Cell_ReconfRqst, ASN1_EXTENSION_ROOT    , dissect_nbap_Synchronisation_Configuration_Cell_ReconfRqst },
  { 244, &hf_nbap_id_Synchronisation_Configuration_Cell_SetupRqst, ASN1_EXTENSION_ROOT    , dissect_nbap_Synchronisation_Configuration_Cell_SetupRqst },
  { 245, &hf_nbap_id_Transmission_Gap_Pattern_Sequence_Information, ASN1_EXTENSION_ROOT    , dissect_nbap_Transmission_Gap_Pattern_Sequence_Information },
  { 246, &hf_nbap_id_UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD },
  { 247, &hf_nbap_id_UL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_UL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD },
  { 248, &hf_nbap_id_UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD },
  { 249, &hf_nbap_id_UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD },
  { 250, &hf_nbap_id_UL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_UL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD },
  { 251, &hf_nbap_id_UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD },
  { 252, &hf_nbap_id_UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD },
  { 253, &hf_nbap_id_UL_DPCH_InformationAddListIE_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_UL_DPCH_InformationAddItem_RL_ReconfPrepTDD },
  { 254, &hf_nbap_id_UL_DPCH_InformationModify_AddListIE_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_UL_DPCH_InformationModify_AddItem_RL_ReconfPrepTDD },
  { 255, &hf_nbap_id_UL_DPCH_InformationModify_DeleteListIE_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_UL_DPCH_InformationModify_DeleteListIE_RL_ReconfPrepTDD },
  { 256, &hf_nbap_id_UL_DPCH_InformationModify_ModifyListIE_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_UL_DPCH_InformationModify_ModifyItem_RL_ReconfPrepTDD },
  { 257, &hf_nbap_id_Unsuccessful_PDSCHSetItem_PSCH_ReconfFailureTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_Unsuccessful_PDSCHSetItem_PSCH_ReconfFailureTDD },
  { 258, &hf_nbap_id_Unsuccessful_PUSCHSetItem_PSCH_ReconfFailureTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_Unsuccessful_PUSCHSetItem_PSCH_ReconfFailureTDD },
  { 259, &hf_nbap_id_CommunicationContextInfoItem_Reset, ASN1_EXTENSION_ROOT    , dissect_nbap_CommunicationContextInfoItem_Reset },
  { 260, &hf_nbap_id_CommunicationControlPortInfoItem_Reset, ASN1_EXTENSION_ROOT    , dissect_nbap_CommunicationControlPortInfoItem_Reset },
  { 261, &hf_nbap_id_ResetIndicator, ASN1_EXTENSION_ROOT    , dissect_nbap_ResetIndicator },
  { 262, &hf_nbap_id_TimingAdvanceApplied, ASN1_EXTENSION_ROOT    , dissect_nbap_TimingAdvanceApplied },
  { 263, &hf_nbap_id_CFNReportingIndicator, ASN1_EXTENSION_ROOT    , dissect_nbap_FNReportingIndicator },
  { 264, &hf_nbap_id_SFNReportingIndicator, ASN1_EXTENSION_ROOT    , dissect_nbap_FNReportingIndicator },
  { 265, &hf_nbap_id_InnerLoopDLPCStatus, ASN1_EXTENSION_ROOT    , dissect_nbap_InnerLoopDLPCStatus },
  { 266, &hf_nbap_id_TimeslotISCPInfo, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_TimeslotISCPInfo },
  { 267, &hf_nbap_id_PICH_ParametersItem_CTCH_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_PICH_ParametersItem_CTCH_SetupRqstTDD },
  { 268, &hf_nbap_id_PRACH_ParametersItem_CTCH_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_PRACH_ParametersItem_CTCH_SetupRqstTDD },
  { 269, &hf_nbap_id_CCTrCH_InformationItem_RL_FailureInd, ASN1_EXTENSION_ROOT    , dissect_nbap_CCTrCH_InformationItem_RL_FailureInd },
  { 270, &hf_nbap_id_CCTrCH_InformationItem_RL_RestoreInd, ASN1_EXTENSION_ROOT    , dissect_nbap_CCTrCH_InformationItem_RL_RestoreInd },
  { 271, &hf_nbap_id_CauseLevel_SyncAdjustmntFailureTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_CauseLevel_SyncAdjustmntFailureTDD },
  { 272, &hf_nbap_id_CellAdjustmentInfo_SyncAdjustmntRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_CellAdjustmentInfo_SyncAdjustmentRqstTDD },
  { 273, &hf_nbap_id_CellAdjustmentInfoItem_SyncAdjustmentRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_CellAdjustmentInfoItem_SyncAdjustmentRqstTDD },
  { 274, &hf_nbap_id_CellSyncBurstInfoList_CellSyncReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_CellSyncBurstInfoList_CellSyncReconfRqstTDD },
  { 275, &hf_nbap_id_CellSyncBurstTransInit_CellSyncInitiationRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_CellSyncBurstTransInit_CellSyncInitiationRqstTDD },
  { 276, &hf_nbap_id_CellSyncBurstMeasureInit_CellSyncInitiationRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_CellSyncBurstMeasureInit_CellSyncInitiationRqstTDD },
  { 277, &hf_nbap_id_CellSyncBurstMeasReconfiguration_CellSyncReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_CellSyncBurstMeasInfo_CellSyncReconfRqstTDD },
  { 278, &hf_nbap_id_CellSyncBurstMeasInfoList_CellSyncReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_CellSyncBurstMeasInfoList_CellSyncReconfRqstTDD },
  { 279, &hf_nbap_id_CellSyncBurstTransReconfInfo_CellSyncReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_CellSyncBurstTransReconfInfo_CellSyncReconfRqstTDD },
  { 280, &hf_nbap_id_CellSyncInfo_CellSyncReprtTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_CellSyncInfo_CellSyncReprtTDD },
  { 281, &hf_nbap_id_CSBTransmissionID, ASN1_EXTENSION_ROOT    , dissect_nbap_CSBTransmissionID },
  { 282, &hf_nbap_id_CSBMeasurementID, ASN1_EXTENSION_ROOT    , dissect_nbap_CSBMeasurementID },
  { 283, &hf_nbap_id_NCyclesPerSFNperiod, ASN1_EXTENSION_ROOT    , dissect_nbap_NCyclesPerSFNperiod },
  { 284, &hf_nbap_id_NRepetitionsPerCyclePeriod, ASN1_EXTENSION_ROOT    , dissect_nbap_NRepetitionsPerCyclePeriod },
  { 285, &hf_nbap_id_SyncFrameNumber, ASN1_EXTENSION_ROOT    , dissect_nbap_SyncFrameNumber },
  { 286, &hf_nbap_id_SynchronisationReportType, ASN1_EXTENSION_ROOT    , dissect_nbap_SynchronisationReportType },
  { 287, &hf_nbap_id_SynchronisationReportCharacteristics, ASN1_EXTENSION_ROOT    , dissect_nbap_SynchronisationReportCharacteristics },
  { 288, &hf_nbap_id_Unsuccessful_cell_InformationRespItem_SyncAdjustmntFailureTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_Unsuccessful_cell_InformationRespItem_SyncAdjustmntFailureTDD },
  { 289, &hf_nbap_id_ReferenceClockAvailability, ASN1_EXTENSION_ROOT    , dissect_nbap_ReferenceClockAvailability },
  { 290, &hf_nbap_id_ReferenceSFNoffset, ASN1_EXTENSION_ROOT    , dissect_nbap_ReferenceSFNoffset },
  { 291, &hf_nbap_id_InformationExchangeID, ASN1_EXTENSION_ROOT    , dissect_nbap_InformationExchangeID },
  { 292, &hf_nbap_id_InformationExchangeObjectType_InfEx_Rqst, ASN1_EXTENSION_ROOT    , dissect_nbap_InformationExchangeObjectType_InfEx_Rqst },
  { 293, &hf_nbap_id_InformationType, ASN1_EXTENSION_ROOT    , dissect_nbap_InformationReportCharacteristics },
  { 294, &hf_nbap_id_InformationReportCharacteristics, ASN1_EXTENSION_ROOT    , dissect_nbap_InformationReportCharacteristics },
  { 295, &hf_nbap_id_InformationExchangeObjectType_InfEx_Rsp, ASN1_EXTENSION_ROOT    , dissect_nbap_InformationExchangeObjectType_InfEx_Rsp },
  { 296, &hf_nbap_id_InformationExchangeObjectType_InfEx_Rprt, ASN1_EXTENSION_ROOT    , dissect_nbap_InformationExchangeObjectType_InfEx_Rprt },
  { 297, &hf_nbap_id_IPDLParameter_Information_Cell_ReconfRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_IPDLParameter_Information_Cell_ReconfRqstFDD },
  { 298, &hf_nbap_id_IPDLParameter_Information_Cell_SetupRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_IPDLParameter_Information_Cell_SetupRqstFDD },
  { 299, &hf_nbap_id_IPDLParameter_Information_Cell_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_IPDLParameter_Information_Cell_SetupRqstTDD },
  { 300, &hf_nbap_id_IPDLParameter_Information_Cell_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_IPDLParameter_Information_Cell_SetupRqstTDD },
  { 301, &hf_nbap_id_DL_DPCH_LCR_Information_RL_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_DPCH_LCR_Information_RL_SetupRqstTDD },
  { 302, &hf_nbap_id_DwPCH_LCR_Information, ASN1_EXTENSION_ROOT    , dissect_nbap_Common_PhysicalChannel_Status_Information },
  { 303, &hf_nbap_id_DwPCH_LCR_InformationList_AuditRsp, ASN1_EXTENSION_ROOT    , dissect_nbap_Common_PhysicalChannel_Status_Information },
  { 304, &hf_nbap_id_DwPCH_LCR_Information_Cell_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DwPCH_LCR_Information_Cell_SetupRqstTDD },
  { 305, &hf_nbap_id_DwPCH_LCR_Information_Cell_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DwPCH_LCR_Information_Cell_ReconfRqstTDD },
  { 306, &hf_nbap_id_DwPCH_LCR_Information_ResourceStatusInd, ASN1_EXTENSION_ROOT    , dissect_nbap_DwPCH_LCR_Information_ResourceStatusInd },
  { 307, &hf_nbap_id_maxFACH_Power_LCR_CTCH_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_Power },
  { 308, &hf_nbap_id_maxFACH_Power_LCR_CTCH_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_Power },
  { 309, &hf_nbap_id_FPACH_LCR_Information, ASN1_EXTENSION_ROOT    , dissect_nbap_Common_PhysicalChannel_Status_Information },
  { 310, &hf_nbap_id_FPACH_LCR_Information_AuditRsp, ASN1_EXTENSION_ROOT    , dissect_nbap_Common_PhysicalChannel_Status_Information },
  { 311, &hf_nbap_id_FPACH_LCR_InformationList_AuditRsp, ASN1_EXTENSION_ROOT    , dissect_nbap_FPACH_LCR_InformationList_AuditRsp },
  { 312, &hf_nbap_id_FPACH_LCR_InformationList_ResourceStatusInd, ASN1_EXTENSION_ROOT    , dissect_nbap_FPACH_LCR_InformationList_ResourceStatusInd },
  { 313, &hf_nbap_id_FPACH_LCR_Parameters_CTCH_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_FPACH_LCR_Parameters_CTCH_SetupRqstTDD },
  { 314, &hf_nbap_id_FPACH_LCR_Parameters_CTCH_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_FPACH_LCR_Parameters_CTCH_ReconfRqstTDD },
  { 315, &hf_nbap_id_PCCPCH_LCR_Information_Cell_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_PCCPCH_LCR_Information_Cell_SetupRqstTDD },
  { 316, &hf_nbap_id_PCH_Power_LCR_CTCH_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_Power },
  { 317, &hf_nbap_id_PCH_Power_LCR_CTCH_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_Power },
  { 318, &hf_nbap_id_PICH_LCR_Parameters_CTCH_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_PICH_LCR_Parameters_CTCH_SetupRqstTDD },
  { 319, &hf_nbap_id_PRACH_LCR_ParametersList_CTCH_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_PRACH_LCR_ParametersList_CTCH_SetupRqstTDD },
  { 320, &hf_nbap_id_RL_InformationResponse_LCR_RL_SetupRspTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_RL_InformationResponse_LCR_RL_SetupRspTDD },
  { 321, &hf_nbap_id_Secondary_CCPCH_LCR_parameterList_CTCH_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_Secondary_CCPCH_LCR_parameterList_CTCH_SetupRqstTDD },
  { 322, &hf_nbap_id_TimeSlot    , ASN1_EXTENSION_ROOT    , dissect_nbap_TimeSlot },
  { 323, &hf_nbap_id_TimeSlotConfigurationList_LCR_Cell_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_TimeSlotConfigurationList_LCR_Cell_ReconfRqstTDD },
  { 324, &hf_nbap_id_TimeSlotConfigurationList_LCR_Cell_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_TimeSlotConfigurationList_LCR_Cell_SetupRqstTDD },
  { 325, &hf_nbap_id_TimeslotISCP_LCR_InfoList_RL_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_TimeslotISCPInfoLCR },
  { 326, &hf_nbap_id_TimeSlotLCR_CM_Rqst, ASN1_EXTENSION_ROOT    , dissect_nbap_TimeSlotLCR },
  { 327, &hf_nbap_id_UL_DPCH_LCR_Information_RL_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_UL_DPCH_LCR_Information_RL_SetupRqstTDD },
  { 328, &hf_nbap_id_DL_DPCH_InformationItem_LCR_RL_AdditionRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_DPCH_InformationItem_LCR_RL_AdditionRqstTDD },
  { 329, &hf_nbap_id_UL_DPCH_InformationItem_LCR_RL_AdditionRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_UL_DPCH_InformationItem_LCR_RL_AdditionRqstTDD },
  { 330, &hf_nbap_id_TimeslotISCP_InformationList_LCR_RL_AdditionRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_TimeslotISCPInfoLCR },
  { 331, &hf_nbap_id_DL_DPCH_LCR_InformationAddList_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_DPCH_LCR_InformationAddList_RL_ReconfPrepTDD },
  { 332, &hf_nbap_id_DL_DPCH_LCR_InformationModify_AddList_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_DPCH_LCR_InformationModify_AddList_RL_ReconfPrepTDD },
  { 333, &hf_nbap_id_DL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD },
  { 334, &hf_nbap_id_TimeslotISCPInfoList_LCR_DL_PC_RqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_TimeslotISCPInfoLCR },
  { 335, &hf_nbap_id_UL_DPCH_LCR_InformationAddListIE_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_UL_DPCH_LCR_InformationAddList_RL_ReconfPrepTDD },
  { 336, &hf_nbap_id_UL_DPCH_LCR_InformationModify_AddList, ASN1_EXTENSION_ROOT    , dissect_nbap_UL_DPCH_LCR_InformationModify_AddList_RL_ReconfPrepTDD },
  { 337, &hf_nbap_id_UL_TimeslotLCR_Information_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_UL_TimeslotLCR_InformationModify_ModifyList_RL_ReconfPrepTDD },
  { 338, &hf_nbap_id_UL_SIRTarget, ASN1_EXTENSION_ROOT    , dissect_nbap_UL_SIR },
  { 339, &hf_nbap_id_PDSCH_AddInformation_LCR_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , dissect_nbap_PDSCH_AddInformation_LCR_AddItem_PSCH_ReconfRqst },
  { 340, &hf_nbap_id_PDSCH_ModifyInformation_LCR_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , dissect_nbap_PDSCH_ModifyInformation_LCR_ModifyItem_PSCH_ReconfRqst },
  { 341, &hf_nbap_id_PUSCH_AddInformation_LCR_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , dissect_nbap_PUSCH_AddInformation_LCR_AddItem_PSCH_ReconfRqst },
  { 342, &hf_nbap_id_PUSCH_ModifyInformation_LCR_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , dissect_nbap_PUSCH_ModifyInformation_LCR_ModifyItem_PSCH_ReconfRqst },
  { 343, &hf_nbap_id_timeslotInfo_CellSyncInitiationRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_TimeslotInfo_CellSyncInitiationRqstTDD },
  { 344, &hf_nbap_id_SyncReportType_CellSyncReprtTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_SyncReportType_CellSyncReprtTDD },
  { 345, &hf_nbap_id_Power_Local_Cell_Group_InformationItem_AuditRsp, ASN1_EXTENSION_ROOT    , dissect_nbap_Power_Local_Cell_Group_InformationItem_AuditRsp },
  { 346, &hf_nbap_id_Power_Local_Cell_Group_InformationItem_ResourceStatusInd, ASN1_EXTENSION_ROOT    , dissect_nbap_Power_Local_Cell_Group_InformationItem_ResourceStatusInd },
  { 347, &hf_nbap_id_Power_Local_Cell_Group_InformationItem2_ResourceStatusInd, ASN1_EXTENSION_ROOT    , dissect_nbap_Power_Local_Cell_Group_InformationItem2_ResourceStatusInd },
  { 348, &hf_nbap_id_Power_Local_Cell_Group_InformationList_AuditRsp, ASN1_EXTENSION_ROOT    , dissect_nbap_Power_Local_Cell_Group_InformationList_AuditRsp },
  { 349, &hf_nbap_id_Power_Local_Cell_Group_InformationList_ResourceStatusInd, ASN1_EXTENSION_ROOT    , dissect_nbap_Power_Local_Cell_Group_InformationList_ResourceStatusInd },
  { 350, &hf_nbap_id_Power_Local_Cell_Group_InformationList2_ResourceStatusInd, ASN1_EXTENSION_ROOT    , dissect_nbap_Power_Local_Cell_Group_InformationList2_ResourceStatusInd },
  { 351, &hf_nbap_id_Power_Local_Cell_Group_ID, ASN1_EXTENSION_ROOT    , dissect_nbap_Local_Cell_ID },
  { 352, &hf_nbap_id_PUSCH_Info_DM_Rqst, ASN1_EXTENSION_ROOT    , dissect_nbap_PUSCH_Info_DM_Rqst },
  { 353, &hf_nbap_id_PUSCH_Info_DM_Rsp, ASN1_EXTENSION_ROOT    , dissect_nbap_PUSCH_Info_DM_Rsp },
  { 354, &hf_nbap_id_PUSCH_Info_DM_Rprt, ASN1_EXTENSION_ROOT    , dissect_nbap_PUSCH_Info_DM_Rprt },
  { 355, &hf_nbap_id_InitDL_Power, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_Power },
  { 356, &hf_nbap_id_cellSyncBurstRepetitionPeriod, ASN1_EXTENSION_ROOT    , dissect_nbap_CellSyncBurstRepetitionPeriod },
  { 357, &hf_nbap_id_ReportCharacteristicsType_OnModification, ASN1_EXTENSION_ROOT    , dissect_nbap_ReportCharacteristicsType_OnModification },
  { 358, &hf_nbap_id_SFNSFNMeasurementValueInformation, ASN1_EXTENSION_ROOT    , dissect_nbap_SFNSFNMeasurementValueInformation },
  { 359, &hf_nbap_id_SFNSFNMeasurementThresholdInformation, ASN1_EXTENSION_ROOT    , dissect_nbap_SFNSFNMeasurementThresholdInformation },
  { 360, &hf_nbap_id_TUTRANGPSMeasurementValueInformation, ASN1_EXTENSION_ROOT    , dissect_nbap_TUTRANGPSMeasurementValueInformation },
  { 361, &hf_nbap_id_TUTRANGPSMeasurementThresholdInformation, ASN1_EXTENSION_ROOT    , dissect_nbap_TUTRANGPSMeasurementThresholdInformation },
  { 362, &hf_nbap_id_Rx_Timing_Deviation_Value_LCR, ASN1_EXTENSION_ROOT    , dissect_nbap_Rx_Timing_Deviation_Value_LCR },
  { 363, &hf_nbap_id_RL_InformationResponse_LCR_RL_AdditionRspTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_RL_InformationResponse_LCR_RL_AdditionRspTDD },
  { 364, &hf_nbap_id_DL_PowerBalancing_Information, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_PowerBalancing_Information },
  { 365, &hf_nbap_id_DL_PowerBalancing_ActivationIndicator, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_PowerBalancing_ActivationIndicator },
  { 366, &hf_nbap_id_DL_PowerBalancing_UpdatedIndicator, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_PowerBalancing_UpdatedIndicator },
  { 367, &hf_nbap_id_CCTrCH_Initial_DL_Power_RL_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_Power },
  { 368, &hf_nbap_id_CCTrCH_Initial_DL_Power_RL_AdditionRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_Power },
  { 369, &hf_nbap_id_CCTrCH_Initial_DL_Power_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_Power },
  { 370, &hf_nbap_id_IPDLParameter_Information_LCR_Cell_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_IPDLParameter_Information_LCR_Cell_SetupRqstTDD },
  { 371, &hf_nbap_id_IPDLParameter_Information_LCR_Cell_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_IPDLParameter_Information_LCR_Cell_ReconfRqstTDD },
  { 372, &hf_nbap_id_HS_PDSCH_HS_SCCH_E_AGCH_E_RGCH_E_HICH_MaxPower_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , dissect_nbap_MaximumTransmissionPower },
  { 373, &hf_nbap_id_HS_PDSCH_HS_SCCH_ScramblingCode_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_ScramblingCode },
  { 374, &hf_nbap_id_HS_PDSCH_FDD_Code_Information_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , dissect_nbap_HS_PDSCH_FDD_Code_Information },
  { 375, &hf_nbap_id_HS_SCCH_FDD_Code_Information_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , dissect_nbap_HS_SCCH_FDD_Code_Information },
  { 376, &hf_nbap_id_HS_PDSCH_TDD_Information_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , dissect_nbap_HS_PDSCH_TDD_Information_PSCH_ReconfRqst },
  { 377, &hf_nbap_id_Add_To_HS_SCCH_Resource_Pool_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , dissect_nbap_Add_To_HS_SCCH_Resource_Pool_PSCH_ReconfRqst },
  { 378, &hf_nbap_id_Modify_HS_SCCH_Resource_Pool_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , dissect_nbap_Modify_HS_SCCH_Resource_Pool_PSCH_ReconfRqst },
  { 379, &hf_nbap_id_Delete_From_HS_SCCH_Resource_Pool_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , dissect_nbap_Delete_From_HS_SCCH_Resource_Pool_PSCH_ReconfRqst },
  { 380, &hf_nbap_id_bindingID   , ASN1_EXTENSION_ROOT    , dissect_nbap_BindingID },
  { 381, &hf_nbap_id_RL_Specific_DCH_Info, ASN1_EXTENSION_ROOT    , dissect_nbap_RL_Specific_DCH_Info },
  { 382, &hf_nbap_id_transportlayeraddress, ASN1_EXTENSION_ROOT    , dissect_nbap_TransportLayerAddress },
  { 383, &hf_nbap_id_DelayedActivation, ASN1_EXTENSION_ROOT    , dissect_nbap_DelayedActivation },
  { 384, &hf_nbap_id_DelayedActivationList_RL_ActivationCmdFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DelayedActivationInformationList_RL_ActivationCmdFDD },
  { 385, &hf_nbap_id_DelayedActivationInformation_RL_ActivationCmdFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DelayedActivationInformation_RL_ActivationCmdFDD },
  { 386, &hf_nbap_id_DelayedActivationList_RL_ActivationCmdTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DelayedActivationInformationList_RL_ActivationCmdTDD },
  { 387, &hf_nbap_id_DelayedActivationInformation_RL_ActivationCmdTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DelayedActivationInformation_RL_ActivationCmdTDD },
  { 388, &hf_nbap_id_neighbouringTDDCellMeasurementInformationLCR, ASN1_EXTENSION_ROOT    , dissect_nbap_NeighbouringTDDCellMeasurementInformationLCR },
  { 389, &hf_nbap_id_SYNCDlCodeId_TransInitLCR_CellSyncInitiationRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_SYNCDlCodeId_TransInitLCR_CellSyncInitiationRqstTDD },
  { 390, &hf_nbap_id_SYNCDlCodeId_MeasureInitLCR_CellSyncInitiationRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_SYNCDlCodeId_MeasureInitLCR_CellSyncInitiationRqstTDD },
  { 391, &hf_nbap_id_SYNCDlCodeIdTransReconfInfoLCR_CellSyncReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_SYNCDlCodeIdTransReconfInfoLCR_CellSyncReconfRqstTDD },
  { 392, &hf_nbap_id_SYNCDlCodeIdMeasReconfigurationLCR_CellSyncReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_SYNCDlCodeIdMeasInfoLCR_CellSyncReconfRqstTDD },
  { 393, &hf_nbap_id_SYNCDlCodeIdMeasInfoList_CellSyncReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_SYNCDlCodeIdMeasInfoList_CellSyncReconfRqstTDD },
  { 394, &hf_nbap_id_SyncDLCodeIdsMeasInfoList_CellSyncReprtTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_SyncDLCodeIdsMeasInfoList_CellSyncReprtTDD },
  { 395, &hf_nbap_id_SyncDLCodeIdThreInfoLCR, ASN1_EXTENSION_ROOT    , dissect_nbap_SyncDLCodeIdThreInfoLCR },
  { 396, &hf_nbap_id_NSubCyclesPerCyclePeriod_CellSyncReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_NSubCyclesPerCyclePeriod },
  { 397, &hf_nbap_id_DwPCH_Power , ASN1_EXTENSION_ROOT    , dissect_nbap_DwPCH_Power },
  { 398, &hf_nbap_id_AccumulatedClockupdate_CellSyncReprtTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_TimingAdjustmentValue },
  { 399, &hf_nbap_id_Angle_Of_Arrival_Value_LCR, ASN1_EXTENSION_ROOT    , dissect_nbap_Angle_Of_Arrival_Value_LCR },
  { 400, &hf_nbap_id_HSDSCH_FDD_Information, ASN1_EXTENSION_ROOT    , dissect_nbap_HSDSCH_FDD_Information },
  { 401, &hf_nbap_id_HSDSCH_FDD_Information_Response, ASN1_EXTENSION_ROOT    , dissect_nbap_HSDSCH_FDD_Information_Response },
  { 402, &hf_nbap_id_HSDSCH_Information_to_Modify, ASN1_EXTENSION_ROOT    , dissect_nbap_HSDSCH_Information_to_Modify },
  { 403, &hf_nbap_id_HSDSCH_RNTI , ASN1_EXTENSION_ROOT    , dissect_nbap_HSDSCH_RNTI },
  { 404, &hf_nbap_id_HSDSCH_TDD_Information, ASN1_EXTENSION_ROOT    , dissect_nbap_HSDSCH_TDD_Information },
  { 405, &hf_nbap_id_HSDSCH_TDD_Information_Response, ASN1_EXTENSION_ROOT    , dissect_nbap_HSDSCH_TDD_Information_Response },
  { 406, &hf_nbap_id_HSPDSCH_RL_ID, ASN1_EXTENSION_ROOT    , dissect_nbap_RL_ID },
  { 407, &hf_nbap_id_PrimCCPCH_RSCP_DL_PC_RqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_PrimaryCCPCH_RSCP },
  { 408, &hf_nbap_id_PDSCH_RL_ID , ASN1_EXTENSION_ROOT    , dissect_nbap_RL_ID },
  { 409, &hf_nbap_id_HSDSCH_RearrangeList_Bearer_RearrangeInd, ASN1_EXTENSION_ROOT    , dissect_nbap_HSDSCH_RearrangeList_Bearer_RearrangeInd },
  { 410, &hf_nbap_id_UL_Synchronisation_Parameters_LCR, ASN1_EXTENSION_ROOT    , dissect_nbap_UL_Synchronisation_Parameters_LCR },
  { 411, &hf_nbap_id_HSDSCH_FDD_Update_Information, ASN1_EXTENSION_ROOT    , dissect_nbap_HSDSCH_FDD_Update_Information },
  { 412, &hf_nbap_id_HSDSCH_TDD_Update_Information, ASN1_EXTENSION_ROOT    , dissect_nbap_HSDSCH_TDD_Update_Information },
  { 413, &hf_nbap_id_DL_DPCH_TimeSlotFormat_LCR_ModifyItem_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_TDD_DL_DPCH_TimeSlotFormat_LCR },
  { 414, &hf_nbap_id_UL_DPCH_TimeSlotFormat_LCR_ModifyItem_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_TDD_UL_DPCH_TimeSlotFormat_LCR },
  { 415, &hf_nbap_id_TDD_TPC_UplinkStepSize_LCR_RL_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_TDD_TPC_UplinkStepSize_LCR },
  { 416, &hf_nbap_id_TDD_TPC_UplinkStepSize_LCR_RL_AdditionRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_TDD_TPC_UplinkStepSize_LCR },
  { 417, &hf_nbap_id_TDD_TPC_DownlinkStepSize_RL_AdditionRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_TDD_TPC_DownlinkStepSize },
  { 418, &hf_nbap_id_TDD_TPC_UplinkStepSize_InformationAdd_LCR_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_TDD_TPC_UplinkStepSize_LCR },
  { 419, &hf_nbap_id_TDD_TPC_UplinkStepSize_InformationModify_LCR_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_TDD_TPC_UplinkStepSize_LCR },
  { 420, &hf_nbap_id_TDD_TPC_DownlinkStepSize_InformationModify_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_TDD_TPC_DownlinkStepSize },
  { 421, &hf_nbap_id_TDD_TPC_DownlinkStepSize_InformationAdd_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_TDD_TPC_DownlinkStepSize },
  { 422, &hf_nbap_id_CCTrCH_Maximum_DL_Power_RL_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_Power },
  { 423, &hf_nbap_id_CCTrCH_Minimum_DL_Power_RL_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_Power },
  { 424, &hf_nbap_id_CCTrCH_Maximum_DL_Power_RL_AdditionRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_Power },
  { 425, &hf_nbap_id_CCTrCH_Minimum_DL_Power_RL_AdditionRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_Power },
  { 426, &hf_nbap_id_CCTrCH_Maximum_DL_Power_InformationAdd_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_Power },
  { 427, &hf_nbap_id_CCTrCH_Minimum_DL_Power_InformationAdd_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_Power },
  { 428, &hf_nbap_id_CCTrCH_Maximum_DL_Power_InformationModify_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_Power },
  { 429, &hf_nbap_id_CCTrCH_Minimum_DL_Power_InformationModify_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_Power },
  { 430, &hf_nbap_id_Maximum_DL_Power_Modify_LCR_InformationModify_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_Power },
  { 431, &hf_nbap_id_Minimum_DL_Power_Modify_LCR_InformationModify_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_Power },
  { 432, &hf_nbap_id_DL_DPCH_LCR_InformationModify_ModifyList_RL_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_DPCH_LCR_InformationModify_ModifyList_RL_ReconfRqstTDD },
  { 433, &hf_nbap_id_CCTrCH_Maximum_DL_Power_InformationModify_RL_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_Power },
  { 434, &hf_nbap_id_CCTrCH_Minimum_DL_Power_InformationModify_RL_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_Power },
  { 435, &hf_nbap_id_Initial_DL_Power_TimeslotLCR_InformationItem, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_Power },
  { 436, &hf_nbap_id_Maximum_DL_Power_TimeslotLCR_InformationItem, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_Power },
  { 437, &hf_nbap_id_Minimum_DL_Power_TimeslotLCR_InformationItem, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_Power },
  { 438, &hf_nbap_id_HS_DSCHProvidedBitRateValueInformation, ASN1_EXTENSION_ROOT    , dissect_nbap_HS_DSCHProvidedBitRate },
  { 439, &hf_nbap_id_HS_DSCHRequiredPowerValueInformation, ASN1_EXTENSION_ROOT    , dissect_nbap_HS_DSCHRequiredPower },
  { 440, &hf_nbap_id_HS_DSCHRequiredPowerValue, ASN1_EXTENSION_ROOT    , dissect_nbap_HS_DSCHRequiredPowerValue },
  { 441, &hf_nbap_id_TransmittedCarrierPowerOfAllCodesNotUsedForHSTransmission, ASN1_EXTENSION_ROOT    , dissect_nbap_TransmittedCarrierPowerOfAllCodesNotUsedForHSTransmissionValue },
  { 442, &hf_nbap_id_HS_SICH_Reception_Quality, ASN1_EXTENSION_ROOT    , dissect_nbap_HS_SICH_Reception_Quality_Value },
  { 443, &hf_nbap_id_HS_SICH_Reception_Quality_Measurement_Value, ASN1_EXTENSION_ROOT    , dissect_nbap_HS_SICH_Reception_Quality_Measurement_Value },
  { 444, &hf_nbap_id_HSSICH_Info_DM_Rprt, ASN1_EXTENSION_ROOT    , dissect_nbap_HS_SICH_ID },
  { 445, &hf_nbap_id_HSSICH_Info_DM_Rqst, ASN1_EXTENSION_ROOT    , dissect_nbap_HSSICH_Info_DM_Rqst },
  { 446, &hf_nbap_id_HSSICH_Info_DM_Rsp, ASN1_EXTENSION_ROOT    , dissect_nbap_HS_SICH_ID },
  { 447, &hf_nbap_id_Best_Cell_Portions_Value, ASN1_EXTENSION_ROOT    , dissect_nbap_Best_Cell_Portions_Value },
  { 448, &hf_nbap_id_Primary_CPICH_Usage_for_Channel_Estimation, ASN1_EXTENSION_ROOT    , dissect_nbap_Primary_CPICH_Usage_for_Channel_Estimation },
  { 449, &hf_nbap_id_Secondary_CPICH_Information_Change, ASN1_EXTENSION_ROOT    , dissect_nbap_Secondary_CPICH_Information_Change },
  { 450, &hf_nbap_id_NumberOfReportedCellPortions, ASN1_EXTENSION_ROOT    , dissect_nbap_NumberOfReportedCellPortions },
  { 451, &hf_nbap_id_CellPortion_InformationItem_Cell_SetupRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_CellPortion_InformationItem_Cell_SetupRqstFDD },
  { 452, &hf_nbap_id_CellPortion_InformationList_Cell_SetupRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_CellPortion_InformationList_Cell_SetupRqstFDD },
  { 453, &hf_nbap_id_TimeslotISCP_LCR_InfoList_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_TimeslotISCPInfoLCR },
  { 454, &hf_nbap_id_Secondary_CPICH_Information, ASN1_EXTENSION_ROOT    , dissect_nbap_CommonPhysicalChannelID },
  { 455, &hf_nbap_id_Received_total_wide_band_power_For_CellPortion, ASN1_EXTENSION_ROOT    , dissect_nbap_Received_total_wide_band_power_Value_IncrDecrThres },
  { 456, &hf_nbap_id_Unidirectional_DCH_Indicator, ASN1_EXTENSION_ROOT    , dissect_nbap_Unidirectional_DCH_Indicator },
  { 457, &hf_nbap_id_TimingAdjustmentValueLCR, ASN1_EXTENSION_ROOT    , dissect_nbap_TimingAdjustmentValueLCR },
  { 458, &hf_nbap_id_multipleRL_ul_DPCH_InformationList, ASN1_EXTENSION_ROOT    , dissect_nbap_MultipleRL_UL_DPCH_InformationAddList_RL_ReconfPrepTDD },
  { 459, &hf_nbap_id_multipleRL_ul_DPCH_InformationModifyList, ASN1_EXTENSION_ROOT    , dissect_nbap_MultipleRL_UL_DPCH_InformationModifyList_RL_ReconfPrepTDD },
  { 460, &hf_nbap_id_RL_ID       , ASN1_EXTENSION_ROOT    , dissect_nbap_RL_ID },
  { 461, &hf_nbap_id_SAT_Info_Almanac_ExtItem, ASN1_EXTENSION_ROOT    , dissect_nbap_SAT_Info_Almanac_ExtItem },
  { 462, &hf_nbap_id_HSDPA_Capability, ASN1_EXTENSION_ROOT    , dissect_nbap_HSDPA_Capability },
  { 463, &hf_nbap_id_HSDSCH_Resources_Information_AuditRsp, ASN1_EXTENSION_ROOT    , dissect_nbap_HS_DSCH_Resources_Information_AuditRsp },
  { 464, &hf_nbap_id_HSDSCH_Resources_Information_ResourceStatusInd, ASN1_EXTENSION_ROOT    , dissect_nbap_HS_DSCH_Resources_Information_ResourceStatusInd },
  { 465, &hf_nbap_id_HSDSCH_MACdFlows_to_Add, ASN1_EXTENSION_ROOT    , dissect_nbap_HSDSCH_MACdFlows_Information },
  { 466, &hf_nbap_id_HSDSCH_MACdFlows_to_Delete, ASN1_EXTENSION_ROOT    , dissect_nbap_HSDSCH_MACdFlows_to_Delete },
  { 467, &hf_nbap_id_HSDSCH_Information_to_Modify_Unsynchronised, ASN1_EXTENSION_ROOT    , dissect_nbap_HSDSCH_Information_to_Modify_Unsynchronised },
  { 468, &hf_nbap_id_TnlQos      , ASN1_EXTENSION_ROOT    , dissect_nbap_TnlQos },
  { 469, &hf_nbap_id_Received_total_wide_band_power_For_CellPortion_Value, ASN1_EXTENSION_ROOT    , dissect_nbap_Received_total_wide_band_power_For_CellPortion_Value },
  { 470, &hf_nbap_id_Transmitted_Carrier_Power_For_CellPortion, ASN1_EXTENSION_ROOT    , dissect_nbap_Transmitted_Carrier_Power_Value },
  { 471, &hf_nbap_id_Transmitted_Carrier_Power_For_CellPortion_Value, ASN1_EXTENSION_ROOT    , dissect_nbap_Transmitted_Carrier_Power_For_CellPortion_Value },
  { 472, &hf_nbap_id_TransmittedCarrierPowerOfAllCodesNotUsedForHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHOrE_HICHTransmissionCellPortion, ASN1_EXTENSION_ROOT    , dissect_nbap_TransmittedCarrierPowerOfAllCodesNotUsedForHSTransmissionValue },
  { 473, &hf_nbap_id_TransmittedCarrierPowerOfAllCodesNotUsedForHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHOrE_HICHTransmissionCellPortionValue, ASN1_EXTENSION_ROOT    , dissect_nbap_TransmittedCarrierPowerOfAllCodesNotUsedForHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHOrE_HICHTransmissionCellPortionValue },
  { 474, &hf_nbap_id_UpPTSInterferenceValue, ASN1_EXTENSION_ROOT    , dissect_nbap_UpPTSInterferenceValue },
  { 475, &hf_nbap_id_PrimaryCCPCH_RSCP_Delta, ASN1_EXTENSION_ROOT    , dissect_nbap_PrimaryCCPCH_RSCP_Delta },
  { 476, &hf_nbap_id_MeasurementRecoveryBehavior, ASN1_EXTENSION_ROOT    , dissect_nbap_MeasurementRecoveryBehavior },
  { 477, &hf_nbap_id_MeasurementRecoveryReportingIndicator, ASN1_EXTENSION_ROOT    , dissect_nbap_MeasurementRecoveryReportingIndicator },
  { 478, &hf_nbap_id_MeasurementRecoverySupportIndicator, ASN1_EXTENSION_ROOT    , dissect_nbap_MeasurementRecoverySupportIndicator },
  { 479, &hf_nbap_id_Tstd_indicator, ASN1_EXTENSION_ROOT    , dissect_nbap_TSTD_Indicator },
  { 480, &hf_nbap_id_multiple_RL_Information_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_MultipleRL_Information_RL_ReconfPrepTDD },
  { 481, &hf_nbap_id_multiple_RL_Information_RL_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_Multiple_RL_Information_RL_ReconfRqstTDD },
  { 482, &hf_nbap_id_DL_DPCH_Power_Information_RL_ReconfPrepFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_DPCH_Power_Information_RL_ReconfPrepFDD },
  { 483, &hf_nbap_id_F_DPCH_Information_RL_ReconfPrepFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_F_DPCH_Information_RL_ReconfPrepFDD },
  { 484, &hf_nbap_id_F_DPCH_Information_RL_SetupRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_F_DPCH_Information_RL_SetupRqstFDD },
  { 485, &hf_nbap_id_Additional_S_CCPCH_Parameters_CTCH_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_Secondary_CCPCH_parameterExtendedList_CTCH_ReconfRqstTDD },
  { 486, &hf_nbap_id_Additional_S_CCPCH_Parameters_CTCH_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_Secondary_CCPCH_parameterExtendedList_CTCH_SetupRqstTDD },
  { 487, &hf_nbap_id_Additional_S_CCPCH_LCR_Parameters_CTCH_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_Secondary_CCPCH_LCR_parameterExtendedList_CTCH_ReconfRqstTDD },
  { 488, &hf_nbap_id_Additional_S_CCPCH_LCR_Parameters_CTCH_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_Secondary_CCPCH_LCR_parameterExtendedList_CTCH_SetupRqstTDD },
  { 489, &hf_nbap_id_MICH_CFN    , ASN1_EXTENSION_ROOT    , dissect_nbap_MICH_CFN },
  { 490, &hf_nbap_id_MICH_Information_AuditRsp, ASN1_EXTENSION_ROOT    , dissect_nbap_Common_PhysicalChannel_Status_Information },
  { 491, &hf_nbap_id_MICH_Information_ResourceStatusInd, ASN1_EXTENSION_ROOT    , dissect_nbap_Common_PhysicalChannel_Status_Information },
  { 492, &hf_nbap_id_MICH_Parameters_CTCH_ReconfRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_MICH_Parameters_CTCH_ReconfRqstFDD },
  { 493, &hf_nbap_id_MICH_Parameters_CTCH_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_MICH_Parameters_CTCH_ReconfRqstTDD },
  { 494, &hf_nbap_id_MICH_Parameters_CTCH_SetupRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_MICH_Parameters_CTCH_SetupRqstFDD },
  { 495, &hf_nbap_id_MICH_Parameters_CTCH_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_MICH_Parameters_CTCH_SetupRqstTDD },
  { 496, &hf_nbap_id_Modification_Period, ASN1_EXTENSION_ROOT    , dissect_nbap_Modification_Period },
  { 497, &hf_nbap_id_NI_Information_NotifUpdateCmd, ASN1_EXTENSION_ROOT    , dissect_nbap_NI_Information },
  { 498, &hf_nbap_id_S_CCPCH_InformationListExt_AuditRsp, ASN1_EXTENSION_ROOT    , dissect_nbap_S_CCPCH_InformationListExt_AuditRsp },
  { 499, &hf_nbap_id_S_CCPCH_InformationListExt_ResourceStatusInd, ASN1_EXTENSION_ROOT    , dissect_nbap_S_CCPCH_InformationListExt_ResourceStatusInd },
  { 500, &hf_nbap_id_S_CCPCH_LCR_InformationListExt_AuditRsp, ASN1_EXTENSION_ROOT    , dissect_nbap_S_CCPCH_LCR_InformationListExt_AuditRsp },
  { 501, &hf_nbap_id_S_CCPCH_LCR_InformationListExt_ResourceStatusInd, ASN1_EXTENSION_ROOT    , dissect_nbap_S_CCPCH_LCR_InformationListExt_ResourceStatusInd },
  { 502, &hf_nbap_id_HARQ_Preamble_Mode, ASN1_EXTENSION_ROOT    , dissect_nbap_HARQ_Preamble_Mode },
  { 503, &hf_nbap_id_Initial_DL_DPCH_TimingAdjustment, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_DPCH_TimingAdjustment },
  { 504, &hf_nbap_id_Initial_DL_DPCH_TimingAdjustment_Allowed, ASN1_EXTENSION_ROOT    , dissect_nbap_Initial_DL_DPCH_TimingAdjustment_Allowed },
  { 505, &hf_nbap_id_DLTransmissionBranchLoadValue, ASN1_EXTENSION_ROOT    , dissect_nbap_DLTransmissionBranchLoadValue },
  { 506, &hf_nbap_id_Power_Local_Cell_Group_choice_CM_Rqst, ASN1_EXTENSION_ROOT    , dissect_nbap_PowerLocalCellGroup_CM_Rqst },
  { 507, &hf_nbap_id_Power_Local_Cell_Group_choice_CM_Rsp, ASN1_EXTENSION_ROOT    , dissect_nbap_PowerLocalCellGroup_CM_Rsp },
  { 508, &hf_nbap_id_Power_Local_Cell_Group_choice_CM_Rprt, ASN1_EXTENSION_ROOT    , dissect_nbap_PowerLocalCellGroup_CM_Rprt },
  { 509, &hf_nbap_id_SynchronisationIndicator, ASN1_EXTENSION_ROOT    , dissect_nbap_SynchronisationIndicator },
  { 510, &hf_nbap_id_HSDPA_And_EDCH_CellPortion_Information_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , dissect_nbap_HSDPA_And_EDCH_CellPortion_Information_PSCH_ReconfRqst },
  { 511, &hf_nbap_id_HSDPA_And_EDCH_CellPortion_InformationList_PSCH_ReconfRqst, ASN1_EXTENSION_ROOT    , dissect_nbap_HSDPA_And_EDCH_CellPortion_InformationList_PSCH_ReconfRqst },
  { 512, &hf_nbap_id_HS_DSCHRequiredPowerValue_For_Cell_Portion, ASN1_EXTENSION_ROOT    , dissect_nbap_HS_DSCHRequiredPowerValue },
  { 513, &hf_nbap_id_HS_DSCHRequiredPowerValueInformation_For_CellPortion, ASN1_EXTENSION_ROOT    , dissect_nbap_HS_DSCHRequiredPowerValueInformation_For_CellPortion },
  { 514, &hf_nbap_id_HS_DSCHProvidedBitRateValueInformation_For_CellPortion, ASN1_EXTENSION_ROOT    , dissect_nbap_HS_DSCHProvidedBitRateValueInformation_For_CellPortion },
  { 515, &hf_nbap_id_E_AGCH_And_E_RGCH_E_HICH_FDD_Scrambling_Code, ASN1_EXTENSION_ROOT    , dissect_nbap_DL_ScramblingCode },
  { 516, &hf_nbap_id_E_AGCH_FDD_Code_Information, ASN1_EXTENSION_ROOT    , dissect_nbap_E_AGCH_FDD_Code_Information },
  { 517, &hf_nbap_id_E_DCH_Capability, ASN1_EXTENSION_ROOT    , dissect_nbap_E_DCH_Capability },
  { 518, &hf_nbap_id_E_DCH_FDD_DL_Control_Channel_Information, ASN1_EXTENSION_ROOT    , dissect_nbap_E_DCH_FDD_DL_Control_Channel_Information },
  { 519, &hf_nbap_id_E_DCH_FDD_Information, ASN1_EXTENSION_ROOT    , dissect_nbap_E_DCH_FDD_Information },
  { 520, &hf_nbap_id_E_DCH_FDD_Information_Response, ASN1_EXTENSION_ROOT    , dissect_nbap_E_DCH_FDD_Information_Response },
  { 521, &hf_nbap_id_E_DCH_FDD_Information_to_Modify, ASN1_EXTENSION_ROOT    , dissect_nbap_E_DCH_FDD_Information_to_Modify },
  { 522, &hf_nbap_id_E_DCH_MACdFlows_to_Add, ASN1_EXTENSION_ROOT    , dissect_nbap_E_DCH_MACdFlows_Information },
  { 523, &hf_nbap_id_E_DCH_MACdFlows_to_Delete, ASN1_EXTENSION_ROOT    , dissect_nbap_E_DCH_MACdFlows_to_Delete },
  { 524, &hf_nbap_id_E_DCH_Resources_Information_AuditRsp, ASN1_EXTENSION_ROOT    , dissect_nbap_E_DCH_Resources_Information_AuditRsp },
  { 525, &hf_nbap_id_E_DCH_Resources_Information_ResourceStatusInd, ASN1_EXTENSION_ROOT    , dissect_nbap_E_DCH_Resources_Information_ResourceStatusInd },
  { 526, &hf_nbap_id_E_DCH_RL_Indication, ASN1_EXTENSION_ROOT    , dissect_nbap_E_DCH_RL_Indication },
  { 527, &hf_nbap_id_E_DCH_RL_Set_ID, ASN1_EXTENSION_ROOT    , dissect_nbap_RL_Set_ID },
  { 528, &hf_nbap_id_E_DPCH_Information_RL_ReconfPrepFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_E_DPCH_Information_RL_ReconfPrepFDD },
  { 529, &hf_nbap_id_E_DPCH_Information_RL_SetupRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_E_DPCH_Information_RL_SetupRqstFDD },
  { 530, &hf_nbap_id_E_RGCH_E_HICH_FDD_Code_Information, ASN1_EXTENSION_ROOT    , dissect_nbap_E_RGCH_E_HICH_FDD_Code_Information },
  { 531, &hf_nbap_id_Serving_E_DCH_RL_ID, ASN1_EXTENSION_ROOT    , dissect_nbap_Serving_E_DCH_RL_ID },
  { 532, &hf_nbap_id_UL_DPDCH_Indicator_For_E_DCH_Operation, ASN1_EXTENSION_ROOT    , dissect_nbap_UL_DPDCH_Indicator_For_E_DCH_Operation },
  { 533, &hf_nbap_id_FDD_S_CCPCH_FrameOffset_CTCH_SetupRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_FDD_S_CCPCH_FrameOffset },
  { 534, &hf_nbap_id_E_DPCH_Information_RL_ReconfRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_E_DPCH_Information_RL_ReconfRqstFDD },
  { 535, &hf_nbap_id_Maximum_Target_ReceivedTotalWideBandPower, ASN1_EXTENSION_ROOT    , dissect_nbap_Maximum_Target_ReceivedTotalWideBandPower },
  { 536, &hf_nbap_id_E_DCHProvidedBitRateValueInformation, ASN1_EXTENSION_ROOT    , dissect_nbap_E_DCHProvidedBitRate },
  { 537, &hf_nbap_id_HARQ_Preamble_Mode_Activation_Indicator, ASN1_EXTENSION_ROOT    , dissect_nbap_HARQ_Preamble_Mode_Activation_Indicator },
  { 538, &hf_nbap_id_RL_Specific_E_DCH_Info, ASN1_EXTENSION_ROOT    , dissect_nbap_RL_Specific_E_DCH_Info },
  { 539, &hf_nbap_id_E_DCH_CapacityConsumptionLaw, ASN1_EXTENSION_ROOT    , dissect_nbap_DedicatedChannelsCapacityConsumptionLaw },
  { 540, &hf_nbap_id_multiple_DedicatedMeasurementValueList_TDD_DM_Rsp, ASN1_EXTENSION_ROOT    , dissect_nbap_Multiple_DedicatedMeasurementValueList_TDD_DM_Rsp },
  { 541, &hf_nbap_id_multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp, ASN1_EXTENSION_ROOT    , dissect_nbap_Multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp },
  { 542, &hf_nbap_id_E_DCH_RearrangeList_Bearer_RearrangeInd, ASN1_EXTENSION_ROOT    , dissect_nbap_E_DCH_RearrangeList_Bearer_RearrangeInd },
  { 543, &hf_nbap_id_multipleRL_dl_CCTrCH_InformationModifyList_RL_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_nbap_MultipleRL_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD },
  { 544, &hf_nbap_id_Target_NonServing_EDCH_To_Total_EDCH_Power_Ratio, ASN1_EXTENSION_ROOT    , dissect_nbap_Target_NonServing_EDCH_To_Total_EDCH_Power_Ratio },
  { 545, &hf_nbap_id_CellPortion_InformationItem_Cell_ReconfRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_CellPortion_InformationItem_Cell_ReconfRqstFDD },
  { 546, &hf_nbap_id_CellPortion_InformationList_Cell_ReconfRqstFDD, ASN1_EXTENSION_ROOT    , dissect_nbap_CellPortion_InformationList_Cell_ReconfRqstFDD },
  { 547, &hf_nbap_id_multiple_PUSCH_InfoList_DM_Rsp, ASN1_EXTENSION_ROOT    , dissect_nbap_Multiple_PUSCH_InfoList_DM_Rsp },
  { 548, &hf_nbap_id_multiple_PUSCH_InfoList_DM_Rprt, ASN1_EXTENSION_ROOT    , dissect_nbap_Multiple_PUSCH_InfoList_DM_Rprt },
  { 549, &hf_nbap_id_Reference_ReceivedTotalWideBandPower, ASN1_EXTENSION_ROOT    , dissect_nbap_Reference_ReceivedTotalWideBandPower },
  { 0, NULL, 0, NULL }
};

static int
dissect_nbap_DymmyProtocolIE_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_nbap_DymmyProtocolIE_ID, DymmyProtocolIE_ID_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t IPDLParameter_Information_Cell_ReconfRqstTDD_sequence[] = {
  { &hf_nbap_iPDL_TDD_Parameters, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_IPDL_TDD_Parameters },
  { &hf_nbap_iPDL_Indicator , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_IPDL_Indicator },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_IPDLParameter_Information_Cell_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_IPDLParameter_Information_Cell_ReconfRqstTDD, IPDLParameter_Information_Cell_ReconfRqstTDD_sequence);

  return offset;
}


static const per_sequence_t RadioLinkAdditionRequestTDD_sequence[] = {
  { &hf_nbap_protocolIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
  { &hf_nbap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RadioLinkAdditionRequestTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RadioLinkAdditionRequestTDD, RadioLinkAdditionRequestTDD_sequence);

  return offset;
}



static int
dissect_nbap_UL_DPCH_InformationList_RL_AdditionRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_nbap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_uL_DPCH_Information1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_DPCH_InformationList_RL_AdditionRqstTDD(tvb, offset, actx, tree, hf_nbap_uL_DPCH_Information1);
}


static const per_sequence_t UL_CCTrCH_InformationItem_RL_AdditionRqstTDD_sequence[] = {
  { &hf_nbap_cCTrCH_ID      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CCTrCH_ID },
  { &hf_nbap_uL_DPCH_Information1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_UL_DPCH_InformationList_RL_AdditionRqstTDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_UL_CCTrCH_InformationItem_RL_AdditionRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_UL_CCTrCH_InformationItem_RL_AdditionRqstTDD, UL_CCTrCH_InformationItem_RL_AdditionRqstTDD_sequence);

  return offset;
}
static int dissect_UL_CCTrCH_InformationList_RL_AdditionRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_UL_CCTrCH_InformationItem_RL_AdditionRqstTDD(tvb, offset, actx, tree, hf_nbap_UL_CCTrCH_InformationList_RL_AdditionRqstTDD_item);
}


static const per_sequence_t UL_CCTrCH_InformationList_RL_AdditionRqstTDD_sequence_of[1] = {
  { &hf_nbap_UL_CCTrCH_InformationList_RL_AdditionRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_UL_CCTrCH_InformationItem_RL_AdditionRqstTDD },
};

static int
dissect_nbap_UL_CCTrCH_InformationList_RL_AdditionRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_UL_CCTrCH_InformationList_RL_AdditionRqstTDD, UL_CCTrCH_InformationList_RL_AdditionRqstTDD_sequence_of,
                                                  1, 16);

  return offset;
}


static const per_sequence_t MultipleRL_DL_DPCH_InformationAddListIE_RL_ReconfPrepTDD_sequence[] = {
  { &hf_nbap_dl_DPCH_InformationList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_DPCH_InformationAddList_RL_ReconfPrepTDD },
  { &hf_nbap_dl_DPCH_InformationListLCR, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_DPCH_LCR_InformationAddList_RL_ReconfPrepTDD },
  { &hf_nbap_cCTrCH_Initial_DL_Power, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_Power },
  { &hf_nbap_tDD_TPC_DownlinkStepSize, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TDD_TPC_DownlinkStepSize },
  { &hf_nbap_cCTrCH_Maximum_DL_Power_InformationAdd_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_Power },
  { &hf_nbap_cCTrCH_Minimum_DL_Power_InformationAdd_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_Power },
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_RL_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_MultipleRL_DL_DPCH_InformationAddListIE_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_MultipleRL_DL_DPCH_InformationAddListIE_RL_ReconfPrepTDD, MultipleRL_DL_DPCH_InformationAddListIE_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_MultipleRL_DL_DPCH_InformationAddList_RL_ReconfPrepTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MultipleRL_DL_DPCH_InformationAddListIE_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_MultipleRL_DL_DPCH_InformationAddList_RL_ReconfPrepTDD_item);
}


static const per_sequence_t MultipleRL_DL_DPCH_InformationAddList_RL_ReconfPrepTDD_sequence_of[1] = {
  { &hf_nbap_MultipleRL_DL_DPCH_InformationAddList_RL_ReconfPrepTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_MultipleRL_DL_DPCH_InformationAddListIE_RL_ReconfPrepTDD },
};

static int
dissect_nbap_MultipleRL_DL_DPCH_InformationAddList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_MultipleRL_DL_DPCH_InformationAddList_RL_ReconfPrepTDD, MultipleRL_DL_DPCH_InformationAddList_RL_ReconfPrepTDD_sequence_of,
                                                  1, 15);

  return offset;
}


static const per_sequence_t MultipleRL_DL_DPCH_InformationModifyListIE_RL_ReconfPrepTDD_sequence[] = {
  { &hf_nbap_dl_DPCH_InformationAddList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_DPCH_InformationModify_AddList_RL_ReconfPrepTDD },
  { &hf_nbap_dl_DPCH_InformationModifyList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_DPCH_InformationModify_ModifyList_RL_ReconfPrepTDD },
  { &hf_nbap_dl_DPCH_InformationDeleteList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_DPCH_InformationModify_DeleteList_RL_ReconfPrepTDD },
  { &hf_nbap_dl_DPCH_InformationAddListLCR, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_DPCH_LCR_InformationModify_AddList_RL_ReconfPrepTDD },
  { &hf_nbap_tDD_TPC_DownlinkStepSize_InformationModify_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_TDD_TPC_DownlinkStepSize },
  { &hf_nbap_cCTrCH_Maximum_DL_Power_InformationModify_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_Power },
  { &hf_nbap_cCTrCH_Minimum_DL_Power_InformationModify_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_Power },
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_RL_ID },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_MultipleRL_DL_DPCH_InformationModifyListIE_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_MultipleRL_DL_DPCH_InformationModifyListIE_RL_ReconfPrepTDD, MultipleRL_DL_DPCH_InformationModifyListIE_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_MultipleRL_DL_DPCH_InformationModifyList_RL_ReconfPrepTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_MultipleRL_DL_DPCH_InformationModifyListIE_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_nbap_MultipleRL_DL_DPCH_InformationModifyList_RL_ReconfPrepTDD_item);
}


static const per_sequence_t MultipleRL_DL_DPCH_InformationModifyList_RL_ReconfPrepTDD_sequence_of[1] = {
  { &hf_nbap_MultipleRL_DL_DPCH_InformationModifyList_RL_ReconfPrepTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_MultipleRL_DL_DPCH_InformationModifyListIE_RL_ReconfPrepTDD },
};

static int
dissect_nbap_MultipleRL_DL_DPCH_InformationModifyList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_MultipleRL_DL_DPCH_InformationModifyList_RL_ReconfPrepTDD, MultipleRL_DL_DPCH_InformationModifyList_RL_ReconfPrepTDD_sequence_of,
                                                  1, 15);

  return offset;
}


static const per_sequence_t DL_DPCH_InformationModify_DeleteListIE_RL_ReconfPrepTDD_sequence_of[1] = {
  { &hf_nbap_DL_DPCH_InformationModify_DeleteListIE_RL_ReconfPrepTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_DL_DPCH_InformationModify_DeleteItem_RL_ReconfPrepTDD },
};

static int
dissect_nbap_DL_DPCH_InformationModify_DeleteListIE_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_DL_DPCH_InformationModify_DeleteListIE_RL_ReconfPrepTDD, DL_DPCH_InformationModify_DeleteListIE_RL_ReconfPrepTDD_sequence_of,
                                                  1, 240);

  return offset;
}


static const per_sequence_t RL_InformationItem_RL_ReconfRqstFDD_sequence[] = {
  { &hf_nbap_rL_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_RL_ID },
  { &hf_nbap_maxDL_Power    , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_Power },
  { &hf_nbap_minDL_Power    , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_DL_Power },
  { &hf_nbap_dl_CodeInformation, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_FDD_DL_CodeInformation },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_RL_InformationItem_RL_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_RL_InformationItem_RL_ReconfRqstFDD, RL_InformationItem_RL_ReconfRqstFDD_sequence);

  return offset;
}


static const per_sequence_t CellSyncBurstMeasInfoItem_CellSyncReconfRqstTDD_sequence[] = {
  { &hf_nbap_syncFrameNrToReceive, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_SyncFrameNumber },
  { &hf_nbap_syncBurstInfo  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_CellSyncBurstInfoList_CellSyncReconfRqstTDD },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_CellSyncBurstMeasInfoItem_CellSyncReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_CellSyncBurstMeasInfoItem_CellSyncReconfRqstTDD, CellSyncBurstMeasInfoItem_CellSyncReconfRqstTDD_sequence);

  return offset;
}
static int dissect_CellSyncBurstMeasInfoListIE_CellSyncReconfRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_CellSyncBurstMeasInfoItem_CellSyncReconfRqstTDD(tvb, offset, actx, tree, hf_nbap_CellSyncBurstMeasInfoListIE_CellSyncReconfRqstTDD_item);
}


static const per_sequence_t CellSyncBurstMeasInfoListIE_CellSyncReconfRqstTDD_sequence_of[1] = {
  { &hf_nbap_CellSyncBurstMeasInfoListIE_CellSyncReconfRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_CellSyncBurstMeasInfoItem_CellSyncReconfRqstTDD },
};

static int
dissect_nbap_CellSyncBurstMeasInfoListIE_CellSyncReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_CellSyncBurstMeasInfoListIE_CellSyncReconfRqstTDD, CellSyncBurstMeasInfoListIE_CellSyncReconfRqstTDD_sequence_of,
                                                  1, 10);

  return offset;
}


static const value_string nbap_GPS_Information_Item_vals[] = {
  {   0, "gps-navigation-model-and-time-recovery" },
  {   1, "gps-ionospheric-model" },
  {   2, "gps-utc-model" },
  {   3, "gps-almanac" },
  {   4, "gps-rt-integrity" },
  { 0, NULL }
};


static int
dissect_nbap_GPS_Information_Item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     5, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_GPS_Information_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_GPS_Information_Item(tvb, offset, actx, tree, hf_nbap_GPS_Information_item);
}


static const per_sequence_t GPS_Information_sequence_of[1] = {
  { &hf_nbap_GPS_Information_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_GPS_Information_Item },
};

static int
dissect_nbap_GPS_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_GPS_Information, GPS_Information_sequence_of,
                                                  0, 8);

  return offset;
}
static int dissect_gPSInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_GPS_Information(tvb, offset, actx, tree, hf_nbap_gPSInformation);
}


static const value_string nbap_Information_Type_Item_vals[] = {
  {   0, "gpsinformation" },
  {   1, "dgpscorrections" },
  {   2, "gpsrxpos" },
  { 0, NULL }
};


static int
dissect_nbap_Information_Type_Item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_information_Type_Item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_Information_Type_Item(tvb, offset, actx, tree, hf_nbap_information_Type_Item);
}


static const per_sequence_t InformationType_sequence[] = {
  { &hf_nbap_information_Type_Item, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_nbap_Information_Type_Item },
  { &hf_nbap_gPSInformation , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_GPS_Information },
  { &hf_nbap_iE_Extensions  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_nbap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_InformationType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_InformationType, InformationType_sequence);

  return offset;
}


static const per_sequence_t SAT_Info_Almanac_ExtList_sequence_of[1] = {
  { &hf_nbap_SAT_Info_Almanac_ExtList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_SAT_Info_Almanac_ExtItem },
};

static int
dissect_nbap_SAT_Info_Almanac_ExtList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_SAT_Info_Almanac_ExtList, SAT_Info_Almanac_ExtList_sequence_of,
                                                  1, 16);

  return offset;
}


static const value_string nbap_Presence_vals[] = {
  {   0, "optional" },
  {   1, "conditional" },
  {   2, "mandatory" },
  { 0, NULL }
};


static int
dissect_nbap_Presence(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_nbap_FirstValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
#line 68 "nbap.cnf"
	/* FIX ME */
	guint length;

	offset = dissect_per_length_determinant(tvb, offset, actx, tree, hf_nbap_IE_length, &length);
	offset = offset + length;
	BYTE_ALIGN_OFFSET(offset)



  return offset;
}
static int dissect_firstValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_FirstValue(tvb, offset, actx, tree, hf_nbap_firstValue);
}



static int
dissect_nbap_SecondValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
#line 76 "nbap.cnf"
	/* FIX ME */
	guint length;

	offset = dissect_per_length_determinant(tvb, offset, actx, tree, hf_nbap_IE_length, &length);
	offset = offset + length;
	BYTE_ALIGN_OFFSET(offset)




  return offset;
}
static int dissect_secondValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_SecondValue(tvb, offset, actx, tree, hf_nbap_secondValue);
}


static const per_sequence_t ProtocolIE_FieldPair_sequence[] = {
  { &hf_nbap_id             , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_ID },
  { &hf_nbap_firstCriticality, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_Criticality },
  { &hf_nbap_firstValue     , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_FirstValue },
  { &hf_nbap_secondCriticality, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_Criticality },
  { &hf_nbap_secondValue    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_SecondValue },
  { NULL, 0, 0, NULL }
};

static int
dissect_nbap_ProtocolIE_FieldPair(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_nbap_ProtocolIE_FieldPair, ProtocolIE_FieldPair_sequence);

  return offset;
}
static int dissect_ProtocolIE_ContainerPair_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_FieldPair(tvb, offset, actx, tree, hf_nbap_ProtocolIE_ContainerPair_item);
}


static const per_sequence_t ProtocolIE_ContainerPair_sequence_of[1] = {
  { &hf_nbap_ProtocolIE_ContainerPair_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_FieldPair },
};

static int
dissect_nbap_ProtocolIE_ContainerPair(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_ProtocolIE_ContainerPair, ProtocolIE_ContainerPair_sequence_of,
                                                  0, 65535);

  return offset;
}
static int dissect_ProtocolIE_ContainerPairList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_nbap_ProtocolIE_ContainerPair(tvb, offset, actx, tree, hf_nbap_ProtocolIE_ContainerPairList_item);
}


static const per_sequence_t ProtocolIE_ContainerList_sequence_of[1] = {
  { &hf_nbap_ProtocolIE_ContainerList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_Container },
};

static int
dissect_nbap_ProtocolIE_ContainerList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_ProtocolIE_ContainerList, ProtocolIE_ContainerList_sequence_of,
                                                  0, 65535);

  return offset;
}


static const per_sequence_t ProtocolIE_ContainerPairList_sequence_of[1] = {
  { &hf_nbap_ProtocolIE_ContainerPairList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_nbap_ProtocolIE_ContainerPair },
};

static int
dissect_nbap_ProtocolIE_ContainerPairList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_nbap_ProtocolIE_ContainerPairList, ProtocolIE_ContainerPairList_sequence_of,
                                                  0, 65535);

  return offset;
}

/*--- PDUs ---*/

static void dissect_NBAP_PDU_PDU(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree) {
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, TRUE, pinfo);
  dissect_nbap_NBAP_PDU(tvb, 0, &asn1_ctx, tree, hf_nbap_NBAP_PDU_PDU);
}


/*--- End of included file: packet-nbap-fn.c ---*/
#line 738 "packet-nbap-template.c"


static int dissect_nbap_InitiatingMessageValueValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree){
	proto_item	*value_item = NULL;
	proto_tree	*value_tree = NULL;
	guint length;

    value_item = proto_tree_add_item(tree, hf_nbap_initiatingMessageValue, tvb, 0, -1, FALSE);
    value_tree = proto_item_add_subtree(value_item, ett_nbap_initiatingMessageValue);

	offset = dissect_per_length_determinant(tvb, offset, actx, value_tree, hf_nbap_pdu_length, &length);
	proto_item_set_len(value_item,length);

	
	switch(ProcedureCode){
	case NBAP_ID_AUDIT:											/* 	0 */
		offset = dissect_id_audit(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_AUDITREQUIRED:									/* 	1 */
		offset = dissect_id_auditRequired(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_BLOCKRESOURCE:									/* 	2 */
		offset = dissect_id_blockResource(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_CELLDELETION:									/* 	3 */
		offset = dissect_id_cellDeletion(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_CELLRECONFIGURATION: 							/* 	4 */
		if (ddMode==NBAP_FDD){
			offset = dissect_id_cellReconfiguration(tvb, offset, actx, value_tree);
		}else{
			offset = dissect_id_cellReconfiguration_tdd(tvb, offset, actx, value_tree);
		}
		break;
	case NBAP_ID_CELLSETUP:										/* 	5 */
		if (ddMode==NBAP_FDD){
			offset = dissect_id_cellSetup(tvb, offset, actx, value_tree);
		}else{
			offset = dissect_id_cellSetup_tdd(tvb, offset, actx, value_tree);
		}
		break;
	case NBAP_ID_COMMONMEASUREMENTFAILURE :						/* 	6 */
		offset = dissect_id_commonMeasurementFailure(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_COMMONMEASUREMENTINITIATION: 					/* 	7 */
		offset = dissect_id_commonMeasurementInitiation(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_COMMONMEASUREMENTREPORT:						/*  8 */
		offset = dissect_id_commonMeasurementReport(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_COMMONMEASUREMENTTERMINATION: 					/*	9 */
		offset = dissect_id_commonMeasurementTermination(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_COMMONTRANSPORTCHANNELDELETE: 					/*	10 */
		offset = dissect_id_commonTransportChannelDelete(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_COMMONTRANSPORTCHANNELRECONFIGURE:				/*	11 */
		if (ddMode==NBAP_FDD){
			offset = dissect_id_commonTransportChannelReconfigure(tvb, offset, actx, value_tree);
		}else{
			offset = dissect_id_commonTransportChannelReconfigure_tdd(tvb, offset, actx, value_tree);
		}
		break;
	case NBAP_ID_COMMONTRANSPORTCHANNELSETUP: 					/*	12 */
		if (ddMode==NBAP_FDD){
			offset = dissect_id_commonTransportChannelSetup(tvb, offset, actx, value_tree);
		}else{
			offset = dissect_id_commonTransportChannelSetup_tdd(tvb, offset, actx, value_tree);
		}
		break;
	case NBAP_ID_RESET:											/*	13 */
		offset = dissect_id_reset(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_COMPRESSEDMODECOMMAND:							/*	14 */
		offset = dissect_id_compressedModeCommand(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_DEDICATEDMEASUREMENTFAILURE: 					/*	16 */
		offset = dissect_id_dedicatedMeasurementFailure(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_DEDICATEDMEASUREMENTINITIATION: 				/*	17 */
		offset = dissect_id_dedicatedMeasurementInitiation(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_DEDICATEDMEASUREMENTREPORT: 					/*	18 */
		offset = dissect_id_dedicatedMeasurementReport(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_DEDICATEDMEASUREMENTTERMINATION: 				/*	19 */
		offset = dissect_id_dedicatedMeasurementTermination(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_DOWNLINKPOWERCONTROL: 							/*	20 */
		offset = dissect_id_downlinkPowerControl(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_ERRORINDICATIONFORDEDICATED:					/*	21 */
		offset = dissect_id_errorIndicationForDedicated(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_PRIVATEMESSAGEFORDEDICATED:					/*	22 */
		offset = dissect_id_compressedModeCommand(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_RADIOLINKADDITION:								/*	23 */
		offset = dissect_id_radioLinkAddition(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_RADIOLINKDELETION:								/*	24 */
		offset = dissect_id_radioLinkDeletion(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_RADIOLINKFAILURE: 								/*	25 */
		offset = dissect_id_radioLinkFailure(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_RADIOLINKRESTORATION: 							/*	26 */
		offset = dissect_id_radioLinkRestoration(tvb, offset, actx, value_tree);
		break;
		break;
	case NBAP_ID_RADIOLINKSETUP: 								/*	27 */
		if (ddMode==NBAP_FDD){
			offset = dissect_id_radioLinkSetup(tvb, offset, actx, value_tree);
		}else{
			offset = dissect_id_radioLinkSetup_tdd(tvb, offset, actx, value_tree);
		}
		break;
	case NBAP_ID_RESOURCESTATUSINDICATION: 						/*	28 */
		offset = dissect_id_resourceStatusIndication(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_SYNCHRONISEDRADIOLINKRECONFIGURATIONCANCELLATION:	/*	29 */
		offset = dissect_id_synchronisedRadioLinkReconfigurationCancellation(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_SYNCHRONISEDRADIOLINKRECONFIGURATIONCOMMIT:		/*	30 */
		offset = dissect_id_synchronisedRadioLinkReconfigurationCommit(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_SYNCHRONISEDRADIOLINKRECONFIGURATIONPREPARATION: /*	31 */
		if (ddMode==NBAP_FDD){
			offset = dissect_id_synchronisedRadioLinkReconfigurationPreparation(tvb, offset, actx, value_tree);
		}else{
			offset = dissect_id_synchronisedRadioLinkReconfigurationPreparation_tdd(tvb, offset, actx, value_tree);
		}
		break;
	case NBAP_ID_SYSTEMINFORMATIONUPDATE: 						/*	32 */
		offset = dissect_id_systemInformationUpdate(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_UNBLOCKRESOURCE: 								/*	33 */
		offset = dissect_id_unblockResource(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_UNSYNCHRONISEDRADIOLINKRECONFIGURATION: 		/*	34 */
		if (ddMode==NBAP_FDD){
			offset = dissect_id_unSynchronisedRadioLinkReconfiguration(tvb, offset, actx, value_tree);
		}else{
			offset = dissect_id_unSynchronisedRadioLinkReconfiguration_tdd(tvb, offset, actx, value_tree);
		}
		break;
	case NBAP_ID_ERRORINDICATIONFORCOMMON:						/*	35 */
		offset = dissect_id_errorIndicationForCommon(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_PRIVATEMESSAGEFORCOMMON:						/*	36 */
		offset = dissect_id_privateMessageForCommon(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_PHYSICALSHAREDCHANNELRECONFIGURATION: 			/*	37 */
		offset = dissect_id_physicalSharedChannelReconfiguration(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_DOWNLINKPOWERTIMESLOTCONTROL: 					/*	38 */
		offset = dissect_id_downlinkPowerTimeslotControl(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_RADIOLINKPREEMPTION:							/*	39 */
		offset = dissect_id_radioLinkPreemption(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_INFORMATIONEXCHANGEFAILURE:					/*	40 */
		offset = dissect_id_informationExchangeFailure(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_INFORMATIONEXCHANGEINITIATION:					/*	41 */
		offset = dissect_id_informationExchangeInitiation(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_INFORMATIONEXCHANGETERMINATION:				/*	42 */
		offset = dissect_id_informationExchangeTermination(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_INFORMATIONREPORTING:							/*	43 */
		offset = dissect_id_informationReporting(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_CELLSYNCHRONISATIONADJUSTMENT:					/*	44 */
		offset = dissect_id_cellSynchronisationAdjustment(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_CELLSYNCHRONISATIONINITIATION:					/*	45 */
		offset = dissect_id_cellSynchronisationInitiation(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_CELLSYNCHRONISATIONRECONFIGURATION:			/*	46 */
		offset = dissect_id_cellSynchronisationReconfiguration(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_CELLSYNCHRONISATIONREPORTING:					/*	47 */
		offset = dissect_id_cellSynchronisationReporting(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_CELLSYNCHRONISATIONTERMINATION:				/*	48 */
		offset = dissect_id_cellSynchronisationTermination(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_CELLSYNCHRONISATIONFAILURE:					/*	49 */
		offset = dissect_id_cellSynchronisationFailure(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_BEARERREARRANGEMENT:							/*	50 */
		offset = dissect_id_BearerRearrangement(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_RADIOLINKACTIVATION:							/*	51 */
		if (ddMode==NBAP_FDD){
			offset = dissect_id_radioLinkActivation(tvb, offset, actx, value_tree);
		}else{
			offset = dissect_id_radioLinkActivation_tdd(tvb, offset, actx, value_tree);
		}
		break;
	case NBAP_ID_RADIOLINKPARAMETERUPDATE:						/*	52 */
		if (ddMode==NBAP_FDD){
			offset = dissect_id_radioLinkParameterUpdate(tvb, offset, actx, value_tree);
		}else{
			offset = dissect_id_radioLinkParameterUpdate_tdd(tvb, offset, actx, value_tree);
		}
		break;
	case NBAP_ID_MBMSNOTIFICATIONUPDATE:						/*	53 */
		offset = dissect_id_mBMSNotificationUpdate(tvb, offset, actx, value_tree);
		break;


	default:
		offset = offset + length;
		break;
	}
	BYTE_ALIGN_OFFSET(offset)
	return offset;
}

static int dissect_nbap_SuccessfulOutcomeValueValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree){
	proto_item	*value_item = NULL;
	proto_tree	*value_tree = NULL;
	guint length;

    value_item = proto_tree_add_item(tree, hf_nbap_successfulOutcomeValue, tvb, 0, -1, FALSE);
    value_tree = proto_item_add_subtree(value_item, ett_nbap_initiatingMessageValue);

	offset = dissect_per_length_determinant(tvb, offset, actx, value_tree, hf_nbap_pdu_length, &length);
	proto_item_set_len(value_item,length);

	
	switch(ProcedureCode){
	case NBAP_ID_AUDIT:											/* 	0 */
		offset = dissect_id_audit1(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_BLOCKRESOURCE:									/* 	2 */
		offset = dissect_id_blockResource1(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_CELLDELETION:									/* 	3 */
		offset = dissect_id_cellDeletion1(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_CELLRECONFIGURATION: 							/* 	4 */
		offset = dissect_id_cellReconfiguration1(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_CELLSETUP:										/* 	5 */
		offset = dissect_id_cellSetup1(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_COMMONMEASUREMENTINITIATION: 					/* 	7 */
		offset = dissect_id_commonMeasurementInitiation1(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_COMMONTRANSPORTCHANNELDELETE: 					/*	10 */
		offset = dissect_id_commonTransportChannelDelete1(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_COMMONTRANSPORTCHANNELRECONFIGURE:				/*	11 */
		offset = dissect_id_commonTransportChannelReconfigure1(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_COMMONTRANSPORTCHANNELSETUP: 					/*	12 */
		offset = dissect_id_commonTransportChannelSetup1(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_RESET:											/*	13 */
		offset = dissect_id_reset1(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_DEDICATEDMEASUREMENTINITIATION: 				/*	17 */
		offset = dissect_id_dedicatedMeasurementInitiation(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_RADIOLINKADDITION:								/*	23 */
		if (ddMode==NBAP_FDD){
			offset = dissect_id_radioLinkAddition1(tvb, offset, actx, value_tree);
		}else{
			offset = dissect_id_radioLinkAddition_tdd(tvb, offset, actx, value_tree);
		}
		break;
		break;
	case NBAP_ID_RADIOLINKDELETION:								/*	24 */
		offset = dissect_id_radioLinkDeletion1(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_RADIOLINKSETUP: 								/*	27 */
		if (ddMode==NBAP_FDD){
			offset = dissect_id_radioLinkSetup1(tvb, offset, actx, value_tree);
		}else{
			offset = dissect_id_radioLinkSetup_tdd1(tvb, offset, actx, value_tree);
		}
		break;
	case NBAP_ID_SYNCHRONISEDRADIOLINKRECONFIGURATIONPREPARATION: /*	31 */
		offset = dissect_id_synchronisedRadioLinkReconfigurationPreparation1(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_SYSTEMINFORMATIONUPDATE: 						/*	32 */
		offset = dissect_id_systemInformationUpdate1(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_UNSYNCHRONISEDRADIOLINKRECONFIGURATION: 		/*	34 */
		offset = dissect_id_unSynchronisedRadioLinkReconfiguration1(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_PHYSICALSHAREDCHANNELRECONFIGURATION: 			/*	37 */
		offset = dissect_id_physicalSharedChannelReconfiguration1(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_INFORMATIONEXCHANGEINITIATION:					/*	41 */
		offset = dissect_id_informationExchangeInitiation1(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_CELLSYNCHRONISATIONADJUSTMENT:					/*	44 */
		offset = dissect_id_cellSynchronisationAdjustment1(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_CELLSYNCHRONISATIONINITIATION:					/*	45 */
		offset = dissect_id_cellSynchronisationInitiation1(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_CELLSYNCHRONISATIONRECONFIGURATION:			/*	46 */
		offset = dissect_id_cellSynchronisationReconfiguration1(tvb, offset, actx, value_tree);
		break;
	default:
		offset = offset + length;
		break;
	}
	BYTE_ALIGN_OFFSET(offset)
	return offset;
}

static int dissect_nbap_UnsuccessfulOutcomeValueValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree){
	proto_item	*value_item = NULL;
	proto_tree	*value_tree = NULL;
	guint length;

    value_item = proto_tree_add_item(tree, hf_nbap_unsuccessfulOutcomeValue, tvb, 0, -1, FALSE);
    value_tree = proto_item_add_subtree(value_item, ett_nbap_UnsuccessfulOutcomeValue);

	offset = dissect_per_length_determinant(tvb, offset, actx, value_tree, hf_nbap_pdu_length, &length);
	proto_item_set_len(value_item,length);

	
	switch(ProcedureCode){
	case NBAP_ID_AUDIT:											/* 	0 */
		offset = dissect_id_audit2(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_BLOCKRESOURCE:									/* 	2 */
		offset = dissect_id_blockResource2(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_CELLDELETION:									/* 	3 */
		offset = dissect_id_cellReconfiguration2(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_CELLSETUP:										/* 	5 */
		offset = dissect_id_cellSetup2(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_COMMONMEASUREMENTINITIATION: 					/* 	7 */
		offset = dissect_id_commonMeasurementInitiation2(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_COMMONTRANSPORTCHANNELRECONFIGURE:				/*	11 */
		offset = dissect_id_commonTransportChannelReconfigure2(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_COMMONTRANSPORTCHANNELSETUP: 					/*	12 */
		offset = dissect_id_commonTransportChannelSetup2(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_DEDICATEDMEASUREMENTINITIATION: 				/*	17 */
		offset = dissect_id_dedicatedMeasurementInitiation2(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_RADIOLINKADDITION:								/*	23 */
		if (ddMode==NBAP_FDD){
			offset = dissect_id_radioLinkAddition2(tvb, offset, actx, value_tree);
		}else{
			offset = dissect_id_radioLinkAddition_tdd1(tvb, offset, actx, value_tree);
		}
		break;
	case NBAP_ID_RADIOLINKSETUP: 								/*	27 */
		if (ddMode==NBAP_FDD){
			offset = dissect_id_radioLinkSetup2(tvb, offset, actx, value_tree);
		}else{
			offset = dissect_id_radioLinkSetup_tdd2(tvb, offset, actx, value_tree);
		}
		break;
	case NBAP_ID_SYNCHRONISEDRADIOLINKRECONFIGURATIONPREPARATION: /*	31 */
		offset = dissect_id_synchronisedRadioLinkReconfigurationPreparation2(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_SYSTEMINFORMATIONUPDATE: 						/*	32 */
		offset = dissect_id_systemInformationUpdate2(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_UNSYNCHRONISEDRADIOLINKRECONFIGURATION: 		/*	34 */
		offset = dissect_id_unSynchronisedRadioLinkReconfiguration2(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_PHYSICALSHAREDCHANNELRECONFIGURATION: 			/*	37 */
		offset = dissect_id_physicalSharedChannelReconfiguration2(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_INFORMATIONEXCHANGEINITIATION:					/*	41 */
		offset = dissect_id_informationExchangeInitiation2(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_CELLSYNCHRONISATIONADJUSTMENT:					/*	44 */
		offset = dissect_id_cellSynchronisationAdjustment2(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_CELLSYNCHRONISATIONINITIATION:					/*	45 */
		offset = dissect_id_cellSynchronisationInitiation2(tvb, offset, actx, value_tree);
		break;
	case NBAP_ID_CELLSYNCHRONISATIONRECONFIGURATION:			/*	46 */
		offset = dissect_id_cellSynchronisationReconfiguration2(tvb, offset, actx, value_tree);
		break;
	default:
		offset = offset + length;
		break;
	}
	BYTE_ALIGN_OFFSET(offset)
	return offset;
}

static int dissect_nbap_ProtocolIEValueValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree){
	proto_item	*value_item = NULL;
	proto_tree	*value_tree = NULL;
	guint length;

    value_item = proto_tree_add_item(tree, hf_nbap_value, tvb, 0, -1, FALSE);
    value_tree = proto_item_add_subtree(value_item, ett_nbap_ProtocolIEValueValue);

	offset = dissect_per_length_determinant(tvb, offset, actx, value_tree, hf_nbap_IE_length, &length);
	proto_item_set_len(value_item,length);

	
	switch(ProtocolIE_ID){
	case NBAP_IE_ID_AICH_INFORMATION:												/*0*/
		offset = dissect_id_AICH_Information(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_AICH_INFORMATIONITEM_RESOURCESTATUSIND:							/*1*/
		offset = offset + length;
		break;
	case NBAP_IE_ID_BCH_INFORMATION:												/*7*/
		offset = dissect_id_BCH_Information(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_BCH_INFORMATIONITEM_RESOURCESTATUSIND:							/*8*/
		offset = offset + length;
		break;
	case NBAP_IE_ID_BCCH_MODIFICATIONTIME:											/*9*/
		offset = dissect_id_BCCH_ModificationTime(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_BLOCKINGPRIORITYINDICATOR:										/*10*/
		offset = dissect_id_BlockingPriorityIndicator(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CAUSE:															/*13*/
		offset = dissect_id_Cause(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CCP_INFORMATIONITEM_AUDITRSP:									/*14*/
		offset = dissect_id_CCP_InformationItem_AuditRsp(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CCP_INFORMATIONLIST_AUDITRSP:									/*15*/
		offset = dissect_id_CCP_InformationList_AuditRsp(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CCP_INFORMATIONITEM_RESOURCESTATUSIND:							/*16*/
		offset = dissect_id_CCP_InformationItem_ResourceStatusInd(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CELL_INFORMATIONITEM_AUDITRSP:									/*17*/
		offset = dissect_id_Cell_InformationItem_AuditRsp(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CELL_INFORMATIONITEM_RESOURCESTATUSIND:							/*18*/
		offset = dissect_id_Cell_InformationItem_ResourceStatusInd(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CELL_INFORMATIONLIST_AUDITRSP:									/*19*/
		offset = dissect_id_Cell_InformationList_AuditRsp(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CELLPARAMETERID:												/*23*/
		offset = dissect_id_CellParameterID(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CFN:															/*24*/
		offset = dissect_id_CFN(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_C_ID:															/*25*/
		offset = dissect_id_C_ID(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_COMMONMEASUREMENTACCURACY:										/*39*/
		offset = dissect_id_CommonMeasurementAccuracy(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_COMMONMEASUREMENTOBJECTTYPE_CM_RPRT:							/* 31 */
		offset = dissect_id_CommonMeasurementObjectType_CM_Rprt(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_COMMONMEASUREMENTOBJECTTYPE_CM_RQST:							/*32*/
		offset = dissect_id_CommonMeasurementObjectType_CM_Rqst(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_COMMONMEASUREMENTOBJECTTYPE_CM_RSP:								/*33*/
		offset = dissect_id_CommonMeasurementObjectType_CM_Rsp(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_COMMONMEASUREMENTTYPE:											/*34*/
		offset = dissect_id_CommonMeasurementType(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_COMMONPHYSICALCHANNELID:										/*35*/
		offset = dissect_id_CommonPhysicalChannelID(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_COMMONPHYSICALCHANNELTYPE_CTCH_SETUPRQSTFDD:					/*36*/
		offset = dissect_id_CommonPhysicalChannelType_CTCH_SetupRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_COMMONPHYSICALCHANNELTYPE_CTCH_SETUPRQSTTDD:					/*37*/
		offset = dissect_id_CommonPhysicalChannelType_CTCH_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_COMMUNICATIONCONTROLPORTID:										/*40*/
		offset = dissect_id_CommunicationControlPortID(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CONFIGURATIONGENERATIONID:										/*43*/
		offset = dissect_id_ConfigurationGenerationID(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CRNC_COMMUNICATIONCONTEXTID:									/*44*/
		offset = dissect_id_CRNC_CommunicationContextID(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CRITICALITYDIAGNOSTICS:											/*45*/
		offset = dissect_id_CriticalityDiagnostics(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DCHS_TO_ADD_FDD:												/*48*/
		offset = dissect_id_DCHs_to_Add_FDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DCH_ADDLIST_RL_RECONFPREPTDD:									/*49*/
		offset = offset + length;
		break;
	case NBAP_IE_ID_DCHS_TO_ADD_TDD:												/*50*/
		offset = dissect_id_DCHs_to_Add_TDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DCH_DELETELIST_RL_RECONFPREPFDD:								/*52*/
		offset = dissect_id_DCH_DeleteList_RL_ReconfPrepFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DCH_DELETELIST_RL_RECONFPREPTDD:								/*53*/
		offset = dissect_id_DCH_DeleteList_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DCH_DELETELIST_RL_RECONFRQSTFDD:								/*54*/
		offset = dissect_id_DCH_DeleteList_RL_ReconfRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DCH_DELETELIST_RL_RECONFRQSTTDD:								/*55*/
		offset = dissect_id_DCH_DeleteList_RL_ReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DCH_FDD_INFORMATION:											/*56*/
		offset = dissect_id_DCH_FDD_Information(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DCH_TDD_INFORMATION:						/*	57 */
		offset = dissect_id_DCH_TDD_Information(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DCH_INFORMATIONRESPONSE:					/*	59 */
		offset = dissect_id_DCH_InformationResponse(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_FDD_DCHS_TO_MODIFY:							/*	62 */
		offset = dissect_id_FDD_DCHs_to_Modify(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_TDD_DCHS_TO_MODIFY:							/*	63 */
		offset = dissect_id_TDD_DCHs_to_Modify(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DCH_MODIFYLIST_RL_RECONFRQSTTDD:			/*	65 */
		/*offset = dissect_id_DCH_ModifyList_RL_ReconfRqstTDD(tvb, offset, actx, value_tree);*/
		break;
	case NBAP_IE_ID_DCH_REARRANGELIST_BEARER_REARRANGEIND:		/*	135 */
		offset = dissect_id_DCH_RearrangeList_Bearer_RearrangeInd(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DEDICATEDMEASUREMENTOBJECTTYPE_DM_RPRT:		/*	67 */
		offset = dissect_id_DedicatedMeasurementObjectType_DM_Rprt(tvb, offset, actx, value_tree);;
		break;
	case NBAP_IE_ID_DEDICATEDMEASUREMENTOBJECTTYPE_DM_RQST:		/*	68 */
		offset = dissect_id_DedicatedMeasurementObjectType_DM_Rqst(tvb, offset, actx, value_tree);;
		break;
	case NBAP_IE_ID_DEDICATEDMEASUREMENTOBJECTTYPE_DM_RSP:		/*	69 */
		offset = dissect_id_DedicatedMeasurementObjectType_DM_Rsp(tvb, offset, actx, value_tree);;
		break;
	case NBAP_IE_ID_DEDICATEDMEASUREMENTTYPE:					/*	70 */
		offset = dissect_id_DedicatedMeasurementType(tvb, offset, actx, value_tree);
		break;	
	case NBAP_IE_ID_DL_CCTRCH_INFORMATIONITEM_RL_SETUPRQSTTDD:	/*	72 */
		offset = dissect_id_DL_CCTrCH_InformationItem_RL_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;	
	case NBAP_IE_ID_DL_CCTRCH_INFORMATIONLIST_RL_ADDITIONRQSTTDD:/*	73 */
		offset = dissect_id_DL_CCTrCH_InformationList_RL_AdditionRqstTDD(tvb, offset, actx, value_tree);
		break;	
	case NBAP_IE_ID_DL_CCTRCH_INFORMATIONLIST_RL_SETUPRQSTTDD:	/*	76 */
		offset = dissect_id_DL_CCTrCH_InformationList_RL_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;	
	case NBAP_IE_ID_DL_DPCH_INFORMATIONITEM_RL_ADDITIONRQSTTDD:	/*	77 */
		offset = dissect_id_DL_DPCH_InformationItem_RL_AdditionRqstTDD(tvb, offset, actx, value_tree);
		break;	
	case NBAP_IE_ID_DL_DPCH_INFORMATIONLIST_RL_SETUPRQSTTDD:	/*	79 */
		offset = dissect_id_DL_DPCH_InformationList_RL_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;	
	case NBAP_IE_ID_DL_DPCH_INFORMATION_RL_RECONFPREPFDD:		/*	81 */
		offset = dissect_id_DL_DPCH_Information_RL_ReconfPrepFDD(tvb, offset, actx, value_tree);
		break;	
	case NBAP_IE_ID_DL_DPCH_INFORMATION_RL_RECONFRQSTFDD:		/*	82 */
		offset = dissect_id_DL_DPCH_Information_RL_ReconfRqstFDD(tvb, offset, actx, value_tree);
		break;	
	case NBAP_IE_ID_DL_DPCH_INFORMATION_RL_SETUPRQSTFDD:		/*	83 */
		offset = dissect_id_DL_DPCH_Information_RL_SetupRqstFDD(tvb, offset, actx, value_tree);
		break;	
	case NBAP_IE_ID_DL_DPCH_TIMINGADJUSTMENT:					/*	21 */
		offset = dissect_id_DL_DPCH_TimingAdjustment(tvb, offset, actx, value_tree);
		break;	
	case NBAP_IE_ID_DL_REFERENCEPOWERINFORMATIONITEM_DL_PC_RQST:/*	84 */
		offset = dissect_id_DL_ReferencePowerInformationItem_DL_PC_Rqst(tvb, offset, actx, value_tree);
		break;	
	case NBAP_IE_ID_DLREFERENCEPOWER:							/*	85 */
		offset = dissect_id_DLReferencePower(tvb, offset, actx, value_tree);
		break;	
	case NBAP_IE_ID_DLREFERENCEPOWERLIST_DL_PC_RQST:			/*	86 */
		offset = dissect_id_DLReferencePowerList_DL_PC_Rqst(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UNUSED_PROTOCOLIE_ID_87:					/*	87 */
		break;
	case NBAP_IE_ID_UNUSED_PROTOCOLIE_ID_89:					/*	89 */
		break;
	case NBAP_IE_ID_UNUSED_PROTOCOLIE_ID_91:					/*	91 */
		break;
	case NBAP_IE_ID_UNUSED_PROTOCOLIE_ID_93:					/*	93 */
		break;
	case NBAP_IE_ID_DSCHS_TO_ADD_TDD:							/*	96 */
		offset = dissect_id_DSCHs_to_Add_TDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DSCH_INFORMATION_DELETELIST_RL_RECONFPREPTDD:/*	98 */
		offset = dissect_id_DSCH_Information_DeleteList_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DSCH_INFORMATION_MODIFYLIST_RL_RECONFPREPTDD:/*	100 */
		offset = dissect_id_DSCH_Information_ModifyList_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DSCH_INFORMATIONRESPONSE:					/*	105 */
		offset = dissect_id_DSCH_InformationResponse(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UNUSED_PROTOCOLIE_ID_106:					/*	106 */
		break;
	case NBAP_IE_ID_DSCH_TDD_INFORMATION:						/*	107 */
		offset = dissect_id_DSCH_TDD_Information(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UNUSED_PROTOCOLIE_ID_108:					/*	108 */
		break;
	case NBAP_IE_ID_UNUSED_PROTOCOLIE_ID_112:					/*	112 */
		break;
	case NBAP_IE_ID_DSCH_REARRANGELIST_BEARER_REARRANGEIND:		/*	136 */
		offset = dissect_id_DSCH_RearrangeList_Bearer_RearrangeInd(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_END_OF_AUDIT_SEQUENCE_INDICATOR: 			/*	113 */
		offset = dissect_id_End_Of_Audit_Sequence_Indicator(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_FACH_INFORMATION:							/*	116 */
		offset = dissect_id_FACH_Information(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_FACH_INFORMATIONITEM_RESOURCESTATUSIND:		/*	117 */
		break;
	case NBAP_IE_ID_FACH_PARAMETERSLIST_CTCH_RECONFRQSTTDD:		/*	120 */
		offset = dissect_id_FACH_ParametersList_CTCH_ReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_FACH_PARAMETERSLISTIE_CTCH_SETUPRQSTFDD:	/*	121 */
		offset = dissect_id_FACH_ParametersListIE_CTCH_SetupRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_FACH_PARAMETERSLISTIE_CTCH_SETUPRQSTTDD:	/*	122 */
		offset = dissect_id_FACH_ParametersListIE_CTCH_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_INDICATIONTYPE_RESOURCESTATUSIND:			/*	123 */
		offset = dissect_id_IndicationType_ResourceStatusInd(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_LOCAL_CELL_ID:								/*	124 */
		offset = dissect_id_Local_Cell_ID(tvb, offset, actx, value_tree);
		break;	
	case NBAP_IE_ID_LOCAL_CELL_GROUP_INFORMATIONITEM_AUDITRSP:			/*	2 */
		offset = dissect_id_Local_Cell_Group_InformationItem_AuditRsp(tvb, offset, actx, value_tree);
		break;	
	case NBAP_IE_ID_LOCAL_CELL_GROUP_INFORMATIONITEM_RESOURCESTATUSIND:	/*	3 */
		offset = dissect_id_Local_Cell_Group_InformationItem_ResourceStatusInd(tvb, offset, actx, value_tree);
		break;	
	case NBAP_IE_ID_LOCAL_CELL_GROUP_INFORMATIONITEM2_RESOURCESTATUSIND:/*	4 */
		offset = dissect_id_Local_Cell_Group_InformationItem2_ResourceStatusInd(tvb, offset, actx, value_tree);
		break;	
	case NBAP_IE_ID_LOCAL_CELL_GROUP_INFORMATIONLIST_AUDITRSP:			/*	5 */
		offset = dissect_id_Local_Cell_Group_InformationList_AuditRsp(tvb, offset, actx, value_tree);
		break;	
	case NBAP_IE_ID_LOCAL_CELL_INFORMATIONITEM_AUDITRSP:				/*	125 */
		offset = dissect_id_Local_Cell_InformationItem_AuditRsp(tvb, offset, actx, value_tree);
		break;	
	case NBAP_IE_ID_LOCAL_CELL_INFORMATIONITEM_RESOURCESTATUSIND:		/*	126 */
		offset = dissect_id_Local_Cell_InformationItem_ResourceStatusInd(tvb, offset, actx, value_tree);
		break;	
	case NBAP_IE_ID_LOCAL_CELL_INFORMATIONITEM2_RESOURCESTATUSIND:		/*	127 */
		offset = dissect_id_Local_Cell_InformationItem2_ResourceStatusInd(tvb, offset, actx, value_tree);
		break;	
	case NBAP_IE_ID_LOCAL_CELL_INFORMATIONLIST_AUDITRSP:				/*	128 */
		offset = dissect_id_Local_Cell_InformationList_AuditRsp(tvb, offset, actx, value_tree);
		break;	
	case NBAP_IE_ID_ADJUSTMENTPERIOD:									/*	129 */
		offset = dissect_id_AdjustmentPeriod(tvb, offset, actx, value_tree);
		break;	
	case NBAP_IE_ID_MAXADJUSTMENTSTEP:									/*	130 */
		offset = dissect_id_MaxAdjustmentStep(tvb, offset, actx, value_tree);
		break;	
	case NBAP_IE_ID_MAXIMUMTRANSMISSIONPOWER:							/*	131 */
		offset = dissect_id_MaximumTransmissionPower(tvb, offset, actx, value_tree);
		break;	
	case NBAP_IE_ID_MEASUREMENTFILTERCOEFFICIENT:						/*	132 */
		offset = dissect_id_MeasurementFilterCoefficient(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_MEASUREMENTID:	/* 133 */
		offset = dissect_id_MeasurementID(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_MESSAGESTRUCTURE: /* 115 */
		offset = dissect_id_MessageStructure(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_MIB_SB_SIB_INFORMATIONLIST_SYSTEMINFOUPDATERQST:	/*	134 */
		offset = dissect_id_MIB_SB_SIB_InformationList_SystemInfoUpdateRqst(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_NODEB_COMMUNICATIONCONTEXTID:						/*	143 */
		offset = dissect_id_NodeB_CommunicationContextID(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_NEIGHBOURINGCELLMEASUREMENTINFORMATION:				/*	455 */
		offset = dissect_id_NeighbouringCellMeasurementInformation(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_P_CCPCH_INFORMATION:								/*	144 */
		offset = dissect_id_P_CCPCH_Information(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_P_CCPCH_INFORMATIONITEM_RESOURCESTATUSIND:			/*	145 */
		offset = dissect_id_P_CPICH_Information(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_P_CPICH_INFORMATION:								/*	146 */
		break;
	case NBAP_IE_ID_P_CPICH_INFORMATIONITEM_RESOURCESTATUSIND:			/*	147 */
		break;
	case NBAP_IE_ID_P_SCH_INFORMATION:									/*	148 */
		offset = dissect_id_P_SCH_Information(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PCCPCH_INFORMATION_CELL_RECONFRQSTTDD:				/*	150 */
		offset = dissect_id_PCCPCH_Information_Cell_ReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PCCPCH_INFORMATION_CELL_SETUPRQSTTDD:				/*	151 */
		offset = dissect_id_PCCPCH_Information_Cell_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PCH_PARAMETERS_CTCH_RECONFRQSTTDD:					/*	155 */
		offset = dissect_id_PCH_Parameters_CTCH_ReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PCH_PARAMETERSITEM_CTCH_SETUPRQSTFDD:				/*	156 */
		offset = dissect_id_PCH_ParametersItem_CTCH_SetupRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PCH_PARAMETERSITEM_CTCH_SETUPRQSTTDD:				/*	157 */
		offset = dissect_id_PCH_ParametersItem_CTCH_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PCH_INFORMATION:									/*	158 */
		offset = dissect_id_PCH_Information(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PDSCH_INFORMATION_ADDLISTIE_PSCH_RECONFRQST:		/*	161 */
		offset = dissect_id_PDSCH_Information_AddListIE_PSCH_ReconfRqst(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PDSCH_INFORMATION_MODIFYLISTIE_PSCH_RECONFRQST:		/*	162 */
		offset = dissect_id_PDSCH_Information_ModifyListIE_PSCH_ReconfRqst(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PDSCHSETS_ADDLIST_PSCH_RECONFRQST:					/*	163 */
		offset = dissect_id_PDSCHSets_AddList_PSCH_ReconfRqst(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PDSCHSETS_DELETELIST_PSCH_RECONFRQST:				/*	164 */
		offset = dissect_id_PDSCHSets_DeleteList_PSCH_ReconfRqst(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PDSCHSETS_MODIFYLIST_PSCH_RECONFRQST:				/*	165 */
		offset = dissect_id_PDSCHSets_ModifyList_PSCH_ReconfRqst(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PICH_INFORMATION:									/*	166 */
		offset = dissect_id_PICH_Information(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PICH_PARAMETERS_CTCH_RECONFRQSTTDD:					/*	168 */
		offset = dissect_id_PICH_Parameters_CTCH_ReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_POWERADJUSTMENTTYPE:								/*	169 */
		offset = dissect_id_PowerAdjustmentType(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PRACH_INFORMATION:									/*	170 */
		offset = dissect_id_PRACH_Information(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PRIMARYCCPCH_INFORMATION_CELL_RECONFRQSTFDD:		/*	175 */
		offset = dissect_id_PrimaryCCPCH_Information_Cell_ReconfRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PRIMARYCCPCH_INFORMATION_CELL_SETUPRQSTFDD:			/*	176 */
		offset = dissect_id_PrimaryCCPCH_Information_Cell_SetupRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PRIMARYCPICH_INFORMATION_CELL_RECONFRQSTFDD:		/*	177 */
		offset = dissect_id_PrimaryCPICH_Information_Cell_ReconfRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PRIMARYCPICH_INFORMATION_CELL_SETUPRQSTFDD:			/*	178 */
		offset = dissect_id_PrimaryCPICH_Information_Cell_SetupRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PRIMARYSCH_INFORMATION_CELL_RECONFRQSTFDD:			/*	179 */
		offset = dissect_id_PrimarySCH_Information_Cell_ReconfRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PRIMARYSCH_INFORMATION_CELL_SETUPRQSTFDD:			/*	180 */
		offset = dissect_id_PrimarySCH_Information_Cell_SetupRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PRIMARYSCRAMBLINGCODE:								/*	181 */
		offset = dissect_id_PrimaryScramblingCode(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_SCH_INFORMATION_CELL_RECONFRQSTTDD:					/*	183 */
		offset = dissect_id_SCH_Information_Cell_ReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_SCH_INFORMATION_CELL_SETUPRQSTTDD:					/*	184 */
		offset = dissect_id_SCH_Information_Cell_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PUSCH_INFORMATION_ADDLISTIE_PSCH_RECONFRQST:		/*	185 */
		offset = dissect_id_PUSCH_Information_AddListIE_PSCH_ReconfRqst(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PUSCH_INFORMATION_MODIFYLISTIE_PSCH_RECONFRQST:		/*	186 */
		offset = dissect_id_PUSCH_Information_ModifyListIE_PSCH_ReconfRqst(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PUSCHSETS_ADDLIST_PSCH_RECONFRQST:					/*	187 */
		offset = dissect_id_PUSCHSets_AddList_PSCH_ReconfRqst(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PUSCHSETS_DELETELIST_PSCH_RECONFRQST:				/*	188 */
		offset = dissect_id_PUSCHSets_DeleteList_PSCH_ReconfRqst(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PUSCHSETS_MODIFYLIST_PSCH_RECONFRQST:				/*	189 */
		offset = dissect_id_PUSCHSets_ModifyList_PSCH_ReconfRqst(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RACH_INFORMATION:									/*	190 */
		offset = dissect_id_RACH_Information(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RACH_PARAMETERSITEM_CTCH_SETUPRQSTFDD:				/*	196 */
		offset = dissect_id_RACH_ParametersItem_CTCH_SetupRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RACH_PARAMETERITEM_CTCH_SETUPRQSTTDD:				/*	197 */
		offset = dissect_id_RACH_ParameterItem_CTCH_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_REPORTCHARACTERISTICS:								/*	198 */
		offset = dissect_id_ReportCharacteristics(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_REPORTING_OBJECT_RL_FAILUREIND:						/*	199 */
		offset = dissect_id_Reporting_Object_RL_FailureInd(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_REPORTING_OBJECT_RL_RESTOREIND:						/*	200 */
		offset = dissect_id_Reporting_Object_RL_RestoreInd(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RL_INFORMATIONITEM_DM_RPRT:							/*	202 */
		offset = dissect_id_RL_InformationItem_DM_Rprt(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RL_INFORMATIONITEM_DM_RQST:							/*	203 */
		offset = dissect_id_RL_InformationItem_DM_Rqst(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RL_INFORMATIONITEM_DM_RSP:							/*	204 */
		offset = dissect_id_RL_InformationItem_DM_Rsp(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RL_INFORMATIONITEM_RL_ADDITIONRQSTFDD:				/*	205 */
		offset = dissect_id_RL_InformationItem_RL_AdditionRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RL_INFORMATIONITEM_RL_DELETIONRQST:					/*	206 */
		offset = dissect_id_RL_informationItem_RL_DeletionRqst(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RL_INFORMATIONITEM_RL_FAILUREIND:					/*	207 */
		offset = dissect_id_RL_InformationItem_RL_FailureInd(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RL_INFORMATIONITEM_RL_PREEMPTREQUIREDIND:			/*	286 */
		offset = dissect_id_RL_InformationItem_RL_PreemptRequiredInd(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RL_INFORMATIONITEM_RL_RECONFPREPFDD:				/*	208 */
		offset = dissect_id_RL_InformationItem_RL_ReconfPrepFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RL_INFORMATIONITEM_RL_RECONFRQSTFDD:				/*	209 */
		offset = dissect_id_RL_InformationItem_RL_ReconfRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RL_INFORMATIONITEM_RL_RESTOREIND:					/*	210 */
		offset = dissect_id_RL_InformationItem_RL_RestoreInd(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RL_INFORMATIONITEM_RL_SETUPRQSTFDD:					/*	211 */
		offset = dissect_id_RL_InformationItem_RL_SetupRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RL_INFORMATIONLIST_RL_ADDITIONRQSTFDD:				/*	212 */
		offset = dissect_id_RL_InformationList_RL_AdditionRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RL_INFORMATIONLIST_RL_DELETIONRQST:					/*	213 */
		offset = dissect_id_RL_informationList_RL_DeletionRqst(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RL_INFORMATIONLIST_RL_PREEMPTREQUIREDIND:			/*	237 */
		offset = dissect_id_RL_InformationList_RL_ReconfPrepFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RL_INFORMATIONLIST_RL_RECONFPREPFDD:				/*	214 */
		/*offset = (tvb, offset, actx, value_tree);*/
		break;
	case NBAP_IE_ID_RL_INFORMATIONLIST_RL_RECONFRQSTFDD:				/*	215 */
		offset = dissect_id_RL_InformationList_RL_ReconfRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RL_INFORMATIONLIST_RL_SETUPRQSTFDD:					/*	216 */
		offset = dissect_id_RL_InformationList_RL_SetupRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RL_INFORMATIONRESPONSEITEM_RL_ADDITIONRSPFDD:		/*	217 */
		offset = dissect_id_RL_InformationResponseItem_RL_AdditionRspFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RL_INFORMATIONRESPONSEITEM_RL_RECONFREADY:			/*	218 */
		offset = dissect_id_RL_InformationResponseItem_RL_ReconfReady(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RL_INFORMATIONRESPONSEITEM_RL_RECONFRSP:			/*	219 */
		offset = dissect_id_RL_InformationResponseItem_RL_ReconfRsp(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RL_INFORMATIONRESPONSEITEM_RL_SETUPRSPFDD:			/*	220 */
		offset = dissect_id_RL_InformationResponseItem_RL_SetupRspFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RL_INFORMATIONRESPONSELIST_RL_ADDITIONRSPFDD:		/*	221 */
		offset = dissect_id_RL_InformationResponseList_RL_AdditionRspFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RL_INFORMATIONRESPONSELIST_RL_RECONFREADY:			/*	222 */
		offset = dissect_id_RL_InformationResponseList_RL_ReconfReady(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RL_INFORMATIONRESPONSELIST_RL_RECONFRSP:			/*	223 */
		offset = dissect_id_RL_InformationResponseList_RL_ReconfRsp(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RL_INFORMATIONRESPONSELIST_RL_SETUPRSPFDD:			/*	224 */
		offset = dissect_id_RL_InformationResponseList_RL_SetupRspFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RL_INFORMATIONRESPONSE_RL_ADDITIONRSPTDD:			/*	225 */
		offset = dissect_id_RL_InformationResponse_RL_AdditionRspTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RL_INFORMATIONRESPONSE_RL_SETUPRSPTDD:				/*	226 */
		offset = dissect_id_RL_InformationResponse_RL_SetupRspTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RL_INFORMATION_RL_ADDITIONRQSTTDD:					/*	227 */
		offset = dissect_id_RL_Information_RL_AdditionRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RL_INFORMATION_RL_RECONFRQSTTDD:					/*	228 */
		offset = dissect_id_RL_Information_RL_ReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RL_INFORMATION_RL_RECONFPREPTDD:					/*	229 */
		offset = dissect_id_RL_Information_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RL_INFORMATION_RL_SETUPRQSTTDD:						/*	230 */
		offset = dissect_id_RL_Information_RL_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RL_RECONFIGURATIONFAILUREITEM_RL_RECONFFAILURE:		/*	236 */
		offset = dissect_id_RL_ReconfigurationFailureItem_RL_ReconfFailure(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RL_SET_INFORMATIONITEM_DM_RPRT:						/*	238 */
		offset = dissect_id_RL_Set_InformationItem_DM_Rprt(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RL_SET_INFORMATIONITEM_DM_RSP:						/*	240 */
		offset = dissect_id_RL_Set_InformationItem_DM_Rsp(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RL_SET_INFORMATIONITEM_RL_FAILUREIND:				/*	241 */
		offset = dissect_id_RL_Set_InformationItem_RL_FailureInd(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RL_SET_INFORMATIONITEM_RL_RESTOREIND:				/*	242 */
		offset = dissect_id_RL_Set_InformationItem_RL_RestoreInd(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_S_CCPCH_INFORMATION:								/*	247 */
		offset = dissect_id_S_CCPCH_Information(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_S_CPICH_INFORMATION:								/*	249 */
		offset = dissect_id_S_CPICH_Information(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_SCH_INFORMATION:									/*	251 */
		offset = dissect_id_SCH_Information(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_S_SCH_INFORMATION:									/*	253 */
		offset = dissect_id_S_SCH_Information(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_SECONDARY_CCPCHLISTIE_CTCH_RECONFRQSTTDD:			/*	257 */
		offset = dissect_id_Secondary_CCPCHListIE_CTCH_ReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_SECONDARY_CCPCH_PARAMETERLISTIE_CTCH_SETUPRQSTTDD:	/*	258 */
		offset = dissect_id_Secondary_CCPCH_parameterListIE_CTCH_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_SECONDARY_CCPCH_PARAMETERS_CTCH_RECONFRQSTTDD:		/*	259 */
		offset = dissect_id_Secondary_CCPCH_Parameters_CTCH_ReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_SECONDARYCPICH_INFORMATIONITEM_CELL_RECONFRQSTFDD:	/*	260 */
		offset = dissect_id_SecondaryCPICH_InformationItem_Cell_ReconfRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_SECONDARYCPICH_INFORMATIONITEM_CELL_SETUPRQSTFDD:	/*	261 */
		offset = dissect_id_SecondaryCPICH_InformationItem_Cell_SetupRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_SECONDARYCPICH_INFORMATIONLIST_CELL_RECONFRQSTFDD:	/*	262 */
		offset = dissect_id_SecondaryCPICH_InformationList_Cell_ReconfRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_SECONDARYCPICH_INFORMATIONLIST_CELL_SETUPRQSTFDD:	/*	263 */
		offset = dissect_id_SecondaryCPICH_InformationList_Cell_SetupRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_SECONDARYSCH_INFORMATION_CELL_RECONFRQSTFDD:		/*	264 */
		offset = dissect_id_SecondarySCH_Information_Cell_ReconfRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_SECONDARYSCH_INFORMATION_CELL_SETUPRQSTFDD:			/*	265 */
		offset = dissect_id_SecondarySCH_Information_Cell_SetupRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_SEGMENTINFORMATIONLISTIE_SYSTEMINFOUPDATE:			/*	266 */
		offset = dissect_id_SegmentInformationListIE_SystemInfoUpdate(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_SFN:												/* 268 */
		offset = dissect_id_SFN(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_SIGNALLINGBEARERREQUESTINDICATOR:					/*	138 */
		offset = dissect_id_SignallingBearerRequestIndicator(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_SHUTDOWNTIMER:										/*	269 */
		offset = dissect_id_ShutdownTimer(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_START_OF_AUDIT_SEQUENCE_INDICATOR:					/*	114 */
		offset = dissect_id_Start_Of_Audit_Sequence_Indicator(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_SUCCESSFUL_RL_INFORMATIONRESPITEM_RL_ADDITIONFAILUREFDD:/*270 */
		offset = dissect_id_Successful_RL_InformationRespItem_RL_AdditionFailureFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_SUCCESSFUL_RL_INFORMATIONRESPITEM_RL_SETUPFAILUREFDD:	/*	271 */
		offset = dissect_id_Successful_RL_InformationRespItem_RL_SetupFailureFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_SYNCCASE:												/*	274 */
		offset = dissect_id_SyncCase(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_SYNCCASEINDICATORITEM_CELL_SETUPRQSTTDD_PSCH:			/*	275 */
		offset = dissect_id_SyncCaseIndicatorItem_Cell_SetupRqstTDD_PSCH(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_T_CELL:													/*	276 */
		offset = dissect_id_T_Cell(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_TARGETCOMMUNICATIONCONTROLPORTID:						/*	139 */
		offset = dissect_id_TargetCommunicationControlPortID(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_TIMESLOTCONFIGURATIONLIST_CELL_RECONFRQSTTDD:			/*	277 */
		offset = dissect_id_TimeSlotConfigurationList_Cell_ReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_TIMESLOTCONFIGURATIONLIST_CELL_SETUPRQSTTDD:			/*	278 */
		offset = dissect_id_TimeSlotConfigurationList_Cell_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_TRANSMISSIONDIVERSITYAPPLIED:							/*	279 */
		offset = dissect_id_TransmissionDiversityApplied(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_TYPEOFERROR:											/*	508 */
		offset = dissect_id_TypeOfError(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UARFCNFORNT:											/*	280 */
		offset = dissect_id_UARFCNforNt(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UARFCNFORND:											/*	281 */
		offset = dissect_id_UARFCNforNd(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UARFCNFORNU:											/*	282 */
		offset = dissect_id_UARFCNforNu(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UL_CCTRCH_INFORMATIONITEM_RL_SETUPRQSTTDD:				/*	284 */
		offset = dissect_id_UL_CCTrCH_InformationItem_RL_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UL_CCTRCH_INFORMATIONLIST_RL_ADDITIONRQSTTDD:			/*	285 */
		offset = dissect_id_UL_CCTrCH_InformationList_RL_AdditionRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UL_CCTRCH_INFORMATIONLIST_RL_SETUPRQSTTDD:				/*	288 */
		offset = dissect_id_UL_CCTrCH_InformationList_RL_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UL_DPCH_INFORMATIONITEM_RL_ADDITIONRQSTTDD:				/*	289 */
		offset = dissect_id_UL_DPCH_InformationItem_RL_AdditionRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UL_DPCH_INFORMATIONLIST_RL_SETUPRQSTTDD:				/*	291 */
		offset = dissect_id_UL_DPCH_InformationList_RL_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UL_DPCH_INFORMATION_RL_RECONFPREPFDD:					/*	293 */
		offset = dissect_id_UL_DPCH_Information_RL_ReconfPrepFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UL_DPCH_INFORMATION_RL_RECONFRQSTFDD:					/*	294 */
		offset = dissect_id_UL_DPCH_Information_RL_ReconfRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UL_DPCH_INFORMATION_RL_SETUPRQSTFDD:					/*	295 */
		offset = dissect_id_UL_DPCH_Information_RL_SetupRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UNSUCCESSFUL_RL_INFORMATIONRESPITEM_RL_ADDITIONFAILUREFDD:	/*	296 */
		offset = dissect_id_Unsuccessful_RL_InformationRespItem_RL_AdditionFailureFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UNSUCCESSFUL_RL_INFORMATIONRESPITEM_RL_SETUPFAILUREFDD:	/*	297 */
		offset = dissect_id_Unsuccessful_RL_InformationRespItem_RL_SetupFailureFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UNSUCCESSFUL_RL_INFORMATIONRESP_RL_ADDITIONFAILURETDD:	/*	300 */
		offset = dissect_id_Unsuccessful_RL_InformationResp_RL_AdditionFailureTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UNSUCCESSFUL_RL_INFORMATIONRESP_RL_SETUPFAILURETDD:		/*	301 */
		offset = dissect_id_Unsuccessful_RL_InformationResp_RL_SetupFailureTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_USCH_INFORMATION_ADD:									/*	302 */
		offset = dissect_id_USCH_Information_Add(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_USCH_INFORMATION_DELETELIST_RL_RECONFPREPTDD:			/*	304 */
		offset = dissect_id_USCH_Information_DeleteList_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_USCH_INFORMATION_MODIFYLIST_RL_RECONFPREPTDD:			/*	306 */
		offset = dissect_id_USCH_Information_ModifyList_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_USCH_INFORMATIONRESPONSE:								/*	309 */
		offset = dissect_id_USCH_InformationResponse(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_USCH_INFORMATION:										/*	310 */
		offset = dissect_id_USCH_Information(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_USCH_REARRANGELIST_BEARER_REARRANGEIND:					/*	141 */
		offset = dissect_id_USCH_RearrangeList_Bearer_RearrangeInd(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_ACTIVE_PATTERN_SEQUENCE_INFORMATION: 					/*	315 */
		offset = dissect_id_Active_Pattern_Sequence_Information(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_AICH_PARAMETERSLISTIE_CTCH_RECONFRQSTFDD:				/*	316 */
		offset = dissect_id_AICH_ParametersListIE_CTCH_ReconfRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_ADJUSTMENTRATIO:										/*	317 */
		offset = dissect_id_AdjustmentRatio(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_NOT_USED_320:											/*	320 */
	case NBAP_IE_ID_NOT_USED_322:											/*	322 */
		break;
	case NBAP_IE_ID_FACH_PARAMETERSLISTIE_CTCH_RECONFRQSTFDD:				/*	323 */
		offset = dissect_id_FACH_ParametersListIE_CTCH_ReconfRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CAUSELEVEL_PSCH_RECONFFAILURE:							/*	324 */
		offset = dissect_id_CauseLevel_PSCH_ReconfFailure(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CAUSELEVEL_RL_ADDITIONFAILUREFDD:						/*	325 */
		offset = dissect_id_CauseLevel_RL_AdditionFailureFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CAUSELEVEL_RL_ADDITIONFAILURETDD:						/*	326 */
		offset = dissect_id_CauseLevel_RL_AdditionFailureTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CAUSELEVEL_RL_RECONFFAILURE:							/*	327 */
		offset = dissect_id_CauseLevel_RL_ReconfFailure(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CAUSELEVEL_RL_SETUPFAILUREFDD:							/*	328 */
		offset = dissect_id_CauseLevel_RL_SetupFailureFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CAUSELEVEL_RL_SETUPFAILURETDD:							/*	329 */
		offset = dissect_id_CauseLevel_RL_SetupFailureTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_NOT_USED_330:											/*	330 */
	case NBAP_IE_ID_NOT_USED_332:											/*	332 */
		break;
	case NBAP_IE_ID_CLOSED_LOOP_TIMING_ADJUSTMENT_MODE:						/*	333 */
		offset = dissect_id_Closed_Loop_Timing_Adjustment_Mode(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_COMMONPHYSICALCHANNELTYPE_CTCH_RECONFRQSTFDD:			/*	334 */
		offset = dissect_id_CommonPhysicalChannelType_CTCH_ReconfRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_COMPRESSED_MODE_DEACTIVATION_FLAG:						/*	335 */
		offset = dissect_id_Compressed_Mode_Deactivation_Flag(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_NOT_USED_336:											/*	336 */
	case NBAP_IE_ID_NOT_USED_342:											/*	342 */
	case NBAP_IE_ID_NOT_USED_343:											/*	343 */
		break;
	case NBAP_IE_ID_DL_CCTRCH_INFORMATIONADDLIST_RL_RECONFPREPTDD:			/*	346 */
		offset = dissect_id_DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DL_CCTRCH_INFORMATIONDELETEITEM_RL_RECONFRQSTTDD:		/*	347 */
		offset = dissect_id_DL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DL_CCTRCH_INFORMATIONDELETELIST_RL_RECONFPREPTDD:		/*	348 */
		offset = dissect_id_DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DL_CCTRCH_INFORMATIONDELETELIST_RL_RECONFRQSTTDD:		/*	349 */
		offset = dissect_id_DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DL_CCTRCH_INFORMATIONMODIFYITEM_RL_RECONFRQSTTDD:		/*	350 */
		offset = dissect_id_DL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DL_CCTRCH_INFORMATIONMODIFYLIST_RL_RECONFPREPTDD:		/*	351 */
		offset = dissect_id_DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DL_CCTRCH_INFORMATIONMODIFYLIST_RL_RECONFRQSTTDD:		/*	352 */
		offset = dissect_id_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DL_DPCH_INFORMATIONADDLISTIE_RL_RECONFPREPTDD:			/*	353 */
		offset = dissect_id_DL_DPCH_InformationAddListIE_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DL_DPCH_INFORMATIONMODIFY_ADDLISTIE_RL_RECONFPREPTDD:	/*	355 */
		offset = dissect_id_DL_DPCH_InformationModify_AddListIE_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DL_DPCH_INFORMATIONMODIFY_DELETELISTIE_RL_RECONFPREPTDD:/*	356 */
		offset = dissect_id_DL_DPCH_InformationModify_DeleteListIE_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DL_DPCH_INFORMATIONMODIFY_MODIFYLISTIE_RL_RECONFPREPTDD:/*	357 */
		offset = dissect_id_DL_DPCH_InformationModify_ModifyListIE_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DL_TPC_PATTERN01COUNT: 									/*	358 */
		offset = dissect_id_DL_TPC_Pattern01Count(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DPC_MODE:												/*	450 */
		offset = dissect_id_DPC_Mode(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DPCHCONSTANT:											/*	359 */
		offset = dissect_id_DPCHConstant(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UNUSED_PROTOCOLIE_ID_94:								/*	94 */
	case NBAP_IE_ID_UNUSED_PROTOCOLIE_ID_110:								/*	110 */
	case NBAP_IE_ID_UNUSED_PROTOCOLIE_ID_111:								/*	111 */
		break;
	case NBAP_IE_ID_FACH_PARAMETERSLIST_CTCH_SETUPRSP:						/*	362 */
		offset = dissect_id_FACH_ParametersList_CTCH_SetupRsp(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_LIMITED_POWER_INCREASE_INFORMATION_CELL_SETUPRQSTFDD:	/*	369 */
		offset = dissect_id_Limited_power_increase_information_Cell_SetupRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PCH_PARAMETERS_CTCH_SETUPRSP:							/*	374 */
		offset = dissect_id_PCH_Parameters_CTCH_SetupRsp(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PCH_PARAMETERSITEM_CTCH_RECONFRQSTFDD:					/*	375 */
		offset = dissect_id_PCH_ParametersItem_CTCH_ReconfRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_NOT_USED_376:											/*	376 */
		break;
	case NBAP_IE_ID_PICH_PARAMETERSITEM_CTCH_RECONFRQSTFDD:					/*	380 */
		offset = dissect_id_PICH_ParametersItem_CTCH_ReconfRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PRACHCONSTANT:											/*	381 */
		offset = dissect_id_PRACHConstant(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PRACH_PARAMETERSLISTIE_CTCH_RECONFRQSTFDD:				/*	383 */
		offset = dissect_id_PRACH_ParametersListIE_CTCH_ReconfRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PUSCHCONSTANT:											/*	384 */
		offset = dissect_id_PUSCHConstant(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RACH_PARAMETERS_CTCH_SETUPRSP:							/*	385 */
		offset = dissect_id_RACH_Parameters_CTCH_SetupRsp(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UNUSED_PROTOCOLIE_ID_443:								/*	443 */
		break;
	case NBAP_IE_ID_SYNCHRONISATION_CONFIGURATION_CELL_RECONFRQST:			/*	393 */
		offset = dissect_id_Synchronisation_Configuration_Cell_ReconfRqst(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_SYNCHRONISATION_CONFIGURATION_CELL_SETUPRQST:			/*	394 */
		offset = dissect_id_Synchronisation_Configuration_Cell_SetupRqst(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_TRANSMISSION_GAP_PATTERN_SEQUENCE_INFORMATION:			/*	395 */
		offset = dissect_id_Transmission_Gap_Pattern_Sequence_Information(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UL_CCTRCH_INFORMATIONADDLIST_RL_RECONFPREPTDD:			/*	396 */
		offset = dissect_id_UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UL_CCTRCH_INFORMATIONDELETEITEM_RL_RECONFRQSTTDD:		/*	397 */
		offset = dissect_id_UL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UL_CCTRCH_INFORMATIONDELETELIST_RL_RECONFPREPTDD:		/*	398 */
		offset = dissect_id_UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UL_CCTRCH_INFORMATIONDELETELIST_RL_RECONFRQSTTDD:		/*	399 */
		offset = dissect_id_UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UL_CCTRCH_INFORMATIONMODIFYITEM_RL_RECONFRQSTTDD:		/*	400 */
		offset = dissect_id_UL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UL_CCTRCH_INFORMATIONMODIFYLIST_RL_RECONFPREPTDD:		/*	401 */
		offset = dissect_id_UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UL_CCTRCH_INFORMATIONMODIFYLIST_RL_RECONFRQSTTDD:		/*	402 */
		offset = dissect_id_UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UL_DPCH_INFORMATIONADDLISTIE_RL_RECONFPREPTDD:			/*	403 */
		offset = dissect_id_UL_DPCH_InformationAddListIE_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UL_DPCH_INFORMATIONMODIFY_ADDLISTIE_RL_RECONFPREPTDD:	/*	405 */
		offset = dissect_id_UL_DPCH_InformationModify_AddListIE_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UL_DPCH_INFORMATIONMODIFY_DELETELISTIE_RL_RECONFPREPTDD:/*	406 */
		offset = dissect_id_UL_DPCH_InformationModify_DeleteListIE_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UL_DPCH_INFORMATIONMODIFY_MODIFYLISTIE_RL_RECONFPREPTDD:/*	407 */
		offset = dissect_id_UL_DPCH_InformationModify_ModifyListIE_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UNSUCCESSFUL_PDSCHSETITEM_PSCH_RECONFFAILURETDD:		/*	408 */
		offset = dissect_id_Unsuccessful_PDSCHSetItem_PSCH_ReconfFailureTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UNSUCCESSFUL_PUSCHSETITEM_PSCH_RECONFFAILURETDD:		/*	409 */
		offset = dissect_id_Unsuccessful_PUSCHSetItem_PSCH_ReconfFailureTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_COMMUNICATIONCONTEXTINFOITEM_RESET:						/*	412 */
		offset = dissect_id_CommunicationContextInfoItem_Reset(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_COMMUNICATIONCONTROLPORTINFOITEM_RESET:					/*	414 */
		offset = dissect_id_CommunicationControlPortInfoItem_Reset(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RESETINDICATOR:											/*	416 */
		offset = dissect_id_ResetIndicator(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UNUSED_PROTOCOLIE_ID_417:								/*	417 */
	case NBAP_IE_ID_UNUSED_PROTOCOLIE_ID_418:								/*	418 */
	case NBAP_IE_ID_UNUSED_PROTOCOLIE_ID_419:								/*	419 */
	case NBAP_IE_ID_UNUSED_PROTOCOLIE_ID_142:								/*	142 */
		break;
	case NBAP_IE_ID_TIMINGADVANCEAPPLIED:									/*	287 */
		offset = dissect_id_TimingAdvanceApplied(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CFNREPORTINGINDICATOR:									/*	6	*/
		offset = dissect_id_CFNReportingIndicator(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_SFNREPORTINGINDICATOR:									/*	11	*/
		offset = dissect_id_SFNReportingIndicator(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_INNERLOOPDLPCSTATUS:									/*	12 */
		offset = dissect_id_InnerLoopDLPCStatus(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_TIMESLOTISCPINFO:										/*	283 */
		offset = dissect_id_TimeslotISCPInfo(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PICH_PARAMETERSITEM_CTCH_SETUPRQSTTDD:					/*	167 */
		offset = dissect_id_PICH_ParametersItem_CTCH_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PRACH_PARAMETERSITEM_CTCH_SETUPRQSTTDD:					/*	20 */
		offset = dissect_id_PRACH_ParametersItem_CTCH_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CCTRCH_INFORMATIONITEM_RL_FAILUREIND:					/*	46 */
		offset = dissect_id_CCTrCH_InformationItem_RL_FailureInd(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CCTRCH_INFORMATIONITEM_RL_RESTOREIND:					/*	47 */
		offset = dissect_id_CCTrCH_InformationItem_RL_RestoreInd(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CAUSELEVEL_SYNCADJUSTMNTFAILURETDD:						/*	420 */
		offset = dissect_id_CauseLevel_SyncAdjustmntFailureTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CELLADJUSTMENTINFO_SYNCADJUSTMNTRQSTTDD:				/*	421 */
		offset = dissect_id_CellAdjustmentInfo_SyncAdjustmntRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CELLADJUSTMENTINFOITEM_SYNCADJUSTMENTRQSTTDD:			/*	494 */
		offset = dissect_id_CellAdjustmentInfoItem_SyncAdjustmentRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CELLSYNCBURSTINFOLIST_CELLSYNCRECONFRQSTTDD:			/*	482 */
		offset = dissect_id_CellSyncBurstInfoList_CellSyncReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CELLSYNCBURSTTRANSINIT_CELLSYNCINITIATIONRQSTTDD:		/*	422 */
		offset = dissect_id_CellSyncBurstTransInit_CellSyncInitiationRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CELLSYNCBURSTMEASUREINIT_CELLSYNCINITIATIONRQSTTDD:		/*	423 */
		offset = dissect_id_CellSyncBurstMeasureInit_CellSyncInitiationRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CELLSYNCBURSTTRANSRECONFIGURATION_CELLSYNCRECONFRQSTTDD:/*	424 */
		offset = dissect_id_CellSyncBurstMeasReconfiguration_CellSyncReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CELLSYNCBURSTMEASRECONFIGURATION_CELLSYNCRECONFRQSTTDD:	/*	425 */
		offset = dissect_id_CellSyncBurstMeasInfoList_CellSyncReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CELLSYNCBURSTTRANSINFOLIST_CELLSYNCRECONFRQSTTDD:		/*	426 */
		offset = dissect_id_CellSyncBurstTransReconfInfo_CellSyncReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CELLSYNCBURSTMEASINFOLIST_CELLSYNCRECONFRQSTTDD:		/*	427 */
		offset = dissect_id_CellSyncInfo_CellSyncReprtTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CELLSYNCBURSTTRANSRECONFINFO_CELLSYNCRECONFRQSTTDD:		/*	428 */
		offset = dissect_id_CellSyncBurstTransReconfInfo_CellSyncReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CELLSYNCINFO_CELLSYNCREPRTTDD:							/*	429 */
		offset = dissect_id_CellSyncInfo_CellSyncReprtTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CSBTRANSMISSIONID:										/*	430 */
		offset = dissect_id_CSBTransmissionID(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CSBMEASUREMENTID:										/*	431 */
		offset = dissect_id_CSBMeasurementID(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_INTSTDPHCELLSYNCINFOITEM_CELLSYNCREPRTTDD:				/*	432 */
		break;
	case NBAP_IE_ID_NCYCLESPERSFNPERIOD: 									/*	433 */
		offset = dissect_id_NCyclesPerSFNperiod(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_NREPETITIONSPERCYCLEPERIOD:								/*	434 */
		offset = dissect_id_NRepetitionsPerCyclePeriod(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_SYNCFRAMENUMBER:										/*	437 */
		offset = dissect_id_SyncFrameNumber(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_SYNCHRONISATIONREPORTTYPE:								/*	438 */
		offset = dissect_id_SynchronisationReportType(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_SYNCHRONISATIONREPORTCHARACTERISTICS:					/*	439 */
		offset = dissect_id_SynchronisationReportCharacteristics(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UNSUCCESSFUL_CELL_INFORMATIONRESPITEM_SYNCADJUSTMNTFAILURETDD:/*	440 */
		offset = dissect_id_Unsuccessful_cell_InformationRespItem_SyncAdjustmntFailureTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_LATEENTRANCECELLSYNCINFOITEM_CELLSYNCREPRTTDD:			/*	119 */
		break;
	case NBAP_IE_ID_REFERENCECLOCKAVAILABILITY:								/*	435 */
		offset = dissect_id_ReferenceClockAvailability(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_REFERENCESFNOFFSET:										/*	436 */
		offset = dissect_id_ReferenceSFNoffset(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_INFORMATIONEXCHANGEID:									/*	444 */
		offset = dissect_id_InformationExchangeID(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_INFORMATIONEXCHANGEOBJECTTYPE_INFEX_RQST:				/*	445 */
		offset = dissect_id_InformationExchangeObjectType_InfEx_Rqst(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_INFORMATIONTYPE:										/*	446 */
		offset = dissect_id_InformationType(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_INFORMATIONREPORTCHARACTERISTICS:						/*	447 */
		offset = dissect_id_InformationReportCharacteristics(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_INFORMATIONEXCHANGEOBJECTTYPE_INFEX_RSP:				/*	448 */
		offset = dissect_id_InformationExchangeObjectType_InfEx_Rsp(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_INFORMATIONEXCHANGEOBJECTTYPE_INFEX_RPRT:				/*	449 */
		offset = dissect_id_InformationExchangeObjectType_InfEx_Rprt(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_IPDLPARAMETER_INFORMATION_CELL_RECONFRQSTFDD:			/*	451 */
		offset = dissect_id_IPDLParameter_Information_Cell_ReconfRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_IPDLPARAMETER_INFORMATION_CELL_SETUPRQSTFDD:			/*	452 */
		offset = dissect_id_IPDLParameter_Information_Cell_SetupRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_IPDLPARAMETER_INFORMATION_CELL_RECONFRQSTTDD:			/*	453 */
		offset = dissect_id_IPDLParameter_Information_Cell_ReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_IPDLPARAMETER_INFORMATION_CELL_SETUPRQSTTDD:			/*	454 */
		offset = dissect_id_IPDLParameter_Information_Cell_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DL_DPCH_LCR_INFORMATION_RL_SETUPRQSTTDD:				/*	74 */
		offset = dissect_id_DL_DPCH_LCR_Information_RL_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;

	case NBAP_IE_ID_DWPCH_LCR_INFORMATION:									/*	78 */
		offset = dissect_id_DwPCH_LCR_Information(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DWPCH_LCR_INFORMATIONLIST_AUDITRSP:                   	/*	90 */
		offset = dissect_id_DwPCH_LCR_InformationList_AuditRsp(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DWPCH_LCR_INFORMATION_CELL_SETUPRQSTTDD:                /*	97 */
		offset = dissect_id_DwPCH_LCR_Information_Cell_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DWPCH_LCR_INFORMATION_CELL_RECONFRQSTTDD:				/*	99 */
		offset = dissect_id_DwPCH_LCR_Information_Cell_ReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DWPCH_LCR_INFORMATION_RESOURCESTATUSIND:				/*	101 */
		offset = dissect_id_DwPCH_LCR_Information_ResourceStatusInd(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_MAXFACH_POWER_LCR_CTCH_SETUPRQSTTDD:                   	/*	154 */
		offset = dissect_id_maxFACH_Power_LCR_CTCH_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_MAXFACH_POWER_LCR_CTCH_RECONFRQSTTDD:                   /*	174 */
		offset = dissect_id_maxFACH_Power_LCR_CTCH_ReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_FPACH_LCR_INFORMATION:									/*	290 */
		offset = dissect_id_FPACH_LCR_Information(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_FPACH_LCR_INFORMATION_AUDITRSP:                   		/*	292 */
		offset = dissect_id_FPACH_LCR_Information_AuditRsp(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_FPACH_LCR_INFORMATIONLIST_AUDITRSP:                   	/*	22 */
		offset = dissect_id_FPACH_LCR_InformationList_AuditRsp(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_FPACH_LCR_INFORMATIONLIST_RESOURCESTATUSIND:            /* 	311 */
		offset = dissect_id_FPACH_LCR_InformationList_ResourceStatusInd(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_FPACH_LCR_PARAMETERS_CTCH_SETUPRQSTTDD:                 /*	312 */
		offset = dissect_id_FPACH_LCR_Parameters_CTCH_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_FPACH_LCR_PARAMETERS_CTCH_RECONFRQSTTDD:                /*	314 */
		offset = dissect_id_FPACH_LCR_Parameters_CTCH_ReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PCCPCH_LCR_INFORMATION_CELL_SETUPRQSTTDD:               /*	456 */
		offset = dissect_id_PCCPCH_LCR_Information_Cell_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PCH_POWER_LCR_CTCH_SETUPRQSTTDD:						/*	457 */
		offset = dissect_id_PCH_Power_LCR_CTCH_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PCH_POWER_LCR_CTCH_RECONFRQSTTDD:						/*	458 */
		offset = dissect_id_PCH_Power_LCR_CTCH_ReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PICH_LCR_PARAMETERS_CTCH_SETUPRQSTTDD:                  /*	459 */
		offset = dissect_id_PICH_LCR_Parameters_CTCH_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PRACH_LCR_PARAMETERSLIST_CTCH_SETUPRQSTTDD:             /*	461 */
		offset = dissect_id_PRACH_LCR_ParametersList_CTCH_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RL_INFORMATIONRESPONSE_LCR_RL_SETUPRSPTDD:	            /* 	463 */
		offset = dissect_id_RL_InformationResponse_LCR_RL_SetupRspTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_SECONDARY_CCPCH_LCR_PARAMETERLIST_CTCH_SETUPRQSTTDD:    /*  465 */
		offset = dissect_id_Secondary_CCPCH_LCR_parameterList_CTCH_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_TIMESLOT:												/*	495 */
		offset = dissect_id_TimeSlot(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_TIMESLOTCONFIGURATIONLIST_LCR_CELL_RECONFRQSTTDD:       /*  466 */
		offset = dissect_id_TimeSlotConfigurationList_LCR_Cell_ReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_TIMESLOTCONFIGURATIONLIST_LCR_CELL_SETUPRQSTTDD:        /*  467 */
		offset = dissect_id_TimeSlotConfigurationList_LCR_Cell_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_TIMESLOTISCP_LCR_INFOLIST_RL_SETUPRQSTTDD:              /*	468 */
		offset = dissect_id_TimeslotISCP_LCR_InfoList_RL_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_TIMESLOTLCR_CM_RQST:                   					/*	469 */
		offset = dissect_id_TimeSlotLCR_CM_Rqst(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UL_DPCH_LCR_INFORMATION_RL_SETUPRQSTTDD:                /*	470 */
		offset = dissect_id_UL_DPCH_LCR_Information_RL_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DL_DPCH_INFORMATIONITEM_LCR_RL_ADDITIONRQSTTDD:			/*	472 */
		offset = dissect_id_DL_DPCH_InformationItem_LCR_RL_AdditionRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UL_DPCH_INFORMATIONITEM_LCR_RL_ADDITIONRQSTTDD:			/*	473 */
		offset = dissect_id_UL_DPCH_InformationItem_LCR_RL_AdditionRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_TIMESLOTISCP_INFORMATIONLIST_LCR_RL_ADDITIONRQSTTDD:	/*	474 */
		offset = dissect_id_TimeslotISCP_InformationList_LCR_RL_AdditionRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DL_DPCH_LCR_INFORMATIONADDLIST_RL_RECONFPREPTDD:		/*	475 */
		offset = dissect_id_DL_DPCH_LCR_InformationAddList_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DL_DPCH_LCR_INFORMATIONMODIFY_ADDLIST_RL_RECONFPREPTDD:			/*	477 */
		offset = dissect_id_DL_DPCH_LCR_InformationModify_AddList_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DL_TIMESLOT_LCR_INFORMATIONMODIFY_MODIFYLIST_RL_RECONFPREPTDD:	/*	479 */
		offset = dissect_id_DL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_TIMESLOTISCPINFOLIST_LCR_DL_PC_RQSTTDD:							/*	480 */
		offset = dissect_id_TimeslotISCPInfoList_LCR_DL_PC_RqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UL_DPCH_LCR_INFORMATIONADDLISTIE_RL_RECONFPREPTDD:				/*	481 */
		offset = dissect_id_UL_DPCH_LCR_InformationAddListIE_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UL_DPCH_LCR_INFORMATIONMODIFY_ADDLIST:							/*	483 */
		offset = dissect_id_UL_DPCH_LCR_InformationModify_AddList(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UL_TIMESLOTLCR_INFORMATION_RL_RECONFPREPTDD:					/*	485 */
		offset = dissect_id_UL_TimeslotLCR_Information_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UL_SIRTARGET:													/*	510 */
		offset = dissect_id_UL_SIRTarget(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PDSCH_ADDINFORMATION_LCR_PSCH_RECONFRQST:						/*	486 */
		offset = dissect_id_PDSCH_AddInformation_LCR_PSCH_ReconfRqst(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PDSCH_ADDINFORMATION_LCR_ADDLISTIE_PSCH_RECONFRQST:				/*	487 */
		offset = dissect_id_PDSCH_ModifyInformation_LCR_PSCH_ReconfRqst(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UNUSED_PROTOCOLIE_ID_26:										/*	 26 */
	case NBAP_IE_ID_UNUSED_PROTOCOLIE_ID_27:										/*	 27 */
		break;
	case NBAP_IE_ID_PDSCH_MODIFYINFORMATION_LCR_PSCH_RECONFRQST:					/*	488 */
		offset = dissect_id_PUSCH_AddInformation_LCR_PSCH_ReconfRqst(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PDSCH_MODIFYINFORMATION_LCR_MODIFYLISTIE_PSCH_RECONFRQST:		/*	489 */
		offset = dissect_id_PUSCH_ModifyInformation_LCR_PSCH_ReconfRqst(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PUSCH_ADDINFORMATION_LCR_PSCH_RECONFRQST:						/*	490 */
		break;
	case NBAP_IE_ID_PUSCH_ADDINFORMATION_LCR_ADDLISTIE_PSCH_RECONFRQST:				/*	491 */
		break;
	case NBAP_IE_ID_PUSCH_MODIFYINFORMATION_LCR_PSCH_RECONFRQST:					/*	492 */
		break;
	case NBAP_IE_ID_PUSCH_MODIFYINFORMATION_LCR_MODIFYLISTIE_PSCH_RECONFRQST:		/*	493 */
		break;
	case NBAP_IE_ID_TIMESLOTINFO_CELLSYNCINITIATIONRQSTTDD:							/*	496 */
		offset = dissect_id_timeslotInfo_CellSyncInitiationRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_SYNCREPORTTYPE_CELLSYNCREPRTTDD:								/*	497 */
		offset = dissect_id_SyncReportType_CellSyncReprtTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_POWER_LOCAL_CELL_GROUP_INFORMATIONITEM_AUDITRSP:				/*	498 */
		offset = dissect_id_Power_Local_Cell_Group_InformationItem_AuditRsp(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_POWER_LOCAL_CELL_GROUP_INFORMATIONITEM_RESOURCESTATUSIND:		/*	499 */
		offset = dissect_id_Power_Local_Cell_Group_InformationItem_ResourceStatusInd(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_POWER_LOCAL_CELL_GROUP_INFORMATIONITEM2_RESOURCESTATUSIND:		/*	500 */
		offset = dissect_id_Power_Local_Cell_Group_InformationItem2_ResourceStatusInd(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_POWER_LOCAL_CELL_GROUP_INFORMATIONLIST_AUDITRSP:				/*	501 */
		offset = dissect_id_Power_Local_Cell_Group_InformationList_AuditRsp(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_POWER_LOCAL_CELL_GROUP_INFORMATIONLIST_RESOURCESTATUSIND:		/*	502 */
		offset = dissect_id_Power_Local_Cell_Group_InformationList_ResourceStatusInd(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_POWER_LOCAL_CELL_GROUP_INFORMATIONLIST2_RESOURCESTATUSIND:		/*	503 */
		offset = dissect_id_Power_Local_Cell_Group_InformationList2_ResourceStatusInd(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_POWER_LOCAL_CELL_GROUP_ID:										/*	504 */
		offset = dissect_id_Power_Local_Cell_Group_ID(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PUSCH_INFO_DM_RQST:												/*	505 */
		offset = dissect_id_PUSCH_Info_DM_Rqst(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PUSCH_INFO_DM_RSP:												/*	506 */
		offset = dissect_id_PUSCH_Info_DM_Rsp(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PUSCH_INFO_DM_RPRT:												/*	507 */
		offset = dissect_id_PUSCH_Info_DM_Rprt(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_INITDL_POWER:													/*	509 */
		offset = dissect_id_InitDL_Power(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CELLSYNCBURSTREPETITIONPERIOD:									/*	511 */
		offset = dissect_id_cellSyncBurstRepetitionPeriod(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_REPORTCHARACTERISTICSTYPE_ONMODIFICATION:						/*	512 */
		offset = dissect_id_ReportCharacteristicsType_OnModification(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_SFNSFNMEASUREMENTVALUEINFORMATION:								/*	513 */
		offset = dissect_id_SFNSFNMeasurementValueInformation(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_SFNSFNMEASUREMENTTHRESHOLDINFORMATION:							/*	514 */
		offset = dissect_id_SFNSFNMeasurementThresholdInformation(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_TUTRANGPSMEASUREMENTVALUEINFORMATION:							/*	515 */
		offset = dissect_id_TUTRANGPSMeasurementValueInformation(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_TUTRANGPSMEASUREMENTTHRESHOLDINFORMATION:						/*	516 */
		offset = dissect_id_TUTRANGPSMeasurementThresholdInformation(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RX_TIMING_DEVIATION_VALUE_LCR:									/*	520 */
		offset = dissect_id_Rx_Timing_Deviation_Value_LCR(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RL_INFORMATIONRESPONSE_LCR_RL_ADDITIONRSPTDD:					/*	51	*/
		offset = dissect_id_RL_InformationResponse_LCR_RL_AdditionRspTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DL_POWERBALANCING_INFORMATION:									/*  28	*/
		offset = dissect_id_DL_PowerBalancing_Information(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DL_POWERBALANCING_ACTIVATIONINDICATOR:							/*	29	*/
		offset = dissect_id_DL_PowerBalancing_ActivationIndicator(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DL_POWERBALANCING_UPDATEDINDICATOR:								/*	30	*/
		offset = dissect_id_DL_PowerBalancing_UpdatedIndicator(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CCTRCH_INITIAL_DL_POWER_RL_SETUPRQSTTDD:						/*	517	*/
		offset = dissect_id_CCTrCH_Initial_DL_Power_RL_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CCTRCH_INITIAL_DL_POWER_RL_ADDITIONRQSTTDD:						/*	518	*/
		offset = dissect_id_CCTrCH_Initial_DL_Power_RL_AdditionRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CCTRCH_INITIAL_DL_POWER_RL_RECONFPREPTDD:						/*	519	*/
		offset = dissect_id_CCTrCH_Initial_DL_Power_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_IPDLPARAMETER_INFORMATION_LCR_CELL_SETUPRQSTTDD:				/*	41	*/
		offset = dissect_id_IPDLParameter_Information_LCR_Cell_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_IPDLPARAMETER_INFORMATION_LCR_CELL_RECONFRQSTTDD: 				/*	42	*/
		offset = dissect_id_IPDLParameter_Information_LCR_Cell_ReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_HS_PDSCH_HS_SCCH_E_AGCH_E_RGCH_E_HICH_MAXPOWER_PSCH_RECONFRQST:	/*	522	*/
		offset = dissect_id_HS_PDSCH_HS_SCCH_E_AGCH_E_RGCH_E_HICH_MaxPower_PSCH_ReconfRqst(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_HS_PDSCH_HS_SCCH_SCRAMBLINGCODE_PSCH_RECONFRQST:				/*	523	*/
		offset = dissect_id_HS_PDSCH_HS_SCCH_ScramblingCode_PSCH_ReconfRqst(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_HS_PDSCH_FDD_CODE_INFORMATION_PSCH_RECONFRQST:					/*	524	*/
		offset = dissect_id_HS_PDSCH_FDD_Code_Information_PSCH_ReconfRqst(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_HS_SCCH_FDD_CODE_INFORMATION_PSCH_RECONFRQST:					/*	525	*/
		offset = dissect_id_HS_SCCH_FDD_Code_Information_PSCH_ReconfRqst(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_HS_PDSCH_TDD_INFORMATION_PSCH_RECONFRQST:						/*	526	*/
		offset = dissect_id_HS_PDSCH_TDD_Information_PSCH_ReconfRqst(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_ADD_TO_HS_SCCH_RESOURCE_POOL_PSCH_RECONFRQST:					/*	527	*/
		offset = dissect_id_Add_To_HS_SCCH_Resource_Pool_PSCH_ReconfRqst(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_MODIFY_HS_SCCH_RESOURCE_POOL_PSCH_RECONFRQST:					/*	528	*/
		offset = dissect_id_Modify_HS_SCCH_Resource_Pool_PSCH_ReconfRqst(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DELETE_FROM_HS_SCCH_RESOURCE_POOL_PSCH_RECONFRQST:				/*	529	*/
		offset = dissect_id_Delete_From_HS_SCCH_Resource_Pool_PSCH_ReconfRqst(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_BINDINGID:														/*	102	*/
		offset = dissect_id_bindingID(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RL_SPECIFIC_DCH_INFO:											/*	103	*/
		offset = dissect_id_RL_Specific_DCH_Info(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_TRANSPORTLAYERADDRESS: 											/*	104	*/
		offset = dissect_id_transportlayeraddress(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DELAYEDACTIVATION:												/*	231	*/
		offset = dissect_id_DelayedActivation(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DELAYEDACTIVATIONLIST_RL_ACTIVATIONCMDFDD:						/*	232	*/
		offset = dissect_id_DelayedActivationList_RL_ActivationCmdFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DELAYEDACTIVATIONINFORMATION_RL_ACTIVATIONCMDFDD:				/*	233	*/
		offset = dissect_id_DelayedActivationInformation_RL_ActivationCmdFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DELAYEDACTIVATIONLIST_RL_ACTIVATIONCMDTDD:						/*	234	*/
		offset = dissect_id_DelayedActivationList_RL_ActivationCmdTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DELAYEDACTIVATIONINFORMATION_RL_ACTIVATIONCMDTDD:				/*	235	*/
		offset = dissect_id_DelayedActivationInformation_RL_ActivationCmdTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_NEIGHBOURINGTDDCELLMEASUREMENTINFORMATIONLCR:					/*	58	*/
		offset = dissect_id_neighbouringTDDCellMeasurementInformationLCR(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_SYNCDLCODEID_TRANSINITLCR_CELLSYNCINITIATIONRQSTTDD:			/*	543	*/
		offset = dissect_id_SYNCDlCodeId_TransInitLCR_CellSyncInitiationRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_SYNCDLCODEID_MEASUREINITLCR_CELLSYNCINITIATIONRQSTTDD:			/*	544	*/
		offset = dissect_id_SYNCDlCodeId_MeasureInitLCR_CellSyncInitiationRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_SYNCDLCODEIDTRANSRECONFINFOLCR_CELLSYNCRECONFRQSTTDD:			/*	545	*/
		offset = dissect_id_SYNCDlCodeIdTransReconfInfoLCR_CellSyncReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_SYNCDLCODEIDMEASRECONFIGURATIONLCR_CELLSYNCRECONFRQSTTDD:		/*	546	*/
		offset = dissect_id_SYNCDlCodeIdMeasReconfigurationLCR_CellSyncReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_SYNCDLCODEIDMEASINFOLIST_CELLSYNCRECONFRQSTTDD:					/*	547	*/
		offset = dissect_id_SYNCDlCodeIdMeasInfoList_CellSyncReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_SYNCDLCODEIDSMEASINFOLIST_CELLSYNCREPRTTDD:						/*	548	*/
		offset = dissect_id_SyncDLCodeIdsMeasInfoList_CellSyncReprtTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_SYNCDLCODEIDTHREINFOLCR:										/*	549	*/
		offset = dissect_id_SyncDLCodeIdThreInfoLCR(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_NSUBCYCLESPERCYCLEPERIOD_CELLSYNCRECONFRQSTTDD:					/*	550	*/
		offset = dissect_id_NSubCyclesPerCyclePeriod_CellSyncReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DWPCH_POWER:													/*	551	*/
		offset = dissect_id_DwPCH_Power(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_ACCUMULATEDCLOCKUPDATE_CELLSYNCREPRTTDD:						/*	552	*/
		offset = dissect_id_AccumulatedClockupdate_CellSyncReprtTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_ANGLE_OF_ARRIVAL_VALUE_LCR:										/*	521	*/
		offset = dissect_id_Angle_Of_Arrival_Value_LCR(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_HSDSCH_FDD_INFORMATION:											/*	530	*/
		offset = dissect_id_HSDSCH_FDD_Information(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_HSDSCH_FDD_INFORMATION_RESPONSE:								/*	531	*/
		offset = dissect_id_HSDSCH_FDD_Information_Response(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_HSDSCH_INFORMATION_TO_MODIFY:									/*	534	*/
		offset = dissect_id_HSDSCH_Information_to_Modify(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_HSDSCH_RNTI:													/*	535	*/
		offset = dissect_id_HSDSCH_RNTI(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_HSDSCH_TDD_INFORMATION:											/*	536	*/
		offset = dissect_id_HSDSCH_TDD_Information(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_HSDSCH_TDD_INFORMATION_RESPONSE:								/*	537	*/
		offset = dissect_id_HSDSCH_TDD_Information_Response(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_HSPDSCH_RL_ID:													/*	541	*/
		offset = dissect_id_HSPDSCH_RL_ID(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PRIMCCPCH_RSCP_DL_PC_RQSTTDD:									/*	542	*/
		offset = dissect_id_PrimCCPCH_RSCP_DL_PC_RqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UNUSED_PROTOCOLIE_ID_64:										/*	64	*/
		break;
	case NBAP_IE_ID_PDSCH_RL_ID:													/*	66	*/
		offset = dissect_id_PDSCH_RL_ID(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_HSDSCH_REARRANGELIST_BEARER_REARRANGEIND:						/*	553	*/
		offset = dissect_id_HSDSCH_RearrangeList_Bearer_RearrangeInd(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UL_SYNCHRONISATION_PARAMETERS_LCR:								/*	554	*/
		offset = dissect_id_UL_Synchronisation_Parameters_LCR(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_HSDSCH_FDD_UPDATE_INFORMATION:									/*	555	*/
		offset = dissect_id_HSDSCH_FDD_Update_Information(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_HSDSCH_TDD_UPDATE_INFORMATION:									/*	556	*/
		offset = dissect_id_HSDSCH_TDD_Update_Information(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DL_DPCH_TIMESLOTFORMAT_LCR_MODIFYITEM_RL_RECONFPREPTDD:			/*	558	*/
		offset = dissect_id_DL_DPCH_TimeSlotFormat_LCR_ModifyItem_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UL_DPCH_TIMESLOTFORMAT_LCR_MODIFYITEM_RL_RECONFPREPTDD:			/*	559	*/
		offset = dissect_id_UL_DPCH_TimeSlotFormat_LCR_ModifyItem_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_TDD_TPC_UPLINKSTEPSIZE_LCR_RL_SETUPRQSTTDD:						/*	560	*/
		offset = dissect_id_TDD_TPC_UplinkStepSize_LCR_RL_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_TDD_TPC_UPLINKSTEPSIZE_LCR_RL_ADDITIONRQSTTDD:					/*	561	*/
		offset = dissect_id_TDD_TPC_UplinkStepSize_LCR_RL_AdditionRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_TDD_TPC_DOWNLINKSTEPSIZE_RL_ADDITIONRQSTTDD:					/*	562	*/
		offset = dissect_id_TDD_TPC_DownlinkStepSize_RL_AdditionRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_TDD_TPC_UPLINKSTEPSIZE_INFORMATIONADD_LCR_RL_RECONFPREPTDD:		/*	563	*/
		offset = dissect_id_TDD_TPC_UplinkStepSize_InformationAdd_LCR_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_TDD_TPC_UPLINKSTEPSIZE_INFORMATIONMODIFY_LCR_RL_RECONFPREPTDD:	/*	564	*/
		offset = dissect_id_TDD_TPC_UplinkStepSize_InformationModify_LCR_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_TDD_TPC_DOWNLINKSTEPSIZE_INFORMATIONMODIFY_RL_RECONFPREPTDD:	/*	565	*/
		offset = dissect_id_TDD_TPC_DownlinkStepSize_InformationModify_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_TDD_TPC_DOWNLINKSTEPSIZE_INFORMATIONADD_RL_RECONFPREPTDD:		/*	566	*/
		offset = dissect_id_TDD_TPC_DownlinkStepSize_InformationAdd_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CCTRCH_MAXIMUM_DL_POWER_RL_SETUPRQSTTDD:						/*	567	*/
		offset = dissect_id_CCTrCH_Maximum_DL_Power_RL_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CCTRCH_MINIMUM_DL_POWER_RL_SETUPRQSTTDD:						/*	568	*/
		offset = dissect_id_CCTrCH_Minimum_DL_Power_RL_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CCTRCH_MAXIMUM_DL_POWER_RL_ADDITIONRQSTTDD:						/*	569	*/
		offset = dissect_id_CCTrCH_Maximum_DL_Power_RL_AdditionRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CCTRCH_MINIMUM_DL_POWER_RL_ADDITIONRQSTTDD:						/*	570	*/
		offset = dissect_id_CCTrCH_Minimum_DL_Power_RL_AdditionRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CCTRCH_MAXIMUM_DL_POWER_INFORMATIONADD_RL_RECONFPREPTDD:		/*	571	*/
		offset = dissect_id_CCTrCH_Maximum_DL_Power_InformationAdd_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CCTRCH_MINIMUM_DL_POWER_INFORMATIONADD_RL_RECONFPREPTDD:		/*	572	*/
		offset = dissect_id_CCTrCH_Minimum_DL_Power_InformationAdd_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CCTRCH_MAXIMUM_DL_POWER_INFORMATIONMODIFY_RL_RECONFPREPTDD:		/*	573	*/
		offset = dissect_id_CCTrCH_Maximum_DL_Power_InformationModify_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CCTRCH_MINIMUM_DL_POWER_INFORMATIONMODIFY_RL_RECONFPREPTDD:		/*	574	*/
		offset = dissect_id_CCTrCH_Minimum_DL_Power_InformationModify_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_MAXIMUM_DL_POWER_MODIFY_LCR_INFORMATIONMODIFY_RL_RECONFPREPTDD:	/*	575	*/
		offset = dissect_id_Maximum_DL_Power_Modify_LCR_InformationModify_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_MINIMUM_DL_POWER_MODIFY_LCR_INFORMATIONMODIFY_RL_RECONFPREPTDD:	/*	576	*/
		offset = dissect_id_Minimum_DL_Power_Modify_LCR_InformationModify_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DL_DPCH_LCR_INFORMATIONMODIFY_MODIFYLIST_RL_RECONFRQSTTDD:		/*	577	*/
		offset = dissect_id_DL_DPCH_LCR_InformationModify_ModifyList_RL_ReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CCTRCH_MAXIMUM_DL_POWER_INFORMATIONMODIFY_RL_RECONFRQSTTDD:		/*	578	*/
		offset = dissect_id_CCTrCH_Maximum_DL_Power_InformationModify_RL_ReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CCTRCH_MINIMUM_DL_POWER_INFORMATIONMODIFY_RL_RECONFRQSTTDD:		/*	579	*/
		offset = dissect_id_CCTrCH_Minimum_DL_Power_InformationModify_RL_ReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_INITIAL_DL_POWER_TIMESLOTLCR_INFORMATIONITEM:					/*	580	*/
		offset = dissect_id_Initial_DL_Power_TimeslotLCR_InformationItem(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_MAXIMUM_DL_POWER_TIMESLOTLCR_INFORMATIONITEM:					/*	581	*/
		offset = dissect_id_Maximum_DL_Power_TimeslotLCR_InformationItem(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_MINIMUM_DL_POWER_TIMESLOTLCR_INFORMATIONITEM:					/*	582	*/
		offset = dissect_id_Minimum_DL_Power_TimeslotLCR_InformationItem(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_HS_DSCHPROVIDEDBITRATEVALUEINFORMATION:							/*	583	*/
		offset = dissect_id_HS_DSCHProvidedBitRateValueInformation(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_HS_DSCHREQUIREDPOWERVALUEINFORMATION:							/*	585	*/
		offset = dissect_id_HS_DSCHRequiredPowerValueInformation(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_HS_DSCHREQUIREDPOWERVALUE:										/*	586	*/
		offset = dissect_id_HS_DSCHRequiredPowerValue(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_TRANSMITTEDCARRIERPOWEROFALLCODESNOTUSEDFORHSTRANSMISSION:		/*	587	*/
		offset = dissect_id_TransmittedCarrierPowerOfAllCodesNotUsedForHSTransmission(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_HS_SICH_RECEPTION_QUALITY: 										/*	588	*/
		offset = dissect_id_HS_SICH_Reception_Quality(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_HS_SICH_RECEPTION_QUALITY_MEASUREMENT_VALUE: 					/*	589	*/
		offset = dissect_id_HS_SICH_Reception_Quality_Measurement_Value(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_HSSICH_INFO_DM_RPRT: 											/*	590	*/
		offset = dissect_id_HSSICH_Info_DM_Rprt(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_HSSICH_INFO_DM_RQST:											/*	591	*/
		offset = dissect_id_HSSICH_Info_DM_Rqst(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_HSSICH_INFO_DM_RSP:												/*	592	*/
		offset = dissect_id_HSSICH_Info_DM_Rsp(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_BEST_CELL_PORTIONS_VALUE:										/*	593	*/
		offset = dissect_id_Best_Cell_Portions_Value(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PRIMARY_CPICH_USAGE_FOR_CHANNEL_ESTIMATION:						/*	594	*/
		offset = dissect_id_Primary_CPICH_Usage_for_Channel_Estimation(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_SECONDARY_CPICH_INFORMATION_CHANGE:								/*	595	*/
		offset = dissect_id_Secondary_CPICH_Information_Change(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_NUMBEROFREPORTEDCELLPORTIONS:									/*	596	*/
		offset = dissect_id_NumberOfReportedCellPortions(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CELLPORTION_INFORMATIONITEM_CELL_SETUPRQSTFDD:					/*	597	*/
		offset = dissect_id_CellPortion_InformationItem_Cell_SetupRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CELLPORTION_INFORMATIONLIST_CELL_SETUPRQSTFDD:					/*	598	*/
		offset = dissect_id_CellPortion_InformationList_Cell_SetupRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_TIMESLOTISCP_LCR_INFOLIST_RL_RECONFPREPTDD:						/*	599	*/
		offset = dissect_id_TimeslotISCP_LCR_InfoList_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_SECONDARY_CPICH_INFORMATION:									/*	600	*/
		offset = dissect_id_Secondary_CPICH_Information(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RECEIVED_TOTAL_WIDE_BAND_POWER_FOR_CELLPORTION:					/*	601	*/
		offset = dissect_id_Received_total_wide_band_power_For_CellPortion(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UNIDIRECTIONAL_DCH_INDICATOR:									/*	602	*/
		offset = dissect_id_Unidirectional_DCH_Indicator(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_TIMINGADJUSTMENTVALUELCR:										/*	603	*/
		offset = dissect_id_TimingAdjustmentValueLCR(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_MULTIPLERL_DL_DPCH_INFORMATIONLIST:								/*	604	*/
		break;
	case NBAP_IE_ID_MULTIPLERL_DL_DPCH_INFORMATIONMODIFYLIST:						/*	605	*/
		break;
	case NBAP_IE_ID_MULTIPLERL_UL_DPCH_INFORMATIONLIST:								/*	606	*/
		offset = dissect_id_multipleRL_ul_DPCH_InformationList(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_MULTIPLERL_UL_DPCH_INFORMATIONMODIFYLIST:						/*	607	*/
		offset = dissect_id_multipleRL_ul_DPCH_InformationModifyList(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RL_ID:															/*	608	*/
		offset = dissect_id_RL_ID(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_SAT_INFO_ALMANAC_EXTITEM:										/*	609	*/
		offset = dissect_id_SAT_Info_Almanac_ExtItem(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_HSDPA_CAPABILITY:												/*	610	*/
		offset = dissect_id_HSDPA_Capability(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_HSDSCH_RESOURCES_INFORMATION_AUDITRSP:							/*	611	*/
		offset = dissect_id_HSDSCH_Resources_Information_AuditRsp(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_HSDSCH_RESOURCES_INFORMATION_RESOURCESTATUSIND:					/*	612	*/
		offset = dissect_id_HSDSCH_Resources_Information_ResourceStatusInd(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_HSDSCH_MACDFLOWS_TO_ADD:										/*	613	*/
		offset = dissect_id_HSDSCH_MACdFlows_to_Add(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_HSDSCH_MACDFLOWS_TO_DELETE:										/*	614	*/
		offset = dissect_id_HSDSCH_MACdFlows_to_Delete(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_HSDSCH_INFORMATION_TO_MODIFY_UNSYNCHRONISED:					/*	615	*/
		offset = dissect_id_HSDSCH_Information_to_Modify_Unsynchronised(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_TNLQOS:															/*	616	*/
		offset = dissect_id_TnlQos(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RECEIVED_TOTAL_WIDE_BAND_POWER_FOR_CELLPORTION_VALUE:			/*	617	*/
		offset = dissect_id_Received_total_wide_band_power_For_CellPortion_Value(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_TRANSMITTED_CARRIER_POWER_FOR_CELLPORTION:						/*	618	*/
		offset = dissect_id_Transmitted_Carrier_Power_For_CellPortion(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_TRANSMITTED_CARRIER_POWER_FOR_CELLPORTION_VALUE :				/*	619	*/
		offset = dissect_id_Transmitted_Carrier_Power_For_CellPortion_Value(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_TRANSMITTEDCARRIERPOWEROFALLCODESNOTUSEDFORHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHORE_HICHTRANSMISSIONCELLPORTION:		/*620*/
		offset = dissect_id_TransmittedCarrierPowerOfAllCodesNotUsedForHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHOrE_HICHTransmissionCellPortion(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_TRANSMITTEDCARRIERPOWEROFALLCODESNOTUSEDFORHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHORE_HICHTRANSMISSIONCELLPORTIONVALUE:	/*621*/
		offset = dissect_id_TransmittedCarrierPowerOfAllCodesNotUsedForHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHOrE_HICHTransmissionCellPortionValue(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UPPTSINTERFERENCEVALUE:											/*	622	*/
		offset = dissect_id_UpPTSInterferenceValue(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_PRIMARYCCPCH_RSCP_DELTA:										/*	623	*/
		offset = dissect_id_PrimaryCCPCH_RSCP_Delta(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_MEASUREMENTRECOVERYBEHAVIOR:									/*	624	*/
		offset = dissect_id_MeasurementRecoveryBehavior(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_MEASUREMENTRECOVERYREPORTINGINDICATOR:							/*	625	*/
		offset = dissect_id_MeasurementRecoveryReportingIndicator(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_MEASUREMENTRECOVERYSUPPORTINDICATOR:							/*	626	*/
		offset = dissect_id_MeasurementRecoverySupportIndicator(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_TSTD_INDICATOR:													/*	627	*/
		offset = dissect_id_Tstd_indicator(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_MULTIPLE_RL_INFORMATION_RL_RECONFPREPTDD:						/*	628	*/
		offset = dissect_id_multiple_RL_Information_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_MULTIPLE_RL_INFORMATION_RL_RECONFRQSTTDD:						/*	629	*/
		offset = dissect_id_multiple_RL_Information_RL_ReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DL_DPCH_POWER_INFORMATION_RL_RECONFPREPFDD:						/*	630	*/
		offset = dissect_id_DL_DPCH_Power_Information_RL_ReconfPrepFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_F_DPCH_INFORMATION_RL_RECONFPREPFDD:							/*	631	*/
		offset = dissect_id_F_DPCH_Information_RL_ReconfPrepFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_F_DPCH_INFORMATION_RL_SETUPRQSTFDD:								/*	632	*/
		offset = dissect_id_F_DPCH_Information_RL_SetupRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_ADDITIONAL_S_CCPCH_PARAMETERS_CTCH_RECONFRQSTTDD:				/*	633	*/
		offset = dissect_id_Additional_S_CCPCH_Parameters_CTCH_ReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_ADDITIONAL_S_CCPCH_PARAMETERS_CTCH_SETUPRQSTTDD:				/*	634	*/
		offset = dissect_id_Additional_S_CCPCH_Parameters_CTCH_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_ADDITIONAL_S_CCPCH_LCR_PARAMETERS_CTCH_RECONFRQSTTDD:			/*	635	*/
		offset = dissect_id_Additional_S_CCPCH_LCR_Parameters_CTCH_ReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_ADDITIONAL_S_CCPCH_LCR_PARAMETERS_CTCH_SETUPRQSTTDD:			/*	636	*/
		offset = dissect_id_Additional_S_CCPCH_LCR_Parameters_CTCH_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_MICH_CFN:														/*	637	*/
		offset = dissect_id_MICH_CFN(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_MICH_INFORMATION_AUDITRSP:										/*	638	*/
		offset = dissect_id_MICH_Information_AuditRsp(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_MICH_INFORMATION_RESOURCESTATUSIND:								/*	639	*/
		offset = dissect_id_MICH_Information_ResourceStatusInd(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_MICH_PARAMETERS_CTCH_RECONFRQSTFDD:								/*	640	*/
		offset = dissect_id_MICH_Parameters_CTCH_ReconfRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_MICH_PARAMETERS_CTCH_RECONFRQSTTDD:								/*	641	*/
		offset = dissect_id_MICH_Parameters_CTCH_ReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_MICH_PARAMETERS_CTCH_SETUPRQSTFDD:								/*	642	*/
		offset = dissect_id_MICH_Parameters_CTCH_SetupRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_MICH_PARAMETERS_CTCH_SETUPRQSTTDD:								/*	643	*/
		offset = dissect_id_MICH_Parameters_CTCH_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_MODIFICATION_PERIOD:											/*	644	*/
		offset = dissect_id_Modification_Period(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_NI_INFORMATION_NOTIFUPDATECMD:									/*	645	*/
		offset = dissect_id_NI_Information_NotifUpdateCmd(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_S_CCPCH_INFORMATIONLISTEXT_AUDITRSP:							/*	646	*/
		offset = dissect_id_S_CCPCH_InformationListExt_AuditRsp(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_S_CCPCH_INFORMATIONLISTEXT_RESOURCESTATUSIND:					/*	647	*/
		offset = dissect_id_S_CCPCH_InformationListExt_ResourceStatusInd(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_S_CCPCH_LCR_INFORMATIONLISTEXT_AUDITRSP:						/*	648	*/
		offset = dissect_id_S_CCPCH_LCR_InformationListExt_AuditRsp(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_S_CCPCH_LCR_INFORMATIONLISTEXT_RESOURCESTATUSIND:				/*	649	*/
		offset = dissect_id_S_CCPCH_LCR_InformationListExt_ResourceStatusInd(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_HARQ_PREAMBLE_MODE:												/*	650	*/
		offset = dissect_id_HARQ_Preamble_Mode(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_INITIAL_DL_DPCH_TIMINGADJUSTMENT:								/*	651	*/
		offset = dissect_id_Initial_DL_DPCH_TimingAdjustment(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_INITIAL_DL_DPCH_TIMINGADJUSTMENT_ALLOWED:						/*	652	*/
		offset = dissect_id_Initial_DL_DPCH_TimingAdjustment_Allowed(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_DLTRANSMISSIONBRANCHLOADVALUE:									/*	653	*/
		offset = dissect_id_DLTransmissionBranchLoadValue(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_POWER_LOCAL_CELL_GROUP_CHOICE_CM_RQST:							/*	654	*/
		offset = dissect_id_Power_Local_Cell_Group_choice_CM_Rqst(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_POWER_LOCAL_CELL_GROUP_CHOICE_CM_RSP:							/*	655	*/
		offset = dissect_id_Power_Local_Cell_Group_choice_CM_Rsp(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_POWER_LOCAL_CELL_GROUP_CHOICE_CM_RPRT:							/*	656	*/
		offset = dissect_id_Power_Local_Cell_Group_choice_CM_Rprt(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_SYNCHRONISATIONINDICATOR:										/*	657	*/
		offset = dissect_id_SynchronisationIndicator(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_HSDPA_AND_EDCH_CELLPORTION_INFORMATION_PSCH_RECONFRQST:			/*	658	*/
		offset = dissect_id_HSDPA_And_EDCH_CellPortion_Information_PSCH_ReconfRqst(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_HSDPA_AND_EDCH_CELLPORTION_INFORMATIONLIST_PSCH_RECONFRQST:		/*	659	*/
		offset = dissect_id_HSDPA_And_EDCH_CellPortion_InformationList_PSCH_ReconfRqst(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_HS_DSCHREQUIREDPOWERVALUE_FOR_CELL_PORTION:						/*	660	*/
		offset = dissect_id_HS_DSCHRequiredPowerValue_For_Cell_Portion(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_HS_DSCHREQUIREDPOWERVALUEINFORMATION_FOR_CELLPORTION:			/*	661	*/
		offset = dissect_id_HS_DSCHRequiredPowerValueInformation_For_CellPortion(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_HS_DSCHPROVIDEDBITRATEVALUEINFORMATION_FOR_CELLPORTION:			/*	662	*/
		offset = dissect_id_HS_DSCHProvidedBitRateValueInformation_For_CellPortion(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_E_AGCH_AND_E_RGCH_E_HICH_FDD_SCRAMBLING_CODE:					/*	663	*/
		offset = dissect_id_E_AGCH_And_E_RGCH_E_HICH_FDD_Scrambling_Code(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_E_AGCH_FDD_CODE_INFORMATION:									/*	664	*/
		offset = dissect_id_E_AGCH_FDD_Code_Information(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_E_DCH_CAPABILITY:												/*	665	*/
		offset = dissect_id_E_DCH_Capability(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_E_DCH_FDD_DL_CONTROL_CHANNEL_INFORMATION:						/*	666	*/
		offset = dissect_id_E_DCH_FDD_DL_Control_Channel_Information(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_E_DCH_FDD_INFORMATION:											/*	667	*/
		offset = dissect_id_E_DCH_FDD_Information(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_E_DCH_FDD_INFORMATION_RESPONSE:									/*	668	*/
		offset = dissect_id_E_DCH_FDD_Information_Response(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_E_DCH_FDD_INFORMATION_TO_MODIFY:								/*	669	*/
		offset = dissect_id_E_DCH_FDD_Information_to_Modify(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_E_DCH_MACDFLOWS_TO_ADD:											/*	670	*/
		offset = dissect_id_E_DCH_MACdFlows_to_Add(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_E_DCH_MACDFLOWS_TO_DELETE:										/*	671	*/
		offset = dissect_id_E_DCH_MACdFlows_to_Delete(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_E_DCH_RESOURCES_INFORMATION_AUDITRSP:							/*	672	*/
		offset = dissect_id_E_DCH_Resources_Information_AuditRsp(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_E_DCH_RESOURCES_INFORMATION_RESOURCESTATUSIND:					/*	673	*/
		offset = dissect_id_E_DCH_Resources_Information_ResourceStatusInd(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_E_DCH_RL_INDICATION:											/*	674	*/
		offset = dissect_id_E_DCH_RL_Indication(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_E_DCH_RL_SET_ID:												/*	675	*/
		offset = dissect_id_E_DCH_RL_Set_ID(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_E_DPCH_INFORMATION_RL_RECONFPREPFDD:							/*	676	*/
		offset = dissect_id_E_DPCH_Information_RL_ReconfPrepFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_E_DPCH_INFORMATION_RL_SETUPRQSTFDD:								/*	677	*/
		offset = dissect_id_E_DPCH_Information_RL_SetupRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_E_RGCH_E_HICH_FDD_CODE_INFORMATION:								/*	678	*/
		offset = dissect_id_E_RGCH_E_HICH_FDD_Code_Information(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_SERVING_E_DCH_RL_ID:											/*	679	*/
		offset = dissect_id_Serving_E_DCH_RL_ID(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_UL_DPDCH_INDICATOR_FOR_E_DCH_OPERATION:							/*	680	*/
		offset = dissect_id_UL_DPDCH_Indicator_For_E_DCH_Operation(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_FDD_S_CCPCH_FRAMEOFFSET_CTCH_SETUPRQSTFDD:						/*	681	*/
		offset = dissect_id_FDD_S_CCPCH_FrameOffset_CTCH_SetupRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_E_DPCH_INFORMATION_RL_RECONFRQSTFDD:							/*	682	*/
		offset = dissect_id_E_DPCH_Information_RL_ReconfRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_MAXIMUM_TARGET_RECEIVEDTOTALWIDEBANDPOWER:						/*	683	*/
		offset = dissect_id_Maximum_Target_ReceivedTotalWideBandPower(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_E_DCHPROVIDEDBITRATEVALUEINFORMATION:							/*	684	*/
		offset = dissect_id_E_DCHProvidedBitRateValueInformation(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_HARQ_PREAMBLE_MODE_ACTIVATION_INDICATOR:						/*	685	*/
		offset = dissect_id_HARQ_Preamble_Mode_Activation_Indicator(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_RL_SPECIFIC_E_DCH_INFO:											/*	686	*/
		offset = dissect_id_RL_Specific_E_DCH_Info(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_E_DCH_CAPACITYCONSUMPTIONLAW:									/*	687	*/
		offset = dissect_id_E_DCH_CapacityConsumptionLaw(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_MULTIPLE_DEDICATEDMEASUREMENTVALUELIST_TDD_DM_RSP:				/*	688	*/
		offset = dissect_id_multiple_DedicatedMeasurementValueList_TDD_DM_Rsp(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_MULTIPLE_DEDICATEDMEASUREMENTVALUELIST_LCR_TDD_DM_RSP:			/*	689	*/
		offset = dissect_id_multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_E_DCH_REARRANGELIST_BEARER_REARRANGEIND:						/*	690	*/
		offset = dissect_id_E_DCH_RearrangeList_Bearer_RearrangeInd(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_HSDPA_AND_EDCH_CELLPORTION_INFORMATIONLISTIE_PSCH_RECONFRQST:	/*	691	*/
		break;
	case NBAP_IE_ID_MULTIPLERL_DL_CCTRCH_INFORMATIONMODIFYLIST_RL_RECONFRQSTTDD:	/*	692	*/
		offset = dissect_id_multipleRL_dl_CCTrCH_InformationModifyList_RL_ReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_TARGET_NONSERVING_EDCH_TO_TOTAL_EDCH_POWER_RATIO:				/*	693	*/
		offset = dissect_id_Target_NonServing_EDCH_To_Total_EDCH_Power_Ratio(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CELLPORTION_INFORMATIONITEM_CELL_RECONFRQSTFDD:					/*	694	*/
		offset = dissect_id_CellPortion_InformationItem_Cell_ReconfRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_CELLPORTION_INFORMATIONLIST_CELL_RECONFRQSTFDD:					/*	695	*/
		offset = dissect_id_CellPortion_InformationList_Cell_ReconfRqstFDD(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_MULTIPLE_PUSCH_INFOLIST_DM_RSP:									/*	696	*/
		offset = dissect_id_multiple_PUSCH_InfoList_DM_Rsp(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_MULTIPLE_PUSCH_INFOLIST_DM_RPRT:								/*	697	*/
		offset = dissect_id_multiple_PUSCH_InfoList_DM_Rprt(tvb, offset, actx, value_tree);
		break;
	case NBAP_IE_ID_REFERENCE_RECEIVEDTOTALWIDEBANDPOWER:							/*	698	*/
		offset = dissect_id_Reference_ReceivedTotalWideBandPower(tvb, offset, actx, value_tree);
		break;
	default:
		offset = offset + length;
		break;
	}
	BYTE_ALIGN_OFFSET(offset);	
	return offset;
}
static void
dissect_nbap(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
{
	proto_item	*nbap_item = NULL;
	proto_tree	*nbap_tree = NULL;

	top_tree = tree;

	/* make entry in the Protocol column on summary display */
	if (check_col(pinfo->cinfo, COL_PROTOCOL))
		col_set_str(pinfo->cinfo, COL_PROTOCOL, "NBAP");

    /* create the nbap protocol tree */
    nbap_item = proto_tree_add_item(tree, proto_nbap, tvb, 0, -1, FALSE);
    nbap_tree = proto_item_add_subtree(nbap_item, ett_nbap);

	dissect_NBAP_PDU_PDU(tvb, pinfo, nbap_tree);
}
/*--- proto_register_nbap -------------------------------------------*/
void proto_register_nbap(void) {

  /* List of fields */

  static hf_register_info hf[] = {
	{ &hf_nbap_pdu_length,
		{ "PDU Length", "nbap.pdu_length", FT_UINT32, BASE_DEC,
		NULL, 0, "Number of octets in the PDU", HFILL }},
	{ &hf_nbap_IE_length,
		{ "IE Length", "nbap.ie_length", FT_UINT32, BASE_DEC,
		NULL, 0, "Number of octets in the IE", HFILL }},

/*--- Included file: packet-nbap-hfarr.c ---*/
#line 1 "packet-nbap-hfarr.c"
    { &hf_nbap_NBAP_PDU_PDU,
      { "NBAP-PDU", "nbap.NBAP_PDU",
        FT_UINT32, BASE_DEC, VALS(nbap_NBAP_PDU_vals), 0,
        "nbap.NBAP_PDU", HFILL }},
    { &hf_nbap_initiatingMessage,
      { "initiatingMessage", "nbap.initiatingMessage",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.InitiatingMessage", HFILL }},
    { &hf_nbap_succesfulOutcome,
      { "succesfulOutcome", "nbap.succesfulOutcome",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.SuccessfulOutcome", HFILL }},
    { &hf_nbap_unsuccesfulOutcome,
      { "unsuccesfulOutcome", "nbap.unsuccesfulOutcome",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UnsuccessfulOutcome", HFILL }},
    { &hf_nbap_outcome,
      { "outcome", "nbap.outcome",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Outcome", HFILL }},
    { &hf_nbap_procedureID,
      { "procedureID", "nbap.procedureID",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProcedureID", HFILL }},
    { &hf_nbap_criticality,
      { "criticality", "nbap.criticality",
        FT_UINT32, BASE_DEC, VALS(nbap_Criticality_vals), 0,
        "nbap.Criticality", HFILL }},
    { &hf_nbap_messageDiscriminator,
      { "messageDiscriminator", "nbap.messageDiscriminator",
        FT_UINT32, BASE_DEC, VALS(nbap_MessageDiscriminator_vals), 0,
        "nbap.MessageDiscriminator", HFILL }},
    { &hf_nbap_transactionID,
      { "transactionID", "nbap.transactionID",
        FT_UINT32, BASE_DEC, VALS(nbap_TransactionID_vals), 0,
        "nbap.TransactionID", HFILL }},
    { &hf_nbap_initiatingMessageValue,
      { "initiatingMessageValue", "nbap.initiatingMessageValue",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.InitiatingMessageValue", HFILL }},
    { &hf_nbap_successfulOutcomeValue,
      { "successfulOutcomeValue", "nbap.successfulOutcomeValue",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.SuccessfulOutcomeValue", HFILL }},
    { &hf_nbap_unsuccessfulOutcomeValue,
      { "unsuccessfulOutcomeValue", "nbap.unsuccessfulOutcomeValue",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UnsuccessfulOutcomeValue", HFILL }},
    { &hf_nbap_outcomeValue,
      { "outcomeValue", "nbap.outcomeValue",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.OutcomeValue", HFILL }},
    { &hf_nbap_id_audit,
      { "id-audit", "nbap.id_audit",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.AuditRequest", HFILL }},
    { &hf_nbap_id_auditRequired,
      { "id-auditRequired", "nbap.id_auditRequired",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.AuditRequiredIndication", HFILL }},
    { &hf_nbap_id_blockResource,
      { "id-blockResource", "nbap.id_blockResource",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.BlockResourceRequest", HFILL }},
    { &hf_nbap_id_cellDeletion,
      { "id-cellDeletion", "nbap.id_cellDeletion",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CellDeletionRequest", HFILL }},
    { &hf_nbap_id_cellReconfiguration,
      { "id-cellReconfiguration", "nbap.id_cellReconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CellReconfigurationRequestFDD", HFILL }},
    { &hf_nbap_id_cellReconfiguration_tdd,
      { "id-cellReconfiguration-tdd", "nbap.id_cellReconfiguration_tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CellReconfigurationRequestTDD", HFILL }},
    { &hf_nbap_id_cellSetup,
      { "id-cellSetup", "nbap.id_cellSetup",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CellSetupRequestFDD", HFILL }},
    { &hf_nbap_id_cellSetup_tdd,
      { "id-cellSetup-tdd", "nbap.id_cellSetup_tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CellSetupRequestTDD", HFILL }},
    { &hf_nbap_id_commonMeasurementFailure,
      { "id-commonMeasurementFailure", "nbap.id_commonMeasurementFailure",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CommonMeasurementFailureIndication", HFILL }},
    { &hf_nbap_id_commonMeasurementInitiation,
      { "id-commonMeasurementInitiation", "nbap.id_commonMeasurementInitiation",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CommonMeasurementInitiationRequest", HFILL }},
    { &hf_nbap_id_commonMeasurementReport,
      { "id-commonMeasurementReport", "nbap.id_commonMeasurementReport",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CommonMeasurementReport", HFILL }},
    { &hf_nbap_id_commonMeasurementTermination,
      { "id-commonMeasurementTermination", "nbap.id_commonMeasurementTermination",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CommonMeasurementTerminationRequest", HFILL }},
    { &hf_nbap_id_commonTransportChannelDelete,
      { "id-commonTransportChannelDelete", "nbap.id_commonTransportChannelDelete",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CommonTransportChannelDeletionRequest", HFILL }},
    { &hf_nbap_id_commonTransportChannelReconfigure,
      { "id-commonTransportChannelReconfigure", "nbap.id_commonTransportChannelReconfigure",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CommonTransportChannelReconfigurationRequestFDD", HFILL }},
    { &hf_nbap_id_commonTransportChannelReconfigure_tdd,
      { "id-commonTransportChannelReconfigure-tdd", "nbap.id_commonTransportChannelReconfigure_tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CommonTransportChannelReconfigurationRequestTDD", HFILL }},
    { &hf_nbap_id_commonTransportChannelSetup,
      { "id-commonTransportChannelSetup", "nbap.id_commonTransportChannelSetup",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CommonTransportChannelSetupRequestFDD", HFILL }},
    { &hf_nbap_id_commonTransportChannelSetup_tdd,
      { "id-commonTransportChannelSetup-tdd", "nbap.id_commonTransportChannelSetup_tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CommonTransportChannelSetupRequestTDD", HFILL }},
    { &hf_nbap_id_reset,
      { "id-reset", "nbap.id_reset",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ResetRequest", HFILL }},
    { &hf_nbap_id_compressedModeCommand,
      { "id-compressedModeCommand", "nbap.id_compressedModeCommand",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CompressedModeCommand", HFILL }},
    { &hf_nbap_id_dedicatedMeasurementFailure,
      { "id-dedicatedMeasurementFailure", "nbap.id_dedicatedMeasurementFailure",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DedicatedMeasurementFailureIndication", HFILL }},
    { &hf_nbap_id_dedicatedMeasurementInitiation,
      { "id-dedicatedMeasurementInitiation", "nbap.id_dedicatedMeasurementInitiation",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DedicatedMeasurementInitiationRequest", HFILL }},
    { &hf_nbap_id_dedicatedMeasurementReport,
      { "id-dedicatedMeasurementReport", "nbap.id_dedicatedMeasurementReport",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DedicatedMeasurementReport", HFILL }},
    { &hf_nbap_id_dedicatedMeasurementTermination,
      { "id-dedicatedMeasurementTermination", "nbap.id_dedicatedMeasurementTermination",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DedicatedMeasurementTerminationRequest", HFILL }},
    { &hf_nbap_id_downlinkPowerControl,
      { "id-downlinkPowerControl", "nbap.id_downlinkPowerControl",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_PowerControlRequest", HFILL }},
    { &hf_nbap_id_errorIndicationForDedicated,
      { "id-errorIndicationForDedicated", "nbap.id_errorIndicationForDedicated",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ErrorIndication", HFILL }},
    { &hf_nbap_id_privateMessageForDedicated,
      { "id-privateMessageForDedicated", "nbap.id_privateMessageForDedicated",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PrivateMessage", HFILL }},
    { &hf_nbap_id_radioLinkAddition,
      { "id-radioLinkAddition", "nbap.id_radioLinkAddition",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RadioLinkAdditionRequestFDD", HFILL }},
    { &hf_nbap_id_radioLinkDeletion,
      { "id-radioLinkDeletion", "nbap.id_radioLinkDeletion",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RadioLinkDeletionRequest", HFILL }},
    { &hf_nbap_id_radioLinkFailure,
      { "id-radioLinkFailure", "nbap.id_radioLinkFailure",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RadioLinkFailureIndication", HFILL }},
    { &hf_nbap_id_radioLinkRestoration,
      { "id-radioLinkRestoration", "nbap.id_radioLinkRestoration",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RadioLinkRestoreIndication", HFILL }},
    { &hf_nbap_id_radioLinkSetup,
      { "id-radioLinkSetup", "nbap.id_radioLinkSetup",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RadioLinkSetupRequestFDD", HFILL }},
    { &hf_nbap_id_radioLinkSetup_tdd,
      { "id-radioLinkSetup-tdd", "nbap.id_radioLinkSetup_tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RadioLinkSetupRequestTDD", HFILL }},
    { &hf_nbap_id_resourceStatusIndication,
      { "id-resourceStatusIndication", "nbap.id_resourceStatusIndication",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ResourceStatusIndication", HFILL }},
    { &hf_nbap_id_synchronisedRadioLinkReconfigurationCancellation,
      { "id-synchronisedRadioLinkReconfigurationCancellation", "nbap.id_synchronisedRadioLinkReconfigurationCancellation",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RadioLinkReconfigurationCancel", HFILL }},
    { &hf_nbap_id_synchronisedRadioLinkReconfigurationCommit,
      { "id-synchronisedRadioLinkReconfigurationCommit", "nbap.id_synchronisedRadioLinkReconfigurationCommit",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RadioLinkReconfigurationCommit", HFILL }},
    { &hf_nbap_id_synchronisedRadioLinkReconfigurationPreparation,
      { "id-synchronisedRadioLinkReconfigurationPreparation", "nbap.id_synchronisedRadioLinkReconfigurationPreparation",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RadioLinkReconfigurationPrepareFDD", HFILL }},
    { &hf_nbap_id_synchronisedRadioLinkReconfigurationPreparation_tdd,
      { "id-synchronisedRadioLinkReconfigurationPreparation-tdd", "nbap.id_synchronisedRadioLinkReconfigurationPreparation_tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RadioLinkReconfigurationPrepareTDD", HFILL }},
    { &hf_nbap_id_systemInformationUpdate,
      { "id-systemInformationUpdate", "nbap.id_systemInformationUpdate",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.SystemInformationUpdateRequest", HFILL }},
    { &hf_nbap_id_unblockResource,
      { "id-unblockResource", "nbap.id_unblockResource",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UnblockResourceIndication", HFILL }},
    { &hf_nbap_id_unSynchronisedRadioLinkReconfiguration,
      { "id-unSynchronisedRadioLinkReconfiguration", "nbap.id_unSynchronisedRadioLinkReconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RadioLinkReconfigurationRequestFDD", HFILL }},
    { &hf_nbap_id_unSynchronisedRadioLinkReconfiguration_tdd,
      { "id-unSynchronisedRadioLinkReconfiguration-tdd", "nbap.id_unSynchronisedRadioLinkReconfiguration_tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RadioLinkReconfigurationRequestTDD", HFILL }},
    { &hf_nbap_id_downlinkPowerTimeslotControl,
      { "id-downlinkPowerTimeslotControl", "nbap.id_downlinkPowerTimeslotControl",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_PowerTimeslotControlRequest", HFILL }},
    { &hf_nbap_id_errorIndicationForCommon,
      { "id-errorIndicationForCommon", "nbap.id_errorIndicationForCommon",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ErrorIndication", HFILL }},
    { &hf_nbap_id_privateMessageForCommon,
      { "id-privateMessageForCommon", "nbap.id_privateMessageForCommon",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PrivateMessage", HFILL }},
    { &hf_nbap_id_physicalSharedChannelReconfiguration,
      { "id-physicalSharedChannelReconfiguration", "nbap.id_physicalSharedChannelReconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PhysicalSharedChannelReconfigurationRequestFDD", HFILL }},
    { &hf_nbap_id_physicalSharedChannelReconfiguration_tdd,
      { "id-physicalSharedChannelReconfiguration-tdd", "nbap.id_physicalSharedChannelReconfiguration_tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PhysicalSharedChannelReconfigurationRequestTDD", HFILL }},
    { &hf_nbap_id_radioLinkPreemption,
      { "id-radioLinkPreemption", "nbap.id_radioLinkPreemption",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RadioLinkPreemptionRequiredIndication", HFILL }},
    { &hf_nbap_id_informationExchangeFailure,
      { "id-informationExchangeFailure", "nbap.id_informationExchangeFailure",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.InformationExchangeFailureIndication", HFILL }},
    { &hf_nbap_id_informationExchangeInitiation,
      { "id-informationExchangeInitiation", "nbap.id_informationExchangeInitiation",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.InformationExchangeInitiationRequest", HFILL }},
    { &hf_nbap_id_informationExchangeTermination,
      { "id-informationExchangeTermination", "nbap.id_informationExchangeTermination",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.InformationExchangeTerminationRequest", HFILL }},
    { &hf_nbap_id_informationReporting,
      { "id-informationReporting", "nbap.id_informationReporting",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.InformationReport", HFILL }},
    { &hf_nbap_id_cellSynchronisationAdjustment,
      { "id-cellSynchronisationAdjustment", "nbap.id_cellSynchronisationAdjustment",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CellSynchronisationAdjustmentRequestTDD", HFILL }},
    { &hf_nbap_id_cellSynchronisationInitiation,
      { "id-cellSynchronisationInitiation", "nbap.id_cellSynchronisationInitiation",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CellSynchronisationInitiationRequestTDD", HFILL }},
    { &hf_nbap_id_cellSynchronisationReconfiguration,
      { "id-cellSynchronisationReconfiguration", "nbap.id_cellSynchronisationReconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CellSynchronisationReconfigurationRequestTDD", HFILL }},
    { &hf_nbap_id_cellSynchronisationReporting,
      { "id-cellSynchronisationReporting", "nbap.id_cellSynchronisationReporting",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CellSynchronisationReportTDD", HFILL }},
    { &hf_nbap_id_cellSynchronisationTermination,
      { "id-cellSynchronisationTermination", "nbap.id_cellSynchronisationTermination",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CellSynchronisationTerminationRequestTDD", HFILL }},
    { &hf_nbap_id_cellSynchronisationFailure,
      { "id-cellSynchronisationFailure", "nbap.id_cellSynchronisationFailure",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CellSynchronisationFailureIndicationTDD", HFILL }},
    { &hf_nbap_id_BearerRearrangement,
      { "id-BearerRearrangement", "nbap.id_BearerRearrangement",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.BearerRearrangementIndication", HFILL }},
    { &hf_nbap_id_radioLinkActivation,
      { "id-radioLinkActivation", "nbap.id_radioLinkActivation",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RadioLinkActivationCommandFDD", HFILL }},
    { &hf_nbap_id_radioLinkActivation_tdd,
      { "id-radioLinkActivation-tdd", "nbap.id_radioLinkActivation_tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RadioLinkActivationCommandTDD", HFILL }},
    { &hf_nbap_id_radioLinkParameterUpdate,
      { "id-radioLinkParameterUpdate", "nbap.id_radioLinkParameterUpdate",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RadioLinkParameterUpdateIndicationFDD", HFILL }},
    { &hf_nbap_id_radioLinkParameterUpdate_tdd,
      { "id-radioLinkParameterUpdate-tdd", "nbap.id_radioLinkParameterUpdate_tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RadioLinkParameterUpdateIndicationTDD", HFILL }},
    { &hf_nbap_id_mBMSNotificationUpdate,
      { "id-mBMSNotificationUpdate", "nbap.id_mBMSNotificationUpdate",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.MBMSNotificationUpdateCommand", HFILL }},
    { &hf_nbap_id_audit1,
      { "id-audit", "nbap.id_audit",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.AuditResponse", HFILL }},
    { &hf_nbap_id_blockResource1,
      { "id-blockResource", "nbap.id_blockResource",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.BlockResourceResponse", HFILL }},
    { &hf_nbap_id_cellDeletion1,
      { "id-cellDeletion", "nbap.id_cellDeletion",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CellDeletionResponse", HFILL }},
    { &hf_nbap_id_cellReconfiguration1,
      { "id-cellReconfiguration", "nbap.id_cellReconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CellReconfigurationResponse", HFILL }},
    { &hf_nbap_id_cellSetup1,
      { "id-cellSetup", "nbap.id_cellSetup",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CellSetupResponse", HFILL }},
    { &hf_nbap_id_commonMeasurementInitiation1,
      { "id-commonMeasurementInitiation", "nbap.id_commonMeasurementInitiation",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CommonMeasurementInitiationFailure", HFILL }},
    { &hf_nbap_id_commonTransportChannelDelete1,
      { "id-commonTransportChannelDelete", "nbap.id_commonTransportChannelDelete",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CommonTransportChannelDeletionResponse", HFILL }},
    { &hf_nbap_id_commonTransportChannelReconfigure1,
      { "id-commonTransportChannelReconfigure", "nbap.id_commonTransportChannelReconfigure",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CommonTransportChannelReconfigurationResponse", HFILL }},
    { &hf_nbap_id_commonTransportChannelSetup1,
      { "id-commonTransportChannelSetup", "nbap.id_commonTransportChannelSetup",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CommonTransportChannelSetupResponse", HFILL }},
    { &hf_nbap_id_reset1,
      { "id-reset", "nbap.id_reset",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ResetResponse", HFILL }},
    { &hf_nbap_id_dedicatedMeasurementInitiation1,
      { "id-dedicatedMeasurementInitiation", "nbap.id_dedicatedMeasurementInitiation",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DedicatedMeasurementInitiationResponse", HFILL }},
    { &hf_nbap_id_radioLinkAddition1,
      { "id-radioLinkAddition", "nbap.id_radioLinkAddition",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RadioLinkAdditionResponseFDD", HFILL }},
    { &hf_nbap_id_radioLinkAddition_tdd,
      { "id-radioLinkAddition-tdd", "nbap.id_radioLinkAddition_tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RadioLinkAdditionResponseTDD", HFILL }},
    { &hf_nbap_id_radioLinkDeletion1,
      { "id-radioLinkDeletion", "nbap.id_radioLinkDeletion",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RadioLinkDeletionResponse", HFILL }},
    { &hf_nbap_id_radioLinkSetup1,
      { "id-radioLinkSetup", "nbap.id_radioLinkSetup",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RadioLinkSetupResponseFDD", HFILL }},
    { &hf_nbap_id_radioLinkSetup_tdd1,
      { "id-radioLinkSetup-tdd", "nbap.id_radioLinkSetup_tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RadioLinkSetupResponseTDD", HFILL }},
    { &hf_nbap_id_synchronisedRadioLinkReconfigurationPreparation1,
      { "id-synchronisedRadioLinkReconfigurationPreparation", "nbap.id_synchronisedRadioLinkReconfigurationPreparation",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RadioLinkReconfigurationReady", HFILL }},
    { &hf_nbap_id_systemInformationUpdate1,
      { "id-systemInformationUpdate", "nbap.id_systemInformationUpdate",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.SystemInformationUpdateResponse", HFILL }},
    { &hf_nbap_id_unSynchronisedRadioLinkReconfiguration1,
      { "id-unSynchronisedRadioLinkReconfiguration", "nbap.id_unSynchronisedRadioLinkReconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RadioLinkReconfigurationResponse", HFILL }},
    { &hf_nbap_id_physicalSharedChannelReconfiguration1,
      { "id-physicalSharedChannelReconfiguration", "nbap.id_physicalSharedChannelReconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PhysicalSharedChannelReconfigurationResponse", HFILL }},
    { &hf_nbap_id_informationExchangeInitiation1,
      { "id-informationExchangeInitiation", "nbap.id_informationExchangeInitiation",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.InformationExchangeInitiationResponse", HFILL }},
    { &hf_nbap_id_cellSynchronisationAdjustment1,
      { "id-cellSynchronisationAdjustment", "nbap.id_cellSynchronisationAdjustment",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CellSynchronisationAdjustmentResponseTDD", HFILL }},
    { &hf_nbap_id_cellSynchronisationInitiation1,
      { "id-cellSynchronisationInitiation", "nbap.id_cellSynchronisationInitiation",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CellSynchronisationInitiationResponseTDD", HFILL }},
    { &hf_nbap_id_cellSynchronisationReconfiguration1,
      { "id-cellSynchronisationReconfiguration", "nbap.id_cellSynchronisationReconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CellSynchronisationReconfigurationResponseTDD", HFILL }},
    { &hf_nbap_id_audit2,
      { "id-audit", "nbap.id_audit",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.AuditFailure", HFILL }},
    { &hf_nbap_id_blockResource2,
      { "id-blockResource", "nbap.id_blockResource",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.BlockResourceFailure", HFILL }},
    { &hf_nbap_id_cellReconfiguration2,
      { "id-cellReconfiguration", "nbap.id_cellReconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CellReconfigurationFailure", HFILL }},
    { &hf_nbap_id_cellSetup2,
      { "id-cellSetup", "nbap.id_cellSetup",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CellSetupFailure", HFILL }},
    { &hf_nbap_id_commonMeasurementInitiation2,
      { "id-commonMeasurementInitiation", "nbap.id_commonMeasurementInitiation",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CommonMeasurementInitiationResponse", HFILL }},
    { &hf_nbap_id_commonTransportChannelReconfigure2,
      { "id-commonTransportChannelReconfigure", "nbap.id_commonTransportChannelReconfigure",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CommonTransportChannelReconfigurationFailure", HFILL }},
    { &hf_nbap_id_commonTransportChannelSetup2,
      { "id-commonTransportChannelSetup", "nbap.id_commonTransportChannelSetup",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CommonTransportChannelSetupFailure", HFILL }},
    { &hf_nbap_id_dedicatedMeasurementInitiation2,
      { "id-dedicatedMeasurementInitiation", "nbap.id_dedicatedMeasurementInitiation",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DedicatedMeasurementInitiationFailure", HFILL }},
    { &hf_nbap_id_radioLinkAddition2,
      { "id-radioLinkAddition", "nbap.id_radioLinkAddition",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RadioLinkAdditionFailureFDD", HFILL }},
    { &hf_nbap_id_radioLinkAddition_tdd1,
      { "id-radioLinkAddition-tdd", "nbap.id_radioLinkAddition_tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RadioLinkAdditionFailureTDD", HFILL }},
    { &hf_nbap_id_radioLinkSetup2,
      { "id-radioLinkSetup", "nbap.id_radioLinkSetup",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RadioLinkSetupFailureFDD", HFILL }},
    { &hf_nbap_id_radioLinkSetup_tdd2,
      { "id-radioLinkSetup-tdd", "nbap.id_radioLinkSetup_tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RadioLinkSetupFailureTDD", HFILL }},
    { &hf_nbap_id_synchronisedRadioLinkReconfigurationPreparation2,
      { "id-synchronisedRadioLinkReconfigurationPreparation", "nbap.id_synchronisedRadioLinkReconfigurationPreparation",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RadioLinkReconfigurationFailure", HFILL }},
    { &hf_nbap_id_systemInformationUpdate2,
      { "id-systemInformationUpdate", "nbap.id_systemInformationUpdate",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.SystemInformationUpdateFailure", HFILL }},
    { &hf_nbap_id_unSynchronisedRadioLinkReconfiguration2,
      { "id-unSynchronisedRadioLinkReconfiguration", "nbap.id_unSynchronisedRadioLinkReconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RadioLinkReconfigurationFailure", HFILL }},
    { &hf_nbap_id_physicalSharedChannelReconfiguration2,
      { "id-physicalSharedChannelReconfiguration", "nbap.id_physicalSharedChannelReconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PhysicalSharedChannelReconfigurationFailure", HFILL }},
    { &hf_nbap_id_informationExchangeInitiation2,
      { "id-informationExchangeInitiation", "nbap.id_informationExchangeInitiation",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.InformationExchangeInitiationFailure", HFILL }},
    { &hf_nbap_id_cellSynchronisationAdjustment2,
      { "id-cellSynchronisationAdjustment", "nbap.id_cellSynchronisationAdjustment",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CellSynchronisationAdjustmentFailureTDD", HFILL }},
    { &hf_nbap_id_cellSynchronisationInitiation2,
      { "id-cellSynchronisationInitiation", "nbap.id_cellSynchronisationInitiation",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CellSynchronisationInitiationFailureTDD", HFILL }},
    { &hf_nbap_id_cellSynchronisationReconfiguration2,
      { "id-cellSynchronisationReconfiguration", "nbap.id_cellSynchronisationReconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CellSynchronisationReconfigurationFailureTDD", HFILL }},
    { &hf_nbap_id_AICH_Information,
      { "id-AICH-Information", "nbap.id_AICH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Common_PhysicalChannel_Status_Information", HFILL }},
    { &hf_nbap_id_BCH_Information,
      { "id-BCH-Information", "nbap.id_BCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Common_TransportChannel_Status_Information", HFILL }},
    { &hf_nbap_id_BCCH_ModificationTime,
      { "id-BCCH-ModificationTime", "nbap.id_BCCH_ModificationTime",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.BCCH_ModificationTime", HFILL }},
    { &hf_nbap_id_BlockingPriorityIndicator,
      { "id-BlockingPriorityIndicator", "nbap.id_BlockingPriorityIndicator",
        FT_UINT32, BASE_DEC, VALS(nbap_BlockingPriorityIndicator_vals), 0,
        "nbap.BlockingPriorityIndicator", HFILL }},
    { &hf_nbap_id_Cause,
      { "id-Cause", "nbap.id_Cause",
        FT_UINT32, BASE_DEC, VALS(nbap_Cause_vals), 0,
        "nbap.Cause", HFILL }},
    { &hf_nbap_id_CCP_InformationItem_AuditRsp,
      { "id-CCP-InformationItem-AuditRsp", "nbap.id_CCP_InformationItem_AuditRsp",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CCP_InformationItem_AuditRsp", HFILL }},
    { &hf_nbap_id_CCP_InformationList_AuditRsp,
      { "id-CCP-InformationList-AuditRsp", "nbap.id_CCP_InformationList_AuditRsp",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CCP_InformationList_AuditRsp", HFILL }},
    { &hf_nbap_id_CCP_InformationItem_ResourceStatusInd,
      { "id-CCP-InformationItem-ResourceStatusInd", "nbap.id_CCP_InformationItem_ResourceStatusInd",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CCP_InformationItem_ResourceStatusInd", HFILL }},
    { &hf_nbap_id_Cell_InformationItem_AuditRsp,
      { "id-Cell-InformationItem-AuditRsp", "nbap.id_Cell_InformationItem_AuditRsp",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Cell_InformationItem_AuditRsp", HFILL }},
    { &hf_nbap_id_Cell_InformationItem_ResourceStatusInd,
      { "id-Cell-InformationItem-ResourceStatusInd", "nbap.id_Cell_InformationItem_ResourceStatusInd",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Cell_InformationItem_ResourceStatusInd", HFILL }},
    { &hf_nbap_id_Cell_InformationList_AuditRsp,
      { "id-Cell-InformationList-AuditRsp", "nbap.id_Cell_InformationList_AuditRsp",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Cell_InformationList_AuditRsp", HFILL }},
    { &hf_nbap_id_CellParameterID,
      { "id-CellParameterID", "nbap.id_CellParameterID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CellParameterID", HFILL }},
    { &hf_nbap_id_CFN,
      { "id-CFN", "nbap.id_CFN",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CFN", HFILL }},
    { &hf_nbap_id_C_ID,
      { "id-C-ID", "nbap.id_C_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.C_ID", HFILL }},
    { &hf_nbap_id_CommonMeasurementAccuracy,
      { "id-CommonMeasurementAccuracy", "nbap.id_CommonMeasurementAccuracy",
        FT_UINT32, BASE_DEC, VALS(nbap_CommonMeasurementAccuracy_vals), 0,
        "nbap.CommonMeasurementAccuracy", HFILL }},
    { &hf_nbap_id_CommonMeasurementObjectType_CM_Rprt,
      { "id-CommonMeasurementObjectType-CM-Rprt", "nbap.id_CommonMeasurementObjectType_CM_Rprt",
        FT_UINT32, BASE_DEC, VALS(nbap_CommonMeasurementObjectType_CM_Rprt_vals), 0,
        "nbap.CommonMeasurementObjectType_CM_Rprt", HFILL }},
    { &hf_nbap_id_CommonMeasurementObjectType_CM_Rqst,
      { "id-CommonMeasurementObjectType-CM-Rqst", "nbap.id_CommonMeasurementObjectType_CM_Rqst",
        FT_UINT32, BASE_DEC, VALS(nbap_CommonMeasurementObjectType_CM_Rqst_vals), 0,
        "nbap.CommonMeasurementObjectType_CM_Rqst", HFILL }},
    { &hf_nbap_id_CommonMeasurementObjectType_CM_Rsp,
      { "id-CommonMeasurementObjectType-CM-Rsp", "nbap.id_CommonMeasurementObjectType_CM_Rsp",
        FT_UINT32, BASE_DEC, VALS(nbap_CommonMeasurementObjectType_CM_Rsp_vals), 0,
        "nbap.CommonMeasurementObjectType_CM_Rsp", HFILL }},
    { &hf_nbap_id_CommonMeasurementType,
      { "id-CommonMeasurementType", "nbap.id_CommonMeasurementType",
        FT_UINT32, BASE_DEC, VALS(nbap_CommonMeasurementType_vals), 0,
        "nbap.CommonMeasurementType", HFILL }},
    { &hf_nbap_id_CommonPhysicalChannelID,
      { "id-CommonPhysicalChannelID", "nbap.id_CommonPhysicalChannelID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CommonPhysicalChannelID", HFILL }},
    { &hf_nbap_id_CommonPhysicalChannelType_CTCH_SetupRqstFDD,
      { "id-CommonPhysicalChannelType-CTCH-SetupRqstFDD", "nbap.id_CommonPhysicalChannelType_CTCH_SetupRqstFDD",
        FT_UINT32, BASE_DEC, VALS(nbap_CommonPhysicalChannelType_CTCH_SetupRqstFDD_vals), 0,
        "nbap.CommonPhysicalChannelType_CTCH_SetupRqstFDD", HFILL }},
    { &hf_nbap_id_CommonPhysicalChannelType_CTCH_SetupRqstTDD,
      { "id-CommonPhysicalChannelType-CTCH-SetupRqstTDD", "nbap.id_CommonPhysicalChannelType_CTCH_SetupRqstTDD",
        FT_UINT32, BASE_DEC, VALS(nbap_CommonPhysicalChannelType_CTCH_SetupRqstTDD_vals), 0,
        "nbap.CommonPhysicalChannelType_CTCH_SetupRqstTDD", HFILL }},
    { &hf_nbap_id_CommunicationControlPortID,
      { "id-CommunicationControlPortID", "nbap.id_CommunicationControlPortID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CommunicationControlPortID", HFILL }},
    { &hf_nbap_id_ConfigurationGenerationID,
      { "id-ConfigurationGenerationID", "nbap.id_ConfigurationGenerationID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.ConfigurationGenerationID", HFILL }},
    { &hf_nbap_id_CRNC_CommunicationContextID,
      { "id-CRNC-CommunicationContextID", "nbap.id_CRNC_CommunicationContextID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CRNC_CommunicationContextID", HFILL }},
    { &hf_nbap_id_CriticalityDiagnostics,
      { "id-CriticalityDiagnostics", "nbap.id_CriticalityDiagnostics",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CriticalityDiagnostics", HFILL }},
    { &hf_nbap_id_DCHs_to_Add_FDD,
      { "id-DCHs-to-Add-FDD", "nbap.id_DCHs_to_Add_FDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DCH_FDD_Information", HFILL }},
    { &hf_nbap_id_DCHs_to_Add_TDD,
      { "id-DCHs-to-Add-TDD", "nbap.id_DCHs_to_Add_TDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DCH_TDD_Information", HFILL }},
    { &hf_nbap_id_DCH_DeleteList_RL_ReconfPrepFDD,
      { "id-DCH-DeleteList-RL-ReconfPrepFDD", "nbap.id_DCH_DeleteList_RL_ReconfPrepFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DCH_DeleteList_RL_ReconfPrepFDD", HFILL }},
    { &hf_nbap_id_DCH_DeleteList_RL_ReconfPrepTDD,
      { "id-DCH-DeleteList-RL-ReconfPrepTDD", "nbap.id_DCH_DeleteList_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DCH_DeleteList_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_id_DCH_DeleteList_RL_ReconfRqstFDD,
      { "id-DCH-DeleteList-RL-ReconfRqstFDD", "nbap.id_DCH_DeleteList_RL_ReconfRqstFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DCH_DeleteList_RL_ReconfRqstFDD", HFILL }},
    { &hf_nbap_id_DCH_DeleteList_RL_ReconfRqstTDD,
      { "id-DCH-DeleteList-RL-ReconfRqstTDD", "nbap.id_DCH_DeleteList_RL_ReconfRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DCH_DeleteList_RL_ReconfRqstTDD", HFILL }},
    { &hf_nbap_id_DCH_FDD_Information,
      { "id-DCH-FDD-Information", "nbap.id_DCH_FDD_Information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DCH_FDD_Information", HFILL }},
    { &hf_nbap_id_DCH_TDD_Information,
      { "id-DCH-TDD-Information", "nbap.id_DCH_TDD_Information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DCH_TDD_Information", HFILL }},
    { &hf_nbap_id_DCH_InformationResponse,
      { "id-DCH-InformationResponse", "nbap.id_DCH_InformationResponse",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DCH_InformationResponse", HFILL }},
    { &hf_nbap_id_FDD_DCHs_to_Modify,
      { "id-FDD-DCHs-to-Modify", "nbap.id_FDD_DCHs_to_Modify",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.FDD_DCHs_to_Modify", HFILL }},
    { &hf_nbap_id_TDD_DCHs_to_Modify,
      { "id-TDD-DCHs-to-Modify", "nbap.id_TDD_DCHs_to_Modify",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.TDD_DCHs_to_Modify", HFILL }},
    { &hf_nbap_id_DCH_RearrangeList_Bearer_RearrangeInd,
      { "id-DCH-RearrangeList-Bearer-RearrangeInd", "nbap.id_DCH_RearrangeList_Bearer_RearrangeInd",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DCH_RearrangeList_Bearer_RearrangeInd", HFILL }},
    { &hf_nbap_id_DedicatedMeasurementObjectType_DM_Rprt,
      { "id-DedicatedMeasurementObjectType-DM-Rprt", "nbap.id_DedicatedMeasurementObjectType_DM_Rprt",
        FT_UINT32, BASE_DEC, VALS(nbap_DedicatedMeasurementObjectType_DM_Rprt_vals), 0,
        "nbap.DedicatedMeasurementObjectType_DM_Rprt", HFILL }},
    { &hf_nbap_id_DedicatedMeasurementObjectType_DM_Rqst,
      { "id-DedicatedMeasurementObjectType-DM-Rqst", "nbap.id_DedicatedMeasurementObjectType_DM_Rqst",
        FT_UINT32, BASE_DEC, VALS(nbap_DedicatedMeasurementObjectType_DM_Rqst_vals), 0,
        "nbap.DedicatedMeasurementObjectType_DM_Rqst", HFILL }},
    { &hf_nbap_id_DedicatedMeasurementObjectType_DM_Rsp,
      { "id-DedicatedMeasurementObjectType-DM-Rsp", "nbap.id_DedicatedMeasurementObjectType_DM_Rsp",
        FT_UINT32, BASE_DEC, VALS(nbap_DedicatedMeasurementObjectType_DM_Rsp_vals), 0,
        "nbap.DedicatedMeasurementObjectType_DM_Rsp", HFILL }},
    { &hf_nbap_id_DedicatedMeasurementType,
      { "id-DedicatedMeasurementType", "nbap.id_DedicatedMeasurementType",
        FT_UINT32, BASE_DEC, VALS(nbap_DedicatedMeasurementType_vals), 0,
        "nbap.DedicatedMeasurementType", HFILL }},
    { &hf_nbap_id_DL_CCTrCH_InformationItem_RL_SetupRqstTDD,
      { "id-DL-CCTrCH-InformationItem-RL-SetupRqstTDD", "nbap.id_DL_CCTrCH_InformationItem_RL_SetupRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_CCTrCH_InformationItem_RL_SetupRqstTDD", HFILL }},
    { &hf_nbap_id_DL_CCTrCH_InformationList_RL_AdditionRqstTDD,
      { "id-DL-CCTrCH-InformationList-RL-AdditionRqstTDD", "nbap.id_DL_CCTrCH_InformationList_RL_AdditionRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_CCTrCH_InformationList_RL_AdditionRqstTDD", HFILL }},
    { &hf_nbap_id_DL_CCTrCH_InformationList_RL_SetupRqstTDD,
      { "id-DL-CCTrCH-InformationList-RL-SetupRqstTDD", "nbap.id_DL_CCTrCH_InformationList_RL_SetupRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_CCTrCH_InformationList_RL_SetupRqstTDD", HFILL }},
    { &hf_nbap_id_DL_DPCH_InformationItem_RL_AdditionRqstTDD,
      { "id-DL-DPCH-InformationItem-RL-AdditionRqstTDD", "nbap.id_DL_DPCH_InformationItem_RL_AdditionRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_DPCH_InformationItem_RL_AdditionRqstTDD", HFILL }},
    { &hf_nbap_id_DL_DPCH_InformationList_RL_SetupRqstTDD,
      { "id-DL-DPCH-InformationList-RL-SetupRqstTDD", "nbap.id_DL_DPCH_InformationList_RL_SetupRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_DPCH_InformationItem_RL_SetupRqstTDD", HFILL }},
    { &hf_nbap_id_DL_DPCH_Information_RL_ReconfPrepFDD,
      { "id-DL-DPCH-Information-RL-ReconfPrepFDD", "nbap.id_DL_DPCH_Information_RL_ReconfPrepFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_DPCH_Information_RL_ReconfPrepFDD", HFILL }},
    { &hf_nbap_id_DL_DPCH_Information_RL_ReconfRqstFDD,
      { "id-DL-DPCH-Information-RL-ReconfRqstFDD", "nbap.id_DL_DPCH_Information_RL_ReconfRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_DPCH_Information_RL_ReconfRqstFDD", HFILL }},
    { &hf_nbap_id_DL_DPCH_Information_RL_SetupRqstFDD,
      { "id-DL-DPCH-Information-RL-SetupRqstFDD", "nbap.id_DL_DPCH_Information_RL_SetupRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_DPCH_Information_RL_SetupRqstFDD", HFILL }},
    { &hf_nbap_id_DL_DPCH_TimingAdjustment,
      { "id-DL-DPCH-TimingAdjustment", "nbap.id_DL_DPCH_TimingAdjustment",
        FT_UINT32, BASE_DEC, VALS(nbap_DL_DPCH_TimingAdjustment_vals), 0,
        "nbap.DL_DPCH_TimingAdjustment", HFILL }},
    { &hf_nbap_id_DL_ReferencePowerInformationItem_DL_PC_Rqst,
      { "id-DL-ReferencePowerInformationItem-DL-PC-Rqst", "nbap.id_DL_ReferencePowerInformationItem_DL_PC_Rqst",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_ReferencePowerInformationItem_DL_PC_Rqst", HFILL }},
    { &hf_nbap_id_DLReferencePower,
      { "id-DLReferencePower", "nbap.id_DLReferencePower",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_id_DLReferencePowerList_DL_PC_Rqst,
      { "id-DLReferencePowerList-DL-PC-Rqst", "nbap.id_DLReferencePowerList_DL_PC_Rqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_ReferencePowerInformationList_DL_PC_Rqst", HFILL }},
    { &hf_nbap_id_DSCHs_to_Add_TDD,
      { "id-DSCHs-to-Add-TDD", "nbap.id_DSCHs_to_Add_TDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DSCH_TDD_Information", HFILL }},
    { &hf_nbap_id_DSCH_Information_DeleteList_RL_ReconfPrepTDD,
      { "id-DSCH-Information-DeleteList-RL-ReconfPrepTDD", "nbap.id_DSCH_Information_DeleteList_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DSCH_Information_DeleteList_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_id_DSCH_Information_ModifyList_RL_ReconfPrepTDD,
      { "id-DSCH-Information-ModifyList-RL-ReconfPrepTDD", "nbap.id_DSCH_Information_ModifyList_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DSCH_Information_ModifyList_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_id_DSCH_InformationResponse,
      { "id-DSCH-InformationResponse", "nbap.id_DSCH_InformationResponse",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DSCH_InformationResponse", HFILL }},
    { &hf_nbap_id_DSCH_TDD_Information,
      { "id-DSCH-TDD-Information", "nbap.id_DSCH_TDD_Information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DSCH_TDD_Information", HFILL }},
    { &hf_nbap_id_DSCH_RearrangeList_Bearer_RearrangeInd,
      { "id-DSCH-RearrangeList-Bearer-RearrangeInd", "nbap.id_DSCH_RearrangeList_Bearer_RearrangeInd",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DSCH_RearrangeList_Bearer_RearrangeInd", HFILL }},
    { &hf_nbap_id_End_Of_Audit_Sequence_Indicator,
      { "id-End-Of-Audit-Sequence-Indicator", "nbap.id_End_Of_Audit_Sequence_Indicator",
        FT_UINT32, BASE_DEC, VALS(nbap_End_Of_Audit_Sequence_Indicator_vals), 0,
        "nbap.End_Of_Audit_Sequence_Indicator", HFILL }},
    { &hf_nbap_id_FACH_Information,
      { "id-FACH-Information", "nbap.id_FACH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Common_TransportChannel_Status_Information", HFILL }},
    { &hf_nbap_id_FACH_ParametersList_CTCH_ReconfRqstTDD,
      { "id-FACH-ParametersList-CTCH-ReconfRqstTDD", "nbap.id_FACH_ParametersList_CTCH_ReconfRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.FACH_ParametersList_CTCH_ReconfRqstTDD", HFILL }},
    { &hf_nbap_id_FACH_ParametersListIE_CTCH_SetupRqstFDD,
      { "id-FACH-ParametersListIE-CTCH-SetupRqstFDD", "nbap.id_FACH_ParametersListIE_CTCH_SetupRqstFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.FACH_ParametersListIE_CTCH_SetupRqstFDD", HFILL }},
    { &hf_nbap_id_FACH_ParametersListIE_CTCH_SetupRqstTDD,
      { "id-FACH-ParametersListIE-CTCH-SetupRqstTDD", "nbap.id_FACH_ParametersListIE_CTCH_SetupRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.FACH_ParametersListIE_CTCH_SetupRqstTDD", HFILL }},
    { &hf_nbap_id_IndicationType_ResourceStatusInd,
      { "id-IndicationType-ResourceStatusInd", "nbap.id_IndicationType_ResourceStatusInd",
        FT_UINT32, BASE_DEC, VALS(nbap_IndicationType_ResourceStatusInd_vals), 0,
        "nbap.IndicationType_ResourceStatusInd", HFILL }},
    { &hf_nbap_id_Local_Cell_ID,
      { "id-Local-Cell-ID", "nbap.id_Local_Cell_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Local_Cell_ID", HFILL }},
    { &hf_nbap_id_Local_Cell_Group_InformationItem_AuditRsp,
      { "id-Local-Cell-Group-InformationItem-AuditRsp", "nbap.id_Local_Cell_Group_InformationItem_AuditRsp",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Local_Cell_Group_InformationItem_AuditRsp", HFILL }},
    { &hf_nbap_id_Local_Cell_Group_InformationItem_ResourceStatusInd,
      { "id-Local-Cell-Group-InformationItem-ResourceStatusInd", "nbap.id_Local_Cell_Group_InformationItem_ResourceStatusInd",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Local_Cell_Group_InformationItem_ResourceStatusInd", HFILL }},
    { &hf_nbap_id_Local_Cell_Group_InformationItem2_ResourceStatusInd,
      { "id-Local-Cell-Group-InformationItem2-ResourceStatusInd", "nbap.id_Local_Cell_Group_InformationItem2_ResourceStatusInd",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Local_Cell_Group_InformationItem2_ResourceStatusInd", HFILL }},
    { &hf_nbap_id_Local_Cell_Group_InformationList_AuditRsp,
      { "id-Local-Cell-Group-InformationList-AuditRsp", "nbap.id_Local_Cell_Group_InformationList_AuditRsp",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Local_Cell_Group_InformationList_AuditRsp", HFILL }},
    { &hf_nbap_id_Local_Cell_InformationItem_AuditRsp,
      { "id-Local-Cell-InformationItem-AuditRsp", "nbap.id_Local_Cell_InformationItem_AuditRsp",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Local_Cell_InformationItem_AuditRsp", HFILL }},
    { &hf_nbap_id_Local_Cell_InformationItem_ResourceStatusInd,
      { "id-Local-Cell-InformationItem-ResourceStatusInd", "nbap.id_Local_Cell_InformationItem_ResourceStatusInd",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Local_Cell_InformationItem_ResourceStatusInd", HFILL }},
    { &hf_nbap_id_Local_Cell_InformationItem2_ResourceStatusInd,
      { "id-Local-Cell-InformationItem2-ResourceStatusInd", "nbap.id_Local_Cell_InformationItem2_ResourceStatusInd",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Local_Cell_InformationItem2_ResourceStatusInd", HFILL }},
    { &hf_nbap_id_Local_Cell_InformationList_AuditRsp,
      { "id-Local-Cell-InformationList-AuditRsp", "nbap.id_Local_Cell_InformationList_AuditRsp",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Local_Cell_InformationList_AuditRsp", HFILL }},
    { &hf_nbap_id_AdjustmentPeriod,
      { "id-AdjustmentPeriod", "nbap.id_AdjustmentPeriod",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.AdjustmentPeriod", HFILL }},
    { &hf_nbap_id_MaxAdjustmentStep,
      { "id-MaxAdjustmentStep", "nbap.id_MaxAdjustmentStep",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.MaxAdjustmentStep", HFILL }},
    { &hf_nbap_id_MaximumTransmissionPower,
      { "id-MaximumTransmissionPower", "nbap.id_MaximumTransmissionPower",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.MaximumTransmissionPower", HFILL }},
    { &hf_nbap_id_MeasurementFilterCoefficient,
      { "id-MeasurementFilterCoefficient", "nbap.id_MeasurementFilterCoefficient",
        FT_UINT32, BASE_DEC, VALS(nbap_MeasurementFilterCoefficient_vals), 0,
        "nbap.MeasurementFilterCoefficient", HFILL }},
    { &hf_nbap_id_MeasurementID,
      { "id-MeasurementID", "nbap.id_MeasurementID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.MeasurementID", HFILL }},
    { &hf_nbap_id_MessageStructure,
      { "id-MessageStructure", "nbap.id_MessageStructure",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.MessageStructure", HFILL }},
    { &hf_nbap_id_MIB_SB_SIB_InformationList_SystemInfoUpdateRqst,
      { "id-MIB-SB-SIB-InformationList-SystemInfoUpdateRqst", "nbap.id_MIB_SB_SIB_InformationList_SystemInfoUpdateRqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.MIB_SB_SIB_InformationList_SystemInfoUpdateRqst", HFILL }},
    { &hf_nbap_id_NodeB_CommunicationContextID,
      { "id-NodeB-CommunicationContextID", "nbap.id_NodeB_CommunicationContextID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.NodeB_CommunicationContextID", HFILL }},
    { &hf_nbap_id_NeighbouringCellMeasurementInformation,
      { "id-NeighbouringCellMeasurementInformation", "nbap.id_NeighbouringCellMeasurementInformation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.NeighbouringCellMeasurementInformation", HFILL }},
    { &hf_nbap_id_P_CCPCH_Information,
      { "id-P-CCPCH-Information", "nbap.id_P_CCPCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Common_PhysicalChannel_Status_Information", HFILL }},
    { &hf_nbap_id_P_CPICH_Information,
      { "id-P-CPICH-Information", "nbap.id_P_CPICH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Common_PhysicalChannel_Status_Information", HFILL }},
    { &hf_nbap_id_P_SCH_Information,
      { "id-P-SCH-Information", "nbap.id_P_SCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Common_PhysicalChannel_Status_Information", HFILL }},
    { &hf_nbap_id_PCCPCH_Information_Cell_ReconfRqstTDD,
      { "id-PCCPCH-Information-Cell-ReconfRqstTDD", "nbap.id_PCCPCH_Information_Cell_ReconfRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PCCPCH_Information_Cell_ReconfRqstTDD", HFILL }},
    { &hf_nbap_id_PCCPCH_Information_Cell_SetupRqstTDD,
      { "id-PCCPCH-Information-Cell-SetupRqstTDD", "nbap.id_PCCPCH_Information_Cell_SetupRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PCCPCH_Information_Cell_SetupRqstTDD", HFILL }},
    { &hf_nbap_id_PCH_Parameters_CTCH_ReconfRqstTDD,
      { "id-PCH-Parameters-CTCH-ReconfRqstTDD", "nbap.id_PCH_Parameters_CTCH_ReconfRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PCH_Parameters_CTCH_ReconfRqstTDD", HFILL }},
    { &hf_nbap_id_PCH_ParametersItem_CTCH_SetupRqstFDD,
      { "id-PCH-ParametersItem-CTCH-SetupRqstFDD", "nbap.id_PCH_ParametersItem_CTCH_SetupRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PCH_ParametersItem_CTCH_SetupRqstFDD", HFILL }},
    { &hf_nbap_id_PCH_ParametersItem_CTCH_SetupRqstTDD,
      { "id-PCH-ParametersItem-CTCH-SetupRqstTDD", "nbap.id_PCH_ParametersItem_CTCH_SetupRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PCH_ParametersItem_CTCH_SetupRqstTDD", HFILL }},
    { &hf_nbap_id_PCH_Information,
      { "id-PCH-Information", "nbap.id_PCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Common_TransportChannel_Status_Information", HFILL }},
    { &hf_nbap_id_PDSCH_Information_AddListIE_PSCH_ReconfRqst,
      { "id-PDSCH-Information-AddListIE-PSCH-ReconfRqst", "nbap.id_PDSCH_Information_AddListIE_PSCH_ReconfRqst",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PDSCH_Information_AddItem_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_id_PDSCH_Information_ModifyListIE_PSCH_ReconfRqst,
      { "id-PDSCH-Information-ModifyListIE-PSCH-ReconfRqst", "nbap.id_PDSCH_Information_ModifyListIE_PSCH_ReconfRqst",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PDSCH_Information_ModifyItem_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_id_PDSCHSets_AddList_PSCH_ReconfRqst,
      { "id-PDSCHSets-AddList-PSCH-ReconfRqst", "nbap.id_PDSCHSets_AddList_PSCH_ReconfRqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.PDSCHSets_AddList_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_id_PDSCHSets_DeleteList_PSCH_ReconfRqst,
      { "id-PDSCHSets-DeleteList-PSCH-ReconfRqst", "nbap.id_PDSCHSets_DeleteList_PSCH_ReconfRqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.PDSCHSets_DeleteList_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_id_PDSCHSets_ModifyList_PSCH_ReconfRqst,
      { "id-PDSCHSets-ModifyList-PSCH-ReconfRqst", "nbap.id_PDSCHSets_ModifyList_PSCH_ReconfRqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.PDSCHSets_ModifyList_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_id_PICH_Information,
      { "id-PICH-Information", "nbap.id_PICH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Common_PhysicalChannel_Status_Information", HFILL }},
    { &hf_nbap_id_PICH_Parameters_CTCH_ReconfRqstTDD,
      { "id-PICH-Parameters-CTCH-ReconfRqstTDD", "nbap.id_PICH_Parameters_CTCH_ReconfRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PICH_Parameters_CTCH_ReconfRqstTDD", HFILL }},
    { &hf_nbap_id_PowerAdjustmentType,
      { "id-PowerAdjustmentType", "nbap.id_PowerAdjustmentType",
        FT_UINT32, BASE_DEC, VALS(nbap_PowerAdjustmentType_vals), 0,
        "nbap.PowerAdjustmentType", HFILL }},
    { &hf_nbap_id_PRACH_Information,
      { "id-PRACH-Information", "nbap.id_PRACH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Common_PhysicalChannel_Status_Information", HFILL }},
    { &hf_nbap_id_PrimaryCCPCH_Information_Cell_ReconfRqstFDD,
      { "id-PrimaryCCPCH-Information-Cell-ReconfRqstFDD", "nbap.id_PrimaryCCPCH_Information_Cell_ReconfRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PrimaryCCPCH_Information_Cell_ReconfRqstFDD", HFILL }},
    { &hf_nbap_id_PrimaryCCPCH_Information_Cell_SetupRqstFDD,
      { "id-PrimaryCCPCH-Information-Cell-SetupRqstFDD", "nbap.id_PrimaryCCPCH_Information_Cell_SetupRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PrimaryCCPCH_Information_Cell_SetupRqstFDD", HFILL }},
    { &hf_nbap_id_PrimaryCPICH_Information_Cell_ReconfRqstFDD,
      { "id-PrimaryCPICH-Information-Cell-ReconfRqstFDD", "nbap.id_PrimaryCPICH_Information_Cell_ReconfRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PrimaryCPICH_Information_Cell_ReconfRqstFDD", HFILL }},
    { &hf_nbap_id_PrimaryCPICH_Information_Cell_SetupRqstFDD,
      { "id-PrimaryCPICH-Information-Cell-SetupRqstFDD", "nbap.id_PrimaryCPICH_Information_Cell_SetupRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PrimaryCPICH_Information_Cell_SetupRqstFDD", HFILL }},
    { &hf_nbap_id_PrimarySCH_Information_Cell_ReconfRqstFDD,
      { "id-PrimarySCH-Information-Cell-ReconfRqstFDD", "nbap.id_PrimarySCH_Information_Cell_ReconfRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PrimarySCH_Information_Cell_ReconfRqstFDD", HFILL }},
    { &hf_nbap_id_PrimarySCH_Information_Cell_SetupRqstFDD,
      { "id-PrimarySCH-Information-Cell-SetupRqstFDD", "nbap.id_PrimarySCH_Information_Cell_SetupRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PrimarySCH_Information_Cell_SetupRqstFDD", HFILL }},
    { &hf_nbap_id_PrimaryScramblingCode,
      { "id-PrimaryScramblingCode", "nbap.id_PrimaryScramblingCode",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.PrimaryScramblingCode", HFILL }},
    { &hf_nbap_id_SCH_Information_Cell_ReconfRqstTDD,
      { "id-SCH-Information-Cell-ReconfRqstTDD", "nbap.id_SCH_Information_Cell_ReconfRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.SCH_Information_Cell_ReconfRqstTDD", HFILL }},
    { &hf_nbap_id_SCH_Information_Cell_SetupRqstTDD,
      { "id-SCH-Information-Cell-SetupRqstTDD", "nbap.id_SCH_Information_Cell_SetupRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.SCH_Information_Cell_SetupRqstTDD", HFILL }},
    { &hf_nbap_id_PUSCH_Information_AddListIE_PSCH_ReconfRqst,
      { "id-PUSCH-Information-AddListIE-PSCH-ReconfRqst", "nbap.id_PUSCH_Information_AddListIE_PSCH_ReconfRqst",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PUSCH_Information_AddItem_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_id_PUSCH_Information_ModifyListIE_PSCH_ReconfRqst,
      { "id-PUSCH-Information-ModifyListIE-PSCH-ReconfRqst", "nbap.id_PUSCH_Information_ModifyListIE_PSCH_ReconfRqst",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PUSCH_Information_ModifyItem_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_id_PUSCHSets_AddList_PSCH_ReconfRqst,
      { "id-PUSCHSets-AddList-PSCH-ReconfRqst", "nbap.id_PUSCHSets_AddList_PSCH_ReconfRqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.PUSCHSets_AddList_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_id_PUSCHSets_DeleteList_PSCH_ReconfRqst,
      { "id-PUSCHSets-DeleteList-PSCH-ReconfRqst", "nbap.id_PUSCHSets_DeleteList_PSCH_ReconfRqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.PUSCHSets_DeleteList_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_id_PUSCHSets_ModifyList_PSCH_ReconfRqst,
      { "id-PUSCHSets-ModifyList-PSCH-ReconfRqst", "nbap.id_PUSCHSets_ModifyList_PSCH_ReconfRqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.PUSCHSets_ModifyList_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_id_RACH_Information,
      { "id-RACH-Information", "nbap.id_RACH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Common_TransportChannel_Status_Information", HFILL }},
    { &hf_nbap_id_RACH_ParametersItem_CTCH_SetupRqstFDD,
      { "id-RACH-ParametersItem-CTCH-SetupRqstFDD", "nbap.id_RACH_ParametersItem_CTCH_SetupRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RACH_ParametersItem_CTCH_SetupRqstFDD", HFILL }},
    { &hf_nbap_id_RACH_ParameterItem_CTCH_SetupRqstTDD,
      { "id-RACH-ParameterItem-CTCH-SetupRqstTDD", "nbap.id_RACH_ParameterItem_CTCH_SetupRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RACH_ParameterItem_CTCH_SetupRqstTDD", HFILL }},
    { &hf_nbap_id_ReportCharacteristics,
      { "id-ReportCharacteristics", "nbap.id_ReportCharacteristics",
        FT_UINT32, BASE_DEC, VALS(nbap_ReportCharacteristics_vals), 0,
        "nbap.ReportCharacteristics", HFILL }},
    { &hf_nbap_id_Reporting_Object_RL_FailureInd,
      { "id-Reporting-Object-RL-FailureInd", "nbap.id_Reporting_Object_RL_FailureInd",
        FT_UINT32, BASE_DEC, VALS(nbap_Reporting_Object_RL_FailureInd_vals), 0,
        "nbap.Reporting_Object_RL_FailureInd", HFILL }},
    { &hf_nbap_id_Reporting_Object_RL_RestoreInd,
      { "id-Reporting-Object-RL-RestoreInd", "nbap.id_Reporting_Object_RL_RestoreInd",
        FT_UINT32, BASE_DEC, VALS(nbap_Reporting_Object_RL_RestoreInd_vals), 0,
        "nbap.Reporting_Object_RL_RestoreInd", HFILL }},
    { &hf_nbap_id_RL_InformationItem_DM_Rprt,
      { "id-RL-InformationItem-DM-Rprt", "nbap.id_RL_InformationItem_DM_Rprt",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_InformationItem_DM_Rprt", HFILL }},
    { &hf_nbap_id_RL_InformationItem_DM_Rqst,
      { "id-RL-InformationItem-DM-Rqst", "nbap.id_RL_InformationItem_DM_Rqst",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_InformationItem_DM_Rqst", HFILL }},
    { &hf_nbap_id_RL_InformationItem_DM_Rsp,
      { "id-RL-InformationItem-DM-Rsp", "nbap.id_RL_InformationItem_DM_Rsp",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_InformationItem_DM_Rsp", HFILL }},
    { &hf_nbap_id_RL_InformationItem_RL_AdditionRqstFDD,
      { "id-RL-InformationItem-RL-AdditionRqstFDD", "nbap.id_RL_InformationItem_RL_AdditionRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_InformationItem_RL_AdditionRqstFDD", HFILL }},
    { &hf_nbap_id_RL_informationItem_RL_DeletionRqst,
      { "id-RL-informationItem-RL-DeletionRqst", "nbap.id_RL_informationItem_RL_DeletionRqst",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_informationItem_RL_DeletionRqst", HFILL }},
    { &hf_nbap_id_RL_InformationItem_RL_FailureInd,
      { "id-RL-InformationItem-RL-FailureInd", "nbap.id_RL_InformationItem_RL_FailureInd",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_InformationItem_RL_FailureInd", HFILL }},
    { &hf_nbap_id_RL_InformationItem_RL_PreemptRequiredInd,
      { "id-RL-InformationItem-RL-PreemptRequiredInd", "nbap.id_RL_InformationItem_RL_PreemptRequiredInd",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_InformationItem_RL_PreemptRequiredInd", HFILL }},
    { &hf_nbap_id_RL_InformationItem_RL_ReconfPrepFDD,
      { "id-RL-InformationItem-RL-ReconfPrepFDD", "nbap.id_RL_InformationItem_RL_ReconfPrepFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_InformationItem_RL_ReconfPrepFDD", HFILL }},
    { &hf_nbap_id_RL_InformationItem_RL_ReconfRqstFDD,
      { "id-RL-InformationItem-RL-ReconfRqstFDD", "nbap.id_RL_InformationItem_RL_ReconfRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_InformationItem_RL_ReconfPrepFDD", HFILL }},
    { &hf_nbap_id_RL_InformationItem_RL_RestoreInd,
      { "id-RL-InformationItem-RL-RestoreInd", "nbap.id_RL_InformationItem_RL_RestoreInd",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_InformationItem_RL_RestoreInd", HFILL }},
    { &hf_nbap_id_RL_InformationItem_RL_SetupRqstFDD,
      { "id-RL-InformationItem-RL-SetupRqstFDD", "nbap.id_RL_InformationItem_RL_SetupRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_InformationItem_RL_SetupRqstFDD", HFILL }},
    { &hf_nbap_id_RL_InformationList_RL_AdditionRqstFDD,
      { "id-RL-InformationList-RL-AdditionRqstFDD", "nbap.id_RL_InformationList_RL_AdditionRqstFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.RL_InformationList_RL_AdditionRqstFDD", HFILL }},
    { &hf_nbap_id_RL_informationList_RL_DeletionRqst,
      { "id-RL-informationList-RL-DeletionRqst", "nbap.id_RL_informationList_RL_DeletionRqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.RL_informationList_RL_DeletionRqst", HFILL }},
    { &hf_nbap_id_RL_InformationList_RL_PreemptRequiredInd,
      { "id-RL-InformationList-RL-PreemptRequiredInd", "nbap.id_RL_InformationList_RL_PreemptRequiredInd",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.RL_InformationList_RL_PreemptRequiredInd", HFILL }},
    { &hf_nbap_id_RL_InformationList_RL_ReconfPrepFDD,
      { "id-RL-InformationList-RL-ReconfPrepFDD", "nbap.id_RL_InformationList_RL_ReconfPrepFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.RL_InformationList_RL_ReconfPrepFDD", HFILL }},
    { &hf_nbap_id_RL_InformationList_RL_ReconfRqstFDD,
      { "id-RL-InformationList-RL-ReconfRqstFDD", "nbap.id_RL_InformationList_RL_ReconfRqstFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.RL_InformationList_RL_ReconfRqstFDD", HFILL }},
    { &hf_nbap_id_RL_InformationList_RL_SetupRqstFDD,
      { "id-RL-InformationList-RL-SetupRqstFDD", "nbap.id_RL_InformationList_RL_SetupRqstFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.RL_InformationList_RL_SetupRqstFDD", HFILL }},
    { &hf_nbap_id_RL_InformationResponseItem_RL_AdditionRspFDD,
      { "id-RL-InformationResponseItem-RL-AdditionRspFDD", "nbap.id_RL_InformationResponseItem_RL_AdditionRspFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_InformationResponseItem_RL_AdditionRspFDD", HFILL }},
    { &hf_nbap_id_RL_InformationResponseItem_RL_ReconfReady,
      { "id-RL-InformationResponseItem-RL-ReconfReady", "nbap.id_RL_InformationResponseItem_RL_ReconfReady",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_InformationResponseItem_RL_ReconfReady", HFILL }},
    { &hf_nbap_id_RL_InformationResponseItem_RL_ReconfRsp,
      { "id-RL-InformationResponseItem-RL-ReconfRsp", "nbap.id_RL_InformationResponseItem_RL_ReconfRsp",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_InformationResponseItem_RL_ReconfRsp", HFILL }},
    { &hf_nbap_id_RL_InformationResponseItem_RL_SetupRspFDD,
      { "id-RL-InformationResponseItem-RL-SetupRspFDD", "nbap.id_RL_InformationResponseItem_RL_SetupRspFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_InformationResponseItem_RL_SetupRspFDD", HFILL }},
    { &hf_nbap_id_RL_InformationResponseList_RL_AdditionRspFDD,
      { "id-RL-InformationResponseList-RL-AdditionRspFDD", "nbap.id_RL_InformationResponseList_RL_AdditionRspFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.RL_InformationResponseList_RL_AdditionRspFDD", HFILL }},
    { &hf_nbap_id_RL_InformationResponseList_RL_ReconfReady,
      { "id-RL-InformationResponseList-RL-ReconfReady", "nbap.id_RL_InformationResponseList_RL_ReconfReady",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.RL_InformationResponseList_RL_ReconfReady", HFILL }},
    { &hf_nbap_id_RL_InformationResponseList_RL_ReconfRsp,
      { "id-RL-InformationResponseList-RL-ReconfRsp", "nbap.id_RL_InformationResponseList_RL_ReconfRsp",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.RL_InformationResponseList_RL_ReconfRsp", HFILL }},
    { &hf_nbap_id_RL_InformationResponseList_RL_SetupRspFDD,
      { "id-RL-InformationResponseList-RL-SetupRspFDD", "nbap.id_RL_InformationResponseList_RL_SetupRspFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.RL_InformationResponseList_RL_SetupRspFDD", HFILL }},
    { &hf_nbap_id_RL_InformationResponse_RL_AdditionRspTDD,
      { "id-RL-InformationResponse-RL-AdditionRspTDD", "nbap.id_RL_InformationResponse_RL_AdditionRspTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_InformationResponse_RL_AdditionRspTDD", HFILL }},
    { &hf_nbap_id_RL_InformationResponse_RL_SetupRspTDD,
      { "id-RL-InformationResponse-RL-SetupRspTDD", "nbap.id_RL_InformationResponse_RL_SetupRspTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_InformationResponse_RL_SetupRspTDD", HFILL }},
    { &hf_nbap_id_RL_Information_RL_AdditionRqstTDD,
      { "id-RL-Information-RL-AdditionRqstTDD", "nbap.id_RL_Information_RL_AdditionRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_Information_RL_AdditionRqstTDD", HFILL }},
    { &hf_nbap_id_RL_Information_RL_ReconfRqstTDD,
      { "id-RL-Information-RL-ReconfRqstTDD", "nbap.id_RL_Information_RL_ReconfRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_Information_RL_ReconfRqstTDD", HFILL }},
    { &hf_nbap_id_RL_Information_RL_ReconfPrepTDD,
      { "id-RL-Information-RL-ReconfPrepTDD", "nbap.id_RL_Information_RL_ReconfPrepTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_Information_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_id_RL_Information_RL_SetupRqstTDD,
      { "id-RL-Information-RL-SetupRqstTDD", "nbap.id_RL_Information_RL_SetupRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_Information_RL_SetupRqstTDD", HFILL }},
    { &hf_nbap_id_RL_ReconfigurationFailureItem_RL_ReconfFailure,
      { "id-RL-ReconfigurationFailureItem-RL-ReconfFailure", "nbap.id_RL_ReconfigurationFailureItem_RL_ReconfFailure",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_ReconfigurationFailureItem_RL_ReconfFailure", HFILL }},
    { &hf_nbap_id_RL_Set_InformationItem_DM_Rprt,
      { "id-RL-Set-InformationItem-DM-Rprt", "nbap.id_RL_Set_InformationItem_DM_Rprt",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_Set_InformationItem_DM_Rprt", HFILL }},
    { &hf_nbap_id_RL_Set_InformationItem_DM_Rsp,
      { "id-RL-Set-InformationItem-DM-Rsp", "nbap.id_RL_Set_InformationItem_DM_Rsp",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_Set_InformationItem_DM_Rsp", HFILL }},
    { &hf_nbap_id_RL_Set_InformationItem_RL_FailureInd,
      { "id-RL-Set-InformationItem-RL-FailureInd", "nbap.id_RL_Set_InformationItem_RL_FailureInd",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_Set_InformationItem_RL_FailureInd", HFILL }},
    { &hf_nbap_id_RL_Set_InformationItem_RL_RestoreInd,
      { "id-RL-Set-InformationItem-RL-RestoreInd", "nbap.id_RL_Set_InformationItem_RL_RestoreInd",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_Set_InformationItem_RL_RestoreInd", HFILL }},
    { &hf_nbap_id_S_CCPCH_Information,
      { "id-S-CCPCH-Information", "nbap.id_S_CCPCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Common_PhysicalChannel_Status_Information", HFILL }},
    { &hf_nbap_id_S_CPICH_Information,
      { "id-S-CPICH-Information", "nbap.id_S_CPICH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Common_PhysicalChannel_Status_Information", HFILL }},
    { &hf_nbap_id_SCH_Information,
      { "id-SCH-Information", "nbap.id_SCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Common_PhysicalChannel_Status_Information", HFILL }},
    { &hf_nbap_id_S_SCH_Information,
      { "id-S-SCH-Information", "nbap.id_S_SCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Common_PhysicalChannel_Status_Information", HFILL }},
    { &hf_nbap_id_Secondary_CCPCHListIE_CTCH_ReconfRqstTDD,
      { "id-Secondary-CCPCHListIE-CTCH-ReconfRqstTDD", "nbap.id_Secondary_CCPCHListIE_CTCH_ReconfRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Secondary_CCPCHListIE_CTCH_ReconfRqstTDD", HFILL }},
    { &hf_nbap_id_Secondary_CCPCH_parameterListIE_CTCH_SetupRqstTDD,
      { "id-Secondary-CCPCH-parameterListIE-CTCH-SetupRqstTDD", "nbap.id_Secondary_CCPCH_parameterListIE_CTCH_SetupRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Secondary_CCPCH_parameterListIE_CTCH_SetupRqstTDD", HFILL }},
    { &hf_nbap_id_Secondary_CCPCH_Parameters_CTCH_ReconfRqstTDD,
      { "id-Secondary-CCPCH-Parameters-CTCH-ReconfRqstTDD", "nbap.id_Secondary_CCPCH_Parameters_CTCH_ReconfRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Secondary_CCPCH_Parameters_CTCH_ReconfRqstTDD", HFILL }},
    { &hf_nbap_id_SecondaryCPICH_InformationItem_Cell_ReconfRqstFDD,
      { "id-SecondaryCPICH-InformationItem-Cell-ReconfRqstFDD", "nbap.id_SecondaryCPICH_InformationItem_Cell_ReconfRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.SecondaryCPICH_InformationItem_Cell_ReconfRqstFDD", HFILL }},
    { &hf_nbap_id_SecondaryCPICH_InformationItem_Cell_SetupRqstFDD,
      { "id-SecondaryCPICH-InformationItem-Cell-SetupRqstFDD", "nbap.id_SecondaryCPICH_InformationItem_Cell_SetupRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.SecondaryCPICH_InformationItem_Cell_SetupRqstFDD", HFILL }},
    { &hf_nbap_id_SecondaryCPICH_InformationList_Cell_ReconfRqstFDD,
      { "id-SecondaryCPICH-InformationList-Cell-ReconfRqstFDD", "nbap.id_SecondaryCPICH_InformationList_Cell_ReconfRqstFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SecondaryCPICH_InformationList_Cell_ReconfRqstFDD", HFILL }},
    { &hf_nbap_id_SecondaryCPICH_InformationList_Cell_SetupRqstFDD,
      { "id-SecondaryCPICH-InformationList-Cell-SetupRqstFDD", "nbap.id_SecondaryCPICH_InformationList_Cell_SetupRqstFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SecondaryCPICH_InformationList_Cell_SetupRqstFDD", HFILL }},
    { &hf_nbap_id_SecondarySCH_Information_Cell_ReconfRqstFDD,
      { "id-SecondarySCH-Information-Cell-ReconfRqstFDD", "nbap.id_SecondarySCH_Information_Cell_ReconfRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.SecondarySCH_Information_Cell_ReconfRqstFDD", HFILL }},
    { &hf_nbap_id_SecondarySCH_Information_Cell_SetupRqstFDD,
      { "id-SecondarySCH-Information-Cell-SetupRqstFDD", "nbap.id_SecondarySCH_Information_Cell_SetupRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.SecondarySCH_Information_Cell_SetupRqstFDD", HFILL }},
    { &hf_nbap_id_SegmentInformationListIE_SystemInfoUpdate,
      { "id-SegmentInformationListIE-SystemInfoUpdate", "nbap.id_SegmentInformationListIE_SystemInfoUpdate",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SegmentInformationListIE_SystemInfoUpdate", HFILL }},
    { &hf_nbap_id_SFN,
      { "id-SFN", "nbap.id_SFN",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SFN", HFILL }},
    { &hf_nbap_id_SignallingBearerRequestIndicator,
      { "id-SignallingBearerRequestIndicator", "nbap.id_SignallingBearerRequestIndicator",
        FT_UINT32, BASE_DEC, VALS(nbap_SignallingBearerRequestIndicator_vals), 0,
        "nbap.SignallingBearerRequestIndicator", HFILL }},
    { &hf_nbap_id_ShutdownTimer,
      { "id-ShutdownTimer", "nbap.id_ShutdownTimer",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.ShutdownTimer", HFILL }},
    { &hf_nbap_id_Start_Of_Audit_Sequence_Indicator,
      { "id-Start-Of-Audit-Sequence-Indicator", "nbap.id_Start_Of_Audit_Sequence_Indicator",
        FT_UINT32, BASE_DEC, VALS(nbap_Start_Of_Audit_Sequence_Indicator_vals), 0,
        "nbap.Start_Of_Audit_Sequence_Indicator", HFILL }},
    { &hf_nbap_id_Successful_RL_InformationRespItem_RL_AdditionFailureFDD,
      { "id-Successful-RL-InformationRespItem-RL-AdditionFailureFDD", "nbap.id_Successful_RL_InformationRespItem_RL_AdditionFailureFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Successful_RL_InformationRespItem_RL_AdditionFailureFDD", HFILL }},
    { &hf_nbap_id_Successful_RL_InformationRespItem_RL_SetupFailureFDD,
      { "id-Successful-RL-InformationRespItem-RL-SetupFailureFDD", "nbap.id_Successful_RL_InformationRespItem_RL_SetupFailureFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Successful_RL_InformationRespItem_RL_SetupFailureFDD", HFILL }},
    { &hf_nbap_id_SyncCase,
      { "id-SyncCase", "nbap.id_SyncCase",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SyncCase", HFILL }},
    { &hf_nbap_id_SyncCaseIndicatorItem_Cell_SetupRqstTDD_PSCH,
      { "id-SyncCaseIndicatorItem-Cell-SetupRqstTDD-PSCH", "nbap.id_SyncCaseIndicatorItem_Cell_SetupRqstTDD_PSCH",
        FT_UINT32, BASE_DEC, VALS(nbap_SyncCaseIndicatorItem_Cell_SetupRqstTDD_PSCH_vals), 0,
        "nbap.SyncCaseIndicatorItem_Cell_SetupRqstTDD_PSCH", HFILL }},
    { &hf_nbap_id_T_Cell,
      { "id-T-Cell", "nbap.id_T_Cell",
        FT_UINT32, BASE_DEC, VALS(nbap_T_Cell_vals), 0,
        "nbap.T_Cell", HFILL }},
    { &hf_nbap_id_TargetCommunicationControlPortID,
      { "id-TargetCommunicationControlPortID", "nbap.id_TargetCommunicationControlPortID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CommunicationControlPortID", HFILL }},
    { &hf_nbap_id_TimeSlotConfigurationList_Cell_ReconfRqstTDD,
      { "id-TimeSlotConfigurationList-Cell-ReconfRqstTDD", "nbap.id_TimeSlotConfigurationList_Cell_ReconfRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.TimeSlotConfigurationList_Cell_ReconfRqstTDD", HFILL }},
    { &hf_nbap_id_TimeSlotConfigurationList_Cell_SetupRqstTDD,
      { "id-TimeSlotConfigurationList-Cell-SetupRqstTDD", "nbap.id_TimeSlotConfigurationList_Cell_SetupRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.TimeSlotConfigurationList_Cell_SetupRqstTDD", HFILL }},
    { &hf_nbap_id_TransmissionDiversityApplied,
      { "id-TransmissionDiversityApplied", "nbap.id_TransmissionDiversityApplied",
        FT_BOOLEAN, 8, NULL, 0,
        "nbap.TransmissionDiversityApplied", HFILL }},
    { &hf_nbap_id_TypeOfError,
      { "id-TypeOfError", "nbap.id_TypeOfError",
        FT_UINT32, BASE_DEC, VALS(nbap_TypeOfError_vals), 0,
        "nbap.TypeOfError", HFILL }},
    { &hf_nbap_id_UARFCNforNt,
      { "id-UARFCNforNt", "nbap.id_UARFCNforNt",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.UARFCN", HFILL }},
    { &hf_nbap_id_UARFCNforNd,
      { "id-UARFCNforNd", "nbap.id_UARFCNforNd",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.UARFCN", HFILL }},
    { &hf_nbap_id_UARFCNforNu,
      { "id-UARFCNforNu", "nbap.id_UARFCNforNu",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.UARFCN", HFILL }},
    { &hf_nbap_id_UL_CCTrCH_InformationItem_RL_SetupRqstTDD,
      { "id-UL-CCTrCH-InformationItem-RL-SetupRqstTDD", "nbap.id_UL_CCTrCH_InformationItem_RL_SetupRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_CCTrCH_InformationItem_RL_SetupRqstTDD", HFILL }},
    { &hf_nbap_id_UL_CCTrCH_InformationList_RL_AdditionRqstTDD,
      { "id-UL-CCTrCH-InformationList-RL-AdditionRqstTDD", "nbap.id_UL_CCTrCH_InformationList_RL_AdditionRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.UL_CCTrCH_InformationList_RL_SetupRqstTDD", HFILL }},
    { &hf_nbap_id_UL_CCTrCH_InformationList_RL_SetupRqstTDD,
      { "id-UL-CCTrCH-InformationList-RL-SetupRqstTDD", "nbap.id_UL_CCTrCH_InformationList_RL_SetupRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.UL_CCTrCH_InformationList_RL_SetupRqstTDD", HFILL }},
    { &hf_nbap_id_UL_DPCH_InformationItem_RL_AdditionRqstTDD,
      { "id-UL-DPCH-InformationItem-RL-AdditionRqstTDD", "nbap.id_UL_DPCH_InformationItem_RL_AdditionRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_DPCH_InformationItem_RL_AdditionRqstTDD", HFILL }},
    { &hf_nbap_id_UL_DPCH_InformationList_RL_SetupRqstTDD,
      { "id-UL-DPCH-InformationList-RL-SetupRqstTDD", "nbap.id_UL_DPCH_InformationList_RL_SetupRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_DPCH_InformationItem_RL_SetupRqstTDD", HFILL }},
    { &hf_nbap_id_UL_DPCH_Information_RL_ReconfPrepFDD,
      { "id-UL-DPCH-Information-RL-ReconfPrepFDD", "nbap.id_UL_DPCH_Information_RL_ReconfPrepFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_DPCH_Information_RL_ReconfPrepFDD", HFILL }},
    { &hf_nbap_id_UL_DPCH_Information_RL_ReconfRqstFDD,
      { "id-UL-DPCH-Information-RL-ReconfRqstFDD", "nbap.id_UL_DPCH_Information_RL_ReconfRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_DPCH_Information_RL_ReconfRqstFDD", HFILL }},
    { &hf_nbap_id_UL_DPCH_Information_RL_SetupRqstFDD,
      { "id-UL-DPCH-Information-RL-SetupRqstFDD", "nbap.id_UL_DPCH_Information_RL_SetupRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_DPCH_Information_RL_SetupRqstFDD", HFILL }},
    { &hf_nbap_id_Unsuccessful_RL_InformationRespItem_RL_AdditionFailureFDD,
      { "id-Unsuccessful-RL-InformationRespItem-RL-AdditionFailureFDD", "nbap.id_Unsuccessful_RL_InformationRespItem_RL_AdditionFailureFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Unsuccessful_RL_InformationRespItem_RL_AdditionFailureFDD", HFILL }},
    { &hf_nbap_id_Unsuccessful_RL_InformationRespItem_RL_SetupFailureFDD,
      { "id-Unsuccessful-RL-InformationRespItem-RL-SetupFailureFDD", "nbap.id_Unsuccessful_RL_InformationRespItem_RL_SetupFailureFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Unsuccessful_RL_InformationRespItem_RL_SetupFailureFDD", HFILL }},
    { &hf_nbap_id_Unsuccessful_RL_InformationResp_RL_AdditionFailureTDD,
      { "id-Unsuccessful-RL-InformationResp-RL-AdditionFailureTDD", "nbap.id_Unsuccessful_RL_InformationResp_RL_AdditionFailureTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Unsuccessful_RL_InformationResp_RL_AdditionFailureTDD", HFILL }},
    { &hf_nbap_id_Unsuccessful_RL_InformationResp_RL_SetupFailureTDD,
      { "id-Unsuccessful-RL-InformationResp-RL-SetupFailureTDD", "nbap.id_Unsuccessful_RL_InformationResp_RL_SetupFailureTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Unsuccessful_RL_InformationResp_RL_SetupFailureTDD", HFILL }},
    { &hf_nbap_id_USCH_Information_Add,
      { "id-USCH-Information-Add", "nbap.id_USCH_Information_Add",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.USCH_Information", HFILL }},
    { &hf_nbap_id_USCH_Information_DeleteList_RL_ReconfPrepTDD,
      { "id-USCH-Information-DeleteList-RL-ReconfPrepTDD", "nbap.id_USCH_Information_DeleteList_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.USCH_Information_DeleteList_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_id_USCH_Information_ModifyList_RL_ReconfPrepTDD,
      { "id-USCH-Information-ModifyList-RL-ReconfPrepTDD", "nbap.id_USCH_Information_ModifyList_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.USCH_Information_ModifyList_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_id_USCH_InformationResponse,
      { "id-USCH-InformationResponse", "nbap.id_USCH_InformationResponse",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.USCH_InformationResponse", HFILL }},
    { &hf_nbap_id_USCH_Information,
      { "id-USCH-Information", "nbap.id_USCH_Information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.USCH_Information", HFILL }},
    { &hf_nbap_id_USCH_RearrangeList_Bearer_RearrangeInd,
      { "id-USCH-RearrangeList-Bearer-RearrangeInd", "nbap.id_USCH_RearrangeList_Bearer_RearrangeInd",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.USCH_RearrangeList_Bearer_RearrangeInd", HFILL }},
    { &hf_nbap_id_Active_Pattern_Sequence_Information,
      { "id-Active-Pattern-Sequence-Information", "nbap.id_Active_Pattern_Sequence_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Active_Pattern_Sequence_Information", HFILL }},
    { &hf_nbap_id_AICH_ParametersListIE_CTCH_ReconfRqstFDD,
      { "id-AICH-ParametersListIE-CTCH-ReconfRqstFDD", "nbap.id_AICH_ParametersListIE_CTCH_ReconfRqstFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.AICH_ParametersListIE_CTCH_ReconfRqstFDD", HFILL }},
    { &hf_nbap_id_AdjustmentRatio,
      { "id-AdjustmentRatio", "nbap.id_AdjustmentRatio",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.ScaledAdjustmentRatio", HFILL }},
    { &hf_nbap_id_FACH_ParametersListIE_CTCH_ReconfRqstFDD,
      { "id-FACH-ParametersListIE-CTCH-ReconfRqstFDD", "nbap.id_FACH_ParametersListIE_CTCH_ReconfRqstFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.FACH_ParametersListIE_CTCH_ReconfRqstFDD", HFILL }},
    { &hf_nbap_id_CauseLevel_PSCH_ReconfFailure,
      { "id-CauseLevel-PSCH-ReconfFailure", "nbap.id_CauseLevel_PSCH_ReconfFailure",
        FT_UINT32, BASE_DEC, VALS(nbap_CauseLevel_PSCH_ReconfFailure_vals), 0,
        "nbap.CauseLevel_PSCH_ReconfFailure", HFILL }},
    { &hf_nbap_id_CauseLevel_RL_AdditionFailureFDD,
      { "id-CauseLevel-RL-AdditionFailureFDD", "nbap.id_CauseLevel_RL_AdditionFailureFDD",
        FT_UINT32, BASE_DEC, VALS(nbap_CauseLevel_RL_AdditionFailureFDD_vals), 0,
        "nbap.CauseLevel_RL_AdditionFailureFDD", HFILL }},
    { &hf_nbap_id_CauseLevel_RL_AdditionFailureTDD,
      { "id-CauseLevel-RL-AdditionFailureTDD", "nbap.id_CauseLevel_RL_AdditionFailureTDD",
        FT_UINT32, BASE_DEC, VALS(nbap_CauseLevel_RL_AdditionFailureTDD_vals), 0,
        "nbap.CauseLevel_RL_AdditionFailureTDD", HFILL }},
    { &hf_nbap_id_CauseLevel_RL_ReconfFailure,
      { "id-CauseLevel-RL-ReconfFailure", "nbap.id_CauseLevel_RL_ReconfFailure",
        FT_UINT32, BASE_DEC, VALS(nbap_CauseLevel_RL_ReconfFailure_vals), 0,
        "nbap.CauseLevel_RL_ReconfFailure", HFILL }},
    { &hf_nbap_id_CauseLevel_RL_SetupFailureFDD,
      { "id-CauseLevel-RL-SetupFailureFDD", "nbap.id_CauseLevel_RL_SetupFailureFDD",
        FT_UINT32, BASE_DEC, VALS(nbap_CauseLevel_RL_SetupFailureFDD_vals), 0,
        "nbap.CauseLevel_RL_SetupFailureFDD", HFILL }},
    { &hf_nbap_id_CauseLevel_RL_SetupFailureTDD,
      { "id-CauseLevel-RL-SetupFailureTDD", "nbap.id_CauseLevel_RL_SetupFailureTDD",
        FT_UINT32, BASE_DEC, VALS(nbap_CauseLevel_RL_SetupFailureTDD_vals), 0,
        "nbap.CauseLevel_RL_SetupFailureTDD", HFILL }},
    { &hf_nbap_id_Closed_Loop_Timing_Adjustment_Mode,
      { "id-Closed-Loop-Timing-Adjustment-Mode", "nbap.id_Closed_Loop_Timing_Adjustment_Mode",
        FT_UINT32, BASE_DEC, VALS(nbap_Closedlooptimingadjustmentmode_vals), 0,
        "nbap.Closedlooptimingadjustmentmode", HFILL }},
    { &hf_nbap_id_CommonPhysicalChannelType_CTCH_ReconfRqstFDD,
      { "id-CommonPhysicalChannelType-CTCH-ReconfRqstFDD", "nbap.id_CommonPhysicalChannelType_CTCH_ReconfRqstFDD",
        FT_UINT32, BASE_DEC, VALS(nbap_CommonPhysicalChannelType_CTCH_ReconfRqstFDD_vals), 0,
        "nbap.CommonPhysicalChannelType_CTCH_ReconfRqstFDD", HFILL }},
    { &hf_nbap_id_Compressed_Mode_Deactivation_Flag,
      { "id-Compressed-Mode-Deactivation-Flag", "nbap.id_Compressed_Mode_Deactivation_Flag",
        FT_UINT32, BASE_DEC, VALS(nbap_Compressed_Mode_Deactivation_Flag_vals), 0,
        "nbap.Compressed_Mode_Deactivation_Flag", HFILL }},
    { &hf_nbap_id_DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD,
      { "id-DL-CCTrCH-InformationAddList-RL-ReconfPrepTDD", "nbap.id_DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_id_DL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD,
      { "id-DL-CCTrCH-InformationDeleteItem-RL-ReconfRqstTDD", "nbap.id_DL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD", HFILL }},
    { &hf_nbap_id_DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD,
      { "id-DL-CCTrCH-InformationDeleteList-RL-ReconfPrepTDD", "nbap.id_DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_id_DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD,
      { "id-DL-CCTrCH-InformationDeleteList-RL-ReconfRqstTDD", "nbap.id_DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD", HFILL }},
    { &hf_nbap_id_DL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD,
      { "id-DL-CCTrCH-InformationModifyItem-RL-ReconfRqstTDD", "nbap.id_DL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD", HFILL }},
    { &hf_nbap_id_DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD,
      { "id-DL-CCTrCH-InformationModifyList-RL-ReconfPrepTDD", "nbap.id_DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_id_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD,
      { "id-DL-CCTrCH-InformationModifyList-RL-ReconfRqstTDD", "nbap.id_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD", HFILL }},
    { &hf_nbap_id_DL_DPCH_InformationAddListIE_RL_ReconfPrepTDD,
      { "id-DL-DPCH-InformationAddListIE-RL-ReconfPrepTDD", "nbap.id_DL_DPCH_InformationAddListIE_RL_ReconfPrepTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_DPCH_InformationAddItem_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_id_DL_DPCH_InformationModify_AddListIE_RL_ReconfPrepTDD,
      { "id-DL-DPCH-InformationModify-AddListIE-RL-ReconfPrepTDD", "nbap.id_DL_DPCH_InformationModify_AddListIE_RL_ReconfPrepTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_DPCH_InformationModify_AddItem_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_id_DL_DPCH_InformationModify_DeleteListIE_RL_ReconfPrepTDD,
      { "id-DL-DPCH-InformationModify-DeleteListIE-RL-ReconfPrepTDD", "nbap.id_DL_DPCH_InformationModify_DeleteListIE_RL_ReconfPrepTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_DPCH_InformationModify_DeleteItem_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_id_DL_DPCH_InformationModify_ModifyListIE_RL_ReconfPrepTDD,
      { "id-DL-DPCH-InformationModify-ModifyListIE-RL-ReconfPrepTDD", "nbap.id_DL_DPCH_InformationModify_ModifyListIE_RL_ReconfPrepTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_DPCH_InformationModify_ModifyItem_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_id_DL_TPC_Pattern01Count,
      { "id-DL-TPC-Pattern01Count", "nbap.id_DL_TPC_Pattern01Count",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_TPC_Pattern01Count", HFILL }},
    { &hf_nbap_id_DPC_Mode,
      { "id-DPC-Mode", "nbap.id_DPC_Mode",
        FT_UINT32, BASE_DEC, VALS(nbap_DPC_Mode_vals), 0,
        "nbap.DPC_Mode", HFILL }},
    { &hf_nbap_id_DPCHConstant,
      { "id-DPCHConstant", "nbap.id_DPCHConstant",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.ConstantValue", HFILL }},
    { &hf_nbap_id_FACH_ParametersList_CTCH_SetupRsp,
      { "id-FACH-ParametersList-CTCH-SetupRsp", "nbap.id_FACH_ParametersList_CTCH_SetupRsp",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.FACH_CommonTransportChannel_InformationResponse", HFILL }},
    { &hf_nbap_id_Limited_power_increase_information_Cell_SetupRqstFDD,
      { "id-Limited-power-increase-information-Cell-SetupRqstFDD", "nbap.id_Limited_power_increase_information_Cell_SetupRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Limited_power_increase_information_Cell_SetupRqstFDD", HFILL }},
    { &hf_nbap_id_PCH_Parameters_CTCH_SetupRsp,
      { "id-PCH-Parameters-CTCH-SetupRsp", "nbap.id_PCH_Parameters_CTCH_SetupRsp",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CommonTransportChannel_InformationResponse", HFILL }},
    { &hf_nbap_id_PCH_ParametersItem_CTCH_ReconfRqstFDD,
      { "id-PCH-ParametersItem-CTCH-ReconfRqstFDD", "nbap.id_PCH_ParametersItem_CTCH_ReconfRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PCH_ParametersItem_CTCH_ReconfRqstFDD", HFILL }},
    { &hf_nbap_id_PICH_ParametersItem_CTCH_ReconfRqstFDD,
      { "id-PICH-ParametersItem-CTCH-ReconfRqstFDD", "nbap.id_PICH_ParametersItem_CTCH_ReconfRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PICH_ParametersItem_CTCH_ReconfRqstFDD", HFILL }},
    { &hf_nbap_id_PRACHConstant,
      { "id-PRACHConstant", "nbap.id_PRACHConstant",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.ConstantValue", HFILL }},
    { &hf_nbap_id_PRACH_ParametersListIE_CTCH_ReconfRqstFDD,
      { "id-PRACH-ParametersListIE-CTCH-ReconfRqstFDD", "nbap.id_PRACH_ParametersListIE_CTCH_ReconfRqstFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.PRACH_ParametersListIE_CTCH_ReconfRqstFDD", HFILL }},
    { &hf_nbap_id_PUSCHConstant,
      { "id-PUSCHConstant", "nbap.id_PUSCHConstant",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.ConstantValue", HFILL }},
    { &hf_nbap_id_RACH_Parameters_CTCH_SetupRsp,
      { "id-RACH-Parameters-CTCH-SetupRsp", "nbap.id_RACH_Parameters_CTCH_SetupRsp",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CommonTransportChannel_InformationResponse", HFILL }},
    { &hf_nbap_id_Synchronisation_Configuration_Cell_ReconfRqst,
      { "id-Synchronisation-Configuration-Cell-ReconfRqst", "nbap.id_Synchronisation_Configuration_Cell_ReconfRqst",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Synchronisation_Configuration_Cell_ReconfRqst", HFILL }},
    { &hf_nbap_id_Synchronisation_Configuration_Cell_SetupRqst,
      { "id-Synchronisation-Configuration-Cell-SetupRqst", "nbap.id_Synchronisation_Configuration_Cell_SetupRqst",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Synchronisation_Configuration_Cell_SetupRqst", HFILL }},
    { &hf_nbap_id_Transmission_Gap_Pattern_Sequence_Information,
      { "id-Transmission-Gap-Pattern-Sequence-Information", "nbap.id_Transmission_Gap_Pattern_Sequence_Information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Transmission_Gap_Pattern_Sequence_Information", HFILL }},
    { &hf_nbap_id_UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD,
      { "id-UL-CCTrCH-InformationAddList-RL-ReconfPrepTDD", "nbap.id_UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_id_UL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD,
      { "id-UL-CCTrCH-InformationDeleteItem-RL-ReconfRqstTDD", "nbap.id_UL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD", HFILL }},
    { &hf_nbap_id_UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD,
      { "id-UL-CCTrCH-InformationDeleteList-RL-ReconfPrepTDD", "nbap.id_UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_id_UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD,
      { "id-UL-CCTrCH-InformationDeleteList-RL-ReconfRqstTDD", "nbap.id_UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD", HFILL }},
    { &hf_nbap_id_UL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD,
      { "id-UL-CCTrCH-InformationModifyItem-RL-ReconfRqstTDD", "nbap.id_UL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD", HFILL }},
    { &hf_nbap_id_UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD,
      { "id-UL-CCTrCH-InformationModifyList-RL-ReconfPrepTDD", "nbap.id_UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_id_UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD,
      { "id-UL-CCTrCH-InformationModifyList-RL-ReconfRqstTDD", "nbap.id_UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD", HFILL }},
    { &hf_nbap_id_UL_DPCH_InformationAddListIE_RL_ReconfPrepTDD,
      { "id-UL-DPCH-InformationAddListIE-RL-ReconfPrepTDD", "nbap.id_UL_DPCH_InformationAddListIE_RL_ReconfPrepTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_DPCH_InformationAddItem_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_id_UL_DPCH_InformationModify_AddListIE_RL_ReconfPrepTDD,
      { "id-UL-DPCH-InformationModify-AddListIE-RL-ReconfPrepTDD", "nbap.id_UL_DPCH_InformationModify_AddListIE_RL_ReconfPrepTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_DPCH_InformationModify_AddItem_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_id_UL_DPCH_InformationModify_DeleteListIE_RL_ReconfPrepTDD,
      { "id-UL-DPCH-InformationModify-DeleteListIE-RL-ReconfPrepTDD", "nbap.id_UL_DPCH_InformationModify_DeleteListIE_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.UL_DPCH_InformationModify_DeleteListIE_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_id_UL_DPCH_InformationModify_ModifyListIE_RL_ReconfPrepTDD,
      { "id-UL-DPCH-InformationModify-ModifyListIE-RL-ReconfPrepTDD", "nbap.id_UL_DPCH_InformationModify_ModifyListIE_RL_ReconfPrepTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_DPCH_InformationModify_ModifyItem_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_id_Unsuccessful_PDSCHSetItem_PSCH_ReconfFailureTDD,
      { "id-Unsuccessful-PDSCHSetItem-PSCH-ReconfFailureTDD", "nbap.id_Unsuccessful_PDSCHSetItem_PSCH_ReconfFailureTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Unsuccessful_PDSCHSetItem_PSCH_ReconfFailureTDD", HFILL }},
    { &hf_nbap_id_Unsuccessful_PUSCHSetItem_PSCH_ReconfFailureTDD,
      { "id-Unsuccessful-PUSCHSetItem-PSCH-ReconfFailureTDD", "nbap.id_Unsuccessful_PUSCHSetItem_PSCH_ReconfFailureTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Unsuccessful_PUSCHSetItem_PSCH_ReconfFailureTDD", HFILL }},
    { &hf_nbap_id_CommunicationContextInfoItem_Reset,
      { "id-CommunicationContextInfoItem-Reset", "nbap.id_CommunicationContextInfoItem_Reset",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CommunicationContextInfoItem_Reset", HFILL }},
    { &hf_nbap_id_CommunicationControlPortInfoItem_Reset,
      { "id-CommunicationControlPortInfoItem-Reset", "nbap.id_CommunicationControlPortInfoItem_Reset",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CommunicationControlPortInfoItem_Reset", HFILL }},
    { &hf_nbap_id_ResetIndicator,
      { "id-ResetIndicator", "nbap.id_ResetIndicator",
        FT_UINT32, BASE_DEC, VALS(nbap_ResetIndicator_vals), 0,
        "nbap.ResetIndicator", HFILL }},
    { &hf_nbap_id_TimingAdvanceApplied,
      { "id-TimingAdvanceApplied", "nbap.id_TimingAdvanceApplied",
        FT_UINT32, BASE_DEC, VALS(nbap_TimingAdvanceApplied_vals), 0,
        "nbap.TimingAdvanceApplied", HFILL }},
    { &hf_nbap_id_CFNReportingIndicator,
      { "id-CFNReportingIndicator", "nbap.id_CFNReportingIndicator",
        FT_UINT32, BASE_DEC, VALS(nbap_FNReportingIndicator_vals), 0,
        "nbap.FNReportingIndicator", HFILL }},
    { &hf_nbap_id_SFNReportingIndicator,
      { "id-SFNReportingIndicator", "nbap.id_SFNReportingIndicator",
        FT_UINT32, BASE_DEC, VALS(nbap_FNReportingIndicator_vals), 0,
        "nbap.FNReportingIndicator", HFILL }},
    { &hf_nbap_id_InnerLoopDLPCStatus,
      { "id-InnerLoopDLPCStatus", "nbap.id_InnerLoopDLPCStatus",
        FT_UINT32, BASE_DEC, VALS(nbap_InnerLoopDLPCStatus_vals), 0,
        "nbap.InnerLoopDLPCStatus", HFILL }},
    { &hf_nbap_id_TimeslotISCPInfo,
      { "id-TimeslotISCPInfo", "nbap.id_TimeslotISCPInfo",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_TimeslotISCPInfo", HFILL }},
    { &hf_nbap_id_PICH_ParametersItem_CTCH_SetupRqstTDD,
      { "id-PICH-ParametersItem-CTCH-SetupRqstTDD", "nbap.id_PICH_ParametersItem_CTCH_SetupRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PICH_ParametersItem_CTCH_SetupRqstTDD", HFILL }},
    { &hf_nbap_id_PRACH_ParametersItem_CTCH_SetupRqstTDD,
      { "id-PRACH-ParametersItem-CTCH-SetupRqstTDD", "nbap.id_PRACH_ParametersItem_CTCH_SetupRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PRACH_ParametersItem_CTCH_SetupRqstTDD", HFILL }},
    { &hf_nbap_id_CCTrCH_InformationItem_RL_FailureInd,
      { "id-CCTrCH-InformationItem-RL-FailureInd", "nbap.id_CCTrCH_InformationItem_RL_FailureInd",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CCTrCH_InformationItem_RL_FailureInd", HFILL }},
    { &hf_nbap_id_CCTrCH_InformationItem_RL_RestoreInd,
      { "id-CCTrCH-InformationItem-RL-RestoreInd", "nbap.id_CCTrCH_InformationItem_RL_RestoreInd",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CCTrCH_InformationItem_RL_RestoreInd", HFILL }},
    { &hf_nbap_id_CauseLevel_SyncAdjustmntFailureTDD,
      { "id-CauseLevel-SyncAdjustmntFailureTDD", "nbap.id_CauseLevel_SyncAdjustmntFailureTDD",
        FT_UINT32, BASE_DEC, VALS(nbap_CauseLevel_SyncAdjustmntFailureTDD_vals), 0,
        "nbap.CauseLevel_SyncAdjustmntFailureTDD", HFILL }},
    { &hf_nbap_id_CellAdjustmentInfo_SyncAdjustmntRqstTDD,
      { "id-CellAdjustmentInfo-SyncAdjustmntRqstTDD", "nbap.id_CellAdjustmentInfo_SyncAdjustmntRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CellAdjustmentInfo_SyncAdjustmentRqstTDD", HFILL }},
    { &hf_nbap_id_CellAdjustmentInfoItem_SyncAdjustmentRqstTDD,
      { "id-CellAdjustmentInfoItem-SyncAdjustmentRqstTDD", "nbap.id_CellAdjustmentInfoItem_SyncAdjustmentRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CellAdjustmentInfoItem_SyncAdjustmentRqstTDD", HFILL }},
    { &hf_nbap_id_CellSyncBurstInfoList_CellSyncReconfRqstTDD,
      { "id-CellSyncBurstInfoList-CellSyncReconfRqstTDD", "nbap.id_CellSyncBurstInfoList_CellSyncReconfRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CellSyncBurstInfoList_CellSyncReconfRqstTDD", HFILL }},
    { &hf_nbap_id_CellSyncBurstTransInit_CellSyncInitiationRqstTDD,
      { "id-CellSyncBurstTransInit-CellSyncInitiationRqstTDD", "nbap.id_CellSyncBurstTransInit_CellSyncInitiationRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CellSyncBurstTransInit_CellSyncInitiationRqstTDD", HFILL }},
    { &hf_nbap_id_CellSyncBurstMeasureInit_CellSyncInitiationRqstTDD,
      { "id-CellSyncBurstMeasureInit-CellSyncInitiationRqstTDD", "nbap.id_CellSyncBurstMeasureInit_CellSyncInitiationRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CellSyncBurstMeasureInit_CellSyncInitiationRqstTDD", HFILL }},
    { &hf_nbap_id_CellSyncBurstMeasReconfiguration_CellSyncReconfRqstTDD,
      { "id-CellSyncBurstMeasReconfiguration-CellSyncReconfRqstTDD", "nbap.id_CellSyncBurstMeasReconfiguration_CellSyncReconfRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CellSyncBurstMeasInfo_CellSyncReconfRqstTDD", HFILL }},
    { &hf_nbap_id_CellSyncBurstMeasInfoList_CellSyncReconfRqstTDD,
      { "id-CellSyncBurstMeasInfoList-CellSyncReconfRqstTDD", "nbap.id_CellSyncBurstMeasInfoList_CellSyncReconfRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CellSyncBurstMeasInfoList_CellSyncReconfRqstTDD", HFILL }},
    { &hf_nbap_id_CellSyncBurstTransReconfInfo_CellSyncReconfRqstTDD,
      { "id-CellSyncBurstTransReconfInfo-CellSyncReconfRqstTDD", "nbap.id_CellSyncBurstTransReconfInfo_CellSyncReconfRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CellSyncBurstTransReconfInfo_CellSyncReconfRqstTDD", HFILL }},
    { &hf_nbap_id_CellSyncInfo_CellSyncReprtTDD,
      { "id-CellSyncInfo-CellSyncReprtTDD", "nbap.id_CellSyncInfo_CellSyncReprtTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CellSyncInfo_CellSyncReprtTDD", HFILL }},
    { &hf_nbap_id_CSBTransmissionID,
      { "id-CSBTransmissionID", "nbap.id_CSBTransmissionID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CSBTransmissionID", HFILL }},
    { &hf_nbap_id_CSBMeasurementID,
      { "id-CSBMeasurementID", "nbap.id_CSBMeasurementID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CSBMeasurementID", HFILL }},
    { &hf_nbap_id_NCyclesPerSFNperiod,
      { "id-NCyclesPerSFNperiod", "nbap.id_NCyclesPerSFNperiod",
        FT_UINT32, BASE_DEC, VALS(nbap_NCyclesPerSFNperiod_vals), 0,
        "nbap.NCyclesPerSFNperiod", HFILL }},
    { &hf_nbap_id_NRepetitionsPerCyclePeriod,
      { "id-NRepetitionsPerCyclePeriod", "nbap.id_NRepetitionsPerCyclePeriod",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.NRepetitionsPerCyclePeriod", HFILL }},
    { &hf_nbap_id_SyncFrameNumber,
      { "id-SyncFrameNumber", "nbap.id_SyncFrameNumber",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SyncFrameNumber", HFILL }},
    { &hf_nbap_id_SynchronisationReportType,
      { "id-SynchronisationReportType", "nbap.id_SynchronisationReportType",
        FT_UINT32, BASE_DEC, VALS(nbap_SynchronisationReportType_vals), 0,
        "nbap.SynchronisationReportType", HFILL }},
    { &hf_nbap_id_SynchronisationReportCharacteristics,
      { "id-SynchronisationReportCharacteristics", "nbap.id_SynchronisationReportCharacteristics",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.SynchronisationReportCharacteristics", HFILL }},
    { &hf_nbap_id_Unsuccessful_cell_InformationRespItem_SyncAdjustmntFailureTDD,
      { "id-Unsuccessful-cell-InformationRespItem-SyncAdjustmntFailureTDD", "nbap.id_Unsuccessful_cell_InformationRespItem_SyncAdjustmntFailureTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Unsuccessful_cell_InformationRespItem_SyncAdjustmntFailureTDD", HFILL }},
    { &hf_nbap_id_ReferenceClockAvailability,
      { "id-ReferenceClockAvailability", "nbap.id_ReferenceClockAvailability",
        FT_UINT32, BASE_DEC, VALS(nbap_ReferenceClockAvailability_vals), 0,
        "nbap.ReferenceClockAvailability", HFILL }},
    { &hf_nbap_id_ReferenceSFNoffset,
      { "id-ReferenceSFNoffset", "nbap.id_ReferenceSFNoffset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.ReferenceSFNoffset", HFILL }},
    { &hf_nbap_id_InformationExchangeID,
      { "id-InformationExchangeID", "nbap.id_InformationExchangeID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.InformationExchangeID", HFILL }},
    { &hf_nbap_id_InformationExchangeObjectType_InfEx_Rqst,
      { "id-InformationExchangeObjectType-InfEx-Rqst", "nbap.id_InformationExchangeObjectType_InfEx_Rqst",
        FT_UINT32, BASE_DEC, VALS(nbap_InformationExchangeObjectType_InfEx_Rqst_vals), 0,
        "nbap.InformationExchangeObjectType_InfEx_Rqst", HFILL }},
    { &hf_nbap_id_InformationType,
      { "id-InformationType", "nbap.id_InformationType",
        FT_UINT32, BASE_DEC, VALS(nbap_InformationReportCharacteristics_vals), 0,
        "nbap.InformationReportCharacteristics", HFILL }},
    { &hf_nbap_id_InformationReportCharacteristics,
      { "id-InformationReportCharacteristics", "nbap.id_InformationReportCharacteristics",
        FT_UINT32, BASE_DEC, VALS(nbap_InformationReportCharacteristics_vals), 0,
        "nbap.InformationReportCharacteristics", HFILL }},
    { &hf_nbap_id_InformationExchangeObjectType_InfEx_Rsp,
      { "id-InformationExchangeObjectType-InfEx-Rsp", "nbap.id_InformationExchangeObjectType_InfEx_Rsp",
        FT_UINT32, BASE_DEC, VALS(nbap_InformationExchangeObjectType_InfEx_Rsp_vals), 0,
        "nbap.InformationExchangeObjectType_InfEx_Rsp", HFILL }},
    { &hf_nbap_id_InformationExchangeObjectType_InfEx_Rprt,
      { "id-InformationExchangeObjectType-InfEx-Rprt", "nbap.id_InformationExchangeObjectType_InfEx_Rprt",
        FT_UINT32, BASE_DEC, VALS(nbap_InformationExchangeObjectType_InfEx_Rprt_vals), 0,
        "nbap.InformationExchangeObjectType_InfEx_Rprt", HFILL }},
    { &hf_nbap_id_IPDLParameter_Information_Cell_ReconfRqstFDD,
      { "id-IPDLParameter-Information-Cell-ReconfRqstFDD", "nbap.id_IPDLParameter_Information_Cell_ReconfRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.IPDLParameter_Information_Cell_ReconfRqstFDD", HFILL }},
    { &hf_nbap_id_IPDLParameter_Information_Cell_SetupRqstFDD,
      { "id-IPDLParameter-Information-Cell-SetupRqstFDD", "nbap.id_IPDLParameter_Information_Cell_SetupRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.IPDLParameter_Information_Cell_SetupRqstFDD", HFILL }},
    { &hf_nbap_id_IPDLParameter_Information_Cell_ReconfRqstTDD,
      { "id-IPDLParameter-Information-Cell-ReconfRqstTDD", "nbap.id_IPDLParameter_Information_Cell_ReconfRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.IPDLParameter_Information_Cell_SetupRqstTDD", HFILL }},
    { &hf_nbap_id_IPDLParameter_Information_Cell_SetupRqstTDD,
      { "id-IPDLParameter-Information-Cell-SetupRqstTDD", "nbap.id_IPDLParameter_Information_Cell_SetupRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.IPDLParameter_Information_Cell_SetupRqstTDD", HFILL }},
    { &hf_nbap_id_DL_DPCH_LCR_Information_RL_SetupRqstTDD,
      { "id-DL-DPCH-LCR-Information-RL-SetupRqstTDD", "nbap.id_DL_DPCH_LCR_Information_RL_SetupRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_DPCH_LCR_Information_RL_SetupRqstTDD", HFILL }},
    { &hf_nbap_id_DwPCH_LCR_Information,
      { "id-DwPCH-LCR-Information", "nbap.id_DwPCH_LCR_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Common_PhysicalChannel_Status_Information", HFILL }},
    { &hf_nbap_id_DwPCH_LCR_InformationList_AuditRsp,
      { "id-DwPCH-LCR-InformationList-AuditRsp", "nbap.id_DwPCH_LCR_InformationList_AuditRsp",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Common_PhysicalChannel_Status_Information", HFILL }},
    { &hf_nbap_id_DwPCH_LCR_Information_Cell_SetupRqstTDD,
      { "id-DwPCH-LCR-Information-Cell-SetupRqstTDD", "nbap.id_DwPCH_LCR_Information_Cell_SetupRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DwPCH_LCR_Information_Cell_SetupRqstTDD", HFILL }},
    { &hf_nbap_id_DwPCH_LCR_Information_Cell_ReconfRqstTDD,
      { "id-DwPCH-LCR-Information-Cell-ReconfRqstTDD", "nbap.id_DwPCH_LCR_Information_Cell_ReconfRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DwPCH_LCR_Information_Cell_ReconfRqstTDD", HFILL }},
    { &hf_nbap_id_DwPCH_LCR_Information_ResourceStatusInd,
      { "id-DwPCH-LCR-Information-ResourceStatusInd", "nbap.id_DwPCH_LCR_Information_ResourceStatusInd",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DwPCH_LCR_Information_ResourceStatusInd", HFILL }},
    { &hf_nbap_id_maxFACH_Power_LCR_CTCH_SetupRqstTDD,
      { "id-maxFACH-Power-LCR-CTCH-SetupRqstTDD", "nbap.id_maxFACH_Power_LCR_CTCH_SetupRqstTDD",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_id_maxFACH_Power_LCR_CTCH_ReconfRqstTDD,
      { "id-maxFACH-Power-LCR-CTCH-ReconfRqstTDD", "nbap.id_maxFACH_Power_LCR_CTCH_ReconfRqstTDD",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_id_FPACH_LCR_Information,
      { "id-FPACH-LCR-Information", "nbap.id_FPACH_LCR_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Common_PhysicalChannel_Status_Information", HFILL }},
    { &hf_nbap_id_FPACH_LCR_Information_AuditRsp,
      { "id-FPACH-LCR-Information-AuditRsp", "nbap.id_FPACH_LCR_Information_AuditRsp",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Common_PhysicalChannel_Status_Information", HFILL }},
    { &hf_nbap_id_FPACH_LCR_InformationList_AuditRsp,
      { "id-FPACH-LCR-InformationList-AuditRsp", "nbap.id_FPACH_LCR_InformationList_AuditRsp",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.FPACH_LCR_InformationList_AuditRsp", HFILL }},
    { &hf_nbap_id_FPACH_LCR_InformationList_ResourceStatusInd,
      { "id-FPACH-LCR-InformationList-ResourceStatusInd", "nbap.id_FPACH_LCR_InformationList_ResourceStatusInd",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.FPACH_LCR_InformationList_ResourceStatusInd", HFILL }},
    { &hf_nbap_id_FPACH_LCR_Parameters_CTCH_SetupRqstTDD,
      { "id-FPACH-LCR-Parameters-CTCH-SetupRqstTDD", "nbap.id_FPACH_LCR_Parameters_CTCH_SetupRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.FPACH_LCR_Parameters_CTCH_SetupRqstTDD", HFILL }},
    { &hf_nbap_id_FPACH_LCR_Parameters_CTCH_ReconfRqstTDD,
      { "id-FPACH-LCR-Parameters-CTCH-ReconfRqstTDD", "nbap.id_FPACH_LCR_Parameters_CTCH_ReconfRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.FPACH_LCR_Parameters_CTCH_ReconfRqstTDD", HFILL }},
    { &hf_nbap_id_PCCPCH_LCR_Information_Cell_SetupRqstTDD,
      { "id-PCCPCH-LCR-Information-Cell-SetupRqstTDD", "nbap.id_PCCPCH_LCR_Information_Cell_SetupRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PCCPCH_LCR_Information_Cell_SetupRqstTDD", HFILL }},
    { &hf_nbap_id_PCH_Power_LCR_CTCH_SetupRqstTDD,
      { "id-PCH-Power-LCR-CTCH-SetupRqstTDD", "nbap.id_PCH_Power_LCR_CTCH_SetupRqstTDD",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_id_PCH_Power_LCR_CTCH_ReconfRqstTDD,
      { "id-PCH-Power-LCR-CTCH-ReconfRqstTDD", "nbap.id_PCH_Power_LCR_CTCH_ReconfRqstTDD",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_id_PICH_LCR_Parameters_CTCH_SetupRqstTDD,
      { "id-PICH-LCR-Parameters-CTCH-SetupRqstTDD", "nbap.id_PICH_LCR_Parameters_CTCH_SetupRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PICH_LCR_Parameters_CTCH_SetupRqstTDD", HFILL }},
    { &hf_nbap_id_PRACH_LCR_ParametersList_CTCH_SetupRqstTDD,
      { "id-PRACH-LCR-ParametersList-CTCH-SetupRqstTDD", "nbap.id_PRACH_LCR_ParametersList_CTCH_SetupRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.PRACH_LCR_ParametersList_CTCH_SetupRqstTDD", HFILL }},
    { &hf_nbap_id_RL_InformationResponse_LCR_RL_SetupRspTDD,
      { "id-RL-InformationResponse-LCR-RL-SetupRspTDD", "nbap.id_RL_InformationResponse_LCR_RL_SetupRspTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_InformationResponse_LCR_RL_SetupRspTDD", HFILL }},
    { &hf_nbap_id_Secondary_CCPCH_LCR_parameterList_CTCH_SetupRqstTDD,
      { "id-Secondary-CCPCH-LCR-parameterList-CTCH-SetupRqstTDD", "nbap.id_Secondary_CCPCH_LCR_parameterList_CTCH_SetupRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Secondary_CCPCH_LCR_parameterList_CTCH_SetupRqstTDD", HFILL }},
    { &hf_nbap_id_TimeSlot,
      { "id-TimeSlot", "nbap.id_TimeSlot",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.TimeSlot", HFILL }},
    { &hf_nbap_id_TimeSlotConfigurationList_LCR_Cell_ReconfRqstTDD,
      { "id-TimeSlotConfigurationList-LCR-Cell-ReconfRqstTDD", "nbap.id_TimeSlotConfigurationList_LCR_Cell_ReconfRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.TimeSlotConfigurationList_LCR_Cell_ReconfRqstTDD", HFILL }},
    { &hf_nbap_id_TimeSlotConfigurationList_LCR_Cell_SetupRqstTDD,
      { "id-TimeSlotConfigurationList-LCR-Cell-SetupRqstTDD", "nbap.id_TimeSlotConfigurationList_LCR_Cell_SetupRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.TimeSlotConfigurationList_LCR_Cell_SetupRqstTDD", HFILL }},
    { &hf_nbap_id_TimeslotISCP_LCR_InfoList_RL_SetupRqstTDD,
      { "id-TimeslotISCP-LCR-InfoList-RL-SetupRqstTDD", "nbap.id_TimeslotISCP_LCR_InfoList_RL_SetupRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_TimeslotISCPInfoLCR", HFILL }},
    { &hf_nbap_id_TimeSlotLCR_CM_Rqst,
      { "id-TimeSlotLCR-CM-Rqst", "nbap.id_TimeSlotLCR_CM_Rqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.TimeSlotLCR", HFILL }},
    { &hf_nbap_id_UL_DPCH_LCR_Information_RL_SetupRqstTDD,
      { "id-UL-DPCH-LCR-Information-RL-SetupRqstTDD", "nbap.id_UL_DPCH_LCR_Information_RL_SetupRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_DPCH_LCR_Information_RL_SetupRqstTDD", HFILL }},
    { &hf_nbap_id_DL_DPCH_InformationItem_LCR_RL_AdditionRqstTDD,
      { "id-DL-DPCH-InformationItem-LCR-RL-AdditionRqstTDD", "nbap.id_DL_DPCH_InformationItem_LCR_RL_AdditionRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_DPCH_InformationItem_LCR_RL_AdditionRqstTDD", HFILL }},
    { &hf_nbap_id_UL_DPCH_InformationItem_LCR_RL_AdditionRqstTDD,
      { "id-UL-DPCH-InformationItem-LCR-RL-AdditionRqstTDD", "nbap.id_UL_DPCH_InformationItem_LCR_RL_AdditionRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_DPCH_InformationItem_LCR_RL_AdditionRqstTDD", HFILL }},
    { &hf_nbap_id_TimeslotISCP_InformationList_LCR_RL_AdditionRqstTDD,
      { "id-TimeslotISCP-InformationList-LCR-RL-AdditionRqstTDD", "nbap.id_TimeslotISCP_InformationList_LCR_RL_AdditionRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_TimeslotISCPInfoLCR", HFILL }},
    { &hf_nbap_id_DL_DPCH_LCR_InformationAddList_RL_ReconfPrepTDD,
      { "id-DL-DPCH-LCR-InformationAddList-RL-ReconfPrepTDD", "nbap.id_DL_DPCH_LCR_InformationAddList_RL_ReconfPrepTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_DPCH_LCR_InformationAddList_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_id_DL_DPCH_LCR_InformationModify_AddList_RL_ReconfPrepTDD,
      { "id-DL-DPCH-LCR-InformationModify-AddList-RL-ReconfPrepTDD", "nbap.id_DL_DPCH_LCR_InformationModify_AddList_RL_ReconfPrepTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_DPCH_LCR_InformationModify_AddList_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_id_DL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD,
      { "id-DL-Timeslot-LCR-InformationModify-ModifyList-RL-ReconfPrepTDD", "nbap.id_DL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_id_TimeslotISCPInfoList_LCR_DL_PC_RqstTDD,
      { "id-TimeslotISCPInfoList-LCR-DL-PC-RqstTDD", "nbap.id_TimeslotISCPInfoList_LCR_DL_PC_RqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_TimeslotISCPInfoLCR", HFILL }},
    { &hf_nbap_id_UL_DPCH_LCR_InformationAddListIE_RL_ReconfPrepTDD,
      { "id-UL-DPCH-LCR-InformationAddListIE-RL-ReconfPrepTDD", "nbap.id_UL_DPCH_LCR_InformationAddListIE_RL_ReconfPrepTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_DPCH_LCR_InformationAddList_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_id_UL_DPCH_LCR_InformationModify_AddList,
      { "id-UL-DPCH-LCR-InformationModify-AddList", "nbap.id_UL_DPCH_LCR_InformationModify_AddList",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_DPCH_LCR_InformationModify_AddList_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_id_UL_TimeslotLCR_Information_RL_ReconfPrepTDD,
      { "id-UL-TimeslotLCR-Information-RL-ReconfPrepTDD", "nbap.id_UL_TimeslotLCR_Information_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.UL_TimeslotLCR_InformationModify_ModifyList_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_id_UL_SIRTarget,
      { "id-UL-SIRTarget", "nbap.id_UL_SIRTarget",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.UL_SIR", HFILL }},
    { &hf_nbap_id_PDSCH_AddInformation_LCR_PSCH_ReconfRqst,
      { "id-PDSCH-AddInformation-LCR-PSCH-ReconfRqst", "nbap.id_PDSCH_AddInformation_LCR_PSCH_ReconfRqst",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PDSCH_AddInformation_LCR_AddItem_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_id_PDSCH_ModifyInformation_LCR_PSCH_ReconfRqst,
      { "id-PDSCH-ModifyInformation-LCR-PSCH-ReconfRqst", "nbap.id_PDSCH_ModifyInformation_LCR_PSCH_ReconfRqst",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PDSCH_ModifyInformation_LCR_ModifyItem_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_id_PUSCH_AddInformation_LCR_PSCH_ReconfRqst,
      { "id-PUSCH-AddInformation-LCR-PSCH-ReconfRqst", "nbap.id_PUSCH_AddInformation_LCR_PSCH_ReconfRqst",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PUSCH_AddInformation_LCR_AddItem_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_id_PUSCH_ModifyInformation_LCR_PSCH_ReconfRqst,
      { "id-PUSCH-ModifyInformation-LCR-PSCH-ReconfRqst", "nbap.id_PUSCH_ModifyInformation_LCR_PSCH_ReconfRqst",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PUSCH_ModifyInformation_LCR_ModifyItem_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_id_timeslotInfo_CellSyncInitiationRqstTDD,
      { "id-timeslotInfo-CellSyncInitiationRqstTDD", "nbap.id_timeslotInfo_CellSyncInitiationRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.TimeslotInfo_CellSyncInitiationRqstTDD", HFILL }},
    { &hf_nbap_id_SyncReportType_CellSyncReprtTDD,
      { "id-SyncReportType-CellSyncReprtTDD", "nbap.id_SyncReportType_CellSyncReprtTDD",
        FT_UINT32, BASE_DEC, VALS(nbap_SyncReportType_CellSyncReprtTDD_vals), 0,
        "nbap.SyncReportType_CellSyncReprtTDD", HFILL }},
    { &hf_nbap_id_Power_Local_Cell_Group_InformationItem_AuditRsp,
      { "id-Power-Local-Cell-Group-InformationItem-AuditRsp", "nbap.id_Power_Local_Cell_Group_InformationItem_AuditRsp",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Power_Local_Cell_Group_InformationItem_AuditRsp", HFILL }},
    { &hf_nbap_id_Power_Local_Cell_Group_InformationItem_ResourceStatusInd,
      { "id-Power-Local-Cell-Group-InformationItem-ResourceStatusInd", "nbap.id_Power_Local_Cell_Group_InformationItem_ResourceStatusInd",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Power_Local_Cell_Group_InformationItem_ResourceStatusInd", HFILL }},
    { &hf_nbap_id_Power_Local_Cell_Group_InformationItem2_ResourceStatusInd,
      { "id-Power-Local-Cell-Group-InformationItem2-ResourceStatusInd", "nbap.id_Power_Local_Cell_Group_InformationItem2_ResourceStatusInd",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Power_Local_Cell_Group_InformationItem2_ResourceStatusInd", HFILL }},
    { &hf_nbap_id_Power_Local_Cell_Group_InformationList_AuditRsp,
      { "id-Power-Local-Cell-Group-InformationList-AuditRsp", "nbap.id_Power_Local_Cell_Group_InformationList_AuditRsp",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Power_Local_Cell_Group_InformationList_AuditRsp", HFILL }},
    { &hf_nbap_id_Power_Local_Cell_Group_InformationList_ResourceStatusInd,
      { "id-Power-Local-Cell-Group-InformationList-ResourceStatusInd", "nbap.id_Power_Local_Cell_Group_InformationList_ResourceStatusInd",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Power_Local_Cell_Group_InformationList_ResourceStatusInd", HFILL }},
    { &hf_nbap_id_Power_Local_Cell_Group_InformationList2_ResourceStatusInd,
      { "id-Power-Local-Cell-Group-InformationList2-ResourceStatusInd", "nbap.id_Power_Local_Cell_Group_InformationList2_ResourceStatusInd",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Power_Local_Cell_Group_InformationList2_ResourceStatusInd", HFILL }},
    { &hf_nbap_id_Power_Local_Cell_Group_ID,
      { "id-Power-Local-Cell-Group-ID", "nbap.id_Power_Local_Cell_Group_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Local_Cell_ID", HFILL }},
    { &hf_nbap_id_PUSCH_Info_DM_Rqst,
      { "id-PUSCH-Info-DM-Rqst", "nbap.id_PUSCH_Info_DM_Rqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.PUSCH_Info_DM_Rqst", HFILL }},
    { &hf_nbap_id_PUSCH_Info_DM_Rsp,
      { "id-PUSCH-Info-DM-Rsp", "nbap.id_PUSCH_Info_DM_Rsp",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.PUSCH_Info_DM_Rsp", HFILL }},
    { &hf_nbap_id_PUSCH_Info_DM_Rprt,
      { "id-PUSCH-Info-DM-Rprt", "nbap.id_PUSCH_Info_DM_Rprt",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.PUSCH_Info_DM_Rprt", HFILL }},
    { &hf_nbap_id_InitDL_Power,
      { "id-InitDL-Power", "nbap.id_InitDL_Power",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_id_cellSyncBurstRepetitionPeriod,
      { "id-cellSyncBurstRepetitionPeriod", "nbap.id_cellSyncBurstRepetitionPeriod",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CellSyncBurstRepetitionPeriod", HFILL }},
    { &hf_nbap_id_ReportCharacteristicsType_OnModification,
      { "id-ReportCharacteristicsType-OnModification", "nbap.id_ReportCharacteristicsType_OnModification",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ReportCharacteristicsType_OnModification", HFILL }},
    { &hf_nbap_id_SFNSFNMeasurementValueInformation,
      { "id-SFNSFNMeasurementValueInformation", "nbap.id_SFNSFNMeasurementValueInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.SFNSFNMeasurementValueInformation", HFILL }},
    { &hf_nbap_id_SFNSFNMeasurementThresholdInformation,
      { "id-SFNSFNMeasurementThresholdInformation", "nbap.id_SFNSFNMeasurementThresholdInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.SFNSFNMeasurementThresholdInformation", HFILL }},
    { &hf_nbap_id_TUTRANGPSMeasurementValueInformation,
      { "id-TUTRANGPSMeasurementValueInformation", "nbap.id_TUTRANGPSMeasurementValueInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.TUTRANGPSMeasurementValueInformation", HFILL }},
    { &hf_nbap_id_TUTRANGPSMeasurementThresholdInformation,
      { "id-TUTRANGPSMeasurementThresholdInformation", "nbap.id_TUTRANGPSMeasurementThresholdInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.TUTRANGPSMeasurementThresholdInformation", HFILL }},
    { &hf_nbap_id_Rx_Timing_Deviation_Value_LCR,
      { "id-Rx-Timing-Deviation-Value-LCR", "nbap.id_Rx_Timing_Deviation_Value_LCR",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Rx_Timing_Deviation_Value_LCR", HFILL }},
    { &hf_nbap_id_RL_InformationResponse_LCR_RL_AdditionRspTDD,
      { "id-RL-InformationResponse-LCR-RL-AdditionRspTDD", "nbap.id_RL_InformationResponse_LCR_RL_AdditionRspTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_InformationResponse_LCR_RL_AdditionRspTDD", HFILL }},
    { &hf_nbap_id_DL_PowerBalancing_Information,
      { "id-DL-PowerBalancing-Information", "nbap.id_DL_PowerBalancing_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_PowerBalancing_Information", HFILL }},
    { &hf_nbap_id_DL_PowerBalancing_ActivationIndicator,
      { "id-DL-PowerBalancing-ActivationIndicator", "nbap.id_DL_PowerBalancing_ActivationIndicator",
        FT_UINT32, BASE_DEC, VALS(nbap_DL_PowerBalancing_ActivationIndicator_vals), 0,
        "nbap.DL_PowerBalancing_ActivationIndicator", HFILL }},
    { &hf_nbap_id_DL_PowerBalancing_UpdatedIndicator,
      { "id-DL-PowerBalancing-UpdatedIndicator", "nbap.id_DL_PowerBalancing_UpdatedIndicator",
        FT_UINT32, BASE_DEC, VALS(nbap_DL_PowerBalancing_UpdatedIndicator_vals), 0,
        "nbap.DL_PowerBalancing_UpdatedIndicator", HFILL }},
    { &hf_nbap_id_CCTrCH_Initial_DL_Power_RL_SetupRqstTDD,
      { "id-CCTrCH-Initial-DL-Power-RL-SetupRqstTDD", "nbap.id_CCTrCH_Initial_DL_Power_RL_SetupRqstTDD",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_id_CCTrCH_Initial_DL_Power_RL_AdditionRqstTDD,
      { "id-CCTrCH-Initial-DL-Power-RL-AdditionRqstTDD", "nbap.id_CCTrCH_Initial_DL_Power_RL_AdditionRqstTDD",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_id_CCTrCH_Initial_DL_Power_RL_ReconfPrepTDD,
      { "id-CCTrCH-Initial-DL-Power-RL-ReconfPrepTDD", "nbap.id_CCTrCH_Initial_DL_Power_RL_ReconfPrepTDD",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_id_IPDLParameter_Information_LCR_Cell_SetupRqstTDD,
      { "id-IPDLParameter-Information-LCR-Cell-SetupRqstTDD", "nbap.id_IPDLParameter_Information_LCR_Cell_SetupRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.IPDLParameter_Information_LCR_Cell_SetupRqstTDD", HFILL }},
    { &hf_nbap_id_IPDLParameter_Information_LCR_Cell_ReconfRqstTDD,
      { "id-IPDLParameter-Information-LCR-Cell-ReconfRqstTDD", "nbap.id_IPDLParameter_Information_LCR_Cell_ReconfRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.IPDLParameter_Information_LCR_Cell_ReconfRqstTDD", HFILL }},
    { &hf_nbap_id_HS_PDSCH_HS_SCCH_E_AGCH_E_RGCH_E_HICH_MaxPower_PSCH_ReconfRqst,
      { "id-HS-PDSCH-HS-SCCH-E-AGCH-E-RGCH-E-HICH-MaxPower-PSCH-ReconfRqst", "nbap.id_HS_PDSCH_HS_SCCH_E_AGCH_E_RGCH_E_HICH_MaxPower_PSCH_ReconfRqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.MaximumTransmissionPower", HFILL }},
    { &hf_nbap_id_HS_PDSCH_HS_SCCH_ScramblingCode_PSCH_ReconfRqst,
      { "id-HS-PDSCH-HS-SCCH-ScramblingCode-PSCH-ReconfRqst", "nbap.id_HS_PDSCH_HS_SCCH_ScramblingCode_PSCH_ReconfRqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_ScramblingCode", HFILL }},
    { &hf_nbap_id_HS_PDSCH_FDD_Code_Information_PSCH_ReconfRqst,
      { "id-HS-PDSCH-FDD-Code-Information-PSCH-ReconfRqst", "nbap.id_HS_PDSCH_FDD_Code_Information_PSCH_ReconfRqst",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HS_PDSCH_FDD_Code_Information", HFILL }},
    { &hf_nbap_id_HS_SCCH_FDD_Code_Information_PSCH_ReconfRqst,
      { "id-HS-SCCH-FDD-Code-Information-PSCH-ReconfRqst", "nbap.id_HS_SCCH_FDD_Code_Information_PSCH_ReconfRqst",
        FT_UINT32, BASE_DEC, VALS(nbap_HS_SCCH_FDD_Code_Information_vals), 0,
        "nbap.HS_SCCH_FDD_Code_Information", HFILL }},
    { &hf_nbap_id_HS_PDSCH_TDD_Information_PSCH_ReconfRqst,
      { "id-HS-PDSCH-TDD-Information-PSCH-ReconfRqst", "nbap.id_HS_PDSCH_TDD_Information_PSCH_ReconfRqst",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HS_PDSCH_TDD_Information_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_id_Add_To_HS_SCCH_Resource_Pool_PSCH_ReconfRqst,
      { "id-Add-To-HS-SCCH-Resource-Pool-PSCH-ReconfRqst", "nbap.id_Add_To_HS_SCCH_Resource_Pool_PSCH_ReconfRqst",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Add_To_HS_SCCH_Resource_Pool_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_id_Modify_HS_SCCH_Resource_Pool_PSCH_ReconfRqst,
      { "id-Modify-HS-SCCH-Resource-Pool-PSCH-ReconfRqst", "nbap.id_Modify_HS_SCCH_Resource_Pool_PSCH_ReconfRqst",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Modify_HS_SCCH_Resource_Pool_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_id_Delete_From_HS_SCCH_Resource_Pool_PSCH_ReconfRqst,
      { "id-Delete-From-HS-SCCH-Resource-Pool-PSCH-ReconfRqst", "nbap.id_Delete_From_HS_SCCH_Resource_Pool_PSCH_ReconfRqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Delete_From_HS_SCCH_Resource_Pool_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_id_bindingID,
      { "id-bindingID", "nbap.id_bindingID",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BindingID", HFILL }},
    { &hf_nbap_id_RL_Specific_DCH_Info,
      { "id-RL-Specific-DCH-Info", "nbap.id_RL_Specific_DCH_Info",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.RL_Specific_DCH_Info", HFILL }},
    { &hf_nbap_id_transportlayeraddress,
      { "id-transportlayeraddress", "nbap.id_transportlayeraddress",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.TransportLayerAddress", HFILL }},
    { &hf_nbap_id_DelayedActivation,
      { "id-DelayedActivation", "nbap.id_DelayedActivation",
        FT_UINT32, BASE_DEC, VALS(nbap_DelayedActivation_vals), 0,
        "nbap.DelayedActivation", HFILL }},
    { &hf_nbap_id_DelayedActivationList_RL_ActivationCmdFDD,
      { "id-DelayedActivationList-RL-ActivationCmdFDD", "nbap.id_DelayedActivationList_RL_ActivationCmdFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DelayedActivationInformationList_RL_ActivationCmdFDD", HFILL }},
    { &hf_nbap_id_DelayedActivationInformation_RL_ActivationCmdFDD,
      { "id-DelayedActivationInformation-RL-ActivationCmdFDD", "nbap.id_DelayedActivationInformation_RL_ActivationCmdFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DelayedActivationInformation_RL_ActivationCmdFDD", HFILL }},
    { &hf_nbap_id_DelayedActivationList_RL_ActivationCmdTDD,
      { "id-DelayedActivationList-RL-ActivationCmdTDD", "nbap.id_DelayedActivationList_RL_ActivationCmdTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DelayedActivationInformationList_RL_ActivationCmdTDD", HFILL }},
    { &hf_nbap_id_DelayedActivationInformation_RL_ActivationCmdTDD,
      { "id-DelayedActivationInformation-RL-ActivationCmdTDD", "nbap.id_DelayedActivationInformation_RL_ActivationCmdTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DelayedActivationInformation_RL_ActivationCmdTDD", HFILL }},
    { &hf_nbap_id_neighbouringTDDCellMeasurementInformationLCR,
      { "id-neighbouringTDDCellMeasurementInformationLCR", "nbap.id_neighbouringTDDCellMeasurementInformationLCR",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NeighbouringTDDCellMeasurementInformationLCR", HFILL }},
    { &hf_nbap_id_SYNCDlCodeId_TransInitLCR_CellSyncInitiationRqstTDD,
      { "id-SYNCDlCodeId-TransInitLCR-CellSyncInitiationRqstTDD", "nbap.id_SYNCDlCodeId_TransInitLCR_CellSyncInitiationRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.SYNCDlCodeId_TransInitLCR_CellSyncInitiationRqstTDD", HFILL }},
    { &hf_nbap_id_SYNCDlCodeId_MeasureInitLCR_CellSyncInitiationRqstTDD,
      { "id-SYNCDlCodeId-MeasureInitLCR-CellSyncInitiationRqstTDD", "nbap.id_SYNCDlCodeId_MeasureInitLCR_CellSyncInitiationRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.SYNCDlCodeId_MeasureInitLCR_CellSyncInitiationRqstTDD", HFILL }},
    { &hf_nbap_id_SYNCDlCodeIdTransReconfInfoLCR_CellSyncReconfRqstTDD,
      { "id-SYNCDlCodeIdTransReconfInfoLCR-CellSyncReconfRqstTDD", "nbap.id_SYNCDlCodeIdTransReconfInfoLCR_CellSyncReconfRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SYNCDlCodeIdTransReconfInfoLCR_CellSyncReconfRqstTDD", HFILL }},
    { &hf_nbap_id_SYNCDlCodeIdMeasReconfigurationLCR_CellSyncReconfRqstTDD,
      { "id-SYNCDlCodeIdMeasReconfigurationLCR-CellSyncReconfRqstTDD", "nbap.id_SYNCDlCodeIdMeasReconfigurationLCR_CellSyncReconfRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.SYNCDlCodeIdMeasInfoLCR_CellSyncReconfRqstTDD", HFILL }},
    { &hf_nbap_id_SYNCDlCodeIdMeasInfoList_CellSyncReconfRqstTDD,
      { "id-SYNCDlCodeIdMeasInfoList-CellSyncReconfRqstTDD", "nbap.id_SYNCDlCodeIdMeasInfoList_CellSyncReconfRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SYNCDlCodeIdMeasInfoList_CellSyncReconfRqstTDD", HFILL }},
    { &hf_nbap_id_SyncDLCodeIdsMeasInfoList_CellSyncReprtTDD,
      { "id-SyncDLCodeIdsMeasInfoList-CellSyncReprtTDD", "nbap.id_SyncDLCodeIdsMeasInfoList_CellSyncReprtTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SyncDLCodeIdsMeasInfoList_CellSyncReprtTDD", HFILL }},
    { &hf_nbap_id_SyncDLCodeIdThreInfoLCR,
      { "id-SyncDLCodeIdThreInfoLCR", "nbap.id_SyncDLCodeIdThreInfoLCR",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SyncDLCodeIdThreInfoLCR", HFILL }},
    { &hf_nbap_id_NSubCyclesPerCyclePeriod_CellSyncReconfRqstTDD,
      { "id-NSubCyclesPerCyclePeriod-CellSyncReconfRqstTDD", "nbap.id_NSubCyclesPerCyclePeriod_CellSyncReconfRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.NSubCyclesPerCyclePeriod", HFILL }},
    { &hf_nbap_id_DwPCH_Power,
      { "id-DwPCH-Power", "nbap.id_DwPCH_Power",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DwPCH_Power", HFILL }},
    { &hf_nbap_id_AccumulatedClockupdate_CellSyncReprtTDD,
      { "id-AccumulatedClockupdate-CellSyncReprtTDD", "nbap.id_AccumulatedClockupdate_CellSyncReprtTDD",
        FT_UINT32, BASE_DEC, VALS(nbap_TimingAdjustmentValue_vals), 0,
        "nbap.TimingAdjustmentValue", HFILL }},
    { &hf_nbap_id_Angle_Of_Arrival_Value_LCR,
      { "id-Angle-Of-Arrival-Value-LCR", "nbap.id_Angle_Of_Arrival_Value_LCR",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Angle_Of_Arrival_Value_LCR", HFILL }},
    { &hf_nbap_id_HSDSCH_FDD_Information,
      { "id-HSDSCH-FDD-Information", "nbap.id_HSDSCH_FDD_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HSDSCH_FDD_Information", HFILL }},
    { &hf_nbap_id_HSDSCH_FDD_Information_Response,
      { "id-HSDSCH-FDD-Information-Response", "nbap.id_HSDSCH_FDD_Information_Response",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HSDSCH_FDD_Information_Response", HFILL }},
    { &hf_nbap_id_HSDSCH_Information_to_Modify,
      { "id-HSDSCH-Information-to-Modify", "nbap.id_HSDSCH_Information_to_Modify",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HSDSCH_Information_to_Modify", HFILL }},
    { &hf_nbap_id_HSDSCH_RNTI,
      { "id-HSDSCH-RNTI", "nbap.id_HSDSCH_RNTI",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HSDSCH_RNTI", HFILL }},
    { &hf_nbap_id_HSDSCH_TDD_Information,
      { "id-HSDSCH-TDD-Information", "nbap.id_HSDSCH_TDD_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HSDSCH_TDD_Information", HFILL }},
    { &hf_nbap_id_HSDSCH_TDD_Information_Response,
      { "id-HSDSCH-TDD-Information-Response", "nbap.id_HSDSCH_TDD_Information_Response",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HSDSCH_TDD_Information_Response", HFILL }},
    { &hf_nbap_id_HSPDSCH_RL_ID,
      { "id-HSPDSCH-RL-ID", "nbap.id_HSPDSCH_RL_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.RL_ID", HFILL }},
    { &hf_nbap_id_PrimCCPCH_RSCP_DL_PC_RqstTDD,
      { "id-PrimCCPCH-RSCP-DL-PC-RqstTDD", "nbap.id_PrimCCPCH_RSCP_DL_PC_RqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.PrimaryCCPCH_RSCP", HFILL }},
    { &hf_nbap_id_PDSCH_RL_ID,
      { "id-PDSCH-RL-ID", "nbap.id_PDSCH_RL_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.RL_ID", HFILL }},
    { &hf_nbap_id_HSDSCH_RearrangeList_Bearer_RearrangeInd,
      { "id-HSDSCH-RearrangeList-Bearer-RearrangeInd", "nbap.id_HSDSCH_RearrangeList_Bearer_RearrangeInd",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HSDSCH_RearrangeList_Bearer_RearrangeInd", HFILL }},
    { &hf_nbap_id_UL_Synchronisation_Parameters_LCR,
      { "id-UL-Synchronisation-Parameters-LCR", "nbap.id_UL_Synchronisation_Parameters_LCR",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_Synchronisation_Parameters_LCR", HFILL }},
    { &hf_nbap_id_HSDSCH_FDD_Update_Information,
      { "id-HSDSCH-FDD-Update-Information", "nbap.id_HSDSCH_FDD_Update_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HSDSCH_FDD_Update_Information", HFILL }},
    { &hf_nbap_id_HSDSCH_TDD_Update_Information,
      { "id-HSDSCH-TDD-Update-Information", "nbap.id_HSDSCH_TDD_Update_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HSDSCH_TDD_Update_Information", HFILL }},
    { &hf_nbap_id_DL_DPCH_TimeSlotFormat_LCR_ModifyItem_RL_ReconfPrepTDD,
      { "id-DL-DPCH-TimeSlotFormat-LCR-ModifyItem-RL-ReconfPrepTDD", "nbap.id_DL_DPCH_TimeSlotFormat_LCR_ModifyItem_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, VALS(nbap_TDD_DL_DPCH_TimeSlotFormat_LCR_vals), 0,
        "nbap.TDD_DL_DPCH_TimeSlotFormat_LCR", HFILL }},
    { &hf_nbap_id_UL_DPCH_TimeSlotFormat_LCR_ModifyItem_RL_ReconfPrepTDD,
      { "id-UL-DPCH-TimeSlotFormat-LCR-ModifyItem-RL-ReconfPrepTDD", "nbap.id_UL_DPCH_TimeSlotFormat_LCR_ModifyItem_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, VALS(nbap_TDD_UL_DPCH_TimeSlotFormat_LCR_vals), 0,
        "nbap.TDD_UL_DPCH_TimeSlotFormat_LCR", HFILL }},
    { &hf_nbap_id_TDD_TPC_UplinkStepSize_LCR_RL_SetupRqstTDD,
      { "id-TDD-TPC-UplinkStepSize-LCR-RL-SetupRqstTDD", "nbap.id_TDD_TPC_UplinkStepSize_LCR_RL_SetupRqstTDD",
        FT_UINT32, BASE_DEC, VALS(nbap_TDD_TPC_UplinkStepSize_LCR_vals), 0,
        "nbap.TDD_TPC_UplinkStepSize_LCR", HFILL }},
    { &hf_nbap_id_TDD_TPC_UplinkStepSize_LCR_RL_AdditionRqstTDD,
      { "id-TDD-TPC-UplinkStepSize-LCR-RL-AdditionRqstTDD", "nbap.id_TDD_TPC_UplinkStepSize_LCR_RL_AdditionRqstTDD",
        FT_UINT32, BASE_DEC, VALS(nbap_TDD_TPC_UplinkStepSize_LCR_vals), 0,
        "nbap.TDD_TPC_UplinkStepSize_LCR", HFILL }},
    { &hf_nbap_id_TDD_TPC_DownlinkStepSize_RL_AdditionRqstTDD,
      { "id-TDD-TPC-DownlinkStepSize-RL-AdditionRqstTDD", "nbap.id_TDD_TPC_DownlinkStepSize_RL_AdditionRqstTDD",
        FT_UINT32, BASE_DEC, VALS(nbap_TDD_TPC_DownlinkStepSize_vals), 0,
        "nbap.TDD_TPC_DownlinkStepSize", HFILL }},
    { &hf_nbap_id_TDD_TPC_UplinkStepSize_InformationAdd_LCR_RL_ReconfPrepTDD,
      { "id-TDD-TPC-UplinkStepSize-InformationAdd-LCR-RL-ReconfPrepTDD", "nbap.id_TDD_TPC_UplinkStepSize_InformationAdd_LCR_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, VALS(nbap_TDD_TPC_UplinkStepSize_LCR_vals), 0,
        "nbap.TDD_TPC_UplinkStepSize_LCR", HFILL }},
    { &hf_nbap_id_TDD_TPC_UplinkStepSize_InformationModify_LCR_RL_ReconfPrepTDD,
      { "id-TDD-TPC-UplinkStepSize-InformationModify-LCR-RL-ReconfPrepTDD", "nbap.id_TDD_TPC_UplinkStepSize_InformationModify_LCR_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, VALS(nbap_TDD_TPC_UplinkStepSize_LCR_vals), 0,
        "nbap.TDD_TPC_UplinkStepSize_LCR", HFILL }},
    { &hf_nbap_id_TDD_TPC_DownlinkStepSize_InformationModify_RL_ReconfPrepTDD,
      { "id-TDD-TPC-DownlinkStepSize-InformationModify-RL-ReconfPrepTDD", "nbap.id_TDD_TPC_DownlinkStepSize_InformationModify_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, VALS(nbap_TDD_TPC_DownlinkStepSize_vals), 0,
        "nbap.TDD_TPC_DownlinkStepSize", HFILL }},
    { &hf_nbap_id_TDD_TPC_DownlinkStepSize_InformationAdd_RL_ReconfPrepTDD,
      { "id-TDD-TPC-DownlinkStepSize-InformationAdd-RL-ReconfPrepTDD", "nbap.id_TDD_TPC_DownlinkStepSize_InformationAdd_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, VALS(nbap_TDD_TPC_DownlinkStepSize_vals), 0,
        "nbap.TDD_TPC_DownlinkStepSize", HFILL }},
    { &hf_nbap_id_CCTrCH_Maximum_DL_Power_RL_SetupRqstTDD,
      { "id-CCTrCH-Maximum-DL-Power-RL-SetupRqstTDD", "nbap.id_CCTrCH_Maximum_DL_Power_RL_SetupRqstTDD",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_id_CCTrCH_Minimum_DL_Power_RL_SetupRqstTDD,
      { "id-CCTrCH-Minimum-DL-Power-RL-SetupRqstTDD", "nbap.id_CCTrCH_Minimum_DL_Power_RL_SetupRqstTDD",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_id_CCTrCH_Maximum_DL_Power_RL_AdditionRqstTDD,
      { "id-CCTrCH-Maximum-DL-Power-RL-AdditionRqstTDD", "nbap.id_CCTrCH_Maximum_DL_Power_RL_AdditionRqstTDD",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_id_CCTrCH_Minimum_DL_Power_RL_AdditionRqstTDD,
      { "id-CCTrCH-Minimum-DL-Power-RL-AdditionRqstTDD", "nbap.id_CCTrCH_Minimum_DL_Power_RL_AdditionRqstTDD",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_id_CCTrCH_Maximum_DL_Power_InformationAdd_RL_ReconfPrepTDD,
      { "id-CCTrCH-Maximum-DL-Power-InformationAdd-RL-ReconfPrepTDD", "nbap.id_CCTrCH_Maximum_DL_Power_InformationAdd_RL_ReconfPrepTDD",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_id_CCTrCH_Minimum_DL_Power_InformationAdd_RL_ReconfPrepTDD,
      { "id-CCTrCH-Minimum-DL-Power-InformationAdd-RL-ReconfPrepTDD", "nbap.id_CCTrCH_Minimum_DL_Power_InformationAdd_RL_ReconfPrepTDD",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_id_CCTrCH_Maximum_DL_Power_InformationModify_RL_ReconfPrepTDD,
      { "id-CCTrCH-Maximum-DL-Power-InformationModify-RL-ReconfPrepTDD", "nbap.id_CCTrCH_Maximum_DL_Power_InformationModify_RL_ReconfPrepTDD",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_id_CCTrCH_Minimum_DL_Power_InformationModify_RL_ReconfPrepTDD,
      { "id-CCTrCH-Minimum-DL-Power-InformationModify-RL-ReconfPrepTDD", "nbap.id_CCTrCH_Minimum_DL_Power_InformationModify_RL_ReconfPrepTDD",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_id_Maximum_DL_Power_Modify_LCR_InformationModify_RL_ReconfPrepTDD,
      { "id-Maximum-DL-Power-Modify-LCR-InformationModify-RL-ReconfPrepTDD", "nbap.id_Maximum_DL_Power_Modify_LCR_InformationModify_RL_ReconfPrepTDD",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_id_Minimum_DL_Power_Modify_LCR_InformationModify_RL_ReconfPrepTDD,
      { "id-Minimum-DL-Power-Modify-LCR-InformationModify-RL-ReconfPrepTDD", "nbap.id_Minimum_DL_Power_Modify_LCR_InformationModify_RL_ReconfPrepTDD",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_id_DL_DPCH_LCR_InformationModify_ModifyList_RL_ReconfRqstTDD,
      { "id-DL-DPCH-LCR-InformationModify-ModifyList-RL-ReconfRqstTDD", "nbap.id_DL_DPCH_LCR_InformationModify_ModifyList_RL_ReconfRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_DPCH_LCR_InformationModify_ModifyList_RL_ReconfRqstTDD", HFILL }},
    { &hf_nbap_id_CCTrCH_Maximum_DL_Power_InformationModify_RL_ReconfRqstTDD,
      { "id-CCTrCH-Maximum-DL-Power-InformationModify-RL-ReconfRqstTDD", "nbap.id_CCTrCH_Maximum_DL_Power_InformationModify_RL_ReconfRqstTDD",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_id_CCTrCH_Minimum_DL_Power_InformationModify_RL_ReconfRqstTDD,
      { "id-CCTrCH-Minimum-DL-Power-InformationModify-RL-ReconfRqstTDD", "nbap.id_CCTrCH_Minimum_DL_Power_InformationModify_RL_ReconfRqstTDD",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_id_Initial_DL_Power_TimeslotLCR_InformationItem,
      { "id-Initial-DL-Power-TimeslotLCR-InformationItem", "nbap.id_Initial_DL_Power_TimeslotLCR_InformationItem",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_id_Maximum_DL_Power_TimeslotLCR_InformationItem,
      { "id-Maximum-DL-Power-TimeslotLCR-InformationItem", "nbap.id_Maximum_DL_Power_TimeslotLCR_InformationItem",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_id_Minimum_DL_Power_TimeslotLCR_InformationItem,
      { "id-Minimum-DL-Power-TimeslotLCR-InformationItem", "nbap.id_Minimum_DL_Power_TimeslotLCR_InformationItem",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_id_HS_DSCHProvidedBitRateValueInformation,
      { "id-HS-DSCHProvidedBitRateValueInformation", "nbap.id_HS_DSCHProvidedBitRateValueInformation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HS_DSCHProvidedBitRate", HFILL }},
    { &hf_nbap_id_HS_DSCHRequiredPowerValueInformation,
      { "id-HS-DSCHRequiredPowerValueInformation", "nbap.id_HS_DSCHRequiredPowerValueInformation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HS_DSCHRequiredPower", HFILL }},
    { &hf_nbap_id_HS_DSCHRequiredPowerValue,
      { "id-HS-DSCHRequiredPowerValue", "nbap.id_HS_DSCHRequiredPowerValue",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HS_DSCHRequiredPowerValue", HFILL }},
    { &hf_nbap_id_TransmittedCarrierPowerOfAllCodesNotUsedForHSTransmission,
      { "id-TransmittedCarrierPowerOfAllCodesNotUsedForHSTransmission", "nbap.id_TransmittedCarrierPowerOfAllCodesNotUsedForHSTransmission",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.TransmittedCarrierPowerOfAllCodesNotUsedForHSTransmissionValue", HFILL }},
    { &hf_nbap_id_HS_SICH_Reception_Quality,
      { "id-HS-SICH-Reception-Quality", "nbap.id_HS_SICH_Reception_Quality",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HS_SICH_Reception_Quality_Value", HFILL }},
    { &hf_nbap_id_HS_SICH_Reception_Quality_Measurement_Value,
      { "id-HS-SICH-Reception-Quality-Measurement-Value", "nbap.id_HS_SICH_Reception_Quality_Measurement_Value",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HS_SICH_Reception_Quality_Measurement_Value", HFILL }},
    { &hf_nbap_id_HSSICH_Info_DM_Rprt,
      { "id-HSSICH-Info-DM-Rprt", "nbap.id_HSSICH_Info_DM_Rprt",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HS_SICH_ID", HFILL }},
    { &hf_nbap_id_HSSICH_Info_DM_Rqst,
      { "id-HSSICH-Info-DM-Rqst", "nbap.id_HSSICH_Info_DM_Rqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HSSICH_Info_DM_Rqst", HFILL }},
    { &hf_nbap_id_HSSICH_Info_DM_Rsp,
      { "id-HSSICH-Info-DM-Rsp", "nbap.id_HSSICH_Info_DM_Rsp",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HS_SICH_ID", HFILL }},
    { &hf_nbap_id_Best_Cell_Portions_Value,
      { "id-Best-Cell-Portions-Value", "nbap.id_Best_Cell_Portions_Value",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Best_Cell_Portions_Value", HFILL }},
    { &hf_nbap_id_Primary_CPICH_Usage_for_Channel_Estimation,
      { "id-Primary-CPICH-Usage-for-Channel-Estimation", "nbap.id_Primary_CPICH_Usage_for_Channel_Estimation",
        FT_UINT32, BASE_DEC, VALS(nbap_Primary_CPICH_Usage_for_Channel_Estimation_vals), 0,
        "nbap.Primary_CPICH_Usage_for_Channel_Estimation", HFILL }},
    { &hf_nbap_id_Secondary_CPICH_Information_Change,
      { "id-Secondary-CPICH-Information-Change", "nbap.id_Secondary_CPICH_Information_Change",
        FT_UINT32, BASE_DEC, VALS(nbap_Secondary_CPICH_Information_Change_vals), 0,
        "nbap.Secondary_CPICH_Information_Change", HFILL }},
    { &hf_nbap_id_NumberOfReportedCellPortions,
      { "id-NumberOfReportedCellPortions", "nbap.id_NumberOfReportedCellPortions",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.NumberOfReportedCellPortions", HFILL }},
    { &hf_nbap_id_CellPortion_InformationItem_Cell_SetupRqstFDD,
      { "id-CellPortion-InformationItem-Cell-SetupRqstFDD", "nbap.id_CellPortion_InformationItem_Cell_SetupRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CellPortion_InformationItem_Cell_SetupRqstFDD", HFILL }},
    { &hf_nbap_id_CellPortion_InformationList_Cell_SetupRqstFDD,
      { "id-CellPortion-InformationList-Cell-SetupRqstFDD", "nbap.id_CellPortion_InformationList_Cell_SetupRqstFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CellPortion_InformationList_Cell_SetupRqstFDD", HFILL }},
    { &hf_nbap_id_TimeslotISCP_LCR_InfoList_RL_ReconfPrepTDD,
      { "id-TimeslotISCP-LCR-InfoList-RL-ReconfPrepTDD", "nbap.id_TimeslotISCP_LCR_InfoList_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_TimeslotISCPInfoLCR", HFILL }},
    { &hf_nbap_id_Secondary_CPICH_Information,
      { "id-Secondary-CPICH-Information", "nbap.id_Secondary_CPICH_Information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CommonPhysicalChannelID", HFILL }},
    { &hf_nbap_id_Received_total_wide_band_power_For_CellPortion,
      { "id-Received-total-wide-band-power-For-CellPortion", "nbap.id_Received_total_wide_band_power_For_CellPortion",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Received_total_wide_band_power_Value_IncrDecrThres", HFILL }},
    { &hf_nbap_id_Unidirectional_DCH_Indicator,
      { "id-Unidirectional-DCH-Indicator", "nbap.id_Unidirectional_DCH_Indicator",
        FT_UINT32, BASE_DEC, VALS(nbap_Unidirectional_DCH_Indicator_vals), 0,
        "nbap.Unidirectional_DCH_Indicator", HFILL }},
    { &hf_nbap_id_TimingAdjustmentValueLCR,
      { "id-TimingAdjustmentValueLCR", "nbap.id_TimingAdjustmentValueLCR",
        FT_UINT32, BASE_DEC, VALS(nbap_TimingAdjustmentValueLCR_vals), 0,
        "nbap.TimingAdjustmentValueLCR", HFILL }},
    { &hf_nbap_id_multipleRL_ul_DPCH_InformationList,
      { "id-multipleRL-ul-DPCH-InformationList", "nbap.id_multipleRL_ul_DPCH_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.MultipleRL_UL_DPCH_InformationAddList_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_id_multipleRL_ul_DPCH_InformationModifyList,
      { "id-multipleRL-ul-DPCH-InformationModifyList", "nbap.id_multipleRL_ul_DPCH_InformationModifyList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.MultipleRL_UL_DPCH_InformationModifyList_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_id_RL_ID,
      { "id-RL-ID", "nbap.id_RL_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.RL_ID", HFILL }},
    { &hf_nbap_id_SAT_Info_Almanac_ExtItem,
      { "id-SAT-Info-Almanac-ExtItem", "nbap.id_SAT_Info_Almanac_ExtItem",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.SAT_Info_Almanac_ExtItem", HFILL }},
    { &hf_nbap_id_HSDPA_Capability,
      { "id-HSDPA-Capability", "nbap.id_HSDPA_Capability",
        FT_UINT32, BASE_DEC, VALS(nbap_HSDPA_Capability_vals), 0,
        "nbap.HSDPA_Capability", HFILL }},
    { &hf_nbap_id_HSDSCH_Resources_Information_AuditRsp,
      { "id-HSDSCH-Resources-Information-AuditRsp", "nbap.id_HSDSCH_Resources_Information_AuditRsp",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HS_DSCH_Resources_Information_AuditRsp", HFILL }},
    { &hf_nbap_id_HSDSCH_Resources_Information_ResourceStatusInd,
      { "id-HSDSCH-Resources-Information-ResourceStatusInd", "nbap.id_HSDSCH_Resources_Information_ResourceStatusInd",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HS_DSCH_Resources_Information_ResourceStatusInd", HFILL }},
    { &hf_nbap_id_HSDSCH_MACdFlows_to_Add,
      { "id-HSDSCH-MACdFlows-to-Add", "nbap.id_HSDSCH_MACdFlows_to_Add",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HSDSCH_MACdFlows_Information", HFILL }},
    { &hf_nbap_id_HSDSCH_MACdFlows_to_Delete,
      { "id-HSDSCH-MACdFlows-to-Delete", "nbap.id_HSDSCH_MACdFlows_to_Delete",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HSDSCH_MACdFlows_to_Delete", HFILL }},
    { &hf_nbap_id_HSDSCH_Information_to_Modify_Unsynchronised,
      { "id-HSDSCH-Information-to-Modify-Unsynchronised", "nbap.id_HSDSCH_Information_to_Modify_Unsynchronised",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HSDSCH_Information_to_Modify_Unsynchronised", HFILL }},
    { &hf_nbap_id_TnlQos,
      { "id-TnlQos", "nbap.id_TnlQos",
        FT_UINT32, BASE_DEC, VALS(nbap_TnlQos_vals), 0,
        "nbap.TnlQos", HFILL }},
    { &hf_nbap_id_Received_total_wide_band_power_For_CellPortion_Value,
      { "id-Received-total-wide-band-power-For-CellPortion-Value", "nbap.id_Received_total_wide_band_power_For_CellPortion_Value",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Received_total_wide_band_power_For_CellPortion_Value", HFILL }},
    { &hf_nbap_id_Transmitted_Carrier_Power_For_CellPortion,
      { "id-Transmitted-Carrier-Power-For-CellPortion", "nbap.id_Transmitted_Carrier_Power_For_CellPortion",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Transmitted_Carrier_Power_Value", HFILL }},
    { &hf_nbap_id_Transmitted_Carrier_Power_For_CellPortion_Value,
      { "id-Transmitted-Carrier-Power-For-CellPortion-Value", "nbap.id_Transmitted_Carrier_Power_For_CellPortion_Value",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Transmitted_Carrier_Power_For_CellPortion_Value", HFILL }},
    { &hf_nbap_id_TransmittedCarrierPowerOfAllCodesNotUsedForHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHOrE_HICHTransmissionCellPortion,
      { "id-TransmittedCarrierPowerOfAllCodesNotUsedForHS-PDSCH-HS-SCCH-E-AGCH-E-RGCHOrE-HICHTransmissionCellPortion", "nbap.id_TransmittedCarrierPowerOfAllCodesNotUsedForHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHOrE_HICHTransmissionCellPortion",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.TransmittedCarrierPowerOfAllCodesNotUsedForHSTransmissionValue", HFILL }},
    { &hf_nbap_id_TransmittedCarrierPowerOfAllCodesNotUsedForHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHOrE_HICHTransmissionCellPortionValue,
      { "id-TransmittedCarrierPowerOfAllCodesNotUsedForHS-PDSCH-HS-SCCH-E-AGCH-E-RGCHOrE-HICHTransmissionCellPortionValue", "nbap.id_TransmittedCarrierPowerOfAllCodesNotUsedForHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHOrE_HICHTransmissionCellPortionValue",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.TransmittedCarrierPowerOfAllCodesNotUsedForHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHOrE_HICHTransmissionCellPortionValue", HFILL }},
    { &hf_nbap_id_UpPTSInterferenceValue,
      { "id-UpPTSInterferenceValue", "nbap.id_UpPTSInterferenceValue",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.UpPTSInterferenceValue", HFILL }},
    { &hf_nbap_id_PrimaryCCPCH_RSCP_Delta,
      { "id-PrimaryCCPCH-RSCP-Delta", "nbap.id_PrimaryCCPCH_RSCP_Delta",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.PrimaryCCPCH_RSCP_Delta", HFILL }},
    { &hf_nbap_id_MeasurementRecoveryBehavior,
      { "id-MeasurementRecoveryBehavior", "nbap.id_MeasurementRecoveryBehavior",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.MeasurementRecoveryBehavior", HFILL }},
    { &hf_nbap_id_MeasurementRecoveryReportingIndicator,
      { "id-MeasurementRecoveryReportingIndicator", "nbap.id_MeasurementRecoveryReportingIndicator",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.MeasurementRecoveryReportingIndicator", HFILL }},
    { &hf_nbap_id_MeasurementRecoverySupportIndicator,
      { "id-MeasurementRecoverySupportIndicator", "nbap.id_MeasurementRecoverySupportIndicator",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.MeasurementRecoverySupportIndicator", HFILL }},
    { &hf_nbap_id_Tstd_indicator,
      { "id-Tstd-indicator", "nbap.id_Tstd_indicator",
        FT_UINT32, BASE_DEC, VALS(nbap_TSTD_Indicator_vals), 0,
        "nbap.TSTD_Indicator", HFILL }},
    { &hf_nbap_id_multiple_RL_Information_RL_ReconfPrepTDD,
      { "id-multiple-RL-Information-RL-ReconfPrepTDD", "nbap.id_multiple_RL_Information_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.MultipleRL_Information_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_id_multiple_RL_Information_RL_ReconfRqstTDD,
      { "id-multiple-RL-Information-RL-ReconfRqstTDD", "nbap.id_multiple_RL_Information_RL_ReconfRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Multiple_RL_Information_RL_ReconfRqstTDD", HFILL }},
    { &hf_nbap_id_DL_DPCH_Power_Information_RL_ReconfPrepFDD,
      { "id-DL-DPCH-Power-Information-RL-ReconfPrepFDD", "nbap.id_DL_DPCH_Power_Information_RL_ReconfPrepFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_DPCH_Power_Information_RL_ReconfPrepFDD", HFILL }},
    { &hf_nbap_id_F_DPCH_Information_RL_ReconfPrepFDD,
      { "id-F-DPCH-Information-RL-ReconfPrepFDD", "nbap.id_F_DPCH_Information_RL_ReconfPrepFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.F_DPCH_Information_RL_ReconfPrepFDD", HFILL }},
    { &hf_nbap_id_F_DPCH_Information_RL_SetupRqstFDD,
      { "id-F-DPCH-Information-RL-SetupRqstFDD", "nbap.id_F_DPCH_Information_RL_SetupRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.F_DPCH_Information_RL_SetupRqstFDD", HFILL }},
    { &hf_nbap_id_Additional_S_CCPCH_Parameters_CTCH_ReconfRqstTDD,
      { "id-Additional-S-CCPCH-Parameters-CTCH-ReconfRqstTDD", "nbap.id_Additional_S_CCPCH_Parameters_CTCH_ReconfRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Secondary_CCPCH_parameterExtendedList_CTCH_ReconfRqstTDD", HFILL }},
    { &hf_nbap_id_Additional_S_CCPCH_Parameters_CTCH_SetupRqstTDD,
      { "id-Additional-S-CCPCH-Parameters-CTCH-SetupRqstTDD", "nbap.id_Additional_S_CCPCH_Parameters_CTCH_SetupRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Secondary_CCPCH_parameterExtendedList_CTCH_SetupRqstTDD", HFILL }},
    { &hf_nbap_id_Additional_S_CCPCH_LCR_Parameters_CTCH_ReconfRqstTDD,
      { "id-Additional-S-CCPCH-LCR-Parameters-CTCH-ReconfRqstTDD", "nbap.id_Additional_S_CCPCH_LCR_Parameters_CTCH_ReconfRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Secondary_CCPCH_LCR_parameterExtendedList_CTCH_ReconfRqstTDD", HFILL }},
    { &hf_nbap_id_Additional_S_CCPCH_LCR_Parameters_CTCH_SetupRqstTDD,
      { "id-Additional-S-CCPCH-LCR-Parameters-CTCH-SetupRqstTDD", "nbap.id_Additional_S_CCPCH_LCR_Parameters_CTCH_SetupRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Secondary_CCPCH_LCR_parameterExtendedList_CTCH_SetupRqstTDD", HFILL }},
    { &hf_nbap_id_MICH_CFN,
      { "id-MICH-CFN", "nbap.id_MICH_CFN",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.MICH_CFN", HFILL }},
    { &hf_nbap_id_MICH_Information_AuditRsp,
      { "id-MICH-Information-AuditRsp", "nbap.id_MICH_Information_AuditRsp",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Common_PhysicalChannel_Status_Information", HFILL }},
    { &hf_nbap_id_MICH_Information_ResourceStatusInd,
      { "id-MICH-Information-ResourceStatusInd", "nbap.id_MICH_Information_ResourceStatusInd",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Common_PhysicalChannel_Status_Information", HFILL }},
    { &hf_nbap_id_MICH_Parameters_CTCH_ReconfRqstFDD,
      { "id-MICH-Parameters-CTCH-ReconfRqstFDD", "nbap.id_MICH_Parameters_CTCH_ReconfRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.MICH_Parameters_CTCH_ReconfRqstFDD", HFILL }},
    { &hf_nbap_id_MICH_Parameters_CTCH_ReconfRqstTDD,
      { "id-MICH-Parameters-CTCH-ReconfRqstTDD", "nbap.id_MICH_Parameters_CTCH_ReconfRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.MICH_Parameters_CTCH_ReconfRqstTDD", HFILL }},
    { &hf_nbap_id_MICH_Parameters_CTCH_SetupRqstFDD,
      { "id-MICH-Parameters-CTCH-SetupRqstFDD", "nbap.id_MICH_Parameters_CTCH_SetupRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.MICH_Parameters_CTCH_SetupRqstFDD", HFILL }},
    { &hf_nbap_id_MICH_Parameters_CTCH_SetupRqstTDD,
      { "id-MICH-Parameters-CTCH-SetupRqstTDD", "nbap.id_MICH_Parameters_CTCH_SetupRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.MICH_Parameters_CTCH_SetupRqstTDD", HFILL }},
    { &hf_nbap_id_Modification_Period,
      { "id-Modification-Period", "nbap.id_Modification_Period",
        FT_UINT32, BASE_DEC, VALS(nbap_Modification_Period_vals), 0,
        "nbap.Modification_Period", HFILL }},
    { &hf_nbap_id_NI_Information_NotifUpdateCmd,
      { "id-NI-Information-NotifUpdateCmd", "nbap.id_NI_Information_NotifUpdateCmd",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.NI_Information", HFILL }},
    { &hf_nbap_id_S_CCPCH_InformationListExt_AuditRsp,
      { "id-S-CCPCH-InformationListExt-AuditRsp", "nbap.id_S_CCPCH_InformationListExt_AuditRsp",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.S_CCPCH_InformationListExt_AuditRsp", HFILL }},
    { &hf_nbap_id_S_CCPCH_InformationListExt_ResourceStatusInd,
      { "id-S-CCPCH-InformationListExt-ResourceStatusInd", "nbap.id_S_CCPCH_InformationListExt_ResourceStatusInd",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.S_CCPCH_InformationListExt_ResourceStatusInd", HFILL }},
    { &hf_nbap_id_S_CCPCH_LCR_InformationListExt_AuditRsp,
      { "id-S-CCPCH-LCR-InformationListExt-AuditRsp", "nbap.id_S_CCPCH_LCR_InformationListExt_AuditRsp",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.S_CCPCH_LCR_InformationListExt_AuditRsp", HFILL }},
    { &hf_nbap_id_S_CCPCH_LCR_InformationListExt_ResourceStatusInd,
      { "id-S-CCPCH-LCR-InformationListExt-ResourceStatusInd", "nbap.id_S_CCPCH_LCR_InformationListExt_ResourceStatusInd",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.S_CCPCH_LCR_InformationListExt_ResourceStatusInd", HFILL }},
    { &hf_nbap_id_HARQ_Preamble_Mode,
      { "id-HARQ-Preamble-Mode", "nbap.id_HARQ_Preamble_Mode",
        FT_UINT32, BASE_DEC, VALS(nbap_HARQ_Preamble_Mode_vals), 0,
        "nbap.HARQ_Preamble_Mode", HFILL }},
    { &hf_nbap_id_Initial_DL_DPCH_TimingAdjustment,
      { "id-Initial-DL-DPCH-TimingAdjustment", "nbap.id_Initial_DL_DPCH_TimingAdjustment",
        FT_UINT32, BASE_DEC, VALS(nbap_DL_DPCH_TimingAdjustment_vals), 0,
        "nbap.DL_DPCH_TimingAdjustment", HFILL }},
    { &hf_nbap_id_Initial_DL_DPCH_TimingAdjustment_Allowed,
      { "id-Initial-DL-DPCH-TimingAdjustment-Allowed", "nbap.id_Initial_DL_DPCH_TimingAdjustment_Allowed",
        FT_UINT32, BASE_DEC, VALS(nbap_Initial_DL_DPCH_TimingAdjustment_Allowed_vals), 0,
        "nbap.Initial_DL_DPCH_TimingAdjustment_Allowed", HFILL }},
    { &hf_nbap_id_DLTransmissionBranchLoadValue,
      { "id-DLTransmissionBranchLoadValue", "nbap.id_DLTransmissionBranchLoadValue",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DLTransmissionBranchLoadValue", HFILL }},
    { &hf_nbap_id_Power_Local_Cell_Group_choice_CM_Rqst,
      { "id-Power-Local-Cell-Group-choice-CM-Rqst", "nbap.id_Power_Local_Cell_Group_choice_CM_Rqst",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PowerLocalCellGroup_CM_Rqst", HFILL }},
    { &hf_nbap_id_Power_Local_Cell_Group_choice_CM_Rsp,
      { "id-Power-Local-Cell-Group-choice-CM-Rsp", "nbap.id_Power_Local_Cell_Group_choice_CM_Rsp",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PowerLocalCellGroup_CM_Rsp", HFILL }},
    { &hf_nbap_id_Power_Local_Cell_Group_choice_CM_Rprt,
      { "id-Power-Local-Cell-Group-choice-CM-Rprt", "nbap.id_Power_Local_Cell_Group_choice_CM_Rprt",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PowerLocalCellGroup_CM_Rprt", HFILL }},
    { &hf_nbap_id_SynchronisationIndicator,
      { "id-SynchronisationIndicator", "nbap.id_SynchronisationIndicator",
        FT_UINT32, BASE_DEC, VALS(nbap_SynchronisationIndicator_vals), 0,
        "nbap.SynchronisationIndicator", HFILL }},
    { &hf_nbap_id_HSDPA_And_EDCH_CellPortion_Information_PSCH_ReconfRqst,
      { "id-HSDPA-And-EDCH-CellPortion-Information-PSCH-ReconfRqst", "nbap.id_HSDPA_And_EDCH_CellPortion_Information_PSCH_ReconfRqst",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HSDPA_And_EDCH_CellPortion_Information_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_id_HSDPA_And_EDCH_CellPortion_InformationList_PSCH_ReconfRqst,
      { "id-HSDPA-And-EDCH-CellPortion-InformationList-PSCH-ReconfRqst", "nbap.id_HSDPA_And_EDCH_CellPortion_InformationList_PSCH_ReconfRqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HSDPA_And_EDCH_CellPortion_InformationList_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_id_HS_DSCHRequiredPowerValue_For_Cell_Portion,
      { "id-HS-DSCHRequiredPowerValue-For-Cell-Portion", "nbap.id_HS_DSCHRequiredPowerValue_For_Cell_Portion",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HS_DSCHRequiredPowerValue", HFILL }},
    { &hf_nbap_id_HS_DSCHRequiredPowerValueInformation_For_CellPortion,
      { "id-HS-DSCHRequiredPowerValueInformation-For-CellPortion", "nbap.id_HS_DSCHRequiredPowerValueInformation_For_CellPortion",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HS_DSCHRequiredPowerValueInformation_For_CellPortion", HFILL }},
    { &hf_nbap_id_HS_DSCHProvidedBitRateValueInformation_For_CellPortion,
      { "id-HS-DSCHProvidedBitRateValueInformation-For-CellPortion", "nbap.id_HS_DSCHProvidedBitRateValueInformation_For_CellPortion",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HS_DSCHProvidedBitRateValueInformation_For_CellPortion", HFILL }},
    { &hf_nbap_id_E_AGCH_And_E_RGCH_E_HICH_FDD_Scrambling_Code,
      { "id-E-AGCH-And-E-RGCH-E-HICH-FDD-Scrambling-Code", "nbap.id_E_AGCH_And_E_RGCH_E_HICH_FDD_Scrambling_Code",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_ScramblingCode", HFILL }},
    { &hf_nbap_id_E_AGCH_FDD_Code_Information,
      { "id-E-AGCH-FDD-Code-Information", "nbap.id_E_AGCH_FDD_Code_Information",
        FT_UINT32, BASE_DEC, VALS(nbap_E_AGCH_FDD_Code_Information_vals), 0,
        "nbap.E_AGCH_FDD_Code_Information", HFILL }},
    { &hf_nbap_id_E_DCH_Capability,
      { "id-E-DCH-Capability", "nbap.id_E_DCH_Capability",
        FT_UINT32, BASE_DEC, VALS(nbap_E_DCH_Capability_vals), 0,
        "nbap.E_DCH_Capability", HFILL }},
    { &hf_nbap_id_E_DCH_FDD_DL_Control_Channel_Information,
      { "id-E-DCH-FDD-DL-Control-Channel-Information", "nbap.id_E_DCH_FDD_DL_Control_Channel_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.E_DCH_FDD_DL_Control_Channel_Information", HFILL }},
    { &hf_nbap_id_E_DCH_FDD_Information,
      { "id-E-DCH-FDD-Information", "nbap.id_E_DCH_FDD_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.E_DCH_FDD_Information", HFILL }},
    { &hf_nbap_id_E_DCH_FDD_Information_Response,
      { "id-E-DCH-FDD-Information-Response", "nbap.id_E_DCH_FDD_Information_Response",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.E_DCH_FDD_Information_Response", HFILL }},
    { &hf_nbap_id_E_DCH_FDD_Information_to_Modify,
      { "id-E-DCH-FDD-Information-to-Modify", "nbap.id_E_DCH_FDD_Information_to_Modify",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.E_DCH_FDD_Information_to_Modify", HFILL }},
    { &hf_nbap_id_E_DCH_MACdFlows_to_Add,
      { "id-E-DCH-MACdFlows-to-Add", "nbap.id_E_DCH_MACdFlows_to_Add",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.E_DCH_MACdFlows_Information", HFILL }},
    { &hf_nbap_id_E_DCH_MACdFlows_to_Delete,
      { "id-E-DCH-MACdFlows-to-Delete", "nbap.id_E_DCH_MACdFlows_to_Delete",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.E_DCH_MACdFlows_to_Delete", HFILL }},
    { &hf_nbap_id_E_DCH_Resources_Information_AuditRsp,
      { "id-E-DCH-Resources-Information-AuditRsp", "nbap.id_E_DCH_Resources_Information_AuditRsp",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.E_DCH_Resources_Information_AuditRsp", HFILL }},
    { &hf_nbap_id_E_DCH_Resources_Information_ResourceStatusInd,
      { "id-E-DCH-Resources-Information-ResourceStatusInd", "nbap.id_E_DCH_Resources_Information_ResourceStatusInd",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.E_DCH_Resources_Information_ResourceStatusInd", HFILL }},
    { &hf_nbap_id_E_DCH_RL_Indication,
      { "id-E-DCH-RL-Indication", "nbap.id_E_DCH_RL_Indication",
        FT_UINT32, BASE_DEC, VALS(nbap_E_DCH_RL_Indication_vals), 0,
        "nbap.E_DCH_RL_Indication", HFILL }},
    { &hf_nbap_id_E_DCH_RL_Set_ID,
      { "id-E-DCH-RL-Set-ID", "nbap.id_E_DCH_RL_Set_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.RL_Set_ID", HFILL }},
    { &hf_nbap_id_E_DPCH_Information_RL_ReconfPrepFDD,
      { "id-E-DPCH-Information-RL-ReconfPrepFDD", "nbap.id_E_DPCH_Information_RL_ReconfPrepFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.E_DPCH_Information_RL_ReconfPrepFDD", HFILL }},
    { &hf_nbap_id_E_DPCH_Information_RL_SetupRqstFDD,
      { "id-E-DPCH-Information-RL-SetupRqstFDD", "nbap.id_E_DPCH_Information_RL_SetupRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.E_DPCH_Information_RL_SetupRqstFDD", HFILL }},
    { &hf_nbap_id_E_RGCH_E_HICH_FDD_Code_Information,
      { "id-E-RGCH-E-HICH-FDD-Code-Information", "nbap.id_E_RGCH_E_HICH_FDD_Code_Information",
        FT_UINT32, BASE_DEC, VALS(nbap_E_RGCH_E_HICH_FDD_Code_Information_vals), 0,
        "nbap.E_RGCH_E_HICH_FDD_Code_Information", HFILL }},
    { &hf_nbap_id_Serving_E_DCH_RL_ID,
      { "id-Serving-E-DCH-RL-ID", "nbap.id_Serving_E_DCH_RL_ID",
        FT_UINT32, BASE_DEC, VALS(nbap_Serving_E_DCH_RL_ID_vals), 0,
        "nbap.Serving_E_DCH_RL_ID", HFILL }},
    { &hf_nbap_id_UL_DPDCH_Indicator_For_E_DCH_Operation,
      { "id-UL-DPDCH-Indicator-For-E-DCH-Operation", "nbap.id_UL_DPDCH_Indicator_For_E_DCH_Operation",
        FT_UINT32, BASE_DEC, VALS(nbap_UL_DPDCH_Indicator_For_E_DCH_Operation_vals), 0,
        "nbap.UL_DPDCH_Indicator_For_E_DCH_Operation", HFILL }},
    { &hf_nbap_id_FDD_S_CCPCH_FrameOffset_CTCH_SetupRqstFDD,
      { "id-FDD-S-CCPCH-FrameOffset-CTCH-SetupRqstFDD", "nbap.id_FDD_S_CCPCH_FrameOffset_CTCH_SetupRqstFDD",
        FT_UINT32, BASE_DEC, VALS(nbap_FDD_S_CCPCH_FrameOffset_vals), 0,
        "nbap.FDD_S_CCPCH_FrameOffset", HFILL }},
    { &hf_nbap_id_E_DPCH_Information_RL_ReconfRqstFDD,
      { "id-E-DPCH-Information-RL-ReconfRqstFDD", "nbap.id_E_DPCH_Information_RL_ReconfRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.E_DPCH_Information_RL_ReconfRqstFDD", HFILL }},
    { &hf_nbap_id_Maximum_Target_ReceivedTotalWideBandPower,
      { "id-Maximum-Target-ReceivedTotalWideBandPower", "nbap.id_Maximum_Target_ReceivedTotalWideBandPower",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Maximum_Target_ReceivedTotalWideBandPower", HFILL }},
    { &hf_nbap_id_E_DCHProvidedBitRateValueInformation,
      { "id-E-DCHProvidedBitRateValueInformation", "nbap.id_E_DCHProvidedBitRateValueInformation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.E_DCHProvidedBitRate", HFILL }},
    { &hf_nbap_id_HARQ_Preamble_Mode_Activation_Indicator,
      { "id-HARQ-Preamble-Mode-Activation-Indicator", "nbap.id_HARQ_Preamble_Mode_Activation_Indicator",
        FT_UINT32, BASE_DEC, VALS(nbap_HARQ_Preamble_Mode_Activation_Indicator_vals), 0,
        "nbap.HARQ_Preamble_Mode_Activation_Indicator", HFILL }},
    { &hf_nbap_id_RL_Specific_E_DCH_Info,
      { "id-RL-Specific-E-DCH-Info", "nbap.id_RL_Specific_E_DCH_Info",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.RL_Specific_E_DCH_Info", HFILL }},
    { &hf_nbap_id_E_DCH_CapacityConsumptionLaw,
      { "id-E-DCH-CapacityConsumptionLaw", "nbap.id_E_DCH_CapacityConsumptionLaw",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DedicatedChannelsCapacityConsumptionLaw", HFILL }},
    { &hf_nbap_id_multiple_DedicatedMeasurementValueList_TDD_DM_Rsp,
      { "id-multiple-DedicatedMeasurementValueList-TDD-DM-Rsp", "nbap.id_multiple_DedicatedMeasurementValueList_TDD_DM_Rsp",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Multiple_DedicatedMeasurementValueList_TDD_DM_Rsp", HFILL }},
    { &hf_nbap_id_multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp,
      { "id-multiple-DedicatedMeasurementValueList-LCR-TDD-DM-Rsp", "nbap.id_multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp", HFILL }},
    { &hf_nbap_id_E_DCH_RearrangeList_Bearer_RearrangeInd,
      { "id-E-DCH-RearrangeList-Bearer-RearrangeInd", "nbap.id_E_DCH_RearrangeList_Bearer_RearrangeInd",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.E_DCH_RearrangeList_Bearer_RearrangeInd", HFILL }},
    { &hf_nbap_id_multipleRL_dl_CCTrCH_InformationModifyList_RL_ReconfRqstTDD,
      { "id-multipleRL-dl-CCTrCH-InformationModifyList-RL-ReconfRqstTDD", "nbap.id_multipleRL_dl_CCTrCH_InformationModifyList_RL_ReconfRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.MultipleRL_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD", HFILL }},
    { &hf_nbap_id_Target_NonServing_EDCH_To_Total_EDCH_Power_Ratio,
      { "id-Target-NonServing-EDCH-To-Total-EDCH-Power-Ratio", "nbap.id_Target_NonServing_EDCH_To_Total_EDCH_Power_Ratio",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Target_NonServing_EDCH_To_Total_EDCH_Power_Ratio", HFILL }},
    { &hf_nbap_id_CellPortion_InformationItem_Cell_ReconfRqstFDD,
      { "id-CellPortion-InformationItem-Cell-ReconfRqstFDD", "nbap.id_CellPortion_InformationItem_Cell_ReconfRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CellPortion_InformationItem_Cell_ReconfRqstFDD", HFILL }},
    { &hf_nbap_id_CellPortion_InformationList_Cell_ReconfRqstFDD,
      { "id-CellPortion-InformationList-Cell-ReconfRqstFDD", "nbap.id_CellPortion_InformationList_Cell_ReconfRqstFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CellPortion_InformationList_Cell_ReconfRqstFDD", HFILL }},
    { &hf_nbap_id_multiple_PUSCH_InfoList_DM_Rsp,
      { "id-multiple-PUSCH-InfoList-DM-Rsp", "nbap.id_multiple_PUSCH_InfoList_DM_Rsp",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Multiple_PUSCH_InfoList_DM_Rsp", HFILL }},
    { &hf_nbap_id_multiple_PUSCH_InfoList_DM_Rprt,
      { "id-multiple-PUSCH-InfoList-DM-Rprt", "nbap.id_multiple_PUSCH_InfoList_DM_Rprt",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Multiple_PUSCH_InfoList_DM_Rprt", HFILL }},
    { &hf_nbap_id_Reference_ReceivedTotalWideBandPower,
      { "id-Reference-ReceivedTotalWideBandPower", "nbap.id_Reference_ReceivedTotalWideBandPower",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Reference_ReceivedTotalWideBandPower", HFILL }},
    { &hf_nbap_protocolIEs,
      { "protocolIEs", "nbap.protocolIEs",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.ProtocolIE_Container", HFILL }},
    { &hf_nbap_protocolExtensions,
      { "protocolExtensions", "nbap.protocolExtensions",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.ProtocolExtensionContainer", HFILL }},
    { &hf_nbap_secondary_CCPCH_parameters,
      { "secondary-CCPCH-parameters", "nbap.secondary_CCPCH_parameters",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Secondary_CCPCH_CTCH_SetupRqstFDD", HFILL }},
    { &hf_nbap_pRACH_parameters,
      { "pRACH-parameters", "nbap.pRACH_parameters",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PRACH_CTCH_SetupRqstFDD", HFILL }},
    { &hf_nbap_notUsed_pCPCHes_parameters,
      { "notUsed-pCPCHes-parameters", "nbap.notUsed_pCPCHes_parameters",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NULL", HFILL }},
    { &hf_nbap_commonPhysicalChannelID,
      { "commonPhysicalChannelID", "nbap.commonPhysicalChannelID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CommonPhysicalChannelID", HFILL }},
    { &hf_nbap_fdd_S_CCPCH_Offset,
      { "fdd-S-CCPCH-Offset", "nbap.fdd_S_CCPCH_Offset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.FDD_S_CCPCH_Offset", HFILL }},
    { &hf_nbap_dl_ScramblingCode,
      { "dl-ScramblingCode", "nbap.dl_ScramblingCode",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_ScramblingCode", HFILL }},
    { &hf_nbap_fdd_DL_ChannelisationCodeNumber,
      { "fdd-DL-ChannelisationCodeNumber", "nbap.fdd_DL_ChannelisationCodeNumber",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.FDD_DL_ChannelisationCodeNumber", HFILL }},
    { &hf_nbap_tFCS,
      { "tFCS", "nbap.tFCS",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.TFCS", HFILL }},
    { &hf_nbap_secondary_CCPCH_SlotFormat,
      { "secondary-CCPCH-SlotFormat", "nbap.secondary_CCPCH_SlotFormat",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SecondaryCCPCH_SlotFormat", HFILL }},
    { &hf_nbap_tFCI_Presence,
      { "tFCI-Presence", "nbap.tFCI_Presence",
        FT_UINT32, BASE_DEC, VALS(nbap_TFCI_Presence_vals), 0,
        "nbap.TFCI_Presence", HFILL }},
    { &hf_nbap_multiplexingPosition,
      { "multiplexingPosition", "nbap.multiplexingPosition",
        FT_UINT32, BASE_DEC, VALS(nbap_MultiplexingPosition_vals), 0,
        "nbap.MultiplexingPosition", HFILL }},
    { &hf_nbap_powerOffsetInformation,
      { "powerOffsetInformation", "nbap.powerOffsetInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PowerOffsetInformation_CTCH_SetupRqstFDD", HFILL }},
    { &hf_nbap_sTTD_Indicator,
      { "sTTD-Indicator", "nbap.sTTD_Indicator",
        FT_UINT32, BASE_DEC, VALS(nbap_STTD_Indicator_vals), 0,
        "nbap.STTD_Indicator", HFILL }},
    { &hf_nbap_fACH_Parameters,
      { "fACH-Parameters", "nbap.fACH_Parameters",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.FACH_ParametersList_CTCH_SetupRqstFDD", HFILL }},
    { &hf_nbap_pCH_Parameters,
      { "pCH-Parameters", "nbap.pCH_Parameters",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PCH_Parameters_CTCH_SetupRqstFDD", HFILL }},
    { &hf_nbap_iE_Extensions,
      { "iE-Extensions", "nbap.iE_Extensions",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.ProtocolExtensionContainer", HFILL }},
    { &hf_nbap_pO1_ForTFCI_Bits,
      { "pO1-ForTFCI-Bits", "nbap.pO1_ForTFCI_Bits",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.PowerOffset", HFILL }},
    { &hf_nbap_pO3_ForPilotBits,
      { "pO3-ForPilotBits", "nbap.pO3_ForPilotBits",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.PowerOffset", HFILL }},
    { &hf_nbap_FACH_ParametersListIE_CTCH_SetupRqstFDD_item,
      { "Item", "nbap.FACH_ParametersListIE_CTCH_SetupRqstFDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.FACH_ParametersItem_CTCH_SetupRqstFDD", HFILL }},
    { &hf_nbap_commonTransportChannelID,
      { "commonTransportChannelID", "nbap.commonTransportChannelID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CommonTransportChannelID", HFILL }},
    { &hf_nbap_transportFormatSet,
      { "transportFormatSet", "nbap.transportFormatSet",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.TransportFormatSet", HFILL }},
    { &hf_nbap_toAWS,
      { "toAWS", "nbap.toAWS",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.ToAWS", HFILL }},
    { &hf_nbap_toAWE,
      { "toAWE", "nbap.toAWE",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.ToAWE", HFILL }},
    { &hf_nbap_maxFACH_Power,
      { "maxFACH-Power", "nbap.maxFACH_Power",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_pCH_Power,
      { "pCH-Power", "nbap.pCH_Power",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_pICH_Parameters,
      { "pICH-Parameters", "nbap.pICH_Parameters",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PICH_Parameters_CTCH_SetupRqstFDD", HFILL }},
    { &hf_nbap_fdd_dl_ChannelisationCodeNumber,
      { "fdd-dl-ChannelisationCodeNumber", "nbap.fdd_dl_ChannelisationCodeNumber",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.FDD_DL_ChannelisationCodeNumber", HFILL }},
    { &hf_nbap_pICH_Power,
      { "pICH-Power", "nbap.pICH_Power",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.PICH_Power", HFILL }},
    { &hf_nbap_pICH_Mode,
      { "pICH-Mode", "nbap.pICH_Mode",
        FT_UINT32, BASE_DEC, VALS(nbap_PICH_Mode_vals), 0,
        "nbap.PICH_Mode", HFILL }},
    { &hf_nbap_mICH_Power,
      { "mICH-Power", "nbap.mICH_Power",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.PICH_Power", HFILL }},
    { &hf_nbap_mICH_Mode,
      { "mICH-Mode", "nbap.mICH_Mode",
        FT_UINT32, BASE_DEC, VALS(nbap_MICH_Mode_vals), 0,
        "nbap.MICH_Mode", HFILL }},
    { &hf_nbap_scramblingCodeNumber,
      { "scramblingCodeNumber", "nbap.scramblingCodeNumber",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.ScramblingCodeNumber", HFILL }},
    { &hf_nbap_preambleSignatures,
      { "preambleSignatures", "nbap.preambleSignatures",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.PreambleSignatures", HFILL }},
    { &hf_nbap_allowedSlotFormatInformation,
      { "allowedSlotFormatInformation", "nbap.allowedSlotFormatInformation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.AllowedSlotFormatInformationList_CTCH_SetupRqstFDD", HFILL }},
    { &hf_nbap_rACH_SubChannelNumbers,
      { "rACH-SubChannelNumbers", "nbap.rACH_SubChannelNumbers",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.RACH_SubChannelNumbers", HFILL }},
    { &hf_nbap_ul_punctureLimit,
      { "ul-punctureLimit", "nbap.ul_punctureLimit",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.PunctureLimit", HFILL }},
    { &hf_nbap_preambleThreshold,
      { "preambleThreshold", "nbap.preambleThreshold",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.PreambleThreshold", HFILL }},
    { &hf_nbap_rACH_Parameters,
      { "rACH-Parameters", "nbap.rACH_Parameters",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RACH_Parameters_CTCH_SetupRqstFDD", HFILL }},
    { &hf_nbap_aICH_Parameters,
      { "aICH-Parameters", "nbap.aICH_Parameters",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.AICH_Parameters_CTCH_SetupRqstFDD", HFILL }},
    { &hf_nbap_AllowedSlotFormatInformationList_CTCH_SetupRqstFDD_item,
      { "Item", "nbap.AllowedSlotFormatInformationList_CTCH_SetupRqstFDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.AllowedSlotFormatInformationItem_CTCH_SetupRqstFDD", HFILL }},
    { &hf_nbap_rACHSlotFormat,
      { "rACHSlotFormat", "nbap.rACHSlotFormat",
        FT_UINT32, BASE_DEC, VALS(nbap_RACH_SlotFormat_vals), 0,
        "nbap.RACH_SlotFormat", HFILL }},
    { &hf_nbap_aICH_TransmissionTiming,
      { "aICH-TransmissionTiming", "nbap.aICH_TransmissionTiming",
        FT_UINT32, BASE_DEC, VALS(nbap_AICH_TransmissionTiming_vals), 0,
        "nbap.AICH_TransmissionTiming", HFILL }},
    { &hf_nbap_aICH_Power,
      { "aICH-Power", "nbap.aICH_Power",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.AICH_Power", HFILL }},
    { &hf_nbap_secondary_CCPCH_parameters1,
      { "secondary-CCPCH-parameters", "nbap.secondary_CCPCH_parameters",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Secondary_CCPCH_CTCH_SetupRqstTDD", HFILL }},
    { &hf_nbap_pRACH_parameters1,
      { "pRACH-parameters", "nbap.pRACH_parameters",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PRACH_CTCH_SetupRqstTDD", HFILL }},
    { &hf_nbap_sCCPCH_CCTrCH_ID,
      { "sCCPCH-CCTrCH-ID", "nbap.sCCPCH_CCTrCH_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CCTrCH_ID", HFILL }},
    { &hf_nbap_tFCI_Coding,
      { "tFCI-Coding", "nbap.tFCI_Coding",
        FT_UINT32, BASE_DEC, VALS(nbap_TFCI_Coding_vals), 0,
        "nbap.TFCI_Coding", HFILL }},
    { &hf_nbap_punctureLimit,
      { "punctureLimit", "nbap.punctureLimit",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.PunctureLimit", HFILL }},
    { &hf_nbap_secondaryCCPCH_parameterList,
      { "secondaryCCPCH-parameterList", "nbap.secondaryCCPCH_parameterList",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Secondary_CCPCH_parameterList_CTCH_SetupRqstTDD", HFILL }},
    { &hf_nbap_fACH_ParametersList,
      { "fACH-ParametersList", "nbap.fACH_ParametersList",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.FACH_ParametersList_CTCH_SetupRqstTDD", HFILL }},
    { &hf_nbap_pCH_Parameters1,
      { "pCH-Parameters", "nbap.pCH_Parameters",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PCH_Parameters_CTCH_SetupRqstTDD", HFILL }},
    { &hf_nbap_Secondary_CCPCH_parameterListIE_CTCH_SetupRqstTDD_item,
      { "Item", "nbap.Secondary_CCPCH_parameterListIE_CTCH_SetupRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Secondary_CCPCH_parameterItem_CTCH_SetupRqstTDD", HFILL }},
    { &hf_nbap_tdd_ChannelisationCode,
      { "tdd-ChannelisationCode", "nbap.tdd_ChannelisationCode",
        FT_UINT32, BASE_DEC, VALS(nbap_TDD_ChannelisationCode_vals), 0,
        "nbap.TDD_ChannelisationCode", HFILL }},
    { &hf_nbap_timeslot,
      { "timeslot", "nbap.timeslot",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.TimeSlot", HFILL }},
    { &hf_nbap_midambleShiftandBurstType,
      { "midambleShiftandBurstType", "nbap.midambleShiftandBurstType",
        FT_UINT32, BASE_DEC, VALS(nbap_MidambleShiftAndBurstType_vals), 0,
        "nbap.MidambleShiftAndBurstType", HFILL }},
    { &hf_nbap_tdd_PhysicalChannelOffset,
      { "tdd-PhysicalChannelOffset", "nbap.tdd_PhysicalChannelOffset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.TDD_PhysicalChannelOffset", HFILL }},
    { &hf_nbap_repetitionPeriod,
      { "repetitionPeriod", "nbap.repetitionPeriod",
        FT_UINT32, BASE_DEC, VALS(nbap_RepetitionPeriod_vals), 0,
        "nbap.RepetitionPeriod", HFILL }},
    { &hf_nbap_repetitionLength,
      { "repetitionLength", "nbap.repetitionLength",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.RepetitionLength", HFILL }},
    { &hf_nbap_s_CCPCH_Power,
      { "s-CCPCH-Power", "nbap.s_CCPCH_Power",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_FACH_ParametersListIE_CTCH_SetupRqstTDD_item,
      { "Item", "nbap.FACH_ParametersListIE_CTCH_SetupRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.FACH_ParametersItem_CTCH_SetupRqstTDD", HFILL }},
    { &hf_nbap_fACH_CCTrCH_ID,
      { "fACH-CCTrCH-ID", "nbap.fACH_CCTrCH_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CCTrCH_ID", HFILL }},
    { &hf_nbap_dl_TransportFormatSet,
      { "dl-TransportFormatSet", "nbap.dl_TransportFormatSet",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.TransportFormatSet", HFILL }},
    { &hf_nbap_pCH_CCTrCH_ID,
      { "pCH-CCTrCH-ID", "nbap.pCH_CCTrCH_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CCTrCH_ID", HFILL }},
    { &hf_nbap_pICH_Parameters1,
      { "pICH-Parameters", "nbap.pICH_Parameters",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PICH_Parameters_CTCH_SetupRqstTDD", HFILL }},
    { &hf_nbap_timeSlot,
      { "timeSlot", "nbap.timeSlot",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.TimeSlot", HFILL }},
    { &hf_nbap_midambleshiftAndBurstType,
      { "midambleshiftAndBurstType", "nbap.midambleshiftAndBurstType",
        FT_UINT32, BASE_DEC, VALS(nbap_MidambleShiftAndBurstType_vals), 0,
        "nbap.MidambleShiftAndBurstType", HFILL }},
    { &hf_nbap_pagingIndicatorLength,
      { "pagingIndicatorLength", "nbap.pagingIndicatorLength",
        FT_UINT32, BASE_DEC, VALS(nbap_PagingIndicatorLength_vals), 0,
        "nbap.PagingIndicatorLength", HFILL }},
    { &hf_nbap_tdd_ChannelisationCodeLCR,
      { "tdd-ChannelisationCodeLCR", "nbap.tdd_ChannelisationCodeLCR",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.TDD_ChannelisationCodeLCR", HFILL }},
    { &hf_nbap_timeSlotLCR,
      { "timeSlotLCR", "nbap.timeSlotLCR",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.TimeSlotLCR", HFILL }},
    { &hf_nbap_midambleShiftLCR,
      { "midambleShiftLCR", "nbap.midambleShiftLCR",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.MidambleShiftLCR", HFILL }},
    { &hf_nbap_second_TDD_ChannelisationCodeLCR,
      { "second-TDD-ChannelisationCodeLCR", "nbap.second_TDD_ChannelisationCodeLCR",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.TDD_ChannelisationCodeLCR", HFILL }},
    { &hf_nbap_Secondary_CCPCH_LCR_parameterList_CTCH_SetupRqstTDD_item,
      { "Item", "nbap.Secondary_CCPCH_LCR_parameterList_CTCH_SetupRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Secondary_CCPCH_LCR_parameterItem_CTCH_SetupRqstTDD", HFILL }},
    { &hf_nbap_timeslotLCR,
      { "timeslotLCR", "nbap.timeslotLCR",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.TimeSlotLCR", HFILL }},
    { &hf_nbap_s_CCPCH_TimeSlotFormat_LCR,
      { "s-CCPCH-TimeSlotFormat-LCR", "nbap.s_CCPCH_TimeSlotFormat_LCR",
        FT_UINT32, BASE_DEC, VALS(nbap_TDD_DL_DPCH_TimeSlotFormat_LCR_vals), 0,
        "nbap.TDD_DL_DPCH_TimeSlotFormat_LCR", HFILL }},
    { &hf_nbap_notificationIndicatorLength,
      { "notificationIndicatorLength", "nbap.notificationIndicatorLength",
        FT_UINT32, BASE_DEC, VALS(nbap_NotificationIndicatorLength_vals), 0,
        "nbap.NotificationIndicatorLength", HFILL }},
    { &hf_nbap_mICH_TDDOption_Specific_Parameters,
      { "mICH-TDDOption-Specific-Parameters", "nbap.mICH_TDDOption_Specific_Parameters",
        FT_UINT32, BASE_DEC, VALS(nbap_MICH_TDDOption_Specific_Parameters_CTCH_SetupRqstTDD_vals), 0,
        "nbap.MICH_TDDOption_Specific_Parameters_CTCH_SetupRqstTDD", HFILL }},
    { &hf_nbap_hCR_TDD,
      { "hCR-TDD", "nbap.hCR_TDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.MICH_HCR_Parameters_CTCH_SetupRqstTDD", HFILL }},
    { &hf_nbap_lCR_TDD,
      { "lCR-TDD", "nbap.lCR_TDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.MICH_LCR_Parameters_CTCH_SetupRqstTDD", HFILL }},
    { &hf_nbap_tSTD_Indicator,
      { "tSTD-Indicator", "nbap.tSTD_Indicator",
        FT_UINT32, BASE_DEC, VALS(nbap_TSTD_Indicator_vals), 0,
        "nbap.TSTD_Indicator", HFILL }},
    { &hf_nbap_Secondary_CCPCH_parameterExtendedList_CTCH_SetupRqstTDD_item,
      { "Item", "nbap.Secondary_CCPCH_parameterExtendedList_CTCH_SetupRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Secondary_CCPCH_parameterItem_CTCH_SetupRqstTDD", HFILL }},
    { &hf_nbap_Secondary_CCPCH_LCR_parameterExtendedList_CTCH_SetupRqstTDD_item,
      { "Item", "nbap.Secondary_CCPCH_LCR_parameterExtendedList_CTCH_SetupRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Secondary_CCPCH_LCR_parameterItem_CTCH_SetupRqstTDD", HFILL }},
    { &hf_nbap_pRACH_Parameters_CTCH_SetupRqstTDD,
      { "pRACH-Parameters-CTCH-SetupRqstTDD", "nbap.pRACH_Parameters_CTCH_SetupRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PRACH_Parameters_CTCH_SetupRqstTDD", HFILL }},
    { &hf_nbap_maxPRACH_MidambleShifts,
      { "maxPRACH-MidambleShifts", "nbap.maxPRACH_MidambleShifts",
        FT_UINT32, BASE_DEC, VALS(nbap_MaxPRACH_MidambleShifts_vals), 0,
        "nbap.MaxPRACH_MidambleShifts", HFILL }},
    { &hf_nbap_pRACH_Midamble,
      { "pRACH-Midamble", "nbap.pRACH_Midamble",
        FT_UINT32, BASE_DEC, VALS(nbap_PRACH_Midamble_vals), 0,
        "nbap.PRACH_Midamble", HFILL }},
    { &hf_nbap_rACH,
      { "rACH", "nbap.rACH",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RACH_Parameter_CTCH_SetupRqstTDD", HFILL }},
    { &hf_nbap_uL_TransportFormatSet,
      { "uL-TransportFormatSet", "nbap.uL_TransportFormatSet",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.TransportFormatSet", HFILL }},
    { &hf_nbap_PRACH_LCR_ParametersList_CTCH_SetupRqstTDD_item,
      { "Item", "nbap.PRACH_LCR_ParametersList_CTCH_SetupRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PRACH_LCR_ParametersItem_CTCH_SetupRqstTDD", HFILL }},
    { &hf_nbap_fPACH_Power,
      { "fPACH-Power", "nbap.fPACH_Power",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.FPACH_Power", HFILL }},
    { &hf_nbap_FACH_CommonTransportChannel_InformationResponse_item,
      { "Item", "nbap.FACH_CommonTransportChannel_InformationResponse_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CommonTransportChannel_InformationResponse", HFILL }},
    { &hf_nbap_secondary_CCPCH_parameters2,
      { "secondary-CCPCH-parameters", "nbap.secondary_CCPCH_parameters",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Secondary_CCPCHList_CTCH_ReconfRqstFDD", HFILL }},
    { &hf_nbap_pRACH_parameters2,
      { "pRACH-parameters", "nbap.pRACH_parameters",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PRACHList_CTCH_ReconfRqstFDD", HFILL }},
    { &hf_nbap_notUsed_cPCH_parameters,
      { "notUsed-cPCH-parameters", "nbap.notUsed_cPCH_parameters",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NULL", HFILL }},
    { &hf_nbap_fACH_ParametersList_CTCH_ReconfRqstFDD,
      { "fACH-ParametersList-CTCH-ReconfRqstFDD", "nbap.fACH_ParametersList_CTCH_ReconfRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.FACH_ParametersList_CTCH_ReconfRqstFDD", HFILL }},
    { &hf_nbap_pCH_Parameters_CTCH_ReconfRqstFDD,
      { "pCH-Parameters-CTCH-ReconfRqstFDD", "nbap.pCH_Parameters_CTCH_ReconfRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PCH_Parameters_CTCH_ReconfRqstFDD", HFILL }},
    { &hf_nbap_pICH_Parameters_CTCH_ReconfRqstFDD,
      { "pICH-Parameters-CTCH-ReconfRqstFDD", "nbap.pICH_Parameters_CTCH_ReconfRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PICH_Parameters_CTCH_ReconfRqstFDD", HFILL }},
    { &hf_nbap_FACH_ParametersListIE_CTCH_ReconfRqstFDD_item,
      { "Item", "nbap.FACH_ParametersListIE_CTCH_ReconfRqstFDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.FACH_ParametersItem_CTCH_ReconfRqstFDD", HFILL }},
    { &hf_nbap_pRACH_ParametersList_CTCH_ReconfRqstFDD,
      { "pRACH-ParametersList-CTCH-ReconfRqstFDD", "nbap.pRACH_ParametersList_CTCH_ReconfRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PRACH_ParametersList_CTCH_ReconfRqstFDD", HFILL }},
    { &hf_nbap_aICH_ParametersList_CTCH_ReconfRqstFDD,
      { "aICH-ParametersList-CTCH-ReconfRqstFDD", "nbap.aICH_ParametersList_CTCH_ReconfRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.AICH_ParametersList_CTCH_ReconfRqstFDD", HFILL }},
    { &hf_nbap_PRACH_ParametersListIE_CTCH_ReconfRqstFDD_item,
      { "Item", "nbap.PRACH_ParametersListIE_CTCH_ReconfRqstFDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PRACH_ParametersItem_CTCH_ReconfRqstFDD", HFILL }},
    { &hf_nbap_allowedSlotFormatInformation1,
      { "allowedSlotFormatInformation", "nbap.allowedSlotFormatInformation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.AllowedSlotFormatInformationList_CTCH_ReconfRqstFDD", HFILL }},
    { &hf_nbap_AllowedSlotFormatInformationList_CTCH_ReconfRqstFDD_item,
      { "Item", "nbap.AllowedSlotFormatInformationList_CTCH_ReconfRqstFDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.AllowedSlotFormatInformationItem_CTCH_ReconfRqstFDD", HFILL }},
    { &hf_nbap_rACH_SlotFormat,
      { "rACH-SlotFormat", "nbap.rACH_SlotFormat",
        FT_UINT32, BASE_DEC, VALS(nbap_RACH_SlotFormat_vals), 0,
        "nbap.RACH_SlotFormat", HFILL }},
    { &hf_nbap_AICH_ParametersListIE_CTCH_ReconfRqstFDD_item,
      { "Item", "nbap.AICH_ParametersListIE_CTCH_ReconfRqstFDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.AICH_ParametersItem_CTCH_ReconfRqstFDD", HFILL }},
    { &hf_nbap_cCTrCH_ID,
      { "cCTrCH-ID", "nbap.cCTrCH_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CCTrCH_ID", HFILL }},
    { &hf_nbap_secondaryCCPCHList,
      { "secondaryCCPCHList", "nbap.secondaryCCPCHList",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Secondary_CCPCHList_CTCH_ReconfRqstTDD", HFILL }},
    { &hf_nbap_Secondary_CCPCHListIE_CTCH_ReconfRqstTDD_item,
      { "Item", "nbap.Secondary_CCPCHListIE_CTCH_ReconfRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Secondary_CCPCHItem_CTCH_ReconfRqstTDD", HFILL }},
    { &hf_nbap_sCCPCH_Power,
      { "sCCPCH-Power", "nbap.sCCPCH_Power",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_FACH_ParametersList_CTCH_ReconfRqstTDD_item,
      { "Item", "nbap.FACH_ParametersList_CTCH_ReconfRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.FACH_ParametersItem_CTCH_ReconfRqstTDD", HFILL }},
    { &hf_nbap_commonPhysicalChannelId,
      { "commonPhysicalChannelId", "nbap.commonPhysicalChannelId",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CommonPhysicalChannelID", HFILL }},
    { &hf_nbap_fPACHPower,
      { "fPACHPower", "nbap.fPACHPower",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.FPACH_Power", HFILL }},
    { &hf_nbap_Secondary_CCPCH_parameterExtendedList_CTCH_ReconfRqstTDD_item,
      { "Item", "nbap.Secondary_CCPCH_parameterExtendedList_CTCH_ReconfRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Secondary_CCPCHItem_CTCH_ReconfRqstTDD", HFILL }},
    { &hf_nbap_Secondary_CCPCH_LCR_parameterExtendedList_CTCH_ReconfRqstTDD_item,
      { "Item", "nbap.Secondary_CCPCH_LCR_parameterExtendedList_CTCH_ReconfRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Secondary_CCPCHItem_CTCH_ReconfRqstTDD", HFILL }},
    { &hf_nbap_Cell_InformationList_AuditRsp_item,
      { "Item", "nbap.Cell_InformationList_AuditRsp_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_c_ID,
      { "c-ID", "nbap.c_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.C_ID", HFILL }},
    { &hf_nbap_configurationGenerationID,
      { "configurationGenerationID", "nbap.configurationGenerationID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.ConfigurationGenerationID", HFILL }},
    { &hf_nbap_resourceOperationalState,
      { "resourceOperationalState", "nbap.resourceOperationalState",
        FT_UINT32, BASE_DEC, VALS(nbap_ResourceOperationalState_vals), 0,
        "nbap.ResourceOperationalState", HFILL }},
    { &hf_nbap_availabilityStatus,
      { "availabilityStatus", "nbap.availabilityStatus",
        FT_UINT32, BASE_DEC, VALS(nbap_AvailabilityStatus_vals), 0,
        "nbap.AvailabilityStatus", HFILL }},
    { &hf_nbap_local_Cell_ID,
      { "local-Cell-ID", "nbap.local_Cell_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Local_Cell_ID", HFILL }},
    { &hf_nbap_primary_SCH_Information,
      { "primary-SCH-Information", "nbap.primary_SCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.P_SCH_Information_AuditRsp", HFILL }},
    { &hf_nbap_secondary_SCH_Information,
      { "secondary-SCH-Information", "nbap.secondary_SCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.S_SCH_Information_AuditRsp", HFILL }},
    { &hf_nbap_primary_CPICH_Information,
      { "primary-CPICH-Information", "nbap.primary_CPICH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.P_CPICH_Information_AuditRsp", HFILL }},
    { &hf_nbap_secondary_CPICH_InformationList,
      { "secondary-CPICH-InformationList", "nbap.secondary_CPICH_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.S_CPICH_InformationList_AuditRsp", HFILL }},
    { &hf_nbap_primary_CCPCH_Information,
      { "primary-CCPCH-Information", "nbap.primary_CCPCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.P_CCPCH_Information_AuditRsp", HFILL }},
    { &hf_nbap_bCH_Information,
      { "bCH-Information", "nbap.bCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.BCH_Information_AuditRsp", HFILL }},
    { &hf_nbap_secondary_CCPCH_InformationList,
      { "secondary-CCPCH-InformationList", "nbap.secondary_CCPCH_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.S_CCPCH_InformationList_AuditRsp", HFILL }},
    { &hf_nbap_pCH_Information,
      { "pCH-Information", "nbap.pCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PCH_Information_AuditRsp", HFILL }},
    { &hf_nbap_pICH_Information,
      { "pICH-Information", "nbap.pICH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PICH_Information_AuditRsp", HFILL }},
    { &hf_nbap_fACH_InformationList,
      { "fACH-InformationList", "nbap.fACH_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.FACH_InformationList_AuditRsp", HFILL }},
    { &hf_nbap_pRACH_InformationList,
      { "pRACH-InformationList", "nbap.pRACH_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.PRACH_InformationList_AuditRsp", HFILL }},
    { &hf_nbap_rACH_InformationList,
      { "rACH-InformationList", "nbap.rACH_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.RACH_InformationList_AuditRsp", HFILL }},
    { &hf_nbap_aICH_InformationList,
      { "aICH-InformationList", "nbap.aICH_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.AICH_InformationList_AuditRsp", HFILL }},
    { &hf_nbap_notUsed_1_pCPCH_InformationList,
      { "notUsed-1-pCPCH-InformationList", "nbap.notUsed_1_pCPCH_InformationList",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NULL", HFILL }},
    { &hf_nbap_notUsed_2_cPCH_InformationList,
      { "notUsed-2-cPCH-InformationList", "nbap.notUsed_2_cPCH_InformationList",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NULL", HFILL }},
    { &hf_nbap_notUsed_3_aP_AICH_InformationList,
      { "notUsed-3-aP-AICH-InformationList", "nbap.notUsed_3_aP_AICH_InformationList",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NULL", HFILL }},
    { &hf_nbap_notUsed_4_cDCA_ICH_InformationList,
      { "notUsed-4-cDCA-ICH-InformationList", "nbap.notUsed_4_cDCA_ICH_InformationList",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NULL", HFILL }},
    { &hf_nbap_sCH_Information,
      { "sCH-Information", "nbap.sCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.SCH_Information_AuditRsp", HFILL }},
    { &hf_nbap_S_CPICH_InformationList_AuditRsp_item,
      { "Item", "nbap.S_CPICH_InformationList_AuditRsp_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_S_CCPCH_InformationList_AuditRsp_item,
      { "Item", "nbap.S_CCPCH_InformationList_AuditRsp_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_FACH_InformationList_AuditRsp_item,
      { "Item", "nbap.FACH_InformationList_AuditRsp_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_PRACH_InformationList_AuditRsp_item,
      { "Item", "nbap.PRACH_InformationList_AuditRsp_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_RACH_InformationList_AuditRsp_item,
      { "Item", "nbap.RACH_InformationList_AuditRsp_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_AICH_InformationList_AuditRsp_item,
      { "Item", "nbap.AICH_InformationList_AuditRsp_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_CCP_InformationList_AuditRsp_item,
      { "Item", "nbap.CCP_InformationList_AuditRsp_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_communicationControlPortID,
      { "communicationControlPortID", "nbap.communicationControlPortID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CommunicationControlPortID", HFILL }},
    { &hf_nbap_FPACH_LCR_InformationList_AuditRsp_item,
      { "Item", "nbap.FPACH_LCR_InformationList_AuditRsp_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_S_CCPCH_InformationListExt_AuditRsp_item,
      { "Item", "nbap.S_CCPCH_InformationListExt_AuditRsp_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_S_CCPCH_LCR_InformationListExt_AuditRsp_item,
      { "Item", "nbap.S_CCPCH_LCR_InformationListExt_AuditRsp_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_Local_Cell_InformationList_AuditRsp_item,
      { "Item", "nbap.Local_Cell_InformationList_AuditRsp_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_dl_or_global_capacityCredit,
      { "dl-or-global-capacityCredit", "nbap.dl_or_global_capacityCredit",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_or_Global_CapacityCredit", HFILL }},
    { &hf_nbap_ul_capacityCredit,
      { "ul-capacityCredit", "nbap.ul_capacityCredit",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.UL_CapacityCredit", HFILL }},
    { &hf_nbap_commonChannelsCapacityConsumptionLaw,
      { "commonChannelsCapacityConsumptionLaw", "nbap.commonChannelsCapacityConsumptionLaw",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CommonChannelsCapacityConsumptionLaw", HFILL }},
    { &hf_nbap_dedicatedChannelsCapacityConsumptionLaw,
      { "dedicatedChannelsCapacityConsumptionLaw", "nbap.dedicatedChannelsCapacityConsumptionLaw",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DedicatedChannelsCapacityConsumptionLaw", HFILL }},
    { &hf_nbap_maximumDL_PowerCapability,
      { "maximumDL-PowerCapability", "nbap.maximumDL_PowerCapability",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.MaximumDL_PowerCapability", HFILL }},
    { &hf_nbap_minSpreadingFactor,
      { "minSpreadingFactor", "nbap.minSpreadingFactor",
        FT_UINT32, BASE_DEC, VALS(nbap_MinSpreadingFactor_vals), 0,
        "nbap.MinSpreadingFactor", HFILL }},
    { &hf_nbap_minimumDL_PowerCapability,
      { "minimumDL-PowerCapability", "nbap.minimumDL_PowerCapability",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.MinimumDL_PowerCapability", HFILL }},
    { &hf_nbap_local_Cell_Group_ID,
      { "local-Cell-Group-ID", "nbap.local_Cell_Group_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Local_Cell_ID", HFILL }},
    { &hf_nbap_Local_Cell_Group_InformationList_AuditRsp_item,
      { "Item", "nbap.Local_Cell_Group_InformationList_AuditRsp_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_Power_Local_Cell_Group_InformationList_AuditRsp_item,
      { "Item", "nbap.Power_Local_Cell_Group_InformationList_AuditRsp_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_power_Local_Cell_Group_ID,
      { "power-Local-Cell-Group-ID", "nbap.power_Local_Cell_Group_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Local_Cell_ID", HFILL }},
    { &hf_nbap_cell,
      { "cell", "nbap.cell",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Cell_CM_Rqst", HFILL }},
    { &hf_nbap_rACH1,
      { "rACH", "nbap.rACH",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RACH_CM_Rqst", HFILL }},
    { &hf_nbap_notUsed_cPCH,
      { "notUsed-cPCH", "nbap.notUsed_cPCH",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NULL", HFILL }},
    { &hf_nbap_extension_CommonMeasurementObjectType_CM_Rqst,
      { "extension-CommonMeasurementObjectType-CM-Rqst", "nbap.extension_CommonMeasurementObjectType_CM_Rqst",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Extension_CommonMeasurementObjectType_CM_Rqst", HFILL }},
    { &hf_nbap_powerLocalCellGroupID,
      { "powerLocalCellGroupID", "nbap.powerLocalCellGroupID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Local_Cell_ID", HFILL }},
    { &hf_nbap_cell1,
      { "cell", "nbap.cell",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Cell_CM_Rsp", HFILL }},
    { &hf_nbap_rACH2,
      { "rACH", "nbap.rACH",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RACH_CM_Rsp", HFILL }},
    { &hf_nbap_extension_CommonMeasurementObjectType_CM_Rsp,
      { "extension-CommonMeasurementObjectType-CM-Rsp", "nbap.extension_CommonMeasurementObjectType_CM_Rsp",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Extension_CommonMeasurementObjectType_CM_Rsp", HFILL }},
    { &hf_nbap_commonMeasurementValue,
      { "commonMeasurementValue", "nbap.commonMeasurementValue",
        FT_UINT32, BASE_DEC, VALS(nbap_CommonMeasurementValue_vals), 0,
        "nbap.CommonMeasurementValue", HFILL }},
    { &hf_nbap_n_INSYNC_IND,
      { "n-INSYNC-IND", "nbap.n_INSYNC_IND",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.N_INSYNC_IND", HFILL }},
    { &hf_nbap_n_OUTSYNC_IND,
      { "n-OUTSYNC-IND", "nbap.n_OUTSYNC_IND",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.N_OUTSYNC_IND", HFILL }},
    { &hf_nbap_t_RLFAILURE,
      { "t-RLFAILURE", "nbap.t_RLFAILURE",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.T_RLFAILURE", HFILL }},
    { &hf_nbap_primarySCH_Power,
      { "primarySCH-Power", "nbap.primarySCH_Power",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_secondarySCH_Power,
      { "secondarySCH-Power", "nbap.secondarySCH_Power",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_primaryCPICH_Power,
      { "primaryCPICH-Power", "nbap.primaryCPICH_Power",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.PrimaryCPICH_Power", HFILL }},
    { &hf_nbap_transmitDiversityIndicator,
      { "transmitDiversityIndicator", "nbap.transmitDiversityIndicator",
        FT_UINT32, BASE_DEC, VALS(nbap_TransmitDiversityIndicator_vals), 0,
        "nbap.TransmitDiversityIndicator", HFILL }},
    { &hf_nbap_SecondaryCPICH_InformationList_Cell_SetupRqstFDD_item,
      { "Item", "nbap.SecondaryCPICH_InformationList_Cell_SetupRqstFDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_fDD_DL_ChannelisationCodeNumber,
      { "fDD-DL-ChannelisationCodeNumber", "nbap.fDD_DL_ChannelisationCodeNumber",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.FDD_DL_ChannelisationCodeNumber", HFILL }},
    { &hf_nbap_secondaryCPICH_Power,
      { "secondaryCPICH-Power", "nbap.secondaryCPICH_Power",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_bCH_information,
      { "bCH-information", "nbap.bCH_information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.BCH_Information_Cell_SetupRqstFDD", HFILL }},
    { &hf_nbap_bCH_Power,
      { "bCH-Power", "nbap.bCH_Power",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_powerRaiseLimit,
      { "powerRaiseLimit", "nbap.powerRaiseLimit",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.PowerRaiseLimit", HFILL }},
    { &hf_nbap_dLPowerAveragingWindowSize,
      { "dLPowerAveragingWindowSize", "nbap.dLPowerAveragingWindowSize",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DLPowerAveragingWindowSize", HFILL }},
    { &hf_nbap_iPDL_FDD_Parameters,
      { "iPDL-FDD-Parameters", "nbap.iPDL_FDD_Parameters",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.IPDL_FDD_Parameters", HFILL }},
    { &hf_nbap_iPDL_Indicator,
      { "iPDL-Indicator", "nbap.iPDL_Indicator",
        FT_UINT32, BASE_DEC, VALS(nbap_IPDL_Indicator_vals), 0,
        "nbap.IPDL_Indicator", HFILL }},
    { &hf_nbap_CellPortion_InformationList_Cell_SetupRqstFDD_item,
      { "Item", "nbap.CellPortion_InformationList_Cell_SetupRqstFDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_cellPortionID,
      { "cellPortionID", "nbap.cellPortionID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CellPortionID", HFILL }},
    { &hf_nbap_associatedSecondaryCPICH,
      { "associatedSecondaryCPICH", "nbap.associatedSecondaryCPICH",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CommonPhysicalChannelID", HFILL }},
    { &hf_nbap_maximumTransmissionPowerforCellPortion,
      { "maximumTransmissionPowerforCellPortion", "nbap.maximumTransmissionPowerforCellPortion",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.MaximumTransmissionPower", HFILL }},
    { &hf_nbap_syncCaseIndicator,
      { "syncCaseIndicator", "nbap.syncCaseIndicator",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.SyncCaseIndicator_Cell_SetupRqstTDD_PSCH", HFILL }},
    { &hf_nbap_sCH_Power,
      { "sCH-Power", "nbap.sCH_Power",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_case1,
      { "case1", "nbap.case1",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Case1_Cell_SetupRqstTDD", HFILL }},
    { &hf_nbap_case2,
      { "case2", "nbap.case2",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Case2_Cell_SetupRqstTDD", HFILL }},
    { &hf_nbap_sCH_TimeSlot,
      { "sCH-TimeSlot", "nbap.sCH_TimeSlot",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SCH_TimeSlot", HFILL }},
    { &hf_nbap_pCCPCH_Power,
      { "pCCPCH-Power", "nbap.pCCPCH_Power",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.PCCPCH_Power", HFILL }},
    { &hf_nbap_sCTD_Indicator,
      { "sCTD-Indicator", "nbap.sCTD_Indicator",
        FT_UINT32, BASE_DEC, VALS(nbap_SCTD_Indicator_vals), 0,
        "nbap.SCTD_Indicator", HFILL }},
    { &hf_nbap_TimeSlotConfigurationList_Cell_SetupRqstTDD_item,
      { "Item", "nbap.TimeSlotConfigurationList_Cell_SetupRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.TimeSlotConfigurationItem_Cell_SetupRqstTDD", HFILL }},
    { &hf_nbap_timeSlotStatus,
      { "timeSlotStatus", "nbap.timeSlotStatus",
        FT_UINT32, BASE_DEC, VALS(nbap_TimeSlotStatus_vals), 0,
        "nbap.TimeSlotStatus", HFILL }},
    { &hf_nbap_timeSlotDirection,
      { "timeSlotDirection", "nbap.timeSlotDirection",
        FT_UINT32, BASE_DEC, VALS(nbap_TimeSlotDirection_vals), 0,
        "nbap.TimeSlotDirection", HFILL }},
    { &hf_nbap_TimeSlotConfigurationList_LCR_Cell_SetupRqstTDD_item,
      { "Item", "nbap.TimeSlotConfigurationList_LCR_Cell_SetupRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.TimeSlotConfigurationItem_LCR_Cell_SetupRqstTDD", HFILL }},
    { &hf_nbap_dwPCH_Power,
      { "dwPCH-Power", "nbap.dwPCH_Power",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DwPCH_Power", HFILL }},
    { &hf_nbap_iPDL_TDD_Parameters,
      { "iPDL-TDD-Parameters", "nbap.iPDL_TDD_Parameters",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.IPDL_TDD_Parameters", HFILL }},
    { &hf_nbap_iPDL_TDD_Parameters_LCR,
      { "iPDL-TDD-Parameters-LCR", "nbap.iPDL_TDD_Parameters_LCR",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.IPDL_TDD_Parameters_LCR", HFILL }},
    { &hf_nbap_SecondaryCPICH_InformationList_Cell_ReconfRqstFDD_item,
      { "Item", "nbap.SecondaryCPICH_InformationList_Cell_ReconfRqstFDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_bCH_information1,
      { "bCH-information", "nbap.bCH_information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.BCH_information_Cell_ReconfRqstFDD", HFILL }},
    { &hf_nbap_CellPortion_InformationList_Cell_ReconfRqstFDD_item,
      { "Item", "nbap.CellPortion_InformationList_Cell_ReconfRqstFDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_TimeSlotConfigurationList_Cell_ReconfRqstTDD_item,
      { "Item", "nbap.TimeSlotConfigurationList_Cell_ReconfRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.TimeSlotConfigurationItem_Cell_ReconfRqstTDD", HFILL }},
    { &hf_nbap_TimeSlotConfigurationList_LCR_Cell_ReconfRqstTDD_item,
      { "Item", "nbap.TimeSlotConfigurationList_LCR_Cell_ReconfRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.TimeSlotConfigurationItem_LCR_Cell_ReconfRqstTDD", HFILL }},
    { &hf_nbap_no_Failure,
      { "no-Failure", "nbap.no_Failure",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.No_Failure_ResourceStatusInd", HFILL }},
    { &hf_nbap_serviceImpacting,
      { "serviceImpacting", "nbap.serviceImpacting",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ServiceImpacting_ResourceStatusInd", HFILL }},
    { &hf_nbap_local_Cell_InformationList,
      { "local-Cell-InformationList", "nbap.local_Cell_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Local_Cell_InformationList_ResourceStatusInd", HFILL }},
    { &hf_nbap_local_Cell_Group_InformationList,
      { "local-Cell-Group-InformationList", "nbap.local_Cell_Group_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Local_Cell_Group_InformationList_ResourceStatusInd", HFILL }},
    { &hf_nbap_Local_Cell_InformationList_ResourceStatusInd_item,
      { "Item", "nbap.Local_Cell_InformationList_ResourceStatusInd_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_local_CellID,
      { "local-CellID", "nbap.local_CellID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Local_Cell_ID", HFILL }},
    { &hf_nbap_addorDeleteIndicator,
      { "addorDeleteIndicator", "nbap.addorDeleteIndicator",
        FT_UINT32, BASE_DEC, VALS(nbap_AddorDeleteIndicator_vals), 0,
        "nbap.AddorDeleteIndicator", HFILL }},
    { &hf_nbap_Local_Cell_Group_InformationList_ResourceStatusInd_item,
      { "Item", "nbap.Local_Cell_Group_InformationList_ResourceStatusInd_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_Power_Local_Cell_Group_InformationList_ResourceStatusInd_item,
      { "Item", "nbap.Power_Local_Cell_Group_InformationList_ResourceStatusInd_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_local_Cell_InformationList1,
      { "local-Cell-InformationList", "nbap.local_Cell_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Local_Cell_InformationList2_ResourceStatusInd", HFILL }},
    { &hf_nbap_local_Cell_Group_InformationList1,
      { "local-Cell-Group-InformationList", "nbap.local_Cell_Group_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Local_Cell_Group_InformationList2_ResourceStatusInd", HFILL }},
    { &hf_nbap_cCP_InformationList,
      { "cCP-InformationList", "nbap.cCP_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CCP_InformationList_ResourceStatusInd", HFILL }},
    { &hf_nbap_cell_InformationList,
      { "cell-InformationList", "nbap.cell_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Cell_InformationList_ResourceStatusInd", HFILL }},
    { &hf_nbap_Local_Cell_InformationList2_ResourceStatusInd_item,
      { "Item", "nbap.Local_Cell_InformationList2_ResourceStatusInd_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_maximum_DL_PowerCapability,
      { "maximum-DL-PowerCapability", "nbap.maximum_DL_PowerCapability",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.MaximumDL_PowerCapability", HFILL }},
    { &hf_nbap_Local_Cell_Group_InformationList2_ResourceStatusInd_item,
      { "Item", "nbap.Local_Cell_Group_InformationList2_ResourceStatusInd_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_Power_Local_Cell_Group_InformationList2_ResourceStatusInd_item,
      { "Item", "nbap.Power_Local_Cell_Group_InformationList2_ResourceStatusInd_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_CCP_InformationList_ResourceStatusInd_item,
      { "Item", "nbap.CCP_InformationList_ResourceStatusInd_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_Cell_InformationList_ResourceStatusInd_item,
      { "Item", "nbap.Cell_InformationList_ResourceStatusInd_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_primary_SCH_Information1,
      { "primary-SCH-Information", "nbap.primary_SCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.P_SCH_Information_ResourceStatusInd", HFILL }},
    { &hf_nbap_secondary_SCH_Information1,
      { "secondary-SCH-Information", "nbap.secondary_SCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.S_SCH_Information_ResourceStatusInd", HFILL }},
    { &hf_nbap_primary_CPICH_Information1,
      { "primary-CPICH-Information", "nbap.primary_CPICH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.P_CPICH_Information_ResourceStatusInd", HFILL }},
    { &hf_nbap_secondary_CPICH_Information,
      { "secondary-CPICH-Information", "nbap.secondary_CPICH_Information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.S_CPICH_InformationList_ResourceStatusInd", HFILL }},
    { &hf_nbap_primary_CCPCH_Information1,
      { "primary-CCPCH-Information", "nbap.primary_CCPCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.P_CCPCH_Information_ResourceStatusInd", HFILL }},
    { &hf_nbap_bCH_Information1,
      { "bCH-Information", "nbap.bCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.BCH_Information_ResourceStatusInd", HFILL }},
    { &hf_nbap_secondary_CCPCH_InformationList1,
      { "secondary-CCPCH-InformationList", "nbap.secondary_CCPCH_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.S_CCPCH_InformationList_ResourceStatusInd", HFILL }},
    { &hf_nbap_pCH_Information1,
      { "pCH-Information", "nbap.pCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PCH_Information_ResourceStatusInd", HFILL }},
    { &hf_nbap_pICH_Information1,
      { "pICH-Information", "nbap.pICH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PICH_Information_ResourceStatusInd", HFILL }},
    { &hf_nbap_fACH_InformationList1,
      { "fACH-InformationList", "nbap.fACH_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.FACH_InformationList_ResourceStatusInd", HFILL }},
    { &hf_nbap_pRACH_InformationList1,
      { "pRACH-InformationList", "nbap.pRACH_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.PRACH_InformationList_ResourceStatusInd", HFILL }},
    { &hf_nbap_rACH_InformationList1,
      { "rACH-InformationList", "nbap.rACH_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.RACH_InformationList_ResourceStatusInd", HFILL }},
    { &hf_nbap_aICH_InformationList1,
      { "aICH-InformationList", "nbap.aICH_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.AICH_InformationList_ResourceStatusInd", HFILL }},
    { &hf_nbap_sCH_Information1,
      { "sCH-Information", "nbap.sCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.SCH_Information_ResourceStatusInd", HFILL }},
    { &hf_nbap_S_CPICH_InformationList_ResourceStatusInd_item,
      { "Item", "nbap.S_CPICH_InformationList_ResourceStatusInd_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_S_CCPCH_InformationList_ResourceStatusInd_item,
      { "Item", "nbap.S_CCPCH_InformationList_ResourceStatusInd_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_FACH_InformationList_ResourceStatusInd_item,
      { "Item", "nbap.FACH_InformationList_ResourceStatusInd_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_PRACH_InformationList_ResourceStatusInd_item,
      { "Item", "nbap.PRACH_InformationList_ResourceStatusInd_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_RACH_InformationList_ResourceStatusInd_item,
      { "Item", "nbap.RACH_InformationList_ResourceStatusInd_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_AICH_InformationList_ResourceStatusInd_item,
      { "Item", "nbap.AICH_InformationList_ResourceStatusInd_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_FPACH_LCR_InformationList_ResourceStatusInd_item,
      { "Item", "nbap.FPACH_LCR_InformationList_ResourceStatusInd_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_S_CCPCH_InformationListExt_ResourceStatusInd_item,
      { "Item", "nbap.S_CCPCH_InformationListExt_ResourceStatusInd_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_S_CCPCH_LCR_InformationListExt_ResourceStatusInd_item,
      { "Item", "nbap.S_CCPCH_LCR_InformationListExt_ResourceStatusInd_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_MIB_SB_SIB_InformationList_SystemInfoUpdateRqst_item,
      { "Item", "nbap.MIB_SB_SIB_InformationList_SystemInfoUpdateRqst_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.MIB_SB_SIB_InformationItem_SystemInfoUpdateRqst", HFILL }},
    { &hf_nbap_iB_Type,
      { "iB-Type", "nbap.iB_Type",
        FT_UINT32, BASE_DEC, VALS(nbap_IB_Type_vals), 0,
        "nbap.IB_Type", HFILL }},
    { &hf_nbap_iB_OC_ID,
      { "iB-OC-ID", "nbap.iB_OC_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.IB_OC_ID", HFILL }},
    { &hf_nbap_deletionIndicator,
      { "deletionIndicator", "nbap.deletionIndicator",
        FT_UINT32, BASE_DEC, VALS(nbap_DeletionIndicator_SystemInfoUpdate_vals), 0,
        "nbap.DeletionIndicator_SystemInfoUpdate", HFILL }},
    { &hf_nbap_no_Deletion,
      { "no-Deletion", "nbap.no_Deletion",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.No_Deletion_SystemInfoUpdate", HFILL }},
    { &hf_nbap_yes_Deletion,
      { "yes-Deletion", "nbap.yes_Deletion",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NULL", HFILL }},
    { &hf_nbap_sIB_Originator,
      { "sIB-Originator", "nbap.sIB_Originator",
        FT_UINT32, BASE_DEC, VALS(nbap_SIB_Originator_vals), 0,
        "nbap.SIB_Originator", HFILL }},
    { &hf_nbap_iB_SG_REP,
      { "iB-SG-REP", "nbap.iB_SG_REP",
        FT_UINT32, BASE_DEC, VALS(nbap_IB_SG_REP_vals), 0,
        "nbap.IB_SG_REP", HFILL }},
    { &hf_nbap_segmentInformationList,
      { "segmentInformationList", "nbap.segmentInformationList",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.SegmentInformationList_SystemInfoUpdate", HFILL }},
    { &hf_nbap_SegmentInformationListIE_SystemInfoUpdate_item,
      { "Item", "nbap.SegmentInformationListIE_SystemInfoUpdate_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.SegmentInformationItem_SystemInfoUpdate", HFILL }},
    { &hf_nbap_iB_SG_POS,
      { "iB-SG-POS", "nbap.iB_SG_POS",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.IB_SG_POS", HFILL }},
    { &hf_nbap_segment_Type,
      { "segment-Type", "nbap.segment_Type",
        FT_UINT32, BASE_DEC, VALS(nbap_Segment_Type_vals), 0,
        "nbap.Segment_Type", HFILL }},
    { &hf_nbap_iB_SG_DATA,
      { "iB-SG-DATA", "nbap.iB_SG_DATA",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.IB_SG_DATA", HFILL }},
    { &hf_nbap_ul_ScramblingCode,
      { "ul-ScramblingCode", "nbap.ul_ScramblingCode",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_ScramblingCode", HFILL }},
    { &hf_nbap_minUL_ChannelisationCodeLength,
      { "minUL-ChannelisationCodeLength", "nbap.minUL_ChannelisationCodeLength",
        FT_UINT32, BASE_DEC, VALS(nbap_MinUL_ChannelisationCodeLength_vals), 0,
        "nbap.MinUL_ChannelisationCodeLength", HFILL }},
    { &hf_nbap_maxNrOfUL_DPDCHs,
      { "maxNrOfUL-DPDCHs", "nbap.maxNrOfUL_DPDCHs",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.MaxNrOfUL_DPDCHs", HFILL }},
    { &hf_nbap_ul_PunctureLimit,
      { "ul-PunctureLimit", "nbap.ul_PunctureLimit",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.PunctureLimit", HFILL }},
    { &hf_nbap_ul_DPCCH_SlotFormat,
      { "ul-DPCCH-SlotFormat", "nbap.ul_DPCCH_SlotFormat",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.UL_DPCCH_SlotFormat", HFILL }},
    { &hf_nbap_ul_SIR_Target,
      { "ul-SIR-Target", "nbap.ul_SIR_Target",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.UL_SIR", HFILL }},
    { &hf_nbap_diversityMode,
      { "diversityMode", "nbap.diversityMode",
        FT_UINT32, BASE_DEC, VALS(nbap_DiversityMode_vals), 0,
        "nbap.DiversityMode", HFILL }},
    { &hf_nbap_not_Used_sSDT_CellID_Length,
      { "not-Used-sSDT-CellID-Length", "nbap.not_Used_sSDT_CellID_Length",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NULL", HFILL }},
    { &hf_nbap_not_Used_s_FieldLength,
      { "not-Used-s-FieldLength", "nbap.not_Used_s_FieldLength",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NULL", HFILL }},
    { &hf_nbap_dl_DPCH_SlotFormat,
      { "dl-DPCH-SlotFormat", "nbap.dl_DPCH_SlotFormat",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_DPCH_SlotFormat", HFILL }},
    { &hf_nbap_tFCI_SignallingMode,
      { "tFCI-SignallingMode", "nbap.tFCI_SignallingMode",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.TFCI_SignallingMode", HFILL }},
    { &hf_nbap_not_Used_pDSCH_RL_ID,
      { "not-Used-pDSCH-RL-ID", "nbap.not_Used_pDSCH_RL_ID",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NULL", HFILL }},
    { &hf_nbap_not_Used_pDSCH_CodeMapping,
      { "not-Used-pDSCH-CodeMapping", "nbap.not_Used_pDSCH_CodeMapping",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NULL", HFILL }},
    { &hf_nbap_powerOffsetInformation1,
      { "powerOffsetInformation", "nbap.powerOffsetInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PowerOffsetInformation_RL_SetupRqstFDD", HFILL }},
    { &hf_nbap_fdd_TPC_DownlinkStepSize,
      { "fdd-TPC-DownlinkStepSize", "nbap.fdd_TPC_DownlinkStepSize",
        FT_UINT32, BASE_DEC, VALS(nbap_FDD_TPC_DownlinkStepSize_vals), 0,
        "nbap.FDD_TPC_DownlinkStepSize", HFILL }},
    { &hf_nbap_limitedPowerIncrease,
      { "limitedPowerIncrease", "nbap.limitedPowerIncrease",
        FT_UINT32, BASE_DEC, VALS(nbap_LimitedPowerIncrease_vals), 0,
        "nbap.LimitedPowerIncrease", HFILL }},
    { &hf_nbap_innerLoopDLPCStatus,
      { "innerLoopDLPCStatus", "nbap.innerLoopDLPCStatus",
        FT_UINT32, BASE_DEC, VALS(nbap_InnerLoopDLPCStatus_vals), 0,
        "nbap.InnerLoopDLPCStatus", HFILL }},
    { &hf_nbap_pO2_ForTPC_Bits,
      { "pO2-ForTPC-Bits", "nbap.pO2_ForTPC_Bits",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.PowerOffset", HFILL }},
    { &hf_nbap_RL_InformationList_RL_SetupRqstFDD_item,
      { "Item", "nbap.RL_InformationList_RL_SetupRqstFDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_rL_ID,
      { "rL-ID", "nbap.rL_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.RL_ID", HFILL }},
    { &hf_nbap_firstRLS_indicator,
      { "firstRLS-indicator", "nbap.firstRLS_indicator",
        FT_UINT32, BASE_DEC, VALS(nbap_FirstRLS_Indicator_vals), 0,
        "nbap.FirstRLS_Indicator", HFILL }},
    { &hf_nbap_frameOffset,
      { "frameOffset", "nbap.frameOffset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.FrameOffset", HFILL }},
    { &hf_nbap_chipOffset,
      { "chipOffset", "nbap.chipOffset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.ChipOffset", HFILL }},
    { &hf_nbap_propagationDelay,
      { "propagationDelay", "nbap.propagationDelay",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.PropagationDelay", HFILL }},
    { &hf_nbap_diversityControlField,
      { "diversityControlField", "nbap.diversityControlField",
        FT_UINT32, BASE_DEC, VALS(nbap_DiversityControlField_vals), 0,
        "nbap.DiversityControlField", HFILL }},
    { &hf_nbap_dl_CodeInformation,
      { "dl-CodeInformation", "nbap.dl_CodeInformation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.FDD_DL_CodeInformation", HFILL }},
    { &hf_nbap_initialDL_transmissionPower,
      { "initialDL-transmissionPower", "nbap.initialDL_transmissionPower",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_maximumDL_power,
      { "maximumDL-power", "nbap.maximumDL_power",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_minimumDL_power,
      { "minimumDL-power", "nbap.minimumDL_power",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_not_Used_sSDT_Cell_Identity,
      { "not-Used-sSDT-Cell-Identity", "nbap.not_Used_sSDT_Cell_Identity",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NULL", HFILL }},
    { &hf_nbap_maxSet_E_DPDCHs,
      { "maxSet-E-DPDCHs", "nbap.maxSet_E_DPDCHs",
        FT_UINT32, BASE_DEC, VALS(nbap_Max_Set_E_DPDCHs_vals), 0,
        "nbap.Max_Set_E_DPDCHs", HFILL }},
    { &hf_nbap_e_TFCS_Information,
      { "e-TFCS-Information", "nbap.e_TFCS_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.E_TFCS_Information", HFILL }},
    { &hf_nbap_e_TTI,
      { "e-TTI", "nbap.e_TTI",
        FT_UINT32, BASE_DEC, VALS(nbap_E_TTI_vals), 0,
        "nbap.E_TTI", HFILL }},
    { &hf_nbap_e_DPCCH_PO,
      { "e-DPCCH-PO", "nbap.e_DPCCH_PO",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.E_DPCCH_PO", HFILL }},
    { &hf_nbap_powerOffsetInformation2,
      { "powerOffsetInformation", "nbap.powerOffsetInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PowerOffsetInformation_F_DPCH_RL_SetupRqstFDD", HFILL }},
    { &hf_nbap_UL_CCTrCH_InformationList_RL_SetupRqstTDD_item,
      { "Item", "nbap.UL_CCTrCH_InformationList_RL_SetupRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_uL_DPCH_Information,
      { "uL-DPCH-Information", "nbap.uL_DPCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_DPCH_Information_RL_SetupRqstTDD", HFILL }},
    { &hf_nbap_tdd_DPCHOffset,
      { "tdd-DPCHOffset", "nbap.tdd_DPCHOffset",
        FT_UINT32, BASE_DEC, VALS(nbap_TDD_DPCHOffset_vals), 0,
        "nbap.TDD_DPCHOffset", HFILL }},
    { &hf_nbap_uL_Timeslot_Information,
      { "uL-Timeslot-Information", "nbap.uL_Timeslot_Information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.UL_Timeslot_Information", HFILL }},
    { &hf_nbap_uL_TimeslotLCR_Information,
      { "uL-TimeslotLCR-Information", "nbap.uL_TimeslotLCR_Information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.UL_TimeslotLCR_Information", HFILL }},
    { &hf_nbap_DL_CCTrCH_InformationList_RL_SetupRqstTDD_item,
      { "Item", "nbap.DL_CCTrCH_InformationList_RL_SetupRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_tdd_TPC_DownlinkStepSize,
      { "tdd-TPC-DownlinkStepSize", "nbap.tdd_TPC_DownlinkStepSize",
        FT_UINT32, BASE_DEC, VALS(nbap_TDD_TPC_DownlinkStepSize_vals), 0,
        "nbap.TDD_TPC_DownlinkStepSize", HFILL }},
    { &hf_nbap_cCTrCH_TPCList,
      { "cCTrCH-TPCList", "nbap.cCTrCH_TPCList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CCTrCH_TPCList_RL_SetupRqstTDD", HFILL }},
    { &hf_nbap_dL_DPCH_Information,
      { "dL-DPCH-Information", "nbap.dL_DPCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_DPCH_Information_RL_SetupRqstTDD", HFILL }},
    { &hf_nbap_CCTrCH_TPCList_RL_SetupRqstTDD_item,
      { "Item", "nbap.CCTrCH_TPCList_RL_SetupRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CCTrCH_TPCItem_RL_SetupRqstTDD", HFILL }},
    { &hf_nbap_dL_Timeslot_Information,
      { "dL-Timeslot-Information", "nbap.dL_Timeslot_Information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_Timeslot_Information", HFILL }},
    { &hf_nbap_dL_TimeslotLCR_Information,
      { "dL-TimeslotLCR-Information", "nbap.dL_TimeslotLCR_Information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_TimeslotLCR_Information", HFILL }},
    { &hf_nbap_tstdIndicator,
      { "tstdIndicator", "nbap.tstdIndicator",
        FT_UINT32, BASE_DEC, VALS(nbap_TSTD_Indicator_vals), 0,
        "nbap.TSTD_Indicator", HFILL }},
    { &hf_nbap_specialBurstScheduling,
      { "specialBurstScheduling", "nbap.specialBurstScheduling",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SpecialBurstScheduling", HFILL }},
    { &hf_nbap_dL_TimeSlotISCPInfo,
      { "dL-TimeSlotISCPInfo", "nbap.dL_TimeSlotISCPInfo",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_TimeslotISCPInfo", HFILL }},
    { &hf_nbap_RL_InformationResponseList_RL_SetupRspFDD_item,
      { "Item", "nbap.RL_InformationResponseList_RL_SetupRspFDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_rL_Set_ID,
      { "rL-Set-ID", "nbap.rL_Set_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.RL_Set_ID", HFILL }},
    { &hf_nbap_received_total_wide_band_power,
      { "received-total-wide-band-power", "nbap.received_total_wide_band_power",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Received_total_wide_band_power_Value", HFILL }},
    { &hf_nbap_diversityIndication,
      { "diversityIndication", "nbap.diversityIndication",
        FT_UINT32, BASE_DEC, VALS(nbap_DiversityIndication_RL_SetupRspFDD_vals), 0,
        "nbap.DiversityIndication_RL_SetupRspFDD", HFILL }},
    { &hf_nbap_not_Used_dSCH_InformationResponseList,
      { "not-Used-dSCH-InformationResponseList", "nbap.not_Used_dSCH_InformationResponseList",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NULL", HFILL }},
    { &hf_nbap_sSDT_SupportIndicator,
      { "sSDT-SupportIndicator", "nbap.sSDT_SupportIndicator",
        FT_UINT32, BASE_DEC, VALS(nbap_SSDT_SupportIndicator_vals), 0,
        "nbap.SSDT_SupportIndicator", HFILL }},
    { &hf_nbap_combining,
      { "combining", "nbap.combining",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Combining_RL_SetupRspFDD", HFILL }},
    { &hf_nbap_nonCombiningOrFirstRL,
      { "nonCombiningOrFirstRL", "nbap.nonCombiningOrFirstRL",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NonCombiningOrFirstRL_RL_SetupRspFDD", HFILL }},
    { &hf_nbap_dCH_InformationResponse,
      { "dCH-InformationResponse", "nbap.dCH_InformationResponse",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DCH_InformationResponse", HFILL }},
    { &hf_nbap_uL_TimeSlot_ISCP_Info,
      { "uL-TimeSlot-ISCP-Info", "nbap.uL_TimeSlot_ISCP_Info",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.UL_TimeSlot_ISCP_Info", HFILL }},
    { &hf_nbap_ul_PhysCH_SF_Variation,
      { "ul-PhysCH-SF-Variation", "nbap.ul_PhysCH_SF_Variation",
        FT_UINT32, BASE_DEC, VALS(nbap_UL_PhysCH_SF_Variation_vals), 0,
        "nbap.UL_PhysCH_SF_Variation", HFILL }},
    { &hf_nbap_dCH_InformationResponseList,
      { "dCH-InformationResponseList", "nbap.dCH_InformationResponseList",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DCH_InformationResponseList_RL_SetupRspTDD", HFILL }},
    { &hf_nbap_dSCH_InformationResponseList,
      { "dSCH-InformationResponseList", "nbap.dSCH_InformationResponseList",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DSCH_InformationResponseList_RL_SetupRspTDD", HFILL }},
    { &hf_nbap_uSCH_InformationResponseList,
      { "uSCH-InformationResponseList", "nbap.uSCH_InformationResponseList",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.USCH_InformationResponseList_RL_SetupRspTDD", HFILL }},
    { &hf_nbap_uL_TimeSlot_ISCP_LCR_Info,
      { "uL-TimeSlot-ISCP-LCR-Info", "nbap.uL_TimeSlot_ISCP_LCR_Info",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.UL_TimeSlot_ISCP_LCR_Info", HFILL }},
    { &hf_nbap_generalCause,
      { "generalCause", "nbap.generalCause",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.GeneralCauseList_RL_SetupFailureFDD", HFILL }},
    { &hf_nbap_rLSpecificCause,
      { "rLSpecificCause", "nbap.rLSpecificCause",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RLSpecificCauseList_RL_SetupFailureFDD", HFILL }},
    { &hf_nbap_cause,
      { "cause", "nbap.cause",
        FT_UINT32, BASE_DEC, VALS(nbap_Cause_vals), 0,
        "nbap.Cause", HFILL }},
    { &hf_nbap_unsuccessful_RL_InformationRespList_RL_SetupFailureFDD,
      { "unsuccessful-RL-InformationRespList-RL-SetupFailureFDD", "nbap.unsuccessful_RL_InformationRespList_RL_SetupFailureFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Unsuccessful_RL_InformationRespList_RL_SetupFailureFDD", HFILL }},
    { &hf_nbap_successful_RL_InformationRespList_RL_SetupFailureFDD,
      { "successful-RL-InformationRespList-RL-SetupFailureFDD", "nbap.successful_RL_InformationRespList_RL_SetupFailureFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Successful_RL_InformationRespList_RL_SetupFailureFDD", HFILL }},
    { &hf_nbap_Unsuccessful_RL_InformationRespList_RL_SetupFailureFDD_item,
      { "Item", "nbap.Unsuccessful_RL_InformationRespList_RL_SetupFailureFDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_Successful_RL_InformationRespList_RL_SetupFailureFDD_item,
      { "Item", "nbap.Successful_RL_InformationRespList_RL_SetupFailureFDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_diversityIndication1,
      { "diversityIndication", "nbap.diversityIndication",
        FT_UINT32, BASE_DEC, VALS(nbap_DiversityIndication_RL_SetupFailureFDD_vals), 0,
        "nbap.DiversityIndication_RL_SetupFailureFDD", HFILL }},
    { &hf_nbap_not_Used_tFCI2_BearerInformationResponse,
      { "not-Used-tFCI2-BearerInformationResponse", "nbap.not_Used_tFCI2_BearerInformationResponse",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NULL", HFILL }},
    { &hf_nbap_combining1,
      { "combining", "nbap.combining",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Combining_RL_SetupFailureFDD", HFILL }},
    { &hf_nbap_nonCombiningOrFirstRL1,
      { "nonCombiningOrFirstRL", "nbap.nonCombiningOrFirstRL",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NonCombiningOrFirstRL_RL_SetupFailureFDD", HFILL }},
    { &hf_nbap_generalCause1,
      { "generalCause", "nbap.generalCause",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.GeneralCauseList_RL_SetupFailureTDD", HFILL }},
    { &hf_nbap_rLSpecificCause1,
      { "rLSpecificCause", "nbap.rLSpecificCause",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RLSpecificCauseList_RL_SetupFailureTDD", HFILL }},
    { &hf_nbap_unsuccessful_RL_InformationRespItem_RL_SetupFailureTDD,
      { "unsuccessful-RL-InformationRespItem-RL-SetupFailureTDD", "nbap.unsuccessful_RL_InformationRespItem_RL_SetupFailureTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Unsuccessful_RL_InformationRespItem_RL_SetupFailureTDD", HFILL }},
    { &hf_nbap_RL_InformationList_RL_AdditionRqstFDD_item,
      { "Item", "nbap.RL_InformationList_RL_AdditionRqstFDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_initialDL_TransmissionPower,
      { "initialDL-TransmissionPower", "nbap.initialDL_TransmissionPower",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_maximumDL_Power,
      { "maximumDL-Power", "nbap.maximumDL_Power",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_minimumDL_Power,
      { "minimumDL-Power", "nbap.minimumDL_Power",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_not_Used_sSDT_CellIdentity,
      { "not-Used-sSDT-CellIdentity", "nbap.not_Used_sSDT_CellIdentity",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NULL", HFILL }},
    { &hf_nbap_UL_CCTrCH_InformationList_RL_AdditionRqstTDD_item,
      { "Item", "nbap.UL_CCTrCH_InformationList_RL_AdditionRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_CCTrCH_InformationItem_RL_AdditionRqstTDD", HFILL }},
    { &hf_nbap_uL_DPCH_Information1,
      { "uL-DPCH-Information", "nbap.uL_DPCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_DPCH_InformationList_RL_AdditionRqstTDD", HFILL }},
    { &hf_nbap_DL_CCTrCH_InformationList_RL_AdditionRqstTDD_item,
      { "Item", "nbap.DL_CCTrCH_InformationList_RL_AdditionRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_CCTrCH_InformationItem_RL_AdditionRqstTDD", HFILL }},
    { &hf_nbap_dL_DPCH_Information1,
      { "dL-DPCH-Information", "nbap.dL_DPCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_DPCH_InformationList_RL_AdditionRqstTDD", HFILL }},
    { &hf_nbap_initial_DL_Transmission_Power,
      { "initial-DL-Transmission-Power", "nbap.initial_DL_Transmission_Power",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_RL_InformationResponseList_RL_AdditionRspFDD_item,
      { "Item", "nbap.RL_InformationResponseList_RL_AdditionRspFDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_diversityIndication2,
      { "diversityIndication", "nbap.diversityIndication",
        FT_UINT32, BASE_DEC, VALS(nbap_DiversityIndication_RL_AdditionRspFDD_vals), 0,
        "nbap.DiversityIndication_RL_AdditionRspFDD", HFILL }},
    { &hf_nbap_combining2,
      { "combining", "nbap.combining",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Combining_RL_AdditionRspFDD", HFILL }},
    { &hf_nbap_non_combining,
      { "non-combining", "nbap.non_combining",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Non_Combining_RL_AdditionRspFDD", HFILL }},
    { &hf_nbap_dCH_Information,
      { "dCH-Information", "nbap.dCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DCH_Information_RL_AdditionRspTDD", HFILL }},
    { &hf_nbap_dSCH_InformationResponseList1,
      { "dSCH-InformationResponseList", "nbap.dSCH_InformationResponseList",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DSCH_InformationResponseList_RL_AdditionRspTDD", HFILL }},
    { &hf_nbap_uSCH_InformationResponseList1,
      { "uSCH-InformationResponseList", "nbap.uSCH_InformationResponseList",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.USCH_InformationResponseList_RL_AdditionRspTDD", HFILL }},
    { &hf_nbap_diversityIndication3,
      { "diversityIndication", "nbap.diversityIndication",
        FT_UINT32, BASE_DEC, VALS(nbap_DiversityIndication_RL_AdditionRspTDD_vals), 0,
        "nbap.DiversityIndication_RL_AdditionRspTDD", HFILL }},
    { &hf_nbap_combining3,
      { "combining", "nbap.combining",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Combining_RL_AdditionRspTDD", HFILL }},
    { &hf_nbap_non_Combining,
      { "non-Combining", "nbap.non_Combining",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Non_Combining_RL_AdditionRspTDD", HFILL }},
    { &hf_nbap_uL_TimeSlot_ISCP_InfoLCR,
      { "uL-TimeSlot-ISCP-InfoLCR", "nbap.uL_TimeSlot_ISCP_InfoLCR",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.UL_TimeSlot_ISCP_LCR_Info", HFILL }},
    { &hf_nbap_generalCause2,
      { "generalCause", "nbap.generalCause",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.GeneralCauseList_RL_AdditionFailureFDD", HFILL }},
    { &hf_nbap_rLSpecificCause2,
      { "rLSpecificCause", "nbap.rLSpecificCause",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RLSpecificCauseList_RL_AdditionFailureFDD", HFILL }},
    { &hf_nbap_unsuccessful_RL_InformationRespList_RL_AdditionFailureFDD,
      { "unsuccessful-RL-InformationRespList-RL-AdditionFailureFDD", "nbap.unsuccessful_RL_InformationRespList_RL_AdditionFailureFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Unsuccessful_RL_InformationRespList_RL_AdditionFailureFDD", HFILL }},
    { &hf_nbap_successful_RL_InformationRespList_RL_AdditionFailureFDD,
      { "successful-RL-InformationRespList-RL-AdditionFailureFDD", "nbap.successful_RL_InformationRespList_RL_AdditionFailureFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Successful_RL_InformationRespList_RL_AdditionFailureFDD", HFILL }},
    { &hf_nbap_Unsuccessful_RL_InformationRespList_RL_AdditionFailureFDD_item,
      { "Item", "nbap.Unsuccessful_RL_InformationRespList_RL_AdditionFailureFDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_Successful_RL_InformationRespList_RL_AdditionFailureFDD_item,
      { "Item", "nbap.Successful_RL_InformationRespList_RL_AdditionFailureFDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_diversityIndication4,
      { "diversityIndication", "nbap.diversityIndication",
        FT_UINT32, BASE_DEC, VALS(nbap_DiversityIndication_RL_AdditionFailureFDD_vals), 0,
        "nbap.DiversityIndication_RL_AdditionFailureFDD", HFILL }},
    { &hf_nbap_combining4,
      { "combining", "nbap.combining",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Combining_RL_AdditionFailureFDD", HFILL }},
    { &hf_nbap_non_Combining1,
      { "non-Combining", "nbap.non_Combining",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Non_Combining_RL_AdditionFailureFDD", HFILL }},
    { &hf_nbap_generalCause3,
      { "generalCause", "nbap.generalCause",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.GeneralCauseList_RL_AdditionFailureTDD", HFILL }},
    { &hf_nbap_rLSpecificCause3,
      { "rLSpecificCause", "nbap.rLSpecificCause",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RLSpecificCauseList_RL_AdditionFailureTDD", HFILL }},
    { &hf_nbap_unsuccessful_RL_InformationRespItem_RL_AdditionFailureTDD,
      { "unsuccessful-RL-InformationRespItem-RL-AdditionFailureTDD", "nbap.unsuccessful_RL_InformationRespItem_RL_AdditionFailureTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Unsuccessful_RL_InformationRespItem_RL_AdditionFailureTDD", HFILL }},
    { &hf_nbap_not_Used_sSDT_CellIDLength,
      { "not-Used-sSDT-CellIDLength", "nbap.not_Used_sSDT_CellIDLength",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NULL", HFILL }},
    { &hf_nbap_powerOffsetInformation3,
      { "powerOffsetInformation", "nbap.powerOffsetInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PowerOffsetInformation_RL_ReconfPrepFDD", HFILL }},
    { &hf_nbap_DCH_DeleteList_RL_ReconfPrepFDD_item,
      { "Item", "nbap.DCH_DeleteList_RL_ReconfPrepFDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DCH_DeleteItem_RL_ReconfPrepFDD", HFILL }},
    { &hf_nbap_dCH_ID,
      { "dCH-ID", "nbap.dCH_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DCH_ID", HFILL }},
    { &hf_nbap_RL_InformationList_RL_ReconfPrepFDD_item,
      { "Item", "nbap.RL_InformationList_RL_ReconfPrepFDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_maxDL_Power,
      { "maxDL-Power", "nbap.maxDL_Power",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_minDL_Power,
      { "minDL-Power", "nbap.minDL_Power",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_not_Used_sSDT_Indication,
      { "not-Used-sSDT-Indication", "nbap.not_Used_sSDT_Indication",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NULL", HFILL }},
    { &hf_nbap_powerOffsetInformation4,
      { "powerOffsetInformation", "nbap.powerOffsetInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PowerOffsetInformation_F_DPCH_RL_ReconfPrepFDD", HFILL }},
    { &hf_nbap_UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD_item,
      { "Item", "nbap.UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_CCTrCH_InformationAddItem_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_ul_DPCH_InformationList,
      { "ul-DPCH-InformationList", "nbap.ul_DPCH_InformationList",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_DPCH_InformationAddList_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_MultipleRL_UL_DPCH_InformationAddList_RL_ReconfPrepTDD_item,
      { "Item", "nbap.MultipleRL_UL_DPCH_InformationAddList_RL_ReconfPrepTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.MultipleRL_UL_DPCH_InformationAddListIE_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_ul_DPCH_InformationListLCR,
      { "ul-DPCH-InformationListLCR", "nbap.ul_DPCH_InformationListLCR",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_DPCH_LCR_InformationAddList_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_ul_sir_target,
      { "ul-sir-target", "nbap.ul_sir_target",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.UL_SIR", HFILL }},
    { &hf_nbap_tDD_TPC_UplinkStepSize_LCR,
      { "tDD-TPC-UplinkStepSize-LCR", "nbap.tDD_TPC_UplinkStepSize_LCR",
        FT_UINT32, BASE_DEC, VALS(nbap_TDD_TPC_UplinkStepSize_LCR_vals), 0,
        "nbap.TDD_TPC_UplinkStepSize_LCR", HFILL }},
    { &hf_nbap_uL_Timeslot_InformationLCR,
      { "uL-Timeslot-InformationLCR", "nbap.uL_Timeslot_InformationLCR",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.UL_TimeslotLCR_Information", HFILL }},
    { &hf_nbap_UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD_item,
      { "Item", "nbap.UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_CCTrCH_InformationModifyItem_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_ul_DPCH_InformationAddList,
      { "ul-DPCH-InformationAddList", "nbap.ul_DPCH_InformationAddList",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_DPCH_InformationModify_AddList_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_ul_DPCH_InformationModifyList,
      { "ul-DPCH-InformationModifyList", "nbap.ul_DPCH_InformationModifyList",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_DPCH_InformationModify_ModifyList_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_ul_DPCH_InformationDeleteList,
      { "ul-DPCH-InformationDeleteList", "nbap.ul_DPCH_InformationDeleteList",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_DPCH_InformationModify_DeleteList_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_MultipleRL_UL_DPCH_InformationModifyList_RL_ReconfPrepTDD_item,
      { "Item", "nbap.MultipleRL_UL_DPCH_InformationModifyList_RL_ReconfPrepTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.MultipleRL_UL_DPCH_InformationModifyListIE_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_ul_DPCH_InformationAddListLCR,
      { "ul-DPCH-InformationAddListLCR", "nbap.ul_DPCH_InformationAddListLCR",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_DPCH_LCR_InformationModify_AddList_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_uL_Timeslot_InformationModify_ModifyList_RL_ReconfPrepTDD,
      { "uL-Timeslot-InformationModify-ModifyList-RL-ReconfPrepTDD", "nbap.uL_Timeslot_InformationModify_ModifyList_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.UL_Timeslot_InformationModify_ModifyList_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_UL_Timeslot_InformationModify_ModifyList_RL_ReconfPrepTDD_item,
      { "Item", "nbap.UL_Timeslot_InformationModify_ModifyList_RL_ReconfPrepTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_Timeslot_InformationModify_ModifyItem_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_midambleShiftAndBurstType,
      { "midambleShiftAndBurstType", "nbap.midambleShiftAndBurstType",
        FT_UINT32, BASE_DEC, VALS(nbap_MidambleShiftAndBurstType_vals), 0,
        "nbap.MidambleShiftAndBurstType", HFILL }},
    { &hf_nbap_uL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD,
      { "uL-Code-InformationModify-ModifyList-RL-ReconfPrepTDD", "nbap.uL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.UL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_UL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD_item,
      { "Item", "nbap.UL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_Code_InformationModify_ModifyItem_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_dPCH_ID,
      { "dPCH-ID", "nbap.dPCH_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DPCH_ID", HFILL }},
    { &hf_nbap_UL_TimeslotLCR_InformationModify_ModifyList_RL_ReconfPrepTDD_item,
      { "Item", "nbap.UL_TimeslotLCR_InformationModify_ModifyList_RL_ReconfPrepTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_Timeslot_LCR_InformationModify_ModifyItem_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_uL_Code_InformationModify_ModifyList_RL_ReconfPrepTDDLCR,
      { "uL-Code-InformationModify-ModifyList-RL-ReconfPrepTDDLCR", "nbap.uL_Code_InformationModify_ModifyList_RL_ReconfPrepTDDLCR",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.UL_Code_InformationModify_ModifyList_RL_ReconfPrepTDDLCR", HFILL }},
    { &hf_nbap_UL_Code_InformationModify_ModifyList_RL_ReconfPrepTDDLCR_item,
      { "Item", "nbap.UL_Code_InformationModify_ModifyList_RL_ReconfPrepTDDLCR_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_Code_InformationModify_ModifyItem_RL_ReconfPrepTDDLCR", HFILL }},
    { &hf_nbap_UL_DPCH_InformationModify_DeleteListIE_RL_ReconfPrepTDD_item,
      { "Item", "nbap.UL_DPCH_InformationModify_DeleteListIE_RL_ReconfPrepTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_DPCH_InformationModify_DeleteItem_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD_item,
      { "Item", "nbap.UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_CCTrCH_InformationDeleteItem_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD_item,
      { "Item", "nbap.DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_CCTrCH_InformationAddItem_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_cCTrCH_TPCList1,
      { "cCTrCH-TPCList", "nbap.cCTrCH_TPCList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CCTrCH_TPCAddList_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_dl_DPCH_InformationList,
      { "dl-DPCH-InformationList", "nbap.dl_DPCH_InformationList",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_DPCH_InformationAddList_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_MultipleRL_DL_DPCH_InformationAddList_RL_ReconfPrepTDD_item,
      { "Item", "nbap.MultipleRL_DL_DPCH_InformationAddList_RL_ReconfPrepTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.MultipleRL_DL_DPCH_InformationAddListIE_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_dl_DPCH_InformationListLCR,
      { "dl-DPCH-InformationListLCR", "nbap.dl_DPCH_InformationListLCR",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_DPCH_LCR_InformationAddList_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_cCTrCH_Initial_DL_Power,
      { "cCTrCH-Initial-DL-Power", "nbap.cCTrCH_Initial_DL_Power",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_tDD_TPC_DownlinkStepSize,
      { "tDD-TPC-DownlinkStepSize", "nbap.tDD_TPC_DownlinkStepSize",
        FT_UINT32, BASE_DEC, VALS(nbap_TDD_TPC_DownlinkStepSize_vals), 0,
        "nbap.TDD_TPC_DownlinkStepSize", HFILL }},
    { &hf_nbap_cCTrCH_Maximum_DL_Power_InformationAdd_RL_ReconfPrepTDD,
      { "cCTrCH-Maximum-DL-Power-InformationAdd-RL-ReconfPrepTDD", "nbap.cCTrCH_Maximum_DL_Power_InformationAdd_RL_ReconfPrepTDD",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_cCTrCH_Minimum_DL_Power_InformationAdd_RL_ReconfPrepTDD,
      { "cCTrCH-Minimum-DL-Power-InformationAdd-RL-ReconfPrepTDD", "nbap.cCTrCH_Minimum_DL_Power_InformationAdd_RL_ReconfPrepTDD",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_CCTrCH_TPCAddList_RL_ReconfPrepTDD_item,
      { "Item", "nbap.CCTrCH_TPCAddList_RL_ReconfPrepTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CCTrCH_TPCAddItem_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_dL_Timeslot_InformationLCR,
      { "dL-Timeslot-InformationLCR", "nbap.dL_Timeslot_InformationLCR",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_TimeslotLCR_Information", HFILL }},
    { &hf_nbap_DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD_item,
      { "Item", "nbap.DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_CCTrCH_InformationModifyItem_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_cCTrCH_TPCList2,
      { "cCTrCH-TPCList", "nbap.cCTrCH_TPCList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CCTrCH_TPCModifyList_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_dl_DPCH_InformationAddList,
      { "dl-DPCH-InformationAddList", "nbap.dl_DPCH_InformationAddList",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_DPCH_InformationModify_AddList_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_dl_DPCH_InformationModifyList,
      { "dl-DPCH-InformationModifyList", "nbap.dl_DPCH_InformationModifyList",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_DPCH_InformationModify_ModifyList_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_dl_DPCH_InformationDeleteList,
      { "dl-DPCH-InformationDeleteList", "nbap.dl_DPCH_InformationDeleteList",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_DPCH_InformationModify_DeleteList_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_MultipleRL_DL_DPCH_InformationModifyList_RL_ReconfPrepTDD_item,
      { "Item", "nbap.MultipleRL_DL_DPCH_InformationModifyList_RL_ReconfPrepTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.MultipleRL_DL_DPCH_InformationModifyListIE_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_dl_DPCH_InformationAddListLCR,
      { "dl-DPCH-InformationAddListLCR", "nbap.dl_DPCH_InformationAddListLCR",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_DPCH_LCR_InformationModify_AddList_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_tDD_TPC_DownlinkStepSize_InformationModify_RL_ReconfPrepTDD,
      { "tDD-TPC-DownlinkStepSize-InformationModify-RL-ReconfPrepTDD", "nbap.tDD_TPC_DownlinkStepSize_InformationModify_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, VALS(nbap_TDD_TPC_DownlinkStepSize_vals), 0,
        "nbap.TDD_TPC_DownlinkStepSize", HFILL }},
    { &hf_nbap_cCTrCH_Maximum_DL_Power_InformationModify_RL_ReconfPrepTDD,
      { "cCTrCH-Maximum-DL-Power-InformationModify-RL-ReconfPrepTDD", "nbap.cCTrCH_Maximum_DL_Power_InformationModify_RL_ReconfPrepTDD",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_cCTrCH_Minimum_DL_Power_InformationModify_RL_ReconfPrepTDD,
      { "cCTrCH-Minimum-DL-Power-InformationModify-RL-ReconfPrepTDD", "nbap.cCTrCH_Minimum_DL_Power_InformationModify_RL_ReconfPrepTDD",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_CCTrCH_TPCModifyList_RL_ReconfPrepTDD_item,
      { "Item", "nbap.CCTrCH_TPCModifyList_RL_ReconfPrepTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CCTrCH_TPCModifyItem_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_dL_Timeslot_InformationAddModify_ModifyList_RL_ReconfPrepTDD,
      { "dL-Timeslot-InformationAddModify-ModifyList-RL-ReconfPrepTDD", "nbap.dL_Timeslot_InformationAddModify_ModifyList_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_Timeslot_InformationModify_ModifyList_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_DL_Timeslot_InformationModify_ModifyList_RL_ReconfPrepTDD_item,
      { "Item", "nbap.DL_Timeslot_InformationModify_ModifyList_RL_ReconfPrepTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_Timeslot_InformationModify_ModifyItem_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_dL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD,
      { "dL-Code-InformationModify-ModifyList-RL-ReconfPrepTDD", "nbap.dL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_DL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD_item,
      { "Item", "nbap.DL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_Code_InformationModify_ModifyItem_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_DL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD_item,
      { "Item", "nbap.DL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_Timeslot_LCR_InformationModify_ModifyItem_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_dL_Code_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD,
      { "dL-Code-LCR-InformationModify-ModifyList-RL-ReconfPrepTDD", "nbap.dL_Code_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_Code_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_DL_Code_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD_item,
      { "Item", "nbap.DL_Code_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_Code_LCR_InformationModify_ModifyItem_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_DL_DPCH_InformationModify_DeleteListIE_RL_ReconfPrepTDD_item,
      { "Item", "nbap.DL_DPCH_InformationModify_DeleteListIE_RL_ReconfPrepTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_DPCH_InformationModify_DeleteItem_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD_item,
      { "Item", "nbap.DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_CCTrCH_InformationDeleteItem_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_DCH_DeleteList_RL_ReconfPrepTDD_item,
      { "Item", "nbap.DCH_DeleteList_RL_ReconfPrepTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DCH_DeleteItem_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_DSCH_Information_ModifyList_RL_ReconfPrepTDD_item,
      { "Item", "nbap.DSCH_Information_ModifyList_RL_ReconfPrepTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DSCH_Information_ModifyItem_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_dSCH_ID,
      { "dSCH-ID", "nbap.dSCH_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DSCH_ID", HFILL }},
    { &hf_nbap_allocationRetentionPriority,
      { "allocationRetentionPriority", "nbap.allocationRetentionPriority",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.AllocationRetentionPriority", HFILL }},
    { &hf_nbap_frameHandlingPriority,
      { "frameHandlingPriority", "nbap.frameHandlingPriority",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.FrameHandlingPriority", HFILL }},
    { &hf_nbap_transportBearerRequestIndicator,
      { "transportBearerRequestIndicator", "nbap.transportBearerRequestIndicator",
        FT_UINT32, BASE_DEC, VALS(nbap_TransportBearerRequestIndicator_vals), 0,
        "nbap.TransportBearerRequestIndicator", HFILL }},
    { &hf_nbap_DSCH_Information_DeleteList_RL_ReconfPrepTDD_item,
      { "Item", "nbap.DSCH_Information_DeleteList_RL_ReconfPrepTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DSCH_Information_DeleteItem_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_USCH_Information_ModifyList_RL_ReconfPrepTDD_item,
      { "Item", "nbap.USCH_Information_ModifyList_RL_ReconfPrepTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.USCH_Information_ModifyItem_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_uSCH_ID,
      { "uSCH-ID", "nbap.uSCH_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.USCH_ID", HFILL }},
    { &hf_nbap_USCH_Information_DeleteList_RL_ReconfPrepTDD_item,
      { "Item", "nbap.USCH_Information_DeleteList_RL_ReconfPrepTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.USCH_Information_DeleteItem_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_MultipleRL_Information_RL_ReconfPrepTDD_item,
      { "Item", "nbap.MultipleRL_Information_RL_ReconfPrepTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_Information_RL_ReconfPrepTDD", HFILL }},
    { &hf_nbap_RL_InformationResponseList_RL_ReconfReady_item,
      { "Item", "nbap.RL_InformationResponseList_RL_ReconfReady_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_dCH_InformationResponseList_RL_ReconfReady,
      { "dCH-InformationResponseList-RL-ReconfReady", "nbap.dCH_InformationResponseList_RL_ReconfReady",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DCH_InformationResponseList_RL_ReconfReady", HFILL }},
    { &hf_nbap_dSCH_InformationResponseList_RL_ReconfReady,
      { "dSCH-InformationResponseList-RL-ReconfReady", "nbap.dSCH_InformationResponseList_RL_ReconfReady",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DSCH_InformationResponseList_RL_ReconfReady", HFILL }},
    { &hf_nbap_uSCH_InformationResponseList_RL_ReconfReady,
      { "uSCH-InformationResponseList-RL-ReconfReady", "nbap.uSCH_InformationResponseList_RL_ReconfReady",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.USCH_InformationResponseList_RL_ReconfReady", HFILL }},
    { &hf_nbap_generalCause4,
      { "generalCause", "nbap.generalCause",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.GeneralCauseList_RL_ReconfFailure", HFILL }},
    { &hf_nbap_rLSpecificCause4,
      { "rLSpecificCause", "nbap.rLSpecificCause",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RLSpecificCauseList_RL_ReconfFailure", HFILL }},
    { &hf_nbap_rL_ReconfigurationFailureList_RL_ReconfFailure,
      { "rL-ReconfigurationFailureList-RL-ReconfFailure", "nbap.rL_ReconfigurationFailureList_RL_ReconfFailure",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.RL_ReconfigurationFailureList_RL_ReconfFailure", HFILL }},
    { &hf_nbap_RL_ReconfigurationFailureList_RL_ReconfFailure_item,
      { "Item", "nbap.RL_ReconfigurationFailureList_RL_ReconfFailure_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_ul_TFCS,
      { "ul-TFCS", "nbap.ul_TFCS",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.TFCS", HFILL }},
    { &hf_nbap_dl_TFCS,
      { "dl-TFCS", "nbap.dl_TFCS",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.TFCS", HFILL }},
    { &hf_nbap_DCH_DeleteList_RL_ReconfRqstFDD_item,
      { "Item", "nbap.DCH_DeleteList_RL_ReconfRqstFDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DCH_DeleteItem_RL_ReconfRqstFDD", HFILL }},
    { &hf_nbap_RL_InformationList_RL_ReconfRqstFDD_item,
      { "Item", "nbap.RL_InformationList_RL_ReconfRqstFDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD_item,
      { "Item", "nbap.UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD_item,
      { "Item", "nbap.UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD_item,
      { "Item", "nbap.DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_MultipleRL_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD_item,
      { "Item", "nbap.MultipleRL_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.MultipleRL_DL_CCTrCH_InformationModifyListIE_RL_ReconfRqstTDD", HFILL }},
    { &hf_nbap_dl_DPCH_LCR_InformationModifyList,
      { "dl-DPCH-LCR-InformationModifyList", "nbap.dl_DPCH_LCR_InformationModifyList",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_DPCH_LCR_InformationModify_ModifyList_RL_ReconfRqstTDD", HFILL }},
    { &hf_nbap_cCTrCH_Maximum_DL_Power_InformationModify_RL_ReconfRqstTDD,
      { "cCTrCH-Maximum-DL-Power-InformationModify-RL-ReconfRqstTDD", "nbap.cCTrCH_Maximum_DL_Power_InformationModify_RL_ReconfRqstTDD",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_cCTrCH_Minimum_DL_Power_InformationModify_RL_ReconfRqstTDD,
      { "cCTrCH-Minimum-DL-Power-InformationModify-RL-ReconfRqstTDD", "nbap.cCTrCH_Minimum_DL_Power_InformationModify_RL_ReconfRqstTDD",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_dL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfRqstTDD,
      { "dL-Timeslot-LCR-InformationModify-ModifyList-RL-ReconfRqstTDD", "nbap.dL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfRqstTDD", HFILL }},
    { &hf_nbap_DL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfRqstTDD_item,
      { "Item", "nbap.DL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_Timeslot_LCR_InformationModify_ModifyItem_RL_ReconfRqstTDD", HFILL }},
    { &hf_nbap_maxPowerLCR,
      { "maxPowerLCR", "nbap.maxPowerLCR",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_minPowerLCR,
      { "minPowerLCR", "nbap.minPowerLCR",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD_item,
      { "Item", "nbap.DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_DCH_DeleteList_RL_ReconfRqstTDD_item,
      { "Item", "nbap.DCH_DeleteList_RL_ReconfRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DCH_DeleteItem_RL_ReconfRqstTDD", HFILL }},
    { &hf_nbap_Multiple_RL_Information_RL_ReconfRqstTDD_item,
      { "Item", "nbap.Multiple_RL_Information_RL_ReconfRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_Information_RL_ReconfRqstTDD", HFILL }},
    { &hf_nbap_RL_InformationResponseList_RL_ReconfRsp_item,
      { "Item", "nbap.RL_InformationResponseList_RL_ReconfRsp_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_dCH_InformationResponseList_RL_ReconfRsp,
      { "dCH-InformationResponseList-RL-ReconfRsp", "nbap.dCH_InformationResponseList_RL_ReconfRsp",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DCH_InformationResponseList_RL_ReconfRsp", HFILL }},
    { &hf_nbap_RL_informationList_RL_DeletionRqst_item,
      { "Item", "nbap.RL_informationList_RL_DeletionRqst_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_DL_ReferencePowerInformationList_DL_PC_Rqst_item,
      { "Item", "nbap.DL_ReferencePowerInformationList_DL_PC_Rqst_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_dl_ReferencePower,
      { "dl-ReferencePower", "nbap.dl_ReferencePower",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_rL,
      { "rL", "nbap.rL",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_DM_Rqst", HFILL }},
    { &hf_nbap_rLS,
      { "rLS", "nbap.rLS",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_Set_DM_Rqst", HFILL }},
    { &hf_nbap_all_RL,
      { "all-RL", "nbap.all_RL",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.AllRL_DM_Rqst", HFILL }},
    { &hf_nbap_all_RLS,
      { "all-RLS", "nbap.all_RLS",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.AllRL_Set_DM_Rqst", HFILL }},
    { &hf_nbap_rL_InformationList,
      { "rL-InformationList", "nbap.rL_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.RL_InformationList_DM_Rqst", HFILL }},
    { &hf_nbap_RL_InformationList_DM_Rqst_item,
      { "Item", "nbap.RL_InformationList_DM_Rqst_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_PUSCH_Info_DM_Rqst_item,
      { "Item", "nbap.PUSCH_Info_DM_Rqst_item",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.PUSCH_ID", HFILL }},
    { &hf_nbap_HSSICH_Info_DM_Rqst_item,
      { "Item", "nbap.HSSICH_Info_DM_Rqst_item",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HS_SICH_ID", HFILL }},
    { &hf_nbap_rL_Set_InformationList_DM_Rqst,
      { "rL-Set-InformationList-DM-Rqst", "nbap.rL_Set_InformationList_DM_Rqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.RL_Set_InformationList_DM_Rqst", HFILL }},
    { &hf_nbap_RL_Set_InformationList_DM_Rqst_item,
      { "Item", "nbap.RL_Set_InformationList_DM_Rqst_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_Set_InformationItem_DM_Rqst", HFILL }},
    { &hf_nbap_rL1,
      { "rL", "nbap.rL",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_DM_Rsp", HFILL }},
    { &hf_nbap_rLS1,
      { "rLS", "nbap.rLS",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_Set_DM_Rsp", HFILL }},
    { &hf_nbap_all_RL1,
      { "all-RL", "nbap.all_RL",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_DM_Rsp", HFILL }},
    { &hf_nbap_all_RLS1,
      { "all-RLS", "nbap.all_RLS",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_Set_DM_Rsp", HFILL }},
    { &hf_nbap_rL_InformationList_DM_Rsp,
      { "rL-InformationList-DM-Rsp", "nbap.rL_InformationList_DM_Rsp",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.RL_InformationList_DM_Rsp", HFILL }},
    { &hf_nbap_RL_InformationList_DM_Rsp_item,
      { "Item", "nbap.RL_InformationList_DM_Rsp_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_dedicatedMeasurementValue,
      { "dedicatedMeasurementValue", "nbap.dedicatedMeasurementValue",
        FT_UINT32, BASE_DEC, VALS(nbap_DedicatedMeasurementValue_vals), 0,
        "nbap.DedicatedMeasurementValue", HFILL }},
    { &hf_nbap_cFN,
      { "cFN", "nbap.cFN",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CFN", HFILL }},
    { &hf_nbap_PUSCH_Info_DM_Rsp_item,
      { "Item", "nbap.PUSCH_Info_DM_Rsp_item",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.PUSCH_ID", HFILL }},
    { &hf_nbap_Multiple_PUSCH_InfoList_DM_Rsp_item,
      { "Item", "nbap.Multiple_PUSCH_InfoList_DM_Rsp_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Multiple_PUSCH_InfoListIE_DM_Rsp", HFILL }},
    { &hf_nbap_pUSCH_ID,
      { "pUSCH-ID", "nbap.pUSCH_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.PUSCH_ID", HFILL }},
    { &hf_nbap_rL_Set_InformationList_DM_Rsp,
      { "rL-Set-InformationList-DM-Rsp", "nbap.rL_Set_InformationList_DM_Rsp",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.RL_Set_InformationList_DM_Rsp", HFILL }},
    { &hf_nbap_RL_Set_InformationList_DM_Rsp_item,
      { "Item", "nbap.RL_Set_InformationList_DM_Rsp_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_Multiple_DedicatedMeasurementValueList_TDD_DM_Rsp_item,
      { "Item", "nbap.Multiple_DedicatedMeasurementValueList_TDD_DM_Rsp_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Multiple_DedicatedMeasurementValueItem_TDD_DM_Rsp", HFILL }},
    { &hf_nbap_Multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp_item,
      { "Item", "nbap.Multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Multiple_DedicatedMeasurementValueItem_LCR_TDD_DM_Rsp", HFILL }},
    { &hf_nbap_rL2,
      { "rL", "nbap.rL",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_DM_Rprt", HFILL }},
    { &hf_nbap_rLS2,
      { "rLS", "nbap.rLS",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_Set_DM_Rprt", HFILL }},
    { &hf_nbap_all_RL2,
      { "all-RL", "nbap.all_RL",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_DM_Rprt", HFILL }},
    { &hf_nbap_all_RLS2,
      { "all-RLS", "nbap.all_RLS",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_Set_DM_Rprt", HFILL }},
    { &hf_nbap_rL_InformationList_DM_Rprt,
      { "rL-InformationList-DM-Rprt", "nbap.rL_InformationList_DM_Rprt",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.RL_InformationList_DM_Rprt", HFILL }},
    { &hf_nbap_RL_InformationList_DM_Rprt_item,
      { "Item", "nbap.RL_InformationList_DM_Rprt_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_dedicatedMeasurementValueInformation,
      { "dedicatedMeasurementValueInformation", "nbap.dedicatedMeasurementValueInformation",
        FT_UINT32, BASE_DEC, VALS(nbap_DedicatedMeasurementValueInformation_vals), 0,
        "nbap.DedicatedMeasurementValueInformation", HFILL }},
    { &hf_nbap_PUSCH_Info_DM_Rprt_item,
      { "Item", "nbap.PUSCH_Info_DM_Rprt_item",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.PUSCH_ID", HFILL }},
    { &hf_nbap_Multiple_PUSCH_InfoList_DM_Rprt_item,
      { "Item", "nbap.Multiple_PUSCH_InfoList_DM_Rprt_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Multiple_PUSCH_InfoListIE_DM_Rprt", HFILL }},
    { &hf_nbap_rL_Set_InformationList_DM_Rprt,
      { "rL-Set-InformationList-DM-Rprt", "nbap.rL_Set_InformationList_DM_Rprt",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.RL_Set_InformationList_DM_Rprt", HFILL }},
    { &hf_nbap_RL_Set_InformationList_DM_Rprt_item,
      { "Item", "nbap.RL_Set_InformationList_DM_Rprt_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_rL3,
      { "rL", "nbap.rL",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_RL_FailureInd", HFILL }},
    { &hf_nbap_rL_Set,
      { "rL-Set", "nbap.rL_Set",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_Set_RL_FailureInd", HFILL }},
    { &hf_nbap_cCTrCH,
      { "cCTrCH", "nbap.cCTrCH",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CCTrCH_RL_FailureInd", HFILL }},
    { &hf_nbap_rL_InformationList_RL_FailureInd,
      { "rL-InformationList-RL-FailureInd", "nbap.rL_InformationList_RL_FailureInd",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.RL_InformationList_RL_FailureInd", HFILL }},
    { &hf_nbap_RL_InformationList_RL_FailureInd_item,
      { "Item", "nbap.RL_InformationList_RL_FailureInd_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_rL_Set_InformationList_RL_FailureInd,
      { "rL-Set-InformationList-RL-FailureInd", "nbap.rL_Set_InformationList_RL_FailureInd",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.RL_Set_InformationList_RL_FailureInd", HFILL }},
    { &hf_nbap_RL_Set_InformationList_RL_FailureInd_item,
      { "Item", "nbap.RL_Set_InformationList_RL_FailureInd_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_cCTrCH_InformationList_RL_FailureInd,
      { "cCTrCH-InformationList-RL-FailureInd", "nbap.cCTrCH_InformationList_RL_FailureInd",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CCTrCH_InformationList_RL_FailureInd", HFILL }},
    { &hf_nbap_CCTrCH_InformationList_RL_FailureInd_item,
      { "Item", "nbap.CCTrCH_InformationList_RL_FailureInd_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_RL_InformationList_RL_PreemptRequiredInd_item,
      { "Item", "nbap.RL_InformationList_RL_PreemptRequiredInd_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_rL4,
      { "rL", "nbap.rL",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_RL_RestoreInd", HFILL }},
    { &hf_nbap_rL_Set1,
      { "rL-Set", "nbap.rL_Set",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_Set_RL_RestoreInd", HFILL }},
    { &hf_nbap_cCTrCH1,
      { "cCTrCH", "nbap.cCTrCH",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CCTrCH_RL_RestoreInd", HFILL }},
    { &hf_nbap_rL_InformationList_RL_RestoreInd,
      { "rL-InformationList-RL-RestoreInd", "nbap.rL_InformationList_RL_RestoreInd",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.RL_InformationList_RL_RestoreInd", HFILL }},
    { &hf_nbap_RL_InformationList_RL_RestoreInd_item,
      { "Item", "nbap.RL_InformationList_RL_RestoreInd_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_rL_Set_InformationList_RL_RestoreInd,
      { "rL-Set-InformationList-RL-RestoreInd", "nbap.rL_Set_InformationList_RL_RestoreInd",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.RL_Set_InformationList_RL_RestoreInd", HFILL }},
    { &hf_nbap_RL_Set_InformationList_RL_RestoreInd_item,
      { "Item", "nbap.RL_Set_InformationList_RL_RestoreInd_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_cCTrCH_InformationList_RL_RestoreInd,
      { "cCTrCH-InformationList-RL-RestoreInd", "nbap.cCTrCH_InformationList_RL_RestoreInd",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CCTrCH_InformationList_RL_RestoreInd", HFILL }},
    { &hf_nbap_CCTrCH_InformationList_RL_RestoreInd_item,
      { "Item", "nbap.CCTrCH_InformationList_RL_RestoreInd_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_privateIEs,
      { "privateIEs", "nbap.privateIEs",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.PrivateIE_Container", HFILL }},
    { &hf_nbap_HSDPA_And_EDCH_CellPortion_InformationList_PSCH_ReconfRqst_item,
      { "Item", "nbap.HSDPA_And_EDCH_CellPortion_InformationList_PSCH_ReconfRqst_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HSDPA_And_EDCH_CellPortion_InformationItem_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_hS_PDSCH_HS_SCCH_ScramblingCode_PSCH_ReconfRqst,
      { "hS-PDSCH-HS-SCCH-ScramblingCode-PSCH-ReconfRqst", "nbap.hS_PDSCH_HS_SCCH_ScramblingCode_PSCH_ReconfRqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_ScramblingCode", HFILL }},
    { &hf_nbap_hS_PDSCH_FDD_Code_Information_PSCH_ReconfRqst,
      { "hS-PDSCH-FDD-Code-Information-PSCH-ReconfRqst", "nbap.hS_PDSCH_FDD_Code_Information_PSCH_ReconfRqst",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HS_PDSCH_FDD_Code_Information", HFILL }},
    { &hf_nbap_hS_SCCH_FDD_Code_Information_PSCH_ReconfRqst,
      { "hS-SCCH-FDD-Code-Information-PSCH-ReconfRqst", "nbap.hS_SCCH_FDD_Code_Information_PSCH_ReconfRqst",
        FT_UINT32, BASE_DEC, VALS(nbap_HS_SCCH_FDD_Code_Information_vals), 0,
        "nbap.HS_SCCH_FDD_Code_Information", HFILL }},
    { &hf_nbap_hS_PDSCH_HS_SCCH_E_AGCH_E_RGCH_E_HICH_MaxPower_PSCH_ReconfRqst,
      { "hS-PDSCH-HS-SCCH-E-AGCH-E-RGCH-E-HICH-MaxPower-PSCH-ReconfRqst", "nbap.hS_PDSCH_HS_SCCH_E_AGCH_E_RGCH_E_HICH_MaxPower_PSCH_ReconfRqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.MaximumTransmissionPower", HFILL }},
    { &hf_nbap_PDSCHSets_AddList_PSCH_ReconfRqst_item,
      { "Item", "nbap.PDSCHSets_AddList_PSCH_ReconfRqst_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PDSCHSets_AddItem_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_pDSCHSet_ID,
      { "pDSCHSet-ID", "nbap.pDSCHSet_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.PDSCHSet_ID", HFILL }},
    { &hf_nbap_pDSCH_InformationList,
      { "pDSCH-InformationList", "nbap.pDSCH_InformationList",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PDSCH_Information_AddList_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_dL_Timeslot_InformationAddList_PSCH_ReconfRqst,
      { "dL-Timeslot-InformationAddList-PSCH-ReconfRqst", "nbap.dL_Timeslot_InformationAddList_PSCH_ReconfRqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_Timeslot_InformationAddList_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_DL_Timeslot_InformationAddList_PSCH_ReconfRqst_item,
      { "Item", "nbap.DL_Timeslot_InformationAddList_PSCH_ReconfRqst_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_Timeslot_InformationAddItem_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_dL_Code_InformationAddList_PSCH_ReconfRqst,
      { "dL-Code-InformationAddList-PSCH-ReconfRqst", "nbap.dL_Code_InformationAddList_PSCH_ReconfRqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_Code_InformationAddList_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_DL_Code_InformationAddList_PSCH_ReconfRqst_item,
      { "Item", "nbap.DL_Code_InformationAddList_PSCH_ReconfRqst_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_Code_InformationAddItem_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_pDSCH_ID,
      { "pDSCH-ID", "nbap.pDSCH_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.PDSCH_ID", HFILL }},
    { &hf_nbap_dL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst,
      { "dL-Timeslot-InformationAddList-LCR-PSCH-ReconfRqst", "nbap.dL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_DL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst_item,
      { "Item", "nbap.DL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_Timeslot_InformationAddItem_LCR_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_dL_Code_InformationAddList_LCR_PSCH_ReconfRqst,
      { "dL-Code-InformationAddList-LCR-PSCH-ReconfRqst", "nbap.dL_Code_InformationAddList_LCR_PSCH_ReconfRqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_Code_InformationAddList_LCR_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_DL_Code_InformationAddList_LCR_PSCH_ReconfRqst_item,
      { "Item", "nbap.DL_Code_InformationAddList_LCR_PSCH_ReconfRqst_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_Code_InformationAddItem_LCR_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_PDSCHSets_ModifyList_PSCH_ReconfRqst_item,
      { "Item", "nbap.PDSCHSets_ModifyList_PSCH_ReconfRqst_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PDSCHSets_ModifyItem_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_pDSCH_InformationList1,
      { "pDSCH-InformationList", "nbap.pDSCH_InformationList",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PDSCH_Information_ModifyList_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_dL_Timeslot_InformationModifyList_PSCH_ReconfRqst,
      { "dL-Timeslot-InformationModifyList-PSCH-ReconfRqst", "nbap.dL_Timeslot_InformationModifyList_PSCH_ReconfRqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_Timeslot_InformationModifyList_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_DL_Timeslot_InformationModifyList_PSCH_ReconfRqst_item,
      { "Item", "nbap.DL_Timeslot_InformationModifyList_PSCH_ReconfRqst_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_Timeslot_InformationModifyItem_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_dL_Code_InformationModifyList_PSCH_ReconfRqst,
      { "dL-Code-InformationModifyList-PSCH-ReconfRqst", "nbap.dL_Code_InformationModifyList_PSCH_ReconfRqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_Code_InformationModifyList_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_DL_Code_InformationModifyList_PSCH_ReconfRqst_item,
      { "Item", "nbap.DL_Code_InformationModifyList_PSCH_ReconfRqst_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_Code_InformationModifyItem_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_dL_Timeslot_LCR_InformationModifyList_PSCH_ReconfRqst,
      { "dL-Timeslot-LCR-InformationModifyList-PSCH-ReconfRqst", "nbap.dL_Timeslot_LCR_InformationModifyList_PSCH_ReconfRqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_Timeslot_LCR_InformationModifyList_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_DL_Timeslot_LCR_InformationModifyList_PSCH_ReconfRqst_item,
      { "Item", "nbap.DL_Timeslot_LCR_InformationModifyList_PSCH_ReconfRqst_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_Timeslot_LCR_InformationModifyItem_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_dL_Code_LCR_InformationModifyList_PSCH_ReconfRqst,
      { "dL-Code-LCR-InformationModifyList-PSCH-ReconfRqst", "nbap.dL_Code_LCR_InformationModifyList_PSCH_ReconfRqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_Code_LCR_InformationModifyList_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_DL_Code_LCR_InformationModifyList_PSCH_ReconfRqst_item,
      { "Item", "nbap.DL_Code_LCR_InformationModifyList_PSCH_ReconfRqst_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_Code_LCR_InformationModifyItem_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_PDSCHSets_DeleteList_PSCH_ReconfRqst_item,
      { "Item", "nbap.PDSCHSets_DeleteList_PSCH_ReconfRqst_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PDSCHSets_DeleteItem_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_PUSCHSets_AddList_PSCH_ReconfRqst_item,
      { "Item", "nbap.PUSCHSets_AddList_PSCH_ReconfRqst_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PUSCHSets_AddItem_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_pUSCHSet_ID,
      { "pUSCHSet-ID", "nbap.pUSCHSet_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.PUSCHSet_ID", HFILL }},
    { &hf_nbap_pUSCH_InformationList,
      { "pUSCH-InformationList", "nbap.pUSCH_InformationList",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PUSCH_Information_AddList_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_uL_Timeslot_InformationAddList_PSCH_ReconfRqst,
      { "uL-Timeslot-InformationAddList-PSCH-ReconfRqst", "nbap.uL_Timeslot_InformationAddList_PSCH_ReconfRqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.UL_Timeslot_InformationAddList_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_UL_Timeslot_InformationAddList_PSCH_ReconfRqst_item,
      { "Item", "nbap.UL_Timeslot_InformationAddList_PSCH_ReconfRqst_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_Timeslot_InformationAddItem_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_uL_Code_InformationAddList_PSCH_ReconfRqst,
      { "uL-Code-InformationAddList-PSCH-ReconfRqst", "nbap.uL_Code_InformationAddList_PSCH_ReconfRqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.UL_Code_InformationAddList_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_UL_Code_InformationAddList_PSCH_ReconfRqst_item,
      { "Item", "nbap.UL_Code_InformationAddList_PSCH_ReconfRqst_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_Code_InformationAddItem_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_uL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst,
      { "uL-Timeslot-InformationAddList-LCR-PSCH-ReconfRqst", "nbap.uL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.UL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_UL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst_item,
      { "Item", "nbap.UL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_Timeslot_InformationAddItem_LCR_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_uL_Code_InformationAddList_LCR_PSCH_ReconfRqst,
      { "uL-Code-InformationAddList-LCR-PSCH-ReconfRqst", "nbap.uL_Code_InformationAddList_LCR_PSCH_ReconfRqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.UL_Code_InformationAddList_LCR_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_UL_Code_InformationAddList_LCR_PSCH_ReconfRqst_item,
      { "Item", "nbap.UL_Code_InformationAddList_LCR_PSCH_ReconfRqst_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_Code_InformationAddItem_LCR_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_PUSCHSets_ModifyList_PSCH_ReconfRqst_item,
      { "Item", "nbap.PUSCHSets_ModifyList_PSCH_ReconfRqst_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PUSCHSets_ModifyItem_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_pUSCH_InformationList1,
      { "pUSCH-InformationList", "nbap.pUSCH_InformationList",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PUSCH_Information_ModifyList_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_uL_Timeslot_InformationModifyList_PSCH_ReconfRqst,
      { "uL-Timeslot-InformationModifyList-PSCH-ReconfRqst", "nbap.uL_Timeslot_InformationModifyList_PSCH_ReconfRqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.UL_Timeslot_InformationModifyList_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_UL_Timeslot_InformationModifyList_PSCH_ReconfRqst_item,
      { "Item", "nbap.UL_Timeslot_InformationModifyList_PSCH_ReconfRqst_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_Timeslot_InformationModifyItem_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_uL_Code_InformationModifyList_PSCH_ReconfRqst,
      { "uL-Code-InformationModifyList-PSCH-ReconfRqst", "nbap.uL_Code_InformationModifyList_PSCH_ReconfRqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.UL_Code_InformationModifyList_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_UL_Code_InformationModifyList_PSCH_ReconfRqst_item,
      { "Item", "nbap.UL_Code_InformationModifyList_PSCH_ReconfRqst_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_Code_InformationModifyItem_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_uL_Timeslot_InformationModifyList_LCR_PSCH_ReconfRqst,
      { "uL-Timeslot-InformationModifyList-LCR-PSCH-ReconfRqst", "nbap.uL_Timeslot_InformationModifyList_LCR_PSCH_ReconfRqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.UL_Timeslot_LCR_InformationModifyList_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_UL_Timeslot_LCR_InformationModifyList_PSCH_ReconfRqst_item,
      { "Item", "nbap.UL_Timeslot_LCR_InformationModifyList_PSCH_ReconfRqst_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_Timeslot_LCR_InformationModifyItem_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_uL_Code_LCR_InformationModifyList_PSCH_ReconfRqst,
      { "uL-Code-LCR-InformationModifyList-PSCH-ReconfRqst", "nbap.uL_Code_LCR_InformationModifyList_PSCH_ReconfRqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.UL_Code_LCR_InformationModifyList_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_UL_Code_LCR_InformationModifyList_PSCH_ReconfRqst_item,
      { "Item", "nbap.UL_Code_LCR_InformationModifyList_PSCH_ReconfRqst_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_Code_LCR_InformationModifyItem_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_PUSCHSets_DeleteList_PSCH_ReconfRqst_item,
      { "Item", "nbap.PUSCHSets_DeleteList_PSCH_ReconfRqst_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PUSCHSets_DeleteItem_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_dL_HS_PDSCH_Timeslot_Information_PSCH_ReconfRqst,
      { "dL-HS-PDSCH-Timeslot-Information-PSCH-ReconfRqst", "nbap.dL_HS_PDSCH_Timeslot_Information_PSCH_ReconfRqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_HS_PDSCH_Timeslot_Information_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_dL_HS_PDSCH_Timeslot_Information_LCR_PSCH_ReconfRqst,
      { "dL-HS-PDSCH-Timeslot-Information-LCR-PSCH-ReconfRqst", "nbap.dL_HS_PDSCH_Timeslot_Information_LCR_PSCH_ReconfRqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_HS_PDSCH_Timeslot_Information_LCR_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_DL_HS_PDSCH_Timeslot_Information_PSCH_ReconfRqst_item,
      { "Item", "nbap.DL_HS_PDSCH_Timeslot_Information_PSCH_ReconfRqst_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_HS_PDSCH_Timeslot_InformationItem_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_dl_HS_PDSCH_Codelist_PSCH_ReconfRqst,
      { "dl-HS-PDSCH-Codelist-PSCH-ReconfRqst", "nbap.dl_HS_PDSCH_Codelist_PSCH_ReconfRqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_HS_PDSCH_Codelist_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_maxHSDSCH_HSSCCH_Power,
      { "maxHSDSCH-HSSCCH-Power", "nbap.maxHSDSCH_HSSCCH_Power",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.MaximumTransmissionPower", HFILL }},
    { &hf_nbap_DL_HS_PDSCH_Codelist_PSCH_ReconfRqst_item,
      { "Item", "nbap.DL_HS_PDSCH_Codelist_PSCH_ReconfRqst_item",
        FT_UINT32, BASE_DEC, VALS(nbap_TDD_ChannelisationCode_vals), 0,
        "nbap.TDD_ChannelisationCode", HFILL }},
    { &hf_nbap_DL_HS_PDSCH_Timeslot_Information_LCR_PSCH_ReconfRqst_item,
      { "Item", "nbap.DL_HS_PDSCH_Timeslot_Information_LCR_PSCH_ReconfRqst_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_HS_PDSCH_Timeslot_InformationItem_LCR_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_timeSlot1,
      { "timeSlot", "nbap.timeSlot",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.TimeSlotLCR", HFILL }},
    { &hf_nbap_midambleShiftAndBurstType1,
      { "midambleShiftAndBurstType", "nbap.midambleShiftAndBurstType",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.MidambleShiftLCR", HFILL }},
    { &hf_nbap_dl_HS_PDSCH_Codelist_LCR_PSCH_ReconfRqst,
      { "dl-HS-PDSCH-Codelist-LCR-PSCH-ReconfRqst", "nbap.dl_HS_PDSCH_Codelist_LCR_PSCH_ReconfRqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_HS_PDSCH_Codelist_LCR_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_DL_HS_PDSCH_Codelist_LCR_PSCH_ReconfRqst_item,
      { "Item", "nbap.DL_HS_PDSCH_Codelist_LCR_PSCH_ReconfRqst_item",
        FT_UINT32, BASE_DEC, VALS(nbap_TDD_ChannelisationCode_vals), 0,
        "nbap.TDD_ChannelisationCode", HFILL }},
    { &hf_nbap_hS_SCCH_Information_PSCH_ReconfRqst,
      { "hS-SCCH-Information-PSCH-ReconfRqst", "nbap.hS_SCCH_Information_PSCH_ReconfRqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HS_SCCH_Information_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_hS_SCCH_Information_LCR_PSCH_ReconfRqst,
      { "hS-SCCH-Information-LCR-PSCH-ReconfRqst", "nbap.hS_SCCH_Information_LCR_PSCH_ReconfRqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HS_SCCH_Information_LCR_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_HS_SCCH_Information_PSCH_ReconfRqst_item,
      { "Item", "nbap.HS_SCCH_Information_PSCH_ReconfRqst_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HS_SCCH_InformationItem_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_hS_SCCH_ID,
      { "hS-SCCH-ID", "nbap.hS_SCCH_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HS_SCCH_ID", HFILL }},
    { &hf_nbap_hS_SCCH_MaxPower,
      { "hS-SCCH-MaxPower", "nbap.hS_SCCH_MaxPower",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_hS_SICH_Information,
      { "hS-SICH-Information", "nbap.hS_SICH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HS_SICH_Information_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_hsSICH_ID,
      { "hsSICH-ID", "nbap.hsSICH_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HS_SICH_ID", HFILL }},
    { &hf_nbap_HS_SCCH_Information_LCR_PSCH_ReconfRqst_item,
      { "Item", "nbap.HS_SCCH_Information_LCR_PSCH_ReconfRqst_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HS_SCCH_InformationItem_LCR_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_first_TDD_ChannelisationCode,
      { "first-TDD-ChannelisationCode", "nbap.first_TDD_ChannelisationCode",
        FT_UINT32, BASE_DEC, VALS(nbap_TDD_ChannelisationCode_vals), 0,
        "nbap.TDD_ChannelisationCode", HFILL }},
    { &hf_nbap_second_TDD_ChannelisationCode,
      { "second-TDD-ChannelisationCode", "nbap.second_TDD_ChannelisationCode",
        FT_UINT32, BASE_DEC, VALS(nbap_TDD_ChannelisationCode_vals), 0,
        "nbap.TDD_ChannelisationCode", HFILL }},
    { &hf_nbap_hS_SICH_Information_LCR,
      { "hS-SICH-Information-LCR", "nbap.hS_SICH_Information_LCR",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HS_SICH_Information_LCR_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_hS_SCCH_InformationModify_PSCH_ReconfRqst,
      { "hS-SCCH-InformationModify-PSCH-ReconfRqst", "nbap.hS_SCCH_InformationModify_PSCH_ReconfRqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HS_SCCH_InformationModify_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_hS_SCCH_InformationModify_LCR_PSCH_ReconfRqst,
      { "hS-SCCH-InformationModify-LCR-PSCH-ReconfRqst", "nbap.hS_SCCH_InformationModify_LCR_PSCH_ReconfRqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HS_SCCH_InformationModify_LCR_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_HS_SCCH_InformationModify_PSCH_ReconfRqst_item,
      { "Item", "nbap.HS_SCCH_InformationModify_PSCH_ReconfRqst_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HS_SCCH_InformationModifyItem_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_hS_SICH_Information1,
      { "hS-SICH-Information", "nbap.hS_SICH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HS_SICH_InformationModify_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_HS_SCCH_InformationModify_LCR_PSCH_ReconfRqst_item,
      { "Item", "nbap.HS_SCCH_InformationModify_LCR_PSCH_ReconfRqst_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HS_SCCH_InformationModifyItem_LCR_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_hS_SICH_Information_LCR1,
      { "hS-SICH-Information-LCR", "nbap.hS_SICH_Information_LCR",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HS_SICH_InformationModify_LCR_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_Delete_From_HS_SCCH_Resource_Pool_PSCH_ReconfRqst_item,
      { "Item", "nbap.Delete_From_HS_SCCH_Resource_Pool_PSCH_ReconfRqst_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Delete_From_HS_SCCH_Resource_PoolItem_PSCH_ReconfRqst", HFILL }},
    { &hf_nbap_generalCause5,
      { "generalCause", "nbap.generalCause",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.GeneralCauseList_PSCH_ReconfFailure", HFILL }},
    { &hf_nbap_setSpecificCause,
      { "setSpecificCause", "nbap.setSpecificCause",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.SetSpecificCauseList_PSCH_ReconfFailureTDD", HFILL }},
    { &hf_nbap_unsuccessful_PDSCHSetList_PSCH_ReconfFailureTDD,
      { "unsuccessful-PDSCHSetList-PSCH-ReconfFailureTDD", "nbap.unsuccessful_PDSCHSetList_PSCH_ReconfFailureTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Unsuccessful_PDSCHSetList_PSCH_ReconfFailureTDD", HFILL }},
    { &hf_nbap_unsuccessful_PUSCHSetList_PSCH_ReconfFailureTDD,
      { "unsuccessful-PUSCHSetList-PSCH-ReconfFailureTDD", "nbap.unsuccessful_PUSCHSetList_PSCH_ReconfFailureTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Unsuccessful_PUSCHSetList_PSCH_ReconfFailureTDD", HFILL }},
    { &hf_nbap_Unsuccessful_PDSCHSetList_PSCH_ReconfFailureTDD_item,
      { "Item", "nbap.Unsuccessful_PDSCHSetList_PSCH_ReconfFailureTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_Unsuccessful_PUSCHSetList_PSCH_ReconfFailureTDD_item,
      { "Item", "nbap.Unsuccessful_PUSCHSetList_PSCH_ReconfFailureTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_communicationContext,
      { "communicationContext", "nbap.communicationContext",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CommunicationContextList_Reset", HFILL }},
    { &hf_nbap_communicationControlPort,
      { "communicationControlPort", "nbap.communicationControlPort",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CommunicationControlPortList_Reset", HFILL }},
    { &hf_nbap_nodeB,
      { "nodeB", "nbap.nodeB",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NULL", HFILL }},
    { &hf_nbap_communicationContextInfoList_Reset,
      { "communicationContextInfoList-Reset", "nbap.communicationContextInfoList_Reset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CommunicationContextInfoList_Reset", HFILL }},
    { &hf_nbap_CommunicationContextInfoList_Reset_item,
      { "Item", "nbap.CommunicationContextInfoList_Reset_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_communicationContextType_Reset,
      { "communicationContextType-Reset", "nbap.communicationContextType_Reset",
        FT_UINT32, BASE_DEC, VALS(nbap_CommunicationContextType_Reset_vals), 0,
        "nbap.CommunicationContextType_Reset", HFILL }},
    { &hf_nbap_cRNC_CommunicationContextID,
      { "cRNC-CommunicationContextID", "nbap.cRNC_CommunicationContextID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CRNC_CommunicationContextID", HFILL }},
    { &hf_nbap_nodeB_CommunicationContextID,
      { "nodeB-CommunicationContextID", "nbap.nodeB_CommunicationContextID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.NodeB_CommunicationContextID", HFILL }},
    { &hf_nbap_communicationControlPortInfoList_Reset,
      { "communicationControlPortInfoList-Reset", "nbap.communicationControlPortInfoList_Reset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CommunicationControlPortInfoList_Reset", HFILL }},
    { &hf_nbap_CommunicationControlPortInfoList_Reset_item,
      { "Item", "nbap.CommunicationControlPortInfoList_Reset_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_cell2,
      { "cell", "nbap.cell",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Cell_InfEx_Rqst", HFILL }},
    { &hf_nbap_cell3,
      { "cell", "nbap.cell",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Cell_InfEx_Rsp", HFILL }},
    { &hf_nbap_requestedDataValue,
      { "requestedDataValue", "nbap.requestedDataValue",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RequestedDataValue", HFILL }},
    { &hf_nbap_cell4,
      { "cell", "nbap.cell",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Cell_Inf_Rprt", HFILL }},
    { &hf_nbap_requestedDataValueInformation,
      { "requestedDataValueInformation", "nbap.requestedDataValueInformation",
        FT_UINT32, BASE_DEC, VALS(nbap_RequestedDataValueInformation_vals), 0,
        "nbap.RequestedDataValueInformation", HFILL }},
    { &hf_nbap_cSBTransmissionID,
      { "cSBTransmissionID", "nbap.cSBTransmissionID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CSBTransmissionID", HFILL }},
    { &hf_nbap_sfn,
      { "sfn", "nbap.sfn",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SFN", HFILL }},
    { &hf_nbap_cellSyncBurstCode,
      { "cellSyncBurstCode", "nbap.cellSyncBurstCode",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CellSyncBurstCode", HFILL }},
    { &hf_nbap_cellSyncBurstCodeShift,
      { "cellSyncBurstCodeShift", "nbap.cellSyncBurstCodeShift",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CellSyncBurstCodeShift", HFILL }},
    { &hf_nbap_initialDLTransPower,
      { "initialDLTransPower", "nbap.initialDLTransPower",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_TimeslotInfo_CellSyncInitiationRqstTDD_item,
      { "Item", "nbap.TimeslotInfo_CellSyncInitiationRqstTDD_item",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.TimeSlot", HFILL }},
    { &hf_nbap_cSBMeasurementID,
      { "cSBMeasurementID", "nbap.cSBMeasurementID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CSBMeasurementID", HFILL }},
    { &hf_nbap_synchronisationReportType,
      { "synchronisationReportType", "nbap.synchronisationReportType",
        FT_UINT32, BASE_DEC, VALS(nbap_SynchronisationReportType_vals), 0,
        "nbap.SynchronisationReportType", HFILL }},
    { &hf_nbap_synchronisationReportCharacteristics,
      { "synchronisationReportCharacteristics", "nbap.synchronisationReportCharacteristics",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.SynchronisationReportCharacteristics", HFILL }},
    { &hf_nbap_uARFCN,
      { "uARFCN", "nbap.uARFCN",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.UARFCN", HFILL }},
    { &hf_nbap_sYNCDlCodeId,
      { "sYNCDlCodeId", "nbap.sYNCDlCodeId",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SYNCDlCodeId", HFILL }},
    { &hf_nbap_CellSyncBurstTransReconfInfo_CellSyncReconfRqstTDD_item,
      { "Item", "nbap.CellSyncBurstTransReconfInfo_CellSyncReconfRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CellSyncBurstTransInfoItem_CellSyncReconfRqstTDD", HFILL }},
    { &hf_nbap_syncFrameNumberToTransmit,
      { "syncFrameNumberToTransmit", "nbap.syncFrameNumberToTransmit",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SyncFrameNumber", HFILL }},
    { &hf_nbap_dlTransPower,
      { "dlTransPower", "nbap.dlTransPower",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_cellSyncBurstMeasInfoList_CellSyncReconfRqstTDD,
      { "cellSyncBurstMeasInfoList-CellSyncReconfRqstTDD", "nbap.cellSyncBurstMeasInfoList_CellSyncReconfRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CellSyncBurstMeasInfoList_CellSyncReconfRqstTDD", HFILL }},
    { &hf_nbap_synchronisationReportType1,
      { "synchronisationReportType", "nbap.synchronisationReportType",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.SynchronisationReportTypeIE", HFILL }},
    { &hf_nbap_synchronisationReportCharacteristics1,
      { "synchronisationReportCharacteristics", "nbap.synchronisationReportCharacteristics",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.SynchronisationReportCharacteristicsIE", HFILL }},
    { &hf_nbap_CellSyncBurstMeasInfoListIE_CellSyncReconfRqstTDD_item,
      { "Item", "nbap.CellSyncBurstMeasInfoListIE_CellSyncReconfRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CellSyncBurstMeasInfoItem_CellSyncReconfRqstTDD", HFILL }},
    { &hf_nbap_syncFrameNrToReceive,
      { "syncFrameNrToReceive", "nbap.syncFrameNrToReceive",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SyncFrameNumber", HFILL }},
    { &hf_nbap_syncBurstInfo,
      { "syncBurstInfo", "nbap.syncBurstInfo",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CellSyncBurstInfoList_CellSyncReconfRqstTDD", HFILL }},
    { &hf_nbap_CellSyncBurstInfoList_CellSyncReconfRqstTDD_item,
      { "Item", "nbap.CellSyncBurstInfoList_CellSyncReconfRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CellSyncBurstInfoItem_CellSyncReconfRqstTDD", HFILL }},
    { &hf_nbap_SYNCDlCodeIdTransReconfInfoLCR_CellSyncReconfRqstTDD_item,
      { "Item", "nbap.SYNCDlCodeIdTransReconfInfoLCR_CellSyncReconfRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.SYNCDlCodeIdTransReconfItemLCR_CellSyncReconfRqstTDD", HFILL }},
    { &hf_nbap_syncFrameNumberforTransmit,
      { "syncFrameNumberforTransmit", "nbap.syncFrameNumberforTransmit",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SyncFrameNumber", HFILL }},
    { &hf_nbap_sYNCDlCodeIdMeasInfoList,
      { "sYNCDlCodeIdMeasInfoList", "nbap.sYNCDlCodeIdMeasInfoList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SYNCDlCodeIdMeasInfoList_CellSyncReconfRqstTDD", HFILL }},
    { &hf_nbap_SYNCDlCodeIdMeasInfoList_CellSyncReconfRqstTDD_item,
      { "Item", "nbap.SYNCDlCodeIdMeasInfoList_CellSyncReconfRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.SYNCDlCodeIdMeasInfoItem_CellSyncReconfRqstTDD", HFILL }},
    { &hf_nbap_sYNCDlCodeIdInfoLCR,
      { "sYNCDlCodeIdInfoLCR", "nbap.sYNCDlCodeIdInfoLCR",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SYNCDlCodeIdInfoListLCR_CellSyncReconfRqstTDD", HFILL }},
    { &hf_nbap_SYNCDlCodeIdInfoListLCR_CellSyncReconfRqstTDD_item,
      { "Item", "nbap.SYNCDlCodeIdInfoListLCR_CellSyncReconfRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.SYNCDlCodeIdInfoItemLCR_CellSyncReconfRqstTDD", HFILL }},
    { &hf_nbap_propagationDelayCompensation,
      { "propagationDelayCompensation", "nbap.propagationDelayCompensation",
        FT_UINT32, BASE_DEC, VALS(nbap_TimingAdjustmentValueLCR_vals), 0,
        "nbap.TimingAdjustmentValueLCR", HFILL }},
    { &hf_nbap_CellAdjustmentInfo_SyncAdjustmentRqstTDD_item,
      { "Item", "nbap.CellAdjustmentInfo_SyncAdjustmentRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_frameAdjustmentValue,
      { "frameAdjustmentValue", "nbap.frameAdjustmentValue",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.FrameAdjustmentValue", HFILL }},
    { &hf_nbap_timingAdjustmentValue,
      { "timingAdjustmentValue", "nbap.timingAdjustmentValue",
        FT_UINT32, BASE_DEC, VALS(nbap_TimingAdjustmentValue_vals), 0,
        "nbap.TimingAdjustmentValue", HFILL }},
    { &hf_nbap_dLTransPower,
      { "dLTransPower", "nbap.dLTransPower",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_generalCause6,
      { "generalCause", "nbap.generalCause",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.GeneralCauseList_SyncAdjustmntFailureTDD", HFILL }},
    { &hf_nbap_cellSpecificCause,
      { "cellSpecificCause", "nbap.cellSpecificCause",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CellSpecificCauseList_SyncAdjustmntFailureTDD", HFILL }},
    { &hf_nbap_unsuccessful_cell_InformationRespList_SyncAdjustmntFailureTDD,
      { "unsuccessful-cell-InformationRespList-SyncAdjustmntFailureTDD", "nbap.unsuccessful_cell_InformationRespList_SyncAdjustmntFailureTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Unsuccessful_cell_InformationRespList_SyncAdjustmntFailureTDD", HFILL }},
    { &hf_nbap_Unsuccessful_cell_InformationRespList_SyncAdjustmntFailureTDD_item,
      { "Item", "nbap.Unsuccessful_cell_InformationRespList_SyncAdjustmntFailureTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_CellSyncInfo_CellSyncReprtTDD_item,
      { "Item", "nbap.CellSyncInfo_CellSyncReprtTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CellSyncInfoItemIE_CellSyncReprtTDD", HFILL }},
    { &hf_nbap_c_ID_CellSyncReprtTDD,
      { "c-ID-CellSyncReprtTDD", "nbap.c_ID_CellSyncReprtTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.C_ID_IE_CellSyncReprtTDD", HFILL }},
    { &hf_nbap_syncReportType_CellSyncReprtTDD,
      { "syncReportType-CellSyncReprtTDD", "nbap.syncReportType_CellSyncReprtTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.SyncReportTypeIE_CellSyncReprtTDD", HFILL }},
    { &hf_nbap_intStdPhSyncInfo_CellSyncReprtTDD,
      { "intStdPhSyncInfo-CellSyncReprtTDD", "nbap.intStdPhSyncInfo_CellSyncReprtTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.IntStdPhCellSyncInfo_CellSyncReprtTDD", HFILL }},
    { &hf_nbap_lateEntrantCell,
      { "lateEntrantCell", "nbap.lateEntrantCell",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NULL", HFILL }},
    { &hf_nbap_frequencyAcquisition,
      { "frequencyAcquisition", "nbap.frequencyAcquisition",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NULL", HFILL }},
    { &hf_nbap_cellSyncBurstMeasuredInfo,
      { "cellSyncBurstMeasuredInfo", "nbap.cellSyncBurstMeasuredInfo",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CellSyncBurstMeasInfoList_CellSyncReprtTDD", HFILL }},
    { &hf_nbap_CellSyncBurstMeasInfoList_CellSyncReprtTDD_item,
      { "Item", "nbap.CellSyncBurstMeasInfoList_CellSyncReprtTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CellSyncBurstMeasInfoItem_CellSyncReprtTDD", HFILL }},
    { &hf_nbap_sFN,
      { "sFN", "nbap.sFN",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SFN", HFILL }},
    { &hf_nbap_cellSyncBurstInfo_CellSyncReprtTDD,
      { "cellSyncBurstInfo-CellSyncReprtTDD", "nbap.cellSyncBurstInfo_CellSyncReprtTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SEQUENCE_SIZE_1_16_OF_CellSyncBurstInfo_CellSyncReprtTDD", HFILL }},
    { &hf_nbap_cellSyncBurstInfo_CellSyncReprtTDD_item,
      { "Item", "nbap.cellSyncBurstInfo_CellSyncReprtTDD_item",
        FT_UINT32, BASE_DEC, VALS(nbap_CellSyncBurstInfo_CellSyncReprtTDD_vals), 0,
        "nbap.CellSyncBurstInfo_CellSyncReprtTDD", HFILL }},
    { &hf_nbap_cellSyncBurstAvailable,
      { "cellSyncBurstAvailable", "nbap.cellSyncBurstAvailable",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CellSyncBurstAvailable_CellSyncReprtTDD", HFILL }},
    { &hf_nbap_cellSyncBurstNotAvailable,
      { "cellSyncBurstNotAvailable", "nbap.cellSyncBurstNotAvailable",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NULL", HFILL }},
    { &hf_nbap_cellSyncBurstTiming,
      { "cellSyncBurstTiming", "nbap.cellSyncBurstTiming",
        FT_UINT32, BASE_DEC, VALS(nbap_CellSyncBurstTiming_vals), 0,
        "nbap.CellSyncBurstTiming", HFILL }},
    { &hf_nbap_cellSyncBurstSIR,
      { "cellSyncBurstSIR", "nbap.cellSyncBurstSIR",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CellSyncBurstSIR", HFILL }},
    { &hf_nbap_SyncDLCodeIdsMeasInfoList_CellSyncReprtTDD_item,
      { "Item", "nbap.SyncDLCodeIdsMeasInfoList_CellSyncReprtTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.SyncDLCodeIdsMeasInfoItem_CellSyncReprtTDD", HFILL }},
    { &hf_nbap_syncDLCodeIdInfo_CellSyncReprtTDD,
      { "syncDLCodeIdInfo-CellSyncReprtTDD", "nbap.syncDLCodeIdInfo_CellSyncReprtTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SyncDLCodeIdInfo_CellSyncReprtTDD", HFILL }},
    { &hf_nbap_SyncDLCodeIdInfo_CellSyncReprtTDD_item,
      { "Item", "nbap.SyncDLCodeIdInfo_CellSyncReprtTDD_item",
        FT_UINT32, BASE_DEC, VALS(nbap_SyncDLCodeIdItem_CellSyncReprtTDD_vals), 0,
        "nbap.SyncDLCodeIdItem_CellSyncReprtTDD", HFILL }},
    { &hf_nbap_syncDLCodeIdAvailable,
      { "syncDLCodeIdAvailable", "nbap.syncDLCodeIdAvailable",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.SyncDLCodeIdAvailable_CellSyncReprtTDD", HFILL }},
    { &hf_nbap_syncDLCodeIDNotAvailable,
      { "syncDLCodeIDNotAvailable", "nbap.syncDLCodeIDNotAvailable",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NULL", HFILL }},
    { &hf_nbap_syncDLCodeIdTiming,
      { "syncDLCodeIdTiming", "nbap.syncDLCodeIdTiming",
        FT_UINT32, BASE_DEC, VALS(nbap_CellSyncBurstTimingLCR_vals), 0,
        "nbap.CellSyncBurstTimingLCR", HFILL }},
    { &hf_nbap_syncDLCodeIdSIR,
      { "syncDLCodeIdSIR", "nbap.syncDLCodeIdSIR",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CellSyncBurstSIR", HFILL }},
    { &hf_nbap_DCH_RearrangeList_Bearer_RearrangeInd_item,
      { "Item", "nbap.DCH_RearrangeList_Bearer_RearrangeInd_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DCH_RearrangeItem_Bearer_RearrangeInd", HFILL }},
    { &hf_nbap_DSCH_RearrangeList_Bearer_RearrangeInd_item,
      { "Item", "nbap.DSCH_RearrangeList_Bearer_RearrangeInd_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DSCH_RearrangeItem_Bearer_RearrangeInd", HFILL }},
    { &hf_nbap_USCH_RearrangeList_Bearer_RearrangeInd_item,
      { "Item", "nbap.USCH_RearrangeList_Bearer_RearrangeInd_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.USCH_RearrangeItem_Bearer_RearrangeInd", HFILL }},
    { &hf_nbap_HSDSCH_RearrangeList_Bearer_RearrangeInd_item,
      { "Item", "nbap.HSDSCH_RearrangeList_Bearer_RearrangeInd_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HSDSCH_RearrangeItem_Bearer_RearrangeInd", HFILL }},
    { &hf_nbap_hsDSCH_MACdFlow_ID,
      { "hsDSCH-MACdFlow-ID", "nbap.hsDSCH_MACdFlow_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HSDSCH_MACdFlow_ID", HFILL }},
    { &hf_nbap_E_DCH_RearrangeList_Bearer_RearrangeInd_item,
      { "Item", "nbap.E_DCH_RearrangeList_Bearer_RearrangeInd_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.E_DCH_RearrangeItem_Bearer_RearrangeInd", HFILL }},
    { &hf_nbap_e_DCH_MACdFlow_ID,
      { "e-DCH-MACdFlow-ID", "nbap.e_DCH_MACdFlow_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.E_DCH_MACdFlow_ID", HFILL }},
    { &hf_nbap_DelayedActivationInformationList_RL_ActivationCmdFDD_item,
      { "Item", "nbap.DelayedActivationInformationList_RL_ActivationCmdFDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_delayed_activation_update,
      { "delayed-activation-update", "nbap.delayed_activation_update",
        FT_UINT32, BASE_DEC, VALS(nbap_DelayedActivationUpdate_vals), 0,
        "nbap.DelayedActivationUpdate", HFILL }},
    { &hf_nbap_DelayedActivationInformationList_RL_ActivationCmdTDD_item,
      { "Item", "nbap.DelayedActivationInformationList_RL_ActivationCmdTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Single_Container", HFILL }},
    { &hf_nbap_cell5,
      { "cell", "nbap.cell",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Cell_CM_Rprt", HFILL }},
    { &hf_nbap_rACH3,
      { "rACH", "nbap.rACH",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RACH_CM_Rprt", HFILL }},
    { &hf_nbap_extension_CommonMeasurementObjectType_CM_Rprt,
      { "extension-CommonMeasurementObjectType-CM-Rprt", "nbap.extension_CommonMeasurementObjectType_CM_Rprt",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Extension_CommonMeasurementObjectType_CM_Rprt", HFILL }},
    { &hf_nbap_commonMeasurementValueInformation,
      { "commonMeasurementValueInformation", "nbap.commonMeasurementValueInformation",
        FT_UINT32, BASE_DEC, VALS(nbap_CommonMeasurementValueInformation_vals), 0,
        "nbap.CommonMeasurementValueInformation", HFILL }},
    { &hf_nbap_cMConfigurationChangeCFN,
      { "cMConfigurationChangeCFN", "nbap.cMConfigurationChangeCFN",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CFN", HFILL }},
    { &hf_nbap_transmission_Gap_Pattern_Sequence_Status,
      { "transmission-Gap-Pattern-Sequence-Status", "nbap.transmission_Gap_Pattern_Sequence_Status",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Transmission_Gap_Pattern_Sequence_Status_List", HFILL }},
    { &hf_nbap_Transmission_Gap_Pattern_Sequence_Status_List_item,
      { "Item", "nbap.Transmission_Gap_Pattern_Sequence_Status_List_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Transmission_Gap_Pattern_Sequence_Status_List_item", HFILL }},
    { &hf_nbap_tGPSID,
      { "tGPSID", "nbap.tGPSID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.TGPSID", HFILL }},
    { &hf_nbap_tGPRC,
      { "tGPRC", "nbap.tGPRC",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.TGPRC", HFILL }},
    { &hf_nbap_tGCFN,
      { "tGCFN", "nbap.tGCFN",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CFN", HFILL }},
    { &hf_nbap_priorityLevel,
      { "priorityLevel", "nbap.priorityLevel",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.PriorityLevel", HFILL }},
    { &hf_nbap_pre_emptionCapability,
      { "pre-emptionCapability", "nbap.pre_emptionCapability",
        FT_UINT32, BASE_DEC, VALS(nbap_Pre_emptionCapability_vals), 0,
        "nbap.Pre_emptionCapability", HFILL }},
    { &hf_nbap_pre_emptionVulnerability,
      { "pre-emptionVulnerability", "nbap.pre_emptionVulnerability",
        FT_UINT32, BASE_DEC, VALS(nbap_Pre_emptionVulnerability_vals), 0,
        "nbap.Pre_emptionVulnerability", HFILL }},
    { &hf_nbap_aOA_LCR,
      { "aOA-LCR", "nbap.aOA_LCR",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.AOA_LCR", HFILL }},
    { &hf_nbap_aOA_LCR_Accuracy_Class,
      { "aOA-LCR-Accuracy-Class", "nbap.aOA_LCR_Accuracy_Class",
        FT_UINT32, BASE_DEC, VALS(nbap_AOA_LCR_Accuracy_Class_vals), 0,
        "nbap.AOA_LCR_Accuracy_Class", HFILL }},
    { &hf_nbap_Best_Cell_Portions_Value_item,
      { "Item", "nbap.Best_Cell_Portions_Value_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Best_Cell_Portions_Item", HFILL }},
    { &hf_nbap_sIRValue,
      { "sIRValue", "nbap.sIRValue",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SIR_Value", HFILL }},
    { &hf_nbap_radioNetwork,
      { "radioNetwork", "nbap.radioNetwork",
        FT_UINT32, BASE_DEC, VALS(nbap_CauseRadioNetwork_vals), 0,
        "nbap.CauseRadioNetwork", HFILL }},
    { &hf_nbap_transport,
      { "transport", "nbap.transport",
        FT_UINT32, BASE_DEC, VALS(nbap_CauseTransport_vals), 0,
        "nbap.CauseTransport", HFILL }},
    { &hf_nbap_protocol,
      { "protocol", "nbap.protocol",
        FT_UINT32, BASE_DEC, VALS(nbap_CauseProtocol_vals), 0,
        "nbap.CauseProtocol", HFILL }},
    { &hf_nbap_misc,
      { "misc", "nbap.misc",
        FT_UINT32, BASE_DEC, VALS(nbap_CauseMisc_vals), 0,
        "nbap.CauseMisc", HFILL }},
    { &hf_nbap_initialPhase_0_1048575,
      { "initialPhase", "nbap.initialPhase",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.INTEGER_0_1048575_", HFILL }},
    { &hf_nbap_steadyStatePhase,
      { "steadyStatePhase", "nbap.steadyStatePhase",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.INTEGER_0_255_", HFILL }},
    { &hf_nbap_initialPhase,
      { "initialPhase", "nbap.initialPhase",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.INTEGER_0_524287_", HFILL }},
    { &hf_nbap_steadyStatePhase1,
      { "steadyStatePhase", "nbap.steadyStatePhase",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.INTEGER_0_127_", HFILL }},
    { &hf_nbap_CommonChannelsCapacityConsumptionLaw_item,
      { "Item", "nbap.CommonChannelsCapacityConsumptionLaw_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CommonChannelsCapacityConsumptionLaw_item", HFILL }},
    { &hf_nbap_dl_Cost,
      { "dl-Cost", "nbap.dl_Cost",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.INTEGER_0_65535", HFILL }},
    { &hf_nbap_ul_Cost,
      { "ul-Cost", "nbap.ul_Cost",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.INTEGER_0_65535", HFILL }},
    { &hf_nbap_tUTRANGPSMeasurementAccuracyClass,
      { "tUTRANGPSMeasurementAccuracyClass", "nbap.tUTRANGPSMeasurementAccuracyClass",
        FT_UINT32, BASE_DEC, VALS(nbap_TUTRANGPSAccuracyClass_vals), 0,
        "nbap.TUTRANGPSAccuracyClass", HFILL }},
    { &hf_nbap_transmitted_carrier_power,
      { "transmitted-carrier-power", "nbap.transmitted_carrier_power",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Transmitted_Carrier_Power_Value", HFILL }},
    { &hf_nbap_acknowledged_prach_preambles,
      { "acknowledged-prach-preambles", "nbap.acknowledged_prach_preambles",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Acknowledged_PRACH_preambles_Value", HFILL }},
    { &hf_nbap_uL_TimeslotISCP,
      { "uL-TimeslotISCP", "nbap.uL_TimeslotISCP",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.UL_TimeslotISCP_Value", HFILL }},
    { &hf_nbap_notUsed_1_acknowledged_PCPCH_access_preambles,
      { "notUsed-1-acknowledged-PCPCH-access-preambles", "nbap.notUsed_1_acknowledged_PCPCH_access_preambles",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NULL", HFILL }},
    { &hf_nbap_notUsed_2_detected_PCPCH_access_preambles,
      { "notUsed-2-detected-PCPCH-access-preambles", "nbap.notUsed_2_detected_PCPCH_access_preambles",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NULL", HFILL }},
    { &hf_nbap_extension_CommonMeasurementValue,
      { "extension-CommonMeasurementValue", "nbap.extension_CommonMeasurementValue",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Extension_CommonMeasurementValue", HFILL }},
    { &hf_nbap_measurementAvailable,
      { "measurementAvailable", "nbap.measurementAvailable",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CommonMeasurementAvailable", HFILL }},
    { &hf_nbap_measurementnotAvailable,
      { "measurementnotAvailable", "nbap.measurementnotAvailable",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CommonMeasurementnotAvailable", HFILL }},
    { &hf_nbap_commonmeasurementValue,
      { "commonmeasurementValue", "nbap.commonmeasurementValue",
        FT_UINT32, BASE_DEC, VALS(nbap_CommonMeasurementValue_vals), 0,
        "nbap.CommonMeasurementValue", HFILL }},
    { &hf_nbap_ie_Extensions,
      { "ie-Extensions", "nbap.ie_Extensions",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.ProtocolExtensionContainer", HFILL }},
    { &hf_nbap_bindingID,
      { "bindingID", "nbap.bindingID",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BindingID", HFILL }},
    { &hf_nbap_transportLayerAddress,
      { "transportLayerAddress", "nbap.transportLayerAddress",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.TransportLayerAddress", HFILL }},
    { &hf_nbap_triggeringMessage,
      { "triggeringMessage", "nbap.triggeringMessage",
        FT_UINT32, BASE_DEC, VALS(nbap_TriggeringMessage_vals), 0,
        "nbap.TriggeringMessage", HFILL }},
    { &hf_nbap_procedureCriticality,
      { "procedureCriticality", "nbap.procedureCriticality",
        FT_UINT32, BASE_DEC, VALS(nbap_Criticality_vals), 0,
        "nbap.Criticality", HFILL }},
    { &hf_nbap_iEsCriticalityDiagnostics,
      { "iEsCriticalityDiagnostics", "nbap.iEsCriticalityDiagnostics",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CriticalityDiagnostics_IE_List", HFILL }},
    { &hf_nbap_CriticalityDiagnostics_IE_List_item,
      { "Item", "nbap.CriticalityDiagnostics_IE_List_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.CriticalityDiagnostics_IE_List_item", HFILL }},
    { &hf_nbap_iECriticality,
      { "iECriticality", "nbap.iECriticality",
        FT_UINT32, BASE_DEC, VALS(nbap_Criticality_vals), 0,
        "nbap.Criticality", HFILL }},
    { &hf_nbap_iE_ID,
      { "iE-ID", "nbap.iE_ID",
        FT_UINT32, BASE_DEC, VALS(nbap_ProtocolIE_ID_vals), 0,
        "nbap.ProtocolIE_ID", HFILL }},
    { &hf_nbap_repetitionNumber,
      { "repetitionNumber", "nbap.repetitionNumber",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.RepetitionNumber0", HFILL }},
    { &hf_nbap_DCH_FDD_Information_item,
      { "Item", "nbap.DCH_FDD_Information_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DCH_FDD_InformationItem", HFILL }},
    { &hf_nbap_payloadCRC_PresenceIndicator,
      { "payloadCRC-PresenceIndicator", "nbap.payloadCRC_PresenceIndicator",
        FT_UINT32, BASE_DEC, VALS(nbap_PayloadCRC_PresenceIndicator_vals), 0,
        "nbap.PayloadCRC_PresenceIndicator", HFILL }},
    { &hf_nbap_ul_FP_Mode,
      { "ul-FP-Mode", "nbap.ul_FP_Mode",
        FT_UINT32, BASE_DEC, VALS(nbap_UL_FP_Mode_vals), 0,
        "nbap.UL_FP_Mode", HFILL }},
    { &hf_nbap_dCH_SpecificInformationList,
      { "dCH-SpecificInformationList", "nbap.dCH_SpecificInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DCH_Specific_FDD_InformationList", HFILL }},
    { &hf_nbap_DCH_Specific_FDD_InformationList_item,
      { "Item", "nbap.DCH_Specific_FDD_InformationList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DCH_Specific_FDD_Item", HFILL }},
    { &hf_nbap_ul_TransportFormatSet,
      { "ul-TransportFormatSet", "nbap.ul_TransportFormatSet",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.TransportFormatSet", HFILL }},
    { &hf_nbap_qE_Selector,
      { "qE-Selector", "nbap.qE_Selector",
        FT_UINT32, BASE_DEC, VALS(nbap_QE_Selector_vals), 0,
        "nbap.QE_Selector", HFILL }},
    { &hf_nbap_DCH_InformationResponse_item,
      { "Item", "nbap.DCH_InformationResponse_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DCH_InformationResponseItem", HFILL }},
    { &hf_nbap_DCH_TDD_Information_item,
      { "Item", "nbap.DCH_TDD_Information_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DCH_TDD_InformationItem", HFILL }},
    { &hf_nbap_dCH_SpecificInformationList1,
      { "dCH-SpecificInformationList", "nbap.dCH_SpecificInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DCH_Specific_TDD_InformationList", HFILL }},
    { &hf_nbap_DCH_Specific_TDD_InformationList_item,
      { "Item", "nbap.DCH_Specific_TDD_InformationList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DCH_Specific_TDD_Item", HFILL }},
    { &hf_nbap_ul_CCTrCH_ID,
      { "ul-CCTrCH-ID", "nbap.ul_CCTrCH_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CCTrCH_ID", HFILL }},
    { &hf_nbap_dl_CCTrCH_ID,
      { "dl-CCTrCH-ID", "nbap.dl_CCTrCH_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CCTrCH_ID", HFILL }},
    { &hf_nbap_FDD_DCHs_to_Modify_item,
      { "Item", "nbap.FDD_DCHs_to_Modify_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.FDD_DCHs_to_ModifyItem", HFILL }},
    { &hf_nbap_dCH_SpecificInformationList2,
      { "dCH-SpecificInformationList", "nbap.dCH_SpecificInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DCH_ModifySpecificInformation_FDD", HFILL }},
    { &hf_nbap_DCH_ModifySpecificInformation_FDD_item,
      { "Item", "nbap.DCH_ModifySpecificInformation_FDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DCH_ModifySpecificItem_FDD", HFILL }},
    { &hf_nbap_TDD_DCHs_to_Modify_item,
      { "Item", "nbap.TDD_DCHs_to_Modify_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DCH_ModifyItem_TDD", HFILL }},
    { &hf_nbap_dCH_SpecificInformationList3,
      { "dCH-SpecificInformationList", "nbap.dCH_SpecificInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DCH_ModifySpecificInformation_TDD", HFILL }},
    { &hf_nbap_DCH_ModifySpecificInformation_TDD_item,
      { "Item", "nbap.DCH_ModifySpecificInformation_TDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DCH_ModifySpecificItem_TDD", HFILL }},
    { &hf_nbap_DedicatedChannelsCapacityConsumptionLaw_item,
      { "Item", "nbap.DedicatedChannelsCapacityConsumptionLaw_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DedicatedChannelsCapacityConsumptionLaw_item", HFILL }},
    { &hf_nbap_dl_Cost_1,
      { "dl-Cost-1", "nbap.dl_Cost_1",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.INTEGER_0_65535", HFILL }},
    { &hf_nbap_dl_Cost_2,
      { "dl-Cost-2", "nbap.dl_Cost_2",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.INTEGER_0_65535", HFILL }},
    { &hf_nbap_ul_Cost_1,
      { "ul-Cost-1", "nbap.ul_Cost_1",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.INTEGER_0_65535", HFILL }},
    { &hf_nbap_ul_Cost_2,
      { "ul-Cost-2", "nbap.ul_Cost_2",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.INTEGER_0_65535", HFILL }},
    { &hf_nbap_sIR_Value,
      { "sIR-Value", "nbap.sIR_Value",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SIR_Value", HFILL }},
    { &hf_nbap_sIR_ErrorValue,
      { "sIR-ErrorValue", "nbap.sIR_ErrorValue",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SIR_Error_Value", HFILL }},
    { &hf_nbap_transmittedCodePowerValue,
      { "transmittedCodePowerValue", "nbap.transmittedCodePowerValue",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Transmitted_Code_Power_Value", HFILL }},
    { &hf_nbap_rSCP,
      { "rSCP", "nbap.rSCP",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.RSCP_Value", HFILL }},
    { &hf_nbap_rxTimingDeviationValue,
      { "rxTimingDeviationValue", "nbap.rxTimingDeviationValue",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Rx_Timing_Deviation_Value", HFILL }},
    { &hf_nbap_roundTripTime,
      { "roundTripTime", "nbap.roundTripTime",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Round_Trip_Time_Value", HFILL }},
    { &hf_nbap_extension_DedicatedMeasurementValue,
      { "extension-DedicatedMeasurementValue", "nbap.extension_DedicatedMeasurementValue",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Extension_DedicatedMeasurementValue", HFILL }},
    { &hf_nbap_measurementAvailable1,
      { "measurementAvailable", "nbap.measurementAvailable",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DedicatedMeasurementAvailable", HFILL }},
    { &hf_nbap_measurementnotAvailable1,
      { "measurementnotAvailable", "nbap.measurementnotAvailable",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DedicatedMeasurementnotAvailable", HFILL }},
    { &hf_nbap_dedicatedmeasurementValue,
      { "dedicatedmeasurementValue", "nbap.dedicatedmeasurementValue",
        FT_UINT32, BASE_DEC, VALS(nbap_DedicatedMeasurementValue_vals), 0,
        "nbap.DedicatedMeasurementValue", HFILL }},
    { &hf_nbap_cfn,
      { "cfn", "nbap.cfn",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CFN", HFILL }},
    { &hf_nbap_separate_indication,
      { "separate-indication", "nbap.separate_indication",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NULL", HFILL }},
    { &hf_nbap_activate,
      { "activate", "nbap.activate",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Activate_Info", HFILL }},
    { &hf_nbap_deactivate,
      { "deactivate", "nbap.deactivate",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Deactivate_Info", HFILL }},
    { &hf_nbap_activation_type,
      { "activation-type", "nbap.activation_type",
        FT_UINT32, BASE_DEC, VALS(nbap_Execution_Type_vals), 0,
        "nbap.Execution_Type", HFILL }},
    { &hf_nbap_initial_dl_tx_power,
      { "initial-dl-tx-power", "nbap.initial_dl_tx_power",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_firstRLS_Indicator,
      { "firstRLS-Indicator", "nbap.firstRLS_Indicator",
        FT_UINT32, BASE_DEC, VALS(nbap_FirstRLS_Indicator_vals), 0,
        "nbap.FirstRLS_Indicator", HFILL }},
    { &hf_nbap_propagation_delay,
      { "propagation-delay", "nbap.propagation_delay",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.PropagationDelay", HFILL }},
    { &hf_nbap_deactivation_type,
      { "deactivation-type", "nbap.deactivation_type",
        FT_UINT32, BASE_DEC, VALS(nbap_Execution_Type_vals), 0,
        "nbap.Execution_Type", HFILL }},
    { &hf_nbap_synchronised,
      { "synchronised", "nbap.synchronised",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CFN", HFILL }},
    { &hf_nbap_unsynchronised,
      { "unsynchronised", "nbap.unsynchronised",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NULL", HFILL }},
    { &hf_nbap_gpstow,
      { "gpstow", "nbap.gpstow",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.GPSTOW", HFILL }},
    { &hf_nbap_status_health,
      { "status-health", "nbap.status_health",
        FT_UINT32, BASE_DEC, VALS(nbap_GPS_Status_Health_vals), 0,
        "nbap.GPS_Status_Health", HFILL }},
    { &hf_nbap_satelliteinfo,
      { "satelliteinfo", "nbap.satelliteinfo",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SAT_Info_DGPSCorrections", HFILL }},
    { &hf_nbap_prcdeviation,
      { "prcdeviation", "nbap.prcdeviation",
        FT_UINT32, BASE_DEC, VALS(nbap_PRCDeviation_vals), 0,
        "nbap.PRCDeviation", HFILL }},
    { &hf_nbap_DL_Timeslot_Information_item,
      { "Item", "nbap.DL_Timeslot_Information_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_Timeslot_InformationItem", HFILL }},
    { &hf_nbap_dL_Code_Information,
      { "dL-Code-Information", "nbap.dL_Code_Information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.TDD_DL_Code_Information", HFILL }},
    { &hf_nbap_DL_TimeslotLCR_Information_item,
      { "Item", "nbap.DL_TimeslotLCR_Information_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_TimeslotLCR_InformationItem", HFILL }},
    { &hf_nbap_dL_Code_LCR_Information,
      { "dL-Code-LCR-Information", "nbap.dL_Code_LCR_Information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.TDD_DL_Code_LCR_Information", HFILL }},
    { &hf_nbap_powerAdjustmentType,
      { "powerAdjustmentType", "nbap.powerAdjustmentType",
        FT_UINT32, BASE_DEC, VALS(nbap_PowerAdjustmentType_vals), 0,
        "nbap.PowerAdjustmentType", HFILL }},
    { &hf_nbap_dLReferencePower,
      { "dLReferencePower", "nbap.dLReferencePower",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_dLReferencePowerList_DL_PC_Rqst,
      { "dLReferencePowerList-DL-PC-Rqst", "nbap.dLReferencePowerList_DL_PC_Rqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_ReferencePowerInformationList", HFILL }},
    { &hf_nbap_maxAdjustmentStep,
      { "maxAdjustmentStep", "nbap.maxAdjustmentStep",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.MaxAdjustmentStep", HFILL }},
    { &hf_nbap_adjustmentPeriod,
      { "adjustmentPeriod", "nbap.adjustmentPeriod",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.AdjustmentPeriod", HFILL }},
    { &hf_nbap_adjustmentRatio,
      { "adjustmentRatio", "nbap.adjustmentRatio",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.ScaledAdjustmentRatio", HFILL }},
    { &hf_nbap_DL_ReferencePowerInformationList_item,
      { "Item", "nbap.DL_ReferencePowerInformationList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_ReferencePowerInformationItem", HFILL }},
    { &hf_nbap_dl_Reference_Power,
      { "dl-Reference-Power", "nbap.dl_Reference_Power",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.DL_Power", HFILL }},
    { &hf_nbap_DL_TimeslotISCPInfo_item,
      { "Item", "nbap.DL_TimeslotISCPInfo_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_TimeslotISCPInfoItem", HFILL }},
    { &hf_nbap_dL_TimeslotISCP,
      { "dL-TimeslotISCP", "nbap.dL_TimeslotISCP",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_TimeslotISCP", HFILL }},
    { &hf_nbap_DL_TimeslotISCPInfoLCR_item,
      { "Item", "nbap.DL_TimeslotISCPInfoLCR_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DL_TimeslotISCPInfoItemLCR", HFILL }},
    { &hf_nbap_DSCH_InformationResponse_item,
      { "Item", "nbap.DSCH_InformationResponse_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DSCH_InformationResponseItem", HFILL }},
    { &hf_nbap_DSCH_TDD_Information_item,
      { "Item", "nbap.DSCH_TDD_Information_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DSCH_TDD_InformationItem", HFILL }},
    { &hf_nbap_replace,
      { "replace", "nbap.replace",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.E_AGCH_FDD_Code_List", HFILL }},
    { &hf_nbap_remove,
      { "remove", "nbap.remove",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NULL", HFILL }},
    { &hf_nbap_E_AGCH_FDD_Code_List_item,
      { "Item", "nbap.E_AGCH_FDD_Code_List_item",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.FDD_DL_ChannelisationCodeNumber", HFILL }},
    { &hf_nbap_e_AGCH_And_E_RGCH_E_HICH_FDD_Scrambling_Code,
      { "e-AGCH-And-E-RGCH-E-HICH-FDD-Scrambling-Code", "nbap.e_AGCH_And_E_RGCH_E_HICH_FDD_Scrambling_Code",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DL_ScramblingCode", HFILL }},
    { &hf_nbap_e_AGCH_Channelisation_Code,
      { "e-AGCH-Channelisation-Code", "nbap.e_AGCH_Channelisation_Code",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.FDD_DL_ChannelisationCodeNumber", HFILL }},
    { &hf_nbap_primary_e_RNTI,
      { "primary-e-RNTI", "nbap.primary_e_RNTI",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.E_RNTI", HFILL }},
    { &hf_nbap_secondary_e_RNTI,
      { "secondary-e-RNTI", "nbap.secondary_e_RNTI",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.E_RNTI", HFILL }},
    { &hf_nbap_e_RGCH_E_HICH_Channelisation_Code,
      { "e-RGCH-E-HICH-Channelisation-Code", "nbap.e_RGCH_E_HICH_Channelisation_Code",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.FDD_DL_ChannelisationCodeNumber", HFILL }},
    { &hf_nbap_e_RGCH_Signature_Sequence,
      { "e-RGCH-Signature-Sequence", "nbap.e_RGCH_Signature_Sequence",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.E_RGCH_Signature_Sequence", HFILL }},
    { &hf_nbap_e_HICH_Signature_Sequence,
      { "e-HICH-Signature-Sequence", "nbap.e_HICH_Signature_Sequence",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.E_HICH_Signature_Sequence", HFILL }},
    { &hf_nbap_serving_Grant_Value,
      { "serving-Grant-Value", "nbap.serving_Grant_Value",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.E_Serving_Grant_Value", HFILL }},
    { &hf_nbap_primary_Secondary_Grant_Selector,
      { "primary-Secondary-Grant-Selector", "nbap.primary_Secondary_Grant_Selector",
        FT_UINT32, BASE_DEC, VALS(nbap_E_Primary_Secondary_Grant_Selector_vals), 0,
        "nbap.E_Primary_Secondary_Grant_Selector", HFILL }},
    { &hf_nbap_e_RGCH_Release_Indicator,
      { "e-RGCH-Release-Indicator", "nbap.e_RGCH_Release_Indicator",
        FT_UINT32, BASE_DEC, VALS(nbap_E_RGCH_Release_Indicator_vals), 0,
        "nbap.E_RGCH_Release_Indicator", HFILL }},
    { &hf_nbap_e_DCH_MACdFlows_Information,
      { "e-DCH-MACdFlows-Information", "nbap.e_DCH_MACdFlows_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.E_DCH_MACdFlows_Information", HFILL }},
    { &hf_nbap_e_DCH_MACdFlow_Specific_InformationResp,
      { "e-DCH-MACdFlow-Specific-InformationResp", "nbap.e_DCH_MACdFlow_Specific_InformationResp",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.E_DCH_MACdFlow_Specific_InformationResp", HFILL }},
    { &hf_nbap_e_DCH_MACdFlow_Specific_Info_to_Modify,
      { "e-DCH-MACdFlow-Specific-Info-to-Modify", "nbap.e_DCH_MACdFlow_Specific_Info_to_Modify",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.E_DCH_MACdFlow_Specific_InfoList_to_Modify", HFILL }},
    { &hf_nbap_e_DCH_Non_Scheduled_Transmission_Grant,
      { "e-DCH-Non-Scheduled-Transmission-Grant", "nbap.e_DCH_Non_Scheduled_Transmission_Grant",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.E_DCH_Non_Scheduled_Transmission_Grant_Items", HFILL }},
    { &hf_nbap_e_DCH_Scheduled_Transmission_Grant,
      { "e-DCH-Scheduled-Transmission-Grant", "nbap.e_DCH_Scheduled_Transmission_Grant",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NULL", HFILL }},
    { &hf_nbap_E_DCH_LogicalChannelInformation_item,
      { "Item", "nbap.E_DCH_LogicalChannelInformation_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.E_DCH_LogicalChannelInformationItem", HFILL }},
    { &hf_nbap_logicalChannelId,
      { "logicalChannelId", "nbap.logicalChannelId",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.LogicalChannelID", HFILL }},
    { &hf_nbap_schedulingPriorityIndicator,
      { "schedulingPriorityIndicator", "nbap.schedulingPriorityIndicator",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SchedulingPriorityIndicator", HFILL }},
    { &hf_nbap_mACesGuaranteedBitRate,
      { "mACesGuaranteedBitRate", "nbap.mACesGuaranteedBitRate",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.MACesGuaranteedBitRate", HFILL }},
    { &hf_nbap_e_DCH_DDI_Value,
      { "e-DCH-DDI-Value", "nbap.e_DCH_DDI_Value",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.E_DCH_DDI_Value", HFILL }},
    { &hf_nbap_mACd_PDU_Size_List,
      { "mACd-PDU-Size-List", "nbap.mACd_PDU_Size_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.E_DCH_MACdPDU_SizeList", HFILL }},
    { &hf_nbap_E_DCH_MACdPDU_SizeList_item,
      { "Item", "nbap.E_DCH_MACdPDU_SizeList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.E_DCH_MACdPDU_SizeListItem", HFILL }},
    { &hf_nbap_mACdPDU_Size,
      { "mACdPDU-Size", "nbap.mACdPDU_Size",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.MACdPDU_Size", HFILL }},
    { &hf_nbap_E_DCH_LogicalChannelToModify_item,
      { "Item", "nbap.E_DCH_LogicalChannelToModify_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.E_DCH_LogicalChannelToModifyItem", HFILL }},
    { &hf_nbap_mACd_PDU_Size_List1,
      { "mACd-PDU-Size-List", "nbap.mACd_PDU_Size_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.E_DCH_MACdPDU_SizeToModifyList", HFILL }},
    { &hf_nbap_E_DCH_MACdPDU_SizeToModifyList_item,
      { "Item", "nbap.E_DCH_MACdPDU_SizeToModifyList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.E_DCH_MACdPDU_SizeListItem", HFILL }},
    { &hf_nbap_E_DCH_LogicalChannelToDelete_item,
      { "Item", "nbap.E_DCH_LogicalChannelToDelete_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.E_DCH_LogicalChannelToDeleteItem", HFILL }},
    { &hf_nbap_e_DCH_MACdFlow_Specific_Info,
      { "e-DCH-MACdFlow-Specific-Info", "nbap.e_DCH_MACdFlow_Specific_Info",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.E_DCH_MACdFlow_Specific_InfoList", HFILL }},
    { &hf_nbap_E_DCH_MACdFlow_Specific_InfoList_item,
      { "Item", "nbap.E_DCH_MACdFlow_Specific_InfoList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.E_DCH_MACdFlow_Specific_InfoItem", HFILL }},
    { &hf_nbap_tnlQos,
      { "tnlQos", "nbap.tnlQos",
        FT_UINT32, BASE_DEC, VALS(nbap_TnlQos_vals), 0,
        "nbap.TnlQos", HFILL }},
    { &hf_nbap_maximum_Number_of_Retransmissions_For_E_DCH,
      { "maximum-Number-of-Retransmissions-For-E-DCH", "nbap.maximum_Number_of_Retransmissions_For_E_DCH",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Maximum_Number_of_Retransmissions_For_E_DCH", HFILL }},
    { &hf_nbap_eDCH_HARQ_PO_FDD,
      { "eDCH-HARQ-PO-FDD", "nbap.eDCH_HARQ_PO_FDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.E_DCH_HARQ_PO_FDD", HFILL }},
    { &hf_nbap_eDCH_MACdFlow_Multiplexing_List,
      { "eDCH-MACdFlow-Multiplexing-List", "nbap.eDCH_MACdFlow_Multiplexing_List",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.E_DCH_MACdFlow_Multiplexing_List", HFILL }},
    { &hf_nbap_eDCH_Grant_Type_Information,
      { "eDCH-Grant-Type-Information", "nbap.eDCH_Grant_Type_Information",
        FT_UINT32, BASE_DEC, VALS(nbap_E_DCH_Grant_Type_Information_vals), 0,
        "nbap.E_DCH_Grant_Type_Information", HFILL }},
    { &hf_nbap_bundlingModeIndicator,
      { "bundlingModeIndicator", "nbap.bundlingModeIndicator",
        FT_UINT32, BASE_DEC, VALS(nbap_BundlingModeIndicator_vals), 0,
        "nbap.BundlingModeIndicator", HFILL }},
    { &hf_nbap_eDCHLogicalChannelInformation,
      { "eDCHLogicalChannelInformation", "nbap.eDCHLogicalChannelInformation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.E_DCH_LogicalChannelInformation", HFILL }},
    { &hf_nbap_E_DCH_MACdFlow_Specific_InformationResp_item,
      { "Item", "nbap.E_DCH_MACdFlow_Specific_InformationResp_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.E_DCH_MACdFlow_Specific_InformationResp_Item", HFILL }},
    { &hf_nbap_E_DCH_MACdFlow_Specific_InfoList_to_Modify_item,
      { "Item", "nbap.E_DCH_MACdFlow_Specific_InfoList_to_Modify_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.E_DCH_MACdFlow_Specific_InfoItem_to_Modify", HFILL }},
    { &hf_nbap_eDCH_LogicalChannelToAdd,
      { "eDCH-LogicalChannelToAdd", "nbap.eDCH_LogicalChannelToAdd",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.E_DCH_LogicalChannelInformation", HFILL }},
    { &hf_nbap_eDCH_LogicalChannelToModify,
      { "eDCH-LogicalChannelToModify", "nbap.eDCH_LogicalChannelToModify",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.E_DCH_LogicalChannelToModify", HFILL }},
    { &hf_nbap_eDCH_LogicalChannelToDelete,
      { "eDCH-LogicalChannelToDelete", "nbap.eDCH_LogicalChannelToDelete",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.E_DCH_LogicalChannelToDelete", HFILL }},
    { &hf_nbap_E_DCH_MACdFlows_to_Delete_item,
      { "Item", "nbap.E_DCH_MACdFlows_to_Delete_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.E_DCH_MACdFlow_to_Delete_Item", HFILL }},
    { &hf_nbap_maxBits_MACe_PDU_non_scheduled,
      { "maxBits-MACe-PDU-non-scheduled", "nbap.maxBits_MACe_PDU_non_scheduled",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Max_Bits_MACe_PDU_non_scheduled", HFILL }},
    { &hf_nbap_hARQ_Process_Allocation_2ms,
      { "hARQ-Process-Allocation-2ms", "nbap.hARQ_Process_Allocation_2ms",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.HARQ_Process_Allocation_2ms_EDCH", HFILL }},
    { &hf_nbap_replace1,
      { "replace", "nbap.replace",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.E_RGCH_E_HICH_FDD_Code_List", HFILL }},
    { &hf_nbap_E_RGCH_E_HICH_FDD_Code_List_item,
      { "Item", "nbap.E_RGCH_E_HICH_FDD_Code_List_item",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.FDD_DL_ChannelisationCodeNumber", HFILL }},
    { &hf_nbap_e_DCH_TFCI_Table_Index,
      { "e-DCH-TFCI-Table-Index", "nbap.e_DCH_TFCI_Table_Index",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.E_DCH_TFCI_Table_Index", HFILL }},
    { &hf_nbap_e_DCH_Min_Set_E_TFCI,
      { "e-DCH-Min-Set-E-TFCI", "nbap.e_DCH_Min_Set_E_TFCI",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.E_TFCI", HFILL }},
    { &hf_nbap_reference_E_TFCI_Information,
      { "reference-E-TFCI-Information", "nbap.reference_E_TFCI_Information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Reference_E_TFCI_Information", HFILL }},
    { &hf_nbap_E_DCHProvidedBitRate_item,
      { "Item", "nbap.E_DCHProvidedBitRate_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.E_DCHProvidedBitRate_Item", HFILL }},
    { &hf_nbap_e_DCHProvidedBitRateValue,
      { "e-DCHProvidedBitRateValue", "nbap.e_DCHProvidedBitRateValue",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.E_DCHProvidedBitRateValue", HFILL }},
    { &hf_nbap_FDD_DL_CodeInformation_item,
      { "Item", "nbap.FDD_DL_CodeInformation_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.FDD_DL_CodeInformationItem", HFILL }},
    { &hf_nbap_transmissionGapPatternSequenceCodeInformation,
      { "transmissionGapPatternSequenceCodeInformation", "nbap.transmissionGapPatternSequenceCodeInformation",
        FT_UINT32, BASE_DEC, VALS(nbap_TransmissionGapPatternSequenceCodeInformation_vals), 0,
        "nbap.TransmissionGapPatternSequenceCodeInformation", HFILL }},
    { &hf_nbap_wna_alm,
      { "wna-alm", "nbap.wna_alm",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_8", HFILL }},
    { &hf_nbap_sat_info_almanac,
      { "sat-info-almanac", "nbap.sat_info_almanac",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SAT_Info_Almanac", HFILL }},
    { &hf_nbap_sVGlobalHealth_alm,
      { "sVGlobalHealth-alm", "nbap.sVGlobalHealth_alm",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_364", HFILL }},
    { &hf_nbap_alpha_zero_ionos,
      { "alpha-zero-ionos", "nbap.alpha_zero_ionos",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_8", HFILL }},
    { &hf_nbap_alpha_one_ionos,
      { "alpha-one-ionos", "nbap.alpha_one_ionos",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_8", HFILL }},
    { &hf_nbap_alpha_two_ionos,
      { "alpha-two-ionos", "nbap.alpha_two_ionos",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_8", HFILL }},
    { &hf_nbap_alpha_three_ionos,
      { "alpha-three-ionos", "nbap.alpha_three_ionos",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_8", HFILL }},
    { &hf_nbap_beta_zero_ionos,
      { "beta-zero-ionos", "nbap.beta_zero_ionos",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_8", HFILL }},
    { &hf_nbap_beta_one_ionos,
      { "beta-one-ionos", "nbap.beta_one_ionos",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_8", HFILL }},
    { &hf_nbap_beta_two_ionos,
      { "beta-two-ionos", "nbap.beta_two_ionos",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_8", HFILL }},
    { &hf_nbap_beta_three_ionos,
      { "beta-three-ionos", "nbap.beta_three_ionos",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_8", HFILL }},
    { &hf_nbap_GPS_Information_item,
      { "Item", "nbap.GPS_Information_item",
        FT_UINT32, BASE_DEC, VALS(nbap_GPS_Information_Item_vals), 0,
        "nbap.GPS_Information_Item", HFILL }},
    { &hf_nbap_bad_satellites,
      { "bad-satellites", "nbap.bad_satellites",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.GPSBadSat_Info_RealTime_Integrity", HFILL }},
    { &hf_nbap_no_bad_satellites,
      { "no-bad-satellites", "nbap.no_bad_satellites",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NULL", HFILL }},
    { &hf_nbap_sat_info,
      { "sat-info", "nbap.sat_info",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SATInfo_RealTime_Integrity", HFILL }},
    { &hf_nbap_GPS_NavigationModel_and_TimeRecovery_item,
      { "Item", "nbap.GPS_NavigationModel_and_TimeRecovery_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.GPS_NavandRecovery_Item", HFILL }},
    { &hf_nbap_tx_tow_nav,
      { "tx-tow-nav", "nbap.tx_tow_nav",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.INTEGER_0_1048575", HFILL }},
    { &hf_nbap_sat_id_nav,
      { "sat-id-nav", "nbap.sat_id_nav",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SAT_ID", HFILL }},
    { &hf_nbap_tlm_message_nav,
      { "tlm-message-nav", "nbap.tlm_message_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_14", HFILL }},
    { &hf_nbap_tlm_revd_c_nav,
      { "tlm-revd-c-nav", "nbap.tlm_revd_c_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_2", HFILL }},
    { &hf_nbap_ho_word_nav,
      { "ho-word-nav", "nbap.ho_word_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_22", HFILL }},
    { &hf_nbap_w_n_nav,
      { "w-n-nav", "nbap.w_n_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_10", HFILL }},
    { &hf_nbap_ca_or_p_on_l2_nav,
      { "ca-or-p-on-l2-nav", "nbap.ca_or_p_on_l2_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_2", HFILL }},
    { &hf_nbap_user_range_accuracy_index_nav,
      { "user-range-accuracy-index-nav", "nbap.user_range_accuracy_index_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_4", HFILL }},
    { &hf_nbap_sv_health_nav,
      { "sv-health-nav", "nbap.sv_health_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_6", HFILL }},
    { &hf_nbap_iodc_nav,
      { "iodc-nav", "nbap.iodc_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_10", HFILL }},
    { &hf_nbap_l2_p_dataflag_nav,
      { "l2-p-dataflag-nav", "nbap.l2_p_dataflag_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_1", HFILL }},
    { &hf_nbap_sf1_reserved_nav,
      { "sf1-reserved-nav", "nbap.sf1_reserved_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_87", HFILL }},
    { &hf_nbap_t_gd_nav,
      { "t-gd-nav", "nbap.t_gd_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_8", HFILL }},
    { &hf_nbap_t_oc_nav,
      { "t-oc-nav", "nbap.t_oc_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_16", HFILL }},
    { &hf_nbap_a_f_2_nav,
      { "a-f-2-nav", "nbap.a_f_2_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_8", HFILL }},
    { &hf_nbap_a_f_1_nav,
      { "a-f-1-nav", "nbap.a_f_1_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_16", HFILL }},
    { &hf_nbap_a_f_zero_nav,
      { "a-f-zero-nav", "nbap.a_f_zero_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_22", HFILL }},
    { &hf_nbap_c_rs_nav,
      { "c-rs-nav", "nbap.c_rs_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_16", HFILL }},
    { &hf_nbap_delta_n_nav,
      { "delta-n-nav", "nbap.delta_n_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_16", HFILL }},
    { &hf_nbap_m_zero_nav,
      { "m-zero-nav", "nbap.m_zero_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_32", HFILL }},
    { &hf_nbap_c_uc_nav,
      { "c-uc-nav", "nbap.c_uc_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_16", HFILL }},
    { &hf_nbap_gps_e_nav,
      { "gps-e-nav", "nbap.gps_e_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_32", HFILL }},
    { &hf_nbap_c_us_nav,
      { "c-us-nav", "nbap.c_us_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_16", HFILL }},
    { &hf_nbap_a_sqrt_nav,
      { "a-sqrt-nav", "nbap.a_sqrt_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_32", HFILL }},
    { &hf_nbap_t_oe_nav,
      { "t-oe-nav", "nbap.t_oe_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_16", HFILL }},
    { &hf_nbap_fit_interval_flag_nav,
      { "fit-interval-flag-nav", "nbap.fit_interval_flag_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_1", HFILL }},
    { &hf_nbap_aodo_nav,
      { "aodo-nav", "nbap.aodo_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_5", HFILL }},
    { &hf_nbap_c_ic_nav,
      { "c-ic-nav", "nbap.c_ic_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_16", HFILL }},
    { &hf_nbap_omega_zero_nav,
      { "omega-zero-nav", "nbap.omega_zero_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_32", HFILL }},
    { &hf_nbap_c_is_nav,
      { "c-is-nav", "nbap.c_is_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_16", HFILL }},
    { &hf_nbap_i_zero_nav,
      { "i-zero-nav", "nbap.i_zero_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_32", HFILL }},
    { &hf_nbap_c_rc_nav,
      { "c-rc-nav", "nbap.c_rc_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_16", HFILL }},
    { &hf_nbap_gps_omega_nav,
      { "gps-omega-nav", "nbap.gps_omega_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_32", HFILL }},
    { &hf_nbap_omegadot_nav,
      { "omegadot-nav", "nbap.omegadot_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_24", HFILL }},
    { &hf_nbap_idot_nav,
      { "idot-nav", "nbap.idot_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_14", HFILL }},
    { &hf_nbap_spare_zero_fill,
      { "spare-zero-fill", "nbap.spare_zero_fill",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_20", HFILL }},
    { &hf_nbap_latitudeSign,
      { "latitudeSign", "nbap.latitudeSign",
        FT_UINT32, BASE_DEC, VALS(nbap_T_latitudeSign_vals), 0,
        "nbap.T_latitudeSign", HFILL }},
    { &hf_nbap_latitude,
      { "latitude", "nbap.latitude",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.INTEGER_0_8388607", HFILL }},
    { &hf_nbap_longitude,
      { "longitude", "nbap.longitude",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.INTEGER_M8388608_8388607", HFILL }},
    { &hf_nbap_directionOfAltitude,
      { "directionOfAltitude", "nbap.directionOfAltitude",
        FT_UINT32, BASE_DEC, VALS(nbap_T_directionOfAltitude_vals), 0,
        "nbap.T_directionOfAltitude", HFILL }},
    { &hf_nbap_altitude,
      { "altitude", "nbap.altitude",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.INTEGER_0_32767", HFILL }},
    { &hf_nbap_a_one_utc,
      { "a-one-utc", "nbap.a_one_utc",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_24", HFILL }},
    { &hf_nbap_a_zero_utc,
      { "a-zero-utc", "nbap.a_zero_utc",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_32", HFILL }},
    { &hf_nbap_t_ot_utc,
      { "t-ot-utc", "nbap.t_ot_utc",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_8", HFILL }},
    { &hf_nbap_delta_t_ls_utc,
      { "delta-t-ls-utc", "nbap.delta_t_ls_utc",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_8", HFILL }},
    { &hf_nbap_w_n_t_utc,
      { "w-n-t-utc", "nbap.w_n_t_utc",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_8", HFILL }},
    { &hf_nbap_w_n_lsf_utc,
      { "w-n-lsf-utc", "nbap.w_n_lsf_utc",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_8", HFILL }},
    { &hf_nbap_dn_utc,
      { "dn-utc", "nbap.dn_utc",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_8", HFILL }},
    { &hf_nbap_delta_t_lsf_utc,
      { "delta-t-lsf-utc", "nbap.delta_t_lsf_utc",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_8", HFILL }},
    { &hf_nbap_implicit,
      { "implicit", "nbap.implicit",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HARQ_MemoryPartitioning_Implicit", HFILL }},
    { &hf_nbap_explicit,
      { "explicit", "nbap.explicit",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HARQ_MemoryPartitioning_Explicit", HFILL }},
    { &hf_nbap_number_of_Processes,
      { "number-of-Processes", "nbap.number_of_Processes",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.INTEGER_1_8_", HFILL }},
    { &hf_nbap_hARQ_MemoryPartitioningList,
      { "hARQ-MemoryPartitioningList", "nbap.hARQ_MemoryPartitioningList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HARQ_MemoryPartitioningList", HFILL }},
    { &hf_nbap_HARQ_MemoryPartitioningList_item,
      { "Item", "nbap.HARQ_MemoryPartitioningList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HARQ_MemoryPartitioningItem", HFILL }},
    { &hf_nbap_process_Memory_Size,
      { "process-Memory-Size", "nbap.process_Memory_Size",
        FT_UINT32, BASE_DEC, VALS(nbap_T_process_Memory_Size_vals), 0,
        "nbap.T_process_Memory_Size", HFILL }},
    { &hf_nbap_HS_DSCHProvidedBitRate_item,
      { "Item", "nbap.HS_DSCHProvidedBitRate_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HS_DSCHProvidedBitRate_Item", HFILL }},
    { &hf_nbap_hS_DSCHProvidedBitRateValue,
      { "hS-DSCHProvidedBitRateValue", "nbap.hS_DSCHProvidedBitRateValue",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HS_DSCHProvidedBitRateValue", HFILL }},
    { &hf_nbap_HS_DSCHProvidedBitRateValueInformation_For_CellPortion_item,
      { "Item", "nbap.HS_DSCHProvidedBitRateValueInformation_For_CellPortion_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HS_DSCHProvidedBitRateValueInformation_For_CellPortion_Item", HFILL }},
    { &hf_nbap_hS_DSCHProvidedBitRateValue1,
      { "hS-DSCHProvidedBitRateValue", "nbap.hS_DSCHProvidedBitRateValue",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HS_DSCHProvidedBitRate", HFILL }},
    { &hf_nbap_HS_DSCHRequiredPower_item,
      { "Item", "nbap.HS_DSCHRequiredPower_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HS_DSCHRequiredPower_Item", HFILL }},
    { &hf_nbap_hS_DSCHRequiredPowerValue,
      { "hS-DSCHRequiredPowerValue", "nbap.hS_DSCHRequiredPowerValue",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HS_DSCHRequiredPowerValue", HFILL }},
    { &hf_nbap_hS_DSCHRequiredPowerPerUEInformation,
      { "hS-DSCHRequiredPowerPerUEInformation", "nbap.hS_DSCHRequiredPowerPerUEInformation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HS_DSCHRequiredPowerPerUEInformation", HFILL }},
    { &hf_nbap_HS_DSCHRequiredPowerPerUEInformation_item,
      { "Item", "nbap.HS_DSCHRequiredPowerPerUEInformation_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HS_DSCHRequiredPowerPerUEInformation_Item", HFILL }},
    { &hf_nbap_hS_DSCHRequiredPowerPerUEWeight,
      { "hS-DSCHRequiredPowerPerUEWeight", "nbap.hS_DSCHRequiredPowerPerUEWeight",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HS_DSCHRequiredPowerPerUEWeight", HFILL }},
    { &hf_nbap_HS_DSCHRequiredPowerValueInformation_For_CellPortion_item,
      { "Item", "nbap.HS_DSCHRequiredPowerValueInformation_For_CellPortion_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HS_DSCHRequiredPowerValueInformation_For_CellPortion_Item", HFILL }},
    { &hf_nbap_hS_DSCHRequiredPowerValue1,
      { "hS-DSCHRequiredPowerValue", "nbap.hS_DSCHRequiredPowerValue",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HS_DSCHRequiredPower", HFILL }},
    { &hf_nbap_hSDSCH_MACdFlows_Information,
      { "hSDSCH-MACdFlows-Information", "nbap.hSDSCH_MACdFlows_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HSDSCH_MACdFlows_Information", HFILL }},
    { &hf_nbap_ueCapability_Info,
      { "ueCapability-Info", "nbap.ueCapability_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UE_Capability_Information", HFILL }},
    { &hf_nbap_mAChs_Reordering_Buffer_Size_for_RLC_UM,
      { "mAChs-Reordering-Buffer-Size-for-RLC-UM", "nbap.mAChs_Reordering_Buffer_Size_for_RLC_UM",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.MAChsReorderingBufferSize_for_RLC_UM", HFILL }},
    { &hf_nbap_cqiFeedback_CycleK,
      { "cqiFeedback-CycleK", "nbap.cqiFeedback_CycleK",
        FT_UINT32, BASE_DEC, VALS(nbap_CQI_Feedback_Cycle_vals), 0,
        "nbap.CQI_Feedback_Cycle", HFILL }},
    { &hf_nbap_cqiRepetitionFactor,
      { "cqiRepetitionFactor", "nbap.cqiRepetitionFactor",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CQI_RepetitionFactor", HFILL }},
    { &hf_nbap_ackNackRepetitionFactor,
      { "ackNackRepetitionFactor", "nbap.ackNackRepetitionFactor",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.AckNack_RepetitionFactor", HFILL }},
    { &hf_nbap_cqiPowerOffset,
      { "cqiPowerOffset", "nbap.cqiPowerOffset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CQI_Power_Offset", HFILL }},
    { &hf_nbap_ackPowerOffset,
      { "ackPowerOffset", "nbap.ackPowerOffset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Ack_Power_Offset", HFILL }},
    { &hf_nbap_nackPowerOffset,
      { "nackPowerOffset", "nbap.nackPowerOffset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Nack_Power_Offset", HFILL }},
    { &hf_nbap_hsscch_PowerOffset,
      { "hsscch-PowerOffset", "nbap.hsscch_PowerOffset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HSSCCH_PowerOffset", HFILL }},
    { &hf_nbap_measurement_Power_Offset,
      { "measurement-Power-Offset", "nbap.measurement_Power_Offset",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.Measurement_Power_Offset", HFILL }},
    { &hf_nbap_tDD_AckNack_Power_Offset,
      { "tDD-AckNack-Power-Offset", "nbap.tDD_AckNack_Power_Offset",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.TDD_AckNack_Power_Offset", HFILL }},
    { &hf_nbap_hsDSCH_MACdFlow_Specific_Info_to_Modify,
      { "hsDSCH-MACdFlow-Specific-Info-to-Modify", "nbap.hsDSCH_MACdFlow_Specific_Info_to_Modify",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HSDSCH_MACdFlow_Specific_InfoList_to_Modify", HFILL }},
    { &hf_nbap_priorityQueueInfotoModify,
      { "priorityQueueInfotoModify", "nbap.priorityQueueInfotoModify",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.PriorityQueue_InfoList_to_Modify", HFILL }},
    { &hf_nbap_hSSCCHCodeChangeGrant,
      { "hSSCCHCodeChangeGrant", "nbap.hSSCCHCodeChangeGrant",
        FT_UINT32, BASE_DEC, VALS(nbap_HSSCCH_Code_Change_Grant_vals), 0,
        "nbap.HSSCCH_Code_Change_Grant", HFILL }},
    { &hf_nbap_tDDAckNackPowerOffset,
      { "tDDAckNackPowerOffset", "nbap.tDDAckNackPowerOffset",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.TDD_AckNack_Power_Offset", HFILL }},
    { &hf_nbap_HSDSCH_MACdFlow_Specific_InfoList_to_Modify_item,
      { "Item", "nbap.HSDSCH_MACdFlow_Specific_InfoList_to_Modify_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HSDSCH_MACdFlow_Specific_InfoItem_to_Modify", HFILL }},
    { &hf_nbap_priorityQueueInfotoModifyUnsynchronised,
      { "priorityQueueInfotoModifyUnsynchronised", "nbap.priorityQueueInfotoModifyUnsynchronised",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.PriorityQueue_InfoList_to_Modify_Unsynchronised", HFILL }},
    { &hf_nbap_hsDSCH_MACdFlow_Specific_InformationResp,
      { "hsDSCH-MACdFlow-Specific-InformationResp", "nbap.hsDSCH_MACdFlow_Specific_InformationResp",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HSDSCH_MACdFlow_Specific_InformationResp", HFILL }},
    { &hf_nbap_hsSCCH_Specific_Information_ResponseFDD,
      { "hsSCCH-Specific-Information-ResponseFDD", "nbap.hsSCCH_Specific_Information_ResponseFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HSSCCH_Specific_InformationRespListFDD", HFILL }},
    { &hf_nbap_hARQ_MemoryPartitioning,
      { "hARQ-MemoryPartitioning", "nbap.hARQ_MemoryPartitioning",
        FT_UINT32, BASE_DEC, VALS(nbap_HARQ_MemoryPartitioning_vals), 0,
        "nbap.HARQ_MemoryPartitioning", HFILL }},
    { &hf_nbap_hsSCCH_Specific_Information_ResponseTDD,
      { "hsSCCH-Specific-Information-ResponseTDD", "nbap.hsSCCH_Specific_Information_ResponseTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HSSCCH_Specific_InformationRespListTDD", HFILL }},
    { &hf_nbap_hsSCCH_Specific_Information_ResponseTDDLCR,
      { "hsSCCH-Specific-Information-ResponseTDDLCR", "nbap.hsSCCH_Specific_Information_ResponseTDDLCR",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HSSCCH_Specific_InformationRespListTDDLCR", HFILL }},
    { &hf_nbap_HSDSCH_MACdFlow_Specific_InformationResp_item,
      { "Item", "nbap.HSDSCH_MACdFlow_Specific_InformationResp_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HSDSCH_MACdFlow_Specific_InformationResp_Item", HFILL }},
    { &hf_nbap_hsDSCHMacdFlow_Id,
      { "hsDSCHMacdFlow-Id", "nbap.hsDSCHMacdFlow_Id",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HSDSCH_MACdFlow_ID", HFILL }},
    { &hf_nbap_hSDSCH_Initial_Capacity_Allocation,
      { "hSDSCH-Initial-Capacity-Allocation", "nbap.hSDSCH_Initial_Capacity_Allocation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HSDSCH_Initial_Capacity_Allocation", HFILL }},
    { &hf_nbap_hSDSCH_MACdFlow_Specific_Info,
      { "hSDSCH-MACdFlow-Specific-Info", "nbap.hSDSCH_MACdFlow_Specific_Info",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HSDSCH_MACdFlow_Specific_InfoList", HFILL }},
    { &hf_nbap_priorityQueue_Info,
      { "priorityQueue-Info", "nbap.priorityQueue_Info",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.PriorityQueue_InfoList", HFILL }},
    { &hf_nbap_HSDSCH_MACdFlow_Specific_InfoList_item,
      { "Item", "nbap.HSDSCH_MACdFlow_Specific_InfoList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HSDSCH_MACdFlow_Specific_InfoItem", HFILL }},
    { &hf_nbap_HSDSCH_MACdFlows_to_Delete_item,
      { "Item", "nbap.HSDSCH_MACdFlows_to_Delete_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HSDSCH_MACdFlows_to_Delete_Item", HFILL }},
    { &hf_nbap_HSDSCH_Initial_Capacity_Allocation_item,
      { "Item", "nbap.HSDSCH_Initial_Capacity_Allocation_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HSDSCH_Initial_Capacity_AllocationItem", HFILL }},
    { &hf_nbap_maximum_MACdPDU_Size,
      { "maximum-MACdPDU-Size", "nbap.maximum_MACdPDU_Size",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.MACdPDU_Size", HFILL }},
    { &hf_nbap_hSDSCH_InitialWindowSize,
      { "hSDSCH-InitialWindowSize", "nbap.hSDSCH_InitialWindowSize",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HSDSCH_InitialWindowSize", HFILL }},
    { &hf_nbap_HSSCCH_Specific_InformationRespListFDD_item,
      { "Item", "nbap.HSSCCH_Specific_InformationRespListFDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HSSCCH_Codes", HFILL }},
    { &hf_nbap_codeNumber,
      { "codeNumber", "nbap.codeNumber",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.INTEGER_0_127", HFILL }},
    { &hf_nbap_HSSCCH_Specific_InformationRespListTDD_item,
      { "Item", "nbap.HSSCCH_Specific_InformationRespListTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HSSCCH_Specific_InformationRespItemTDD", HFILL }},
    { &hf_nbap_tDD_ChannelisationCode,
      { "tDD-ChannelisationCode", "nbap.tDD_ChannelisationCode",
        FT_UINT32, BASE_DEC, VALS(nbap_TDD_ChannelisationCode_vals), 0,
        "nbap.TDD_ChannelisationCode", HFILL }},
    { &hf_nbap_hSSICH_Info,
      { "hSSICH-Info", "nbap.hSSICH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HSSICH_Info", HFILL }},
    { &hf_nbap_HSSCCH_Specific_InformationRespListTDDLCR_item,
      { "Item", "nbap.HSSCCH_Specific_InformationRespListTDDLCR_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HSSCCH_Specific_InformationRespItemTDDLCR", HFILL }},
    { &hf_nbap_hSSICH_InfoLCR,
      { "hSSICH-InfoLCR", "nbap.hSSICH_InfoLCR",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.HSSICH_InfoLCR", HFILL }},
    { &hf_nbap_failed_HS_SICH,
      { "failed-HS-SICH", "nbap.failed_HS_SICH",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HS_SICH_failed", HFILL }},
    { &hf_nbap_missed_HS_SICH,
      { "missed-HS-SICH", "nbap.missed_HS_SICH",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HS_SICH_missed", HFILL }},
    { &hf_nbap_total_HS_SICH,
      { "total-HS-SICH", "nbap.total_HS_SICH",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HS_SICH_total", HFILL }},
    { &hf_nbap_number_of_HS_PDSCH_codes,
      { "number-of-HS-PDSCH-codes", "nbap.number_of_HS_PDSCH_codes",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.INTEGER_0_15", HFILL }},
    { &hf_nbap_hS_PDSCH_Start_code_number,
      { "hS-PDSCH-Start-code-number", "nbap.hS_PDSCH_Start_code_number",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HS_PDSCH_Start_code_number", HFILL }},
    { &hf_nbap_replace2,
      { "replace", "nbap.replace",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HS_SCCH_FDD_Code_List", HFILL }},
    { &hf_nbap_HS_SCCH_FDD_Code_List_item,
      { "Item", "nbap.HS_SCCH_FDD_Code_List_item",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HS_SCCH_FDD_Code_Information_Item", HFILL }},
    { &hf_nbap_hsSCCHCodeChangeIndicator,
      { "hsSCCHCodeChangeIndicator", "nbap.hsSCCHCodeChangeIndicator",
        FT_UINT32, BASE_DEC, VALS(nbap_HSSCCH_CodeChangeIndicator_vals), 0,
        "nbap.HSSCCH_CodeChangeIndicator", HFILL }},
    { &hf_nbap_onDemand,
      { "onDemand", "nbap.onDemand",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NULL", HFILL }},
    { &hf_nbap_periodic,
      { "periodic", "nbap.periodic",
        FT_UINT32, BASE_DEC, VALS(nbap_InformationReportCharacteristicsType_ReportPeriodicity_vals), 0,
        "nbap.InformationReportCharacteristicsType_ReportPeriodicity", HFILL }},
    { &hf_nbap_onModification,
      { "onModification", "nbap.onModification",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.InformationReportCharacteristicsType_OnModification", HFILL }},
    { &hf_nbap_min,
      { "min", "nbap.min",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.ReportPeriodicity_Scaledmin", HFILL }},
    { &hf_nbap_hours,
      { "hours", "nbap.hours",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.ReportPeriodicity_Scaledhour", HFILL }},
    { &hf_nbap_information_thresholds,
      { "information-thresholds", "nbap.information_thresholds",
        FT_UINT32, BASE_DEC, VALS(nbap_InformationThresholds_vals), 0,
        "nbap.InformationThresholds", HFILL }},
    { &hf_nbap_dgps,
      { "dgps", "nbap.dgps",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DGPSThresholds", HFILL }},
    { &hf_nbap_information_Type_Item,
      { "information-Type-Item", "nbap.information_Type_Item",
        FT_UINT32, BASE_DEC, VALS(nbap_Information_Type_Item_vals), 0,
        "nbap.Information_Type_Item", HFILL }},
    { &hf_nbap_gPSInformation,
      { "gPSInformation", "nbap.gPSInformation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.GPS_Information", HFILL }},
    { &hf_nbap_iP_SpacingFDD,
      { "iP-SpacingFDD", "nbap.iP_SpacingFDD",
        FT_UINT32, BASE_DEC, VALS(nbap_T_iP_SpacingFDD_vals), 0,
        "nbap.T_iP_SpacingFDD", HFILL }},
    { &hf_nbap_iP_Length,
      { "iP-Length", "nbap.iP_Length",
        FT_UINT32, BASE_DEC, VALS(nbap_T_iP_Length_vals), 0,
        "nbap.T_iP_Length", HFILL }},
    { &hf_nbap_seed,
      { "seed", "nbap.seed",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.INTEGER_0_63", HFILL }},
    { &hf_nbap_burstModeParams,
      { "burstModeParams", "nbap.burstModeParams",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.BurstModeParams", HFILL }},
    { &hf_nbap_iP_Offset,
      { "iP-Offset", "nbap.iP_Offset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.INTEGER_0_9", HFILL }},
    { &hf_nbap_iP_SpacingTDD,
      { "iP-SpacingTDD", "nbap.iP_SpacingTDD",
        FT_UINT32, BASE_DEC, VALS(nbap_T_iP_SpacingTDD_vals), 0,
        "nbap.T_iP_SpacingTDD", HFILL }},
    { &hf_nbap_iP_Start,
      { "iP-Start", "nbap.iP_Start",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.INTEGER_0_4095", HFILL }},
    { &hf_nbap_iP_Slot,
      { "iP-Slot", "nbap.iP_Slot",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.INTEGER_0_14", HFILL }},
    { &hf_nbap_iP_PCCPCH,
      { "iP-PCCPCH", "nbap.iP_PCCPCH",
        FT_UINT32, BASE_DEC, VALS(nbap_T_iP_PCCPCH_vals), 0,
        "nbap.T_iP_PCCPCH", HFILL }},
    { &hf_nbap_iP_SpacingTDD1,
      { "iP-SpacingTDD", "nbap.iP_SpacingTDD",
        FT_UINT32, BASE_DEC, VALS(nbap_T_iP_SpacingTDD1_vals), 0,
        "nbap.T_iP_SpacingTDD1", HFILL }},
    { &hf_nbap_iP_Sub,
      { "iP-Sub", "nbap.iP_Sub",
        FT_UINT32, BASE_DEC, VALS(nbap_T_iP_Sub_vals), 0,
        "nbap.T_iP_Sub", HFILL }},
    { &hf_nbap_burstStart,
      { "burstStart", "nbap.burstStart",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.INTEGER_0_15", HFILL }},
    { &hf_nbap_burstLength,
      { "burstLength", "nbap.burstLength",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.INTEGER_10_25", HFILL }},
    { &hf_nbap_burstFreq,
      { "burstFreq", "nbap.burstFreq",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.INTEGER_1_16", HFILL }},
    { &hf_nbap_MACdPDU_Size_Indexlist_item,
      { "Item", "nbap.MACdPDU_Size_Indexlist_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.MACdPDU_Size_IndexItem", HFILL }},
    { &hf_nbap_sID,
      { "sID", "nbap.sID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SID", HFILL }},
    { &hf_nbap_macdPDU_Size,
      { "macdPDU-Size", "nbap.macdPDU_Size",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.MACdPDU_Size", HFILL }},
    { &hf_nbap_MACdPDU_Size_Indexlist_to_Modify_item,
      { "Item", "nbap.MACdPDU_Size_Indexlist_to_Modify_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.MACdPDU_Size_IndexItem_to_Modify", HFILL }},
    { &hf_nbap_MessageStructure_item,
      { "Item", "nbap.MessageStructure_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.MessageStructure_item", HFILL }},
    { &hf_nbap_repetitionNumber1,
      { "repetitionNumber", "nbap.repetitionNumber",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.RepetitionNumber1", HFILL }},
    { &hf_nbap_type1,
      { "type1", "nbap.type1",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.T_type1", HFILL }},
    { &hf_nbap_midambleConfigurationBurstType1And3,
      { "midambleConfigurationBurstType1And3", "nbap.midambleConfigurationBurstType1And3",
        FT_UINT32, BASE_DEC, VALS(nbap_MidambleConfigurationBurstType1And3_vals), 0,
        "nbap.MidambleConfigurationBurstType1And3", HFILL }},
    { &hf_nbap_midambleAllocationMode,
      { "midambleAllocationMode", "nbap.midambleAllocationMode",
        FT_UINT32, BASE_DEC, VALS(nbap_T_midambleAllocationMode_vals), 0,
        "nbap.T_midambleAllocationMode", HFILL }},
    { &hf_nbap_defaultMidamble,
      { "defaultMidamble", "nbap.defaultMidamble",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NULL", HFILL }},
    { &hf_nbap_commonMidamble,
      { "commonMidamble", "nbap.commonMidamble",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NULL", HFILL }},
    { &hf_nbap_ueSpecificMidamble,
      { "ueSpecificMidamble", "nbap.ueSpecificMidamble",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.MidambleShiftLong", HFILL }},
    { &hf_nbap_type2,
      { "type2", "nbap.type2",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.T_type2", HFILL }},
    { &hf_nbap_midambleConfigurationBurstType2,
      { "midambleConfigurationBurstType2", "nbap.midambleConfigurationBurstType2",
        FT_UINT32, BASE_DEC, VALS(nbap_MidambleConfigurationBurstType2_vals), 0,
        "nbap.MidambleConfigurationBurstType2", HFILL }},
    { &hf_nbap_midambleAllocationMode1,
      { "midambleAllocationMode", "nbap.midambleAllocationMode",
        FT_UINT32, BASE_DEC, VALS(nbap_T_midambleAllocationMode1_vals), 0,
        "nbap.T_midambleAllocationMode1", HFILL }},
    { &hf_nbap_ueSpecificMidamble1,
      { "ueSpecificMidamble", "nbap.ueSpecificMidamble",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.MidambleShiftShort", HFILL }},
    { &hf_nbap_type3,
      { "type3", "nbap.type3",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.T_type3", HFILL }},
    { &hf_nbap_midambleAllocationMode2,
      { "midambleAllocationMode", "nbap.midambleAllocationMode",
        FT_UINT32, BASE_DEC, VALS(nbap_T_midambleAllocationMode2_vals), 0,
        "nbap.T_midambleAllocationMode2", HFILL }},
    { &hf_nbap_midambleAllocationMode3,
      { "midambleAllocationMode", "nbap.midambleAllocationMode",
        FT_UINT32, BASE_DEC, VALS(nbap_MidambleAllocationMode_vals), 0,
        "nbap.MidambleAllocationMode", HFILL }},
    { &hf_nbap_midambleShift,
      { "midambleShift", "nbap.midambleShift",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.MidambleShiftLong", HFILL }},
    { &hf_nbap_midambleConfigurationLCR,
      { "midambleConfigurationLCR", "nbap.midambleConfigurationLCR",
        FT_UINT32, BASE_DEC, VALS(nbap_MidambleConfigurationLCR_vals), 0,
        "nbap.MidambleConfigurationLCR", HFILL }},
    { &hf_nbap_addPriorityQueue,
      { "addPriorityQueue", "nbap.addPriorityQueue",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PriorityQueue_InfoItem_to_Add", HFILL }},
    { &hf_nbap_modifyPriorityQueue,
      { "modifyPriorityQueue", "nbap.modifyPriorityQueue",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PriorityQueue_InfoItem_to_Modify", HFILL }},
    { &hf_nbap_deletePriorityQueue,
      { "deletePriorityQueue", "nbap.deletePriorityQueue",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.PriorityQueue_Id", HFILL }},
    { &hf_nbap_NeighbouringCellMeasurementInformation_item,
      { "Item", "nbap.NeighbouringCellMeasurementInformation_item",
        FT_UINT32, BASE_DEC, VALS(nbap_NeighbouringCellMeasurementInformation_item_vals), 0,
        "nbap.NeighbouringCellMeasurementInformation_item", HFILL }},
    { &hf_nbap_neighbouringFDDCellMeasurementInformation,
      { "neighbouringFDDCellMeasurementInformation", "nbap.neighbouringFDDCellMeasurementInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NeighbouringFDDCellMeasurementInformation", HFILL }},
    { &hf_nbap_neighbouringTDDCellMeasurementInformation,
      { "neighbouringTDDCellMeasurementInformation", "nbap.neighbouringTDDCellMeasurementInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NeighbouringTDDCellMeasurementInformation", HFILL }},
    { &hf_nbap_extension_neighbouringCellMeasurementInformation,
      { "extension-neighbouringCellMeasurementInformation", "nbap.extension_neighbouringCellMeasurementInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Extension_neighbouringCellMeasurementInformation", HFILL }},
    { &hf_nbap_uC_Id,
      { "uC-Id", "nbap.uC_Id",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UC_Id", HFILL }},
    { &hf_nbap_primaryScramblingCode,
      { "primaryScramblingCode", "nbap.primaryScramblingCode",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.PrimaryScramblingCode", HFILL }},
    { &hf_nbap_cellParameterID,
      { "cellParameterID", "nbap.cellParameterID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CellParameterID", HFILL }},
    { &hf_nbap_NI_Information_item,
      { "Item", "nbap.NI_Information_item",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Notification_Indicator", HFILL }},
    { &hf_nbap_PriorityQueue_InfoList_item,
      { "Item", "nbap.PriorityQueue_InfoList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PriorityQueue_InfoItem", HFILL }},
    { &hf_nbap_priorityQueueId,
      { "priorityQueueId", "nbap.priorityQueueId",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.PriorityQueue_Id", HFILL }},
    { &hf_nbap_associatedHSDSCH_MACdFlow,
      { "associatedHSDSCH-MACdFlow", "nbap.associatedHSDSCH_MACdFlow",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.HSDSCH_MACdFlow_ID", HFILL }},
    { &hf_nbap_t1,
      { "t1", "nbap.t1",
        FT_UINT32, BASE_DEC, VALS(nbap_T1_vals), 0,
        "nbap.T1", HFILL }},
    { &hf_nbap_discardTimer,
      { "discardTimer", "nbap.discardTimer",
        FT_UINT32, BASE_DEC, VALS(nbap_DiscardTimer_vals), 0,
        "nbap.DiscardTimer", HFILL }},
    { &hf_nbap_mAC_hsWindowSize,
      { "mAC-hsWindowSize", "nbap.mAC_hsWindowSize",
        FT_UINT32, BASE_DEC, VALS(nbap_MAC_hsWindowSize_vals), 0,
        "nbap.MAC_hsWindowSize", HFILL }},
    { &hf_nbap_mAChsGuaranteedBitRate,
      { "mAChsGuaranteedBitRate", "nbap.mAChsGuaranteedBitRate",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.MAChsGuaranteedBitRate", HFILL }},
    { &hf_nbap_macdPDU_Size_Index,
      { "macdPDU-Size-Index", "nbap.macdPDU_Size_Index",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.MACdPDU_Size_Indexlist", HFILL }},
    { &hf_nbap_rLC_Mode,
      { "rLC-Mode", "nbap.rLC_Mode",
        FT_UINT32, BASE_DEC, VALS(nbap_RLC_Mode_vals), 0,
        "nbap.RLC_Mode", HFILL }},
    { &hf_nbap_PriorityQueue_InfoList_to_Modify_item,
      { "Item", "nbap.PriorityQueue_InfoList_to_Modify_item",
        FT_UINT32, BASE_DEC, VALS(nbap_ModifyPriorityQueue_vals), 0,
        "nbap.ModifyPriorityQueue", HFILL }},
    { &hf_nbap_macdPDU_Size_Index_to_Modify,
      { "macdPDU-Size-Index-to-Modify", "nbap.macdPDU_Size_Index_to_Modify",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.MACdPDU_Size_Indexlist_to_Modify", HFILL }},
    { &hf_nbap_PriorityQueue_InfoList_to_Modify_Unsynchronised_item,
      { "Item", "nbap.PriorityQueue_InfoList_to_Modify_Unsynchronised_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PriorityQueue_InfoItem_to_Modify_Unsynchronised", HFILL }},
    { &hf_nbap_RL_Specific_DCH_Info_item,
      { "Item", "nbap.RL_Specific_DCH_Info_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_Specific_DCH_Info_Item", HFILL }},
    { &hf_nbap_dCH_id,
      { "dCH-id", "nbap.dCH_id",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DCH_ID", HFILL }},
    { &hf_nbap_transportlayeraddress,
      { "transportlayeraddress", "nbap.transportlayeraddress",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.TransportLayerAddress", HFILL }},
    { &hf_nbap_RL_Specific_E_DCH_Info_item,
      { "Item", "nbap.RL_Specific_E_DCH_Info_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RL_Specific_E_DCH_Info_Item", HFILL }},
    { &hf_nbap_Reference_E_TFCI_Information_item,
      { "Item", "nbap.Reference_E_TFCI_Information_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Reference_E_TFCI_Information_Item", HFILL }},
    { &hf_nbap_reference_E_TFCI,
      { "reference-E-TFCI", "nbap.reference_E_TFCI",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.E_TFCI", HFILL }},
    { &hf_nbap_reference_E_TFCI_PO,
      { "reference-E-TFCI-PO", "nbap.reference_E_TFCI_PO",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Reference_E_TFCI_PO", HFILL }},
    { &hf_nbap_periodic1,
      { "periodic", "nbap.periodic",
        FT_UINT32, BASE_DEC, VALS(nbap_ReportCharacteristicsType_ReportPeriodicity_vals), 0,
        "nbap.ReportCharacteristicsType_ReportPeriodicity", HFILL }},
    { &hf_nbap_event_a,
      { "event-a", "nbap.event_a",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ReportCharacteristicsType_EventA", HFILL }},
    { &hf_nbap_event_b,
      { "event-b", "nbap.event_b",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ReportCharacteristicsType_EventB", HFILL }},
    { &hf_nbap_event_c,
      { "event-c", "nbap.event_c",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ReportCharacteristicsType_EventC", HFILL }},
    { &hf_nbap_event_d,
      { "event-d", "nbap.event_d",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ReportCharacteristicsType_EventD", HFILL }},
    { &hf_nbap_event_e,
      { "event-e", "nbap.event_e",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ReportCharacteristicsType_EventE", HFILL }},
    { &hf_nbap_event_f,
      { "event-f", "nbap.event_f",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ReportCharacteristicsType_EventF", HFILL }},
    { &hf_nbap_extension_ReportCharacteristics,
      { "extension-ReportCharacteristics", "nbap.extension_ReportCharacteristics",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Extension_ReportCharacteristics", HFILL }},
    { &hf_nbap_measurementThreshold,
      { "measurementThreshold", "nbap.measurementThreshold",
        FT_UINT32, BASE_DEC, VALS(nbap_ReportCharacteristicsType_MeasurementThreshold_vals), 0,
        "nbap.ReportCharacteristicsType_MeasurementThreshold", HFILL }},
    { &hf_nbap_measurementHysteresisTime,
      { "measurementHysteresisTime", "nbap.measurementHysteresisTime",
        FT_UINT32, BASE_DEC, VALS(nbap_ReportCharacteristicsType_ScaledMeasurementHysteresisTime_vals), 0,
        "nbap.ReportCharacteristicsType_ScaledMeasurementHysteresisTime", HFILL }},
    { &hf_nbap_measurementIncreaseThreshold,
      { "measurementIncreaseThreshold", "nbap.measurementIncreaseThreshold",
        FT_UINT32, BASE_DEC, VALS(nbap_ReportCharacteristicsType_MeasurementIncreaseDecreaseThreshold_vals), 0,
        "nbap.ReportCharacteristicsType_MeasurementIncreaseDecreaseThreshold", HFILL }},
    { &hf_nbap_measurementChangeTime,
      { "measurementChangeTime", "nbap.measurementChangeTime",
        FT_UINT32, BASE_DEC, VALS(nbap_ReportCharacteristicsType_ScaledMeasurementChangeTime_vals), 0,
        "nbap.ReportCharacteristicsType_ScaledMeasurementChangeTime", HFILL }},
    { &hf_nbap_measurementDecreaseThreshold,
      { "measurementDecreaseThreshold", "nbap.measurementDecreaseThreshold",
        FT_UINT32, BASE_DEC, VALS(nbap_ReportCharacteristicsType_MeasurementIncreaseDecreaseThreshold_vals), 0,
        "nbap.ReportCharacteristicsType_MeasurementIncreaseDecreaseThreshold", HFILL }},
    { &hf_nbap_measurementThreshold1,
      { "measurementThreshold1", "nbap.measurementThreshold1",
        FT_UINT32, BASE_DEC, VALS(nbap_ReportCharacteristicsType_MeasurementThreshold_vals), 0,
        "nbap.ReportCharacteristicsType_MeasurementThreshold", HFILL }},
    { &hf_nbap_measurementThreshold2,
      { "measurementThreshold2", "nbap.measurementThreshold2",
        FT_UINT32, BASE_DEC, VALS(nbap_ReportCharacteristicsType_MeasurementThreshold_vals), 0,
        "nbap.ReportCharacteristicsType_MeasurementThreshold", HFILL }},
    { &hf_nbap_reportPeriodicity,
      { "reportPeriodicity", "nbap.reportPeriodicity",
        FT_UINT32, BASE_DEC, VALS(nbap_ReportCharacteristicsType_ReportPeriodicity_vals), 0,
        "nbap.ReportCharacteristicsType_ReportPeriodicity", HFILL }},
    { &hf_nbap_received_total_wide_band_power1,
      { "received-total-wide-band-power", "nbap.received_total_wide_band_power",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Received_total_wide_band_power_Value_IncrDecrThres", HFILL }},
    { &hf_nbap_uL_TimeslotISCP1,
      { "uL-TimeslotISCP", "nbap.uL_TimeslotISCP",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.UL_TimeslotISCP_Value_IncrDecrThres", HFILL }},
    { &hf_nbap_sir,
      { "sir", "nbap.sir",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SIR_Value_IncrDecrThres", HFILL }},
    { &hf_nbap_sir_error,
      { "sir-error", "nbap.sir_error",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SIR_Error_Value_IncrDecrThres", HFILL }},
    { &hf_nbap_transmitted_code_power,
      { "transmitted-code-power", "nbap.transmitted_code_power",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Transmitted_Code_Power_Value_IncrDecrThres", HFILL }},
    { &hf_nbap_rscp,
      { "rscp", "nbap.rscp",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.RSCP_Value_IncrDecrThres", HFILL }},
    { &hf_nbap_round_trip_time,
      { "round-trip-time", "nbap.round_trip_time",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Round_Trip_Time_IncrDecrThres", HFILL }},
    { &hf_nbap_extension_ReportCharacteristicsType_MeasurementIncreaseDecreaseThreshold,
      { "extension-ReportCharacteristicsType-MeasurementIncreaseDecreaseThreshold", "nbap.extension_ReportCharacteristicsType_MeasurementIncreaseDecreaseThreshold",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Extension_ReportCharacteristicsType_MeasurementIncreaseDecreaseThreshold", HFILL }},
    { &hf_nbap_sir1,
      { "sir", "nbap.sir",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SIR_Value", HFILL }},
    { &hf_nbap_sir_error1,
      { "sir-error", "nbap.sir_error",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SIR_Error_Value", HFILL }},
    { &hf_nbap_transmitted_code_power1,
      { "transmitted-code-power", "nbap.transmitted_code_power",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Transmitted_Code_Power_Value", HFILL }},
    { &hf_nbap_rscp1,
      { "rscp", "nbap.rscp",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.RSCP_Value", HFILL }},
    { &hf_nbap_rx_timing_deviation,
      { "rx-timing-deviation", "nbap.rx_timing_deviation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Rx_Timing_Deviation_Value", HFILL }},
    { &hf_nbap_round_trip_time1,
      { "round-trip-time", "nbap.round_trip_time",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Round_Trip_Time_Value", HFILL }},
    { &hf_nbap_extension_ReportCharacteristicsType_MeasurementThreshold,
      { "extension-ReportCharacteristicsType-MeasurementThreshold", "nbap.extension_ReportCharacteristicsType_MeasurementThreshold",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Extension_ReportCharacteristicsType_MeasurementThreshold", HFILL }},
    { &hf_nbap_msec,
      { "msec", "nbap.msec",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.MeasurementChangeTime_Scaledmsec", HFILL }},
    { &hf_nbap_msec1,
      { "msec", "nbap.msec",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.MeasurementHysteresisTime_Scaledmsec", HFILL }},
    { &hf_nbap_msec2,
      { "msec", "nbap.msec",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.ReportPeriodicity_Scaledmsec", HFILL }},
    { &hf_nbap_Received_total_wide_band_power_For_CellPortion_Value_item,
      { "Item", "nbap.Received_total_wide_band_power_For_CellPortion_Value_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Received_total_wide_band_power_For_CellPortion_Value_Item", HFILL }},
    { &hf_nbap_received_total_wide_band_power_value,
      { "received-total-wide-band-power-value", "nbap.received_total_wide_band_power_value",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Received_total_wide_band_power_Value", HFILL }},
    { &hf_nbap_informationAvailable,
      { "informationAvailable", "nbap.informationAvailable",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.InformationAvailable", HFILL }},
    { &hf_nbap_informationnotAvailable,
      { "informationnotAvailable", "nbap.informationnotAvailable",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.InformationnotAvailable", HFILL }},
    { &hf_nbap_requesteddataValue,
      { "requesteddataValue", "nbap.requesteddataValue",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.RequestedDataValue", HFILL }},
    { &hf_nbap_dgps_corrections,
      { "dgps-corrections", "nbap.dgps_corrections",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.DGPSCorrections", HFILL }},
    { &hf_nbap_gps_navandrecovery,
      { "gps-navandrecovery", "nbap.gps_navandrecovery",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.GPS_NavigationModel_and_TimeRecovery", HFILL }},
    { &hf_nbap_gps_ionos_model,
      { "gps-ionos-model", "nbap.gps_ionos_model",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.GPS_Ionospheric_Model", HFILL }},
    { &hf_nbap_gps_utc_model,
      { "gps-utc-model", "nbap.gps_utc_model",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.GPS_UTC_Model", HFILL }},
    { &hf_nbap_gps_almanac,
      { "gps-almanac", "nbap.gps_almanac",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.GPS_Almanac", HFILL }},
    { &hf_nbap_gps_rt_integrity,
      { "gps-rt-integrity", "nbap.gps_rt_integrity",
        FT_UINT32, BASE_DEC, VALS(nbap_GPS_RealTime_Integrity_vals), 0,
        "nbap.GPS_RealTime_Integrity", HFILL }},
    { &hf_nbap_gpsrxpos,
      { "gpsrxpos", "nbap.gpsrxpos",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.GPS_RX_POS", HFILL }},
    { &hf_nbap_SAT_Info_Almanac_item,
      { "Item", "nbap.SAT_Info_Almanac_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.SAT_Info_Almanac_Item", HFILL }},
    { &hf_nbap_data_id,
      { "data-id", "nbap.data_id",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DATA_ID", HFILL }},
    { &hf_nbap_sat_id,
      { "sat-id", "nbap.sat_id",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SAT_ID", HFILL }},
    { &hf_nbap_gps_e_alm,
      { "gps-e-alm", "nbap.gps_e_alm",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_16", HFILL }},
    { &hf_nbap_gps_toa_alm,
      { "gps-toa-alm", "nbap.gps_toa_alm",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_8", HFILL }},
    { &hf_nbap_gps_delta_I_alm,
      { "gps-delta-I-alm", "nbap.gps_delta_I_alm",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_16", HFILL }},
    { &hf_nbap_omegadot_alm,
      { "omegadot-alm", "nbap.omegadot_alm",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_16", HFILL }},
    { &hf_nbap_svhealth_alm,
      { "svhealth-alm", "nbap.svhealth_alm",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_8", HFILL }},
    { &hf_nbap_gps_a_sqrt_alm,
      { "gps-a-sqrt-alm", "nbap.gps_a_sqrt_alm",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_24", HFILL }},
    { &hf_nbap_omegazero_alm,
      { "omegazero-alm", "nbap.omegazero_alm",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_24", HFILL }},
    { &hf_nbap_m_zero_alm,
      { "m-zero-alm", "nbap.m_zero_alm",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_24", HFILL }},
    { &hf_nbap_gps_omega_alm,
      { "gps-omega-alm", "nbap.gps_omega_alm",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_24", HFILL }},
    { &hf_nbap_gps_af_zero_alm,
      { "gps-af-zero-alm", "nbap.gps_af_zero_alm",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_11", HFILL }},
    { &hf_nbap_gps_af_one_alm,
      { "gps-af-one-alm", "nbap.gps_af_one_alm",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_11", HFILL }},
    { &hf_nbap_SAT_Info_Almanac_ExtList_item,
      { "Item", "nbap.SAT_Info_Almanac_ExtList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.SAT_Info_Almanac_ExtItem", HFILL }},
    { &hf_nbap_SAT_Info_DGPSCorrections_item,
      { "Item", "nbap.SAT_Info_DGPSCorrections_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.SAT_Info_DGPSCorrections_Item", HFILL }},
    { &hf_nbap_iode_dgps,
      { "iode-dgps", "nbap.iode_dgps",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.BIT_STRING_SIZE_8", HFILL }},
    { &hf_nbap_udre,
      { "udre", "nbap.udre",
        FT_UINT32, BASE_DEC, VALS(nbap_UDRE_vals), 0,
        "nbap.UDRE", HFILL }},
    { &hf_nbap_prc,
      { "prc", "nbap.prc",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.PRC", HFILL }},
    { &hf_nbap_range_correction_rate,
      { "range-correction-rate", "nbap.range_correction_rate",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.Range_Correction_Rate", HFILL }},
    { &hf_nbap_SATInfo_RealTime_Integrity_item,
      { "Item", "nbap.SATInfo_RealTime_Integrity_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.SAT_Info_RealTime_Integrity_Item", HFILL }},
    { &hf_nbap_bad_sat_id,
      { "bad-sat-id", "nbap.bad_sat_id",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SAT_ID", HFILL }},
    { &hf_nbap_new_secondary_CPICH,
      { "new-secondary-CPICH", "nbap.new_secondary_CPICH",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CommonPhysicalChannelID", HFILL }},
    { &hf_nbap_secondary_CPICH_shall_not_be_used,
      { "secondary-CPICH-shall-not-be-used", "nbap.secondary_CPICH_shall_not_be_used",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NULL", HFILL }},
    { &hf_nbap_serving_E_DCH_RL_in_this_NodeB,
      { "serving-E-DCH-RL-in-this-NodeB", "nbap.serving_E_DCH_RL_in_this_NodeB",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Serving_E_DCH_RL_in_this_NodeB", HFILL }},
    { &hf_nbap_serving_E_DCH_RL_not_in_this_NodeB,
      { "serving-E-DCH-RL-not-in-this-NodeB", "nbap.serving_E_DCH_RL_not_in_this_NodeB",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NULL", HFILL }},
    { &hf_nbap_sFNSFNChangeLimit,
      { "sFNSFNChangeLimit", "nbap.sFNSFNChangeLimit",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SFNSFNChangeLimit", HFILL }},
    { &hf_nbap_predictedSFNSFNDeviationLimit,
      { "predictedSFNSFNDeviationLimit", "nbap.predictedSFNSFNDeviationLimit",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.PredictedSFNSFNDeviationLimit", HFILL }},
    { &hf_nbap_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation,
      { "successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation", "nbap.successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.T_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation", HFILL }},
    { &hf_nbap_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item,
      { "Item", "nbap.successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.T_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item", HFILL }},
    { &hf_nbap_sFNSFNValue,
      { "sFNSFNValue", "nbap.sFNSFNValue",
        FT_UINT32, BASE_DEC, VALS(nbap_SFNSFNValue_vals), 0,
        "nbap.SFNSFNValue", HFILL }},
    { &hf_nbap_sFNSFNQuality,
      { "sFNSFNQuality", "nbap.sFNSFNQuality",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SFNSFNQuality", HFILL }},
    { &hf_nbap_sFNSFNDriftRate,
      { "sFNSFNDriftRate", "nbap.sFNSFNDriftRate",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.SFNSFNDriftRate", HFILL }},
    { &hf_nbap_sFNSFNDriftRateQuality,
      { "sFNSFNDriftRateQuality", "nbap.sFNSFNDriftRateQuality",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SFNSFNDriftRateQuality", HFILL }},
    { &hf_nbap_sFNSFNTimeStampInformation,
      { "sFNSFNTimeStampInformation", "nbap.sFNSFNTimeStampInformation",
        FT_UINT32, BASE_DEC, VALS(nbap_SFNSFNTimeStampInformation_vals), 0,
        "nbap.SFNSFNTimeStampInformation", HFILL }},
    { &hf_nbap_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation,
      { "unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation", "nbap.unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.T_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation", HFILL }},
    { &hf_nbap_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item,
      { "Item", "nbap.unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.T_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item", HFILL }},
    { &hf_nbap_sFNSFNTimeStamp_FDD,
      { "sFNSFNTimeStamp-FDD", "nbap.sFNSFNTimeStamp_FDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SFN", HFILL }},
    { &hf_nbap_sFNSFNTimeStamp_TDD,
      { "sFNSFNTimeStamp-TDD", "nbap.sFNSFNTimeStamp_TDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.SFNSFNTimeStamp_TDD", HFILL }},
    { &hf_nbap_sFNSFN_FDD,
      { "sFNSFN-FDD", "nbap.sFNSFN_FDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SFNSFN_FDD", HFILL }},
    { &hf_nbap_sFNSFN_TDD,
      { "sFNSFN-TDD", "nbap.sFNSFN_TDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SFNSFN_TDD", HFILL }},
    { &hf_nbap_synchronisationReportCharacteristicsType,
      { "synchronisationReportCharacteristicsType", "nbap.synchronisationReportCharacteristicsType",
        FT_UINT32, BASE_DEC, VALS(nbap_SynchronisationReportCharacteristicsType_vals), 0,
        "nbap.SynchronisationReportCharacteristicsType", HFILL }},
    { &hf_nbap_synchronisationReportCharactThreExc,
      { "synchronisationReportCharactThreExc", "nbap.synchronisationReportCharactThreExc",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SynchronisationReportCharactThreExc", HFILL }},
    { &hf_nbap_SynchronisationReportCharactThreExc_item,
      { "Item", "nbap.SynchronisationReportCharactThreExc_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.SynchronisationReportCharactThreInfoItem", HFILL }},
    { &hf_nbap_syncFrameNumber,
      { "syncFrameNumber", "nbap.syncFrameNumber",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SyncFrameNumber", HFILL }},
    { &hf_nbap_cellSyncBurstInformation,
      { "cellSyncBurstInformation", "nbap.cellSyncBurstInformation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SEQUENCE_SIZE_1_16_OF_SynchronisationReportCharactCellSyncBurstInfoItem", HFILL }},
    { &hf_nbap_cellSyncBurstInformation_item,
      { "Item", "nbap.cellSyncBurstInformation_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.SynchronisationReportCharactCellSyncBurstInfoItem", HFILL }},
    { &hf_nbap_cellSyncBurstTimingThreshold,
      { "cellSyncBurstTimingThreshold", "nbap.cellSyncBurstTimingThreshold",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CellSyncBurstTimingThreshold", HFILL }},
    { &hf_nbap_SyncDLCodeIdThreInfoLCR_item,
      { "Item", "nbap.SyncDLCodeIdThreInfoLCR_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.SyncDLCodeIdThreInfoList", HFILL }},
    { &hf_nbap_syncFrameNoToReceive,
      { "syncFrameNoToReceive", "nbap.syncFrameNoToReceive",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SyncFrameNumber", HFILL }},
    { &hf_nbap_syncDLCodeIdInfoLCR,
      { "syncDLCodeIdInfoLCR", "nbap.syncDLCodeIdInfoLCR",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SyncDLCodeInfoListLCR", HFILL }},
    { &hf_nbap_SyncDLCodeInfoListLCR_item,
      { "Item", "nbap.SyncDLCodeInfoListLCR_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.SyncDLCodeInfoItemLCR", HFILL }},
    { &hf_nbap_syncDLCodeId,
      { "syncDLCodeId", "nbap.syncDLCodeId",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.SYNCDlCodeId", HFILL }},
    { &hf_nbap_syncDLCodeIdArrivTime,
      { "syncDLCodeIdArrivTime", "nbap.syncDLCodeIdArrivTime",
        FT_UINT32, BASE_DEC, VALS(nbap_CellSyncBurstTimingLCR_vals), 0,
        "nbap.CellSyncBurstTimingLCR", HFILL }},
    { &hf_nbap_syncDLCodeIdTimingThre,
      { "syncDLCodeIdTimingThre", "nbap.syncDLCodeIdTimingThre",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.CellSyncBurstTimingThreshold", HFILL }},
    { &hf_nbap_modulation,
      { "modulation", "nbap.modulation",
        FT_UINT32, BASE_DEC, VALS(nbap_Modulation_vals), 0,
        "nbap.Modulation", HFILL }},
    { &hf_nbap_TDD_DL_Code_Information_item,
      { "Item", "nbap.TDD_DL_Code_Information_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.TDD_DL_Code_InformationItem", HFILL }},
    { &hf_nbap_TDD_DL_Code_LCR_Information_item,
      { "Item", "nbap.TDD_DL_Code_LCR_Information_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.TDD_DL_Code_LCR_InformationItem", HFILL }},
    { &hf_nbap_tdd_DL_DPCH_TimeSlotFormat_LCR,
      { "tdd-DL-DPCH-TimeSlotFormat-LCR", "nbap.tdd_DL_DPCH_TimeSlotFormat_LCR",
        FT_UINT32, BASE_DEC, VALS(nbap_TDD_DL_DPCH_TimeSlotFormat_LCR_vals), 0,
        "nbap.TDD_DL_DPCH_TimeSlotFormat_LCR", HFILL }},
    { &hf_nbap_qPSK,
      { "qPSK", "nbap.qPSK",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.QPSK_DL_DPCH_TimeSlotFormatTDD_LCR", HFILL }},
    { &hf_nbap_eightPSK,
      { "eightPSK", "nbap.eightPSK",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.EightPSK_DL_DPCH_TimeSlotFormatTDD_LCR", HFILL }},
    { &hf_nbap_initialOffset,
      { "initialOffset", "nbap.initialOffset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.INTEGER_0_255", HFILL }},
    { &hf_nbap_noinitialOffset,
      { "noinitialOffset", "nbap.noinitialOffset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.INTEGER_0_63", HFILL }},
    { &hf_nbap_signalledGainFactors,
      { "signalledGainFactors", "nbap.signalledGainFactors",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.T_signalledGainFactors", HFILL }},
    { &hf_nbap_gainFactor,
      { "gainFactor", "nbap.gainFactor",
        FT_UINT32, BASE_DEC, VALS(nbap_T_gainFactor_vals), 0,
        "nbap.T_gainFactor", HFILL }},
    { &hf_nbap_fdd,
      { "fdd", "nbap.fdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.T_fdd", HFILL }},
    { &hf_nbap_betaC,
      { "betaC", "nbap.betaC",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.BetaCD", HFILL }},
    { &hf_nbap_betaD,
      { "betaD", "nbap.betaD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.BetaCD", HFILL }},
    { &hf_nbap_tdd,
      { "tdd", "nbap.tdd",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.BetaCD", HFILL }},
    { &hf_nbap_refTFCNumber,
      { "refTFCNumber", "nbap.refTFCNumber",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.RefTFCNumber", HFILL }},
    { &hf_nbap_computedGainFactors,
      { "computedGainFactors", "nbap.computedGainFactors",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.RefTFCNumber", HFILL }},
    { &hf_nbap_TDD_UL_Code_Information_item,
      { "Item", "nbap.TDD_UL_Code_Information_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.TDD_UL_Code_InformationItem", HFILL }},
    { &hf_nbap_TDD_UL_Code_LCR_Information_item,
      { "Item", "nbap.TDD_UL_Code_LCR_Information_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.TDD_UL_Code_LCR_InformationItem", HFILL }},
    { &hf_nbap_tdd_UL_DPCH_TimeSlotFormat_LCR,
      { "tdd-UL-DPCH-TimeSlotFormat-LCR", "nbap.tdd_UL_DPCH_TimeSlotFormat_LCR",
        FT_UINT32, BASE_DEC, VALS(nbap_TDD_UL_DPCH_TimeSlotFormat_LCR_vals), 0,
        "nbap.TDD_UL_DPCH_TimeSlotFormat_LCR", HFILL }},
    { &hf_nbap_qPSK1,
      { "qPSK", "nbap.qPSK",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.QPSK_UL_DPCH_TimeSlotFormatTDD_LCR", HFILL }},
    { &hf_nbap_eightPSK1,
      { "eightPSK", "nbap.eightPSK",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.EightPSK_UL_DPCH_TimeSlotFormatTDD_LCR", HFILL }},
    { &hf_nbap_tFCI_SignallingOption,
      { "tFCI-SignallingOption", "nbap.tFCI_SignallingOption",
        FT_UINT32, BASE_DEC, VALS(nbap_TFCI_SignallingMode_TFCI_SignallingOption_vals), 0,
        "nbap.TFCI_SignallingMode_TFCI_SignallingOption", HFILL }},
    { &hf_nbap_not_Used_splitType,
      { "not-Used-splitType", "nbap.not_Used_splitType",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NULL", HFILL }},
    { &hf_nbap_not_Used_lengthOfTFCI2,
      { "not-Used-lengthOfTFCI2", "nbap.not_Used_lengthOfTFCI2",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NULL", HFILL }},
    { &hf_nbap_initialPhase1,
      { "initialPhase", "nbap.initialPhase",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.INTEGER_0_1048575_", HFILL }},
    { &hf_nbap_dsField,
      { "dsField", "nbap.dsField",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.DsField", HFILL }},
    { &hf_nbap_genericTrafficCategory,
      { "genericTrafficCategory", "nbap.genericTrafficCategory",
        FT_BYTES, BASE_HEX, NULL, 0,
        "nbap.GenericTrafficCategory", HFILL }},
    { &hf_nbap_Transmission_Gap_Pattern_Sequence_Information_item,
      { "Item", "nbap.Transmission_Gap_Pattern_Sequence_Information_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Transmission_Gap_Pattern_Sequence_Information_item", HFILL }},
    { &hf_nbap_tGSN,
      { "tGSN", "nbap.tGSN",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.TGSN", HFILL }},
    { &hf_nbap_tGL1,
      { "tGL1", "nbap.tGL1",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.GapLength", HFILL }},
    { &hf_nbap_tGL2,
      { "tGL2", "nbap.tGL2",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.GapLength", HFILL }},
    { &hf_nbap_tGD,
      { "tGD", "nbap.tGD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.TGD", HFILL }},
    { &hf_nbap_tGPL1,
      { "tGPL1", "nbap.tGPL1",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.GapDuration", HFILL }},
    { &hf_nbap_not_to_be_used_1,
      { "not-to-be-used-1", "nbap.not_to_be_used_1",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.GapDuration", HFILL }},
    { &hf_nbap_uL_DL_mode,
      { "uL-DL-mode", "nbap.uL_DL_mode",
        FT_UINT32, BASE_DEC, VALS(nbap_UL_DL_mode_vals), 0,
        "nbap.UL_DL_mode", HFILL }},
    { &hf_nbap_downlink_Compressed_Mode_Method,
      { "downlink-Compressed-Mode-Method", "nbap.downlink_Compressed_Mode_Method",
        FT_UINT32, BASE_DEC, VALS(nbap_Downlink_Compressed_Mode_Method_vals), 0,
        "nbap.Downlink_Compressed_Mode_Method", HFILL }},
    { &hf_nbap_uplink_Compressed_Mode_Method,
      { "uplink-Compressed-Mode-Method", "nbap.uplink_Compressed_Mode_Method",
        FT_UINT32, BASE_DEC, VALS(nbap_Uplink_Compressed_Mode_Method_vals), 0,
        "nbap.Uplink_Compressed_Mode_Method", HFILL }},
    { &hf_nbap_dL_FrameType,
      { "dL-FrameType", "nbap.dL_FrameType",
        FT_UINT32, BASE_DEC, VALS(nbap_DL_FrameType_vals), 0,
        "nbap.DL_FrameType", HFILL }},
    { &hf_nbap_delta_SIR1,
      { "delta-SIR1", "nbap.delta_SIR1",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DeltaSIR", HFILL }},
    { &hf_nbap_delta_SIR_after1,
      { "delta-SIR-after1", "nbap.delta_SIR_after1",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DeltaSIR", HFILL }},
    { &hf_nbap_delta_SIR2,
      { "delta-SIR2", "nbap.delta_SIR2",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DeltaSIR", HFILL }},
    { &hf_nbap_delta_SIR_after2,
      { "delta-SIR-after2", "nbap.delta_SIR_after2",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.DeltaSIR", HFILL }},
    { &hf_nbap_TransmittedCarrierPowerOfAllCodesNotUsedForHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHOrE_HICHTransmissionCellPortionValue_item,
      { "Item", "nbap.TransmittedCarrierPowerOfAllCodesNotUsedForHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHOrE_HICHTransmissionCellPortionValue_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.TransmittedCarrierPowerOfAllCodesNotUsedForHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHOrE_HICHTransmissionCellPortionValue_Item", HFILL }},
    { &hf_nbap_transmittedCarrierPowerOfAllCodesNotUsedForHSTransmissionValue,
      { "transmittedCarrierPowerOfAllCodesNotUsedForHSTransmissionValue", "nbap.transmittedCarrierPowerOfAllCodesNotUsedForHSTransmissionValue",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.TransmittedCarrierPowerOfAllCodesNotUsedForHSTransmissionValue", HFILL }},
    { &hf_nbap_Transmitted_Carrier_Power_For_CellPortion_Value_item,
      { "Item", "nbap.Transmitted_Carrier_Power_For_CellPortion_Value_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Transmitted_Carrier_Power_For_CellPortion_Value_Item", HFILL }},
    { &hf_nbap_transmitted_Carrier_Power_Value,
      { "transmitted-Carrier-Power-Value", "nbap.transmitted_Carrier_Power_Value",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.Transmitted_Carrier_Power_Value", HFILL }},
    { &hf_nbap_tFCSvalues,
      { "tFCSvalues", "nbap.tFCSvalues",
        FT_UINT32, BASE_DEC, VALS(nbap_T_tFCSvalues_vals), 0,
        "nbap.T_tFCSvalues", HFILL }},
    { &hf_nbap_no_Split_in_TFCI,
      { "no-Split-in-TFCI", "nbap.no_Split_in_TFCI",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.TFCS_TFCSList", HFILL }},
    { &hf_nbap_not_Used_split_in_TFCI,
      { "not-Used-split-in-TFCI", "nbap.not_Used_split_in_TFCI",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NULL", HFILL }},
    { &hf_nbap_TFCS_TFCSList_item,
      { "Item", "nbap.TFCS_TFCSList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.TFCS_TFCSList_item", HFILL }},
    { &hf_nbap_cTFC,
      { "cTFC", "nbap.cTFC",
        FT_UINT32, BASE_DEC, VALS(nbap_TFCS_CTFC_vals), 0,
        "nbap.TFCS_CTFC", HFILL }},
    { &hf_nbap_tFC_Beta,
      { "tFC-Beta", "nbap.tFC_Beta",
        FT_UINT32, BASE_DEC, VALS(nbap_TransportFormatCombination_Beta_vals), 0,
        "nbap.TransportFormatCombination_Beta", HFILL }},
    { &hf_nbap_ctfc2bit,
      { "ctfc2bit", "nbap.ctfc2bit",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.INTEGER_0_3", HFILL }},
    { &hf_nbap_ctfc4bit,
      { "ctfc4bit", "nbap.ctfc4bit",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.INTEGER_0_15", HFILL }},
    { &hf_nbap_ctfc6bit,
      { "ctfc6bit", "nbap.ctfc6bit",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.INTEGER_0_63", HFILL }},
    { &hf_nbap_ctfc8bit,
      { "ctfc8bit", "nbap.ctfc8bit",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.INTEGER_0_255", HFILL }},
    { &hf_nbap_ctfc12bit,
      { "ctfc12bit", "nbap.ctfc12bit",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.INTEGER_0_4095", HFILL }},
    { &hf_nbap_ctfc16bit,
      { "ctfc16bit", "nbap.ctfc16bit",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.INTEGER_0_65535", HFILL }},
    { &hf_nbap_ctfcmaxbit,
      { "ctfcmaxbit", "nbap.ctfcmaxbit",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.INTEGER_0_16777215", HFILL }},
    { &hf_nbap_dynamicParts,
      { "dynamicParts", "nbap.dynamicParts",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.TransportFormatSet_DynamicPartList", HFILL }},
    { &hf_nbap_semi_staticPart,
      { "semi-staticPart", "nbap.semi_staticPart",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.TransportFormatSet_Semi_staticPart", HFILL }},
    { &hf_nbap_TransportFormatSet_DynamicPartList_item,
      { "Item", "nbap.TransportFormatSet_DynamicPartList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.TransportFormatSet_DynamicPartList_item", HFILL }},
    { &hf_nbap_nrOfTransportBlocks,
      { "nrOfTransportBlocks", "nbap.nrOfTransportBlocks",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.TransportFormatSet_NrOfTransportBlocks", HFILL }},
    { &hf_nbap_transportBlockSize,
      { "transportBlockSize", "nbap.transportBlockSize",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.TransportFormatSet_TransportBlockSize", HFILL }},
    { &hf_nbap_mode,
      { "mode", "nbap.mode",
        FT_UINT32, BASE_DEC, VALS(nbap_TransportFormatSet_ModeDP_vals), 0,
        "nbap.TransportFormatSet_ModeDP", HFILL }},
    { &hf_nbap_transmissionTimeIntervalInformation,
      { "transmissionTimeIntervalInformation", "nbap.transmissionTimeIntervalInformation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.TransmissionTimeIntervalInformation", HFILL }},
    { &hf_nbap_TransmissionTimeIntervalInformation_item,
      { "Item", "nbap.TransmissionTimeIntervalInformation_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.TransmissionTimeIntervalInformation_item", HFILL }},
    { &hf_nbap_transmissionTimeInterval,
      { "transmissionTimeInterval", "nbap.transmissionTimeInterval",
        FT_UINT32, BASE_DEC, VALS(nbap_TransportFormatSet_TransmissionTimeIntervalDynamic_vals), 0,
        "nbap.TransportFormatSet_TransmissionTimeIntervalDynamic", HFILL }},
    { &hf_nbap_transmissionTimeInterval1,
      { "transmissionTimeInterval", "nbap.transmissionTimeInterval",
        FT_UINT32, BASE_DEC, VALS(nbap_TransportFormatSet_TransmissionTimeIntervalSemiStatic_vals), 0,
        "nbap.TransportFormatSet_TransmissionTimeIntervalSemiStatic", HFILL }},
    { &hf_nbap_channelCoding,
      { "channelCoding", "nbap.channelCoding",
        FT_UINT32, BASE_DEC, VALS(nbap_TransportFormatSet_ChannelCodingType_vals), 0,
        "nbap.TransportFormatSet_ChannelCodingType", HFILL }},
    { &hf_nbap_codingRate,
      { "codingRate", "nbap.codingRate",
        FT_UINT32, BASE_DEC, VALS(nbap_TransportFormatSet_CodingRate_vals), 0,
        "nbap.TransportFormatSet_CodingRate", HFILL }},
    { &hf_nbap_rateMatchingAttribute,
      { "rateMatchingAttribute", "nbap.rateMatchingAttribute",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.TransportFormatSet_RateMatchingAttribute", HFILL }},
    { &hf_nbap_cRC_Size,
      { "cRC-Size", "nbap.cRC_Size",
        FT_UINT32, BASE_DEC, VALS(nbap_TransportFormatSet_CRC_Size_vals), 0,
        "nbap.TransportFormatSet_CRC_Size", HFILL }},
    { &hf_nbap_mode1,
      { "mode", "nbap.mode",
        FT_UINT32, BASE_DEC, VALS(nbap_TransportFormatSet_ModeSSP_vals), 0,
        "nbap.TransportFormatSet_ModeSSP", HFILL }},
    { &hf_nbap_tdd1,
      { "tdd", "nbap.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.TDD_TransportFormatSet_ModeDP", HFILL }},
    { &hf_nbap_notApplicable,
      { "notApplicable", "nbap.notApplicable",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.NULL", HFILL }},
    { &hf_nbap_tdd2,
      { "tdd", "nbap.tdd",
        FT_UINT32, BASE_DEC, VALS(nbap_TransportFormatSet_SecondInterleavingMode_vals), 0,
        "nbap.TransportFormatSet_SecondInterleavingMode", HFILL }},
    { &hf_nbap_ms_part,
      { "ms-part", "nbap.ms_part",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.INTEGER_0_16383", HFILL }},
    { &hf_nbap_ls_part,
      { "ls-part", "nbap.ls_part",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.INTEGER_0_4294967295", HFILL }},
    { &hf_nbap_tUTRANGPSChangeLimit,
      { "tUTRANGPSChangeLimit", "nbap.tUTRANGPSChangeLimit",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.TUTRANGPSChangeLimit", HFILL }},
    { &hf_nbap_predictedTUTRANGPSDeviationLimit,
      { "predictedTUTRANGPSDeviationLimit", "nbap.predictedTUTRANGPSDeviationLimit",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.PredictedTUTRANGPSDeviationLimit", HFILL }},
    { &hf_nbap_tUTRANGPS,
      { "tUTRANGPS", "nbap.tUTRANGPS",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.TUTRANGPS", HFILL }},
    { &hf_nbap_tUTRANGPSQuality,
      { "tUTRANGPSQuality", "nbap.tUTRANGPSQuality",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.TUTRANGPSQuality", HFILL }},
    { &hf_nbap_tUTRANGPSDriftRate,
      { "tUTRANGPSDriftRate", "nbap.tUTRANGPSDriftRate",
        FT_INT32, BASE_DEC, NULL, 0,
        "nbap.TUTRANGPSDriftRate", HFILL }},
    { &hf_nbap_tUTRANGPSDriftRateQuality,
      { "tUTRANGPSDriftRateQuality", "nbap.tUTRANGPSDriftRateQuality",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.TUTRANGPSDriftRateQuality", HFILL }},
    { &hf_nbap_rNC_ID,
      { "rNC-ID", "nbap.rNC_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.RNC_ID", HFILL }},
    { &hf_nbap_hSDSCH_Physical_Layer_Category,
      { "hSDSCH-Physical-Layer-Category", "nbap.hSDSCH_Physical_Layer_Category",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.INTEGER_1_64_", HFILL }},
    { &hf_nbap_UL_Timeslot_Information_item,
      { "Item", "nbap.UL_Timeslot_Information_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_Timeslot_InformationItem", HFILL }},
    { &hf_nbap_uL_Code_InformationList,
      { "uL-Code-InformationList", "nbap.uL_Code_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.TDD_UL_Code_Information", HFILL }},
    { &hf_nbap_UL_TimeslotLCR_Information_item,
      { "Item", "nbap.UL_TimeslotLCR_Information_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_TimeslotLCR_InformationItem", HFILL }},
    { &hf_nbap_uL_Code_InformationList1,
      { "uL-Code-InformationList", "nbap.uL_Code_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.TDD_UL_Code_LCR_Information", HFILL }},
    { &hf_nbap_uL_ScramblingCodeNumber,
      { "uL-ScramblingCodeNumber", "nbap.uL_ScramblingCodeNumber",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.UL_ScramblingCodeNumber", HFILL }},
    { &hf_nbap_uL_ScramblingCodeLength,
      { "uL-ScramblingCodeLength", "nbap.uL_ScramblingCodeLength",
        FT_UINT32, BASE_DEC, VALS(nbap_UL_ScramblingCodeLength_vals), 0,
        "nbap.UL_ScramblingCodeLength", HFILL }},
    { &hf_nbap_uL_Synchronisation_StepSize,
      { "uL-Synchronisation-StepSize", "nbap.uL_Synchronisation_StepSize",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.UL_Synchronisation_StepSize", HFILL }},
    { &hf_nbap_uL_Synchronisation_Frequency,
      { "uL-Synchronisation-Frequency", "nbap.uL_Synchronisation_Frequency",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.UL_Synchronisation_Frequency", HFILL }},
    { &hf_nbap_UL_TimeSlot_ISCP_Info_item,
      { "Item", "nbap.UL_TimeSlot_ISCP_Info_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_TimeSlot_ISCP_InfoItem", HFILL }},
    { &hf_nbap_iSCP,
      { "iSCP", "nbap.iSCP",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.UL_TimeslotISCP_Value", HFILL }},
    { &hf_nbap_UL_TimeSlot_ISCP_LCR_Info_item,
      { "Item", "nbap.UL_TimeSlot_ISCP_LCR_Info_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.UL_TimeSlot_ISCP_LCR_InfoItem", HFILL }},
    { &hf_nbap_USCH_Information_item,
      { "Item", "nbap.USCH_Information_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.USCH_InformationItem", HFILL }},
    { &hf_nbap_USCH_InformationResponse_item,
      { "Item", "nbap.USCH_InformationResponse_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.USCH_InformationResponseItem", HFILL }},
    { &hf_nbap_local,
      { "local", "nbap.local",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.INTEGER_0_65535", HFILL }},
    { &hf_nbap_global,
      { "global", "nbap.global",
        FT_OID, BASE_NONE, NULL, 0,
        "nbap.OBJECT_IDENTIFIER", HFILL }},
    { &hf_nbap_procedureCode,
      { "procedureCode", "nbap.procedureCode",
        FT_UINT32, BASE_DEC, VALS(nbap_ProcedureCode_vals), 0,
        "nbap.ProcedureCode", HFILL }},
    { &hf_nbap_ddMode,
      { "ddMode", "nbap.ddMode",
        FT_UINT32, BASE_DEC, VALS(nbap_T_ddMode_vals), 0,
        "nbap.T_ddMode", HFILL }},
    { &hf_nbap_shortTransActionId,
      { "shortTransActionId", "nbap.shortTransActionId",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.INTEGER_0_127", HFILL }},
    { &hf_nbap_longTransActionId,
      { "longTransActionId", "nbap.longTransActionId",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.INTEGER_0_32767", HFILL }},
    { &hf_nbap_ProtocolIE_Container_item,
      { "Item", "nbap.ProtocolIE_Container_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_Field", HFILL }},
    { &hf_nbap_id,
      { "id", "nbap.id",
        FT_UINT32, BASE_DEC, VALS(nbap_ProtocolIE_ID_vals), 0,
        "nbap.ProtocolIE_ID", HFILL }},
    { &hf_nbap_value,
      { "value", "nbap.value",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIEValue", HFILL }},
    { &hf_nbap_ProtocolIE_ContainerPair_item,
      { "Item", "nbap.ProtocolIE_ContainerPair_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolIE_FieldPair", HFILL }},
    { &hf_nbap_firstCriticality,
      { "firstCriticality", "nbap.firstCriticality",
        FT_UINT32, BASE_DEC, VALS(nbap_Criticality_vals), 0,
        "nbap.Criticality", HFILL }},
    { &hf_nbap_firstValue,
      { "firstValue", "nbap.firstValue",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.FirstValue", HFILL }},
    { &hf_nbap_secondCriticality,
      { "secondCriticality", "nbap.secondCriticality",
        FT_UINT32, BASE_DEC, VALS(nbap_Criticality_vals), 0,
        "nbap.Criticality", HFILL }},
    { &hf_nbap_secondValue,
      { "secondValue", "nbap.secondValue",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.SecondValue", HFILL }},
    { &hf_nbap_ProtocolIE_ContainerList_item,
      { "Item", "nbap.ProtocolIE_ContainerList_item",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.ProtocolIE_Container", HFILL }},
    { &hf_nbap_ProtocolIE_ContainerPairList_item,
      { "Item", "nbap.ProtocolIE_ContainerPairList_item",
        FT_UINT32, BASE_DEC, NULL, 0,
        "nbap.ProtocolIE_ContainerPair", HFILL }},
    { &hf_nbap_ProtocolExtensionContainer_item,
      { "Item", "nbap.ProtocolExtensionContainer_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.ProtocolExtensionField", HFILL }},
    { &hf_nbap_extensionValue,
      { "extensionValue", "nbap.extensionValue",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.Extension", HFILL }},
    { &hf_nbap_PrivateIE_Container_item,
      { "Item", "nbap.PrivateIE_Container_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PrivateIE_Field", HFILL }},
    { &hf_nbap_privateIEid,
      { "privateIEid", "nbap.privateIEid",
        FT_UINT32, BASE_DEC, VALS(nbap_PrivateIE_ID_vals), 0,
        "nbap.PrivateIE_ID", HFILL }},
    { &hf_nbap_privateIEvalue,
      { "privateIEvalue", "nbap.privateIEvalue",
        FT_NONE, BASE_NONE, NULL, 0,
        "nbap.PrivateIEvalue", HFILL }},
    { &hf_nbap_PreambleSignatures_signature15,
      { "signature15", "nbap.signature15",
        FT_BOOLEAN, 8, NULL, 0x80,
        "", HFILL }},
    { &hf_nbap_PreambleSignatures_signature14,
      { "signature14", "nbap.signature14",
        FT_BOOLEAN, 8, NULL, 0x40,
        "", HFILL }},
    { &hf_nbap_PreambleSignatures_signature13,
      { "signature13", "nbap.signature13",
        FT_BOOLEAN, 8, NULL, 0x20,
        "", HFILL }},
    { &hf_nbap_PreambleSignatures_signature12,
      { "signature12", "nbap.signature12",
        FT_BOOLEAN, 8, NULL, 0x10,
        "", HFILL }},
    { &hf_nbap_PreambleSignatures_signature11,
      { "signature11", "nbap.signature11",
        FT_BOOLEAN, 8, NULL, 0x08,
        "", HFILL }},
    { &hf_nbap_PreambleSignatures_signature10,
      { "signature10", "nbap.signature10",
        FT_BOOLEAN, 8, NULL, 0x04,
        "", HFILL }},
    { &hf_nbap_PreambleSignatures_signature9,
      { "signature9", "nbap.signature9",
        FT_BOOLEAN, 8, NULL, 0x02,
        "", HFILL }},
    { &hf_nbap_PreambleSignatures_signature8,
      { "signature8", "nbap.signature8",
        FT_BOOLEAN, 8, NULL, 0x01,
        "", HFILL }},
    { &hf_nbap_PreambleSignatures_signature7,
      { "signature7", "nbap.signature7",
        FT_BOOLEAN, 8, NULL, 0x80,
        "", HFILL }},
    { &hf_nbap_PreambleSignatures_signature6,
      { "signature6", "nbap.signature6",
        FT_BOOLEAN, 8, NULL, 0x40,
        "", HFILL }},
    { &hf_nbap_PreambleSignatures_signature5,
      { "signature5", "nbap.signature5",
        FT_BOOLEAN, 8, NULL, 0x20,
        "", HFILL }},
    { &hf_nbap_PreambleSignatures_signature4,
      { "signature4", "nbap.signature4",
        FT_BOOLEAN, 8, NULL, 0x10,
        "", HFILL }},
    { &hf_nbap_PreambleSignatures_signature3,
      { "signature3", "nbap.signature3",
        FT_BOOLEAN, 8, NULL, 0x08,
        "", HFILL }},
    { &hf_nbap_PreambleSignatures_signature2,
      { "signature2", "nbap.signature2",
        FT_BOOLEAN, 8, NULL, 0x04,
        "", HFILL }},
    { &hf_nbap_PreambleSignatures_signature1,
      { "signature1", "nbap.signature1",
        FT_BOOLEAN, 8, NULL, 0x02,
        "", HFILL }},
    { &hf_nbap_PreambleSignatures_signature0,
      { "signature0", "nbap.signature0",
        FT_BOOLEAN, 8, NULL, 0x01,
        "", HFILL }},
    { &hf_nbap_RACH_SubChannelNumbers_subCh11,
      { "subCh11", "nbap.subCh11",
        FT_BOOLEAN, 8, NULL, 0x80,
        "", HFILL }},
    { &hf_nbap_RACH_SubChannelNumbers_subCh10,
      { "subCh10", "nbap.subCh10",
        FT_BOOLEAN, 8, NULL, 0x40,
        "", HFILL }},
    { &hf_nbap_RACH_SubChannelNumbers_subCh9,
      { "subCh9", "nbap.subCh9",
        FT_BOOLEAN, 8, NULL, 0x20,
        "", HFILL }},
    { &hf_nbap_RACH_SubChannelNumbers_subCh8,
      { "subCh8", "nbap.subCh8",
        FT_BOOLEAN, 8, NULL, 0x10,
        "", HFILL }},
    { &hf_nbap_RACH_SubChannelNumbers_subCh7,
      { "subCh7", "nbap.subCh7",
        FT_BOOLEAN, 8, NULL, 0x08,
        "", HFILL }},
    { &hf_nbap_RACH_SubChannelNumbers_subCh6,
      { "subCh6", "nbap.subCh6",
        FT_BOOLEAN, 8, NULL, 0x04,
        "", HFILL }},
    { &hf_nbap_RACH_SubChannelNumbers_subCh5,
      { "subCh5", "nbap.subCh5",
        FT_BOOLEAN, 8, NULL, 0x02,
        "", HFILL }},
    { &hf_nbap_RACH_SubChannelNumbers_subCh4,
      { "subCh4", "nbap.subCh4",
        FT_BOOLEAN, 8, NULL, 0x01,
        "", HFILL }},
    { &hf_nbap_RACH_SubChannelNumbers_subCh3,
      { "subCh3", "nbap.subCh3",
        FT_BOOLEAN, 8, NULL, 0x80,
        "", HFILL }},
    { &hf_nbap_RACH_SubChannelNumbers_subCh2,
      { "subCh2", "nbap.subCh2",
        FT_BOOLEAN, 8, NULL, 0x40,
        "", HFILL }},
    { &hf_nbap_RACH_SubChannelNumbers_subCh1,
      { "subCh1", "nbap.subCh1",
        FT_BOOLEAN, 8, NULL, 0x20,
        "", HFILL }},
    { &hf_nbap_RACH_SubChannelNumbers_subCh0,
      { "subCh0", "nbap.subCh0",
        FT_BOOLEAN, 8, NULL, 0x10,
        "", HFILL }},

/*--- End of included file: packet-nbap-hfarr.c ---*/
#line 2924 "packet-nbap-template.c"
  };

  /* List of subtrees */
  static gint *ett[] = {
		  &ett_nbap,
		  &ett_nbap_initiatingMessageValue,
		  &ett_nbap_ProtocolIEValueValue,
		  &ett_nbap_SuccessfulOutcomeValue,
		  &ett_nbap_UnsuccessfulOutcomeValue,

/*--- Included file: packet-nbap-ettarr.c ---*/
#line 1 "packet-nbap-ettarr.c"
    &ett_nbap_NBAP_PDU,
    &ett_nbap_InitiatingMessage,
    &ett_nbap_SuccessfulOutcome,
    &ett_nbap_UnsuccessfulOutcome,
    &ett_nbap_Outcome,
    &ett_nbap_DummyInitiatingmessages,
    &ett_nbap_DummySuccessfullOutcomemessages,
    &ett_nbap_DummyUnsuccessfullOutcomemessages,
    &ett_nbap_DymmyProtocolIE_ID,
    &ett_nbap_CommonTransportChannelSetupRequestFDD,
    &ett_nbap_CommonPhysicalChannelType_CTCH_SetupRqstFDD,
    &ett_nbap_Secondary_CCPCH_CTCH_SetupRqstFDD,
    &ett_nbap_PowerOffsetInformation_CTCH_SetupRqstFDD,
    &ett_nbap_FACH_ParametersListIE_CTCH_SetupRqstFDD,
    &ett_nbap_FACH_ParametersItem_CTCH_SetupRqstFDD,
    &ett_nbap_PCH_ParametersItem_CTCH_SetupRqstFDD,
    &ett_nbap_PICH_Parameters_CTCH_SetupRqstFDD,
    &ett_nbap_MICH_Parameters_CTCH_SetupRqstFDD,
    &ett_nbap_PRACH_CTCH_SetupRqstFDD,
    &ett_nbap_AllowedSlotFormatInformationList_CTCH_SetupRqstFDD,
    &ett_nbap_AllowedSlotFormatInformationItem_CTCH_SetupRqstFDD,
    &ett_nbap_RACH_ParametersItem_CTCH_SetupRqstFDD,
    &ett_nbap_AICH_Parameters_CTCH_SetupRqstFDD,
    &ett_nbap_CommonTransportChannelSetupRequestTDD,
    &ett_nbap_CommonPhysicalChannelType_CTCH_SetupRqstTDD,
    &ett_nbap_Secondary_CCPCH_CTCH_SetupRqstTDD,
    &ett_nbap_Secondary_CCPCH_parameterListIE_CTCH_SetupRqstTDD,
    &ett_nbap_Secondary_CCPCH_parameterItem_CTCH_SetupRqstTDD,
    &ett_nbap_FACH_ParametersListIE_CTCH_SetupRqstTDD,
    &ett_nbap_FACH_ParametersItem_CTCH_SetupRqstTDD,
    &ett_nbap_PCH_ParametersItem_CTCH_SetupRqstTDD,
    &ett_nbap_PICH_ParametersItem_CTCH_SetupRqstTDD,
    &ett_nbap_PICH_LCR_Parameters_CTCH_SetupRqstTDD,
    &ett_nbap_Secondary_CCPCH_LCR_parameterList_CTCH_SetupRqstTDD,
    &ett_nbap_Secondary_CCPCH_LCR_parameterItem_CTCH_SetupRqstTDD,
    &ett_nbap_MICH_Parameters_CTCH_SetupRqstTDD,
    &ett_nbap_MICH_TDDOption_Specific_Parameters_CTCH_SetupRqstTDD,
    &ett_nbap_MICH_HCR_Parameters_CTCH_SetupRqstTDD,
    &ett_nbap_MICH_LCR_Parameters_CTCH_SetupRqstTDD,
    &ett_nbap_Secondary_CCPCH_parameterExtendedList_CTCH_SetupRqstTDD,
    &ett_nbap_Secondary_CCPCH_LCR_parameterExtendedList_CTCH_SetupRqstTDD,
    &ett_nbap_PRACH_CTCH_SetupRqstTDD,
    &ett_nbap_PRACH_ParametersItem_CTCH_SetupRqstTDD,
    &ett_nbap_RACH_ParameterItem_CTCH_SetupRqstTDD,
    &ett_nbap_PRACH_LCR_ParametersList_CTCH_SetupRqstTDD,
    &ett_nbap_PRACH_LCR_ParametersItem_CTCH_SetupRqstTDD,
    &ett_nbap_FPACH_LCR_Parameters_CTCH_SetupRqstTDD,
    &ett_nbap_CommonTransportChannelSetupResponse,
    &ett_nbap_FACH_CommonTransportChannel_InformationResponse,
    &ett_nbap_CommonTransportChannelSetupFailure,
    &ett_nbap_CommonTransportChannelReconfigurationRequestFDD,
    &ett_nbap_CommonPhysicalChannelType_CTCH_ReconfRqstFDD,
    &ett_nbap_Secondary_CCPCHList_CTCH_ReconfRqstFDD,
    &ett_nbap_FACH_ParametersListIE_CTCH_ReconfRqstFDD,
    &ett_nbap_FACH_ParametersItem_CTCH_ReconfRqstFDD,
    &ett_nbap_PCH_ParametersItem_CTCH_ReconfRqstFDD,
    &ett_nbap_PICH_ParametersItem_CTCH_ReconfRqstFDD,
    &ett_nbap_MICH_Parameters_CTCH_ReconfRqstFDD,
    &ett_nbap_PRACHList_CTCH_ReconfRqstFDD,
    &ett_nbap_PRACH_ParametersListIE_CTCH_ReconfRqstFDD,
    &ett_nbap_PRACH_ParametersItem_CTCH_ReconfRqstFDD,
    &ett_nbap_AllowedSlotFormatInformationList_CTCH_ReconfRqstFDD,
    &ett_nbap_AllowedSlotFormatInformationItem_CTCH_ReconfRqstFDD,
    &ett_nbap_AICH_ParametersListIE_CTCH_ReconfRqstFDD,
    &ett_nbap_AICH_ParametersItem_CTCH_ReconfRqstFDD,
    &ett_nbap_CommonTransportChannelReconfigurationRequestTDD,
    &ett_nbap_Secondary_CCPCH_Parameters_CTCH_ReconfRqstTDD,
    &ett_nbap_Secondary_CCPCHListIE_CTCH_ReconfRqstTDD,
    &ett_nbap_Secondary_CCPCHItem_CTCH_ReconfRqstTDD,
    &ett_nbap_PICH_Parameters_CTCH_ReconfRqstTDD,
    &ett_nbap_FACH_ParametersList_CTCH_ReconfRqstTDD,
    &ett_nbap_FACH_ParametersItem_CTCH_ReconfRqstTDD,
    &ett_nbap_PCH_Parameters_CTCH_ReconfRqstTDD,
    &ett_nbap_FPACH_LCR_Parameters_CTCH_ReconfRqstTDD,
    &ett_nbap_MICH_Parameters_CTCH_ReconfRqstTDD,
    &ett_nbap_Secondary_CCPCH_parameterExtendedList_CTCH_ReconfRqstTDD,
    &ett_nbap_Secondary_CCPCH_LCR_parameterExtendedList_CTCH_ReconfRqstTDD,
    &ett_nbap_CommonTransportChannelReconfigurationResponse,
    &ett_nbap_CommonTransportChannelReconfigurationFailure,
    &ett_nbap_CommonTransportChannelDeletionRequest,
    &ett_nbap_CommonTransportChannelDeletionResponse,
    &ett_nbap_BlockResourceRequest,
    &ett_nbap_BlockResourceResponse,
    &ett_nbap_BlockResourceFailure,
    &ett_nbap_UnblockResourceIndication,
    &ett_nbap_AuditRequiredIndication,
    &ett_nbap_AuditRequest,
    &ett_nbap_AuditResponse,
    &ett_nbap_Cell_InformationList_AuditRsp,
    &ett_nbap_Cell_InformationItem_AuditRsp,
    &ett_nbap_S_CPICH_InformationList_AuditRsp,
    &ett_nbap_S_CCPCH_InformationList_AuditRsp,
    &ett_nbap_FACH_InformationList_AuditRsp,
    &ett_nbap_PRACH_InformationList_AuditRsp,
    &ett_nbap_RACH_InformationList_AuditRsp,
    &ett_nbap_AICH_InformationList_AuditRsp,
    &ett_nbap_CCP_InformationList_AuditRsp,
    &ett_nbap_CCP_InformationItem_AuditRsp,
    &ett_nbap_FPACH_LCR_InformationList_AuditRsp,
    &ett_nbap_HS_DSCH_Resources_Information_AuditRsp,
    &ett_nbap_S_CCPCH_InformationListExt_AuditRsp,
    &ett_nbap_S_CCPCH_LCR_InformationListExt_AuditRsp,
    &ett_nbap_E_DCH_Resources_Information_AuditRsp,
    &ett_nbap_Local_Cell_InformationList_AuditRsp,
    &ett_nbap_Local_Cell_InformationItem_AuditRsp,
    &ett_nbap_Local_Cell_Group_InformationList_AuditRsp,
    &ett_nbap_Local_Cell_Group_InformationItem_AuditRsp,
    &ett_nbap_Power_Local_Cell_Group_InformationList_AuditRsp,
    &ett_nbap_Power_Local_Cell_Group_InformationItem_AuditRsp,
    &ett_nbap_AuditFailure,
    &ett_nbap_CommonMeasurementInitiationRequest,
    &ett_nbap_CommonMeasurementObjectType_CM_Rqst,
    &ett_nbap_Cell_CM_Rqst,
    &ett_nbap_RACH_CM_Rqst,
    &ett_nbap_PowerLocalCellGroup_CM_Rqst,
    &ett_nbap_CommonMeasurementInitiationResponse,
    &ett_nbap_CommonMeasurementObjectType_CM_Rsp,
    &ett_nbap_Cell_CM_Rsp,
    &ett_nbap_RACH_CM_Rsp,
    &ett_nbap_PowerLocalCellGroup_CM_Rsp,
    &ett_nbap_CommonMeasurementInitiationFailure,
    &ett_nbap_CellSetupRequestFDD,
    &ett_nbap_Synchronisation_Configuration_Cell_SetupRqst,
    &ett_nbap_PrimarySCH_Information_Cell_SetupRqstFDD,
    &ett_nbap_SecondarySCH_Information_Cell_SetupRqstFDD,
    &ett_nbap_PrimaryCPICH_Information_Cell_SetupRqstFDD,
    &ett_nbap_SecondaryCPICH_InformationList_Cell_SetupRqstFDD,
    &ett_nbap_SecondaryCPICH_InformationItem_Cell_SetupRqstFDD,
    &ett_nbap_PrimaryCCPCH_Information_Cell_SetupRqstFDD,
    &ett_nbap_BCH_Information_Cell_SetupRqstFDD,
    &ett_nbap_Limited_power_increase_information_Cell_SetupRqstFDD,
    &ett_nbap_IPDLParameter_Information_Cell_SetupRqstFDD,
    &ett_nbap_CellPortion_InformationList_Cell_SetupRqstFDD,
    &ett_nbap_CellPortion_InformationItem_Cell_SetupRqstFDD,
    &ett_nbap_CellSetupRequestTDD,
    &ett_nbap_SCH_Information_Cell_SetupRqstTDD,
    &ett_nbap_SyncCaseIndicatorItem_Cell_SetupRqstTDD_PSCH,
    &ett_nbap_Case1_Cell_SetupRqstTDD,
    &ett_nbap_Case2_Cell_SetupRqstTDD,
    &ett_nbap_PCCPCH_Information_Cell_SetupRqstTDD,
    &ett_nbap_TimeSlotConfigurationList_Cell_SetupRqstTDD,
    &ett_nbap_TimeSlotConfigurationItem_Cell_SetupRqstTDD,
    &ett_nbap_TimeSlotConfigurationList_LCR_Cell_SetupRqstTDD,
    &ett_nbap_TimeSlotConfigurationItem_LCR_Cell_SetupRqstTDD,
    &ett_nbap_PCCPCH_LCR_Information_Cell_SetupRqstTDD,
    &ett_nbap_DwPCH_LCR_Information_Cell_SetupRqstTDD,
    &ett_nbap_IPDLParameter_Information_Cell_SetupRqstTDD,
    &ett_nbap_IPDLParameter_Information_LCR_Cell_SetupRqstTDD,
    &ett_nbap_CellSetupResponse,
    &ett_nbap_CellSetupFailure,
    &ett_nbap_CellReconfigurationRequestFDD,
    &ett_nbap_Synchronisation_Configuration_Cell_ReconfRqst,
    &ett_nbap_PrimarySCH_Information_Cell_ReconfRqstFDD,
    &ett_nbap_SecondarySCH_Information_Cell_ReconfRqstFDD,
    &ett_nbap_PrimaryCPICH_Information_Cell_ReconfRqstFDD,
    &ett_nbap_SecondaryCPICH_InformationList_Cell_ReconfRqstFDD,
    &ett_nbap_SecondaryCPICH_InformationItem_Cell_ReconfRqstFDD,
    &ett_nbap_PrimaryCCPCH_Information_Cell_ReconfRqstFDD,
    &ett_nbap_BCH_information_Cell_ReconfRqstFDD,
    &ett_nbap_IPDLParameter_Information_Cell_ReconfRqstFDD,
    &ett_nbap_CellPortion_InformationList_Cell_ReconfRqstFDD,
    &ett_nbap_CellPortion_InformationItem_Cell_ReconfRqstFDD,
    &ett_nbap_CellReconfigurationRequestTDD,
    &ett_nbap_SCH_Information_Cell_ReconfRqstTDD,
    &ett_nbap_PCCPCH_Information_Cell_ReconfRqstTDD,
    &ett_nbap_TimeSlotConfigurationList_Cell_ReconfRqstTDD,
    &ett_nbap_TimeSlotConfigurationItem_Cell_ReconfRqstTDD,
    &ett_nbap_TimeSlotConfigurationList_LCR_Cell_ReconfRqstTDD,
    &ett_nbap_TimeSlotConfigurationItem_LCR_Cell_ReconfRqstTDD,
    &ett_nbap_DwPCH_LCR_Information_Cell_ReconfRqstTDD,
    &ett_nbap_IPDLParameter_Information_Cell_ReconfRqstTDD,
    &ett_nbap_IPDLParameter_Information_LCR_Cell_ReconfRqstTDD,
    &ett_nbap_CellReconfigurationResponse,
    &ett_nbap_CellReconfigurationFailure,
    &ett_nbap_CellDeletionRequest,
    &ett_nbap_CellDeletionResponse,
    &ett_nbap_ResourceStatusIndication,
    &ett_nbap_IndicationType_ResourceStatusInd,
    &ett_nbap_No_Failure_ResourceStatusInd,
    &ett_nbap_Local_Cell_InformationList_ResourceStatusInd,
    &ett_nbap_Local_Cell_InformationItem_ResourceStatusInd,
    &ett_nbap_Local_Cell_Group_InformationList_ResourceStatusInd,
    &ett_nbap_Local_Cell_Group_InformationItem_ResourceStatusInd,
    &ett_nbap_Power_Local_Cell_Group_InformationList_ResourceStatusInd,
    &ett_nbap_Power_Local_Cell_Group_InformationItem_ResourceStatusInd,
    &ett_nbap_ServiceImpacting_ResourceStatusInd,
    &ett_nbap_Local_Cell_InformationList2_ResourceStatusInd,
    &ett_nbap_Local_Cell_InformationItem2_ResourceStatusInd,
    &ett_nbap_Local_Cell_Group_InformationList2_ResourceStatusInd,
    &ett_nbap_Local_Cell_Group_InformationItem2_ResourceStatusInd,
    &ett_nbap_Power_Local_Cell_Group_InformationList2_ResourceStatusInd,
    &ett_nbap_Power_Local_Cell_Group_InformationItem2_ResourceStatusInd,
    &ett_nbap_CCP_InformationList_ResourceStatusInd,
    &ett_nbap_CCP_InformationItem_ResourceStatusInd,
    &ett_nbap_Cell_InformationList_ResourceStatusInd,
    &ett_nbap_Cell_InformationItem_ResourceStatusInd,
    &ett_nbap_S_CPICH_InformationList_ResourceStatusInd,
    &ett_nbap_S_CCPCH_InformationList_ResourceStatusInd,
    &ett_nbap_FACH_InformationList_ResourceStatusInd,
    &ett_nbap_PRACH_InformationList_ResourceStatusInd,
    &ett_nbap_RACH_InformationList_ResourceStatusInd,
    &ett_nbap_AICH_InformationList_ResourceStatusInd,
    &ett_nbap_FPACH_LCR_InformationList_ResourceStatusInd,
    &ett_nbap_HS_DSCH_Resources_Information_ResourceStatusInd,
    &ett_nbap_S_CCPCH_InformationListExt_ResourceStatusInd,
    &ett_nbap_S_CCPCH_LCR_InformationListExt_ResourceStatusInd,
    &ett_nbap_E_DCH_Resources_Information_ResourceStatusInd,
    &ett_nbap_SystemInformationUpdateRequest,
    &ett_nbap_MIB_SB_SIB_InformationList_SystemInfoUpdateRqst,
    &ett_nbap_MIB_SB_SIB_InformationItem_SystemInfoUpdateRqst,
    &ett_nbap_DeletionIndicator_SystemInfoUpdate,
    &ett_nbap_No_Deletion_SystemInfoUpdate,
    &ett_nbap_SegmentInformationListIE_SystemInfoUpdate,
    &ett_nbap_SegmentInformationItem_SystemInfoUpdate,
    &ett_nbap_SystemInformationUpdateResponse,
    &ett_nbap_SystemInformationUpdateFailure,
    &ett_nbap_RadioLinkSetupRequestFDD,
    &ett_nbap_UL_DPCH_Information_RL_SetupRqstFDD,
    &ett_nbap_DL_DPCH_Information_RL_SetupRqstFDD,
    &ett_nbap_PowerOffsetInformation_RL_SetupRqstFDD,
    &ett_nbap_RL_InformationList_RL_SetupRqstFDD,
    &ett_nbap_RL_InformationItem_RL_SetupRqstFDD,
    &ett_nbap_E_DPCH_Information_RL_SetupRqstFDD,
    &ett_nbap_F_DPCH_Information_RL_SetupRqstFDD,
    &ett_nbap_PowerOffsetInformation_F_DPCH_RL_SetupRqstFDD,
    &ett_nbap_RadioLinkSetupRequestTDD,
    &ett_nbap_UL_CCTrCH_InformationList_RL_SetupRqstTDD,
    &ett_nbap_UL_CCTrCH_InformationItem_RL_SetupRqstTDD,
    &ett_nbap_UL_DPCH_InformationItem_RL_SetupRqstTDD,
    &ett_nbap_UL_DPCH_LCR_Information_RL_SetupRqstTDD,
    &ett_nbap_DL_CCTrCH_InformationList_RL_SetupRqstTDD,
    &ett_nbap_DL_CCTrCH_InformationItem_RL_SetupRqstTDD,
    &ett_nbap_CCTrCH_TPCList_RL_SetupRqstTDD,
    &ett_nbap_CCTrCH_TPCItem_RL_SetupRqstTDD,
    &ett_nbap_DL_DPCH_InformationItem_RL_SetupRqstTDD,
    &ett_nbap_DL_DPCH_LCR_Information_RL_SetupRqstTDD,
    &ett_nbap_RL_Information_RL_SetupRqstTDD,
    &ett_nbap_RadioLinkSetupResponseFDD,
    &ett_nbap_RL_InformationResponseList_RL_SetupRspFDD,
    &ett_nbap_RL_InformationResponseItem_RL_SetupRspFDD,
    &ett_nbap_DiversityIndication_RL_SetupRspFDD,
    &ett_nbap_Combining_RL_SetupRspFDD,
    &ett_nbap_NonCombiningOrFirstRL_RL_SetupRspFDD,
    &ett_nbap_RadioLinkSetupResponseTDD,
    &ett_nbap_RL_InformationResponse_RL_SetupRspTDD,
    &ett_nbap_RL_InformationResponse_LCR_RL_SetupRspTDD,
    &ett_nbap_RadioLinkSetupFailureFDD,
    &ett_nbap_CauseLevel_RL_SetupFailureFDD,
    &ett_nbap_GeneralCauseList_RL_SetupFailureFDD,
    &ett_nbap_RLSpecificCauseList_RL_SetupFailureFDD,
    &ett_nbap_Unsuccessful_RL_InformationRespList_RL_SetupFailureFDD,
    &ett_nbap_Unsuccessful_RL_InformationRespItem_RL_SetupFailureFDD,
    &ett_nbap_Successful_RL_InformationRespList_RL_SetupFailureFDD,
    &ett_nbap_Successful_RL_InformationRespItem_RL_SetupFailureFDD,
    &ett_nbap_DiversityIndication_RL_SetupFailureFDD,
    &ett_nbap_Combining_RL_SetupFailureFDD,
    &ett_nbap_NonCombiningOrFirstRL_RL_SetupFailureFDD,
    &ett_nbap_RadioLinkSetupFailureTDD,
    &ett_nbap_CauseLevel_RL_SetupFailureTDD,
    &ett_nbap_GeneralCauseList_RL_SetupFailureTDD,
    &ett_nbap_RLSpecificCauseList_RL_SetupFailureTDD,
    &ett_nbap_Unsuccessful_RL_InformationResp_RL_SetupFailureTDD,
    &ett_nbap_RadioLinkAdditionRequestFDD,
    &ett_nbap_RL_InformationList_RL_AdditionRqstFDD,
    &ett_nbap_RL_InformationItem_RL_AdditionRqstFDD,
    &ett_nbap_RadioLinkAdditionRequestTDD,
    &ett_nbap_UL_CCTrCH_InformationList_RL_AdditionRqstTDD,
    &ett_nbap_UL_CCTrCH_InformationItem_RL_AdditionRqstTDD,
    &ett_nbap_UL_DPCH_InformationItem_RL_AdditionRqstTDD,
    &ett_nbap_DL_CCTrCH_InformationList_RL_AdditionRqstTDD,
    &ett_nbap_DL_CCTrCH_InformationItem_RL_AdditionRqstTDD,
    &ett_nbap_DL_DPCH_InformationItem_RL_AdditionRqstTDD,
    &ett_nbap_RL_Information_RL_AdditionRqstTDD,
    &ett_nbap_UL_DPCH_InformationItem_LCR_RL_AdditionRqstTDD,
    &ett_nbap_DL_DPCH_InformationItem_LCR_RL_AdditionRqstTDD,
    &ett_nbap_RadioLinkAdditionResponseFDD,
    &ett_nbap_RL_InformationResponseList_RL_AdditionRspFDD,
    &ett_nbap_RL_InformationResponseItem_RL_AdditionRspFDD,
    &ett_nbap_DiversityIndication_RL_AdditionRspFDD,
    &ett_nbap_Combining_RL_AdditionRspFDD,
    &ett_nbap_Non_Combining_RL_AdditionRspFDD,
    &ett_nbap_RadioLinkAdditionResponseTDD,
    &ett_nbap_RL_InformationResponse_RL_AdditionRspTDD,
    &ett_nbap_DCH_Information_RL_AdditionRspTDD,
    &ett_nbap_DiversityIndication_RL_AdditionRspTDD,
    &ett_nbap_Combining_RL_AdditionRspTDD,
    &ett_nbap_Non_Combining_RL_AdditionRspTDD,
    &ett_nbap_RL_InformationResponse_LCR_RL_AdditionRspTDD,
    &ett_nbap_RadioLinkAdditionFailureFDD,
    &ett_nbap_CauseLevel_RL_AdditionFailureFDD,
    &ett_nbap_GeneralCauseList_RL_AdditionFailureFDD,
    &ett_nbap_RLSpecificCauseList_RL_AdditionFailureFDD,
    &ett_nbap_Unsuccessful_RL_InformationRespList_RL_AdditionFailureFDD,
    &ett_nbap_Unsuccessful_RL_InformationRespItem_RL_AdditionFailureFDD,
    &ett_nbap_Successful_RL_InformationRespList_RL_AdditionFailureFDD,
    &ett_nbap_Successful_RL_InformationRespItem_RL_AdditionFailureFDD,
    &ett_nbap_DiversityIndication_RL_AdditionFailureFDD,
    &ett_nbap_Combining_RL_AdditionFailureFDD,
    &ett_nbap_Non_Combining_RL_AdditionFailureFDD,
    &ett_nbap_RadioLinkAdditionFailureTDD,
    &ett_nbap_CauseLevel_RL_AdditionFailureTDD,
    &ett_nbap_GeneralCauseList_RL_AdditionFailureTDD,
    &ett_nbap_RLSpecificCauseList_RL_AdditionFailureTDD,
    &ett_nbap_Unsuccessful_RL_InformationResp_RL_AdditionFailureTDD,
    &ett_nbap_RadioLinkReconfigurationPrepareFDD,
    &ett_nbap_UL_DPCH_Information_RL_ReconfPrepFDD,
    &ett_nbap_DL_DPCH_Information_RL_ReconfPrepFDD,
    &ett_nbap_DL_DPCH_Power_Information_RL_ReconfPrepFDD,
    &ett_nbap_PowerOffsetInformation_RL_ReconfPrepFDD,
    &ett_nbap_DCH_DeleteList_RL_ReconfPrepFDD,
    &ett_nbap_DCH_DeleteItem_RL_ReconfPrepFDD,
    &ett_nbap_RL_InformationList_RL_ReconfPrepFDD,
    &ett_nbap_RL_InformationItem_RL_ReconfPrepFDD,
    &ett_nbap_E_DPCH_Information_RL_ReconfPrepFDD,
    &ett_nbap_F_DPCH_Information_RL_ReconfPrepFDD,
    &ett_nbap_PowerOffsetInformation_F_DPCH_RL_ReconfPrepFDD,
    &ett_nbap_RadioLinkReconfigurationPrepareTDD,
    &ett_nbap_UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD,
    &ett_nbap_UL_CCTrCH_InformationAddItem_RL_ReconfPrepTDD,
    &ett_nbap_MultipleRL_UL_DPCH_InformationAddList_RL_ReconfPrepTDD,
    &ett_nbap_MultipleRL_UL_DPCH_InformationAddListIE_RL_ReconfPrepTDD,
    &ett_nbap_UL_DPCH_InformationAddItem_RL_ReconfPrepTDD,
    &ett_nbap_UL_DPCH_LCR_InformationAddList_RL_ReconfPrepTDD,
    &ett_nbap_UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD,
    &ett_nbap_UL_CCTrCH_InformationModifyItem_RL_ReconfPrepTDD,
    &ett_nbap_MultipleRL_UL_DPCH_InformationModifyList_RL_ReconfPrepTDD,
    &ett_nbap_MultipleRL_UL_DPCH_InformationModifyListIE_RL_ReconfPrepTDD,
    &ett_nbap_UL_DPCH_InformationModify_AddItem_RL_ReconfPrepTDD,
    &ett_nbap_UL_DPCH_LCR_InformationModify_AddList_RL_ReconfPrepTDD,
    &ett_nbap_UL_DPCH_InformationModify_ModifyItem_RL_ReconfPrepTDD,
    &ett_nbap_UL_Timeslot_InformationModify_ModifyList_RL_ReconfPrepTDD,
    &ett_nbap_UL_Timeslot_InformationModify_ModifyItem_RL_ReconfPrepTDD,
    &ett_nbap_UL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD,
    &ett_nbap_UL_Code_InformationModify_ModifyItem_RL_ReconfPrepTDD,
    &ett_nbap_UL_TimeslotLCR_InformationModify_ModifyList_RL_ReconfPrepTDD,
    &ett_nbap_UL_Timeslot_LCR_InformationModify_ModifyItem_RL_ReconfPrepTDD,
    &ett_nbap_UL_Code_InformationModify_ModifyList_RL_ReconfPrepTDDLCR,
    &ett_nbap_UL_Code_InformationModify_ModifyItem_RL_ReconfPrepTDDLCR,
    &ett_nbap_UL_DPCH_InformationModify_DeleteListIE_RL_ReconfPrepTDD,
    &ett_nbap_UL_DPCH_InformationModify_DeleteItem_RL_ReconfPrepTDD,
    &ett_nbap_UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD,
    &ett_nbap_UL_CCTrCH_InformationDeleteItem_RL_ReconfPrepTDD,
    &ett_nbap_DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD,
    &ett_nbap_DL_CCTrCH_InformationAddItem_RL_ReconfPrepTDD,
    &ett_nbap_MultipleRL_DL_DPCH_InformationAddList_RL_ReconfPrepTDD,
    &ett_nbap_MultipleRL_DL_DPCH_InformationAddListIE_RL_ReconfPrepTDD,
    &ett_nbap_CCTrCH_TPCAddList_RL_ReconfPrepTDD,
    &ett_nbap_CCTrCH_TPCAddItem_RL_ReconfPrepTDD,
    &ett_nbap_DL_DPCH_InformationAddItem_RL_ReconfPrepTDD,
    &ett_nbap_DL_DPCH_LCR_InformationAddList_RL_ReconfPrepTDD,
    &ett_nbap_DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD,
    &ett_nbap_DL_CCTrCH_InformationModifyItem_RL_ReconfPrepTDD,
    &ett_nbap_MultipleRL_DL_DPCH_InformationModifyList_RL_ReconfPrepTDD,
    &ett_nbap_MultipleRL_DL_DPCH_InformationModifyListIE_RL_ReconfPrepTDD,
    &ett_nbap_CCTrCH_TPCModifyList_RL_ReconfPrepTDD,
    &ett_nbap_CCTrCH_TPCModifyItem_RL_ReconfPrepTDD,
    &ett_nbap_DL_DPCH_InformationModify_AddItem_RL_ReconfPrepTDD,
    &ett_nbap_DL_DPCH_LCR_InformationModify_AddList_RL_ReconfPrepTDD,
    &ett_nbap_DL_DPCH_InformationModify_ModifyItem_RL_ReconfPrepTDD,
    &ett_nbap_DL_Timeslot_InformationModify_ModifyList_RL_ReconfPrepTDD,
    &ett_nbap_DL_Timeslot_InformationModify_ModifyItem_RL_ReconfPrepTDD,
    &ett_nbap_DL_Code_InformationModify_ModifyList_RL_ReconfPrepTDD,
    &ett_nbap_DL_Code_InformationModify_ModifyItem_RL_ReconfPrepTDD,
    &ett_nbap_DL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD,
    &ett_nbap_DL_Timeslot_LCR_InformationModify_ModifyItem_RL_ReconfPrepTDD,
    &ett_nbap_DL_Code_LCR_InformationModify_ModifyList_RL_ReconfPrepTDD,
    &ett_nbap_DL_Code_LCR_InformationModify_ModifyItem_RL_ReconfPrepTDD,
    &ett_nbap_DL_DPCH_InformationModify_DeleteListIE_RL_ReconfPrepTDD,
    &ett_nbap_DL_DPCH_InformationModify_DeleteItem_RL_ReconfPrepTDD,
    &ett_nbap_DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD,
    &ett_nbap_DL_CCTrCH_InformationDeleteItem_RL_ReconfPrepTDD,
    &ett_nbap_DCH_DeleteList_RL_ReconfPrepTDD,
    &ett_nbap_DCH_DeleteItem_RL_ReconfPrepTDD,
    &ett_nbap_DSCH_Information_ModifyList_RL_ReconfPrepTDD,
    &ett_nbap_DSCH_Information_ModifyItem_RL_ReconfPrepTDD,
    &ett_nbap_DSCH_Information_DeleteList_RL_ReconfPrepTDD,
    &ett_nbap_DSCH_Information_DeleteItem_RL_ReconfPrepTDD,
    &ett_nbap_USCH_Information_ModifyList_RL_ReconfPrepTDD,
    &ett_nbap_USCH_Information_ModifyItem_RL_ReconfPrepTDD,
    &ett_nbap_USCH_Information_DeleteList_RL_ReconfPrepTDD,
    &ett_nbap_USCH_Information_DeleteItem_RL_ReconfPrepTDD,
    &ett_nbap_MultipleRL_Information_RL_ReconfPrepTDD,
    &ett_nbap_RL_Information_RL_ReconfPrepTDD,
    &ett_nbap_RadioLinkReconfigurationReady,
    &ett_nbap_RL_InformationResponseList_RL_ReconfReady,
    &ett_nbap_RL_InformationResponseItem_RL_ReconfReady,
    &ett_nbap_RadioLinkReconfigurationFailure,
    &ett_nbap_CauseLevel_RL_ReconfFailure,
    &ett_nbap_GeneralCauseList_RL_ReconfFailure,
    &ett_nbap_RLSpecificCauseList_RL_ReconfFailure,
    &ett_nbap_RL_ReconfigurationFailureList_RL_ReconfFailure,
    &ett_nbap_RL_ReconfigurationFailureItem_RL_ReconfFailure,
    &ett_nbap_RadioLinkReconfigurationCommit,
    &ett_nbap_RadioLinkReconfigurationCancel,
    &ett_nbap_RadioLinkReconfigurationRequestFDD,
    &ett_nbap_UL_DPCH_Information_RL_ReconfRqstFDD,
    &ett_nbap_DL_DPCH_Information_RL_ReconfRqstFDD,
    &ett_nbap_DCH_DeleteList_RL_ReconfRqstFDD,
    &ett_nbap_DCH_DeleteItem_RL_ReconfRqstFDD,
    &ett_nbap_RL_InformationList_RL_ReconfRqstFDD,
    &ett_nbap_RL_InformationItem_RL_ReconfRqstFDD,
    &ett_nbap_E_DPCH_Information_RL_ReconfRqstFDD,
    &ett_nbap_RadioLinkReconfigurationRequestTDD,
    &ett_nbap_UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD,
    &ett_nbap_UL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD,
    &ett_nbap_UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD,
    &ett_nbap_UL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD,
    &ett_nbap_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD,
    &ett_nbap_DL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD,
    &ett_nbap_MultipleRL_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD,
    &ett_nbap_MultipleRL_DL_CCTrCH_InformationModifyListIE_RL_ReconfRqstTDD,
    &ett_nbap_DL_DPCH_LCR_InformationModify_ModifyList_RL_ReconfRqstTDD,
    &ett_nbap_DL_Timeslot_LCR_InformationModify_ModifyList_RL_ReconfRqstTDD,
    &ett_nbap_DL_Timeslot_LCR_InformationModify_ModifyItem_RL_ReconfRqstTDD,
    &ett_nbap_DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD,
    &ett_nbap_DL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD,
    &ett_nbap_DCH_DeleteList_RL_ReconfRqstTDD,
    &ett_nbap_DCH_DeleteItem_RL_ReconfRqstTDD,
    &ett_nbap_Multiple_RL_Information_RL_ReconfRqstTDD,
    &ett_nbap_RL_Information_RL_ReconfRqstTDD,
    &ett_nbap_RadioLinkReconfigurationResponse,
    &ett_nbap_RL_InformationResponseList_RL_ReconfRsp,
    &ett_nbap_RL_InformationResponseItem_RL_ReconfRsp,
    &ett_nbap_RadioLinkDeletionRequest,
    &ett_nbap_RL_informationList_RL_DeletionRqst,
    &ett_nbap_RL_informationItem_RL_DeletionRqst,
    &ett_nbap_RadioLinkDeletionResponse,
    &ett_nbap_DL_PowerControlRequest,
    &ett_nbap_DL_ReferencePowerInformationList_DL_PC_Rqst,
    &ett_nbap_DL_ReferencePowerInformationItem_DL_PC_Rqst,
    &ett_nbap_DL_PowerTimeslotControlRequest,
    &ett_nbap_DedicatedMeasurementInitiationRequest,
    &ett_nbap_DedicatedMeasurementObjectType_DM_Rqst,
    &ett_nbap_RL_DM_Rqst,
    &ett_nbap_RL_InformationList_DM_Rqst,
    &ett_nbap_RL_InformationItem_DM_Rqst,
    &ett_nbap_PUSCH_Info_DM_Rqst,
    &ett_nbap_HSSICH_Info_DM_Rqst,
    &ett_nbap_RL_Set_DM_Rqst,
    &ett_nbap_RL_Set_InformationList_DM_Rqst,
    &ett_nbap_RL_Set_InformationItem_DM_Rqst,
    &ett_nbap_DedicatedMeasurementInitiationResponse,
    &ett_nbap_DedicatedMeasurementObjectType_DM_Rsp,
    &ett_nbap_RL_DM_Rsp,
    &ett_nbap_RL_InformationList_DM_Rsp,
    &ett_nbap_RL_InformationItem_DM_Rsp,
    &ett_nbap_PUSCH_Info_DM_Rsp,
    &ett_nbap_Multiple_PUSCH_InfoList_DM_Rsp,
    &ett_nbap_Multiple_PUSCH_InfoListIE_DM_Rsp,
    &ett_nbap_RL_Set_DM_Rsp,
    &ett_nbap_RL_Set_InformationList_DM_Rsp,
    &ett_nbap_RL_Set_InformationItem_DM_Rsp,
    &ett_nbap_Multiple_DedicatedMeasurementValueList_TDD_DM_Rsp,
    &ett_nbap_Multiple_DedicatedMeasurementValueItem_TDD_DM_Rsp,
    &ett_nbap_Multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp,
    &ett_nbap_Multiple_DedicatedMeasurementValueItem_LCR_TDD_DM_Rsp,
    &ett_nbap_DedicatedMeasurementInitiationFailure,
    &ett_nbap_DedicatedMeasurementReport,
    &ett_nbap_DedicatedMeasurementObjectType_DM_Rprt,
    &ett_nbap_RL_DM_Rprt,
    &ett_nbap_RL_InformationList_DM_Rprt,
    &ett_nbap_RL_InformationItem_DM_Rprt,
    &ett_nbap_PUSCH_Info_DM_Rprt,
    &ett_nbap_Multiple_PUSCH_InfoList_DM_Rprt,
    &ett_nbap_Multiple_PUSCH_InfoListIE_DM_Rprt,
    &ett_nbap_RL_Set_DM_Rprt,
    &ett_nbap_RL_Set_InformationList_DM_Rprt,
    &ett_nbap_RL_Set_InformationItem_DM_Rprt,
    &ett_nbap_DedicatedMeasurementTerminationRequest,
    &ett_nbap_DedicatedMeasurementFailureIndication,
    &ett_nbap_RadioLinkFailureIndication,
    &ett_nbap_Reporting_Object_RL_FailureInd,
    &ett_nbap_RL_RL_FailureInd,
    &ett_nbap_RL_InformationList_RL_FailureInd,
    &ett_nbap_RL_InformationItem_RL_FailureInd,
    &ett_nbap_RL_Set_RL_FailureInd,
    &ett_nbap_RL_Set_InformationList_RL_FailureInd,
    &ett_nbap_RL_Set_InformationItem_RL_FailureInd,
    &ett_nbap_CCTrCH_RL_FailureInd,
    &ett_nbap_CCTrCH_InformationList_RL_FailureInd,
    &ett_nbap_CCTrCH_InformationItem_RL_FailureInd,
    &ett_nbap_RadioLinkPreemptionRequiredIndication,
    &ett_nbap_RL_InformationList_RL_PreemptRequiredInd,
    &ett_nbap_RL_InformationItem_RL_PreemptRequiredInd,
    &ett_nbap_RadioLinkRestoreIndication,
    &ett_nbap_Reporting_Object_RL_RestoreInd,
    &ett_nbap_RL_RL_RestoreInd,
    &ett_nbap_RL_InformationList_RL_RestoreInd,
    &ett_nbap_RL_InformationItem_RL_RestoreInd,
    &ett_nbap_RL_Set_RL_RestoreInd,
    &ett_nbap_RL_Set_InformationList_RL_RestoreInd,
    &ett_nbap_RL_Set_InformationItem_RL_RestoreInd,
    &ett_nbap_CCTrCH_RL_RestoreInd,
    &ett_nbap_CCTrCH_InformationList_RL_RestoreInd,
    &ett_nbap_CCTrCH_InformationItem_RL_RestoreInd,
    &ett_nbap_CompressedModeCommand,
    &ett_nbap_ErrorIndication,
    &ett_nbap_PrivateMessage,
    &ett_nbap_PhysicalSharedChannelReconfigurationRequestFDD,
    &ett_nbap_HSDPA_And_EDCH_CellPortion_InformationList_PSCH_ReconfRqst,
    &ett_nbap_HSDPA_And_EDCH_CellPortion_InformationItem_PSCH_ReconfRqst,
    &ett_nbap_PhysicalSharedChannelReconfigurationRequestTDD,
    &ett_nbap_PDSCHSets_AddList_PSCH_ReconfRqst,
    &ett_nbap_PDSCHSets_AddItem_PSCH_ReconfRqst,
    &ett_nbap_PDSCH_Information_AddItem_PSCH_ReconfRqst,
    &ett_nbap_DL_Timeslot_InformationAddList_PSCH_ReconfRqst,
    &ett_nbap_DL_Timeslot_InformationAddItem_PSCH_ReconfRqst,
    &ett_nbap_DL_Code_InformationAddList_PSCH_ReconfRqst,
    &ett_nbap_DL_Code_InformationAddItem_PSCH_ReconfRqst,
    &ett_nbap_PDSCH_AddInformation_LCR_AddItem_PSCH_ReconfRqst,
    &ett_nbap_DL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst,
    &ett_nbap_DL_Timeslot_InformationAddItem_LCR_PSCH_ReconfRqst,
    &ett_nbap_DL_Code_InformationAddList_LCR_PSCH_ReconfRqst,
    &ett_nbap_DL_Code_InformationAddItem_LCR_PSCH_ReconfRqst,
    &ett_nbap_PDSCHSets_ModifyList_PSCH_ReconfRqst,
    &ett_nbap_PDSCHSets_ModifyItem_PSCH_ReconfRqst,
    &ett_nbap_PDSCH_Information_ModifyItem_PSCH_ReconfRqst,
    &ett_nbap_DL_Timeslot_InformationModifyList_PSCH_ReconfRqst,
    &ett_nbap_DL_Timeslot_InformationModifyItem_PSCH_ReconfRqst,
    &ett_nbap_DL_Code_InformationModifyList_PSCH_ReconfRqst,
    &ett_nbap_DL_Code_InformationModifyItem_PSCH_ReconfRqst,
    &ett_nbap_PDSCH_ModifyInformation_LCR_ModifyItem_PSCH_ReconfRqst,
    &ett_nbap_DL_Timeslot_LCR_InformationModifyList_PSCH_ReconfRqst,
    &ett_nbap_DL_Timeslot_LCR_InformationModifyItem_PSCH_ReconfRqst,
    &ett_nbap_DL_Code_LCR_InformationModifyList_PSCH_ReconfRqst,
    &ett_nbap_DL_Code_LCR_InformationModifyItem_PSCH_ReconfRqst,
    &ett_nbap_PDSCHSets_DeleteList_PSCH_ReconfRqst,
    &ett_nbap_PDSCHSets_DeleteItem_PSCH_ReconfRqst,
    &ett_nbap_PUSCHSets_AddList_PSCH_ReconfRqst,
    &ett_nbap_PUSCHSets_AddItem_PSCH_ReconfRqst,
    &ett_nbap_PUSCH_Information_AddItem_PSCH_ReconfRqst,
    &ett_nbap_UL_Timeslot_InformationAddList_PSCH_ReconfRqst,
    &ett_nbap_UL_Timeslot_InformationAddItem_PSCH_ReconfRqst,
    &ett_nbap_UL_Code_InformationAddList_PSCH_ReconfRqst,
    &ett_nbap_UL_Code_InformationAddItem_PSCH_ReconfRqst,
    &ett_nbap_PUSCH_AddInformation_LCR_AddItem_PSCH_ReconfRqst,
    &ett_nbap_UL_Timeslot_InformationAddList_LCR_PSCH_ReconfRqst,
    &ett_nbap_UL_Timeslot_InformationAddItem_LCR_PSCH_ReconfRqst,
    &ett_nbap_UL_Code_InformationAddList_LCR_PSCH_ReconfRqst,
    &ett_nbap_UL_Code_InformationAddItem_LCR_PSCH_ReconfRqst,
    &ett_nbap_PUSCHSets_ModifyList_PSCH_ReconfRqst,
    &ett_nbap_PUSCHSets_ModifyItem_PSCH_ReconfRqst,
    &ett_nbap_PUSCH_Information_ModifyItem_PSCH_ReconfRqst,
    &ett_nbap_UL_Timeslot_InformationModifyList_PSCH_ReconfRqst,
    &ett_nbap_UL_Timeslot_InformationModifyItem_PSCH_ReconfRqst,
    &ett_nbap_UL_Code_InformationModifyList_PSCH_ReconfRqst,
    &ett_nbap_UL_Code_InformationModifyItem_PSCH_ReconfRqst,
    &ett_nbap_PUSCH_ModifyInformation_LCR_ModifyItem_PSCH_ReconfRqst,
    &ett_nbap_UL_Timeslot_LCR_InformationModifyList_PSCH_ReconfRqst,
    &ett_nbap_UL_Timeslot_LCR_InformationModifyItem_PSCH_ReconfRqst,
    &ett_nbap_UL_Code_LCR_InformationModifyList_PSCH_ReconfRqst,
    &ett_nbap_UL_Code_LCR_InformationModifyItem_PSCH_ReconfRqst,
    &ett_nbap_PUSCHSets_DeleteList_PSCH_ReconfRqst,
    &ett_nbap_PUSCHSets_DeleteItem_PSCH_ReconfRqst,
    &ett_nbap_HS_PDSCH_TDD_Information_PSCH_ReconfRqst,
    &ett_nbap_DL_HS_PDSCH_Timeslot_Information_PSCH_ReconfRqst,
    &ett_nbap_DL_HS_PDSCH_Timeslot_InformationItem_PSCH_ReconfRqst,
    &ett_nbap_DL_HS_PDSCH_Codelist_PSCH_ReconfRqst,
    &ett_nbap_DL_HS_PDSCH_Timeslot_Information_LCR_PSCH_ReconfRqst,
    &ett_nbap_DL_HS_PDSCH_Timeslot_InformationItem_LCR_PSCH_ReconfRqst,
    &ett_nbap_DL_HS_PDSCH_Codelist_LCR_PSCH_ReconfRqst,
    &ett_nbap_Add_To_HS_SCCH_Resource_Pool_PSCH_ReconfRqst,
    &ett_nbap_HS_SCCH_Information_PSCH_ReconfRqst,
    &ett_nbap_HS_SCCH_InformationItem_PSCH_ReconfRqst,
    &ett_nbap_HS_SICH_Information_PSCH_ReconfRqst,
    &ett_nbap_HS_SCCH_Information_LCR_PSCH_ReconfRqst,
    &ett_nbap_HS_SCCH_InformationItem_LCR_PSCH_ReconfRqst,
    &ett_nbap_HS_SICH_Information_LCR_PSCH_ReconfRqst,
    &ett_nbap_Modify_HS_SCCH_Resource_Pool_PSCH_ReconfRqst,
    &ett_nbap_HS_SCCH_InformationModify_PSCH_ReconfRqst,
    &ett_nbap_HS_SCCH_InformationModifyItem_PSCH_ReconfRqst,
    &ett_nbap_HS_SICH_InformationModify_PSCH_ReconfRqst,
    &ett_nbap_HS_SCCH_InformationModify_LCR_PSCH_ReconfRqst,
    &ett_nbap_HS_SCCH_InformationModifyItem_LCR_PSCH_ReconfRqst,
    &ett_nbap_HS_SICH_InformationModify_LCR_PSCH_ReconfRqst,
    &ett_nbap_Delete_From_HS_SCCH_Resource_Pool_PSCH_ReconfRqst,
    &ett_nbap_Delete_From_HS_SCCH_Resource_PoolItem_PSCH_ReconfRqst,
    &ett_nbap_PhysicalSharedChannelReconfigurationResponse,
    &ett_nbap_PhysicalSharedChannelReconfigurationFailure,
    &ett_nbap_CauseLevel_PSCH_ReconfFailure,
    &ett_nbap_GeneralCauseList_PSCH_ReconfFailure,
    &ett_nbap_SetSpecificCauseList_PSCH_ReconfFailureTDD,
    &ett_nbap_Unsuccessful_PDSCHSetList_PSCH_ReconfFailureTDD,
    &ett_nbap_Unsuccessful_PDSCHSetItem_PSCH_ReconfFailureTDD,
    &ett_nbap_Unsuccessful_PUSCHSetList_PSCH_ReconfFailureTDD,
    &ett_nbap_Unsuccessful_PUSCHSetItem_PSCH_ReconfFailureTDD,
    &ett_nbap_ResetRequest,
    &ett_nbap_ResetIndicator,
    &ett_nbap_CommunicationContextList_Reset,
    &ett_nbap_CommunicationContextInfoList_Reset,
    &ett_nbap_CommunicationContextInfoItem_Reset,
    &ett_nbap_CommunicationContextType_Reset,
    &ett_nbap_CommunicationControlPortList_Reset,
    &ett_nbap_CommunicationControlPortInfoList_Reset,
    &ett_nbap_CommunicationControlPortInfoItem_Reset,
    &ett_nbap_ResetResponse,
    &ett_nbap_InformationExchangeInitiationRequest,
    &ett_nbap_InformationExchangeObjectType_InfEx_Rqst,
    &ett_nbap_Cell_InfEx_Rqst,
    &ett_nbap_InformationExchangeInitiationResponse,
    &ett_nbap_InformationExchangeObjectType_InfEx_Rsp,
    &ett_nbap_Cell_InfEx_Rsp,
    &ett_nbap_InformationExchangeInitiationFailure,
    &ett_nbap_InformationReport,
    &ett_nbap_InformationExchangeObjectType_InfEx_Rprt,
    &ett_nbap_Cell_Inf_Rprt,
    &ett_nbap_InformationExchangeTerminationRequest,
    &ett_nbap_InformationExchangeFailureIndication,
    &ett_nbap_CellSynchronisationInitiationRequestTDD,
    &ett_nbap_CellSyncBurstTransInit_CellSyncInitiationRqstTDD,
    &ett_nbap_TimeslotInfo_CellSyncInitiationRqstTDD,
    &ett_nbap_CellSyncBurstMeasureInit_CellSyncInitiationRqstTDD,
    &ett_nbap_SYNCDlCodeId_TransInitLCR_CellSyncInitiationRqstTDD,
    &ett_nbap_SYNCDlCodeId_MeasureInitLCR_CellSyncInitiationRqstTDD,
    &ett_nbap_CellSynchronisationInitiationResponseTDD,
    &ett_nbap_CellSynchronisationInitiationFailureTDD,
    &ett_nbap_CellSynchronisationReconfigurationRequestTDD,
    &ett_nbap_CellSyncBurstTransReconfInfo_CellSyncReconfRqstTDD,
    &ett_nbap_CellSyncBurstTransInfoItem_CellSyncReconfRqstTDD,
    &ett_nbap_CellSyncBurstMeasInfo_CellSyncReconfRqstTDD,
    &ett_nbap_CellSyncBurstMeasInfoListIE_CellSyncReconfRqstTDD,
    &ett_nbap_CellSyncBurstMeasInfoItem_CellSyncReconfRqstTDD,
    &ett_nbap_CellSyncBurstInfoList_CellSyncReconfRqstTDD,
    &ett_nbap_CellSyncBurstInfoItem_CellSyncReconfRqstTDD,
    &ett_nbap_SYNCDlCodeIdTransReconfInfoLCR_CellSyncReconfRqstTDD,
    &ett_nbap_SYNCDlCodeIdTransReconfItemLCR_CellSyncReconfRqstTDD,
    &ett_nbap_SYNCDlCodeIdMeasInfoLCR_CellSyncReconfRqstTDD,
    &ett_nbap_SYNCDlCodeIdMeasInfoList_CellSyncReconfRqstTDD,
    &ett_nbap_SYNCDlCodeIdMeasInfoItem_CellSyncReconfRqstTDD,
    &ett_nbap_SYNCDlCodeIdInfoListLCR_CellSyncReconfRqstTDD,
    &ett_nbap_SYNCDlCodeIdInfoItemLCR_CellSyncReconfRqstTDD,
    &ett_nbap_CellSynchronisationReconfigurationResponseTDD,
    &ett_nbap_CellSynchronisationReconfigurationFailureTDD,
    &ett_nbap_CellSynchronisationAdjustmentRequestTDD,
    &ett_nbap_CellAdjustmentInfo_SyncAdjustmentRqstTDD,
    &ett_nbap_CellAdjustmentInfoItem_SyncAdjustmentRqstTDD,
    &ett_nbap_CellSynchronisationAdjustmentResponseTDD,
    &ett_nbap_CellSynchronisationAdjustmentFailureTDD,
    &ett_nbap_CauseLevel_SyncAdjustmntFailureTDD,
    &ett_nbap_GeneralCauseList_SyncAdjustmntFailureTDD,
    &ett_nbap_CellSpecificCauseList_SyncAdjustmntFailureTDD,
    &ett_nbap_Unsuccessful_cell_InformationRespList_SyncAdjustmntFailureTDD,
    &ett_nbap_Unsuccessful_cell_InformationRespItem_SyncAdjustmntFailureTDD,
    &ett_nbap_CellSynchronisationTerminationRequestTDD,
    &ett_nbap_CellSynchronisationFailureIndicationTDD,
    &ett_nbap_CellSynchronisationReportTDD,
    &ett_nbap_CellSyncInfo_CellSyncReprtTDD,
    &ett_nbap_CellSyncInfoItemIE_CellSyncReprtTDD,
    &ett_nbap_SyncReportType_CellSyncReprtTDD,
    &ett_nbap_IntStdPhCellSyncInfo_CellSyncReprtTDD,
    &ett_nbap_CellSyncBurstMeasInfoList_CellSyncReprtTDD,
    &ett_nbap_CellSyncBurstMeasInfoItem_CellSyncReprtTDD,
    &ett_nbap_SEQUENCE_SIZE_1_16_OF_CellSyncBurstInfo_CellSyncReprtTDD,
    &ett_nbap_CellSyncBurstInfo_CellSyncReprtTDD,
    &ett_nbap_CellSyncBurstAvailable_CellSyncReprtTDD,
    &ett_nbap_SyncDLCodeIdsMeasInfoList_CellSyncReprtTDD,
    &ett_nbap_SyncDLCodeIdsMeasInfoItem_CellSyncReprtTDD,
    &ett_nbap_SyncDLCodeIdInfo_CellSyncReprtTDD,
    &ett_nbap_SyncDLCodeIdItem_CellSyncReprtTDD,
    &ett_nbap_SyncDLCodeIdAvailable_CellSyncReprtTDD,
    &ett_nbap_BearerRearrangementIndication,
    &ett_nbap_DCH_RearrangeList_Bearer_RearrangeInd,
    &ett_nbap_DCH_RearrangeItem_Bearer_RearrangeInd,
    &ett_nbap_DSCH_RearrangeList_Bearer_RearrangeInd,
    &ett_nbap_DSCH_RearrangeItem_Bearer_RearrangeInd,
    &ett_nbap_USCH_RearrangeList_Bearer_RearrangeInd,
    &ett_nbap_USCH_RearrangeItem_Bearer_RearrangeInd,
    &ett_nbap_HSDSCH_RearrangeList_Bearer_RearrangeInd,
    &ett_nbap_HSDSCH_RearrangeItem_Bearer_RearrangeInd,
    &ett_nbap_E_DCH_RearrangeList_Bearer_RearrangeInd,
    &ett_nbap_E_DCH_RearrangeItem_Bearer_RearrangeInd,
    &ett_nbap_RadioLinkActivationCommandFDD,
    &ett_nbap_DelayedActivationInformationList_RL_ActivationCmdFDD,
    &ett_nbap_DelayedActivationInformation_RL_ActivationCmdFDD,
    &ett_nbap_RadioLinkActivationCommandTDD,
    &ett_nbap_DelayedActivationInformationList_RL_ActivationCmdTDD,
    &ett_nbap_DelayedActivationInformation_RL_ActivationCmdTDD,
    &ett_nbap_RadioLinkParameterUpdateIndicationFDD,
    &ett_nbap_RadioLinkParameterUpdateIndicationTDD,
    &ett_nbap_MBMSNotificationUpdateCommand,
    &ett_nbap_CommonMeasurementReport,
    &ett_nbap_CommonMeasurementObjectType_CM_Rprt,
    &ett_nbap_Cell_CM_Rprt,
    &ett_nbap_RACH_CM_Rprt,
    &ett_nbap_PowerLocalCellGroup_CM_Rprt,
    &ett_nbap_CommonMeasurementTerminationRequest,
    &ett_nbap_CommonMeasurementFailureIndication,
    &ett_nbap_Active_Pattern_Sequence_Information,
    &ett_nbap_Transmission_Gap_Pattern_Sequence_Status_List,
    &ett_nbap_Transmission_Gap_Pattern_Sequence_Status_List_item,
    &ett_nbap_AllocationRetentionPriority,
    &ett_nbap_Angle_Of_Arrival_Value_LCR,
    &ett_nbap_Best_Cell_Portions_Value,
    &ett_nbap_Best_Cell_Portions_Item,
    &ett_nbap_Cause,
    &ett_nbap_CellSyncBurstTiming,
    &ett_nbap_CellSyncBurstTimingLCR,
    &ett_nbap_CommonChannelsCapacityConsumptionLaw,
    &ett_nbap_CommonChannelsCapacityConsumptionLaw_item,
    &ett_nbap_CommonMeasurementAccuracy,
    &ett_nbap_CommonMeasurementValue,
    &ett_nbap_CommonMeasurementValueInformation,
    &ett_nbap_CommonMeasurementAvailable,
    &ett_nbap_Common_PhysicalChannel_Status_Information,
    &ett_nbap_CommonTransportChannel_InformationResponse,
    &ett_nbap_Common_TransportChannel_Status_Information,
    &ett_nbap_CriticalityDiagnostics,
    &ett_nbap_CriticalityDiagnostics_IE_List,
    &ett_nbap_CriticalityDiagnostics_IE_List_item,
    &ett_nbap_DCH_FDD_Information,
    &ett_nbap_DCH_FDD_InformationItem,
    &ett_nbap_DCH_Specific_FDD_InformationList,
    &ett_nbap_DCH_Specific_FDD_Item,
    &ett_nbap_DCH_InformationResponse,
    &ett_nbap_DCH_InformationResponseItem,
    &ett_nbap_DCH_TDD_Information,
    &ett_nbap_DCH_TDD_InformationItem,
    &ett_nbap_DCH_Specific_TDD_InformationList,
    &ett_nbap_DCH_Specific_TDD_Item,
    &ett_nbap_FDD_DCHs_to_Modify,
    &ett_nbap_FDD_DCHs_to_ModifyItem,
    &ett_nbap_DCH_ModifySpecificInformation_FDD,
    &ett_nbap_DCH_ModifySpecificItem_FDD,
    &ett_nbap_TDD_DCHs_to_Modify,
    &ett_nbap_DCH_ModifyItem_TDD,
    &ett_nbap_DCH_ModifySpecificInformation_TDD,
    &ett_nbap_DCH_ModifySpecificItem_TDD,
    &ett_nbap_DedicatedChannelsCapacityConsumptionLaw,
    &ett_nbap_DedicatedChannelsCapacityConsumptionLaw_item,
    &ett_nbap_DedicatedMeasurementValue,
    &ett_nbap_DedicatedMeasurementValueInformation,
    &ett_nbap_DedicatedMeasurementAvailable,
    &ett_nbap_DelayedActivation,
    &ett_nbap_DelayedActivationUpdate,
    &ett_nbap_Activate_Info,
    &ett_nbap_Deactivate_Info,
    &ett_nbap_Execution_Type,
    &ett_nbap_DGPSCorrections,
    &ett_nbap_DGPSThresholds,
    &ett_nbap_DL_Timeslot_Information,
    &ett_nbap_DL_Timeslot_InformationItem,
    &ett_nbap_DL_TimeslotLCR_Information,
    &ett_nbap_DL_TimeslotLCR_InformationItem,
    &ett_nbap_DL_PowerBalancing_Information,
    &ett_nbap_DL_ReferencePowerInformationList,
    &ett_nbap_DL_ReferencePowerInformationItem,
    &ett_nbap_DL_TimeslotISCPInfo,
    &ett_nbap_DL_TimeslotISCPInfoItem,
    &ett_nbap_DL_TimeslotISCPInfoLCR,
    &ett_nbap_DL_TimeslotISCPInfoItemLCR,
    &ett_nbap_DSCH_InformationResponse,
    &ett_nbap_DSCH_InformationResponseItem,
    &ett_nbap_DSCH_TDD_Information,
    &ett_nbap_DSCH_TDD_InformationItem,
    &ett_nbap_E_AGCH_FDD_Code_Information,
    &ett_nbap_E_AGCH_FDD_Code_List,
    &ett_nbap_E_DCH_FDD_DL_Control_Channel_Information,
    &ett_nbap_E_DCH_FDD_Information,
    &ett_nbap_E_DCH_FDD_Information_Response,
    &ett_nbap_E_DCH_FDD_Information_to_Modify,
    &ett_nbap_E_DCH_Grant_Type_Information,
    &ett_nbap_E_DCH_LogicalChannelInformation,
    &ett_nbap_E_DCH_LogicalChannelInformationItem,
    &ett_nbap_E_DCH_MACdPDU_SizeList,
    &ett_nbap_E_DCH_MACdPDU_SizeListItem,
    &ett_nbap_E_DCH_LogicalChannelToModify,
    &ett_nbap_E_DCH_LogicalChannelToModifyItem,
    &ett_nbap_E_DCH_MACdPDU_SizeToModifyList,
    &ett_nbap_E_DCH_LogicalChannelToDelete,
    &ett_nbap_E_DCH_LogicalChannelToDeleteItem,
    &ett_nbap_E_DCH_MACdFlows_Information,
    &ett_nbap_E_DCH_MACdFlow_Specific_InfoList,
    &ett_nbap_E_DCH_MACdFlow_Specific_InfoItem,
    &ett_nbap_E_DCH_MACdFlow_Specific_InformationResp,
    &ett_nbap_E_DCH_MACdFlow_Specific_InformationResp_Item,
    &ett_nbap_E_DCH_MACdFlow_Specific_InfoList_to_Modify,
    &ett_nbap_E_DCH_MACdFlow_Specific_InfoItem_to_Modify,
    &ett_nbap_E_DCH_MACdFlows_to_Delete,
    &ett_nbap_E_DCH_MACdFlow_to_Delete_Item,
    &ett_nbap_E_DCH_Non_Scheduled_Transmission_Grant_Items,
    &ett_nbap_E_RGCH_E_HICH_FDD_Code_Information,
    &ett_nbap_E_RGCH_E_HICH_FDD_Code_List,
    &ett_nbap_E_TFCS_Information,
    &ett_nbap_E_DCHProvidedBitRate,
    &ett_nbap_E_DCHProvidedBitRate_Item,
    &ett_nbap_FDD_DL_CodeInformation,
    &ett_nbap_FDD_DL_CodeInformationItem,
    &ett_nbap_GPS_Almanac,
    &ett_nbap_GPS_Ionospheric_Model,
    &ett_nbap_GPS_Information,
    &ett_nbap_GPS_RealTime_Integrity,
    &ett_nbap_GPSBadSat_Info_RealTime_Integrity,
    &ett_nbap_GPS_NavigationModel_and_TimeRecovery,
    &ett_nbap_GPS_NavandRecovery_Item,
    &ett_nbap_GPS_RX_POS,
    &ett_nbap_GPS_UTC_Model,
    &ett_nbap_HARQ_MemoryPartitioning,
    &ett_nbap_HARQ_MemoryPartitioning_Implicit,
    &ett_nbap_HARQ_MemoryPartitioning_Explicit,
    &ett_nbap_HARQ_MemoryPartitioningList,
    &ett_nbap_HARQ_MemoryPartitioningItem,
    &ett_nbap_HS_DSCHProvidedBitRate,
    &ett_nbap_HS_DSCHProvidedBitRate_Item,
    &ett_nbap_HS_DSCHProvidedBitRateValueInformation_For_CellPortion,
    &ett_nbap_HS_DSCHProvidedBitRateValueInformation_For_CellPortion_Item,
    &ett_nbap_HS_DSCHRequiredPower,
    &ett_nbap_HS_DSCHRequiredPower_Item,
    &ett_nbap_HS_DSCHRequiredPowerPerUEInformation,
    &ett_nbap_HS_DSCHRequiredPowerPerUEInformation_Item,
    &ett_nbap_HS_DSCHRequiredPowerValueInformation_For_CellPortion,
    &ett_nbap_HS_DSCHRequiredPowerValueInformation_For_CellPortion_Item,
    &ett_nbap_HSDSCH_FDD_Information,
    &ett_nbap_HSDSCH_TDD_Information,
    &ett_nbap_HSDSCH_Information_to_Modify,
    &ett_nbap_HSDSCH_MACdFlow_Specific_InfoList_to_Modify,
    &ett_nbap_HSDSCH_MACdFlow_Specific_InfoItem_to_Modify,
    &ett_nbap_HSDSCH_Information_to_Modify_Unsynchronised,
    &ett_nbap_HSDSCH_FDD_Information_Response,
    &ett_nbap_HSDSCH_TDD_Information_Response,
    &ett_nbap_HSDSCH_MACdFlow_Specific_InformationResp,
    &ett_nbap_HSDSCH_MACdFlow_Specific_InformationResp_Item,
    &ett_nbap_HSDSCH_MACdFlows_Information,
    &ett_nbap_HSDSCH_MACdFlow_Specific_InfoList,
    &ett_nbap_HSDSCH_MACdFlow_Specific_InfoItem,
    &ett_nbap_HSDSCH_MACdFlows_to_Delete,
    &ett_nbap_HSDSCH_MACdFlows_to_Delete_Item,
    &ett_nbap_HSDSCH_Initial_Capacity_Allocation,
    &ett_nbap_HSDSCH_Initial_Capacity_AllocationItem,
    &ett_nbap_HSSCCH_Specific_InformationRespListFDD,
    &ett_nbap_HSSCCH_Codes,
    &ett_nbap_HSSCCH_Specific_InformationRespListTDD,
    &ett_nbap_HSSCCH_Specific_InformationRespItemTDD,
    &ett_nbap_HSSCCH_Specific_InformationRespListTDDLCR,
    &ett_nbap_HSSCCH_Specific_InformationRespItemTDDLCR,
    &ett_nbap_HSSICH_Info,
    &ett_nbap_HSSICH_InfoLCR,
    &ett_nbap_HS_SICH_Reception_Quality_Value,
    &ett_nbap_HS_PDSCH_FDD_Code_Information,
    &ett_nbap_HS_SCCH_FDD_Code_Information,
    &ett_nbap_HS_SCCH_FDD_Code_List,
    &ett_nbap_HSDSCH_FDD_Update_Information,
    &ett_nbap_HSDSCH_TDD_Update_Information,
    &ett_nbap_InformationReportCharacteristics,
    &ett_nbap_InformationReportCharacteristicsType_ReportPeriodicity,
    &ett_nbap_InformationReportCharacteristicsType_OnModification,
    &ett_nbap_InformationThresholds,
    &ett_nbap_InformationType,
    &ett_nbap_IPDL_FDD_Parameters,
    &ett_nbap_IPDL_TDD_Parameters,
    &ett_nbap_IPDL_TDD_Parameters_LCR,
    &ett_nbap_BurstModeParams,
    &ett_nbap_MACdPDU_Size_Indexlist,
    &ett_nbap_MACdPDU_Size_IndexItem,
    &ett_nbap_MACdPDU_Size_Indexlist_to_Modify,
    &ett_nbap_MACdPDU_Size_IndexItem_to_Modify,
    &ett_nbap_MessageStructure,
    &ett_nbap_MessageStructure_item,
    &ett_nbap_MidambleShiftAndBurstType,
    &ett_nbap_T_type1,
    &ett_nbap_T_midambleAllocationMode,
    &ett_nbap_T_type2,
    &ett_nbap_T_midambleAllocationMode1,
    &ett_nbap_T_type3,
    &ett_nbap_T_midambleAllocationMode2,
    &ett_nbap_MidambleShiftLCR,
    &ett_nbap_ModifyPriorityQueue,
    &ett_nbap_NeighbouringCellMeasurementInformation,
    &ett_nbap_NeighbouringCellMeasurementInformation_item,
    &ett_nbap_NeighbouringFDDCellMeasurementInformation,
    &ett_nbap_NeighbouringTDDCellMeasurementInformation,
    &ett_nbap_NeighbouringTDDCellMeasurementInformationLCR,
    &ett_nbap_NI_Information,
    &ett_nbap_PreambleSignatures,
    &ett_nbap_PriorityQueue_InfoList,
    &ett_nbap_PriorityQueue_InfoItem,
    &ett_nbap_PriorityQueue_InfoList_to_Modify,
    &ett_nbap_PriorityQueue_InfoItem_to_Add,
    &ett_nbap_PriorityQueue_InfoItem_to_Modify,
    &ett_nbap_PriorityQueue_InfoList_to_Modify_Unsynchronised,
    &ett_nbap_PriorityQueue_InfoItem_to_Modify_Unsynchronised,
    &ett_nbap_RACH_SubChannelNumbers,
    &ett_nbap_RL_Specific_DCH_Info,
    &ett_nbap_RL_Specific_DCH_Info_Item,
    &ett_nbap_RL_Specific_E_DCH_Info,
    &ett_nbap_RL_Specific_E_DCH_Info_Item,
    &ett_nbap_Reference_E_TFCI_Information,
    &ett_nbap_Reference_E_TFCI_Information_Item,
    &ett_nbap_ReportCharacteristics,
    &ett_nbap_ReportCharacteristicsType_EventA,
    &ett_nbap_ReportCharacteristicsType_EventB,
    &ett_nbap_ReportCharacteristicsType_EventC,
    &ett_nbap_ReportCharacteristicsType_EventD,
    &ett_nbap_ReportCharacteristicsType_EventE,
    &ett_nbap_ReportCharacteristicsType_EventF,
    &ett_nbap_ReportCharacteristicsType_OnModification,
    &ett_nbap_ReportCharacteristicsType_MeasurementIncreaseDecreaseThreshold,
    &ett_nbap_ReportCharacteristicsType_MeasurementThreshold,
    &ett_nbap_ReportCharacteristicsType_ScaledMeasurementChangeTime,
    &ett_nbap_ReportCharacteristicsType_ScaledMeasurementHysteresisTime,
    &ett_nbap_ReportCharacteristicsType_ReportPeriodicity,
    &ett_nbap_Received_total_wide_band_power_For_CellPortion_Value,
    &ett_nbap_Received_total_wide_band_power_For_CellPortion_Value_Item,
    &ett_nbap_RequestedDataValueInformation,
    &ett_nbap_InformationAvailable,
    &ett_nbap_RequestedDataValue,
    &ett_nbap_SAT_Info_Almanac,
    &ett_nbap_SAT_Info_Almanac_Item,
    &ett_nbap_SAT_Info_Almanac_ExtList,
    &ett_nbap_SAT_Info_Almanac_ExtItem,
    &ett_nbap_SAT_Info_DGPSCorrections,
    &ett_nbap_SAT_Info_DGPSCorrections_Item,
    &ett_nbap_SATInfo_RealTime_Integrity,
    &ett_nbap_SAT_Info_RealTime_Integrity_Item,
    &ett_nbap_Secondary_CPICH_Information_Change,
    &ett_nbap_Serving_E_DCH_RL_ID,
    &ett_nbap_Serving_E_DCH_RL_in_this_NodeB,
    &ett_nbap_SFNSFNMeasurementThresholdInformation,
    &ett_nbap_SFNSFNMeasurementValueInformation,
    &ett_nbap_T_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation,
    &ett_nbap_T_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item,
    &ett_nbap_T_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation,
    &ett_nbap_T_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item,
    &ett_nbap_SFNSFNTimeStampInformation,
    &ett_nbap_SFNSFNTimeStamp_TDD,
    &ett_nbap_SFNSFNValue,
    &ett_nbap_SynchronisationReportCharacteristics,
    &ett_nbap_SynchronisationReportCharactThreExc,
    &ett_nbap_SynchronisationReportCharactThreInfoItem,
    &ett_nbap_SEQUENCE_SIZE_1_16_OF_SynchronisationReportCharactCellSyncBurstInfoItem,
    &ett_nbap_SynchronisationReportCharactCellSyncBurstInfoItem,
    &ett_nbap_SyncDLCodeIdThreInfoLCR,
    &ett_nbap_SyncDLCodeIdThreInfoList,
    &ett_nbap_SyncDLCodeInfoListLCR,
    &ett_nbap_SyncDLCodeInfoItemLCR,
    &ett_nbap_TDD_ChannelisationCodeLCR,
    &ett_nbap_TDD_DL_Code_Information,
    &ett_nbap_TDD_DL_Code_InformationItem,
    &ett_nbap_TDD_DL_Code_LCR_Information,
    &ett_nbap_TDD_DL_Code_LCR_InformationItem,
    &ett_nbap_TDD_DL_DPCH_TimeSlotFormat_LCR,
    &ett_nbap_TDD_DPCHOffset,
    &ett_nbap_TransportFormatCombination_Beta,
    &ett_nbap_T_signalledGainFactors,
    &ett_nbap_T_gainFactor,
    &ett_nbap_T_fdd,
    &ett_nbap_TDD_UL_Code_Information,
    &ett_nbap_TDD_UL_Code_InformationItem,
    &ett_nbap_TDD_UL_Code_LCR_Information,
    &ett_nbap_TDD_UL_Code_LCR_InformationItem,
    &ett_nbap_TDD_UL_DPCH_TimeSlotFormat_LCR,
    &ett_nbap_TFCI_SignallingMode,
    &ett_nbap_TimingAdjustmentValue,
    &ett_nbap_TimingAdjustmentValueLCR,
    &ett_nbap_TnlQos,
    &ett_nbap_Transmission_Gap_Pattern_Sequence_Information,
    &ett_nbap_Transmission_Gap_Pattern_Sequence_Information_item,
    &ett_nbap_TransmittedCarrierPowerOfAllCodesNotUsedForHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHOrE_HICHTransmissionCellPortionValue,
    &ett_nbap_TransmittedCarrierPowerOfAllCodesNotUsedForHS_PDSCH_HS_SCCH_E_AGCH_E_RGCHOrE_HICHTransmissionCellPortionValue_Item,
    &ett_nbap_Transmitted_Carrier_Power_For_CellPortion_Value,
    &ett_nbap_Transmitted_Carrier_Power_For_CellPortion_Value_Item,
    &ett_nbap_TFCS,
    &ett_nbap_T_tFCSvalues,
    &ett_nbap_TFCS_TFCSList,
    &ett_nbap_TFCS_TFCSList_item,
    &ett_nbap_TFCS_CTFC,
    &ett_nbap_TransportFormatSet,
    &ett_nbap_TransportFormatSet_DynamicPartList,
    &ett_nbap_TransportFormatSet_DynamicPartList_item,
    &ett_nbap_TDD_TransportFormatSet_ModeDP,
    &ett_nbap_TransmissionTimeIntervalInformation,
    &ett_nbap_TransmissionTimeIntervalInformation_item,
    &ett_nbap_TransportFormatSet_Semi_staticPart,
    &ett_nbap_TransportFormatSet_ModeDP,
    &ett_nbap_TransportFormatSet_ModeSSP,
    &ett_nbap_TUTRANGPS,
    &ett_nbap_TUTRANGPSMeasurementThresholdInformation,
    &ett_nbap_TUTRANGPSMeasurementValueInformation,
    &ett_nbap_UC_Id,
    &ett_nbap_UE_Capability_Information,
    &ett_nbap_UL_Timeslot_Information,
    &ett_nbap_UL_Timeslot_InformationItem,
    &ett_nbap_UL_TimeslotLCR_Information,
    &ett_nbap_UL_TimeslotLCR_InformationItem,
    &ett_nbap_UL_ScramblingCode,
    &ett_nbap_UL_Synchronisation_Parameters_LCR,
    &ett_nbap_UL_TimeSlot_ISCP_Info,
    &ett_nbap_UL_TimeSlot_ISCP_InfoItem,
    &ett_nbap_UL_TimeSlot_ISCP_LCR_Info,
    &ett_nbap_UL_TimeSlot_ISCP_LCR_InfoItem,
    &ett_nbap_USCH_Information,
    &ett_nbap_USCH_InformationItem,
    &ett_nbap_USCH_InformationResponse,
    &ett_nbap_USCH_InformationResponseItem,
    &ett_nbap_PrivateIE_ID,
    &ett_nbap_ProcedureID,
    &ett_nbap_TransactionID,
    &ett_nbap_ProtocolIE_Container,
    &ett_nbap_ProtocolIE_Field,
    &ett_nbap_ProtocolIE_ContainerPair,
    &ett_nbap_ProtocolIE_FieldPair,
    &ett_nbap_ProtocolIE_ContainerList,
    &ett_nbap_ProtocolIE_ContainerPairList,
    &ett_nbap_ProtocolExtensionContainer,
    &ett_nbap_ProtocolExtensionField,
    &ett_nbap_PrivateIE_Container,
    &ett_nbap_PrivateIE_Field,

/*--- End of included file: packet-nbap-ettarr.c ---*/
#line 2934 "packet-nbap-template.c"
  };


  /* Register protocol */
  proto_nbap = proto_register_protocol(PNAME, PSNAME, PFNAME);
  /* Register fields and subtrees */
  proto_register_field_array(proto_nbap, hf, array_length(hf));
  proto_register_subtree_array(ett, array_length(ett));

 
  register_dissector("nbap", dissect_nbap, proto_nbap);


}


/*--- proto_reg_handoff_nbap ---------------------------------------*/
void
proto_reg_handoff_nbap(void)
{

	nbap_handle = find_dissector("nbap");

}


