/* Do not modify this file.                                                   */
/* It is created automatically by the ASN.1 to Wireshark dissector compiler   */
/* .\packet-rnsap.c                                                           */
/* ../../tools/asn2wrs.py -e -F -p rnsap -c rnsap.cnf -s packet-rnsap-template rnsap.asn */

/* Input file: packet-rnsap-template.c */

#line 1 "packet-rnsap-template.c"
/* packet-rnsap.c
 * Routines for dissecting Universal Mobile Telecommunications System (UMTS);
 * UTRAN Iur interface Radio Network Subsystem
 * Application Part (RNSAP) signalling
 * (3GPP TS 25.423 version 6.7.0 Release 6) packet dissection
 * Copyright 2005 - 2006, Anders Broman <anders.broman@ericsson.com>
 *
 * $Id: packet-rnsap.c 18921 2006-08-16 05:58:39Z kukosa $
 *
 * Wireshark - Network traffic analyzer
 * By Gerald Combs <gerald@wireshark.org>
 * Copyright 1998 Gerald Combs
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Ref: 3GPP TS 25.423 version 6.7.0 Release 6
 */

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <glib.h>
#include <epan/packet.h>
#include <epan/conversation.h>

#include <stdio.h>
#include <string.h>

#include "packet-ber.h"
#include "packet-per.h"
#include "packet-rnsap.h"
#include "packet-umts_rrc.h"
/*#include "packet-umts_rrc.h"*/

#define PNAME  "UTRAN Iur interface Radio Network Subsystem Application Part"
#define PSNAME "RNSAP"
#define PFNAME "rnsap"

#define SCCP_SSN_RNSAP 143

#define RNSAP_FDD 1
/* Procedure codes */
#define	RNSAP_ID_COMMONTRANSPORTCHANNELRESOURCESINITIALISATION			 0
#define	RNSAP_ID_COMMONTRANSPORTCHANNELRESOURCESRELEASE					 1
#define	RNSAP_ID_COMPRESSEDMODECOMMAND									 2
#define	RNSAP_ID_DOWNLINKPOWERCONTROL									 3
#define	RNSAP_ID_DOWNLINKPOWERTIMESLOTCONTROL							 4
#define	RNSAP_ID_DOWNLINKSIGNALLINGTRANSFER								 5
#define	RNSAP_ID_ERRORINDICATION										 6
#define	RNSAP_ID_DEDICATEDMEASUREMENTFAILURE							 7
#define	RNSAP_ID_DEDICATEDMEASUREMENTINITIATION							 8
#define	RNSAP_ID_DEDICATEDMEASUREMENTREPORTING							 9
#define	RNSAP_ID_DEDICATEDMEASUREMENTTERMINATION						 10
#define	RNSAP_ID_PAGING													 11
#define	RNSAP_ID_PHYSICALCHANNELRECONFIGURATION							 12
#define	RNSAP_ID_PRIVATEMESSAGE											 13
#define	RNSAP_ID_RADIOLINKADDITION										 14
#define	RNSAP_ID_RADIOLINKCONGESTION									 34
#define	RNSAP_ID_RADIOLINKDELETION										 15
#define	RNSAP_ID_RADIOLINKFAILURE										 16
#define	RNSAP_ID_RADIOLINKPREEMPTION									 17
#define	RNSAP_ID_RADIOLINKRESTORATION									 18
#define	RNSAP_ID_RADIOLINKSETUP											 19
#define	RNSAP_ID_RELOCATIONCOMMIT										 20
#define	RNSAP_ID_SYNCHRONISEDRADIOLINKRECONFIGURATIONCANCELLATION		 21
#define	RNSAP_ID_SYNCHRONISEDRADIOLINKRECONFIGURATIONCOMMIT				 22
#define	RNSAP_ID_SYNCHRONISEDRADIOLINKRECONFIGURATIONPREPARATION		 23
#define	RNSAP_ID_UNSYNCHRONISEDRADIOLINKRECONFIGURATION					 24
#define	RNSAP_ID_UPLINKSIGNALLINGTRANSFER								 25
#define	RNSAP_ID_COMMONMEASUREMENTFAILURE								 26
#define	RNSAP_ID_COMMONMEASUREMENTINITIATION							 27
#define	RNSAP_ID_COMMONMEASUREMENTREPORTING								 28
#define	RNSAP_ID_COMMONMEASUREMENTTERMINATION							 29
#define	RNSAP_ID_INFORMATIONEXCHANGEFAILURE								 30
#define	RNSAP_ID_INFORMATIONEXCHANGEINITIATION							 31
#define	RNSAP_ID_INFORMATIONREPORTING									 32
#define	RNSAP_ID_INFORMATIONEXCHANGETERMINATION							 33
#define	RNSAP_ID_RESET													 35
#define	RNSAP_ID_RADIOLINKACTIVATION 									 36
#define	RNSAP_ID_GERANUPLINKSIGNALLINGTRANSFER							 37
#define	RNSAP_ID_RADIOLINKPARAMETERUPDATE								 38
#define	RNSAP_ID_UEMEASUREMENTFAILURE									 39
#define	RNSAP_ID_UEMEASUREMENTINITIATION								 40
#define	RNSAP_ID_UEMEASUREMENTREPORTING									 41
#define	RNSAP_ID_UEMEASUREMENTTERMINATION								 42
#define	RNSAP_ID_IURDEACTIVATETRACE										 43
#define	RNSAP_ID_IURINVOKETRACE											 44
#define	RNSAP_ID_MBMSATTACH												 45
#define	RNSAP_ID_MBMSDETACH												 46
#define	RNSAP_ID_DIRECTINFORMATIONTRANSFER								 48

#define	RNSAP_ID_ALLOWEDQUEUINGTIME														 4
#define	RNSAP_ID_ALLOWED_RATE_INFORMATION												 42
#define	RNSAP_ID_ANTENNACOLOCATIONINDICATOR												 309
#define	RNSAP_ID_BINDINGID																 5
#define	RNSAP_ID_C_ID																	 6
#define	RNSAP_ID_C_RNTI																	 7
#define	RNSAP_ID_CELL_CAPACITY_CLASS_VALUE												 303
#define	RNSAP_ID_CFN																	 8
#define	RNSAP_ID_CN_CS_DOMAINIDENTIFIER													 9
#define	RNSAP_ID_CN_PS_DOMAINIDENTIFIER													 10
#define	RNSAP_ID_CAUSE																	 11
#define	RNSAP_ID_COVERAGEINDICATOR														 310
#define	RNSAP_ID_CRITICALITYDIAGNOSTICS													 20
#define	RNSAP_ID_CONTEXTINFOITEM_RESET													 211
#define	RNSAP_ID_CONTEXTGROUPINFOITEM_RESET												 515
#define	RNSAP_ID_D_RNTI																	 21
#define	RNSAP_ID_D_RNTI_RELEASEINDICATION												 22
#define	RNSAP_ID_DCHS_TO_ADD_FDD														 26
#define	RNSAP_ID_DCHS_TO_ADD_TDD														 27
#define	RNSAP_ID_DCH_DELETELIST_RL_RECONFPREPFDD										 30
#define	RNSAP_ID_DCH_DELETELIST_RL_RECONFPREPTDD										 31
#define	RNSAP_ID_DCH_DELETELIST_RL_RECONFRQSTFDD										 32
#define	RNSAP_ID_DCH_DELETELIST_RL_RECONFRQSTTDD										 33
#define	RNSAP_ID_DCH_FDD_INFORMATION													 34
#define	RNSAP_ID_DCH_TDD_INFORMATION													 35
#define	RNSAP_ID_FDD_DCHS_TO_MODIFY														 39
#define	RNSAP_ID_TDD_DCHS_TO_MODIFY														 40
#define	RNSAP_ID_DCH_INFORMATIONRESPONSE												 43
#define	RNSAP_ID_DCH_RATE_INFORMATIONITEM_RL_CONGESTIND									 38
#define	RNSAP_ID_DL_CCTRCH_INFORMATIONADDITEM_RL_RECONFPREPTDD							 44
#define	RNSAP_ID_DL_CCTRCH_INFORMATIONLISTIE_RL_RECONFREADYTDD							 45
#define	RNSAP_ID_DL_CCTRCH_INFORMATIONDELETEITEM_RL_RECONFRQSTTDD						 46
#define	RNSAP_ID_DL_CCTRCH_INFORMATIONITEM_RL_SETUPRQSTTDD								 47
#define	RNSAP_ID_DL_CCTRCH_INFORMATIONLISTIE_PHYCHRECONFRQSTTDD							 48
#define	RNSAP_ID_DL_CCTRCH_INFORMATIONLISTIE_RL_ADDITIONRSPTDD							 49
#define	RNSAP_ID_DL_CCTRCH_INFORMATIONLISTIE_RL_SETUPRSPTDD								 50
#define	RNSAP_ID_DL_CCTRCH_INFORMATIONADDLIST_RL_RECONFPREPTDD							 51
#define	RNSAP_ID_DL_CCTRCH_INFORMATIONDELETELIST_RL_RECONFRQSTTDD						 52
#define	RNSAP_ID_DL_CCTRCH_INFORMATIONLIST_RL_SETUPRQSTTDD								 53
#define	RNSAP_ID_FDD_DL_CODEINFORMATION													 54
#define	RNSAP_ID_DL_DPCH_INFORMATION_RL_RECONFPREPFDD									 59
#define	RNSAP_ID_DL_DPCH_INFORMATION_RL_SETUPRQSTFDD									 60
#define	RNSAP_ID_DL_DPCH_INFORMATION_RL_RECONFRQSTFDD									 61
#define	RNSAP_ID_DL_DPCH_INFORMATIONITEM_PHYCHRECONFRQSTTDD								 62
#define	RNSAP_ID_DL_DPCH_INFORMATIONITEM_RL_ADDITIONRSPTDD								 63
#define	RNSAP_ID_DL_DPCH_INFORMATIONITEM_RL_SETUPRSPTDD									 64
#define	RNSAP_ID_DL_DPCH_TIMINGADJUSTMENT												 278
#define	RNSAP_ID_DLREFERENCEPOWER														 67
#define	RNSAP_ID_DLREFERENCEPOWERLIST_DL_PC_RQST										 68
#define	RNSAP_ID_DL_REFERENCEPOWERINFORMATION_DL_PC_RQST								 69
#define	RNSAP_ID_DPC_MODE																 12
#define	RNSAP_ID_DRXCYCLELENGTHCOEFFICIENT												 70
#define	RNSAP_ID_DEDICATEDMEASUREMENTOBJECTTYPE_DM_FAIL_IND								 470
#define	RNSAP_ID_DEDICATEDMEASUREMENTOBJECTTYPE_DM_FAIL									 471
#define	RNSAP_ID_DEDICATEDMEASUREMENTOBJECTTYPE_DM_RPRT									 71
#define	RNSAP_ID_DEDICATEDMEASUREMENTOBJECTTYPE_DM_RQST									 72
#define	RNSAP_ID_DEDICATEDMEASUREMENTOBJECTTYPE_DM_RSP									 73
#define	RNSAP_ID_DEDICATEDMEASUREMENTTYPE												 74
#define	RNSAP_ID_FACH_INFOFORUESELECTEDS_CCPCH_CTCH_RESOURCERSPFDD						 82
#define	RNSAP_ID_FACH_INFOFORUESELECTEDS_CCPCH_CTCH_RESOURCERSPTDD						 83
#define	RNSAP_ID_GUARANTEED_RATE_INFORMATION											 41
#define	RNSAP_ID_IMSI																	 84
#define	RNSAP_ID_HCS_PRIO																 311
#define	RNSAP_ID_L3_INFORMATION															 85
#define	RNSAP_ID_ADJUSTMENTPERIOD														 90
#define	RNSAP_ID_MAXADJUSTMENTSTEP														 91
#define	RNSAP_ID_MEASUREMENTFILTERCOEFFICIENT											 92
#define	RNSAP_ID_MESSAGESTRUCTURE														 57
#define	RNSAP_ID_MEASUREMENTID															 93
#define	RNSAP_ID_NEIGHBOURING_GSM_CELLINFORMATION										 13
#define	RNSAP_ID_NEIGHBOURING_UMTS_CELLINFORMATIONITEM									 95
#define	RNSAP_ID_NRT_LOAD_INFORMATION_VALUE												 305
#define	RNSAP_ID_NRT_LOAD_INFORMATION_VALUE_INCRDECRTHRES								 306
#define	RNSAP_ID_PAGINGAREA_PAGINGRQST													 102
#define	RNSAP_ID_FACH_FLOWCONTROLINFORMATION											 103
#define	RNSAP_ID_PARTIALREPORTINGINDICATOR												 472
#define	RNSAP_ID_PERMANENT_NAS_UE_IDENTITY												 17
#define	RNSAP_ID_POWERADJUSTMENTTYPE													 107
#define	RNSAP_ID_RANAP_RELOCATIONINFORMATION											 109
#define	RNSAP_ID_RL_INFORMATION_PHYCHRECONFRQSTFDD										 110
#define	RNSAP_ID_RL_INFORMATION_PHYCHRECONFRQSTTDD										 111
#define	RNSAP_ID_RL_INFORMATION_RL_ADDITIONRQSTFDD										 112
#define	RNSAP_ID_RL_INFORMATION_RL_ADDITIONRQSTTDD										 113
#define	RNSAP_ID_RL_INFORMATION_RL_DELETIONRQST											 114
#define	RNSAP_ID_RL_INFORMATION_RL_FAILUREIND											 115
#define	RNSAP_ID_RL_INFORMATION_RL_RECONFPREPFDD										 116
#define	RNSAP_ID_RL_INFORMATION_RL_RESTOREIND											 117
#define	RNSAP_ID_RL_INFORMATION_RL_SETUPRQSTFDD											 118
#define	RNSAP_ID_RL_INFORMATION_RL_SETUPRQSTTDD											 119
#define	RNSAP_ID_RL_INFORMATIONITEM_RL_CONGESTIND										 55
#define	RNSAP_ID_RL_INFORMATIONITEM_DM_RPRT												 120
#define	RNSAP_ID_RL_INFORMATIONITEM_DM_RQST												 121
#define	RNSAP_ID_RL_INFORMATIONITEM_DM_RSP												 122
#define	RNSAP_ID_RL_INFORMATIONITEM_RL_PREEMPTREQUIREDIND								 2
#define	RNSAP_ID_RL_INFORMATIONITEM_RL_SETUPRQSTFDD										 123
#define	RNSAP_ID_RL_INFORMATIONLIST_RL_CONGESTIND										 56
#define	RNSAP_ID_RL_INFORMATIONLIST_RL_ADDITIONRQSTFDD									 124
#define	RNSAP_ID_RL_INFORMATIONLIST_RL_DELETIONRQST										 125
#define	RNSAP_ID_RL_INFORMATIONLIST_RL_PREEMPTREQUIREDIND								 1
#define	RNSAP_ID_RL_INFORMATIONLIST_RL_RECONFPREPFDD									 126
#define	RNSAP_ID_RL_INFORMATIONRESPONSE_RL_ADDITIONRSPTDD								 127
#define	RNSAP_ID_RL_INFORMATIONRESPONSE_RL_RECONFREADYTDD								 128
#define	RNSAP_ID_RL_INFORMATIONRESPONSE_RL_SETUPRSPTDD									 129
#define	RNSAP_ID_RL_INFORMATIONRESPONSEITEM_RL_ADDITIONRSPFDD							 130
#define	RNSAP_ID_RL_INFORMATIONRESPONSEITEM_RL_RECONFREADYFDD							 131
#define	RNSAP_ID_RL_INFORMATIONRESPONSEITEM_RL_RECONFRSPFDD								 132
#define	RNSAP_ID_RL_INFORMATIONRESPONSEITEM_RL_SETUPRSPFDD								 133
#define	RNSAP_ID_RL_INFORMATIONRESPONSELIST_RL_ADDITIONRSPFDD							 134
#define	RNSAP_ID_RL_INFORMATIONRESPONSELIST_RL_RECONFREADYFDD							 135
#define	RNSAP_ID_RL_INFORMATIONRESPONSELIST_RL_RECONFRSPFDD								 136
#define	RNSAP_ID_RL_INFORMATIONRESPONSE_RL_RECONFRSPTDD									 28
#define	RNSAP_ID_RL_INFORMATIONRESPONSELIST_RL_SETUPRSPFDD								 137
#define	RNSAP_ID_RL_RECONFIGURATIONFAILURE_RL_RECONFFAIL								 141
#define	RNSAP_ID_RL_SET_INFORMATIONITEM_DM_RPRT											 143
#define	RNSAP_ID_RL_SET_INFORMATIONITEM_DM_RQST											 144
#define	RNSAP_ID_RL_SET_INFORMATIONITEM_DM_RSP											 145
#define	RNSAP_ID_RL_SET_INFORMATION_RL_FAILUREIND										 146
#define	RNSAP_ID_RL_SET_INFORMATION_RL_RESTOREIND										 147
#define	RNSAP_ID_RL_SET_SUCCESSFUL_INFORMATIONITEM_DM_FAIL								 473
#define	RNSAP_ID_RL_SET_UNSUCCESSFUL_INFORMATIONITEM_DM_FAIL							 474
#define	RNSAP_ID_RL_SET_UNSUCCESSFUL_INFORMATIONITEM_DM_FAIL_IND						 475
#define	RNSAP_ID_RL_SUCCESSFUL_INFORMATIONITEM_DM_FAIL									 476
#define	RNSAP_ID_RL_UNSUCCESSFUL_INFORMATIONITEM_DM_FAIL								 477
#define	RNSAP_ID_RL_UNSUCCESSFUL_INFORMATIONITEM_DM_FAIL_IND							 478
#define	RNSAP_ID_REPORTCHARACTERISTICS													 152
#define	RNSAP_ID_REPORTING_OBJECT_RL_FAILUREIND											 153
#define	RNSAP_ID_REPORING_OBJECT_RL_RESTOREIND											 154
#define	RNSAP_ID_RT_LOAD_VALUE															 307
#define	RNSAP_ID_RT_LOAD_VALUE_INCRDECRTHRES											 308
#define	RNSAP_ID_S_RNTI																	 155
#define	RNSAP_ID_RESETINDICATOR															 244
#define	RNSAP_ID_RNC_ID																	 245
#define	RNSAP_ID_SAI																	 156
#define	RNSAP_ID_SRNC_ID																 157
#define	RNSAP_ID_SUCCESSFULRL_INFORMATIONRESPONSE_RL_ADDITIONFAILUREFDD					 159
#define	RNSAP_ID_SUCCESSFULRL_INFORMATIONRESPONSE_RL_SETUPFAILUREFDD					 160
#define	RNSAP_ID_TRANSPORTBEARERID														 163
#define	RNSAP_ID_TRANSPORTBEARERREQUESTINDICATOR										 164
#define	RNSAP_ID_TRANSPORTLAYERADDRESS													 165
#define	RNSAP_ID_TYPEOFERROR															 140
#define	RNSAP_ID_UC_ID																	 166
#define	RNSAP_ID_UL_CCTRCH_ADDINFORMATION_RL_RECONFPREPTDD								 167
#define	RNSAP_ID_UL_CCTRCH_INFORMATIONADDLIST_RL_RECONFPREPTDD							 169
#define	RNSAP_ID_UL_CCTRCH_INFORMATIONITEM_RL_SETUPRQSTTDD								 171
#define	RNSAP_ID_UL_CCTRCH_INFORMATIONLIST_RL_SETUPRQSTTDD								 172
#define	RNSAP_ID_UL_CCTRCH_INFORMATIONLISTIE_PHYCHRECONFRQSTTDD							 173
#define	RNSAP_ID_UL_CCTRCH_INFORMATIONLISTIE_RL_ADDITIONRSPTDD							 174
#define	RNSAP_ID_UL_CCTRCH_INFORMATIONLISTIE_RL_RECONFREADYTDD							 175
#define	RNSAP_ID_UL_CCTRCH_INFORMATIONLISTIE_RL_SETUPRSPTDD								 176
#define	RNSAP_ID_UL_DPCH_INFORMATION_RL_RECONFPREPFDD									 177
#define	RNSAP_ID_UL_DPCH_INFORMATION_RL_RECONFRQSTFDD									 178
#define	RNSAP_ID_UL_DPCH_INFORMATION_RL_SETUPRQSTFDD									 179
#define	RNSAP_ID_UL_DPCH_INFORMATIONITEM_PHYCHRECONFRQSTTDD								 180
#define	RNSAP_ID_UL_DPCH_INFORMATIONITEM_RL_ADDITIONRSPTDD								 181
#define	RNSAP_ID_UL_DPCH_INFORMATIONITEM_RL_SETUPRSPTDD									 182
#define	RNSAP_ID_UL_DPCH_INFORMATIONADDLISTIE_RL_RECONFREADYTDD							 183
#define	RNSAP_ID_UL_SIRTARGET															 184
#define	RNSAP_ID_URA_INFORMATION														 185
#define	RNSAP_ID_UNSUCCESSFULRL_INFORMATIONRESPONSE_RL_ADDITIONFAILUREFDD				 188
#define	RNSAP_ID_UNSUCCESSFULRL_INFORMATIONRESPONSE_RL_SETUPFAILUREFDD					 189
#define	RNSAP_ID_UNSUCCESSFULRL_INFORMATIONRESPONSE_RL_SETUPFAILURETDD					 190
#define	RNSAP_ID_ACTIVE_PATTERN_SEQUENCE_INFORMATION									 193
#define	RNSAP_ID_ADJUSTMENTRATIO														 194
#define	RNSAP_ID_CAUSELEVEL_RL_ADDITIONFAILUREFDD										 197
#define	RNSAP_ID_CAUSELEVEL_RL_ADDITIONFAILURETDD										 198
#define	RNSAP_ID_CAUSELEVEL_RL_RECONFFAILURE											 199
#define	RNSAP_ID_CAUSELEVEL_RL_SETUPFAILUREFDD											 200
#define	RNSAP_ID_CAUSELEVEL_RL_SETUPFAILURETDD											 201
#define	RNSAP_ID_DL_CCTRCH_INFORMATIONDELETEITEM_RL_RECONFPREPTDD						 205
#define	RNSAP_ID_DL_CCTRCH_INFORMATIONMODIFYITEM_RL_RECONFPREPTDD						 206
#define	RNSAP_ID_DL_CCTRCH_INFORMATIONMODIFYITEM_RL_RECONFRQSTTDD						 207
#define	RNSAP_ID_DL_CCTRCH_INFORMATIONDELETELIST_RL_RECONFPREPTDD						 208
#define	RNSAP_ID_DL_CCTRCH_INFORMATIONMODIFYLIST_RL_RECONFPREPTDD						 209
#define	RNSAP_ID_DL_CCTRCH_INFORMATIONMODIFYLIST_RL_RECONFRQSTTDD						 210
#define	RNSAP_ID_DL_DPCH_INFORMATIONADDLISTIE_RL_RECONFREADYTDD							 212
#define	RNSAP_ID_DL_DPCH_INFORMATIONDELETELISTIE_RL_RECONFREADYTDD						 213
#define	RNSAP_ID_DL_DPCH_INFORMATIONMODIFYLISTIE_RL_RECONFREADYTDD						 214
#define	RNSAP_ID_DSCHS_TO_ADD_TDD														 215
#define	RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_216											 216
#define	RNSAP_ID_DSCH_DELETELIST_RL_RECONFPREPTDD										 217
#define	RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_218											 218
#define	RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_219											 219
#define	RNSAP_ID_DSCH_INFORMATIONLISTIE_RL_ADDITIONRSPTDD								 220
#define	RNSAP_ID_DSCH_INFORMATIONLISTIES_RL_SETUPRSPTDD									 221
#define	RNSAP_ID_DSCH_TDD_INFORMATION													 222
#define	RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_223											 223
#define	RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_226											 226
#define	RNSAP_ID_DSCH_MODIFYLIST_RL_RECONFPREPTDD										 227
#define	RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_228											 228
#define	RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_324											 324
#define	RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_229											 229
#define	RNSAP_ID_DSCHTOBEADDEDORMODIFIEDLIST_RL_RECONFREADYTDD							 230
#define	RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_29											 29
#define	RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_225											 225
#define	RNSAP_ID_GA_CELL																 232
#define	RNSAP_ID_GA_CELLADDITIONALSHAPES												 3
#define	RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_246											 246
#define	RNSAP_ID_TRANSMISSION_GAP_PATTERN_SEQUENCE_INFORMATION							 255
#define	RNSAP_ID_UL_CCTRCH_DELETEINFORMATION_RL_RECONFPREPTDD							 256
#define	RNSAP_ID_UL_CCTRCH_MODIFYINFORMATION_RL_RECONFPREPTDD							 257
#define	RNSAP_ID_UL_CCTRCH_INFORMATIONMODIFYITEM_RL_RECONFRQSTTDD						 258
#define	RNSAP_ID_UL_CCTRCH_INFORMATIONDELETELIST_RL_RECONFPREPTDD						 259
#define	RNSAP_ID_UL_CCTRCH_INFORMATIONMODIFYLIST_RL_RECONFPREPTDD						 260
#define	RNSAP_ID_UL_CCTRCH_INFORMATIONMODIFYLIST_RL_RECONFRQSTTDD						 261
#define	RNSAP_ID_UL_CCTRCH_INFORMATIONDELETEITEM_RL_RECONFRQSTTDD						 262
#define	RNSAP_ID_UL_CCTRCH_INFORMATIONDELETELIST_RL_RECONFRQSTTDD						 263
#define	RNSAP_ID_UL_DPCH_INFORMATIONDELETELISTIE_RL_RECONFREADYTDD						 264
#define	RNSAP_ID_UL_DPCH_INFORMATIONMODIFYLISTIE_RL_RECONFREADYTDD						 265
#define	RNSAP_ID_UNSUCCESSFULRL_INFORMATIONRESPONSE_RL_ADDITIONFAILURETDD				 266
#define	RNSAP_ID_USCHS_TO_ADD															 267
#define	RNSAP_ID_USCH_DELETELIST_RL_RECONFPREPTDD										 268
#define	RNSAP_ID_USCH_INFORMATIONLISTIE_RL_ADDITIONRSPTDD								 269
#define	RNSAP_ID_USCH_INFORMATIONLISTIES_RL_SETUPRSPTDD									 270
#define	RNSAP_ID_USCH_INFORMATION														 271
#define	RNSAP_ID_USCH_MODIFYLIST_RL_RECONFPREPTDD										 272
#define	RNSAP_ID_USCHTOBEADDEDORMODIFIEDLIST_RL_RECONFREADYTDD							 273
#define	RNSAP_ID_DL_PHYSICAL_CHANNEL_INFORMATION_RL_SETUPRQSTTDD						 274
#define	RNSAP_ID_UL_PHYSICAL_CHANNEL_INFORMATION_RL_SETUPRQSTTDD						 275
#define	RNSAP_ID_CLOSEDLOOPMODE1_SUPPORTINDICATOR										 276
#define	RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_277											 277
#define	RNSAP_ID_STTD_SUPPORTINDICATOR													 279
#define	RNSAP_ID_CFNREPORTINGINDICATOR             										 14
#define	RNSAP_ID_CNORIGINATEDPAGE_PAGINGRQST											 23
#define	RNSAP_ID_INNERLOOPDLPCSTATUS													 24
#define	RNSAP_ID_PROPAGATIONDELAY														 25
#define	RNSAP_ID_RXTIMINGDEVIATIONFORTA													 36
#define	RNSAP_ID_TIMESLOT_ISCP															 37
#define	RNSAP_ID_CCTRCH_INFORMATIONITEM_RL_FAILUREIND									 15
#define	RNSAP_ID_CCTRCH_INFORMATIONITEM_RL_RESTOREIND									 16
#define	RNSAP_ID_COMMONMEASUREMENTACCURACY												 280
#define	RNSAP_ID_COMMONMEASUREMENTOBJECTTYPE_CM_RPRT									 281
#define	RNSAP_ID_COMMONMEASUREMENTOBJECTTYPE_CM_RQST									 282
#define	RNSAP_ID_COMMONMEASUREMENTOBJECTTYPE_CM_RSP										 283
#define	RNSAP_ID_COMMONMEASUREMENTTYPE													 284
#define	RNSAP_ID_CONGESTIONCAUSE 														 18
#define	RNSAP_ID_SFN																	 285
#define	RNSAP_ID_SFNREPORTINGINDICATOR													 286
#define	RNSAP_ID_INFORMATIONEXCHANGEID													 287
#define	RNSAP_ID_INFORMATIONEXCHANGEOBJECTTYPE_INFEX_RPRT								 288
#define	RNSAP_ID_INFORMATIONEXCHANGEOBJECTTYPE_INFEX_RQST								 289
#define	RNSAP_ID_INFORMATIONEXCHANGEOBJECTTYPE_INFEX_RSP								 290
#define	RNSAP_ID_INFORMATIONREPORTCHARACTERISTICS										 291
#define	RNSAP_ID_INFORMATIONTYPE														 292
#define	RNSAP_ID_NEIGHBOURING_LCR_TDD_CELLINFORMATION									 58
#define	RNSAP_ID_DL_TIMESLOT_ISCP_LCR_INFORMATION_RL_SETUPRQSTTDD						 65
#define	RNSAP_ID_RL_LCR_INFORMATIONRESPONSE_RL_SETUPRSPTDD								 66
#define	RNSAP_ID_UL_CCTRCH_LCR_INFORMATIONLISTIE_RL_SETUPRSPTDD							 75
#define	RNSAP_ID_UL_DPCH_LCR_INFORMATIONITEM_RL_SETUPRSPTDD								 76
#define	RNSAP_ID_DL_CCTRCH_LCR_INFORMATIONLISTIE_RL_SETUPRSPTDD							 77
#define	RNSAP_ID_DL_DPCH_LCR_INFORMATIONITEM_RL_SETUPRSPTDD								 78
#define	RNSAP_ID_DSCH_LCR_INFORMATIONLISTIES_RL_SETUPRSPTDD								 79
#define	RNSAP_ID_USCH_LCR_INFORMATIONLISTIES_RL_SETUPRSPTDD								 80
#define	RNSAP_ID_DL_TIMESLOT_ISCP_LCR_INFORMATION_RL_ADDITIONRQSTTDD					 81
#define	RNSAP_ID_RL_LCR_INFORMATIONRESPONSE_RL_ADDITIONRSPTDD							 86
#define	RNSAP_ID_UL_CCTRCH_LCR_INFORMATIONLISTIE_RL_ADDITIONRSPTDD						 87
#define	RNSAP_ID_UL_DPCH_LCR_INFORMATIONITEM_RL_ADDITIONRSPTDD							 88
#define	RNSAP_ID_DL_CCTRCH_LCR_INFORMATIONLISTIE_RL_ADDITIONRSPTDD						 89
#define	RNSAP_ID_DL_DPCH_LCR_INFORMATIONITEM_RL_ADDITIONRSPTDD							 94
#define	RNSAP_ID_DSCH_LCR_INFORMATIONLISTIES_RL_ADDITIONRSPTDD							 96
#define	RNSAP_ID_USCH_LCR_INFORMATIONLISTIES_RL_ADDITIONRSPTDD							 97
#define	RNSAP_ID_UL_DPCH_LCR_INFORMATIONADDLISTIE_RL_RECONFREADYTDD						 98
#define	RNSAP_ID_UL_TIMESLOT_LCR_INFORMATIONMODIFYLIST_RL_RECONFREADYTDD				 100
#define	RNSAP_ID_DL_DPCH_LCR_INFORMATIONADDLISTIE_RL_RECONFREADYTDD						 101
#define	RNSAP_ID_DL_TIMESLOT_LCR_INFORMATIONMODIFYLIST_RL_RECONFREADYTDD				 104
#define	RNSAP_ID_UL_TIMESLOT_LCR_INFORMATIONLIST_PHYCHRECONFRQSTTDD						 105
#define	RNSAP_ID_DL_TIMESLOT_LCR_INFORMATIONLIST_PHYCHRECONFRQSTTDD						 106
#define	RNSAP_ID_TIMESLOT_ISCP_LCR_LIST_DL_PC_RQST_TDD									 138
#define	RNSAP_ID_TSTD_SUPPORT_INDICATOR_RL_SETUPRQSTTDD									 139
#define	RNSAP_ID_RESTRICTIONSTATEINDICATOR												 142
#define	RNSAP_ID_LOAD_VALUE																 233
#define	RNSAP_ID_LOAD_VALUE_INCRDECRTHRES												 234
#define	RNSAP_ID_ONMODIFICATION															 235
#define	RNSAP_ID_RECEIVED_TOTAL_WIDEBAND_POWER_VALUE									 236
#define	RNSAP_ID_RECEIVED_TOTAL_WIDEBAND_POWER_VALUE_INCRDECRTHRES						 237
#define	RNSAP_ID_SFNSFNMEASUREMENTTHRESHOLDINFORMATION									 238
#define	RNSAP_ID_TRANSMITTED_CARRIER_POWER_VALUE										 239
#define	RNSAP_ID_TRANSMITTED_CARRIER_POWER_VALUE_INCRDECRTHRES							 240
#define	RNSAP_ID_TUTRANGPSMEASUREMENTTHRESHOLDINFORMATION								 241
#define	RNSAP_ID_UL_TIMESLOT_ISCP_VALUE													 242
#define	RNSAP_ID_UL_TIMESLOT_ISCP_VALUE_INCRDECRTHRES									 243
#define	RNSAP_ID_RX_TIMING_DEVIATION_VALUE_LCR											 293
#define	RNSAP_ID_DPC_MODE_CHANGE_SUPPORTINDICATOR										 19
#define	RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_247											 247
#define	RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_295											 295
#define	RNSAP_ID_PRIMARYCCPCH_RSCP_RL_RECONFPREPTDD										 202
#define	RNSAP_ID_DL_TIMESLOT_ISCP_INFO_RL_RECONFPREPTDD									 203
#define	RNSAP_ID_DL_TIMESLOT_ISCP_LCR_INFORMATION_RL_RECONFPREPTDD						 204
#define	RNSAP_ID_DSCH_RNTI																 249
#define	RNSAP_ID_DL_POWERBALANCING_INFORMATION											 296
#define	RNSAP_ID_DL_POWERBALANCING_ACTIVATIONINDICATOR									 297
#define	RNSAP_ID_DL_POWERBALANCING_UPDATEDINDICATOR										 298
#define	RNSAP_ID_DL_REFERENCEPOWERINFORMATION											 299
#define	RNSAP_ID_ENHANCED_PRIMARYCPICH_ECNO												 224
#define	RNSAP_ID_IPDL_TDD_PARAMETERSLCR													 252
#define	RNSAP_ID_CELLCAPABILITYCONTAINER_FDD											 300
#define	RNSAP_ID_CELLCAPABILITYCONTAINER_TDD											 301
#define	RNSAP_ID_CELLCAPABILITYCONTAINER_TDD_LCR										 302
#define	RNSAP_ID_RL_SPECIFIC_DCH_INFO													 317
#define	RNSAP_ID_RL_RECONFIGURATIONREQUESTFDD_RL_INFORMATIONLIST 						 318
#define	RNSAP_ID_RL_RECONFIGURATIONREQUESTFDD_RL_INFORMATION_IES						 319
#define	RNSAP_ID_RL_RECONFIGURATIONREQUESTTDD_RL_INFORMATION							 321
#define	RNSAP_ID_COMMONTRANSPORTCHANNELRESOURCESINITIALISATIONNOTREQUIRED				 250
#define	RNSAP_ID_DELAYEDACTIVATION 														 312
#define	RNSAP_ID_DELAYEDACTIVATIONLIST_RL_ACTIVATIONCMDFDD								 313
#define	RNSAP_ID_DELAYEDACTIVATIONINFORMATION_RL_ACTIVATIONCMDFDD						 314
#define	RNSAP_ID_DELAYEDACTIVATIONLIST_RL_ACTIVATIONCMDTDD								 315
#define	RNSAP_ID_DELAYEDACTIVATIONINFORMATION_RL_ACTIVATIONCMDTDD						 316
#define	RNSAP_ID_NEIGHBOURINGTDDCELLMEASUREMENTINFORMATIONLCR							 251
#define	RNSAP_ID_UL_SIR_TARGET_CCTRCH_INFORMATIONITEM_RL_SETUPRSPTDD					 150
#define	RNSAP_ID_UL_SIR_TARGET_CCTRCH_LCR_INFORMATIONITEM_RL_SETUPRSPTDD				 151
#define	RNSAP_ID_PRIMCCPCH_RSCP_DL_PC_RQSTTDD											 451
#define	RNSAP_ID_HSDSCH_FDD_INFORMATION													 452
#define	RNSAP_ID_HSDSCH_FDD_INFORMATION_RESPONSE										 453
#define	RNSAP_ID_HSDSCH_FDD_UPDATE_INFORMATION											 466
#define	RNSAP_ID_HSDSCH_INFORMATION_TO_MODIFY											 456
#define	RNSAP_ID_HSDSCHMACDFLOWSPECIFICINFORMATIONLIST_RL_PREEMPTREQUIREDIND			 516
#define	RNSAP_ID_HSDSCHMACDFLOWSPECIFICINFORMATIONITEM_RL_PREEMPTREQUIREDIND			 517
#define	RNSAP_ID_HSDSCH_RNTI															 457
#define	RNSAP_ID_HSDSCH_TDD_INFORMATION													 458
#define	RNSAP_ID_HSDSCH_TDD_INFORMATION_RESPONSE										 459
#define	RNSAP_ID_HSDSCH_TDD_UPDATE_INFORMATION											 467
#define	RNSAP_ID_HSPDSCH_RL_ID															 463
#define	RNSAP_ID_HSDSCH_MACDFLOWS_TO_ADD 												 531
#define	RNSAP_ID_HSDSCH_MACDFLOWS_TO_DELETE												 532
#define	RNSAP_ID_ANGLE_OF_ARRIVAL_VALUE_LCR												 148
#define	RNSAP_ID_TRAFFICCLASS															 158
#define	RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_248											 248
#define	RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_253											 253
#define	RNSAP_ID_PDSCH_RL_ID															 323
#define	RNSAP_ID_TIMESLOT_RL_SETUPRSPTDD												 325
#define	RNSAP_ID_GERAN_CELL_CAPABILITY													 468
#define	RNSAP_ID_GERAN_CLASSMARK														 469
#define	RNSAP_ID_DSCH_INITIALWINDOWSIZE													 480
#define	RNSAP_ID_UL_SYNCHRONISATION_PARAMETERS_LCR										 464
#define	RNSAP_ID_SNA_INFORMATION														 479
#define	RNSAP_ID_MACHS_RESETINDICATOR													 465
#define	RNSAP_ID_TDD_DL_DPCH_TIMESLOTFORMATMODIFYITEM_LCR_RL_RECONFREADYTDD				 481
#define	RNSAP_ID_TDD_UL_DPCH_TIMESLOTFORMATMODIFYITEM_LCR_RL_RECONFREADYTDD				 482
#define	RNSAP_ID_TDD_TPC_UPLINKSTEPSIZE_LCR_RL_SETUPRQSTTDD								 483
#define	RNSAP_ID_UL_CCTRCH_INFORMATIONLIST_RL_ADDITIONRQSTTDD							 484
#define	RNSAP_ID_UL_CCTRCH_INFORMATIONITEM_RL_ADDITIONRQSTTDD							 485
#define	RNSAP_ID_DL_CCTRCH_INFORMATIONLIST_RL_ADDITIONRQSTTDD							 486
#define	RNSAP_ID_DL_CCTRCH_INFORMATIONITEM_RL_ADDITIONRQSTTDD							 487
#define	RNSAP_ID_TDD_TPC_UPLINKSTEPSIZE_INFORMATIONADD_LCR_RL_RECONFPREPTDD				 488
#define	RNSAP_ID_TDD_TPC_UPLINKSTEPSIZE_INFORMATIONMODIFY_LCR_RL_RECONFPREPTDD			 489
#define	RNSAP_ID_TDD_TPC_DOWNLINKSTEPSIZE_INFORMATIONADD_RL_RECONFPREPTDD				 490
#define	RNSAP_ID_TDD_TPC_DOWNLINKSTEPSIZE_INFORMATIONMODIFY_RL_RECONFPREPTDD			 491
#define	RNSAP_ID_UL_TIMINGADVANCECTRL_LCR												 492
#define	RNSAP_ID_HSPDSCH_TIMESLOT_INFORMATIONLIST_PHYCHRECONFRQSTTDD					 493
#define	RNSAP_ID_HSPDSCH_TIMESLOT_INFORMATIONLISTLCR_PHYCHRECONFRQSTTDD					 494
#define	RNSAP_ID_HS_SICH_RECEPTION_QUALITY 												 495
#define	RNSAP_ID_HS_SICH_RECEPTION_QUALITY_MEASUREMENT_VALUE 							 496
#define	RNSAP_ID_HSSICH_INFO_DM_RPRT													 497
#define	RNSAP_ID_HSSICH_INFO_DM_RQST													 498
#define	RNSAP_ID_HSSICH_INFO_DM															 499
#define	RNSAP_ID_CCTRCH_MAXIMUM_DL_POWER_RL_SETUPRSPTDD									 500
#define	RNSAP_ID_CCTRCH_MINIMUM_DL_POWER_RL_SETUPRSPTDD									 501
#define	RNSAP_ID_CCTRCH_MAXIMUM_DL_POWER_RL_ADDITIONRSPTDD								 502
#define	RNSAP_ID_CCTRCH_MINIMUM_DL_POWER_RL_ADDITIONRSPTDD								 503
#define	RNSAP_ID_CCTRCH_MAXIMUM_DL_POWER_RL_RECONFREADYTDD								 504
#define	RNSAP_ID_CCTRCH_MINIMUM_DL_POWER_RL_RECONFREADYTDD								 505
#define	RNSAP_ID_MAXIMUM_DL_POWER_TIMESLOTLCR_INFORMATIONMODIFYITEM_RL_RECONFREADYTDD	 506
#define	RNSAP_ID_MINIMUM_DL_POWER_TIMESLOTLCR_INFORMATIONMODIFYITEM_RL_RECONFREADYTDD	 507
#define	RNSAP_ID_DL_CCTRCH_INFORMATIONLIST_RL_RECONFRSPTDD								 508
#define	RNSAP_ID_DL_DPCH_INFORMATIONMODIFYITEM_LCR_RL_RECONFRSPTDD						 509
#define	RNSAP_ID_MAXIMUM_DL_POWER_TIMESLOTLCR_INFORMATIONITEM							 510
#define	RNSAP_ID_MINIMUM_DL_POWER_TIMESLOTLCR_INFORMATIONITEM							 511
#define	RNSAP_ID_TDD_SUPPORT_8PSK														 512
#define	RNSAP_ID_TDD_MAXNRDLPHYSICALCHANNELS											 513
#define	RNSAP_ID_EXTENDEDGSMCELLINDIVIDUALOFFSET										 514
#define	RNSAP_ID_RL_PARAMETERUPDATEINDICATIONFDD_RL_INFORMATIONLIST 					 518
#define	RNSAP_ID_PRIMARY_CPICH_USAGE_FOR_CHANNEL_ESTIMATION								 519
#define	RNSAP_ID_SECONDARY_CPICH_INFORMATION											 520
#define	RNSAP_ID_SECONDARY_CPICH_INFORMATION_CHANGE										 521
#define	RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_522											 522
#define	RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_523											 523
#define	RNSAP_ID_RL_PARAMETERUPDATEINDICATIONFDD_RL_INFORMATION_ITEM					 524
#define	RNSAP_ID_PHASE_REFERENCE_UPDATE_INDICATOR										 525
#define	RNSAP_ID_UNIDIRECTIONAL_DCH_INDICATOR											 526
#define	RNSAP_ID_RL_INFORMATION_RL_RECONFPREPTDD										 527
#define	RNSAP_ID_MULTIPLE_RL_INFORMATIONRESPONSE_RL_RECONFREADYTDD						 528
#define	RNSAP_ID_RL_RECONFIGURATIONRESPONSETDD_RL_INFORMATION							 529
#define	RNSAP_ID_SATELLITE_ALMANAC_INFORMATION_EXTITEM									 530
#define	RNSAP_ID_HSDSCH_INFORMATION_TO_MODIFY_UNSYNCHRONISED							 533
#define	RNSAP_ID_TNLQOS																	 534
#define	RNSAP_ID_RTLOADVALUE															 535
#define	RNSAP_ID_NRTLOADINFORMATIONVALUE												 536
#define	RNSAP_ID_CELLPORTIONID															 537
#define	RNSAP_ID_UPPTSINTERFERENCEVALUE													 538
#define	RNSAP_ID_PRIMARYCCPCH_RSCP_DELTA												 539
#define	RNSAP_ID_UEMEASUREMENTTYPE														 540
#define	RNSAP_ID_UEMEASUREMENTTIMESLOTINFOHCR											 541
#define	RNSAP_ID_UEMEASUREMENTTIMESLOTINFOLCR											 542
#define	RNSAP_ID_UEMEASUREMENTREPORTCHARACTERISTICS										 543
#define	RNSAP_ID_UEMEASUREMENTPARAMETERMODALLOW											 544
#define	RNSAP_ID_UEMEASUREMENTVALUEINFORMATION											 545
#define	RNSAP_ID_INTERFACESTOTRACEITEM													 546
#define	RNSAP_ID_LISTOFINTERFACESTOTRACE												 547
#define	RNSAP_ID_TRACEDEPTH																 548
#define	RNSAP_ID_TRACERECORDINGSESSIONREFERENCE											 549
#define	RNSAP_ID_TRACEREFERENCE															 550
#define	RNSAP_ID_UEIDENTITY																 551
#define	RNSAP_ID_NACC_RELATED_DATA														 552
#define	RNSAP_ID_GSM_CELL_INFEX_RQST													 553
#define	RNSAP_ID_MEASUREMENTRECOVERYBEHAVIOR											 554
#define	RNSAP_ID_MEASUREMENTRECOVERYREPORTINGINDICATOR									 555
#define	RNSAP_ID_MEASUREMENTRECOVERYSUPPORTINDICATOR									 556
#define	RNSAP_ID_DL_DPCH_POWER_INFORMATION_RL_RECONFPREPFDD								 557
#define	RNSAP_ID_F_DPCH_INFORMATION_RL_RECONFPREPFDD									 558
#define	RNSAP_ID_F_DPCH_INFORMATION_RL_SETUPRQSTFDD										 559
#define	RNSAP_ID_MBMS_BEARER_SERVICE_LIST												 560
#define	RNSAP_ID_MBMS_BEARER_SERVICE_LIST_INFEX_RSP										 561
#define	RNSAP_ID_ACTIVE_MBMS_BEARER_SERVICEFDD											 562
#define	RNSAP_ID_ACTIVE_MBMS_BEARER_SERVICETDD											 563
#define	RNSAP_ID_OLD_URA_ID																 564
#define	RNSAP_ID_UE_STATE																 568
#define	RNSAP_ID_URA_ID																	 569
#define	RNSAP_ID_HARQ_PREAMBLE_MODE														 571
#define	RNSAP_ID_SYNCHRONISATIONINDICATOR												 572
#define	RNSAP_ID_UL_DPDCHINDICATOREDCH													 573
#define	RNSAP_ID_EDPCH_INFORMATION														 574
#define	RNSAP_ID_RL_SPECIFIC_EDCH_INFORMATION											 575
#define	RNSAP_ID_EDCH_RL_INDICATION														 576
#define	RNSAP_ID_EDCH_FDD_INFORMATION													 577
#define	RNSAP_ID_EDCH_RLSET_ID															 578
#define	RNSAP_ID_SERVING_EDCHRL_ID														 579
#define	RNSAP_ID_EDCH_FDD_DL_CONTROLCHANNELINFORMATION									 580
#define	RNSAP_ID_EDCH_FDD_INFORMATIONRESPONSE											 581
#define	RNSAP_ID_EDCH_MACDFLOWS_TO_ADD													 582
#define	RNSAP_ID_EDCH_FDD_INFORMATION_TO_MODIFY											 583
#define	RNSAP_ID_EDCH_MACDFLOWS_TO_DELETE												 584
#define	RNSAP_ID_EDPCH_INFORMATION_RLRECONFREQUEST_FDD									 585
#define	RNSAP_ID_EDCH_MACDFLOWSPECIFICINFORMATIONLIST_RL_PREEMPTREQUIREDIND				 586
#define	RNSAP_ID_EDCH_MACDFLOWSPECIFICINFORMATIONITEM_RL_PREEMPTREQUIREDIND				 587
#define	RNSAP_ID_EDCH_MACDFLOWSPECIFICINFORMATIONLIST_RL_CONGESTIND  					 588
#define	RNSAP_ID_EDCH_MACDFLOWSPECIFICINFORMATIONITEM_RL_CONGESTIND						 589
#define	RNSAP_ID_MBMS_BEARER_SERVICE_FULL_ADDRESS										 590
#define	RNSAP_ID_INITIAL_DL_DPCH_TIMINGADJUSTMENT										 591
#define	RNSAP_ID_INITIAL_DL_DPCH_TIMINGADJUSTMENT_ALLOWED								 592
#define	RNSAP_ID_USER_PLANE_CONGESTION_FIELDS_INCLUSION									 593
#define	RNSAP_ID_HARQ_PREAMBLE_MODE_ACTIVATION_INDICATOR								 594
#define	RNSAP_ID_MULTIPLE_DEDICATEDMEASUREMENTVALUELIST_TDD_DM_RSP						 595
#define	RNSAP_ID_MULTIPLE_DEDICATEDMEASUREMENTVALUELIST_LCR_TDD_DM_RSP					 596
#define	RNSAP_ID_PROVIDEDINFORMATION													 597
#define	RNSAP_ID_ACTIVE_MBMS_BEARER_SERVICEFDD_PFL										 598
#define	RNSAP_ID_ACTIVE_MBMS_BEARER_SERVICETDD_PFL										 599
#define	RNSAP_ID_FREQUENCYBANDINDICATOR													 600
static dissector_handle_t rnsap_handle=NULL;

/* Initialize the protocol and registered fields */
static int proto_rnsap = -1;

static int hf_rnsap_pdu_length = -1;
static int hf_rnsap_IE_length = -1;
static int hf_rnsap_L3_DL_DCCH_Message_PDU = -1;


/*--- Included file: packet-rnsap-hf.c ---*/
#line 1 "packet-rnsap-hf.c"
static int hf_rnsap_RNSAP_PDU_PDU = -1;           /* RNSAP_PDU */
static int hf_rnsap_initiatingMessage = -1;       /* InitiatingMessage */
static int hf_rnsap_successfulOutcome = -1;       /* SuccessfulOutcome */
static int hf_rnsap_unsuccessfulOutcome = -1;     /* UnsuccessfulOutcome */
static int hf_rnsap_outcome = -1;                 /* Outcome */
static int hf_rnsap_procedureID = -1;             /* ProcedureID */
static int hf_rnsap_criticality = -1;             /* Criticality */
static int hf_rnsap_transactionID = -1;           /* TransactionID */
static int hf_rnsap_initiatingMessageValue = -1;  /* InitiatingMessageValue */
static int hf_rnsap_successfulOutcomeValue = -1;  /* SuccessfulOutcomeValue */
static int hf_rnsap_unsuccessfulOutcomeValue = -1;  /* UnsuccessfulOutcomeValue */
static int hf_rnsap_outcomeValue = -1;            /* OutcomeValue */
static int hf_rnsap_id_commonTransportChannelResourcesInitialisation = -1;  /* CommonTransportChannelResourcesRequest */
static int hf_rnsap_id_commonTransportChannelResourcesRelease = -1;  /* CommonTransportChannelResourcesReleaseRequest */
static int hf_rnsap_id_compressedModeCommand = -1;  /* CompressedModeCommand */
static int hf_rnsap_id_downlinkPowerControl = -1;  /* DL_PowerControlRequest */
static int hf_rnsap_id_downlinkPowerTimeslotControl = -1;  /* DL_PowerTimeslotControlRequest */
static int hf_rnsap_id_downlinkSignallingTransfer = -1;  /* DownlinkSignallingTransferRequest */
static int hf_rnsap_id_errorIndication = -1;      /* ErrorIndication */
static int hf_rnsap_id_dedicatedMeasurementFailure = -1;  /* DedicatedMeasurementFailureIndication */
static int hf_rnsap_id_dedicatedMeasurementInitiation = -1;  /* DedicatedMeasurementInitiationRequest */
static int hf_rnsap_id_dedicatedMeasurementReporting = -1;  /* DedicatedMeasurementReport */
static int hf_rnsap_id_dedicatedMeasurementTermination = -1;  /* DedicatedMeasurementTerminationRequest */
static int hf_rnsap_id_paging = -1;               /* PagingRequest */
static int hf_rnsap_id_physicalChannelReconfiguration = -1;  /* PhysicalChannelReconfigurationRequestTDD */
static int hf_rnsap_id_privateMessage = -1;       /* PrivateMessage */
static int hf_rnsap_id_radioLinkAddition = -1;    /* RadioLinkAdditionRequestFDD */
static int hf_rnsap_id_radioLinkAddition_TDD = -1;  /* RadioLinkAdditionRequestTDD */
static int hf_rnsap_id_radioLinkCongestion = -1;  /* RadioLinkCongestionIndication */
static int hf_rnsap_id_radioLinkDeletion = -1;    /* RadioLinkDeletionRequest */
static int hf_rnsap_id_radioLinkFailure = -1;     /* RadioLinkFailureIndication */
static int hf_rnsap_id_radioLinkPreemption = -1;  /* RadioLinkPreemptionRequiredIndication */
static int hf_rnsap_id_radioLinkRestoration = -1;  /* RadioLinkRestoreIndication */
static int hf_rnsap_id_radioLinkSetup = -1;       /* RadioLinkSetupRequestFDD */
static int hf_rnsap_id_radioLinkSetupTdd = -1;    /* RadioLinkSetupRequestTDD */
static int hf_rnsap_id_relocationCommit = -1;     /* RelocationCommit */
static int hf_rnsap_id_synchronisedRadioLinkReconfigurationCancellation = -1;  /* RadioLinkReconfigurationCancel */
static int hf_rnsap_id_synchronisedRadioLinkReconfigurationCommit = -1;  /* RadioLinkReconfigurationCommit */
static int hf_rnsap_id_synchronisedRadioLinkReconfigurationPreparation = -1;  /* RadioLinkReconfigurationPrepareFDD */
static int hf_rnsap_id_synchronisedRadioLinkReconfigurationPreparation_TDD = -1;  /* RadioLinkReconfigurationReadyTDD */
static int hf_rnsap_id_unSynchronisedRadioLinkReconfiguration = -1;  /* RadioLinkReconfigurationRequestFDD */
static int hf_rnsap_id_unSynchronisedRadioLinkReconfiguration_TDD = -1;  /* RadioLinkReconfigurationRequestTDD */
static int hf_rnsap_id_uplinkSignallingTransfer = -1;  /* UplinkSignallingTransferIndicationFDD */
static int hf_rnsap_id_uplinkSignallingTransfer_TDD = -1;  /* UplinkSignallingTransferIndicationTDD */
static int hf_rnsap_id_commonMeasurementFailure = -1;  /* CommonMeasurementFailureIndication */
static int hf_rnsap_id_commonMeasurementInitiation = -1;  /* CommonMeasurementInitiationRequest */
static int hf_rnsap_id_commonMeasurementReporting = -1;  /* CommonMeasurementReport */
static int hf_rnsap_id_commonMeasurementTermination = -1;  /* CommonMeasurementTerminationRequest */
static int hf_rnsap_id_informationExchangeFailure = -1;  /* InformationExchangeFailureIndication */
static int hf_rnsap_id_informationExchangeInitiation = -1;  /* InformationExchangeInitiationRequest */
static int hf_rnsap_id_informationReporting = -1;  /* InformationReport */
static int hf_rnsap_id_informationExchangeTermination = -1;  /* InformationExchangeTerminationRequest */
static int hf_rnsap_id_reset = -1;                /* ResetRequest */
static int hf_rnsap_id_radioLinkActivation = -1;  /* RadioLinkActivationCommandFDD */
static int hf_rnsap_id_radioLinkActivation_TDD = -1;  /* RadioLinkActivationCommandTDD */
static int hf_rnsap_id_gERANuplinkSignallingTransfer = -1;  /* GERANUplinkSignallingTransferIndication */
static int hf_rnsap_id_radioLinkParameterUpdate = -1;  /* RadioLinkParameterUpdateIndicationFDD */
static int hf_rnsap_id_radioLinkParameterUpdate_TDD = -1;  /* RadioLinkParameterUpdateIndicationTDD */
static int hf_rnsap_id_uEMeasurementFailure = -1;  /* UEMeasurementFailureIndication */
static int hf_rnsap_id_uEMeasurementInitiation = -1;  /* UEMeasurementInitiationRequest */
static int hf_rnsap_id_uEMeasurementReporting = -1;  /* UEMeasurementReport */
static int hf_rnsap_id_uEMeasurementTermination = -1;  /* UEMeasurementTerminationRequest */
static int hf_rnsap_id_iurDeactivateTrace = -1;   /* IurDeactivateTrace */
static int hf_rnsap_id_iurInvokeTrace = -1;       /* IurInvokeTrace */
static int hf_rnsap_id_mBMSAttach = -1;           /* MBMSAttachCommand */
static int hf_rnsap_id_mBMSDetach = -1;           /* MBMSDetachCommand */
static int hf_rnsap_id_directInformationTransfer = -1;  /* DirectInformationTransfer */
static int hf_rnsap_id_commonTransportChannelResourcesInitialisation1 = -1;  /* CommonTransportChannelResourcesResponseFDD */
static int hf_rnsap_id_commonTransportChannelResourcesInitialisation_TDD = -1;  /* CommonTransportChannelResourcesResponseTDD */
static int hf_rnsap_id_dedicatedMeasurementInitiation1 = -1;  /* DedicatedMeasurementInitiationResponse */
static int hf_rnsap_id_physicalChannelReconfiguration1 = -1;  /* PhysicalChannelReconfigurationCommand */
static int hf_rnsap_id_radioLinkAddition1 = -1;   /* RadioLinkAdditionResponseFDD */
static int hf_rnsap_id_radioLinkAddition_TDD1 = -1;  /* RadioLinkAdditionResponseTDD */
static int hf_rnsap_id_radioLinkDeletion1 = -1;   /* RadioLinkDeletionResponse */
static int hf_rnsap_id_radioLinkSetup1 = -1;      /* RadioLinkSetupResponseFDD */
static int hf_rnsap_id_radioLinkSetupTdd1 = -1;   /* RadioLinkSetupResponseTDD */
static int hf_rnsap_id_synchronisedRadioLinkReconfigurationPreparation1 = -1;  /* RadioLinkReconfigurationReadyFDD */
static int hf_rnsap_id_unSynchronisedRadioLinkReconfiguration1 = -1;  /* RadioLinkReconfigurationResponseFDD */
static int hf_rnsap_id_unSynchronisedRadioLinkReconfiguration_TDD1 = -1;  /* RadioLinkReconfigurationResponseTDD */
static int hf_rnsap_id_commonMeasurementInitiation1 = -1;  /* CommonMeasurementInitiationResponse */
static int hf_rnsap_id_informationExchangeInitiation1 = -1;  /* InformationExchangeInitiationResponse */
static int hf_rnsap_id_reset1 = -1;               /* ResetResponse */
static int hf_rnsap_id_uEMeasurementInitiation1 = -1;  /* UEMeasurementInitiationResponse */
static int hf_rnsap_id_commonTransportChannelResourcesInitialisation2 = -1;  /* CommonTransportChannelResourcesFailure */
static int hf_rnsap_id_dedicatedMeasurementInitiation2 = -1;  /* DedicatedMeasurementInitiationFailure */
static int hf_rnsap_id_physicalChannelReconfiguration2 = -1;  /* PhysicalChannelReconfigurationFailure */
static int hf_rnsap_id_radioLinkAddition2 = -1;   /* RadioLinkAdditionFailureFDD */
static int hf_rnsap_id_radioLinkAddition_TDD2 = -1;  /* RadioLinkAdditionFailureTDD */
static int hf_rnsap_id_radioLinkSetup2 = -1;      /* RadioLinkSetupFailureFDD */
static int hf_rnsap_id_radioLinkSetupTdd2 = -1;   /* RadioLinkSetupFailureTDD */
static int hf_rnsap_id_synchronisedRadioLinkReconfigurationPreparation2 = -1;  /* RadioLinkReconfigurationFailure */
static int hf_rnsap_id_unSynchronisedRadioLinkReconfiguration2 = -1;  /* RadioLinkReconfigurationFailure */
static int hf_rnsap_id_unSynchronisedRadioLinkReconfiguration_TDD2 = -1;  /* RadioLinkReconfigurationFailure */
static int hf_rnsap_id_commonMeasurementInitiation2 = -1;  /* CommonMeasurementInitiationFailure */
static int hf_rnsap_id_informationExchangeInitiation2 = -1;  /* InformationExchangeInitiationFailure */
static int hf_rnsap_id_uEMeasurementInitiation2 = -1;  /* UEMeasurementInitiationFailure */
static int hf_rnsap_id_AllowedQueuingTime = -1;   /* AllowedQueuingTime */
static int hf_rnsap_id_Allowed_Rate_Information = -1;  /* Allowed_Rate_Information */
static int hf_rnsap_id_AntennaColocationIndicator = -1;  /* AntennaColocationIndicator */
static int hf_rnsap_id_BindingID = -1;            /* BindingID */
static int hf_rnsap_id_C_ID = -1;                 /* C_ID */
static int hf_rnsap_id_C_RNTI = -1;               /* C_RNTI */
static int hf_rnsap_id_Cell_Capacity_Class_Value = -1;  /* Cell_Capacity_Class_Value */
static int hf_rnsap_id_CFN = -1;                  /* CFN */
static int hf_rnsap_id_CN_CS_DomainIdentifier = -1;  /* CN_CS_DomainIdentifier */
static int hf_rnsap_id_CN_PS_DomainIdentifier = -1;  /* CN_PS_DomainIdentifier */
static int hf_rnsap_id_Cause = -1;                /* Cause */
static int hf_rnsap_id_CoverageIndicator = -1;    /* CoverageIndicator */
static int hf_rnsap_id_CriticalityDiagnostics = -1;  /* CriticalityDiagnostics */
static int hf_rnsap_id_ContextInfoItem_Reset = -1;  /* ContextInfoItem_Reset */
static int hf_rnsap_id_ContextGroupInfoItem_Reset = -1;  /* ContextGroupInfoItem_Reset */
static int hf_rnsap_id_D_RNTI = -1;               /* D_RNTI */
static int hf_rnsap_id_D_RNTI_ReleaseIndication = -1;  /* D_RNTI_ReleaseIndication */
static int hf_rnsap_id_DCHs_to_Add_FDD = -1;      /* DCH_FDD_Information */
static int hf_rnsap_id_DCHs_to_Add_TDD = -1;      /* DCH_TDD_Information */
static int hf_rnsap_id_DCH_DeleteList_RL_ReconfPrepFDD = -1;  /* DCH_DeleteList_RL_ReconfPrepFDD */
static int hf_rnsap_id_DCH_DeleteList_RL_ReconfPrepTDD = -1;  /* DCH_DeleteList_RL_ReconfPrepTDD */
static int hf_rnsap_id_DCH_DeleteList_RL_ReconfRqstFDD = -1;  /* DCH_DeleteList_RL_ReconfRqstFDD */
static int hf_rnsap_id_DCH_DeleteList_RL_ReconfRqstTDD = -1;  /* DCH_DeleteList_RL_ReconfRqstTDD */
static int hf_rnsap_id_DCH_FDD_Information = -1;  /* DCH_FDD_Information */
static int hf_rnsap_id_DCH_TDD_Information = -1;  /* DCH_TDD_Information */
static int hf_rnsap_id_FDD_DCHs_to_Modify = -1;   /* FDD_DCHs_to_Modify */
static int hf_rnsap_id_TDD_DCHs_to_Modify = -1;   /* TDD_DCHs_to_Modify */
static int hf_rnsap_id_DCH_InformationResponse = -1;  /* DCH_InformationResponse */
static int hf_rnsap_id_DCH_Rate_InformationItem_RL_CongestInd = -1;  /* DCH_Rate_InformationItem_RL_CongestInd */
static int hf_rnsap_id_DL_CCTrCH_InformationAddItem_RL_ReconfPrepTDD = -1;  /* DL_CCTrCH_InformationAddItem_RL_ReconfPrepTDD */
static int hf_rnsap_id_DL_CCTrCH_InformationListIE_RL_ReconfReadyTDD = -1;  /* DL_CCTrCHInformationListIE_RL_ReconfReadyTDD */
static int hf_rnsap_id_DL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD = -1;  /* DL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD */
static int hf_rnsap_id_DL_CCTrCH_InformationItem_RL_SetupRqstTDD = -1;  /* DL_CCTrCH_InformationItem_RL_SetupRqstTDD */
static int hf_rnsap_id_DL_CCTrCH_InformationListIE_PhyChReconfRqstTDD = -1;  /* DL_CCTrCH_InformationListIE_PhyChReconfRqstTDD */
static int hf_rnsap_id_DL_CCTrCH_InformationListIE_RL_AdditionRspTDD = -1;  /* DL_CCTrCHInformationListIE_RL_AdditionRspTDD */
static int hf_rnsap_id_DL_CCTrCH_InformationListIE_RL_SetupRspTDD = -1;  /* DL_CCTrCH_InformationItem_RL_SetupRqstTDD */
static int hf_rnsap_id_DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD = -1;  /* DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD */
static int hf_rnsap_id_DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD = -1;  /* DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD */
static int hf_rnsap_id_DL_CCTrCH_InformationList_RL_SetupRqstTDD = -1;  /* DL_CCTrCH_InformationList_RL_SetupRqstTDD */
static int hf_rnsap_id_FDD_DL_CodeInformation = -1;  /* FDD_DL_CodeInformation */
static int hf_rnsap_id_DL_DPCH_Information_RL_ReconfPrepFDD = -1;  /* DL_DPCH_Information_RL_ReconfPrepFDD */
static int hf_rnsap_id_DL_DPCH_Information_RL_SetupRqstFDD = -1;  /* DL_DPCH_Information_RL_SetupRqstFDD */
static int hf_rnsap_id_DL_DPCH_Information_RL_ReconfRqstFDD = -1;  /* DL_DPCH_Information_RL_ReconfRqstFDD */
static int hf_rnsap_id_DL_DPCH_InformationItem_PhyChReconfRqstTDD = -1;  /* DL_DPCH_InformationItem_PhyChReconfRqstTDD */
static int hf_rnsap_id_DL_DPCH_InformationItem_RL_AdditionRspTDD = -1;  /* DL_DPCH_InformationItem_RL_AdditionRspTDD */
static int hf_rnsap_id_DL_DPCH_InformationItem_RL_SetupRspTDD = -1;  /* DL_DPCH_InformationItem_RL_SetupRspTDD */
static int hf_rnsap_id_DL_DPCH_TimingAdjustment = -1;  /* DL_DPCH_TimingAdjustment */
static int hf_rnsap_id_DLReferencePower = -1;     /* DL_Power */
static int hf_rnsap_id_DLReferencePowerList_DL_PC_Rqst = -1;  /* DL_ReferencePowerInformationList_DL_PC_Rqst */
static int hf_rnsap_id_DL_ReferencePowerInformation_DL_PC_Rqst = -1;  /* DL_ReferencePowerInformation_DL_PC_Rqst */
static int hf_rnsap_id_DPC_Mode = -1;             /* DPC_Mode */
static int hf_rnsap_id_DRXCycleLengthCoefficient = -1;  /* DRXCycleLengthCoefficient */
static int hf_rnsap_id_DedicatedMeasurementObjectType_DM_Fail_Ind = -1;  /* DedicatedMeasurementObjectType_DM_Fail_Ind */
static int hf_rnsap_id_DedicatedMeasurementObjectType_DM_Fail = -1;  /* DedicatedMeasurementObjectType_DM_Fail */
static int hf_rnsap_id_DedicatedMeasurementObjectType_DM_Rprt = -1;  /* DedicatedMeasurementObjectType_DM_Rprt */
static int hf_rnsap_id_DedicatedMeasurementObjectType_DM_Rqst = -1;  /* DedicatedMeasurementObjectType_DM_Rqst */
static int hf_rnsap_id_DedicatedMeasurementObjectType_DM_Rsp = -1;  /* DedicatedMeasurementObjectType_DM_Rsp */
static int hf_rnsap_id_DedicatedMeasurementType = -1;  /* DedicatedMeasurementType */
static int hf_rnsap_id_FACH_InfoForUESelectedS_CCPCH_CTCH_ResourceRspFDD = -1;  /* FACH_InfoForUESelectedS_CCPCH_CTCH_ResourceRspFDD */
static int hf_rnsap_id_FACH_InfoForUESelectedS_CCPCH_CTCH_ResourceRspTDD = -1;  /* FACH_InfoForUESelectedS_CCPCH_CTCH_ResourceRspTDD */
static int hf_rnsap_id_Guaranteed_Rate_Information = -1;  /* Guaranteed_Rate_Information */
static int hf_rnsap_id_IMSI = -1;                 /* IMSI */
static int hf_rnsap_id_HCS_Prio = -1;             /* HCS_Prio */
static int hf_rnsap_id_L3_Information = -1;       /* L3_Information */
static int hf_rnsap_id_AdjustmentPeriod = -1;     /* AdjustmentPeriod */
static int hf_rnsap_id_MaxAdjustmentStep = -1;    /* MaxAdjustmentStep */
static int hf_rnsap_id_MeasurementFilterCoefficient = -1;  /* MeasurementFilterCoefficient */
static int hf_rnsap_id_MessageStructure = -1;     /* MessageStructure */
static int hf_rnsap_id_MeasurementID = -1;        /* MeasurementID */
static int hf_rnsap_id_Neighbouring_GSM_CellInformation = -1;  /* Neighbouring_GSM_CellInformation */
static int hf_rnsap_id_Neighbouring_UMTS_CellInformationItem = -1;  /* Neighbouring_UMTS_CellInformationItem */
static int hf_rnsap_id_NRT_Load_Information_Value = -1;  /* NRT_Load_Information_Value */
static int hf_rnsap_id_NRT_Load_Information_Value_IncrDecrThres = -1;  /* NRT_Load_Information_Value_IncrDecrThres */
static int hf_rnsap_id_PagingArea_PagingRqst = -1;  /* PagingArea_PagingRqst */
static int hf_rnsap_id_FACH_FlowControlInformation = -1;  /* FACH_FlowControlInformation */
static int hf_rnsap_id_PartialReportingIndicator = -1;  /* PartialReportingIndicator */
static int hf_rnsap_id_Permanent_NAS_UE_Identity = -1;  /* Permanent_NAS_UE_Identity */
static int hf_rnsap_id_PowerAdjustmentType = -1;  /* PowerAdjustmentType */
static int hf_rnsap_id_RANAP_RelocationInformation = -1;  /* RANAP_RelocationInformation */
static int hf_rnsap_id_RL_Information_PhyChReconfRqstFDD = -1;  /* RL_Information_PhyChReconfRqstFDD */
static int hf_rnsap_id_RL_Information_PhyChReconfRqstTDD = -1;  /* RL_Information_PhyChReconfRqstTDD */
static int hf_rnsap_id_RL_Information_RL_AdditionRqstFDD = -1;  /* RL_Information_RL_AdditionRqstFDD */
static int hf_rnsap_id_RL_Information_RL_AdditionRqstTDD = -1;  /* RL_Information_RL_AdditionRqstTDD */
static int hf_rnsap_id_RL_Information_RL_DeletionRqst = -1;  /* RL_Information_RL_DeletionRqst */
static int hf_rnsap_id_RL_Information_RL_FailureInd = -1;  /* RL_Information_RL_FailureInd */
static int hf_rnsap_id_RL_Information_RL_ReconfPrepFDD = -1;  /* RL_Information_RL_ReconfPrepFDD */
static int hf_rnsap_id_RL_Information_RL_RestoreInd = -1;  /* RL_Information_RL_RestoreInd */
static int hf_rnsap_id_RL_Information_RL_SetupRqstFDD = -1;  /* RL_InformationList_RL_SetupRqstFDD */
static int hf_rnsap_id_RL_Information_RL_SetupRqstTDD = -1;  /* RL_Information_RL_SetupRqstTDD */
static int hf_rnsap_id_RL_InformationItem_RL_CongestInd = -1;  /* RL_InformationItem_RL_CongestInd */
static int hf_rnsap_id_RL_InformationItem_DM_Rprt = -1;  /* RL_InformationItem_DM_Rprt */
static int hf_rnsap_id_RL_InformationItem_DM_Rqst = -1;  /* RL_InformationItem_DM_Rqst */
static int hf_rnsap_id_RL_InformationItem_DM_Rsp = -1;  /* RL_InformationItem_DM_Rsp */
static int hf_rnsap_id_RL_InformationItem_RL_PreemptRequiredInd = -1;  /* RL_InformationItem_RL_PreemptRequiredInd */
static int hf_rnsap_id_RL_InformationItem_RL_SetupRqstFDD = -1;  /* RL_InformationItem_RL_SetupRqstFDD */
static int hf_rnsap_id_RL_InformationList_RL_CongestInd = -1;  /* RL_InformationList_RL_CongestInd */
static int hf_rnsap_id_RL_InformationList_RL_AdditionRqstFDD = -1;  /* RL_InformationList_RL_AdditionRqstFDD */
static int hf_rnsap_id_RL_InformationList_RL_DeletionRqst = -1;  /* RL_InformationList_RL_DeletionRqst */
static int hf_rnsap_id_RL_InformationList_RL_PreemptRequiredInd = -1;  /* RL_InformationList_RL_PreemptRequiredInd */
static int hf_rnsap_id_RL_InformationList_RL_ReconfPrepFDD = -1;  /* RL_InformationList_RL_ReconfPrepFDD */
static int hf_rnsap_id_RL_InformationResponse_RL_AdditionRspTDD = -1;  /* RL_InformationResponse_RL_AdditionRspTDD */
static int hf_rnsap_id_RL_InformationResponse_RL_ReconfReadyTDD = -1;  /* RL_InformationResponse_RL_ReconfReadyTDD */
static int hf_rnsap_id_RL_InformationResponse_RL_SetupRspTDD = -1;  /* RL_InformationResponse_RL_SetupRspTDD */
static int hf_rnsap_id_RL_InformationResponseItem_RL_AdditionRspFDD = -1;  /* RL_InformationResponseItem_RL_AdditionRspFDD */
static int hf_rnsap_id_RL_InformationResponseItem_RL_ReconfReadyFDD = -1;  /* RL_InformationResponseItem_RL_ReconfReadyFDD */
static int hf_rnsap_id_RL_InformationResponseItem_RL_ReconfRspFDD = -1;  /* RL_InformationResponseItem_RL_ReconfRspFDD */
static int hf_rnsap_id_RL_InformationResponseItem_RL_SetupRspFDD = -1;  /* RL_InformationResponseItem_RL_SetupRspFDD */
static int hf_rnsap_id_RL_InformationResponseList_RL_AdditionRspFDD = -1;  /* RL_InformationResponseList_RL_AdditionRspFDD */
static int hf_rnsap_id_RL_InformationResponseList_RL_ReconfReadyFDD = -1;  /* RL_InformationResponseList_RL_ReconfReadyFDD */
static int hf_rnsap_id_RL_InformationResponseList_RL_ReconfRspFDD = -1;  /* RL_InformationResponseList_RL_ReconfRspFDD */
static int hf_rnsap_id_RL_InformationResponse_RL_ReconfRspTDD = -1;  /* RL_InformationResponse_RL_ReconfRspTDD */
static int hf_rnsap_id_RL_InformationResponseList_RL_SetupRspFDD = -1;  /* RL_InformationResponseList_RL_SetupRspFDD */
static int hf_rnsap_id_RL_ReconfigurationFailure_RL_ReconfFail = -1;  /* RL_ReconfigurationFailure_RL_ReconfFail */
static int hf_rnsap_id_RL_Set_InformationItem_DM_Rprt = -1;  /* RL_Set_InformationItem_DM_Rprt */
static int hf_rnsap_id_RL_Set_InformationItem_DM_Rqst = -1;  /* RL_Set_InformationItem_DM_Rqst */
static int hf_rnsap_id_RL_Set_InformationItem_DM_Rsp = -1;  /* RL_Set_InformationItem_DM_Rsp */
static int hf_rnsap_id_RL_Set_Information_RL_FailureInd = -1;  /* RL_Set_Information_RL_FailureInd */
static int hf_rnsap_id_RL_Set_Information_RL_RestoreInd = -1;  /* RL_Set_Information_RL_RestoreInd */
static int hf_rnsap_id_RL_Set_Successful_InformationItem_DM_Fail = -1;  /* RL_Set_Successful_InformationItem_DM_Fail */
static int hf_rnsap_id_RL_Set_Unsuccessful_InformationItem_DM_Fail = -1;  /* RL_Set_Unsuccessful_InformationItem_DM_Fail */
static int hf_rnsap_id_RL_Set_Unsuccessful_InformationItem_DM_Fail_Ind = -1;  /* RL_Set_Unsuccessful_InformationItem_DM_Fail_Ind */
static int hf_rnsap_id_RL_Successful_InformationItem_DM_Fail = -1;  /* RL_Successful_InformationItem_DM_Fail */
static int hf_rnsap_id_RL_Unsuccessful_InformationItem_DM_Fail = -1;  /* RL_Unsuccessful_InformationItem_DM_Fail */
static int hf_rnsap_id_RL_Unsuccessful_InformationItem_DM_Fail_Ind = -1;  /* RL_Unsuccessful_InformationItem_DM_Fail_Ind */
static int hf_rnsap_id_ReportCharacteristics = -1;  /* ReportCharacteristics */
static int hf_rnsap_id_Reporting_Object_RL_FailureInd = -1;  /* Reporting_Object_RL_FailureInd */
static int hf_rnsap_id_Reporing_Object_RL_RestoreInd = -1;  /* Reporting_Object_RL_RestoreInd */
static int hf_rnsap_id_RT_Load_Value = -1;        /* RT_Load_Value */
static int hf_rnsap_id_RT_Load_Value_IncrDecrThres = -1;  /* RT_Load_Value_IncrDecrThres */
static int hf_rnsap_id_S_RNTI = -1;               /* S_RNTI */
static int hf_rnsap_id_ResetIndicator = -1;       /* ResetIndicator */
static int hf_rnsap_id_RNC_ID = -1;               /* RNC_ID */
static int hf_rnsap_id_SAI = -1;                  /* SAI */
static int hf_rnsap_id_SRNC_ID = -1;              /* RNC_ID */
static int hf_rnsap_id_SuccessfulRL_InformationResponse_RL_AdditionFailureFDD = -1;  /* SuccessfulRL_InformationResponse_RL_AdditionFailureFDD */
static int hf_rnsap_id_SuccessfulRL_InformationResponse_RL_SetupFailureFDD = -1;  /* SuccessfulRL_InformationResponse_RL_SetupFailureFDD */
static int hf_rnsap_id_TransportBearerID = -1;    /* TransportBearerID */
static int hf_rnsap_id_TransportBearerRequestIndicator = -1;  /* TransportBearerRequestIndicator */
static int hf_rnsap_id_TransportLayerAddress = -1;  /* TransportLayerAddress */
static int hf_rnsap_id_TypeOfError = -1;          /* TypeOfError */
static int hf_rnsap_id_UC_ID = -1;                /* UC_ID */
static int hf_rnsap_id_UL_CCTrCH_AddInformation_RL_ReconfPrepTDD = -1;  /* UL_CCTrCH_AddInformation_RL_ReconfPrepTDD */
static int hf_rnsap_id_UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD = -1;  /* UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD */
static int hf_rnsap_id_UL_CCTrCH_InformationItem_RL_SetupRqstTDD = -1;  /* UL_CCTrCH_InformationItem_RL_SetupRqstTDD */
static int hf_rnsap_id_UL_CCTrCH_InformationList_RL_SetupRqstTDD = -1;  /* UL_CCTrCH_InformationList_RL_SetupRqstTDD */
static int hf_rnsap_id_UL_CCTrCH_InformationListIE_PhyChReconfRqstTDD = -1;  /* UL_CCTrCH_InformationListIE_PhyChReconfRqstTDD */
static int hf_rnsap_id_UL_CCTrCH_InformationListIE_RL_AdditionRspTDD = -1;  /* UL_CCTrCHInformationListIE_RL_AdditionRspTDD */
static int hf_rnsap_id_UL_CCTrCH_InformationListIE_RL_ReconfReadyTDD = -1;  /* UL_CCTrCHInformationListIE_RL_ReconfReadyTDD */
static int hf_rnsap_id_UL_CCTrCH_InformationListIE_RL_SetupRspTDD = -1;  /* UL_CCTrCHInformationListIE_RL_SetupRspTDD */
static int hf_rnsap_id_UL_DPCH_Information_RL_ReconfPrepFDD = -1;  /* UL_DPCH_Information_RL_ReconfPrepFDD */
static int hf_rnsap_id_UL_DPCH_Information_RL_ReconfRqstFDD = -1;  /* UL_DPCH_Information_RL_ReconfRqstFDD */
static int hf_rnsap_id_UL_DPCH_Information_RL_SetupRqstFDD = -1;  /* UL_DPCH_Information_RL_SetupRqstFDD */
static int hf_rnsap_id_UL_DPCH_InformationItem_PhyChReconfRqstTDD = -1;  /* UL_DPCH_InformationItem_PhyChReconfRqstTDD */
static int hf_rnsap_id_UL_DPCH_InformationItem_RL_AdditionRspTDD = -1;  /* UL_DPCH_InformationItem_RL_AdditionRspTDD */
static int hf_rnsap_id_UL_DPCH_InformationItem_RL_SetupRspTDD = -1;  /* UL_DPCH_InformationItem_RL_SetupRspTDD */
static int hf_rnsap_id_UL_DPCH_InformationAddListIE_RL_ReconfReadyTDD = -1;  /* UL_DPCH_InformationAddListIE_RL_ReconfReadyTDD */
static int hf_rnsap_id_UL_SIRTarget = -1;         /* UL_SIR */
static int hf_rnsap_id_URA_Information = -1;      /* URA_Information */
static int hf_rnsap_id_UnsuccessfulRL_InformationResponse_RL_AdditionFailureFDD = -1;  /* UnsuccessfulRL_InformationResponse_RL_AdditionFailureFDD */
static int hf_rnsap_id_UnsuccessfulRL_InformationResponse_RL_SetupFailureFDD = -1;  /* UnsuccessfulRL_InformationResponse_RL_SetupFailureFDD */
static int hf_rnsap_id_UnsuccessfulRL_InformationResponse_RL_SetupFailureTDD = -1;  /* UnsuccessfulRL_InformationResponse_RL_SetupFailureTDD */
static int hf_rnsap_id_Active_Pattern_Sequence_Information = -1;  /* Active_Pattern_Sequence_Information */
static int hf_rnsap_id_AdjustmentRatio = -1;      /* ScaledAdjustmentRatio */
static int hf_rnsap_id_CauseLevel_RL_AdditionFailureFDD = -1;  /* CauseLevel_RL_AdditionFailureFDD */
static int hf_rnsap_id_CauseLevel_RL_AdditionFailureTDD = -1;  /* CauseLevel_RL_AdditionFailureTDD */
static int hf_rnsap_id_CauseLevel_RL_ReconfFailure = -1;  /* CauseLevel_RL_ReconfFailure */
static int hf_rnsap_id_CauseLevel_RL_SetupFailureFDD = -1;  /* CauseLevel_RL_SetupFailureFDD */
static int hf_rnsap_id_CauseLevel_RL_SetupFailureTDD = -1;  /* CauseLevel_RL_SetupFailureTDD */
static int hf_rnsap_id_DL_CCTrCH_InformationDeleteItem_RL_ReconfPrepTDD = -1;  /* DL_CCTrCH_InformationDeleteItem_RL_ReconfPrepTDD */
static int hf_rnsap_id_DL_CCTrCH_InformationModifyItem_RL_ReconfPrepTDD = -1;  /* DL_CCTrCH_InformationModifyItem_RL_ReconfPrepTDD */
static int hf_rnsap_id_DL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD = -1;  /* DL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD */
static int hf_rnsap_id_DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD = -1;  /* DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD */
static int hf_rnsap_id_DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD = -1;  /* DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD */
static int hf_rnsap_id_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD = -1;  /* DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD */
static int hf_rnsap_id_DL_DPCH_InformationAddListIE_RL_ReconfReadyTDD = -1;  /* DL_DPCH_InformationAddListIE_RL_ReconfReadyTDD */
static int hf_rnsap_id_DL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD = -1;  /* DL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD */
static int hf_rnsap_id_DL_DPCH_InformationModifyListIE_RL_ReconfReadyTDD = -1;  /* DL_DPCH_InformationModifyListIE_RL_ReconfReadyTDD */
static int hf_rnsap_id_DSCHs_to_Add_TDD = -1;     /* DSCH_TDD_Information */
static int hf_rnsap_id_DSCH_DeleteList_RL_ReconfPrepTDD = -1;  /* DSCH_DeleteList_RL_ReconfPrepTDD */
static int hf_rnsap_id_DSCH_InformationListIE_RL_AdditionRspTDD = -1;  /* DSCH_InformationListIE_RL_AdditionRspTDD */
static int hf_rnsap_id_DSCH_InformationListIEs_RL_SetupRspTDD = -1;  /* DSCH_InformationListIEs_RL_SetupRspTDD */
static int hf_rnsap_id_DSCH_TDD_Information = -1;  /* DSCH_TDD_Information */
static int hf_rnsap_id_DSCH_ModifyList_RL_ReconfPrepTDD = -1;  /* DSCH_ModifyList_RL_ReconfPrepTDD */
static int hf_rnsap_id_DSCHToBeAddedOrModifiedList_RL_ReconfReadyTDD = -1;  /* DSCHToBeAddedOrModifiedList_RL_ReconfReadyTDD */
static int hf_rnsap_id_GA_Cell = -1;              /* GA_Cell */
static int hf_rnsap_id_GA_CellAdditionalShapes = -1;  /* GA_CellAdditionalShapes */
static int hf_rnsap_id_Transmission_Gap_Pattern_Sequence_Information = -1;  /* Transmission_Gap_Pattern_Sequence_Information */
static int hf_rnsap_id_UL_CCTrCH_DeleteInformation_RL_ReconfPrepTDD = -1;  /* UL_CCTrCH_DeleteInformation_RL_ReconfPrepTDD */
static int hf_rnsap_id_UL_CCTrCH_ModifyInformation_RL_ReconfPrepTDD = -1;  /* UL_CCTrCH_ModifyInformation_RL_ReconfPrepTDD */
static int hf_rnsap_id_UL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD = -1;  /* UL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD */
static int hf_rnsap_id_UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD = -1;  /* UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD */
static int hf_rnsap_id_UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD = -1;  /* UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD */
static int hf_rnsap_id_UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD = -1;  /* UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD */
static int hf_rnsap_id_UL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD = -1;  /* UL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD */
static int hf_rnsap_id_UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD = -1;  /* UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD */
static int hf_rnsap_id_UL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD = -1;  /* UL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD */
static int hf_rnsap_id_UL_DPCH_InformationModifyListIE_RL_ReconfReadyTDD = -1;  /* UL_DPCH_InformationModifyListIE_RL_ReconfReadyTDD */
static int hf_rnsap_id_UnsuccessfulRL_InformationResponse_RL_AdditionFailureTDD = -1;  /* UnsuccessfulRL_InformationResponse_RL_AdditionFailureTDD */
static int hf_rnsap_id_USCHs_to_Add = -1;         /* USCH_Information */
static int hf_rnsap_id_USCH_DeleteList_RL_ReconfPrepTDD = -1;  /* USCH_DeleteList_RL_ReconfPrepTDD */
static int hf_rnsap_id_USCH_InformationListIE_RL_AdditionRspTDD = -1;  /* USCH_InformationListIE_RL_AdditionRspTDD */
static int hf_rnsap_id_USCH_InformationListIEs_RL_SetupRspTDD = -1;  /* USCH_InformationListIEs_RL_SetupRspTDD */
static int hf_rnsap_id_USCH_Information = -1;     /* USCH_Information */
static int hf_rnsap_id_USCH_ModifyList_RL_ReconfPrepTDD = -1;  /* USCH_ModifyList_RL_ReconfPrepTDD */
static int hf_rnsap_id_USCHToBeAddedOrModifiedList_RL_ReconfReadyTDD = -1;  /* USCHToBeAddedOrModifiedList_RL_ReconfReadyTDD */
static int hf_rnsap_id_DL_Physical_Channel_Information_RL_SetupRqstTDD = -1;  /* DL_Physical_Channel_Information_RL_SetupRqstTDD */
static int hf_rnsap_id_UL_Physical_Channel_Information_RL_SetupRqstTDD = -1;  /* UL_Physical_Channel_Information_RL_SetupRqstTDD */
static int hf_rnsap_id_ClosedLoopMode1_SupportIndicator = -1;  /* ClosedLoopMode1_SupportIndicator */
static int hf_rnsap_id_STTD_SupportIndicator = -1;  /* STTD_SupportIndicator */
static int hf_rnsap_id_CFNReportingIndicator = -1;  /* FNReportingIndicator */
static int hf_rnsap_id_CNOriginatedPage_PagingRqst = -1;  /* CNOriginatedPage_PagingRqst */
static int hf_rnsap_id_InnerLoopDLPCStatus = -1;  /* InnerLoopDLPCStatus */
static int hf_rnsap_id_PropagationDelay = -1;     /* PropagationDelay */
static int hf_rnsap_id_RxTimingDeviationForTA = -1;  /* RxTimingDeviationForTA */
static int hf_rnsap_id_timeSlot_ISCP = -1;        /* DL_TimeSlot_ISCP_Info */
static int hf_rnsap_id_CCTrCH_InformationItem_RL_FailureInd = -1;  /* CCTrCH_InformationItem_RL_FailureInd */
static int hf_rnsap_id_CCTrCH_InformationItem_RL_RestoreInd = -1;  /* CCTrCH_InformationItem_RL_RestoreInd */
static int hf_rnsap_id_CommonMeasurementAccuracy = -1;  /* CommonMeasurementAccuracy */
static int hf_rnsap_id_CommonMeasurementObjectType_CM_Rprt = -1;  /* CommonMeasurementObjectType_CM_Rprt */
static int hf_rnsap_id_CommonMeasurementObjectType_CM_Rqst = -1;  /* CommonMeasurementObjectType_CM_Rqst */
static int hf_rnsap_id_CommonMeasurementObjectType_CM_Rsp = -1;  /* CommonMeasurementObjectType_CM_Rsp */
static int hf_rnsap_id_CommonMeasurementType = -1;  /* CommonMeasurementType */
static int hf_rnsap_id_CongestionCause = -1;      /* CongestionCause */
static int hf_rnsap_id_SFN = -1;                  /* SFN */
static int hf_rnsap_id_SFNReportingIndicator = -1;  /* FNReportingIndicator */
static int hf_rnsap_id_InformationExchangeID = -1;  /* InformationExchangeID */
static int hf_rnsap_id_InformationExchangeObjectType_InfEx_Rprt = -1;  /* InformationExchangeObjectType_InfEx_Rprt */
static int hf_rnsap_id_InformationExchangeObjectType_InfEx_Rqst = -1;  /* InformationExchangeObjectType_InfEx_Rqst */
static int hf_rnsap_id_InformationExchangeObjectType_InfEx_Rsp = -1;  /* InformationReportCharacteristics */
static int hf_rnsap_id_InformationReportCharacteristics = -1;  /* InformationReportCharacteristics */
static int hf_rnsap_id_InformationType = -1;      /* InformationType */
static int hf_rnsap_id_neighbouring_LCR_TDD_CellInformation = -1;  /* Neighbouring_LCR_TDD_CellInformation */
static int hf_rnsap_id_DL_Timeslot_ISCP_LCR_Information_RL_SetupRqstTDD = -1;  /* DL_TimeSlot_ISCP_LCR_Information */
static int hf_rnsap_id_RL_LCR_InformationResponse_RL_SetupRspTDD = -1;  /* RL_LCR_InformationResponse_RL_SetupRspTDD */
static int hf_rnsap_id_UL_CCTrCH_LCR_InformationListIE_RL_SetupRspTDD = -1;  /* UL_LCR_CCTrCHInformationListIE_RL_SetupRspTDD */
static int hf_rnsap_id_UL_DPCH_LCR_InformationItem_RL_SetupRspTDD = -1;  /* UL_DPCH_LCR_InformationItem_RL_SetupRspTDD */
static int hf_rnsap_id_DL_CCTrCH_LCR_InformationListIE_RL_SetupRspTDD = -1;  /* DL_CCTrCH_LCR_InformationListIE_RL_SetupRspTDD */
static int hf_rnsap_id_DL_DPCH_LCR_InformationItem_RL_SetupRspTDD = -1;  /* DL_DPCH_LCR_InformationItem_RL_SetupRspTDD */
static int hf_rnsap_id_DSCH_LCR_InformationListIEs_RL_SetupRspTDD = -1;  /* DSCH_LCR_InformationListIEs_RL_SetupRspTDD */
static int hf_rnsap_id_USCH_LCR_InformationListIEs_RL_SetupRspTDD = -1;  /* USCH_LCR_InformationListIEs_RL_SetupRspTDD */
static int hf_rnsap_id_DL_Timeslot_ISCP_LCR_Information_RL_AdditionRqstTDD = -1;  /* DL_TimeSlot_ISCP_LCR_Information */
static int hf_rnsap_id_RL_LCR_InformationResponse_RL_AdditionRspTDD = -1;  /* RL_LCR_InformationResponse_RL_AdditionRspTDD */
static int hf_rnsap_id_UL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD = -1;  /* UL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD */
static int hf_rnsap_id_UL_DPCH_LCR_InformationItem_RL_AdditionRspTDD = -1;  /* UL_DPCH_LCR_InformationItem_RL_AdditionRspTDD */
static int hf_rnsap_id_DL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD = -1;  /* DL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD */
static int hf_rnsap_id_DL_DPCH_LCR_InformationItem_RL_AdditionRspTDD = -1;  /* DL_DPCH_LCR_InformationItem_RL_AdditionRspTDD */
static int hf_rnsap_id_DSCH_LCR_InformationListIEs_RL_AdditionRspTDD = -1;  /* DSCH_LCR_InformationListIEs_RL_AdditionRspTDD */
static int hf_rnsap_id_USCH_LCR_InformationListIEs_RL_AdditionRspTDD = -1;  /* USCH_LCR_InformationListIEs_RL_AdditionRspTDD */
static int hf_rnsap_id_UL_DPCH_LCR_InformationAddListIE_RL_ReconfReadyTDD = -1;  /* UL_DPCH_LCR_InformationAddList_RL_ReconfReadyTDD */
static int hf_rnsap_id_UL_Timeslot_LCR_InformationModifyList_RL_ReconfReadyTDD = -1;  /* UL_TimeslotLCR_InformationModifyList_RL_ReconfReadyTDD */
static int hf_rnsap_id_DL_DPCH_LCR_InformationAddListIE_RL_ReconfReadyTDD = -1;  /* DL_DPCH_LCR_InformationAddList_RL_ReconfReadyTDD */
static int hf_rnsap_id_DL_Timeslot_LCR_InformationModifyList_RL_ReconfReadyTDD = -1;  /* DL_TimeslotLCR_InformationModifyList_RL_ReconfReadyTDD */
static int hf_rnsap_id_UL_Timeslot_LCR_InformationList_PhyChReconfRqstTDD = -1;  /* UL_TimeslotLCR_InformationList_PhyChReconfRqstTDD */
static int hf_rnsap_id_DL_Timeslot_LCR_InformationList_PhyChReconfRqstTDD = -1;  /* DL_TimeslotLCR_InformationList_PhyChReconfRqstTDD */
static int hf_rnsap_id_timeSlot_ISCP_LCR_List_DL_PC_Rqst_TDD = -1;  /* DL_TimeSlot_ISCP_LCR_Information */
static int hf_rnsap_id_TSTD_Support_Indicator_RL_SetupRqstTDD = -1;  /* TSTD_Support_Indicator */
static int hf_rnsap_id_RestrictionStateIndicator = -1;  /* RestrictionStateIndicator */
static int hf_rnsap_id_Load_Value = -1;           /* Load_Value */
static int hf_rnsap_id_Load_Value_IncrDecrThres = -1;  /* Load_Value_IncrDecrThres */
static int hf_rnsap_id_OnModification = -1;       /* OnModification */
static int hf_rnsap_id_Received_Total_Wideband_Power_Value = -1;  /* Received_Total_Wideband_Power_Value */
static int hf_rnsap_id_Received_Total_Wideband_Power_Value_IncrDecrThres = -1;  /* SFNSFNMeasurementThresholdInformation */
static int hf_rnsap_id_SFNSFNMeasurementThresholdInformation = -1;  /* SFNSFNMeasurementThresholdInformation */
static int hf_rnsap_id_Transmitted_Carrier_Power_Value = -1;  /* Transmitted_Carrier_Power_Value */
static int hf_rnsap_id_Transmitted_Carrier_Power_Value_IncrDecrThres = -1;  /* Transmitted_Carrier_Power_Value_IncrDecrThres */
static int hf_rnsap_id_TUTRANGPSMeasurementThresholdInformation = -1;  /* UL_Timeslot_ISCP_Value */
static int hf_rnsap_id_UL_Timeslot_ISCP_Value = -1;  /* UL_Timeslot_ISCP_Value */
static int hf_rnsap_id_UL_Timeslot_ISCP_Value_IncrDecrThres = -1;  /* UL_Timeslot_ISCP_Value_IncrDecrThres */
static int hf_rnsap_id_Rx_Timing_Deviation_Value_LCR = -1;  /* Rx_Timing_Deviation_Value_LCR */
static int hf_rnsap_id_DPC_Mode_Change_SupportIndicator = -1;  /* DPC_Mode_Change_SupportIndicator */
static int hf_rnsap_id_PrimaryCCPCH_RSCP_RL_ReconfPrepTDD = -1;  /* PrimaryCCPCH_RSCP */
static int hf_rnsap_id_DL_TimeSlot_ISCP_Info_RL_ReconfPrepTDD = -1;  /* DL_TimeSlot_ISCP_Info */
static int hf_rnsap_id_DL_Timeslot_ISCP_LCR_Information_RL_ReconfPrepTDD = -1;  /* DL_TimeSlot_ISCP_LCR_Information */
static int hf_rnsap_id_DSCH_RNTI = -1;            /* DSCH_RNTI */
static int hf_rnsap_id_DL_PowerBalancing_Information = -1;  /* DL_PowerBalancing_Information */
static int hf_rnsap_id_DL_PowerBalancing_ActivationIndicator = -1;  /* DL_PowerBalancing_ActivationIndicator */
static int hf_rnsap_id_DL_PowerBalancing_UpdatedIndicator = -1;  /* DL_PowerBalancing_UpdatedIndicator */
static int hf_rnsap_id_DL_ReferencePowerInformation = -1;  /* DL_ReferencePowerInformation */
static int hf_rnsap_id_Enhanced_PrimaryCPICH_EcNo = -1;  /* Enhanced_PrimaryCPICH_EcNo */
static int hf_rnsap_id_IPDL_TDD_ParametersLCR = -1;  /* IPDL_TDD_ParametersLCR */
static int hf_rnsap_id_CellCapabilityContainer_FDD = -1;  /* CellCapabilityContainer_FDD */
static int hf_rnsap_id_CellCapabilityContainer_TDD = -1;  /* CellCapabilityContainer_TDD */
static int hf_rnsap_id_CellCapabilityContainer_TDD_LCR = -1;  /* CellCapabilityContainer_TDD_LCR */
static int hf_rnsap_id_RL_Specific_DCH_Info = -1;  /* RL_Specific_DCH_Info */
static int hf_rnsap_id_RL_ReconfigurationRequestFDD_RL_InformationList = -1;  /* RL_ReconfigurationRequestFDD_RL_InformationList */
static int hf_rnsap_id_RL_ReconfigurationRequestFDD_RL_Information_IEs = -1;  /* RL_ReconfigurationRequestFDD_RL_Information_IEs */
static int hf_rnsap_id_RL_ReconfigurationRequestTDD_RL_Information = -1;  /* RL_ReconfigurationRequestTDD_RL_Information */
static int hf_rnsap_id_CommonTransportChannelResourcesInitialisationNotRequired = -1;  /* CommonTransportChannelResourcesInitialisationNotRequired */
static int hf_rnsap_id_DelayedActivation = -1;    /* DelayedActivation */
static int hf_rnsap_id_DelayedActivationList_RL_ActivationCmdFDD = -1;  /* DelayedActivationInformationList_RL_ActivationCmdFDD */
static int hf_rnsap_id_DelayedActivationInformation_RL_ActivationCmdFDD = -1;  /* DelayedActivationInformation_RL_ActivationCmdFDD */
static int hf_rnsap_id_DelayedActivationList_RL_ActivationCmdTDD = -1;  /* DelayedActivationInformationList_RL_ActivationCmdTDD */
static int hf_rnsap_id_DelayedActivationInformation_RL_ActivationCmdTDD = -1;  /* DelayedActivationInformation_RL_ActivationCmdTDD */
static int hf_rnsap_id_neighbouringTDDCellMeasurementInformationLCR = -1;  /* NeighbouringTDDCellMeasurementInformationLCR */
static int hf_rnsap_id_UL_SIR_Target_CCTrCH_InformationItem_RL_SetupRspTDD = -1;  /* UL_SIR */
static int hf_rnsap_id_UL_SIR_Target_CCTrCH_LCR_InformationItem_RL_SetupRspTDD = -1;  /* UL_SIR */
static int hf_rnsap_id_PrimCCPCH_RSCP_DL_PC_RqstTDD = -1;  /* PrimaryCCPCH_RSCP */
static int hf_rnsap_id_HSDSCH_FDD_Information = -1;  /* HSDSCH_FDD_Information */
static int hf_rnsap_id_HSDSCH_FDD_Information_Response = -1;  /* HSDSCH_FDD_Information_Response */
static int hf_rnsap_id_HSDSCH_FDD_Update_Information = -1;  /* HSDSCH_FDD_Update_Information */
static int hf_rnsap_id_HSDSCH_Information_to_Modify = -1;  /* HSDSCH_Information_to_Modify */
static int hf_rnsap_id_HSDSCHMacdFlowSpecificInformationList_RL_PreemptRequiredInd = -1;  /* HSDSCHMacdFlowSpecificInformationList_RL_PreemptRequiredInd */
static int hf_rnsap_id_HSDSCHMacdFlowSpecificInformationItem_RL_PreemptRequiredInd = -1;  /* HSDSCHMacdFlowSpecificInformationItem_RL_PreemptRequiredInd */
static int hf_rnsap_id_HSDSCH_RNTI = -1;          /* HSDSCH_RNTI */
static int hf_rnsap_id_HSDSCH_TDD_Information = -1;  /* HSDSCH_TDD_Information */
static int hf_rnsap_id_HSDSCH_TDD_Information_Response = -1;  /* HSDSCH_TDD_Information_Response */
static int hf_rnsap_id_HSDSCH_TDD_Update_Information = -1;  /* HSDSCH_TDD_Update_Information */
static int hf_rnsap_id_HSPDSCH_RL_ID = -1;        /* RL_ID */
static int hf_rnsap_id_HSDSCH_MACdFlows_to_Add = -1;  /* HSDSCH_MACdFlows_Information */
static int hf_rnsap_id_HSDSCH_MACdFlows_to_Delete = -1;  /* HSDSCH_MACdFlows_to_Delete */
static int hf_rnsap_id_Angle_Of_Arrival_Value_LCR = -1;  /* Angle_Of_Arrival_Value_LCR */
static int hf_rnsap_id_TrafficClass = -1;         /* TrafficClass */
static int hf_rnsap_id_PDSCH_RL_ID = -1;          /* RL_ID */
static int hf_rnsap_id_TimeSlot_RL_SetupRspTDD = -1;  /* TimeSlot */
static int hf_rnsap_id_GERAN_Cell_Capability = -1;  /* GERAN_Cell_Capability */
static int hf_rnsap_id_GERAN_Classmark = -1;      /* GERAN_Classmark */
static int hf_rnsap_id_DSCH_InitialWindowSize = -1;  /* DSCH_InitialWindowSize */
static int hf_rnsap_id_UL_Synchronisation_Parameters_LCR = -1;  /* UL_Synchronisation_Parameters_LCR */
static int hf_rnsap_id_SNA_Information = -1;      /* SNA_Information */
static int hf_rnsap_id_MAChs_ResetIndicator = -1;  /* MAChs_ResetIndicator */
static int hf_rnsap_id_TDD_DL_DPCH_TimeSlotFormatModifyItem_LCR_RL_ReconfReadyTDD = -1;  /* TDD_DL_DPCH_TimeSlotFormat_LCR */
static int hf_rnsap_id_TDD_UL_DPCH_TimeSlotFormatModifyItem_LCR_RL_ReconfReadyTDD = -1;  /* TDD_UL_DPCH_TimeSlotFormat_LCR */
static int hf_rnsap_id_TDD_TPC_UplinkStepSize_LCR_RL_SetupRqstTDD = -1;  /* TDD_TPC_UplinkStepSize_LCR */
static int hf_rnsap_id_UL_CCTrCH_InformationList_RL_AdditionRqstTDD = -1;  /* UL_CCTrCH_InformationList_RL_AdditionRqstTDD */
static int hf_rnsap_id_UL_CCTrCH_InformationItem_RL_AdditionRqstTDD = -1;  /* UL_CCTrCH_InformationItem_RL_AdditionRqstTDD */
static int hf_rnsap_id_DL_CCTrCH_InformationList_RL_AdditionRqstTDD = -1;  /* DL_CCTrCH_InformationList_RL_AdditionRqstTDD */
static int hf_rnsap_id_DL_CCTrCH_InformationItem_RL_AdditionRqstTDD = -1;  /* DL_CCTrCH_InformationItem_RL_AdditionRqstTDD */
static int hf_rnsap_id_TDD_TPC_UplinkStepSize_InformationAdd_LCR_RL_ReconfPrepTDD = -1;  /* TDD_TPC_UplinkStepSize_LCR */
static int hf_rnsap_id_TDD_TPC_UplinkStepSize_InformationModify_LCR_RL_ReconfPrepTDD = -1;  /* TDD_TPC_UplinkStepSize_LCR */
static int hf_rnsap_id_TDD_TPC_DownlinkStepSize_InformationAdd_RL_ReconfPrepTDD = -1;  /* TDD_TPC_DownlinkStepSize */
static int hf_rnsap_id_TDD_TPC_DownlinkStepSize_InformationModify_RL_ReconfPrepTDD = -1;  /* TDD_TPC_DownlinkStepSize */
static int hf_rnsap_id_UL_TimingAdvanceCtrl_LCR = -1;  /* UL_TimingAdvanceCtrl_LCR */
static int hf_rnsap_id_HSPDSCH_Timeslot_InformationList_PhyChReconfRqstTDD = -1;  /* HSPDSCH_Timeslot_InformationList_PhyChReconfRqstTDD */
static int hf_rnsap_id_HSPDSCH_Timeslot_InformationListLCR_PhyChReconfRqstTDD = -1;  /* HSPDSCH_Timeslot_InformationListLCR_PhyChReconfRqstTDD */
static int hf_rnsap_id_HS_SICH_Reception_Quality = -1;  /* HS_SICH_Reception_Quality_Value */
static int hf_rnsap_id_HS_SICH_Reception_Quality_Measurement_Value = -1;  /* HS_SICH_Reception_Quality_Measurement_Value */
static int hf_rnsap_id_HSSICH_Info_DM_Rprt = -1;  /* HS_SICH_ID */
static int hf_rnsap_id_HSSICH_Info_DM_Rqst = -1;  /* HSSICH_Info_DM_Rqst */
static int hf_rnsap_id_HSSICH_Info_DM = -1;       /* HS_SICH_ID */
static int hf_rnsap_id_CCTrCH_Maximum_DL_Power_RL_SetupRspTDD = -1;  /* DL_Power */
static int hf_rnsap_id_CCTrCH_Minimum_DL_Power_RL_SetupRspTDD = -1;  /* DL_Power */
static int hf_rnsap_id_CCTrCH_Maximum_DL_Power_RL_AdditionRspTDD = -1;  /* DL_Power */
static int hf_rnsap_id_CCTrCH_Minimum_DL_Power_RL_AdditionRspTDD = -1;  /* DL_Power */
static int hf_rnsap_id_CCTrCH_Maximum_DL_Power_RL_ReconfReadyTDD = -1;  /* DL_Power */
static int hf_rnsap_id_CCTrCH_Minimum_DL_Power_RL_ReconfReadyTDD = -1;  /* DL_Power */
static int hf_rnsap_id_Maximum_DL_Power_TimeslotLCR_InformationModifyItem_RL_ReconfReadyTDD = -1;  /* DL_Power */
static int hf_rnsap_id_Minimum_DL_Power_TimeslotLCR_InformationModifyItem_RL_ReconfReadyTDD = -1;  /* DL_Power */
static int hf_rnsap_id_DL_CCTrCH_InformationList_RL_ReconfRspTDD = -1;  /* DL_CCTrCH_InformationList_RL_ReconfRspTDD */
static int hf_rnsap_id_DL_DPCH_InformationModifyItem_LCR_RL_ReconfRspTDD = -1;  /* DL_DPCH_InformationModifyItem_LCR_RL_ReconfRspTDD */
static int hf_rnsap_id_Maximum_DL_Power_TimeslotLCR_InformationItem = -1;  /* DL_Power */
static int hf_rnsap_id_Minimum_DL_Power_TimeslotLCR_InformationItem = -1;  /* DL_Power */
static int hf_rnsap_id_TDD_Support_8PSK = -1;     /* Support_8PSK */
static int hf_rnsap_id_TDD_maxNrDLPhysicalchannels = -1;  /* MaxNrDLPhysicalchannelsTS */
static int hf_rnsap_id_ExtendedGSMCellIndividualOffset = -1;  /* ExtendedGSMCellIndividualOffset */
static int hf_rnsap_id_RL_ParameterUpdateIndicationFDD_RL_InformationList = -1;  /* RL_ParameterUpdateIndicationFDD_RL_InformationList */
static int hf_rnsap_id_Primary_CPICH_Usage_For_Channel_Estimation = -1;  /* Primary_CPICH_Usage_For_Channel_Estimation */
static int hf_rnsap_id_Secondary_CPICH_Information = -1;  /* Secondary_CPICH_Information */
static int hf_rnsap_id_Secondary_CPICH_Information_Change = -1;  /* Secondary_CPICH_Information_Change */
static int hf_rnsap_id_RL_ParameterUpdateIndicationFDD_RL_Information_Item = -1;  /* RL_ParameterUpdateIndicationFDD_RL_Information_Item */
static int hf_rnsap_id_Phase_Reference_Update_Indicator = -1;  /* Phase_Reference_Update_Indicator */
static int hf_rnsap_id_Unidirectional_DCH_Indicator = -1;  /* Unidirectional_DCH_Indicator */
static int hf_rnsap_id_RL_Information_RL_ReconfPrepTDD = -1;  /* RL_Information_RL_ReconfPrepTDD */
static int hf_rnsap_id_Multiple_RL_InformationResponse_RL_ReconfReadyTDD = -1;  /* Multiple_RL_InformationResponse_RL_ReconfReadyTDD */
static int hf_rnsap_id_RL_ReconfigurationResponseTDD_RL_Information = -1;  /* Multiple_RL_InformationResponse_RL_ReconfRspTDD */
static int hf_rnsap_id_Satellite_Almanac_Information_ExtItem = -1;  /* Satellite_Almanac_Information_ExtItem */
static int hf_rnsap_id_HSDSCH_Information_to_Modify_Unsynchronised = -1;  /* HSDSCH_Information_to_Modify_Unsynchronised */
static int hf_rnsap_id_TnlQos = -1;               /* TnlQos */
static int hf_rnsap_id_RTLoadValue = -1;          /* RTLoadValue */
static int hf_rnsap_id_NRTLoadInformationValue = -1;  /* NRTLoadInformationValue */
static int hf_rnsap_id_CellPortionID = -1;        /* CellPortionID */
static int hf_rnsap_id_UpPTSInterferenceValue = -1;  /* UpPTSInterferenceValue */
static int hf_rnsap_id_PrimaryCCPCH_RSCP_Delta = -1;  /* PrimaryCCPCH_RSCP_Delta */
static int hf_rnsap_id_UEMeasurementType = -1;    /* UEMeasurementType */
static int hf_rnsap_id_UEMeasurementTimeslotInfoHCR = -1;  /* UEMeasurementTimeslotInfoHCR */
static int hf_rnsap_id_UEMeasurementTimeslotInfoLCR = -1;  /* UEMeasurementTimeslotInfoLCR */
static int hf_rnsap_id_UEMeasurementReportCharacteristics = -1;  /* UEMeasurementReportCharacteristics */
static int hf_rnsap_id_UEMeasurementParameterModAllow = -1;  /* UEMeasurementParameterModAllow */
static int hf_rnsap_id_UEMeasurementValueInformation = -1;  /* UEMeasurementValueInformation */
static int hf_rnsap_id_InterfacesToTraceItem = -1;  /* InterfacesToTraceItem */
static int hf_rnsap_id_ListOfInterfacesToTrace = -1;  /* ListOfInterfacesToTrace */
static int hf_rnsap_id_TraceDepth = -1;           /* TraceDepth */
static int hf_rnsap_id_TraceRecordingSessionReference = -1;  /* TraceRecordingSessionReference */
static int hf_rnsap_id_TraceReference = -1;       /* TraceReference */
static int hf_rnsap_id_UEIdentity = -1;           /* UEIdentity */
static int hf_rnsap_id_NACC_Related_Data = -1;    /* NACC_Related_Data */
static int hf_rnsap_id_GSM_Cell_InfEx_Rqst = -1;  /* GSM_Cell_InfEx_Rqst */
static int hf_rnsap_id_MeasurementRecoveryBehavior = -1;  /* MeasurementRecoveryBehavior */
static int hf_rnsap_id_MeasurementRecoveryReportingIndicator = -1;  /* MeasurementRecoveryReportingIndicator */
static int hf_rnsap_id_MeasurementRecoverySupportIndicator = -1;  /* MeasurementRecoverySupportIndicator */
static int hf_rnsap_id_DL_DPCH_Power_Information_RL_ReconfPrepFDD = -1;  /* DL_DPCH_Power_Information_RL_ReconfPrepFDD */
static int hf_rnsap_id_F_DPCH_Information_RL_ReconfPrepFDD = -1;  /* F_DPCH_Information_RL_ReconfPrepFDD */
static int hf_rnsap_id_F_DPCH_Information_RL_SetupRqstFDD = -1;  /* F_DPCH_Information_RL_SetupRqstFDD */
static int hf_rnsap_id_MBMS_Bearer_Service_List = -1;  /* MBMS_Bearer_Service_List */
static int hf_rnsap_id_MBMS_Bearer_Service_List_InfEx_Rsp = -1;  /* MBMS_Bearer_Service_List_InfEx_Rsp */
static int hf_rnsap_id_Active_MBMS_Bearer_ServiceFDD = -1;  /* Active_MBMS_Bearer_Service_ListFDD */
static int hf_rnsap_id_Active_MBMS_Bearer_ServiceTDD = -1;  /* Active_MBMS_Bearer_Service_ListTDD */
static int hf_rnsap_id_Old_URA_ID = -1;           /* URA_ID */
static int hf_rnsap_id_UE_State = -1;             /* UE_State */
static int hf_rnsap_id_URA_ID = -1;               /* URA_ID */
static int hf_rnsap_id_HARQ_Preamble_Mode = -1;   /* HARQ_Preamble_Mode */
static int hf_rnsap_id_SynchronisationIndicator = -1;  /* SynchronisationIndicator */
static int hf_rnsap_id_UL_DPDCHIndicatorEDCH = -1;  /* UL_DPDCHIndicatorEDCH */
static int hf_rnsap_id_EDPCH_Information = -1;    /* EDPCH_Information_FDD */
static int hf_rnsap_id_RL_Specific_EDCH_Information = -1;  /* RL_Specific_EDCH_Information */
static int hf_rnsap_id_EDCH_RL_Indication = -1;   /* EDCH_RL_Indication */
static int hf_rnsap_id_EDCH_FDD_Information = -1;  /* EDCH_FDD_Information */
static int hf_rnsap_id_EDCH_RLSet_Id = -1;        /* RL_Set_ID */
static int hf_rnsap_id_Serving_EDCHRL_Id = -1;    /* EDCH_Serving_RL */
static int hf_rnsap_id_EDCH_FDD_DL_ControlChannelInformation = -1;  /* EDCH_FDD_DL_ControlChannelInformation */
static int hf_rnsap_id_EDCH_FDD_InformationResponse = -1;  /* EDCH_FDD_InformationResponse */
static int hf_rnsap_id_EDCH_MACdFlows_To_Add = -1;  /* RL_Specific_EDCH_Information */
static int hf_rnsap_id_EDCH_FDD_Information_To_Modify = -1;  /* EDCH_FDD_Information_To_Modify */
static int hf_rnsap_id_EDCH_MACdFlows_To_Delete = -1;  /* EDCH_MACdFlows_To_Delete */
static int hf_rnsap_id_EDPCH_Information_RLReconfRequest_FDD = -1;  /* EDPCH_Information_RLReconfRequest_FDD */
static int hf_rnsap_id_EDCH_MacdFlowSpecificInformationList_RL_PreemptRequiredInd = -1;  /* EDCH_MacdFlowSpecificInformationList_RL_PreemptRequiredInd */
static int hf_rnsap_id_EDCH_MacdFlowSpecificInformationItem_RL_PreemptRequiredInd = -1;  /* EDCH_MacdFlowSpecificInformationItem_RL_PreemptRequiredInd */
static int hf_rnsap_id_EDCH_MacdFlowSpecificInformationList_RL_CongestInd = -1;  /* EDCH_MacdFlowSpecificInformationList_RL_CongestInd */
static int hf_rnsap_id_EDCH_MacdFlowSpecificInformationItem_RL_CongestInd = -1;  /* EDCH_MacdFlowSpecificInformationItem_RL_CongestInd */
static int hf_rnsap_id_MBMS_Bearer_Service_Full_Address = -1;  /* MBMS_Bearer_Service_Full_Address */
static int hf_rnsap_id_Initial_DL_DPCH_TimingAdjustment = -1;  /* DL_DPCH_TimingAdjustment */
static int hf_rnsap_id_Initial_DL_DPCH_TimingAdjustment_Allowed = -1;  /* Initial_DL_DPCH_TimingAdjustment_Allowed */
static int hf_rnsap_id_User_Plane_Congestion_Fields_Inclusion = -1;  /* User_Plane_Congestion_Fields_Inclusion */
static int hf_rnsap_id_HARQ_Preamble_Mode_Activation_Indicator = -1;  /* HARQ_Preamble_Mode_Activation_Indicator */
static int hf_rnsap_id_multiple_DedicatedMeasurementValueList_TDD_DM_Rsp = -1;  /* Multiple_DedicatedMeasurementValueList_TDD_DM_Rsp */
static int hf_rnsap_id_multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp = -1;  /* Multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp */
static int hf_rnsap_id_ProvidedInformation = -1;  /* ProvidedInformation */
static int hf_rnsap_id_Active_MBMS_Bearer_ServiceFDD_PFL = -1;  /* Active_MBMS_Bearer_Service_ListTDD_PFL */
static int hf_rnsap_id_Active_MBMS_Bearer_ServiceTDD_PFL = -1;  /* Active_MBMS_Bearer_Service_ListTDD_PFL */
static int hf_rnsap_id_FrequencyBandIndicator = -1;  /* FrequencyBandIndicator */
static int hf_rnsap_protocolIEs = -1;             /* ProtocolIE_Container */
static int hf_rnsap_protocolExtensions = -1;      /* ProtocolExtensionContainer */
static int hf_rnsap_ul_ScramblingCode = -1;       /* UL_ScramblingCode */
static int hf_rnsap_minUL_ChannelisationCodeLength = -1;  /* MinUL_ChannelisationCodeLength */
static int hf_rnsap_maxNrOfUL_DPCHs = -1;         /* MaxNrOfUL_DPCHs */
static int hf_rnsap_ul_PunctureLimit = -1;        /* PunctureLimit */
static int hf_rnsap_ul_TFCS = -1;                 /* TFCS */
static int hf_rnsap_ul_DPCCH_SlotFormat = -1;     /* UL_DPCCH_SlotFormat */
static int hf_rnsap_ul_SIRTarget = -1;            /* UL_SIR */
static int hf_rnsap_diversityMode = -1;           /* DiversityMode */
static int hf_rnsap_not_Used_sSDT_CellIdLength = -1;  /* NULL */
static int hf_rnsap_not_Used_s_FieldLength = -1;  /* NULL */
static int hf_rnsap_iE_Extensions = -1;           /* ProtocolExtensionContainer */
static int hf_rnsap_tFCS = -1;                    /* TFCS */
static int hf_rnsap_dl_DPCH_SlotFormat = -1;      /* DL_DPCH_SlotFormat */
static int hf_rnsap_nrOfDLchannelisationcodes = -1;  /* NrOfDLchannelisationcodes */
static int hf_rnsap_tFCI_SignallingMode = -1;     /* TFCI_SignallingMode */
static int hf_rnsap_tFCI_Presence = -1;           /* TFCI_Presence */
static int hf_rnsap_multiplexingPosition = -1;    /* MultiplexingPosition */
static int hf_rnsap_powerOffsetInformation = -1;  /* PowerOffsetInformation_RL_SetupRqstFDD */
static int hf_rnsap_fdd_dl_TPC_DownlinkStepSize = -1;  /* FDD_TPC_DownlinkStepSize */
static int hf_rnsap_limitedPowerIncrease = -1;    /* LimitedPowerIncrease */
static int hf_rnsap_innerLoopDLPCStatus = -1;     /* InnerLoopDLPCStatus */
static int hf_rnsap_po1_ForTFCI_Bits = -1;        /* PowerOffset */
static int hf_rnsap_po2_ForTPC_Bits = -1;         /* PowerOffset */
static int hf_rnsap_po3_ForPilotBits = -1;        /* PowerOffset */
static int hf_rnsap_RL_InformationList_RL_SetupRqstFDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_rL_ID = -1;                   /* RL_ID */
static int hf_rnsap_c_ID = -1;                    /* C_ID */
static int hf_rnsap_firstRLS_indicator = -1;      /* FirstRLS_Indicator */
static int hf_rnsap_frameOffset = -1;             /* FrameOffset */
static int hf_rnsap_chipOffset = -1;              /* ChipOffset */
static int hf_rnsap_propagationDelay = -1;        /* PropagationDelay */
static int hf_rnsap_diversityControlField = -1;   /* DiversityControlField */
static int hf_rnsap_dl_InitialTX_Power = -1;      /* DL_Power */
static int hf_rnsap_primaryCPICH_EcNo = -1;       /* PrimaryCPICH_EcNo */
static int hf_rnsap_not_Used_sSDT_CellID = -1;    /* NULL */
static int hf_rnsap_transmitDiversityIndicator = -1;  /* TransmitDiversityIndicator */
static int hf_rnsap_powerOffsetInformation1 = -1;  /* PowerOffsetInformation_F_DPCH_RL_SetupRqstFDD */
static int hf_rnsap_maxNrTimeslots_UL = -1;       /* MaxNrTimeslots */
static int hf_rnsap_minimumSpreadingFactor_UL = -1;  /* MinimumSpreadingFactor */
static int hf_rnsap_maxNrULPhysicalchannels = -1;  /* MaxNrULPhysicalchannels */
static int hf_rnsap_maxNrTimeslots_DL = -1;       /* MaxNrTimeslots */
static int hf_rnsap_minimumSpreadingFactor_DL = -1;  /* MinimumSpreadingFactor */
static int hf_rnsap_maxNrDLPhysicalchannels = -1;  /* MaxNrDLPhysicalchannels */
static int hf_rnsap_UL_CCTrCH_InformationList_RL_SetupRqstTDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_cCTrCH_ID = -1;               /* CCTrCH_ID */
static int hf_rnsap_tFCI_Coding = -1;             /* TFCI_Coding */
static int hf_rnsap_DL_CCTrCH_InformationList_RL_SetupRqstTDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_dl_TFCS = -1;                 /* TFCS */
static int hf_rnsap_dl_PunctureLimit = -1;        /* PunctureLimit */
static int hf_rnsap_tdd_TPC_DownlinkStepSize = -1;  /* TDD_TPC_DownlinkStepSize */
static int hf_rnsap_cCTrCH_TPCList = -1;          /* CCTrCH_TPCList_RL_SetupRqstTDD */
static int hf_rnsap_CCTrCH_TPCList_RL_SetupRqstTDD_item = -1;  /* CCTrCH_TPCItem_RL_SetupRqstTDD */
static int hf_rnsap_specialBurstScheduling = -1;  /* SpecialBurstScheduling */
static int hf_rnsap_primaryCCPCH_RSCP = -1;       /* PrimaryCCPCH_RSCP */
static int hf_rnsap_dL_TimeSlot_ISCP = -1;        /* DL_TimeSlot_ISCP_Info */
static int hf_rnsap_RL_InformationResponseList_RL_SetupRspFDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_rL_Set_ID = -1;               /* RL_Set_ID */
static int hf_rnsap_uRA_Information = -1;         /* URA_Information */
static int hf_rnsap_sAI = -1;                     /* SAI */
static int hf_rnsap_gA_Cell = -1;                 /* GA_Cell */
static int hf_rnsap_gA_AccessPointPosition = -1;  /* GA_AccessPointPosition */
static int hf_rnsap_received_total_wide_band_power = -1;  /* Received_total_wide_band_power */
static int hf_rnsap_not_Used_secondary_CCPCH_Info = -1;  /* NULL */
static int hf_rnsap_dl_CodeInformation = -1;      /* FDD_DL_CodeInformation */
static int hf_rnsap_diversityIndication = -1;     /* DiversityIndication_RL_SetupRspFDD */
static int hf_rnsap_sSDT_SupportIndicator = -1;   /* SSDT_SupportIndicator */
static int hf_rnsap_maxUL_SIR = -1;               /* UL_SIR */
static int hf_rnsap_minUL_SIR = -1;               /* UL_SIR */
static int hf_rnsap_closedlooptimingadjustmentmode = -1;  /* Closedlooptimingadjustmentmode */
static int hf_rnsap_maximumAllowedULTxPower = -1;  /* MaximumAllowedULTxPower */
static int hf_rnsap_maximumDLTxPower = -1;        /* DL_Power */
static int hf_rnsap_minimumDLTxPower = -1;        /* DL_Power */
static int hf_rnsap_primaryScramblingCode = -1;   /* PrimaryScramblingCode */
static int hf_rnsap_uL_UARFCN = -1;               /* UARFCN */
static int hf_rnsap_dL_UARFCN = -1;               /* UARFCN */
static int hf_rnsap_primaryCPICH_Power = -1;      /* PrimaryCPICH_Power */
static int hf_rnsap_not_Used_dSCHInformationResponse = -1;  /* NULL */
static int hf_rnsap_neighbouring_UMTS_CellInformation = -1;  /* Neighbouring_UMTS_CellInformation */
static int hf_rnsap_neighbouring_GSM_CellInformation = -1;  /* Neighbouring_GSM_CellInformation */
static int hf_rnsap_pC_Preamble = -1;             /* PC_Preamble */
static int hf_rnsap_sRB_Delay = -1;               /* SRB_Delay */
static int hf_rnsap_combining = -1;               /* Combining_RL_SetupRspFDD */
static int hf_rnsap_nonCombiningOrFirstRL = -1;   /* NonCombiningOrFirstRL_RL_SetupRspFDD */
static int hf_rnsap_dCH_InformationResponse = -1;  /* DCH_InformationResponse */
static int hf_rnsap_ul_TimeSlot_ISCP_Info = -1;   /* UL_TimeSlot_ISCP_Info */
static int hf_rnsap_uARFCNforNt = -1;             /* UARFCN */
static int hf_rnsap_cellParameterID = -1;         /* CellParameterID */
static int hf_rnsap_syncCase = -1;                /* SyncCase */
static int hf_rnsap_sCH_TimeSlot = -1;            /* SCH_TimeSlot */
static int hf_rnsap_sCTD_Indicator = -1;          /* SCTD_Indicator */
static int hf_rnsap_pCCPCH_Power = -1;            /* PCCPCH_Power */
static int hf_rnsap_timingAdvanceApplied = -1;    /* TimingAdvanceApplied */
static int hf_rnsap_alphaValue = -1;              /* AlphaValue */
static int hf_rnsap_ul_PhysCH_SF_Variation = -1;  /* UL_PhysCH_SF_Variation */
static int hf_rnsap_synchronisationConfiguration = -1;  /* SynchronisationConfiguration */
static int hf_rnsap_secondary_CCPCH_Info_TDD = -1;  /* Secondary_CCPCH_Info_TDD */
static int hf_rnsap_ul_CCTrCHInformation = -1;    /* UL_CCTrCHInformationList_RL_SetupRspTDD */
static int hf_rnsap_dl_CCTrCHInformation = -1;    /* DL_CCTrCHInformationList_RL_SetupRspTDD */
static int hf_rnsap_dCH_InformationResponse1 = -1;  /* DCH_InformationResponseList_RL_SetupRspTDD */
static int hf_rnsap_dsch_InformationResponse = -1;  /* DSCH_InformationResponse_RL_SetupRspTDD */
static int hf_rnsap_usch_InformationResponse = -1;  /* USCH_InformationResponse_RL_SetupRspTDD */
static int hf_rnsap_UL_CCTrCHInformationListIE_RL_SetupRspTDD_item = -1;  /* UL_CCTrCHInformationItem_RL_SetupRspTDD */
static int hf_rnsap_ul_DPCH_Information = -1;     /* UL_DPCH_InformationList_RL_SetupRspTDD */
static int hf_rnsap_repetitionPeriod = -1;        /* RepetitionPeriod */
static int hf_rnsap_repetitionLength = -1;        /* RepetitionLength */
static int hf_rnsap_tDD_DPCHOffset = -1;          /* TDD_DPCHOffset */
static int hf_rnsap_uL_Timeslot_Information = -1;  /* UL_Timeslot_Information */
static int hf_rnsap_DL_CCTrCHInformationListIE_RL_SetupRspTDD_item = -1;  /* DL_CCTrCHInformationItem_RL_SetupRspTDD */
static int hf_rnsap_dl_DPCH_Information = -1;     /* DL_DPCH_InformationList_RL_SetupRspTDD */
static int hf_rnsap_dL_Timeslot_Information = -1;  /* DL_Timeslot_Information */
static int hf_rnsap_DSCH_InformationListIEs_RL_SetupRspTDD_item = -1;  /* DSCHInformationItem_RL_SetupRspTDD */
static int hf_rnsap_dsch_ID = -1;                 /* DSCH_ID */
static int hf_rnsap_dSCH_FlowControlInformation = -1;  /* DSCH_FlowControlInformation */
static int hf_rnsap_bindingID = -1;               /* BindingID */
static int hf_rnsap_transportLayerAddress = -1;   /* TransportLayerAddress */
static int hf_rnsap_transportFormatManagement = -1;  /* TransportFormatManagement */
static int hf_rnsap_USCH_InformationListIEs_RL_SetupRspTDD_item = -1;  /* USCHInformationItem_RL_SetupRspTDD */
static int hf_rnsap_usch_ID = -1;                 /* USCH_ID */
static int hf_rnsap_ul_TimeSlot_ISCP_LCR_Info = -1;  /* UL_TimeSlot_ISCP_LCR_Info */
static int hf_rnsap_secondary_LCR_CCPCH_Info_TDD = -1;  /* Secondary_LCR_CCPCH_Info_TDD */
static int hf_rnsap_ul_LCR_CCTrCHInformation = -1;  /* UL_LCR_CCTrCHInformationList_RL_SetupRspTDD */
static int hf_rnsap_dl_LCR_CCTrCHInformation = -1;  /* DL_LCR_CCTrCHInformationList_RL_SetupRspTDD */
static int hf_rnsap_dsch_LCR_InformationResponse = -1;  /* DSCH_LCR_InformationResponse_RL_SetupRspTDD */
static int hf_rnsap_usch_LCR_InformationResponse = -1;  /* USCH_LCR_InformationResponse_RL_SetupRspTDD */
static int hf_rnsap_UL_LCR_CCTrCHInformationListIE_RL_SetupRspTDD_item = -1;  /* UL_LCR_CCTrCHInformationItem_RL_SetupRspTDD */
static int hf_rnsap_ul_DPCH_LCR_Information = -1;  /* UL_DPCH_LCR_InformationList_RL_SetupRspTDD */
static int hf_rnsap_uL_TimeslotLCR_Information = -1;  /* UL_TimeslotLCR_Information */
static int hf_rnsap_DL_CCTrCH_LCR_InformationListIE_RL_SetupRspTDD_item = -1;  /* DL_CCTrCH_LCR_InformationItem_RL_SetupRspTDD */
static int hf_rnsap_dl_DPCH_LCR_Information = -1;  /* DL_DPCH_LCR_InformationList_RL_SetupRspTDD */
static int hf_rnsap_dL_Timeslot_LCR_Information = -1;  /* DL_TimeslotLCR_Information */
static int hf_rnsap_tSTD_Indicator = -1;          /* TSTD_Indicator */
static int hf_rnsap_DSCH_LCR_InformationListIEs_RL_SetupRspTDD_item = -1;  /* DSCH_LCR_InformationItem_RL_SetupRspTDD */
static int hf_rnsap_USCH_LCR_InformationListIEs_RL_SetupRspTDD_item = -1;  /* USCH_LCR_InformationItem_RL_SetupRspTDD */
static int hf_rnsap_generalCause = -1;            /* GeneralCauseList_RL_SetupFailureFDD */
static int hf_rnsap_rLSpecificCause = -1;         /* RLSpecificCauseList_RL_SetupFailureFDD */
static int hf_rnsap_cause = -1;                   /* Cause */
static int hf_rnsap_unsuccessful_RL_InformationRespList_RL_SetupFailureFDD = -1;  /* UnsuccessfulRL_InformationResponseList_RL_SetupFailureFDD */
static int hf_rnsap_successful_RL_InformationRespList_RL_SetupFailureFDD = -1;  /* SuccessfulRL_InformationResponseList_RL_SetupFailureFDD */
static int hf_rnsap_UnsuccessfulRL_InformationResponseList_RL_SetupFailureFDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_SuccessfulRL_InformationResponseList_RL_SetupFailureFDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_diversityIndication1 = -1;    /* DiversityIndication_RL_SetupFailureFDD */
static int hf_rnsap_not_Used_dSCH_InformationResponse_RL_SetupFailureFDD = -1;  /* NULL */
static int hf_rnsap_combining1 = -1;              /* Combining_RL_SetupFailureFDD */
static int hf_rnsap_nonCombiningOrFirstRL1 = -1;  /* NonCombiningOrFirstRL_RL_SetupFailureFDD */
static int hf_rnsap_generalCause1 = -1;           /* GeneralCauseList_RL_SetupFailureTDD */
static int hf_rnsap_rLSpecificCause1 = -1;        /* RLSpecificCauseList_RL_SetupFailureTDD */
static int hf_rnsap_unsuccessful_RL_InformationRespItem_RL_SetupFailureTDD = -1;  /* Unsuccessful_RL_InformationRespItem_RL_SetupFailureTDD */
static int hf_rnsap_RL_InformationList_RL_AdditionRqstFDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_dL_TimeSlot_ISCP_Info = -1;   /* DL_TimeSlot_ISCP_Info */
static int hf_rnsap_UL_CCTrCH_InformationList_RL_AdditionRqstTDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_uplinkStepSizeLCR = -1;       /* TDD_TPC_UplinkStepSize_LCR */
static int hf_rnsap_DL_CCTrCH_InformationList_RL_AdditionRqstTDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_downlinkStepSize = -1;        /* TDD_TPC_DownlinkStepSize */
static int hf_rnsap_RL_InformationResponseList_RL_AdditionRspFDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_dl_CodeInformation1 = -1;     /* DL_CodeInformationList_RL_AdditionRspFDD */
static int hf_rnsap_diversityIndication2 = -1;    /* DiversityIndication_RL_AdditionRspFDD */
static int hf_rnsap_combining2 = -1;              /* Combining_RL_AdditionRspFDD */
static int hf_rnsap_nonCombining = -1;            /* NonCombining_RL_AdditionRspFDD */
static int hf_rnsap_ul_CCTrCHInformation1 = -1;   /* UL_CCTrCHInformationList_RL_AdditionRspTDD */
static int hf_rnsap_dl_CCTrCHInformation1 = -1;   /* DL_CCTrCHInformationList_RL_AdditionRspTDD */
static int hf_rnsap_dCH_Information = -1;         /* DCH_Information_RL_AdditionRspTDD */
static int hf_rnsap_dSCH_InformationResponse = -1;  /* DSCH_InformationResponse_RL_AdditionRspTDD */
static int hf_rnsap_uSCH_InformationResponse = -1;  /* USCH_InformationResponse_RL_AdditionRspTDD */
static int hf_rnsap_UL_CCTrCHInformationListIE_RL_AdditionRspTDD_item = -1;  /* UL_CCTrCHInformationItem_RL_AdditionRspTDD */
static int hf_rnsap_ul_DPCH_Information1 = -1;    /* UL_DPCH_InformationList_RL_AdditionRspTDD */
static int hf_rnsap_DL_CCTrCHInformationListIE_RL_AdditionRspTDD_item = -1;  /* DL_CCTrCHInformationItem_RL_AdditionRspTDD */
static int hf_rnsap_dl_DPCH_Information1 = -1;    /* DL_DPCH_InformationList_RL_AdditionRspTDD */
static int hf_rnsap_diversityIndication3 = -1;    /* DiversityIndication_RL_AdditionRspTDD */
static int hf_rnsap_combining3 = -1;              /* Combining_RL_AdditionRspTDD */
static int hf_rnsap_nonCombining1 = -1;           /* NonCombining_RL_AdditionRspTDD */
static int hf_rnsap_DSCH_InformationListIE_RL_AdditionRspTDD_item = -1;  /* DSCHInformationItem_RL_AdditionRspTDD */
static int hf_rnsap_diversityIndication4 = -1;    /* DiversityIndication_RL_AdditionRspTDD2 */
static int hf_rnsap_USCH_InformationListIE_RL_AdditionRspTDD_item = -1;  /* USCHInformationItem_RL_AdditionRspTDD */
static int hf_rnsap_uSCH_ID = -1;                 /* USCH_ID */
static int hf_rnsap_ul_CCTrCH_LCR_Information = -1;  /* UL_CCTrCH_LCR_InformationList_RL_AdditionRspTDD */
static int hf_rnsap_dl_CCTrCH_LCR_Information = -1;  /* DL_CCTrCH_LCR_InformationList_RL_AdditionRspTDD */
static int hf_rnsap_dCH_InformationResponse2 = -1;  /* DCH_InformationResponseList_RL_AdditionRspTDD */
static int hf_rnsap_dsch_LCR_InformationResponse1 = -1;  /* DSCH_LCR_InformationResponse_RL_AdditionRspTDD */
static int hf_rnsap_usch_LCR_InformationResponse1 = -1;  /* USCH_LCR_InformationResponse_RL_AdditionRspTDD */
static int hf_rnsap_UL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD_item = -1;  /* UL_CCTrCH_LCR_InformationItem_RL_AdditionRspTDD */
static int hf_rnsap_ul_DPCH_LCR_Information1 = -1;  /* UL_DPCH_LCR_InformationList_RL_AdditionRspTDD */
static int hf_rnsap_DL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD_item = -1;  /* DL_CCTrCH_LCR_InformationItem_RL_AdditionRspTDD */
static int hf_rnsap_dl_DPCH_LCR_Information1 = -1;  /* DL_DPCH_LCR_InformationList_RL_AdditionRspTDD */
static int hf_rnsap_dL_TimeslotLCR_Information = -1;  /* DL_TimeslotLCR_Information */
static int hf_rnsap_DSCH_LCR_InformationListIEs_RL_AdditionRspTDD_item = -1;  /* DSCH_LCR_InformationItem_RL_AdditionRspTDD */
static int hf_rnsap_USCH_LCR_InformationListIEs_RL_AdditionRspTDD_item = -1;  /* USCH_LCR_InformationItem_RL_AdditionRspTDD */
static int hf_rnsap_generalCause2 = -1;           /* GeneralCauseList_RL_AdditionFailureFDD */
static int hf_rnsap_rLSpecificCause2 = -1;        /* RLSpecificCauseList_RL_AdditionFailureFDD */
static int hf_rnsap_unsuccessful_RL_InformationRespList_RL_AdditionFailureFDD = -1;  /* UnsuccessfulRL_InformationResponseList_RL_AdditionFailureFDD */
static int hf_rnsap_successful_RL_InformationRespList_RL_AdditionFailureFDD = -1;  /* SuccessfulRL_InformationResponseList_RL_AdditionFailureFDD */
static int hf_rnsap_UnsuccessfulRL_InformationResponseList_RL_AdditionFailureFDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_SuccessfulRL_InformationResponseList_RL_AdditionFailureFDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_dl_CodeInformation2 = -1;     /* DL_CodeInformationList_RL_AdditionFailureFDD */
static int hf_rnsap_diversityIndication5 = -1;    /* DiversityIndication_RL_AdditionFailureFDD */
static int hf_rnsap_combining4 = -1;              /* Combining_RL_AdditionFailureFDD */
static int hf_rnsap_nonCombining2 = -1;           /* NonCombining_RL_AdditionFailureFDD */
static int hf_rnsap_generalCause3 = -1;           /* GeneralCauseList_RL_AdditionFailureTDD */
static int hf_rnsap_rLSpecificCause3 = -1;        /* RLSpecificCauseList_RL_AdditionFailureTDD */
static int hf_rnsap_unsuccessful_RL_InformationRespItem_RL_AdditionFailureTDD = -1;  /* Unsuccessful_RL_InformationRespItem_RL_AdditionFailureTDD */
static int hf_rnsap_RL_InformationList_RL_DeletionRqst_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_maxNrOfUL_DPDCHs = -1;        /* MaxNrOfUL_DPCHs */
static int hf_rnsap_not_Used_sSDT_CellIDLength = -1;  /* NULL */
static int hf_rnsap_powerOffsetInformation2 = -1;  /* PowerOffsetInformation_RL_ReconfPrepFDD */
static int hf_rnsap_fdd_TPC_DownlinkStepSize = -1;  /* FDD_TPC_DownlinkStepSize */
static int hf_rnsap_pO1_ForTFCI_Bits = -1;        /* PowerOffset */
static int hf_rnsap_pO2_ForTPC_Bits = -1;         /* PowerOffset */
static int hf_rnsap_pO3_ForPilotBits = -1;        /* PowerOffset */
static int hf_rnsap_DCH_DeleteList_RL_ReconfPrepFDD_item = -1;  /* DCH_DeleteItem_RL_ReconfPrepFDD */
static int hf_rnsap_dCH_ID = -1;                  /* DCH_ID */
static int hf_rnsap_RL_InformationList_RL_ReconfPrepFDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_not_Used_sSDT_Indication = -1;  /* NULL */
static int hf_rnsap_not_Used_sSDT_CellIdentity = -1;  /* NULL */
static int hf_rnsap_powerOffsetInformation3 = -1;  /* PowerOffsetInformation_F_DPCH_RL_ReconfPrepFDD */
static int hf_rnsap_UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_punctureLimit = -1;           /* PunctureLimit */
static int hf_rnsap_UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_cCTrCH_TPCList1 = -1;         /* CCTrCH_TPCAddList_RL_ReconfPrepTDD */
static int hf_rnsap_CCTrCH_TPCAddList_RL_ReconfPrepTDD_item = -1;  /* CCTrCH_TPCAddItem_RL_ReconfPrepTDD */
static int hf_rnsap_DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_cCTrCH_TPCList2 = -1;         /* CCTrCH_TPCModifyList_RL_ReconfPrepTDD */
static int hf_rnsap_CCTrCH_TPCModifyList_RL_ReconfPrepTDD_item = -1;  /* CCTrCH_TPCModifyItem_RL_ReconfPrepTDD */
static int hf_rnsap_DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_DCH_DeleteList_RL_ReconfPrepTDD_item = -1;  /* DCH_DeleteItem_RL_ReconfPrepTDD */
static int hf_rnsap_DSCH_ModifyList_RL_ReconfPrepTDD_item = -1;  /* DSCH_ModifyItem_RL_ReconfPrepTDD */
static int hf_rnsap_dSCH_ID = -1;                 /* DSCH_ID */
static int hf_rnsap_dl_ccTrCHID = -1;             /* CCTrCH_ID */
static int hf_rnsap_trChSourceStatisticsDescriptor = -1;  /* TrCH_SrcStatisticsDescr */
static int hf_rnsap_transportFormatSet = -1;      /* TransportFormatSet */
static int hf_rnsap_allocationRetentionPriority = -1;  /* AllocationRetentionPriority */
static int hf_rnsap_schedulingPriorityIndicator = -1;  /* SchedulingPriorityIndicator */
static int hf_rnsap_bLER = -1;                    /* BLER */
static int hf_rnsap_transportBearerRequestIndicator = -1;  /* TransportBearerRequestIndicator */
static int hf_rnsap_DSCH_DeleteList_RL_ReconfPrepTDD_item = -1;  /* DSCH_DeleteItem_RL_ReconfPrepTDD */
static int hf_rnsap_USCH_ModifyList_RL_ReconfPrepTDD_item = -1;  /* USCH_ModifyItem_RL_ReconfPrepTDD */
static int hf_rnsap_ul_ccTrCHID = -1;             /* CCTrCH_ID */
static int hf_rnsap_rb_Info = -1;                 /* RB_Info */
static int hf_rnsap_USCH_DeleteList_RL_ReconfPrepTDD_item = -1;  /* USCH_DeleteItem_RL_ReconfPrepTDD */
static int hf_rnsap_RL_Information_RL_ReconfPrepTDD_item = -1;  /* RL_InformationIE_RL_ReconfPrepTDD */
static int hf_rnsap_rL_Specific_DCH_Info = -1;    /* RL_Specific_DCH_Info */
static int hf_rnsap_RL_InformationResponseList_RL_ReconfReadyFDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_max_UL_SIR = -1;              /* UL_SIR */
static int hf_rnsap_min_UL_SIR = -1;              /* UL_SIR */
static int hf_rnsap_dl_CodeInformationList = -1;  /* DL_CodeInformationList_RL_ReconfReadyFDD */
static int hf_rnsap_dCHInformationResponse = -1;  /* DCH_InformationResponseList_RL_ReconfReadyFDD */
static int hf_rnsap_not_Used_dSCHsToBeAddedOrModified = -1;  /* NULL */
static int hf_rnsap_ul_CCTrCH_Information = -1;   /* UL_CCTrCH_InformationList_RL_ReconfReadyTDD */
static int hf_rnsap_dl_CCTrCH_Information = -1;   /* DL_CCTrCH_InformationList_RL_ReconfReadyTDD */
static int hf_rnsap_dCHInformationResponse1 = -1;  /* DCH_InformationResponseList_RL_ReconfReadyTDD */
static int hf_rnsap_dSCHsToBeAddedOrModified = -1;  /* DSCHToBeAddedOrModified_RL_ReconfReadyTDD */
static int hf_rnsap_uSCHsToBeAddedOrModified = -1;  /* USCHToBeAddedOrModified_RL_ReconfReadyTDD */
static int hf_rnsap_UL_CCTrCHInformationListIE_RL_ReconfReadyTDD_item = -1;  /* UL_CCTrCH_InformationItem_RL_ReconfReadyTDD */
static int hf_rnsap_ul_DPCH_AddInformation = -1;  /* UL_DPCH_InformationAddList_RL_ReconfReadyTDD */
static int hf_rnsap_ul_DPCH_ModifyInformation = -1;  /* UL_DPCH_InformationModifyList_RL_ReconfReadyTDD */
static int hf_rnsap_ul_DPCH_DeleteInformation = -1;  /* UL_DPCH_InformationDeleteList_RL_ReconfReadyTDD */
static int hf_rnsap_uL_TimeslotLCR_Info = -1;     /* UL_TimeslotLCR_Information */
static int hf_rnsap_rxTimingDeviationForTA = -1;  /* RxTimingDeviationForTA */
static int hf_rnsap_uL_Timeslot_InformationModifyList_RL_ReconfReadyTDD = -1;  /* UL_Timeslot_InformationModifyList_RL_ReconfReadyTDD */
static int hf_rnsap_UL_TimeslotLCR_InformationModifyList_RL_ReconfReadyTDD_item = -1;  /* UL_TimeslotLCR_InformationModifyItem_RL_ReconfReadyTDD */
static int hf_rnsap_timeSlotLCR = -1;             /* TimeSlotLCR */
static int hf_rnsap_midambleShiftLCR = -1;        /* MidambleShiftLCR */
static int hf_rnsap_tDD_uL_Code_LCR_Information = -1;  /* TDD_UL_Code_LCR_InformationModifyList_RL_ReconfReadyTDD */
static int hf_rnsap_TDD_UL_Code_LCR_InformationModifyList_RL_ReconfReadyTDD_item = -1;  /* TDD_UL_Code_LCR_InformationModifyItem_RL_ReconfReadyTDD */
static int hf_rnsap_dPCH_ID = -1;                 /* DPCH_ID */
static int hf_rnsap_tDD_ChannelisationCodeLCR = -1;  /* TDD_ChannelisationCodeLCR */
static int hf_rnsap_UL_Timeslot_InformationModifyList_RL_ReconfReadyTDD_item = -1;  /* UL_Timeslot_InformationModifyItem_RL_ReconfReadyTDD */
static int hf_rnsap_timeSlot = -1;                /* TimeSlot */
static int hf_rnsap_midambleShiftAndBurstType = -1;  /* MidambleShiftAndBurstType */
static int hf_rnsap_uL_Code_Information = -1;     /* TDD_UL_Code_InformationModifyList_RL_ReconfReadyTDD */
static int hf_rnsap_TDD_UL_Code_InformationModifyList_RL_ReconfReadyTDD_item = -1;  /* TDD_UL_Code_InformationModifyItem_RL_ReconfReadyTDD */
static int hf_rnsap_tDD_ChannelisationCode = -1;  /* TDD_ChannelisationCode */
static int hf_rnsap_UL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD_item = -1;  /* UL_DPCH_InformationDeleteItem_RL_ReconfReadyTDD */
static int hf_rnsap_DL_CCTrCHInformationListIE_RL_ReconfReadyTDD_item = -1;  /* DL_CCTrCH_InformationItem_RL_ReconfReadyTDD */
static int hf_rnsap_dl_DPCH_AddInformation = -1;  /* DL_DPCH_InformationAddList_RL_ReconfReadyTDD */
static int hf_rnsap_dl_DPCH_ModifyInformation = -1;  /* DL_DPCH_InformationModifyList_RL_ReconfReadyTDD */
static int hf_rnsap_dl_DPCH_DeleteInformation = -1;  /* DL_DPCH_InformationDeleteList_RL_ReconfReadyTDD */
static int hf_rnsap_dL_TimeslotLCR_Info = -1;     /* DL_TimeslotLCR_Information */
static int hf_rnsap_dL_Timeslot_InformationModifyList_RL_ReconfReadyTDD = -1;  /* DL_Timeslot_InformationModifyList_RL_ReconfReadyTDD */
static int hf_rnsap_DL_TimeslotLCR_InformationModifyList_RL_ReconfReadyTDD_item = -1;  /* DL_TimeslotLCR_InformationModifyItem_RL_ReconfReadyTDD */
static int hf_rnsap_tDD_dL_Code_LCR_Information = -1;  /* TDD_DL_Code_LCR_InformationModifyList_RL_ReconfReadyTDD */
static int hf_rnsap_TDD_DL_Code_LCR_InformationModifyList_RL_ReconfReadyTDD_item = -1;  /* TDD_DL_Code_LCR_InformationModifyItem_RL_ReconfReadyTDD */
static int hf_rnsap_DL_Timeslot_InformationModifyList_RL_ReconfReadyTDD_item = -1;  /* DL_Timeslot_InformationModifyItem_RL_ReconfReadyTDD */
static int hf_rnsap_dL_Code_Information = -1;     /* TDD_DL_Code_InformationModifyList_RL_ReconfReadyTDD */
static int hf_rnsap_TDD_DL_Code_InformationModifyList_RL_ReconfReadyTDD_item = -1;  /* TDD_DL_Code_InformationModifyItem_RL_ReconfReadyTDD */
static int hf_rnsap_DL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD_item = -1;  /* DL_DPCH_InformationDeleteItem_RL_ReconfReadyTDD */
static int hf_rnsap_DSCHToBeAddedOrModifiedList_RL_ReconfReadyTDD_item = -1;  /* DSCHToBeAddedOrModifiedItem_RL_ReconfReadyTDD */
static int hf_rnsap_USCHToBeAddedOrModifiedList_RL_ReconfReadyTDD_item = -1;  /* USCHToBeAddedOrModifiedItem_RL_ReconfReadyTDD */
static int hf_rnsap_Multiple_RL_InformationResponse_RL_ReconfReadyTDD_item = -1;  /* RL_InformationResponse_RL_ReconfReadyTDD */
static int hf_rnsap_generalCause4 = -1;           /* GeneralCauseList_RL_ReconfFailure */
static int hf_rnsap_rLSpecificCause4 = -1;        /* RLSpecificCauseList_RL_ReconfFailure */
static int hf_rnsap_rL_ReconfigurationFailureList_RL_ReconfFailure = -1;  /* RL_ReconfigurationFailureList_RL_ReconfFailure */
static int hf_rnsap_RL_ReconfigurationFailureList_RL_ReconfFailure_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_DCH_DeleteList_RL_ReconfRqstFDD_item = -1;  /* DCH_DeleteItem_RL_ReconfRqstFDD */
static int hf_rnsap_RL_ReconfigurationRequestFDD_RL_InformationList_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_DCH_DeleteList_RL_ReconfRqstTDD_item = -1;  /* DCH_DeleteItem_RL_ReconfRqstTDD */
static int hf_rnsap_Multiple_RL_ReconfigurationRequestTDD_RL_Information_item = -1;  /* RL_ReconfigurationRequestTDD_RL_Information */
static int hf_rnsap_RL_InformationResponseList_RL_ReconfRspFDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_dCHsInformationResponseList = -1;  /* DCH_InformationResponseList_RL_ReconfRspFDD */
static int hf_rnsap_dL_CodeInformationList_RL_ReconfResp = -1;  /* DL_CodeInformationList_RL_ReconfRspFDD */
static int hf_rnsap_dCHsInformationResponseList1 = -1;  /* DCH_InformationResponseList_RL_ReconfRspTDD */
static int hf_rnsap_DL_CCTrCH_InformationList_RL_ReconfRspTDD_item = -1;  /* DL_CCTrCH_InformationItem_RL_ReconfRspTDD */
static int hf_rnsap_dl_DPCH_ModifyInformation_LCR = -1;  /* DL_DPCH_InformationModifyList_LCR_RL_ReconfRspTDD */
static int hf_rnsap_cCTrCH_Maximum_DL_Power = -1;  /* DL_Power */
static int hf_rnsap_cCTrCH_Minimum_DL_Power = -1;  /* DL_Power */
static int hf_rnsap_dL_Timeslot_LCR_InformationModifyList_RL_ReconfRqstTDD = -1;  /* DL_Timeslot_LCR_InformationModifyList_RL_ReconfRspTDD */
static int hf_rnsap_DL_Timeslot_LCR_InformationModifyList_RL_ReconfRspTDD_item = -1;  /* DL_Timeslot_LCR_InformationModifyItem_RL_ReconfRspTDD */
static int hf_rnsap_maxPowerLCR = -1;             /* DL_Power */
static int hf_rnsap_minPowerLCR = -1;             /* DL_Power */
static int hf_rnsap_Multiple_RL_InformationResponse_RL_ReconfRspTDD_item = -1;  /* RL_InformationResponse_RL_ReconfRspTDD */
static int hf_rnsap_rL = -1;                      /* RL_RL_FailureInd */
static int hf_rnsap_rL_Set = -1;                  /* RL_Set_RL_FailureInd */
static int hf_rnsap_cCTrCH = -1;                  /* CCTrCH_RL_FailureInd */
static int hf_rnsap_rL_InformationList_RL_FailureInd = -1;  /* RL_InformationList_RL_FailureInd */
static int hf_rnsap_RL_InformationList_RL_FailureInd_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_rL_Set_InformationList_RL_FailureInd = -1;  /* RL_Set_InformationList_RL_FailureInd */
static int hf_rnsap_RL_Set_InformationList_RL_FailureInd_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_cCTrCH_InformationList_RL_FailureInd = -1;  /* CCTrCH_InformationList_RL_FailureInd */
static int hf_rnsap_CCTrCH_InformationList_RL_FailureInd_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_RL_InformationList_RL_PreemptRequiredInd_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_HSDSCHMacdFlowSpecificInformationList_RL_PreemptRequiredInd_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_hSDSCH_MACdFlow_ID = -1;      /* HSDSCH_MACdFlow_ID */
static int hf_rnsap_EDCH_MacdFlowSpecificInformationList_RL_PreemptRequiredInd_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_eDCH_MACdFlow_ID = -1;        /* EDCH_MACdFlow_ID */
static int hf_rnsap_rL1 = -1;                     /* RL_RL_RestoreInd */
static int hf_rnsap_rL_Set1 = -1;                 /* RL_Set_RL_RestoreInd */
static int hf_rnsap_cCTrCH1 = -1;                 /* CCTrCH_RL_RestoreInd */
static int hf_rnsap_rL_InformationList_RL_RestoreInd = -1;  /* RL_InformationList_RL_RestoreInd */
static int hf_rnsap_RL_InformationList_RL_RestoreInd_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_rL_Set_InformationList_RL_RestoreInd = -1;  /* RL_Set_InformationList_RL_RestoreInd */
static int hf_rnsap_RL_Set_InformationList_RL_RestoreInd_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_cCTrCH_InformationList_RL_RestoreInd = -1;  /* CCTrCH_InformationList_RL_RestoreInd */
static int hf_rnsap_CCTrCH_InformationList_RL_RestoreInd_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_DL_ReferencePowerInformationList_DL_PC_Rqst_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_dl_Reference_Power = -1;      /* DL_Power */
static int hf_rnsap_dl_CodeInformation3 = -1;     /* DL_CodeInformationList_PhyChReconfRqstFDD */
static int hf_rnsap_ul_CCTrCH_Information1 = -1;  /* UL_CCTrCH_InformationList_PhyChReconfRqstTDD */
static int hf_rnsap_dl_CCTrCH_Information1 = -1;  /* DL_CCTrCH_InformationList_PhyChReconfRqstTDD */
static int hf_rnsap_UL_CCTrCH_InformationListIE_PhyChReconfRqstTDD_item = -1;  /* UL_CCTrCH_InformationItem_PhyChReconfRqstTDD */
static int hf_rnsap_ul_DPCH_Information2 = -1;    /* UL_DPCH_InformationList_PhyChReconfRqstTDD */
static int hf_rnsap_uL_Timeslot_InformationList_PhyChReconfRqstTDD = -1;  /* UL_Timeslot_InformationList_PhyChReconfRqstTDD */
static int hf_rnsap_UL_TimeslotLCR_InformationList_PhyChReconfRqstTDD_item = -1;  /* UL_TimeslotLCR_InformationItem_PhyChReconfRqstTDD */
static int hf_rnsap_uL_Code_LCR_Information = -1;  /* TDD_UL_Code_LCR_Information */
static int hf_rnsap_UL_Timeslot_InformationList_PhyChReconfRqstTDD_item = -1;  /* UL_Timeslot_InformationItem_PhyChReconfRqstTDD */
static int hf_rnsap_uL_Code_Information1 = -1;    /* TDD_UL_Code_Information */
static int hf_rnsap_DL_CCTrCH_InformationListIE_PhyChReconfRqstTDD_item = -1;  /* DL_CCTrCH_InformationItem_PhyChReconfRqstTDD */
static int hf_rnsap_dl_DPCH_Information2 = -1;    /* DL_DPCH_InformationList_PhyChReconfRqstTDD */
static int hf_rnsap_dL_Timeslot_InformationList_PhyChReconfRqstTDD = -1;  /* DL_Timeslot_InformationList_PhyChReconfRqstTDD */
static int hf_rnsap_DL_TimeslotLCR_InformationList_PhyChReconfRqstTDD_item = -1;  /* DL_TimeslotLCR_InformationItem_PhyChReconfRqstTDD */
static int hf_rnsap_dL_Code_LCR_Information = -1;  /* TDD_DL_Code_LCR_Information */
static int hf_rnsap_DL_Timeslot_InformationList_PhyChReconfRqstTDD_item = -1;  /* DL_Timeslot_InformationItem_PhyChReconfRqstTDD */
static int hf_rnsap_dL_Code_Information1 = -1;    /* TDD_DL_Code_Information */
static int hf_rnsap_HSPDSCH_Timeslot_InformationList_PhyChReconfRqstTDD_item = -1;  /* HSPDSCH_Timeslot_InformationItem_PhyChReconfRqstTDD */
static int hf_rnsap_timeslot = -1;                /* TimeSlot */
static int hf_rnsap_HSPDSCH_Timeslot_InformationListLCR_PhyChReconfRqstTDD_item = -1;  /* HSPDSCH_Timeslot_InformationItemLCR_PhyChReconfRqstTDD */
static int hf_rnsap_timeslotLCR = -1;             /* TimeSlotLCR */
static int hf_rnsap_RL_InformationList_RL_CongestInd_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_dCH_Rate_Information = -1;    /* DCH_Rate_Information_RL_CongestInd */
static int hf_rnsap_DCH_Rate_Information_RL_CongestInd_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_allowed_Rate_Information = -1;  /* Allowed_Rate_Information */
static int hf_rnsap_EDCH_MacdFlowSpecificInformationList_RL_CongestInd_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_uRA = -1;                     /* URA_PagingRqst */
static int hf_rnsap_cell = -1;                    /* Cell_PagingRqst */
static int hf_rnsap_uRA_ID = -1;                  /* URA_ID */
static int hf_rnsap_pagingCause = -1;             /* PagingCause */
static int hf_rnsap_cNDomainType = -1;            /* CNDomainType */
static int hf_rnsap_pagingRecordType = -1;        /* PagingRecordType */
static int hf_rnsap_rL2 = -1;                     /* RL_DM_Rqst */
static int hf_rnsap_rLS = -1;                     /* RL_Set_DM_Rqst */
static int hf_rnsap_allRL = -1;                   /* All_RL_DM_Rqst */
static int hf_rnsap_allRLS = -1;                  /* All_RL_Set_DM_Rqst */
static int hf_rnsap_rL_InformationList_DM_Rqst = -1;  /* RL_InformationList_DM_Rqst */
static int hf_rnsap_RL_InformationList_DM_Rqst_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_HSSICH_Info_DM_Rqst_item = -1;  /* HS_SICH_ID */
static int hf_rnsap_rL_Set_InformationList_DM_Rqst = -1;  /* RL_Set_InformationList_DM_Rqst */
static int hf_rnsap_RL_Set_InformationList_DM_Rqst_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_rLs = -1;                     /* RL_DM_Rsp */
static int hf_rnsap_rLS1 = -1;                    /* RL_Set_DM_Rsp */
static int hf_rnsap_allRL1 = -1;                  /* RL_DM_Rsp */
static int hf_rnsap_allRLS1 = -1;                 /* RL_Set_DM_Rsp */
static int hf_rnsap_rL_InformationList_DM_Rsp = -1;  /* RL_InformationList_DM_Rsp */
static int hf_rnsap_rL_Set_InformationList_DM_Rsp = -1;  /* RL_Set_InformationList_DM_Rsp */
static int hf_rnsap_RL_InformationList_DM_Rsp_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_dedicatedMeasurementValue = -1;  /* DedicatedMeasurementValue */
static int hf_rnsap_cFN = -1;                     /* CFN */
static int hf_rnsap_RL_Set_InformationList_DM_Rsp_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_Multiple_DedicatedMeasurementValueList_TDD_DM_Rsp_item = -1;  /* Multiple_DedicatedMeasurementValueItem_TDD_DM_Rsp */
static int hf_rnsap_Multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp_item = -1;  /* Multiple_DedicatedMeasurementValueItem_LCR_TDD_DM_Rsp */
static int hf_rnsap_rL3 = -1;                     /* RL_DM_Fail */
static int hf_rnsap_rLS2 = -1;                    /* RL_Set_DM_Fail */
static int hf_rnsap_allRL2 = -1;                  /* RL_DM_Fail */
static int hf_rnsap_allRLS2 = -1;                 /* RL_Set_DM_Fail */
static int hf_rnsap_rL_unsuccessful_InformationRespList_DM_Fail = -1;  /* RL_Unsuccessful_InformationRespList_DM_Fail */
static int hf_rnsap_rL_successful_InformationRespList_DM_Fail = -1;  /* RL_Successful_InformationRespList_DM_Fail */
static int hf_rnsap_rL_Set_unsuccessful_InformationRespList_DM_Fail = -1;  /* RL_Set_Unsuccessful_InformationRespList_DM_Fail */
static int hf_rnsap_rL_Set_successful_InformationRespList_DM_Fail = -1;  /* RL_Set_Successful_InformationRespList_DM_Fail */
static int hf_rnsap_RL_Unsuccessful_InformationRespList_DM_Fail_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_individualcause = -1;         /* Cause */
static int hf_rnsap_RL_Successful_InformationRespList_DM_Fail_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_RL_Set_Unsuccessful_InformationRespList_DM_Fail_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_RL_Set_Successful_InformationRespList_DM_Fail_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_rLs1 = -1;                    /* RL_DM_Rprt */
static int hf_rnsap_rLS3 = -1;                    /* RL_Set_DM_Rprt */
static int hf_rnsap_allRL3 = -1;                  /* RL_DM_Rprt */
static int hf_rnsap_allRLS3 = -1;                 /* RL_Set_DM_Rprt */
static int hf_rnsap_rL_InformationList_DM_Rprt = -1;  /* RL_InformationList_DM_Rprt */
static int hf_rnsap_rL_Set_InformationList_DM_Rprt = -1;  /* RL_Set_InformationList_DM_Rprt */
static int hf_rnsap_RL_InformationList_DM_Rprt_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_dedicatedMeasurementValueInformation = -1;  /* DedicatedMeasurementValueInformation */
static int hf_rnsap_RL_Set_InformationList_DM_Rprt_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_rL4 = -1;                     /* RL_DM_Fail_Ind */
static int hf_rnsap_rLS4 = -1;                    /* RL_Set_DM_Fail_Ind */
static int hf_rnsap_allRL4 = -1;                  /* RL_DM_Fail_Ind */
static int hf_rnsap_allRLS4 = -1;                 /* RL_Set_DM_Fail_Ind */
static int hf_rnsap_rL_unsuccessful_InformationRespList_DM_Fail_Ind = -1;  /* RL_Unsuccessful_InformationRespList_DM_Fail_Ind */
static int hf_rnsap_rL_Set_unsuccessful_InformationRespList_DM_Fail_Ind = -1;  /* RL_Set_Unsuccessful_InformationRespList_DM_Fail_Ind */
static int hf_rnsap_RL_Unsuccessful_InformationRespList_DM_Fail_Ind_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_RL_Set_Unsuccessful_InformationRespList_DM_Fail_Ind_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_fACH_FlowControlInformation = -1;  /* FACH_FlowControlInformation_CTCH_ResourceRspFDD */
static int hf_rnsap_fACH_FlowControlInformation1 = -1;  /* FACH_FlowControlInformation_CTCH_ResourceRspTDD */
static int hf_rnsap_cell1 = -1;                   /* Cell_CM_Rqst */
static int hf_rnsap_uC_ID = -1;                   /* UC_ID */
static int hf_rnsap_neighbouringCellMeasurementInformation = -1;  /* NeighbouringCellMeasurementInfo */
static int hf_rnsap_NeighbouringCellMeasurementInfo_item = -1;  /* NeighbouringCellMeasurementInfo_item */
static int hf_rnsap_neighbouringFDDCellMeasurementInformation = -1;  /* NeighbouringFDDCellMeasurementInformation */
static int hf_rnsap_neighbouringTDDCellMeasurementInformation = -1;  /* NeighbouringTDDCellMeasurementInformation */
static int hf_rnsap_extension_neighbouringCellMeasurementInformation = -1;  /* Extension_neighbouringCellMeasurementInformation */
static int hf_rnsap_cell2 = -1;                   /* Cell_CM_Rsp */
static int hf_rnsap_commonMeasurementValue = -1;  /* CommonMeasurementValue */
static int hf_rnsap_cell3 = -1;                   /* Cell_CM_Rprt */
static int hf_rnsap_commonMeasurementValueInformation = -1;  /* CommonMeasurementValueInformation */
static int hf_rnsap_cell4 = -1;                   /* Cell_InfEx_Rqst */
static int hf_rnsap_extension_InformationExchangeObjectType_InfEx_Rqst = -1;  /* Extension_InformationExchangeObjectType_InfEx_Rqst */
static int hf_rnsap_cGI = -1;                     /* CGI */
static int hf_rnsap_cell5 = -1;                   /* Cell_InfEx_Rsp */
static int hf_rnsap_extension_InformationExchangeObjectType_InfEx_Rsp = -1;  /* Extension_InformationExchangeObjectType_InfEx_Rsp */
static int hf_rnsap_requestedDataValue = -1;      /* RequestedDataValue */
static int hf_rnsap_MBMS_Bearer_Service_List_InfEx_Rsp_item = -1;  /* MBMS_Bearer_ServiceItemIEs_InfEx_Rsp */
static int hf_rnsap_tmgi = -1;                    /* TMGI */
static int hf_rnsap_cell6 = -1;                   /* Cell_InfEx_Rprt */
static int hf_rnsap_requestedDataValueInformation = -1;  /* RequestedDataValueInformation */
static int hf_rnsap_context = -1;                 /* ContextList_Reset */
static int hf_rnsap_all_contexts = -1;            /* NULL */
static int hf_rnsap_contextGroup = -1;            /* ContextGroupList_Reset */
static int hf_rnsap_contextInfoList_Reset = -1;   /* ContextInfoList_Reset */
static int hf_rnsap_ContextInfoList_Reset_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_contextType_Reset = -1;       /* ContextType_Reset */
static int hf_rnsap_sRNTI = -1;                   /* S_RNTI */
static int hf_rnsap_dRNTI = -1;                   /* D_RNTI */
static int hf_rnsap_contextGroupInfoList_Reset = -1;  /* ContextGroupInfoList_Reset */
static int hf_rnsap_ContextGroupInfoList_Reset_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_s_RNTI_Group = -1;            /* S_RNTI_Group */
static int hf_rnsap_DelayedActivationInformationList_RL_ActivationCmdFDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_delayed_activation_update = -1;  /* DelayedActivationUpdate */
static int hf_rnsap_DelayedActivationInformationList_RL_ActivationCmdTDD_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_RL_ParameterUpdateIndicationFDD_RL_InformationList_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_phase_Reference_Update_Indicator = -1;  /* Phase_Reference_Update_Indicator */
static int hf_rnsap_ListOfInterfacesToTrace_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_interface = -1;               /* T_interface */
static int hf_rnsap_privateIEs = -1;              /* PrivateIE_Container */
static int hf_rnsap_Active_MBMS_Bearer_Service_ListFDD_item = -1;  /* MBMS_Bearer_ServiceItemFDD */
static int hf_rnsap_Active_MBMS_Bearer_Service_ListFDD_PFL_item = -1;  /* MBMS_Bearer_ServiceItemFDD_PFL */
static int hf_rnsap_Active_MBMS_Bearer_Service_ListTDD_item = -1;  /* MBMS_Bearer_ServiceItemTDD */
static int hf_rnsap_Active_MBMS_Bearer_Service_ListTDD_PFL_item = -1;  /* MBMS_Bearer_ServiceItemTDD_PFL */
static int hf_rnsap_cMConfigurationChangeCFN = -1;  /* CFN */
static int hf_rnsap_transmission_Gap_Pattern_Sequence_Status = -1;  /* Transmission_Gap_Pattern_Sequence_Status_List */
static int hf_rnsap_AdditionalPreferredFrequency_item = -1;  /* AdditionalPreferredFrequencyItem */
static int hf_rnsap_correspondingCells = -1;      /* CorrespondingCells */
static int hf_rnsap_AffectedUEInformationForMBMS_item = -1;  /* S_RNTI */
static int hf_rnsap_priorityLevel = -1;           /* PriorityLevel */
static int hf_rnsap_pre_emptionCapability = -1;   /* Pre_emptionCapability */
static int hf_rnsap_pre_emptionVulnerability = -1;  /* Pre_emptionVulnerability */
static int hf_rnsap_allowed_UL_Rate = -1;         /* Allowed_Rate */
static int hf_rnsap_allowed_DL_Rate = -1;         /* Allowed_Rate */
static int hf_rnsap_aOA_LCR = -1;                 /* AOA_LCR */
static int hf_rnsap_aOA_LCR_Accuracy_Class = -1;  /* AOA_LCR_Accuracy_Class */
static int hf_rnsap_badSatelliteInformation = -1;  /* T_badSatelliteInformation */
static int hf_rnsap_badSatelliteInformation_item = -1;  /* T_badSatelliteInformation_item */
static int hf_rnsap_badSAT_ID = -1;               /* SAT_ID */
static int hf_rnsap_nCC = -1;                     /* NCC */
static int hf_rnsap_bCC = -1;                     /* BCC */
static int hf_rnsap_burstStart = -1;              /* INTEGER_0_15 */
static int hf_rnsap_burstLength = -1;             /* INTEGER_10_25 */
static int hf_rnsap_burstFreq = -1;               /* INTEGER_1_16 */
static int hf_rnsap_radioNetwork = -1;            /* CauseRadioNetwork */
static int hf_rnsap_transport = -1;               /* CauseTransport */
static int hf_rnsap_protocol = -1;                /* CauseProtocol */
static int hf_rnsap_misc = -1;                    /* CauseMisc */
static int hf_rnsap_uplinkCellCapacityClassValue = -1;  /* INTEGER_1_100_ */
static int hf_rnsap_downlinkCellCapacityClassValue = -1;  /* INTEGER_1_100_ */
static int hf_rnsap_lAI = -1;                     /* T_lAI */
static int hf_rnsap_pLMN_Identity = -1;           /* PLMN_Identity */
static int hf_rnsap_lAC = -1;                     /* LAC */
static int hf_rnsap_cI = -1;                      /* CI */
static int hf_rnsap_tUTRANGPSMeasurementAccuracyClass = -1;  /* TUTRANGPSAccuracyClass */
static int hf_rnsap_tUTRANGPSMeasurementValueInformation = -1;  /* TUTRANGPSMeasurementValueInformation */
static int hf_rnsap_sFNSFNMeasurementValueInformation = -1;  /* SFNSFNMeasurementValueInformation */
static int hf_rnsap_loadValue = -1;               /* LoadValue */
static int hf_rnsap_transmittedCarrierPowerValue = -1;  /* INTEGER_0_100 */
static int hf_rnsap_receivedTotalWideBandPowerValue = -1;  /* INTEGER_0_621 */
static int hf_rnsap_uplinkTimeslotISCPValue = -1;  /* UL_TimeslotISCP */
static int hf_rnsap_extension_CommonMeasurementValue = -1;  /* Extension_CommonMeasurementValue */
static int hf_rnsap_measurementAvailable = -1;    /* CommonMeasurementAvailable */
static int hf_rnsap_measurementnotAvailable = -1;  /* NULL */
static int hf_rnsap_CorrespondingCells_item = -1;  /* C_ID */
static int hf_rnsap_triggeringMessage = -1;       /* TriggeringMessage */
static int hf_rnsap_procedureCriticality = -1;    /* Criticality */
static int hf_rnsap_iEsCriticalityDiagnostics = -1;  /* CriticalityDiagnostics_IE_List */
static int hf_rnsap_CriticalityDiagnostics_IE_List_item = -1;  /* CriticalityDiagnostics_IE_List_item */
static int hf_rnsap_iECriticality = -1;           /* Criticality */
static int hf_rnsap_iE_ID = -1;                   /* ProtocolIE_ID */
static int hf_rnsap_repetitionNumber = -1;        /* RepetitionNumber0 */
static int hf_rnsap_MessageStructure_item = -1;   /* MessageStructure_item */
static int hf_rnsap_repetitionNumber1 = -1;       /* RepetitionNumber1 */
static int hf_rnsap_rAC = -1;                     /* RAC */
static int hf_rnsap_DCH_FDD_Information_item = -1;  /* DCH_FDD_InformationItem */
static int hf_rnsap_payloadCRC_PresenceIndicator = -1;  /* PayloadCRC_PresenceIndicator */
static int hf_rnsap_ul_FP_Mode = -1;              /* UL_FP_Mode */
static int hf_rnsap_toAWS = -1;                   /* ToAWS */
static int hf_rnsap_toAWE = -1;                   /* ToAWE */
static int hf_rnsap_dCH_SpecificInformationList = -1;  /* DCH_Specific_FDD_InformationList */
static int hf_rnsap_DCH_Specific_FDD_InformationList_item = -1;  /* DCH_Specific_FDD_Item */
static int hf_rnsap_trCH_SrcStatisticsDescr = -1;  /* TrCH_SrcStatisticsDescr */
static int hf_rnsap_ul_transportFormatSet = -1;   /* TransportFormatSet */
static int hf_rnsap_dl_transportFormatSet = -1;   /* TransportFormatSet */
static int hf_rnsap_ul_BLER = -1;                 /* BLER */
static int hf_rnsap_dl_BLER = -1;                 /* BLER */
static int hf_rnsap_frameHandlingPriority = -1;   /* FrameHandlingPriority */
static int hf_rnsap_qE_Selector = -1;             /* QE_Selector */
static int hf_rnsap_dRACControl = -1;             /* DRACControl */
static int hf_rnsap_DCH_InformationResponse_item = -1;  /* DCH_InformationResponseItem */
static int hf_rnsap_DCH_TDD_Information_item = -1;  /* DCH_TDD_InformationItem */
static int hf_rnsap_dCH_SpecificInformationList1 = -1;  /* DCH_Specific_TDD_InformationList */
static int hf_rnsap_DCH_Specific_TDD_InformationList_item = -1;  /* DCH_Specific_TDD_Item */
static int hf_rnsap_ul_cCTrCH_ID = -1;            /* CCTrCH_ID */
static int hf_rnsap_dl_cCTrCH_ID = -1;            /* CCTrCH_ID */
static int hf_rnsap_sIR_Value = -1;               /* SIR_Value */
static int hf_rnsap_sIR_ErrorValue = -1;          /* SIR_Error_Value */
static int hf_rnsap_transmittedCodePowerValue = -1;  /* Transmitted_Code_Power_Value */
static int hf_rnsap_rSCP = -1;                    /* RSCP_Value */
static int hf_rnsap_rxTimingDeviationValue = -1;  /* Rx_Timing_Deviation_Value */
static int hf_rnsap_roundTripTime = -1;           /* Round_Trip_Time_Value */
static int hf_rnsap_extension_DedicatedMeasurementValue = -1;  /* Extension_DedicatedMeasurementValue */
static int hf_rnsap_measurementAvailable1 = -1;   /* DedicatedMeasurementAvailable */
static int hf_rnsap_measurementnotAvailable1 = -1;  /* DedicatedMeasurementnotAvailable */
static int hf_rnsap_dedicatedmeasurementValue = -1;  /* DedicatedMeasurementValue */
static int hf_rnsap_cfn = -1;                     /* CFN */
static int hf_rnsap_separate_indication = -1;     /* NULL */
static int hf_rnsap_activate = -1;                /* Activate_Info */
static int hf_rnsap_deactivate = -1;              /* Deactivate_Info */
static int hf_rnsap_activation_type = -1;         /* Execution_Type */
static int hf_rnsap_initial_dl_tx_power = -1;     /* DL_Power */
static int hf_rnsap_firstRLS_Indicator = -1;      /* FirstRLS_Indicator */
static int hf_rnsap_propagation_delay = -1;       /* PropagationDelay */
static int hf_rnsap_deactivation_type = -1;       /* Execution_Type */
static int hf_rnsap_synchronised = -1;            /* CFN */
static int hf_rnsap_unsynchronised = -1;          /* NULL */
static int hf_rnsap_gPSTOW = -1;                  /* GPSTOW */
static int hf_rnsap_gPS_Status_Health = -1;       /* GPS_Status_Health */
static int hf_rnsap_satellite_DGPSCorrections_Information = -1;  /* T_satellite_DGPSCorrections_Information */
static int hf_rnsap_satellite_DGPSCorrections_Information_item = -1;  /* T_satellite_DGPSCorrections_Information_item */
static int hf_rnsap_sAT_ID = -1;                  /* SAT_ID */
static int hf_rnsap_iode_dgps = -1;               /* BIT_STRING_SIZE_8 */
static int hf_rnsap_uDRE = -1;                    /* UDRE */
static int hf_rnsap_pRC = -1;                     /* PRC */
static int hf_rnsap_range_Correction_Rate = -1;   /* Range_Correction_Rate */
static int hf_rnsap_pRCDeviation = -1;            /* PRCDeviation */
static int hf_rnsap_powerAdjustmentType = -1;     /* PowerAdjustmentType */
static int hf_rnsap_dLReferencePower = -1;        /* DL_Power */
static int hf_rnsap_dLReferencePowerList = -1;    /* DL_ReferencePowerInformationList */
static int hf_rnsap_maxAdjustmentStep = -1;       /* MaxAdjustmentStep */
static int hf_rnsap_adjustmentPeriod = -1;        /* AdjustmentPeriod */
static int hf_rnsap_adjustmentRatio = -1;         /* ScaledAdjustmentRatio */
static int hf_rnsap_DL_ReferencePowerInformationList_item = -1;  /* DL_ReferencePowerInformationItem */
static int hf_rnsap_common_DL_ReferencePowerInformation = -1;  /* DL_Power */
static int hf_rnsap_individual_DL_ReferencePowerInformation = -1;  /* DL_ReferencePowerInformationList */
static int hf_rnsap_DL_Timeslot_Information_item = -1;  /* DL_Timeslot_InformationItem */
static int hf_rnsap_DL_TimeslotLCR_Information_item = -1;  /* DL_TimeslotLCR_InformationItem */
static int hf_rnsap_DL_TimeSlot_ISCP_Info_item = -1;  /* DL_TimeSlot_ISCP_InfoItem */
static int hf_rnsap_dL_TimeslotISCP = -1;         /* DL_TimeslotISCP */
static int hf_rnsap_DL_TimeSlot_ISCP_LCR_Information_item = -1;  /* DL_TimeSlot_ISCP_LCR_InfoItem */
static int hf_rnsap_DSCH_FlowControlInformation_item = -1;  /* DSCH_FlowControlItem */
static int hf_rnsap_dSCH_SchedulingPriority = -1;  /* SchedulingPriorityIndicator */
static int hf_rnsap_mAC_c_sh_SDU_Lengths = -1;    /* MAC_c_sh_SDU_LengthList */
static int hf_rnsap_DSCH_TDD_Information_item = -1;  /* DSCH_TDD_InformationItem */
static int hf_rnsap_eAGCH_ERGCH_EHICH_FDD_ScramblingCode = -1;  /* DL_ScramblingCode */
static int hf_rnsap_eAGCH_ChannelisationCode = -1;  /* FDD_DL_ChannelisationCodeNumber */
static int hf_rnsap_primary_e_RNTI = -1;          /* E_RNTI */
static int hf_rnsap_secondary_e_RNTI = -1;        /* E_RNTI */
static int hf_rnsap_eRGCH_EHICH_ChannelisationCode = -1;  /* FDD_DL_ChannelisationCodeNumber */
static int hf_rnsap_eRGCH_SignatureSequence = -1;  /* ERGCH_SignatureSequence */
static int hf_rnsap_eHICH_SignatureSequence = -1;  /* EHICH_SignatureSequence */
static int hf_rnsap_serving_Grant_Value = -1;     /* E_Serving_Grant_Value */
static int hf_rnsap_primary_Secondary_Grant_Selector = -1;  /* E_Primary_Secondary_Grant_Selector */
static int hf_rnsap_e_RGCH_Release_Indicator = -1;  /* E_RGCH_Release_Indicator */
static int hf_rnsap_eDCH_MACdFlows_Information = -1;  /* EDCH_MACdFlows_Information */
static int hf_rnsap_EDCH_FDD_InformationResponse_item = -1;  /* EDCH_FDD_InformationResponseItem */
static int hf_rnsap_eDCH_MACdFlow_Specific_Information = -1;  /* EDCH_MACdFlow_Specific_InfoToModifyList */
static int hf_rnsap_EDCH_FDD_Update_Information_item = -1;  /* EDCH_FDD_Update_InfoItem */
static int hf_rnsap_eDCH_FDD_DL_ControlChannelInformation = -1;  /* EDCH_FDD_DL_ControlChannelInformation */
static int hf_rnsap_e_DCH_Non_Scheduled_Transmission_Grant = -1;  /* E_DCH_Non_Scheduled_Transmission_Grant_Items */
static int hf_rnsap_e_DCH_Scheduled_Transmission_Grant = -1;  /* NULL */
static int hf_rnsap_E_DCH_LogicalChannelInformation_item = -1;  /* E_DCH_LogicalChannelInformationItem */
static int hf_rnsap_logicalChannelId = -1;        /* LogicalChannelID */
static int hf_rnsap_mACes_GuaranteedBitRate = -1;  /* MACes_Guaranteed_Bitrate */
static int hf_rnsap_eDCH_DDI_Value = -1;          /* EDCH_DDI_Value */
static int hf_rnsap_mACd_PDU_Size_List = -1;      /* E_DCH_MACdPDU_SizeList */
static int hf_rnsap_E_DCH_MACdPDU_SizeList_item = -1;  /* E_DCH_MACdPDU_SizeListItem */
static int hf_rnsap_mACdPDU_Size = -1;            /* MACdPDU_Size */
static int hf_rnsap_E_DCH_LogicalChannelToModify_item = -1;  /* E_DCH_LogicalChannelToModifyItem */
static int hf_rnsap_mACd_PDU_Size_List1 = -1;     /* E_DCH_MACdPDU_SizeToModifyList */
static int hf_rnsap_E_DCH_MACdPDU_SizeToModifyList_item = -1;  /* E_DCH_MACdPDU_SizeListItem */
static int hf_rnsap_E_DCH_LogicalChannelToDelete_item = -1;  /* E_DCH_LogicalChannelToDeleteItem */
static int hf_rnsap_eDCH_MACdFlow_Specific_Information1 = -1;  /* EDCH_MACdFlow_Specific_InfoList */
static int hf_rnsap_EDCH_MACdFlow_Specific_InfoList_item = -1;  /* EDCH_MACdFlow_Specific_InfoItem */
static int hf_rnsap_eDSCH_MACdFlow_ID = -1;       /* EDCH_MACdFlow_ID */
static int hf_rnsap_tnlQoS = -1;                  /* TnlQos */
static int hf_rnsap_maxNr_Retransmissions_EDCH = -1;  /* MaxNr_Retransmissions_EDCH */
static int hf_rnsap_trafficClass = -1;            /* TrafficClass */
static int hf_rnsap_eDCH_HARQ_PO_FDD = -1;        /* E_DCH_HARQ_PO_FDD */
static int hf_rnsap_eDCH_MACdFlow_Multiplexing_List = -1;  /* E_DCH_MACdFlow_Multiplexing_List */
static int hf_rnsap_eDCH_Grant_Type_Information = -1;  /* E_DCH_Grant_Type_Information */
static int hf_rnsap_bundlingModeIndicator = -1;   /* BundlingModeIndicator */
static int hf_rnsap_eDCHLogicalChannelInformation = -1;  /* E_DCH_LogicalChannelInformation */
static int hf_rnsap_EDCH_MACdFlow_Specific_InfoToModifyList_item = -1;  /* EDCH_MACdFlow_Specific_InfoToModifyItem */
static int hf_rnsap_eDCH_LogicalChannelToAdd = -1;  /* E_DCH_LogicalChannelInformation */
static int hf_rnsap_eDCH_LogicalChannelToModify = -1;  /* E_DCH_LogicalChannelToModify */
static int hf_rnsap_eDCH_LogicalChannelToDelete = -1;  /* E_DCH_LogicalChannelToDelete */
static int hf_rnsap_EDCH_MACdFlows_To_Delete_item = -1;  /* EDCH_MACdFlows_To_Delete_Item */
static int hf_rnsap_maxBits_MACe_PDU_non_scheduled = -1;  /* Max_Bits_MACe_PDU_non_scheduled */
static int hf_rnsap_hARQ_Process_Allocation_2ms = -1;  /* HARQ_Process_Allocation_2ms_EDCH */
static int hf_rnsap_maxSet_E_DPDCHs = -1;         /* Max_Set_E_DPDCHs */
static int hf_rnsap_e_TFCS_Information = -1;      /* E_TFCS_Information */
static int hf_rnsap_e_TTI = -1;                   /* E_TTI */
static int hf_rnsap_e_DPCCH_PO = -1;              /* E_DPCCH_PO */
static int hf_rnsap_e_DCH_Serving_RL_in_this_DRNS = -1;  /* EDCH_Serving_RL_in_this_DRNS */
static int hf_rnsap_e_DCH_Serving_RL_not_in_this_DRNS = -1;  /* NULL */
static int hf_rnsap_e_DCH_Serving_RL_Id = -1;     /* RL_ID */
static int hf_rnsap_e_DCH_TFCI_Table_Index = -1;  /* E_DCH_TFCI_Table_Index */
static int hf_rnsap_e_DCH_Min_Set_E_TFCI = -1;    /* E_TFCI */
static int hf_rnsap_reference_E_TFCI_Information = -1;  /* Reference_E_TFCI_Information */
static int hf_rnsap_measurementTreshold = -1;     /* MeasurementThreshold */
static int hf_rnsap_measurementHysteresisTime = -1;  /* MeasurementHysteresisTime */
static int hf_rnsap_measurementIncreaseDecreaseThreshold = -1;  /* MeasurementIncreaseDecreaseThreshold */
static int hf_rnsap_measurementChangeTime = -1;   /* MeasurementChangeTime */
static int hf_rnsap_measurementThreshold1 = -1;   /* MeasurementThreshold */
static int hf_rnsap_measurementThreshold2 = -1;   /* MeasurementThreshold */
static int hf_rnsap_reportPeriodicity = -1;       /* ReportPeriodicity */
static int hf_rnsap_FACH_FlowControlInformation_item = -1;  /* FACH_FlowControlInformationItem */
static int hf_rnsap_fACH_SchedulingPriority = -1;  /* SchedulingPriorityIndicator */
static int hf_rnsap_fACH_InitialWindowSize = -1;  /* FACH_InitialWindowSize */
static int hf_rnsap_FACH_InformationList_item = -1;  /* FACH_InformationItem */
static int hf_rnsap_FACH_PCH_InformationList_item = -1;  /* FACH_PCH_InformationItem */
static int hf_rnsap_FDD_DCHs_to_Modify_item = -1;  /* FDD_DCHs_to_ModifyItem */
static int hf_rnsap_dCH_SpecificInformationList2 = -1;  /* FDD_DCHs_to_ModifySpecificInformationList */
static int hf_rnsap_FDD_DCHs_to_ModifySpecificInformationList_item = -1;  /* FDD_DCHs_to_ModifySpecificItem */
static int hf_rnsap_ul_TransportformatSet = -1;   /* TransportFormatSet */
static int hf_rnsap_dl_TransportformatSet = -1;   /* TransportFormatSet */
static int hf_rnsap_not_Used_dRACControl = -1;    /* NULL */
static int hf_rnsap_FDD_DL_CodeInformation_item = -1;  /* FDD_DL_CodeInformationItem */
static int hf_rnsap_dl_ScramblingCode = -1;       /* DL_ScramblingCode */
static int hf_rnsap_fDD_DL_ChannelisationCodeNumber = -1;  /* FDD_DL_ChannelisationCodeNumber */
static int hf_rnsap_transmission_Gap_Pattern_Sequence_ScramblingCode_Information = -1;  /* Transmission_Gap_Pattern_Sequence_ScramblingCode_Information */
static int hf_rnsap_wT = -1;                      /* INTEGER_1_4 */
static int hf_rnsap_GA_Cell_item = -1;            /* GA_Cell_item */
static int hf_rnsap_cell_GAIgeographicalCoordinate = -1;  /* GeographicalCoordinate */
static int hf_rnsap_pointWithUncertainty = -1;    /* GA_PointWithUnCertainty */
static int hf_rnsap_pointWithUncertaintyEllipse = -1;  /* GA_PointWithUnCertaintyEllipse */
static int hf_rnsap_pointWithAltitude = -1;       /* GA_PointWithAltitude */
static int hf_rnsap_pointWithAltitudeAndUncertaintyEllipsoid = -1;  /* GA_PointWithAltitudeAndUncertaintyEllipsoid */
static int hf_rnsap_ellipsoidArc = -1;            /* GA_EllipsoidArc */
static int hf_rnsap_directionOfAltitude = -1;     /* T_directionOfAltitude */
static int hf_rnsap_altitude = -1;                /* INTEGER_0_32767 */
static int hf_rnsap_geographicalCoordinates = -1;  /* GeographicalCoordinate */
static int hf_rnsap_innerRadius = -1;             /* INTEGER_0_65535 */
static int hf_rnsap_uncertaintyRadius = -1;       /* INTEGER_0_127 */
static int hf_rnsap_offsetAngle = -1;             /* INTEGER_0_179 */
static int hf_rnsap_includedAngle = -1;           /* INTEGER_0_179 */
static int hf_rnsap_confidence = -1;              /* INTEGER_0_127 */
static int hf_rnsap_altitudeAndDirection = -1;    /* GA_AltitudeAndDirection */
static int hf_rnsap_uncertaintyEllipse = -1;      /* GA_UncertaintyEllipse */
static int hf_rnsap_uncertaintyAltitude = -1;     /* INTEGER_0_127 */
static int hf_rnsap_uncertaintySemi_major = -1;   /* INTEGER_0_127 */
static int hf_rnsap_uncertaintySemi_minor = -1;   /* INTEGER_0_127 */
static int hf_rnsap_orientationOfMajorAxis = -1;  /* INTEGER_0_179 */
static int hf_rnsap_uncertaintyCode = -1;         /* INTEGER_0_127 */
static int hf_rnsap_geographicalCoordinate = -1;  /* GeographicalCoordinate */
static int hf_rnsap_latitudeSign = -1;            /* T_latitudeSign */
static int hf_rnsap_latitude = -1;                /* INTEGER_0_8388607 */
static int hf_rnsap_longitude = -1;               /* INTEGER_M8388608_8388607 */
static int hf_rnsap_sI = -1;                      /* GERAN_SystemInfo */
static int hf_rnsap_pSI = -1;                     /* GERAN_SystemInfo */
static int hf_rnsap_GERAN_SystemInfo_item = -1;   /* GERAN_SystemInfo_item */
static int hf_rnsap_gERAN_SI_block = -1;          /* OCTET_STRING_SIZE_1_23 */
static int hf_rnsap_wna_alm = -1;                 /* BIT_STRING_SIZE_8 */
static int hf_rnsap_satellite_Almanac_Information = -1;  /* T_satellite_Almanac_Information */
static int hf_rnsap_satellite_Almanac_Information_item = -1;  /* T_satellite_Almanac_Information_item */
static int hf_rnsap_dATA_ID = -1;                 /* DATA_ID */
static int hf_rnsap_gps_e_alm = -1;               /* BIT_STRING_SIZE_16 */
static int hf_rnsap_gps_toa_alm = -1;             /* BIT_STRING_SIZE_8 */
static int hf_rnsap_gps_delta_I_alm = -1;         /* BIT_STRING_SIZE_16 */
static int hf_rnsap_omegadot_alm = -1;            /* BIT_STRING_SIZE_16 */
static int hf_rnsap_svhealth_alm = -1;            /* BIT_STRING_SIZE_8 */
static int hf_rnsap_gps_a_sqrt_alm = -1;          /* BIT_STRING_SIZE_24 */
static int hf_rnsap_omegazero_alm = -1;           /* BIT_STRING_SIZE_24 */
static int hf_rnsap_m_zero_alm = -1;              /* BIT_STRING_SIZE_24 */
static int hf_rnsap_gps_omega_alm = -1;           /* BIT_STRING_SIZE_24 */
static int hf_rnsap_gps_af_zero_alm = -1;         /* BIT_STRING_SIZE_11 */
static int hf_rnsap_gps_af_one_alm = -1;          /* BIT_STRING_SIZE_11 */
static int hf_rnsap_sVGlobalHealth_alm = -1;      /* BIT_STRING_SIZE_364 */
static int hf_rnsap_Satellite_Almanac_Information_ExtItem_item = -1;  /* Satellite_Almanac_Information_ExtItem_item */
static int hf_rnsap_GPSInformation_item = -1;     /* GPSInformation_item */
static int hf_rnsap_gPSInformationItem = -1;      /* T_gPSInformationItem */
static int hf_rnsap_alpha_zero_ionos = -1;        /* BIT_STRING_SIZE_8 */
static int hf_rnsap_alpha_one_ionos = -1;         /* BIT_STRING_SIZE_8 */
static int hf_rnsap_alpha_two_ionos = -1;         /* BIT_STRING_SIZE_8 */
static int hf_rnsap_alpha_three_ionos = -1;       /* BIT_STRING_SIZE_8 */
static int hf_rnsap_beta_zero_ionos = -1;         /* BIT_STRING_SIZE_8 */
static int hf_rnsap_beta_one_ionos = -1;          /* BIT_STRING_SIZE_8 */
static int hf_rnsap_beta_two_ionos = -1;          /* BIT_STRING_SIZE_8 */
static int hf_rnsap_beta_three_ionos = -1;        /* BIT_STRING_SIZE_8 */
static int hf_rnsap_GPS_NavigationModel_and_TimeRecovery_item = -1;  /* GPS_NavigationModel_and_TimeRecovery_item */
static int hf_rnsap_tx_tow_nav = -1;              /* INTEGER_0_1048575 */
static int hf_rnsap_tlm_message_nav = -1;         /* BIT_STRING_SIZE_14 */
static int hf_rnsap_tlm_revd_c_nav = -1;          /* BIT_STRING_SIZE_2 */
static int hf_rnsap_ho_word_nav = -1;             /* BIT_STRING_SIZE_22 */
static int hf_rnsap_w_n_nav = -1;                 /* BIT_STRING_SIZE_10 */
static int hf_rnsap_ca_or_p_on_l2_nav = -1;       /* BIT_STRING_SIZE_2 */
static int hf_rnsap_user_range_accuracy_index_nav = -1;  /* BIT_STRING_SIZE_4 */
static int hf_rnsap_sv_health_nav = -1;           /* BIT_STRING_SIZE_6 */
static int hf_rnsap_iodc_nav = -1;                /* BIT_STRING_SIZE_10 */
static int hf_rnsap_l2_p_dataflag_nav = -1;       /* BIT_STRING_SIZE_1 */
static int hf_rnsap_sf1_reserved_nav = -1;        /* BIT_STRING_SIZE_87 */
static int hf_rnsap_t_gd_nav = -1;                /* BIT_STRING_SIZE_8 */
static int hf_rnsap_t_oc_nav = -1;                /* BIT_STRING_SIZE_16 */
static int hf_rnsap_a_f_2_nav = -1;               /* BIT_STRING_SIZE_8 */
static int hf_rnsap_a_f_1_nav = -1;               /* BIT_STRING_SIZE_16 */
static int hf_rnsap_a_f_zero_nav = -1;            /* BIT_STRING_SIZE_22 */
static int hf_rnsap_c_rs_nav = -1;                /* BIT_STRING_SIZE_16 */
static int hf_rnsap_delta_n_nav = -1;             /* BIT_STRING_SIZE_16 */
static int hf_rnsap_m_zero_nav = -1;              /* BIT_STRING_SIZE_32 */
static int hf_rnsap_c_uc_nav = -1;                /* BIT_STRING_SIZE_16 */
static int hf_rnsap_gps_e_nav = -1;               /* BIT_STRING_SIZE_32 */
static int hf_rnsap_c_us_nav = -1;                /* BIT_STRING_SIZE_16 */
static int hf_rnsap_a_sqrt_nav = -1;              /* BIT_STRING_SIZE_32 */
static int hf_rnsap_t_oe_nav = -1;                /* BIT_STRING_SIZE_16 */
static int hf_rnsap_fit_interval_flag_nav = -1;   /* BIT_STRING_SIZE_1 */
static int hf_rnsap_aodo_nav = -1;                /* BIT_STRING_SIZE_5 */
static int hf_rnsap_c_ic_nav = -1;                /* BIT_STRING_SIZE_16 */
static int hf_rnsap_omega_zero_nav = -1;          /* BIT_STRING_SIZE_32 */
static int hf_rnsap_c_is_nav = -1;                /* BIT_STRING_SIZE_16 */
static int hf_rnsap_i_zero_nav = -1;              /* BIT_STRING_SIZE_32 */
static int hf_rnsap_c_rc_nav = -1;                /* BIT_STRING_SIZE_16 */
static int hf_rnsap_gps_omega_nav = -1;           /* BIT_STRING_SIZE_32 */
static int hf_rnsap_omegadot_nav = -1;            /* BIT_STRING_SIZE_24 */
static int hf_rnsap_idot_nav = -1;                /* BIT_STRING_SIZE_14 */
static int hf_rnsap_spare_zero_fill = -1;         /* BIT_STRING_SIZE_20 */
static int hf_rnsap_badSatellites = -1;           /* BadSatellites */
static int hf_rnsap_noBadSatellite = -1;          /* NULL */
static int hf_rnsap_a_one_utc = -1;               /* BIT_STRING_SIZE_24 */
static int hf_rnsap_a_zero_utc = -1;              /* BIT_STRING_SIZE_32 */
static int hf_rnsap_t_ot_utc = -1;                /* BIT_STRING_SIZE_8 */
static int hf_rnsap_delta_t_ls_utc = -1;          /* BIT_STRING_SIZE_8 */
static int hf_rnsap_w_n_t_utc = -1;               /* BIT_STRING_SIZE_8 */
static int hf_rnsap_w_n_lsf_utc = -1;             /* BIT_STRING_SIZE_8 */
static int hf_rnsap_dn_utc = -1;                  /* BIT_STRING_SIZE_8 */
static int hf_rnsap_delta_t_lsf_utc = -1;         /* BIT_STRING_SIZE_8 */
static int hf_rnsap_guaranteed_UL_Rate = -1;      /* Guaranteed_Rate */
static int hf_rnsap_guaranteed_DL_Rate = -1;      /* Guaranteed_Rate */
static int hf_rnsap_implicit = -1;                /* HARQ_MemoryPartitioning_Implicit */
static int hf_rnsap_explicit = -1;                /* HARQ_MemoryPartitioning_Explicit */
static int hf_rnsap_number_of_Processes = -1;     /* INTEGER_1_8_ */
static int hf_rnsap_hARQ_MemoryPartitioningList = -1;  /* HARQ_MemoryPartitioningList */
static int hf_rnsap_HARQ_MemoryPartitioningList_item = -1;  /* HARQ_MemoryPartitioningItem */
static int hf_rnsap_process_Memory_Size = -1;     /* T_process_Memory_Size */
static int hf_rnsap_hSDSCH_MACdFlows_Information = -1;  /* HSDSCH_MACdFlows_Information */
static int hf_rnsap_uE_Capabilities_Info = -1;    /* UE_Capabilities_Info */
static int hf_rnsap_mAChs_Reordering_Buffer_Size_for_RLC_UM = -1;  /* MAChsReorderingBufferSize_for_RLC_UM */
static int hf_rnsap_cqiFeedback_CycleK = -1;      /* CQI_Feedback_Cycle */
static int hf_rnsap_cqiRepetitionFactor = -1;     /* CQI_RepetitionFactor */
static int hf_rnsap_ackNackRepetitionFactor = -1;  /* AckNack_RepetitionFactor */
static int hf_rnsap_cqiPowerOffset = -1;          /* CQI_Power_Offset */
static int hf_rnsap_ackPowerOffset = -1;          /* Ack_Power_Offset */
static int hf_rnsap_nackPowerOffset = -1;         /* Nack_Power_Offset */
static int hf_rnsap_hsscch_PowerOffset = -1;      /* HSSCCH_PowerOffset */
static int hf_rnsap_hSDSCH_MACdFlow_Specific_InfoList_Response = -1;  /* HSDSCH_MACdFlow_Specific_InfoList_Response */
static int hf_rnsap_hSSCCH_Specific_InfoList_Response = -1;  /* HSSCCH_FDD_Specific_InfoList_Response */
static int hf_rnsap_hSPDSCH_and_HSSCCH_ScramblingCode = -1;  /* DL_ScramblingCode */
static int hf_rnsap_measurement_Power_Offset = -1;  /* Measurement_Power_Offset */
static int hf_rnsap_hARQ_MemoryPartitioning = -1;  /* HARQ_MemoryPartitioning */
static int hf_rnsap_hSDSCH_MACdFlow_Specific_InfoList_to_Modify = -1;  /* HSDSCH_MACdFlow_Specific_InfoList_to_Modify */
static int hf_rnsap_priorityQueue_Info_to_Modify = -1;  /* PriorityQueue_InfoList_to_Modify */
static int hf_rnsap_hSSCCH_CodeChangeGrant = -1;  /* HSSCCH_Code_Change_Grant */
static int hf_rnsap_tDDAckNackPowerOffset = -1;   /* TDD_AckNack_Power_Offset */
static int hf_rnsap_priorityQueueInfotoModifyUnsynchronised = -1;  /* PriorityQueue_InfoList_to_Modify_Unsynchronised */
static int hf_rnsap_HSDSCH_MACdFlow_Specific_InfoList_item = -1;  /* HSDSCH_MACdFlow_Specific_InfoItem */
static int hf_rnsap_HSDSCH_MACdFlow_Specific_InfoList_Response_item = -1;  /* HSDSCH_MACdFlow_Specific_InfoItem_Response */
static int hf_rnsap_hSDSCH_Initial_Capacity_Allocation = -1;  /* HSDSCH_Initial_Capacity_Allocation */
static int hf_rnsap_HSDSCH_MACdFlow_Specific_InfoList_to_Modify_item = -1;  /* HSDSCH_MACdFlow_Specific_InfoItem_to_Modify */
static int hf_rnsap_hSDSCH_MACdFlow_Specific_Info = -1;  /* HSDSCH_MACdFlow_Specific_InfoList */
static int hf_rnsap_priorityQueue_Info = -1;      /* PriorityQueue_InfoList */
static int hf_rnsap_HSDSCH_MACdFlows_to_Delete_item = -1;  /* HSDSCH_MACdFlows_to_Delete_Item */
static int hf_rnsap_hsDSCH_MACdFlow_ID = -1;      /* HSDSCH_MACdFlow_ID */
static int hf_rnsap_HSDSCH_Initial_Capacity_Allocation_item = -1;  /* HSDSCH_Initial_Capacity_AllocationItem */
static int hf_rnsap_maximum_MACdPDU_Size = -1;    /* MACdPDU_Size */
static int hf_rnsap_hSDSCH_InitialWindowSize = -1;  /* HSDSCH_InitialWindowSize */
static int hf_rnsap_tDD_AckNack_Power_Offset = -1;  /* TDD_AckNack_Power_Offset */
static int hf_rnsap_hSSCCH_TDD_Specific_InfoList_Response = -1;  /* HSSCCH_TDD_Specific_InfoList_Response */
static int hf_rnsap_hSSCCH_TDD_Specific_InfoList_Response_LCR = -1;  /* HSSCCH_TDD_Specific_InfoList_Response_LCR */
static int hf_rnsap_hSPDSCH_TDD_Specific_InfoList_Response = -1;  /* HSPDSCH_TDD_Specific_InfoList_Response */
static int hf_rnsap_hSPDSCH_TDD_Specific_InfoList_Response_LCR = -1;  /* HSPDSCH_TDD_Specific_InfoList_Response_LCR */
static int hf_rnsap_HSPDSCH_TDD_Specific_InfoList_Response_item = -1;  /* HSPDSCH_TDD_Specific_InfoItem_Response */
static int hf_rnsap_HSPDSCH_TDD_Specific_InfoList_Response_LCR_item = -1;  /* HSPDSCH_TDD_Specific_InfoItem_Response_LCR */
static int hf_rnsap_HSSCCH_FDD_Specific_InfoList_Response_item = -1;  /* HSSCCH_FDD_Specific_InfoItem_Response */
static int hf_rnsap_code_Number = -1;             /* INTEGER_0_127 */
static int hf_rnsap_HSSCCH_TDD_Specific_InfoList_Response_item = -1;  /* HSSCCH_TDD_Specific_InfoItem_Response */
static int hf_rnsap_hSSICH_Info = -1;             /* HSSICH_Info */
static int hf_rnsap_HSSCCH_TDD_Specific_InfoList_Response_LCR_item = -1;  /* HSSCCH_TDD_Specific_InfoItem_Response_LCR */
static int hf_rnsap_first_TDD_ChannelisationCode = -1;  /* TDD_ChannelisationCode */
static int hf_rnsap_second_TDD_ChannelisationCode = -1;  /* TDD_ChannelisationCode */
static int hf_rnsap_hSSICH_InfoLCR = -1;          /* HSSICH_InfoLCR */
static int hf_rnsap_hsSICH_ID = -1;               /* HS_SICH_ID */
static int hf_rnsap_failed_HS_SICH = -1;          /* HS_SICH_failed */
static int hf_rnsap_missed_HS_SICH = -1;          /* HS_SICH_missed */
static int hf_rnsap_total_HS_SICH = -1;           /* HS_SICH_total */
static int hf_rnsap_hsSCCHCodeChangeIndicator = -1;  /* HSSCCH_CodeChangeIndicator */
static int hf_rnsap_onDemand = -1;                /* NULL */
static int hf_rnsap_periodic = -1;                /* PeriodicInformation */
static int hf_rnsap_onModification = -1;          /* OnModificationInformation */
static int hf_rnsap_min = -1;                     /* INTEGER_1_60_ */
static int hf_rnsap_hour = -1;                    /* INTEGER_1_24_ */
static int hf_rnsap_dGPSThreshold = -1;           /* DGPSThreshold */
static int hf_rnsap_informationTypeItem = -1;     /* T_informationTypeItem */
static int hf_rnsap_gPSInformation = -1;          /* GPSInformation */
static int hf_rnsap_iPDL_FDD_Parameters = -1;     /* IPDL_FDD_Parameters */
static int hf_rnsap_iPDL_TDD_Parameters = -1;     /* IPDL_TDD_Parameters */
static int hf_rnsap_extension_IPDLParameters = -1;  /* Extension_IPDLParameters */
static int hf_rnsap_iPSpacingFDD = -1;            /* IPSpacingFDD */
static int hf_rnsap_iPLength = -1;                /* IPLength */
static int hf_rnsap_iPOffset = -1;                /* IPOffset */
static int hf_rnsap_seed = -1;                    /* Seed */
static int hf_rnsap_burstModeParameters = -1;     /* BurstModeParameters */
static int hf_rnsap_iPSpacingTDD = -1;            /* IPSpacingTDD */
static int hf_rnsap_iPStart = -1;                 /* IPStart */
static int hf_rnsap_iPSlot = -1;                  /* IPSlot */
static int hf_rnsap_iP_P_CCPCH = -1;              /* IP_P_CCPCH */
static int hf_rnsap_iPSub = -1;                   /* IPSub */
static int hf_rnsap_uplinkLoadValue = -1;         /* INTEGER_0_100 */
static int hf_rnsap_downlinkLoadValue = -1;       /* INTEGER_0_100 */
static int hf_rnsap_MAC_c_sh_SDU_LengthList_item = -1;  /* MAC_c_sh_SDU_Length */
static int hf_rnsap_MACdPDU_Size_IndexList_item = -1;  /* MACdPDU_Size_IndexItem */
static int hf_rnsap_sID = -1;                     /* SID */
static int hf_rnsap_MACdPDU_Size_IndexList_to_Modify_item = -1;  /* MACdPDU_Size_IndexItem_to_Modify */
static int hf_rnsap_accessPointName = -1;         /* AccessPointName */
static int hf_rnsap_iPMulticastAddress = -1;      /* IPMulticastAddress */
static int hf_rnsap_MBMS_Bearer_Service_List_item = -1;  /* TMGI */
static int hf_rnsap_transmissionMode = -1;        /* TransmissionMode */
static int hf_rnsap_preferredFrequencyLayer = -1;  /* UARFCN */
static int hf_rnsap_tMGI = -1;                    /* TMGI */
static int hf_rnsap_pTM_Cell_List = -1;           /* PTMCellList */
static int hf_rnsap_pTP_Cell_List = -1;           /* PTPCellList */
static int hf_rnsap_not_Provided_Cell_List = -1;  /* NotProvidedCellList */
static int hf_rnsap_affectedUEInformationForMBMS = -1;  /* AffectedUEInformationForMBMS */
static int hf_rnsap_preferredFrequencyLayerInfo = -1;  /* PreferredFrequencyLayerInfo */
static int hf_rnsap_sir = -1;                     /* SIR_Value_IncrDecrThres */
static int hf_rnsap_sir_error = -1;               /* SIR_Error_Value_IncrDecrThres */
static int hf_rnsap_transmitted_code_power = -1;  /* Transmitted_Code_Power_Value_IncrDecrThres */
static int hf_rnsap_rscp = -1;                    /* RSCP_Value_IncrDecrThres */
static int hf_rnsap_round_trip_time = -1;         /* Round_Trip_Time_IncrDecrThres */
static int hf_rnsap_extension_MeasurementIncreaseDecreaseThreshold = -1;  /* Extension_MeasurementIncreaseDecreaseThreshold */
static int hf_rnsap_sir1 = -1;                    /* SIR_Value */
static int hf_rnsap_sir_error1 = -1;              /* SIR_Error_Value */
static int hf_rnsap_transmitted_code_power1 = -1;  /* Transmitted_Code_Power_Value */
static int hf_rnsap_rscp1 = -1;                   /* RSCP_Value */
static int hf_rnsap_rx_timing_deviation = -1;     /* Rx_Timing_Deviation_Value */
static int hf_rnsap_round_trip_time1 = -1;        /* Round_Trip_Time_Value */
static int hf_rnsap_extension_MeasurementThreshold = -1;  /* Extension_MeasurementThreshold */
static int hf_rnsap_type1 = -1;                   /* T_type1 */
static int hf_rnsap_midambleConfigurationBurstType1And3 = -1;  /* MidambleConfigurationBurstType1And3 */
static int hf_rnsap_midambleAllocationMode = -1;  /* T_midambleAllocationMode */
static int hf_rnsap_defaultMidamble = -1;         /* NULL */
static int hf_rnsap_commonMidamble = -1;          /* NULL */
static int hf_rnsap_ueSpecificMidamble = -1;      /* MidambleShiftLong */
static int hf_rnsap_type2 = -1;                   /* T_type2 */
static int hf_rnsap_midambleConfigurationBurstType2 = -1;  /* MidambleConfigurationBurstType2 */
static int hf_rnsap_midambleAllocationMode1 = -1;  /* T_midambleAllocationMode1 */
static int hf_rnsap_ueSpecificMidamble1 = -1;     /* MidambleShiftShort */
static int hf_rnsap_type3 = -1;                   /* T_type3 */
static int hf_rnsap_midambleAllocationMode2 = -1;  /* T_midambleAllocationMode2 */
static int hf_rnsap_midambleAllocationMode3 = -1;  /* MidambleAllocationMode */
static int hf_rnsap_midambleShift = -1;           /* MidambleShiftLong */
static int hf_rnsap_midambleConfigurationLCR = -1;  /* MidambleConfigurationLCR */
static int hf_rnsap_addPriorityQueue = -1;        /* PriorityQueue_InfoItem_to_Add */
static int hf_rnsap_modifyPriorityQueue = -1;     /* PriorityQueue_InfoItem_to_Modify */
static int hf_rnsap_deletePriorityQueue = -1;     /* PriorityQueue_Id */
static int hf_rnsap_gERAN_SI_Type = -1;           /* GERAN_SI_Type */
static int hf_rnsap_Neighbouring_UMTS_CellInformation_item = -1;  /* ProtocolIE_Single_Container */
static int hf_rnsap_rNC_ID = -1;                  /* RNC_ID */
static int hf_rnsap_cN_PS_DomainIdentifier = -1;  /* CN_PS_DomainIdentifier */
static int hf_rnsap_cN_CS_DomainIdentifier = -1;  /* CN_CS_DomainIdentifier */
static int hf_rnsap_neighbouring_FDD_CellInformation = -1;  /* Neighbouring_FDD_CellInformation */
static int hf_rnsap_neighbouring_TDD_CellInformation = -1;  /* Neighbouring_TDD_CellInformation */
static int hf_rnsap_Neighbouring_FDD_CellInformation_item = -1;  /* Neighbouring_FDD_CellInformationItem */
static int hf_rnsap_uARFCNforNu = -1;             /* UARFCN */
static int hf_rnsap_uARFCNforNd = -1;             /* UARFCN */
static int hf_rnsap_cellIndividualOffset = -1;    /* CellIndividualOffset */
static int hf_rnsap_txDiversityIndicator = -1;    /* TxDiversityIndicator */
static int hf_rnsap_sTTD_SupportIndicator = -1;   /* STTD_SupportIndicator */
static int hf_rnsap_closedLoopMode1_SupportIndicator = -1;  /* ClosedLoopMode1_SupportIndicator */
static int hf_rnsap_not_used_closedLoopMode2_SupportIndicator = -1;  /* NULL */
static int hf_rnsap_uARFCN = -1;                  /* UARFCN */
static int hf_rnsap_Neighbouring_GSM_CellInformationIEs_item = -1;  /* Neighbouring_GSM_CellInformationItem */
static int hf_rnsap_bSIC = -1;                    /* BSIC */
static int hf_rnsap_band_Indicator = -1;          /* Band_Indicator */
static int hf_rnsap_bCCH_ARFCN = -1;              /* BCCH_ARFCN */
static int hf_rnsap_Neighbouring_TDD_CellInformation_item = -1;  /* Neighbouring_TDD_CellInformationItem */
static int hf_rnsap_dPCHConstantValue = -1;       /* DPCHConstantValue */
static int hf_rnsap_Neighbouring_LCR_TDD_CellInformation_item = -1;  /* Neighbouring_LCR_TDD_CellInformationItem */
static int hf_rnsap_restrictionStateIndicator = -1;  /* RestrictionStateIndicator */
static int hf_rnsap_NotProvidedCellList_item = -1;  /* MBMSChannelTypeCellList */
static int hf_rnsap_uplinkNRTLoadInformationValue = -1;  /* INTEGER_0_3 */
static int hf_rnsap_downlinkNRTLoadInformationValue = -1;  /* INTEGER_0_3 */
static int hf_rnsap_measurementThreshold = -1;    /* MeasurementThreshold */
static int hf_rnsap_informationThreshold = -1;    /* InformationThreshold */
static int hf_rnsap_PCH_InformationList_item = -1;  /* PCH_InformationItem */
static int hf_rnsap_informationReportPeriodicity = -1;  /* InformationReportPeriodicity */
static int hf_rnsap_imsi = -1;                    /* IMSI */
static int hf_rnsap_defaultPreferredFrequency = -1;  /* UARFCN */
static int hf_rnsap_additionalPreferredFrequency = -1;  /* AdditionalPreferredFrequency */
static int hf_rnsap_PriorityQueue_InfoList_item = -1;  /* PriorityQueue_InfoItem */
static int hf_rnsap_priorityQueue_Id = -1;        /* PriorityQueue_Id */
static int hf_rnsap_associatedHSDSCH_MACdFlow = -1;  /* HSDSCH_MACdFlow_ID */
static int hf_rnsap_t1 = -1;                      /* T1 */
static int hf_rnsap_discardTimer = -1;            /* DiscardTimer */
static int hf_rnsap_mAC_hsWindowSize = -1;        /* MAC_hsWindowSize */
static int hf_rnsap_mAChsGuaranteedBitRate = -1;  /* MAChsGuaranteedBitRate */
static int hf_rnsap_mACdPDU_Size_Index = -1;      /* MACdPDU_Size_IndexList */
static int hf_rnsap_rLC_Mode = -1;                /* RLC_Mode */
static int hf_rnsap_PriorityQueue_InfoList_to_Modify_item = -1;  /* ModifyPriorityQueue */
static int hf_rnsap_mACdPDU_Size_Index_to_Modify = -1;  /* MACdPDU_Size_IndexList_to_Modify */
static int hf_rnsap_PriorityQueue_InfoList_to_Modify_Unsynchronised_item = -1;  /* PriorityQueue_InfoItem_to_Modify_Unsynchronised */
static int hf_rnsap_priorityQueueId = -1;         /* PriorityQueue_Id */
static int hf_rnsap_mBMSChannelTypeInfo = -1;     /* MBMSChannelTypeInfo */
static int hf_rnsap_mBMSPreferredFreqLayerInfo = -1;  /* MBMSPreferredFreqLayerInfo */
static int hf_rnsap_PTMCellList_item = -1;        /* MBMSChannelTypeCellList */
static int hf_rnsap_PTPCellList_item = -1;        /* MBMSChannelTypeCellList */
static int hf_rnsap_RB_Info_item = -1;            /* RB_Identity */
static int hf_rnsap_Reference_E_TFCI_Information_item = -1;  /* Reference_E_TFCI_Information_Item */
static int hf_rnsap_reference_E_TFCI = -1;        /* E_TFCI */
static int hf_rnsap_reference_E_TFCI_PO = -1;     /* Reference_E_TFCI_PO */
static int hf_rnsap_periodic1 = -1;               /* Periodic */
static int hf_rnsap_eventA = -1;                  /* EventA */
static int hf_rnsap_eventB = -1;                  /* EventB */
static int hf_rnsap_eventC = -1;                  /* EventC */
static int hf_rnsap_eventD = -1;                  /* EventD */
static int hf_rnsap_eventE = -1;                  /* EventE */
static int hf_rnsap_eventF = -1;                  /* EventF */
static int hf_rnsap_extension_ReportCharacteristics = -1;  /* Extension_ReportCharacteristics */
static int hf_rnsap_ten_msec = -1;                /* INTEGER_1_6000_ */
static int hf_rnsap_gA_AccessPointPositionwithAltitude = -1;  /* GA_AccessPointPositionwithOptionalAltitude */
static int hf_rnsap_iPDLParameters = -1;          /* IPDLParameters */
static int hf_rnsap_dGPSCorrections = -1;         /* DGPSCorrections */
static int hf_rnsap_gPS_NavigationModel_and_TimeRecovery = -1;  /* GPS_NavigationModel_and_TimeRecovery */
static int hf_rnsap_gPS_Ionospheric_Model = -1;   /* GPS_Ionospheric_Model */
static int hf_rnsap_gPS_UTC_Model = -1;           /* GPS_UTC_Model */
static int hf_rnsap_gPS_Almanac = -1;             /* GPS_Almanac */
static int hf_rnsap_gPS_RealTime_Integrity = -1;  /* GPS_RealTime_Integrity */
static int hf_rnsap_gPS_RX_POS = -1;              /* GPS_RX_POS */
static int hf_rnsap_sFNSFN_GA_AccessPointPosition = -1;  /* GA_AccessPointPositionwithOptionalAltitude */
static int hf_rnsap_informationAvailable = -1;    /* InformationAvailable */
static int hf_rnsap_informationNotAvailable = -1;  /* InformationNotAvailable */
static int hf_rnsap_RL_Specific_DCH_Info_item = -1;  /* RL_Specific_DCH_Info_Item */
static int hf_rnsap_dCH_id = -1;                  /* DCH_ID */
static int hf_rnsap_RL_Specific_EDCH_Information_item = -1;  /* RL_Specific_EDCH_InfoItem */
static int hf_rnsap_uplinkRTLoadValue = -1;       /* INTEGER_0_100 */
static int hf_rnsap_downlinkRTLoadValue = -1;     /* INTEGER_0_100 */
static int hf_rnsap_sAC = -1;                     /* SAC */
static int hf_rnsap_secondary_CCPCH_TDD_InformationList = -1;  /* Secondary_CCPCH_TDD_InformationList */
static int hf_rnsap_fACH_InformationList = -1;    /* FACH_InformationList */
static int hf_rnsap_pCH_InformationList = -1;     /* PCH_InformationList */
static int hf_rnsap_new_secondary_CPICH = -1;     /* Secondary_CPICH_Information */
static int hf_rnsap_secondary_CPICH_shall_not_be_used = -1;  /* NULL */
static int hf_rnsap_secondary_LCR_CCPCH_TDD_InformationList = -1;  /* Secondary_LCR_CCPCH_TDD_InformationList */
static int hf_rnsap_Secondary_CCPCH_TDD_InformationList_item = -1;  /* Secondary_CCPCH_TDD_InformationItem */
static int hf_rnsap_secondary_CCPCH_TDD_Code_Information = -1;  /* Secondary_CCPCH_TDD_Code_Information */
static int hf_rnsap_tDD_PhysicalChannelOffset = -1;  /* TDD_PhysicalChannelOffset */
static int hf_rnsap_Secondary_LCR_CCPCH_TDD_InformationList_item = -1;  /* Secondary_LCR_CCPCH_TDD_InformationItem */
static int hf_rnsap_secondary_LCR_CCPCH_TDD_Code_Information = -1;  /* Secondary_LCR_CCPCH_TDD_Code_Information */
static int hf_rnsap_Secondary_CCPCH_TDD_Code_Information_item = -1;  /* Secondary_CCPCH_TDD_Code_InformationItem */
static int hf_rnsap_Secondary_LCR_CCPCH_TDD_Code_Information_item = -1;  /* Secondary_LCR_CCPCH_TDD_Code_InformationItem */
static int hf_rnsap_s_CCPCH_TimeSlotFormat_LCR = -1;  /* TDD_DL_DPCH_TimeSlotFormat_LCR */
static int hf_rnsap_sFNSFNChangeLimit = -1;       /* SFNSFNChangeLimit */
static int hf_rnsap_predictedSFNSFNDeviationLimit = -1;  /* PredictedSFNSFNDeviationLimit */
static int hf_rnsap_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation = -1;  /* T_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation */
static int hf_rnsap_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item = -1;  /* T_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item */
static int hf_rnsap_sFNSFNValue = -1;             /* SFNSFNValue */
static int hf_rnsap_sFNSFNQuality = -1;           /* SFNSFNQuality */
static int hf_rnsap_sFNSFNDriftRate = -1;         /* SFNSFNDriftRate */
static int hf_rnsap_sFNSFNDriftRateQuality = -1;  /* SFNSFNDriftRateQuality */
static int hf_rnsap_sFNSFNTimeStampInformation = -1;  /* SFNSFNTimeStampInformation */
static int hf_rnsap_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation = -1;  /* T_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation */
static int hf_rnsap_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item = -1;  /* T_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item */
static int hf_rnsap_sFNSFNTimeStamp_FDD = -1;     /* SFN */
static int hf_rnsap_sFNSFNTimeStamp_TDD = -1;     /* SFNSFNTimeStamp_TDD */
static int hf_rnsap_sFN = -1;                     /* SFN */
static int hf_rnsap_sFNSFN_FDD = -1;              /* SFNSFN_FDD */
static int hf_rnsap_sFNSFN_TDD = -1;              /* SFNSFN_TDD */
static int hf_rnsap_listOfSNAs = -1;              /* ListOfSNAs */
static int hf_rnsap_ListOfSNAs_item = -1;         /* SNACode */
static int hf_rnsap_sRNTI_BitMaskIndex = -1;      /* T_sRNTI_BitMaskIndex */
static int hf_rnsap_n_INSYNC_IND = -1;            /* INTEGER_1_256 */
static int hf_rnsap_n_OUTSYNC_IND = -1;           /* INTEGER_1_256 */
static int hf_rnsap_t_RLFAILURE = -1;             /* INTEGER_0_255 */
static int hf_rnsap_maxSYNC_UL_transmissions = -1;  /* T_maxSYNC_UL_transmissions */
static int hf_rnsap_powerRampStep = -1;           /* INTEGER_0_3_ */
static int hf_rnsap_modulation = -1;              /* Modulation */
static int hf_rnsap_TDD_DCHs_to_Modify_item = -1;  /* TDD_DCHs_to_ModifyItem */
static int hf_rnsap_dCH_SpecificInformationList3 = -1;  /* TDD_DCHs_to_ModifySpecificInformationList */
static int hf_rnsap_TDD_DCHs_to_ModifySpecificInformationList_item = -1;  /* TDD_DCHs_to_ModifySpecificItem */
static int hf_rnsap_ul_CCTrCH_ID = -1;            /* CCTrCH_ID */
static int hf_rnsap_dl_CCTrCH_ID = -1;            /* CCTrCH_ID */
static int hf_rnsap_TDD_DL_Code_Information_item = -1;  /* TDD_DL_Code_InformationItem */
static int hf_rnsap_TDD_DL_Code_LCR_Information_item = -1;  /* TDD_DL_Code_LCR_InformationItem */
static int hf_rnsap_tdd_ChannelisationCodeLCR = -1;  /* TDD_ChannelisationCodeLCR */
static int hf_rnsap_tdd_DL_DPCH_TimeSlotFormat_LCR = -1;  /* TDD_DL_DPCH_TimeSlotFormat_LCR */
static int hf_rnsap_qPSK = -1;                    /* QPSK_DL_DPCH_TimeSlotFormatTDD_LCR */
static int hf_rnsap_eightPSK = -1;                /* EightPSK_DL_DPCH_TimeSlotFormatTDD_LCR */
static int hf_rnsap_initialOffset = -1;           /* INTEGER_0_255 */
static int hf_rnsap_noinitialOffset = -1;         /* INTEGER_0_63 */
static int hf_rnsap_TDD_UL_Code_Information_item = -1;  /* TDD_UL_Code_InformationItem */
static int hf_rnsap_TDD_UL_Code_LCR_Information_item = -1;  /* TDD_UL_Code_LCR_InformationItem */
static int hf_rnsap_tdd_UL_DPCH_TimeSlotFormat_LCR = -1;  /* TDD_UL_DPCH_TimeSlotFormat_LCR */
static int hf_rnsap_qPSK1 = -1;                   /* QPSK_UL_DPCH_TimeSlotFormatTDD_LCR */
static int hf_rnsap_eightPSK1 = -1;               /* EightPSK_UL_DPCH_TimeSlotFormatTDD_LCR */
static int hf_rnsap_plmn_id = -1;                 /* PLMN_Identity */
static int hf_rnsap_service_id = -1;              /* Service_ID */
static int hf_rnsap_dsField = -1;                 /* DsField */
static int hf_rnsap_genericTrafficCategory = -1;  /* GenericTrafficCategory */
static int hf_rnsap_Transmission_Gap_Pattern_Sequence_Information_item = -1;  /* Transmission_Gap_Pattern_Sequence_Information_item */
static int hf_rnsap_tGPSID = -1;                  /* TGPSID */
static int hf_rnsap_tGSN = -1;                    /* TGSN */
static int hf_rnsap_tGL1 = -1;                    /* GapLength */
static int hf_rnsap_tGL2 = -1;                    /* GapLength */
static int hf_rnsap_tGD = -1;                     /* TGD */
static int hf_rnsap_tGPL1 = -1;                   /* GapDuration */
static int hf_rnsap_not_to_be_used_1 = -1;        /* GapDuration */
static int hf_rnsap_uL_DL_mode = -1;              /* UL_DL_mode */
static int hf_rnsap_downlink_Compressed_Mode_Method = -1;  /* Downlink_Compressed_Mode_Method */
static int hf_rnsap_uplink_Compressed_Mode_Method = -1;  /* Uplink_Compressed_Mode_Method */
static int hf_rnsap_dL_FrameType = -1;            /* DL_FrameType */
static int hf_rnsap_delta_SIR1 = -1;              /* DeltaSIR */
static int hf_rnsap_delta_SIR_after1 = -1;        /* DeltaSIR */
static int hf_rnsap_delta_SIR2 = -1;              /* DeltaSIR */
static int hf_rnsap_delta_SIR_after2 = -1;        /* DeltaSIR */
static int hf_rnsap_Transmission_Gap_Pattern_Sequence_Status_List_item = -1;  /* Transmission_Gap_Pattern_Sequence_Status_List_item */
static int hf_rnsap_tGPRC = -1;                   /* TGPRC */
static int hf_rnsap_tGCFN = -1;                   /* CFN */
static int hf_rnsap_ms_part = -1;                 /* INTEGER_0_16383 */
static int hf_rnsap_ls_part = -1;                 /* INTEGER_0_4294967295 */
static int hf_rnsap_tUTRANGPSChangeLimit = -1;    /* TUTRANGPSChangeLimit */
static int hf_rnsap_predictedTUTRANGPSDeviationLimit = -1;  /* PredictedTUTRANGPSDeviationLimit */
static int hf_rnsap_tUTRANGPS = -1;               /* TUTRANGPS */
static int hf_rnsap_tUTRANGPSQuality = -1;        /* TUTRANGPSQuality */
static int hf_rnsap_tUTRANGPSDriftRate = -1;      /* TUTRANGPSDriftRate */
static int hf_rnsap_tUTRANGPSDriftRateQuality = -1;  /* TUTRANGPSDriftRateQuality */
static int hf_rnsap_iEe_Extensions = -1;          /* ProtocolExtensionContainer */
static int hf_rnsap_signalledGainFactors = -1;    /* T_signalledGainFactors */
static int hf_rnsap_betaC = -1;                   /* BetaCD */
static int hf_rnsap_betaD = -1;                   /* BetaCD */
static int hf_rnsap_refTFCNumber = -1;            /* RefTFCNumber */
static int hf_rnsap_tFCSvalues = -1;              /* T_tFCSvalues */
static int hf_rnsap_no_Split_in_TFCI = -1;        /* TFCS_TFCSList */
static int hf_rnsap_not_Used_split_in_TFCI = -1;  /* NULL */
static int hf_rnsap_TFCS_TFCSList_item = -1;      /* TFCS_TFCSList_item */
static int hf_rnsap_cTFC = -1;                    /* TFCS_CTFC */
static int hf_rnsap_tFC_Beta = -1;                /* TransportFormatCombination_Beta */
static int hf_rnsap_ctfc2bit = -1;                /* INTEGER_0_3 */
static int hf_rnsap_ctfc4bit = -1;                /* INTEGER_0_15 */
static int hf_rnsap_ctfc6bit = -1;                /* INTEGER_0_63 */
static int hf_rnsap_ctfc8bit = -1;                /* INTEGER_0_255 */
static int hf_rnsap_ctfc12bit = -1;               /* INTEGER_0_4095 */
static int hf_rnsap_ctfc16bit = -1;               /* INTEGER_0_65535 */
static int hf_rnsap_ctfcmaxbit = -1;              /* INTEGER_0_16777215 */
static int hf_rnsap_dynamicParts = -1;            /* TransportFormatSet_DynamicPartList */
static int hf_rnsap_semi_staticPart = -1;         /* TransportFormatSet_Semi_staticPart */
static int hf_rnsap_TransportFormatSet_DynamicPartList_item = -1;  /* TransportFormatSet_DynamicPartList_item */
static int hf_rnsap_nrOfTransportBlocks = -1;     /* NrOfTransportBlocks */
static int hf_rnsap_transportBlockSize = -1;      /* TransportBlockSize */
static int hf_rnsap_mode = -1;                    /* TransportFormatSet_ModeDP */
static int hf_rnsap_tdd = -1;                     /* TDD_TransportFormatSet_ModeDP */
static int hf_rnsap_notApplicable = -1;           /* NULL */
static int hf_rnsap_transmissionTimeIntervalInformation = -1;  /* TransmissionTimeIntervalInformation */
static int hf_rnsap_TransmissionTimeIntervalInformation_item = -1;  /* TransmissionTimeIntervalInformation_item */
static int hf_rnsap_transmissionTimeInterval = -1;  /* TransmissionTimeIntervalDynamic */
static int hf_rnsap_transmissionTime = -1;        /* TransmissionTimeIntervalSemiStatic */
static int hf_rnsap_channelCoding = -1;           /* ChannelCodingType */
static int hf_rnsap_codingRate = -1;              /* CodingRate */
static int hf_rnsap_rateMatcingAttribute = -1;    /* RateMatchingAttribute */
static int hf_rnsap_cRC_Size = -1;                /* CRC_Size */
static int hf_rnsap_mode1 = -1;                   /* TransportFormatSet_ModeSSP */
static int hf_rnsap_tdd1 = -1;                    /* SecondInterleavingMode */
static int hf_rnsap_hSDSCH_Physical_Layer_Category = -1;  /* INTEGER_1_64_ */
static int hf_rnsap_imei = -1;                    /* IMEI */
static int hf_rnsap_imeisv = -1;                  /* IMEISV */
static int hf_rnsap_periodic2 = -1;               /* UEMeasurementReportCharacteristicsPeriodic */
static int hf_rnsap_event1h = -1;                 /* UEMeasurementReportCharacteristicsEvent1h */
static int hf_rnsap_event1i = -1;                 /* UEMeasurementReportCharacteristicsEvent1i */
static int hf_rnsap_event6a = -1;                 /* UEMeasurementReportCharacteristicsEvent6a */
static int hf_rnsap_event6b = -1;                 /* UEMeasurementReportCharacteristicsEvent6b */
static int hf_rnsap_event6c = -1;                 /* UEMeasurementReportCharacteristicsEvent6c */
static int hf_rnsap_event6d = -1;                 /* UEMeasurementReportCharacteristicsEvent6d */
static int hf_rnsap_extension_ReportCharacteristics1 = -1;  /* UEMeasurementReportCharacteristics_Extension */
static int hf_rnsap_uEMeasurementTreshold = -1;   /* UEMeasurementThreshold */
static int hf_rnsap_uEMeasurementTimeToTrigger = -1;  /* UEMeasurementTimeToTrigger */
static int hf_rnsap_uEMeasurementHysteresisTime = -1;  /* UEMeasurementHysteresisTime */
static int hf_rnsap_amountofReporting = -1;       /* UEMeasurementReportCharacteristicsPeriodicAmountofReporting */
static int hf_rnsap_reportingInterval = -1;       /* UEMeasurementReportCharacteristicsPeriodicReportingInterval */
static int hf_rnsap_timeslotISCP = -1;            /* UEMeasurementThresholdDLTimeslotISCP */
static int hf_rnsap_uETransmitPower = -1;         /* UEMeasurementThresholdUETransmitPower */
static int hf_rnsap_extension_UEMeasurementThreshold = -1;  /* UEMeasurementThreshold_Extension */
static int hf_rnsap_UEMeasurementTimeslotInfoHCR_item = -1;  /* UEMeasurementTimeslotInfoHCR_IEs */
static int hf_rnsap_burstType = -1;               /* UEMeasurementTimeslotInfoHCRBurstType */
static int hf_rnsap_UEMeasurementTimeslotInfoLCR_item = -1;  /* UEMeasurementTimeslotInfoLCR_IEs */
static int hf_rnsap_timeSlot1 = -1;               /* TimeSlotLCR */
static int hf_rnsap_uE_Transmitted_Power = -1;    /* UE_MeasurementValue_UE_Transmitted_Power */
static int hf_rnsap_primary_CCPCH_RSCP = -1;      /* UE_MeasurementValue_Primary_CCPCH_RSCP */
static int hf_rnsap_dL_Timeslot_ISCP = -1;        /* UE_MeasurementValue_DL_Timeslot_ISCP */
static int hf_rnsap_extension_UEMeasurementValue = -1;  /* UEMeasurementValue_Extension */
static int hf_rnsap_uEMeasurementTransmittedPowerListHCR = -1;  /* UEMeasurementValueTransmittedPowerListHCR */
static int hf_rnsap_uEMeasurementTransmittedPowerListLCR = -1;  /* UEMeasurementValueTransmittedPowerListLCR */
static int hf_rnsap_UEMeasurementValueTransmittedPowerListHCR_item = -1;  /* UEMeasurementValueTransmittedPowerListHCR_IEs */
static int hf_rnsap_uETransmitPower1 = -1;        /* INTEGER_0_104 */
static int hf_rnsap_UEMeasurementValueTransmittedPowerListLCR_item = -1;  /* UEMeasurementValueTransmittedPowerListLCR_IEs */
static int hf_rnsap_primaryCCPCH_RSCP_Delta = -1;  /* PrimaryCCPCH_RSCP_Delta */
static int hf_rnsap_uEMeasurementTimeslotISCPListHCR = -1;  /* UEMeasurementValueTimeslotISCPListHCR */
static int hf_rnsap_uEMeasurementTimeslotISCPListLCR = -1;  /* UEMeasurementValueTimeslotISCPListLCR */
static int hf_rnsap_UEMeasurementValueTimeslotISCPListHCR_item = -1;  /* UEMeasurementValueTimeslotISCPListHCR_IEs */
static int hf_rnsap_UEMeasurementValueTimeslotISCPListLCR_item = -1;  /* UEMeasurementValueTimeslotISCPListLCR_IEs */
static int hf_rnsap_measurementAvailable2 = -1;   /* UEMeasurementValueInformationAvailable */
static int hf_rnsap_measurementnotAvailable2 = -1;  /* UEMeasurementValueInformationnotAvailable */
static int hf_rnsap_uEmeasurementValue = -1;      /* UEMeasurementValue */
static int hf_rnsap_cell_fach_pch = -1;           /* Cell_Fach_Pch_State */
static int hf_rnsap_ura_pch = -1;                 /* Ura_Pch_State */
static int hf_rnsap_d_RNTI = -1;                  /* D_RNTI */
static int hf_rnsap_srnc_id = -1;                 /* RNC_ID */
static int hf_rnsap_ura_id = -1;                  /* URA_ID */
static int hf_rnsap_UL_Timeslot_Information_item = -1;  /* UL_Timeslot_InformationItem */
static int hf_rnsap_UL_TimeslotLCR_Information_item = -1;  /* UL_TimeslotLCR_InformationItem */
static int hf_rnsap_uL_Code_LCR_InformationList = -1;  /* TDD_UL_Code_LCR_Information */
static int hf_rnsap_UL_TimeSlot_ISCP_Info_item = -1;  /* UL_TimeSlot_ISCP_InfoItem */
static int hf_rnsap_uL_TimeslotISCP = -1;         /* UL_TimeslotISCP */
static int hf_rnsap_UL_TimeSlot_ISCP_LCR_Info_item = -1;  /* UL_TimeSlot_ISCP_LCR_InfoItem */
static int hf_rnsap_iSCP = -1;                    /* UL_Timeslot_ISCP_Value */
static int hf_rnsap_sync_UL_codes_bitmap = -1;    /* BIT_STRING_SIZE_8 */
static int hf_rnsap_fPACH_info = -1;              /* FPACH_Information */
static int hf_rnsap_prxUpPCHdes = -1;             /* INTEGER_M120_M58_ */
static int hf_rnsap_syncUL_procParameter = -1;    /* SYNC_UL_ProcParameters */
static int hf_rnsap_mMax = -1;                    /* INTEGER_1_32 */
static int hf_rnsap_ul_ScramblingCodeNumber = -1;  /* UL_ScramblingCodeNumber */
static int hf_rnsap_ul_ScramblingCodeLength = -1;  /* UL_ScramblingCodeLength */
static int hf_rnsap_uL_Synchronisation_StepSize = -1;  /* UL_Synchronisation_StepSize */
static int hf_rnsap_uL_Synchronisation_Frequency = -1;  /* UL_Synchronisation_Frequency */
static int hf_rnsap_multipleURAsIndicator = -1;   /* MultipleURAsIndicator */
static int hf_rnsap_rNCsWithCellsInTheAccessedURA_List = -1;  /* RNCsWithCellsInTheAccessedURA_List */
static int hf_rnsap_RNCsWithCellsInTheAccessedURA_List_item = -1;  /* RNCsWithCellsInTheAccessedURA_Item */
static int hf_rnsap_USCH_Information_item = -1;   /* USCH_InformationItem */
static int hf_rnsap_local = -1;                   /* INTEGER_0_65535 */
static int hf_rnsap_global = -1;                  /* OBJECT_IDENTIFIER */
static int hf_rnsap_procedureCode = -1;           /* ProcedureCode */
static int hf_rnsap_ddMode = -1;                  /* T_ddMode */
static int hf_rnsap_shortTransActionId = -1;      /* INTEGER_0_127 */
static int hf_rnsap_longTransActionId = -1;       /* INTEGER_0_32767 */
static int hf_rnsap_ProtocolIE_Container_item = -1;  /* ProtocolIE_Field */
static int hf_rnsap_id = -1;                      /* ProtocolIE_ID */
static int hf_rnsap_value = -1;                   /* ProtocolIEValue */
static int hf_rnsap_ProtocolIE_ContainerPair_item = -1;  /* ProtocolIE_FieldPair */
static int hf_rnsap_firstCriticality = -1;        /* Criticality */
static int hf_rnsap_firstValue = -1;              /* FirstValue */
static int hf_rnsap_secondCriticality = -1;       /* Criticality */
static int hf_rnsap_secondValue = -1;             /* SecondValue */
static int hf_rnsap_ProtocolIE_ContainerList_item = -1;  /* ProtocolIE_Container */
static int hf_rnsap_ProtocolIE_ContainerPairList_item = -1;  /* ProtocolIE_ContainerPair */
static int hf_rnsap_ProtocolExtensionContainer_item = -1;  /* ProtocolExtensionField */
static int hf_rnsap_extensionValue = -1;          /* Extension */
static int hf_rnsap_PrivateIE_Container_item = -1;  /* PrivateIE_Field */
static int hf_rnsap_privateIEid = -1;             /* PrivateIE_ID */
static int hf_rnsap_privateIEvalue = -1;          /* PrivateIEvalue */

/*--- End of included file: packet-rnsap-hf.c ---*/
#line 562 "packet-rnsap-template.c"

/* Initialize the subtree pointers */
static int ett_rnsap = -1;
static int ett_rnsap_initiatingMessageValue = -1;
static int ett_rnsap_ProtocolIEValueValue = -1;
static int ett_rnsap_SuccessfulOutcomeValue = -1;
static int ett_rnsap_UnsuccessfulOutcomeValue = -1;


/*--- Included file: packet-rnsap-ett.c ---*/
#line 1 "packet-rnsap-ett.c"
static gint ett_rnsap_RNSAP_PDU = -1;
static gint ett_rnsap_InitiatingMessage = -1;
static gint ett_rnsap_SuccessfulOutcome = -1;
static gint ett_rnsap_UnsuccessfulOutcome = -1;
static gint ett_rnsap_Outcome = -1;
static gint ett_rnsap_DummyInitiatingValue = -1;
static gint ett_rnsap_DummySuccessfulOutcomeValue = -1;
static gint ett_rnsap_DummyUnSuccessfulOutcomeValue = -1;
static gint ett_rnsap_DummyProtocolIE = -1;
static gint ett_rnsap_RadioLinkSetupRequestFDD = -1;
static gint ett_rnsap_UL_DPCH_Information_RL_SetupRqstFDD = -1;
static gint ett_rnsap_DL_DPCH_Information_RL_SetupRqstFDD = -1;
static gint ett_rnsap_PowerOffsetInformation_RL_SetupRqstFDD = -1;
static gint ett_rnsap_RL_InformationList_RL_SetupRqstFDD = -1;
static gint ett_rnsap_RL_InformationItem_RL_SetupRqstFDD = -1;
static gint ett_rnsap_F_DPCH_Information_RL_SetupRqstFDD = -1;
static gint ett_rnsap_PowerOffsetInformation_F_DPCH_RL_SetupRqstFDD = -1;
static gint ett_rnsap_RadioLinkSetupRequestTDD = -1;
static gint ett_rnsap_UL_Physical_Channel_Information_RL_SetupRqstTDD = -1;
static gint ett_rnsap_DL_Physical_Channel_Information_RL_SetupRqstTDD = -1;
static gint ett_rnsap_UL_CCTrCH_InformationList_RL_SetupRqstTDD = -1;
static gint ett_rnsap_UL_CCTrCH_InformationItem_RL_SetupRqstTDD = -1;
static gint ett_rnsap_DL_CCTrCH_InformationList_RL_SetupRqstTDD = -1;
static gint ett_rnsap_DL_CCTrCH_InformationItem_RL_SetupRqstTDD = -1;
static gint ett_rnsap_CCTrCH_TPCList_RL_SetupRqstTDD = -1;
static gint ett_rnsap_CCTrCH_TPCItem_RL_SetupRqstTDD = -1;
static gint ett_rnsap_RL_Information_RL_SetupRqstTDD = -1;
static gint ett_rnsap_RadioLinkSetupResponseFDD = -1;
static gint ett_rnsap_RL_InformationResponseList_RL_SetupRspFDD = -1;
static gint ett_rnsap_RL_InformationResponseItem_RL_SetupRspFDD = -1;
static gint ett_rnsap_DiversityIndication_RL_SetupRspFDD = -1;
static gint ett_rnsap_Combining_RL_SetupRspFDD = -1;
static gint ett_rnsap_NonCombiningOrFirstRL_RL_SetupRspFDD = -1;
static gint ett_rnsap_RadioLinkSetupResponseTDD = -1;
static gint ett_rnsap_RL_InformationResponse_RL_SetupRspTDD = -1;
static gint ett_rnsap_UL_CCTrCHInformationListIE_RL_SetupRspTDD = -1;
static gint ett_rnsap_UL_CCTrCHInformationItem_RL_SetupRspTDD = -1;
static gint ett_rnsap_UL_DPCH_InformationItem_RL_SetupRspTDD = -1;
static gint ett_rnsap_DL_CCTrCHInformationListIE_RL_SetupRspTDD = -1;
static gint ett_rnsap_DL_CCTrCHInformationItem_RL_SetupRspTDD = -1;
static gint ett_rnsap_DL_DPCH_InformationItem_RL_SetupRspTDD = -1;
static gint ett_rnsap_DSCH_InformationListIEs_RL_SetupRspTDD = -1;
static gint ett_rnsap_DSCHInformationItem_RL_SetupRspTDD = -1;
static gint ett_rnsap_USCH_InformationListIEs_RL_SetupRspTDD = -1;
static gint ett_rnsap_USCHInformationItem_RL_SetupRspTDD = -1;
static gint ett_rnsap_RL_LCR_InformationResponse_RL_SetupRspTDD = -1;
static gint ett_rnsap_UL_LCR_CCTrCHInformationListIE_RL_SetupRspTDD = -1;
static gint ett_rnsap_UL_LCR_CCTrCHInformationItem_RL_SetupRspTDD = -1;
static gint ett_rnsap_UL_DPCH_LCR_InformationItem_RL_SetupRspTDD = -1;
static gint ett_rnsap_DL_CCTrCH_LCR_InformationListIE_RL_SetupRspTDD = -1;
static gint ett_rnsap_DL_CCTrCH_LCR_InformationItem_RL_SetupRspTDD = -1;
static gint ett_rnsap_DL_DPCH_LCR_InformationItem_RL_SetupRspTDD = -1;
static gint ett_rnsap_DSCH_LCR_InformationListIEs_RL_SetupRspTDD = -1;
static gint ett_rnsap_DSCH_LCR_InformationItem_RL_SetupRspTDD = -1;
static gint ett_rnsap_USCH_LCR_InformationListIEs_RL_SetupRspTDD = -1;
static gint ett_rnsap_USCH_LCR_InformationItem_RL_SetupRspTDD = -1;
static gint ett_rnsap_RadioLinkSetupFailureFDD = -1;
static gint ett_rnsap_CauseLevel_RL_SetupFailureFDD = -1;
static gint ett_rnsap_GeneralCauseList_RL_SetupFailureFDD = -1;
static gint ett_rnsap_RLSpecificCauseList_RL_SetupFailureFDD = -1;
static gint ett_rnsap_UnsuccessfulRL_InformationResponseList_RL_SetupFailureFDD = -1;
static gint ett_rnsap_UnsuccessfulRL_InformationResponse_RL_SetupFailureFDD = -1;
static gint ett_rnsap_SuccessfulRL_InformationResponseList_RL_SetupFailureFDD = -1;
static gint ett_rnsap_SuccessfulRL_InformationResponse_RL_SetupFailureFDD = -1;
static gint ett_rnsap_DiversityIndication_RL_SetupFailureFDD = -1;
static gint ett_rnsap_Combining_RL_SetupFailureFDD = -1;
static gint ett_rnsap_NonCombiningOrFirstRL_RL_SetupFailureFDD = -1;
static gint ett_rnsap_RadioLinkSetupFailureTDD = -1;
static gint ett_rnsap_CauseLevel_RL_SetupFailureTDD = -1;
static gint ett_rnsap_GeneralCauseList_RL_SetupFailureTDD = -1;
static gint ett_rnsap_RLSpecificCauseList_RL_SetupFailureTDD = -1;
static gint ett_rnsap_UnsuccessfulRL_InformationResponse_RL_SetupFailureTDD = -1;
static gint ett_rnsap_RadioLinkAdditionRequestFDD = -1;
static gint ett_rnsap_RL_InformationList_RL_AdditionRqstFDD = -1;
static gint ett_rnsap_RL_Information_RL_AdditionRqstFDD = -1;
static gint ett_rnsap_RadioLinkAdditionRequestTDD = -1;
static gint ett_rnsap_RL_Information_RL_AdditionRqstTDD = -1;
static gint ett_rnsap_UL_CCTrCH_InformationList_RL_AdditionRqstTDD = -1;
static gint ett_rnsap_UL_CCTrCH_InformationItem_RL_AdditionRqstTDD = -1;
static gint ett_rnsap_DL_CCTrCH_InformationList_RL_AdditionRqstTDD = -1;
static gint ett_rnsap_DL_CCTrCH_InformationItem_RL_AdditionRqstTDD = -1;
static gint ett_rnsap_RadioLinkAdditionResponseFDD = -1;
static gint ett_rnsap_RL_InformationResponseList_RL_AdditionRspFDD = -1;
static gint ett_rnsap_RL_InformationResponseItem_RL_AdditionRspFDD = -1;
static gint ett_rnsap_DiversityIndication_RL_AdditionRspFDD = -1;
static gint ett_rnsap_Combining_RL_AdditionRspFDD = -1;
static gint ett_rnsap_NonCombining_RL_AdditionRspFDD = -1;
static gint ett_rnsap_RadioLinkAdditionResponseTDD = -1;
static gint ett_rnsap_RL_InformationResponse_RL_AdditionRspTDD = -1;
static gint ett_rnsap_UL_CCTrCHInformationListIE_RL_AdditionRspTDD = -1;
static gint ett_rnsap_UL_CCTrCHInformationItem_RL_AdditionRspTDD = -1;
static gint ett_rnsap_UL_DPCH_InformationItem_RL_AdditionRspTDD = -1;
static gint ett_rnsap_DL_CCTrCHInformationListIE_RL_AdditionRspTDD = -1;
static gint ett_rnsap_DL_CCTrCHInformationItem_RL_AdditionRspTDD = -1;
static gint ett_rnsap_DL_DPCH_InformationItem_RL_AdditionRspTDD = -1;
static gint ett_rnsap_DCH_Information_RL_AdditionRspTDD = -1;
static gint ett_rnsap_DiversityIndication_RL_AdditionRspTDD = -1;
static gint ett_rnsap_Combining_RL_AdditionRspTDD = -1;
static gint ett_rnsap_NonCombining_RL_AdditionRspTDD = -1;
static gint ett_rnsap_DSCH_InformationListIE_RL_AdditionRspTDD = -1;
static gint ett_rnsap_DSCHInformationItem_RL_AdditionRspTDD = -1;
static gint ett_rnsap_DiversityIndication_RL_AdditionRspTDD2 = -1;
static gint ett_rnsap_USCH_InformationListIE_RL_AdditionRspTDD = -1;
static gint ett_rnsap_USCHInformationItem_RL_AdditionRspTDD = -1;
static gint ett_rnsap_RL_LCR_InformationResponse_RL_AdditionRspTDD = -1;
static gint ett_rnsap_UL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD = -1;
static gint ett_rnsap_UL_CCTrCH_LCR_InformationItem_RL_AdditionRspTDD = -1;
static gint ett_rnsap_UL_DPCH_LCR_InformationItem_RL_AdditionRspTDD = -1;
static gint ett_rnsap_DL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD = -1;
static gint ett_rnsap_DL_CCTrCH_LCR_InformationItem_RL_AdditionRspTDD = -1;
static gint ett_rnsap_DL_DPCH_LCR_InformationItem_RL_AdditionRspTDD = -1;
static gint ett_rnsap_DSCH_LCR_InformationListIEs_RL_AdditionRspTDD = -1;
static gint ett_rnsap_DSCH_LCR_InformationItem_RL_AdditionRspTDD = -1;
static gint ett_rnsap_USCH_LCR_InformationListIEs_RL_AdditionRspTDD = -1;
static gint ett_rnsap_USCH_LCR_InformationItem_RL_AdditionRspTDD = -1;
static gint ett_rnsap_RadioLinkAdditionFailureFDD = -1;
static gint ett_rnsap_CauseLevel_RL_AdditionFailureFDD = -1;
static gint ett_rnsap_GeneralCauseList_RL_AdditionFailureFDD = -1;
static gint ett_rnsap_RLSpecificCauseList_RL_AdditionFailureFDD = -1;
static gint ett_rnsap_UnsuccessfulRL_InformationResponseList_RL_AdditionFailureFDD = -1;
static gint ett_rnsap_UnsuccessfulRL_InformationResponse_RL_AdditionFailureFDD = -1;
static gint ett_rnsap_SuccessfulRL_InformationResponseList_RL_AdditionFailureFDD = -1;
static gint ett_rnsap_SuccessfulRL_InformationResponse_RL_AdditionFailureFDD = -1;
static gint ett_rnsap_DiversityIndication_RL_AdditionFailureFDD = -1;
static gint ett_rnsap_Combining_RL_AdditionFailureFDD = -1;
static gint ett_rnsap_NonCombining_RL_AdditionFailureFDD = -1;
static gint ett_rnsap_RadioLinkAdditionFailureTDD = -1;
static gint ett_rnsap_CauseLevel_RL_AdditionFailureTDD = -1;
static gint ett_rnsap_GeneralCauseList_RL_AdditionFailureTDD = -1;
static gint ett_rnsap_RLSpecificCauseList_RL_AdditionFailureTDD = -1;
static gint ett_rnsap_UnsuccessfulRL_InformationResponse_RL_AdditionFailureTDD = -1;
static gint ett_rnsap_RadioLinkDeletionRequest = -1;
static gint ett_rnsap_RL_InformationList_RL_DeletionRqst = -1;
static gint ett_rnsap_RL_Information_RL_DeletionRqst = -1;
static gint ett_rnsap_RadioLinkDeletionResponse = -1;
static gint ett_rnsap_RadioLinkReconfigurationPrepareFDD = -1;
static gint ett_rnsap_UL_DPCH_Information_RL_ReconfPrepFDD = -1;
static gint ett_rnsap_DL_DPCH_Information_RL_ReconfPrepFDD = -1;
static gint ett_rnsap_DL_DPCH_Power_Information_RL_ReconfPrepFDD = -1;
static gint ett_rnsap_PowerOffsetInformation_RL_ReconfPrepFDD = -1;
static gint ett_rnsap_DCH_DeleteList_RL_ReconfPrepFDD = -1;
static gint ett_rnsap_DCH_DeleteItem_RL_ReconfPrepFDD = -1;
static gint ett_rnsap_RL_InformationList_RL_ReconfPrepFDD = -1;
static gint ett_rnsap_RL_Information_RL_ReconfPrepFDD = -1;
static gint ett_rnsap_F_DPCH_Information_RL_ReconfPrepFDD = -1;
static gint ett_rnsap_PowerOffsetInformation_F_DPCH_RL_ReconfPrepFDD = -1;
static gint ett_rnsap_RadioLinkReconfigurationPrepareTDD = -1;
static gint ett_rnsap_UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD = -1;
static gint ett_rnsap_UL_CCTrCH_AddInformation_RL_ReconfPrepTDD = -1;
static gint ett_rnsap_UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD = -1;
static gint ett_rnsap_UL_CCTrCH_ModifyInformation_RL_ReconfPrepTDD = -1;
static gint ett_rnsap_UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD = -1;
static gint ett_rnsap_UL_CCTrCH_DeleteInformation_RL_ReconfPrepTDD = -1;
static gint ett_rnsap_DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD = -1;
static gint ett_rnsap_DL_CCTrCH_InformationAddItem_RL_ReconfPrepTDD = -1;
static gint ett_rnsap_CCTrCH_TPCAddList_RL_ReconfPrepTDD = -1;
static gint ett_rnsap_CCTrCH_TPCAddItem_RL_ReconfPrepTDD = -1;
static gint ett_rnsap_DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD = -1;
static gint ett_rnsap_DL_CCTrCH_InformationModifyItem_RL_ReconfPrepTDD = -1;
static gint ett_rnsap_CCTrCH_TPCModifyList_RL_ReconfPrepTDD = -1;
static gint ett_rnsap_CCTrCH_TPCModifyItem_RL_ReconfPrepTDD = -1;
static gint ett_rnsap_DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD = -1;
static gint ett_rnsap_DL_CCTrCH_InformationDeleteItem_RL_ReconfPrepTDD = -1;
static gint ett_rnsap_DCH_DeleteList_RL_ReconfPrepTDD = -1;
static gint ett_rnsap_DCH_DeleteItem_RL_ReconfPrepTDD = -1;
static gint ett_rnsap_DSCH_ModifyList_RL_ReconfPrepTDD = -1;
static gint ett_rnsap_DSCH_ModifyItem_RL_ReconfPrepTDD = -1;
static gint ett_rnsap_DSCH_DeleteList_RL_ReconfPrepTDD = -1;
static gint ett_rnsap_DSCH_DeleteItem_RL_ReconfPrepTDD = -1;
static gint ett_rnsap_USCH_ModifyList_RL_ReconfPrepTDD = -1;
static gint ett_rnsap_USCH_ModifyItem_RL_ReconfPrepTDD = -1;
static gint ett_rnsap_USCH_DeleteList_RL_ReconfPrepTDD = -1;
static gint ett_rnsap_USCH_DeleteItem_RL_ReconfPrepTDD = -1;
static gint ett_rnsap_RL_Information_RL_ReconfPrepTDD = -1;
static gint ett_rnsap_RL_InformationIE_RL_ReconfPrepTDD = -1;
static gint ett_rnsap_RadioLinkReconfigurationReadyFDD = -1;
static gint ett_rnsap_RL_InformationResponseList_RL_ReconfReadyFDD = -1;
static gint ett_rnsap_RL_InformationResponseItem_RL_ReconfReadyFDD = -1;
static gint ett_rnsap_RadioLinkReconfigurationReadyTDD = -1;
static gint ett_rnsap_RL_InformationResponse_RL_ReconfReadyTDD = -1;
static gint ett_rnsap_UL_CCTrCHInformationListIE_RL_ReconfReadyTDD = -1;
static gint ett_rnsap_UL_CCTrCH_InformationItem_RL_ReconfReadyTDD = -1;
static gint ett_rnsap_UL_DPCH_LCR_InformationAddList_RL_ReconfReadyTDD = -1;
static gint ett_rnsap_UL_DPCH_InformationAddListIE_RL_ReconfReadyTDD = -1;
static gint ett_rnsap_UL_DPCH_InformationModifyListIE_RL_ReconfReadyTDD = -1;
static gint ett_rnsap_UL_TimeslotLCR_InformationModifyList_RL_ReconfReadyTDD = -1;
static gint ett_rnsap_UL_TimeslotLCR_InformationModifyItem_RL_ReconfReadyTDD = -1;
static gint ett_rnsap_TDD_UL_Code_LCR_InformationModifyList_RL_ReconfReadyTDD = -1;
static gint ett_rnsap_TDD_UL_Code_LCR_InformationModifyItem_RL_ReconfReadyTDD = -1;
static gint ett_rnsap_UL_Timeslot_InformationModifyList_RL_ReconfReadyTDD = -1;
static gint ett_rnsap_UL_Timeslot_InformationModifyItem_RL_ReconfReadyTDD = -1;
static gint ett_rnsap_TDD_UL_Code_InformationModifyList_RL_ReconfReadyTDD = -1;
static gint ett_rnsap_TDD_UL_Code_InformationModifyItem_RL_ReconfReadyTDD = -1;
static gint ett_rnsap_UL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD = -1;
static gint ett_rnsap_UL_DPCH_InformationDeleteItem_RL_ReconfReadyTDD = -1;
static gint ett_rnsap_DL_CCTrCHInformationListIE_RL_ReconfReadyTDD = -1;
static gint ett_rnsap_DL_CCTrCH_InformationItem_RL_ReconfReadyTDD = -1;
static gint ett_rnsap_DL_DPCH_LCR_InformationAddList_RL_ReconfReadyTDD = -1;
static gint ett_rnsap_DL_DPCH_InformationAddListIE_RL_ReconfReadyTDD = -1;
static gint ett_rnsap_DL_DPCH_InformationModifyListIE_RL_ReconfReadyTDD = -1;
static gint ett_rnsap_DL_TimeslotLCR_InformationModifyList_RL_ReconfReadyTDD = -1;
static gint ett_rnsap_DL_TimeslotLCR_InformationModifyItem_RL_ReconfReadyTDD = -1;
static gint ett_rnsap_TDD_DL_Code_LCR_InformationModifyList_RL_ReconfReadyTDD = -1;
static gint ett_rnsap_TDD_DL_Code_LCR_InformationModifyItem_RL_ReconfReadyTDD = -1;
static gint ett_rnsap_DL_Timeslot_InformationModifyList_RL_ReconfReadyTDD = -1;
static gint ett_rnsap_DL_Timeslot_InformationModifyItem_RL_ReconfReadyTDD = -1;
static gint ett_rnsap_TDD_DL_Code_InformationModifyList_RL_ReconfReadyTDD = -1;
static gint ett_rnsap_TDD_DL_Code_InformationModifyItem_RL_ReconfReadyTDD = -1;
static gint ett_rnsap_DL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD = -1;
static gint ett_rnsap_DL_DPCH_InformationDeleteItem_RL_ReconfReadyTDD = -1;
static gint ett_rnsap_DSCHToBeAddedOrModifiedList_RL_ReconfReadyTDD = -1;
static gint ett_rnsap_DSCHToBeAddedOrModifiedItem_RL_ReconfReadyTDD = -1;
static gint ett_rnsap_USCHToBeAddedOrModifiedList_RL_ReconfReadyTDD = -1;
static gint ett_rnsap_USCHToBeAddedOrModifiedItem_RL_ReconfReadyTDD = -1;
static gint ett_rnsap_Multiple_RL_InformationResponse_RL_ReconfReadyTDD = -1;
static gint ett_rnsap_RadioLinkReconfigurationCommit = -1;
static gint ett_rnsap_RadioLinkReconfigurationFailure = -1;
static gint ett_rnsap_CauseLevel_RL_ReconfFailure = -1;
static gint ett_rnsap_GeneralCauseList_RL_ReconfFailure = -1;
static gint ett_rnsap_RLSpecificCauseList_RL_ReconfFailure = -1;
static gint ett_rnsap_RL_ReconfigurationFailureList_RL_ReconfFailure = -1;
static gint ett_rnsap_RL_ReconfigurationFailure_RL_ReconfFail = -1;
static gint ett_rnsap_RadioLinkReconfigurationCancel = -1;
static gint ett_rnsap_RadioLinkReconfigurationRequestFDD = -1;
static gint ett_rnsap_UL_DPCH_Information_RL_ReconfRqstFDD = -1;
static gint ett_rnsap_DL_DPCH_Information_RL_ReconfRqstFDD = -1;
static gint ett_rnsap_DCH_DeleteList_RL_ReconfRqstFDD = -1;
static gint ett_rnsap_DCH_DeleteItem_RL_ReconfRqstFDD = -1;
static gint ett_rnsap_RL_ReconfigurationRequestFDD_RL_InformationList = -1;
static gint ett_rnsap_RL_ReconfigurationRequestFDD_RL_Information_IEs = -1;
static gint ett_rnsap_RadioLinkReconfigurationRequestTDD = -1;
static gint ett_rnsap_UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD = -1;
static gint ett_rnsap_UL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD = -1;
static gint ett_rnsap_UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD = -1;
static gint ett_rnsap_UL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD = -1;
static gint ett_rnsap_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD = -1;
static gint ett_rnsap_DL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD = -1;
static gint ett_rnsap_DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD = -1;
static gint ett_rnsap_DL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD = -1;
static gint ett_rnsap_DCH_DeleteList_RL_ReconfRqstTDD = -1;
static gint ett_rnsap_DCH_DeleteItem_RL_ReconfRqstTDD = -1;
static gint ett_rnsap_Multiple_RL_ReconfigurationRequestTDD_RL_Information = -1;
static gint ett_rnsap_RL_ReconfigurationRequestTDD_RL_Information = -1;
static gint ett_rnsap_RadioLinkReconfigurationResponseFDD = -1;
static gint ett_rnsap_RL_InformationResponseList_RL_ReconfRspFDD = -1;
static gint ett_rnsap_RL_InformationResponseItem_RL_ReconfRspFDD = -1;
static gint ett_rnsap_RadioLinkReconfigurationResponseTDD = -1;
static gint ett_rnsap_RL_InformationResponse_RL_ReconfRspTDD = -1;
static gint ett_rnsap_DL_CCTrCH_InformationList_RL_ReconfRspTDD = -1;
static gint ett_rnsap_DL_CCTrCH_InformationItem_RL_ReconfRspTDD = -1;
static gint ett_rnsap_DL_DPCH_InformationModifyItem_LCR_RL_ReconfRspTDD = -1;
static gint ett_rnsap_DL_Timeslot_LCR_InformationModifyList_RL_ReconfRspTDD = -1;
static gint ett_rnsap_DL_Timeslot_LCR_InformationModifyItem_RL_ReconfRspTDD = -1;
static gint ett_rnsap_Multiple_RL_InformationResponse_RL_ReconfRspTDD = -1;
static gint ett_rnsap_RadioLinkFailureIndication = -1;
static gint ett_rnsap_Reporting_Object_RL_FailureInd = -1;
static gint ett_rnsap_RL_RL_FailureInd = -1;
static gint ett_rnsap_RL_InformationList_RL_FailureInd = -1;
static gint ett_rnsap_RL_Information_RL_FailureInd = -1;
static gint ett_rnsap_RL_Set_RL_FailureInd = -1;
static gint ett_rnsap_RL_Set_InformationList_RL_FailureInd = -1;
static gint ett_rnsap_RL_Set_Information_RL_FailureInd = -1;
static gint ett_rnsap_CCTrCH_RL_FailureInd = -1;
static gint ett_rnsap_CCTrCH_InformationList_RL_FailureInd = -1;
static gint ett_rnsap_CCTrCH_InformationItem_RL_FailureInd = -1;
static gint ett_rnsap_RadioLinkPreemptionRequiredIndication = -1;
static gint ett_rnsap_RL_InformationList_RL_PreemptRequiredInd = -1;
static gint ett_rnsap_RL_InformationItem_RL_PreemptRequiredInd = -1;
static gint ett_rnsap_HSDSCHMacdFlowSpecificInformationList_RL_PreemptRequiredInd = -1;
static gint ett_rnsap_HSDSCHMacdFlowSpecificInformationItem_RL_PreemptRequiredInd = -1;
static gint ett_rnsap_EDCH_MacdFlowSpecificInformationList_RL_PreemptRequiredInd = -1;
static gint ett_rnsap_EDCH_MacdFlowSpecificInformationItem_RL_PreemptRequiredInd = -1;
static gint ett_rnsap_RadioLinkRestoreIndication = -1;
static gint ett_rnsap_Reporting_Object_RL_RestoreInd = -1;
static gint ett_rnsap_RL_RL_RestoreInd = -1;
static gint ett_rnsap_RL_InformationList_RL_RestoreInd = -1;
static gint ett_rnsap_RL_Information_RL_RestoreInd = -1;
static gint ett_rnsap_RL_Set_RL_RestoreInd = -1;
static gint ett_rnsap_RL_Set_InformationList_RL_RestoreInd = -1;
static gint ett_rnsap_RL_Set_Information_RL_RestoreInd = -1;
static gint ett_rnsap_CCTrCH_RL_RestoreInd = -1;
static gint ett_rnsap_CCTrCH_InformationList_RL_RestoreInd = -1;
static gint ett_rnsap_CCTrCH_InformationItem_RL_RestoreInd = -1;
static gint ett_rnsap_DL_PowerControlRequest = -1;
static gint ett_rnsap_DL_ReferencePowerInformationList_DL_PC_Rqst = -1;
static gint ett_rnsap_DL_ReferencePowerInformation_DL_PC_Rqst = -1;
static gint ett_rnsap_DL_PowerTimeslotControlRequest = -1;
static gint ett_rnsap_PhysicalChannelReconfigurationRequestFDD = -1;
static gint ett_rnsap_RL_Information_PhyChReconfRqstFDD = -1;
static gint ett_rnsap_PhysicalChannelReconfigurationRequestTDD = -1;
static gint ett_rnsap_RL_Information_PhyChReconfRqstTDD = -1;
static gint ett_rnsap_UL_CCTrCH_InformationListIE_PhyChReconfRqstTDD = -1;
static gint ett_rnsap_UL_CCTrCH_InformationItem_PhyChReconfRqstTDD = -1;
static gint ett_rnsap_UL_DPCH_InformationItem_PhyChReconfRqstTDD = -1;
static gint ett_rnsap_UL_TimeslotLCR_InformationList_PhyChReconfRqstTDD = -1;
static gint ett_rnsap_UL_TimeslotLCR_InformationItem_PhyChReconfRqstTDD = -1;
static gint ett_rnsap_UL_Timeslot_InformationList_PhyChReconfRqstTDD = -1;
static gint ett_rnsap_UL_Timeslot_InformationItem_PhyChReconfRqstTDD = -1;
static gint ett_rnsap_DL_CCTrCH_InformationListIE_PhyChReconfRqstTDD = -1;
static gint ett_rnsap_DL_CCTrCH_InformationItem_PhyChReconfRqstTDD = -1;
static gint ett_rnsap_DL_DPCH_InformationItem_PhyChReconfRqstTDD = -1;
static gint ett_rnsap_DL_TimeslotLCR_InformationList_PhyChReconfRqstTDD = -1;
static gint ett_rnsap_DL_TimeslotLCR_InformationItem_PhyChReconfRqstTDD = -1;
static gint ett_rnsap_DL_Timeslot_InformationList_PhyChReconfRqstTDD = -1;
static gint ett_rnsap_DL_Timeslot_InformationItem_PhyChReconfRqstTDD = -1;
static gint ett_rnsap_HSPDSCH_Timeslot_InformationList_PhyChReconfRqstTDD = -1;
static gint ett_rnsap_HSPDSCH_Timeslot_InformationItem_PhyChReconfRqstTDD = -1;
static gint ett_rnsap_HSPDSCH_Timeslot_InformationListLCR_PhyChReconfRqstTDD = -1;
static gint ett_rnsap_HSPDSCH_Timeslot_InformationItemLCR_PhyChReconfRqstTDD = -1;
static gint ett_rnsap_PhysicalChannelReconfigurationCommand = -1;
static gint ett_rnsap_PhysicalChannelReconfigurationFailure = -1;
static gint ett_rnsap_RadioLinkCongestionIndication = -1;
static gint ett_rnsap_RL_InformationList_RL_CongestInd = -1;
static gint ett_rnsap_RL_InformationItem_RL_CongestInd = -1;
static gint ett_rnsap_DCH_Rate_Information_RL_CongestInd = -1;
static gint ett_rnsap_DCH_Rate_InformationItem_RL_CongestInd = -1;
static gint ett_rnsap_EDCH_MacdFlowSpecificInformationList_RL_CongestInd = -1;
static gint ett_rnsap_EDCH_MacdFlowSpecificInformationItem_RL_CongestInd = -1;
static gint ett_rnsap_UplinkSignallingTransferIndicationFDD = -1;
static gint ett_rnsap_UplinkSignallingTransferIndicationTDD = -1;
static gint ett_rnsap_DownlinkSignallingTransferRequest = -1;
static gint ett_rnsap_RelocationCommit = -1;
static gint ett_rnsap_PagingRequest = -1;
static gint ett_rnsap_PagingArea_PagingRqst = -1;
static gint ett_rnsap_URA_PagingRqst = -1;
static gint ett_rnsap_Cell_PagingRqst = -1;
static gint ett_rnsap_CNOriginatedPage_PagingRqst = -1;
static gint ett_rnsap_DedicatedMeasurementInitiationRequest = -1;
static gint ett_rnsap_DedicatedMeasurementObjectType_DM_Rqst = -1;
static gint ett_rnsap_RL_DM_Rqst = -1;
static gint ett_rnsap_RL_InformationList_DM_Rqst = -1;
static gint ett_rnsap_RL_InformationItem_DM_Rqst = -1;
static gint ett_rnsap_HSSICH_Info_DM_Rqst = -1;
static gint ett_rnsap_RL_Set_DM_Rqst = -1;
static gint ett_rnsap_RL_Set_InformationList_DM_Rqst = -1;
static gint ett_rnsap_RL_Set_InformationItem_DM_Rqst = -1;
static gint ett_rnsap_DedicatedMeasurementInitiationResponse = -1;
static gint ett_rnsap_DedicatedMeasurementObjectType_DM_Rsp = -1;
static gint ett_rnsap_RL_DM_Rsp = -1;
static gint ett_rnsap_RL_Set_DM_Rsp = -1;
static gint ett_rnsap_RL_InformationList_DM_Rsp = -1;
static gint ett_rnsap_RL_InformationItem_DM_Rsp = -1;
static gint ett_rnsap_RL_Set_InformationList_DM_Rsp = -1;
static gint ett_rnsap_RL_Set_InformationItem_DM_Rsp = -1;
static gint ett_rnsap_Multiple_DedicatedMeasurementValueList_TDD_DM_Rsp = -1;
static gint ett_rnsap_Multiple_DedicatedMeasurementValueItem_TDD_DM_Rsp = -1;
static gint ett_rnsap_Multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp = -1;
static gint ett_rnsap_Multiple_DedicatedMeasurementValueItem_LCR_TDD_DM_Rsp = -1;
static gint ett_rnsap_DedicatedMeasurementInitiationFailure = -1;
static gint ett_rnsap_DedicatedMeasurementObjectType_DM_Fail = -1;
static gint ett_rnsap_RL_DM_Fail = -1;
static gint ett_rnsap_RL_Set_DM_Fail = -1;
static gint ett_rnsap_RL_Unsuccessful_InformationRespList_DM_Fail = -1;
static gint ett_rnsap_RL_Unsuccessful_InformationItem_DM_Fail = -1;
static gint ett_rnsap_RL_Successful_InformationRespList_DM_Fail = -1;
static gint ett_rnsap_RL_Successful_InformationItem_DM_Fail = -1;
static gint ett_rnsap_RL_Set_Unsuccessful_InformationRespList_DM_Fail = -1;
static gint ett_rnsap_RL_Set_Unsuccessful_InformationItem_DM_Fail = -1;
static gint ett_rnsap_RL_Set_Successful_InformationRespList_DM_Fail = -1;
static gint ett_rnsap_RL_Set_Successful_InformationItem_DM_Fail = -1;
static gint ett_rnsap_DedicatedMeasurementReport = -1;
static gint ett_rnsap_DedicatedMeasurementObjectType_DM_Rprt = -1;
static gint ett_rnsap_RL_DM_Rprt = -1;
static gint ett_rnsap_RL_Set_DM_Rprt = -1;
static gint ett_rnsap_RL_InformationList_DM_Rprt = -1;
static gint ett_rnsap_RL_InformationItem_DM_Rprt = -1;
static gint ett_rnsap_RL_Set_InformationList_DM_Rprt = -1;
static gint ett_rnsap_RL_Set_InformationItem_DM_Rprt = -1;
static gint ett_rnsap_DedicatedMeasurementTerminationRequest = -1;
static gint ett_rnsap_DedicatedMeasurementFailureIndication = -1;
static gint ett_rnsap_DedicatedMeasurementObjectType_DM_Fail_Ind = -1;
static gint ett_rnsap_RL_DM_Fail_Ind = -1;
static gint ett_rnsap_RL_Set_DM_Fail_Ind = -1;
static gint ett_rnsap_RL_Unsuccessful_InformationRespList_DM_Fail_Ind = -1;
static gint ett_rnsap_RL_Unsuccessful_InformationItem_DM_Fail_Ind = -1;
static gint ett_rnsap_RL_Set_Unsuccessful_InformationRespList_DM_Fail_Ind = -1;
static gint ett_rnsap_RL_Set_Unsuccessful_InformationItem_DM_Fail_Ind = -1;
static gint ett_rnsap_CommonTransportChannelResourcesReleaseRequest = -1;
static gint ett_rnsap_CommonTransportChannelResourcesRequest = -1;
static gint ett_rnsap_CommonTransportChannelResourcesResponseFDD = -1;
static gint ett_rnsap_FACH_InfoForUESelectedS_CCPCH_CTCH_ResourceRspFDD = -1;
static gint ett_rnsap_CommonTransportChannelResourcesResponseTDD = -1;
static gint ett_rnsap_FACH_InfoForUESelectedS_CCPCH_CTCH_ResourceRspTDD = -1;
static gint ett_rnsap_CommonTransportChannelResourcesFailure = -1;
static gint ett_rnsap_CompressedModeCommand = -1;
static gint ett_rnsap_ErrorIndication = -1;
static gint ett_rnsap_CommonMeasurementInitiationRequest = -1;
static gint ett_rnsap_CommonMeasurementObjectType_CM_Rqst = -1;
static gint ett_rnsap_Cell_CM_Rqst = -1;
static gint ett_rnsap_NeighbouringCellMeasurementInfo = -1;
static gint ett_rnsap_NeighbouringCellMeasurementInfo_item = -1;
static gint ett_rnsap_CommonMeasurementInitiationResponse = -1;
static gint ett_rnsap_CommonMeasurementObjectType_CM_Rsp = -1;
static gint ett_rnsap_Cell_CM_Rsp = -1;
static gint ett_rnsap_CommonMeasurementInitiationFailure = -1;
static gint ett_rnsap_CommonMeasurementReport = -1;
static gint ett_rnsap_CommonMeasurementObjectType_CM_Rprt = -1;
static gint ett_rnsap_Cell_CM_Rprt = -1;
static gint ett_rnsap_CommonMeasurementTerminationRequest = -1;
static gint ett_rnsap_CommonMeasurementFailureIndication = -1;
static gint ett_rnsap_InformationExchangeInitiationRequest = -1;
static gint ett_rnsap_InformationExchangeObjectType_InfEx_Rqst = -1;
static gint ett_rnsap_Cell_InfEx_Rqst = -1;
static gint ett_rnsap_GSM_Cell_InfEx_Rqst = -1;
static gint ett_rnsap_InformationExchangeInitiationResponse = -1;
static gint ett_rnsap_InformationExchangeObjectType_InfEx_Rsp = -1;
static gint ett_rnsap_Cell_InfEx_Rsp = -1;
static gint ett_rnsap_MBMS_Bearer_Service_List_InfEx_Rsp = -1;
static gint ett_rnsap_MBMS_Bearer_ServiceItemIEs_InfEx_Rsp = -1;
static gint ett_rnsap_InformationExchangeInitiationFailure = -1;
static gint ett_rnsap_InformationReport = -1;
static gint ett_rnsap_InformationExchangeObjectType_InfEx_Rprt = -1;
static gint ett_rnsap_Cell_InfEx_Rprt = -1;
static gint ett_rnsap_InformationExchangeTerminationRequest = -1;
static gint ett_rnsap_InformationExchangeFailureIndication = -1;
static gint ett_rnsap_ResetRequest = -1;
static gint ett_rnsap_ResetIndicator = -1;
static gint ett_rnsap_ContextList_Reset = -1;
static gint ett_rnsap_ContextInfoList_Reset = -1;
static gint ett_rnsap_ContextInfoItem_Reset = -1;
static gint ett_rnsap_ContextType_Reset = -1;
static gint ett_rnsap_ContextGroupList_Reset = -1;
static gint ett_rnsap_ContextGroupInfoList_Reset = -1;
static gint ett_rnsap_ContextGroupInfoItem_Reset = -1;
static gint ett_rnsap_ResetResponse = -1;
static gint ett_rnsap_RadioLinkActivationCommandFDD = -1;
static gint ett_rnsap_DelayedActivationInformationList_RL_ActivationCmdFDD = -1;
static gint ett_rnsap_DelayedActivationInformation_RL_ActivationCmdFDD = -1;
static gint ett_rnsap_RadioLinkActivationCommandTDD = -1;
static gint ett_rnsap_DelayedActivationInformationList_RL_ActivationCmdTDD = -1;
static gint ett_rnsap_DelayedActivationInformation_RL_ActivationCmdTDD = -1;
static gint ett_rnsap_GERANUplinkSignallingTransferIndication = -1;
static gint ett_rnsap_RadioLinkParameterUpdateIndicationFDD = -1;
static gint ett_rnsap_RL_ParameterUpdateIndicationFDD_RL_InformationList = -1;
static gint ett_rnsap_RL_ParameterUpdateIndicationFDD_RL_Information_Item = -1;
static gint ett_rnsap_RadioLinkParameterUpdateIndicationTDD = -1;
static gint ett_rnsap_UEMeasurementInitiationRequest = -1;
static gint ett_rnsap_UEMeasurementInitiationResponse = -1;
static gint ett_rnsap_UEMeasurementInitiationFailure = -1;
static gint ett_rnsap_UEMeasurementReport = -1;
static gint ett_rnsap_UEMeasurementTerminationRequest = -1;
static gint ett_rnsap_UEMeasurementFailureIndication = -1;
static gint ett_rnsap_IurInvokeTrace = -1;
static gint ett_rnsap_ListOfInterfacesToTrace = -1;
static gint ett_rnsap_InterfacesToTraceItem = -1;
static gint ett_rnsap_IurDeactivateTrace = -1;
static gint ett_rnsap_MBMSAttachCommand = -1;
static gint ett_rnsap_MBMSDetachCommand = -1;
static gint ett_rnsap_DirectInformationTransfer = -1;
static gint ett_rnsap_PrivateMessage = -1;
static gint ett_rnsap_Active_MBMS_Bearer_Service_ListFDD = -1;
static gint ett_rnsap_Active_MBMS_Bearer_Service_ListFDD_PFL = -1;
static gint ett_rnsap_Active_MBMS_Bearer_Service_ListTDD = -1;
static gint ett_rnsap_Active_MBMS_Bearer_Service_ListTDD_PFL = -1;
static gint ett_rnsap_Active_Pattern_Sequence_Information = -1;
static gint ett_rnsap_AdditionalPreferredFrequency = -1;
static gint ett_rnsap_AdditionalPreferredFrequencyItem = -1;
static gint ett_rnsap_AffectedUEInformationForMBMS = -1;
static gint ett_rnsap_AllocationRetentionPriority = -1;
static gint ett_rnsap_Allowed_Rate_Information = -1;
static gint ett_rnsap_Angle_Of_Arrival_Value_LCR = -1;
static gint ett_rnsap_BadSatellites = -1;
static gint ett_rnsap_T_badSatelliteInformation = -1;
static gint ett_rnsap_T_badSatelliteInformation_item = -1;
static gint ett_rnsap_BSIC = -1;
static gint ett_rnsap_BurstModeParameters = -1;
static gint ett_rnsap_Cause = -1;
static gint ett_rnsap_Cell_Capacity_Class_Value = -1;
static gint ett_rnsap_CGI = -1;
static gint ett_rnsap_T_lAI = -1;
static gint ett_rnsap_CommonMeasurementAccuracy = -1;
static gint ett_rnsap_CommonMeasurementValue = -1;
static gint ett_rnsap_CommonMeasurementValueInformation = -1;
static gint ett_rnsap_CommonMeasurementAvailable = -1;
static gint ett_rnsap_CorrespondingCells = -1;
static gint ett_rnsap_CriticalityDiagnostics = -1;
static gint ett_rnsap_CriticalityDiagnostics_IE_List = -1;
static gint ett_rnsap_CriticalityDiagnostics_IE_List_item = -1;
static gint ett_rnsap_MessageStructure = -1;
static gint ett_rnsap_MessageStructure_item = -1;
static gint ett_rnsap_CN_CS_DomainIdentifier = -1;
static gint ett_rnsap_CN_PS_DomainIdentifier = -1;
static gint ett_rnsap_DCH_FDD_Information = -1;
static gint ett_rnsap_DCH_FDD_InformationItem = -1;
static gint ett_rnsap_DCH_Specific_FDD_InformationList = -1;
static gint ett_rnsap_DCH_Specific_FDD_Item = -1;
static gint ett_rnsap_DCH_InformationResponse = -1;
static gint ett_rnsap_DCH_InformationResponseItem = -1;
static gint ett_rnsap_DCH_TDD_Information = -1;
static gint ett_rnsap_DCH_TDD_InformationItem = -1;
static gint ett_rnsap_DCH_Specific_TDD_InformationList = -1;
static gint ett_rnsap_DCH_Specific_TDD_Item = -1;
static gint ett_rnsap_DedicatedMeasurementValue = -1;
static gint ett_rnsap_DedicatedMeasurementValueInformation = -1;
static gint ett_rnsap_DedicatedMeasurementAvailable = -1;
static gint ett_rnsap_DelayedActivation = -1;
static gint ett_rnsap_DelayedActivationUpdate = -1;
static gint ett_rnsap_Activate_Info = -1;
static gint ett_rnsap_Deactivate_Info = -1;
static gint ett_rnsap_Execution_Type = -1;
static gint ett_rnsap_DGPSCorrections = -1;
static gint ett_rnsap_T_satellite_DGPSCorrections_Information = -1;
static gint ett_rnsap_T_satellite_DGPSCorrections_Information_item = -1;
static gint ett_rnsap_DGPSThreshold = -1;
static gint ett_rnsap_DL_PowerBalancing_Information = -1;
static gint ett_rnsap_DL_ReferencePowerInformationList = -1;
static gint ett_rnsap_DL_ReferencePowerInformationItem = -1;
static gint ett_rnsap_DL_ReferencePowerInformation = -1;
static gint ett_rnsap_DL_Timeslot_Information = -1;
static gint ett_rnsap_DL_Timeslot_InformationItem = -1;
static gint ett_rnsap_DL_TimeslotLCR_Information = -1;
static gint ett_rnsap_DL_TimeslotLCR_InformationItem = -1;
static gint ett_rnsap_DL_TimeSlot_ISCP_Info = -1;
static gint ett_rnsap_DL_TimeSlot_ISCP_InfoItem = -1;
static gint ett_rnsap_DL_TimeSlot_ISCP_LCR_Information = -1;
static gint ett_rnsap_DL_TimeSlot_ISCP_LCR_InfoItem = -1;
static gint ett_rnsap_DSCH_FlowControlInformation = -1;
static gint ett_rnsap_DSCH_FlowControlItem = -1;
static gint ett_rnsap_DSCH_TDD_Information = -1;
static gint ett_rnsap_DSCH_TDD_InformationItem = -1;
static gint ett_rnsap_EDCH_FDD_DL_ControlChannelInformation = -1;
static gint ett_rnsap_EDCH_FDD_Information = -1;
static gint ett_rnsap_EDCH_FDD_InformationResponse = -1;
static gint ett_rnsap_EDCH_FDD_InformationResponseItem = -1;
static gint ett_rnsap_EDCH_FDD_Information_To_Modify = -1;
static gint ett_rnsap_EDCH_FDD_Update_Information = -1;
static gint ett_rnsap_EDCH_FDD_Update_InfoItem = -1;
static gint ett_rnsap_E_DCH_Grant_Type_Information = -1;
static gint ett_rnsap_E_DCH_LogicalChannelInformation = -1;
static gint ett_rnsap_E_DCH_LogicalChannelInformationItem = -1;
static gint ett_rnsap_E_DCH_MACdPDU_SizeList = -1;
static gint ett_rnsap_E_DCH_MACdPDU_SizeListItem = -1;
static gint ett_rnsap_E_DCH_LogicalChannelToModify = -1;
static gint ett_rnsap_E_DCH_LogicalChannelToModifyItem = -1;
static gint ett_rnsap_E_DCH_MACdPDU_SizeToModifyList = -1;
static gint ett_rnsap_E_DCH_LogicalChannelToDelete = -1;
static gint ett_rnsap_E_DCH_LogicalChannelToDeleteItem = -1;
static gint ett_rnsap_EDCH_MACdFlows_Information = -1;
static gint ett_rnsap_EDCH_MACdFlow_Specific_InfoList = -1;
static gint ett_rnsap_EDCH_MACdFlow_Specific_InfoItem = -1;
static gint ett_rnsap_EDCH_MACdFlow_Specific_InfoToModifyList = -1;
static gint ett_rnsap_EDCH_MACdFlow_Specific_InfoToModifyItem = -1;
static gint ett_rnsap_EDCH_MACdFlows_To_Delete = -1;
static gint ett_rnsap_EDCH_MACdFlows_To_Delete_Item = -1;
static gint ett_rnsap_E_DCH_Non_Scheduled_Transmission_Grant_Items = -1;
static gint ett_rnsap_EDPCH_Information_FDD = -1;
static gint ett_rnsap_EDPCH_Information_RLReconfPrepare_FDD = -1;
static gint ett_rnsap_EDPCH_Information_RLReconfRequest_FDD = -1;
static gint ett_rnsap_EDCH_Serving_RL = -1;
static gint ett_rnsap_EDCH_Serving_RL_in_this_DRNS = -1;
static gint ett_rnsap_E_TFCS_Information = -1;
static gint ett_rnsap_EventA = -1;
static gint ett_rnsap_EventB = -1;
static gint ett_rnsap_EventC = -1;
static gint ett_rnsap_EventD = -1;
static gint ett_rnsap_EventE = -1;
static gint ett_rnsap_EventF = -1;
static gint ett_rnsap_FACH_FlowControlInformation = -1;
static gint ett_rnsap_FACH_FlowControlInformationItem = -1;
static gint ett_rnsap_FACH_InformationList = -1;
static gint ett_rnsap_FACH_InformationItem = -1;
static gint ett_rnsap_FACH_PCH_InformationList = -1;
static gint ett_rnsap_FACH_PCH_InformationItem = -1;
static gint ett_rnsap_FDD_DCHs_to_Modify = -1;
static gint ett_rnsap_FDD_DCHs_to_ModifyItem = -1;
static gint ett_rnsap_FDD_DCHs_to_ModifySpecificInformationList = -1;
static gint ett_rnsap_FDD_DCHs_to_ModifySpecificItem = -1;
static gint ett_rnsap_FDD_DL_CodeInformation = -1;
static gint ett_rnsap_FDD_DL_CodeInformationItem = -1;
static gint ett_rnsap_FPACH_Information = -1;
static gint ett_rnsap_GA_Cell = -1;
static gint ett_rnsap_GA_Cell_item = -1;
static gint ett_rnsap_GA_CellAdditionalShapes = -1;
static gint ett_rnsap_GA_AltitudeAndDirection = -1;
static gint ett_rnsap_GA_EllipsoidArc = -1;
static gint ett_rnsap_GA_PointWithAltitude = -1;
static gint ett_rnsap_GA_PointWithAltitudeAndUncertaintyEllipsoid = -1;
static gint ett_rnsap_GA_PointWithUnCertaintyEllipse = -1;
static gint ett_rnsap_GA_UncertaintyEllipse = -1;
static gint ett_rnsap_GA_PointWithUnCertainty = -1;
static gint ett_rnsap_GA_AccessPointPosition = -1;
static gint ett_rnsap_GeographicalCoordinate = -1;
static gint ett_rnsap_GERAN_SI_Type = -1;
static gint ett_rnsap_GERAN_SystemInfo = -1;
static gint ett_rnsap_GERAN_SystemInfo_item = -1;
static gint ett_rnsap_GPS_Almanac = -1;
static gint ett_rnsap_T_satellite_Almanac_Information = -1;
static gint ett_rnsap_T_satellite_Almanac_Information_item = -1;
static gint ett_rnsap_Satellite_Almanac_Information_ExtItem = -1;
static gint ett_rnsap_Satellite_Almanac_Information_ExtItem_item = -1;
static gint ett_rnsap_GPSInformation = -1;
static gint ett_rnsap_GPSInformation_item = -1;
static gint ett_rnsap_GPS_Ionospheric_Model = -1;
static gint ett_rnsap_GPS_NavigationModel_and_TimeRecovery = -1;
static gint ett_rnsap_GPS_NavigationModel_and_TimeRecovery_item = -1;
static gint ett_rnsap_GPS_RealTime_Integrity = -1;
static gint ett_rnsap_GPS_RX_POS = -1;
static gint ett_rnsap_GPS_UTC_Model = -1;
static gint ett_rnsap_Guaranteed_Rate_Information = -1;
static gint ett_rnsap_HARQ_MemoryPartitioning = -1;
static gint ett_rnsap_HARQ_MemoryPartitioning_Implicit = -1;
static gint ett_rnsap_HARQ_MemoryPartitioning_Explicit = -1;
static gint ett_rnsap_HARQ_MemoryPartitioningList = -1;
static gint ett_rnsap_HARQ_MemoryPartitioningItem = -1;
static gint ett_rnsap_HSDSCH_FDD_Information = -1;
static gint ett_rnsap_HSDSCH_FDD_Information_Response = -1;
static gint ett_rnsap_HSDSCH_Information_to_Modify = -1;
static gint ett_rnsap_HSDSCH_Information_to_Modify_Unsynchronised = -1;
static gint ett_rnsap_HSDSCH_MACdFlow_Specific_InfoList = -1;
static gint ett_rnsap_HSDSCH_MACdFlow_Specific_InfoItem = -1;
static gint ett_rnsap_HSDSCH_MACdFlow_Specific_InfoList_Response = -1;
static gint ett_rnsap_HSDSCH_MACdFlow_Specific_InfoItem_Response = -1;
static gint ett_rnsap_HSDSCH_MACdFlow_Specific_InfoList_to_Modify = -1;
static gint ett_rnsap_HSDSCH_MACdFlow_Specific_InfoItem_to_Modify = -1;
static gint ett_rnsap_HSDSCH_MACdFlows_Information = -1;
static gint ett_rnsap_HSDSCH_MACdFlows_to_Delete = -1;
static gint ett_rnsap_HSDSCH_MACdFlows_to_Delete_Item = -1;
static gint ett_rnsap_HSDSCH_Initial_Capacity_Allocation = -1;
static gint ett_rnsap_HSDSCH_Initial_Capacity_AllocationItem = -1;
static gint ett_rnsap_HSDSCH_TDD_Information = -1;
static gint ett_rnsap_HSDSCH_TDD_Information_Response = -1;
static gint ett_rnsap_HSPDSCH_TDD_Specific_InfoList_Response = -1;
static gint ett_rnsap_HSPDSCH_TDD_Specific_InfoItem_Response = -1;
static gint ett_rnsap_HSPDSCH_TDD_Specific_InfoList_Response_LCR = -1;
static gint ett_rnsap_HSPDSCH_TDD_Specific_InfoItem_Response_LCR = -1;
static gint ett_rnsap_HSSCCH_FDD_Specific_InfoList_Response = -1;
static gint ett_rnsap_HSSCCH_FDD_Specific_InfoItem_Response = -1;
static gint ett_rnsap_HSSCCH_TDD_Specific_InfoList_Response = -1;
static gint ett_rnsap_HSSCCH_TDD_Specific_InfoItem_Response = -1;
static gint ett_rnsap_HSSCCH_TDD_Specific_InfoList_Response_LCR = -1;
static gint ett_rnsap_HSSCCH_TDD_Specific_InfoItem_Response_LCR = -1;
static gint ett_rnsap_HSSICH_Info = -1;
static gint ett_rnsap_HSSICH_InfoLCR = -1;
static gint ett_rnsap_HS_SICH_Reception_Quality_Value = -1;
static gint ett_rnsap_HSDSCH_FDD_Update_Information = -1;
static gint ett_rnsap_HSDSCH_TDD_Update_Information = -1;
static gint ett_rnsap_InformationAvailable = -1;
static gint ett_rnsap_InformationReportCharacteristics = -1;
static gint ett_rnsap_InformationReportPeriodicity = -1;
static gint ett_rnsap_InformationThreshold = -1;
static gint ett_rnsap_InformationType = -1;
static gint ett_rnsap_IPDLParameters = -1;
static gint ett_rnsap_IPDL_FDD_Parameters = -1;
static gint ett_rnsap_IPDL_TDD_Parameters = -1;
static gint ett_rnsap_IPDL_TDD_ParametersLCR = -1;
static gint ett_rnsap_LoadValue = -1;
static gint ett_rnsap_MAC_c_sh_SDU_LengthList = -1;
static gint ett_rnsap_MACdPDU_Size_IndexList = -1;
static gint ett_rnsap_MACdPDU_Size_IndexItem = -1;
static gint ett_rnsap_MACdPDU_Size_IndexList_to_Modify = -1;
static gint ett_rnsap_MACdPDU_Size_IndexItem_to_Modify = -1;
static gint ett_rnsap_MBMS_Bearer_Service_Full_Address = -1;
static gint ett_rnsap_MBMS_Bearer_Service_List = -1;
static gint ett_rnsap_MBMS_Bearer_ServiceItemFDD = -1;
static gint ett_rnsap_MBMS_Bearer_ServiceItemFDD_PFL = -1;
static gint ett_rnsap_MBMS_Bearer_ServiceItemTDD = -1;
static gint ett_rnsap_MBMS_Bearer_ServiceItemTDD_PFL = -1;
static gint ett_rnsap_MBMSChannelTypeInfo = -1;
static gint ett_rnsap_MBMSChannelTypeCellList = -1;
static gint ett_rnsap_MBMSPreferredFreqLayerInfo = -1;
static gint ett_rnsap_MeasurementIncreaseDecreaseThreshold = -1;
static gint ett_rnsap_MeasurementThreshold = -1;
static gint ett_rnsap_MidambleShiftAndBurstType = -1;
static gint ett_rnsap_T_type1 = -1;
static gint ett_rnsap_T_midambleAllocationMode = -1;
static gint ett_rnsap_T_type2 = -1;
static gint ett_rnsap_T_midambleAllocationMode1 = -1;
static gint ett_rnsap_T_type3 = -1;
static gint ett_rnsap_T_midambleAllocationMode2 = -1;
static gint ett_rnsap_MidambleShiftLCR = -1;
static gint ett_rnsap_ModifyPriorityQueue = -1;
static gint ett_rnsap_NACC_Related_Data = -1;
static gint ett_rnsap_Neighbouring_UMTS_CellInformation = -1;
static gint ett_rnsap_Neighbouring_UMTS_CellInformationItem = -1;
static gint ett_rnsap_Neighbouring_FDD_CellInformation = -1;
static gint ett_rnsap_Neighbouring_FDD_CellInformationItem = -1;
static gint ett_rnsap_NeighbouringFDDCellMeasurementInformation = -1;
static gint ett_rnsap_Neighbouring_GSM_CellInformationIEs = -1;
static gint ett_rnsap_Neighbouring_GSM_CellInformationItem = -1;
static gint ett_rnsap_Neighbouring_TDD_CellInformation = -1;
static gint ett_rnsap_Neighbouring_TDD_CellInformationItem = -1;
static gint ett_rnsap_NeighbouringTDDCellMeasurementInformation = -1;
static gint ett_rnsap_NeighbouringTDDCellMeasurementInformationLCR = -1;
static gint ett_rnsap_Neighbouring_LCR_TDD_CellInformation = -1;
static gint ett_rnsap_Neighbouring_LCR_TDD_CellInformationItem = -1;
static gint ett_rnsap_NotProvidedCellList = -1;
static gint ett_rnsap_NRTLoadInformationValue = -1;
static gint ett_rnsap_OnModification = -1;
static gint ett_rnsap_OnModificationInformation = -1;
static gint ett_rnsap_PCH_InformationList = -1;
static gint ett_rnsap_PCH_InformationItem = -1;
static gint ett_rnsap_Periodic = -1;
static gint ett_rnsap_PeriodicInformation = -1;
static gint ett_rnsap_Permanent_NAS_UE_Identity = -1;
static gint ett_rnsap_PreferredFrequencyLayerInfo = -1;
static gint ett_rnsap_PriorityQueue_InfoList = -1;
static gint ett_rnsap_PriorityQueue_InfoItem = -1;
static gint ett_rnsap_PriorityQueue_InfoList_to_Modify = -1;
static gint ett_rnsap_PriorityQueue_InfoItem_to_Add = -1;
static gint ett_rnsap_PriorityQueue_InfoItem_to_Modify = -1;
static gint ett_rnsap_PriorityQueue_InfoList_to_Modify_Unsynchronised = -1;
static gint ett_rnsap_PriorityQueue_InfoItem_to_Modify_Unsynchronised = -1;
static gint ett_rnsap_ProvidedInformation = -1;
static gint ett_rnsap_PTMCellList = -1;
static gint ett_rnsap_PTPCellList = -1;
static gint ett_rnsap_RB_Info = -1;
static gint ett_rnsap_Reference_E_TFCI_Information = -1;
static gint ett_rnsap_Reference_E_TFCI_Information_Item = -1;
static gint ett_rnsap_ReportCharacteristics = -1;
static gint ett_rnsap_ReportPeriodicity = -1;
static gint ett_rnsap_RequestedDataValue = -1;
static gint ett_rnsap_RequestedDataValueInformation = -1;
static gint ett_rnsap_RL_Specific_DCH_Info = -1;
static gint ett_rnsap_RL_Specific_DCH_Info_Item = -1;
static gint ett_rnsap_RL_Specific_EDCH_Information = -1;
static gint ett_rnsap_RL_Specific_EDCH_InfoItem = -1;
static gint ett_rnsap_RTLoadValue = -1;
static gint ett_rnsap_SAI = -1;
static gint ett_rnsap_Secondary_CCPCH_Info_TDD = -1;
static gint ett_rnsap_Secondary_CPICH_Information = -1;
static gint ett_rnsap_Secondary_CPICH_Information_Change = -1;
static gint ett_rnsap_Secondary_LCR_CCPCH_Info_TDD = -1;
static gint ett_rnsap_Secondary_CCPCH_TDD_InformationList = -1;
static gint ett_rnsap_Secondary_CCPCH_TDD_InformationItem = -1;
static gint ett_rnsap_Secondary_LCR_CCPCH_TDD_InformationList = -1;
static gint ett_rnsap_Secondary_LCR_CCPCH_TDD_InformationItem = -1;
static gint ett_rnsap_Secondary_CCPCH_TDD_Code_Information = -1;
static gint ett_rnsap_Secondary_CCPCH_TDD_Code_InformationItem = -1;
static gint ett_rnsap_Secondary_LCR_CCPCH_TDD_Code_Information = -1;
static gint ett_rnsap_Secondary_LCR_CCPCH_TDD_Code_InformationItem = -1;
static gint ett_rnsap_GA_AccessPointPositionwithOptionalAltitude = -1;
static gint ett_rnsap_SFNSFNMeasurementThresholdInformation = -1;
static gint ett_rnsap_SFNSFNMeasurementValueInformation = -1;
static gint ett_rnsap_T_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation = -1;
static gint ett_rnsap_T_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item = -1;
static gint ett_rnsap_T_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation = -1;
static gint ett_rnsap_T_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item = -1;
static gint ett_rnsap_SFNSFNTimeStampInformation = -1;
static gint ett_rnsap_SFNSFNTimeStamp_TDD = -1;
static gint ett_rnsap_SFNSFNValue = -1;
static gint ett_rnsap_SNA_Information = -1;
static gint ett_rnsap_ListOfSNAs = -1;
static gint ett_rnsap_S_RNTI_Group = -1;
static gint ett_rnsap_SynchronisationConfiguration = -1;
static gint ett_rnsap_SYNC_UL_ProcParameters = -1;
static gint ett_rnsap_TDD_ChannelisationCodeLCR = -1;
static gint ett_rnsap_TDD_DCHs_to_Modify = -1;
static gint ett_rnsap_TDD_DCHs_to_ModifyItem = -1;
static gint ett_rnsap_TDD_DCHs_to_ModifySpecificInformationList = -1;
static gint ett_rnsap_TDD_DCHs_to_ModifySpecificItem = -1;
static gint ett_rnsap_TDD_DL_Code_Information = -1;
static gint ett_rnsap_TDD_DL_Code_InformationItem = -1;
static gint ett_rnsap_TDD_DL_Code_LCR_Information = -1;
static gint ett_rnsap_TDD_DL_Code_LCR_InformationItem = -1;
static gint ett_rnsap_TDD_DL_DPCH_TimeSlotFormat_LCR = -1;
static gint ett_rnsap_TDD_DPCHOffset = -1;
static gint ett_rnsap_TDD_UL_Code_Information = -1;
static gint ett_rnsap_TDD_UL_Code_InformationItem = -1;
static gint ett_rnsap_TDD_UL_Code_LCR_Information = -1;
static gint ett_rnsap_TDD_UL_Code_LCR_InformationItem = -1;
static gint ett_rnsap_TDD_UL_DPCH_TimeSlotFormat_LCR = -1;
static gint ett_rnsap_TMGI = -1;
static gint ett_rnsap_TnlQos = -1;
static gint ett_rnsap_Transmission_Gap_Pattern_Sequence_Information = -1;
static gint ett_rnsap_Transmission_Gap_Pattern_Sequence_Information_item = -1;
static gint ett_rnsap_Transmission_Gap_Pattern_Sequence_Status_List = -1;
static gint ett_rnsap_Transmission_Gap_Pattern_Sequence_Status_List_item = -1;
static gint ett_rnsap_TUTRANGPS = -1;
static gint ett_rnsap_TUTRANGPSMeasurementThresholdInformation = -1;
static gint ett_rnsap_TUTRANGPSMeasurementValueInformation = -1;
static gint ett_rnsap_TransportFormatCombination_Beta = -1;
static gint ett_rnsap_T_signalledGainFactors = -1;
static gint ett_rnsap_TFCS = -1;
static gint ett_rnsap_T_tFCSvalues = -1;
static gint ett_rnsap_TFCS_TFCSList = -1;
static gint ett_rnsap_TFCS_TFCSList_item = -1;
static gint ett_rnsap_TFCS_CTFC = -1;
static gint ett_rnsap_TransportFormatSet = -1;
static gint ett_rnsap_TransportFormatSet_DynamicPartList = -1;
static gint ett_rnsap_TransportFormatSet_DynamicPartList_item = -1;
static gint ett_rnsap_TransportFormatSet_ModeDP = -1;
static gint ett_rnsap_TDD_TransportFormatSet_ModeDP = -1;
static gint ett_rnsap_TransmissionTimeIntervalInformation = -1;
static gint ett_rnsap_TransmissionTimeIntervalInformation_item = -1;
static gint ett_rnsap_TransportFormatSet_Semi_staticPart = -1;
static gint ett_rnsap_TransportFormatSet_ModeSSP = -1;
static gint ett_rnsap_UE_Capabilities_Info = -1;
static gint ett_rnsap_UEIdentity = -1;
static gint ett_rnsap_UEMeasurementReportCharacteristics = -1;
static gint ett_rnsap_UEMeasurementReportCharacteristicsEvent1h = -1;
static gint ett_rnsap_UEMeasurementReportCharacteristicsEvent1i = -1;
static gint ett_rnsap_UEMeasurementReportCharacteristicsEvent6a = -1;
static gint ett_rnsap_UEMeasurementReportCharacteristicsEvent6b = -1;
static gint ett_rnsap_UEMeasurementReportCharacteristicsEvent6c = -1;
static gint ett_rnsap_UEMeasurementReportCharacteristicsEvent6d = -1;
static gint ett_rnsap_UEMeasurementReportCharacteristicsPeriodic = -1;
static gint ett_rnsap_UEMeasurementThreshold = -1;
static gint ett_rnsap_UEMeasurementTimeslotInfoHCR = -1;
static gint ett_rnsap_UEMeasurementTimeslotInfoHCR_IEs = -1;
static gint ett_rnsap_UEMeasurementTimeslotInfoLCR = -1;
static gint ett_rnsap_UEMeasurementTimeslotInfoLCR_IEs = -1;
static gint ett_rnsap_UEMeasurementValue = -1;
static gint ett_rnsap_UE_MeasurementValue_UE_Transmitted_Power = -1;
static gint ett_rnsap_UEMeasurementValueTransmittedPowerListHCR = -1;
static gint ett_rnsap_UEMeasurementValueTransmittedPowerListHCR_IEs = -1;
static gint ett_rnsap_UEMeasurementValueTransmittedPowerListLCR = -1;
static gint ett_rnsap_UEMeasurementValueTransmittedPowerListLCR_IEs = -1;
static gint ett_rnsap_UE_MeasurementValue_Primary_CCPCH_RSCP = -1;
static gint ett_rnsap_UE_MeasurementValue_DL_Timeslot_ISCP = -1;
static gint ett_rnsap_UEMeasurementValueTimeslotISCPListHCR = -1;
static gint ett_rnsap_UEMeasurementValueTimeslotISCPListHCR_IEs = -1;
static gint ett_rnsap_UEMeasurementValueTimeslotISCPListLCR = -1;
static gint ett_rnsap_UEMeasurementValueTimeslotISCPListLCR_IEs = -1;
static gint ett_rnsap_UEMeasurementValueInformation = -1;
static gint ett_rnsap_UEMeasurementValueInformationAvailable = -1;
static gint ett_rnsap_UE_State = -1;
static gint ett_rnsap_Cell_Fach_Pch_State = -1;
static gint ett_rnsap_Ura_Pch_State = -1;
static gint ett_rnsap_UL_Timeslot_Information = -1;
static gint ett_rnsap_UL_Timeslot_InformationItem = -1;
static gint ett_rnsap_UL_TimeslotLCR_Information = -1;
static gint ett_rnsap_UL_TimeslotLCR_InformationItem = -1;
static gint ett_rnsap_UL_TimeSlot_ISCP_Info = -1;
static gint ett_rnsap_UL_TimeSlot_ISCP_InfoItem = -1;
static gint ett_rnsap_UL_TimeSlot_ISCP_LCR_Info = -1;
static gint ett_rnsap_UL_TimeSlot_ISCP_LCR_InfoItem = -1;
static gint ett_rnsap_UL_TimingAdvanceCtrl_LCR = -1;
static gint ett_rnsap_UC_ID = -1;
static gint ett_rnsap_UL_ScramblingCode = -1;
static gint ett_rnsap_UL_Synchronisation_Parameters_LCR = -1;
static gint ett_rnsap_URA_Information = -1;
static gint ett_rnsap_RNCsWithCellsInTheAccessedURA_List = -1;
static gint ett_rnsap_RNCsWithCellsInTheAccessedURA_Item = -1;
static gint ett_rnsap_USCH_Information = -1;
static gint ett_rnsap_USCH_InformationItem = -1;
static gint ett_rnsap_PrivateIE_ID = -1;
static gint ett_rnsap_ProcedureID = -1;
static gint ett_rnsap_TransactionID = -1;
static gint ett_rnsap_ProtocolIE_Container = -1;
static gint ett_rnsap_ProtocolIE_Field = -1;
static gint ett_rnsap_ProtocolIE_ContainerPair = -1;
static gint ett_rnsap_ProtocolIE_FieldPair = -1;
static gint ett_rnsap_ProtocolIE_ContainerList = -1;
static gint ett_rnsap_ProtocolIE_ContainerPairList = -1;
static gint ett_rnsap_ProtocolExtensionContainer = -1;
static gint ett_rnsap_ProtocolExtensionField = -1;
static gint ett_rnsap_PrivateIE_Container = -1;
static gint ett_rnsap_PrivateIE_Field = -1;

/*--- End of included file: packet-rnsap-ett.c ---*/
#line 571 "packet-rnsap-template.c"

/* Global variables */
static proto_tree *top_tree;
static guint32 ProcedureCode;
static guint32 ProtocolIE_ID;
static guint32 ddMode;

#define BYTE_ALIGN_OFFSET(offset)		\
	if(offset&0x07){			\
		offset=(offset&0xfffffff8)+8;	\
	}
#define RNSAP_FDD 1
/* Prodedure ID:s */

/* Protocol IE:s */



static int dissect_rnsap_InitiatingMessageValueValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree);
static int dissect_rnsap_SuccessfulOutcomeValueValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree);
static int dissect_rnsap_UnsuccessfulOutcomeValueValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree);
static int dissect_rnsap_ProtocolIEValueValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree);

/*--- Included file: packet-rnsap-fn.c ---*/
#line 1 "packet-rnsap-fn.c"
/*--- Fields for imported types ---*/



static const value_string rnsap_ProcedureCode_vals[] = {
  {   0, "id-commonTransportChannelResourcesInitialisation" },
  {   1, "id-commonTransportChannelResourcesRelease" },
  {   2, "id-compressedModeCommand" },
  {   3, "id-downlinkPowerControl" },
  {   4, "id-downlinkPowerTimeslotControl" },
  {   5, "id-downlinkSignallingTransfer" },
  {   6, "id-errorIndication" },
  {   7, "id-dedicatedMeasurementFailure" },
  {   8, "id-dedicatedMeasurementInitiation" },
  {   9, "id-dedicatedMeasurementReporting" },
  {  10, "id-dedicatedMeasurementTermination" },
  {  11, "id-paging" },
  {  12, "id-physicalChannelReconfiguration" },
  {  13, "id-privateMessage" },
  {  14, "id-radioLinkAddition" },
  {  34, "id-radioLinkCongestion" },
  {  15, "id-radioLinkDeletion" },
  {  16, "id-radioLinkFailure" },
  {  17, "id-radioLinkPreemption" },
  {  18, "id-radioLinkRestoration" },
  {  19, "id-radioLinkSetup" },
  {  20, "id-relocationCommit" },
  {  21, "id-synchronisedRadioLinkReconfigurationCancellation" },
  {  22, "id-synchronisedRadioLinkReconfigurationCommit" },
  {  23, "id-synchronisedRadioLinkReconfigurationPreparation" },
  {  24, "id-unSynchronisedRadioLinkReconfiguration" },
  {  25, "id-uplinkSignallingTransfer" },
  {  26, "id-commonMeasurementFailure" },
  {  27, "id-commonMeasurementInitiation" },
  {  28, "id-commonMeasurementReporting" },
  {  29, "id-commonMeasurementTermination" },
  {  30, "id-informationExchangeFailure" },
  {  31, "id-informationExchangeInitiation" },
  {  32, "id-informationReporting" },
  {  33, "id-informationExchangeTermination" },
  {  35, "id-reset" },
  {  36, "id-radioLinkActivation" },
  {  37, "id-gERANuplinkSignallingTransfer" },
  {  38, "id-radioLinkParameterUpdate" },
  {  39, "id-uEMeasurementFailure" },
  {  40, "id-uEMeasurementInitiation" },
  {  41, "id-uEMeasurementReporting" },
  {  42, "id-uEMeasurementTermination" },
  {  43, "id-iurDeactivateTrace" },
  {  44, "id-iurInvokeTrace" },
  {  45, "id-mBMSAttach" },
  {  46, "id-mBMSDetach" },
  {  48, "id-directInformationTransfer" },
  { 0, NULL }
};


static int
dissect_rnsap_ProcedureCode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 255U, &ProcedureCode, FALSE);

#line 18 "rnsap.cnf"
	if (check_col(actx->pinfo->cinfo, COL_INFO))
       col_add_fstr(actx->pinfo->cinfo, COL_INFO, "%s ",
                   val_to_str(ProcedureCode, rnsap_ProcedureCode_vals,
                              "unknown message"));

  return offset;
}
static int dissect_procedureCode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProcedureCode(tvb, offset, actx, tree, hf_rnsap_procedureCode);
}


static const value_string rnsap_T_ddMode_vals[] = {
  {   0, "tdd" },
  {   1, "fdd" },
  {   2, "common" },
  { 0, NULL }
};


static int
dissect_rnsap_T_ddMode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, &ddMode, TRUE, 0, NULL);

  return offset;
}
static int dissect_ddMode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_T_ddMode(tvb, offset, actx, tree, hf_rnsap_ddMode);
}


static const per_sequence_t ProcedureID_sequence[] = {
  { &hf_rnsap_procedureCode , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProcedureCode },
  { &hf_rnsap_ddMode        , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_T_ddMode },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_ProcedureID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_ProcedureID, ProcedureID_sequence);

  return offset;
}
static int dissect_procedureID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProcedureID(tvb, offset, actx, tree, hf_rnsap_procedureID);
}


static const value_string rnsap_Criticality_vals[] = {
  {   0, "reject" },
  {   1, "ignore" },
  {   2, "notify" },
  { 0, NULL }
};


static int
dissect_rnsap_Criticality(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_criticality(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Criticality(tvb, offset, actx, tree, hf_rnsap_criticality);
}
static int dissect_procedureCriticality(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Criticality(tvb, offset, actx, tree, hf_rnsap_procedureCriticality);
}
static int dissect_iECriticality(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Criticality(tvb, offset, actx, tree, hf_rnsap_iECriticality);
}
static int dissect_firstCriticality(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Criticality(tvb, offset, actx, tree, hf_rnsap_firstCriticality);
}
static int dissect_secondCriticality(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Criticality(tvb, offset, actx, tree, hf_rnsap_secondCriticality);
}



static int
dissect_rnsap_INTEGER_0_127(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 127U, NULL, FALSE);

  return offset;
}
static int dissect_uncertaintyRadius(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_0_127(tvb, offset, actx, tree, hf_rnsap_uncertaintyRadius);
}
static int dissect_confidence(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_0_127(tvb, offset, actx, tree, hf_rnsap_confidence);
}
static int dissect_uncertaintyAltitude(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_0_127(tvb, offset, actx, tree, hf_rnsap_uncertaintyAltitude);
}
static int dissect_uncertaintySemi_major(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_0_127(tvb, offset, actx, tree, hf_rnsap_uncertaintySemi_major);
}
static int dissect_uncertaintySemi_minor(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_0_127(tvb, offset, actx, tree, hf_rnsap_uncertaintySemi_minor);
}
static int dissect_uncertaintyCode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_0_127(tvb, offset, actx, tree, hf_rnsap_uncertaintyCode);
}
static int dissect_code_Number(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_0_127(tvb, offset, actx, tree, hf_rnsap_code_Number);
}
static int dissect_shortTransActionId(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_0_127(tvb, offset, actx, tree, hf_rnsap_shortTransActionId);
}



static int
dissect_rnsap_INTEGER_0_32767(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 32767U, NULL, FALSE);

  return offset;
}
static int dissect_altitude(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_0_32767(tvb, offset, actx, tree, hf_rnsap_altitude);
}
static int dissect_longTransActionId(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_0_32767(tvb, offset, actx, tree, hf_rnsap_longTransActionId);
}


static const value_string rnsap_TransactionID_vals[] = {
  {   0, "shortTransActionId" },
  {   1, "longTransActionId" },
  { 0, NULL }
};

static const per_choice_t TransactionID_choice[] = {
  {   0, &hf_rnsap_shortTransActionId, ASN1_NO_EXTENSIONS     , dissect_rnsap_INTEGER_0_127 },
  {   1, &hf_rnsap_longTransActionId, ASN1_NO_EXTENSIONS     , dissect_rnsap_INTEGER_0_32767 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_TransactionID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_TransactionID, TransactionID_choice,
                                 NULL);

  return offset;
}
static int dissect_transactionID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TransactionID(tvb, offset, actx, tree, hf_rnsap_transactionID);
}



static int
dissect_rnsap_InitiatingMessageValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
#line 35 "rnsap.cnf"

	offset = dissect_rnsap_InitiatingMessageValueValue(tvb, offset, actx, tree);



  return offset;
}
static int dissect_initiatingMessageValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_InitiatingMessageValue(tvb, offset, actx, tree, hf_rnsap_initiatingMessageValue);
}


static const per_sequence_t InitiatingMessage_sequence[] = {
  { &hf_rnsap_procedureID   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProcedureID },
  { &hf_rnsap_criticality   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_Criticality },
  { &hf_rnsap_transactionID , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_TransactionID },
  { &hf_rnsap_initiatingMessageValue, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_InitiatingMessageValue },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_InitiatingMessage(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_InitiatingMessage, InitiatingMessage_sequence);

  return offset;
}
static int dissect_initiatingMessage(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_InitiatingMessage(tvb, offset, actx, tree, hf_rnsap_initiatingMessage);
}



static int
dissect_rnsap_SuccessfulOutcomeValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
#line 39 "rnsap.cnf"

	offset = dissect_rnsap_SuccessfulOutcomeValueValue(tvb, offset, actx, tree);



  return offset;
}
static int dissect_successfulOutcomeValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SuccessfulOutcomeValue(tvb, offset, actx, tree, hf_rnsap_successfulOutcomeValue);
}


static const per_sequence_t SuccessfulOutcome_sequence[] = {
  { &hf_rnsap_procedureID   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProcedureID },
  { &hf_rnsap_criticality   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_Criticality },
  { &hf_rnsap_transactionID , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_TransactionID },
  { &hf_rnsap_successfulOutcomeValue, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_SuccessfulOutcomeValue },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_SuccessfulOutcome(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_SuccessfulOutcome, SuccessfulOutcome_sequence);

  return offset;
}
static int dissect_successfulOutcome(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SuccessfulOutcome(tvb, offset, actx, tree, hf_rnsap_successfulOutcome);
}



static int
dissect_rnsap_UnsuccessfulOutcomeValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
#line 51 "rnsap.cnf"

	offset = dissect_rnsap_UnsuccessfulOutcomeValueValue(tvb, offset, actx, tree);



  return offset;
}
static int dissect_unsuccessfulOutcomeValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UnsuccessfulOutcomeValue(tvb, offset, actx, tree, hf_rnsap_unsuccessfulOutcomeValue);
}


static const per_sequence_t UnsuccessfulOutcome_sequence[] = {
  { &hf_rnsap_procedureID   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProcedureID },
  { &hf_rnsap_criticality   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_Criticality },
  { &hf_rnsap_transactionID , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_TransactionID },
  { &hf_rnsap_unsuccessfulOutcomeValue, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_UnsuccessfulOutcomeValue },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UnsuccessfulOutcome(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UnsuccessfulOutcome, UnsuccessfulOutcome_sequence);

  return offset;
}
static int dissect_unsuccessfulOutcome(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UnsuccessfulOutcome(tvb, offset, actx, tree, hf_rnsap_unsuccessfulOutcome);
}



static int
dissect_rnsap_OutcomeValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
#line 43 "rnsap.cnf"
	/* FIX ME */
	guint length;

	offset = dissect_per_length_determinant(tvb, offset, actx, tree, hf_rnsap_pdu_length, &length);
	offset = offset + length;
	BYTE_ALIGN_OFFSET(offset)



  return offset;
}
static int dissect_outcomeValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_OutcomeValue(tvb, offset, actx, tree, hf_rnsap_outcomeValue);
}


static const per_sequence_t Outcome_sequence[] = {
  { &hf_rnsap_procedureID   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProcedureID },
  { &hf_rnsap_criticality   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_Criticality },
  { &hf_rnsap_transactionID , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_TransactionID },
  { &hf_rnsap_outcomeValue  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_OutcomeValue },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_Outcome(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_Outcome, Outcome_sequence);

  return offset;
}
static int dissect_outcome(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Outcome(tvb, offset, actx, tree, hf_rnsap_outcome);
}


static const value_string rnsap_RNSAP_PDU_vals[] = {
  {   0, "initiatingMessage" },
  {   1, "successfulOutcome" },
  {   2, "unsuccessfulOutcome" },
  {   3, "outcome" },
  { 0, NULL }
};

static const per_choice_t RNSAP_PDU_choice[] = {
  {   0, &hf_rnsap_initiatingMessage, ASN1_EXTENSION_ROOT    , dissect_rnsap_InitiatingMessage },
  {   1, &hf_rnsap_successfulOutcome, ASN1_EXTENSION_ROOT    , dissect_rnsap_SuccessfulOutcome },
  {   2, &hf_rnsap_unsuccessfulOutcome, ASN1_EXTENSION_ROOT    , dissect_rnsap_UnsuccessfulOutcome },
  {   3, &hf_rnsap_outcome       , ASN1_EXTENSION_ROOT    , dissect_rnsap_Outcome },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_RNSAP_PDU(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_RNSAP_PDU, RNSAP_PDU_choice,
                                 NULL);

  return offset;
}


static const value_string rnsap_ProtocolIE_ID_vals[] = {
  {   4, "id-AllowedQueuingTime" },
  {  42, "id-Allowed-Rate-Information" },
  { 309, "id-AntennaColocationIndicator" },
  {   5, "id-BindingID" },
  {   6, "id-C-ID" },
  {   7, "id-C-RNTI" },
  { 303, "id-Cell-Capacity-Class-Value" },
  {   8, "id-CFN" },
  {   9, "id-CN-CS-DomainIdentifier" },
  {  10, "id-CN-PS-DomainIdentifier" },
  {  11, "id-Cause" },
  { 310, "id-CoverageIndicator" },
  {  20, "id-CriticalityDiagnostics" },
  { 211, "id-ContextInfoItem-Reset" },
  { 515, "id-ContextGroupInfoItem-Reset" },
  {  21, "id-D-RNTI" },
  {  22, "id-D-RNTI-ReleaseIndication" },
  {  26, "id-DCHs-to-Add-FDD" },
  {  27, "id-DCHs-to-Add-TDD" },
  {  30, "id-DCH-DeleteList-RL-ReconfPrepFDD" },
  {  31, "id-DCH-DeleteList-RL-ReconfPrepTDD" },
  {  32, "id-DCH-DeleteList-RL-ReconfRqstFDD" },
  {  33, "id-DCH-DeleteList-RL-ReconfRqstTDD" },
  {  34, "id-DCH-FDD-Information" },
  {  35, "id-DCH-TDD-Information" },
  {  39, "id-FDD-DCHs-to-Modify" },
  {  40, "id-TDD-DCHs-to-Modify" },
  {  43, "id-DCH-InformationResponse" },
  {  38, "id-DCH-Rate-InformationItem-RL-CongestInd" },
  {  44, "id-DL-CCTrCH-InformationAddItem-RL-ReconfPrepTDD" },
  {  45, "id-DL-CCTrCH-InformationListIE-RL-ReconfReadyTDD" },
  {  46, "id-DL-CCTrCH-InformationDeleteItem-RL-ReconfRqstTDD" },
  {  47, "id-DL-CCTrCH-InformationItem-RL-SetupRqstTDD" },
  {  48, "id-DL-CCTrCH-InformationListIE-PhyChReconfRqstTDD" },
  {  49, "id-DL-CCTrCH-InformationListIE-RL-AdditionRspTDD" },
  {  50, "id-DL-CCTrCH-InformationListIE-RL-SetupRspTDD" },
  {  51, "id-DL-CCTrCH-InformationAddList-RL-ReconfPrepTDD" },
  {  52, "id-DL-CCTrCH-InformationDeleteList-RL-ReconfRqstTDD" },
  {  53, "id-DL-CCTrCH-InformationList-RL-SetupRqstTDD" },
  {  54, "id-FDD-DL-CodeInformation" },
  {  59, "id-DL-DPCH-Information-RL-ReconfPrepFDD" },
  {  60, "id-DL-DPCH-Information-RL-SetupRqstFDD" },
  {  61, "id-DL-DPCH-Information-RL-ReconfRqstFDD" },
  {  62, "id-DL-DPCH-InformationItem-PhyChReconfRqstTDD" },
  {  63, "id-DL-DPCH-InformationItem-RL-AdditionRspTDD" },
  {  64, "id-DL-DPCH-InformationItem-RL-SetupRspTDD" },
  { 278, "id-DL-DPCH-TimingAdjustment" },
  {  67, "id-DLReferencePower" },
  {  68, "id-DLReferencePowerList-DL-PC-Rqst" },
  {  69, "id-DL-ReferencePowerInformation-DL-PC-Rqst" },
  {  12, "id-DPC-Mode" },
  {  70, "id-DRXCycleLengthCoefficient" },
  { 470, "id-DedicatedMeasurementObjectType-DM-Fail-Ind" },
  { 471, "id-DedicatedMeasurementObjectType-DM-Fail" },
  {  71, "id-DedicatedMeasurementObjectType-DM-Rprt" },
  {  72, "id-DedicatedMeasurementObjectType-DM-Rqst" },
  {  73, "id-DedicatedMeasurementObjectType-DM-Rsp" },
  {  74, "id-DedicatedMeasurementType" },
  {  82, "id-FACH-InfoForUESelectedS-CCPCH-CTCH-ResourceRspFDD" },
  {  83, "id-FACH-InfoForUESelectedS-CCPCH-CTCH-ResourceRspTDD" },
  {  41, "id-Guaranteed-Rate-Information" },
  {  84, "id-IMSI" },
  { 311, "id-HCS-Prio" },
  {  85, "id-L3-Information" },
  {  90, "id-AdjustmentPeriod" },
  {  91, "id-MaxAdjustmentStep" },
  {  92, "id-MeasurementFilterCoefficient" },
  {  57, "id-MessageStructure" },
  {  93, "id-MeasurementID" },
  {  13, "id-Neighbouring-GSM-CellInformation" },
  {  95, "id-Neighbouring-UMTS-CellInformationItem" },
  { 305, "id-NRT-Load-Information-Value" },
  { 306, "id-NRT-Load-Information-Value-IncrDecrThres" },
  { 102, "id-PagingArea-PagingRqst" },
  { 103, "id-FACH-FlowControlInformation" },
  { 472, "id-PartialReportingIndicator" },
  {  17, "id-Permanent-NAS-UE-Identity" },
  { 107, "id-PowerAdjustmentType" },
  { 109, "id-RANAP-RelocationInformation" },
  { 110, "id-RL-Information-PhyChReconfRqstFDD" },
  { 111, "id-RL-Information-PhyChReconfRqstTDD" },
  { 112, "id-RL-Information-RL-AdditionRqstFDD" },
  { 113, "id-RL-Information-RL-AdditionRqstTDD" },
  { 114, "id-RL-Information-RL-DeletionRqst" },
  { 115, "id-RL-Information-RL-FailureInd" },
  { 116, "id-RL-Information-RL-ReconfPrepFDD" },
  { 117, "id-RL-Information-RL-RestoreInd" },
  { 118, "id-RL-Information-RL-SetupRqstFDD" },
  { 119, "id-RL-Information-RL-SetupRqstTDD" },
  {  55, "id-RL-InformationItem-RL-CongestInd" },
  { 120, "id-RL-InformationItem-DM-Rprt" },
  { 121, "id-RL-InformationItem-DM-Rqst" },
  { 122, "id-RL-InformationItem-DM-Rsp" },
  {   2, "id-RL-InformationItem-RL-PreemptRequiredInd" },
  { 123, "id-RL-InformationItem-RL-SetupRqstFDD" },
  {  56, "id-RL-InformationList-RL-CongestInd" },
  { 124, "id-RL-InformationList-RL-AdditionRqstFDD" },
  { 125, "id-RL-InformationList-RL-DeletionRqst" },
  {   1, "id-RL-InformationList-RL-PreemptRequiredInd" },
  { 126, "id-RL-InformationList-RL-ReconfPrepFDD" },
  { 127, "id-RL-InformationResponse-RL-AdditionRspTDD" },
  { 128, "id-RL-InformationResponse-RL-ReconfReadyTDD" },
  { 129, "id-RL-InformationResponse-RL-SetupRspTDD" },
  { 130, "id-RL-InformationResponseItem-RL-AdditionRspFDD" },
  { 131, "id-RL-InformationResponseItem-RL-ReconfReadyFDD" },
  { 132, "id-RL-InformationResponseItem-RL-ReconfRspFDD" },
  { 133, "id-RL-InformationResponseItem-RL-SetupRspFDD" },
  { 134, "id-RL-InformationResponseList-RL-AdditionRspFDD" },
  { 135, "id-RL-InformationResponseList-RL-ReconfReadyFDD" },
  { 136, "id-RL-InformationResponseList-RL-ReconfRspFDD" },
  {  28, "id-RL-InformationResponse-RL-ReconfRspTDD" },
  { 137, "id-RL-InformationResponseList-RL-SetupRspFDD" },
  { 141, "id-RL-ReconfigurationFailure-RL-ReconfFail" },
  { 143, "id-RL-Set-InformationItem-DM-Rprt" },
  { 144, "id-RL-Set-InformationItem-DM-Rqst" },
  { 145, "id-RL-Set-InformationItem-DM-Rsp" },
  { 146, "id-RL-Set-Information-RL-FailureInd" },
  { 147, "id-RL-Set-Information-RL-RestoreInd" },
  { 473, "id-RL-Set-Successful-InformationItem-DM-Fail" },
  { 474, "id-RL-Set-Unsuccessful-InformationItem-DM-Fail" },
  { 475, "id-RL-Set-Unsuccessful-InformationItem-DM-Fail-Ind" },
  { 476, "id-RL-Successful-InformationItem-DM-Fail" },
  { 477, "id-RL-Unsuccessful-InformationItem-DM-Fail" },
  { 478, "id-RL-Unsuccessful-InformationItem-DM-Fail-Ind" },
  { 152, "id-ReportCharacteristics" },
  { 153, "id-Reporting-Object-RL-FailureInd" },
  { 154, "id-Reporing-Object-RL-RestoreInd" },
  { 307, "id-RT-Load-Value" },
  { 308, "id-RT-Load-Value-IncrDecrThres" },
  { 155, "id-S-RNTI" },
  { 244, "id-ResetIndicator" },
  { 245, "id-RNC-ID" },
  { 156, "id-SAI" },
  { 157, "id-SRNC-ID" },
  { 159, "id-SuccessfulRL-InformationResponse-RL-AdditionFailureFDD" },
  { 160, "id-SuccessfulRL-InformationResponse-RL-SetupFailureFDD" },
  { 163, "id-TransportBearerID" },
  { 164, "id-TransportBearerRequestIndicator" },
  { 165, "id-TransportLayerAddress" },
  { 140, "id-TypeOfError" },
  { 166, "id-UC-ID" },
  { 167, "id-UL-CCTrCH-AddInformation-RL-ReconfPrepTDD" },
  { 169, "id-UL-CCTrCH-InformationAddList-RL-ReconfPrepTDD" },
  { 171, "id-UL-CCTrCH-InformationItem-RL-SetupRqstTDD" },
  { 172, "id-UL-CCTrCH-InformationList-RL-SetupRqstTDD" },
  { 173, "id-UL-CCTrCH-InformationListIE-PhyChReconfRqstTDD" },
  { 174, "id-UL-CCTrCH-InformationListIE-RL-AdditionRspTDD" },
  { 175, "id-UL-CCTrCH-InformationListIE-RL-ReconfReadyTDD" },
  { 176, "id-UL-CCTrCH-InformationListIE-RL-SetupRspTDD" },
  { 177, "id-UL-DPCH-Information-RL-ReconfPrepFDD" },
  { 178, "id-UL-DPCH-Information-RL-ReconfRqstFDD" },
  { 179, "id-UL-DPCH-Information-RL-SetupRqstFDD" },
  { 180, "id-UL-DPCH-InformationItem-PhyChReconfRqstTDD" },
  { 181, "id-UL-DPCH-InformationItem-RL-AdditionRspTDD" },
  { 182, "id-UL-DPCH-InformationItem-RL-SetupRspTDD" },
  { 183, "id-UL-DPCH-InformationAddListIE-RL-ReconfReadyTDD" },
  { 184, "id-UL-SIRTarget" },
  { 185, "id-URA-Information" },
  { 188, "id-UnsuccessfulRL-InformationResponse-RL-AdditionFailureFDD" },
  { 189, "id-UnsuccessfulRL-InformationResponse-RL-SetupFailureFDD" },
  { 190, "id-UnsuccessfulRL-InformationResponse-RL-SetupFailureTDD" },
  { 193, "id-Active-Pattern-Sequence-Information" },
  { 194, "id-AdjustmentRatio" },
  { 197, "id-CauseLevel-RL-AdditionFailureFDD" },
  { 198, "id-CauseLevel-RL-AdditionFailureTDD" },
  { 199, "id-CauseLevel-RL-ReconfFailure" },
  { 200, "id-CauseLevel-RL-SetupFailureFDD" },
  { 201, "id-CauseLevel-RL-SetupFailureTDD" },
  { 205, "id-DL-CCTrCH-InformationDeleteItem-RL-ReconfPrepTDD" },
  { 206, "id-DL-CCTrCH-InformationModifyItem-RL-ReconfPrepTDD" },
  { 207, "id-DL-CCTrCH-InformationModifyItem-RL-ReconfRqstTDD" },
  { 208, "id-DL-CCTrCH-InformationDeleteList-RL-ReconfPrepTDD" },
  { 209, "id-DL-CCTrCH-InformationModifyList-RL-ReconfPrepTDD" },
  { 210, "id-DL-CCTrCH-InformationModifyList-RL-ReconfRqstTDD" },
  { 212, "id-DL-DPCH-InformationAddListIE-RL-ReconfReadyTDD" },
  { 213, "id-DL-DPCH-InformationDeleteListIE-RL-ReconfReadyTDD" },
  { 214, "id-DL-DPCH-InformationModifyListIE-RL-ReconfReadyTDD" },
  { 215, "id-DSCHs-to-Add-TDD" },
  { 216, "id-Unused-ProtocolIE-ID-216" },
  { 217, "id-DSCH-DeleteList-RL-ReconfPrepTDD" },
  { 218, "id-Unused-ProtocolIE-ID-218" },
  { 219, "id-Unused-ProtocolIE-ID-219" },
  { 220, "id-DSCH-InformationListIE-RL-AdditionRspTDD" },
  { 221, "id-DSCH-InformationListIEs-RL-SetupRspTDD" },
  { 222, "id-DSCH-TDD-Information" },
  { 223, "id-Unused-ProtocolIE-ID-223" },
  { 226, "id-Unused-ProtocolIE-ID-226" },
  { 227, "id-DSCH-ModifyList-RL-ReconfPrepTDD" },
  { 228, "id-Unused-ProtocolIE-ID-228" },
  { 324, "id-Unused-ProtocolIE-ID-324" },
  { 229, "id-Unused-ProtocolIE-ID-229" },
  { 230, "id-DSCHToBeAddedOrModifiedList-RL-ReconfReadyTDD" },
  {  29, "id-Unused-ProtocolIE-ID-29" },
  { 225, "id-Unused-ProtocolIE-ID-225" },
  { 232, "id-GA-Cell" },
  {   3, "id-GA-CellAdditionalShapes" },
  { 246, "id-Unused-ProtocolIE-ID-246" },
  { 255, "id-Transmission-Gap-Pattern-Sequence-Information" },
  { 256, "id-UL-CCTrCH-DeleteInformation-RL-ReconfPrepTDD" },
  { 257, "id-UL-CCTrCH-ModifyInformation-RL-ReconfPrepTDD" },
  { 258, "id-UL-CCTrCH-InformationModifyItem-RL-ReconfRqstTDD" },
  { 259, "id-UL-CCTrCH-InformationDeleteList-RL-ReconfPrepTDD" },
  { 260, "id-UL-CCTrCH-InformationModifyList-RL-ReconfPrepTDD" },
  { 261, "id-UL-CCTrCH-InformationModifyList-RL-ReconfRqstTDD" },
  { 262, "id-UL-CCTrCH-InformationDeleteItem-RL-ReconfRqstTDD" },
  { 263, "id-UL-CCTrCH-InformationDeleteList-RL-ReconfRqstTDD" },
  { 264, "id-UL-DPCH-InformationDeleteListIE-RL-ReconfReadyTDD" },
  { 265, "id-UL-DPCH-InformationModifyListIE-RL-ReconfReadyTDD" },
  { 266, "id-UnsuccessfulRL-InformationResponse-RL-AdditionFailureTDD" },
  { 267, "id-USCHs-to-Add" },
  { 268, "id-USCH-DeleteList-RL-ReconfPrepTDD" },
  { 269, "id-USCH-InformationListIE-RL-AdditionRspTDD" },
  { 270, "id-USCH-InformationListIEs-RL-SetupRspTDD" },
  { 271, "id-USCH-Information" },
  { 272, "id-USCH-ModifyList-RL-ReconfPrepTDD" },
  { 273, "id-USCHToBeAddedOrModifiedList-RL-ReconfReadyTDD" },
  { 274, "id-DL-Physical-Channel-Information-RL-SetupRqstTDD" },
  { 275, "id-UL-Physical-Channel-Information-RL-SetupRqstTDD" },
  { 276, "id-ClosedLoopMode1-SupportIndicator" },
  { 277, "id-Unused-ProtocolIE-ID-277" },
  { 279, "id-STTD-SupportIndicator" },
  {  14, "id-CFNReportingIndicator" },
  {  23, "id-CNOriginatedPage-PagingRqst" },
  {  24, "id-InnerLoopDLPCStatus" },
  {  25, "id-PropagationDelay" },
  {  36, "id-RxTimingDeviationForTA" },
  {  37, "id-timeSlot-ISCP" },
  {  15, "id-CCTrCH-InformationItem-RL-FailureInd" },
  {  16, "id-CCTrCH-InformationItem-RL-RestoreInd" },
  { 280, "id-CommonMeasurementAccuracy" },
  { 281, "id-CommonMeasurementObjectType-CM-Rprt" },
  { 282, "id-CommonMeasurementObjectType-CM-Rqst" },
  { 283, "id-CommonMeasurementObjectType-CM-Rsp" },
  { 284, "id-CommonMeasurementType" },
  {  18, "id-CongestionCause" },
  { 285, "id-SFN" },
  { 286, "id-SFNReportingIndicator" },
  { 287, "id-InformationExchangeID" },
  { 288, "id-InformationExchangeObjectType-InfEx-Rprt" },
  { 289, "id-InformationExchangeObjectType-InfEx-Rqst" },
  { 290, "id-InformationExchangeObjectType-InfEx-Rsp" },
  { 291, "id-InformationReportCharacteristics" },
  { 292, "id-InformationType" },
  {  58, "id-neighbouring-LCR-TDD-CellInformation" },
  {  65, "id-DL-Timeslot-ISCP-LCR-Information-RL-SetupRqstTDD" },
  {  66, "id-RL-LCR-InformationResponse-RL-SetupRspTDD" },
  {  75, "id-UL-CCTrCH-LCR-InformationListIE-RL-SetupRspTDD" },
  {  76, "id-UL-DPCH-LCR-InformationItem-RL-SetupRspTDD" },
  {  77, "id-DL-CCTrCH-LCR-InformationListIE-RL-SetupRspTDD" },
  {  78, "id-DL-DPCH-LCR-InformationItem-RL-SetupRspTDD" },
  {  79, "id-DSCH-LCR-InformationListIEs-RL-SetupRspTDD" },
  {  80, "id-USCH-LCR-InformationListIEs-RL-SetupRspTDD" },
  {  81, "id-DL-Timeslot-ISCP-LCR-Information-RL-AdditionRqstTDD" },
  {  86, "id-RL-LCR-InformationResponse-RL-AdditionRspTDD" },
  {  87, "id-UL-CCTrCH-LCR-InformationListIE-RL-AdditionRspTDD" },
  {  88, "id-UL-DPCH-LCR-InformationItem-RL-AdditionRspTDD" },
  {  89, "id-DL-CCTrCH-LCR-InformationListIE-RL-AdditionRspTDD" },
  {  94, "id-DL-DPCH-LCR-InformationItem-RL-AdditionRspTDD" },
  {  96, "id-DSCH-LCR-InformationListIEs-RL-AdditionRspTDD" },
  {  97, "id-USCH-LCR-InformationListIEs-RL-AdditionRspTDD" },
  {  98, "id-UL-DPCH-LCR-InformationAddListIE-RL-ReconfReadyTDD" },
  { 100, "id-UL-Timeslot-LCR-InformationModifyList-RL-ReconfReadyTDD" },
  { 101, "id-DL-DPCH-LCR-InformationAddListIE-RL-ReconfReadyTDD" },
  { 104, "id-DL-Timeslot-LCR-InformationModifyList-RL-ReconfReadyTDD" },
  { 105, "id-UL-Timeslot-LCR-InformationList-PhyChReconfRqstTDD" },
  { 106, "id-DL-Timeslot-LCR-InformationList-PhyChReconfRqstTDD" },
  { 138, "id-timeSlot-ISCP-LCR-List-DL-PC-Rqst-TDD" },
  { 139, "id-TSTD-Support-Indicator-RL-SetupRqstTDD" },
  { 142, "id-RestrictionStateIndicator" },
  { 233, "id-Load-Value" },
  { 234, "id-Load-Value-IncrDecrThres" },
  { 235, "id-OnModification" },
  { 236, "id-Received-Total-Wideband-Power-Value" },
  { 237, "id-Received-Total-Wideband-Power-Value-IncrDecrThres" },
  { 238, "id-SFNSFNMeasurementThresholdInformation" },
  { 239, "id-Transmitted-Carrier-Power-Value" },
  { 240, "id-Transmitted-Carrier-Power-Value-IncrDecrThres" },
  { 241, "id-TUTRANGPSMeasurementThresholdInformation" },
  { 242, "id-UL-Timeslot-ISCP-Value" },
  { 243, "id-UL-Timeslot-ISCP-Value-IncrDecrThres" },
  { 293, "id-Rx-Timing-Deviation-Value-LCR" },
  {  19, "id-DPC-Mode-Change-SupportIndicator" },
  { 247, "id-Unused-ProtocolIE-ID-247" },
  { 295, "id-Unused-ProtocolIE-ID-295" },
  { 202, "id-PrimaryCCPCH-RSCP-RL-ReconfPrepTDD" },
  { 203, "id-DL-TimeSlot-ISCP-Info-RL-ReconfPrepTDD" },
  { 204, "id-DL-Timeslot-ISCP-LCR-Information-RL-ReconfPrepTDD" },
  { 249, "id-DSCH-RNTI" },
  { 296, "id-DL-PowerBalancing-Information" },
  { 297, "id-DL-PowerBalancing-ActivationIndicator" },
  { 298, "id-DL-PowerBalancing-UpdatedIndicator" },
  { 299, "id-DL-ReferencePowerInformation" },
  { 224, "id-Enhanced-PrimaryCPICH-EcNo" },
  { 252, "id-IPDL-TDD-ParametersLCR" },
  { 300, "id-CellCapabilityContainer-FDD" },
  { 301, "id-CellCapabilityContainer-TDD" },
  { 302, "id-CellCapabilityContainer-TDD-LCR" },
  { 317, "id-RL-Specific-DCH-Info" },
  { 318, "id-RL-ReconfigurationRequestFDD-RL-InformationList" },
  { 319, "id-RL-ReconfigurationRequestFDD-RL-Information-IEs" },
  { 321, "id-RL-ReconfigurationRequestTDD-RL-Information" },
  { 250, "id-CommonTransportChannelResourcesInitialisationNotRequired" },
  { 312, "id-DelayedActivation" },
  { 313, "id-DelayedActivationList-RL-ActivationCmdFDD" },
  { 314, "id-DelayedActivationInformation-RL-ActivationCmdFDD" },
  { 315, "id-DelayedActivationList-RL-ActivationCmdTDD" },
  { 316, "id-DelayedActivationInformation-RL-ActivationCmdTDD" },
  { 251, "id-neighbouringTDDCellMeasurementInformationLCR" },
  { 150, "id-UL-SIR-Target-CCTrCH-InformationItem-RL-SetupRspTDD" },
  { 151, "id-UL-SIR-Target-CCTrCH-LCR-InformationItem-RL-SetupRspTDD" },
  { 451, "id-PrimCCPCH-RSCP-DL-PC-RqstTDD" },
  { 452, "id-HSDSCH-FDD-Information" },
  { 453, "id-HSDSCH-FDD-Information-Response" },
  { 466, "id-HSDSCH-FDD-Update-Information" },
  { 456, "id-HSDSCH-Information-to-Modify" },
  { 516, "id-HSDSCHMacdFlowSpecificInformationList-RL-PreemptRequiredInd" },
  { 517, "id-HSDSCHMacdFlowSpecificInformationItem-RL-PreemptRequiredInd" },
  { 457, "id-HSDSCH-RNTI" },
  { 458, "id-HSDSCH-TDD-Information" },
  { 459, "id-HSDSCH-TDD-Information-Response" },
  { 467, "id-HSDSCH-TDD-Update-Information" },
  { 463, "id-HSPDSCH-RL-ID" },
  { 531, "id-HSDSCH-MACdFlows-to-Add" },
  { 532, "id-HSDSCH-MACdFlows-to-Delete" },
  { 148, "id-Angle-Of-Arrival-Value-LCR" },
  { 158, "id-TrafficClass" },
  { 248, "id-Unused-ProtocolIE-ID-248" },
  { 253, "id-Unused-ProtocolIE-ID-253" },
  { 323, "id-PDSCH-RL-ID" },
  { 325, "id-TimeSlot-RL-SetupRspTDD" },
  { 468, "id-GERAN-Cell-Capability" },
  { 469, "id-GERAN-Classmark" },
  { 480, "id-DSCH-InitialWindowSize" },
  { 464, "id-UL-Synchronisation-Parameters-LCR" },
  { 479, "id-SNA-Information" },
  { 465, "id-MAChs-ResetIndicator" },
  { 481, "id-TDD-DL-DPCH-TimeSlotFormatModifyItem-LCR-RL-ReconfReadyTDD" },
  { 482, "id-TDD-UL-DPCH-TimeSlotFormatModifyItem-LCR-RL-ReconfReadyTDD" },
  { 483, "id-TDD-TPC-UplinkStepSize-LCR-RL-SetupRqstTDD" },
  { 484, "id-UL-CCTrCH-InformationList-RL-AdditionRqstTDD" },
  { 485, "id-UL-CCTrCH-InformationItem-RL-AdditionRqstTDD" },
  { 486, "id-DL-CCTrCH-InformationList-RL-AdditionRqstTDD" },
  { 487, "id-DL-CCTrCH-InformationItem-RL-AdditionRqstTDD" },
  { 488, "id-TDD-TPC-UplinkStepSize-InformationAdd-LCR-RL-ReconfPrepTDD" },
  { 489, "id-TDD-TPC-UplinkStepSize-InformationModify-LCR-RL-ReconfPrepTDD" },
  { 490, "id-TDD-TPC-DownlinkStepSize-InformationAdd-RL-ReconfPrepTDD" },
  { 491, "id-TDD-TPC-DownlinkStepSize-InformationModify-RL-ReconfPrepTDD" },
  { 492, "id-UL-TimingAdvanceCtrl-LCR" },
  { 493, "id-HSPDSCH-Timeslot-InformationList-PhyChReconfRqstTDD" },
  { 494, "id-HSPDSCH-Timeslot-InformationListLCR-PhyChReconfRqstTDD" },
  { 495, "id-HS-SICH-Reception-Quality" },
  { 496, "id-HS-SICH-Reception-Quality-Measurement-Value" },
  { 497, "id-HSSICH-Info-DM-Rprt" },
  { 498, "id-HSSICH-Info-DM-Rqst" },
  { 499, "id-HSSICH-Info-DM" },
  { 500, "id-CCTrCH-Maximum-DL-Power-RL-SetupRspTDD" },
  { 501, "id-CCTrCH-Minimum-DL-Power-RL-SetupRspTDD" },
  { 502, "id-CCTrCH-Maximum-DL-Power-RL-AdditionRspTDD" },
  { 503, "id-CCTrCH-Minimum-DL-Power-RL-AdditionRspTDD" },
  { 504, "id-CCTrCH-Maximum-DL-Power-RL-ReconfReadyTDD" },
  { 505, "id-CCTrCH-Minimum-DL-Power-RL-ReconfReadyTDD" },
  { 506, "id-Maximum-DL-Power-TimeslotLCR-InformationModifyItem-RL-ReconfReadyTDD" },
  { 507, "id-Minimum-DL-Power-TimeslotLCR-InformationModifyItem-RL-ReconfReadyTDD" },
  { 508, "id-DL-CCTrCH-InformationList-RL-ReconfRspTDD" },
  { 509, "id-DL-DPCH-InformationModifyItem-LCR-RL-ReconfRspTDD" },
  { 510, "id-Maximum-DL-Power-TimeslotLCR-InformationItem" },
  { 511, "id-Minimum-DL-Power-TimeslotLCR-InformationItem" },
  { 512, "id-TDD-Support-8PSK" },
  { 513, "id-TDD-maxNrDLPhysicalchannels" },
  { 514, "id-ExtendedGSMCellIndividualOffset" },
  { 518, "id-RL-ParameterUpdateIndicationFDD-RL-InformationList" },
  { 519, "id-Primary-CPICH-Usage-For-Channel-Estimation" },
  { 520, "id-Secondary-CPICH-Information" },
  { 521, "id-Secondary-CPICH-Information-Change" },
  { 522, "id-Unused-ProtocolIE-ID-522" },
  { 523, "id-Unused-ProtocolIE-ID-523" },
  { 524, "id-RL-ParameterUpdateIndicationFDD-RL-Information-Item" },
  { 525, "id-Phase-Reference-Update-Indicator" },
  { 526, "id-Unidirectional-DCH-Indicator" },
  { 527, "id-RL-Information-RL-ReconfPrepTDD" },
  { 528, "id-Multiple-RL-InformationResponse-RL-ReconfReadyTDD" },
  { 529, "id-RL-ReconfigurationResponseTDD-RL-Information" },
  { 530, "id-Satellite-Almanac-Information-ExtItem" },
  { 533, "id-HSDSCH-Information-to-Modify-Unsynchronised" },
  { 534, "id-TnlQos" },
  { 535, "id-RTLoadValue" },
  { 536, "id-NRTLoadInformationValue" },
  { 537, "id-CellPortionID" },
  { 538, "id-UpPTSInterferenceValue" },
  { 539, "id-PrimaryCCPCH-RSCP-Delta" },
  { 540, "id-UEMeasurementType" },
  { 541, "id-UEMeasurementTimeslotInfoHCR" },
  { 542, "id-UEMeasurementTimeslotInfoLCR" },
  { 543, "id-UEMeasurementReportCharacteristics" },
  { 544, "id-UEMeasurementParameterModAllow" },
  { 545, "id-UEMeasurementValueInformation" },
  { 546, "id-InterfacesToTraceItem" },
  { 547, "id-ListOfInterfacesToTrace" },
  { 548, "id-TraceDepth" },
  { 549, "id-TraceRecordingSessionReference" },
  { 550, "id-TraceReference" },
  { 551, "id-UEIdentity" },
  { 552, "id-NACC-Related-Data" },
  { 553, "id-GSM-Cell-InfEx-Rqst" },
  { 554, "id-MeasurementRecoveryBehavior" },
  { 555, "id-MeasurementRecoveryReportingIndicator" },
  { 556, "id-MeasurementRecoverySupportIndicator" },
  { 557, "id-DL-DPCH-Power-Information-RL-ReconfPrepFDD" },
  { 558, "id-F-DPCH-Information-RL-ReconfPrepFDD" },
  { 559, "id-F-DPCH-Information-RL-SetupRqstFDD" },
  { 560, "id-MBMS-Bearer-Service-List" },
  { 561, "id-MBMS-Bearer-Service-List-InfEx-Rsp" },
  { 562, "id-Active-MBMS-Bearer-ServiceFDD" },
  { 563, "id-Active-MBMS-Bearer-ServiceTDD" },
  { 564, "id-Old-URA-ID" },
  { 568, "id-UE-State" },
  { 569, "id-URA-ID" },
  { 571, "id-HARQ-Preamble-Mode" },
  { 572, "id-SynchronisationIndicator" },
  { 573, "id-UL-DPDCHIndicatorEDCH" },
  { 574, "id-EDPCH-Information" },
  { 575, "id-RL-Specific-EDCH-Information" },
  { 576, "id-EDCH-RL-Indication" },
  { 577, "id-EDCH-FDD-Information" },
  { 578, "id-EDCH-RLSet-Id" },
  { 579, "id-Serving-EDCHRL-Id" },
  { 580, "id-EDCH-FDD-DL-ControlChannelInformation" },
  { 581, "id-EDCH-FDD-InformationResponse" },
  { 582, "id-EDCH-MACdFlows-To-Add" },
  { 583, "id-EDCH-FDD-Information-To-Modify" },
  { 584, "id-EDCH-MACdFlows-To-Delete" },
  { 585, "id-EDPCH-Information-RLReconfRequest-FDD" },
  { 586, "id-EDCH-MacdFlowSpecificInformationList-RL-PreemptRequiredInd" },
  { 587, "id-EDCH-MacdFlowSpecificInformationItem-RL-PreemptRequiredInd" },
  { 588, "id-EDCH-MacdFlowSpecificInformationList-RL-CongestInd" },
  { 589, "id-EDCH-MacdFlowSpecificInformationItem-RL-CongestInd" },
  { 590, "id-MBMS-Bearer-Service-Full-Address" },
  { 591, "id-Initial-DL-DPCH-TimingAdjustment" },
  { 592, "id-Initial-DL-DPCH-TimingAdjustment-Allowed" },
  { 593, "id-User-Plane-Congestion-Fields-Inclusion" },
  { 594, "id-HARQ-Preamble-Mode-Activation-Indicator" },
  { 595, "id-multiple-DedicatedMeasurementValueList-TDD-DM-Rsp" },
  { 596, "id-multiple-DedicatedMeasurementValueList-LCR-TDD-DM-Rsp" },
  { 597, "id-ProvidedInformation" },
  { 598, "id-Active-MBMS-Bearer-ServiceFDD-PFL" },
  { 599, "id-Active-MBMS-Bearer-ServiceTDD-PFL" },
  { 600, "id-FrequencyBandIndicator" },
  { 0, NULL }
};


static int
dissect_rnsap_ProtocolIE_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 65535U, &ProtocolIE_ID, FALSE);

  return offset;
}
static int dissect_iE_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_ID(tvb, offset, actx, tree, hf_rnsap_iE_ID);
}
static int dissect_id(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_ID(tvb, offset, actx, tree, hf_rnsap_id);
}



static int
dissect_rnsap_ProtocolIEValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
#line 55 "rnsap.cnf"

	offset = dissect_rnsap_ProtocolIEValueValue(tvb, offset, actx, tree);



  return offset;
}
static int dissect_value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIEValue(tvb, offset, actx, tree, hf_rnsap_value);
}


static const per_sequence_t ProtocolIE_Field_sequence[] = {
  { &hf_rnsap_id            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_ID },
  { &hf_rnsap_criticality   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_Criticality },
  { &hf_rnsap_value         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIEValue },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_ProtocolIE_Field(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_ProtocolIE_Field, ProtocolIE_Field_sequence);

  return offset;
}
static int dissect_ProtocolIE_Container_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Field(tvb, offset, actx, tree, hf_rnsap_ProtocolIE_Container_item);
}


static const per_sequence_t ProtocolIE_Container_sequence_of[1] = {
  { &hf_rnsap_ProtocolIE_Container_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Field },
};

static int
dissect_rnsap_ProtocolIE_Container(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_ProtocolIE_Container, ProtocolIE_Container_sequence_of,
                                                  0, 65535);

  return offset;
}
static int dissect_protocolIEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Container(tvb, offset, actx, tree, hf_rnsap_protocolIEs);
}
static int dissect_ProtocolIE_ContainerList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Container(tvb, offset, actx, tree, hf_rnsap_ProtocolIE_ContainerList_item);
}



static int
dissect_rnsap_Extension(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
#line 31 "rnsap.cnf"

	offset = dissect_rnsap_ProtocolIEValueValue(tvb, offset, actx, tree);



  return offset;
}
static int dissect_extensionValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Extension(tvb, offset, actx, tree, hf_rnsap_extensionValue);
}


static const per_sequence_t ProtocolExtensionField_sequence[] = {
  { &hf_rnsap_id            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_ID },
  { &hf_rnsap_criticality   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_Criticality },
  { &hf_rnsap_extensionValue, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_Extension },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_ProtocolExtensionField(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_ProtocolExtensionField, ProtocolExtensionField_sequence);

  return offset;
}
static int dissect_ProtocolExtensionContainer_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolExtensionField(tvb, offset, actx, tree, hf_rnsap_ProtocolExtensionContainer_item);
}


static const per_sequence_t ProtocolExtensionContainer_sequence_of[1] = {
  { &hf_rnsap_ProtocolExtensionContainer_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolExtensionField },
};

static int
dissect_rnsap_ProtocolExtensionContainer(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_ProtocolExtensionContainer, ProtocolExtensionContainer_sequence_of,
                                                  1, 65535);

  return offset;
}
static int dissect_protocolExtensions(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolExtensionContainer(tvb, offset, actx, tree, hf_rnsap_protocolExtensions);
}
static int dissect_iE_Extensions(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolExtensionContainer(tvb, offset, actx, tree, hf_rnsap_iE_Extensions);
}
static int dissect_iEe_Extensions(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolExtensionContainer(tvb, offset, actx, tree, hf_rnsap_iEe_Extensions);
}


static const per_sequence_t CommonTransportChannelResourcesRequest_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_CommonTransportChannelResourcesRequest(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_CommonTransportChannelResourcesRequest, CommonTransportChannelResourcesRequest_sequence);

  return offset;
}
static int dissect_id_commonTransportChannelResourcesInitialisation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CommonTransportChannelResourcesRequest(tvb, offset, actx, tree, hf_rnsap_id_commonTransportChannelResourcesInitialisation);
}


static const per_sequence_t CommonTransportChannelResourcesReleaseRequest_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_CommonTransportChannelResourcesReleaseRequest(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_CommonTransportChannelResourcesReleaseRequest, CommonTransportChannelResourcesReleaseRequest_sequence);

  return offset;
}
static int dissect_id_commonTransportChannelResourcesRelease(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CommonTransportChannelResourcesReleaseRequest(tvb, offset, actx, tree, hf_rnsap_id_commonTransportChannelResourcesRelease);
}


static const per_sequence_t CompressedModeCommand_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_CompressedModeCommand(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_CompressedModeCommand, CompressedModeCommand_sequence);

  return offset;
}
static int dissect_id_compressedModeCommand(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CompressedModeCommand(tvb, offset, actx, tree, hf_rnsap_id_compressedModeCommand);
}


static const per_sequence_t DL_PowerControlRequest_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_PowerControlRequest(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_PowerControlRequest, DL_PowerControlRequest_sequence);

  return offset;
}
static int dissect_id_downlinkPowerControl(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_PowerControlRequest(tvb, offset, actx, tree, hf_rnsap_id_downlinkPowerControl);
}


static const per_sequence_t DL_PowerTimeslotControlRequest_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_PowerTimeslotControlRequest(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_PowerTimeslotControlRequest, DL_PowerTimeslotControlRequest_sequence);

  return offset;
}
static int dissect_id_downlinkPowerTimeslotControl(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_PowerTimeslotControlRequest(tvb, offset, actx, tree, hf_rnsap_id_downlinkPowerTimeslotControl);
}


static const per_sequence_t DownlinkSignallingTransferRequest_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DownlinkSignallingTransferRequest(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DownlinkSignallingTransferRequest, DownlinkSignallingTransferRequest_sequence);

  return offset;
}
static int dissect_id_downlinkSignallingTransfer(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DownlinkSignallingTransferRequest(tvb, offset, actx, tree, hf_rnsap_id_downlinkSignallingTransfer);
}


static const per_sequence_t ErrorIndication_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_ErrorIndication(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_ErrorIndication, ErrorIndication_sequence);

  return offset;
}
static int dissect_id_errorIndication(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ErrorIndication(tvb, offset, actx, tree, hf_rnsap_id_errorIndication);
}


static const per_sequence_t DedicatedMeasurementFailureIndication_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DedicatedMeasurementFailureIndication(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DedicatedMeasurementFailureIndication, DedicatedMeasurementFailureIndication_sequence);

  return offset;
}
static int dissect_id_dedicatedMeasurementFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DedicatedMeasurementFailureIndication(tvb, offset, actx, tree, hf_rnsap_id_dedicatedMeasurementFailure);
}


static const per_sequence_t DedicatedMeasurementInitiationRequest_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DedicatedMeasurementInitiationRequest(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DedicatedMeasurementInitiationRequest, DedicatedMeasurementInitiationRequest_sequence);

  return offset;
}
static int dissect_id_dedicatedMeasurementInitiation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DedicatedMeasurementInitiationRequest(tvb, offset, actx, tree, hf_rnsap_id_dedicatedMeasurementInitiation);
}


static const per_sequence_t DedicatedMeasurementReport_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DedicatedMeasurementReport(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DedicatedMeasurementReport, DedicatedMeasurementReport_sequence);

  return offset;
}
static int dissect_id_dedicatedMeasurementReporting(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DedicatedMeasurementReport(tvb, offset, actx, tree, hf_rnsap_id_dedicatedMeasurementReporting);
}


static const per_sequence_t DedicatedMeasurementTerminationRequest_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DedicatedMeasurementTerminationRequest(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DedicatedMeasurementTerminationRequest, DedicatedMeasurementTerminationRequest_sequence);

  return offset;
}
static int dissect_id_dedicatedMeasurementTermination(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DedicatedMeasurementTerminationRequest(tvb, offset, actx, tree, hf_rnsap_id_dedicatedMeasurementTermination);
}


static const per_sequence_t PagingRequest_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_PagingRequest(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_PagingRequest, PagingRequest_sequence);

  return offset;
}
static int dissect_id_paging(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PagingRequest(tvb, offset, actx, tree, hf_rnsap_id_paging);
}


static const per_sequence_t PhysicalChannelReconfigurationRequestTDD_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_PhysicalChannelReconfigurationRequestTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_PhysicalChannelReconfigurationRequestTDD, PhysicalChannelReconfigurationRequestTDD_sequence);

  return offset;
}
static int dissect_id_physicalChannelReconfiguration(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PhysicalChannelReconfigurationRequestTDD(tvb, offset, actx, tree, hf_rnsap_id_physicalChannelReconfiguration);
}



static int
dissect_rnsap_INTEGER_0_65535(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 65535U, NULL, FALSE);

  return offset;
}
static int dissect_innerRadius(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_0_65535(tvb, offset, actx, tree, hf_rnsap_innerRadius);
}
static int dissect_ctfc16bit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_0_65535(tvb, offset, actx, tree, hf_rnsap_ctfc16bit);
}
static int dissect_local(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_0_65535(tvb, offset, actx, tree, hf_rnsap_local);
}



static int
dissect_rnsap_OBJECT_IDENTIFIER(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_object_identifier(tvb, offset, actx, tree, hf_index, NULL);

  return offset;
}
static int dissect_global(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_OBJECT_IDENTIFIER(tvb, offset, actx, tree, hf_rnsap_global);
}


static const value_string rnsap_PrivateIE_ID_vals[] = {
  {   0, "local" },
  {   1, "global" },
  { 0, NULL }
};

static const per_choice_t PrivateIE_ID_choice[] = {
  {   0, &hf_rnsap_local         , ASN1_NO_EXTENSIONS     , dissect_rnsap_INTEGER_0_65535 },
  {   1, &hf_rnsap_global        , ASN1_NO_EXTENSIONS     , dissect_rnsap_OBJECT_IDENTIFIER },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_PrivateIE_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_PrivateIE_ID, PrivateIE_ID_choice,
                                 NULL);

  return offset;
}
static int dissect_privateIEid(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PrivateIE_ID(tvb, offset, actx, tree, hf_rnsap_privateIEid);
}



static int
dissect_rnsap_PrivateIEvalue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
#line 76 "rnsap.cnf"
	/* FIX ME */
	guint length;

	offset = dissect_per_length_determinant(tvb, offset, actx, tree, hf_rnsap_IE_length, &length);
	offset = offset + length;
	BYTE_ALIGN_OFFSET(offset)



  return offset;
}
static int dissect_privateIEvalue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PrivateIEvalue(tvb, offset, actx, tree, hf_rnsap_privateIEvalue);
}


static const per_sequence_t PrivateIE_Field_sequence[] = {
  { &hf_rnsap_privateIEid   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_PrivateIE_ID },
  { &hf_rnsap_criticality   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_Criticality },
  { &hf_rnsap_privateIEvalue, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_PrivateIEvalue },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_PrivateIE_Field(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_PrivateIE_Field, PrivateIE_Field_sequence);

  return offset;
}
static int dissect_PrivateIE_Container_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PrivateIE_Field(tvb, offset, actx, tree, hf_rnsap_PrivateIE_Container_item);
}


static const per_sequence_t PrivateIE_Container_sequence_of[1] = {
  { &hf_rnsap_PrivateIE_Container_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_PrivateIE_Field },
};

static int
dissect_rnsap_PrivateIE_Container(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_PrivateIE_Container, PrivateIE_Container_sequence_of,
                                                  1, 65535);

  return offset;
}
static int dissect_privateIEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PrivateIE_Container(tvb, offset, actx, tree, hf_rnsap_privateIEs);
}


static const per_sequence_t PrivateMessage_sequence[] = {
  { &hf_rnsap_privateIEs    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PrivateIE_Container },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_PrivateMessage(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_PrivateMessage, PrivateMessage_sequence);

  return offset;
}
static int dissect_id_privateMessage(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PrivateMessage(tvb, offset, actx, tree, hf_rnsap_id_privateMessage);
}


static const per_sequence_t RadioLinkAdditionRequestFDD_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RadioLinkAdditionRequestFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RadioLinkAdditionRequestFDD, RadioLinkAdditionRequestFDD_sequence);

  return offset;
}
static int dissect_id_radioLinkAddition(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RadioLinkAdditionRequestFDD(tvb, offset, actx, tree, hf_rnsap_id_radioLinkAddition);
}


static const per_sequence_t RadioLinkAdditionRequestTDD_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RadioLinkAdditionRequestTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RadioLinkAdditionRequestTDD, RadioLinkAdditionRequestTDD_sequence);

  return offset;
}
static int dissect_id_radioLinkAddition_TDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RadioLinkAdditionRequestTDD(tvb, offset, actx, tree, hf_rnsap_id_radioLinkAddition_TDD);
}


static const per_sequence_t RadioLinkCongestionIndication_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RadioLinkCongestionIndication(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RadioLinkCongestionIndication, RadioLinkCongestionIndication_sequence);

  return offset;
}
static int dissect_id_radioLinkCongestion(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RadioLinkCongestionIndication(tvb, offset, actx, tree, hf_rnsap_id_radioLinkCongestion);
}


static const per_sequence_t RadioLinkDeletionRequest_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RadioLinkDeletionRequest(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RadioLinkDeletionRequest, RadioLinkDeletionRequest_sequence);

  return offset;
}
static int dissect_id_radioLinkDeletion(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RadioLinkDeletionRequest(tvb, offset, actx, tree, hf_rnsap_id_radioLinkDeletion);
}


static const per_sequence_t RadioLinkFailureIndication_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RadioLinkFailureIndication(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RadioLinkFailureIndication, RadioLinkFailureIndication_sequence);

  return offset;
}
static int dissect_id_radioLinkFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RadioLinkFailureIndication(tvb, offset, actx, tree, hf_rnsap_id_radioLinkFailure);
}


static const per_sequence_t RadioLinkPreemptionRequiredIndication_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RadioLinkPreemptionRequiredIndication(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RadioLinkPreemptionRequiredIndication, RadioLinkPreemptionRequiredIndication_sequence);

  return offset;
}
static int dissect_id_radioLinkPreemption(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RadioLinkPreemptionRequiredIndication(tvb, offset, actx, tree, hf_rnsap_id_radioLinkPreemption);
}


static const per_sequence_t RadioLinkRestoreIndication_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RadioLinkRestoreIndication(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RadioLinkRestoreIndication, RadioLinkRestoreIndication_sequence);

  return offset;
}
static int dissect_id_radioLinkRestoration(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RadioLinkRestoreIndication(tvb, offset, actx, tree, hf_rnsap_id_radioLinkRestoration);
}


static const per_sequence_t RadioLinkSetupRequestFDD_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RadioLinkSetupRequestFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RadioLinkSetupRequestFDD, RadioLinkSetupRequestFDD_sequence);

  return offset;
}
static int dissect_id_radioLinkSetup(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RadioLinkSetupRequestFDD(tvb, offset, actx, tree, hf_rnsap_id_radioLinkSetup);
}


static const per_sequence_t RadioLinkSetupRequestTDD_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RadioLinkSetupRequestTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RadioLinkSetupRequestTDD, RadioLinkSetupRequestTDD_sequence);

  return offset;
}
static int dissect_id_radioLinkSetupTdd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RadioLinkSetupRequestTDD(tvb, offset, actx, tree, hf_rnsap_id_radioLinkSetupTdd);
}


static const per_sequence_t RelocationCommit_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RelocationCommit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RelocationCommit, RelocationCommit_sequence);

  return offset;
}
static int dissect_id_relocationCommit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RelocationCommit(tvb, offset, actx, tree, hf_rnsap_id_relocationCommit);
}


static const per_sequence_t RadioLinkReconfigurationCancel_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RadioLinkReconfigurationCancel(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RadioLinkReconfigurationCancel, RadioLinkReconfigurationCancel_sequence);

  return offset;
}
static int dissect_id_synchronisedRadioLinkReconfigurationCancellation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RadioLinkReconfigurationCancel(tvb, offset, actx, tree, hf_rnsap_id_synchronisedRadioLinkReconfigurationCancellation);
}


static const per_sequence_t RadioLinkReconfigurationCommit_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RadioLinkReconfigurationCommit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RadioLinkReconfigurationCommit, RadioLinkReconfigurationCommit_sequence);

  return offset;
}
static int dissect_id_synchronisedRadioLinkReconfigurationCommit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RadioLinkReconfigurationCommit(tvb, offset, actx, tree, hf_rnsap_id_synchronisedRadioLinkReconfigurationCommit);
}


static const per_sequence_t RadioLinkReconfigurationPrepareFDD_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RadioLinkReconfigurationPrepareFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RadioLinkReconfigurationPrepareFDD, RadioLinkReconfigurationPrepareFDD_sequence);

  return offset;
}
static int dissect_id_synchronisedRadioLinkReconfigurationPreparation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RadioLinkReconfigurationPrepareFDD(tvb, offset, actx, tree, hf_rnsap_id_synchronisedRadioLinkReconfigurationPreparation);
}


static const per_sequence_t RadioLinkReconfigurationReadyTDD_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RadioLinkReconfigurationReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RadioLinkReconfigurationReadyTDD, RadioLinkReconfigurationReadyTDD_sequence);

  return offset;
}
static int dissect_id_synchronisedRadioLinkReconfigurationPreparation_TDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RadioLinkReconfigurationReadyTDD(tvb, offset, actx, tree, hf_rnsap_id_synchronisedRadioLinkReconfigurationPreparation_TDD);
}


static const per_sequence_t RadioLinkReconfigurationRequestFDD_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RadioLinkReconfigurationRequestFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RadioLinkReconfigurationRequestFDD, RadioLinkReconfigurationRequestFDD_sequence);

  return offset;
}
static int dissect_id_unSynchronisedRadioLinkReconfiguration(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RadioLinkReconfigurationRequestFDD(tvb, offset, actx, tree, hf_rnsap_id_unSynchronisedRadioLinkReconfiguration);
}


static const per_sequence_t RadioLinkReconfigurationRequestTDD_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RadioLinkReconfigurationRequestTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RadioLinkReconfigurationRequestTDD, RadioLinkReconfigurationRequestTDD_sequence);

  return offset;
}
static int dissect_id_unSynchronisedRadioLinkReconfiguration_TDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RadioLinkReconfigurationRequestTDD(tvb, offset, actx, tree, hf_rnsap_id_unSynchronisedRadioLinkReconfiguration_TDD);
}


static const per_sequence_t UplinkSignallingTransferIndicationFDD_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UplinkSignallingTransferIndicationFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UplinkSignallingTransferIndicationFDD, UplinkSignallingTransferIndicationFDD_sequence);

  return offset;
}
static int dissect_id_uplinkSignallingTransfer(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UplinkSignallingTransferIndicationFDD(tvb, offset, actx, tree, hf_rnsap_id_uplinkSignallingTransfer);
}


static const per_sequence_t UplinkSignallingTransferIndicationTDD_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UplinkSignallingTransferIndicationTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UplinkSignallingTransferIndicationTDD, UplinkSignallingTransferIndicationTDD_sequence);

  return offset;
}
static int dissect_id_uplinkSignallingTransfer_TDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UplinkSignallingTransferIndicationTDD(tvb, offset, actx, tree, hf_rnsap_id_uplinkSignallingTransfer_TDD);
}


static const per_sequence_t CommonMeasurementFailureIndication_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_CommonMeasurementFailureIndication(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_CommonMeasurementFailureIndication, CommonMeasurementFailureIndication_sequence);

  return offset;
}
static int dissect_id_commonMeasurementFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CommonMeasurementFailureIndication(tvb, offset, actx, tree, hf_rnsap_id_commonMeasurementFailure);
}


static const per_sequence_t CommonMeasurementInitiationRequest_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_CommonMeasurementInitiationRequest(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_CommonMeasurementInitiationRequest, CommonMeasurementInitiationRequest_sequence);

  return offset;
}
static int dissect_id_commonMeasurementInitiation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CommonMeasurementInitiationRequest(tvb, offset, actx, tree, hf_rnsap_id_commonMeasurementInitiation);
}


static const per_sequence_t CommonMeasurementReport_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_CommonMeasurementReport(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_CommonMeasurementReport, CommonMeasurementReport_sequence);

  return offset;
}
static int dissect_id_commonMeasurementReporting(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CommonMeasurementReport(tvb, offset, actx, tree, hf_rnsap_id_commonMeasurementReporting);
}


static const per_sequence_t CommonMeasurementTerminationRequest_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_CommonMeasurementTerminationRequest(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_CommonMeasurementTerminationRequest, CommonMeasurementTerminationRequest_sequence);

  return offset;
}
static int dissect_id_commonMeasurementTermination(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CommonMeasurementTerminationRequest(tvb, offset, actx, tree, hf_rnsap_id_commonMeasurementTermination);
}


static const per_sequence_t InformationExchangeFailureIndication_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_InformationExchangeFailureIndication(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_InformationExchangeFailureIndication, InformationExchangeFailureIndication_sequence);

  return offset;
}
static int dissect_id_informationExchangeFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_InformationExchangeFailureIndication(tvb, offset, actx, tree, hf_rnsap_id_informationExchangeFailure);
}


static const per_sequence_t InformationExchangeInitiationRequest_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_InformationExchangeInitiationRequest(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_InformationExchangeInitiationRequest, InformationExchangeInitiationRequest_sequence);

  return offset;
}
static int dissect_id_informationExchangeInitiation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_InformationExchangeInitiationRequest(tvb, offset, actx, tree, hf_rnsap_id_informationExchangeInitiation);
}


static const per_sequence_t InformationReport_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_InformationReport(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_InformationReport, InformationReport_sequence);

  return offset;
}
static int dissect_id_informationReporting(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_InformationReport(tvb, offset, actx, tree, hf_rnsap_id_informationReporting);
}


static const per_sequence_t InformationExchangeTerminationRequest_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_InformationExchangeTerminationRequest(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_InformationExchangeTerminationRequest, InformationExchangeTerminationRequest_sequence);

  return offset;
}
static int dissect_id_informationExchangeTermination(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_InformationExchangeTerminationRequest(tvb, offset, actx, tree, hf_rnsap_id_informationExchangeTermination);
}


static const per_sequence_t ResetRequest_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_ResetRequest(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_ResetRequest, ResetRequest_sequence);

  return offset;
}
static int dissect_id_reset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ResetRequest(tvb, offset, actx, tree, hf_rnsap_id_reset);
}


static const per_sequence_t RadioLinkActivationCommandFDD_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RadioLinkActivationCommandFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RadioLinkActivationCommandFDD, RadioLinkActivationCommandFDD_sequence);

  return offset;
}
static int dissect_id_radioLinkActivation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RadioLinkActivationCommandFDD(tvb, offset, actx, tree, hf_rnsap_id_radioLinkActivation);
}


static const per_sequence_t RadioLinkActivationCommandTDD_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RadioLinkActivationCommandTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RadioLinkActivationCommandTDD, RadioLinkActivationCommandTDD_sequence);

  return offset;
}
static int dissect_id_radioLinkActivation_TDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RadioLinkActivationCommandTDD(tvb, offset, actx, tree, hf_rnsap_id_radioLinkActivation_TDD);
}


static const per_sequence_t GERANUplinkSignallingTransferIndication_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_GERANUplinkSignallingTransferIndication(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_GERANUplinkSignallingTransferIndication, GERANUplinkSignallingTransferIndication_sequence);

  return offset;
}
static int dissect_id_gERANuplinkSignallingTransfer(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GERANUplinkSignallingTransferIndication(tvb, offset, actx, tree, hf_rnsap_id_gERANuplinkSignallingTransfer);
}


static const per_sequence_t RadioLinkParameterUpdateIndicationFDD_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RadioLinkParameterUpdateIndicationFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RadioLinkParameterUpdateIndicationFDD, RadioLinkParameterUpdateIndicationFDD_sequence);

  return offset;
}
static int dissect_id_radioLinkParameterUpdate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RadioLinkParameterUpdateIndicationFDD(tvb, offset, actx, tree, hf_rnsap_id_radioLinkParameterUpdate);
}


static const per_sequence_t RadioLinkParameterUpdateIndicationTDD_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RadioLinkParameterUpdateIndicationTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RadioLinkParameterUpdateIndicationTDD, RadioLinkParameterUpdateIndicationTDD_sequence);

  return offset;
}
static int dissect_id_radioLinkParameterUpdate_TDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RadioLinkParameterUpdateIndicationTDD(tvb, offset, actx, tree, hf_rnsap_id_radioLinkParameterUpdate_TDD);
}


static const per_sequence_t UEMeasurementFailureIndication_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UEMeasurementFailureIndication(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UEMeasurementFailureIndication, UEMeasurementFailureIndication_sequence);

  return offset;
}
static int dissect_id_uEMeasurementFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEMeasurementFailureIndication(tvb, offset, actx, tree, hf_rnsap_id_uEMeasurementFailure);
}


static const per_sequence_t UEMeasurementInitiationRequest_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UEMeasurementInitiationRequest(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UEMeasurementInitiationRequest, UEMeasurementInitiationRequest_sequence);

  return offset;
}
static int dissect_id_uEMeasurementInitiation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEMeasurementInitiationRequest(tvb, offset, actx, tree, hf_rnsap_id_uEMeasurementInitiation);
}


static const per_sequence_t UEMeasurementReport_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UEMeasurementReport(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UEMeasurementReport, UEMeasurementReport_sequence);

  return offset;
}
static int dissect_id_uEMeasurementReporting(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEMeasurementReport(tvb, offset, actx, tree, hf_rnsap_id_uEMeasurementReporting);
}


static const per_sequence_t UEMeasurementTerminationRequest_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UEMeasurementTerminationRequest(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UEMeasurementTerminationRequest, UEMeasurementTerminationRequest_sequence);

  return offset;
}
static int dissect_id_uEMeasurementTermination(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEMeasurementTerminationRequest(tvb, offset, actx, tree, hf_rnsap_id_uEMeasurementTermination);
}


static const per_sequence_t IurDeactivateTrace_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_IurDeactivateTrace(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_IurDeactivateTrace, IurDeactivateTrace_sequence);

  return offset;
}
static int dissect_id_iurDeactivateTrace(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_IurDeactivateTrace(tvb, offset, actx, tree, hf_rnsap_id_iurDeactivateTrace);
}


static const per_sequence_t IurInvokeTrace_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_IurInvokeTrace(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_IurInvokeTrace, IurInvokeTrace_sequence);

  return offset;
}
static int dissect_id_iurInvokeTrace(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_IurInvokeTrace(tvb, offset, actx, tree, hf_rnsap_id_iurInvokeTrace);
}


static const per_sequence_t MBMSAttachCommand_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_MBMSAttachCommand(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_MBMSAttachCommand, MBMSAttachCommand_sequence);

  return offset;
}
static int dissect_id_mBMSAttach(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MBMSAttachCommand(tvb, offset, actx, tree, hf_rnsap_id_mBMSAttach);
}


static const per_sequence_t MBMSDetachCommand_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_MBMSDetachCommand(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_MBMSDetachCommand, MBMSDetachCommand_sequence);

  return offset;
}
static int dissect_id_mBMSDetach(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MBMSDetachCommand(tvb, offset, actx, tree, hf_rnsap_id_mBMSDetach);
}


static const per_sequence_t DirectInformationTransfer_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DirectInformationTransfer(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DirectInformationTransfer, DirectInformationTransfer_sequence);

  return offset;
}
static int dissect_id_directInformationTransfer(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DirectInformationTransfer(tvb, offset, actx, tree, hf_rnsap_id_directInformationTransfer);
}


static const value_string rnsap_DummyInitiatingValue_vals[] = {
  {   0, "id-commonTransportChannelResourcesInitialisation" },
  {   1, "id-commonTransportChannelResourcesRelease" },
  {   2, "id-compressedModeCommand" },
  {   3, "id-downlinkPowerControl" },
  {   4, "id-downlinkPowerTimeslotControl" },
  {   5, "id-downlinkSignallingTransfer" },
  {   6, "id-errorIndication" },
  {   7, "id-dedicatedMeasurementFailure" },
  {   8, "id-dedicatedMeasurementInitiation" },
  {   9, "id-dedicatedMeasurementReporting" },
  {  10, "id-dedicatedMeasurementTermination" },
  {  11, "id-paging" },
  {  12, "id-physicalChannelReconfiguration" },
  {  13, "id-privateMessage" },
  {  14, "id-radioLinkAddition" },
  {  15, "id-radioLinkAddition-TDD" },
  {  16, "id-radioLinkCongestion" },
  {  17, "id-radioLinkDeletion" },
  {  18, "id-radioLinkFailure" },
  {  19, "id-radioLinkPreemption" },
  {  20, "id-radioLinkRestoration" },
  {  21, "id-radioLinkSetup" },
  {  22, "id-radioLinkSetupTdd" },
  {  23, "id-relocationCommit" },
  {  24, "id-synchronisedRadioLinkReconfigurationCancellation" },
  {  25, "id-synchronisedRadioLinkReconfigurationCommit" },
  {  26, "id-synchronisedRadioLinkReconfigurationPreparation" },
  {  27, "id-synchronisedRadioLinkReconfigurationPreparation-TDD" },
  {  28, "id-unSynchronisedRadioLinkReconfiguration" },
  {  29, "id-unSynchronisedRadioLinkReconfiguration-TDD" },
  {  30, "id-uplinkSignallingTransfer" },
  {  31, "id-uplinkSignallingTransfer-TDD" },
  {  32, "id-commonMeasurementFailure" },
  {  33, "id-commonMeasurementInitiation" },
  {  34, "id-commonMeasurementReporting" },
  {  35, "id-commonMeasurementTermination" },
  {  36, "id-informationExchangeFailure" },
  {  37, "id-informationExchangeInitiation" },
  {  38, "id-informationReporting" },
  {  39, "id-informationExchangeTermination" },
  {  40, "id-reset" },
  {  41, "id-radioLinkActivation" },
  {  42, "id-radioLinkActivation-TDD" },
  {  43, "id-gERANuplinkSignallingTransfer" },
  {  44, "id-radioLinkParameterUpdate" },
  {  45, "id-radioLinkParameterUpdate-TDD" },
  {  46, "id-uEMeasurementFailure" },
  {  47, "id-uEMeasurementInitiation" },
  {  48, "id-uEMeasurementReporting" },
  {  49, "id-uEMeasurementTermination" },
  {  50, "id-iurDeactivateTrace" },
  {  51, "id-iurInvokeTrace" },
  {  52, "id-mBMSAttach" },
  {  53, "id-mBMSDetach" },
  {  54, "id-directInformationTransfer" },
  { 0, NULL }
};

static const per_choice_t DummyInitiatingValue_choice[] = {
  {   0, &hf_rnsap_id_commonTransportChannelResourcesInitialisation, ASN1_EXTENSION_ROOT    , dissect_rnsap_CommonTransportChannelResourcesRequest },
  {   1, &hf_rnsap_id_commonTransportChannelResourcesRelease, ASN1_EXTENSION_ROOT    , dissect_rnsap_CommonTransportChannelResourcesReleaseRequest },
  {   2, &hf_rnsap_id_compressedModeCommand, ASN1_EXTENSION_ROOT    , dissect_rnsap_CompressedModeCommand },
  {   3, &hf_rnsap_id_downlinkPowerControl, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_PowerControlRequest },
  {   4, &hf_rnsap_id_downlinkPowerTimeslotControl, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_PowerTimeslotControlRequest },
  {   5, &hf_rnsap_id_downlinkSignallingTransfer, ASN1_EXTENSION_ROOT    , dissect_rnsap_DownlinkSignallingTransferRequest },
  {   6, &hf_rnsap_id_errorIndication, ASN1_EXTENSION_ROOT    , dissect_rnsap_ErrorIndication },
  {   7, &hf_rnsap_id_dedicatedMeasurementFailure, ASN1_EXTENSION_ROOT    , dissect_rnsap_DedicatedMeasurementFailureIndication },
  {   8, &hf_rnsap_id_dedicatedMeasurementInitiation, ASN1_EXTENSION_ROOT    , dissect_rnsap_DedicatedMeasurementInitiationRequest },
  {   9, &hf_rnsap_id_dedicatedMeasurementReporting, ASN1_EXTENSION_ROOT    , dissect_rnsap_DedicatedMeasurementReport },
  {  10, &hf_rnsap_id_dedicatedMeasurementTermination, ASN1_EXTENSION_ROOT    , dissect_rnsap_DedicatedMeasurementTerminationRequest },
  {  11, &hf_rnsap_id_paging     , ASN1_EXTENSION_ROOT    , dissect_rnsap_PagingRequest },
  {  12, &hf_rnsap_id_physicalChannelReconfiguration, ASN1_EXTENSION_ROOT    , dissect_rnsap_PhysicalChannelReconfigurationRequestTDD },
  {  13, &hf_rnsap_id_privateMessage, ASN1_EXTENSION_ROOT    , dissect_rnsap_PrivateMessage },
  {  14, &hf_rnsap_id_radioLinkAddition, ASN1_EXTENSION_ROOT    , dissect_rnsap_RadioLinkAdditionRequestFDD },
  {  15, &hf_rnsap_id_radioLinkAddition_TDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_RadioLinkAdditionRequestTDD },
  {  16, &hf_rnsap_id_radioLinkCongestion, ASN1_EXTENSION_ROOT    , dissect_rnsap_RadioLinkCongestionIndication },
  {  17, &hf_rnsap_id_radioLinkDeletion, ASN1_EXTENSION_ROOT    , dissect_rnsap_RadioLinkDeletionRequest },
  {  18, &hf_rnsap_id_radioLinkFailure, ASN1_EXTENSION_ROOT    , dissect_rnsap_RadioLinkFailureIndication },
  {  19, &hf_rnsap_id_radioLinkPreemption, ASN1_EXTENSION_ROOT    , dissect_rnsap_RadioLinkPreemptionRequiredIndication },
  {  20, &hf_rnsap_id_radioLinkRestoration, ASN1_EXTENSION_ROOT    , dissect_rnsap_RadioLinkRestoreIndication },
  {  21, &hf_rnsap_id_radioLinkSetup, ASN1_EXTENSION_ROOT    , dissect_rnsap_RadioLinkSetupRequestFDD },
  {  22, &hf_rnsap_id_radioLinkSetupTdd, ASN1_EXTENSION_ROOT    , dissect_rnsap_RadioLinkSetupRequestTDD },
  {  23, &hf_rnsap_id_relocationCommit, ASN1_EXTENSION_ROOT    , dissect_rnsap_RelocationCommit },
  {  24, &hf_rnsap_id_synchronisedRadioLinkReconfigurationCancellation, ASN1_EXTENSION_ROOT    , dissect_rnsap_RadioLinkReconfigurationCancel },
  {  25, &hf_rnsap_id_synchronisedRadioLinkReconfigurationCommit, ASN1_EXTENSION_ROOT    , dissect_rnsap_RadioLinkReconfigurationCommit },
  {  26, &hf_rnsap_id_synchronisedRadioLinkReconfigurationPreparation, ASN1_EXTENSION_ROOT    , dissect_rnsap_RadioLinkReconfigurationPrepareFDD },
  {  27, &hf_rnsap_id_synchronisedRadioLinkReconfigurationPreparation_TDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_RadioLinkReconfigurationReadyTDD },
  {  28, &hf_rnsap_id_unSynchronisedRadioLinkReconfiguration, ASN1_EXTENSION_ROOT    , dissect_rnsap_RadioLinkReconfigurationRequestFDD },
  {  29, &hf_rnsap_id_unSynchronisedRadioLinkReconfiguration_TDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_RadioLinkReconfigurationRequestTDD },
  {  30, &hf_rnsap_id_uplinkSignallingTransfer, ASN1_EXTENSION_ROOT    , dissect_rnsap_UplinkSignallingTransferIndicationFDD },
  {  31, &hf_rnsap_id_uplinkSignallingTransfer_TDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_UplinkSignallingTransferIndicationTDD },
  {  32, &hf_rnsap_id_commonMeasurementFailure, ASN1_EXTENSION_ROOT    , dissect_rnsap_CommonMeasurementFailureIndication },
  {  33, &hf_rnsap_id_commonMeasurementInitiation, ASN1_EXTENSION_ROOT    , dissect_rnsap_CommonMeasurementInitiationRequest },
  {  34, &hf_rnsap_id_commonMeasurementReporting, ASN1_EXTENSION_ROOT    , dissect_rnsap_CommonMeasurementReport },
  {  35, &hf_rnsap_id_commonMeasurementTermination, ASN1_EXTENSION_ROOT    , dissect_rnsap_CommonMeasurementTerminationRequest },
  {  36, &hf_rnsap_id_informationExchangeFailure, ASN1_EXTENSION_ROOT    , dissect_rnsap_InformationExchangeFailureIndication },
  {  37, &hf_rnsap_id_informationExchangeInitiation, ASN1_EXTENSION_ROOT    , dissect_rnsap_InformationExchangeInitiationRequest },
  {  38, &hf_rnsap_id_informationReporting, ASN1_EXTENSION_ROOT    , dissect_rnsap_InformationReport },
  {  39, &hf_rnsap_id_informationExchangeTermination, ASN1_EXTENSION_ROOT    , dissect_rnsap_InformationExchangeTerminationRequest },
  {  40, &hf_rnsap_id_reset      , ASN1_EXTENSION_ROOT    , dissect_rnsap_ResetRequest },
  {  41, &hf_rnsap_id_radioLinkActivation, ASN1_EXTENSION_ROOT    , dissect_rnsap_RadioLinkActivationCommandFDD },
  {  42, &hf_rnsap_id_radioLinkActivation_TDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_RadioLinkActivationCommandTDD },
  {  43, &hf_rnsap_id_gERANuplinkSignallingTransfer, ASN1_EXTENSION_ROOT    , dissect_rnsap_GERANUplinkSignallingTransferIndication },
  {  44, &hf_rnsap_id_radioLinkParameterUpdate, ASN1_EXTENSION_ROOT    , dissect_rnsap_RadioLinkParameterUpdateIndicationFDD },
  {  45, &hf_rnsap_id_radioLinkParameterUpdate_TDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_RadioLinkParameterUpdateIndicationTDD },
  {  46, &hf_rnsap_id_uEMeasurementFailure, ASN1_EXTENSION_ROOT    , dissect_rnsap_UEMeasurementFailureIndication },
  {  47, &hf_rnsap_id_uEMeasurementInitiation, ASN1_EXTENSION_ROOT    , dissect_rnsap_UEMeasurementInitiationRequest },
  {  48, &hf_rnsap_id_uEMeasurementReporting, ASN1_EXTENSION_ROOT    , dissect_rnsap_UEMeasurementReport },
  {  49, &hf_rnsap_id_uEMeasurementTermination, ASN1_EXTENSION_ROOT    , dissect_rnsap_UEMeasurementTerminationRequest },
  {  50, &hf_rnsap_id_iurDeactivateTrace, ASN1_EXTENSION_ROOT    , dissect_rnsap_IurDeactivateTrace },
  {  51, &hf_rnsap_id_iurInvokeTrace, ASN1_EXTENSION_ROOT    , dissect_rnsap_IurInvokeTrace },
  {  52, &hf_rnsap_id_mBMSAttach , ASN1_EXTENSION_ROOT    , dissect_rnsap_MBMSAttachCommand },
  {  53, &hf_rnsap_id_mBMSDetach , ASN1_EXTENSION_ROOT    , dissect_rnsap_MBMSDetachCommand },
  {  54, &hf_rnsap_id_directInformationTransfer, ASN1_EXTENSION_ROOT    , dissect_rnsap_DirectInformationTransfer },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_DummyInitiatingValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_DummyInitiatingValue, DummyInitiatingValue_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t CommonTransportChannelResourcesResponseFDD_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_CommonTransportChannelResourcesResponseFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_CommonTransportChannelResourcesResponseFDD, CommonTransportChannelResourcesResponseFDD_sequence);

  return offset;
}
static int dissect_id_commonTransportChannelResourcesInitialisation1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CommonTransportChannelResourcesResponseFDD(tvb, offset, actx, tree, hf_rnsap_id_commonTransportChannelResourcesInitialisation1);
}


static const per_sequence_t CommonTransportChannelResourcesResponseTDD_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_CommonTransportChannelResourcesResponseTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_CommonTransportChannelResourcesResponseTDD, CommonTransportChannelResourcesResponseTDD_sequence);

  return offset;
}
static int dissect_id_commonTransportChannelResourcesInitialisation_TDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CommonTransportChannelResourcesResponseTDD(tvb, offset, actx, tree, hf_rnsap_id_commonTransportChannelResourcesInitialisation_TDD);
}


static const per_sequence_t DedicatedMeasurementInitiationResponse_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DedicatedMeasurementInitiationResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DedicatedMeasurementInitiationResponse, DedicatedMeasurementInitiationResponse_sequence);

  return offset;
}
static int dissect_id_dedicatedMeasurementInitiation1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DedicatedMeasurementInitiationResponse(tvb, offset, actx, tree, hf_rnsap_id_dedicatedMeasurementInitiation1);
}


static const per_sequence_t PhysicalChannelReconfigurationCommand_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_PhysicalChannelReconfigurationCommand(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_PhysicalChannelReconfigurationCommand, PhysicalChannelReconfigurationCommand_sequence);

  return offset;
}
static int dissect_id_physicalChannelReconfiguration1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PhysicalChannelReconfigurationCommand(tvb, offset, actx, tree, hf_rnsap_id_physicalChannelReconfiguration1);
}


static const per_sequence_t RadioLinkAdditionResponseFDD_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RadioLinkAdditionResponseFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RadioLinkAdditionResponseFDD, RadioLinkAdditionResponseFDD_sequence);

  return offset;
}
static int dissect_id_radioLinkAddition1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RadioLinkAdditionResponseFDD(tvb, offset, actx, tree, hf_rnsap_id_radioLinkAddition1);
}


static const per_sequence_t RadioLinkAdditionResponseTDD_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RadioLinkAdditionResponseTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RadioLinkAdditionResponseTDD, RadioLinkAdditionResponseTDD_sequence);

  return offset;
}
static int dissect_id_radioLinkAddition_TDD1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RadioLinkAdditionResponseTDD(tvb, offset, actx, tree, hf_rnsap_id_radioLinkAddition_TDD1);
}


static const per_sequence_t RadioLinkDeletionResponse_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RadioLinkDeletionResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RadioLinkDeletionResponse, RadioLinkDeletionResponse_sequence);

  return offset;
}
static int dissect_id_radioLinkDeletion1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RadioLinkDeletionResponse(tvb, offset, actx, tree, hf_rnsap_id_radioLinkDeletion1);
}


static const per_sequence_t RadioLinkSetupResponseFDD_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RadioLinkSetupResponseFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RadioLinkSetupResponseFDD, RadioLinkSetupResponseFDD_sequence);

  return offset;
}
static int dissect_id_radioLinkSetup1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RadioLinkSetupResponseFDD(tvb, offset, actx, tree, hf_rnsap_id_radioLinkSetup1);
}


static const per_sequence_t RadioLinkSetupResponseTDD_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RadioLinkSetupResponseTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RadioLinkSetupResponseTDD, RadioLinkSetupResponseTDD_sequence);

  return offset;
}
static int dissect_id_radioLinkSetupTdd1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RadioLinkSetupResponseTDD(tvb, offset, actx, tree, hf_rnsap_id_radioLinkSetupTdd1);
}


static const per_sequence_t RadioLinkReconfigurationReadyFDD_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RadioLinkReconfigurationReadyFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RadioLinkReconfigurationReadyFDD, RadioLinkReconfigurationReadyFDD_sequence);

  return offset;
}
static int dissect_id_synchronisedRadioLinkReconfigurationPreparation1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RadioLinkReconfigurationReadyFDD(tvb, offset, actx, tree, hf_rnsap_id_synchronisedRadioLinkReconfigurationPreparation1);
}


static const per_sequence_t RadioLinkReconfigurationResponseFDD_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RadioLinkReconfigurationResponseFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RadioLinkReconfigurationResponseFDD, RadioLinkReconfigurationResponseFDD_sequence);

  return offset;
}
static int dissect_id_unSynchronisedRadioLinkReconfiguration1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RadioLinkReconfigurationResponseFDD(tvb, offset, actx, tree, hf_rnsap_id_unSynchronisedRadioLinkReconfiguration1);
}


static const per_sequence_t RadioLinkReconfigurationResponseTDD_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RadioLinkReconfigurationResponseTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RadioLinkReconfigurationResponseTDD, RadioLinkReconfigurationResponseTDD_sequence);

  return offset;
}
static int dissect_id_unSynchronisedRadioLinkReconfiguration_TDD1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RadioLinkReconfigurationResponseTDD(tvb, offset, actx, tree, hf_rnsap_id_unSynchronisedRadioLinkReconfiguration_TDD1);
}


static const per_sequence_t CommonMeasurementInitiationResponse_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_CommonMeasurementInitiationResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_CommonMeasurementInitiationResponse, CommonMeasurementInitiationResponse_sequence);

  return offset;
}
static int dissect_id_commonMeasurementInitiation1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CommonMeasurementInitiationResponse(tvb, offset, actx, tree, hf_rnsap_id_commonMeasurementInitiation1);
}


static const per_sequence_t InformationExchangeInitiationResponse_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_InformationExchangeInitiationResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_InformationExchangeInitiationResponse, InformationExchangeInitiationResponse_sequence);

  return offset;
}
static int dissect_id_informationExchangeInitiation1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_InformationExchangeInitiationResponse(tvb, offset, actx, tree, hf_rnsap_id_informationExchangeInitiation1);
}


static const per_sequence_t ResetResponse_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_ResetResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_ResetResponse, ResetResponse_sequence);

  return offset;
}
static int dissect_id_reset1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ResetResponse(tvb, offset, actx, tree, hf_rnsap_id_reset1);
}


static const per_sequence_t UEMeasurementInitiationResponse_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UEMeasurementInitiationResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UEMeasurementInitiationResponse, UEMeasurementInitiationResponse_sequence);

  return offset;
}
static int dissect_id_uEMeasurementInitiation1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEMeasurementInitiationResponse(tvb, offset, actx, tree, hf_rnsap_id_uEMeasurementInitiation1);
}


static const value_string rnsap_DummySuccessfulOutcomeValue_vals[] = {
  {   0, "id-commonTransportChannelResourcesInitialisation" },
  {   1, "id-commonTransportChannelResourcesInitialisation-TDD" },
  {   2, "id-dedicatedMeasurementInitiation" },
  {   3, "id-physicalChannelReconfiguration" },
  {   4, "id-radioLinkAddition" },
  {   5, "id-radioLinkAddition-TDD" },
  {   6, "id-radioLinkDeletion" },
  {   7, "id-radioLinkSetup" },
  {   8, "id-radioLinkSetupTdd" },
  {   9, "id-synchronisedRadioLinkReconfigurationPreparation" },
  {  10, "id-synchronisedRadioLinkReconfigurationPreparation-TDD" },
  {  11, "id-unSynchronisedRadioLinkReconfiguration" },
  {  12, "id-unSynchronisedRadioLinkReconfiguration-TDD" },
  {  13, "id-commonMeasurementInitiation" },
  {  14, "id-informationExchangeInitiation" },
  {  15, "id-reset" },
  {  16, "id-uEMeasurementInitiation" },
  { 0, NULL }
};

static const per_choice_t DummySuccessfulOutcomeValue_choice[] = {
  {   0, &hf_rnsap_id_commonTransportChannelResourcesInitialisation1, ASN1_EXTENSION_ROOT    , dissect_rnsap_CommonTransportChannelResourcesResponseFDD },
  {   1, &hf_rnsap_id_commonTransportChannelResourcesInitialisation_TDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_CommonTransportChannelResourcesResponseTDD },
  {   2, &hf_rnsap_id_dedicatedMeasurementInitiation1, ASN1_EXTENSION_ROOT    , dissect_rnsap_DedicatedMeasurementInitiationResponse },
  {   3, &hf_rnsap_id_physicalChannelReconfiguration1, ASN1_EXTENSION_ROOT    , dissect_rnsap_PhysicalChannelReconfigurationCommand },
  {   4, &hf_rnsap_id_radioLinkAddition1, ASN1_EXTENSION_ROOT    , dissect_rnsap_RadioLinkAdditionResponseFDD },
  {   5, &hf_rnsap_id_radioLinkAddition_TDD1, ASN1_EXTENSION_ROOT    , dissect_rnsap_RadioLinkAdditionResponseTDD },
  {   6, &hf_rnsap_id_radioLinkDeletion1, ASN1_EXTENSION_ROOT    , dissect_rnsap_RadioLinkDeletionResponse },
  {   7, &hf_rnsap_id_radioLinkSetup1, ASN1_EXTENSION_ROOT    , dissect_rnsap_RadioLinkSetupResponseFDD },
  {   8, &hf_rnsap_id_radioLinkSetupTdd1, ASN1_EXTENSION_ROOT    , dissect_rnsap_RadioLinkSetupResponseTDD },
  {   9, &hf_rnsap_id_synchronisedRadioLinkReconfigurationPreparation1, ASN1_EXTENSION_ROOT    , dissect_rnsap_RadioLinkReconfigurationReadyFDD },
  {  10, &hf_rnsap_id_synchronisedRadioLinkReconfigurationPreparation_TDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_RadioLinkReconfigurationReadyTDD },
  {  11, &hf_rnsap_id_unSynchronisedRadioLinkReconfiguration1, ASN1_EXTENSION_ROOT    , dissect_rnsap_RadioLinkReconfigurationResponseFDD },
  {  12, &hf_rnsap_id_unSynchronisedRadioLinkReconfiguration_TDD1, ASN1_EXTENSION_ROOT    , dissect_rnsap_RadioLinkReconfigurationResponseTDD },
  {  13, &hf_rnsap_id_commonMeasurementInitiation1, ASN1_EXTENSION_ROOT    , dissect_rnsap_CommonMeasurementInitiationResponse },
  {  14, &hf_rnsap_id_informationExchangeInitiation1, ASN1_EXTENSION_ROOT    , dissect_rnsap_InformationExchangeInitiationResponse },
  {  15, &hf_rnsap_id_reset1     , ASN1_EXTENSION_ROOT    , dissect_rnsap_ResetResponse },
  {  16, &hf_rnsap_id_uEMeasurementInitiation1, ASN1_EXTENSION_ROOT    , dissect_rnsap_UEMeasurementInitiationResponse },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_DummySuccessfulOutcomeValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_DummySuccessfulOutcomeValue, DummySuccessfulOutcomeValue_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t CommonTransportChannelResourcesFailure_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_CommonTransportChannelResourcesFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_CommonTransportChannelResourcesFailure, CommonTransportChannelResourcesFailure_sequence);

  return offset;
}
static int dissect_id_commonTransportChannelResourcesInitialisation2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CommonTransportChannelResourcesFailure(tvb, offset, actx, tree, hf_rnsap_id_commonTransportChannelResourcesInitialisation2);
}


static const per_sequence_t DedicatedMeasurementInitiationFailure_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DedicatedMeasurementInitiationFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DedicatedMeasurementInitiationFailure, DedicatedMeasurementInitiationFailure_sequence);

  return offset;
}
static int dissect_id_dedicatedMeasurementInitiation2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DedicatedMeasurementInitiationFailure(tvb, offset, actx, tree, hf_rnsap_id_dedicatedMeasurementInitiation2);
}


static const per_sequence_t PhysicalChannelReconfigurationFailure_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_PhysicalChannelReconfigurationFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_PhysicalChannelReconfigurationFailure, PhysicalChannelReconfigurationFailure_sequence);

  return offset;
}
static int dissect_id_physicalChannelReconfiguration2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PhysicalChannelReconfigurationFailure(tvb, offset, actx, tree, hf_rnsap_id_physicalChannelReconfiguration2);
}


static const per_sequence_t RadioLinkAdditionFailureFDD_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RadioLinkAdditionFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RadioLinkAdditionFailureFDD, RadioLinkAdditionFailureFDD_sequence);

  return offset;
}
static int dissect_id_radioLinkAddition2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RadioLinkAdditionFailureFDD(tvb, offset, actx, tree, hf_rnsap_id_radioLinkAddition2);
}


static const per_sequence_t RadioLinkAdditionFailureTDD_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RadioLinkAdditionFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RadioLinkAdditionFailureTDD, RadioLinkAdditionFailureTDD_sequence);

  return offset;
}
static int dissect_id_radioLinkAddition_TDD2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RadioLinkAdditionFailureTDD(tvb, offset, actx, tree, hf_rnsap_id_radioLinkAddition_TDD2);
}


static const per_sequence_t RadioLinkSetupFailureFDD_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RadioLinkSetupFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RadioLinkSetupFailureFDD, RadioLinkSetupFailureFDD_sequence);

  return offset;
}
static int dissect_id_radioLinkSetup2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RadioLinkSetupFailureFDD(tvb, offset, actx, tree, hf_rnsap_id_radioLinkSetup2);
}


static const per_sequence_t RadioLinkSetupFailureTDD_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RadioLinkSetupFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RadioLinkSetupFailureTDD, RadioLinkSetupFailureTDD_sequence);

  return offset;
}
static int dissect_id_radioLinkSetupTdd2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RadioLinkSetupFailureTDD(tvb, offset, actx, tree, hf_rnsap_id_radioLinkSetupTdd2);
}


static const per_sequence_t RadioLinkReconfigurationFailure_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RadioLinkReconfigurationFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RadioLinkReconfigurationFailure, RadioLinkReconfigurationFailure_sequence);

  return offset;
}
static int dissect_id_synchronisedRadioLinkReconfigurationPreparation2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RadioLinkReconfigurationFailure(tvb, offset, actx, tree, hf_rnsap_id_synchronisedRadioLinkReconfigurationPreparation2);
}
static int dissect_id_unSynchronisedRadioLinkReconfiguration2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RadioLinkReconfigurationFailure(tvb, offset, actx, tree, hf_rnsap_id_unSynchronisedRadioLinkReconfiguration2);
}
static int dissect_id_unSynchronisedRadioLinkReconfiguration_TDD2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RadioLinkReconfigurationFailure(tvb, offset, actx, tree, hf_rnsap_id_unSynchronisedRadioLinkReconfiguration_TDD2);
}


static const per_sequence_t CommonMeasurementInitiationFailure_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_CommonMeasurementInitiationFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_CommonMeasurementInitiationFailure, CommonMeasurementInitiationFailure_sequence);

  return offset;
}
static int dissect_id_commonMeasurementInitiation2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CommonMeasurementInitiationFailure(tvb, offset, actx, tree, hf_rnsap_id_commonMeasurementInitiation2);
}


static const per_sequence_t InformationExchangeInitiationFailure_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_InformationExchangeInitiationFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_InformationExchangeInitiationFailure, InformationExchangeInitiationFailure_sequence);

  return offset;
}
static int dissect_id_informationExchangeInitiation2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_InformationExchangeInitiationFailure(tvb, offset, actx, tree, hf_rnsap_id_informationExchangeInitiation2);
}


static const per_sequence_t UEMeasurementInitiationFailure_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UEMeasurementInitiationFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UEMeasurementInitiationFailure, UEMeasurementInitiationFailure_sequence);

  return offset;
}
static int dissect_id_uEMeasurementInitiation2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEMeasurementInitiationFailure(tvb, offset, actx, tree, hf_rnsap_id_uEMeasurementInitiation2);
}


static const value_string rnsap_DummyUnSuccessfulOutcomeValue_vals[] = {
  {   0, "id-commonTransportChannelResourcesInitialisation" },
  {   1, "id-dedicatedMeasurementInitiation" },
  {   2, "id-physicalChannelReconfiguration" },
  {   3, "id-radioLinkAddition" },
  {   4, "id-radioLinkAddition-TDD" },
  {   5, "id-radioLinkSetup" },
  {   6, "id-radioLinkSetupTdd" },
  {   7, "id-synchronisedRadioLinkReconfigurationPreparation" },
  {   8, "id-unSynchronisedRadioLinkReconfiguration" },
  {   9, "id-unSynchronisedRadioLinkReconfiguration-TDD" },
  {  10, "id-commonMeasurementInitiation" },
  {  11, "id-informationExchangeInitiation" },
  {  12, "id-uEMeasurementInitiation" },
  { 0, NULL }
};

static const per_choice_t DummyUnSuccessfulOutcomeValue_choice[] = {
  {   0, &hf_rnsap_id_commonTransportChannelResourcesInitialisation2, ASN1_EXTENSION_ROOT    , dissect_rnsap_CommonTransportChannelResourcesFailure },
  {   1, &hf_rnsap_id_dedicatedMeasurementInitiation2, ASN1_EXTENSION_ROOT    , dissect_rnsap_DedicatedMeasurementInitiationFailure },
  {   2, &hf_rnsap_id_physicalChannelReconfiguration2, ASN1_EXTENSION_ROOT    , dissect_rnsap_PhysicalChannelReconfigurationFailure },
  {   3, &hf_rnsap_id_radioLinkAddition2, ASN1_EXTENSION_ROOT    , dissect_rnsap_RadioLinkAdditionFailureFDD },
  {   4, &hf_rnsap_id_radioLinkAddition_TDD2, ASN1_EXTENSION_ROOT    , dissect_rnsap_RadioLinkAdditionFailureTDD },
  {   5, &hf_rnsap_id_radioLinkSetup2, ASN1_EXTENSION_ROOT    , dissect_rnsap_RadioLinkSetupFailureFDD },
  {   6, &hf_rnsap_id_radioLinkSetupTdd2, ASN1_EXTENSION_ROOT    , dissect_rnsap_RadioLinkSetupFailureTDD },
  {   7, &hf_rnsap_id_synchronisedRadioLinkReconfigurationPreparation2, ASN1_EXTENSION_ROOT    , dissect_rnsap_RadioLinkReconfigurationFailure },
  {   8, &hf_rnsap_id_unSynchronisedRadioLinkReconfiguration2, ASN1_EXTENSION_ROOT    , dissect_rnsap_RadioLinkReconfigurationFailure },
  {   9, &hf_rnsap_id_unSynchronisedRadioLinkReconfiguration_TDD2, ASN1_EXTENSION_ROOT    , dissect_rnsap_RadioLinkReconfigurationFailure },
  {  10, &hf_rnsap_id_commonMeasurementInitiation2, ASN1_EXTENSION_ROOT    , dissect_rnsap_CommonMeasurementInitiationFailure },
  {  11, &hf_rnsap_id_informationExchangeInitiation2, ASN1_EXTENSION_ROOT    , dissect_rnsap_InformationExchangeInitiationFailure },
  {  12, &hf_rnsap_id_uEMeasurementInitiation2, ASN1_EXTENSION_ROOT    , dissect_rnsap_UEMeasurementInitiationFailure },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_DummyUnSuccessfulOutcomeValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_DummyUnSuccessfulOutcomeValue, DummyUnSuccessfulOutcomeValue_choice,
                                 NULL);

  return offset;
}



static int
dissect_rnsap_AllowedQueuingTime(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 60U, NULL, FALSE);

  return offset;
}
static int dissect_id_AllowedQueuingTime(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_AllowedQueuingTime(tvb, offset, actx, tree, hf_rnsap_id_AllowedQueuingTime);
}



static int
dissect_rnsap_Allowed_Rate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 32U, NULL, FALSE);

  return offset;
}
static int dissect_allowed_UL_Rate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Allowed_Rate(tvb, offset, actx, tree, hf_rnsap_allowed_UL_Rate);
}
static int dissect_allowed_DL_Rate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Allowed_Rate(tvb, offset, actx, tree, hf_rnsap_allowed_DL_Rate);
}


static const per_sequence_t Allowed_Rate_Information_sequence[] = {
  { &hf_rnsap_allowed_UL_Rate, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Allowed_Rate },
  { &hf_rnsap_allowed_DL_Rate, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Allowed_Rate },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_Allowed_Rate_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_Allowed_Rate_Information, Allowed_Rate_Information_sequence);

  return offset;
}
static int dissect_id_Allowed_Rate_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Allowed_Rate_Information(tvb, offset, actx, tree, hf_rnsap_id_Allowed_Rate_Information);
}
static int dissect_allowed_Rate_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Allowed_Rate_Information(tvb, offset, actx, tree, hf_rnsap_allowed_Rate_Information);
}


static const value_string rnsap_AntennaColocationIndicator_vals[] = {
  {   0, "co-located" },
  { 0, NULL }
};


static int
dissect_rnsap_AntennaColocationIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_id_AntennaColocationIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_AntennaColocationIndicator(tvb, offset, actx, tree, hf_rnsap_id_AntennaColocationIndicator);
}



static int
dissect_rnsap_BindingID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_octet_string(tvb, offset, actx, tree, hf_index,
                                       1, 4, NULL);

  return offset;
}
static int dissect_id_BindingID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BindingID(tvb, offset, actx, tree, hf_rnsap_id_BindingID);
}
static int dissect_bindingID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BindingID(tvb, offset, actx, tree, hf_rnsap_bindingID);
}



static int
dissect_rnsap_C_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 65535U, NULL, FALSE);

  return offset;
}
static int dissect_id_C_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_C_ID(tvb, offset, actx, tree, hf_rnsap_id_C_ID);
}
static int dissect_c_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_C_ID(tvb, offset, actx, tree, hf_rnsap_c_ID);
}
static int dissect_CorrespondingCells_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_C_ID(tvb, offset, actx, tree, hf_rnsap_CorrespondingCells_item);
}



static int
dissect_rnsap_C_RNTI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 65535U, NULL, FALSE);

  return offset;
}
static int dissect_id_C_RNTI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_C_RNTI(tvb, offset, actx, tree, hf_rnsap_id_C_RNTI);
}



static int
dissect_rnsap_INTEGER_1_100_(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 100U, NULL, TRUE);

  return offset;
}
static int dissect_uplinkCellCapacityClassValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_1_100_(tvb, offset, actx, tree, hf_rnsap_uplinkCellCapacityClassValue);
}
static int dissect_downlinkCellCapacityClassValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_1_100_(tvb, offset, actx, tree, hf_rnsap_downlinkCellCapacityClassValue);
}


static const per_sequence_t Cell_Capacity_Class_Value_sequence[] = {
  { &hf_rnsap_uplinkCellCapacityClassValue, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_INTEGER_1_100_ },
  { &hf_rnsap_downlinkCellCapacityClassValue, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_INTEGER_1_100_ },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_Cell_Capacity_Class_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_Cell_Capacity_Class_Value, Cell_Capacity_Class_Value_sequence);

  return offset;
}
static int dissect_id_Cell_Capacity_Class_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Cell_Capacity_Class_Value(tvb, offset, actx, tree, hf_rnsap_id_Cell_Capacity_Class_Value);
}



static int
dissect_rnsap_CFN(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 255U, NULL, FALSE);

  return offset;
}
static int dissect_id_CFN(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CFN(tvb, offset, actx, tree, hf_rnsap_id_CFN);
}
static int dissect_cFN(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CFN(tvb, offset, actx, tree, hf_rnsap_cFN);
}
static int dissect_cMConfigurationChangeCFN(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CFN(tvb, offset, actx, tree, hf_rnsap_cMConfigurationChangeCFN);
}
static int dissect_cfn(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CFN(tvb, offset, actx, tree, hf_rnsap_cfn);
}
static int dissect_synchronised(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CFN(tvb, offset, actx, tree, hf_rnsap_synchronised);
}
static int dissect_tGCFN(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CFN(tvb, offset, actx, tree, hf_rnsap_tGCFN);
}



static int
dissect_rnsap_PLMN_Identity(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_octet_string(tvb, offset, actx, tree, hf_index,
                                       3, 3, NULL);

  return offset;
}
static int dissect_pLMN_Identity(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PLMN_Identity(tvb, offset, actx, tree, hf_rnsap_pLMN_Identity);
}
static int dissect_plmn_id(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PLMN_Identity(tvb, offset, actx, tree, hf_rnsap_plmn_id);
}



static int
dissect_rnsap_LAC(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_octet_string(tvb, offset, actx, tree, hf_index,
                                       2, 2, NULL);

  return offset;
}
static int dissect_lAC(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_LAC(tvb, offset, actx, tree, hf_rnsap_lAC);
}


static const per_sequence_t CN_CS_DomainIdentifier_sequence[] = {
  { &hf_rnsap_pLMN_Identity , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_PLMN_Identity },
  { &hf_rnsap_lAC           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_LAC },
  { &hf_rnsap_iE_Extensions , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_CN_CS_DomainIdentifier(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_CN_CS_DomainIdentifier, CN_CS_DomainIdentifier_sequence);

  return offset;
}
static int dissect_id_CN_CS_DomainIdentifier(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CN_CS_DomainIdentifier(tvb, offset, actx, tree, hf_rnsap_id_CN_CS_DomainIdentifier);
}
static int dissect_cN_CS_DomainIdentifier(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CN_CS_DomainIdentifier(tvb, offset, actx, tree, hf_rnsap_cN_CS_DomainIdentifier);
}



static int
dissect_rnsap_RAC(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_octet_string(tvb, offset, actx, tree, hf_index,
                                       1, 1, NULL);

  return offset;
}
static int dissect_rAC(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RAC(tvb, offset, actx, tree, hf_rnsap_rAC);
}


static const per_sequence_t CN_PS_DomainIdentifier_sequence[] = {
  { &hf_rnsap_pLMN_Identity , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_PLMN_Identity },
  { &hf_rnsap_lAC           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_LAC },
  { &hf_rnsap_rAC           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_RAC },
  { &hf_rnsap_iE_Extensions , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_CN_PS_DomainIdentifier(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_CN_PS_DomainIdentifier, CN_PS_DomainIdentifier_sequence);

  return offset;
}
static int dissect_id_CN_PS_DomainIdentifier(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CN_PS_DomainIdentifier(tvb, offset, actx, tree, hf_rnsap_id_CN_PS_DomainIdentifier);
}
static int dissect_cN_PS_DomainIdentifier(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CN_PS_DomainIdentifier(tvb, offset, actx, tree, hf_rnsap_cN_PS_DomainIdentifier);
}


static const value_string rnsap_CauseRadioNetwork_vals[] = {
  {   0, "unknown-C-ID" },
  {   1, "cell-not-available" },
  {   2, "power-level-not-supported" },
  {   3, "ul-scrambling-code-already-in-use" },
  {   4, "dl-radio-resources-not-available" },
  {   5, "ul-radio-resources-not-available" },
  {   6, "measurement-not-supported-for-the-object" },
  {   7, "combining-resources-not-available" },
  {   8, "combining-not-supported" },
  {   9, "reconfiguration-not-allowed" },
  {  10, "requested-configuration-not-supported" },
  {  11, "synchronisation-failure" },
  {  12, "requested-tx-diversity-mode-not-supported" },
  {  13, "measurement-temporaily-not-available" },
  {  14, "unspecified" },
  {  15, "invalid-CM-settings" },
  {  16, "reconfiguration-CFN-not-elapsed" },
  {  17, "number-of-DL-codes-not-supported" },
  {  18, "dedicated-transport-channel-type-not-supported" },
  {  19, "dl-shared-channel-type-not-supported" },
  {  20, "ul-shared-channel-type-not-supported" },
  {  21, "common-transport-channel-type-not-supported" },
  {  22, "ul-spreading-factor-not-supported" },
  {  23, "dl-spreading-factor-not-supported" },
  {  24, "cm-not-supported" },
  {  25, "transaction-not-supported-by-destination-node-b" },
  {  26, "rl-already-activated-or-alocated" },
  {  27, "number-of-UL-codes-not-supported" },
  {  28, "cell-reserved-for-operator-use" },
  {  29, "dpc-mode-change-not-supported" },
  {  30, "information-temporarily-not-available" },
  {  31, "information-provision-not-supported-for-the-object" },
  {  32, "power-balancing-status-not-compatible" },
  {  33, "delayed-activation-not-supported" },
  {  34, "rl-timing-adjustment-not-supported" },
  {  35, "unknown-RNTI" },
  {  36, "measurement-repetition-rate-not-compatible" },
  {  37, "ue-not-capable-of-support" },
  {  38, "f-dpch-not-supported" },
  { 0, NULL }
};


static int
dissect_rnsap_CauseRadioNetwork(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     27, NULL, TRUE, 12, NULL);

  return offset;
}
static int dissect_radioNetwork(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CauseRadioNetwork(tvb, offset, actx, tree, hf_rnsap_radioNetwork);
}


static const value_string rnsap_CauseTransport_vals[] = {
  {   0, "transport-resource-unavailable" },
  {   1, "unspecified" },
  { 0, NULL }
};


static int
dissect_rnsap_CauseTransport(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_transport(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CauseTransport(tvb, offset, actx, tree, hf_rnsap_transport);
}


static const value_string rnsap_CauseProtocol_vals[] = {
  {   0, "transfer-syntax-error" },
  {   1, "abstract-syntax-error-reject" },
  {   2, "abstract-syntax-error-ignore-and-notify" },
  {   3, "message-not-compatible-with-receiver-state" },
  {   4, "semantic-error" },
  {   5, "unspecified" },
  {   6, "abstract-syntax-error-falsely-constructed-message" },
  { 0, NULL }
};


static int
dissect_rnsap_CauseProtocol(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     7, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_protocol(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CauseProtocol(tvb, offset, actx, tree, hf_rnsap_protocol);
}


static const value_string rnsap_CauseMisc_vals[] = {
  {   0, "control-processing-overload" },
  {   1, "hardware-failure" },
  {   2, "om-intervention" },
  {   3, "not-enough-user-plane-processing-resources" },
  {   4, "unspecified" },
  { 0, NULL }
};


static int
dissect_rnsap_CauseMisc(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     5, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_misc(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CauseMisc(tvb, offset, actx, tree, hf_rnsap_misc);
}


static const value_string rnsap_Cause_vals[] = {
  {   0, "radioNetwork" },
  {   1, "transport" },
  {   2, "protocol" },
  {   3, "misc" },
  { 0, NULL }
};

static const per_choice_t Cause_choice[] = {
  {   0, &hf_rnsap_radioNetwork  , ASN1_EXTENSION_ROOT    , dissect_rnsap_CauseRadioNetwork },
  {   1, &hf_rnsap_transport     , ASN1_EXTENSION_ROOT    , dissect_rnsap_CauseTransport },
  {   2, &hf_rnsap_protocol      , ASN1_EXTENSION_ROOT    , dissect_rnsap_CauseProtocol },
  {   3, &hf_rnsap_misc          , ASN1_EXTENSION_ROOT    , dissect_rnsap_CauseMisc },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_Cause(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_Cause, Cause_choice,
                                 NULL);

  return offset;
}
static int dissect_id_Cause(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Cause(tvb, offset, actx, tree, hf_rnsap_id_Cause);
}
static int dissect_cause(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Cause(tvb, offset, actx, tree, hf_rnsap_cause);
}
static int dissect_individualcause(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Cause(tvb, offset, actx, tree, hf_rnsap_individualcause);
}


static const value_string rnsap_CoverageIndicator_vals[] = {
  {   0, "overlap" },
  {   1, "covers" },
  {   2, "containedIn" },
  { 0, NULL }
};


static int
dissect_rnsap_CoverageIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_id_CoverageIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CoverageIndicator(tvb, offset, actx, tree, hf_rnsap_id_CoverageIndicator);
}


static const value_string rnsap_TriggeringMessage_vals[] = {
  {   0, "initiating-message" },
  {   1, "successful-outcome" },
  {   2, "unsuccessful-outcome" },
  {   3, "outcome" },
  { 0, NULL }
};


static int
dissect_rnsap_TriggeringMessage(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_triggeringMessage(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TriggeringMessage(tvb, offset, actx, tree, hf_rnsap_triggeringMessage);
}



static int
dissect_rnsap_RepetitionNumber0(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 255U, NULL, FALSE);

  return offset;
}
static int dissect_repetitionNumber(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RepetitionNumber0(tvb, offset, actx, tree, hf_rnsap_repetitionNumber);
}


static const per_sequence_t CriticalityDiagnostics_IE_List_item_sequence[] = {
  { &hf_rnsap_iECriticality , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_Criticality },
  { &hf_rnsap_iE_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_ID },
  { &hf_rnsap_repetitionNumber, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_RepetitionNumber0 },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_CriticalityDiagnostics_IE_List_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_CriticalityDiagnostics_IE_List_item, CriticalityDiagnostics_IE_List_item_sequence);

  return offset;
}
static int dissect_CriticalityDiagnostics_IE_List_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CriticalityDiagnostics_IE_List_item(tvb, offset, actx, tree, hf_rnsap_CriticalityDiagnostics_IE_List_item);
}


static const per_sequence_t CriticalityDiagnostics_IE_List_sequence_of[1] = {
  { &hf_rnsap_CriticalityDiagnostics_IE_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_CriticalityDiagnostics_IE_List_item },
};

static int
dissect_rnsap_CriticalityDiagnostics_IE_List(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_CriticalityDiagnostics_IE_List, CriticalityDiagnostics_IE_List_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_iEsCriticalityDiagnostics(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CriticalityDiagnostics_IE_List(tvb, offset, actx, tree, hf_rnsap_iEsCriticalityDiagnostics);
}


static const per_sequence_t CriticalityDiagnostics_sequence[] = {
  { &hf_rnsap_procedureID   , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProcedureID },
  { &hf_rnsap_triggeringMessage, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TriggeringMessage },
  { &hf_rnsap_procedureCriticality, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Criticality },
  { &hf_rnsap_transactionID , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TransactionID },
  { &hf_rnsap_iEsCriticalityDiagnostics, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_CriticalityDiagnostics_IE_List },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_CriticalityDiagnostics(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_CriticalityDiagnostics, CriticalityDiagnostics_sequence);

  return offset;
}
static int dissect_id_CriticalityDiagnostics(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CriticalityDiagnostics(tvb, offset, actx, tree, hf_rnsap_id_CriticalityDiagnostics);
}



static int
dissect_rnsap_S_RNTI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 1048575U, NULL, FALSE);

  return offset;
}
static int dissect_id_S_RNTI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_S_RNTI(tvb, offset, actx, tree, hf_rnsap_id_S_RNTI);
}
static int dissect_sRNTI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_S_RNTI(tvb, offset, actx, tree, hf_rnsap_sRNTI);
}
static int dissect_AffectedUEInformationForMBMS_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_S_RNTI(tvb, offset, actx, tree, hf_rnsap_AffectedUEInformationForMBMS_item);
}



static int
dissect_rnsap_D_RNTI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 1048575U, NULL, FALSE);

  return offset;
}
static int dissect_id_D_RNTI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_D_RNTI(tvb, offset, actx, tree, hf_rnsap_id_D_RNTI);
}
static int dissect_dRNTI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_D_RNTI(tvb, offset, actx, tree, hf_rnsap_dRNTI);
}
static int dissect_d_RNTI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_D_RNTI(tvb, offset, actx, tree, hf_rnsap_d_RNTI);
}


static const value_string rnsap_ContextType_Reset_vals[] = {
  {   0, "sRNTI" },
  {   1, "dRNTI" },
  { 0, NULL }
};

static const per_choice_t ContextType_Reset_choice[] = {
  {   0, &hf_rnsap_sRNTI         , ASN1_EXTENSION_ROOT    , dissect_rnsap_S_RNTI },
  {   1, &hf_rnsap_dRNTI         , ASN1_EXTENSION_ROOT    , dissect_rnsap_D_RNTI },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_ContextType_Reset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_ContextType_Reset, ContextType_Reset_choice,
                                 NULL);

  return offset;
}
static int dissect_contextType_Reset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ContextType_Reset(tvb, offset, actx, tree, hf_rnsap_contextType_Reset);
}


static const per_sequence_t ContextInfoItem_Reset_sequence[] = {
  { &hf_rnsap_contextType_Reset, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ContextType_Reset },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_ContextInfoItem_Reset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_ContextInfoItem_Reset, ContextInfoItem_Reset_sequence);

  return offset;
}
static int dissect_id_ContextInfoItem_Reset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ContextInfoItem_Reset(tvb, offset, actx, tree, hf_rnsap_id_ContextInfoItem_Reset);
}


static const value_string rnsap_T_sRNTI_BitMaskIndex_vals[] = {
  {   0, "b1" },
  {   1, "b2" },
  {   2, "b3" },
  {   3, "b4" },
  {   4, "b5" },
  {   5, "b6" },
  {   6, "b7" },
  {   7, "b8" },
  {   8, "b9" },
  {   9, "b10" },
  {  10, "b11" },
  {  11, "b12" },
  {  12, "b13" },
  {  13, "b14" },
  {  14, "b15" },
  {  15, "b16" },
  {  16, "b17" },
  {  17, "b18" },
  {  18, "b19" },
  { 0, NULL }
};


static int
dissect_rnsap_T_sRNTI_BitMaskIndex(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     19, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_sRNTI_BitMaskIndex(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_T_sRNTI_BitMaskIndex(tvb, offset, actx, tree, hf_rnsap_sRNTI_BitMaskIndex);
}


static const per_sequence_t S_RNTI_Group_sequence[] = {
  { &hf_rnsap_sRNTI         , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_S_RNTI },
  { &hf_rnsap_sRNTI_BitMaskIndex, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_T_sRNTI_BitMaskIndex },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_S_RNTI_Group(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_S_RNTI_Group, S_RNTI_Group_sequence);

  return offset;
}
static int dissect_s_RNTI_Group(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_S_RNTI_Group(tvb, offset, actx, tree, hf_rnsap_s_RNTI_Group);
}


static const per_sequence_t ContextGroupInfoItem_Reset_sequence[] = {
  { &hf_rnsap_s_RNTI_Group  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_S_RNTI_Group },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_ContextGroupInfoItem_Reset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_ContextGroupInfoItem_Reset, ContextGroupInfoItem_Reset_sequence);

  return offset;
}
static int dissect_id_ContextGroupInfoItem_Reset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ContextGroupInfoItem_Reset(tvb, offset, actx, tree, hf_rnsap_id_ContextGroupInfoItem_Reset);
}


static const value_string rnsap_D_RNTI_ReleaseIndication_vals[] = {
  {   0, "release-D-RNTI" },
  {   1, "not-release-D-RNTI" },
  { 0, NULL }
};


static int
dissect_rnsap_D_RNTI_ReleaseIndication(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_D_RNTI_ReleaseIndication(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_D_RNTI_ReleaseIndication(tvb, offset, actx, tree, hf_rnsap_id_D_RNTI_ReleaseIndication);
}


static const value_string rnsap_PayloadCRC_PresenceIndicator_vals[] = {
  {   0, "crc-included" },
  {   1, "crc-not-included" },
  { 0, NULL }
};


static int
dissect_rnsap_PayloadCRC_PresenceIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_payloadCRC_PresenceIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PayloadCRC_PresenceIndicator(tvb, offset, actx, tree, hf_rnsap_payloadCRC_PresenceIndicator);
}


static const value_string rnsap_UL_FP_Mode_vals[] = {
  {   0, "normal" },
  {   1, "silent" },
  { 0, NULL }
};


static int
dissect_rnsap_UL_FP_Mode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_ul_FP_Mode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_FP_Mode(tvb, offset, actx, tree, hf_rnsap_ul_FP_Mode);
}



static int
dissect_rnsap_ToAWS(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 1279U, NULL, FALSE);

  return offset;
}
static int dissect_toAWS(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ToAWS(tvb, offset, actx, tree, hf_rnsap_toAWS);
}



static int
dissect_rnsap_ToAWE(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 2559U, NULL, FALSE);

  return offset;
}
static int dissect_toAWE(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ToAWE(tvb, offset, actx, tree, hf_rnsap_toAWE);
}



static int
dissect_rnsap_DCH_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 255U, NULL, FALSE);

  return offset;
}
static int dissect_dCH_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DCH_ID(tvb, offset, actx, tree, hf_rnsap_dCH_ID);
}
static int dissect_dCH_id(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DCH_ID(tvb, offset, actx, tree, hf_rnsap_dCH_id);
}


static const value_string rnsap_TrCH_SrcStatisticsDescr_vals[] = {
  {   0, "speech" },
  {   1, "rRC" },
  {   2, "unknown" },
  { 0, NULL }
};


static int
dissect_rnsap_TrCH_SrcStatisticsDescr(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_trChSourceStatisticsDescriptor(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TrCH_SrcStatisticsDescr(tvb, offset, actx, tree, hf_rnsap_trChSourceStatisticsDescriptor);
}
static int dissect_trCH_SrcStatisticsDescr(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TrCH_SrcStatisticsDescr(tvb, offset, actx, tree, hf_rnsap_trCH_SrcStatisticsDescr);
}



static int
dissect_rnsap_NrOfTransportBlocks(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 512U, NULL, FALSE);

  return offset;
}
static int dissect_nrOfTransportBlocks(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_NrOfTransportBlocks(tvb, offset, actx, tree, hf_rnsap_nrOfTransportBlocks);
}



static int
dissect_rnsap_TransportBlockSize(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 5000U, NULL, FALSE);

  return offset;
}
static int dissect_transportBlockSize(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TransportBlockSize(tvb, offset, actx, tree, hf_rnsap_transportBlockSize);
}


static const value_string rnsap_TransmissionTimeIntervalDynamic_vals[] = {
  {   0, "msec-10" },
  {   1, "msec-20" },
  {   2, "msec-40" },
  {   3, "msec-80" },
  { 0, NULL }
};


static int
dissect_rnsap_TransmissionTimeIntervalDynamic(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_transmissionTimeInterval(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TransmissionTimeIntervalDynamic(tvb, offset, actx, tree, hf_rnsap_transmissionTimeInterval);
}


static const per_sequence_t TransmissionTimeIntervalInformation_item_sequence[] = {
  { &hf_rnsap_transmissionTimeInterval, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TransmissionTimeIntervalDynamic },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_TransmissionTimeIntervalInformation_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_TransmissionTimeIntervalInformation_item, TransmissionTimeIntervalInformation_item_sequence);

  return offset;
}
static int dissect_TransmissionTimeIntervalInformation_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TransmissionTimeIntervalInformation_item(tvb, offset, actx, tree, hf_rnsap_TransmissionTimeIntervalInformation_item);
}


static const per_sequence_t TransmissionTimeIntervalInformation_sequence_of[1] = {
  { &hf_rnsap_TransmissionTimeIntervalInformation_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_TransmissionTimeIntervalInformation_item },
};

static int
dissect_rnsap_TransmissionTimeIntervalInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_TransmissionTimeIntervalInformation, TransmissionTimeIntervalInformation_sequence_of,
                                                  1, 4);

  return offset;
}
static int dissect_transmissionTimeIntervalInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TransmissionTimeIntervalInformation(tvb, offset, actx, tree, hf_rnsap_transmissionTimeIntervalInformation);
}


static const per_sequence_t TDD_TransportFormatSet_ModeDP_sequence[] = {
  { &hf_rnsap_transmissionTimeIntervalInformation, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TransmissionTimeIntervalInformation },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_TDD_TransportFormatSet_ModeDP(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_TDD_TransportFormatSet_ModeDP, TDD_TransportFormatSet_ModeDP_sequence);

  return offset;
}
static int dissect_tdd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_TransportFormatSet_ModeDP(tvb, offset, actx, tree, hf_rnsap_tdd);
}



static int
dissect_rnsap_NULL(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_null(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_not_Used_sSDT_CellIdLength(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_NULL(tvb, offset, actx, tree, hf_rnsap_not_Used_sSDT_CellIdLength);
}
static int dissect_not_Used_s_FieldLength(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_NULL(tvb, offset, actx, tree, hf_rnsap_not_Used_s_FieldLength);
}
static int dissect_not_Used_sSDT_CellID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_NULL(tvb, offset, actx, tree, hf_rnsap_not_Used_sSDT_CellID);
}
static int dissect_not_Used_secondary_CCPCH_Info(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_NULL(tvb, offset, actx, tree, hf_rnsap_not_Used_secondary_CCPCH_Info);
}
static int dissect_not_Used_dSCHInformationResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_NULL(tvb, offset, actx, tree, hf_rnsap_not_Used_dSCHInformationResponse);
}
static int dissect_not_Used_dSCH_InformationResponse_RL_SetupFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_NULL(tvb, offset, actx, tree, hf_rnsap_not_Used_dSCH_InformationResponse_RL_SetupFailureFDD);
}
static int dissect_not_Used_sSDT_CellIDLength(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_NULL(tvb, offset, actx, tree, hf_rnsap_not_Used_sSDT_CellIDLength);
}
static int dissect_not_Used_sSDT_Indication(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_NULL(tvb, offset, actx, tree, hf_rnsap_not_Used_sSDT_Indication);
}
static int dissect_not_Used_sSDT_CellIdentity(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_NULL(tvb, offset, actx, tree, hf_rnsap_not_Used_sSDT_CellIdentity);
}
static int dissect_not_Used_dSCHsToBeAddedOrModified(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_NULL(tvb, offset, actx, tree, hf_rnsap_not_Used_dSCHsToBeAddedOrModified);
}
static int dissect_all_contexts(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_NULL(tvb, offset, actx, tree, hf_rnsap_all_contexts);
}
static int dissect_measurementnotAvailable(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_NULL(tvb, offset, actx, tree, hf_rnsap_measurementnotAvailable);
}
static int dissect_separate_indication(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_NULL(tvb, offset, actx, tree, hf_rnsap_separate_indication);
}
static int dissect_unsynchronised(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_NULL(tvb, offset, actx, tree, hf_rnsap_unsynchronised);
}
static int dissect_e_DCH_Scheduled_Transmission_Grant(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_NULL(tvb, offset, actx, tree, hf_rnsap_e_DCH_Scheduled_Transmission_Grant);
}
static int dissect_e_DCH_Serving_RL_not_in_this_DRNS(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_NULL(tvb, offset, actx, tree, hf_rnsap_e_DCH_Serving_RL_not_in_this_DRNS);
}
static int dissect_not_Used_dRACControl(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_NULL(tvb, offset, actx, tree, hf_rnsap_not_Used_dRACControl);
}
static int dissect_noBadSatellite(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_NULL(tvb, offset, actx, tree, hf_rnsap_noBadSatellite);
}
static int dissect_onDemand(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_NULL(tvb, offset, actx, tree, hf_rnsap_onDemand);
}
static int dissect_defaultMidamble(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_NULL(tvb, offset, actx, tree, hf_rnsap_defaultMidamble);
}
static int dissect_commonMidamble(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_NULL(tvb, offset, actx, tree, hf_rnsap_commonMidamble);
}
static int dissect_not_used_closedLoopMode2_SupportIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_NULL(tvb, offset, actx, tree, hf_rnsap_not_used_closedLoopMode2_SupportIndicator);
}
static int dissect_secondary_CPICH_shall_not_be_used(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_NULL(tvb, offset, actx, tree, hf_rnsap_secondary_CPICH_shall_not_be_used);
}
static int dissect_not_Used_split_in_TFCI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_NULL(tvb, offset, actx, tree, hf_rnsap_not_Used_split_in_TFCI);
}
static int dissect_notApplicable(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_NULL(tvb, offset, actx, tree, hf_rnsap_notApplicable);
}


static const value_string rnsap_TransportFormatSet_ModeDP_vals[] = {
  {   0, "tdd" },
  {   1, "notApplicable" },
  { 0, NULL }
};

static const per_choice_t TransportFormatSet_ModeDP_choice[] = {
  {   0, &hf_rnsap_tdd           , ASN1_EXTENSION_ROOT    , dissect_rnsap_TDD_TransportFormatSet_ModeDP },
  {   1, &hf_rnsap_notApplicable , ASN1_EXTENSION_ROOT    , dissect_rnsap_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_TransportFormatSet_ModeDP(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_TransportFormatSet_ModeDP, TransportFormatSet_ModeDP_choice,
                                 NULL);

  return offset;
}
static int dissect_mode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TransportFormatSet_ModeDP(tvb, offset, actx, tree, hf_rnsap_mode);
}


static const per_sequence_t TransportFormatSet_DynamicPartList_item_sequence[] = {
  { &hf_rnsap_nrOfTransportBlocks, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_NrOfTransportBlocks },
  { &hf_rnsap_transportBlockSize, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TransportBlockSize },
  { &hf_rnsap_mode          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TransportFormatSet_ModeDP },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_TransportFormatSet_DynamicPartList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_TransportFormatSet_DynamicPartList_item, TransportFormatSet_DynamicPartList_item_sequence);

  return offset;
}
static int dissect_TransportFormatSet_DynamicPartList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TransportFormatSet_DynamicPartList_item(tvb, offset, actx, tree, hf_rnsap_TransportFormatSet_DynamicPartList_item);
}


static const per_sequence_t TransportFormatSet_DynamicPartList_sequence_of[1] = {
  { &hf_rnsap_TransportFormatSet_DynamicPartList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_TransportFormatSet_DynamicPartList_item },
};

static int
dissect_rnsap_TransportFormatSet_DynamicPartList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_TransportFormatSet_DynamicPartList, TransportFormatSet_DynamicPartList_sequence_of,
                                                  1, 32);

  return offset;
}
static int dissect_dynamicParts(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TransportFormatSet_DynamicPartList(tvb, offset, actx, tree, hf_rnsap_dynamicParts);
}


static const value_string rnsap_TransmissionTimeIntervalSemiStatic_vals[] = {
  {   0, "msec-10" },
  {   1, "msec-20" },
  {   2, "msec-40" },
  {   3, "msec-80" },
  {   4, "dynamic" },
  { 0, NULL }
};


static int
dissect_rnsap_TransmissionTimeIntervalSemiStatic(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     5, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_transmissionTime(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TransmissionTimeIntervalSemiStatic(tvb, offset, actx, tree, hf_rnsap_transmissionTime);
}


static const value_string rnsap_ChannelCodingType_vals[] = {
  {   0, "no-codingTDD" },
  {   1, "convolutional-coding" },
  {   2, "turbo-coding" },
  { 0, NULL }
};


static int
dissect_rnsap_ChannelCodingType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_channelCoding(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ChannelCodingType(tvb, offset, actx, tree, hf_rnsap_channelCoding);
}


static const value_string rnsap_CodingRate_vals[] = {
  {   0, "half" },
  {   1, "third" },
  { 0, NULL }
};


static int
dissect_rnsap_CodingRate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_codingRate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CodingRate(tvb, offset, actx, tree, hf_rnsap_codingRate);
}



static int
dissect_rnsap_RateMatchingAttribute(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 256U, NULL, FALSE);

  return offset;
}
static int dissect_rateMatcingAttribute(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RateMatchingAttribute(tvb, offset, actx, tree, hf_rnsap_rateMatcingAttribute);
}


static const value_string rnsap_CRC_Size_vals[] = {
  {   0, "v0" },
  {   1, "v8" },
  {   2, "v12" },
  {   3, "v16" },
  {   4, "v24" },
  { 0, NULL }
};


static int
dissect_rnsap_CRC_Size(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     5, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_cRC_Size(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CRC_Size(tvb, offset, actx, tree, hf_rnsap_cRC_Size);
}


static const value_string rnsap_SecondInterleavingMode_vals[] = {
  {   0, "frame-related" },
  {   1, "timeslot-related" },
  { 0, NULL }
};


static int
dissect_rnsap_SecondInterleavingMode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_tdd1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SecondInterleavingMode(tvb, offset, actx, tree, hf_rnsap_tdd1);
}


static const value_string rnsap_TransportFormatSet_ModeSSP_vals[] = {
  {   0, "tdd" },
  {   1, "notApplicable" },
  { 0, NULL }
};

static const per_choice_t TransportFormatSet_ModeSSP_choice[] = {
  {   0, &hf_rnsap_tdd1          , ASN1_EXTENSION_ROOT    , dissect_rnsap_SecondInterleavingMode },
  {   1, &hf_rnsap_notApplicable , ASN1_EXTENSION_ROOT    , dissect_rnsap_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_TransportFormatSet_ModeSSP(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_TransportFormatSet_ModeSSP, TransportFormatSet_ModeSSP_choice,
                                 NULL);

  return offset;
}
static int dissect_mode1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TransportFormatSet_ModeSSP(tvb, offset, actx, tree, hf_rnsap_mode1);
}


static const per_sequence_t TransportFormatSet_Semi_staticPart_sequence[] = {
  { &hf_rnsap_transmissionTime, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TransmissionTimeIntervalSemiStatic },
  { &hf_rnsap_channelCoding , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ChannelCodingType },
  { &hf_rnsap_codingRate    , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_CodingRate },
  { &hf_rnsap_rateMatcingAttribute, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RateMatchingAttribute },
  { &hf_rnsap_cRC_Size      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CRC_Size },
  { &hf_rnsap_mode1         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TransportFormatSet_ModeSSP },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_TransportFormatSet_Semi_staticPart(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_TransportFormatSet_Semi_staticPart, TransportFormatSet_Semi_staticPart_sequence);

  return offset;
}
static int dissect_semi_staticPart(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TransportFormatSet_Semi_staticPart(tvb, offset, actx, tree, hf_rnsap_semi_staticPart);
}


static const per_sequence_t TransportFormatSet_sequence[] = {
  { &hf_rnsap_dynamicParts  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TransportFormatSet_DynamicPartList },
  { &hf_rnsap_semi_staticPart, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TransportFormatSet_Semi_staticPart },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_TransportFormatSet(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_TransportFormatSet, TransportFormatSet_sequence);

  return offset;
}
static int dissect_transportFormatSet(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TransportFormatSet(tvb, offset, actx, tree, hf_rnsap_transportFormatSet);
}
static int dissect_ul_transportFormatSet(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TransportFormatSet(tvb, offset, actx, tree, hf_rnsap_ul_transportFormatSet);
}
static int dissect_dl_transportFormatSet(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TransportFormatSet(tvb, offset, actx, tree, hf_rnsap_dl_transportFormatSet);
}
static int dissect_ul_TransportformatSet(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TransportFormatSet(tvb, offset, actx, tree, hf_rnsap_ul_TransportformatSet);
}
static int dissect_dl_TransportformatSet(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TransportFormatSet(tvb, offset, actx, tree, hf_rnsap_dl_TransportformatSet);
}



static int
dissect_rnsap_BLER(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              -63, 0U, NULL, FALSE);

  return offset;
}
static int dissect_bLER(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BLER(tvb, offset, actx, tree, hf_rnsap_bLER);
}
static int dissect_ul_BLER(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BLER(tvb, offset, actx, tree, hf_rnsap_ul_BLER);
}
static int dissect_dl_BLER(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BLER(tvb, offset, actx, tree, hf_rnsap_dl_BLER);
}



static int
dissect_rnsap_PriorityLevel(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 15U, NULL, FALSE);

  return offset;
}
static int dissect_priorityLevel(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PriorityLevel(tvb, offset, actx, tree, hf_rnsap_priorityLevel);
}


static const value_string rnsap_Pre_emptionCapability_vals[] = {
  {   0, "shall-not-trigger-pre-emption" },
  {   1, "may-trigger-pre-emption" },
  { 0, NULL }
};


static int
dissect_rnsap_Pre_emptionCapability(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_pre_emptionCapability(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Pre_emptionCapability(tvb, offset, actx, tree, hf_rnsap_pre_emptionCapability);
}


static const value_string rnsap_Pre_emptionVulnerability_vals[] = {
  {   0, "not-pre-emptable" },
  {   1, "pre-emptable" },
  { 0, NULL }
};


static int
dissect_rnsap_Pre_emptionVulnerability(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_pre_emptionVulnerability(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Pre_emptionVulnerability(tvb, offset, actx, tree, hf_rnsap_pre_emptionVulnerability);
}


static const per_sequence_t AllocationRetentionPriority_sequence[] = {
  { &hf_rnsap_priorityLevel , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PriorityLevel },
  { &hf_rnsap_pre_emptionCapability, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_Pre_emptionCapability },
  { &hf_rnsap_pre_emptionVulnerability, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_Pre_emptionVulnerability },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_AllocationRetentionPriority(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_AllocationRetentionPriority, AllocationRetentionPriority_sequence);

  return offset;
}
static int dissect_allocationRetentionPriority(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_AllocationRetentionPriority(tvb, offset, actx, tree, hf_rnsap_allocationRetentionPriority);
}


static const value_string rnsap_FrameHandlingPriority_vals[] = {
  {   0, "lowest" },
  {  15, "highest" },
  { 0, NULL }
};


static int
dissect_rnsap_FrameHandlingPriority(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 15U, NULL, FALSE);

  return offset;
}
static int dissect_frameHandlingPriority(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_FrameHandlingPriority(tvb, offset, actx, tree, hf_rnsap_frameHandlingPriority);
}


static const value_string rnsap_QE_Selector_vals[] = {
  {   0, "selected" },
  {   1, "non-selected" },
  { 0, NULL }
};


static int
dissect_rnsap_QE_Selector(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_qE_Selector(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_QE_Selector(tvb, offset, actx, tree, hf_rnsap_qE_Selector);
}


static const value_string rnsap_DRACControl_vals[] = {
  {   0, "not-Used-requested" },
  {   1, "not-requested" },
  { 0, NULL }
};


static int
dissect_rnsap_DRACControl(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_dRACControl(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DRACControl(tvb, offset, actx, tree, hf_rnsap_dRACControl);
}


static const per_sequence_t DCH_Specific_FDD_Item_sequence[] = {
  { &hf_rnsap_dCH_ID        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DCH_ID },
  { &hf_rnsap_trCH_SrcStatisticsDescr, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TrCH_SrcStatisticsDescr },
  { &hf_rnsap_ul_transportFormatSet, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TransportFormatSet },
  { &hf_rnsap_dl_transportFormatSet, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TransportFormatSet },
  { &hf_rnsap_ul_BLER       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BLER },
  { &hf_rnsap_dl_BLER       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BLER },
  { &hf_rnsap_allocationRetentionPriority, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_AllocationRetentionPriority },
  { &hf_rnsap_frameHandlingPriority, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_FrameHandlingPriority },
  { &hf_rnsap_qE_Selector   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_QE_Selector },
  { &hf_rnsap_dRACControl   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DRACControl },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DCH_Specific_FDD_Item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DCH_Specific_FDD_Item, DCH_Specific_FDD_Item_sequence);

  return offset;
}
static int dissect_DCH_Specific_FDD_InformationList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DCH_Specific_FDD_Item(tvb, offset, actx, tree, hf_rnsap_DCH_Specific_FDD_InformationList_item);
}


static const per_sequence_t DCH_Specific_FDD_InformationList_sequence_of[1] = {
  { &hf_rnsap_DCH_Specific_FDD_InformationList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_DCH_Specific_FDD_Item },
};

static int
dissect_rnsap_DCH_Specific_FDD_InformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DCH_Specific_FDD_InformationList, DCH_Specific_FDD_InformationList_sequence_of,
                                                  1, 128);

  return offset;
}
static int dissect_dCH_SpecificInformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DCH_Specific_FDD_InformationList(tvb, offset, actx, tree, hf_rnsap_dCH_SpecificInformationList);
}


static const per_sequence_t DCH_FDD_InformationItem_sequence[] = {
  { &hf_rnsap_payloadCRC_PresenceIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PayloadCRC_PresenceIndicator },
  { &hf_rnsap_ul_FP_Mode    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_FP_Mode },
  { &hf_rnsap_toAWS         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ToAWS },
  { &hf_rnsap_toAWE         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ToAWE },
  { &hf_rnsap_dCH_SpecificInformationList, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DCH_Specific_FDD_InformationList },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DCH_FDD_InformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DCH_FDD_InformationItem, DCH_FDD_InformationItem_sequence);

  return offset;
}
static int dissect_DCH_FDD_Information_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DCH_FDD_InformationItem(tvb, offset, actx, tree, hf_rnsap_DCH_FDD_Information_item);
}


static const per_sequence_t DCH_FDD_Information_sequence_of[1] = {
  { &hf_rnsap_DCH_FDD_Information_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_DCH_FDD_InformationItem },
};

static int
dissect_rnsap_DCH_FDD_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DCH_FDD_Information, DCH_FDD_Information_sequence_of,
                                                  1, 128);

  return offset;
}
static int dissect_id_DCHs_to_Add_FDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DCH_FDD_Information(tvb, offset, actx, tree, hf_rnsap_id_DCHs_to_Add_FDD);
}
static int dissect_id_DCH_FDD_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DCH_FDD_Information(tvb, offset, actx, tree, hf_rnsap_id_DCH_FDD_Information);
}



static int
dissect_rnsap_CCTrCH_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 15U, NULL, FALSE);

  return offset;
}
static int dissect_cCTrCH_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CCTrCH_ID(tvb, offset, actx, tree, hf_rnsap_cCTrCH_ID);
}
static int dissect_dl_ccTrCHID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CCTrCH_ID(tvb, offset, actx, tree, hf_rnsap_dl_ccTrCHID);
}
static int dissect_ul_ccTrCHID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CCTrCH_ID(tvb, offset, actx, tree, hf_rnsap_ul_ccTrCHID);
}
static int dissect_ul_cCTrCH_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CCTrCH_ID(tvb, offset, actx, tree, hf_rnsap_ul_cCTrCH_ID);
}
static int dissect_dl_cCTrCH_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CCTrCH_ID(tvb, offset, actx, tree, hf_rnsap_dl_cCTrCH_ID);
}
static int dissect_ul_CCTrCH_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CCTrCH_ID(tvb, offset, actx, tree, hf_rnsap_ul_CCTrCH_ID);
}
static int dissect_dl_CCTrCH_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CCTrCH_ID(tvb, offset, actx, tree, hf_rnsap_dl_CCTrCH_ID);
}


static const per_sequence_t DCH_Specific_TDD_Item_sequence[] = {
  { &hf_rnsap_dCH_ID        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DCH_ID },
  { &hf_rnsap_ul_cCTrCH_ID  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CCTrCH_ID },
  { &hf_rnsap_dl_cCTrCH_ID  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CCTrCH_ID },
  { &hf_rnsap_trCH_SrcStatisticsDescr, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TrCH_SrcStatisticsDescr },
  { &hf_rnsap_ul_transportFormatSet, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TransportFormatSet },
  { &hf_rnsap_dl_transportFormatSet, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TransportFormatSet },
  { &hf_rnsap_ul_BLER       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BLER },
  { &hf_rnsap_dl_BLER       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BLER },
  { &hf_rnsap_allocationRetentionPriority, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_AllocationRetentionPriority },
  { &hf_rnsap_frameHandlingPriority, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_FrameHandlingPriority },
  { &hf_rnsap_qE_Selector   , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_QE_Selector },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DCH_Specific_TDD_Item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DCH_Specific_TDD_Item, DCH_Specific_TDD_Item_sequence);

  return offset;
}
static int dissect_DCH_Specific_TDD_InformationList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DCH_Specific_TDD_Item(tvb, offset, actx, tree, hf_rnsap_DCH_Specific_TDD_InformationList_item);
}


static const per_sequence_t DCH_Specific_TDD_InformationList_sequence_of[1] = {
  { &hf_rnsap_DCH_Specific_TDD_InformationList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_DCH_Specific_TDD_Item },
};

static int
dissect_rnsap_DCH_Specific_TDD_InformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DCH_Specific_TDD_InformationList, DCH_Specific_TDD_InformationList_sequence_of,
                                                  1, 128);

  return offset;
}
static int dissect_dCH_SpecificInformationList1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DCH_Specific_TDD_InformationList(tvb, offset, actx, tree, hf_rnsap_dCH_SpecificInformationList1);
}


static const per_sequence_t DCH_TDD_InformationItem_sequence[] = {
  { &hf_rnsap_payloadCRC_PresenceIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PayloadCRC_PresenceIndicator },
  { &hf_rnsap_ul_FP_Mode    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_FP_Mode },
  { &hf_rnsap_toAWS         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ToAWS },
  { &hf_rnsap_toAWE         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ToAWE },
  { &hf_rnsap_dCH_SpecificInformationList1, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DCH_Specific_TDD_InformationList },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DCH_TDD_InformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DCH_TDD_InformationItem, DCH_TDD_InformationItem_sequence);

  return offset;
}
static int dissect_DCH_TDD_Information_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DCH_TDD_InformationItem(tvb, offset, actx, tree, hf_rnsap_DCH_TDD_Information_item);
}


static const per_sequence_t DCH_TDD_Information_sequence_of[1] = {
  { &hf_rnsap_DCH_TDD_Information_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_DCH_TDD_InformationItem },
};

static int
dissect_rnsap_DCH_TDD_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DCH_TDD_Information, DCH_TDD_Information_sequence_of,
                                                  1, 128);

  return offset;
}
static int dissect_id_DCHs_to_Add_TDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DCH_TDD_Information(tvb, offset, actx, tree, hf_rnsap_id_DCHs_to_Add_TDD);
}
static int dissect_id_DCH_TDD_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DCH_TDD_Information(tvb, offset, actx, tree, hf_rnsap_id_DCH_TDD_Information);
}


static const per_sequence_t DCH_DeleteItem_RL_ReconfPrepFDD_sequence[] = {
  { &hf_rnsap_dCH_ID        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DCH_ID },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DCH_DeleteItem_RL_ReconfPrepFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DCH_DeleteItem_RL_ReconfPrepFDD, DCH_DeleteItem_RL_ReconfPrepFDD_sequence);

  return offset;
}
static int dissect_DCH_DeleteList_RL_ReconfPrepFDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DCH_DeleteItem_RL_ReconfPrepFDD(tvb, offset, actx, tree, hf_rnsap_DCH_DeleteList_RL_ReconfPrepFDD_item);
}


static const per_sequence_t DCH_DeleteList_RL_ReconfPrepFDD_sequence_of[1] = {
  { &hf_rnsap_DCH_DeleteList_RL_ReconfPrepFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_DCH_DeleteItem_RL_ReconfPrepFDD },
};

static int
dissect_rnsap_DCH_DeleteList_RL_ReconfPrepFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DCH_DeleteList_RL_ReconfPrepFDD, DCH_DeleteList_RL_ReconfPrepFDD_sequence_of,
                                                  0, 128);

  return offset;
}
static int dissect_id_DCH_DeleteList_RL_ReconfPrepFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DCH_DeleteList_RL_ReconfPrepFDD(tvb, offset, actx, tree, hf_rnsap_id_DCH_DeleteList_RL_ReconfPrepFDD);
}


static const per_sequence_t DCH_DeleteItem_RL_ReconfPrepTDD_sequence[] = {
  { &hf_rnsap_dCH_ID        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DCH_ID },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DCH_DeleteItem_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DCH_DeleteItem_RL_ReconfPrepTDD, DCH_DeleteItem_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_DCH_DeleteList_RL_ReconfPrepTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DCH_DeleteItem_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_rnsap_DCH_DeleteList_RL_ReconfPrepTDD_item);
}


static const per_sequence_t DCH_DeleteList_RL_ReconfPrepTDD_sequence_of[1] = {
  { &hf_rnsap_DCH_DeleteList_RL_ReconfPrepTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_DCH_DeleteItem_RL_ReconfPrepTDD },
};

static int
dissect_rnsap_DCH_DeleteList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DCH_DeleteList_RL_ReconfPrepTDD, DCH_DeleteList_RL_ReconfPrepTDD_sequence_of,
                                                  0, 128);

  return offset;
}
static int dissect_id_DCH_DeleteList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DCH_DeleteList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_rnsap_id_DCH_DeleteList_RL_ReconfPrepTDD);
}


static const per_sequence_t DCH_DeleteItem_RL_ReconfRqstFDD_sequence[] = {
  { &hf_rnsap_dCH_ID        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DCH_ID },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DCH_DeleteItem_RL_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DCH_DeleteItem_RL_ReconfRqstFDD, DCH_DeleteItem_RL_ReconfRqstFDD_sequence);

  return offset;
}
static int dissect_DCH_DeleteList_RL_ReconfRqstFDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DCH_DeleteItem_RL_ReconfRqstFDD(tvb, offset, actx, tree, hf_rnsap_DCH_DeleteList_RL_ReconfRqstFDD_item);
}


static const per_sequence_t DCH_DeleteList_RL_ReconfRqstFDD_sequence_of[1] = {
  { &hf_rnsap_DCH_DeleteList_RL_ReconfRqstFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_DCH_DeleteItem_RL_ReconfRqstFDD },
};

static int
dissect_rnsap_DCH_DeleteList_RL_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DCH_DeleteList_RL_ReconfRqstFDD, DCH_DeleteList_RL_ReconfRqstFDD_sequence_of,
                                                  0, 128);

  return offset;
}
static int dissect_id_DCH_DeleteList_RL_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DCH_DeleteList_RL_ReconfRqstFDD(tvb, offset, actx, tree, hf_rnsap_id_DCH_DeleteList_RL_ReconfRqstFDD);
}


static const per_sequence_t DCH_DeleteItem_RL_ReconfRqstTDD_sequence[] = {
  { &hf_rnsap_dCH_ID        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DCH_ID },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DCH_DeleteItem_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DCH_DeleteItem_RL_ReconfRqstTDD, DCH_DeleteItem_RL_ReconfRqstTDD_sequence);

  return offset;
}
static int dissect_DCH_DeleteList_RL_ReconfRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DCH_DeleteItem_RL_ReconfRqstTDD(tvb, offset, actx, tree, hf_rnsap_DCH_DeleteList_RL_ReconfRqstTDD_item);
}


static const per_sequence_t DCH_DeleteList_RL_ReconfRqstTDD_sequence_of[1] = {
  { &hf_rnsap_DCH_DeleteList_RL_ReconfRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_DCH_DeleteItem_RL_ReconfRqstTDD },
};

static int
dissect_rnsap_DCH_DeleteList_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DCH_DeleteList_RL_ReconfRqstTDD, DCH_DeleteList_RL_ReconfRqstTDD_sequence_of,
                                                  0, 128);

  return offset;
}
static int dissect_id_DCH_DeleteList_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DCH_DeleteList_RL_ReconfRqstTDD(tvb, offset, actx, tree, hf_rnsap_id_DCH_DeleteList_RL_ReconfRqstTDD);
}


static const value_string rnsap_TransportBearerRequestIndicator_vals[] = {
  {   0, "bearer-requested" },
  {   1, "bearer-not-requested" },
  { 0, NULL }
};


static int
dissect_rnsap_TransportBearerRequestIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_id_TransportBearerRequestIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TransportBearerRequestIndicator(tvb, offset, actx, tree, hf_rnsap_id_TransportBearerRequestIndicator);
}
static int dissect_transportBearerRequestIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TransportBearerRequestIndicator(tvb, offset, actx, tree, hf_rnsap_transportBearerRequestIndicator);
}


static const per_sequence_t FDD_DCHs_to_ModifySpecificItem_sequence[] = {
  { &hf_rnsap_dCH_ID        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DCH_ID },
  { &hf_rnsap_ul_TransportformatSet, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TransportFormatSet },
  { &hf_rnsap_dl_TransportformatSet, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TransportFormatSet },
  { &hf_rnsap_allocationRetentionPriority, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_AllocationRetentionPriority },
  { &hf_rnsap_frameHandlingPriority, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_FrameHandlingPriority },
  { &hf_rnsap_not_Used_dRACControl, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_NULL },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_FDD_DCHs_to_ModifySpecificItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_FDD_DCHs_to_ModifySpecificItem, FDD_DCHs_to_ModifySpecificItem_sequence);

  return offset;
}
static int dissect_FDD_DCHs_to_ModifySpecificInformationList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_FDD_DCHs_to_ModifySpecificItem(tvb, offset, actx, tree, hf_rnsap_FDD_DCHs_to_ModifySpecificInformationList_item);
}


static const per_sequence_t FDD_DCHs_to_ModifySpecificInformationList_sequence_of[1] = {
  { &hf_rnsap_FDD_DCHs_to_ModifySpecificInformationList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_FDD_DCHs_to_ModifySpecificItem },
};

static int
dissect_rnsap_FDD_DCHs_to_ModifySpecificInformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_FDD_DCHs_to_ModifySpecificInformationList, FDD_DCHs_to_ModifySpecificInformationList_sequence_of,
                                                  1, 128);

  return offset;
}
static int dissect_dCH_SpecificInformationList2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_FDD_DCHs_to_ModifySpecificInformationList(tvb, offset, actx, tree, hf_rnsap_dCH_SpecificInformationList2);
}


static const per_sequence_t FDD_DCHs_to_ModifyItem_sequence[] = {
  { &hf_rnsap_ul_FP_Mode    , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_UL_FP_Mode },
  { &hf_rnsap_toAWS         , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ToAWS },
  { &hf_rnsap_toAWE         , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ToAWE },
  { &hf_rnsap_transportBearerRequestIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TransportBearerRequestIndicator },
  { &hf_rnsap_dCH_SpecificInformationList2, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_FDD_DCHs_to_ModifySpecificInformationList },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_FDD_DCHs_to_ModifyItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_FDD_DCHs_to_ModifyItem, FDD_DCHs_to_ModifyItem_sequence);

  return offset;
}
static int dissect_FDD_DCHs_to_Modify_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_FDD_DCHs_to_ModifyItem(tvb, offset, actx, tree, hf_rnsap_FDD_DCHs_to_Modify_item);
}


static const per_sequence_t FDD_DCHs_to_Modify_sequence_of[1] = {
  { &hf_rnsap_FDD_DCHs_to_Modify_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_FDD_DCHs_to_ModifyItem },
};

static int
dissect_rnsap_FDD_DCHs_to_Modify(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_FDD_DCHs_to_Modify, FDD_DCHs_to_Modify_sequence_of,
                                                  1, 128);

  return offset;
}
static int dissect_id_FDD_DCHs_to_Modify(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_FDD_DCHs_to_Modify(tvb, offset, actx, tree, hf_rnsap_id_FDD_DCHs_to_Modify);
}


static const per_sequence_t TDD_DCHs_to_ModifySpecificItem_sequence[] = {
  { &hf_rnsap_dCH_ID        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DCH_ID },
  { &hf_rnsap_ul_CCTrCH_ID  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_CCTrCH_ID },
  { &hf_rnsap_dl_CCTrCH_ID  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_CCTrCH_ID },
  { &hf_rnsap_ul_TransportformatSet, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TransportFormatSet },
  { &hf_rnsap_dl_TransportformatSet, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TransportFormatSet },
  { &hf_rnsap_allocationRetentionPriority, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_AllocationRetentionPriority },
  { &hf_rnsap_frameHandlingPriority, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_FrameHandlingPriority },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_TDD_DCHs_to_ModifySpecificItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_TDD_DCHs_to_ModifySpecificItem, TDD_DCHs_to_ModifySpecificItem_sequence);

  return offset;
}
static int dissect_TDD_DCHs_to_ModifySpecificInformationList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_DCHs_to_ModifySpecificItem(tvb, offset, actx, tree, hf_rnsap_TDD_DCHs_to_ModifySpecificInformationList_item);
}


static const per_sequence_t TDD_DCHs_to_ModifySpecificInformationList_sequence_of[1] = {
  { &hf_rnsap_TDD_DCHs_to_ModifySpecificInformationList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_DCHs_to_ModifySpecificItem },
};

static int
dissect_rnsap_TDD_DCHs_to_ModifySpecificInformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_TDD_DCHs_to_ModifySpecificInformationList, TDD_DCHs_to_ModifySpecificInformationList_sequence_of,
                                                  1, 128);

  return offset;
}
static int dissect_dCH_SpecificInformationList3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_DCHs_to_ModifySpecificInformationList(tvb, offset, actx, tree, hf_rnsap_dCH_SpecificInformationList3);
}


static const per_sequence_t TDD_DCHs_to_ModifyItem_sequence[] = {
  { &hf_rnsap_ul_FP_Mode    , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_UL_FP_Mode },
  { &hf_rnsap_toAWS         , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ToAWS },
  { &hf_rnsap_toAWE         , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ToAWE },
  { &hf_rnsap_transportBearerRequestIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TransportBearerRequestIndicator },
  { &hf_rnsap_dCH_SpecificInformationList3, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_DCHs_to_ModifySpecificInformationList },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_TDD_DCHs_to_ModifyItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_TDD_DCHs_to_ModifyItem, TDD_DCHs_to_ModifyItem_sequence);

  return offset;
}
static int dissect_TDD_DCHs_to_Modify_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_DCHs_to_ModifyItem(tvb, offset, actx, tree, hf_rnsap_TDD_DCHs_to_Modify_item);
}


static const per_sequence_t TDD_DCHs_to_Modify_sequence_of[1] = {
  { &hf_rnsap_TDD_DCHs_to_Modify_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_DCHs_to_ModifyItem },
};

static int
dissect_rnsap_TDD_DCHs_to_Modify(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_TDD_DCHs_to_Modify, TDD_DCHs_to_Modify_sequence_of,
                                                  1, 128);

  return offset;
}
static int dissect_id_TDD_DCHs_to_Modify(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_DCHs_to_Modify(tvb, offset, actx, tree, hf_rnsap_id_TDD_DCHs_to_Modify);
}



static int
dissect_rnsap_TransportLayerAddress(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     1, 160, TRUE, NULL);

  return offset;
}
static int dissect_id_TransportLayerAddress(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TransportLayerAddress(tvb, offset, actx, tree, hf_rnsap_id_TransportLayerAddress);
}
static int dissect_transportLayerAddress(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TransportLayerAddress(tvb, offset, actx, tree, hf_rnsap_transportLayerAddress);
}


static const per_sequence_t DCH_InformationResponseItem_sequence[] = {
  { &hf_rnsap_dCH_ID        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DCH_ID },
  { &hf_rnsap_bindingID     , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_BindingID },
  { &hf_rnsap_transportLayerAddress, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TransportLayerAddress },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DCH_InformationResponseItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DCH_InformationResponseItem, DCH_InformationResponseItem_sequence);

  return offset;
}
static int dissect_DCH_InformationResponse_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DCH_InformationResponseItem(tvb, offset, actx, tree, hf_rnsap_DCH_InformationResponse_item);
}


static const per_sequence_t DCH_InformationResponse_sequence_of[1] = {
  { &hf_rnsap_DCH_InformationResponse_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_DCH_InformationResponseItem },
};

static int
dissect_rnsap_DCH_InformationResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DCH_InformationResponse, DCH_InformationResponse_sequence_of,
                                                  1, 128);

  return offset;
}
static int dissect_id_DCH_InformationResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DCH_InformationResponse(tvb, offset, actx, tree, hf_rnsap_id_DCH_InformationResponse);
}
static int dissect_dCH_InformationResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DCH_InformationResponse(tvb, offset, actx, tree, hf_rnsap_dCH_InformationResponse);
}


static const per_sequence_t DCH_Rate_InformationItem_RL_CongestInd_sequence[] = {
  { &hf_rnsap_dCH_ID        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DCH_ID },
  { &hf_rnsap_allowed_Rate_Information, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Allowed_Rate_Information },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DCH_Rate_InformationItem_RL_CongestInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DCH_Rate_InformationItem_RL_CongestInd, DCH_Rate_InformationItem_RL_CongestInd_sequence);

  return offset;
}
static int dissect_id_DCH_Rate_InformationItem_RL_CongestInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DCH_Rate_InformationItem_RL_CongestInd(tvb, offset, actx, tree, hf_rnsap_id_DCH_Rate_InformationItem_RL_CongestInd);
}



static int
dissect_rnsap_INTEGER_0_3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 3U, NULL, FALSE);

  return offset;
}
static int dissect_uplinkNRTLoadInformationValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_0_3(tvb, offset, actx, tree, hf_rnsap_uplinkNRTLoadInformationValue);
}
static int dissect_downlinkNRTLoadInformationValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_0_3(tvb, offset, actx, tree, hf_rnsap_downlinkNRTLoadInformationValue);
}
static int dissect_ctfc2bit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_0_3(tvb, offset, actx, tree, hf_rnsap_ctfc2bit);
}



static int
dissect_rnsap_INTEGER_0_15(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 15U, NULL, FALSE);

  return offset;
}
static int dissect_burstStart(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_0_15(tvb, offset, actx, tree, hf_rnsap_burstStart);
}
static int dissect_ctfc4bit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_0_15(tvb, offset, actx, tree, hf_rnsap_ctfc4bit);
}



static int
dissect_rnsap_INTEGER_0_63(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 63U, NULL, FALSE);

  return offset;
}
static int dissect_noinitialOffset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_0_63(tvb, offset, actx, tree, hf_rnsap_noinitialOffset);
}
static int dissect_ctfc6bit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_0_63(tvb, offset, actx, tree, hf_rnsap_ctfc6bit);
}



static int
dissect_rnsap_INTEGER_0_255(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 255U, NULL, FALSE);

  return offset;
}
static int dissect_t_RLFAILURE(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_0_255(tvb, offset, actx, tree, hf_rnsap_t_RLFAILURE);
}
static int dissect_initialOffset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_0_255(tvb, offset, actx, tree, hf_rnsap_initialOffset);
}
static int dissect_ctfc8bit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_0_255(tvb, offset, actx, tree, hf_rnsap_ctfc8bit);
}



static int
dissect_rnsap_INTEGER_0_4095(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 4095U, NULL, FALSE);

  return offset;
}
static int dissect_ctfc12bit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_0_4095(tvb, offset, actx, tree, hf_rnsap_ctfc12bit);
}



static int
dissect_rnsap_INTEGER_0_16777215(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 16777215U, NULL, FALSE);

  return offset;
}
static int dissect_ctfcmaxbit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_0_16777215(tvb, offset, actx, tree, hf_rnsap_ctfcmaxbit);
}


static const value_string rnsap_TFCS_CTFC_vals[] = {
  {   0, "ctfc2bit" },
  {   1, "ctfc4bit" },
  {   2, "ctfc6bit" },
  {   3, "ctfc8bit" },
  {   4, "ctfc12bit" },
  {   5, "ctfc16bit" },
  {   6, "ctfcmaxbit" },
  { 0, NULL }
};

static const per_choice_t TFCS_CTFC_choice[] = {
  {   0, &hf_rnsap_ctfc2bit      , ASN1_NO_EXTENSIONS     , dissect_rnsap_INTEGER_0_3 },
  {   1, &hf_rnsap_ctfc4bit      , ASN1_NO_EXTENSIONS     , dissect_rnsap_INTEGER_0_15 },
  {   2, &hf_rnsap_ctfc6bit      , ASN1_NO_EXTENSIONS     , dissect_rnsap_INTEGER_0_63 },
  {   3, &hf_rnsap_ctfc8bit      , ASN1_NO_EXTENSIONS     , dissect_rnsap_INTEGER_0_255 },
  {   4, &hf_rnsap_ctfc12bit     , ASN1_NO_EXTENSIONS     , dissect_rnsap_INTEGER_0_4095 },
  {   5, &hf_rnsap_ctfc16bit     , ASN1_NO_EXTENSIONS     , dissect_rnsap_INTEGER_0_65535 },
  {   6, &hf_rnsap_ctfcmaxbit    , ASN1_NO_EXTENSIONS     , dissect_rnsap_INTEGER_0_16777215 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_TFCS_CTFC(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_TFCS_CTFC, TFCS_CTFC_choice,
                                 NULL);

  return offset;
}
static int dissect_cTFC(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TFCS_CTFC(tvb, offset, actx, tree, hf_rnsap_cTFC);
}



static int
dissect_rnsap_BetaCD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 15U, NULL, FALSE);

  return offset;
}
static int dissect_betaC(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BetaCD(tvb, offset, actx, tree, hf_rnsap_betaC);
}
static int dissect_betaD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BetaCD(tvb, offset, actx, tree, hf_rnsap_betaD);
}



static int
dissect_rnsap_RefTFCNumber(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 15U, NULL, FALSE);

  return offset;
}
static int dissect_refTFCNumber(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RefTFCNumber(tvb, offset, actx, tree, hf_rnsap_refTFCNumber);
}


static const per_sequence_t T_signalledGainFactors_sequence[] = {
  { &hf_rnsap_betaC         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BetaCD },
  { &hf_rnsap_betaD         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BetaCD },
  { &hf_rnsap_refTFCNumber  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_RefTFCNumber },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_T_signalledGainFactors(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_T_signalledGainFactors, T_signalledGainFactors_sequence);

  return offset;
}
static int dissect_signalledGainFactors(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_T_signalledGainFactors(tvb, offset, actx, tree, hf_rnsap_signalledGainFactors);
}


static const value_string rnsap_TransportFormatCombination_Beta_vals[] = {
  {   0, "signalledGainFactors" },
  {   1, "refTFCNumber" },
  { 0, NULL }
};

static const per_choice_t TransportFormatCombination_Beta_choice[] = {
  {   0, &hf_rnsap_signalledGainFactors, ASN1_EXTENSION_ROOT    , dissect_rnsap_T_signalledGainFactors },
  {   1, &hf_rnsap_refTFCNumber  , ASN1_EXTENSION_ROOT    , dissect_rnsap_RefTFCNumber },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_TransportFormatCombination_Beta(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_TransportFormatCombination_Beta, TransportFormatCombination_Beta_choice,
                                 NULL);

  return offset;
}
static int dissect_tFC_Beta(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TransportFormatCombination_Beta(tvb, offset, actx, tree, hf_rnsap_tFC_Beta);
}


static const per_sequence_t TFCS_TFCSList_item_sequence[] = {
  { &hf_rnsap_cTFC          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TFCS_CTFC },
  { &hf_rnsap_tFC_Beta      , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TransportFormatCombination_Beta },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_TFCS_TFCSList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_TFCS_TFCSList_item, TFCS_TFCSList_item_sequence);

  return offset;
}
static int dissect_TFCS_TFCSList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TFCS_TFCSList_item(tvb, offset, actx, tree, hf_rnsap_TFCS_TFCSList_item);
}


static const per_sequence_t TFCS_TFCSList_sequence_of[1] = {
  { &hf_rnsap_TFCS_TFCSList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_TFCS_TFCSList_item },
};

static int
dissect_rnsap_TFCS_TFCSList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_TFCS_TFCSList, TFCS_TFCSList_sequence_of,
                                                  1, 1024);

  return offset;
}
static int dissect_no_Split_in_TFCI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TFCS_TFCSList(tvb, offset, actx, tree, hf_rnsap_no_Split_in_TFCI);
}


static const value_string rnsap_T_tFCSvalues_vals[] = {
  {   0, "no-Split-in-TFCI" },
  {   1, "not-Used-split-in-TFCI" },
  { 0, NULL }
};

static const per_choice_t T_tFCSvalues_choice[] = {
  {   0, &hf_rnsap_no_Split_in_TFCI, ASN1_EXTENSION_ROOT    , dissect_rnsap_TFCS_TFCSList },
  {   1, &hf_rnsap_not_Used_split_in_TFCI, ASN1_EXTENSION_ROOT    , dissect_rnsap_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_T_tFCSvalues(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_T_tFCSvalues, T_tFCSvalues_choice,
                                 NULL);

  return offset;
}
static int dissect_tFCSvalues(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_T_tFCSvalues(tvb, offset, actx, tree, hf_rnsap_tFCSvalues);
}


static const per_sequence_t TFCS_sequence[] = {
  { &hf_rnsap_tFCSvalues    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_T_tFCSvalues },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_TFCS(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_TFCS, TFCS_sequence);

  return offset;
}
static int dissect_ul_TFCS(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TFCS(tvb, offset, actx, tree, hf_rnsap_ul_TFCS);
}
static int dissect_tFCS(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TFCS(tvb, offset, actx, tree, hf_rnsap_tFCS);
}
static int dissect_dl_TFCS(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TFCS(tvb, offset, actx, tree, hf_rnsap_dl_TFCS);
}


static const value_string rnsap_TFCI_Coding_vals[] = {
  {   0, "v4" },
  {   1, "v8" },
  {   2, "v16" },
  {   3, "v32" },
  { 0, NULL }
};


static int
dissect_rnsap_TFCI_Coding(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_tFCI_Coding(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TFCI_Coding(tvb, offset, actx, tree, hf_rnsap_tFCI_Coding);
}



static int
dissect_rnsap_PunctureLimit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 15U, NULL, FALSE);

  return offset;
}
static int dissect_ul_PunctureLimit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PunctureLimit(tvb, offset, actx, tree, hf_rnsap_ul_PunctureLimit);
}
static int dissect_dl_PunctureLimit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PunctureLimit(tvb, offset, actx, tree, hf_rnsap_dl_PunctureLimit);
}
static int dissect_punctureLimit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PunctureLimit(tvb, offset, actx, tree, hf_rnsap_punctureLimit);
}


static const per_sequence_t CCTrCH_TPCAddItem_RL_ReconfPrepTDD_sequence[] = {
  { &hf_rnsap_cCTrCH_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CCTrCH_ID },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_CCTrCH_TPCAddItem_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_CCTrCH_TPCAddItem_RL_ReconfPrepTDD, CCTrCH_TPCAddItem_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_CCTrCH_TPCAddList_RL_ReconfPrepTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CCTrCH_TPCAddItem_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_rnsap_CCTrCH_TPCAddList_RL_ReconfPrepTDD_item);
}


static const per_sequence_t CCTrCH_TPCAddList_RL_ReconfPrepTDD_sequence_of[1] = {
  { &hf_rnsap_CCTrCH_TPCAddList_RL_ReconfPrepTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_CCTrCH_TPCAddItem_RL_ReconfPrepTDD },
};

static int
dissect_rnsap_CCTrCH_TPCAddList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_CCTrCH_TPCAddList_RL_ReconfPrepTDD, CCTrCH_TPCAddList_RL_ReconfPrepTDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_cCTrCH_TPCList1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CCTrCH_TPCAddList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_rnsap_cCTrCH_TPCList1);
}


static const per_sequence_t DL_CCTrCH_InformationAddItem_RL_ReconfPrepTDD_sequence[] = {
  { &hf_rnsap_cCTrCH_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CCTrCH_ID },
  { &hf_rnsap_tFCS          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TFCS },
  { &hf_rnsap_tFCI_Coding   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TFCI_Coding },
  { &hf_rnsap_punctureLimit , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PunctureLimit },
  { &hf_rnsap_cCTrCH_TPCList1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_CCTrCH_TPCAddList_RL_ReconfPrepTDD },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_CCTrCH_InformationAddItem_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_CCTrCH_InformationAddItem_RL_ReconfPrepTDD, DL_CCTrCH_InformationAddItem_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_id_DL_CCTrCH_InformationAddItem_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_CCTrCH_InformationAddItem_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_rnsap_id_DL_CCTrCH_InformationAddItem_RL_ReconfPrepTDD);
}



static int
dissect_rnsap_ProtocolIE_Single_Container(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Field(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_RL_InformationList_RL_SetupRqstFDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_RL_InformationList_RL_SetupRqstFDD_item);
}
static int dissect_UL_CCTrCH_InformationList_RL_SetupRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_UL_CCTrCH_InformationList_RL_SetupRqstTDD_item);
}
static int dissect_DL_CCTrCH_InformationList_RL_SetupRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_DL_CCTrCH_InformationList_RL_SetupRqstTDD_item);
}
static int dissect_RL_InformationResponseList_RL_SetupRspFDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_RL_InformationResponseList_RL_SetupRspFDD_item);
}
static int dissect_UnsuccessfulRL_InformationResponseList_RL_SetupFailureFDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_UnsuccessfulRL_InformationResponseList_RL_SetupFailureFDD_item);
}
static int dissect_SuccessfulRL_InformationResponseList_RL_SetupFailureFDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_SuccessfulRL_InformationResponseList_RL_SetupFailureFDD_item);
}
static int dissect_RL_InformationList_RL_AdditionRqstFDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_RL_InformationList_RL_AdditionRqstFDD_item);
}
static int dissect_UL_CCTrCH_InformationList_RL_AdditionRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_UL_CCTrCH_InformationList_RL_AdditionRqstTDD_item);
}
static int dissect_DL_CCTrCH_InformationList_RL_AdditionRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_DL_CCTrCH_InformationList_RL_AdditionRqstTDD_item);
}
static int dissect_RL_InformationResponseList_RL_AdditionRspFDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_RL_InformationResponseList_RL_AdditionRspFDD_item);
}
static int dissect_UnsuccessfulRL_InformationResponseList_RL_AdditionFailureFDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_UnsuccessfulRL_InformationResponseList_RL_AdditionFailureFDD_item);
}
static int dissect_SuccessfulRL_InformationResponseList_RL_AdditionFailureFDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_SuccessfulRL_InformationResponseList_RL_AdditionFailureFDD_item);
}
static int dissect_RL_InformationList_RL_DeletionRqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_RL_InformationList_RL_DeletionRqst_item);
}
static int dissect_RL_InformationList_RL_ReconfPrepFDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_RL_InformationList_RL_ReconfPrepFDD_item);
}
static int dissect_UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD_item);
}
static int dissect_UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD_item);
}
static int dissect_UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD_item);
}
static int dissect_DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD_item);
}
static int dissect_DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD_item);
}
static int dissect_DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD_item);
}
static int dissect_RL_InformationResponseList_RL_ReconfReadyFDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_RL_InformationResponseList_RL_ReconfReadyFDD_item);
}
static int dissect_RL_ReconfigurationFailureList_RL_ReconfFailure_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_RL_ReconfigurationFailureList_RL_ReconfFailure_item);
}
static int dissect_RL_ReconfigurationRequestFDD_RL_InformationList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_RL_ReconfigurationRequestFDD_RL_InformationList_item);
}
static int dissect_UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD_item);
}
static int dissect_UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD_item);
}
static int dissect_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD_item);
}
static int dissect_DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD_item);
}
static int dissect_RL_InformationResponseList_RL_ReconfRspFDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_RL_InformationResponseList_RL_ReconfRspFDD_item);
}
static int dissect_RL_InformationList_RL_FailureInd_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_RL_InformationList_RL_FailureInd_item);
}
static int dissect_RL_Set_InformationList_RL_FailureInd_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_RL_Set_InformationList_RL_FailureInd_item);
}
static int dissect_CCTrCH_InformationList_RL_FailureInd_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_CCTrCH_InformationList_RL_FailureInd_item);
}
static int dissect_RL_InformationList_RL_PreemptRequiredInd_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_RL_InformationList_RL_PreemptRequiredInd_item);
}
static int dissect_HSDSCHMacdFlowSpecificInformationList_RL_PreemptRequiredInd_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_HSDSCHMacdFlowSpecificInformationList_RL_PreemptRequiredInd_item);
}
static int dissect_EDCH_MacdFlowSpecificInformationList_RL_PreemptRequiredInd_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_EDCH_MacdFlowSpecificInformationList_RL_PreemptRequiredInd_item);
}
static int dissect_RL_InformationList_RL_RestoreInd_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_RL_InformationList_RL_RestoreInd_item);
}
static int dissect_RL_Set_InformationList_RL_RestoreInd_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_RL_Set_InformationList_RL_RestoreInd_item);
}
static int dissect_CCTrCH_InformationList_RL_RestoreInd_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_CCTrCH_InformationList_RL_RestoreInd_item);
}
static int dissect_DL_ReferencePowerInformationList_DL_PC_Rqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_DL_ReferencePowerInformationList_DL_PC_Rqst_item);
}
static int dissect_RL_InformationList_RL_CongestInd_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_RL_InformationList_RL_CongestInd_item);
}
static int dissect_DCH_Rate_Information_RL_CongestInd_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_DCH_Rate_Information_RL_CongestInd_item);
}
static int dissect_EDCH_MacdFlowSpecificInformationList_RL_CongestInd_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_EDCH_MacdFlowSpecificInformationList_RL_CongestInd_item);
}
static int dissect_RL_InformationList_DM_Rqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_RL_InformationList_DM_Rqst_item);
}
static int dissect_RL_Set_InformationList_DM_Rqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_RL_Set_InformationList_DM_Rqst_item);
}
static int dissect_RL_InformationList_DM_Rsp_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_RL_InformationList_DM_Rsp_item);
}
static int dissect_RL_Set_InformationList_DM_Rsp_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_RL_Set_InformationList_DM_Rsp_item);
}
static int dissect_RL_Unsuccessful_InformationRespList_DM_Fail_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_RL_Unsuccessful_InformationRespList_DM_Fail_item);
}
static int dissect_RL_Successful_InformationRespList_DM_Fail_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_RL_Successful_InformationRespList_DM_Fail_item);
}
static int dissect_RL_Set_Unsuccessful_InformationRespList_DM_Fail_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_RL_Set_Unsuccessful_InformationRespList_DM_Fail_item);
}
static int dissect_RL_Set_Successful_InformationRespList_DM_Fail_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_RL_Set_Successful_InformationRespList_DM_Fail_item);
}
static int dissect_RL_InformationList_DM_Rprt_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_RL_InformationList_DM_Rprt_item);
}
static int dissect_RL_Set_InformationList_DM_Rprt_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_RL_Set_InformationList_DM_Rprt_item);
}
static int dissect_RL_Unsuccessful_InformationRespList_DM_Fail_Ind_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_RL_Unsuccessful_InformationRespList_DM_Fail_Ind_item);
}
static int dissect_RL_Set_Unsuccessful_InformationRespList_DM_Fail_Ind_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_RL_Set_Unsuccessful_InformationRespList_DM_Fail_Ind_item);
}
static int dissect_ContextInfoList_Reset_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_ContextInfoList_Reset_item);
}
static int dissect_ContextGroupInfoList_Reset_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_ContextGroupInfoList_Reset_item);
}
static int dissect_DelayedActivationInformationList_RL_ActivationCmdFDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_DelayedActivationInformationList_RL_ActivationCmdFDD_item);
}
static int dissect_DelayedActivationInformationList_RL_ActivationCmdTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_DelayedActivationInformationList_RL_ActivationCmdTDD_item);
}
static int dissect_RL_ParameterUpdateIndicationFDD_RL_InformationList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_RL_ParameterUpdateIndicationFDD_RL_InformationList_item);
}
static int dissect_ListOfInterfacesToTrace_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_ListOfInterfacesToTrace_item);
}
static int dissect_Neighbouring_UMTS_CellInformation_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_rnsap_Neighbouring_UMTS_CellInformation_item);
}



static int
dissect_rnsap_DL_DPCH_InformationAddList_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_dl_DPCH_AddInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_DPCH_InformationAddList_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_dl_DPCH_AddInformation);
}



static int
dissect_rnsap_DL_DPCH_InformationModifyList_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_dl_DPCH_ModifyInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_DPCH_InformationModifyList_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_dl_DPCH_ModifyInformation);
}



static int
dissect_rnsap_DL_DPCH_InformationDeleteList_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_dl_DPCH_DeleteInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_DPCH_InformationDeleteList_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_dl_DPCH_DeleteInformation);
}


static const per_sequence_t DL_CCTrCH_InformationItem_RL_ReconfReadyTDD_sequence[] = {
  { &hf_rnsap_cCTrCH_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CCTrCH_ID },
  { &hf_rnsap_dl_DPCH_AddInformation, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DL_DPCH_InformationAddList_RL_ReconfReadyTDD },
  { &hf_rnsap_dl_DPCH_ModifyInformation, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DL_DPCH_InformationModifyList_RL_ReconfReadyTDD },
  { &hf_rnsap_dl_DPCH_DeleteInformation, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DL_DPCH_InformationDeleteList_RL_ReconfReadyTDD },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_CCTrCH_InformationItem_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_CCTrCH_InformationItem_RL_ReconfReadyTDD, DL_CCTrCH_InformationItem_RL_ReconfReadyTDD_sequence);

  return offset;
}
static int dissect_DL_CCTrCHInformationListIE_RL_ReconfReadyTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_CCTrCH_InformationItem_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_DL_CCTrCHInformationListIE_RL_ReconfReadyTDD_item);
}


static const per_sequence_t DL_CCTrCHInformationListIE_RL_ReconfReadyTDD_sequence_of[1] = {
  { &hf_rnsap_DL_CCTrCHInformationListIE_RL_ReconfReadyTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_CCTrCH_InformationItem_RL_ReconfReadyTDD },
};

static int
dissect_rnsap_DL_CCTrCHInformationListIE_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DL_CCTrCHInformationListIE_RL_ReconfReadyTDD, DL_CCTrCHInformationListIE_RL_ReconfReadyTDD_sequence_of,
                                                  0, 16);

  return offset;
}
static int dissect_id_DL_CCTrCH_InformationListIE_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_CCTrCHInformationListIE_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_id_DL_CCTrCH_InformationListIE_RL_ReconfReadyTDD);
}


static const per_sequence_t DL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD_sequence[] = {
  { &hf_rnsap_cCTrCH_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CCTrCH_ID },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD, DL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD_sequence);

  return offset;
}
static int dissect_id_DL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD(tvb, offset, actx, tree, hf_rnsap_id_DL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD);
}


static const value_string rnsap_TDD_TPC_DownlinkStepSize_vals[] = {
  {   0, "step-size1" },
  {   1, "step-size2" },
  {   2, "step-size3" },
  { 0, NULL }
};


static int
dissect_rnsap_TDD_TPC_DownlinkStepSize(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_id_TDD_TPC_DownlinkStepSize_InformationAdd_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_TPC_DownlinkStepSize(tvb, offset, actx, tree, hf_rnsap_id_TDD_TPC_DownlinkStepSize_InformationAdd_RL_ReconfPrepTDD);
}
static int dissect_id_TDD_TPC_DownlinkStepSize_InformationModify_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_TPC_DownlinkStepSize(tvb, offset, actx, tree, hf_rnsap_id_TDD_TPC_DownlinkStepSize_InformationModify_RL_ReconfPrepTDD);
}
static int dissect_tdd_TPC_DownlinkStepSize(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_TPC_DownlinkStepSize(tvb, offset, actx, tree, hf_rnsap_tdd_TPC_DownlinkStepSize);
}
static int dissect_downlinkStepSize(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_TPC_DownlinkStepSize(tvb, offset, actx, tree, hf_rnsap_downlinkStepSize);
}


static const per_sequence_t CCTrCH_TPCItem_RL_SetupRqstTDD_sequence[] = {
  { &hf_rnsap_cCTrCH_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CCTrCH_ID },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_CCTrCH_TPCItem_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_CCTrCH_TPCItem_RL_SetupRqstTDD, CCTrCH_TPCItem_RL_SetupRqstTDD_sequence);

  return offset;
}
static int dissect_CCTrCH_TPCList_RL_SetupRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CCTrCH_TPCItem_RL_SetupRqstTDD(tvb, offset, actx, tree, hf_rnsap_CCTrCH_TPCList_RL_SetupRqstTDD_item);
}


static const per_sequence_t CCTrCH_TPCList_RL_SetupRqstTDD_sequence_of[1] = {
  { &hf_rnsap_CCTrCH_TPCList_RL_SetupRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_CCTrCH_TPCItem_RL_SetupRqstTDD },
};

static int
dissect_rnsap_CCTrCH_TPCList_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_CCTrCH_TPCList_RL_SetupRqstTDD, CCTrCH_TPCList_RL_SetupRqstTDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_cCTrCH_TPCList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CCTrCH_TPCList_RL_SetupRqstTDD(tvb, offset, actx, tree, hf_rnsap_cCTrCH_TPCList);
}


static const per_sequence_t DL_CCTrCH_InformationItem_RL_SetupRqstTDD_sequence[] = {
  { &hf_rnsap_cCTrCH_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CCTrCH_ID },
  { &hf_rnsap_dl_TFCS       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TFCS },
  { &hf_rnsap_tFCI_Coding   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TFCI_Coding },
  { &hf_rnsap_dl_PunctureLimit, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PunctureLimit },
  { &hf_rnsap_tdd_TPC_DownlinkStepSize, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_TPC_DownlinkStepSize },
  { &hf_rnsap_cCTrCH_TPCList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_CCTrCH_TPCList_RL_SetupRqstTDD },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_CCTrCH_InformationItem_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_CCTrCH_InformationItem_RL_SetupRqstTDD, DL_CCTrCH_InformationItem_RL_SetupRqstTDD_sequence);

  return offset;
}
static int dissect_id_DL_CCTrCH_InformationItem_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_CCTrCH_InformationItem_RL_SetupRqstTDD(tvb, offset, actx, tree, hf_rnsap_id_DL_CCTrCH_InformationItem_RL_SetupRqstTDD);
}
static int dissect_id_DL_CCTrCH_InformationListIE_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_CCTrCH_InformationItem_RL_SetupRqstTDD(tvb, offset, actx, tree, hf_rnsap_id_DL_CCTrCH_InformationListIE_RL_SetupRspTDD);
}



static int
dissect_rnsap_DL_DPCH_InformationList_PhyChReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_dl_DPCH_Information2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_DPCH_InformationList_PhyChReconfRqstTDD(tvb, offset, actx, tree, hf_rnsap_dl_DPCH_Information2);
}


static const per_sequence_t DL_CCTrCH_InformationItem_PhyChReconfRqstTDD_sequence[] = {
  { &hf_rnsap_cCTrCH_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CCTrCH_ID },
  { &hf_rnsap_dl_DPCH_Information2, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_DPCH_InformationList_PhyChReconfRqstTDD },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_CCTrCH_InformationItem_PhyChReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_CCTrCH_InformationItem_PhyChReconfRqstTDD, DL_CCTrCH_InformationItem_PhyChReconfRqstTDD_sequence);

  return offset;
}
static int dissect_DL_CCTrCH_InformationListIE_PhyChReconfRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_CCTrCH_InformationItem_PhyChReconfRqstTDD(tvb, offset, actx, tree, hf_rnsap_DL_CCTrCH_InformationListIE_PhyChReconfRqstTDD_item);
}


static const per_sequence_t DL_CCTrCH_InformationListIE_PhyChReconfRqstTDD_sequence_of[1] = {
  { &hf_rnsap_DL_CCTrCH_InformationListIE_PhyChReconfRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_CCTrCH_InformationItem_PhyChReconfRqstTDD },
};

static int
dissect_rnsap_DL_CCTrCH_InformationListIE_PhyChReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DL_CCTrCH_InformationListIE_PhyChReconfRqstTDD, DL_CCTrCH_InformationListIE_PhyChReconfRqstTDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_DL_CCTrCH_InformationListIE_PhyChReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_CCTrCH_InformationListIE_PhyChReconfRqstTDD(tvb, offset, actx, tree, hf_rnsap_id_DL_CCTrCH_InformationListIE_PhyChReconfRqstTDD);
}



static int
dissect_rnsap_DL_DPCH_InformationList_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_dl_DPCH_Information1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_DPCH_InformationList_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_rnsap_dl_DPCH_Information1);
}


static const per_sequence_t DL_CCTrCHInformationItem_RL_AdditionRspTDD_sequence[] = {
  { &hf_rnsap_cCTrCH_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CCTrCH_ID },
  { &hf_rnsap_dl_DPCH_Information1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DL_DPCH_InformationList_RL_AdditionRspTDD },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_CCTrCHInformationItem_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_CCTrCHInformationItem_RL_AdditionRspTDD, DL_CCTrCHInformationItem_RL_AdditionRspTDD_sequence);

  return offset;
}
static int dissect_DL_CCTrCHInformationListIE_RL_AdditionRspTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_CCTrCHInformationItem_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_rnsap_DL_CCTrCHInformationListIE_RL_AdditionRspTDD_item);
}


static const per_sequence_t DL_CCTrCHInformationListIE_RL_AdditionRspTDD_sequence_of[1] = {
  { &hf_rnsap_DL_CCTrCHInformationListIE_RL_AdditionRspTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_CCTrCHInformationItem_RL_AdditionRspTDD },
};

static int
dissect_rnsap_DL_CCTrCHInformationListIE_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DL_CCTrCHInformationListIE_RL_AdditionRspTDD, DL_CCTrCHInformationListIE_RL_AdditionRspTDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_DL_CCTrCH_InformationListIE_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_CCTrCHInformationListIE_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_rnsap_id_DL_CCTrCH_InformationListIE_RL_AdditionRspTDD);
}


static const per_sequence_t DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD_sequence_of[1] = {
  { &hf_rnsap_DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD, DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD_sequence_of,
                                                  0, 16);

  return offset;
}
static int dissect_id_DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_rnsap_id_DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD);
}


static const per_sequence_t DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD_sequence_of[1] = {
  { &hf_rnsap_DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD, DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD_sequence_of,
                                                  0, 16);

  return offset;
}
static int dissect_id_DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD(tvb, offset, actx, tree, hf_rnsap_id_DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD);
}


static const per_sequence_t DL_CCTrCH_InformationList_RL_SetupRqstTDD_sequence_of[1] = {
  { &hf_rnsap_DL_CCTrCH_InformationList_RL_SetupRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_DL_CCTrCH_InformationList_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DL_CCTrCH_InformationList_RL_SetupRqstTDD, DL_CCTrCH_InformationList_RL_SetupRqstTDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_DL_CCTrCH_InformationList_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_CCTrCH_InformationList_RL_SetupRqstTDD(tvb, offset, actx, tree, hf_rnsap_id_DL_CCTrCH_InformationList_RL_SetupRqstTDD);
}



static int
dissect_rnsap_DL_ScramblingCode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 15U, NULL, FALSE);

  return offset;
}
static int dissect_eAGCH_ERGCH_EHICH_FDD_ScramblingCode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_ScramblingCode(tvb, offset, actx, tree, hf_rnsap_eAGCH_ERGCH_EHICH_FDD_ScramblingCode);
}
static int dissect_dl_ScramblingCode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_ScramblingCode(tvb, offset, actx, tree, hf_rnsap_dl_ScramblingCode);
}
static int dissect_hSPDSCH_and_HSSCCH_ScramblingCode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_ScramblingCode(tvb, offset, actx, tree, hf_rnsap_hSPDSCH_and_HSSCCH_ScramblingCode);
}



static int
dissect_rnsap_FDD_DL_ChannelisationCodeNumber(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 511U, NULL, FALSE);

  return offset;
}
static int dissect_eAGCH_ChannelisationCode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_FDD_DL_ChannelisationCodeNumber(tvb, offset, actx, tree, hf_rnsap_eAGCH_ChannelisationCode);
}
static int dissect_eRGCH_EHICH_ChannelisationCode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_FDD_DL_ChannelisationCodeNumber(tvb, offset, actx, tree, hf_rnsap_eRGCH_EHICH_ChannelisationCode);
}
static int dissect_fDD_DL_ChannelisationCodeNumber(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_FDD_DL_ChannelisationCodeNumber(tvb, offset, actx, tree, hf_rnsap_fDD_DL_ChannelisationCodeNumber);
}


static const value_string rnsap_Transmission_Gap_Pattern_Sequence_ScramblingCode_Information_vals[] = {
  {   0, "code-change" },
  {   1, "nocode-change" },
  { 0, NULL }
};


static int
dissect_rnsap_Transmission_Gap_Pattern_Sequence_ScramblingCode_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_transmission_Gap_Pattern_Sequence_ScramblingCode_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Transmission_Gap_Pattern_Sequence_ScramblingCode_Information(tvb, offset, actx, tree, hf_rnsap_transmission_Gap_Pattern_Sequence_ScramblingCode_Information);
}


static const per_sequence_t FDD_DL_CodeInformationItem_sequence[] = {
  { &hf_rnsap_dl_ScramblingCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_ScramblingCode },
  { &hf_rnsap_fDD_DL_ChannelisationCodeNumber, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_FDD_DL_ChannelisationCodeNumber },
  { &hf_rnsap_transmission_Gap_Pattern_Sequence_ScramblingCode_Information, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Transmission_Gap_Pattern_Sequence_ScramblingCode_Information },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_FDD_DL_CodeInformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_FDD_DL_CodeInformationItem, FDD_DL_CodeInformationItem_sequence);

  return offset;
}
static int dissect_FDD_DL_CodeInformation_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_FDD_DL_CodeInformationItem(tvb, offset, actx, tree, hf_rnsap_FDD_DL_CodeInformation_item);
}


static const per_sequence_t FDD_DL_CodeInformation_sequence_of[1] = {
  { &hf_rnsap_FDD_DL_CodeInformation_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_FDD_DL_CodeInformationItem },
};

static int
dissect_rnsap_FDD_DL_CodeInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_FDD_DL_CodeInformation, FDD_DL_CodeInformation_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_id_FDD_DL_CodeInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_FDD_DL_CodeInformation(tvb, offset, actx, tree, hf_rnsap_id_FDD_DL_CodeInformation);
}
static int dissect_dl_CodeInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_FDD_DL_CodeInformation(tvb, offset, actx, tree, hf_rnsap_dl_CodeInformation);
}



static int
dissect_rnsap_DL_DPCH_SlotFormat(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 16U, NULL, TRUE);

  return offset;
}
static int dissect_dl_DPCH_SlotFormat(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_DPCH_SlotFormat(tvb, offset, actx, tree, hf_rnsap_dl_DPCH_SlotFormat);
}



static int
dissect_rnsap_NrOfDLchannelisationcodes(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 8U, NULL, FALSE);

  return offset;
}
static int dissect_nrOfDLchannelisationcodes(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_NrOfDLchannelisationcodes(tvb, offset, actx, tree, hf_rnsap_nrOfDLchannelisationcodes);
}


static const value_string rnsap_TFCI_SignallingMode_vals[] = {
  {   0, "normal" },
  {   1, "not-Used-split" },
  { 0, NULL }
};


static int
dissect_rnsap_TFCI_SignallingMode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_tFCI_SignallingMode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TFCI_SignallingMode(tvb, offset, actx, tree, hf_rnsap_tFCI_SignallingMode);
}


static const value_string rnsap_TFCI_Presence_vals[] = {
  {   0, "present" },
  {   1, "not-present" },
  { 0, NULL }
};


static int
dissect_rnsap_TFCI_Presence(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_tFCI_Presence(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TFCI_Presence(tvb, offset, actx, tree, hf_rnsap_tFCI_Presence);
}


static const value_string rnsap_MultiplexingPosition_vals[] = {
  {   0, "fixed" },
  {   1, "flexible" },
  { 0, NULL }
};


static int
dissect_rnsap_MultiplexingPosition(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_multiplexingPosition(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MultiplexingPosition(tvb, offset, actx, tree, hf_rnsap_multiplexingPosition);
}


static const value_string rnsap_LimitedPowerIncrease_vals[] = {
  {   0, "used" },
  {   1, "not-used" },
  { 0, NULL }
};


static int
dissect_rnsap_LimitedPowerIncrease(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_limitedPowerIncrease(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_LimitedPowerIncrease(tvb, offset, actx, tree, hf_rnsap_limitedPowerIncrease);
}


static const per_sequence_t DL_DPCH_Information_RL_ReconfPrepFDD_sequence[] = {
  { &hf_rnsap_tFCS          , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TFCS },
  { &hf_rnsap_dl_DPCH_SlotFormat, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DL_DPCH_SlotFormat },
  { &hf_rnsap_nrOfDLchannelisationcodes, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_NrOfDLchannelisationcodes },
  { &hf_rnsap_tFCI_SignallingMode, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TFCI_SignallingMode },
  { &hf_rnsap_tFCI_Presence , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TFCI_Presence },
  { &hf_rnsap_multiplexingPosition, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_MultiplexingPosition },
  { &hf_rnsap_limitedPowerIncrease, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_LimitedPowerIncrease },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_DPCH_Information_RL_ReconfPrepFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_DPCH_Information_RL_ReconfPrepFDD, DL_DPCH_Information_RL_ReconfPrepFDD_sequence);

  return offset;
}
static int dissect_id_DL_DPCH_Information_RL_ReconfPrepFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_DPCH_Information_RL_ReconfPrepFDD(tvb, offset, actx, tree, hf_rnsap_id_DL_DPCH_Information_RL_ReconfPrepFDD);
}



static int
dissect_rnsap_PowerOffset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 24U, NULL, FALSE);

  return offset;
}
static int dissect_po1_ForTFCI_Bits(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PowerOffset(tvb, offset, actx, tree, hf_rnsap_po1_ForTFCI_Bits);
}
static int dissect_po2_ForTPC_Bits(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PowerOffset(tvb, offset, actx, tree, hf_rnsap_po2_ForTPC_Bits);
}
static int dissect_po3_ForPilotBits(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PowerOffset(tvb, offset, actx, tree, hf_rnsap_po3_ForPilotBits);
}
static int dissect_pO1_ForTFCI_Bits(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PowerOffset(tvb, offset, actx, tree, hf_rnsap_pO1_ForTFCI_Bits);
}
static int dissect_pO2_ForTPC_Bits(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PowerOffset(tvb, offset, actx, tree, hf_rnsap_pO2_ForTPC_Bits);
}
static int dissect_pO3_ForPilotBits(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PowerOffset(tvb, offset, actx, tree, hf_rnsap_pO3_ForPilotBits);
}


static const per_sequence_t PowerOffsetInformation_RL_SetupRqstFDD_sequence[] = {
  { &hf_rnsap_po1_ForTFCI_Bits, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PowerOffset },
  { &hf_rnsap_po2_ForTPC_Bits, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PowerOffset },
  { &hf_rnsap_po3_ForPilotBits, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PowerOffset },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_PowerOffsetInformation_RL_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_PowerOffsetInformation_RL_SetupRqstFDD, PowerOffsetInformation_RL_SetupRqstFDD_sequence);

  return offset;
}
static int dissect_powerOffsetInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PowerOffsetInformation_RL_SetupRqstFDD(tvb, offset, actx, tree, hf_rnsap_powerOffsetInformation);
}


static const value_string rnsap_FDD_TPC_DownlinkStepSize_vals[] = {
  {   0, "step-size0-5" },
  {   1, "step-size1" },
  {   2, "step-size1-5" },
  {   3, "step-size2" },
  { 0, NULL }
};


static int
dissect_rnsap_FDD_TPC_DownlinkStepSize(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_fdd_dl_TPC_DownlinkStepSize(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_FDD_TPC_DownlinkStepSize(tvb, offset, actx, tree, hf_rnsap_fdd_dl_TPC_DownlinkStepSize);
}
static int dissect_fdd_TPC_DownlinkStepSize(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_FDD_TPC_DownlinkStepSize(tvb, offset, actx, tree, hf_rnsap_fdd_TPC_DownlinkStepSize);
}


static const value_string rnsap_InnerLoopDLPCStatus_vals[] = {
  {   0, "active" },
  {   1, "inactive" },
  { 0, NULL }
};


static int
dissect_rnsap_InnerLoopDLPCStatus(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_InnerLoopDLPCStatus(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_InnerLoopDLPCStatus(tvb, offset, actx, tree, hf_rnsap_id_InnerLoopDLPCStatus);
}
static int dissect_innerLoopDLPCStatus(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_InnerLoopDLPCStatus(tvb, offset, actx, tree, hf_rnsap_innerLoopDLPCStatus);
}


static const per_sequence_t DL_DPCH_Information_RL_SetupRqstFDD_sequence[] = {
  { &hf_rnsap_tFCS          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TFCS },
  { &hf_rnsap_dl_DPCH_SlotFormat, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_DPCH_SlotFormat },
  { &hf_rnsap_nrOfDLchannelisationcodes, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_NrOfDLchannelisationcodes },
  { &hf_rnsap_tFCI_SignallingMode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TFCI_SignallingMode },
  { &hf_rnsap_tFCI_Presence , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TFCI_Presence },
  { &hf_rnsap_multiplexingPosition, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MultiplexingPosition },
  { &hf_rnsap_powerOffsetInformation, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PowerOffsetInformation_RL_SetupRqstFDD },
  { &hf_rnsap_fdd_dl_TPC_DownlinkStepSize, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_FDD_TPC_DownlinkStepSize },
  { &hf_rnsap_limitedPowerIncrease, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_LimitedPowerIncrease },
  { &hf_rnsap_innerLoopDLPCStatus, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_InnerLoopDLPCStatus },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_DPCH_Information_RL_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_DPCH_Information_RL_SetupRqstFDD, DL_DPCH_Information_RL_SetupRqstFDD_sequence);

  return offset;
}
static int dissect_id_DL_DPCH_Information_RL_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_DPCH_Information_RL_SetupRqstFDD(tvb, offset, actx, tree, hf_rnsap_id_DL_DPCH_Information_RL_SetupRqstFDD);
}


static const per_sequence_t DL_DPCH_Information_RL_ReconfRqstFDD_sequence[] = {
  { &hf_rnsap_tFCS          , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TFCS },
  { &hf_rnsap_tFCI_SignallingMode, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TFCI_SignallingMode },
  { &hf_rnsap_limitedPowerIncrease, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_LimitedPowerIncrease },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_DPCH_Information_RL_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_DPCH_Information_RL_ReconfRqstFDD, DL_DPCH_Information_RL_ReconfRqstFDD_sequence);

  return offset;
}
static int dissect_id_DL_DPCH_Information_RL_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_DPCH_Information_RL_ReconfRqstFDD(tvb, offset, actx, tree, hf_rnsap_id_DL_DPCH_Information_RL_ReconfRqstFDD);
}


static const value_string rnsap_RepetitionPeriod_vals[] = {
  {   0, "v1" },
  {   1, "v2" },
  {   2, "v4" },
  {   3, "v8" },
  {   4, "v16" },
  {   5, "v32" },
  {   6, "v64" },
  { 0, NULL }
};


static int
dissect_rnsap_RepetitionPeriod(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     7, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_repetitionPeriod(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RepetitionPeriod(tvb, offset, actx, tree, hf_rnsap_repetitionPeriod);
}



static int
dissect_rnsap_RepetitionLength(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 63U, NULL, FALSE);

  return offset;
}
static int dissect_repetitionLength(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RepetitionLength(tvb, offset, actx, tree, hf_rnsap_repetitionLength);
}


static const value_string rnsap_TDD_DPCHOffset_vals[] = {
  {   0, "initialOffset" },
  {   1, "noinitialOffset" },
  { 0, NULL }
};

static const per_choice_t TDD_DPCHOffset_choice[] = {
  {   0, &hf_rnsap_initialOffset , ASN1_NO_EXTENSIONS     , dissect_rnsap_INTEGER_0_255 },
  {   1, &hf_rnsap_noinitialOffset, ASN1_NO_EXTENSIONS     , dissect_rnsap_INTEGER_0_63 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_TDD_DPCHOffset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_TDD_DPCHOffset, TDD_DPCHOffset_choice,
                                 NULL);

  return offset;
}
static int dissect_tDD_DPCHOffset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_DPCHOffset(tvb, offset, actx, tree, hf_rnsap_tDD_DPCHOffset);
}



static int
dissect_rnsap_TimeSlot(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 14U, NULL, FALSE);

  return offset;
}
static int dissect_id_TimeSlot_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TimeSlot(tvb, offset, actx, tree, hf_rnsap_id_TimeSlot_RL_SetupRspTDD);
}
static int dissect_timeSlot(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TimeSlot(tvb, offset, actx, tree, hf_rnsap_timeSlot);
}
static int dissect_timeslot(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TimeSlot(tvb, offset, actx, tree, hf_rnsap_timeslot);
}


static const value_string rnsap_MidambleConfigurationBurstType1And3_vals[] = {
  {   0, "v4" },
  {   1, "v8" },
  {   2, "v16" },
  { 0, NULL }
};


static int
dissect_rnsap_MidambleConfigurationBurstType1And3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_midambleConfigurationBurstType1And3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MidambleConfigurationBurstType1And3(tvb, offset, actx, tree, hf_rnsap_midambleConfigurationBurstType1And3);
}



static int
dissect_rnsap_MidambleShiftLong(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 15U, NULL, FALSE);

  return offset;
}
static int dissect_ueSpecificMidamble(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MidambleShiftLong(tvb, offset, actx, tree, hf_rnsap_ueSpecificMidamble);
}
static int dissect_midambleShift(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MidambleShiftLong(tvb, offset, actx, tree, hf_rnsap_midambleShift);
}


static const value_string rnsap_T_midambleAllocationMode_vals[] = {
  {   0, "defaultMidamble" },
  {   1, "commonMidamble" },
  {   2, "ueSpecificMidamble" },
  { 0, NULL }
};

static const per_choice_t T_midambleAllocationMode_choice[] = {
  {   0, &hf_rnsap_defaultMidamble, ASN1_EXTENSION_ROOT    , dissect_rnsap_NULL },
  {   1, &hf_rnsap_commonMidamble, ASN1_EXTENSION_ROOT    , dissect_rnsap_NULL },
  {   2, &hf_rnsap_ueSpecificMidamble, ASN1_EXTENSION_ROOT    , dissect_rnsap_MidambleShiftLong },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_T_midambleAllocationMode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_T_midambleAllocationMode, T_midambleAllocationMode_choice,
                                 NULL);

  return offset;
}
static int dissect_midambleAllocationMode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_T_midambleAllocationMode(tvb, offset, actx, tree, hf_rnsap_midambleAllocationMode);
}


static const per_sequence_t T_type1_sequence[] = {
  { &hf_rnsap_midambleConfigurationBurstType1And3, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MidambleConfigurationBurstType1And3 },
  { &hf_rnsap_midambleAllocationMode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_T_midambleAllocationMode },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_T_type1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_T_type1, T_type1_sequence);

  return offset;
}
static int dissect_type1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_T_type1(tvb, offset, actx, tree, hf_rnsap_type1);
}


static const value_string rnsap_MidambleConfigurationBurstType2_vals[] = {
  {   0, "v3" },
  {   1, "v6" },
  { 0, NULL }
};


static int
dissect_rnsap_MidambleConfigurationBurstType2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_midambleConfigurationBurstType2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MidambleConfigurationBurstType2(tvb, offset, actx, tree, hf_rnsap_midambleConfigurationBurstType2);
}



static int
dissect_rnsap_MidambleShiftShort(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 5U, NULL, FALSE);

  return offset;
}
static int dissect_ueSpecificMidamble1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MidambleShiftShort(tvb, offset, actx, tree, hf_rnsap_ueSpecificMidamble1);
}


static const value_string rnsap_T_midambleAllocationMode1_vals[] = {
  {   0, "defaultMidamble" },
  {   1, "commonMidamble" },
  {   2, "ueSpecificMidamble" },
  { 0, NULL }
};

static const per_choice_t T_midambleAllocationMode1_choice[] = {
  {   0, &hf_rnsap_defaultMidamble, ASN1_EXTENSION_ROOT    , dissect_rnsap_NULL },
  {   1, &hf_rnsap_commonMidamble, ASN1_EXTENSION_ROOT    , dissect_rnsap_NULL },
  {   2, &hf_rnsap_ueSpecificMidamble1, ASN1_EXTENSION_ROOT    , dissect_rnsap_MidambleShiftShort },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_T_midambleAllocationMode1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_T_midambleAllocationMode1, T_midambleAllocationMode1_choice,
                                 NULL);

  return offset;
}
static int dissect_midambleAllocationMode1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_T_midambleAllocationMode1(tvb, offset, actx, tree, hf_rnsap_midambleAllocationMode1);
}


static const per_sequence_t T_type2_sequence[] = {
  { &hf_rnsap_midambleConfigurationBurstType2, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MidambleConfigurationBurstType2 },
  { &hf_rnsap_midambleAllocationMode1, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_T_midambleAllocationMode1 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_T_type2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_T_type2, T_type2_sequence);

  return offset;
}
static int dissect_type2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_T_type2(tvb, offset, actx, tree, hf_rnsap_type2);
}


static const value_string rnsap_T_midambleAllocationMode2_vals[] = {
  {   0, "defaultMidamble" },
  {   1, "ueSpecificMidamble" },
  { 0, NULL }
};

static const per_choice_t T_midambleAllocationMode2_choice[] = {
  {   0, &hf_rnsap_defaultMidamble, ASN1_EXTENSION_ROOT    , dissect_rnsap_NULL },
  {   1, &hf_rnsap_ueSpecificMidamble, ASN1_EXTENSION_ROOT    , dissect_rnsap_MidambleShiftLong },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_T_midambleAllocationMode2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_T_midambleAllocationMode2, T_midambleAllocationMode2_choice,
                                 NULL);

  return offset;
}
static int dissect_midambleAllocationMode2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_T_midambleAllocationMode2(tvb, offset, actx, tree, hf_rnsap_midambleAllocationMode2);
}


static const per_sequence_t T_type3_sequence[] = {
  { &hf_rnsap_midambleConfigurationBurstType1And3, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MidambleConfigurationBurstType1And3 },
  { &hf_rnsap_midambleAllocationMode2, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_T_midambleAllocationMode2 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_T_type3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_T_type3, T_type3_sequence);

  return offset;
}
static int dissect_type3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_T_type3(tvb, offset, actx, tree, hf_rnsap_type3);
}


static const value_string rnsap_MidambleShiftAndBurstType_vals[] = {
  {   0, "type1" },
  {   1, "type2" },
  {   2, "type3" },
  { 0, NULL }
};

static const per_choice_t MidambleShiftAndBurstType_choice[] = {
  {   0, &hf_rnsap_type1         , ASN1_EXTENSION_ROOT    , dissect_rnsap_T_type1 },
  {   1, &hf_rnsap_type2         , ASN1_EXTENSION_ROOT    , dissect_rnsap_T_type2 },
  {   2, &hf_rnsap_type3         , ASN1_EXTENSION_ROOT    , dissect_rnsap_T_type3 },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_MidambleShiftAndBurstType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_MidambleShiftAndBurstType, MidambleShiftAndBurstType_choice,
                                 NULL);

  return offset;
}
static int dissect_midambleShiftAndBurstType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MidambleShiftAndBurstType(tvb, offset, actx, tree, hf_rnsap_midambleShiftAndBurstType);
}



static int
dissect_rnsap_DPCH_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 239U, NULL, FALSE);

  return offset;
}
static int dissect_dPCH_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DPCH_ID(tvb, offset, actx, tree, hf_rnsap_dPCH_ID);
}


static const value_string rnsap_TDD_ChannelisationCode_vals[] = {
  {   0, "chCode1div1" },
  {   1, "chCode2div1" },
  {   2, "chCode2div2" },
  {   3, "chCode4div1" },
  {   4, "chCode4div2" },
  {   5, "chCode4div3" },
  {   6, "chCode4div4" },
  {   7, "chCode8div1" },
  {   8, "chCode8div2" },
  {   9, "chCode8div3" },
  {  10, "chCode8div4" },
  {  11, "chCode8div5" },
  {  12, "chCode8div6" },
  {  13, "chCode8div7" },
  {  14, "chCode8div8" },
  {  15, "chCode16div1" },
  {  16, "chCode16div2" },
  {  17, "chCode16div3" },
  {  18, "chCode16div4" },
  {  19, "chCode16div5" },
  {  20, "chCode16div6" },
  {  21, "chCode16div7" },
  {  22, "chCode16div8" },
  {  23, "chCode16div9" },
  {  24, "chCode16div10" },
  {  25, "chCode16div11" },
  {  26, "chCode16div12" },
  {  27, "chCode16div13" },
  {  28, "chCode16div14" },
  {  29, "chCode16div15" },
  {  30, "chCode16div16" },
  { 0, NULL }
};


static int
dissect_rnsap_TDD_ChannelisationCode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     31, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_tDD_ChannelisationCode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_ChannelisationCode(tvb, offset, actx, tree, hf_rnsap_tDD_ChannelisationCode);
}
static int dissect_first_TDD_ChannelisationCode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_ChannelisationCode(tvb, offset, actx, tree, hf_rnsap_first_TDD_ChannelisationCode);
}
static int dissect_second_TDD_ChannelisationCode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_ChannelisationCode(tvb, offset, actx, tree, hf_rnsap_second_TDD_ChannelisationCode);
}


static const per_sequence_t TDD_DL_Code_InformationItem_sequence[] = {
  { &hf_rnsap_dPCH_ID       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DPCH_ID },
  { &hf_rnsap_tDD_ChannelisationCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_ChannelisationCode },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_TDD_DL_Code_InformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_TDD_DL_Code_InformationItem, TDD_DL_Code_InformationItem_sequence);

  return offset;
}
static int dissect_TDD_DL_Code_Information_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_DL_Code_InformationItem(tvb, offset, actx, tree, hf_rnsap_TDD_DL_Code_Information_item);
}


static const per_sequence_t TDD_DL_Code_Information_sequence_of[1] = {
  { &hf_rnsap_TDD_DL_Code_Information_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_DL_Code_InformationItem },
};

static int
dissect_rnsap_TDD_DL_Code_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_TDD_DL_Code_Information, TDD_DL_Code_Information_sequence_of,
                                                  1, 240);

  return offset;
}
static int dissect_dL_Code_Information1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_DL_Code_Information(tvb, offset, actx, tree, hf_rnsap_dL_Code_Information1);
}


static const per_sequence_t DL_Timeslot_InformationItem_PhyChReconfRqstTDD_sequence[] = {
  { &hf_rnsap_timeSlot      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TimeSlot },
  { &hf_rnsap_midambleShiftAndBurstType, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_MidambleShiftAndBurstType },
  { &hf_rnsap_tFCI_Presence , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TFCI_Presence },
  { &hf_rnsap_dL_Code_Information1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TDD_DL_Code_Information },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_Timeslot_InformationItem_PhyChReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_Timeslot_InformationItem_PhyChReconfRqstTDD, DL_Timeslot_InformationItem_PhyChReconfRqstTDD_sequence);

  return offset;
}
static int dissect_DL_Timeslot_InformationList_PhyChReconfRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_Timeslot_InformationItem_PhyChReconfRqstTDD(tvb, offset, actx, tree, hf_rnsap_DL_Timeslot_InformationList_PhyChReconfRqstTDD_item);
}


static const per_sequence_t DL_Timeslot_InformationList_PhyChReconfRqstTDD_sequence_of[1] = {
  { &hf_rnsap_DL_Timeslot_InformationList_PhyChReconfRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_Timeslot_InformationItem_PhyChReconfRqstTDD },
};

static int
dissect_rnsap_DL_Timeslot_InformationList_PhyChReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DL_Timeslot_InformationList_PhyChReconfRqstTDD, DL_Timeslot_InformationList_PhyChReconfRqstTDD_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_dL_Timeslot_InformationList_PhyChReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_Timeslot_InformationList_PhyChReconfRqstTDD(tvb, offset, actx, tree, hf_rnsap_dL_Timeslot_InformationList_PhyChReconfRqstTDD);
}


static const per_sequence_t DL_DPCH_InformationItem_PhyChReconfRqstTDD_sequence[] = {
  { &hf_rnsap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_RepetitionPeriod },
  { &hf_rnsap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_RepetitionLength },
  { &hf_rnsap_tDD_DPCHOffset, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TDD_DPCHOffset },
  { &hf_rnsap_dL_Timeslot_InformationList_PhyChReconfRqstTDD, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DL_Timeslot_InformationList_PhyChReconfRqstTDD },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_DPCH_InformationItem_PhyChReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_DPCH_InformationItem_PhyChReconfRqstTDD, DL_DPCH_InformationItem_PhyChReconfRqstTDD_sequence);

  return offset;
}
static int dissect_id_DL_DPCH_InformationItem_PhyChReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_DPCH_InformationItem_PhyChReconfRqstTDD(tvb, offset, actx, tree, hf_rnsap_id_DL_DPCH_InformationItem_PhyChReconfRqstTDD);
}


static const per_sequence_t DL_Timeslot_InformationItem_sequence[] = {
  { &hf_rnsap_timeSlot      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TimeSlot },
  { &hf_rnsap_midambleShiftAndBurstType, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MidambleShiftAndBurstType },
  { &hf_rnsap_tFCI_Presence , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TFCI_Presence },
  { &hf_rnsap_dL_Code_Information1, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_DL_Code_Information },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_Timeslot_InformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_Timeslot_InformationItem, DL_Timeslot_InformationItem_sequence);

  return offset;
}
static int dissect_DL_Timeslot_Information_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_Timeslot_InformationItem(tvb, offset, actx, tree, hf_rnsap_DL_Timeslot_Information_item);
}


static const per_sequence_t DL_Timeslot_Information_sequence_of[1] = {
  { &hf_rnsap_DL_Timeslot_Information_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_Timeslot_InformationItem },
};

static int
dissect_rnsap_DL_Timeslot_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DL_Timeslot_Information, DL_Timeslot_Information_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_dL_Timeslot_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_Timeslot_Information(tvb, offset, actx, tree, hf_rnsap_dL_Timeslot_Information);
}


static const per_sequence_t DL_DPCH_InformationItem_RL_AdditionRspTDD_sequence[] = {
  { &hf_rnsap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RepetitionPeriod },
  { &hf_rnsap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RepetitionLength },
  { &hf_rnsap_tDD_DPCHOffset, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_DPCHOffset },
  { &hf_rnsap_dL_Timeslot_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_Timeslot_Information },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_DPCH_InformationItem_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_DPCH_InformationItem_RL_AdditionRspTDD, DL_DPCH_InformationItem_RL_AdditionRspTDD_sequence);

  return offset;
}
static int dissect_id_DL_DPCH_InformationItem_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_DPCH_InformationItem_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_rnsap_id_DL_DPCH_InformationItem_RL_AdditionRspTDD);
}


static const per_sequence_t DL_DPCH_InformationItem_RL_SetupRspTDD_sequence[] = {
  { &hf_rnsap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RepetitionPeriod },
  { &hf_rnsap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RepetitionLength },
  { &hf_rnsap_tDD_DPCHOffset, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_DPCHOffset },
  { &hf_rnsap_dL_Timeslot_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_Timeslot_Information },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_DPCH_InformationItem_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_DPCH_InformationItem_RL_SetupRspTDD, DL_DPCH_InformationItem_RL_SetupRspTDD_sequence);

  return offset;
}
static int dissect_id_DL_DPCH_InformationItem_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_DPCH_InformationItem_RL_SetupRspTDD(tvb, offset, actx, tree, hf_rnsap_id_DL_DPCH_InformationItem_RL_SetupRspTDD);
}


static const value_string rnsap_DL_DPCH_TimingAdjustment_vals[] = {
  {   0, "timing-advance" },
  {   1, "timing-delay" },
  { 0, NULL }
};


static int
dissect_rnsap_DL_DPCH_TimingAdjustment(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_DL_DPCH_TimingAdjustment(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_DPCH_TimingAdjustment(tvb, offset, actx, tree, hf_rnsap_id_DL_DPCH_TimingAdjustment);
}
static int dissect_id_Initial_DL_DPCH_TimingAdjustment(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_DPCH_TimingAdjustment(tvb, offset, actx, tree, hf_rnsap_id_Initial_DL_DPCH_TimingAdjustment);
}



static int
dissect_rnsap_DL_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              -350, 150U, NULL, FALSE);

  return offset;
}
static int dissect_id_DLReferencePower(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_Power(tvb, offset, actx, tree, hf_rnsap_id_DLReferencePower);
}
static int dissect_id_CCTrCH_Maximum_DL_Power_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_Power(tvb, offset, actx, tree, hf_rnsap_id_CCTrCH_Maximum_DL_Power_RL_SetupRspTDD);
}
static int dissect_id_CCTrCH_Minimum_DL_Power_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_Power(tvb, offset, actx, tree, hf_rnsap_id_CCTrCH_Minimum_DL_Power_RL_SetupRspTDD);
}
static int dissect_id_CCTrCH_Maximum_DL_Power_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_Power(tvb, offset, actx, tree, hf_rnsap_id_CCTrCH_Maximum_DL_Power_RL_AdditionRspTDD);
}
static int dissect_id_CCTrCH_Minimum_DL_Power_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_Power(tvb, offset, actx, tree, hf_rnsap_id_CCTrCH_Minimum_DL_Power_RL_AdditionRspTDD);
}
static int dissect_id_CCTrCH_Maximum_DL_Power_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_Power(tvb, offset, actx, tree, hf_rnsap_id_CCTrCH_Maximum_DL_Power_RL_ReconfReadyTDD);
}
static int dissect_id_CCTrCH_Minimum_DL_Power_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_Power(tvb, offset, actx, tree, hf_rnsap_id_CCTrCH_Minimum_DL_Power_RL_ReconfReadyTDD);
}
static int dissect_id_Maximum_DL_Power_TimeslotLCR_InformationModifyItem_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_Power(tvb, offset, actx, tree, hf_rnsap_id_Maximum_DL_Power_TimeslotLCR_InformationModifyItem_RL_ReconfReadyTDD);
}
static int dissect_id_Minimum_DL_Power_TimeslotLCR_InformationModifyItem_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_Power(tvb, offset, actx, tree, hf_rnsap_id_Minimum_DL_Power_TimeslotLCR_InformationModifyItem_RL_ReconfReadyTDD);
}
static int dissect_id_Maximum_DL_Power_TimeslotLCR_InformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_Power(tvb, offset, actx, tree, hf_rnsap_id_Maximum_DL_Power_TimeslotLCR_InformationItem);
}
static int dissect_id_Minimum_DL_Power_TimeslotLCR_InformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_Power(tvb, offset, actx, tree, hf_rnsap_id_Minimum_DL_Power_TimeslotLCR_InformationItem);
}
static int dissect_dl_InitialTX_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_Power(tvb, offset, actx, tree, hf_rnsap_dl_InitialTX_Power);
}
static int dissect_maximumDLTxPower(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_Power(tvb, offset, actx, tree, hf_rnsap_maximumDLTxPower);
}
static int dissect_minimumDLTxPower(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_Power(tvb, offset, actx, tree, hf_rnsap_minimumDLTxPower);
}
static int dissect_cCTrCH_Maximum_DL_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_Power(tvb, offset, actx, tree, hf_rnsap_cCTrCH_Maximum_DL_Power);
}
static int dissect_cCTrCH_Minimum_DL_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_Power(tvb, offset, actx, tree, hf_rnsap_cCTrCH_Minimum_DL_Power);
}
static int dissect_maxPowerLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_Power(tvb, offset, actx, tree, hf_rnsap_maxPowerLCR);
}
static int dissect_minPowerLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_Power(tvb, offset, actx, tree, hf_rnsap_minPowerLCR);
}
static int dissect_dl_Reference_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_Power(tvb, offset, actx, tree, hf_rnsap_dl_Reference_Power);
}
static int dissect_initial_dl_tx_power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_Power(tvb, offset, actx, tree, hf_rnsap_initial_dl_tx_power);
}
static int dissect_dLReferencePower(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_Power(tvb, offset, actx, tree, hf_rnsap_dLReferencePower);
}
static int dissect_common_DL_ReferencePowerInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_Power(tvb, offset, actx, tree, hf_rnsap_common_DL_ReferencePowerInformation);
}


static const per_sequence_t DL_ReferencePowerInformationList_DL_PC_Rqst_sequence_of[1] = {
  { &hf_rnsap_DL_ReferencePowerInformationList_DL_PC_Rqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_DL_ReferencePowerInformationList_DL_PC_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DL_ReferencePowerInformationList_DL_PC_Rqst, DL_ReferencePowerInformationList_DL_PC_Rqst_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_DLReferencePowerList_DL_PC_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_ReferencePowerInformationList_DL_PC_Rqst(tvb, offset, actx, tree, hf_rnsap_id_DLReferencePowerList_DL_PC_Rqst);
}



static int
dissect_rnsap_RL_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 31U, NULL, FALSE);

  return offset;
}
static int dissect_id_HSPDSCH_RL_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_ID(tvb, offset, actx, tree, hf_rnsap_id_HSPDSCH_RL_ID);
}
static int dissect_id_PDSCH_RL_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_ID(tvb, offset, actx, tree, hf_rnsap_id_PDSCH_RL_ID);
}
static int dissect_rL_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_ID(tvb, offset, actx, tree, hf_rnsap_rL_ID);
}
static int dissect_e_DCH_Serving_RL_Id(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_ID(tvb, offset, actx, tree, hf_rnsap_e_DCH_Serving_RL_Id);
}


static const per_sequence_t DL_ReferencePowerInformation_DL_PC_Rqst_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_dl_Reference_Power, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_Power },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_ReferencePowerInformation_DL_PC_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_ReferencePowerInformation_DL_PC_Rqst, DL_ReferencePowerInformation_DL_PC_Rqst_sequence);

  return offset;
}
static int dissect_id_DL_ReferencePowerInformation_DL_PC_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_ReferencePowerInformation_DL_PC_Rqst(tvb, offset, actx, tree, hf_rnsap_id_DL_ReferencePowerInformation_DL_PC_Rqst);
}


static const value_string rnsap_DPC_Mode_vals[] = {
  {   0, "mode0" },
  {   1, "mode1" },
  { 0, NULL }
};


static int
dissect_rnsap_DPC_Mode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_id_DPC_Mode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DPC_Mode(tvb, offset, actx, tree, hf_rnsap_id_DPC_Mode);
}



static int
dissect_rnsap_DRXCycleLengthCoefficient(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              3U, 9U, NULL, FALSE);

  return offset;
}
static int dissect_id_DRXCycleLengthCoefficient(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DRXCycleLengthCoefficient(tvb, offset, actx, tree, hf_rnsap_id_DRXCycleLengthCoefficient);
}


static const per_sequence_t RL_Unsuccessful_InformationRespList_DM_Fail_Ind_sequence_of[1] = {
  { &hf_rnsap_RL_Unsuccessful_InformationRespList_DM_Fail_Ind_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_RL_Unsuccessful_InformationRespList_DM_Fail_Ind(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_RL_Unsuccessful_InformationRespList_DM_Fail_Ind, RL_Unsuccessful_InformationRespList_DM_Fail_Ind_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_rL_unsuccessful_InformationRespList_DM_Fail_Ind(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Unsuccessful_InformationRespList_DM_Fail_Ind(tvb, offset, actx, tree, hf_rnsap_rL_unsuccessful_InformationRespList_DM_Fail_Ind);
}


static const per_sequence_t RL_DM_Fail_Ind_sequence[] = {
  { &hf_rnsap_rL_unsuccessful_InformationRespList_DM_Fail_Ind, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_Unsuccessful_InformationRespList_DM_Fail_Ind },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_DM_Fail_Ind(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_DM_Fail_Ind, RL_DM_Fail_Ind_sequence);

  return offset;
}
static int dissect_rL4(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_DM_Fail_Ind(tvb, offset, actx, tree, hf_rnsap_rL4);
}
static int dissect_allRL4(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_DM_Fail_Ind(tvb, offset, actx, tree, hf_rnsap_allRL4);
}


static const per_sequence_t RL_Set_Unsuccessful_InformationRespList_DM_Fail_Ind_sequence_of[1] = {
  { &hf_rnsap_RL_Set_Unsuccessful_InformationRespList_DM_Fail_Ind_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_RL_Set_Unsuccessful_InformationRespList_DM_Fail_Ind(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_RL_Set_Unsuccessful_InformationRespList_DM_Fail_Ind, RL_Set_Unsuccessful_InformationRespList_DM_Fail_Ind_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_rL_Set_unsuccessful_InformationRespList_DM_Fail_Ind(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Set_Unsuccessful_InformationRespList_DM_Fail_Ind(tvb, offset, actx, tree, hf_rnsap_rL_Set_unsuccessful_InformationRespList_DM_Fail_Ind);
}


static const per_sequence_t RL_Set_DM_Fail_Ind_sequence[] = {
  { &hf_rnsap_rL_Set_unsuccessful_InformationRespList_DM_Fail_Ind, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_Set_Unsuccessful_InformationRespList_DM_Fail_Ind },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_Set_DM_Fail_Ind(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_Set_DM_Fail_Ind, RL_Set_DM_Fail_Ind_sequence);

  return offset;
}
static int dissect_rLS4(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Set_DM_Fail_Ind(tvb, offset, actx, tree, hf_rnsap_rLS4);
}
static int dissect_allRLS4(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Set_DM_Fail_Ind(tvb, offset, actx, tree, hf_rnsap_allRLS4);
}


static const value_string rnsap_DedicatedMeasurementObjectType_DM_Fail_Ind_vals[] = {
  {   0, "rL" },
  {   1, "rLS" },
  {   2, "allRL" },
  {   3, "allRLS" },
  { 0, NULL }
};

static const per_choice_t DedicatedMeasurementObjectType_DM_Fail_Ind_choice[] = {
  {   0, &hf_rnsap_rL4           , ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_DM_Fail_Ind },
  {   1, &hf_rnsap_rLS4          , ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_Set_DM_Fail_Ind },
  {   2, &hf_rnsap_allRL4        , ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_DM_Fail_Ind },
  {   3, &hf_rnsap_allRLS4       , ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_Set_DM_Fail_Ind },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_DedicatedMeasurementObjectType_DM_Fail_Ind(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_DedicatedMeasurementObjectType_DM_Fail_Ind, DedicatedMeasurementObjectType_DM_Fail_Ind_choice,
                                 NULL);

  return offset;
}
static int dissect_id_DedicatedMeasurementObjectType_DM_Fail_Ind(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DedicatedMeasurementObjectType_DM_Fail_Ind(tvb, offset, actx, tree, hf_rnsap_id_DedicatedMeasurementObjectType_DM_Fail_Ind);
}


static const per_sequence_t RL_Unsuccessful_InformationRespList_DM_Fail_sequence_of[1] = {
  { &hf_rnsap_RL_Unsuccessful_InformationRespList_DM_Fail_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_RL_Unsuccessful_InformationRespList_DM_Fail(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_RL_Unsuccessful_InformationRespList_DM_Fail, RL_Unsuccessful_InformationRespList_DM_Fail_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_rL_unsuccessful_InformationRespList_DM_Fail(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Unsuccessful_InformationRespList_DM_Fail(tvb, offset, actx, tree, hf_rnsap_rL_unsuccessful_InformationRespList_DM_Fail);
}


static const per_sequence_t RL_Successful_InformationRespList_DM_Fail_sequence_of[1] = {
  { &hf_rnsap_RL_Successful_InformationRespList_DM_Fail_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_RL_Successful_InformationRespList_DM_Fail(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_RL_Successful_InformationRespList_DM_Fail, RL_Successful_InformationRespList_DM_Fail_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_rL_successful_InformationRespList_DM_Fail(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Successful_InformationRespList_DM_Fail(tvb, offset, actx, tree, hf_rnsap_rL_successful_InformationRespList_DM_Fail);
}


static const per_sequence_t RL_DM_Fail_sequence[] = {
  { &hf_rnsap_rL_unsuccessful_InformationRespList_DM_Fail, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_Unsuccessful_InformationRespList_DM_Fail },
  { &hf_rnsap_rL_successful_InformationRespList_DM_Fail, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_RL_Successful_InformationRespList_DM_Fail },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_DM_Fail(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_DM_Fail, RL_DM_Fail_sequence);

  return offset;
}
static int dissect_rL3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_DM_Fail(tvb, offset, actx, tree, hf_rnsap_rL3);
}
static int dissect_allRL2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_DM_Fail(tvb, offset, actx, tree, hf_rnsap_allRL2);
}


static const per_sequence_t RL_Set_Unsuccessful_InformationRespList_DM_Fail_sequence_of[1] = {
  { &hf_rnsap_RL_Set_Unsuccessful_InformationRespList_DM_Fail_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_RL_Set_Unsuccessful_InformationRespList_DM_Fail(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_RL_Set_Unsuccessful_InformationRespList_DM_Fail, RL_Set_Unsuccessful_InformationRespList_DM_Fail_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_rL_Set_unsuccessful_InformationRespList_DM_Fail(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Set_Unsuccessful_InformationRespList_DM_Fail(tvb, offset, actx, tree, hf_rnsap_rL_Set_unsuccessful_InformationRespList_DM_Fail);
}


static const per_sequence_t RL_Set_Successful_InformationRespList_DM_Fail_sequence_of[1] = {
  { &hf_rnsap_RL_Set_Successful_InformationRespList_DM_Fail_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_RL_Set_Successful_InformationRespList_DM_Fail(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_RL_Set_Successful_InformationRespList_DM_Fail, RL_Set_Successful_InformationRespList_DM_Fail_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_rL_Set_successful_InformationRespList_DM_Fail(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Set_Successful_InformationRespList_DM_Fail(tvb, offset, actx, tree, hf_rnsap_rL_Set_successful_InformationRespList_DM_Fail);
}


static const per_sequence_t RL_Set_DM_Fail_sequence[] = {
  { &hf_rnsap_rL_Set_unsuccessful_InformationRespList_DM_Fail, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_Set_Unsuccessful_InformationRespList_DM_Fail },
  { &hf_rnsap_rL_Set_successful_InformationRespList_DM_Fail, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_RL_Set_Successful_InformationRespList_DM_Fail },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_Set_DM_Fail(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_Set_DM_Fail, RL_Set_DM_Fail_sequence);

  return offset;
}
static int dissect_rLS2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Set_DM_Fail(tvb, offset, actx, tree, hf_rnsap_rLS2);
}
static int dissect_allRLS2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Set_DM_Fail(tvb, offset, actx, tree, hf_rnsap_allRLS2);
}


static const value_string rnsap_DedicatedMeasurementObjectType_DM_Fail_vals[] = {
  {   0, "rL" },
  {   1, "rLS" },
  {   2, "allRL" },
  {   3, "allRLS" },
  { 0, NULL }
};

static const per_choice_t DedicatedMeasurementObjectType_DM_Fail_choice[] = {
  {   0, &hf_rnsap_rL3           , ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_DM_Fail },
  {   1, &hf_rnsap_rLS2          , ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_Set_DM_Fail },
  {   2, &hf_rnsap_allRL2        , ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_DM_Fail },
  {   3, &hf_rnsap_allRLS2       , ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_Set_DM_Fail },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_DedicatedMeasurementObjectType_DM_Fail(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_DedicatedMeasurementObjectType_DM_Fail, DedicatedMeasurementObjectType_DM_Fail_choice,
                                 NULL);

  return offset;
}
static int dissect_id_DedicatedMeasurementObjectType_DM_Fail(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DedicatedMeasurementObjectType_DM_Fail(tvb, offset, actx, tree, hf_rnsap_id_DedicatedMeasurementObjectType_DM_Fail);
}


static const per_sequence_t RL_InformationList_DM_Rprt_sequence_of[1] = {
  { &hf_rnsap_RL_InformationList_DM_Rprt_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_RL_InformationList_DM_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_RL_InformationList_DM_Rprt, RL_InformationList_DM_Rprt_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_rL_InformationList_DM_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_InformationList_DM_Rprt(tvb, offset, actx, tree, hf_rnsap_rL_InformationList_DM_Rprt);
}


static const per_sequence_t RL_DM_Rprt_sequence[] = {
  { &hf_rnsap_rL_InformationList_DM_Rprt, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_InformationList_DM_Rprt },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_DM_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_DM_Rprt, RL_DM_Rprt_sequence);

  return offset;
}
static int dissect_rLs1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_DM_Rprt(tvb, offset, actx, tree, hf_rnsap_rLs1);
}
static int dissect_allRL3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_DM_Rprt(tvb, offset, actx, tree, hf_rnsap_allRL3);
}


static const per_sequence_t RL_Set_InformationList_DM_Rprt_sequence_of[1] = {
  { &hf_rnsap_RL_Set_InformationList_DM_Rprt_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_RL_Set_InformationList_DM_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_RL_Set_InformationList_DM_Rprt, RL_Set_InformationList_DM_Rprt_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_rL_Set_InformationList_DM_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Set_InformationList_DM_Rprt(tvb, offset, actx, tree, hf_rnsap_rL_Set_InformationList_DM_Rprt);
}


static const per_sequence_t RL_Set_DM_Rprt_sequence[] = {
  { &hf_rnsap_rL_Set_InformationList_DM_Rprt, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_Set_InformationList_DM_Rprt },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_Set_DM_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_Set_DM_Rprt, RL_Set_DM_Rprt_sequence);

  return offset;
}
static int dissect_rLS3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Set_DM_Rprt(tvb, offset, actx, tree, hf_rnsap_rLS3);
}
static int dissect_allRLS3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Set_DM_Rprt(tvb, offset, actx, tree, hf_rnsap_allRLS3);
}


static const value_string rnsap_DedicatedMeasurementObjectType_DM_Rprt_vals[] = {
  {   0, "rLs" },
  {   1, "rLS" },
  {   2, "allRL" },
  {   3, "allRLS" },
  { 0, NULL }
};

static const per_choice_t DedicatedMeasurementObjectType_DM_Rprt_choice[] = {
  {   0, &hf_rnsap_rLs1          , ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_DM_Rprt },
  {   1, &hf_rnsap_rLS3          , ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_Set_DM_Rprt },
  {   2, &hf_rnsap_allRL3        , ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_DM_Rprt },
  {   3, &hf_rnsap_allRLS3       , ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_Set_DM_Rprt },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_DedicatedMeasurementObjectType_DM_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_DedicatedMeasurementObjectType_DM_Rprt, DedicatedMeasurementObjectType_DM_Rprt_choice,
                                 NULL);

  return offset;
}
static int dissect_id_DedicatedMeasurementObjectType_DM_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DedicatedMeasurementObjectType_DM_Rprt(tvb, offset, actx, tree, hf_rnsap_id_DedicatedMeasurementObjectType_DM_Rprt);
}


static const per_sequence_t RL_InformationList_DM_Rqst_sequence_of[1] = {
  { &hf_rnsap_RL_InformationList_DM_Rqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_RL_InformationList_DM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_RL_InformationList_DM_Rqst, RL_InformationList_DM_Rqst_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_rL_InformationList_DM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_InformationList_DM_Rqst(tvb, offset, actx, tree, hf_rnsap_rL_InformationList_DM_Rqst);
}


static const per_sequence_t RL_DM_Rqst_sequence[] = {
  { &hf_rnsap_rL_InformationList_DM_Rqst, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_InformationList_DM_Rqst },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_DM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_DM_Rqst, RL_DM_Rqst_sequence);

  return offset;
}
static int dissect_rL2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_DM_Rqst(tvb, offset, actx, tree, hf_rnsap_rL2);
}


static const per_sequence_t RL_Set_InformationList_DM_Rqst_sequence_of[1] = {
  { &hf_rnsap_RL_Set_InformationList_DM_Rqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_RL_Set_InformationList_DM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_RL_Set_InformationList_DM_Rqst, RL_Set_InformationList_DM_Rqst_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_rL_Set_InformationList_DM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Set_InformationList_DM_Rqst(tvb, offset, actx, tree, hf_rnsap_rL_Set_InformationList_DM_Rqst);
}


static const per_sequence_t RL_Set_DM_Rqst_sequence[] = {
  { &hf_rnsap_rL_Set_InformationList_DM_Rqst, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_Set_InformationList_DM_Rqst },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_Set_DM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_Set_DM_Rqst, RL_Set_DM_Rqst_sequence);

  return offset;
}
static int dissect_rLS(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Set_DM_Rqst(tvb, offset, actx, tree, hf_rnsap_rLS);
}



static int
dissect_rnsap_All_RL_DM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_null(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_allRL(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_All_RL_DM_Rqst(tvb, offset, actx, tree, hf_rnsap_allRL);
}



static int
dissect_rnsap_All_RL_Set_DM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_null(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_allRLS(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_All_RL_Set_DM_Rqst(tvb, offset, actx, tree, hf_rnsap_allRLS);
}


static const value_string rnsap_DedicatedMeasurementObjectType_DM_Rqst_vals[] = {
  {   0, "rL" },
  {   1, "rLS" },
  {   2, "allRL" },
  {   3, "allRLS" },
  { 0, NULL }
};

static const per_choice_t DedicatedMeasurementObjectType_DM_Rqst_choice[] = {
  {   0, &hf_rnsap_rL2           , ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_DM_Rqst },
  {   1, &hf_rnsap_rLS           , ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_Set_DM_Rqst },
  {   2, &hf_rnsap_allRL         , ASN1_EXTENSION_ROOT    , dissect_rnsap_All_RL_DM_Rqst },
  {   3, &hf_rnsap_allRLS        , ASN1_EXTENSION_ROOT    , dissect_rnsap_All_RL_Set_DM_Rqst },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_DedicatedMeasurementObjectType_DM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_DedicatedMeasurementObjectType_DM_Rqst, DedicatedMeasurementObjectType_DM_Rqst_choice,
                                 NULL);

  return offset;
}
static int dissect_id_DedicatedMeasurementObjectType_DM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DedicatedMeasurementObjectType_DM_Rqst(tvb, offset, actx, tree, hf_rnsap_id_DedicatedMeasurementObjectType_DM_Rqst);
}


static const per_sequence_t RL_InformationList_DM_Rsp_sequence_of[1] = {
  { &hf_rnsap_RL_InformationList_DM_Rsp_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_RL_InformationList_DM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_RL_InformationList_DM_Rsp, RL_InformationList_DM_Rsp_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_rL_InformationList_DM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_InformationList_DM_Rsp(tvb, offset, actx, tree, hf_rnsap_rL_InformationList_DM_Rsp);
}


static const per_sequence_t RL_DM_Rsp_sequence[] = {
  { &hf_rnsap_rL_InformationList_DM_Rsp, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_InformationList_DM_Rsp },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_DM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_DM_Rsp, RL_DM_Rsp_sequence);

  return offset;
}
static int dissect_rLs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_DM_Rsp(tvb, offset, actx, tree, hf_rnsap_rLs);
}
static int dissect_allRL1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_DM_Rsp(tvb, offset, actx, tree, hf_rnsap_allRL1);
}


static const per_sequence_t RL_Set_InformationList_DM_Rsp_sequence_of[1] = {
  { &hf_rnsap_RL_Set_InformationList_DM_Rsp_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_RL_Set_InformationList_DM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_RL_Set_InformationList_DM_Rsp, RL_Set_InformationList_DM_Rsp_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_rL_Set_InformationList_DM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Set_InformationList_DM_Rsp(tvb, offset, actx, tree, hf_rnsap_rL_Set_InformationList_DM_Rsp);
}


static const per_sequence_t RL_Set_DM_Rsp_sequence[] = {
  { &hf_rnsap_rL_Set_InformationList_DM_Rsp, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_Set_InformationList_DM_Rsp },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_Set_DM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_Set_DM_Rsp, RL_Set_DM_Rsp_sequence);

  return offset;
}
static int dissect_rLS1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Set_DM_Rsp(tvb, offset, actx, tree, hf_rnsap_rLS1);
}
static int dissect_allRLS1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Set_DM_Rsp(tvb, offset, actx, tree, hf_rnsap_allRLS1);
}


static const value_string rnsap_DedicatedMeasurementObjectType_DM_Rsp_vals[] = {
  {   0, "rLs" },
  {   1, "rLS" },
  {   2, "allRL" },
  {   3, "allRLS" },
  { 0, NULL }
};

static const per_choice_t DedicatedMeasurementObjectType_DM_Rsp_choice[] = {
  {   0, &hf_rnsap_rLs           , ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_DM_Rsp },
  {   1, &hf_rnsap_rLS1          , ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_Set_DM_Rsp },
  {   2, &hf_rnsap_allRL1        , ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_DM_Rsp },
  {   3, &hf_rnsap_allRLS1       , ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_Set_DM_Rsp },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_DedicatedMeasurementObjectType_DM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_DedicatedMeasurementObjectType_DM_Rsp, DedicatedMeasurementObjectType_DM_Rsp_choice,
                                 NULL);

  return offset;
}
static int dissect_id_DedicatedMeasurementObjectType_DM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DedicatedMeasurementObjectType_DM_Rsp(tvb, offset, actx, tree, hf_rnsap_id_DedicatedMeasurementObjectType_DM_Rsp);
}


static const value_string rnsap_DedicatedMeasurementType_vals[] = {
  {   0, "sir" },
  {   1, "sir-error" },
  {   2, "transmitted-code-power" },
  {   3, "rSCP" },
  {   4, "rx-timing-deviation" },
  {   5, "round-trip-time" },
  {   6, "rx-timing-deviation-LCR" },
  {   7, "angle-Of-Arrival-LCR" },
  {   8, "hs-sich-quality" },
  { 0, NULL }
};


static int
dissect_rnsap_DedicatedMeasurementType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     6, NULL, TRUE, 3, NULL);

  return offset;
}
static int dissect_id_DedicatedMeasurementType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DedicatedMeasurementType(tvb, offset, actx, tree, hf_rnsap_id_DedicatedMeasurementType);
}



static int
dissect_rnsap_FACH_FlowControlInformation_CTCH_ResourceRspFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_fACH_FlowControlInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_FACH_FlowControlInformation_CTCH_ResourceRspFDD(tvb, offset, actx, tree, hf_rnsap_fACH_FlowControlInformation);
}


static const per_sequence_t FACH_InfoForUESelectedS_CCPCH_CTCH_ResourceRspFDD_sequence[] = {
  { &hf_rnsap_fACH_FlowControlInformation, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_FACH_FlowControlInformation_CTCH_ResourceRspFDD },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_FACH_InfoForUESelectedS_CCPCH_CTCH_ResourceRspFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_FACH_InfoForUESelectedS_CCPCH_CTCH_ResourceRspFDD, FACH_InfoForUESelectedS_CCPCH_CTCH_ResourceRspFDD_sequence);

  return offset;
}
static int dissect_id_FACH_InfoForUESelectedS_CCPCH_CTCH_ResourceRspFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_FACH_InfoForUESelectedS_CCPCH_CTCH_ResourceRspFDD(tvb, offset, actx, tree, hf_rnsap_id_FACH_InfoForUESelectedS_CCPCH_CTCH_ResourceRspFDD);
}



static int
dissect_rnsap_FACH_FlowControlInformation_CTCH_ResourceRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_fACH_FlowControlInformation1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_FACH_FlowControlInformation_CTCH_ResourceRspTDD(tvb, offset, actx, tree, hf_rnsap_fACH_FlowControlInformation1);
}


static const per_sequence_t FACH_InfoForUESelectedS_CCPCH_CTCH_ResourceRspTDD_sequence[] = {
  { &hf_rnsap_fACH_FlowControlInformation1, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_FACH_FlowControlInformation_CTCH_ResourceRspTDD },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_FACH_InfoForUESelectedS_CCPCH_CTCH_ResourceRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_FACH_InfoForUESelectedS_CCPCH_CTCH_ResourceRspTDD, FACH_InfoForUESelectedS_CCPCH_CTCH_ResourceRspTDD_sequence);

  return offset;
}
static int dissect_id_FACH_InfoForUESelectedS_CCPCH_CTCH_ResourceRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_FACH_InfoForUESelectedS_CCPCH_CTCH_ResourceRspTDD(tvb, offset, actx, tree, hf_rnsap_id_FACH_InfoForUESelectedS_CCPCH_CTCH_ResourceRspTDD);
}



static int
dissect_rnsap_Guaranteed_Rate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 32U, NULL, FALSE);

  return offset;
}
static int dissect_guaranteed_UL_Rate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Guaranteed_Rate(tvb, offset, actx, tree, hf_rnsap_guaranteed_UL_Rate);
}
static int dissect_guaranteed_DL_Rate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Guaranteed_Rate(tvb, offset, actx, tree, hf_rnsap_guaranteed_DL_Rate);
}


static const per_sequence_t Guaranteed_Rate_Information_sequence[] = {
  { &hf_rnsap_guaranteed_UL_Rate, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Guaranteed_Rate },
  { &hf_rnsap_guaranteed_DL_Rate, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Guaranteed_Rate },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_Guaranteed_Rate_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_Guaranteed_Rate_Information, Guaranteed_Rate_Information_sequence);

  return offset;
}
static int dissect_id_Guaranteed_Rate_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Guaranteed_Rate_Information(tvb, offset, actx, tree, hf_rnsap_id_Guaranteed_Rate_Information);
}



static int
dissect_rnsap_IMSI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_octet_string(tvb, offset, actx, tree, hf_index,
                                       3, 8, NULL);

  return offset;
}
static int dissect_id_IMSI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_IMSI(tvb, offset, actx, tree, hf_rnsap_id_IMSI);
}
static int dissect_imsi(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_IMSI(tvb, offset, actx, tree, hf_rnsap_imsi);
}



static int
dissect_rnsap_HCS_Prio(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 7U, NULL, FALSE);

  return offset;
}
static int dissect_id_HCS_Prio(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HCS_Prio(tvb, offset, actx, tree, hf_rnsap_id_HCS_Prio);
}



static int
dissect_rnsap_L3_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
#line 84 "rnsap.cnf"

	tvbuff_t *parameter_tvb;

  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     NO_BOUND, NO_BOUND, FALSE, &parameter_tvb);


	if (!parameter_tvb)
		return offset;

	switch (ProcedureCode){

	case RNSAP_ID_DOWNLINKSIGNALLINGTRANSFER:
		actx->encoding =FALSE;
		dissect_umts_rrc_DL_DCCH_Message(parameter_tvb, 0,  actx, tree, hf_rnsap_L3_DL_DCCH_Message_PDU);
		actx->encoding =TRUE;
		break;
	case RNSAP_ID_UPLINKSIGNALLINGTRANSFER:
	default:
		break;
	}



  return offset;
}
static int dissect_id_L3_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_L3_Information(tvb, offset, actx, tree, hf_rnsap_id_L3_Information);
}



static int
dissect_rnsap_AdjustmentPeriod(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 256U, NULL, FALSE);

  return offset;
}
static int dissect_id_AdjustmentPeriod(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_AdjustmentPeriod(tvb, offset, actx, tree, hf_rnsap_id_AdjustmentPeriod);
}
static int dissect_adjustmentPeriod(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_AdjustmentPeriod(tvb, offset, actx, tree, hf_rnsap_adjustmentPeriod);
}



static int
dissect_rnsap_MaxAdjustmentStep(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 10U, NULL, FALSE);

  return offset;
}
static int dissect_id_MaxAdjustmentStep(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MaxAdjustmentStep(tvb, offset, actx, tree, hf_rnsap_id_MaxAdjustmentStep);
}
static int dissect_maxAdjustmentStep(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MaxAdjustmentStep(tvb, offset, actx, tree, hf_rnsap_maxAdjustmentStep);
}


static const value_string rnsap_MeasurementFilterCoefficient_vals[] = {
  {   0, "k0" },
  {   1, "k1" },
  {   2, "k2" },
  {   3, "k3" },
  {   4, "k4" },
  {   5, "k5" },
  {   6, "k6" },
  {   7, "k7" },
  {   8, "k8" },
  {   9, "k9" },
  {  10, "k11" },
  {  11, "k13" },
  {  12, "k15" },
  {  13, "k17" },
  {  14, "k19" },
  { 0, NULL }
};


static int
dissect_rnsap_MeasurementFilterCoefficient(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     15, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_id_MeasurementFilterCoefficient(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MeasurementFilterCoefficient(tvb, offset, actx, tree, hf_rnsap_id_MeasurementFilterCoefficient);
}



static int
dissect_rnsap_RepetitionNumber1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 256U, NULL, FALSE);

  return offset;
}
static int dissect_repetitionNumber1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RepetitionNumber1(tvb, offset, actx, tree, hf_rnsap_repetitionNumber1);
}


static const per_sequence_t MessageStructure_item_sequence[] = {
  { &hf_rnsap_iE_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_ID },
  { &hf_rnsap_repetitionNumber1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_RepetitionNumber1 },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_MessageStructure_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_MessageStructure_item, MessageStructure_item_sequence);

  return offset;
}
static int dissect_MessageStructure_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MessageStructure_item(tvb, offset, actx, tree, hf_rnsap_MessageStructure_item);
}


static const per_sequence_t MessageStructure_sequence_of[1] = {
  { &hf_rnsap_MessageStructure_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_MessageStructure_item },
};

static int
dissect_rnsap_MessageStructure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_MessageStructure, MessageStructure_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_id_MessageStructure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MessageStructure(tvb, offset, actx, tree, hf_rnsap_id_MessageStructure);
}



static int
dissect_rnsap_MeasurementID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 1048575U, NULL, FALSE);

  return offset;
}
static int dissect_id_MeasurementID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MeasurementID(tvb, offset, actx, tree, hf_rnsap_id_MeasurementID);
}



static int
dissect_rnsap_Neighbouring_GSM_CellInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_id_Neighbouring_GSM_CellInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Neighbouring_GSM_CellInformation(tvb, offset, actx, tree, hf_rnsap_id_Neighbouring_GSM_CellInformation);
}
static int dissect_neighbouring_GSM_CellInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Neighbouring_GSM_CellInformation(tvb, offset, actx, tree, hf_rnsap_neighbouring_GSM_CellInformation);
}



static int
dissect_rnsap_RNC_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 4095U, NULL, FALSE);

  return offset;
}
static int dissect_id_RNC_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RNC_ID(tvb, offset, actx, tree, hf_rnsap_id_RNC_ID);
}
static int dissect_id_SRNC_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RNC_ID(tvb, offset, actx, tree, hf_rnsap_id_SRNC_ID);
}
static int dissect_rNC_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RNC_ID(tvb, offset, actx, tree, hf_rnsap_rNC_ID);
}
static int dissect_srnc_id(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RNC_ID(tvb, offset, actx, tree, hf_rnsap_srnc_id);
}



static int
dissect_rnsap_UARFCN(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 16383U, NULL, TRUE);

  return offset;
}
static int dissect_uL_UARFCN(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UARFCN(tvb, offset, actx, tree, hf_rnsap_uL_UARFCN);
}
static int dissect_dL_UARFCN(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UARFCN(tvb, offset, actx, tree, hf_rnsap_dL_UARFCN);
}
static int dissect_uARFCNforNt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UARFCN(tvb, offset, actx, tree, hf_rnsap_uARFCNforNt);
}
static int dissect_preferredFrequencyLayer(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UARFCN(tvb, offset, actx, tree, hf_rnsap_preferredFrequencyLayer);
}
static int dissect_uARFCNforNu(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UARFCN(tvb, offset, actx, tree, hf_rnsap_uARFCNforNu);
}
static int dissect_uARFCNforNd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UARFCN(tvb, offset, actx, tree, hf_rnsap_uARFCNforNd);
}
static int dissect_uARFCN(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UARFCN(tvb, offset, actx, tree, hf_rnsap_uARFCN);
}
static int dissect_defaultPreferredFrequency(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UARFCN(tvb, offset, actx, tree, hf_rnsap_defaultPreferredFrequency);
}



static int
dissect_rnsap_FrameOffset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 255U, NULL, FALSE);

  return offset;
}
static int dissect_frameOffset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_FrameOffset(tvb, offset, actx, tree, hf_rnsap_frameOffset);
}



static int
dissect_rnsap_PrimaryScramblingCode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 511U, NULL, FALSE);

  return offset;
}
static int dissect_primaryScramblingCode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PrimaryScramblingCode(tvb, offset, actx, tree, hf_rnsap_primaryScramblingCode);
}



static int
dissect_rnsap_PrimaryCPICH_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              -100, 500U, NULL, FALSE);

  return offset;
}
static int dissect_primaryCPICH_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PrimaryCPICH_Power(tvb, offset, actx, tree, hf_rnsap_primaryCPICH_Power);
}



static int
dissect_rnsap_CellIndividualOffset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              -20, 20U, NULL, FALSE);

  return offset;
}
static int dissect_cellIndividualOffset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CellIndividualOffset(tvb, offset, actx, tree, hf_rnsap_cellIndividualOffset);
}


static const value_string rnsap_TxDiversityIndicator_vals[] = {
  {   0, "true" },
  {   1, "false" },
  { 0, NULL }
};


static int
dissect_rnsap_TxDiversityIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_txDiversityIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TxDiversityIndicator(tvb, offset, actx, tree, hf_rnsap_txDiversityIndicator);
}


static const value_string rnsap_STTD_SupportIndicator_vals[] = {
  {   0, "sTTD-Supported" },
  {   1, "sTTD-not-Supported" },
  { 0, NULL }
};


static int
dissect_rnsap_STTD_SupportIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_STTD_SupportIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_STTD_SupportIndicator(tvb, offset, actx, tree, hf_rnsap_id_STTD_SupportIndicator);
}
static int dissect_sTTD_SupportIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_STTD_SupportIndicator(tvb, offset, actx, tree, hf_rnsap_sTTD_SupportIndicator);
}


static const value_string rnsap_ClosedLoopMode1_SupportIndicator_vals[] = {
  {   0, "closedLoop-Mode1-Supported" },
  {   1, "closedLoop-Mode1-not-Supported" },
  { 0, NULL }
};


static int
dissect_rnsap_ClosedLoopMode1_SupportIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_ClosedLoopMode1_SupportIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ClosedLoopMode1_SupportIndicator(tvb, offset, actx, tree, hf_rnsap_id_ClosedLoopMode1_SupportIndicator);
}
static int dissect_closedLoopMode1_SupportIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ClosedLoopMode1_SupportIndicator(tvb, offset, actx, tree, hf_rnsap_closedLoopMode1_SupportIndicator);
}


static const per_sequence_t Neighbouring_FDD_CellInformationItem_sequence[] = {
  { &hf_rnsap_c_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_C_ID },
  { &hf_rnsap_uARFCNforNu   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UARFCN },
  { &hf_rnsap_uARFCNforNd   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UARFCN },
  { &hf_rnsap_frameOffset   , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_FrameOffset },
  { &hf_rnsap_primaryScramblingCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PrimaryScramblingCode },
  { &hf_rnsap_primaryCPICH_Power, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_PrimaryCPICH_Power },
  { &hf_rnsap_cellIndividualOffset, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_CellIndividualOffset },
  { &hf_rnsap_txDiversityIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TxDiversityIndicator },
  { &hf_rnsap_sTTD_SupportIndicator, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_STTD_SupportIndicator },
  { &hf_rnsap_closedLoopMode1_SupportIndicator, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ClosedLoopMode1_SupportIndicator },
  { &hf_rnsap_not_used_closedLoopMode2_SupportIndicator, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_NULL },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_Neighbouring_FDD_CellInformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_Neighbouring_FDD_CellInformationItem, Neighbouring_FDD_CellInformationItem_sequence);

  return offset;
}
static int dissect_Neighbouring_FDD_CellInformation_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Neighbouring_FDD_CellInformationItem(tvb, offset, actx, tree, hf_rnsap_Neighbouring_FDD_CellInformation_item);
}


static const per_sequence_t Neighbouring_FDD_CellInformation_sequence_of[1] = {
  { &hf_rnsap_Neighbouring_FDD_CellInformation_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_Neighbouring_FDD_CellInformationItem },
};

static int
dissect_rnsap_Neighbouring_FDD_CellInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_Neighbouring_FDD_CellInformation, Neighbouring_FDD_CellInformation_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_neighbouring_FDD_CellInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Neighbouring_FDD_CellInformation(tvb, offset, actx, tree, hf_rnsap_neighbouring_FDD_CellInformation);
}



static int
dissect_rnsap_CellParameterID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 127U, NULL, TRUE);

  return offset;
}
static int dissect_cellParameterID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CellParameterID(tvb, offset, actx, tree, hf_rnsap_cellParameterID);
}



static int
dissect_rnsap_SyncCase(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 2U, NULL, TRUE);

  return offset;
}
static int dissect_syncCase(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SyncCase(tvb, offset, actx, tree, hf_rnsap_syncCase);
}



static int
dissect_rnsap_SCH_TimeSlot(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 6U, NULL, FALSE);

  return offset;
}
static int dissect_sCH_TimeSlot(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SCH_TimeSlot(tvb, offset, actx, tree, hf_rnsap_sCH_TimeSlot);
}


static const value_string rnsap_SCTD_Indicator_vals[] = {
  {   0, "active" },
  {   1, "inactive" },
  { 0, NULL }
};


static int
dissect_rnsap_SCTD_Indicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_sCTD_Indicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SCTD_Indicator(tvb, offset, actx, tree, hf_rnsap_sCTD_Indicator);
}



static int
dissect_rnsap_DPCHConstantValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              -10, 10U, NULL, FALSE);

  return offset;
}
static int dissect_dPCHConstantValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DPCHConstantValue(tvb, offset, actx, tree, hf_rnsap_dPCHConstantValue);
}



static int
dissect_rnsap_PCCPCH_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              -150, 400U, NULL, TRUE);

  return offset;
}
static int dissect_pCCPCH_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PCCPCH_Power(tvb, offset, actx, tree, hf_rnsap_pCCPCH_Power);
}


static const per_sequence_t Neighbouring_TDD_CellInformationItem_sequence[] = {
  { &hf_rnsap_c_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_C_ID },
  { &hf_rnsap_uARFCNforNt   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UARFCN },
  { &hf_rnsap_frameOffset   , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_FrameOffset },
  { &hf_rnsap_cellParameterID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CellParameterID },
  { &hf_rnsap_syncCase      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SyncCase },
  { &hf_rnsap_timeSlot      , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TimeSlot },
  { &hf_rnsap_sCH_TimeSlot  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_SCH_TimeSlot },
  { &hf_rnsap_sCTD_Indicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SCTD_Indicator },
  { &hf_rnsap_cellIndividualOffset, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_CellIndividualOffset },
  { &hf_rnsap_dPCHConstantValue, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DPCHConstantValue },
  { &hf_rnsap_pCCPCH_Power  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_PCCPCH_Power },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_Neighbouring_TDD_CellInformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_Neighbouring_TDD_CellInformationItem, Neighbouring_TDD_CellInformationItem_sequence);

  return offset;
}
static int dissect_Neighbouring_TDD_CellInformation_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Neighbouring_TDD_CellInformationItem(tvb, offset, actx, tree, hf_rnsap_Neighbouring_TDD_CellInformation_item);
}


static const per_sequence_t Neighbouring_TDD_CellInformation_sequence_of[1] = {
  { &hf_rnsap_Neighbouring_TDD_CellInformation_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_Neighbouring_TDD_CellInformationItem },
};

static int
dissect_rnsap_Neighbouring_TDD_CellInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_Neighbouring_TDD_CellInformation, Neighbouring_TDD_CellInformation_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_neighbouring_TDD_CellInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Neighbouring_TDD_CellInformation(tvb, offset, actx, tree, hf_rnsap_neighbouring_TDD_CellInformation);
}


static const per_sequence_t Neighbouring_UMTS_CellInformationItem_sequence[] = {
  { &hf_rnsap_rNC_ID        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RNC_ID },
  { &hf_rnsap_cN_PS_DomainIdentifier, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_CN_PS_DomainIdentifier },
  { &hf_rnsap_cN_CS_DomainIdentifier, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_CN_CS_DomainIdentifier },
  { &hf_rnsap_neighbouring_FDD_CellInformation, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Neighbouring_FDD_CellInformation },
  { &hf_rnsap_neighbouring_TDD_CellInformation, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Neighbouring_TDD_CellInformation },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_Neighbouring_UMTS_CellInformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_Neighbouring_UMTS_CellInformationItem, Neighbouring_UMTS_CellInformationItem_sequence);

  return offset;
}
static int dissect_id_Neighbouring_UMTS_CellInformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Neighbouring_UMTS_CellInformationItem(tvb, offset, actx, tree, hf_rnsap_id_Neighbouring_UMTS_CellInformationItem);
}



static int
dissect_rnsap_NRT_Load_Information_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 3U, NULL, FALSE);

  return offset;
}
static int dissect_id_NRT_Load_Information_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_NRT_Load_Information_Value(tvb, offset, actx, tree, hf_rnsap_id_NRT_Load_Information_Value);
}



static int
dissect_rnsap_NRT_Load_Information_Value_IncrDecrThres(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 3U, NULL, FALSE);

  return offset;
}
static int dissect_id_NRT_Load_Information_Value_IncrDecrThres(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_NRT_Load_Information_Value_IncrDecrThres(tvb, offset, actx, tree, hf_rnsap_id_NRT_Load_Information_Value_IncrDecrThres);
}



static int
dissect_rnsap_URA_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 65535U, NULL, FALSE);

  return offset;
}
static int dissect_id_Old_URA_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_URA_ID(tvb, offset, actx, tree, hf_rnsap_id_Old_URA_ID);
}
static int dissect_id_URA_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_URA_ID(tvb, offset, actx, tree, hf_rnsap_id_URA_ID);
}
static int dissect_uRA_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_URA_ID(tvb, offset, actx, tree, hf_rnsap_uRA_ID);
}
static int dissect_ura_id(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_URA_ID(tvb, offset, actx, tree, hf_rnsap_ura_id);
}


static const per_sequence_t URA_PagingRqst_sequence[] = {
  { &hf_rnsap_uRA_ID        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_URA_ID },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_URA_PagingRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_URA_PagingRqst, URA_PagingRqst_sequence);

  return offset;
}
static int dissect_uRA(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_URA_PagingRqst(tvb, offset, actx, tree, hf_rnsap_uRA);
}


static const per_sequence_t Cell_PagingRqst_sequence[] = {
  { &hf_rnsap_c_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_C_ID },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_Cell_PagingRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_Cell_PagingRqst, Cell_PagingRqst_sequence);

  return offset;
}
static int dissect_cell(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Cell_PagingRqst(tvb, offset, actx, tree, hf_rnsap_cell);
}


static const value_string rnsap_PagingArea_PagingRqst_vals[] = {
  {   0, "uRA" },
  {   1, "cell" },
  { 0, NULL }
};

static const per_choice_t PagingArea_PagingRqst_choice[] = {
  {   0, &hf_rnsap_uRA           , ASN1_EXTENSION_ROOT    , dissect_rnsap_URA_PagingRqst },
  {   1, &hf_rnsap_cell          , ASN1_EXTENSION_ROOT    , dissect_rnsap_Cell_PagingRqst },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_PagingArea_PagingRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_PagingArea_PagingRqst, PagingArea_PagingRqst_choice,
                                 NULL);

  return offset;
}
static int dissect_id_PagingArea_PagingRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PagingArea_PagingRqst(tvb, offset, actx, tree, hf_rnsap_id_PagingArea_PagingRqst);
}


static const value_string rnsap_SchedulingPriorityIndicator_vals[] = {
  {   0, "lowest" },
  {  15, "highest" },
  { 0, NULL }
};


static int
dissect_rnsap_SchedulingPriorityIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 15U, NULL, FALSE);

  return offset;
}
static int dissect_schedulingPriorityIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SchedulingPriorityIndicator(tvb, offset, actx, tree, hf_rnsap_schedulingPriorityIndicator);
}
static int dissect_dSCH_SchedulingPriority(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SchedulingPriorityIndicator(tvb, offset, actx, tree, hf_rnsap_dSCH_SchedulingPriority);
}
static int dissect_fACH_SchedulingPriority(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SchedulingPriorityIndicator(tvb, offset, actx, tree, hf_rnsap_fACH_SchedulingPriority);
}



static int
dissect_rnsap_MAC_c_sh_SDU_Length(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 5000U, NULL, FALSE);

  return offset;
}
static int dissect_MAC_c_sh_SDU_LengthList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MAC_c_sh_SDU_Length(tvb, offset, actx, tree, hf_rnsap_MAC_c_sh_SDU_LengthList_item);
}


static const per_sequence_t MAC_c_sh_SDU_LengthList_sequence_of[1] = {
  { &hf_rnsap_MAC_c_sh_SDU_LengthList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_MAC_c_sh_SDU_Length },
};

static int
dissect_rnsap_MAC_c_sh_SDU_LengthList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_MAC_c_sh_SDU_LengthList, MAC_c_sh_SDU_LengthList_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_mAC_c_sh_SDU_Lengths(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MAC_c_sh_SDU_LengthList(tvb, offset, actx, tree, hf_rnsap_mAC_c_sh_SDU_Lengths);
}


static const value_string rnsap_FACH_InitialWindowSize_vals[] = {
  { 255, "unlimited" },
  { 0, NULL }
};


static int
dissect_rnsap_FACH_InitialWindowSize(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 255U, NULL, FALSE);

  return offset;
}
static int dissect_fACH_InitialWindowSize(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_FACH_InitialWindowSize(tvb, offset, actx, tree, hf_rnsap_fACH_InitialWindowSize);
}


static const per_sequence_t FACH_FlowControlInformationItem_sequence[] = {
  { &hf_rnsap_fACH_SchedulingPriority, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SchedulingPriorityIndicator },
  { &hf_rnsap_mAC_c_sh_SDU_Lengths, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MAC_c_sh_SDU_LengthList },
  { &hf_rnsap_fACH_InitialWindowSize, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_FACH_InitialWindowSize },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_FACH_FlowControlInformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_FACH_FlowControlInformationItem, FACH_FlowControlInformationItem_sequence);

  return offset;
}
static int dissect_FACH_FlowControlInformation_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_FACH_FlowControlInformationItem(tvb, offset, actx, tree, hf_rnsap_FACH_FlowControlInformation_item);
}


static const per_sequence_t FACH_FlowControlInformation_sequence_of[1] = {
  { &hf_rnsap_FACH_FlowControlInformation_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_FACH_FlowControlInformationItem },
};

static int
dissect_rnsap_FACH_FlowControlInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_FACH_FlowControlInformation, FACH_FlowControlInformation_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_FACH_FlowControlInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_FACH_FlowControlInformation(tvb, offset, actx, tree, hf_rnsap_id_FACH_FlowControlInformation);
}


static const value_string rnsap_PartialReportingIndicator_vals[] = {
  {   0, "partial-reporting-allowed" },
  { 0, NULL }
};


static int
dissect_rnsap_PartialReportingIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_PartialReportingIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PartialReportingIndicator(tvb, offset, actx, tree, hf_rnsap_id_PartialReportingIndicator);
}


static const value_string rnsap_Permanent_NAS_UE_Identity_vals[] = {
  {   0, "imsi" },
  { 0, NULL }
};

static const per_choice_t Permanent_NAS_UE_Identity_choice[] = {
  {   0, &hf_rnsap_imsi          , ASN1_EXTENSION_ROOT    , dissect_rnsap_IMSI },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_Permanent_NAS_UE_Identity(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_Permanent_NAS_UE_Identity, Permanent_NAS_UE_Identity_choice,
                                 NULL);

  return offset;
}
static int dissect_id_Permanent_NAS_UE_Identity(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Permanent_NAS_UE_Identity(tvb, offset, actx, tree, hf_rnsap_id_Permanent_NAS_UE_Identity);
}


static const value_string rnsap_PowerAdjustmentType_vals[] = {
  {   0, "none" },
  {   1, "common" },
  {   2, "individual" },
  { 0, NULL }
};


static int
dissect_rnsap_PowerAdjustmentType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_PowerAdjustmentType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PowerAdjustmentType(tvb, offset, actx, tree, hf_rnsap_id_PowerAdjustmentType);
}
static int dissect_powerAdjustmentType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PowerAdjustmentType(tvb, offset, actx, tree, hf_rnsap_powerAdjustmentType);
}



static int
dissect_rnsap_RANAP_RelocationInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     NO_BOUND, NO_BOUND, FALSE, NULL);

  return offset;
}
static int dissect_id_RANAP_RelocationInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RANAP_RelocationInformation(tvb, offset, actx, tree, hf_rnsap_id_RANAP_RelocationInformation);
}



static int
dissect_rnsap_DL_CodeInformationList_PhyChReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_dl_CodeInformation3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_CodeInformationList_PhyChReconfRqstFDD(tvb, offset, actx, tree, hf_rnsap_dl_CodeInformation3);
}


static const per_sequence_t RL_Information_PhyChReconfRqstFDD_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_dl_CodeInformation3, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_CodeInformationList_PhyChReconfRqstFDD },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_Information_PhyChReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_Information_PhyChReconfRqstFDD, RL_Information_PhyChReconfRqstFDD_sequence);

  return offset;
}
static int dissect_id_RL_Information_PhyChReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Information_PhyChReconfRqstFDD(tvb, offset, actx, tree, hf_rnsap_id_RL_Information_PhyChReconfRqstFDD);
}



static int
dissect_rnsap_UL_CCTrCH_InformationList_PhyChReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_ul_CCTrCH_Information1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_CCTrCH_InformationList_PhyChReconfRqstTDD(tvb, offset, actx, tree, hf_rnsap_ul_CCTrCH_Information1);
}



static int
dissect_rnsap_DL_CCTrCH_InformationList_PhyChReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_dl_CCTrCH_Information1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_CCTrCH_InformationList_PhyChReconfRqstTDD(tvb, offset, actx, tree, hf_rnsap_dl_CCTrCH_Information1);
}


static const per_sequence_t RL_Information_PhyChReconfRqstTDD_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_ul_CCTrCH_Information1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_UL_CCTrCH_InformationList_PhyChReconfRqstTDD },
  { &hf_rnsap_dl_CCTrCH_Information1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DL_CCTrCH_InformationList_PhyChReconfRqstTDD },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_Information_PhyChReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_Information_PhyChReconfRqstTDD, RL_Information_PhyChReconfRqstTDD_sequence);

  return offset;
}
static int dissect_id_RL_Information_PhyChReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Information_PhyChReconfRqstTDD(tvb, offset, actx, tree, hf_rnsap_id_RL_Information_PhyChReconfRqstTDD);
}



static int
dissect_rnsap_ChipOffset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 38399U, NULL, FALSE);

  return offset;
}
static int dissect_chipOffset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ChipOffset(tvb, offset, actx, tree, hf_rnsap_chipOffset);
}


static const value_string rnsap_DiversityControlField_vals[] = {
  {   0, "may" },
  {   1, "must" },
  {   2, "must-not" },
  { 0, NULL }
};


static int
dissect_rnsap_DiversityControlField(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_diversityControlField(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DiversityControlField(tvb, offset, actx, tree, hf_rnsap_diversityControlField);
}



static int
dissect_rnsap_PrimaryCPICH_EcNo(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              -30, 30U, NULL, FALSE);

  return offset;
}
static int dissect_primaryCPICH_EcNo(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PrimaryCPICH_EcNo(tvb, offset, actx, tree, hf_rnsap_primaryCPICH_EcNo);
}


static const value_string rnsap_TransmitDiversityIndicator_vals[] = {
  {   0, "active" },
  {   1, "inactive" },
  { 0, NULL }
};


static int
dissect_rnsap_TransmitDiversityIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_transmitDiversityIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TransmitDiversityIndicator(tvb, offset, actx, tree, hf_rnsap_transmitDiversityIndicator);
}


static const per_sequence_t RL_Information_RL_AdditionRqstFDD_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_c_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_C_ID },
  { &hf_rnsap_frameOffset   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_FrameOffset },
  { &hf_rnsap_chipOffset    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ChipOffset },
  { &hf_rnsap_diversityControlField, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DiversityControlField },
  { &hf_rnsap_primaryCPICH_EcNo, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_PrimaryCPICH_EcNo },
  { &hf_rnsap_not_Used_sSDT_CellID, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_NULL },
  { &hf_rnsap_transmitDiversityIndicator, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TransmitDiversityIndicator },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_Information_RL_AdditionRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_Information_RL_AdditionRqstFDD, RL_Information_RL_AdditionRqstFDD_sequence);

  return offset;
}
static int dissect_id_RL_Information_RL_AdditionRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Information_RL_AdditionRqstFDD(tvb, offset, actx, tree, hf_rnsap_id_RL_Information_RL_AdditionRqstFDD);
}



static int
dissect_rnsap_PrimaryCCPCH_RSCP(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 91U, NULL, FALSE);

  return offset;
}
static int dissect_id_PrimaryCCPCH_RSCP_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PrimaryCCPCH_RSCP(tvb, offset, actx, tree, hf_rnsap_id_PrimaryCCPCH_RSCP_RL_ReconfPrepTDD);
}
static int dissect_id_PrimCCPCH_RSCP_DL_PC_RqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PrimaryCCPCH_RSCP(tvb, offset, actx, tree, hf_rnsap_id_PrimCCPCH_RSCP_DL_PC_RqstTDD);
}
static int dissect_primaryCCPCH_RSCP(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PrimaryCCPCH_RSCP(tvb, offset, actx, tree, hf_rnsap_primaryCCPCH_RSCP);
}



static int
dissect_rnsap_DL_TimeslotISCP(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 91U, NULL, FALSE);

  return offset;
}
static int dissect_dL_TimeslotISCP(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_TimeslotISCP(tvb, offset, actx, tree, hf_rnsap_dL_TimeslotISCP);
}


static const per_sequence_t DL_TimeSlot_ISCP_InfoItem_sequence[] = {
  { &hf_rnsap_timeSlot      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TimeSlot },
  { &hf_rnsap_dL_TimeslotISCP, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_TimeslotISCP },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_TimeSlot_ISCP_InfoItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_TimeSlot_ISCP_InfoItem, DL_TimeSlot_ISCP_InfoItem_sequence);

  return offset;
}
static int dissect_DL_TimeSlot_ISCP_Info_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_TimeSlot_ISCP_InfoItem(tvb, offset, actx, tree, hf_rnsap_DL_TimeSlot_ISCP_Info_item);
}


static const per_sequence_t DL_TimeSlot_ISCP_Info_sequence_of[1] = {
  { &hf_rnsap_DL_TimeSlot_ISCP_Info_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_TimeSlot_ISCP_InfoItem },
};

static int
dissect_rnsap_DL_TimeSlot_ISCP_Info(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DL_TimeSlot_ISCP_Info, DL_TimeSlot_ISCP_Info_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_id_timeSlot_ISCP(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_TimeSlot_ISCP_Info(tvb, offset, actx, tree, hf_rnsap_id_timeSlot_ISCP);
}
static int dissect_id_DL_TimeSlot_ISCP_Info_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_TimeSlot_ISCP_Info(tvb, offset, actx, tree, hf_rnsap_id_DL_TimeSlot_ISCP_Info_RL_ReconfPrepTDD);
}
static int dissect_dL_TimeSlot_ISCP(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_TimeSlot_ISCP_Info(tvb, offset, actx, tree, hf_rnsap_dL_TimeSlot_ISCP);
}
static int dissect_dL_TimeSlot_ISCP_Info(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_TimeSlot_ISCP_Info(tvb, offset, actx, tree, hf_rnsap_dL_TimeSlot_ISCP_Info);
}


static const per_sequence_t RL_Information_RL_AdditionRqstTDD_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_c_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_C_ID },
  { &hf_rnsap_frameOffset   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_FrameOffset },
  { &hf_rnsap_diversityControlField, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DiversityControlField },
  { &hf_rnsap_primaryCCPCH_RSCP, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_PrimaryCCPCH_RSCP },
  { &hf_rnsap_dL_TimeSlot_ISCP_Info, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DL_TimeSlot_ISCP_Info },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_Information_RL_AdditionRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_Information_RL_AdditionRqstTDD, RL_Information_RL_AdditionRqstTDD_sequence);

  return offset;
}
static int dissect_id_RL_Information_RL_AdditionRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Information_RL_AdditionRqstTDD(tvb, offset, actx, tree, hf_rnsap_id_RL_Information_RL_AdditionRqstTDD);
}


static const per_sequence_t RL_Information_RL_DeletionRqst_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_Information_RL_DeletionRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_Information_RL_DeletionRqst, RL_Information_RL_DeletionRqst_sequence);

  return offset;
}
static int dissect_id_RL_Information_RL_DeletionRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Information_RL_DeletionRqst(tvb, offset, actx, tree, hf_rnsap_id_RL_Information_RL_DeletionRqst);
}


static const per_sequence_t RL_Information_RL_FailureInd_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_cause         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_Cause },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_Information_RL_FailureInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_Information_RL_FailureInd, RL_Information_RL_FailureInd_sequence);

  return offset;
}
static int dissect_id_RL_Information_RL_FailureInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Information_RL_FailureInd(tvb, offset, actx, tree, hf_rnsap_id_RL_Information_RL_FailureInd);
}


static const per_sequence_t RL_Information_RL_ReconfPrepFDD_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_not_Used_sSDT_Indication, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_NULL },
  { &hf_rnsap_not_Used_sSDT_CellIdentity, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_NULL },
  { &hf_rnsap_transmitDiversityIndicator, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TransmitDiversityIndicator },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_Information_RL_ReconfPrepFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_Information_RL_ReconfPrepFDD, RL_Information_RL_ReconfPrepFDD_sequence);

  return offset;
}
static int dissect_id_RL_Information_RL_ReconfPrepFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Information_RL_ReconfPrepFDD(tvb, offset, actx, tree, hf_rnsap_id_RL_Information_RL_ReconfPrepFDD);
}


static const per_sequence_t RL_Information_RL_RestoreInd_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_Information_RL_RestoreInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_Information_RL_RestoreInd, RL_Information_RL_RestoreInd_sequence);

  return offset;
}
static int dissect_id_RL_Information_RL_RestoreInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Information_RL_RestoreInd(tvb, offset, actx, tree, hf_rnsap_id_RL_Information_RL_RestoreInd);
}


static const per_sequence_t RL_InformationList_RL_SetupRqstFDD_sequence_of[1] = {
  { &hf_rnsap_RL_InformationList_RL_SetupRqstFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_RL_InformationList_RL_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_RL_InformationList_RL_SetupRqstFDD, RL_InformationList_RL_SetupRqstFDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_RL_Information_RL_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_InformationList_RL_SetupRqstFDD(tvb, offset, actx, tree, hf_rnsap_id_RL_Information_RL_SetupRqstFDD);
}



static int
dissect_rnsap_SpecialBurstScheduling(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 256U, NULL, FALSE);

  return offset;
}
static int dissect_specialBurstScheduling(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SpecialBurstScheduling(tvb, offset, actx, tree, hf_rnsap_specialBurstScheduling);
}


static const per_sequence_t RL_Information_RL_SetupRqstTDD_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_c_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_C_ID },
  { &hf_rnsap_frameOffset   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_FrameOffset },
  { &hf_rnsap_specialBurstScheduling, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SpecialBurstScheduling },
  { &hf_rnsap_primaryCCPCH_RSCP, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_PrimaryCCPCH_RSCP },
  { &hf_rnsap_dL_TimeSlot_ISCP, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DL_TimeSlot_ISCP_Info },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_Information_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_Information_RL_SetupRqstTDD, RL_Information_RL_SetupRqstTDD_sequence);

  return offset;
}
static int dissect_id_RL_Information_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Information_RL_SetupRqstTDD(tvb, offset, actx, tree, hf_rnsap_id_RL_Information_RL_SetupRqstTDD);
}


static const per_sequence_t DCH_Rate_Information_RL_CongestInd_sequence_of[1] = {
  { &hf_rnsap_DCH_Rate_Information_RL_CongestInd_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_DCH_Rate_Information_RL_CongestInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DCH_Rate_Information_RL_CongestInd, DCH_Rate_Information_RL_CongestInd_sequence_of,
                                                  1, 128);

  return offset;
}
static int dissect_dCH_Rate_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DCH_Rate_Information_RL_CongestInd(tvb, offset, actx, tree, hf_rnsap_dCH_Rate_Information);
}


static const per_sequence_t RL_InformationItem_RL_CongestInd_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_dCH_Rate_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DCH_Rate_Information_RL_CongestInd },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_InformationItem_RL_CongestInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_InformationItem_RL_CongestInd, RL_InformationItem_RL_CongestInd_sequence);

  return offset;
}
static int dissect_id_RL_InformationItem_RL_CongestInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_InformationItem_RL_CongestInd(tvb, offset, actx, tree, hf_rnsap_id_RL_InformationItem_RL_CongestInd);
}



static int
dissect_rnsap_SIR_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 63U, NULL, FALSE);

  return offset;
}
static int dissect_sIR_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SIR_Value(tvb, offset, actx, tree, hf_rnsap_sIR_Value);
}
static int dissect_sir1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SIR_Value(tvb, offset, actx, tree, hf_rnsap_sir1);
}



static int
dissect_rnsap_SIR_Error_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 125U, NULL, FALSE);

  return offset;
}
static int dissect_sIR_ErrorValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SIR_Error_Value(tvb, offset, actx, tree, hf_rnsap_sIR_ErrorValue);
}
static int dissect_sir_error1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SIR_Error_Value(tvb, offset, actx, tree, hf_rnsap_sir_error1);
}



static int
dissect_rnsap_Transmitted_Code_Power_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 127U, NULL, FALSE);

  return offset;
}
static int dissect_transmittedCodePowerValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Transmitted_Code_Power_Value(tvb, offset, actx, tree, hf_rnsap_transmittedCodePowerValue);
}
static int dissect_transmitted_code_power1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Transmitted_Code_Power_Value(tvb, offset, actx, tree, hf_rnsap_transmitted_code_power1);
}



static int
dissect_rnsap_RSCP_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 127U, NULL, FALSE);

  return offset;
}
static int dissect_rSCP(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RSCP_Value(tvb, offset, actx, tree, hf_rnsap_rSCP);
}
static int dissect_rscp1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RSCP_Value(tvb, offset, actx, tree, hf_rnsap_rscp1);
}



static int
dissect_rnsap_Rx_Timing_Deviation_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 8191U, NULL, FALSE);

  return offset;
}
static int dissect_rxTimingDeviationValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Rx_Timing_Deviation_Value(tvb, offset, actx, tree, hf_rnsap_rxTimingDeviationValue);
}
static int dissect_rx_timing_deviation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Rx_Timing_Deviation_Value(tvb, offset, actx, tree, hf_rnsap_rx_timing_deviation);
}



static int
dissect_rnsap_Round_Trip_Time_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 32767U, NULL, FALSE);

  return offset;
}
static int dissect_roundTripTime(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Round_Trip_Time_Value(tvb, offset, actx, tree, hf_rnsap_roundTripTime);
}
static int dissect_round_trip_time1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Round_Trip_Time_Value(tvb, offset, actx, tree, hf_rnsap_round_trip_time1);
}



static int
dissect_rnsap_Extension_DedicatedMeasurementValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_extension_DedicatedMeasurementValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Extension_DedicatedMeasurementValue(tvb, offset, actx, tree, hf_rnsap_extension_DedicatedMeasurementValue);
}


static const value_string rnsap_DedicatedMeasurementValue_vals[] = {
  {   0, "sIR-Value" },
  {   1, "sIR-ErrorValue" },
  {   2, "transmittedCodePowerValue" },
  {   3, "rSCP" },
  {   4, "rxTimingDeviationValue" },
  {   5, "roundTripTime" },
  {   6, "extension-DedicatedMeasurementValue" },
  { 0, NULL }
};

static const per_choice_t DedicatedMeasurementValue_choice[] = {
  {   0, &hf_rnsap_sIR_Value     , ASN1_EXTENSION_ROOT    , dissect_rnsap_SIR_Value },
  {   1, &hf_rnsap_sIR_ErrorValue, ASN1_EXTENSION_ROOT    , dissect_rnsap_SIR_Error_Value },
  {   2, &hf_rnsap_transmittedCodePowerValue, ASN1_EXTENSION_ROOT    , dissect_rnsap_Transmitted_Code_Power_Value },
  {   3, &hf_rnsap_rSCP          , ASN1_EXTENSION_ROOT    , dissect_rnsap_RSCP_Value },
  {   4, &hf_rnsap_rxTimingDeviationValue, ASN1_EXTENSION_ROOT    , dissect_rnsap_Rx_Timing_Deviation_Value },
  {   5, &hf_rnsap_roundTripTime , ASN1_EXTENSION_ROOT    , dissect_rnsap_Round_Trip_Time_Value },
  {   6, &hf_rnsap_extension_DedicatedMeasurementValue, ASN1_NOT_EXTENSION_ROOT, dissect_rnsap_Extension_DedicatedMeasurementValue },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_DedicatedMeasurementValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_DedicatedMeasurementValue, DedicatedMeasurementValue_choice,
                                 NULL);

  return offset;
}
static int dissect_dedicatedMeasurementValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DedicatedMeasurementValue(tvb, offset, actx, tree, hf_rnsap_dedicatedMeasurementValue);
}
static int dissect_dedicatedmeasurementValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DedicatedMeasurementValue(tvb, offset, actx, tree, hf_rnsap_dedicatedmeasurementValue);
}


static const per_sequence_t DedicatedMeasurementAvailable_sequence[] = {
  { &hf_rnsap_dedicatedmeasurementValue, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DedicatedMeasurementValue },
  { &hf_rnsap_cFN           , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_CFN },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DedicatedMeasurementAvailable(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DedicatedMeasurementAvailable, DedicatedMeasurementAvailable_sequence);

  return offset;
}
static int dissect_measurementAvailable1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DedicatedMeasurementAvailable(tvb, offset, actx, tree, hf_rnsap_measurementAvailable1);
}



static int
dissect_rnsap_DedicatedMeasurementnotAvailable(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_null(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_measurementnotAvailable1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DedicatedMeasurementnotAvailable(tvb, offset, actx, tree, hf_rnsap_measurementnotAvailable1);
}


static const value_string rnsap_DedicatedMeasurementValueInformation_vals[] = {
  {   0, "measurementAvailable" },
  {   1, "measurementnotAvailable" },
  { 0, NULL }
};

static const per_choice_t DedicatedMeasurementValueInformation_choice[] = {
  {   0, &hf_rnsap_measurementAvailable1, ASN1_NO_EXTENSIONS     , dissect_rnsap_DedicatedMeasurementAvailable },
  {   1, &hf_rnsap_measurementnotAvailable1, ASN1_NO_EXTENSIONS     , dissect_rnsap_DedicatedMeasurementnotAvailable },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_DedicatedMeasurementValueInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_DedicatedMeasurementValueInformation, DedicatedMeasurementValueInformation_choice,
                                 NULL);

  return offset;
}
static int dissect_dedicatedMeasurementValueInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DedicatedMeasurementValueInformation(tvb, offset, actx, tree, hf_rnsap_dedicatedMeasurementValueInformation);
}


static const per_sequence_t RL_InformationItem_DM_Rprt_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_dPCH_ID       , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DPCH_ID },
  { &hf_rnsap_dedicatedMeasurementValueInformation, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DedicatedMeasurementValueInformation },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_InformationItem_DM_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_InformationItem_DM_Rprt, RL_InformationItem_DM_Rprt_sequence);

  return offset;
}
static int dissect_id_RL_InformationItem_DM_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_InformationItem_DM_Rprt(tvb, offset, actx, tree, hf_rnsap_id_RL_InformationItem_DM_Rprt);
}


static const per_sequence_t RL_InformationItem_DM_Rqst_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_dPCH_ID       , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DPCH_ID },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_InformationItem_DM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_InformationItem_DM_Rqst, RL_InformationItem_DM_Rqst_sequence);

  return offset;
}
static int dissect_id_RL_InformationItem_DM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_InformationItem_DM_Rqst(tvb, offset, actx, tree, hf_rnsap_id_RL_InformationItem_DM_Rqst);
}


static const per_sequence_t RL_InformationItem_DM_Rsp_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_dPCH_ID       , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DPCH_ID },
  { &hf_rnsap_dedicatedMeasurementValue, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DedicatedMeasurementValue },
  { &hf_rnsap_cFN           , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_CFN },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_InformationItem_DM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_InformationItem_DM_Rsp, RL_InformationItem_DM_Rsp_sequence);

  return offset;
}
static int dissect_id_RL_InformationItem_DM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_InformationItem_DM_Rsp(tvb, offset, actx, tree, hf_rnsap_id_RL_InformationItem_DM_Rsp);
}


static const per_sequence_t RL_InformationItem_RL_PreemptRequiredInd_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_InformationItem_RL_PreemptRequiredInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_InformationItem_RL_PreemptRequiredInd, RL_InformationItem_RL_PreemptRequiredInd_sequence);

  return offset;
}
static int dissect_id_RL_InformationItem_RL_PreemptRequiredInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_InformationItem_RL_PreemptRequiredInd(tvb, offset, actx, tree, hf_rnsap_id_RL_InformationItem_RL_PreemptRequiredInd);
}


static const value_string rnsap_FirstRLS_Indicator_vals[] = {
  {   0, "first-RLS" },
  {   1, "not-first-RLS" },
  { 0, NULL }
};


static int
dissect_rnsap_FirstRLS_Indicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_firstRLS_indicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_FirstRLS_Indicator(tvb, offset, actx, tree, hf_rnsap_firstRLS_indicator);
}
static int dissect_firstRLS_Indicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_FirstRLS_Indicator(tvb, offset, actx, tree, hf_rnsap_firstRLS_Indicator);
}



static int
dissect_rnsap_PropagationDelay(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 255U, NULL, FALSE);

  return offset;
}
static int dissect_id_PropagationDelay(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PropagationDelay(tvb, offset, actx, tree, hf_rnsap_id_PropagationDelay);
}
static int dissect_propagationDelay(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PropagationDelay(tvb, offset, actx, tree, hf_rnsap_propagationDelay);
}
static int dissect_propagation_delay(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PropagationDelay(tvb, offset, actx, tree, hf_rnsap_propagation_delay);
}


static const per_sequence_t RL_InformationItem_RL_SetupRqstFDD_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_c_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_C_ID },
  { &hf_rnsap_firstRLS_indicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_FirstRLS_Indicator },
  { &hf_rnsap_frameOffset   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_FrameOffset },
  { &hf_rnsap_chipOffset    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ChipOffset },
  { &hf_rnsap_propagationDelay, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_PropagationDelay },
  { &hf_rnsap_diversityControlField, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DiversityControlField },
  { &hf_rnsap_dl_InitialTX_Power, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DL_Power },
  { &hf_rnsap_primaryCPICH_EcNo, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_PrimaryCPICH_EcNo },
  { &hf_rnsap_not_Used_sSDT_CellID, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_NULL },
  { &hf_rnsap_transmitDiversityIndicator, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TransmitDiversityIndicator },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_InformationItem_RL_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_InformationItem_RL_SetupRqstFDD, RL_InformationItem_RL_SetupRqstFDD_sequence);

  return offset;
}
static int dissect_id_RL_InformationItem_RL_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_InformationItem_RL_SetupRqstFDD(tvb, offset, actx, tree, hf_rnsap_id_RL_InformationItem_RL_SetupRqstFDD);
}


static const per_sequence_t RL_InformationList_RL_CongestInd_sequence_of[1] = {
  { &hf_rnsap_RL_InformationList_RL_CongestInd_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_RL_InformationList_RL_CongestInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_RL_InformationList_RL_CongestInd, RL_InformationList_RL_CongestInd_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_RL_InformationList_RL_CongestInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_InformationList_RL_CongestInd(tvb, offset, actx, tree, hf_rnsap_id_RL_InformationList_RL_CongestInd);
}


static const per_sequence_t RL_InformationList_RL_AdditionRqstFDD_sequence_of[1] = {
  { &hf_rnsap_RL_InformationList_RL_AdditionRqstFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_RL_InformationList_RL_AdditionRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_RL_InformationList_RL_AdditionRqstFDD, RL_InformationList_RL_AdditionRqstFDD_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_id_RL_InformationList_RL_AdditionRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_InformationList_RL_AdditionRqstFDD(tvb, offset, actx, tree, hf_rnsap_id_RL_InformationList_RL_AdditionRqstFDD);
}


static const per_sequence_t RL_InformationList_RL_DeletionRqst_sequence_of[1] = {
  { &hf_rnsap_RL_InformationList_RL_DeletionRqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_RL_InformationList_RL_DeletionRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_RL_InformationList_RL_DeletionRqst, RL_InformationList_RL_DeletionRqst_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_RL_InformationList_RL_DeletionRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_InformationList_RL_DeletionRqst(tvb, offset, actx, tree, hf_rnsap_id_RL_InformationList_RL_DeletionRqst);
}


static const per_sequence_t RL_InformationList_RL_PreemptRequiredInd_sequence_of[1] = {
  { &hf_rnsap_RL_InformationList_RL_PreemptRequiredInd_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_RL_InformationList_RL_PreemptRequiredInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_RL_InformationList_RL_PreemptRequiredInd, RL_InformationList_RL_PreemptRequiredInd_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_RL_InformationList_RL_PreemptRequiredInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_InformationList_RL_PreemptRequiredInd(tvb, offset, actx, tree, hf_rnsap_id_RL_InformationList_RL_PreemptRequiredInd);
}


static const per_sequence_t RL_InformationList_RL_ReconfPrepFDD_sequence_of[1] = {
  { &hf_rnsap_RL_InformationList_RL_ReconfPrepFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_RL_InformationList_RL_ReconfPrepFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_RL_InformationList_RL_ReconfPrepFDD, RL_InformationList_RL_ReconfPrepFDD_sequence_of,
                                                  0, 16);

  return offset;
}
static int dissect_id_RL_InformationList_RL_ReconfPrepFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_InformationList_RL_ReconfPrepFDD(tvb, offset, actx, tree, hf_rnsap_id_RL_InformationList_RL_ReconfPrepFDD);
}


static const value_string rnsap_MultipleURAsIndicator_vals[] = {
  {   0, "multiple-URAs-exist" },
  {   1, "single-URA-exists" },
  { 0, NULL }
};


static int
dissect_rnsap_MultipleURAsIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_multipleURAsIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MultipleURAsIndicator(tvb, offset, actx, tree, hf_rnsap_multipleURAsIndicator);
}


static const per_sequence_t RNCsWithCellsInTheAccessedURA_Item_sequence[] = {
  { &hf_rnsap_rNC_ID        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RNC_ID },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RNCsWithCellsInTheAccessedURA_Item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RNCsWithCellsInTheAccessedURA_Item, RNCsWithCellsInTheAccessedURA_Item_sequence);

  return offset;
}
static int dissect_RNCsWithCellsInTheAccessedURA_List_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RNCsWithCellsInTheAccessedURA_Item(tvb, offset, actx, tree, hf_rnsap_RNCsWithCellsInTheAccessedURA_List_item);
}


static const per_sequence_t RNCsWithCellsInTheAccessedURA_List_sequence_of[1] = {
  { &hf_rnsap_RNCsWithCellsInTheAccessedURA_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_RNCsWithCellsInTheAccessedURA_Item },
};

static int
dissect_rnsap_RNCsWithCellsInTheAccessedURA_List(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_RNCsWithCellsInTheAccessedURA_List, RNCsWithCellsInTheAccessedURA_List_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_rNCsWithCellsInTheAccessedURA_List(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RNCsWithCellsInTheAccessedURA_List(tvb, offset, actx, tree, hf_rnsap_rNCsWithCellsInTheAccessedURA_List);
}


static const per_sequence_t URA_Information_sequence[] = {
  { &hf_rnsap_uRA_ID        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_URA_ID },
  { &hf_rnsap_multipleURAsIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MultipleURAsIndicator },
  { &hf_rnsap_rNCsWithCellsInTheAccessedURA_List, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_RNCsWithCellsInTheAccessedURA_List },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_URA_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_URA_Information, URA_Information_sequence);

  return offset;
}
static int dissect_id_URA_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_URA_Information(tvb, offset, actx, tree, hf_rnsap_id_URA_Information);
}
static int dissect_uRA_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_URA_Information(tvb, offset, actx, tree, hf_rnsap_uRA_Information);
}



static int
dissect_rnsap_SAC(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_octet_string(tvb, offset, actx, tree, hf_index,
                                       2, 2, NULL);

  return offset;
}
static int dissect_sAC(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SAC(tvb, offset, actx, tree, hf_rnsap_sAC);
}


static const per_sequence_t SAI_sequence[] = {
  { &hf_rnsap_pLMN_Identity , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_PLMN_Identity },
  { &hf_rnsap_lAC           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_LAC },
  { &hf_rnsap_sAC           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_SAC },
  { &hf_rnsap_iE_Extensions , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_SAI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_SAI, SAI_sequence);

  return offset;
}
static int dissect_id_SAI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SAI(tvb, offset, actx, tree, hf_rnsap_id_SAI);
}
static int dissect_sAI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SAI(tvb, offset, actx, tree, hf_rnsap_sAI);
}


static const value_string rnsap_T_latitudeSign_vals[] = {
  {   0, "north" },
  {   1, "south" },
  { 0, NULL }
};


static int
dissect_rnsap_T_latitudeSign(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_latitudeSign(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_T_latitudeSign(tvb, offset, actx, tree, hf_rnsap_latitudeSign);
}



static int
dissect_rnsap_INTEGER_0_8388607(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 8388607U, NULL, FALSE);

  return offset;
}
static int dissect_latitude(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_0_8388607(tvb, offset, actx, tree, hf_rnsap_latitude);
}



static int
dissect_rnsap_INTEGER_M8388608_8388607(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              -8388608, 8388607U, NULL, FALSE);

  return offset;
}
static int dissect_longitude(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_M8388608_8388607(tvb, offset, actx, tree, hf_rnsap_longitude);
}


static const per_sequence_t GeographicalCoordinate_sequence[] = {
  { &hf_rnsap_latitudeSign  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_T_latitudeSign },
  { &hf_rnsap_latitude      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_INTEGER_0_8388607 },
  { &hf_rnsap_longitude     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_INTEGER_M8388608_8388607 },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_GeographicalCoordinate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_GeographicalCoordinate, GeographicalCoordinate_sequence);

  return offset;
}
static int dissect_cell_GAIgeographicalCoordinate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GeographicalCoordinate(tvb, offset, actx, tree, hf_rnsap_cell_GAIgeographicalCoordinate);
}
static int dissect_geographicalCoordinates(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GeographicalCoordinate(tvb, offset, actx, tree, hf_rnsap_geographicalCoordinates);
}
static int dissect_geographicalCoordinate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GeographicalCoordinate(tvb, offset, actx, tree, hf_rnsap_geographicalCoordinate);
}


static const per_sequence_t GA_Cell_item_sequence[] = {
  { &hf_rnsap_cell_GAIgeographicalCoordinate, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_GeographicalCoordinate },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_GA_Cell_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_GA_Cell_item, GA_Cell_item_sequence);

  return offset;
}
static int dissect_GA_Cell_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GA_Cell_item(tvb, offset, actx, tree, hf_rnsap_GA_Cell_item);
}


static const per_sequence_t GA_Cell_sequence_of[1] = {
  { &hf_rnsap_GA_Cell_item  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_GA_Cell_item },
};

static int
dissect_rnsap_GA_Cell(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_GA_Cell, GA_Cell_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_GA_Cell(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GA_Cell(tvb, offset, actx, tree, hf_rnsap_id_GA_Cell);
}
static int dissect_gA_Cell(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GA_Cell(tvb, offset, actx, tree, hf_rnsap_gA_Cell);
}


static const per_sequence_t GA_AccessPointPosition_sequence[] = {
  { &hf_rnsap_geographicalCoordinate, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_GeographicalCoordinate },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_GA_AccessPointPosition(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_GA_AccessPointPosition, GA_AccessPointPosition_sequence);

  return offset;
}
static int dissect_gA_AccessPointPosition(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GA_AccessPointPosition(tvb, offset, actx, tree, hf_rnsap_gA_AccessPointPosition);
}



static int
dissect_rnsap_UL_TimeslotISCP(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 127U, NULL, FALSE);

  return offset;
}
static int dissect_uplinkTimeslotISCPValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_TimeslotISCP(tvb, offset, actx, tree, hf_rnsap_uplinkTimeslotISCPValue);
}
static int dissect_uL_TimeslotISCP(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_TimeslotISCP(tvb, offset, actx, tree, hf_rnsap_uL_TimeslotISCP);
}


static const per_sequence_t UL_TimeSlot_ISCP_InfoItem_sequence[] = {
  { &hf_rnsap_timeSlot      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TimeSlot },
  { &hf_rnsap_uL_TimeslotISCP, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_TimeslotISCP },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UL_TimeSlot_ISCP_InfoItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UL_TimeSlot_ISCP_InfoItem, UL_TimeSlot_ISCP_InfoItem_sequence);

  return offset;
}
static int dissect_UL_TimeSlot_ISCP_Info_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_TimeSlot_ISCP_InfoItem(tvb, offset, actx, tree, hf_rnsap_UL_TimeSlot_ISCP_Info_item);
}


static const per_sequence_t UL_TimeSlot_ISCP_Info_sequence_of[1] = {
  { &hf_rnsap_UL_TimeSlot_ISCP_Info_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_TimeSlot_ISCP_InfoItem },
};

static int
dissect_rnsap_UL_TimeSlot_ISCP_Info(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_UL_TimeSlot_ISCP_Info, UL_TimeSlot_ISCP_Info_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_ul_TimeSlot_ISCP_Info(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_TimeSlot_ISCP_Info(tvb, offset, actx, tree, hf_rnsap_ul_TimeSlot_ISCP_Info);
}



static int
dissect_rnsap_UL_SIR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              -82, 173U, NULL, FALSE);

  return offset;
}
static int dissect_id_UL_SIRTarget(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_SIR(tvb, offset, actx, tree, hf_rnsap_id_UL_SIRTarget);
}
static int dissect_id_UL_SIR_Target_CCTrCH_InformationItem_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_SIR(tvb, offset, actx, tree, hf_rnsap_id_UL_SIR_Target_CCTrCH_InformationItem_RL_SetupRspTDD);
}
static int dissect_id_UL_SIR_Target_CCTrCH_LCR_InformationItem_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_SIR(tvb, offset, actx, tree, hf_rnsap_id_UL_SIR_Target_CCTrCH_LCR_InformationItem_RL_SetupRspTDD);
}
static int dissect_ul_SIRTarget(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_SIR(tvb, offset, actx, tree, hf_rnsap_ul_SIRTarget);
}
static int dissect_maxUL_SIR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_SIR(tvb, offset, actx, tree, hf_rnsap_maxUL_SIR);
}
static int dissect_minUL_SIR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_SIR(tvb, offset, actx, tree, hf_rnsap_minUL_SIR);
}
static int dissect_max_UL_SIR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_SIR(tvb, offset, actx, tree, hf_rnsap_max_UL_SIR);
}
static int dissect_min_UL_SIR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_SIR(tvb, offset, actx, tree, hf_rnsap_min_UL_SIR);
}



static int
dissect_rnsap_MaximumAllowedULTxPower(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              -50, 33U, NULL, FALSE);

  return offset;
}
static int dissect_maximumAllowedULTxPower(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MaximumAllowedULTxPower(tvb, offset, actx, tree, hf_rnsap_maximumAllowedULTxPower);
}


static const value_string rnsap_TimingAdvanceApplied_vals[] = {
  {   0, "yes" },
  {   1, "no" },
  { 0, NULL }
};


static int
dissect_rnsap_TimingAdvanceApplied(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_timingAdvanceApplied(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TimingAdvanceApplied(tvb, offset, actx, tree, hf_rnsap_timingAdvanceApplied);
}



static int
dissect_rnsap_AlphaValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 8U, NULL, FALSE);

  return offset;
}
static int dissect_alphaValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_AlphaValue(tvb, offset, actx, tree, hf_rnsap_alphaValue);
}


static const value_string rnsap_UL_PhysCH_SF_Variation_vals[] = {
  {   0, "sf-variation-supported" },
  {   1, "sf-variation-not-supported" },
  { 0, NULL }
};


static int
dissect_rnsap_UL_PhysCH_SF_Variation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_ul_PhysCH_SF_Variation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_PhysCH_SF_Variation(tvb, offset, actx, tree, hf_rnsap_ul_PhysCH_SF_Variation);
}



static int
dissect_rnsap_INTEGER_1_256(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 256U, NULL, FALSE);

  return offset;
}
static int dissect_n_INSYNC_IND(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_1_256(tvb, offset, actx, tree, hf_rnsap_n_INSYNC_IND);
}
static int dissect_n_OUTSYNC_IND(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_1_256(tvb, offset, actx, tree, hf_rnsap_n_OUTSYNC_IND);
}


static const per_sequence_t SynchronisationConfiguration_sequence[] = {
  { &hf_rnsap_n_INSYNC_IND  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_INTEGER_1_256 },
  { &hf_rnsap_n_OUTSYNC_IND , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_INTEGER_1_256 },
  { &hf_rnsap_t_RLFAILURE   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_INTEGER_0_255 },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_SynchronisationConfiguration(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_SynchronisationConfiguration, SynchronisationConfiguration_sequence);

  return offset;
}
static int dissect_synchronisationConfiguration(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SynchronisationConfiguration(tvb, offset, actx, tree, hf_rnsap_synchronisationConfiguration);
}


static const per_sequence_t Secondary_CCPCH_TDD_Code_InformationItem_sequence[] = {
  { &hf_rnsap_tDD_ChannelisationCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_ChannelisationCode },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_Secondary_CCPCH_TDD_Code_InformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_Secondary_CCPCH_TDD_Code_InformationItem, Secondary_CCPCH_TDD_Code_InformationItem_sequence);

  return offset;
}
static int dissect_Secondary_CCPCH_TDD_Code_Information_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Secondary_CCPCH_TDD_Code_InformationItem(tvb, offset, actx, tree, hf_rnsap_Secondary_CCPCH_TDD_Code_Information_item);
}


static const per_sequence_t Secondary_CCPCH_TDD_Code_Information_sequence_of[1] = {
  { &hf_rnsap_Secondary_CCPCH_TDD_Code_Information_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_Secondary_CCPCH_TDD_Code_InformationItem },
};

static int
dissect_rnsap_Secondary_CCPCH_TDD_Code_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_Secondary_CCPCH_TDD_Code_Information, Secondary_CCPCH_TDD_Code_Information_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_secondary_CCPCH_TDD_Code_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Secondary_CCPCH_TDD_Code_Information(tvb, offset, actx, tree, hf_rnsap_secondary_CCPCH_TDD_Code_Information);
}



static int
dissect_rnsap_TDD_PhysicalChannelOffset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 63U, NULL, FALSE);

  return offset;
}
static int dissect_tDD_PhysicalChannelOffset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_PhysicalChannelOffset(tvb, offset, actx, tree, hf_rnsap_tDD_PhysicalChannelOffset);
}


static const per_sequence_t Secondary_CCPCH_TDD_InformationItem_sequence[] = {
  { &hf_rnsap_timeSlot      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TimeSlot },
  { &hf_rnsap_midambleShiftAndBurstType, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MidambleShiftAndBurstType },
  { &hf_rnsap_tFCI_Presence , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TFCI_Presence },
  { &hf_rnsap_secondary_CCPCH_TDD_Code_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_Secondary_CCPCH_TDD_Code_Information },
  { &hf_rnsap_tDD_PhysicalChannelOffset, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_PhysicalChannelOffset },
  { &hf_rnsap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RepetitionLength },
  { &hf_rnsap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RepetitionPeriod },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_Secondary_CCPCH_TDD_InformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_Secondary_CCPCH_TDD_InformationItem, Secondary_CCPCH_TDD_InformationItem_sequence);

  return offset;
}
static int dissect_Secondary_CCPCH_TDD_InformationList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Secondary_CCPCH_TDD_InformationItem(tvb, offset, actx, tree, hf_rnsap_Secondary_CCPCH_TDD_InformationList_item);
}


static const per_sequence_t Secondary_CCPCH_TDD_InformationList_sequence_of[1] = {
  { &hf_rnsap_Secondary_CCPCH_TDD_InformationList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_Secondary_CCPCH_TDD_InformationItem },
};

static int
dissect_rnsap_Secondary_CCPCH_TDD_InformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_Secondary_CCPCH_TDD_InformationList, Secondary_CCPCH_TDD_InformationList_sequence_of,
                                                  0, 8);

  return offset;
}
static int dissect_secondary_CCPCH_TDD_InformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Secondary_CCPCH_TDD_InformationList(tvb, offset, actx, tree, hf_rnsap_secondary_CCPCH_TDD_InformationList);
}


static const per_sequence_t FACH_InformationItem_sequence[] = {
  { &hf_rnsap_transportFormatSet, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TransportFormatSet },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_FACH_InformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_FACH_InformationItem, FACH_InformationItem_sequence);

  return offset;
}
static int dissect_FACH_InformationList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_FACH_InformationItem(tvb, offset, actx, tree, hf_rnsap_FACH_InformationList_item);
}


static const per_sequence_t FACH_InformationList_sequence_of[1] = {
  { &hf_rnsap_FACH_InformationList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_FACH_InformationItem },
};

static int
dissect_rnsap_FACH_InformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_FACH_InformationList, FACH_InformationList_sequence_of,
                                                  0, 8);

  return offset;
}
static int dissect_fACH_InformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_FACH_InformationList(tvb, offset, actx, tree, hf_rnsap_fACH_InformationList);
}


static const per_sequence_t PCH_InformationItem_sequence[] = {
  { &hf_rnsap_transportFormatSet, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TransportFormatSet },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_PCH_InformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_PCH_InformationItem, PCH_InformationItem_sequence);

  return offset;
}
static int dissect_PCH_InformationList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PCH_InformationItem(tvb, offset, actx, tree, hf_rnsap_PCH_InformationList_item);
}


static const per_sequence_t PCH_InformationList_sequence_of[1] = {
  { &hf_rnsap_PCH_InformationList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_PCH_InformationItem },
};

static int
dissect_rnsap_PCH_InformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_PCH_InformationList, PCH_InformationList_sequence_of,
                                                  0, 1);

  return offset;
}
static int dissect_pCH_InformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PCH_InformationList(tvb, offset, actx, tree, hf_rnsap_pCH_InformationList);
}


static const per_sequence_t Secondary_CCPCH_Info_TDD_sequence[] = {
  { &hf_rnsap_dl_TFCS       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TFCS },
  { &hf_rnsap_tFCI_Coding   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TFCI_Coding },
  { &hf_rnsap_secondary_CCPCH_TDD_InformationList, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_Secondary_CCPCH_TDD_InformationList },
  { &hf_rnsap_fACH_InformationList, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_FACH_InformationList },
  { &hf_rnsap_pCH_InformationList, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PCH_InformationList },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_Secondary_CCPCH_Info_TDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_Secondary_CCPCH_Info_TDD, Secondary_CCPCH_Info_TDD_sequence);

  return offset;
}
static int dissect_secondary_CCPCH_Info_TDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Secondary_CCPCH_Info_TDD(tvb, offset, actx, tree, hf_rnsap_secondary_CCPCH_Info_TDD);
}



static int
dissect_rnsap_UL_CCTrCHInformationList_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_ul_CCTrCHInformation1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_CCTrCHInformationList_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_rnsap_ul_CCTrCHInformation1);
}



static int
dissect_rnsap_DL_CCTrCHInformationList_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_dl_CCTrCHInformation1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_CCTrCHInformationList_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_rnsap_dl_CCTrCHInformation1);
}


static const per_sequence_t Combining_RL_AdditionRspTDD_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_Combining_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_Combining_RL_AdditionRspTDD, Combining_RL_AdditionRspTDD_sequence);

  return offset;
}
static int dissect_combining3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Combining_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_rnsap_combining3);
}


static const per_sequence_t NonCombining_RL_AdditionRspTDD_sequence[] = {
  { &hf_rnsap_dCH_InformationResponse, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DCH_InformationResponse },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_NonCombining_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_NonCombining_RL_AdditionRspTDD, NonCombining_RL_AdditionRspTDD_sequence);

  return offset;
}
static int dissect_nonCombining1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_NonCombining_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_rnsap_nonCombining1);
}


static const value_string rnsap_DiversityIndication_RL_AdditionRspTDD_vals[] = {
  {   0, "combining" },
  {   1, "nonCombining" },
  { 0, NULL }
};

static const per_choice_t DiversityIndication_RL_AdditionRspTDD_choice[] = {
  {   0, &hf_rnsap_combining3    , ASN1_NO_EXTENSIONS     , dissect_rnsap_Combining_RL_AdditionRspTDD },
  {   1, &hf_rnsap_nonCombining1 , ASN1_NO_EXTENSIONS     , dissect_rnsap_NonCombining_RL_AdditionRspTDD },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_DiversityIndication_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_DiversityIndication_RL_AdditionRspTDD, DiversityIndication_RL_AdditionRspTDD_choice,
                                 NULL);

  return offset;
}
static int dissect_diversityIndication3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DiversityIndication_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_rnsap_diversityIndication3);
}


static const per_sequence_t DCH_Information_RL_AdditionRspTDD_sequence[] = {
  { &hf_rnsap_diversityIndication3, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DiversityIndication_RL_AdditionRspTDD },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DCH_Information_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DCH_Information_RL_AdditionRspTDD, DCH_Information_RL_AdditionRspTDD_sequence);

  return offset;
}
static int dissect_dCH_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DCH_Information_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_rnsap_dCH_Information);
}



static int
dissect_rnsap_DSCH_InformationResponse_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_dSCH_InformationResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DSCH_InformationResponse_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_rnsap_dSCH_InformationResponse);
}



static int
dissect_rnsap_USCH_InformationResponse_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_uSCH_InformationResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_USCH_InformationResponse_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_rnsap_uSCH_InformationResponse);
}


static const per_sequence_t Neighbouring_UMTS_CellInformation_sequence_of[1] = {
  { &hf_rnsap_Neighbouring_UMTS_CellInformation_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_Neighbouring_UMTS_CellInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_Neighbouring_UMTS_CellInformation, Neighbouring_UMTS_CellInformation_sequence_of,
                                                  1, 10);

  return offset;
}
static int dissect_neighbouring_UMTS_CellInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Neighbouring_UMTS_CellInformation(tvb, offset, actx, tree, hf_rnsap_neighbouring_UMTS_CellInformation);
}


static const per_sequence_t RL_InformationResponse_RL_AdditionRspTDD_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_uRA_Information, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_URA_Information },
  { &hf_rnsap_sAI           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SAI },
  { &hf_rnsap_gA_Cell       , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_GA_Cell },
  { &hf_rnsap_gA_AccessPointPosition, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_GA_AccessPointPosition },
  { &hf_rnsap_ul_TimeSlot_ISCP_Info, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_TimeSlot_ISCP_Info },
  { &hf_rnsap_minUL_SIR     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_SIR },
  { &hf_rnsap_maxUL_SIR     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_SIR },
  { &hf_rnsap_maximumAllowedULTxPower, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MaximumAllowedULTxPower },
  { &hf_rnsap_maximumDLTxPower, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_Power },
  { &hf_rnsap_minimumDLTxPower, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_Power },
  { &hf_rnsap_pCCPCH_Power  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PCCPCH_Power },
  { &hf_rnsap_timingAdvanceApplied, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TimingAdvanceApplied },
  { &hf_rnsap_alphaValue    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_AlphaValue },
  { &hf_rnsap_ul_PhysCH_SF_Variation, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_PhysCH_SF_Variation },
  { &hf_rnsap_synchronisationConfiguration, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SynchronisationConfiguration },
  { &hf_rnsap_secondary_CCPCH_Info_TDD, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Secondary_CCPCH_Info_TDD },
  { &hf_rnsap_ul_CCTrCHInformation1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_UL_CCTrCHInformationList_RL_AdditionRspTDD },
  { &hf_rnsap_dl_CCTrCHInformation1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DL_CCTrCHInformationList_RL_AdditionRspTDD },
  { &hf_rnsap_dCH_Information, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DCH_Information_RL_AdditionRspTDD },
  { &hf_rnsap_dSCH_InformationResponse, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DSCH_InformationResponse_RL_AdditionRspTDD },
  { &hf_rnsap_uSCH_InformationResponse, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_USCH_InformationResponse_RL_AdditionRspTDD },
  { &hf_rnsap_neighbouring_UMTS_CellInformation, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Neighbouring_UMTS_CellInformation },
  { &hf_rnsap_neighbouring_GSM_CellInformation, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Neighbouring_GSM_CellInformation },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_InformationResponse_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_InformationResponse_RL_AdditionRspTDD, RL_InformationResponse_RL_AdditionRspTDD_sequence);

  return offset;
}
static int dissect_id_RL_InformationResponse_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_InformationResponse_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_rnsap_id_RL_InformationResponse_RL_AdditionRspTDD);
}



static int
dissect_rnsap_UL_CCTrCH_InformationList_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_ul_CCTrCH_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_CCTrCH_InformationList_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_ul_CCTrCH_Information);
}



static int
dissect_rnsap_DL_CCTrCH_InformationList_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_dl_CCTrCH_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_CCTrCH_InformationList_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_dl_CCTrCH_Information);
}



static int
dissect_rnsap_DCH_InformationResponseList_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_dCHInformationResponse1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DCH_InformationResponseList_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_dCHInformationResponse1);
}



static int
dissect_rnsap_DSCHToBeAddedOrModified_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_dSCHsToBeAddedOrModified(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DSCHToBeAddedOrModified_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_dSCHsToBeAddedOrModified);
}



static int
dissect_rnsap_USCHToBeAddedOrModified_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_uSCHsToBeAddedOrModified(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_USCHToBeAddedOrModified_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_uSCHsToBeAddedOrModified);
}


static const per_sequence_t RL_InformationResponse_RL_ReconfReadyTDD_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_max_UL_SIR    , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_UL_SIR },
  { &hf_rnsap_min_UL_SIR    , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_UL_SIR },
  { &hf_rnsap_maximumDLTxPower, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DL_Power },
  { &hf_rnsap_minimumDLTxPower, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DL_Power },
  { &hf_rnsap_secondary_CCPCH_Info_TDD, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Secondary_CCPCH_Info_TDD },
  { &hf_rnsap_ul_CCTrCH_Information, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_UL_CCTrCH_InformationList_RL_ReconfReadyTDD },
  { &hf_rnsap_dl_CCTrCH_Information, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DL_CCTrCH_InformationList_RL_ReconfReadyTDD },
  { &hf_rnsap_dCHInformationResponse1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DCH_InformationResponseList_RL_ReconfReadyTDD },
  { &hf_rnsap_dSCHsToBeAddedOrModified, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DSCHToBeAddedOrModified_RL_ReconfReadyTDD },
  { &hf_rnsap_uSCHsToBeAddedOrModified, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_USCHToBeAddedOrModified_RL_ReconfReadyTDD },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_InformationResponse_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_InformationResponse_RL_ReconfReadyTDD, RL_InformationResponse_RL_ReconfReadyTDD_sequence);

  return offset;
}
static int dissect_id_RL_InformationResponse_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_InformationResponse_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_id_RL_InformationResponse_RL_ReconfReadyTDD);
}
static int dissect_Multiple_RL_InformationResponse_RL_ReconfReadyTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_InformationResponse_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_Multiple_RL_InformationResponse_RL_ReconfReadyTDD_item);
}



static int
dissect_rnsap_UL_CCTrCHInformationList_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_ul_CCTrCHInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_CCTrCHInformationList_RL_SetupRspTDD(tvb, offset, actx, tree, hf_rnsap_ul_CCTrCHInformation);
}



static int
dissect_rnsap_DL_CCTrCHInformationList_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_dl_CCTrCHInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_CCTrCHInformationList_RL_SetupRspTDD(tvb, offset, actx, tree, hf_rnsap_dl_CCTrCHInformation);
}



static int
dissect_rnsap_DCH_InformationResponseList_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_dCH_InformationResponse1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DCH_InformationResponseList_RL_SetupRspTDD(tvb, offset, actx, tree, hf_rnsap_dCH_InformationResponse1);
}



static int
dissect_rnsap_DSCH_InformationResponse_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_dsch_InformationResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DSCH_InformationResponse_RL_SetupRspTDD(tvb, offset, actx, tree, hf_rnsap_dsch_InformationResponse);
}



static int
dissect_rnsap_USCH_InformationResponse_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_usch_InformationResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_USCH_InformationResponse_RL_SetupRspTDD(tvb, offset, actx, tree, hf_rnsap_usch_InformationResponse);
}


static const per_sequence_t RL_InformationResponse_RL_SetupRspTDD_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_uRA_Information, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_URA_Information },
  { &hf_rnsap_sAI           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SAI },
  { &hf_rnsap_gA_Cell       , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_GA_Cell },
  { &hf_rnsap_gA_AccessPointPosition, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_GA_AccessPointPosition },
  { &hf_rnsap_ul_TimeSlot_ISCP_Info, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_TimeSlot_ISCP_Info },
  { &hf_rnsap_maxUL_SIR     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_SIR },
  { &hf_rnsap_minUL_SIR     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_SIR },
  { &hf_rnsap_maximumAllowedULTxPower, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MaximumAllowedULTxPower },
  { &hf_rnsap_maximumDLTxPower, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_Power },
  { &hf_rnsap_minimumDLTxPower, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_Power },
  { &hf_rnsap_uARFCNforNt   , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_UARFCN },
  { &hf_rnsap_cellParameterID, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_CellParameterID },
  { &hf_rnsap_syncCase      , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_SyncCase },
  { &hf_rnsap_sCH_TimeSlot  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_SCH_TimeSlot },
  { &hf_rnsap_sCTD_Indicator, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_SCTD_Indicator },
  { &hf_rnsap_pCCPCH_Power  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PCCPCH_Power },
  { &hf_rnsap_timingAdvanceApplied, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TimingAdvanceApplied },
  { &hf_rnsap_alphaValue    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_AlphaValue },
  { &hf_rnsap_ul_PhysCH_SF_Variation, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_PhysCH_SF_Variation },
  { &hf_rnsap_synchronisationConfiguration, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SynchronisationConfiguration },
  { &hf_rnsap_secondary_CCPCH_Info_TDD, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Secondary_CCPCH_Info_TDD },
  { &hf_rnsap_ul_CCTrCHInformation, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_UL_CCTrCHInformationList_RL_SetupRspTDD },
  { &hf_rnsap_dl_CCTrCHInformation, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DL_CCTrCHInformationList_RL_SetupRspTDD },
  { &hf_rnsap_dCH_InformationResponse1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DCH_InformationResponseList_RL_SetupRspTDD },
  { &hf_rnsap_dsch_InformationResponse, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DSCH_InformationResponse_RL_SetupRspTDD },
  { &hf_rnsap_usch_InformationResponse, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_USCH_InformationResponse_RL_SetupRspTDD },
  { &hf_rnsap_neighbouring_UMTS_CellInformation, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Neighbouring_UMTS_CellInformation },
  { &hf_rnsap_neighbouring_GSM_CellInformation, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Neighbouring_GSM_CellInformation },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_InformationResponse_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_InformationResponse_RL_SetupRspTDD, RL_InformationResponse_RL_SetupRspTDD_sequence);

  return offset;
}
static int dissect_id_RL_InformationResponse_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_InformationResponse_RL_SetupRspTDD(tvb, offset, actx, tree, hf_rnsap_id_RL_InformationResponse_RL_SetupRspTDD);
}



static int
dissect_rnsap_RL_Set_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 31U, NULL, FALSE);

  return offset;
}
static int dissect_id_EDCH_RLSet_Id(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Set_ID(tvb, offset, actx, tree, hf_rnsap_id_EDCH_RLSet_Id);
}
static int dissect_rL_Set_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Set_ID(tvb, offset, actx, tree, hf_rnsap_rL_Set_ID);
}



static int
dissect_rnsap_Received_total_wide_band_power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 621U, NULL, FALSE);

  return offset;
}
static int dissect_received_total_wide_band_power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Received_total_wide_band_power(tvb, offset, actx, tree, hf_rnsap_received_total_wide_band_power);
}



static int
dissect_rnsap_DL_CodeInformationList_RL_AdditionRspFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_dl_CodeInformation1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_CodeInformationList_RL_AdditionRspFDD(tvb, offset, actx, tree, hf_rnsap_dl_CodeInformation1);
}


static const per_sequence_t Combining_RL_AdditionRspFDD_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_Combining_RL_AdditionRspFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_Combining_RL_AdditionRspFDD, Combining_RL_AdditionRspFDD_sequence);

  return offset;
}
static int dissect_combining2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Combining_RL_AdditionRspFDD(tvb, offset, actx, tree, hf_rnsap_combining2);
}


static const per_sequence_t NonCombining_RL_AdditionRspFDD_sequence[] = {
  { &hf_rnsap_dCH_InformationResponse, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DCH_InformationResponse },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_NonCombining_RL_AdditionRspFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_NonCombining_RL_AdditionRspFDD, NonCombining_RL_AdditionRspFDD_sequence);

  return offset;
}
static int dissect_nonCombining(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_NonCombining_RL_AdditionRspFDD(tvb, offset, actx, tree, hf_rnsap_nonCombining);
}


static const value_string rnsap_DiversityIndication_RL_AdditionRspFDD_vals[] = {
  {   0, "combining" },
  {   1, "nonCombining" },
  { 0, NULL }
};

static const per_choice_t DiversityIndication_RL_AdditionRspFDD_choice[] = {
  {   0, &hf_rnsap_combining2    , ASN1_NO_EXTENSIONS     , dissect_rnsap_Combining_RL_AdditionRspFDD },
  {   1, &hf_rnsap_nonCombining  , ASN1_NO_EXTENSIONS     , dissect_rnsap_NonCombining_RL_AdditionRspFDD },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_DiversityIndication_RL_AdditionRspFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_DiversityIndication_RL_AdditionRspFDD, DiversityIndication_RL_AdditionRspFDD_choice,
                                 NULL);

  return offset;
}
static int dissect_diversityIndication2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DiversityIndication_RL_AdditionRspFDD(tvb, offset, actx, tree, hf_rnsap_diversityIndication2);
}


static const value_string rnsap_SSDT_SupportIndicator_vals[] = {
  {   0, "not-Used-sSDT-supported" },
  {   1, "sSDT-not-supported" },
  { 0, NULL }
};


static int
dissect_rnsap_SSDT_SupportIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_sSDT_SupportIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SSDT_SupportIndicator(tvb, offset, actx, tree, hf_rnsap_sSDT_SupportIndicator);
}


static const value_string rnsap_Closedlooptimingadjustmentmode_vals[] = {
  {   0, "adj-1-slot" },
  {   1, "adj-2-slot" },
  { 0, NULL }
};


static int
dissect_rnsap_Closedlooptimingadjustmentmode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_closedlooptimingadjustmentmode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Closedlooptimingadjustmentmode(tvb, offset, actx, tree, hf_rnsap_closedlooptimingadjustmentmode);
}



static int
dissect_rnsap_PC_Preamble(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 7U, NULL, TRUE);

  return offset;
}
static int dissect_pC_Preamble(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PC_Preamble(tvb, offset, actx, tree, hf_rnsap_pC_Preamble);
}



static int
dissect_rnsap_SRB_Delay(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 7U, NULL, TRUE);

  return offset;
}
static int dissect_sRB_Delay(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SRB_Delay(tvb, offset, actx, tree, hf_rnsap_sRB_Delay);
}


static const per_sequence_t RL_InformationResponseItem_RL_AdditionRspFDD_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_rL_Set_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_Set_ID },
  { &hf_rnsap_uRA_Information, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_URA_Information },
  { &hf_rnsap_sAI           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SAI },
  { &hf_rnsap_gA_Cell       , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_GA_Cell },
  { &hf_rnsap_gA_AccessPointPosition, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_GA_AccessPointPosition },
  { &hf_rnsap_received_total_wide_band_power, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_Received_total_wide_band_power },
  { &hf_rnsap_not_Used_secondary_CCPCH_Info, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_NULL },
  { &hf_rnsap_dl_CodeInformation1, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_CodeInformationList_RL_AdditionRspFDD },
  { &hf_rnsap_diversityIndication2, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DiversityIndication_RL_AdditionRspFDD },
  { &hf_rnsap_sSDT_SupportIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SSDT_SupportIndicator },
  { &hf_rnsap_minUL_SIR     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_SIR },
  { &hf_rnsap_maxUL_SIR     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_SIR },
  { &hf_rnsap_closedlooptimingadjustmentmode, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Closedlooptimingadjustmentmode },
  { &hf_rnsap_maximumAllowedULTxPower, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MaximumAllowedULTxPower },
  { &hf_rnsap_maximumDLTxPower, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_Power },
  { &hf_rnsap_minimumDLTxPower, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_Power },
  { &hf_rnsap_neighbouring_UMTS_CellInformation, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Neighbouring_UMTS_CellInformation },
  { &hf_rnsap_neighbouring_GSM_CellInformation, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Neighbouring_GSM_CellInformation },
  { &hf_rnsap_pC_Preamble   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PC_Preamble },
  { &hf_rnsap_sRB_Delay     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SRB_Delay },
  { &hf_rnsap_primaryCPICH_Power, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PrimaryCPICH_Power },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_InformationResponseItem_RL_AdditionRspFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_InformationResponseItem_RL_AdditionRspFDD, RL_InformationResponseItem_RL_AdditionRspFDD_sequence);

  return offset;
}
static int dissect_id_RL_InformationResponseItem_RL_AdditionRspFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_InformationResponseItem_RL_AdditionRspFDD(tvb, offset, actx, tree, hf_rnsap_id_RL_InformationResponseItem_RL_AdditionRspFDD);
}



static int
dissect_rnsap_DL_CodeInformationList_RL_ReconfReadyFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_dl_CodeInformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_CodeInformationList_RL_ReconfReadyFDD(tvb, offset, actx, tree, hf_rnsap_dl_CodeInformationList);
}



static int
dissect_rnsap_DCH_InformationResponseList_RL_ReconfReadyFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_dCHInformationResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DCH_InformationResponseList_RL_ReconfReadyFDD(tvb, offset, actx, tree, hf_rnsap_dCHInformationResponse);
}


static const per_sequence_t RL_InformationResponseItem_RL_ReconfReadyFDD_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_max_UL_SIR    , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_UL_SIR },
  { &hf_rnsap_min_UL_SIR    , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_UL_SIR },
  { &hf_rnsap_maximumDLTxPower, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DL_Power },
  { &hf_rnsap_minimumDLTxPower, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DL_Power },
  { &hf_rnsap_not_Used_secondary_CCPCH_Info, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_NULL },
  { &hf_rnsap_dl_CodeInformationList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DL_CodeInformationList_RL_ReconfReadyFDD },
  { &hf_rnsap_dCHInformationResponse, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DCH_InformationResponseList_RL_ReconfReadyFDD },
  { &hf_rnsap_not_Used_dSCHsToBeAddedOrModified, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_NULL },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_InformationResponseItem_RL_ReconfReadyFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_InformationResponseItem_RL_ReconfReadyFDD, RL_InformationResponseItem_RL_ReconfReadyFDD_sequence);

  return offset;
}
static int dissect_id_RL_InformationResponseItem_RL_ReconfReadyFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_InformationResponseItem_RL_ReconfReadyFDD(tvb, offset, actx, tree, hf_rnsap_id_RL_InformationResponseItem_RL_ReconfReadyFDD);
}



static int
dissect_rnsap_DCH_InformationResponseList_RL_ReconfRspFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_dCHsInformationResponseList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DCH_InformationResponseList_RL_ReconfRspFDD(tvb, offset, actx, tree, hf_rnsap_dCHsInformationResponseList);
}



static int
dissect_rnsap_DL_CodeInformationList_RL_ReconfRspFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_dL_CodeInformationList_RL_ReconfResp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_CodeInformationList_RL_ReconfRspFDD(tvb, offset, actx, tree, hf_rnsap_dL_CodeInformationList_RL_ReconfResp);
}


static const per_sequence_t RL_InformationResponseItem_RL_ReconfRspFDD_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_max_UL_SIR    , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_UL_SIR },
  { &hf_rnsap_min_UL_SIR    , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_UL_SIR },
  { &hf_rnsap_maximumDLTxPower, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DL_Power },
  { &hf_rnsap_minimumDLTxPower, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DL_Power },
  { &hf_rnsap_not_Used_secondary_CCPCH_Info, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_NULL },
  { &hf_rnsap_dCHsInformationResponseList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DCH_InformationResponseList_RL_ReconfRspFDD },
  { &hf_rnsap_dL_CodeInformationList_RL_ReconfResp, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DL_CodeInformationList_RL_ReconfRspFDD },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_InformationResponseItem_RL_ReconfRspFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_InformationResponseItem_RL_ReconfRspFDD, RL_InformationResponseItem_RL_ReconfRspFDD_sequence);

  return offset;
}
static int dissect_id_RL_InformationResponseItem_RL_ReconfRspFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_InformationResponseItem_RL_ReconfRspFDD(tvb, offset, actx, tree, hf_rnsap_id_RL_InformationResponseItem_RL_ReconfRspFDD);
}


static const per_sequence_t Combining_RL_SetupRspFDD_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_Combining_RL_SetupRspFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_Combining_RL_SetupRspFDD, Combining_RL_SetupRspFDD_sequence);

  return offset;
}
static int dissect_combining(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Combining_RL_SetupRspFDD(tvb, offset, actx, tree, hf_rnsap_combining);
}


static const per_sequence_t NonCombiningOrFirstRL_RL_SetupRspFDD_sequence[] = {
  { &hf_rnsap_dCH_InformationResponse, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DCH_InformationResponse },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_NonCombiningOrFirstRL_RL_SetupRspFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_NonCombiningOrFirstRL_RL_SetupRspFDD, NonCombiningOrFirstRL_RL_SetupRspFDD_sequence);

  return offset;
}
static int dissect_nonCombiningOrFirstRL(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_NonCombiningOrFirstRL_RL_SetupRspFDD(tvb, offset, actx, tree, hf_rnsap_nonCombiningOrFirstRL);
}


static const value_string rnsap_DiversityIndication_RL_SetupRspFDD_vals[] = {
  {   0, "combining" },
  {   1, "nonCombiningOrFirstRL" },
  { 0, NULL }
};

static const per_choice_t DiversityIndication_RL_SetupRspFDD_choice[] = {
  {   0, &hf_rnsap_combining     , ASN1_NO_EXTENSIONS     , dissect_rnsap_Combining_RL_SetupRspFDD },
  {   1, &hf_rnsap_nonCombiningOrFirstRL, ASN1_NO_EXTENSIONS     , dissect_rnsap_NonCombiningOrFirstRL_RL_SetupRspFDD },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_DiversityIndication_RL_SetupRspFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_DiversityIndication_RL_SetupRspFDD, DiversityIndication_RL_SetupRspFDD_choice,
                                 NULL);

  return offset;
}
static int dissect_diversityIndication(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DiversityIndication_RL_SetupRspFDD(tvb, offset, actx, tree, hf_rnsap_diversityIndication);
}


static const per_sequence_t RL_InformationResponseItem_RL_SetupRspFDD_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_rL_Set_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_Set_ID },
  { &hf_rnsap_uRA_Information, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_URA_Information },
  { &hf_rnsap_sAI           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SAI },
  { &hf_rnsap_gA_Cell       , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_GA_Cell },
  { &hf_rnsap_gA_AccessPointPosition, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_GA_AccessPointPosition },
  { &hf_rnsap_received_total_wide_band_power, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_Received_total_wide_band_power },
  { &hf_rnsap_not_Used_secondary_CCPCH_Info, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_NULL },
  { &hf_rnsap_dl_CodeInformation, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_FDD_DL_CodeInformation },
  { &hf_rnsap_diversityIndication, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DiversityIndication_RL_SetupRspFDD },
  { &hf_rnsap_sSDT_SupportIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SSDT_SupportIndicator },
  { &hf_rnsap_maxUL_SIR     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_SIR },
  { &hf_rnsap_minUL_SIR     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_SIR },
  { &hf_rnsap_closedlooptimingadjustmentmode, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Closedlooptimingadjustmentmode },
  { &hf_rnsap_maximumAllowedULTxPower, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MaximumAllowedULTxPower },
  { &hf_rnsap_maximumDLTxPower, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_Power },
  { &hf_rnsap_minimumDLTxPower, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_Power },
  { &hf_rnsap_primaryScramblingCode, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_PrimaryScramblingCode },
  { &hf_rnsap_uL_UARFCN     , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_UARFCN },
  { &hf_rnsap_dL_UARFCN     , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_UARFCN },
  { &hf_rnsap_primaryCPICH_Power, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PrimaryCPICH_Power },
  { &hf_rnsap_not_Used_dSCHInformationResponse, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_NULL },
  { &hf_rnsap_neighbouring_UMTS_CellInformation, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Neighbouring_UMTS_CellInformation },
  { &hf_rnsap_neighbouring_GSM_CellInformation, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Neighbouring_GSM_CellInformation },
  { &hf_rnsap_pC_Preamble   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PC_Preamble },
  { &hf_rnsap_sRB_Delay     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SRB_Delay },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_InformationResponseItem_RL_SetupRspFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_InformationResponseItem_RL_SetupRspFDD, RL_InformationResponseItem_RL_SetupRspFDD_sequence);

  return offset;
}
static int dissect_id_RL_InformationResponseItem_RL_SetupRspFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_InformationResponseItem_RL_SetupRspFDD(tvb, offset, actx, tree, hf_rnsap_id_RL_InformationResponseItem_RL_SetupRspFDD);
}


static const per_sequence_t RL_InformationResponseList_RL_AdditionRspFDD_sequence_of[1] = {
  { &hf_rnsap_RL_InformationResponseList_RL_AdditionRspFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_RL_InformationResponseList_RL_AdditionRspFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_RL_InformationResponseList_RL_AdditionRspFDD, RL_InformationResponseList_RL_AdditionRspFDD_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_id_RL_InformationResponseList_RL_AdditionRspFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_InformationResponseList_RL_AdditionRspFDD(tvb, offset, actx, tree, hf_rnsap_id_RL_InformationResponseList_RL_AdditionRspFDD);
}


static const per_sequence_t RL_InformationResponseList_RL_ReconfReadyFDD_sequence_of[1] = {
  { &hf_rnsap_RL_InformationResponseList_RL_ReconfReadyFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_RL_InformationResponseList_RL_ReconfReadyFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_RL_InformationResponseList_RL_ReconfReadyFDD, RL_InformationResponseList_RL_ReconfReadyFDD_sequence_of,
                                                  0, 16);

  return offset;
}
static int dissect_id_RL_InformationResponseList_RL_ReconfReadyFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_InformationResponseList_RL_ReconfReadyFDD(tvb, offset, actx, tree, hf_rnsap_id_RL_InformationResponseList_RL_ReconfReadyFDD);
}


static const per_sequence_t RL_InformationResponseList_RL_ReconfRspFDD_sequence_of[1] = {
  { &hf_rnsap_RL_InformationResponseList_RL_ReconfRspFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_RL_InformationResponseList_RL_ReconfRspFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_RL_InformationResponseList_RL_ReconfRspFDD, RL_InformationResponseList_RL_ReconfRspFDD_sequence_of,
                                                  0, 16);

  return offset;
}
static int dissect_id_RL_InformationResponseList_RL_ReconfRspFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_InformationResponseList_RL_ReconfRspFDD(tvb, offset, actx, tree, hf_rnsap_id_RL_InformationResponseList_RL_ReconfRspFDD);
}



static int
dissect_rnsap_DCH_InformationResponseList_RL_ReconfRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_dCHsInformationResponseList1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DCH_InformationResponseList_RL_ReconfRspTDD(tvb, offset, actx, tree, hf_rnsap_dCHsInformationResponseList1);
}


static const per_sequence_t RL_InformationResponse_RL_ReconfRspTDD_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_max_UL_SIR    , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_UL_SIR },
  { &hf_rnsap_min_UL_SIR    , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_UL_SIR },
  { &hf_rnsap_maximumDLTxPower, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DL_Power },
  { &hf_rnsap_minimumDLTxPower, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DL_Power },
  { &hf_rnsap_dCHsInformationResponseList1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DCH_InformationResponseList_RL_ReconfRspTDD },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_InformationResponse_RL_ReconfRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_InformationResponse_RL_ReconfRspTDD, RL_InformationResponse_RL_ReconfRspTDD_sequence);

  return offset;
}
static int dissect_id_RL_InformationResponse_RL_ReconfRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_InformationResponse_RL_ReconfRspTDD(tvb, offset, actx, tree, hf_rnsap_id_RL_InformationResponse_RL_ReconfRspTDD);
}
static int dissect_Multiple_RL_InformationResponse_RL_ReconfRspTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_InformationResponse_RL_ReconfRspTDD(tvb, offset, actx, tree, hf_rnsap_Multiple_RL_InformationResponse_RL_ReconfRspTDD_item);
}


static const per_sequence_t RL_InformationResponseList_RL_SetupRspFDD_sequence_of[1] = {
  { &hf_rnsap_RL_InformationResponseList_RL_SetupRspFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_RL_InformationResponseList_RL_SetupRspFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_RL_InformationResponseList_RL_SetupRspFDD, RL_InformationResponseList_RL_SetupRspFDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_RL_InformationResponseList_RL_SetupRspFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_InformationResponseList_RL_SetupRspFDD(tvb, offset, actx, tree, hf_rnsap_id_RL_InformationResponseList_RL_SetupRspFDD);
}


static const per_sequence_t RL_ReconfigurationFailure_RL_ReconfFail_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_cause         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_Cause },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_ReconfigurationFailure_RL_ReconfFail(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_ReconfigurationFailure_RL_ReconfFail, RL_ReconfigurationFailure_RL_ReconfFail_sequence);

  return offset;
}
static int dissect_id_RL_ReconfigurationFailure_RL_ReconfFail(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_ReconfigurationFailure_RL_ReconfFail(tvb, offset, actx, tree, hf_rnsap_id_RL_ReconfigurationFailure_RL_ReconfFail);
}


static const per_sequence_t RL_Set_InformationItem_DM_Rprt_sequence[] = {
  { &hf_rnsap_rL_Set_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_Set_ID },
  { &hf_rnsap_dedicatedMeasurementValueInformation, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DedicatedMeasurementValueInformation },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_Set_InformationItem_DM_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_Set_InformationItem_DM_Rprt, RL_Set_InformationItem_DM_Rprt_sequence);

  return offset;
}
static int dissect_id_RL_Set_InformationItem_DM_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Set_InformationItem_DM_Rprt(tvb, offset, actx, tree, hf_rnsap_id_RL_Set_InformationItem_DM_Rprt);
}


static const per_sequence_t RL_Set_InformationItem_DM_Rqst_sequence[] = {
  { &hf_rnsap_rL_Set_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_Set_ID },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_Set_InformationItem_DM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_Set_InformationItem_DM_Rqst, RL_Set_InformationItem_DM_Rqst_sequence);

  return offset;
}
static int dissect_id_RL_Set_InformationItem_DM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Set_InformationItem_DM_Rqst(tvb, offset, actx, tree, hf_rnsap_id_RL_Set_InformationItem_DM_Rqst);
}


static const per_sequence_t RL_Set_InformationItem_DM_Rsp_sequence[] = {
  { &hf_rnsap_rL_Set_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_Set_ID },
  { &hf_rnsap_dedicatedMeasurementValue, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DedicatedMeasurementValue },
  { &hf_rnsap_cFN           , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_CFN },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_Set_InformationItem_DM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_Set_InformationItem_DM_Rsp, RL_Set_InformationItem_DM_Rsp_sequence);

  return offset;
}
static int dissect_id_RL_Set_InformationItem_DM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Set_InformationItem_DM_Rsp(tvb, offset, actx, tree, hf_rnsap_id_RL_Set_InformationItem_DM_Rsp);
}


static const per_sequence_t RL_Set_Information_RL_FailureInd_sequence[] = {
  { &hf_rnsap_rL_Set_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_Set_ID },
  { &hf_rnsap_cause         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_Cause },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_Set_Information_RL_FailureInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_Set_Information_RL_FailureInd, RL_Set_Information_RL_FailureInd_sequence);

  return offset;
}
static int dissect_id_RL_Set_Information_RL_FailureInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Set_Information_RL_FailureInd(tvb, offset, actx, tree, hf_rnsap_id_RL_Set_Information_RL_FailureInd);
}


static const per_sequence_t RL_Set_Information_RL_RestoreInd_sequence[] = {
  { &hf_rnsap_rL_Set_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_Set_ID },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_Set_Information_RL_RestoreInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_Set_Information_RL_RestoreInd, RL_Set_Information_RL_RestoreInd_sequence);

  return offset;
}
static int dissect_id_RL_Set_Information_RL_RestoreInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Set_Information_RL_RestoreInd(tvb, offset, actx, tree, hf_rnsap_id_RL_Set_Information_RL_RestoreInd);
}


static const per_sequence_t RL_Set_Successful_InformationItem_DM_Fail_sequence[] = {
  { &hf_rnsap_rL_Set_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_Set_ID },
  { &hf_rnsap_dedicatedMeasurementValue, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DedicatedMeasurementValue },
  { &hf_rnsap_cFN           , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_CFN },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_Set_Successful_InformationItem_DM_Fail(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_Set_Successful_InformationItem_DM_Fail, RL_Set_Successful_InformationItem_DM_Fail_sequence);

  return offset;
}
static int dissect_id_RL_Set_Successful_InformationItem_DM_Fail(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Set_Successful_InformationItem_DM_Fail(tvb, offset, actx, tree, hf_rnsap_id_RL_Set_Successful_InformationItem_DM_Fail);
}


static const per_sequence_t RL_Set_Unsuccessful_InformationItem_DM_Fail_sequence[] = {
  { &hf_rnsap_rL_Set_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_Set_ID },
  { &hf_rnsap_individualcause, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Cause },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_Set_Unsuccessful_InformationItem_DM_Fail(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_Set_Unsuccessful_InformationItem_DM_Fail, RL_Set_Unsuccessful_InformationItem_DM_Fail_sequence);

  return offset;
}
static int dissect_id_RL_Set_Unsuccessful_InformationItem_DM_Fail(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Set_Unsuccessful_InformationItem_DM_Fail(tvb, offset, actx, tree, hf_rnsap_id_RL_Set_Unsuccessful_InformationItem_DM_Fail);
}


static const per_sequence_t RL_Set_Unsuccessful_InformationItem_DM_Fail_Ind_sequence[] = {
  { &hf_rnsap_rL_Set_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_Set_ID },
  { &hf_rnsap_individualcause, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Cause },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_Set_Unsuccessful_InformationItem_DM_Fail_Ind(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_Set_Unsuccessful_InformationItem_DM_Fail_Ind, RL_Set_Unsuccessful_InformationItem_DM_Fail_Ind_sequence);

  return offset;
}
static int dissect_id_RL_Set_Unsuccessful_InformationItem_DM_Fail_Ind(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Set_Unsuccessful_InformationItem_DM_Fail_Ind(tvb, offset, actx, tree, hf_rnsap_id_RL_Set_Unsuccessful_InformationItem_DM_Fail_Ind);
}


static const per_sequence_t RL_Successful_InformationItem_DM_Fail_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_dPCH_ID       , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DPCH_ID },
  { &hf_rnsap_dedicatedMeasurementValue, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DedicatedMeasurementValue },
  { &hf_rnsap_cFN           , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_CFN },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_Successful_InformationItem_DM_Fail(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_Successful_InformationItem_DM_Fail, RL_Successful_InformationItem_DM_Fail_sequence);

  return offset;
}
static int dissect_id_RL_Successful_InformationItem_DM_Fail(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Successful_InformationItem_DM_Fail(tvb, offset, actx, tree, hf_rnsap_id_RL_Successful_InformationItem_DM_Fail);
}


static const per_sequence_t RL_Unsuccessful_InformationItem_DM_Fail_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_individualcause, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Cause },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_Unsuccessful_InformationItem_DM_Fail(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_Unsuccessful_InformationItem_DM_Fail, RL_Unsuccessful_InformationItem_DM_Fail_sequence);

  return offset;
}
static int dissect_id_RL_Unsuccessful_InformationItem_DM_Fail(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Unsuccessful_InformationItem_DM_Fail(tvb, offset, actx, tree, hf_rnsap_id_RL_Unsuccessful_InformationItem_DM_Fail);
}


static const per_sequence_t RL_Unsuccessful_InformationItem_DM_Fail_Ind_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_individualcause, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Cause },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_Unsuccessful_InformationItem_DM_Fail_Ind(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_Unsuccessful_InformationItem_DM_Fail_Ind, RL_Unsuccessful_InformationItem_DM_Fail_Ind_sequence);

  return offset;
}
static int dissect_id_RL_Unsuccessful_InformationItem_DM_Fail_Ind(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Unsuccessful_InformationItem_DM_Fail_Ind(tvb, offset, actx, tree, hf_rnsap_id_RL_Unsuccessful_InformationItem_DM_Fail_Ind);
}



static int
dissect_rnsap_INTEGER_1_6000_(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 6000U, NULL, TRUE);

  return offset;
}
static int dissect_ten_msec(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_1_6000_(tvb, offset, actx, tree, hf_rnsap_ten_msec);
}



static int
dissect_rnsap_INTEGER_1_60_(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 60U, NULL, TRUE);

  return offset;
}
static int dissect_min(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_1_60_(tvb, offset, actx, tree, hf_rnsap_min);
}


static const value_string rnsap_ReportPeriodicity_vals[] = {
  {   0, "ten-msec" },
  {   1, "min" },
  { 0, NULL }
};

static const per_choice_t ReportPeriodicity_choice[] = {
  {   0, &hf_rnsap_ten_msec      , ASN1_EXTENSION_ROOT    , dissect_rnsap_INTEGER_1_6000_ },
  {   1, &hf_rnsap_min           , ASN1_EXTENSION_ROOT    , dissect_rnsap_INTEGER_1_60_ },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_ReportPeriodicity(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_ReportPeriodicity, ReportPeriodicity_choice,
                                 NULL);

  return offset;
}
static int dissect_reportPeriodicity(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ReportPeriodicity(tvb, offset, actx, tree, hf_rnsap_reportPeriodicity);
}


static const per_sequence_t Periodic_sequence[] = {
  { &hf_rnsap_reportPeriodicity, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ReportPeriodicity },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_Periodic(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_Periodic, Periodic_sequence);

  return offset;
}
static int dissect_periodic1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Periodic(tvb, offset, actx, tree, hf_rnsap_periodic1);
}



static int
dissect_rnsap_Extension_MeasurementThreshold(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_extension_MeasurementThreshold(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Extension_MeasurementThreshold(tvb, offset, actx, tree, hf_rnsap_extension_MeasurementThreshold);
}


static const value_string rnsap_MeasurementThreshold_vals[] = {
  {   0, "sir" },
  {   1, "sir-error" },
  {   2, "transmitted-code-power" },
  {   3, "rscp" },
  {   4, "rx-timing-deviation" },
  {   5, "round-trip-time" },
  {   6, "extension-MeasurementThreshold" },
  { 0, NULL }
};

static const per_choice_t MeasurementThreshold_choice[] = {
  {   0, &hf_rnsap_sir1          , ASN1_EXTENSION_ROOT    , dissect_rnsap_SIR_Value },
  {   1, &hf_rnsap_sir_error1    , ASN1_EXTENSION_ROOT    , dissect_rnsap_SIR_Error_Value },
  {   2, &hf_rnsap_transmitted_code_power1, ASN1_EXTENSION_ROOT    , dissect_rnsap_Transmitted_Code_Power_Value },
  {   3, &hf_rnsap_rscp1         , ASN1_EXTENSION_ROOT    , dissect_rnsap_RSCP_Value },
  {   4, &hf_rnsap_rx_timing_deviation, ASN1_EXTENSION_ROOT    , dissect_rnsap_Rx_Timing_Deviation_Value },
  {   5, &hf_rnsap_round_trip_time1, ASN1_EXTENSION_ROOT    , dissect_rnsap_Round_Trip_Time_Value },
  {   6, &hf_rnsap_extension_MeasurementThreshold, ASN1_NOT_EXTENSION_ROOT, dissect_rnsap_Extension_MeasurementThreshold },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_MeasurementThreshold(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_MeasurementThreshold, MeasurementThreshold_choice,
                                 NULL);

  return offset;
}
static int dissect_measurementTreshold(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MeasurementThreshold(tvb, offset, actx, tree, hf_rnsap_measurementTreshold);
}
static int dissect_measurementThreshold1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MeasurementThreshold(tvb, offset, actx, tree, hf_rnsap_measurementThreshold1);
}
static int dissect_measurementThreshold2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MeasurementThreshold(tvb, offset, actx, tree, hf_rnsap_measurementThreshold2);
}
static int dissect_measurementThreshold(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MeasurementThreshold(tvb, offset, actx, tree, hf_rnsap_measurementThreshold);
}



static int
dissect_rnsap_MeasurementHysteresisTime(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 6000U, NULL, TRUE);

  return offset;
}
static int dissect_measurementHysteresisTime(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MeasurementHysteresisTime(tvb, offset, actx, tree, hf_rnsap_measurementHysteresisTime);
}


static const per_sequence_t EventA_sequence[] = {
  { &hf_rnsap_measurementTreshold, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MeasurementThreshold },
  { &hf_rnsap_measurementHysteresisTime, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_MeasurementHysteresisTime },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_EventA(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_EventA, EventA_sequence);

  return offset;
}
static int dissect_eventA(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_EventA(tvb, offset, actx, tree, hf_rnsap_eventA);
}


static const per_sequence_t EventB_sequence[] = {
  { &hf_rnsap_measurementTreshold, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MeasurementThreshold },
  { &hf_rnsap_measurementHysteresisTime, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_MeasurementHysteresisTime },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_EventB(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_EventB, EventB_sequence);

  return offset;
}
static int dissect_eventB(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_EventB(tvb, offset, actx, tree, hf_rnsap_eventB);
}



static int
dissect_rnsap_SIR_Value_IncrDecrThres(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 62U, NULL, FALSE);

  return offset;
}
static int dissect_sir(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SIR_Value_IncrDecrThres(tvb, offset, actx, tree, hf_rnsap_sir);
}



static int
dissect_rnsap_SIR_Error_Value_IncrDecrThres(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 124U, NULL, FALSE);

  return offset;
}
static int dissect_sir_error(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SIR_Error_Value_IncrDecrThres(tvb, offset, actx, tree, hf_rnsap_sir_error);
}



static int
dissect_rnsap_Transmitted_Code_Power_Value_IncrDecrThres(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 112U, NULL, TRUE);

  return offset;
}
static int dissect_transmitted_code_power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Transmitted_Code_Power_Value_IncrDecrThres(tvb, offset, actx, tree, hf_rnsap_transmitted_code_power);
}



static int
dissect_rnsap_RSCP_Value_IncrDecrThres(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 126U, NULL, FALSE);

  return offset;
}
static int dissect_rscp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RSCP_Value_IncrDecrThres(tvb, offset, actx, tree, hf_rnsap_rscp);
}



static int
dissect_rnsap_Round_Trip_Time_IncrDecrThres(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 32766U, NULL, FALSE);

  return offset;
}
static int dissect_round_trip_time(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Round_Trip_Time_IncrDecrThres(tvb, offset, actx, tree, hf_rnsap_round_trip_time);
}



static int
dissect_rnsap_Extension_MeasurementIncreaseDecreaseThreshold(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_extension_MeasurementIncreaseDecreaseThreshold(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Extension_MeasurementIncreaseDecreaseThreshold(tvb, offset, actx, tree, hf_rnsap_extension_MeasurementIncreaseDecreaseThreshold);
}


static const value_string rnsap_MeasurementIncreaseDecreaseThreshold_vals[] = {
  {   0, "sir" },
  {   1, "sir-error" },
  {   2, "transmitted-code-power" },
  {   3, "rscp" },
  {   4, "round-trip-time" },
  {   5, "extension-MeasurementIncreaseDecreaseThreshold" },
  { 0, NULL }
};

static const per_choice_t MeasurementIncreaseDecreaseThreshold_choice[] = {
  {   0, &hf_rnsap_sir           , ASN1_EXTENSION_ROOT    , dissect_rnsap_SIR_Value_IncrDecrThres },
  {   1, &hf_rnsap_sir_error     , ASN1_EXTENSION_ROOT    , dissect_rnsap_SIR_Error_Value_IncrDecrThres },
  {   2, &hf_rnsap_transmitted_code_power, ASN1_EXTENSION_ROOT    , dissect_rnsap_Transmitted_Code_Power_Value_IncrDecrThres },
  {   3, &hf_rnsap_rscp          , ASN1_EXTENSION_ROOT    , dissect_rnsap_RSCP_Value_IncrDecrThres },
  {   4, &hf_rnsap_round_trip_time, ASN1_EXTENSION_ROOT    , dissect_rnsap_Round_Trip_Time_IncrDecrThres },
  {   5, &hf_rnsap_extension_MeasurementIncreaseDecreaseThreshold, ASN1_NOT_EXTENSION_ROOT, dissect_rnsap_Extension_MeasurementIncreaseDecreaseThreshold },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_MeasurementIncreaseDecreaseThreshold(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_MeasurementIncreaseDecreaseThreshold, MeasurementIncreaseDecreaseThreshold_choice,
                                 NULL);

  return offset;
}
static int dissect_measurementIncreaseDecreaseThreshold(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MeasurementIncreaseDecreaseThreshold(tvb, offset, actx, tree, hf_rnsap_measurementIncreaseDecreaseThreshold);
}



static int
dissect_rnsap_MeasurementChangeTime(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 6000U, NULL, TRUE);

  return offset;
}
static int dissect_measurementChangeTime(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MeasurementChangeTime(tvb, offset, actx, tree, hf_rnsap_measurementChangeTime);
}


static const per_sequence_t EventC_sequence[] = {
  { &hf_rnsap_measurementIncreaseDecreaseThreshold, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MeasurementIncreaseDecreaseThreshold },
  { &hf_rnsap_measurementChangeTime, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MeasurementChangeTime },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_EventC(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_EventC, EventC_sequence);

  return offset;
}
static int dissect_eventC(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_EventC(tvb, offset, actx, tree, hf_rnsap_eventC);
}


static const per_sequence_t EventD_sequence[] = {
  { &hf_rnsap_measurementIncreaseDecreaseThreshold, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MeasurementIncreaseDecreaseThreshold },
  { &hf_rnsap_measurementChangeTime, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MeasurementChangeTime },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_EventD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_EventD, EventD_sequence);

  return offset;
}
static int dissect_eventD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_EventD(tvb, offset, actx, tree, hf_rnsap_eventD);
}


static const per_sequence_t EventE_sequence[] = {
  { &hf_rnsap_measurementThreshold1, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MeasurementThreshold },
  { &hf_rnsap_measurementThreshold2, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_MeasurementThreshold },
  { &hf_rnsap_measurementHysteresisTime, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_MeasurementHysteresisTime },
  { &hf_rnsap_reportPeriodicity, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ReportPeriodicity },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_EventE(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_EventE, EventE_sequence);

  return offset;
}
static int dissect_eventE(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_EventE(tvb, offset, actx, tree, hf_rnsap_eventE);
}


static const per_sequence_t EventF_sequence[] = {
  { &hf_rnsap_measurementThreshold1, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MeasurementThreshold },
  { &hf_rnsap_measurementThreshold2, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_MeasurementThreshold },
  { &hf_rnsap_measurementHysteresisTime, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_MeasurementHysteresisTime },
  { &hf_rnsap_reportPeriodicity, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ReportPeriodicity },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_EventF(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_EventF, EventF_sequence);

  return offset;
}
static int dissect_eventF(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_EventF(tvb, offset, actx, tree, hf_rnsap_eventF);
}



static int
dissect_rnsap_Extension_ReportCharacteristics(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_extension_ReportCharacteristics(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Extension_ReportCharacteristics(tvb, offset, actx, tree, hf_rnsap_extension_ReportCharacteristics);
}


static const value_string rnsap_ReportCharacteristics_vals[] = {
  {   0, "onDemand" },
  {   1, "periodic" },
  {   2, "eventA" },
  {   3, "eventB" },
  {   4, "eventC" },
  {   5, "eventD" },
  {   6, "eventE" },
  {   7, "eventF" },
  {   8, "extension-ReportCharacteristics" },
  { 0, NULL }
};

static const per_choice_t ReportCharacteristics_choice[] = {
  {   0, &hf_rnsap_onDemand      , ASN1_EXTENSION_ROOT    , dissect_rnsap_NULL },
  {   1, &hf_rnsap_periodic1     , ASN1_EXTENSION_ROOT    , dissect_rnsap_Periodic },
  {   2, &hf_rnsap_eventA        , ASN1_EXTENSION_ROOT    , dissect_rnsap_EventA },
  {   3, &hf_rnsap_eventB        , ASN1_EXTENSION_ROOT    , dissect_rnsap_EventB },
  {   4, &hf_rnsap_eventC        , ASN1_EXTENSION_ROOT    , dissect_rnsap_EventC },
  {   5, &hf_rnsap_eventD        , ASN1_EXTENSION_ROOT    , dissect_rnsap_EventD },
  {   6, &hf_rnsap_eventE        , ASN1_EXTENSION_ROOT    , dissect_rnsap_EventE },
  {   7, &hf_rnsap_eventF        , ASN1_EXTENSION_ROOT    , dissect_rnsap_EventF },
  {   8, &hf_rnsap_extension_ReportCharacteristics, ASN1_NOT_EXTENSION_ROOT, dissect_rnsap_Extension_ReportCharacteristics },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_ReportCharacteristics(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_ReportCharacteristics, ReportCharacteristics_choice,
                                 NULL);

  return offset;
}
static int dissect_id_ReportCharacteristics(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ReportCharacteristics(tvb, offset, actx, tree, hf_rnsap_id_ReportCharacteristics);
}


static const per_sequence_t RL_InformationList_RL_FailureInd_sequence_of[1] = {
  { &hf_rnsap_RL_InformationList_RL_FailureInd_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_RL_InformationList_RL_FailureInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_RL_InformationList_RL_FailureInd, RL_InformationList_RL_FailureInd_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_rL_InformationList_RL_FailureInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_InformationList_RL_FailureInd(tvb, offset, actx, tree, hf_rnsap_rL_InformationList_RL_FailureInd);
}


static const per_sequence_t RL_RL_FailureInd_sequence[] = {
  { &hf_rnsap_rL_InformationList_RL_FailureInd, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_InformationList_RL_FailureInd },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_RL_FailureInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_RL_FailureInd, RL_RL_FailureInd_sequence);

  return offset;
}
static int dissect_rL(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_RL_FailureInd(tvb, offset, actx, tree, hf_rnsap_rL);
}


static const per_sequence_t RL_Set_InformationList_RL_FailureInd_sequence_of[1] = {
  { &hf_rnsap_RL_Set_InformationList_RL_FailureInd_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_RL_Set_InformationList_RL_FailureInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_RL_Set_InformationList_RL_FailureInd, RL_Set_InformationList_RL_FailureInd_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_rL_Set_InformationList_RL_FailureInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Set_InformationList_RL_FailureInd(tvb, offset, actx, tree, hf_rnsap_rL_Set_InformationList_RL_FailureInd);
}


static const per_sequence_t RL_Set_RL_FailureInd_sequence[] = {
  { &hf_rnsap_rL_Set_InformationList_RL_FailureInd, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_Set_InformationList_RL_FailureInd },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_Set_RL_FailureInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_Set_RL_FailureInd, RL_Set_RL_FailureInd_sequence);

  return offset;
}
static int dissect_rL_Set(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Set_RL_FailureInd(tvb, offset, actx, tree, hf_rnsap_rL_Set);
}


static const per_sequence_t CCTrCH_InformationList_RL_FailureInd_sequence_of[1] = {
  { &hf_rnsap_CCTrCH_InformationList_RL_FailureInd_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_CCTrCH_InformationList_RL_FailureInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_CCTrCH_InformationList_RL_FailureInd, CCTrCH_InformationList_RL_FailureInd_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_cCTrCH_InformationList_RL_FailureInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CCTrCH_InformationList_RL_FailureInd(tvb, offset, actx, tree, hf_rnsap_cCTrCH_InformationList_RL_FailureInd);
}


static const per_sequence_t CCTrCH_RL_FailureInd_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_cCTrCH_InformationList_RL_FailureInd, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CCTrCH_InformationList_RL_FailureInd },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_CCTrCH_RL_FailureInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_CCTrCH_RL_FailureInd, CCTrCH_RL_FailureInd_sequence);

  return offset;
}
static int dissect_cCTrCH(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CCTrCH_RL_FailureInd(tvb, offset, actx, tree, hf_rnsap_cCTrCH);
}


static const value_string rnsap_Reporting_Object_RL_FailureInd_vals[] = {
  {   0, "rL" },
  {   1, "rL-Set" },
  {   2, "cCTrCH" },
  { 0, NULL }
};

static const per_choice_t Reporting_Object_RL_FailureInd_choice[] = {
  {   0, &hf_rnsap_rL            , ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_RL_FailureInd },
  {   1, &hf_rnsap_rL_Set        , ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_Set_RL_FailureInd },
  {   2, &hf_rnsap_cCTrCH        , ASN1_NOT_EXTENSION_ROOT, dissect_rnsap_CCTrCH_RL_FailureInd },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_Reporting_Object_RL_FailureInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_Reporting_Object_RL_FailureInd, Reporting_Object_RL_FailureInd_choice,
                                 NULL);

  return offset;
}
static int dissect_id_Reporting_Object_RL_FailureInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Reporting_Object_RL_FailureInd(tvb, offset, actx, tree, hf_rnsap_id_Reporting_Object_RL_FailureInd);
}


static const per_sequence_t RL_InformationList_RL_RestoreInd_sequence_of[1] = {
  { &hf_rnsap_RL_InformationList_RL_RestoreInd_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_RL_InformationList_RL_RestoreInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_RL_InformationList_RL_RestoreInd, RL_InformationList_RL_RestoreInd_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_rL_InformationList_RL_RestoreInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_InformationList_RL_RestoreInd(tvb, offset, actx, tree, hf_rnsap_rL_InformationList_RL_RestoreInd);
}


static const per_sequence_t RL_RL_RestoreInd_sequence[] = {
  { &hf_rnsap_rL_InformationList_RL_RestoreInd, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_InformationList_RL_RestoreInd },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_RL_RestoreInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_RL_RestoreInd, RL_RL_RestoreInd_sequence);

  return offset;
}
static int dissect_rL1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_RL_RestoreInd(tvb, offset, actx, tree, hf_rnsap_rL1);
}


static const per_sequence_t RL_Set_InformationList_RL_RestoreInd_sequence_of[1] = {
  { &hf_rnsap_RL_Set_InformationList_RL_RestoreInd_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_RL_Set_InformationList_RL_RestoreInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_RL_Set_InformationList_RL_RestoreInd, RL_Set_InformationList_RL_RestoreInd_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_rL_Set_InformationList_RL_RestoreInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Set_InformationList_RL_RestoreInd(tvb, offset, actx, tree, hf_rnsap_rL_Set_InformationList_RL_RestoreInd);
}


static const per_sequence_t RL_Set_RL_RestoreInd_sequence[] = {
  { &hf_rnsap_rL_Set_InformationList_RL_RestoreInd, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_Set_InformationList_RL_RestoreInd },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_Set_RL_RestoreInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_Set_RL_RestoreInd, RL_Set_RL_RestoreInd_sequence);

  return offset;
}
static int dissect_rL_Set1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Set_RL_RestoreInd(tvb, offset, actx, tree, hf_rnsap_rL_Set1);
}


static const per_sequence_t CCTrCH_InformationList_RL_RestoreInd_sequence_of[1] = {
  { &hf_rnsap_CCTrCH_InformationList_RL_RestoreInd_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_CCTrCH_InformationList_RL_RestoreInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_CCTrCH_InformationList_RL_RestoreInd, CCTrCH_InformationList_RL_RestoreInd_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_cCTrCH_InformationList_RL_RestoreInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CCTrCH_InformationList_RL_RestoreInd(tvb, offset, actx, tree, hf_rnsap_cCTrCH_InformationList_RL_RestoreInd);
}


static const per_sequence_t CCTrCH_RL_RestoreInd_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_cCTrCH_InformationList_RL_RestoreInd, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CCTrCH_InformationList_RL_RestoreInd },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_CCTrCH_RL_RestoreInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_CCTrCH_RL_RestoreInd, CCTrCH_RL_RestoreInd_sequence);

  return offset;
}
static int dissect_cCTrCH1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CCTrCH_RL_RestoreInd(tvb, offset, actx, tree, hf_rnsap_cCTrCH1);
}


static const value_string rnsap_Reporting_Object_RL_RestoreInd_vals[] = {
  {   0, "rL" },
  {   1, "rL-Set" },
  {   2, "cCTrCH" },
  { 0, NULL }
};

static const per_choice_t Reporting_Object_RL_RestoreInd_choice[] = {
  {   0, &hf_rnsap_rL1           , ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_RL_RestoreInd },
  {   1, &hf_rnsap_rL_Set1       , ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_Set_RL_RestoreInd },
  {   2, &hf_rnsap_cCTrCH1       , ASN1_NOT_EXTENSION_ROOT, dissect_rnsap_CCTrCH_RL_RestoreInd },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_Reporting_Object_RL_RestoreInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_Reporting_Object_RL_RestoreInd, Reporting_Object_RL_RestoreInd_choice,
                                 NULL);

  return offset;
}
static int dissect_id_Reporing_Object_RL_RestoreInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Reporting_Object_RL_RestoreInd(tvb, offset, actx, tree, hf_rnsap_id_Reporing_Object_RL_RestoreInd);
}



static int
dissect_rnsap_RT_Load_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 100U, NULL, FALSE);

  return offset;
}
static int dissect_id_RT_Load_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RT_Load_Value(tvb, offset, actx, tree, hf_rnsap_id_RT_Load_Value);
}



static int
dissect_rnsap_RT_Load_Value_IncrDecrThres(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 100U, NULL, FALSE);

  return offset;
}
static int dissect_id_RT_Load_Value_IncrDecrThres(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RT_Load_Value_IncrDecrThres(tvb, offset, actx, tree, hf_rnsap_id_RT_Load_Value_IncrDecrThres);
}


static const per_sequence_t ContextInfoList_Reset_sequence_of[1] = {
  { &hf_rnsap_ContextInfoList_Reset_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_ContextInfoList_Reset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_ContextInfoList_Reset, ContextInfoList_Reset_sequence_of,
                                                  1, 250);

  return offset;
}
static int dissect_contextInfoList_Reset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ContextInfoList_Reset(tvb, offset, actx, tree, hf_rnsap_contextInfoList_Reset);
}


static const per_sequence_t ContextList_Reset_sequence[] = {
  { &hf_rnsap_contextInfoList_Reset, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ContextInfoList_Reset },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_ContextList_Reset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_ContextList_Reset, ContextList_Reset_sequence);

  return offset;
}
static int dissect_context(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ContextList_Reset(tvb, offset, actx, tree, hf_rnsap_context);
}


static const per_sequence_t ContextGroupInfoList_Reset_sequence_of[1] = {
  { &hf_rnsap_ContextGroupInfoList_Reset_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_ContextGroupInfoList_Reset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_ContextGroupInfoList_Reset, ContextGroupInfoList_Reset_sequence_of,
                                                  1, 32);

  return offset;
}
static int dissect_contextGroupInfoList_Reset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ContextGroupInfoList_Reset(tvb, offset, actx, tree, hf_rnsap_contextGroupInfoList_Reset);
}


static const per_sequence_t ContextGroupList_Reset_sequence[] = {
  { &hf_rnsap_contextGroupInfoList_Reset, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ContextGroupInfoList_Reset },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_ContextGroupList_Reset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_ContextGroupList_Reset, ContextGroupList_Reset_sequence);

  return offset;
}
static int dissect_contextGroup(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ContextGroupList_Reset(tvb, offset, actx, tree, hf_rnsap_contextGroup);
}


static const value_string rnsap_ResetIndicator_vals[] = {
  {   0, "context" },
  {   1, "all-contexts" },
  {   2, "contextGroup" },
  { 0, NULL }
};

static const per_choice_t ResetIndicator_choice[] = {
  {   0, &hf_rnsap_context       , ASN1_EXTENSION_ROOT    , dissect_rnsap_ContextList_Reset },
  {   1, &hf_rnsap_all_contexts  , ASN1_EXTENSION_ROOT    , dissect_rnsap_NULL },
  {   2, &hf_rnsap_contextGroup  , ASN1_NOT_EXTENSION_ROOT, dissect_rnsap_ContextGroupList_Reset },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_ResetIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_ResetIndicator, ResetIndicator_choice,
                                 NULL);

  return offset;
}
static int dissect_id_ResetIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ResetIndicator(tvb, offset, actx, tree, hf_rnsap_id_ResetIndicator);
}



static int
dissect_rnsap_DL_CodeInformationList_RL_AdditionFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_dl_CodeInformation2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_CodeInformationList_RL_AdditionFailureFDD(tvb, offset, actx, tree, hf_rnsap_dl_CodeInformation2);
}


static const per_sequence_t Combining_RL_AdditionFailureFDD_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_Combining_RL_AdditionFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_Combining_RL_AdditionFailureFDD, Combining_RL_AdditionFailureFDD_sequence);

  return offset;
}
static int dissect_combining4(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Combining_RL_AdditionFailureFDD(tvb, offset, actx, tree, hf_rnsap_combining4);
}


static const per_sequence_t NonCombining_RL_AdditionFailureFDD_sequence[] = {
  { &hf_rnsap_dCH_InformationResponse, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DCH_InformationResponse },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_NonCombining_RL_AdditionFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_NonCombining_RL_AdditionFailureFDD, NonCombining_RL_AdditionFailureFDD_sequence);

  return offset;
}
static int dissect_nonCombining2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_NonCombining_RL_AdditionFailureFDD(tvb, offset, actx, tree, hf_rnsap_nonCombining2);
}


static const value_string rnsap_DiversityIndication_RL_AdditionFailureFDD_vals[] = {
  {   0, "combining" },
  {   1, "nonCombining" },
  { 0, NULL }
};

static const per_choice_t DiversityIndication_RL_AdditionFailureFDD_choice[] = {
  {   0, &hf_rnsap_combining4    , ASN1_NO_EXTENSIONS     , dissect_rnsap_Combining_RL_AdditionFailureFDD },
  {   1, &hf_rnsap_nonCombining2 , ASN1_NO_EXTENSIONS     , dissect_rnsap_NonCombining_RL_AdditionFailureFDD },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_DiversityIndication_RL_AdditionFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_DiversityIndication_RL_AdditionFailureFDD, DiversityIndication_RL_AdditionFailureFDD_choice,
                                 NULL);

  return offset;
}
static int dissect_diversityIndication5(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DiversityIndication_RL_AdditionFailureFDD(tvb, offset, actx, tree, hf_rnsap_diversityIndication5);
}


static const per_sequence_t SuccessfulRL_InformationResponse_RL_AdditionFailureFDD_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_rL_Set_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_Set_ID },
  { &hf_rnsap_uRA_Information, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_URA_Information },
  { &hf_rnsap_sAI           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SAI },
  { &hf_rnsap_gA_Cell       , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_GA_Cell },
  { &hf_rnsap_gA_AccessPointPosition, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_GA_AccessPointPosition },
  { &hf_rnsap_received_total_wide_band_power, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_Received_total_wide_band_power },
  { &hf_rnsap_not_Used_secondary_CCPCH_Info, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_NULL },
  { &hf_rnsap_dl_CodeInformation2, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_CodeInformationList_RL_AdditionFailureFDD },
  { &hf_rnsap_diversityIndication5, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DiversityIndication_RL_AdditionFailureFDD },
  { &hf_rnsap_sSDT_SupportIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SSDT_SupportIndicator },
  { &hf_rnsap_minUL_SIR     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_SIR },
  { &hf_rnsap_maxUL_SIR     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_SIR },
  { &hf_rnsap_closedlooptimingadjustmentmode, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Closedlooptimingadjustmentmode },
  { &hf_rnsap_maximumAllowedULTxPower, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MaximumAllowedULTxPower },
  { &hf_rnsap_maximumDLTxPower, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_Power },
  { &hf_rnsap_minimumDLTxPower, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_Power },
  { &hf_rnsap_neighbouring_UMTS_CellInformation, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Neighbouring_UMTS_CellInformation },
  { &hf_rnsap_neighbouring_GSM_CellInformation, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Neighbouring_GSM_CellInformation },
  { &hf_rnsap_primaryCPICH_Power, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PrimaryCPICH_Power },
  { &hf_rnsap_pC_Preamble   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PC_Preamble },
  { &hf_rnsap_sRB_Delay     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SRB_Delay },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_SuccessfulRL_InformationResponse_RL_AdditionFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_SuccessfulRL_InformationResponse_RL_AdditionFailureFDD, SuccessfulRL_InformationResponse_RL_AdditionFailureFDD_sequence);

  return offset;
}
static int dissect_id_SuccessfulRL_InformationResponse_RL_AdditionFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SuccessfulRL_InformationResponse_RL_AdditionFailureFDD(tvb, offset, actx, tree, hf_rnsap_id_SuccessfulRL_InformationResponse_RL_AdditionFailureFDD);
}


static const per_sequence_t Combining_RL_SetupFailureFDD_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_Combining_RL_SetupFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_Combining_RL_SetupFailureFDD, Combining_RL_SetupFailureFDD_sequence);

  return offset;
}
static int dissect_combining1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Combining_RL_SetupFailureFDD(tvb, offset, actx, tree, hf_rnsap_combining1);
}


static const per_sequence_t NonCombiningOrFirstRL_RL_SetupFailureFDD_sequence[] = {
  { &hf_rnsap_dCH_InformationResponse, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DCH_InformationResponse },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_NonCombiningOrFirstRL_RL_SetupFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_NonCombiningOrFirstRL_RL_SetupFailureFDD, NonCombiningOrFirstRL_RL_SetupFailureFDD_sequence);

  return offset;
}
static int dissect_nonCombiningOrFirstRL1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_NonCombiningOrFirstRL_RL_SetupFailureFDD(tvb, offset, actx, tree, hf_rnsap_nonCombiningOrFirstRL1);
}


static const value_string rnsap_DiversityIndication_RL_SetupFailureFDD_vals[] = {
  {   0, "combining" },
  {   1, "nonCombiningOrFirstRL" },
  { 0, NULL }
};

static const per_choice_t DiversityIndication_RL_SetupFailureFDD_choice[] = {
  {   0, &hf_rnsap_combining1    , ASN1_NO_EXTENSIONS     , dissect_rnsap_Combining_RL_SetupFailureFDD },
  {   1, &hf_rnsap_nonCombiningOrFirstRL1, ASN1_NO_EXTENSIONS     , dissect_rnsap_NonCombiningOrFirstRL_RL_SetupFailureFDD },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_DiversityIndication_RL_SetupFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_DiversityIndication_RL_SetupFailureFDD, DiversityIndication_RL_SetupFailureFDD_choice,
                                 NULL);

  return offset;
}
static int dissect_diversityIndication1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DiversityIndication_RL_SetupFailureFDD(tvb, offset, actx, tree, hf_rnsap_diversityIndication1);
}


static const per_sequence_t SuccessfulRL_InformationResponse_RL_SetupFailureFDD_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_rL_Set_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_Set_ID },
  { &hf_rnsap_uRA_Information, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_URA_Information },
  { &hf_rnsap_sAI           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SAI },
  { &hf_rnsap_gA_Cell       , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_GA_Cell },
  { &hf_rnsap_gA_AccessPointPosition, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_GA_AccessPointPosition },
  { &hf_rnsap_received_total_wide_band_power, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_Received_total_wide_band_power },
  { &hf_rnsap_not_Used_secondary_CCPCH_Info, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_NULL },
  { &hf_rnsap_dl_CodeInformation, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_FDD_DL_CodeInformation },
  { &hf_rnsap_diversityIndication1, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DiversityIndication_RL_SetupFailureFDD },
  { &hf_rnsap_sSDT_SupportIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SSDT_SupportIndicator },
  { &hf_rnsap_maxUL_SIR     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_SIR },
  { &hf_rnsap_minUL_SIR     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_SIR },
  { &hf_rnsap_closedlooptimingadjustmentmode, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Closedlooptimingadjustmentmode },
  { &hf_rnsap_maximumAllowedULTxPower, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MaximumAllowedULTxPower },
  { &hf_rnsap_maximumDLTxPower, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_Power },
  { &hf_rnsap_minimumDLTxPower, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_Power },
  { &hf_rnsap_primaryCPICH_Power, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PrimaryCPICH_Power },
  { &hf_rnsap_primaryScramblingCode, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_PrimaryScramblingCode },
  { &hf_rnsap_uL_UARFCN     , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_UARFCN },
  { &hf_rnsap_dL_UARFCN     , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_UARFCN },
  { &hf_rnsap_not_Used_dSCH_InformationResponse_RL_SetupFailureFDD, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_NULL },
  { &hf_rnsap_neighbouring_UMTS_CellInformation, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Neighbouring_UMTS_CellInformation },
  { &hf_rnsap_neighbouring_GSM_CellInformation, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Neighbouring_GSM_CellInformation },
  { &hf_rnsap_pC_Preamble   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PC_Preamble },
  { &hf_rnsap_sRB_Delay     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SRB_Delay },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_SuccessfulRL_InformationResponse_RL_SetupFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_SuccessfulRL_InformationResponse_RL_SetupFailureFDD, SuccessfulRL_InformationResponse_RL_SetupFailureFDD_sequence);

  return offset;
}
static int dissect_id_SuccessfulRL_InformationResponse_RL_SetupFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SuccessfulRL_InformationResponse_RL_SetupFailureFDD(tvb, offset, actx, tree, hf_rnsap_id_SuccessfulRL_InformationResponse_RL_SetupFailureFDD);
}



static int
dissect_rnsap_TransportBearerID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 4095U, NULL, FALSE);

  return offset;
}
static int dissect_id_TransportBearerID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TransportBearerID(tvb, offset, actx, tree, hf_rnsap_id_TransportBearerID);
}


static const value_string rnsap_TypeOfError_vals[] = {
  {   0, "not-understood" },
  {   1, "missing" },
  { 0, NULL }
};


static int
dissect_rnsap_TypeOfError(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_id_TypeOfError(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TypeOfError(tvb, offset, actx, tree, hf_rnsap_id_TypeOfError);
}


static const per_sequence_t UC_ID_sequence[] = {
  { &hf_rnsap_rNC_ID        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RNC_ID },
  { &hf_rnsap_c_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_C_ID },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UC_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UC_ID, UC_ID_sequence);

  return offset;
}
static int dissect_id_UC_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UC_ID(tvb, offset, actx, tree, hf_rnsap_id_UC_ID);
}
static int dissect_uC_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UC_ID(tvb, offset, actx, tree, hf_rnsap_uC_ID);
}


static const per_sequence_t UL_CCTrCH_AddInformation_RL_ReconfPrepTDD_sequence[] = {
  { &hf_rnsap_cCTrCH_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CCTrCH_ID },
  { &hf_rnsap_tFCS          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TFCS },
  { &hf_rnsap_tFCI_Coding   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TFCI_Coding },
  { &hf_rnsap_punctureLimit , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PunctureLimit },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UL_CCTrCH_AddInformation_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UL_CCTrCH_AddInformation_RL_ReconfPrepTDD, UL_CCTrCH_AddInformation_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_id_UL_CCTrCH_AddInformation_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_CCTrCH_AddInformation_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_rnsap_id_UL_CCTrCH_AddInformation_RL_ReconfPrepTDD);
}


static const per_sequence_t UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD_sequence_of[1] = {
  { &hf_rnsap_UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD, UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD_sequence_of,
                                                  0, 16);

  return offset;
}
static int dissect_id_UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_rnsap_id_UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD);
}


static const per_sequence_t UL_CCTrCH_InformationItem_RL_SetupRqstTDD_sequence[] = {
  { &hf_rnsap_cCTrCH_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CCTrCH_ID },
  { &hf_rnsap_ul_TFCS       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TFCS },
  { &hf_rnsap_tFCI_Coding   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TFCI_Coding },
  { &hf_rnsap_ul_PunctureLimit, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PunctureLimit },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UL_CCTrCH_InformationItem_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UL_CCTrCH_InformationItem_RL_SetupRqstTDD, UL_CCTrCH_InformationItem_RL_SetupRqstTDD_sequence);

  return offset;
}
static int dissect_id_UL_CCTrCH_InformationItem_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_CCTrCH_InformationItem_RL_SetupRqstTDD(tvb, offset, actx, tree, hf_rnsap_id_UL_CCTrCH_InformationItem_RL_SetupRqstTDD);
}


static const per_sequence_t UL_CCTrCH_InformationList_RL_SetupRqstTDD_sequence_of[1] = {
  { &hf_rnsap_UL_CCTrCH_InformationList_RL_SetupRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_UL_CCTrCH_InformationList_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_UL_CCTrCH_InformationList_RL_SetupRqstTDD, UL_CCTrCH_InformationList_RL_SetupRqstTDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_UL_CCTrCH_InformationList_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_CCTrCH_InformationList_RL_SetupRqstTDD(tvb, offset, actx, tree, hf_rnsap_id_UL_CCTrCH_InformationList_RL_SetupRqstTDD);
}



static int
dissect_rnsap_UL_DPCH_InformationList_PhyChReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_ul_DPCH_Information2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_DPCH_InformationList_PhyChReconfRqstTDD(tvb, offset, actx, tree, hf_rnsap_ul_DPCH_Information2);
}


static const per_sequence_t UL_CCTrCH_InformationItem_PhyChReconfRqstTDD_sequence[] = {
  { &hf_rnsap_cCTrCH_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CCTrCH_ID },
  { &hf_rnsap_ul_DPCH_Information2, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_DPCH_InformationList_PhyChReconfRqstTDD },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UL_CCTrCH_InformationItem_PhyChReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UL_CCTrCH_InformationItem_PhyChReconfRqstTDD, UL_CCTrCH_InformationItem_PhyChReconfRqstTDD_sequence);

  return offset;
}
static int dissect_UL_CCTrCH_InformationListIE_PhyChReconfRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_CCTrCH_InformationItem_PhyChReconfRqstTDD(tvb, offset, actx, tree, hf_rnsap_UL_CCTrCH_InformationListIE_PhyChReconfRqstTDD_item);
}


static const per_sequence_t UL_CCTrCH_InformationListIE_PhyChReconfRqstTDD_sequence_of[1] = {
  { &hf_rnsap_UL_CCTrCH_InformationListIE_PhyChReconfRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_CCTrCH_InformationItem_PhyChReconfRqstTDD },
};

static int
dissect_rnsap_UL_CCTrCH_InformationListIE_PhyChReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_UL_CCTrCH_InformationListIE_PhyChReconfRqstTDD, UL_CCTrCH_InformationListIE_PhyChReconfRqstTDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_UL_CCTrCH_InformationListIE_PhyChReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_CCTrCH_InformationListIE_PhyChReconfRqstTDD(tvb, offset, actx, tree, hf_rnsap_id_UL_CCTrCH_InformationListIE_PhyChReconfRqstTDD);
}



static int
dissect_rnsap_UL_DPCH_InformationList_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_ul_DPCH_Information1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_DPCH_InformationList_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_rnsap_ul_DPCH_Information1);
}


static const per_sequence_t UL_CCTrCHInformationItem_RL_AdditionRspTDD_sequence[] = {
  { &hf_rnsap_cCTrCH_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CCTrCH_ID },
  { &hf_rnsap_ul_DPCH_Information1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_UL_DPCH_InformationList_RL_AdditionRspTDD },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UL_CCTrCHInformationItem_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UL_CCTrCHInformationItem_RL_AdditionRspTDD, UL_CCTrCHInformationItem_RL_AdditionRspTDD_sequence);

  return offset;
}
static int dissect_UL_CCTrCHInformationListIE_RL_AdditionRspTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_CCTrCHInformationItem_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_rnsap_UL_CCTrCHInformationListIE_RL_AdditionRspTDD_item);
}


static const per_sequence_t UL_CCTrCHInformationListIE_RL_AdditionRspTDD_sequence_of[1] = {
  { &hf_rnsap_UL_CCTrCHInformationListIE_RL_AdditionRspTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_CCTrCHInformationItem_RL_AdditionRspTDD },
};

static int
dissect_rnsap_UL_CCTrCHInformationListIE_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_UL_CCTrCHInformationListIE_RL_AdditionRspTDD, UL_CCTrCHInformationListIE_RL_AdditionRspTDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_UL_CCTrCH_InformationListIE_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_CCTrCHInformationListIE_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_rnsap_id_UL_CCTrCH_InformationListIE_RL_AdditionRspTDD);
}



static int
dissect_rnsap_UL_DPCH_InformationAddList_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_ul_DPCH_AddInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_DPCH_InformationAddList_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_ul_DPCH_AddInformation);
}



static int
dissect_rnsap_UL_DPCH_InformationModifyList_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_ul_DPCH_ModifyInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_DPCH_InformationModifyList_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_ul_DPCH_ModifyInformation);
}



static int
dissect_rnsap_UL_DPCH_InformationDeleteList_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_ul_DPCH_DeleteInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_DPCH_InformationDeleteList_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_ul_DPCH_DeleteInformation);
}


static const per_sequence_t UL_CCTrCH_InformationItem_RL_ReconfReadyTDD_sequence[] = {
  { &hf_rnsap_cCTrCH_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CCTrCH_ID },
  { &hf_rnsap_ul_DPCH_AddInformation, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_UL_DPCH_InformationAddList_RL_ReconfReadyTDD },
  { &hf_rnsap_ul_DPCH_ModifyInformation, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_UL_DPCH_InformationModifyList_RL_ReconfReadyTDD },
  { &hf_rnsap_ul_DPCH_DeleteInformation, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_UL_DPCH_InformationDeleteList_RL_ReconfReadyTDD },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UL_CCTrCH_InformationItem_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UL_CCTrCH_InformationItem_RL_ReconfReadyTDD, UL_CCTrCH_InformationItem_RL_ReconfReadyTDD_sequence);

  return offset;
}
static int dissect_UL_CCTrCHInformationListIE_RL_ReconfReadyTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_CCTrCH_InformationItem_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_UL_CCTrCHInformationListIE_RL_ReconfReadyTDD_item);
}


static const per_sequence_t UL_CCTrCHInformationListIE_RL_ReconfReadyTDD_sequence_of[1] = {
  { &hf_rnsap_UL_CCTrCHInformationListIE_RL_ReconfReadyTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_CCTrCH_InformationItem_RL_ReconfReadyTDD },
};

static int
dissect_rnsap_UL_CCTrCHInformationListIE_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_UL_CCTrCHInformationListIE_RL_ReconfReadyTDD, UL_CCTrCHInformationListIE_RL_ReconfReadyTDD_sequence_of,
                                                  0, 16);

  return offset;
}
static int dissect_id_UL_CCTrCH_InformationListIE_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_CCTrCHInformationListIE_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_id_UL_CCTrCH_InformationListIE_RL_ReconfReadyTDD);
}



static int
dissect_rnsap_UL_DPCH_InformationList_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_ul_DPCH_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_DPCH_InformationList_RL_SetupRspTDD(tvb, offset, actx, tree, hf_rnsap_ul_DPCH_Information);
}


static const per_sequence_t UL_CCTrCHInformationItem_RL_SetupRspTDD_sequence[] = {
  { &hf_rnsap_cCTrCH_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CCTrCH_ID },
  { &hf_rnsap_ul_DPCH_Information, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_UL_DPCH_InformationList_RL_SetupRspTDD },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UL_CCTrCHInformationItem_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UL_CCTrCHInformationItem_RL_SetupRspTDD, UL_CCTrCHInformationItem_RL_SetupRspTDD_sequence);

  return offset;
}
static int dissect_UL_CCTrCHInformationListIE_RL_SetupRspTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_CCTrCHInformationItem_RL_SetupRspTDD(tvb, offset, actx, tree, hf_rnsap_UL_CCTrCHInformationListIE_RL_SetupRspTDD_item);
}


static const per_sequence_t UL_CCTrCHInformationListIE_RL_SetupRspTDD_sequence_of[1] = {
  { &hf_rnsap_UL_CCTrCHInformationListIE_RL_SetupRspTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_CCTrCHInformationItem_RL_SetupRspTDD },
};

static int
dissect_rnsap_UL_CCTrCHInformationListIE_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_UL_CCTrCHInformationListIE_RL_SetupRspTDD, UL_CCTrCHInformationListIE_RL_SetupRspTDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_UL_CCTrCH_InformationListIE_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_CCTrCHInformationListIE_RL_SetupRspTDD(tvb, offset, actx, tree, hf_rnsap_id_UL_CCTrCH_InformationListIE_RL_SetupRspTDD);
}



static int
dissect_rnsap_UL_ScramblingCodeNumber(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 16777215U, NULL, FALSE);

  return offset;
}
static int dissect_ul_ScramblingCodeNumber(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_ScramblingCodeNumber(tvb, offset, actx, tree, hf_rnsap_ul_ScramblingCodeNumber);
}


static const value_string rnsap_UL_ScramblingCodeLength_vals[] = {
  {   0, "short" },
  {   1, "long" },
  { 0, NULL }
};


static int
dissect_rnsap_UL_ScramblingCodeLength(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_ul_ScramblingCodeLength(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_ScramblingCodeLength(tvb, offset, actx, tree, hf_rnsap_ul_ScramblingCodeLength);
}


static const per_sequence_t UL_ScramblingCode_sequence[] = {
  { &hf_rnsap_ul_ScramblingCodeNumber, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_ScramblingCodeNumber },
  { &hf_rnsap_ul_ScramblingCodeLength, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_ScramblingCodeLength },
  { &hf_rnsap_iE_Extensions , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UL_ScramblingCode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UL_ScramblingCode, UL_ScramblingCode_sequence);

  return offset;
}
static int dissect_ul_ScramblingCode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_ScramblingCode(tvb, offset, actx, tree, hf_rnsap_ul_ScramblingCode);
}


static const value_string rnsap_MinUL_ChannelisationCodeLength_vals[] = {
  {   0, "v4" },
  {   1, "v8" },
  {   2, "v16" },
  {   3, "v32" },
  {   4, "v64" },
  {   5, "v128" },
  {   6, "v256" },
  { 0, NULL }
};


static int
dissect_rnsap_MinUL_ChannelisationCodeLength(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     7, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_minUL_ChannelisationCodeLength(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MinUL_ChannelisationCodeLength(tvb, offset, actx, tree, hf_rnsap_minUL_ChannelisationCodeLength);
}



static int
dissect_rnsap_MaxNrOfUL_DPCHs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 6U, NULL, FALSE);

  return offset;
}
static int dissect_maxNrOfUL_DPCHs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MaxNrOfUL_DPCHs(tvb, offset, actx, tree, hf_rnsap_maxNrOfUL_DPCHs);
}
static int dissect_maxNrOfUL_DPDCHs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MaxNrOfUL_DPCHs(tvb, offset, actx, tree, hf_rnsap_maxNrOfUL_DPDCHs);
}



static int
dissect_rnsap_UL_DPCCH_SlotFormat(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 5U, NULL, TRUE);

  return offset;
}
static int dissect_ul_DPCCH_SlotFormat(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_DPCCH_SlotFormat(tvb, offset, actx, tree, hf_rnsap_ul_DPCCH_SlotFormat);
}


static const value_string rnsap_DiversityMode_vals[] = {
  {   0, "none" },
  {   1, "sTTD" },
  {   2, "closedLoopMode1" },
  {   3, "not-used-closedLoopMode2" },
  { 0, NULL }
};


static int
dissect_rnsap_DiversityMode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_diversityMode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DiversityMode(tvb, offset, actx, tree, hf_rnsap_diversityMode);
}


static const per_sequence_t UL_DPCH_Information_RL_ReconfPrepFDD_sequence[] = {
  { &hf_rnsap_ul_ScramblingCode, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_UL_ScramblingCode },
  { &hf_rnsap_ul_SIRTarget  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_UL_SIR },
  { &hf_rnsap_minUL_ChannelisationCodeLength, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_MinUL_ChannelisationCodeLength },
  { &hf_rnsap_maxNrOfUL_DPDCHs, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_MaxNrOfUL_DPCHs },
  { &hf_rnsap_ul_PunctureLimit, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_PunctureLimit },
  { &hf_rnsap_tFCS          , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TFCS },
  { &hf_rnsap_ul_DPCCH_SlotFormat, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_UL_DPCCH_SlotFormat },
  { &hf_rnsap_diversityMode , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DiversityMode },
  { &hf_rnsap_not_Used_sSDT_CellIDLength, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_NULL },
  { &hf_rnsap_not_Used_s_FieldLength, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_NULL },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UL_DPCH_Information_RL_ReconfPrepFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UL_DPCH_Information_RL_ReconfPrepFDD, UL_DPCH_Information_RL_ReconfPrepFDD_sequence);

  return offset;
}
static int dissect_id_UL_DPCH_Information_RL_ReconfPrepFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_DPCH_Information_RL_ReconfPrepFDD(tvb, offset, actx, tree, hf_rnsap_id_UL_DPCH_Information_RL_ReconfPrepFDD);
}


static const per_sequence_t UL_DPCH_Information_RL_ReconfRqstFDD_sequence[] = {
  { &hf_rnsap_tFCS          , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TFCS },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UL_DPCH_Information_RL_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UL_DPCH_Information_RL_ReconfRqstFDD, UL_DPCH_Information_RL_ReconfRqstFDD_sequence);

  return offset;
}
static int dissect_id_UL_DPCH_Information_RL_ReconfRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_DPCH_Information_RL_ReconfRqstFDD(tvb, offset, actx, tree, hf_rnsap_id_UL_DPCH_Information_RL_ReconfRqstFDD);
}


static const per_sequence_t UL_DPCH_Information_RL_SetupRqstFDD_sequence[] = {
  { &hf_rnsap_ul_ScramblingCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_ScramblingCode },
  { &hf_rnsap_minUL_ChannelisationCodeLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MinUL_ChannelisationCodeLength },
  { &hf_rnsap_maxNrOfUL_DPCHs, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_MaxNrOfUL_DPCHs },
  { &hf_rnsap_ul_PunctureLimit, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PunctureLimit },
  { &hf_rnsap_ul_TFCS       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TFCS },
  { &hf_rnsap_ul_DPCCH_SlotFormat, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_DPCCH_SlotFormat },
  { &hf_rnsap_ul_SIRTarget  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_UL_SIR },
  { &hf_rnsap_diversityMode , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DiversityMode },
  { &hf_rnsap_not_Used_sSDT_CellIdLength, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_NULL },
  { &hf_rnsap_not_Used_s_FieldLength, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_NULL },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UL_DPCH_Information_RL_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UL_DPCH_Information_RL_SetupRqstFDD, UL_DPCH_Information_RL_SetupRqstFDD_sequence);

  return offset;
}
static int dissect_id_UL_DPCH_Information_RL_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_DPCH_Information_RL_SetupRqstFDD(tvb, offset, actx, tree, hf_rnsap_id_UL_DPCH_Information_RL_SetupRqstFDD);
}


static const per_sequence_t TDD_UL_Code_InformationItem_sequence[] = {
  { &hf_rnsap_dPCH_ID       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DPCH_ID },
  { &hf_rnsap_tDD_ChannelisationCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_ChannelisationCode },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_TDD_UL_Code_InformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_TDD_UL_Code_InformationItem, TDD_UL_Code_InformationItem_sequence);

  return offset;
}
static int dissect_TDD_UL_Code_Information_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_UL_Code_InformationItem(tvb, offset, actx, tree, hf_rnsap_TDD_UL_Code_Information_item);
}


static const per_sequence_t TDD_UL_Code_Information_sequence_of[1] = {
  { &hf_rnsap_TDD_UL_Code_Information_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_UL_Code_InformationItem },
};

static int
dissect_rnsap_TDD_UL_Code_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_TDD_UL_Code_Information, TDD_UL_Code_Information_sequence_of,
                                                  1, 240);

  return offset;
}
static int dissect_uL_Code_Information1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_UL_Code_Information(tvb, offset, actx, tree, hf_rnsap_uL_Code_Information1);
}


static const per_sequence_t UL_Timeslot_InformationItem_PhyChReconfRqstTDD_sequence[] = {
  { &hf_rnsap_timeSlot      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TimeSlot },
  { &hf_rnsap_midambleShiftAndBurstType, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_MidambleShiftAndBurstType },
  { &hf_rnsap_tFCI_Presence , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TFCI_Presence },
  { &hf_rnsap_uL_Code_Information1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TDD_UL_Code_Information },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UL_Timeslot_InformationItem_PhyChReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UL_Timeslot_InformationItem_PhyChReconfRqstTDD, UL_Timeslot_InformationItem_PhyChReconfRqstTDD_sequence);

  return offset;
}
static int dissect_UL_Timeslot_InformationList_PhyChReconfRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_Timeslot_InformationItem_PhyChReconfRqstTDD(tvb, offset, actx, tree, hf_rnsap_UL_Timeslot_InformationList_PhyChReconfRqstTDD_item);
}


static const per_sequence_t UL_Timeslot_InformationList_PhyChReconfRqstTDD_sequence_of[1] = {
  { &hf_rnsap_UL_Timeslot_InformationList_PhyChReconfRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_Timeslot_InformationItem_PhyChReconfRqstTDD },
};

static int
dissect_rnsap_UL_Timeslot_InformationList_PhyChReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_UL_Timeslot_InformationList_PhyChReconfRqstTDD, UL_Timeslot_InformationList_PhyChReconfRqstTDD_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_uL_Timeslot_InformationList_PhyChReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_Timeslot_InformationList_PhyChReconfRqstTDD(tvb, offset, actx, tree, hf_rnsap_uL_Timeslot_InformationList_PhyChReconfRqstTDD);
}


static const per_sequence_t UL_DPCH_InformationItem_PhyChReconfRqstTDD_sequence[] = {
  { &hf_rnsap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_RepetitionPeriod },
  { &hf_rnsap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_RepetitionLength },
  { &hf_rnsap_tDD_DPCHOffset, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TDD_DPCHOffset },
  { &hf_rnsap_uL_Timeslot_InformationList_PhyChReconfRqstTDD, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_UL_Timeslot_InformationList_PhyChReconfRqstTDD },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UL_DPCH_InformationItem_PhyChReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UL_DPCH_InformationItem_PhyChReconfRqstTDD, UL_DPCH_InformationItem_PhyChReconfRqstTDD_sequence);

  return offset;
}
static int dissect_id_UL_DPCH_InformationItem_PhyChReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_DPCH_InformationItem_PhyChReconfRqstTDD(tvb, offset, actx, tree, hf_rnsap_id_UL_DPCH_InformationItem_PhyChReconfRqstTDD);
}


static const per_sequence_t UL_Timeslot_InformationItem_sequence[] = {
  { &hf_rnsap_timeSlot      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TimeSlot },
  { &hf_rnsap_midambleShiftAndBurstType, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MidambleShiftAndBurstType },
  { &hf_rnsap_tFCI_Presence , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TFCI_Presence },
  { &hf_rnsap_uL_Code_Information1, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_UL_Code_Information },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UL_Timeslot_InformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UL_Timeslot_InformationItem, UL_Timeslot_InformationItem_sequence);

  return offset;
}
static int dissect_UL_Timeslot_Information_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_Timeslot_InformationItem(tvb, offset, actx, tree, hf_rnsap_UL_Timeslot_Information_item);
}


static const per_sequence_t UL_Timeslot_Information_sequence_of[1] = {
  { &hf_rnsap_UL_Timeslot_Information_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_Timeslot_InformationItem },
};

static int
dissect_rnsap_UL_Timeslot_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_UL_Timeslot_Information, UL_Timeslot_Information_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_uL_Timeslot_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_Timeslot_Information(tvb, offset, actx, tree, hf_rnsap_uL_Timeslot_Information);
}


static const per_sequence_t UL_DPCH_InformationItem_RL_AdditionRspTDD_sequence[] = {
  { &hf_rnsap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RepetitionPeriod },
  { &hf_rnsap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RepetitionLength },
  { &hf_rnsap_tDD_DPCHOffset, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_DPCHOffset },
  { &hf_rnsap_uL_Timeslot_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_Timeslot_Information },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UL_DPCH_InformationItem_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UL_DPCH_InformationItem_RL_AdditionRspTDD, UL_DPCH_InformationItem_RL_AdditionRspTDD_sequence);

  return offset;
}
static int dissect_id_UL_DPCH_InformationItem_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_DPCH_InformationItem_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_rnsap_id_UL_DPCH_InformationItem_RL_AdditionRspTDD);
}


static const per_sequence_t UL_DPCH_InformationItem_RL_SetupRspTDD_sequence[] = {
  { &hf_rnsap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RepetitionPeriod },
  { &hf_rnsap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RepetitionLength },
  { &hf_rnsap_tDD_DPCHOffset, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_DPCHOffset },
  { &hf_rnsap_uL_Timeslot_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_Timeslot_Information },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UL_DPCH_InformationItem_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UL_DPCH_InformationItem_RL_SetupRspTDD, UL_DPCH_InformationItem_RL_SetupRspTDD_sequence);

  return offset;
}
static int dissect_id_UL_DPCH_InformationItem_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_DPCH_InformationItem_RL_SetupRspTDD(tvb, offset, actx, tree, hf_rnsap_id_UL_DPCH_InformationItem_RL_SetupRspTDD);
}



static int
dissect_rnsap_RxTimingDeviationForTA(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 127U, NULL, FALSE);

  return offset;
}
static int dissect_id_RxTimingDeviationForTA(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RxTimingDeviationForTA(tvb, offset, actx, tree, hf_rnsap_id_RxTimingDeviationForTA);
}
static int dissect_rxTimingDeviationForTA(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RxTimingDeviationForTA(tvb, offset, actx, tree, hf_rnsap_rxTimingDeviationForTA);
}


static const per_sequence_t UL_DPCH_InformationAddListIE_RL_ReconfReadyTDD_sequence[] = {
  { &hf_rnsap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RepetitionPeriod },
  { &hf_rnsap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RepetitionLength },
  { &hf_rnsap_tDD_DPCHOffset, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_DPCHOffset },
  { &hf_rnsap_rxTimingDeviationForTA, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_RxTimingDeviationForTA },
  { &hf_rnsap_uL_Timeslot_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_Timeslot_Information },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UL_DPCH_InformationAddListIE_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UL_DPCH_InformationAddListIE_RL_ReconfReadyTDD, UL_DPCH_InformationAddListIE_RL_ReconfReadyTDD_sequence);

  return offset;
}
static int dissect_id_UL_DPCH_InformationAddListIE_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_DPCH_InformationAddListIE_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_id_UL_DPCH_InformationAddListIE_RL_ReconfReadyTDD);
}


static const per_sequence_t UnsuccessfulRL_InformationResponse_RL_AdditionFailureFDD_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_cause         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_Cause },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UnsuccessfulRL_InformationResponse_RL_AdditionFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UnsuccessfulRL_InformationResponse_RL_AdditionFailureFDD, UnsuccessfulRL_InformationResponse_RL_AdditionFailureFDD_sequence);

  return offset;
}
static int dissect_id_UnsuccessfulRL_InformationResponse_RL_AdditionFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UnsuccessfulRL_InformationResponse_RL_AdditionFailureFDD(tvb, offset, actx, tree, hf_rnsap_id_UnsuccessfulRL_InformationResponse_RL_AdditionFailureFDD);
}


static const per_sequence_t UnsuccessfulRL_InformationResponse_RL_SetupFailureFDD_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_cause         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_Cause },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UnsuccessfulRL_InformationResponse_RL_SetupFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UnsuccessfulRL_InformationResponse_RL_SetupFailureFDD, UnsuccessfulRL_InformationResponse_RL_SetupFailureFDD_sequence);

  return offset;
}
static int dissect_id_UnsuccessfulRL_InformationResponse_RL_SetupFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UnsuccessfulRL_InformationResponse_RL_SetupFailureFDD(tvb, offset, actx, tree, hf_rnsap_id_UnsuccessfulRL_InformationResponse_RL_SetupFailureFDD);
}


static const per_sequence_t UnsuccessfulRL_InformationResponse_RL_SetupFailureTDD_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_cause         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_Cause },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UnsuccessfulRL_InformationResponse_RL_SetupFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UnsuccessfulRL_InformationResponse_RL_SetupFailureTDD, UnsuccessfulRL_InformationResponse_RL_SetupFailureTDD_sequence);

  return offset;
}
static int dissect_id_UnsuccessfulRL_InformationResponse_RL_SetupFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UnsuccessfulRL_InformationResponse_RL_SetupFailureTDD(tvb, offset, actx, tree, hf_rnsap_id_UnsuccessfulRL_InformationResponse_RL_SetupFailureTDD);
}



static int
dissect_rnsap_TGPSID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 6U, NULL, FALSE);

  return offset;
}
static int dissect_tGPSID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TGPSID(tvb, offset, actx, tree, hf_rnsap_tGPSID);
}



static int
dissect_rnsap_TGPRC(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 511U, NULL, FALSE);

  return offset;
}
static int dissect_tGPRC(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TGPRC(tvb, offset, actx, tree, hf_rnsap_tGPRC);
}


static const per_sequence_t Transmission_Gap_Pattern_Sequence_Status_List_item_sequence[] = {
  { &hf_rnsap_tGPSID        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TGPSID },
  { &hf_rnsap_tGPRC         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TGPRC },
  { &hf_rnsap_tGCFN         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CFN },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_Transmission_Gap_Pattern_Sequence_Status_List_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_Transmission_Gap_Pattern_Sequence_Status_List_item, Transmission_Gap_Pattern_Sequence_Status_List_item_sequence);

  return offset;
}
static int dissect_Transmission_Gap_Pattern_Sequence_Status_List_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Transmission_Gap_Pattern_Sequence_Status_List_item(tvb, offset, actx, tree, hf_rnsap_Transmission_Gap_Pattern_Sequence_Status_List_item);
}


static const per_sequence_t Transmission_Gap_Pattern_Sequence_Status_List_sequence_of[1] = {
  { &hf_rnsap_Transmission_Gap_Pattern_Sequence_Status_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_Transmission_Gap_Pattern_Sequence_Status_List_item },
};

static int
dissect_rnsap_Transmission_Gap_Pattern_Sequence_Status_List(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_Transmission_Gap_Pattern_Sequence_Status_List, Transmission_Gap_Pattern_Sequence_Status_List_sequence_of,
                                                  0, 6);

  return offset;
}
static int dissect_transmission_Gap_Pattern_Sequence_Status(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Transmission_Gap_Pattern_Sequence_Status_List(tvb, offset, actx, tree, hf_rnsap_transmission_Gap_Pattern_Sequence_Status);
}


static const per_sequence_t Active_Pattern_Sequence_Information_sequence[] = {
  { &hf_rnsap_cMConfigurationChangeCFN, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CFN },
  { &hf_rnsap_transmission_Gap_Pattern_Sequence_Status, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Transmission_Gap_Pattern_Sequence_Status_List },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_Active_Pattern_Sequence_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_Active_Pattern_Sequence_Information, Active_Pattern_Sequence_Information_sequence);

  return offset;
}
static int dissect_id_Active_Pattern_Sequence_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Active_Pattern_Sequence_Information(tvb, offset, actx, tree, hf_rnsap_id_Active_Pattern_Sequence_Information);
}



static int
dissect_rnsap_ScaledAdjustmentRatio(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 100U, NULL, FALSE);

  return offset;
}
static int dissect_id_AdjustmentRatio(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ScaledAdjustmentRatio(tvb, offset, actx, tree, hf_rnsap_id_AdjustmentRatio);
}
static int dissect_adjustmentRatio(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ScaledAdjustmentRatio(tvb, offset, actx, tree, hf_rnsap_adjustmentRatio);
}


static const per_sequence_t GeneralCauseList_RL_AdditionFailureFDD_sequence[] = {
  { &hf_rnsap_cause         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_Cause },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_GeneralCauseList_RL_AdditionFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_GeneralCauseList_RL_AdditionFailureFDD, GeneralCauseList_RL_AdditionFailureFDD_sequence);

  return offset;
}
static int dissect_generalCause2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GeneralCauseList_RL_AdditionFailureFDD(tvb, offset, actx, tree, hf_rnsap_generalCause2);
}


static const per_sequence_t UnsuccessfulRL_InformationResponseList_RL_AdditionFailureFDD_sequence_of[1] = {
  { &hf_rnsap_UnsuccessfulRL_InformationResponseList_RL_AdditionFailureFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_UnsuccessfulRL_InformationResponseList_RL_AdditionFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_UnsuccessfulRL_InformationResponseList_RL_AdditionFailureFDD, UnsuccessfulRL_InformationResponseList_RL_AdditionFailureFDD_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_unsuccessful_RL_InformationRespList_RL_AdditionFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UnsuccessfulRL_InformationResponseList_RL_AdditionFailureFDD(tvb, offset, actx, tree, hf_rnsap_unsuccessful_RL_InformationRespList_RL_AdditionFailureFDD);
}


static const per_sequence_t SuccessfulRL_InformationResponseList_RL_AdditionFailureFDD_sequence_of[1] = {
  { &hf_rnsap_SuccessfulRL_InformationResponseList_RL_AdditionFailureFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_SuccessfulRL_InformationResponseList_RL_AdditionFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_SuccessfulRL_InformationResponseList_RL_AdditionFailureFDD, SuccessfulRL_InformationResponseList_RL_AdditionFailureFDD_sequence_of,
                                                  0, 14);

  return offset;
}
static int dissect_successful_RL_InformationRespList_RL_AdditionFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SuccessfulRL_InformationResponseList_RL_AdditionFailureFDD(tvb, offset, actx, tree, hf_rnsap_successful_RL_InformationRespList_RL_AdditionFailureFDD);
}


static const per_sequence_t RLSpecificCauseList_RL_AdditionFailureFDD_sequence[] = {
  { &hf_rnsap_unsuccessful_RL_InformationRespList_RL_AdditionFailureFDD, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UnsuccessfulRL_InformationResponseList_RL_AdditionFailureFDD },
  { &hf_rnsap_successful_RL_InformationRespList_RL_AdditionFailureFDD, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_SuccessfulRL_InformationResponseList_RL_AdditionFailureFDD },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RLSpecificCauseList_RL_AdditionFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RLSpecificCauseList_RL_AdditionFailureFDD, RLSpecificCauseList_RL_AdditionFailureFDD_sequence);

  return offset;
}
static int dissect_rLSpecificCause2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RLSpecificCauseList_RL_AdditionFailureFDD(tvb, offset, actx, tree, hf_rnsap_rLSpecificCause2);
}


static const value_string rnsap_CauseLevel_RL_AdditionFailureFDD_vals[] = {
  {   0, "generalCause" },
  {   1, "rLSpecificCause" },
  { 0, NULL }
};

static const per_choice_t CauseLevel_RL_AdditionFailureFDD_choice[] = {
  {   0, &hf_rnsap_generalCause2 , ASN1_EXTENSION_ROOT    , dissect_rnsap_GeneralCauseList_RL_AdditionFailureFDD },
  {   1, &hf_rnsap_rLSpecificCause2, ASN1_EXTENSION_ROOT    , dissect_rnsap_RLSpecificCauseList_RL_AdditionFailureFDD },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_CauseLevel_RL_AdditionFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_CauseLevel_RL_AdditionFailureFDD, CauseLevel_RL_AdditionFailureFDD_choice,
                                 NULL);

  return offset;
}
static int dissect_id_CauseLevel_RL_AdditionFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CauseLevel_RL_AdditionFailureFDD(tvb, offset, actx, tree, hf_rnsap_id_CauseLevel_RL_AdditionFailureFDD);
}


static const per_sequence_t GeneralCauseList_RL_AdditionFailureTDD_sequence[] = {
  { &hf_rnsap_cause         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_Cause },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_GeneralCauseList_RL_AdditionFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_GeneralCauseList_RL_AdditionFailureTDD, GeneralCauseList_RL_AdditionFailureTDD_sequence);

  return offset;
}
static int dissect_generalCause3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GeneralCauseList_RL_AdditionFailureTDD(tvb, offset, actx, tree, hf_rnsap_generalCause3);
}



static int
dissect_rnsap_Unsuccessful_RL_InformationRespItem_RL_AdditionFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_unsuccessful_RL_InformationRespItem_RL_AdditionFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Unsuccessful_RL_InformationRespItem_RL_AdditionFailureTDD(tvb, offset, actx, tree, hf_rnsap_unsuccessful_RL_InformationRespItem_RL_AdditionFailureTDD);
}


static const per_sequence_t RLSpecificCauseList_RL_AdditionFailureTDD_sequence[] = {
  { &hf_rnsap_unsuccessful_RL_InformationRespItem_RL_AdditionFailureTDD, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_Unsuccessful_RL_InformationRespItem_RL_AdditionFailureTDD },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RLSpecificCauseList_RL_AdditionFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RLSpecificCauseList_RL_AdditionFailureTDD, RLSpecificCauseList_RL_AdditionFailureTDD_sequence);

  return offset;
}
static int dissect_rLSpecificCause3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RLSpecificCauseList_RL_AdditionFailureTDD(tvb, offset, actx, tree, hf_rnsap_rLSpecificCause3);
}


static const value_string rnsap_CauseLevel_RL_AdditionFailureTDD_vals[] = {
  {   0, "generalCause" },
  {   1, "rLSpecificCause" },
  { 0, NULL }
};

static const per_choice_t CauseLevel_RL_AdditionFailureTDD_choice[] = {
  {   0, &hf_rnsap_generalCause3 , ASN1_EXTENSION_ROOT    , dissect_rnsap_GeneralCauseList_RL_AdditionFailureTDD },
  {   1, &hf_rnsap_rLSpecificCause3, ASN1_EXTENSION_ROOT    , dissect_rnsap_RLSpecificCauseList_RL_AdditionFailureTDD },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_CauseLevel_RL_AdditionFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_CauseLevel_RL_AdditionFailureTDD, CauseLevel_RL_AdditionFailureTDD_choice,
                                 NULL);

  return offset;
}
static int dissect_id_CauseLevel_RL_AdditionFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CauseLevel_RL_AdditionFailureTDD(tvb, offset, actx, tree, hf_rnsap_id_CauseLevel_RL_AdditionFailureTDD);
}


static const per_sequence_t GeneralCauseList_RL_ReconfFailure_sequence[] = {
  { &hf_rnsap_cause         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_Cause },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_GeneralCauseList_RL_ReconfFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_GeneralCauseList_RL_ReconfFailure, GeneralCauseList_RL_ReconfFailure_sequence);

  return offset;
}
static int dissect_generalCause4(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GeneralCauseList_RL_ReconfFailure(tvb, offset, actx, tree, hf_rnsap_generalCause4);
}


static const per_sequence_t RL_ReconfigurationFailureList_RL_ReconfFailure_sequence_of[1] = {
  { &hf_rnsap_RL_ReconfigurationFailureList_RL_ReconfFailure_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_RL_ReconfigurationFailureList_RL_ReconfFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_RL_ReconfigurationFailureList_RL_ReconfFailure, RL_ReconfigurationFailureList_RL_ReconfFailure_sequence_of,
                                                  0, 16);

  return offset;
}
static int dissect_rL_ReconfigurationFailureList_RL_ReconfFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_ReconfigurationFailureList_RL_ReconfFailure(tvb, offset, actx, tree, hf_rnsap_rL_ReconfigurationFailureList_RL_ReconfFailure);
}


static const per_sequence_t RLSpecificCauseList_RL_ReconfFailure_sequence[] = {
  { &hf_rnsap_rL_ReconfigurationFailureList_RL_ReconfFailure, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_RL_ReconfigurationFailureList_RL_ReconfFailure },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RLSpecificCauseList_RL_ReconfFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RLSpecificCauseList_RL_ReconfFailure, RLSpecificCauseList_RL_ReconfFailure_sequence);

  return offset;
}
static int dissect_rLSpecificCause4(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RLSpecificCauseList_RL_ReconfFailure(tvb, offset, actx, tree, hf_rnsap_rLSpecificCause4);
}


static const value_string rnsap_CauseLevel_RL_ReconfFailure_vals[] = {
  {   0, "generalCause" },
  {   1, "rLSpecificCause" },
  { 0, NULL }
};

static const per_choice_t CauseLevel_RL_ReconfFailure_choice[] = {
  {   0, &hf_rnsap_generalCause4 , ASN1_EXTENSION_ROOT    , dissect_rnsap_GeneralCauseList_RL_ReconfFailure },
  {   1, &hf_rnsap_rLSpecificCause4, ASN1_EXTENSION_ROOT    , dissect_rnsap_RLSpecificCauseList_RL_ReconfFailure },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_CauseLevel_RL_ReconfFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_CauseLevel_RL_ReconfFailure, CauseLevel_RL_ReconfFailure_choice,
                                 NULL);

  return offset;
}
static int dissect_id_CauseLevel_RL_ReconfFailure(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CauseLevel_RL_ReconfFailure(tvb, offset, actx, tree, hf_rnsap_id_CauseLevel_RL_ReconfFailure);
}


static const per_sequence_t GeneralCauseList_RL_SetupFailureFDD_sequence[] = {
  { &hf_rnsap_cause         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_Cause },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_GeneralCauseList_RL_SetupFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_GeneralCauseList_RL_SetupFailureFDD, GeneralCauseList_RL_SetupFailureFDD_sequence);

  return offset;
}
static int dissect_generalCause(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GeneralCauseList_RL_SetupFailureFDD(tvb, offset, actx, tree, hf_rnsap_generalCause);
}


static const per_sequence_t UnsuccessfulRL_InformationResponseList_RL_SetupFailureFDD_sequence_of[1] = {
  { &hf_rnsap_UnsuccessfulRL_InformationResponseList_RL_SetupFailureFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_UnsuccessfulRL_InformationResponseList_RL_SetupFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_UnsuccessfulRL_InformationResponseList_RL_SetupFailureFDD, UnsuccessfulRL_InformationResponseList_RL_SetupFailureFDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_unsuccessful_RL_InformationRespList_RL_SetupFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UnsuccessfulRL_InformationResponseList_RL_SetupFailureFDD(tvb, offset, actx, tree, hf_rnsap_unsuccessful_RL_InformationRespList_RL_SetupFailureFDD);
}


static const per_sequence_t SuccessfulRL_InformationResponseList_RL_SetupFailureFDD_sequence_of[1] = {
  { &hf_rnsap_SuccessfulRL_InformationResponseList_RL_SetupFailureFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_SuccessfulRL_InformationResponseList_RL_SetupFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_SuccessfulRL_InformationResponseList_RL_SetupFailureFDD, SuccessfulRL_InformationResponseList_RL_SetupFailureFDD_sequence_of,
                                                  0, 15);

  return offset;
}
static int dissect_successful_RL_InformationRespList_RL_SetupFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SuccessfulRL_InformationResponseList_RL_SetupFailureFDD(tvb, offset, actx, tree, hf_rnsap_successful_RL_InformationRespList_RL_SetupFailureFDD);
}


static const per_sequence_t RLSpecificCauseList_RL_SetupFailureFDD_sequence[] = {
  { &hf_rnsap_unsuccessful_RL_InformationRespList_RL_SetupFailureFDD, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UnsuccessfulRL_InformationResponseList_RL_SetupFailureFDD },
  { &hf_rnsap_successful_RL_InformationRespList_RL_SetupFailureFDD, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_SuccessfulRL_InformationResponseList_RL_SetupFailureFDD },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RLSpecificCauseList_RL_SetupFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RLSpecificCauseList_RL_SetupFailureFDD, RLSpecificCauseList_RL_SetupFailureFDD_sequence);

  return offset;
}
static int dissect_rLSpecificCause(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RLSpecificCauseList_RL_SetupFailureFDD(tvb, offset, actx, tree, hf_rnsap_rLSpecificCause);
}


static const value_string rnsap_CauseLevel_RL_SetupFailureFDD_vals[] = {
  {   0, "generalCause" },
  {   1, "rLSpecificCause" },
  { 0, NULL }
};

static const per_choice_t CauseLevel_RL_SetupFailureFDD_choice[] = {
  {   0, &hf_rnsap_generalCause  , ASN1_EXTENSION_ROOT    , dissect_rnsap_GeneralCauseList_RL_SetupFailureFDD },
  {   1, &hf_rnsap_rLSpecificCause, ASN1_EXTENSION_ROOT    , dissect_rnsap_RLSpecificCauseList_RL_SetupFailureFDD },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_CauseLevel_RL_SetupFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_CauseLevel_RL_SetupFailureFDD, CauseLevel_RL_SetupFailureFDD_choice,
                                 NULL);

  return offset;
}
static int dissect_id_CauseLevel_RL_SetupFailureFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CauseLevel_RL_SetupFailureFDD(tvb, offset, actx, tree, hf_rnsap_id_CauseLevel_RL_SetupFailureFDD);
}


static const per_sequence_t GeneralCauseList_RL_SetupFailureTDD_sequence[] = {
  { &hf_rnsap_cause         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_Cause },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_GeneralCauseList_RL_SetupFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_GeneralCauseList_RL_SetupFailureTDD, GeneralCauseList_RL_SetupFailureTDD_sequence);

  return offset;
}
static int dissect_generalCause1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GeneralCauseList_RL_SetupFailureTDD(tvb, offset, actx, tree, hf_rnsap_generalCause1);
}



static int
dissect_rnsap_Unsuccessful_RL_InformationRespItem_RL_SetupFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_unsuccessful_RL_InformationRespItem_RL_SetupFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Unsuccessful_RL_InformationRespItem_RL_SetupFailureTDD(tvb, offset, actx, tree, hf_rnsap_unsuccessful_RL_InformationRespItem_RL_SetupFailureTDD);
}


static const per_sequence_t RLSpecificCauseList_RL_SetupFailureTDD_sequence[] = {
  { &hf_rnsap_unsuccessful_RL_InformationRespItem_RL_SetupFailureTDD, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_Unsuccessful_RL_InformationRespItem_RL_SetupFailureTDD },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RLSpecificCauseList_RL_SetupFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RLSpecificCauseList_RL_SetupFailureTDD, RLSpecificCauseList_RL_SetupFailureTDD_sequence);

  return offset;
}
static int dissect_rLSpecificCause1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RLSpecificCauseList_RL_SetupFailureTDD(tvb, offset, actx, tree, hf_rnsap_rLSpecificCause1);
}


static const value_string rnsap_CauseLevel_RL_SetupFailureTDD_vals[] = {
  {   0, "generalCause" },
  {   1, "rLSpecificCause" },
  { 0, NULL }
};

static const per_choice_t CauseLevel_RL_SetupFailureTDD_choice[] = {
  {   0, &hf_rnsap_generalCause1 , ASN1_EXTENSION_ROOT    , dissect_rnsap_GeneralCauseList_RL_SetupFailureTDD },
  {   1, &hf_rnsap_rLSpecificCause1, ASN1_EXTENSION_ROOT    , dissect_rnsap_RLSpecificCauseList_RL_SetupFailureTDD },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_CauseLevel_RL_SetupFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_CauseLevel_RL_SetupFailureTDD, CauseLevel_RL_SetupFailureTDD_choice,
                                 NULL);

  return offset;
}
static int dissect_id_CauseLevel_RL_SetupFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CauseLevel_RL_SetupFailureTDD(tvb, offset, actx, tree, hf_rnsap_id_CauseLevel_RL_SetupFailureTDD);
}


static const per_sequence_t DL_CCTrCH_InformationDeleteItem_RL_ReconfPrepTDD_sequence[] = {
  { &hf_rnsap_cCTrCH_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CCTrCH_ID },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_CCTrCH_InformationDeleteItem_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_CCTrCH_InformationDeleteItem_RL_ReconfPrepTDD, DL_CCTrCH_InformationDeleteItem_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_id_DL_CCTrCH_InformationDeleteItem_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_CCTrCH_InformationDeleteItem_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_rnsap_id_DL_CCTrCH_InformationDeleteItem_RL_ReconfPrepTDD);
}


static const per_sequence_t CCTrCH_TPCModifyItem_RL_ReconfPrepTDD_sequence[] = {
  { &hf_rnsap_cCTrCH_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CCTrCH_ID },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_CCTrCH_TPCModifyItem_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_CCTrCH_TPCModifyItem_RL_ReconfPrepTDD, CCTrCH_TPCModifyItem_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_CCTrCH_TPCModifyList_RL_ReconfPrepTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CCTrCH_TPCModifyItem_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_rnsap_CCTrCH_TPCModifyList_RL_ReconfPrepTDD_item);
}


static const per_sequence_t CCTrCH_TPCModifyList_RL_ReconfPrepTDD_sequence_of[1] = {
  { &hf_rnsap_CCTrCH_TPCModifyList_RL_ReconfPrepTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_CCTrCH_TPCModifyItem_RL_ReconfPrepTDD },
};

static int
dissect_rnsap_CCTrCH_TPCModifyList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_CCTrCH_TPCModifyList_RL_ReconfPrepTDD, CCTrCH_TPCModifyList_RL_ReconfPrepTDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_cCTrCH_TPCList2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CCTrCH_TPCModifyList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_rnsap_cCTrCH_TPCList2);
}


static const per_sequence_t DL_CCTrCH_InformationModifyItem_RL_ReconfPrepTDD_sequence[] = {
  { &hf_rnsap_cCTrCH_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CCTrCH_ID },
  { &hf_rnsap_tFCS          , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TFCS },
  { &hf_rnsap_tFCI_Coding   , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TFCI_Coding },
  { &hf_rnsap_punctureLimit , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_PunctureLimit },
  { &hf_rnsap_cCTrCH_TPCList2, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_CCTrCH_TPCModifyList_RL_ReconfPrepTDD },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_CCTrCH_InformationModifyItem_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_CCTrCH_InformationModifyItem_RL_ReconfPrepTDD, DL_CCTrCH_InformationModifyItem_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_id_DL_CCTrCH_InformationModifyItem_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_CCTrCH_InformationModifyItem_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_rnsap_id_DL_CCTrCH_InformationModifyItem_RL_ReconfPrepTDD);
}


static const per_sequence_t DL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD_sequence[] = {
  { &hf_rnsap_cCTrCH_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CCTrCH_ID },
  { &hf_rnsap_tFCS          , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TFCS },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD, DL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD_sequence);

  return offset;
}
static int dissect_id_DL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD(tvb, offset, actx, tree, hf_rnsap_id_DL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD);
}


static const per_sequence_t DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD_sequence_of[1] = {
  { &hf_rnsap_DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD, DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD_sequence_of,
                                                  0, 16);

  return offset;
}
static int dissect_id_DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_rnsap_id_DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD);
}


static const per_sequence_t DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD_sequence_of[1] = {
  { &hf_rnsap_DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD, DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD_sequence_of,
                                                  0, 16);

  return offset;
}
static int dissect_id_DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_rnsap_id_DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD);
}


static const per_sequence_t DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD_sequence_of[1] = {
  { &hf_rnsap_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD, DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD_sequence_of,
                                                  0, 16);

  return offset;
}
static int dissect_id_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD(tvb, offset, actx, tree, hf_rnsap_id_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD);
}


static const per_sequence_t DL_DPCH_InformationAddListIE_RL_ReconfReadyTDD_sequence[] = {
  { &hf_rnsap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RepetitionPeriod },
  { &hf_rnsap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RepetitionLength },
  { &hf_rnsap_tDD_DPCHOffset, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_DPCHOffset },
  { &hf_rnsap_dL_Timeslot_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_Timeslot_Information },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_DPCH_InformationAddListIE_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_DPCH_InformationAddListIE_RL_ReconfReadyTDD, DL_DPCH_InformationAddListIE_RL_ReconfReadyTDD_sequence);

  return offset;
}
static int dissect_id_DL_DPCH_InformationAddListIE_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_DPCH_InformationAddListIE_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_id_DL_DPCH_InformationAddListIE_RL_ReconfReadyTDD);
}


static const per_sequence_t DL_DPCH_InformationDeleteItem_RL_ReconfReadyTDD_sequence[] = {
  { &hf_rnsap_dPCH_ID       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DPCH_ID },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_DPCH_InformationDeleteItem_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_DPCH_InformationDeleteItem_RL_ReconfReadyTDD, DL_DPCH_InformationDeleteItem_RL_ReconfReadyTDD_sequence);

  return offset;
}
static int dissect_DL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_DPCH_InformationDeleteItem_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_DL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD_item);
}


static const per_sequence_t DL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD_sequence_of[1] = {
  { &hf_rnsap_DL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_DPCH_InformationDeleteItem_RL_ReconfReadyTDD },
};

static int
dissect_rnsap_DL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD, DL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD_sequence_of,
                                                  0, 240);

  return offset;
}
static int dissect_id_DL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_id_DL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD);
}


static const per_sequence_t TDD_DL_Code_InformationModifyItem_RL_ReconfReadyTDD_sequence[] = {
  { &hf_rnsap_dPCH_ID       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DPCH_ID },
  { &hf_rnsap_tDD_ChannelisationCode, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TDD_ChannelisationCode },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_TDD_DL_Code_InformationModifyItem_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_TDD_DL_Code_InformationModifyItem_RL_ReconfReadyTDD, TDD_DL_Code_InformationModifyItem_RL_ReconfReadyTDD_sequence);

  return offset;
}
static int dissect_TDD_DL_Code_InformationModifyList_RL_ReconfReadyTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_DL_Code_InformationModifyItem_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_TDD_DL_Code_InformationModifyList_RL_ReconfReadyTDD_item);
}


static const per_sequence_t TDD_DL_Code_InformationModifyList_RL_ReconfReadyTDD_sequence_of[1] = {
  { &hf_rnsap_TDD_DL_Code_InformationModifyList_RL_ReconfReadyTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_DL_Code_InformationModifyItem_RL_ReconfReadyTDD },
};

static int
dissect_rnsap_TDD_DL_Code_InformationModifyList_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_TDD_DL_Code_InformationModifyList_RL_ReconfReadyTDD, TDD_DL_Code_InformationModifyList_RL_ReconfReadyTDD_sequence_of,
                                                  1, 240);

  return offset;
}
static int dissect_dL_Code_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_DL_Code_InformationModifyList_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_dL_Code_Information);
}


static const per_sequence_t DL_Timeslot_InformationModifyItem_RL_ReconfReadyTDD_sequence[] = {
  { &hf_rnsap_timeSlot      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TimeSlot },
  { &hf_rnsap_midambleShiftAndBurstType, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_MidambleShiftAndBurstType },
  { &hf_rnsap_tFCI_Presence , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TFCI_Presence },
  { &hf_rnsap_dL_Code_Information, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TDD_DL_Code_InformationModifyList_RL_ReconfReadyTDD },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_Timeslot_InformationModifyItem_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_Timeslot_InformationModifyItem_RL_ReconfReadyTDD, DL_Timeslot_InformationModifyItem_RL_ReconfReadyTDD_sequence);

  return offset;
}
static int dissect_DL_Timeslot_InformationModifyList_RL_ReconfReadyTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_Timeslot_InformationModifyItem_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_DL_Timeslot_InformationModifyList_RL_ReconfReadyTDD_item);
}


static const per_sequence_t DL_Timeslot_InformationModifyList_RL_ReconfReadyTDD_sequence_of[1] = {
  { &hf_rnsap_DL_Timeslot_InformationModifyList_RL_ReconfReadyTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_Timeslot_InformationModifyItem_RL_ReconfReadyTDD },
};

static int
dissect_rnsap_DL_Timeslot_InformationModifyList_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DL_Timeslot_InformationModifyList_RL_ReconfReadyTDD, DL_Timeslot_InformationModifyList_RL_ReconfReadyTDD_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_dL_Timeslot_InformationModifyList_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_Timeslot_InformationModifyList_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_dL_Timeslot_InformationModifyList_RL_ReconfReadyTDD);
}


static const per_sequence_t DL_DPCH_InformationModifyListIE_RL_ReconfReadyTDD_sequence[] = {
  { &hf_rnsap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_RepetitionPeriod },
  { &hf_rnsap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_RepetitionLength },
  { &hf_rnsap_tDD_DPCHOffset, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TDD_DPCHOffset },
  { &hf_rnsap_dL_Timeslot_InformationModifyList_RL_ReconfReadyTDD, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DL_Timeslot_InformationModifyList_RL_ReconfReadyTDD },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_DPCH_InformationModifyListIE_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_DPCH_InformationModifyListIE_RL_ReconfReadyTDD, DL_DPCH_InformationModifyListIE_RL_ReconfReadyTDD_sequence);

  return offset;
}
static int dissect_id_DL_DPCH_InformationModifyListIE_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_DPCH_InformationModifyListIE_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_id_DL_DPCH_InformationModifyListIE_RL_ReconfReadyTDD);
}



static int
dissect_rnsap_DSCH_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 255U, NULL, FALSE);

  return offset;
}
static int dissect_dsch_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DSCH_ID(tvb, offset, actx, tree, hf_rnsap_dsch_ID);
}
static int dissect_dSCH_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DSCH_ID(tvb, offset, actx, tree, hf_rnsap_dSCH_ID);
}


static const per_sequence_t DSCH_TDD_InformationItem_sequence[] = {
  { &hf_rnsap_dSCH_ID       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DSCH_ID },
  { &hf_rnsap_dl_ccTrCHID   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CCTrCH_ID },
  { &hf_rnsap_trChSourceStatisticsDescriptor, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TrCH_SrcStatisticsDescr },
  { &hf_rnsap_transportFormatSet, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TransportFormatSet },
  { &hf_rnsap_allocationRetentionPriority, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_AllocationRetentionPriority },
  { &hf_rnsap_schedulingPriorityIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SchedulingPriorityIndicator },
  { &hf_rnsap_bLER          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BLER },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DSCH_TDD_InformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DSCH_TDD_InformationItem, DSCH_TDD_InformationItem_sequence);

  return offset;
}
static int dissect_DSCH_TDD_Information_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DSCH_TDD_InformationItem(tvb, offset, actx, tree, hf_rnsap_DSCH_TDD_Information_item);
}


static const per_sequence_t DSCH_TDD_Information_sequence_of[1] = {
  { &hf_rnsap_DSCH_TDD_Information_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_DSCH_TDD_InformationItem },
};

static int
dissect_rnsap_DSCH_TDD_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DSCH_TDD_Information, DSCH_TDD_Information_sequence_of,
                                                  1, 10);

  return offset;
}
static int dissect_id_DSCHs_to_Add_TDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DSCH_TDD_Information(tvb, offset, actx, tree, hf_rnsap_id_DSCHs_to_Add_TDD);
}
static int dissect_id_DSCH_TDD_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DSCH_TDD_Information(tvb, offset, actx, tree, hf_rnsap_id_DSCH_TDD_Information);
}


static const per_sequence_t DSCH_DeleteItem_RL_ReconfPrepTDD_sequence[] = {
  { &hf_rnsap_dSCH_ID       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DSCH_ID },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DSCH_DeleteItem_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DSCH_DeleteItem_RL_ReconfPrepTDD, DSCH_DeleteItem_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_DSCH_DeleteList_RL_ReconfPrepTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DSCH_DeleteItem_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_rnsap_DSCH_DeleteList_RL_ReconfPrepTDD_item);
}


static const per_sequence_t DSCH_DeleteList_RL_ReconfPrepTDD_sequence_of[1] = {
  { &hf_rnsap_DSCH_DeleteList_RL_ReconfPrepTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_DSCH_DeleteItem_RL_ReconfPrepTDD },
};

static int
dissect_rnsap_DSCH_DeleteList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DSCH_DeleteList_RL_ReconfPrepTDD, DSCH_DeleteList_RL_ReconfPrepTDD_sequence_of,
                                                  0, 128);

  return offset;
}
static int dissect_id_DSCH_DeleteList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DSCH_DeleteList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_rnsap_id_DSCH_DeleteList_RL_ReconfPrepTDD);
}


static const value_string rnsap_TransportFormatManagement_vals[] = {
  {   0, "cell-based" },
  {   1, "ue-based" },
  { 0, NULL }
};


static int
dissect_rnsap_TransportFormatManagement(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_transportFormatManagement(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TransportFormatManagement(tvb, offset, actx, tree, hf_rnsap_transportFormatManagement);
}


static const per_sequence_t DSCH_FlowControlItem_sequence[] = {
  { &hf_rnsap_dSCH_SchedulingPriority, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SchedulingPriorityIndicator },
  { &hf_rnsap_mAC_c_sh_SDU_Lengths, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MAC_c_sh_SDU_LengthList },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DSCH_FlowControlItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DSCH_FlowControlItem, DSCH_FlowControlItem_sequence);

  return offset;
}
static int dissect_DSCH_FlowControlInformation_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DSCH_FlowControlItem(tvb, offset, actx, tree, hf_rnsap_DSCH_FlowControlInformation_item);
}


static const per_sequence_t DSCH_FlowControlInformation_sequence_of[1] = {
  { &hf_rnsap_DSCH_FlowControlInformation_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_DSCH_FlowControlItem },
};

static int
dissect_rnsap_DSCH_FlowControlInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DSCH_FlowControlInformation, DSCH_FlowControlInformation_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_dSCH_FlowControlInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DSCH_FlowControlInformation(tvb, offset, actx, tree, hf_rnsap_dSCH_FlowControlInformation);
}


static const per_sequence_t DiversityIndication_RL_AdditionRspTDD2_sequence[] = {
  { &hf_rnsap_bindingID     , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_BindingID },
  { &hf_rnsap_transportLayerAddress, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TransportLayerAddress },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DiversityIndication_RL_AdditionRspTDD2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DiversityIndication_RL_AdditionRspTDD2, DiversityIndication_RL_AdditionRspTDD2_sequence);

  return offset;
}
static int dissect_diversityIndication4(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DiversityIndication_RL_AdditionRspTDD2(tvb, offset, actx, tree, hf_rnsap_diversityIndication4);
}


static const per_sequence_t DSCHInformationItem_RL_AdditionRspTDD_sequence[] = {
  { &hf_rnsap_dsch_ID       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DSCH_ID },
  { &hf_rnsap_transportFormatManagement, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TransportFormatManagement },
  { &hf_rnsap_dSCH_FlowControlInformation, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DSCH_FlowControlInformation },
  { &hf_rnsap_diversityIndication4, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DiversityIndication_RL_AdditionRspTDD2 },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DSCHInformationItem_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DSCHInformationItem_RL_AdditionRspTDD, DSCHInformationItem_RL_AdditionRspTDD_sequence);

  return offset;
}
static int dissect_DSCH_InformationListIE_RL_AdditionRspTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DSCHInformationItem_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_rnsap_DSCH_InformationListIE_RL_AdditionRspTDD_item);
}


static const per_sequence_t DSCH_InformationListIE_RL_AdditionRspTDD_sequence_of[1] = {
  { &hf_rnsap_DSCH_InformationListIE_RL_AdditionRspTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_DSCHInformationItem_RL_AdditionRspTDD },
};

static int
dissect_rnsap_DSCH_InformationListIE_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DSCH_InformationListIE_RL_AdditionRspTDD, DSCH_InformationListIE_RL_AdditionRspTDD_sequence_of,
                                                  0, 10);

  return offset;
}
static int dissect_id_DSCH_InformationListIE_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DSCH_InformationListIE_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_rnsap_id_DSCH_InformationListIE_RL_AdditionRspTDD);
}


static const per_sequence_t DSCHInformationItem_RL_SetupRspTDD_sequence[] = {
  { &hf_rnsap_dsch_ID       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DSCH_ID },
  { &hf_rnsap_dSCH_FlowControlInformation, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DSCH_FlowControlInformation },
  { &hf_rnsap_bindingID     , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_BindingID },
  { &hf_rnsap_transportLayerAddress, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TransportLayerAddress },
  { &hf_rnsap_transportFormatManagement, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TransportFormatManagement },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DSCHInformationItem_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DSCHInformationItem_RL_SetupRspTDD, DSCHInformationItem_RL_SetupRspTDD_sequence);

  return offset;
}
static int dissect_DSCH_InformationListIEs_RL_SetupRspTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DSCHInformationItem_RL_SetupRspTDD(tvb, offset, actx, tree, hf_rnsap_DSCH_InformationListIEs_RL_SetupRspTDD_item);
}


static const per_sequence_t DSCH_InformationListIEs_RL_SetupRspTDD_sequence_of[1] = {
  { &hf_rnsap_DSCH_InformationListIEs_RL_SetupRspTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_DSCHInformationItem_RL_SetupRspTDD },
};

static int
dissect_rnsap_DSCH_InformationListIEs_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DSCH_InformationListIEs_RL_SetupRspTDD, DSCH_InformationListIEs_RL_SetupRspTDD_sequence_of,
                                                  0, 10);

  return offset;
}
static int dissect_id_DSCH_InformationListIEs_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DSCH_InformationListIEs_RL_SetupRspTDD(tvb, offset, actx, tree, hf_rnsap_id_DSCH_InformationListIEs_RL_SetupRspTDD);
}


static const per_sequence_t DSCH_ModifyItem_RL_ReconfPrepTDD_sequence[] = {
  { &hf_rnsap_dSCH_ID       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DSCH_ID },
  { &hf_rnsap_dl_ccTrCHID   , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_CCTrCH_ID },
  { &hf_rnsap_trChSourceStatisticsDescriptor, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TrCH_SrcStatisticsDescr },
  { &hf_rnsap_transportFormatSet, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TransportFormatSet },
  { &hf_rnsap_allocationRetentionPriority, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_AllocationRetentionPriority },
  { &hf_rnsap_schedulingPriorityIndicator, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_SchedulingPriorityIndicator },
  { &hf_rnsap_bLER          , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_BLER },
  { &hf_rnsap_transportBearerRequestIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TransportBearerRequestIndicator },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DSCH_ModifyItem_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DSCH_ModifyItem_RL_ReconfPrepTDD, DSCH_ModifyItem_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_DSCH_ModifyList_RL_ReconfPrepTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DSCH_ModifyItem_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_rnsap_DSCH_ModifyList_RL_ReconfPrepTDD_item);
}


static const per_sequence_t DSCH_ModifyList_RL_ReconfPrepTDD_sequence_of[1] = {
  { &hf_rnsap_DSCH_ModifyList_RL_ReconfPrepTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_DSCH_ModifyItem_RL_ReconfPrepTDD },
};

static int
dissect_rnsap_DSCH_ModifyList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DSCH_ModifyList_RL_ReconfPrepTDD, DSCH_ModifyList_RL_ReconfPrepTDD_sequence_of,
                                                  0, 128);

  return offset;
}
static int dissect_id_DSCH_ModifyList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DSCH_ModifyList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_rnsap_id_DSCH_ModifyList_RL_ReconfPrepTDD);
}


static const per_sequence_t DSCHToBeAddedOrModifiedItem_RL_ReconfReadyTDD_sequence[] = {
  { &hf_rnsap_dsch_ID       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DSCH_ID },
  { &hf_rnsap_transportFormatManagement, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TransportFormatManagement },
  { &hf_rnsap_dSCH_FlowControlInformation, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DSCH_FlowControlInformation },
  { &hf_rnsap_bindingID     , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_BindingID },
  { &hf_rnsap_transportLayerAddress, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TransportLayerAddress },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DSCHToBeAddedOrModifiedItem_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DSCHToBeAddedOrModifiedItem_RL_ReconfReadyTDD, DSCHToBeAddedOrModifiedItem_RL_ReconfReadyTDD_sequence);

  return offset;
}
static int dissect_DSCHToBeAddedOrModifiedList_RL_ReconfReadyTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DSCHToBeAddedOrModifiedItem_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_DSCHToBeAddedOrModifiedList_RL_ReconfReadyTDD_item);
}


static const per_sequence_t DSCHToBeAddedOrModifiedList_RL_ReconfReadyTDD_sequence_of[1] = {
  { &hf_rnsap_DSCHToBeAddedOrModifiedList_RL_ReconfReadyTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_DSCHToBeAddedOrModifiedItem_RL_ReconfReadyTDD },
};

static int
dissect_rnsap_DSCHToBeAddedOrModifiedList_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DSCHToBeAddedOrModifiedList_RL_ReconfReadyTDD, DSCHToBeAddedOrModifiedList_RL_ReconfReadyTDD_sequence_of,
                                                  0, 10);

  return offset;
}
static int dissect_id_DSCHToBeAddedOrModifiedList_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DSCHToBeAddedOrModifiedList_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_id_DSCHToBeAddedOrModifiedList_RL_ReconfReadyTDD);
}


static const per_sequence_t GA_PointWithUnCertainty_sequence[] = {
  { &hf_rnsap_geographicalCoordinates, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_GeographicalCoordinate },
  { &hf_rnsap_uncertaintyCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_INTEGER_0_127 },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_GA_PointWithUnCertainty(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_GA_PointWithUnCertainty, GA_PointWithUnCertainty_sequence);

  return offset;
}
static int dissect_pointWithUncertainty(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GA_PointWithUnCertainty(tvb, offset, actx, tree, hf_rnsap_pointWithUncertainty);
}



static int
dissect_rnsap_INTEGER_0_179(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 179U, NULL, FALSE);

  return offset;
}
static int dissect_offsetAngle(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_0_179(tvb, offset, actx, tree, hf_rnsap_offsetAngle);
}
static int dissect_includedAngle(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_0_179(tvb, offset, actx, tree, hf_rnsap_includedAngle);
}
static int dissect_orientationOfMajorAxis(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_0_179(tvb, offset, actx, tree, hf_rnsap_orientationOfMajorAxis);
}


static const per_sequence_t GA_UncertaintyEllipse_sequence[] = {
  { &hf_rnsap_uncertaintySemi_major, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_INTEGER_0_127 },
  { &hf_rnsap_uncertaintySemi_minor, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_INTEGER_0_127 },
  { &hf_rnsap_orientationOfMajorAxis, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_INTEGER_0_179 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_GA_UncertaintyEllipse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_GA_UncertaintyEllipse, GA_UncertaintyEllipse_sequence);

  return offset;
}
static int dissect_uncertaintyEllipse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GA_UncertaintyEllipse(tvb, offset, actx, tree, hf_rnsap_uncertaintyEllipse);
}


static const per_sequence_t GA_PointWithUnCertaintyEllipse_sequence[] = {
  { &hf_rnsap_geographicalCoordinates, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_GeographicalCoordinate },
  { &hf_rnsap_uncertaintyEllipse, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_GA_UncertaintyEllipse },
  { &hf_rnsap_confidence    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_INTEGER_0_127 },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_GA_PointWithUnCertaintyEllipse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_GA_PointWithUnCertaintyEllipse, GA_PointWithUnCertaintyEllipse_sequence);

  return offset;
}
static int dissect_pointWithUncertaintyEllipse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GA_PointWithUnCertaintyEllipse(tvb, offset, actx, tree, hf_rnsap_pointWithUncertaintyEllipse);
}


static const value_string rnsap_T_directionOfAltitude_vals[] = {
  {   0, "height" },
  {   1, "depth" },
  { 0, NULL }
};


static int
dissect_rnsap_T_directionOfAltitude(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_directionOfAltitude(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_T_directionOfAltitude(tvb, offset, actx, tree, hf_rnsap_directionOfAltitude);
}


static const per_sequence_t GA_AltitudeAndDirection_sequence[] = {
  { &hf_rnsap_directionOfAltitude, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_T_directionOfAltitude },
  { &hf_rnsap_altitude      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_INTEGER_0_32767 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_GA_AltitudeAndDirection(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_GA_AltitudeAndDirection, GA_AltitudeAndDirection_sequence);

  return offset;
}
static int dissect_altitudeAndDirection(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GA_AltitudeAndDirection(tvb, offset, actx, tree, hf_rnsap_altitudeAndDirection);
}


static const per_sequence_t GA_PointWithAltitude_sequence[] = {
  { &hf_rnsap_geographicalCoordinates, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_GeographicalCoordinate },
  { &hf_rnsap_altitudeAndDirection, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_GA_AltitudeAndDirection },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_GA_PointWithAltitude(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_GA_PointWithAltitude, GA_PointWithAltitude_sequence);

  return offset;
}
static int dissect_pointWithAltitude(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GA_PointWithAltitude(tvb, offset, actx, tree, hf_rnsap_pointWithAltitude);
}


static const per_sequence_t GA_PointWithAltitudeAndUncertaintyEllipsoid_sequence[] = {
  { &hf_rnsap_geographicalCoordinates, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_GeographicalCoordinate },
  { &hf_rnsap_altitudeAndDirection, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_GA_AltitudeAndDirection },
  { &hf_rnsap_uncertaintyEllipse, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_GA_UncertaintyEllipse },
  { &hf_rnsap_uncertaintyAltitude, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_INTEGER_0_127 },
  { &hf_rnsap_confidence    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_INTEGER_0_127 },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_GA_PointWithAltitudeAndUncertaintyEllipsoid(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_GA_PointWithAltitudeAndUncertaintyEllipsoid, GA_PointWithAltitudeAndUncertaintyEllipsoid_sequence);

  return offset;
}
static int dissect_pointWithAltitudeAndUncertaintyEllipsoid(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GA_PointWithAltitudeAndUncertaintyEllipsoid(tvb, offset, actx, tree, hf_rnsap_pointWithAltitudeAndUncertaintyEllipsoid);
}


static const per_sequence_t GA_EllipsoidArc_sequence[] = {
  { &hf_rnsap_geographicalCoordinates, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_GeographicalCoordinate },
  { &hf_rnsap_innerRadius   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_INTEGER_0_65535 },
  { &hf_rnsap_uncertaintyRadius, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_INTEGER_0_127 },
  { &hf_rnsap_offsetAngle   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_INTEGER_0_179 },
  { &hf_rnsap_includedAngle , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_INTEGER_0_179 },
  { &hf_rnsap_confidence    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_INTEGER_0_127 },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_GA_EllipsoidArc(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_GA_EllipsoidArc, GA_EllipsoidArc_sequence);

  return offset;
}
static int dissect_ellipsoidArc(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GA_EllipsoidArc(tvb, offset, actx, tree, hf_rnsap_ellipsoidArc);
}


static const value_string rnsap_GA_CellAdditionalShapes_vals[] = {
  {   0, "pointWithUncertainty" },
  {   1, "pointWithUncertaintyEllipse" },
  {   2, "pointWithAltitude" },
  {   3, "pointWithAltitudeAndUncertaintyEllipsoid" },
  {   4, "ellipsoidArc" },
  { 0, NULL }
};

static const per_choice_t GA_CellAdditionalShapes_choice[] = {
  {   0, &hf_rnsap_pointWithUncertainty, ASN1_EXTENSION_ROOT    , dissect_rnsap_GA_PointWithUnCertainty },
  {   1, &hf_rnsap_pointWithUncertaintyEllipse, ASN1_EXTENSION_ROOT    , dissect_rnsap_GA_PointWithUnCertaintyEllipse },
  {   2, &hf_rnsap_pointWithAltitude, ASN1_EXTENSION_ROOT    , dissect_rnsap_GA_PointWithAltitude },
  {   3, &hf_rnsap_pointWithAltitudeAndUncertaintyEllipsoid, ASN1_EXTENSION_ROOT    , dissect_rnsap_GA_PointWithAltitudeAndUncertaintyEllipsoid },
  {   4, &hf_rnsap_ellipsoidArc  , ASN1_EXTENSION_ROOT    , dissect_rnsap_GA_EllipsoidArc },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_GA_CellAdditionalShapes(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_GA_CellAdditionalShapes, GA_CellAdditionalShapes_choice,
                                 NULL);

  return offset;
}
static int dissect_id_GA_CellAdditionalShapes(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GA_CellAdditionalShapes(tvb, offset, actx, tree, hf_rnsap_id_GA_CellAdditionalShapes);
}



static int
dissect_rnsap_TGSN(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 14U, NULL, FALSE);

  return offset;
}
static int dissect_tGSN(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TGSN(tvb, offset, actx, tree, hf_rnsap_tGSN);
}



static int
dissect_rnsap_GapLength(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 14U, NULL, FALSE);

  return offset;
}
static int dissect_tGL1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GapLength(tvb, offset, actx, tree, hf_rnsap_tGL1);
}
static int dissect_tGL2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GapLength(tvb, offset, actx, tree, hf_rnsap_tGL2);
}



static int
dissect_rnsap_TGD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 269U, NULL, FALSE);

  return offset;
}
static int dissect_tGD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TGD(tvb, offset, actx, tree, hf_rnsap_tGD);
}



static int
dissect_rnsap_GapDuration(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 144U, NULL, TRUE);

  return offset;
}
static int dissect_tGPL1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GapDuration(tvb, offset, actx, tree, hf_rnsap_tGPL1);
}
static int dissect_not_to_be_used_1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GapDuration(tvb, offset, actx, tree, hf_rnsap_not_to_be_used_1);
}


static const value_string rnsap_UL_DL_mode_vals[] = {
  {   0, "ul-only" },
  {   1, "dl-only" },
  {   2, "both-ul-and-dl" },
  { 0, NULL }
};


static int
dissect_rnsap_UL_DL_mode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_uL_DL_mode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_DL_mode(tvb, offset, actx, tree, hf_rnsap_uL_DL_mode);
}


static const value_string rnsap_Downlink_Compressed_Mode_Method_vals[] = {
  {   0, "not-Used-puncturing" },
  {   1, "sFdiv2" },
  {   2, "higher-layer-scheduling" },
  { 0, NULL }
};


static int
dissect_rnsap_Downlink_Compressed_Mode_Method(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_downlink_Compressed_Mode_Method(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Downlink_Compressed_Mode_Method(tvb, offset, actx, tree, hf_rnsap_downlink_Compressed_Mode_Method);
}


static const value_string rnsap_Uplink_Compressed_Mode_Method_vals[] = {
  {   0, "sFdiv2" },
  {   1, "higher-layer-scheduling" },
  { 0, NULL }
};


static int
dissect_rnsap_Uplink_Compressed_Mode_Method(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_uplink_Compressed_Mode_Method(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Uplink_Compressed_Mode_Method(tvb, offset, actx, tree, hf_rnsap_uplink_Compressed_Mode_Method);
}


static const value_string rnsap_DL_FrameType_vals[] = {
  {   0, "typeA" },
  {   1, "typeB" },
  { 0, NULL }
};


static int
dissect_rnsap_DL_FrameType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_dL_FrameType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_FrameType(tvb, offset, actx, tree, hf_rnsap_dL_FrameType);
}



static int
dissect_rnsap_DeltaSIR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 30U, NULL, FALSE);

  return offset;
}
static int dissect_delta_SIR1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DeltaSIR(tvb, offset, actx, tree, hf_rnsap_delta_SIR1);
}
static int dissect_delta_SIR_after1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DeltaSIR(tvb, offset, actx, tree, hf_rnsap_delta_SIR_after1);
}
static int dissect_delta_SIR2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DeltaSIR(tvb, offset, actx, tree, hf_rnsap_delta_SIR2);
}
static int dissect_delta_SIR_after2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DeltaSIR(tvb, offset, actx, tree, hf_rnsap_delta_SIR_after2);
}


static const per_sequence_t Transmission_Gap_Pattern_Sequence_Information_item_sequence[] = {
  { &hf_rnsap_tGPSID        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TGPSID },
  { &hf_rnsap_tGSN          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TGSN },
  { &hf_rnsap_tGL1          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_GapLength },
  { &hf_rnsap_tGL2          , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_GapLength },
  { &hf_rnsap_tGD           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TGD },
  { &hf_rnsap_tGPL1         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_GapDuration },
  { &hf_rnsap_not_to_be_used_1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_GapDuration },
  { &hf_rnsap_uL_DL_mode    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_DL_mode },
  { &hf_rnsap_downlink_Compressed_Mode_Method, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Downlink_Compressed_Mode_Method },
  { &hf_rnsap_uplink_Compressed_Mode_Method, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Uplink_Compressed_Mode_Method },
  { &hf_rnsap_dL_FrameType  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_FrameType },
  { &hf_rnsap_delta_SIR1    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DeltaSIR },
  { &hf_rnsap_delta_SIR_after1, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DeltaSIR },
  { &hf_rnsap_delta_SIR2    , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DeltaSIR },
  { &hf_rnsap_delta_SIR_after2, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DeltaSIR },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_Transmission_Gap_Pattern_Sequence_Information_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_Transmission_Gap_Pattern_Sequence_Information_item, Transmission_Gap_Pattern_Sequence_Information_item_sequence);

  return offset;
}
static int dissect_Transmission_Gap_Pattern_Sequence_Information_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Transmission_Gap_Pattern_Sequence_Information_item(tvb, offset, actx, tree, hf_rnsap_Transmission_Gap_Pattern_Sequence_Information_item);
}


static const per_sequence_t Transmission_Gap_Pattern_Sequence_Information_sequence_of[1] = {
  { &hf_rnsap_Transmission_Gap_Pattern_Sequence_Information_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_Transmission_Gap_Pattern_Sequence_Information_item },
};

static int
dissect_rnsap_Transmission_Gap_Pattern_Sequence_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_Transmission_Gap_Pattern_Sequence_Information, Transmission_Gap_Pattern_Sequence_Information_sequence_of,
                                                  1, 6);

  return offset;
}
static int dissect_id_Transmission_Gap_Pattern_Sequence_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Transmission_Gap_Pattern_Sequence_Information(tvb, offset, actx, tree, hf_rnsap_id_Transmission_Gap_Pattern_Sequence_Information);
}


static const per_sequence_t UL_CCTrCH_DeleteInformation_RL_ReconfPrepTDD_sequence[] = {
  { &hf_rnsap_cCTrCH_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CCTrCH_ID },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UL_CCTrCH_DeleteInformation_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UL_CCTrCH_DeleteInformation_RL_ReconfPrepTDD, UL_CCTrCH_DeleteInformation_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_id_UL_CCTrCH_DeleteInformation_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_CCTrCH_DeleteInformation_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_rnsap_id_UL_CCTrCH_DeleteInformation_RL_ReconfPrepTDD);
}


static const per_sequence_t UL_CCTrCH_ModifyInformation_RL_ReconfPrepTDD_sequence[] = {
  { &hf_rnsap_cCTrCH_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CCTrCH_ID },
  { &hf_rnsap_tFCS          , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TFCS },
  { &hf_rnsap_tFCI_Coding   , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TFCI_Coding },
  { &hf_rnsap_punctureLimit , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_PunctureLimit },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UL_CCTrCH_ModifyInformation_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UL_CCTrCH_ModifyInformation_RL_ReconfPrepTDD, UL_CCTrCH_ModifyInformation_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_id_UL_CCTrCH_ModifyInformation_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_CCTrCH_ModifyInformation_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_rnsap_id_UL_CCTrCH_ModifyInformation_RL_ReconfPrepTDD);
}


static const per_sequence_t UL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD_sequence[] = {
  { &hf_rnsap_cCTrCH_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CCTrCH_ID },
  { &hf_rnsap_tFCS          , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TFCS },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD, UL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD_sequence);

  return offset;
}
static int dissect_id_UL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD(tvb, offset, actx, tree, hf_rnsap_id_UL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD);
}


static const per_sequence_t UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD_sequence_of[1] = {
  { &hf_rnsap_UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD, UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD_sequence_of,
                                                  0, 16);

  return offset;
}
static int dissect_id_UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_rnsap_id_UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD);
}


static const per_sequence_t UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD_sequence_of[1] = {
  { &hf_rnsap_UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD, UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD_sequence_of,
                                                  0, 16);

  return offset;
}
static int dissect_id_UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_rnsap_id_UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD);
}


static const per_sequence_t UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD_sequence_of[1] = {
  { &hf_rnsap_UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD, UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD_sequence_of,
                                                  0, 16);

  return offset;
}
static int dissect_id_UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD(tvb, offset, actx, tree, hf_rnsap_id_UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD);
}


static const per_sequence_t UL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD_sequence[] = {
  { &hf_rnsap_cCTrCH_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CCTrCH_ID },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD, UL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD_sequence);

  return offset;
}
static int dissect_id_UL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD(tvb, offset, actx, tree, hf_rnsap_id_UL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD);
}


static const per_sequence_t UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD_sequence_of[1] = {
  { &hf_rnsap_UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD, UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD_sequence_of,
                                                  0, 16);

  return offset;
}
static int dissect_id_UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD(tvb, offset, actx, tree, hf_rnsap_id_UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD);
}


static const per_sequence_t UL_DPCH_InformationDeleteItem_RL_ReconfReadyTDD_sequence[] = {
  { &hf_rnsap_dPCH_ID       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DPCH_ID },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UL_DPCH_InformationDeleteItem_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UL_DPCH_InformationDeleteItem_RL_ReconfReadyTDD, UL_DPCH_InformationDeleteItem_RL_ReconfReadyTDD_sequence);

  return offset;
}
static int dissect_UL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_DPCH_InformationDeleteItem_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_UL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD_item);
}


static const per_sequence_t UL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD_sequence_of[1] = {
  { &hf_rnsap_UL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_DPCH_InformationDeleteItem_RL_ReconfReadyTDD },
};

static int
dissect_rnsap_UL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_UL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD, UL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD_sequence_of,
                                                  0, 240);

  return offset;
}
static int dissect_id_UL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_id_UL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD);
}


static const per_sequence_t TDD_UL_Code_InformationModifyItem_RL_ReconfReadyTDD_sequence[] = {
  { &hf_rnsap_dPCH_ID       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DPCH_ID },
  { &hf_rnsap_tDD_ChannelisationCode, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TDD_ChannelisationCode },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_TDD_UL_Code_InformationModifyItem_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_TDD_UL_Code_InformationModifyItem_RL_ReconfReadyTDD, TDD_UL_Code_InformationModifyItem_RL_ReconfReadyTDD_sequence);

  return offset;
}
static int dissect_TDD_UL_Code_InformationModifyList_RL_ReconfReadyTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_UL_Code_InformationModifyItem_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_TDD_UL_Code_InformationModifyList_RL_ReconfReadyTDD_item);
}


static const per_sequence_t TDD_UL_Code_InformationModifyList_RL_ReconfReadyTDD_sequence_of[1] = {
  { &hf_rnsap_TDD_UL_Code_InformationModifyList_RL_ReconfReadyTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_UL_Code_InformationModifyItem_RL_ReconfReadyTDD },
};

static int
dissect_rnsap_TDD_UL_Code_InformationModifyList_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_TDD_UL_Code_InformationModifyList_RL_ReconfReadyTDD, TDD_UL_Code_InformationModifyList_RL_ReconfReadyTDD_sequence_of,
                                                  1, 240);

  return offset;
}
static int dissect_uL_Code_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_UL_Code_InformationModifyList_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_uL_Code_Information);
}


static const per_sequence_t UL_Timeslot_InformationModifyItem_RL_ReconfReadyTDD_sequence[] = {
  { &hf_rnsap_timeSlot      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TimeSlot },
  { &hf_rnsap_midambleShiftAndBurstType, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_MidambleShiftAndBurstType },
  { &hf_rnsap_tFCI_Presence , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TFCI_Presence },
  { &hf_rnsap_uL_Code_Information, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TDD_UL_Code_InformationModifyList_RL_ReconfReadyTDD },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UL_Timeslot_InformationModifyItem_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UL_Timeslot_InformationModifyItem_RL_ReconfReadyTDD, UL_Timeslot_InformationModifyItem_RL_ReconfReadyTDD_sequence);

  return offset;
}
static int dissect_UL_Timeslot_InformationModifyList_RL_ReconfReadyTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_Timeslot_InformationModifyItem_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_UL_Timeslot_InformationModifyList_RL_ReconfReadyTDD_item);
}


static const per_sequence_t UL_Timeslot_InformationModifyList_RL_ReconfReadyTDD_sequence_of[1] = {
  { &hf_rnsap_UL_Timeslot_InformationModifyList_RL_ReconfReadyTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_Timeslot_InformationModifyItem_RL_ReconfReadyTDD },
};

static int
dissect_rnsap_UL_Timeslot_InformationModifyList_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_UL_Timeslot_InformationModifyList_RL_ReconfReadyTDD, UL_Timeslot_InformationModifyList_RL_ReconfReadyTDD_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_uL_Timeslot_InformationModifyList_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_Timeslot_InformationModifyList_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_uL_Timeslot_InformationModifyList_RL_ReconfReadyTDD);
}


static const per_sequence_t UL_DPCH_InformationModifyListIE_RL_ReconfReadyTDD_sequence[] = {
  { &hf_rnsap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_RepetitionPeriod },
  { &hf_rnsap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_RepetitionLength },
  { &hf_rnsap_tDD_DPCHOffset, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TDD_DPCHOffset },
  { &hf_rnsap_uL_Timeslot_InformationModifyList_RL_ReconfReadyTDD, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_UL_Timeslot_InformationModifyList_RL_ReconfReadyTDD },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UL_DPCH_InformationModifyListIE_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UL_DPCH_InformationModifyListIE_RL_ReconfReadyTDD, UL_DPCH_InformationModifyListIE_RL_ReconfReadyTDD_sequence);

  return offset;
}
static int dissect_id_UL_DPCH_InformationModifyListIE_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_DPCH_InformationModifyListIE_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_id_UL_DPCH_InformationModifyListIE_RL_ReconfReadyTDD);
}


static const per_sequence_t UnsuccessfulRL_InformationResponse_RL_AdditionFailureTDD_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_cause         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_Cause },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UnsuccessfulRL_InformationResponse_RL_AdditionFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UnsuccessfulRL_InformationResponse_RL_AdditionFailureTDD, UnsuccessfulRL_InformationResponse_RL_AdditionFailureTDD_sequence);

  return offset;
}
static int dissect_id_UnsuccessfulRL_InformationResponse_RL_AdditionFailureTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UnsuccessfulRL_InformationResponse_RL_AdditionFailureTDD(tvb, offset, actx, tree, hf_rnsap_id_UnsuccessfulRL_InformationResponse_RL_AdditionFailureTDD);
}



static int
dissect_rnsap_USCH_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 255U, NULL, FALSE);

  return offset;
}
static int dissect_usch_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_USCH_ID(tvb, offset, actx, tree, hf_rnsap_usch_ID);
}
static int dissect_uSCH_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_USCH_ID(tvb, offset, actx, tree, hf_rnsap_uSCH_ID);
}



static int
dissect_rnsap_RB_Identity(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 31U, NULL, FALSE);

  return offset;
}
static int dissect_RB_Info_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RB_Identity(tvb, offset, actx, tree, hf_rnsap_RB_Info_item);
}


static const per_sequence_t RB_Info_sequence_of[1] = {
  { &hf_rnsap_RB_Info_item  , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_RB_Identity },
};

static int
dissect_rnsap_RB_Info(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_RB_Info, RB_Info_sequence_of,
                                                  1, 32);

  return offset;
}
static int dissect_rb_Info(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RB_Info(tvb, offset, actx, tree, hf_rnsap_rb_Info);
}


static const per_sequence_t USCH_InformationItem_sequence[] = {
  { &hf_rnsap_uSCH_ID       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_USCH_ID },
  { &hf_rnsap_ul_CCTrCH_ID  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CCTrCH_ID },
  { &hf_rnsap_trChSourceStatisticsDescriptor, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TrCH_SrcStatisticsDescr },
  { &hf_rnsap_transportFormatSet, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TransportFormatSet },
  { &hf_rnsap_allocationRetentionPriority, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_AllocationRetentionPriority },
  { &hf_rnsap_schedulingPriorityIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SchedulingPriorityIndicator },
  { &hf_rnsap_rb_Info       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RB_Info },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_USCH_InformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_USCH_InformationItem, USCH_InformationItem_sequence);

  return offset;
}
static int dissect_USCH_Information_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_USCH_InformationItem(tvb, offset, actx, tree, hf_rnsap_USCH_Information_item);
}


static const per_sequence_t USCH_Information_sequence_of[1] = {
  { &hf_rnsap_USCH_Information_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_USCH_InformationItem },
};

static int
dissect_rnsap_USCH_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_USCH_Information, USCH_Information_sequence_of,
                                                  1, 10);

  return offset;
}
static int dissect_id_USCHs_to_Add(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_USCH_Information(tvb, offset, actx, tree, hf_rnsap_id_USCHs_to_Add);
}
static int dissect_id_USCH_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_USCH_Information(tvb, offset, actx, tree, hf_rnsap_id_USCH_Information);
}


static const per_sequence_t USCH_DeleteItem_RL_ReconfPrepTDD_sequence[] = {
  { &hf_rnsap_uSCH_ID       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_USCH_ID },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_USCH_DeleteItem_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_USCH_DeleteItem_RL_ReconfPrepTDD, USCH_DeleteItem_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_USCH_DeleteList_RL_ReconfPrepTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_USCH_DeleteItem_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_rnsap_USCH_DeleteList_RL_ReconfPrepTDD_item);
}


static const per_sequence_t USCH_DeleteList_RL_ReconfPrepTDD_sequence_of[1] = {
  { &hf_rnsap_USCH_DeleteList_RL_ReconfPrepTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_USCH_DeleteItem_RL_ReconfPrepTDD },
};

static int
dissect_rnsap_USCH_DeleteList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_USCH_DeleteList_RL_ReconfPrepTDD, USCH_DeleteList_RL_ReconfPrepTDD_sequence_of,
                                                  0, 10);

  return offset;
}
static int dissect_id_USCH_DeleteList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_USCH_DeleteList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_rnsap_id_USCH_DeleteList_RL_ReconfPrepTDD);
}


static const per_sequence_t USCHInformationItem_RL_AdditionRspTDD_sequence[] = {
  { &hf_rnsap_uSCH_ID       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_USCH_ID },
  { &hf_rnsap_transportFormatManagement, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TransportFormatManagement },
  { &hf_rnsap_diversityIndication4, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DiversityIndication_RL_AdditionRspTDD2 },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_USCHInformationItem_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_USCHInformationItem_RL_AdditionRspTDD, USCHInformationItem_RL_AdditionRspTDD_sequence);

  return offset;
}
static int dissect_USCH_InformationListIE_RL_AdditionRspTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_USCHInformationItem_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_rnsap_USCH_InformationListIE_RL_AdditionRspTDD_item);
}


static const per_sequence_t USCH_InformationListIE_RL_AdditionRspTDD_sequence_of[1] = {
  { &hf_rnsap_USCH_InformationListIE_RL_AdditionRspTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_USCHInformationItem_RL_AdditionRspTDD },
};

static int
dissect_rnsap_USCH_InformationListIE_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_USCH_InformationListIE_RL_AdditionRspTDD, USCH_InformationListIE_RL_AdditionRspTDD_sequence_of,
                                                  0, 10);

  return offset;
}
static int dissect_id_USCH_InformationListIE_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_USCH_InformationListIE_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_rnsap_id_USCH_InformationListIE_RL_AdditionRspTDD);
}


static const per_sequence_t USCHInformationItem_RL_SetupRspTDD_sequence[] = {
  { &hf_rnsap_usch_ID       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_USCH_ID },
  { &hf_rnsap_bindingID     , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_BindingID },
  { &hf_rnsap_transportLayerAddress, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TransportLayerAddress },
  { &hf_rnsap_transportFormatManagement, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TransportFormatManagement },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_USCHInformationItem_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_USCHInformationItem_RL_SetupRspTDD, USCHInformationItem_RL_SetupRspTDD_sequence);

  return offset;
}
static int dissect_USCH_InformationListIEs_RL_SetupRspTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_USCHInformationItem_RL_SetupRspTDD(tvb, offset, actx, tree, hf_rnsap_USCH_InformationListIEs_RL_SetupRspTDD_item);
}


static const per_sequence_t USCH_InformationListIEs_RL_SetupRspTDD_sequence_of[1] = {
  { &hf_rnsap_USCH_InformationListIEs_RL_SetupRspTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_USCHInformationItem_RL_SetupRspTDD },
};

static int
dissect_rnsap_USCH_InformationListIEs_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_USCH_InformationListIEs_RL_SetupRspTDD, USCH_InformationListIEs_RL_SetupRspTDD_sequence_of,
                                                  0, 10);

  return offset;
}
static int dissect_id_USCH_InformationListIEs_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_USCH_InformationListIEs_RL_SetupRspTDD(tvb, offset, actx, tree, hf_rnsap_id_USCH_InformationListIEs_RL_SetupRspTDD);
}


static const per_sequence_t USCH_ModifyItem_RL_ReconfPrepTDD_sequence[] = {
  { &hf_rnsap_uSCH_ID       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_USCH_ID },
  { &hf_rnsap_ul_ccTrCHID   , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_CCTrCH_ID },
  { &hf_rnsap_trChSourceStatisticsDescriptor, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TrCH_SrcStatisticsDescr },
  { &hf_rnsap_transportFormatSet, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TransportFormatSet },
  { &hf_rnsap_allocationRetentionPriority, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_AllocationRetentionPriority },
  { &hf_rnsap_schedulingPriorityIndicator, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_SchedulingPriorityIndicator },
  { &hf_rnsap_bLER          , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_BLER },
  { &hf_rnsap_transportBearerRequestIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TransportBearerRequestIndicator },
  { &hf_rnsap_rb_Info       , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_RB_Info },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_USCH_ModifyItem_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_USCH_ModifyItem_RL_ReconfPrepTDD, USCH_ModifyItem_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_USCH_ModifyList_RL_ReconfPrepTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_USCH_ModifyItem_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_rnsap_USCH_ModifyList_RL_ReconfPrepTDD_item);
}


static const per_sequence_t USCH_ModifyList_RL_ReconfPrepTDD_sequence_of[1] = {
  { &hf_rnsap_USCH_ModifyList_RL_ReconfPrepTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_USCH_ModifyItem_RL_ReconfPrepTDD },
};

static int
dissect_rnsap_USCH_ModifyList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_USCH_ModifyList_RL_ReconfPrepTDD, USCH_ModifyList_RL_ReconfPrepTDD_sequence_of,
                                                  0, 10);

  return offset;
}
static int dissect_id_USCH_ModifyList_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_USCH_ModifyList_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_rnsap_id_USCH_ModifyList_RL_ReconfPrepTDD);
}


static const per_sequence_t USCHToBeAddedOrModifiedItem_RL_ReconfReadyTDD_sequence[] = {
  { &hf_rnsap_uSCH_ID       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_USCH_ID },
  { &hf_rnsap_transportFormatManagement, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TransportFormatManagement },
  { &hf_rnsap_bindingID     , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_BindingID },
  { &hf_rnsap_transportLayerAddress, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TransportLayerAddress },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_USCHToBeAddedOrModifiedItem_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_USCHToBeAddedOrModifiedItem_RL_ReconfReadyTDD, USCHToBeAddedOrModifiedItem_RL_ReconfReadyTDD_sequence);

  return offset;
}
static int dissect_USCHToBeAddedOrModifiedList_RL_ReconfReadyTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_USCHToBeAddedOrModifiedItem_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_USCHToBeAddedOrModifiedList_RL_ReconfReadyTDD_item);
}


static const per_sequence_t USCHToBeAddedOrModifiedList_RL_ReconfReadyTDD_sequence_of[1] = {
  { &hf_rnsap_USCHToBeAddedOrModifiedList_RL_ReconfReadyTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_USCHToBeAddedOrModifiedItem_RL_ReconfReadyTDD },
};

static int
dissect_rnsap_USCHToBeAddedOrModifiedList_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_USCHToBeAddedOrModifiedList_RL_ReconfReadyTDD, USCHToBeAddedOrModifiedList_RL_ReconfReadyTDD_sequence_of,
                                                  0, 10);

  return offset;
}
static int dissect_id_USCHToBeAddedOrModifiedList_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_USCHToBeAddedOrModifiedList_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_id_USCHToBeAddedOrModifiedList_RL_ReconfReadyTDD);
}



static int
dissect_rnsap_MaxNrTimeslots(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 14U, NULL, FALSE);

  return offset;
}
static int dissect_maxNrTimeslots_UL(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MaxNrTimeslots(tvb, offset, actx, tree, hf_rnsap_maxNrTimeslots_UL);
}
static int dissect_maxNrTimeslots_DL(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MaxNrTimeslots(tvb, offset, actx, tree, hf_rnsap_maxNrTimeslots_DL);
}



static int
dissect_rnsap_MinimumSpreadingFactor(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 16U, NULL, FALSE);

  return offset;
}
static int dissect_minimumSpreadingFactor_UL(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MinimumSpreadingFactor(tvb, offset, actx, tree, hf_rnsap_minimumSpreadingFactor_UL);
}
static int dissect_minimumSpreadingFactor_DL(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MinimumSpreadingFactor(tvb, offset, actx, tree, hf_rnsap_minimumSpreadingFactor_DL);
}



static int
dissect_rnsap_MaxNrDLPhysicalchannels(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 224U, NULL, FALSE);

  return offset;
}
static int dissect_maxNrDLPhysicalchannels(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MaxNrDLPhysicalchannels(tvb, offset, actx, tree, hf_rnsap_maxNrDLPhysicalchannels);
}


static const per_sequence_t DL_Physical_Channel_Information_RL_SetupRqstTDD_sequence[] = {
  { &hf_rnsap_maxNrTimeslots_DL, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MaxNrTimeslots },
  { &hf_rnsap_minimumSpreadingFactor_DL, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MinimumSpreadingFactor },
  { &hf_rnsap_maxNrDLPhysicalchannels, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MaxNrDLPhysicalchannels },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_Physical_Channel_Information_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_Physical_Channel_Information_RL_SetupRqstTDD, DL_Physical_Channel_Information_RL_SetupRqstTDD_sequence);

  return offset;
}
static int dissect_id_DL_Physical_Channel_Information_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_Physical_Channel_Information_RL_SetupRqstTDD(tvb, offset, actx, tree, hf_rnsap_id_DL_Physical_Channel_Information_RL_SetupRqstTDD);
}



static int
dissect_rnsap_MaxNrULPhysicalchannels(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 2U, NULL, FALSE);

  return offset;
}
static int dissect_maxNrULPhysicalchannels(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MaxNrULPhysicalchannels(tvb, offset, actx, tree, hf_rnsap_maxNrULPhysicalchannels);
}


static const per_sequence_t UL_Physical_Channel_Information_RL_SetupRqstTDD_sequence[] = {
  { &hf_rnsap_maxNrTimeslots_UL, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MaxNrTimeslots },
  { &hf_rnsap_minimumSpreadingFactor_UL, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MinimumSpreadingFactor },
  { &hf_rnsap_maxNrULPhysicalchannels, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MaxNrULPhysicalchannels },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UL_Physical_Channel_Information_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UL_Physical_Channel_Information_RL_SetupRqstTDD, UL_Physical_Channel_Information_RL_SetupRqstTDD_sequence);

  return offset;
}
static int dissect_id_UL_Physical_Channel_Information_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_Physical_Channel_Information_RL_SetupRqstTDD(tvb, offset, actx, tree, hf_rnsap_id_UL_Physical_Channel_Information_RL_SetupRqstTDD);
}


static const value_string rnsap_FNReportingIndicator_vals[] = {
  {   0, "fN-reporting-required" },
  {   1, "fN-reporting-not-required" },
  { 0, NULL }
};


static int
dissect_rnsap_FNReportingIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_CFNReportingIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_FNReportingIndicator(tvb, offset, actx, tree, hf_rnsap_id_CFNReportingIndicator);
}
static int dissect_id_SFNReportingIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_FNReportingIndicator(tvb, offset, actx, tree, hf_rnsap_id_SFNReportingIndicator);
}


static const value_string rnsap_PagingCause_vals[] = {
  {   0, "terminating-conversational-call" },
  {   1, "terminating-streaming-call" },
  {   2, "terminating-interactive-call" },
  {   3, "terminating-background-call" },
  {   4, "terminating-low-priority-signalling" },
  {   5, "terminating-high-priority-signalling" },
  {   6, "terminating-cause-unknown" },
  { 0, NULL }
};


static int
dissect_rnsap_PagingCause(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     5, NULL, TRUE, 2, NULL);

  return offset;
}
static int dissect_pagingCause(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PagingCause(tvb, offset, actx, tree, hf_rnsap_pagingCause);
}


static const value_string rnsap_CNDomainType_vals[] = {
  {   0, "cs-domain" },
  {   1, "ps-domain" },
  {   2, "dont-care" },
  { 0, NULL }
};


static int
dissect_rnsap_CNDomainType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_cNDomainType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CNDomainType(tvb, offset, actx, tree, hf_rnsap_cNDomainType);
}


static const value_string rnsap_PagingRecordType_vals[] = {
  {   0, "imsi-gsm-map" },
  {   1, "tmsi-gsm-map" },
  {   2, "p-tmsi-gsm-map" },
  {   3, "imsi-ds-41" },
  {   4, "tmsi-ds-41" },
  { 0, NULL }
};


static int
dissect_rnsap_PagingRecordType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     5, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_pagingRecordType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PagingRecordType(tvb, offset, actx, tree, hf_rnsap_pagingRecordType);
}


static const per_sequence_t CNOriginatedPage_PagingRqst_sequence[] = {
  { &hf_rnsap_pagingCause   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PagingCause },
  { &hf_rnsap_cNDomainType  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CNDomainType },
  { &hf_rnsap_pagingRecordType, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PagingRecordType },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_CNOriginatedPage_PagingRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_CNOriginatedPage_PagingRqst, CNOriginatedPage_PagingRqst_sequence);

  return offset;
}
static int dissect_id_CNOriginatedPage_PagingRqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CNOriginatedPage_PagingRqst(tvb, offset, actx, tree, hf_rnsap_id_CNOriginatedPage_PagingRqst);
}


static const per_sequence_t CCTrCH_InformationItem_RL_FailureInd_sequence[] = {
  { &hf_rnsap_cCTrCH_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CCTrCH_ID },
  { &hf_rnsap_cause         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_Cause },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_CCTrCH_InformationItem_RL_FailureInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_CCTrCH_InformationItem_RL_FailureInd, CCTrCH_InformationItem_RL_FailureInd_sequence);

  return offset;
}
static int dissect_id_CCTrCH_InformationItem_RL_FailureInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CCTrCH_InformationItem_RL_FailureInd(tvb, offset, actx, tree, hf_rnsap_id_CCTrCH_InformationItem_RL_FailureInd);
}


static const per_sequence_t CCTrCH_InformationItem_RL_RestoreInd_sequence[] = {
  { &hf_rnsap_cCTrCH_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CCTrCH_ID },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_CCTrCH_InformationItem_RL_RestoreInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_CCTrCH_InformationItem_RL_RestoreInd, CCTrCH_InformationItem_RL_RestoreInd_sequence);

  return offset;
}
static int dissect_id_CCTrCH_InformationItem_RL_RestoreInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CCTrCH_InformationItem_RL_RestoreInd(tvb, offset, actx, tree, hf_rnsap_id_CCTrCH_InformationItem_RL_RestoreInd);
}


static const value_string rnsap_TUTRANGPSAccuracyClass_vals[] = {
  {   0, "accuracy-class-A" },
  {   1, "accuracy-class-B" },
  {   2, "accuracy-class-C" },
  { 0, NULL }
};


static int
dissect_rnsap_TUTRANGPSAccuracyClass(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_tUTRANGPSMeasurementAccuracyClass(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TUTRANGPSAccuracyClass(tvb, offset, actx, tree, hf_rnsap_tUTRANGPSMeasurementAccuracyClass);
}


static const value_string rnsap_CommonMeasurementAccuracy_vals[] = {
  {   0, "tUTRANGPSMeasurementAccuracyClass" },
  { 0, NULL }
};

static const per_choice_t CommonMeasurementAccuracy_choice[] = {
  {   0, &hf_rnsap_tUTRANGPSMeasurementAccuracyClass, ASN1_EXTENSION_ROOT    , dissect_rnsap_TUTRANGPSAccuracyClass },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_CommonMeasurementAccuracy(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_CommonMeasurementAccuracy, CommonMeasurementAccuracy_choice,
                                 NULL);

  return offset;
}
static int dissect_id_CommonMeasurementAccuracy(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CommonMeasurementAccuracy(tvb, offset, actx, tree, hf_rnsap_id_CommonMeasurementAccuracy);
}



static int
dissect_rnsap_INTEGER_0_16383(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 16383U, NULL, FALSE);

  return offset;
}
static int dissect_ms_part(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_0_16383(tvb, offset, actx, tree, hf_rnsap_ms_part);
}



static int
dissect_rnsap_INTEGER_0_4294967295(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 4294967295U, NULL, FALSE);

  return offset;
}
static int dissect_ls_part(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_0_4294967295(tvb, offset, actx, tree, hf_rnsap_ls_part);
}


static const per_sequence_t TUTRANGPS_sequence[] = {
  { &hf_rnsap_ms_part       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_INTEGER_0_16383 },
  { &hf_rnsap_ls_part       , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_INTEGER_0_4294967295 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_TUTRANGPS(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_TUTRANGPS, TUTRANGPS_sequence);

  return offset;
}
static int dissect_tUTRANGPS(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TUTRANGPS(tvb, offset, actx, tree, hf_rnsap_tUTRANGPS);
}



static int
dissect_rnsap_TUTRANGPSQuality(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 255U, NULL, FALSE);

  return offset;
}
static int dissect_tUTRANGPSQuality(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TUTRANGPSQuality(tvb, offset, actx, tree, hf_rnsap_tUTRANGPSQuality);
}



static int
dissect_rnsap_TUTRANGPSDriftRate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              -50, 50U, NULL, FALSE);

  return offset;
}
static int dissect_tUTRANGPSDriftRate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TUTRANGPSDriftRate(tvb, offset, actx, tree, hf_rnsap_tUTRANGPSDriftRate);
}



static int
dissect_rnsap_TUTRANGPSDriftRateQuality(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 50U, NULL, FALSE);

  return offset;
}
static int dissect_tUTRANGPSDriftRateQuality(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TUTRANGPSDriftRateQuality(tvb, offset, actx, tree, hf_rnsap_tUTRANGPSDriftRateQuality);
}


static const per_sequence_t TUTRANGPSMeasurementValueInformation_sequence[] = {
  { &hf_rnsap_tUTRANGPS     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TUTRANGPS },
  { &hf_rnsap_tUTRANGPSQuality, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TUTRANGPSQuality },
  { &hf_rnsap_tUTRANGPSDriftRate, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TUTRANGPSDriftRate },
  { &hf_rnsap_tUTRANGPSDriftRateQuality, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TUTRANGPSDriftRateQuality },
  { &hf_rnsap_iEe_Extensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_TUTRANGPSMeasurementValueInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_TUTRANGPSMeasurementValueInformation, TUTRANGPSMeasurementValueInformation_sequence);

  return offset;
}
static int dissect_tUTRANGPSMeasurementValueInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TUTRANGPSMeasurementValueInformation(tvb, offset, actx, tree, hf_rnsap_tUTRANGPSMeasurementValueInformation);
}



static int
dissect_rnsap_SFNSFN_FDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 614399U, NULL, FALSE);

  return offset;
}
static int dissect_sFNSFN_FDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SFNSFN_FDD(tvb, offset, actx, tree, hf_rnsap_sFNSFN_FDD);
}



static int
dissect_rnsap_SFNSFN_TDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 40961U, NULL, FALSE);

  return offset;
}
static int dissect_sFNSFN_TDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SFNSFN_TDD(tvb, offset, actx, tree, hf_rnsap_sFNSFN_TDD);
}


static const value_string rnsap_SFNSFNValue_vals[] = {
  {   0, "sFNSFN-FDD" },
  {   1, "sFNSFN-TDD" },
  { 0, NULL }
};

static const per_choice_t SFNSFNValue_choice[] = {
  {   0, &hf_rnsap_sFNSFN_FDD    , ASN1_EXTENSION_ROOT    , dissect_rnsap_SFNSFN_FDD },
  {   1, &hf_rnsap_sFNSFN_TDD    , ASN1_EXTENSION_ROOT    , dissect_rnsap_SFNSFN_TDD },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_SFNSFNValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_SFNSFNValue, SFNSFNValue_choice,
                                 NULL);

  return offset;
}
static int dissect_sFNSFNValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SFNSFNValue(tvb, offset, actx, tree, hf_rnsap_sFNSFNValue);
}



static int
dissect_rnsap_SFNSFNQuality(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 255U, NULL, FALSE);

  return offset;
}
static int dissect_sFNSFNQuality(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SFNSFNQuality(tvb, offset, actx, tree, hf_rnsap_sFNSFNQuality);
}



static int
dissect_rnsap_SFNSFNDriftRate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              -100, 100U, NULL, FALSE);

  return offset;
}
static int dissect_sFNSFNDriftRate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SFNSFNDriftRate(tvb, offset, actx, tree, hf_rnsap_sFNSFNDriftRate);
}



static int
dissect_rnsap_SFNSFNDriftRateQuality(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 100U, NULL, FALSE);

  return offset;
}
static int dissect_sFNSFNDriftRateQuality(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SFNSFNDriftRateQuality(tvb, offset, actx, tree, hf_rnsap_sFNSFNDriftRateQuality);
}



static int
dissect_rnsap_SFN(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 4095U, NULL, FALSE);

  return offset;
}
static int dissect_id_SFN(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SFN(tvb, offset, actx, tree, hf_rnsap_id_SFN);
}
static int dissect_sFNSFNTimeStamp_FDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SFN(tvb, offset, actx, tree, hf_rnsap_sFNSFNTimeStamp_FDD);
}
static int dissect_sFN(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SFN(tvb, offset, actx, tree, hf_rnsap_sFN);
}


static const per_sequence_t SFNSFNTimeStamp_TDD_sequence[] = {
  { &hf_rnsap_sFN           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SFN },
  { &hf_rnsap_timeSlot      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TimeSlot },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_SFNSFNTimeStamp_TDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_SFNSFNTimeStamp_TDD, SFNSFNTimeStamp_TDD_sequence);

  return offset;
}
static int dissect_sFNSFNTimeStamp_TDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SFNSFNTimeStamp_TDD(tvb, offset, actx, tree, hf_rnsap_sFNSFNTimeStamp_TDD);
}


static const value_string rnsap_SFNSFNTimeStampInformation_vals[] = {
  {   0, "sFNSFNTimeStamp-FDD" },
  {   1, "sFNSFNTimeStamp-TDD" },
  { 0, NULL }
};

static const per_choice_t SFNSFNTimeStampInformation_choice[] = {
  {   0, &hf_rnsap_sFNSFNTimeStamp_FDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_SFN },
  {   1, &hf_rnsap_sFNSFNTimeStamp_TDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_SFNSFNTimeStamp_TDD },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_SFNSFNTimeStampInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_SFNSFNTimeStampInformation, SFNSFNTimeStampInformation_choice,
                                 NULL);

  return offset;
}
static int dissect_sFNSFNTimeStampInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SFNSFNTimeStampInformation(tvb, offset, actx, tree, hf_rnsap_sFNSFNTimeStampInformation);
}


static const per_sequence_t T_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item_sequence[] = {
  { &hf_rnsap_uC_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UC_ID },
  { &hf_rnsap_sFNSFNValue   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SFNSFNValue },
  { &hf_rnsap_sFNSFNQuality , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_SFNSFNQuality },
  { &hf_rnsap_sFNSFNDriftRate, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SFNSFNDriftRate },
  { &hf_rnsap_sFNSFNDriftRateQuality, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_SFNSFNDriftRateQuality },
  { &hf_rnsap_sFNSFNTimeStampInformation, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SFNSFNTimeStampInformation },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_T_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_T_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item, T_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item_sequence);

  return offset;
}
static int dissect_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_T_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item(tvb, offset, actx, tree, hf_rnsap_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item);
}


static const per_sequence_t T_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_sequence_of[1] = {
  { &hf_rnsap_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_T_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item },
};

static int
dissect_rnsap_T_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_T_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation, T_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_sequence_of,
                                                  1, 96);

  return offset;
}
static int dissect_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_T_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation(tvb, offset, actx, tree, hf_rnsap_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation);
}


static const per_sequence_t T_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item_sequence[] = {
  { &hf_rnsap_uC_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UC_ID },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_T_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_T_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item, T_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item_sequence);

  return offset;
}
static int dissect_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_T_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item(tvb, offset, actx, tree, hf_rnsap_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item);
}


static const per_sequence_t T_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_sequence_of[1] = {
  { &hf_rnsap_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_T_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item },
};

static int
dissect_rnsap_T_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_T_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation, T_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_sequence_of,
                                                  0, 95);

  return offset;
}
static int dissect_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_T_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation(tvb, offset, actx, tree, hf_rnsap_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation);
}


static const per_sequence_t SFNSFNMeasurementValueInformation_sequence[] = {
  { &hf_rnsap_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_T_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation },
  { &hf_rnsap_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_T_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_SFNSFNMeasurementValueInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_SFNSFNMeasurementValueInformation, SFNSFNMeasurementValueInformation_sequence);

  return offset;
}
static int dissect_sFNSFNMeasurementValueInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SFNSFNMeasurementValueInformation(tvb, offset, actx, tree, hf_rnsap_sFNSFNMeasurementValueInformation);
}



static int
dissect_rnsap_INTEGER_0_100(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 100U, NULL, FALSE);

  return offset;
}
static int dissect_transmittedCarrierPowerValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_0_100(tvb, offset, actx, tree, hf_rnsap_transmittedCarrierPowerValue);
}
static int dissect_uplinkLoadValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_0_100(tvb, offset, actx, tree, hf_rnsap_uplinkLoadValue);
}
static int dissect_downlinkLoadValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_0_100(tvb, offset, actx, tree, hf_rnsap_downlinkLoadValue);
}
static int dissect_uplinkRTLoadValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_0_100(tvb, offset, actx, tree, hf_rnsap_uplinkRTLoadValue);
}
static int dissect_downlinkRTLoadValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_0_100(tvb, offset, actx, tree, hf_rnsap_downlinkRTLoadValue);
}


static const per_sequence_t LoadValue_sequence[] = {
  { &hf_rnsap_uplinkLoadValue, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_INTEGER_0_100 },
  { &hf_rnsap_downlinkLoadValue, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_INTEGER_0_100 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_LoadValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_LoadValue, LoadValue_sequence);

  return offset;
}
static int dissect_loadValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_LoadValue(tvb, offset, actx, tree, hf_rnsap_loadValue);
}



static int
dissect_rnsap_INTEGER_0_621(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 621U, NULL, FALSE);

  return offset;
}
static int dissect_receivedTotalWideBandPowerValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_0_621(tvb, offset, actx, tree, hf_rnsap_receivedTotalWideBandPowerValue);
}



static int
dissect_rnsap_Extension_CommonMeasurementValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_extension_CommonMeasurementValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Extension_CommonMeasurementValue(tvb, offset, actx, tree, hf_rnsap_extension_CommonMeasurementValue);
}


static const value_string rnsap_CommonMeasurementValue_vals[] = {
  {   0, "tUTRANGPSMeasurementValueInformation" },
  {   1, "sFNSFNMeasurementValueInformation" },
  {   2, "loadValue" },
  {   3, "transmittedCarrierPowerValue" },
  {   4, "receivedTotalWideBandPowerValue" },
  {   5, "uplinkTimeslotISCPValue" },
  {   6, "extension-CommonMeasurementValue" },
  { 0, NULL }
};

static const per_choice_t CommonMeasurementValue_choice[] = {
  {   0, &hf_rnsap_tUTRANGPSMeasurementValueInformation, ASN1_EXTENSION_ROOT    , dissect_rnsap_TUTRANGPSMeasurementValueInformation },
  {   1, &hf_rnsap_sFNSFNMeasurementValueInformation, ASN1_EXTENSION_ROOT    , dissect_rnsap_SFNSFNMeasurementValueInformation },
  {   2, &hf_rnsap_loadValue     , ASN1_EXTENSION_ROOT    , dissect_rnsap_LoadValue },
  {   3, &hf_rnsap_transmittedCarrierPowerValue, ASN1_EXTENSION_ROOT    , dissect_rnsap_INTEGER_0_100 },
  {   4, &hf_rnsap_receivedTotalWideBandPowerValue, ASN1_EXTENSION_ROOT    , dissect_rnsap_INTEGER_0_621 },
  {   5, &hf_rnsap_uplinkTimeslotISCPValue, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_TimeslotISCP },
  {   6, &hf_rnsap_extension_CommonMeasurementValue, ASN1_NOT_EXTENSION_ROOT, dissect_rnsap_Extension_CommonMeasurementValue },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_CommonMeasurementValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_CommonMeasurementValue, CommonMeasurementValue_choice,
                                 NULL);

  return offset;
}
static int dissect_commonMeasurementValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CommonMeasurementValue(tvb, offset, actx, tree, hf_rnsap_commonMeasurementValue);
}


static const per_sequence_t CommonMeasurementAvailable_sequence[] = {
  { &hf_rnsap_commonMeasurementValue, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CommonMeasurementValue },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_CommonMeasurementAvailable(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_CommonMeasurementAvailable, CommonMeasurementAvailable_sequence);

  return offset;
}
static int dissect_measurementAvailable(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CommonMeasurementAvailable(tvb, offset, actx, tree, hf_rnsap_measurementAvailable);
}


static const value_string rnsap_CommonMeasurementValueInformation_vals[] = {
  {   0, "measurementAvailable" },
  {   1, "measurementnotAvailable" },
  { 0, NULL }
};

static const per_choice_t CommonMeasurementValueInformation_choice[] = {
  {   0, &hf_rnsap_measurementAvailable, ASN1_NO_EXTENSIONS     , dissect_rnsap_CommonMeasurementAvailable },
  {   1, &hf_rnsap_measurementnotAvailable, ASN1_NO_EXTENSIONS     , dissect_rnsap_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_CommonMeasurementValueInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_CommonMeasurementValueInformation, CommonMeasurementValueInformation_choice,
                                 NULL);

  return offset;
}
static int dissect_commonMeasurementValueInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CommonMeasurementValueInformation(tvb, offset, actx, tree, hf_rnsap_commonMeasurementValueInformation);
}


static const per_sequence_t Cell_CM_Rprt_sequence[] = {
  { &hf_rnsap_commonMeasurementValueInformation, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CommonMeasurementValueInformation },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_Cell_CM_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_Cell_CM_Rprt, Cell_CM_Rprt_sequence);

  return offset;
}
static int dissect_cell3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Cell_CM_Rprt(tvb, offset, actx, tree, hf_rnsap_cell3);
}


static const value_string rnsap_CommonMeasurementObjectType_CM_Rprt_vals[] = {
  {   0, "cell" },
  { 0, NULL }
};

static const per_choice_t CommonMeasurementObjectType_CM_Rprt_choice[] = {
  {   0, &hf_rnsap_cell3         , ASN1_EXTENSION_ROOT    , dissect_rnsap_Cell_CM_Rprt },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_CommonMeasurementObjectType_CM_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_CommonMeasurementObjectType_CM_Rprt, CommonMeasurementObjectType_CM_Rprt_choice,
                                 NULL);

  return offset;
}
static int dissect_id_CommonMeasurementObjectType_CM_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CommonMeasurementObjectType_CM_Rprt(tvb, offset, actx, tree, hf_rnsap_id_CommonMeasurementObjectType_CM_Rprt);
}



static int
dissect_rnsap_TimeSlotLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 6U, NULL, FALSE);

  return offset;
}
static int dissect_timeSlotLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TimeSlotLCR(tvb, offset, actx, tree, hf_rnsap_timeSlotLCR);
}
static int dissect_timeslotLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TimeSlotLCR(tvb, offset, actx, tree, hf_rnsap_timeslotLCR);
}
static int dissect_timeSlot1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TimeSlotLCR(tvb, offset, actx, tree, hf_rnsap_timeSlot1);
}


static const per_sequence_t NeighbouringFDDCellMeasurementInformation_sequence[] = {
  { &hf_rnsap_uC_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UC_ID },
  { &hf_rnsap_uARFCN        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UARFCN },
  { &hf_rnsap_primaryScramblingCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PrimaryScramblingCode },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_NeighbouringFDDCellMeasurementInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_NeighbouringFDDCellMeasurementInformation, NeighbouringFDDCellMeasurementInformation_sequence);

  return offset;
}
static int dissect_neighbouringFDDCellMeasurementInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_NeighbouringFDDCellMeasurementInformation(tvb, offset, actx, tree, hf_rnsap_neighbouringFDDCellMeasurementInformation);
}


static const per_sequence_t NeighbouringTDDCellMeasurementInformation_sequence[] = {
  { &hf_rnsap_uC_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UC_ID },
  { &hf_rnsap_uARFCN        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UARFCN },
  { &hf_rnsap_cellParameterID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CellParameterID },
  { &hf_rnsap_timeSlot      , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TimeSlot },
  { &hf_rnsap_midambleShiftAndBurstType, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_MidambleShiftAndBurstType },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_NeighbouringTDDCellMeasurementInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_NeighbouringTDDCellMeasurementInformation, NeighbouringTDDCellMeasurementInformation_sequence);

  return offset;
}
static int dissect_neighbouringTDDCellMeasurementInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_NeighbouringTDDCellMeasurementInformation(tvb, offset, actx, tree, hf_rnsap_neighbouringTDDCellMeasurementInformation);
}



static int
dissect_rnsap_Extension_neighbouringCellMeasurementInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_extension_neighbouringCellMeasurementInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Extension_neighbouringCellMeasurementInformation(tvb, offset, actx, tree, hf_rnsap_extension_neighbouringCellMeasurementInformation);
}


static const value_string rnsap_NeighbouringCellMeasurementInfo_item_vals[] = {
  {   0, "neighbouringFDDCellMeasurementInformation" },
  {   1, "neighbouringTDDCellMeasurementInformation" },
  {   2, "extension-neighbouringCellMeasurementInformation" },
  { 0, NULL }
};

static const per_choice_t NeighbouringCellMeasurementInfo_item_choice[] = {
  {   0, &hf_rnsap_neighbouringFDDCellMeasurementInformation, ASN1_EXTENSION_ROOT    , dissect_rnsap_NeighbouringFDDCellMeasurementInformation },
  {   1, &hf_rnsap_neighbouringTDDCellMeasurementInformation, ASN1_EXTENSION_ROOT    , dissect_rnsap_NeighbouringTDDCellMeasurementInformation },
  {   2, &hf_rnsap_extension_neighbouringCellMeasurementInformation, ASN1_NOT_EXTENSION_ROOT, dissect_rnsap_Extension_neighbouringCellMeasurementInformation },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_NeighbouringCellMeasurementInfo_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_NeighbouringCellMeasurementInfo_item, NeighbouringCellMeasurementInfo_item_choice,
                                 NULL);

  return offset;
}
static int dissect_NeighbouringCellMeasurementInfo_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_NeighbouringCellMeasurementInfo_item(tvb, offset, actx, tree, hf_rnsap_NeighbouringCellMeasurementInfo_item);
}


static const per_sequence_t NeighbouringCellMeasurementInfo_sequence_of[1] = {
  { &hf_rnsap_NeighbouringCellMeasurementInfo_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_NeighbouringCellMeasurementInfo_item },
};

static int
dissect_rnsap_NeighbouringCellMeasurementInfo(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_NeighbouringCellMeasurementInfo, NeighbouringCellMeasurementInfo_sequence_of,
                                                  1, 96);

  return offset;
}
static int dissect_neighbouringCellMeasurementInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_NeighbouringCellMeasurementInfo(tvb, offset, actx, tree, hf_rnsap_neighbouringCellMeasurementInformation);
}


static const per_sequence_t Cell_CM_Rqst_sequence[] = {
  { &hf_rnsap_uC_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UC_ID },
  { &hf_rnsap_timeSlot      , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TimeSlot },
  { &hf_rnsap_timeSlotLCR   , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TimeSlotLCR },
  { &hf_rnsap_neighbouringCellMeasurementInformation, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_NeighbouringCellMeasurementInfo },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_Cell_CM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_Cell_CM_Rqst, Cell_CM_Rqst_sequence);

  return offset;
}
static int dissect_cell1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Cell_CM_Rqst(tvb, offset, actx, tree, hf_rnsap_cell1);
}


static const value_string rnsap_CommonMeasurementObjectType_CM_Rqst_vals[] = {
  {   0, "cell" },
  { 0, NULL }
};

static const per_choice_t CommonMeasurementObjectType_CM_Rqst_choice[] = {
  {   0, &hf_rnsap_cell1         , ASN1_EXTENSION_ROOT    , dissect_rnsap_Cell_CM_Rqst },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_CommonMeasurementObjectType_CM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_CommonMeasurementObjectType_CM_Rqst, CommonMeasurementObjectType_CM_Rqst_choice,
                                 NULL);

  return offset;
}
static int dissect_id_CommonMeasurementObjectType_CM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CommonMeasurementObjectType_CM_Rqst(tvb, offset, actx, tree, hf_rnsap_id_CommonMeasurementObjectType_CM_Rqst);
}


static const per_sequence_t Cell_CM_Rsp_sequence[] = {
  { &hf_rnsap_commonMeasurementValue, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CommonMeasurementValue },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_Cell_CM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_Cell_CM_Rsp, Cell_CM_Rsp_sequence);

  return offset;
}
static int dissect_cell2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Cell_CM_Rsp(tvb, offset, actx, tree, hf_rnsap_cell2);
}


static const value_string rnsap_CommonMeasurementObjectType_CM_Rsp_vals[] = {
  {   0, "cell" },
  { 0, NULL }
};

static const per_choice_t CommonMeasurementObjectType_CM_Rsp_choice[] = {
  {   0, &hf_rnsap_cell2         , ASN1_EXTENSION_ROOT    , dissect_rnsap_Cell_CM_Rsp },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_CommonMeasurementObjectType_CM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_CommonMeasurementObjectType_CM_Rsp, CommonMeasurementObjectType_CM_Rsp_choice,
                                 NULL);

  return offset;
}
static int dissect_id_CommonMeasurementObjectType_CM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CommonMeasurementObjectType_CM_Rsp(tvb, offset, actx, tree, hf_rnsap_id_CommonMeasurementObjectType_CM_Rsp);
}


static const value_string rnsap_CommonMeasurementType_vals[] = {
  {   0, "uTRAN-GPS-timing-of-cell-frames-for-UE-Positioning" },
  {   1, "sFN-SFN-observerd-time-difference" },
  {   2, "load" },
  {   3, "transmitted-carrier-power" },
  {   4, "received-total-wide-band-power" },
  {   5, "uplink-timeslot-iscp" },
  {   6, "rT-load" },
  {   7, "nRT-load-Information" },
  {   8, "upPTSInterference" },
  { 0, NULL }
};


static int
dissect_rnsap_CommonMeasurementType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     6, NULL, TRUE, 3, NULL);

  return offset;
}
static int dissect_id_CommonMeasurementType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CommonMeasurementType(tvb, offset, actx, tree, hf_rnsap_id_CommonMeasurementType);
}


static const value_string rnsap_CongestionCause_vals[] = {
  {   0, "uTRAN-dynamic-resources" },
  {   1, "uTRAN-semistatic-resources" },
  { 0, NULL }
};


static int
dissect_rnsap_CongestionCause(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_id_CongestionCause(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CongestionCause(tvb, offset, actx, tree, hf_rnsap_id_CongestionCause);
}



static int
dissect_rnsap_InformationExchangeID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 1048575U, NULL, FALSE);

  return offset;
}
static int dissect_id_InformationExchangeID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_InformationExchangeID(tvb, offset, actx, tree, hf_rnsap_id_InformationExchangeID);
}


static const per_sequence_t GA_AccessPointPositionwithOptionalAltitude_sequence[] = {
  { &hf_rnsap_geographicalCoordinate, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_GeographicalCoordinate },
  { &hf_rnsap_altitudeAndDirection, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_GA_AltitudeAndDirection },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_GA_AccessPointPositionwithOptionalAltitude(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_GA_AccessPointPositionwithOptionalAltitude, GA_AccessPointPositionwithOptionalAltitude_sequence);

  return offset;
}
static int dissect_gA_AccessPointPositionwithAltitude(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GA_AccessPointPositionwithOptionalAltitude(tvb, offset, actx, tree, hf_rnsap_gA_AccessPointPositionwithAltitude);
}
static int dissect_sFNSFN_GA_AccessPointPosition(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GA_AccessPointPositionwithOptionalAltitude(tvb, offset, actx, tree, hf_rnsap_sFNSFN_GA_AccessPointPosition);
}


static const value_string rnsap_IPSpacingFDD_vals[] = {
  {   0, "ipsF5" },
  {   1, "ipsF7" },
  {   2, "ipsF10" },
  {   3, "ipsF15" },
  {   4, "ipsF20" },
  {   5, "ipsF30" },
  {   6, "ipsF40" },
  {   7, "ipsF50" },
  { 0, NULL }
};


static int
dissect_rnsap_IPSpacingFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_iPSpacingFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_IPSpacingFDD(tvb, offset, actx, tree, hf_rnsap_iPSpacingFDD);
}


static const value_string rnsap_IPLength_vals[] = {
  {   0, "ipl5" },
  {   1, "ipl10" },
  { 0, NULL }
};


static int
dissect_rnsap_IPLength(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_iPLength(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_IPLength(tvb, offset, actx, tree, hf_rnsap_iPLength);
}



static int
dissect_rnsap_IPOffset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 9U, NULL, FALSE);

  return offset;
}
static int dissect_iPOffset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_IPOffset(tvb, offset, actx, tree, hf_rnsap_iPOffset);
}



static int
dissect_rnsap_Seed(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 63U, NULL, FALSE);

  return offset;
}
static int dissect_seed(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Seed(tvb, offset, actx, tree, hf_rnsap_seed);
}



static int
dissect_rnsap_INTEGER_10_25(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              10U, 25U, NULL, FALSE);

  return offset;
}
static int dissect_burstLength(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_10_25(tvb, offset, actx, tree, hf_rnsap_burstLength);
}



static int
dissect_rnsap_INTEGER_1_16(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 16U, NULL, FALSE);

  return offset;
}
static int dissect_burstFreq(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_1_16(tvb, offset, actx, tree, hf_rnsap_burstFreq);
}


static const per_sequence_t BurstModeParameters_sequence[] = {
  { &hf_rnsap_burstStart    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_INTEGER_0_15 },
  { &hf_rnsap_burstLength   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_INTEGER_10_25 },
  { &hf_rnsap_burstFreq     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_INTEGER_1_16 },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_BurstModeParameters(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_BurstModeParameters, BurstModeParameters_sequence);

  return offset;
}
static int dissect_burstModeParameters(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BurstModeParameters(tvb, offset, actx, tree, hf_rnsap_burstModeParameters);
}


static const per_sequence_t IPDL_FDD_Parameters_sequence[] = {
  { &hf_rnsap_iPSpacingFDD  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_IPSpacingFDD },
  { &hf_rnsap_iPLength      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_IPLength },
  { &hf_rnsap_iPOffset      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_IPOffset },
  { &hf_rnsap_seed          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_Seed },
  { &hf_rnsap_burstModeParameters, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_BurstModeParameters },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_IPDL_FDD_Parameters(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_IPDL_FDD_Parameters, IPDL_FDD_Parameters_sequence);

  return offset;
}
static int dissect_iPDL_FDD_Parameters(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_IPDL_FDD_Parameters(tvb, offset, actx, tree, hf_rnsap_iPDL_FDD_Parameters);
}


static const value_string rnsap_IPSpacingTDD_vals[] = {
  {   0, "ipsT30" },
  {   1, "ipsT40" },
  {   2, "ipsT50" },
  {   3, "ipsT70" },
  {   4, "ipsT100" },
  { 0, NULL }
};


static int
dissect_rnsap_IPSpacingTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     5, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_iPSpacingTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_IPSpacingTDD(tvb, offset, actx, tree, hf_rnsap_iPSpacingTDD);
}



static int
dissect_rnsap_IPStart(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 4095U, NULL, FALSE);

  return offset;
}
static int dissect_iPStart(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_IPStart(tvb, offset, actx, tree, hf_rnsap_iPStart);
}



static int
dissect_rnsap_IPSlot(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 14U, NULL, FALSE);

  return offset;
}
static int dissect_iPSlot(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_IPSlot(tvb, offset, actx, tree, hf_rnsap_iPSlot);
}


static const value_string rnsap_IP_P_CCPCH_vals[] = {
  {   0, "switchOff-1-Frame" },
  {   1, "switchOff-2-Frames" },
  { 0, NULL }
};


static int
dissect_rnsap_IP_P_CCPCH(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_iP_P_CCPCH(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_IP_P_CCPCH(tvb, offset, actx, tree, hf_rnsap_iP_P_CCPCH);
}


static const per_sequence_t IPDL_TDD_Parameters_sequence[] = {
  { &hf_rnsap_iPSpacingTDD  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_IPSpacingTDD },
  { &hf_rnsap_iPStart       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_IPStart },
  { &hf_rnsap_iPSlot        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_IPSlot },
  { &hf_rnsap_iP_P_CCPCH    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_IP_P_CCPCH },
  { &hf_rnsap_burstModeParameters, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_BurstModeParameters },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_IPDL_TDD_Parameters(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_IPDL_TDD_Parameters, IPDL_TDD_Parameters_sequence);

  return offset;
}
static int dissect_iPDL_TDD_Parameters(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_IPDL_TDD_Parameters(tvb, offset, actx, tree, hf_rnsap_iPDL_TDD_Parameters);
}



static int
dissect_rnsap_Extension_IPDLParameters(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_extension_IPDLParameters(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Extension_IPDLParameters(tvb, offset, actx, tree, hf_rnsap_extension_IPDLParameters);
}


static const value_string rnsap_IPDLParameters_vals[] = {
  {   0, "iPDL-FDD-Parameters" },
  {   1, "iPDL-TDD-Parameters" },
  {   2, "extension-IPDLParameters" },
  { 0, NULL }
};

static const per_choice_t IPDLParameters_choice[] = {
  {   0, &hf_rnsap_iPDL_FDD_Parameters, ASN1_EXTENSION_ROOT    , dissect_rnsap_IPDL_FDD_Parameters },
  {   1, &hf_rnsap_iPDL_TDD_Parameters, ASN1_EXTENSION_ROOT    , dissect_rnsap_IPDL_TDD_Parameters },
  {   2, &hf_rnsap_extension_IPDLParameters, ASN1_NOT_EXTENSION_ROOT, dissect_rnsap_Extension_IPDLParameters },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_IPDLParameters(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_IPDLParameters, IPDLParameters_choice,
                                 NULL);

  return offset;
}
static int dissect_iPDLParameters(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_IPDLParameters(tvb, offset, actx, tree, hf_rnsap_iPDLParameters);
}



static int
dissect_rnsap_GPSTOW(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 604799U, NULL, FALSE);

  return offset;
}
static int dissect_gPSTOW(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GPSTOW(tvb, offset, actx, tree, hf_rnsap_gPSTOW);
}


static const value_string rnsap_GPS_Status_Health_vals[] = {
  {   0, "udre-1-0" },
  {   1, "udre-0-75" },
  {   2, "udre-0-5" },
  {   3, "udre-0-3" },
  {   4, "udre-0-1" },
  {   5, "no-data" },
  {   6, "invalid-data" },
  { 0, NULL }
};


static int
dissect_rnsap_GPS_Status_Health(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     7, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_gPS_Status_Health(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GPS_Status_Health(tvb, offset, actx, tree, hf_rnsap_gPS_Status_Health);
}



static int
dissect_rnsap_SAT_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 63U, NULL, FALSE);

  return offset;
}
static int dissect_badSAT_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SAT_ID(tvb, offset, actx, tree, hf_rnsap_badSAT_ID);
}
static int dissect_sAT_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SAT_ID(tvb, offset, actx, tree, hf_rnsap_sAT_ID);
}



static int
dissect_rnsap_BIT_STRING_SIZE_8(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     8, 8, FALSE, NULL);

  return offset;
}
static int dissect_iode_dgps(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_8(tvb, offset, actx, tree, hf_rnsap_iode_dgps);
}
static int dissect_wna_alm(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_8(tvb, offset, actx, tree, hf_rnsap_wna_alm);
}
static int dissect_gps_toa_alm(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_8(tvb, offset, actx, tree, hf_rnsap_gps_toa_alm);
}
static int dissect_svhealth_alm(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_8(tvb, offset, actx, tree, hf_rnsap_svhealth_alm);
}
static int dissect_alpha_zero_ionos(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_8(tvb, offset, actx, tree, hf_rnsap_alpha_zero_ionos);
}
static int dissect_alpha_one_ionos(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_8(tvb, offset, actx, tree, hf_rnsap_alpha_one_ionos);
}
static int dissect_alpha_two_ionos(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_8(tvb, offset, actx, tree, hf_rnsap_alpha_two_ionos);
}
static int dissect_alpha_three_ionos(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_8(tvb, offset, actx, tree, hf_rnsap_alpha_three_ionos);
}
static int dissect_beta_zero_ionos(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_8(tvb, offset, actx, tree, hf_rnsap_beta_zero_ionos);
}
static int dissect_beta_one_ionos(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_8(tvb, offset, actx, tree, hf_rnsap_beta_one_ionos);
}
static int dissect_beta_two_ionos(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_8(tvb, offset, actx, tree, hf_rnsap_beta_two_ionos);
}
static int dissect_beta_three_ionos(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_8(tvb, offset, actx, tree, hf_rnsap_beta_three_ionos);
}
static int dissect_t_gd_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_8(tvb, offset, actx, tree, hf_rnsap_t_gd_nav);
}
static int dissect_a_f_2_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_8(tvb, offset, actx, tree, hf_rnsap_a_f_2_nav);
}
static int dissect_t_ot_utc(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_8(tvb, offset, actx, tree, hf_rnsap_t_ot_utc);
}
static int dissect_delta_t_ls_utc(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_8(tvb, offset, actx, tree, hf_rnsap_delta_t_ls_utc);
}
static int dissect_w_n_t_utc(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_8(tvb, offset, actx, tree, hf_rnsap_w_n_t_utc);
}
static int dissect_w_n_lsf_utc(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_8(tvb, offset, actx, tree, hf_rnsap_w_n_lsf_utc);
}
static int dissect_dn_utc(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_8(tvb, offset, actx, tree, hf_rnsap_dn_utc);
}
static int dissect_delta_t_lsf_utc(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_8(tvb, offset, actx, tree, hf_rnsap_delta_t_lsf_utc);
}
static int dissect_sync_UL_codes_bitmap(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_8(tvb, offset, actx, tree, hf_rnsap_sync_UL_codes_bitmap);
}


static const value_string rnsap_UDRE_vals[] = {
  {   0, "lessThan1" },
  {   1, "between1-and-4" },
  {   2, "between4-and-8" },
  {   3, "over8" },
  { 0, NULL }
};


static int
dissect_rnsap_UDRE(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_uDRE(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UDRE(tvb, offset, actx, tree, hf_rnsap_uDRE);
}



static int
dissect_rnsap_PRC(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              -2047, 2047U, NULL, FALSE);

  return offset;
}
static int dissect_pRC(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PRC(tvb, offset, actx, tree, hf_rnsap_pRC);
}



static int
dissect_rnsap_Range_Correction_Rate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              -127, 127U, NULL, FALSE);

  return offset;
}
static int dissect_range_Correction_Rate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Range_Correction_Rate(tvb, offset, actx, tree, hf_rnsap_range_Correction_Rate);
}


static const per_sequence_t T_satellite_DGPSCorrections_Information_item_sequence[] = {
  { &hf_rnsap_sAT_ID        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SAT_ID },
  { &hf_rnsap_iode_dgps     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_8 },
  { &hf_rnsap_uDRE          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UDRE },
  { &hf_rnsap_pRC           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PRC },
  { &hf_rnsap_range_Correction_Rate, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_Range_Correction_Rate },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_T_satellite_DGPSCorrections_Information_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_T_satellite_DGPSCorrections_Information_item, T_satellite_DGPSCorrections_Information_item_sequence);

  return offset;
}
static int dissect_satellite_DGPSCorrections_Information_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_T_satellite_DGPSCorrections_Information_item(tvb, offset, actx, tree, hf_rnsap_satellite_DGPSCorrections_Information_item);
}


static const per_sequence_t T_satellite_DGPSCorrections_Information_sequence_of[1] = {
  { &hf_rnsap_satellite_DGPSCorrections_Information_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_T_satellite_DGPSCorrections_Information_item },
};

static int
dissect_rnsap_T_satellite_DGPSCorrections_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_T_satellite_DGPSCorrections_Information, T_satellite_DGPSCorrections_Information_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_satellite_DGPSCorrections_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_T_satellite_DGPSCorrections_Information(tvb, offset, actx, tree, hf_rnsap_satellite_DGPSCorrections_Information);
}


static const per_sequence_t DGPSCorrections_sequence[] = {
  { &hf_rnsap_gPSTOW        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_GPSTOW },
  { &hf_rnsap_gPS_Status_Health, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_GPS_Status_Health },
  { &hf_rnsap_satellite_DGPSCorrections_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_T_satellite_DGPSCorrections_Information },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DGPSCorrections(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DGPSCorrections, DGPSCorrections_sequence);

  return offset;
}
static int dissect_dGPSCorrections(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DGPSCorrections(tvb, offset, actx, tree, hf_rnsap_dGPSCorrections);
}



static int
dissect_rnsap_INTEGER_0_1048575(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 1048575U, NULL, FALSE);

  return offset;
}
static int dissect_tx_tow_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_0_1048575(tvb, offset, actx, tree, hf_rnsap_tx_tow_nav);
}



static int
dissect_rnsap_BIT_STRING_SIZE_14(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     14, 14, FALSE, NULL);

  return offset;
}
static int dissect_tlm_message_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_14(tvb, offset, actx, tree, hf_rnsap_tlm_message_nav);
}
static int dissect_idot_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_14(tvb, offset, actx, tree, hf_rnsap_idot_nav);
}



static int
dissect_rnsap_BIT_STRING_SIZE_2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     2, 2, FALSE, NULL);

  return offset;
}
static int dissect_tlm_revd_c_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_2(tvb, offset, actx, tree, hf_rnsap_tlm_revd_c_nav);
}
static int dissect_ca_or_p_on_l2_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_2(tvb, offset, actx, tree, hf_rnsap_ca_or_p_on_l2_nav);
}



static int
dissect_rnsap_BIT_STRING_SIZE_22(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     22, 22, FALSE, NULL);

  return offset;
}
static int dissect_ho_word_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_22(tvb, offset, actx, tree, hf_rnsap_ho_word_nav);
}
static int dissect_a_f_zero_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_22(tvb, offset, actx, tree, hf_rnsap_a_f_zero_nav);
}



static int
dissect_rnsap_BIT_STRING_SIZE_10(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     10, 10, FALSE, NULL);

  return offset;
}
static int dissect_w_n_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_10(tvb, offset, actx, tree, hf_rnsap_w_n_nav);
}
static int dissect_iodc_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_10(tvb, offset, actx, tree, hf_rnsap_iodc_nav);
}



static int
dissect_rnsap_BIT_STRING_SIZE_4(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     4, 4, FALSE, NULL);

  return offset;
}
static int dissect_user_range_accuracy_index_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_4(tvb, offset, actx, tree, hf_rnsap_user_range_accuracy_index_nav);
}



static int
dissect_rnsap_BIT_STRING_SIZE_6(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     6, 6, FALSE, NULL);

  return offset;
}
static int dissect_sv_health_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_6(tvb, offset, actx, tree, hf_rnsap_sv_health_nav);
}



static int
dissect_rnsap_BIT_STRING_SIZE_1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     1, 1, FALSE, NULL);

  return offset;
}
static int dissect_l2_p_dataflag_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_1(tvb, offset, actx, tree, hf_rnsap_l2_p_dataflag_nav);
}
static int dissect_fit_interval_flag_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_1(tvb, offset, actx, tree, hf_rnsap_fit_interval_flag_nav);
}



static int
dissect_rnsap_BIT_STRING_SIZE_87(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     87, 87, FALSE, NULL);

  return offset;
}
static int dissect_sf1_reserved_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_87(tvb, offset, actx, tree, hf_rnsap_sf1_reserved_nav);
}



static int
dissect_rnsap_BIT_STRING_SIZE_16(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     16, 16, FALSE, NULL);

  return offset;
}
static int dissect_gps_e_alm(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_16(tvb, offset, actx, tree, hf_rnsap_gps_e_alm);
}
static int dissect_gps_delta_I_alm(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_16(tvb, offset, actx, tree, hf_rnsap_gps_delta_I_alm);
}
static int dissect_omegadot_alm(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_16(tvb, offset, actx, tree, hf_rnsap_omegadot_alm);
}
static int dissect_t_oc_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_16(tvb, offset, actx, tree, hf_rnsap_t_oc_nav);
}
static int dissect_a_f_1_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_16(tvb, offset, actx, tree, hf_rnsap_a_f_1_nav);
}
static int dissect_c_rs_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_16(tvb, offset, actx, tree, hf_rnsap_c_rs_nav);
}
static int dissect_delta_n_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_16(tvb, offset, actx, tree, hf_rnsap_delta_n_nav);
}
static int dissect_c_uc_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_16(tvb, offset, actx, tree, hf_rnsap_c_uc_nav);
}
static int dissect_c_us_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_16(tvb, offset, actx, tree, hf_rnsap_c_us_nav);
}
static int dissect_t_oe_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_16(tvb, offset, actx, tree, hf_rnsap_t_oe_nav);
}
static int dissect_c_ic_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_16(tvb, offset, actx, tree, hf_rnsap_c_ic_nav);
}
static int dissect_c_is_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_16(tvb, offset, actx, tree, hf_rnsap_c_is_nav);
}
static int dissect_c_rc_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_16(tvb, offset, actx, tree, hf_rnsap_c_rc_nav);
}



static int
dissect_rnsap_BIT_STRING_SIZE_32(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     32, 32, FALSE, NULL);

  return offset;
}
static int dissect_m_zero_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_32(tvb, offset, actx, tree, hf_rnsap_m_zero_nav);
}
static int dissect_gps_e_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_32(tvb, offset, actx, tree, hf_rnsap_gps_e_nav);
}
static int dissect_a_sqrt_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_32(tvb, offset, actx, tree, hf_rnsap_a_sqrt_nav);
}
static int dissect_omega_zero_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_32(tvb, offset, actx, tree, hf_rnsap_omega_zero_nav);
}
static int dissect_i_zero_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_32(tvb, offset, actx, tree, hf_rnsap_i_zero_nav);
}
static int dissect_gps_omega_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_32(tvb, offset, actx, tree, hf_rnsap_gps_omega_nav);
}
static int dissect_a_zero_utc(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_32(tvb, offset, actx, tree, hf_rnsap_a_zero_utc);
}



static int
dissect_rnsap_BIT_STRING_SIZE_5(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     5, 5, FALSE, NULL);

  return offset;
}
static int dissect_aodo_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_5(tvb, offset, actx, tree, hf_rnsap_aodo_nav);
}



static int
dissect_rnsap_BIT_STRING_SIZE_24(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     24, 24, FALSE, NULL);

  return offset;
}
static int dissect_gps_a_sqrt_alm(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_24(tvb, offset, actx, tree, hf_rnsap_gps_a_sqrt_alm);
}
static int dissect_omegazero_alm(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_24(tvb, offset, actx, tree, hf_rnsap_omegazero_alm);
}
static int dissect_m_zero_alm(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_24(tvb, offset, actx, tree, hf_rnsap_m_zero_alm);
}
static int dissect_gps_omega_alm(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_24(tvb, offset, actx, tree, hf_rnsap_gps_omega_alm);
}
static int dissect_omegadot_nav(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_24(tvb, offset, actx, tree, hf_rnsap_omegadot_nav);
}
static int dissect_a_one_utc(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_24(tvb, offset, actx, tree, hf_rnsap_a_one_utc);
}



static int
dissect_rnsap_BIT_STRING_SIZE_20(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     20, 20, FALSE, NULL);

  return offset;
}
static int dissect_spare_zero_fill(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_20(tvb, offset, actx, tree, hf_rnsap_spare_zero_fill);
}


static const per_sequence_t GPS_NavigationModel_and_TimeRecovery_item_sequence[] = {
  { &hf_rnsap_tx_tow_nav    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_INTEGER_0_1048575 },
  { &hf_rnsap_sAT_ID        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SAT_ID },
  { &hf_rnsap_tlm_message_nav, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_14 },
  { &hf_rnsap_tlm_revd_c_nav, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_2 },
  { &hf_rnsap_ho_word_nav   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_22 },
  { &hf_rnsap_w_n_nav       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_10 },
  { &hf_rnsap_ca_or_p_on_l2_nav, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_2 },
  { &hf_rnsap_user_range_accuracy_index_nav, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_4 },
  { &hf_rnsap_sv_health_nav , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_6 },
  { &hf_rnsap_iodc_nav      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_10 },
  { &hf_rnsap_l2_p_dataflag_nav, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_1 },
  { &hf_rnsap_sf1_reserved_nav, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_87 },
  { &hf_rnsap_t_gd_nav      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_8 },
  { &hf_rnsap_t_oc_nav      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_16 },
  { &hf_rnsap_a_f_2_nav     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_8 },
  { &hf_rnsap_a_f_1_nav     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_16 },
  { &hf_rnsap_a_f_zero_nav  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_22 },
  { &hf_rnsap_c_rs_nav      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_16 },
  { &hf_rnsap_delta_n_nav   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_16 },
  { &hf_rnsap_m_zero_nav    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_32 },
  { &hf_rnsap_c_uc_nav      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_16 },
  { &hf_rnsap_gps_e_nav     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_32 },
  { &hf_rnsap_c_us_nav      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_16 },
  { &hf_rnsap_a_sqrt_nav    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_32 },
  { &hf_rnsap_t_oe_nav      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_16 },
  { &hf_rnsap_fit_interval_flag_nav, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_1 },
  { &hf_rnsap_aodo_nav      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_5 },
  { &hf_rnsap_c_ic_nav      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_16 },
  { &hf_rnsap_omega_zero_nav, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_32 },
  { &hf_rnsap_c_is_nav      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_16 },
  { &hf_rnsap_i_zero_nav    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_32 },
  { &hf_rnsap_c_rc_nav      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_16 },
  { &hf_rnsap_gps_omega_nav , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_32 },
  { &hf_rnsap_omegadot_nav  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_24 },
  { &hf_rnsap_idot_nav      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_14 },
  { &hf_rnsap_spare_zero_fill, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_20 },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_GPS_NavigationModel_and_TimeRecovery_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_GPS_NavigationModel_and_TimeRecovery_item, GPS_NavigationModel_and_TimeRecovery_item_sequence);

  return offset;
}
static int dissect_GPS_NavigationModel_and_TimeRecovery_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GPS_NavigationModel_and_TimeRecovery_item(tvb, offset, actx, tree, hf_rnsap_GPS_NavigationModel_and_TimeRecovery_item);
}


static const per_sequence_t GPS_NavigationModel_and_TimeRecovery_sequence_of[1] = {
  { &hf_rnsap_GPS_NavigationModel_and_TimeRecovery_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_GPS_NavigationModel_and_TimeRecovery_item },
};

static int
dissect_rnsap_GPS_NavigationModel_and_TimeRecovery(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_GPS_NavigationModel_and_TimeRecovery, GPS_NavigationModel_and_TimeRecovery_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_gPS_NavigationModel_and_TimeRecovery(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GPS_NavigationModel_and_TimeRecovery(tvb, offset, actx, tree, hf_rnsap_gPS_NavigationModel_and_TimeRecovery);
}


static const per_sequence_t GPS_Ionospheric_Model_sequence[] = {
  { &hf_rnsap_alpha_zero_ionos, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_8 },
  { &hf_rnsap_alpha_one_ionos, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_8 },
  { &hf_rnsap_alpha_two_ionos, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_8 },
  { &hf_rnsap_alpha_three_ionos, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_8 },
  { &hf_rnsap_beta_zero_ionos, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_8 },
  { &hf_rnsap_beta_one_ionos, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_8 },
  { &hf_rnsap_beta_two_ionos, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_8 },
  { &hf_rnsap_beta_three_ionos, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_8 },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_GPS_Ionospheric_Model(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_GPS_Ionospheric_Model, GPS_Ionospheric_Model_sequence);

  return offset;
}
static int dissect_gPS_Ionospheric_Model(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GPS_Ionospheric_Model(tvb, offset, actx, tree, hf_rnsap_gPS_Ionospheric_Model);
}


static const per_sequence_t GPS_UTC_Model_sequence[] = {
  { &hf_rnsap_a_one_utc     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_24 },
  { &hf_rnsap_a_zero_utc    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_32 },
  { &hf_rnsap_t_ot_utc      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_8 },
  { &hf_rnsap_delta_t_ls_utc, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_8 },
  { &hf_rnsap_w_n_t_utc     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_8 },
  { &hf_rnsap_w_n_lsf_utc   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_8 },
  { &hf_rnsap_dn_utc        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_8 },
  { &hf_rnsap_delta_t_lsf_utc, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_8 },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_GPS_UTC_Model(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_GPS_UTC_Model, GPS_UTC_Model_sequence);

  return offset;
}
static int dissect_gPS_UTC_Model(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GPS_UTC_Model(tvb, offset, actx, tree, hf_rnsap_gPS_UTC_Model);
}



static int
dissect_rnsap_DATA_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 3U, NULL, FALSE);

  return offset;
}
static int dissect_dATA_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DATA_ID(tvb, offset, actx, tree, hf_rnsap_dATA_ID);
}



static int
dissect_rnsap_BIT_STRING_SIZE_11(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     11, 11, FALSE, NULL);

  return offset;
}
static int dissect_gps_af_zero_alm(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_11(tvb, offset, actx, tree, hf_rnsap_gps_af_zero_alm);
}
static int dissect_gps_af_one_alm(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_11(tvb, offset, actx, tree, hf_rnsap_gps_af_one_alm);
}


static const per_sequence_t T_satellite_Almanac_Information_item_sequence[] = {
  { &hf_rnsap_dATA_ID       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DATA_ID },
  { &hf_rnsap_sAT_ID        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SAT_ID },
  { &hf_rnsap_gps_e_alm     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_16 },
  { &hf_rnsap_gps_toa_alm   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_8 },
  { &hf_rnsap_gps_delta_I_alm, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_16 },
  { &hf_rnsap_omegadot_alm  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_16 },
  { &hf_rnsap_svhealth_alm  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_8 },
  { &hf_rnsap_gps_a_sqrt_alm, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_24 },
  { &hf_rnsap_omegazero_alm , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_24 },
  { &hf_rnsap_m_zero_alm    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_24 },
  { &hf_rnsap_gps_omega_alm , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_24 },
  { &hf_rnsap_gps_af_zero_alm, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_11 },
  { &hf_rnsap_gps_af_one_alm, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_11 },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_T_satellite_Almanac_Information_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_T_satellite_Almanac_Information_item, T_satellite_Almanac_Information_item_sequence);

  return offset;
}
static int dissect_satellite_Almanac_Information_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_T_satellite_Almanac_Information_item(tvb, offset, actx, tree, hf_rnsap_satellite_Almanac_Information_item);
}


static const per_sequence_t T_satellite_Almanac_Information_sequence_of[1] = {
  { &hf_rnsap_satellite_Almanac_Information_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_T_satellite_Almanac_Information_item },
};

static int
dissect_rnsap_T_satellite_Almanac_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_T_satellite_Almanac_Information, T_satellite_Almanac_Information_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_satellite_Almanac_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_T_satellite_Almanac_Information(tvb, offset, actx, tree, hf_rnsap_satellite_Almanac_Information);
}



static int
dissect_rnsap_BIT_STRING_SIZE_364(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     364, 364, FALSE, NULL);

  return offset;
}
static int dissect_sVGlobalHealth_alm(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BIT_STRING_SIZE_364(tvb, offset, actx, tree, hf_rnsap_sVGlobalHealth_alm);
}


static const per_sequence_t GPS_Almanac_sequence[] = {
  { &hf_rnsap_wna_alm       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_8 },
  { &hf_rnsap_satellite_Almanac_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_T_satellite_Almanac_Information },
  { &hf_rnsap_sVGlobalHealth_alm, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_BIT_STRING_SIZE_364 },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_GPS_Almanac(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_GPS_Almanac, GPS_Almanac_sequence);

  return offset;
}
static int dissect_gPS_Almanac(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GPS_Almanac(tvb, offset, actx, tree, hf_rnsap_gPS_Almanac);
}


static const per_sequence_t T_badSatelliteInformation_item_sequence[] = {
  { &hf_rnsap_badSAT_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SAT_ID },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_T_badSatelliteInformation_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_T_badSatelliteInformation_item, T_badSatelliteInformation_item_sequence);

  return offset;
}
static int dissect_badSatelliteInformation_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_T_badSatelliteInformation_item(tvb, offset, actx, tree, hf_rnsap_badSatelliteInformation_item);
}


static const per_sequence_t T_badSatelliteInformation_sequence_of[1] = {
  { &hf_rnsap_badSatelliteInformation_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_T_badSatelliteInformation_item },
};

static int
dissect_rnsap_T_badSatelliteInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_T_badSatelliteInformation, T_badSatelliteInformation_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_badSatelliteInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_T_badSatelliteInformation(tvb, offset, actx, tree, hf_rnsap_badSatelliteInformation);
}


static const per_sequence_t BadSatellites_sequence[] = {
  { &hf_rnsap_badSatelliteInformation, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_T_badSatelliteInformation },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_BadSatellites(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_BadSatellites, BadSatellites_sequence);

  return offset;
}
static int dissect_badSatellites(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BadSatellites(tvb, offset, actx, tree, hf_rnsap_badSatellites);
}


static const value_string rnsap_GPS_RealTime_Integrity_vals[] = {
  {   0, "badSatellites" },
  {   1, "noBadSatellite" },
  { 0, NULL }
};

static const per_choice_t GPS_RealTime_Integrity_choice[] = {
  {   0, &hf_rnsap_badSatellites , ASN1_NO_EXTENSIONS     , dissect_rnsap_BadSatellites },
  {   1, &hf_rnsap_noBadSatellite, ASN1_NO_EXTENSIONS     , dissect_rnsap_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_GPS_RealTime_Integrity(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_GPS_RealTime_Integrity, GPS_RealTime_Integrity_choice,
                                 NULL);

  return offset;
}
static int dissect_gPS_RealTime_Integrity(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GPS_RealTime_Integrity(tvb, offset, actx, tree, hf_rnsap_gPS_RealTime_Integrity);
}


static const per_sequence_t GPS_RX_POS_sequence[] = {
  { &hf_rnsap_geographicalCoordinate, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_GeographicalCoordinate },
  { &hf_rnsap_altitudeAndDirection, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_GA_AltitudeAndDirection },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_GPS_RX_POS(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_GPS_RX_POS, GPS_RX_POS_sequence);

  return offset;
}
static int dissect_gPS_RX_POS(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GPS_RX_POS(tvb, offset, actx, tree, hf_rnsap_gPS_RX_POS);
}


static const per_sequence_t RequestedDataValue_sequence[] = {
  { &hf_rnsap_gA_AccessPointPositionwithAltitude, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_GA_AccessPointPositionwithOptionalAltitude },
  { &hf_rnsap_iPDLParameters, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_IPDLParameters },
  { &hf_rnsap_dGPSCorrections, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DGPSCorrections },
  { &hf_rnsap_gPS_NavigationModel_and_TimeRecovery, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_GPS_NavigationModel_and_TimeRecovery },
  { &hf_rnsap_gPS_Ionospheric_Model, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_GPS_Ionospheric_Model },
  { &hf_rnsap_gPS_UTC_Model , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_GPS_UTC_Model },
  { &hf_rnsap_gPS_Almanac   , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_GPS_Almanac },
  { &hf_rnsap_gPS_RealTime_Integrity, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_GPS_RealTime_Integrity },
  { &hf_rnsap_gPS_RX_POS    , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_GPS_RX_POS },
  { &hf_rnsap_sFNSFN_GA_AccessPointPosition, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_GA_AccessPointPositionwithOptionalAltitude },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RequestedDataValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RequestedDataValue, RequestedDataValue_sequence);

  return offset;
}
static int dissect_requestedDataValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RequestedDataValue(tvb, offset, actx, tree, hf_rnsap_requestedDataValue);
}


static const per_sequence_t InformationAvailable_sequence[] = {
  { &hf_rnsap_requestedDataValue, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RequestedDataValue },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_InformationAvailable(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_InformationAvailable, InformationAvailable_sequence);

  return offset;
}
static int dissect_informationAvailable(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_InformationAvailable(tvb, offset, actx, tree, hf_rnsap_informationAvailable);
}



static int
dissect_rnsap_InformationNotAvailable(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_null(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_informationNotAvailable(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_InformationNotAvailable(tvb, offset, actx, tree, hf_rnsap_informationNotAvailable);
}


static const value_string rnsap_RequestedDataValueInformation_vals[] = {
  {   0, "informationAvailable" },
  {   1, "informationNotAvailable" },
  { 0, NULL }
};

static const per_choice_t RequestedDataValueInformation_choice[] = {
  {   0, &hf_rnsap_informationAvailable, ASN1_NO_EXTENSIONS     , dissect_rnsap_InformationAvailable },
  {   1, &hf_rnsap_informationNotAvailable, ASN1_NO_EXTENSIONS     , dissect_rnsap_InformationNotAvailable },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_RequestedDataValueInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_RequestedDataValueInformation, RequestedDataValueInformation_choice,
                                 NULL);

  return offset;
}
static int dissect_requestedDataValueInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RequestedDataValueInformation(tvb, offset, actx, tree, hf_rnsap_requestedDataValueInformation);
}


static const per_sequence_t Cell_InfEx_Rprt_sequence[] = {
  { &hf_rnsap_requestedDataValueInformation, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RequestedDataValueInformation },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_Cell_InfEx_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_Cell_InfEx_Rprt, Cell_InfEx_Rprt_sequence);

  return offset;
}
static int dissect_cell6(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Cell_InfEx_Rprt(tvb, offset, actx, tree, hf_rnsap_cell6);
}


static const value_string rnsap_InformationExchangeObjectType_InfEx_Rprt_vals[] = {
  {   0, "cell" },
  { 0, NULL }
};

static const per_choice_t InformationExchangeObjectType_InfEx_Rprt_choice[] = {
  {   0, &hf_rnsap_cell6         , ASN1_EXTENSION_ROOT    , dissect_rnsap_Cell_InfEx_Rprt },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_InformationExchangeObjectType_InfEx_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_InformationExchangeObjectType_InfEx_Rprt, InformationExchangeObjectType_InfEx_Rprt_choice,
                                 NULL);

  return offset;
}
static int dissect_id_InformationExchangeObjectType_InfEx_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_InformationExchangeObjectType_InfEx_Rprt(tvb, offset, actx, tree, hf_rnsap_id_InformationExchangeObjectType_InfEx_Rprt);
}


static const per_sequence_t Cell_InfEx_Rqst_sequence[] = {
  { &hf_rnsap_c_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_C_ID },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_Cell_InfEx_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_Cell_InfEx_Rqst, Cell_InfEx_Rqst_sequence);

  return offset;
}
static int dissect_cell4(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Cell_InfEx_Rqst(tvb, offset, actx, tree, hf_rnsap_cell4);
}



static int
dissect_rnsap_Extension_InformationExchangeObjectType_InfEx_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_extension_InformationExchangeObjectType_InfEx_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Extension_InformationExchangeObjectType_InfEx_Rqst(tvb, offset, actx, tree, hf_rnsap_extension_InformationExchangeObjectType_InfEx_Rqst);
}


static const value_string rnsap_InformationExchangeObjectType_InfEx_Rqst_vals[] = {
  {   0, "cell" },
  {   1, "extension-InformationExchangeObjectType-InfEx-Rqst" },
  { 0, NULL }
};

static const per_choice_t InformationExchangeObjectType_InfEx_Rqst_choice[] = {
  {   0, &hf_rnsap_cell4         , ASN1_EXTENSION_ROOT    , dissect_rnsap_Cell_InfEx_Rqst },
  {   1, &hf_rnsap_extension_InformationExchangeObjectType_InfEx_Rqst, ASN1_NOT_EXTENSION_ROOT, dissect_rnsap_Extension_InformationExchangeObjectType_InfEx_Rqst },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_InformationExchangeObjectType_InfEx_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_InformationExchangeObjectType_InfEx_Rqst, InformationExchangeObjectType_InfEx_Rqst_choice,
                                 NULL);

  return offset;
}
static int dissect_id_InformationExchangeObjectType_InfEx_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_InformationExchangeObjectType_InfEx_Rqst(tvb, offset, actx, tree, hf_rnsap_id_InformationExchangeObjectType_InfEx_Rqst);
}



static int
dissect_rnsap_INTEGER_1_24_(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 24U, NULL, TRUE);

  return offset;
}
static int dissect_hour(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_1_24_(tvb, offset, actx, tree, hf_rnsap_hour);
}


static const value_string rnsap_InformationReportPeriodicity_vals[] = {
  {   0, "min" },
  {   1, "hour" },
  { 0, NULL }
};

static const per_choice_t InformationReportPeriodicity_choice[] = {
  {   0, &hf_rnsap_min           , ASN1_EXTENSION_ROOT    , dissect_rnsap_INTEGER_1_60_ },
  {   1, &hf_rnsap_hour          , ASN1_EXTENSION_ROOT    , dissect_rnsap_INTEGER_1_24_ },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_InformationReportPeriodicity(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_InformationReportPeriodicity, InformationReportPeriodicity_choice,
                                 NULL);

  return offset;
}
static int dissect_informationReportPeriodicity(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_InformationReportPeriodicity(tvb, offset, actx, tree, hf_rnsap_informationReportPeriodicity);
}


static const per_sequence_t PeriodicInformation_sequence[] = {
  { &hf_rnsap_informationReportPeriodicity, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_InformationReportPeriodicity },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_PeriodicInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_PeriodicInformation, PeriodicInformation_sequence);

  return offset;
}
static int dissect_periodic(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PeriodicInformation(tvb, offset, actx, tree, hf_rnsap_periodic);
}


static const value_string rnsap_PRCDeviation_vals[] = {
  {   0, "prcd1" },
  {   1, "prcd2" },
  {   2, "prcd5" },
  {   3, "prcd10" },
  { 0, NULL }
};


static int
dissect_rnsap_PRCDeviation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_pRCDeviation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PRCDeviation(tvb, offset, actx, tree, hf_rnsap_pRCDeviation);
}


static const per_sequence_t DGPSThreshold_sequence[] = {
  { &hf_rnsap_pRCDeviation  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PRCDeviation },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DGPSThreshold(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DGPSThreshold, DGPSThreshold_sequence);

  return offset;
}
static int dissect_dGPSThreshold(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DGPSThreshold(tvb, offset, actx, tree, hf_rnsap_dGPSThreshold);
}


static const value_string rnsap_InformationThreshold_vals[] = {
  {   0, "dGPSThreshold" },
  { 0, NULL }
};

static const per_choice_t InformationThreshold_choice[] = {
  {   0, &hf_rnsap_dGPSThreshold , ASN1_EXTENSION_ROOT    , dissect_rnsap_DGPSThreshold },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_InformationThreshold(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_InformationThreshold, InformationThreshold_choice,
                                 NULL);

  return offset;
}
static int dissect_informationThreshold(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_InformationThreshold(tvb, offset, actx, tree, hf_rnsap_informationThreshold);
}


static const per_sequence_t OnModificationInformation_sequence[] = {
  { &hf_rnsap_informationThreshold, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_InformationThreshold },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_OnModificationInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_OnModificationInformation, OnModificationInformation_sequence);

  return offset;
}
static int dissect_onModification(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_OnModificationInformation(tvb, offset, actx, tree, hf_rnsap_onModification);
}


static const value_string rnsap_InformationReportCharacteristics_vals[] = {
  {   0, "onDemand" },
  {   1, "periodic" },
  {   2, "onModification" },
  { 0, NULL }
};

static const per_choice_t InformationReportCharacteristics_choice[] = {
  {   0, &hf_rnsap_onDemand      , ASN1_EXTENSION_ROOT    , dissect_rnsap_NULL },
  {   1, &hf_rnsap_periodic      , ASN1_EXTENSION_ROOT    , dissect_rnsap_PeriodicInformation },
  {   2, &hf_rnsap_onModification, ASN1_EXTENSION_ROOT    , dissect_rnsap_OnModificationInformation },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_InformationReportCharacteristics(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_InformationReportCharacteristics, InformationReportCharacteristics_choice,
                                 NULL);

  return offset;
}
static int dissect_id_InformationExchangeObjectType_InfEx_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_InformationReportCharacteristics(tvb, offset, actx, tree, hf_rnsap_id_InformationExchangeObjectType_InfEx_Rsp);
}
static int dissect_id_InformationReportCharacteristics(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_InformationReportCharacteristics(tvb, offset, actx, tree, hf_rnsap_id_InformationReportCharacteristics);
}


static const value_string rnsap_T_informationTypeItem_vals[] = {
  {   0, "gA-AccessPointPositionwithAltitude" },
  {   1, "gA-AccessPointPosition" },
  {   2, "iPDLParameters" },
  {   3, "gPSInformation" },
  {   4, "dGPSCorrections" },
  {   5, "gPS-RX-POS" },
  {   6, "sFNSFN-GA-AccessPointPosition" },
  {   7, "cell-Capacity-Class" },
  {   8, "nACC-Related-Data" },
  {   9, "mBMSBearerServiceFullAddress" },
  { 0, NULL }
};


static int
dissect_rnsap_T_informationTypeItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     7, NULL, TRUE, 3, NULL);

  return offset;
}
static int dissect_informationTypeItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_T_informationTypeItem(tvb, offset, actx, tree, hf_rnsap_informationTypeItem);
}


static const value_string rnsap_T_gPSInformationItem_vals[] = {
  {   0, "gPS-NavigationModel-and-TimeRecovery" },
  {   1, "gPS-Ionospheric-Model" },
  {   2, "gPS-UTC-Model" },
  {   3, "gPS-Almanac" },
  {   4, "gPS-RealTime-Integrity" },
  { 0, NULL }
};


static int
dissect_rnsap_T_gPSInformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     5, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_gPSInformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_T_gPSInformationItem(tvb, offset, actx, tree, hf_rnsap_gPSInformationItem);
}


static const per_sequence_t GPSInformation_item_sequence[] = {
  { &hf_rnsap_gPSInformationItem, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_T_gPSInformationItem },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_GPSInformation_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_GPSInformation_item, GPSInformation_item_sequence);

  return offset;
}
static int dissect_GPSInformation_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GPSInformation_item(tvb, offset, actx, tree, hf_rnsap_GPSInformation_item);
}


static const per_sequence_t GPSInformation_sequence_of[1] = {
  { &hf_rnsap_GPSInformation_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_GPSInformation_item },
};

static int
dissect_rnsap_GPSInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_GPSInformation, GPSInformation_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_gPSInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GPSInformation(tvb, offset, actx, tree, hf_rnsap_gPSInformation);
}


static const per_sequence_t InformationType_sequence[] = {
  { &hf_rnsap_informationTypeItem, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_T_informationTypeItem },
  { &hf_rnsap_gPSInformation, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_GPSInformation },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_InformationType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_InformationType, InformationType_sequence);

  return offset;
}
static int dissect_id_InformationType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_InformationType(tvb, offset, actx, tree, hf_rnsap_id_InformationType);
}


static const value_string rnsap_RestrictionStateIndicator_vals[] = {
  {   0, "cellNotResevedForOperatorUse" },
  {   1, "cellResevedForOperatorUse" },
  { 0, NULL }
};


static int
dissect_rnsap_RestrictionStateIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_id_RestrictionStateIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RestrictionStateIndicator(tvb, offset, actx, tree, hf_rnsap_id_RestrictionStateIndicator);
}
static int dissect_restrictionStateIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RestrictionStateIndicator(tvb, offset, actx, tree, hf_rnsap_restrictionStateIndicator);
}


static const per_sequence_t Neighbouring_LCR_TDD_CellInformationItem_sequence[] = {
  { &hf_rnsap_c_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_C_ID },
  { &hf_rnsap_uARFCNforNt   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UARFCN },
  { &hf_rnsap_frameOffset   , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_FrameOffset },
  { &hf_rnsap_cellParameterID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CellParameterID },
  { &hf_rnsap_sCTD_Indicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SCTD_Indicator },
  { &hf_rnsap_cellIndividualOffset, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_CellIndividualOffset },
  { &hf_rnsap_dPCHConstantValue, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DPCHConstantValue },
  { &hf_rnsap_pCCPCH_Power  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_PCCPCH_Power },
  { &hf_rnsap_restrictionStateIndicator, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_RestrictionStateIndicator },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_Neighbouring_LCR_TDD_CellInformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_Neighbouring_LCR_TDD_CellInformationItem, Neighbouring_LCR_TDD_CellInformationItem_sequence);

  return offset;
}
static int dissect_Neighbouring_LCR_TDD_CellInformation_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Neighbouring_LCR_TDD_CellInformationItem(tvb, offset, actx, tree, hf_rnsap_Neighbouring_LCR_TDD_CellInformation_item);
}


static const per_sequence_t Neighbouring_LCR_TDD_CellInformation_sequence_of[1] = {
  { &hf_rnsap_Neighbouring_LCR_TDD_CellInformation_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_Neighbouring_LCR_TDD_CellInformationItem },
};

static int
dissect_rnsap_Neighbouring_LCR_TDD_CellInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_Neighbouring_LCR_TDD_CellInformation, Neighbouring_LCR_TDD_CellInformation_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_id_neighbouring_LCR_TDD_CellInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Neighbouring_LCR_TDD_CellInformation(tvb, offset, actx, tree, hf_rnsap_id_neighbouring_LCR_TDD_CellInformation);
}


static const per_sequence_t DL_TimeSlot_ISCP_LCR_InfoItem_sequence[] = {
  { &hf_rnsap_timeSlotLCR   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TimeSlotLCR },
  { &hf_rnsap_dL_TimeslotISCP, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_TimeslotISCP },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_TimeSlot_ISCP_LCR_InfoItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_TimeSlot_ISCP_LCR_InfoItem, DL_TimeSlot_ISCP_LCR_InfoItem_sequence);

  return offset;
}
static int dissect_DL_TimeSlot_ISCP_LCR_Information_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_TimeSlot_ISCP_LCR_InfoItem(tvb, offset, actx, tree, hf_rnsap_DL_TimeSlot_ISCP_LCR_Information_item);
}


static const per_sequence_t DL_TimeSlot_ISCP_LCR_Information_sequence_of[1] = {
  { &hf_rnsap_DL_TimeSlot_ISCP_LCR_Information_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_TimeSlot_ISCP_LCR_InfoItem },
};

static int
dissect_rnsap_DL_TimeSlot_ISCP_LCR_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DL_TimeSlot_ISCP_LCR_Information, DL_TimeSlot_ISCP_LCR_Information_sequence_of,
                                                  1, 6);

  return offset;
}
static int dissect_id_DL_Timeslot_ISCP_LCR_Information_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_TimeSlot_ISCP_LCR_Information(tvb, offset, actx, tree, hf_rnsap_id_DL_Timeslot_ISCP_LCR_Information_RL_SetupRqstTDD);
}
static int dissect_id_DL_Timeslot_ISCP_LCR_Information_RL_AdditionRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_TimeSlot_ISCP_LCR_Information(tvb, offset, actx, tree, hf_rnsap_id_DL_Timeslot_ISCP_LCR_Information_RL_AdditionRqstTDD);
}
static int dissect_id_timeSlot_ISCP_LCR_List_DL_PC_Rqst_TDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_TimeSlot_ISCP_LCR_Information(tvb, offset, actx, tree, hf_rnsap_id_timeSlot_ISCP_LCR_List_DL_PC_Rqst_TDD);
}
static int dissect_id_DL_Timeslot_ISCP_LCR_Information_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_TimeSlot_ISCP_LCR_Information(tvb, offset, actx, tree, hf_rnsap_id_DL_Timeslot_ISCP_LCR_Information_RL_ReconfPrepTDD);
}



static int
dissect_rnsap_UL_Timeslot_ISCP_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_UL_TimeslotISCP(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_id_TUTRANGPSMeasurementThresholdInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_Timeslot_ISCP_Value(tvb, offset, actx, tree, hf_rnsap_id_TUTRANGPSMeasurementThresholdInformation);
}
static int dissect_id_UL_Timeslot_ISCP_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_Timeslot_ISCP_Value(tvb, offset, actx, tree, hf_rnsap_id_UL_Timeslot_ISCP_Value);
}
static int dissect_iSCP(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_Timeslot_ISCP_Value(tvb, offset, actx, tree, hf_rnsap_iSCP);
}


static const per_sequence_t UL_TimeSlot_ISCP_LCR_InfoItem_sequence[] = {
  { &hf_rnsap_timeSlotLCR   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TimeSlotLCR },
  { &hf_rnsap_iSCP          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_Timeslot_ISCP_Value },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UL_TimeSlot_ISCP_LCR_InfoItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UL_TimeSlot_ISCP_LCR_InfoItem, UL_TimeSlot_ISCP_LCR_InfoItem_sequence);

  return offset;
}
static int dissect_UL_TimeSlot_ISCP_LCR_Info_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_TimeSlot_ISCP_LCR_InfoItem(tvb, offset, actx, tree, hf_rnsap_UL_TimeSlot_ISCP_LCR_Info_item);
}


static const per_sequence_t UL_TimeSlot_ISCP_LCR_Info_sequence_of[1] = {
  { &hf_rnsap_UL_TimeSlot_ISCP_LCR_Info_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_TimeSlot_ISCP_LCR_InfoItem },
};

static int
dissect_rnsap_UL_TimeSlot_ISCP_LCR_Info(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_UL_TimeSlot_ISCP_LCR_Info, UL_TimeSlot_ISCP_LCR_Info_sequence_of,
                                                  1, 6);

  return offset;
}
static int dissect_ul_TimeSlot_ISCP_LCR_Info(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_TimeSlot_ISCP_LCR_Info(tvb, offset, actx, tree, hf_rnsap_ul_TimeSlot_ISCP_LCR_Info);
}


static const value_string rnsap_MidambleAllocationMode_vals[] = {
  {   0, "defaultMidamble" },
  {   1, "commonMidamble" },
  {   2, "uESpecificMidamble" },
  { 0, NULL }
};


static int
dissect_rnsap_MidambleAllocationMode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_midambleAllocationMode3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MidambleAllocationMode(tvb, offset, actx, tree, hf_rnsap_midambleAllocationMode3);
}


static const value_string rnsap_MidambleConfigurationLCR_vals[] = {
  {   0, "v2" },
  {   1, "v4" },
  {   2, "v6" },
  {   3, "v8" },
  {   4, "v10" },
  {   5, "v12" },
  {   6, "v14" },
  {   7, "v16" },
  { 0, NULL }
};


static int
dissect_rnsap_MidambleConfigurationLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_midambleConfigurationLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MidambleConfigurationLCR(tvb, offset, actx, tree, hf_rnsap_midambleConfigurationLCR);
}


static const per_sequence_t MidambleShiftLCR_sequence[] = {
  { &hf_rnsap_midambleAllocationMode3, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MidambleAllocationMode },
  { &hf_rnsap_midambleShift , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_MidambleShiftLong },
  { &hf_rnsap_midambleConfigurationLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MidambleConfigurationLCR },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_MidambleShiftLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_MidambleShiftLCR, MidambleShiftLCR_sequence);

  return offset;
}
static int dissect_midambleShiftLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MidambleShiftLCR(tvb, offset, actx, tree, hf_rnsap_midambleShiftLCR);
}


static const value_string rnsap_Modulation_vals[] = {
  {   0, "qPSK" },
  {   1, "eightPSK" },
  { 0, NULL }
};


static int
dissect_rnsap_Modulation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_modulation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Modulation(tvb, offset, actx, tree, hf_rnsap_modulation);
}


static const per_sequence_t TDD_ChannelisationCodeLCR_sequence[] = {
  { &hf_rnsap_tDD_ChannelisationCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_ChannelisationCode },
  { &hf_rnsap_modulation    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_Modulation },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_TDD_ChannelisationCodeLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_TDD_ChannelisationCodeLCR, TDD_ChannelisationCodeLCR_sequence);

  return offset;
}
static int dissect_tDD_ChannelisationCodeLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_ChannelisationCodeLCR(tvb, offset, actx, tree, hf_rnsap_tDD_ChannelisationCodeLCR);
}
static int dissect_tdd_ChannelisationCodeLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_ChannelisationCodeLCR(tvb, offset, actx, tree, hf_rnsap_tdd_ChannelisationCodeLCR);
}



static int
dissect_rnsap_QPSK_DL_DPCH_TimeSlotFormatTDD_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 24U, NULL, TRUE);

  return offset;
}
static int dissect_qPSK(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_QPSK_DL_DPCH_TimeSlotFormatTDD_LCR(tvb, offset, actx, tree, hf_rnsap_qPSK);
}



static int
dissect_rnsap_EightPSK_DL_DPCH_TimeSlotFormatTDD_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 24U, NULL, TRUE);

  return offset;
}
static int dissect_eightPSK(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_EightPSK_DL_DPCH_TimeSlotFormatTDD_LCR(tvb, offset, actx, tree, hf_rnsap_eightPSK);
}


static const value_string rnsap_TDD_DL_DPCH_TimeSlotFormat_LCR_vals[] = {
  {   0, "qPSK" },
  {   1, "eightPSK" },
  { 0, NULL }
};

static const per_choice_t TDD_DL_DPCH_TimeSlotFormat_LCR_choice[] = {
  {   0, &hf_rnsap_qPSK          , ASN1_EXTENSION_ROOT    , dissect_rnsap_QPSK_DL_DPCH_TimeSlotFormatTDD_LCR },
  {   1, &hf_rnsap_eightPSK      , ASN1_EXTENSION_ROOT    , dissect_rnsap_EightPSK_DL_DPCH_TimeSlotFormatTDD_LCR },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_TDD_DL_DPCH_TimeSlotFormat_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_TDD_DL_DPCH_TimeSlotFormat_LCR, TDD_DL_DPCH_TimeSlotFormat_LCR_choice,
                                 NULL);

  return offset;
}
static int dissect_id_TDD_DL_DPCH_TimeSlotFormatModifyItem_LCR_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_DL_DPCH_TimeSlotFormat_LCR(tvb, offset, actx, tree, hf_rnsap_id_TDD_DL_DPCH_TimeSlotFormatModifyItem_LCR_RL_ReconfReadyTDD);
}
static int dissect_s_CCPCH_TimeSlotFormat_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_DL_DPCH_TimeSlotFormat_LCR(tvb, offset, actx, tree, hf_rnsap_s_CCPCH_TimeSlotFormat_LCR);
}
static int dissect_tdd_DL_DPCH_TimeSlotFormat_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_DL_DPCH_TimeSlotFormat_LCR(tvb, offset, actx, tree, hf_rnsap_tdd_DL_DPCH_TimeSlotFormat_LCR);
}


static const per_sequence_t Secondary_LCR_CCPCH_TDD_Code_InformationItem_sequence[] = {
  { &hf_rnsap_tDD_ChannelisationCodeLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_ChannelisationCodeLCR },
  { &hf_rnsap_s_CCPCH_TimeSlotFormat_LCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_DL_DPCH_TimeSlotFormat_LCR },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_Secondary_LCR_CCPCH_TDD_Code_InformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_Secondary_LCR_CCPCH_TDD_Code_InformationItem, Secondary_LCR_CCPCH_TDD_Code_InformationItem_sequence);

  return offset;
}
static int dissect_Secondary_LCR_CCPCH_TDD_Code_Information_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Secondary_LCR_CCPCH_TDD_Code_InformationItem(tvb, offset, actx, tree, hf_rnsap_Secondary_LCR_CCPCH_TDD_Code_Information_item);
}


static const per_sequence_t Secondary_LCR_CCPCH_TDD_Code_Information_sequence_of[1] = {
  { &hf_rnsap_Secondary_LCR_CCPCH_TDD_Code_Information_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_Secondary_LCR_CCPCH_TDD_Code_InformationItem },
};

static int
dissect_rnsap_Secondary_LCR_CCPCH_TDD_Code_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_Secondary_LCR_CCPCH_TDD_Code_Information, Secondary_LCR_CCPCH_TDD_Code_Information_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_secondary_LCR_CCPCH_TDD_Code_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Secondary_LCR_CCPCH_TDD_Code_Information(tvb, offset, actx, tree, hf_rnsap_secondary_LCR_CCPCH_TDD_Code_Information);
}


static const per_sequence_t Secondary_LCR_CCPCH_TDD_InformationItem_sequence[] = {
  { &hf_rnsap_timeSlotLCR   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TimeSlotLCR },
  { &hf_rnsap_midambleShiftLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MidambleShiftLCR },
  { &hf_rnsap_tFCI_Presence , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TFCI_Presence },
  { &hf_rnsap_secondary_LCR_CCPCH_TDD_Code_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_Secondary_LCR_CCPCH_TDD_Code_Information },
  { &hf_rnsap_tDD_PhysicalChannelOffset, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_PhysicalChannelOffset },
  { &hf_rnsap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RepetitionLength },
  { &hf_rnsap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RepetitionPeriod },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_Secondary_LCR_CCPCH_TDD_InformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_Secondary_LCR_CCPCH_TDD_InformationItem, Secondary_LCR_CCPCH_TDD_InformationItem_sequence);

  return offset;
}
static int dissect_Secondary_LCR_CCPCH_TDD_InformationList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Secondary_LCR_CCPCH_TDD_InformationItem(tvb, offset, actx, tree, hf_rnsap_Secondary_LCR_CCPCH_TDD_InformationList_item);
}


static const per_sequence_t Secondary_LCR_CCPCH_TDD_InformationList_sequence_of[1] = {
  { &hf_rnsap_Secondary_LCR_CCPCH_TDD_InformationList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_Secondary_LCR_CCPCH_TDD_InformationItem },
};

static int
dissect_rnsap_Secondary_LCR_CCPCH_TDD_InformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_Secondary_LCR_CCPCH_TDD_InformationList, Secondary_LCR_CCPCH_TDD_InformationList_sequence_of,
                                                  0, 8);

  return offset;
}
static int dissect_secondary_LCR_CCPCH_TDD_InformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Secondary_LCR_CCPCH_TDD_InformationList(tvb, offset, actx, tree, hf_rnsap_secondary_LCR_CCPCH_TDD_InformationList);
}


static const per_sequence_t Secondary_LCR_CCPCH_Info_TDD_sequence[] = {
  { &hf_rnsap_dl_TFCS       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TFCS },
  { &hf_rnsap_tFCI_Coding   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TFCI_Coding },
  { &hf_rnsap_secondary_LCR_CCPCH_TDD_InformationList, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_Secondary_LCR_CCPCH_TDD_InformationList },
  { &hf_rnsap_fACH_InformationList, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_FACH_InformationList },
  { &hf_rnsap_pCH_InformationList, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PCH_InformationList },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_Secondary_LCR_CCPCH_Info_TDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_Secondary_LCR_CCPCH_Info_TDD, Secondary_LCR_CCPCH_Info_TDD_sequence);

  return offset;
}
static int dissect_secondary_LCR_CCPCH_Info_TDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Secondary_LCR_CCPCH_Info_TDD(tvb, offset, actx, tree, hf_rnsap_secondary_LCR_CCPCH_Info_TDD);
}



static int
dissect_rnsap_UL_LCR_CCTrCHInformationList_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_ul_LCR_CCTrCHInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_LCR_CCTrCHInformationList_RL_SetupRspTDD(tvb, offset, actx, tree, hf_rnsap_ul_LCR_CCTrCHInformation);
}



static int
dissect_rnsap_DL_LCR_CCTrCHInformationList_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_dl_LCR_CCTrCHInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_LCR_CCTrCHInformationList_RL_SetupRspTDD(tvb, offset, actx, tree, hf_rnsap_dl_LCR_CCTrCHInformation);
}



static int
dissect_rnsap_DSCH_LCR_InformationResponse_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_dsch_LCR_InformationResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DSCH_LCR_InformationResponse_RL_SetupRspTDD(tvb, offset, actx, tree, hf_rnsap_dsch_LCR_InformationResponse);
}



static int
dissect_rnsap_USCH_LCR_InformationResponse_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_usch_LCR_InformationResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_USCH_LCR_InformationResponse_RL_SetupRspTDD(tvb, offset, actx, tree, hf_rnsap_usch_LCR_InformationResponse);
}


static const per_sequence_t RL_LCR_InformationResponse_RL_SetupRspTDD_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_uRA_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_URA_Information },
  { &hf_rnsap_sAI           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SAI },
  { &hf_rnsap_gA_Cell       , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_GA_Cell },
  { &hf_rnsap_gA_AccessPointPosition, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_GA_AccessPointPosition },
  { &hf_rnsap_ul_TimeSlot_ISCP_LCR_Info, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_TimeSlot_ISCP_LCR_Info },
  { &hf_rnsap_maxUL_SIR     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_SIR },
  { &hf_rnsap_minUL_SIR     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_SIR },
  { &hf_rnsap_maximumAllowedULTxPower, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MaximumAllowedULTxPower },
  { &hf_rnsap_maximumDLTxPower, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_Power },
  { &hf_rnsap_minimumDLTxPower, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_Power },
  { &hf_rnsap_uARFCNforNt   , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_UARFCN },
  { &hf_rnsap_cellParameterID, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_CellParameterID },
  { &hf_rnsap_sCTD_Indicator, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_SCTD_Indicator },
  { &hf_rnsap_pCCPCH_Power  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PCCPCH_Power },
  { &hf_rnsap_alphaValue    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_AlphaValue },
  { &hf_rnsap_ul_PhysCH_SF_Variation, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_PhysCH_SF_Variation },
  { &hf_rnsap_synchronisationConfiguration, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SynchronisationConfiguration },
  { &hf_rnsap_secondary_LCR_CCPCH_Info_TDD, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Secondary_LCR_CCPCH_Info_TDD },
  { &hf_rnsap_ul_LCR_CCTrCHInformation, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_UL_LCR_CCTrCHInformationList_RL_SetupRspTDD },
  { &hf_rnsap_dl_LCR_CCTrCHInformation, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DL_LCR_CCTrCHInformationList_RL_SetupRspTDD },
  { &hf_rnsap_dCH_InformationResponse1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DCH_InformationResponseList_RL_SetupRspTDD },
  { &hf_rnsap_dsch_LCR_InformationResponse, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DSCH_LCR_InformationResponse_RL_SetupRspTDD },
  { &hf_rnsap_usch_LCR_InformationResponse, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_USCH_LCR_InformationResponse_RL_SetupRspTDD },
  { &hf_rnsap_neighbouring_UMTS_CellInformation, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Neighbouring_UMTS_CellInformation },
  { &hf_rnsap_neighbouring_GSM_CellInformation, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Neighbouring_GSM_CellInformation },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_LCR_InformationResponse_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_LCR_InformationResponse_RL_SetupRspTDD, RL_LCR_InformationResponse_RL_SetupRspTDD_sequence);

  return offset;
}
static int dissect_id_RL_LCR_InformationResponse_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_LCR_InformationResponse_RL_SetupRspTDD(tvb, offset, actx, tree, hf_rnsap_id_RL_LCR_InformationResponse_RL_SetupRspTDD);
}



static int
dissect_rnsap_UL_DPCH_LCR_InformationList_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_ul_DPCH_LCR_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_DPCH_LCR_InformationList_RL_SetupRspTDD(tvb, offset, actx, tree, hf_rnsap_ul_DPCH_LCR_Information);
}


static const per_sequence_t UL_LCR_CCTrCHInformationItem_RL_SetupRspTDD_sequence[] = {
  { &hf_rnsap_cCTrCH_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CCTrCH_ID },
  { &hf_rnsap_ul_DPCH_LCR_Information, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_UL_DPCH_LCR_InformationList_RL_SetupRspTDD },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UL_LCR_CCTrCHInformationItem_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UL_LCR_CCTrCHInformationItem_RL_SetupRspTDD, UL_LCR_CCTrCHInformationItem_RL_SetupRspTDD_sequence);

  return offset;
}
static int dissect_UL_LCR_CCTrCHInformationListIE_RL_SetupRspTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_LCR_CCTrCHInformationItem_RL_SetupRspTDD(tvb, offset, actx, tree, hf_rnsap_UL_LCR_CCTrCHInformationListIE_RL_SetupRspTDD_item);
}


static const per_sequence_t UL_LCR_CCTrCHInformationListIE_RL_SetupRspTDD_sequence_of[1] = {
  { &hf_rnsap_UL_LCR_CCTrCHInformationListIE_RL_SetupRspTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_LCR_CCTrCHInformationItem_RL_SetupRspTDD },
};

static int
dissect_rnsap_UL_LCR_CCTrCHInformationListIE_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_UL_LCR_CCTrCHInformationListIE_RL_SetupRspTDD, UL_LCR_CCTrCHInformationListIE_RL_SetupRspTDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_UL_CCTrCH_LCR_InformationListIE_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_LCR_CCTrCHInformationListIE_RL_SetupRspTDD(tvb, offset, actx, tree, hf_rnsap_id_UL_CCTrCH_LCR_InformationListIE_RL_SetupRspTDD);
}



static int
dissect_rnsap_QPSK_UL_DPCH_TimeSlotFormatTDD_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 69U, NULL, TRUE);

  return offset;
}
static int dissect_qPSK1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_QPSK_UL_DPCH_TimeSlotFormatTDD_LCR(tvb, offset, actx, tree, hf_rnsap_qPSK1);
}



static int
dissect_rnsap_EightPSK_UL_DPCH_TimeSlotFormatTDD_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 24U, NULL, TRUE);

  return offset;
}
static int dissect_eightPSK1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_EightPSK_UL_DPCH_TimeSlotFormatTDD_LCR(tvb, offset, actx, tree, hf_rnsap_eightPSK1);
}


static const value_string rnsap_TDD_UL_DPCH_TimeSlotFormat_LCR_vals[] = {
  {   0, "qPSK" },
  {   1, "eightPSK" },
  { 0, NULL }
};

static const per_choice_t TDD_UL_DPCH_TimeSlotFormat_LCR_choice[] = {
  {   0, &hf_rnsap_qPSK1         , ASN1_EXTENSION_ROOT    , dissect_rnsap_QPSK_UL_DPCH_TimeSlotFormatTDD_LCR },
  {   1, &hf_rnsap_eightPSK1     , ASN1_EXTENSION_ROOT    , dissect_rnsap_EightPSK_UL_DPCH_TimeSlotFormatTDD_LCR },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_TDD_UL_DPCH_TimeSlotFormat_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_TDD_UL_DPCH_TimeSlotFormat_LCR, TDD_UL_DPCH_TimeSlotFormat_LCR_choice,
                                 NULL);

  return offset;
}
static int dissect_id_TDD_UL_DPCH_TimeSlotFormatModifyItem_LCR_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_UL_DPCH_TimeSlotFormat_LCR(tvb, offset, actx, tree, hf_rnsap_id_TDD_UL_DPCH_TimeSlotFormatModifyItem_LCR_RL_ReconfReadyTDD);
}
static int dissect_tdd_UL_DPCH_TimeSlotFormat_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_UL_DPCH_TimeSlotFormat_LCR(tvb, offset, actx, tree, hf_rnsap_tdd_UL_DPCH_TimeSlotFormat_LCR);
}


static const per_sequence_t TDD_UL_Code_LCR_InformationItem_sequence[] = {
  { &hf_rnsap_dPCH_ID       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DPCH_ID },
  { &hf_rnsap_tdd_ChannelisationCodeLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_ChannelisationCodeLCR },
  { &hf_rnsap_tdd_UL_DPCH_TimeSlotFormat_LCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_UL_DPCH_TimeSlotFormat_LCR },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_TDD_UL_Code_LCR_InformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_TDD_UL_Code_LCR_InformationItem, TDD_UL_Code_LCR_InformationItem_sequence);

  return offset;
}
static int dissect_TDD_UL_Code_LCR_Information_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_UL_Code_LCR_InformationItem(tvb, offset, actx, tree, hf_rnsap_TDD_UL_Code_LCR_Information_item);
}


static const per_sequence_t TDD_UL_Code_LCR_Information_sequence_of[1] = {
  { &hf_rnsap_TDD_UL_Code_LCR_Information_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_UL_Code_LCR_InformationItem },
};

static int
dissect_rnsap_TDD_UL_Code_LCR_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_TDD_UL_Code_LCR_Information, TDD_UL_Code_LCR_Information_sequence_of,
                                                  1, 240);

  return offset;
}
static int dissect_uL_Code_LCR_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_UL_Code_LCR_Information(tvb, offset, actx, tree, hf_rnsap_uL_Code_LCR_Information);
}
static int dissect_uL_Code_LCR_InformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_UL_Code_LCR_Information(tvb, offset, actx, tree, hf_rnsap_uL_Code_LCR_InformationList);
}


static const per_sequence_t UL_TimeslotLCR_InformationItem_sequence[] = {
  { &hf_rnsap_timeSlotLCR   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TimeSlotLCR },
  { &hf_rnsap_midambleShiftLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MidambleShiftLCR },
  { &hf_rnsap_tFCI_Presence , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TFCI_Presence },
  { &hf_rnsap_uL_Code_LCR_InformationList, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_UL_Code_LCR_Information },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UL_TimeslotLCR_InformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UL_TimeslotLCR_InformationItem, UL_TimeslotLCR_InformationItem_sequence);

  return offset;
}
static int dissect_UL_TimeslotLCR_Information_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_TimeslotLCR_InformationItem(tvb, offset, actx, tree, hf_rnsap_UL_TimeslotLCR_Information_item);
}


static const per_sequence_t UL_TimeslotLCR_Information_sequence_of[1] = {
  { &hf_rnsap_UL_TimeslotLCR_Information_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_TimeslotLCR_InformationItem },
};

static int
dissect_rnsap_UL_TimeslotLCR_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_UL_TimeslotLCR_Information, UL_TimeslotLCR_Information_sequence_of,
                                                  1, 6);

  return offset;
}
static int dissect_uL_TimeslotLCR_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_TimeslotLCR_Information(tvb, offset, actx, tree, hf_rnsap_uL_TimeslotLCR_Information);
}
static int dissect_uL_TimeslotLCR_Info(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_TimeslotLCR_Information(tvb, offset, actx, tree, hf_rnsap_uL_TimeslotLCR_Info);
}


static const per_sequence_t UL_DPCH_LCR_InformationItem_RL_SetupRspTDD_sequence[] = {
  { &hf_rnsap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RepetitionPeriod },
  { &hf_rnsap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RepetitionLength },
  { &hf_rnsap_tDD_DPCHOffset, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_DPCHOffset },
  { &hf_rnsap_uL_TimeslotLCR_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_TimeslotLCR_Information },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UL_DPCH_LCR_InformationItem_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UL_DPCH_LCR_InformationItem_RL_SetupRspTDD, UL_DPCH_LCR_InformationItem_RL_SetupRspTDD_sequence);

  return offset;
}
static int dissect_id_UL_DPCH_LCR_InformationItem_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_DPCH_LCR_InformationItem_RL_SetupRspTDD(tvb, offset, actx, tree, hf_rnsap_id_UL_DPCH_LCR_InformationItem_RL_SetupRspTDD);
}



static int
dissect_rnsap_DL_DPCH_LCR_InformationList_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_dl_DPCH_LCR_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_DPCH_LCR_InformationList_RL_SetupRspTDD(tvb, offset, actx, tree, hf_rnsap_dl_DPCH_LCR_Information);
}


static const per_sequence_t DL_CCTrCH_LCR_InformationItem_RL_SetupRspTDD_sequence[] = {
  { &hf_rnsap_cCTrCH_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CCTrCH_ID },
  { &hf_rnsap_dl_DPCH_LCR_Information, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DL_DPCH_LCR_InformationList_RL_SetupRspTDD },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_CCTrCH_LCR_InformationItem_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_CCTrCH_LCR_InformationItem_RL_SetupRspTDD, DL_CCTrCH_LCR_InformationItem_RL_SetupRspTDD_sequence);

  return offset;
}
static int dissect_DL_CCTrCH_LCR_InformationListIE_RL_SetupRspTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_CCTrCH_LCR_InformationItem_RL_SetupRspTDD(tvb, offset, actx, tree, hf_rnsap_DL_CCTrCH_LCR_InformationListIE_RL_SetupRspTDD_item);
}


static const per_sequence_t DL_CCTrCH_LCR_InformationListIE_RL_SetupRspTDD_sequence_of[1] = {
  { &hf_rnsap_DL_CCTrCH_LCR_InformationListIE_RL_SetupRspTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_CCTrCH_LCR_InformationItem_RL_SetupRspTDD },
};

static int
dissect_rnsap_DL_CCTrCH_LCR_InformationListIE_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DL_CCTrCH_LCR_InformationListIE_RL_SetupRspTDD, DL_CCTrCH_LCR_InformationListIE_RL_SetupRspTDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_DL_CCTrCH_LCR_InformationListIE_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_CCTrCH_LCR_InformationListIE_RL_SetupRspTDD(tvb, offset, actx, tree, hf_rnsap_id_DL_CCTrCH_LCR_InformationListIE_RL_SetupRspTDD);
}


static const per_sequence_t TDD_DL_Code_LCR_InformationItem_sequence[] = {
  { &hf_rnsap_dPCH_ID       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DPCH_ID },
  { &hf_rnsap_tdd_ChannelisationCodeLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_ChannelisationCodeLCR },
  { &hf_rnsap_tdd_DL_DPCH_TimeSlotFormat_LCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_DL_DPCH_TimeSlotFormat_LCR },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_TDD_DL_Code_LCR_InformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_TDD_DL_Code_LCR_InformationItem, TDD_DL_Code_LCR_InformationItem_sequence);

  return offset;
}
static int dissect_TDD_DL_Code_LCR_Information_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_DL_Code_LCR_InformationItem(tvb, offset, actx, tree, hf_rnsap_TDD_DL_Code_LCR_Information_item);
}


static const per_sequence_t TDD_DL_Code_LCR_Information_sequence_of[1] = {
  { &hf_rnsap_TDD_DL_Code_LCR_Information_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_DL_Code_LCR_InformationItem },
};

static int
dissect_rnsap_TDD_DL_Code_LCR_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_TDD_DL_Code_LCR_Information, TDD_DL_Code_LCR_Information_sequence_of,
                                                  1, 240);

  return offset;
}
static int dissect_dL_Code_LCR_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_DL_Code_LCR_Information(tvb, offset, actx, tree, hf_rnsap_dL_Code_LCR_Information);
}


static const per_sequence_t DL_TimeslotLCR_InformationItem_sequence[] = {
  { &hf_rnsap_timeSlotLCR   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TimeSlotLCR },
  { &hf_rnsap_midambleShiftLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MidambleShiftLCR },
  { &hf_rnsap_tFCI_Presence , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TFCI_Presence },
  { &hf_rnsap_dL_Code_LCR_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_DL_Code_LCR_Information },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_TimeslotLCR_InformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_TimeslotLCR_InformationItem, DL_TimeslotLCR_InformationItem_sequence);

  return offset;
}
static int dissect_DL_TimeslotLCR_Information_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_TimeslotLCR_InformationItem(tvb, offset, actx, tree, hf_rnsap_DL_TimeslotLCR_Information_item);
}


static const per_sequence_t DL_TimeslotLCR_Information_sequence_of[1] = {
  { &hf_rnsap_DL_TimeslotLCR_Information_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_TimeslotLCR_InformationItem },
};

static int
dissect_rnsap_DL_TimeslotLCR_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DL_TimeslotLCR_Information, DL_TimeslotLCR_Information_sequence_of,
                                                  1, 6);

  return offset;
}
static int dissect_dL_Timeslot_LCR_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_TimeslotLCR_Information(tvb, offset, actx, tree, hf_rnsap_dL_Timeslot_LCR_Information);
}
static int dissect_dL_TimeslotLCR_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_TimeslotLCR_Information(tvb, offset, actx, tree, hf_rnsap_dL_TimeslotLCR_Information);
}
static int dissect_dL_TimeslotLCR_Info(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_TimeslotLCR_Information(tvb, offset, actx, tree, hf_rnsap_dL_TimeslotLCR_Info);
}


static const value_string rnsap_TSTD_Indicator_vals[] = {
  {   0, "active" },
  {   1, "inactive" },
  { 0, NULL }
};


static int
dissect_rnsap_TSTD_Indicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_tSTD_Indicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TSTD_Indicator(tvb, offset, actx, tree, hf_rnsap_tSTD_Indicator);
}


static const per_sequence_t DL_DPCH_LCR_InformationItem_RL_SetupRspTDD_sequence[] = {
  { &hf_rnsap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RepetitionPeriod },
  { &hf_rnsap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RepetitionLength },
  { &hf_rnsap_tDD_DPCHOffset, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_DPCHOffset },
  { &hf_rnsap_dL_Timeslot_LCR_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_TimeslotLCR_Information },
  { &hf_rnsap_tSTD_Indicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TSTD_Indicator },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_DPCH_LCR_InformationItem_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_DPCH_LCR_InformationItem_RL_SetupRspTDD, DL_DPCH_LCR_InformationItem_RL_SetupRspTDD_sequence);

  return offset;
}
static int dissect_id_DL_DPCH_LCR_InformationItem_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_DPCH_LCR_InformationItem_RL_SetupRspTDD(tvb, offset, actx, tree, hf_rnsap_id_DL_DPCH_LCR_InformationItem_RL_SetupRspTDD);
}


static const per_sequence_t DSCH_LCR_InformationItem_RL_SetupRspTDD_sequence[] = {
  { &hf_rnsap_dsch_ID       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DSCH_ID },
  { &hf_rnsap_dSCH_FlowControlInformation, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DSCH_FlowControlInformation },
  { &hf_rnsap_bindingID     , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_BindingID },
  { &hf_rnsap_transportLayerAddress, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TransportLayerAddress },
  { &hf_rnsap_transportFormatManagement, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TransportFormatManagement },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DSCH_LCR_InformationItem_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DSCH_LCR_InformationItem_RL_SetupRspTDD, DSCH_LCR_InformationItem_RL_SetupRspTDD_sequence);

  return offset;
}
static int dissect_DSCH_LCR_InformationListIEs_RL_SetupRspTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DSCH_LCR_InformationItem_RL_SetupRspTDD(tvb, offset, actx, tree, hf_rnsap_DSCH_LCR_InformationListIEs_RL_SetupRspTDD_item);
}


static const per_sequence_t DSCH_LCR_InformationListIEs_RL_SetupRspTDD_sequence_of[1] = {
  { &hf_rnsap_DSCH_LCR_InformationListIEs_RL_SetupRspTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_DSCH_LCR_InformationItem_RL_SetupRspTDD },
};

static int
dissect_rnsap_DSCH_LCR_InformationListIEs_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DSCH_LCR_InformationListIEs_RL_SetupRspTDD, DSCH_LCR_InformationListIEs_RL_SetupRspTDD_sequence_of,
                                                  0, 10);

  return offset;
}
static int dissect_id_DSCH_LCR_InformationListIEs_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DSCH_LCR_InformationListIEs_RL_SetupRspTDD(tvb, offset, actx, tree, hf_rnsap_id_DSCH_LCR_InformationListIEs_RL_SetupRspTDD);
}


static const per_sequence_t USCH_LCR_InformationItem_RL_SetupRspTDD_sequence[] = {
  { &hf_rnsap_usch_ID       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_USCH_ID },
  { &hf_rnsap_bindingID     , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_BindingID },
  { &hf_rnsap_transportLayerAddress, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TransportLayerAddress },
  { &hf_rnsap_transportFormatManagement, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TransportFormatManagement },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_USCH_LCR_InformationItem_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_USCH_LCR_InformationItem_RL_SetupRspTDD, USCH_LCR_InformationItem_RL_SetupRspTDD_sequence);

  return offset;
}
static int dissect_USCH_LCR_InformationListIEs_RL_SetupRspTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_USCH_LCR_InformationItem_RL_SetupRspTDD(tvb, offset, actx, tree, hf_rnsap_USCH_LCR_InformationListIEs_RL_SetupRspTDD_item);
}


static const per_sequence_t USCH_LCR_InformationListIEs_RL_SetupRspTDD_sequence_of[1] = {
  { &hf_rnsap_USCH_LCR_InformationListIEs_RL_SetupRspTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_USCH_LCR_InformationItem_RL_SetupRspTDD },
};

static int
dissect_rnsap_USCH_LCR_InformationListIEs_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_USCH_LCR_InformationListIEs_RL_SetupRspTDD, USCH_LCR_InformationListIEs_RL_SetupRspTDD_sequence_of,
                                                  0, 10);

  return offset;
}
static int dissect_id_USCH_LCR_InformationListIEs_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_USCH_LCR_InformationListIEs_RL_SetupRspTDD(tvb, offset, actx, tree, hf_rnsap_id_USCH_LCR_InformationListIEs_RL_SetupRspTDD);
}



static int
dissect_rnsap_UL_CCTrCH_LCR_InformationList_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_ul_CCTrCH_LCR_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_CCTrCH_LCR_InformationList_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_rnsap_ul_CCTrCH_LCR_Information);
}



static int
dissect_rnsap_DL_CCTrCH_LCR_InformationList_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_dl_CCTrCH_LCR_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_CCTrCH_LCR_InformationList_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_rnsap_dl_CCTrCH_LCR_Information);
}



static int
dissect_rnsap_DCH_InformationResponseList_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_dCH_InformationResponse2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DCH_InformationResponseList_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_rnsap_dCH_InformationResponse2);
}



static int
dissect_rnsap_DSCH_LCR_InformationResponse_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_dsch_LCR_InformationResponse1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DSCH_LCR_InformationResponse_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_rnsap_dsch_LCR_InformationResponse1);
}



static int
dissect_rnsap_USCH_LCR_InformationResponse_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_usch_LCR_InformationResponse1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_USCH_LCR_InformationResponse_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_rnsap_usch_LCR_InformationResponse1);
}


static const per_sequence_t RL_LCR_InformationResponse_RL_AdditionRspTDD_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_uRA_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_URA_Information },
  { &hf_rnsap_sAI           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SAI },
  { &hf_rnsap_gA_Cell       , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_GA_Cell },
  { &hf_rnsap_gA_AccessPointPosition, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_GA_AccessPointPosition },
  { &hf_rnsap_ul_TimeSlot_ISCP_LCR_Info, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_TimeSlot_ISCP_LCR_Info },
  { &hf_rnsap_maxUL_SIR     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_SIR },
  { &hf_rnsap_minUL_SIR     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_SIR },
  { &hf_rnsap_pCCPCH_Power  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PCCPCH_Power },
  { &hf_rnsap_maximumAllowedULTxPower, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MaximumAllowedULTxPower },
  { &hf_rnsap_maximumDLTxPower, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_Power },
  { &hf_rnsap_minimumDLTxPower, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_Power },
  { &hf_rnsap_alphaValue    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_AlphaValue },
  { &hf_rnsap_ul_PhysCH_SF_Variation, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_PhysCH_SF_Variation },
  { &hf_rnsap_synchronisationConfiguration, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SynchronisationConfiguration },
  { &hf_rnsap_secondary_LCR_CCPCH_Info_TDD, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Secondary_LCR_CCPCH_Info_TDD },
  { &hf_rnsap_ul_CCTrCH_LCR_Information, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_UL_CCTrCH_LCR_InformationList_RL_AdditionRspTDD },
  { &hf_rnsap_dl_CCTrCH_LCR_Information, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DL_CCTrCH_LCR_InformationList_RL_AdditionRspTDD },
  { &hf_rnsap_dCH_InformationResponse2, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DCH_InformationResponseList_RL_AdditionRspTDD },
  { &hf_rnsap_dsch_LCR_InformationResponse1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DSCH_LCR_InformationResponse_RL_AdditionRspTDD },
  { &hf_rnsap_usch_LCR_InformationResponse1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_USCH_LCR_InformationResponse_RL_AdditionRspTDD },
  { &hf_rnsap_neighbouring_UMTS_CellInformation, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Neighbouring_UMTS_CellInformation },
  { &hf_rnsap_neighbouring_GSM_CellInformation, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Neighbouring_GSM_CellInformation },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_LCR_InformationResponse_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_LCR_InformationResponse_RL_AdditionRspTDD, RL_LCR_InformationResponse_RL_AdditionRspTDD_sequence);

  return offset;
}
static int dissect_id_RL_LCR_InformationResponse_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_LCR_InformationResponse_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_rnsap_id_RL_LCR_InformationResponse_RL_AdditionRspTDD);
}



static int
dissect_rnsap_UL_DPCH_LCR_InformationList_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_ul_DPCH_LCR_Information1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_DPCH_LCR_InformationList_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_rnsap_ul_DPCH_LCR_Information1);
}


static const per_sequence_t UL_CCTrCH_LCR_InformationItem_RL_AdditionRspTDD_sequence[] = {
  { &hf_rnsap_cCTrCH_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CCTrCH_ID },
  { &hf_rnsap_ul_DPCH_LCR_Information1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_UL_DPCH_LCR_InformationList_RL_AdditionRspTDD },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UL_CCTrCH_LCR_InformationItem_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UL_CCTrCH_LCR_InformationItem_RL_AdditionRspTDD, UL_CCTrCH_LCR_InformationItem_RL_AdditionRspTDD_sequence);

  return offset;
}
static int dissect_UL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_CCTrCH_LCR_InformationItem_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_rnsap_UL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD_item);
}


static const per_sequence_t UL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD_sequence_of[1] = {
  { &hf_rnsap_UL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_CCTrCH_LCR_InformationItem_RL_AdditionRspTDD },
};

static int
dissect_rnsap_UL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_UL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD, UL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_UL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_rnsap_id_UL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD);
}


static const per_sequence_t UL_DPCH_LCR_InformationItem_RL_AdditionRspTDD_sequence[] = {
  { &hf_rnsap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RepetitionPeriod },
  { &hf_rnsap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RepetitionLength },
  { &hf_rnsap_tDD_DPCHOffset, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_DPCHOffset },
  { &hf_rnsap_uL_TimeslotLCR_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_TimeslotLCR_Information },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UL_DPCH_LCR_InformationItem_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UL_DPCH_LCR_InformationItem_RL_AdditionRspTDD, UL_DPCH_LCR_InformationItem_RL_AdditionRspTDD_sequence);

  return offset;
}
static int dissect_id_UL_DPCH_LCR_InformationItem_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_DPCH_LCR_InformationItem_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_rnsap_id_UL_DPCH_LCR_InformationItem_RL_AdditionRspTDD);
}



static int
dissect_rnsap_DL_DPCH_LCR_InformationList_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_dl_DPCH_LCR_Information1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_DPCH_LCR_InformationList_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_rnsap_dl_DPCH_LCR_Information1);
}


static const per_sequence_t DL_CCTrCH_LCR_InformationItem_RL_AdditionRspTDD_sequence[] = {
  { &hf_rnsap_cCTrCH_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CCTrCH_ID },
  { &hf_rnsap_dl_DPCH_LCR_Information1, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DL_DPCH_LCR_InformationList_RL_AdditionRspTDD },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_CCTrCH_LCR_InformationItem_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_CCTrCH_LCR_InformationItem_RL_AdditionRspTDD, DL_CCTrCH_LCR_InformationItem_RL_AdditionRspTDD_sequence);

  return offset;
}
static int dissect_DL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_CCTrCH_LCR_InformationItem_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_rnsap_DL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD_item);
}


static const per_sequence_t DL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD_sequence_of[1] = {
  { &hf_rnsap_DL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_CCTrCH_LCR_InformationItem_RL_AdditionRspTDD },
};

static int
dissect_rnsap_DL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD, DL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_DL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_rnsap_id_DL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD);
}


static const per_sequence_t DL_DPCH_LCR_InformationItem_RL_AdditionRspTDD_sequence[] = {
  { &hf_rnsap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RepetitionPeriod },
  { &hf_rnsap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RepetitionLength },
  { &hf_rnsap_tDD_DPCHOffset, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_DPCHOffset },
  { &hf_rnsap_dL_TimeslotLCR_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_TimeslotLCR_Information },
  { &hf_rnsap_tSTD_Indicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TSTD_Indicator },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_DPCH_LCR_InformationItem_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_DPCH_LCR_InformationItem_RL_AdditionRspTDD, DL_DPCH_LCR_InformationItem_RL_AdditionRspTDD_sequence);

  return offset;
}
static int dissect_id_DL_DPCH_LCR_InformationItem_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_DPCH_LCR_InformationItem_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_rnsap_id_DL_DPCH_LCR_InformationItem_RL_AdditionRspTDD);
}


static const per_sequence_t DSCH_LCR_InformationItem_RL_AdditionRspTDD_sequence[] = {
  { &hf_rnsap_dsch_ID       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DSCH_ID },
  { &hf_rnsap_dSCH_FlowControlInformation, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DSCH_FlowControlInformation },
  { &hf_rnsap_bindingID     , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_BindingID },
  { &hf_rnsap_transportLayerAddress, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TransportLayerAddress },
  { &hf_rnsap_transportFormatManagement, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TransportFormatManagement },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DSCH_LCR_InformationItem_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DSCH_LCR_InformationItem_RL_AdditionRspTDD, DSCH_LCR_InformationItem_RL_AdditionRspTDD_sequence);

  return offset;
}
static int dissect_DSCH_LCR_InformationListIEs_RL_AdditionRspTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DSCH_LCR_InformationItem_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_rnsap_DSCH_LCR_InformationListIEs_RL_AdditionRspTDD_item);
}


static const per_sequence_t DSCH_LCR_InformationListIEs_RL_AdditionRspTDD_sequence_of[1] = {
  { &hf_rnsap_DSCH_LCR_InformationListIEs_RL_AdditionRspTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_DSCH_LCR_InformationItem_RL_AdditionRspTDD },
};

static int
dissect_rnsap_DSCH_LCR_InformationListIEs_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DSCH_LCR_InformationListIEs_RL_AdditionRspTDD, DSCH_LCR_InformationListIEs_RL_AdditionRspTDD_sequence_of,
                                                  0, 10);

  return offset;
}
static int dissect_id_DSCH_LCR_InformationListIEs_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DSCH_LCR_InformationListIEs_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_rnsap_id_DSCH_LCR_InformationListIEs_RL_AdditionRspTDD);
}


static const per_sequence_t USCH_LCR_InformationItem_RL_AdditionRspTDD_sequence[] = {
  { &hf_rnsap_usch_ID       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_USCH_ID },
  { &hf_rnsap_transportFormatManagement, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TransportFormatManagement },
  { &hf_rnsap_diversityIndication4, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DiversityIndication_RL_AdditionRspTDD2 },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_USCH_LCR_InformationItem_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_USCH_LCR_InformationItem_RL_AdditionRspTDD, USCH_LCR_InformationItem_RL_AdditionRspTDD_sequence);

  return offset;
}
static int dissect_USCH_LCR_InformationListIEs_RL_AdditionRspTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_USCH_LCR_InformationItem_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_rnsap_USCH_LCR_InformationListIEs_RL_AdditionRspTDD_item);
}


static const per_sequence_t USCH_LCR_InformationListIEs_RL_AdditionRspTDD_sequence_of[1] = {
  { &hf_rnsap_USCH_LCR_InformationListIEs_RL_AdditionRspTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_USCH_LCR_InformationItem_RL_AdditionRspTDD },
};

static int
dissect_rnsap_USCH_LCR_InformationListIEs_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_USCH_LCR_InformationListIEs_RL_AdditionRspTDD, USCH_LCR_InformationListIEs_RL_AdditionRspTDD_sequence_of,
                                                  0, 10);

  return offset;
}
static int dissect_id_USCH_LCR_InformationListIEs_RL_AdditionRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_USCH_LCR_InformationListIEs_RL_AdditionRspTDD(tvb, offset, actx, tree, hf_rnsap_id_USCH_LCR_InformationListIEs_RL_AdditionRspTDD);
}


static const per_sequence_t UL_DPCH_LCR_InformationAddList_RL_ReconfReadyTDD_sequence[] = {
  { &hf_rnsap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RepetitionPeriod },
  { &hf_rnsap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RepetitionLength },
  { &hf_rnsap_tDD_DPCHOffset, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_DPCHOffset },
  { &hf_rnsap_uL_TimeslotLCR_Info, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_TimeslotLCR_Information },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UL_DPCH_LCR_InformationAddList_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UL_DPCH_LCR_InformationAddList_RL_ReconfReadyTDD, UL_DPCH_LCR_InformationAddList_RL_ReconfReadyTDD_sequence);

  return offset;
}
static int dissect_id_UL_DPCH_LCR_InformationAddListIE_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_DPCH_LCR_InformationAddList_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_id_UL_DPCH_LCR_InformationAddListIE_RL_ReconfReadyTDD);
}


static const per_sequence_t TDD_UL_Code_LCR_InformationModifyItem_RL_ReconfReadyTDD_sequence[] = {
  { &hf_rnsap_dPCH_ID       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DPCH_ID },
  { &hf_rnsap_tDD_ChannelisationCodeLCR, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TDD_ChannelisationCodeLCR },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_TDD_UL_Code_LCR_InformationModifyItem_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_TDD_UL_Code_LCR_InformationModifyItem_RL_ReconfReadyTDD, TDD_UL_Code_LCR_InformationModifyItem_RL_ReconfReadyTDD_sequence);

  return offset;
}
static int dissect_TDD_UL_Code_LCR_InformationModifyList_RL_ReconfReadyTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_UL_Code_LCR_InformationModifyItem_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_TDD_UL_Code_LCR_InformationModifyList_RL_ReconfReadyTDD_item);
}


static const per_sequence_t TDD_UL_Code_LCR_InformationModifyList_RL_ReconfReadyTDD_sequence_of[1] = {
  { &hf_rnsap_TDD_UL_Code_LCR_InformationModifyList_RL_ReconfReadyTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_UL_Code_LCR_InformationModifyItem_RL_ReconfReadyTDD },
};

static int
dissect_rnsap_TDD_UL_Code_LCR_InformationModifyList_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_TDD_UL_Code_LCR_InformationModifyList_RL_ReconfReadyTDD, TDD_UL_Code_LCR_InformationModifyList_RL_ReconfReadyTDD_sequence_of,
                                                  1, 240);

  return offset;
}
static int dissect_tDD_uL_Code_LCR_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_UL_Code_LCR_InformationModifyList_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_tDD_uL_Code_LCR_Information);
}


static const per_sequence_t UL_TimeslotLCR_InformationModifyItem_RL_ReconfReadyTDD_sequence[] = {
  { &hf_rnsap_timeSlotLCR   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TimeSlotLCR },
  { &hf_rnsap_midambleShiftLCR, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_MidambleShiftLCR },
  { &hf_rnsap_tFCI_Presence , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TFCI_Presence },
  { &hf_rnsap_tDD_uL_Code_LCR_Information, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TDD_UL_Code_LCR_InformationModifyList_RL_ReconfReadyTDD },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UL_TimeslotLCR_InformationModifyItem_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UL_TimeslotLCR_InformationModifyItem_RL_ReconfReadyTDD, UL_TimeslotLCR_InformationModifyItem_RL_ReconfReadyTDD_sequence);

  return offset;
}
static int dissect_UL_TimeslotLCR_InformationModifyList_RL_ReconfReadyTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_TimeslotLCR_InformationModifyItem_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_UL_TimeslotLCR_InformationModifyList_RL_ReconfReadyTDD_item);
}


static const per_sequence_t UL_TimeslotLCR_InformationModifyList_RL_ReconfReadyTDD_sequence_of[1] = {
  { &hf_rnsap_UL_TimeslotLCR_InformationModifyList_RL_ReconfReadyTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_TimeslotLCR_InformationModifyItem_RL_ReconfReadyTDD },
};

static int
dissect_rnsap_UL_TimeslotLCR_InformationModifyList_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_UL_TimeslotLCR_InformationModifyList_RL_ReconfReadyTDD, UL_TimeslotLCR_InformationModifyList_RL_ReconfReadyTDD_sequence_of,
                                                  1, 6);

  return offset;
}
static int dissect_id_UL_Timeslot_LCR_InformationModifyList_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_TimeslotLCR_InformationModifyList_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_id_UL_Timeslot_LCR_InformationModifyList_RL_ReconfReadyTDD);
}


static const per_sequence_t DL_DPCH_LCR_InformationAddList_RL_ReconfReadyTDD_sequence[] = {
  { &hf_rnsap_repetitionPeriod, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RepetitionPeriod },
  { &hf_rnsap_repetitionLength, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RepetitionLength },
  { &hf_rnsap_tDD_DPCHOffset, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_DPCHOffset },
  { &hf_rnsap_dL_TimeslotLCR_Info, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_TimeslotLCR_Information },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_DPCH_LCR_InformationAddList_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_DPCH_LCR_InformationAddList_RL_ReconfReadyTDD, DL_DPCH_LCR_InformationAddList_RL_ReconfReadyTDD_sequence);

  return offset;
}
static int dissect_id_DL_DPCH_LCR_InformationAddListIE_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_DPCH_LCR_InformationAddList_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_id_DL_DPCH_LCR_InformationAddListIE_RL_ReconfReadyTDD);
}


static const per_sequence_t TDD_DL_Code_LCR_InformationModifyItem_RL_ReconfReadyTDD_sequence[] = {
  { &hf_rnsap_dPCH_ID       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DPCH_ID },
  { &hf_rnsap_tDD_ChannelisationCodeLCR, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TDD_ChannelisationCodeLCR },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_TDD_DL_Code_LCR_InformationModifyItem_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_TDD_DL_Code_LCR_InformationModifyItem_RL_ReconfReadyTDD, TDD_DL_Code_LCR_InformationModifyItem_RL_ReconfReadyTDD_sequence);

  return offset;
}
static int dissect_TDD_DL_Code_LCR_InformationModifyList_RL_ReconfReadyTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_DL_Code_LCR_InformationModifyItem_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_TDD_DL_Code_LCR_InformationModifyList_RL_ReconfReadyTDD_item);
}


static const per_sequence_t TDD_DL_Code_LCR_InformationModifyList_RL_ReconfReadyTDD_sequence_of[1] = {
  { &hf_rnsap_TDD_DL_Code_LCR_InformationModifyList_RL_ReconfReadyTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_DL_Code_LCR_InformationModifyItem_RL_ReconfReadyTDD },
};

static int
dissect_rnsap_TDD_DL_Code_LCR_InformationModifyList_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_TDD_DL_Code_LCR_InformationModifyList_RL_ReconfReadyTDD, TDD_DL_Code_LCR_InformationModifyList_RL_ReconfReadyTDD_sequence_of,
                                                  1, 240);

  return offset;
}
static int dissect_tDD_dL_Code_LCR_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_DL_Code_LCR_InformationModifyList_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_tDD_dL_Code_LCR_Information);
}


static const per_sequence_t DL_TimeslotLCR_InformationModifyItem_RL_ReconfReadyTDD_sequence[] = {
  { &hf_rnsap_timeSlotLCR   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TimeSlotLCR },
  { &hf_rnsap_midambleShiftLCR, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_MidambleShiftLCR },
  { &hf_rnsap_tFCI_Presence , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TFCI_Presence },
  { &hf_rnsap_tDD_dL_Code_LCR_Information, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TDD_DL_Code_LCR_InformationModifyList_RL_ReconfReadyTDD },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_TimeslotLCR_InformationModifyItem_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_TimeslotLCR_InformationModifyItem_RL_ReconfReadyTDD, DL_TimeslotLCR_InformationModifyItem_RL_ReconfReadyTDD_sequence);

  return offset;
}
static int dissect_DL_TimeslotLCR_InformationModifyList_RL_ReconfReadyTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_TimeslotLCR_InformationModifyItem_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_DL_TimeslotLCR_InformationModifyList_RL_ReconfReadyTDD_item);
}


static const per_sequence_t DL_TimeslotLCR_InformationModifyList_RL_ReconfReadyTDD_sequence_of[1] = {
  { &hf_rnsap_DL_TimeslotLCR_InformationModifyList_RL_ReconfReadyTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_TimeslotLCR_InformationModifyItem_RL_ReconfReadyTDD },
};

static int
dissect_rnsap_DL_TimeslotLCR_InformationModifyList_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DL_TimeslotLCR_InformationModifyList_RL_ReconfReadyTDD, DL_TimeslotLCR_InformationModifyList_RL_ReconfReadyTDD_sequence_of,
                                                  1, 6);

  return offset;
}
static int dissect_id_DL_Timeslot_LCR_InformationModifyList_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_TimeslotLCR_InformationModifyList_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_id_DL_Timeslot_LCR_InformationModifyList_RL_ReconfReadyTDD);
}


static const per_sequence_t UL_TimeslotLCR_InformationItem_PhyChReconfRqstTDD_sequence[] = {
  { &hf_rnsap_timeSlotLCR   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TimeSlotLCR },
  { &hf_rnsap_midambleShiftLCR, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_MidambleShiftLCR },
  { &hf_rnsap_tFCI_Presence , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TFCI_Presence },
  { &hf_rnsap_uL_Code_LCR_Information, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TDD_UL_Code_LCR_Information },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UL_TimeslotLCR_InformationItem_PhyChReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UL_TimeslotLCR_InformationItem_PhyChReconfRqstTDD, UL_TimeslotLCR_InformationItem_PhyChReconfRqstTDD_sequence);

  return offset;
}
static int dissect_UL_TimeslotLCR_InformationList_PhyChReconfRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_TimeslotLCR_InformationItem_PhyChReconfRqstTDD(tvb, offset, actx, tree, hf_rnsap_UL_TimeslotLCR_InformationList_PhyChReconfRqstTDD_item);
}


static const per_sequence_t UL_TimeslotLCR_InformationList_PhyChReconfRqstTDD_sequence_of[1] = {
  { &hf_rnsap_UL_TimeslotLCR_InformationList_PhyChReconfRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_TimeslotLCR_InformationItem_PhyChReconfRqstTDD },
};

static int
dissect_rnsap_UL_TimeslotLCR_InformationList_PhyChReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_UL_TimeslotLCR_InformationList_PhyChReconfRqstTDD, UL_TimeslotLCR_InformationList_PhyChReconfRqstTDD_sequence_of,
                                                  1, 6);

  return offset;
}
static int dissect_id_UL_Timeslot_LCR_InformationList_PhyChReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_TimeslotLCR_InformationList_PhyChReconfRqstTDD(tvb, offset, actx, tree, hf_rnsap_id_UL_Timeslot_LCR_InformationList_PhyChReconfRqstTDD);
}


static const per_sequence_t DL_TimeslotLCR_InformationItem_PhyChReconfRqstTDD_sequence[] = {
  { &hf_rnsap_timeSlotLCR   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TimeSlotLCR },
  { &hf_rnsap_midambleShiftLCR, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_MidambleShiftLCR },
  { &hf_rnsap_tFCI_Presence , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TFCI_Presence },
  { &hf_rnsap_dL_Code_LCR_Information, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TDD_DL_Code_LCR_Information },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_TimeslotLCR_InformationItem_PhyChReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_TimeslotLCR_InformationItem_PhyChReconfRqstTDD, DL_TimeslotLCR_InformationItem_PhyChReconfRqstTDD_sequence);

  return offset;
}
static int dissect_DL_TimeslotLCR_InformationList_PhyChReconfRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_TimeslotLCR_InformationItem_PhyChReconfRqstTDD(tvb, offset, actx, tree, hf_rnsap_DL_TimeslotLCR_InformationList_PhyChReconfRqstTDD_item);
}


static const per_sequence_t DL_TimeslotLCR_InformationList_PhyChReconfRqstTDD_sequence_of[1] = {
  { &hf_rnsap_DL_TimeslotLCR_InformationList_PhyChReconfRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_TimeslotLCR_InformationItem_PhyChReconfRqstTDD },
};

static int
dissect_rnsap_DL_TimeslotLCR_InformationList_PhyChReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DL_TimeslotLCR_InformationList_PhyChReconfRqstTDD, DL_TimeslotLCR_InformationList_PhyChReconfRqstTDD_sequence_of,
                                                  1, 6);

  return offset;
}
static int dissect_id_DL_Timeslot_LCR_InformationList_PhyChReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_TimeslotLCR_InformationList_PhyChReconfRqstTDD(tvb, offset, actx, tree, hf_rnsap_id_DL_Timeslot_LCR_InformationList_PhyChReconfRqstTDD);
}


static const value_string rnsap_TSTD_Support_Indicator_vals[] = {
  {   0, "tSTD-supported" },
  {   1, "tSTD-not-supported" },
  { 0, NULL }
};


static int
dissect_rnsap_TSTD_Support_Indicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_TSTD_Support_Indicator_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TSTD_Support_Indicator(tvb, offset, actx, tree, hf_rnsap_id_TSTD_Support_Indicator_RL_SetupRqstTDD);
}



static int
dissect_rnsap_Load_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 100U, NULL, FALSE);

  return offset;
}
static int dissect_id_Load_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Load_Value(tvb, offset, actx, tree, hf_rnsap_id_Load_Value);
}



static int
dissect_rnsap_Load_Value_IncrDecrThres(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 100U, NULL, FALSE);

  return offset;
}
static int dissect_id_Load_Value_IncrDecrThres(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Load_Value_IncrDecrThres(tvb, offset, actx, tree, hf_rnsap_id_Load_Value_IncrDecrThres);
}


static const per_sequence_t OnModification_sequence[] = {
  { &hf_rnsap_measurementThreshold, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MeasurementThreshold },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_OnModification(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_OnModification, OnModification_sequence);

  return offset;
}
static int dissect_id_OnModification(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_OnModification(tvb, offset, actx, tree, hf_rnsap_id_OnModification);
}



static int
dissect_rnsap_Received_Total_Wideband_Power_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_Received_total_wide_band_power(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_id_Received_Total_Wideband_Power_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Received_Total_Wideband_Power_Value(tvb, offset, actx, tree, hf_rnsap_id_Received_Total_Wideband_Power_Value);
}



static int
dissect_rnsap_SFNSFNChangeLimit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 256U, NULL, FALSE);

  return offset;
}
static int dissect_sFNSFNChangeLimit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SFNSFNChangeLimit(tvb, offset, actx, tree, hf_rnsap_sFNSFNChangeLimit);
}



static int
dissect_rnsap_PredictedSFNSFNDeviationLimit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 256U, NULL, FALSE);

  return offset;
}
static int dissect_predictedSFNSFNDeviationLimit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PredictedSFNSFNDeviationLimit(tvb, offset, actx, tree, hf_rnsap_predictedSFNSFNDeviationLimit);
}


static const per_sequence_t SFNSFNMeasurementThresholdInformation_sequence[] = {
  { &hf_rnsap_sFNSFNChangeLimit, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_SFNSFNChangeLimit },
  { &hf_rnsap_predictedSFNSFNDeviationLimit, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_PredictedSFNSFNDeviationLimit },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_SFNSFNMeasurementThresholdInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_SFNSFNMeasurementThresholdInformation, SFNSFNMeasurementThresholdInformation_sequence);

  return offset;
}
static int dissect_id_Received_Total_Wideband_Power_Value_IncrDecrThres(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SFNSFNMeasurementThresholdInformation(tvb, offset, actx, tree, hf_rnsap_id_Received_Total_Wideband_Power_Value_IncrDecrThres);
}
static int dissect_id_SFNSFNMeasurementThresholdInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SFNSFNMeasurementThresholdInformation(tvb, offset, actx, tree, hf_rnsap_id_SFNSFNMeasurementThresholdInformation);
}



static int
dissect_rnsap_Transmitted_Carrier_Power_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 100U, NULL, FALSE);

  return offset;
}
static int dissect_id_Transmitted_Carrier_Power_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Transmitted_Carrier_Power_Value(tvb, offset, actx, tree, hf_rnsap_id_Transmitted_Carrier_Power_Value);
}



static int
dissect_rnsap_Transmitted_Carrier_Power_Value_IncrDecrThres(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 100U, NULL, FALSE);

  return offset;
}
static int dissect_id_Transmitted_Carrier_Power_Value_IncrDecrThres(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Transmitted_Carrier_Power_Value_IncrDecrThres(tvb, offset, actx, tree, hf_rnsap_id_Transmitted_Carrier_Power_Value_IncrDecrThres);
}



static int
dissect_rnsap_UL_Timeslot_ISCP_Value_IncrDecrThres(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 126U, NULL, FALSE);

  return offset;
}
static int dissect_id_UL_Timeslot_ISCP_Value_IncrDecrThres(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_Timeslot_ISCP_Value_IncrDecrThres(tvb, offset, actx, tree, hf_rnsap_id_UL_Timeslot_ISCP_Value_IncrDecrThres);
}



static int
dissect_rnsap_Rx_Timing_Deviation_Value_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 511U, NULL, FALSE);

  return offset;
}
static int dissect_id_Rx_Timing_Deviation_Value_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Rx_Timing_Deviation_Value_LCR(tvb, offset, actx, tree, hf_rnsap_id_Rx_Timing_Deviation_Value_LCR);
}


static const value_string rnsap_DPC_Mode_Change_SupportIndicator_vals[] = {
  {   0, "dPC-ModeChangeSupported" },
  { 0, NULL }
};


static int
dissect_rnsap_DPC_Mode_Change_SupportIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_DPC_Mode_Change_SupportIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DPC_Mode_Change_SupportIndicator(tvb, offset, actx, tree, hf_rnsap_id_DPC_Mode_Change_SupportIndicator);
}



static int
dissect_rnsap_DSCH_RNTI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 65535U, NULL, FALSE);

  return offset;
}
static int dissect_id_DSCH_RNTI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DSCH_RNTI(tvb, offset, actx, tree, hf_rnsap_id_DSCH_RNTI);
}


static const per_sequence_t DL_ReferencePowerInformationItem_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_dl_Reference_Power, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_Power },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_ReferencePowerInformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_ReferencePowerInformationItem, DL_ReferencePowerInformationItem_sequence);

  return offset;
}
static int dissect_DL_ReferencePowerInformationList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_ReferencePowerInformationItem(tvb, offset, actx, tree, hf_rnsap_DL_ReferencePowerInformationList_item);
}


static const per_sequence_t DL_ReferencePowerInformationList_sequence_of[1] = {
  { &hf_rnsap_DL_ReferencePowerInformationList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_ReferencePowerInformationItem },
};

static int
dissect_rnsap_DL_ReferencePowerInformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DL_ReferencePowerInformationList, DL_ReferencePowerInformationList_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_dLReferencePowerList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_ReferencePowerInformationList(tvb, offset, actx, tree, hf_rnsap_dLReferencePowerList);
}
static int dissect_individual_DL_ReferencePowerInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_ReferencePowerInformationList(tvb, offset, actx, tree, hf_rnsap_individual_DL_ReferencePowerInformation);
}


static const per_sequence_t DL_PowerBalancing_Information_sequence[] = {
  { &hf_rnsap_powerAdjustmentType, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PowerAdjustmentType },
  { &hf_rnsap_dLReferencePower, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DL_Power },
  { &hf_rnsap_dLReferencePowerList, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DL_ReferencePowerInformationList },
  { &hf_rnsap_maxAdjustmentStep, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_MaxAdjustmentStep },
  { &hf_rnsap_adjustmentPeriod, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_AdjustmentPeriod },
  { &hf_rnsap_adjustmentRatio, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ScaledAdjustmentRatio },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_PowerBalancing_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_PowerBalancing_Information, DL_PowerBalancing_Information_sequence);

  return offset;
}
static int dissect_id_DL_PowerBalancing_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_PowerBalancing_Information(tvb, offset, actx, tree, hf_rnsap_id_DL_PowerBalancing_Information);
}


static const value_string rnsap_DL_PowerBalancing_ActivationIndicator_vals[] = {
  {   0, "dL-PowerBalancing-Activated" },
  { 0, NULL }
};


static int
dissect_rnsap_DL_PowerBalancing_ActivationIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_DL_PowerBalancing_ActivationIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_PowerBalancing_ActivationIndicator(tvb, offset, actx, tree, hf_rnsap_id_DL_PowerBalancing_ActivationIndicator);
}


static const value_string rnsap_DL_PowerBalancing_UpdatedIndicator_vals[] = {
  {   0, "dL-PowerBalancing-Updated" },
  { 0, NULL }
};


static int
dissect_rnsap_DL_PowerBalancing_UpdatedIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_DL_PowerBalancing_UpdatedIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_PowerBalancing_UpdatedIndicator(tvb, offset, actx, tree, hf_rnsap_id_DL_PowerBalancing_UpdatedIndicator);
}


static const per_sequence_t DL_ReferencePowerInformation_sequence[] = {
  { &hf_rnsap_common_DL_ReferencePowerInformation, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DL_Power },
  { &hf_rnsap_individual_DL_ReferencePowerInformation, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DL_ReferencePowerInformationList },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_ReferencePowerInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_ReferencePowerInformation, DL_ReferencePowerInformation_sequence);

  return offset;
}
static int dissect_id_DL_ReferencePowerInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_ReferencePowerInformation(tvb, offset, actx, tree, hf_rnsap_id_DL_ReferencePowerInformation);
}



static int
dissect_rnsap_Enhanced_PrimaryCPICH_EcNo(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 49U, NULL, FALSE);

  return offset;
}
static int dissect_id_Enhanced_PrimaryCPICH_EcNo(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Enhanced_PrimaryCPICH_EcNo(tvb, offset, actx, tree, hf_rnsap_id_Enhanced_PrimaryCPICH_EcNo);
}


static const value_string rnsap_IPSub_vals[] = {
  {   0, "first" },
  {   1, "second" },
  {   2, "both" },
  { 0, NULL }
};


static int
dissect_rnsap_IPSub(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_iPSub(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_IPSub(tvb, offset, actx, tree, hf_rnsap_iPSub);
}


static const per_sequence_t IPDL_TDD_ParametersLCR_sequence[] = {
  { &hf_rnsap_iPSpacingTDD  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_IPSpacingTDD },
  { &hf_rnsap_iPStart       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_IPStart },
  { &hf_rnsap_iPSub         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_IPSub },
  { &hf_rnsap_burstModeParameters, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_BurstModeParameters },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_IPDL_TDD_ParametersLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_IPDL_TDD_ParametersLCR, IPDL_TDD_ParametersLCR_sequence);

  return offset;
}
static int dissect_id_IPDL_TDD_ParametersLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_IPDL_TDD_ParametersLCR(tvb, offset, actx, tree, hf_rnsap_id_IPDL_TDD_ParametersLCR);
}



static int
dissect_rnsap_CellCapabilityContainer_FDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     32, 32, FALSE, NULL);

  return offset;
}
static int dissect_id_CellCapabilityContainer_FDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CellCapabilityContainer_FDD(tvb, offset, actx, tree, hf_rnsap_id_CellCapabilityContainer_FDD);
}



static int
dissect_rnsap_CellCapabilityContainer_TDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     32, 32, FALSE, NULL);

  return offset;
}
static int dissect_id_CellCapabilityContainer_TDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CellCapabilityContainer_TDD(tvb, offset, actx, tree, hf_rnsap_id_CellCapabilityContainer_TDD);
}



static int
dissect_rnsap_CellCapabilityContainer_TDD_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     32, 32, FALSE, NULL);

  return offset;
}
static int dissect_id_CellCapabilityContainer_TDD_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CellCapabilityContainer_TDD_LCR(tvb, offset, actx, tree, hf_rnsap_id_CellCapabilityContainer_TDD_LCR);
}


static const per_sequence_t RL_Specific_DCH_Info_Item_sequence[] = {
  { &hf_rnsap_dCH_id        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DCH_ID },
  { &hf_rnsap_bindingID     , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_BindingID },
  { &hf_rnsap_transportLayerAddress, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TransportLayerAddress },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_Specific_DCH_Info_Item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_Specific_DCH_Info_Item, RL_Specific_DCH_Info_Item_sequence);

  return offset;
}
static int dissect_RL_Specific_DCH_Info_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Specific_DCH_Info_Item(tvb, offset, actx, tree, hf_rnsap_RL_Specific_DCH_Info_item);
}


static const per_sequence_t RL_Specific_DCH_Info_sequence_of[1] = {
  { &hf_rnsap_RL_Specific_DCH_Info_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_Specific_DCH_Info_Item },
};

static int
dissect_rnsap_RL_Specific_DCH_Info(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_RL_Specific_DCH_Info, RL_Specific_DCH_Info_sequence_of,
                                                  1, 128);

  return offset;
}
static int dissect_id_RL_Specific_DCH_Info(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Specific_DCH_Info(tvb, offset, actx, tree, hf_rnsap_id_RL_Specific_DCH_Info);
}
static int dissect_rL_Specific_DCH_Info(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Specific_DCH_Info(tvb, offset, actx, tree, hf_rnsap_rL_Specific_DCH_Info);
}


static const per_sequence_t RL_ReconfigurationRequestFDD_RL_InformationList_sequence_of[1] = {
  { &hf_rnsap_RL_ReconfigurationRequestFDD_RL_InformationList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_RL_ReconfigurationRequestFDD_RL_InformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_RL_ReconfigurationRequestFDD_RL_InformationList, RL_ReconfigurationRequestFDD_RL_InformationList_sequence_of,
                                                  0, 16);

  return offset;
}
static int dissect_id_RL_ReconfigurationRequestFDD_RL_InformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_ReconfigurationRequestFDD_RL_InformationList(tvb, offset, actx, tree, hf_rnsap_id_RL_ReconfigurationRequestFDD_RL_InformationList);
}


static const per_sequence_t RL_ReconfigurationRequestFDD_RL_Information_IEs_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_rL_Specific_DCH_Info, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_RL_Specific_DCH_Info },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_ReconfigurationRequestFDD_RL_Information_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_ReconfigurationRequestFDD_RL_Information_IEs, RL_ReconfigurationRequestFDD_RL_Information_IEs_sequence);

  return offset;
}
static int dissect_id_RL_ReconfigurationRequestFDD_RL_Information_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_ReconfigurationRequestFDD_RL_Information_IEs(tvb, offset, actx, tree, hf_rnsap_id_RL_ReconfigurationRequestFDD_RL_Information_IEs);
}


static const per_sequence_t RL_ReconfigurationRequestTDD_RL_Information_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_rL_Specific_DCH_Info, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_RL_Specific_DCH_Info },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_ReconfigurationRequestTDD_RL_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_ReconfigurationRequestTDD_RL_Information, RL_ReconfigurationRequestTDD_RL_Information_sequence);

  return offset;
}
static int dissect_id_RL_ReconfigurationRequestTDD_RL_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_ReconfigurationRequestTDD_RL_Information(tvb, offset, actx, tree, hf_rnsap_id_RL_ReconfigurationRequestTDD_RL_Information);
}
static int dissect_Multiple_RL_ReconfigurationRequestTDD_RL_Information_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_ReconfigurationRequestTDD_RL_Information(tvb, offset, actx, tree, hf_rnsap_Multiple_RL_ReconfigurationRequestTDD_RL_Information_item);
}


static const value_string rnsap_CommonTransportChannelResourcesInitialisationNotRequired_vals[] = {
  {   0, "not-Required" },
  { 0, NULL }
};


static int
dissect_rnsap_CommonTransportChannelResourcesInitialisationNotRequired(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_CommonTransportChannelResourcesInitialisationNotRequired(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CommonTransportChannelResourcesInitialisationNotRequired(tvb, offset, actx, tree, hf_rnsap_id_CommonTransportChannelResourcesInitialisationNotRequired);
}


static const value_string rnsap_DelayedActivation_vals[] = {
  {   0, "cfn" },
  {   1, "separate-indication" },
  { 0, NULL }
};

static const per_choice_t DelayedActivation_choice[] = {
  {   0, &hf_rnsap_cfn           , ASN1_NO_EXTENSIONS     , dissect_rnsap_CFN },
  {   1, &hf_rnsap_separate_indication, ASN1_NO_EXTENSIONS     , dissect_rnsap_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_DelayedActivation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_DelayedActivation, DelayedActivation_choice,
                                 NULL);

  return offset;
}
static int dissect_id_DelayedActivation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DelayedActivation(tvb, offset, actx, tree, hf_rnsap_id_DelayedActivation);
}


static const per_sequence_t DelayedActivationInformationList_RL_ActivationCmdFDD_sequence_of[1] = {
  { &hf_rnsap_DelayedActivationInformationList_RL_ActivationCmdFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_DelayedActivationInformationList_RL_ActivationCmdFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DelayedActivationInformationList_RL_ActivationCmdFDD, DelayedActivationInformationList_RL_ActivationCmdFDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_DelayedActivationList_RL_ActivationCmdFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DelayedActivationInformationList_RL_ActivationCmdFDD(tvb, offset, actx, tree, hf_rnsap_id_DelayedActivationList_RL_ActivationCmdFDD);
}


static const value_string rnsap_Execution_Type_vals[] = {
  {   0, "synchronised" },
  {   1, "unsynchronised" },
  { 0, NULL }
};

static const per_choice_t Execution_Type_choice[] = {
  {   0, &hf_rnsap_synchronised  , ASN1_NO_EXTENSIONS     , dissect_rnsap_CFN },
  {   1, &hf_rnsap_unsynchronised, ASN1_NO_EXTENSIONS     , dissect_rnsap_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_Execution_Type(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_Execution_Type, Execution_Type_choice,
                                 NULL);

  return offset;
}
static int dissect_activation_type(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Execution_Type(tvb, offset, actx, tree, hf_rnsap_activation_type);
}
static int dissect_deactivation_type(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Execution_Type(tvb, offset, actx, tree, hf_rnsap_deactivation_type);
}


static const per_sequence_t Activate_Info_sequence[] = {
  { &hf_rnsap_activation_type, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_Execution_Type },
  { &hf_rnsap_initial_dl_tx_power, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_Power },
  { &hf_rnsap_firstRLS_Indicator, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_FirstRLS_Indicator },
  { &hf_rnsap_propagation_delay, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_PropagationDelay },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_Activate_Info(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_Activate_Info, Activate_Info_sequence);

  return offset;
}
static int dissect_activate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Activate_Info(tvb, offset, actx, tree, hf_rnsap_activate);
}


static const per_sequence_t Deactivate_Info_sequence[] = {
  { &hf_rnsap_deactivation_type, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_Execution_Type },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_Deactivate_Info(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_Deactivate_Info, Deactivate_Info_sequence);

  return offset;
}
static int dissect_deactivate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Deactivate_Info(tvb, offset, actx, tree, hf_rnsap_deactivate);
}


static const value_string rnsap_DelayedActivationUpdate_vals[] = {
  {   0, "activate" },
  {   1, "deactivate" },
  { 0, NULL }
};

static const per_choice_t DelayedActivationUpdate_choice[] = {
  {   0, &hf_rnsap_activate      , ASN1_NO_EXTENSIONS     , dissect_rnsap_Activate_Info },
  {   1, &hf_rnsap_deactivate    , ASN1_NO_EXTENSIONS     , dissect_rnsap_Deactivate_Info },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_DelayedActivationUpdate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_DelayedActivationUpdate, DelayedActivationUpdate_choice,
                                 NULL);

  return offset;
}
static int dissect_delayed_activation_update(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DelayedActivationUpdate(tvb, offset, actx, tree, hf_rnsap_delayed_activation_update);
}


static const per_sequence_t DelayedActivationInformation_RL_ActivationCmdFDD_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_delayed_activation_update, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DelayedActivationUpdate },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DelayedActivationInformation_RL_ActivationCmdFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DelayedActivationInformation_RL_ActivationCmdFDD, DelayedActivationInformation_RL_ActivationCmdFDD_sequence);

  return offset;
}
static int dissect_id_DelayedActivationInformation_RL_ActivationCmdFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DelayedActivationInformation_RL_ActivationCmdFDD(tvb, offset, actx, tree, hf_rnsap_id_DelayedActivationInformation_RL_ActivationCmdFDD);
}


static const per_sequence_t DelayedActivationInformationList_RL_ActivationCmdTDD_sequence_of[1] = {
  { &hf_rnsap_DelayedActivationInformationList_RL_ActivationCmdTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_DelayedActivationInformationList_RL_ActivationCmdTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DelayedActivationInformationList_RL_ActivationCmdTDD, DelayedActivationInformationList_RL_ActivationCmdTDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_DelayedActivationList_RL_ActivationCmdTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DelayedActivationInformationList_RL_ActivationCmdTDD(tvb, offset, actx, tree, hf_rnsap_id_DelayedActivationList_RL_ActivationCmdTDD);
}


static const per_sequence_t DelayedActivationInformation_RL_ActivationCmdTDD_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_delayed_activation_update, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DelayedActivationUpdate },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DelayedActivationInformation_RL_ActivationCmdTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DelayedActivationInformation_RL_ActivationCmdTDD, DelayedActivationInformation_RL_ActivationCmdTDD_sequence);

  return offset;
}
static int dissect_id_DelayedActivationInformation_RL_ActivationCmdTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DelayedActivationInformation_RL_ActivationCmdTDD(tvb, offset, actx, tree, hf_rnsap_id_DelayedActivationInformation_RL_ActivationCmdTDD);
}


static const per_sequence_t NeighbouringTDDCellMeasurementInformationLCR_sequence[] = {
  { &hf_rnsap_uC_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UC_ID },
  { &hf_rnsap_uARFCN        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UARFCN },
  { &hf_rnsap_cellParameterID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CellParameterID },
  { &hf_rnsap_timeSlotLCR   , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TimeSlotLCR },
  { &hf_rnsap_midambleShiftLCR, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_MidambleShiftLCR },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_NeighbouringTDDCellMeasurementInformationLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_NeighbouringTDDCellMeasurementInformationLCR, NeighbouringTDDCellMeasurementInformationLCR_sequence);

  return offset;
}
static int dissect_id_neighbouringTDDCellMeasurementInformationLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_NeighbouringTDDCellMeasurementInformationLCR(tvb, offset, actx, tree, hf_rnsap_id_neighbouringTDDCellMeasurementInformationLCR);
}



static int
dissect_rnsap_HSDSCH_MACdFlow_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 7U, NULL, FALSE);

  return offset;
}
static int dissect_hSDSCH_MACdFlow_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSDSCH_MACdFlow_ID(tvb, offset, actx, tree, hf_rnsap_hSDSCH_MACdFlow_ID);
}
static int dissect_hsDSCH_MACdFlow_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSDSCH_MACdFlow_ID(tvb, offset, actx, tree, hf_rnsap_hsDSCH_MACdFlow_ID);
}
static int dissect_associatedHSDSCH_MACdFlow(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSDSCH_MACdFlow_ID(tvb, offset, actx, tree, hf_rnsap_associatedHSDSCH_MACdFlow);
}


static const value_string rnsap_TrafficClass_vals[] = {
  {   0, "conversational" },
  {   1, "streaming" },
  {   2, "interactive" },
  {   3, "background" },
  { 0, NULL }
};


static int
dissect_rnsap_TrafficClass(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_id_TrafficClass(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TrafficClass(tvb, offset, actx, tree, hf_rnsap_id_TrafficClass);
}
static int dissect_trafficClass(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TrafficClass(tvb, offset, actx, tree, hf_rnsap_trafficClass);
}


static const per_sequence_t HSDSCH_MACdFlow_Specific_InfoItem_sequence[] = {
  { &hf_rnsap_hSDSCH_MACdFlow_ID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_HSDSCH_MACdFlow_ID },
  { &hf_rnsap_allocationRetentionPriority, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_AllocationRetentionPriority },
  { &hf_rnsap_trafficClass  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TrafficClass },
  { &hf_rnsap_bindingID     , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_BindingID },
  { &hf_rnsap_transportLayerAddress, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TransportLayerAddress },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_HSDSCH_MACdFlow_Specific_InfoItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_HSDSCH_MACdFlow_Specific_InfoItem, HSDSCH_MACdFlow_Specific_InfoItem_sequence);

  return offset;
}
static int dissect_HSDSCH_MACdFlow_Specific_InfoList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSDSCH_MACdFlow_Specific_InfoItem(tvb, offset, actx, tree, hf_rnsap_HSDSCH_MACdFlow_Specific_InfoList_item);
}


static const per_sequence_t HSDSCH_MACdFlow_Specific_InfoList_sequence_of[1] = {
  { &hf_rnsap_HSDSCH_MACdFlow_Specific_InfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_HSDSCH_MACdFlow_Specific_InfoItem },
};

static int
dissect_rnsap_HSDSCH_MACdFlow_Specific_InfoList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_HSDSCH_MACdFlow_Specific_InfoList, HSDSCH_MACdFlow_Specific_InfoList_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_hSDSCH_MACdFlow_Specific_Info(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSDSCH_MACdFlow_Specific_InfoList(tvb, offset, actx, tree, hf_rnsap_hSDSCH_MACdFlow_Specific_Info);
}



static int
dissect_rnsap_PriorityQueue_Id(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 7U, NULL, FALSE);

  return offset;
}
static int dissect_deletePriorityQueue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PriorityQueue_Id(tvb, offset, actx, tree, hf_rnsap_deletePriorityQueue);
}
static int dissect_priorityQueue_Id(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PriorityQueue_Id(tvb, offset, actx, tree, hf_rnsap_priorityQueue_Id);
}
static int dissect_priorityQueueId(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PriorityQueue_Id(tvb, offset, actx, tree, hf_rnsap_priorityQueueId);
}


static const value_string rnsap_T1_vals[] = {
  {   0, "v10" },
  {   1, "v20" },
  {   2, "v30" },
  {   3, "v40" },
  {   4, "v50" },
  {   5, "v60" },
  {   6, "v70" },
  {   7, "v80" },
  {   8, "v90" },
  {   9, "v100" },
  {  10, "v120" },
  {  11, "v140" },
  {  12, "v160" },
  {  13, "v200" },
  {  14, "v300" },
  {  15, "v400" },
  { 0, NULL }
};


static int
dissect_rnsap_T1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_t1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_T1(tvb, offset, actx, tree, hf_rnsap_t1);
}


static const value_string rnsap_DiscardTimer_vals[] = {
  {   0, "v20" },
  {   1, "v40" },
  {   2, "v60" },
  {   3, "v80" },
  {   4, "v100" },
  {   5, "v120" },
  {   6, "v140" },
  {   7, "v160" },
  {   8, "v180" },
  {   9, "v200" },
  {  10, "v250" },
  {  11, "v300" },
  {  12, "v400" },
  {  13, "v500" },
  {  14, "v750" },
  {  15, "v1000" },
  {  16, "v1250" },
  {  17, "v1500" },
  {  18, "v1750" },
  {  19, "v2000" },
  {  20, "v2500" },
  {  21, "v3000" },
  {  22, "v3500" },
  {  23, "v4000" },
  {  24, "v4500" },
  {  25, "v5000" },
  {  26, "v7500" },
  { 0, NULL }
};


static int
dissect_rnsap_DiscardTimer(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     27, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_discardTimer(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DiscardTimer(tvb, offset, actx, tree, hf_rnsap_discardTimer);
}


static const value_string rnsap_MAC_hsWindowSize_vals[] = {
  {   0, "v4" },
  {   1, "v6" },
  {   2, "v8" },
  {   3, "v12" },
  {   4, "v16" },
  {   5, "v24" },
  {   6, "v32" },
  { 0, NULL }
};


static int
dissect_rnsap_MAC_hsWindowSize(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     7, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_mAC_hsWindowSize(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MAC_hsWindowSize(tvb, offset, actx, tree, hf_rnsap_mAC_hsWindowSize);
}



static int
dissect_rnsap_MAChsGuaranteedBitRate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 16777215U, NULL, TRUE);

  return offset;
}
static int dissect_mAChsGuaranteedBitRate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MAChsGuaranteedBitRate(tvb, offset, actx, tree, hf_rnsap_mAChsGuaranteedBitRate);
}



static int
dissect_rnsap_SID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 7U, NULL, FALSE);

  return offset;
}
static int dissect_sID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SID(tvb, offset, actx, tree, hf_rnsap_sID);
}



static int
dissect_rnsap_MACdPDU_Size(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 5000U, NULL, TRUE);

  return offset;
}
static int dissect_mACdPDU_Size(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MACdPDU_Size(tvb, offset, actx, tree, hf_rnsap_mACdPDU_Size);
}
static int dissect_maximum_MACdPDU_Size(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MACdPDU_Size(tvb, offset, actx, tree, hf_rnsap_maximum_MACdPDU_Size);
}


static const per_sequence_t MACdPDU_Size_IndexItem_sequence[] = {
  { &hf_rnsap_sID           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SID },
  { &hf_rnsap_mACdPDU_Size  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MACdPDU_Size },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_MACdPDU_Size_IndexItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_MACdPDU_Size_IndexItem, MACdPDU_Size_IndexItem_sequence);

  return offset;
}
static int dissect_MACdPDU_Size_IndexList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MACdPDU_Size_IndexItem(tvb, offset, actx, tree, hf_rnsap_MACdPDU_Size_IndexList_item);
}


static const per_sequence_t MACdPDU_Size_IndexList_sequence_of[1] = {
  { &hf_rnsap_MACdPDU_Size_IndexList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_MACdPDU_Size_IndexItem },
};

static int
dissect_rnsap_MACdPDU_Size_IndexList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_MACdPDU_Size_IndexList, MACdPDU_Size_IndexList_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_mACdPDU_Size_Index(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MACdPDU_Size_IndexList(tvb, offset, actx, tree, hf_rnsap_mACdPDU_Size_Index);
}


static const value_string rnsap_RLC_Mode_vals[] = {
  {   0, "rLC-AM" },
  {   1, "rLC-UM" },
  { 0, NULL }
};


static int
dissect_rnsap_RLC_Mode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_rLC_Mode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RLC_Mode(tvb, offset, actx, tree, hf_rnsap_rLC_Mode);
}


static const per_sequence_t PriorityQueue_InfoItem_sequence[] = {
  { &hf_rnsap_priorityQueue_Id, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PriorityQueue_Id },
  { &hf_rnsap_associatedHSDSCH_MACdFlow, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_HSDSCH_MACdFlow_ID },
  { &hf_rnsap_schedulingPriorityIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SchedulingPriorityIndicator },
  { &hf_rnsap_t1            , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_T1 },
  { &hf_rnsap_discardTimer  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DiscardTimer },
  { &hf_rnsap_mAC_hsWindowSize, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MAC_hsWindowSize },
  { &hf_rnsap_mAChsGuaranteedBitRate, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_MAChsGuaranteedBitRate },
  { &hf_rnsap_mACdPDU_Size_Index, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MACdPDU_Size_IndexList },
  { &hf_rnsap_rLC_Mode      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RLC_Mode },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_PriorityQueue_InfoItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_PriorityQueue_InfoItem, PriorityQueue_InfoItem_sequence);

  return offset;
}
static int dissect_PriorityQueue_InfoList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PriorityQueue_InfoItem(tvb, offset, actx, tree, hf_rnsap_PriorityQueue_InfoList_item);
}


static const per_sequence_t PriorityQueue_InfoList_sequence_of[1] = {
  { &hf_rnsap_PriorityQueue_InfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_PriorityQueue_InfoItem },
};

static int
dissect_rnsap_PriorityQueue_InfoList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_PriorityQueue_InfoList, PriorityQueue_InfoList_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_priorityQueue_Info(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PriorityQueue_InfoList(tvb, offset, actx, tree, hf_rnsap_priorityQueue_Info);
}


static const per_sequence_t HSDSCH_MACdFlows_Information_sequence[] = {
  { &hf_rnsap_hSDSCH_MACdFlow_Specific_Info, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_HSDSCH_MACdFlow_Specific_InfoList },
  { &hf_rnsap_priorityQueue_Info, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PriorityQueue_InfoList },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_HSDSCH_MACdFlows_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_HSDSCH_MACdFlows_Information, HSDSCH_MACdFlows_Information_sequence);

  return offset;
}
static int dissect_id_HSDSCH_MACdFlows_to_Add(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSDSCH_MACdFlows_Information(tvb, offset, actx, tree, hf_rnsap_id_HSDSCH_MACdFlows_to_Add);
}
static int dissect_hSDSCH_MACdFlows_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSDSCH_MACdFlows_Information(tvb, offset, actx, tree, hf_rnsap_hSDSCH_MACdFlows_Information);
}



static int
dissect_rnsap_INTEGER_1_64_(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 64U, NULL, TRUE);

  return offset;
}
static int dissect_hSDSCH_Physical_Layer_Category(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_1_64_(tvb, offset, actx, tree, hf_rnsap_hSDSCH_Physical_Layer_Category);
}


static const per_sequence_t UE_Capabilities_Info_sequence[] = {
  { &hf_rnsap_hSDSCH_Physical_Layer_Category, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_INTEGER_1_64_ },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UE_Capabilities_Info(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UE_Capabilities_Info, UE_Capabilities_Info_sequence);

  return offset;
}
static int dissect_uE_Capabilities_Info(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UE_Capabilities_Info(tvb, offset, actx, tree, hf_rnsap_uE_Capabilities_Info);
}



static int
dissect_rnsap_MAChsReorderingBufferSize_for_RLC_UM(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 300U, NULL, TRUE);

  return offset;
}
static int dissect_mAChs_Reordering_Buffer_Size_for_RLC_UM(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MAChsReorderingBufferSize_for_RLC_UM(tvb, offset, actx, tree, hf_rnsap_mAChs_Reordering_Buffer_Size_for_RLC_UM);
}


static const value_string rnsap_CQI_Feedback_Cycle_vals[] = {
  {   0, "v0" },
  {   1, "v2" },
  {   2, "v4" },
  {   3, "v8" },
  {   4, "v10" },
  {   5, "v20" },
  {   6, "v40" },
  {   7, "v80" },
  {   8, "v160" },
  { 0, NULL }
};


static int
dissect_rnsap_CQI_Feedback_Cycle(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     9, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_cqiFeedback_CycleK(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CQI_Feedback_Cycle(tvb, offset, actx, tree, hf_rnsap_cqiFeedback_CycleK);
}



static int
dissect_rnsap_CQI_RepetitionFactor(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 4U, NULL, TRUE);

  return offset;
}
static int dissect_cqiRepetitionFactor(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CQI_RepetitionFactor(tvb, offset, actx, tree, hf_rnsap_cqiRepetitionFactor);
}



static int
dissect_rnsap_AckNack_RepetitionFactor(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 4U, NULL, TRUE);

  return offset;
}
static int dissect_ackNackRepetitionFactor(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_AckNack_RepetitionFactor(tvb, offset, actx, tree, hf_rnsap_ackNackRepetitionFactor);
}



static int
dissect_rnsap_CQI_Power_Offset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 8U, NULL, TRUE);

  return offset;
}
static int dissect_cqiPowerOffset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CQI_Power_Offset(tvb, offset, actx, tree, hf_rnsap_cqiPowerOffset);
}



static int
dissect_rnsap_Ack_Power_Offset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 8U, NULL, TRUE);

  return offset;
}
static int dissect_ackPowerOffset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Ack_Power_Offset(tvb, offset, actx, tree, hf_rnsap_ackPowerOffset);
}



static int
dissect_rnsap_Nack_Power_Offset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 8U, NULL, TRUE);

  return offset;
}
static int dissect_nackPowerOffset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Nack_Power_Offset(tvb, offset, actx, tree, hf_rnsap_nackPowerOffset);
}



static int
dissect_rnsap_HSSCCH_PowerOffset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 255U, NULL, FALSE);

  return offset;
}
static int dissect_hsscch_PowerOffset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSSCCH_PowerOffset(tvb, offset, actx, tree, hf_rnsap_hsscch_PowerOffset);
}


static const per_sequence_t HSDSCH_FDD_Information_sequence[] = {
  { &hf_rnsap_hSDSCH_MACdFlows_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_HSDSCH_MACdFlows_Information },
  { &hf_rnsap_uE_Capabilities_Info, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UE_Capabilities_Info },
  { &hf_rnsap_mAChs_Reordering_Buffer_Size_for_RLC_UM, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MAChsReorderingBufferSize_for_RLC_UM },
  { &hf_rnsap_cqiFeedback_CycleK, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CQI_Feedback_Cycle },
  { &hf_rnsap_cqiRepetitionFactor, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_CQI_RepetitionFactor },
  { &hf_rnsap_ackNackRepetitionFactor, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_AckNack_RepetitionFactor },
  { &hf_rnsap_cqiPowerOffset, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CQI_Power_Offset },
  { &hf_rnsap_ackPowerOffset, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_Ack_Power_Offset },
  { &hf_rnsap_nackPowerOffset, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_Nack_Power_Offset },
  { &hf_rnsap_hsscch_PowerOffset, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_HSSCCH_PowerOffset },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_HSDSCH_FDD_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_HSDSCH_FDD_Information, HSDSCH_FDD_Information_sequence);

  return offset;
}
static int dissect_id_HSDSCH_FDD_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSDSCH_FDD_Information(tvb, offset, actx, tree, hf_rnsap_id_HSDSCH_FDD_Information);
}



static int
dissect_rnsap_HSDSCH_InitialWindowSize(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 255U, NULL, FALSE);

  return offset;
}
static int dissect_hSDSCH_InitialWindowSize(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSDSCH_InitialWindowSize(tvb, offset, actx, tree, hf_rnsap_hSDSCH_InitialWindowSize);
}


static const per_sequence_t HSDSCH_Initial_Capacity_AllocationItem_sequence[] = {
  { &hf_rnsap_schedulingPriorityIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SchedulingPriorityIndicator },
  { &hf_rnsap_maximum_MACdPDU_Size, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MACdPDU_Size },
  { &hf_rnsap_hSDSCH_InitialWindowSize, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_HSDSCH_InitialWindowSize },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_HSDSCH_Initial_Capacity_AllocationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_HSDSCH_Initial_Capacity_AllocationItem, HSDSCH_Initial_Capacity_AllocationItem_sequence);

  return offset;
}
static int dissect_HSDSCH_Initial_Capacity_Allocation_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSDSCH_Initial_Capacity_AllocationItem(tvb, offset, actx, tree, hf_rnsap_HSDSCH_Initial_Capacity_Allocation_item);
}


static const per_sequence_t HSDSCH_Initial_Capacity_Allocation_sequence_of[1] = {
  { &hf_rnsap_HSDSCH_Initial_Capacity_Allocation_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_HSDSCH_Initial_Capacity_AllocationItem },
};

static int
dissect_rnsap_HSDSCH_Initial_Capacity_Allocation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_HSDSCH_Initial_Capacity_Allocation, HSDSCH_Initial_Capacity_Allocation_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_hSDSCH_Initial_Capacity_Allocation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSDSCH_Initial_Capacity_Allocation(tvb, offset, actx, tree, hf_rnsap_hSDSCH_Initial_Capacity_Allocation);
}


static const per_sequence_t HSDSCH_MACdFlow_Specific_InfoItem_Response_sequence[] = {
  { &hf_rnsap_hSDSCH_MACdFlow_ID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_HSDSCH_MACdFlow_ID },
  { &hf_rnsap_bindingID     , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_BindingID },
  { &hf_rnsap_transportLayerAddress, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TransportLayerAddress },
  { &hf_rnsap_hSDSCH_Initial_Capacity_Allocation, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_HSDSCH_Initial_Capacity_Allocation },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_HSDSCH_MACdFlow_Specific_InfoItem_Response(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_HSDSCH_MACdFlow_Specific_InfoItem_Response, HSDSCH_MACdFlow_Specific_InfoItem_Response_sequence);

  return offset;
}
static int dissect_HSDSCH_MACdFlow_Specific_InfoList_Response_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSDSCH_MACdFlow_Specific_InfoItem_Response(tvb, offset, actx, tree, hf_rnsap_HSDSCH_MACdFlow_Specific_InfoList_Response_item);
}


static const per_sequence_t HSDSCH_MACdFlow_Specific_InfoList_Response_sequence_of[1] = {
  { &hf_rnsap_HSDSCH_MACdFlow_Specific_InfoList_Response_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_HSDSCH_MACdFlow_Specific_InfoItem_Response },
};

static int
dissect_rnsap_HSDSCH_MACdFlow_Specific_InfoList_Response(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_HSDSCH_MACdFlow_Specific_InfoList_Response, HSDSCH_MACdFlow_Specific_InfoList_Response_sequence_of,
                                                  0, 8);

  return offset;
}
static int dissect_hSDSCH_MACdFlow_Specific_InfoList_Response(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSDSCH_MACdFlow_Specific_InfoList_Response(tvb, offset, actx, tree, hf_rnsap_hSDSCH_MACdFlow_Specific_InfoList_Response);
}


static const per_sequence_t HSSCCH_FDD_Specific_InfoItem_Response_sequence[] = {
  { &hf_rnsap_code_Number   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_INTEGER_0_127 },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_HSSCCH_FDD_Specific_InfoItem_Response(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_HSSCCH_FDD_Specific_InfoItem_Response, HSSCCH_FDD_Specific_InfoItem_Response_sequence);

  return offset;
}
static int dissect_HSSCCH_FDD_Specific_InfoList_Response_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSSCCH_FDD_Specific_InfoItem_Response(tvb, offset, actx, tree, hf_rnsap_HSSCCH_FDD_Specific_InfoList_Response_item);
}


static const per_sequence_t HSSCCH_FDD_Specific_InfoList_Response_sequence_of[1] = {
  { &hf_rnsap_HSSCCH_FDD_Specific_InfoList_Response_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_HSSCCH_FDD_Specific_InfoItem_Response },
};

static int
dissect_rnsap_HSSCCH_FDD_Specific_InfoList_Response(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_HSSCCH_FDD_Specific_InfoList_Response, HSSCCH_FDD_Specific_InfoList_Response_sequence_of,
                                                  0, 4);

  return offset;
}
static int dissect_hSSCCH_Specific_InfoList_Response(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSSCCH_FDD_Specific_InfoList_Response(tvb, offset, actx, tree, hf_rnsap_hSSCCH_Specific_InfoList_Response);
}



static int
dissect_rnsap_Measurement_Power_Offset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              -12, 26U, NULL, FALSE);

  return offset;
}
static int dissect_measurement_Power_Offset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Measurement_Power_Offset(tvb, offset, actx, tree, hf_rnsap_measurement_Power_Offset);
}



static int
dissect_rnsap_INTEGER_1_8_(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 8U, NULL, TRUE);

  return offset;
}
static int dissect_number_of_Processes(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_1_8_(tvb, offset, actx, tree, hf_rnsap_number_of_Processes);
}


static const per_sequence_t HARQ_MemoryPartitioning_Implicit_sequence[] = {
  { &hf_rnsap_number_of_Processes, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_INTEGER_1_8_ },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_HARQ_MemoryPartitioning_Implicit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_HARQ_MemoryPartitioning_Implicit, HARQ_MemoryPartitioning_Implicit_sequence);

  return offset;
}
static int dissect_implicit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HARQ_MemoryPartitioning_Implicit(tvb, offset, actx, tree, hf_rnsap_implicit);
}


static const value_string rnsap_T_process_Memory_Size_vals[] = {
  {   0, "hms800" },
  {   1, "hms1600" },
  {   2, "hms2400" },
  {   3, "hms3200" },
  {   4, "hms4000" },
  {   5, "hms4800" },
  {   6, "hms5600" },
  {   7, "hms6400" },
  {   8, "hms7200" },
  {   9, "hms8000" },
  {  10, "hms8800" },
  {  11, "hms9600" },
  {  12, "hms10400" },
  {  13, "hms11200" },
  {  14, "hms12000" },
  {  15, "hms12800" },
  {  16, "hms13600" },
  {  17, "hms14400" },
  {  18, "hms15200" },
  {  19, "hms16000" },
  {  20, "hms17600" },
  {  21, "hms19200" },
  {  22, "hms20800" },
  {  23, "hms22400" },
  {  24, "hms24000" },
  {  25, "hms25600" },
  {  26, "hms27200" },
  {  27, "hms28800" },
  {  28, "hms30400" },
  {  29, "hms32000" },
  {  30, "hms36000" },
  {  31, "hms40000" },
  {  32, "hms44000" },
  {  33, "hms48000" },
  {  34, "hms52000" },
  {  35, "hms56000" },
  {  36, "hms60000" },
  {  37, "hms64000" },
  {  38, "hms68000" },
  {  39, "hms72000" },
  {  40, "hms76000" },
  {  41, "hms80000" },
  {  42, "hms88000" },
  {  43, "hms96000" },
  {  44, "hms104000" },
  {  45, "hms112000" },
  {  46, "hms120000" },
  {  47, "hms128000" },
  {  48, "hms136000" },
  {  49, "hms144000" },
  {  50, "hms152000" },
  {  51, "hms160000" },
  {  52, "hms176000" },
  {  53, "hms192000" },
  {  54, "hms208000" },
  {  55, "hms224000" },
  {  56, "hms240000" },
  {  57, "hms256000" },
  {  58, "hms272000" },
  {  59, "hms288000" },
  {  60, "hms304000" },
  { 0, NULL }
};


static int
dissect_rnsap_T_process_Memory_Size(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     61, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_process_Memory_Size(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_T_process_Memory_Size(tvb, offset, actx, tree, hf_rnsap_process_Memory_Size);
}


static const per_sequence_t HARQ_MemoryPartitioningItem_sequence[] = {
  { &hf_rnsap_process_Memory_Size, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_T_process_Memory_Size },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_HARQ_MemoryPartitioningItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_HARQ_MemoryPartitioningItem, HARQ_MemoryPartitioningItem_sequence);

  return offset;
}
static int dissect_HARQ_MemoryPartitioningList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HARQ_MemoryPartitioningItem(tvb, offset, actx, tree, hf_rnsap_HARQ_MemoryPartitioningList_item);
}


static const per_sequence_t HARQ_MemoryPartitioningList_sequence_of[1] = {
  { &hf_rnsap_HARQ_MemoryPartitioningList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_HARQ_MemoryPartitioningItem },
};

static int
dissect_rnsap_HARQ_MemoryPartitioningList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_HARQ_MemoryPartitioningList, HARQ_MemoryPartitioningList_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_hARQ_MemoryPartitioningList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HARQ_MemoryPartitioningList(tvb, offset, actx, tree, hf_rnsap_hARQ_MemoryPartitioningList);
}


static const per_sequence_t HARQ_MemoryPartitioning_Explicit_sequence[] = {
  { &hf_rnsap_hARQ_MemoryPartitioningList, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_HARQ_MemoryPartitioningList },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_HARQ_MemoryPartitioning_Explicit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_HARQ_MemoryPartitioning_Explicit, HARQ_MemoryPartitioning_Explicit_sequence);

  return offset;
}
static int dissect_explicit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HARQ_MemoryPartitioning_Explicit(tvb, offset, actx, tree, hf_rnsap_explicit);
}


static const value_string rnsap_HARQ_MemoryPartitioning_vals[] = {
  {   0, "implicit" },
  {   1, "explicit" },
  { 0, NULL }
};

static const per_choice_t HARQ_MemoryPartitioning_choice[] = {
  {   0, &hf_rnsap_implicit      , ASN1_EXTENSION_ROOT    , dissect_rnsap_HARQ_MemoryPartitioning_Implicit },
  {   1, &hf_rnsap_explicit      , ASN1_EXTENSION_ROOT    , dissect_rnsap_HARQ_MemoryPartitioning_Explicit },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_HARQ_MemoryPartitioning(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_HARQ_MemoryPartitioning, HARQ_MemoryPartitioning_choice,
                                 NULL);

  return offset;
}
static int dissect_hARQ_MemoryPartitioning(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HARQ_MemoryPartitioning(tvb, offset, actx, tree, hf_rnsap_hARQ_MemoryPartitioning);
}


static const per_sequence_t HSDSCH_FDD_Information_Response_sequence[] = {
  { &hf_rnsap_hSDSCH_MACdFlow_Specific_InfoList_Response, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_HSDSCH_MACdFlow_Specific_InfoList_Response },
  { &hf_rnsap_hSSCCH_Specific_InfoList_Response, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_HSSCCH_FDD_Specific_InfoList_Response },
  { &hf_rnsap_hSPDSCH_and_HSSCCH_ScramblingCode, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DL_ScramblingCode },
  { &hf_rnsap_measurement_Power_Offset, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Measurement_Power_Offset },
  { &hf_rnsap_hARQ_MemoryPartitioning, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_HARQ_MemoryPartitioning },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_HSDSCH_FDD_Information_Response(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_HSDSCH_FDD_Information_Response, HSDSCH_FDD_Information_Response_sequence);

  return offset;
}
static int dissect_id_HSDSCH_FDD_Information_Response(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSDSCH_FDD_Information_Response(tvb, offset, actx, tree, hf_rnsap_id_HSDSCH_FDD_Information_Response);
}


static const value_string rnsap_HSSCCH_CodeChangeIndicator_vals[] = {
  {   0, "hsSCCHCodeChangeNeeded" },
  { 0, NULL }
};


static int
dissect_rnsap_HSSCCH_CodeChangeIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_hsSCCHCodeChangeIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSSCCH_CodeChangeIndicator(tvb, offset, actx, tree, hf_rnsap_hsSCCHCodeChangeIndicator);
}


static const per_sequence_t HSDSCH_FDD_Update_Information_sequence[] = {
  { &hf_rnsap_hsSCCHCodeChangeIndicator, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_HSSCCH_CodeChangeIndicator },
  { &hf_rnsap_cqiFeedback_CycleK, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_CQI_Feedback_Cycle },
  { &hf_rnsap_cqiRepetitionFactor, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_CQI_RepetitionFactor },
  { &hf_rnsap_ackNackRepetitionFactor, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_AckNack_RepetitionFactor },
  { &hf_rnsap_cqiPowerOffset, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_CQI_Power_Offset },
  { &hf_rnsap_ackPowerOffset, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Ack_Power_Offset },
  { &hf_rnsap_nackPowerOffset, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Nack_Power_Offset },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_HSDSCH_FDD_Update_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_HSDSCH_FDD_Update_Information, HSDSCH_FDD_Update_Information_sequence);

  return offset;
}
static int dissect_id_HSDSCH_FDD_Update_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSDSCH_FDD_Update_Information(tvb, offset, actx, tree, hf_rnsap_id_HSDSCH_FDD_Update_Information);
}


static const per_sequence_t HSDSCH_MACdFlow_Specific_InfoItem_to_Modify_sequence[] = {
  { &hf_rnsap_hSDSCH_MACdFlow_ID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_HSDSCH_MACdFlow_ID },
  { &hf_rnsap_allocationRetentionPriority, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_AllocationRetentionPriority },
  { &hf_rnsap_transportBearerRequestIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TransportBearerRequestIndicator },
  { &hf_rnsap_trafficClass  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TrafficClass },
  { &hf_rnsap_bindingID     , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_BindingID },
  { &hf_rnsap_transportLayerAddress, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TransportLayerAddress },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_HSDSCH_MACdFlow_Specific_InfoItem_to_Modify(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_HSDSCH_MACdFlow_Specific_InfoItem_to_Modify, HSDSCH_MACdFlow_Specific_InfoItem_to_Modify_sequence);

  return offset;
}
static int dissect_HSDSCH_MACdFlow_Specific_InfoList_to_Modify_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSDSCH_MACdFlow_Specific_InfoItem_to_Modify(tvb, offset, actx, tree, hf_rnsap_HSDSCH_MACdFlow_Specific_InfoList_to_Modify_item);
}


static const per_sequence_t HSDSCH_MACdFlow_Specific_InfoList_to_Modify_sequence_of[1] = {
  { &hf_rnsap_HSDSCH_MACdFlow_Specific_InfoList_to_Modify_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_HSDSCH_MACdFlow_Specific_InfoItem_to_Modify },
};

static int
dissect_rnsap_HSDSCH_MACdFlow_Specific_InfoList_to_Modify(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_HSDSCH_MACdFlow_Specific_InfoList_to_Modify, HSDSCH_MACdFlow_Specific_InfoList_to_Modify_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_hSDSCH_MACdFlow_Specific_InfoList_to_Modify(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSDSCH_MACdFlow_Specific_InfoList_to_Modify(tvb, offset, actx, tree, hf_rnsap_hSDSCH_MACdFlow_Specific_InfoList_to_Modify);
}


static const per_sequence_t PriorityQueue_InfoItem_to_Add_sequence[] = {
  { &hf_rnsap_priorityQueue_Id, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PriorityQueue_Id },
  { &hf_rnsap_associatedHSDSCH_MACdFlow, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_HSDSCH_MACdFlow_ID },
  { &hf_rnsap_schedulingPriorityIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SchedulingPriorityIndicator },
  { &hf_rnsap_t1            , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_T1 },
  { &hf_rnsap_discardTimer  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DiscardTimer },
  { &hf_rnsap_mAC_hsWindowSize, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MAC_hsWindowSize },
  { &hf_rnsap_mAChsGuaranteedBitRate, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_MAChsGuaranteedBitRate },
  { &hf_rnsap_mACdPDU_Size_Index, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MACdPDU_Size_IndexList },
  { &hf_rnsap_rLC_Mode      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RLC_Mode },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_PriorityQueue_InfoItem_to_Add(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_PriorityQueue_InfoItem_to_Add, PriorityQueue_InfoItem_to_Add_sequence);

  return offset;
}
static int dissect_addPriorityQueue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PriorityQueue_InfoItem_to_Add(tvb, offset, actx, tree, hf_rnsap_addPriorityQueue);
}


static const per_sequence_t MACdPDU_Size_IndexItem_to_Modify_sequence[] = {
  { &hf_rnsap_sID           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SID },
  { &hf_rnsap_mACdPDU_Size  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MACdPDU_Size },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_MACdPDU_Size_IndexItem_to_Modify(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_MACdPDU_Size_IndexItem_to_Modify, MACdPDU_Size_IndexItem_to_Modify_sequence);

  return offset;
}
static int dissect_MACdPDU_Size_IndexList_to_Modify_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MACdPDU_Size_IndexItem_to_Modify(tvb, offset, actx, tree, hf_rnsap_MACdPDU_Size_IndexList_to_Modify_item);
}


static const per_sequence_t MACdPDU_Size_IndexList_to_Modify_sequence_of[1] = {
  { &hf_rnsap_MACdPDU_Size_IndexList_to_Modify_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_MACdPDU_Size_IndexItem_to_Modify },
};

static int
dissect_rnsap_MACdPDU_Size_IndexList_to_Modify(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_MACdPDU_Size_IndexList_to_Modify, MACdPDU_Size_IndexList_to_Modify_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_mACdPDU_Size_Index_to_Modify(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MACdPDU_Size_IndexList_to_Modify(tvb, offset, actx, tree, hf_rnsap_mACdPDU_Size_Index_to_Modify);
}


static const per_sequence_t PriorityQueue_InfoItem_to_Modify_sequence[] = {
  { &hf_rnsap_priorityQueue_Id, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PriorityQueue_Id },
  { &hf_rnsap_schedulingPriorityIndicator, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_SchedulingPriorityIndicator },
  { &hf_rnsap_t1            , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_T1 },
  { &hf_rnsap_discardTimer  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DiscardTimer },
  { &hf_rnsap_mAC_hsWindowSize, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_MAC_hsWindowSize },
  { &hf_rnsap_mAChsGuaranteedBitRate, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_MAChsGuaranteedBitRate },
  { &hf_rnsap_mACdPDU_Size_Index_to_Modify, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_MACdPDU_Size_IndexList_to_Modify },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_PriorityQueue_InfoItem_to_Modify(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_PriorityQueue_InfoItem_to_Modify, PriorityQueue_InfoItem_to_Modify_sequence);

  return offset;
}
static int dissect_modifyPriorityQueue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PriorityQueue_InfoItem_to_Modify(tvb, offset, actx, tree, hf_rnsap_modifyPriorityQueue);
}


static const value_string rnsap_ModifyPriorityQueue_vals[] = {
  {   0, "addPriorityQueue" },
  {   1, "modifyPriorityQueue" },
  {   2, "deletePriorityQueue" },
  { 0, NULL }
};

static const per_choice_t ModifyPriorityQueue_choice[] = {
  {   0, &hf_rnsap_addPriorityQueue, ASN1_EXTENSION_ROOT    , dissect_rnsap_PriorityQueue_InfoItem_to_Add },
  {   1, &hf_rnsap_modifyPriorityQueue, ASN1_EXTENSION_ROOT    , dissect_rnsap_PriorityQueue_InfoItem_to_Modify },
  {   2, &hf_rnsap_deletePriorityQueue, ASN1_EXTENSION_ROOT    , dissect_rnsap_PriorityQueue_Id },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_ModifyPriorityQueue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_ModifyPriorityQueue, ModifyPriorityQueue_choice,
                                 NULL);

  return offset;
}
static int dissect_PriorityQueue_InfoList_to_Modify_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ModifyPriorityQueue(tvb, offset, actx, tree, hf_rnsap_PriorityQueue_InfoList_to_Modify_item);
}


static const per_sequence_t PriorityQueue_InfoList_to_Modify_sequence_of[1] = {
  { &hf_rnsap_PriorityQueue_InfoList_to_Modify_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ModifyPriorityQueue },
};

static int
dissect_rnsap_PriorityQueue_InfoList_to_Modify(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_PriorityQueue_InfoList_to_Modify, PriorityQueue_InfoList_to_Modify_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_priorityQueue_Info_to_Modify(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PriorityQueue_InfoList_to_Modify(tvb, offset, actx, tree, hf_rnsap_priorityQueue_Info_to_Modify);
}


static const value_string rnsap_HSSCCH_Code_Change_Grant_vals[] = {
  {   0, "changeGranted" },
  { 0, NULL }
};


static int
dissect_rnsap_HSSCCH_Code_Change_Grant(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_hSSCCH_CodeChangeGrant(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSSCCH_Code_Change_Grant(tvb, offset, actx, tree, hf_rnsap_hSSCCH_CodeChangeGrant);
}



static int
dissect_rnsap_TDD_AckNack_Power_Offset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              -7, 8U, NULL, TRUE);

  return offset;
}
static int dissect_tDDAckNackPowerOffset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_AckNack_Power_Offset(tvb, offset, actx, tree, hf_rnsap_tDDAckNackPowerOffset);
}
static int dissect_tDD_AckNack_Power_Offset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_AckNack_Power_Offset(tvb, offset, actx, tree, hf_rnsap_tDD_AckNack_Power_Offset);
}


static const per_sequence_t HSDSCH_Information_to_Modify_sequence[] = {
  { &hf_rnsap_hSDSCH_MACdFlow_Specific_InfoList_to_Modify, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_HSDSCH_MACdFlow_Specific_InfoList_to_Modify },
  { &hf_rnsap_priorityQueue_Info_to_Modify, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_PriorityQueue_InfoList_to_Modify },
  { &hf_rnsap_mAChs_Reordering_Buffer_Size_for_RLC_UM, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_MAChsReorderingBufferSize_for_RLC_UM },
  { &hf_rnsap_cqiFeedback_CycleK, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_CQI_Feedback_Cycle },
  { &hf_rnsap_cqiRepetitionFactor, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_CQI_RepetitionFactor },
  { &hf_rnsap_ackNackRepetitionFactor, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_AckNack_RepetitionFactor },
  { &hf_rnsap_cqiPowerOffset, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_CQI_Power_Offset },
  { &hf_rnsap_ackPowerOffset, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Ack_Power_Offset },
  { &hf_rnsap_nackPowerOffset, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Nack_Power_Offset },
  { &hf_rnsap_hsscch_PowerOffset, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_HSSCCH_PowerOffset },
  { &hf_rnsap_hSSCCH_CodeChangeGrant, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_HSSCCH_Code_Change_Grant },
  { &hf_rnsap_tDDAckNackPowerOffset, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TDD_AckNack_Power_Offset },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_HSDSCH_Information_to_Modify(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_HSDSCH_Information_to_Modify, HSDSCH_Information_to_Modify_sequence);

  return offset;
}
static int dissect_id_HSDSCH_Information_to_Modify(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSDSCH_Information_to_Modify(tvb, offset, actx, tree, hf_rnsap_id_HSDSCH_Information_to_Modify);
}


static const per_sequence_t HSDSCHMacdFlowSpecificInformationList_RL_PreemptRequiredInd_sequence_of[1] = {
  { &hf_rnsap_HSDSCHMacdFlowSpecificInformationList_RL_PreemptRequiredInd_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_HSDSCHMacdFlowSpecificInformationList_RL_PreemptRequiredInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_HSDSCHMacdFlowSpecificInformationList_RL_PreemptRequiredInd, HSDSCHMacdFlowSpecificInformationList_RL_PreemptRequiredInd_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_id_HSDSCHMacdFlowSpecificInformationList_RL_PreemptRequiredInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSDSCHMacdFlowSpecificInformationList_RL_PreemptRequiredInd(tvb, offset, actx, tree, hf_rnsap_id_HSDSCHMacdFlowSpecificInformationList_RL_PreemptRequiredInd);
}


static const per_sequence_t HSDSCHMacdFlowSpecificInformationItem_RL_PreemptRequiredInd_sequence[] = {
  { &hf_rnsap_hSDSCH_MACdFlow_ID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_HSDSCH_MACdFlow_ID },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_HSDSCHMacdFlowSpecificInformationItem_RL_PreemptRequiredInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_HSDSCHMacdFlowSpecificInformationItem_RL_PreemptRequiredInd, HSDSCHMacdFlowSpecificInformationItem_RL_PreemptRequiredInd_sequence);

  return offset;
}
static int dissect_id_HSDSCHMacdFlowSpecificInformationItem_RL_PreemptRequiredInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSDSCHMacdFlowSpecificInformationItem_RL_PreemptRequiredInd(tvb, offset, actx, tree, hf_rnsap_id_HSDSCHMacdFlowSpecificInformationItem_RL_PreemptRequiredInd);
}



static int
dissect_rnsap_HSDSCH_RNTI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 65535U, NULL, FALSE);

  return offset;
}
static int dissect_id_HSDSCH_RNTI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSDSCH_RNTI(tvb, offset, actx, tree, hf_rnsap_id_HSDSCH_RNTI);
}


static const per_sequence_t HSDSCH_TDD_Information_sequence[] = {
  { &hf_rnsap_hSDSCH_MACdFlows_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_HSDSCH_MACdFlows_Information },
  { &hf_rnsap_uE_Capabilities_Info, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UE_Capabilities_Info },
  { &hf_rnsap_mAChs_Reordering_Buffer_Size_for_RLC_UM, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MAChsReorderingBufferSize_for_RLC_UM },
  { &hf_rnsap_tDD_AckNack_Power_Offset, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_AckNack_Power_Offset },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_HSDSCH_TDD_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_HSDSCH_TDD_Information, HSDSCH_TDD_Information_sequence);

  return offset;
}
static int dissect_id_HSDSCH_TDD_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSDSCH_TDD_Information(tvb, offset, actx, tree, hf_rnsap_id_HSDSCH_TDD_Information);
}



static int
dissect_rnsap_HS_SICH_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 31U, NULL, FALSE);

  return offset;
}
static int dissect_id_HSSICH_Info_DM_Rprt(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HS_SICH_ID(tvb, offset, actx, tree, hf_rnsap_id_HSSICH_Info_DM_Rprt);
}
static int dissect_id_HSSICH_Info_DM(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HS_SICH_ID(tvb, offset, actx, tree, hf_rnsap_id_HSSICH_Info_DM);
}
static int dissect_HSSICH_Info_DM_Rqst_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HS_SICH_ID(tvb, offset, actx, tree, hf_rnsap_HSSICH_Info_DM_Rqst_item);
}
static int dissect_hsSICH_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HS_SICH_ID(tvb, offset, actx, tree, hf_rnsap_hsSICH_ID);
}


static const per_sequence_t HSSICH_Info_sequence[] = {
  { &hf_rnsap_hsSICH_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_HS_SICH_ID },
  { &hf_rnsap_timeslot      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TimeSlot },
  { &hf_rnsap_midambleShiftAndBurstType, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MidambleShiftAndBurstType },
  { &hf_rnsap_tDD_ChannelisationCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_ChannelisationCode },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_HSSICH_Info(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_HSSICH_Info, HSSICH_Info_sequence);

  return offset;
}
static int dissect_hSSICH_Info(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSSICH_Info(tvb, offset, actx, tree, hf_rnsap_hSSICH_Info);
}


static const per_sequence_t HSSCCH_TDD_Specific_InfoItem_Response_sequence[] = {
  { &hf_rnsap_timeslot      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TimeSlot },
  { &hf_rnsap_midambleShiftAndBurstType, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MidambleShiftAndBurstType },
  { &hf_rnsap_tDD_ChannelisationCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_ChannelisationCode },
  { &hf_rnsap_hSSICH_Info   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_HSSICH_Info },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_HSSCCH_TDD_Specific_InfoItem_Response(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_HSSCCH_TDD_Specific_InfoItem_Response, HSSCCH_TDD_Specific_InfoItem_Response_sequence);

  return offset;
}
static int dissect_HSSCCH_TDD_Specific_InfoList_Response_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSSCCH_TDD_Specific_InfoItem_Response(tvb, offset, actx, tree, hf_rnsap_HSSCCH_TDD_Specific_InfoList_Response_item);
}


static const per_sequence_t HSSCCH_TDD_Specific_InfoList_Response_sequence_of[1] = {
  { &hf_rnsap_HSSCCH_TDD_Specific_InfoList_Response_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_HSSCCH_TDD_Specific_InfoItem_Response },
};

static int
dissect_rnsap_HSSCCH_TDD_Specific_InfoList_Response(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_HSSCCH_TDD_Specific_InfoList_Response, HSSCCH_TDD_Specific_InfoList_Response_sequence_of,
                                                  0, 4);

  return offset;
}
static int dissect_hSSCCH_TDD_Specific_InfoList_Response(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSSCCH_TDD_Specific_InfoList_Response(tvb, offset, actx, tree, hf_rnsap_hSSCCH_TDD_Specific_InfoList_Response);
}


static const per_sequence_t HSSICH_InfoLCR_sequence[] = {
  { &hf_rnsap_hsSICH_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_HS_SICH_ID },
  { &hf_rnsap_timeslotLCR   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TimeSlotLCR },
  { &hf_rnsap_midambleShiftLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MidambleShiftLCR },
  { &hf_rnsap_tDD_ChannelisationCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_ChannelisationCode },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_HSSICH_InfoLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_HSSICH_InfoLCR, HSSICH_InfoLCR_sequence);

  return offset;
}
static int dissect_hSSICH_InfoLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSSICH_InfoLCR(tvb, offset, actx, tree, hf_rnsap_hSSICH_InfoLCR);
}


static const per_sequence_t HSSCCH_TDD_Specific_InfoItem_Response_LCR_sequence[] = {
  { &hf_rnsap_timeslotLCR   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TimeSlotLCR },
  { &hf_rnsap_midambleShiftLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MidambleShiftLCR },
  { &hf_rnsap_first_TDD_ChannelisationCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_ChannelisationCode },
  { &hf_rnsap_second_TDD_ChannelisationCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_ChannelisationCode },
  { &hf_rnsap_hSSICH_InfoLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_HSSICH_InfoLCR },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_HSSCCH_TDD_Specific_InfoItem_Response_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_HSSCCH_TDD_Specific_InfoItem_Response_LCR, HSSCCH_TDD_Specific_InfoItem_Response_LCR_sequence);

  return offset;
}
static int dissect_HSSCCH_TDD_Specific_InfoList_Response_LCR_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSSCCH_TDD_Specific_InfoItem_Response_LCR(tvb, offset, actx, tree, hf_rnsap_HSSCCH_TDD_Specific_InfoList_Response_LCR_item);
}


static const per_sequence_t HSSCCH_TDD_Specific_InfoList_Response_LCR_sequence_of[1] = {
  { &hf_rnsap_HSSCCH_TDD_Specific_InfoList_Response_LCR_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_HSSCCH_TDD_Specific_InfoItem_Response_LCR },
};

static int
dissect_rnsap_HSSCCH_TDD_Specific_InfoList_Response_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_HSSCCH_TDD_Specific_InfoList_Response_LCR, HSSCCH_TDD_Specific_InfoList_Response_LCR_sequence_of,
                                                  0, 4);

  return offset;
}
static int dissect_hSSCCH_TDD_Specific_InfoList_Response_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSSCCH_TDD_Specific_InfoList_Response_LCR(tvb, offset, actx, tree, hf_rnsap_hSSCCH_TDD_Specific_InfoList_Response_LCR);
}


static const per_sequence_t HSPDSCH_TDD_Specific_InfoItem_Response_sequence[] = {
  { &hf_rnsap_timeslot      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TimeSlot },
  { &hf_rnsap_midambleShiftAndBurstType, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MidambleShiftAndBurstType },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_HSPDSCH_TDD_Specific_InfoItem_Response(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_HSPDSCH_TDD_Specific_InfoItem_Response, HSPDSCH_TDD_Specific_InfoItem_Response_sequence);

  return offset;
}
static int dissect_HSPDSCH_TDD_Specific_InfoList_Response_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSPDSCH_TDD_Specific_InfoItem_Response(tvb, offset, actx, tree, hf_rnsap_HSPDSCH_TDD_Specific_InfoList_Response_item);
}


static const per_sequence_t HSPDSCH_TDD_Specific_InfoList_Response_sequence_of[1] = {
  { &hf_rnsap_HSPDSCH_TDD_Specific_InfoList_Response_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_HSPDSCH_TDD_Specific_InfoItem_Response },
};

static int
dissect_rnsap_HSPDSCH_TDD_Specific_InfoList_Response(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_HSPDSCH_TDD_Specific_InfoList_Response, HSPDSCH_TDD_Specific_InfoList_Response_sequence_of,
                                                  0, 15);

  return offset;
}
static int dissect_hSPDSCH_TDD_Specific_InfoList_Response(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSPDSCH_TDD_Specific_InfoList_Response(tvb, offset, actx, tree, hf_rnsap_hSPDSCH_TDD_Specific_InfoList_Response);
}


static const per_sequence_t HSPDSCH_TDD_Specific_InfoItem_Response_LCR_sequence[] = {
  { &hf_rnsap_timeslotLCR   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TimeSlotLCR },
  { &hf_rnsap_midambleShiftLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MidambleShiftLCR },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_HSPDSCH_TDD_Specific_InfoItem_Response_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_HSPDSCH_TDD_Specific_InfoItem_Response_LCR, HSPDSCH_TDD_Specific_InfoItem_Response_LCR_sequence);

  return offset;
}
static int dissect_HSPDSCH_TDD_Specific_InfoList_Response_LCR_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSPDSCH_TDD_Specific_InfoItem_Response_LCR(tvb, offset, actx, tree, hf_rnsap_HSPDSCH_TDD_Specific_InfoList_Response_LCR_item);
}


static const per_sequence_t HSPDSCH_TDD_Specific_InfoList_Response_LCR_sequence_of[1] = {
  { &hf_rnsap_HSPDSCH_TDD_Specific_InfoList_Response_LCR_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_HSPDSCH_TDD_Specific_InfoItem_Response_LCR },
};

static int
dissect_rnsap_HSPDSCH_TDD_Specific_InfoList_Response_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_HSPDSCH_TDD_Specific_InfoList_Response_LCR, HSPDSCH_TDD_Specific_InfoList_Response_LCR_sequence_of,
                                                  1, 6);

  return offset;
}
static int dissect_hSPDSCH_TDD_Specific_InfoList_Response_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSPDSCH_TDD_Specific_InfoList_Response_LCR(tvb, offset, actx, tree, hf_rnsap_hSPDSCH_TDD_Specific_InfoList_Response_LCR);
}


static const per_sequence_t HSDSCH_TDD_Information_Response_sequence[] = {
  { &hf_rnsap_hSDSCH_MACdFlow_Specific_InfoList_Response, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_HSDSCH_MACdFlow_Specific_InfoList_Response },
  { &hf_rnsap_hSSCCH_TDD_Specific_InfoList_Response, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_HSSCCH_TDD_Specific_InfoList_Response },
  { &hf_rnsap_hSSCCH_TDD_Specific_InfoList_Response_LCR, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_HSSCCH_TDD_Specific_InfoList_Response_LCR },
  { &hf_rnsap_hSPDSCH_TDD_Specific_InfoList_Response, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_HSPDSCH_TDD_Specific_InfoList_Response },
  { &hf_rnsap_hSPDSCH_TDD_Specific_InfoList_Response_LCR, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_HSPDSCH_TDD_Specific_InfoList_Response_LCR },
  { &hf_rnsap_hARQ_MemoryPartitioning, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_HARQ_MemoryPartitioning },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_HSDSCH_TDD_Information_Response(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_HSDSCH_TDD_Information_Response, HSDSCH_TDD_Information_Response_sequence);

  return offset;
}
static int dissect_id_HSDSCH_TDD_Information_Response(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSDSCH_TDD_Information_Response(tvb, offset, actx, tree, hf_rnsap_id_HSDSCH_TDD_Information_Response);
}


static const per_sequence_t HSDSCH_TDD_Update_Information_sequence[] = {
  { &hf_rnsap_hsSCCHCodeChangeIndicator, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_HSSCCH_CodeChangeIndicator },
  { &hf_rnsap_tDDAckNackPowerOffset, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TDD_AckNack_Power_Offset },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_HSDSCH_TDD_Update_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_HSDSCH_TDD_Update_Information, HSDSCH_TDD_Update_Information_sequence);

  return offset;
}
static int dissect_id_HSDSCH_TDD_Update_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSDSCH_TDD_Update_Information(tvb, offset, actx, tree, hf_rnsap_id_HSDSCH_TDD_Update_Information);
}


static const per_sequence_t HSDSCH_MACdFlows_to_Delete_Item_sequence[] = {
  { &hf_rnsap_hsDSCH_MACdFlow_ID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_HSDSCH_MACdFlow_ID },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_HSDSCH_MACdFlows_to_Delete_Item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_HSDSCH_MACdFlows_to_Delete_Item, HSDSCH_MACdFlows_to_Delete_Item_sequence);

  return offset;
}
static int dissect_HSDSCH_MACdFlows_to_Delete_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSDSCH_MACdFlows_to_Delete_Item(tvb, offset, actx, tree, hf_rnsap_HSDSCH_MACdFlows_to_Delete_item);
}


static const per_sequence_t HSDSCH_MACdFlows_to_Delete_sequence_of[1] = {
  { &hf_rnsap_HSDSCH_MACdFlows_to_Delete_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_HSDSCH_MACdFlows_to_Delete_Item },
};

static int
dissect_rnsap_HSDSCH_MACdFlows_to_Delete(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_HSDSCH_MACdFlows_to_Delete, HSDSCH_MACdFlows_to_Delete_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_id_HSDSCH_MACdFlows_to_Delete(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSDSCH_MACdFlows_to_Delete(tvb, offset, actx, tree, hf_rnsap_id_HSDSCH_MACdFlows_to_Delete);
}



static int
dissect_rnsap_AOA_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 719U, NULL, FALSE);

  return offset;
}
static int dissect_aOA_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_AOA_LCR(tvb, offset, actx, tree, hf_rnsap_aOA_LCR);
}


static const value_string rnsap_AOA_LCR_Accuracy_Class_vals[] = {
  {   0, "a" },
  {   1, "b" },
  {   2, "c" },
  {   3, "d" },
  {   4, "e" },
  {   5, "f" },
  {   6, "g" },
  {   7, "h" },
  { 0, NULL }
};


static int
dissect_rnsap_AOA_LCR_Accuracy_Class(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_aOA_LCR_Accuracy_Class(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_AOA_LCR_Accuracy_Class(tvb, offset, actx, tree, hf_rnsap_aOA_LCR_Accuracy_Class);
}


static const per_sequence_t Angle_Of_Arrival_Value_LCR_sequence[] = {
  { &hf_rnsap_aOA_LCR       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_AOA_LCR },
  { &hf_rnsap_aOA_LCR_Accuracy_Class, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_AOA_LCR_Accuracy_Class },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_Angle_Of_Arrival_Value_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_Angle_Of_Arrival_Value_LCR, Angle_Of_Arrival_Value_LCR_sequence);

  return offset;
}
static int dissect_id_Angle_Of_Arrival_Value_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Angle_Of_Arrival_Value_LCR(tvb, offset, actx, tree, hf_rnsap_id_Angle_Of_Arrival_Value_LCR);
}



static int
dissect_rnsap_GERAN_Cell_Capability(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     16, 16, FALSE, NULL);

  return offset;
}
static int dissect_id_GERAN_Cell_Capability(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GERAN_Cell_Capability(tvb, offset, actx, tree, hf_rnsap_id_GERAN_Cell_Capability);
}



static int
dissect_rnsap_GERAN_Classmark(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_octet_string(tvb, offset, actx, tree, hf_index,
                                       NO_BOUND, NO_BOUND, NULL);

  return offset;
}
static int dissect_id_GERAN_Classmark(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GERAN_Classmark(tvb, offset, actx, tree, hf_rnsap_id_GERAN_Classmark);
}



static int
dissect_rnsap_DSCH_InitialWindowSize(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 255U, NULL, FALSE);

  return offset;
}
static int dissect_id_DSCH_InitialWindowSize(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DSCH_InitialWindowSize(tvb, offset, actx, tree, hf_rnsap_id_DSCH_InitialWindowSize);
}



static int
dissect_rnsap_UL_Synchronisation_StepSize(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 8U, NULL, FALSE);

  return offset;
}
static int dissect_uL_Synchronisation_StepSize(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_Synchronisation_StepSize(tvb, offset, actx, tree, hf_rnsap_uL_Synchronisation_StepSize);
}



static int
dissect_rnsap_UL_Synchronisation_Frequency(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 8U, NULL, FALSE);

  return offset;
}
static int dissect_uL_Synchronisation_Frequency(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_Synchronisation_Frequency(tvb, offset, actx, tree, hf_rnsap_uL_Synchronisation_Frequency);
}


static const per_sequence_t UL_Synchronisation_Parameters_LCR_sequence[] = {
  { &hf_rnsap_uL_Synchronisation_StepSize, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_Synchronisation_StepSize },
  { &hf_rnsap_uL_Synchronisation_Frequency, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UL_Synchronisation_Frequency },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UL_Synchronisation_Parameters_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UL_Synchronisation_Parameters_LCR, UL_Synchronisation_Parameters_LCR_sequence);

  return offset;
}
static int dissect_id_UL_Synchronisation_Parameters_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_Synchronisation_Parameters_LCR(tvb, offset, actx, tree, hf_rnsap_id_UL_Synchronisation_Parameters_LCR);
}



static int
dissect_rnsap_SNACode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 65535U, NULL, FALSE);

  return offset;
}
static int dissect_ListOfSNAs_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SNACode(tvb, offset, actx, tree, hf_rnsap_ListOfSNAs_item);
}


static const per_sequence_t ListOfSNAs_sequence_of[1] = {
  { &hf_rnsap_ListOfSNAs_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_SNACode },
};

static int
dissect_rnsap_ListOfSNAs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_ListOfSNAs, ListOfSNAs_sequence_of,
                                                  1, 65536);

  return offset;
}
static int dissect_listOfSNAs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ListOfSNAs(tvb, offset, actx, tree, hf_rnsap_listOfSNAs);
}


static const per_sequence_t SNA_Information_sequence[] = {
  { &hf_rnsap_pLMN_Identity , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PLMN_Identity },
  { &hf_rnsap_listOfSNAs    , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ListOfSNAs },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_SNA_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_SNA_Information, SNA_Information_sequence);

  return offset;
}
static int dissect_id_SNA_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SNA_Information(tvb, offset, actx, tree, hf_rnsap_id_SNA_Information);
}


static const value_string rnsap_MAChs_ResetIndicator_vals[] = {
  {   0, "mAChs-NotReset" },
  { 0, NULL }
};


static int
dissect_rnsap_MAChs_ResetIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_MAChs_ResetIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MAChs_ResetIndicator(tvb, offset, actx, tree, hf_rnsap_id_MAChs_ResetIndicator);
}


static const value_string rnsap_TDD_TPC_UplinkStepSize_LCR_vals[] = {
  {   0, "step-size1" },
  {   1, "step-size2" },
  {   2, "step-size3" },
  { 0, NULL }
};


static int
dissect_rnsap_TDD_TPC_UplinkStepSize_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_id_TDD_TPC_UplinkStepSize_LCR_RL_SetupRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_TPC_UplinkStepSize_LCR(tvb, offset, actx, tree, hf_rnsap_id_TDD_TPC_UplinkStepSize_LCR_RL_SetupRqstTDD);
}
static int dissect_id_TDD_TPC_UplinkStepSize_InformationAdd_LCR_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_TPC_UplinkStepSize_LCR(tvb, offset, actx, tree, hf_rnsap_id_TDD_TPC_UplinkStepSize_InformationAdd_LCR_RL_ReconfPrepTDD);
}
static int dissect_id_TDD_TPC_UplinkStepSize_InformationModify_LCR_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_TPC_UplinkStepSize_LCR(tvb, offset, actx, tree, hf_rnsap_id_TDD_TPC_UplinkStepSize_InformationModify_LCR_RL_ReconfPrepTDD);
}
static int dissect_uplinkStepSizeLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TDD_TPC_UplinkStepSize_LCR(tvb, offset, actx, tree, hf_rnsap_uplinkStepSizeLCR);
}


static const per_sequence_t UL_CCTrCH_InformationList_RL_AdditionRqstTDD_sequence_of[1] = {
  { &hf_rnsap_UL_CCTrCH_InformationList_RL_AdditionRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_UL_CCTrCH_InformationList_RL_AdditionRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_UL_CCTrCH_InformationList_RL_AdditionRqstTDD, UL_CCTrCH_InformationList_RL_AdditionRqstTDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_UL_CCTrCH_InformationList_RL_AdditionRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_CCTrCH_InformationList_RL_AdditionRqstTDD(tvb, offset, actx, tree, hf_rnsap_id_UL_CCTrCH_InformationList_RL_AdditionRqstTDD);
}


static const per_sequence_t UL_CCTrCH_InformationItem_RL_AdditionRqstTDD_sequence[] = {
  { &hf_rnsap_cCTrCH_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CCTrCH_ID },
  { &hf_rnsap_uplinkStepSizeLCR, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TDD_TPC_UplinkStepSize_LCR },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UL_CCTrCH_InformationItem_RL_AdditionRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UL_CCTrCH_InformationItem_RL_AdditionRqstTDD, UL_CCTrCH_InformationItem_RL_AdditionRqstTDD_sequence);

  return offset;
}
static int dissect_id_UL_CCTrCH_InformationItem_RL_AdditionRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_CCTrCH_InformationItem_RL_AdditionRqstTDD(tvb, offset, actx, tree, hf_rnsap_id_UL_CCTrCH_InformationItem_RL_AdditionRqstTDD);
}


static const per_sequence_t DL_CCTrCH_InformationList_RL_AdditionRqstTDD_sequence_of[1] = {
  { &hf_rnsap_DL_CCTrCH_InformationList_RL_AdditionRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_DL_CCTrCH_InformationList_RL_AdditionRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DL_CCTrCH_InformationList_RL_AdditionRqstTDD, DL_CCTrCH_InformationList_RL_AdditionRqstTDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_DL_CCTrCH_InformationList_RL_AdditionRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_CCTrCH_InformationList_RL_AdditionRqstTDD(tvb, offset, actx, tree, hf_rnsap_id_DL_CCTrCH_InformationList_RL_AdditionRqstTDD);
}


static const per_sequence_t DL_CCTrCH_InformationItem_RL_AdditionRqstTDD_sequence[] = {
  { &hf_rnsap_cCTrCH_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CCTrCH_ID },
  { &hf_rnsap_downlinkStepSize, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TDD_TPC_DownlinkStepSize },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_CCTrCH_InformationItem_RL_AdditionRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_CCTrCH_InformationItem_RL_AdditionRqstTDD, DL_CCTrCH_InformationItem_RL_AdditionRqstTDD_sequence);

  return offset;
}
static int dissect_id_DL_CCTrCH_InformationItem_RL_AdditionRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_CCTrCH_InformationItem_RL_AdditionRqstTDD(tvb, offset, actx, tree, hf_rnsap_id_DL_CCTrCH_InformationItem_RL_AdditionRqstTDD);
}



static int
dissect_rnsap_INTEGER_1_4(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 4U, NULL, FALSE);

  return offset;
}
static int dissect_wT(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_1_4(tvb, offset, actx, tree, hf_rnsap_wT);
}


static const per_sequence_t FPACH_Information_sequence[] = {
  { &hf_rnsap_timeSlotLCR   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TimeSlotLCR },
  { &hf_rnsap_tDD_ChannelisationCodeLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TDD_ChannelisationCodeLCR },
  { &hf_rnsap_midambleShiftLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MidambleShiftLCR },
  { &hf_rnsap_wT            , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_INTEGER_1_4 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_FPACH_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_FPACH_Information, FPACH_Information_sequence);

  return offset;
}
static int dissect_fPACH_info(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_FPACH_Information(tvb, offset, actx, tree, hf_rnsap_fPACH_info);
}



static int
dissect_rnsap_INTEGER_M120_M58_(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              -120, -58, NULL, TRUE);

  return offset;
}
static int dissect_prxUpPCHdes(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_M120_M58_(tvb, offset, actx, tree, hf_rnsap_prxUpPCHdes);
}


static const value_string rnsap_T_maxSYNC_UL_transmissions_vals[] = {
  {   0, "v1" },
  {   1, "v2" },
  {   2, "v4" },
  {   3, "v8" },
  { 0, NULL }
};


static int
dissect_rnsap_T_maxSYNC_UL_transmissions(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     4, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_maxSYNC_UL_transmissions(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_T_maxSYNC_UL_transmissions(tvb, offset, actx, tree, hf_rnsap_maxSYNC_UL_transmissions);
}



static int
dissect_rnsap_INTEGER_0_3_(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 3U, NULL, TRUE);

  return offset;
}
static int dissect_powerRampStep(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_0_3_(tvb, offset, actx, tree, hf_rnsap_powerRampStep);
}


static const per_sequence_t SYNC_UL_ProcParameters_sequence[] = {
  { &hf_rnsap_maxSYNC_UL_transmissions, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_T_maxSYNC_UL_transmissions },
  { &hf_rnsap_powerRampStep , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_INTEGER_0_3_ },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_SYNC_UL_ProcParameters(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_SYNC_UL_ProcParameters, SYNC_UL_ProcParameters_sequence);

  return offset;
}
static int dissect_syncUL_procParameter(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SYNC_UL_ProcParameters(tvb, offset, actx, tree, hf_rnsap_syncUL_procParameter);
}



static int
dissect_rnsap_INTEGER_1_32(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 32U, NULL, FALSE);

  return offset;
}
static int dissect_mMax(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_1_32(tvb, offset, actx, tree, hf_rnsap_mMax);
}


static const per_sequence_t UL_TimingAdvanceCtrl_LCR_sequence[] = {
  { &hf_rnsap_sync_UL_codes_bitmap, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_8 },
  { &hf_rnsap_fPACH_info    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_FPACH_Information },
  { &hf_rnsap_prxUpPCHdes   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_INTEGER_M120_M58_ },
  { &hf_rnsap_syncUL_procParameter, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SYNC_UL_ProcParameters },
  { &hf_rnsap_mMax          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_INTEGER_1_32 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UL_TimingAdvanceCtrl_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UL_TimingAdvanceCtrl_LCR, UL_TimingAdvanceCtrl_LCR_sequence);

  return offset;
}
static int dissect_id_UL_TimingAdvanceCtrl_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_TimingAdvanceCtrl_LCR(tvb, offset, actx, tree, hf_rnsap_id_UL_TimingAdvanceCtrl_LCR);
}


static const per_sequence_t HSPDSCH_Timeslot_InformationItem_PhyChReconfRqstTDD_sequence[] = {
  { &hf_rnsap_timeslot      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TimeSlot },
  { &hf_rnsap_midambleShiftAndBurstType, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MidambleShiftAndBurstType },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_HSPDSCH_Timeslot_InformationItem_PhyChReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_HSPDSCH_Timeslot_InformationItem_PhyChReconfRqstTDD, HSPDSCH_Timeslot_InformationItem_PhyChReconfRqstTDD_sequence);

  return offset;
}
static int dissect_HSPDSCH_Timeslot_InformationList_PhyChReconfRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSPDSCH_Timeslot_InformationItem_PhyChReconfRqstTDD(tvb, offset, actx, tree, hf_rnsap_HSPDSCH_Timeslot_InformationList_PhyChReconfRqstTDD_item);
}


static const per_sequence_t HSPDSCH_Timeslot_InformationList_PhyChReconfRqstTDD_sequence_of[1] = {
  { &hf_rnsap_HSPDSCH_Timeslot_InformationList_PhyChReconfRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_HSPDSCH_Timeslot_InformationItem_PhyChReconfRqstTDD },
};

static int
dissect_rnsap_HSPDSCH_Timeslot_InformationList_PhyChReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_HSPDSCH_Timeslot_InformationList_PhyChReconfRqstTDD, HSPDSCH_Timeslot_InformationList_PhyChReconfRqstTDD_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_id_HSPDSCH_Timeslot_InformationList_PhyChReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSPDSCH_Timeslot_InformationList_PhyChReconfRqstTDD(tvb, offset, actx, tree, hf_rnsap_id_HSPDSCH_Timeslot_InformationList_PhyChReconfRqstTDD);
}


static const per_sequence_t HSPDSCH_Timeslot_InformationItemLCR_PhyChReconfRqstTDD_sequence[] = {
  { &hf_rnsap_timeslotLCR   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TimeSlotLCR },
  { &hf_rnsap_midambleShiftLCR, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MidambleShiftLCR },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_HSPDSCH_Timeslot_InformationItemLCR_PhyChReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_HSPDSCH_Timeslot_InformationItemLCR_PhyChReconfRqstTDD, HSPDSCH_Timeslot_InformationItemLCR_PhyChReconfRqstTDD_sequence);

  return offset;
}
static int dissect_HSPDSCH_Timeslot_InformationListLCR_PhyChReconfRqstTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSPDSCH_Timeslot_InformationItemLCR_PhyChReconfRqstTDD(tvb, offset, actx, tree, hf_rnsap_HSPDSCH_Timeslot_InformationListLCR_PhyChReconfRqstTDD_item);
}


static const per_sequence_t HSPDSCH_Timeslot_InformationListLCR_PhyChReconfRqstTDD_sequence_of[1] = {
  { &hf_rnsap_HSPDSCH_Timeslot_InformationListLCR_PhyChReconfRqstTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_HSPDSCH_Timeslot_InformationItemLCR_PhyChReconfRqstTDD },
};

static int
dissect_rnsap_HSPDSCH_Timeslot_InformationListLCR_PhyChReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_HSPDSCH_Timeslot_InformationListLCR_PhyChReconfRqstTDD, HSPDSCH_Timeslot_InformationListLCR_PhyChReconfRqstTDD_sequence_of,
                                                  1, 6);

  return offset;
}
static int dissect_id_HSPDSCH_Timeslot_InformationListLCR_PhyChReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSPDSCH_Timeslot_InformationListLCR_PhyChReconfRqstTDD(tvb, offset, actx, tree, hf_rnsap_id_HSPDSCH_Timeslot_InformationListLCR_PhyChReconfRqstTDD);
}



static int
dissect_rnsap_HS_SICH_failed(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 20U, NULL, FALSE);

  return offset;
}
static int dissect_failed_HS_SICH(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HS_SICH_failed(tvb, offset, actx, tree, hf_rnsap_failed_HS_SICH);
}



static int
dissect_rnsap_HS_SICH_missed(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 20U, NULL, FALSE);

  return offset;
}
static int dissect_missed_HS_SICH(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HS_SICH_missed(tvb, offset, actx, tree, hf_rnsap_missed_HS_SICH);
}



static int
dissect_rnsap_HS_SICH_total(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 20U, NULL, FALSE);

  return offset;
}
static int dissect_total_HS_SICH(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HS_SICH_total(tvb, offset, actx, tree, hf_rnsap_total_HS_SICH);
}


static const per_sequence_t HS_SICH_Reception_Quality_Value_sequence[] = {
  { &hf_rnsap_failed_HS_SICH, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_HS_SICH_failed },
  { &hf_rnsap_missed_HS_SICH, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_HS_SICH_missed },
  { &hf_rnsap_total_HS_SICH , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_HS_SICH_total },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_HS_SICH_Reception_Quality_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_HS_SICH_Reception_Quality_Value, HS_SICH_Reception_Quality_Value_sequence);

  return offset;
}
static int dissect_id_HS_SICH_Reception_Quality(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HS_SICH_Reception_Quality_Value(tvb, offset, actx, tree, hf_rnsap_id_HS_SICH_Reception_Quality);
}



static int
dissect_rnsap_HS_SICH_Reception_Quality_Measurement_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 20U, NULL, FALSE);

  return offset;
}
static int dissect_id_HS_SICH_Reception_Quality_Measurement_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HS_SICH_Reception_Quality_Measurement_Value(tvb, offset, actx, tree, hf_rnsap_id_HS_SICH_Reception_Quality_Measurement_Value);
}


static const per_sequence_t HSSICH_Info_DM_Rqst_sequence_of[1] = {
  { &hf_rnsap_HSSICH_Info_DM_Rqst_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_HS_SICH_ID },
};

static int
dissect_rnsap_HSSICH_Info_DM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_HSSICH_Info_DM_Rqst, HSSICH_Info_DM_Rqst_sequence_of,
                                                  1, 4);

  return offset;
}
static int dissect_id_HSSICH_Info_DM_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSSICH_Info_DM_Rqst(tvb, offset, actx, tree, hf_rnsap_id_HSSICH_Info_DM_Rqst);
}



static int
dissect_rnsap_DL_DPCH_InformationModifyList_LCR_RL_ReconfRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_dl_DPCH_ModifyInformation_LCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_DPCH_InformationModifyList_LCR_RL_ReconfRspTDD(tvb, offset, actx, tree, hf_rnsap_dl_DPCH_ModifyInformation_LCR);
}


static const per_sequence_t DL_CCTrCH_InformationItem_RL_ReconfRspTDD_sequence[] = {
  { &hf_rnsap_cCTrCH_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CCTrCH_ID },
  { &hf_rnsap_dl_DPCH_ModifyInformation_LCR, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DL_DPCH_InformationModifyList_LCR_RL_ReconfRspTDD },
  { &hf_rnsap_cCTrCH_Maximum_DL_Power, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DL_Power },
  { &hf_rnsap_cCTrCH_Minimum_DL_Power, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DL_Power },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_CCTrCH_InformationItem_RL_ReconfRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_CCTrCH_InformationItem_RL_ReconfRspTDD, DL_CCTrCH_InformationItem_RL_ReconfRspTDD_sequence);

  return offset;
}
static int dissect_DL_CCTrCH_InformationList_RL_ReconfRspTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_CCTrCH_InformationItem_RL_ReconfRspTDD(tvb, offset, actx, tree, hf_rnsap_DL_CCTrCH_InformationList_RL_ReconfRspTDD_item);
}


static const per_sequence_t DL_CCTrCH_InformationList_RL_ReconfRspTDD_sequence_of[1] = {
  { &hf_rnsap_DL_CCTrCH_InformationList_RL_ReconfRspTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_CCTrCH_InformationItem_RL_ReconfRspTDD },
};

static int
dissect_rnsap_DL_CCTrCH_InformationList_RL_ReconfRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DL_CCTrCH_InformationList_RL_ReconfRspTDD, DL_CCTrCH_InformationList_RL_ReconfRspTDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_DL_CCTrCH_InformationList_RL_ReconfRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_CCTrCH_InformationList_RL_ReconfRspTDD(tvb, offset, actx, tree, hf_rnsap_id_DL_CCTrCH_InformationList_RL_ReconfRspTDD);
}


static const per_sequence_t DL_Timeslot_LCR_InformationModifyItem_RL_ReconfRspTDD_sequence[] = {
  { &hf_rnsap_timeSlotLCR   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TimeSlotLCR },
  { &hf_rnsap_maxPowerLCR   , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DL_Power },
  { &hf_rnsap_minPowerLCR   , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DL_Power },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_Timeslot_LCR_InformationModifyItem_RL_ReconfRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_Timeslot_LCR_InformationModifyItem_RL_ReconfRspTDD, DL_Timeslot_LCR_InformationModifyItem_RL_ReconfRspTDD_sequence);

  return offset;
}
static int dissect_DL_Timeslot_LCR_InformationModifyList_RL_ReconfRspTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_Timeslot_LCR_InformationModifyItem_RL_ReconfRspTDD(tvb, offset, actx, tree, hf_rnsap_DL_Timeslot_LCR_InformationModifyList_RL_ReconfRspTDD_item);
}


static const per_sequence_t DL_Timeslot_LCR_InformationModifyList_RL_ReconfRspTDD_sequence_of[1] = {
  { &hf_rnsap_DL_Timeslot_LCR_InformationModifyList_RL_ReconfRspTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_Timeslot_LCR_InformationModifyItem_RL_ReconfRspTDD },
};

static int
dissect_rnsap_DL_Timeslot_LCR_InformationModifyList_RL_ReconfRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DL_Timeslot_LCR_InformationModifyList_RL_ReconfRspTDD, DL_Timeslot_LCR_InformationModifyList_RL_ReconfRspTDD_sequence_of,
                                                  1, 6);

  return offset;
}
static int dissect_dL_Timeslot_LCR_InformationModifyList_RL_ReconfRqstTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_Timeslot_LCR_InformationModifyList_RL_ReconfRspTDD(tvb, offset, actx, tree, hf_rnsap_dL_Timeslot_LCR_InformationModifyList_RL_ReconfRqstTDD);
}


static const per_sequence_t DL_DPCH_InformationModifyItem_LCR_RL_ReconfRspTDD_sequence[] = {
  { &hf_rnsap_dL_Timeslot_LCR_InformationModifyList_RL_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DL_Timeslot_LCR_InformationModifyList_RL_ReconfRspTDD },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_DPCH_InformationModifyItem_LCR_RL_ReconfRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_DPCH_InformationModifyItem_LCR_RL_ReconfRspTDD, DL_DPCH_InformationModifyItem_LCR_RL_ReconfRspTDD_sequence);

  return offset;
}
static int dissect_id_DL_DPCH_InformationModifyItem_LCR_RL_ReconfRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_DPCH_InformationModifyItem_LCR_RL_ReconfRspTDD(tvb, offset, actx, tree, hf_rnsap_id_DL_DPCH_InformationModifyItem_LCR_RL_ReconfRspTDD);
}


static const value_string rnsap_Support_8PSK_vals[] = {
  {   0, "v8PSK-Supported" },
  { 0, NULL }
};


static int
dissect_rnsap_Support_8PSK(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_TDD_Support_8PSK(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Support_8PSK(tvb, offset, actx, tree, hf_rnsap_id_TDD_Support_8PSK);
}



static int
dissect_rnsap_MaxNrDLPhysicalchannelsTS(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 16U, NULL, FALSE);

  return offset;
}
static int dissect_id_TDD_maxNrDLPhysicalchannels(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MaxNrDLPhysicalchannelsTS(tvb, offset, actx, tree, hf_rnsap_id_TDD_maxNrDLPhysicalchannels);
}



static int
dissect_rnsap_ExtendedGSMCellIndividualOffset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              -50, 50U, NULL, FALSE);

  return offset;
}
static int dissect_id_ExtendedGSMCellIndividualOffset(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ExtendedGSMCellIndividualOffset(tvb, offset, actx, tree, hf_rnsap_id_ExtendedGSMCellIndividualOffset);
}


static const per_sequence_t RL_ParameterUpdateIndicationFDD_RL_InformationList_sequence_of[1] = {
  { &hf_rnsap_RL_ParameterUpdateIndicationFDD_RL_InformationList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_RL_ParameterUpdateIndicationFDD_RL_InformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_RL_ParameterUpdateIndicationFDD_RL_InformationList, RL_ParameterUpdateIndicationFDD_RL_InformationList_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_RL_ParameterUpdateIndicationFDD_RL_InformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_ParameterUpdateIndicationFDD_RL_InformationList(tvb, offset, actx, tree, hf_rnsap_id_RL_ParameterUpdateIndicationFDD_RL_InformationList);
}


static const value_string rnsap_Primary_CPICH_Usage_For_Channel_Estimation_vals[] = {
  {   0, "primary-CPICH-may-be-used" },
  {   1, "primary-CPICH-shall-not-be-used" },
  { 0, NULL }
};


static int
dissect_rnsap_Primary_CPICH_Usage_For_Channel_Estimation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_Primary_CPICH_Usage_For_Channel_Estimation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Primary_CPICH_Usage_For_Channel_Estimation(tvb, offset, actx, tree, hf_rnsap_id_Primary_CPICH_Usage_For_Channel_Estimation);
}


static const per_sequence_t Secondary_CPICH_Information_sequence[] = {
  { &hf_rnsap_dl_ScramblingCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_ScramblingCode },
  { &hf_rnsap_fDD_DL_ChannelisationCodeNumber, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_FDD_DL_ChannelisationCodeNumber },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_Secondary_CPICH_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_Secondary_CPICH_Information, Secondary_CPICH_Information_sequence);

  return offset;
}
static int dissect_id_Secondary_CPICH_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Secondary_CPICH_Information(tvb, offset, actx, tree, hf_rnsap_id_Secondary_CPICH_Information);
}
static int dissect_new_secondary_CPICH(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Secondary_CPICH_Information(tvb, offset, actx, tree, hf_rnsap_new_secondary_CPICH);
}


static const value_string rnsap_Secondary_CPICH_Information_Change_vals[] = {
  {   0, "new-secondary-CPICH" },
  {   1, "secondary-CPICH-shall-not-be-used" },
  { 0, NULL }
};

static const per_choice_t Secondary_CPICH_Information_Change_choice[] = {
  {   0, &hf_rnsap_new_secondary_CPICH, ASN1_EXTENSION_ROOT    , dissect_rnsap_Secondary_CPICH_Information },
  {   1, &hf_rnsap_secondary_CPICH_shall_not_be_used, ASN1_EXTENSION_ROOT    , dissect_rnsap_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_Secondary_CPICH_Information_Change(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_Secondary_CPICH_Information_Change, Secondary_CPICH_Information_Change_choice,
                                 NULL);

  return offset;
}
static int dissect_id_Secondary_CPICH_Information_Change(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Secondary_CPICH_Information_Change(tvb, offset, actx, tree, hf_rnsap_id_Secondary_CPICH_Information_Change);
}


static const value_string rnsap_Phase_Reference_Update_Indicator_vals[] = {
  {   0, "phase-reference-needs-to-be-changed" },
  { 0, NULL }
};


static int
dissect_rnsap_Phase_Reference_Update_Indicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_Phase_Reference_Update_Indicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Phase_Reference_Update_Indicator(tvb, offset, actx, tree, hf_rnsap_id_Phase_Reference_Update_Indicator);
}
static int dissect_phase_Reference_Update_Indicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Phase_Reference_Update_Indicator(tvb, offset, actx, tree, hf_rnsap_phase_Reference_Update_Indicator);
}


static const per_sequence_t RL_ParameterUpdateIndicationFDD_RL_Information_Item_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_phase_Reference_Update_Indicator, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Phase_Reference_Update_Indicator },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_ParameterUpdateIndicationFDD_RL_Information_Item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_ParameterUpdateIndicationFDD_RL_Information_Item, RL_ParameterUpdateIndicationFDD_RL_Information_Item_sequence);

  return offset;
}
static int dissect_id_RL_ParameterUpdateIndicationFDD_RL_Information_Item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_ParameterUpdateIndicationFDD_RL_Information_Item(tvb, offset, actx, tree, hf_rnsap_id_RL_ParameterUpdateIndicationFDD_RL_Information_Item);
}


static const value_string rnsap_Unidirectional_DCH_Indicator_vals[] = {
  {   0, "downlink-DCH-only" },
  {   1, "uplink-DCH-only" },
  { 0, NULL }
};


static int
dissect_rnsap_Unidirectional_DCH_Indicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_Unidirectional_DCH_Indicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Unidirectional_DCH_Indicator(tvb, offset, actx, tree, hf_rnsap_id_Unidirectional_DCH_Indicator);
}


static const per_sequence_t RL_InformationIE_RL_ReconfPrepTDD_sequence[] = {
  { &hf_rnsap_rL_ID         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_rL_Specific_DCH_Info, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_RL_Specific_DCH_Info },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_InformationIE_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_InformationIE_RL_ReconfPrepTDD, RL_InformationIE_RL_ReconfPrepTDD_sequence);

  return offset;
}
static int dissect_RL_Information_RL_ReconfPrepTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_InformationIE_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_rnsap_RL_Information_RL_ReconfPrepTDD_item);
}


static const per_sequence_t RL_Information_RL_ReconfPrepTDD_sequence_of[1] = {
  { &hf_rnsap_RL_Information_RL_ReconfPrepTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_InformationIE_RL_ReconfPrepTDD },
};

static int
dissect_rnsap_RL_Information_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_RL_Information_RL_ReconfPrepTDD, RL_Information_RL_ReconfPrepTDD_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_RL_Information_RL_ReconfPrepTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Information_RL_ReconfPrepTDD(tvb, offset, actx, tree, hf_rnsap_id_RL_Information_RL_ReconfPrepTDD);
}


static const per_sequence_t Multiple_RL_InformationResponse_RL_ReconfReadyTDD_sequence_of[1] = {
  { &hf_rnsap_Multiple_RL_InformationResponse_RL_ReconfReadyTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_InformationResponse_RL_ReconfReadyTDD },
};

static int
dissect_rnsap_Multiple_RL_InformationResponse_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_Multiple_RL_InformationResponse_RL_ReconfReadyTDD, Multiple_RL_InformationResponse_RL_ReconfReadyTDD_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_id_Multiple_RL_InformationResponse_RL_ReconfReadyTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Multiple_RL_InformationResponse_RL_ReconfReadyTDD(tvb, offset, actx, tree, hf_rnsap_id_Multiple_RL_InformationResponse_RL_ReconfReadyTDD);
}


static const per_sequence_t Multiple_RL_InformationResponse_RL_ReconfRspTDD_sequence_of[1] = {
  { &hf_rnsap_Multiple_RL_InformationResponse_RL_ReconfRspTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_InformationResponse_RL_ReconfRspTDD },
};

static int
dissect_rnsap_Multiple_RL_InformationResponse_RL_ReconfRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_Multiple_RL_InformationResponse_RL_ReconfRspTDD, Multiple_RL_InformationResponse_RL_ReconfRspTDD_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_id_RL_ReconfigurationResponseTDD_RL_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Multiple_RL_InformationResponse_RL_ReconfRspTDD(tvb, offset, actx, tree, hf_rnsap_id_RL_ReconfigurationResponseTDD_RL_Information);
}


static const per_sequence_t Satellite_Almanac_Information_ExtItem_item_sequence[] = {
  { &hf_rnsap_dATA_ID       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DATA_ID },
  { &hf_rnsap_sAT_ID        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SAT_ID },
  { &hf_rnsap_gps_e_alm     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_16 },
  { &hf_rnsap_gps_toa_alm   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_8 },
  { &hf_rnsap_gps_delta_I_alm, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_16 },
  { &hf_rnsap_omegadot_alm  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_16 },
  { &hf_rnsap_svhealth_alm  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_8 },
  { &hf_rnsap_gps_a_sqrt_alm, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_24 },
  { &hf_rnsap_omegazero_alm , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_24 },
  { &hf_rnsap_m_zero_alm    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_24 },
  { &hf_rnsap_gps_omega_alm , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_24 },
  { &hf_rnsap_gps_af_zero_alm, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_11 },
  { &hf_rnsap_gps_af_one_alm, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BIT_STRING_SIZE_11 },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_Satellite_Almanac_Information_ExtItem_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_Satellite_Almanac_Information_ExtItem_item, Satellite_Almanac_Information_ExtItem_item_sequence);

  return offset;
}
static int dissect_Satellite_Almanac_Information_ExtItem_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Satellite_Almanac_Information_ExtItem_item(tvb, offset, actx, tree, hf_rnsap_Satellite_Almanac_Information_ExtItem_item);
}


static const per_sequence_t Satellite_Almanac_Information_ExtItem_sequence_of[1] = {
  { &hf_rnsap_Satellite_Almanac_Information_ExtItem_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_Satellite_Almanac_Information_ExtItem_item },
};

static int
dissect_rnsap_Satellite_Almanac_Information_ExtItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_Satellite_Almanac_Information_ExtItem, Satellite_Almanac_Information_ExtItem_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_Satellite_Almanac_Information_ExtItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Satellite_Almanac_Information_ExtItem(tvb, offset, actx, tree, hf_rnsap_id_Satellite_Almanac_Information_ExtItem);
}


static const per_sequence_t PriorityQueue_InfoItem_to_Modify_Unsynchronised_sequence[] = {
  { &hf_rnsap_priorityQueueId, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PriorityQueue_Id },
  { &hf_rnsap_schedulingPriorityIndicator, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_SchedulingPriorityIndicator },
  { &hf_rnsap_discardTimer  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DiscardTimer },
  { &hf_rnsap_mAChsGuaranteedBitRate, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_MAChsGuaranteedBitRate },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_PriorityQueue_InfoItem_to_Modify_Unsynchronised(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_PriorityQueue_InfoItem_to_Modify_Unsynchronised, PriorityQueue_InfoItem_to_Modify_Unsynchronised_sequence);

  return offset;
}
static int dissect_PriorityQueue_InfoList_to_Modify_Unsynchronised_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PriorityQueue_InfoItem_to_Modify_Unsynchronised(tvb, offset, actx, tree, hf_rnsap_PriorityQueue_InfoList_to_Modify_Unsynchronised_item);
}


static const per_sequence_t PriorityQueue_InfoList_to_Modify_Unsynchronised_sequence_of[1] = {
  { &hf_rnsap_PriorityQueue_InfoList_to_Modify_Unsynchronised_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_PriorityQueue_InfoItem_to_Modify_Unsynchronised },
};

static int
dissect_rnsap_PriorityQueue_InfoList_to_Modify_Unsynchronised(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_PriorityQueue_InfoList_to_Modify_Unsynchronised, PriorityQueue_InfoList_to_Modify_Unsynchronised_sequence_of,
                                                  0, 8);

  return offset;
}
static int dissect_priorityQueueInfotoModifyUnsynchronised(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PriorityQueue_InfoList_to_Modify_Unsynchronised(tvb, offset, actx, tree, hf_rnsap_priorityQueueInfotoModifyUnsynchronised);
}


static const per_sequence_t HSDSCH_Information_to_Modify_Unsynchronised_sequence[] = {
  { &hf_rnsap_hSDSCH_MACdFlow_Specific_InfoList_to_Modify, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_HSDSCH_MACdFlow_Specific_InfoList_to_Modify },
  { &hf_rnsap_priorityQueueInfotoModifyUnsynchronised, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_PriorityQueue_InfoList_to_Modify_Unsynchronised },
  { &hf_rnsap_cqiPowerOffset, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_CQI_Power_Offset },
  { &hf_rnsap_ackPowerOffset, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Ack_Power_Offset },
  { &hf_rnsap_nackPowerOffset, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Nack_Power_Offset },
  { &hf_rnsap_hsscch_PowerOffset, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_HSSCCH_PowerOffset },
  { &hf_rnsap_tDDAckNackPowerOffset, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TDD_AckNack_Power_Offset },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_HSDSCH_Information_to_Modify_Unsynchronised(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_HSDSCH_Information_to_Modify_Unsynchronised, HSDSCH_Information_to_Modify_Unsynchronised_sequence);

  return offset;
}
static int dissect_id_HSDSCH_Information_to_Modify_Unsynchronised(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HSDSCH_Information_to_Modify_Unsynchronised(tvb, offset, actx, tree, hf_rnsap_id_HSDSCH_Information_to_Modify_Unsynchronised);
}



static int
dissect_rnsap_DsField(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     8, 8, FALSE, NULL);

  return offset;
}
static int dissect_dsField(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DsField(tvb, offset, actx, tree, hf_rnsap_dsField);
}



static int
dissect_rnsap_GenericTrafficCategory(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     8, 8, FALSE, NULL);

  return offset;
}
static int dissect_genericTrafficCategory(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GenericTrafficCategory(tvb, offset, actx, tree, hf_rnsap_genericTrafficCategory);
}


static const value_string rnsap_TnlQos_vals[] = {
  {   0, "dsField" },
  {   1, "genericTrafficCategory" },
  { 0, NULL }
};

static const per_choice_t TnlQos_choice[] = {
  {   0, &hf_rnsap_dsField       , ASN1_EXTENSION_ROOT    , dissect_rnsap_DsField },
  {   1, &hf_rnsap_genericTrafficCategory, ASN1_EXTENSION_ROOT    , dissect_rnsap_GenericTrafficCategory },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_TnlQos(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_TnlQos, TnlQos_choice,
                                 NULL);

  return offset;
}
static int dissect_id_TnlQos(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TnlQos(tvb, offset, actx, tree, hf_rnsap_id_TnlQos);
}
static int dissect_tnlQoS(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TnlQos(tvb, offset, actx, tree, hf_rnsap_tnlQoS);
}


static const per_sequence_t RTLoadValue_sequence[] = {
  { &hf_rnsap_uplinkRTLoadValue, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_INTEGER_0_100 },
  { &hf_rnsap_downlinkRTLoadValue, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_INTEGER_0_100 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RTLoadValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RTLoadValue, RTLoadValue_sequence);

  return offset;
}
static int dissect_id_RTLoadValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RTLoadValue(tvb, offset, actx, tree, hf_rnsap_id_RTLoadValue);
}


static const per_sequence_t NRTLoadInformationValue_sequence[] = {
  { &hf_rnsap_uplinkNRTLoadInformationValue, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_INTEGER_0_3 },
  { &hf_rnsap_downlinkNRTLoadInformationValue, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_INTEGER_0_3 },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_NRTLoadInformationValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_NRTLoadInformationValue, NRTLoadInformationValue_sequence);

  return offset;
}
static int dissect_id_NRTLoadInformationValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_NRTLoadInformationValue(tvb, offset, actx, tree, hf_rnsap_id_NRTLoadInformationValue);
}



static int
dissect_rnsap_CellPortionID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 63U, NULL, TRUE);

  return offset;
}
static int dissect_id_CellPortionID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CellPortionID(tvb, offset, actx, tree, hf_rnsap_id_CellPortionID);
}



static int
dissect_rnsap_UpPTSInterferenceValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 127U, NULL, TRUE);

  return offset;
}
static int dissect_id_UpPTSInterferenceValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UpPTSInterferenceValue(tvb, offset, actx, tree, hf_rnsap_id_UpPTSInterferenceValue);
}



static int
dissect_rnsap_PrimaryCCPCH_RSCP_Delta(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              -5, -1, NULL, TRUE);

  return offset;
}
static int dissect_id_PrimaryCCPCH_RSCP_Delta(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PrimaryCCPCH_RSCP_Delta(tvb, offset, actx, tree, hf_rnsap_id_PrimaryCCPCH_RSCP_Delta);
}
static int dissect_primaryCCPCH_RSCP_Delta(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PrimaryCCPCH_RSCP_Delta(tvb, offset, actx, tree, hf_rnsap_primaryCCPCH_RSCP_Delta);
}


static const value_string rnsap_UEMeasurementType_vals[] = {
  {   0, "primary-CCPCH-RSCP" },
  {   1, "dL-Timeslot-ISCP" },
  {   2, "uE-Transmitted-power" },
  { 0, NULL }
};


static int
dissect_rnsap_UEMeasurementType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_id_UEMeasurementType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEMeasurementType(tvb, offset, actx, tree, hf_rnsap_id_UEMeasurementType);
}


static const value_string rnsap_UEMeasurementTimeslotInfoHCRBurstType_vals[] = {
  {   0, "type1" },
  {   1, "type2" },
  {   2, "type3" },
  { 0, NULL }
};


static int
dissect_rnsap_UEMeasurementTimeslotInfoHCRBurstType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_burstType(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEMeasurementTimeslotInfoHCRBurstType(tvb, offset, actx, tree, hf_rnsap_burstType);
}


static const per_sequence_t UEMeasurementTimeslotInfoHCR_IEs_sequence[] = {
  { &hf_rnsap_timeSlot      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TimeSlot },
  { &hf_rnsap_burstType     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UEMeasurementTimeslotInfoHCRBurstType },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UEMeasurementTimeslotInfoHCR_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UEMeasurementTimeslotInfoHCR_IEs, UEMeasurementTimeslotInfoHCR_IEs_sequence);

  return offset;
}
static int dissect_UEMeasurementTimeslotInfoHCR_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEMeasurementTimeslotInfoHCR_IEs(tvb, offset, actx, tree, hf_rnsap_UEMeasurementTimeslotInfoHCR_item);
}


static const per_sequence_t UEMeasurementTimeslotInfoHCR_sequence_of[1] = {
  { &hf_rnsap_UEMeasurementTimeslotInfoHCR_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_UEMeasurementTimeslotInfoHCR_IEs },
};

static int
dissect_rnsap_UEMeasurementTimeslotInfoHCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_UEMeasurementTimeslotInfoHCR, UEMeasurementTimeslotInfoHCR_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_id_UEMeasurementTimeslotInfoHCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEMeasurementTimeslotInfoHCR(tvb, offset, actx, tree, hf_rnsap_id_UEMeasurementTimeslotInfoHCR);
}


static const per_sequence_t UEMeasurementTimeslotInfoLCR_IEs_sequence[] = {
  { &hf_rnsap_timeSlot1     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TimeSlotLCR },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UEMeasurementTimeslotInfoLCR_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UEMeasurementTimeslotInfoLCR_IEs, UEMeasurementTimeslotInfoLCR_IEs_sequence);

  return offset;
}
static int dissect_UEMeasurementTimeslotInfoLCR_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEMeasurementTimeslotInfoLCR_IEs(tvb, offset, actx, tree, hf_rnsap_UEMeasurementTimeslotInfoLCR_item);
}


static const per_sequence_t UEMeasurementTimeslotInfoLCR_sequence_of[1] = {
  { &hf_rnsap_UEMeasurementTimeslotInfoLCR_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_UEMeasurementTimeslotInfoLCR_IEs },
};

static int
dissect_rnsap_UEMeasurementTimeslotInfoLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_UEMeasurementTimeslotInfoLCR, UEMeasurementTimeslotInfoLCR_sequence_of,
                                                  1, 6);

  return offset;
}
static int dissect_id_UEMeasurementTimeslotInfoLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEMeasurementTimeslotInfoLCR(tvb, offset, actx, tree, hf_rnsap_id_UEMeasurementTimeslotInfoLCR);
}


static const value_string rnsap_UEMeasurementReportCharacteristicsPeriodicAmountofReporting_vals[] = {
  {   0, "r1" },
  {   1, "r2" },
  {   2, "r4" },
  {   3, "r8" },
  {   4, "r16" },
  {   5, "r32" },
  {   6, "r64" },
  {   7, "rInfinity" },
  { 0, NULL }
};


static int
dissect_rnsap_UEMeasurementReportCharacteristicsPeriodicAmountofReporting(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_amountofReporting(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEMeasurementReportCharacteristicsPeriodicAmountofReporting(tvb, offset, actx, tree, hf_rnsap_amountofReporting);
}


static const value_string rnsap_UEMeasurementReportCharacteristicsPeriodicReportingInterval_vals[] = {
  {   0, "r250" },
  {   1, "r500" },
  {   2, "r1000" },
  {   3, "r2000" },
  {   4, "r3000" },
  {   5, "r4000" },
  {   6, "r6000" },
  {   7, "r8000" },
  {   8, "r12000" },
  {   9, "r16000" },
  {  10, "r20000" },
  {  11, "r24000" },
  {  12, "r28000" },
  {  13, "r32000" },
  {  14, "r64000" },
  { 0, NULL }
};


static int
dissect_rnsap_UEMeasurementReportCharacteristicsPeriodicReportingInterval(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     15, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_reportingInterval(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEMeasurementReportCharacteristicsPeriodicReportingInterval(tvb, offset, actx, tree, hf_rnsap_reportingInterval);
}


static const per_sequence_t UEMeasurementReportCharacteristicsPeriodic_sequence[] = {
  { &hf_rnsap_amountofReporting, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UEMeasurementReportCharacteristicsPeriodicAmountofReporting },
  { &hf_rnsap_reportingInterval, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UEMeasurementReportCharacteristicsPeriodicReportingInterval },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UEMeasurementReportCharacteristicsPeriodic(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UEMeasurementReportCharacteristicsPeriodic, UEMeasurementReportCharacteristicsPeriodic_sequence);

  return offset;
}
static int dissect_periodic2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEMeasurementReportCharacteristicsPeriodic(tvb, offset, actx, tree, hf_rnsap_periodic2);
}



static int
dissect_rnsap_UEMeasurementThresholdDLTimeslotISCP(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              -115, -25, NULL, FALSE);

  return offset;
}
static int dissect_timeslotISCP(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEMeasurementThresholdDLTimeslotISCP(tvb, offset, actx, tree, hf_rnsap_timeslotISCP);
}



static int
dissect_rnsap_UEMeasurementThresholdUETransmitPower(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              -50, 33U, NULL, FALSE);

  return offset;
}
static int dissect_uETransmitPower(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEMeasurementThresholdUETransmitPower(tvb, offset, actx, tree, hf_rnsap_uETransmitPower);
}



static int
dissect_rnsap_UEMeasurementThreshold_Extension(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_extension_UEMeasurementThreshold(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEMeasurementThreshold_Extension(tvb, offset, actx, tree, hf_rnsap_extension_UEMeasurementThreshold);
}


static const value_string rnsap_UEMeasurementThreshold_vals[] = {
  {   0, "timeslotISCP" },
  {   1, "uETransmitPower" },
  {   2, "extension-UEMeasurementThreshold" },
  { 0, NULL }
};

static const per_choice_t UEMeasurementThreshold_choice[] = {
  {   0, &hf_rnsap_timeslotISCP  , ASN1_EXTENSION_ROOT    , dissect_rnsap_UEMeasurementThresholdDLTimeslotISCP },
  {   1, &hf_rnsap_uETransmitPower, ASN1_EXTENSION_ROOT    , dissect_rnsap_UEMeasurementThresholdUETransmitPower },
  {   2, &hf_rnsap_extension_UEMeasurementThreshold, ASN1_NOT_EXTENSION_ROOT, dissect_rnsap_UEMeasurementThreshold_Extension },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_UEMeasurementThreshold(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_UEMeasurementThreshold, UEMeasurementThreshold_choice,
                                 NULL);

  return offset;
}
static int dissect_uEMeasurementTreshold(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEMeasurementThreshold(tvb, offset, actx, tree, hf_rnsap_uEMeasurementTreshold);
}


static const value_string rnsap_UEMeasurementTimeToTrigger_vals[] = {
  {   0, "r0" },
  {   1, "r10" },
  {   2, "r20" },
  {   3, "r40" },
  {   4, "r60" },
  {   5, "r80" },
  {   6, "r100" },
  {   7, "r120" },
  {   8, "r160" },
  {   9, "r200" },
  {  10, "r240" },
  {  11, "r320" },
  {  12, "r640" },
  {  13, "r1280" },
  {  14, "r2560" },
  {  15, "r5000" },
  { 0, NULL }
};


static int
dissect_rnsap_UEMeasurementTimeToTrigger(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     16, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_uEMeasurementTimeToTrigger(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEMeasurementTimeToTrigger(tvb, offset, actx, tree, hf_rnsap_uEMeasurementTimeToTrigger);
}



static int
dissect_rnsap_UEMeasurementHysteresisTime(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 15U, NULL, FALSE);

  return offset;
}
static int dissect_uEMeasurementHysteresisTime(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEMeasurementHysteresisTime(tvb, offset, actx, tree, hf_rnsap_uEMeasurementHysteresisTime);
}


static const per_sequence_t UEMeasurementReportCharacteristicsEvent1h_sequence[] = {
  { &hf_rnsap_uEMeasurementTreshold, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UEMeasurementThreshold },
  { &hf_rnsap_uEMeasurementTimeToTrigger, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UEMeasurementTimeToTrigger },
  { &hf_rnsap_uEMeasurementHysteresisTime, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UEMeasurementHysteresisTime },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UEMeasurementReportCharacteristicsEvent1h(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UEMeasurementReportCharacteristicsEvent1h, UEMeasurementReportCharacteristicsEvent1h_sequence);

  return offset;
}
static int dissect_event1h(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEMeasurementReportCharacteristicsEvent1h(tvb, offset, actx, tree, hf_rnsap_event1h);
}


static const per_sequence_t UEMeasurementReportCharacteristicsEvent1i_sequence[] = {
  { &hf_rnsap_uEMeasurementTreshold, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UEMeasurementThreshold },
  { &hf_rnsap_uEMeasurementTimeToTrigger, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UEMeasurementTimeToTrigger },
  { &hf_rnsap_uEMeasurementHysteresisTime, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UEMeasurementHysteresisTime },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UEMeasurementReportCharacteristicsEvent1i(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UEMeasurementReportCharacteristicsEvent1i, UEMeasurementReportCharacteristicsEvent1i_sequence);

  return offset;
}
static int dissect_event1i(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEMeasurementReportCharacteristicsEvent1i(tvb, offset, actx, tree, hf_rnsap_event1i);
}


static const per_sequence_t UEMeasurementReportCharacteristicsEvent6a_sequence[] = {
  { &hf_rnsap_uEMeasurementTreshold, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UEMeasurementThreshold },
  { &hf_rnsap_uEMeasurementTimeToTrigger, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UEMeasurementTimeToTrigger },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UEMeasurementReportCharacteristicsEvent6a(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UEMeasurementReportCharacteristicsEvent6a, UEMeasurementReportCharacteristicsEvent6a_sequence);

  return offset;
}
static int dissect_event6a(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEMeasurementReportCharacteristicsEvent6a(tvb, offset, actx, tree, hf_rnsap_event6a);
}


static const per_sequence_t UEMeasurementReportCharacteristicsEvent6b_sequence[] = {
  { &hf_rnsap_uEMeasurementTreshold, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UEMeasurementThreshold },
  { &hf_rnsap_uEMeasurementTimeToTrigger, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UEMeasurementTimeToTrigger },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UEMeasurementReportCharacteristicsEvent6b(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UEMeasurementReportCharacteristicsEvent6b, UEMeasurementReportCharacteristicsEvent6b_sequence);

  return offset;
}
static int dissect_event6b(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEMeasurementReportCharacteristicsEvent6b(tvb, offset, actx, tree, hf_rnsap_event6b);
}


static const per_sequence_t UEMeasurementReportCharacteristicsEvent6c_sequence[] = {
  { &hf_rnsap_uEMeasurementTimeToTrigger, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UEMeasurementTimeToTrigger },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UEMeasurementReportCharacteristicsEvent6c(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UEMeasurementReportCharacteristicsEvent6c, UEMeasurementReportCharacteristicsEvent6c_sequence);

  return offset;
}
static int dissect_event6c(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEMeasurementReportCharacteristicsEvent6c(tvb, offset, actx, tree, hf_rnsap_event6c);
}


static const per_sequence_t UEMeasurementReportCharacteristicsEvent6d_sequence[] = {
  { &hf_rnsap_uEMeasurementTimeToTrigger, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UEMeasurementTimeToTrigger },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UEMeasurementReportCharacteristicsEvent6d(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UEMeasurementReportCharacteristicsEvent6d, UEMeasurementReportCharacteristicsEvent6d_sequence);

  return offset;
}
static int dissect_event6d(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEMeasurementReportCharacteristicsEvent6d(tvb, offset, actx, tree, hf_rnsap_event6d);
}



static int
dissect_rnsap_UEMeasurementReportCharacteristics_Extension(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_extension_ReportCharacteristics1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEMeasurementReportCharacteristics_Extension(tvb, offset, actx, tree, hf_rnsap_extension_ReportCharacteristics1);
}


static const value_string rnsap_UEMeasurementReportCharacteristics_vals[] = {
  {   0, "periodic" },
  {   1, "event1h" },
  {   2, "event1i" },
  {   3, "event6a" },
  {   4, "event6b" },
  {   5, "event6c" },
  {   6, "event6d" },
  {   7, "extension-ReportCharacteristics" },
  { 0, NULL }
};

static const per_choice_t UEMeasurementReportCharacteristics_choice[] = {
  {   0, &hf_rnsap_periodic2     , ASN1_EXTENSION_ROOT    , dissect_rnsap_UEMeasurementReportCharacteristicsPeriodic },
  {   1, &hf_rnsap_event1h       , ASN1_EXTENSION_ROOT    , dissect_rnsap_UEMeasurementReportCharacteristicsEvent1h },
  {   2, &hf_rnsap_event1i       , ASN1_EXTENSION_ROOT    , dissect_rnsap_UEMeasurementReportCharacteristicsEvent1i },
  {   3, &hf_rnsap_event6a       , ASN1_EXTENSION_ROOT    , dissect_rnsap_UEMeasurementReportCharacteristicsEvent6a },
  {   4, &hf_rnsap_event6b       , ASN1_EXTENSION_ROOT    , dissect_rnsap_UEMeasurementReportCharacteristicsEvent6b },
  {   5, &hf_rnsap_event6c       , ASN1_EXTENSION_ROOT    , dissect_rnsap_UEMeasurementReportCharacteristicsEvent6c },
  {   6, &hf_rnsap_event6d       , ASN1_EXTENSION_ROOT    , dissect_rnsap_UEMeasurementReportCharacteristicsEvent6d },
  {   7, &hf_rnsap_extension_ReportCharacteristics1, ASN1_NOT_EXTENSION_ROOT, dissect_rnsap_UEMeasurementReportCharacteristics_Extension },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_UEMeasurementReportCharacteristics(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_UEMeasurementReportCharacteristics, UEMeasurementReportCharacteristics_choice,
                                 NULL);

  return offset;
}
static int dissect_id_UEMeasurementReportCharacteristics(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEMeasurementReportCharacteristics(tvb, offset, actx, tree, hf_rnsap_id_UEMeasurementReportCharacteristics);
}


static const value_string rnsap_UEMeasurementParameterModAllow_vals[] = {
  {   0, "parameterModificationAllowed" },
  { 0, NULL }
};


static int
dissect_rnsap_UEMeasurementParameterModAllow(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_id_UEMeasurementParameterModAllow(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEMeasurementParameterModAllow(tvb, offset, actx, tree, hf_rnsap_id_UEMeasurementParameterModAllow);
}



static int
dissect_rnsap_INTEGER_0_104(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 104U, NULL, FALSE);

  return offset;
}
static int dissect_uETransmitPower1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_INTEGER_0_104(tvb, offset, actx, tree, hf_rnsap_uETransmitPower1);
}


static const per_sequence_t UEMeasurementValueTransmittedPowerListHCR_IEs_sequence[] = {
  { &hf_rnsap_timeSlot      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TimeSlot },
  { &hf_rnsap_uETransmitPower1, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_INTEGER_0_104 },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UEMeasurementValueTransmittedPowerListHCR_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UEMeasurementValueTransmittedPowerListHCR_IEs, UEMeasurementValueTransmittedPowerListHCR_IEs_sequence);

  return offset;
}
static int dissect_UEMeasurementValueTransmittedPowerListHCR_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEMeasurementValueTransmittedPowerListHCR_IEs(tvb, offset, actx, tree, hf_rnsap_UEMeasurementValueTransmittedPowerListHCR_item);
}


static const per_sequence_t UEMeasurementValueTransmittedPowerListHCR_sequence_of[1] = {
  { &hf_rnsap_UEMeasurementValueTransmittedPowerListHCR_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_UEMeasurementValueTransmittedPowerListHCR_IEs },
};

static int
dissect_rnsap_UEMeasurementValueTransmittedPowerListHCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_UEMeasurementValueTransmittedPowerListHCR, UEMeasurementValueTransmittedPowerListHCR_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_uEMeasurementTransmittedPowerListHCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEMeasurementValueTransmittedPowerListHCR(tvb, offset, actx, tree, hf_rnsap_uEMeasurementTransmittedPowerListHCR);
}


static const per_sequence_t UEMeasurementValueTransmittedPowerListLCR_IEs_sequence[] = {
  { &hf_rnsap_timeSlotLCR   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TimeSlotLCR },
  { &hf_rnsap_uETransmitPower1, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_INTEGER_0_104 },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UEMeasurementValueTransmittedPowerListLCR_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UEMeasurementValueTransmittedPowerListLCR_IEs, UEMeasurementValueTransmittedPowerListLCR_IEs_sequence);

  return offset;
}
static int dissect_UEMeasurementValueTransmittedPowerListLCR_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEMeasurementValueTransmittedPowerListLCR_IEs(tvb, offset, actx, tree, hf_rnsap_UEMeasurementValueTransmittedPowerListLCR_item);
}


static const per_sequence_t UEMeasurementValueTransmittedPowerListLCR_sequence_of[1] = {
  { &hf_rnsap_UEMeasurementValueTransmittedPowerListLCR_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_UEMeasurementValueTransmittedPowerListLCR_IEs },
};

static int
dissect_rnsap_UEMeasurementValueTransmittedPowerListLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_UEMeasurementValueTransmittedPowerListLCR, UEMeasurementValueTransmittedPowerListLCR_sequence_of,
                                                  1, 6);

  return offset;
}
static int dissect_uEMeasurementTransmittedPowerListLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEMeasurementValueTransmittedPowerListLCR(tvb, offset, actx, tree, hf_rnsap_uEMeasurementTransmittedPowerListLCR);
}


static const per_sequence_t UE_MeasurementValue_UE_Transmitted_Power_sequence[] = {
  { &hf_rnsap_uEMeasurementTransmittedPowerListHCR, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_UEMeasurementValueTransmittedPowerListHCR },
  { &hf_rnsap_uEMeasurementTransmittedPowerListLCR, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_UEMeasurementValueTransmittedPowerListLCR },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UE_MeasurementValue_UE_Transmitted_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UE_MeasurementValue_UE_Transmitted_Power, UE_MeasurementValue_UE_Transmitted_Power_sequence);

  return offset;
}
static int dissect_uE_Transmitted_Power(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UE_MeasurementValue_UE_Transmitted_Power(tvb, offset, actx, tree, hf_rnsap_uE_Transmitted_Power);
}


static const per_sequence_t UE_MeasurementValue_Primary_CCPCH_RSCP_sequence[] = {
  { &hf_rnsap_primaryCCPCH_RSCP, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_PrimaryCCPCH_RSCP },
  { &hf_rnsap_primaryCCPCH_RSCP_Delta, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_PrimaryCCPCH_RSCP_Delta },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UE_MeasurementValue_Primary_CCPCH_RSCP(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UE_MeasurementValue_Primary_CCPCH_RSCP, UE_MeasurementValue_Primary_CCPCH_RSCP_sequence);

  return offset;
}
static int dissect_primary_CCPCH_RSCP(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UE_MeasurementValue_Primary_CCPCH_RSCP(tvb, offset, actx, tree, hf_rnsap_primary_CCPCH_RSCP);
}


static const per_sequence_t UEMeasurementValueTimeslotISCPListHCR_IEs_sequence[] = {
  { &hf_rnsap_timeSlot      , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TimeSlot },
  { &hf_rnsap_dL_TimeslotISCP, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_TimeslotISCP },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UEMeasurementValueTimeslotISCPListHCR_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UEMeasurementValueTimeslotISCPListHCR_IEs, UEMeasurementValueTimeslotISCPListHCR_IEs_sequence);

  return offset;
}
static int dissect_UEMeasurementValueTimeslotISCPListHCR_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEMeasurementValueTimeslotISCPListHCR_IEs(tvb, offset, actx, tree, hf_rnsap_UEMeasurementValueTimeslotISCPListHCR_item);
}


static const per_sequence_t UEMeasurementValueTimeslotISCPListHCR_sequence_of[1] = {
  { &hf_rnsap_UEMeasurementValueTimeslotISCPListHCR_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_UEMeasurementValueTimeslotISCPListHCR_IEs },
};

static int
dissect_rnsap_UEMeasurementValueTimeslotISCPListHCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_UEMeasurementValueTimeslotISCPListHCR, UEMeasurementValueTimeslotISCPListHCR_sequence_of,
                                                  1, 15);

  return offset;
}
static int dissect_uEMeasurementTimeslotISCPListHCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEMeasurementValueTimeslotISCPListHCR(tvb, offset, actx, tree, hf_rnsap_uEMeasurementTimeslotISCPListHCR);
}


static const per_sequence_t UEMeasurementValueTimeslotISCPListLCR_IEs_sequence[] = {
  { &hf_rnsap_timeSlotLCR   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TimeSlotLCR },
  { &hf_rnsap_dL_TimeslotISCP, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_TimeslotISCP },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UEMeasurementValueTimeslotISCPListLCR_IEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UEMeasurementValueTimeslotISCPListLCR_IEs, UEMeasurementValueTimeslotISCPListLCR_IEs_sequence);

  return offset;
}
static int dissect_UEMeasurementValueTimeslotISCPListLCR_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEMeasurementValueTimeslotISCPListLCR_IEs(tvb, offset, actx, tree, hf_rnsap_UEMeasurementValueTimeslotISCPListLCR_item);
}


static const per_sequence_t UEMeasurementValueTimeslotISCPListLCR_sequence_of[1] = {
  { &hf_rnsap_UEMeasurementValueTimeslotISCPListLCR_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_UEMeasurementValueTimeslotISCPListLCR_IEs },
};

static int
dissect_rnsap_UEMeasurementValueTimeslotISCPListLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_UEMeasurementValueTimeslotISCPListLCR, UEMeasurementValueTimeslotISCPListLCR_sequence_of,
                                                  1, 6);

  return offset;
}
static int dissect_uEMeasurementTimeslotISCPListLCR(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEMeasurementValueTimeslotISCPListLCR(tvb, offset, actx, tree, hf_rnsap_uEMeasurementTimeslotISCPListLCR);
}


static const per_sequence_t UE_MeasurementValue_DL_Timeslot_ISCP_sequence[] = {
  { &hf_rnsap_uEMeasurementTimeslotISCPListHCR, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_UEMeasurementValueTimeslotISCPListHCR },
  { &hf_rnsap_uEMeasurementTimeslotISCPListLCR, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_UEMeasurementValueTimeslotISCPListLCR },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UE_MeasurementValue_DL_Timeslot_ISCP(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UE_MeasurementValue_DL_Timeslot_ISCP, UE_MeasurementValue_DL_Timeslot_ISCP_sequence);

  return offset;
}
static int dissect_dL_Timeslot_ISCP(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UE_MeasurementValue_DL_Timeslot_ISCP(tvb, offset, actx, tree, hf_rnsap_dL_Timeslot_ISCP);
}



static int
dissect_rnsap_UEMeasurementValue_Extension(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_extension_UEMeasurementValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEMeasurementValue_Extension(tvb, offset, actx, tree, hf_rnsap_extension_UEMeasurementValue);
}


static const value_string rnsap_UEMeasurementValue_vals[] = {
  {   0, "uE-Transmitted-Power" },
  {   1, "primary-CCPCH-RSCP" },
  {   2, "dL-Timeslot-ISCP" },
  {   3, "extension-UEMeasurementValue" },
  { 0, NULL }
};

static const per_choice_t UEMeasurementValue_choice[] = {
  {   0, &hf_rnsap_uE_Transmitted_Power, ASN1_EXTENSION_ROOT    , dissect_rnsap_UE_MeasurementValue_UE_Transmitted_Power },
  {   1, &hf_rnsap_primary_CCPCH_RSCP, ASN1_EXTENSION_ROOT    , dissect_rnsap_UE_MeasurementValue_Primary_CCPCH_RSCP },
  {   2, &hf_rnsap_dL_Timeslot_ISCP, ASN1_EXTENSION_ROOT    , dissect_rnsap_UE_MeasurementValue_DL_Timeslot_ISCP },
  {   3, &hf_rnsap_extension_UEMeasurementValue, ASN1_NOT_EXTENSION_ROOT, dissect_rnsap_UEMeasurementValue_Extension },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_UEMeasurementValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_UEMeasurementValue, UEMeasurementValue_choice,
                                 NULL);

  return offset;
}
static int dissect_uEmeasurementValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEMeasurementValue(tvb, offset, actx, tree, hf_rnsap_uEmeasurementValue);
}


static const per_sequence_t UEMeasurementValueInformationAvailable_sequence[] = {
  { &hf_rnsap_uEmeasurementValue, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UEMeasurementValue },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_UEMeasurementValueInformationAvailable(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_UEMeasurementValueInformationAvailable, UEMeasurementValueInformationAvailable_sequence);

  return offset;
}
static int dissect_measurementAvailable2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEMeasurementValueInformationAvailable(tvb, offset, actx, tree, hf_rnsap_measurementAvailable2);
}



static int
dissect_rnsap_UEMeasurementValueInformationnotAvailable(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_null(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_measurementnotAvailable2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEMeasurementValueInformationnotAvailable(tvb, offset, actx, tree, hf_rnsap_measurementnotAvailable2);
}


static const value_string rnsap_UEMeasurementValueInformation_vals[] = {
  {   0, "measurementAvailable" },
  {   1, "measurementnotAvailable" },
  { 0, NULL }
};

static const per_choice_t UEMeasurementValueInformation_choice[] = {
  {   0, &hf_rnsap_measurementAvailable2, ASN1_NO_EXTENSIONS     , dissect_rnsap_UEMeasurementValueInformationAvailable },
  {   1, &hf_rnsap_measurementnotAvailable2, ASN1_NO_EXTENSIONS     , dissect_rnsap_UEMeasurementValueInformationnotAvailable },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_UEMeasurementValueInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_UEMeasurementValueInformation, UEMeasurementValueInformation_choice,
                                 NULL);

  return offset;
}
static int dissect_id_UEMeasurementValueInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEMeasurementValueInformation(tvb, offset, actx, tree, hf_rnsap_id_UEMeasurementValueInformation);
}


static const value_string rnsap_T_interface_vals[] = {
  {   0, "iub" },
  {   1, "iur" },
  { 0, NULL }
};


static int
dissect_rnsap_T_interface(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_interface(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_T_interface(tvb, offset, actx, tree, hf_rnsap_interface);
}


static const per_sequence_t InterfacesToTraceItem_sequence[] = {
  { &hf_rnsap_interface     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_T_interface },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_InterfacesToTraceItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_InterfacesToTraceItem, InterfacesToTraceItem_sequence);

  return offset;
}
static int dissect_id_InterfacesToTraceItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_InterfacesToTraceItem(tvb, offset, actx, tree, hf_rnsap_id_InterfacesToTraceItem);
}


static const per_sequence_t ListOfInterfacesToTrace_sequence_of[1] = {
  { &hf_rnsap_ListOfInterfacesToTrace_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_ListOfInterfacesToTrace(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_ListOfInterfacesToTrace, ListOfInterfacesToTrace_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_id_ListOfInterfacesToTrace(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ListOfInterfacesToTrace(tvb, offset, actx, tree, hf_rnsap_id_ListOfInterfacesToTrace);
}


static const value_string rnsap_TraceDepth_vals[] = {
  {   0, "minimum" },
  {   1, "medium" },
  {   2, "maximum" },
  { 0, NULL }
};


static int
dissect_rnsap_TraceDepth(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_id_TraceDepth(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TraceDepth(tvb, offset, actx, tree, hf_rnsap_id_TraceDepth);
}



static int
dissect_rnsap_TraceRecordingSessionReference(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 65535U, NULL, FALSE);

  return offset;
}
static int dissect_id_TraceRecordingSessionReference(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TraceRecordingSessionReference(tvb, offset, actx, tree, hf_rnsap_id_TraceRecordingSessionReference);
}



static int
dissect_rnsap_TraceReference(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_octet_string(tvb, offset, actx, tree, hf_index,
                                       2, 3, NULL);

  return offset;
}
static int dissect_id_TraceReference(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TraceReference(tvb, offset, actx, tree, hf_rnsap_id_TraceReference);
}



static int
dissect_rnsap_IMEI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_octet_string(tvb, offset, actx, tree, hf_index,
                                       8, 8, NULL);

  return offset;
}
static int dissect_imei(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_IMEI(tvb, offset, actx, tree, hf_rnsap_imei);
}



static int
dissect_rnsap_IMEISV(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_octet_string(tvb, offset, actx, tree, hf_index,
                                       8, 8, NULL);

  return offset;
}
static int dissect_imeisv(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_IMEISV(tvb, offset, actx, tree, hf_rnsap_imeisv);
}


static const value_string rnsap_UEIdentity_vals[] = {
  {   0, "imsi" },
  {   1, "imei" },
  {   2, "imeisv" },
  { 0, NULL }
};

static const per_choice_t UEIdentity_choice[] = {
  {   0, &hf_rnsap_imsi          , ASN1_EXTENSION_ROOT    , dissect_rnsap_IMSI },
  {   1, &hf_rnsap_imei          , ASN1_EXTENSION_ROOT    , dissect_rnsap_IMEI },
  {   2, &hf_rnsap_imeisv        , ASN1_EXTENSION_ROOT    , dissect_rnsap_IMEISV },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_UEIdentity(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_UEIdentity, UEIdentity_choice,
                                 NULL);

  return offset;
}
static int dissect_id_UEIdentity(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UEIdentity(tvb, offset, actx, tree, hf_rnsap_id_UEIdentity);
}



static int
dissect_rnsap_OCTET_STRING_SIZE_1_23(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_octet_string(tvb, offset, actx, tree, hf_index,
                                       1, 23, NULL);

  return offset;
}
static int dissect_gERAN_SI_block(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_OCTET_STRING_SIZE_1_23(tvb, offset, actx, tree, hf_rnsap_gERAN_SI_block);
}


static const per_sequence_t GERAN_SystemInfo_item_sequence[] = {
  { &hf_rnsap_gERAN_SI_block, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_OCTET_STRING_SIZE_1_23 },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_GERAN_SystemInfo_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_GERAN_SystemInfo_item, GERAN_SystemInfo_item_sequence);

  return offset;
}
static int dissect_GERAN_SystemInfo_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GERAN_SystemInfo_item(tvb, offset, actx, tree, hf_rnsap_GERAN_SystemInfo_item);
}


static const per_sequence_t GERAN_SystemInfo_sequence_of[1] = {
  { &hf_rnsap_GERAN_SystemInfo_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_GERAN_SystemInfo_item },
};

static int
dissect_rnsap_GERAN_SystemInfo(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_GERAN_SystemInfo, GERAN_SystemInfo_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_sI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GERAN_SystemInfo(tvb, offset, actx, tree, hf_rnsap_sI);
}
static int dissect_pSI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GERAN_SystemInfo(tvb, offset, actx, tree, hf_rnsap_pSI);
}


static const value_string rnsap_GERAN_SI_Type_vals[] = {
  {   0, "sI" },
  {   1, "pSI" },
  { 0, NULL }
};

static const per_choice_t GERAN_SI_Type_choice[] = {
  {   0, &hf_rnsap_sI            , ASN1_EXTENSION_ROOT    , dissect_rnsap_GERAN_SystemInfo },
  {   1, &hf_rnsap_pSI           , ASN1_EXTENSION_ROOT    , dissect_rnsap_GERAN_SystemInfo },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_GERAN_SI_Type(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_GERAN_SI_Type, GERAN_SI_Type_choice,
                                 NULL);

  return offset;
}
static int dissect_gERAN_SI_Type(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GERAN_SI_Type(tvb, offset, actx, tree, hf_rnsap_gERAN_SI_Type);
}


static const per_sequence_t NACC_Related_Data_sequence[] = {
  { &hf_rnsap_gERAN_SI_Type , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_GERAN_SI_Type },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_NACC_Related_Data(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_NACC_Related_Data, NACC_Related_Data_sequence);

  return offset;
}
static int dissect_id_NACC_Related_Data(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_NACC_Related_Data(tvb, offset, actx, tree, hf_rnsap_id_NACC_Related_Data);
}


static const per_sequence_t T_lAI_sequence[] = {
  { &hf_rnsap_pLMN_Identity , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PLMN_Identity },
  { &hf_rnsap_lAC           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_LAC },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_T_lAI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_T_lAI, T_lAI_sequence);

  return offset;
}
static int dissect_lAI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_T_lAI(tvb, offset, actx, tree, hf_rnsap_lAI);
}



static int
dissect_rnsap_CI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_octet_string(tvb, offset, actx, tree, hf_index,
                                       2, 2, NULL);

  return offset;
}
static int dissect_cI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CI(tvb, offset, actx, tree, hf_rnsap_cI);
}


static const per_sequence_t CGI_sequence[] = {
  { &hf_rnsap_lAI           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_T_lAI },
  { &hf_rnsap_cI            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_CI },
  { &hf_rnsap_iE_Extensions , ASN1_NO_EXTENSIONS     , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_CGI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_CGI, CGI_sequence);

  return offset;
}
static int dissect_cGI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CGI(tvb, offset, actx, tree, hf_rnsap_cGI);
}


static const per_sequence_t GSM_Cell_InfEx_Rqst_sequence[] = {
  { &hf_rnsap_cGI           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CGI },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_GSM_Cell_InfEx_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_GSM_Cell_InfEx_Rqst, GSM_Cell_InfEx_Rqst_sequence);

  return offset;
}
static int dissect_id_GSM_Cell_InfEx_Rqst(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_GSM_Cell_InfEx_Rqst(tvb, offset, actx, tree, hf_rnsap_id_GSM_Cell_InfEx_Rqst);
}



static int
dissect_rnsap_MeasurementRecoveryBehavior(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_null(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_id_MeasurementRecoveryBehavior(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MeasurementRecoveryBehavior(tvb, offset, actx, tree, hf_rnsap_id_MeasurementRecoveryBehavior);
}



static int
dissect_rnsap_MeasurementRecoveryReportingIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_null(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_id_MeasurementRecoveryReportingIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MeasurementRecoveryReportingIndicator(tvb, offset, actx, tree, hf_rnsap_id_MeasurementRecoveryReportingIndicator);
}



static int
dissect_rnsap_MeasurementRecoverySupportIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_null(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_id_MeasurementRecoverySupportIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MeasurementRecoverySupportIndicator(tvb, offset, actx, tree, hf_rnsap_id_MeasurementRecoverySupportIndicator);
}


static const per_sequence_t PowerOffsetInformation_RL_ReconfPrepFDD_sequence[] = {
  { &hf_rnsap_pO1_ForTFCI_Bits, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PowerOffset },
  { &hf_rnsap_pO2_ForTPC_Bits, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PowerOffset },
  { &hf_rnsap_pO3_ForPilotBits, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PowerOffset },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_PowerOffsetInformation_RL_ReconfPrepFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_PowerOffsetInformation_RL_ReconfPrepFDD, PowerOffsetInformation_RL_ReconfPrepFDD_sequence);

  return offset;
}
static int dissect_powerOffsetInformation2(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PowerOffsetInformation_RL_ReconfPrepFDD(tvb, offset, actx, tree, hf_rnsap_powerOffsetInformation2);
}


static const per_sequence_t DL_DPCH_Power_Information_RL_ReconfPrepFDD_sequence[] = {
  { &hf_rnsap_powerOffsetInformation2, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PowerOffsetInformation_RL_ReconfPrepFDD },
  { &hf_rnsap_fdd_TPC_DownlinkStepSize, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_FDD_TPC_DownlinkStepSize },
  { &hf_rnsap_innerLoopDLPCStatus, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_InnerLoopDLPCStatus },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_DPCH_Power_Information_RL_ReconfPrepFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_DPCH_Power_Information_RL_ReconfPrepFDD, DL_DPCH_Power_Information_RL_ReconfPrepFDD_sequence);

  return offset;
}
static int dissect_id_DL_DPCH_Power_Information_RL_ReconfPrepFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_DPCH_Power_Information_RL_ReconfPrepFDD(tvb, offset, actx, tree, hf_rnsap_id_DL_DPCH_Power_Information_RL_ReconfPrepFDD);
}


static const per_sequence_t PowerOffsetInformation_F_DPCH_RL_ReconfPrepFDD_sequence[] = {
  { &hf_rnsap_po2_ForTPC_Bits, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PowerOffset },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_PowerOffsetInformation_F_DPCH_RL_ReconfPrepFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_PowerOffsetInformation_F_DPCH_RL_ReconfPrepFDD, PowerOffsetInformation_F_DPCH_RL_ReconfPrepFDD_sequence);

  return offset;
}
static int dissect_powerOffsetInformation3(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PowerOffsetInformation_F_DPCH_RL_ReconfPrepFDD(tvb, offset, actx, tree, hf_rnsap_powerOffsetInformation3);
}


static const per_sequence_t F_DPCH_Information_RL_ReconfPrepFDD_sequence[] = {
  { &hf_rnsap_powerOffsetInformation3, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PowerOffsetInformation_F_DPCH_RL_ReconfPrepFDD },
  { &hf_rnsap_fdd_dl_TPC_DownlinkStepSize, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_FDD_TPC_DownlinkStepSize },
  { &hf_rnsap_limitedPowerIncrease, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_LimitedPowerIncrease },
  { &hf_rnsap_innerLoopDLPCStatus, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_InnerLoopDLPCStatus },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_F_DPCH_Information_RL_ReconfPrepFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_F_DPCH_Information_RL_ReconfPrepFDD, F_DPCH_Information_RL_ReconfPrepFDD_sequence);

  return offset;
}
static int dissect_id_F_DPCH_Information_RL_ReconfPrepFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_F_DPCH_Information_RL_ReconfPrepFDD(tvb, offset, actx, tree, hf_rnsap_id_F_DPCH_Information_RL_ReconfPrepFDD);
}


static const per_sequence_t PowerOffsetInformation_F_DPCH_RL_SetupRqstFDD_sequence[] = {
  { &hf_rnsap_po2_ForTPC_Bits, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PowerOffset },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_PowerOffsetInformation_F_DPCH_RL_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_PowerOffsetInformation_F_DPCH_RL_SetupRqstFDD, PowerOffsetInformation_F_DPCH_RL_SetupRqstFDD_sequence);

  return offset;
}
static int dissect_powerOffsetInformation1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PowerOffsetInformation_F_DPCH_RL_SetupRqstFDD(tvb, offset, actx, tree, hf_rnsap_powerOffsetInformation1);
}


static const per_sequence_t F_DPCH_Information_RL_SetupRqstFDD_sequence[] = {
  { &hf_rnsap_powerOffsetInformation1, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PowerOffsetInformation_F_DPCH_RL_SetupRqstFDD },
  { &hf_rnsap_fdd_dl_TPC_DownlinkStepSize, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_FDD_TPC_DownlinkStepSize },
  { &hf_rnsap_limitedPowerIncrease, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_LimitedPowerIncrease },
  { &hf_rnsap_innerLoopDLPCStatus, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_InnerLoopDLPCStatus },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_F_DPCH_Information_RL_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_F_DPCH_Information_RL_SetupRqstFDD, F_DPCH_Information_RL_SetupRqstFDD_sequence);

  return offset;
}
static int dissect_id_F_DPCH_Information_RL_SetupRqstFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_F_DPCH_Information_RL_SetupRqstFDD(tvb, offset, actx, tree, hf_rnsap_id_F_DPCH_Information_RL_SetupRqstFDD);
}



static int
dissect_rnsap_Service_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_octet_string(tvb, offset, actx, tree, hf_index,
                                       3, 3, NULL);

  return offset;
}
static int dissect_service_id(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Service_ID(tvb, offset, actx, tree, hf_rnsap_service_id);
}


static const per_sequence_t TMGI_sequence[] = {
  { &hf_rnsap_plmn_id       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PLMN_Identity },
  { &hf_rnsap_service_id    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_Service_ID },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_TMGI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_TMGI, TMGI_sequence);

  return offset;
}
static int dissect_tmgi(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TMGI(tvb, offset, actx, tree, hf_rnsap_tmgi);
}
static int dissect_MBMS_Bearer_Service_List_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TMGI(tvb, offset, actx, tree, hf_rnsap_MBMS_Bearer_Service_List_item);
}
static int dissect_tMGI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TMGI(tvb, offset, actx, tree, hf_rnsap_tMGI);
}


static const per_sequence_t MBMS_Bearer_Service_List_sequence_of[1] = {
  { &hf_rnsap_MBMS_Bearer_Service_List_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_TMGI },
};

static int
dissect_rnsap_MBMS_Bearer_Service_List(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_MBMS_Bearer_Service_List, MBMS_Bearer_Service_List_sequence_of,
                                                  1, 128);

  return offset;
}
static int dissect_id_MBMS_Bearer_Service_List(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MBMS_Bearer_Service_List(tvb, offset, actx, tree, hf_rnsap_id_MBMS_Bearer_Service_List);
}


static const per_sequence_t MBMS_Bearer_ServiceItemIEs_InfEx_Rsp_sequence[] = {
  { &hf_rnsap_tmgi          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TMGI },
  { &hf_rnsap_requestedDataValue, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RequestedDataValue },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_MBMS_Bearer_ServiceItemIEs_InfEx_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_MBMS_Bearer_ServiceItemIEs_InfEx_Rsp, MBMS_Bearer_ServiceItemIEs_InfEx_Rsp_sequence);

  return offset;
}
static int dissect_MBMS_Bearer_Service_List_InfEx_Rsp_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MBMS_Bearer_ServiceItemIEs_InfEx_Rsp(tvb, offset, actx, tree, hf_rnsap_MBMS_Bearer_Service_List_InfEx_Rsp_item);
}


static const per_sequence_t MBMS_Bearer_Service_List_InfEx_Rsp_sequence_of[1] = {
  { &hf_rnsap_MBMS_Bearer_Service_List_InfEx_Rsp_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_MBMS_Bearer_ServiceItemIEs_InfEx_Rsp },
};

static int
dissect_rnsap_MBMS_Bearer_Service_List_InfEx_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_MBMS_Bearer_Service_List_InfEx_Rsp, MBMS_Bearer_Service_List_InfEx_Rsp_sequence_of,
                                                  1, 128);

  return offset;
}
static int dissect_id_MBMS_Bearer_Service_List_InfEx_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MBMS_Bearer_Service_List_InfEx_Rsp(tvb, offset, actx, tree, hf_rnsap_id_MBMS_Bearer_Service_List_InfEx_Rsp);
}


static const value_string rnsap_TransmissionMode_vals[] = {
  {   0, "p-t-p" },
  {   1, "p-t-m" },
  {   2, "not-provided" },
  { 0, NULL }
};


static int
dissect_rnsap_TransmissionMode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_transmissionMode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TransmissionMode(tvb, offset, actx, tree, hf_rnsap_transmissionMode);
}


static const per_sequence_t MBMS_Bearer_ServiceItemFDD_sequence[] = {
  { &hf_rnsap_tmgi          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TMGI },
  { &hf_rnsap_transmissionMode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TransmissionMode },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_MBMS_Bearer_ServiceItemFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_MBMS_Bearer_ServiceItemFDD, MBMS_Bearer_ServiceItemFDD_sequence);

  return offset;
}
static int dissect_Active_MBMS_Bearer_Service_ListFDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MBMS_Bearer_ServiceItemFDD(tvb, offset, actx, tree, hf_rnsap_Active_MBMS_Bearer_Service_ListFDD_item);
}


static const per_sequence_t Active_MBMS_Bearer_Service_ListFDD_sequence_of[1] = {
  { &hf_rnsap_Active_MBMS_Bearer_Service_ListFDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_MBMS_Bearer_ServiceItemFDD },
};

static int
dissect_rnsap_Active_MBMS_Bearer_Service_ListFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_Active_MBMS_Bearer_Service_ListFDD, Active_MBMS_Bearer_Service_ListFDD_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_id_Active_MBMS_Bearer_ServiceFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Active_MBMS_Bearer_Service_ListFDD(tvb, offset, actx, tree, hf_rnsap_id_Active_MBMS_Bearer_ServiceFDD);
}


static const per_sequence_t MBMS_Bearer_ServiceItemTDD_sequence[] = {
  { &hf_rnsap_tmgi          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TMGI },
  { &hf_rnsap_transmissionMode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TransmissionMode },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_MBMS_Bearer_ServiceItemTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_MBMS_Bearer_ServiceItemTDD, MBMS_Bearer_ServiceItemTDD_sequence);

  return offset;
}
static int dissect_Active_MBMS_Bearer_Service_ListTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MBMS_Bearer_ServiceItemTDD(tvb, offset, actx, tree, hf_rnsap_Active_MBMS_Bearer_Service_ListTDD_item);
}


static const per_sequence_t Active_MBMS_Bearer_Service_ListTDD_sequence_of[1] = {
  { &hf_rnsap_Active_MBMS_Bearer_Service_ListTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_MBMS_Bearer_ServiceItemTDD },
};

static int
dissect_rnsap_Active_MBMS_Bearer_Service_ListTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_Active_MBMS_Bearer_Service_ListTDD, Active_MBMS_Bearer_Service_ListTDD_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_id_Active_MBMS_Bearer_ServiceTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Active_MBMS_Bearer_Service_ListTDD(tvb, offset, actx, tree, hf_rnsap_id_Active_MBMS_Bearer_ServiceTDD);
}


static const per_sequence_t Cell_Fach_Pch_State_sequence[] = {
  { &hf_rnsap_d_RNTI        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_D_RNTI },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_Cell_Fach_Pch_State(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_Cell_Fach_Pch_State, Cell_Fach_Pch_State_sequence);

  return offset;
}
static int dissect_cell_fach_pch(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Cell_Fach_Pch_State(tvb, offset, actx, tree, hf_rnsap_cell_fach_pch);
}


static const per_sequence_t Ura_Pch_State_sequence[] = {
  { &hf_rnsap_srnc_id       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RNC_ID },
  { &hf_rnsap_ura_id        , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_URA_ID },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_Ura_Pch_State(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_Ura_Pch_State, Ura_Pch_State_sequence);

  return offset;
}
static int dissect_ura_pch(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Ura_Pch_State(tvb, offset, actx, tree, hf_rnsap_ura_pch);
}


static const value_string rnsap_UE_State_vals[] = {
  {   0, "cell-fach-pch" },
  {   1, "ura-pch" },
  { 0, NULL }
};

static const per_choice_t UE_State_choice[] = {
  {   0, &hf_rnsap_cell_fach_pch , ASN1_EXTENSION_ROOT    , dissect_rnsap_Cell_Fach_Pch_State },
  {   1, &hf_rnsap_ura_pch       , ASN1_EXTENSION_ROOT    , dissect_rnsap_Ura_Pch_State },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_UE_State(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_UE_State, UE_State_choice,
                                 NULL);

  return offset;
}
static int dissect_id_UE_State(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UE_State(tvb, offset, actx, tree, hf_rnsap_id_UE_State);
}


static const value_string rnsap_HARQ_Preamble_Mode_vals[] = {
  {   0, "mode0" },
  {   1, "mode1" },
  { 0, NULL }
};


static int
dissect_rnsap_HARQ_Preamble_Mode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_HARQ_Preamble_Mode(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HARQ_Preamble_Mode(tvb, offset, actx, tree, hf_rnsap_id_HARQ_Preamble_Mode);
}


static const value_string rnsap_SynchronisationIndicator_vals[] = {
  {   0, "timingMaintainedSynchronisation" },
  { 0, NULL }
};


static int
dissect_rnsap_SynchronisationIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_id_SynchronisationIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SynchronisationIndicator(tvb, offset, actx, tree, hf_rnsap_id_SynchronisationIndicator);
}


static const value_string rnsap_UL_DPDCHIndicatorEDCH_vals[] = {
  {   0, "uL-DPDCH-present" },
  {   1, "uL-DPDCH-not-present" },
  { 0, NULL }
};


static int
dissect_rnsap_UL_DPDCHIndicatorEDCH(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_UL_DPDCHIndicatorEDCH(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_UL_DPDCHIndicatorEDCH(tvb, offset, actx, tree, hf_rnsap_id_UL_DPDCHIndicatorEDCH);
}


static const value_string rnsap_Max_Set_E_DPDCHs_vals[] = {
  {   0, "vN64" },
  {   1, "vN32" },
  {   2, "vN16" },
  {   3, "vN8" },
  {   4, "vN4" },
  {   5, "v2xN4" },
  {   6, "v2xN2" },
  {   7, "v2xN2plus2xN4" },
  { 0, NULL }
};


static int
dissect_rnsap_Max_Set_E_DPDCHs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     8, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_maxSet_E_DPDCHs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Max_Set_E_DPDCHs(tvb, offset, actx, tree, hf_rnsap_maxSet_E_DPDCHs);
}



static int
dissect_rnsap_E_DCH_TFCI_Table_Index(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 1U, NULL, TRUE);

  return offset;
}
static int dissect_e_DCH_TFCI_Table_Index(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_E_DCH_TFCI_Table_Index(tvb, offset, actx, tree, hf_rnsap_e_DCH_TFCI_Table_Index);
}



static int
dissect_rnsap_E_TFCI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 127U, NULL, FALSE);

  return offset;
}
static int dissect_e_DCH_Min_Set_E_TFCI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_E_TFCI(tvb, offset, actx, tree, hf_rnsap_e_DCH_Min_Set_E_TFCI);
}
static int dissect_reference_E_TFCI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_E_TFCI(tvb, offset, actx, tree, hf_rnsap_reference_E_TFCI);
}



static int
dissect_rnsap_Reference_E_TFCI_PO(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 29U, NULL, FALSE);

  return offset;
}
static int dissect_reference_E_TFCI_PO(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Reference_E_TFCI_PO(tvb, offset, actx, tree, hf_rnsap_reference_E_TFCI_PO);
}


static const per_sequence_t Reference_E_TFCI_Information_Item_sequence[] = {
  { &hf_rnsap_reference_E_TFCI, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_E_TFCI },
  { &hf_rnsap_reference_E_TFCI_PO, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_Reference_E_TFCI_PO },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_Reference_E_TFCI_Information_Item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_Reference_E_TFCI_Information_Item, Reference_E_TFCI_Information_Item_sequence);

  return offset;
}
static int dissect_Reference_E_TFCI_Information_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Reference_E_TFCI_Information_Item(tvb, offset, actx, tree, hf_rnsap_Reference_E_TFCI_Information_item);
}


static const per_sequence_t Reference_E_TFCI_Information_sequence_of[1] = {
  { &hf_rnsap_Reference_E_TFCI_Information_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_Reference_E_TFCI_Information_Item },
};

static int
dissect_rnsap_Reference_E_TFCI_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_Reference_E_TFCI_Information, Reference_E_TFCI_Information_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_reference_E_TFCI_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Reference_E_TFCI_Information(tvb, offset, actx, tree, hf_rnsap_reference_E_TFCI_Information);
}


static const per_sequence_t E_TFCS_Information_sequence[] = {
  { &hf_rnsap_e_DCH_TFCI_Table_Index, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_E_DCH_TFCI_Table_Index },
  { &hf_rnsap_e_DCH_Min_Set_E_TFCI, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_E_TFCI },
  { &hf_rnsap_reference_E_TFCI_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_Reference_E_TFCI_Information },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_E_TFCS_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_E_TFCS_Information, E_TFCS_Information_sequence);

  return offset;
}
static int dissect_e_TFCS_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_E_TFCS_Information(tvb, offset, actx, tree, hf_rnsap_e_TFCS_Information);
}


static const value_string rnsap_E_TTI_vals[] = {
  {   0, "tti10" },
  {   1, "tti2" },
  { 0, NULL }
};


static int
dissect_rnsap_E_TTI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_e_TTI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_E_TTI(tvb, offset, actx, tree, hf_rnsap_e_TTI);
}



static int
dissect_rnsap_E_DPCCH_PO(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 8U, NULL, FALSE);

  return offset;
}
static int dissect_e_DPCCH_PO(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_E_DPCCH_PO(tvb, offset, actx, tree, hf_rnsap_e_DPCCH_PO);
}


static const per_sequence_t EDPCH_Information_FDD_sequence[] = {
  { &hf_rnsap_maxSet_E_DPDCHs, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_Max_Set_E_DPDCHs },
  { &hf_rnsap_punctureLimit , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PunctureLimit },
  { &hf_rnsap_e_TFCS_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_E_TFCS_Information },
  { &hf_rnsap_e_TTI         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_E_TTI },
  { &hf_rnsap_e_DPCCH_PO    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_E_DPCCH_PO },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_EDPCH_Information_FDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_EDPCH_Information_FDD, EDPCH_Information_FDD_sequence);

  return offset;
}
static int dissect_id_EDPCH_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_EDPCH_Information_FDD(tvb, offset, actx, tree, hf_rnsap_id_EDPCH_Information);
}



static int
dissect_rnsap_EDCH_MACdFlow_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 7U, NULL, FALSE);

  return offset;
}
static int dissect_eDCH_MACdFlow_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_EDCH_MACdFlow_ID(tvb, offset, actx, tree, hf_rnsap_eDCH_MACdFlow_ID);
}
static int dissect_eDSCH_MACdFlow_ID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_EDCH_MACdFlow_ID(tvb, offset, actx, tree, hf_rnsap_eDSCH_MACdFlow_ID);
}


static const per_sequence_t RL_Specific_EDCH_InfoItem_sequence[] = {
  { &hf_rnsap_eDCH_MACdFlow_ID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_EDCH_MACdFlow_ID },
  { &hf_rnsap_bindingID     , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_BindingID },
  { &hf_rnsap_transportLayerAddress, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TransportLayerAddress },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RL_Specific_EDCH_InfoItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RL_Specific_EDCH_InfoItem, RL_Specific_EDCH_InfoItem_sequence);

  return offset;
}
static int dissect_RL_Specific_EDCH_Information_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Specific_EDCH_InfoItem(tvb, offset, actx, tree, hf_rnsap_RL_Specific_EDCH_Information_item);
}


static const per_sequence_t RL_Specific_EDCH_Information_sequence_of[1] = {
  { &hf_rnsap_RL_Specific_EDCH_Information_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_Specific_EDCH_InfoItem },
};

static int
dissect_rnsap_RL_Specific_EDCH_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_RL_Specific_EDCH_Information, RL_Specific_EDCH_Information_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_id_RL_Specific_EDCH_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Specific_EDCH_Information(tvb, offset, actx, tree, hf_rnsap_id_RL_Specific_EDCH_Information);
}
static int dissect_id_EDCH_MACdFlows_To_Add(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_RL_Specific_EDCH_Information(tvb, offset, actx, tree, hf_rnsap_id_EDCH_MACdFlows_To_Add);
}


static const value_string rnsap_EDCH_RL_Indication_vals[] = {
  {   0, "eDCH" },
  {   1, "non-EDCH" },
  { 0, NULL }
};


static int
dissect_rnsap_EDCH_RL_Indication(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_EDCH_RL_Indication(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_EDCH_RL_Indication(tvb, offset, actx, tree, hf_rnsap_id_EDCH_RL_Indication);
}



static int
dissect_rnsap_MaxNr_Retransmissions_EDCH(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 15U, NULL, FALSE);

  return offset;
}
static int dissect_maxNr_Retransmissions_EDCH(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MaxNr_Retransmissions_EDCH(tvb, offset, actx, tree, hf_rnsap_maxNr_Retransmissions_EDCH);
}



static int
dissect_rnsap_E_DCH_HARQ_PO_FDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 6U, NULL, FALSE);

  return offset;
}
static int dissect_eDCH_HARQ_PO_FDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_E_DCH_HARQ_PO_FDD(tvb, offset, actx, tree, hf_rnsap_eDCH_HARQ_PO_FDD);
}



static int
dissect_rnsap_E_DCH_MACdFlow_Multiplexing_List(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     8, 8, FALSE, NULL);

  return offset;
}
static int dissect_eDCH_MACdFlow_Multiplexing_List(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_E_DCH_MACdFlow_Multiplexing_List(tvb, offset, actx, tree, hf_rnsap_eDCH_MACdFlow_Multiplexing_List);
}



static int
dissect_rnsap_Max_Bits_MACe_PDU_non_scheduled(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 19982U, NULL, FALSE);

  return offset;
}
static int dissect_maxBits_MACe_PDU_non_scheduled(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Max_Bits_MACe_PDU_non_scheduled(tvb, offset, actx, tree, hf_rnsap_maxBits_MACe_PDU_non_scheduled);
}



static int
dissect_rnsap_HARQ_Process_Allocation_2ms_EDCH(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     8, 8, FALSE, NULL);

  return offset;
}
static int dissect_hARQ_Process_Allocation_2ms(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HARQ_Process_Allocation_2ms_EDCH(tvb, offset, actx, tree, hf_rnsap_hARQ_Process_Allocation_2ms);
}


static const per_sequence_t E_DCH_Non_Scheduled_Transmission_Grant_Items_sequence[] = {
  { &hf_rnsap_maxBits_MACe_PDU_non_scheduled, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_Max_Bits_MACe_PDU_non_scheduled },
  { &hf_rnsap_hARQ_Process_Allocation_2ms, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_HARQ_Process_Allocation_2ms_EDCH },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_E_DCH_Non_Scheduled_Transmission_Grant_Items(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_E_DCH_Non_Scheduled_Transmission_Grant_Items, E_DCH_Non_Scheduled_Transmission_Grant_Items_sequence);

  return offset;
}
static int dissect_e_DCH_Non_Scheduled_Transmission_Grant(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_E_DCH_Non_Scheduled_Transmission_Grant_Items(tvb, offset, actx, tree, hf_rnsap_e_DCH_Non_Scheduled_Transmission_Grant);
}


static const value_string rnsap_E_DCH_Grant_Type_Information_vals[] = {
  {   0, "e-DCH-Non-Scheduled-Transmission-Grant" },
  {   1, "e-DCH-Scheduled-Transmission-Grant" },
  { 0, NULL }
};

static const per_choice_t E_DCH_Grant_Type_Information_choice[] = {
  {   0, &hf_rnsap_e_DCH_Non_Scheduled_Transmission_Grant, ASN1_EXTENSION_ROOT    , dissect_rnsap_E_DCH_Non_Scheduled_Transmission_Grant_Items },
  {   1, &hf_rnsap_e_DCH_Scheduled_Transmission_Grant, ASN1_EXTENSION_ROOT    , dissect_rnsap_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_E_DCH_Grant_Type_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_E_DCH_Grant_Type_Information, E_DCH_Grant_Type_Information_choice,
                                 NULL);

  return offset;
}
static int dissect_eDCH_Grant_Type_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_E_DCH_Grant_Type_Information(tvb, offset, actx, tree, hf_rnsap_eDCH_Grant_Type_Information);
}


static const value_string rnsap_BundlingModeIndicator_vals[] = {
  {   0, "bundling" },
  {   1, "no-bundling" },
  { 0, NULL }
};


static int
dissect_rnsap_BundlingModeIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_bundlingModeIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BundlingModeIndicator(tvb, offset, actx, tree, hf_rnsap_bundlingModeIndicator);
}



static int
dissect_rnsap_LogicalChannelID(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 15U, NULL, FALSE);

  return offset;
}
static int dissect_logicalChannelId(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_LogicalChannelID(tvb, offset, actx, tree, hf_rnsap_logicalChannelId);
}



static int
dissect_rnsap_MACes_Guaranteed_Bitrate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 16777215U, NULL, TRUE);

  return offset;
}
static int dissect_mACes_GuaranteedBitRate(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MACes_Guaranteed_Bitrate(tvb, offset, actx, tree, hf_rnsap_mACes_GuaranteedBitRate);
}



static int
dissect_rnsap_EDCH_DDI_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 63U, NULL, FALSE);

  return offset;
}
static int dissect_eDCH_DDI_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_EDCH_DDI_Value(tvb, offset, actx, tree, hf_rnsap_eDCH_DDI_Value);
}


static const per_sequence_t E_DCH_MACdPDU_SizeListItem_sequence[] = {
  { &hf_rnsap_mACdPDU_Size  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MACdPDU_Size },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_E_DCH_MACdPDU_SizeListItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_E_DCH_MACdPDU_SizeListItem, E_DCH_MACdPDU_SizeListItem_sequence);

  return offset;
}
static int dissect_E_DCH_MACdPDU_SizeList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_E_DCH_MACdPDU_SizeListItem(tvb, offset, actx, tree, hf_rnsap_E_DCH_MACdPDU_SizeList_item);
}
static int dissect_E_DCH_MACdPDU_SizeToModifyList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_E_DCH_MACdPDU_SizeListItem(tvb, offset, actx, tree, hf_rnsap_E_DCH_MACdPDU_SizeToModifyList_item);
}


static const per_sequence_t E_DCH_MACdPDU_SizeList_sequence_of[1] = {
  { &hf_rnsap_E_DCH_MACdPDU_SizeList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_E_DCH_MACdPDU_SizeListItem },
};

static int
dissect_rnsap_E_DCH_MACdPDU_SizeList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_E_DCH_MACdPDU_SizeList, E_DCH_MACdPDU_SizeList_sequence_of,
                                                  1, 64);

  return offset;
}
static int dissect_mACd_PDU_Size_List(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_E_DCH_MACdPDU_SizeList(tvb, offset, actx, tree, hf_rnsap_mACd_PDU_Size_List);
}


static const per_sequence_t E_DCH_LogicalChannelInformationItem_sequence[] = {
  { &hf_rnsap_logicalChannelId, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_LogicalChannelID },
  { &hf_rnsap_schedulingPriorityIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_SchedulingPriorityIndicator },
  { &hf_rnsap_mACes_GuaranteedBitRate, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_MACes_Guaranteed_Bitrate },
  { &hf_rnsap_eDCH_DDI_Value, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_EDCH_DDI_Value },
  { &hf_rnsap_mACd_PDU_Size_List, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_E_DCH_MACdPDU_SizeList },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_E_DCH_LogicalChannelInformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_E_DCH_LogicalChannelInformationItem, E_DCH_LogicalChannelInformationItem_sequence);

  return offset;
}
static int dissect_E_DCH_LogicalChannelInformation_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_E_DCH_LogicalChannelInformationItem(tvb, offset, actx, tree, hf_rnsap_E_DCH_LogicalChannelInformation_item);
}


static const per_sequence_t E_DCH_LogicalChannelInformation_sequence_of[1] = {
  { &hf_rnsap_E_DCH_LogicalChannelInformation_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_E_DCH_LogicalChannelInformationItem },
};

static int
dissect_rnsap_E_DCH_LogicalChannelInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_E_DCH_LogicalChannelInformation, E_DCH_LogicalChannelInformation_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_eDCHLogicalChannelInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_E_DCH_LogicalChannelInformation(tvb, offset, actx, tree, hf_rnsap_eDCHLogicalChannelInformation);
}
static int dissect_eDCH_LogicalChannelToAdd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_E_DCH_LogicalChannelInformation(tvb, offset, actx, tree, hf_rnsap_eDCH_LogicalChannelToAdd);
}


static const per_sequence_t EDCH_MACdFlow_Specific_InfoItem_sequence[] = {
  { &hf_rnsap_eDSCH_MACdFlow_ID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_EDCH_MACdFlow_ID },
  { &hf_rnsap_allocationRetentionPriority, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_AllocationRetentionPriority },
  { &hf_rnsap_tnlQoS        , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TnlQos },
  { &hf_rnsap_payloadCRC_PresenceIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PayloadCRC_PresenceIndicator },
  { &hf_rnsap_maxNr_Retransmissions_EDCH, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_MaxNr_Retransmissions_EDCH },
  { &hf_rnsap_trafficClass  , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TrafficClass },
  { &hf_rnsap_eDCH_HARQ_PO_FDD, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_E_DCH_HARQ_PO_FDD },
  { &hf_rnsap_eDCH_MACdFlow_Multiplexing_List, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_E_DCH_MACdFlow_Multiplexing_List },
  { &hf_rnsap_eDCH_Grant_Type_Information, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_E_DCH_Grant_Type_Information },
  { &hf_rnsap_bundlingModeIndicator, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_BundlingModeIndicator },
  { &hf_rnsap_eDCHLogicalChannelInformation, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_E_DCH_LogicalChannelInformation },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_EDCH_MACdFlow_Specific_InfoItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_EDCH_MACdFlow_Specific_InfoItem, EDCH_MACdFlow_Specific_InfoItem_sequence);

  return offset;
}
static int dissect_EDCH_MACdFlow_Specific_InfoList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_EDCH_MACdFlow_Specific_InfoItem(tvb, offset, actx, tree, hf_rnsap_EDCH_MACdFlow_Specific_InfoList_item);
}


static const per_sequence_t EDCH_MACdFlow_Specific_InfoList_sequence_of[1] = {
  { &hf_rnsap_EDCH_MACdFlow_Specific_InfoList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_EDCH_MACdFlow_Specific_InfoItem },
};

static int
dissect_rnsap_EDCH_MACdFlow_Specific_InfoList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_EDCH_MACdFlow_Specific_InfoList, EDCH_MACdFlow_Specific_InfoList_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_eDCH_MACdFlow_Specific_Information1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_EDCH_MACdFlow_Specific_InfoList(tvb, offset, actx, tree, hf_rnsap_eDCH_MACdFlow_Specific_Information1);
}


static const per_sequence_t EDCH_MACdFlows_Information_sequence[] = {
  { &hf_rnsap_eDCH_MACdFlow_Specific_Information1, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_EDCH_MACdFlow_Specific_InfoList },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_EDCH_MACdFlows_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_EDCH_MACdFlows_Information, EDCH_MACdFlows_Information_sequence);

  return offset;
}
static int dissect_eDCH_MACdFlows_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_EDCH_MACdFlows_Information(tvb, offset, actx, tree, hf_rnsap_eDCH_MACdFlows_Information);
}


static const per_sequence_t EDCH_FDD_Information_sequence[] = {
  { &hf_rnsap_eDCH_MACdFlows_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_EDCH_MACdFlows_Information },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_EDCH_FDD_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_EDCH_FDD_Information, EDCH_FDD_Information_sequence);

  return offset;
}
static int dissect_id_EDCH_FDD_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_EDCH_FDD_Information(tvb, offset, actx, tree, hf_rnsap_id_EDCH_FDD_Information);
}


static const per_sequence_t EDCH_Serving_RL_in_this_DRNS_sequence[] = {
  { &hf_rnsap_e_DCH_Serving_RL_Id, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ID },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_EDCH_Serving_RL_in_this_DRNS(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_EDCH_Serving_RL_in_this_DRNS, EDCH_Serving_RL_in_this_DRNS_sequence);

  return offset;
}
static int dissect_e_DCH_Serving_RL_in_this_DRNS(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_EDCH_Serving_RL_in_this_DRNS(tvb, offset, actx, tree, hf_rnsap_e_DCH_Serving_RL_in_this_DRNS);
}


static const value_string rnsap_EDCH_Serving_RL_vals[] = {
  {   0, "e-DCH-Serving-RL-in-this-DRNS" },
  {   1, "e-DCH-Serving-RL-not-in-this-DRNS" },
  { 0, NULL }
};

static const per_choice_t EDCH_Serving_RL_choice[] = {
  {   0, &hf_rnsap_e_DCH_Serving_RL_in_this_DRNS, ASN1_EXTENSION_ROOT    , dissect_rnsap_EDCH_Serving_RL_in_this_DRNS },
  {   1, &hf_rnsap_e_DCH_Serving_RL_not_in_this_DRNS, ASN1_EXTENSION_ROOT    , dissect_rnsap_NULL },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_EDCH_Serving_RL(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_EDCH_Serving_RL, EDCH_Serving_RL_choice,
                                 NULL);

  return offset;
}
static int dissect_id_Serving_EDCHRL_Id(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_EDCH_Serving_RL(tvb, offset, actx, tree, hf_rnsap_id_Serving_EDCHRL_Id);
}



static int
dissect_rnsap_E_RNTI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 65535U, NULL, FALSE);

  return offset;
}
static int dissect_primary_e_RNTI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_E_RNTI(tvb, offset, actx, tree, hf_rnsap_primary_e_RNTI);
}
static int dissect_secondary_e_RNTI(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_E_RNTI(tvb, offset, actx, tree, hf_rnsap_secondary_e_RNTI);
}



static int
dissect_rnsap_ERGCH_SignatureSequence(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 39U, NULL, FALSE);

  return offset;
}
static int dissect_eRGCH_SignatureSequence(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ERGCH_SignatureSequence(tvb, offset, actx, tree, hf_rnsap_eRGCH_SignatureSequence);
}



static int
dissect_rnsap_EHICH_SignatureSequence(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 39U, NULL, FALSE);

  return offset;
}
static int dissect_eHICH_SignatureSequence(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_EHICH_SignatureSequence(tvb, offset, actx, tree, hf_rnsap_eHICH_SignatureSequence);
}



static int
dissect_rnsap_E_Serving_Grant_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 31U, NULL, TRUE);

  return offset;
}
static int dissect_serving_Grant_Value(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_E_Serving_Grant_Value(tvb, offset, actx, tree, hf_rnsap_serving_Grant_Value);
}


static const value_string rnsap_E_Primary_Secondary_Grant_Selector_vals[] = {
  {   0, "primary" },
  {   1, "secondary" },
  { 0, NULL }
};


static int
dissect_rnsap_E_Primary_Secondary_Grant_Selector(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_primary_Secondary_Grant_Selector(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_E_Primary_Secondary_Grant_Selector(tvb, offset, actx, tree, hf_rnsap_primary_Secondary_Grant_Selector);
}


static const value_string rnsap_E_RGCH_Release_Indicator_vals[] = {
  {   0, "e-RGCHreleased" },
  { 0, NULL }
};


static int
dissect_rnsap_E_RGCH_Release_Indicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_e_RGCH_Release_Indicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_E_RGCH_Release_Indicator(tvb, offset, actx, tree, hf_rnsap_e_RGCH_Release_Indicator);
}


static const per_sequence_t EDCH_FDD_DL_ControlChannelInformation_sequence[] = {
  { &hf_rnsap_eAGCH_ERGCH_EHICH_FDD_ScramblingCode, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DL_ScramblingCode },
  { &hf_rnsap_eAGCH_ChannelisationCode, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_FDD_DL_ChannelisationCodeNumber },
  { &hf_rnsap_primary_e_RNTI, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_E_RNTI },
  { &hf_rnsap_secondary_e_RNTI, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_E_RNTI },
  { &hf_rnsap_eRGCH_EHICH_ChannelisationCode, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_FDD_DL_ChannelisationCodeNumber },
  { &hf_rnsap_eRGCH_SignatureSequence, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ERGCH_SignatureSequence },
  { &hf_rnsap_eHICH_SignatureSequence, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_EHICH_SignatureSequence },
  { &hf_rnsap_serving_Grant_Value, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_E_Serving_Grant_Value },
  { &hf_rnsap_primary_Secondary_Grant_Selector, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_E_Primary_Secondary_Grant_Selector },
  { &hf_rnsap_e_RGCH_Release_Indicator, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_E_RGCH_Release_Indicator },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_EDCH_FDD_DL_ControlChannelInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_EDCH_FDD_DL_ControlChannelInformation, EDCH_FDD_DL_ControlChannelInformation_sequence);

  return offset;
}
static int dissect_id_EDCH_FDD_DL_ControlChannelInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_EDCH_FDD_DL_ControlChannelInformation(tvb, offset, actx, tree, hf_rnsap_id_EDCH_FDD_DL_ControlChannelInformation);
}
static int dissect_eDCH_FDD_DL_ControlChannelInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_EDCH_FDD_DL_ControlChannelInformation(tvb, offset, actx, tree, hf_rnsap_eDCH_FDD_DL_ControlChannelInformation);
}


static const per_sequence_t EDCH_FDD_InformationResponseItem_sequence[] = {
  { &hf_rnsap_eDCH_MACdFlow_ID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_EDCH_MACdFlow_ID },
  { &hf_rnsap_bindingID     , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_BindingID },
  { &hf_rnsap_transportLayerAddress, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TransportLayerAddress },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_EDCH_FDD_InformationResponseItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_EDCH_FDD_InformationResponseItem, EDCH_FDD_InformationResponseItem_sequence);

  return offset;
}
static int dissect_EDCH_FDD_InformationResponse_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_EDCH_FDD_InformationResponseItem(tvb, offset, actx, tree, hf_rnsap_EDCH_FDD_InformationResponse_item);
}


static const per_sequence_t EDCH_FDD_InformationResponse_sequence_of[1] = {
  { &hf_rnsap_EDCH_FDD_InformationResponse_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_EDCH_FDD_InformationResponseItem },
};

static int
dissect_rnsap_EDCH_FDD_InformationResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_EDCH_FDD_InformationResponse, EDCH_FDD_InformationResponse_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_id_EDCH_FDD_InformationResponse(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_EDCH_FDD_InformationResponse(tvb, offset, actx, tree, hf_rnsap_id_EDCH_FDD_InformationResponse);
}


static const per_sequence_t E_DCH_MACdPDU_SizeToModifyList_sequence_of[1] = {
  { &hf_rnsap_E_DCH_MACdPDU_SizeToModifyList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_E_DCH_MACdPDU_SizeListItem },
};

static int
dissect_rnsap_E_DCH_MACdPDU_SizeToModifyList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_E_DCH_MACdPDU_SizeToModifyList, E_DCH_MACdPDU_SizeToModifyList_sequence_of,
                                                  0, 64);

  return offset;
}
static int dissect_mACd_PDU_Size_List1(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_E_DCH_MACdPDU_SizeToModifyList(tvb, offset, actx, tree, hf_rnsap_mACd_PDU_Size_List1);
}


static const per_sequence_t E_DCH_LogicalChannelToModifyItem_sequence[] = {
  { &hf_rnsap_logicalChannelId, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_LogicalChannelID },
  { &hf_rnsap_schedulingPriorityIndicator, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_SchedulingPriorityIndicator },
  { &hf_rnsap_mACes_GuaranteedBitRate, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_MACes_Guaranteed_Bitrate },
  { &hf_rnsap_eDCH_DDI_Value, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_EDCH_DDI_Value },
  { &hf_rnsap_mACd_PDU_Size_List1, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_E_DCH_MACdPDU_SizeToModifyList },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_E_DCH_LogicalChannelToModifyItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_E_DCH_LogicalChannelToModifyItem, E_DCH_LogicalChannelToModifyItem_sequence);

  return offset;
}
static int dissect_E_DCH_LogicalChannelToModify_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_E_DCH_LogicalChannelToModifyItem(tvb, offset, actx, tree, hf_rnsap_E_DCH_LogicalChannelToModify_item);
}


static const per_sequence_t E_DCH_LogicalChannelToModify_sequence_of[1] = {
  { &hf_rnsap_E_DCH_LogicalChannelToModify_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_E_DCH_LogicalChannelToModifyItem },
};

static int
dissect_rnsap_E_DCH_LogicalChannelToModify(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_E_DCH_LogicalChannelToModify, E_DCH_LogicalChannelToModify_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_eDCH_LogicalChannelToModify(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_E_DCH_LogicalChannelToModify(tvb, offset, actx, tree, hf_rnsap_eDCH_LogicalChannelToModify);
}


static const per_sequence_t E_DCH_LogicalChannelToDeleteItem_sequence[] = {
  { &hf_rnsap_logicalChannelId, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_LogicalChannelID },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_E_DCH_LogicalChannelToDeleteItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_E_DCH_LogicalChannelToDeleteItem, E_DCH_LogicalChannelToDeleteItem_sequence);

  return offset;
}
static int dissect_E_DCH_LogicalChannelToDelete_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_E_DCH_LogicalChannelToDeleteItem(tvb, offset, actx, tree, hf_rnsap_E_DCH_LogicalChannelToDelete_item);
}


static const per_sequence_t E_DCH_LogicalChannelToDelete_sequence_of[1] = {
  { &hf_rnsap_E_DCH_LogicalChannelToDelete_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_E_DCH_LogicalChannelToDeleteItem },
};

static int
dissect_rnsap_E_DCH_LogicalChannelToDelete(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_E_DCH_LogicalChannelToDelete, E_DCH_LogicalChannelToDelete_sequence_of,
                                                  1, 16);

  return offset;
}
static int dissect_eDCH_LogicalChannelToDelete(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_E_DCH_LogicalChannelToDelete(tvb, offset, actx, tree, hf_rnsap_eDCH_LogicalChannelToDelete);
}


static const per_sequence_t EDCH_MACdFlow_Specific_InfoToModifyItem_sequence[] = {
  { &hf_rnsap_eDSCH_MACdFlow_ID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_EDCH_MACdFlow_ID },
  { &hf_rnsap_allocationRetentionPriority, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_AllocationRetentionPriority },
  { &hf_rnsap_transportBearerRequestIndicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TransportBearerRequestIndicator },
  { &hf_rnsap_tnlQoS        , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TnlQos },
  { &hf_rnsap_maxNr_Retransmissions_EDCH, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_MaxNr_Retransmissions_EDCH },
  { &hf_rnsap_trafficClass  , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TrafficClass },
  { &hf_rnsap_eDCH_HARQ_PO_FDD, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_E_DCH_HARQ_PO_FDD },
  { &hf_rnsap_eDCH_MACdFlow_Multiplexing_List, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_E_DCH_MACdFlow_Multiplexing_List },
  { &hf_rnsap_eDCH_Grant_Type_Information, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_E_DCH_Grant_Type_Information },
  { &hf_rnsap_bundlingModeIndicator, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_BundlingModeIndicator },
  { &hf_rnsap_eDCH_LogicalChannelToAdd, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_E_DCH_LogicalChannelInformation },
  { &hf_rnsap_eDCH_LogicalChannelToModify, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_E_DCH_LogicalChannelToModify },
  { &hf_rnsap_eDCH_LogicalChannelToDelete, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_E_DCH_LogicalChannelToDelete },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_EDCH_MACdFlow_Specific_InfoToModifyItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_EDCH_MACdFlow_Specific_InfoToModifyItem, EDCH_MACdFlow_Specific_InfoToModifyItem_sequence);

  return offset;
}
static int dissect_EDCH_MACdFlow_Specific_InfoToModifyList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_EDCH_MACdFlow_Specific_InfoToModifyItem(tvb, offset, actx, tree, hf_rnsap_EDCH_MACdFlow_Specific_InfoToModifyList_item);
}


static const per_sequence_t EDCH_MACdFlow_Specific_InfoToModifyList_sequence_of[1] = {
  { &hf_rnsap_EDCH_MACdFlow_Specific_InfoToModifyList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_EDCH_MACdFlow_Specific_InfoToModifyItem },
};

static int
dissect_rnsap_EDCH_MACdFlow_Specific_InfoToModifyList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_EDCH_MACdFlow_Specific_InfoToModifyList, EDCH_MACdFlow_Specific_InfoToModifyList_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_eDCH_MACdFlow_Specific_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_EDCH_MACdFlow_Specific_InfoToModifyList(tvb, offset, actx, tree, hf_rnsap_eDCH_MACdFlow_Specific_Information);
}


static const per_sequence_t EDCH_FDD_Information_To_Modify_sequence[] = {
  { &hf_rnsap_eDCH_MACdFlow_Specific_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_EDCH_MACdFlow_Specific_InfoToModifyList },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_EDCH_FDD_Information_To_Modify(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_EDCH_FDD_Information_To_Modify, EDCH_FDD_Information_To_Modify_sequence);

  return offset;
}
static int dissect_id_EDCH_FDD_Information_To_Modify(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_EDCH_FDD_Information_To_Modify(tvb, offset, actx, tree, hf_rnsap_id_EDCH_FDD_Information_To_Modify);
}


static const per_sequence_t EDCH_MACdFlows_To_Delete_Item_sequence[] = {
  { &hf_rnsap_eDSCH_MACdFlow_ID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_EDCH_MACdFlow_ID },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_EDCH_MACdFlows_To_Delete_Item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_EDCH_MACdFlows_To_Delete_Item, EDCH_MACdFlows_To_Delete_Item_sequence);

  return offset;
}
static int dissect_EDCH_MACdFlows_To_Delete_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_EDCH_MACdFlows_To_Delete_Item(tvb, offset, actx, tree, hf_rnsap_EDCH_MACdFlows_To_Delete_item);
}


static const per_sequence_t EDCH_MACdFlows_To_Delete_sequence_of[1] = {
  { &hf_rnsap_EDCH_MACdFlows_To_Delete_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_EDCH_MACdFlows_To_Delete_Item },
};

static int
dissect_rnsap_EDCH_MACdFlows_To_Delete(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_EDCH_MACdFlows_To_Delete, EDCH_MACdFlows_To_Delete_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_id_EDCH_MACdFlows_To_Delete(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_EDCH_MACdFlows_To_Delete(tvb, offset, actx, tree, hf_rnsap_id_EDCH_MACdFlows_To_Delete);
}


static const per_sequence_t EDPCH_Information_RLReconfRequest_FDD_sequence[] = {
  { &hf_rnsap_e_TFCS_Information, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_E_TFCS_Information },
  { &hf_rnsap_e_DPCCH_PO    , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_E_DPCCH_PO },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_EDPCH_Information_RLReconfRequest_FDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_EDPCH_Information_RLReconfRequest_FDD, EDPCH_Information_RLReconfRequest_FDD_sequence);

  return offset;
}
static int dissect_id_EDPCH_Information_RLReconfRequest_FDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_EDPCH_Information_RLReconfRequest_FDD(tvb, offset, actx, tree, hf_rnsap_id_EDPCH_Information_RLReconfRequest_FDD);
}


static const per_sequence_t EDCH_MacdFlowSpecificInformationList_RL_PreemptRequiredInd_sequence_of[1] = {
  { &hf_rnsap_EDCH_MacdFlowSpecificInformationList_RL_PreemptRequiredInd_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_EDCH_MacdFlowSpecificInformationList_RL_PreemptRequiredInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_EDCH_MacdFlowSpecificInformationList_RL_PreemptRequiredInd, EDCH_MacdFlowSpecificInformationList_RL_PreemptRequiredInd_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_id_EDCH_MacdFlowSpecificInformationList_RL_PreemptRequiredInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_EDCH_MacdFlowSpecificInformationList_RL_PreemptRequiredInd(tvb, offset, actx, tree, hf_rnsap_id_EDCH_MacdFlowSpecificInformationList_RL_PreemptRequiredInd);
}


static const per_sequence_t EDCH_MacdFlowSpecificInformationItem_RL_PreemptRequiredInd_sequence[] = {
  { &hf_rnsap_eDCH_MACdFlow_ID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_EDCH_MACdFlow_ID },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_EDCH_MacdFlowSpecificInformationItem_RL_PreemptRequiredInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_EDCH_MacdFlowSpecificInformationItem_RL_PreemptRequiredInd, EDCH_MacdFlowSpecificInformationItem_RL_PreemptRequiredInd_sequence);

  return offset;
}
static int dissect_id_EDCH_MacdFlowSpecificInformationItem_RL_PreemptRequiredInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_EDCH_MacdFlowSpecificInformationItem_RL_PreemptRequiredInd(tvb, offset, actx, tree, hf_rnsap_id_EDCH_MacdFlowSpecificInformationItem_RL_PreemptRequiredInd);
}


static const per_sequence_t EDCH_MacdFlowSpecificInformationList_RL_CongestInd_sequence_of[1] = {
  { &hf_rnsap_EDCH_MacdFlowSpecificInformationList_RL_CongestInd_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Single_Container },
};

static int
dissect_rnsap_EDCH_MacdFlowSpecificInformationList_RL_CongestInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_EDCH_MacdFlowSpecificInformationList_RL_CongestInd, EDCH_MacdFlowSpecificInformationList_RL_CongestInd_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_id_EDCH_MacdFlowSpecificInformationList_RL_CongestInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_EDCH_MacdFlowSpecificInformationList_RL_CongestInd(tvb, offset, actx, tree, hf_rnsap_id_EDCH_MacdFlowSpecificInformationList_RL_CongestInd);
}


static const per_sequence_t EDCH_MacdFlowSpecificInformationItem_RL_CongestInd_sequence[] = {
  { &hf_rnsap_eDCH_MACdFlow_ID, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_EDCH_MACdFlow_ID },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_EDCH_MacdFlowSpecificInformationItem_RL_CongestInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_EDCH_MacdFlowSpecificInformationItem_RL_CongestInd, EDCH_MacdFlowSpecificInformationItem_RL_CongestInd_sequence);

  return offset;
}
static int dissect_id_EDCH_MacdFlowSpecificInformationItem_RL_CongestInd(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_EDCH_MacdFlowSpecificInformationItem_RL_CongestInd(tvb, offset, actx, tree, hf_rnsap_id_EDCH_MacdFlowSpecificInformationItem_RL_CongestInd);
}



static int
dissect_rnsap_AccessPointName(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_octet_string(tvb, offset, actx, tree, hf_index,
                                       1, 255, NULL);

  return offset;
}
static int dissect_accessPointName(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_AccessPointName(tvb, offset, actx, tree, hf_rnsap_accessPointName);
}



static int
dissect_rnsap_IPMulticastAddress(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_octet_string(tvb, offset, actx, tree, hf_index,
                                       4, 16, NULL);

  return offset;
}
static int dissect_iPMulticastAddress(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_IPMulticastAddress(tvb, offset, actx, tree, hf_rnsap_iPMulticastAddress);
}


static const per_sequence_t MBMS_Bearer_Service_Full_Address_sequence[] = {
  { &hf_rnsap_accessPointName, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_AccessPointName },
  { &hf_rnsap_iPMulticastAddress, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_IPMulticastAddress },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_MBMS_Bearer_Service_Full_Address(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_MBMS_Bearer_Service_Full_Address, MBMS_Bearer_Service_Full_Address_sequence);

  return offset;
}
static int dissect_id_MBMS_Bearer_Service_Full_Address(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MBMS_Bearer_Service_Full_Address(tvb, offset, actx, tree, hf_rnsap_id_MBMS_Bearer_Service_Full_Address);
}


static const value_string rnsap_Initial_DL_DPCH_TimingAdjustment_Allowed_vals[] = {
  {   0, "initial-DL-DPCH-TimingAdjustment-Allowed" },
  { 0, NULL }
};


static int
dissect_rnsap_Initial_DL_DPCH_TimingAdjustment_Allowed(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_Initial_DL_DPCH_TimingAdjustment_Allowed(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Initial_DL_DPCH_TimingAdjustment_Allowed(tvb, offset, actx, tree, hf_rnsap_id_Initial_DL_DPCH_TimingAdjustment_Allowed);
}


static const value_string rnsap_User_Plane_Congestion_Fields_Inclusion_vals[] = {
  {   0, "shall-be-included" },
  { 0, NULL }
};


static int
dissect_rnsap_User_Plane_Congestion_Fields_Inclusion(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_User_Plane_Congestion_Fields_Inclusion(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_User_Plane_Congestion_Fields_Inclusion(tvb, offset, actx, tree, hf_rnsap_id_User_Plane_Congestion_Fields_Inclusion);
}


static const value_string rnsap_HARQ_Preamble_Mode_Activation_Indicator_vals[] = {
  {   0, "harqPreambleModeSupported" },
  { 0, NULL }
};


static int
dissect_rnsap_HARQ_Preamble_Mode_Activation_Indicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     1, NULL, FALSE, 0, NULL);

  return offset;
}
static int dissect_id_HARQ_Preamble_Mode_Activation_Indicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_HARQ_Preamble_Mode_Activation_Indicator(tvb, offset, actx, tree, hf_rnsap_id_HARQ_Preamble_Mode_Activation_Indicator);
}


static const per_sequence_t Multiple_DedicatedMeasurementValueItem_TDD_DM_Rsp_sequence[] = {
  { &hf_rnsap_dPCH_ID       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DPCH_ID },
  { &hf_rnsap_dedicatedMeasurementValue, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DedicatedMeasurementValue },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_Multiple_DedicatedMeasurementValueItem_TDD_DM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_Multiple_DedicatedMeasurementValueItem_TDD_DM_Rsp, Multiple_DedicatedMeasurementValueItem_TDD_DM_Rsp_sequence);

  return offset;
}
static int dissect_Multiple_DedicatedMeasurementValueList_TDD_DM_Rsp_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Multiple_DedicatedMeasurementValueItem_TDD_DM_Rsp(tvb, offset, actx, tree, hf_rnsap_Multiple_DedicatedMeasurementValueList_TDD_DM_Rsp_item);
}


static const per_sequence_t Multiple_DedicatedMeasurementValueList_TDD_DM_Rsp_sequence_of[1] = {
  { &hf_rnsap_Multiple_DedicatedMeasurementValueList_TDD_DM_Rsp_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_Multiple_DedicatedMeasurementValueItem_TDD_DM_Rsp },
};

static int
dissect_rnsap_Multiple_DedicatedMeasurementValueList_TDD_DM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_Multiple_DedicatedMeasurementValueList_TDD_DM_Rsp, Multiple_DedicatedMeasurementValueList_TDD_DM_Rsp_sequence_of,
                                                  1, 239);

  return offset;
}
static int dissect_id_multiple_DedicatedMeasurementValueList_TDD_DM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Multiple_DedicatedMeasurementValueList_TDD_DM_Rsp(tvb, offset, actx, tree, hf_rnsap_id_multiple_DedicatedMeasurementValueList_TDD_DM_Rsp);
}


static const per_sequence_t Multiple_DedicatedMeasurementValueItem_LCR_TDD_DM_Rsp_sequence[] = {
  { &hf_rnsap_dPCH_ID       , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DPCH_ID },
  { &hf_rnsap_dedicatedMeasurementValue, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_DedicatedMeasurementValue },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_Multiple_DedicatedMeasurementValueItem_LCR_TDD_DM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_Multiple_DedicatedMeasurementValueItem_LCR_TDD_DM_Rsp, Multiple_DedicatedMeasurementValueItem_LCR_TDD_DM_Rsp_sequence);

  return offset;
}
static int dissect_Multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Multiple_DedicatedMeasurementValueItem_LCR_TDD_DM_Rsp(tvb, offset, actx, tree, hf_rnsap_Multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp_item);
}


static const per_sequence_t Multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp_sequence_of[1] = {
  { &hf_rnsap_Multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_Multiple_DedicatedMeasurementValueItem_LCR_TDD_DM_Rsp },
};

static int
dissect_rnsap_Multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_Multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp, Multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp_sequence_of,
                                                  1, 239);

  return offset;
}
static int dissect_id_multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp(tvb, offset, actx, tree, hf_rnsap_id_multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp);
}


static const per_sequence_t AffectedUEInformationForMBMS_sequence_of[1] = {
  { &hf_rnsap_AffectedUEInformationForMBMS_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_S_RNTI },
};

static int
dissect_rnsap_AffectedUEInformationForMBMS(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_AffectedUEInformationForMBMS, AffectedUEInformationForMBMS_sequence_of,
                                                  1, 4096);

  return offset;
}
static int dissect_affectedUEInformationForMBMS(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_AffectedUEInformationForMBMS(tvb, offset, actx, tree, hf_rnsap_affectedUEInformationForMBMS);
}


static const per_sequence_t MBMSChannelTypeCellList_sequence[] = {
  { &hf_rnsap_c_ID          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_C_ID },
  { &hf_rnsap_affectedUEInformationForMBMS, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_AffectedUEInformationForMBMS },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_MBMSChannelTypeCellList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_MBMSChannelTypeCellList, MBMSChannelTypeCellList_sequence);

  return offset;
}
static int dissect_NotProvidedCellList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MBMSChannelTypeCellList(tvb, offset, actx, tree, hf_rnsap_NotProvidedCellList_item);
}
static int dissect_PTMCellList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MBMSChannelTypeCellList(tvb, offset, actx, tree, hf_rnsap_PTMCellList_item);
}
static int dissect_PTPCellList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MBMSChannelTypeCellList(tvb, offset, actx, tree, hf_rnsap_PTPCellList_item);
}


static const per_sequence_t PTMCellList_sequence_of[1] = {
  { &hf_rnsap_PTMCellList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_MBMSChannelTypeCellList },
};

static int
dissect_rnsap_PTMCellList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_PTMCellList, PTMCellList_sequence_of,
                                                  1, 65536);

  return offset;
}
static int dissect_pTM_Cell_List(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PTMCellList(tvb, offset, actx, tree, hf_rnsap_pTM_Cell_List);
}


static const per_sequence_t PTPCellList_sequence_of[1] = {
  { &hf_rnsap_PTPCellList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_MBMSChannelTypeCellList },
};

static int
dissect_rnsap_PTPCellList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_PTPCellList, PTPCellList_sequence_of,
                                                  1, 65536);

  return offset;
}
static int dissect_pTP_Cell_List(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PTPCellList(tvb, offset, actx, tree, hf_rnsap_pTP_Cell_List);
}


static const per_sequence_t NotProvidedCellList_sequence_of[1] = {
  { &hf_rnsap_NotProvidedCellList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_MBMSChannelTypeCellList },
};

static int
dissect_rnsap_NotProvidedCellList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_NotProvidedCellList, NotProvidedCellList_sequence_of,
                                                  1, 65536);

  return offset;
}
static int dissect_not_Provided_Cell_List(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_NotProvidedCellList(tvb, offset, actx, tree, hf_rnsap_not_Provided_Cell_List);
}


static const per_sequence_t MBMSChannelTypeInfo_sequence[] = {
  { &hf_rnsap_tMGI          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TMGI },
  { &hf_rnsap_pTM_Cell_List , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_PTMCellList },
  { &hf_rnsap_pTP_Cell_List , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_PTPCellList },
  { &hf_rnsap_not_Provided_Cell_List, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_NotProvidedCellList },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_MBMSChannelTypeInfo(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_MBMSChannelTypeInfo, MBMSChannelTypeInfo_sequence);

  return offset;
}
static int dissect_mBMSChannelTypeInfo(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MBMSChannelTypeInfo(tvb, offset, actx, tree, hf_rnsap_mBMSChannelTypeInfo);
}


static const per_sequence_t CorrespondingCells_sequence_of[1] = {
  { &hf_rnsap_CorrespondingCells_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_C_ID },
};

static int
dissect_rnsap_CorrespondingCells(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_CorrespondingCells, CorrespondingCells_sequence_of,
                                                  1, 65536);

  return offset;
}
static int dissect_correspondingCells(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_CorrespondingCells(tvb, offset, actx, tree, hf_rnsap_correspondingCells);
}


static const per_sequence_t AdditionalPreferredFrequencyItem_sequence[] = {
  { &hf_rnsap_dL_UARFCN     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UARFCN },
  { &hf_rnsap_correspondingCells, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CorrespondingCells },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_AdditionalPreferredFrequencyItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_AdditionalPreferredFrequencyItem, AdditionalPreferredFrequencyItem_sequence);

  return offset;
}
static int dissect_AdditionalPreferredFrequency_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_AdditionalPreferredFrequencyItem(tvb, offset, actx, tree, hf_rnsap_AdditionalPreferredFrequency_item);
}


static const per_sequence_t AdditionalPreferredFrequency_sequence_of[1] = {
  { &hf_rnsap_AdditionalPreferredFrequency_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_AdditionalPreferredFrequencyItem },
};

static int
dissect_rnsap_AdditionalPreferredFrequency(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_AdditionalPreferredFrequency, AdditionalPreferredFrequency_sequence_of,
                                                  1, 8);

  return offset;
}
static int dissect_additionalPreferredFrequency(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_AdditionalPreferredFrequency(tvb, offset, actx, tree, hf_rnsap_additionalPreferredFrequency);
}


static const per_sequence_t PreferredFrequencyLayerInfo_sequence[] = {
  { &hf_rnsap_defaultPreferredFrequency, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_UARFCN },
  { &hf_rnsap_additionalPreferredFrequency, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_AdditionalPreferredFrequency },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_PreferredFrequencyLayerInfo(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_PreferredFrequencyLayerInfo, PreferredFrequencyLayerInfo_sequence);

  return offset;
}
static int dissect_preferredFrequencyLayerInfo(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PreferredFrequencyLayerInfo(tvb, offset, actx, tree, hf_rnsap_preferredFrequencyLayerInfo);
}


static const per_sequence_t MBMSPreferredFreqLayerInfo_sequence[] = {
  { &hf_rnsap_tMGI          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TMGI },
  { &hf_rnsap_preferredFrequencyLayerInfo, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PreferredFrequencyLayerInfo },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_MBMSPreferredFreqLayerInfo(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_MBMSPreferredFreqLayerInfo, MBMSPreferredFreqLayerInfo_sequence);

  return offset;
}
static int dissect_mBMSPreferredFreqLayerInfo(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MBMSPreferredFreqLayerInfo(tvb, offset, actx, tree, hf_rnsap_mBMSPreferredFreqLayerInfo);
}


static const per_sequence_t ProvidedInformation_sequence[] = {
  { &hf_rnsap_mBMSChannelTypeInfo, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_MBMSChannelTypeInfo },
  { &hf_rnsap_mBMSPreferredFreqLayerInfo, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_MBMSPreferredFreqLayerInfo },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_ProvidedInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_ProvidedInformation, ProvidedInformation_sequence);

  return offset;
}
static int dissect_id_ProvidedInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProvidedInformation(tvb, offset, actx, tree, hf_rnsap_id_ProvidedInformation);
}


static const per_sequence_t MBMS_Bearer_ServiceItemTDD_PFL_sequence[] = {
  { &hf_rnsap_tmgi          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TMGI },
  { &hf_rnsap_transmissionMode, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TransmissionMode },
  { &hf_rnsap_preferredFrequencyLayer, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_UARFCN },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_MBMS_Bearer_ServiceItemTDD_PFL(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_MBMS_Bearer_ServiceItemTDD_PFL, MBMS_Bearer_ServiceItemTDD_PFL_sequence);

  return offset;
}
static int dissect_Active_MBMS_Bearer_Service_ListTDD_PFL_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MBMS_Bearer_ServiceItemTDD_PFL(tvb, offset, actx, tree, hf_rnsap_Active_MBMS_Bearer_Service_ListTDD_PFL_item);
}


static const per_sequence_t Active_MBMS_Bearer_Service_ListTDD_PFL_sequence_of[1] = {
  { &hf_rnsap_Active_MBMS_Bearer_Service_ListTDD_PFL_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_MBMS_Bearer_ServiceItemTDD_PFL },
};

static int
dissect_rnsap_Active_MBMS_Bearer_Service_ListTDD_PFL(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_Active_MBMS_Bearer_Service_ListTDD_PFL, Active_MBMS_Bearer_Service_ListTDD_PFL_sequence_of,
                                                  1, 256);

  return offset;
}
static int dissect_id_Active_MBMS_Bearer_ServiceFDD_PFL(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Active_MBMS_Bearer_Service_ListTDD_PFL(tvb, offset, actx, tree, hf_rnsap_id_Active_MBMS_Bearer_ServiceFDD_PFL);
}
static int dissect_id_Active_MBMS_Bearer_ServiceTDD_PFL(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Active_MBMS_Bearer_Service_ListTDD_PFL(tvb, offset, actx, tree, hf_rnsap_id_Active_MBMS_Bearer_ServiceTDD_PFL);
}


static const value_string rnsap_FrequencyBandIndicator_vals[] = {
  {   0, "bandI" },
  {   1, "bandII" },
  {   2, "bandIII" },
  {   3, "bandIV" },
  {   4, "bandV" },
  {   5, "bandVI" },
  {   6, "bandVII" },
  {   7, "bandVIII" },
  {   8, "bandIX" },
  {   9, "bandX" },
  {  10, "bandXI" },
  {  11, "bandXII" },
  {  12, "bandXIII" },
  {  13, "bandXIV" },
  {  14, "bandXV" },
  {  15, "bandXVI" },
  {  16, "bandXVII" },
  {  17, "bandXVIII" },
  {  18, "bandXIX" },
  {  19, "bandXX" },
  {  20, "bandXXI" },
  {  21, "bandXXII" },
  { 0, NULL }
};


static int
dissect_rnsap_FrequencyBandIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     22, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_id_FrequencyBandIndicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_FrequencyBandIndicator(tvb, offset, actx, tree, hf_rnsap_id_FrequencyBandIndicator);
}


static const value_string rnsap_DummyProtocolIE_vals[] = {
  {   0, "id-AllowedQueuingTime" },
  {   1, "id-Allowed-Rate-Information" },
  {   2, "id-AntennaColocationIndicator" },
  {   3, "id-BindingID" },
  {   4, "id-C-ID" },
  {   5, "id-C-RNTI" },
  {   6, "id-Cell-Capacity-Class-Value" },
  {   7, "id-CFN" },
  {   8, "id-CN-CS-DomainIdentifier" },
  {   9, "id-CN-PS-DomainIdentifier" },
  {  10, "id-Cause" },
  {  11, "id-CoverageIndicator" },
  {  12, "id-CriticalityDiagnostics" },
  {  13, "id-ContextInfoItem-Reset" },
  {  14, "id-ContextGroupInfoItem-Reset" },
  {  15, "id-D-RNTI" },
  {  16, "id-D-RNTI-ReleaseIndication" },
  {  17, "id-DCHs-to-Add-FDD" },
  {  18, "id-DCHs-to-Add-TDD" },
  {  19, "id-DCH-DeleteList-RL-ReconfPrepFDD" },
  {  20, "id-DCH-DeleteList-RL-ReconfPrepTDD" },
  {  21, "id-DCH-DeleteList-RL-ReconfRqstFDD" },
  {  22, "id-DCH-DeleteList-RL-ReconfRqstTDD" },
  {  23, "id-DCH-FDD-Information" },
  {  24, "id-DCH-TDD-Information" },
  {  25, "id-FDD-DCHs-to-Modify" },
  {  26, "id-TDD-DCHs-to-Modify" },
  {  27, "id-DCH-InformationResponse" },
  {  28, "id-DCH-Rate-InformationItem-RL-CongestInd" },
  {  29, "id-DL-CCTrCH-InformationAddItem-RL-ReconfPrepTDD" },
  {  30, "id-DL-CCTrCH-InformationListIE-RL-ReconfReadyTDD" },
  {  31, "id-DL-CCTrCH-InformationDeleteItem-RL-ReconfRqstTDD" },
  {  32, "id-DL-CCTrCH-InformationItem-RL-SetupRqstTDD" },
  {  33, "id-DL-CCTrCH-InformationListIE-PhyChReconfRqstTDD" },
  {  34, "id-DL-CCTrCH-InformationListIE-RL-AdditionRspTDD" },
  {  35, "id-DL-CCTrCH-InformationListIE-RL-SetupRspTDD" },
  {  36, "id-DL-CCTrCH-InformationAddList-RL-ReconfPrepTDD" },
  {  37, "id-DL-CCTrCH-InformationDeleteList-RL-ReconfRqstTDD" },
  {  38, "id-DL-CCTrCH-InformationList-RL-SetupRqstTDD" },
  {  39, "id-FDD-DL-CodeInformation" },
  {  40, "id-DL-DPCH-Information-RL-ReconfPrepFDD" },
  {  41, "id-DL-DPCH-Information-RL-SetupRqstFDD" },
  {  42, "id-DL-DPCH-Information-RL-ReconfRqstFDD" },
  {  43, "id-DL-DPCH-InformationItem-PhyChReconfRqstTDD" },
  {  44, "id-DL-DPCH-InformationItem-RL-AdditionRspTDD" },
  {  45, "id-DL-DPCH-InformationItem-RL-SetupRspTDD" },
  {  46, "id-DL-DPCH-TimingAdjustment" },
  {  47, "id-DLReferencePower" },
  {  48, "id-DLReferencePowerList-DL-PC-Rqst" },
  {  49, "id-DL-ReferencePowerInformation-DL-PC-Rqst" },
  {  50, "id-DPC-Mode" },
  {  51, "id-DRXCycleLengthCoefficient" },
  {  52, "id-DedicatedMeasurementObjectType-DM-Fail-Ind" },
  {  53, "id-DedicatedMeasurementObjectType-DM-Fail" },
  {  54, "id-DedicatedMeasurementObjectType-DM-Rprt" },
  {  55, "id-DedicatedMeasurementObjectType-DM-Rqst" },
  {  56, "id-DedicatedMeasurementObjectType-DM-Rsp" },
  {  57, "id-DedicatedMeasurementType" },
  {  58, "id-FACH-InfoForUESelectedS-CCPCH-CTCH-ResourceRspFDD" },
  {  59, "id-FACH-InfoForUESelectedS-CCPCH-CTCH-ResourceRspTDD" },
  {  60, "id-Guaranteed-Rate-Information" },
  {  61, "id-IMSI" },
  {  62, "id-HCS-Prio" },
  {  63, "id-L3-Information" },
  {  64, "id-AdjustmentPeriod" },
  {  65, "id-MaxAdjustmentStep" },
  {  66, "id-MeasurementFilterCoefficient" },
  {  67, "id-MessageStructure" },
  {  68, "id-MeasurementID" },
  {  69, "id-Neighbouring-GSM-CellInformation" },
  {  70, "id-Neighbouring-UMTS-CellInformationItem" },
  {  71, "id-NRT-Load-Information-Value" },
  {  72, "id-NRT-Load-Information-Value-IncrDecrThres" },
  {  73, "id-PagingArea-PagingRqst" },
  {  74, "id-FACH-FlowControlInformation" },
  {  75, "id-PartialReportingIndicator" },
  {  76, "id-Permanent-NAS-UE-Identity" },
  {  77, "id-PowerAdjustmentType" },
  {  78, "id-RANAP-RelocationInformation" },
  {  79, "id-RL-Information-PhyChReconfRqstFDD" },
  {  80, "id-RL-Information-PhyChReconfRqstTDD" },
  {  81, "id-RL-Information-RL-AdditionRqstFDD" },
  {  82, "id-RL-Information-RL-AdditionRqstTDD" },
  {  83, "id-RL-Information-RL-DeletionRqst" },
  {  84, "id-RL-Information-RL-FailureInd" },
  {  85, "id-RL-Information-RL-ReconfPrepFDD" },
  {  86, "id-RL-Information-RL-RestoreInd" },
  {  87, "id-RL-Information-RL-SetupRqstFDD" },
  {  88, "id-RL-Information-RL-SetupRqstTDD" },
  {  89, "id-RL-InformationItem-RL-CongestInd" },
  {  90, "id-RL-InformationItem-DM-Rprt" },
  {  91, "id-RL-InformationItem-DM-Rqst" },
  {  92, "id-RL-InformationItem-DM-Rsp" },
  {  93, "id-RL-InformationItem-RL-PreemptRequiredInd" },
  {  94, "id-RL-InformationItem-RL-SetupRqstFDD" },
  {  95, "id-RL-InformationList-RL-CongestInd" },
  {  96, "id-RL-InformationList-RL-AdditionRqstFDD" },
  {  97, "id-RL-InformationList-RL-DeletionRqst" },
  {  98, "id-RL-InformationList-RL-PreemptRequiredInd" },
  {  99, "id-RL-InformationList-RL-ReconfPrepFDD" },
  { 100, "id-RL-InformationResponse-RL-AdditionRspTDD" },
  { 101, "id-RL-InformationResponse-RL-ReconfReadyTDD" },
  { 102, "id-RL-InformationResponse-RL-SetupRspTDD" },
  { 103, "id-RL-InformationResponseItem-RL-AdditionRspFDD" },
  { 104, "id-RL-InformationResponseItem-RL-ReconfReadyFDD" },
  { 105, "id-RL-InformationResponseItem-RL-ReconfRspFDD" },
  { 106, "id-RL-InformationResponseItem-RL-SetupRspFDD" },
  { 107, "id-RL-InformationResponseList-RL-AdditionRspFDD" },
  { 108, "id-RL-InformationResponseList-RL-ReconfReadyFDD" },
  { 109, "id-RL-InformationResponseList-RL-ReconfRspFDD" },
  { 110, "id-RL-InformationResponse-RL-ReconfRspTDD" },
  { 111, "id-RL-InformationResponseList-RL-SetupRspFDD" },
  { 112, "id-RL-ReconfigurationFailure-RL-ReconfFail" },
  { 113, "id-RL-Set-InformationItem-DM-Rprt" },
  { 114, "id-RL-Set-InformationItem-DM-Rqst" },
  { 115, "id-RL-Set-InformationItem-DM-Rsp" },
  { 116, "id-RL-Set-Information-RL-FailureInd" },
  { 117, "id-RL-Set-Information-RL-RestoreInd" },
  { 118, "id-RL-Set-Successful-InformationItem-DM-Fail" },
  { 119, "id-RL-Set-Unsuccessful-InformationItem-DM-Fail" },
  { 120, "id-RL-Set-Unsuccessful-InformationItem-DM-Fail-Ind" },
  { 121, "id-RL-Successful-InformationItem-DM-Fail" },
  { 122, "id-RL-Unsuccessful-InformationItem-DM-Fail" },
  { 123, "id-RL-Unsuccessful-InformationItem-DM-Fail-Ind" },
  { 124, "id-ReportCharacteristics" },
  { 125, "id-Reporting-Object-RL-FailureInd" },
  { 126, "id-Reporing-Object-RL-RestoreInd" },
  { 127, "id-RT-Load-Value" },
  { 128, "id-RT-Load-Value-IncrDecrThres" },
  { 129, "id-S-RNTI" },
  { 130, "id-ResetIndicator" },
  { 131, "id-RNC-ID" },
  { 132, "id-SAI" },
  { 133, "id-SRNC-ID" },
  { 134, "id-SuccessfulRL-InformationResponse-RL-AdditionFailureFDD" },
  { 135, "id-SuccessfulRL-InformationResponse-RL-SetupFailureFDD" },
  { 136, "id-TransportBearerID" },
  { 137, "id-TransportBearerRequestIndicator" },
  { 138, "id-TransportLayerAddress" },
  { 139, "id-TypeOfError" },
  { 140, "id-UC-ID" },
  { 141, "id-UL-CCTrCH-AddInformation-RL-ReconfPrepTDD" },
  { 142, "id-UL-CCTrCH-InformationAddList-RL-ReconfPrepTDD" },
  { 143, "id-UL-CCTrCH-InformationItem-RL-SetupRqstTDD" },
  { 144, "id-UL-CCTrCH-InformationList-RL-SetupRqstTDD" },
  { 145, "id-UL-CCTrCH-InformationListIE-PhyChReconfRqstTDD" },
  { 146, "id-UL-CCTrCH-InformationListIE-RL-AdditionRspTDD" },
  { 147, "id-UL-CCTrCH-InformationListIE-RL-ReconfReadyTDD" },
  { 148, "id-UL-CCTrCH-InformationListIE-RL-SetupRspTDD" },
  { 149, "id-UL-DPCH-Information-RL-ReconfPrepFDD" },
  { 150, "id-UL-DPCH-Information-RL-ReconfRqstFDD" },
  { 151, "id-UL-DPCH-Information-RL-SetupRqstFDD" },
  { 152, "id-UL-DPCH-InformationItem-PhyChReconfRqstTDD" },
  { 153, "id-UL-DPCH-InformationItem-RL-AdditionRspTDD" },
  { 154, "id-UL-DPCH-InformationItem-RL-SetupRspTDD" },
  { 155, "id-UL-DPCH-InformationAddListIE-RL-ReconfReadyTDD" },
  { 156, "id-UL-SIRTarget" },
  { 157, "id-URA-Information" },
  { 158, "id-UnsuccessfulRL-InformationResponse-RL-AdditionFailureFDD" },
  { 159, "id-UnsuccessfulRL-InformationResponse-RL-SetupFailureFDD" },
  { 160, "id-UnsuccessfulRL-InformationResponse-RL-SetupFailureTDD" },
  { 161, "id-Active-Pattern-Sequence-Information" },
  { 162, "id-AdjustmentRatio" },
  { 163, "id-CauseLevel-RL-AdditionFailureFDD" },
  { 164, "id-CauseLevel-RL-AdditionFailureTDD" },
  { 165, "id-CauseLevel-RL-ReconfFailure" },
  { 166, "id-CauseLevel-RL-SetupFailureFDD" },
  { 167, "id-CauseLevel-RL-SetupFailureTDD" },
  { 168, "id-DL-CCTrCH-InformationDeleteItem-RL-ReconfPrepTDD" },
  { 169, "id-DL-CCTrCH-InformationModifyItem-RL-ReconfPrepTDD" },
  { 170, "id-DL-CCTrCH-InformationModifyItem-RL-ReconfRqstTDD" },
  { 171, "id-DL-CCTrCH-InformationDeleteList-RL-ReconfPrepTDD" },
  { 172, "id-DL-CCTrCH-InformationModifyList-RL-ReconfPrepTDD" },
  { 173, "id-DL-CCTrCH-InformationModifyList-RL-ReconfRqstTDD" },
  { 174, "id-DL-DPCH-InformationAddListIE-RL-ReconfReadyTDD" },
  { 175, "id-DL-DPCH-InformationDeleteListIE-RL-ReconfReadyTDD" },
  { 176, "id-DL-DPCH-InformationModifyListIE-RL-ReconfReadyTDD" },
  { 177, "id-DSCHs-to-Add-TDD" },
  { 178, "id-DSCH-DeleteList-RL-ReconfPrepTDD" },
  { 179, "id-DSCH-InformationListIE-RL-AdditionRspTDD" },
  { 180, "id-DSCH-InformationListIEs-RL-SetupRspTDD" },
  { 181, "id-DSCH-TDD-Information" },
  { 182, "id-DSCH-ModifyList-RL-ReconfPrepTDD" },
  { 183, "id-DSCHToBeAddedOrModifiedList-RL-ReconfReadyTDD" },
  { 184, "id-GA-Cell" },
  { 185, "id-GA-CellAdditionalShapes" },
  { 186, "id-Transmission-Gap-Pattern-Sequence-Information" },
  { 187, "id-UL-CCTrCH-DeleteInformation-RL-ReconfPrepTDD" },
  { 188, "id-UL-CCTrCH-ModifyInformation-RL-ReconfPrepTDD" },
  { 189, "id-UL-CCTrCH-InformationModifyItem-RL-ReconfRqstTDD" },
  { 190, "id-UL-CCTrCH-InformationDeleteList-RL-ReconfPrepTDD" },
  { 191, "id-UL-CCTrCH-InformationModifyList-RL-ReconfPrepTDD" },
  { 192, "id-UL-CCTrCH-InformationModifyList-RL-ReconfRqstTDD" },
  { 193, "id-UL-CCTrCH-InformationDeleteItem-RL-ReconfRqstTDD" },
  { 194, "id-UL-CCTrCH-InformationDeleteList-RL-ReconfRqstTDD" },
  { 195, "id-UL-DPCH-InformationDeleteListIE-RL-ReconfReadyTDD" },
  { 196, "id-UL-DPCH-InformationModifyListIE-RL-ReconfReadyTDD" },
  { 197, "id-UnsuccessfulRL-InformationResponse-RL-AdditionFailureTDD" },
  { 198, "id-USCHs-to-Add" },
  { 199, "id-USCH-DeleteList-RL-ReconfPrepTDD" },
  { 200, "id-USCH-InformationListIE-RL-AdditionRspTDD" },
  { 201, "id-USCH-InformationListIEs-RL-SetupRspTDD" },
  { 202, "id-USCH-Information" },
  { 203, "id-USCH-ModifyList-RL-ReconfPrepTDD" },
  { 204, "id-USCHToBeAddedOrModifiedList-RL-ReconfReadyTDD" },
  { 205, "id-DL-Physical-Channel-Information-RL-SetupRqstTDD" },
  { 206, "id-UL-Physical-Channel-Information-RL-SetupRqstTDD" },
  { 207, "id-ClosedLoopMode1-SupportIndicator" },
  { 208, "id-STTD-SupportIndicator" },
  { 209, "id-CFNReportingIndicator" },
  { 210, "id-CNOriginatedPage-PagingRqst" },
  { 211, "id-InnerLoopDLPCStatus" },
  { 212, "id-PropagationDelay" },
  { 213, "id-RxTimingDeviationForTA" },
  { 214, "id-timeSlot-ISCP" },
  { 215, "id-CCTrCH-InformationItem-RL-FailureInd" },
  { 216, "id-CCTrCH-InformationItem-RL-RestoreInd" },
  { 217, "id-CommonMeasurementAccuracy" },
  { 218, "id-CommonMeasurementObjectType-CM-Rprt" },
  { 219, "id-CommonMeasurementObjectType-CM-Rqst" },
  { 220, "id-CommonMeasurementObjectType-CM-Rsp" },
  { 221, "id-CommonMeasurementType" },
  { 222, "id-CongestionCause" },
  { 223, "id-SFN" },
  { 224, "id-SFNReportingIndicator" },
  { 225, "id-InformationExchangeID" },
  { 226, "id-InformationExchangeObjectType-InfEx-Rprt" },
  { 227, "id-InformationExchangeObjectType-InfEx-Rqst" },
  { 228, "id-InformationExchangeObjectType-InfEx-Rsp" },
  { 229, "id-InformationReportCharacteristics" },
  { 230, "id-InformationType" },
  { 231, "id-neighbouring-LCR-TDD-CellInformation" },
  { 232, "id-DL-Timeslot-ISCP-LCR-Information-RL-SetupRqstTDD" },
  { 233, "id-RL-LCR-InformationResponse-RL-SetupRspTDD" },
  { 234, "id-UL-CCTrCH-LCR-InformationListIE-RL-SetupRspTDD" },
  { 235, "id-UL-DPCH-LCR-InformationItem-RL-SetupRspTDD" },
  { 236, "id-DL-CCTrCH-LCR-InformationListIE-RL-SetupRspTDD" },
  { 237, "id-DL-DPCH-LCR-InformationItem-RL-SetupRspTDD" },
  { 238, "id-DSCH-LCR-InformationListIEs-RL-SetupRspTDD" },
  { 239, "id-USCH-LCR-InformationListIEs-RL-SetupRspTDD" },
  { 240, "id-DL-Timeslot-ISCP-LCR-Information-RL-AdditionRqstTDD" },
  { 241, "id-RL-LCR-InformationResponse-RL-AdditionRspTDD" },
  { 242, "id-UL-CCTrCH-LCR-InformationListIE-RL-AdditionRspTDD" },
  { 243, "id-UL-DPCH-LCR-InformationItem-RL-AdditionRspTDD" },
  { 244, "id-DL-CCTrCH-LCR-InformationListIE-RL-AdditionRspTDD" },
  { 245, "id-DL-DPCH-LCR-InformationItem-RL-AdditionRspTDD" },
  { 246, "id-DSCH-LCR-InformationListIEs-RL-AdditionRspTDD" },
  { 247, "id-USCH-LCR-InformationListIEs-RL-AdditionRspTDD" },
  { 248, "id-UL-DPCH-LCR-InformationAddListIE-RL-ReconfReadyTDD" },
  { 249, "id-UL-Timeslot-LCR-InformationModifyList-RL-ReconfReadyTDD" },
  { 250, "id-DL-DPCH-LCR-InformationAddListIE-RL-ReconfReadyTDD" },
  { 251, "id-DL-Timeslot-LCR-InformationModifyList-RL-ReconfReadyTDD" },
  { 252, "id-UL-Timeslot-LCR-InformationList-PhyChReconfRqstTDD" },
  { 253, "id-DL-Timeslot-LCR-InformationList-PhyChReconfRqstTDD" },
  { 254, "id-timeSlot-ISCP-LCR-List-DL-PC-Rqst-TDD" },
  { 255, "id-TSTD-Support-Indicator-RL-SetupRqstTDD" },
  { 256, "id-RestrictionStateIndicator" },
  { 257, "id-Load-Value" },
  { 258, "id-Load-Value-IncrDecrThres" },
  { 259, "id-OnModification" },
  { 260, "id-Received-Total-Wideband-Power-Value" },
  { 261, "id-Received-Total-Wideband-Power-Value-IncrDecrThres" },
  { 262, "id-SFNSFNMeasurementThresholdInformation" },
  { 263, "id-Transmitted-Carrier-Power-Value" },
  { 264, "id-Transmitted-Carrier-Power-Value-IncrDecrThres" },
  { 265, "id-TUTRANGPSMeasurementThresholdInformation" },
  { 266, "id-UL-Timeslot-ISCP-Value" },
  { 267, "id-UL-Timeslot-ISCP-Value-IncrDecrThres" },
  { 268, "id-Rx-Timing-Deviation-Value-LCR" },
  { 269, "id-DPC-Mode-Change-SupportIndicator" },
  { 270, "id-PrimaryCCPCH-RSCP-RL-ReconfPrepTDD" },
  { 271, "id-DL-TimeSlot-ISCP-Info-RL-ReconfPrepTDD" },
  { 272, "id-DL-Timeslot-ISCP-LCR-Information-RL-ReconfPrepTDD" },
  { 273, "id-DSCH-RNTI" },
  { 274, "id-DL-PowerBalancing-Information" },
  { 275, "id-DL-PowerBalancing-ActivationIndicator" },
  { 276, "id-DL-PowerBalancing-UpdatedIndicator" },
  { 277, "id-DL-ReferencePowerInformation" },
  { 278, "id-Enhanced-PrimaryCPICH-EcNo" },
  { 279, "id-IPDL-TDD-ParametersLCR" },
  { 280, "id-CellCapabilityContainer-FDD" },
  { 281, "id-CellCapabilityContainer-TDD" },
  { 282, "id-CellCapabilityContainer-TDD-LCR" },
  { 283, "id-RL-Specific-DCH-Info" },
  { 284, "id-RL-ReconfigurationRequestFDD-RL-InformationList" },
  { 285, "id-RL-ReconfigurationRequestFDD-RL-Information-IEs" },
  { 286, "id-RL-ReconfigurationRequestTDD-RL-Information" },
  { 287, "id-CommonTransportChannelResourcesInitialisationNotRequired" },
  { 288, "id-DelayedActivation" },
  { 289, "id-DelayedActivationList-RL-ActivationCmdFDD" },
  { 290, "id-DelayedActivationInformation-RL-ActivationCmdFDD" },
  { 291, "id-DelayedActivationList-RL-ActivationCmdTDD" },
  { 292, "id-DelayedActivationInformation-RL-ActivationCmdTDD" },
  { 293, "id-neighbouringTDDCellMeasurementInformationLCR" },
  { 294, "id-UL-SIR-Target-CCTrCH-InformationItem-RL-SetupRspTDD" },
  { 295, "id-UL-SIR-Target-CCTrCH-LCR-InformationItem-RL-SetupRspTDD" },
  { 296, "id-PrimCCPCH-RSCP-DL-PC-RqstTDD" },
  { 297, "id-HSDSCH-FDD-Information" },
  { 298, "id-HSDSCH-FDD-Information-Response" },
  { 299, "id-HSDSCH-FDD-Update-Information" },
  { 300, "id-HSDSCH-Information-to-Modify" },
  { 301, "id-HSDSCHMacdFlowSpecificInformationList-RL-PreemptRequiredInd" },
  { 302, "id-HSDSCHMacdFlowSpecificInformationItem-RL-PreemptRequiredInd" },
  { 303, "id-HSDSCH-RNTI" },
  { 304, "id-HSDSCH-TDD-Information" },
  { 305, "id-HSDSCH-TDD-Information-Response" },
  { 306, "id-HSDSCH-TDD-Update-Information" },
  { 307, "id-HSPDSCH-RL-ID" },
  { 308, "id-HSDSCH-MACdFlows-to-Add" },
  { 309, "id-HSDSCH-MACdFlows-to-Delete" },
  { 310, "id-Angle-Of-Arrival-Value-LCR" },
  { 311, "id-TrafficClass" },
  { 312, "id-PDSCH-RL-ID" },
  { 313, "id-TimeSlot-RL-SetupRspTDD" },
  { 314, "id-GERAN-Cell-Capability" },
  { 315, "id-GERAN-Classmark" },
  { 316, "id-DSCH-InitialWindowSize" },
  { 317, "id-UL-Synchronisation-Parameters-LCR" },
  { 318, "id-SNA-Information" },
  { 319, "id-MAChs-ResetIndicator" },
  { 320, "id-TDD-DL-DPCH-TimeSlotFormatModifyItem-LCR-RL-ReconfReadyTDD" },
  { 321, "id-TDD-UL-DPCH-TimeSlotFormatModifyItem-LCR-RL-ReconfReadyTDD" },
  { 322, "id-TDD-TPC-UplinkStepSize-LCR-RL-SetupRqstTDD" },
  { 323, "id-UL-CCTrCH-InformationList-RL-AdditionRqstTDD" },
  { 324, "id-UL-CCTrCH-InformationItem-RL-AdditionRqstTDD" },
  { 325, "id-DL-CCTrCH-InformationList-RL-AdditionRqstTDD" },
  { 326, "id-DL-CCTrCH-InformationItem-RL-AdditionRqstTDD" },
  { 327, "id-TDD-TPC-UplinkStepSize-InformationAdd-LCR-RL-ReconfPrepTDD" },
  { 328, "id-TDD-TPC-UplinkStepSize-InformationModify-LCR-RL-ReconfPrepTDD" },
  { 329, "id-TDD-TPC-DownlinkStepSize-InformationAdd-RL-ReconfPrepTDD" },
  { 330, "id-TDD-TPC-DownlinkStepSize-InformationModify-RL-ReconfPrepTDD" },
  { 331, "id-UL-TimingAdvanceCtrl-LCR" },
  { 332, "id-HSPDSCH-Timeslot-InformationList-PhyChReconfRqstTDD" },
  { 333, "id-HSPDSCH-Timeslot-InformationListLCR-PhyChReconfRqstTDD" },
  { 334, "id-HS-SICH-Reception-Quality" },
  { 335, "id-HS-SICH-Reception-Quality-Measurement-Value" },
  { 336, "id-HSSICH-Info-DM-Rprt" },
  { 337, "id-HSSICH-Info-DM-Rqst" },
  { 338, "id-HSSICH-Info-DM" },
  { 339, "id-CCTrCH-Maximum-DL-Power-RL-SetupRspTDD" },
  { 340, "id-CCTrCH-Minimum-DL-Power-RL-SetupRspTDD" },
  { 341, "id-CCTrCH-Maximum-DL-Power-RL-AdditionRspTDD" },
  { 342, "id-CCTrCH-Minimum-DL-Power-RL-AdditionRspTDD" },
  { 343, "id-CCTrCH-Maximum-DL-Power-RL-ReconfReadyTDD" },
  { 344, "id-CCTrCH-Minimum-DL-Power-RL-ReconfReadyTDD" },
  { 345, "id-Maximum-DL-Power-TimeslotLCR-InformationModifyItem-RL-ReconfReadyTDD" },
  { 346, "id-Minimum-DL-Power-TimeslotLCR-InformationModifyItem-RL-ReconfReadyTDD" },
  { 347, "id-DL-CCTrCH-InformationList-RL-ReconfRspTDD" },
  { 348, "id-DL-DPCH-InformationModifyItem-LCR-RL-ReconfRspTDD" },
  { 349, "id-Maximum-DL-Power-TimeslotLCR-InformationItem" },
  { 350, "id-Minimum-DL-Power-TimeslotLCR-InformationItem" },
  { 351, "id-TDD-Support-8PSK" },
  { 352, "id-TDD-maxNrDLPhysicalchannels" },
  { 353, "id-ExtendedGSMCellIndividualOffset" },
  { 354, "id-RL-ParameterUpdateIndicationFDD-RL-InformationList" },
  { 355, "id-Primary-CPICH-Usage-For-Channel-Estimation" },
  { 356, "id-Secondary-CPICH-Information" },
  { 357, "id-Secondary-CPICH-Information-Change" },
  { 358, "id-RL-ParameterUpdateIndicationFDD-RL-Information-Item" },
  { 359, "id-Phase-Reference-Update-Indicator" },
  { 360, "id-Unidirectional-DCH-Indicator" },
  { 361, "id-RL-Information-RL-ReconfPrepTDD" },
  { 362, "id-Multiple-RL-InformationResponse-RL-ReconfReadyTDD" },
  { 363, "id-RL-ReconfigurationResponseTDD-RL-Information" },
  { 364, "id-Satellite-Almanac-Information-ExtItem" },
  { 365, "id-HSDSCH-Information-to-Modify-Unsynchronised" },
  { 366, "id-TnlQos" },
  { 367, "id-RTLoadValue" },
  { 368, "id-NRTLoadInformationValue" },
  { 369, "id-CellPortionID" },
  { 370, "id-UpPTSInterferenceValue" },
  { 371, "id-PrimaryCCPCH-RSCP-Delta" },
  { 372, "id-UEMeasurementType" },
  { 373, "id-UEMeasurementTimeslotInfoHCR" },
  { 374, "id-UEMeasurementTimeslotInfoLCR" },
  { 375, "id-UEMeasurementReportCharacteristics" },
  { 376, "id-UEMeasurementParameterModAllow" },
  { 377, "id-UEMeasurementValueInformation" },
  { 378, "id-InterfacesToTraceItem" },
  { 379, "id-ListOfInterfacesToTrace" },
  { 380, "id-TraceDepth" },
  { 381, "id-TraceRecordingSessionReference" },
  { 382, "id-TraceReference" },
  { 383, "id-UEIdentity" },
  { 384, "id-NACC-Related-Data" },
  { 385, "id-GSM-Cell-InfEx-Rqst" },
  { 386, "id-MeasurementRecoveryBehavior" },
  { 387, "id-MeasurementRecoveryReportingIndicator" },
  { 388, "id-MeasurementRecoverySupportIndicator" },
  { 389, "id-DL-DPCH-Power-Information-RL-ReconfPrepFDD" },
  { 390, "id-F-DPCH-Information-RL-ReconfPrepFDD" },
  { 391, "id-F-DPCH-Information-RL-SetupRqstFDD" },
  { 392, "id-MBMS-Bearer-Service-List" },
  { 393, "id-MBMS-Bearer-Service-List-InfEx-Rsp" },
  { 394, "id-Active-MBMS-Bearer-ServiceFDD" },
  { 395, "id-Active-MBMS-Bearer-ServiceTDD" },
  { 396, "id-Old-URA-ID" },
  { 397, "id-UE-State" },
  { 398, "id-URA-ID" },
  { 399, "id-HARQ-Preamble-Mode" },
  { 400, "id-SynchronisationIndicator" },
  { 401, "id-UL-DPDCHIndicatorEDCH" },
  { 402, "id-EDPCH-Information" },
  { 403, "id-RL-Specific-EDCH-Information" },
  { 404, "id-EDCH-RL-Indication" },
  { 405, "id-EDCH-FDD-Information" },
  { 406, "id-EDCH-RLSet-Id" },
  { 407, "id-Serving-EDCHRL-Id" },
  { 408, "id-EDCH-FDD-DL-ControlChannelInformation" },
  { 409, "id-EDCH-FDD-InformationResponse" },
  { 410, "id-EDCH-MACdFlows-To-Add" },
  { 411, "id-EDCH-FDD-Information-To-Modify" },
  { 412, "id-EDCH-MACdFlows-To-Delete" },
  { 413, "id-EDPCH-Information-RLReconfRequest-FDD" },
  { 414, "id-EDCH-MacdFlowSpecificInformationList-RL-PreemptRequiredInd" },
  { 415, "id-EDCH-MacdFlowSpecificInformationItem-RL-PreemptRequiredInd" },
  { 416, "id-EDCH-MacdFlowSpecificInformationList-RL-CongestInd" },
  { 417, "id-EDCH-MacdFlowSpecificInformationItem-RL-CongestInd" },
  { 418, "id-MBMS-Bearer-Service-Full-Address" },
  { 419, "id-Initial-DL-DPCH-TimingAdjustment" },
  { 420, "id-Initial-DL-DPCH-TimingAdjustment-Allowed" },
  { 421, "id-User-Plane-Congestion-Fields-Inclusion" },
  { 422, "id-HARQ-Preamble-Mode-Activation-Indicator" },
  { 423, "id-multiple-DedicatedMeasurementValueList-TDD-DM-Rsp" },
  { 424, "id-multiple-DedicatedMeasurementValueList-LCR-TDD-DM-Rsp" },
  { 425, "id-ProvidedInformation" },
  { 426, "id-Active-MBMS-Bearer-ServiceFDD-PFL" },
  { 427, "id-Active-MBMS-Bearer-ServiceTDD-PFL" },
  { 428, "id-FrequencyBandIndicator" },
  { 0, NULL }
};

static const per_choice_t DummyProtocolIE_choice[] = {
  {   0, &hf_rnsap_id_AllowedQueuingTime, ASN1_EXTENSION_ROOT    , dissect_rnsap_AllowedQueuingTime },
  {   1, &hf_rnsap_id_Allowed_Rate_Information, ASN1_EXTENSION_ROOT    , dissect_rnsap_Allowed_Rate_Information },
  {   2, &hf_rnsap_id_AntennaColocationIndicator, ASN1_EXTENSION_ROOT    , dissect_rnsap_AntennaColocationIndicator },
  {   3, &hf_rnsap_id_BindingID  , ASN1_EXTENSION_ROOT    , dissect_rnsap_BindingID },
  {   4, &hf_rnsap_id_C_ID       , ASN1_EXTENSION_ROOT    , dissect_rnsap_C_ID },
  {   5, &hf_rnsap_id_C_RNTI     , ASN1_EXTENSION_ROOT    , dissect_rnsap_C_RNTI },
  {   6, &hf_rnsap_id_Cell_Capacity_Class_Value, ASN1_EXTENSION_ROOT    , dissect_rnsap_Cell_Capacity_Class_Value },
  {   7, &hf_rnsap_id_CFN        , ASN1_EXTENSION_ROOT    , dissect_rnsap_CFN },
  {   8, &hf_rnsap_id_CN_CS_DomainIdentifier, ASN1_EXTENSION_ROOT    , dissect_rnsap_CN_CS_DomainIdentifier },
  {   9, &hf_rnsap_id_CN_PS_DomainIdentifier, ASN1_EXTENSION_ROOT    , dissect_rnsap_CN_PS_DomainIdentifier },
  {  10, &hf_rnsap_id_Cause      , ASN1_EXTENSION_ROOT    , dissect_rnsap_Cause },
  {  11, &hf_rnsap_id_CoverageIndicator, ASN1_EXTENSION_ROOT    , dissect_rnsap_CoverageIndicator },
  {  12, &hf_rnsap_id_CriticalityDiagnostics, ASN1_EXTENSION_ROOT    , dissect_rnsap_CriticalityDiagnostics },
  {  13, &hf_rnsap_id_ContextInfoItem_Reset, ASN1_EXTENSION_ROOT    , dissect_rnsap_ContextInfoItem_Reset },
  {  14, &hf_rnsap_id_ContextGroupInfoItem_Reset, ASN1_EXTENSION_ROOT    , dissect_rnsap_ContextGroupInfoItem_Reset },
  {  15, &hf_rnsap_id_D_RNTI     , ASN1_EXTENSION_ROOT    , dissect_rnsap_D_RNTI },
  {  16, &hf_rnsap_id_D_RNTI_ReleaseIndication, ASN1_EXTENSION_ROOT    , dissect_rnsap_D_RNTI_ReleaseIndication },
  {  17, &hf_rnsap_id_DCHs_to_Add_FDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DCH_FDD_Information },
  {  18, &hf_rnsap_id_DCHs_to_Add_TDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DCH_TDD_Information },
  {  19, &hf_rnsap_id_DCH_DeleteList_RL_ReconfPrepFDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DCH_DeleteList_RL_ReconfPrepFDD },
  {  20, &hf_rnsap_id_DCH_DeleteList_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DCH_DeleteList_RL_ReconfPrepTDD },
  {  21, &hf_rnsap_id_DCH_DeleteList_RL_ReconfRqstFDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DCH_DeleteList_RL_ReconfRqstFDD },
  {  22, &hf_rnsap_id_DCH_DeleteList_RL_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DCH_DeleteList_RL_ReconfRqstTDD },
  {  23, &hf_rnsap_id_DCH_FDD_Information, ASN1_EXTENSION_ROOT    , dissect_rnsap_DCH_FDD_Information },
  {  24, &hf_rnsap_id_DCH_TDD_Information, ASN1_EXTENSION_ROOT    , dissect_rnsap_DCH_TDD_Information },
  {  25, &hf_rnsap_id_FDD_DCHs_to_Modify, ASN1_EXTENSION_ROOT    , dissect_rnsap_FDD_DCHs_to_Modify },
  {  26, &hf_rnsap_id_TDD_DCHs_to_Modify, ASN1_EXTENSION_ROOT    , dissect_rnsap_TDD_DCHs_to_Modify },
  {  27, &hf_rnsap_id_DCH_InformationResponse, ASN1_EXTENSION_ROOT    , dissect_rnsap_DCH_InformationResponse },
  {  28, &hf_rnsap_id_DCH_Rate_InformationItem_RL_CongestInd, ASN1_EXTENSION_ROOT    , dissect_rnsap_DCH_Rate_InformationItem_RL_CongestInd },
  {  29, &hf_rnsap_id_DL_CCTrCH_InformationAddItem_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_CCTrCH_InformationAddItem_RL_ReconfPrepTDD },
  {  30, &hf_rnsap_id_DL_CCTrCH_InformationListIE_RL_ReconfReadyTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_CCTrCHInformationListIE_RL_ReconfReadyTDD },
  {  31, &hf_rnsap_id_DL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD },
  {  32, &hf_rnsap_id_DL_CCTrCH_InformationItem_RL_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_CCTrCH_InformationItem_RL_SetupRqstTDD },
  {  33, &hf_rnsap_id_DL_CCTrCH_InformationListIE_PhyChReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_CCTrCH_InformationListIE_PhyChReconfRqstTDD },
  {  34, &hf_rnsap_id_DL_CCTrCH_InformationListIE_RL_AdditionRspTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_CCTrCHInformationListIE_RL_AdditionRspTDD },
  {  35, &hf_rnsap_id_DL_CCTrCH_InformationListIE_RL_SetupRspTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_CCTrCH_InformationItem_RL_SetupRqstTDD },
  {  36, &hf_rnsap_id_DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD },
  {  37, &hf_rnsap_id_DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD },
  {  38, &hf_rnsap_id_DL_CCTrCH_InformationList_RL_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_CCTrCH_InformationList_RL_SetupRqstTDD },
  {  39, &hf_rnsap_id_FDD_DL_CodeInformation, ASN1_EXTENSION_ROOT    , dissect_rnsap_FDD_DL_CodeInformation },
  {  40, &hf_rnsap_id_DL_DPCH_Information_RL_ReconfPrepFDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_DPCH_Information_RL_ReconfPrepFDD },
  {  41, &hf_rnsap_id_DL_DPCH_Information_RL_SetupRqstFDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_DPCH_Information_RL_SetupRqstFDD },
  {  42, &hf_rnsap_id_DL_DPCH_Information_RL_ReconfRqstFDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_DPCH_Information_RL_ReconfRqstFDD },
  {  43, &hf_rnsap_id_DL_DPCH_InformationItem_PhyChReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_DPCH_InformationItem_PhyChReconfRqstTDD },
  {  44, &hf_rnsap_id_DL_DPCH_InformationItem_RL_AdditionRspTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_DPCH_InformationItem_RL_AdditionRspTDD },
  {  45, &hf_rnsap_id_DL_DPCH_InformationItem_RL_SetupRspTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_DPCH_InformationItem_RL_SetupRspTDD },
  {  46, &hf_rnsap_id_DL_DPCH_TimingAdjustment, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_DPCH_TimingAdjustment },
  {  47, &hf_rnsap_id_DLReferencePower, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_Power },
  {  48, &hf_rnsap_id_DLReferencePowerList_DL_PC_Rqst, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_ReferencePowerInformationList_DL_PC_Rqst },
  {  49, &hf_rnsap_id_DL_ReferencePowerInformation_DL_PC_Rqst, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_ReferencePowerInformation_DL_PC_Rqst },
  {  50, &hf_rnsap_id_DPC_Mode   , ASN1_EXTENSION_ROOT    , dissect_rnsap_DPC_Mode },
  {  51, &hf_rnsap_id_DRXCycleLengthCoefficient, ASN1_EXTENSION_ROOT    , dissect_rnsap_DRXCycleLengthCoefficient },
  {  52, &hf_rnsap_id_DedicatedMeasurementObjectType_DM_Fail_Ind, ASN1_EXTENSION_ROOT    , dissect_rnsap_DedicatedMeasurementObjectType_DM_Fail_Ind },
  {  53, &hf_rnsap_id_DedicatedMeasurementObjectType_DM_Fail, ASN1_EXTENSION_ROOT    , dissect_rnsap_DedicatedMeasurementObjectType_DM_Fail },
  {  54, &hf_rnsap_id_DedicatedMeasurementObjectType_DM_Rprt, ASN1_EXTENSION_ROOT    , dissect_rnsap_DedicatedMeasurementObjectType_DM_Rprt },
  {  55, &hf_rnsap_id_DedicatedMeasurementObjectType_DM_Rqst, ASN1_EXTENSION_ROOT    , dissect_rnsap_DedicatedMeasurementObjectType_DM_Rqst },
  {  56, &hf_rnsap_id_DedicatedMeasurementObjectType_DM_Rsp, ASN1_EXTENSION_ROOT    , dissect_rnsap_DedicatedMeasurementObjectType_DM_Rsp },
  {  57, &hf_rnsap_id_DedicatedMeasurementType, ASN1_EXTENSION_ROOT    , dissect_rnsap_DedicatedMeasurementType },
  {  58, &hf_rnsap_id_FACH_InfoForUESelectedS_CCPCH_CTCH_ResourceRspFDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_FACH_InfoForUESelectedS_CCPCH_CTCH_ResourceRspFDD },
  {  59, &hf_rnsap_id_FACH_InfoForUESelectedS_CCPCH_CTCH_ResourceRspTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_FACH_InfoForUESelectedS_CCPCH_CTCH_ResourceRspTDD },
  {  60, &hf_rnsap_id_Guaranteed_Rate_Information, ASN1_EXTENSION_ROOT    , dissect_rnsap_Guaranteed_Rate_Information },
  {  61, &hf_rnsap_id_IMSI       , ASN1_EXTENSION_ROOT    , dissect_rnsap_IMSI },
  {  62, &hf_rnsap_id_HCS_Prio   , ASN1_EXTENSION_ROOT    , dissect_rnsap_HCS_Prio },
  {  63, &hf_rnsap_id_L3_Information, ASN1_EXTENSION_ROOT    , dissect_rnsap_L3_Information },
  {  64, &hf_rnsap_id_AdjustmentPeriod, ASN1_EXTENSION_ROOT    , dissect_rnsap_AdjustmentPeriod },
  {  65, &hf_rnsap_id_MaxAdjustmentStep, ASN1_EXTENSION_ROOT    , dissect_rnsap_MaxAdjustmentStep },
  {  66, &hf_rnsap_id_MeasurementFilterCoefficient, ASN1_EXTENSION_ROOT    , dissect_rnsap_MeasurementFilterCoefficient },
  {  67, &hf_rnsap_id_MessageStructure, ASN1_EXTENSION_ROOT    , dissect_rnsap_MessageStructure },
  {  68, &hf_rnsap_id_MeasurementID, ASN1_EXTENSION_ROOT    , dissect_rnsap_MeasurementID },
  {  69, &hf_rnsap_id_Neighbouring_GSM_CellInformation, ASN1_EXTENSION_ROOT    , dissect_rnsap_Neighbouring_GSM_CellInformation },
  {  70, &hf_rnsap_id_Neighbouring_UMTS_CellInformationItem, ASN1_EXTENSION_ROOT    , dissect_rnsap_Neighbouring_UMTS_CellInformationItem },
  {  71, &hf_rnsap_id_NRT_Load_Information_Value, ASN1_EXTENSION_ROOT    , dissect_rnsap_NRT_Load_Information_Value },
  {  72, &hf_rnsap_id_NRT_Load_Information_Value_IncrDecrThres, ASN1_EXTENSION_ROOT    , dissect_rnsap_NRT_Load_Information_Value_IncrDecrThres },
  {  73, &hf_rnsap_id_PagingArea_PagingRqst, ASN1_EXTENSION_ROOT    , dissect_rnsap_PagingArea_PagingRqst },
  {  74, &hf_rnsap_id_FACH_FlowControlInformation, ASN1_EXTENSION_ROOT    , dissect_rnsap_FACH_FlowControlInformation },
  {  75, &hf_rnsap_id_PartialReportingIndicator, ASN1_EXTENSION_ROOT    , dissect_rnsap_PartialReportingIndicator },
  {  76, &hf_rnsap_id_Permanent_NAS_UE_Identity, ASN1_EXTENSION_ROOT    , dissect_rnsap_Permanent_NAS_UE_Identity },
  {  77, &hf_rnsap_id_PowerAdjustmentType, ASN1_EXTENSION_ROOT    , dissect_rnsap_PowerAdjustmentType },
  {  78, &hf_rnsap_id_RANAP_RelocationInformation, ASN1_EXTENSION_ROOT    , dissect_rnsap_RANAP_RelocationInformation },
  {  79, &hf_rnsap_id_RL_Information_PhyChReconfRqstFDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_Information_PhyChReconfRqstFDD },
  {  80, &hf_rnsap_id_RL_Information_PhyChReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_Information_PhyChReconfRqstTDD },
  {  81, &hf_rnsap_id_RL_Information_RL_AdditionRqstFDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_Information_RL_AdditionRqstFDD },
  {  82, &hf_rnsap_id_RL_Information_RL_AdditionRqstTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_Information_RL_AdditionRqstTDD },
  {  83, &hf_rnsap_id_RL_Information_RL_DeletionRqst, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_Information_RL_DeletionRqst },
  {  84, &hf_rnsap_id_RL_Information_RL_FailureInd, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_Information_RL_FailureInd },
  {  85, &hf_rnsap_id_RL_Information_RL_ReconfPrepFDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_Information_RL_ReconfPrepFDD },
  {  86, &hf_rnsap_id_RL_Information_RL_RestoreInd, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_Information_RL_RestoreInd },
  {  87, &hf_rnsap_id_RL_Information_RL_SetupRqstFDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_InformationList_RL_SetupRqstFDD },
  {  88, &hf_rnsap_id_RL_Information_RL_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_Information_RL_SetupRqstTDD },
  {  89, &hf_rnsap_id_RL_InformationItem_RL_CongestInd, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_InformationItem_RL_CongestInd },
  {  90, &hf_rnsap_id_RL_InformationItem_DM_Rprt, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_InformationItem_DM_Rprt },
  {  91, &hf_rnsap_id_RL_InformationItem_DM_Rqst, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_InformationItem_DM_Rqst },
  {  92, &hf_rnsap_id_RL_InformationItem_DM_Rsp, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_InformationItem_DM_Rsp },
  {  93, &hf_rnsap_id_RL_InformationItem_RL_PreemptRequiredInd, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_InformationItem_RL_PreemptRequiredInd },
  {  94, &hf_rnsap_id_RL_InformationItem_RL_SetupRqstFDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_InformationItem_RL_SetupRqstFDD },
  {  95, &hf_rnsap_id_RL_InformationList_RL_CongestInd, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_InformationList_RL_CongestInd },
  {  96, &hf_rnsap_id_RL_InformationList_RL_AdditionRqstFDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_InformationList_RL_AdditionRqstFDD },
  {  97, &hf_rnsap_id_RL_InformationList_RL_DeletionRqst, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_InformationList_RL_DeletionRqst },
  {  98, &hf_rnsap_id_RL_InformationList_RL_PreemptRequiredInd, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_InformationList_RL_PreemptRequiredInd },
  {  99, &hf_rnsap_id_RL_InformationList_RL_ReconfPrepFDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_InformationList_RL_ReconfPrepFDD },
  { 100, &hf_rnsap_id_RL_InformationResponse_RL_AdditionRspTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_InformationResponse_RL_AdditionRspTDD },
  { 101, &hf_rnsap_id_RL_InformationResponse_RL_ReconfReadyTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_InformationResponse_RL_ReconfReadyTDD },
  { 102, &hf_rnsap_id_RL_InformationResponse_RL_SetupRspTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_InformationResponse_RL_SetupRspTDD },
  { 103, &hf_rnsap_id_RL_InformationResponseItem_RL_AdditionRspFDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_InformationResponseItem_RL_AdditionRspFDD },
  { 104, &hf_rnsap_id_RL_InformationResponseItem_RL_ReconfReadyFDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_InformationResponseItem_RL_ReconfReadyFDD },
  { 105, &hf_rnsap_id_RL_InformationResponseItem_RL_ReconfRspFDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_InformationResponseItem_RL_ReconfRspFDD },
  { 106, &hf_rnsap_id_RL_InformationResponseItem_RL_SetupRspFDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_InformationResponseItem_RL_SetupRspFDD },
  { 107, &hf_rnsap_id_RL_InformationResponseList_RL_AdditionRspFDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_InformationResponseList_RL_AdditionRspFDD },
  { 108, &hf_rnsap_id_RL_InformationResponseList_RL_ReconfReadyFDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_InformationResponseList_RL_ReconfReadyFDD },
  { 109, &hf_rnsap_id_RL_InformationResponseList_RL_ReconfRspFDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_InformationResponseList_RL_ReconfRspFDD },
  { 110, &hf_rnsap_id_RL_InformationResponse_RL_ReconfRspTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_InformationResponse_RL_ReconfRspTDD },
  { 111, &hf_rnsap_id_RL_InformationResponseList_RL_SetupRspFDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_InformationResponseList_RL_SetupRspFDD },
  { 112, &hf_rnsap_id_RL_ReconfigurationFailure_RL_ReconfFail, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_ReconfigurationFailure_RL_ReconfFail },
  { 113, &hf_rnsap_id_RL_Set_InformationItem_DM_Rprt, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_Set_InformationItem_DM_Rprt },
  { 114, &hf_rnsap_id_RL_Set_InformationItem_DM_Rqst, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_Set_InformationItem_DM_Rqst },
  { 115, &hf_rnsap_id_RL_Set_InformationItem_DM_Rsp, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_Set_InformationItem_DM_Rsp },
  { 116, &hf_rnsap_id_RL_Set_Information_RL_FailureInd, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_Set_Information_RL_FailureInd },
  { 117, &hf_rnsap_id_RL_Set_Information_RL_RestoreInd, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_Set_Information_RL_RestoreInd },
  { 118, &hf_rnsap_id_RL_Set_Successful_InformationItem_DM_Fail, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_Set_Successful_InformationItem_DM_Fail },
  { 119, &hf_rnsap_id_RL_Set_Unsuccessful_InformationItem_DM_Fail, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_Set_Unsuccessful_InformationItem_DM_Fail },
  { 120, &hf_rnsap_id_RL_Set_Unsuccessful_InformationItem_DM_Fail_Ind, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_Set_Unsuccessful_InformationItem_DM_Fail_Ind },
  { 121, &hf_rnsap_id_RL_Successful_InformationItem_DM_Fail, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_Successful_InformationItem_DM_Fail },
  { 122, &hf_rnsap_id_RL_Unsuccessful_InformationItem_DM_Fail, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_Unsuccessful_InformationItem_DM_Fail },
  { 123, &hf_rnsap_id_RL_Unsuccessful_InformationItem_DM_Fail_Ind, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_Unsuccessful_InformationItem_DM_Fail_Ind },
  { 124, &hf_rnsap_id_ReportCharacteristics, ASN1_EXTENSION_ROOT    , dissect_rnsap_ReportCharacteristics },
  { 125, &hf_rnsap_id_Reporting_Object_RL_FailureInd, ASN1_EXTENSION_ROOT    , dissect_rnsap_Reporting_Object_RL_FailureInd },
  { 126, &hf_rnsap_id_Reporing_Object_RL_RestoreInd, ASN1_EXTENSION_ROOT    , dissect_rnsap_Reporting_Object_RL_RestoreInd },
  { 127, &hf_rnsap_id_RT_Load_Value, ASN1_EXTENSION_ROOT    , dissect_rnsap_RT_Load_Value },
  { 128, &hf_rnsap_id_RT_Load_Value_IncrDecrThres, ASN1_EXTENSION_ROOT    , dissect_rnsap_RT_Load_Value_IncrDecrThres },
  { 129, &hf_rnsap_id_S_RNTI     , ASN1_EXTENSION_ROOT    , dissect_rnsap_S_RNTI },
  { 130, &hf_rnsap_id_ResetIndicator, ASN1_EXTENSION_ROOT    , dissect_rnsap_ResetIndicator },
  { 131, &hf_rnsap_id_RNC_ID     , ASN1_EXTENSION_ROOT    , dissect_rnsap_RNC_ID },
  { 132, &hf_rnsap_id_SAI        , ASN1_EXTENSION_ROOT    , dissect_rnsap_SAI },
  { 133, &hf_rnsap_id_SRNC_ID    , ASN1_EXTENSION_ROOT    , dissect_rnsap_RNC_ID },
  { 134, &hf_rnsap_id_SuccessfulRL_InformationResponse_RL_AdditionFailureFDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_SuccessfulRL_InformationResponse_RL_AdditionFailureFDD },
  { 135, &hf_rnsap_id_SuccessfulRL_InformationResponse_RL_SetupFailureFDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_SuccessfulRL_InformationResponse_RL_SetupFailureFDD },
  { 136, &hf_rnsap_id_TransportBearerID, ASN1_EXTENSION_ROOT    , dissect_rnsap_TransportBearerID },
  { 137, &hf_rnsap_id_TransportBearerRequestIndicator, ASN1_EXTENSION_ROOT    , dissect_rnsap_TransportBearerRequestIndicator },
  { 138, &hf_rnsap_id_TransportLayerAddress, ASN1_EXTENSION_ROOT    , dissect_rnsap_TransportLayerAddress },
  { 139, &hf_rnsap_id_TypeOfError, ASN1_EXTENSION_ROOT    , dissect_rnsap_TypeOfError },
  { 140, &hf_rnsap_id_UC_ID      , ASN1_EXTENSION_ROOT    , dissect_rnsap_UC_ID },
  { 141, &hf_rnsap_id_UL_CCTrCH_AddInformation_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_CCTrCH_AddInformation_RL_ReconfPrepTDD },
  { 142, &hf_rnsap_id_UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD },
  { 143, &hf_rnsap_id_UL_CCTrCH_InformationItem_RL_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_CCTrCH_InformationItem_RL_SetupRqstTDD },
  { 144, &hf_rnsap_id_UL_CCTrCH_InformationList_RL_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_CCTrCH_InformationList_RL_SetupRqstTDD },
  { 145, &hf_rnsap_id_UL_CCTrCH_InformationListIE_PhyChReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_CCTrCH_InformationListIE_PhyChReconfRqstTDD },
  { 146, &hf_rnsap_id_UL_CCTrCH_InformationListIE_RL_AdditionRspTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_CCTrCHInformationListIE_RL_AdditionRspTDD },
  { 147, &hf_rnsap_id_UL_CCTrCH_InformationListIE_RL_ReconfReadyTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_CCTrCHInformationListIE_RL_ReconfReadyTDD },
  { 148, &hf_rnsap_id_UL_CCTrCH_InformationListIE_RL_SetupRspTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_CCTrCHInformationListIE_RL_SetupRspTDD },
  { 149, &hf_rnsap_id_UL_DPCH_Information_RL_ReconfPrepFDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_DPCH_Information_RL_ReconfPrepFDD },
  { 150, &hf_rnsap_id_UL_DPCH_Information_RL_ReconfRqstFDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_DPCH_Information_RL_ReconfRqstFDD },
  { 151, &hf_rnsap_id_UL_DPCH_Information_RL_SetupRqstFDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_DPCH_Information_RL_SetupRqstFDD },
  { 152, &hf_rnsap_id_UL_DPCH_InformationItem_PhyChReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_DPCH_InformationItem_PhyChReconfRqstTDD },
  { 153, &hf_rnsap_id_UL_DPCH_InformationItem_RL_AdditionRspTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_DPCH_InformationItem_RL_AdditionRspTDD },
  { 154, &hf_rnsap_id_UL_DPCH_InformationItem_RL_SetupRspTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_DPCH_InformationItem_RL_SetupRspTDD },
  { 155, &hf_rnsap_id_UL_DPCH_InformationAddListIE_RL_ReconfReadyTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_DPCH_InformationAddListIE_RL_ReconfReadyTDD },
  { 156, &hf_rnsap_id_UL_SIRTarget, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_SIR },
  { 157, &hf_rnsap_id_URA_Information, ASN1_EXTENSION_ROOT    , dissect_rnsap_URA_Information },
  { 158, &hf_rnsap_id_UnsuccessfulRL_InformationResponse_RL_AdditionFailureFDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_UnsuccessfulRL_InformationResponse_RL_AdditionFailureFDD },
  { 159, &hf_rnsap_id_UnsuccessfulRL_InformationResponse_RL_SetupFailureFDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_UnsuccessfulRL_InformationResponse_RL_SetupFailureFDD },
  { 160, &hf_rnsap_id_UnsuccessfulRL_InformationResponse_RL_SetupFailureTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_UnsuccessfulRL_InformationResponse_RL_SetupFailureTDD },
  { 161, &hf_rnsap_id_Active_Pattern_Sequence_Information, ASN1_EXTENSION_ROOT    , dissect_rnsap_Active_Pattern_Sequence_Information },
  { 162, &hf_rnsap_id_AdjustmentRatio, ASN1_EXTENSION_ROOT    , dissect_rnsap_ScaledAdjustmentRatio },
  { 163, &hf_rnsap_id_CauseLevel_RL_AdditionFailureFDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_CauseLevel_RL_AdditionFailureFDD },
  { 164, &hf_rnsap_id_CauseLevel_RL_AdditionFailureTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_CauseLevel_RL_AdditionFailureTDD },
  { 165, &hf_rnsap_id_CauseLevel_RL_ReconfFailure, ASN1_EXTENSION_ROOT    , dissect_rnsap_CauseLevel_RL_ReconfFailure },
  { 166, &hf_rnsap_id_CauseLevel_RL_SetupFailureFDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_CauseLevel_RL_SetupFailureFDD },
  { 167, &hf_rnsap_id_CauseLevel_RL_SetupFailureTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_CauseLevel_RL_SetupFailureTDD },
  { 168, &hf_rnsap_id_DL_CCTrCH_InformationDeleteItem_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_CCTrCH_InformationDeleteItem_RL_ReconfPrepTDD },
  { 169, &hf_rnsap_id_DL_CCTrCH_InformationModifyItem_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_CCTrCH_InformationModifyItem_RL_ReconfPrepTDD },
  { 170, &hf_rnsap_id_DL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD },
  { 171, &hf_rnsap_id_DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD },
  { 172, &hf_rnsap_id_DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD },
  { 173, &hf_rnsap_id_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD },
  { 174, &hf_rnsap_id_DL_DPCH_InformationAddListIE_RL_ReconfReadyTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_DPCH_InformationAddListIE_RL_ReconfReadyTDD },
  { 175, &hf_rnsap_id_DL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD },
  { 176, &hf_rnsap_id_DL_DPCH_InformationModifyListIE_RL_ReconfReadyTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_DPCH_InformationModifyListIE_RL_ReconfReadyTDD },
  { 177, &hf_rnsap_id_DSCHs_to_Add_TDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DSCH_TDD_Information },
  { 178, &hf_rnsap_id_DSCH_DeleteList_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DSCH_DeleteList_RL_ReconfPrepTDD },
  { 179, &hf_rnsap_id_DSCH_InformationListIE_RL_AdditionRspTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DSCH_InformationListIE_RL_AdditionRspTDD },
  { 180, &hf_rnsap_id_DSCH_InformationListIEs_RL_SetupRspTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DSCH_InformationListIEs_RL_SetupRspTDD },
  { 181, &hf_rnsap_id_DSCH_TDD_Information, ASN1_EXTENSION_ROOT    , dissect_rnsap_DSCH_TDD_Information },
  { 182, &hf_rnsap_id_DSCH_ModifyList_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DSCH_ModifyList_RL_ReconfPrepTDD },
  { 183, &hf_rnsap_id_DSCHToBeAddedOrModifiedList_RL_ReconfReadyTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DSCHToBeAddedOrModifiedList_RL_ReconfReadyTDD },
  { 184, &hf_rnsap_id_GA_Cell    , ASN1_EXTENSION_ROOT    , dissect_rnsap_GA_Cell },
  { 185, &hf_rnsap_id_GA_CellAdditionalShapes, ASN1_EXTENSION_ROOT    , dissect_rnsap_GA_CellAdditionalShapes },
  { 186, &hf_rnsap_id_Transmission_Gap_Pattern_Sequence_Information, ASN1_EXTENSION_ROOT    , dissect_rnsap_Transmission_Gap_Pattern_Sequence_Information },
  { 187, &hf_rnsap_id_UL_CCTrCH_DeleteInformation_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_CCTrCH_DeleteInformation_RL_ReconfPrepTDD },
  { 188, &hf_rnsap_id_UL_CCTrCH_ModifyInformation_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_CCTrCH_ModifyInformation_RL_ReconfPrepTDD },
  { 189, &hf_rnsap_id_UL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD },
  { 190, &hf_rnsap_id_UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD },
  { 191, &hf_rnsap_id_UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD },
  { 192, &hf_rnsap_id_UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD },
  { 193, &hf_rnsap_id_UL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD },
  { 194, &hf_rnsap_id_UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD },
  { 195, &hf_rnsap_id_UL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD },
  { 196, &hf_rnsap_id_UL_DPCH_InformationModifyListIE_RL_ReconfReadyTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_DPCH_InformationModifyListIE_RL_ReconfReadyTDD },
  { 197, &hf_rnsap_id_UnsuccessfulRL_InformationResponse_RL_AdditionFailureTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_UnsuccessfulRL_InformationResponse_RL_AdditionFailureTDD },
  { 198, &hf_rnsap_id_USCHs_to_Add, ASN1_EXTENSION_ROOT    , dissect_rnsap_USCH_Information },
  { 199, &hf_rnsap_id_USCH_DeleteList_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_USCH_DeleteList_RL_ReconfPrepTDD },
  { 200, &hf_rnsap_id_USCH_InformationListIE_RL_AdditionRspTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_USCH_InformationListIE_RL_AdditionRspTDD },
  { 201, &hf_rnsap_id_USCH_InformationListIEs_RL_SetupRspTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_USCH_InformationListIEs_RL_SetupRspTDD },
  { 202, &hf_rnsap_id_USCH_Information, ASN1_EXTENSION_ROOT    , dissect_rnsap_USCH_Information },
  { 203, &hf_rnsap_id_USCH_ModifyList_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_USCH_ModifyList_RL_ReconfPrepTDD },
  { 204, &hf_rnsap_id_USCHToBeAddedOrModifiedList_RL_ReconfReadyTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_USCHToBeAddedOrModifiedList_RL_ReconfReadyTDD },
  { 205, &hf_rnsap_id_DL_Physical_Channel_Information_RL_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_Physical_Channel_Information_RL_SetupRqstTDD },
  { 206, &hf_rnsap_id_UL_Physical_Channel_Information_RL_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_Physical_Channel_Information_RL_SetupRqstTDD },
  { 207, &hf_rnsap_id_ClosedLoopMode1_SupportIndicator, ASN1_EXTENSION_ROOT    , dissect_rnsap_ClosedLoopMode1_SupportIndicator },
  { 208, &hf_rnsap_id_STTD_SupportIndicator, ASN1_EXTENSION_ROOT    , dissect_rnsap_STTD_SupportIndicator },
  { 209, &hf_rnsap_id_CFNReportingIndicator, ASN1_EXTENSION_ROOT    , dissect_rnsap_FNReportingIndicator },
  { 210, &hf_rnsap_id_CNOriginatedPage_PagingRqst, ASN1_EXTENSION_ROOT    , dissect_rnsap_CNOriginatedPage_PagingRqst },
  { 211, &hf_rnsap_id_InnerLoopDLPCStatus, ASN1_EXTENSION_ROOT    , dissect_rnsap_InnerLoopDLPCStatus },
  { 212, &hf_rnsap_id_PropagationDelay, ASN1_EXTENSION_ROOT    , dissect_rnsap_PropagationDelay },
  { 213, &hf_rnsap_id_RxTimingDeviationForTA, ASN1_EXTENSION_ROOT    , dissect_rnsap_RxTimingDeviationForTA },
  { 214, &hf_rnsap_id_timeSlot_ISCP, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_TimeSlot_ISCP_Info },
  { 215, &hf_rnsap_id_CCTrCH_InformationItem_RL_FailureInd, ASN1_EXTENSION_ROOT    , dissect_rnsap_CCTrCH_InformationItem_RL_FailureInd },
  { 216, &hf_rnsap_id_CCTrCH_InformationItem_RL_RestoreInd, ASN1_EXTENSION_ROOT    , dissect_rnsap_CCTrCH_InformationItem_RL_RestoreInd },
  { 217, &hf_rnsap_id_CommonMeasurementAccuracy, ASN1_EXTENSION_ROOT    , dissect_rnsap_CommonMeasurementAccuracy },
  { 218, &hf_rnsap_id_CommonMeasurementObjectType_CM_Rprt, ASN1_EXTENSION_ROOT    , dissect_rnsap_CommonMeasurementObjectType_CM_Rprt },
  { 219, &hf_rnsap_id_CommonMeasurementObjectType_CM_Rqst, ASN1_EXTENSION_ROOT    , dissect_rnsap_CommonMeasurementObjectType_CM_Rqst },
  { 220, &hf_rnsap_id_CommonMeasurementObjectType_CM_Rsp, ASN1_EXTENSION_ROOT    , dissect_rnsap_CommonMeasurementObjectType_CM_Rsp },
  { 221, &hf_rnsap_id_CommonMeasurementType, ASN1_EXTENSION_ROOT    , dissect_rnsap_CommonMeasurementType },
  { 222, &hf_rnsap_id_CongestionCause, ASN1_EXTENSION_ROOT    , dissect_rnsap_CongestionCause },
  { 223, &hf_rnsap_id_SFN        , ASN1_EXTENSION_ROOT    , dissect_rnsap_SFN },
  { 224, &hf_rnsap_id_SFNReportingIndicator, ASN1_EXTENSION_ROOT    , dissect_rnsap_FNReportingIndicator },
  { 225, &hf_rnsap_id_InformationExchangeID, ASN1_EXTENSION_ROOT    , dissect_rnsap_InformationExchangeID },
  { 226, &hf_rnsap_id_InformationExchangeObjectType_InfEx_Rprt, ASN1_EXTENSION_ROOT    , dissect_rnsap_InformationExchangeObjectType_InfEx_Rprt },
  { 227, &hf_rnsap_id_InformationExchangeObjectType_InfEx_Rqst, ASN1_EXTENSION_ROOT    , dissect_rnsap_InformationExchangeObjectType_InfEx_Rqst },
  { 228, &hf_rnsap_id_InformationExchangeObjectType_InfEx_Rsp, ASN1_EXTENSION_ROOT    , dissect_rnsap_InformationReportCharacteristics },
  { 229, &hf_rnsap_id_InformationReportCharacteristics, ASN1_EXTENSION_ROOT    , dissect_rnsap_InformationReportCharacteristics },
  { 230, &hf_rnsap_id_InformationType, ASN1_EXTENSION_ROOT    , dissect_rnsap_InformationType },
  { 231, &hf_rnsap_id_neighbouring_LCR_TDD_CellInformation, ASN1_EXTENSION_ROOT    , dissect_rnsap_Neighbouring_LCR_TDD_CellInformation },
  { 232, &hf_rnsap_id_DL_Timeslot_ISCP_LCR_Information_RL_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_TimeSlot_ISCP_LCR_Information },
  { 233, &hf_rnsap_id_RL_LCR_InformationResponse_RL_SetupRspTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_LCR_InformationResponse_RL_SetupRspTDD },
  { 234, &hf_rnsap_id_UL_CCTrCH_LCR_InformationListIE_RL_SetupRspTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_LCR_CCTrCHInformationListIE_RL_SetupRspTDD },
  { 235, &hf_rnsap_id_UL_DPCH_LCR_InformationItem_RL_SetupRspTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_DPCH_LCR_InformationItem_RL_SetupRspTDD },
  { 236, &hf_rnsap_id_DL_CCTrCH_LCR_InformationListIE_RL_SetupRspTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_CCTrCH_LCR_InformationListIE_RL_SetupRspTDD },
  { 237, &hf_rnsap_id_DL_DPCH_LCR_InformationItem_RL_SetupRspTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_DPCH_LCR_InformationItem_RL_SetupRspTDD },
  { 238, &hf_rnsap_id_DSCH_LCR_InformationListIEs_RL_SetupRspTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DSCH_LCR_InformationListIEs_RL_SetupRspTDD },
  { 239, &hf_rnsap_id_USCH_LCR_InformationListIEs_RL_SetupRspTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_USCH_LCR_InformationListIEs_RL_SetupRspTDD },
  { 240, &hf_rnsap_id_DL_Timeslot_ISCP_LCR_Information_RL_AdditionRqstTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_TimeSlot_ISCP_LCR_Information },
  { 241, &hf_rnsap_id_RL_LCR_InformationResponse_RL_AdditionRspTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_LCR_InformationResponse_RL_AdditionRspTDD },
  { 242, &hf_rnsap_id_UL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD },
  { 243, &hf_rnsap_id_UL_DPCH_LCR_InformationItem_RL_AdditionRspTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_DPCH_LCR_InformationItem_RL_AdditionRspTDD },
  { 244, &hf_rnsap_id_DL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD },
  { 245, &hf_rnsap_id_DL_DPCH_LCR_InformationItem_RL_AdditionRspTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_DPCH_LCR_InformationItem_RL_AdditionRspTDD },
  { 246, &hf_rnsap_id_DSCH_LCR_InformationListIEs_RL_AdditionRspTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DSCH_LCR_InformationListIEs_RL_AdditionRspTDD },
  { 247, &hf_rnsap_id_USCH_LCR_InformationListIEs_RL_AdditionRspTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_USCH_LCR_InformationListIEs_RL_AdditionRspTDD },
  { 248, &hf_rnsap_id_UL_DPCH_LCR_InformationAddListIE_RL_ReconfReadyTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_DPCH_LCR_InformationAddList_RL_ReconfReadyTDD },
  { 249, &hf_rnsap_id_UL_Timeslot_LCR_InformationModifyList_RL_ReconfReadyTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_TimeslotLCR_InformationModifyList_RL_ReconfReadyTDD },
  { 250, &hf_rnsap_id_DL_DPCH_LCR_InformationAddListIE_RL_ReconfReadyTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_DPCH_LCR_InformationAddList_RL_ReconfReadyTDD },
  { 251, &hf_rnsap_id_DL_Timeslot_LCR_InformationModifyList_RL_ReconfReadyTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_TimeslotLCR_InformationModifyList_RL_ReconfReadyTDD },
  { 252, &hf_rnsap_id_UL_Timeslot_LCR_InformationList_PhyChReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_TimeslotLCR_InformationList_PhyChReconfRqstTDD },
  { 253, &hf_rnsap_id_DL_Timeslot_LCR_InformationList_PhyChReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_TimeslotLCR_InformationList_PhyChReconfRqstTDD },
  { 254, &hf_rnsap_id_timeSlot_ISCP_LCR_List_DL_PC_Rqst_TDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_TimeSlot_ISCP_LCR_Information },
  { 255, &hf_rnsap_id_TSTD_Support_Indicator_RL_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_TSTD_Support_Indicator },
  { 256, &hf_rnsap_id_RestrictionStateIndicator, ASN1_EXTENSION_ROOT    , dissect_rnsap_RestrictionStateIndicator },
  { 257, &hf_rnsap_id_Load_Value , ASN1_EXTENSION_ROOT    , dissect_rnsap_Load_Value },
  { 258, &hf_rnsap_id_Load_Value_IncrDecrThres, ASN1_EXTENSION_ROOT    , dissect_rnsap_Load_Value_IncrDecrThres },
  { 259, &hf_rnsap_id_OnModification, ASN1_EXTENSION_ROOT    , dissect_rnsap_OnModification },
  { 260, &hf_rnsap_id_Received_Total_Wideband_Power_Value, ASN1_EXTENSION_ROOT    , dissect_rnsap_Received_Total_Wideband_Power_Value },
  { 261, &hf_rnsap_id_Received_Total_Wideband_Power_Value_IncrDecrThres, ASN1_EXTENSION_ROOT    , dissect_rnsap_SFNSFNMeasurementThresholdInformation },
  { 262, &hf_rnsap_id_SFNSFNMeasurementThresholdInformation, ASN1_EXTENSION_ROOT    , dissect_rnsap_SFNSFNMeasurementThresholdInformation },
  { 263, &hf_rnsap_id_Transmitted_Carrier_Power_Value, ASN1_EXTENSION_ROOT    , dissect_rnsap_Transmitted_Carrier_Power_Value },
  { 264, &hf_rnsap_id_Transmitted_Carrier_Power_Value_IncrDecrThres, ASN1_EXTENSION_ROOT    , dissect_rnsap_Transmitted_Carrier_Power_Value_IncrDecrThres },
  { 265, &hf_rnsap_id_TUTRANGPSMeasurementThresholdInformation, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_Timeslot_ISCP_Value },
  { 266, &hf_rnsap_id_UL_Timeslot_ISCP_Value, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_Timeslot_ISCP_Value },
  { 267, &hf_rnsap_id_UL_Timeslot_ISCP_Value_IncrDecrThres, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_Timeslot_ISCP_Value_IncrDecrThres },
  { 268, &hf_rnsap_id_Rx_Timing_Deviation_Value_LCR, ASN1_EXTENSION_ROOT    , dissect_rnsap_Rx_Timing_Deviation_Value_LCR },
  { 269, &hf_rnsap_id_DPC_Mode_Change_SupportIndicator, ASN1_EXTENSION_ROOT    , dissect_rnsap_DPC_Mode_Change_SupportIndicator },
  { 270, &hf_rnsap_id_PrimaryCCPCH_RSCP_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_PrimaryCCPCH_RSCP },
  { 271, &hf_rnsap_id_DL_TimeSlot_ISCP_Info_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_TimeSlot_ISCP_Info },
  { 272, &hf_rnsap_id_DL_Timeslot_ISCP_LCR_Information_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_TimeSlot_ISCP_LCR_Information },
  { 273, &hf_rnsap_id_DSCH_RNTI  , ASN1_EXTENSION_ROOT    , dissect_rnsap_DSCH_RNTI },
  { 274, &hf_rnsap_id_DL_PowerBalancing_Information, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_PowerBalancing_Information },
  { 275, &hf_rnsap_id_DL_PowerBalancing_ActivationIndicator, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_PowerBalancing_ActivationIndicator },
  { 276, &hf_rnsap_id_DL_PowerBalancing_UpdatedIndicator, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_PowerBalancing_UpdatedIndicator },
  { 277, &hf_rnsap_id_DL_ReferencePowerInformation, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_ReferencePowerInformation },
  { 278, &hf_rnsap_id_Enhanced_PrimaryCPICH_EcNo, ASN1_EXTENSION_ROOT    , dissect_rnsap_Enhanced_PrimaryCPICH_EcNo },
  { 279, &hf_rnsap_id_IPDL_TDD_ParametersLCR, ASN1_EXTENSION_ROOT    , dissect_rnsap_IPDL_TDD_ParametersLCR },
  { 280, &hf_rnsap_id_CellCapabilityContainer_FDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_CellCapabilityContainer_FDD },
  { 281, &hf_rnsap_id_CellCapabilityContainer_TDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_CellCapabilityContainer_TDD },
  { 282, &hf_rnsap_id_CellCapabilityContainer_TDD_LCR, ASN1_EXTENSION_ROOT    , dissect_rnsap_CellCapabilityContainer_TDD_LCR },
  { 283, &hf_rnsap_id_RL_Specific_DCH_Info, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_Specific_DCH_Info },
  { 284, &hf_rnsap_id_RL_ReconfigurationRequestFDD_RL_InformationList, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_ReconfigurationRequestFDD_RL_InformationList },
  { 285, &hf_rnsap_id_RL_ReconfigurationRequestFDD_RL_Information_IEs, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_ReconfigurationRequestFDD_RL_Information_IEs },
  { 286, &hf_rnsap_id_RL_ReconfigurationRequestTDD_RL_Information, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_ReconfigurationRequestTDD_RL_Information },
  { 287, &hf_rnsap_id_CommonTransportChannelResourcesInitialisationNotRequired, ASN1_EXTENSION_ROOT    , dissect_rnsap_CommonTransportChannelResourcesInitialisationNotRequired },
  { 288, &hf_rnsap_id_DelayedActivation, ASN1_EXTENSION_ROOT    , dissect_rnsap_DelayedActivation },
  { 289, &hf_rnsap_id_DelayedActivationList_RL_ActivationCmdFDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DelayedActivationInformationList_RL_ActivationCmdFDD },
  { 290, &hf_rnsap_id_DelayedActivationInformation_RL_ActivationCmdFDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DelayedActivationInformation_RL_ActivationCmdFDD },
  { 291, &hf_rnsap_id_DelayedActivationList_RL_ActivationCmdTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DelayedActivationInformationList_RL_ActivationCmdTDD },
  { 292, &hf_rnsap_id_DelayedActivationInformation_RL_ActivationCmdTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DelayedActivationInformation_RL_ActivationCmdTDD },
  { 293, &hf_rnsap_id_neighbouringTDDCellMeasurementInformationLCR, ASN1_EXTENSION_ROOT    , dissect_rnsap_NeighbouringTDDCellMeasurementInformationLCR },
  { 294, &hf_rnsap_id_UL_SIR_Target_CCTrCH_InformationItem_RL_SetupRspTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_SIR },
  { 295, &hf_rnsap_id_UL_SIR_Target_CCTrCH_LCR_InformationItem_RL_SetupRspTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_SIR },
  { 296, &hf_rnsap_id_PrimCCPCH_RSCP_DL_PC_RqstTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_PrimaryCCPCH_RSCP },
  { 297, &hf_rnsap_id_HSDSCH_FDD_Information, ASN1_EXTENSION_ROOT    , dissect_rnsap_HSDSCH_FDD_Information },
  { 298, &hf_rnsap_id_HSDSCH_FDD_Information_Response, ASN1_EXTENSION_ROOT    , dissect_rnsap_HSDSCH_FDD_Information_Response },
  { 299, &hf_rnsap_id_HSDSCH_FDD_Update_Information, ASN1_EXTENSION_ROOT    , dissect_rnsap_HSDSCH_FDD_Update_Information },
  { 300, &hf_rnsap_id_HSDSCH_Information_to_Modify, ASN1_EXTENSION_ROOT    , dissect_rnsap_HSDSCH_Information_to_Modify },
  { 301, &hf_rnsap_id_HSDSCHMacdFlowSpecificInformationList_RL_PreemptRequiredInd, ASN1_EXTENSION_ROOT    , dissect_rnsap_HSDSCHMacdFlowSpecificInformationList_RL_PreemptRequiredInd },
  { 302, &hf_rnsap_id_HSDSCHMacdFlowSpecificInformationItem_RL_PreemptRequiredInd, ASN1_EXTENSION_ROOT    , dissect_rnsap_HSDSCHMacdFlowSpecificInformationItem_RL_PreemptRequiredInd },
  { 303, &hf_rnsap_id_HSDSCH_RNTI, ASN1_EXTENSION_ROOT    , dissect_rnsap_HSDSCH_RNTI },
  { 304, &hf_rnsap_id_HSDSCH_TDD_Information, ASN1_EXTENSION_ROOT    , dissect_rnsap_HSDSCH_TDD_Information },
  { 305, &hf_rnsap_id_HSDSCH_TDD_Information_Response, ASN1_EXTENSION_ROOT    , dissect_rnsap_HSDSCH_TDD_Information_Response },
  { 306, &hf_rnsap_id_HSDSCH_TDD_Update_Information, ASN1_EXTENSION_ROOT    , dissect_rnsap_HSDSCH_TDD_Update_Information },
  { 307, &hf_rnsap_id_HSPDSCH_RL_ID, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_ID },
  { 308, &hf_rnsap_id_HSDSCH_MACdFlows_to_Add, ASN1_EXTENSION_ROOT    , dissect_rnsap_HSDSCH_MACdFlows_Information },
  { 309, &hf_rnsap_id_HSDSCH_MACdFlows_to_Delete, ASN1_EXTENSION_ROOT    , dissect_rnsap_HSDSCH_MACdFlows_to_Delete },
  { 310, &hf_rnsap_id_Angle_Of_Arrival_Value_LCR, ASN1_EXTENSION_ROOT    , dissect_rnsap_Angle_Of_Arrival_Value_LCR },
  { 311, &hf_rnsap_id_TrafficClass, ASN1_EXTENSION_ROOT    , dissect_rnsap_TrafficClass },
  { 312, &hf_rnsap_id_PDSCH_RL_ID, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_ID },
  { 313, &hf_rnsap_id_TimeSlot_RL_SetupRspTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_TimeSlot },
  { 314, &hf_rnsap_id_GERAN_Cell_Capability, ASN1_EXTENSION_ROOT    , dissect_rnsap_GERAN_Cell_Capability },
  { 315, &hf_rnsap_id_GERAN_Classmark, ASN1_EXTENSION_ROOT    , dissect_rnsap_GERAN_Classmark },
  { 316, &hf_rnsap_id_DSCH_InitialWindowSize, ASN1_EXTENSION_ROOT    , dissect_rnsap_DSCH_InitialWindowSize },
  { 317, &hf_rnsap_id_UL_Synchronisation_Parameters_LCR, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_Synchronisation_Parameters_LCR },
  { 318, &hf_rnsap_id_SNA_Information, ASN1_EXTENSION_ROOT    , dissect_rnsap_SNA_Information },
  { 319, &hf_rnsap_id_MAChs_ResetIndicator, ASN1_EXTENSION_ROOT    , dissect_rnsap_MAChs_ResetIndicator },
  { 320, &hf_rnsap_id_TDD_DL_DPCH_TimeSlotFormatModifyItem_LCR_RL_ReconfReadyTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_TDD_DL_DPCH_TimeSlotFormat_LCR },
  { 321, &hf_rnsap_id_TDD_UL_DPCH_TimeSlotFormatModifyItem_LCR_RL_ReconfReadyTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_TDD_UL_DPCH_TimeSlotFormat_LCR },
  { 322, &hf_rnsap_id_TDD_TPC_UplinkStepSize_LCR_RL_SetupRqstTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_TDD_TPC_UplinkStepSize_LCR },
  { 323, &hf_rnsap_id_UL_CCTrCH_InformationList_RL_AdditionRqstTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_CCTrCH_InformationList_RL_AdditionRqstTDD },
  { 324, &hf_rnsap_id_UL_CCTrCH_InformationItem_RL_AdditionRqstTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_CCTrCH_InformationItem_RL_AdditionRqstTDD },
  { 325, &hf_rnsap_id_DL_CCTrCH_InformationList_RL_AdditionRqstTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_CCTrCH_InformationList_RL_AdditionRqstTDD },
  { 326, &hf_rnsap_id_DL_CCTrCH_InformationItem_RL_AdditionRqstTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_CCTrCH_InformationItem_RL_AdditionRqstTDD },
  { 327, &hf_rnsap_id_TDD_TPC_UplinkStepSize_InformationAdd_LCR_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_TDD_TPC_UplinkStepSize_LCR },
  { 328, &hf_rnsap_id_TDD_TPC_UplinkStepSize_InformationModify_LCR_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_TDD_TPC_UplinkStepSize_LCR },
  { 329, &hf_rnsap_id_TDD_TPC_DownlinkStepSize_InformationAdd_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_TDD_TPC_DownlinkStepSize },
  { 330, &hf_rnsap_id_TDD_TPC_DownlinkStepSize_InformationModify_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_TDD_TPC_DownlinkStepSize },
  { 331, &hf_rnsap_id_UL_TimingAdvanceCtrl_LCR, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_TimingAdvanceCtrl_LCR },
  { 332, &hf_rnsap_id_HSPDSCH_Timeslot_InformationList_PhyChReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_HSPDSCH_Timeslot_InformationList_PhyChReconfRqstTDD },
  { 333, &hf_rnsap_id_HSPDSCH_Timeslot_InformationListLCR_PhyChReconfRqstTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_HSPDSCH_Timeslot_InformationListLCR_PhyChReconfRqstTDD },
  { 334, &hf_rnsap_id_HS_SICH_Reception_Quality, ASN1_EXTENSION_ROOT    , dissect_rnsap_HS_SICH_Reception_Quality_Value },
  { 335, &hf_rnsap_id_HS_SICH_Reception_Quality_Measurement_Value, ASN1_EXTENSION_ROOT    , dissect_rnsap_HS_SICH_Reception_Quality_Measurement_Value },
  { 336, &hf_rnsap_id_HSSICH_Info_DM_Rprt, ASN1_EXTENSION_ROOT    , dissect_rnsap_HS_SICH_ID },
  { 337, &hf_rnsap_id_HSSICH_Info_DM_Rqst, ASN1_EXTENSION_ROOT    , dissect_rnsap_HSSICH_Info_DM_Rqst },
  { 338, &hf_rnsap_id_HSSICH_Info_DM, ASN1_EXTENSION_ROOT    , dissect_rnsap_HS_SICH_ID },
  { 339, &hf_rnsap_id_CCTrCH_Maximum_DL_Power_RL_SetupRspTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_Power },
  { 340, &hf_rnsap_id_CCTrCH_Minimum_DL_Power_RL_SetupRspTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_Power },
  { 341, &hf_rnsap_id_CCTrCH_Maximum_DL_Power_RL_AdditionRspTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_Power },
  { 342, &hf_rnsap_id_CCTrCH_Minimum_DL_Power_RL_AdditionRspTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_Power },
  { 343, &hf_rnsap_id_CCTrCH_Maximum_DL_Power_RL_ReconfReadyTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_Power },
  { 344, &hf_rnsap_id_CCTrCH_Minimum_DL_Power_RL_ReconfReadyTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_Power },
  { 345, &hf_rnsap_id_Maximum_DL_Power_TimeslotLCR_InformationModifyItem_RL_ReconfReadyTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_Power },
  { 346, &hf_rnsap_id_Minimum_DL_Power_TimeslotLCR_InformationModifyItem_RL_ReconfReadyTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_Power },
  { 347, &hf_rnsap_id_DL_CCTrCH_InformationList_RL_ReconfRspTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_CCTrCH_InformationList_RL_ReconfRspTDD },
  { 348, &hf_rnsap_id_DL_DPCH_InformationModifyItem_LCR_RL_ReconfRspTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_DPCH_InformationModifyItem_LCR_RL_ReconfRspTDD },
  { 349, &hf_rnsap_id_Maximum_DL_Power_TimeslotLCR_InformationItem, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_Power },
  { 350, &hf_rnsap_id_Minimum_DL_Power_TimeslotLCR_InformationItem, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_Power },
  { 351, &hf_rnsap_id_TDD_Support_8PSK, ASN1_EXTENSION_ROOT    , dissect_rnsap_Support_8PSK },
  { 352, &hf_rnsap_id_TDD_maxNrDLPhysicalchannels, ASN1_EXTENSION_ROOT    , dissect_rnsap_MaxNrDLPhysicalchannelsTS },
  { 353, &hf_rnsap_id_ExtendedGSMCellIndividualOffset, ASN1_EXTENSION_ROOT    , dissect_rnsap_ExtendedGSMCellIndividualOffset },
  { 354, &hf_rnsap_id_RL_ParameterUpdateIndicationFDD_RL_InformationList, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_ParameterUpdateIndicationFDD_RL_InformationList },
  { 355, &hf_rnsap_id_Primary_CPICH_Usage_For_Channel_Estimation, ASN1_EXTENSION_ROOT    , dissect_rnsap_Primary_CPICH_Usage_For_Channel_Estimation },
  { 356, &hf_rnsap_id_Secondary_CPICH_Information, ASN1_EXTENSION_ROOT    , dissect_rnsap_Secondary_CPICH_Information },
  { 357, &hf_rnsap_id_Secondary_CPICH_Information_Change, ASN1_EXTENSION_ROOT    , dissect_rnsap_Secondary_CPICH_Information_Change },
  { 358, &hf_rnsap_id_RL_ParameterUpdateIndicationFDD_RL_Information_Item, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_ParameterUpdateIndicationFDD_RL_Information_Item },
  { 359, &hf_rnsap_id_Phase_Reference_Update_Indicator, ASN1_EXTENSION_ROOT    , dissect_rnsap_Phase_Reference_Update_Indicator },
  { 360, &hf_rnsap_id_Unidirectional_DCH_Indicator, ASN1_EXTENSION_ROOT    , dissect_rnsap_Unidirectional_DCH_Indicator },
  { 361, &hf_rnsap_id_RL_Information_RL_ReconfPrepTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_Information_RL_ReconfPrepTDD },
  { 362, &hf_rnsap_id_Multiple_RL_InformationResponse_RL_ReconfReadyTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_Multiple_RL_InformationResponse_RL_ReconfReadyTDD },
  { 363, &hf_rnsap_id_RL_ReconfigurationResponseTDD_RL_Information, ASN1_EXTENSION_ROOT    , dissect_rnsap_Multiple_RL_InformationResponse_RL_ReconfRspTDD },
  { 364, &hf_rnsap_id_Satellite_Almanac_Information_ExtItem, ASN1_EXTENSION_ROOT    , dissect_rnsap_Satellite_Almanac_Information_ExtItem },
  { 365, &hf_rnsap_id_HSDSCH_Information_to_Modify_Unsynchronised, ASN1_EXTENSION_ROOT    , dissect_rnsap_HSDSCH_Information_to_Modify_Unsynchronised },
  { 366, &hf_rnsap_id_TnlQos     , ASN1_EXTENSION_ROOT    , dissect_rnsap_TnlQos },
  { 367, &hf_rnsap_id_RTLoadValue, ASN1_EXTENSION_ROOT    , dissect_rnsap_RTLoadValue },
  { 368, &hf_rnsap_id_NRTLoadInformationValue, ASN1_EXTENSION_ROOT    , dissect_rnsap_NRTLoadInformationValue },
  { 369, &hf_rnsap_id_CellPortionID, ASN1_EXTENSION_ROOT    , dissect_rnsap_CellPortionID },
  { 370, &hf_rnsap_id_UpPTSInterferenceValue, ASN1_EXTENSION_ROOT    , dissect_rnsap_UpPTSInterferenceValue },
  { 371, &hf_rnsap_id_PrimaryCCPCH_RSCP_Delta, ASN1_EXTENSION_ROOT    , dissect_rnsap_PrimaryCCPCH_RSCP_Delta },
  { 372, &hf_rnsap_id_UEMeasurementType, ASN1_EXTENSION_ROOT    , dissect_rnsap_UEMeasurementType },
  { 373, &hf_rnsap_id_UEMeasurementTimeslotInfoHCR, ASN1_EXTENSION_ROOT    , dissect_rnsap_UEMeasurementTimeslotInfoHCR },
  { 374, &hf_rnsap_id_UEMeasurementTimeslotInfoLCR, ASN1_EXTENSION_ROOT    , dissect_rnsap_UEMeasurementTimeslotInfoLCR },
  { 375, &hf_rnsap_id_UEMeasurementReportCharacteristics, ASN1_EXTENSION_ROOT    , dissect_rnsap_UEMeasurementReportCharacteristics },
  { 376, &hf_rnsap_id_UEMeasurementParameterModAllow, ASN1_EXTENSION_ROOT    , dissect_rnsap_UEMeasurementParameterModAllow },
  { 377, &hf_rnsap_id_UEMeasurementValueInformation, ASN1_EXTENSION_ROOT    , dissect_rnsap_UEMeasurementValueInformation },
  { 378, &hf_rnsap_id_InterfacesToTraceItem, ASN1_EXTENSION_ROOT    , dissect_rnsap_InterfacesToTraceItem },
  { 379, &hf_rnsap_id_ListOfInterfacesToTrace, ASN1_EXTENSION_ROOT    , dissect_rnsap_ListOfInterfacesToTrace },
  { 380, &hf_rnsap_id_TraceDepth , ASN1_EXTENSION_ROOT    , dissect_rnsap_TraceDepth },
  { 381, &hf_rnsap_id_TraceRecordingSessionReference, ASN1_EXTENSION_ROOT    , dissect_rnsap_TraceRecordingSessionReference },
  { 382, &hf_rnsap_id_TraceReference, ASN1_EXTENSION_ROOT    , dissect_rnsap_TraceReference },
  { 383, &hf_rnsap_id_UEIdentity , ASN1_EXTENSION_ROOT    , dissect_rnsap_UEIdentity },
  { 384, &hf_rnsap_id_NACC_Related_Data, ASN1_EXTENSION_ROOT    , dissect_rnsap_NACC_Related_Data },
  { 385, &hf_rnsap_id_GSM_Cell_InfEx_Rqst, ASN1_EXTENSION_ROOT    , dissect_rnsap_GSM_Cell_InfEx_Rqst },
  { 386, &hf_rnsap_id_MeasurementRecoveryBehavior, ASN1_EXTENSION_ROOT    , dissect_rnsap_MeasurementRecoveryBehavior },
  { 387, &hf_rnsap_id_MeasurementRecoveryReportingIndicator, ASN1_EXTENSION_ROOT    , dissect_rnsap_MeasurementRecoveryReportingIndicator },
  { 388, &hf_rnsap_id_MeasurementRecoverySupportIndicator, ASN1_EXTENSION_ROOT    , dissect_rnsap_MeasurementRecoverySupportIndicator },
  { 389, &hf_rnsap_id_DL_DPCH_Power_Information_RL_ReconfPrepFDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_DPCH_Power_Information_RL_ReconfPrepFDD },
  { 390, &hf_rnsap_id_F_DPCH_Information_RL_ReconfPrepFDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_F_DPCH_Information_RL_ReconfPrepFDD },
  { 391, &hf_rnsap_id_F_DPCH_Information_RL_SetupRqstFDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_F_DPCH_Information_RL_SetupRqstFDD },
  { 392, &hf_rnsap_id_MBMS_Bearer_Service_List, ASN1_EXTENSION_ROOT    , dissect_rnsap_MBMS_Bearer_Service_List },
  { 393, &hf_rnsap_id_MBMS_Bearer_Service_List_InfEx_Rsp, ASN1_EXTENSION_ROOT    , dissect_rnsap_MBMS_Bearer_Service_List_InfEx_Rsp },
  { 394, &hf_rnsap_id_Active_MBMS_Bearer_ServiceFDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_Active_MBMS_Bearer_Service_ListFDD },
  { 395, &hf_rnsap_id_Active_MBMS_Bearer_ServiceTDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_Active_MBMS_Bearer_Service_ListTDD },
  { 396, &hf_rnsap_id_Old_URA_ID , ASN1_EXTENSION_ROOT    , dissect_rnsap_URA_ID },
  { 397, &hf_rnsap_id_UE_State   , ASN1_EXTENSION_ROOT    , dissect_rnsap_UE_State },
  { 398, &hf_rnsap_id_URA_ID     , ASN1_EXTENSION_ROOT    , dissect_rnsap_URA_ID },
  { 399, &hf_rnsap_id_HARQ_Preamble_Mode, ASN1_EXTENSION_ROOT    , dissect_rnsap_HARQ_Preamble_Mode },
  { 400, &hf_rnsap_id_SynchronisationIndicator, ASN1_EXTENSION_ROOT    , dissect_rnsap_SynchronisationIndicator },
  { 401, &hf_rnsap_id_UL_DPDCHIndicatorEDCH, ASN1_EXTENSION_ROOT    , dissect_rnsap_UL_DPDCHIndicatorEDCH },
  { 402, &hf_rnsap_id_EDPCH_Information, ASN1_EXTENSION_ROOT    , dissect_rnsap_EDPCH_Information_FDD },
  { 403, &hf_rnsap_id_RL_Specific_EDCH_Information, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_Specific_EDCH_Information },
  { 404, &hf_rnsap_id_EDCH_RL_Indication, ASN1_EXTENSION_ROOT    , dissect_rnsap_EDCH_RL_Indication },
  { 405, &hf_rnsap_id_EDCH_FDD_Information, ASN1_EXTENSION_ROOT    , dissect_rnsap_EDCH_FDD_Information },
  { 406, &hf_rnsap_id_EDCH_RLSet_Id, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_Set_ID },
  { 407, &hf_rnsap_id_Serving_EDCHRL_Id, ASN1_EXTENSION_ROOT    , dissect_rnsap_EDCH_Serving_RL },
  { 408, &hf_rnsap_id_EDCH_FDD_DL_ControlChannelInformation, ASN1_EXTENSION_ROOT    , dissect_rnsap_EDCH_FDD_DL_ControlChannelInformation },
  { 409, &hf_rnsap_id_EDCH_FDD_InformationResponse, ASN1_EXTENSION_ROOT    , dissect_rnsap_EDCH_FDD_InformationResponse },
  { 410, &hf_rnsap_id_EDCH_MACdFlows_To_Add, ASN1_EXTENSION_ROOT    , dissect_rnsap_RL_Specific_EDCH_Information },
  { 411, &hf_rnsap_id_EDCH_FDD_Information_To_Modify, ASN1_EXTENSION_ROOT    , dissect_rnsap_EDCH_FDD_Information_To_Modify },
  { 412, &hf_rnsap_id_EDCH_MACdFlows_To_Delete, ASN1_EXTENSION_ROOT    , dissect_rnsap_EDCH_MACdFlows_To_Delete },
  { 413, &hf_rnsap_id_EDPCH_Information_RLReconfRequest_FDD, ASN1_EXTENSION_ROOT    , dissect_rnsap_EDPCH_Information_RLReconfRequest_FDD },
  { 414, &hf_rnsap_id_EDCH_MacdFlowSpecificInformationList_RL_PreemptRequiredInd, ASN1_EXTENSION_ROOT    , dissect_rnsap_EDCH_MacdFlowSpecificInformationList_RL_PreemptRequiredInd },
  { 415, &hf_rnsap_id_EDCH_MacdFlowSpecificInformationItem_RL_PreemptRequiredInd, ASN1_EXTENSION_ROOT    , dissect_rnsap_EDCH_MacdFlowSpecificInformationItem_RL_PreemptRequiredInd },
  { 416, &hf_rnsap_id_EDCH_MacdFlowSpecificInformationList_RL_CongestInd, ASN1_EXTENSION_ROOT    , dissect_rnsap_EDCH_MacdFlowSpecificInformationList_RL_CongestInd },
  { 417, &hf_rnsap_id_EDCH_MacdFlowSpecificInformationItem_RL_CongestInd, ASN1_EXTENSION_ROOT    , dissect_rnsap_EDCH_MacdFlowSpecificInformationItem_RL_CongestInd },
  { 418, &hf_rnsap_id_MBMS_Bearer_Service_Full_Address, ASN1_EXTENSION_ROOT    , dissect_rnsap_MBMS_Bearer_Service_Full_Address },
  { 419, &hf_rnsap_id_Initial_DL_DPCH_TimingAdjustment, ASN1_EXTENSION_ROOT    , dissect_rnsap_DL_DPCH_TimingAdjustment },
  { 420, &hf_rnsap_id_Initial_DL_DPCH_TimingAdjustment_Allowed, ASN1_EXTENSION_ROOT    , dissect_rnsap_Initial_DL_DPCH_TimingAdjustment_Allowed },
  { 421, &hf_rnsap_id_User_Plane_Congestion_Fields_Inclusion, ASN1_EXTENSION_ROOT    , dissect_rnsap_User_Plane_Congestion_Fields_Inclusion },
  { 422, &hf_rnsap_id_HARQ_Preamble_Mode_Activation_Indicator, ASN1_EXTENSION_ROOT    , dissect_rnsap_HARQ_Preamble_Mode_Activation_Indicator },
  { 423, &hf_rnsap_id_multiple_DedicatedMeasurementValueList_TDD_DM_Rsp, ASN1_EXTENSION_ROOT    , dissect_rnsap_Multiple_DedicatedMeasurementValueList_TDD_DM_Rsp },
  { 424, &hf_rnsap_id_multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp, ASN1_EXTENSION_ROOT    , dissect_rnsap_Multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp },
  { 425, &hf_rnsap_id_ProvidedInformation, ASN1_EXTENSION_ROOT    , dissect_rnsap_ProvidedInformation },
  { 426, &hf_rnsap_id_Active_MBMS_Bearer_ServiceFDD_PFL, ASN1_EXTENSION_ROOT    , dissect_rnsap_Active_MBMS_Bearer_Service_ListTDD_PFL },
  { 427, &hf_rnsap_id_Active_MBMS_Bearer_ServiceTDD_PFL, ASN1_EXTENSION_ROOT    , dissect_rnsap_Active_MBMS_Bearer_Service_ListTDD_PFL },
  { 428, &hf_rnsap_id_FrequencyBandIndicator, ASN1_EXTENSION_ROOT    , dissect_rnsap_FrequencyBandIndicator },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_DummyProtocolIE(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_DummyProtocolIE, DummyProtocolIE_choice,
                                 NULL);

  return offset;
}



static int
dissect_rnsap_DL_DPCH_InformationList_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_dl_DPCH_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_DPCH_InformationList_RL_SetupRspTDD(tvb, offset, actx, tree, hf_rnsap_dl_DPCH_Information);
}


static const per_sequence_t DL_CCTrCHInformationItem_RL_SetupRspTDD_sequence[] = {
  { &hf_rnsap_cCTrCH_ID     , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CCTrCH_ID },
  { &hf_rnsap_dl_DPCH_Information, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_DL_DPCH_InformationList_RL_SetupRspTDD },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_DL_CCTrCHInformationItem_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_DL_CCTrCHInformationItem_RL_SetupRspTDD, DL_CCTrCHInformationItem_RL_SetupRspTDD_sequence);

  return offset;
}
static int dissect_DL_CCTrCHInformationListIE_RL_SetupRspTDD_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_DL_CCTrCHInformationItem_RL_SetupRspTDD(tvb, offset, actx, tree, hf_rnsap_DL_CCTrCHInformationListIE_RL_SetupRspTDD_item);
}


static const per_sequence_t DL_CCTrCHInformationListIE_RL_SetupRspTDD_sequence_of[1] = {
  { &hf_rnsap_DL_CCTrCHInformationListIE_RL_SetupRspTDD_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_DL_CCTrCHInformationItem_RL_SetupRspTDD },
};

static int
dissect_rnsap_DL_CCTrCHInformationListIE_RL_SetupRspTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_DL_CCTrCHInformationListIE_RL_SetupRspTDD, DL_CCTrCHInformationListIE_RL_SetupRspTDD_sequence_of,
                                                  1, 16);

  return offset;
}


static const per_sequence_t RadioLinkReconfigurationPrepareTDD_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_RadioLinkReconfigurationPrepareTDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_RadioLinkReconfigurationPrepareTDD, RadioLinkReconfigurationPrepareTDD_sequence);

  return offset;
}


static const per_sequence_t Multiple_RL_ReconfigurationRequestTDD_RL_Information_sequence_of[1] = {
  { &hf_rnsap_Multiple_RL_ReconfigurationRequestTDD_RL_Information_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_RL_ReconfigurationRequestTDD_RL_Information },
};

static int
dissect_rnsap_Multiple_RL_ReconfigurationRequestTDD_RL_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_Multiple_RL_ReconfigurationRequestTDD_RL_Information, Multiple_RL_ReconfigurationRequestTDD_RL_Information_sequence_of,
                                                  1, 16);

  return offset;
}


static const per_sequence_t PhysicalChannelReconfigurationRequestFDD_sequence[] = {
  { &hf_rnsap_protocolIEs   , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
  { &hf_rnsap_protocolExtensions, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_PhysicalChannelReconfigurationRequestFDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_PhysicalChannelReconfigurationRequestFDD, PhysicalChannelReconfigurationRequestFDD_sequence);

  return offset;
}


static const per_sequence_t Cell_InfEx_Rsp_sequence[] = {
  { &hf_rnsap_requestedDataValue, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_RequestedDataValue },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_Cell_InfEx_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_Cell_InfEx_Rsp, Cell_InfEx_Rsp_sequence);

  return offset;
}
static int dissect_cell5(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Cell_InfEx_Rsp(tvb, offset, actx, tree, hf_rnsap_cell5);
}



static int
dissect_rnsap_Extension_InformationExchangeObjectType_InfEx_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_rnsap_ProtocolIE_Single_Container(tvb, offset, actx, tree, hf_index);

  return offset;
}
static int dissect_extension_InformationExchangeObjectType_InfEx_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Extension_InformationExchangeObjectType_InfEx_Rsp(tvb, offset, actx, tree, hf_rnsap_extension_InformationExchangeObjectType_InfEx_Rsp);
}


static const value_string rnsap_InformationExchangeObjectType_InfEx_Rsp_vals[] = {
  {   0, "cell" },
  {   1, "extension-InformationExchangeObjectType-InfEx-Rsp" },
  { 0, NULL }
};

static const per_choice_t InformationExchangeObjectType_InfEx_Rsp_choice[] = {
  {   0, &hf_rnsap_cell5         , ASN1_EXTENSION_ROOT    , dissect_rnsap_Cell_InfEx_Rsp },
  {   1, &hf_rnsap_extension_InformationExchangeObjectType_InfEx_Rsp, ASN1_NOT_EXTENSION_ROOT, dissect_rnsap_Extension_InformationExchangeObjectType_InfEx_Rsp },
  { 0, NULL, 0, NULL }
};

static int
dissect_rnsap_InformationExchangeObjectType_InfEx_Rsp(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_choice(tvb, offset, actx, tree, hf_index,
                                 ett_rnsap_InformationExchangeObjectType_InfEx_Rsp, InformationExchangeObjectType_InfEx_Rsp_choice,
                                 NULL);

  return offset;
}


static const per_sequence_t MBMS_Bearer_ServiceItemFDD_PFL_sequence[] = {
  { &hf_rnsap_tmgi          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TMGI },
  { &hf_rnsap_transmissionMode, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TransmissionMode },
  { &hf_rnsap_preferredFrequencyLayer, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_UARFCN },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_MBMS_Bearer_ServiceItemFDD_PFL(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_MBMS_Bearer_ServiceItemFDD_PFL, MBMS_Bearer_ServiceItemFDD_PFL_sequence);

  return offset;
}
static int dissect_Active_MBMS_Bearer_Service_ListFDD_PFL_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_MBMS_Bearer_ServiceItemFDD_PFL(tvb, offset, actx, tree, hf_rnsap_Active_MBMS_Bearer_Service_ListFDD_PFL_item);
}


static const per_sequence_t Active_MBMS_Bearer_Service_ListFDD_PFL_sequence_of[1] = {
  { &hf_rnsap_Active_MBMS_Bearer_Service_ListFDD_PFL_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_MBMS_Bearer_ServiceItemFDD_PFL },
};

static int
dissect_rnsap_Active_MBMS_Bearer_Service_ListFDD_PFL(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_Active_MBMS_Bearer_Service_ListFDD_PFL, Active_MBMS_Bearer_Service_ListFDD_PFL_sequence_of,
                                                  1, 256);

  return offset;
}


static const value_string rnsap_Band_Indicator_vals[] = {
  {   0, "dcs1800Band" },
  {   1, "pcs1900Band" },
  { 0, NULL }
};


static int
dissect_rnsap_Band_Indicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     2, NULL, TRUE, 0, NULL);

  return offset;
}
static int dissect_band_Indicator(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Band_Indicator(tvb, offset, actx, tree, hf_rnsap_band_Indicator);
}



static int
dissect_rnsap_BCC(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     3, 3, FALSE, NULL);

  return offset;
}
static int dissect_bCC(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BCC(tvb, offset, actx, tree, hf_rnsap_bCC);
}



static int
dissect_rnsap_BCCH_ARFCN(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 1023U, NULL, FALSE);

  return offset;
}
static int dissect_bCCH_ARFCN(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BCCH_ARFCN(tvb, offset, actx, tree, hf_rnsap_bCCH_ARFCN);
}



static int
dissect_rnsap_NCC(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_bit_string(tvb, offset, actx, tree, hf_index,
                                     3, 3, FALSE, NULL);

  return offset;
}
static int dissect_nCC(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_NCC(tvb, offset, actx, tree, hf_rnsap_nCC);
}


static const per_sequence_t BSIC_sequence[] = {
  { &hf_rnsap_nCC           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_NCC },
  { &hf_rnsap_bCC           , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_BCC },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_BSIC(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_BSIC, BSIC_sequence);

  return offset;
}
static int dissect_bSIC(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_BSIC(tvb, offset, actx, tree, hf_rnsap_bSIC);
}


static const per_sequence_t EDCH_FDD_Update_InfoItem_sequence[] = {
  { &hf_rnsap_eDCH_FDD_DL_ControlChannelInformation, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_EDCH_FDD_DL_ControlChannelInformation },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_EDCH_FDD_Update_InfoItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_EDCH_FDD_Update_InfoItem, EDCH_FDD_Update_InfoItem_sequence);

  return offset;
}
static int dissect_EDCH_FDD_Update_Information_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_EDCH_FDD_Update_InfoItem(tvb, offset, actx, tree, hf_rnsap_EDCH_FDD_Update_Information_item);
}


static const per_sequence_t EDCH_FDD_Update_Information_sequence_of[1] = {
  { &hf_rnsap_EDCH_FDD_Update_Information_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_EDCH_FDD_Update_InfoItem },
};

static int
dissect_rnsap_EDCH_FDD_Update_Information(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_EDCH_FDD_Update_Information, EDCH_FDD_Update_Information_sequence_of,
                                                  1, 16);

  return offset;
}



static int
dissect_rnsap_EDCH_Physical_Layer_Category(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_integer(tvb, offset, actx, tree, hf_index, NULL);

  return offset;
}


static const per_sequence_t EDPCH_Information_RLReconfPrepare_FDD_sequence[] = {
  { &hf_rnsap_maxSet_E_DPDCHs, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_Max_Set_E_DPDCHs },
  { &hf_rnsap_punctureLimit , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_PunctureLimit },
  { &hf_rnsap_e_TFCS_Information, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_E_TFCS_Information },
  { &hf_rnsap_e_TTI         , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_E_TTI },
  { &hf_rnsap_e_DPCCH_PO    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_E_DPCCH_PO },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_EDPCH_Information_RLReconfPrepare_FDD(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_EDPCH_Information_RLReconfPrepare_FDD, EDPCH_Information_RLReconfPrepare_FDD_sequence);

  return offset;
}


static const per_sequence_t FACH_PCH_InformationItem_sequence[] = {
  { &hf_rnsap_transportFormatSet, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_TransportFormatSet },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_FACH_PCH_InformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_FACH_PCH_InformationItem, FACH_PCH_InformationItem_sequence);

  return offset;
}
static int dissect_FACH_PCH_InformationList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_FACH_PCH_InformationItem(tvb, offset, actx, tree, hf_rnsap_FACH_PCH_InformationList_item);
}


static const per_sequence_t FACH_PCH_InformationList_sequence_of[1] = {
  { &hf_rnsap_FACH_PCH_InformationList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_FACH_PCH_InformationItem },
};

static int
dissect_rnsap_FACH_PCH_InformationList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_FACH_PCH_InformationList, FACH_PCH_InformationList_sequence_of,
                                                  1, 10);

  return offset;
}


static const per_sequence_t Neighbouring_GSM_CellInformationItem_sequence[] = {
  { &hf_rnsap_cGI           , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_CGI },
  { &hf_rnsap_cellIndividualOffset, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_CellIndividualOffset },
  { &hf_rnsap_bSIC          , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BSIC },
  { &hf_rnsap_band_Indicator, ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_Band_Indicator },
  { &hf_rnsap_bCCH_ARFCN    , ASN1_EXTENSION_ROOT    , ASN1_NOT_OPTIONAL, dissect_rnsap_BCCH_ARFCN },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_Neighbouring_GSM_CellInformationItem(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_Neighbouring_GSM_CellInformationItem, Neighbouring_GSM_CellInformationItem_sequence);

  return offset;
}
static int dissect_Neighbouring_GSM_CellInformationIEs_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_Neighbouring_GSM_CellInformationItem(tvb, offset, actx, tree, hf_rnsap_Neighbouring_GSM_CellInformationIEs_item);
}


static const per_sequence_t Neighbouring_GSM_CellInformationIEs_sequence_of[1] = {
  { &hf_rnsap_Neighbouring_GSM_CellInformationIEs_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_Neighbouring_GSM_CellInformationItem },
};

static int
dissect_rnsap_Neighbouring_GSM_CellInformationIEs(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_Neighbouring_GSM_CellInformationIEs, Neighbouring_GSM_CellInformationIEs_sequence_of,
                                                  1, 256);

  return offset;
}



static int
dissect_rnsap_PredictedTUTRANGPSDeviationLimit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 256U, NULL, FALSE);

  return offset;
}
static int dissect_predictedTUTRANGPSDeviationLimit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_PredictedTUTRANGPSDeviationLimit(tvb, offset, actx, tree, hf_rnsap_predictedTUTRANGPSDeviationLimit);
}



static int
dissect_rnsap_Received_Total_Wideband_Power_Value_IncrDecrThres(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              0U, 620U, NULL, FALSE);

  return offset;
}



static int
dissect_rnsap_TUTRANGPSChangeLimit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                              1U, 256U, NULL, FALSE);

  return offset;
}
static int dissect_tUTRANGPSChangeLimit(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_TUTRANGPSChangeLimit(tvb, offset, actx, tree, hf_rnsap_tUTRANGPSChangeLimit);
}


static const per_sequence_t TUTRANGPSMeasurementThresholdInformation_sequence[] = {
  { &hf_rnsap_tUTRANGPSChangeLimit, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_TUTRANGPSChangeLimit },
  { &hf_rnsap_predictedTUTRANGPSDeviationLimit, ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_PredictedTUTRANGPSDeviationLimit },
  { &hf_rnsap_iE_Extensions , ASN1_EXTENSION_ROOT    , ASN1_OPTIONAL    , dissect_rnsap_ProtocolExtensionContainer },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_TUTRANGPSMeasurementThresholdInformation(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_TUTRANGPSMeasurementThresholdInformation, TUTRANGPSMeasurementThresholdInformation_sequence);

  return offset;
}


static const value_string rnsap_Presence_vals[] = {
  {   0, "optional" },
  {   1, "conditional" },
  {   2, "mandatory" },
  { 0, NULL }
};


static int
dissect_rnsap_Presence(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_enumerated(tvb, offset, actx, tree, hf_index,
                                     3, NULL, FALSE, 0, NULL);

  return offset;
}



static int
dissect_rnsap_FirstValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
#line 59 "rnsap.cnf"
	/* FIX ME */
	guint length;

	offset = dissect_per_length_determinant(tvb, offset, actx, tree, hf_rnsap_IE_length, &length);
	offset = offset + length;
	BYTE_ALIGN_OFFSET(offset)



  return offset;
}
static int dissect_firstValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_FirstValue(tvb, offset, actx, tree, hf_rnsap_firstValue);
}



static int
dissect_rnsap_SecondValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
#line 67 "rnsap.cnf"
	/* FIX ME */
	guint length;

	offset = dissect_per_length_determinant(tvb, offset, actx, tree, hf_rnsap_IE_length, &length);
	offset = offset + length;
	BYTE_ALIGN_OFFSET(offset)




  return offset;
}
static int dissect_secondValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_SecondValue(tvb, offset, actx, tree, hf_rnsap_secondValue);
}


static const per_sequence_t ProtocolIE_FieldPair_sequence[] = {
  { &hf_rnsap_id            , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_ID },
  { &hf_rnsap_firstCriticality, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_Criticality },
  { &hf_rnsap_firstValue    , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_FirstValue },
  { &hf_rnsap_secondCriticality, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_Criticality },
  { &hf_rnsap_secondValue   , ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_SecondValue },
  { NULL, 0, 0, NULL }
};

static int
dissect_rnsap_ProtocolIE_FieldPair(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_sequence(tvb, offset, actx, tree, hf_index,
                                   ett_rnsap_ProtocolIE_FieldPair, ProtocolIE_FieldPair_sequence);

  return offset;
}
static int dissect_ProtocolIE_ContainerPair_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_FieldPair(tvb, offset, actx, tree, hf_rnsap_ProtocolIE_ContainerPair_item);
}


static const per_sequence_t ProtocolIE_ContainerPair_sequence_of[1] = {
  { &hf_rnsap_ProtocolIE_ContainerPair_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_FieldPair },
};

static int
dissect_rnsap_ProtocolIE_ContainerPair(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_ProtocolIE_ContainerPair, ProtocolIE_ContainerPair_sequence_of,
                                                  0, 65535);

  return offset;
}
static int dissect_ProtocolIE_ContainerPairList_item(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree) {
  return dissect_rnsap_ProtocolIE_ContainerPair(tvb, offset, actx, tree, hf_rnsap_ProtocolIE_ContainerPairList_item);
}


static const per_sequence_t ProtocolIE_ContainerList_sequence_of[1] = {
  { &hf_rnsap_ProtocolIE_ContainerList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_Container },
};

static int
dissect_rnsap_ProtocolIE_ContainerList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_ProtocolIE_ContainerList, ProtocolIE_ContainerList_sequence_of,
                                                  0, 65535);

  return offset;
}


static const per_sequence_t ProtocolIE_ContainerPairList_sequence_of[1] = {
  { &hf_rnsap_ProtocolIE_ContainerPairList_item, ASN1_NO_EXTENSIONS     , ASN1_NOT_OPTIONAL, dissect_rnsap_ProtocolIE_ContainerPair },
};

static int
dissect_rnsap_ProtocolIE_ContainerPairList(tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
  offset = dissect_per_constrained_sequence_of(tvb, offset, actx, tree, hf_index,
                                                  ett_rnsap_ProtocolIE_ContainerPairList, ProtocolIE_ContainerPairList_sequence_of,
                                                  0, 65535);

  return offset;
}

/*--- PDUs ---*/

static void dissect_RNSAP_PDU_PDU(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree) {
  asn1_ctx_t asn1_ctx;
  asn1_ctx_init(&asn1_ctx, ASN1_ENC_PER, TRUE, pinfo);
  dissect_rnsap_RNSAP_PDU(tvb, 0, &asn1_ctx, tree, hf_rnsap_RNSAP_PDU_PDU);
}


/*--- End of included file: packet-rnsap-fn.c ---*/
#line 594 "packet-rnsap-template.c"


static int dissect_rnsap_InitiatingMessageValueValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree){
	proto_item	*value_item = NULL;
	proto_tree	*value_tree = NULL;
	guint length;

	value_item = proto_tree_add_item(tree, hf_rnsap_initiatingMessageValue, tvb, 0, -1, FALSE);
	value_tree = proto_item_add_subtree(value_item, ett_rnsap_initiatingMessageValue);

	offset = dissect_per_length_determinant(tvb, offset, actx, value_tree, hf_rnsap_pdu_length, &length);
	proto_item_set_len(value_item,length);

	
	switch(ProcedureCode){
	case RNSAP_ID_COMMONTRANSPORTCHANNELRESOURCESINITIALISATION:	/* 0 */
		offset = dissect_id_commonTransportChannelResourcesInitialisation(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_COMMONTRANSPORTCHANNELRESOURCESRELEASE:			/* 1 */
		offset = dissect_id_commonTransportChannelResourcesRelease(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_COMPRESSEDMODECOMMAND:							 /* 2 */
		offset = dissect_id_compressedModeCommand(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DOWNLINKPOWERCONTROL:								 /* 3 */
		offset = dissect_id_downlinkPowerTimeslotControl(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DOWNLINKPOWERTIMESLOTCONTROL:						 /* 4 */
		offset = dissect_id_downlinkPowerTimeslotControl(tvb, offset, actx, value_tree);
		break;
		break;
	case RNSAP_ID_DOWNLINKSIGNALLINGTRANSFER:						 /* 5 */
		offset = dissect_id_downlinkSignallingTransfer(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_ERRORINDICATION:									 /* 6 */
		offset = dissect_id_errorIndication(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DEDICATEDMEASUREMENTFAILURE:						 /* 7 */
		offset = dissect_id_dedicatedMeasurementFailure(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DEDICATEDMEASUREMENTINITIATION:					 /* 8 */
		offset = dissect_id_dedicatedMeasurementInitiation(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DEDICATEDMEASUREMENTREPORTING:					 /* 9 */
		offset = dissect_id_dedicatedMeasurementReporting(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DEDICATEDMEASUREMENTTERMINATION:					 /* 10 */
		offset = dissect_id_dedicatedMeasurementTermination(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_PAGING:											 /* 11 */
		offset = dissect_id_paging(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_PHYSICALCHANNELRECONFIGURATION:					 /* 12 */
		offset = dissect_id_physicalChannelReconfiguration(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_PRIVATEMESSAGE:									 /* 13 */
		offset = dissect_id_privateMessage(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RADIOLINKADDITION:								 /* 14 */
		if (ddMode==RNSAP_FDD){
			offset = dissect_id_radioLinkAddition(tvb, offset, actx, value_tree);
		}else{
			offset = dissect_id_radioLinkAddition_TDD(tvb, offset, actx, value_tree);
		}
		break;
	case RNSAP_ID_RADIOLINKCONGESTION:								 /* 34 */
		offset = dissect_id_radioLinkCongestion(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RADIOLINKDELETION:								 /* 15 */
		offset = dissect_id_radioLinkDeletion(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RADIOLINKFAILURE:									 /* 16 */
		offset = dissect_id_radioLinkFailure(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RADIOLINKPREEMPTION:								 /* 17 */
		offset = dissect_id_radioLinkPreemption(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RADIOLINKRESTORATION:								 /* 18 */
		offset = dissect_id_radioLinkRestoration(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RADIOLINKSETUP:									 /* 19 */
		if (ddMode==RNSAP_FDD){
			offset = dissect_id_radioLinkSetup(tvb, offset, actx, value_tree);
		}else{
			offset = dissect_id_radioLinkSetupTdd(tvb, offset, actx, value_tree);
		}
		break;
	case RNSAP_ID_RELOCATIONCOMMIT:									 /* 20 */
		offset = dissect_id_relocationCommit(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_SYNCHRONISEDRADIOLINKRECONFIGURATIONCANCELLATION:	 /* 21 */
		offset = dissect_id_synchronisedRadioLinkReconfigurationCancellation(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_SYNCHRONISEDRADIOLINKRECONFIGURATIONCOMMIT:		 /* 22 */
		offset = dissect_id_synchronisedRadioLinkReconfigurationCommit(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_SYNCHRONISEDRADIOLINKRECONFIGURATIONPREPARATION:	 /* 23 */
		if (ddMode==RNSAP_FDD){
			offset = dissect_id_synchronisedRadioLinkReconfigurationPreparation(tvb, offset, actx, value_tree);
		}else{
			offset = dissect_id_synchronisedRadioLinkReconfigurationPreparation_TDD(tvb, offset, actx, value_tree);
		}
		break;
	case RNSAP_ID_UNSYNCHRONISEDRADIOLINKRECONFIGURATION:			 /* 24 */
		if (ddMode==RNSAP_FDD){
			offset = dissect_id_unSynchronisedRadioLinkReconfiguration(tvb, offset, actx, value_tree);
		}else{
			offset = dissect_id_unSynchronisedRadioLinkReconfiguration_TDD(tvb, offset, actx, value_tree);
		}
		break;
	case RNSAP_ID_UPLINKSIGNALLINGTRANSFER:							 /* 25 */
		if (ddMode==RNSAP_FDD){
			offset = dissect_id_uplinkSignallingTransfer(tvb, offset, actx, value_tree);
		}else{
			offset = dissect_id_uplinkSignallingTransfer_TDD(tvb, offset, actx, value_tree);
		}
		break;
	case RNSAP_ID_COMMONMEASUREMENTFAILURE:							 /* 26 */
		offset = dissect_id_commonMeasurementFailure(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_COMMONMEASUREMENTINITIATION:						 /* 27 */
		offset = dissect_id_commonMeasurementInitiation(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_COMMONMEASUREMENTREPORTING:						 /* 28 */
		offset = dissect_id_commonMeasurementReporting(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_COMMONMEASUREMENTTERMINATION:						 /* 29 */
		offset = dissect_id_commonMeasurementTermination(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_INFORMATIONEXCHANGEFAILURE:						 /* 30 */
		offset = dissect_id_informationExchangeFailure(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_INFORMATIONEXCHANGEINITIATION:					 /* 31 */
		offset = dissect_id_informationExchangeInitiation(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_INFORMATIONREPORTING:								 /* 32 */
		offset = dissect_id_informationReporting(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_INFORMATIONEXCHANGETERMINATION:					 /* 33 */
		offset = dissect_id_informationExchangeTermination(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RESET:											 /* 35 */
		offset = dissect_id_reset(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RADIOLINKACTIVATION: 								 /* 36 */
		if (ddMode==RNSAP_FDD){
			offset = dissect_id_radioLinkActivation(tvb, offset, actx, value_tree);
		}else{
			offset = dissect_id_radioLinkActivation_TDD(tvb, offset, actx, value_tree);
		}
		break;
	case RNSAP_ID_GERANUPLINKSIGNALLINGTRANSFER:					 /* 37 */
		offset = dissect_id_gERANuplinkSignallingTransfer(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RADIOLINKPARAMETERUPDATE:							 /* 38 */
		if (ddMode==RNSAP_FDD){
			offset = dissect_id_radioLinkParameterUpdate(tvb, offset, actx, value_tree);
		}else{
			offset = dissect_id_radioLinkParameterUpdate_TDD(tvb, offset, actx, value_tree);
		}
		break;
	case RNSAP_ID_UEMEASUREMENTFAILURE:								 /* 39 */
		offset = dissect_id_uEMeasurementFailure(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UEMEASUREMENTINITIATION:							 /* 40 */
		offset = dissect_id_uEMeasurementInitiation(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UEMEASUREMENTREPORTING:							 /* 41 */
		offset = dissect_id_uEMeasurementReporting(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UEMEASUREMENTTERMINATION:							 /* 42 */
		offset = dissect_id_uEMeasurementTermination(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_IURDEACTIVATETRACE:								 /* 43 */
		offset = dissect_id_iurDeactivateTrace(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_IURINVOKETRACE:									 /* 44 */
		offset = dissect_id_iurInvokeTrace(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_MBMSATTACH:										 /* 45 */
		offset = dissect_id_mBMSAttach(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_MBMSDETACH:										 /* 46 */
		offset = dissect_id_mBMSDetach(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DIRECTINFORMATIONTRANSFER:						 /* 48 */
		offset = dissect_id_directInformationTransfer(tvb, offset, actx, value_tree);
		break;
	default:
		offset = offset + (length<<3);
		break;
	}
	BYTE_ALIGN_OFFSET(offset)
	return offset;
}

static int dissect_rnsap_SuccessfulOutcomeValueValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree){
	proto_item	*value_item = NULL;
	proto_tree	*value_tree = NULL;
	guint length;

	value_item = proto_tree_add_item(tree, hf_rnsap_successfulOutcomeValue, tvb, 0, -1, FALSE);
	value_tree = proto_item_add_subtree(value_item, ett_rnsap_initiatingMessageValue);

	offset = dissect_per_length_determinant(tvb, offset, actx, value_tree, hf_rnsap_pdu_length, &length);
	proto_item_set_len(value_item,length);

	
	switch(ProcedureCode){
	case RNSAP_ID_COMMONTRANSPORTCHANNELRESOURCESINITIALISATION:
		if (ddMode==RNSAP_FDD){
			offset = dissect_id_commonTransportChannelResourcesInitialisation1(tvb, offset, actx, value_tree);
		}else{
			offset = dissect_id_commonTransportChannelResourcesInitialisation_TDD(tvb, offset, actx, value_tree);
		}
		break;
	case RNSAP_ID_DEDICATEDMEASUREMENTINITIATION:					 /* 8 */
		offset = dissect_id_dedicatedMeasurementInitiation2(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_PHYSICALCHANNELRECONFIGURATION:					 /* 12 */
		offset = dissect_id_physicalChannelReconfiguration1(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RADIOLINKADDITION:								 /* 14 */
		if (ddMode==RNSAP_FDD){
			offset = dissect_id_radioLinkAddition1(tvb, offset, actx, value_tree);
		}else{
			offset = dissect_id_radioLinkAddition_TDD1(tvb, offset, actx, value_tree);
		}
		break;
	case RNSAP_ID_RADIOLINKDELETION:								 /* 15 */
		offset = dissect_id_radioLinkDeletion1(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RADIOLINKSETUP:									 /* 19 */
		if (ddMode==RNSAP_FDD){
			offset = dissect_id_radioLinkSetup1(tvb, offset, actx, value_tree);
		}else{
			offset = dissect_id_radioLinkSetupTdd1(tvb, offset, actx, value_tree);
		}
		break;
	case RNSAP_ID_SYNCHRONISEDRADIOLINKRECONFIGURATIONPREPARATION:	 /* 23 */
		if (ddMode==RNSAP_FDD){
			offset = dissect_id_synchronisedRadioLinkReconfigurationPreparation1(tvb, offset, actx, value_tree);
		}else{
			offset = dissect_id_synchronisedRadioLinkReconfigurationPreparation_TDD(tvb, offset, actx, value_tree);
		}
	case RNSAP_ID_UNSYNCHRONISEDRADIOLINKRECONFIGURATION:			 /* 24 */
		if (ddMode==RNSAP_FDD){
			offset = dissect_id_unSynchronisedRadioLinkReconfiguration1(tvb, offset, actx, value_tree);
		}else{
			offset = dissect_id_unSynchronisedRadioLinkReconfiguration_TDD1(tvb, offset, actx, value_tree);
		}
		break;
	case RNSAP_ID_COMMONMEASUREMENTINITIATION:						 /* 27 */
		offset = dissect_id_commonMeasurementInitiation1(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_INFORMATIONEXCHANGEINITIATION:					 /* 31 */
		offset = dissect_id_informationExchangeInitiation1(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RESET:											 /* 35 */
		offset = dissect_id_reset1(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UEMEASUREMENTINITIATION:							 /* 40 */
		offset = dissect_id_uEMeasurementInitiation1(tvb, offset, actx, value_tree);
		break;

	default:
		offset = offset + (length<<3);
		break;
	}
	BYTE_ALIGN_OFFSET(offset)
	return offset;
}

static int dissect_rnsap_UnsuccessfulOutcomeValueValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree){
	proto_item	*value_item = NULL;
	proto_tree	*value_tree = NULL;
	guint length;

	value_item = proto_tree_add_item(tree, hf_rnsap_unsuccessfulOutcomeValue, tvb, 0, -1, FALSE);
	value_tree = proto_item_add_subtree(value_item, ett_rnsap_UnsuccessfulOutcomeValue);

	offset = dissect_per_length_determinant(tvb, offset, actx, value_tree, hf_rnsap_pdu_length, &length);
	proto_item_set_len(value_item,length);

	
	switch(ProcedureCode){
	case RNSAP_ID_COMMONTRANSPORTCHANNELRESOURCESINITIALISATION:
		offset = dissect_id_commonTransportChannelResourcesInitialisation2(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DEDICATEDMEASUREMENTINITIATION:					 /* 8 */
		offset = dissect_id_dedicatedMeasurementInitiation2(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_PHYSICALCHANNELRECONFIGURATION:					 /* 12 */
		offset = dissect_id_physicalChannelReconfiguration2(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RADIOLINKADDITION:								 /* 14 */
		if (ddMode==RNSAP_FDD){
			offset = dissect_id_radioLinkAddition2(tvb, offset, actx, value_tree);
		}else{
			offset = dissect_id_radioLinkAddition_TDD2(tvb, offset, actx, value_tree);
		}
		break;
	case RNSAP_ID_RADIOLINKSETUP:									 /* 19 */
		if (ddMode==RNSAP_FDD){
			offset = dissect_id_radioLinkSetup2(tvb, offset, actx, value_tree);
		}else{
			offset = dissect_id_radioLinkSetupTdd2(tvb, offset, actx, value_tree);
		}
		break;
	case RNSAP_ID_SYNCHRONISEDRADIOLINKRECONFIGURATIONPREPARATION:	 /* 23 */
		offset = dissect_id_synchronisedRadioLinkReconfigurationPreparation2(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UNSYNCHRONISEDRADIOLINKRECONFIGURATION:			 /* 24 */
		if (ddMode==RNSAP_FDD){
			offset = dissect_id_unSynchronisedRadioLinkReconfiguration2(tvb, offset, actx, value_tree);
		}else{
			offset = dissect_id_unSynchronisedRadioLinkReconfiguration_TDD2(tvb, offset, actx, value_tree);
		}
		break;
	case RNSAP_ID_COMMONMEASUREMENTINITIATION:						 /* 27 */
		offset = dissect_id_commonMeasurementInitiation2(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_INFORMATIONEXCHANGEINITIATION:					 /* 31 */
		offset = dissect_id_informationExchangeInitiation2(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UEMEASUREMENTINITIATION:							 /* 40 */
		offset = dissect_id_uEMeasurementInitiation2(tvb, offset, actx, value_tree);
		break;

	default:
		offset = offset + (length<<3);
		break;
	}
	BYTE_ALIGN_OFFSET(offset)
	return offset;
}

static int dissect_rnsap_ProtocolIEValueValue(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree){
	proto_item	*value_item = NULL;
	proto_tree	*value_tree = NULL;
	guint length;

	value_item = proto_tree_add_item(tree, hf_rnsap_value, tvb, 0, -1, FALSE);
	value_tree = proto_item_add_subtree(value_item, ett_rnsap_ProtocolIEValueValue);

	offset = dissect_per_length_determinant(tvb, offset, actx, value_tree, hf_rnsap_IE_length, &length);
	proto_item_set_len(value_item,length);

	
	switch(ProtocolIE_ID){

	case RNSAP_ID_ALLOWEDQUEUINGTIME:											/*  4 */
		offset = dissect_id_AllowedQueuingTime(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_ALLOWED_RATE_INFORMATION:										/*  42 */
		offset = dissect_id_Allowed_Rate_Information(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_ANTENNACOLOCATIONINDICATOR:									/*  309 */
		offset = dissect_id_AntennaColocationIndicator(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_BINDINGID:													/*  5 */
		offset = dissect_id_BindingID(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_C_ID:															/*  6 */
		offset = dissect_id_C_ID(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_C_RNTI:														/*  7 */
		offset = dissect_id_C_RNTI(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_CELL_CAPACITY_CLASS_VALUE:									/*  303 */
		offset = dissect_id_Cell_Capacity_Class_Value(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_CFN:															/*  8 */
		offset = dissect_id_CFN(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_CN_CS_DOMAINIDENTIFIER:										/*  9 */
		offset = dissect_id_CN_CS_DomainIdentifier(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_CN_PS_DOMAINIDENTIFIER:										/*  10 */
		offset = dissect_id_CN_PS_DomainIdentifier(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_CAUSE:														/*  11 */
		offset = dissect_id_Cause(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_COVERAGEINDICATOR:											/*  310 */
		offset = dissect_id_CoverageIndicator(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_CRITICALITYDIAGNOSTICS:										/*  20 */
		offset = dissect_id_CriticalityDiagnostics(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_CONTEXTINFOITEM_RESET:										/*  211 */
		offset = offset + (length<<3);
		break;
	case RNSAP_ID_CONTEXTGROUPINFOITEM_RESET:									/*  515 */
		offset = offset + (length<<3);
		break;
	case RNSAP_ID_D_RNTI:														/*  21 */
		offset = dissect_id_D_RNTI(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_D_RNTI_RELEASEINDICATION:										/*  22 */
		offset = dissect_id_D_RNTI_ReleaseIndication(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DCHS_TO_ADD_FDD:												/*  26 */
		offset = dissect_id_DCHs_to_Add_FDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DCHS_TO_ADD_TDD:												/*  27 */
		offset = dissect_id_DCHs_to_Add_TDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DCH_DELETELIST_RL_RECONFPREPFDD:								/*  30 */
		offset = dissect_id_DCH_DeleteList_RL_ReconfPrepFDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DCH_DELETELIST_RL_RECONFPREPTDD:								/*  31 */
		offset = dissect_id_DCH_DeleteList_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DCH_DELETELIST_RL_RECONFRQSTFDD:								/*  32 */
		offset = dissect_id_DCH_DeleteList_RL_ReconfRqstFDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DCH_DELETELIST_RL_RECONFRQSTTDD:								/*  33 */
		offset = dissect_id_DCH_DeleteList_RL_ReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DCH_FDD_INFORMATION:											/*  34 */
		offset = dissect_id_DCH_FDD_Information(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DCH_TDD_INFORMATION:											/*  35 */
		offset = dissect_id_DCH_TDD_Information(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_FDD_DCHS_TO_MODIFY:											/*  39 */
		offset = dissect_id_FDD_DCHs_to_Modify(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_TDD_DCHS_TO_MODIFY:											/*  40 */
		offset = dissect_id_TDD_DCHs_to_Modify(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DCH_INFORMATIONRESPONSE:										/*  43 */
		offset = dissect_id_DCH_InformationResponse(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DCH_RATE_INFORMATIONITEM_RL_CONGESTIND:						/*  38 */
		offset = dissect_id_DCH_Rate_InformationItem_RL_CongestInd(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_CCTRCH_INFORMATIONADDITEM_RL_RECONFPREPTDD:				/*  44 */
		offset = dissect_id_DL_CCTrCH_InformationAddItem_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_CCTRCH_INFORMATIONLISTIE_RL_RECONFREADYTDD:				/*  45 */
		offset = dissect_id_DL_CCTrCH_InformationListIE_RL_ReconfReadyTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_CCTRCH_INFORMATIONDELETEITEM_RL_RECONFRQSTTDD:				/*  46 */
		offset = dissect_id_DL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_CCTRCH_INFORMATIONITEM_RL_SETUPRQSTTDD:					/*  47 */
		offset = dissect_id_DL_CCTrCH_InformationItem_RL_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_CCTRCH_INFORMATIONLISTIE_PHYCHRECONFRQSTTDD:				/*  48 */
		offset = dissect_id_DL_CCTrCH_InformationListIE_PhyChReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_CCTRCH_INFORMATIONLISTIE_RL_ADDITIONRSPTDD:				/*  49 */
		offset = dissect_id_DL_CCTrCH_InformationListIE_RL_AdditionRspTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_CCTRCH_INFORMATIONLISTIE_RL_SETUPRSPTDD:					/*  50 */
		offset = dissect_id_DL_CCTrCH_InformationListIE_RL_SetupRspTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_CCTRCH_INFORMATIONADDLIST_RL_RECONFPREPTDD:				/*  51 */
		offset = dissect_id_DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_CCTRCH_INFORMATIONDELETELIST_RL_RECONFRQSTTDD:				/*  52 */
		offset = dissect_id_DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_CCTRCH_INFORMATIONLIST_RL_SETUPRQSTTDD:					/*  53 */
		offset = dissect_id_DL_CCTrCH_InformationList_RL_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_FDD_DL_CODEINFORMATION:										/*  54 */
		offset = dissect_id_FDD_DL_CodeInformation(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_DPCH_INFORMATION_RL_RECONFPREPFDD:							/*  59 */
		offset = dissect_id_DL_DPCH_Information_RL_ReconfPrepFDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_DPCH_INFORMATION_RL_SETUPRQSTFDD:							/*  60 */
		offset = dissect_id_DL_DPCH_Information_RL_SetupRqstFDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_DPCH_INFORMATION_RL_RECONFRQSTFDD:							/*  61 */
		offset = dissect_id_DL_DPCH_Information_RL_ReconfRqstFDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_DPCH_INFORMATIONITEM_PHYCHRECONFRQSTTDD:					/*  62 */
		offset = dissect_id_DL_DPCH_InformationItem_PhyChReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_DPCH_INFORMATIONITEM_RL_ADDITIONRSPTDD:					/*  63 */
		offset = dissect_id_DL_DPCH_InformationItem_RL_AdditionRspTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_DPCH_INFORMATIONITEM_RL_SETUPRSPTDD:						/*  64 */
		offset = dissect_id_DL_DPCH_InformationItem_RL_SetupRspTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_DPCH_TIMINGADJUSTMENT:										/*  278 */
		offset = dissect_id_DL_DPCH_TimingAdjustment(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DLREFERENCEPOWER:												/*  67 */
		offset = dissect_id_DLReferencePower(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DLREFERENCEPOWERLIST_DL_PC_RQST:								/*  68 */
		offset = dissect_id_DLReferencePowerList_DL_PC_Rqst(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_REFERENCEPOWERINFORMATION_DL_PC_RQST:						/*  69 */
		offset = dissect_id_DL_ReferencePowerInformation_DL_PC_Rqst(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DPC_MODE:														/*  12 */
		offset = dissect_id_DPC_Mode(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DRXCYCLELENGTHCOEFFICIENT:									/*  70 */
		offset = dissect_id_DRXCycleLengthCoefficient(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DEDICATEDMEASUREMENTOBJECTTYPE_DM_FAIL_IND:					/*  470 */
		offset = dissect_id_DedicatedMeasurementObjectType_DM_Fail_Ind(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DEDICATEDMEASUREMENTOBJECTTYPE_DM_FAIL:						/*  471 */
		offset = dissect_id_DedicatedMeasurementObjectType_DM_Fail(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DEDICATEDMEASUREMENTOBJECTTYPE_DM_RPRT:						/*  71 */
		offset = dissect_id_DedicatedMeasurementObjectType_DM_Rprt(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DEDICATEDMEASUREMENTOBJECTTYPE_DM_RQST:						/*  72 */
		offset = dissect_id_DedicatedMeasurementObjectType_DM_Rqst(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DEDICATEDMEASUREMENTOBJECTTYPE_DM_RSP:						/*  73 */
		offset = dissect_id_DedicatedMeasurementObjectType_DM_Rsp(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DEDICATEDMEASUREMENTTYPE:										/*  74 */
		offset = dissect_id_DedicatedMeasurementType(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_FACH_INFOFORUESELECTEDS_CCPCH_CTCH_RESOURCERSPFDD:			/*  82 */
		offset = dissect_id_FACH_InfoForUESelectedS_CCPCH_CTCH_ResourceRspFDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_FACH_INFOFORUESELECTEDS_CCPCH_CTCH_RESOURCERSPTDD:			/*  83 */
		offset = dissect_id_FACH_InfoForUESelectedS_CCPCH_CTCH_ResourceRspTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_GUARANTEED_RATE_INFORMATION:									/*  41 */
		offset = dissect_id_Guaranteed_Rate_Information(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_IMSI:															/*  84 */
		offset = dissect_id_IMSI(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_HCS_PRIO:														/*  311 */
		offset = dissect_id_HCS_Prio(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_L3_INFORMATION:												/*  85 */
		offset = dissect_id_L3_Information(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_ADJUSTMENTPERIOD:												/*  90 */
		offset = dissect_id_AdjustmentPeriod(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_MAXADJUSTMENTSTEP:											/*  91 */
		offset = dissect_id_MaxAdjustmentStep(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_MEASUREMENTFILTERCOEFFICIENT:									/*  92 */
		offset = dissect_id_MeasurementFilterCoefficient(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_MESSAGESTRUCTURE:												/*  57 */
		offset = dissect_id_MessageStructure(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_MEASUREMENTID:												/*  93 */
		offset = dissect_id_MeasurementID(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_NEIGHBOURING_GSM_CELLINFORMATION:								/*  13 */
		offset = dissect_id_Neighbouring_GSM_CellInformation(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_NEIGHBOURING_UMTS_CELLINFORMATIONITEM:						/*  95 */
		offset = dissect_id_Neighbouring_UMTS_CellInformationItem(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_NRT_LOAD_INFORMATION_VALUE:									/*  305 */
		offset = dissect_id_NRT_Load_Information_Value(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_NRT_LOAD_INFORMATION_VALUE_INCRDECRTHRES:						/*  306 */
		offset = dissect_id_NRT_Load_Information_Value_IncrDecrThres(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_PAGINGAREA_PAGINGRQST:										/*  102 */
		offset = dissect_id_PagingArea_PagingRqst(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_FACH_FLOWCONTROLINFORMATION:									/*  103 */
		offset = dissect_id_FACH_FlowControlInformation(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_PARTIALREPORTINGINDICATOR:									/*  472 */
		offset = dissect_id_PartialReportingIndicator(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_PERMANENT_NAS_UE_IDENTITY:									/*  17 */
		offset = dissect_id_Permanent_NAS_UE_Identity(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_POWERADJUSTMENTTYPE:											/*  107 */
		offset = dissect_id_PowerAdjustmentType(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RANAP_RELOCATIONINFORMATION:									/*  109 */
		offset = dissect_id_RANAP_RelocationInformation(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_INFORMATION_PHYCHRECONFRQSTFDD:							/*  110 */
		offset = dissect_id_RL_Information_PhyChReconfRqstFDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_INFORMATION_PHYCHRECONFRQSTTDD:							/*  111 */
		offset = dissect_id_RL_Information_PhyChReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_INFORMATION_RL_ADDITIONRQSTFDD:							/*  112 */
		offset = dissect_id_RL_Information_RL_AdditionRqstFDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_INFORMATION_RL_ADDITIONRQSTTDD:							/*  113 */
		offset = dissect_id_RL_Information_RL_AdditionRqstTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_INFORMATION_RL_DELETIONRQST:								/*  114 */
		offset = dissect_id_RL_Information_RL_DeletionRqst(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_INFORMATION_RL_FAILUREIND:									/*  115 */
		offset = dissect_id_RL_Information_RL_FailureInd(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_INFORMATION_RL_RECONFPREPFDD:								/*  116 */
		offset = dissect_id_RL_Information_RL_ReconfPrepFDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_INFORMATION_RL_RESTOREIND:									/*  117 */
		offset = dissect_id_RL_Information_RL_RestoreInd(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_INFORMATION_RL_SETUPRQSTFDD:								/*  118 */
		offset = dissect_id_RL_Information_RL_SetupRqstFDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_INFORMATION_RL_SETUPRQSTTDD:								/*  119 */
		offset = dissect_id_RL_Information_RL_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_INFORMATIONITEM_RL_CONGESTIND:								/*  55 */
		offset = dissect_id_RL_InformationItem_RL_CongestInd(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_INFORMATIONITEM_DM_RPRT:									/*  120 */
		offset = dissect_id_RL_InformationItem_DM_Rprt(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_INFORMATIONITEM_DM_RQST:									/*  121 */
		offset = dissect_id_RL_InformationItem_DM_Rqst(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_INFORMATIONITEM_DM_RSP:									/*  122 */
		offset = dissect_id_RL_InformationItem_DM_Rsp(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_INFORMATIONITEM_RL_PREEMPTREQUIREDIND:						/*  2 */
		offset = dissect_id_RL_InformationItem_RL_PreemptRequiredInd(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_INFORMATIONITEM_RL_SETUPRQSTFDD:							/*  123 */
		offset = dissect_id_RL_InformationItem_RL_SetupRqstFDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_INFORMATIONLIST_RL_CONGESTIND:								/*  56 */
		offset = dissect_id_RL_InformationList_RL_CongestInd(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_INFORMATIONLIST_RL_ADDITIONRQSTFDD:						/*  124 */
		offset = dissect_id_RL_InformationList_RL_AdditionRqstFDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_INFORMATIONLIST_RL_DELETIONRQST:							/*  125 */
		offset = dissect_id_RL_InformationList_RL_DeletionRqst(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_INFORMATIONLIST_RL_PREEMPTREQUIREDIND:						/*  1 */
		offset = dissect_id_RL_InformationList_RL_PreemptRequiredInd(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_INFORMATIONLIST_RL_RECONFPREPFDD:							/*  126 */
		offset = dissect_id_RL_InformationList_RL_ReconfPrepFDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_INFORMATIONRESPONSE_RL_ADDITIONRSPTDD:						/*  127 */
		offset = dissect_id_RL_InformationResponse_RL_AdditionRspTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_INFORMATIONRESPONSE_RL_RECONFREADYTDD:						/*  128 */
		offset = dissect_id_RL_InformationResponse_RL_ReconfReadyTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_INFORMATIONRESPONSE_RL_SETUPRSPTDD:						/*  129 */
		offset = dissect_id_RL_InformationResponse_RL_SetupRspTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_INFORMATIONRESPONSEITEM_RL_ADDITIONRSPFDD:					/*  130 */
		offset = dissect_id_RL_InformationResponseItem_RL_AdditionRspFDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_INFORMATIONRESPONSEITEM_RL_RECONFREADYFDD:					/*  131 */
		offset = dissect_id_RL_InformationResponseItem_RL_ReconfReadyFDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_INFORMATIONRESPONSEITEM_RL_RECONFRSPFDD:					/*  132 */
		offset = dissect_id_RL_InformationResponseItem_RL_ReconfRspFDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_INFORMATIONRESPONSEITEM_RL_SETUPRSPFDD:					/*  133 */
		offset = dissect_id_RL_InformationResponseItem_RL_SetupRspFDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_INFORMATIONRESPONSELIST_RL_ADDITIONRSPFDD:					/*  134 */
		offset = dissect_id_RL_InformationResponseList_RL_AdditionRspFDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_INFORMATIONRESPONSELIST_RL_RECONFREADYFDD:					/*  135 */
		offset = dissect_id_RL_InformationResponseList_RL_ReconfReadyFDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_INFORMATIONRESPONSELIST_RL_RECONFRSPFDD:					/*  136 */
		offset = dissect_id_RL_InformationResponseList_RL_ReconfRspFDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_INFORMATIONRESPONSE_RL_RECONFRSPTDD:						/*  28 */
		offset = dissect_id_RL_InformationResponse_RL_ReconfRspTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_INFORMATIONRESPONSELIST_RL_SETUPRSPFDD:					/*  137 */
		offset = dissect_id_RL_InformationResponseList_RL_SetupRspFDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_RECONFIGURATIONFAILURE_RL_RECONFFAIL:						/*  141 */
		offset = dissect_id_RL_ReconfigurationFailure_RL_ReconfFail(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_SET_INFORMATIONITEM_DM_RPRT:								/*  143 */
		offset = dissect_id_RL_Set_InformationItem_DM_Rprt(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_SET_INFORMATIONITEM_DM_RQST:								/*  144 */
		offset = dissect_id_RL_Set_InformationItem_DM_Rqst(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_SET_INFORMATIONITEM_DM_RSP:								/*  145 */
		offset = dissect_id_RL_Set_InformationItem_DM_Rsp(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_SET_INFORMATION_RL_FAILUREIND:								/*  146 */
		offset = dissect_id_RL_Set_Information_RL_FailureInd(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_SET_INFORMATION_RL_RESTOREIND:								/*  147 */
		offset = dissect_id_RL_Set_Information_RL_RestoreInd(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_SET_SUCCESSFUL_INFORMATIONITEM_DM_FAIL:					/*  473 */
		offset = dissect_id_RL_Set_Successful_InformationItem_DM_Fail(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_SET_UNSUCCESSFUL_INFORMATIONITEM_DM_FAIL:					/*  474 */
		offset = dissect_id_RL_Set_Unsuccessful_InformationItem_DM_Fail(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_SET_UNSUCCESSFUL_INFORMATIONITEM_DM_FAIL_IND:				/*  475 */
		offset = dissect_id_RL_Set_Unsuccessful_InformationItem_DM_Fail_Ind(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_SUCCESSFUL_INFORMATIONITEM_DM_FAIL:						/*  476 */
		offset = dissect_id_RL_Successful_InformationItem_DM_Fail(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_UNSUCCESSFUL_INFORMATIONITEM_DM_FAIL:						/*  477 */
		offset = dissect_id_RL_Unsuccessful_InformationItem_DM_Fail(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_UNSUCCESSFUL_INFORMATIONITEM_DM_FAIL_IND:					/*  478 */
		offset = dissect_id_RL_Unsuccessful_InformationItem_DM_Fail_Ind(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_REPORTCHARACTERISTICS:										/*  152 */
		offset = dissect_id_ReportCharacteristics(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_REPORTING_OBJECT_RL_FAILUREIND:								/*  153 */
		offset = dissect_id_Reporting_Object_RL_FailureInd(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_REPORING_OBJECT_RL_RESTOREIND:								/*  154 */
		offset = dissect_id_Reporing_Object_RL_RestoreInd(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RT_LOAD_VALUE:												/*  307 */
		offset = dissect_id_RT_Load_Value(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RT_LOAD_VALUE_INCRDECRTHRES:									/*  308 */
		offset = dissect_id_RT_Load_Value_IncrDecrThres(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_S_RNTI:														/*  155 */
		offset = dissect_id_S_RNTI(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RESETINDICATOR:												/*  244 */
		offset = dissect_id_ResetIndicator(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RNC_ID:														/*  245 */
		offset = dissect_id_RNC_ID(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_SAI:															/*  156 */
		offset = dissect_id_SAI(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_SRNC_ID:														/*  157 */
		offset = dissect_id_SRNC_ID(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_SUCCESSFULRL_INFORMATIONRESPONSE_RL_ADDITIONFAILUREFDD:		/*  159 */
		offset = dissect_id_SuccessfulRL_InformationResponse_RL_AdditionFailureFDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_SUCCESSFULRL_INFORMATIONRESPONSE_RL_SETUPFAILUREFDD:			/*  160 */
		offset = dissect_id_SuccessfulRL_InformationResponse_RL_SetupFailureFDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_TRANSPORTBEARERID:											/*  163 */
		offset = dissect_id_TransportBearerID(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_TRANSPORTBEARERREQUESTINDICATOR:								/*  164 */
		offset = dissect_id_TransportBearerRequestIndicator(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_TRANSPORTLAYERADDRESS:										/*  165 */
		offset = dissect_id_TransportLayerAddress(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_TYPEOFERROR:													/*  140 */
		offset = dissect_id_TypeOfError(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UC_ID:														/*  166 */
		offset = dissect_id_UC_ID(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UL_CCTRCH_ADDINFORMATION_RL_RECONFPREPTDD:					/*  167 */
		offset = dissect_id_UL_CCTrCH_AddInformation_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UL_CCTRCH_INFORMATIONADDLIST_RL_RECONFPREPTDD:				/*  169 */
		offset = dissect_id_UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UL_CCTRCH_INFORMATIONITEM_RL_SETUPRQSTTDD:					/*  171 */
		offset = dissect_id_UL_CCTrCH_InformationItem_RL_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UL_CCTRCH_INFORMATIONLIST_RL_SETUPRQSTTDD:					/*  172 */
		offset = dissect_id_UL_CCTrCH_InformationList_RL_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UL_CCTRCH_INFORMATIONLISTIE_PHYCHRECONFRQSTTDD:				/*  173 */
		offset = dissect_id_UL_CCTrCH_InformationListIE_PhyChReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UL_CCTRCH_INFORMATIONLISTIE_RL_ADDITIONRSPTDD:				/*  174 */
		offset = dissect_id_UL_CCTrCH_InformationListIE_RL_AdditionRspTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UL_CCTRCH_INFORMATIONLISTIE_RL_RECONFREADYTDD:				/*  175 */
		offset = dissect_id_UL_CCTrCH_InformationListIE_RL_ReconfReadyTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UL_CCTRCH_INFORMATIONLISTIE_RL_SETUPRSPTDD:					/*  176 */
		offset = dissect_id_UL_CCTrCH_InformationListIE_RL_SetupRspTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UL_DPCH_INFORMATION_RL_RECONFPREPFDD:							/*  177 */
		offset = dissect_id_UL_DPCH_Information_RL_ReconfPrepFDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UL_DPCH_INFORMATION_RL_RECONFRQSTFDD:							/*  178 */
		offset = dissect_id_UL_DPCH_Information_RL_ReconfRqstFDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UL_DPCH_INFORMATION_RL_SETUPRQSTFDD:							/*  179 */
		offset = dissect_id_UL_DPCH_Information_RL_SetupRqstFDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UL_DPCH_INFORMATIONITEM_PHYCHRECONFRQSTTDD:					/*  180 */
		offset = dissect_id_UL_DPCH_InformationItem_PhyChReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UL_DPCH_INFORMATIONITEM_RL_ADDITIONRSPTDD:					/*  181 */
		offset = dissect_id_UL_DPCH_InformationItem_RL_AdditionRspTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UL_DPCH_INFORMATIONITEM_RL_SETUPRSPTDD:						/*  182 */
		offset = dissect_id_UL_DPCH_InformationItem_RL_SetupRspTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UL_DPCH_INFORMATIONADDLISTIE_RL_RECONFREADYTDD:				/*  183 */
		offset = dissect_id_UL_DPCH_InformationAddListIE_RL_ReconfReadyTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UL_SIRTARGET:													/*  184 */
		offset = dissect_id_UL_SIRTarget(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_URA_INFORMATION:												/*  185 */
		offset = dissect_id_URA_Information(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UNSUCCESSFULRL_INFORMATIONRESPONSE_RL_ADDITIONFAILUREFDD:		/*  188 */
		offset = dissect_id_UnsuccessfulRL_InformationResponse_RL_AdditionFailureFDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UNSUCCESSFULRL_INFORMATIONRESPONSE_RL_SETUPFAILUREFDD:		/*  189 */
		offset = dissect_id_UnsuccessfulRL_InformationResponse_RL_SetupFailureFDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UNSUCCESSFULRL_INFORMATIONRESPONSE_RL_SETUPFAILURETDD:		/*  190 */
		offset = dissect_id_UnsuccessfulRL_InformationResponse_RL_SetupFailureTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_ACTIVE_PATTERN_SEQUENCE_INFORMATION:							/*  193 */
		offset = dissect_id_Active_Pattern_Sequence_Information(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_ADJUSTMENTRATIO:												/*  194 */
		offset = dissect_id_AdjustmentRatio(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_CAUSELEVEL_RL_ADDITIONFAILUREFDD:								/*  197 */
		offset = dissect_id_CauseLevel_RL_AdditionFailureFDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_CAUSELEVEL_RL_ADDITIONFAILURETDD:								/*  198 */
		offset = dissect_id_CauseLevel_RL_AdditionFailureTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_CAUSELEVEL_RL_RECONFFAILURE:									/*  199 */
		offset = dissect_id_CauseLevel_RL_ReconfFailure(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_CAUSELEVEL_RL_SETUPFAILUREFDD:								/*  200 */
		offset = dissect_id_CauseLevel_RL_SetupFailureFDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_CAUSELEVEL_RL_SETUPFAILURETDD:								/*  201 */
		offset = dissect_id_CauseLevel_RL_SetupFailureTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_CCTRCH_INFORMATIONDELETEITEM_RL_RECONFPREPTDD:				/*  205 */
		offset = dissect_id_DL_CCTrCH_InformationDeleteItem_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_CCTRCH_INFORMATIONMODIFYITEM_RL_RECONFPREPTDD:				/*  206 */
		offset = dissect_id_DL_CCTrCH_InformationModifyItem_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_CCTRCH_INFORMATIONMODIFYITEM_RL_RECONFRQSTTDD:				/*  207 */
		offset = dissect_id_DL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_CCTRCH_INFORMATIONDELETELIST_RL_RECONFPREPTDD:				/*  208 */
		offset = dissect_id_DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_CCTRCH_INFORMATIONMODIFYLIST_RL_RECONFPREPTDD:				/*  209 */
		offset = dissect_id_DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_CCTRCH_INFORMATIONMODIFYLIST_RL_RECONFRQSTTDD:				/*  210 */
		offset = dissect_id_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_DPCH_INFORMATIONADDLISTIE_RL_RECONFREADYTDD:				/*  212 */
		offset = dissect_id_DL_DPCH_InformationAddListIE_RL_ReconfReadyTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_DPCH_INFORMATIONDELETELISTIE_RL_RECONFREADYTDD:			/*  213 */
		offset = dissect_id_DL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_DPCH_INFORMATIONMODIFYLISTIE_RL_RECONFREADYTDD:			/*  214 */
		offset = dissect_id_DL_DPCH_InformationModifyListIE_RL_ReconfReadyTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DSCHS_TO_ADD_TDD:												/*  215 */
		offset = dissect_id_DSCHs_to_Add_TDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_216:								/*  216 */
		break;
	case RNSAP_ID_DSCH_DELETELIST_RL_RECONFPREPTDD:								/*  217 */
		offset = dissect_id_DSCH_DeleteList_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_218:								/*  218 */
		break;
	case RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_219:								/*  219 */
		break;
	case RNSAP_ID_DSCH_INFORMATIONLISTIE_RL_ADDITIONRSPTDD:						/*  220 */
		offset = dissect_id_DSCH_InformationListIE_RL_AdditionRspTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DSCH_INFORMATIONLISTIES_RL_SETUPRSPTDD:						/*  221 */
		offset = dissect_id_DSCH_InformationListIEs_RL_SetupRspTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DSCH_TDD_INFORMATION:											/*  222 */
		offset = dissect_id_DSCH_TDD_Information(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_223:								/*  223 */
	case RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_226:								/*  226 */
		break;
	case RNSAP_ID_DSCH_MODIFYLIST_RL_RECONFPREPTDD:								/*  227 */
		offset = dissect_id_DSCH_ModifyList_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_228:								/*  228 */
	case RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_324:								/*  324 */
	case RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_229:								/*  229 */
		break;
	case RNSAP_ID_DSCHTOBEADDEDORMODIFIEDLIST_RL_RECONFREADYTDD:				/*  230 */
		offset = dissect_id_DSCHToBeAddedOrModifiedList_RL_ReconfReadyTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_29:								/*  29 */
	case RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_225:								/*  225 */
		break;
	case RNSAP_ID_GA_CELL:														/*  232 */
		offset = dissect_id_GA_Cell(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_GA_CELLADDITIONALSHAPES:										/*  3 */
		offset = dissect_id_GA_CellAdditionalShapes(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_246:								/*  246 */
		break;
	case RNSAP_ID_TRANSMISSION_GAP_PATTERN_SEQUENCE_INFORMATION:				/*  255 */
		offset = dissect_id_Transmission_Gap_Pattern_Sequence_Information(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UL_CCTRCH_DELETEINFORMATION_RL_RECONFPREPTDD:					/*  256 */
		offset = dissect_id_UL_CCTrCH_DeleteInformation_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UL_CCTRCH_MODIFYINFORMATION_RL_RECONFPREPTDD:					/*  257 */
		offset = dissect_id_UL_CCTrCH_ModifyInformation_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UL_CCTRCH_INFORMATIONMODIFYITEM_RL_RECONFRQSTTDD:				/*  258 */
		offset = dissect_id_UL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UL_CCTRCH_INFORMATIONDELETELIST_RL_RECONFPREPTDD:				/*  259 */
		offset = dissect_id_UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UL_CCTRCH_INFORMATIONMODIFYLIST_RL_RECONFPREPTDD:				/*  260 */
		offset = dissect_id_UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UL_CCTRCH_INFORMATIONMODIFYLIST_RL_RECONFRQSTTDD:				/*  261 */
		offset = dissect_id_UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UL_CCTRCH_INFORMATIONDELETEITEM_RL_RECONFRQSTTDD:				/*  262 */
		offset = dissect_id_UL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UL_CCTRCH_INFORMATIONDELETELIST_RL_RECONFRQSTTDD:				/*  263 */
		offset = dissect_id_UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UL_DPCH_INFORMATIONDELETELISTIE_RL_RECONFREADYTDD:			/*  264 */
		offset = dissect_id_UL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UL_DPCH_INFORMATIONMODIFYLISTIE_RL_RECONFREADYTDD:			/*  265 */
		offset = dissect_id_UL_DPCH_InformationModifyListIE_RL_ReconfReadyTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UNSUCCESSFULRL_INFORMATIONRESPONSE_RL_ADDITIONFAILURETDD:		/*  266 */
		offset = dissect_id_UnsuccessfulRL_InformationResponse_RL_AdditionFailureTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_USCHS_TO_ADD:													/*  267 */
		offset = dissect_id_USCHs_to_Add(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_USCH_DELETELIST_RL_RECONFPREPTDD:								/*  268 */
		offset = dissect_id_USCH_DeleteList_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_USCH_INFORMATIONLISTIE_RL_ADDITIONRSPTDD:						/*  269 */
		offset = dissect_id_USCH_InformationListIE_RL_AdditionRspTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_USCH_INFORMATIONLISTIES_RL_SETUPRSPTDD:						/*  270 */
		offset = dissect_id_USCH_InformationListIEs_RL_SetupRspTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_USCH_INFORMATION:												/*  271 */
		offset = dissect_id_USCH_Information(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_USCH_MODIFYLIST_RL_RECONFPREPTDD:								/*  272 */
		offset = dissect_id_USCH_ModifyList_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_USCHTOBEADDEDORMODIFIEDLIST_RL_RECONFREADYTDD:				/*  273 */
		offset = dissect_id_USCHToBeAddedOrModifiedList_RL_ReconfReadyTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_PHYSICAL_CHANNEL_INFORMATION_RL_SETUPRQSTTDD:				/*  274 */
		offset = dissect_id_DL_Physical_Channel_Information_RL_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UL_PHYSICAL_CHANNEL_INFORMATION_RL_SETUPRQSTTDD:				/*  275 */
		offset = dissect_id_UL_Physical_Channel_Information_RL_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_CLOSEDLOOPMODE1_SUPPORTINDICATOR:								/*  276 */
		offset = dissect_id_ClosedLoopMode1_SupportIndicator(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_277:								/*  277 */
		break;
	case RNSAP_ID_STTD_SUPPORTINDICATOR:										/*  279 */
		offset = dissect_id_STTD_SupportIndicator(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_CFNREPORTINGINDICATOR:   										/*  14 */
		offset = dissect_id_CFNReportingIndicator(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_CNORIGINATEDPAGE_PAGINGRQST:									/*  23 */
		offset = dissect_id_CNOriginatedPage_PagingRqst(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_INNERLOOPDLPCSTATUS:											/*  24 */
		offset = dissect_id_InnerLoopDLPCStatus(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_PROPAGATIONDELAY:												/*  25 */
		offset = dissect_id_PropagationDelay(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RXTIMINGDEVIATIONFORTA:										/*  36 */
		offset = dissect_id_RxTimingDeviationForTA(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_TIMESLOT_ISCP:												/*  37 */
		offset = dissect_id_timeSlot_ISCP(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_CCTRCH_INFORMATIONITEM_RL_FAILUREIND:							/*  15 */
		offset = dissect_id_CCTrCH_InformationItem_RL_FailureInd(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_CCTRCH_INFORMATIONITEM_RL_RESTOREIND:							/*  16 */
		offset = dissect_id_CCTrCH_InformationItem_RL_RestoreInd(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_COMMONMEASUREMENTACCURACY:									/*  280 */
		offset = dissect_id_CommonMeasurementAccuracy(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_COMMONMEASUREMENTOBJECTTYPE_CM_RPRT:							/*  281 */
		offset = dissect_id_CommonMeasurementObjectType_CM_Rprt(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_COMMONMEASUREMENTOBJECTTYPE_CM_RQST:							/*  282 */
		offset = dissect_id_CommonMeasurementObjectType_CM_Rqst(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_COMMONMEASUREMENTOBJECTTYPE_CM_RSP:							/*  283 */
		offset = dissect_id_CommonMeasurementObjectType_CM_Rsp(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_COMMONMEASUREMENTTYPE:										/*  284 */
		offset = dissect_id_CommonMeasurementType(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_CONGESTIONCAUSE:												/*  18 */
		offset = dissect_id_CongestionCause(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_SFN:															/*  285 */
		offset = dissect_id_SFN(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_SFNREPORTINGINDICATOR:										/*  286 */
		offset = dissect_id_SFNReportingIndicator(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_INFORMATIONEXCHANGEID:										/*  287 */
		offset = dissect_id_InformationExchangeID(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_INFORMATIONEXCHANGEOBJECTTYPE_INFEX_RPRT:						/*  288 */
		offset = dissect_id_InformationExchangeObjectType_InfEx_Rprt(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_INFORMATIONEXCHANGEOBJECTTYPE_INFEX_RQST:						/*  289 */
		offset = dissect_id_InformationExchangeObjectType_InfEx_Rqst(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_INFORMATIONEXCHANGEOBJECTTYPE_INFEX_RSP:						/*  290 */
		offset = dissect_id_InformationExchangeObjectType_InfEx_Rsp(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_INFORMATIONREPORTCHARACTERISTICS:								/*  291 */
		offset = dissect_id_InformationReportCharacteristics(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_INFORMATIONTYPE:												/*  292 */
		offset = dissect_id_InformationType(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_NEIGHBOURING_LCR_TDD_CELLINFORMATION:							/*  58 */
		offset = dissect_id_neighbouring_LCR_TDD_CellInformation(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_TIMESLOT_ISCP_LCR_INFORMATION_RL_SETUPRQSTTDD:				/*  65 */
		offset = dissect_id_DL_Timeslot_ISCP_LCR_Information_RL_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_LCR_INFORMATIONRESPONSE_RL_SETUPRSPTDD:					/*  66 */
		offset = dissect_id_RL_LCR_InformationResponse_RL_SetupRspTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UL_CCTRCH_LCR_INFORMATIONLISTIE_RL_SETUPRSPTDD:				/*  75 */
		offset = dissect_id_UL_CCTrCH_LCR_InformationListIE_RL_SetupRspTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UL_DPCH_LCR_INFORMATIONITEM_RL_SETUPRSPTDD:					/*  76 */
		offset = dissect_id_UL_DPCH_LCR_InformationItem_RL_SetupRspTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_CCTRCH_LCR_INFORMATIONLISTIE_RL_SETUPRSPTDD:				/*  77 */
		offset = dissect_id_DL_CCTrCH_LCR_InformationListIE_RL_SetupRspTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_DPCH_LCR_INFORMATIONITEM_RL_SETUPRSPTDD:					/*  78 */
		offset = dissect_id_DL_DPCH_LCR_InformationItem_RL_SetupRspTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DSCH_LCR_INFORMATIONLISTIES_RL_SETUPRSPTDD:					/*  79 */
		offset = dissect_id_DSCH_LCR_InformationListIEs_RL_SetupRspTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_USCH_LCR_INFORMATIONLISTIES_RL_SETUPRSPTDD:					/*  80 */
		offset = dissect_id_USCH_LCR_InformationListIEs_RL_SetupRspTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_TIMESLOT_ISCP_LCR_INFORMATION_RL_ADDITIONRQSTTDD:			/*  81 */
		offset = dissect_id_DL_Timeslot_ISCP_LCR_Information_RL_AdditionRqstTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_LCR_INFORMATIONRESPONSE_RL_ADDITIONRSPTDD:					/*  86 */
		offset = dissect_id_RL_LCR_InformationResponse_RL_AdditionRspTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UL_CCTRCH_LCR_INFORMATIONLISTIE_RL_ADDITIONRSPTDD:			/*  87 */
		offset = dissect_id_UL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UL_DPCH_LCR_INFORMATIONITEM_RL_ADDITIONRSPTDD:				/*  88 */
		offset = dissect_id_UL_DPCH_LCR_InformationItem_RL_AdditionRspTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_CCTRCH_LCR_INFORMATIONLISTIE_RL_ADDITIONRSPTDD:			/*  89 */
		offset = dissect_id_DL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_DPCH_LCR_INFORMATIONITEM_RL_ADDITIONRSPTDD:				/*  94 */
		offset = dissect_id_DL_DPCH_LCR_InformationItem_RL_AdditionRspTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DSCH_LCR_INFORMATIONLISTIES_RL_ADDITIONRSPTDD:				/*  96 */
		offset = dissect_id_DSCH_LCR_InformationListIEs_RL_AdditionRspTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_USCH_LCR_INFORMATIONLISTIES_RL_ADDITIONRSPTDD:				/*  97 */
		offset = dissect_id_USCH_LCR_InformationListIEs_RL_AdditionRspTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UL_DPCH_LCR_INFORMATIONADDLISTIE_RL_RECONFREADYTDD:			/*  98 */
		offset = dissect_id_UL_DPCH_LCR_InformationAddListIE_RL_ReconfReadyTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UL_TIMESLOT_LCR_INFORMATIONMODIFYLIST_RL_RECONFREADYTDD:		/*  100 */
		offset = dissect_id_UL_Timeslot_LCR_InformationModifyList_RL_ReconfReadyTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_DPCH_LCR_INFORMATIONADDLISTIE_RL_RECONFREADYTDD:			/*  101 */
		offset = dissect_id_DL_DPCH_LCR_InformationAddListIE_RL_ReconfReadyTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_TIMESLOT_LCR_INFORMATIONMODIFYLIST_RL_RECONFREADYTDD:		/*  104 */
		offset = dissect_id_DL_Timeslot_LCR_InformationModifyList_RL_ReconfReadyTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UL_TIMESLOT_LCR_INFORMATIONLIST_PHYCHRECONFRQSTTDD:			/*  105 */
		offset = dissect_id_UL_Timeslot_LCR_InformationList_PhyChReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_TIMESLOT_LCR_INFORMATIONLIST_PHYCHRECONFRQSTTDD:			/*  106 */
		offset = dissect_id_DL_Timeslot_LCR_InformationList_PhyChReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_TIMESLOT_ISCP_LCR_LIST_DL_PC_RQST_TDD:						/*  138 */
		offset = dissect_id_timeSlot_ISCP_LCR_List_DL_PC_Rqst_TDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_TSTD_SUPPORT_INDICATOR_RL_SETUPRQSTTDD:						/*  139 */
		offset = dissect_id_TSTD_Support_Indicator_RL_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RESTRICTIONSTATEINDICATOR:									/*  142 */
		offset = dissect_id_RestrictionStateIndicator(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_LOAD_VALUE:													/*  233 */
		offset = dissect_id_Load_Value(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_LOAD_VALUE_INCRDECRTHRES:										/*  234 */
		offset = dissect_id_Load_Value_IncrDecrThres(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_ONMODIFICATION:												/*  235 */
		offset = dissect_id_OnModification(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RECEIVED_TOTAL_WIDEBAND_POWER_VALUE:							/*  236 */
		offset = dissect_id_Received_Total_Wideband_Power_Value(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RECEIVED_TOTAL_WIDEBAND_POWER_VALUE_INCRDECRTHRES:			/*  237 */
		offset = dissect_id_Received_Total_Wideband_Power_Value_IncrDecrThres(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_SFNSFNMEASUREMENTTHRESHOLDINFORMATION:						/*  238 */
		offset = dissect_id_SFNSFNMeasurementThresholdInformation(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_TRANSMITTED_CARRIER_POWER_VALUE:								/*  239 */
		offset = dissect_id_Transmitted_Carrier_Power_Value(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_TRANSMITTED_CARRIER_POWER_VALUE_INCRDECRTHRES:				/*  240 */
		offset = dissect_id_Transmitted_Carrier_Power_Value_IncrDecrThres(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_TUTRANGPSMEASUREMENTTHRESHOLDINFORMATION:						/*  241 */
		offset = dissect_id_TUTRANGPSMeasurementThresholdInformation(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UL_TIMESLOT_ISCP_VALUE:										/*  242 */
		offset = dissect_id_UL_Timeslot_ISCP_Value(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UL_TIMESLOT_ISCP_VALUE_INCRDECRTHRES:							/*  243 */
		offset = dissect_id_UL_Timeslot_ISCP_Value_IncrDecrThres(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RX_TIMING_DEVIATION_VALUE_LCR:								/*  293 */
		offset = dissect_id_Rx_Timing_Deviation_Value_LCR(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DPC_MODE_CHANGE_SUPPORTINDICATOR:								/*  19 */
		offset = dissect_id_DPC_Mode_Change_SupportIndicator(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_247:								/*  247 */
		break;
	case RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_295:								/*  295 */
		break;
	case RNSAP_ID_PRIMARYCCPCH_RSCP_RL_RECONFPREPTDD:							/*  202 */
		offset = dissect_id_PrimaryCCPCH_RSCP_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_TIMESLOT_ISCP_INFO_RL_RECONFPREPTDD:						/*  203 */
		offset = dissect_id_DL_TimeSlot_ISCP_Info_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_TIMESLOT_ISCP_LCR_INFORMATION_RL_RECONFPREPTDD:			/*  204 */
		offset = dissect_id_DL_Timeslot_ISCP_LCR_Information_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DSCH_RNTI:													/*  249 */
		offset = dissect_id_DSCH_RNTI(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_POWERBALANCING_INFORMATION:								/*  296 */
		offset = dissect_id_DL_PowerBalancing_Information(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_POWERBALANCING_ACTIVATIONINDICATOR:						/*  297 */
		offset = dissect_id_DL_PowerBalancing_ActivationIndicator(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_POWERBALANCING_UPDATEDINDICATOR:							/*  298 */
		offset = dissect_id_DL_PowerBalancing_UpdatedIndicator(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_REFERENCEPOWERINFORMATION:									/*  299 */
		offset = dissect_id_DL_ReferencePowerInformation(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_ENHANCED_PRIMARYCPICH_ECNO:									/*  224 */
		offset = dissect_id_Enhanced_PrimaryCPICH_EcNo(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_IPDL_TDD_PARAMETERSLCR:										/*  252 */
		offset = dissect_id_IPDL_TDD_ParametersLCR(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_CELLCAPABILITYCONTAINER_FDD:									/*  300 */
		offset = dissect_id_CellCapabilityContainer_FDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_CELLCAPABILITYCONTAINER_TDD:									/*  301 */
		offset = dissect_id_CellCapabilityContainer_TDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_CELLCAPABILITYCONTAINER_TDD_LCR:								/*  302 */
		offset = dissect_id_CellCapabilityContainer_TDD_LCR(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_SPECIFIC_DCH_INFO:											/*  317 */
		offset = dissect_id_RL_Specific_DCH_Info(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_RECONFIGURATIONREQUESTFDD_RL_INFORMATIONLIST:				/*  318 */
		offset = dissect_id_RL_ReconfigurationRequestFDD_RL_InformationList(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_RECONFIGURATIONREQUESTFDD_RL_INFORMATION_IES:				/*  319 */
		offset = dissect_id_RL_ReconfigurationRequestFDD_RL_Information_IEs(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_RECONFIGURATIONREQUESTTDD_RL_INFORMATION:					/*  321 */
		offset = dissect_id_RL_ReconfigurationRequestTDD_RL_Information(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_COMMONTRANSPORTCHANNELRESOURCESINITIALISATIONNOTREQUIRED:		/*  250 */
		offset = dissect_id_CommonTransportChannelResourcesInitialisationNotRequired(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DELAYEDACTIVATION:											/*  312 */
		offset = dissect_id_DelayedActivation(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DELAYEDACTIVATIONLIST_RL_ACTIVATIONCMDFDD:					/*  313 */
		offset = dissect_id_DelayedActivationList_RL_ActivationCmdFDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DELAYEDACTIVATIONINFORMATION_RL_ACTIVATIONCMDFDD:				/*  314 */
		offset = dissect_id_DelayedActivationInformation_RL_ActivationCmdFDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DELAYEDACTIVATIONLIST_RL_ACTIVATIONCMDTDD:					/*  315 */
		offset = dissect_id_DelayedActivationList_RL_ActivationCmdTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DELAYEDACTIVATIONINFORMATION_RL_ACTIVATIONCMDTDD:				/*  316 */
		offset = dissect_id_DelayedActivationInformation_RL_ActivationCmdTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_NEIGHBOURINGTDDCELLMEASUREMENTINFORMATIONLCR:					/*  251 */
		offset = dissect_id_neighbouringTDDCellMeasurementInformationLCR(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UL_SIR_TARGET_CCTRCH_INFORMATIONITEM_RL_SETUPRSPTDD:			/*  150 */
		offset = dissect_id_UL_SIR_Target_CCTrCH_InformationItem_RL_SetupRspTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UL_SIR_TARGET_CCTRCH_LCR_INFORMATIONITEM_RL_SETUPRSPTDD:		/*  151 */
		offset = dissect_id_UL_SIR_Target_CCTrCH_LCR_InformationItem_RL_SetupRspTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_PRIMCCPCH_RSCP_DL_PC_RQSTTDD:									/*  451 */
		offset = dissect_id_PrimCCPCH_RSCP_DL_PC_RqstTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_HSDSCH_FDD_INFORMATION:										/*  452 */
		offset = dissect_id_HSDSCH_FDD_Information(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_HSDSCH_FDD_INFORMATION_RESPONSE:								/*  453 */
		offset = dissect_id_HSDSCH_FDD_Information_Response(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_HSDSCH_FDD_UPDATE_INFORMATION:								/*  466 */
		offset = dissect_id_HSDSCH_FDD_Update_Information(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_HSDSCH_INFORMATION_TO_MODIFY:									/*  456 */
		offset = dissect_id_HSDSCH_Information_to_Modify(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_HSDSCHMACDFLOWSPECIFICINFORMATIONLIST_RL_PREEMPTREQUIREDIND:	/*  516 */
		offset = dissect_id_HSDSCHMacdFlowSpecificInformationList_RL_PreemptRequiredInd(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_HSDSCHMACDFLOWSPECIFICINFORMATIONITEM_RL_PREEMPTREQUIREDIND:	/*  517 */
		offset = dissect_id_HSDSCHMacdFlowSpecificInformationItem_RL_PreemptRequiredInd(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_HSDSCH_RNTI:													/*  457 */
		offset = dissect_id_HSDSCH_RNTI(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_HSDSCH_TDD_INFORMATION:										/*  458 */
		offset = dissect_id_HSDSCH_TDD_Information(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_HSDSCH_TDD_INFORMATION_RESPONSE:								/*  459 */
		offset = dissect_id_HSDSCH_TDD_Information_Response(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_HSDSCH_TDD_UPDATE_INFORMATION:								/*  467 */
		offset = dissect_id_HSDSCH_TDD_Update_Information(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_HSPDSCH_RL_ID:												/*  463 */
		offset = dissect_id_HSPDSCH_RL_ID(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_HSDSCH_MACDFLOWS_TO_ADD:										/*  531 */
		offset = dissect_id_HSDSCH_MACdFlows_to_Add(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_HSDSCH_MACDFLOWS_TO_DELETE:									/*  532 */
		offset = dissect_id_HSDSCH_MACdFlows_to_Delete(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_ANGLE_OF_ARRIVAL_VALUE_LCR:									/*  148 */
		offset = dissect_id_Angle_Of_Arrival_Value_LCR(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_TRAFFICCLASS:													/*  158 */
		offset = dissect_id_TrafficClass(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_248:								/*  248 */
		break;
	case RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_253:								/*  253 */
		break;
	case RNSAP_ID_PDSCH_RL_ID:													/*  323 */
		offset = dissect_id_PDSCH_RL_ID(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_TIMESLOT_RL_SETUPRSPTDD:										/*  325 */
		offset = dissect_id_TimeSlot_RL_SetupRspTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_GERAN_CELL_CAPABILITY:										/*  468 */
		offset = dissect_id_GERAN_Cell_Capability(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_GERAN_CLASSMARK:												/*  469 */
		offset = dissect_id_GERAN_Classmark(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DSCH_INITIALWINDOWSIZE:										/*  480 */
		offset = dissect_id_DSCH_InitialWindowSize(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UL_SYNCHRONISATION_PARAMETERS_LCR:							/*  464 */
		offset = dissect_id_UL_Synchronisation_Parameters_LCR(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_SNA_INFORMATION:												/*  479 */
		offset = dissect_id_SNA_Information(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_MACHS_RESETINDICATOR:											/*  465 */
		offset = dissect_id_MAChs_ResetIndicator(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_TDD_DL_DPCH_TIMESLOTFORMATMODIFYITEM_LCR_RL_RECONFREADYTDD:	/*  481 */
		offset = dissect_id_TDD_DL_DPCH_TimeSlotFormatModifyItem_LCR_RL_ReconfReadyTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_TDD_UL_DPCH_TIMESLOTFORMATMODIFYITEM_LCR_RL_RECONFREADYTDD:	/*  482 */
		offset = dissect_id_TDD_UL_DPCH_TimeSlotFormatModifyItem_LCR_RL_ReconfReadyTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_TDD_TPC_UPLINKSTEPSIZE_LCR_RL_SETUPRQSTTDD:					/*  483 */
		offset = dissect_id_TDD_TPC_UplinkStepSize_LCR_RL_SetupRqstTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UL_CCTRCH_INFORMATIONLIST_RL_ADDITIONRQSTTDD:					/*  484 */
		offset = dissect_id_UL_CCTrCH_InformationList_RL_AdditionRqstTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UL_CCTRCH_INFORMATIONITEM_RL_ADDITIONRQSTTDD:					/*  485 */
		offset = dissect_id_UL_CCTrCH_InformationItem_RL_AdditionRqstTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_CCTRCH_INFORMATIONLIST_RL_ADDITIONRQSTTDD:					/*  486 */
		offset = dissect_id_DL_CCTrCH_InformationList_RL_AdditionRqstTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_CCTRCH_INFORMATIONITEM_RL_ADDITIONRQSTTDD:					/*  487 */
		offset = dissect_id_DL_CCTrCH_InformationItem_RL_AdditionRqstTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_TDD_TPC_UPLINKSTEPSIZE_INFORMATIONADD_LCR_RL_RECONFPREPTDD:	/*  488 */
		offset = dissect_id_TDD_TPC_UplinkStepSize_InformationAdd_LCR_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_TDD_TPC_UPLINKSTEPSIZE_INFORMATIONMODIFY_LCR_RL_RECONFPREPTDD:/*  489 */
		offset = dissect_id_TDD_TPC_UplinkStepSize_InformationModify_LCR_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_TDD_TPC_DOWNLINKSTEPSIZE_INFORMATIONADD_RL_RECONFPREPTDD:		/*  490 */
		offset = dissect_id_TDD_TPC_DownlinkStepSize_InformationAdd_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_TDD_TPC_DOWNLINKSTEPSIZE_INFORMATIONMODIFY_RL_RECONFPREPTDD:	/*  491 */
		offset = dissect_id_TDD_TPC_DownlinkStepSize_InformationModify_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UL_TIMINGADVANCECTRL_LCR:										/*  492 */
		offset = dissect_id_UL_TimingAdvanceCtrl_LCR(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_HSPDSCH_TIMESLOT_INFORMATIONLIST_PHYCHRECONFRQSTTDD:			/*  493 */
		offset = dissect_id_HSPDSCH_Timeslot_InformationList_PhyChReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_HSPDSCH_TIMESLOT_INFORMATIONLISTLCR_PHYCHRECONFRQSTTDD:		/*  494 */
		offset = dissect_id_HSPDSCH_Timeslot_InformationListLCR_PhyChReconfRqstTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_HS_SICH_RECEPTION_QUALITY: 									/*  495 */
		offset = dissect_id_HS_SICH_Reception_Quality(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_HS_SICH_RECEPTION_QUALITY_MEASUREMENT_VALUE:					/*  496 */
		offset = dissect_id_HS_SICH_Reception_Quality_Measurement_Value(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_HSSICH_INFO_DM_RPRT:											/*  497 */
		offset = dissect_id_HSSICH_Info_DM_Rprt(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_HSSICH_INFO_DM_RQST:											/*  498 */
		offset = dissect_id_HSSICH_Info_DM_Rqst(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_HSSICH_INFO_DM:												/*  499 */
		offset = dissect_id_HSSICH_Info_DM(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_CCTRCH_MAXIMUM_DL_POWER_RL_SETUPRSPTDD:						/*  500 */
		offset = dissect_id_CCTrCH_Maximum_DL_Power_RL_SetupRspTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_CCTRCH_MINIMUM_DL_POWER_RL_SETUPRSPTDD:						/*  501 */
		offset = dissect_id_CCTrCH_Minimum_DL_Power_RL_SetupRspTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_CCTRCH_MAXIMUM_DL_POWER_RL_ADDITIONRSPTDD:					/*  502 */
		offset = dissect_id_CCTrCH_Maximum_DL_Power_RL_AdditionRspTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_CCTRCH_MINIMUM_DL_POWER_RL_ADDITIONRSPTDD:					/*  503 */
		offset = dissect_id_CCTrCH_Minimum_DL_Power_RL_AdditionRspTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_CCTRCH_MAXIMUM_DL_POWER_RL_RECONFREADYTDD:					/*  504 */
		offset = dissect_id_CCTrCH_Maximum_DL_Power_RL_ReconfReadyTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_CCTRCH_MINIMUM_DL_POWER_RL_RECONFREADYTDD:					/*  505 */
		offset = dissect_id_CCTrCH_Minimum_DL_Power_RL_ReconfReadyTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_MAXIMUM_DL_POWER_TIMESLOTLCR_INFORMATIONMODIFYITEM_RL_RECONFREADYTDD:	/*  506 */
		offset = dissect_id_Maximum_DL_Power_TimeslotLCR_InformationModifyItem_RL_ReconfReadyTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_MINIMUM_DL_POWER_TIMESLOTLCR_INFORMATIONMODIFYITEM_RL_RECONFREADYTDD:	/*  507 */
		offset = dissect_id_Minimum_DL_Power_TimeslotLCR_InformationModifyItem_RL_ReconfReadyTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_CCTRCH_INFORMATIONLIST_RL_RECONFRSPTDD:					/*  508 */
		offset = dissect_id_DL_CCTrCH_InformationList_RL_ReconfRspTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_DPCH_INFORMATIONMODIFYITEM_LCR_RL_RECONFRSPTDD:			/*  509 */
		offset = dissect_id_DL_DPCH_InformationModifyItem_LCR_RL_ReconfRspTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_MAXIMUM_DL_POWER_TIMESLOTLCR_INFORMATIONITEM:					/*  510 */
		offset = dissect_id_Maximum_DL_Power_TimeslotLCR_InformationItem(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_MINIMUM_DL_POWER_TIMESLOTLCR_INFORMATIONITEM:					/*  511 */
		offset = dissect_id_Minimum_DL_Power_TimeslotLCR_InformationItem(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_TDD_SUPPORT_8PSK:												/*  512 */
		offset = dissect_id_TDD_Support_8PSK(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_TDD_MAXNRDLPHYSICALCHANNELS:									/*  513 */
		offset = dissect_id_TDD_maxNrDLPhysicalchannels(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_EXTENDEDGSMCELLINDIVIDUALOFFSET:								/*  514 */
		offset = dissect_id_ExtendedGSMCellIndividualOffset(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_PARAMETERUPDATEINDICATIONFDD_RL_INFORMATIONLIST:			/*  518 */
		offset = dissect_id_RL_ParameterUpdateIndicationFDD_RL_InformationList(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_PRIMARY_CPICH_USAGE_FOR_CHANNEL_ESTIMATION:					/*  519 */
		offset = dissect_id_Primary_CPICH_Usage_For_Channel_Estimation(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_SECONDARY_CPICH_INFORMATION:									/*  520 */
		offset = dissect_id_Secondary_CPICH_Information(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_SECONDARY_CPICH_INFORMATION_CHANGE:							/*  521 */
		offset = dissect_id_Secondary_CPICH_Information_Change(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_522:								/*  522 */
		break;
	case RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_523:								/*  523 */
		break;
	case RNSAP_ID_RL_PARAMETERUPDATEINDICATIONFDD_RL_INFORMATION_ITEM:			/*  524 */
		offset = dissect_id_RL_ParameterUpdateIndicationFDD_RL_Information_Item(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_PHASE_REFERENCE_UPDATE_INDICATOR:								/*  525 */
		offset = dissect_id_Phase_Reference_Update_Indicator(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UNIDIRECTIONAL_DCH_INDICATOR:									/*  526 */
		offset = dissect_id_Unidirectional_DCH_Indicator(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_INFORMATION_RL_RECONFPREPTDD:								/*  527 */
		offset = dissect_id_RL_Information_RL_ReconfPrepTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_MULTIPLE_RL_INFORMATIONRESPONSE_RL_RECONFREADYTDD:			/*  528 */
		offset = dissect_id_Multiple_RL_InformationResponse_RL_ReconfReadyTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_RECONFIGURATIONRESPONSETDD_RL_INFORMATION:					/*  529 */
		offset = dissect_id_RL_ReconfigurationResponseTDD_RL_Information(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_SATELLITE_ALMANAC_INFORMATION_EXTITEM:						/*  530 */
		offset = dissect_id_Satellite_Almanac_Information_ExtItem(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_HSDSCH_INFORMATION_TO_MODIFY_UNSYNCHRONISED:					/*  533 */
		offset = dissect_id_HSDSCH_Information_to_Modify_Unsynchronised(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_TNLQOS:														/*  534 */
		offset = dissect_id_TnlQos(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RTLOADVALUE:													/*  535 */
		offset = dissect_id_RTLoadValue(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_NRTLOADINFORMATIONVALUE:										/*  536 */
		offset = dissect_id_NRTLoadInformationValue(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_CELLPORTIONID:												/*  537 */
		offset = dissect_id_CellPortionID(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UPPTSINTERFERENCEVALUE:										/*  538 */
		offset = dissect_id_UpPTSInterferenceValue(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_PRIMARYCCPCH_RSCP_DELTA:										/*  539 */
		offset = dissect_id_PrimaryCCPCH_RSCP_Delta(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UEMEASUREMENTTYPE:											/*  540 */
		offset = dissect_id_UEMeasurementType(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UEMEASUREMENTTIMESLOTINFOHCR:									/*  541 */
		offset = dissect_id_UEMeasurementTimeslotInfoHCR(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UEMEASUREMENTTIMESLOTINFOLCR:									/*  542 */
		offset = dissect_id_UEMeasurementTimeslotInfoLCR(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UEMEASUREMENTREPORTCHARACTERISTICS:							/*  543 */
		offset = dissect_id_UEMeasurementReportCharacteristics(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UEMEASUREMENTPARAMETERMODALLOW:								/*  544 */
		offset = dissect_id_UEMeasurementParameterModAllow(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UEMEASUREMENTVALUEINFORMATION:								/*  545 */
		offset = dissect_id_UEMeasurementValueInformation(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_INTERFACESTOTRACEITEM:										/*  546 */
		offset = dissect_id_InterfacesToTraceItem(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_LISTOFINTERFACESTOTRACE:										/*  547 */
		offset = dissect_id_ListOfInterfacesToTrace(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_TRACEDEPTH:													/*  548 */
		offset = dissect_id_TraceDepth(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_TRACERECORDINGSESSIONREFERENCE:								/*  549 */
		offset = dissect_id_TraceRecordingSessionReference(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_TRACEREFERENCE:												/*  550 */
		offset = dissect_id_TraceReference(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UEIDENTITY:													/*  551 */
		offset = dissect_id_UEIdentity(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_NACC_RELATED_DATA:											/*  552 */
		offset = dissect_id_NACC_Related_Data(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_GSM_CELL_INFEX_RQST:											/*  553 */
		offset = dissect_id_GSM_Cell_InfEx_Rqst(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_MEASUREMENTRECOVERYBEHAVIOR:									/*  554 */
		offset = dissect_id_MeasurementRecoveryBehavior(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_MEASUREMENTRECOVERYREPORTINGINDICATOR:						/*  555 */
		offset = dissect_id_MeasurementRecoveryReportingIndicator(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_MEASUREMENTRECOVERYSUPPORTINDICATOR:							/*  556 */
		offset = dissect_id_MeasurementRecoverySupportIndicator(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_DL_DPCH_POWER_INFORMATION_RL_RECONFPREPFDD:					/*  557 */
		offset = dissect_id_DL_DPCH_Power_Information_RL_ReconfPrepFDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_F_DPCH_INFORMATION_RL_RECONFPREPFDD:							/*  558 */
		offset = dissect_id_F_DPCH_Information_RL_ReconfPrepFDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_F_DPCH_INFORMATION_RL_SETUPRQSTFDD:							/*  559 */
		offset = dissect_id_F_DPCH_Information_RL_SetupRqstFDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_MBMS_BEARER_SERVICE_LIST:										/*  560 */
		offset = dissect_id_MBMS_Bearer_Service_List(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_MBMS_BEARER_SERVICE_LIST_INFEX_RSP:							/*  561 */
		offset = dissect_id_MBMS_Bearer_Service_List_InfEx_Rsp(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_ACTIVE_MBMS_BEARER_SERVICEFDD:								/*  562 */
		offset = dissect_id_Active_MBMS_Bearer_ServiceFDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_ACTIVE_MBMS_BEARER_SERVICETDD:								/*  563 */
		offset = dissect_id_Active_MBMS_Bearer_ServiceTDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_OLD_URA_ID:													/*  564 */
		offset = dissect_id_Old_URA_ID(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UE_STATE:														/*  568 */
		offset = dissect_id_UE_State(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_URA_ID:														/*  569 */
		offset = dissect_id_URA_ID(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_HARQ_PREAMBLE_MODE:											/*  571 */
		offset = dissect_id_HARQ_Preamble_Mode(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_SYNCHRONISATIONINDICATOR:										/*  572 */
		offset = dissect_id_SynchronisationIndicator(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_UL_DPDCHINDICATOREDCH:										/*  573 */
		offset = dissect_id_UL_DPDCHIndicatorEDCH(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_EDPCH_INFORMATION:											/*  574 */
		offset = dissect_id_EDPCH_Information(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_RL_SPECIFIC_EDCH_INFORMATION:									/*  575 */
		offset = dissect_id_RL_Specific_EDCH_Information(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_EDCH_RL_INDICATION:											/*  576 */
		offset = dissect_id_EDCH_RL_Indication(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_EDCH_FDD_INFORMATION:											/*  577 */
		offset = dissect_id_EDCH_FDD_Information(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_EDCH_RLSET_ID:												/*  578 */
		offset = dissect_id_EDCH_RLSet_Id(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_SERVING_EDCHRL_ID:											/*  579 */
		offset = dissect_id_Serving_EDCHRL_Id(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_EDCH_FDD_DL_CONTROLCHANNELINFORMATION:						/*  580 */
		offset = dissect_id_EDCH_FDD_DL_ControlChannelInformation(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_EDCH_FDD_INFORMATIONRESPONSE:									/*  581 */
		offset = dissect_id_EDCH_FDD_InformationResponse(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_EDCH_MACDFLOWS_TO_ADD:										/*  582 */
		offset = dissect_id_EDCH_MACdFlows_To_Add(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_EDCH_FDD_INFORMATION_TO_MODIFY:								/*  583 */
		offset = dissect_id_EDCH_FDD_Information_To_Modify(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_EDCH_MACDFLOWS_TO_DELETE:										/*  584 */
		offset = dissect_id_EDCH_MACdFlows_To_Delete(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_EDPCH_INFORMATION_RLRECONFREQUEST_FDD:						/*  585 */
		offset = dissect_id_EDPCH_Information_RLReconfRequest_FDD(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_EDCH_MACDFLOWSPECIFICINFORMATIONLIST_RL_PREEMPTREQUIREDIND:	/*  586 */
		offset = dissect_id_EDCH_MacdFlowSpecificInformationList_RL_PreemptRequiredInd(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_EDCH_MACDFLOWSPECIFICINFORMATIONITEM_RL_PREEMPTREQUIREDIND:	/*  587 */
		offset = dissect_id_EDCH_MacdFlowSpecificInformationItem_RL_PreemptRequiredInd(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_EDCH_MACDFLOWSPECIFICINFORMATIONLIST_RL_CONGESTIND:			/*  588 */
		offset = dissect_id_EDCH_MacdFlowSpecificInformationList_RL_CongestInd(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_EDCH_MACDFLOWSPECIFICINFORMATIONITEM_RL_CONGESTIND:			/*  589 */
		offset = dissect_id_EDCH_MacdFlowSpecificInformationItem_RL_CongestInd(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_MBMS_BEARER_SERVICE_FULL_ADDRESS:								/*  590 */
		offset = dissect_id_MBMS_Bearer_Service_Full_Address(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_INITIAL_DL_DPCH_TIMINGADJUSTMENT:								/*  591 */
		offset = dissect_id_Initial_DL_DPCH_TimingAdjustment(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_INITIAL_DL_DPCH_TIMINGADJUSTMENT_ALLOWED:						/*  592 */
		offset = dissect_id_Initial_DL_DPCH_TimingAdjustment_Allowed(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_USER_PLANE_CONGESTION_FIELDS_INCLUSION:						/*  593 */
		offset = dissect_id_User_Plane_Congestion_Fields_Inclusion(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_HARQ_PREAMBLE_MODE_ACTIVATION_INDICATOR:						/*  594 */
		offset = dissect_id_HARQ_Preamble_Mode_Activation_Indicator(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_MULTIPLE_DEDICATEDMEASUREMENTVALUELIST_TDD_DM_RSP:			/*  595 */
		offset = dissect_id_multiple_DedicatedMeasurementValueList_TDD_DM_Rsp(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_MULTIPLE_DEDICATEDMEASUREMENTVALUELIST_LCR_TDD_DM_RSP:		/*  596 */
		offset = dissect_id_multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_PROVIDEDINFORMATION:											/*  597 */
		offset = dissect_id_ProvidedInformation(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_ACTIVE_MBMS_BEARER_SERVICEFDD_PFL:							/*  598 */
		offset = dissect_id_Active_MBMS_Bearer_ServiceFDD_PFL(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_ACTIVE_MBMS_BEARER_SERVICETDD_PFL:							/*  599 */
		offset = dissect_id_Active_MBMS_Bearer_ServiceTDD_PFL(tvb, offset, actx, value_tree);
		break;
	case RNSAP_ID_FREQUENCYBANDINDICATOR:										/*  600	*/
		offset = dissect_id_FrequencyBandIndicator(tvb, offset, actx, value_tree);
		break;
	default:
		offset = offset + (length<<3);
		break;
	}
	BYTE_ALIGN_OFFSET(offset);	
	return offset;
}


static void
dissect_rnsap(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
{
	proto_item	*rnsap_item = NULL;
	proto_tree	*rnsap_tree = NULL;

	top_tree = tree;

	/* make entry in the Protocol column on summary display */
	if (check_col(pinfo->cinfo, COL_PROTOCOL))
		col_set_str(pinfo->cinfo, COL_PROTOCOL, "RNSAP");

	/* create the rnsap protocol tree */
	rnsap_item = proto_tree_add_item(tree, proto_rnsap, tvb, 0, -1, FALSE);
	rnsap_tree = proto_item_add_subtree(rnsap_item, ett_rnsap);
	
	dissect_RNSAP_PDU_PDU(tvb, pinfo, rnsap_tree);
}

#if 0
static gboolean
dissect_sccp_rnsap_heur(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
{
    guint8 temp;

    dissect_rnsap(tvb, pinfo, tree);

    /*
     * Is it a rnsap packet?
     */
    return TRUE;
}
#endif

/*--- proto_register_rnsap -------------------------------------------*/
void proto_register_rnsap(void) {

  /* List of fields */

  static hf_register_info hf[] = {
	{ &hf_rnsap_pdu_length,
		{ "PDU Length", "rnsap.pdu_length", FT_UINT32, BASE_DEC,
		NULL, 0, "Number of octets in the PDU", HFILL }},
	{ &hf_rnsap_IE_length,
		{ "IE Length", "rnsap.ie_length", FT_UINT32, BASE_DEC,
		NULL, 0, "Number of octets in the IE", HFILL }},
    { &hf_rnsap_L3_DL_DCCH_Message_PDU,
      { "DL-DCCH-Message", "rnsap.DL_DCCH_Message",
        FT_NONE, BASE_NONE, NULL, 0,
        "DL-DCCH-Message", HFILL }},


/*--- Included file: packet-rnsap-hfarr.c ---*/
#line 1 "packet-rnsap-hfarr.c"
    { &hf_rnsap_RNSAP_PDU_PDU,
      { "RNSAP-PDU", "rnsap.RNSAP_PDU",
        FT_UINT32, BASE_DEC, VALS(rnsap_RNSAP_PDU_vals), 0,
        "rnsap.RNSAP_PDU", HFILL }},
    { &hf_rnsap_initiatingMessage,
      { "initiatingMessage", "rnsap.initiatingMessage",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.InitiatingMessage", HFILL }},
    { &hf_rnsap_successfulOutcome,
      { "successfulOutcome", "rnsap.successfulOutcome",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.SuccessfulOutcome", HFILL }},
    { &hf_rnsap_unsuccessfulOutcome,
      { "unsuccessfulOutcome", "rnsap.unsuccessfulOutcome",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UnsuccessfulOutcome", HFILL }},
    { &hf_rnsap_outcome,
      { "outcome", "rnsap.outcome",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Outcome", HFILL }},
    { &hf_rnsap_procedureID,
      { "procedureID", "rnsap.procedureID",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProcedureID", HFILL }},
    { &hf_rnsap_criticality,
      { "criticality", "rnsap.criticality",
        FT_UINT32, BASE_DEC, VALS(rnsap_Criticality_vals), 0,
        "rnsap.Criticality", HFILL }},
    { &hf_rnsap_transactionID,
      { "transactionID", "rnsap.transactionID",
        FT_UINT32, BASE_DEC, VALS(rnsap_TransactionID_vals), 0,
        "rnsap.TransactionID", HFILL }},
    { &hf_rnsap_initiatingMessageValue,
      { "initiatingMessageValue", "rnsap.initiatingMessageValue",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.InitiatingMessageValue", HFILL }},
    { &hf_rnsap_successfulOutcomeValue,
      { "successfulOutcomeValue", "rnsap.successfulOutcomeValue",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.SuccessfulOutcomeValue", HFILL }},
    { &hf_rnsap_unsuccessfulOutcomeValue,
      { "unsuccessfulOutcomeValue", "rnsap.unsuccessfulOutcomeValue",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UnsuccessfulOutcomeValue", HFILL }},
    { &hf_rnsap_outcomeValue,
      { "outcomeValue", "rnsap.outcomeValue",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.OutcomeValue", HFILL }},
    { &hf_rnsap_id_commonTransportChannelResourcesInitialisation,
      { "id-commonTransportChannelResourcesInitialisation", "rnsap.id_commonTransportChannelResourcesInitialisation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.CommonTransportChannelResourcesRequest", HFILL }},
    { &hf_rnsap_id_commonTransportChannelResourcesRelease,
      { "id-commonTransportChannelResourcesRelease", "rnsap.id_commonTransportChannelResourcesRelease",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.CommonTransportChannelResourcesReleaseRequest", HFILL }},
    { &hf_rnsap_id_compressedModeCommand,
      { "id-compressedModeCommand", "rnsap.id_compressedModeCommand",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.CompressedModeCommand", HFILL }},
    { &hf_rnsap_id_downlinkPowerControl,
      { "id-downlinkPowerControl", "rnsap.id_downlinkPowerControl",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_PowerControlRequest", HFILL }},
    { &hf_rnsap_id_downlinkPowerTimeslotControl,
      { "id-downlinkPowerTimeslotControl", "rnsap.id_downlinkPowerTimeslotControl",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_PowerTimeslotControlRequest", HFILL }},
    { &hf_rnsap_id_downlinkSignallingTransfer,
      { "id-downlinkSignallingTransfer", "rnsap.id_downlinkSignallingTransfer",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DownlinkSignallingTransferRequest", HFILL }},
    { &hf_rnsap_id_errorIndication,
      { "id-errorIndication", "rnsap.id_errorIndication",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ErrorIndication", HFILL }},
    { &hf_rnsap_id_dedicatedMeasurementFailure,
      { "id-dedicatedMeasurementFailure", "rnsap.id_dedicatedMeasurementFailure",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DedicatedMeasurementFailureIndication", HFILL }},
    { &hf_rnsap_id_dedicatedMeasurementInitiation,
      { "id-dedicatedMeasurementInitiation", "rnsap.id_dedicatedMeasurementInitiation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DedicatedMeasurementInitiationRequest", HFILL }},
    { &hf_rnsap_id_dedicatedMeasurementReporting,
      { "id-dedicatedMeasurementReporting", "rnsap.id_dedicatedMeasurementReporting",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DedicatedMeasurementReport", HFILL }},
    { &hf_rnsap_id_dedicatedMeasurementTermination,
      { "id-dedicatedMeasurementTermination", "rnsap.id_dedicatedMeasurementTermination",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DedicatedMeasurementTerminationRequest", HFILL }},
    { &hf_rnsap_id_paging,
      { "id-paging", "rnsap.id_paging",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.PagingRequest", HFILL }},
    { &hf_rnsap_id_physicalChannelReconfiguration,
      { "id-physicalChannelReconfiguration", "rnsap.id_physicalChannelReconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.PhysicalChannelReconfigurationRequestTDD", HFILL }},
    { &hf_rnsap_id_privateMessage,
      { "id-privateMessage", "rnsap.id_privateMessage",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.PrivateMessage", HFILL }},
    { &hf_rnsap_id_radioLinkAddition,
      { "id-radioLinkAddition", "rnsap.id_radioLinkAddition",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RadioLinkAdditionRequestFDD", HFILL }},
    { &hf_rnsap_id_radioLinkAddition_TDD,
      { "id-radioLinkAddition-TDD", "rnsap.id_radioLinkAddition_TDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RadioLinkAdditionRequestTDD", HFILL }},
    { &hf_rnsap_id_radioLinkCongestion,
      { "id-radioLinkCongestion", "rnsap.id_radioLinkCongestion",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RadioLinkCongestionIndication", HFILL }},
    { &hf_rnsap_id_radioLinkDeletion,
      { "id-radioLinkDeletion", "rnsap.id_radioLinkDeletion",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RadioLinkDeletionRequest", HFILL }},
    { &hf_rnsap_id_radioLinkFailure,
      { "id-radioLinkFailure", "rnsap.id_radioLinkFailure",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RadioLinkFailureIndication", HFILL }},
    { &hf_rnsap_id_radioLinkPreemption,
      { "id-radioLinkPreemption", "rnsap.id_radioLinkPreemption",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RadioLinkPreemptionRequiredIndication", HFILL }},
    { &hf_rnsap_id_radioLinkRestoration,
      { "id-radioLinkRestoration", "rnsap.id_radioLinkRestoration",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RadioLinkRestoreIndication", HFILL }},
    { &hf_rnsap_id_radioLinkSetup,
      { "id-radioLinkSetup", "rnsap.id_radioLinkSetup",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RadioLinkSetupRequestFDD", HFILL }},
    { &hf_rnsap_id_radioLinkSetupTdd,
      { "id-radioLinkSetupTdd", "rnsap.id_radioLinkSetupTdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RadioLinkSetupRequestTDD", HFILL }},
    { &hf_rnsap_id_relocationCommit,
      { "id-relocationCommit", "rnsap.id_relocationCommit",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RelocationCommit", HFILL }},
    { &hf_rnsap_id_synchronisedRadioLinkReconfigurationCancellation,
      { "id-synchronisedRadioLinkReconfigurationCancellation", "rnsap.id_synchronisedRadioLinkReconfigurationCancellation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RadioLinkReconfigurationCancel", HFILL }},
    { &hf_rnsap_id_synchronisedRadioLinkReconfigurationCommit,
      { "id-synchronisedRadioLinkReconfigurationCommit", "rnsap.id_synchronisedRadioLinkReconfigurationCommit",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RadioLinkReconfigurationCommit", HFILL }},
    { &hf_rnsap_id_synchronisedRadioLinkReconfigurationPreparation,
      { "id-synchronisedRadioLinkReconfigurationPreparation", "rnsap.id_synchronisedRadioLinkReconfigurationPreparation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RadioLinkReconfigurationPrepareFDD", HFILL }},
    { &hf_rnsap_id_synchronisedRadioLinkReconfigurationPreparation_TDD,
      { "id-synchronisedRadioLinkReconfigurationPreparation-TDD", "rnsap.id_synchronisedRadioLinkReconfigurationPreparation_TDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RadioLinkReconfigurationReadyTDD", HFILL }},
    { &hf_rnsap_id_unSynchronisedRadioLinkReconfiguration,
      { "id-unSynchronisedRadioLinkReconfiguration", "rnsap.id_unSynchronisedRadioLinkReconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RadioLinkReconfigurationRequestFDD", HFILL }},
    { &hf_rnsap_id_unSynchronisedRadioLinkReconfiguration_TDD,
      { "id-unSynchronisedRadioLinkReconfiguration-TDD", "rnsap.id_unSynchronisedRadioLinkReconfiguration_TDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RadioLinkReconfigurationRequestTDD", HFILL }},
    { &hf_rnsap_id_uplinkSignallingTransfer,
      { "id-uplinkSignallingTransfer", "rnsap.id_uplinkSignallingTransfer",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UplinkSignallingTransferIndicationFDD", HFILL }},
    { &hf_rnsap_id_uplinkSignallingTransfer_TDD,
      { "id-uplinkSignallingTransfer-TDD", "rnsap.id_uplinkSignallingTransfer_TDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UplinkSignallingTransferIndicationTDD", HFILL }},
    { &hf_rnsap_id_commonMeasurementFailure,
      { "id-commonMeasurementFailure", "rnsap.id_commonMeasurementFailure",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.CommonMeasurementFailureIndication", HFILL }},
    { &hf_rnsap_id_commonMeasurementInitiation,
      { "id-commonMeasurementInitiation", "rnsap.id_commonMeasurementInitiation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.CommonMeasurementInitiationRequest", HFILL }},
    { &hf_rnsap_id_commonMeasurementReporting,
      { "id-commonMeasurementReporting", "rnsap.id_commonMeasurementReporting",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.CommonMeasurementReport", HFILL }},
    { &hf_rnsap_id_commonMeasurementTermination,
      { "id-commonMeasurementTermination", "rnsap.id_commonMeasurementTermination",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.CommonMeasurementTerminationRequest", HFILL }},
    { &hf_rnsap_id_informationExchangeFailure,
      { "id-informationExchangeFailure", "rnsap.id_informationExchangeFailure",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.InformationExchangeFailureIndication", HFILL }},
    { &hf_rnsap_id_informationExchangeInitiation,
      { "id-informationExchangeInitiation", "rnsap.id_informationExchangeInitiation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.InformationExchangeInitiationRequest", HFILL }},
    { &hf_rnsap_id_informationReporting,
      { "id-informationReporting", "rnsap.id_informationReporting",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.InformationReport", HFILL }},
    { &hf_rnsap_id_informationExchangeTermination,
      { "id-informationExchangeTermination", "rnsap.id_informationExchangeTermination",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.InformationExchangeTerminationRequest", HFILL }},
    { &hf_rnsap_id_reset,
      { "id-reset", "rnsap.id_reset",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ResetRequest", HFILL }},
    { &hf_rnsap_id_radioLinkActivation,
      { "id-radioLinkActivation", "rnsap.id_radioLinkActivation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RadioLinkActivationCommandFDD", HFILL }},
    { &hf_rnsap_id_radioLinkActivation_TDD,
      { "id-radioLinkActivation-TDD", "rnsap.id_radioLinkActivation_TDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RadioLinkActivationCommandTDD", HFILL }},
    { &hf_rnsap_id_gERANuplinkSignallingTransfer,
      { "id-gERANuplinkSignallingTransfer", "rnsap.id_gERANuplinkSignallingTransfer",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.GERANUplinkSignallingTransferIndication", HFILL }},
    { &hf_rnsap_id_radioLinkParameterUpdate,
      { "id-radioLinkParameterUpdate", "rnsap.id_radioLinkParameterUpdate",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RadioLinkParameterUpdateIndicationFDD", HFILL }},
    { &hf_rnsap_id_radioLinkParameterUpdate_TDD,
      { "id-radioLinkParameterUpdate-TDD", "rnsap.id_radioLinkParameterUpdate_TDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RadioLinkParameterUpdateIndicationTDD", HFILL }},
    { &hf_rnsap_id_uEMeasurementFailure,
      { "id-uEMeasurementFailure", "rnsap.id_uEMeasurementFailure",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UEMeasurementFailureIndication", HFILL }},
    { &hf_rnsap_id_uEMeasurementInitiation,
      { "id-uEMeasurementInitiation", "rnsap.id_uEMeasurementInitiation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UEMeasurementInitiationRequest", HFILL }},
    { &hf_rnsap_id_uEMeasurementReporting,
      { "id-uEMeasurementReporting", "rnsap.id_uEMeasurementReporting",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UEMeasurementReport", HFILL }},
    { &hf_rnsap_id_uEMeasurementTermination,
      { "id-uEMeasurementTermination", "rnsap.id_uEMeasurementTermination",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UEMeasurementTerminationRequest", HFILL }},
    { &hf_rnsap_id_iurDeactivateTrace,
      { "id-iurDeactivateTrace", "rnsap.id_iurDeactivateTrace",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.IurDeactivateTrace", HFILL }},
    { &hf_rnsap_id_iurInvokeTrace,
      { "id-iurInvokeTrace", "rnsap.id_iurInvokeTrace",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.IurInvokeTrace", HFILL }},
    { &hf_rnsap_id_mBMSAttach,
      { "id-mBMSAttach", "rnsap.id_mBMSAttach",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.MBMSAttachCommand", HFILL }},
    { &hf_rnsap_id_mBMSDetach,
      { "id-mBMSDetach", "rnsap.id_mBMSDetach",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.MBMSDetachCommand", HFILL }},
    { &hf_rnsap_id_directInformationTransfer,
      { "id-directInformationTransfer", "rnsap.id_directInformationTransfer",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DirectInformationTransfer", HFILL }},
    { &hf_rnsap_id_commonTransportChannelResourcesInitialisation1,
      { "id-commonTransportChannelResourcesInitialisation", "rnsap.id_commonTransportChannelResourcesInitialisation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.CommonTransportChannelResourcesResponseFDD", HFILL }},
    { &hf_rnsap_id_commonTransportChannelResourcesInitialisation_TDD,
      { "id-commonTransportChannelResourcesInitialisation-TDD", "rnsap.id_commonTransportChannelResourcesInitialisation_TDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.CommonTransportChannelResourcesResponseTDD", HFILL }},
    { &hf_rnsap_id_dedicatedMeasurementInitiation1,
      { "id-dedicatedMeasurementInitiation", "rnsap.id_dedicatedMeasurementInitiation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DedicatedMeasurementInitiationResponse", HFILL }},
    { &hf_rnsap_id_physicalChannelReconfiguration1,
      { "id-physicalChannelReconfiguration", "rnsap.id_physicalChannelReconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.PhysicalChannelReconfigurationCommand", HFILL }},
    { &hf_rnsap_id_radioLinkAddition1,
      { "id-radioLinkAddition", "rnsap.id_radioLinkAddition",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RadioLinkAdditionResponseFDD", HFILL }},
    { &hf_rnsap_id_radioLinkAddition_TDD1,
      { "id-radioLinkAddition-TDD", "rnsap.id_radioLinkAddition_TDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RadioLinkAdditionResponseTDD", HFILL }},
    { &hf_rnsap_id_radioLinkDeletion1,
      { "id-radioLinkDeletion", "rnsap.id_radioLinkDeletion",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RadioLinkDeletionResponse", HFILL }},
    { &hf_rnsap_id_radioLinkSetup1,
      { "id-radioLinkSetup", "rnsap.id_radioLinkSetup",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RadioLinkSetupResponseFDD", HFILL }},
    { &hf_rnsap_id_radioLinkSetupTdd1,
      { "id-radioLinkSetupTdd", "rnsap.id_radioLinkSetupTdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RadioLinkSetupResponseTDD", HFILL }},
    { &hf_rnsap_id_synchronisedRadioLinkReconfigurationPreparation1,
      { "id-synchronisedRadioLinkReconfigurationPreparation", "rnsap.id_synchronisedRadioLinkReconfigurationPreparation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RadioLinkReconfigurationReadyFDD", HFILL }},
    { &hf_rnsap_id_unSynchronisedRadioLinkReconfiguration1,
      { "id-unSynchronisedRadioLinkReconfiguration", "rnsap.id_unSynchronisedRadioLinkReconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RadioLinkReconfigurationResponseFDD", HFILL }},
    { &hf_rnsap_id_unSynchronisedRadioLinkReconfiguration_TDD1,
      { "id-unSynchronisedRadioLinkReconfiguration-TDD", "rnsap.id_unSynchronisedRadioLinkReconfiguration_TDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RadioLinkReconfigurationResponseTDD", HFILL }},
    { &hf_rnsap_id_commonMeasurementInitiation1,
      { "id-commonMeasurementInitiation", "rnsap.id_commonMeasurementInitiation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.CommonMeasurementInitiationResponse", HFILL }},
    { &hf_rnsap_id_informationExchangeInitiation1,
      { "id-informationExchangeInitiation", "rnsap.id_informationExchangeInitiation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.InformationExchangeInitiationResponse", HFILL }},
    { &hf_rnsap_id_reset1,
      { "id-reset", "rnsap.id_reset",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ResetResponse", HFILL }},
    { &hf_rnsap_id_uEMeasurementInitiation1,
      { "id-uEMeasurementInitiation", "rnsap.id_uEMeasurementInitiation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UEMeasurementInitiationResponse", HFILL }},
    { &hf_rnsap_id_commonTransportChannelResourcesInitialisation2,
      { "id-commonTransportChannelResourcesInitialisation", "rnsap.id_commonTransportChannelResourcesInitialisation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.CommonTransportChannelResourcesFailure", HFILL }},
    { &hf_rnsap_id_dedicatedMeasurementInitiation2,
      { "id-dedicatedMeasurementInitiation", "rnsap.id_dedicatedMeasurementInitiation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DedicatedMeasurementInitiationFailure", HFILL }},
    { &hf_rnsap_id_physicalChannelReconfiguration2,
      { "id-physicalChannelReconfiguration", "rnsap.id_physicalChannelReconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.PhysicalChannelReconfigurationFailure", HFILL }},
    { &hf_rnsap_id_radioLinkAddition2,
      { "id-radioLinkAddition", "rnsap.id_radioLinkAddition",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RadioLinkAdditionFailureFDD", HFILL }},
    { &hf_rnsap_id_radioLinkAddition_TDD2,
      { "id-radioLinkAddition-TDD", "rnsap.id_radioLinkAddition_TDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RadioLinkAdditionFailureTDD", HFILL }},
    { &hf_rnsap_id_radioLinkSetup2,
      { "id-radioLinkSetup", "rnsap.id_radioLinkSetup",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RadioLinkSetupFailureFDD", HFILL }},
    { &hf_rnsap_id_radioLinkSetupTdd2,
      { "id-radioLinkSetupTdd", "rnsap.id_radioLinkSetupTdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RadioLinkSetupFailureTDD", HFILL }},
    { &hf_rnsap_id_synchronisedRadioLinkReconfigurationPreparation2,
      { "id-synchronisedRadioLinkReconfigurationPreparation", "rnsap.id_synchronisedRadioLinkReconfigurationPreparation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RadioLinkReconfigurationFailure", HFILL }},
    { &hf_rnsap_id_unSynchronisedRadioLinkReconfiguration2,
      { "id-unSynchronisedRadioLinkReconfiguration", "rnsap.id_unSynchronisedRadioLinkReconfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RadioLinkReconfigurationFailure", HFILL }},
    { &hf_rnsap_id_unSynchronisedRadioLinkReconfiguration_TDD2,
      { "id-unSynchronisedRadioLinkReconfiguration-TDD", "rnsap.id_unSynchronisedRadioLinkReconfiguration_TDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RadioLinkReconfigurationFailure", HFILL }},
    { &hf_rnsap_id_commonMeasurementInitiation2,
      { "id-commonMeasurementInitiation", "rnsap.id_commonMeasurementInitiation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.CommonMeasurementInitiationFailure", HFILL }},
    { &hf_rnsap_id_informationExchangeInitiation2,
      { "id-informationExchangeInitiation", "rnsap.id_informationExchangeInitiation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.InformationExchangeInitiationFailure", HFILL }},
    { &hf_rnsap_id_uEMeasurementInitiation2,
      { "id-uEMeasurementInitiation", "rnsap.id_uEMeasurementInitiation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UEMeasurementInitiationFailure", HFILL }},
    { &hf_rnsap_id_AllowedQueuingTime,
      { "id-AllowedQueuingTime", "rnsap.id_AllowedQueuingTime",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.AllowedQueuingTime", HFILL }},
    { &hf_rnsap_id_Allowed_Rate_Information,
      { "id-Allowed-Rate-Information", "rnsap.id_Allowed_Rate_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Allowed_Rate_Information", HFILL }},
    { &hf_rnsap_id_AntennaColocationIndicator,
      { "id-AntennaColocationIndicator", "rnsap.id_AntennaColocationIndicator",
        FT_UINT32, BASE_DEC, VALS(rnsap_AntennaColocationIndicator_vals), 0,
        "rnsap.AntennaColocationIndicator", HFILL }},
    { &hf_rnsap_id_BindingID,
      { "id-BindingID", "rnsap.id_BindingID",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BindingID", HFILL }},
    { &hf_rnsap_id_C_ID,
      { "id-C-ID", "rnsap.id_C_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.C_ID", HFILL }},
    { &hf_rnsap_id_C_RNTI,
      { "id-C-RNTI", "rnsap.id_C_RNTI",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.C_RNTI", HFILL }},
    { &hf_rnsap_id_Cell_Capacity_Class_Value,
      { "id-Cell-Capacity-Class-Value", "rnsap.id_Cell_Capacity_Class_Value",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Cell_Capacity_Class_Value", HFILL }},
    { &hf_rnsap_id_CFN,
      { "id-CFN", "rnsap.id_CFN",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.CFN", HFILL }},
    { &hf_rnsap_id_CN_CS_DomainIdentifier,
      { "id-CN-CS-DomainIdentifier", "rnsap.id_CN_CS_DomainIdentifier",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.CN_CS_DomainIdentifier", HFILL }},
    { &hf_rnsap_id_CN_PS_DomainIdentifier,
      { "id-CN-PS-DomainIdentifier", "rnsap.id_CN_PS_DomainIdentifier",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.CN_PS_DomainIdentifier", HFILL }},
    { &hf_rnsap_id_Cause,
      { "id-Cause", "rnsap.id_Cause",
        FT_UINT32, BASE_DEC, VALS(rnsap_Cause_vals), 0,
        "rnsap.Cause", HFILL }},
    { &hf_rnsap_id_CoverageIndicator,
      { "id-CoverageIndicator", "rnsap.id_CoverageIndicator",
        FT_UINT32, BASE_DEC, VALS(rnsap_CoverageIndicator_vals), 0,
        "rnsap.CoverageIndicator", HFILL }},
    { &hf_rnsap_id_CriticalityDiagnostics,
      { "id-CriticalityDiagnostics", "rnsap.id_CriticalityDiagnostics",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.CriticalityDiagnostics", HFILL }},
    { &hf_rnsap_id_ContextInfoItem_Reset,
      { "id-ContextInfoItem-Reset", "rnsap.id_ContextInfoItem_Reset",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ContextInfoItem_Reset", HFILL }},
    { &hf_rnsap_id_ContextGroupInfoItem_Reset,
      { "id-ContextGroupInfoItem-Reset", "rnsap.id_ContextGroupInfoItem_Reset",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ContextGroupInfoItem_Reset", HFILL }},
    { &hf_rnsap_id_D_RNTI,
      { "id-D-RNTI", "rnsap.id_D_RNTI",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.D_RNTI", HFILL }},
    { &hf_rnsap_id_D_RNTI_ReleaseIndication,
      { "id-D-RNTI-ReleaseIndication", "rnsap.id_D_RNTI_ReleaseIndication",
        FT_UINT32, BASE_DEC, VALS(rnsap_D_RNTI_ReleaseIndication_vals), 0,
        "rnsap.D_RNTI_ReleaseIndication", HFILL }},
    { &hf_rnsap_id_DCHs_to_Add_FDD,
      { "id-DCHs-to-Add-FDD", "rnsap.id_DCHs_to_Add_FDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DCH_FDD_Information", HFILL }},
    { &hf_rnsap_id_DCHs_to_Add_TDD,
      { "id-DCHs-to-Add-TDD", "rnsap.id_DCHs_to_Add_TDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DCH_TDD_Information", HFILL }},
    { &hf_rnsap_id_DCH_DeleteList_RL_ReconfPrepFDD,
      { "id-DCH-DeleteList-RL-ReconfPrepFDD", "rnsap.id_DCH_DeleteList_RL_ReconfPrepFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DCH_DeleteList_RL_ReconfPrepFDD", HFILL }},
    { &hf_rnsap_id_DCH_DeleteList_RL_ReconfPrepTDD,
      { "id-DCH-DeleteList-RL-ReconfPrepTDD", "rnsap.id_DCH_DeleteList_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DCH_DeleteList_RL_ReconfPrepTDD", HFILL }},
    { &hf_rnsap_id_DCH_DeleteList_RL_ReconfRqstFDD,
      { "id-DCH-DeleteList-RL-ReconfRqstFDD", "rnsap.id_DCH_DeleteList_RL_ReconfRqstFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DCH_DeleteList_RL_ReconfRqstFDD", HFILL }},
    { &hf_rnsap_id_DCH_DeleteList_RL_ReconfRqstTDD,
      { "id-DCH-DeleteList-RL-ReconfRqstTDD", "rnsap.id_DCH_DeleteList_RL_ReconfRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DCH_DeleteList_RL_ReconfRqstTDD", HFILL }},
    { &hf_rnsap_id_DCH_FDD_Information,
      { "id-DCH-FDD-Information", "rnsap.id_DCH_FDD_Information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DCH_FDD_Information", HFILL }},
    { &hf_rnsap_id_DCH_TDD_Information,
      { "id-DCH-TDD-Information", "rnsap.id_DCH_TDD_Information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DCH_TDD_Information", HFILL }},
    { &hf_rnsap_id_FDD_DCHs_to_Modify,
      { "id-FDD-DCHs-to-Modify", "rnsap.id_FDD_DCHs_to_Modify",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.FDD_DCHs_to_Modify", HFILL }},
    { &hf_rnsap_id_TDD_DCHs_to_Modify,
      { "id-TDD-DCHs-to-Modify", "rnsap.id_TDD_DCHs_to_Modify",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.TDD_DCHs_to_Modify", HFILL }},
    { &hf_rnsap_id_DCH_InformationResponse,
      { "id-DCH-InformationResponse", "rnsap.id_DCH_InformationResponse",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DCH_InformationResponse", HFILL }},
    { &hf_rnsap_id_DCH_Rate_InformationItem_RL_CongestInd,
      { "id-DCH-Rate-InformationItem-RL-CongestInd", "rnsap.id_DCH_Rate_InformationItem_RL_CongestInd",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DCH_Rate_InformationItem_RL_CongestInd", HFILL }},
    { &hf_rnsap_id_DL_CCTrCH_InformationAddItem_RL_ReconfPrepTDD,
      { "id-DL-CCTrCH-InformationAddItem-RL-ReconfPrepTDD", "rnsap.id_DL_CCTrCH_InformationAddItem_RL_ReconfPrepTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_CCTrCH_InformationAddItem_RL_ReconfPrepTDD", HFILL }},
    { &hf_rnsap_id_DL_CCTrCH_InformationListIE_RL_ReconfReadyTDD,
      { "id-DL-CCTrCH-InformationListIE-RL-ReconfReadyTDD", "rnsap.id_DL_CCTrCH_InformationListIE_RL_ReconfReadyTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DL_CCTrCHInformationListIE_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_id_DL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD,
      { "id-DL-CCTrCH-InformationDeleteItem-RL-ReconfRqstTDD", "rnsap.id_DL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD", HFILL }},
    { &hf_rnsap_id_DL_CCTrCH_InformationItem_RL_SetupRqstTDD,
      { "id-DL-CCTrCH-InformationItem-RL-SetupRqstTDD", "rnsap.id_DL_CCTrCH_InformationItem_RL_SetupRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_CCTrCH_InformationItem_RL_SetupRqstTDD", HFILL }},
    { &hf_rnsap_id_DL_CCTrCH_InformationListIE_PhyChReconfRqstTDD,
      { "id-DL-CCTrCH-InformationListIE-PhyChReconfRqstTDD", "rnsap.id_DL_CCTrCH_InformationListIE_PhyChReconfRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DL_CCTrCH_InformationListIE_PhyChReconfRqstTDD", HFILL }},
    { &hf_rnsap_id_DL_CCTrCH_InformationListIE_RL_AdditionRspTDD,
      { "id-DL-CCTrCH-InformationListIE-RL-AdditionRspTDD", "rnsap.id_DL_CCTrCH_InformationListIE_RL_AdditionRspTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DL_CCTrCHInformationListIE_RL_AdditionRspTDD", HFILL }},
    { &hf_rnsap_id_DL_CCTrCH_InformationListIE_RL_SetupRspTDD,
      { "id-DL-CCTrCH-InformationListIE-RL-SetupRspTDD", "rnsap.id_DL_CCTrCH_InformationListIE_RL_SetupRspTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_CCTrCH_InformationItem_RL_SetupRqstTDD", HFILL }},
    { &hf_rnsap_id_DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD,
      { "id-DL-CCTrCH-InformationAddList-RL-ReconfPrepTDD", "rnsap.id_DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD", HFILL }},
    { &hf_rnsap_id_DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD,
      { "id-DL-CCTrCH-InformationDeleteList-RL-ReconfRqstTDD", "rnsap.id_DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD", HFILL }},
    { &hf_rnsap_id_DL_CCTrCH_InformationList_RL_SetupRqstTDD,
      { "id-DL-CCTrCH-InformationList-RL-SetupRqstTDD", "rnsap.id_DL_CCTrCH_InformationList_RL_SetupRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DL_CCTrCH_InformationList_RL_SetupRqstTDD", HFILL }},
    { &hf_rnsap_id_FDD_DL_CodeInformation,
      { "id-FDD-DL-CodeInformation", "rnsap.id_FDD_DL_CodeInformation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.FDD_DL_CodeInformation", HFILL }},
    { &hf_rnsap_id_DL_DPCH_Information_RL_ReconfPrepFDD,
      { "id-DL-DPCH-Information-RL-ReconfPrepFDD", "rnsap.id_DL_DPCH_Information_RL_ReconfPrepFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_DPCH_Information_RL_ReconfPrepFDD", HFILL }},
    { &hf_rnsap_id_DL_DPCH_Information_RL_SetupRqstFDD,
      { "id-DL-DPCH-Information-RL-SetupRqstFDD", "rnsap.id_DL_DPCH_Information_RL_SetupRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_DPCH_Information_RL_SetupRqstFDD", HFILL }},
    { &hf_rnsap_id_DL_DPCH_Information_RL_ReconfRqstFDD,
      { "id-DL-DPCH-Information-RL-ReconfRqstFDD", "rnsap.id_DL_DPCH_Information_RL_ReconfRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_DPCH_Information_RL_ReconfRqstFDD", HFILL }},
    { &hf_rnsap_id_DL_DPCH_InformationItem_PhyChReconfRqstTDD,
      { "id-DL-DPCH-InformationItem-PhyChReconfRqstTDD", "rnsap.id_DL_DPCH_InformationItem_PhyChReconfRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_DPCH_InformationItem_PhyChReconfRqstTDD", HFILL }},
    { &hf_rnsap_id_DL_DPCH_InformationItem_RL_AdditionRspTDD,
      { "id-DL-DPCH-InformationItem-RL-AdditionRspTDD", "rnsap.id_DL_DPCH_InformationItem_RL_AdditionRspTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_DPCH_InformationItem_RL_AdditionRspTDD", HFILL }},
    { &hf_rnsap_id_DL_DPCH_InformationItem_RL_SetupRspTDD,
      { "id-DL-DPCH-InformationItem-RL-SetupRspTDD", "rnsap.id_DL_DPCH_InformationItem_RL_SetupRspTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_DPCH_InformationItem_RL_SetupRspTDD", HFILL }},
    { &hf_rnsap_id_DL_DPCH_TimingAdjustment,
      { "id-DL-DPCH-TimingAdjustment", "rnsap.id_DL_DPCH_TimingAdjustment",
        FT_UINT32, BASE_DEC, VALS(rnsap_DL_DPCH_TimingAdjustment_vals), 0,
        "rnsap.DL_DPCH_TimingAdjustment", HFILL }},
    { &hf_rnsap_id_DLReferencePower,
      { "id-DLReferencePower", "rnsap.id_DLReferencePower",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.DL_Power", HFILL }},
    { &hf_rnsap_id_DLReferencePowerList_DL_PC_Rqst,
      { "id-DLReferencePowerList-DL-PC-Rqst", "rnsap.id_DLReferencePowerList_DL_PC_Rqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DL_ReferencePowerInformationList_DL_PC_Rqst", HFILL }},
    { &hf_rnsap_id_DL_ReferencePowerInformation_DL_PC_Rqst,
      { "id-DL-ReferencePowerInformation-DL-PC-Rqst", "rnsap.id_DL_ReferencePowerInformation_DL_PC_Rqst",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_ReferencePowerInformation_DL_PC_Rqst", HFILL }},
    { &hf_rnsap_id_DPC_Mode,
      { "id-DPC-Mode", "rnsap.id_DPC_Mode",
        FT_UINT32, BASE_DEC, VALS(rnsap_DPC_Mode_vals), 0,
        "rnsap.DPC_Mode", HFILL }},
    { &hf_rnsap_id_DRXCycleLengthCoefficient,
      { "id-DRXCycleLengthCoefficient", "rnsap.id_DRXCycleLengthCoefficient",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DRXCycleLengthCoefficient", HFILL }},
    { &hf_rnsap_id_DedicatedMeasurementObjectType_DM_Fail_Ind,
      { "id-DedicatedMeasurementObjectType-DM-Fail-Ind", "rnsap.id_DedicatedMeasurementObjectType_DM_Fail_Ind",
        FT_UINT32, BASE_DEC, VALS(rnsap_DedicatedMeasurementObjectType_DM_Fail_Ind_vals), 0,
        "rnsap.DedicatedMeasurementObjectType_DM_Fail_Ind", HFILL }},
    { &hf_rnsap_id_DedicatedMeasurementObjectType_DM_Fail,
      { "id-DedicatedMeasurementObjectType-DM-Fail", "rnsap.id_DedicatedMeasurementObjectType_DM_Fail",
        FT_UINT32, BASE_DEC, VALS(rnsap_DedicatedMeasurementObjectType_DM_Fail_vals), 0,
        "rnsap.DedicatedMeasurementObjectType_DM_Fail", HFILL }},
    { &hf_rnsap_id_DedicatedMeasurementObjectType_DM_Rprt,
      { "id-DedicatedMeasurementObjectType-DM-Rprt", "rnsap.id_DedicatedMeasurementObjectType_DM_Rprt",
        FT_UINT32, BASE_DEC, VALS(rnsap_DedicatedMeasurementObjectType_DM_Rprt_vals), 0,
        "rnsap.DedicatedMeasurementObjectType_DM_Rprt", HFILL }},
    { &hf_rnsap_id_DedicatedMeasurementObjectType_DM_Rqst,
      { "id-DedicatedMeasurementObjectType-DM-Rqst", "rnsap.id_DedicatedMeasurementObjectType_DM_Rqst",
        FT_UINT32, BASE_DEC, VALS(rnsap_DedicatedMeasurementObjectType_DM_Rqst_vals), 0,
        "rnsap.DedicatedMeasurementObjectType_DM_Rqst", HFILL }},
    { &hf_rnsap_id_DedicatedMeasurementObjectType_DM_Rsp,
      { "id-DedicatedMeasurementObjectType-DM-Rsp", "rnsap.id_DedicatedMeasurementObjectType_DM_Rsp",
        FT_UINT32, BASE_DEC, VALS(rnsap_DedicatedMeasurementObjectType_DM_Rsp_vals), 0,
        "rnsap.DedicatedMeasurementObjectType_DM_Rsp", HFILL }},
    { &hf_rnsap_id_DedicatedMeasurementType,
      { "id-DedicatedMeasurementType", "rnsap.id_DedicatedMeasurementType",
        FT_UINT32, BASE_DEC, VALS(rnsap_DedicatedMeasurementType_vals), 0,
        "rnsap.DedicatedMeasurementType", HFILL }},
    { &hf_rnsap_id_FACH_InfoForUESelectedS_CCPCH_CTCH_ResourceRspFDD,
      { "id-FACH-InfoForUESelectedS-CCPCH-CTCH-ResourceRspFDD", "rnsap.id_FACH_InfoForUESelectedS_CCPCH_CTCH_ResourceRspFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.FACH_InfoForUESelectedS_CCPCH_CTCH_ResourceRspFDD", HFILL }},
    { &hf_rnsap_id_FACH_InfoForUESelectedS_CCPCH_CTCH_ResourceRspTDD,
      { "id-FACH-InfoForUESelectedS-CCPCH-CTCH-ResourceRspTDD", "rnsap.id_FACH_InfoForUESelectedS_CCPCH_CTCH_ResourceRspTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.FACH_InfoForUESelectedS_CCPCH_CTCH_ResourceRspTDD", HFILL }},
    { &hf_rnsap_id_Guaranteed_Rate_Information,
      { "id-Guaranteed-Rate-Information", "rnsap.id_Guaranteed_Rate_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Guaranteed_Rate_Information", HFILL }},
    { &hf_rnsap_id_IMSI,
      { "id-IMSI", "rnsap.id_IMSI",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.IMSI", HFILL }},
    { &hf_rnsap_id_HCS_Prio,
      { "id-HCS-Prio", "rnsap.id_HCS_Prio",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.HCS_Prio", HFILL }},
    { &hf_rnsap_id_L3_Information,
      { "id-L3-Information", "rnsap.id_L3_Information",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.L3_Information", HFILL }},
    { &hf_rnsap_id_AdjustmentPeriod,
      { "id-AdjustmentPeriod", "rnsap.id_AdjustmentPeriod",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.AdjustmentPeriod", HFILL }},
    { &hf_rnsap_id_MaxAdjustmentStep,
      { "id-MaxAdjustmentStep", "rnsap.id_MaxAdjustmentStep",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.MaxAdjustmentStep", HFILL }},
    { &hf_rnsap_id_MeasurementFilterCoefficient,
      { "id-MeasurementFilterCoefficient", "rnsap.id_MeasurementFilterCoefficient",
        FT_UINT32, BASE_DEC, VALS(rnsap_MeasurementFilterCoefficient_vals), 0,
        "rnsap.MeasurementFilterCoefficient", HFILL }},
    { &hf_rnsap_id_MessageStructure,
      { "id-MessageStructure", "rnsap.id_MessageStructure",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.MessageStructure", HFILL }},
    { &hf_rnsap_id_MeasurementID,
      { "id-MeasurementID", "rnsap.id_MeasurementID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.MeasurementID", HFILL }},
    { &hf_rnsap_id_Neighbouring_GSM_CellInformation,
      { "id-Neighbouring-GSM-CellInformation", "rnsap.id_Neighbouring_GSM_CellInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Neighbouring_GSM_CellInformation", HFILL }},
    { &hf_rnsap_id_Neighbouring_UMTS_CellInformationItem,
      { "id-Neighbouring-UMTS-CellInformationItem", "rnsap.id_Neighbouring_UMTS_CellInformationItem",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Neighbouring_UMTS_CellInformationItem", HFILL }},
    { &hf_rnsap_id_NRT_Load_Information_Value,
      { "id-NRT-Load-Information-Value", "rnsap.id_NRT_Load_Information_Value",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.NRT_Load_Information_Value", HFILL }},
    { &hf_rnsap_id_NRT_Load_Information_Value_IncrDecrThres,
      { "id-NRT-Load-Information-Value-IncrDecrThres", "rnsap.id_NRT_Load_Information_Value_IncrDecrThres",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.NRT_Load_Information_Value_IncrDecrThres", HFILL }},
    { &hf_rnsap_id_PagingArea_PagingRqst,
      { "id-PagingArea-PagingRqst", "rnsap.id_PagingArea_PagingRqst",
        FT_UINT32, BASE_DEC, VALS(rnsap_PagingArea_PagingRqst_vals), 0,
        "rnsap.PagingArea_PagingRqst", HFILL }},
    { &hf_rnsap_id_FACH_FlowControlInformation,
      { "id-FACH-FlowControlInformation", "rnsap.id_FACH_FlowControlInformation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.FACH_FlowControlInformation", HFILL }},
    { &hf_rnsap_id_PartialReportingIndicator,
      { "id-PartialReportingIndicator", "rnsap.id_PartialReportingIndicator",
        FT_UINT32, BASE_DEC, VALS(rnsap_PartialReportingIndicator_vals), 0,
        "rnsap.PartialReportingIndicator", HFILL }},
    { &hf_rnsap_id_Permanent_NAS_UE_Identity,
      { "id-Permanent-NAS-UE-Identity", "rnsap.id_Permanent_NAS_UE_Identity",
        FT_UINT32, BASE_DEC, VALS(rnsap_Permanent_NAS_UE_Identity_vals), 0,
        "rnsap.Permanent_NAS_UE_Identity", HFILL }},
    { &hf_rnsap_id_PowerAdjustmentType,
      { "id-PowerAdjustmentType", "rnsap.id_PowerAdjustmentType",
        FT_UINT32, BASE_DEC, VALS(rnsap_PowerAdjustmentType_vals), 0,
        "rnsap.PowerAdjustmentType", HFILL }},
    { &hf_rnsap_id_RANAP_RelocationInformation,
      { "id-RANAP-RelocationInformation", "rnsap.id_RANAP_RelocationInformation",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.RANAP_RelocationInformation", HFILL }},
    { &hf_rnsap_id_RL_Information_PhyChReconfRqstFDD,
      { "id-RL-Information-PhyChReconfRqstFDD", "rnsap.id_RL_Information_PhyChReconfRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_Information_PhyChReconfRqstFDD", HFILL }},
    { &hf_rnsap_id_RL_Information_PhyChReconfRqstTDD,
      { "id-RL-Information-PhyChReconfRqstTDD", "rnsap.id_RL_Information_PhyChReconfRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_Information_PhyChReconfRqstTDD", HFILL }},
    { &hf_rnsap_id_RL_Information_RL_AdditionRqstFDD,
      { "id-RL-Information-RL-AdditionRqstFDD", "rnsap.id_RL_Information_RL_AdditionRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_Information_RL_AdditionRqstFDD", HFILL }},
    { &hf_rnsap_id_RL_Information_RL_AdditionRqstTDD,
      { "id-RL-Information-RL-AdditionRqstTDD", "rnsap.id_RL_Information_RL_AdditionRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_Information_RL_AdditionRqstTDD", HFILL }},
    { &hf_rnsap_id_RL_Information_RL_DeletionRqst,
      { "id-RL-Information-RL-DeletionRqst", "rnsap.id_RL_Information_RL_DeletionRqst",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_Information_RL_DeletionRqst", HFILL }},
    { &hf_rnsap_id_RL_Information_RL_FailureInd,
      { "id-RL-Information-RL-FailureInd", "rnsap.id_RL_Information_RL_FailureInd",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_Information_RL_FailureInd", HFILL }},
    { &hf_rnsap_id_RL_Information_RL_ReconfPrepFDD,
      { "id-RL-Information-RL-ReconfPrepFDD", "rnsap.id_RL_Information_RL_ReconfPrepFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_Information_RL_ReconfPrepFDD", HFILL }},
    { &hf_rnsap_id_RL_Information_RL_RestoreInd,
      { "id-RL-Information-RL-RestoreInd", "rnsap.id_RL_Information_RL_RestoreInd",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_Information_RL_RestoreInd", HFILL }},
    { &hf_rnsap_id_RL_Information_RL_SetupRqstFDD,
      { "id-RL-Information-RL-SetupRqstFDD", "rnsap.id_RL_Information_RL_SetupRqstFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RL_InformationList_RL_SetupRqstFDD", HFILL }},
    { &hf_rnsap_id_RL_Information_RL_SetupRqstTDD,
      { "id-RL-Information-RL-SetupRqstTDD", "rnsap.id_RL_Information_RL_SetupRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_Information_RL_SetupRqstTDD", HFILL }},
    { &hf_rnsap_id_RL_InformationItem_RL_CongestInd,
      { "id-RL-InformationItem-RL-CongestInd", "rnsap.id_RL_InformationItem_RL_CongestInd",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_InformationItem_RL_CongestInd", HFILL }},
    { &hf_rnsap_id_RL_InformationItem_DM_Rprt,
      { "id-RL-InformationItem-DM-Rprt", "rnsap.id_RL_InformationItem_DM_Rprt",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_InformationItem_DM_Rprt", HFILL }},
    { &hf_rnsap_id_RL_InformationItem_DM_Rqst,
      { "id-RL-InformationItem-DM-Rqst", "rnsap.id_RL_InformationItem_DM_Rqst",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_InformationItem_DM_Rqst", HFILL }},
    { &hf_rnsap_id_RL_InformationItem_DM_Rsp,
      { "id-RL-InformationItem-DM-Rsp", "rnsap.id_RL_InformationItem_DM_Rsp",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_InformationItem_DM_Rsp", HFILL }},
    { &hf_rnsap_id_RL_InformationItem_RL_PreemptRequiredInd,
      { "id-RL-InformationItem-RL-PreemptRequiredInd", "rnsap.id_RL_InformationItem_RL_PreemptRequiredInd",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_InformationItem_RL_PreemptRequiredInd", HFILL }},
    { &hf_rnsap_id_RL_InformationItem_RL_SetupRqstFDD,
      { "id-RL-InformationItem-RL-SetupRqstFDD", "rnsap.id_RL_InformationItem_RL_SetupRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_InformationItem_RL_SetupRqstFDD", HFILL }},
    { &hf_rnsap_id_RL_InformationList_RL_CongestInd,
      { "id-RL-InformationList-RL-CongestInd", "rnsap.id_RL_InformationList_RL_CongestInd",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RL_InformationList_RL_CongestInd", HFILL }},
    { &hf_rnsap_id_RL_InformationList_RL_AdditionRqstFDD,
      { "id-RL-InformationList-RL-AdditionRqstFDD", "rnsap.id_RL_InformationList_RL_AdditionRqstFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RL_InformationList_RL_AdditionRqstFDD", HFILL }},
    { &hf_rnsap_id_RL_InformationList_RL_DeletionRqst,
      { "id-RL-InformationList-RL-DeletionRqst", "rnsap.id_RL_InformationList_RL_DeletionRqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RL_InformationList_RL_DeletionRqst", HFILL }},
    { &hf_rnsap_id_RL_InformationList_RL_PreemptRequiredInd,
      { "id-RL-InformationList-RL-PreemptRequiredInd", "rnsap.id_RL_InformationList_RL_PreemptRequiredInd",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RL_InformationList_RL_PreemptRequiredInd", HFILL }},
    { &hf_rnsap_id_RL_InformationList_RL_ReconfPrepFDD,
      { "id-RL-InformationList-RL-ReconfPrepFDD", "rnsap.id_RL_InformationList_RL_ReconfPrepFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RL_InformationList_RL_ReconfPrepFDD", HFILL }},
    { &hf_rnsap_id_RL_InformationResponse_RL_AdditionRspTDD,
      { "id-RL-InformationResponse-RL-AdditionRspTDD", "rnsap.id_RL_InformationResponse_RL_AdditionRspTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_InformationResponse_RL_AdditionRspTDD", HFILL }},
    { &hf_rnsap_id_RL_InformationResponse_RL_ReconfReadyTDD,
      { "id-RL-InformationResponse-RL-ReconfReadyTDD", "rnsap.id_RL_InformationResponse_RL_ReconfReadyTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_InformationResponse_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_id_RL_InformationResponse_RL_SetupRspTDD,
      { "id-RL-InformationResponse-RL-SetupRspTDD", "rnsap.id_RL_InformationResponse_RL_SetupRspTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_InformationResponse_RL_SetupRspTDD", HFILL }},
    { &hf_rnsap_id_RL_InformationResponseItem_RL_AdditionRspFDD,
      { "id-RL-InformationResponseItem-RL-AdditionRspFDD", "rnsap.id_RL_InformationResponseItem_RL_AdditionRspFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_InformationResponseItem_RL_AdditionRspFDD", HFILL }},
    { &hf_rnsap_id_RL_InformationResponseItem_RL_ReconfReadyFDD,
      { "id-RL-InformationResponseItem-RL-ReconfReadyFDD", "rnsap.id_RL_InformationResponseItem_RL_ReconfReadyFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_InformationResponseItem_RL_ReconfReadyFDD", HFILL }},
    { &hf_rnsap_id_RL_InformationResponseItem_RL_ReconfRspFDD,
      { "id-RL-InformationResponseItem-RL-ReconfRspFDD", "rnsap.id_RL_InformationResponseItem_RL_ReconfRspFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_InformationResponseItem_RL_ReconfRspFDD", HFILL }},
    { &hf_rnsap_id_RL_InformationResponseItem_RL_SetupRspFDD,
      { "id-RL-InformationResponseItem-RL-SetupRspFDD", "rnsap.id_RL_InformationResponseItem_RL_SetupRspFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_InformationResponseItem_RL_SetupRspFDD", HFILL }},
    { &hf_rnsap_id_RL_InformationResponseList_RL_AdditionRspFDD,
      { "id-RL-InformationResponseList-RL-AdditionRspFDD", "rnsap.id_RL_InformationResponseList_RL_AdditionRspFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RL_InformationResponseList_RL_AdditionRspFDD", HFILL }},
    { &hf_rnsap_id_RL_InformationResponseList_RL_ReconfReadyFDD,
      { "id-RL-InformationResponseList-RL-ReconfReadyFDD", "rnsap.id_RL_InformationResponseList_RL_ReconfReadyFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RL_InformationResponseList_RL_ReconfReadyFDD", HFILL }},
    { &hf_rnsap_id_RL_InformationResponseList_RL_ReconfRspFDD,
      { "id-RL-InformationResponseList-RL-ReconfRspFDD", "rnsap.id_RL_InformationResponseList_RL_ReconfRspFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RL_InformationResponseList_RL_ReconfRspFDD", HFILL }},
    { &hf_rnsap_id_RL_InformationResponse_RL_ReconfRspTDD,
      { "id-RL-InformationResponse-RL-ReconfRspTDD", "rnsap.id_RL_InformationResponse_RL_ReconfRspTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_InformationResponse_RL_ReconfRspTDD", HFILL }},
    { &hf_rnsap_id_RL_InformationResponseList_RL_SetupRspFDD,
      { "id-RL-InformationResponseList-RL-SetupRspFDD", "rnsap.id_RL_InformationResponseList_RL_SetupRspFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RL_InformationResponseList_RL_SetupRspFDD", HFILL }},
    { &hf_rnsap_id_RL_ReconfigurationFailure_RL_ReconfFail,
      { "id-RL-ReconfigurationFailure-RL-ReconfFail", "rnsap.id_RL_ReconfigurationFailure_RL_ReconfFail",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_ReconfigurationFailure_RL_ReconfFail", HFILL }},
    { &hf_rnsap_id_RL_Set_InformationItem_DM_Rprt,
      { "id-RL-Set-InformationItem-DM-Rprt", "rnsap.id_RL_Set_InformationItem_DM_Rprt",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_Set_InformationItem_DM_Rprt", HFILL }},
    { &hf_rnsap_id_RL_Set_InformationItem_DM_Rqst,
      { "id-RL-Set-InformationItem-DM-Rqst", "rnsap.id_RL_Set_InformationItem_DM_Rqst",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_Set_InformationItem_DM_Rqst", HFILL }},
    { &hf_rnsap_id_RL_Set_InformationItem_DM_Rsp,
      { "id-RL-Set-InformationItem-DM-Rsp", "rnsap.id_RL_Set_InformationItem_DM_Rsp",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_Set_InformationItem_DM_Rsp", HFILL }},
    { &hf_rnsap_id_RL_Set_Information_RL_FailureInd,
      { "id-RL-Set-Information-RL-FailureInd", "rnsap.id_RL_Set_Information_RL_FailureInd",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_Set_Information_RL_FailureInd", HFILL }},
    { &hf_rnsap_id_RL_Set_Information_RL_RestoreInd,
      { "id-RL-Set-Information-RL-RestoreInd", "rnsap.id_RL_Set_Information_RL_RestoreInd",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_Set_Information_RL_RestoreInd", HFILL }},
    { &hf_rnsap_id_RL_Set_Successful_InformationItem_DM_Fail,
      { "id-RL-Set-Successful-InformationItem-DM-Fail", "rnsap.id_RL_Set_Successful_InformationItem_DM_Fail",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_Set_Successful_InformationItem_DM_Fail", HFILL }},
    { &hf_rnsap_id_RL_Set_Unsuccessful_InformationItem_DM_Fail,
      { "id-RL-Set-Unsuccessful-InformationItem-DM-Fail", "rnsap.id_RL_Set_Unsuccessful_InformationItem_DM_Fail",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_Set_Unsuccessful_InformationItem_DM_Fail", HFILL }},
    { &hf_rnsap_id_RL_Set_Unsuccessful_InformationItem_DM_Fail_Ind,
      { "id-RL-Set-Unsuccessful-InformationItem-DM-Fail-Ind", "rnsap.id_RL_Set_Unsuccessful_InformationItem_DM_Fail_Ind",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_Set_Unsuccessful_InformationItem_DM_Fail_Ind", HFILL }},
    { &hf_rnsap_id_RL_Successful_InformationItem_DM_Fail,
      { "id-RL-Successful-InformationItem-DM-Fail", "rnsap.id_RL_Successful_InformationItem_DM_Fail",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_Successful_InformationItem_DM_Fail", HFILL }},
    { &hf_rnsap_id_RL_Unsuccessful_InformationItem_DM_Fail,
      { "id-RL-Unsuccessful-InformationItem-DM-Fail", "rnsap.id_RL_Unsuccessful_InformationItem_DM_Fail",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_Unsuccessful_InformationItem_DM_Fail", HFILL }},
    { &hf_rnsap_id_RL_Unsuccessful_InformationItem_DM_Fail_Ind,
      { "id-RL-Unsuccessful-InformationItem-DM-Fail-Ind", "rnsap.id_RL_Unsuccessful_InformationItem_DM_Fail_Ind",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_Unsuccessful_InformationItem_DM_Fail_Ind", HFILL }},
    { &hf_rnsap_id_ReportCharacteristics,
      { "id-ReportCharacteristics", "rnsap.id_ReportCharacteristics",
        FT_UINT32, BASE_DEC, VALS(rnsap_ReportCharacteristics_vals), 0,
        "rnsap.ReportCharacteristics", HFILL }},
    { &hf_rnsap_id_Reporting_Object_RL_FailureInd,
      { "id-Reporting-Object-RL-FailureInd", "rnsap.id_Reporting_Object_RL_FailureInd",
        FT_UINT32, BASE_DEC, VALS(rnsap_Reporting_Object_RL_FailureInd_vals), 0,
        "rnsap.Reporting_Object_RL_FailureInd", HFILL }},
    { &hf_rnsap_id_Reporing_Object_RL_RestoreInd,
      { "id-Reporing-Object-RL-RestoreInd", "rnsap.id_Reporing_Object_RL_RestoreInd",
        FT_UINT32, BASE_DEC, VALS(rnsap_Reporting_Object_RL_RestoreInd_vals), 0,
        "rnsap.Reporting_Object_RL_RestoreInd", HFILL }},
    { &hf_rnsap_id_RT_Load_Value,
      { "id-RT-Load-Value", "rnsap.id_RT_Load_Value",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RT_Load_Value", HFILL }},
    { &hf_rnsap_id_RT_Load_Value_IncrDecrThres,
      { "id-RT-Load-Value-IncrDecrThres", "rnsap.id_RT_Load_Value_IncrDecrThres",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RT_Load_Value_IncrDecrThres", HFILL }},
    { &hf_rnsap_id_S_RNTI,
      { "id-S-RNTI", "rnsap.id_S_RNTI",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.S_RNTI", HFILL }},
    { &hf_rnsap_id_ResetIndicator,
      { "id-ResetIndicator", "rnsap.id_ResetIndicator",
        FT_UINT32, BASE_DEC, VALS(rnsap_ResetIndicator_vals), 0,
        "rnsap.ResetIndicator", HFILL }},
    { &hf_rnsap_id_RNC_ID,
      { "id-RNC-ID", "rnsap.id_RNC_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RNC_ID", HFILL }},
    { &hf_rnsap_id_SAI,
      { "id-SAI", "rnsap.id_SAI",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.SAI", HFILL }},
    { &hf_rnsap_id_SRNC_ID,
      { "id-SRNC-ID", "rnsap.id_SRNC_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RNC_ID", HFILL }},
    { &hf_rnsap_id_SuccessfulRL_InformationResponse_RL_AdditionFailureFDD,
      { "id-SuccessfulRL-InformationResponse-RL-AdditionFailureFDD", "rnsap.id_SuccessfulRL_InformationResponse_RL_AdditionFailureFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.SuccessfulRL_InformationResponse_RL_AdditionFailureFDD", HFILL }},
    { &hf_rnsap_id_SuccessfulRL_InformationResponse_RL_SetupFailureFDD,
      { "id-SuccessfulRL-InformationResponse-RL-SetupFailureFDD", "rnsap.id_SuccessfulRL_InformationResponse_RL_SetupFailureFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.SuccessfulRL_InformationResponse_RL_SetupFailureFDD", HFILL }},
    { &hf_rnsap_id_TransportBearerID,
      { "id-TransportBearerID", "rnsap.id_TransportBearerID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.TransportBearerID", HFILL }},
    { &hf_rnsap_id_TransportBearerRequestIndicator,
      { "id-TransportBearerRequestIndicator", "rnsap.id_TransportBearerRequestIndicator",
        FT_UINT32, BASE_DEC, VALS(rnsap_TransportBearerRequestIndicator_vals), 0,
        "rnsap.TransportBearerRequestIndicator", HFILL }},
    { &hf_rnsap_id_TransportLayerAddress,
      { "id-TransportLayerAddress", "rnsap.id_TransportLayerAddress",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.TransportLayerAddress", HFILL }},
    { &hf_rnsap_id_TypeOfError,
      { "id-TypeOfError", "rnsap.id_TypeOfError",
        FT_UINT32, BASE_DEC, VALS(rnsap_TypeOfError_vals), 0,
        "rnsap.TypeOfError", HFILL }},
    { &hf_rnsap_id_UC_ID,
      { "id-UC-ID", "rnsap.id_UC_ID",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UC_ID", HFILL }},
    { &hf_rnsap_id_UL_CCTrCH_AddInformation_RL_ReconfPrepTDD,
      { "id-UL-CCTrCH-AddInformation-RL-ReconfPrepTDD", "rnsap.id_UL_CCTrCH_AddInformation_RL_ReconfPrepTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_CCTrCH_AddInformation_RL_ReconfPrepTDD", HFILL }},
    { &hf_rnsap_id_UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD,
      { "id-UL-CCTrCH-InformationAddList-RL-ReconfPrepTDD", "rnsap.id_UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD", HFILL }},
    { &hf_rnsap_id_UL_CCTrCH_InformationItem_RL_SetupRqstTDD,
      { "id-UL-CCTrCH-InformationItem-RL-SetupRqstTDD", "rnsap.id_UL_CCTrCH_InformationItem_RL_SetupRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_CCTrCH_InformationItem_RL_SetupRqstTDD", HFILL }},
    { &hf_rnsap_id_UL_CCTrCH_InformationList_RL_SetupRqstTDD,
      { "id-UL-CCTrCH-InformationList-RL-SetupRqstTDD", "rnsap.id_UL_CCTrCH_InformationList_RL_SetupRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UL_CCTrCH_InformationList_RL_SetupRqstTDD", HFILL }},
    { &hf_rnsap_id_UL_CCTrCH_InformationListIE_PhyChReconfRqstTDD,
      { "id-UL-CCTrCH-InformationListIE-PhyChReconfRqstTDD", "rnsap.id_UL_CCTrCH_InformationListIE_PhyChReconfRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UL_CCTrCH_InformationListIE_PhyChReconfRqstTDD", HFILL }},
    { &hf_rnsap_id_UL_CCTrCH_InformationListIE_RL_AdditionRspTDD,
      { "id-UL-CCTrCH-InformationListIE-RL-AdditionRspTDD", "rnsap.id_UL_CCTrCH_InformationListIE_RL_AdditionRspTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UL_CCTrCHInformationListIE_RL_AdditionRspTDD", HFILL }},
    { &hf_rnsap_id_UL_CCTrCH_InformationListIE_RL_ReconfReadyTDD,
      { "id-UL-CCTrCH-InformationListIE-RL-ReconfReadyTDD", "rnsap.id_UL_CCTrCH_InformationListIE_RL_ReconfReadyTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UL_CCTrCHInformationListIE_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_id_UL_CCTrCH_InformationListIE_RL_SetupRspTDD,
      { "id-UL-CCTrCH-InformationListIE-RL-SetupRspTDD", "rnsap.id_UL_CCTrCH_InformationListIE_RL_SetupRspTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UL_CCTrCHInformationListIE_RL_SetupRspTDD", HFILL }},
    { &hf_rnsap_id_UL_DPCH_Information_RL_ReconfPrepFDD,
      { "id-UL-DPCH-Information-RL-ReconfPrepFDD", "rnsap.id_UL_DPCH_Information_RL_ReconfPrepFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_DPCH_Information_RL_ReconfPrepFDD", HFILL }},
    { &hf_rnsap_id_UL_DPCH_Information_RL_ReconfRqstFDD,
      { "id-UL-DPCH-Information-RL-ReconfRqstFDD", "rnsap.id_UL_DPCH_Information_RL_ReconfRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_DPCH_Information_RL_ReconfRqstFDD", HFILL }},
    { &hf_rnsap_id_UL_DPCH_Information_RL_SetupRqstFDD,
      { "id-UL-DPCH-Information-RL-SetupRqstFDD", "rnsap.id_UL_DPCH_Information_RL_SetupRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_DPCH_Information_RL_SetupRqstFDD", HFILL }},
    { &hf_rnsap_id_UL_DPCH_InformationItem_PhyChReconfRqstTDD,
      { "id-UL-DPCH-InformationItem-PhyChReconfRqstTDD", "rnsap.id_UL_DPCH_InformationItem_PhyChReconfRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_DPCH_InformationItem_PhyChReconfRqstTDD", HFILL }},
    { &hf_rnsap_id_UL_DPCH_InformationItem_RL_AdditionRspTDD,
      { "id-UL-DPCH-InformationItem-RL-AdditionRspTDD", "rnsap.id_UL_DPCH_InformationItem_RL_AdditionRspTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_DPCH_InformationItem_RL_AdditionRspTDD", HFILL }},
    { &hf_rnsap_id_UL_DPCH_InformationItem_RL_SetupRspTDD,
      { "id-UL-DPCH-InformationItem-RL-SetupRspTDD", "rnsap.id_UL_DPCH_InformationItem_RL_SetupRspTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_DPCH_InformationItem_RL_SetupRspTDD", HFILL }},
    { &hf_rnsap_id_UL_DPCH_InformationAddListIE_RL_ReconfReadyTDD,
      { "id-UL-DPCH-InformationAddListIE-RL-ReconfReadyTDD", "rnsap.id_UL_DPCH_InformationAddListIE_RL_ReconfReadyTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_DPCH_InformationAddListIE_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_id_UL_SIRTarget,
      { "id-UL-SIRTarget", "rnsap.id_UL_SIRTarget",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.UL_SIR", HFILL }},
    { &hf_rnsap_id_URA_Information,
      { "id-URA-Information", "rnsap.id_URA_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.URA_Information", HFILL }},
    { &hf_rnsap_id_UnsuccessfulRL_InformationResponse_RL_AdditionFailureFDD,
      { "id-UnsuccessfulRL-InformationResponse-RL-AdditionFailureFDD", "rnsap.id_UnsuccessfulRL_InformationResponse_RL_AdditionFailureFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UnsuccessfulRL_InformationResponse_RL_AdditionFailureFDD", HFILL }},
    { &hf_rnsap_id_UnsuccessfulRL_InformationResponse_RL_SetupFailureFDD,
      { "id-UnsuccessfulRL-InformationResponse-RL-SetupFailureFDD", "rnsap.id_UnsuccessfulRL_InformationResponse_RL_SetupFailureFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UnsuccessfulRL_InformationResponse_RL_SetupFailureFDD", HFILL }},
    { &hf_rnsap_id_UnsuccessfulRL_InformationResponse_RL_SetupFailureTDD,
      { "id-UnsuccessfulRL-InformationResponse-RL-SetupFailureTDD", "rnsap.id_UnsuccessfulRL_InformationResponse_RL_SetupFailureTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UnsuccessfulRL_InformationResponse_RL_SetupFailureTDD", HFILL }},
    { &hf_rnsap_id_Active_Pattern_Sequence_Information,
      { "id-Active-Pattern-Sequence-Information", "rnsap.id_Active_Pattern_Sequence_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Active_Pattern_Sequence_Information", HFILL }},
    { &hf_rnsap_id_AdjustmentRatio,
      { "id-AdjustmentRatio", "rnsap.id_AdjustmentRatio",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.ScaledAdjustmentRatio", HFILL }},
    { &hf_rnsap_id_CauseLevel_RL_AdditionFailureFDD,
      { "id-CauseLevel-RL-AdditionFailureFDD", "rnsap.id_CauseLevel_RL_AdditionFailureFDD",
        FT_UINT32, BASE_DEC, VALS(rnsap_CauseLevel_RL_AdditionFailureFDD_vals), 0,
        "rnsap.CauseLevel_RL_AdditionFailureFDD", HFILL }},
    { &hf_rnsap_id_CauseLevel_RL_AdditionFailureTDD,
      { "id-CauseLevel-RL-AdditionFailureTDD", "rnsap.id_CauseLevel_RL_AdditionFailureTDD",
        FT_UINT32, BASE_DEC, VALS(rnsap_CauseLevel_RL_AdditionFailureTDD_vals), 0,
        "rnsap.CauseLevel_RL_AdditionFailureTDD", HFILL }},
    { &hf_rnsap_id_CauseLevel_RL_ReconfFailure,
      { "id-CauseLevel-RL-ReconfFailure", "rnsap.id_CauseLevel_RL_ReconfFailure",
        FT_UINT32, BASE_DEC, VALS(rnsap_CauseLevel_RL_ReconfFailure_vals), 0,
        "rnsap.CauseLevel_RL_ReconfFailure", HFILL }},
    { &hf_rnsap_id_CauseLevel_RL_SetupFailureFDD,
      { "id-CauseLevel-RL-SetupFailureFDD", "rnsap.id_CauseLevel_RL_SetupFailureFDD",
        FT_UINT32, BASE_DEC, VALS(rnsap_CauseLevel_RL_SetupFailureFDD_vals), 0,
        "rnsap.CauseLevel_RL_SetupFailureFDD", HFILL }},
    { &hf_rnsap_id_CauseLevel_RL_SetupFailureTDD,
      { "id-CauseLevel-RL-SetupFailureTDD", "rnsap.id_CauseLevel_RL_SetupFailureTDD",
        FT_UINT32, BASE_DEC, VALS(rnsap_CauseLevel_RL_SetupFailureTDD_vals), 0,
        "rnsap.CauseLevel_RL_SetupFailureTDD", HFILL }},
    { &hf_rnsap_id_DL_CCTrCH_InformationDeleteItem_RL_ReconfPrepTDD,
      { "id-DL-CCTrCH-InformationDeleteItem-RL-ReconfPrepTDD", "rnsap.id_DL_CCTrCH_InformationDeleteItem_RL_ReconfPrepTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_CCTrCH_InformationDeleteItem_RL_ReconfPrepTDD", HFILL }},
    { &hf_rnsap_id_DL_CCTrCH_InformationModifyItem_RL_ReconfPrepTDD,
      { "id-DL-CCTrCH-InformationModifyItem-RL-ReconfPrepTDD", "rnsap.id_DL_CCTrCH_InformationModifyItem_RL_ReconfPrepTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_CCTrCH_InformationModifyItem_RL_ReconfPrepTDD", HFILL }},
    { &hf_rnsap_id_DL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD,
      { "id-DL-CCTrCH-InformationModifyItem-RL-ReconfRqstTDD", "rnsap.id_DL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD", HFILL }},
    { &hf_rnsap_id_DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD,
      { "id-DL-CCTrCH-InformationDeleteList-RL-ReconfPrepTDD", "rnsap.id_DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD", HFILL }},
    { &hf_rnsap_id_DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD,
      { "id-DL-CCTrCH-InformationModifyList-RL-ReconfPrepTDD", "rnsap.id_DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD", HFILL }},
    { &hf_rnsap_id_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD,
      { "id-DL-CCTrCH-InformationModifyList-RL-ReconfRqstTDD", "rnsap.id_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD", HFILL }},
    { &hf_rnsap_id_DL_DPCH_InformationAddListIE_RL_ReconfReadyTDD,
      { "id-DL-DPCH-InformationAddListIE-RL-ReconfReadyTDD", "rnsap.id_DL_DPCH_InformationAddListIE_RL_ReconfReadyTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_DPCH_InformationAddListIE_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_id_DL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD,
      { "id-DL-DPCH-InformationDeleteListIE-RL-ReconfReadyTDD", "rnsap.id_DL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_id_DL_DPCH_InformationModifyListIE_RL_ReconfReadyTDD,
      { "id-DL-DPCH-InformationModifyListIE-RL-ReconfReadyTDD", "rnsap.id_DL_DPCH_InformationModifyListIE_RL_ReconfReadyTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_DPCH_InformationModifyListIE_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_id_DSCHs_to_Add_TDD,
      { "id-DSCHs-to-Add-TDD", "rnsap.id_DSCHs_to_Add_TDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DSCH_TDD_Information", HFILL }},
    { &hf_rnsap_id_DSCH_DeleteList_RL_ReconfPrepTDD,
      { "id-DSCH-DeleteList-RL-ReconfPrepTDD", "rnsap.id_DSCH_DeleteList_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DSCH_DeleteList_RL_ReconfPrepTDD", HFILL }},
    { &hf_rnsap_id_DSCH_InformationListIE_RL_AdditionRspTDD,
      { "id-DSCH-InformationListIE-RL-AdditionRspTDD", "rnsap.id_DSCH_InformationListIE_RL_AdditionRspTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DSCH_InformationListIE_RL_AdditionRspTDD", HFILL }},
    { &hf_rnsap_id_DSCH_InformationListIEs_RL_SetupRspTDD,
      { "id-DSCH-InformationListIEs-RL-SetupRspTDD", "rnsap.id_DSCH_InformationListIEs_RL_SetupRspTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DSCH_InformationListIEs_RL_SetupRspTDD", HFILL }},
    { &hf_rnsap_id_DSCH_TDD_Information,
      { "id-DSCH-TDD-Information", "rnsap.id_DSCH_TDD_Information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DSCH_TDD_Information", HFILL }},
    { &hf_rnsap_id_DSCH_ModifyList_RL_ReconfPrepTDD,
      { "id-DSCH-ModifyList-RL-ReconfPrepTDD", "rnsap.id_DSCH_ModifyList_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DSCH_ModifyList_RL_ReconfPrepTDD", HFILL }},
    { &hf_rnsap_id_DSCHToBeAddedOrModifiedList_RL_ReconfReadyTDD,
      { "id-DSCHToBeAddedOrModifiedList-RL-ReconfReadyTDD", "rnsap.id_DSCHToBeAddedOrModifiedList_RL_ReconfReadyTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DSCHToBeAddedOrModifiedList_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_id_GA_Cell,
      { "id-GA-Cell", "rnsap.id_GA_Cell",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.GA_Cell", HFILL }},
    { &hf_rnsap_id_GA_CellAdditionalShapes,
      { "id-GA-CellAdditionalShapes", "rnsap.id_GA_CellAdditionalShapes",
        FT_UINT32, BASE_DEC, VALS(rnsap_GA_CellAdditionalShapes_vals), 0,
        "rnsap.GA_CellAdditionalShapes", HFILL }},
    { &hf_rnsap_id_Transmission_Gap_Pattern_Sequence_Information,
      { "id-Transmission-Gap-Pattern-Sequence-Information", "rnsap.id_Transmission_Gap_Pattern_Sequence_Information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Transmission_Gap_Pattern_Sequence_Information", HFILL }},
    { &hf_rnsap_id_UL_CCTrCH_DeleteInformation_RL_ReconfPrepTDD,
      { "id-UL-CCTrCH-DeleteInformation-RL-ReconfPrepTDD", "rnsap.id_UL_CCTrCH_DeleteInformation_RL_ReconfPrepTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_CCTrCH_DeleteInformation_RL_ReconfPrepTDD", HFILL }},
    { &hf_rnsap_id_UL_CCTrCH_ModifyInformation_RL_ReconfPrepTDD,
      { "id-UL-CCTrCH-ModifyInformation-RL-ReconfPrepTDD", "rnsap.id_UL_CCTrCH_ModifyInformation_RL_ReconfPrepTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_CCTrCH_ModifyInformation_RL_ReconfPrepTDD", HFILL }},
    { &hf_rnsap_id_UL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD,
      { "id-UL-CCTrCH-InformationModifyItem-RL-ReconfRqstTDD", "rnsap.id_UL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD", HFILL }},
    { &hf_rnsap_id_UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD,
      { "id-UL-CCTrCH-InformationDeleteList-RL-ReconfPrepTDD", "rnsap.id_UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD", HFILL }},
    { &hf_rnsap_id_UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD,
      { "id-UL-CCTrCH-InformationModifyList-RL-ReconfPrepTDD", "rnsap.id_UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD", HFILL }},
    { &hf_rnsap_id_UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD,
      { "id-UL-CCTrCH-InformationModifyList-RL-ReconfRqstTDD", "rnsap.id_UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD", HFILL }},
    { &hf_rnsap_id_UL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD,
      { "id-UL-CCTrCH-InformationDeleteItem-RL-ReconfRqstTDD", "rnsap.id_UL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD", HFILL }},
    { &hf_rnsap_id_UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD,
      { "id-UL-CCTrCH-InformationDeleteList-RL-ReconfRqstTDD", "rnsap.id_UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD", HFILL }},
    { &hf_rnsap_id_UL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD,
      { "id-UL-DPCH-InformationDeleteListIE-RL-ReconfReadyTDD", "rnsap.id_UL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_id_UL_DPCH_InformationModifyListIE_RL_ReconfReadyTDD,
      { "id-UL-DPCH-InformationModifyListIE-RL-ReconfReadyTDD", "rnsap.id_UL_DPCH_InformationModifyListIE_RL_ReconfReadyTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_DPCH_InformationModifyListIE_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_id_UnsuccessfulRL_InformationResponse_RL_AdditionFailureTDD,
      { "id-UnsuccessfulRL-InformationResponse-RL-AdditionFailureTDD", "rnsap.id_UnsuccessfulRL_InformationResponse_RL_AdditionFailureTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UnsuccessfulRL_InformationResponse_RL_AdditionFailureTDD", HFILL }},
    { &hf_rnsap_id_USCHs_to_Add,
      { "id-USCHs-to-Add", "rnsap.id_USCHs_to_Add",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.USCH_Information", HFILL }},
    { &hf_rnsap_id_USCH_DeleteList_RL_ReconfPrepTDD,
      { "id-USCH-DeleteList-RL-ReconfPrepTDD", "rnsap.id_USCH_DeleteList_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.USCH_DeleteList_RL_ReconfPrepTDD", HFILL }},
    { &hf_rnsap_id_USCH_InformationListIE_RL_AdditionRspTDD,
      { "id-USCH-InformationListIE-RL-AdditionRspTDD", "rnsap.id_USCH_InformationListIE_RL_AdditionRspTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.USCH_InformationListIE_RL_AdditionRspTDD", HFILL }},
    { &hf_rnsap_id_USCH_InformationListIEs_RL_SetupRspTDD,
      { "id-USCH-InformationListIEs-RL-SetupRspTDD", "rnsap.id_USCH_InformationListIEs_RL_SetupRspTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.USCH_InformationListIEs_RL_SetupRspTDD", HFILL }},
    { &hf_rnsap_id_USCH_Information,
      { "id-USCH-Information", "rnsap.id_USCH_Information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.USCH_Information", HFILL }},
    { &hf_rnsap_id_USCH_ModifyList_RL_ReconfPrepTDD,
      { "id-USCH-ModifyList-RL-ReconfPrepTDD", "rnsap.id_USCH_ModifyList_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.USCH_ModifyList_RL_ReconfPrepTDD", HFILL }},
    { &hf_rnsap_id_USCHToBeAddedOrModifiedList_RL_ReconfReadyTDD,
      { "id-USCHToBeAddedOrModifiedList-RL-ReconfReadyTDD", "rnsap.id_USCHToBeAddedOrModifiedList_RL_ReconfReadyTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.USCHToBeAddedOrModifiedList_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_id_DL_Physical_Channel_Information_RL_SetupRqstTDD,
      { "id-DL-Physical-Channel-Information-RL-SetupRqstTDD", "rnsap.id_DL_Physical_Channel_Information_RL_SetupRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_Physical_Channel_Information_RL_SetupRqstTDD", HFILL }},
    { &hf_rnsap_id_UL_Physical_Channel_Information_RL_SetupRqstTDD,
      { "id-UL-Physical-Channel-Information-RL-SetupRqstTDD", "rnsap.id_UL_Physical_Channel_Information_RL_SetupRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_Physical_Channel_Information_RL_SetupRqstTDD", HFILL }},
    { &hf_rnsap_id_ClosedLoopMode1_SupportIndicator,
      { "id-ClosedLoopMode1-SupportIndicator", "rnsap.id_ClosedLoopMode1_SupportIndicator",
        FT_UINT32, BASE_DEC, VALS(rnsap_ClosedLoopMode1_SupportIndicator_vals), 0,
        "rnsap.ClosedLoopMode1_SupportIndicator", HFILL }},
    { &hf_rnsap_id_STTD_SupportIndicator,
      { "id-STTD-SupportIndicator", "rnsap.id_STTD_SupportIndicator",
        FT_UINT32, BASE_DEC, VALS(rnsap_STTD_SupportIndicator_vals), 0,
        "rnsap.STTD_SupportIndicator", HFILL }},
    { &hf_rnsap_id_CFNReportingIndicator,
      { "id-CFNReportingIndicator", "rnsap.id_CFNReportingIndicator",
        FT_UINT32, BASE_DEC, VALS(rnsap_FNReportingIndicator_vals), 0,
        "rnsap.FNReportingIndicator", HFILL }},
    { &hf_rnsap_id_CNOriginatedPage_PagingRqst,
      { "id-CNOriginatedPage-PagingRqst", "rnsap.id_CNOriginatedPage_PagingRqst",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.CNOriginatedPage_PagingRqst", HFILL }},
    { &hf_rnsap_id_InnerLoopDLPCStatus,
      { "id-InnerLoopDLPCStatus", "rnsap.id_InnerLoopDLPCStatus",
        FT_UINT32, BASE_DEC, VALS(rnsap_InnerLoopDLPCStatus_vals), 0,
        "rnsap.InnerLoopDLPCStatus", HFILL }},
    { &hf_rnsap_id_PropagationDelay,
      { "id-PropagationDelay", "rnsap.id_PropagationDelay",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.PropagationDelay", HFILL }},
    { &hf_rnsap_id_RxTimingDeviationForTA,
      { "id-RxTimingDeviationForTA", "rnsap.id_RxTimingDeviationForTA",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RxTimingDeviationForTA", HFILL }},
    { &hf_rnsap_id_timeSlot_ISCP,
      { "id-timeSlot-ISCP", "rnsap.id_timeSlot_ISCP",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DL_TimeSlot_ISCP_Info", HFILL }},
    { &hf_rnsap_id_CCTrCH_InformationItem_RL_FailureInd,
      { "id-CCTrCH-InformationItem-RL-FailureInd", "rnsap.id_CCTrCH_InformationItem_RL_FailureInd",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.CCTrCH_InformationItem_RL_FailureInd", HFILL }},
    { &hf_rnsap_id_CCTrCH_InformationItem_RL_RestoreInd,
      { "id-CCTrCH-InformationItem-RL-RestoreInd", "rnsap.id_CCTrCH_InformationItem_RL_RestoreInd",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.CCTrCH_InformationItem_RL_RestoreInd", HFILL }},
    { &hf_rnsap_id_CommonMeasurementAccuracy,
      { "id-CommonMeasurementAccuracy", "rnsap.id_CommonMeasurementAccuracy",
        FT_UINT32, BASE_DEC, VALS(rnsap_CommonMeasurementAccuracy_vals), 0,
        "rnsap.CommonMeasurementAccuracy", HFILL }},
    { &hf_rnsap_id_CommonMeasurementObjectType_CM_Rprt,
      { "id-CommonMeasurementObjectType-CM-Rprt", "rnsap.id_CommonMeasurementObjectType_CM_Rprt",
        FT_UINT32, BASE_DEC, VALS(rnsap_CommonMeasurementObjectType_CM_Rprt_vals), 0,
        "rnsap.CommonMeasurementObjectType_CM_Rprt", HFILL }},
    { &hf_rnsap_id_CommonMeasurementObjectType_CM_Rqst,
      { "id-CommonMeasurementObjectType-CM-Rqst", "rnsap.id_CommonMeasurementObjectType_CM_Rqst",
        FT_UINT32, BASE_DEC, VALS(rnsap_CommonMeasurementObjectType_CM_Rqst_vals), 0,
        "rnsap.CommonMeasurementObjectType_CM_Rqst", HFILL }},
    { &hf_rnsap_id_CommonMeasurementObjectType_CM_Rsp,
      { "id-CommonMeasurementObjectType-CM-Rsp", "rnsap.id_CommonMeasurementObjectType_CM_Rsp",
        FT_UINT32, BASE_DEC, VALS(rnsap_CommonMeasurementObjectType_CM_Rsp_vals), 0,
        "rnsap.CommonMeasurementObjectType_CM_Rsp", HFILL }},
    { &hf_rnsap_id_CommonMeasurementType,
      { "id-CommonMeasurementType", "rnsap.id_CommonMeasurementType",
        FT_UINT32, BASE_DEC, VALS(rnsap_CommonMeasurementType_vals), 0,
        "rnsap.CommonMeasurementType", HFILL }},
    { &hf_rnsap_id_CongestionCause,
      { "id-CongestionCause", "rnsap.id_CongestionCause",
        FT_UINT32, BASE_DEC, VALS(rnsap_CongestionCause_vals), 0,
        "rnsap.CongestionCause", HFILL }},
    { &hf_rnsap_id_SFN,
      { "id-SFN", "rnsap.id_SFN",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.SFN", HFILL }},
    { &hf_rnsap_id_SFNReportingIndicator,
      { "id-SFNReportingIndicator", "rnsap.id_SFNReportingIndicator",
        FT_UINT32, BASE_DEC, VALS(rnsap_FNReportingIndicator_vals), 0,
        "rnsap.FNReportingIndicator", HFILL }},
    { &hf_rnsap_id_InformationExchangeID,
      { "id-InformationExchangeID", "rnsap.id_InformationExchangeID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.InformationExchangeID", HFILL }},
    { &hf_rnsap_id_InformationExchangeObjectType_InfEx_Rprt,
      { "id-InformationExchangeObjectType-InfEx-Rprt", "rnsap.id_InformationExchangeObjectType_InfEx_Rprt",
        FT_UINT32, BASE_DEC, VALS(rnsap_InformationExchangeObjectType_InfEx_Rprt_vals), 0,
        "rnsap.InformationExchangeObjectType_InfEx_Rprt", HFILL }},
    { &hf_rnsap_id_InformationExchangeObjectType_InfEx_Rqst,
      { "id-InformationExchangeObjectType-InfEx-Rqst", "rnsap.id_InformationExchangeObjectType_InfEx_Rqst",
        FT_UINT32, BASE_DEC, VALS(rnsap_InformationExchangeObjectType_InfEx_Rqst_vals), 0,
        "rnsap.InformationExchangeObjectType_InfEx_Rqst", HFILL }},
    { &hf_rnsap_id_InformationExchangeObjectType_InfEx_Rsp,
      { "id-InformationExchangeObjectType-InfEx-Rsp", "rnsap.id_InformationExchangeObjectType_InfEx_Rsp",
        FT_UINT32, BASE_DEC, VALS(rnsap_InformationReportCharacteristics_vals), 0,
        "rnsap.InformationReportCharacteristics", HFILL }},
    { &hf_rnsap_id_InformationReportCharacteristics,
      { "id-InformationReportCharacteristics", "rnsap.id_InformationReportCharacteristics",
        FT_UINT32, BASE_DEC, VALS(rnsap_InformationReportCharacteristics_vals), 0,
        "rnsap.InformationReportCharacteristics", HFILL }},
    { &hf_rnsap_id_InformationType,
      { "id-InformationType", "rnsap.id_InformationType",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.InformationType", HFILL }},
    { &hf_rnsap_id_neighbouring_LCR_TDD_CellInformation,
      { "id-neighbouring-LCR-TDD-CellInformation", "rnsap.id_neighbouring_LCR_TDD_CellInformation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Neighbouring_LCR_TDD_CellInformation", HFILL }},
    { &hf_rnsap_id_DL_Timeslot_ISCP_LCR_Information_RL_SetupRqstTDD,
      { "id-DL-Timeslot-ISCP-LCR-Information-RL-SetupRqstTDD", "rnsap.id_DL_Timeslot_ISCP_LCR_Information_RL_SetupRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DL_TimeSlot_ISCP_LCR_Information", HFILL }},
    { &hf_rnsap_id_RL_LCR_InformationResponse_RL_SetupRspTDD,
      { "id-RL-LCR-InformationResponse-RL-SetupRspTDD", "rnsap.id_RL_LCR_InformationResponse_RL_SetupRspTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_LCR_InformationResponse_RL_SetupRspTDD", HFILL }},
    { &hf_rnsap_id_UL_CCTrCH_LCR_InformationListIE_RL_SetupRspTDD,
      { "id-UL-CCTrCH-LCR-InformationListIE-RL-SetupRspTDD", "rnsap.id_UL_CCTrCH_LCR_InformationListIE_RL_SetupRspTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UL_LCR_CCTrCHInformationListIE_RL_SetupRspTDD", HFILL }},
    { &hf_rnsap_id_UL_DPCH_LCR_InformationItem_RL_SetupRspTDD,
      { "id-UL-DPCH-LCR-InformationItem-RL-SetupRspTDD", "rnsap.id_UL_DPCH_LCR_InformationItem_RL_SetupRspTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_DPCH_LCR_InformationItem_RL_SetupRspTDD", HFILL }},
    { &hf_rnsap_id_DL_CCTrCH_LCR_InformationListIE_RL_SetupRspTDD,
      { "id-DL-CCTrCH-LCR-InformationListIE-RL-SetupRspTDD", "rnsap.id_DL_CCTrCH_LCR_InformationListIE_RL_SetupRspTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DL_CCTrCH_LCR_InformationListIE_RL_SetupRspTDD", HFILL }},
    { &hf_rnsap_id_DL_DPCH_LCR_InformationItem_RL_SetupRspTDD,
      { "id-DL-DPCH-LCR-InformationItem-RL-SetupRspTDD", "rnsap.id_DL_DPCH_LCR_InformationItem_RL_SetupRspTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_DPCH_LCR_InformationItem_RL_SetupRspTDD", HFILL }},
    { &hf_rnsap_id_DSCH_LCR_InformationListIEs_RL_SetupRspTDD,
      { "id-DSCH-LCR-InformationListIEs-RL-SetupRspTDD", "rnsap.id_DSCH_LCR_InformationListIEs_RL_SetupRspTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DSCH_LCR_InformationListIEs_RL_SetupRspTDD", HFILL }},
    { &hf_rnsap_id_USCH_LCR_InformationListIEs_RL_SetupRspTDD,
      { "id-USCH-LCR-InformationListIEs-RL-SetupRspTDD", "rnsap.id_USCH_LCR_InformationListIEs_RL_SetupRspTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.USCH_LCR_InformationListIEs_RL_SetupRspTDD", HFILL }},
    { &hf_rnsap_id_DL_Timeslot_ISCP_LCR_Information_RL_AdditionRqstTDD,
      { "id-DL-Timeslot-ISCP-LCR-Information-RL-AdditionRqstTDD", "rnsap.id_DL_Timeslot_ISCP_LCR_Information_RL_AdditionRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DL_TimeSlot_ISCP_LCR_Information", HFILL }},
    { &hf_rnsap_id_RL_LCR_InformationResponse_RL_AdditionRspTDD,
      { "id-RL-LCR-InformationResponse-RL-AdditionRspTDD", "rnsap.id_RL_LCR_InformationResponse_RL_AdditionRspTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_LCR_InformationResponse_RL_AdditionRspTDD", HFILL }},
    { &hf_rnsap_id_UL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD,
      { "id-UL-CCTrCH-LCR-InformationListIE-RL-AdditionRspTDD", "rnsap.id_UL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD", HFILL }},
    { &hf_rnsap_id_UL_DPCH_LCR_InformationItem_RL_AdditionRspTDD,
      { "id-UL-DPCH-LCR-InformationItem-RL-AdditionRspTDD", "rnsap.id_UL_DPCH_LCR_InformationItem_RL_AdditionRspTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_DPCH_LCR_InformationItem_RL_AdditionRspTDD", HFILL }},
    { &hf_rnsap_id_DL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD,
      { "id-DL-CCTrCH-LCR-InformationListIE-RL-AdditionRspTDD", "rnsap.id_DL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD", HFILL }},
    { &hf_rnsap_id_DL_DPCH_LCR_InformationItem_RL_AdditionRspTDD,
      { "id-DL-DPCH-LCR-InformationItem-RL-AdditionRspTDD", "rnsap.id_DL_DPCH_LCR_InformationItem_RL_AdditionRspTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_DPCH_LCR_InformationItem_RL_AdditionRspTDD", HFILL }},
    { &hf_rnsap_id_DSCH_LCR_InformationListIEs_RL_AdditionRspTDD,
      { "id-DSCH-LCR-InformationListIEs-RL-AdditionRspTDD", "rnsap.id_DSCH_LCR_InformationListIEs_RL_AdditionRspTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DSCH_LCR_InformationListIEs_RL_AdditionRspTDD", HFILL }},
    { &hf_rnsap_id_USCH_LCR_InformationListIEs_RL_AdditionRspTDD,
      { "id-USCH-LCR-InformationListIEs-RL-AdditionRspTDD", "rnsap.id_USCH_LCR_InformationListIEs_RL_AdditionRspTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.USCH_LCR_InformationListIEs_RL_AdditionRspTDD", HFILL }},
    { &hf_rnsap_id_UL_DPCH_LCR_InformationAddListIE_RL_ReconfReadyTDD,
      { "id-UL-DPCH-LCR-InformationAddListIE-RL-ReconfReadyTDD", "rnsap.id_UL_DPCH_LCR_InformationAddListIE_RL_ReconfReadyTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_DPCH_LCR_InformationAddList_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_id_UL_Timeslot_LCR_InformationModifyList_RL_ReconfReadyTDD,
      { "id-UL-Timeslot-LCR-InformationModifyList-RL-ReconfReadyTDD", "rnsap.id_UL_Timeslot_LCR_InformationModifyList_RL_ReconfReadyTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UL_TimeslotLCR_InformationModifyList_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_id_DL_DPCH_LCR_InformationAddListIE_RL_ReconfReadyTDD,
      { "id-DL-DPCH-LCR-InformationAddListIE-RL-ReconfReadyTDD", "rnsap.id_DL_DPCH_LCR_InformationAddListIE_RL_ReconfReadyTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_DPCH_LCR_InformationAddList_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_id_DL_Timeslot_LCR_InformationModifyList_RL_ReconfReadyTDD,
      { "id-DL-Timeslot-LCR-InformationModifyList-RL-ReconfReadyTDD", "rnsap.id_DL_Timeslot_LCR_InformationModifyList_RL_ReconfReadyTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DL_TimeslotLCR_InformationModifyList_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_id_UL_Timeslot_LCR_InformationList_PhyChReconfRqstTDD,
      { "id-UL-Timeslot-LCR-InformationList-PhyChReconfRqstTDD", "rnsap.id_UL_Timeslot_LCR_InformationList_PhyChReconfRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UL_TimeslotLCR_InformationList_PhyChReconfRqstTDD", HFILL }},
    { &hf_rnsap_id_DL_Timeslot_LCR_InformationList_PhyChReconfRqstTDD,
      { "id-DL-Timeslot-LCR-InformationList-PhyChReconfRqstTDD", "rnsap.id_DL_Timeslot_LCR_InformationList_PhyChReconfRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DL_TimeslotLCR_InformationList_PhyChReconfRqstTDD", HFILL }},
    { &hf_rnsap_id_timeSlot_ISCP_LCR_List_DL_PC_Rqst_TDD,
      { "id-timeSlot-ISCP-LCR-List-DL-PC-Rqst-TDD", "rnsap.id_timeSlot_ISCP_LCR_List_DL_PC_Rqst_TDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DL_TimeSlot_ISCP_LCR_Information", HFILL }},
    { &hf_rnsap_id_TSTD_Support_Indicator_RL_SetupRqstTDD,
      { "id-TSTD-Support-Indicator-RL-SetupRqstTDD", "rnsap.id_TSTD_Support_Indicator_RL_SetupRqstTDD",
        FT_UINT32, BASE_DEC, VALS(rnsap_TSTD_Support_Indicator_vals), 0,
        "rnsap.TSTD_Support_Indicator", HFILL }},
    { &hf_rnsap_id_RestrictionStateIndicator,
      { "id-RestrictionStateIndicator", "rnsap.id_RestrictionStateIndicator",
        FT_UINT32, BASE_DEC, VALS(rnsap_RestrictionStateIndicator_vals), 0,
        "rnsap.RestrictionStateIndicator", HFILL }},
    { &hf_rnsap_id_Load_Value,
      { "id-Load-Value", "rnsap.id_Load_Value",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Load_Value", HFILL }},
    { &hf_rnsap_id_Load_Value_IncrDecrThres,
      { "id-Load-Value-IncrDecrThres", "rnsap.id_Load_Value_IncrDecrThres",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Load_Value_IncrDecrThres", HFILL }},
    { &hf_rnsap_id_OnModification,
      { "id-OnModification", "rnsap.id_OnModification",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.OnModification", HFILL }},
    { &hf_rnsap_id_Received_Total_Wideband_Power_Value,
      { "id-Received-Total-Wideband-Power-Value", "rnsap.id_Received_Total_Wideband_Power_Value",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Received_Total_Wideband_Power_Value", HFILL }},
    { &hf_rnsap_id_Received_Total_Wideband_Power_Value_IncrDecrThres,
      { "id-Received-Total-Wideband-Power-Value-IncrDecrThres", "rnsap.id_Received_Total_Wideband_Power_Value_IncrDecrThres",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.SFNSFNMeasurementThresholdInformation", HFILL }},
    { &hf_rnsap_id_SFNSFNMeasurementThresholdInformation,
      { "id-SFNSFNMeasurementThresholdInformation", "rnsap.id_SFNSFNMeasurementThresholdInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.SFNSFNMeasurementThresholdInformation", HFILL }},
    { &hf_rnsap_id_Transmitted_Carrier_Power_Value,
      { "id-Transmitted-Carrier-Power-Value", "rnsap.id_Transmitted_Carrier_Power_Value",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Transmitted_Carrier_Power_Value", HFILL }},
    { &hf_rnsap_id_Transmitted_Carrier_Power_Value_IncrDecrThres,
      { "id-Transmitted-Carrier-Power-Value-IncrDecrThres", "rnsap.id_Transmitted_Carrier_Power_Value_IncrDecrThres",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Transmitted_Carrier_Power_Value_IncrDecrThres", HFILL }},
    { &hf_rnsap_id_TUTRANGPSMeasurementThresholdInformation,
      { "id-TUTRANGPSMeasurementThresholdInformation", "rnsap.id_TUTRANGPSMeasurementThresholdInformation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UL_Timeslot_ISCP_Value", HFILL }},
    { &hf_rnsap_id_UL_Timeslot_ISCP_Value,
      { "id-UL-Timeslot-ISCP-Value", "rnsap.id_UL_Timeslot_ISCP_Value",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UL_Timeslot_ISCP_Value", HFILL }},
    { &hf_rnsap_id_UL_Timeslot_ISCP_Value_IncrDecrThres,
      { "id-UL-Timeslot-ISCP-Value-IncrDecrThres", "rnsap.id_UL_Timeslot_ISCP_Value_IncrDecrThres",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UL_Timeslot_ISCP_Value_IncrDecrThres", HFILL }},
    { &hf_rnsap_id_Rx_Timing_Deviation_Value_LCR,
      { "id-Rx-Timing-Deviation-Value-LCR", "rnsap.id_Rx_Timing_Deviation_Value_LCR",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Rx_Timing_Deviation_Value_LCR", HFILL }},
    { &hf_rnsap_id_DPC_Mode_Change_SupportIndicator,
      { "id-DPC-Mode-Change-SupportIndicator", "rnsap.id_DPC_Mode_Change_SupportIndicator",
        FT_UINT32, BASE_DEC, VALS(rnsap_DPC_Mode_Change_SupportIndicator_vals), 0,
        "rnsap.DPC_Mode_Change_SupportIndicator", HFILL }},
    { &hf_rnsap_id_PrimaryCCPCH_RSCP_RL_ReconfPrepTDD,
      { "id-PrimaryCCPCH-RSCP-RL-ReconfPrepTDD", "rnsap.id_PrimaryCCPCH_RSCP_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.PrimaryCCPCH_RSCP", HFILL }},
    { &hf_rnsap_id_DL_TimeSlot_ISCP_Info_RL_ReconfPrepTDD,
      { "id-DL-TimeSlot-ISCP-Info-RL-ReconfPrepTDD", "rnsap.id_DL_TimeSlot_ISCP_Info_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DL_TimeSlot_ISCP_Info", HFILL }},
    { &hf_rnsap_id_DL_Timeslot_ISCP_LCR_Information_RL_ReconfPrepTDD,
      { "id-DL-Timeslot-ISCP-LCR-Information-RL-ReconfPrepTDD", "rnsap.id_DL_Timeslot_ISCP_LCR_Information_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DL_TimeSlot_ISCP_LCR_Information", HFILL }},
    { &hf_rnsap_id_DSCH_RNTI,
      { "id-DSCH-RNTI", "rnsap.id_DSCH_RNTI",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DSCH_RNTI", HFILL }},
    { &hf_rnsap_id_DL_PowerBalancing_Information,
      { "id-DL-PowerBalancing-Information", "rnsap.id_DL_PowerBalancing_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_PowerBalancing_Information", HFILL }},
    { &hf_rnsap_id_DL_PowerBalancing_ActivationIndicator,
      { "id-DL-PowerBalancing-ActivationIndicator", "rnsap.id_DL_PowerBalancing_ActivationIndicator",
        FT_UINT32, BASE_DEC, VALS(rnsap_DL_PowerBalancing_ActivationIndicator_vals), 0,
        "rnsap.DL_PowerBalancing_ActivationIndicator", HFILL }},
    { &hf_rnsap_id_DL_PowerBalancing_UpdatedIndicator,
      { "id-DL-PowerBalancing-UpdatedIndicator", "rnsap.id_DL_PowerBalancing_UpdatedIndicator",
        FT_UINT32, BASE_DEC, VALS(rnsap_DL_PowerBalancing_UpdatedIndicator_vals), 0,
        "rnsap.DL_PowerBalancing_UpdatedIndicator", HFILL }},
    { &hf_rnsap_id_DL_ReferencePowerInformation,
      { "id-DL-ReferencePowerInformation", "rnsap.id_DL_ReferencePowerInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_ReferencePowerInformation", HFILL }},
    { &hf_rnsap_id_Enhanced_PrimaryCPICH_EcNo,
      { "id-Enhanced-PrimaryCPICH-EcNo", "rnsap.id_Enhanced_PrimaryCPICH_EcNo",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Enhanced_PrimaryCPICH_EcNo", HFILL }},
    { &hf_rnsap_id_IPDL_TDD_ParametersLCR,
      { "id-IPDL-TDD-ParametersLCR", "rnsap.id_IPDL_TDD_ParametersLCR",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.IPDL_TDD_ParametersLCR", HFILL }},
    { &hf_rnsap_id_CellCapabilityContainer_FDD,
      { "id-CellCapabilityContainer-FDD", "rnsap.id_CellCapabilityContainer_FDD",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.CellCapabilityContainer_FDD", HFILL }},
    { &hf_rnsap_id_CellCapabilityContainer_TDD,
      { "id-CellCapabilityContainer-TDD", "rnsap.id_CellCapabilityContainer_TDD",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.CellCapabilityContainer_TDD", HFILL }},
    { &hf_rnsap_id_CellCapabilityContainer_TDD_LCR,
      { "id-CellCapabilityContainer-TDD-LCR", "rnsap.id_CellCapabilityContainer_TDD_LCR",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.CellCapabilityContainer_TDD_LCR", HFILL }},
    { &hf_rnsap_id_RL_Specific_DCH_Info,
      { "id-RL-Specific-DCH-Info", "rnsap.id_RL_Specific_DCH_Info",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RL_Specific_DCH_Info", HFILL }},
    { &hf_rnsap_id_RL_ReconfigurationRequestFDD_RL_InformationList,
      { "id-RL-ReconfigurationRequestFDD-RL-InformationList", "rnsap.id_RL_ReconfigurationRequestFDD_RL_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RL_ReconfigurationRequestFDD_RL_InformationList", HFILL }},
    { &hf_rnsap_id_RL_ReconfigurationRequestFDD_RL_Information_IEs,
      { "id-RL-ReconfigurationRequestFDD-RL-Information-IEs", "rnsap.id_RL_ReconfigurationRequestFDD_RL_Information_IEs",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_ReconfigurationRequestFDD_RL_Information_IEs", HFILL }},
    { &hf_rnsap_id_RL_ReconfigurationRequestTDD_RL_Information,
      { "id-RL-ReconfigurationRequestTDD-RL-Information", "rnsap.id_RL_ReconfigurationRequestTDD_RL_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_ReconfigurationRequestTDD_RL_Information", HFILL }},
    { &hf_rnsap_id_CommonTransportChannelResourcesInitialisationNotRequired,
      { "id-CommonTransportChannelResourcesInitialisationNotRequired", "rnsap.id_CommonTransportChannelResourcesInitialisationNotRequired",
        FT_UINT32, BASE_DEC, VALS(rnsap_CommonTransportChannelResourcesInitialisationNotRequired_vals), 0,
        "rnsap.CommonTransportChannelResourcesInitialisationNotRequired", HFILL }},
    { &hf_rnsap_id_DelayedActivation,
      { "id-DelayedActivation", "rnsap.id_DelayedActivation",
        FT_UINT32, BASE_DEC, VALS(rnsap_DelayedActivation_vals), 0,
        "rnsap.DelayedActivation", HFILL }},
    { &hf_rnsap_id_DelayedActivationList_RL_ActivationCmdFDD,
      { "id-DelayedActivationList-RL-ActivationCmdFDD", "rnsap.id_DelayedActivationList_RL_ActivationCmdFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DelayedActivationInformationList_RL_ActivationCmdFDD", HFILL }},
    { &hf_rnsap_id_DelayedActivationInformation_RL_ActivationCmdFDD,
      { "id-DelayedActivationInformation-RL-ActivationCmdFDD", "rnsap.id_DelayedActivationInformation_RL_ActivationCmdFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DelayedActivationInformation_RL_ActivationCmdFDD", HFILL }},
    { &hf_rnsap_id_DelayedActivationList_RL_ActivationCmdTDD,
      { "id-DelayedActivationList-RL-ActivationCmdTDD", "rnsap.id_DelayedActivationList_RL_ActivationCmdTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DelayedActivationInformationList_RL_ActivationCmdTDD", HFILL }},
    { &hf_rnsap_id_DelayedActivationInformation_RL_ActivationCmdTDD,
      { "id-DelayedActivationInformation-RL-ActivationCmdTDD", "rnsap.id_DelayedActivationInformation_RL_ActivationCmdTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DelayedActivationInformation_RL_ActivationCmdTDD", HFILL }},
    { &hf_rnsap_id_neighbouringTDDCellMeasurementInformationLCR,
      { "id-neighbouringTDDCellMeasurementInformationLCR", "rnsap.id_neighbouringTDDCellMeasurementInformationLCR",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.NeighbouringTDDCellMeasurementInformationLCR", HFILL }},
    { &hf_rnsap_id_UL_SIR_Target_CCTrCH_InformationItem_RL_SetupRspTDD,
      { "id-UL-SIR-Target-CCTrCH-InformationItem-RL-SetupRspTDD", "rnsap.id_UL_SIR_Target_CCTrCH_InformationItem_RL_SetupRspTDD",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.UL_SIR", HFILL }},
    { &hf_rnsap_id_UL_SIR_Target_CCTrCH_LCR_InformationItem_RL_SetupRspTDD,
      { "id-UL-SIR-Target-CCTrCH-LCR-InformationItem-RL-SetupRspTDD", "rnsap.id_UL_SIR_Target_CCTrCH_LCR_InformationItem_RL_SetupRspTDD",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.UL_SIR", HFILL }},
    { &hf_rnsap_id_PrimCCPCH_RSCP_DL_PC_RqstTDD,
      { "id-PrimCCPCH-RSCP-DL-PC-RqstTDD", "rnsap.id_PrimCCPCH_RSCP_DL_PC_RqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.PrimaryCCPCH_RSCP", HFILL }},
    { &hf_rnsap_id_HSDSCH_FDD_Information,
      { "id-HSDSCH-FDD-Information", "rnsap.id_HSDSCH_FDD_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.HSDSCH_FDD_Information", HFILL }},
    { &hf_rnsap_id_HSDSCH_FDD_Information_Response,
      { "id-HSDSCH-FDD-Information-Response", "rnsap.id_HSDSCH_FDD_Information_Response",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.HSDSCH_FDD_Information_Response", HFILL }},
    { &hf_rnsap_id_HSDSCH_FDD_Update_Information,
      { "id-HSDSCH-FDD-Update-Information", "rnsap.id_HSDSCH_FDD_Update_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.HSDSCH_FDD_Update_Information", HFILL }},
    { &hf_rnsap_id_HSDSCH_Information_to_Modify,
      { "id-HSDSCH-Information-to-Modify", "rnsap.id_HSDSCH_Information_to_Modify",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.HSDSCH_Information_to_Modify", HFILL }},
    { &hf_rnsap_id_HSDSCHMacdFlowSpecificInformationList_RL_PreemptRequiredInd,
      { "id-HSDSCHMacdFlowSpecificInformationList-RL-PreemptRequiredInd", "rnsap.id_HSDSCHMacdFlowSpecificInformationList_RL_PreemptRequiredInd",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.HSDSCHMacdFlowSpecificInformationList_RL_PreemptRequiredInd", HFILL }},
    { &hf_rnsap_id_HSDSCHMacdFlowSpecificInformationItem_RL_PreemptRequiredInd,
      { "id-HSDSCHMacdFlowSpecificInformationItem-RL-PreemptRequiredInd", "rnsap.id_HSDSCHMacdFlowSpecificInformationItem_RL_PreemptRequiredInd",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.HSDSCHMacdFlowSpecificInformationItem_RL_PreemptRequiredInd", HFILL }},
    { &hf_rnsap_id_HSDSCH_RNTI,
      { "id-HSDSCH-RNTI", "rnsap.id_HSDSCH_RNTI",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.HSDSCH_RNTI", HFILL }},
    { &hf_rnsap_id_HSDSCH_TDD_Information,
      { "id-HSDSCH-TDD-Information", "rnsap.id_HSDSCH_TDD_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.HSDSCH_TDD_Information", HFILL }},
    { &hf_rnsap_id_HSDSCH_TDD_Information_Response,
      { "id-HSDSCH-TDD-Information-Response", "rnsap.id_HSDSCH_TDD_Information_Response",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.HSDSCH_TDD_Information_Response", HFILL }},
    { &hf_rnsap_id_HSDSCH_TDD_Update_Information,
      { "id-HSDSCH-TDD-Update-Information", "rnsap.id_HSDSCH_TDD_Update_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.HSDSCH_TDD_Update_Information", HFILL }},
    { &hf_rnsap_id_HSPDSCH_RL_ID,
      { "id-HSPDSCH-RL-ID", "rnsap.id_HSPDSCH_RL_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RL_ID", HFILL }},
    { &hf_rnsap_id_HSDSCH_MACdFlows_to_Add,
      { "id-HSDSCH-MACdFlows-to-Add", "rnsap.id_HSDSCH_MACdFlows_to_Add",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.HSDSCH_MACdFlows_Information", HFILL }},
    { &hf_rnsap_id_HSDSCH_MACdFlows_to_Delete,
      { "id-HSDSCH-MACdFlows-to-Delete", "rnsap.id_HSDSCH_MACdFlows_to_Delete",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.HSDSCH_MACdFlows_to_Delete", HFILL }},
    { &hf_rnsap_id_Angle_Of_Arrival_Value_LCR,
      { "id-Angle-Of-Arrival-Value-LCR", "rnsap.id_Angle_Of_Arrival_Value_LCR",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Angle_Of_Arrival_Value_LCR", HFILL }},
    { &hf_rnsap_id_TrafficClass,
      { "id-TrafficClass", "rnsap.id_TrafficClass",
        FT_UINT32, BASE_DEC, VALS(rnsap_TrafficClass_vals), 0,
        "rnsap.TrafficClass", HFILL }},
    { &hf_rnsap_id_PDSCH_RL_ID,
      { "id-PDSCH-RL-ID", "rnsap.id_PDSCH_RL_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RL_ID", HFILL }},
    { &hf_rnsap_id_TimeSlot_RL_SetupRspTDD,
      { "id-TimeSlot-RL-SetupRspTDD", "rnsap.id_TimeSlot_RL_SetupRspTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.TimeSlot", HFILL }},
    { &hf_rnsap_id_GERAN_Cell_Capability,
      { "id-GERAN-Cell-Capability", "rnsap.id_GERAN_Cell_Capability",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.GERAN_Cell_Capability", HFILL }},
    { &hf_rnsap_id_GERAN_Classmark,
      { "id-GERAN-Classmark", "rnsap.id_GERAN_Classmark",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.GERAN_Classmark", HFILL }},
    { &hf_rnsap_id_DSCH_InitialWindowSize,
      { "id-DSCH-InitialWindowSize", "rnsap.id_DSCH_InitialWindowSize",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DSCH_InitialWindowSize", HFILL }},
    { &hf_rnsap_id_UL_Synchronisation_Parameters_LCR,
      { "id-UL-Synchronisation-Parameters-LCR", "rnsap.id_UL_Synchronisation_Parameters_LCR",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_Synchronisation_Parameters_LCR", HFILL }},
    { &hf_rnsap_id_SNA_Information,
      { "id-SNA-Information", "rnsap.id_SNA_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.SNA_Information", HFILL }},
    { &hf_rnsap_id_MAChs_ResetIndicator,
      { "id-MAChs-ResetIndicator", "rnsap.id_MAChs_ResetIndicator",
        FT_UINT32, BASE_DEC, VALS(rnsap_MAChs_ResetIndicator_vals), 0,
        "rnsap.MAChs_ResetIndicator", HFILL }},
    { &hf_rnsap_id_TDD_DL_DPCH_TimeSlotFormatModifyItem_LCR_RL_ReconfReadyTDD,
      { "id-TDD-DL-DPCH-TimeSlotFormatModifyItem-LCR-RL-ReconfReadyTDD", "rnsap.id_TDD_DL_DPCH_TimeSlotFormatModifyItem_LCR_RL_ReconfReadyTDD",
        FT_UINT32, BASE_DEC, VALS(rnsap_TDD_DL_DPCH_TimeSlotFormat_LCR_vals), 0,
        "rnsap.TDD_DL_DPCH_TimeSlotFormat_LCR", HFILL }},
    { &hf_rnsap_id_TDD_UL_DPCH_TimeSlotFormatModifyItem_LCR_RL_ReconfReadyTDD,
      { "id-TDD-UL-DPCH-TimeSlotFormatModifyItem-LCR-RL-ReconfReadyTDD", "rnsap.id_TDD_UL_DPCH_TimeSlotFormatModifyItem_LCR_RL_ReconfReadyTDD",
        FT_UINT32, BASE_DEC, VALS(rnsap_TDD_UL_DPCH_TimeSlotFormat_LCR_vals), 0,
        "rnsap.TDD_UL_DPCH_TimeSlotFormat_LCR", HFILL }},
    { &hf_rnsap_id_TDD_TPC_UplinkStepSize_LCR_RL_SetupRqstTDD,
      { "id-TDD-TPC-UplinkStepSize-LCR-RL-SetupRqstTDD", "rnsap.id_TDD_TPC_UplinkStepSize_LCR_RL_SetupRqstTDD",
        FT_UINT32, BASE_DEC, VALS(rnsap_TDD_TPC_UplinkStepSize_LCR_vals), 0,
        "rnsap.TDD_TPC_UplinkStepSize_LCR", HFILL }},
    { &hf_rnsap_id_UL_CCTrCH_InformationList_RL_AdditionRqstTDD,
      { "id-UL-CCTrCH-InformationList-RL-AdditionRqstTDD", "rnsap.id_UL_CCTrCH_InformationList_RL_AdditionRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UL_CCTrCH_InformationList_RL_AdditionRqstTDD", HFILL }},
    { &hf_rnsap_id_UL_CCTrCH_InformationItem_RL_AdditionRqstTDD,
      { "id-UL-CCTrCH-InformationItem-RL-AdditionRqstTDD", "rnsap.id_UL_CCTrCH_InformationItem_RL_AdditionRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_CCTrCH_InformationItem_RL_AdditionRqstTDD", HFILL }},
    { &hf_rnsap_id_DL_CCTrCH_InformationList_RL_AdditionRqstTDD,
      { "id-DL-CCTrCH-InformationList-RL-AdditionRqstTDD", "rnsap.id_DL_CCTrCH_InformationList_RL_AdditionRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DL_CCTrCH_InformationList_RL_AdditionRqstTDD", HFILL }},
    { &hf_rnsap_id_DL_CCTrCH_InformationItem_RL_AdditionRqstTDD,
      { "id-DL-CCTrCH-InformationItem-RL-AdditionRqstTDD", "rnsap.id_DL_CCTrCH_InformationItem_RL_AdditionRqstTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_CCTrCH_InformationItem_RL_AdditionRqstTDD", HFILL }},
    { &hf_rnsap_id_TDD_TPC_UplinkStepSize_InformationAdd_LCR_RL_ReconfPrepTDD,
      { "id-TDD-TPC-UplinkStepSize-InformationAdd-LCR-RL-ReconfPrepTDD", "rnsap.id_TDD_TPC_UplinkStepSize_InformationAdd_LCR_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, VALS(rnsap_TDD_TPC_UplinkStepSize_LCR_vals), 0,
        "rnsap.TDD_TPC_UplinkStepSize_LCR", HFILL }},
    { &hf_rnsap_id_TDD_TPC_UplinkStepSize_InformationModify_LCR_RL_ReconfPrepTDD,
      { "id-TDD-TPC-UplinkStepSize-InformationModify-LCR-RL-ReconfPrepTDD", "rnsap.id_TDD_TPC_UplinkStepSize_InformationModify_LCR_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, VALS(rnsap_TDD_TPC_UplinkStepSize_LCR_vals), 0,
        "rnsap.TDD_TPC_UplinkStepSize_LCR", HFILL }},
    { &hf_rnsap_id_TDD_TPC_DownlinkStepSize_InformationAdd_RL_ReconfPrepTDD,
      { "id-TDD-TPC-DownlinkStepSize-InformationAdd-RL-ReconfPrepTDD", "rnsap.id_TDD_TPC_DownlinkStepSize_InformationAdd_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, VALS(rnsap_TDD_TPC_DownlinkStepSize_vals), 0,
        "rnsap.TDD_TPC_DownlinkStepSize", HFILL }},
    { &hf_rnsap_id_TDD_TPC_DownlinkStepSize_InformationModify_RL_ReconfPrepTDD,
      { "id-TDD-TPC-DownlinkStepSize-InformationModify-RL-ReconfPrepTDD", "rnsap.id_TDD_TPC_DownlinkStepSize_InformationModify_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, VALS(rnsap_TDD_TPC_DownlinkStepSize_vals), 0,
        "rnsap.TDD_TPC_DownlinkStepSize", HFILL }},
    { &hf_rnsap_id_UL_TimingAdvanceCtrl_LCR,
      { "id-UL-TimingAdvanceCtrl-LCR", "rnsap.id_UL_TimingAdvanceCtrl_LCR",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_TimingAdvanceCtrl_LCR", HFILL }},
    { &hf_rnsap_id_HSPDSCH_Timeslot_InformationList_PhyChReconfRqstTDD,
      { "id-HSPDSCH-Timeslot-InformationList-PhyChReconfRqstTDD", "rnsap.id_HSPDSCH_Timeslot_InformationList_PhyChReconfRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.HSPDSCH_Timeslot_InformationList_PhyChReconfRqstTDD", HFILL }},
    { &hf_rnsap_id_HSPDSCH_Timeslot_InformationListLCR_PhyChReconfRqstTDD,
      { "id-HSPDSCH-Timeslot-InformationListLCR-PhyChReconfRqstTDD", "rnsap.id_HSPDSCH_Timeslot_InformationListLCR_PhyChReconfRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.HSPDSCH_Timeslot_InformationListLCR_PhyChReconfRqstTDD", HFILL }},
    { &hf_rnsap_id_HS_SICH_Reception_Quality,
      { "id-HS-SICH-Reception-Quality", "rnsap.id_HS_SICH_Reception_Quality",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.HS_SICH_Reception_Quality_Value", HFILL }},
    { &hf_rnsap_id_HS_SICH_Reception_Quality_Measurement_Value,
      { "id-HS-SICH-Reception-Quality-Measurement-Value", "rnsap.id_HS_SICH_Reception_Quality_Measurement_Value",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.HS_SICH_Reception_Quality_Measurement_Value", HFILL }},
    { &hf_rnsap_id_HSSICH_Info_DM_Rprt,
      { "id-HSSICH-Info-DM-Rprt", "rnsap.id_HSSICH_Info_DM_Rprt",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.HS_SICH_ID", HFILL }},
    { &hf_rnsap_id_HSSICH_Info_DM_Rqst,
      { "id-HSSICH-Info-DM-Rqst", "rnsap.id_HSSICH_Info_DM_Rqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.HSSICH_Info_DM_Rqst", HFILL }},
    { &hf_rnsap_id_HSSICH_Info_DM,
      { "id-HSSICH-Info-DM", "rnsap.id_HSSICH_Info_DM",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.HS_SICH_ID", HFILL }},
    { &hf_rnsap_id_CCTrCH_Maximum_DL_Power_RL_SetupRspTDD,
      { "id-CCTrCH-Maximum-DL-Power-RL-SetupRspTDD", "rnsap.id_CCTrCH_Maximum_DL_Power_RL_SetupRspTDD",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.DL_Power", HFILL }},
    { &hf_rnsap_id_CCTrCH_Minimum_DL_Power_RL_SetupRspTDD,
      { "id-CCTrCH-Minimum-DL-Power-RL-SetupRspTDD", "rnsap.id_CCTrCH_Minimum_DL_Power_RL_SetupRspTDD",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.DL_Power", HFILL }},
    { &hf_rnsap_id_CCTrCH_Maximum_DL_Power_RL_AdditionRspTDD,
      { "id-CCTrCH-Maximum-DL-Power-RL-AdditionRspTDD", "rnsap.id_CCTrCH_Maximum_DL_Power_RL_AdditionRspTDD",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.DL_Power", HFILL }},
    { &hf_rnsap_id_CCTrCH_Minimum_DL_Power_RL_AdditionRspTDD,
      { "id-CCTrCH-Minimum-DL-Power-RL-AdditionRspTDD", "rnsap.id_CCTrCH_Minimum_DL_Power_RL_AdditionRspTDD",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.DL_Power", HFILL }},
    { &hf_rnsap_id_CCTrCH_Maximum_DL_Power_RL_ReconfReadyTDD,
      { "id-CCTrCH-Maximum-DL-Power-RL-ReconfReadyTDD", "rnsap.id_CCTrCH_Maximum_DL_Power_RL_ReconfReadyTDD",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.DL_Power", HFILL }},
    { &hf_rnsap_id_CCTrCH_Minimum_DL_Power_RL_ReconfReadyTDD,
      { "id-CCTrCH-Minimum-DL-Power-RL-ReconfReadyTDD", "rnsap.id_CCTrCH_Minimum_DL_Power_RL_ReconfReadyTDD",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.DL_Power", HFILL }},
    { &hf_rnsap_id_Maximum_DL_Power_TimeslotLCR_InformationModifyItem_RL_ReconfReadyTDD,
      { "id-Maximum-DL-Power-TimeslotLCR-InformationModifyItem-RL-ReconfReadyTDD", "rnsap.id_Maximum_DL_Power_TimeslotLCR_InformationModifyItem_RL_ReconfReadyTDD",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.DL_Power", HFILL }},
    { &hf_rnsap_id_Minimum_DL_Power_TimeslotLCR_InformationModifyItem_RL_ReconfReadyTDD,
      { "id-Minimum-DL-Power-TimeslotLCR-InformationModifyItem-RL-ReconfReadyTDD", "rnsap.id_Minimum_DL_Power_TimeslotLCR_InformationModifyItem_RL_ReconfReadyTDD",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.DL_Power", HFILL }},
    { &hf_rnsap_id_DL_CCTrCH_InformationList_RL_ReconfRspTDD,
      { "id-DL-CCTrCH-InformationList-RL-ReconfRspTDD", "rnsap.id_DL_CCTrCH_InformationList_RL_ReconfRspTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DL_CCTrCH_InformationList_RL_ReconfRspTDD", HFILL }},
    { &hf_rnsap_id_DL_DPCH_InformationModifyItem_LCR_RL_ReconfRspTDD,
      { "id-DL-DPCH-InformationModifyItem-LCR-RL-ReconfRspTDD", "rnsap.id_DL_DPCH_InformationModifyItem_LCR_RL_ReconfRspTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_DPCH_InformationModifyItem_LCR_RL_ReconfRspTDD", HFILL }},
    { &hf_rnsap_id_Maximum_DL_Power_TimeslotLCR_InformationItem,
      { "id-Maximum-DL-Power-TimeslotLCR-InformationItem", "rnsap.id_Maximum_DL_Power_TimeslotLCR_InformationItem",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.DL_Power", HFILL }},
    { &hf_rnsap_id_Minimum_DL_Power_TimeslotLCR_InformationItem,
      { "id-Minimum-DL-Power-TimeslotLCR-InformationItem", "rnsap.id_Minimum_DL_Power_TimeslotLCR_InformationItem",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.DL_Power", HFILL }},
    { &hf_rnsap_id_TDD_Support_8PSK,
      { "id-TDD-Support-8PSK", "rnsap.id_TDD_Support_8PSK",
        FT_UINT32, BASE_DEC, VALS(rnsap_Support_8PSK_vals), 0,
        "rnsap.Support_8PSK", HFILL }},
    { &hf_rnsap_id_TDD_maxNrDLPhysicalchannels,
      { "id-TDD-maxNrDLPhysicalchannels", "rnsap.id_TDD_maxNrDLPhysicalchannels",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.MaxNrDLPhysicalchannelsTS", HFILL }},
    { &hf_rnsap_id_ExtendedGSMCellIndividualOffset,
      { "id-ExtendedGSMCellIndividualOffset", "rnsap.id_ExtendedGSMCellIndividualOffset",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.ExtendedGSMCellIndividualOffset", HFILL }},
    { &hf_rnsap_id_RL_ParameterUpdateIndicationFDD_RL_InformationList,
      { "id-RL-ParameterUpdateIndicationFDD-RL-InformationList", "rnsap.id_RL_ParameterUpdateIndicationFDD_RL_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RL_ParameterUpdateIndicationFDD_RL_InformationList", HFILL }},
    { &hf_rnsap_id_Primary_CPICH_Usage_For_Channel_Estimation,
      { "id-Primary-CPICH-Usage-For-Channel-Estimation", "rnsap.id_Primary_CPICH_Usage_For_Channel_Estimation",
        FT_UINT32, BASE_DEC, VALS(rnsap_Primary_CPICH_Usage_For_Channel_Estimation_vals), 0,
        "rnsap.Primary_CPICH_Usage_For_Channel_Estimation", HFILL }},
    { &hf_rnsap_id_Secondary_CPICH_Information,
      { "id-Secondary-CPICH-Information", "rnsap.id_Secondary_CPICH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Secondary_CPICH_Information", HFILL }},
    { &hf_rnsap_id_Secondary_CPICH_Information_Change,
      { "id-Secondary-CPICH-Information-Change", "rnsap.id_Secondary_CPICH_Information_Change",
        FT_UINT32, BASE_DEC, VALS(rnsap_Secondary_CPICH_Information_Change_vals), 0,
        "rnsap.Secondary_CPICH_Information_Change", HFILL }},
    { &hf_rnsap_id_RL_ParameterUpdateIndicationFDD_RL_Information_Item,
      { "id-RL-ParameterUpdateIndicationFDD-RL-Information-Item", "rnsap.id_RL_ParameterUpdateIndicationFDD_RL_Information_Item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_ParameterUpdateIndicationFDD_RL_Information_Item", HFILL }},
    { &hf_rnsap_id_Phase_Reference_Update_Indicator,
      { "id-Phase-Reference-Update-Indicator", "rnsap.id_Phase_Reference_Update_Indicator",
        FT_UINT32, BASE_DEC, VALS(rnsap_Phase_Reference_Update_Indicator_vals), 0,
        "rnsap.Phase_Reference_Update_Indicator", HFILL }},
    { &hf_rnsap_id_Unidirectional_DCH_Indicator,
      { "id-Unidirectional-DCH-Indicator", "rnsap.id_Unidirectional_DCH_Indicator",
        FT_UINT32, BASE_DEC, VALS(rnsap_Unidirectional_DCH_Indicator_vals), 0,
        "rnsap.Unidirectional_DCH_Indicator", HFILL }},
    { &hf_rnsap_id_RL_Information_RL_ReconfPrepTDD,
      { "id-RL-Information-RL-ReconfPrepTDD", "rnsap.id_RL_Information_RL_ReconfPrepTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RL_Information_RL_ReconfPrepTDD", HFILL }},
    { &hf_rnsap_id_Multiple_RL_InformationResponse_RL_ReconfReadyTDD,
      { "id-Multiple-RL-InformationResponse-RL-ReconfReadyTDD", "rnsap.id_Multiple_RL_InformationResponse_RL_ReconfReadyTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Multiple_RL_InformationResponse_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_id_RL_ReconfigurationResponseTDD_RL_Information,
      { "id-RL-ReconfigurationResponseTDD-RL-Information", "rnsap.id_RL_ReconfigurationResponseTDD_RL_Information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Multiple_RL_InformationResponse_RL_ReconfRspTDD", HFILL }},
    { &hf_rnsap_id_Satellite_Almanac_Information_ExtItem,
      { "id-Satellite-Almanac-Information-ExtItem", "rnsap.id_Satellite_Almanac_Information_ExtItem",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Satellite_Almanac_Information_ExtItem", HFILL }},
    { &hf_rnsap_id_HSDSCH_Information_to_Modify_Unsynchronised,
      { "id-HSDSCH-Information-to-Modify-Unsynchronised", "rnsap.id_HSDSCH_Information_to_Modify_Unsynchronised",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.HSDSCH_Information_to_Modify_Unsynchronised", HFILL }},
    { &hf_rnsap_id_TnlQos,
      { "id-TnlQos", "rnsap.id_TnlQos",
        FT_UINT32, BASE_DEC, VALS(rnsap_TnlQos_vals), 0,
        "rnsap.TnlQos", HFILL }},
    { &hf_rnsap_id_RTLoadValue,
      { "id-RTLoadValue", "rnsap.id_RTLoadValue",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RTLoadValue", HFILL }},
    { &hf_rnsap_id_NRTLoadInformationValue,
      { "id-NRTLoadInformationValue", "rnsap.id_NRTLoadInformationValue",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.NRTLoadInformationValue", HFILL }},
    { &hf_rnsap_id_CellPortionID,
      { "id-CellPortionID", "rnsap.id_CellPortionID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.CellPortionID", HFILL }},
    { &hf_rnsap_id_UpPTSInterferenceValue,
      { "id-UpPTSInterferenceValue", "rnsap.id_UpPTSInterferenceValue",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UpPTSInterferenceValue", HFILL }},
    { &hf_rnsap_id_PrimaryCCPCH_RSCP_Delta,
      { "id-PrimaryCCPCH-RSCP-Delta", "rnsap.id_PrimaryCCPCH_RSCP_Delta",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.PrimaryCCPCH_RSCP_Delta", HFILL }},
    { &hf_rnsap_id_UEMeasurementType,
      { "id-UEMeasurementType", "rnsap.id_UEMeasurementType",
        FT_UINT32, BASE_DEC, VALS(rnsap_UEMeasurementType_vals), 0,
        "rnsap.UEMeasurementType", HFILL }},
    { &hf_rnsap_id_UEMeasurementTimeslotInfoHCR,
      { "id-UEMeasurementTimeslotInfoHCR", "rnsap.id_UEMeasurementTimeslotInfoHCR",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UEMeasurementTimeslotInfoHCR", HFILL }},
    { &hf_rnsap_id_UEMeasurementTimeslotInfoLCR,
      { "id-UEMeasurementTimeslotInfoLCR", "rnsap.id_UEMeasurementTimeslotInfoLCR",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UEMeasurementTimeslotInfoLCR", HFILL }},
    { &hf_rnsap_id_UEMeasurementReportCharacteristics,
      { "id-UEMeasurementReportCharacteristics", "rnsap.id_UEMeasurementReportCharacteristics",
        FT_UINT32, BASE_DEC, VALS(rnsap_UEMeasurementReportCharacteristics_vals), 0,
        "rnsap.UEMeasurementReportCharacteristics", HFILL }},
    { &hf_rnsap_id_UEMeasurementParameterModAllow,
      { "id-UEMeasurementParameterModAllow", "rnsap.id_UEMeasurementParameterModAllow",
        FT_UINT32, BASE_DEC, VALS(rnsap_UEMeasurementParameterModAllow_vals), 0,
        "rnsap.UEMeasurementParameterModAllow", HFILL }},
    { &hf_rnsap_id_UEMeasurementValueInformation,
      { "id-UEMeasurementValueInformation", "rnsap.id_UEMeasurementValueInformation",
        FT_UINT32, BASE_DEC, VALS(rnsap_UEMeasurementValueInformation_vals), 0,
        "rnsap.UEMeasurementValueInformation", HFILL }},
    { &hf_rnsap_id_InterfacesToTraceItem,
      { "id-InterfacesToTraceItem", "rnsap.id_InterfacesToTraceItem",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.InterfacesToTraceItem", HFILL }},
    { &hf_rnsap_id_ListOfInterfacesToTrace,
      { "id-ListOfInterfacesToTrace", "rnsap.id_ListOfInterfacesToTrace",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.ListOfInterfacesToTrace", HFILL }},
    { &hf_rnsap_id_TraceDepth,
      { "id-TraceDepth", "rnsap.id_TraceDepth",
        FT_UINT32, BASE_DEC, VALS(rnsap_TraceDepth_vals), 0,
        "rnsap.TraceDepth", HFILL }},
    { &hf_rnsap_id_TraceRecordingSessionReference,
      { "id-TraceRecordingSessionReference", "rnsap.id_TraceRecordingSessionReference",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.TraceRecordingSessionReference", HFILL }},
    { &hf_rnsap_id_TraceReference,
      { "id-TraceReference", "rnsap.id_TraceReference",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.TraceReference", HFILL }},
    { &hf_rnsap_id_UEIdentity,
      { "id-UEIdentity", "rnsap.id_UEIdentity",
        FT_UINT32, BASE_DEC, VALS(rnsap_UEIdentity_vals), 0,
        "rnsap.UEIdentity", HFILL }},
    { &hf_rnsap_id_NACC_Related_Data,
      { "id-NACC-Related-Data", "rnsap.id_NACC_Related_Data",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.NACC_Related_Data", HFILL }},
    { &hf_rnsap_id_GSM_Cell_InfEx_Rqst,
      { "id-GSM-Cell-InfEx-Rqst", "rnsap.id_GSM_Cell_InfEx_Rqst",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.GSM_Cell_InfEx_Rqst", HFILL }},
    { &hf_rnsap_id_MeasurementRecoveryBehavior,
      { "id-MeasurementRecoveryBehavior", "rnsap.id_MeasurementRecoveryBehavior",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.MeasurementRecoveryBehavior", HFILL }},
    { &hf_rnsap_id_MeasurementRecoveryReportingIndicator,
      { "id-MeasurementRecoveryReportingIndicator", "rnsap.id_MeasurementRecoveryReportingIndicator",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.MeasurementRecoveryReportingIndicator", HFILL }},
    { &hf_rnsap_id_MeasurementRecoverySupportIndicator,
      { "id-MeasurementRecoverySupportIndicator", "rnsap.id_MeasurementRecoverySupportIndicator",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.MeasurementRecoverySupportIndicator", HFILL }},
    { &hf_rnsap_id_DL_DPCH_Power_Information_RL_ReconfPrepFDD,
      { "id-DL-DPCH-Power-Information-RL-ReconfPrepFDD", "rnsap.id_DL_DPCH_Power_Information_RL_ReconfPrepFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_DPCH_Power_Information_RL_ReconfPrepFDD", HFILL }},
    { &hf_rnsap_id_F_DPCH_Information_RL_ReconfPrepFDD,
      { "id-F-DPCH-Information-RL-ReconfPrepFDD", "rnsap.id_F_DPCH_Information_RL_ReconfPrepFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.F_DPCH_Information_RL_ReconfPrepFDD", HFILL }},
    { &hf_rnsap_id_F_DPCH_Information_RL_SetupRqstFDD,
      { "id-F-DPCH-Information-RL-SetupRqstFDD", "rnsap.id_F_DPCH_Information_RL_SetupRqstFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.F_DPCH_Information_RL_SetupRqstFDD", HFILL }},
    { &hf_rnsap_id_MBMS_Bearer_Service_List,
      { "id-MBMS-Bearer-Service-List", "rnsap.id_MBMS_Bearer_Service_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.MBMS_Bearer_Service_List", HFILL }},
    { &hf_rnsap_id_MBMS_Bearer_Service_List_InfEx_Rsp,
      { "id-MBMS-Bearer-Service-List-InfEx-Rsp", "rnsap.id_MBMS_Bearer_Service_List_InfEx_Rsp",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.MBMS_Bearer_Service_List_InfEx_Rsp", HFILL }},
    { &hf_rnsap_id_Active_MBMS_Bearer_ServiceFDD,
      { "id-Active-MBMS-Bearer-ServiceFDD", "rnsap.id_Active_MBMS_Bearer_ServiceFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Active_MBMS_Bearer_Service_ListFDD", HFILL }},
    { &hf_rnsap_id_Active_MBMS_Bearer_ServiceTDD,
      { "id-Active-MBMS-Bearer-ServiceTDD", "rnsap.id_Active_MBMS_Bearer_ServiceTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Active_MBMS_Bearer_Service_ListTDD", HFILL }},
    { &hf_rnsap_id_Old_URA_ID,
      { "id-Old-URA-ID", "rnsap.id_Old_URA_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.URA_ID", HFILL }},
    { &hf_rnsap_id_UE_State,
      { "id-UE-State", "rnsap.id_UE_State",
        FT_UINT32, BASE_DEC, VALS(rnsap_UE_State_vals), 0,
        "rnsap.UE_State", HFILL }},
    { &hf_rnsap_id_URA_ID,
      { "id-URA-ID", "rnsap.id_URA_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.URA_ID", HFILL }},
    { &hf_rnsap_id_HARQ_Preamble_Mode,
      { "id-HARQ-Preamble-Mode", "rnsap.id_HARQ_Preamble_Mode",
        FT_UINT32, BASE_DEC, VALS(rnsap_HARQ_Preamble_Mode_vals), 0,
        "rnsap.HARQ_Preamble_Mode", HFILL }},
    { &hf_rnsap_id_SynchronisationIndicator,
      { "id-SynchronisationIndicator", "rnsap.id_SynchronisationIndicator",
        FT_UINT32, BASE_DEC, VALS(rnsap_SynchronisationIndicator_vals), 0,
        "rnsap.SynchronisationIndicator", HFILL }},
    { &hf_rnsap_id_UL_DPDCHIndicatorEDCH,
      { "id-UL-DPDCHIndicatorEDCH", "rnsap.id_UL_DPDCHIndicatorEDCH",
        FT_UINT32, BASE_DEC, VALS(rnsap_UL_DPDCHIndicatorEDCH_vals), 0,
        "rnsap.UL_DPDCHIndicatorEDCH", HFILL }},
    { &hf_rnsap_id_EDPCH_Information,
      { "id-EDPCH-Information", "rnsap.id_EDPCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.EDPCH_Information_FDD", HFILL }},
    { &hf_rnsap_id_RL_Specific_EDCH_Information,
      { "id-RL-Specific-EDCH-Information", "rnsap.id_RL_Specific_EDCH_Information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RL_Specific_EDCH_Information", HFILL }},
    { &hf_rnsap_id_EDCH_RL_Indication,
      { "id-EDCH-RL-Indication", "rnsap.id_EDCH_RL_Indication",
        FT_UINT32, BASE_DEC, VALS(rnsap_EDCH_RL_Indication_vals), 0,
        "rnsap.EDCH_RL_Indication", HFILL }},
    { &hf_rnsap_id_EDCH_FDD_Information,
      { "id-EDCH-FDD-Information", "rnsap.id_EDCH_FDD_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.EDCH_FDD_Information", HFILL }},
    { &hf_rnsap_id_EDCH_RLSet_Id,
      { "id-EDCH-RLSet-Id", "rnsap.id_EDCH_RLSet_Id",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RL_Set_ID", HFILL }},
    { &hf_rnsap_id_Serving_EDCHRL_Id,
      { "id-Serving-EDCHRL-Id", "rnsap.id_Serving_EDCHRL_Id",
        FT_UINT32, BASE_DEC, VALS(rnsap_EDCH_Serving_RL_vals), 0,
        "rnsap.EDCH_Serving_RL", HFILL }},
    { &hf_rnsap_id_EDCH_FDD_DL_ControlChannelInformation,
      { "id-EDCH-FDD-DL-ControlChannelInformation", "rnsap.id_EDCH_FDD_DL_ControlChannelInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.EDCH_FDD_DL_ControlChannelInformation", HFILL }},
    { &hf_rnsap_id_EDCH_FDD_InformationResponse,
      { "id-EDCH-FDD-InformationResponse", "rnsap.id_EDCH_FDD_InformationResponse",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.EDCH_FDD_InformationResponse", HFILL }},
    { &hf_rnsap_id_EDCH_MACdFlows_To_Add,
      { "id-EDCH-MACdFlows-To-Add", "rnsap.id_EDCH_MACdFlows_To_Add",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RL_Specific_EDCH_Information", HFILL }},
    { &hf_rnsap_id_EDCH_FDD_Information_To_Modify,
      { "id-EDCH-FDD-Information-To-Modify", "rnsap.id_EDCH_FDD_Information_To_Modify",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.EDCH_FDD_Information_To_Modify", HFILL }},
    { &hf_rnsap_id_EDCH_MACdFlows_To_Delete,
      { "id-EDCH-MACdFlows-To-Delete", "rnsap.id_EDCH_MACdFlows_To_Delete",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.EDCH_MACdFlows_To_Delete", HFILL }},
    { &hf_rnsap_id_EDPCH_Information_RLReconfRequest_FDD,
      { "id-EDPCH-Information-RLReconfRequest-FDD", "rnsap.id_EDPCH_Information_RLReconfRequest_FDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.EDPCH_Information_RLReconfRequest_FDD", HFILL }},
    { &hf_rnsap_id_EDCH_MacdFlowSpecificInformationList_RL_PreemptRequiredInd,
      { "id-EDCH-MacdFlowSpecificInformationList-RL-PreemptRequiredInd", "rnsap.id_EDCH_MacdFlowSpecificInformationList_RL_PreemptRequiredInd",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.EDCH_MacdFlowSpecificInformationList_RL_PreemptRequiredInd", HFILL }},
    { &hf_rnsap_id_EDCH_MacdFlowSpecificInformationItem_RL_PreemptRequiredInd,
      { "id-EDCH-MacdFlowSpecificInformationItem-RL-PreemptRequiredInd", "rnsap.id_EDCH_MacdFlowSpecificInformationItem_RL_PreemptRequiredInd",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.EDCH_MacdFlowSpecificInformationItem_RL_PreemptRequiredInd", HFILL }},
    { &hf_rnsap_id_EDCH_MacdFlowSpecificInformationList_RL_CongestInd,
      { "id-EDCH-MacdFlowSpecificInformationList-RL-CongestInd", "rnsap.id_EDCH_MacdFlowSpecificInformationList_RL_CongestInd",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.EDCH_MacdFlowSpecificInformationList_RL_CongestInd", HFILL }},
    { &hf_rnsap_id_EDCH_MacdFlowSpecificInformationItem_RL_CongestInd,
      { "id-EDCH-MacdFlowSpecificInformationItem-RL-CongestInd", "rnsap.id_EDCH_MacdFlowSpecificInformationItem_RL_CongestInd",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.EDCH_MacdFlowSpecificInformationItem_RL_CongestInd", HFILL }},
    { &hf_rnsap_id_MBMS_Bearer_Service_Full_Address,
      { "id-MBMS-Bearer-Service-Full-Address", "rnsap.id_MBMS_Bearer_Service_Full_Address",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.MBMS_Bearer_Service_Full_Address", HFILL }},
    { &hf_rnsap_id_Initial_DL_DPCH_TimingAdjustment,
      { "id-Initial-DL-DPCH-TimingAdjustment", "rnsap.id_Initial_DL_DPCH_TimingAdjustment",
        FT_UINT32, BASE_DEC, VALS(rnsap_DL_DPCH_TimingAdjustment_vals), 0,
        "rnsap.DL_DPCH_TimingAdjustment", HFILL }},
    { &hf_rnsap_id_Initial_DL_DPCH_TimingAdjustment_Allowed,
      { "id-Initial-DL-DPCH-TimingAdjustment-Allowed", "rnsap.id_Initial_DL_DPCH_TimingAdjustment_Allowed",
        FT_UINT32, BASE_DEC, VALS(rnsap_Initial_DL_DPCH_TimingAdjustment_Allowed_vals), 0,
        "rnsap.Initial_DL_DPCH_TimingAdjustment_Allowed", HFILL }},
    { &hf_rnsap_id_User_Plane_Congestion_Fields_Inclusion,
      { "id-User-Plane-Congestion-Fields-Inclusion", "rnsap.id_User_Plane_Congestion_Fields_Inclusion",
        FT_UINT32, BASE_DEC, VALS(rnsap_User_Plane_Congestion_Fields_Inclusion_vals), 0,
        "rnsap.User_Plane_Congestion_Fields_Inclusion", HFILL }},
    { &hf_rnsap_id_HARQ_Preamble_Mode_Activation_Indicator,
      { "id-HARQ-Preamble-Mode-Activation-Indicator", "rnsap.id_HARQ_Preamble_Mode_Activation_Indicator",
        FT_UINT32, BASE_DEC, VALS(rnsap_HARQ_Preamble_Mode_Activation_Indicator_vals), 0,
        "rnsap.HARQ_Preamble_Mode_Activation_Indicator", HFILL }},
    { &hf_rnsap_id_multiple_DedicatedMeasurementValueList_TDD_DM_Rsp,
      { "id-multiple-DedicatedMeasurementValueList-TDD-DM-Rsp", "rnsap.id_multiple_DedicatedMeasurementValueList_TDD_DM_Rsp",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Multiple_DedicatedMeasurementValueList_TDD_DM_Rsp", HFILL }},
    { &hf_rnsap_id_multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp,
      { "id-multiple-DedicatedMeasurementValueList-LCR-TDD-DM-Rsp", "rnsap.id_multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp", HFILL }},
    { &hf_rnsap_id_ProvidedInformation,
      { "id-ProvidedInformation", "rnsap.id_ProvidedInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProvidedInformation", HFILL }},
    { &hf_rnsap_id_Active_MBMS_Bearer_ServiceFDD_PFL,
      { "id-Active-MBMS-Bearer-ServiceFDD-PFL", "rnsap.id_Active_MBMS_Bearer_ServiceFDD_PFL",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Active_MBMS_Bearer_Service_ListTDD_PFL", HFILL }},
    { &hf_rnsap_id_Active_MBMS_Bearer_ServiceTDD_PFL,
      { "id-Active-MBMS-Bearer-ServiceTDD-PFL", "rnsap.id_Active_MBMS_Bearer_ServiceTDD_PFL",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Active_MBMS_Bearer_Service_ListTDD_PFL", HFILL }},
    { &hf_rnsap_id_FrequencyBandIndicator,
      { "id-FrequencyBandIndicator", "rnsap.id_FrequencyBandIndicator",
        FT_UINT32, BASE_DEC, VALS(rnsap_FrequencyBandIndicator_vals), 0,
        "rnsap.FrequencyBandIndicator", HFILL }},
    { &hf_rnsap_protocolIEs,
      { "protocolIEs", "rnsap.protocolIEs",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.ProtocolIE_Container", HFILL }},
    { &hf_rnsap_protocolExtensions,
      { "protocolExtensions", "rnsap.protocolExtensions",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.ProtocolExtensionContainer", HFILL }},
    { &hf_rnsap_ul_ScramblingCode,
      { "ul-ScramblingCode", "rnsap.ul_ScramblingCode",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_ScramblingCode", HFILL }},
    { &hf_rnsap_minUL_ChannelisationCodeLength,
      { "minUL-ChannelisationCodeLength", "rnsap.minUL_ChannelisationCodeLength",
        FT_UINT32, BASE_DEC, VALS(rnsap_MinUL_ChannelisationCodeLength_vals), 0,
        "rnsap.MinUL_ChannelisationCodeLength", HFILL }},
    { &hf_rnsap_maxNrOfUL_DPCHs,
      { "maxNrOfUL-DPCHs", "rnsap.maxNrOfUL_DPCHs",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.MaxNrOfUL_DPCHs", HFILL }},
    { &hf_rnsap_ul_PunctureLimit,
      { "ul-PunctureLimit", "rnsap.ul_PunctureLimit",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.PunctureLimit", HFILL }},
    { &hf_rnsap_ul_TFCS,
      { "ul-TFCS", "rnsap.ul_TFCS",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.TFCS", HFILL }},
    { &hf_rnsap_ul_DPCCH_SlotFormat,
      { "ul-DPCCH-SlotFormat", "rnsap.ul_DPCCH_SlotFormat",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UL_DPCCH_SlotFormat", HFILL }},
    { &hf_rnsap_ul_SIRTarget,
      { "ul-SIRTarget", "rnsap.ul_SIRTarget",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.UL_SIR", HFILL }},
    { &hf_rnsap_diversityMode,
      { "diversityMode", "rnsap.diversityMode",
        FT_UINT32, BASE_DEC, VALS(rnsap_DiversityMode_vals), 0,
        "rnsap.DiversityMode", HFILL }},
    { &hf_rnsap_not_Used_sSDT_CellIdLength,
      { "not-Used-sSDT-CellIdLength", "rnsap.not_Used_sSDT_CellIdLength",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.NULL", HFILL }},
    { &hf_rnsap_not_Used_s_FieldLength,
      { "not-Used-s-FieldLength", "rnsap.not_Used_s_FieldLength",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.NULL", HFILL }},
    { &hf_rnsap_iE_Extensions,
      { "iE-Extensions", "rnsap.iE_Extensions",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.ProtocolExtensionContainer", HFILL }},
    { &hf_rnsap_tFCS,
      { "tFCS", "rnsap.tFCS",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.TFCS", HFILL }},
    { &hf_rnsap_dl_DPCH_SlotFormat,
      { "dl-DPCH-SlotFormat", "rnsap.dl_DPCH_SlotFormat",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DL_DPCH_SlotFormat", HFILL }},
    { &hf_rnsap_nrOfDLchannelisationcodes,
      { "nrOfDLchannelisationcodes", "rnsap.nrOfDLchannelisationcodes",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.NrOfDLchannelisationcodes", HFILL }},
    { &hf_rnsap_tFCI_SignallingMode,
      { "tFCI-SignallingMode", "rnsap.tFCI_SignallingMode",
        FT_UINT32, BASE_DEC, VALS(rnsap_TFCI_SignallingMode_vals), 0,
        "rnsap.TFCI_SignallingMode", HFILL }},
    { &hf_rnsap_tFCI_Presence,
      { "tFCI-Presence", "rnsap.tFCI_Presence",
        FT_UINT32, BASE_DEC, VALS(rnsap_TFCI_Presence_vals), 0,
        "rnsap.TFCI_Presence", HFILL }},
    { &hf_rnsap_multiplexingPosition,
      { "multiplexingPosition", "rnsap.multiplexingPosition",
        FT_UINT32, BASE_DEC, VALS(rnsap_MultiplexingPosition_vals), 0,
        "rnsap.MultiplexingPosition", HFILL }},
    { &hf_rnsap_powerOffsetInformation,
      { "powerOffsetInformation", "rnsap.powerOffsetInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.PowerOffsetInformation_RL_SetupRqstFDD", HFILL }},
    { &hf_rnsap_fdd_dl_TPC_DownlinkStepSize,
      { "fdd-dl-TPC-DownlinkStepSize", "rnsap.fdd_dl_TPC_DownlinkStepSize",
        FT_UINT32, BASE_DEC, VALS(rnsap_FDD_TPC_DownlinkStepSize_vals), 0,
        "rnsap.FDD_TPC_DownlinkStepSize", HFILL }},
    { &hf_rnsap_limitedPowerIncrease,
      { "limitedPowerIncrease", "rnsap.limitedPowerIncrease",
        FT_UINT32, BASE_DEC, VALS(rnsap_LimitedPowerIncrease_vals), 0,
        "rnsap.LimitedPowerIncrease", HFILL }},
    { &hf_rnsap_innerLoopDLPCStatus,
      { "innerLoopDLPCStatus", "rnsap.innerLoopDLPCStatus",
        FT_UINT32, BASE_DEC, VALS(rnsap_InnerLoopDLPCStatus_vals), 0,
        "rnsap.InnerLoopDLPCStatus", HFILL }},
    { &hf_rnsap_po1_ForTFCI_Bits,
      { "po1-ForTFCI-Bits", "rnsap.po1_ForTFCI_Bits",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.PowerOffset", HFILL }},
    { &hf_rnsap_po2_ForTPC_Bits,
      { "po2-ForTPC-Bits", "rnsap.po2_ForTPC_Bits",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.PowerOffset", HFILL }},
    { &hf_rnsap_po3_ForPilotBits,
      { "po3-ForPilotBits", "rnsap.po3_ForPilotBits",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.PowerOffset", HFILL }},
    { &hf_rnsap_RL_InformationList_RL_SetupRqstFDD_item,
      { "Item", "rnsap.RL_InformationList_RL_SetupRqstFDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_rL_ID,
      { "rL-ID", "rnsap.rL_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RL_ID", HFILL }},
    { &hf_rnsap_c_ID,
      { "c-ID", "rnsap.c_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.C_ID", HFILL }},
    { &hf_rnsap_firstRLS_indicator,
      { "firstRLS-indicator", "rnsap.firstRLS_indicator",
        FT_UINT32, BASE_DEC, VALS(rnsap_FirstRLS_Indicator_vals), 0,
        "rnsap.FirstRLS_Indicator", HFILL }},
    { &hf_rnsap_frameOffset,
      { "frameOffset", "rnsap.frameOffset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.FrameOffset", HFILL }},
    { &hf_rnsap_chipOffset,
      { "chipOffset", "rnsap.chipOffset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.ChipOffset", HFILL }},
    { &hf_rnsap_propagationDelay,
      { "propagationDelay", "rnsap.propagationDelay",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.PropagationDelay", HFILL }},
    { &hf_rnsap_diversityControlField,
      { "diversityControlField", "rnsap.diversityControlField",
        FT_UINT32, BASE_DEC, VALS(rnsap_DiversityControlField_vals), 0,
        "rnsap.DiversityControlField", HFILL }},
    { &hf_rnsap_dl_InitialTX_Power,
      { "dl-InitialTX-Power", "rnsap.dl_InitialTX_Power",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.DL_Power", HFILL }},
    { &hf_rnsap_primaryCPICH_EcNo,
      { "primaryCPICH-EcNo", "rnsap.primaryCPICH_EcNo",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.PrimaryCPICH_EcNo", HFILL }},
    { &hf_rnsap_not_Used_sSDT_CellID,
      { "not-Used-sSDT-CellID", "rnsap.not_Used_sSDT_CellID",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.NULL", HFILL }},
    { &hf_rnsap_transmitDiversityIndicator,
      { "transmitDiversityIndicator", "rnsap.transmitDiversityIndicator",
        FT_UINT32, BASE_DEC, VALS(rnsap_TransmitDiversityIndicator_vals), 0,
        "rnsap.TransmitDiversityIndicator", HFILL }},
    { &hf_rnsap_powerOffsetInformation1,
      { "powerOffsetInformation", "rnsap.powerOffsetInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.PowerOffsetInformation_F_DPCH_RL_SetupRqstFDD", HFILL }},
    { &hf_rnsap_maxNrTimeslots_UL,
      { "maxNrTimeslots-UL", "rnsap.maxNrTimeslots_UL",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.MaxNrTimeslots", HFILL }},
    { &hf_rnsap_minimumSpreadingFactor_UL,
      { "minimumSpreadingFactor-UL", "rnsap.minimumSpreadingFactor_UL",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.MinimumSpreadingFactor", HFILL }},
    { &hf_rnsap_maxNrULPhysicalchannels,
      { "maxNrULPhysicalchannels", "rnsap.maxNrULPhysicalchannels",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.MaxNrULPhysicalchannels", HFILL }},
    { &hf_rnsap_maxNrTimeslots_DL,
      { "maxNrTimeslots-DL", "rnsap.maxNrTimeslots_DL",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.MaxNrTimeslots", HFILL }},
    { &hf_rnsap_minimumSpreadingFactor_DL,
      { "minimumSpreadingFactor-DL", "rnsap.minimumSpreadingFactor_DL",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.MinimumSpreadingFactor", HFILL }},
    { &hf_rnsap_maxNrDLPhysicalchannels,
      { "maxNrDLPhysicalchannels", "rnsap.maxNrDLPhysicalchannels",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.MaxNrDLPhysicalchannels", HFILL }},
    { &hf_rnsap_UL_CCTrCH_InformationList_RL_SetupRqstTDD_item,
      { "Item", "rnsap.UL_CCTrCH_InformationList_RL_SetupRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_cCTrCH_ID,
      { "cCTrCH-ID", "rnsap.cCTrCH_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.CCTrCH_ID", HFILL }},
    { &hf_rnsap_tFCI_Coding,
      { "tFCI-Coding", "rnsap.tFCI_Coding",
        FT_UINT32, BASE_DEC, VALS(rnsap_TFCI_Coding_vals), 0,
        "rnsap.TFCI_Coding", HFILL }},
    { &hf_rnsap_DL_CCTrCH_InformationList_RL_SetupRqstTDD_item,
      { "Item", "rnsap.DL_CCTrCH_InformationList_RL_SetupRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_dl_TFCS,
      { "dl-TFCS", "rnsap.dl_TFCS",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.TFCS", HFILL }},
    { &hf_rnsap_dl_PunctureLimit,
      { "dl-PunctureLimit", "rnsap.dl_PunctureLimit",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.PunctureLimit", HFILL }},
    { &hf_rnsap_tdd_TPC_DownlinkStepSize,
      { "tdd-TPC-DownlinkStepSize", "rnsap.tdd_TPC_DownlinkStepSize",
        FT_UINT32, BASE_DEC, VALS(rnsap_TDD_TPC_DownlinkStepSize_vals), 0,
        "rnsap.TDD_TPC_DownlinkStepSize", HFILL }},
    { &hf_rnsap_cCTrCH_TPCList,
      { "cCTrCH-TPCList", "rnsap.cCTrCH_TPCList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.CCTrCH_TPCList_RL_SetupRqstTDD", HFILL }},
    { &hf_rnsap_CCTrCH_TPCList_RL_SetupRqstTDD_item,
      { "Item", "rnsap.CCTrCH_TPCList_RL_SetupRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.CCTrCH_TPCItem_RL_SetupRqstTDD", HFILL }},
    { &hf_rnsap_specialBurstScheduling,
      { "specialBurstScheduling", "rnsap.specialBurstScheduling",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.SpecialBurstScheduling", HFILL }},
    { &hf_rnsap_primaryCCPCH_RSCP,
      { "primaryCCPCH-RSCP", "rnsap.primaryCCPCH_RSCP",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.PrimaryCCPCH_RSCP", HFILL }},
    { &hf_rnsap_dL_TimeSlot_ISCP,
      { "dL-TimeSlot-ISCP", "rnsap.dL_TimeSlot_ISCP",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DL_TimeSlot_ISCP_Info", HFILL }},
    { &hf_rnsap_RL_InformationResponseList_RL_SetupRspFDD_item,
      { "Item", "rnsap.RL_InformationResponseList_RL_SetupRspFDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_rL_Set_ID,
      { "rL-Set-ID", "rnsap.rL_Set_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RL_Set_ID", HFILL }},
    { &hf_rnsap_uRA_Information,
      { "uRA-Information", "rnsap.uRA_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.URA_Information", HFILL }},
    { &hf_rnsap_sAI,
      { "sAI", "rnsap.sAI",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.SAI", HFILL }},
    { &hf_rnsap_gA_Cell,
      { "gA-Cell", "rnsap.gA_Cell",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.GA_Cell", HFILL }},
    { &hf_rnsap_gA_AccessPointPosition,
      { "gA-AccessPointPosition", "rnsap.gA_AccessPointPosition",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.GA_AccessPointPosition", HFILL }},
    { &hf_rnsap_received_total_wide_band_power,
      { "received-total-wide-band-power", "rnsap.received_total_wide_band_power",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Received_total_wide_band_power", HFILL }},
    { &hf_rnsap_not_Used_secondary_CCPCH_Info,
      { "not-Used-secondary-CCPCH-Info", "rnsap.not_Used_secondary_CCPCH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.NULL", HFILL }},
    { &hf_rnsap_dl_CodeInformation,
      { "dl-CodeInformation", "rnsap.dl_CodeInformation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.FDD_DL_CodeInformation", HFILL }},
    { &hf_rnsap_diversityIndication,
      { "diversityIndication", "rnsap.diversityIndication",
        FT_UINT32, BASE_DEC, VALS(rnsap_DiversityIndication_RL_SetupRspFDD_vals), 0,
        "rnsap.DiversityIndication_RL_SetupRspFDD", HFILL }},
    { &hf_rnsap_sSDT_SupportIndicator,
      { "sSDT-SupportIndicator", "rnsap.sSDT_SupportIndicator",
        FT_UINT32, BASE_DEC, VALS(rnsap_SSDT_SupportIndicator_vals), 0,
        "rnsap.SSDT_SupportIndicator", HFILL }},
    { &hf_rnsap_maxUL_SIR,
      { "maxUL-SIR", "rnsap.maxUL_SIR",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.UL_SIR", HFILL }},
    { &hf_rnsap_minUL_SIR,
      { "minUL-SIR", "rnsap.minUL_SIR",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.UL_SIR", HFILL }},
    { &hf_rnsap_closedlooptimingadjustmentmode,
      { "closedlooptimingadjustmentmode", "rnsap.closedlooptimingadjustmentmode",
        FT_UINT32, BASE_DEC, VALS(rnsap_Closedlooptimingadjustmentmode_vals), 0,
        "rnsap.Closedlooptimingadjustmentmode", HFILL }},
    { &hf_rnsap_maximumAllowedULTxPower,
      { "maximumAllowedULTxPower", "rnsap.maximumAllowedULTxPower",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.MaximumAllowedULTxPower", HFILL }},
    { &hf_rnsap_maximumDLTxPower,
      { "maximumDLTxPower", "rnsap.maximumDLTxPower",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.DL_Power", HFILL }},
    { &hf_rnsap_minimumDLTxPower,
      { "minimumDLTxPower", "rnsap.minimumDLTxPower",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.DL_Power", HFILL }},
    { &hf_rnsap_primaryScramblingCode,
      { "primaryScramblingCode", "rnsap.primaryScramblingCode",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.PrimaryScramblingCode", HFILL }},
    { &hf_rnsap_uL_UARFCN,
      { "uL-UARFCN", "rnsap.uL_UARFCN",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UARFCN", HFILL }},
    { &hf_rnsap_dL_UARFCN,
      { "dL-UARFCN", "rnsap.dL_UARFCN",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UARFCN", HFILL }},
    { &hf_rnsap_primaryCPICH_Power,
      { "primaryCPICH-Power", "rnsap.primaryCPICH_Power",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.PrimaryCPICH_Power", HFILL }},
    { &hf_rnsap_not_Used_dSCHInformationResponse,
      { "not-Used-dSCHInformationResponse", "rnsap.not_Used_dSCHInformationResponse",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.NULL", HFILL }},
    { &hf_rnsap_neighbouring_UMTS_CellInformation,
      { "neighbouring-UMTS-CellInformation", "rnsap.neighbouring_UMTS_CellInformation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Neighbouring_UMTS_CellInformation", HFILL }},
    { &hf_rnsap_neighbouring_GSM_CellInformation,
      { "neighbouring-GSM-CellInformation", "rnsap.neighbouring_GSM_CellInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Neighbouring_GSM_CellInformation", HFILL }},
    { &hf_rnsap_pC_Preamble,
      { "pC-Preamble", "rnsap.pC_Preamble",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.PC_Preamble", HFILL }},
    { &hf_rnsap_sRB_Delay,
      { "sRB-Delay", "rnsap.sRB_Delay",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.SRB_Delay", HFILL }},
    { &hf_rnsap_combining,
      { "combining", "rnsap.combining",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Combining_RL_SetupRspFDD", HFILL }},
    { &hf_rnsap_nonCombiningOrFirstRL,
      { "nonCombiningOrFirstRL", "rnsap.nonCombiningOrFirstRL",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.NonCombiningOrFirstRL_RL_SetupRspFDD", HFILL }},
    { &hf_rnsap_dCH_InformationResponse,
      { "dCH-InformationResponse", "rnsap.dCH_InformationResponse",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DCH_InformationResponse", HFILL }},
    { &hf_rnsap_ul_TimeSlot_ISCP_Info,
      { "ul-TimeSlot-ISCP-Info", "rnsap.ul_TimeSlot_ISCP_Info",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UL_TimeSlot_ISCP_Info", HFILL }},
    { &hf_rnsap_uARFCNforNt,
      { "uARFCNforNt", "rnsap.uARFCNforNt",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UARFCN", HFILL }},
    { &hf_rnsap_cellParameterID,
      { "cellParameterID", "rnsap.cellParameterID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.CellParameterID", HFILL }},
    { &hf_rnsap_syncCase,
      { "syncCase", "rnsap.syncCase",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.SyncCase", HFILL }},
    { &hf_rnsap_sCH_TimeSlot,
      { "sCH-TimeSlot", "rnsap.sCH_TimeSlot",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.SCH_TimeSlot", HFILL }},
    { &hf_rnsap_sCTD_Indicator,
      { "sCTD-Indicator", "rnsap.sCTD_Indicator",
        FT_UINT32, BASE_DEC, VALS(rnsap_SCTD_Indicator_vals), 0,
        "rnsap.SCTD_Indicator", HFILL }},
    { &hf_rnsap_pCCPCH_Power,
      { "pCCPCH-Power", "rnsap.pCCPCH_Power",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.PCCPCH_Power", HFILL }},
    { &hf_rnsap_timingAdvanceApplied,
      { "timingAdvanceApplied", "rnsap.timingAdvanceApplied",
        FT_UINT32, BASE_DEC, VALS(rnsap_TimingAdvanceApplied_vals), 0,
        "rnsap.TimingAdvanceApplied", HFILL }},
    { &hf_rnsap_alphaValue,
      { "alphaValue", "rnsap.alphaValue",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.AlphaValue", HFILL }},
    { &hf_rnsap_ul_PhysCH_SF_Variation,
      { "ul-PhysCH-SF-Variation", "rnsap.ul_PhysCH_SF_Variation",
        FT_UINT32, BASE_DEC, VALS(rnsap_UL_PhysCH_SF_Variation_vals), 0,
        "rnsap.UL_PhysCH_SF_Variation", HFILL }},
    { &hf_rnsap_synchronisationConfiguration,
      { "synchronisationConfiguration", "rnsap.synchronisationConfiguration",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.SynchronisationConfiguration", HFILL }},
    { &hf_rnsap_secondary_CCPCH_Info_TDD,
      { "secondary-CCPCH-Info-TDD", "rnsap.secondary_CCPCH_Info_TDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Secondary_CCPCH_Info_TDD", HFILL }},
    { &hf_rnsap_ul_CCTrCHInformation,
      { "ul-CCTrCHInformation", "rnsap.ul_CCTrCHInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_CCTrCHInformationList_RL_SetupRspTDD", HFILL }},
    { &hf_rnsap_dl_CCTrCHInformation,
      { "dl-CCTrCHInformation", "rnsap.dl_CCTrCHInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_CCTrCHInformationList_RL_SetupRspTDD", HFILL }},
    { &hf_rnsap_dCH_InformationResponse1,
      { "dCH-InformationResponse", "rnsap.dCH_InformationResponse",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DCH_InformationResponseList_RL_SetupRspTDD", HFILL }},
    { &hf_rnsap_dsch_InformationResponse,
      { "dsch-InformationResponse", "rnsap.dsch_InformationResponse",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DSCH_InformationResponse_RL_SetupRspTDD", HFILL }},
    { &hf_rnsap_usch_InformationResponse,
      { "usch-InformationResponse", "rnsap.usch_InformationResponse",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.USCH_InformationResponse_RL_SetupRspTDD", HFILL }},
    { &hf_rnsap_UL_CCTrCHInformationListIE_RL_SetupRspTDD_item,
      { "Item", "rnsap.UL_CCTrCHInformationListIE_RL_SetupRspTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_CCTrCHInformationItem_RL_SetupRspTDD", HFILL }},
    { &hf_rnsap_ul_DPCH_Information,
      { "ul-DPCH-Information", "rnsap.ul_DPCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_DPCH_InformationList_RL_SetupRspTDD", HFILL }},
    { &hf_rnsap_repetitionPeriod,
      { "repetitionPeriod", "rnsap.repetitionPeriod",
        FT_UINT32, BASE_DEC, VALS(rnsap_RepetitionPeriod_vals), 0,
        "rnsap.RepetitionPeriod", HFILL }},
    { &hf_rnsap_repetitionLength,
      { "repetitionLength", "rnsap.repetitionLength",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RepetitionLength", HFILL }},
    { &hf_rnsap_tDD_DPCHOffset,
      { "tDD-DPCHOffset", "rnsap.tDD_DPCHOffset",
        FT_UINT32, BASE_DEC, VALS(rnsap_TDD_DPCHOffset_vals), 0,
        "rnsap.TDD_DPCHOffset", HFILL }},
    { &hf_rnsap_uL_Timeslot_Information,
      { "uL-Timeslot-Information", "rnsap.uL_Timeslot_Information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UL_Timeslot_Information", HFILL }},
    { &hf_rnsap_DL_CCTrCHInformationListIE_RL_SetupRspTDD_item,
      { "Item", "rnsap.DL_CCTrCHInformationListIE_RL_SetupRspTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_CCTrCHInformationItem_RL_SetupRspTDD", HFILL }},
    { &hf_rnsap_dl_DPCH_Information,
      { "dl-DPCH-Information", "rnsap.dl_DPCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_DPCH_InformationList_RL_SetupRspTDD", HFILL }},
    { &hf_rnsap_dL_Timeslot_Information,
      { "dL-Timeslot-Information", "rnsap.dL_Timeslot_Information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DL_Timeslot_Information", HFILL }},
    { &hf_rnsap_DSCH_InformationListIEs_RL_SetupRspTDD_item,
      { "Item", "rnsap.DSCH_InformationListIEs_RL_SetupRspTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DSCHInformationItem_RL_SetupRspTDD", HFILL }},
    { &hf_rnsap_dsch_ID,
      { "dsch-ID", "rnsap.dsch_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DSCH_ID", HFILL }},
    { &hf_rnsap_dSCH_FlowControlInformation,
      { "dSCH-FlowControlInformation", "rnsap.dSCH_FlowControlInformation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DSCH_FlowControlInformation", HFILL }},
    { &hf_rnsap_bindingID,
      { "bindingID", "rnsap.bindingID",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BindingID", HFILL }},
    { &hf_rnsap_transportLayerAddress,
      { "transportLayerAddress", "rnsap.transportLayerAddress",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.TransportLayerAddress", HFILL }},
    { &hf_rnsap_transportFormatManagement,
      { "transportFormatManagement", "rnsap.transportFormatManagement",
        FT_UINT32, BASE_DEC, VALS(rnsap_TransportFormatManagement_vals), 0,
        "rnsap.TransportFormatManagement", HFILL }},
    { &hf_rnsap_USCH_InformationListIEs_RL_SetupRspTDD_item,
      { "Item", "rnsap.USCH_InformationListIEs_RL_SetupRspTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.USCHInformationItem_RL_SetupRspTDD", HFILL }},
    { &hf_rnsap_usch_ID,
      { "usch-ID", "rnsap.usch_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.USCH_ID", HFILL }},
    { &hf_rnsap_ul_TimeSlot_ISCP_LCR_Info,
      { "ul-TimeSlot-ISCP-LCR-Info", "rnsap.ul_TimeSlot_ISCP_LCR_Info",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UL_TimeSlot_ISCP_LCR_Info", HFILL }},
    { &hf_rnsap_secondary_LCR_CCPCH_Info_TDD,
      { "secondary-LCR-CCPCH-Info-TDD", "rnsap.secondary_LCR_CCPCH_Info_TDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Secondary_LCR_CCPCH_Info_TDD", HFILL }},
    { &hf_rnsap_ul_LCR_CCTrCHInformation,
      { "ul-LCR-CCTrCHInformation", "rnsap.ul_LCR_CCTrCHInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_LCR_CCTrCHInformationList_RL_SetupRspTDD", HFILL }},
    { &hf_rnsap_dl_LCR_CCTrCHInformation,
      { "dl-LCR-CCTrCHInformation", "rnsap.dl_LCR_CCTrCHInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_LCR_CCTrCHInformationList_RL_SetupRspTDD", HFILL }},
    { &hf_rnsap_dsch_LCR_InformationResponse,
      { "dsch-LCR-InformationResponse", "rnsap.dsch_LCR_InformationResponse",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DSCH_LCR_InformationResponse_RL_SetupRspTDD", HFILL }},
    { &hf_rnsap_usch_LCR_InformationResponse,
      { "usch-LCR-InformationResponse", "rnsap.usch_LCR_InformationResponse",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.USCH_LCR_InformationResponse_RL_SetupRspTDD", HFILL }},
    { &hf_rnsap_UL_LCR_CCTrCHInformationListIE_RL_SetupRspTDD_item,
      { "Item", "rnsap.UL_LCR_CCTrCHInformationListIE_RL_SetupRspTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_LCR_CCTrCHInformationItem_RL_SetupRspTDD", HFILL }},
    { &hf_rnsap_ul_DPCH_LCR_Information,
      { "ul-DPCH-LCR-Information", "rnsap.ul_DPCH_LCR_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_DPCH_LCR_InformationList_RL_SetupRspTDD", HFILL }},
    { &hf_rnsap_uL_TimeslotLCR_Information,
      { "uL-TimeslotLCR-Information", "rnsap.uL_TimeslotLCR_Information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UL_TimeslotLCR_Information", HFILL }},
    { &hf_rnsap_DL_CCTrCH_LCR_InformationListIE_RL_SetupRspTDD_item,
      { "Item", "rnsap.DL_CCTrCH_LCR_InformationListIE_RL_SetupRspTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_CCTrCH_LCR_InformationItem_RL_SetupRspTDD", HFILL }},
    { &hf_rnsap_dl_DPCH_LCR_Information,
      { "dl-DPCH-LCR-Information", "rnsap.dl_DPCH_LCR_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_DPCH_LCR_InformationList_RL_SetupRspTDD", HFILL }},
    { &hf_rnsap_dL_Timeslot_LCR_Information,
      { "dL-Timeslot-LCR-Information", "rnsap.dL_Timeslot_LCR_Information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DL_TimeslotLCR_Information", HFILL }},
    { &hf_rnsap_tSTD_Indicator,
      { "tSTD-Indicator", "rnsap.tSTD_Indicator",
        FT_UINT32, BASE_DEC, VALS(rnsap_TSTD_Indicator_vals), 0,
        "rnsap.TSTD_Indicator", HFILL }},
    { &hf_rnsap_DSCH_LCR_InformationListIEs_RL_SetupRspTDD_item,
      { "Item", "rnsap.DSCH_LCR_InformationListIEs_RL_SetupRspTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DSCH_LCR_InformationItem_RL_SetupRspTDD", HFILL }},
    { &hf_rnsap_USCH_LCR_InformationListIEs_RL_SetupRspTDD_item,
      { "Item", "rnsap.USCH_LCR_InformationListIEs_RL_SetupRspTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.USCH_LCR_InformationItem_RL_SetupRspTDD", HFILL }},
    { &hf_rnsap_generalCause,
      { "generalCause", "rnsap.generalCause",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.GeneralCauseList_RL_SetupFailureFDD", HFILL }},
    { &hf_rnsap_rLSpecificCause,
      { "rLSpecificCause", "rnsap.rLSpecificCause",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RLSpecificCauseList_RL_SetupFailureFDD", HFILL }},
    { &hf_rnsap_cause,
      { "cause", "rnsap.cause",
        FT_UINT32, BASE_DEC, VALS(rnsap_Cause_vals), 0,
        "rnsap.Cause", HFILL }},
    { &hf_rnsap_unsuccessful_RL_InformationRespList_RL_SetupFailureFDD,
      { "unsuccessful-RL-InformationRespList-RL-SetupFailureFDD", "rnsap.unsuccessful_RL_InformationRespList_RL_SetupFailureFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UnsuccessfulRL_InformationResponseList_RL_SetupFailureFDD", HFILL }},
    { &hf_rnsap_successful_RL_InformationRespList_RL_SetupFailureFDD,
      { "successful-RL-InformationRespList-RL-SetupFailureFDD", "rnsap.successful_RL_InformationRespList_RL_SetupFailureFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.SuccessfulRL_InformationResponseList_RL_SetupFailureFDD", HFILL }},
    { &hf_rnsap_UnsuccessfulRL_InformationResponseList_RL_SetupFailureFDD_item,
      { "Item", "rnsap.UnsuccessfulRL_InformationResponseList_RL_SetupFailureFDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_SuccessfulRL_InformationResponseList_RL_SetupFailureFDD_item,
      { "Item", "rnsap.SuccessfulRL_InformationResponseList_RL_SetupFailureFDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_diversityIndication1,
      { "diversityIndication", "rnsap.diversityIndication",
        FT_UINT32, BASE_DEC, VALS(rnsap_DiversityIndication_RL_SetupFailureFDD_vals), 0,
        "rnsap.DiversityIndication_RL_SetupFailureFDD", HFILL }},
    { &hf_rnsap_not_Used_dSCH_InformationResponse_RL_SetupFailureFDD,
      { "not-Used-dSCH-InformationResponse-RL-SetupFailureFDD", "rnsap.not_Used_dSCH_InformationResponse_RL_SetupFailureFDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.NULL", HFILL }},
    { &hf_rnsap_combining1,
      { "combining", "rnsap.combining",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Combining_RL_SetupFailureFDD", HFILL }},
    { &hf_rnsap_nonCombiningOrFirstRL1,
      { "nonCombiningOrFirstRL", "rnsap.nonCombiningOrFirstRL",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.NonCombiningOrFirstRL_RL_SetupFailureFDD", HFILL }},
    { &hf_rnsap_generalCause1,
      { "generalCause", "rnsap.generalCause",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.GeneralCauseList_RL_SetupFailureTDD", HFILL }},
    { &hf_rnsap_rLSpecificCause1,
      { "rLSpecificCause", "rnsap.rLSpecificCause",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RLSpecificCauseList_RL_SetupFailureTDD", HFILL }},
    { &hf_rnsap_unsuccessful_RL_InformationRespItem_RL_SetupFailureTDD,
      { "unsuccessful-RL-InformationRespItem-RL-SetupFailureTDD", "rnsap.unsuccessful_RL_InformationRespItem_RL_SetupFailureTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Unsuccessful_RL_InformationRespItem_RL_SetupFailureTDD", HFILL }},
    { &hf_rnsap_RL_InformationList_RL_AdditionRqstFDD_item,
      { "Item", "rnsap.RL_InformationList_RL_AdditionRqstFDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_dL_TimeSlot_ISCP_Info,
      { "dL-TimeSlot-ISCP-Info", "rnsap.dL_TimeSlot_ISCP_Info",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DL_TimeSlot_ISCP_Info", HFILL }},
    { &hf_rnsap_UL_CCTrCH_InformationList_RL_AdditionRqstTDD_item,
      { "Item", "rnsap.UL_CCTrCH_InformationList_RL_AdditionRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_uplinkStepSizeLCR,
      { "uplinkStepSizeLCR", "rnsap.uplinkStepSizeLCR",
        FT_UINT32, BASE_DEC, VALS(rnsap_TDD_TPC_UplinkStepSize_LCR_vals), 0,
        "rnsap.TDD_TPC_UplinkStepSize_LCR", HFILL }},
    { &hf_rnsap_DL_CCTrCH_InformationList_RL_AdditionRqstTDD_item,
      { "Item", "rnsap.DL_CCTrCH_InformationList_RL_AdditionRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_downlinkStepSize,
      { "downlinkStepSize", "rnsap.downlinkStepSize",
        FT_UINT32, BASE_DEC, VALS(rnsap_TDD_TPC_DownlinkStepSize_vals), 0,
        "rnsap.TDD_TPC_DownlinkStepSize", HFILL }},
    { &hf_rnsap_RL_InformationResponseList_RL_AdditionRspFDD_item,
      { "Item", "rnsap.RL_InformationResponseList_RL_AdditionRspFDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_dl_CodeInformation1,
      { "dl-CodeInformation", "rnsap.dl_CodeInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_CodeInformationList_RL_AdditionRspFDD", HFILL }},
    { &hf_rnsap_diversityIndication2,
      { "diversityIndication", "rnsap.diversityIndication",
        FT_UINT32, BASE_DEC, VALS(rnsap_DiversityIndication_RL_AdditionRspFDD_vals), 0,
        "rnsap.DiversityIndication_RL_AdditionRspFDD", HFILL }},
    { &hf_rnsap_combining2,
      { "combining", "rnsap.combining",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Combining_RL_AdditionRspFDD", HFILL }},
    { &hf_rnsap_nonCombining,
      { "nonCombining", "rnsap.nonCombining",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.NonCombining_RL_AdditionRspFDD", HFILL }},
    { &hf_rnsap_ul_CCTrCHInformation1,
      { "ul-CCTrCHInformation", "rnsap.ul_CCTrCHInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_CCTrCHInformationList_RL_AdditionRspTDD", HFILL }},
    { &hf_rnsap_dl_CCTrCHInformation1,
      { "dl-CCTrCHInformation", "rnsap.dl_CCTrCHInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_CCTrCHInformationList_RL_AdditionRspTDD", HFILL }},
    { &hf_rnsap_dCH_Information,
      { "dCH-Information", "rnsap.dCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DCH_Information_RL_AdditionRspTDD", HFILL }},
    { &hf_rnsap_dSCH_InformationResponse,
      { "dSCH-InformationResponse", "rnsap.dSCH_InformationResponse",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DSCH_InformationResponse_RL_AdditionRspTDD", HFILL }},
    { &hf_rnsap_uSCH_InformationResponse,
      { "uSCH-InformationResponse", "rnsap.uSCH_InformationResponse",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.USCH_InformationResponse_RL_AdditionRspTDD", HFILL }},
    { &hf_rnsap_UL_CCTrCHInformationListIE_RL_AdditionRspTDD_item,
      { "Item", "rnsap.UL_CCTrCHInformationListIE_RL_AdditionRspTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_CCTrCHInformationItem_RL_AdditionRspTDD", HFILL }},
    { &hf_rnsap_ul_DPCH_Information1,
      { "ul-DPCH-Information", "rnsap.ul_DPCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_DPCH_InformationList_RL_AdditionRspTDD", HFILL }},
    { &hf_rnsap_DL_CCTrCHInformationListIE_RL_AdditionRspTDD_item,
      { "Item", "rnsap.DL_CCTrCHInformationListIE_RL_AdditionRspTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_CCTrCHInformationItem_RL_AdditionRspTDD", HFILL }},
    { &hf_rnsap_dl_DPCH_Information1,
      { "dl-DPCH-Information", "rnsap.dl_DPCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_DPCH_InformationList_RL_AdditionRspTDD", HFILL }},
    { &hf_rnsap_diversityIndication3,
      { "diversityIndication", "rnsap.diversityIndication",
        FT_UINT32, BASE_DEC, VALS(rnsap_DiversityIndication_RL_AdditionRspTDD_vals), 0,
        "rnsap.DiversityIndication_RL_AdditionRspTDD", HFILL }},
    { &hf_rnsap_combining3,
      { "combining", "rnsap.combining",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Combining_RL_AdditionRspTDD", HFILL }},
    { &hf_rnsap_nonCombining1,
      { "nonCombining", "rnsap.nonCombining",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.NonCombining_RL_AdditionRspTDD", HFILL }},
    { &hf_rnsap_DSCH_InformationListIE_RL_AdditionRspTDD_item,
      { "Item", "rnsap.DSCH_InformationListIE_RL_AdditionRspTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DSCHInformationItem_RL_AdditionRspTDD", HFILL }},
    { &hf_rnsap_diversityIndication4,
      { "diversityIndication", "rnsap.diversityIndication",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DiversityIndication_RL_AdditionRspTDD2", HFILL }},
    { &hf_rnsap_USCH_InformationListIE_RL_AdditionRspTDD_item,
      { "Item", "rnsap.USCH_InformationListIE_RL_AdditionRspTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.USCHInformationItem_RL_AdditionRspTDD", HFILL }},
    { &hf_rnsap_uSCH_ID,
      { "uSCH-ID", "rnsap.uSCH_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.USCH_ID", HFILL }},
    { &hf_rnsap_ul_CCTrCH_LCR_Information,
      { "ul-CCTrCH-LCR-Information", "rnsap.ul_CCTrCH_LCR_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_CCTrCH_LCR_InformationList_RL_AdditionRspTDD", HFILL }},
    { &hf_rnsap_dl_CCTrCH_LCR_Information,
      { "dl-CCTrCH-LCR-Information", "rnsap.dl_CCTrCH_LCR_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_CCTrCH_LCR_InformationList_RL_AdditionRspTDD", HFILL }},
    { &hf_rnsap_dCH_InformationResponse2,
      { "dCH-InformationResponse", "rnsap.dCH_InformationResponse",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DCH_InformationResponseList_RL_AdditionRspTDD", HFILL }},
    { &hf_rnsap_dsch_LCR_InformationResponse1,
      { "dsch-LCR-InformationResponse", "rnsap.dsch_LCR_InformationResponse",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DSCH_LCR_InformationResponse_RL_AdditionRspTDD", HFILL }},
    { &hf_rnsap_usch_LCR_InformationResponse1,
      { "usch-LCR-InformationResponse", "rnsap.usch_LCR_InformationResponse",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.USCH_LCR_InformationResponse_RL_AdditionRspTDD", HFILL }},
    { &hf_rnsap_UL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD_item,
      { "Item", "rnsap.UL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_CCTrCH_LCR_InformationItem_RL_AdditionRspTDD", HFILL }},
    { &hf_rnsap_ul_DPCH_LCR_Information1,
      { "ul-DPCH-LCR-Information", "rnsap.ul_DPCH_LCR_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_DPCH_LCR_InformationList_RL_AdditionRspTDD", HFILL }},
    { &hf_rnsap_DL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD_item,
      { "Item", "rnsap.DL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_CCTrCH_LCR_InformationItem_RL_AdditionRspTDD", HFILL }},
    { &hf_rnsap_dl_DPCH_LCR_Information1,
      { "dl-DPCH-LCR-Information", "rnsap.dl_DPCH_LCR_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_DPCH_LCR_InformationList_RL_AdditionRspTDD", HFILL }},
    { &hf_rnsap_dL_TimeslotLCR_Information,
      { "dL-TimeslotLCR-Information", "rnsap.dL_TimeslotLCR_Information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DL_TimeslotLCR_Information", HFILL }},
    { &hf_rnsap_DSCH_LCR_InformationListIEs_RL_AdditionRspTDD_item,
      { "Item", "rnsap.DSCH_LCR_InformationListIEs_RL_AdditionRspTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DSCH_LCR_InformationItem_RL_AdditionRspTDD", HFILL }},
    { &hf_rnsap_USCH_LCR_InformationListIEs_RL_AdditionRspTDD_item,
      { "Item", "rnsap.USCH_LCR_InformationListIEs_RL_AdditionRspTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.USCH_LCR_InformationItem_RL_AdditionRspTDD", HFILL }},
    { &hf_rnsap_generalCause2,
      { "generalCause", "rnsap.generalCause",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.GeneralCauseList_RL_AdditionFailureFDD", HFILL }},
    { &hf_rnsap_rLSpecificCause2,
      { "rLSpecificCause", "rnsap.rLSpecificCause",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RLSpecificCauseList_RL_AdditionFailureFDD", HFILL }},
    { &hf_rnsap_unsuccessful_RL_InformationRespList_RL_AdditionFailureFDD,
      { "unsuccessful-RL-InformationRespList-RL-AdditionFailureFDD", "rnsap.unsuccessful_RL_InformationRespList_RL_AdditionFailureFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UnsuccessfulRL_InformationResponseList_RL_AdditionFailureFDD", HFILL }},
    { &hf_rnsap_successful_RL_InformationRespList_RL_AdditionFailureFDD,
      { "successful-RL-InformationRespList-RL-AdditionFailureFDD", "rnsap.successful_RL_InformationRespList_RL_AdditionFailureFDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.SuccessfulRL_InformationResponseList_RL_AdditionFailureFDD", HFILL }},
    { &hf_rnsap_UnsuccessfulRL_InformationResponseList_RL_AdditionFailureFDD_item,
      { "Item", "rnsap.UnsuccessfulRL_InformationResponseList_RL_AdditionFailureFDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_SuccessfulRL_InformationResponseList_RL_AdditionFailureFDD_item,
      { "Item", "rnsap.SuccessfulRL_InformationResponseList_RL_AdditionFailureFDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_dl_CodeInformation2,
      { "dl-CodeInformation", "rnsap.dl_CodeInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_CodeInformationList_RL_AdditionFailureFDD", HFILL }},
    { &hf_rnsap_diversityIndication5,
      { "diversityIndication", "rnsap.diversityIndication",
        FT_UINT32, BASE_DEC, VALS(rnsap_DiversityIndication_RL_AdditionFailureFDD_vals), 0,
        "rnsap.DiversityIndication_RL_AdditionFailureFDD", HFILL }},
    { &hf_rnsap_combining4,
      { "combining", "rnsap.combining",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Combining_RL_AdditionFailureFDD", HFILL }},
    { &hf_rnsap_nonCombining2,
      { "nonCombining", "rnsap.nonCombining",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.NonCombining_RL_AdditionFailureFDD", HFILL }},
    { &hf_rnsap_generalCause3,
      { "generalCause", "rnsap.generalCause",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.GeneralCauseList_RL_AdditionFailureTDD", HFILL }},
    { &hf_rnsap_rLSpecificCause3,
      { "rLSpecificCause", "rnsap.rLSpecificCause",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RLSpecificCauseList_RL_AdditionFailureTDD", HFILL }},
    { &hf_rnsap_unsuccessful_RL_InformationRespItem_RL_AdditionFailureTDD,
      { "unsuccessful-RL-InformationRespItem-RL-AdditionFailureTDD", "rnsap.unsuccessful_RL_InformationRespItem_RL_AdditionFailureTDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Unsuccessful_RL_InformationRespItem_RL_AdditionFailureTDD", HFILL }},
    { &hf_rnsap_RL_InformationList_RL_DeletionRqst_item,
      { "Item", "rnsap.RL_InformationList_RL_DeletionRqst_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_maxNrOfUL_DPDCHs,
      { "maxNrOfUL-DPDCHs", "rnsap.maxNrOfUL_DPDCHs",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.MaxNrOfUL_DPCHs", HFILL }},
    { &hf_rnsap_not_Used_sSDT_CellIDLength,
      { "not-Used-sSDT-CellIDLength", "rnsap.not_Used_sSDT_CellIDLength",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.NULL", HFILL }},
    { &hf_rnsap_powerOffsetInformation2,
      { "powerOffsetInformation", "rnsap.powerOffsetInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.PowerOffsetInformation_RL_ReconfPrepFDD", HFILL }},
    { &hf_rnsap_fdd_TPC_DownlinkStepSize,
      { "fdd-TPC-DownlinkStepSize", "rnsap.fdd_TPC_DownlinkStepSize",
        FT_UINT32, BASE_DEC, VALS(rnsap_FDD_TPC_DownlinkStepSize_vals), 0,
        "rnsap.FDD_TPC_DownlinkStepSize", HFILL }},
    { &hf_rnsap_pO1_ForTFCI_Bits,
      { "pO1-ForTFCI-Bits", "rnsap.pO1_ForTFCI_Bits",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.PowerOffset", HFILL }},
    { &hf_rnsap_pO2_ForTPC_Bits,
      { "pO2-ForTPC-Bits", "rnsap.pO2_ForTPC_Bits",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.PowerOffset", HFILL }},
    { &hf_rnsap_pO3_ForPilotBits,
      { "pO3-ForPilotBits", "rnsap.pO3_ForPilotBits",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.PowerOffset", HFILL }},
    { &hf_rnsap_DCH_DeleteList_RL_ReconfPrepFDD_item,
      { "Item", "rnsap.DCH_DeleteList_RL_ReconfPrepFDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DCH_DeleteItem_RL_ReconfPrepFDD", HFILL }},
    { &hf_rnsap_dCH_ID,
      { "dCH-ID", "rnsap.dCH_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DCH_ID", HFILL }},
    { &hf_rnsap_RL_InformationList_RL_ReconfPrepFDD_item,
      { "Item", "rnsap.RL_InformationList_RL_ReconfPrepFDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_not_Used_sSDT_Indication,
      { "not-Used-sSDT-Indication", "rnsap.not_Used_sSDT_Indication",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.NULL", HFILL }},
    { &hf_rnsap_not_Used_sSDT_CellIdentity,
      { "not-Used-sSDT-CellIdentity", "rnsap.not_Used_sSDT_CellIdentity",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.NULL", HFILL }},
    { &hf_rnsap_powerOffsetInformation3,
      { "powerOffsetInformation", "rnsap.powerOffsetInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.PowerOffsetInformation_F_DPCH_RL_ReconfPrepFDD", HFILL }},
    { &hf_rnsap_UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD_item,
      { "Item", "rnsap.UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_punctureLimit,
      { "punctureLimit", "rnsap.punctureLimit",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.PunctureLimit", HFILL }},
    { &hf_rnsap_UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD_item,
      { "Item", "rnsap.UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD_item,
      { "Item", "rnsap.UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD_item,
      { "Item", "rnsap.DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_cCTrCH_TPCList1,
      { "cCTrCH-TPCList", "rnsap.cCTrCH_TPCList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.CCTrCH_TPCAddList_RL_ReconfPrepTDD", HFILL }},
    { &hf_rnsap_CCTrCH_TPCAddList_RL_ReconfPrepTDD_item,
      { "Item", "rnsap.CCTrCH_TPCAddList_RL_ReconfPrepTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.CCTrCH_TPCAddItem_RL_ReconfPrepTDD", HFILL }},
    { &hf_rnsap_DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD_item,
      { "Item", "rnsap.DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_cCTrCH_TPCList2,
      { "cCTrCH-TPCList", "rnsap.cCTrCH_TPCList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.CCTrCH_TPCModifyList_RL_ReconfPrepTDD", HFILL }},
    { &hf_rnsap_CCTrCH_TPCModifyList_RL_ReconfPrepTDD_item,
      { "Item", "rnsap.CCTrCH_TPCModifyList_RL_ReconfPrepTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.CCTrCH_TPCModifyItem_RL_ReconfPrepTDD", HFILL }},
    { &hf_rnsap_DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD_item,
      { "Item", "rnsap.DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_DCH_DeleteList_RL_ReconfPrepTDD_item,
      { "Item", "rnsap.DCH_DeleteList_RL_ReconfPrepTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DCH_DeleteItem_RL_ReconfPrepTDD", HFILL }},
    { &hf_rnsap_DSCH_ModifyList_RL_ReconfPrepTDD_item,
      { "Item", "rnsap.DSCH_ModifyList_RL_ReconfPrepTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DSCH_ModifyItem_RL_ReconfPrepTDD", HFILL }},
    { &hf_rnsap_dSCH_ID,
      { "dSCH-ID", "rnsap.dSCH_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DSCH_ID", HFILL }},
    { &hf_rnsap_dl_ccTrCHID,
      { "dl-ccTrCHID", "rnsap.dl_ccTrCHID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.CCTrCH_ID", HFILL }},
    { &hf_rnsap_trChSourceStatisticsDescriptor,
      { "trChSourceStatisticsDescriptor", "rnsap.trChSourceStatisticsDescriptor",
        FT_UINT32, BASE_DEC, VALS(rnsap_TrCH_SrcStatisticsDescr_vals), 0,
        "rnsap.TrCH_SrcStatisticsDescr", HFILL }},
    { &hf_rnsap_transportFormatSet,
      { "transportFormatSet", "rnsap.transportFormatSet",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.TransportFormatSet", HFILL }},
    { &hf_rnsap_allocationRetentionPriority,
      { "allocationRetentionPriority", "rnsap.allocationRetentionPriority",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.AllocationRetentionPriority", HFILL }},
    { &hf_rnsap_schedulingPriorityIndicator,
      { "schedulingPriorityIndicator", "rnsap.schedulingPriorityIndicator",
        FT_UINT32, BASE_DEC, VALS(rnsap_SchedulingPriorityIndicator_vals), 0,
        "rnsap.SchedulingPriorityIndicator", HFILL }},
    { &hf_rnsap_bLER,
      { "bLER", "rnsap.bLER",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.BLER", HFILL }},
    { &hf_rnsap_transportBearerRequestIndicator,
      { "transportBearerRequestIndicator", "rnsap.transportBearerRequestIndicator",
        FT_UINT32, BASE_DEC, VALS(rnsap_TransportBearerRequestIndicator_vals), 0,
        "rnsap.TransportBearerRequestIndicator", HFILL }},
    { &hf_rnsap_DSCH_DeleteList_RL_ReconfPrepTDD_item,
      { "Item", "rnsap.DSCH_DeleteList_RL_ReconfPrepTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DSCH_DeleteItem_RL_ReconfPrepTDD", HFILL }},
    { &hf_rnsap_USCH_ModifyList_RL_ReconfPrepTDD_item,
      { "Item", "rnsap.USCH_ModifyList_RL_ReconfPrepTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.USCH_ModifyItem_RL_ReconfPrepTDD", HFILL }},
    { &hf_rnsap_ul_ccTrCHID,
      { "ul-ccTrCHID", "rnsap.ul_ccTrCHID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.CCTrCH_ID", HFILL }},
    { &hf_rnsap_rb_Info,
      { "rb-Info", "rnsap.rb_Info",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RB_Info", HFILL }},
    { &hf_rnsap_USCH_DeleteList_RL_ReconfPrepTDD_item,
      { "Item", "rnsap.USCH_DeleteList_RL_ReconfPrepTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.USCH_DeleteItem_RL_ReconfPrepTDD", HFILL }},
    { &hf_rnsap_RL_Information_RL_ReconfPrepTDD_item,
      { "Item", "rnsap.RL_Information_RL_ReconfPrepTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_InformationIE_RL_ReconfPrepTDD", HFILL }},
    { &hf_rnsap_rL_Specific_DCH_Info,
      { "rL-Specific-DCH-Info", "rnsap.rL_Specific_DCH_Info",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RL_Specific_DCH_Info", HFILL }},
    { &hf_rnsap_RL_InformationResponseList_RL_ReconfReadyFDD_item,
      { "Item", "rnsap.RL_InformationResponseList_RL_ReconfReadyFDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_max_UL_SIR,
      { "max-UL-SIR", "rnsap.max_UL_SIR",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.UL_SIR", HFILL }},
    { &hf_rnsap_min_UL_SIR,
      { "min-UL-SIR", "rnsap.min_UL_SIR",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.UL_SIR", HFILL }},
    { &hf_rnsap_dl_CodeInformationList,
      { "dl-CodeInformationList", "rnsap.dl_CodeInformationList",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_CodeInformationList_RL_ReconfReadyFDD", HFILL }},
    { &hf_rnsap_dCHInformationResponse,
      { "dCHInformationResponse", "rnsap.dCHInformationResponse",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DCH_InformationResponseList_RL_ReconfReadyFDD", HFILL }},
    { &hf_rnsap_not_Used_dSCHsToBeAddedOrModified,
      { "not-Used-dSCHsToBeAddedOrModified", "rnsap.not_Used_dSCHsToBeAddedOrModified",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.NULL", HFILL }},
    { &hf_rnsap_ul_CCTrCH_Information,
      { "ul-CCTrCH-Information", "rnsap.ul_CCTrCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_CCTrCH_InformationList_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_dl_CCTrCH_Information,
      { "dl-CCTrCH-Information", "rnsap.dl_CCTrCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_CCTrCH_InformationList_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_dCHInformationResponse1,
      { "dCHInformationResponse", "rnsap.dCHInformationResponse",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DCH_InformationResponseList_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_dSCHsToBeAddedOrModified,
      { "dSCHsToBeAddedOrModified", "rnsap.dSCHsToBeAddedOrModified",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DSCHToBeAddedOrModified_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_uSCHsToBeAddedOrModified,
      { "uSCHsToBeAddedOrModified", "rnsap.uSCHsToBeAddedOrModified",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.USCHToBeAddedOrModified_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_UL_CCTrCHInformationListIE_RL_ReconfReadyTDD_item,
      { "Item", "rnsap.UL_CCTrCHInformationListIE_RL_ReconfReadyTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_CCTrCH_InformationItem_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_ul_DPCH_AddInformation,
      { "ul-DPCH-AddInformation", "rnsap.ul_DPCH_AddInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_DPCH_InformationAddList_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_ul_DPCH_ModifyInformation,
      { "ul-DPCH-ModifyInformation", "rnsap.ul_DPCH_ModifyInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_DPCH_InformationModifyList_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_ul_DPCH_DeleteInformation,
      { "ul-DPCH-DeleteInformation", "rnsap.ul_DPCH_DeleteInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_DPCH_InformationDeleteList_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_uL_TimeslotLCR_Info,
      { "uL-TimeslotLCR-Info", "rnsap.uL_TimeslotLCR_Info",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UL_TimeslotLCR_Information", HFILL }},
    { &hf_rnsap_rxTimingDeviationForTA,
      { "rxTimingDeviationForTA", "rnsap.rxTimingDeviationForTA",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RxTimingDeviationForTA", HFILL }},
    { &hf_rnsap_uL_Timeslot_InformationModifyList_RL_ReconfReadyTDD,
      { "uL-Timeslot-InformationModifyList-RL-ReconfReadyTDD", "rnsap.uL_Timeslot_InformationModifyList_RL_ReconfReadyTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UL_Timeslot_InformationModifyList_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_UL_TimeslotLCR_InformationModifyList_RL_ReconfReadyTDD_item,
      { "Item", "rnsap.UL_TimeslotLCR_InformationModifyList_RL_ReconfReadyTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_TimeslotLCR_InformationModifyItem_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_timeSlotLCR,
      { "timeSlotLCR", "rnsap.timeSlotLCR",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.TimeSlotLCR", HFILL }},
    { &hf_rnsap_midambleShiftLCR,
      { "midambleShiftLCR", "rnsap.midambleShiftLCR",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.MidambleShiftLCR", HFILL }},
    { &hf_rnsap_tDD_uL_Code_LCR_Information,
      { "tDD-uL-Code-LCR-Information", "rnsap.tDD_uL_Code_LCR_Information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.TDD_UL_Code_LCR_InformationModifyList_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_TDD_UL_Code_LCR_InformationModifyList_RL_ReconfReadyTDD_item,
      { "Item", "rnsap.TDD_UL_Code_LCR_InformationModifyList_RL_ReconfReadyTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.TDD_UL_Code_LCR_InformationModifyItem_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_dPCH_ID,
      { "dPCH-ID", "rnsap.dPCH_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DPCH_ID", HFILL }},
    { &hf_rnsap_tDD_ChannelisationCodeLCR,
      { "tDD-ChannelisationCodeLCR", "rnsap.tDD_ChannelisationCodeLCR",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.TDD_ChannelisationCodeLCR", HFILL }},
    { &hf_rnsap_UL_Timeslot_InformationModifyList_RL_ReconfReadyTDD_item,
      { "Item", "rnsap.UL_Timeslot_InformationModifyList_RL_ReconfReadyTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_Timeslot_InformationModifyItem_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_timeSlot,
      { "timeSlot", "rnsap.timeSlot",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.TimeSlot", HFILL }},
    { &hf_rnsap_midambleShiftAndBurstType,
      { "midambleShiftAndBurstType", "rnsap.midambleShiftAndBurstType",
        FT_UINT32, BASE_DEC, VALS(rnsap_MidambleShiftAndBurstType_vals), 0,
        "rnsap.MidambleShiftAndBurstType", HFILL }},
    { &hf_rnsap_uL_Code_Information,
      { "uL-Code-Information", "rnsap.uL_Code_Information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.TDD_UL_Code_InformationModifyList_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_TDD_UL_Code_InformationModifyList_RL_ReconfReadyTDD_item,
      { "Item", "rnsap.TDD_UL_Code_InformationModifyList_RL_ReconfReadyTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.TDD_UL_Code_InformationModifyItem_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_tDD_ChannelisationCode,
      { "tDD-ChannelisationCode", "rnsap.tDD_ChannelisationCode",
        FT_UINT32, BASE_DEC, VALS(rnsap_TDD_ChannelisationCode_vals), 0,
        "rnsap.TDD_ChannelisationCode", HFILL }},
    { &hf_rnsap_UL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD_item,
      { "Item", "rnsap.UL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_DPCH_InformationDeleteItem_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_DL_CCTrCHInformationListIE_RL_ReconfReadyTDD_item,
      { "Item", "rnsap.DL_CCTrCHInformationListIE_RL_ReconfReadyTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_CCTrCH_InformationItem_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_dl_DPCH_AddInformation,
      { "dl-DPCH-AddInformation", "rnsap.dl_DPCH_AddInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_DPCH_InformationAddList_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_dl_DPCH_ModifyInformation,
      { "dl-DPCH-ModifyInformation", "rnsap.dl_DPCH_ModifyInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_DPCH_InformationModifyList_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_dl_DPCH_DeleteInformation,
      { "dl-DPCH-DeleteInformation", "rnsap.dl_DPCH_DeleteInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_DPCH_InformationDeleteList_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_dL_TimeslotLCR_Info,
      { "dL-TimeslotLCR-Info", "rnsap.dL_TimeslotLCR_Info",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DL_TimeslotLCR_Information", HFILL }},
    { &hf_rnsap_dL_Timeslot_InformationModifyList_RL_ReconfReadyTDD,
      { "dL-Timeslot-InformationModifyList-RL-ReconfReadyTDD", "rnsap.dL_Timeslot_InformationModifyList_RL_ReconfReadyTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DL_Timeslot_InformationModifyList_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_DL_TimeslotLCR_InformationModifyList_RL_ReconfReadyTDD_item,
      { "Item", "rnsap.DL_TimeslotLCR_InformationModifyList_RL_ReconfReadyTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_TimeslotLCR_InformationModifyItem_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_tDD_dL_Code_LCR_Information,
      { "tDD-dL-Code-LCR-Information", "rnsap.tDD_dL_Code_LCR_Information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.TDD_DL_Code_LCR_InformationModifyList_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_TDD_DL_Code_LCR_InformationModifyList_RL_ReconfReadyTDD_item,
      { "Item", "rnsap.TDD_DL_Code_LCR_InformationModifyList_RL_ReconfReadyTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.TDD_DL_Code_LCR_InformationModifyItem_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_DL_Timeslot_InformationModifyList_RL_ReconfReadyTDD_item,
      { "Item", "rnsap.DL_Timeslot_InformationModifyList_RL_ReconfReadyTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_Timeslot_InformationModifyItem_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_dL_Code_Information,
      { "dL-Code-Information", "rnsap.dL_Code_Information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.TDD_DL_Code_InformationModifyList_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_TDD_DL_Code_InformationModifyList_RL_ReconfReadyTDD_item,
      { "Item", "rnsap.TDD_DL_Code_InformationModifyList_RL_ReconfReadyTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.TDD_DL_Code_InformationModifyItem_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_DL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD_item,
      { "Item", "rnsap.DL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_DPCH_InformationDeleteItem_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_DSCHToBeAddedOrModifiedList_RL_ReconfReadyTDD_item,
      { "Item", "rnsap.DSCHToBeAddedOrModifiedList_RL_ReconfReadyTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DSCHToBeAddedOrModifiedItem_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_USCHToBeAddedOrModifiedList_RL_ReconfReadyTDD_item,
      { "Item", "rnsap.USCHToBeAddedOrModifiedList_RL_ReconfReadyTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.USCHToBeAddedOrModifiedItem_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_Multiple_RL_InformationResponse_RL_ReconfReadyTDD_item,
      { "Item", "rnsap.Multiple_RL_InformationResponse_RL_ReconfReadyTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_InformationResponse_RL_ReconfReadyTDD", HFILL }},
    { &hf_rnsap_generalCause4,
      { "generalCause", "rnsap.generalCause",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.GeneralCauseList_RL_ReconfFailure", HFILL }},
    { &hf_rnsap_rLSpecificCause4,
      { "rLSpecificCause", "rnsap.rLSpecificCause",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RLSpecificCauseList_RL_ReconfFailure", HFILL }},
    { &hf_rnsap_rL_ReconfigurationFailureList_RL_ReconfFailure,
      { "rL-ReconfigurationFailureList-RL-ReconfFailure", "rnsap.rL_ReconfigurationFailureList_RL_ReconfFailure",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RL_ReconfigurationFailureList_RL_ReconfFailure", HFILL }},
    { &hf_rnsap_RL_ReconfigurationFailureList_RL_ReconfFailure_item,
      { "Item", "rnsap.RL_ReconfigurationFailureList_RL_ReconfFailure_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_DCH_DeleteList_RL_ReconfRqstFDD_item,
      { "Item", "rnsap.DCH_DeleteList_RL_ReconfRqstFDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DCH_DeleteItem_RL_ReconfRqstFDD", HFILL }},
    { &hf_rnsap_RL_ReconfigurationRequestFDD_RL_InformationList_item,
      { "Item", "rnsap.RL_ReconfigurationRequestFDD_RL_InformationList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD_item,
      { "Item", "rnsap.UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD_item,
      { "Item", "rnsap.UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD_item,
      { "Item", "rnsap.DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD_item,
      { "Item", "rnsap.DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_DCH_DeleteList_RL_ReconfRqstTDD_item,
      { "Item", "rnsap.DCH_DeleteList_RL_ReconfRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DCH_DeleteItem_RL_ReconfRqstTDD", HFILL }},
    { &hf_rnsap_Multiple_RL_ReconfigurationRequestTDD_RL_Information_item,
      { "Item", "rnsap.Multiple_RL_ReconfigurationRequestTDD_RL_Information_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_ReconfigurationRequestTDD_RL_Information", HFILL }},
    { &hf_rnsap_RL_InformationResponseList_RL_ReconfRspFDD_item,
      { "Item", "rnsap.RL_InformationResponseList_RL_ReconfRspFDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_dCHsInformationResponseList,
      { "dCHsInformationResponseList", "rnsap.dCHsInformationResponseList",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DCH_InformationResponseList_RL_ReconfRspFDD", HFILL }},
    { &hf_rnsap_dL_CodeInformationList_RL_ReconfResp,
      { "dL-CodeInformationList-RL-ReconfResp", "rnsap.dL_CodeInformationList_RL_ReconfResp",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_CodeInformationList_RL_ReconfRspFDD", HFILL }},
    { &hf_rnsap_dCHsInformationResponseList1,
      { "dCHsInformationResponseList", "rnsap.dCHsInformationResponseList",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DCH_InformationResponseList_RL_ReconfRspTDD", HFILL }},
    { &hf_rnsap_DL_CCTrCH_InformationList_RL_ReconfRspTDD_item,
      { "Item", "rnsap.DL_CCTrCH_InformationList_RL_ReconfRspTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_CCTrCH_InformationItem_RL_ReconfRspTDD", HFILL }},
    { &hf_rnsap_dl_DPCH_ModifyInformation_LCR,
      { "dl-DPCH-ModifyInformation-LCR", "rnsap.dl_DPCH_ModifyInformation_LCR",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_DPCH_InformationModifyList_LCR_RL_ReconfRspTDD", HFILL }},
    { &hf_rnsap_cCTrCH_Maximum_DL_Power,
      { "cCTrCH-Maximum-DL-Power", "rnsap.cCTrCH_Maximum_DL_Power",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.DL_Power", HFILL }},
    { &hf_rnsap_cCTrCH_Minimum_DL_Power,
      { "cCTrCH-Minimum-DL-Power", "rnsap.cCTrCH_Minimum_DL_Power",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.DL_Power", HFILL }},
    { &hf_rnsap_dL_Timeslot_LCR_InformationModifyList_RL_ReconfRqstTDD,
      { "dL-Timeslot-LCR-InformationModifyList-RL-ReconfRqstTDD", "rnsap.dL_Timeslot_LCR_InformationModifyList_RL_ReconfRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DL_Timeslot_LCR_InformationModifyList_RL_ReconfRspTDD", HFILL }},
    { &hf_rnsap_DL_Timeslot_LCR_InformationModifyList_RL_ReconfRspTDD_item,
      { "Item", "rnsap.DL_Timeslot_LCR_InformationModifyList_RL_ReconfRspTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_Timeslot_LCR_InformationModifyItem_RL_ReconfRspTDD", HFILL }},
    { &hf_rnsap_maxPowerLCR,
      { "maxPowerLCR", "rnsap.maxPowerLCR",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.DL_Power", HFILL }},
    { &hf_rnsap_minPowerLCR,
      { "minPowerLCR", "rnsap.minPowerLCR",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.DL_Power", HFILL }},
    { &hf_rnsap_Multiple_RL_InformationResponse_RL_ReconfRspTDD_item,
      { "Item", "rnsap.Multiple_RL_InformationResponse_RL_ReconfRspTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_InformationResponse_RL_ReconfRspTDD", HFILL }},
    { &hf_rnsap_rL,
      { "rL", "rnsap.rL",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_RL_FailureInd", HFILL }},
    { &hf_rnsap_rL_Set,
      { "rL-Set", "rnsap.rL_Set",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_Set_RL_FailureInd", HFILL }},
    { &hf_rnsap_cCTrCH,
      { "cCTrCH", "rnsap.cCTrCH",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.CCTrCH_RL_FailureInd", HFILL }},
    { &hf_rnsap_rL_InformationList_RL_FailureInd,
      { "rL-InformationList-RL-FailureInd", "rnsap.rL_InformationList_RL_FailureInd",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RL_InformationList_RL_FailureInd", HFILL }},
    { &hf_rnsap_RL_InformationList_RL_FailureInd_item,
      { "Item", "rnsap.RL_InformationList_RL_FailureInd_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_rL_Set_InformationList_RL_FailureInd,
      { "rL-Set-InformationList-RL-FailureInd", "rnsap.rL_Set_InformationList_RL_FailureInd",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RL_Set_InformationList_RL_FailureInd", HFILL }},
    { &hf_rnsap_RL_Set_InformationList_RL_FailureInd_item,
      { "Item", "rnsap.RL_Set_InformationList_RL_FailureInd_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_cCTrCH_InformationList_RL_FailureInd,
      { "cCTrCH-InformationList-RL-FailureInd", "rnsap.cCTrCH_InformationList_RL_FailureInd",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.CCTrCH_InformationList_RL_FailureInd", HFILL }},
    { &hf_rnsap_CCTrCH_InformationList_RL_FailureInd_item,
      { "Item", "rnsap.CCTrCH_InformationList_RL_FailureInd_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_RL_InformationList_RL_PreemptRequiredInd_item,
      { "Item", "rnsap.RL_InformationList_RL_PreemptRequiredInd_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_HSDSCHMacdFlowSpecificInformationList_RL_PreemptRequiredInd_item,
      { "Item", "rnsap.HSDSCHMacdFlowSpecificInformationList_RL_PreemptRequiredInd_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_hSDSCH_MACdFlow_ID,
      { "hSDSCH-MACdFlow-ID", "rnsap.hSDSCH_MACdFlow_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.HSDSCH_MACdFlow_ID", HFILL }},
    { &hf_rnsap_EDCH_MacdFlowSpecificInformationList_RL_PreemptRequiredInd_item,
      { "Item", "rnsap.EDCH_MacdFlowSpecificInformationList_RL_PreemptRequiredInd_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_eDCH_MACdFlow_ID,
      { "eDCH-MACdFlow-ID", "rnsap.eDCH_MACdFlow_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.EDCH_MACdFlow_ID", HFILL }},
    { &hf_rnsap_rL1,
      { "rL", "rnsap.rL",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_RL_RestoreInd", HFILL }},
    { &hf_rnsap_rL_Set1,
      { "rL-Set", "rnsap.rL_Set",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_Set_RL_RestoreInd", HFILL }},
    { &hf_rnsap_cCTrCH1,
      { "cCTrCH", "rnsap.cCTrCH",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.CCTrCH_RL_RestoreInd", HFILL }},
    { &hf_rnsap_rL_InformationList_RL_RestoreInd,
      { "rL-InformationList-RL-RestoreInd", "rnsap.rL_InformationList_RL_RestoreInd",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RL_InformationList_RL_RestoreInd", HFILL }},
    { &hf_rnsap_RL_InformationList_RL_RestoreInd_item,
      { "Item", "rnsap.RL_InformationList_RL_RestoreInd_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_rL_Set_InformationList_RL_RestoreInd,
      { "rL-Set-InformationList-RL-RestoreInd", "rnsap.rL_Set_InformationList_RL_RestoreInd",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RL_Set_InformationList_RL_RestoreInd", HFILL }},
    { &hf_rnsap_RL_Set_InformationList_RL_RestoreInd_item,
      { "Item", "rnsap.RL_Set_InformationList_RL_RestoreInd_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_cCTrCH_InformationList_RL_RestoreInd,
      { "cCTrCH-InformationList-RL-RestoreInd", "rnsap.cCTrCH_InformationList_RL_RestoreInd",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.CCTrCH_InformationList_RL_RestoreInd", HFILL }},
    { &hf_rnsap_CCTrCH_InformationList_RL_RestoreInd_item,
      { "Item", "rnsap.CCTrCH_InformationList_RL_RestoreInd_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_DL_ReferencePowerInformationList_DL_PC_Rqst_item,
      { "Item", "rnsap.DL_ReferencePowerInformationList_DL_PC_Rqst_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_dl_Reference_Power,
      { "dl-Reference-Power", "rnsap.dl_Reference_Power",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.DL_Power", HFILL }},
    { &hf_rnsap_dl_CodeInformation3,
      { "dl-CodeInformation", "rnsap.dl_CodeInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_CodeInformationList_PhyChReconfRqstFDD", HFILL }},
    { &hf_rnsap_ul_CCTrCH_Information1,
      { "ul-CCTrCH-Information", "rnsap.ul_CCTrCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_CCTrCH_InformationList_PhyChReconfRqstTDD", HFILL }},
    { &hf_rnsap_dl_CCTrCH_Information1,
      { "dl-CCTrCH-Information", "rnsap.dl_CCTrCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_CCTrCH_InformationList_PhyChReconfRqstTDD", HFILL }},
    { &hf_rnsap_UL_CCTrCH_InformationListIE_PhyChReconfRqstTDD_item,
      { "Item", "rnsap.UL_CCTrCH_InformationListIE_PhyChReconfRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_CCTrCH_InformationItem_PhyChReconfRqstTDD", HFILL }},
    { &hf_rnsap_ul_DPCH_Information2,
      { "ul-DPCH-Information", "rnsap.ul_DPCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_DPCH_InformationList_PhyChReconfRqstTDD", HFILL }},
    { &hf_rnsap_uL_Timeslot_InformationList_PhyChReconfRqstTDD,
      { "uL-Timeslot-InformationList-PhyChReconfRqstTDD", "rnsap.uL_Timeslot_InformationList_PhyChReconfRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UL_Timeslot_InformationList_PhyChReconfRqstTDD", HFILL }},
    { &hf_rnsap_UL_TimeslotLCR_InformationList_PhyChReconfRqstTDD_item,
      { "Item", "rnsap.UL_TimeslotLCR_InformationList_PhyChReconfRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_TimeslotLCR_InformationItem_PhyChReconfRqstTDD", HFILL }},
    { &hf_rnsap_uL_Code_LCR_Information,
      { "uL-Code-LCR-Information", "rnsap.uL_Code_LCR_Information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.TDD_UL_Code_LCR_Information", HFILL }},
    { &hf_rnsap_UL_Timeslot_InformationList_PhyChReconfRqstTDD_item,
      { "Item", "rnsap.UL_Timeslot_InformationList_PhyChReconfRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_Timeslot_InformationItem_PhyChReconfRqstTDD", HFILL }},
    { &hf_rnsap_uL_Code_Information1,
      { "uL-Code-Information", "rnsap.uL_Code_Information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.TDD_UL_Code_Information", HFILL }},
    { &hf_rnsap_DL_CCTrCH_InformationListIE_PhyChReconfRqstTDD_item,
      { "Item", "rnsap.DL_CCTrCH_InformationListIE_PhyChReconfRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_CCTrCH_InformationItem_PhyChReconfRqstTDD", HFILL }},
    { &hf_rnsap_dl_DPCH_Information2,
      { "dl-DPCH-Information", "rnsap.dl_DPCH_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_DPCH_InformationList_PhyChReconfRqstTDD", HFILL }},
    { &hf_rnsap_dL_Timeslot_InformationList_PhyChReconfRqstTDD,
      { "dL-Timeslot-InformationList-PhyChReconfRqstTDD", "rnsap.dL_Timeslot_InformationList_PhyChReconfRqstTDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DL_Timeslot_InformationList_PhyChReconfRqstTDD", HFILL }},
    { &hf_rnsap_DL_TimeslotLCR_InformationList_PhyChReconfRqstTDD_item,
      { "Item", "rnsap.DL_TimeslotLCR_InformationList_PhyChReconfRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_TimeslotLCR_InformationItem_PhyChReconfRqstTDD", HFILL }},
    { &hf_rnsap_dL_Code_LCR_Information,
      { "dL-Code-LCR-Information", "rnsap.dL_Code_LCR_Information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.TDD_DL_Code_LCR_Information", HFILL }},
    { &hf_rnsap_DL_Timeslot_InformationList_PhyChReconfRqstTDD_item,
      { "Item", "rnsap.DL_Timeslot_InformationList_PhyChReconfRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_Timeslot_InformationItem_PhyChReconfRqstTDD", HFILL }},
    { &hf_rnsap_dL_Code_Information1,
      { "dL-Code-Information", "rnsap.dL_Code_Information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.TDD_DL_Code_Information", HFILL }},
    { &hf_rnsap_HSPDSCH_Timeslot_InformationList_PhyChReconfRqstTDD_item,
      { "Item", "rnsap.HSPDSCH_Timeslot_InformationList_PhyChReconfRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.HSPDSCH_Timeslot_InformationItem_PhyChReconfRqstTDD", HFILL }},
    { &hf_rnsap_timeslot,
      { "timeslot", "rnsap.timeslot",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.TimeSlot", HFILL }},
    { &hf_rnsap_HSPDSCH_Timeslot_InformationListLCR_PhyChReconfRqstTDD_item,
      { "Item", "rnsap.HSPDSCH_Timeslot_InformationListLCR_PhyChReconfRqstTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.HSPDSCH_Timeslot_InformationItemLCR_PhyChReconfRqstTDD", HFILL }},
    { &hf_rnsap_timeslotLCR,
      { "timeslotLCR", "rnsap.timeslotLCR",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.TimeSlotLCR", HFILL }},
    { &hf_rnsap_RL_InformationList_RL_CongestInd_item,
      { "Item", "rnsap.RL_InformationList_RL_CongestInd_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_dCH_Rate_Information,
      { "dCH-Rate-Information", "rnsap.dCH_Rate_Information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DCH_Rate_Information_RL_CongestInd", HFILL }},
    { &hf_rnsap_DCH_Rate_Information_RL_CongestInd_item,
      { "Item", "rnsap.DCH_Rate_Information_RL_CongestInd_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_allowed_Rate_Information,
      { "allowed-Rate-Information", "rnsap.allowed_Rate_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Allowed_Rate_Information", HFILL }},
    { &hf_rnsap_EDCH_MacdFlowSpecificInformationList_RL_CongestInd_item,
      { "Item", "rnsap.EDCH_MacdFlowSpecificInformationList_RL_CongestInd_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_uRA,
      { "uRA", "rnsap.uRA",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.URA_PagingRqst", HFILL }},
    { &hf_rnsap_cell,
      { "cell", "rnsap.cell",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Cell_PagingRqst", HFILL }},
    { &hf_rnsap_uRA_ID,
      { "uRA-ID", "rnsap.uRA_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.URA_ID", HFILL }},
    { &hf_rnsap_pagingCause,
      { "pagingCause", "rnsap.pagingCause",
        FT_UINT32, BASE_DEC, VALS(rnsap_PagingCause_vals), 0,
        "rnsap.PagingCause", HFILL }},
    { &hf_rnsap_cNDomainType,
      { "cNDomainType", "rnsap.cNDomainType",
        FT_UINT32, BASE_DEC, VALS(rnsap_CNDomainType_vals), 0,
        "rnsap.CNDomainType", HFILL }},
    { &hf_rnsap_pagingRecordType,
      { "pagingRecordType", "rnsap.pagingRecordType",
        FT_UINT32, BASE_DEC, VALS(rnsap_PagingRecordType_vals), 0,
        "rnsap.PagingRecordType", HFILL }},
    { &hf_rnsap_rL2,
      { "rL", "rnsap.rL",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_DM_Rqst", HFILL }},
    { &hf_rnsap_rLS,
      { "rLS", "rnsap.rLS",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_Set_DM_Rqst", HFILL }},
    { &hf_rnsap_allRL,
      { "allRL", "rnsap.allRL",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.All_RL_DM_Rqst", HFILL }},
    { &hf_rnsap_allRLS,
      { "allRLS", "rnsap.allRLS",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.All_RL_Set_DM_Rqst", HFILL }},
    { &hf_rnsap_rL_InformationList_DM_Rqst,
      { "rL-InformationList-DM-Rqst", "rnsap.rL_InformationList_DM_Rqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RL_InformationList_DM_Rqst", HFILL }},
    { &hf_rnsap_RL_InformationList_DM_Rqst_item,
      { "Item", "rnsap.RL_InformationList_DM_Rqst_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_HSSICH_Info_DM_Rqst_item,
      { "Item", "rnsap.HSSICH_Info_DM_Rqst_item",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.HS_SICH_ID", HFILL }},
    { &hf_rnsap_rL_Set_InformationList_DM_Rqst,
      { "rL-Set-InformationList-DM-Rqst", "rnsap.rL_Set_InformationList_DM_Rqst",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RL_Set_InformationList_DM_Rqst", HFILL }},
    { &hf_rnsap_RL_Set_InformationList_DM_Rqst_item,
      { "Item", "rnsap.RL_Set_InformationList_DM_Rqst_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_rLs,
      { "rLs", "rnsap.rLs",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_DM_Rsp", HFILL }},
    { &hf_rnsap_rLS1,
      { "rLS", "rnsap.rLS",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_Set_DM_Rsp", HFILL }},
    { &hf_rnsap_allRL1,
      { "allRL", "rnsap.allRL",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_DM_Rsp", HFILL }},
    { &hf_rnsap_allRLS1,
      { "allRLS", "rnsap.allRLS",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_Set_DM_Rsp", HFILL }},
    { &hf_rnsap_rL_InformationList_DM_Rsp,
      { "rL-InformationList-DM-Rsp", "rnsap.rL_InformationList_DM_Rsp",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RL_InformationList_DM_Rsp", HFILL }},
    { &hf_rnsap_rL_Set_InformationList_DM_Rsp,
      { "rL-Set-InformationList-DM-Rsp", "rnsap.rL_Set_InformationList_DM_Rsp",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RL_Set_InformationList_DM_Rsp", HFILL }},
    { &hf_rnsap_RL_InformationList_DM_Rsp_item,
      { "Item", "rnsap.RL_InformationList_DM_Rsp_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_dedicatedMeasurementValue,
      { "dedicatedMeasurementValue", "rnsap.dedicatedMeasurementValue",
        FT_UINT32, BASE_DEC, VALS(rnsap_DedicatedMeasurementValue_vals), 0,
        "rnsap.DedicatedMeasurementValue", HFILL }},
    { &hf_rnsap_cFN,
      { "cFN", "rnsap.cFN",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.CFN", HFILL }},
    { &hf_rnsap_RL_Set_InformationList_DM_Rsp_item,
      { "Item", "rnsap.RL_Set_InformationList_DM_Rsp_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_Multiple_DedicatedMeasurementValueList_TDD_DM_Rsp_item,
      { "Item", "rnsap.Multiple_DedicatedMeasurementValueList_TDD_DM_Rsp_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Multiple_DedicatedMeasurementValueItem_TDD_DM_Rsp", HFILL }},
    { &hf_rnsap_Multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp_item,
      { "Item", "rnsap.Multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Multiple_DedicatedMeasurementValueItem_LCR_TDD_DM_Rsp", HFILL }},
    { &hf_rnsap_rL3,
      { "rL", "rnsap.rL",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_DM_Fail", HFILL }},
    { &hf_rnsap_rLS2,
      { "rLS", "rnsap.rLS",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_Set_DM_Fail", HFILL }},
    { &hf_rnsap_allRL2,
      { "allRL", "rnsap.allRL",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_DM_Fail", HFILL }},
    { &hf_rnsap_allRLS2,
      { "allRLS", "rnsap.allRLS",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_Set_DM_Fail", HFILL }},
    { &hf_rnsap_rL_unsuccessful_InformationRespList_DM_Fail,
      { "rL-unsuccessful-InformationRespList-DM-Fail", "rnsap.rL_unsuccessful_InformationRespList_DM_Fail",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RL_Unsuccessful_InformationRespList_DM_Fail", HFILL }},
    { &hf_rnsap_rL_successful_InformationRespList_DM_Fail,
      { "rL-successful-InformationRespList-DM-Fail", "rnsap.rL_successful_InformationRespList_DM_Fail",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RL_Successful_InformationRespList_DM_Fail", HFILL }},
    { &hf_rnsap_rL_Set_unsuccessful_InformationRespList_DM_Fail,
      { "rL-Set-unsuccessful-InformationRespList-DM-Fail", "rnsap.rL_Set_unsuccessful_InformationRespList_DM_Fail",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RL_Set_Unsuccessful_InformationRespList_DM_Fail", HFILL }},
    { &hf_rnsap_rL_Set_successful_InformationRespList_DM_Fail,
      { "rL-Set-successful-InformationRespList-DM-Fail", "rnsap.rL_Set_successful_InformationRespList_DM_Fail",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RL_Set_Successful_InformationRespList_DM_Fail", HFILL }},
    { &hf_rnsap_RL_Unsuccessful_InformationRespList_DM_Fail_item,
      { "Item", "rnsap.RL_Unsuccessful_InformationRespList_DM_Fail_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_individualcause,
      { "individualcause", "rnsap.individualcause",
        FT_UINT32, BASE_DEC, VALS(rnsap_Cause_vals), 0,
        "rnsap.Cause", HFILL }},
    { &hf_rnsap_RL_Successful_InformationRespList_DM_Fail_item,
      { "Item", "rnsap.RL_Successful_InformationRespList_DM_Fail_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_RL_Set_Unsuccessful_InformationRespList_DM_Fail_item,
      { "Item", "rnsap.RL_Set_Unsuccessful_InformationRespList_DM_Fail_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_RL_Set_Successful_InformationRespList_DM_Fail_item,
      { "Item", "rnsap.RL_Set_Successful_InformationRespList_DM_Fail_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_rLs1,
      { "rLs", "rnsap.rLs",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_DM_Rprt", HFILL }},
    { &hf_rnsap_rLS3,
      { "rLS", "rnsap.rLS",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_Set_DM_Rprt", HFILL }},
    { &hf_rnsap_allRL3,
      { "allRL", "rnsap.allRL",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_DM_Rprt", HFILL }},
    { &hf_rnsap_allRLS3,
      { "allRLS", "rnsap.allRLS",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_Set_DM_Rprt", HFILL }},
    { &hf_rnsap_rL_InformationList_DM_Rprt,
      { "rL-InformationList-DM-Rprt", "rnsap.rL_InformationList_DM_Rprt",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RL_InformationList_DM_Rprt", HFILL }},
    { &hf_rnsap_rL_Set_InformationList_DM_Rprt,
      { "rL-Set-InformationList-DM-Rprt", "rnsap.rL_Set_InformationList_DM_Rprt",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RL_Set_InformationList_DM_Rprt", HFILL }},
    { &hf_rnsap_RL_InformationList_DM_Rprt_item,
      { "Item", "rnsap.RL_InformationList_DM_Rprt_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_dedicatedMeasurementValueInformation,
      { "dedicatedMeasurementValueInformation", "rnsap.dedicatedMeasurementValueInformation",
        FT_UINT32, BASE_DEC, VALS(rnsap_DedicatedMeasurementValueInformation_vals), 0,
        "rnsap.DedicatedMeasurementValueInformation", HFILL }},
    { &hf_rnsap_RL_Set_InformationList_DM_Rprt_item,
      { "Item", "rnsap.RL_Set_InformationList_DM_Rprt_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_rL4,
      { "rL", "rnsap.rL",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_DM_Fail_Ind", HFILL }},
    { &hf_rnsap_rLS4,
      { "rLS", "rnsap.rLS",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_Set_DM_Fail_Ind", HFILL }},
    { &hf_rnsap_allRL4,
      { "allRL", "rnsap.allRL",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_DM_Fail_Ind", HFILL }},
    { &hf_rnsap_allRLS4,
      { "allRLS", "rnsap.allRLS",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_Set_DM_Fail_Ind", HFILL }},
    { &hf_rnsap_rL_unsuccessful_InformationRespList_DM_Fail_Ind,
      { "rL-unsuccessful-InformationRespList-DM-Fail-Ind", "rnsap.rL_unsuccessful_InformationRespList_DM_Fail_Ind",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RL_Unsuccessful_InformationRespList_DM_Fail_Ind", HFILL }},
    { &hf_rnsap_rL_Set_unsuccessful_InformationRespList_DM_Fail_Ind,
      { "rL-Set-unsuccessful-InformationRespList-DM-Fail-Ind", "rnsap.rL_Set_unsuccessful_InformationRespList_DM_Fail_Ind",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RL_Set_Unsuccessful_InformationRespList_DM_Fail_Ind", HFILL }},
    { &hf_rnsap_RL_Unsuccessful_InformationRespList_DM_Fail_Ind_item,
      { "Item", "rnsap.RL_Unsuccessful_InformationRespList_DM_Fail_Ind_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_RL_Set_Unsuccessful_InformationRespList_DM_Fail_Ind_item,
      { "Item", "rnsap.RL_Set_Unsuccessful_InformationRespList_DM_Fail_Ind_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_fACH_FlowControlInformation,
      { "fACH-FlowControlInformation", "rnsap.fACH_FlowControlInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.FACH_FlowControlInformation_CTCH_ResourceRspFDD", HFILL }},
    { &hf_rnsap_fACH_FlowControlInformation1,
      { "fACH-FlowControlInformation", "rnsap.fACH_FlowControlInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.FACH_FlowControlInformation_CTCH_ResourceRspTDD", HFILL }},
    { &hf_rnsap_cell1,
      { "cell", "rnsap.cell",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Cell_CM_Rqst", HFILL }},
    { &hf_rnsap_uC_ID,
      { "uC-ID", "rnsap.uC_ID",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UC_ID", HFILL }},
    { &hf_rnsap_neighbouringCellMeasurementInformation,
      { "neighbouringCellMeasurementInformation", "rnsap.neighbouringCellMeasurementInformation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.NeighbouringCellMeasurementInfo", HFILL }},
    { &hf_rnsap_NeighbouringCellMeasurementInfo_item,
      { "Item", "rnsap.NeighbouringCellMeasurementInfo_item",
        FT_UINT32, BASE_DEC, VALS(rnsap_NeighbouringCellMeasurementInfo_item_vals), 0,
        "rnsap.NeighbouringCellMeasurementInfo_item", HFILL }},
    { &hf_rnsap_neighbouringFDDCellMeasurementInformation,
      { "neighbouringFDDCellMeasurementInformation", "rnsap.neighbouringFDDCellMeasurementInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.NeighbouringFDDCellMeasurementInformation", HFILL }},
    { &hf_rnsap_neighbouringTDDCellMeasurementInformation,
      { "neighbouringTDDCellMeasurementInformation", "rnsap.neighbouringTDDCellMeasurementInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.NeighbouringTDDCellMeasurementInformation", HFILL }},
    { &hf_rnsap_extension_neighbouringCellMeasurementInformation,
      { "extension-neighbouringCellMeasurementInformation", "rnsap.extension_neighbouringCellMeasurementInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Extension_neighbouringCellMeasurementInformation", HFILL }},
    { &hf_rnsap_cell2,
      { "cell", "rnsap.cell",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Cell_CM_Rsp", HFILL }},
    { &hf_rnsap_commonMeasurementValue,
      { "commonMeasurementValue", "rnsap.commonMeasurementValue",
        FT_UINT32, BASE_DEC, VALS(rnsap_CommonMeasurementValue_vals), 0,
        "rnsap.CommonMeasurementValue", HFILL }},
    { &hf_rnsap_cell3,
      { "cell", "rnsap.cell",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Cell_CM_Rprt", HFILL }},
    { &hf_rnsap_commonMeasurementValueInformation,
      { "commonMeasurementValueInformation", "rnsap.commonMeasurementValueInformation",
        FT_UINT32, BASE_DEC, VALS(rnsap_CommonMeasurementValueInformation_vals), 0,
        "rnsap.CommonMeasurementValueInformation", HFILL }},
    { &hf_rnsap_cell4,
      { "cell", "rnsap.cell",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Cell_InfEx_Rqst", HFILL }},
    { &hf_rnsap_extension_InformationExchangeObjectType_InfEx_Rqst,
      { "extension-InformationExchangeObjectType-InfEx-Rqst", "rnsap.extension_InformationExchangeObjectType_InfEx_Rqst",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Extension_InformationExchangeObjectType_InfEx_Rqst", HFILL }},
    { &hf_rnsap_cGI,
      { "cGI", "rnsap.cGI",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.CGI", HFILL }},
    { &hf_rnsap_cell5,
      { "cell", "rnsap.cell",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Cell_InfEx_Rsp", HFILL }},
    { &hf_rnsap_extension_InformationExchangeObjectType_InfEx_Rsp,
      { "extension-InformationExchangeObjectType-InfEx-Rsp", "rnsap.extension_InformationExchangeObjectType_InfEx_Rsp",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Extension_InformationExchangeObjectType_InfEx_Rsp", HFILL }},
    { &hf_rnsap_requestedDataValue,
      { "requestedDataValue", "rnsap.requestedDataValue",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RequestedDataValue", HFILL }},
    { &hf_rnsap_MBMS_Bearer_Service_List_InfEx_Rsp_item,
      { "Item", "rnsap.MBMS_Bearer_Service_List_InfEx_Rsp_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.MBMS_Bearer_ServiceItemIEs_InfEx_Rsp", HFILL }},
    { &hf_rnsap_tmgi,
      { "tmgi", "rnsap.tmgi",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.TMGI", HFILL }},
    { &hf_rnsap_cell6,
      { "cell", "rnsap.cell",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Cell_InfEx_Rprt", HFILL }},
    { &hf_rnsap_requestedDataValueInformation,
      { "requestedDataValueInformation", "rnsap.requestedDataValueInformation",
        FT_UINT32, BASE_DEC, VALS(rnsap_RequestedDataValueInformation_vals), 0,
        "rnsap.RequestedDataValueInformation", HFILL }},
    { &hf_rnsap_context,
      { "context", "rnsap.context",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ContextList_Reset", HFILL }},
    { &hf_rnsap_all_contexts,
      { "all-contexts", "rnsap.all_contexts",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.NULL", HFILL }},
    { &hf_rnsap_contextGroup,
      { "contextGroup", "rnsap.contextGroup",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ContextGroupList_Reset", HFILL }},
    { &hf_rnsap_contextInfoList_Reset,
      { "contextInfoList-Reset", "rnsap.contextInfoList_Reset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.ContextInfoList_Reset", HFILL }},
    { &hf_rnsap_ContextInfoList_Reset_item,
      { "Item", "rnsap.ContextInfoList_Reset_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_contextType_Reset,
      { "contextType-Reset", "rnsap.contextType_Reset",
        FT_UINT32, BASE_DEC, VALS(rnsap_ContextType_Reset_vals), 0,
        "rnsap.ContextType_Reset", HFILL }},
    { &hf_rnsap_sRNTI,
      { "sRNTI", "rnsap.sRNTI",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.S_RNTI", HFILL }},
    { &hf_rnsap_dRNTI,
      { "dRNTI", "rnsap.dRNTI",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.D_RNTI", HFILL }},
    { &hf_rnsap_contextGroupInfoList_Reset,
      { "contextGroupInfoList-Reset", "rnsap.contextGroupInfoList_Reset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.ContextGroupInfoList_Reset", HFILL }},
    { &hf_rnsap_ContextGroupInfoList_Reset_item,
      { "Item", "rnsap.ContextGroupInfoList_Reset_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_s_RNTI_Group,
      { "s-RNTI-Group", "rnsap.s_RNTI_Group",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.S_RNTI_Group", HFILL }},
    { &hf_rnsap_DelayedActivationInformationList_RL_ActivationCmdFDD_item,
      { "Item", "rnsap.DelayedActivationInformationList_RL_ActivationCmdFDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_delayed_activation_update,
      { "delayed-activation-update", "rnsap.delayed_activation_update",
        FT_UINT32, BASE_DEC, VALS(rnsap_DelayedActivationUpdate_vals), 0,
        "rnsap.DelayedActivationUpdate", HFILL }},
    { &hf_rnsap_DelayedActivationInformationList_RL_ActivationCmdTDD_item,
      { "Item", "rnsap.DelayedActivationInformationList_RL_ActivationCmdTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_RL_ParameterUpdateIndicationFDD_RL_InformationList_item,
      { "Item", "rnsap.RL_ParameterUpdateIndicationFDD_RL_InformationList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_phase_Reference_Update_Indicator,
      { "phase-Reference-Update-Indicator", "rnsap.phase_Reference_Update_Indicator",
        FT_UINT32, BASE_DEC, VALS(rnsap_Phase_Reference_Update_Indicator_vals), 0,
        "rnsap.Phase_Reference_Update_Indicator", HFILL }},
    { &hf_rnsap_ListOfInterfacesToTrace_item,
      { "Item", "rnsap.ListOfInterfacesToTrace_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_interface,
      { "interface", "rnsap.interface",
        FT_UINT32, BASE_DEC, VALS(rnsap_T_interface_vals), 0,
        "rnsap.T_interface", HFILL }},
    { &hf_rnsap_privateIEs,
      { "privateIEs", "rnsap.privateIEs",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.PrivateIE_Container", HFILL }},
    { &hf_rnsap_Active_MBMS_Bearer_Service_ListFDD_item,
      { "Item", "rnsap.Active_MBMS_Bearer_Service_ListFDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.MBMS_Bearer_ServiceItemFDD", HFILL }},
    { &hf_rnsap_Active_MBMS_Bearer_Service_ListFDD_PFL_item,
      { "Item", "rnsap.Active_MBMS_Bearer_Service_ListFDD_PFL_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.MBMS_Bearer_ServiceItemFDD_PFL", HFILL }},
    { &hf_rnsap_Active_MBMS_Bearer_Service_ListTDD_item,
      { "Item", "rnsap.Active_MBMS_Bearer_Service_ListTDD_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.MBMS_Bearer_ServiceItemTDD", HFILL }},
    { &hf_rnsap_Active_MBMS_Bearer_Service_ListTDD_PFL_item,
      { "Item", "rnsap.Active_MBMS_Bearer_Service_ListTDD_PFL_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.MBMS_Bearer_ServiceItemTDD_PFL", HFILL }},
    { &hf_rnsap_cMConfigurationChangeCFN,
      { "cMConfigurationChangeCFN", "rnsap.cMConfigurationChangeCFN",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.CFN", HFILL }},
    { &hf_rnsap_transmission_Gap_Pattern_Sequence_Status,
      { "transmission-Gap-Pattern-Sequence-Status", "rnsap.transmission_Gap_Pattern_Sequence_Status",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Transmission_Gap_Pattern_Sequence_Status_List", HFILL }},
    { &hf_rnsap_AdditionalPreferredFrequency_item,
      { "Item", "rnsap.AdditionalPreferredFrequency_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.AdditionalPreferredFrequencyItem", HFILL }},
    { &hf_rnsap_correspondingCells,
      { "correspondingCells", "rnsap.correspondingCells",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.CorrespondingCells", HFILL }},
    { &hf_rnsap_AffectedUEInformationForMBMS_item,
      { "Item", "rnsap.AffectedUEInformationForMBMS_item",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.S_RNTI", HFILL }},
    { &hf_rnsap_priorityLevel,
      { "priorityLevel", "rnsap.priorityLevel",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.PriorityLevel", HFILL }},
    { &hf_rnsap_pre_emptionCapability,
      { "pre-emptionCapability", "rnsap.pre_emptionCapability",
        FT_UINT32, BASE_DEC, VALS(rnsap_Pre_emptionCapability_vals), 0,
        "rnsap.Pre_emptionCapability", HFILL }},
    { &hf_rnsap_pre_emptionVulnerability,
      { "pre-emptionVulnerability", "rnsap.pre_emptionVulnerability",
        FT_UINT32, BASE_DEC, VALS(rnsap_Pre_emptionVulnerability_vals), 0,
        "rnsap.Pre_emptionVulnerability", HFILL }},
    { &hf_rnsap_allowed_UL_Rate,
      { "allowed-UL-Rate", "rnsap.allowed_UL_Rate",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Allowed_Rate", HFILL }},
    { &hf_rnsap_allowed_DL_Rate,
      { "allowed-DL-Rate", "rnsap.allowed_DL_Rate",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Allowed_Rate", HFILL }},
    { &hf_rnsap_aOA_LCR,
      { "aOA-LCR", "rnsap.aOA_LCR",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.AOA_LCR", HFILL }},
    { &hf_rnsap_aOA_LCR_Accuracy_Class,
      { "aOA-LCR-Accuracy-Class", "rnsap.aOA_LCR_Accuracy_Class",
        FT_UINT32, BASE_DEC, VALS(rnsap_AOA_LCR_Accuracy_Class_vals), 0,
        "rnsap.AOA_LCR_Accuracy_Class", HFILL }},
    { &hf_rnsap_badSatelliteInformation,
      { "badSatelliteInformation", "rnsap.badSatelliteInformation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.T_badSatelliteInformation", HFILL }},
    { &hf_rnsap_badSatelliteInformation_item,
      { "Item", "rnsap.badSatelliteInformation_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.T_badSatelliteInformation_item", HFILL }},
    { &hf_rnsap_badSAT_ID,
      { "badSAT-ID", "rnsap.badSAT_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.SAT_ID", HFILL }},
    { &hf_rnsap_nCC,
      { "nCC", "rnsap.nCC",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.NCC", HFILL }},
    { &hf_rnsap_bCC,
      { "bCC", "rnsap.bCC",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BCC", HFILL }},
    { &hf_rnsap_burstStart,
      { "burstStart", "rnsap.burstStart",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_0_15", HFILL }},
    { &hf_rnsap_burstLength,
      { "burstLength", "rnsap.burstLength",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_10_25", HFILL }},
    { &hf_rnsap_burstFreq,
      { "burstFreq", "rnsap.burstFreq",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_1_16", HFILL }},
    { &hf_rnsap_radioNetwork,
      { "radioNetwork", "rnsap.radioNetwork",
        FT_UINT32, BASE_DEC, VALS(rnsap_CauseRadioNetwork_vals), 0,
        "rnsap.CauseRadioNetwork", HFILL }},
    { &hf_rnsap_transport,
      { "transport", "rnsap.transport",
        FT_UINT32, BASE_DEC, VALS(rnsap_CauseTransport_vals), 0,
        "rnsap.CauseTransport", HFILL }},
    { &hf_rnsap_protocol,
      { "protocol", "rnsap.protocol",
        FT_UINT32, BASE_DEC, VALS(rnsap_CauseProtocol_vals), 0,
        "rnsap.CauseProtocol", HFILL }},
    { &hf_rnsap_misc,
      { "misc", "rnsap.misc",
        FT_UINT32, BASE_DEC, VALS(rnsap_CauseMisc_vals), 0,
        "rnsap.CauseMisc", HFILL }},
    { &hf_rnsap_uplinkCellCapacityClassValue,
      { "uplinkCellCapacityClassValue", "rnsap.uplinkCellCapacityClassValue",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_1_100_", HFILL }},
    { &hf_rnsap_downlinkCellCapacityClassValue,
      { "downlinkCellCapacityClassValue", "rnsap.downlinkCellCapacityClassValue",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_1_100_", HFILL }},
    { &hf_rnsap_lAI,
      { "lAI", "rnsap.lAI",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.T_lAI", HFILL }},
    { &hf_rnsap_pLMN_Identity,
      { "pLMN-Identity", "rnsap.pLMN_Identity",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.PLMN_Identity", HFILL }},
    { &hf_rnsap_lAC,
      { "lAC", "rnsap.lAC",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.LAC", HFILL }},
    { &hf_rnsap_cI,
      { "cI", "rnsap.cI",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.CI", HFILL }},
    { &hf_rnsap_tUTRANGPSMeasurementAccuracyClass,
      { "tUTRANGPSMeasurementAccuracyClass", "rnsap.tUTRANGPSMeasurementAccuracyClass",
        FT_UINT32, BASE_DEC, VALS(rnsap_TUTRANGPSAccuracyClass_vals), 0,
        "rnsap.TUTRANGPSAccuracyClass", HFILL }},
    { &hf_rnsap_tUTRANGPSMeasurementValueInformation,
      { "tUTRANGPSMeasurementValueInformation", "rnsap.tUTRANGPSMeasurementValueInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.TUTRANGPSMeasurementValueInformation", HFILL }},
    { &hf_rnsap_sFNSFNMeasurementValueInformation,
      { "sFNSFNMeasurementValueInformation", "rnsap.sFNSFNMeasurementValueInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.SFNSFNMeasurementValueInformation", HFILL }},
    { &hf_rnsap_loadValue,
      { "loadValue", "rnsap.loadValue",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.LoadValue", HFILL }},
    { &hf_rnsap_transmittedCarrierPowerValue,
      { "transmittedCarrierPowerValue", "rnsap.transmittedCarrierPowerValue",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_0_100", HFILL }},
    { &hf_rnsap_receivedTotalWideBandPowerValue,
      { "receivedTotalWideBandPowerValue", "rnsap.receivedTotalWideBandPowerValue",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_0_621", HFILL }},
    { &hf_rnsap_uplinkTimeslotISCPValue,
      { "uplinkTimeslotISCPValue", "rnsap.uplinkTimeslotISCPValue",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UL_TimeslotISCP", HFILL }},
    { &hf_rnsap_extension_CommonMeasurementValue,
      { "extension-CommonMeasurementValue", "rnsap.extension_CommonMeasurementValue",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Extension_CommonMeasurementValue", HFILL }},
    { &hf_rnsap_measurementAvailable,
      { "measurementAvailable", "rnsap.measurementAvailable",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.CommonMeasurementAvailable", HFILL }},
    { &hf_rnsap_measurementnotAvailable,
      { "measurementnotAvailable", "rnsap.measurementnotAvailable",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.NULL", HFILL }},
    { &hf_rnsap_CorrespondingCells_item,
      { "Item", "rnsap.CorrespondingCells_item",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.C_ID", HFILL }},
    { &hf_rnsap_triggeringMessage,
      { "triggeringMessage", "rnsap.triggeringMessage",
        FT_UINT32, BASE_DEC, VALS(rnsap_TriggeringMessage_vals), 0,
        "rnsap.TriggeringMessage", HFILL }},
    { &hf_rnsap_procedureCriticality,
      { "procedureCriticality", "rnsap.procedureCriticality",
        FT_UINT32, BASE_DEC, VALS(rnsap_Criticality_vals), 0,
        "rnsap.Criticality", HFILL }},
    { &hf_rnsap_iEsCriticalityDiagnostics,
      { "iEsCriticalityDiagnostics", "rnsap.iEsCriticalityDiagnostics",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.CriticalityDiagnostics_IE_List", HFILL }},
    { &hf_rnsap_CriticalityDiagnostics_IE_List_item,
      { "Item", "rnsap.CriticalityDiagnostics_IE_List_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.CriticalityDiagnostics_IE_List_item", HFILL }},
    { &hf_rnsap_iECriticality,
      { "iECriticality", "rnsap.iECriticality",
        FT_UINT32, BASE_DEC, VALS(rnsap_Criticality_vals), 0,
        "rnsap.Criticality", HFILL }},
    { &hf_rnsap_iE_ID,
      { "iE-ID", "rnsap.iE_ID",
        FT_UINT32, BASE_DEC, VALS(rnsap_ProtocolIE_ID_vals), 0,
        "rnsap.ProtocolIE_ID", HFILL }},
    { &hf_rnsap_repetitionNumber,
      { "repetitionNumber", "rnsap.repetitionNumber",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RepetitionNumber0", HFILL }},
    { &hf_rnsap_MessageStructure_item,
      { "Item", "rnsap.MessageStructure_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.MessageStructure_item", HFILL }},
    { &hf_rnsap_repetitionNumber1,
      { "repetitionNumber", "rnsap.repetitionNumber",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RepetitionNumber1", HFILL }},
    { &hf_rnsap_rAC,
      { "rAC", "rnsap.rAC",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.RAC", HFILL }},
    { &hf_rnsap_DCH_FDD_Information_item,
      { "Item", "rnsap.DCH_FDD_Information_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DCH_FDD_InformationItem", HFILL }},
    { &hf_rnsap_payloadCRC_PresenceIndicator,
      { "payloadCRC-PresenceIndicator", "rnsap.payloadCRC_PresenceIndicator",
        FT_UINT32, BASE_DEC, VALS(rnsap_PayloadCRC_PresenceIndicator_vals), 0,
        "rnsap.PayloadCRC_PresenceIndicator", HFILL }},
    { &hf_rnsap_ul_FP_Mode,
      { "ul-FP-Mode", "rnsap.ul_FP_Mode",
        FT_UINT32, BASE_DEC, VALS(rnsap_UL_FP_Mode_vals), 0,
        "rnsap.UL_FP_Mode", HFILL }},
    { &hf_rnsap_toAWS,
      { "toAWS", "rnsap.toAWS",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.ToAWS", HFILL }},
    { &hf_rnsap_toAWE,
      { "toAWE", "rnsap.toAWE",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.ToAWE", HFILL }},
    { &hf_rnsap_dCH_SpecificInformationList,
      { "dCH-SpecificInformationList", "rnsap.dCH_SpecificInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DCH_Specific_FDD_InformationList", HFILL }},
    { &hf_rnsap_DCH_Specific_FDD_InformationList_item,
      { "Item", "rnsap.DCH_Specific_FDD_InformationList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DCH_Specific_FDD_Item", HFILL }},
    { &hf_rnsap_trCH_SrcStatisticsDescr,
      { "trCH-SrcStatisticsDescr", "rnsap.trCH_SrcStatisticsDescr",
        FT_UINT32, BASE_DEC, VALS(rnsap_TrCH_SrcStatisticsDescr_vals), 0,
        "rnsap.TrCH_SrcStatisticsDescr", HFILL }},
    { &hf_rnsap_ul_transportFormatSet,
      { "ul-transportFormatSet", "rnsap.ul_transportFormatSet",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.TransportFormatSet", HFILL }},
    { &hf_rnsap_dl_transportFormatSet,
      { "dl-transportFormatSet", "rnsap.dl_transportFormatSet",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.TransportFormatSet", HFILL }},
    { &hf_rnsap_ul_BLER,
      { "ul-BLER", "rnsap.ul_BLER",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.BLER", HFILL }},
    { &hf_rnsap_dl_BLER,
      { "dl-BLER", "rnsap.dl_BLER",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.BLER", HFILL }},
    { &hf_rnsap_frameHandlingPriority,
      { "frameHandlingPriority", "rnsap.frameHandlingPriority",
        FT_UINT32, BASE_DEC, VALS(rnsap_FrameHandlingPriority_vals), 0,
        "rnsap.FrameHandlingPriority", HFILL }},
    { &hf_rnsap_qE_Selector,
      { "qE-Selector", "rnsap.qE_Selector",
        FT_UINT32, BASE_DEC, VALS(rnsap_QE_Selector_vals), 0,
        "rnsap.QE_Selector", HFILL }},
    { &hf_rnsap_dRACControl,
      { "dRACControl", "rnsap.dRACControl",
        FT_UINT32, BASE_DEC, VALS(rnsap_DRACControl_vals), 0,
        "rnsap.DRACControl", HFILL }},
    { &hf_rnsap_DCH_InformationResponse_item,
      { "Item", "rnsap.DCH_InformationResponse_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DCH_InformationResponseItem", HFILL }},
    { &hf_rnsap_DCH_TDD_Information_item,
      { "Item", "rnsap.DCH_TDD_Information_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DCH_TDD_InformationItem", HFILL }},
    { &hf_rnsap_dCH_SpecificInformationList1,
      { "dCH-SpecificInformationList", "rnsap.dCH_SpecificInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DCH_Specific_TDD_InformationList", HFILL }},
    { &hf_rnsap_DCH_Specific_TDD_InformationList_item,
      { "Item", "rnsap.DCH_Specific_TDD_InformationList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DCH_Specific_TDD_Item", HFILL }},
    { &hf_rnsap_ul_cCTrCH_ID,
      { "ul-cCTrCH-ID", "rnsap.ul_cCTrCH_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.CCTrCH_ID", HFILL }},
    { &hf_rnsap_dl_cCTrCH_ID,
      { "dl-cCTrCH-ID", "rnsap.dl_cCTrCH_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.CCTrCH_ID", HFILL }},
    { &hf_rnsap_sIR_Value,
      { "sIR-Value", "rnsap.sIR_Value",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.SIR_Value", HFILL }},
    { &hf_rnsap_sIR_ErrorValue,
      { "sIR-ErrorValue", "rnsap.sIR_ErrorValue",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.SIR_Error_Value", HFILL }},
    { &hf_rnsap_transmittedCodePowerValue,
      { "transmittedCodePowerValue", "rnsap.transmittedCodePowerValue",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Transmitted_Code_Power_Value", HFILL }},
    { &hf_rnsap_rSCP,
      { "rSCP", "rnsap.rSCP",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RSCP_Value", HFILL }},
    { &hf_rnsap_rxTimingDeviationValue,
      { "rxTimingDeviationValue", "rnsap.rxTimingDeviationValue",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Rx_Timing_Deviation_Value", HFILL }},
    { &hf_rnsap_roundTripTime,
      { "roundTripTime", "rnsap.roundTripTime",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Round_Trip_Time_Value", HFILL }},
    { &hf_rnsap_extension_DedicatedMeasurementValue,
      { "extension-DedicatedMeasurementValue", "rnsap.extension_DedicatedMeasurementValue",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Extension_DedicatedMeasurementValue", HFILL }},
    { &hf_rnsap_measurementAvailable1,
      { "measurementAvailable", "rnsap.measurementAvailable",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DedicatedMeasurementAvailable", HFILL }},
    { &hf_rnsap_measurementnotAvailable1,
      { "measurementnotAvailable", "rnsap.measurementnotAvailable",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DedicatedMeasurementnotAvailable", HFILL }},
    { &hf_rnsap_dedicatedmeasurementValue,
      { "dedicatedmeasurementValue", "rnsap.dedicatedmeasurementValue",
        FT_UINT32, BASE_DEC, VALS(rnsap_DedicatedMeasurementValue_vals), 0,
        "rnsap.DedicatedMeasurementValue", HFILL }},
    { &hf_rnsap_cfn,
      { "cfn", "rnsap.cfn",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.CFN", HFILL }},
    { &hf_rnsap_separate_indication,
      { "separate-indication", "rnsap.separate_indication",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.NULL", HFILL }},
    { &hf_rnsap_activate,
      { "activate", "rnsap.activate",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Activate_Info", HFILL }},
    { &hf_rnsap_deactivate,
      { "deactivate", "rnsap.deactivate",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Deactivate_Info", HFILL }},
    { &hf_rnsap_activation_type,
      { "activation-type", "rnsap.activation_type",
        FT_UINT32, BASE_DEC, VALS(rnsap_Execution_Type_vals), 0,
        "rnsap.Execution_Type", HFILL }},
    { &hf_rnsap_initial_dl_tx_power,
      { "initial-dl-tx-power", "rnsap.initial_dl_tx_power",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.DL_Power", HFILL }},
    { &hf_rnsap_firstRLS_Indicator,
      { "firstRLS-Indicator", "rnsap.firstRLS_Indicator",
        FT_UINT32, BASE_DEC, VALS(rnsap_FirstRLS_Indicator_vals), 0,
        "rnsap.FirstRLS_Indicator", HFILL }},
    { &hf_rnsap_propagation_delay,
      { "propagation-delay", "rnsap.propagation_delay",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.PropagationDelay", HFILL }},
    { &hf_rnsap_deactivation_type,
      { "deactivation-type", "rnsap.deactivation_type",
        FT_UINT32, BASE_DEC, VALS(rnsap_Execution_Type_vals), 0,
        "rnsap.Execution_Type", HFILL }},
    { &hf_rnsap_synchronised,
      { "synchronised", "rnsap.synchronised",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.CFN", HFILL }},
    { &hf_rnsap_unsynchronised,
      { "unsynchronised", "rnsap.unsynchronised",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.NULL", HFILL }},
    { &hf_rnsap_gPSTOW,
      { "gPSTOW", "rnsap.gPSTOW",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.GPSTOW", HFILL }},
    { &hf_rnsap_gPS_Status_Health,
      { "gPS-Status-Health", "rnsap.gPS_Status_Health",
        FT_UINT32, BASE_DEC, VALS(rnsap_GPS_Status_Health_vals), 0,
        "rnsap.GPS_Status_Health", HFILL }},
    { &hf_rnsap_satellite_DGPSCorrections_Information,
      { "satellite-DGPSCorrections-Information", "rnsap.satellite_DGPSCorrections_Information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.T_satellite_DGPSCorrections_Information", HFILL }},
    { &hf_rnsap_satellite_DGPSCorrections_Information_item,
      { "Item", "rnsap.satellite_DGPSCorrections_Information_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.T_satellite_DGPSCorrections_Information_item", HFILL }},
    { &hf_rnsap_sAT_ID,
      { "sAT-ID", "rnsap.sAT_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.SAT_ID", HFILL }},
    { &hf_rnsap_iode_dgps,
      { "iode-dgps", "rnsap.iode_dgps",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_8", HFILL }},
    { &hf_rnsap_uDRE,
      { "uDRE", "rnsap.uDRE",
        FT_UINT32, BASE_DEC, VALS(rnsap_UDRE_vals), 0,
        "rnsap.UDRE", HFILL }},
    { &hf_rnsap_pRC,
      { "pRC", "rnsap.pRC",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.PRC", HFILL }},
    { &hf_rnsap_range_Correction_Rate,
      { "range-Correction-Rate", "rnsap.range_Correction_Rate",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.Range_Correction_Rate", HFILL }},
    { &hf_rnsap_pRCDeviation,
      { "pRCDeviation", "rnsap.pRCDeviation",
        FT_UINT32, BASE_DEC, VALS(rnsap_PRCDeviation_vals), 0,
        "rnsap.PRCDeviation", HFILL }},
    { &hf_rnsap_powerAdjustmentType,
      { "powerAdjustmentType", "rnsap.powerAdjustmentType",
        FT_UINT32, BASE_DEC, VALS(rnsap_PowerAdjustmentType_vals), 0,
        "rnsap.PowerAdjustmentType", HFILL }},
    { &hf_rnsap_dLReferencePower,
      { "dLReferencePower", "rnsap.dLReferencePower",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.DL_Power", HFILL }},
    { &hf_rnsap_dLReferencePowerList,
      { "dLReferencePowerList", "rnsap.dLReferencePowerList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DL_ReferencePowerInformationList", HFILL }},
    { &hf_rnsap_maxAdjustmentStep,
      { "maxAdjustmentStep", "rnsap.maxAdjustmentStep",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.MaxAdjustmentStep", HFILL }},
    { &hf_rnsap_adjustmentPeriod,
      { "adjustmentPeriod", "rnsap.adjustmentPeriod",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.AdjustmentPeriod", HFILL }},
    { &hf_rnsap_adjustmentRatio,
      { "adjustmentRatio", "rnsap.adjustmentRatio",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.ScaledAdjustmentRatio", HFILL }},
    { &hf_rnsap_DL_ReferencePowerInformationList_item,
      { "Item", "rnsap.DL_ReferencePowerInformationList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_ReferencePowerInformationItem", HFILL }},
    { &hf_rnsap_common_DL_ReferencePowerInformation,
      { "common-DL-ReferencePowerInformation", "rnsap.common_DL_ReferencePowerInformation",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.DL_Power", HFILL }},
    { &hf_rnsap_individual_DL_ReferencePowerInformation,
      { "individual-DL-ReferencePowerInformation", "rnsap.individual_DL_ReferencePowerInformation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DL_ReferencePowerInformationList", HFILL }},
    { &hf_rnsap_DL_Timeslot_Information_item,
      { "Item", "rnsap.DL_Timeslot_Information_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_Timeslot_InformationItem", HFILL }},
    { &hf_rnsap_DL_TimeslotLCR_Information_item,
      { "Item", "rnsap.DL_TimeslotLCR_Information_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_TimeslotLCR_InformationItem", HFILL }},
    { &hf_rnsap_DL_TimeSlot_ISCP_Info_item,
      { "Item", "rnsap.DL_TimeSlot_ISCP_Info_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_TimeSlot_ISCP_InfoItem", HFILL }},
    { &hf_rnsap_dL_TimeslotISCP,
      { "dL-TimeslotISCP", "rnsap.dL_TimeslotISCP",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DL_TimeslotISCP", HFILL }},
    { &hf_rnsap_DL_TimeSlot_ISCP_LCR_Information_item,
      { "Item", "rnsap.DL_TimeSlot_ISCP_LCR_Information_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DL_TimeSlot_ISCP_LCR_InfoItem", HFILL }},
    { &hf_rnsap_DSCH_FlowControlInformation_item,
      { "Item", "rnsap.DSCH_FlowControlInformation_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DSCH_FlowControlItem", HFILL }},
    { &hf_rnsap_dSCH_SchedulingPriority,
      { "dSCH-SchedulingPriority", "rnsap.dSCH_SchedulingPriority",
        FT_UINT32, BASE_DEC, VALS(rnsap_SchedulingPriorityIndicator_vals), 0,
        "rnsap.SchedulingPriorityIndicator", HFILL }},
    { &hf_rnsap_mAC_c_sh_SDU_Lengths,
      { "mAC-c-sh-SDU-Lengths", "rnsap.mAC_c_sh_SDU_Lengths",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.MAC_c_sh_SDU_LengthList", HFILL }},
    { &hf_rnsap_DSCH_TDD_Information_item,
      { "Item", "rnsap.DSCH_TDD_Information_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DSCH_TDD_InformationItem", HFILL }},
    { &hf_rnsap_eAGCH_ERGCH_EHICH_FDD_ScramblingCode,
      { "eAGCH-ERGCH-EHICH-FDD-ScramblingCode", "rnsap.eAGCH_ERGCH_EHICH_FDD_ScramblingCode",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DL_ScramblingCode", HFILL }},
    { &hf_rnsap_eAGCH_ChannelisationCode,
      { "eAGCH-ChannelisationCode", "rnsap.eAGCH_ChannelisationCode",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.FDD_DL_ChannelisationCodeNumber", HFILL }},
    { &hf_rnsap_primary_e_RNTI,
      { "primary-e-RNTI", "rnsap.primary_e_RNTI",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.E_RNTI", HFILL }},
    { &hf_rnsap_secondary_e_RNTI,
      { "secondary-e-RNTI", "rnsap.secondary_e_RNTI",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.E_RNTI", HFILL }},
    { &hf_rnsap_eRGCH_EHICH_ChannelisationCode,
      { "eRGCH-EHICH-ChannelisationCode", "rnsap.eRGCH_EHICH_ChannelisationCode",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.FDD_DL_ChannelisationCodeNumber", HFILL }},
    { &hf_rnsap_eRGCH_SignatureSequence,
      { "eRGCH-SignatureSequence", "rnsap.eRGCH_SignatureSequence",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.ERGCH_SignatureSequence", HFILL }},
    { &hf_rnsap_eHICH_SignatureSequence,
      { "eHICH-SignatureSequence", "rnsap.eHICH_SignatureSequence",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.EHICH_SignatureSequence", HFILL }},
    { &hf_rnsap_serving_Grant_Value,
      { "serving-Grant-Value", "rnsap.serving_Grant_Value",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.E_Serving_Grant_Value", HFILL }},
    { &hf_rnsap_primary_Secondary_Grant_Selector,
      { "primary-Secondary-Grant-Selector", "rnsap.primary_Secondary_Grant_Selector",
        FT_UINT32, BASE_DEC, VALS(rnsap_E_Primary_Secondary_Grant_Selector_vals), 0,
        "rnsap.E_Primary_Secondary_Grant_Selector", HFILL }},
    { &hf_rnsap_e_RGCH_Release_Indicator,
      { "e-RGCH-Release-Indicator", "rnsap.e_RGCH_Release_Indicator",
        FT_UINT32, BASE_DEC, VALS(rnsap_E_RGCH_Release_Indicator_vals), 0,
        "rnsap.E_RGCH_Release_Indicator", HFILL }},
    { &hf_rnsap_eDCH_MACdFlows_Information,
      { "eDCH-MACdFlows-Information", "rnsap.eDCH_MACdFlows_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.EDCH_MACdFlows_Information", HFILL }},
    { &hf_rnsap_EDCH_FDD_InformationResponse_item,
      { "Item", "rnsap.EDCH_FDD_InformationResponse_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.EDCH_FDD_InformationResponseItem", HFILL }},
    { &hf_rnsap_eDCH_MACdFlow_Specific_Information,
      { "eDCH-MACdFlow-Specific-Information", "rnsap.eDCH_MACdFlow_Specific_Information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.EDCH_MACdFlow_Specific_InfoToModifyList", HFILL }},
    { &hf_rnsap_EDCH_FDD_Update_Information_item,
      { "Item", "rnsap.EDCH_FDD_Update_Information_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.EDCH_FDD_Update_InfoItem", HFILL }},
    { &hf_rnsap_eDCH_FDD_DL_ControlChannelInformation,
      { "eDCH-FDD-DL-ControlChannelInformation", "rnsap.eDCH_FDD_DL_ControlChannelInformation",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.EDCH_FDD_DL_ControlChannelInformation", HFILL }},
    { &hf_rnsap_e_DCH_Non_Scheduled_Transmission_Grant,
      { "e-DCH-Non-Scheduled-Transmission-Grant", "rnsap.e_DCH_Non_Scheduled_Transmission_Grant",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.E_DCH_Non_Scheduled_Transmission_Grant_Items", HFILL }},
    { &hf_rnsap_e_DCH_Scheduled_Transmission_Grant,
      { "e-DCH-Scheduled-Transmission-Grant", "rnsap.e_DCH_Scheduled_Transmission_Grant",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.NULL", HFILL }},
    { &hf_rnsap_E_DCH_LogicalChannelInformation_item,
      { "Item", "rnsap.E_DCH_LogicalChannelInformation_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.E_DCH_LogicalChannelInformationItem", HFILL }},
    { &hf_rnsap_logicalChannelId,
      { "logicalChannelId", "rnsap.logicalChannelId",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.LogicalChannelID", HFILL }},
    { &hf_rnsap_mACes_GuaranteedBitRate,
      { "mACes-GuaranteedBitRate", "rnsap.mACes_GuaranteedBitRate",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.MACes_Guaranteed_Bitrate", HFILL }},
    { &hf_rnsap_eDCH_DDI_Value,
      { "eDCH-DDI-Value", "rnsap.eDCH_DDI_Value",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.EDCH_DDI_Value", HFILL }},
    { &hf_rnsap_mACd_PDU_Size_List,
      { "mACd-PDU-Size-List", "rnsap.mACd_PDU_Size_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.E_DCH_MACdPDU_SizeList", HFILL }},
    { &hf_rnsap_E_DCH_MACdPDU_SizeList_item,
      { "Item", "rnsap.E_DCH_MACdPDU_SizeList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.E_DCH_MACdPDU_SizeListItem", HFILL }},
    { &hf_rnsap_mACdPDU_Size,
      { "mACdPDU-Size", "rnsap.mACdPDU_Size",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.MACdPDU_Size", HFILL }},
    { &hf_rnsap_E_DCH_LogicalChannelToModify_item,
      { "Item", "rnsap.E_DCH_LogicalChannelToModify_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.E_DCH_LogicalChannelToModifyItem", HFILL }},
    { &hf_rnsap_mACd_PDU_Size_List1,
      { "mACd-PDU-Size-List", "rnsap.mACd_PDU_Size_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.E_DCH_MACdPDU_SizeToModifyList", HFILL }},
    { &hf_rnsap_E_DCH_MACdPDU_SizeToModifyList_item,
      { "Item", "rnsap.E_DCH_MACdPDU_SizeToModifyList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.E_DCH_MACdPDU_SizeListItem", HFILL }},
    { &hf_rnsap_E_DCH_LogicalChannelToDelete_item,
      { "Item", "rnsap.E_DCH_LogicalChannelToDelete_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.E_DCH_LogicalChannelToDeleteItem", HFILL }},
    { &hf_rnsap_eDCH_MACdFlow_Specific_Information1,
      { "eDCH-MACdFlow-Specific-Information", "rnsap.eDCH_MACdFlow_Specific_Information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.EDCH_MACdFlow_Specific_InfoList", HFILL }},
    { &hf_rnsap_EDCH_MACdFlow_Specific_InfoList_item,
      { "Item", "rnsap.EDCH_MACdFlow_Specific_InfoList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.EDCH_MACdFlow_Specific_InfoItem", HFILL }},
    { &hf_rnsap_eDSCH_MACdFlow_ID,
      { "eDSCH-MACdFlow-ID", "rnsap.eDSCH_MACdFlow_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.EDCH_MACdFlow_ID", HFILL }},
    { &hf_rnsap_tnlQoS,
      { "tnlQoS", "rnsap.tnlQoS",
        FT_UINT32, BASE_DEC, VALS(rnsap_TnlQos_vals), 0,
        "rnsap.TnlQos", HFILL }},
    { &hf_rnsap_maxNr_Retransmissions_EDCH,
      { "maxNr-Retransmissions-EDCH", "rnsap.maxNr_Retransmissions_EDCH",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.MaxNr_Retransmissions_EDCH", HFILL }},
    { &hf_rnsap_trafficClass,
      { "trafficClass", "rnsap.trafficClass",
        FT_UINT32, BASE_DEC, VALS(rnsap_TrafficClass_vals), 0,
        "rnsap.TrafficClass", HFILL }},
    { &hf_rnsap_eDCH_HARQ_PO_FDD,
      { "eDCH-HARQ-PO-FDD", "rnsap.eDCH_HARQ_PO_FDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.E_DCH_HARQ_PO_FDD", HFILL }},
    { &hf_rnsap_eDCH_MACdFlow_Multiplexing_List,
      { "eDCH-MACdFlow-Multiplexing-List", "rnsap.eDCH_MACdFlow_Multiplexing_List",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.E_DCH_MACdFlow_Multiplexing_List", HFILL }},
    { &hf_rnsap_eDCH_Grant_Type_Information,
      { "eDCH-Grant-Type-Information", "rnsap.eDCH_Grant_Type_Information",
        FT_UINT32, BASE_DEC, VALS(rnsap_E_DCH_Grant_Type_Information_vals), 0,
        "rnsap.E_DCH_Grant_Type_Information", HFILL }},
    { &hf_rnsap_bundlingModeIndicator,
      { "bundlingModeIndicator", "rnsap.bundlingModeIndicator",
        FT_UINT32, BASE_DEC, VALS(rnsap_BundlingModeIndicator_vals), 0,
        "rnsap.BundlingModeIndicator", HFILL }},
    { &hf_rnsap_eDCHLogicalChannelInformation,
      { "eDCHLogicalChannelInformation", "rnsap.eDCHLogicalChannelInformation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.E_DCH_LogicalChannelInformation", HFILL }},
    { &hf_rnsap_EDCH_MACdFlow_Specific_InfoToModifyList_item,
      { "Item", "rnsap.EDCH_MACdFlow_Specific_InfoToModifyList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.EDCH_MACdFlow_Specific_InfoToModifyItem", HFILL }},
    { &hf_rnsap_eDCH_LogicalChannelToAdd,
      { "eDCH-LogicalChannelToAdd", "rnsap.eDCH_LogicalChannelToAdd",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.E_DCH_LogicalChannelInformation", HFILL }},
    { &hf_rnsap_eDCH_LogicalChannelToModify,
      { "eDCH-LogicalChannelToModify", "rnsap.eDCH_LogicalChannelToModify",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.E_DCH_LogicalChannelToModify", HFILL }},
    { &hf_rnsap_eDCH_LogicalChannelToDelete,
      { "eDCH-LogicalChannelToDelete", "rnsap.eDCH_LogicalChannelToDelete",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.E_DCH_LogicalChannelToDelete", HFILL }},
    { &hf_rnsap_EDCH_MACdFlows_To_Delete_item,
      { "Item", "rnsap.EDCH_MACdFlows_To_Delete_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.EDCH_MACdFlows_To_Delete_Item", HFILL }},
    { &hf_rnsap_maxBits_MACe_PDU_non_scheduled,
      { "maxBits-MACe-PDU-non-scheduled", "rnsap.maxBits_MACe_PDU_non_scheduled",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Max_Bits_MACe_PDU_non_scheduled", HFILL }},
    { &hf_rnsap_hARQ_Process_Allocation_2ms,
      { "hARQ-Process-Allocation-2ms", "rnsap.hARQ_Process_Allocation_2ms",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.HARQ_Process_Allocation_2ms_EDCH", HFILL }},
    { &hf_rnsap_maxSet_E_DPDCHs,
      { "maxSet-E-DPDCHs", "rnsap.maxSet_E_DPDCHs",
        FT_UINT32, BASE_DEC, VALS(rnsap_Max_Set_E_DPDCHs_vals), 0,
        "rnsap.Max_Set_E_DPDCHs", HFILL }},
    { &hf_rnsap_e_TFCS_Information,
      { "e-TFCS-Information", "rnsap.e_TFCS_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.E_TFCS_Information", HFILL }},
    { &hf_rnsap_e_TTI,
      { "e-TTI", "rnsap.e_TTI",
        FT_UINT32, BASE_DEC, VALS(rnsap_E_TTI_vals), 0,
        "rnsap.E_TTI", HFILL }},
    { &hf_rnsap_e_DPCCH_PO,
      { "e-DPCCH-PO", "rnsap.e_DPCCH_PO",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.E_DPCCH_PO", HFILL }},
    { &hf_rnsap_e_DCH_Serving_RL_in_this_DRNS,
      { "e-DCH-Serving-RL-in-this-DRNS", "rnsap.e_DCH_Serving_RL_in_this_DRNS",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.EDCH_Serving_RL_in_this_DRNS", HFILL }},
    { &hf_rnsap_e_DCH_Serving_RL_not_in_this_DRNS,
      { "e-DCH-Serving-RL-not-in-this-DRNS", "rnsap.e_DCH_Serving_RL_not_in_this_DRNS",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.NULL", HFILL }},
    { &hf_rnsap_e_DCH_Serving_RL_Id,
      { "e-DCH-Serving-RL-Id", "rnsap.e_DCH_Serving_RL_Id",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RL_ID", HFILL }},
    { &hf_rnsap_e_DCH_TFCI_Table_Index,
      { "e-DCH-TFCI-Table-Index", "rnsap.e_DCH_TFCI_Table_Index",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.E_DCH_TFCI_Table_Index", HFILL }},
    { &hf_rnsap_e_DCH_Min_Set_E_TFCI,
      { "e-DCH-Min-Set-E-TFCI", "rnsap.e_DCH_Min_Set_E_TFCI",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.E_TFCI", HFILL }},
    { &hf_rnsap_reference_E_TFCI_Information,
      { "reference-E-TFCI-Information", "rnsap.reference_E_TFCI_Information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Reference_E_TFCI_Information", HFILL }},
    { &hf_rnsap_measurementTreshold,
      { "measurementTreshold", "rnsap.measurementTreshold",
        FT_UINT32, BASE_DEC, VALS(rnsap_MeasurementThreshold_vals), 0,
        "rnsap.MeasurementThreshold", HFILL }},
    { &hf_rnsap_measurementHysteresisTime,
      { "measurementHysteresisTime", "rnsap.measurementHysteresisTime",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.MeasurementHysteresisTime", HFILL }},
    { &hf_rnsap_measurementIncreaseDecreaseThreshold,
      { "measurementIncreaseDecreaseThreshold", "rnsap.measurementIncreaseDecreaseThreshold",
        FT_UINT32, BASE_DEC, VALS(rnsap_MeasurementIncreaseDecreaseThreshold_vals), 0,
        "rnsap.MeasurementIncreaseDecreaseThreshold", HFILL }},
    { &hf_rnsap_measurementChangeTime,
      { "measurementChangeTime", "rnsap.measurementChangeTime",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.MeasurementChangeTime", HFILL }},
    { &hf_rnsap_measurementThreshold1,
      { "measurementThreshold1", "rnsap.measurementThreshold1",
        FT_UINT32, BASE_DEC, VALS(rnsap_MeasurementThreshold_vals), 0,
        "rnsap.MeasurementThreshold", HFILL }},
    { &hf_rnsap_measurementThreshold2,
      { "measurementThreshold2", "rnsap.measurementThreshold2",
        FT_UINT32, BASE_DEC, VALS(rnsap_MeasurementThreshold_vals), 0,
        "rnsap.MeasurementThreshold", HFILL }},
    { &hf_rnsap_reportPeriodicity,
      { "reportPeriodicity", "rnsap.reportPeriodicity",
        FT_UINT32, BASE_DEC, VALS(rnsap_ReportPeriodicity_vals), 0,
        "rnsap.ReportPeriodicity", HFILL }},
    { &hf_rnsap_FACH_FlowControlInformation_item,
      { "Item", "rnsap.FACH_FlowControlInformation_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.FACH_FlowControlInformationItem", HFILL }},
    { &hf_rnsap_fACH_SchedulingPriority,
      { "fACH-SchedulingPriority", "rnsap.fACH_SchedulingPriority",
        FT_UINT32, BASE_DEC, VALS(rnsap_SchedulingPriorityIndicator_vals), 0,
        "rnsap.SchedulingPriorityIndicator", HFILL }},
    { &hf_rnsap_fACH_InitialWindowSize,
      { "fACH-InitialWindowSize", "rnsap.fACH_InitialWindowSize",
        FT_UINT32, BASE_DEC, VALS(rnsap_FACH_InitialWindowSize_vals), 0,
        "rnsap.FACH_InitialWindowSize", HFILL }},
    { &hf_rnsap_FACH_InformationList_item,
      { "Item", "rnsap.FACH_InformationList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.FACH_InformationItem", HFILL }},
    { &hf_rnsap_FACH_PCH_InformationList_item,
      { "Item", "rnsap.FACH_PCH_InformationList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.FACH_PCH_InformationItem", HFILL }},
    { &hf_rnsap_FDD_DCHs_to_Modify_item,
      { "Item", "rnsap.FDD_DCHs_to_Modify_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.FDD_DCHs_to_ModifyItem", HFILL }},
    { &hf_rnsap_dCH_SpecificInformationList2,
      { "dCH-SpecificInformationList", "rnsap.dCH_SpecificInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.FDD_DCHs_to_ModifySpecificInformationList", HFILL }},
    { &hf_rnsap_FDD_DCHs_to_ModifySpecificInformationList_item,
      { "Item", "rnsap.FDD_DCHs_to_ModifySpecificInformationList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.FDD_DCHs_to_ModifySpecificItem", HFILL }},
    { &hf_rnsap_ul_TransportformatSet,
      { "ul-TransportformatSet", "rnsap.ul_TransportformatSet",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.TransportFormatSet", HFILL }},
    { &hf_rnsap_dl_TransportformatSet,
      { "dl-TransportformatSet", "rnsap.dl_TransportformatSet",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.TransportFormatSet", HFILL }},
    { &hf_rnsap_not_Used_dRACControl,
      { "not-Used-dRACControl", "rnsap.not_Used_dRACControl",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.NULL", HFILL }},
    { &hf_rnsap_FDD_DL_CodeInformation_item,
      { "Item", "rnsap.FDD_DL_CodeInformation_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.FDD_DL_CodeInformationItem", HFILL }},
    { &hf_rnsap_dl_ScramblingCode,
      { "dl-ScramblingCode", "rnsap.dl_ScramblingCode",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DL_ScramblingCode", HFILL }},
    { &hf_rnsap_fDD_DL_ChannelisationCodeNumber,
      { "fDD-DL-ChannelisationCodeNumber", "rnsap.fDD_DL_ChannelisationCodeNumber",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.FDD_DL_ChannelisationCodeNumber", HFILL }},
    { &hf_rnsap_transmission_Gap_Pattern_Sequence_ScramblingCode_Information,
      { "transmission-Gap-Pattern-Sequence-ScramblingCode-Information", "rnsap.transmission_Gap_Pattern_Sequence_ScramblingCode_Information",
        FT_UINT32, BASE_DEC, VALS(rnsap_Transmission_Gap_Pattern_Sequence_ScramblingCode_Information_vals), 0,
        "rnsap.Transmission_Gap_Pattern_Sequence_ScramblingCode_Information", HFILL }},
    { &hf_rnsap_wT,
      { "wT", "rnsap.wT",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_1_4", HFILL }},
    { &hf_rnsap_GA_Cell_item,
      { "Item", "rnsap.GA_Cell_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.GA_Cell_item", HFILL }},
    { &hf_rnsap_cell_GAIgeographicalCoordinate,
      { "cell-GAIgeographicalCoordinate", "rnsap.cell_GAIgeographicalCoordinate",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.GeographicalCoordinate", HFILL }},
    { &hf_rnsap_pointWithUncertainty,
      { "pointWithUncertainty", "rnsap.pointWithUncertainty",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.GA_PointWithUnCertainty", HFILL }},
    { &hf_rnsap_pointWithUncertaintyEllipse,
      { "pointWithUncertaintyEllipse", "rnsap.pointWithUncertaintyEllipse",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.GA_PointWithUnCertaintyEllipse", HFILL }},
    { &hf_rnsap_pointWithAltitude,
      { "pointWithAltitude", "rnsap.pointWithAltitude",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.GA_PointWithAltitude", HFILL }},
    { &hf_rnsap_pointWithAltitudeAndUncertaintyEllipsoid,
      { "pointWithAltitudeAndUncertaintyEllipsoid", "rnsap.pointWithAltitudeAndUncertaintyEllipsoid",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.GA_PointWithAltitudeAndUncertaintyEllipsoid", HFILL }},
    { &hf_rnsap_ellipsoidArc,
      { "ellipsoidArc", "rnsap.ellipsoidArc",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.GA_EllipsoidArc", HFILL }},
    { &hf_rnsap_directionOfAltitude,
      { "directionOfAltitude", "rnsap.directionOfAltitude",
        FT_UINT32, BASE_DEC, VALS(rnsap_T_directionOfAltitude_vals), 0,
        "rnsap.T_directionOfAltitude", HFILL }},
    { &hf_rnsap_altitude,
      { "altitude", "rnsap.altitude",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_0_32767", HFILL }},
    { &hf_rnsap_geographicalCoordinates,
      { "geographicalCoordinates", "rnsap.geographicalCoordinates",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.GeographicalCoordinate", HFILL }},
    { &hf_rnsap_innerRadius,
      { "innerRadius", "rnsap.innerRadius",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_0_65535", HFILL }},
    { &hf_rnsap_uncertaintyRadius,
      { "uncertaintyRadius", "rnsap.uncertaintyRadius",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_0_127", HFILL }},
    { &hf_rnsap_offsetAngle,
      { "offsetAngle", "rnsap.offsetAngle",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_0_179", HFILL }},
    { &hf_rnsap_includedAngle,
      { "includedAngle", "rnsap.includedAngle",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_0_179", HFILL }},
    { &hf_rnsap_confidence,
      { "confidence", "rnsap.confidence",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_0_127", HFILL }},
    { &hf_rnsap_altitudeAndDirection,
      { "altitudeAndDirection", "rnsap.altitudeAndDirection",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.GA_AltitudeAndDirection", HFILL }},
    { &hf_rnsap_uncertaintyEllipse,
      { "uncertaintyEllipse", "rnsap.uncertaintyEllipse",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.GA_UncertaintyEllipse", HFILL }},
    { &hf_rnsap_uncertaintyAltitude,
      { "uncertaintyAltitude", "rnsap.uncertaintyAltitude",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_0_127", HFILL }},
    { &hf_rnsap_uncertaintySemi_major,
      { "uncertaintySemi-major", "rnsap.uncertaintySemi_major",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_0_127", HFILL }},
    { &hf_rnsap_uncertaintySemi_minor,
      { "uncertaintySemi-minor", "rnsap.uncertaintySemi_minor",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_0_127", HFILL }},
    { &hf_rnsap_orientationOfMajorAxis,
      { "orientationOfMajorAxis", "rnsap.orientationOfMajorAxis",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_0_179", HFILL }},
    { &hf_rnsap_uncertaintyCode,
      { "uncertaintyCode", "rnsap.uncertaintyCode",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_0_127", HFILL }},
    { &hf_rnsap_geographicalCoordinate,
      { "geographicalCoordinate", "rnsap.geographicalCoordinate",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.GeographicalCoordinate", HFILL }},
    { &hf_rnsap_latitudeSign,
      { "latitudeSign", "rnsap.latitudeSign",
        FT_UINT32, BASE_DEC, VALS(rnsap_T_latitudeSign_vals), 0,
        "rnsap.T_latitudeSign", HFILL }},
    { &hf_rnsap_latitude,
      { "latitude", "rnsap.latitude",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_0_8388607", HFILL }},
    { &hf_rnsap_longitude,
      { "longitude", "rnsap.longitude",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_M8388608_8388607", HFILL }},
    { &hf_rnsap_sI,
      { "sI", "rnsap.sI",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.GERAN_SystemInfo", HFILL }},
    { &hf_rnsap_pSI,
      { "pSI", "rnsap.pSI",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.GERAN_SystemInfo", HFILL }},
    { &hf_rnsap_GERAN_SystemInfo_item,
      { "Item", "rnsap.GERAN_SystemInfo_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.GERAN_SystemInfo_item", HFILL }},
    { &hf_rnsap_gERAN_SI_block,
      { "gERAN-SI-block", "rnsap.gERAN_SI_block",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.OCTET_STRING_SIZE_1_23", HFILL }},
    { &hf_rnsap_wna_alm,
      { "wna-alm", "rnsap.wna_alm",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_8", HFILL }},
    { &hf_rnsap_satellite_Almanac_Information,
      { "satellite-Almanac-Information", "rnsap.satellite_Almanac_Information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.T_satellite_Almanac_Information", HFILL }},
    { &hf_rnsap_satellite_Almanac_Information_item,
      { "Item", "rnsap.satellite_Almanac_Information_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.T_satellite_Almanac_Information_item", HFILL }},
    { &hf_rnsap_dATA_ID,
      { "dATA-ID", "rnsap.dATA_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DATA_ID", HFILL }},
    { &hf_rnsap_gps_e_alm,
      { "gps-e-alm", "rnsap.gps_e_alm",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_16", HFILL }},
    { &hf_rnsap_gps_toa_alm,
      { "gps-toa-alm", "rnsap.gps_toa_alm",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_8", HFILL }},
    { &hf_rnsap_gps_delta_I_alm,
      { "gps-delta-I-alm", "rnsap.gps_delta_I_alm",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_16", HFILL }},
    { &hf_rnsap_omegadot_alm,
      { "omegadot-alm", "rnsap.omegadot_alm",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_16", HFILL }},
    { &hf_rnsap_svhealth_alm,
      { "svhealth-alm", "rnsap.svhealth_alm",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_8", HFILL }},
    { &hf_rnsap_gps_a_sqrt_alm,
      { "gps-a-sqrt-alm", "rnsap.gps_a_sqrt_alm",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_24", HFILL }},
    { &hf_rnsap_omegazero_alm,
      { "omegazero-alm", "rnsap.omegazero_alm",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_24", HFILL }},
    { &hf_rnsap_m_zero_alm,
      { "m-zero-alm", "rnsap.m_zero_alm",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_24", HFILL }},
    { &hf_rnsap_gps_omega_alm,
      { "gps-omega-alm", "rnsap.gps_omega_alm",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_24", HFILL }},
    { &hf_rnsap_gps_af_zero_alm,
      { "gps-af-zero-alm", "rnsap.gps_af_zero_alm",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_11", HFILL }},
    { &hf_rnsap_gps_af_one_alm,
      { "gps-af-one-alm", "rnsap.gps_af_one_alm",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_11", HFILL }},
    { &hf_rnsap_sVGlobalHealth_alm,
      { "sVGlobalHealth-alm", "rnsap.sVGlobalHealth_alm",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_364", HFILL }},
    { &hf_rnsap_Satellite_Almanac_Information_ExtItem_item,
      { "Item", "rnsap.Satellite_Almanac_Information_ExtItem_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Satellite_Almanac_Information_ExtItem_item", HFILL }},
    { &hf_rnsap_GPSInformation_item,
      { "Item", "rnsap.GPSInformation_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.GPSInformation_item", HFILL }},
    { &hf_rnsap_gPSInformationItem,
      { "gPSInformationItem", "rnsap.gPSInformationItem",
        FT_UINT32, BASE_DEC, VALS(rnsap_T_gPSInformationItem_vals), 0,
        "rnsap.T_gPSInformationItem", HFILL }},
    { &hf_rnsap_alpha_zero_ionos,
      { "alpha-zero-ionos", "rnsap.alpha_zero_ionos",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_8", HFILL }},
    { &hf_rnsap_alpha_one_ionos,
      { "alpha-one-ionos", "rnsap.alpha_one_ionos",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_8", HFILL }},
    { &hf_rnsap_alpha_two_ionos,
      { "alpha-two-ionos", "rnsap.alpha_two_ionos",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_8", HFILL }},
    { &hf_rnsap_alpha_three_ionos,
      { "alpha-three-ionos", "rnsap.alpha_three_ionos",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_8", HFILL }},
    { &hf_rnsap_beta_zero_ionos,
      { "beta-zero-ionos", "rnsap.beta_zero_ionos",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_8", HFILL }},
    { &hf_rnsap_beta_one_ionos,
      { "beta-one-ionos", "rnsap.beta_one_ionos",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_8", HFILL }},
    { &hf_rnsap_beta_two_ionos,
      { "beta-two-ionos", "rnsap.beta_two_ionos",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_8", HFILL }},
    { &hf_rnsap_beta_three_ionos,
      { "beta-three-ionos", "rnsap.beta_three_ionos",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_8", HFILL }},
    { &hf_rnsap_GPS_NavigationModel_and_TimeRecovery_item,
      { "Item", "rnsap.GPS_NavigationModel_and_TimeRecovery_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.GPS_NavigationModel_and_TimeRecovery_item", HFILL }},
    { &hf_rnsap_tx_tow_nav,
      { "tx-tow-nav", "rnsap.tx_tow_nav",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_0_1048575", HFILL }},
    { &hf_rnsap_tlm_message_nav,
      { "tlm-message-nav", "rnsap.tlm_message_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_14", HFILL }},
    { &hf_rnsap_tlm_revd_c_nav,
      { "tlm-revd-c-nav", "rnsap.tlm_revd_c_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_2", HFILL }},
    { &hf_rnsap_ho_word_nav,
      { "ho-word-nav", "rnsap.ho_word_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_22", HFILL }},
    { &hf_rnsap_w_n_nav,
      { "w-n-nav", "rnsap.w_n_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_10", HFILL }},
    { &hf_rnsap_ca_or_p_on_l2_nav,
      { "ca-or-p-on-l2-nav", "rnsap.ca_or_p_on_l2_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_2", HFILL }},
    { &hf_rnsap_user_range_accuracy_index_nav,
      { "user-range-accuracy-index-nav", "rnsap.user_range_accuracy_index_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_4", HFILL }},
    { &hf_rnsap_sv_health_nav,
      { "sv-health-nav", "rnsap.sv_health_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_6", HFILL }},
    { &hf_rnsap_iodc_nav,
      { "iodc-nav", "rnsap.iodc_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_10", HFILL }},
    { &hf_rnsap_l2_p_dataflag_nav,
      { "l2-p-dataflag-nav", "rnsap.l2_p_dataflag_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_1", HFILL }},
    { &hf_rnsap_sf1_reserved_nav,
      { "sf1-reserved-nav", "rnsap.sf1_reserved_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_87", HFILL }},
    { &hf_rnsap_t_gd_nav,
      { "t-gd-nav", "rnsap.t_gd_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_8", HFILL }},
    { &hf_rnsap_t_oc_nav,
      { "t-oc-nav", "rnsap.t_oc_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_16", HFILL }},
    { &hf_rnsap_a_f_2_nav,
      { "a-f-2-nav", "rnsap.a_f_2_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_8", HFILL }},
    { &hf_rnsap_a_f_1_nav,
      { "a-f-1-nav", "rnsap.a_f_1_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_16", HFILL }},
    { &hf_rnsap_a_f_zero_nav,
      { "a-f-zero-nav", "rnsap.a_f_zero_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_22", HFILL }},
    { &hf_rnsap_c_rs_nav,
      { "c-rs-nav", "rnsap.c_rs_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_16", HFILL }},
    { &hf_rnsap_delta_n_nav,
      { "delta-n-nav", "rnsap.delta_n_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_16", HFILL }},
    { &hf_rnsap_m_zero_nav,
      { "m-zero-nav", "rnsap.m_zero_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_32", HFILL }},
    { &hf_rnsap_c_uc_nav,
      { "c-uc-nav", "rnsap.c_uc_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_16", HFILL }},
    { &hf_rnsap_gps_e_nav,
      { "gps-e-nav", "rnsap.gps_e_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_32", HFILL }},
    { &hf_rnsap_c_us_nav,
      { "c-us-nav", "rnsap.c_us_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_16", HFILL }},
    { &hf_rnsap_a_sqrt_nav,
      { "a-sqrt-nav", "rnsap.a_sqrt_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_32", HFILL }},
    { &hf_rnsap_t_oe_nav,
      { "t-oe-nav", "rnsap.t_oe_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_16", HFILL }},
    { &hf_rnsap_fit_interval_flag_nav,
      { "fit-interval-flag-nav", "rnsap.fit_interval_flag_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_1", HFILL }},
    { &hf_rnsap_aodo_nav,
      { "aodo-nav", "rnsap.aodo_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_5", HFILL }},
    { &hf_rnsap_c_ic_nav,
      { "c-ic-nav", "rnsap.c_ic_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_16", HFILL }},
    { &hf_rnsap_omega_zero_nav,
      { "omega-zero-nav", "rnsap.omega_zero_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_32", HFILL }},
    { &hf_rnsap_c_is_nav,
      { "c-is-nav", "rnsap.c_is_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_16", HFILL }},
    { &hf_rnsap_i_zero_nav,
      { "i-zero-nav", "rnsap.i_zero_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_32", HFILL }},
    { &hf_rnsap_c_rc_nav,
      { "c-rc-nav", "rnsap.c_rc_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_16", HFILL }},
    { &hf_rnsap_gps_omega_nav,
      { "gps-omega-nav", "rnsap.gps_omega_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_32", HFILL }},
    { &hf_rnsap_omegadot_nav,
      { "omegadot-nav", "rnsap.omegadot_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_24", HFILL }},
    { &hf_rnsap_idot_nav,
      { "idot-nav", "rnsap.idot_nav",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_14", HFILL }},
    { &hf_rnsap_spare_zero_fill,
      { "spare-zero-fill", "rnsap.spare_zero_fill",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_20", HFILL }},
    { &hf_rnsap_badSatellites,
      { "badSatellites", "rnsap.badSatellites",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.BadSatellites", HFILL }},
    { &hf_rnsap_noBadSatellite,
      { "noBadSatellite", "rnsap.noBadSatellite",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.NULL", HFILL }},
    { &hf_rnsap_a_one_utc,
      { "a-one-utc", "rnsap.a_one_utc",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_24", HFILL }},
    { &hf_rnsap_a_zero_utc,
      { "a-zero-utc", "rnsap.a_zero_utc",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_32", HFILL }},
    { &hf_rnsap_t_ot_utc,
      { "t-ot-utc", "rnsap.t_ot_utc",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_8", HFILL }},
    { &hf_rnsap_delta_t_ls_utc,
      { "delta-t-ls-utc", "rnsap.delta_t_ls_utc",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_8", HFILL }},
    { &hf_rnsap_w_n_t_utc,
      { "w-n-t-utc", "rnsap.w_n_t_utc",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_8", HFILL }},
    { &hf_rnsap_w_n_lsf_utc,
      { "w-n-lsf-utc", "rnsap.w_n_lsf_utc",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_8", HFILL }},
    { &hf_rnsap_dn_utc,
      { "dn-utc", "rnsap.dn_utc",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_8", HFILL }},
    { &hf_rnsap_delta_t_lsf_utc,
      { "delta-t-lsf-utc", "rnsap.delta_t_lsf_utc",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_8", HFILL }},
    { &hf_rnsap_guaranteed_UL_Rate,
      { "guaranteed-UL-Rate", "rnsap.guaranteed_UL_Rate",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Guaranteed_Rate", HFILL }},
    { &hf_rnsap_guaranteed_DL_Rate,
      { "guaranteed-DL-Rate", "rnsap.guaranteed_DL_Rate",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Guaranteed_Rate", HFILL }},
    { &hf_rnsap_implicit,
      { "implicit", "rnsap.implicit",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.HARQ_MemoryPartitioning_Implicit", HFILL }},
    { &hf_rnsap_explicit,
      { "explicit", "rnsap.explicit",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.HARQ_MemoryPartitioning_Explicit", HFILL }},
    { &hf_rnsap_number_of_Processes,
      { "number-of-Processes", "rnsap.number_of_Processes",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_1_8_", HFILL }},
    { &hf_rnsap_hARQ_MemoryPartitioningList,
      { "hARQ-MemoryPartitioningList", "rnsap.hARQ_MemoryPartitioningList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.HARQ_MemoryPartitioningList", HFILL }},
    { &hf_rnsap_HARQ_MemoryPartitioningList_item,
      { "Item", "rnsap.HARQ_MemoryPartitioningList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.HARQ_MemoryPartitioningItem", HFILL }},
    { &hf_rnsap_process_Memory_Size,
      { "process-Memory-Size", "rnsap.process_Memory_Size",
        FT_UINT32, BASE_DEC, VALS(rnsap_T_process_Memory_Size_vals), 0,
        "rnsap.T_process_Memory_Size", HFILL }},
    { &hf_rnsap_hSDSCH_MACdFlows_Information,
      { "hSDSCH-MACdFlows-Information", "rnsap.hSDSCH_MACdFlows_Information",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.HSDSCH_MACdFlows_Information", HFILL }},
    { &hf_rnsap_uE_Capabilities_Info,
      { "uE-Capabilities-Info", "rnsap.uE_Capabilities_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UE_Capabilities_Info", HFILL }},
    { &hf_rnsap_mAChs_Reordering_Buffer_Size_for_RLC_UM,
      { "mAChs-Reordering-Buffer-Size-for-RLC-UM", "rnsap.mAChs_Reordering_Buffer_Size_for_RLC_UM",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.MAChsReorderingBufferSize_for_RLC_UM", HFILL }},
    { &hf_rnsap_cqiFeedback_CycleK,
      { "cqiFeedback-CycleK", "rnsap.cqiFeedback_CycleK",
        FT_UINT32, BASE_DEC, VALS(rnsap_CQI_Feedback_Cycle_vals), 0,
        "rnsap.CQI_Feedback_Cycle", HFILL }},
    { &hf_rnsap_cqiRepetitionFactor,
      { "cqiRepetitionFactor", "rnsap.cqiRepetitionFactor",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.CQI_RepetitionFactor", HFILL }},
    { &hf_rnsap_ackNackRepetitionFactor,
      { "ackNackRepetitionFactor", "rnsap.ackNackRepetitionFactor",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.AckNack_RepetitionFactor", HFILL }},
    { &hf_rnsap_cqiPowerOffset,
      { "cqiPowerOffset", "rnsap.cqiPowerOffset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.CQI_Power_Offset", HFILL }},
    { &hf_rnsap_ackPowerOffset,
      { "ackPowerOffset", "rnsap.ackPowerOffset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Ack_Power_Offset", HFILL }},
    { &hf_rnsap_nackPowerOffset,
      { "nackPowerOffset", "rnsap.nackPowerOffset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Nack_Power_Offset", HFILL }},
    { &hf_rnsap_hsscch_PowerOffset,
      { "hsscch-PowerOffset", "rnsap.hsscch_PowerOffset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.HSSCCH_PowerOffset", HFILL }},
    { &hf_rnsap_hSDSCH_MACdFlow_Specific_InfoList_Response,
      { "hSDSCH-MACdFlow-Specific-InfoList-Response", "rnsap.hSDSCH_MACdFlow_Specific_InfoList_Response",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.HSDSCH_MACdFlow_Specific_InfoList_Response", HFILL }},
    { &hf_rnsap_hSSCCH_Specific_InfoList_Response,
      { "hSSCCH-Specific-InfoList-Response", "rnsap.hSSCCH_Specific_InfoList_Response",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.HSSCCH_FDD_Specific_InfoList_Response", HFILL }},
    { &hf_rnsap_hSPDSCH_and_HSSCCH_ScramblingCode,
      { "hSPDSCH-and-HSSCCH-ScramblingCode", "rnsap.hSPDSCH_and_HSSCCH_ScramblingCode",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DL_ScramblingCode", HFILL }},
    { &hf_rnsap_measurement_Power_Offset,
      { "measurement-Power-Offset", "rnsap.measurement_Power_Offset",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.Measurement_Power_Offset", HFILL }},
    { &hf_rnsap_hARQ_MemoryPartitioning,
      { "hARQ-MemoryPartitioning", "rnsap.hARQ_MemoryPartitioning",
        FT_UINT32, BASE_DEC, VALS(rnsap_HARQ_MemoryPartitioning_vals), 0,
        "rnsap.HARQ_MemoryPartitioning", HFILL }},
    { &hf_rnsap_hSDSCH_MACdFlow_Specific_InfoList_to_Modify,
      { "hSDSCH-MACdFlow-Specific-InfoList-to-Modify", "rnsap.hSDSCH_MACdFlow_Specific_InfoList_to_Modify",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.HSDSCH_MACdFlow_Specific_InfoList_to_Modify", HFILL }},
    { &hf_rnsap_priorityQueue_Info_to_Modify,
      { "priorityQueue-Info-to-Modify", "rnsap.priorityQueue_Info_to_Modify",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.PriorityQueue_InfoList_to_Modify", HFILL }},
    { &hf_rnsap_hSSCCH_CodeChangeGrant,
      { "hSSCCH-CodeChangeGrant", "rnsap.hSSCCH_CodeChangeGrant",
        FT_UINT32, BASE_DEC, VALS(rnsap_HSSCCH_Code_Change_Grant_vals), 0,
        "rnsap.HSSCCH_Code_Change_Grant", HFILL }},
    { &hf_rnsap_tDDAckNackPowerOffset,
      { "tDDAckNackPowerOffset", "rnsap.tDDAckNackPowerOffset",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.TDD_AckNack_Power_Offset", HFILL }},
    { &hf_rnsap_priorityQueueInfotoModifyUnsynchronised,
      { "priorityQueueInfotoModifyUnsynchronised", "rnsap.priorityQueueInfotoModifyUnsynchronised",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.PriorityQueue_InfoList_to_Modify_Unsynchronised", HFILL }},
    { &hf_rnsap_HSDSCH_MACdFlow_Specific_InfoList_item,
      { "Item", "rnsap.HSDSCH_MACdFlow_Specific_InfoList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.HSDSCH_MACdFlow_Specific_InfoItem", HFILL }},
    { &hf_rnsap_HSDSCH_MACdFlow_Specific_InfoList_Response_item,
      { "Item", "rnsap.HSDSCH_MACdFlow_Specific_InfoList_Response_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.HSDSCH_MACdFlow_Specific_InfoItem_Response", HFILL }},
    { &hf_rnsap_hSDSCH_Initial_Capacity_Allocation,
      { "hSDSCH-Initial-Capacity-Allocation", "rnsap.hSDSCH_Initial_Capacity_Allocation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.HSDSCH_Initial_Capacity_Allocation", HFILL }},
    { &hf_rnsap_HSDSCH_MACdFlow_Specific_InfoList_to_Modify_item,
      { "Item", "rnsap.HSDSCH_MACdFlow_Specific_InfoList_to_Modify_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.HSDSCH_MACdFlow_Specific_InfoItem_to_Modify", HFILL }},
    { &hf_rnsap_hSDSCH_MACdFlow_Specific_Info,
      { "hSDSCH-MACdFlow-Specific-Info", "rnsap.hSDSCH_MACdFlow_Specific_Info",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.HSDSCH_MACdFlow_Specific_InfoList", HFILL }},
    { &hf_rnsap_priorityQueue_Info,
      { "priorityQueue-Info", "rnsap.priorityQueue_Info",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.PriorityQueue_InfoList", HFILL }},
    { &hf_rnsap_HSDSCH_MACdFlows_to_Delete_item,
      { "Item", "rnsap.HSDSCH_MACdFlows_to_Delete_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.HSDSCH_MACdFlows_to_Delete_Item", HFILL }},
    { &hf_rnsap_hsDSCH_MACdFlow_ID,
      { "hsDSCH-MACdFlow-ID", "rnsap.hsDSCH_MACdFlow_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.HSDSCH_MACdFlow_ID", HFILL }},
    { &hf_rnsap_HSDSCH_Initial_Capacity_Allocation_item,
      { "Item", "rnsap.HSDSCH_Initial_Capacity_Allocation_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.HSDSCH_Initial_Capacity_AllocationItem", HFILL }},
    { &hf_rnsap_maximum_MACdPDU_Size,
      { "maximum-MACdPDU-Size", "rnsap.maximum_MACdPDU_Size",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.MACdPDU_Size", HFILL }},
    { &hf_rnsap_hSDSCH_InitialWindowSize,
      { "hSDSCH-InitialWindowSize", "rnsap.hSDSCH_InitialWindowSize",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.HSDSCH_InitialWindowSize", HFILL }},
    { &hf_rnsap_tDD_AckNack_Power_Offset,
      { "tDD-AckNack-Power-Offset", "rnsap.tDD_AckNack_Power_Offset",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.TDD_AckNack_Power_Offset", HFILL }},
    { &hf_rnsap_hSSCCH_TDD_Specific_InfoList_Response,
      { "hSSCCH-TDD-Specific-InfoList-Response", "rnsap.hSSCCH_TDD_Specific_InfoList_Response",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.HSSCCH_TDD_Specific_InfoList_Response", HFILL }},
    { &hf_rnsap_hSSCCH_TDD_Specific_InfoList_Response_LCR,
      { "hSSCCH-TDD-Specific-InfoList-Response-LCR", "rnsap.hSSCCH_TDD_Specific_InfoList_Response_LCR",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.HSSCCH_TDD_Specific_InfoList_Response_LCR", HFILL }},
    { &hf_rnsap_hSPDSCH_TDD_Specific_InfoList_Response,
      { "hSPDSCH-TDD-Specific-InfoList-Response", "rnsap.hSPDSCH_TDD_Specific_InfoList_Response",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.HSPDSCH_TDD_Specific_InfoList_Response", HFILL }},
    { &hf_rnsap_hSPDSCH_TDD_Specific_InfoList_Response_LCR,
      { "hSPDSCH-TDD-Specific-InfoList-Response-LCR", "rnsap.hSPDSCH_TDD_Specific_InfoList_Response_LCR",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.HSPDSCH_TDD_Specific_InfoList_Response_LCR", HFILL }},
    { &hf_rnsap_HSPDSCH_TDD_Specific_InfoList_Response_item,
      { "Item", "rnsap.HSPDSCH_TDD_Specific_InfoList_Response_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.HSPDSCH_TDD_Specific_InfoItem_Response", HFILL }},
    { &hf_rnsap_HSPDSCH_TDD_Specific_InfoList_Response_LCR_item,
      { "Item", "rnsap.HSPDSCH_TDD_Specific_InfoList_Response_LCR_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.HSPDSCH_TDD_Specific_InfoItem_Response_LCR", HFILL }},
    { &hf_rnsap_HSSCCH_FDD_Specific_InfoList_Response_item,
      { "Item", "rnsap.HSSCCH_FDD_Specific_InfoList_Response_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.HSSCCH_FDD_Specific_InfoItem_Response", HFILL }},
    { &hf_rnsap_code_Number,
      { "code-Number", "rnsap.code_Number",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_0_127", HFILL }},
    { &hf_rnsap_HSSCCH_TDD_Specific_InfoList_Response_item,
      { "Item", "rnsap.HSSCCH_TDD_Specific_InfoList_Response_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.HSSCCH_TDD_Specific_InfoItem_Response", HFILL }},
    { &hf_rnsap_hSSICH_Info,
      { "hSSICH-Info", "rnsap.hSSICH_Info",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.HSSICH_Info", HFILL }},
    { &hf_rnsap_HSSCCH_TDD_Specific_InfoList_Response_LCR_item,
      { "Item", "rnsap.HSSCCH_TDD_Specific_InfoList_Response_LCR_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.HSSCCH_TDD_Specific_InfoItem_Response_LCR", HFILL }},
    { &hf_rnsap_first_TDD_ChannelisationCode,
      { "first-TDD-ChannelisationCode", "rnsap.first_TDD_ChannelisationCode",
        FT_UINT32, BASE_DEC, VALS(rnsap_TDD_ChannelisationCode_vals), 0,
        "rnsap.TDD_ChannelisationCode", HFILL }},
    { &hf_rnsap_second_TDD_ChannelisationCode,
      { "second-TDD-ChannelisationCode", "rnsap.second_TDD_ChannelisationCode",
        FT_UINT32, BASE_DEC, VALS(rnsap_TDD_ChannelisationCode_vals), 0,
        "rnsap.TDD_ChannelisationCode", HFILL }},
    { &hf_rnsap_hSSICH_InfoLCR,
      { "hSSICH-InfoLCR", "rnsap.hSSICH_InfoLCR",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.HSSICH_InfoLCR", HFILL }},
    { &hf_rnsap_hsSICH_ID,
      { "hsSICH-ID", "rnsap.hsSICH_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.HS_SICH_ID", HFILL }},
    { &hf_rnsap_failed_HS_SICH,
      { "failed-HS-SICH", "rnsap.failed_HS_SICH",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.HS_SICH_failed", HFILL }},
    { &hf_rnsap_missed_HS_SICH,
      { "missed-HS-SICH", "rnsap.missed_HS_SICH",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.HS_SICH_missed", HFILL }},
    { &hf_rnsap_total_HS_SICH,
      { "total-HS-SICH", "rnsap.total_HS_SICH",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.HS_SICH_total", HFILL }},
    { &hf_rnsap_hsSCCHCodeChangeIndicator,
      { "hsSCCHCodeChangeIndicator", "rnsap.hsSCCHCodeChangeIndicator",
        FT_UINT32, BASE_DEC, VALS(rnsap_HSSCCH_CodeChangeIndicator_vals), 0,
        "rnsap.HSSCCH_CodeChangeIndicator", HFILL }},
    { &hf_rnsap_onDemand,
      { "onDemand", "rnsap.onDemand",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.NULL", HFILL }},
    { &hf_rnsap_periodic,
      { "periodic", "rnsap.periodic",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.PeriodicInformation", HFILL }},
    { &hf_rnsap_onModification,
      { "onModification", "rnsap.onModification",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.OnModificationInformation", HFILL }},
    { &hf_rnsap_min,
      { "min", "rnsap.min",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_1_60_", HFILL }},
    { &hf_rnsap_hour,
      { "hour", "rnsap.hour",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_1_24_", HFILL }},
    { &hf_rnsap_dGPSThreshold,
      { "dGPSThreshold", "rnsap.dGPSThreshold",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DGPSThreshold", HFILL }},
    { &hf_rnsap_informationTypeItem,
      { "informationTypeItem", "rnsap.informationTypeItem",
        FT_UINT32, BASE_DEC, VALS(rnsap_T_informationTypeItem_vals), 0,
        "rnsap.T_informationTypeItem", HFILL }},
    { &hf_rnsap_gPSInformation,
      { "gPSInformation", "rnsap.gPSInformation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.GPSInformation", HFILL }},
    { &hf_rnsap_iPDL_FDD_Parameters,
      { "iPDL-FDD-Parameters", "rnsap.iPDL_FDD_Parameters",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.IPDL_FDD_Parameters", HFILL }},
    { &hf_rnsap_iPDL_TDD_Parameters,
      { "iPDL-TDD-Parameters", "rnsap.iPDL_TDD_Parameters",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.IPDL_TDD_Parameters", HFILL }},
    { &hf_rnsap_extension_IPDLParameters,
      { "extension-IPDLParameters", "rnsap.extension_IPDLParameters",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Extension_IPDLParameters", HFILL }},
    { &hf_rnsap_iPSpacingFDD,
      { "iPSpacingFDD", "rnsap.iPSpacingFDD",
        FT_UINT32, BASE_DEC, VALS(rnsap_IPSpacingFDD_vals), 0,
        "rnsap.IPSpacingFDD", HFILL }},
    { &hf_rnsap_iPLength,
      { "iPLength", "rnsap.iPLength",
        FT_UINT32, BASE_DEC, VALS(rnsap_IPLength_vals), 0,
        "rnsap.IPLength", HFILL }},
    { &hf_rnsap_iPOffset,
      { "iPOffset", "rnsap.iPOffset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.IPOffset", HFILL }},
    { &hf_rnsap_seed,
      { "seed", "rnsap.seed",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Seed", HFILL }},
    { &hf_rnsap_burstModeParameters,
      { "burstModeParameters", "rnsap.burstModeParameters",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.BurstModeParameters", HFILL }},
    { &hf_rnsap_iPSpacingTDD,
      { "iPSpacingTDD", "rnsap.iPSpacingTDD",
        FT_UINT32, BASE_DEC, VALS(rnsap_IPSpacingTDD_vals), 0,
        "rnsap.IPSpacingTDD", HFILL }},
    { &hf_rnsap_iPStart,
      { "iPStart", "rnsap.iPStart",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.IPStart", HFILL }},
    { &hf_rnsap_iPSlot,
      { "iPSlot", "rnsap.iPSlot",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.IPSlot", HFILL }},
    { &hf_rnsap_iP_P_CCPCH,
      { "iP-P-CCPCH", "rnsap.iP_P_CCPCH",
        FT_UINT32, BASE_DEC, VALS(rnsap_IP_P_CCPCH_vals), 0,
        "rnsap.IP_P_CCPCH", HFILL }},
    { &hf_rnsap_iPSub,
      { "iPSub", "rnsap.iPSub",
        FT_UINT32, BASE_DEC, VALS(rnsap_IPSub_vals), 0,
        "rnsap.IPSub", HFILL }},
    { &hf_rnsap_uplinkLoadValue,
      { "uplinkLoadValue", "rnsap.uplinkLoadValue",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_0_100", HFILL }},
    { &hf_rnsap_downlinkLoadValue,
      { "downlinkLoadValue", "rnsap.downlinkLoadValue",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_0_100", HFILL }},
    { &hf_rnsap_MAC_c_sh_SDU_LengthList_item,
      { "Item", "rnsap.MAC_c_sh_SDU_LengthList_item",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.MAC_c_sh_SDU_Length", HFILL }},
    { &hf_rnsap_MACdPDU_Size_IndexList_item,
      { "Item", "rnsap.MACdPDU_Size_IndexList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.MACdPDU_Size_IndexItem", HFILL }},
    { &hf_rnsap_sID,
      { "sID", "rnsap.sID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.SID", HFILL }},
    { &hf_rnsap_MACdPDU_Size_IndexList_to_Modify_item,
      { "Item", "rnsap.MACdPDU_Size_IndexList_to_Modify_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.MACdPDU_Size_IndexItem_to_Modify", HFILL }},
    { &hf_rnsap_accessPointName,
      { "accessPointName", "rnsap.accessPointName",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.AccessPointName", HFILL }},
    { &hf_rnsap_iPMulticastAddress,
      { "iPMulticastAddress", "rnsap.iPMulticastAddress",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.IPMulticastAddress", HFILL }},
    { &hf_rnsap_MBMS_Bearer_Service_List_item,
      { "Item", "rnsap.MBMS_Bearer_Service_List_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.TMGI", HFILL }},
    { &hf_rnsap_transmissionMode,
      { "transmissionMode", "rnsap.transmissionMode",
        FT_UINT32, BASE_DEC, VALS(rnsap_TransmissionMode_vals), 0,
        "rnsap.TransmissionMode", HFILL }},
    { &hf_rnsap_preferredFrequencyLayer,
      { "preferredFrequencyLayer", "rnsap.preferredFrequencyLayer",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UARFCN", HFILL }},
    { &hf_rnsap_tMGI,
      { "tMGI", "rnsap.tMGI",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.TMGI", HFILL }},
    { &hf_rnsap_pTM_Cell_List,
      { "pTM-Cell-List", "rnsap.pTM_Cell_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.PTMCellList", HFILL }},
    { &hf_rnsap_pTP_Cell_List,
      { "pTP-Cell-List", "rnsap.pTP_Cell_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.PTPCellList", HFILL }},
    { &hf_rnsap_not_Provided_Cell_List,
      { "not-Provided-Cell-List", "rnsap.not_Provided_Cell_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.NotProvidedCellList", HFILL }},
    { &hf_rnsap_affectedUEInformationForMBMS,
      { "affectedUEInformationForMBMS", "rnsap.affectedUEInformationForMBMS",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.AffectedUEInformationForMBMS", HFILL }},
    { &hf_rnsap_preferredFrequencyLayerInfo,
      { "preferredFrequencyLayerInfo", "rnsap.preferredFrequencyLayerInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.PreferredFrequencyLayerInfo", HFILL }},
    { &hf_rnsap_sir,
      { "sir", "rnsap.sir",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.SIR_Value_IncrDecrThres", HFILL }},
    { &hf_rnsap_sir_error,
      { "sir-error", "rnsap.sir_error",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.SIR_Error_Value_IncrDecrThres", HFILL }},
    { &hf_rnsap_transmitted_code_power,
      { "transmitted-code-power", "rnsap.transmitted_code_power",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Transmitted_Code_Power_Value_IncrDecrThres", HFILL }},
    { &hf_rnsap_rscp,
      { "rscp", "rnsap.rscp",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RSCP_Value_IncrDecrThres", HFILL }},
    { &hf_rnsap_round_trip_time,
      { "round-trip-time", "rnsap.round_trip_time",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Round_Trip_Time_IncrDecrThres", HFILL }},
    { &hf_rnsap_extension_MeasurementIncreaseDecreaseThreshold,
      { "extension-MeasurementIncreaseDecreaseThreshold", "rnsap.extension_MeasurementIncreaseDecreaseThreshold",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Extension_MeasurementIncreaseDecreaseThreshold", HFILL }},
    { &hf_rnsap_sir1,
      { "sir", "rnsap.sir",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.SIR_Value", HFILL }},
    { &hf_rnsap_sir_error1,
      { "sir-error", "rnsap.sir_error",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.SIR_Error_Value", HFILL }},
    { &hf_rnsap_transmitted_code_power1,
      { "transmitted-code-power", "rnsap.transmitted_code_power",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Transmitted_Code_Power_Value", HFILL }},
    { &hf_rnsap_rscp1,
      { "rscp", "rnsap.rscp",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RSCP_Value", HFILL }},
    { &hf_rnsap_rx_timing_deviation,
      { "rx-timing-deviation", "rnsap.rx_timing_deviation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Rx_Timing_Deviation_Value", HFILL }},
    { &hf_rnsap_round_trip_time1,
      { "round-trip-time", "rnsap.round_trip_time",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Round_Trip_Time_Value", HFILL }},
    { &hf_rnsap_extension_MeasurementThreshold,
      { "extension-MeasurementThreshold", "rnsap.extension_MeasurementThreshold",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Extension_MeasurementThreshold", HFILL }},
    { &hf_rnsap_type1,
      { "type1", "rnsap.type1",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.T_type1", HFILL }},
    { &hf_rnsap_midambleConfigurationBurstType1And3,
      { "midambleConfigurationBurstType1And3", "rnsap.midambleConfigurationBurstType1And3",
        FT_UINT32, BASE_DEC, VALS(rnsap_MidambleConfigurationBurstType1And3_vals), 0,
        "rnsap.MidambleConfigurationBurstType1And3", HFILL }},
    { &hf_rnsap_midambleAllocationMode,
      { "midambleAllocationMode", "rnsap.midambleAllocationMode",
        FT_UINT32, BASE_DEC, VALS(rnsap_T_midambleAllocationMode_vals), 0,
        "rnsap.T_midambleAllocationMode", HFILL }},
    { &hf_rnsap_defaultMidamble,
      { "defaultMidamble", "rnsap.defaultMidamble",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.NULL", HFILL }},
    { &hf_rnsap_commonMidamble,
      { "commonMidamble", "rnsap.commonMidamble",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.NULL", HFILL }},
    { &hf_rnsap_ueSpecificMidamble,
      { "ueSpecificMidamble", "rnsap.ueSpecificMidamble",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.MidambleShiftLong", HFILL }},
    { &hf_rnsap_type2,
      { "type2", "rnsap.type2",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.T_type2", HFILL }},
    { &hf_rnsap_midambleConfigurationBurstType2,
      { "midambleConfigurationBurstType2", "rnsap.midambleConfigurationBurstType2",
        FT_UINT32, BASE_DEC, VALS(rnsap_MidambleConfigurationBurstType2_vals), 0,
        "rnsap.MidambleConfigurationBurstType2", HFILL }},
    { &hf_rnsap_midambleAllocationMode1,
      { "midambleAllocationMode", "rnsap.midambleAllocationMode",
        FT_UINT32, BASE_DEC, VALS(rnsap_T_midambleAllocationMode1_vals), 0,
        "rnsap.T_midambleAllocationMode1", HFILL }},
    { &hf_rnsap_ueSpecificMidamble1,
      { "ueSpecificMidamble", "rnsap.ueSpecificMidamble",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.MidambleShiftShort", HFILL }},
    { &hf_rnsap_type3,
      { "type3", "rnsap.type3",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.T_type3", HFILL }},
    { &hf_rnsap_midambleAllocationMode2,
      { "midambleAllocationMode", "rnsap.midambleAllocationMode",
        FT_UINT32, BASE_DEC, VALS(rnsap_T_midambleAllocationMode2_vals), 0,
        "rnsap.T_midambleAllocationMode2", HFILL }},
    { &hf_rnsap_midambleAllocationMode3,
      { "midambleAllocationMode", "rnsap.midambleAllocationMode",
        FT_UINT32, BASE_DEC, VALS(rnsap_MidambleAllocationMode_vals), 0,
        "rnsap.MidambleAllocationMode", HFILL }},
    { &hf_rnsap_midambleShift,
      { "midambleShift", "rnsap.midambleShift",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.MidambleShiftLong", HFILL }},
    { &hf_rnsap_midambleConfigurationLCR,
      { "midambleConfigurationLCR", "rnsap.midambleConfigurationLCR",
        FT_UINT32, BASE_DEC, VALS(rnsap_MidambleConfigurationLCR_vals), 0,
        "rnsap.MidambleConfigurationLCR", HFILL }},
    { &hf_rnsap_addPriorityQueue,
      { "addPriorityQueue", "rnsap.addPriorityQueue",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.PriorityQueue_InfoItem_to_Add", HFILL }},
    { &hf_rnsap_modifyPriorityQueue,
      { "modifyPriorityQueue", "rnsap.modifyPriorityQueue",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.PriorityQueue_InfoItem_to_Modify", HFILL }},
    { &hf_rnsap_deletePriorityQueue,
      { "deletePriorityQueue", "rnsap.deletePriorityQueue",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.PriorityQueue_Id", HFILL }},
    { &hf_rnsap_gERAN_SI_Type,
      { "gERAN-SI-Type", "rnsap.gERAN_SI_Type",
        FT_UINT32, BASE_DEC, VALS(rnsap_GERAN_SI_Type_vals), 0,
        "rnsap.GERAN_SI_Type", HFILL }},
    { &hf_rnsap_Neighbouring_UMTS_CellInformation_item,
      { "Item", "rnsap.Neighbouring_UMTS_CellInformation_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Single_Container", HFILL }},
    { &hf_rnsap_rNC_ID,
      { "rNC-ID", "rnsap.rNC_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RNC_ID", HFILL }},
    { &hf_rnsap_cN_PS_DomainIdentifier,
      { "cN-PS-DomainIdentifier", "rnsap.cN_PS_DomainIdentifier",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.CN_PS_DomainIdentifier", HFILL }},
    { &hf_rnsap_cN_CS_DomainIdentifier,
      { "cN-CS-DomainIdentifier", "rnsap.cN_CS_DomainIdentifier",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.CN_CS_DomainIdentifier", HFILL }},
    { &hf_rnsap_neighbouring_FDD_CellInformation,
      { "neighbouring-FDD-CellInformation", "rnsap.neighbouring_FDD_CellInformation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Neighbouring_FDD_CellInformation", HFILL }},
    { &hf_rnsap_neighbouring_TDD_CellInformation,
      { "neighbouring-TDD-CellInformation", "rnsap.neighbouring_TDD_CellInformation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Neighbouring_TDD_CellInformation", HFILL }},
    { &hf_rnsap_Neighbouring_FDD_CellInformation_item,
      { "Item", "rnsap.Neighbouring_FDD_CellInformation_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Neighbouring_FDD_CellInformationItem", HFILL }},
    { &hf_rnsap_uARFCNforNu,
      { "uARFCNforNu", "rnsap.uARFCNforNu",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UARFCN", HFILL }},
    { &hf_rnsap_uARFCNforNd,
      { "uARFCNforNd", "rnsap.uARFCNforNd",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UARFCN", HFILL }},
    { &hf_rnsap_cellIndividualOffset,
      { "cellIndividualOffset", "rnsap.cellIndividualOffset",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.CellIndividualOffset", HFILL }},
    { &hf_rnsap_txDiversityIndicator,
      { "txDiversityIndicator", "rnsap.txDiversityIndicator",
        FT_UINT32, BASE_DEC, VALS(rnsap_TxDiversityIndicator_vals), 0,
        "rnsap.TxDiversityIndicator", HFILL }},
    { &hf_rnsap_sTTD_SupportIndicator,
      { "sTTD-SupportIndicator", "rnsap.sTTD_SupportIndicator",
        FT_UINT32, BASE_DEC, VALS(rnsap_STTD_SupportIndicator_vals), 0,
        "rnsap.STTD_SupportIndicator", HFILL }},
    { &hf_rnsap_closedLoopMode1_SupportIndicator,
      { "closedLoopMode1-SupportIndicator", "rnsap.closedLoopMode1_SupportIndicator",
        FT_UINT32, BASE_DEC, VALS(rnsap_ClosedLoopMode1_SupportIndicator_vals), 0,
        "rnsap.ClosedLoopMode1_SupportIndicator", HFILL }},
    { &hf_rnsap_not_used_closedLoopMode2_SupportIndicator,
      { "not-used-closedLoopMode2-SupportIndicator", "rnsap.not_used_closedLoopMode2_SupportIndicator",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.NULL", HFILL }},
    { &hf_rnsap_uARFCN,
      { "uARFCN", "rnsap.uARFCN",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UARFCN", HFILL }},
    { &hf_rnsap_Neighbouring_GSM_CellInformationIEs_item,
      { "Item", "rnsap.Neighbouring_GSM_CellInformationIEs_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Neighbouring_GSM_CellInformationItem", HFILL }},
    { &hf_rnsap_bSIC,
      { "bSIC", "rnsap.bSIC",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.BSIC", HFILL }},
    { &hf_rnsap_band_Indicator,
      { "band-Indicator", "rnsap.band_Indicator",
        FT_UINT32, BASE_DEC, VALS(rnsap_Band_Indicator_vals), 0,
        "rnsap.Band_Indicator", HFILL }},
    { &hf_rnsap_bCCH_ARFCN,
      { "bCCH-ARFCN", "rnsap.bCCH_ARFCN",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.BCCH_ARFCN", HFILL }},
    { &hf_rnsap_Neighbouring_TDD_CellInformation_item,
      { "Item", "rnsap.Neighbouring_TDD_CellInformation_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Neighbouring_TDD_CellInformationItem", HFILL }},
    { &hf_rnsap_dPCHConstantValue,
      { "dPCHConstantValue", "rnsap.dPCHConstantValue",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.DPCHConstantValue", HFILL }},
    { &hf_rnsap_Neighbouring_LCR_TDD_CellInformation_item,
      { "Item", "rnsap.Neighbouring_LCR_TDD_CellInformation_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Neighbouring_LCR_TDD_CellInformationItem", HFILL }},
    { &hf_rnsap_restrictionStateIndicator,
      { "restrictionStateIndicator", "rnsap.restrictionStateIndicator",
        FT_UINT32, BASE_DEC, VALS(rnsap_RestrictionStateIndicator_vals), 0,
        "rnsap.RestrictionStateIndicator", HFILL }},
    { &hf_rnsap_NotProvidedCellList_item,
      { "Item", "rnsap.NotProvidedCellList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.MBMSChannelTypeCellList", HFILL }},
    { &hf_rnsap_uplinkNRTLoadInformationValue,
      { "uplinkNRTLoadInformationValue", "rnsap.uplinkNRTLoadInformationValue",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_0_3", HFILL }},
    { &hf_rnsap_downlinkNRTLoadInformationValue,
      { "downlinkNRTLoadInformationValue", "rnsap.downlinkNRTLoadInformationValue",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_0_3", HFILL }},
    { &hf_rnsap_measurementThreshold,
      { "measurementThreshold", "rnsap.measurementThreshold",
        FT_UINT32, BASE_DEC, VALS(rnsap_MeasurementThreshold_vals), 0,
        "rnsap.MeasurementThreshold", HFILL }},
    { &hf_rnsap_informationThreshold,
      { "informationThreshold", "rnsap.informationThreshold",
        FT_UINT32, BASE_DEC, VALS(rnsap_InformationThreshold_vals), 0,
        "rnsap.InformationThreshold", HFILL }},
    { &hf_rnsap_PCH_InformationList_item,
      { "Item", "rnsap.PCH_InformationList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.PCH_InformationItem", HFILL }},
    { &hf_rnsap_informationReportPeriodicity,
      { "informationReportPeriodicity", "rnsap.informationReportPeriodicity",
        FT_UINT32, BASE_DEC, VALS(rnsap_InformationReportPeriodicity_vals), 0,
        "rnsap.InformationReportPeriodicity", HFILL }},
    { &hf_rnsap_imsi,
      { "imsi", "rnsap.imsi",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.IMSI", HFILL }},
    { &hf_rnsap_defaultPreferredFrequency,
      { "defaultPreferredFrequency", "rnsap.defaultPreferredFrequency",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UARFCN", HFILL }},
    { &hf_rnsap_additionalPreferredFrequency,
      { "additionalPreferredFrequency", "rnsap.additionalPreferredFrequency",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.AdditionalPreferredFrequency", HFILL }},
    { &hf_rnsap_PriorityQueue_InfoList_item,
      { "Item", "rnsap.PriorityQueue_InfoList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.PriorityQueue_InfoItem", HFILL }},
    { &hf_rnsap_priorityQueue_Id,
      { "priorityQueue-Id", "rnsap.priorityQueue_Id",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.PriorityQueue_Id", HFILL }},
    { &hf_rnsap_associatedHSDSCH_MACdFlow,
      { "associatedHSDSCH-MACdFlow", "rnsap.associatedHSDSCH_MACdFlow",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.HSDSCH_MACdFlow_ID", HFILL }},
    { &hf_rnsap_t1,
      { "t1", "rnsap.t1",
        FT_UINT32, BASE_DEC, VALS(rnsap_T1_vals), 0,
        "rnsap.T1", HFILL }},
    { &hf_rnsap_discardTimer,
      { "discardTimer", "rnsap.discardTimer",
        FT_UINT32, BASE_DEC, VALS(rnsap_DiscardTimer_vals), 0,
        "rnsap.DiscardTimer", HFILL }},
    { &hf_rnsap_mAC_hsWindowSize,
      { "mAC-hsWindowSize", "rnsap.mAC_hsWindowSize",
        FT_UINT32, BASE_DEC, VALS(rnsap_MAC_hsWindowSize_vals), 0,
        "rnsap.MAC_hsWindowSize", HFILL }},
    { &hf_rnsap_mAChsGuaranteedBitRate,
      { "mAChsGuaranteedBitRate", "rnsap.mAChsGuaranteedBitRate",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.MAChsGuaranteedBitRate", HFILL }},
    { &hf_rnsap_mACdPDU_Size_Index,
      { "mACdPDU-Size-Index", "rnsap.mACdPDU_Size_Index",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.MACdPDU_Size_IndexList", HFILL }},
    { &hf_rnsap_rLC_Mode,
      { "rLC-Mode", "rnsap.rLC_Mode",
        FT_UINT32, BASE_DEC, VALS(rnsap_RLC_Mode_vals), 0,
        "rnsap.RLC_Mode", HFILL }},
    { &hf_rnsap_PriorityQueue_InfoList_to_Modify_item,
      { "Item", "rnsap.PriorityQueue_InfoList_to_Modify_item",
        FT_UINT32, BASE_DEC, VALS(rnsap_ModifyPriorityQueue_vals), 0,
        "rnsap.ModifyPriorityQueue", HFILL }},
    { &hf_rnsap_mACdPDU_Size_Index_to_Modify,
      { "mACdPDU-Size-Index-to-Modify", "rnsap.mACdPDU_Size_Index_to_Modify",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.MACdPDU_Size_IndexList_to_Modify", HFILL }},
    { &hf_rnsap_PriorityQueue_InfoList_to_Modify_Unsynchronised_item,
      { "Item", "rnsap.PriorityQueue_InfoList_to_Modify_Unsynchronised_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.PriorityQueue_InfoItem_to_Modify_Unsynchronised", HFILL }},
    { &hf_rnsap_priorityQueueId,
      { "priorityQueueId", "rnsap.priorityQueueId",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.PriorityQueue_Id", HFILL }},
    { &hf_rnsap_mBMSChannelTypeInfo,
      { "mBMSChannelTypeInfo", "rnsap.mBMSChannelTypeInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.MBMSChannelTypeInfo", HFILL }},
    { &hf_rnsap_mBMSPreferredFreqLayerInfo,
      { "mBMSPreferredFreqLayerInfo", "rnsap.mBMSPreferredFreqLayerInfo",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.MBMSPreferredFreqLayerInfo", HFILL }},
    { &hf_rnsap_PTMCellList_item,
      { "Item", "rnsap.PTMCellList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.MBMSChannelTypeCellList", HFILL }},
    { &hf_rnsap_PTPCellList_item,
      { "Item", "rnsap.PTPCellList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.MBMSChannelTypeCellList", HFILL }},
    { &hf_rnsap_RB_Info_item,
      { "Item", "rnsap.RB_Info_item",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RB_Identity", HFILL }},
    { &hf_rnsap_Reference_E_TFCI_Information_item,
      { "Item", "rnsap.Reference_E_TFCI_Information_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Reference_E_TFCI_Information_Item", HFILL }},
    { &hf_rnsap_reference_E_TFCI,
      { "reference-E-TFCI", "rnsap.reference_E_TFCI",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.E_TFCI", HFILL }},
    { &hf_rnsap_reference_E_TFCI_PO,
      { "reference-E-TFCI-PO", "rnsap.reference_E_TFCI_PO",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Reference_E_TFCI_PO", HFILL }},
    { &hf_rnsap_periodic1,
      { "periodic", "rnsap.periodic",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Periodic", HFILL }},
    { &hf_rnsap_eventA,
      { "eventA", "rnsap.eventA",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.EventA", HFILL }},
    { &hf_rnsap_eventB,
      { "eventB", "rnsap.eventB",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.EventB", HFILL }},
    { &hf_rnsap_eventC,
      { "eventC", "rnsap.eventC",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.EventC", HFILL }},
    { &hf_rnsap_eventD,
      { "eventD", "rnsap.eventD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.EventD", HFILL }},
    { &hf_rnsap_eventE,
      { "eventE", "rnsap.eventE",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.EventE", HFILL }},
    { &hf_rnsap_eventF,
      { "eventF", "rnsap.eventF",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.EventF", HFILL }},
    { &hf_rnsap_extension_ReportCharacteristics,
      { "extension-ReportCharacteristics", "rnsap.extension_ReportCharacteristics",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Extension_ReportCharacteristics", HFILL }},
    { &hf_rnsap_ten_msec,
      { "ten-msec", "rnsap.ten_msec",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_1_6000_", HFILL }},
    { &hf_rnsap_gA_AccessPointPositionwithAltitude,
      { "gA-AccessPointPositionwithAltitude", "rnsap.gA_AccessPointPositionwithAltitude",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.GA_AccessPointPositionwithOptionalAltitude", HFILL }},
    { &hf_rnsap_iPDLParameters,
      { "iPDLParameters", "rnsap.iPDLParameters",
        FT_UINT32, BASE_DEC, VALS(rnsap_IPDLParameters_vals), 0,
        "rnsap.IPDLParameters", HFILL }},
    { &hf_rnsap_dGPSCorrections,
      { "dGPSCorrections", "rnsap.dGPSCorrections",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.DGPSCorrections", HFILL }},
    { &hf_rnsap_gPS_NavigationModel_and_TimeRecovery,
      { "gPS-NavigationModel-and-TimeRecovery", "rnsap.gPS_NavigationModel_and_TimeRecovery",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.GPS_NavigationModel_and_TimeRecovery", HFILL }},
    { &hf_rnsap_gPS_Ionospheric_Model,
      { "gPS-Ionospheric-Model", "rnsap.gPS_Ionospheric_Model",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.GPS_Ionospheric_Model", HFILL }},
    { &hf_rnsap_gPS_UTC_Model,
      { "gPS-UTC-Model", "rnsap.gPS_UTC_Model",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.GPS_UTC_Model", HFILL }},
    { &hf_rnsap_gPS_Almanac,
      { "gPS-Almanac", "rnsap.gPS_Almanac",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.GPS_Almanac", HFILL }},
    { &hf_rnsap_gPS_RealTime_Integrity,
      { "gPS-RealTime-Integrity", "rnsap.gPS_RealTime_Integrity",
        FT_UINT32, BASE_DEC, VALS(rnsap_GPS_RealTime_Integrity_vals), 0,
        "rnsap.GPS_RealTime_Integrity", HFILL }},
    { &hf_rnsap_gPS_RX_POS,
      { "gPS-RX-POS", "rnsap.gPS_RX_POS",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.GPS_RX_POS", HFILL }},
    { &hf_rnsap_sFNSFN_GA_AccessPointPosition,
      { "sFNSFN-GA-AccessPointPosition", "rnsap.sFNSFN_GA_AccessPointPosition",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.GA_AccessPointPositionwithOptionalAltitude", HFILL }},
    { &hf_rnsap_informationAvailable,
      { "informationAvailable", "rnsap.informationAvailable",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.InformationAvailable", HFILL }},
    { &hf_rnsap_informationNotAvailable,
      { "informationNotAvailable", "rnsap.informationNotAvailable",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.InformationNotAvailable", HFILL }},
    { &hf_rnsap_RL_Specific_DCH_Info_item,
      { "Item", "rnsap.RL_Specific_DCH_Info_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_Specific_DCH_Info_Item", HFILL }},
    { &hf_rnsap_dCH_id,
      { "dCH-id", "rnsap.dCH_id",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DCH_ID", HFILL }},
    { &hf_rnsap_RL_Specific_EDCH_Information_item,
      { "Item", "rnsap.RL_Specific_EDCH_Information_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RL_Specific_EDCH_InfoItem", HFILL }},
    { &hf_rnsap_uplinkRTLoadValue,
      { "uplinkRTLoadValue", "rnsap.uplinkRTLoadValue",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_0_100", HFILL }},
    { &hf_rnsap_downlinkRTLoadValue,
      { "downlinkRTLoadValue", "rnsap.downlinkRTLoadValue",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_0_100", HFILL }},
    { &hf_rnsap_sAC,
      { "sAC", "rnsap.sAC",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.SAC", HFILL }},
    { &hf_rnsap_secondary_CCPCH_TDD_InformationList,
      { "secondary-CCPCH-TDD-InformationList", "rnsap.secondary_CCPCH_TDD_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Secondary_CCPCH_TDD_InformationList", HFILL }},
    { &hf_rnsap_fACH_InformationList,
      { "fACH-InformationList", "rnsap.fACH_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.FACH_InformationList", HFILL }},
    { &hf_rnsap_pCH_InformationList,
      { "pCH-InformationList", "rnsap.pCH_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.PCH_InformationList", HFILL }},
    { &hf_rnsap_new_secondary_CPICH,
      { "new-secondary-CPICH", "rnsap.new_secondary_CPICH",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Secondary_CPICH_Information", HFILL }},
    { &hf_rnsap_secondary_CPICH_shall_not_be_used,
      { "secondary-CPICH-shall-not-be-used", "rnsap.secondary_CPICH_shall_not_be_used",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.NULL", HFILL }},
    { &hf_rnsap_secondary_LCR_CCPCH_TDD_InformationList,
      { "secondary-LCR-CCPCH-TDD-InformationList", "rnsap.secondary_LCR_CCPCH_TDD_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Secondary_LCR_CCPCH_TDD_InformationList", HFILL }},
    { &hf_rnsap_Secondary_CCPCH_TDD_InformationList_item,
      { "Item", "rnsap.Secondary_CCPCH_TDD_InformationList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Secondary_CCPCH_TDD_InformationItem", HFILL }},
    { &hf_rnsap_secondary_CCPCH_TDD_Code_Information,
      { "secondary-CCPCH-TDD-Code-Information", "rnsap.secondary_CCPCH_TDD_Code_Information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Secondary_CCPCH_TDD_Code_Information", HFILL }},
    { &hf_rnsap_tDD_PhysicalChannelOffset,
      { "tDD-PhysicalChannelOffset", "rnsap.tDD_PhysicalChannelOffset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.TDD_PhysicalChannelOffset", HFILL }},
    { &hf_rnsap_Secondary_LCR_CCPCH_TDD_InformationList_item,
      { "Item", "rnsap.Secondary_LCR_CCPCH_TDD_InformationList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Secondary_LCR_CCPCH_TDD_InformationItem", HFILL }},
    { &hf_rnsap_secondary_LCR_CCPCH_TDD_Code_Information,
      { "secondary-LCR-CCPCH-TDD-Code-Information", "rnsap.secondary_LCR_CCPCH_TDD_Code_Information",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.Secondary_LCR_CCPCH_TDD_Code_Information", HFILL }},
    { &hf_rnsap_Secondary_CCPCH_TDD_Code_Information_item,
      { "Item", "rnsap.Secondary_CCPCH_TDD_Code_Information_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Secondary_CCPCH_TDD_Code_InformationItem", HFILL }},
    { &hf_rnsap_Secondary_LCR_CCPCH_TDD_Code_Information_item,
      { "Item", "rnsap.Secondary_LCR_CCPCH_TDD_Code_Information_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Secondary_LCR_CCPCH_TDD_Code_InformationItem", HFILL }},
    { &hf_rnsap_s_CCPCH_TimeSlotFormat_LCR,
      { "s-CCPCH-TimeSlotFormat-LCR", "rnsap.s_CCPCH_TimeSlotFormat_LCR",
        FT_UINT32, BASE_DEC, VALS(rnsap_TDD_DL_DPCH_TimeSlotFormat_LCR_vals), 0,
        "rnsap.TDD_DL_DPCH_TimeSlotFormat_LCR", HFILL }},
    { &hf_rnsap_sFNSFNChangeLimit,
      { "sFNSFNChangeLimit", "rnsap.sFNSFNChangeLimit",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.SFNSFNChangeLimit", HFILL }},
    { &hf_rnsap_predictedSFNSFNDeviationLimit,
      { "predictedSFNSFNDeviationLimit", "rnsap.predictedSFNSFNDeviationLimit",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.PredictedSFNSFNDeviationLimit", HFILL }},
    { &hf_rnsap_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation,
      { "successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation", "rnsap.successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.T_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation", HFILL }},
    { &hf_rnsap_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item,
      { "Item", "rnsap.successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.T_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item", HFILL }},
    { &hf_rnsap_sFNSFNValue,
      { "sFNSFNValue", "rnsap.sFNSFNValue",
        FT_UINT32, BASE_DEC, VALS(rnsap_SFNSFNValue_vals), 0,
        "rnsap.SFNSFNValue", HFILL }},
    { &hf_rnsap_sFNSFNQuality,
      { "sFNSFNQuality", "rnsap.sFNSFNQuality",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.SFNSFNQuality", HFILL }},
    { &hf_rnsap_sFNSFNDriftRate,
      { "sFNSFNDriftRate", "rnsap.sFNSFNDriftRate",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.SFNSFNDriftRate", HFILL }},
    { &hf_rnsap_sFNSFNDriftRateQuality,
      { "sFNSFNDriftRateQuality", "rnsap.sFNSFNDriftRateQuality",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.SFNSFNDriftRateQuality", HFILL }},
    { &hf_rnsap_sFNSFNTimeStampInformation,
      { "sFNSFNTimeStampInformation", "rnsap.sFNSFNTimeStampInformation",
        FT_UINT32, BASE_DEC, VALS(rnsap_SFNSFNTimeStampInformation_vals), 0,
        "rnsap.SFNSFNTimeStampInformation", HFILL }},
    { &hf_rnsap_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation,
      { "unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation", "rnsap.unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.T_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation", HFILL }},
    { &hf_rnsap_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item,
      { "Item", "rnsap.unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.T_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item", HFILL }},
    { &hf_rnsap_sFNSFNTimeStamp_FDD,
      { "sFNSFNTimeStamp-FDD", "rnsap.sFNSFNTimeStamp_FDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.SFN", HFILL }},
    { &hf_rnsap_sFNSFNTimeStamp_TDD,
      { "sFNSFNTimeStamp-TDD", "rnsap.sFNSFNTimeStamp_TDD",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.SFNSFNTimeStamp_TDD", HFILL }},
    { &hf_rnsap_sFN,
      { "sFN", "rnsap.sFN",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.SFN", HFILL }},
    { &hf_rnsap_sFNSFN_FDD,
      { "sFNSFN-FDD", "rnsap.sFNSFN_FDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.SFNSFN_FDD", HFILL }},
    { &hf_rnsap_sFNSFN_TDD,
      { "sFNSFN-TDD", "rnsap.sFNSFN_TDD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.SFNSFN_TDD", HFILL }},
    { &hf_rnsap_listOfSNAs,
      { "listOfSNAs", "rnsap.listOfSNAs",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.ListOfSNAs", HFILL }},
    { &hf_rnsap_ListOfSNAs_item,
      { "Item", "rnsap.ListOfSNAs_item",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.SNACode", HFILL }},
    { &hf_rnsap_sRNTI_BitMaskIndex,
      { "sRNTI-BitMaskIndex", "rnsap.sRNTI_BitMaskIndex",
        FT_UINT32, BASE_DEC, VALS(rnsap_T_sRNTI_BitMaskIndex_vals), 0,
        "rnsap.T_sRNTI_BitMaskIndex", HFILL }},
    { &hf_rnsap_n_INSYNC_IND,
      { "n-INSYNC-IND", "rnsap.n_INSYNC_IND",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_1_256", HFILL }},
    { &hf_rnsap_n_OUTSYNC_IND,
      { "n-OUTSYNC-IND", "rnsap.n_OUTSYNC_IND",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_1_256", HFILL }},
    { &hf_rnsap_t_RLFAILURE,
      { "t-RLFAILURE", "rnsap.t_RLFAILURE",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_0_255", HFILL }},
    { &hf_rnsap_maxSYNC_UL_transmissions,
      { "maxSYNC-UL-transmissions", "rnsap.maxSYNC_UL_transmissions",
        FT_UINT32, BASE_DEC, VALS(rnsap_T_maxSYNC_UL_transmissions_vals), 0,
        "rnsap.T_maxSYNC_UL_transmissions", HFILL }},
    { &hf_rnsap_powerRampStep,
      { "powerRampStep", "rnsap.powerRampStep",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_0_3_", HFILL }},
    { &hf_rnsap_modulation,
      { "modulation", "rnsap.modulation",
        FT_UINT32, BASE_DEC, VALS(rnsap_Modulation_vals), 0,
        "rnsap.Modulation", HFILL }},
    { &hf_rnsap_TDD_DCHs_to_Modify_item,
      { "Item", "rnsap.TDD_DCHs_to_Modify_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.TDD_DCHs_to_ModifyItem", HFILL }},
    { &hf_rnsap_dCH_SpecificInformationList3,
      { "dCH-SpecificInformationList", "rnsap.dCH_SpecificInformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.TDD_DCHs_to_ModifySpecificInformationList", HFILL }},
    { &hf_rnsap_TDD_DCHs_to_ModifySpecificInformationList_item,
      { "Item", "rnsap.TDD_DCHs_to_ModifySpecificInformationList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.TDD_DCHs_to_ModifySpecificItem", HFILL }},
    { &hf_rnsap_ul_CCTrCH_ID,
      { "ul-CCTrCH-ID", "rnsap.ul_CCTrCH_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.CCTrCH_ID", HFILL }},
    { &hf_rnsap_dl_CCTrCH_ID,
      { "dl-CCTrCH-ID", "rnsap.dl_CCTrCH_ID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.CCTrCH_ID", HFILL }},
    { &hf_rnsap_TDD_DL_Code_Information_item,
      { "Item", "rnsap.TDD_DL_Code_Information_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.TDD_DL_Code_InformationItem", HFILL }},
    { &hf_rnsap_TDD_DL_Code_LCR_Information_item,
      { "Item", "rnsap.TDD_DL_Code_LCR_Information_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.TDD_DL_Code_LCR_InformationItem", HFILL }},
    { &hf_rnsap_tdd_ChannelisationCodeLCR,
      { "tdd-ChannelisationCodeLCR", "rnsap.tdd_ChannelisationCodeLCR",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.TDD_ChannelisationCodeLCR", HFILL }},
    { &hf_rnsap_tdd_DL_DPCH_TimeSlotFormat_LCR,
      { "tdd-DL-DPCH-TimeSlotFormat-LCR", "rnsap.tdd_DL_DPCH_TimeSlotFormat_LCR",
        FT_UINT32, BASE_DEC, VALS(rnsap_TDD_DL_DPCH_TimeSlotFormat_LCR_vals), 0,
        "rnsap.TDD_DL_DPCH_TimeSlotFormat_LCR", HFILL }},
    { &hf_rnsap_qPSK,
      { "qPSK", "rnsap.qPSK",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.QPSK_DL_DPCH_TimeSlotFormatTDD_LCR", HFILL }},
    { &hf_rnsap_eightPSK,
      { "eightPSK", "rnsap.eightPSK",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.EightPSK_DL_DPCH_TimeSlotFormatTDD_LCR", HFILL }},
    { &hf_rnsap_initialOffset,
      { "initialOffset", "rnsap.initialOffset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_0_255", HFILL }},
    { &hf_rnsap_noinitialOffset,
      { "noinitialOffset", "rnsap.noinitialOffset",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_0_63", HFILL }},
    { &hf_rnsap_TDD_UL_Code_Information_item,
      { "Item", "rnsap.TDD_UL_Code_Information_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.TDD_UL_Code_InformationItem", HFILL }},
    { &hf_rnsap_TDD_UL_Code_LCR_Information_item,
      { "Item", "rnsap.TDD_UL_Code_LCR_Information_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.TDD_UL_Code_LCR_InformationItem", HFILL }},
    { &hf_rnsap_tdd_UL_DPCH_TimeSlotFormat_LCR,
      { "tdd-UL-DPCH-TimeSlotFormat-LCR", "rnsap.tdd_UL_DPCH_TimeSlotFormat_LCR",
        FT_UINT32, BASE_DEC, VALS(rnsap_TDD_UL_DPCH_TimeSlotFormat_LCR_vals), 0,
        "rnsap.TDD_UL_DPCH_TimeSlotFormat_LCR", HFILL }},
    { &hf_rnsap_qPSK1,
      { "qPSK", "rnsap.qPSK",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.QPSK_UL_DPCH_TimeSlotFormatTDD_LCR", HFILL }},
    { &hf_rnsap_eightPSK1,
      { "eightPSK", "rnsap.eightPSK",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.EightPSK_UL_DPCH_TimeSlotFormatTDD_LCR", HFILL }},
    { &hf_rnsap_plmn_id,
      { "plmn-id", "rnsap.plmn_id",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.PLMN_Identity", HFILL }},
    { &hf_rnsap_service_id,
      { "service-id", "rnsap.service_id",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.Service_ID", HFILL }},
    { &hf_rnsap_dsField,
      { "dsField", "rnsap.dsField",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.DsField", HFILL }},
    { &hf_rnsap_genericTrafficCategory,
      { "genericTrafficCategory", "rnsap.genericTrafficCategory",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.GenericTrafficCategory", HFILL }},
    { &hf_rnsap_Transmission_Gap_Pattern_Sequence_Information_item,
      { "Item", "rnsap.Transmission_Gap_Pattern_Sequence_Information_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Transmission_Gap_Pattern_Sequence_Information_item", HFILL }},
    { &hf_rnsap_tGPSID,
      { "tGPSID", "rnsap.tGPSID",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.TGPSID", HFILL }},
    { &hf_rnsap_tGSN,
      { "tGSN", "rnsap.tGSN",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.TGSN", HFILL }},
    { &hf_rnsap_tGL1,
      { "tGL1", "rnsap.tGL1",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.GapLength", HFILL }},
    { &hf_rnsap_tGL2,
      { "tGL2", "rnsap.tGL2",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.GapLength", HFILL }},
    { &hf_rnsap_tGD,
      { "tGD", "rnsap.tGD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.TGD", HFILL }},
    { &hf_rnsap_tGPL1,
      { "tGPL1", "rnsap.tGPL1",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.GapDuration", HFILL }},
    { &hf_rnsap_not_to_be_used_1,
      { "not-to-be-used-1", "rnsap.not_to_be_used_1",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.GapDuration", HFILL }},
    { &hf_rnsap_uL_DL_mode,
      { "uL-DL-mode", "rnsap.uL_DL_mode",
        FT_UINT32, BASE_DEC, VALS(rnsap_UL_DL_mode_vals), 0,
        "rnsap.UL_DL_mode", HFILL }},
    { &hf_rnsap_downlink_Compressed_Mode_Method,
      { "downlink-Compressed-Mode-Method", "rnsap.downlink_Compressed_Mode_Method",
        FT_UINT32, BASE_DEC, VALS(rnsap_Downlink_Compressed_Mode_Method_vals), 0,
        "rnsap.Downlink_Compressed_Mode_Method", HFILL }},
    { &hf_rnsap_uplink_Compressed_Mode_Method,
      { "uplink-Compressed-Mode-Method", "rnsap.uplink_Compressed_Mode_Method",
        FT_UINT32, BASE_DEC, VALS(rnsap_Uplink_Compressed_Mode_Method_vals), 0,
        "rnsap.Uplink_Compressed_Mode_Method", HFILL }},
    { &hf_rnsap_dL_FrameType,
      { "dL-FrameType", "rnsap.dL_FrameType",
        FT_UINT32, BASE_DEC, VALS(rnsap_DL_FrameType_vals), 0,
        "rnsap.DL_FrameType", HFILL }},
    { &hf_rnsap_delta_SIR1,
      { "delta-SIR1", "rnsap.delta_SIR1",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DeltaSIR", HFILL }},
    { &hf_rnsap_delta_SIR_after1,
      { "delta-SIR-after1", "rnsap.delta_SIR_after1",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DeltaSIR", HFILL }},
    { &hf_rnsap_delta_SIR2,
      { "delta-SIR2", "rnsap.delta_SIR2",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DeltaSIR", HFILL }},
    { &hf_rnsap_delta_SIR_after2,
      { "delta-SIR-after2", "rnsap.delta_SIR_after2",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.DeltaSIR", HFILL }},
    { &hf_rnsap_Transmission_Gap_Pattern_Sequence_Status_List_item,
      { "Item", "rnsap.Transmission_Gap_Pattern_Sequence_Status_List_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Transmission_Gap_Pattern_Sequence_Status_List_item", HFILL }},
    { &hf_rnsap_tGPRC,
      { "tGPRC", "rnsap.tGPRC",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.TGPRC", HFILL }},
    { &hf_rnsap_tGCFN,
      { "tGCFN", "rnsap.tGCFN",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.CFN", HFILL }},
    { &hf_rnsap_ms_part,
      { "ms-part", "rnsap.ms_part",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_0_16383", HFILL }},
    { &hf_rnsap_ls_part,
      { "ls-part", "rnsap.ls_part",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_0_4294967295", HFILL }},
    { &hf_rnsap_tUTRANGPSChangeLimit,
      { "tUTRANGPSChangeLimit", "rnsap.tUTRANGPSChangeLimit",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.TUTRANGPSChangeLimit", HFILL }},
    { &hf_rnsap_predictedTUTRANGPSDeviationLimit,
      { "predictedTUTRANGPSDeviationLimit", "rnsap.predictedTUTRANGPSDeviationLimit",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.PredictedTUTRANGPSDeviationLimit", HFILL }},
    { &hf_rnsap_tUTRANGPS,
      { "tUTRANGPS", "rnsap.tUTRANGPS",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.TUTRANGPS", HFILL }},
    { &hf_rnsap_tUTRANGPSQuality,
      { "tUTRANGPSQuality", "rnsap.tUTRANGPSQuality",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.TUTRANGPSQuality", HFILL }},
    { &hf_rnsap_tUTRANGPSDriftRate,
      { "tUTRANGPSDriftRate", "rnsap.tUTRANGPSDriftRate",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.TUTRANGPSDriftRate", HFILL }},
    { &hf_rnsap_tUTRANGPSDriftRateQuality,
      { "tUTRANGPSDriftRateQuality", "rnsap.tUTRANGPSDriftRateQuality",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.TUTRANGPSDriftRateQuality", HFILL }},
    { &hf_rnsap_iEe_Extensions,
      { "iEe-Extensions", "rnsap.iEe_Extensions",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.ProtocolExtensionContainer", HFILL }},
    { &hf_rnsap_signalledGainFactors,
      { "signalledGainFactors", "rnsap.signalledGainFactors",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.T_signalledGainFactors", HFILL }},
    { &hf_rnsap_betaC,
      { "betaC", "rnsap.betaC",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.BetaCD", HFILL }},
    { &hf_rnsap_betaD,
      { "betaD", "rnsap.betaD",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.BetaCD", HFILL }},
    { &hf_rnsap_refTFCNumber,
      { "refTFCNumber", "rnsap.refTFCNumber",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RefTFCNumber", HFILL }},
    { &hf_rnsap_tFCSvalues,
      { "tFCSvalues", "rnsap.tFCSvalues",
        FT_UINT32, BASE_DEC, VALS(rnsap_T_tFCSvalues_vals), 0,
        "rnsap.T_tFCSvalues", HFILL }},
    { &hf_rnsap_no_Split_in_TFCI,
      { "no-Split-in-TFCI", "rnsap.no_Split_in_TFCI",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.TFCS_TFCSList", HFILL }},
    { &hf_rnsap_not_Used_split_in_TFCI,
      { "not-Used-split-in-TFCI", "rnsap.not_Used_split_in_TFCI",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.NULL", HFILL }},
    { &hf_rnsap_TFCS_TFCSList_item,
      { "Item", "rnsap.TFCS_TFCSList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.TFCS_TFCSList_item", HFILL }},
    { &hf_rnsap_cTFC,
      { "cTFC", "rnsap.cTFC",
        FT_UINT32, BASE_DEC, VALS(rnsap_TFCS_CTFC_vals), 0,
        "rnsap.TFCS_CTFC", HFILL }},
    { &hf_rnsap_tFC_Beta,
      { "tFC-Beta", "rnsap.tFC_Beta",
        FT_UINT32, BASE_DEC, VALS(rnsap_TransportFormatCombination_Beta_vals), 0,
        "rnsap.TransportFormatCombination_Beta", HFILL }},
    { &hf_rnsap_ctfc2bit,
      { "ctfc2bit", "rnsap.ctfc2bit",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_0_3", HFILL }},
    { &hf_rnsap_ctfc4bit,
      { "ctfc4bit", "rnsap.ctfc4bit",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_0_15", HFILL }},
    { &hf_rnsap_ctfc6bit,
      { "ctfc6bit", "rnsap.ctfc6bit",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_0_63", HFILL }},
    { &hf_rnsap_ctfc8bit,
      { "ctfc8bit", "rnsap.ctfc8bit",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_0_255", HFILL }},
    { &hf_rnsap_ctfc12bit,
      { "ctfc12bit", "rnsap.ctfc12bit",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_0_4095", HFILL }},
    { &hf_rnsap_ctfc16bit,
      { "ctfc16bit", "rnsap.ctfc16bit",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_0_65535", HFILL }},
    { &hf_rnsap_ctfcmaxbit,
      { "ctfcmaxbit", "rnsap.ctfcmaxbit",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_0_16777215", HFILL }},
    { &hf_rnsap_dynamicParts,
      { "dynamicParts", "rnsap.dynamicParts",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.TransportFormatSet_DynamicPartList", HFILL }},
    { &hf_rnsap_semi_staticPart,
      { "semi-staticPart", "rnsap.semi_staticPart",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.TransportFormatSet_Semi_staticPart", HFILL }},
    { &hf_rnsap_TransportFormatSet_DynamicPartList_item,
      { "Item", "rnsap.TransportFormatSet_DynamicPartList_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.TransportFormatSet_DynamicPartList_item", HFILL }},
    { &hf_rnsap_nrOfTransportBlocks,
      { "nrOfTransportBlocks", "rnsap.nrOfTransportBlocks",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.NrOfTransportBlocks", HFILL }},
    { &hf_rnsap_transportBlockSize,
      { "transportBlockSize", "rnsap.transportBlockSize",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.TransportBlockSize", HFILL }},
    { &hf_rnsap_mode,
      { "mode", "rnsap.mode",
        FT_UINT32, BASE_DEC, VALS(rnsap_TransportFormatSet_ModeDP_vals), 0,
        "rnsap.TransportFormatSet_ModeDP", HFILL }},
    { &hf_rnsap_tdd,
      { "tdd", "rnsap.tdd",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.TDD_TransportFormatSet_ModeDP", HFILL }},
    { &hf_rnsap_notApplicable,
      { "notApplicable", "rnsap.notApplicable",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.NULL", HFILL }},
    { &hf_rnsap_transmissionTimeIntervalInformation,
      { "transmissionTimeIntervalInformation", "rnsap.transmissionTimeIntervalInformation",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.TransmissionTimeIntervalInformation", HFILL }},
    { &hf_rnsap_TransmissionTimeIntervalInformation_item,
      { "Item", "rnsap.TransmissionTimeIntervalInformation_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.TransmissionTimeIntervalInformation_item", HFILL }},
    { &hf_rnsap_transmissionTimeInterval,
      { "transmissionTimeInterval", "rnsap.transmissionTimeInterval",
        FT_UINT32, BASE_DEC, VALS(rnsap_TransmissionTimeIntervalDynamic_vals), 0,
        "rnsap.TransmissionTimeIntervalDynamic", HFILL }},
    { &hf_rnsap_transmissionTime,
      { "transmissionTime", "rnsap.transmissionTime",
        FT_UINT32, BASE_DEC, VALS(rnsap_TransmissionTimeIntervalSemiStatic_vals), 0,
        "rnsap.TransmissionTimeIntervalSemiStatic", HFILL }},
    { &hf_rnsap_channelCoding,
      { "channelCoding", "rnsap.channelCoding",
        FT_UINT32, BASE_DEC, VALS(rnsap_ChannelCodingType_vals), 0,
        "rnsap.ChannelCodingType", HFILL }},
    { &hf_rnsap_codingRate,
      { "codingRate", "rnsap.codingRate",
        FT_UINT32, BASE_DEC, VALS(rnsap_CodingRate_vals), 0,
        "rnsap.CodingRate", HFILL }},
    { &hf_rnsap_rateMatcingAttribute,
      { "rateMatcingAttribute", "rnsap.rateMatcingAttribute",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RateMatchingAttribute", HFILL }},
    { &hf_rnsap_cRC_Size,
      { "cRC-Size", "rnsap.cRC_Size",
        FT_UINT32, BASE_DEC, VALS(rnsap_CRC_Size_vals), 0,
        "rnsap.CRC_Size", HFILL }},
    { &hf_rnsap_mode1,
      { "mode", "rnsap.mode",
        FT_UINT32, BASE_DEC, VALS(rnsap_TransportFormatSet_ModeSSP_vals), 0,
        "rnsap.TransportFormatSet_ModeSSP", HFILL }},
    { &hf_rnsap_tdd1,
      { "tdd", "rnsap.tdd",
        FT_UINT32, BASE_DEC, VALS(rnsap_SecondInterleavingMode_vals), 0,
        "rnsap.SecondInterleavingMode", HFILL }},
    { &hf_rnsap_hSDSCH_Physical_Layer_Category,
      { "hSDSCH-Physical-Layer-Category", "rnsap.hSDSCH_Physical_Layer_Category",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_1_64_", HFILL }},
    { &hf_rnsap_imei,
      { "imei", "rnsap.imei",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.IMEI", HFILL }},
    { &hf_rnsap_imeisv,
      { "imeisv", "rnsap.imeisv",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.IMEISV", HFILL }},
    { &hf_rnsap_periodic2,
      { "periodic", "rnsap.periodic",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UEMeasurementReportCharacteristicsPeriodic", HFILL }},
    { &hf_rnsap_event1h,
      { "event1h", "rnsap.event1h",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UEMeasurementReportCharacteristicsEvent1h", HFILL }},
    { &hf_rnsap_event1i,
      { "event1i", "rnsap.event1i",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UEMeasurementReportCharacteristicsEvent1i", HFILL }},
    { &hf_rnsap_event6a,
      { "event6a", "rnsap.event6a",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UEMeasurementReportCharacteristicsEvent6a", HFILL }},
    { &hf_rnsap_event6b,
      { "event6b", "rnsap.event6b",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UEMeasurementReportCharacteristicsEvent6b", HFILL }},
    { &hf_rnsap_event6c,
      { "event6c", "rnsap.event6c",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UEMeasurementReportCharacteristicsEvent6c", HFILL }},
    { &hf_rnsap_event6d,
      { "event6d", "rnsap.event6d",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UEMeasurementReportCharacteristicsEvent6d", HFILL }},
    { &hf_rnsap_extension_ReportCharacteristics1,
      { "extension-ReportCharacteristics", "rnsap.extension_ReportCharacteristics",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UEMeasurementReportCharacteristics_Extension", HFILL }},
    { &hf_rnsap_uEMeasurementTreshold,
      { "uEMeasurementTreshold", "rnsap.uEMeasurementTreshold",
        FT_UINT32, BASE_DEC, VALS(rnsap_UEMeasurementThreshold_vals), 0,
        "rnsap.UEMeasurementThreshold", HFILL }},
    { &hf_rnsap_uEMeasurementTimeToTrigger,
      { "uEMeasurementTimeToTrigger", "rnsap.uEMeasurementTimeToTrigger",
        FT_UINT32, BASE_DEC, VALS(rnsap_UEMeasurementTimeToTrigger_vals), 0,
        "rnsap.UEMeasurementTimeToTrigger", HFILL }},
    { &hf_rnsap_uEMeasurementHysteresisTime,
      { "uEMeasurementHysteresisTime", "rnsap.uEMeasurementHysteresisTime",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UEMeasurementHysteresisTime", HFILL }},
    { &hf_rnsap_amountofReporting,
      { "amountofReporting", "rnsap.amountofReporting",
        FT_UINT32, BASE_DEC, VALS(rnsap_UEMeasurementReportCharacteristicsPeriodicAmountofReporting_vals), 0,
        "rnsap.UEMeasurementReportCharacteristicsPeriodicAmountofReporting", HFILL }},
    { &hf_rnsap_reportingInterval,
      { "reportingInterval", "rnsap.reportingInterval",
        FT_UINT32, BASE_DEC, VALS(rnsap_UEMeasurementReportCharacteristicsPeriodicReportingInterval_vals), 0,
        "rnsap.UEMeasurementReportCharacteristicsPeriodicReportingInterval", HFILL }},
    { &hf_rnsap_timeslotISCP,
      { "timeslotISCP", "rnsap.timeslotISCP",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.UEMeasurementThresholdDLTimeslotISCP", HFILL }},
    { &hf_rnsap_uETransmitPower,
      { "uETransmitPower", "rnsap.uETransmitPower",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.UEMeasurementThresholdUETransmitPower", HFILL }},
    { &hf_rnsap_extension_UEMeasurementThreshold,
      { "extension-UEMeasurementThreshold", "rnsap.extension_UEMeasurementThreshold",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UEMeasurementThreshold_Extension", HFILL }},
    { &hf_rnsap_UEMeasurementTimeslotInfoHCR_item,
      { "Item", "rnsap.UEMeasurementTimeslotInfoHCR_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UEMeasurementTimeslotInfoHCR_IEs", HFILL }},
    { &hf_rnsap_burstType,
      { "burstType", "rnsap.burstType",
        FT_UINT32, BASE_DEC, VALS(rnsap_UEMeasurementTimeslotInfoHCRBurstType_vals), 0,
        "rnsap.UEMeasurementTimeslotInfoHCRBurstType", HFILL }},
    { &hf_rnsap_UEMeasurementTimeslotInfoLCR_item,
      { "Item", "rnsap.UEMeasurementTimeslotInfoLCR_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UEMeasurementTimeslotInfoLCR_IEs", HFILL }},
    { &hf_rnsap_timeSlot1,
      { "timeSlot", "rnsap.timeSlot",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.TimeSlotLCR", HFILL }},
    { &hf_rnsap_uE_Transmitted_Power,
      { "uE-Transmitted-Power", "rnsap.uE_Transmitted_Power",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UE_MeasurementValue_UE_Transmitted_Power", HFILL }},
    { &hf_rnsap_primary_CCPCH_RSCP,
      { "primary-CCPCH-RSCP", "rnsap.primary_CCPCH_RSCP",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UE_MeasurementValue_Primary_CCPCH_RSCP", HFILL }},
    { &hf_rnsap_dL_Timeslot_ISCP,
      { "dL-Timeslot-ISCP", "rnsap.dL_Timeslot_ISCP",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UE_MeasurementValue_DL_Timeslot_ISCP", HFILL }},
    { &hf_rnsap_extension_UEMeasurementValue,
      { "extension-UEMeasurementValue", "rnsap.extension_UEMeasurementValue",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UEMeasurementValue_Extension", HFILL }},
    { &hf_rnsap_uEMeasurementTransmittedPowerListHCR,
      { "uEMeasurementTransmittedPowerListHCR", "rnsap.uEMeasurementTransmittedPowerListHCR",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UEMeasurementValueTransmittedPowerListHCR", HFILL }},
    { &hf_rnsap_uEMeasurementTransmittedPowerListLCR,
      { "uEMeasurementTransmittedPowerListLCR", "rnsap.uEMeasurementTransmittedPowerListLCR",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UEMeasurementValueTransmittedPowerListLCR", HFILL }},
    { &hf_rnsap_UEMeasurementValueTransmittedPowerListHCR_item,
      { "Item", "rnsap.UEMeasurementValueTransmittedPowerListHCR_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UEMeasurementValueTransmittedPowerListHCR_IEs", HFILL }},
    { &hf_rnsap_uETransmitPower1,
      { "uETransmitPower", "rnsap.uETransmitPower",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_0_104", HFILL }},
    { &hf_rnsap_UEMeasurementValueTransmittedPowerListLCR_item,
      { "Item", "rnsap.UEMeasurementValueTransmittedPowerListLCR_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UEMeasurementValueTransmittedPowerListLCR_IEs", HFILL }},
    { &hf_rnsap_primaryCCPCH_RSCP_Delta,
      { "primaryCCPCH-RSCP-Delta", "rnsap.primaryCCPCH_RSCP_Delta",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.PrimaryCCPCH_RSCP_Delta", HFILL }},
    { &hf_rnsap_uEMeasurementTimeslotISCPListHCR,
      { "uEMeasurementTimeslotISCPListHCR", "rnsap.uEMeasurementTimeslotISCPListHCR",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UEMeasurementValueTimeslotISCPListHCR", HFILL }},
    { &hf_rnsap_uEMeasurementTimeslotISCPListLCR,
      { "uEMeasurementTimeslotISCPListLCR", "rnsap.uEMeasurementTimeslotISCPListLCR",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UEMeasurementValueTimeslotISCPListLCR", HFILL }},
    { &hf_rnsap_UEMeasurementValueTimeslotISCPListHCR_item,
      { "Item", "rnsap.UEMeasurementValueTimeslotISCPListHCR_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UEMeasurementValueTimeslotISCPListHCR_IEs", HFILL }},
    { &hf_rnsap_UEMeasurementValueTimeslotISCPListLCR_item,
      { "Item", "rnsap.UEMeasurementValueTimeslotISCPListLCR_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UEMeasurementValueTimeslotISCPListLCR_IEs", HFILL }},
    { &hf_rnsap_measurementAvailable2,
      { "measurementAvailable", "rnsap.measurementAvailable",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UEMeasurementValueInformationAvailable", HFILL }},
    { &hf_rnsap_measurementnotAvailable2,
      { "measurementnotAvailable", "rnsap.measurementnotAvailable",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UEMeasurementValueInformationnotAvailable", HFILL }},
    { &hf_rnsap_uEmeasurementValue,
      { "uEmeasurementValue", "rnsap.uEmeasurementValue",
        FT_UINT32, BASE_DEC, VALS(rnsap_UEMeasurementValue_vals), 0,
        "rnsap.UEMeasurementValue", HFILL }},
    { &hf_rnsap_cell_fach_pch,
      { "cell-fach-pch", "rnsap.cell_fach_pch",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Cell_Fach_Pch_State", HFILL }},
    { &hf_rnsap_ura_pch,
      { "ura-pch", "rnsap.ura_pch",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Ura_Pch_State", HFILL }},
    { &hf_rnsap_d_RNTI,
      { "d-RNTI", "rnsap.d_RNTI",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.D_RNTI", HFILL }},
    { &hf_rnsap_srnc_id,
      { "srnc-id", "rnsap.srnc_id",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RNC_ID", HFILL }},
    { &hf_rnsap_ura_id,
      { "ura-id", "rnsap.ura_id",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.URA_ID", HFILL }},
    { &hf_rnsap_UL_Timeslot_Information_item,
      { "Item", "rnsap.UL_Timeslot_Information_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_Timeslot_InformationItem", HFILL }},
    { &hf_rnsap_UL_TimeslotLCR_Information_item,
      { "Item", "rnsap.UL_TimeslotLCR_Information_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_TimeslotLCR_InformationItem", HFILL }},
    { &hf_rnsap_uL_Code_LCR_InformationList,
      { "uL-Code-LCR-InformationList", "rnsap.uL_Code_LCR_InformationList",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.TDD_UL_Code_LCR_Information", HFILL }},
    { &hf_rnsap_UL_TimeSlot_ISCP_Info_item,
      { "Item", "rnsap.UL_TimeSlot_ISCP_Info_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_TimeSlot_ISCP_InfoItem", HFILL }},
    { &hf_rnsap_uL_TimeslotISCP,
      { "uL-TimeslotISCP", "rnsap.uL_TimeslotISCP",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UL_TimeslotISCP", HFILL }},
    { &hf_rnsap_UL_TimeSlot_ISCP_LCR_Info_item,
      { "Item", "rnsap.UL_TimeSlot_ISCP_LCR_Info_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.UL_TimeSlot_ISCP_LCR_InfoItem", HFILL }},
    { &hf_rnsap_iSCP,
      { "iSCP", "rnsap.iSCP",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UL_Timeslot_ISCP_Value", HFILL }},
    { &hf_rnsap_sync_UL_codes_bitmap,
      { "sync-UL-codes-bitmap", "rnsap.sync_UL_codes_bitmap",
        FT_BYTES, BASE_HEX, NULL, 0,
        "rnsap.BIT_STRING_SIZE_8", HFILL }},
    { &hf_rnsap_fPACH_info,
      { "fPACH-info", "rnsap.fPACH_info",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.FPACH_Information", HFILL }},
    { &hf_rnsap_prxUpPCHdes,
      { "prxUpPCHdes", "rnsap.prxUpPCHdes",
        FT_INT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_M120_M58_", HFILL }},
    { &hf_rnsap_syncUL_procParameter,
      { "syncUL-procParameter", "rnsap.syncUL_procParameter",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.SYNC_UL_ProcParameters", HFILL }},
    { &hf_rnsap_mMax,
      { "mMax", "rnsap.mMax",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_1_32", HFILL }},
    { &hf_rnsap_ul_ScramblingCodeNumber,
      { "ul-ScramblingCodeNumber", "rnsap.ul_ScramblingCodeNumber",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UL_ScramblingCodeNumber", HFILL }},
    { &hf_rnsap_ul_ScramblingCodeLength,
      { "ul-ScramblingCodeLength", "rnsap.ul_ScramblingCodeLength",
        FT_UINT32, BASE_DEC, VALS(rnsap_UL_ScramblingCodeLength_vals), 0,
        "rnsap.UL_ScramblingCodeLength", HFILL }},
    { &hf_rnsap_uL_Synchronisation_StepSize,
      { "uL-Synchronisation-StepSize", "rnsap.uL_Synchronisation_StepSize",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UL_Synchronisation_StepSize", HFILL }},
    { &hf_rnsap_uL_Synchronisation_Frequency,
      { "uL-Synchronisation-Frequency", "rnsap.uL_Synchronisation_Frequency",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.UL_Synchronisation_Frequency", HFILL }},
    { &hf_rnsap_multipleURAsIndicator,
      { "multipleURAsIndicator", "rnsap.multipleURAsIndicator",
        FT_UINT32, BASE_DEC, VALS(rnsap_MultipleURAsIndicator_vals), 0,
        "rnsap.MultipleURAsIndicator", HFILL }},
    { &hf_rnsap_rNCsWithCellsInTheAccessedURA_List,
      { "rNCsWithCellsInTheAccessedURA-List", "rnsap.rNCsWithCellsInTheAccessedURA_List",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.RNCsWithCellsInTheAccessedURA_List", HFILL }},
    { &hf_rnsap_RNCsWithCellsInTheAccessedURA_List_item,
      { "Item", "rnsap.RNCsWithCellsInTheAccessedURA_List_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.RNCsWithCellsInTheAccessedURA_Item", HFILL }},
    { &hf_rnsap_USCH_Information_item,
      { "Item", "rnsap.USCH_Information_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.USCH_InformationItem", HFILL }},
    { &hf_rnsap_local,
      { "local", "rnsap.local",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_0_65535", HFILL }},
    { &hf_rnsap_global,
      { "global", "rnsap.global",
        FT_OID, BASE_NONE, NULL, 0,
        "rnsap.OBJECT_IDENTIFIER", HFILL }},
    { &hf_rnsap_procedureCode,
      { "procedureCode", "rnsap.procedureCode",
        FT_UINT32, BASE_DEC, VALS(rnsap_ProcedureCode_vals), 0,
        "rnsap.ProcedureCode", HFILL }},
    { &hf_rnsap_ddMode,
      { "ddMode", "rnsap.ddMode",
        FT_UINT32, BASE_DEC, VALS(rnsap_T_ddMode_vals), 0,
        "rnsap.T_ddMode", HFILL }},
    { &hf_rnsap_shortTransActionId,
      { "shortTransActionId", "rnsap.shortTransActionId",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_0_127", HFILL }},
    { &hf_rnsap_longTransActionId,
      { "longTransActionId", "rnsap.longTransActionId",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.INTEGER_0_32767", HFILL }},
    { &hf_rnsap_ProtocolIE_Container_item,
      { "Item", "rnsap.ProtocolIE_Container_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_Field", HFILL }},
    { &hf_rnsap_id,
      { "id", "rnsap.id",
        FT_UINT32, BASE_DEC, VALS(rnsap_ProtocolIE_ID_vals), 0,
        "rnsap.ProtocolIE_ID", HFILL }},
    { &hf_rnsap_value,
      { "value", "rnsap.value",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIEValue", HFILL }},
    { &hf_rnsap_ProtocolIE_ContainerPair_item,
      { "Item", "rnsap.ProtocolIE_ContainerPair_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolIE_FieldPair", HFILL }},
    { &hf_rnsap_firstCriticality,
      { "firstCriticality", "rnsap.firstCriticality",
        FT_UINT32, BASE_DEC, VALS(rnsap_Criticality_vals), 0,
        "rnsap.Criticality", HFILL }},
    { &hf_rnsap_firstValue,
      { "firstValue", "rnsap.firstValue",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.FirstValue", HFILL }},
    { &hf_rnsap_secondCriticality,
      { "secondCriticality", "rnsap.secondCriticality",
        FT_UINT32, BASE_DEC, VALS(rnsap_Criticality_vals), 0,
        "rnsap.Criticality", HFILL }},
    { &hf_rnsap_secondValue,
      { "secondValue", "rnsap.secondValue",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.SecondValue", HFILL }},
    { &hf_rnsap_ProtocolIE_ContainerList_item,
      { "Item", "rnsap.ProtocolIE_ContainerList_item",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.ProtocolIE_Container", HFILL }},
    { &hf_rnsap_ProtocolIE_ContainerPairList_item,
      { "Item", "rnsap.ProtocolIE_ContainerPairList_item",
        FT_UINT32, BASE_DEC, NULL, 0,
        "rnsap.ProtocolIE_ContainerPair", HFILL }},
    { &hf_rnsap_ProtocolExtensionContainer_item,
      { "Item", "rnsap.ProtocolExtensionContainer_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.ProtocolExtensionField", HFILL }},
    { &hf_rnsap_extensionValue,
      { "extensionValue", "rnsap.extensionValue",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.Extension", HFILL }},
    { &hf_rnsap_PrivateIE_Container_item,
      { "Item", "rnsap.PrivateIE_Container_item",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.PrivateIE_Field", HFILL }},
    { &hf_rnsap_privateIEid,
      { "privateIEid", "rnsap.privateIEid",
        FT_UINT32, BASE_DEC, VALS(rnsap_PrivateIE_ID_vals), 0,
        "rnsap.PrivateIE_ID", HFILL }},
    { &hf_rnsap_privateIEvalue,
      { "privateIEvalue", "rnsap.privateIEvalue",
        FT_NONE, BASE_NONE, NULL, 0,
        "rnsap.PrivateIEvalue", HFILL }},

/*--- End of included file: packet-rnsap-hfarr.c ---*/
#line 2325 "packet-rnsap-template.c"
  };

  /* List of subtrees */
  static gint *ett[] = {
		  &ett_rnsap,
		  &ett_rnsap_initiatingMessageValue,
		  &ett_rnsap_ProtocolIEValueValue,
		  &ett_rnsap_SuccessfulOutcomeValue,
		  &ett_rnsap_UnsuccessfulOutcomeValue,

/*--- Included file: packet-rnsap-ettarr.c ---*/
#line 1 "packet-rnsap-ettarr.c"
    &ett_rnsap_RNSAP_PDU,
    &ett_rnsap_InitiatingMessage,
    &ett_rnsap_SuccessfulOutcome,
    &ett_rnsap_UnsuccessfulOutcome,
    &ett_rnsap_Outcome,
    &ett_rnsap_DummyInitiatingValue,
    &ett_rnsap_DummySuccessfulOutcomeValue,
    &ett_rnsap_DummyUnSuccessfulOutcomeValue,
    &ett_rnsap_DummyProtocolIE,
    &ett_rnsap_RadioLinkSetupRequestFDD,
    &ett_rnsap_UL_DPCH_Information_RL_SetupRqstFDD,
    &ett_rnsap_DL_DPCH_Information_RL_SetupRqstFDD,
    &ett_rnsap_PowerOffsetInformation_RL_SetupRqstFDD,
    &ett_rnsap_RL_InformationList_RL_SetupRqstFDD,
    &ett_rnsap_RL_InformationItem_RL_SetupRqstFDD,
    &ett_rnsap_F_DPCH_Information_RL_SetupRqstFDD,
    &ett_rnsap_PowerOffsetInformation_F_DPCH_RL_SetupRqstFDD,
    &ett_rnsap_RadioLinkSetupRequestTDD,
    &ett_rnsap_UL_Physical_Channel_Information_RL_SetupRqstTDD,
    &ett_rnsap_DL_Physical_Channel_Information_RL_SetupRqstTDD,
    &ett_rnsap_UL_CCTrCH_InformationList_RL_SetupRqstTDD,
    &ett_rnsap_UL_CCTrCH_InformationItem_RL_SetupRqstTDD,
    &ett_rnsap_DL_CCTrCH_InformationList_RL_SetupRqstTDD,
    &ett_rnsap_DL_CCTrCH_InformationItem_RL_SetupRqstTDD,
    &ett_rnsap_CCTrCH_TPCList_RL_SetupRqstTDD,
    &ett_rnsap_CCTrCH_TPCItem_RL_SetupRqstTDD,
    &ett_rnsap_RL_Information_RL_SetupRqstTDD,
    &ett_rnsap_RadioLinkSetupResponseFDD,
    &ett_rnsap_RL_InformationResponseList_RL_SetupRspFDD,
    &ett_rnsap_RL_InformationResponseItem_RL_SetupRspFDD,
    &ett_rnsap_DiversityIndication_RL_SetupRspFDD,
    &ett_rnsap_Combining_RL_SetupRspFDD,
    &ett_rnsap_NonCombiningOrFirstRL_RL_SetupRspFDD,
    &ett_rnsap_RadioLinkSetupResponseTDD,
    &ett_rnsap_RL_InformationResponse_RL_SetupRspTDD,
    &ett_rnsap_UL_CCTrCHInformationListIE_RL_SetupRspTDD,
    &ett_rnsap_UL_CCTrCHInformationItem_RL_SetupRspTDD,
    &ett_rnsap_UL_DPCH_InformationItem_RL_SetupRspTDD,
    &ett_rnsap_DL_CCTrCHInformationListIE_RL_SetupRspTDD,
    &ett_rnsap_DL_CCTrCHInformationItem_RL_SetupRspTDD,
    &ett_rnsap_DL_DPCH_InformationItem_RL_SetupRspTDD,
    &ett_rnsap_DSCH_InformationListIEs_RL_SetupRspTDD,
    &ett_rnsap_DSCHInformationItem_RL_SetupRspTDD,
    &ett_rnsap_USCH_InformationListIEs_RL_SetupRspTDD,
    &ett_rnsap_USCHInformationItem_RL_SetupRspTDD,
    &ett_rnsap_RL_LCR_InformationResponse_RL_SetupRspTDD,
    &ett_rnsap_UL_LCR_CCTrCHInformationListIE_RL_SetupRspTDD,
    &ett_rnsap_UL_LCR_CCTrCHInformationItem_RL_SetupRspTDD,
    &ett_rnsap_UL_DPCH_LCR_InformationItem_RL_SetupRspTDD,
    &ett_rnsap_DL_CCTrCH_LCR_InformationListIE_RL_SetupRspTDD,
    &ett_rnsap_DL_CCTrCH_LCR_InformationItem_RL_SetupRspTDD,
    &ett_rnsap_DL_DPCH_LCR_InformationItem_RL_SetupRspTDD,
    &ett_rnsap_DSCH_LCR_InformationListIEs_RL_SetupRspTDD,
    &ett_rnsap_DSCH_LCR_InformationItem_RL_SetupRspTDD,
    &ett_rnsap_USCH_LCR_InformationListIEs_RL_SetupRspTDD,
    &ett_rnsap_USCH_LCR_InformationItem_RL_SetupRspTDD,
    &ett_rnsap_RadioLinkSetupFailureFDD,
    &ett_rnsap_CauseLevel_RL_SetupFailureFDD,
    &ett_rnsap_GeneralCauseList_RL_SetupFailureFDD,
    &ett_rnsap_RLSpecificCauseList_RL_SetupFailureFDD,
    &ett_rnsap_UnsuccessfulRL_InformationResponseList_RL_SetupFailureFDD,
    &ett_rnsap_UnsuccessfulRL_InformationResponse_RL_SetupFailureFDD,
    &ett_rnsap_SuccessfulRL_InformationResponseList_RL_SetupFailureFDD,
    &ett_rnsap_SuccessfulRL_InformationResponse_RL_SetupFailureFDD,
    &ett_rnsap_DiversityIndication_RL_SetupFailureFDD,
    &ett_rnsap_Combining_RL_SetupFailureFDD,
    &ett_rnsap_NonCombiningOrFirstRL_RL_SetupFailureFDD,
    &ett_rnsap_RadioLinkSetupFailureTDD,
    &ett_rnsap_CauseLevel_RL_SetupFailureTDD,
    &ett_rnsap_GeneralCauseList_RL_SetupFailureTDD,
    &ett_rnsap_RLSpecificCauseList_RL_SetupFailureTDD,
    &ett_rnsap_UnsuccessfulRL_InformationResponse_RL_SetupFailureTDD,
    &ett_rnsap_RadioLinkAdditionRequestFDD,
    &ett_rnsap_RL_InformationList_RL_AdditionRqstFDD,
    &ett_rnsap_RL_Information_RL_AdditionRqstFDD,
    &ett_rnsap_RadioLinkAdditionRequestTDD,
    &ett_rnsap_RL_Information_RL_AdditionRqstTDD,
    &ett_rnsap_UL_CCTrCH_InformationList_RL_AdditionRqstTDD,
    &ett_rnsap_UL_CCTrCH_InformationItem_RL_AdditionRqstTDD,
    &ett_rnsap_DL_CCTrCH_InformationList_RL_AdditionRqstTDD,
    &ett_rnsap_DL_CCTrCH_InformationItem_RL_AdditionRqstTDD,
    &ett_rnsap_RadioLinkAdditionResponseFDD,
    &ett_rnsap_RL_InformationResponseList_RL_AdditionRspFDD,
    &ett_rnsap_RL_InformationResponseItem_RL_AdditionRspFDD,
    &ett_rnsap_DiversityIndication_RL_AdditionRspFDD,
    &ett_rnsap_Combining_RL_AdditionRspFDD,
    &ett_rnsap_NonCombining_RL_AdditionRspFDD,
    &ett_rnsap_RadioLinkAdditionResponseTDD,
    &ett_rnsap_RL_InformationResponse_RL_AdditionRspTDD,
    &ett_rnsap_UL_CCTrCHInformationListIE_RL_AdditionRspTDD,
    &ett_rnsap_UL_CCTrCHInformationItem_RL_AdditionRspTDD,
    &ett_rnsap_UL_DPCH_InformationItem_RL_AdditionRspTDD,
    &ett_rnsap_DL_CCTrCHInformationListIE_RL_AdditionRspTDD,
    &ett_rnsap_DL_CCTrCHInformationItem_RL_AdditionRspTDD,
    &ett_rnsap_DL_DPCH_InformationItem_RL_AdditionRspTDD,
    &ett_rnsap_DCH_Information_RL_AdditionRspTDD,
    &ett_rnsap_DiversityIndication_RL_AdditionRspTDD,
    &ett_rnsap_Combining_RL_AdditionRspTDD,
    &ett_rnsap_NonCombining_RL_AdditionRspTDD,
    &ett_rnsap_DSCH_InformationListIE_RL_AdditionRspTDD,
    &ett_rnsap_DSCHInformationItem_RL_AdditionRspTDD,
    &ett_rnsap_DiversityIndication_RL_AdditionRspTDD2,
    &ett_rnsap_USCH_InformationListIE_RL_AdditionRspTDD,
    &ett_rnsap_USCHInformationItem_RL_AdditionRspTDD,
    &ett_rnsap_RL_LCR_InformationResponse_RL_AdditionRspTDD,
    &ett_rnsap_UL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD,
    &ett_rnsap_UL_CCTrCH_LCR_InformationItem_RL_AdditionRspTDD,
    &ett_rnsap_UL_DPCH_LCR_InformationItem_RL_AdditionRspTDD,
    &ett_rnsap_DL_CCTrCH_LCR_InformationListIE_RL_AdditionRspTDD,
    &ett_rnsap_DL_CCTrCH_LCR_InformationItem_RL_AdditionRspTDD,
    &ett_rnsap_DL_DPCH_LCR_InformationItem_RL_AdditionRspTDD,
    &ett_rnsap_DSCH_LCR_InformationListIEs_RL_AdditionRspTDD,
    &ett_rnsap_DSCH_LCR_InformationItem_RL_AdditionRspTDD,
    &ett_rnsap_USCH_LCR_InformationListIEs_RL_AdditionRspTDD,
    &ett_rnsap_USCH_LCR_InformationItem_RL_AdditionRspTDD,
    &ett_rnsap_RadioLinkAdditionFailureFDD,
    &ett_rnsap_CauseLevel_RL_AdditionFailureFDD,
    &ett_rnsap_GeneralCauseList_RL_AdditionFailureFDD,
    &ett_rnsap_RLSpecificCauseList_RL_AdditionFailureFDD,
    &ett_rnsap_UnsuccessfulRL_InformationResponseList_RL_AdditionFailureFDD,
    &ett_rnsap_UnsuccessfulRL_InformationResponse_RL_AdditionFailureFDD,
    &ett_rnsap_SuccessfulRL_InformationResponseList_RL_AdditionFailureFDD,
    &ett_rnsap_SuccessfulRL_InformationResponse_RL_AdditionFailureFDD,
    &ett_rnsap_DiversityIndication_RL_AdditionFailureFDD,
    &ett_rnsap_Combining_RL_AdditionFailureFDD,
    &ett_rnsap_NonCombining_RL_AdditionFailureFDD,
    &ett_rnsap_RadioLinkAdditionFailureTDD,
    &ett_rnsap_CauseLevel_RL_AdditionFailureTDD,
    &ett_rnsap_GeneralCauseList_RL_AdditionFailureTDD,
    &ett_rnsap_RLSpecificCauseList_RL_AdditionFailureTDD,
    &ett_rnsap_UnsuccessfulRL_InformationResponse_RL_AdditionFailureTDD,
    &ett_rnsap_RadioLinkDeletionRequest,
    &ett_rnsap_RL_InformationList_RL_DeletionRqst,
    &ett_rnsap_RL_Information_RL_DeletionRqst,
    &ett_rnsap_RadioLinkDeletionResponse,
    &ett_rnsap_RadioLinkReconfigurationPrepareFDD,
    &ett_rnsap_UL_DPCH_Information_RL_ReconfPrepFDD,
    &ett_rnsap_DL_DPCH_Information_RL_ReconfPrepFDD,
    &ett_rnsap_DL_DPCH_Power_Information_RL_ReconfPrepFDD,
    &ett_rnsap_PowerOffsetInformation_RL_ReconfPrepFDD,
    &ett_rnsap_DCH_DeleteList_RL_ReconfPrepFDD,
    &ett_rnsap_DCH_DeleteItem_RL_ReconfPrepFDD,
    &ett_rnsap_RL_InformationList_RL_ReconfPrepFDD,
    &ett_rnsap_RL_Information_RL_ReconfPrepFDD,
    &ett_rnsap_F_DPCH_Information_RL_ReconfPrepFDD,
    &ett_rnsap_PowerOffsetInformation_F_DPCH_RL_ReconfPrepFDD,
    &ett_rnsap_RadioLinkReconfigurationPrepareTDD,
    &ett_rnsap_UL_CCTrCH_InformationAddList_RL_ReconfPrepTDD,
    &ett_rnsap_UL_CCTrCH_AddInformation_RL_ReconfPrepTDD,
    &ett_rnsap_UL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD,
    &ett_rnsap_UL_CCTrCH_ModifyInformation_RL_ReconfPrepTDD,
    &ett_rnsap_UL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD,
    &ett_rnsap_UL_CCTrCH_DeleteInformation_RL_ReconfPrepTDD,
    &ett_rnsap_DL_CCTrCH_InformationAddList_RL_ReconfPrepTDD,
    &ett_rnsap_DL_CCTrCH_InformationAddItem_RL_ReconfPrepTDD,
    &ett_rnsap_CCTrCH_TPCAddList_RL_ReconfPrepTDD,
    &ett_rnsap_CCTrCH_TPCAddItem_RL_ReconfPrepTDD,
    &ett_rnsap_DL_CCTrCH_InformationModifyList_RL_ReconfPrepTDD,
    &ett_rnsap_DL_CCTrCH_InformationModifyItem_RL_ReconfPrepTDD,
    &ett_rnsap_CCTrCH_TPCModifyList_RL_ReconfPrepTDD,
    &ett_rnsap_CCTrCH_TPCModifyItem_RL_ReconfPrepTDD,
    &ett_rnsap_DL_CCTrCH_InformationDeleteList_RL_ReconfPrepTDD,
    &ett_rnsap_DL_CCTrCH_InformationDeleteItem_RL_ReconfPrepTDD,
    &ett_rnsap_DCH_DeleteList_RL_ReconfPrepTDD,
    &ett_rnsap_DCH_DeleteItem_RL_ReconfPrepTDD,
    &ett_rnsap_DSCH_ModifyList_RL_ReconfPrepTDD,
    &ett_rnsap_DSCH_ModifyItem_RL_ReconfPrepTDD,
    &ett_rnsap_DSCH_DeleteList_RL_ReconfPrepTDD,
    &ett_rnsap_DSCH_DeleteItem_RL_ReconfPrepTDD,
    &ett_rnsap_USCH_ModifyList_RL_ReconfPrepTDD,
    &ett_rnsap_USCH_ModifyItem_RL_ReconfPrepTDD,
    &ett_rnsap_USCH_DeleteList_RL_ReconfPrepTDD,
    &ett_rnsap_USCH_DeleteItem_RL_ReconfPrepTDD,
    &ett_rnsap_RL_Information_RL_ReconfPrepTDD,
    &ett_rnsap_RL_InformationIE_RL_ReconfPrepTDD,
    &ett_rnsap_RadioLinkReconfigurationReadyFDD,
    &ett_rnsap_RL_InformationResponseList_RL_ReconfReadyFDD,
    &ett_rnsap_RL_InformationResponseItem_RL_ReconfReadyFDD,
    &ett_rnsap_RadioLinkReconfigurationReadyTDD,
    &ett_rnsap_RL_InformationResponse_RL_ReconfReadyTDD,
    &ett_rnsap_UL_CCTrCHInformationListIE_RL_ReconfReadyTDD,
    &ett_rnsap_UL_CCTrCH_InformationItem_RL_ReconfReadyTDD,
    &ett_rnsap_UL_DPCH_LCR_InformationAddList_RL_ReconfReadyTDD,
    &ett_rnsap_UL_DPCH_InformationAddListIE_RL_ReconfReadyTDD,
    &ett_rnsap_UL_DPCH_InformationModifyListIE_RL_ReconfReadyTDD,
    &ett_rnsap_UL_TimeslotLCR_InformationModifyList_RL_ReconfReadyTDD,
    &ett_rnsap_UL_TimeslotLCR_InformationModifyItem_RL_ReconfReadyTDD,
    &ett_rnsap_TDD_UL_Code_LCR_InformationModifyList_RL_ReconfReadyTDD,
    &ett_rnsap_TDD_UL_Code_LCR_InformationModifyItem_RL_ReconfReadyTDD,
    &ett_rnsap_UL_Timeslot_InformationModifyList_RL_ReconfReadyTDD,
    &ett_rnsap_UL_Timeslot_InformationModifyItem_RL_ReconfReadyTDD,
    &ett_rnsap_TDD_UL_Code_InformationModifyList_RL_ReconfReadyTDD,
    &ett_rnsap_TDD_UL_Code_InformationModifyItem_RL_ReconfReadyTDD,
    &ett_rnsap_UL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD,
    &ett_rnsap_UL_DPCH_InformationDeleteItem_RL_ReconfReadyTDD,
    &ett_rnsap_DL_CCTrCHInformationListIE_RL_ReconfReadyTDD,
    &ett_rnsap_DL_CCTrCH_InformationItem_RL_ReconfReadyTDD,
    &ett_rnsap_DL_DPCH_LCR_InformationAddList_RL_ReconfReadyTDD,
    &ett_rnsap_DL_DPCH_InformationAddListIE_RL_ReconfReadyTDD,
    &ett_rnsap_DL_DPCH_InformationModifyListIE_RL_ReconfReadyTDD,
    &ett_rnsap_DL_TimeslotLCR_InformationModifyList_RL_ReconfReadyTDD,
    &ett_rnsap_DL_TimeslotLCR_InformationModifyItem_RL_ReconfReadyTDD,
    &ett_rnsap_TDD_DL_Code_LCR_InformationModifyList_RL_ReconfReadyTDD,
    &ett_rnsap_TDD_DL_Code_LCR_InformationModifyItem_RL_ReconfReadyTDD,
    &ett_rnsap_DL_Timeslot_InformationModifyList_RL_ReconfReadyTDD,
    &ett_rnsap_DL_Timeslot_InformationModifyItem_RL_ReconfReadyTDD,
    &ett_rnsap_TDD_DL_Code_InformationModifyList_RL_ReconfReadyTDD,
    &ett_rnsap_TDD_DL_Code_InformationModifyItem_RL_ReconfReadyTDD,
    &ett_rnsap_DL_DPCH_InformationDeleteListIE_RL_ReconfReadyTDD,
    &ett_rnsap_DL_DPCH_InformationDeleteItem_RL_ReconfReadyTDD,
    &ett_rnsap_DSCHToBeAddedOrModifiedList_RL_ReconfReadyTDD,
    &ett_rnsap_DSCHToBeAddedOrModifiedItem_RL_ReconfReadyTDD,
    &ett_rnsap_USCHToBeAddedOrModifiedList_RL_ReconfReadyTDD,
    &ett_rnsap_USCHToBeAddedOrModifiedItem_RL_ReconfReadyTDD,
    &ett_rnsap_Multiple_RL_InformationResponse_RL_ReconfReadyTDD,
    &ett_rnsap_RadioLinkReconfigurationCommit,
    &ett_rnsap_RadioLinkReconfigurationFailure,
    &ett_rnsap_CauseLevel_RL_ReconfFailure,
    &ett_rnsap_GeneralCauseList_RL_ReconfFailure,
    &ett_rnsap_RLSpecificCauseList_RL_ReconfFailure,
    &ett_rnsap_RL_ReconfigurationFailureList_RL_ReconfFailure,
    &ett_rnsap_RL_ReconfigurationFailure_RL_ReconfFail,
    &ett_rnsap_RadioLinkReconfigurationCancel,
    &ett_rnsap_RadioLinkReconfigurationRequestFDD,
    &ett_rnsap_UL_DPCH_Information_RL_ReconfRqstFDD,
    &ett_rnsap_DL_DPCH_Information_RL_ReconfRqstFDD,
    &ett_rnsap_DCH_DeleteList_RL_ReconfRqstFDD,
    &ett_rnsap_DCH_DeleteItem_RL_ReconfRqstFDD,
    &ett_rnsap_RL_ReconfigurationRequestFDD_RL_InformationList,
    &ett_rnsap_RL_ReconfigurationRequestFDD_RL_Information_IEs,
    &ett_rnsap_RadioLinkReconfigurationRequestTDD,
    &ett_rnsap_UL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD,
    &ett_rnsap_UL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD,
    &ett_rnsap_UL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD,
    &ett_rnsap_UL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD,
    &ett_rnsap_DL_CCTrCH_InformationModifyList_RL_ReconfRqstTDD,
    &ett_rnsap_DL_CCTrCH_InformationModifyItem_RL_ReconfRqstTDD,
    &ett_rnsap_DL_CCTrCH_InformationDeleteList_RL_ReconfRqstTDD,
    &ett_rnsap_DL_CCTrCH_InformationDeleteItem_RL_ReconfRqstTDD,
    &ett_rnsap_DCH_DeleteList_RL_ReconfRqstTDD,
    &ett_rnsap_DCH_DeleteItem_RL_ReconfRqstTDD,
    &ett_rnsap_Multiple_RL_ReconfigurationRequestTDD_RL_Information,
    &ett_rnsap_RL_ReconfigurationRequestTDD_RL_Information,
    &ett_rnsap_RadioLinkReconfigurationResponseFDD,
    &ett_rnsap_RL_InformationResponseList_RL_ReconfRspFDD,
    &ett_rnsap_RL_InformationResponseItem_RL_ReconfRspFDD,
    &ett_rnsap_RadioLinkReconfigurationResponseTDD,
    &ett_rnsap_RL_InformationResponse_RL_ReconfRspTDD,
    &ett_rnsap_DL_CCTrCH_InformationList_RL_ReconfRspTDD,
    &ett_rnsap_DL_CCTrCH_InformationItem_RL_ReconfRspTDD,
    &ett_rnsap_DL_DPCH_InformationModifyItem_LCR_RL_ReconfRspTDD,
    &ett_rnsap_DL_Timeslot_LCR_InformationModifyList_RL_ReconfRspTDD,
    &ett_rnsap_DL_Timeslot_LCR_InformationModifyItem_RL_ReconfRspTDD,
    &ett_rnsap_Multiple_RL_InformationResponse_RL_ReconfRspTDD,
    &ett_rnsap_RadioLinkFailureIndication,
    &ett_rnsap_Reporting_Object_RL_FailureInd,
    &ett_rnsap_RL_RL_FailureInd,
    &ett_rnsap_RL_InformationList_RL_FailureInd,
    &ett_rnsap_RL_Information_RL_FailureInd,
    &ett_rnsap_RL_Set_RL_FailureInd,
    &ett_rnsap_RL_Set_InformationList_RL_FailureInd,
    &ett_rnsap_RL_Set_Information_RL_FailureInd,
    &ett_rnsap_CCTrCH_RL_FailureInd,
    &ett_rnsap_CCTrCH_InformationList_RL_FailureInd,
    &ett_rnsap_CCTrCH_InformationItem_RL_FailureInd,
    &ett_rnsap_RadioLinkPreemptionRequiredIndication,
    &ett_rnsap_RL_InformationList_RL_PreemptRequiredInd,
    &ett_rnsap_RL_InformationItem_RL_PreemptRequiredInd,
    &ett_rnsap_HSDSCHMacdFlowSpecificInformationList_RL_PreemptRequiredInd,
    &ett_rnsap_HSDSCHMacdFlowSpecificInformationItem_RL_PreemptRequiredInd,
    &ett_rnsap_EDCH_MacdFlowSpecificInformationList_RL_PreemptRequiredInd,
    &ett_rnsap_EDCH_MacdFlowSpecificInformationItem_RL_PreemptRequiredInd,
    &ett_rnsap_RadioLinkRestoreIndication,
    &ett_rnsap_Reporting_Object_RL_RestoreInd,
    &ett_rnsap_RL_RL_RestoreInd,
    &ett_rnsap_RL_InformationList_RL_RestoreInd,
    &ett_rnsap_RL_Information_RL_RestoreInd,
    &ett_rnsap_RL_Set_RL_RestoreInd,
    &ett_rnsap_RL_Set_InformationList_RL_RestoreInd,
    &ett_rnsap_RL_Set_Information_RL_RestoreInd,
    &ett_rnsap_CCTrCH_RL_RestoreInd,
    &ett_rnsap_CCTrCH_InformationList_RL_RestoreInd,
    &ett_rnsap_CCTrCH_InformationItem_RL_RestoreInd,
    &ett_rnsap_DL_PowerControlRequest,
    &ett_rnsap_DL_ReferencePowerInformationList_DL_PC_Rqst,
    &ett_rnsap_DL_ReferencePowerInformation_DL_PC_Rqst,
    &ett_rnsap_DL_PowerTimeslotControlRequest,
    &ett_rnsap_PhysicalChannelReconfigurationRequestFDD,
    &ett_rnsap_RL_Information_PhyChReconfRqstFDD,
    &ett_rnsap_PhysicalChannelReconfigurationRequestTDD,
    &ett_rnsap_RL_Information_PhyChReconfRqstTDD,
    &ett_rnsap_UL_CCTrCH_InformationListIE_PhyChReconfRqstTDD,
    &ett_rnsap_UL_CCTrCH_InformationItem_PhyChReconfRqstTDD,
    &ett_rnsap_UL_DPCH_InformationItem_PhyChReconfRqstTDD,
    &ett_rnsap_UL_TimeslotLCR_InformationList_PhyChReconfRqstTDD,
    &ett_rnsap_UL_TimeslotLCR_InformationItem_PhyChReconfRqstTDD,
    &ett_rnsap_UL_Timeslot_InformationList_PhyChReconfRqstTDD,
    &ett_rnsap_UL_Timeslot_InformationItem_PhyChReconfRqstTDD,
    &ett_rnsap_DL_CCTrCH_InformationListIE_PhyChReconfRqstTDD,
    &ett_rnsap_DL_CCTrCH_InformationItem_PhyChReconfRqstTDD,
    &ett_rnsap_DL_DPCH_InformationItem_PhyChReconfRqstTDD,
    &ett_rnsap_DL_TimeslotLCR_InformationList_PhyChReconfRqstTDD,
    &ett_rnsap_DL_TimeslotLCR_InformationItem_PhyChReconfRqstTDD,
    &ett_rnsap_DL_Timeslot_InformationList_PhyChReconfRqstTDD,
    &ett_rnsap_DL_Timeslot_InformationItem_PhyChReconfRqstTDD,
    &ett_rnsap_HSPDSCH_Timeslot_InformationList_PhyChReconfRqstTDD,
    &ett_rnsap_HSPDSCH_Timeslot_InformationItem_PhyChReconfRqstTDD,
    &ett_rnsap_HSPDSCH_Timeslot_InformationListLCR_PhyChReconfRqstTDD,
    &ett_rnsap_HSPDSCH_Timeslot_InformationItemLCR_PhyChReconfRqstTDD,
    &ett_rnsap_PhysicalChannelReconfigurationCommand,
    &ett_rnsap_PhysicalChannelReconfigurationFailure,
    &ett_rnsap_RadioLinkCongestionIndication,
    &ett_rnsap_RL_InformationList_RL_CongestInd,
    &ett_rnsap_RL_InformationItem_RL_CongestInd,
    &ett_rnsap_DCH_Rate_Information_RL_CongestInd,
    &ett_rnsap_DCH_Rate_InformationItem_RL_CongestInd,
    &ett_rnsap_EDCH_MacdFlowSpecificInformationList_RL_CongestInd,
    &ett_rnsap_EDCH_MacdFlowSpecificInformationItem_RL_CongestInd,
    &ett_rnsap_UplinkSignallingTransferIndicationFDD,
    &ett_rnsap_UplinkSignallingTransferIndicationTDD,
    &ett_rnsap_DownlinkSignallingTransferRequest,
    &ett_rnsap_RelocationCommit,
    &ett_rnsap_PagingRequest,
    &ett_rnsap_PagingArea_PagingRqst,
    &ett_rnsap_URA_PagingRqst,
    &ett_rnsap_Cell_PagingRqst,
    &ett_rnsap_CNOriginatedPage_PagingRqst,
    &ett_rnsap_DedicatedMeasurementInitiationRequest,
    &ett_rnsap_DedicatedMeasurementObjectType_DM_Rqst,
    &ett_rnsap_RL_DM_Rqst,
    &ett_rnsap_RL_InformationList_DM_Rqst,
    &ett_rnsap_RL_InformationItem_DM_Rqst,
    &ett_rnsap_HSSICH_Info_DM_Rqst,
    &ett_rnsap_RL_Set_DM_Rqst,
    &ett_rnsap_RL_Set_InformationList_DM_Rqst,
    &ett_rnsap_RL_Set_InformationItem_DM_Rqst,
    &ett_rnsap_DedicatedMeasurementInitiationResponse,
    &ett_rnsap_DedicatedMeasurementObjectType_DM_Rsp,
    &ett_rnsap_RL_DM_Rsp,
    &ett_rnsap_RL_Set_DM_Rsp,
    &ett_rnsap_RL_InformationList_DM_Rsp,
    &ett_rnsap_RL_InformationItem_DM_Rsp,
    &ett_rnsap_RL_Set_InformationList_DM_Rsp,
    &ett_rnsap_RL_Set_InformationItem_DM_Rsp,
    &ett_rnsap_Multiple_DedicatedMeasurementValueList_TDD_DM_Rsp,
    &ett_rnsap_Multiple_DedicatedMeasurementValueItem_TDD_DM_Rsp,
    &ett_rnsap_Multiple_DedicatedMeasurementValueList_LCR_TDD_DM_Rsp,
    &ett_rnsap_Multiple_DedicatedMeasurementValueItem_LCR_TDD_DM_Rsp,
    &ett_rnsap_DedicatedMeasurementInitiationFailure,
    &ett_rnsap_DedicatedMeasurementObjectType_DM_Fail,
    &ett_rnsap_RL_DM_Fail,
    &ett_rnsap_RL_Set_DM_Fail,
    &ett_rnsap_RL_Unsuccessful_InformationRespList_DM_Fail,
    &ett_rnsap_RL_Unsuccessful_InformationItem_DM_Fail,
    &ett_rnsap_RL_Successful_InformationRespList_DM_Fail,
    &ett_rnsap_RL_Successful_InformationItem_DM_Fail,
    &ett_rnsap_RL_Set_Unsuccessful_InformationRespList_DM_Fail,
    &ett_rnsap_RL_Set_Unsuccessful_InformationItem_DM_Fail,
    &ett_rnsap_RL_Set_Successful_InformationRespList_DM_Fail,
    &ett_rnsap_RL_Set_Successful_InformationItem_DM_Fail,
    &ett_rnsap_DedicatedMeasurementReport,
    &ett_rnsap_DedicatedMeasurementObjectType_DM_Rprt,
    &ett_rnsap_RL_DM_Rprt,
    &ett_rnsap_RL_Set_DM_Rprt,
    &ett_rnsap_RL_InformationList_DM_Rprt,
    &ett_rnsap_RL_InformationItem_DM_Rprt,
    &ett_rnsap_RL_Set_InformationList_DM_Rprt,
    &ett_rnsap_RL_Set_InformationItem_DM_Rprt,
    &ett_rnsap_DedicatedMeasurementTerminationRequest,
    &ett_rnsap_DedicatedMeasurementFailureIndication,
    &ett_rnsap_DedicatedMeasurementObjectType_DM_Fail_Ind,
    &ett_rnsap_RL_DM_Fail_Ind,
    &ett_rnsap_RL_Set_DM_Fail_Ind,
    &ett_rnsap_RL_Unsuccessful_InformationRespList_DM_Fail_Ind,
    &ett_rnsap_RL_Unsuccessful_InformationItem_DM_Fail_Ind,
    &ett_rnsap_RL_Set_Unsuccessful_InformationRespList_DM_Fail_Ind,
    &ett_rnsap_RL_Set_Unsuccessful_InformationItem_DM_Fail_Ind,
    &ett_rnsap_CommonTransportChannelResourcesReleaseRequest,
    &ett_rnsap_CommonTransportChannelResourcesRequest,
    &ett_rnsap_CommonTransportChannelResourcesResponseFDD,
    &ett_rnsap_FACH_InfoForUESelectedS_CCPCH_CTCH_ResourceRspFDD,
    &ett_rnsap_CommonTransportChannelResourcesResponseTDD,
    &ett_rnsap_FACH_InfoForUESelectedS_CCPCH_CTCH_ResourceRspTDD,
    &ett_rnsap_CommonTransportChannelResourcesFailure,
    &ett_rnsap_CompressedModeCommand,
    &ett_rnsap_ErrorIndication,
    &ett_rnsap_CommonMeasurementInitiationRequest,
    &ett_rnsap_CommonMeasurementObjectType_CM_Rqst,
    &ett_rnsap_Cell_CM_Rqst,
    &ett_rnsap_NeighbouringCellMeasurementInfo,
    &ett_rnsap_NeighbouringCellMeasurementInfo_item,
    &ett_rnsap_CommonMeasurementInitiationResponse,
    &ett_rnsap_CommonMeasurementObjectType_CM_Rsp,
    &ett_rnsap_Cell_CM_Rsp,
    &ett_rnsap_CommonMeasurementInitiationFailure,
    &ett_rnsap_CommonMeasurementReport,
    &ett_rnsap_CommonMeasurementObjectType_CM_Rprt,
    &ett_rnsap_Cell_CM_Rprt,
    &ett_rnsap_CommonMeasurementTerminationRequest,
    &ett_rnsap_CommonMeasurementFailureIndication,
    &ett_rnsap_InformationExchangeInitiationRequest,
    &ett_rnsap_InformationExchangeObjectType_InfEx_Rqst,
    &ett_rnsap_Cell_InfEx_Rqst,
    &ett_rnsap_GSM_Cell_InfEx_Rqst,
    &ett_rnsap_InformationExchangeInitiationResponse,
    &ett_rnsap_InformationExchangeObjectType_InfEx_Rsp,
    &ett_rnsap_Cell_InfEx_Rsp,
    &ett_rnsap_MBMS_Bearer_Service_List_InfEx_Rsp,
    &ett_rnsap_MBMS_Bearer_ServiceItemIEs_InfEx_Rsp,
    &ett_rnsap_InformationExchangeInitiationFailure,
    &ett_rnsap_InformationReport,
    &ett_rnsap_InformationExchangeObjectType_InfEx_Rprt,
    &ett_rnsap_Cell_InfEx_Rprt,
    &ett_rnsap_InformationExchangeTerminationRequest,
    &ett_rnsap_InformationExchangeFailureIndication,
    &ett_rnsap_ResetRequest,
    &ett_rnsap_ResetIndicator,
    &ett_rnsap_ContextList_Reset,
    &ett_rnsap_ContextInfoList_Reset,
    &ett_rnsap_ContextInfoItem_Reset,
    &ett_rnsap_ContextType_Reset,
    &ett_rnsap_ContextGroupList_Reset,
    &ett_rnsap_ContextGroupInfoList_Reset,
    &ett_rnsap_ContextGroupInfoItem_Reset,
    &ett_rnsap_ResetResponse,
    &ett_rnsap_RadioLinkActivationCommandFDD,
    &ett_rnsap_DelayedActivationInformationList_RL_ActivationCmdFDD,
    &ett_rnsap_DelayedActivationInformation_RL_ActivationCmdFDD,
    &ett_rnsap_RadioLinkActivationCommandTDD,
    &ett_rnsap_DelayedActivationInformationList_RL_ActivationCmdTDD,
    &ett_rnsap_DelayedActivationInformation_RL_ActivationCmdTDD,
    &ett_rnsap_GERANUplinkSignallingTransferIndication,
    &ett_rnsap_RadioLinkParameterUpdateIndicationFDD,
    &ett_rnsap_RL_ParameterUpdateIndicationFDD_RL_InformationList,
    &ett_rnsap_RL_ParameterUpdateIndicationFDD_RL_Information_Item,
    &ett_rnsap_RadioLinkParameterUpdateIndicationTDD,
    &ett_rnsap_UEMeasurementInitiationRequest,
    &ett_rnsap_UEMeasurementInitiationResponse,
    &ett_rnsap_UEMeasurementInitiationFailure,
    &ett_rnsap_UEMeasurementReport,
    &ett_rnsap_UEMeasurementTerminationRequest,
    &ett_rnsap_UEMeasurementFailureIndication,
    &ett_rnsap_IurInvokeTrace,
    &ett_rnsap_ListOfInterfacesToTrace,
    &ett_rnsap_InterfacesToTraceItem,
    &ett_rnsap_IurDeactivateTrace,
    &ett_rnsap_MBMSAttachCommand,
    &ett_rnsap_MBMSDetachCommand,
    &ett_rnsap_DirectInformationTransfer,
    &ett_rnsap_PrivateMessage,
    &ett_rnsap_Active_MBMS_Bearer_Service_ListFDD,
    &ett_rnsap_Active_MBMS_Bearer_Service_ListFDD_PFL,
    &ett_rnsap_Active_MBMS_Bearer_Service_ListTDD,
    &ett_rnsap_Active_MBMS_Bearer_Service_ListTDD_PFL,
    &ett_rnsap_Active_Pattern_Sequence_Information,
    &ett_rnsap_AdditionalPreferredFrequency,
    &ett_rnsap_AdditionalPreferredFrequencyItem,
    &ett_rnsap_AffectedUEInformationForMBMS,
    &ett_rnsap_AllocationRetentionPriority,
    &ett_rnsap_Allowed_Rate_Information,
    &ett_rnsap_Angle_Of_Arrival_Value_LCR,
    &ett_rnsap_BadSatellites,
    &ett_rnsap_T_badSatelliteInformation,
    &ett_rnsap_T_badSatelliteInformation_item,
    &ett_rnsap_BSIC,
    &ett_rnsap_BurstModeParameters,
    &ett_rnsap_Cause,
    &ett_rnsap_Cell_Capacity_Class_Value,
    &ett_rnsap_CGI,
    &ett_rnsap_T_lAI,
    &ett_rnsap_CommonMeasurementAccuracy,
    &ett_rnsap_CommonMeasurementValue,
    &ett_rnsap_CommonMeasurementValueInformation,
    &ett_rnsap_CommonMeasurementAvailable,
    &ett_rnsap_CorrespondingCells,
    &ett_rnsap_CriticalityDiagnostics,
    &ett_rnsap_CriticalityDiagnostics_IE_List,
    &ett_rnsap_CriticalityDiagnostics_IE_List_item,
    &ett_rnsap_MessageStructure,
    &ett_rnsap_MessageStructure_item,
    &ett_rnsap_CN_CS_DomainIdentifier,
    &ett_rnsap_CN_PS_DomainIdentifier,
    &ett_rnsap_DCH_FDD_Information,
    &ett_rnsap_DCH_FDD_InformationItem,
    &ett_rnsap_DCH_Specific_FDD_InformationList,
    &ett_rnsap_DCH_Specific_FDD_Item,
    &ett_rnsap_DCH_InformationResponse,
    &ett_rnsap_DCH_InformationResponseItem,
    &ett_rnsap_DCH_TDD_Information,
    &ett_rnsap_DCH_TDD_InformationItem,
    &ett_rnsap_DCH_Specific_TDD_InformationList,
    &ett_rnsap_DCH_Specific_TDD_Item,
    &ett_rnsap_DedicatedMeasurementValue,
    &ett_rnsap_DedicatedMeasurementValueInformation,
    &ett_rnsap_DedicatedMeasurementAvailable,
    &ett_rnsap_DelayedActivation,
    &ett_rnsap_DelayedActivationUpdate,
    &ett_rnsap_Activate_Info,
    &ett_rnsap_Deactivate_Info,
    &ett_rnsap_Execution_Type,
    &ett_rnsap_DGPSCorrections,
    &ett_rnsap_T_satellite_DGPSCorrections_Information,
    &ett_rnsap_T_satellite_DGPSCorrections_Information_item,
    &ett_rnsap_DGPSThreshold,
    &ett_rnsap_DL_PowerBalancing_Information,
    &ett_rnsap_DL_ReferencePowerInformationList,
    &ett_rnsap_DL_ReferencePowerInformationItem,
    &ett_rnsap_DL_ReferencePowerInformation,
    &ett_rnsap_DL_Timeslot_Information,
    &ett_rnsap_DL_Timeslot_InformationItem,
    &ett_rnsap_DL_TimeslotLCR_Information,
    &ett_rnsap_DL_TimeslotLCR_InformationItem,
    &ett_rnsap_DL_TimeSlot_ISCP_Info,
    &ett_rnsap_DL_TimeSlot_ISCP_InfoItem,
    &ett_rnsap_DL_TimeSlot_ISCP_LCR_Information,
    &ett_rnsap_DL_TimeSlot_ISCP_LCR_InfoItem,
    &ett_rnsap_DSCH_FlowControlInformation,
    &ett_rnsap_DSCH_FlowControlItem,
    &ett_rnsap_DSCH_TDD_Information,
    &ett_rnsap_DSCH_TDD_InformationItem,
    &ett_rnsap_EDCH_FDD_DL_ControlChannelInformation,
    &ett_rnsap_EDCH_FDD_Information,
    &ett_rnsap_EDCH_FDD_InformationResponse,
    &ett_rnsap_EDCH_FDD_InformationResponseItem,
    &ett_rnsap_EDCH_FDD_Information_To_Modify,
    &ett_rnsap_EDCH_FDD_Update_Information,
    &ett_rnsap_EDCH_FDD_Update_InfoItem,
    &ett_rnsap_E_DCH_Grant_Type_Information,
    &ett_rnsap_E_DCH_LogicalChannelInformation,
    &ett_rnsap_E_DCH_LogicalChannelInformationItem,
    &ett_rnsap_E_DCH_MACdPDU_SizeList,
    &ett_rnsap_E_DCH_MACdPDU_SizeListItem,
    &ett_rnsap_E_DCH_LogicalChannelToModify,
    &ett_rnsap_E_DCH_LogicalChannelToModifyItem,
    &ett_rnsap_E_DCH_MACdPDU_SizeToModifyList,
    &ett_rnsap_E_DCH_LogicalChannelToDelete,
    &ett_rnsap_E_DCH_LogicalChannelToDeleteItem,
    &ett_rnsap_EDCH_MACdFlows_Information,
    &ett_rnsap_EDCH_MACdFlow_Specific_InfoList,
    &ett_rnsap_EDCH_MACdFlow_Specific_InfoItem,
    &ett_rnsap_EDCH_MACdFlow_Specific_InfoToModifyList,
    &ett_rnsap_EDCH_MACdFlow_Specific_InfoToModifyItem,
    &ett_rnsap_EDCH_MACdFlows_To_Delete,
    &ett_rnsap_EDCH_MACdFlows_To_Delete_Item,
    &ett_rnsap_E_DCH_Non_Scheduled_Transmission_Grant_Items,
    &ett_rnsap_EDPCH_Information_FDD,
    &ett_rnsap_EDPCH_Information_RLReconfPrepare_FDD,
    &ett_rnsap_EDPCH_Information_RLReconfRequest_FDD,
    &ett_rnsap_EDCH_Serving_RL,
    &ett_rnsap_EDCH_Serving_RL_in_this_DRNS,
    &ett_rnsap_E_TFCS_Information,
    &ett_rnsap_EventA,
    &ett_rnsap_EventB,
    &ett_rnsap_EventC,
    &ett_rnsap_EventD,
    &ett_rnsap_EventE,
    &ett_rnsap_EventF,
    &ett_rnsap_FACH_FlowControlInformation,
    &ett_rnsap_FACH_FlowControlInformationItem,
    &ett_rnsap_FACH_InformationList,
    &ett_rnsap_FACH_InformationItem,
    &ett_rnsap_FACH_PCH_InformationList,
    &ett_rnsap_FACH_PCH_InformationItem,
    &ett_rnsap_FDD_DCHs_to_Modify,
    &ett_rnsap_FDD_DCHs_to_ModifyItem,
    &ett_rnsap_FDD_DCHs_to_ModifySpecificInformationList,
    &ett_rnsap_FDD_DCHs_to_ModifySpecificItem,
    &ett_rnsap_FDD_DL_CodeInformation,
    &ett_rnsap_FDD_DL_CodeInformationItem,
    &ett_rnsap_FPACH_Information,
    &ett_rnsap_GA_Cell,
    &ett_rnsap_GA_Cell_item,
    &ett_rnsap_GA_CellAdditionalShapes,
    &ett_rnsap_GA_AltitudeAndDirection,
    &ett_rnsap_GA_EllipsoidArc,
    &ett_rnsap_GA_PointWithAltitude,
    &ett_rnsap_GA_PointWithAltitudeAndUncertaintyEllipsoid,
    &ett_rnsap_GA_PointWithUnCertaintyEllipse,
    &ett_rnsap_GA_UncertaintyEllipse,
    &ett_rnsap_GA_PointWithUnCertainty,
    &ett_rnsap_GA_AccessPointPosition,
    &ett_rnsap_GeographicalCoordinate,
    &ett_rnsap_GERAN_SI_Type,
    &ett_rnsap_GERAN_SystemInfo,
    &ett_rnsap_GERAN_SystemInfo_item,
    &ett_rnsap_GPS_Almanac,
    &ett_rnsap_T_satellite_Almanac_Information,
    &ett_rnsap_T_satellite_Almanac_Information_item,
    &ett_rnsap_Satellite_Almanac_Information_ExtItem,
    &ett_rnsap_Satellite_Almanac_Information_ExtItem_item,
    &ett_rnsap_GPSInformation,
    &ett_rnsap_GPSInformation_item,
    &ett_rnsap_GPS_Ionospheric_Model,
    &ett_rnsap_GPS_NavigationModel_and_TimeRecovery,
    &ett_rnsap_GPS_NavigationModel_and_TimeRecovery_item,
    &ett_rnsap_GPS_RealTime_Integrity,
    &ett_rnsap_GPS_RX_POS,
    &ett_rnsap_GPS_UTC_Model,
    &ett_rnsap_Guaranteed_Rate_Information,
    &ett_rnsap_HARQ_MemoryPartitioning,
    &ett_rnsap_HARQ_MemoryPartitioning_Implicit,
    &ett_rnsap_HARQ_MemoryPartitioning_Explicit,
    &ett_rnsap_HARQ_MemoryPartitioningList,
    &ett_rnsap_HARQ_MemoryPartitioningItem,
    &ett_rnsap_HSDSCH_FDD_Information,
    &ett_rnsap_HSDSCH_FDD_Information_Response,
    &ett_rnsap_HSDSCH_Information_to_Modify,
    &ett_rnsap_HSDSCH_Information_to_Modify_Unsynchronised,
    &ett_rnsap_HSDSCH_MACdFlow_Specific_InfoList,
    &ett_rnsap_HSDSCH_MACdFlow_Specific_InfoItem,
    &ett_rnsap_HSDSCH_MACdFlow_Specific_InfoList_Response,
    &ett_rnsap_HSDSCH_MACdFlow_Specific_InfoItem_Response,
    &ett_rnsap_HSDSCH_MACdFlow_Specific_InfoList_to_Modify,
    &ett_rnsap_HSDSCH_MACdFlow_Specific_InfoItem_to_Modify,
    &ett_rnsap_HSDSCH_MACdFlows_Information,
    &ett_rnsap_HSDSCH_MACdFlows_to_Delete,
    &ett_rnsap_HSDSCH_MACdFlows_to_Delete_Item,
    &ett_rnsap_HSDSCH_Initial_Capacity_Allocation,
    &ett_rnsap_HSDSCH_Initial_Capacity_AllocationItem,
    &ett_rnsap_HSDSCH_TDD_Information,
    &ett_rnsap_HSDSCH_TDD_Information_Response,
    &ett_rnsap_HSPDSCH_TDD_Specific_InfoList_Response,
    &ett_rnsap_HSPDSCH_TDD_Specific_InfoItem_Response,
    &ett_rnsap_HSPDSCH_TDD_Specific_InfoList_Response_LCR,
    &ett_rnsap_HSPDSCH_TDD_Specific_InfoItem_Response_LCR,
    &ett_rnsap_HSSCCH_FDD_Specific_InfoList_Response,
    &ett_rnsap_HSSCCH_FDD_Specific_InfoItem_Response,
    &ett_rnsap_HSSCCH_TDD_Specific_InfoList_Response,
    &ett_rnsap_HSSCCH_TDD_Specific_InfoItem_Response,
    &ett_rnsap_HSSCCH_TDD_Specific_InfoList_Response_LCR,
    &ett_rnsap_HSSCCH_TDD_Specific_InfoItem_Response_LCR,
    &ett_rnsap_HSSICH_Info,
    &ett_rnsap_HSSICH_InfoLCR,
    &ett_rnsap_HS_SICH_Reception_Quality_Value,
    &ett_rnsap_HSDSCH_FDD_Update_Information,
    &ett_rnsap_HSDSCH_TDD_Update_Information,
    &ett_rnsap_InformationAvailable,
    &ett_rnsap_InformationReportCharacteristics,
    &ett_rnsap_InformationReportPeriodicity,
    &ett_rnsap_InformationThreshold,
    &ett_rnsap_InformationType,
    &ett_rnsap_IPDLParameters,
    &ett_rnsap_IPDL_FDD_Parameters,
    &ett_rnsap_IPDL_TDD_Parameters,
    &ett_rnsap_IPDL_TDD_ParametersLCR,
    &ett_rnsap_LoadValue,
    &ett_rnsap_MAC_c_sh_SDU_LengthList,
    &ett_rnsap_MACdPDU_Size_IndexList,
    &ett_rnsap_MACdPDU_Size_IndexItem,
    &ett_rnsap_MACdPDU_Size_IndexList_to_Modify,
    &ett_rnsap_MACdPDU_Size_IndexItem_to_Modify,
    &ett_rnsap_MBMS_Bearer_Service_Full_Address,
    &ett_rnsap_MBMS_Bearer_Service_List,
    &ett_rnsap_MBMS_Bearer_ServiceItemFDD,
    &ett_rnsap_MBMS_Bearer_ServiceItemFDD_PFL,
    &ett_rnsap_MBMS_Bearer_ServiceItemTDD,
    &ett_rnsap_MBMS_Bearer_ServiceItemTDD_PFL,
    &ett_rnsap_MBMSChannelTypeInfo,
    &ett_rnsap_MBMSChannelTypeCellList,
    &ett_rnsap_MBMSPreferredFreqLayerInfo,
    &ett_rnsap_MeasurementIncreaseDecreaseThreshold,
    &ett_rnsap_MeasurementThreshold,
    &ett_rnsap_MidambleShiftAndBurstType,
    &ett_rnsap_T_type1,
    &ett_rnsap_T_midambleAllocationMode,
    &ett_rnsap_T_type2,
    &ett_rnsap_T_midambleAllocationMode1,
    &ett_rnsap_T_type3,
    &ett_rnsap_T_midambleAllocationMode2,
    &ett_rnsap_MidambleShiftLCR,
    &ett_rnsap_ModifyPriorityQueue,
    &ett_rnsap_NACC_Related_Data,
    &ett_rnsap_Neighbouring_UMTS_CellInformation,
    &ett_rnsap_Neighbouring_UMTS_CellInformationItem,
    &ett_rnsap_Neighbouring_FDD_CellInformation,
    &ett_rnsap_Neighbouring_FDD_CellInformationItem,
    &ett_rnsap_NeighbouringFDDCellMeasurementInformation,
    &ett_rnsap_Neighbouring_GSM_CellInformationIEs,
    &ett_rnsap_Neighbouring_GSM_CellInformationItem,
    &ett_rnsap_Neighbouring_TDD_CellInformation,
    &ett_rnsap_Neighbouring_TDD_CellInformationItem,
    &ett_rnsap_NeighbouringTDDCellMeasurementInformation,
    &ett_rnsap_NeighbouringTDDCellMeasurementInformationLCR,
    &ett_rnsap_Neighbouring_LCR_TDD_CellInformation,
    &ett_rnsap_Neighbouring_LCR_TDD_CellInformationItem,
    &ett_rnsap_NotProvidedCellList,
    &ett_rnsap_NRTLoadInformationValue,
    &ett_rnsap_OnModification,
    &ett_rnsap_OnModificationInformation,
    &ett_rnsap_PCH_InformationList,
    &ett_rnsap_PCH_InformationItem,
    &ett_rnsap_Periodic,
    &ett_rnsap_PeriodicInformation,
    &ett_rnsap_Permanent_NAS_UE_Identity,
    &ett_rnsap_PreferredFrequencyLayerInfo,
    &ett_rnsap_PriorityQueue_InfoList,
    &ett_rnsap_PriorityQueue_InfoItem,
    &ett_rnsap_PriorityQueue_InfoList_to_Modify,
    &ett_rnsap_PriorityQueue_InfoItem_to_Add,
    &ett_rnsap_PriorityQueue_InfoItem_to_Modify,
    &ett_rnsap_PriorityQueue_InfoList_to_Modify_Unsynchronised,
    &ett_rnsap_PriorityQueue_InfoItem_to_Modify_Unsynchronised,
    &ett_rnsap_ProvidedInformation,
    &ett_rnsap_PTMCellList,
    &ett_rnsap_PTPCellList,
    &ett_rnsap_RB_Info,
    &ett_rnsap_Reference_E_TFCI_Information,
    &ett_rnsap_Reference_E_TFCI_Information_Item,
    &ett_rnsap_ReportCharacteristics,
    &ett_rnsap_ReportPeriodicity,
    &ett_rnsap_RequestedDataValue,
    &ett_rnsap_RequestedDataValueInformation,
    &ett_rnsap_RL_Specific_DCH_Info,
    &ett_rnsap_RL_Specific_DCH_Info_Item,
    &ett_rnsap_RL_Specific_EDCH_Information,
    &ett_rnsap_RL_Specific_EDCH_InfoItem,
    &ett_rnsap_RTLoadValue,
    &ett_rnsap_SAI,
    &ett_rnsap_Secondary_CCPCH_Info_TDD,
    &ett_rnsap_Secondary_CPICH_Information,
    &ett_rnsap_Secondary_CPICH_Information_Change,
    &ett_rnsap_Secondary_LCR_CCPCH_Info_TDD,
    &ett_rnsap_Secondary_CCPCH_TDD_InformationList,
    &ett_rnsap_Secondary_CCPCH_TDD_InformationItem,
    &ett_rnsap_Secondary_LCR_CCPCH_TDD_InformationList,
    &ett_rnsap_Secondary_LCR_CCPCH_TDD_InformationItem,
    &ett_rnsap_Secondary_CCPCH_TDD_Code_Information,
    &ett_rnsap_Secondary_CCPCH_TDD_Code_InformationItem,
    &ett_rnsap_Secondary_LCR_CCPCH_TDD_Code_Information,
    &ett_rnsap_Secondary_LCR_CCPCH_TDD_Code_InformationItem,
    &ett_rnsap_GA_AccessPointPositionwithOptionalAltitude,
    &ett_rnsap_SFNSFNMeasurementThresholdInformation,
    &ett_rnsap_SFNSFNMeasurementValueInformation,
    &ett_rnsap_T_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation,
    &ett_rnsap_T_successfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item,
    &ett_rnsap_T_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation,
    &ett_rnsap_T_unsuccessfullNeighbouringCellSFNSFNObservedTimeDifferenceMeasurementInformation_item,
    &ett_rnsap_SFNSFNTimeStampInformation,
    &ett_rnsap_SFNSFNTimeStamp_TDD,
    &ett_rnsap_SFNSFNValue,
    &ett_rnsap_SNA_Information,
    &ett_rnsap_ListOfSNAs,
    &ett_rnsap_S_RNTI_Group,
    &ett_rnsap_SynchronisationConfiguration,
    &ett_rnsap_SYNC_UL_ProcParameters,
    &ett_rnsap_TDD_ChannelisationCodeLCR,
    &ett_rnsap_TDD_DCHs_to_Modify,
    &ett_rnsap_TDD_DCHs_to_ModifyItem,
    &ett_rnsap_TDD_DCHs_to_ModifySpecificInformationList,
    &ett_rnsap_TDD_DCHs_to_ModifySpecificItem,
    &ett_rnsap_TDD_DL_Code_Information,
    &ett_rnsap_TDD_DL_Code_InformationItem,
    &ett_rnsap_TDD_DL_Code_LCR_Information,
    &ett_rnsap_TDD_DL_Code_LCR_InformationItem,
    &ett_rnsap_TDD_DL_DPCH_TimeSlotFormat_LCR,
    &ett_rnsap_TDD_DPCHOffset,
    &ett_rnsap_TDD_UL_Code_Information,
    &ett_rnsap_TDD_UL_Code_InformationItem,
    &ett_rnsap_TDD_UL_Code_LCR_Information,
    &ett_rnsap_TDD_UL_Code_LCR_InformationItem,
    &ett_rnsap_TDD_UL_DPCH_TimeSlotFormat_LCR,
    &ett_rnsap_TMGI,
    &ett_rnsap_TnlQos,
    &ett_rnsap_Transmission_Gap_Pattern_Sequence_Information,
    &ett_rnsap_Transmission_Gap_Pattern_Sequence_Information_item,
    &ett_rnsap_Transmission_Gap_Pattern_Sequence_Status_List,
    &ett_rnsap_Transmission_Gap_Pattern_Sequence_Status_List_item,
    &ett_rnsap_TUTRANGPS,
    &ett_rnsap_TUTRANGPSMeasurementThresholdInformation,
    &ett_rnsap_TUTRANGPSMeasurementValueInformation,
    &ett_rnsap_TransportFormatCombination_Beta,
    &ett_rnsap_T_signalledGainFactors,
    &ett_rnsap_TFCS,
    &ett_rnsap_T_tFCSvalues,
    &ett_rnsap_TFCS_TFCSList,
    &ett_rnsap_TFCS_TFCSList_item,
    &ett_rnsap_TFCS_CTFC,
    &ett_rnsap_TransportFormatSet,
    &ett_rnsap_TransportFormatSet_DynamicPartList,
    &ett_rnsap_TransportFormatSet_DynamicPartList_item,
    &ett_rnsap_TransportFormatSet_ModeDP,
    &ett_rnsap_TDD_TransportFormatSet_ModeDP,
    &ett_rnsap_TransmissionTimeIntervalInformation,
    &ett_rnsap_TransmissionTimeIntervalInformation_item,
    &ett_rnsap_TransportFormatSet_Semi_staticPart,
    &ett_rnsap_TransportFormatSet_ModeSSP,
    &ett_rnsap_UE_Capabilities_Info,
    &ett_rnsap_UEIdentity,
    &ett_rnsap_UEMeasurementReportCharacteristics,
    &ett_rnsap_UEMeasurementReportCharacteristicsEvent1h,
    &ett_rnsap_UEMeasurementReportCharacteristicsEvent1i,
    &ett_rnsap_UEMeasurementReportCharacteristicsEvent6a,
    &ett_rnsap_UEMeasurementReportCharacteristicsEvent6b,
    &ett_rnsap_UEMeasurementReportCharacteristicsEvent6c,
    &ett_rnsap_UEMeasurementReportCharacteristicsEvent6d,
    &ett_rnsap_UEMeasurementReportCharacteristicsPeriodic,
    &ett_rnsap_UEMeasurementThreshold,
    &ett_rnsap_UEMeasurementTimeslotInfoHCR,
    &ett_rnsap_UEMeasurementTimeslotInfoHCR_IEs,
    &ett_rnsap_UEMeasurementTimeslotInfoLCR,
    &ett_rnsap_UEMeasurementTimeslotInfoLCR_IEs,
    &ett_rnsap_UEMeasurementValue,
    &ett_rnsap_UE_MeasurementValue_UE_Transmitted_Power,
    &ett_rnsap_UEMeasurementValueTransmittedPowerListHCR,
    &ett_rnsap_UEMeasurementValueTransmittedPowerListHCR_IEs,
    &ett_rnsap_UEMeasurementValueTransmittedPowerListLCR,
    &ett_rnsap_UEMeasurementValueTransmittedPowerListLCR_IEs,
    &ett_rnsap_UE_MeasurementValue_Primary_CCPCH_RSCP,
    &ett_rnsap_UE_MeasurementValue_DL_Timeslot_ISCP,
    &ett_rnsap_UEMeasurementValueTimeslotISCPListHCR,
    &ett_rnsap_UEMeasurementValueTimeslotISCPListHCR_IEs,
    &ett_rnsap_UEMeasurementValueTimeslotISCPListLCR,
    &ett_rnsap_UEMeasurementValueTimeslotISCPListLCR_IEs,
    &ett_rnsap_UEMeasurementValueInformation,
    &ett_rnsap_UEMeasurementValueInformationAvailable,
    &ett_rnsap_UE_State,
    &ett_rnsap_Cell_Fach_Pch_State,
    &ett_rnsap_Ura_Pch_State,
    &ett_rnsap_UL_Timeslot_Information,
    &ett_rnsap_UL_Timeslot_InformationItem,
    &ett_rnsap_UL_TimeslotLCR_Information,
    &ett_rnsap_UL_TimeslotLCR_InformationItem,
    &ett_rnsap_UL_TimeSlot_ISCP_Info,
    &ett_rnsap_UL_TimeSlot_ISCP_InfoItem,
    &ett_rnsap_UL_TimeSlot_ISCP_LCR_Info,
    &ett_rnsap_UL_TimeSlot_ISCP_LCR_InfoItem,
    &ett_rnsap_UL_TimingAdvanceCtrl_LCR,
    &ett_rnsap_UC_ID,
    &ett_rnsap_UL_ScramblingCode,
    &ett_rnsap_UL_Synchronisation_Parameters_LCR,
    &ett_rnsap_URA_Information,
    &ett_rnsap_RNCsWithCellsInTheAccessedURA_List,
    &ett_rnsap_RNCsWithCellsInTheAccessedURA_Item,
    &ett_rnsap_USCH_Information,
    &ett_rnsap_USCH_InformationItem,
    &ett_rnsap_PrivateIE_ID,
    &ett_rnsap_ProcedureID,
    &ett_rnsap_TransactionID,
    &ett_rnsap_ProtocolIE_Container,
    &ett_rnsap_ProtocolIE_Field,
    &ett_rnsap_ProtocolIE_ContainerPair,
    &ett_rnsap_ProtocolIE_FieldPair,
    &ett_rnsap_ProtocolIE_ContainerList,
    &ett_rnsap_ProtocolIE_ContainerPairList,
    &ett_rnsap_ProtocolExtensionContainer,
    &ett_rnsap_ProtocolExtensionField,
    &ett_rnsap_PrivateIE_Container,
    &ett_rnsap_PrivateIE_Field,

/*--- End of included file: packet-rnsap-ettarr.c ---*/
#line 2335 "packet-rnsap-template.c"
  };


  /* Register protocol */
  proto_rnsap = proto_register_protocol(PNAME, PSNAME, PFNAME);
  /* Register fields and subtrees */
  proto_register_field_array(proto_rnsap, hf, array_length(hf));
  proto_register_subtree_array(ett, array_length(ett));

 
  register_dissector("rnsap", dissect_rnsap, proto_rnsap);


}


/*--- proto_reg_handoff_rnsap ---------------------------------------*/
void
proto_reg_handoff_rnsap(void)
{

	rnsap_handle = find_dissector("rnsap");
	dissector_add("sccp.ssn", SCCP_SSN_RNSAP, rnsap_handle);
	/* Add heuristic dissector
	 * Perhaps we want a preference whether the heuristic dissector
	 * is or isn't enabled
	 */
	/*heur_dissector_add("sccp", dissect_sccp_rnsap_heur, proto_rnsap); */

}


